@@ -264,7 +264,9 @@ def is_interesting_file(file_name):
     """Return true if this file should be checked."""
     return (file_name.startswith("jstests")
             or file_name.startswith("src") and not file_name.startswith("src/third_party/")
-            and not file_name.startswith("src/mongo/gotools/")) and FILES_RE.search(file_name)
+            and not file_name.startswith("src/mongo/gotools/")
+            and not file_name.startswith("src/mongo/db/modules/enterprise/src/streams/third_party")
+            and not file_name.startswith("src/streams/third_party")) and FILES_RE.search(file_name)
 
 
 def get_list_from_lines(lines):
