@@ -159,6 +159,36 @@ def get_thread_id():
     raise ValueError("Failed to find thread id in {}".format(thread_info))
 
 
+MAIN_GLOBAL_BLOCK = None
+
+
+def lookup_type(gdb_type_str: str) -> gdb.Type:
+    """
+    Try to find the type object from string.
+
+    GDB says it searches the global blocks, however this appear not to be the
+    case or at least it doesn't search all global blocks, sometimes it required
+    to get the global block based off the current frame.
+    """
+    global MAIN_GLOBAL_BLOCK  # pylint: disable=global-statement
+
+    exceptions = []
+    try:
+        return gdb.lookup_type(gdb_type_str)
+    except Exception as exc:
+        exceptions.append(exc)
+
+    if MAIN_GLOBAL_BLOCK is None:
+        MAIN_GLOBAL_BLOCK = gdb.lookup_symbol("main")[0].symtab.global_block()
+
+    try:
+        return gdb.lookup_type(gdb_type_str, MAIN_GLOBAL_BLOCK)
+    except Exception as exc:
+        exceptions.append(exc)
+
+    raise gdb.error("Failed to get type, tried:\n%s" % '\n'.join([str(exc) for exc in exceptions]))
+
+
 def get_current_thread_name():
     """Return the name of the current GDB thread."""
     fallback_name = '"%s"' % (gdb.selected_thread().name or '')
@@ -217,7 +247,7 @@ def get_wt_session(recovery_unit, recovery_unit_impl_type):
     if not wt_session_handle.dereference().address:
         return None
     wt_session = wt_session_handle.dereference().cast(
-        gdb.lookup_type("mongo::WiredTigerSession"))["_session"]
+        lookup_type("mongo::WiredTigerSession"))["_session"]
     return wt_session
 
 
@@ -226,40 +256,50 @@ def get_decorations(obj):
 
     Each object returned by the iterator is a tuple whose first element is the type name of the
     decoration and whose second element is the decoration object itself.
-
-    TODO: De-duplicate the logic between here and DecorablePrinter. This code was copied from there.
     """
     type_name = str(obj.type).replace("class", "").replace(" ", "")
-    decorable = obj.cast(gdb.lookup_type("mongo::Decorable<{}>".format(type_name)))
-    decl_vector = decorable["_decorations"]["_registry"]["_decorationInfo"]
+    decorable = obj.cast(lookup_type("mongo::Decorable<{}>".format(type_name)))
+    start, count = get_decorable_info(decorable)
+    for i in range(count):
+        deco_type_name, obj, _ = get_object_decoration(decorable, start, i)
+        try:
+            yield (deco_type_name, obj)
+        except Exception as err:
+            print("Failed to look up decoration type: " + deco_type_name + ": " + str(err))
+
+
+def get_object_decoration(decorable, start, index):
+    decoration_data = get_unique_ptr_bytes(decorable["_decorations"]["_data"])
+    entry = start[index]
+    deco_type_info = str(entry["typeInfo"])
+    deco_type_name = re.sub(r'.* <typeinfo for (.*)>', r'\1', deco_type_info)
+    offset = int(entry["offset"])
+    obj = decoration_data[offset]
+    obj_addr = re.sub(r'^(.*) .*', r'\1', str(obj.address))
+    obj = _cast_decoration_value(deco_type_name, int(obj.address))
+    return (deco_type_name, obj, obj_addr)
+
+
+def get_decorable_info(decorable):
+    decorable_t = decorable.type.template_argument(0)
+    reg_sym, _ = gdb.lookup_symbol("mongo::decorable_detail::gdbRegistry<{}>".format(decorable_t))
+    decl_vector = reg_sym.value()["_entries"]
     start = decl_vector["_M_impl"]["_M_start"]
     finish = decl_vector["_M_impl"]["_M_finish"]
-
-    decorable_t = decorable.type.template_argument(0)
-    decinfo_t = gdb.lookup_type('mongo::DecorationRegistry<{}>::DecorationInfo'.format(
-        str(decorable_t).replace("class", "").strip()))
+    decinfo_t = lookup_type('mongo::decorable_detail::Registry::Entry')
     count = int((int(finish) - int(start)) / decinfo_t.sizeof)
+    return start, count
 
-    for i in range(count):
-        descriptor = start[i]
-        dindex = int(descriptor["descriptor"]["_index"])
 
-        type_name = str(descriptor["constructor"])
-        type_name = type_name[0:len(type_name) - 1]
-        type_name = type_name[0:type_name.rindex(">")]
-        type_name = type_name[type_name.index("constructAt<"):].replace("constructAt<", "")
-        # get_unique_ptr should be loaded from 'mongo_printers.py'.
-        decoration_data = get_unique_ptr_bytes(decorable["_decorations"]["_decorationData"])
-
-        if type_name.endswith('*'):
-            type_name = type_name[0:len(type_name) - 1]
-        type_name = type_name.rstrip()
-        try:
-            type_t = gdb.lookup_type(type_name)
-            obj = decoration_data[dindex].cast(type_t)
-            yield (type_name, obj)
-        except Exception as err:
-            print("Failed to look up decoration type: " + type_name + ": " + str(err))
+def _cast_decoration_value(type_name: str, decoration_address: int, /) -> gdb.Value:
+    # We cannot use gdb.lookup_type() when the decoration type is a pointer type, e.g.
+    # ServiceContext::declareDecoration<VectorClock*>(). gdb.parse_and_eval() is one of the few
+    # ways to convert a type expression into a gdb.Type value. Some care is taken to quote the
+    # non-pointer portion of the type so resolution for a type defined within an anonymous
+    # namespace works correctly.
+    type_name_regex = re.compile(r"^(.*[\w>])([\s\*]*)$")
+    escaped = type_name_regex.sub(r"'\1'\2*", type_name)
+    return gdb.parse_and_eval(f"({escaped}) {decoration_address}").dereference()
 
 
 def get_decoration(obj, type_name):
@@ -341,14 +381,14 @@ class GetMongoDecoration(gdb.Command):
     """
     Search for a decoration on an object by typename and print it e.g.
 
-    (gdb) mongo-decoration opCtx ReadConcernArgs
+    (gdb) mongodb-decoration opCtx ReadConcernArgs
 
     would print out a decoration on opCtx whose type name contains the string "ReadConcernArgs".
     """
 
     def __init__(self):
         """Initialize GetMongoDecoration."""
-        RegisterMongoCommand.register(self, "mongo-decoration", gdb.COMMAND_DATA)
+        RegisterMongoCommand.register(self, "mongodb-decoration", gdb.COMMAND_DATA)
 
     def invoke(self, args, _from_tty):
         """Invoke GetMongoDecoration."""
@@ -501,7 +541,7 @@ class DumpMongoDSessionCatalog(gdb.Command):
             val = get_boost_optional(txn_part_observable_state['txnResourceStash'])
             if val:
                 locker_addr = get_unique_ptr(val["_locker"])
-                locker_obj = locker_addr.dereference().cast(gdb.lookup_type("mongo::LockerImpl"))
+                locker_obj = locker_addr.dereference().cast(lookup_type("mongo::Locker"))
                 print('txnResourceStash._locker', "@", locker_addr)
                 print("txnResourceStash._locker._id", "=", locker_obj["_id"])
             else:
@@ -584,11 +624,7 @@ class MongoDBDumpLocks(gdb.Command):
         try:
             # Call into mongod, and dump the state of lock manager
             # Note that output will go to mongod's standard output, not the debugger output window
-            # Do not call mongo::getGlobalLockManager() due to the compiler optimizing this function in a very weird way
-            # See SERVER-72816 for more context
-            gdb.execute(
-                "call mongo::LockManager::get((mongo::ServiceContext*) mongo::getGlobalServiceContext())->dump()",
-                from_tty=False, to_string=False)
+            gdb.execute("call mongo::dumpLockManager()", from_tty=False, to_string=False)
         except gdb.error as gdberr:
             print("Ignoring error '%s' in dump_mongod_locks" % str(gdberr))
 
@@ -647,7 +683,7 @@ class MongoDBDumpRecoveryUnits(gdb.Command):
                 recovery_unit_handle = get_unique_ptr(operation_context["_recoveryUnit"])
                 # By default, cast the recovery unit as "mongo::WiredTigerRecoveryUnit"
                 recovery_unit = recovery_unit_handle.dereference().cast(
-                    gdb.lookup_type(recovery_unit_impl_type))
+                    lookup_type(recovery_unit_impl_type))
 
             output_doc["recoveryUnit"] = hex(recovery_unit_handle) if recovery_unit else "0x0"
             wt_session = get_wt_session(recovery_unit, recovery_unit_impl_type)
@@ -692,7 +728,7 @@ class MongoDBDumpRecoveryUnits(gdb.Command):
                 recovery_unit_handle = get_unique_ptr(txn_resource_stash["_recoveryUnit"])
                 # By default, cast the recovery unit as "mongo::WiredTigerRecoveryUnit"
                 recovery_unit = recovery_unit_handle.dereference().cast(
-                    gdb.lookup_type(recovery_unit_impl_type))
+                    lookup_type(recovery_unit_impl_type))
 
         output_doc["recoveryUnit"] = hex(recovery_unit_handle) if recovery_unit else "0x0"
         wt_session = get_wt_session(recovery_unit, recovery_unit_impl_type)
