@@ -15,8 +15,8 @@ if ROOT_PATH not in sys.path:
 from src.third_party.immer.dist.tools.gdb_pretty_printers.printers import ListIter as ImmerListIter  # pylint: disable=wrong-import-position
 
 if not gdb:
-    from buildscripts.gdb.mongo import get_boost_optional
-    from buildscripts.gdb.optimizer_printers import register_abt_printers
+    from buildscripts.gdb.mongo import get_boost_optional, lookup_type, get_decorable_info, get_object_decoration
+    from buildscripts.gdb.optimizer_printers import register_optimizer_printers
 
 try:
     import bson
@@ -118,7 +118,13 @@ class StringDataPrinter(object):
 
     def to_string(self):
         """Return data for printing."""
-        size = self.val["_size"]
+        # As of SERVER-82604, StringData is based on std::string_view, so try with that first
+        sv = self.val['_sv']
+        if sv is not None:
+            return sv
+
+        # ... back-off to the legacy format otherwise
+        size = self.val['_size']
         if size == -1:
             return self.val['_data'].lazy_string()
         return self.val['_data'].lazy_string(length=size)
@@ -142,7 +148,7 @@ class BSONObjPrinter(object):
     def __init__(self, val):
         """Initialize BSONObjPrinter."""
         self.val = val
-        self.ptr = self.val['_objdata'].cast(gdb.lookup_type('void').pointer())
+        self.ptr = self.val['_objdata'].cast(lookup_type('void').pointer())
         self.is_valid = False
 
         # Handle the endianness of the BSON object size, which is represented as a 32-bit integer
@@ -215,11 +221,6 @@ class OplogEntryPrinter(object):
         """Initialize OplogEntryPrinter."""
         self.val = val
 
-    @staticmethod
-    def display_hint():
-        """Display hint."""
-        return 'string'
-
     def to_string(self):
         """Return OplogEntry for printing."""
         optime = self.val['_entry']['_opTimeBase']
@@ -227,7 +228,7 @@ class OplogEntryPrinter(object):
         return "OplogEntry(%s, %s, %s, %s)" % (
             str(self.val['_entry']['_durableReplOperation']['_opType']).split('::')[-1],
             str(self.val['_entry']['_commandType']).split('::')[-1],
-            self.val['_entry']['_durableReplOperation']['_nss']['_ns'], optime_str)
+            self.val['_entry']['_durableReplOperation']['_nss'], optime_str)
 
 
 class UUIDPrinter(object):
@@ -263,7 +264,7 @@ class OIDPrinter(object):
 
     def to_string(self):
         """Return OID for printing."""
-        raw_bytes = [int(self.val['_data'][i]) for i in range(12)]
+        raw_bytes = [int(self.val['_data'][i]) for i in range(OBJECT_ID_WIDTH)]
         oid_hex_bytes = [hex(b & 0xFF)[2:].zfill(2) for b in raw_bytes]
         return "ObjectID('%s')" % "".join(oid_hex_bytes)
 
@@ -280,29 +281,35 @@ class RecordIdPrinter(object):
         """Display hint."""
         return 'string'
 
+    ## Get the address at given offset of data as the selected pointer type
+    def __get_data_address(self, ptr, offset):
+        ptr_type = gdb.lookup_type(ptr).pointer()
+        return self.val['_data']['_M_elems'][offset].address.cast(ptr_type)
+
     def to_string(self):
         """Return RecordId for printing."""
         rid_format = int(self.val["_format"])
         if rid_format == 0:
             return "null RecordId"
         elif rid_format == 1:
-            long_id = int(self.val['_data']['longId']['id'])
-            return "RecordId long: %d" % (long_id)
+            koffset = 8 - 1  ##  std::alignment_of_v<int64_t> - sizeof(Format); (see record_id.h)
+            rid_address = self.__get_data_address('int64_t', koffset)
+            return "RecordId long: %d" % int(rid_address.dereference())
         elif rid_format == 2:
-            inline_str = self.val['_data']['inlineStr']
-            str_len = int(inline_str['size'])
-            str_array = inline_str['dataArr']
-            # Reading the std::array elements
-            raw_bytes = [int(str_array['_M_elems'][i]) for i in range(0, str_len)]
+            str_len = self.__get_data_address('int8_t', 0).dereference()
+            array_address = self.__get_data_address('int8_t', 1)
+            raw_bytes = [array_address[i] for i in range(0, str_len)]
             hex_bytes = [hex(b & 0xFF)[2:].zfill(2) for b in raw_bytes]
             return "RecordId small string %d hex bytes: %s" % (str_len, str("".join(hex_bytes)))
         elif rid_format == 3:
-            holder_ptr = self.val['_data']['heapStr']["buffer"]['_buffer']["_holder"]["px"]
-            holder = holder_ptr.dereference()
+            koffset = 8 - 1  ## std::alignment_of_v<ConstSharedBuffer> - sizeof(Format); (see record_id.h)
+            buffer = self.__get_data_address('mongo::ConstSharedBuffer', koffset).dereference()
+            holder_ptr = holder = buffer['_buffer']["_holder"]["px"]
+            holder = holder.dereference()
             str_len = int(holder["_capacity"])
             # Start of data is immediately after pointer for holder
-            start_ptr = (holder_ptr + 1).dereference().cast(gdb.lookup_type("char")).address
-            raw_bytes = [int(start_ptr[i]) for i in range(0, str_len)]
+            start_ptr = (holder_ptr + 1).dereference().cast(lookup_type("char")).address
+            raw_bytes = [start_ptr[i] for i in range(0, str_len)]
             hex_bytes = [hex(b & 0xFF)[2:].zfill(2) for b in raw_bytes]
             return "RecordId big string %d hex bytes @ %s: %s" % (str_len, holder_ptr + 1,
                                                                   str("".join(hex_bytes)))
@@ -310,21 +317,72 @@ class RecordIdPrinter(object):
             return "unknown RecordId format: %d" % rid_format
 
 
+TENANT_ID_MASK = 0x80
+OBJECT_ID_WIDTH = 12
+
+
+def extract_tenant_id(data):
+    raw_bytes = [int(data[i]) for i in range(1, OBJECT_ID_WIDTH + 1)]
+    return "".join([hex(b & 0xFF)[2:].zfill(2) for b in raw_bytes])
+
+
+def is_small_string(flags):
+    return bool(flags & 0b00000010)
+
+
+def small_string_size(flags):
+    return flags >> 2
+
+
+class DatabaseNamePrinter(object):
+    """Pretty-printer for mongo::DatabaseName."""
+
+    def __init__(self, val):
+        """Initialize DatabaseNamePrinter."""
+        self.val = val
+
+    @staticmethod
+    def display_hint():
+        """Display hint."""
+        return 'string'
+
+    def _get_storage_data(self):
+        """Return the data pointer from the _data Storage class."""
+        data = self.val['_data']
+        footer = data['_footer']
+        f_size = footer.type.sizeof
+
+        # The last byte of _footer contain the flags (and the size when using small string).
+        flags = footer.cast(gdb.lookup_type('char').array(f_size))[f_size - 1]
+
+        data_ptr = data['_data']
+        if is_small_string(flags):
+            return data_ptr.address, small_string_size(flags)
+        else:
+            return data_ptr, data['_length']
+
+    def _get_string(self, address, size):
+        data = gdb.selected_inferior().read_memory(address, size).tobytes()
+        tenant = data[0] & TENANT_ID_MASK
+
+        if tenant:
+            return f"{extract_tenant_id(data)}_{data[1+OBJECT_ID_WIDTH:].decode()}"
+        else:
+            return data[1:].decode()
+
+    def to_string(self):
+        """Return string representation of NamespaceString."""
+        address, size = self._get_storage_data()
+        return self._get_string(address, size)
+
+
 class DecorablePrinter(object):
     """Pretty-printer for mongo::Decorable<>."""
 
     def __init__(self, val):
         """Initialize DecorablePrinter."""
         self.val = val
-
-        decl_vector = val["_decorations"]["_registry"]["_decorationInfo"]
-        # TODO: abstract out navigating a std::vector
-        self.start = decl_vector["_M_impl"]["_M_start"]
-        finish = decl_vector["_M_impl"]["_M_finish"]
-        decorable_t = val.type.template_argument(0)
-        decinfo_t = gdb.lookup_type('mongo::DecorationRegistry<{}>::DecorationInfo'.format(
-            str(decorable_t).replace("class", "").strip()))
-        self.count = int((int(finish) - int(self.start)) / decinfo_t.sizeof)
+        self.start, self.count = get_decorable_info(val)
 
     @staticmethod
     def display_hint():
@@ -333,35 +391,17 @@ class DecorablePrinter(object):
 
     def to_string(self):
         """Return Decorable for printing."""
-        return "Decorable<%s> with %s elems " % (self.val.type.template_argument(0), self.count)
+        return "Decorable<{}> with {} elems ".format(self.val.type.template_argument(0), self.count)
 
     def children(self):
         """Children."""
-        decoration_data = get_unique_ptr_bytes(self.val["_decorations"]["_decorationData"])
-
         for index in range(self.count):
-            descriptor = self.start[index]
-            dindex = int(descriptor["descriptor"]["_index"])
-
-            # In order to get the type stored in the decorable, we examine the type of its
-            # constructor, and do some string manipulations.
-            # TODO: abstract out navigating a std::function
-            type_name = str(descriptor["constructor"])
-            type_name = type_name[0:len(type_name) - 1]
-            type_name = type_name[0:type_name.rindex(">")]
-            type_name = type_name[type_name.index("constructAt<"):].replace("constructAt<", "")
-
-            # If the type is a pointer type, strip the * at the end.
-            if type_name.endswith('*'):
-                type_name = type_name[0:len(type_name) - 1]
-            type_name = type_name.rstrip()
-
-            # Cast the raw char[] into the actual object that is stored there.
-            type_t = gdb.lookup_type(type_name)
-            obj = decoration_data[dindex].cast(type_t)
-
-            yield ('key', "%d:%s:%s" % (index, obj.address, type_name))
-            yield ('value', obj)
+            try:
+                deco_type_name, obj, obj_addr = get_object_decoration(self.val, self.start, index)
+                yield ('key', "{}:{}:{}".format(index, obj_addr, deco_type_name))
+                yield ('value', obj)
+            except Exception as err:
+                print("Failed to look up decoration type: " + deco_type_name + ": " + str(err))
 
 
 def _get_flags(flag_val, flags):
@@ -488,23 +528,30 @@ class WtTxnPrinter(object):
                 yield (field.name, field_val)
 
 
+def absl_container_size(val):
+    return val["settings_"]["value"]["compressed_tuple_"]["value"]
+
+
 def absl_get_nodes(val):
     """Return a generator of every node in absl::container_internal::raw_hash_set and derived classes."""
-    size = val["size_"]
+    size = absl_container_size(val)
 
     if size == 0:
         return
 
     table = val
-    capacity = int(table["capacity_"])
-    ctrl = table["ctrl_"]
+    capacity = int(table["settings_"]["value"]["capacity_"])
+    ctrl = table["settings_"]["value"]["control_"]
+
+    # Derive the underlying type stored in the container.
+    slot_type = lookup_type(str(val.type.strip_typedefs()) + "::slot_type").strip_typedefs()
 
     # Using the array of ctrl bytes, search for in-use slots and return them
-    # https://github.com/abseil/abseil-cpp/blob/7ffbe09f3d85504bd018783bbe1e2c12992fe47c/absl/container/internal/raw_hash_set.h#L787-L788
+    # https://github.com/abseil/abseil-cpp/blob/8a3caf7dea955b513a6c1b572a2423c6b4213402/absl/container/internal/raw_hash_set.h#L2108-L2113
     for item in range(capacity):
         ctrl_t = int(ctrl[item])
         if ctrl_t >= 0:
-            yield table["slots_"][item]
+            yield table["settings_"]["value"]["slots_"].cast(slot_type.pointer())[item]
 
 
 class AbslHashSetPrinterBase(object):
@@ -523,7 +570,7 @@ class AbslHashSetPrinterBase(object):
     def to_string(self):
         """Return absl::[node/flat]_hash_set for printing."""
         return "absl::%s_hash_set<%s> with %s elems " % (
-            self.to_str, self.val.type.template_argument(0), self.val["size_"])
+            self.to_str, self.val.type.template_argument(0), absl_container_size(self.val))
 
 
 class AbslNodeHashSetPrinter(AbslHashSetPrinterBase):
@@ -573,7 +620,7 @@ class AbslHashMapPrinterBase(object):
         """Return absl::[node/flat]_hash_map for printing."""
         return "absl::%s_hash_map<%s, %s> with %s elems " % (
             self.to_str, self.val.type.template_argument(0), self.val.type.template_argument(1),
-            self.val["size_"])
+            absl_container_size(self.val))
 
 
 class AbslNodeHashMapPrinter(AbslHashMapPrinterBase):
@@ -601,7 +648,7 @@ class AbslFlatHashMapPrinter(AbslHashMapPrinterBase):
         """Children."""
         for kvp in absl_get_nodes(self.val):
             yield ('key', kvp['key'])
-            yield ('value', kvp['value'])
+            yield ('value', kvp['value']["second"])
 
 
 class ImmutableMapIter(ImmerListIter):
@@ -791,7 +838,7 @@ def make_inverse_enum_dict(enum_type_name):
     For example, if the enum type is 'mongo::sbe::vm::Builtin' with an element 'regexMatch', the
     dictionary will contain 'regexMatch' value and not 'mongo::sbe::vm::Builtin::regexMatch'.
     """
-    enum_dict = gdb.types.make_enum_dict(gdb.lookup_type(enum_type_name))
+    enum_dict = gdb.types.make_enum_dict(lookup_type(enum_type_name))
     enum_inverse_dic = dict()
     for key, value in enum_dict.items():
         enum_inverse_dic[int(value)] = key.split('::')[-1]  # take last element
@@ -838,11 +885,11 @@ class SbeCodeFragmentPrinter(object):
         # either use an inline buffer or an allocated one. The choice of storage is decoded in the
         # last bit of the 'metadata_' field.
         storage = self.val['_instrs']['storage_']
-        meta = storage['metadata_'].cast(gdb.lookup_type('size_t'))
+        meta = storage['metadata_'].cast(lookup_type('size_t'))
         self.is_inlined = (meta % 2 == 0)
         self.size = (meta >> 1)
         self.pdata = \
-            storage['data_']['inlined']['inlined_data'].cast(gdb.lookup_type('uint8_t').pointer()) \
+            storage['data_']['inlined']['inlined_data'].cast(lookup_type('uint8_t').pointer()) \
             if self.is_inlined \
             else storage['data_']['allocated']['allocated_data']
 
@@ -866,17 +913,17 @@ class SbeCodeFragmentPrinter(object):
         yield 'instrs total size', self.size
 
         # Sizes for types we'll use when parsing the insructions stream.
-        int_size = gdb.lookup_type('int').sizeof
-        ptr_size = gdb.lookup_type('void').pointer().sizeof
-        tag_size = gdb.lookup_type('mongo::sbe::value::TypeTags').sizeof
-        value_size = gdb.lookup_type('mongo::sbe::value::Value').sizeof
-        uint8_size = gdb.lookup_type('uint8_t').sizeof
-        uint32_size = gdb.lookup_type('uint32_t').sizeof
-        uint64_size = gdb.lookup_type('uint64_t').sizeof
-        builtin_size = gdb.lookup_type('mongo::sbe::vm::Builtin').sizeof
-        time_unit_size = gdb.lookup_type('mongo::TimeUnit').sizeof
-        timezone_size = gdb.lookup_type('mongo::TimeZone').sizeof
-        day_of_week_size = gdb.lookup_type('mongo::DayOfWeek').sizeof
+        int_size = lookup_type('int').sizeof
+        ptr_size = lookup_type('void').pointer().sizeof
+        tag_size = lookup_type('mongo::sbe::value::TypeTags').sizeof
+        value_size = lookup_type('mongo::sbe::value::Value').sizeof
+        uint8_size = lookup_type('uint8_t').sizeof
+        uint32_size = lookup_type('uint32_t').sizeof
+        uint64_size = lookup_type('uint64_t').sizeof
+        builtin_size = lookup_type('mongo::sbe::vm::Builtin').sizeof
+        time_unit_size = lookup_type('mongo::TimeUnit').sizeof
+        timezone_size = lookup_type('mongo::TimeZone').sizeof
+        day_of_week_size = lookup_type('mongo::DayOfWeek').sizeof
 
         cur_op = self.pdata
         end_op = self.pdata + self.size
@@ -921,9 +968,9 @@ class SbeCodeFragmentPrinter(object):
                 cur_op += uint32_size
             elif op_name in ['function', 'functionSmall']:
                 arity_size = \
-                    gdb.lookup_type('mongo::sbe::vm::ArityType').sizeof \
+                    lookup_type('mongo::sbe::vm::ArityType').sizeof \
                     if op_name == 'function' \
-                    else gdb.lookup_type('mongo::sbe::vm::SmallArityType').sizeof
+                    else lookup_type('mongo::sbe::vm::SmallArityType').sizeof
                 builtin_id = read_as_integer(cur_op, builtin_size)
                 args = 'builtin: ' + self.builtins_lookup.get(builtin_id, "unknown")
                 args += ' arity: ' + str(read_as_integer(cur_op + builtin_size, arity_size))
@@ -966,14 +1013,16 @@ def build_pretty_printer():
     """Build a pretty printer."""
     pp = MongoPrettyPrinterCollection()
     pp.add('BSONObj', 'mongo::BSONObj', False, BSONObjPrinter)
+    pp.add('DatabaseName', 'mongo::DatabaseName', False, DatabaseNamePrinter)
+    pp.add('NamespaceString', 'mongo::NamespaceString', False, DatabaseNamePrinter)
     pp.add('Decorable', 'mongo::Decorable', True, DecorablePrinter)
     pp.add('Status', 'mongo::Status', False, StatusPrinter)
     pp.add('StatusWith', 'mongo::StatusWith', True, StatusWithPrinter)
     pp.add('StringData', 'mongo::StringData', False, StringDataPrinter)
-    pp.add('node_hash_map', 'absl::lts_20211102::node_hash_map', True, AbslNodeHashMapPrinter)
-    pp.add('node_hash_set', 'absl::lts_20211102::node_hash_set', True, AbslNodeHashSetPrinter)
-    pp.add('flat_hash_map', 'absl::lts_20211102::flat_hash_map', True, AbslFlatHashMapPrinter)
-    pp.add('flat_hash_set', 'absl::lts_20211102::flat_hash_set', True, AbslFlatHashSetPrinter)
+    pp.add('node_hash_map', 'absl::lts_20230802::node_hash_map', True, AbslNodeHashMapPrinter)
+    pp.add('node_hash_set', 'absl::lts_20230802::node_hash_set', True, AbslNodeHashSetPrinter)
+    pp.add('flat_hash_map', 'absl::lts_20230802::flat_hash_map', True, AbslFlatHashMapPrinter)
+    pp.add('flat_hash_set', 'absl::lts_20230802::flat_hash_set', True, AbslFlatHashSetPrinter)
     pp.add('RecordId', 'mongo::RecordId', False, RecordIdPrinter)
     pp.add('UUID', 'mongo::UUID', False, UUIDPrinter)
     pp.add('OID', 'mongo::OID', False, OIDPrinter)
@@ -988,7 +1037,7 @@ def build_pretty_printer():
     pp.add('immutable::set', 'mongo::immutable::set', True, ImmutableSetPrinter)
 
     # Optimizer/ABT related pretty printers that can be used only with a running process.
-    register_abt_printers(pp)
+    register_optimizer_printers(pp)
 
     return pp
 
