@@ -113,6 +113,9 @@ ALLOW_ANY_TYPE_LIST: List[str] = [
     'aggregate-param-allowDiskUse',
     'aggregate-param-cursor',
     'aggregate-param-hint',
+    'aggregate-param-comment',
+    'aggregate-param-allowedIndexes',
+    'aggregate-param-indexHints',
     'aggregate-param-needsMerge',
     'aggregate-param-fromMongos',
     'aggregate-param-$_requestReshardingResumeToken',
@@ -127,6 +130,9 @@ ALLOW_ANY_TYPE_LIST: List[str] = [
     'find-param-projection',
     'find-param-sort',
     'find-param-hint',
+    'find-param-comment',
+    'find-param-allowedIndexes',
+    'find-param-indexHints',
     'find-param-collation',
     'find-param-singleBatch',
     'find-param-allowDiskUse',
@@ -144,7 +150,6 @@ ALLOW_ANY_TYPE_LIST: List[str] = [
     'find-param-allowSpeculativeMajorityRead',
     'find-param-$_requestResumeToken',
     'find-param-$_resumeAfter',
-    "find-param-$_startAt",
     'find-param-maxTimeMS',
     'update-param-u',
     'update-param-hint',
@@ -168,13 +173,47 @@ ALLOW_ANY_TYPE_LIST: List[str] = [
     'create-param-min',
     'create-param-max',
     'bulkWrite-param-updateMods',
-    'bulkWrite-param-hint'
+    'bulkWrite-param-hint',
+
+    # No actual user-facing difference
+    'bulkWrite-reply-opTime',
+    'getMore-param-lastKnownCommittedOpTime',
+    'hello-reply-opTime',
+    'hello-reply-majorityOpTime',
+]
+
+# Permit a parameter to move from bson serialisation type any
+# to a non-any serialisation type.
+IGNORE_ANY_TO_NON_ANY_LIST: List[str] = [
+    # These parameters were type-checked "by hand" previously;
+    # enforcing this from IDL instead does not narrow the range
+    # of permitted values
+    'find-param-maxTimeMS',
+    'count-param-maxTimeMS',
+]
+
+# Permit a parameter to move from a non-any bson serialisation type to any.
+IGNORE_NON_ANY_TO_ANY_LIST: List[str] = [
+    'aggregate-param-indexHints',
+    'bulkWrite-reply-opTime',
+    'find-param-indexHints',
+    'getMore-param-lastKnownCommittedOpTime',
+    'hello-reply-opTime',
+    'hello-reply-majorityOpTime',
+]
+
+# Permit the cpp type of a parameter to change
+ALLOW_CPP_TYPE_CHANGE_LIST: List[str] = [
+    # maxTimeMS has been widened for consistency with
+    # equivalent params for other commands (aggregate)
+    'find-param-maxTimeMS-std::int32_t-std::int64_t',
+    'count-param-maxTimeMS-std::int32_t-std::int64_t',
 ]
 
 # Do not add user visible fields already released in earlier versions.
 # We generally don't allow changing a field from stable to unstable, but we permit it in special cases,
 # such as when we want to avoid making internal fields part of the stable API.
-# Additions to this list must be approved by the Stable API PM and code reviewed by the Replication
+# Additions to this list must be approved by the Stable API PM and code reviewed by the Query Optimization
 # team.
 IGNORE_STABLE_TO_UNSTABLE_LIST: List[str] = [
     # This list is only used in unit-tests.
@@ -196,51 +235,13 @@ IGNORE_STABLE_TO_UNSTABLE_LIST: List[str] = [
     'newReplyFieldTypeStructIgnoreList-reply-unstableNewFieldIgnoreList',
 
     # Real use cases for changing a field from 'stable' to 'unstable'.
+    'find-param-maxTimeMS',
+    'count-param-maxTimeMS',
 
-    # The 'originalSpec' field was introduced in v5.1 behind a disabled feature flag and is not user
-    # visible. This is part of the listIndexes output when executed against system.bucket.*
-    # collections, which users should avoid doing.
-    'listIndexes-reply-originalSpec',
-    # The 'vars' field was introduced to facilitate communication between mongot and mongod and is
-    # not user visible.
-    'find-reply-vars',
-    'aggregate-reply-vars',
-    # The 'cursor' field is now optional in a reply, as inter-node communication in aggregation
-    # can return one or more cursors. Multiple cursors are covered under the 'cursors' field.
-    'find-reply-cursor',
-    'aggregate-reply-cursor',
-    # The 'ignoreUnknownIndexOptions' field is for internal use only and is not documented to users.
-    'createIndexes-param-ignoreUnknownIndexOptions',
-    # The 'runtimeConstants' field is a legacy field for internal use only and is not documented to
-    # users.
-    'delete-param-runtimeConstants',
-    # The 'isTimeseriesNamespace' field is sent from mongos to shards for internal use.
-    'collMod-param-isTimeseriesNamespace',
-    'createIndexes-param-isTimeseriesNamespace',
-    'dropIndexes-param-isTimeseriesNamespace',
-    'listIndexes-param-isTimeseriesNamespace',
-    'insert-param-isTimeseriesNamespace',
-    'update-param-isTimeseriesNamespace',
-    'delete-param-isTimeseriesNamespace',
-    'findAndModify-param-isTimeseriesNamespace',
-    # The 'needsMerge' and 'fromMongos' fields of aggregation are sent from mongos to shards for internal use.
-    'aggregate-param-needsMerge',
-    'aggregate-param-fromMongos',
-    # Bulk fixes for fields that are strictly internal all along and should thus be marked unstable.
-    'aggregate-param-$_generateV2ResumeTokens',
-    'endSessions-param-txnNumber',
-    'endSessions-param-txnUUID',
-    'findAndModify-param-stmtId',
-    'hello-reply-cwwc',
-    'hello-reply-isImplicitDefaultMajorityWC',
-    'hello-param-loadBalanced',
-    'hello-reply-serviceId',
-    'refreshSessions-param-txnNumber',
-    'refreshSessions-param-txnUUID',
-    # upsertSupplied is an internal implementation detail of $merge
-    'update-param-upsertSupplied',
     # No actual user-facing difference
-    'find-param-$_startAt',
+    'bulkWrite-reply-opTime',
+    'hello-reply-opTime',
+    'hello-reply-majorityOpTime',
 ]
 
 # Once a field is part of the stable API, either by direct addition or by changing it from unstable
@@ -248,7 +249,7 @@ IGNORE_STABLE_TO_UNSTABLE_LIST: List[str] = [
 # want to make sure such changes are always intentional. Therefore, the checker will throw errors
 # unless the field is also added to this list below, with which the author ackhowledges they are
 # aware of the above implications.
-# Additions to this list must be approved by the Stable API PM and code reviewed by the Replication
+# Additions to this list must be approved by the Stable API PM and code reviewed by the Query Optimization
 # team.
 ALLOWED_STABLE_FIELDS_LIST: List[str] = [
     # This list is only used in unit-tests. These cases modify fields from unstable to stable.
@@ -270,7 +271,9 @@ ALLOWED_STABLE_FIELDS_LIST: List[str] = [
     'commandCppTypeNotEqualUnstable-param-cppTypeNotEqualStructUnstableField',
     'newlyAddedTypeFieldBsonAnyNotAllowed-param-newlyAddedBsonSerializationTypeAnyStructField',
     'typeWithIncompatibleChainedStruct-param-newBsonSerializationTypeAnyUnstableStructField',
+    'addedCommandParameterDefault-param-newStableParameter',
     'addedCommandParameterStable-param-newOptionalStableParam',
+    'addedCommandParameterStableRequired-param-newStableParam',
     'addedCommandParameterStableWithDefault-param-newStableParamWithDefault',
     'newCommandParameterTypeStructRecursiveOne-param-unstableToStableOptionalField',
     'oldUnstableParamTypeChanges-param-oldUnstableTypeChangesParam',
@@ -323,11 +326,47 @@ ALLOWED_STABLE_FIELDS_LIST: List[str] = [
     'update-param-isTimeseriesNamespace',
     'delete-param-isTimeseriesNamespace',
     'findAndModify-param-stmtId',
-    'aggregate-param-$_generateV2ResumeTokens',
     'hello-param-loadBalanced',
     'hello-reply-serviceId',
     'hello-reply-isImplicitDefaultMajorityWC',
     'hello-reply-cwwc',
+
+    # BulkWrite fields
+    'bulkWrite-param-ops',
+    'bulkWrite-param-insert',
+    'bulkWrite-param-document',
+    'bulkWrite-param-update',
+    'bulkWrite-param-filter',
+    'bulkWrite-param-multi',
+    'bulkWrite-param-updateMods',
+    'bulkWrite-param-upsert',
+    'bulkWrite-param-arrayFilters',
+    'bulkWrite-param-hint',
+    'bulkWrite-param-collation',
+    'bulkWrite-param-delete',
+    'bulkWrite-param-collation',
+    'bulkWrite-param-nsInfo',
+    'bulkWrite-param-ns',
+    'bulkWrite-param-cursor',
+    'bulkWrite-param-bypassDocumentValidation',
+    'bulkWrite-param-constants',
+    'bulkWrite-param-ordered',
+    'bulkWrite-param-stmtId',
+    'bulkWrite-param-stmtIds',
+    'bulkWrite-param-let',
+    'bulkWrite-param-errorsOnly',
+    'bulkWrite-reply-cursor',
+    'bulkWrite-reply-id',
+    'bulkWrite-reply-firstBatch',
+    'bulkWrite-reply-ns',
+    'bulkWrite-reply-electionId',
+    'bulkWrite-reply-opTime',
+    'bulkWrite-reply-nErrors',
+    'bulkWrite-reply-nInserted',
+    'bulkWrite-reply-nMatched',
+    'bulkWrite-reply-nModified',
+    'bulkWrite-reply-nUpserted',
+    'bulkWrite-reply-nDeleted',
 ]
 
 SKIPPED_FILES = [
@@ -443,6 +482,11 @@ def is_unstable(stability: Optional[str]) -> bool:
     return stability is not None and stability != 'stable'
 
 
+def is_stable(stability: Optional[str]) -> bool:
+    """Check whether the given stability value is considered as stable."""
+    return not is_unstable(stability)
+
+
 def get_new_commands(
         ctxt: IDLCompatibilityContext, new_idl_dir: str, import_directories: List[str]
 ) -> Tuple[Dict[str, syntax.Command], Dict[str, syntax.IDLParsedSpec], Dict[str, str]]:
@@ -460,7 +504,7 @@ def get_new_commands(
             with open(new_idl_file_path) as new_file:
                 new_idl_file = parser.parse(
                     new_file, new_idl_file_path,
-                    CompilerImportResolver(import_directories + [new_idl_dir]))
+                    CompilerImportResolver(import_directories + [new_idl_dir]), False)
                 if new_idl_file.errors:
                     new_idl_file.errors.dump_errors()
                     raise ValueError(f"Cannot parse {new_idl_file_path}")
@@ -562,9 +606,10 @@ def check_reply_field_type_recursive(ctxt: IDLCompatibilityContext,
 
     # If bson_serialization_type switches from non-any to 'any' type.
     if "any" not in old_field_type.bson_serialization_type and "any" in new_field_type.bson_serialization_type:
-        ctxt.add_new_reply_field_bson_any_error(cmd_name, field_name, old_field_type.name,
-                                                new_field_type.name, new_field.idl_file_path)
-        return
+        if ignore_list_name not in IGNORE_NON_ANY_TO_ANY_LIST:
+            ctxt.add_new_reply_field_bson_any_error(cmd_name, field_name, old_field_type.name,
+                                                    new_field_type.name, new_field.idl_file_path)
+            return
 
     if "any" in old_field_type.bson_serialization_type:
         # If 'any' is not explicitly allowed as the bson_serialization_type.
@@ -900,13 +945,14 @@ def check_param_or_command_type_recursive(ctxt: IDLCompatibilityContext,
 
     # If bson_serialization_type switches from 'any' to non-any type.
     if "any" in old_type.bson_serialization_type and "any" not in new_type.bson_serialization_type:
-        ctxt.add_old_command_or_param_type_bson_any_error(cmd_name, old_type.name, new_type.name,
-                                                          old_field.idl_file_path, param_name,
-                                                          is_command_parameter)
-        return
+        if ignore_list_name not in IGNORE_ANY_TO_NON_ANY_LIST:
+            ctxt.add_old_command_or_param_type_bson_any_error(
+                cmd_name, old_type.name, new_type.name, old_field.idl_file_path, param_name,
+                is_command_parameter)
+            return
 
     # If bson_serialization_type switches from non-any to 'any' type.
-    if "any" not in old_type.bson_serialization_type and "any" in new_type.bson_serialization_type:
+    if "any" not in old_type.bson_serialization_type and "any" in new_type.bson_serialization_type and ignore_list_name not in IGNORE_NON_ANY_TO_ANY_LIST:
         ctxt.add_new_command_or_param_type_bson_any_error(cmd_name, old_type.name, new_type.name,
                                                           new_field.idl_file_path, param_name,
                                                           is_command_parameter)
@@ -921,8 +967,11 @@ def check_param_or_command_type_recursive(ctxt: IDLCompatibilityContext,
 
         # If cpp_type is changed, it's a potential breaking change.
         if old_type.cpp_type != new_type.cpp_type:
-            ctxt.add_command_or_param_cpp_type_not_equal_error(
-                cmd_name, new_type.name, new_field.idl_file_path, param_name, is_command_parameter)
+            ignore_list_name_with_types: str = f"{ignore_list_name}-{old_type.cpp_type}-{new_type.cpp_type}"
+            if ignore_list_name_with_types not in ALLOW_CPP_TYPE_CHANGE_LIST:
+                ctxt.add_command_or_param_cpp_type_not_equal_error(cmd_name, new_type.name,
+                                                                   new_field.idl_file_path,
+                                                                   param_name, is_command_parameter)
 
         # If serializer is changed, it's a potential breaking change.
         if (not is_unstable(old_field.stability)
@@ -1107,9 +1156,7 @@ def check_param_or_type_validator(ctxt: IDLCompatibilityContext, old_field: synt
     # SERVER-71601.
     #
     # Do not add additional parameters to this list.
-    ignore_validator_check_list = [
-        "create-param-bucketMaxSpanSeconds", "create-param-bucketRoundingSeconds"
-    ]
+    ignore_validator_check_list: List[str] = []
 
     if new_field.validator:
         if old_field.validator:
@@ -1182,9 +1229,6 @@ def check_command_params_or_type_struct_fields(
     # We allow collMod isTimeseriesNamespace parameter to be removed because it's implicitly
     # added from mongos and not documented in the API.
     allow_list += ["collMod-param-isTimeseriesNamespace"]
-    # We allow collMod "recordPreImages" parameter to be removed because it was incorrectly marked as stable
-    # in 5.0.x versions.
-    allow_list += ["collMod-param-recordPreImages"]
 
     for old_field in old_struct_fields or []:
         new_field_exists = False
@@ -1225,11 +1269,16 @@ def check_command_params_or_type_struct_fields(
             new_field_type = get_field_type(new_field, new_idl_file, new_idl_file_path)
             new_field_optional = new_field.optional or (new_field_type
                                                         and new_field_type.name == 'optionalBool')
-            if not new_field_optional and not is_unstable(new_field.stability):
+            if not new_field_optional and new_field.default is None and not is_unstable(
+                    new_field.stability):
                 ctxt.add_new_param_or_command_type_field_added_required_error(
                     cmd_name, new_field.name, new_idl_file_path, new_struct.name,
                     is_command_parameter)
 
+            if is_unstable(new_field.stability) and not new_field_optional:
+                ctxt.add_new_param_or_type_field_added_as_unstable_required_error(
+                    cmd_name, new_field.name, new_idl_file_path, is_command_parameter)
+
             # Check that a new field does not have an unallowed use of 'any' as the bson_serialization_type.
             any_allow_name: str = (cmd_name + "-param-" + new_field.name
                                    if is_command_parameter else cmd_name)
@@ -1264,7 +1313,7 @@ def check_command_param_or_type_struct_field(
             cmd_name, old_field.name, old_idl_file_path, is_command_parameter)
 
     # If old field is unstable and new field is stable, the new field should either be optional or
-    # have a default value.
+    # have a default value, unless the old field was a required field.
     old_field_type = get_field_type(old_field, old_idl_file, old_idl_file_path)
     new_field_type = get_field_type(new_field, new_idl_file, new_idl_file_path)
     old_field_optional = old_field.optional or (old_field_type
@@ -1273,8 +1322,10 @@ def check_command_param_or_type_struct_field(
                                                 and new_field_type.name == "optionalBool")
     if is_unstable(old_field.stability) and not is_unstable(
             new_field.stability) and not new_field_optional and new_field.default is None:
-        ctxt.add_new_param_or_command_type_field_stable_required_no_default_error(
-            cmd_name, old_field.name, old_idl_file_path, type_name, is_command_parameter)
+        # Only error if the old field was not a required field already.
+        if old_field_optional or old_field.default is not None:
+            ctxt.add_new_param_or_command_type_field_stable_required_no_default_error(
+                cmd_name, old_field.name, old_idl_file_path, type_name, is_command_parameter)
 
     if not is_unstable(
             old_field.stability
@@ -1351,9 +1402,11 @@ def check_error_reply(old_basic_types_path: str, new_basic_types_path: str,
     ctxt = IDLCompatibilityContext(old_idl_dir, new_idl_dir, IDLCompatibilityErrorCollection())
     with open(old_basic_types_path) as old_file:
         old_idl_file = parser.parse(old_file, old_basic_types_path,
-                                    CompilerImportResolver(old_import_directories))
+                                    CompilerImportResolver(old_import_directories), False)
         if old_idl_file.errors:
             old_idl_file.errors.dump_errors()
+            # If parsing old IDL files fails, it might be because the parser has been recently
+            # updated to require something that isn't present in older IDL files.
             raise ValueError(f"Cannot parse {old_basic_types_path}")
 
         old_error_reply_struct = old_idl_file.spec.symbols.get_struct("ErrorReply")
@@ -1363,7 +1416,7 @@ def check_error_reply(old_basic_types_path: str, new_basic_types_path: str,
         else:
             with open(new_basic_types_path) as new_file:
                 new_idl_file = parser.parse(new_file, new_basic_types_path,
-                                            CompilerImportResolver(new_import_directories))
+                                            CompilerImportResolver(new_import_directories), False)
                 if new_idl_file.errors:
                     new_idl_file.errors.dump_errors()
                     raise ValueError(f"Cannot parse {new_basic_types_path}")
@@ -1537,9 +1590,11 @@ def check_compatibility(old_idl_dir: str, new_idl_dir: str, old_import_directori
             with open(old_idl_file_path) as old_file:
                 old_idl_file = parser.parse(
                     old_file, old_idl_file_path,
-                    CompilerImportResolver(old_import_directories + [old_idl_dir]))
+                    CompilerImportResolver(old_import_directories + [old_idl_dir]), False)
                 if old_idl_file.errors:
                     old_idl_file.errors.dump_errors()
+                    # If parsing old IDL files fails, it might be because the parser has been
+                    # recently updated to require something that isn't present in older IDL files.
                     raise ValueError(f"Cannot parse {old_idl_file_path}")
 
                 for old_cmd in old_idl_file.spec.symbols.commands:
@@ -1599,31 +1654,52 @@ def check_compatibility(old_idl_dir: str, new_idl_dir: str, old_import_directori
     return ctxt.errors
 
 
-def get_generic_arguments(gen_args_file_path: str, includes: str) -> Tuple[Set[str], Set[str]]:
+def get_generic_arguments(gen_args_file_path: str,
+                          includes: List[str]) -> Tuple[Set[str], Set[str]]:
     """Get arguments and reply fields from generic_argument.idl and check validity."""
     arguments: Set[str] = set()
     reply_fields: Set[str] = set()
 
     with open(gen_args_file_path) as gen_args_file:
         parsed_idl_file = parser.parse(gen_args_file, gen_args_file_path,
-                                       CompilerImportResolver(includes))
+                                       CompilerImportResolver(includes), False)
         if parsed_idl_file.errors:
             parsed_idl_file.errors.dump_errors()
-            raise ValueError(f"Cannot parse {gen_args_file_path}")
-        for argument in parsed_idl_file.spec.symbols.get_generic_argument_list(
-                "generic_args_api_v1").fields:
-            arguments.add(argument.name)
+            raise ValueError(f"Cannot parse {gen_args_file_path} {parsed_idl_file.errors}")
 
-        for reply_field in parsed_idl_file.spec.symbols.get_generic_reply_field_list(
-                "generic_reply_fields_api_v1").fields:
-            reply_fields.add(reply_field.name)
+        # The generic argument/reply field structs have been renamed a few times, so to
+        # account for this when comparing against older releases, we try each set of names.
+        struct_names = [
+            # 8.0.0rc5 and forward
+            ("GenericArguments", "GenericReplyFields"),
+            # 8.0.0rc4
+            ("GenericArgsAPIV1", "GenericReplyFieldsAPIV1"),
+            # Before 8.0.0rc4
+            ("generic_args_api_v1", "generic_reply_fields_api_v1")
+        ]
+        for args_struct, reply_struct in struct_names:
+            generic_arguments = parsed_idl_file.spec.symbols.get_generic_argument_list(args_struct)
+            if generic_arguments is None:
+                continue
+            else:
+                generic_reply_fields = parsed_idl_file.spec.symbols.get_generic_reply_field_list(
+                    reply_struct)
+                break
+
+        for argument in generic_arguments.fields:
+            if is_stable(argument.stability):
+                arguments.add(argument.name)
+
+        for reply_field in generic_reply_fields.fields:
+            if is_stable(reply_field.stability):
+                reply_fields.add(reply_field.name)
 
     return arguments, reply_fields
 
 
-def check_generic_arguments_compatibility(old_gen_args_file_path: str, new_gen_args_file_path: str,
-                                          old_includes: str,
-                                          new_includes: str) -> IDLCompatibilityErrorCollection:
+def check_generic_arguments_compatibility(
+        old_gen_args_file_path: str, new_gen_args_file_path: str, old_includes: List[str],
+        new_includes: List[str]) -> IDLCompatibilityErrorCollection:
     """Check IDL compatibility between old and new generic_argument.idl files."""
     # IDLCompatibilityContext takes in both 'old_idl_dir' and 'new_idl_dir',
     # but for generic_argument.idl, the parent directories aren't helpful for logging purposes.
