@@ -77,6 +77,7 @@ class TestBinder(testcase.IDLTestcase):
             bson_serialization_type: object
             serializer: foo
             deserializer: foo
+            is_view: false
 
         bool:
             description: foo
@@ -84,6 +85,7 @@ class TestBinder(testcase.IDLTestcase):
             bson_serialization_type: any
             serializer: foo
             deserializer: foo
+            is_view: false
 
         string:
             description: foo
@@ -91,6 +93,7 @@ class TestBinder(testcase.IDLTestcase):
             bson_serialization_type: string
             serializer: foo
             deserializer: foo
+            is_view: false
 
         any_type:
             description: foo
@@ -98,6 +101,7 @@ class TestBinder(testcase.IDLTestcase):
             bson_serialization_type: any
             serializer: foo
             deserializer: foo
+            is_view: false
 
         tenant_id:
             bson_serialization_type: any
@@ -105,6 +109,7 @@ class TestBinder(testcase.IDLTestcase):
             cpp_type: foo
             deserializer: foo
             serializer: foo
+            is_view: false
 
         database_name:
             bson_serialization_type: string
@@ -112,12 +117,14 @@ class TestBinder(testcase.IDLTestcase):
             cpp_type: foo
             serializer: foo
             deserializer: foo
+            is_view: false
 
         serialization_context:
             bson_serialization_type: any
             description: foo
             cpp_type: foo
             internal_only: true
+            is_view: false
     """)
 
     def test_empty(self):
@@ -131,13 +138,29 @@ class TestBinder(testcase.IDLTestcase):
         spec = self.assert_bind(
             textwrap.dedent("""
         global:
-            cpp_namespace: 'something'
+            cpp_namespace: 'mongo'
             cpp_includes:
                 - 'bar'
                 - 'foo'"""))
-        self.assertEqual(spec.globals.cpp_namespace, "something")
+        self.assertEqual(spec.globals.cpp_namespace, "mongo")
         self.assertListEqual(spec.globals.cpp_includes, ['bar', 'foo'])
 
+        spec = self.assert_bind(
+            textwrap.dedent("""
+        global:
+            cpp_namespace: 'mongo::nested'
+        """))
+        self.assertEqual(spec.globals.cpp_namespace, "mongo::nested")
+
+    def test_global_negatives(self):
+        # type: () -> None
+        """Postive global tests."""
+        self.assert_bind_fail(
+            textwrap.dedent("""
+        global:
+            cpp_namespace: 'something'
+        """), idl.errors.ERROR_ID_BAD_CPP_NAMESPACE)
+
     def test_type_positive(self):
         # type: () -> None
         """Positive type tests."""
@@ -151,6 +174,7 @@ class TestBinder(testcase.IDLTestcase):
                 serializer: foo
                 deserializer: foo
                 default: foo
+                is_view: false
             """))
 
         # Test supported types
@@ -167,6 +191,7 @@ class TestBinder(testcase.IDLTestcase):
                 bson_serialization_type: %s
                 default: foo
                 deserializer: BSONElement::fake
+                is_view: false
             """ % (bson_type)))
 
         # Test supported numeric types
@@ -186,6 +211,7 @@ class TestBinder(testcase.IDLTestcase):
                         cpp_type: %s
                         bson_serialization_type: int
                         deserializer: BSONElement::fake
+                        is_view: false
                 """ % (cpp_type)))
 
         # Test object
@@ -199,6 +225,7 @@ class TestBinder(testcase.IDLTestcase):
                 serializer: foo
                 deserializer: foo
                 default: foo
+                is_view: false
             """))
 
         # Test 'any'
@@ -212,6 +239,7 @@ class TestBinder(testcase.IDLTestcase):
                 serializer: foo
                 deserializer: foo
                 default: foo
+                is_view: false
             """))
 
         # Test 'chain'
@@ -225,6 +253,7 @@ class TestBinder(testcase.IDLTestcase):
                 serializer: foo
                 deserializer: foo
                 default: foo
+                is_view: false
             """))
 
         # Test supported bindata_subtype
@@ -238,6 +267,7 @@ class TestBinder(testcase.IDLTestcase):
                     bson_serialization_type: bindata
                     bindata_subtype: %s
                     deserializer: BSONElement::fake
+                    is_view: false
                 """ % (bindata_subtype)))
 
     def test_type_negative(self):
@@ -252,6 +282,7 @@ class TestBinder(testcase.IDLTestcase):
                     description: foo
                     cpp_type: foo
                     bson_serialization_type: foo
+                    is_view: false
             """), idl.errors.ERROR_ID_BAD_BSON_TYPE)
 
         # Test bad cpp_type name
@@ -263,6 +294,7 @@ class TestBinder(testcase.IDLTestcase):
                     cpp_type: StringData
                     bson_serialization_type: string
                     deserializer: bar
+                    is_view: false
             """), idl.errors.ERROR_ID_NO_STRINGDATA)
 
         # Test unsupported serialization
@@ -313,6 +345,7 @@ class TestBinder(testcase.IDLTestcase):
                         cpp_type: %s
                         bson_serialization_type: int
                         deserializer: BSONElement::int
+                        is_view: false
                     """ % (cpp_type)), idl.errors.ERROR_ID_BAD_NUMERIC_CPP_TYPE)
 
         # Test the std prefix 8 and 16-byte integers fail
@@ -325,6 +358,7 @@ class TestBinder(testcase.IDLTestcase):
                         cpp_type: %s
                         bson_serialization_type: int
                         deserializer: BSONElement::int
+                        is_view: false
                     """ % (std_cpp_type)), idl.errors.ERROR_ID_BAD_NUMERIC_CPP_TYPE)
 
         # Test bindata_subtype missing
@@ -336,6 +370,7 @@ class TestBinder(testcase.IDLTestcase):
                     cpp_type: foo
                     bson_serialization_type: bindata
                     deserializer: BSONElement::fake
+                    is_view: false
             """), idl.errors.ERROR_ID_BAD_BSON_BINDATA_SUBTYPE_VALUE)
 
         # Test fake bindata_subtype is wrong
@@ -348,6 +383,7 @@ class TestBinder(testcase.IDLTestcase):
                     bson_serialization_type: bindata
                     bindata_subtype: foo
                     deserializer: BSONElement::fake
+                    is_view: false
             """), idl.errors.ERROR_ID_BAD_BSON_BINDATA_SUBTYPE_VALUE)
 
         # Test deprecated bindata_subtype 'binary', and 'uuid_old' are wrong
@@ -359,6 +395,7 @@ class TestBinder(testcase.IDLTestcase):
                     cpp_type: foo
                     bson_serialization_type: bindata
                     bindata_subtype: binary
+                    is_view: false
             """), idl.errors.ERROR_ID_BAD_BSON_BINDATA_SUBTYPE_VALUE)
 
         self.assert_bind_fail(
@@ -369,6 +406,7 @@ class TestBinder(testcase.IDLTestcase):
                     cpp_type: foo
                     bson_serialization_type: bindata
                     bindata_subtype: uuid_old
+                    is_view: false
             """), idl.errors.ERROR_ID_BAD_BSON_BINDATA_SUBTYPE_VALUE)
 
         # Test bindata_subtype on wrong type
@@ -381,6 +419,7 @@ class TestBinder(testcase.IDLTestcase):
                     bson_serialization_type: string
                     bindata_subtype: generic
                     deserializer: BSONElement::fake
+                    is_view: false
             """), idl.errors.ERROR_ID_BAD_BSON_BINDATA_SUBTYPE_TYPE)
 
         # Test bindata with default
@@ -393,6 +432,7 @@ class TestBinder(testcase.IDLTestcase):
                     bson_serialization_type: bindata
                     bindata_subtype: uuid
                     default: 42
+                    is_view: false
             """), idl.errors.ERROR_ID_BAD_BINDATA_DEFAULT)
 
         # Test bindata in list of types
@@ -405,6 +445,7 @@ class TestBinder(testcase.IDLTestcase):
                     bson_serialization_type:
                                 - bindata
                                 - string
+                    is_view: false
             """), idl.errors.ERROR_ID_BAD_BSON_TYPE)
 
         # Test bindata in list of types
@@ -417,6 +458,7 @@ class TestBinder(testcase.IDLTestcase):
                     bson_serialization_type:
                                 - bindata
                                 - string
+                    is_view: false
             """), idl.errors.ERROR_ID_BAD_BSON_TYPE)
 
         # Test 'any' in list of types
@@ -429,6 +471,7 @@ class TestBinder(testcase.IDLTestcase):
                     bson_serialization_type:
                                 - any
                                 - int
+                    is_view: false
             """), idl.errors.ERROR_ID_BAD_ANY_TYPE_USE)
 
         # Test object in list of types
@@ -441,6 +484,7 @@ class TestBinder(testcase.IDLTestcase):
                     bson_serialization_type:
                                 - object
                                 - int
+                    is_view: false
             """), idl.errors.ERROR_ID_BAD_BSON_TYPE_LIST)
 
         # Test fake in list of types
@@ -453,6 +497,7 @@ class TestBinder(testcase.IDLTestcase):
                     bson_serialization_type:
                                 - int
                                 - fake
+                    is_view: false
             """), idl.errors.ERROR_ID_BAD_BSON_TYPE)
 
         # Test 'chain' in list of types
@@ -465,6 +510,7 @@ class TestBinder(testcase.IDLTestcase):
                     bson_serialization_type:
                                 - chain
                                 - int
+                    is_view: false
             """), idl.errors.ERROR_ID_BAD_ANY_TYPE_USE)
 
         # Test unsupported serialization
@@ -481,6 +527,7 @@ class TestBinder(testcase.IDLTestcase):
                         bson_serialization_type: %s
                         serializer: foo
                         deserializer: BSONElement::fake
+                        is_view: false
                     """ % (bson_type)),
                 idl.errors.ERROR_ID_CUSTOM_SCALAR_SERIALIZATION_NOT_SUPPORTED)
 
@@ -492,6 +539,7 @@ class TestBinder(testcase.IDLTestcase):
                         cpp_type: std::string
                         bson_serialization_type: %s
                         deserializer: foo
+                        is_view: false
                     """ % (bson_type)),
                 idl.errors.ERROR_ID_CUSTOM_SCALAR_SERIALIZATION_NOT_SUPPORTED)
 
@@ -503,6 +551,7 @@ class TestBinder(testcase.IDLTestcase):
                     description: foo
                     cpp_type: foo
                     bson_serialization_type: any
+                    is_view: false
             """), idl.errors.ERROR_ID_MISSING_AST_REQUIRED_FIELD)
 
         # Test 'chain' serialization needs deserializer
@@ -514,6 +563,7 @@ class TestBinder(testcase.IDLTestcase):
                     cpp_type: foo
                     bson_serialization_type: chain
                     serializer: bar
+                    is_view: false
             """), idl.errors.ERROR_ID_MISSING_AST_REQUIRED_FIELD)
 
         # Test 'string' serialization needs deserializer
@@ -525,6 +575,7 @@ class TestBinder(testcase.IDLTestcase):
                     cpp_type: foo
                     bson_serialization_type: string
                     serializer: bar
+                    is_view: false
             """), idl.errors.ERROR_ID_MISSING_AST_REQUIRED_FIELD)
 
         # Test 'date' serialization needs deserializer
@@ -535,6 +586,7 @@ class TestBinder(testcase.IDLTestcase):
                     description: foo
                     cpp_type: foo
                     bson_serialization_type: date
+                    is_view: false
             """), idl.errors.ERROR_ID_MISSING_AST_REQUIRED_FIELD)
 
         # Test 'chain' serialization needs serializer
@@ -546,6 +598,7 @@ class TestBinder(testcase.IDLTestcase):
                     cpp_type: foo
                     bson_serialization_type: chain
                     deserializer: bar
+                    is_view: false
             """), idl.errors.ERROR_ID_MISSING_AST_REQUIRED_FIELD)
 
         # Test list of bson types needs deserializer
@@ -558,6 +611,7 @@ class TestBinder(testcase.IDLTestcase):
                     bson_serialization_type:
                                 - int
                                 - string
+                    is_view: false
             """), idl.errors.ERROR_ID_MISSING_AST_REQUIRED_FIELD)
 
         # Test array as name
@@ -569,6 +623,7 @@ class TestBinder(testcase.IDLTestcase):
                     cpp_type: foo
                     bson_serialization_type: string
                     deserializer: bar
+                    is_view: false
             """), idl.errors.ERROR_ID_ARRAY_NOT_VALID_TYPE)
 
     def test_struct_positive(self):
@@ -584,6 +639,7 @@ class TestBinder(testcase.IDLTestcase):
                 cpp_type: std::int32_t
                 bson_serialization_type: int
                 deserializer: mongo::BSONElement::_numberInt
+                is_view: false
         """))
 
         self.assert_bind(test_preamble + textwrap.dedent("""
@@ -617,6 +673,7 @@ class TestBinder(testcase.IDLTestcase):
                 cpp_type: std::int32_t
                 bson_serialization_type: int
                 deserializer: mongo::BSONElement::_numberInt
+                is_view: false
         """))
 
         # Test array as name
@@ -652,12 +709,14 @@ class TestBinder(testcase.IDLTestcase):
                 cpp_type: std::int32_t
                 bson_serialization_type: int
                 deserializer: mongo::BSONElement::_numberInt
+                is_view: false
             bindata_function:
                 bson_serialization_type: bindata
                 bindata_subtype: function
                 description: "A BSON bindata of function sub type"
                 cpp_type: "std::vector<std::uint8_t>"
                 deserializer: "mongo::BSONElement::_binDataVector"
+                is_view: false
         """))
 
         self.assert_bind(test_preamble + textwrap.dedent("""
@@ -730,6 +789,7 @@ class TestBinder(testcase.IDLTestcase):
                 cpp_type: std::int32_t
                 bson_serialization_type: int
                 deserializer: mongo::BSONElement::_numberInt
+                is_view: false
             safeInt:
                 bson_serialization_type:
                 - long
@@ -739,6 +799,7 @@ class TestBinder(testcase.IDLTestcase):
                 description: foo
                 cpp_type: "std::int32_t"
                 deserializer: "mongo::BSONElement::safeNumberInt"
+                is_view: false
         """)) + textwrap.dedent("""
         enums:
             foo_enum:
@@ -773,7 +834,7 @@ class TestBinder(testcase.IDLTestcase):
                             - string
                             - int
                             - not_defined
-            """), idl.errors.ERROR_ID_UNKNOWN_TYPE)
+            """), idl.errors.ERROR_ID_UNKNOWN_TYPE, True)
 
         # Enums are banned in variants for now.
         self.assert_bind_fail(
@@ -787,7 +848,7 @@ class TestBinder(testcase.IDLTestcase):
                             variant:
                             - string
                             - foo_enum
-            """), idl.errors.ERROR_ID_NO_VARIANT_ENUM)
+            """), idl.errors.ERROR_ID_NO_VARIANT_ENUM, True)
 
         self.assert_bind_fail(
             test_preamble + textwrap.dedent("""
@@ -861,7 +922,7 @@ class TestBinder(testcase.IDLTestcase):
                             - one_string
                             - one_string
                             - int
-            """), idl.errors.ERROR_ID_VARIANT_STRUCTS)
+            """), idl.errors.ERROR_ID_VARIANT_STRUCTS, True)
 
         # For multiple BSON serialization type Objects they must have different field names
         # for their first field.
@@ -883,7 +944,7 @@ class TestBinder(testcase.IDLTestcase):
                             - one_string
                             - one_int
                             - int
-            """), idl.errors.ERROR_ID_VARIANT_STRUCTS)
+            """), idl.errors.ERROR_ID_VARIANT_STRUCTS, True)
 
         # At most one type can have BSON serialization type NumberInt.
         self.assert_bind_fail(
@@ -959,6 +1020,7 @@ class TestBinder(testcase.IDLTestcase):
                     bson_serialization_type: string
                     serializer: foo
                     deserializer: foo
+                    is_view: false
             """)) + textwrap.dedent("""
             structs:
                 foo:
@@ -1012,18 +1074,21 @@ class TestBinder(testcase.IDLTestcase):
                 serializer: foo
                 deserializer: foo
                 default: foo
+                is_view: false
 
             bindata:
                 description: foo
                 cpp_type: foo
                 bson_serialization_type: bindata
                 bindata_subtype: uuid
+                is_view: false
 
             serialization_context:
                 bson_serialization_type: any
                 description: foo
                 cpp_type: foo
                 internal_only: true
+                is_view: false
         """)
 
         # Test field of a struct type with a non-true default
@@ -1064,7 +1129,7 @@ class TestBinder(testcase.IDLTestcase):
                     strict: true
                     fields:
                         foo: array<array<string>>
-            """), idl.errors.ERROR_ID_BAD_ARRAY_TYPE_NAME)
+            """), idl.errors.ERROR_ID_BAD_ARRAY_TYPE_NAME, True)
 
         # Test inherited default with array
         self.assert_bind_fail(
@@ -1195,6 +1260,7 @@ class TestBinder(testcase.IDLTestcase):
                 serializer: foo
                 deserializer: foo
                 default: foo
+                is_view: false
         """))
 
         # Chaining only
@@ -1220,6 +1286,7 @@ class TestBinder(testcase.IDLTestcase):
                 bson_serialization_type: chain
                 serializer: foo
                 deserializer: foo
+                is_view: false
         """))
 
         # Chaining with strict struct
@@ -1268,7 +1335,7 @@ class TestBinder(testcase.IDLTestcase):
                     foobar1: alias
                 fields:
                     foo1: string
-        """), idl.errors.ERROR_ID_UNKNOWN_TYPE)
+        """), idl.errors.ERROR_ID_UNKNOWN_TYPE, True)
 
         # A regular field as a chained type
         self.assert_bind_fail(
@@ -1280,7 +1347,7 @@ class TestBinder(testcase.IDLTestcase):
                 fields:
                     foo1: string
                     foo2: foobar1
-        """), idl.errors.ERROR_ID_UNKNOWN_TYPE)
+        """), idl.errors.ERROR_ID_UNKNOWN_TYPE, True)
 
         # Array of chained types
         self.assert_bind_fail(
@@ -1307,6 +1374,7 @@ class TestBinder(testcase.IDLTestcase):
                 serializer: foo
                 deserializer: foo
                 default: foo
+                is_view: false
         """)) + textwrap.dedent("""
         structs:
             chained:
@@ -1428,6 +1496,7 @@ class TestBinder(testcase.IDLTestcase):
                 serializer: foo
                 deserializer: foo
                 default: foo
+                is_view: false
         """)) + textwrap.dedent("""
         structs:
             chained:
@@ -1453,7 +1522,7 @@ class TestBinder(testcase.IDLTestcase):
                 strict: true
                 chained_structs:
                     foobar1: alias
-        """)), idl.errors.ERROR_ID_UNKNOWN_TYPE)
+        """)), idl.errors.ERROR_ID_UNKNOWN_TYPE, True)
 
         # Type as chained struct
         self.assert_bind_fail(
@@ -1587,6 +1656,18 @@ class TestBinder(testcase.IDLTestcase):
                     v3: 2
             """))
 
+        # Test int - non continuous
+        self.assert_bind(
+            textwrap.dedent("""
+        enums:
+            foo:
+                description: foo
+                type: int
+                values:
+                    v1: 0
+                    v3: 2
+            """))
+
         # Test string
         self.assert_bind(
             textwrap.dedent("""
@@ -1615,18 +1696,6 @@ class TestBinder(testcase.IDLTestcase):
                     v1: 0
             """), idl.errors.ERROR_ID_ENUM_BAD_TYPE)
 
-        # Test int - non continuous
-        self.assert_bind_fail(
-            textwrap.dedent("""
-        enums:
-            foo:
-                description: foo
-                type: int
-                values:
-                    v1: 0
-                    v3: 2
-            """), idl.errors.ERROR_ID_ENUM_NON_CONTINUOUS_RANGE)
-
         # Test int - dups
         self.assert_bind_fail(
             textwrap.dedent("""
@@ -1685,7 +1754,7 @@ class TestBinder(testcase.IDLTestcase):
                 description: foo
                 fields:
                     foo1: array<foo>
-            """), idl.errors.ERROR_ID_NO_ARRAY_ENUM)
+            """), idl.errors.ERROR_ID_NO_ARRAY_ENUM, True)
 
     def test_command_positive(self):
         # type: () -> None
@@ -1961,7 +2030,7 @@ class TestBinder(testcase.IDLTestcase):
                 type: int
                 fields:
                     field1: string
-            """), idl.errors.ERROR_ID_UNKNOWN_TYPE)
+            """), idl.errors.ERROR_ID_UNKNOWN_TYPE, True)
 
     def test_server_parameter_positive(self):
         # type: () -> None
@@ -1970,14 +2039,27 @@ class TestBinder(testcase.IDLTestcase):
         # server parameter with storage.
         # Also try valid set_at values.
         for set_at in ["startup", "runtime", "[ startup, runtime ]", "cluster"]:
-            self.assert_bind(
-                textwrap.dedent("""
-            server_parameters:
-                foo:
-                    set_at: %s
-                    description: bar
-                    cpp_varname: baz
-                """ % (set_at)))
+            if set_at != 'cluster':
+                self.assert_bind(
+                    textwrap.dedent("""
+                server_parameters:
+                    foo:
+                        set_at: %s
+                        description: bar
+                        redact: false
+                        cpp_varname: baz
+                    """ % (set_at)))
+            else:
+                self.assert_bind(
+                    textwrap.dedent("""
+                server_parameters:
+                    foo:
+                        set_at: %s
+                        description: bar
+                        redact: false
+                        cpp_varname: baz
+                        omit_in_ftdc: false
+                    """ % (set_at)))
 
         # server parameter with storage and optional fields.
         self.assert_bind(
@@ -1986,6 +2068,7 @@ class TestBinder(testcase.IDLTestcase):
             foo:
                 set_at: startup
                 description: bar
+                redact: false
                 cpp_varname: baz
                 default: 42
                 on_update: buzz
@@ -2004,9 +2087,11 @@ class TestBinder(testcase.IDLTestcase):
             foo:
                 set_at: cluster
                 description: bar
+                redact: false
                 cpp_varname: baz
                 cpp_vartype: bazStorage
                 on_update: buzz
+                omit_in_ftdc: false
                 validator:
                     gt: 0
                     gte: 1
@@ -2022,6 +2107,7 @@ class TestBinder(testcase.IDLTestcase):
             foo:
                 set_at: startup
                 description: bar
+                redact: false
                 cpp_varname: baz
                 default:
                     expr: 'kDefaultValue'
@@ -2043,6 +2129,7 @@ class TestBinder(testcase.IDLTestcase):
             foo:
                 set_at: startup
                 description: bar
+                redact: false
                 cpp_class: baz
         """))
 
@@ -2052,6 +2139,7 @@ class TestBinder(testcase.IDLTestcase):
             foo:
                 set_at: startup
                 description: bar
+                redact: false
                 cpp_class:
                     name: baz
         """))
@@ -2062,6 +2150,7 @@ class TestBinder(testcase.IDLTestcase):
             foo:
                 set_at: startup
                 description: bar
+                redact: false
                 cpp_class:
                     name: baz
                     data: bling
@@ -2094,6 +2183,7 @@ class TestBinder(testcase.IDLTestcase):
                     override_validate: true
                 condition: { expr: "true" }
                 redact: true
+                omit_in_ftdc: true
                 test_only: true
                 deprecated_name: bling
         """))
@@ -2105,6 +2195,7 @@ class TestBinder(testcase.IDLTestcase):
             foo:
                 set_at: startup
                 description: bar
+                redact: false
                 cpp_class: baz
                 default: blong
             """))
@@ -2120,6 +2211,7 @@ class TestBinder(testcase.IDLTestcase):
                 foo:
                     set_at: shutdown
                     description: bar
+                    redact: false
                     cpp_varname: baz
             """), idl.errors.ERROR_ID_BAD_SETAT_SPECIFIER)
 
@@ -2130,10 +2222,78 @@ class TestBinder(testcase.IDLTestcase):
                 foo:
                     set_at: startup
                     description: bar
+                    redact: false
                     cpp_class: baz
                     cpp_varname: bling
             """), idl.errors.ERROR_ID_SERVER_PARAMETER_INVALID_ATTR)
 
+        # Startup with omit_in_ftdc=true.
+        self.assert_bind_fail(
+            textwrap.dedent("""
+            server_parameters:
+                foo:
+                    set_at: startup
+                    description: bar
+                    cpp_varname: baz
+                    redact: false
+                    omit_in_ftdc: true
+            """), idl.errors.ERROR_ID_SERVER_PARAMETER_INVALID_ATTR)
+
+        # Startup with omit_in_ftdc=false.
+        self.assert_bind_fail(
+            textwrap.dedent("""
+            server_parameters:
+                foo:
+                    set_at: startup
+                    description: bar
+                    cpp_varname: baz
+                    redact: false
+                    omit_in_ftdc: false
+            """), idl.errors.ERROR_ID_SERVER_PARAMETER_INVALID_ATTR)
+
+        # Runtime with omit_in_ftdc=true.
+        self.assert_bind_fail(
+            textwrap.dedent("""
+            server_parameters:
+                foo:
+                    set_at: runtime
+                    description: bar
+                    cpp_varname: baz
+                    redact: false
+                    omit_in_ftdc: true
+            """), idl.errors.ERROR_ID_SERVER_PARAMETER_INVALID_ATTR)
+
+        # Runtime with omit_in_ftdc=false.
+        self.assert_bind_fail(
+            textwrap.dedent("""
+            server_parameters:
+                foo:
+                    set_at: runtime
+                    description: bar
+                    cpp_varname: baz
+                    redact: false
+                    omit_in_ftdc: false
+            """), idl.errors.ERROR_ID_SERVER_PARAMETER_INVALID_ATTR)
+
+        # Cluster with omit_in_ftdc unspecified.
+        self.assert_bind_fail(
+            textwrap.dedent("""
+            server_parameters:
+                foo:
+                    set_at: cluster
+                    description: bar
+                    redact: false
+                    cpp_varname: baz
+                    cpp_vartype: bazStorage
+                    on_update: buzz
+                    validator:
+                        gt: 0
+                        gte: 1
+                        lte: 999
+                        lt: 1000
+                        callback: qux
+                """), idl.errors.ERROR_ID_SERVER_PARAMETER_REQUIRED_ATTR)
+
     def test_config_option_positive(self):
         # type: () -> None
         """Positive config option test cases."""
@@ -2335,7 +2495,7 @@ class TestBinder(testcase.IDLTestcase):
         # type: () -> None
         """Test feature flag checks around version."""
 
-        # feature flag can default to false without a version
+        # feature flag can default to false without a version (shouldBeFCVGated can be true or false)
         self.assert_bind(
             textwrap.dedent("""
             feature_flags:
@@ -2343,9 +2503,20 @@ class TestBinder(testcase.IDLTestcase):
                     description: "Make toast"
                     cpp_varname: gToaster
                     default: false
+                    shouldBeFCVGated: false
             """))
 
-        # feature flag can default to true with a version
+        self.assert_bind(
+            textwrap.dedent("""
+            feature_flags:
+                featureFlagToaster:
+                    description: "Make toast"
+                    cpp_varname: gToaster
+                    default: false
+                    shouldBeFCVGated: true
+            """))
+
+        # if shouldBeFCVGated: true, feature flag can default to true with a version
         self.assert_bind(
             textwrap.dedent("""
             feature_flags:
@@ -2354,9 +2525,21 @@ class TestBinder(testcase.IDLTestcase):
                     cpp_varname: gToaster
                     default: true
                     version: 123
+                    shouldBeFCVGated: true
             """))
 
-        # true is only allowed with a version
+        # if shouldBeFCVGated: false, we do not need a version
+        self.assert_bind(
+            textwrap.dedent("""
+            feature_flags:
+                featureFlagToaster:
+                    description: "Make toast"
+                    cpp_varname: gToaster
+                    default: true
+                    shouldBeFCVGated: false
+            """))
+
+        # if shouldBeFCVGated: true and default: true, a version is required
         self.assert_bind_fail(
             textwrap.dedent("""
             feature_flags:
@@ -2364,9 +2547,10 @@ class TestBinder(testcase.IDLTestcase):
                     description: "Make toast"
                     cpp_varname: gToaster
                     default: true
+                    shouldBeFCVGated: true
             """), idl.errors.ERROR_ID_FEATURE_FLAG_DEFAULT_TRUE_MISSING_VERSION)
 
-        # false is not allowed with a version
+        # false is not allowed with a version and shouldBeFCVGated: true
         self.assert_bind_fail(
             textwrap.dedent("""
             feature_flags:
@@ -2375,8 +2559,33 @@ class TestBinder(testcase.IDLTestcase):
                     cpp_varname: gToaster
                     default: false
                     version: 123
+                    shouldBeFCVGated: true
             """), idl.errors.ERROR_ID_FEATURE_FLAG_DEFAULT_FALSE_HAS_VERSION)
 
+        # false is not allowed with a version and shouldBeFCVGated: false
+        self.assert_bind_fail(
+            textwrap.dedent("""
+            feature_flags:
+                featureFlagToaster:
+                    description: "Make toast"
+                    cpp_varname: gToaster
+                    default: false
+                    version: 123
+                    shouldBeFCVGated: false
+            """), idl.errors.ERROR_ID_FEATURE_FLAG_DEFAULT_FALSE_HAS_VERSION)
+
+        # if shouldBeFCVGated is false, a version is not allowed
+        self.assert_bind_fail(
+            textwrap.dedent("""
+            feature_flags:
+                featureFlagToaster:
+                    description: "Make toast"
+                    cpp_varname: gToaster
+                    default: true
+                    version: 123
+                    shouldBeFCVGated: false
+            """), idl.errors.ERROR_ID_FEATURE_FLAG_SHOULD_BE_FCV_GATED_FALSE_HAS_VERSION)
+
     def test_access_check(self):
         # type: () -> None
         """Test access check."""
@@ -2659,64 +2868,6 @@ class TestBinder(testcase.IDLTestcase):
                 reply_type: reply
             """), idl.errors.ERROR_ID_MISSING_ACCESS_CHECK)
 
-    # pylint: disable=invalid-name
-    def test_struct_unsafe_dangerous_disable_extra_field_duplicate_checks_negative(self):
-        # type: () -> None
-        """Negative struct tests for unsafe_dangerous_disable_extra_field_duplicate_checks."""
-
-        # Setup some common types
-        test_preamble = self.common_types + \
-            textwrap.dedent("""
-            structs:
-                danger:
-                    description: foo
-                    strict: false
-                    unsafe_dangerous_disable_extra_field_duplicate_checks: true
-                    fields:
-                        foo: string
-        """)
-
-        # Test strict and unsafe_dangerous_disable_extra_field_duplicate_checks are not allowed
-        self.assert_bind_fail(
-            test_preamble + indent_text(
-                1,
-                textwrap.dedent("""
-                danger1:
-                    description: foo
-                    strict: true
-                    unsafe_dangerous_disable_extra_field_duplicate_checks: true
-                    fields:
-                        foo: string
-            """)), idl.errors.ERROR_ID_STRICT_AND_DISABLE_CHECK_NOT_ALLOWED)
-
-        # Test inheritance is prohibited through structs
-        self.assert_bind_fail(
-            test_preamble + indent_text(
-                1,
-                textwrap.dedent("""
-                danger2:
-                    description: foo
-                    strict: true
-                    fields:
-                        foo: string
-                        d1: danger
-            """)), idl.errors.ERROR_ID_INHERITANCE_AND_DISABLE_CHECK_NOT_ALLOWED)
-
-        # Test inheritance is prohibited through commands
-        self.assert_bind_fail(
-            test_preamble + textwrap.dedent("""
-            commands:
-                dangerc:
-                    description: foo
-                    namespace: ignored
-                    command_name: dangerc
-                    strict: false
-                    api_version: ""
-                    fields:
-                        foo: string
-                        d1: danger
-            """), idl.errors.ERROR_ID_INHERITANCE_AND_DISABLE_CHECK_NOT_ALLOWED)
-
     def test_query_shape_component_validation(self):
         self.assert_bind(self.common_types + textwrap.dedent("""
             structs:
@@ -2770,16 +2921,19 @@ class TestBinder(testcase.IDLTestcase):
                     description: "A BSON UTF-8 string"
                     cpp_type: "std::string"
                     deserializer: "mongo::BSONElement::str"
+                    is_view: false
                 bool:
                     bson_serialization_type: bool
                     description: "A BSON bool"
                     cpp_type: "bool"
                     deserializer: "mongo::BSONElement::boolean"
+                    is_view: false
                 serialization_context:
                     bson_serialization_type: any
                     description: foo
                     cpp_type: foo
                     internal_only: true
+                    is_view: false
         """)
         self.assert_bind(basic_types + textwrap.dedent("""
             structs:
@@ -2887,6 +3041,64 @@ class TestBinder(testcase.IDLTestcase):
                             query_shape: literal
             """), idl.errors.ERROR_ID_CANNOT_DECLARE_SHAPE_LITERAL)
 
+    # pylint: disable=invalid-name
+    def test_struct_unsafe_dangerous_disable_extra_field_duplicate_checks_negative(self):
+        # type: () -> None
+        """Negative struct tests for unsafe_dangerous_disable_extra_field_duplicate_checks."""
+
+        # Setup some common types
+        test_preamble = self.common_types + \
+            textwrap.dedent("""
+            structs:
+                danger:
+                    description: foo
+                    strict: false
+                    unsafe_dangerous_disable_extra_field_duplicate_checks: true
+                    fields:
+                        foo: string
+        """)
+
+        # Test strict and unsafe_dangerous_disable_extra_field_duplicate_checks are not allowed
+        self.assert_bind_fail(
+            test_preamble + indent_text(
+                1,
+                textwrap.dedent("""
+                danger1:
+                    description: foo
+                    strict: true
+                    unsafe_dangerous_disable_extra_field_duplicate_checks: true
+                    fields:
+                        foo: string
+            """)), idl.errors.ERROR_ID_STRICT_AND_DISABLE_CHECK_NOT_ALLOWED)
+
+        # Test inheritance is prohibited through structs
+        self.assert_bind_fail(
+            test_preamble + indent_text(
+                1,
+                textwrap.dedent("""
+                danger2:
+                    description: foo
+                    strict: true
+                    fields:
+                        foo: string
+                        d1: danger
+            """)), idl.errors.ERROR_ID_INHERITANCE_AND_DISABLE_CHECK_NOT_ALLOWED)
+
+        # Test inheritance is prohibited through commands
+        self.assert_bind_fail(
+            test_preamble + textwrap.dedent("""
+            commands:
+                dangerc:
+                    description: foo
+                    namespace: ignored
+                    command_name: dangerc
+                    strict: false
+                    api_version: ""
+                    fields:
+                        foo: string
+                        d1: danger
+            """), idl.errors.ERROR_ID_INHERITANCE_AND_DISABLE_CHECK_NOT_ALLOWED)
+
 
 if __name__ == '__main__':
 
