@@ -57,6 +57,8 @@ class LinterSplitArgs(argparse.Action):
             selected_choices.remove(CountTypes.ALL.name)
         if selected_choices == []:
             selected_choices = copy.copy(self.default_choices)
+        if values == [""]:
+            selected_choices = []
         setattr(namespace, self.dest, [opt.replace('-', '_') for opt in selected_choices])
 
 
@@ -142,6 +144,10 @@ def setup_args_parser():
     parser.add_argument('--direct-depends', action='append', default=[],
                         help="Print the nodes which depends on a given node.")
 
+    parser.add_argument(
+        '--program-depends', action='append', default=[],
+        help="Print the programs which depend (transitively or directly) on a given node.")
+
     parser.add_argument('--common-depends', nargs='+', action='append', default=[],
                         help="Print the nodes which have a common dependency on all N nodes.")
 
@@ -156,13 +162,38 @@ def setup_args_parser():
     parser.add_argument(
         '--critical-edges', nargs='+', action='append', default=[], help=
         "[from_node] [to_node]: Print edges between two nodes, which if removed would break the dependency between those "
-        + "nodes,.")
+        + "nodes.")
+
+    parser.add_argument(
+        '--symbol-depends', nargs='+', action='append', default=[],
+        help="[from_node] [to_node]: Print symbols defined in from_node used by to_node.")
+
+    parser.add_argument(
+        '--efficiency', nargs='+', action='append', default=[], help=
+        "[from_node ...]: Print efficiencies of public direct edges off each from_node in a list of nodes."
+    )
+
+    parser.add_argument(
+        '--efficiency-lint', nargs='?', type=int, const=2, help=
+        "[threshold]: Analyze efficiency of all public direct edges, print those below efficiency threshold percentage."
+    )
+
+    parser.add_argument(
+        '--bazel-order', action='store_true', default=False, help=
+        "Find candidate nodes for merging by searching the graph for nodes with only one node which depends on them."
+    )
 
     parser.add_argument(
         '--indegree-one', action='store_true', default=False, help=
         "Find candidate nodes for merging by searching the graph for nodes with only one node which depends on them."
     )
 
+    parser.add_argument(
+        '--bazel-conv-candidates', action='store_true', default=False, help=
+        "Find candidate nodes ready for bazel conversion. This effectively means the node is currently not being built "
+        "with bazel and the node does not have any dependency nodes that are not being built in bazel."
+    )
+
     args = parser.parse_args()
 
     for arg_list in args.graph_paths:
@@ -175,13 +206,21 @@ def setup_args_parser():
             parser.error(
                 f'Must pass two args for --critical-edges, [from_node] [to_node], not {arg_list}')
 
+    for arg_list in args.symbol_depends:
+        if len(arg_list) != 2:
+            parser.error(
+                f'Must pass two args for --symbol-depends, [from_node] [to_node], not {arg_list}')
+
     return parser.parse_args()
 
 
 def strip_build_dir(build_dir, node):
     """Small util function for making args match the graph paths."""
 
-    return str(Path(node).relative_to(build_dir))
+    try:
+        return str(Path(node).relative_to(build_dir))
+    except ValueError:
+        return node
 
 
 def strip_build_dirs(build_dir, nodes):
@@ -220,6 +259,11 @@ def main():
             libdeps_analyzer.DirectDependents(libdeps_graph,
                                               strip_build_dir(build_dir, analyzer_args)))
 
+    for analyzer_args in args.program_depends:
+        analysis.append(
+            libdeps_analyzer.TransitiveProgramDependents(libdeps_graph,
+                                                         strip_build_dir(build_dir, analyzer_args)))
+
     for analyzer_args in args.common_depends:
         analysis.append(
             libdeps_analyzer.CommonDependents(libdeps_graph,
@@ -235,6 +279,24 @@ def main():
             libdeps_analyzer.GraphPaths(libdeps_graph, strip_build_dir(build_dir, analyzer_args[0]),
                                         strip_build_dir(build_dir, analyzer_args[1])))
 
+    for analyzer_args in args.symbol_depends:
+        analysis.append(
+            libdeps_analyzer.SymbolDependents(libdeps_graph,
+                                              strip_build_dir(build_dir, analyzer_args[0]),
+                                              strip_build_dir(build_dir, analyzer_args[1])))
+
+    for analyzer_args in args.efficiency:
+        nodes = []
+        for arg in analyzer_args:
+            nodes.append(strip_build_dir(build_dir, arg))
+        analysis.append(libdeps_analyzer.Efficiency(libdeps_graph, nodes))
+
+    if args.efficiency_lint:
+        analysis.append(libdeps_analyzer.EfficiencyLinter(libdeps_graph, args.efficiency_lint))
+
+    if args.bazel_order:
+        analysis.append(libdeps_analyzer.BazelOrder(libdeps_graph))
+
     for analyzer_args in args.critical_edges:
         analysis.append(
             libdeps_analyzer.CriticalEdges(libdeps_graph,
@@ -244,6 +306,9 @@ def main():
     if args.indegree_one:
         analysis.append(libdeps_analyzer.InDegreeOne(libdeps_graph))
 
+    if args.bazel_conv_candidates:
+        analysis.append(libdeps_analyzer.BazelConversionCandidates(libdeps_graph))
+
     analysis += libdeps_analyzer.linter_factory(libdeps_graph, args.lint)
 
     if args.build_data:
