@@ -23,13 +23,17 @@ MONGO_REVISION_ENV_VAR = "REVISION"
 
 def _get_repos_and_revisions() -> Tuple[List[Repo], RevisionMap]:
     """Get the repo object and a map of revisions to compare against."""
-    modules = git.get_module_paths()
+    modules = [
+        path for path in git.get_module_paths()
+        # Exclude enterprise module; it's in the "modules" folder but does not correspond to a repo
+        if "src/mongo/db/modules/enterprise" not in path
+    ]
     repos = [Repo(path) for path in modules]
     revision_map = generate_revision_map(repos, {"mongo": os.environ.get(MONGO_REVISION_ENV_VAR)})
     return repos, revision_map
 
 
-def _filter_file(filename: str, is_interesting_file: Callable) -> bool:
+def _filter_file(filename: str, is_interesting_file: Callable[[str], bool]) -> bool:
     """
     Determine if file should be included based on existence and passed in method.
 
@@ -40,7 +44,7 @@ def _filter_file(filename: str, is_interesting_file: Callable) -> bool:
     return os.path.exists(filename) and is_interesting_file(filename)
 
 
-def gather_changed_files_for_lint(is_interesting_file: Callable) -> List[str]:
+def gather_changed_files_for_lint(is_interesting_file: Callable[[str], bool]) -> List[str]:
     """
     Get the files that have changes since the last git commit.
 
