@@ -28,6 +28,7 @@
 # echo "Now put the dist gnupg signing keys in ~root/.gnupg"
 
 import errno
+import git
 from glob import glob
 import os
 import re
@@ -119,8 +120,8 @@ class EnterpriseDistro(packager.Distro):
     def build_os(self, arch):
         """Return the build os label in the binary package to download.
 
-        The labels "rhel57", "rhel62", "rhel67", "rhel70", "rhel80", "rhel90" are for redhat,
-        the others are delegated to the super class.
+        The labels "rhel57", "rhel62", "rhel67", "rhel70", "rhel79", "rhel80", "rhel88", "rhel90"
+        are for redhat, the others are delegated to the super class.
         """
         if arch == "ppc64le":
             if self.dname == 'ubuntu':
@@ -138,10 +139,10 @@ class EnterpriseDistro(packager.Distro):
             return []
         if arch == "arm64":
             if self.dname == 'ubuntu':
-                return ["ubuntu1804", "ubuntu2004", "ubuntu2204"]
+                return ["ubuntu1804", "ubuntu2004", "ubuntu2204", "ubuntu2404"]
         if arch == "aarch64":
             if self.dname == 'redhat':
-                return ["rhel82", "rhel90"]
+                return ["rhel82", "rhel88", "rhel90", "rhel93"]
             if self.dname == 'amazon2':
                 return ["amazon2"]
             if self.dname == 'amazon2023':
@@ -149,7 +150,7 @@ class EnterpriseDistro(packager.Distro):
             return []
 
         if re.search("(redhat|fedora|centos)", self.dname):
-            return ["rhel90", "rhel80", "rhel70", "rhel62", "rhel57"]
+            return ["rhel93", "rhel90", "rhel88", "rhel80", "rhel70", "rhel79", "rhel62", "rhel57"]
         return super(EnterpriseDistro, self).build_os(arch)
 
 
@@ -228,7 +229,7 @@ def unpack_binaries_into(build_os, arch, spec, where):
     try:
         packager.sysassert(["tar", "xvzf", rootdir + "/" + tarfile(build_os, arch, spec)])
         release_dir = glob('mongodb-linux-*')[0]
-        for releasefile in "bin", "LICENSE-Enterprise.txt", "README", "THIRD-PARTY-NOTICES", "MPL-2", "sbom_from_silk.json":
+        for releasefile in "bin", "LICENSE-Enterprise.txt", "README", "THIRD-PARTY-NOTICES", "MPL-2":
             os.rename("%s/%s" % (release_dir, releasefile), releasefile)
         os.rmdir(release_dir)
     except Exception:
@@ -251,12 +252,33 @@ def make_package(distro, build_os, arch, spec, srcdir):
     # innocuous in the debianoids' sdirs).
     for pkgdir in ["debian", "rpm"]:
         print("Copying packaging files from %s to %s" % ("%s/%s" % (srcdir, pkgdir), sdir))
+        git_repo = git.Repo(srcdir)
+        # get the original HEAD position of repo
+        head_commit_sha = git_repo.head.object.hexsha
+
+        # add and commit the uncommited changes
+        print("Commiting uncommited changes")
+        git_repo.git.add(all=True)
+        # only commit changes if there are any
+        if len(git_repo.index.diff("HEAD")) != 0:
+            with git_repo.git.custom_environment(GIT_COMMITTER_NAME="Evergreen",
+                                                 GIT_COMMITTER_EMAIL="evergreen@mongodb.com"):
+                git_repo.git.commit("--author='Evergreen <>'", "-m", "temp commit")
+
+        # original command to preserve functionality
+        # FIXME: make consistent with the rest of the code when we have more packaging testing
         # FIXME: sh-dash-cee is bad. See if tarfile can do this.
+        print("Copying packaging files from specified gitspec:", spec.metadata_gitspec())
         packager.sysassert([
             "sh", "-c",
             "(cd \"%s\" && git archive %s %s/ ) | (cd \"%s\" && tar xvf -)" %
             (srcdir, spec.metadata_gitspec(), pkgdir, sdir)
         ])
+
+        # reset branch to original state
+        print("Resetting branch to original state")
+        git_repo.git.reset("--mixed", head_commit_sha)
+
     # Splat the binaries under sdir.  The "build" stages of the
     # packaging infrastructure will move the files to wherever they
     # need to go.
