@@ -29,6 +29,9 @@ DEFAULT_DBTEST_EXECUTABLE = os.path.join(os.curdir, "dbtest")
 DEFAULT_MONGO_EXECUTABLE = "mongo"
 DEFAULT_MONGOD_EXECUTABLE = "mongod"
 DEFAULT_MONGOS_EXECUTABLE = "mongos"
+# TODO SERVER-85977 potentially replace with "mongot" if possible to add a symlink from raw path
+# below as part of setup-mongot-repro.
+DEFAULT_MONGOT_EXECUTABLE = "mongot-localdev/mongot"
 
 DEFAULT_BENCHMARK_REPETITIONS = 3
 DEFAULT_BENCHMARK_MIN_TIME = datetime.timedelta(seconds=5)
@@ -57,6 +60,7 @@ DEFAULTS = {
     "base_port": 20000,
     "backup_on_restart_dir": None,
     "buildlogger_url": "https://logkeeper2.build.10gen.cc",
+    "embedded_router": None,
     "config_shard": None,
     "continue_on_failure": False,
     "dbpath_prefix": None,
@@ -67,6 +71,7 @@ DEFAULTS = {
     "flow_control_tickets": None,
     "force_excluded_tests": False,
     "fuzz_mongod_configs": None,
+    "fuzz_mongos_configs": None,
     "config_fuzz_seed": None,
     "genny_executable": None,
     "include_with_any_tags": None,
@@ -79,11 +84,14 @@ DEFAULTS = {
     "mongod_set_parameters": [],
     "mongos_executable": None,
     "mongos_set_parameters": [],
+    "mongot-localdev/mongot_executable": None,
+    "mongot_set_parameters": [],
     "mongocryptd_set_parameters": [],
     "mrlog": None,
     "no_journal": False,
     "num_clients_per_fixture": 1,
-    "perf_report_file": None,
+    "use_tenant_client": False,
+    "origin_suite": None,
     "cedar_report_file": None,
     "repeat_suites": 1,
     "repeat_tests": 1,
@@ -110,7 +118,6 @@ DEFAULTS = {
     "suite_files": "with_server",
     "tag_files": [],
     "test_files": [],
-    "transport_layer": None,
     "user_friendly_output": None,
     "mixed_bin_versions": None,
     "old_bin_version": None,
@@ -118,6 +125,13 @@ DEFAULTS = {
     "num_replset_nodes": None,
     "num_shards": None,
     "export_mongod_config": "off",
+    "tls_mode": None,
+    "tls_ca_file": None,
+    "shell_grpc": False,
+    "shell_tls_enabled": False,
+    "shell_tls_certificate_key_file": None,
+    "mongos_tls_certificate_key_file": None,
+    "mongod_tls_certificate_key_file": None,
 
     # Internal testing options.
     "internal_params": [],
@@ -137,6 +151,7 @@ DEFAULTS = {
     "variant_name": None,
     "version_id": None,
     "work_dir": None,
+    "evg_project_config_path": "etc/evergreen.yml",
 
     # WiredTiger options.
     "wt_coll_config": None,
@@ -152,6 +167,9 @@ DEFAULTS = {
     # Config Dir
     "config_dir": "buildscripts/resmokeconfig",
 
+    # Directory with jstests
+    "jstests_dir": "jstests",
+
     # UndoDB options
     "undo_recorder_path": None,
 
@@ -160,6 +178,26 @@ DEFAULTS = {
 
     # Limit the number of tests to execute
     "max_test_queue_size": None,
+
+    # Sanity check
+    "sanity_check": False,
+
+    # otel info
+    "otel_trace_id": None,
+    "otel_parent_id": None,
+    "otel_collector_dir": None,
+
+    # The images to build for an External System Under Test
+    "docker_compose_build_images": None,
+
+    # Where the `--dockerComposeBuildImages` is happening.
+    "docker_compose_build_env": "local",
+
+    # Tag to use for images built & used for an External System Under Test
+    "docker_compose_tag": "development",
+
+    # Whether or not this resmoke suite is running against an External System Under Test
+    "external_sut": False,
 }
 
 _SuiteOptions = collections.namedtuple("_SuiteOptions", [
@@ -305,6 +343,9 @@ DRY_RUN = None
 # If set, specifies which node is the config shard. Can also be set to 'any'.
 CONFIG_SHARD = None
 
+# If set, use mongod's embedded router functionality for all sharding tests instead of mongos.
+EMBEDDED_ROUTER = None
+
 # if set, enables enterprise jstest to automatically be included
 ENABLE_ENTERPRISE_TESTS = None
 
@@ -353,6 +394,9 @@ EVERGREEN_VERSION_ID = None
 # The Evergreen task's working directory.
 EVERGREEN_WORK_DIR = None
 
+# Path to evergreen project configuration yaml file
+EVERGREEN_PROJECT_CONFIG_PATH = None
+
 # If set, then any jstests that have any of the specified tags will be excluded from the suite(s).
 EXCLUDE_WITH_ANY_TAGS = None
 
@@ -365,7 +409,13 @@ EXCLUDED_TAG = "__TEMPORARILY_DISABLED__"
 # If true, then a test failure or error will cause resmoke.py to exit and not run any more tests.
 FAIL_FAST = None
 
+# Defines how to fuzz mongod parameters
 FUZZ_MONGOD_CONFIGS = None
+
+# Defines how to fuzz mongos parameters
+FUZZ_MONGOS_CONFIGS = None
+
+# This seeds the random number generator used to fuzz mongod and mongos parameters
 CONFIG_FUZZ_SEED = None
 
 # Executable file for genny, passed in as a command line arg.
@@ -418,6 +468,12 @@ MONGOS_EXECUTABLE = None
 # The --setParameter options passed to mongos.
 MONGOS_SET_PARAMETERS = []
 
+# The path to the mongot executable used by resmoke.py.
+MONGOT_EXECUTABLE = None
+
+# The --setParameter options passed to mongot.
+MONGOT_SET_PARAMETERS = []
+
 # The --setParameter options passed to mongocryptd.
 MONGOCRYPTD_SET_PARAMETERS = []
 
@@ -428,8 +484,11 @@ NO_JOURNAL = None
 # If set, then each fixture runs tests with the specified number of clients.
 NUM_CLIENTS_PER_FIXTURE = None
 
-# Report file for the Evergreen performance plugin.
-PERF_REPORT_FILE = None
+# If set, each client will be constructed with a generated tenant id.
+USE_TENANT_CLIENT = False
+
+# Indicates the name of the test suite prior to the suite being split up by uite generation
+ORIGIN_SUITE = None
 
 # Report file for Cedar.
 CEDAR_REPORT_FILE = None
@@ -484,6 +543,25 @@ MIXED_BIN_VERSIONS = None
 # Specifies the binary version of last-lts or last-continous when multiversion enabled
 MULTIVERSION_BIN_VERSION = None
 
+# Specifies whether to use gRPC when connecting via the shell by default.
+SHELL_GRPC = None
+
+# Specifies what tlsMode the server(s) should be started with.
+TLS_MODE = None
+
+# Specifies the CA file that all servers and shell instances should use.
+TLS_CA_FILE = None
+
+# Shell TLS options.
+SHELL_TLS_ENABLED = None
+SHELL_TLS_CERTIFICATE_KEY_FILE = None
+
+# Specifies the TLS certicicate that all mongods in the cluster should use.
+MONGOD_TLS_CERTIFICATE_KEY_FILE = None
+
+# Specifies the TLS certicicate that all mongoses in the cluster should use.
+MONGOS_TLS_CERTIFICATE_KEY_FILE = None
+
 # Specifies the number of replica set members in a ReplicaSetFixture.
 NUM_REPLSET_NODES = None
 
@@ -523,8 +601,10 @@ TAG_FILES = None
 # The test files to execute.
 TEST_FILES = None
 
-# If set, then mongod/mongos's started by resmoke.py will use the specified transport layer.
-TRANSPORT_LAYER = None
+# Metrics for open telemetry
+OTEL_TRACE_ID = None
+OTEL_PARENT_ID = None
+OTEL_COLLECTOR_DIR = None
 
 # If set, then all mongod's started by resmoke.py and by the mongo shell will use the specified
 # WiredTiger collection configuration settings.
@@ -594,6 +674,9 @@ EXTERNAL_SUITE_SELECTORS = (DEFAULT_BENCHMARK_TEST_LIST, DEFAULT_UNIT_TEST_LIST,
 CONFIG_DIR = None
 LOGGER_DIR = None
 
+# Where to look for jstests existence
+JSTESTS_DIR = None
+
 # Generated logging config for the current invocation.
 LOGGING_CONFIG: dict = {}
 SHORTEN_LOGGER_NAME_CONFIG: dict = {}
@@ -612,3 +695,27 @@ EXPANSIONS_FILE = "../expansions.yml" if 'CI' in os.environ else "expansions.yml
 # Symbolizer secrets
 SYMBOLIZER_CLIENT_SECRET = None
 SYMBOLIZER_CLIENT_ID = None
+
+# Sanity check
+SANITY_CHECK = False
+
+# The images to build for an External System Under Test
+DOCKER_COMPOSE_BUILD_IMAGES = None
+
+# Where the `--dockerComposeBuildImages` is happening.
+DOCKER_COMPOSE_BUILD_ENV = "local"
+
+# Tag to use for images built & used for an External System Under Test
+DOCKER_COMPOSE_TAG = "development"
+
+# Whether or not this resmoke suite is running against an External System Under Test
+EXTERNAL_SUT = False
+
+# This will be set to True when:
+# (1) We are building images for an External SUT
+# (2) Running against an External SUT
+NOOP_MONGO_D_S_PROCESSES = False
+
+# If resmoke is running from within a `workload` container,
+# we may need to do additional setup to run the suite successfully.
+REQUIRES_WORKLOAD_CONTAINER_SETUP = False
