@@ -98,7 +98,13 @@ class Process(object):
         if env_vars is not None:
             self.env.update(env_vars)
 
-        self.pid = None
+        # If we are running against an External System Under Test & this is a `mongo{d,s}` process, we make this process a NOOP.
+        # `mongo{d,s}` processes are not running locally for an External System Under Test.
+        self.NOOP = _config.NOOP_MONGO_D_S_PROCESSES and os.path.basename(
+            self.args[0]) in ["mongod", "mongos"]
+
+        # The `pid` attribute is assigned after the local process is started. If this process is a NOOP, we assign it a dummy value.
+        self.pid = 1 if self.NOOP else None
 
         self._process = None
         self._recorder = None
@@ -106,8 +112,13 @@ class Process(object):
         self._stderr_pipe = None
         self._cwd = cwd
 
+        if sys.platform == "win32":
+            self._windows_shutdown_event_set = False
+
     def start(self):
         """Start the process and the logger pipes for its stdout and stderr."""
+        if self.NOOP:
+            return None
 
         creation_flags = 0
         if sys.platform == "win32" and _JOB_OBJECT is not None:
@@ -175,56 +186,18 @@ class Process(object):
 
     def stop(self, mode=None):
         """Terminate the process."""
+        if self.NOOP:
+            return None
+
         if mode is None:
             mode = fixture_interface.TeardownMode.TERMINATE
 
         if sys.platform == "win32":
-
-            # Attempt to cleanly shutdown mongod.
             if mode != fixture_interface.TeardownMode.KILL and self.args and self.args[0].find(
                     "mongod") != -1:
-                mongo_signal_handle = None
-                try:
-                    mongo_signal_handle = win32event.OpenEvent(
-                        win32event.EVENT_MODIFY_STATE, False,
-                        "Global\\Mongo_" + str(self._process.pid))
-
-                    if not mongo_signal_handle:
-                        # The process has already died.
-                        return
-                    win32event.SetEvent(mongo_signal_handle)
-                    # Wait 60 seconds for the program to exit.
-                    status = win32event.WaitForSingleObject(self._process._handle, 60 * 1000)
-                    if status == win32event.WAIT_OBJECT_0:
-                        return
-                except win32process.error as err:
-                    # ERROR_FILE_NOT_FOUND (winerror=2)
-                    # ERROR_ACCESS_DENIED (winerror=5)
-                    # ERROR_INVALID_HANDLE (winerror=6)
-                    # One of the above errors is received if the process has
-                    # already died.
-                    if err.winerror not in (2, 5, 6):
-                        raise
-                finally:
-                    win32api.CloseHandle(mongo_signal_handle)
-
-                print("Failed to cleanly exit the program, calling TerminateProcess() on PID: " +\
-                    str(self._process.pid))
-
-            # Adapted from implementation of Popen.terminate() in subprocess.py of Python 2.7
-            # because earlier versions do not catch exceptions.
-            try:
-                # Have the process exit with code 0 if it is terminated by us to simplify the
-                # success-checking logic later on.
-                win32process.TerminateProcess(self._process._handle, 0)
-            except win32process.error as err:
-                # ERROR_ACCESS_DENIED (winerror=5) is received when the process
-                # has already died.
-                if err.winerror != winerror.ERROR_ACCESS_DENIED:
-                    raise
-                return_code = win32process.GetExitCodeProcess(self._process._handle)
-                if return_code == win32con.STILL_ACTIVE:
-                    raise
+                self._request_clean_shutdown_on_windows()
+            else:
+                self._terminate_on_windows()
         else:
             try:
                 if mode == fixture_interface.TeardownMode.KILL:
@@ -244,10 +217,34 @@ class Process(object):
 
     def poll(self):
         """Poll."""
+        if self.NOOP:
+            return None
         return self._process.poll()
 
     def wait(self, timeout=None):
         """Wait until process has terminated and all output has been consumed by the logger pipes."""
+        if self.NOOP:
+            return None
+
+        if sys.platform == "win32" and self._windows_shutdown_event_set:
+            status = None
+            try:
+                # Wait 60 seconds for the program to exit.
+                status = win32event.WaitForSingleObject(self._process._handle, 60 * 1000)
+            except win32process.error as err:
+                # ERROR_FILE_NOT_FOUND (winerror=2)
+                # ERROR_ACCESS_DENIED (winerror=5)
+                # ERROR_INVALID_HANDLE (winerror=6)
+                # One of the above errors is received if the process has
+                # already died.
+                if err.winerror not in (2, 5, 6):
+                    raise
+
+            if status is not None and status != win32event.WAIT_OBJECT_0:
+                self.logger.info(
+                    f"Failed to cleanly exit the program, calling TerminateProcess() on PID:"
+                    f" {str(self._process.pid)}")
+                self._terminate_on_windows()
 
         return_code = self._process.wait(timeout)
 
@@ -286,6 +283,8 @@ class Process(object):
 
     def pause(self):
         """Send the SIGSTOP signal to the process and wait for it to be stopped."""
+        if self.NOOP:
+            return None
         while True:
             self._process.send_signal(signal.SIGSTOP)
             mongod_process = psutil.Process(self.pid)
@@ -297,9 +296,55 @@ class Process(object):
 
     def resume(self):
         """Send the SIGCONT signal to the process."""
+        if self.NOOP:
+            return None
         self._process.send_signal(signal.SIGCONT)
 
     def __str__(self):
         if self.pid is None:
             return self.as_command()
         return "%s (%d)" % (self.as_command(), self.pid)
+
+    if sys.platform == "win32":
+
+        def _request_clean_shutdown_on_windows(self):
+            """Request clean shutdown on Windows."""
+            _windows_mongo_signal_handle = None
+            try:
+                _windows_mongo_signal_handle = win32event.OpenEvent(
+                    win32event.EVENT_MODIFY_STATE, False, "Global\\Mongo_" + str(self._process.pid))
+
+                if not _windows_mongo_signal_handle:
+                    # The process has already died.
+                    return
+                win32event.SetEvent(_windows_mongo_signal_handle)
+                self._windows_shutdown_event_set = True
+
+            except win32process.error as err:
+                # ERROR_FILE_NOT_FOUND (winerror=2)
+                # ERROR_ACCESS_DENIED (winerror=5)
+                # ERROR_INVALID_HANDLE (winerror=6)
+                # One of the above errors is received if the process has
+                # already died.
+                if err.winerror not in (2, 5, 6):
+                    raise
+
+            finally:
+                win32api.CloseHandle(_windows_mongo_signal_handle)
+
+        def _terminate_on_windows(self):
+            """Terminate the process on Windows."""
+            # Adapted from implementation of Popen.terminate() in subprocess.py of Python 2.7
+            # because earlier versions do not catch exceptions.
+            try:
+                # Have the process exit with code 0 if it is terminated by us to simplify the
+                # success-checking logic later on.
+                win32process.TerminateProcess(self._process._handle, 0)
+            except win32process.error as err:
+                # ERROR_ACCESS_DENIED (winerror=5) is received when the process
+                # has already died.
+                if err.winerror != winerror.ERROR_ACCESS_DENIED:
+                    raise
+                return_code = win32process.GetExitCodeProcess(self._process._handle)
+                if return_code == win32con.STILL_ACTIVE:
+                    raise
