@@ -85,6 +85,32 @@ def mongod_program(logger, job_num, executable, process_kwargs, mongod_options):
     args = [executable]
     mongod_options = mongod_options.copy()
 
+    if config.NOOP_MONGO_D_S_PROCESSES:
+        args[0] = os.path.basename(args[0])
+        mongod_options["set_parameters"]["fassertOnLockTimeoutForStepUpDown"] = 0
+        mongod_options["set_parameters"].pop("backtraceLogFile", None)
+        mongod_options.update({
+            "logpath": "/var/log/mongodb/mongodb.log",
+            "dbpath": "/data/db",
+            "bind_ip": "0.0.0.0",
+            "oplogSize": "256",
+            "wiredTigerCacheSizeGB": "1",
+        })
+
+    if config.TLS_MODE:
+        mongod_options["tlsMode"] = config.TLS_MODE
+        if config.TLS_MODE != "disabled":
+            # Note: "tlsAllowInvalidCertificates" is enabled to avoid
+            # hostname conflicts with our testing certificates.
+            # The ssl and ssl_special suites handle hostname validation testing.
+            mongod_options["tlsAllowInvalidHostnames"] = ""
+
+    if config.MONGOD_TLS_CERTIFICATE_KEY_FILE:
+        mongod_options["tlsCertificateKeyFile"] = config.MONGOD_TLS_CERTIFICATE_KEY_FILE
+
+    if config.TLS_CA_FILE:
+        mongod_options["tlsCAFile"] = config.TLS_CA_FILE
+
     if "port" not in mongod_options:
         mongod_options["port"] = network.PortAllocator.next_fixture_port(job_num)
 
@@ -93,11 +119,24 @@ def mongod_program(logger, job_num, executable, process_kwargs, mongod_options):
         suite_set_parameters, "queryAnalysisSamplerConfigurationRefreshSecs", bin_version, "7.0.0")
     remove_set_parameter_if_before_version(suite_set_parameters, "queryAnalysisWriterIntervalSecs",
                                            bin_version, "7.0.0")
+    remove_set_parameter_if_before_version(suite_set_parameters, "defaultConfigCommandTimeoutMS",
+                                           bin_version, "7.3.0")
+
+    if "grpcPort" not in mongod_options and suite_set_parameters.get("featureFlagGRPC"):
+        mongod_options["grpcPort"] = network.PortAllocator.next_fixture_port(job_num)
+
     remove_set_parameter_if_before_version(suite_set_parameters, "internalQueryStatsRateLimit",
-                                           bin_version, "7.0")
+                                           bin_version, "7.3.0")
     remove_set_parameter_if_before_version(
-        suite_set_parameters, "internalQueryStatsErrorsAreCommandFatal", bin_version, "7.0")
+        suite_set_parameters, "internalQueryStatsErrorsAreCommandFatal", bin_version, "7.3.0")
+    remove_set_parameter_if_before_version(suite_set_parameters, "enableAutoCompaction",
+                                           bin_version, "7.3.0")
+
+    if "grpcPort" not in mongod_options and suite_set_parameters.get("featureFlagGRPC"):
+        mongod_options["grpcPort"] = network.PortAllocator.next_fixture_port(job_num)
+
     _apply_set_parameters(args, suite_set_parameters)
+    final_mongod_options = mongod_options.copy()
     mongod_options.pop("set_parameters")
 
     # Apply the rest of the command line arguments.
@@ -110,7 +149,8 @@ def mongod_program(logger, job_num, executable, process_kwargs, mongod_options):
         mongod_options.dump_history(f"{logger.name}_config.yml")
     elif config.EXPORT_MONGOD_CONFIG == "detailed":
         mongod_options.dump_history(f"{logger.name}_config.yml", include_location=True)
-    return make_process(logger, args, **process_kwargs), mongod_options["port"]
+
+    return make_process(logger, args, **process_kwargs), final_mongod_options
 
 
 def mongos_program(logger, job_num, executable=None, process_kwargs=None, mongos_options=None):
@@ -120,17 +160,44 @@ def mongos_program(logger, job_num, executable=None, process_kwargs=None, mongos
 
     mongos_options = mongos_options.copy()
 
+    if config.NOOP_MONGO_D_S_PROCESSES:
+        args[0] = os.path.basename(args[0])
+        mongos_options["set_parameters"]["fassertOnLockTimeoutForStepUpDown"] = 0
+        mongos_options.update({"logpath": "/var/log/mongodb/mongodb.log", "bind_ip": "0.0.0.0"})
+
+    if config.TLS_MODE:
+        mongos_options["tlsMode"] = config.TLS_MODE
+        if config.TLS_MODE != "disabled":
+            mongos_options["tlsAllowInvalidHostnames"] = ""
+
+    if config.MONGOS_TLS_CERTIFICATE_KEY_FILE:
+        mongos_options["tlsCertificateKeyFile"] = config.MONGOS_TLS_CERTIFICATE_KEY_FILE
+
+    if config.TLS_CA_FILE:
+        mongos_options["tlsCAFile"] = config.TLS_CA_FILE
+
     if "port" not in mongos_options:
         mongos_options["port"] = network.PortAllocator.next_fixture_port(job_num)
 
     suite_set_parameters = mongos_options.get("set_parameters", {})
     remove_set_parameter_if_before_version(
         suite_set_parameters, "queryAnalysisSamplerConfigurationRefreshSecs", bin_version, "7.0.0")
+    remove_set_parameter_if_before_version(suite_set_parameters, "defaultConfigCommandTimeoutMS",
+                                           bin_version, "7.3.0")
+
+    if "grpcPort" not in mongos_options and suite_set_parameters.get("featureFlagGRPC"):
+        mongos_options["grpcPort"] = network.PortAllocator.next_fixture_port(job_num)
+
     remove_set_parameter_if_before_version(suite_set_parameters, "internalQueryStatsRateLimit",
-                                           bin_version, "7.0")
+                                           bin_version, "7.3.0")
     remove_set_parameter_if_before_version(
-        suite_set_parameters, "internalQueryStatsErrorsAreCommandFatal", bin_version, "7.0")
+        suite_set_parameters, "internalQueryStatsErrorsAreCommandFatal", bin_version, "7.3.0")
+
+    if "grpcPort" not in mongos_options and suite_set_parameters.get("featureFlagGRPC"):
+        mongos_options["grpcPort"] = network.PortAllocator.next_fixture_port(job_num)
+
     _apply_set_parameters(args, suite_set_parameters)
+    final_mongos_options = mongos_options.copy()
     mongos_options.pop("set_parameters")
 
     # Apply the rest of the command line arguments.
@@ -139,7 +206,19 @@ def mongos_program(logger, job_num, executable=None, process_kwargs=None, mongos
     _set_keyfile_permissions(mongos_options)
 
     process_kwargs = make_historic(utils.default_if_none(process_kwargs, {}))
-    return make_process(logger, args, **process_kwargs), mongos_options["port"]
+
+    return make_process(logger, args, **process_kwargs), final_mongos_options
+
+
+def mongot_program(logger, job_num, executable=None, process_kwargs=None, mongot_options=None):
+    """Return a Process instance that starts a mongot."""
+    args = [executable]
+    mongot_options = mongot_options.copy()
+    final_mongot_options = mongot_options.copy()
+    # Apply the rest of the command line arguments.
+    _apply_kwargs(args, mongot_options)
+    process_kwargs = make_historic(utils.default_if_none(process_kwargs, {}))
+    return make_process(logger, args, **process_kwargs), final_mongot_options
 
 
 def mongo_shell_program(logger, executable=None, connection_string=None, filename=None,
@@ -182,7 +261,6 @@ def mongo_shell_program(logger, executable=None, connection_string=None, filenam
         "storageEngine": (config.STORAGE_ENGINE, ""),
         "storageEngineCacheSizeGB": (config.STORAGE_ENGINE_CACHE_SIZE, ""),
         "testName": (test_name, ""),
-        "transportLayer": (config.TRANSPORT_LAYER, ""),
         "wiredTigerCollectionConfigString": (config.WT_COLL_CONFIG, ""),
         "wiredTigerEngineConfigString": (config.WT_ENGINE_CONFIG, ""),
         "wiredTigerIndexConfigString": (config.WT_INDEX_CONFIG, ""),
@@ -197,11 +275,33 @@ def mongo_shell_program(logger, executable=None, connection_string=None, filenam
             # Only use 'opt_default' if the property wasn't set in the YAML configuration.
             test_data[opt_name] = opt_default
 
+    if config.SHELL_TLS_ENABLED:
+        test_data["shellTlsEnabled"] = True
+
+        if config.SHELL_TLS_CERTIFICATE_KEY_FILE:
+            test_data["shellTlsCertificateKeyFile"] = config.SHELL_TLS_CERTIFICATE_KEY_FILE
+
+    if config.SHELL_GRPC:
+        test_data["shellGRPC"] = True
+
+    if config.TLS_CA_FILE:
+        test_data["tlsCAFile"] = config.TLS_CA_FILE
+
+    if config.TLS_MODE:
+        test_data["tlsMode"] = config.TLS_MODE
+
+    if config.MONGOD_TLS_CERTIFICATE_KEY_FILE:
+        test_data["mongodTlsCertificateKeyFile"] = config.MONGOD_TLS_CERTIFICATE_KEY_FILE
+
+    if config.MONGOS_TLS_CERTIFICATE_KEY_FILE:
+        test_data["mongosTlsCertificateKeyFile"] = config.MONGOS_TLS_CERTIFICATE_KEY_FILE
+
     global_vars["TestData"] = test_data
 
     if config.EVERGREEN_TASK_ID is not None:
         test_data["inEvergreen"] = True
         test_data["evergreenTaskId"] = config.EVERGREEN_TASK_ID
+        test_data["evergreenVariantName"] = config.EVERGREEN_VARIANT_NAME
 
     if config.SHELL_SEED is not None:
         test_data["seed"] = int(config.SHELL_SEED)
@@ -263,6 +363,9 @@ def mongo_shell_program(logger, executable=None, connection_string=None, filenam
     if "configShard" not in test_data and config.CONFIG_SHARD is not None:
         test_data["configShard"] = True
 
+    if "embeddedRouter" not in test_data and config.EMBEDDED_ROUTER is not None:
+        test_data["embeddedRouter"] = True
+
     # There's a periodic background thread that checks for and aborts expired transactions.
     # "transactionLifetimeLimitSeconds" specifies for how long a transaction can run before expiring
     # and being aborted by the background thread. It defaults to 60 seconds, which is too short to
@@ -280,6 +383,9 @@ def mongo_shell_program(logger, executable=None, connection_string=None, filenam
         test_data["connectionString"] = connection_string
         connection_string = None
 
+    if config.FUZZ_MONGOD_CONFIGS is not None and config.FUZZ_MONGOD_CONFIGS is not False:
+        test_data["fuzzMongodConfigs"] = True
+
     for var_name in global_vars:
         _format_shell_vars(eval_sb, [var_name], global_vars[var_name])
 
@@ -287,47 +393,61 @@ def mongo_shell_program(logger, executable=None, connection_string=None, filenam
         eval_sb.append(str(kwargs.pop("eval")))
 
     # Load a callback to check that the cluster-wide metadata is consistent.
-    eval_sb.append("load('jstests/libs/override_methods/check_metadata_consistency.js');")
+    eval_sb.append('await import("jstests/libs/override_methods/check_metadata_consistency.js")')
 
     # Load this file to allow a callback to validate collections before shutting down mongod.
-    eval_sb.append("load('jstests/libs/override_methods/validate_collections_on_shutdown.js');")
+    eval_sb.append(
+        'await import("jstests/libs/override_methods/validate_collections_on_shutdown.js")')
 
     # Load a callback to check UUID consistency before shutting down a ShardingTest.
     eval_sb.append(
-        "load('jstests/libs/override_methods/check_uuids_consistent_across_cluster.js');")
+        'await import("jstests/libs/override_methods/check_uuids_consistent_across_cluster.js")')
 
     # Load a callback to check index consistency before shutting down a ShardingTest.
     eval_sb.append(
-        "load('jstests/libs/override_methods/check_indexes_consistent_across_cluster.js');")
+        'await import("jstests/libs/override_methods/check_indexes_consistent_across_cluster.js")')
 
     # Load a callback to check that all orphans are deleted before shutting down a ShardingTest.
-    eval_sb.append("load('jstests/libs/override_methods/check_orphans_are_deleted.js');")
+    eval_sb.append('await import("jstests/libs/override_methods/check_orphans_are_deleted.js")')
 
     # Load a callback to check that the info stored in config.collections and config.chunks is
     # semantically correct before shutting down a ShardingTest.
-    eval_sb.append("load('jstests/libs/override_methods/check_routing_table_consistency.js');")
+    eval_sb.append(
+        'await import("jstests/libs/override_methods/check_routing_table_consistency.js")')
 
     # Load a callback to check that all shards have correct filtering information before shutting
     # down a ShardingTest.
-    eval_sb.append("load('jstests/libs/override_methods/check_shard_filtering_metadata.js');")
+    eval_sb.append(
+        'await import("jstests/libs/override_methods/check_shard_filtering_metadata.js")')
 
     if config.FUZZ_MONGOD_CONFIGS is not None and config.FUZZ_MONGOD_CONFIGS is not False:
         # Prevent commands from running with the config fuzzer.
         eval_sb.append(
-            "load('jstests/libs/override_methods/config_fuzzer_incompatible_commands.js');")
+            'await import("jstests/libs/override_methods/config_fuzzer_incompatible_commands.js")')
 
     # Load this file to retry operations that fail due to in-progress background operations.
     eval_sb.append(
-        "load('jstests/libs/override_methods/implicitly_retry_on_background_op_in_progress.js');")
+        'await import("jstests/libs/override_methods/implicitly_retry_on_background_op_in_progress.js")'
+    )
 
     eval_sb.append(
-        "(function() { Timestamp.prototype.toString = function() { throw new Error(\"Cannot toString timestamps. Consider using timestampCmp() for comparison or tojson(<variable>) for output.\"); } })();"
+        "(function() { Timestamp.prototype.toString = function() { throw new Error(\"Cannot toString timestamps. Consider using timestampCmp() for comparison or tojson(<variable>) for output.\"); } })()"
     )
 
     eval_str = "; ".join(eval_sb)
     args.append("--eval")
     args.append(eval_str)
 
+    if config.SHELL_TLS_ENABLED:
+        args.extend(["--tls", "--tlsAllowInvalidHostnames"])
+        if config.TLS_CA_FILE:
+            kwargs["tlsCAFile"] = config.TLS_CA_FILE
+        if config.SHELL_TLS_CERTIFICATE_KEY_FILE:
+            kwargs["tlsCertificateKeyFile"] = config.SHELL_TLS_CERTIFICATE_KEY_FILE
+
+    if config.SHELL_GRPC:
+        args.append("--gRPC")
+
     if connection_string is not None:
         # The --host and --port options are ignored by the mongo shell when an explicit connection
         # string is specified. We remove these options to avoid any ambiguity with what server the
@@ -462,6 +582,11 @@ def _set_keyfile_permissions(opts):
     We can't permanently set the keyfile permissions because git is not
     aware of them.
     """
+    for keysuffix in ["1", "2", "ForRollover"]:
+        keyfile = "jstests/libs/key%s" % keysuffix
+        if os.path.exists(keyfile):
+            os.chmod(keyfile, stat.S_IRUSR | stat.S_IWUSR)
+
     if "keyFile" in opts:
         os.chmod(opts["keyFile"], stat.S_IRUSR | stat.S_IWUSR)
     if "encryptionKeyFile" in opts:
