@@ -15,6 +15,7 @@ import platform
 import re
 import signal
 import sys
+import time
 import traceback
 import getpass
 
@@ -112,22 +113,68 @@ class HangAnalyzer(Subcommand):
         dump_pids = {}
         # Dump core files of all processes, except python & java.
         if self.options.dump_core:
-            for pinfo in [
-                    pinfo for pinfo in processes if not re.match("^(java|python)", pinfo.name)
-            ]:
-                if self._check_enough_free_space():
-                    try:
-                        dumpers.dbg.dump_info(pinfo, take_dump=True)
-                    except dumper.DumpError as err:
-                        self.root_logger.error(err.message)
-                        dump_pids = {**err.dump_pids, **dump_pids}
-                    except Exception as err:  # pylint: disable=broad-except
-                        self.root_logger.info("Error encountered when invoking debugger %s", err)
-                        trapped_exceptions.append(traceback.format_exc())
-                else:
-                    self.root_logger.info(
-                        "Not enough space for a core dump, skipping %s processes with PIDs %s",
-                        pinfo.name, str(pinfo.pidv))
+            take_core_processes = [
+                pinfo for pinfo in processes if not re.match("^(java|python)", pinfo.name)
+            ]
+            if (os.getenv('ASAN_OPTIONS') or os.getenv('TSAN_OPTIONS')):
+                quit_processes: list[psutil.Process] = []
+                for pinfo in take_core_processes:
+                    for pid in pinfo.pidv:
+                        # The mongo signal processing thread needs to be resumed to handle the SIGABRT.
+                        quit_process = psutil.Process(pid)
+                        process.resume_process(self.root_logger, pinfo.name, pid)
+                        self.root_logger.info(
+                            "Process %d may be running a sanitizer which uses a large amount of virtual memory.",
+                            pid)
+                        self.root_logger.info(
+                            "Attempting to send SIGABRT from resmoke to capture a more manageable sized core dump"
+                        )
+                        process.signal_process(self.root_logger, pid, signal.SIGABRT)
+                        quit_processes.append(quit_process)
+                self.root_logger.info("Waiting for all processes to end after SIGABRT")
+                assert isinstance(dumpers.dbg, dumper.GDBDumper)
+                timeout = dumpers.dbg.get_timeout_secs()
+                start_time = time.time()
+                # Wait until all processes successfully end
+                while True:
+                    alive_processes = []
+                    # This loop filters out processes that have ended or become a zombie
+                    for quit_process in quit_processes:
+                        if quit_process.is_running(
+                        ) and quit_process.status() != psutil.STATUS_ZOMBIE:
+                            alive_processes.append(quit_process)
+
+                    # Update the quit_processes list with only the ones left alive
+                    quit_processes = alive_processes
+
+                    # All the processes have ended
+                    if not alive_processes:
+                        break
+
+                    if time.time() - start_time > timeout:
+                        raise RuntimeError(
+                            f"The following processes took too long to end after SIGABRT: {alive_processes}"
+                        )
+
+                    time.sleep(.1)
+                self.root_logger.info("Finished waiting for all processes to end.")
+            else:
+                for pinfo in take_core_processes:
+                    if self._check_enough_free_space():
+                        try:
+                            dumpers.dbg.dump_info(pinfo, take_dump=True)
+                        except dumper.DumpError as err:
+                            self.root_logger.error(err.message)
+                            dump_pids = {**err.dump_pids, **dump_pids}
+                        except Exception as err:  # pylint: disable=broad-except
+                            self.root_logger.info("Error encountered when invoking debugger %s",
+                                                  err)
+
+                            trapped_exceptions.append(traceback.format_exc())
+                    else:
+                        self.root_logger.info(
+                            "Not enough space for a core dump, skipping %s processes with PIDs %s",
+                            pinfo.name, str(pinfo.pidv))
 
         # Download symbols after pausing if the task ID is not None and not running with sanitizers.
         # Sanitizer builds are not stripped and don't require debug symbols.
@@ -185,8 +232,7 @@ class HangAnalyzer(Subcommand):
         if self.options.debugger_output is None:
             self.options.debugger_output = ['stdout']
 
-        # add != "" check to avoid empty process_ids
-        if self.options.process_ids is not None and self.options.process_ids != "":
+        if self.options.process_ids is not None:
             # self.process_ids is an int list of PIDs
             self.process_ids = [int(pid) for pid in self.options.process_ids.split(',')]
 
@@ -200,19 +246,18 @@ class HangAnalyzer(Subcommand):
     def _setup_logging(self, logger):
         if logger is None:
             self.root_logger = logging.Logger("hang_analyzer", level=logging.DEBUG)
+            handler = logging.StreamHandler(sys.stdout)
+            handler.setFormatter(logging.Formatter(fmt="%(message)s"))
+            self.root_logger.addHandler(handler)
         else:
             self.root_logger = logger
 
-        handler = logging.StreamHandler(sys.stdout)
-        handler.setFormatter(logging.Formatter(fmt="%(message)s"))
-        self.root_logger.addHandler(handler)
-
     def _log_system_info(self):
         self.root_logger.info("Python Version: %s", sys.version)
         self.root_logger.info("OS: %s", platform.platform())
 
         try:
-            if sys.platform == "win32" or sys.platform == "cygwin":
+            if sys.platform in ["win32", "cygwin"]:
                 self.root_logger.info("Windows Distribution: %s", platform.win32_ver())
             else:
                 self.root_logger.info("Linux Distribution: %s", distro.linux_distribution())
