@@ -7,7 +7,9 @@ from buildscripts.resmokelib import configure_resmoke
 from buildscripts.resmokelib.discovery import DiscoveryPlugin
 from buildscripts.resmokelib.generate_fcv_constants import \
     GenerateFCVConstantsPlugin
+from buildscripts.resmokelib.generate_fuzz_config import GenerateFuzzConfigPlugin
 from buildscripts.resmokelib.hang_analyzer import HangAnalyzerPlugin
+from buildscripts.resmokelib.hang_analyzer.core_analyzer import CoreAnalyzerPlugin
 from buildscripts.resmokelib.multiversion import MultiversionPlugin
 from buildscripts.resmokelib.powercycle import PowercyclePlugin
 from buildscripts.resmokelib.run import RunPlugin
@@ -15,27 +17,38 @@ from buildscripts.resmokelib.undodb import UndoDbPlugin
 
 _PLUGINS = [
     RunPlugin(),
+    CoreAnalyzerPlugin(),
     HangAnalyzerPlugin(),
     UndoDbPlugin(),
     PowercyclePlugin(),
     GenerateFCVConstantsPlugin(),
     DiscoveryPlugin(),
     MultiversionPlugin(),
+    GenerateFuzzConfigPlugin(),
 ]
 
 
-def parse(sys_args, usage=None):
-    """Parse the CLI args."""
-
+def get_parser(usage=None):
+    """Get the resmoke parser."""
     parser = argparse.ArgumentParser(usage=usage)
     subparsers = parser.add_subparsers(dest="command")
     parser.add_argument("--configDir", dest="config_dir", metavar="CONFIG_DIR",
                         help="Directory to search for resmoke configuration files")
+    parser.add_argument(
+        "--jstestsDir", dest="jstests_dir", metavar="CONFIG_DIR",
+        help="Directory to search for jstests files existence while suite validation")
 
     # Add sub-commands.
     for plugin in _PLUGINS:
         plugin.add_subcommand(subparsers)
 
+    return parser
+
+
+def parse(sys_args, usage=None):
+    """Parse the CLI args."""
+
+    parser = get_parser(usage=usage)
     parsed_args = parser.parse_args(sys_args)
 
     return parser, parsed_args
