@@ -38,7 +38,7 @@ from buildscripts.resmokelib.powercycle import powercycle_config, powercycle_con
 from buildscripts.resmokelib.powercycle.lib.services import WindowsService, PosixService
 from buildscripts.resmokelib.utils.filesystem import build_hygienic_bin_path
 
-_IS_WINDOWS = sys.platform == "win32" or sys.platform == "cygwin"
+_IS_WINDOWS = sys.platform in ["win32", "cygwin"]
 _IS_LINUX = sys.platform.startswith("linux")
 _IS_DARWIN = sys.platform == "darwin"
 
@@ -192,7 +192,7 @@ def dump_stacks_and_exit(signum, frame):  # pylint: disable=unused-argument
     LOGGER.info("Dumping stacks!")
 
     sb = []
-    frames = sys._current_frames()  # pylint: disable=protected-access
+    frames = sys._current_frames()
     sb.append("Total threads: {}\n".format(len(frames)))
     sb.append("")
 
@@ -205,7 +205,7 @@ def dump_stacks_and_exit(signum, frame):  # pylint: disable=unused-argument
 
     if _IS_WINDOWS:
         exit_handler()
-        os._exit(1)  # pylint: disable=protected-access
+        os._exit(1)
     else:
         sys.exit(1)
 
@@ -981,7 +981,7 @@ def rsync(src_dir, dest_dir, exclude_files=None):
             exclude_options = "{} --exclude '{}'".format(exclude_options, exclude_file)
 
     LOGGER.info("Rsync'ing %s to %s%s", src_dir, dest_dir, exclude_str)
-    if not distutils.spawn.find_executable("rsync"):
+    if shutil.which("rsync") is None:
         return 1, "No rsync exists on the host, not rsync'ing"
 
     # We retry running the rsync command up to 'max_attempts' times in order to work around how it
@@ -1060,7 +1060,7 @@ def crash_server_or_kill_mongod(task_config, crash_canary, local_ops, script_nam
     LOGGER.info("%s in %d seconds", message_prefix, crash_wait_time)
     time.sleep(crash_wait_time)
 
-    if task_config.crash_method == "internal" or task_config.crash_method == "kill":
+    if task_config.crash_method in ["internal", "kill"]:
         crash_cmd = "crash_server" if task_config.crash_method == "internal" else "kill_mongod"
         crash_func = local_ops.shell
         remote_python = get_remote_python()
@@ -1372,9 +1372,9 @@ def main(parser_actions, options):
     backup_path_after = f"{backup_path_after}-1"
 
     # Setup the mongo client, mongo_path is required if there are local clients.
-    mongo_executable = distutils.spawn.find_executable(
-        "dist-test/bin/mongo",
-        os.getcwd() + os.pathsep + os.environ["PATH"])
+    mongo_executable = shutil.which(cmd="dist-test/bin/mongo",
+                                    path=os.getcwd() + os.pathsep + os.environ["PATH"])
+    # Note: No check for `if mongo_executable is None`
     mongo_path = os.path.abspath(os.path.normpath(mongo_executable))
 
     # Setup the CRUD & FSM clients.
