@@ -2,19 +2,23 @@
 
 import argparse
 import collections
+from logging import Logger
 import os
 import os.path
+import platform
 import random
 import shlex
+import subprocess
 import sys
 import textwrap
 import time
-import shutil
+from typing import List, Optional
 
-import curatorbin
-import pkg_resources
 import psutil
+from opentelemetry import trace
+from opentelemetry.trace.status import StatusCode
 
+from buildscripts.ciconfig.evergreen import parse_evergreen_file
 from buildscripts.resmokelib import parser as main_parser
 from buildscripts.resmokelib import config
 from buildscripts.resmokelib import configure_resmoke
@@ -31,6 +35,9 @@ from buildscripts.resmokelib.run import runtime_recorder
 from buildscripts.resmokelib.run import list_tags
 from buildscripts.resmokelib.run.runtime_recorder import compare_start_time
 from buildscripts.resmokelib.suitesconfig import get_suite_files
+from buildscripts.resmokelib.testing.docker_cluster_image_builder import build_images
+from buildscripts.resmokelib.testing.suite import Suite
+from buildscripts.resmokelib.utils.dictionary import get_dict_value
 
 _INTERNAL_OPTIONS_TITLE = "Internal Options"
 _MONGODB_SERVER_OPTIONS_TITLE = "MongoDB Server Options"
@@ -38,16 +45,18 @@ _BENCHMARK_ARGUMENT_TITLE = "Benchmark/Benchrun test options"
 _EVERGREEN_ARGUMENT_TITLE = "Evergreen options"
 _CEDAR_ARGUMENT_TITLE = "Cedar options"
 
+TRACER = trace.get_tracer("resmoke")
+
 
 class TestRunner(Subcommand):
     """The main class to run tests with resmoke."""
 
-    def __init__(self, command, start_time=time.time()):
+    def __init__(self, command: str, start_time=time.time()):
         """Initialize the Resmoke instance."""
         self.__start_time = start_time
         self.__command = command
-        self._exec_logger = None
-        self._resmoke_logger = None
+        self._exec_logger: Optional[Logger] = None
+        self._resmoke_logger: Optional[Logger] = None
         self._archive = None
         self._interrupted = False
         self._exit_code = 0
@@ -181,7 +190,7 @@ class TestRunner(Subcommand):
             self._resmoke_logger)
 
     @staticmethod
-    def _find_suites_by_test(suites):
+    def _find_suites_by_test(suites: List[Suite]):
         """
         Look up what other resmoke suites run the tests specified in the suites parameter.
 
@@ -207,6 +216,10 @@ class TestRunner(Subcommand):
 
     def run_tests(self):
         """Run the suite and tests specified."""
+        # This code path should only execute when resmoke is running from a workload container.
+        if config.REQUIRES_WORKLOAD_CONTAINER_SETUP:
+            self._setup_workload_container()
+
         self._resmoke_logger.info("verbatim resmoke.py invocation: %s",
                                   " ".join([shlex.quote(arg) for arg in sys.argv]))
         self._check_for_mongo_processes()
@@ -249,7 +262,32 @@ class TestRunner(Subcommand):
             if suites:
                 reportfile.write(suites)
 
-    def _run_suite(self, suite):
+    def _setup_workload_container(self):
+        """Perform any setup needed to run this resmoke suite from within the workload container."""
+        # Right now, this is only needed to setup jstestfuzz suites. We do a simple check
+        # of the 'roots' and generate jstestfuzz test files if they are expected.
+        #
+        # If this expands, we should definitely do this more intelligently.
+        # ie: run the exact bash scripts from the corresponding evergreen task defintion
+        #     so that everything is setup exactly how it would be in Evergreen.
+
+        jstestfuzz_repo_dir = "/mongo/jstestfuzz"
+        jstests_dir = "/mongo/jstests"
+        jstestfuzz_tests_dir = "/mongo/jstestfuzz/out"
+
+        # Currently, you can only run one suite at a time from within a workload container
+        suite = self._get_suites()[0]
+        if "jstestfuzz/out/*.js" in suite.get_selector_config().get("roots", []) and not any(
+                filename.endswith(".js") for filename in os.listdir(jstestfuzz_tests_dir)):
+            subprocess.run([
+                "./src/scripts/npm_run.sh",
+                "jstestfuzz",
+                "--",
+                "--jsTestsDir",
+                jstests_dir,
+            ], cwd=jstestfuzz_repo_dir, stdout=sys.stdout, stderr=sys.stderr, check=True)
+
+    def _run_suite(self, suite: Suite):
         """Run a test suite."""
         self._log_suite_config(suite)
         suite.record_suite_start()
@@ -260,49 +298,145 @@ class TestRunner(Subcommand):
 
     def _log_local_resmoke_invocation(self):
         """Log local resmoke invocation example."""
+
+        # Do not log local args if this is not being ran in evergreen
+        if not config.EVERGREEN_TASK_ID:
+            print("Skipping local invocation because evergreen task id was not provided.")
+            return
+
+        evg_conf = parse_evergreen_file(config.EVERGREEN_PROJECT_CONFIG_PATH)
+
+        suite = self._get_suites()[0]
+        suite_name = config.ORIGIN_SUITE or suite.get_name()
+
+        # try to find the evergreen task from the resmoke suite name
+        task = evg_conf.get_task(suite_name) or evg_conf.get_task(f"{suite_name}_gen")
+
+        multiversion_bin_version = None
+        # Some evergreen task names do not reflect what suite names they run.
+        # The suite names should be in the evergreen functions in this case
+        if task is None:
+            for current_task in evg_conf.tasks:
+                func = current_task.find_func_command("run tests") \
+                    or current_task.find_func_command("generate resmoke tasks") \
+                    or current_task.find_func_command("run benchmark tests")
+                if func and get_dict_value(func, ["vars", "suite"]) == suite_name:
+                    task = current_task
+                    break
+
+                func = current_task.find_func_command("initialize multiversion tasks")
+                if not func:
+                    continue
+                for subtask in func["vars"]:
+                    if subtask == suite_name:
+                        task = current_task
+                        multiversion_bin_version = func["vars"][subtask]
+                        break
+
+                if task:
+                    break
+
+        if task is None:
+            raise RuntimeError(f"Error: Could not find evergreen task definition for {suite_name}")
+
+        is_multiversion = "multiversion" in task.tags
+        generate_func = task.find_func_command("generate resmoke tasks")
+        is_jstestfuzz = False
+        if generate_func:
+            is_jstestfuzz = get_dict_value(generate_func, ["vars", "is_jstestfuzz"]) == "true"
+
         local_args = to_local_args()
+        local_args = strip_fuzz_config_params(local_args)
         local_resmoke_invocation = (
             f"{os.path.join('buildscripts', 'resmoke.py')} {' '.join(local_args)}")
 
+        using_config_fuzzer = False
         if config.FUZZ_MONGOD_CONFIGS:
-            local_args = strip_fuzz_config_params(local_args)
-            local_resmoke_invocation = (
-                f"{os.path.join('buildscripts', 'resmoke.py')} {' '.join(local_args)}"
-                f" --fuzzMongodConfigs={config.FUZZ_MONGOD_CONFIGS} --configFuzzSeed={str(config.CONFIG_FUZZ_SEED)}"
-            )
+            using_config_fuzzer = True
+            local_resmoke_invocation += f" --fuzzMongodConfigs={config.FUZZ_MONGOD_CONFIGS}"
 
             self._resmoke_logger.info("Fuzzed mongodSetParameters:\n%s",
                                       config.MONGOD_SET_PARAMETERS)
             self._resmoke_logger.info("Fuzzed wiredTigerConnectionString: %s",
                                       config.WT_ENGINE_CONFIG)
+
+        if config.FUZZ_MONGOS_CONFIGS:
+            using_config_fuzzer = True
+            local_resmoke_invocation += f" --fuzzMongosConfigs={config.FUZZ_MONGOS_CONFIGS}"
+
+            self._resmoke_logger.info("Fuzzed mongosSetParameters:\n%s",
+                                      config.MONGOS_SET_PARAMETERS)
+
+        if using_config_fuzzer:
+            local_resmoke_invocation += f" --configFuzzSeed={str(config.CONFIG_FUZZ_SEED)}"
+
+        if multiversion_bin_version:
+            default_tag_file = config.DEFAULTS["exclude_tags_file_path"]
+            local_resmoke_invocation += f" --tagFile={default_tag_file}"
+
         resmoke_env_options = ''
         if os.path.exists('resmoke_env_options.txt'):
             with open('resmoke_env_options.txt') as fin:
                 resmoke_env_options = fin.read().strip()
 
-        self._resmoke_logger.info("resmoke.py invocation for local usage: %s %s",
-                                  resmoke_env_options, local_resmoke_invocation)
+        local_resmoke_invocation = f"{resmoke_env_options} {local_resmoke_invocation}"
+        self._resmoke_logger.info("resmoke.py invocation for local usage: %s",
+                                  local_resmoke_invocation)
+
+        lines = []
+
+        if is_multiversion:
+            lines.append("# DISCLAIMER:")
+            lines.append(
+                "#     The `db-contrib-tool` command downloads the latest last-continuous/lts mongo shell binaries available in CI."
+            )
+            if multiversion_bin_version:
+                lines.append(
+                    "#     The generated `multiversion_exclude_tags.yml` is dependent on the `backports_required_for_multiversion_tests.yml` file of the last-continuous/lts mongo shell binary git commit."
+                )
+            lines.append(
+                "#     If there have been new commits to last-continuous/lts, the excluded tests & binaries may be slightly different on this task vs locally."
+            )
+        if is_jstestfuzz:
+            lines.append(
+                "# This is a jstestfuzz suite and is dependent on the generated tests specific to this task execution."
+            )
 
-        suite = self._get_suites()[0]
         if suite.get_description():
-            self._resmoke_logger.info("'%s' suite description:\n\n%s\n", suite.get_name(),
-                                      suite.get_description())
+            lines.append(f"# {suite.get_description()}")
 
-        if suite.is_matrix_suite():
-            self._resmoke_logger.info(
-                "This suite is a matrix suite. To view the generated matrix suite run python3 ./buildscripts/resmoke.py suiteconfig %s",
-                suite.get_name())
+        lines.append(
+            "# Having trouble reproducing your failure with this? Feel free to reach out in #server-testing."
+        )
+        lines.append("")
+        if is_multiversion:
+            if not os.path.exists("local-db-contrib-tool-invocation.txt"):
+                raise RuntimeError(
+                    "ERROR: local-db-contrib-tool-invocation.txt does not exist for multiversion task"
+                )
 
-        if config.EVERGREEN_TASK_ID:
-            with open("local-resmoke-invocation.txt", "w") as fh:
-                lines = [f"{resmoke_env_options} {local_resmoke_invocation}"]
-                if suite.get_description():
-                    lines.append(f"{suite.get_name()}: {suite.get_description()}")
-                if suite.is_matrix_suite():
-                    lines.append(
-                        f"This suite is a matrix suite. To view the generated matrix suite run python3 ./buildscripts/resmoke.py suiteconfig {suite.get_name()}"
-                    )
-                fh.write("\n".join(lines))
+            with open("local-db-contrib-tool-invocation.txt", "r") as fh:
+                db_contrib_tool_invocation = fh.read().strip() + " && \\"
+                lines.append(db_contrib_tool_invocation)
+
+            if multiversion_bin_version:
+                generate_tag_file_invocation = f"buildscripts/resmoke.py generate-multiversion-exclude-tags --oldBinVersion={multiversion_bin_version} && \\"
+                lines.append(generate_tag_file_invocation)
+
+        if is_jstestfuzz:
+            download_url = f"https://mciuploads.s3.amazonaws.com/{config.EVERGREEN_PROJECT_NAME}/{config.EVERGREEN_VARIANT_NAME}/{config.EVERGREEN_REVISION}/jstestfuzz/{config.EVERGREEN_TASK_ID}-{config.EVERGREEN_EXECUTION}.tgz"
+            jstestfuzz_dir = "jstestfuzz/"
+            jstests_tar = "jstests.tgz"
+            lines.append(f"mkdir -p {jstestfuzz_dir} && \\")
+            lines.append(f"rm -rf {jstestfuzz_dir}* && \\")
+            lines.append(f"wget '{download_url}' -O {jstests_tar} && \\")
+            lines.append(f"tar -xf {jstests_tar} -C {jstestfuzz_dir} && \\")
+            lines.append(f"rm {jstests_tar} && \\")
+
+        lines.append(local_resmoke_invocation)
+
+        with open("local-resmoke-invocation.txt", "w") as fh:
+            fh.write("\n".join(lines))
 
     def _check_for_mongo_processes(self):
         """Check for existing mongo processes as they could interfere with running the tests."""
@@ -318,6 +452,20 @@ class TestRunner(Subcommand):
                 parent_resmoke_ctime = proc.environ().get('RESMOKE_PARENT_CTIME')
                 if not parent_resmoke_pid:
                     continue
+                if platform.system() == "Darwin":
+                    # On macOS, 'psutil.Process.environ' gives non-sensical output if the calling process
+                    # does not have permission/entitlement to do so. Refer to the doc for psutil at
+                    # https://psutil.readthedocs.io/en/stable/#psutil.Process.environ.
+                    # Worst case, it returns the environment variables of a different process,
+                    # which may unluckily contain RESMOKE_PARENT_PROCESS. To avoid attempting to kill arbitrary
+                    # processes, double-check the environment against the results from 'ps'. Double-check rather
+                    # than use ps for all processes because the subprocess handling is slower in the common case
+                    # where there are no rogue resmoke processes.
+                    cmd = ["ps", "-E", str(proc.pid)]
+                    ps_proc = subprocess.run(cmd, capture_output=True)
+                    if (f"RESMOKE_PARENT_PROCESS={parent_resmoke_pid}" not in
+                            ps_proc.stdout.decode()):
+                        continue
                 if psutil.pid_exists(int(parent_resmoke_pid)):
                     # Double check `parent_resmoke_pid` is really a rooting resmoke process. Having
                     # the RESMOKE_PARENT_PROCESS environment variable proves it is a process which
@@ -391,41 +539,85 @@ class TestRunner(Subcommand):
         if len(config.SUITE_FILES) > 1:
             testing.suite.Suite.log_summaries(self._resmoke_logger, suites, time_taken)
 
-    def _log_suite_summary(self, suite):
+    def _log_suite_summary(self, suite: Suite):
         """Log a summary of the suite run."""
         self._resmoke_logger.info("=" * 80)
         self._resmoke_logger.info("Summary of %s suite: %s", suite.get_display_name(),
                                   self._get_suite_summary(suite))
 
-    def _execute_suite(self, suite):
-        """Execute a suite and return True if interrupted, False otherwise."""
+    @TRACER.start_as_current_span("run.__init__._execute_suite")
+    def _execute_suite(self, suite: Suite) -> bool:
+        """Execute Fa suite and return True if interrupted, False otherwise."""
+        execute_suite_span = trace.get_current_span()
+        execute_suite_span.set_attributes(attributes=suite.get_suite_otel_attributes())
         self._shuffle_tests(suite)
-        if not suite.tests:
+
+        # During "docker image build", we are not actually running any tests, so we do not care if there are no tests.
+        # Specifically, when building the images for a jstestfuzz external SUT the jstestfuzz test files will not exist yet -- but we still want to
+        # build the external SUT infrastructure. The jstestfuzz tests will be generated at runtime via the `_setup_workload_container` method when
+        # actually running resmoke from within the workload container against the jstestfuzz external SUT.
+        if not config.DOCKER_COMPOSE_BUILD_IMAGES and not suite.tests:
             self._exec_logger.info("Skipping %s, no tests to run", suite.test_kind)
             suite.return_code = 0
+            execute_suite_span.set_status(StatusCode.OK)
+            execute_suite_span.set_attributes({
+                Suite.METRIC_NAMES.RETURN_CODE: suite.return_code,
+                Suite.METRIC_NAMES.RETURN_STATUS: "skipped",
+            })
             return False
         executor_config = suite.get_executor_config()
         try:
             executor = testing.executor.TestSuiteExecutor(
                 self._exec_logger, suite, archive_instance=self._archive, **executor_config)
-            executor.run()
+            # If this is a "docker compose build", we just build the docker compose images for
+            # this resmoke configuration and exit.
+            if config.DOCKER_COMPOSE_BUILD_IMAGES:
+                build_images(suite.get_name(), executor._jobs[0].fixture)
+                suite.return_code = 0
+            else:
+                executor.run()
         except (errors.UserInterrupt, errors.LoggerRuntimeConfigError) as err:
             self._exec_logger.error("Encountered an error when running %ss of suite %s: %s",
                                     suite.test_kind, suite.get_display_name(), err)
             suite.return_code = err.EXIT_CODE
+            return_status = "user_interrupt" if isinstance(
+                err, errors.UserInterrupt) else "logger_runtime_config"
+            execute_suite_span.set_status(StatusCode.ERROR, description=return_status)
+            execute_suite_span.set_attributes({
+                Suite.METRIC_NAMES.RETURN_CODE: suite.return_code,
+                Suite.METRIC_NAMES.RETURN_STATUS: return_status,
+            })
             return True
         except OSError as err:
             self._exec_logger.error("Encountered an OSError: %s", err)
             suite.return_code = 74  # Exit code for OSError on POSIX systems.
+            return_status = "os_error"
+            execute_suite_span.set_status(StatusCode.ERROR, description=return_status)
+            execute_suite_span.set_attributes({
+                Suite.METRIC_NAMES.RETURN_CODE: suite.return_code,
+                Suite.METRIC_NAMES.RETURN_STATUS: return_status,
+                Suite.METRIC_NAMES.ERRORNO: err.errno
+            })
             return True
         except:  # pylint: disable=bare-except
             self._exec_logger.exception("Encountered an error when running %ss of suite %s.",
                                         suite.test_kind, suite.get_display_name())
             suite.return_code = 2
+            return_status = "unknown_error"
+            execute_suite_span.set_status(StatusCode.ERROR, description=return_status)
+            execute_suite_span.set_attributes({
+                Suite.METRIC_NAMES.RETURN_CODE: suite.return_code,
+                Suite.METRIC_NAMES.RETURN_STATUS: return_status,
+            })
             return False
+        execute_suite_span.set_status(StatusCode.OK)
+        execute_suite_span.set_attributes({
+            Suite.METRIC_NAMES.RETURN_CODE: suite.return_code,
+            Suite.METRIC_NAMES.RETURN_STATUS: "success",
+        })
         return False
 
-    def _shuffle_tests(self, suite):
+    def _shuffle_tests(self, suite: Suite):
         """Shuffle the tests if the shuffle cli option was set."""
         random.seed(config.RANDOM_SEED)
         if not config.SHUFFLE:
@@ -435,7 +627,7 @@ class TestRunner(Subcommand):
         random.shuffle(suite.tests)
 
     # pylint: disable=inconsistent-return-statements
-    def _get_suites(self):
+    def _get_suites(self) -> List[Suite]:
         """Return the list of suites for this resmoke invocation."""
         try:
             return suitesconfig.get_suites(config.SUITE_FILES, config.TEST_FILES)
@@ -443,13 +635,17 @@ class TestRunner(Subcommand):
             self._resmoke_logger.error("Failed to parse YAML suite definition: %s", str(err))
             self.list_suites()
             self.exit(1)
-        except errors.ResmokeError as err:
+        except errors.InvalidMatrixSuiteError as err:
+            self._resmoke_logger.error("Failed to get matrix suite: %s", str(err))
+            self.exit(1)
+        except errors.TestExcludedFromSuiteError as err:
             self._resmoke_logger.error(
                 "Cannot run excluded test in suite config. Use '--force-excluded-tests' to override: %s",
                 str(err))
             self.exit(1)
+        return []
 
-    def _log_suite_config(self, suite):
+    def _log_suite_config(self, suite: Suite):
         sb = [
             "YAML configuration of suite {}".format(suite.get_display_name()),
             utils.dump_yaml({"test_kind": suite.get_test_kind_config()}), "",
@@ -460,13 +656,13 @@ class TestRunner(Subcommand):
         self._resmoke_logger.info("\n".join(sb))
 
     @staticmethod
-    def _get_suite_summary(suite):
+    def _get_suite_summary(suite: Suite):
         """Return a summary of the suite run."""
-        sb = []
+        sb: List[str] = []
         suite.summarize(sb)
         return "\n".join(sb)
 
-    def _setup_signal_handler(self, suites):
+    def _setup_signal_handler(self, suites: List[Suite]):
         """Set up a SIGUSR1 signal handler that logs a test result summary and a thread dump."""
         sighandler.register(self._resmoke_logger, suites, self.__start_time)
 
@@ -482,7 +678,7 @@ class TestRunner(Subcommand):
         if self._archive and not self._interrupted:
             self._archive.exit()
 
-    def exit(self, exit_code):
+    def exit(self, exit_code: int):
         """Exit with the provided exit code."""
         self._exit_code = exit_code
         self._resmoke_logger.info("Exiting with code: %d", exit_code)
@@ -544,7 +740,7 @@ class TestRunnerEvg(TestRunner):
 
         return combinations
 
-    def _get_suites(self):
+    def _get_suites(self) -> List[Suite]:
         """Return a list of resmokelib.testing.suite.Suite instances to execute.
 
         For every resmokelib.testing.suite.Suite instance returned by resmoke.Main._get_suites(),
@@ -555,7 +751,7 @@ class TestRunnerEvg(TestRunner):
         suites = []
 
         for suite in TestRunner._get_suites(self):
-            if suite.test_kind != "js_test":
+            if suite.test_kind not in ("js_test", "fsm_workload_test"):
                 # Tags are only support for JavaScript tests, so we leave the test suite alone when
                 # running any other kind of test.
                 suites.append(suite)
@@ -622,7 +818,7 @@ class RunPlugin(PluginInterface):
         return None
 
     @classmethod
-    def _add_run(cls, subparsers):
+    def _add_run(cls, subparsers: argparse._SubParsersAction):
         """Create and add the parser for the Run subcommand."""
         parser = subparsers.add_parser("run", help="Runs the specified tests.")
 
@@ -692,6 +888,40 @@ class RunPlugin(PluginInterface):
                   " only tests which have at least one of the specified tags will be"
                   " run."))
 
+        parser.add_argument(
+            "--dockerComposeBuildImages", dest="docker_compose_build_images",
+            metavar="IMAGE1,IMAGE2,IMAGE3", help=
+            ("Comma separated list of base images to build for running resmoke against an External System Under Test:"
+             " (1) `workload`: Your mongo repo with a python development environment setup."
+             " (2) `mongo-binaries`: The `mongo`, `mongod`, `mongos` binaries to run tests with."
+             " (3) `config`: The target suite's `docker-compose.yml` file, startup scripts & configuration."
+             " All three images are needed to successfully setup an External System Under Test."
+             " This will not run any tests. It will just build the images and generate"
+             " the `docker-compose.yml` configuration to set up the External System Under Test for the desired suite."
+             ))
+
+        parser.add_argument(
+            "--dockerComposeBuildEnv", dest="docker_compose_build_env",
+            choices=["local", "evergreen"], default="local", help=
+            ("Set the environment where this `--dockerComposeBuildImages` is happening -- defaults to: `local`."
+             ))
+
+        parser.add_argument(
+            "--dockerComposeTag", dest="docker_compose_tag", metavar="TAG", default="development",
+            help=("The `tag` name to use for images built during a `--dockerComposeBuildImages`."))
+
+        parser.add_argument(
+            "--externalSUT", dest="external_sut", action="store_true", default=False, help=
+            ("This option should only be used when running resmoke against an External System Under Test."
+             " The External System Under Test should be setup via the command generated after"
+             " running: `buildscripts/resmoke.py run --suite [suite_name] ... --dockerComposeBuildImages"
+             " config,workload,mongo-binaries`."))
+
+        parser.add_argument(
+            "--sanityCheck", action="store_true", dest="sanity_check", help=
+            "Truncate the test queue to 1 item, just in order to verify the suite is properly set up."
+        )
+
         parser.add_argument(
             "--includeWithAllTags", action="append", dest="include_with_all_tags",
             metavar="TAG1,TAG2",
@@ -734,6 +964,11 @@ class RunPlugin(PluginInterface):
         parser.add_argument("--numClientsPerFixture", type=int, dest="num_clients_per_fixture",
                             help="Number of clients running tests per fixture.")
 
+        parser.add_argument(
+            "--useTenantClient", default=False, dest="use_tenant_client", action="store_true", help=
+            "Use tenant client. If set, each client will be constructed with a generated tenant id."
+        )
+
         parser.add_argument(
             "--shellConnString", dest="shell_conn_string", metavar="CONN_STRING",
             help="Overrides the default fixture and connects with a mongodb:// connection"
@@ -747,6 +982,16 @@ class RunPlugin(PluginInterface):
             " existing MongoDB cluster with the URL mongodb://localhost:[PORT]."
             " This is useful for connecting to a server running in a debugger.")
 
+        parser.add_argument("--shellGRPC", dest="shell_grpc", action="store_true",
+                            help="Whether to use gRPC by default when connecting via the shell.")
+
+        parser.add_argument("--shellTls", dest="shell_tls_enabled", action="store_true",
+                            help="Whether to use TLS when connecting.")
+
+        parser.add_argument("--shellTlsCertificateKeyFile", dest="shell_tls_certificate_key_file",
+                            metavar="SHELL_TLS_CERTIFICATE_KEY_FILE",
+                            help="The TLS certificate to use when connecting.")
+
         parser.add_argument("--repeat", "--repeatSuites", type=int, dest="repeat_suites",
                             metavar="N",
                             help="Repeats the given suite(s) N times, or until one fails.")
@@ -851,6 +1096,11 @@ class RunPlugin(PluginInterface):
         parser.add_argument("--maxTestQueueSize", type=int, dest="max_test_queue_size",
                             help=argparse.SUPPRESS)
 
+        parser.add_argument("--tagFile", action="append", dest="tag_files", metavar="TAG_FILES",
+                            help="One or more YAML files that associate tests and tags.")
+
+        configure_resmoke.add_otel_args(parser)
+
         mongodb_server_options = parser.add_argument_group(
             title=_MONGODB_SERVER_OPTIONS_TITLE,
             description=("Options related to starting a MongoDB cluster that are forwarded from"
@@ -911,6 +1161,25 @@ class RunPlugin(PluginInterface):
             help="Sets the storage engine cache size configuration"
             " setting for all mongod's.")
 
+        mongodb_server_options.add_argument(
+            "--tlsMode", dest="tls_mode", metavar="TLS_MODE", help="Indicates what TLS mode mongod "
+            "and mongos servers should be started with. See also: https://www.mongodb.com"
+            "/docs/manual/reference/configuration-options/#mongodb-setting-net.tls.mode")
+
+        mongodb_server_options.add_argument(
+            "--tlsCAFile", dest="tls_ca_file", metavar="TLS_CA_FILE",
+            help="Path to the CA certificate file to be used by all clients and servers.")
+
+        mongodb_server_options.add_argument(
+            "--mongodTlsCertificateKeyFile", dest="mongod_tls_certificate_key_file",
+            metavar="MONGOD_TLS_CERTIFICATE_KEY_FILE",
+            help="Path to the TLS certificate to be used by all mongods.")
+
+        mongodb_server_options.add_argument(
+            "--mongosTlsCertificateKeyFile", dest="mongos_tls_certificate_key_file",
+            metavar="MONGOS_TLS_CERTIFICATE_KEY_FILE",
+            help="Path to the TLS certificate to be used by all mongoses.")
+
         mongodb_server_options.add_argument(
             "--numReplSetNodes", type=int, dest="num_replset_nodes", metavar="N",
             help="The number of nodes to initialize per ReplicaSetFixture. This is also "
@@ -933,25 +1202,30 @@ class RunPlugin(PluginInterface):
             "--wiredTigerIndexConfigString", dest="wt_index_config", metavar="CONFIG",
             help="Sets the WiredTiger index configuration setting for all mongod's.")
 
-        mongodb_server_options.add_argument("--transportLayer", dest="transport_layer",
-                                            metavar="TRANSPORT",
-                                            help="The transport layer used by jstests")
-
         mongodb_server_options.add_argument(
             "--fuzzMongodConfigs", dest="fuzz_mongod_configs",
-            help="Randomly chooses server parameters that were not specified. Use 'stress' to fuzz "
+            help="Randomly chooses mongod parameters that were not specified. Use 'stress' to fuzz "
             "all configs including stressful storage configurations that may significantly "
             "slow down the server. Use 'normal' to only fuzz non-stressful configurations. ",
             metavar="MODE", choices=('normal', 'stress'))
 
-        mongodb_server_options.add_argument("--configFuzzSeed", dest="config_fuzz_seed",
-                                            metavar="PATH",
-                                            help="Sets the seed used by storage config fuzzer")
+        mongodb_server_options.add_argument(
+            "--fuzzMongosConfigs", dest="fuzz_mongos_configs",
+            help="Randomly chooses mongos parameters that were not specified", metavar="MODE",
+            choices=('normal', ))
+
+        mongodb_server_options.add_argument(
+            "--configFuzzSeed", dest="config_fuzz_seed", metavar="PATH",
+            help="Sets the seed used by mongod and mongos config fuzzers")
 
         mongodb_server_options.add_argument(
             "--configShard", dest="config_shard", metavar="CONFIG",
             help="If set, specifies which node is the config shard. Can also be set to 'any'.")
 
+        mongodb_server_options.add_argument(
+            "--embeddedRouter", dest="embedded_router", metavar="CONFIG",
+            help="If set, uses embedded routers instead of dedicated mongos.")
+
         internal_options = parser.add_argument_group(
             title=_INTERNAL_OPTIONS_TITLE,
             description=("Internal options for advanced users and resmoke developers."
@@ -981,10 +1255,6 @@ class RunPlugin(PluginInterface):
         internal_options.add_argument("--internalParam", action="append", dest="internal_params",
                                       help=argparse.SUPPRESS)
 
-        internal_options.add_argument("--perfReportFile", dest="perf_report_file",
-                                      metavar="PERF_REPORT",
-                                      help="Writes a JSON file with performance test results.")
-
         internal_options.add_argument("--cedarReportFile", dest="cedar_report_file",
                                       metavar="CEDAR_REPORT",
                                       help="Writes a JSON file with performance test results.")
@@ -1068,10 +1338,6 @@ class RunPlugin(PluginInterface):
                                        metavar="REVISION_ORDER_ID",
                                        help="Sets the chronological order number of this commit.")
 
-        evergreen_options.add_argument("--tagFile", action="append", dest="tag_files",
-                                       metavar="TAG_FILES",
-                                       help="One or more YAML files that associate tests and tags.")
-
         evergreen_options.add_argument(
             "--taskName", dest="task_name", metavar="TASK_NAME",
             help="Sets the name of the Evergreen task running the tests.")
@@ -1090,6 +1356,10 @@ class RunPlugin(PluginInterface):
         evergreen_options.add_argument("--taskWorkDir", dest="work_dir", metavar="TASK_WORK_DIR",
                                        help="Sets the working directory of the task.")
 
+        evergreen_options.add_argument(
+            "--projectConfigPath", dest="evg_project_config_path",
+            help="Sets the path to evergreen project configuration yaml.")
+
         benchmark_options = parser.add_argument_group(
             title=_BENCHMARK_ARGUMENT_TITLE,
             description="Options for running Benchmark/Benchrun tests")
@@ -1123,7 +1393,7 @@ class RunPlugin(PluginInterface):
             metavar="BENCHMARK_REPETITIONS", help=benchmark_repetitions_help)
 
     @classmethod
-    def _add_list_suites(cls, subparsers):
+    def _add_list_suites(cls, subparsers: argparse._SubParsersAction):
         """Create and add the parser for the list-suites subcommand."""
         parser = subparsers.add_parser("list-suites",
                                        help="Lists the names of the suites available to execute.")
@@ -1136,13 +1406,13 @@ class RunPlugin(PluginInterface):
         parser.set_defaults(logger_file="console")
 
     @classmethod
-    def _add_generate(cls, subparsers):
+    def _add_generate(cls, subparsers: argparse._SubParsersAction):
         """Create and add the parser for the generate subcommand."""
         subparsers.add_parser("generate-matrix-suites",
                               help="Generate matrix suite config files from the mapping files.")
 
     @classmethod
-    def _add_find_suites(cls, subparsers):
+    def _add_find_suites(cls, subparsers: argparse._SubParsersAction):
         """Create and add the parser for the find-suites subcommand."""
         parser = subparsers.add_parser(
             "find-suites",
@@ -1156,7 +1426,7 @@ class RunPlugin(PluginInterface):
                             help="Explicit test files to run")
 
     @classmethod
-    def _add_list_tags(cls, subparsers):
+    def _add_list_tags(cls, subparsers: argparse._SubParsersAction):
         """Create and add the parser for the list-tags subcommand."""
         parser = subparsers.add_parser(
             "list-tags", help="Lists the tags and their documentation available in the suites.")
@@ -1167,7 +1437,7 @@ class RunPlugin(PluginInterface):
                   " All suites are used if unspecified."))
 
     @classmethod
-    def _add_generate_multiversion_exclude_tags(cls, subparser):
+    def _add_generate_multiversion_exclude_tags(cls, subparser: argparse._SubParsersAction):
         """Create and add the parser for the generate-multiversion-exclude-tags subcommand."""
         parser = subparser.add_parser(
             "generate-multiversion-exclude-tags",
@@ -1184,7 +1454,7 @@ class RunPlugin(PluginInterface):
                             help="Where to output the generated tags.")
 
 
-def to_local_args(input_args=None):
+def to_local_args(input_args: Optional[List[str]] = None):
     """
     Return a command line invocation for resmoke.py suitable for being run outside of Evergreen.
 
@@ -1217,6 +1487,9 @@ def to_local_args(input_args=None):
 
     run_parser = command_subparser.choices.get("run")
 
+    # arguments that are in the standard run parser that we do not want to include in the local invocation
+    skipped_args = ["install_dir", "tag_files"]
+
     suites_arg = None
     storage_engine_arg = None
     other_local_args = []
@@ -1228,7 +1501,10 @@ def to_local_args(input_args=None):
 
         This function assumes that 'option_name' is always "--" prefix and isn't "-" prefixed.
         """
-        return f"{option_name}={option_value}"
+        if " " not in str(option_value):
+            return f"{option_name}={option_value}"
+        else:
+            return f"'{option_name}={option_value}'"
 
     # Trim the argument namespace of any args we don't want to return.
     for group in run_parser._action_groups:  # pylint: disable=protected-access
@@ -1254,6 +1530,8 @@ def to_local_args(input_args=None):
                     _INTERNAL_OPTIONS_TITLE, _EVERGREEN_ARGUMENT_TITLE, _CEDAR_ARGUMENT_TITLE
             ]:
                 continue
+            elif arg_dest in skipped_args:
+                continue
             elif group.title == 'positional arguments':
                 positional_args.extend(arg_value)
             # Keep all remaining args.
@@ -1272,11 +1550,6 @@ def to_local_args(input_args=None):
                 else:
                     arg = format_option(arg_name, arg_value)
 
-                    # In evergreen we use additionalFeatureFlagsFile because the all_feature_flags.txt
-                    # file is generated at a previous step. Developers should use runAllFeatureFlagTests
-                    if arg == "--additionalFeatureFlagsFile=all_feature_flags.txt":
-                        arg = "--runAllFeatureFlagTests"
-
                     # We track the value for the --suites and --storageEngine command line options
                     # separately in order to more easily sort them to the front.
                     if arg_dest == "suite_files":
@@ -1290,12 +1563,12 @@ def to_local_args(input_args=None):
                       ] + other_local_args + positional_args
 
 
-def strip_fuzz_config_params(input_args):
+def strip_fuzz_config_params(input_args: List[str]):
     """Delete fuzz related command line args because we have to add the seed manually."""
 
     ret = []
     for arg in input_args:
-        if "--fuzzMongodConfigs" not in arg and "--fuzzConfigSeed" not in arg:
+        if not arg.startswith(("--fuzzMongodConfigs", "--fuzzMongosConfigs", "--configFuzzSeed")):
             ret.append(arg)
 
     return ret
