@@ -1,22 +1,21 @@
 """Utilities for constructing fixtures that may span multiple versions."""
-import io
 import logging
-import os
 import threading
 from abc import ABC, abstractmethod
 from typing import Any, Dict, List, Optional, Tuple, Type
 
-from git import Repo
-
-import buildscripts.resmokelib.config as config
-import buildscripts.resmokelib.utils.registry as registry
+from buildscripts.resmokelib import config
+from buildscripts.resmokelib import logging
+from buildscripts.resmokelib.utils import registry
 from buildscripts.resmokelib import errors
 from buildscripts.resmokelib.testing.fixtures.fixturelib import FixtureLib
 from buildscripts.resmokelib.testing.fixtures.interface import _FIXTURES
+from buildscripts.resmokelib.suitesconfig import _get_suite_config
+from buildscripts.resmokelib.testing import suite as _suite
 from buildscripts.resmokelib.testing.fixtures.replicaset import \
     ReplicaSetFixture
 from buildscripts.resmokelib.testing.fixtures.shardedcluster import \
-    ShardedClusterFixture
+    ShardedClusterFixture, _RouterView
 from buildscripts.resmokelib.testing.fixtures.standalone import MongoDFixture
 from buildscripts.resmokelib.utils import autoloader, default_if_none, pick_catalog_shard_node
 
@@ -24,12 +23,11 @@ MONGO_REPO_LOCATION = "."
 FIXTURE_DIR = "buildscripts/resmokelib/testing/fixtures"
 RETRIEVE_DIR = "build/multiversionfixtures"
 RETRIEVE_LOCK = threading.Lock()
-MULTIVERSION_CLASS_SUFFIX = "_multiversion_class_suffix"
 
 _BUILDERS = {}  # type: ignore
 
 
-def make_fixture(class_name, logger, job_num, *args, **kwargs):
+def make_fixture(class_name, logger, job_num, *args, enable_feature_flags=True, **kwargs):
     """Provide factory function for creating Fixture instances."""
 
     fixturelib = FixtureLib()
@@ -43,13 +41,33 @@ def make_fixture(class_name, logger, job_num, *args, **kwargs):
 
     # Special case MongoDFixture or _MongosFixture for now since we only add one option.
     # If there's more logic, we should add a builder class for them.
-    if class_name in ["MongoDFixture", "_MongoSFixture"]:
+    if class_name in ["MongoDFixture", "_MongoSFixture"] and enable_feature_flags:
         return _FIXTURES[class_name](logger, job_num, fixturelib, *args,
                                      add_feature_flags=bool(config.ENABLED_FEATURE_FLAGS), **kwargs)
 
     return _FIXTURES[class_name](logger, job_num, fixturelib, *args, **kwargs)
 
 
+def make_dummy_fixture(suite_name):
+    """Create a dummy fixture for the given suite.
+
+    This fixture is not meant to be used for testing.
+
+    This fixture should only be used to inspect the test topology for a given suite.
+    """
+
+    # Get info to create the target fixture
+    suite = _suite.Suite(suite_name, _get_suite_config(suite_name))
+    fixture_config = suite.get_executor_config()["fixture"]
+    fixture_class = fixture_config.pop("class")
+
+    # This is a noop. A job logger is expected to have been created in order to create a fixture.
+    _ = logging.loggers.new_job_logger(suite.test_kind, job_num=0)
+
+    fixture_logger = logging.loggers.new_fixture_logger(fixture_class, job_num=0)
+    return make_fixture(fixture_class, fixture_logger, job_num=0, **fixture_config)
+
+
 class FixtureBuilder(ABC, metaclass=registry.make_registry_metaclass(_BUILDERS, type(ABC))):  # pylint: disable=invalid-metaclass
     """
     ABC for fixture builders.
@@ -159,10 +177,12 @@ class ReplSetBuilder(FixtureBuilder):
         :param existing_nodes: the list of mongod fixtures
         :return: configured replica set fixture
         """
+
+        launch_mongot = kwargs.get("launch_mongot")
         self._mutate_kwargs(kwargs)
         mixed_bin_versions, old_bin_version = _extract_multiversion_options(kwargs)
         self._validate_multiversion_options(kwargs, mixed_bin_versions)
-        mongod_classes, mongod_executables, mongod_binary_versions = self._get_mongod_assets(
+        mongod_class, mongod_executables, mongod_binary_versions = self._get_mongod_assets(
             kwargs, mixed_bin_versions, old_bin_version)
 
         replset = _FIXTURES[self.REGISTERED_NAME](logger, job_num, fixturelib, *args, **kwargs)
@@ -180,16 +200,17 @@ class ReplSetBuilder(FixtureBuilder):
             return replset
 
         for node_index in range(replset.num_nodes):
-            node = self._new_mongod(replset, node_index, mongod_executables, mongod_classes,
-                                    mongod_binary_versions[node_index], is_multiversion)
+            node = self._new_mongod(replset, node_index, mongod_executables, mongod_class,
+                                    mongod_binary_versions[node_index], is_multiversion,
+                                    launch_mongot)
             replset.install_mongod(node)
 
         if replset.start_initial_sync_node:
             if not replset.initial_sync_node:
                 replset.initial_sync_node_idx = replset.num_nodes
-                replset.initial_sync_node = self._new_mongod(replset, replset.initial_sync_node_idx,
-                                                             mongod_executables, mongod_classes,
-                                                             BinVersionEnum.NEW, is_multiversion)
+                replset.initial_sync_node = self._new_mongod(
+                    replset, replset.initial_sync_node_idx, mongod_executables, mongod_class,
+                    BinVersionEnum.NEW, is_multiversion, launch_mongot)
 
         return replset
 
@@ -240,19 +261,13 @@ class ReplSetBuilder(FixtureBuilder):
                  and the list of binary versions
         """
         executables = {BinVersionEnum.NEW: kwargs["mongod_executable"]}
-        classes = {BinVersionEnum.NEW: cls.LATEST_MONGOD_CLASS}
+        _class = cls.LATEST_MONGOD_CLASS
 
         # Default to NEW for all bin versions; may be overridden below.
         binary_versions = [BinVersionEnum.NEW for _ in range(kwargs["num_nodes"])]
 
         if mixed_bin_versions is not None:
             from buildscripts.resmokelib import multiversionconstants
-            old_shell_version = {
-                config.MultiversionOptions.LAST_LTS:
-                    multiversionconstants.LAST_LTS_MONGO_BINARY,
-                config.MultiversionOptions.LAST_CONTINUOUS:
-                    multiversionconstants.LAST_CONTINUOUS_MONGO_BINARY,
-            }[old_bin_version]
 
             old_mongod_version = {
                 config.MultiversionOptions.LAST_LTS:
@@ -262,13 +277,9 @@ class ReplSetBuilder(FixtureBuilder):
             }[old_bin_version]
 
             executables[BinVersionEnum.OLD] = old_mongod_version
-            classes[BinVersionEnum.OLD] = f"{cls.LATEST_MONGOD_CLASS}{MULTIVERSION_CLASS_SUFFIX}"
             binary_versions = [x for x in mixed_bin_versions]
 
-            load_version(version_path_suffix=MULTIVERSION_CLASS_SUFFIX,
-                         shell_path=old_shell_version)
-
-        return classes, executables, binary_versions
+        return _class, executables, binary_versions
 
     @staticmethod
     def _get_fcv(is_multiversion: bool, old_bin_version: Optional[str]) -> str:
@@ -293,8 +304,8 @@ class ReplSetBuilder(FixtureBuilder):
 
     @staticmethod
     def _new_mongod(replset: ReplicaSetFixture, replset_node_index: int,
-                    executables: Dict[str, str], classes: Dict[str, str], cur_version: str,
-                    is_multiversion: bool) -> FixtureContainer:
+                    executables: Dict[str, str], _class: str, cur_version: str,
+                    is_multiversion: bool, launch_mongot: bool) -> FixtureContainer:
         """Make a fixture container with configured mongod fixture(s) in it.
 
         In non-multiversion mode only a new mongod fixture will be in the fixture container.
@@ -303,7 +314,7 @@ class ReplSetBuilder(FixtureBuilder):
         :param replset: replica set fixture
         :param replset_node_index: the index of node in replica set
         :param executables: dict with a new and the old (if multiversion) mongod executable names
-        :param classes: dict with a new and the old (if multiversion) mongod fixture names
+        :param _class: str with the mongod fixture name
         :param cur_version: old or new version
         :param is_multiversion: whether we are in multiversion mode
         :return: fixture container with configured mongod fixture(s) in it
@@ -315,10 +326,11 @@ class ReplSetBuilder(FixtureBuilder):
         old_fixture = None
 
         if is_multiversion:
-            old_fixture = make_fixture(classes[BinVersionEnum.OLD], mongod_logger, replset.job_num,
-                                       mongod_executable=executables[BinVersionEnum.OLD],
-                                       mongod_options=mongod_options,
-                                       preserve_dbpath=replset.preserve_dbpath)
+            # We do not run old versions with feature flags enabled
+            old_fixture = make_fixture(
+                _class, mongod_logger, replset.job_num, enable_feature_flags=False,
+                mongod_executable=executables[BinVersionEnum.OLD], mongod_options=mongod_options,
+                preserve_dbpath=replset.preserve_dbpath)
 
             # Assign the same port for old and new fixtures so upgrade/downgrade can be done without
             # changing the replicaset config.
@@ -326,10 +338,15 @@ class ReplSetBuilder(FixtureBuilder):
 
         new_fixture_mongod_options = replset.get_options_for_mongod(replset_node_index)
 
-        new_fixture = make_fixture(classes[BinVersionEnum.NEW], mongod_logger, replset.job_num,
+        # In multiversion, new binaries must add the command line option --upgradeBackCompat.
+        if is_multiversion:
+            new_fixture_mongod_options["upgradeBackCompat"] = ""
+
+        new_fixture = make_fixture(_class, mongod_logger, replset.job_num,
                                    mongod_executable=executables[BinVersionEnum.NEW],
                                    mongod_options=new_fixture_mongod_options,
-                                   preserve_dbpath=replset.preserve_dbpath, port=new_fixture_port)
+                                   preserve_dbpath=replset.preserve_dbpath, port=new_fixture_port,
+                                   launch_mongot=launch_mongot)
 
         return FixtureContainer(new_fixture, old_fixture, cur_version)
 
@@ -343,40 +360,6 @@ def get_package_name(dir_path: str) -> str:
     return dir_path.replace('/', '.').replace("\\", ".")
 
 
-def load_version(version_path_suffix=None, shell_path=None):
-    """Load the last_lts/last_continuous fixtures."""
-    with RETRIEVE_LOCK, registry.suffix(version_path_suffix):
-        # Only one thread needs to retrieve the fixtures.
-        retrieve_dir = os.path.relpath(os.path.join(RETRIEVE_DIR, version_path_suffix))
-        if not os.path.exists(retrieve_dir):
-            try:
-                # Avoid circular import
-                import buildscripts.resmokelib.run.generate_multiversion_exclude_tags as gen_tests
-                commit = gen_tests.get_backports_required_hash_for_shell_version(
-                    mongo_shell_path=shell_path)
-            except FileNotFoundError as err:
-                print("Error running the mongo shell, please ensure it's in your $PATH: ", err)
-                raise
-            retrieve_fixtures(retrieve_dir, commit)
-
-        package_name = get_package_name(retrieve_dir)
-        autoloader.load_all_modules(name=package_name, path=[retrieve_dir])  # type: ignore
-
-
-def retrieve_fixtures(directory, commit):
-    """Populate a directory with the fixture files corresponding to a commit."""
-    repo = Repo(MONGO_REPO_LOCATION)
-    real_commit = repo.commit(commit)
-    tree = real_commit.tree / FIXTURE_DIR
-
-    os.makedirs(directory, exist_ok=True)
-
-    for blob in tree.blobs:
-        output = os.path.join(directory, blob.name)
-        with io.BytesIO(blob.data_stream.read()) as retrieved, open(output, "w") as file:
-            file.write(retrieved.read().decode("utf-8"))
-
-
 class ShardedClusterBuilder(FixtureBuilder):
     """Builder class for sharded cluster fixtures."""
 
@@ -394,32 +377,62 @@ class ShardedClusterBuilder(FixtureBuilder):
         """
         self._mutate_kwargs(kwargs)
         mixed_bin_versions, old_bin_version = _extract_multiversion_options(kwargs)
+        is_multiversion = mixed_bin_versions is not None
+        is_config_shard = kwargs["config_shard"] is not None
         self._validate_multiversion_options(kwargs, mixed_bin_versions)
-        mongos_classes, mongos_executables = self._get_mongos_assets(kwargs, mixed_bin_versions,
-                                                                     old_bin_version)
+        self._validate_embedded_router_mode_options(kwargs, is_config_shard, is_multiversion)
+
+        mongos_class, mongos_executables = self._get_mongos_assets(kwargs, mixed_bin_versions,
+                                                                   old_bin_version)
 
         sharded_cluster = _FIXTURES[self.REGISTERED_NAME](logger, job_num, fixturelib, *args,
                                                           **kwargs)
 
-        is_multiversion = mixed_bin_versions is not None
-
-        for rs_shard_index in range(kwargs["num_shards"]):
-            rs_shard = self._new_rs_shard(sharded_cluster, mixed_bin_versions, old_bin_version,
-                                          rs_shard_index, kwargs["num_rs_nodes_per_shard"])
-            sharded_cluster.install_rs_shard(rs_shard)
-
         config_shard = kwargs["config_shard"]
         config_svr = None
+        # We install the configsvr before the shards, so that embedded-router shards can know the
+        # config-server connection string when they are created. Since config servers do not
+        # currently hold collection data, a mongot enabled shared cluster doesn't couple/launch
+        # the config server with an accompanying mongot
         if config_shard is None:
             config_svr = self._new_configsvr(sharded_cluster, is_multiversion, old_bin_version)
         else:
-            config_svr = sharded_cluster.shards[config_shard]
+            config_svr = self._new_rs_shard(sharded_cluster, mixed_bin_versions, old_bin_version,
+                                            config_shard, kwargs["num_rs_nodes_per_shard"],
+                                            launch_mongot=False)
         sharded_cluster.install_configsvr(config_svr)
 
-        for mongos_index in range(kwargs["num_mongos"]):
-            mongos = self._new_mongos(sharded_cluster, mongos_executables, mongos_classes,
-                                      mongos_index, kwargs["num_mongos"], is_multiversion)
-            sharded_cluster.install_mongos(mongos)
+        # Persist a list of all nodes from the cluster with a boolean that indicates if that node
+        # acts as a config server or not.
+        nodes = [(node, True) for node in config_svr._all_mongo_d_s_t()]
+
+        launch_mongot = kwargs.get("launch_mongot")
+        for rs_shard_index in range(kwargs["num_shards"]):
+            if rs_shard_index != config_shard:
+                rs_shard = self._new_rs_shard(sharded_cluster, mixed_bin_versions, old_bin_version,
+                                              rs_shard_index, kwargs["num_rs_nodes_per_shard"],
+                                              launch_mongot)
+                sharded_cluster.install_rs_shard(rs_shard)
+                # Extend the list of nodes to be sure configsvr nodes are placed at first places.
+                nodes.extend([(node, False) for node in rs_shard._all_mongo_d_s_t()])
+            else:
+                sharded_cluster.install_rs_shard(config_svr)
+
+        num_routers = kwargs["num_mongos"]
+
+        def install_router():
+            if not kwargs.get("embedded_router", None):
+                mongos = self._new_mongos(sharded_cluster, mongos_executables, mongos_class,
+                                          mongos_index, num_routers, is_multiversion)
+                sharded_cluster.install_mongos(mongos)
+            else:
+                node = nodes.pop(0)
+                router_view = self._new_router_view(sharded_cluster, mongos_index, num_routers,
+                                                    node[0], node[1])
+                sharded_cluster.install_mongos(router_view)
+
+        for mongos_index in range(num_routers):
+            install_router()
 
         return sharded_cluster
 
@@ -447,6 +460,11 @@ class ShardedClusterBuilder(FixtureBuilder):
 
         config_shard = pick_catalog_shard_node(
             kwargs.pop("config_shard", config.CONFIG_SHARD), num_shards)
+        # Currently the auto_boostrap_procedure requires us to have a config_shard
+        if "use_auto_bootstrap_procedure" in kwargs and kwargs[
+                "use_auto_bootstrap_procedure"] and not config_shard:
+            config_shard = 0
+        kwargs["embedded_router"] = kwargs.pop("embedded_router", config.EMBEDDED_ROUTER)
         kwargs["config_shard"] = config_shard
 
     @staticmethod
@@ -466,6 +484,35 @@ class ShardedClusterBuilder(FixtureBuilder):
                        " nodes in the sharded cluster: {}.").format(len_versions, num_mongods)
                 raise errors.ServerFailure(msg)
 
+    @staticmethod
+    def _validate_embedded_router_mode_options(kwargs: Dict[str, Any], is_config_shard: bool,
+                                               is_multiversion: bool) -> None:
+        """Raise an exception if the configuration for the sharded cluster can't support embedded_router_mode.
+
+        :param kwargs: sharded cluster fixture kwargs.
+        :param is_multiversion: True if this is a multiversion test.
+        """
+        # TODO SERVER-81458: Support multiversion testing with embedded routers.
+        # TODO SERVER-81459: Support testing a cluster with a combination of dedicated and embedded routers.
+        embedded_router_mode = kwargs.get("embedded_router", None)
+        num_routers = kwargs["num_mongos"]
+        num_shardsvrs = kwargs["num_shards"] * kwargs["num_rs_nodes_per_shard"]
+
+        if embedded_router_mode:
+            # Add the configsvr as a mongos if it is not already counted as a config shard.
+            if not is_config_shard:
+                num_configsvr_nodes = 1
+                if "configsvr_options" in kwargs and "num_nodes" in kwargs["configsvr_options"]:
+                    num_configsvr_nodes = kwargs["configsvr_options"]["num_nodes"]
+                num_routers += num_configsvr_nodes
+
+            if num_routers > num_shardsvrs:
+                raise ValueError(
+                    "When running in embedded router mode, num_mongos must be <= the total number of shardsvrs in the cluster."
+                )
+            if is_multiversion:
+                raise ValueError("Embedded router mode does not support multiversion testing.")
+
     @classmethod
     def _get_mongos_assets(cls, kwargs: Dict[str, Any], mixed_bin_versions: Optional[List[str]],
                            old_bin_version: Optional[str]) -> Tuple[Dict[str, str], Dict[str, str]]:
@@ -477,17 +524,11 @@ class ShardedClusterBuilder(FixtureBuilder):
         :return: tuple with dicts that contain mongos new/old class and executable names
         """
 
-        executables = {BinVersionEnum.NEW: kwargs["mongos_executable"]}
-        classes = {BinVersionEnum.NEW: cls.LATEST_MONGOS_CLASS}
+        executables = {BinVersionEnum.NEW: kwargs.pop("mongos_executable")}
+        _class = cls.LATEST_MONGOS_CLASS
 
         if mixed_bin_versions is not None:
             from buildscripts.resmokelib import multiversionconstants
-            old_shell_version = {
-                config.MultiversionOptions.LAST_LTS:
-                    multiversionconstants.LAST_LTS_MONGO_BINARY,
-                config.MultiversionOptions.LAST_CONTINUOUS:
-                    multiversionconstants.LAST_CONTINUOUS_MONGO_BINARY,
-            }[old_bin_version]
 
             old_mongos_version = {
                 config.MultiversionOptions.LAST_LTS:
@@ -497,12 +538,7 @@ class ShardedClusterBuilder(FixtureBuilder):
             }[old_bin_version]
 
             executables[BinVersionEnum.OLD] = old_mongos_version
-            classes[BinVersionEnum.OLD] = f"{cls.LATEST_MONGOS_CLASS}{MULTIVERSION_CLASS_SUFFIX}"
-
-            load_version(version_path_suffix=MULTIVERSION_CLASS_SUFFIX,
-                         shell_path=old_shell_version)
-
-        return classes, executables
+        return _class, executables
 
     @staticmethod
     def _new_configsvr(sharded_cluster: ShardedClusterFixture, is_multiversion: bool,
@@ -531,7 +567,8 @@ class ShardedClusterBuilder(FixtureBuilder):
     @staticmethod
     def _new_rs_shard(sharded_cluster: ShardedClusterFixture,
                       mixed_bin_versions: Optional[List[str]], old_bin_version: Optional[str],
-                      rs_shard_index: int, num_rs_nodes_per_shard: int) -> ReplicaSetFixture:
+                      rs_shard_index: int, num_rs_nodes_per_shard: int,
+                      launch_mongot: bool) -> ReplicaSetFixture:
         """Return a replica set fixture configured as a shard in a sharded cluster.
 
         :param sharded_cluster: sharded cluster fixture we are configuring config server for
@@ -544,6 +581,7 @@ class ShardedClusterBuilder(FixtureBuilder):
 
         rs_shard_logger = sharded_cluster.get_rs_shard_logger(rs_shard_index)
         rs_shard_kwargs = sharded_cluster.get_rs_shard_kwargs(rs_shard_index)
+        rs_shard_kwargs["launch_mongot"] = launch_mongot
 
         if mixed_bin_versions is not None:
             start_index = rs_shard_index * num_rs_nodes_per_shard
@@ -556,7 +594,7 @@ class ShardedClusterBuilder(FixtureBuilder):
 
     @staticmethod
     def _new_mongos(sharded_cluster: ShardedClusterFixture, executables: Dict[str, str],
-                    classes: Dict[str, str], mongos_index: int, total: int,
+                    _class: str, mongos_index: int, total: int,
                     is_multiversion: bool) -> FixtureContainer:
         """Make a fixture container with configured mongos fixture(s) in it.
 
@@ -565,7 +603,7 @@ class ShardedClusterBuilder(FixtureBuilder):
 
         :param sharded_cluster: sharded cluster fixture we are configuring mongos for
         :param executables: dict with a new and the old (if multiversion) mongos executable names
-        :param classes: dict with a new and the old (if multiversion) mongos fixture names
+        :param _class: str with the mongos fixture name
         :param mongos_index: the index of mongos
         :param total: total number of mongos
         :param is_multiversion: whether we are in multiversion mode
@@ -578,16 +616,35 @@ class ShardedClusterBuilder(FixtureBuilder):
         old_fixture = None
 
         if is_multiversion:
+            # We do not run old versions with feature flags enabled
             old_fixture = make_fixture(
-                classes[BinVersionEnum.OLD], mongos_logger, sharded_cluster.job_num,
+                _class, mongos_logger, sharded_cluster.job_num, enable_feature_flags=False,
                 mongos_executable=executables[BinVersionEnum.OLD], **mongos_kwargs)
 
         # We can't restart mongos since explicit ports are not supported.
         new_fixture_mongos_kwargs = sharded_cluster.get_mongos_kwargs()
-        new_fixture = make_fixture(
-            classes[BinVersionEnum.NEW], mongos_logger, sharded_cluster.job_num,
-            mongos_executable=executables[BinVersionEnum.NEW], **new_fixture_mongos_kwargs)
+
+        # In multiversion, new binaries must add the command line option --upgradeBackCompat.
+        if is_multiversion:
+            new_fixture_mongos_kwargs["mongos_options"]["upgradeBackCompat"] = ""
+
+        new_fixture = make_fixture(_class, mongos_logger, sharded_cluster.job_num,
+                                   mongos_executable=executables[BinVersionEnum.NEW],
+                                   **new_fixture_mongos_kwargs)
 
         # Always spin up an old mongos if in multiversion mode given mongos is the last thing in the update path.
         return FixtureContainer(new_fixture, old_fixture,
                                 BinVersionEnum.OLD if is_multiversion else BinVersionEnum.NEW)
+
+    @staticmethod
+    def _new_router_view(sharded_cluster: ShardedClusterFixture, mongos_index: int, total: int,
+                         mongod: MongoDFixture, is_configsvr: bool) -> _RouterView:
+        """Make a fixture that allows ShardedClusterFixture to treat a shardsvr as a router."""
+
+        router_logger = sharded_cluster.get_mongos_logger(mongos_index, total)
+        router_kwargs = {}
+        router_kwargs["mongod"] = mongod
+
+        fix = make_fixture("_RouterView", router_logger, sharded_cluster.job_num, is_configsvr,
+                           **router_kwargs)
+        return fix
