@@ -9,7 +9,7 @@ import pymongo
 import pymongo.errors
 import pymongo.write_concern
 
-import buildscripts.resmokelib.testing.fixtures.interface as interface
+from buildscripts.resmokelib.testing.fixtures import interface
 
 
 def compare_timestamp(timestamp1, timestamp2):
@@ -37,17 +37,21 @@ def compare_optime(optime1, optime2):
         return compare_timestamp(optime1["ts"], optime2["ts"])
 
 
-class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-instance-attributes, too-many-public-methods
+class ReplicaSetFixture(interface.ReplFixture, interface._DockerComposeInterface):
     """Fixture which provides JSTests with a replica set to run against."""
 
-    def __init__(  # pylint: disable=too-many-arguments, too-many-locals
+    AWAIT_SHARDING_INITIALIZATION_TIMEOUT_SECS = 60
+
+    def __init__(
             self, logger, job_num, fixturelib, mongod_executable=None, mongod_options=None,
             dbpath_prefix=None, preserve_dbpath=False, num_nodes=2, start_initial_sync_node=False,
-            write_concern_majority_journal_default=None, auth_options=None,
-            replset_config_options=None, voting_secondaries=True, all_nodes_electable=False,
-            use_replica_set_connection_string=None, linear_chain=False, default_read_concern=None,
-            default_write_concern=None, shard_logging_prefix=None, replicaset_logging_prefix=None,
-            replset_name=None):
+            electable_initial_sync_node=False, write_concern_majority_journal_default=None,
+            auth_options=None, replset_config_options=None, voting_secondaries=True,
+            all_nodes_electable=False, use_replica_set_connection_string=None, linear_chain=False,
+            default_read_concern=None, default_write_concern=None, shard_logging_prefix=None,
+            replicaset_logging_prefix=None, replset_name=None, use_auto_bootstrap_procedure=None,
+            initial_sync_uninitialized_fcv=False, hide_initial_sync_node_from_conn_string=False,
+            launch_mongot=False, initial_sync_uninitialized_fcv_in_shard_svr=False):
         """Initialize ReplicaSetFixture."""
 
         interface.ReplFixture.__init__(self, logger, job_num, fixturelib,
@@ -58,6 +62,7 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
             self.fixturelib.default_if_none(mongod_options, {}))
         self.preserve_dbpath = preserve_dbpath
         self.start_initial_sync_node = start_initial_sync_node
+        self.electable_initial_sync_node = electable_initial_sync_node
         self.write_concern_majority_journal_default = write_concern_majority_journal_default
         self.auth_options = auth_options
         self.replset_config_options = self.fixturelib.make_historic(
@@ -71,14 +76,19 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
         self.replicaset_logging_prefix = replicaset_logging_prefix
         self.num_nodes = num_nodes
         self.replset_name = replset_name
+        self.initial_sync_uninitialized_fcv = initial_sync_uninitialized_fcv
+        self.hide_initial_sync_node_from_conn_string = hide_initial_sync_node_from_conn_string
+        self.initial_sync_uninitialized_fcv_in_shard_svr = initial_sync_uninitialized_fcv_in_shard_svr
         # Used by the enhanced multiversion system to signify multiversion mode.
         # None implies no multiversion run.
         self.fcv = None
-
+        # Used by suites that run search integration tests.
+        self.launch_mongot = launch_mongot
         # Use the values given from the command line if they exist for linear_chain and num_nodes.
         linear_chain_option = self.fixturelib.default_if_none(self.config.LINEAR_CHAIN,
                                                               linear_chain)
         self.linear_chain = linear_chain_option if linear_chain_option else linear_chain
+        self.repl_set_config = {}
 
         # By default, we only use a replica set connection string if all nodes are capable of being
         # elected primary.
@@ -106,26 +116,63 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
 
         self.nodes = []
         if "serverless" not in self.mongod_options:
-            self.replset_name = self.mongod_options.setdefault("replSet", "rs")
+            if not self.replset_name:
+                self.replset_name = "rs"
+            self.replset_name = self.mongod_options.setdefault("replSet", self.replset_name)
         self.initial_sync_node = None
         self.initial_sync_node_idx = -1
+        self.use_auto_bootstrap_procedure = use_auto_bootstrap_procedure
+        # This will be set in setup() after the MongoTFixture has been launched.
+        self.mongot_port = None
+        # Track the fixture removal [teardown] performed during removeShard testing.
+        # This is needed, because we expect the fixture to be in the 'running' state
+        # when the evergeen job performs the final teardown. Therefore if the fixture was
+        # teared down earlier, it must be skipped during those final checks.
+        self.removeshard_teardown_marker = False
 
-    def setup(self):  # pylint: disable=too-many-branches,too-many-statements,too-many-locals
+    def setup(self):
         """Set up the replica set."""
+        start_node = 0
+        if self.use_auto_bootstrap_procedure:
+            # We need to wait for the first node to finish auto-bootstrapping so that we can
+            # get the auto generated replSet name and update the replSet name of the other mongods with it.
+            self.nodes[0].setup()
+            self.nodes[0].await_ready()
+            self._await_primary(
+            )  # Wait for writeable primary (this indicates replSet auto-intiiate finished).
+
+            client = interface.build_client(self.nodes[0], self.auth_options)
+            res = client.admin.command("hello")
+
+            self.logger.info(
+                f"ReplicaSetFixture using auto generated replSet name {res['setName']} instead of {self.replset_name}"
+            )
+
+            self.replset_name = res["setName"]
+            self.mongod_options.setdefault("replSet", self.replset_name)
+            # The first node should not have the --replSet option because it is auto-bootstrapped.
+            for i in range(1, self.num_nodes):
+                self.nodes[i].mongod_options["replSet"] = self.replset_name
+
+            start_node = 1
 
         # Version-agnostic options for mongod/s can be set here.
         # Version-specific options should be set in get_version_specific_options_for_mongod()
         # to avoid options for old versions being applied to new Replicaset fixtures.
-        for i in range(self.num_nodes):
+        # If we are using the auto_bootstrap_procedure we don't need to setup the first node again.
+        for i in range(start_node, self.num_nodes):
             self.nodes[i].setup()
 
         if self.initial_sync_node:
             self.initial_sync_node.setup()
             self.initial_sync_node.await_ready()
+            if self.initial_sync_uninitialized_fcv:
+                self._pause_initial_sync_at_uninitialized_fcv(self.initial_sync_node)
 
-        # We need only to wait to connect to the first node of the replica set because we first
-        # initiate it as a single node replica set.
-        self.nodes[0].await_ready()
+        if not self.use_auto_bootstrap_procedure:
+            # We need only to wait to connect to the first node of the replica set because we first
+            # initiate it as a single node replica set.
+            self.nodes[0].await_ready()
 
         # Initiate the replica set.
         members = []
@@ -139,19 +186,23 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
                     # non-voting when this fixture is configured to have voting secondaries.
                     member_info["votes"] = 0
             members.append(member_info)
-        if self.initial_sync_node:
-            members.append({
-                "_id": self.initial_sync_node_idx,
-                "host": self.initial_sync_node.get_internal_connection_string(), "priority": 0,
-                "hidden": 1, "votes": 0
-            })
+        # If this is an initial sync node in a shard server that will be hung in the uninitialized
+        # FCV state, don't add it to the replica set yet. It will be added to the set later after
+        # the shard is added to the cluster, so that if the shard is a config shard, it can transition
+        # to being a config shard properly.
+        if self.initial_sync_node and not self.initial_sync_uninitialized_fcv_in_shard_svr:
+            initial_sync_config = self._create_initial_sync_config()
+            members.append(initial_sync_config)
 
         repl_config = {"_id": self.replset_name, "protocolVersion": 1}
-        client = self.nodes[0].mongo_client()
-        interface.authenticate(client, self.auth_options)
+        client = interface.build_client(self.nodes[0], self.auth_options)
 
-        if client.local.system.replset.count_documents(filter={}):
+        if client.local.system.replset.count_documents(
+                filter={}) and not self.use_auto_bootstrap_procedure:
             # Skip initializing the replset if there is an existing configuration.
+            # Auto-bootstrapping will automatically create a configuration document but we do not
+            # want to skip reconfiguring the replset (which adds the other nodes
+            # to the auto-bootstrapped replset).
             self.logger.info("Configuration exists. Skipping initializing the replset.")
             return
 
@@ -160,12 +211,11 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
                 "writeConcernMajorityJournalDefault"] = self.write_concern_majority_journal_default
         else:
             server_status = client.admin.command({"serverStatus": 1})
-            cmd_line_opts = client.admin.command({"getCmdLineOpts": 1})
-            if not (server_status["storageEngine"]["persistent"] and cmd_line_opts["parsed"].get(
-                    "storage", {}).get("journal", {}).get("enabled", True)):
+            if not server_status["storageEngine"]["persistent"]:
                 repl_config["writeConcernMajorityJournalDefault"] = False
 
-        if self.replset_config_options.get("configsvr", False):
+        if (self.replset_config_options.get("configsvr", False)
+                or (self.use_auto_bootstrap_procedure and not "shardsvr" in self.mongod_options)):
             repl_config["configsvr"] = True
         if self.replset_config_options.get("settings"):
             replset_settings = self.replset_config_options["settings"]
@@ -179,9 +229,22 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
         # Start up a single node replica set then reconfigure to the correct size (if the config
         # contains more than 1 node), so the primary is elected more quickly.
         repl_config["members"] = [members[0]]
-        self.logger.info("Issuing replSetInitiate command: %s", repl_config)
-        self._initiate_repl_set(client, repl_config)
-        self._await_primary()
+
+        # When this is True, we are running in Antithesis & modify the config to surface more bugs
+        if self.config.NOOP_MONGO_D_S_PROCESSES:
+            repl_config["settings"]["electionTimeoutMillis"] = 2000
+            repl_config["settings"]["chainingAllowed"] = False
+            repl_config["settings"]["heartbeatTimeoutSecs"] = 1
+            repl_config["settings"]["catchUpTimeoutMillis"] = 0
+
+        if self.use_auto_bootstrap_procedure:
+            # Auto-bootstrap already initiates automatically on the first node, but we still need
+            # to apply the requested repl_config settings using reconfig.
+            self._reconfig_repl_set(client, repl_config)
+        else:
+            self.logger.info("Issuing replSetInitiate command: %s", repl_config)
+            self._initiate_repl_set(client, repl_config)
+            self._await_primary()
 
         if self.fcv is not None:
             # Initiating a replica set with a single node will use "latest" FCV. This will
@@ -207,6 +270,58 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
         self._await_secondaries()
         self._await_newly_added_removals()
 
+        if self.launch_mongot:
+            # To model Atlas Search's coupled architecture, resmoke deploys a mongot for each
+            # mongod node in a replica set.
+            for node in self.nodes:
+                node.setup_mongot()
+            # Saving the mongot port to the ReplicaSetFixture allows the ShardedClusterFixture
+            # to spin up a mongos with a connection to the last launched mongot.
+            self.mongot_port = node.mongot_port
+
+        self.removeshard_teardown_marker = False
+
+    def _create_initial_sync_config(self):
+        """Helper function to create and return the config for an initial sync node."""
+
+        initial_sync_config = {
+            "_id": self.initial_sync_node_idx,
+            "host": self.initial_sync_node.get_internal_connection_string(),
+        }
+        if not self.electable_initial_sync_node:
+            initial_sync_config["hidden"] = 1
+            initial_sync_config["votes"] = 0
+            initial_sync_config["priority"] = 0
+        return initial_sync_config
+
+    def add_initial_sync_node_to_replica_set(self):
+        """Adds the initial sync node to the replica set.
+        
+        This is used so that we can add in the initial sync node after the setup() function at a 
+        later time.
+        """
+
+        self.logger.info("Adding in initial sync node to replica set")
+        initial_sync_config = self._create_initial_sync_config()
+        self.initial_sync_node.await_ready()
+
+        client = interface.build_client(self.nodes[0], self.auth_options)
+        self.repl_set_config["members"].append(initial_sync_config)
+        self._reconfig_repl_set(client, self.repl_set_config)
+
+    def _all_mongo_d_s_t(self):
+        """Return a list of all `mongo{d,s,t}` `Process` instances in this fixture."""
+        nodes = sum([node._all_mongo_d_s_t() for node in self.nodes], [])
+
+        if self.initial_sync_node:
+            nodes.extend(self.initial_sync_node._all_mongo_d_s_t())
+
+        return nodes
+
+    def _all_mongots(self):
+        """Return a list of all `mongot` `Process` instances in this fixture."""
+        return [node.mongot for node in self.nodes]
+
     def pids(self):
         """:return: all pids owned by this fixture if any."""
         pids = []
@@ -218,30 +333,37 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
 
     def _add_node_to_repl_set(self, client, repl_config, member_index, members):
         self.logger.info("Adding in node %d: %s", member_index, members[member_index - 1])
+        repl_config["members"] = members[:member_index]
+        self._reconfig_repl_set(client, repl_config)
+
+    def _reconfig_repl_set(self, client, repl_config):
         while True:
             try:
                 # 'newlyAdded' removal reconfigs could bump the version.
                 # Get the current version to be safe.
                 curr_version = client.admin.command({"replSetGetConfig": 1})['config']['version']
                 repl_config["version"] = curr_version + 1
-                repl_config["members"] = members[:member_index]
+
                 self.logger.info("Issuing replSetReconfig command: %s", repl_config)
                 client.admin.command({
                     "replSetReconfig": repl_config,
                     "maxTimeMS": self.AWAIT_REPL_TIMEOUT_MINS * 60 * 1000
                 })
+                self.repl_set_config = repl_config
                 break
             except pymongo.errors.OperationFailure as err:
                 # These error codes may be transient, and so we retry the reconfig with a
                 # (potentially) higher config version. We should not receive these codes
                 # indefinitely.
                 # pylint: disable=too-many-boolean-expressions
-                if (err.code != ReplicaSetFixture._NEW_REPLICA_SET_CONFIGURATION_INCOMPATIBLE
-                        and err.code != ReplicaSetFixture._CURRENT_CONFIG_NOT_COMMITTED_YET
-                        and err.code != ReplicaSetFixture._CONFIGURATION_IN_PROGRESS
-                        and err.code != ReplicaSetFixture._NODE_NOT_FOUND
-                        and err.code != ReplicaSetFixture._INTERRUPTED_DUE_TO_REPL_STATE_CHANGE
-                        and err.code != ReplicaSetFixture._INTERRUPTED_DUE_TO_STORAGE_CHANGE):
+                if err.code not in [
+                        ReplicaSetFixture._NEW_REPLICA_SET_CONFIGURATION_INCOMPATIBLE,
+                        ReplicaSetFixture._CURRENT_CONFIG_NOT_COMMITTED_YET,
+                        ReplicaSetFixture._CONFIGURATION_IN_PROGRESS,
+                        ReplicaSetFixture._NODE_NOT_FOUND,
+                        ReplicaSetFixture._INTERRUPTED_DUE_TO_REPL_STATE_CHANGE,
+                        ReplicaSetFixture._INTERRUPTED_DUE_TO_STORAGE_CHANGE
+                ]:
                     msg = ("Operation failure while setting up the "
                            "replica set fixture: {}").format(err)
                     self.logger.error(msg)
@@ -259,6 +381,7 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
         for attempt in range(1, num_initiate_attempts + 1):
             try:
                 client.admin.command({"replSetInitiate": repl_config})
+                self.repl_set_config = repl_config
                 break
             except pymongo.errors.OperationFailure as err:
                 # Retry on NodeNotFound errors from the "replSetInitiate" command.
@@ -279,8 +402,7 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
 
     def await_last_op_committed(self, timeout_secs=None):
         """Wait for the last majority committed op to be visible."""
-        primary_client = self.get_primary().mongo_client()
-        interface.authenticate(primary_client, self.auth_options)
+        primary_client = interface.build_client(self.get_primary(), self.auth_options)
 
         primary_optime = get_last_optime(primary_client, self.fixturelib)
         up_to_date_nodes = set()
@@ -288,10 +410,14 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
         def check_rcmaj_optime(client, node):
             """Return True if all nodes have caught up with the primary."""
             res = client.admin.command({"replSetGetStatus": 1})
+
+            if "readConcernMajorityOpTime" not in res["optimes"]:
+                # This can be null when the node is starting up.
+                return False
+
             read_concern_majority_optime = res["optimes"]["readConcernMajorityOpTime"]
 
-            if (read_concern_majority_optime["t"] == primary_optime["t"]
-                    and read_concern_majority_optime["ts"] >= primary_optime["ts"]):
+            if compare_optime(read_concern_majority_optime, primary_optime) >= 0:
                 up_to_date_nodes.add(node.port)
 
             return len(up_to_date_nodes) == len(self.nodes)
@@ -305,6 +431,9 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
         self._await_secondaries()
         self._await_stable_recovery_timestamp()
         self._setup_cwrwc_defaults()
+        if self.use_auto_bootstrap_procedure:
+            # TODO: Remove this in SERVER-80010.
+            self._await_auto_bootstrapped_config_shard()
 
     def _await_primary(self):
         # Wait for the primary to be elected.
@@ -325,7 +454,7 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
         # Since this method is called at startup we expect the nodes 1 to n to be secondaries even
         # when self.all_nodes_electable is True.
         secondaries = self.nodes[1:]
-        if self.initial_sync_node:
+        if self.initial_sync_node and not self.initial_sync_uninitialized_fcv:
             secondaries.append(self.initial_sync_node)
 
         for secondary in secondaries:
@@ -340,6 +469,8 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
                 except pymongo.errors.OperationFailure as err:
                     if err.code != ReplicaSetFixture._INTERRUPTED_DUE_TO_STORAGE_CHANGE:
                         raise
+                except pymongo.errors.AutoReconnect:
+                    self.logger.info("AutoReconnect exception thrown, retrying...")
                 time.sleep(0.1)  # Wait a little bit before trying again.
             self.logger.info("Secondary on port %d is now available.", secondary.port)
 
@@ -359,8 +490,7 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
 
         # Since this method is called at startup we expect the first node to be primary even when
         # self.all_nodes_electable is True.
-        primary_client = self.nodes[0].mongo_client()
-        interface.authenticate(primary_client, self.auth_options)
+        primary_client = interface.build_client(self.nodes[0], self.auth_options)
 
         # All nodes must be in primary/secondary state prior to this point. Perform a majority
         # write to ensure there is a committed operation on the set. The commit point will
@@ -373,8 +503,8 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
         for node in self.nodes:
             self.logger.info("Waiting for node on port %d to have a stable recovery timestamp.",
                              node.port)
-            client = node.mongo_client(read_preference=pymongo.ReadPreference.SECONDARY)
-            interface.authenticate(client, self.auth_options)
+            client = interface.build_client(node, self.auth_options,
+                                            read_preference=pymongo.ReadPreference.SECONDARY)
 
             client_admin = client["admin"]
 
@@ -431,8 +561,7 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
 
         self.logger.info("Waiting to remove all 'newlyAdded' fields")
         primary = self.get_primary()
-        client = primary.mongo_client()
-        interface.authenticate(client, self.auth_options)
+        client = interface.build_client(primary, self.auth_options)
         while self._should_await_newly_added_removals_longer(client):
             time.sleep(0.1)  # Wait a little bit before trying again.
         self.logger.info("All 'newlyAdded' fields removed")
@@ -449,8 +578,81 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
         primary = self.nodes[0]
         primary.mongo_client().admin.command(cmd)
 
+    # TODO: Remove this in SERVER-80010.
+    def _await_auto_bootstrapped_config_shard(self):
+        connection_string = self.get_driver_connection_url()
+        self.logger.info("Waiting for %s to auto-bootstrap as a config shard...", connection_string)
+
+        deadline = time.time() + ReplicaSetFixture.AWAIT_SHARDING_INITIALIZATION_TIMEOUT_SECS
+        timeout_occurred = lambda: deadline - time.time() <= 0.0
+
+        while True:
+            client = interface.build_client(self.get_primary(), self.auth_options)
+            config_shard_count = client.get_database("config").command(
+                {"count": "shards", "query": {"_id": "config"}})
+
+            if config_shard_count['n'] == 1:
+                break
+
+            if timeout_occurred():
+                port = self.get_primary().port
+                raise self.fixturelib.ServerFailure(
+                    "mongod on port: {} failed waiting for auto-bootstrapped config shard success after {} seconds"
+                    .format(port, interface.Fixture.AWAIT_READY_TIMEOUT_SECS))
+            time.sleep(0.1)
+
+        self.logger.info("%s successfully auto-bootstrapped as a config shard...",
+                         connection_string)
+
+    def _check_initial_sync_node_has_uninitialized_fcv(self, initial_sync_node):
+        sync_node_conn = initial_sync_node.mongo_client()
+        self.logger.info("Checking that initial sync node has uninitialized fcv")
+        try:
+            fcv = sync_node_conn.admin.command(
+                {'getParameter': 1, 'featureCompatibilityVersion': 1})
+
+            msg = "Initial sync node should have an uninitialized FCV, but got fcv: " + str(fcv)
+            raise self.fixturelib.ServerFailure(msg)
+        except pymongo.errors.OperationFailure as err:
+            if err.code == 258:  #codeName == 'UnknownFeatureCompatibilityVersion'
+                return
+            raise
+
+    def _pause_initial_sync_at_uninitialized_fcv(self, initial_sync_node):
+        failpointOnCmd = {
+            'configureFailPoint': 'initialSyncHangAfterResettingFCV', 'mode': 'alwaysOn'
+        }
+        sync_node_conn = initial_sync_node.mongo_client()
+        self.logger.info("Pausing initial sync at failpoint")
+        sync_node_conn.admin.command(failpointOnCmd)
+        self._check_initial_sync_node_has_uninitialized_fcv(initial_sync_node)
+
+    def _unpause_and_finish_initial_sync(self, initial_sync_node):
+        failpoint_off_cmd = {
+            'configureFailPoint': 'initialSyncHangAfterResettingFCV', 'mode': 'off'
+        }
+        self.logger.info("Unpausing initial sync")
+        sync_node_conn = initial_sync_node.mongo_client()
+        sync_node_conn.admin.command(failpoint_off_cmd)
+
+        wait_for_initial_sync_finish_cmd = bson.SON(
+            [("replSetTest", 1), ("waitForMemberState", 2),
+             ("timeoutMillis", interface.ReplFixture.AWAIT_REPL_TIMEOUT_FOREVER_MINS * 60 * 1000)])
+        while True:
+            try:
+                self.logger.info("Waiting for initial sync to finish")
+                sync_node_conn.admin.command(wait_for_initial_sync_finish_cmd)
+                break
+            except pymongo.errors.OperationFailure as err:
+                if err.code not in (self.INTERRUPTED_DUE_TO_REPL_STATE_CHANGE,
+                                    self.INTERRUPTED_DUE_TO_STORAGE_CHANGE):
+                    raise
+                msg = ("Interrupted while waiting for node to reach secondary state, retrying: {}"
+                       ).format(err)
+                self.logger.error(msg)
+
     def _do_teardown(self, mode=None):
-        self.logger.info("Stopping all members of the replica set...")
+        self.logger.info("Stopping all members of the replica set '%s'...", self.replset_name)
 
         running_at_start = self.is_running()
         if not running_at_start:
@@ -460,6 +662,9 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
         teardown_handler = interface.FixtureTeardownHandler(self.logger)
 
         if self.initial_sync_node:
+            if self.initial_sync_uninitialized_fcv:
+                self._check_initial_sync_node_has_uninitialized_fcv(self.initial_sync_node)
+                self._unpause_and_finish_initial_sync(self.initial_sync_node)
             teardown_handler.teardown(self.initial_sync_node, "initial sync node", mode=mode)
 
         # Terminate the secondaries first to reduce noise in the logs.
@@ -477,11 +682,11 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
         running = all(node.is_running() for node in self.nodes)
 
         if self.initial_sync_node:
-            running = self.initial_sync_node.is_running() or running
+            running = self.initial_sync_node.is_running() and running
 
         return running
 
-    def get_primary(self, timeout_secs=30):  # pylint: disable=arguments-differ
+    def get_primary(self, timeout_secs=30):
         """Return the primary from a replica set."""
         if not self.all_nodes_electable:
             # The primary is always the first element of the 'nodes' list because all other members
@@ -512,8 +717,13 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
             timeout_secs = self.AWAIT_REPL_TIMEOUT_MINS * 60
         start = time.time()
         clients = {}
+
+        all_nodes = self.nodes.copy()
+        if self.electable_initial_sync_node:
+            all_nodes.append(self.initial_sync_node)
+
         while True:
-            for node in self.nodes:
+            for node in all_nodes:
                 now = time.time()
                 if (now - start) >= timeout_secs:
                     msg = "Timed out while {} for replica set '{}'.".format(msg, self.replset_name)
@@ -522,8 +732,7 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
 
                 try:
                     if node.port not in clients:
-                        clients[node.port] = interface.authenticate(node.mongo_client(),
-                                                                    self.auth_options)
+                        clients[node.port] = interface.build_client(node, self.auth_options)
 
                     if fn(clients[node.port], node):
                         return node
@@ -539,9 +748,9 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
         # Check that the fixture is still running before stepping down or killing the primary.
         # This ensures we still detect some cases in which the fixture has already crashed.
         if not self.is_running():
-            raise pymongo.errors.ServerFailure("ReplicaSetFixture {} expected to be running in"
-                                               " ContinuousStepdown, but wasn't.".format(
-                                                   self.replset_name))
+            raise self.fixturelib.ServerFailure("ReplicaSetFixture {} expected to be running in"
+                                                " ContinuousStepdown, but wasn't.".format(
+                                                    self.replset_name))
 
         # If we're running with background reconfigs, it's possible to be in a scenario
         # where we kill a necessary voting node (i.e. in a 5 node repl set), only 2 are
@@ -604,7 +813,7 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
 
                 return self.nodes[chosen_index]
 
-        primary_client = interface.authenticate(primary.mongo_client(), auth_options)
+        primary_client = interface.build_client(primary, auth_options)
         retry_time_secs = self.AWAIT_REPL_TIMEOUT_MINS * 60
         retry_start_time = time.time()
 
@@ -633,7 +842,7 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
                 break
 
             if time.time() - retry_start_time > retry_time_secs:
-                raise pymongo.errors.ServerFailure(
+                raise self.fixturelib.ServerFailure(
                     "The old primary on port {} of replica set {} did not step up in"
                     " {} seconds.".format(chosen_node.port, self.replset_name, retry_time_secs))
 
@@ -645,7 +854,7 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
             self.logger.info(
                 "Attempting to step up the chosen secondary on port %d of replica set '%s'.",
                 node.port, self.replset_name)
-            client = interface.authenticate(node.mongo_client(), auth_options)
+            client = interface.build_client(node, auth_options)
             client.admin.command("replSetStepUp")
             return True
         except pymongo.errors.OperationFailure:
@@ -666,7 +875,17 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
         self.logger.info("Waiting for the old primary on port %d of replica set '%s' to exit.",
                          chosen.port, self.replset_name)
 
-        chosen.mongod.wait()
+        exit_code = chosen.mongod.wait()
+        # This function is called after stop_primary() which could kill or cleanly shutdown the
+        # process. We therefore also allow an exit code of -9.
+        if exit_code in (0, -interface.TeardownMode.KILL.value):
+            self.logger.info("Successfully stopped the mongod on port {:d}.".format(chosen.port))
+        else:
+            self.logger.warning("Stopped the mongod on port {:d}. "
+                                "Process exited with code {:d}.".format(chosen.port, exit_code))
+            raise self.fixturelib.ServerFailure(
+                "mongod on port {:d} with pid {:d} exited with code {:d}".format(
+                    chosen.port, chosen.mongod.pid, exit_code))
 
         self.logger.info("Attempting to restart the old primary on port %d of replica set '%s'.",
                          chosen.port, self.replset_name)
@@ -723,6 +942,9 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
                                                               self.fixturelib.make_historic(
                                                                   {})).copy()
 
+        if index == 0 and self.use_auto_bootstrap_procedure:
+            del mongod_options["replSet"]
+
         if self.linear_chain and index > 0:
             self.mongod_options["set_parameters"][
                 "failpoint.forceSyncSourceCandidate"] = self.fixturelib.make_historic({
@@ -761,7 +983,14 @@ class ReplicaSetFixture(interface.ReplFixture):  # pylint: disable=too-many-inst
     def get_internal_connection_string(self):
         """Return the internal connection string."""
         conn_strs = [node.get_internal_connection_string() for node in self.nodes]
-        if self.initial_sync_node:
+
+        # ReplicaSetFixture sets initial sync nodes as hidden,
+        # which causes a mismatch if the replica set is added to the sharded cluster
+        # through addShard, because the replica set's internal connection string normally
+        # does include the initial sync node, but the list of hosts in the replica set from
+        # running `hello`/`isMaster` does not include it. Setting hide_initial_sync_node_from_conn_string
+        # to True force-hides it from the connection string.
+        if self.initial_sync_node and not self.hide_initial_sync_node_from_conn_string:
             conn_strs.append(self.initial_sync_node.get_internal_connection_string())
         return self.replset_name + "/" + ",".join(conn_strs)
 
