@@ -12,6 +12,8 @@ from buildscripts.resmokelib.testing.fixtures import replicaset
 from buildscripts.resmokelib.testing.hooks import dbhash
 from buildscripts.resmokelib.testing.hooks import interface
 from buildscripts.resmokelib.testing.hooks import oplog
+from buildscripts.resmokelib.testing.hooks import preimages_consistency
+from buildscripts.resmokelib.testing.hooks import change_collection_consistency
 from buildscripts.resmokelib.testing.hooks import validate
 
 
@@ -126,8 +128,7 @@ class PeriodicKillSecondariesTestCase(interface.DynamicTestCase):
     INTERRUPTED_DUE_TO_REPL_STATE_CHANGE = 11602
     INTERRUPTED_DUE_TO_STORAGE_CHANGE = 355
 
-    def __init__(  # pylint: disable=too-many-arguments
-            self, logger, test_name, description, base_test_name, hook, test_report):
+    def __init__(self, logger, test_name, description, base_test_name, hook, test_report):
         """Initialize PeriodicKillSecondariesTestCase."""
         interface.DynamicTestCase.__init__(self, logger, test_name, description, base_test_name,
                                            hook)
@@ -142,6 +143,14 @@ class PeriodicKillSecondariesTestCase(interface.DynamicTestCase):
         # the secondaries.
         self._check_repl_oplog(self._test_report)
 
+        # The CheckReplPreImagesConsistency hook checks that config.system.preimages is compatible
+        # across nodes and that there are no holes in the collection.
+        self._check_pre_images_consistency(self._test_report)
+
+        # The CheckReplChangeCollectionConsistency hook checks that config.system.change_collection
+        # for each tenant is compatible across nodes and that there are no holes in the collection.
+        self._check_change_collection_consistency(self._test_report)
+
         # The CheckReplDBHash hook waits until all operations have replicated to and have been
         # applied on the secondaries, so we run the ValidateCollections hook after it to ensure
         # we're validating the entire contents of the collection.
@@ -160,7 +169,7 @@ class PeriodicKillSecondariesTestCase(interface.DynamicTestCase):
         for secondary in self.fixture.get_secondaries():
             # Disable the "rsSyncApplyStop" failpoint on the secondary to have it resume applying
             # oplog entries.
-            self._hook._disable_rssyncapplystop(secondary)  # pylint: disable=protected-access
+            self._hook._disable_rssyncapplystop(secondary)
 
             # Wait a little bit for the secondary to start apply oplog entries so that we are more
             # likely to kill the mongod process while it is partway into applying a batch.
@@ -191,6 +200,18 @@ class PeriodicKillSecondariesTestCase(interface.DynamicTestCase):
 
         for secondary in self.fixture.get_secondaries():
             self._check_invariants_as_standalone(secondary)
+            # We disable the remover for pre-images. Otherwise the remover might run and fix any
+            # potential consistency issues before we validate the config.system.preimages
+            # collection.
+            if "set_parameters" in secondary.mongod_options:
+                secondary.mongod_options["set_parameters"]["disableExpiredPreImagesRemover"] = True
+                secondary.mongod_options["set_parameters"][
+                    "disableExpiredChangeCollectionRemover"] = True
+            else:
+                secondary.mongod_options["set_parameters"] = {
+                    "disableExpiredPreImagesRemover": True,
+                    "disableExpiredChangeCollectionRemover": True,
+                }
 
             self.logger.info(
                 "Restarting the secondary on port %d as a replica set node with"
@@ -243,6 +264,22 @@ class PeriodicKillSecondariesTestCase(interface.DynamicTestCase):
         oplog_test_case.after_test(self, test_report)
         oplog_test_case.after_suite(test_report)
 
+    def _check_pre_images_consistency(self, test_report):
+        preimages_test_case = preimages_consistency.CheckReplPreImagesConsistency(
+            self._hook.logger, self.fixture)
+        preimages_test_case.before_suite(test_report)
+        preimages_test_case.before_test(self, test_report)
+        preimages_test_case.after_test(self, test_report)
+        preimages_test_case.after_suite(test_report)
+
+    def _check_change_collection_consistency(self, test_report):
+        change_collection_test_case = change_collection_consistency.CheckReplChangeCollectionConsistency(
+            self._hook.logger, self.fixture)
+        change_collection_test_case.before_suite(test_report)
+        change_collection_test_case.before_test(self, test_report)
+        change_collection_test_case.after_test(self, test_report)
+        change_collection_test_case.after_suite(test_report)
+
     def _restart_and_clear_fixture(self):
         # We restart the fixture after setting 'preserve_dbpath' back to its original value in order
         # to clear the contents of the data directory if desired. The CleanEveryN hook cannot be
@@ -257,15 +294,29 @@ class PeriodicKillSecondariesTestCase(interface.DynamicTestCase):
             raise errors.ServerFailure(
                 "{} did not exit cleanly after verifying data consistency".format(self.fixture))
 
+        for secondary in self.fixture.get_secondaries():
+            # We re-enable the removers for pre-images and change collections. These were disabled
+            # before re-joining the replSet as a secondary during the consistency checks.
+            secondary.mongod_options["set_parameters"].pop("disableExpiredPreImagesRemover")
+            secondary.mongod_options["set_parameters"].pop("disableExpiredChangeCollectionRemover")
+
         self.logger.info("Starting the fixture back up again with no data...")
         self.fixture.setup()
         self.logger.info(fixture.create_fixture_table(self.fixture))
         self.fixture.await_ready()
 
-    def _check_invariants_as_standalone(self, secondary):  # pylint: disable=too-many-locals
-        # pylint: disable=too-many-branches,too-many-statements
+    def _check_invariants_as_standalone(self, secondary):
         # We remove the --replSet option in order to start the node as a standalone.
-        replset_name = secondary.mongod_options.pop("replSet")
+        def try_pop_default_to_none(option):
+            try:
+                return secondary.mongod_options.pop(option)
+            except KeyError:
+                return None
+
+        # On serverless, replSet is not set, but the serverless flag still enables replication.
+        replset_name = try_pop_default_to_none("replSet")
+        serverless = try_pop_default_to_none("serverless")
+
         self.logger.info(
             "Restarting the secondary on port %d as a standalone node with"
             " its data files intact...", secondary.port)
@@ -337,7 +388,10 @@ class PeriodicKillSecondariesTestCase(interface.DynamicTestCase):
                 " mongod on port {}: {}".format(secondary.port, err.args[0]))
         finally:
             # Set the secondary's options back to their original values.
-            secondary.mongod_options["replSet"] = replset_name
+            if replset_name:
+                secondary.mongod_options["replSet"] = replset_name
+            if serverless:
+                secondary.mongod_options["serverless"] = serverless
 
     def _await_secondary_state(self, secondary):
         client = secondary.mongo_client()
@@ -352,8 +406,8 @@ class PeriodicKillSecondariesTestCase(interface.DynamicTestCase):
                     ]))
                 break
             except pymongo.errors.OperationFailure as err:
-                if (err.code != self.INTERRUPTED_DUE_TO_REPL_STATE_CHANGE
-                        and err.code != self.INTERRUPTED_DUE_TO_STORAGE_CHANGE):
+                if err.code not in (self.INTERRUPTED_DUE_TO_REPL_STATE_CHANGE,
+                                    self.INTERRUPTED_DUE_TO_STORAGE_CHANGE):
                     self.logger.exception(
                         "mongod on port %d failed to reach state SECONDARY after %d seconds",
                         secondary.port, fixture.ReplFixture.AWAIT_REPL_TIMEOUT_FOREVER_MINS * 60)
