@@ -9,10 +9,12 @@ import os
 import threading
 import time
 import unittest
+from logging import Logger
 
 from buildscripts.resmokelib import config as _config
 from buildscripts.resmokelib import logging
 from buildscripts.resmokelib.testing.symbolizer_service import ResmokeSymbolizer
+from buildscripts.resmokelib.testing.testcases.interface import TestCase
 
 
 # pylint: disable=attribute-defined-outside-init
@@ -73,7 +75,11 @@ class TestReport(unittest.TestResult):
                     if test_info.status is None or test_info.return_code is None:
                         # Mark the test as having timed out if it was interrupted. It might have
                         # passed if the suite ran to completion, but we wouldn't know for sure.
-                        #
+                        for logger in test_info.loggers:
+                            if "AFTER_TIMEOUT" not in logger.name:
+                                logger.name = f"{logger.name}:AFTER_TIMEOUT"
+                            logger.error(
+                                "HIT EVERGREEN TIMEOUT: Hang analyzer will kill or abort processes")
                         # Until EVG-1536 is completed, we shouldn't distinguish between failures and
                         # interrupted tests in the report.json file. In Evergreen, the behavior to
                         # sort tests with the "timeout" test status after tests with the "pass" test
@@ -105,7 +111,7 @@ class TestReport(unittest.TestResult):
 
         return combined_report
 
-    def startTest(self, test):
+    def startTest(self, test: TestCase):
         """Call before 'test' is run."""
 
         unittest.TestResult.startTest(self, test)
@@ -131,6 +137,8 @@ class TestReport(unittest.TestResult):
                                                                       test.logger, self.job_num,
                                                                       test.id(), self.job_logger)
 
+        test_info.add_logger(test_logger)
+        test_info.add_logger(self.job_logger)
         # Set up logging handlers to capture exceptions.
         test_info.exception_extractors = logging.loggers.configure_exception_capture(test_logger)
 
@@ -148,7 +156,7 @@ class TestReport(unittest.TestResult):
         test.override_logger(test_logger)
         test_info.start_time = time.time()
 
-    def stopTest(self, test):
+    def stopTest(self, test: TestCase):
         """Call after 'test' has run."""
 
         try:
@@ -181,7 +189,7 @@ class TestReport(unittest.TestResult):
             # Restore the original logger for the test.
             test.reset_logger()
 
-    def addError(self, test, err):
+    def addError(self, test: TestCase, err):
         """Call when a non-failureException was raised during the execution of 'test'."""
 
         unittest.TestResult.addError(self, test, err)
@@ -205,7 +213,6 @@ class TestReport(unittest.TestResult):
             if test_info.end_time is None:
                 raise ValueError("stopTest was not called on %s" % (test.basename()))
 
-            changed = test_info.status != "error"
             # We don't distinguish between test failures and Python errors in Evergreen.
             test_info.status = "error"
             test_info.evergreen_status = "fail"
@@ -218,9 +225,6 @@ class TestReport(unittest.TestResult):
         self.num_errored = len(self.get_errored())
         self.num_interrupted = len(self.get_interrupted())
 
-        if changed:
-            self._log_outcome_change(test, "error")
-
     def addFailure(self, test, err):
         """Call when a failureException was raised during the execution of 'test'."""
 
@@ -234,7 +238,7 @@ class TestReport(unittest.TestResult):
             test_info.evergreen_status = "fail"
             test_info.return_code = test.return_code
 
-    def setFailure(self, test, return_code=1, reason=""):
+    def setFailure(self, test, return_code=1):
         """Change the outcome of an existing test to a failure."""
 
         with self._lock:
@@ -242,7 +246,6 @@ class TestReport(unittest.TestResult):
             if test_info.end_time is None:
                 raise ValueError("stopTest was not called on %s" % (test.basename()))
 
-            changed = test_info.status != "fail"
             test_info.status = "fail"
             test_info.evergreen_status = "fail"
             test_info.return_code = return_code
@@ -253,9 +256,6 @@ class TestReport(unittest.TestResult):
         self.num_errored = len(self.get_errored())
         self.num_interrupted = len(self.get_interrupted())
 
-        if changed:
-            self._log_outcome_change(test, "fail", reason)
-
     def addSuccess(self, test):
         """Call when 'test' executed successfully."""
 
@@ -396,17 +396,6 @@ class TestReport(unittest.TestResult):
 
         raise ValueError("Details for %s not found in the report" % (test.basename()))
 
-    def _log_outcome_change(self, test, outcome, reason=""):
-        # Recreate the test logger for this test in order to append to the existing log.
-        (logger,
-         _) = logging.loggers.new_test_logger(test.short_name(), test.basename(), None, test.logger,
-                                              self.job_num, test.id(), self.job_logger)
-        logger.info(
-            'Sometime after completion of %s, the test outcome was changed to "%s" because: %s',
-            test.short_description(), outcome, reason if reason else ".")
-        for handler in logger.handlers:
-            logging.flush.close_later(handler)
-
 
 class TestInfo(object):
     """Holder for the test status and timing information."""
@@ -419,6 +408,7 @@ class TestInfo(object):
         self.group_id = None
         self.display_test_name = None
         self.dynamic = dynamic
+        self.loggers = []
 
         self.start_time = None
         self.end_time = None
@@ -430,6 +420,10 @@ class TestInfo(object):
         self.exception_extractors = []
         self.error = None
 
+    def add_logger(self, logger: Logger) -> None:
+        """Add logger instance."""
+        self.loggers.append(logger)
+
 
 def test_order(test_name):
     """
