@@ -2,40 +2,60 @@
 
 import hashlib
 import threading
+import uuid
 
 from buildscripts.resmokelib.testing.testcases import interface
 from buildscripts.resmokelib.testing.testcases import jsrunnerfile
+from buildscripts.resmokelib.testing.testcases import jstest
+from buildscripts.resmokelib.utils import registry
+from buildscripts.resmokelib import utils
 
 
-class FSMWorkloadTestCase(jsrunnerfile.JSRunnerFileTestCase):
+class _SingleFSMWorkloadTestCase(jsrunnerfile.JSRunnerFileTestCase):
     """An FSM workload to execute."""
 
-    REGISTERED_NAME = "fsm_workload_test"
+    REGISTERED_NAME = registry.LEAVE_UNREGISTERED
 
-    _COUNTER_LOCK = threading.Lock()
-    _COUNTER = 0
-
-    def __init__(self, logger, selected_tests, shell_executable=None, shell_options=None,
-                 same_db=False, same_collection=False, db_name_prefix=None):
-        """Initialize the FSMWorkloadTestCase with the FSM workload file."""
-
-        self.same_collection = same_collection
-        self.same_db = same_db or self.same_collection
-        self.db_name_prefix = db_name_prefix
-        self.dbpath_prefix = None
-        self.fsm_workload_group = self.get_workload_group(selected_tests)
-
-        test_name = self.get_workload_uid(selected_tests)
+    def __init__(self, logger, test_name, test_id, shell_executable=None, shell_options=None):
+        """Initialize the _SingleFSMWorkloadTestCase with the FSM workload file."""
 
         jsrunnerfile.JSRunnerFileTestCase.__init__(
             self, logger, "FSM workload", test_name,
             test_runner_file="jstests/concurrency/fsm_libs/resmoke_runner.js",
             shell_executable=shell_executable, shell_options=shell_options)
+        self._id = test_id
 
     def configure(self, fixture, *args, **kwargs):
         """Configure the FSMWorkloadTestCase runner."""
         interface.ProcessTestCase.configure(self, fixture, *args, **kwargs)
 
+
+class _FSMWorkloadTestCaseBuilder(interface.TestCaseFactory):
+    """Build the real TestCase in the FSMWorkloadTestCase wrapper."""
+
+    _COUNTER_LOCK = threading.Lock()
+    _COUNTER = 0
+
+    def __init__(self, logger, fsm_workload_group, test_name, test_id, shell_executable=None,
+                 shell_options=None, same_db=False, same_collection=False, db_name_prefix=None):
+        """Initialize the _FSMWorkloadTestCaseBuilder."""
+        interface.TestCaseFactory.__init__(self, _SingleFSMWorkloadTestCase, shell_options)
+        self.logger = logger
+        self.fsm_workload_group = fsm_workload_group
+        self.test_name = test_name
+        self.test_id = test_id
+        self.shell_executable = shell_executable
+        self.shell_options = utils.default_if_none(shell_options, {}).copy()
+
+        self.same_collection = same_collection
+        self.same_db = same_db or self.same_collection
+        self.db_name_prefix = db_name_prefix
+        self.dbpath_prefix = None
+
+        self.fixture = None
+
+    def configure(self, fixture, *args, **kwargs):
+        self.fixture = fixture
         self.dbpath_prefix = self.fixture.get_dbpath_prefix()
 
         global_vars = self.shell_options.get("global_vars", {}).copy()
@@ -50,9 +70,9 @@ class FSMWorkloadTestCase(jsrunnerfile.JSRunnerFileTestCase):
         test_data["fsmWorkloads"] = self.fsm_workload_group
         test_data["resmokeDbPathPrefix"] = self.dbpath_prefix
 
-        with FSMWorkloadTestCase._COUNTER_LOCK:
-            count = FSMWorkloadTestCase._COUNTER
-            FSMWorkloadTestCase._COUNTER += 1
+        with _FSMWorkloadTestCaseBuilder._COUNTER_LOCK:
+            count = _FSMWorkloadTestCaseBuilder._COUNTER
+            _FSMWorkloadTestCaseBuilder._COUNTER += 1
 
         # We use a global incrementing counter as a prefix for the database name to avoid any
         # collection lifecycle related issues in sharded clusters. This more closely matches how
@@ -64,6 +84,37 @@ class FSMWorkloadTestCase(jsrunnerfile.JSRunnerFileTestCase):
         test_data["sameCollection"] = self.same_collection
         test_data["peerPids"] = self.fixture.pids()
 
+    def make_process(self):
+        # This function should only be called by MultiClientsTestCase's _make_process().
+        test_case = self.create_test_case(self.logger, self.shell_options)
+        return test_case._make_process()  # pylint: disable=protected-access
+
+    def create_test_case(self, logger, shell_options) -> _SingleFSMWorkloadTestCase:
+        test_case = _SingleFSMWorkloadTestCase(logger, self.test_name, self.test_id,
+                                               self.shell_executable, shell_options)
+        test_case.configure(self.fixture)
+        return test_case
+
+
+class FSMWorkloadTestCase(jstest.MultiClientsTestCase):
+    """A wrapper for several copies of a _SingleFSMWorkloadTestCase to execute."""
+
+    REGISTERED_NAME = "fsm_workload_test"
+    TEST_KIND = "FSM workload"
+
+    def __init__(self, logger, selected_tests, shell_executable=None, shell_options=None,
+                 same_db=False, same_collection=False, db_name_prefix=None):
+        """Initialize the FSMWorkloadTestCase with the FSM workload file."""
+        fsm_workload_group = self.get_workload_group(selected_tests)
+        test_name = self.get_workload_uid(selected_tests)
+        test_id = uuid.uuid4()
+
+        factory = _FSMWorkloadTestCaseBuilder(logger, fsm_workload_group, test_name, test_id,
+                                              shell_executable, shell_options, same_db,
+                                              same_collection, db_name_prefix)
+        jstest.MultiClientsTestCase.__init__(self, logger, self.TEST_KIND, test_name, test_id,
+                                             factory)
+
     @staticmethod
     def get_workload_group(selected_tests):
         """Generate an FSM workload group from tests selected by the selector."""
