@@ -14,7 +14,7 @@ import math
 
 from buildscripts.resmokelib import config
 
-_IS_WINDOWS = sys.platform == "win32" or sys.platform == "cygwin"
+_IS_WINDOWS = sys.platform in ("win32", "cygwin")
 
 if _IS_WINDOWS:
     import ctypes
@@ -126,6 +126,29 @@ class Archival(object):
     def _get_s3_client():
         # Since boto3 is a 3rd party module, we import locally.
         import boto3
+        import botocore.session
+        botocore.session.Session()
+
+        if sys.platform in ("win32", "cygwin"):
+            # These overriden values can be found here
+            # https://github.com/boto/botocore/blob/13468bc9d8923eccd0816ce2dd9cd8de5a6f6e0e/botocore/configprovider.py#L49C7-L49C7
+            # This is due to the backwards breaking changed python introduced https://bugs.python.org/issue36264
+            botocore_session = botocore.session.Session(
+                session_vars={
+                    'config_file': (
+                        None,
+                        'AWS_CONFIG_FILE',
+                        os.path.join(os.environ['HOME'], '.aws', 'config'),
+                        None,
+                    ),
+                    'credentials_file': (
+                        None,
+                        'AWS_SHARED_CREDENTIALS_FILE',
+                        os.path.join(os.environ['HOME'], '.aws', 'credentials'),
+                        None,
+                    ),
+                })
+            boto3.setup_default_session(botocore_session=botocore_session)
         return boto3.client("s3")
 
     def archive_files_to_s3(self, display_name, input_files, s3_bucket, s3_path):
@@ -277,7 +300,7 @@ class Archival(object):
             return 1, str(err), 0
 
         # Round up the size of the archive.
-        size_mb = int(math.ceil(float(file_list_size(temp_file)) / (1024 * 1024)))  # pylint: disable=c-extension-no-member
+        size_mb = int(math.ceil(float(file_list_size(temp_file)) / (1024 * 1024)))
         self._upload_queue.put(
             UploadArgs(self.archival_json_file, display_name, temp_file, "application/x-gzip",
                        s3_bucket, s3_path, True))
