@@ -4,7 +4,7 @@
 import os
 import sys
 
-SCONS_VERSION = os.environ.get('SCONS_VERSION', "4.9.1")
+SCONS_VERSION = os.environ.get('SCONS_VERSION', "3.1.2")
 
 MONGODB_ROOT = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
 SCONS_DIR = os.path.join(MONGODB_ROOT, 'src', 'third_party', 'scons-' + SCONS_VERSION,
@@ -22,7 +22,7 @@ sys.path = [SCONS_DIR, SITE_TOOLS_DIR] + sys.path
 from mongo.pip_requirements import verify_requirements, MissingRequirements
 
 try:
-    verify_requirements('etc/pip/compile-requirements.txt')
+    verify_requirements()
 except MissingRequirements as ex:
     print(ex)
     sys.exit(1)
