@@ -1,7 +1,8 @@
 """Test resmoke's handling of test/task timeouts and archival."""
 
-import logging
+import datetime
 import json
+import logging
 import os
 import os.path
 import re
@@ -13,7 +14,13 @@ from shutil import rmtree
 
 import yaml
 
-from buildscripts.resmokelib import core
+from buildscripts.ciconfig.evergreen import parse_evergreen_file
+from buildscripts.resmokelib import config, core, suitesconfig
+from buildscripts.resmokelib.hang_analyzer.attach_core_analyzer_task import \
+    matches_generated_task_pattern
+from buildscripts.resmokelib.hang_analyzer.gen_hang_analyzer_tasks import \
+    get_generated_task_name
+from buildscripts.resmokelib.utils.dictionary import get_dict_value
 
 
 class _ResmokeSelftest(unittest.TestCase):
@@ -62,8 +69,13 @@ class TestArchivalOnFailure(_ResmokeSelftest):
         cls.archival_file = "test_archival.txt"
 
     def test_archival_on_task_failure(self):
+        # The --originSuite argument is to trick the resmoke local invocation into passing
+        # because when we pass --taskId into resmoke it thinks that it is being ran in evergreen
+        # and cannot normally find an evergreen task associated with
+        # buildscripts/tests/resmoke_end2end/suites/resmoke_selftest_task_failure.yml
         resmoke_args = [
             "--suites=buildscripts/tests/resmoke_end2end/suites/resmoke_selftest_task_failure.yml",
+            "--originSuite=resmoke_end2end_tests",
             "--taskId=123",
             "--internalParam=test_archival",
             "--repeatTests=2",
@@ -77,9 +89,14 @@ class TestArchivalOnFailure(_ResmokeSelftest):
         self.assert_dir_file_count(self.test_dir, self.archival_file, archival_dirs_to_expect)
 
     def test_archival_on_task_failure_no_passthrough(self):
+        # The --originSuite argument is to trick the resmoke local invocation into passing
+        # because when we pass --taskId into resmoke it thinks that it is being ran in evergreen
+        # and cannot normally find an evergreen task associated with
+        # buildscripts/tests/resmoke_end2end/suites/resmoke_selftest_task_failure_no_passthrough.yml
         resmoke_args = [
             "--suites=buildscripts/tests/resmoke_end2end/suites/resmoke_selftest_task_failure_no_passthrough.yml",
             "--taskId=123",
+            "--originSuite=resmoke_end2end_tests",
             "--internalParam=test_archival",
             "--repeatTests=2",
             "--jobs=2",
@@ -135,24 +152,43 @@ class TestTimeout(_ResmokeSelftest):
             self.resmoke_process.stop()
         self.assertEqual(return_code, 0)
 
-    def execute_resmoke(self, resmoke_args, sleep_secs=30, **kwargs):  # pylint: disable=arguments-differ
+    def execute_resmoke(self, resmoke_args, sentinel_file, **kwargs):
+        # Since this test is designed to start remoke, wait for it to be up-and-running, and then
+        # kill resmoke, we use a sentinel file to accomplish this.
+
+        # Form sentinel path, and make sure it's absent:
+        sentinel_path = f"{os.environ.get('TMPDIR') or os.environ.get('TMP_DIR') or '/tmp'}/{sentinel_file}.js.sentinel"
+        if os.path.isfile(sentinel_path):
+            os.remove(sentinel_path)
+
+        # Spawn resmoke (async):
         super(TestTimeout, self).execute_resmoke(resmoke_args, **kwargs)
 
-        time.sleep(sleep_secs
-                   )  # TODO: Change to more durable way of ensuring the fixtures have been set up.
+        # Wait for sentinel file to appear; bail if it takes too long:
+        started_polling_datetime = datetime.datetime.now()
+        while not os.path.isfile(sentinel_path):
+            time.sleep(0.1)
+            if datetime.datetime.now() - started_polling_datetime > datetime.timedelta(minutes=5):
+                self.fail("SUT is not available within 99 seconds; aborting test")
 
+        # Kill resmoke:
         self.signal_resmoke()
 
     def test_task_timeout(self):
+        # The --originSuite argument is to trick the resmoke local invocation into passing
+        # because when we pass --taskId into resmoke it thinks that it is being ran in evergreen
+        # and cannot normally find an evergreen task associated with
+        # buildscripts/tests/resmoke_end2end/suites/resmoke_selftest_task_timeout.yml
         resmoke_args = [
             "--suites=buildscripts/tests/resmoke_end2end/suites/resmoke_selftest_task_timeout.yml",
             "--taskId=123",
+            "--originSuite=resmoke_end2end_tests",
             "--internalParam=test_archival",
             "--internalParam=test_analysis",
             "--repeatTests=2",
             "--jobs=2",
         ]
-        self.execute_resmoke(resmoke_args)
+        self.execute_resmoke(resmoke_args, sentinel_file="timeout0")
 
         archival_dirs_to_expect = 4  # 2 tests * 2 mongod
         self.assert_dir_file_count(self.test_dir, self.archival_file, archival_dirs_to_expect)
@@ -161,15 +197,20 @@ class TestTimeout(_ResmokeSelftest):
         self.assert_dir_file_count(self.test_dir, self.analysis_file, analysis_pids_to_expect)
 
     def test_task_timeout_no_passthrough(self):
+        # The --originSuite argument is to trick the resmoke local invocation into passing
+        # because when we pass --taskId into resmoke it thinks that it is being ran in evergreen
+        # and cannot normally find an evergreen task associated with
+        # buildscripts/tests/resmoke_end2end/suites/resmoke_selftest_task_timeout_no_passthrough.yml
         resmoke_args = [
             "--suites=buildscripts/tests/resmoke_end2end/suites/resmoke_selftest_task_timeout_no_passthrough.yml",
             "--taskId=123",
+            "--originSuite=resmoke_end2end_tests",
             "--internalParam=test_archival",
             "--internalParam=test_analysis",
             "--repeatTests=2",
             "--jobs=2",
         ]
-        self.execute_resmoke(resmoke_args)
+        self.execute_resmoke(resmoke_args, sentinel_file="timeout1")
 
         archival_dirs_to_expect = 8  # (2 tests + 2 stacktrace files) * 2 nodes
         self.assert_dir_file_count(self.test_dir, self.archival_file, archival_dirs_to_expect)
@@ -179,15 +220,20 @@ class TestTimeout(_ResmokeSelftest):
 
     # Test scenarios where an resmoke-launched process launches resmoke.
     def test_nested_timeout(self):
+        # The --originSuite argument is to trick the resmoke local invocation into passing
+        # because when we pass --taskId into resmoke it thinks that it is being ran in evergreen
+        # and cannot normally find an evergreen task associated with
+        # buildscripts/tests/resmoke_end2end/suites/resmoke_selftest_nested_timeout.yml
         resmoke_args = [
             "--suites=buildscripts/tests/resmoke_end2end/suites/resmoke_selftest_nested_timeout.yml",
             "--taskId=123",
+            "--originSuite=resmoke_end2end_tests",
             "--internalParam=test_archival",
             "--internalParam=test_analysis",
             "jstests/resmoke_selftest/end2end/timeout/nested/top_level_timeout.js",
         ]
 
-        self.execute_resmoke(resmoke_args, sleep_secs=25)
+        self.execute_resmoke(resmoke_args, sentinel_file="inner_level_timeout")
 
         archival_dirs_to_expect = 4  # ((2 tests + 2 stacktrace files) * 2 nodes) / 2 data_file directories
         self.assert_dir_file_count(self.test_dir, self.archival_file, archival_dirs_to_expect)
@@ -221,6 +267,19 @@ class TestTestSelection(_ResmokeSelftest):
 
         return tests_run
 
+    def test_missing_excluded_file(self):
+        # Tests a suite that excludes a missing file
+        self.assertEqual(
+            0,
+            self.execute_resmoke([
+                f"--reportFile={self.report_file}", "--repeatTests=2",
+                f"--suites={self.suites_root}/resmoke_missing_test.yml",
+                f"{self.testfiles_root}/one.js", f"{self.testfiles_root}/one.js",
+                f"{self.testfiles_root}/one.js"
+            ]).wait())
+
+        self.assertEqual(6 * [f"{self.testfiles_root}/one.js"], self.get_tests_run())
+
     def test_positional_arguments(self):
         self.assertEqual(
             0,
@@ -275,15 +334,16 @@ class TestTestSelection(_ResmokeSelftest):
         # Additionally can assert on the error message.
         self.assertEqual(
             2,
-            self.execute_resmoke([f"--replay={self.test_dir}/replay",
-                                  "jstests/filename.js"]).wait())
+            self.execute_resmoke(
+                [f"--replay={self.test_dir}/replay", f"{self.testfiles_root}/one.js"]).wait())
 
-        # When multiple positional arguments are presented, they're all treated as test files. Technically errors on file `@<testdir>/replay` not existing. It's not a requirement that this invocation errors in this less specific way.
+        # When multiple positional arguments are presented, they're all treated as test files.
         self.assertEqual(
-            1,
-            self.execute_resmoke([f"@{self.test_dir}/replay", "jstests/filename.js"]).wait())
+            2,
+            self.execute_resmoke([f"@{self.test_dir}/replay",
+                                  f"{self.testfiles_root}/one.js"]).wait())
         self.assertEqual(
-            1,
+            2,
             self.execute_resmoke([f"{self.testfiles_root}/one.js",
                                   f"@{self.test_dir}/replay"]).wait())
 
@@ -330,10 +390,10 @@ class TestSetParameters(_ResmokeSelftest):
             "Running test. Template suite: %s Rewritten suite: %s Resmoke Args: %s Test output file: %s.",
             suite_template, suite_file, resmoke_args, self.shell_output_file)
 
-        resmoke_process = core.programs.make_process(self.logger, [
-            sys.executable, "buildscripts/resmoke.py", "run", f"--suites={suite_file}",
-            f"{self.testfiles_root}/fixture_info.js"
-        ] + resmoke_args)
+        resmoke_process = core.programs.make_process(
+            self.logger,
+            [sys.executable, "buildscripts/resmoke.py", "run", f"--suites={suite_file}"
+             ] + resmoke_args)
         resmoke_process.start()
 
         return resmoke_process
@@ -541,6 +601,77 @@ class TestSetShellSeed(unittest.TestCase):
             len(random_seeds) > 1, msg="Resmoke generated the same random seed 10 times in a row.")
 
 
+# In resmoke we expect certain parts of the evergreen config to be a certain way
+# These tests will fail if something is not as expected and also needs to change somewhere else in resmoke
+class TestEvergreenYML(unittest.TestCase):
+    @classmethod
+    def setUpClass(cls):
+        cls.evg_conf = parse_evergreen_file("etc/evergreen.yml")
+        config.CONFIG_DIR = "buildscripts/resmokeconfig"
+
+    def validate_jstestfuzz_selector(self, suite_names):
+        for suite_name in suite_names:
+            suite_config = suitesconfig.get_suite(suite_name).get_config()
+            expected_selector = ["jstestfuzz/out/*.js"]
+            self.assertEqual(
+                suite_config["selector"]["roots"], expected_selector,
+                msg=f"The jstestfuzz selector for {suite_name} did not match 'jstestfuzz/out/*.js'")
+
+    # This test asserts that the jstestfuzz tasks uploads the the URL we expect it to
+    # If the remote url changes, also change it in the _log_local_resmoke_invocation method
+    # before fixing this test to the correct url
+    def test_jstestfuzz_download_url(self):
+        functions = self.evg_conf.functions
+        run_jstestfuzz = functions["run jstestfuzz"]
+        contains_correct_url = False
+        for item in run_jstestfuzz:
+            if item["command"] != "s3.put":
+                continue
+
+            remote_url = item["params"]["remote_file"]
+            if remote_url == "${project}/${build_variant}/${revision}/jstestfuzz/${task_id}-${execution}.tgz":
+                contains_correct_url = True
+                break
+
+        self.assertTrue(
+            contains_correct_url, msg=
+            "The 'run jstestfuzz' function in evergreen did not contain the remote_url that was expected"
+        )
+
+    # This tasks asserts that the way implicit multiversion tasks are defined has not changed
+    # If this fails, you will need to correct the _log_local_resmoke_invocation method before fixing
+    # this test
+    def test_implicit_multiversion_tasks(self):
+        multiverson_task_names = self.evg_conf.get_task_names_by_tag("multiversion")
+        implicit_multiversion_count = 0
+        for multiversion_task_name in multiverson_task_names:
+            task_config = self.evg_conf.get_task(multiversion_task_name)
+            func = task_config.find_func_command("initialize multiversion tasks")
+            if func is not None:
+                implicit_multiversion_count += 1
+
+        self.assertNotEqual(0, implicit_multiversion_count,
+                            msg="Could not find any implicit multiversion tasks in evergreen")
+
+    # This tasks asserts that the way jstestfuzz tasks are defined has not changed
+    # It also asserts that the selector for jstestfuzz tasks always points to jstestfuzz/out/*.js
+    # If this fails, you will need to correct the _log_local_resmoke_invocation method before fixing
+    # this test
+    def test_jstestfuzz_tasks(self):
+        jstestfuzz_count = 0
+        for task in self.evg_conf.tasks:
+            generate_func = task.find_func_command("generate resmoke tasks")
+            if (generate_func is None
+                    or get_dict_value(generate_func, ["vars", "is_jstestfuzz"]) != "true"):
+                continue
+
+            jstestfuzz_count += 1
+
+            self.validate_jstestfuzz_selector(task.get_suite_names())
+
+        self.assertNotEqual(0, jstestfuzz_count, msg="Could not find any jstestfuzz tasks")
+
+
 class TestMultiversionConfig(unittest.TestCase):
     def test_valid_yaml(self):
         file_name = "multiversion-config.yml"
@@ -557,3 +688,12 @@ class TestMultiversionConfig(unittest.TestCase):
             self.fail(msg="`resmoke.py multiversion-config` does not output valid yaml.")
 
         os.remove(file_name)
+
+
+class TestCoreAnalyzerFunctions(unittest.TestCase):
+    def test_generated_task_name(self):
+        task_name = "test_tast_name"
+        execution = "0"
+        generated_task_name = get_generated_task_name(task_name, execution)
+        self.assertEquals(matches_generated_task_pattern(task_name, generated_task_name), execution)
+        self.assertIsNone(matches_generated_task_pattern("not_same_task", generated_task_name))
