@@ -4,6 +4,7 @@ import unittest
 from collections import defaultdict
 from typing import Optional
 
+from buildscripts.idl.gen_all_feature_flag_list import get_all_feature_flags_turned_on_by_default
 from buildscripts.resmokelib.multiversionconstants import (
     REQUIRES_FCV_TAG_LATEST,
     REQUIRES_FCV_TAGS_LESS_THAN_LATEST,
@@ -30,6 +31,20 @@ class JstestTagRule:
         return None
 
 
+class FeatureFlagIncompatibleTagRule(JstestTagRule):
+    def __init__(self):
+        super().__init__(
+            failure_message=
+            "The following tags are not allowed for feature flags that default to true")
+        self.disallowed_tags = {
+            f"{flag}_incompatible"
+            for flag in get_all_feature_flags_turned_on_by_default()
+        }
+
+    def _tag_failed(self, file: str, tag: str) -> bool:
+        return tag in self.disallowed_tags
+
+
 class RequiresFcvTagRule(JstestTagRule):
     def __init__(self):
         super().__init__(
@@ -45,6 +60,7 @@ class TestJstestTags(unittest.TestCase):
         globs = ["src/mongo/db/modules/enterprise/jstests/**/*.js", "jstests/**/*.js"]
 
         tag_rules = [
+            FeatureFlagIncompatibleTagRule(),
             RequiresFcvTagRule(),
         ]
 
