@@ -3,8 +3,6 @@ import unittest
 
 from buildscripts.validate_commit_message import main, STATUS_OK, STATUS_ERROR
 
-# pylint: disable=missing-docstring,no-self-use
-
 
 class ValidateCommitMessageTest(unittest.TestCase):
     def test_valid(self):
@@ -22,7 +20,7 @@ class ValidateCommitMessageTest(unittest.TestCase):
             "SERVER-",  # missing number
             "Revert SERVER-60",  # missing quote before SERVER
             "",  # empty value
-            "nonsense",  # nonsense values
+            "nonsense",  # nonsense value
         ]
 
         self.assertTrue(all(main([message]) == STATUS_ERROR for message in messages))
