@@ -1,6 +1,4 @@
 // SERVER-14421 minDistance for $geoNear aggregation operator
-(function() {
-'use strict';
 var coll = db.mindistance;
 coll.drop();
 assert.commandWorked(coll.insert([
@@ -36,5 +34,4 @@ results = coll.aggregate([{
     }
 }]);
 assert.eq(results.itcount(), 2);
-coll.drop();
-}());
+coll.drop();
\ No newline at end of file
