@@ -1,6 +1,4 @@
 // SERVER-18222: Add $isArray aggregation expression.
-(function() {
-'use strict';
 var coll = db.is_array_expr;
 coll.drop();
 
@@ -39,5 +37,4 @@ var expectedResults = [
     {_id: 10, isArray: true},
 ];
 
-assert.eq(results, expectedResults);
-}());
+assert.eq(results, expectedResults);
\ No newline at end of file
