@@ -6,7 +6,7 @@
 // ]
 
 // server-3253 Unsharded support for $out
-load('jstests/aggregation/extras/utils.js');
+import {anyEq, assertErrorCode, collectionExists} from "jstests/aggregation/extras/utils.js";
 
 const testDb = db.getSiblingDB("server3253");
 var input = testDb.server3253_in;
