@@ -39,7 +39,7 @@ var a6 = s3832.aggregate({$match: {b: "baz"}}, {$sort: {a: 1}});
 
 assert.eq(a6.toArray(), a1result, 's3832.a6 failed');
 
-var dropb = db.s3832.dropIndex("s3832_b");
+db.s3832.dropIndex("s3832_b");
 
 db.s3832.createIndex({a: 1}, {name: "s3832_a"});
 
