@@ -1,5 +1,3 @@
-load('jstests/aggregation/extras/utils.js');
-
 db.server5782.drop();
 db.server5782.save({string: "foo"});
 
@@ -12,7 +10,7 @@ var result = db.runCommand({
 assert.eq(result.cursor.firstBatch[0].stringis$string, true);
 
 // check that with $literal we end up comparing a field with '$string' and the result is true
-var result = db.runCommand({
+result = db.runCommand({
     aggregate: "server5782",
     pipeline: [{$project: {stringis$string: {$eq: ["$string", {$literal: '$string'}]}}}],
     cursor: {}
