@@ -1,9 +1,7 @@
 /**
  * Test that $near queries are disallowed in $match stages.
  */
-(function() {
-"use strict";
-load("jstests/aggregation/extras/utils.js");
+import {assertErrorCode} from "jstests/aggregation/extras/utils.js";
 
 const coll = db.getCollection("no_near_in_match");
 coll.drop();
@@ -34,5 +32,4 @@ assert.eq(1, coll.find(geoNearQuery).count());
 // Test that we refuse to run $match with a near query.
 assertErrorCode(coll, {$match: nearQuery}, 5626500);
 assertErrorCode(coll, {$match: nearSphereQuery}, 5626500);
-assertErrorCode(coll, {$match: geoNearQuery}, 5626500);
-}());
+assertErrorCode(coll, {$match: geoNearQuery}, 5626500);
\ No newline at end of file
