@@ -1,6 +1,6 @@
 // SERVER-6531 support $within in $match aggregation operations
 
-c = db.s6531;
+let c = db.s6531;
 c.drop();
 
 for (var x = 0; x < 10; x++) {
@@ -10,10 +10,10 @@ for (var x = 0; x < 10; x++) {
 }
 
 function test(variant) {
-    query = {loc: {$within: {$center: [[5, 5], 3]}}};
-    sort = {_id: 1};
-    aggOut = c.aggregate({$match: query}, {$sort: sort});
-    cursor = c.find(query).sort(sort);
+    let query = {loc: {$within: {$center: [[5, 5], 3]}}};
+    let sort = {_id: 1};
+    let aggOut = c.aggregate({$match: query}, {$sort: sort});
+    let cursor = c.find(query).sort(sort);
 
     assert.eq(aggOut.toArray(), cursor.toArray());
 }
