@@ -1,9 +1,6 @@
 // SERVER-75670 Incorrect projection pushdown caused by dependency analysis for multiple $project
 // stages on the same dotted path.
-(function() {
-"use strict";
-
-load('jstests/aggregation/extras/utils.js');  // For resultsEq.
+import {resultsEq} from "jstests/aggregation/extras/utils.js";
 
 const collName = jsTestName();
 const coll = db.getCollection(collName);
@@ -26,5 +23,4 @@ assert(resultsEq(
             {$project: {"obj.str": 1}},
         ])
         .toArray(),
-    ));
-})();
\ No newline at end of file
+    ));
\ No newline at end of file
