@@ -3,13 +3,13 @@
  * including verifying the bug in SERVER-85892 is addressed when the spec 'whenMatched' is not
  * empty.
  *
- * @tags: [
- *   assumes_against_mongod_not_mongos,  # not yet supported until 7.2 with SERVER-65534
- * ]
+ * @tags: [ requires_fcv_80 ]
  */
 
-load("jstests/aggregation/extras/merge_helpers.js");  // For withEachMergeMode and
-                                                      // dropWithoutImplicitRecreate.
+import {
+    dropWithoutImplicitRecreate,
+    withEachMergeMode
+} from "jstests/aggregation/extras/merge_helpers.js";
 
 const outColl = db[`${jsTest.name()}_out`];
 const outCollName = outColl.getName();
