@@ -1,8 +1,10 @@
-load("jstests/aggregation/extras/utils.js");        // For assertErrorCode and assertErrMsgContains.
-load("jstests/libs/sbe_assert_error_override.js");  // Override error-code-checking APIs.
+import "jstests/libs/sbe_assert_error_override.js";
 
-(function() {
-"use strict";
+import {
+    anyEq,
+    assertErrCodeAndErrMsgContains,
+    assertErrorCode
+} from "jstests/aggregation/extras/utils.js";
 
 const coll = db.date_from_string;
 
@@ -799,4 +801,3 @@ assertErrCodeAndErrMsgContains(coll, pipeline, ErrorCodes.ConversionFailure, "No
 pipeline =
     [{$project: {date: {$dateFromString: {dateString: "12/31/2018", format: "%m/%d\0/%Y"}}}}];
 assertErrCodeAndErrMsgContains(coll, pipeline, ErrorCodes.ConversionFailure, "Trailing data");
-})();
