@@ -1,10 +1,8 @@
 // Test $filter aggregation expression.
 
-load('jstests/aggregation/extras/utils.js');        // For assertErrorCode.
-load("jstests/libs/sbe_assert_error_override.js");  // Override error-code-checking APIs.
+import "jstests/libs/sbe_assert_error_override.js";
 
-(function() {
-'use strict';
+import {assertArrayEq, assertErrorCode} from "jstests/aggregation/extras/utils.js";
 
 function runAndAssert(filterSpec, expectedResult) {
     const actualResult = coll.aggregate([{$project: {b: filterSpec}}, {$sort: {_id: 1}}]).toArray();
@@ -75,28 +73,18 @@ runAndAssert(filterDoc, expectedResults);
 // The 'limit' argument must be greater than zero and fit into int32 type. Otherwise, we throw an
 // error.
 filterDoc = {input: '$a', as: 'x', cond: true, limit: 0.5};
-let projectSpec = {$project: {b: {$filter: filterDoc}}};
 runAndAssertThrows(filterDoc, 327391);
 
 filterDoc = {input: '$a', as: 'x', cond: true, limit: 0};
-projectSpec = {
-    $project: {b: {$filter: filterDoc}}
-};
 runAndAssertThrows(filterDoc, 327392);
 
 filterDoc = {input: '$a', as: 'x', cond: true, limit: -100};
-projectSpec = {
-    $project: {b: {$filter: filterDoc}}
-};
 runAndAssertThrows(filterDoc, 327392);
 
 // Passing 'maxInt32 + 1' value for the 'limit' argument should throw an exception
 filterDoc = {
     input: '$a', as: 'x', cond: true, limit: 2147483648
 };
-projectSpec = {
-    $project: {b: {$filter: filterDoc}}
-};
 runAndAssertThrows(filterDoc, 327391);
 
 // Create filter that uses the default variable name in 'cond'.
@@ -115,7 +103,7 @@ expectedResults = [
 runAndAssert(filterDoc, expectedResults);
 
 // Create filter with path expressions inside $let expression.
-filterDoc = 
+filterDoc =
     {
                 $let: {
                     vars: {
@@ -321,7 +309,7 @@ expectedResults = [
 
 filterDoc = {$filter: {
     input: "$items",
-    limit: {$add: ["$maxSaleItems", 1]}, 
+    limit: {$add: ["$maxSaleItems", 1]},
     as: "item",
     cond: { $gte: [ "$$item.price", 100 ] }
  }};
@@ -436,4 +424,3 @@ filterDoc = {
     $filter: {input: '$a', cond: {$or: [{$lt: ['$$this', 0]}, {$ln: '$$this'}]}}
 };
 runAndAssert(filterDoc, expectedResults);
-}());
