@@ -1,9 +1,6 @@
 /**
  * Tests behavior of $first and $last aggregation operators.
  */
-(function() {
-"use strict";
-
 const coll = db.expression_first_last;
 
 coll.drop();
@@ -33,5 +30,4 @@ assert.eq(result, [
     {_id: 4, f: null, l: null},
     {_id: 5, f: null, l: null},
     {_id: 6, f: null, l: null},
-]);
-}());
+]);
\ No newline at end of file
