@@ -1,9 +1,6 @@
 /**
  * Tests the 'n' family of accumulators implemented as expressions.
  */
-(function() {
-'use strict';
-
 const coll = db[jsTestName()];
 const doc = {
     a: [1, 2, 3, 5, 7, 9],
@@ -55,5 +52,4 @@ args = {
 testExpr({$minN: args}, [3, 4]);
 testExpr({$maxN: args}, [5, 4]);
 testExpr({$firstN: args}, [3, 4]);
-testExpr({$lastN: args}, [4, 5]);
-})();
\ No newline at end of file
+testExpr({$lastN: args}, [4, 5]);
\ No newline at end of file
