@@ -15,145 +15,25 @@
  *   # Index filter commands do not support causal consistency.
  *   does_not_support_causal_consistency,
  *   requires_fcv_63,
+ *   not_allowed_with_signed_security_token,
  * ]
  */
 
-(function() {
-"use strict";
+import {
+    addIndex,
+    assertPipelineResultsAndExplain,
+    assertPlanDoesNotUseDistinctScan,
+    assertPlanUsesCollScan,
+    assertPlanUsesDistinctScan,
+    assertPlanUsesIndexScan,
+    coll,
+    documents,
+    prepareCollection,
+    removeIndex,
+} from "jstests/aggregation/libs/group_to_distinct_scan_utils.js";
+import {getAggPlanStage, getQueryPlanner} from "jstests/libs/analyze_plan.js";
 
-load("jstests/libs/analyze_plan.js");
-
-const coll = db.group_conversion_to_distinct_scan;
-coll.drop();
-
-// Add test data and indexes. Fields prefixed with "mk" are multikey.
-let indexList = [
-    {pattern: {a: 1, b: 1, c: 1}, option: {}},
-    {pattern: {mkA: 1, b: 1, c: 1}, option: {}},
-    {pattern: {aa: 1, mkB: 1, c: 1}, option: {}},
-    {pattern: {aa: 1, bb: 1, c: 1}, option: {}},
-    {pattern: {"foo.a": 1, "foo.b": 1}, option: {}},
-    {pattern: {"mkFoo.a": 1, "mkFoo.b": 1}, option: {}},
-    {pattern: {"foo.a": 1, "mkFoo.b": 1}, option: {}}
-];
-
-function createIndexes() {
-    for (const indexSpec of indexList) {
-        assert.commandWorked(coll.createIndex(indexSpec.pattern, indexSpec.option));
-    }
-}
-createIndexes();
-
-const documents = [
-    {_id: 0, a: 1, b: 1, c: 1},
-    {_id: 1, a: 1, b: 2, c: 2},
-    {_id: 2, a: 1, b: 2, c: 3},
-    {_id: 3, a: 1, b: 3, c: 2},
-    {_id: 4, a: 2, b: 2, c: 2},
-    {_id: 5, b: 1, c: 1},
-    {_id: 6, a: null, b: 1, c: 1.5},
-
-    {_id: 7, aa: 1, mkB: 2, bb: 2},
-    {_id: 8, aa: 1, mkB: [1, 3], bb: 1},
-    {_id: 9, aa: 2, mkB: [], bb: 3},
-
-    {_id: 10, mkA: 1, c: 3},
-    {_id: 11, mkA: [2, 3, 4], c: 3},
-    {_id: 12, mkA: 2, c: 2},
-    {_id: 13, mkA: 3, c: 4},
-
-    {_id: 14, foo: {a: 1, b: 1}, mkFoo: {a: 1, b: 1}},
-    {_id: 15, foo: {a: 1, b: 2}, mkFoo: {a: 1, b: 2}},
-    {_id: 16, foo: {a: 2, b: 2}, mkFoo: {a: 2, b: 2}},
-    {_id: 17, foo: {b: 1}, mkFoo: {b: 1}},
-    {_id: 18, foo: {a: null, b: 1}, mkFoo: {a: null, b: 1}},
-    {_id: 19, foo: {a: 3}, mkFoo: [{a: 3, b: 4}, {a: 4, b: 3}]},
-
-    {_id: 20, str: "foo", d: 1},
-    {_id: 21, str: "FoO", d: 2},
-    {_id: 22, str: "bar", d: 4},
-    {_id: 23, str: "bAr", d: 3}
-];
-assert.commandWorked(coll.insert(documents));
-
-// Helper for dropping an index and removing it from the list of indexes.
-function removeIndex(pattern) {
-    assert.commandWorked(coll.dropIndex(pattern));
-    indexList = indexList.filter((ix) => bsonWoCompare(ix.pattern, pattern) != 0);
-}
-
-function addIndex(pattern, option) {
-    indexList.push({pattern: pattern, option: option});
-    assert.commandWorked(coll.createIndex(pattern, option));
-}
-
-// Check that 'pipeline' returns the correct results with and without a hint added to the query.
-// We also test with and without indices to check all the possibilities. 'options' is the
-// options to pass to aggregate() and may be omitted. Similarly, the hint object can be omitted
-// and will default to a $natural hint.
-function assertResultsMatchWithAndWithoutHintandIndexes(pipeline,
-                                                        expectedResults,
-                                                        hintObj = {
-                                                            $natural: 1
-                                                        },
-                                                        options = {}) {
-    assert.commandWorked(coll.dropIndexes());
-    const resultsNoIndex = coll.aggregate(pipeline, options).toArray();
-
-    createIndexes();
-    const resultsWithIndex = coll.aggregate(pipeline, options).toArray();
-
-    const passedOptions = Object.assign({}, {hint: hintObj}, options);
-    const resultsWithHint = coll.aggregate(pipeline, passedOptions).toArray();
-
-    assert.sameMembers(resultsNoIndex, resultsWithIndex);
-    assert.sameMembers(resultsWithIndex, resultsWithHint);
-    assert.sameMembers(resultsWithHint, expectedResults);
-}
-
-function assertPlanUsesDistinctScan(explain, keyPattern) {
-    assert.neq(null, getAggPlanStage(explain, "DISTINCT_SCAN"), explain);
-
-    if (keyPattern) {
-        assert.eq(keyPattern, getAggPlanStage(explain, "DISTINCT_SCAN").keyPattern);
-    }
-
-    // Pipelines that use the DISTINCT_SCAN optimization should not also have a blocking sort.
-    assert.eq(null, getAggPlanStage(explain, "SORT"), explain);
-}
-
-function assertPlanDoesNotUseDistinctScan(explain) {
-    assert.eq(null, getAggPlanStage(explain, "DISTINCT_SCAN"), explain);
-}
-
-function assertPlanUsesIndexScan(explain, keyPattern) {
-    assertPlanDoesNotUseDistinctScan(explain);
-    assert.neq(null, getAggPlanStage(explain, "IXSCAN"), explain);
-    assert.eq(keyPattern, getAggPlanStage(explain, "IXSCAN").keyPattern);
-}
-
-function assertPlanUsesCollScan(explain) {
-    assertPlanDoesNotUseDistinctScan(explain);
-    assert.eq(null, getAggPlanStage(explain, "IXSCAN"), explain);
-    assert.neq(null, getAggPlanStage(explain, "COLLSCAN"), explain);
-}
-
-function assertPipelineResultsAndExplain({
-    pipeline,
-    options = {},
-    hint = undefined,
-    expectsIndexFilter = false,
-    expectedOutput,
-    validateExplain,
-}) {
-    assertResultsMatchWithAndWithoutHintandIndexes(pipeline, expectedOutput, hint, options);
-    const passedOptions = hint ? Object.assign({}, {hint}, options) : options;
-    const explain = coll.explain().aggregate(pipeline, passedOptions);
-    validateExplain(explain);
-    if (expectsIndexFilter) {
-        assert.eq(true, explain.stages[0].$cursor.queryPlanner.indexFilterSet);
-    }
-}
+prepareCollection();
 
 //
 // Verify that a $sort-$group pipeline can use DISTINCT_SCAN when the sort is available from an
@@ -243,7 +123,7 @@ assertPipelineResultsAndExplain({
         pipeline,
         expectedOutput,
         hint: {$natural: 1},
-        expectsIndexFilter: true,
+        expectsIndexFilter: !TestData.isHintsToQuerySettingsSuite,
         validateExplain: (explain) => assertPlanUsesDistinctScan(explain, keyPattern),
     });
 
@@ -254,7 +134,7 @@ assertPipelineResultsAndExplain({
         pipeline,
         expectedOutput,
         hint: {_id: 1},
-        expectsIndexFilter: true,
+        expectsIndexFilter: !TestData.isHintsToQuerySettingsSuite,
         validateExplain: (explain) => assertPlanUsesDistinctScan(explain, keyPattern),
     });
 
@@ -593,8 +473,16 @@ assertPipelineResultsAndExplain({
 // accumulator that includes an expression.
 //
 assertPipelineResultsAndExplain({
-    pipeline: [{$sort: {a: 1, b: 1}}, {$group: {_id: "$a", accum: {$last: {$add: ["$b", "$c"]}}}}],
-    expectedOutput: [{_id: null, accum: 2.5}, {_id: 1, accum: 5}, {_id: 2, accum: 4}],
+    pipeline: [
+        {$sort: {a: 1, b: 1}},
+        {$group: {_id: "$a", accum: {$last: {$add: ["$b", "$c"]}}}},
+        /* There are 2 documents with {a: null}, one of which is missing that key entirely.
+         * Because those compare equal, we don't know whether the $last one will be the one
+         * matching {c: 1} or {c: 1.5}. To make the test deterministic, we add 0.6 to
+         * whatever result we get and round it to the nearest integer. */
+        {$project: {_id: "$_id", accum: {$round: [{$add: ["$accum", 0.6]}, 0]}}}
+    ],
+    expectedOutput: [{_id: null, accum: 3.0}, {_id: 1, accum: 6}, {_id: 2, accum: 5}],
     validateExplain: (explain) => assertPlanUsesDistinctScan(explain, {a: 1, b: 1, c: 1}),
 });
 
@@ -1068,4 +956,3 @@ assertPipelineResultsAndExplain({
     expectedOutput: expectedResult,
     validateExplain: (explain) => assertPlanUsesDistinctScan(explain, {_id: 1})
 });
-}());
