@@ -2,11 +2,6 @@
  * Tests that extra-large BSON objects (>16MB) can be materialized for the '$match' stage in the
  * middle of the query plan without throwing 'BSONObjectTooLarge' exception.
  */
-(function() {
-"use strict";
-
-load("jstests/libs/analyze_plan.js");  // For 'getAggPlanStage()'.
-
 const testDB = db.getSiblingDB("jsTestName");
 assert.commandWorked(testDB.dropDatabase());
 
@@ -23,5 +18,4 @@ const pipeline = [
     {$project: {a: 1}}
 ];
 
-assert.doesNotThrow(() => coll.aggregate(pipeline).toArray());
-})();
+assert.doesNotThrow(() => coll.aggregate(pipeline).toArray());
\ No newline at end of file
