@@ -8,11 +8,12 @@
 //  assumes_read_concern_unchanged,
 //  assumes_against_mongod_not_mongos,
 //  does_not_support_repeated_reads,
+//  # Multi clients run concurrently and may modify the serverStatus metrices read in this test.
+//  multi_clients_incompatible,
+//  # oplog queries could affect the statistics
+//  assumes_standalone_mongod
 // ]
 
-(function() {
-"use strict";
-
 const testDB = db.getSiblingDB("facet_stats");
 const local = testDB.facetLookupLocal;
 const foreign = testDB.facetLookupForeign;
@@ -54,4 +55,3 @@ curScannedKeys = queryExecutor.scanned - curScannedKeys;
 assert.eq(12, curScannedObjects);
 // $facet sub-pipelines cannot make use of indexes. Hence scanned keys should be 0.
 assert.eq(0, curScannedKeys);
-})();
