@@ -5,11 +5,7 @@
  * ]
  */
 
-(function() {
-"use strict";
-load("jstests/libs/fixture_helpers.js");
-load("jstests/libs/feature_flag_util.js");    // For isEnabled.
-load("jstests/aggregation/extras/utils.js");  // For arrayEq.
+import {assertArrayEq} from "jstests/aggregation/extras/utils.js";
 
 const coll = db[jsTestName()];
 coll.drop();
@@ -62,7 +58,7 @@ assert.commandWorked(coll.insert(documents));
 pipeline = [
     {$project: {_id: 0}},
     {$densify: {field: "val", range: {step: 1, bounds: "partition"}, partitionByFields: ["part"]}},
-    {$fill: {output: {toFill: {method: "locf"}}, partitionByFields: ["part"]}}
+    {$fill: {sortBy: {val: 1}, output: {toFill: {method: "locf"}}, partitionByFields: ["part"]}},
 ];
 result = coll.aggregate(pipeline).toArray();
 
@@ -91,7 +87,7 @@ assertArrayEq({actual: result, expected: expected});
 pipeline = [
     {$project: {_id: 0}},
     {$densify: {field: "val", range: {step: 1, bounds: "full"}, partitionByFields: ["part"]}},
-    {$fill: {output: {toFill: {method: "locf"}}, partitionByFields: ["part"]}}
+    {$fill: {sortBy: {val: 1}, output: {toFill: {method: "locf"}}, partitionByFields: ["part"]}}
 ];
 result = coll.aggregate(pipeline).toArray();
 
@@ -154,4 +150,3 @@ expected = [
     {"val": 9, "toFill": 16, "possible": 8, "part": 2}
 ];
 assertArrayEq({actual: result, expected: expected});
-})();
