@@ -2,9 +2,6 @@
 // @tags: [
 //  does_not_support_repeated_reads,
 // ]
-(function() {
-"use strict";
-
 const collName = jsTest.name();
 const coll = db[collName];
 coll.drop();
@@ -52,5 +49,4 @@ for (const indexHint of [simpleGeoNearIndexName, compoundGeoNearIndexName]) {
         assert(indexAccesses.hasOwnProperty("ops"), indexStats);
         assert.eq(1, indexAccesses["ops"], indexStats);
     }
-}
-}());
+}
\ No newline at end of file
