@@ -1,11 +1,12 @@
 /**
  * Tests for $lookup with localField/foreignField syntax using indexed nested loop join algorithm.
  */
-(function() {
-"use strict";
-
-load("jstests/aggregation/sources/lookup/lookup_equijoin_semantics_lib.js");  // For runTests and
-                                                                              // runTest_*.
+import {
+    JoinAlgorithm,
+    runTest_SingleForeignRecord,
+    runTest_SingleLocalRecord,
+    runTests,
+} from "jstests/aggregation/sources/lookup/lookup_equijoin_semantics_lib.js";
 
 /**
  * Run the tests with sorted ascending/descending indexes.
@@ -148,4 +149,3 @@ runTests({
         });
     })();
 })();
-})();
