@@ -5,11 +5,9 @@
  *   requires_pipeline_optimization,
  * ]
  */
-load("jstests/aggregation/extras/utils.js");  // For anyEq.
-
-(function() {
-
-"use strict";
+import {anyEq} from "jstests/aggregation/extras/utils.js";
+import {getSingleNodeExplain} from "jstests/libs/analyze_plan.js";
+import {FixtureHelpers} from "jstests/libs/fixture_helpers.js";
 
 const testDB = db.getSiblingDB(jsTestName());
 const localColl = testDB.local_no_collation;
@@ -49,20 +47,6 @@ function setup() {
 (function testCollationPermutations() {
     setup();
 
-    // Pipeline style $lookup with cases insensitive collation.
-    const lookupWithPipeline = (foreignColl) => {
-        return {
-          $lookup: {from: foreignColl.getName(), as: "foreignMatch", _internalCollation: caseInsensitiveCollation, let: {l_id: "$_id"}, pipeline: [{$match: {$expr: {$eq: ["$_id", "$$l_id"]}}}]}
-      };
-    };
-
-    // Local-field foreign-field style $lookup with cases insensitive collation.
-    const lookupWithLocalForeignField = (foreignColl) => {
-        return {
-          $lookup: {from: foreignColl.getName(), localField: "_id", foreignField: "_id", as: "foreignMatch", _internalCollation: caseInsensitiveCollation}
-      };
-    };
-
     const resultSetCaseInsensitive = [
         {_id: "a", foreignMatch: [{_id: "a"}]},
         {_id: "b", foreignMatch: [{_id: "B"}]},
@@ -88,18 +72,14 @@ function setup() {
     //    expectedResults: Results expected from the aggregate invocation
     //
     function assertExpectedResultSet(localColl, foreignColl, commandCollation, expectedResults) {
-        const lookupWithPipeline = {
-            $lookup: {
-                from: foreignColl.getName(),
-                as: "foreignMatch",
-                let : {l_id: "$_id"},
-                pipeline: [{$match: {$expr: {$eq: ["$_id", "$$l_id"]}}}]
-            }
-        };
+        const lookupWithPipeline = {$lookup: {from: foreignColl.getName(),
+                                 as: "foreignMatch",
+                                 let: {l_id: "$_id"},
+                                 pipeline: [{$match: {$expr: {$eq: ["$_id", "$$l_id"]}}}]}};
 
-        const lookupWithLocalForeignField = {$lookup: {from: foreignColl.getName(), 
-         localField: "_id", 
-         foreignField: "_id", 
+        const lookupWithLocalForeignField = {$lookup: {from: foreignColl.getName(),
+         localField: "_id",
+         foreignField: "_id",
          as: "foreignMatch"}};
 
         const aggOptions = {};
@@ -130,5 +110,4 @@ function setup() {
             assertExpectedResultSet(local, foreign, simpleCollation, resultSetCaseSensitive);
         }
     }
-})();
-})();
+})();
\ No newline at end of file
