@@ -14,14 +14,14 @@
  *     requires_pipeline_optimization
  * ]
  */
-(function() {
-"use strict";
-
-load("jstests/libs/analyze_plan.js");         // For 'getAggPlanStages'
-load("jstests/libs/sbe_util.js");             // For checkSbeRestrictedOrFullyEnabled.
-load("jstests/libs/sbe_explain_helpers.js");  // For getSbePlanStages and
-                                              // getQueryInfoAtTopLevelOrFirstStage.
+import {getAggPlanStages} from "jstests/libs/analyze_plan.js";
+import {
+    getQueryInfoAtTopLevelOrFirstStage,
+    getSbePlanStages
+} from "jstests/libs/sbe_explain_helpers.js";
+import {checkSbeFullyEnabled, checkSbeRestrictedOrFullyEnabled} from "jstests/libs/sbe_util.js";
 
+const isSBEFullyEnabled = checkSbeFullyEnabled(db);
 const isSBELookupEnabled = checkSbeRestrictedOrFullyEnabled(db);
 const testDB = db.getSiblingDB("lookup_query_stats");
 testDB.dropDatabase();
@@ -88,27 +88,34 @@ let getCurrentQueryExecutorStats = function() {
 
 let checkExplainOutputForVerLevel = function(
     explainOutput, expected, verbosityLevel, expectedQueryPlan, withUnwind) {
-    const lkpStages = getAggPlanStages(explainOutput, "$lookup");
-
     // Only make SBE specific assertions when we know that our $lookup has been pushed down.
-    if (isSBELookupEnabled && !withUnwind) {
-        // If the SBE lookup is enabled, the $lookup stage is pushed down to the SBE and it's
+    if (isSBEFullyEnabled || (isSBELookupEnabled && !withUnwind)) {
+        // If the SBE lookup is enabled, the "$lookup" stage is pushed down to the SBE and it's
         // not visible in 'stages' field of the explain output. Instead, 'queryPlan.stage' must be
-        // "EQ_LOOKUP".
-        assert.eq(lkpStages.length, 0, lkpStages, explainOutput);
+        // "EQ_LOOKUP" or "EQ_LOOKUP_UNWIND".
+        let lkpStages = getAggPlanStages(explainOutput, "EQ_LOOKUP", true);
+        if (lkpStages.length == 0) {
+            lkpStages = getAggPlanStages(explainOutput, "EQ_LOOKUP_UNWIND", true);
+        }
+        assert.eq(lkpStages.length, 1, lkpStages);
+        const lkpStage = lkpStages[0];
+
         const queryInfo = getQueryInfoAtTopLevelOrFirstStage(explainOutput);
         const planner = queryInfo.queryPlanner;
         assert(planner.hasOwnProperty("winningPlan") &&
                    planner.winningPlan.hasOwnProperty("queryPlan"),
                explainOutput);
         const plan = planner.winningPlan.queryPlan;
-        assert(plan.hasOwnProperty("stage") && plan.stage == "EQ_LOOKUP", explainOutput);
+
+        assert(lkpStage.hasOwnProperty("stage"), lkpStage);
+        assert(lkpStage.stage == "EQ_LOOKUP" || lkpStage.stage == "EQ_LOOKUP_UNWIND", lkpStage);
         assert(expectedQueryPlan.hasOwnProperty("strategy"), expectedQueryPlan);
-        assert(plan.hasOwnProperty("strategy") && plan.strategy == expectedQueryPlan.strategy,
-               explainOutput);
+        assert(
+            lkpStage.hasOwnProperty("strategy") && lkpStage.strategy == expectedQueryPlan.strategy,
+            lkpStage);
         if (expectedQueryPlan.strategy == "IndexedLoopJoin") {
-            assert(plan.hasOwnProperty("indexName"), expectedQueryPlan);
-            assert.eq(plan.indexName, expectedQueryPlan.indexName, explainOutput);
+            assert(lkpStage.hasOwnProperty("indexName"), lkpStage);
+            assert.eq(lkpStage.indexName, expectedQueryPlan.indexName);
         }
 
         const expectedTopLevelJoinStage =
@@ -140,6 +147,7 @@ let checkExplainOutputForVerLevel = function(
             assert.eq(sbeNljStages.length, 0, explainOutput);
         }
     } else {
+        const lkpStages = getAggPlanStages(explainOutput, "$lookup");
         assert.eq(lkpStages.length, 1, lkpStages);
         const lkpStage = lkpStages[0];
         assert.eq(
@@ -224,7 +232,7 @@ let testQueryExecutorStatsWithCollectionScan = function(params) {
     // There is no index in the collection.
     assert.eq(0, curScannedKeys);
 
-    if (isSBELookupEnabled && !params.withUnwind) {
+    if (isSBEFullyEnabled || (isSBELookupEnabled && !params.withUnwind)) {
         checkExplainOutputForAllVerbosityLevels(
             localColl,
             fromColl,
@@ -347,7 +355,7 @@ let testQueryExecutorStatsWithIndexScan = function(params) {
     // collection.
     assert.eq(localDocCount, curScannedKeys);
 
-    if (isSBELookupEnabled && !params.withUnwind) {
+    if (isSBEFullyEnabled || (isSBELookupEnabled && !params.withUnwind)) {
         checkExplainOutputForAllVerbosityLevels(
             localColl,
             fromColl,
@@ -396,6 +404,10 @@ insertDocumentToCollection(localColl, localDocCount, "localField");
 // lastScannedObjects and lastScannedKeys with existing stats values in that case.
 getCurrentQueryExecutorStats();
 
+////////////////////////////////////////////////////////////////////////////////////////////////////
+// TESTS
+////////////////////////////////////////////////////////////////////////////////////////////////////
+
 testQueryExecutorStatsWithCollectionScan({withUnwind: false});
 testQueryExecutorStatsWithHashLookup();
 testQueryExecutorStatsWithIndexScan({withUnwind: false});
@@ -403,5 +415,4 @@ testQueryExecutorStatsWithIndexScan({withUnwind: false});
 // Now test $lookup including an $unwind of the output field. This should result in the unwind
 // taking place within the lookup stage.
 testQueryExecutorStatsWithCollectionScan({withUnwind: true});
-testQueryExecutorStatsWithIndexScan({withUnwind: true});
-}());
+testQueryExecutorStatsWithIndexScan({withUnwind: true});
\ No newline at end of file
