@@ -4,12 +4,7 @@
  *   requires_fcv_52,
  * ]
  */
-(function() {
-"use strict";
-
-load("jstests/aggregation/extras/window_function_helpers.js");
-load("jstests/aggregation/extras/utils.js");  // For arrayEq.
-load("jstests/libs/feature_flag_util.js");    // For isEnabled.
+import "jstests/libs/sbe_assert_error_override.js";
 
 const coll = db.linear_fill;
 coll.drop();
@@ -53,6 +48,60 @@ let expected = [
 ];
 assert.eq(result, expected);
 
+coll.drop();
+collection = [
+    {_id: 0, val: 0},
+    {_id: 1},
+    {_id: 2},
+    {_id: 9},
+    {_id: 10, val: 10},
+];
+assert.commandWorked(coll.insert(collection));
+
+result = coll.aggregate([{
+                 $setWindowFields: {
+                     sortBy: {_id: 1},
+                     output: {val: {$linearFill: "$val"}},
+                 }
+             }])
+             .toArray();
+
+expected = [
+    {_id: 0, val: 0},
+    {_id: 1, val: 1},
+    {_id: 2, val: 2},
+    {_id: 9, val: 9},
+    {_id: 10, val: 10},
+];
+assert.eq(result, expected);
+
+coll.drop();
+collection = [
+    {_id: 0},
+    {_id: 1},
+    {_id: 2},
+    {_id: 9},
+    {_id: 10},
+];
+assert.commandWorked(coll.insert(collection));
+
+result = coll.aggregate([{
+                 $setWindowFields: {
+                     sortBy: {_id: 1},
+                     output: {val: {$linearFill: "$val"}},
+                 }
+             }])
+             .toArray();
+
+expected = [
+    {_id: 0, val: null},
+    {_id: 1, val: null},
+    {_id: 2, val: null},
+    {_id: 9, val: null},
+    {_id: 10, val: null},
+];
+assert.eq(result, expected);
+
 coll.drop();
 collection = [
     {_id: 0, val: 0},
@@ -516,5 +565,4 @@ assert.commandFailedWithCode(db.runCommand({
     ],
     cursor: {}
 }),
-                             ErrorCodes.TypeMismatch);
-})();
+                             ErrorCodes.TypeMismatch);
\ No newline at end of file
