@@ -6,9 +6,6 @@
  *   do_not_wrap_aggregations_in_facets,
  * ]
  */
-(function() {
-"use strict";
-
 assert.commandWorked(db[jsTestName()].insert({dummy: 1}));
 
 let windowResults = db.aggregate([
@@ -26,5 +23,4 @@ windowResults = db.aggregate([
     {$setWindowFields: {sortBy: {_id: 1}, output: {'obj.subPath': {$last: 1}}}}
 ]);
 
-assert.eq(windowResults.toArray(), [{_id: 0, obj: {k1: "v1", k2: "v2", subPath: 1}}]);
-})();
+assert.eq(windowResults.toArray(), [{_id: 0, obj: {k1: "v1", k2: "v2", subPath: 1}}]);
\ No newline at end of file
