@@ -6,9 +6,6 @@
  * - When something is not allowed, what error message do we expect?
  */
 
-(function() {
-"use strict";
-
 const coll = db.setWindowFields_parse;
 coll.drop();
 
@@ -35,7 +32,8 @@ assert.commandWorked(
     run({$setWindowFields: {partitionBy: "$$myobj.a", output: {}}}, {let : {myobj: {a: 456}}}));
 
 // Test that parsing fails for unrecognized parameters.
-assert.commandFailedWithCode(run({$setWindowFields: {what_is_this: 1}}), 40415);
+assert.commandFailedWithCode(run({$setWindowFields: {what_is_this: 1}}),
+                             ErrorCodes.IDLUnknownField);
 
 // Test for a successful parse, ignoring the response documents.
 assert.commandWorked(run({
@@ -258,4 +256,3 @@ assert.commandWorked(run({$setWindowFields: {output: {v: {$max: {mergeObjects: {
 err = assert.commandFailedWithCode(
     run({$setWindowFields: {output: {a: {$sum: 1}, 'a.b': {$sum: 1}}}}), 6307900);
 assert.includes(err.errmsg, 'specification contains two conflicting paths');
-})();
