@@ -1,10 +1,7 @@
 /**
  * Test time-based window bounds.
  */
-(function() {
-"use strict";
-
-load("jstests/aggregation/extras/window_function_helpers.js");
+import "jstests/libs/sbe_assert_error_override.js";
 
 const coll = db.setWindowFields_time;
 coll.drop();
@@ -288,4 +285,3 @@ const res3 =
 
 assert.sameMembers(res3,
                    [{"res": 4}, {"res": 8}, {"res": 8}, {"res": 12}, {"res": 12}, {"res": 12}]);
-})();
