@@ -4,9 +4,6 @@
  *
  * @tags: [assumes_no_implicit_collection_creation_after_drop]
  */
-(function() {
-"use strict";
-
 const testDBName = jsTestName();
 const testDB = db.getSiblingDB(testDBName);
 const siblingDBName = jsTestName() + "_otherDB";
@@ -56,5 +53,4 @@ for (const writeStage of [mergeStage, outStage]) {
     siblingDB[collidingCollName].drop();
 }
 
-siblingDB.dropDatabase();
-})();
+siblingDB.dropDatabase();
\ No newline at end of file
