@@ -1,9 +1,6 @@
 // Test for auth counters in serverStatus.
 // @tags: [requires_replication]
 
-(function() {
-'use strict';
-
 const keyfile = 'jstests/libs/key1';
 const badKeyfile = 'jstests/libs/key2';
 let replTest = new ReplSetTest({nodes: 1, keyFile: keyfile, nodeOptions: {auth: ""}});
@@ -133,5 +130,4 @@ const finalStats =
     assert.commandWorked(admin.runCommand({serverStatus: 1})).security.authentication.mechanisms;
 replTest.stopSet();
 
-printjson(finalStats);
-})();
+printjson(finalStats);
\ No newline at end of file
