@@ -1,10 +1,7 @@
 // Test for auth counters in serverStatus.
 
-(function() {
-'use strict';
-
-let expectedSuccessLogs;
-let expectedFailureLogs;
+let expectedSuccessLogs = 0;
+let expectedFailureLogs = 0;
 
 function authnSuccessIncrementsServerStatusTotalAuthTime(mongodRunner) {
     jsTest.log(
@@ -53,29 +50,6 @@ function authnFailureIncrementsServerStatusTotalAuthTime(mongodRunner) {
     admin.logout();
 }
 
-function newConnectionWithoutAuthn(mongodRunner, connectionHealthLoggingOn) {
-    jsTest.log(
-        "============================ newConnectionWithoutAuthn ============================");
-
-    // Create a new connection without authentication.
-    new Mongo(mongodRunner.host);
-
-    if (connectionHealthLoggingOn) {
-        assert.soon(() => checkLog.checkContainsWithCountJson(
-                        mongodRunner,
-                        10483900,
-                        {"doc": {"application": {"name": "MongoDB Shell"}}},
-                        2,  // 1 for the mongodRunner connection and 1 for the new connection above
-                        null,
-                        true),
-                    "Did not find expected 1 metadata log entry for non-auth connection");
-    } else {
-        assert.eq(checkLog.checkContainsOnceJson(mongodRunner, 10483900, {}),
-                  false,
-                  "Expected not to find metadata log entry for non-auth connection");
-    }
-}
-
 // Test that a successful authentication is logged correctly
 function authnSuccessLogsMetricsReportWithSuccessStatus(mongodRunner, connectionHealthLoggingOn) {
     jsTest.log(
@@ -95,17 +69,16 @@ function authnSuccessLogsMetricsReportWithSuccessStatus(mongodRunner, connection
                 5286306,
                 {
                     "result": 0,
-                    "metrics": {"conversation_duration": {"summary": [{"step": 1}, {"step": 2}]}},
-                    "doc": {"application": {"name": "MongoDB Shell"}}
+                    "metrics": {"conversation_duration": {"summary": [{"step": 1}, {"step": 2}]}}
                 },
                 expectedSuccessLogs,
                 null,
                 true),
-            "Did not find expected 1 successful log entry");
+            "Did not find expected 1 successful metric log entries");
     } else {
-        assert.eq(checkLog.checkContainsOnceJson(mongodRunner, 5286306, {}),
+        assert.eq(checkLog.checkContainsWithCountJson(mongodRunner, 5286306, {}),
                   false,
-                  "Expected not to find log entry");
+                  "Expected not to find metric log entries");
     }
 
     admin.logout();
@@ -121,22 +94,19 @@ function authnFailureLogsMetricsReportWithFailedStatus(mongodRunner, connectionH
     expectedFailureLogs += 1;
 
     if (connectionHealthLoggingOn) {
-        assert.soon(() => checkLog.checkContainsWithCountJson(
-                        mongodRunner,
-                        5286307,
-                        {
-                            "result": 18,
-                            "metrics": {"conversation_duration": {"summary": [{"step": 1}]}},
-                            "doc": {"application": {"name": "MongoDB Shell"}}
-                        },
-                        expectedFailureLogs,
-                        null,
-                        true),
-                    "Did not find expected 1 failure log entry");
+        assert.soon(
+            () => checkLog.checkContainsWithCountJson(
+                mongodRunner,
+                5286307,
+                {"result": 18, "metrics": {"conversation_duration": {"summary": [{"step": 1}]}}},
+                expectedFailureLogs,
+                null,
+                true),
+            "Did not find expected 1 failure metric log entries");
     } else {
-        assert.eq(checkLog.checkContainsOnceJson(mongodRunner, 5286307, {}),
+        assert.eq(checkLog.checkContainsWithCountJson(mongodRunner, 5286307, {}),
                   false,
-                  "Expected not to find failure log entry");
+                  "Expected not to find failure metric log entries");
     }
 
     admin.logout();
@@ -163,17 +133,16 @@ function multipleAuthnSuccessLogsMultipleCorrectReports(mongodRunner, connection
                 5286306,
                 {
                     "result": 0,
-                    "metrics": {"conversation_duration": {"summary": [{"step": 1}, {"step": 2}]}},
-                    "doc": {"application": {"name": "MongoDB Shell"}}
+                    "metrics": {"conversation_duration": {"summary": [{"step": 1}, {"step": 2}]}}
                 },
                 expectedSuccessLogs,
                 null,
                 true),
-            "Did not find expected 2 successful log entries");
+            "Did not find expected 2 successful metric log entries");
     } else {
-        assert.eq(checkLog.checkContainsOnceJson(mongodRunner, 5286306, {}),
+        assert.eq(checkLog.checkContainsWithCountJson(mongodRunner, 5286306, {}),
                   false,
-                  "Expected not to find successful log entries");
+                  "Expected not to find successful metric log entries");
     }
 }
 
@@ -190,22 +159,19 @@ function multiAuthnFailureLogsMetricsReportWithFailedStatus(mongodRunner,
     expectedFailureLogs += 2;
 
     if (connectionHealthLoggingOn) {
-        assert.soon(() => checkLog.checkContainsWithCountJson(
-                        mongodRunner,
-                        5286307,
-                        {
-                            "result": 18,
-                            "metrics": {"conversation_duration": {"summary": [{"step": 1}]}},
-                            "doc": {"application": {"name": "MongoDB Shell"}}
-                        },
-                        expectedFailureLogs,
-                        null,
-                        true),
-                    "Did not find expected 2 failure log entries");
+        assert.soon(
+            () => checkLog.checkContainsWithCountJson(
+                mongodRunner,
+                5286307,
+                {"result": 18, "metrics": {"conversation_duration": {"summary": [{"step": 1}]}}},
+                expectedFailureLogs,
+                null,
+                true),
+            "Did not find expected 2 failure metric log entries");
     } else {
-        assert.eq(checkLog.checkContainsOnceJson(mongodRunner, 5286307, {}),
+        assert.eq(checkLog.checkContainsWithCountJson(mongodRunner, 5286307, {}),
                   false,
-                  "Expected not to find failure log entries");
+                  "Expected not to find failure metric log entries");
     }
 
     admin.logout();
@@ -231,17 +197,16 @@ function multipleAuthnMixedLogsMultipleCorrectReports(mongodRunner, connectionHe
                 5286306,
                 {
                     "result": 0,
-                    "metrics": {"conversation_duration": {"summary": [{"step": 1}, {"step": 2}]}},
-                    "doc": {"application": {"name": "MongoDB Shell"}}
+                    "metrics": {"conversation_duration": {"summary": [{"step": 1}, {"step": 2}]}}
                 },
                 expectedSuccessLogs,
                 null,
                 true),
-            "Did not find expected 1 successful log entry");
+            "Did not find expected 1 successful metric log entries");
     } else {
-        assert.eq(checkLog.checkContainsOnceJson(mongodRunner, 5286306, {}),
+        assert.eq(checkLog.checkContainsWithCountJson(mongodRunner, 5286306, {}),
                   false,
-                  "Expected not to find successful log entry");
+                  "Expected not to find successful metric log entries");
     }
 
     expectedFailureLogs += 1;
@@ -249,22 +214,19 @@ function multipleAuthnMixedLogsMultipleCorrectReports(mongodRunner, connectionHe
     admin.auth('admin', 'wrong');
 
     if (connectionHealthLoggingOn) {
-        assert.soon(() => checkLog.checkContainsWithCountJson(
-                        mongodRunner,
-                        5286307,
-                        {
-                            "result": 18,
-                            "metrics": {"conversation_duration": {"summary": [{"step": 1}]}},
-                            "doc": {"application": {"name": "MongoDB Shell"}}
-                        },
-                        expectedFailureLogs,
-                        null,
-                        true),
-                    "Did not find expected 1 failure log entry");
+        assert.soon(
+            () => checkLog.checkContainsWithCountJson(
+                mongodRunner,
+                5286307,
+                {"result": 18, "metrics": {"conversation_duration": {"summary": [{"step": 1}]}}},
+                expectedFailureLogs,
+                null,
+                true),
+            "Did not find expected 1 failure metric log entries");
     } else {
-        assert.eq(checkLog.checkContainsOnceJson(mongodRunner, 5286307, {}),
+        assert.eq(checkLog.checkContainsWithCountJson(mongodRunner, 5286307, {}),
                   false,
-                  "Expected not to find failure log entry");
+                  "Expected not to find failure metric log entries");
     }
 }
 
@@ -272,14 +234,10 @@ let runTest = (connectionHealthLoggingOn) => {
     const mongod = MongoRunner.runMongod(
         {setParameter: {enableDetailedConnectionHealthMetricLogLines: connectionHealthLoggingOn}});
 
-    expectedSuccessLogs = 0;
-    expectedFailureLogs = 0;
-
     try {
         mongod.getDB("admin").createUser(
             {user: 'admin', pwd: 'pwd', roles: ['root'], mechanisms: ['SCRAM-SHA-256']});
 
-        newConnectionWithoutAuthn(mongod, connectionHealthLoggingOn);
         authnSuccessLogsMetricsReportWithSuccessStatus(mongod, connectionHealthLoggingOn);
         authnFailureLogsMetricsReportWithFailedStatus(mongod, connectionHealthLoggingOn);
         multiAuthnFailureLogsMetricsReportWithFailedStatus(mongod, connectionHealthLoggingOn);
@@ -296,5 +254,4 @@ let runTest = (connectionHealthLoggingOn) => {
 
 // Parameterized on enabling/disabling connection health logging.
 runTest(true);
-runTest(false);
-})();
+runTest(false);
\ No newline at end of file
