@@ -1,7 +1,4 @@
 // Test db.auth with password prompt
-(function() {
-'use strict';
-
 // Cannot run on windows because of a workaround for runProgram for Windows.
 if (!_isWindows()) {
     const conn = MongoRunner.runMongod();
@@ -17,7 +14,7 @@ if (!_isWindows()) {
     const mongo = 'mongo';
     const host = conn.host;
     const port = conn.port;
-    const ret = runProgram(
+    runProgram(
         binshell, '-c', `echo password | ${mongo} --host ${host} --port ${port} --eval ${auth}`);
 
     assert.soon(() => {
@@ -27,4 +24,3 @@ if (!_isWindows()) {
 
     MongoRunner.stopMongod(conn);
 }
-}());
