@@ -1,10 +1,10 @@
 /**
  * This tests that the user cache is invalidated after any changes are made to system collections
+ * @tags: [
+ *   requires_fcv_72,
+ * ]
  */
 
-(function() {
-"use strict";
-
 const conn = MongoRunner.runMongod({auth: ''});
 let db = conn.getDB('admin');
 
@@ -75,4 +75,3 @@ db.logout();
 })();
 
 MongoRunner.stopMongod(conn);
-})();
