@@ -2,9 +2,6 @@
  * This test checks that cluster IP allowlists can be set and respected.
  */
 
-(function() {
-'use strict';
-
 print("When allowlist is empty, the server does not start.");
 assert.throws(() => MongoRunner.runMongod(
                   {auth: null, keyFile: "jstests/libs/key1", clusterIpSourceAllowlist: ""}),
@@ -148,5 +145,4 @@ testIpAllowlist(
     "192.0.2.0/24",
     false);
 
-testIpAllowlistRuntimeGarbage();
-}());
+testIpAllowlistRuntimeGarbage();
\ No newline at end of file
