@@ -5,10 +5,8 @@
  * that aren't part of the global "roles" array.
  */
 
-(function() {
-'use strict';
-
-load('jstests/auth/lib/commands_builtin_roles.js');
+import {roles} from "jstests/auth/lib/commands_builtin_roles.js";
+import {authCommandsLib} from "jstests/auth/lib/commands_lib.js";
 
 function checkForNonExistentRoles() {
     const tests = authCommandsLib.tests;
@@ -33,4 +31,3 @@ function checkForNonExistentRoles() {
 }
 
 checkForNonExistentRoles();
-})();
