@@ -15,7 +15,12 @@ const dbPath = MongoRunner.toRealDir("$dataDir/commands_built_in_roles_sharded/"
 mkdir(dbPath);
 const opts = {
     auth: "",
-    setParameter: "trafficRecordingDirectory=" + dbPath
+    setParameter: {
+        trafficRecordingDirectory: dbPath,
+        mongotHost: "localhost:27017",  // We have to set the mongotHost parameter for the
+                                        // $search-related tests to pass configuration checks.
+        syncdelay: 0  // Disable checkpoints as this can cause some commands to fail transiently.
+    }
 };
 // run all tests sharded
 const conn = new ShardingTest({
@@ -23,8 +28,13 @@ const conn = new ShardingTest({
     mongos: 1,
     config: 1,
     keyFile: "jstests/libs/key1",
-    other:
-        {shardOptions: opts, mongosOptions: {setParameter: "trafficRecordingDirectory=" + dbPath}}
+    other: {
+        shardOptions: opts,
+        // We have to set the mongotHost parameter for the $search-related tests to pass
+        // configuration checks.
+        mongosOptions:
+            {setParameter: {trafficRecordingDirectory: dbPath, mongotHost: "localhost:27017"}}
+    }
 });
 runAllCommandsBuiltinRoles(conn);
 conn.stop();
