@@ -13,7 +13,12 @@ const dbPath = MongoRunner.toRealDir("$dataDir/commands_built_in_roles_standalon
 mkdir(dbPath);
 const opts = {
     auth: "",
-    setParameter: "trafficRecordingDirectory=" + dbPath
+    setParameter: {
+        trafficRecordingDirectory: dbPath,
+        mongotHost: "localhost:27017",  // We have to set the mongotHost parameter for the
+                                        // $search-relatead tests to pass configuration checks.
+        syncdelay: 0  // Disable checkpoints as this can cause some commands to fail transiently.
+    }
 };
 
 // run all tests standalone
