@@ -15,8 +15,7 @@ import {
     authErrCode,
     commandNotSupportedCode
 } from "jstests/auth/lib/commands_lib.js";
-
-load("jstests/libs/fail_point_util.js");
+import {configureFailPoint} from "jstests/libs/fail_point_util.js";
 
 // This test involves killing all sessions, which will not work as expected if the kill command is
 // sent with an implicit session.
@@ -100,11 +99,15 @@ function runOneTest(conn, t) {
 
     // Some tests requires mongot, however, setting this failpoint will make search queries to
     // return EOF, that way all the hassle of setting it up can be avoided.
-    let disableSearchFailpoint;
+    let disableSearchFailpointShard, disableSearchFailpointRouter;
     if (t.disableSearch) {
-        disableSearchFailpoint = configureFailPoint(conn.rs0 ? conn.rs0.getPrimary() : conn,
-                                                    'searchReturnEofImmediately');
+        disableSearchFailpointShard = configureFailPoint(conn.rs0 ? conn.rs0.getPrimary() : conn,
+                                                         'searchReturnEofImmediately');
+        if (conn.s) {
+            disableSearchFailpointRouter = configureFailPoint(conn.s, 'searchReturnEofImmediately');
+        }
     }
+
     for (let i = 0; i < t.testcases.length; i++) {
         const testcase = t.testcases[i];
         if (!("privileges" in testcase)) {
@@ -193,8 +196,12 @@ function runOneTest(conn, t) {
         }
     }
 
-    if (disableSearchFailpoint) {
-        disableSearchFailpoint.off();
+    if (disableSearchFailpointShard) {
+        disableSearchFailpointShard.off();
+    }
+
+    if (disableSearchFailpointRouter) {
+        disableSearchFailpointRouter.off();
     }
 
     return failures;
@@ -217,12 +224,16 @@ function createUsers(conn) {
 }
 
 const opts = {
-    auth: ""
+    auth: "",
+    setParameter: {
+        mongotHost: "localhost:27017",  // We have to set the mongotHost parameter for the
+                                        // $search-related tests to pass configuration checks.
+        syncdelay: 0  // Disable checkpoints as this can cause some commands to fail transiently.
+    }
 };
 const impls = {
     createUsers: createUsers,
-    runOneTest: runOneTest,
-    getSideChannel: conn => conn.sidechannel,
+    runOneTest: runOneTest
 };
 
 // run all tests standalone
@@ -243,7 +254,9 @@ const impls = {
         mongos: 1,
         config: 1,
         keyFile: "jstests/libs/key1",
-        other: {shardOptions: opts}
+        // We have to set the mongotHost parameter for the $search-related tests to pass
+        // configuration checks.
+        other: {shardOptions: opts, mongosOptions: {setParameter: {mongotHost: "localhost:27017"}}}
     });
     conn.sidechannel = new Mongo(conn.s0.host);
     authCommandsLib.runTests(conn, impls);
