@@ -2,10 +2,7 @@
  * Test that sessions on sharded clusters cannot be resumed by deleted and recreated user.
  * @tags: [requires_sharding]
  */
-(function() {
-'use strict';
-
-load("jstests/auth/deleted_recreated_user_base.js");
+import {kInvalidationIntervalSecs, runTest} from "jstests/auth/deleted_recreated_user_base.js";
 
 const st = new ShardingTest({
     shards: 1,
@@ -19,5 +16,4 @@ const st = new ShardingTest({
     },
 });
 runTest(st.s0, st.s1);
-st.stop();
-})();
+st.stop();
\ No newline at end of file
