@@ -1,16 +1,10 @@
 // Tests that a user can only run a getMore on a cursor that they created.
 // @tags: [requires_sharding]
-(function() {
-"use strict";
-
 // Multiple users cannot be authenticated on one connection within a session.
 TestData.disableImplicitSessions = true;
 
 function runTest(conn) {
     const adminDB = conn.getDB("admin");
-    const hello = adminDB.runCommand("hello");
-    assert.commandWorked(hello);
-    const isMongos = (hello.msg === "isdbgrid");
 
     // Create the admin user.
     assert.commandWorked(adminDB.runCommand({createUser: "admin", pwd: "admin", roles: ["root"]}));
@@ -263,5 +257,4 @@ MongoRunner.stopMongod(conn);
 const cluster = new ShardingTest(
     {shards: 1, mongos: 1, keyFile: "jstests/libs/key1", other: {shardOptions: {auth: ""}}});
 runTest(cluster);
-cluster.stop();
-}());
+cluster.stop();
\ No newline at end of file
