@@ -7,9 +7,6 @@
 //
 //       These transitions are tested for $where and MapReduce.
 
-(function() {
-'use strict';
-
 const conn = MongoRunner.runMongod();
 const test = conn.getDB("test");
 
@@ -102,5 +99,4 @@ function testMapReduce() {
 testMapReduce();
 testMapReduce();
 
-MongoRunner.stopMongod(conn);
-})();
+MongoRunner.stopMongod(conn);
\ No newline at end of file
