@@ -1,8 +1,5 @@
 // Test the killCursors command.
 // @tags: [requires_sharding]
-(function() {
-'use strict';
-
 // Multiple users cannot be authenticated on one connection within a session.
 TestData.disableImplicitSessions = true;
 
@@ -130,5 +127,4 @@ MongoRunner.stopMongod(mongod);
 const st =
     new ShardingTest({shards: 1, mongos: 1, config: 1, other: {keyFile: 'jstests/libs/key1'}});
 runTest(st.s0);
-st.stop();
-})();
+st.stop();
\ No newline at end of file
