@@ -7,10 +7,8 @@
  * @tags: [requires_sharding]
  */
 
-(function() {
-'use strict';
-
-load("jstests/libs/fixture_helpers.js");  // For isMongos.
+import {configureFailPoint} from "jstests/libs/fail_point_util.js";
+import {FixtureHelpers} from "jstests/libs/fixture_helpers.js";
 
 function runTest(m, failPointName) {
     var db = m.getDB("foo");
@@ -65,7 +63,7 @@ function runTest(m, failPointName) {
 
     jsTestLog("Starting long-running operation");
     db.auth('reader', 'reader');
-    assert.commandWorked(db.adminCommand({configureFailPoint: failPointName, mode: "alwaysOn"}));
+    const fp = configureFailPoint(m, failPointName);
     var s1 = startParallelShell(queryAsReader, m.port);
     jsTestLog("Finding ops in $currentOp output");
     var o = [];
@@ -92,7 +90,7 @@ function runTest(m, failPointName) {
     db.auth('reader', 'reader');
     assert.commandWorked(db.killOp(o[0]));
     checkLog.contains(db, '"msg":"Successful killOp"');
-    assert.commandWorked(db.adminCommand({configureFailPoint: failPointName, mode: "off"}));
+    fp.off();
 
     jsTestLog("Waiting for ops to terminate");
     var exitCode = s1({checkExitSuccess: false});
@@ -106,7 +104,7 @@ function runTest(m, failPointName) {
     assert.lt(diff, 30000, "Start: " + start + "; end: " + end + "; diff: " + diff);
 
     jsTestLog("Starting a second long-running operation");
-    assert.commandWorked(db.adminCommand({configureFailPoint: failPointName, mode: "alwaysOn"}));
+    const fp2 = configureFailPoint(m, failPointName);
     var s2 = startParallelShell(queryAsReader, m.port);
     jsTestLog("Finding ops in $currentOp output");
     var o2 = [];
@@ -130,16 +128,16 @@ function runTest(m, failPointName) {
     assert.eq([], ops());
 
     jsTestLog("Checking that an administrative user can kill others' operations");
-    var start = new Date();
+    start = new Date();
     assert.commandWorked(db.killOp(o2[0]));
-    assert.commandWorked(db.adminCommand({configureFailPoint: failPointName, mode: "off"}));
+    fp2.off();
     jsTestLog("Waiting for ops to terminate");
-    var exitCode = s2({checkExitSuccess: false});
+    exitCode = s2({checkExitSuccess: false});
     assert.neq(
         0, exitCode, "expected shell to exit abnormally due to JS execution being terminated");
 
-    var end = new Date();
-    var diff = end - start;
+    end = new Date();
+    diff = end - start;
     assert.lt(diff, 30000, "Start: " + start + "; end: " + end + "; diff: " + diff);
 }
 
@@ -152,4 +150,3 @@ var st = new ShardingTest({shards: 1, keyFile: 'jstests/libs/key1'});
 // setYieldAlllocksHang failpoint.
 runTest(st.s, "waitInFindBeforeMakingBatch");
 st.stop();
-})();
