@@ -12,8 +12,7 @@ import {
     commandNotSupportedCode,
     firstDbName
 } from "jstests/auth/lib/commands_lib.js";
-
-load("jstests/libs/fail_point_util.js");
+import {configureFailPoint} from "jstests/libs/fail_point_util.js";
 
 // This test involves killing all sessions, which will not work as expected if the kill command is
 // sent with an implicit session.
@@ -98,15 +97,18 @@ function testProperAuthorization(conn, t, testcase, r) {
  * First of two entry points for this test library.
  * To be invoked as an test argument to authCommandsLib.runTests().
  */
-export function runOneTest(conn, t) {
+function runOneTest(conn, t) {
     var failures = [];
 
     // Some tests requires mongot, however, setting this failpoint will make search queries to
     // return EOF, that way all the hassle of setting it up can be avoided.
-    let disableSearchFailpoint;
+    let disableSearchFailpointShard, disableSearchFailpointRouter;
     if (t.disableSearch) {
-        disableSearchFailpoint = configureFailPoint(conn.rs0 ? conn.rs0.getPrimary() : conn,
-                                                    'searchReturnEofImmediately');
+        disableSearchFailpointShard = configureFailPoint(conn.rs0 ? conn.rs0.getPrimary() : conn,
+                                                         'searchReturnEofImmediately');
+        if (conn.s) {
+            disableSearchFailpointRouter = configureFailPoint(conn.s, 'searchReturnEofImmediately');
+        }
     }
 
     for (var i = 0; i < t.testcases.length; i++) {
@@ -122,10 +124,13 @@ export function runOneTest(conn, t) {
         }
     }
 
-    if (disableSearchFailpoint) {
-        disableSearchFailpoint.off();
+    if (disableSearchFailpointShard) {
+        disableSearchFailpointShard.off();
     }
 
+    if (disableSearchFailpointRouter) {
+        disableSearchFailpointRouter.off();
+    }
     return failures;
 }
 
@@ -133,7 +138,7 @@ export function runOneTest(conn, t) {
  * Second entry point for this test library.
  * To be invoked as an test argument to authCommandsLib.runTests().
  */
-export function createUsers(conn) {
+function createUsers(conn) {
     var adminDb = conn.getDB(adminDbName);
     adminDb.createUser({user: "admin", pwd: "password", roles: ["__system"]});
 
