@@ -1,12 +1,8 @@
 /*
  * Make sure that listCommands on mongods doesn't require authentication.
  */
-(function() {
-'use strict';
-
-load("jstests/auth/listcommands_preauth_base.js");
+import {runTest} from "jstests/auth/listcommands_preauth_base.js";
 
 const mongod = MongoRunner.runMongod({auth: ""});
 runTest(mongod);
-MongoRunner.stopMongod(mongod);
-})();
\ No newline at end of file
+MongoRunner.stopMongod(mongod);
\ No newline at end of file
