@@ -1,8 +1,9 @@
-(function() {
-'use strict';
-
 // SERVER-8623: Test that renameCollection can't be used to bypass auth checks on system
 // namespaces
+// @tags: [
+//   requires_fcv_72,
+// ]
+
 const conn = MongoRunner.runMongod({auth: ""});
 
 const adminDB = conn.getDB("admin");
@@ -117,4 +118,3 @@ assert.eq(ErrorCodes.IllegalOperation, res.code);
 // At this point, all the user documents are gone, so further activity may be unauthorized,
 // depending on cluster configuration.  So, this is the end of the test.
 MongoRunner.stopMongod(conn, {user: 'userAdmin', pwd: 'password'});
-})();
