@@ -6,11 +6,6 @@
  * requires_fcv_61,
  * ]
  */
-load("jstests/fle2/libs/encrypted_client_util.js");
-
-(function() {
-'use strict';
-
 function runTestWithAuth(conn, allowsRename, verifyFunction) {
     const db = conn.getDB("test");
     const srcDbName = 'rename_encrypted_collection_src_db';
@@ -35,8 +30,6 @@ function runTestWithAuth(conn, allowsRename, verifyFunction) {
     const srcEncryptedErrmsg = "Cannot rename an encrypted collection";
     const tgtEncryptedErrmsg = "Cannot rename to an existing encrypted collection";
 
-    const adminDB = conn.getDB("admin");
-
     assert.commandWorked(
         dbSrc.createCollection("encrypted", {encryptedFields: sampleEncryptedFields}));
     assert.commandWorked(dbSrc.createCollection("unencrypted"));
@@ -126,4 +119,3 @@ jsTestLog("Sharding: Testing fle2 collection rename");
 
     st.stop();
 }
-}());
