@@ -1,8 +1,5 @@
 // Test for saslStart invoked with invalid/missing mechanism.
 
-(function() {
-'use strict';
-
 const mongod = MongoRunner.runMongod({auth: ''});
 const admin = mongod.getDB('admin');
 
@@ -42,8 +39,6 @@ function saslStartFailure(mechanism, expectedCode) {
 saslStartFailure('scram-sha-1', ErrorCodes.MechanismUnavailable);
 saslStartFailure('MONGODB-CR', ErrorCodes.MechanismUnavailable);
 
-const kIDLParserMissingRequiredField = 40414;
-saslStartFailure(undefined, kIDLParserMissingRequiredField);
+saslStartFailure(undefined, ErrorCodes.IDLFailedToParse);
 
-MongoRunner.stopMongod(mongod);
-})();
+MongoRunner.stopMongod(mongod);
\ No newline at end of file
