@@ -2,9 +2,6 @@
  * Tests that the __system user can auth using SCRAM-SHA-256
  * @tags: [requires_replication]
  */
-(function() {
-'use strict';
-
 const keyfile = 'jstests/libs/key1';
 const keyfileContents = cat(keyfile).replace(/[\011-\015\040]/g, '');
 const rs = new ReplSetTest({nodes: 3, keyFile: keyfile});
@@ -20,5 +17,4 @@ db.logout();
 jsTestLog("Testing scram-sha-1");
 assert.eq(db.auth({mechanism: 'SCRAM-SHA-1', user: '__system', pwd: keyfileContents}), 0);
 
-rs.stopSet();
-})();
+rs.stopSet();
\ No newline at end of file
