@@ -2,12 +2,8 @@
 // Multiversion incompatible because this was backported, and is not in the previous version yet.
 // @tags: [requires_replication,does_not_support_stepdowns,multiversion_incompatible]
 
-(function() {
-'use strict';
-
-// Pull in Thread.
-load('jstests/libs/parallelTester.js');
-load("jstests/libs/fail_point_util.js");
+import {configureFailPoint} from "jstests/libs/fail_point_util.js";
+import {Thread} from "jstests/libs/parallelTester.js";
 
 const testUser = "user1";
 const testDB = "user_cache_invalidation";
@@ -223,9 +219,13 @@ function runTest(writeNode, readNode, awaitReplication) {
     for (let i = 0; i < cfg.members.length; ++i) {
         cfg.members[i].priority = i ? 0 : 1;
     }
-    rst.initiate(cfg);
+    rst.initiateWithHighElectionTimeout(cfg);
     rst.awaitSecondaryNodes();
 
+    // Freeze secondaries to avoid surprise stepdowns.
+    rst.getSecondaries().forEach(rst.freeze);
+    rst.awaitReplication();
+
     // Now identify the permanent primary and secondary we'll use.
     const primary = rst.getPrimary();
     const secondary = rst.getSecondaries()[0];
@@ -233,4 +233,3 @@ function runTest(writeNode, readNode, awaitReplication) {
     runTest(primary, secondary, () => rst.awaitReplication());
     rst.stopSet();
 }
-})();
