@@ -5,14 +5,8 @@
  *   requires_fcv_60,
  * ]
  */
-(function() {
-"use strict";
-
-load('jstests/libs/collection_drop_recreate.js');  // For 'assertDropAndRecreateCollection' and
-                                                   // 'assertDropCollection'.
-load('jstests/libs/change_stream_util.js');        // For 'ChangeStreamTest' and
-                                                   // 'assertChangeStreamEventEq'.
-load("jstests/libs/fixture_helpers.js");
+import {ChangeStreamTest} from "jstests/libs/change_stream_util.js";
+import {FixtureHelpers} from "jstests/libs/fixture_helpers.js";
 
 const testDB = db.getSiblingDB(jsTestName());
 
@@ -138,7 +132,6 @@ function runTest(startChangeStream) {
             arr.push(key[property]);
         }
         let indexName = arr.join("_");
-        let opDesc = {indexes: [Object.assign({v: 2, key: key, name: indexName}, options)]};
 
         // Create an index.
         assert.commandWorked(testDB[collName].createIndex(key, options));
@@ -247,4 +240,3 @@ runTest((() => cst.startWatchingChanges({pipeline, collection: 1})));
 
 // Run the test using a single collection change stream.
 runTest((() => cst.startWatchingChanges({pipeline, collection: collName})));
-}());
