@@ -2,38 +2,23 @@
  * Tests the behaviour of the 'fullDocumentBeforeChange' argument to the $changeStream stage.
  *
  * @tags: [
- *   requires_fcv_60,
  *   assumes_unsharded_collection,
  *   do_not_wrap_aggregations_in_facets,
  *   uses_multiple_connections,
  * ]
  */
-(function() {
-"use strict";
-
-load("jstests/libs/change_stream_util.js");        // For ChangeStreamTest and
-                                                   // isChangeStreamPreAndPostImagesEnabled.
-load("jstests/libs/collection_drop_recreate.js");  // For assert[Drop|Create]Collection.
-load("jstests/libs/fixture_helpers.js");           // For FixtureHelpers.
-
-const canRecordPreImagesInConfigDb = isChangeStreamPreAndPostImagesEnabled(db);
-
-if (!canRecordPreImagesInConfigDb && FixtureHelpers.isMongos(db)) {
-    jsTestLog("Skipping test as pre image lookup is not supported in sharded cluster with feature" +
-              "flag 'featureFlagChangeStreamPreAndPostImages' disabled.");
-    return;
-}
+import {assertChangeStreamEventEq, ChangeStreamTest} from "jstests/libs/change_stream_util.js";
+import {
+    assertDropAndRecreateCollection,
+    assertDropCollection
+} from "jstests/libs/collection_drop_recreate.js";
 
 const coll = assertDropAndRecreateCollection(db, "change_stream_pre_images");
 const cst = new ChangeStreamTest(db);
 
 // Enable pre-image recording on the test collection.
-if (!canRecordPreImagesInConfigDb) {
-    assert.commandWorked(db.runCommand({collMod: coll.getName(), recordPreImages: true}));
-} else {
-    assert.commandWorked(
-        db.runCommand({collMod: coll.getName(), changeStreamPreAndPostImages: {enabled: true}}));
-}
+assert.commandWorked(
+    db.runCommand({collMod: coll.getName(), changeStreamPreAndPostImages: {enabled: true}}));
 
 // Open three streams on the collection, one for each "fullDocumentBeforeChange" mode.
 const csNoPreImages = cst.startWatchingChanges({
@@ -57,7 +42,7 @@ assert.commandWorked(coll.insert({_id: "x"}));
 let latestChange = cst.getOneChange(csNoPreImages);
 assert.eq(latestChange.operationType, "insert");
 assert(!latestChange.hasOwnProperty("fullDocumentBeforeChange"));
-assert.docEq(latestChange.fullDocument, {_id: "x"});
+assert.docEq({_id: "x"}, latestChange.fullDocument);
 assert.docEq(latestChange, cst.getOneChange(csPreImageWhenAvailableCursor));
 assert.docEq(latestChange, cst.getOneChange(csPreImageRequiredCursor));
 
@@ -66,7 +51,7 @@ assert.commandWorked(coll.update({_id: "x"}, {foo: "bar"}));
 latestChange = cst.getOneChange(csNoPreImages);
 assert.eq(latestChange.operationType, "replace");
 assert(!latestChange.hasOwnProperty("fullDocumentBeforeChange"));
-assert.docEq(latestChange.fullDocument, {_id: "x", foo: "bar"});
+assert.docEq({_id: "x", foo: "bar"}, latestChange.fullDocument);
 // Add the expected "fullDocumentBeforeChange" and confirm that both pre-image cursors see it.
 latestChange.fullDocumentBeforeChange = {
     _id: "x"
@@ -110,19 +95,15 @@ assert.docEq(latestChange, cst.getOneChange(csPreImageWhenAvailableCursor));
 assert.docEq(latestChange, cst.getOneChange(csPreImageRequiredCursor));
 
 // Now disable pre-image generation on the test collection and re-test.
-if (!canRecordPreImagesInConfigDb) {
-    assert.commandWorked(db.runCommand({collMod: coll.getName(), recordPreImages: false}));
-} else {
-    assert.commandWorked(
-        db.runCommand({collMod: coll.getName(), changeStreamPreAndPostImages: {enabled: false}}));
-}
+assert.commandWorked(
+    db.runCommand({collMod: coll.getName(), changeStreamPreAndPostImages: {enabled: false}}));
 
 // Test pre-image lookup for an insertion. No pre-image exists on any cursor.
 assert.commandWorked(coll.insert({_id: "y"}));
 latestChange = cst.getOneChange(csNoPreImages);
 assert.eq(latestChange.operationType, "insert");
 assert(!latestChange.hasOwnProperty("fullDocumentBeforeChange"));
-assert.docEq(latestChange.fullDocument, {_id: "y"});
+assert.docEq({_id: "y"}, latestChange.fullDocument);
 assert.docEq(latestChange, cst.getOneChange(csPreImageWhenAvailableCursor));
 assert.docEq(latestChange, cst.getOneChange(csPreImageRequiredCursor));
 
@@ -131,7 +112,7 @@ assert.commandWorked(coll.update({_id: "y"}, {foo: "bar"}));
 latestChange = cst.getOneChange(csNoPreImages);
 assert.eq(latestChange.operationType, "replace");
 assert(!latestChange.hasOwnProperty("fullDocumentBeforeChange"));
-assert.docEq(latestChange.fullDocument, {_id: "y", foo: "bar"});
+assert.docEq({_id: "y", foo: "bar"}, latestChange.fullDocument);
 
 // Add the expected "fullDocumentBeforeChange" and confirm that pre-image is not present.
 latestChange.fullDocumentBeforeChange = null;
@@ -140,7 +121,7 @@ latestChange.fullDocumentBeforeChange = null;
 assert.docEq(latestChange, cst.getOneChange(csPreImageWhenAvailableCursor));
 // ... but the "required" cursor throws an exception.
 assert.throwsWithCode(() => cst.getOneChange(csPreImageRequiredCursor),
-                      [ErrorCodes.NoMatchingDocument, 51770]);
+                      ErrorCodes.NoMatchingDocument);
 
 // Test pre-image lookup for an op-style update operation.
 assert.commandWorked(coll.update({_id: "y"}, {$set: {foo: "baz"}}));
@@ -175,5 +156,4 @@ latestChange.fullDocumentBeforeChange = null;
 // The "whenAvailable" cursor returns an event without the pre-image.
 assert.docEq(latestChange, cst.getOneChange(csPreImageWhenAvailableCursor));
 
-assertDropCollection(db, coll.getName());
-})();
+assertDropCollection(db, coll.getName());
\ No newline at end of file
