@@ -11,12 +11,9 @@
  * ]
  */
 
-(function() {
-"use strict";
+import {assertChangeStreamEventEq, ChangeStreamTest} from "jstests/libs/change_stream_util.js";
+import {assertDropCollection} from "jstests/libs/collection_drop_recreate.js";
 
-load("jstests/libs/collection_drop_recreate.js");  // For assertDropCollection.
-load('jstests/libs/change_stream_util.js');        // For 'ChangeStreamTest' and
-                                                   // 'assertChangeStreamEventEq'.
 const dbName = jsTestName();
 const collName = "test";
 const collNS = dbName + "." + collName;
@@ -127,8 +124,7 @@ function validateExpectedEventAndConfirmResumability(collParam, expectedOutput)
     });
 }
 
-assert.commandWorked(db.adminCommand({enableSharding: dbName}));
-assert.commandWorked(st.s.adminCommand({movePrimary: dbName, to: st.shard0.shardName}));
+assert.commandWorked(db.adminCommand({enableSharding: dbName, primaryShard: st.shard0.shardName}));
 
 // Test the behaviour of migrateLastChunkFromShard for a single-collection stream
 validateExpectedEventAndConfirmResumability(collName, {
@@ -152,4 +148,3 @@ validateExpectedEventAndConfirmResumability(1, {
 validateShowSystemEventsFalse();
 
 st.stop();
-}());
