@@ -2,18 +2,18 @@
  * Tests that change streams correctly handle rewrites of null, existence and equality checks, for
  * both existent and non-existent fields and subfields.
  * @tags: [
- *   featureFlagChangeStreamsRewrite,
  *   requires_fcv_51,
  *   requires_pipeline_optimization,
  *   uses_change_streams
  * ]
  */
-(function() {
-"use strict";
+import {
+    generateChangeStreamWriteWorkload,
+    getAllChangeStreamEvents,
+    isPlainObject
+} from "jstests/libs/change_stream_rewrite_util.js";
 
-load("jstests/libs/change_stream_rewrite_util.js");  // For rewrite helpers.
-
-const dbName = "change_stream_rewrite_null_existence_test";
+const dbName = jsTestName();
 const collName = "coll1";
 const testDB = db.getSiblingDB(dbName);
 
@@ -81,14 +81,6 @@ function traverseEvent(event, outputMap, prefixPath = "") {
     }
 }
 
-// Ensure all events are replicated and visible to the change stream.
-assert.soon(() => {
-    const allEvents = getAllChangeStreamEvents(
-        testDB, [], {fullDocument: "updateLookup", showExpandedEvents: true}, startPoint);
-    const ns = {db: dbName, coll: "view"};
-    return allEvents.some(event => event.operationType == "drop" && friendlyEqual(event.ns, ns));
-});
-
 // Obtain a list of all events that occurred during the write workload.
 const allEvents = getAllChangeStreamEvents(
     testDB, [], {fullDocument: "updateLookup", showExpandedEvents: true}, startPoint);
@@ -219,4 +211,3 @@ for (let csConfig of [{fullDocument: "updateLookup", showExpandedEvents: true}])
 
 // Assert that there were no failed test cases.
 assert(failedTestCases.length == 0, failedTestCases);
-})();
