@@ -3,10 +3,12 @@
  * nss by default and does an additional collection UUID check when
  * 'matchCollectionUUIDForUpdateLookup: true' option is set.
  */
-load("jstests/libs/change_stream_util.js");        // For 'ChangeStreamTest'.
-load("jstests/libs/collection_drop_recreate.js");  // For 'assertDropCollection',
-                                                   //     'assertCreateCollection.
-load("jstests/libs/fixture_helpers.js");           // For 'FixtureHelpers'.
+import {ChangeStreamTest} from "jstests/libs/change_stream_util.js";
+import {
+    assertCreateCollection,
+    assertDropCollection
+} from "jstests/libs/collection_drop_recreate.js";
+import {FixtureHelpers} from "jstests/libs/fixture_helpers.js";
 
 function assertUpdateLookupResultBeforeOp(op, changeStreamOptions) {
     jsTest.log(`Running change stream with update lookup test after '${op}' occurred with ${
@@ -78,6 +80,7 @@ function assertUpdateLookupResultBeforeOp(op, changeStreamOptions) {
         assert.commandWorked(db.adminCommand({
             shardCollection: db.getCollection(collNameA).getFullName(),
             key: {_id: 1},
+            numInitialChunks: 2
         }));
         expectedFullDocument = updatedDocInCollA;
     } else if (op === 'reshardCollection') {
