@@ -1,14 +1,12 @@
-'use strict';
-
 /**
  * Represents a MongoDB cluster.
  */
-load('jstests/hooks/validate_collections.js');          // For validateCollections.
-load('jstests/concurrency/fsm_libs/shard_fixture.js');  // For FSMShardingTest.
+import {FSMShardingTest} from "jstests/concurrency/fsm_libs/shard_fixture.js";
+import {validateCollections} from "jstests/hooks/validate_collections.js";
 
-var Cluster = function(options) {
+export const Cluster = function(clusterOptions, sessionOptions) {
     if (!(this instanceof Cluster)) {
-        return new Cluster(options);
+        return new Cluster(clusterOptions);
     }
 
     function getObjectKeys(obj) {
@@ -160,6 +158,7 @@ var Cluster = function(options) {
     }
 
     var conn;
+    var session;
     var secondaryConns;
 
     var st;
@@ -172,25 +171,25 @@ var Cluster = function(options) {
     var replSets = [];
     var rst;
 
-    validateClusterOptions(options);
-    Object.freeze(options);
+    validateClusterOptions(clusterOptions);
+    Object.freeze(clusterOptions);
 
-    this.setup = function setup() {
+    this.setup = async function setup() {
         var verbosityLevel = 0;
 
         if (initialized) {
             throw new Error('cluster has already been initialized');
         }
 
-        if (options.sharded.enabled) {
+        if (clusterOptions.sharded.enabled) {
             st = new FSMShardingTest(`mongodb://${db.getMongo().host}`);
 
             conn = st.s(0);  // First mongos
 
             this.teardown = function teardown() {
-                options.teardownFunctions.mongod.forEach(this.executeOnMongodNodes);
-                options.teardownFunctions.mongos.forEach(this.executeOnMongosNodes);
-                options.teardownFunctions.config.forEach(this.executeOnConfigNodes);
+                clusterOptions.teardownFunctions.mongod.forEach(this.executeOnMongodNodes);
+                clusterOptions.teardownFunctions.mongos.forEach(this.executeOnMongosNodes);
+                clusterOptions.teardownFunctions.config.forEach(this.executeOnConfigNodes);
             };
 
             this.reestablishConnectionsAfterFailover = function() {
@@ -217,28 +216,12 @@ var Cluster = function(options) {
 
             i = 0;
             while (st.rs(i)) {
-                var rs = st.rs(i++);
+                const rs = st.rs(i++);
                 this._addReplicaSetConns(rs);
                 replSets.push(rs);
             }
 
-            if (options.sharded.enableBalancer === true) {
-                st._configServers.forEach((conn) => {
-                    const configDb = conn.getDB('admin');
-
-                    configDb.adminCommand({
-                        configureFailPoint: 'balancerShouldReturnRandomMigrations',
-                        mode: 'alwaysOn'
-                    });
-                    configDb.adminCommand({
-                        configureFailPoint: 'overrideBalanceRoundInterval',
-                        mode: 'alwaysOn',
-                        data: {intervalMs: 100}
-                    });
-                });
-            }
-
-        } else if (options.replication.enabled) {
+        } else if (clusterOptions.replication.enabled) {
             rst = new ReplSetTest(db.getMongo().host);
 
             conn = rst.getPrimary();
@@ -246,7 +229,7 @@ var Cluster = function(options) {
             replSets = [rst];
 
             this.teardown = function teardown() {
-                options.teardownFunctions.mongod.forEach(this.executeOnMongodNodes);
+                clusterOptions.teardownFunctions.mongod.forEach(this.executeOnMongodNodes);
             };
 
             this._addReplicaSetConns(rst);
@@ -258,13 +241,29 @@ var Cluster = function(options) {
             _conns.mongod = [conn];
         }
 
+        if (typeof sessionOptions !== 'undefined') {
+            session = conn.startSession(sessionOptions);
+            const readPreference = session.getOptions().getReadPreference();
+            if (readPreference && readPreference.mode === 'secondary') {
+                // Unset the explicit read preference so set_read_preference_secondary.js can do
+                // the right thing based on the DB.
+                session.getOptions().setReadPreference(undefined);
+
+                // We import set_read_preference_secondary.js in order to avoid running
+                // commands against the "admin" and "config" databases via mongos with
+                // readPreference={mode: "secondary"} when there's only a single node in
+                // the CSRS.
+                await import("jstests/libs/override_methods/set_read_preference_secondary.js");
+            }
+        }
+
         initialized = true;
         clusterStartTime = new Date();
 
-        options.setupFunctions.mongod.forEach(this.executeOnMongodNodes);
-        options.setupFunctions.config.forEach(this.executeOnConfigNodes);
-        if (options.sharded) {
-            options.setupFunctions.mongos.forEach(this.executeOnMongosNodes);
+        clusterOptions.setupFunctions.mongod.forEach(this.executeOnMongodNodes);
+        clusterOptions.setupFunctions.config.forEach(this.executeOnConfigNodes);
+        if (clusterOptions.sharded) {
+            clusterOptions.setupFunctions.mongos.forEach(this.executeOnMongosNodes);
         }
     };
 
@@ -281,9 +280,22 @@ var Cluster = function(options) {
         if (!fn || typeof (fn) !== 'function' || fn.length !== 1) {
             throw new Error('mongod function must be a function that takes a db as an argument');
         }
-        _conns.mongod.forEach(function(mongodConn) {
-            fn(mongodConn.getDB('admin'));
-        });
+
+        if (TestData.shardsAddedRemoved) {
+            _conns.mongod.forEach(function(mongodConn) {
+                try {
+                    fn(mongodConn.getDB('admin'));
+                } catch (e) {
+                    print(
+                        "Error executing function on mongod nodes, with addShard/removeShard running in background: " +
+                        tojson(e));
+                }
+            });
+        } else {
+            _conns.mongod.forEach(function(mongodConn) {
+                fn(mongodConn.getDB('admin'));
+            });
+        }
     };
 
     this.executeOnMongosNodes = function executeOnMongosNodes(fn) {
@@ -328,11 +340,14 @@ var Cluster = function(options) {
 
     this.teardown = function teardown() {
         assert(initialized, 'cluster must be initialized first');
-        options.teardownFunctions.mongod.forEach(this.executeOnMongodNodes);
+        clusterOptions.teardownFunctions.mongod.forEach(this.executeOnMongodNodes);
     };
 
     this.getDB = function getDB(dbName) {
         assert(initialized, 'cluster must be initialized first');
+        if (session) {
+            return session.getDatabase(dbName);
+        }
         return conn.getDB(dbName);
     };
 
@@ -364,19 +379,19 @@ var Cluster = function(options) {
 
     this.getReplSetNumNodes = function getReplSetNumNodes() {
         assert(this.isReplication() && !this.isSharded(), 'cluster must be a replica set');
-        return options.replication.numNodes;
+        return clusterOptions.replication.numNodes;
     };
 
     this.isSharded = function isSharded() {
-        return Cluster.isSharded(options);
+        return Cluster.isSharded(clusterOptions);
     };
 
     this.isReplication = function isReplication() {
-        return Cluster.isReplication(options);
+        return Cluster.isReplication(clusterOptions);
     };
 
     this.isStandalone = function isStandalone() {
-        return Cluster.isStandalone(options);
+        return Cluster.isStandalone(clusterOptions);
     };
 
     this.shardCollection = function shardCollection() {
@@ -443,7 +458,10 @@ var Cluster = function(options) {
         i = 0;
         var shard = st.shard(0);
         while (shard) {
-            if (shard.name.includes('/')) {
+            if (TestData.shardsAddedRemoved && shard.shardName === "config") {
+                // Skip the config shard if it's transitioning in and out of being a shard to avoid
+                // tests that use manual shard operations failing with ShardNotFound.
+            } else if (shard.name.includes('/')) {
                 // If the shard is a replica set, the format of st.shard(0).name in ShardingTest is
                 // "test-rs0/localhost:20006,localhost:20007,localhost:20008".
                 var [_, shards] = shard.name.split('/');
@@ -466,13 +484,12 @@ var Cluster = function(options) {
     };
 
     this.isBalancerEnabled = function isBalancerEnabled() {
-        return this.isSharded() && options.sharded.enableBalancer;
+        return this.isSharded() && clusterOptions.sharded.enableBalancer;
     };
 
     this.validateAllCollections = function validateAllCollections(phase) {
         assert(initialized, 'cluster must be initialized first');
 
-        const isSteppingDownConfigServers = this.isSteppingDownConfigServers();
         var _validateCollections = function _validateCollections(db, isMongos = false) {
             // Validate all the collections on each node.
             var res = db.adminCommand({listDatabases: 1});
@@ -510,7 +527,6 @@ var Cluster = function(options) {
 
         replSets.forEach(rst => {
             var startTime = Date.now();
-            var res;
             var primary = rst.getPrimary();
 
             if (shouldCheckDBHashes) {
@@ -542,37 +558,6 @@ var Cluster = function(options) {
         });
     };
 
-    this.recordConfigServerData = function recordConfigServerData(configServer) {
-        assert(initialized, 'cluster must be initialized first');
-        assert(this.isSharded(), 'cluster is not sharded');
-
-        var data = {};
-        var configDB = configServer.getDB('config');
-
-        // We record the contents of the 'lockpings' and 'locks' collections to make it easier to
-        // debug issues with distributed locks in the sharded cluster.
-        // TODO SERVER-68551: remove once 7.0 becomes last-lts
-        data.lockpings = configDB.lockpings.find({ping: {$gte: clusterStartTime}}).toArray();
-
-        // We suppress some fields from the result set to reduce the amount of data recorded.
-        // TODO SERVER-68551: remove once 7.0 becomes last-lts
-        data.locks =
-            configDB.locks.find({when: {$gte: clusterStartTime}}, {process: 0, ts: 0}).toArray();
-
-        return data;
-    };
-
-    this.recordAllConfigServerData = function recordAllConfigServerData() {
-        assert(initialized, 'cluster must be initialized first');
-        assert(this.isSharded(), 'cluster is not sharded');
-
-        var data = {};
-        st._configServers.forEach(config =>
-                                      (data[config.host] = this.recordConfigServerData(config)));
-
-        return data;
-    };
-
     this.isRunningWiredTigerLSM = function isRunningWiredTigerLSM() {
         var adminDB = this.getDB('admin');
 
@@ -596,7 +581,7 @@ var Cluster = function(options) {
     };
 
     this.shouldPerformContinuousStepdowns = function shouldPerformContinuousStepdowns() {
-        return this.isSharded() && (typeof options.sharded.stepdownOptions !== 'undefined');
+        return this.isSharded() && (typeof clusterOptions.sharded.stepdownOptions !== 'undefined');
     };
 
     /*
@@ -617,12 +602,12 @@ var Cluster = function(options) {
 
     this.isSteppingDownConfigServers = function isSteppingDownConfigServers() {
         return this.shouldPerformContinuousStepdowns() &&
-            options.sharded.stepdownOptions.configStepdown;
+            clusterOptions.sharded.stepdownOptions.configStepdown;
     };
 
     this.isSteppingDownShards = function isSteppingDownShards() {
         return this.shouldPerformContinuousStepdowns() &&
-            options.sharded.stepdownOptions.shardStepdown;
+            clusterOptions.sharded.stepdownOptions.shardStepdown;
     };
 
     this.awaitReplication = () => {
