@@ -1,8 +1,7 @@
-(function() {
-'use strict';
-
-load('jstests/concurrency/fsm_libs/runner.js');  // for runner.internals
-load('jstests/libs/discover_topology.js');       // For Topology and DiscoverTopology.
+import {Cluster} from "jstests/concurrency/fsm_libs/cluster.js";
+import {runner} from "jstests/concurrency/fsm_libs/runner.js";
+import {ThreadManager} from "jstests/concurrency/fsm_libs/thread_mgr.js";
+import {DiscoverTopology, Topology} from "jstests/libs/discover_topology.js";
 
 const validateExecutionOptions = runner.internals.validateExecutionOptions;
 const prepareCollections = runner.internals.prepareCollections;
@@ -16,8 +15,6 @@ const loadWorkloadContext = runner.internals.loadWorkloadContext;
 
 // Returns true if the workload's teardown succeeds and false if the workload's teardown fails.
 function cleanupWorkload(workload, context, cluster, errors, header) {
-    const phase = 'before workload ' + workload + ' teardown';
-
     try {
         teardownWorkload(workload, context, cluster);
     } catch (e) {
@@ -28,39 +25,24 @@ function cleanupWorkload(workload, context, cluster, errors, header) {
     return true;
 }
 
-function runWorkloads(workloads,
-                      {cluster: clusterOptions = {}, execution: executionOptions = {}} = {}) {
+async function runWorkloads(workloads,
+                            {cluster: clusterOptions = {}, execution: executionOptions = {}} = {}) {
     assert.gt(workloads.length, 0, 'need at least one workload to run');
 
     const executionMode = {serial: true};
     validateExecutionOptions(executionMode, executionOptions);
     Object.freeze(executionOptions);  // immutable after validation (and normalization)
 
-    // Determine how strong to make assertions while simultaneously executing different
-    // workloads.
-    let assertLevel = AssertLevel.OWN_DB;
-    if (clusterOptions.sameDB) {
-        // The database is shared by multiple workloads, so only make the asserts that apply
-        // when the collection is owned by an individual workload.
-        assertLevel = AssertLevel.OWN_COLL;
-    }
-    if (clusterOptions.sameCollection) {
-        // The collection is shared by multiple workloads, so only make the asserts that always
-        // apply.
-        assertLevel = AssertLevel.ALWAYS;
-    }
-    globalAssertLevel = assertLevel;
-
     const context = {};
     const applyMultipliers = true;
-    loadWorkloadContext(workloads, context, executionOptions, applyMultipliers);
+    await loadWorkloadContext(workloads, context, executionOptions, applyMultipliers);
 
     // Constructing a Cluster instance calls its internal validateClusterOptions() function,
     // which fills in any properties that aren't explicitly present in 'clusterOptions'. We do
     // this before constructing a ThreadManager instance to make its dependency on the
     // 'clusterOptions' being filled in explicit.
-    const cluster = new Cluster(clusterOptions);
-    const threadMgr = new ThreadManager(clusterOptions, executionMode);
+    const cluster = new Cluster(clusterOptions, executionOptions.sessionOptions);
+    const threadMgr = new ThreadManager(clusterOptions);
 
     Random.setRandomSeed(clusterOptions.seed);
 
@@ -70,7 +52,13 @@ function runWorkloads(workloads,
     let startTime = Date.now();  // Initialize in case setupWorkload fails below.
     let totalTime;
 
-    cluster.setup();
+    await cluster.setup();
+
+    if (typeof executionOptions.tenantId !== 'undefined') {
+        // Import simulate_atlas_proxy.js to override requests for tenant during preparing
+        // collections, setup and teardown of workloads.
+        await import("jstests/libs/override_methods/simulate_atlas_proxy.js");
+    }
 
     jsTest.log('Workload(s) started: ' + workloads.join(' '));
 
@@ -104,17 +92,9 @@ function runWorkloads(workloads,
             cluster.awaitReplication();
         }
 
-        // Transactions run at snapshot read concern unless defaultTransactionReadConcernLevel
-        // is set to another level.
-        const transactionsWouldUseSnapshotReadConcern =
-            !TestData.hasOwnProperty("defaultTransactionReadConcernLevel") ||
-            TestData.defaultTransactionReadConcernLevel === "snapshot";
-
-        // Synchronize the cluster times across all routers if the tests will be overriden to
-        // use transactions, so the earliest global snapshots chosen by each router will include
-        // the effects of each setup function. This is only required for snapshot read concern.
-        if (cluster.isSharded() && TestData.runInsideTransaction &&
-            transactionsWouldUseSnapshotReadConcern) {
+        // Synchronize the cluster times across all routers, so the earliest global snapshots chosen
+        // by each router will include the effects of each setup function.
+        if (cluster.isSharded()) {
             cluster.synchronizeMongosClusterTimes();
         }
 
@@ -269,7 +249,8 @@ if (TestData.runningWithConfigStepdowns || TestData.runningWithShardStepdowns) {
 }
 
 const executionOptions = {
-    dbNamePrefix: TestData.dbNamePrefix || ""
+    dbNamePrefix: TestData.dbNamePrefix || "",
+    tenantId: TestData.tenantId
 };
 const resmokeDbPathPrefix = TestData.resmokeDbPathPrefix || ".";
 
@@ -287,5 +268,4 @@ if (Object.keys(sessionOptions).length > 0 || TestData.runningWithSessions) {
     executionOptions.sessionOptions = sessionOptions;
 }
 
-runWorkloads(workloads, {cluster: clusterOptions, execution: executionOptions});
-})();
+await runWorkloads(workloads, {cluster: clusterOptions, execution: executionOptions});
