@@ -1,16 +1,14 @@
-'use strict';
-
 /**
  * auth_create_role.js
  *
  * Repeatedly creates new roles on a database.
  */
-load('jstests/concurrency/fsm_workload_helpers/drop_utils.js');  // for dropRoles
+import {dropRoles} from "jstests/concurrency/fsm_workload_helpers/drop_utils.js";
 
 // UMC commands are not supported in transactions.
 TestData.runInsideTransaction = false;
 
-var $config = (function() {
+export const $config = (function() {
     var data = {
         // Use the workload name as a prefix for the role name,
         // since the workload name is assumed to be unique.
@@ -55,9 +53,9 @@ var $config = (function() {
             for (var i = 0; i < this.num; ++i) {
                 var name = uniqueRoleName(this.prefix, this.tid, i);
                 var res = db.getRole(name);
-                assertAlways(res !== null, "role '" + name + "' should exist");
-                assertAlways.eq(name, res.role);
-                assertAlways(!res.isBuiltin, 'role should be user-defined');
+                assert(res !== null, "role '" + name + "' should exist");
+                assert.eq(name, res.role);
+                assert(!res.isBuiltin, 'role should be user-defined');
             }
         }
 
