@@ -1,5 +1,3 @@
-'use strict';
-
 // UMC commands are not supported in transactions.
 TestData.runInsideTransaction = false;
 
@@ -10,7 +8,8 @@ TestData.runInsideTransaction = false;
  * drops the user from the database.
  * @tags: [incompatible_with_concurrency_simultaneous]
  */
-var $config = (function() {
+
+export const $config = (function() {
     var data = {
         // Use the workload name as a prefix for the username,
         // since the workload name is assumed to be unique.
@@ -47,9 +46,9 @@ var $config = (function() {
                 kCreateUserRetryInterval);
 
             const res = db.getUser(username);
-            assertAlways(res !== null, "user '" + username + "' should exist");
-            assertAlways.eq(username, res.user);
-            assertAlways.eq(db.getName(), res.db);
+            assert(res !== null, "user '" + username + "' should exist");
+            assert.eq(username, res.user);
+            assert.eq(db.getName(), res.db);
 
             const kDropUserRetries = 5;
             const kDropUserRetryInterval = 5 * 1000;
@@ -62,7 +61,7 @@ var $config = (function() {
                     return false;
                 }
             }, "Failed dropping user '" + username + "'", kDropUserRetries, kDropUserRetryInterval);
-            assertAlways.isnull(db.getUser(username), "user '" + username + "' should not exist");
+            assert.isnull(db.getUser(username), "user '" + username + "' should not exist");
         }
 
         return {init: init, createAndDropUser: createAndDropUser};
