@@ -1,18 +1,18 @@
-'use strict';
-
 /**
  * auth_privilege_cache_miss.js
  *
  * Validate user permission consistency during cache miss and slow load.
  *
- * @tags: [incompatible_with_concurrency_simultaneous]
+ * @tags: [incompatible_with_concurrency_simultaneous,multiversion_incompatible]
  */
 
 // Use the auth_privilege_consistency workload as a base.
-load('jstests/concurrency/fsm_libs/extend_workload.js');
-load('jstests/concurrency/fsm_workloads/auth_privilege_consistency.js');
+import {extendWorkload} from "jstests/concurrency/fsm_libs/extend_workload.js";
+import {
+    $config as $baseConfig
+} from "jstests/concurrency/fsm_workloads/auth_privilege_consistency.js";
 
-var $config = extendWorkload($config, function($config, $super) {
+export const $config = extendWorkload($baseConfig, function($config, $super) {
     // Override setup() to also set cache-miss and slow load failpoints.
     const kResolveRolesDelayMS = 100;
 
@@ -26,7 +26,7 @@ var $config = extendWorkload($config, function($config, $super) {
 
         const cacheBypass = {configureFailPoint: 'authUserCacheBypass', mode: 'alwaysOn'};
         const getUser = {
-            configureFailPoint: 'authLocalGetUser',
+            configureFailPoint: 'authLocalGetSubRoles',
             mode: 'alwaysOn',
             data: {resolveRolesDelayMS: NumberInt(kResolveRolesDelayMS)}
         };
@@ -43,7 +43,7 @@ var $config = extendWorkload($config, function($config, $super) {
 
     $config.teardown = function(db, collName, cluster) {
         const cacheBypass = {configureFailPoint: 'authUserCacheBypass', mode: 'off'};
-        const getUser = {configureFailPoint: 'authLocalGetUser', mode: 'off'};
+        const getUser = {configureFailPoint: 'authLocalGetSubRoles', mode: 'off'};
 
         cluster.executeOnMongosNodes(function(nodeAdminDB) {
             assert.commandWorked(nodeAdminDB.runCommand(cacheBypass));
