@@ -1,18 +1,18 @@
-'use strict';
-
 /**
  * Tests running operations with 'collectionUUID' parameter while the collection is being renamed
  * concurrently, and makes sure that all operations will succeed eventually when using the correct
  * 'collectionUUID' obtained from 'CollectionUUIDMismatch' error.
  * @tags: [
+ *   # balancer may move unsplittable collections and change the uuid
+ *   assumes_balancer_off,
  *   # This test just performs rename operations that can't be executed in transactions.
  *   does_not_support_transactions,
  *   requires_non_retryable_writes,
  *   requires_fcv_60,
  *  ]
  */
-load("jstests/libs/fixture_helpers.js");  // For isMongos.
-load("jstests/libs/namespace_utils.js");  // For getCollectionNameFromFullNamespace.
+import {isMongos} from "jstests/concurrency/fsm_workload_helpers/server_types.js";
+import {getCollectionNameFromFullNamespace} from "jstests/libs/namespace_utils.js";
 
 const otherDbName = "otherDb";
 const otherDbCollName = "otherDbColl";
@@ -89,6 +89,14 @@ const runCommandInLoop = function(
             continue;
         }
 
+        if (cmdResult.code === ErrorCodes.OplogOperationUnsupported &&
+            cmdResult.errmsg.includes("Command not supported during resharding") &&
+            cmdResult.errmsg.includes("commitIndexBuild")) {
+            // TODO (SERVER-91708): Resharding should consider queued index builds waiting for
+            // active number of index builds to be below the threshold.
+            continue;
+        }
+
         if (cmdResult.code === ErrorCodes.CollectionUUIDMismatch) {
             if (expectedNonRetryableErrors.includes(ErrorCodes.CollectionUUIDMismatch)) {
                 break;
@@ -133,7 +141,7 @@ const verifyFailingWithCollectionUUIDMismatch = function(
     assert.eq(res.actualCollection, actualCollection);
 };
 
-const testCommand = function(
+export const testCommand = function(
     db, namespace, cmdName, cmdObj, data, expectedNonRetryableErrors = []) {
     verifyFailingWithCollectionUUIDMismatch(
         db, cmdName, cmdObj, data.sameDbCollUUID, sameDbCollName, namespace, data);
@@ -152,7 +160,7 @@ const testCommand = function(
     runCommandInLoop(db, namespace, cmdName, cmdObj, data, expectedNonRetryableErrors);
 };
 
-var $config = (function() {
+export const $config = (function() {
     const data = {};
 
     const states = (function() {
@@ -242,33 +250,43 @@ var $config = (function() {
     })();
 
     const setup = function(db, collName, cluster) {
+        if (isMongos(db)) {
+            db.getSiblingDB(otherDbName).dropDatabase();
+            db[sameDbCollName].drop();
+            const shardNames = Object.keys(cluster.getSerializedCluster().shards);
+            const numShards = shardNames.length;
+            let otherDbShard;
+
+            if (numShards > 1) {
+                const currDb = db.getSiblingDB('config')['databases'].findOne({_id: db.getName()});
+                shardNames.some((shard) => {
+                    if (shard != currDb.primary) {
+                        otherDbShard = shard;
+                        return false;
+                    }
+                    return true;
+                });
+            } else {
+                otherDbShard = shardNames[0];
+            }
+            assert.commandWorked(
+                db.adminCommand({enableSharding: otherDbName, primaryShard: otherDbShard}));
+        } else {
+            db[sameDbCollName].drop();
+            db.getSiblingDB(otherDbName)[otherDbCollName].drop();
+        }
+
         // Add documents with wide range '_id' to end up with the data distributed across multiple
         // shards in the sharded scenario.
         for (let i = 0; i < this.threadCount; ++i) {
             for (let j = 0; j < 100; ++j) {
                 const uniqueNum = i * 100 + j;
-                assertAlways.commandWorked(db[collName].insert(
+                assert.commandWorked(db[collName].insert(
                     {_id: uniqueNum, x: i, ["y_" + i]: uniqueNum, a: uniqueNum, b: uniqueNum}));
             }
         }
-
-        db[sameDbCollName].drop();
-        db.getSiblingDB(otherDbName)[otherDbCollName].drop();
-        assertAlways.commandWorked(db[sameDbCollName].insert({_id: 0}));
-        assertAlways.commandWorked(db.getSiblingDB(otherDbName)[otherDbCollName].insert({_id: 0}));
-
-        if (isMongos(db)) {
-            const shardNames = Object.keys(cluster.getSerializedCluster().shards);
-            const numShards = shardNames.length;
-            if (numShards > 1) {
-                assertAlways.commandWorked(
-                    db.adminCommand({movePrimary: db.getName(), to: shardNames[0]}));
-                assertAlways.commandWorked(db.getSiblingDB(otherDbName).adminCommand({
-                    movePrimary: otherDbName,
-                    to: shardNames[1]
-                }));
-            }
-        }
+        assert.commandWorked(db[sameDbCollName].insert({_id: 0}));
+        assert.commandWorked(db.getSiblingDB(otherDbName)[otherDbCollName].insert({_id: 0}));
     };
 
     const teardown = function(db, collName, cluster) {
