@@ -1,5 +1,3 @@
-'use strict';
-
 /**
  * count_indexed.js
  *
@@ -9,11 +7,16 @@
  * and a random 'countPerNum' in range [50, 100]
  * and then inserts 'modulus * countPerNum' documents. [250, 1000]
  * Each thread inserts docs into a unique collection.
+ *
+ * TODO SERVER-91708: An index build queued to start but not added to the list of active index
+ * builds yet will allow a moveCollection to proceed and the index build will fail to build
+ * on account of the collection being dropped on the donor.
+ * @tags: [assumes_balancer_off]
  */
-load('jstests/concurrency/fsm_libs/extend_workload.js');  // for extendWorkload
-load('jstests/concurrency/fsm_workloads/count.js');       // for $config
+import {extendWorkload} from "jstests/concurrency/fsm_libs/extend_workload.js";
+import {$config as $baseConfig} from "jstests/concurrency/fsm_workloads/count.js";
 
-var $config = extendWorkload($config, function($config, $super) {
+export const $config = extendWorkload($baseConfig, function($config, $super) {
     $config.data.prefix = 'count_fsm';
     $config.data.shardKey = {tid: 1, i: 1};
 
@@ -25,7 +28,7 @@ var $config = extendWorkload($config, function($config, $super) {
     $config.states.init = function init(db, collName) {
         this.threadCollName = this.prefix + '_' + this.tid;
         $super.states.init.apply(this, arguments);
-        assertAlways.commandWorked(db[this.threadCollName].createIndex({tid: 1, i: 1}));
+        assert.commandWorked(db[this.threadCollName].createIndex({tid: 1, i: 1}));
     };
 
     return $config;
