@@ -1,12 +1,9 @@
-'use strict';
-
 /**
  * create_collection.js
  *
  * Repeatedly creates a collection.
  */
-
-var $config = (function() {
+export const $config = (function() {
     var data = {
         // Use the workload name as a prefix for the collection name,
         // since the workload name is assumed to be unique.
@@ -26,7 +23,7 @@ var $config = (function() {
         function create(db, collName) {
             // TODO: should we ever do something different?
             var myCollName = uniqueCollectionName(this.prefix, this.tid, this.num++);
-            assertAlways.commandWorked(db.createCollection(myCollName));
+            assert.commandWorked(db.createCollection(myCollName));
         }
 
         return {init: init, create: create};
@@ -36,7 +33,11 @@ var $config = (function() {
 
     return {
         threadCount: 5,
-        iterations: 20,
+        iterations:
+            // The config transition suites involve moving unsharded collections in and out of
+            // the config server. Having up to 100 unsharded collections to move may make this test
+            // take too long to run and get killed by resmoke.
+            TestData.shardsAddedRemoved ? 5 : 20,
         data: data,
         states: states,
         transitions: transitions,
