@@ -1,5 +1,3 @@
-'use strict';
-
 /**
  * Executes the create_index_background.js workload, but with a partial filter expression on the
  * indexed field.
@@ -9,10 +7,10 @@
  *   creates_background_indexes
  * ]
  */
-load('jstests/concurrency/fsm_libs/extend_workload.js');               // For extendWorkload.
-load('jstests/concurrency/fsm_workloads/create_index_background.js');  // For $config.
+import {extendWorkload} from "jstests/concurrency/fsm_libs/extend_workload.js";
+import {$config as $baseConfig} from "jstests/concurrency/fsm_workloads/create_index_background.js";
 
-var $config = extendWorkload($config, function($config, $super) {
+export const $config = extendWorkload($baseConfig, function($config, $super) {
     const fieldName = "isIndexed";
 
     $config.data.getIndexSpec = function() {
@@ -31,7 +29,7 @@ var $config = extendWorkload($config, function($config, $super) {
 
     $config.data.extendDocument = function extendDocument(originalDoc) {
         // Be sure we're not overwriting an existing field.
-        assertAlways.eq(originalDoc.hasOwnProperty(fieldName), false);
+        assert.eq(originalDoc.hasOwnProperty(fieldName), false);
 
         // Create documents so that about half are indexed by applying to the partial filter
         // expression.
