@@ -1,5 +1,3 @@
-"use strict";
-
 /**
  * Repeatedly creates a time-series collection, inserts data and drops it.
  *
@@ -8,7 +6,10 @@
  *   does_not_support_stepdowns,
  * ]
  */
-var $config = (function() {
+
+import {TimeseriesTest} from "jstests/core/timeseries/libs/timeseries.js";
+
+export const $config = (function() {
     var data = {prefix: "create_timeseries_collection"};
 
     var states = (function() {
@@ -24,7 +25,7 @@ var $config = (function() {
             collName = getCollectionName(this.prefix, collName, this.tid);
 
             const timeFieldName = "time";
-            assertAlways.commandWorked(
+            assert.commandWorked(
                 db.createCollection(collName, {timeseries: {timeField: timeFieldName}}));
         }
 
@@ -32,7 +33,7 @@ var $config = (function() {
             collName = getCollectionName(this.prefix, collName, this.tid);
 
             const coll = db.getCollection(collName);
-            assertAlways.commandWorked(coll.insert({
+            TimeseriesTest.assertInsertWorked(coll.insert({
                 _id: this.num,
                 measurement: "measurement",
                 time: ISODate(),
@@ -41,7 +42,7 @@ var $config = (function() {
 
         function drop(db, collName) {
             collName = getCollectionName(this.prefix, collName, this.tid);
-            assertAlways(db.getCollection(collName).drop(), "failed to drop " + collName);
+            assert(db.getCollection(collName).drop(), "failed to drop " + collName);
         }
 
         return {init: init, create: create, insert: insert, drop: drop};
