@@ -1,28 +1,29 @@
-'use strict';
-
 /**
  * Repeatedly creates and drops a database in concurrency with FCV changes
  *
+ * This test is config shard incompatible because we do not currently allow downgrading the FCV
+ * while there is a config shard.
  * @tags: [
  *   requires_sharding,
+ *   config_shard_incompatible,
  *   # Requires all nodes to be running the latest binary.
  *   multiversion_incompatible,
- *   # TODO SERVER-75391: Re-enable this test.
- *   config_shard_incompatible,
+ *   # TODO (SERVER-88539) Remove the 'assumes_balancer_off' tag
+ *   assumes_balancer_off,
  *  ]
  */
 
-load('jstests/concurrency/fsm_libs/extend_workload.js');
-load('jstests/concurrency/fsm_workloads/drop_database_sharded.js');
+import {extendWorkload} from "jstests/concurrency/fsm_libs/extend_workload.js";
+import {$config as $baseConfig} from "jstests/concurrency/fsm_workloads/drop_database_sharded.js";
 
-var $config = extendWorkload($config, function($config, $super) {
+export const $config = extendWorkload($baseConfig, function($config, $super) {
     $config.states.setFCV = function(db, collName) {
         const fcvValues = [lastLTSFCV, lastContinuousFCV, latestFCV];
         const targetFCV = fcvValues[Random.randInt(3)];
         jsTestLog('Executing FCV state, setting to:' + targetFCV);
         try {
-            assertAlways.commandWorked(
-                db.adminCommand({setFeatureCompatibilityVersion: targetFCV}));
+            assert.commandWorked(
+                db.adminCommand({setFeatureCompatibilityVersion: targetFCV, confirm: true}));
         } catch (e) {
             if (e.code === 5147403) {
                 // Invalid fcv transition (e.g lastContinuous -> lastLTS)
@@ -32,12 +33,20 @@ var $config = extendWorkload($config, function($config, $super) {
             if (e.code === 7428200) {
                 // Cannot upgrade FCV if a previous FCV downgrade stopped in the middle of cleaning
                 // up internal server metadata.
-                assertAlways.eq(latestFCV, targetFCV);
+                assert.eq(latestFCV, targetFCV);
                 jsTestLog(
                     'setFCV: Cannot upgrade FCV if a previous FCV downgrade stopped in the middle \
                     of cleaning up internal server metadata');
                 return;
             }
+            if (e.code === 12587) {
+                // Cannot downgrade FCV that requires a collMod command when index builds are
+                // concurrently taking place.
+                jsTestLog(
+                    'setFCV: Cannot downgrade the FCV that requires a collMod command when index \
+                    builds are concurrently running');
+                return;
+            }
             throw e;
         }
         jsTestLog('setFCV state finished');
@@ -75,7 +84,8 @@ var $config = extendWorkload($config, function($config, $super) {
     };
 
     $config.teardown = function(db, collName) {
-        assert.commandWorked(db.adminCommand({setFeatureCompatibilityVersion: latestFCV}));
+        assert.commandWorked(
+            db.adminCommand({setFeatureCompatibilityVersion: latestFCV, confirm: true}));
         $super.teardown(db, collName);
     };
 
