@@ -1,12 +1,10 @@
-'use strict';
-
 /**
  * findAndModify_remove.js
  *
  * Each thread repeatedly inserts a document, and subsequently performs
  * the findAndModify command to remove it.
  */
-var $config = (function() {
+export const $config = (function() {
     var data = {shardKey: {tid: 1}};
 
     var states = (function() {
@@ -16,8 +14,8 @@ var $config = (function() {
 
         function insertAndRemove(db, collName) {
             var res = db[collName].insert({tid: this.tid, value: this.iter});
-            assertAlways.commandWorked(res);
-            assertAlways.eq(1, res.nInserted);
+            assert.commandWorked(res);
+            assert.eq(1, res.nInserted);
 
             res = db.runCommand({
                 findandmodify: db[collName].getName(),
@@ -25,14 +23,15 @@ var $config = (function() {
                 sort: {iter: -1},
                 remove: true
             });
-            assertAlways.commandWorked(res);
+            assert.commandWorked(res);
 
             var doc = res.value;
-            assertWhenOwnColl(doc !== null, 'query spec should have matched a document');
+            assert(doc !== null,
+                   'query spec should have matched a document, returned ' + tojson(res));
 
             if (doc !== null) {
-                assertAlways.eq(this.tid, doc.tid);
-                assertWhenOwnColl.eq(this.iter, doc.value);
+                assert.eq(this.tid, doc.tid);
+                assert.eq(this.iter, doc.value);
             }
 
             this.iter++;
