@@ -1,5 +1,3 @@
-'use strict';
-
 /**
  * indexed_insert_2dsphere.js
  *
@@ -7,10 +5,10 @@
  * appear in both a collection scan and an index scan. The indexed value is a
  * legacy coordinate pair, indexed with a 2dsphere index.
  */
-load('jstests/concurrency/fsm_libs/extend_workload.js');         // for extendWorkload
-load('jstests/concurrency/fsm_workloads/indexed_insert_2d.js');  // for $config
+import {extendWorkload} from "jstests/concurrency/fsm_libs/extend_workload.js";
+import {$config as $baseConfig} from "jstests/concurrency/fsm_workloads/indexed_insert_2d.js";
 
-var $config = extendWorkload($config, function($config, $super) {
+export const $config = extendWorkload($baseConfig, function($config, $super) {
     $config.data.indexedField = 'indexed_insert_2dsphere';
 
     $config.data.getIndexSpec = function getIndexSpec() {
