@@ -1,5 +1,3 @@
-'use strict';
-
 /**
  * indexed_insert_unordered_bulk.js
  *
@@ -8,10 +6,10 @@
  *
  * Uses an unordered, bulk operation to perform the inserts.
  */
-load('jstests/concurrency/fsm_libs/extend_workload.js');           // for extendWorkload
-load('jstests/concurrency/fsm_workloads/indexed_insert_base.js');  // for $config
+import {extendWorkload} from "jstests/concurrency/fsm_libs/extend_workload.js";
+import {$config as $baseConfig} from "jstests/concurrency/fsm_workloads/indexed_insert_base.js";
 
-var $config = extendWorkload($config, function($config, $super) {
+export const $config = extendWorkload($baseConfig, function($config, $super) {
     $config.data.indexedField = 'indexed_insert_unordered_bulk';
     $config.data.shardKey = {};
     $config.data.shardKey[$config.data.indexedField] = 1;
@@ -25,8 +23,8 @@ var $config = extendWorkload($config, function($config, $super) {
             bulk.insert(doc);
         }
         var res = bulk.execute();
-        assertAlways.commandWorked(res);
-        assertAlways.eq(this.docsPerInsert, res.nInserted, tojson(res));
+        assert.commandWorked(res);
+        assert.eq(this.docsPerInsert, res.nInserted, tojson(res));
 
         this.nInserted += this.docsPerInsert;
     };
