@@ -1,5 +1,3 @@
-'use strict';
-
 /**
  * kill_rooted_or.js
  *
@@ -9,8 +7,11 @@
  *
  * This workload was designed to reproduce SERVER-24761.
  */
-load("jstests/concurrency/fsm_workload_helpers/assert_handle_fail_in_transaction.js");
-var $config = (function() {
+import {
+    assertWorkedOrFailedHandleTxnErrors
+} from "jstests/concurrency/fsm_workload_helpers/assert_handle_fail_in_transaction.js";
+
+export const $config = (function() {
     // Use the workload name as the collection name, since the workload name is assumed to be
     // unique. Note that we choose our own collection name instead of using the collection provided
     // by the concurrency framework, because this workload drops its collection.
@@ -69,11 +70,13 @@ var $config = (function() {
                                                     ErrorCodes.IndexBuildAborted,
                                                     ErrorCodes.IndexBuildAlreadyInProgress,
                                                     ErrorCodes.NoMatchingDocument,
+                                                    ErrorCodes.StaleConfig
                                                 ],
                                                 [
                                                     ErrorCodes.CannotImplicitlyCreateCollection,
                                                     ErrorCodes.IndexBuildAborted,
                                                     ErrorCodes.NoMatchingDocument,
+                                                    ErrorCodes.StaleConfig
                                                 ]);
         }
     };
@@ -98,11 +101,13 @@ var $config = (function() {
                                                     ErrorCodes.IndexBuildAborted,
                                                     ErrorCodes.IndexBuildAlreadyInProgress,
                                                     ErrorCodes.NoMatchingDocument,
+                                                    ErrorCodes.StaleConfig
                                                 ],
                                                 [
                                                     ErrorCodes.CannotImplicitlyCreateCollection,
                                                     ErrorCodes.IndexBuildAborted,
                                                     ErrorCodes.NoMatchingDocument,
+                                                    ErrorCodes.StaleConfig
                                                 ]);
         });
     }
@@ -112,10 +117,13 @@ var $config = (function() {
         // Similarly, this bulk insert can also give up with a NoProgressMade error after repeated
         // attempts in the sharded causal consistency configuration. We also ignore that error.
         const bulkInsertResult = coll.insert(Array(numDocs).fill({a: 0, b: 0, c: 0}));
-        assertAlways(!bulkInsertResult.hasWriteConcernError(), bulkInsertResult);
+        assert(!bulkInsertResult.hasWriteConcernError(), bulkInsertResult);
         bulkInsertResult.getWriteErrors().forEach(err => {
-            assertAlways.contains(
-                err.code, [ErrorCodes.CannotImplicitlyCreateCollection, ErrorCodes.NoProgressMade]);
+            assert.contains(err.code, [
+                ErrorCodes.CannotImplicitlyCreateCollection,
+                ErrorCodes.NoProgressMade,
+                ErrorCodes.StaleConfig
+            ]);
         }, bulkInsertResult);
     }
 
