@@ -1,5 +1,3 @@
-'use strict';
-
 /**
  * map_reduce_drop.js
  *
@@ -16,7 +14,7 @@
  *   does_not_support_causal_consistency,
  * ]
  */
-var $config = (function() {
+export const $config = (function() {
     var data = {
         mapper: function mapper() {
             emit(this.key, 1);
@@ -59,10 +57,10 @@ var $config = (function() {
                     bulk.insert({key: Random.randInt(10000)});
                 }
                 var res = bulk.execute();
-                assertAlways.commandWorked(res);
+                assert.commandWorked(res);
             } catch (ex) {
-                assert.writeErrorWithCode(ex, ErrorCodes.DatabaseDropPending);
                 assert.eq(true, ex instanceof BulkWriteError, tojson(ex));
+                assert.writeErrorWithCode(ex, ErrorCodes.DatabaseDropPending);
             }
 
             var options = {
