@@ -1,5 +1,3 @@
-'use strict';
-
 /**
  * map_reduce_with_chunk_migrations.js
  *
@@ -12,15 +10,20 @@
  *
  * @tags: [
  *  requires_sharding, assumes_balancer_off,
+ *  resource_intensive,
  *  requires_non_retryable_writes,
  *  # mapReduce does not support afterClusterTime.
- *  does_not_support_causal_consistency
+ *  does_not_support_causal_consistency,
+ *   # Use mapReduce.
+ *   requires_scripting,
  * ]
  */
-load('jstests/concurrency/fsm_libs/extend_workload.js');                     // for extendWorkload
-load('jstests/concurrency/fsm_workloads/sharded_moveChunk_partitioned.js');  // for $config
+import {extendWorkload} from "jstests/concurrency/fsm_libs/extend_workload.js";
+import {
+    $config as $baseConfig
+} from "jstests/concurrency/fsm_workloads/sharded_moveChunk_partitioned.js";
 
-var $config = extendWorkload($config, function($config, $super) {
+export const $config = extendWorkload($baseConfig, function($config, $super) {
     // The base setup will insert 'partitionSize' number of documents per thread, evenly
     // distributing across the chunks. Documents will only have the "_id" field.
     $config.data.partitionSize = 50;
@@ -60,7 +63,7 @@ var $config = extendWorkload($config, function($config, $super) {
         };
 
         const res = db[collName].mapReduce(map, reduce, {out: {replace: this.resultsCollection}});
-        assertWhenOwnColl.commandWorked(res);
+        assert.commandWorked(res);
 
         assert.eq(this.numDocs, db[this.resultsCollection].find().itcount());
     };
