@@ -1,5 +1,3 @@
-'use strict';
-
 /**
  * multi_statement_transaction_atomicity_isolation.js
  *
@@ -43,16 +41,14 @@
  * @tags: [uses_transactions, assumes_snapshot_transactions]
  */
 
-// for Graph
-load('jstests/libs/cycle_detection.js');
+import {arrayEq} from "jstests/aggregation/extras/utils.js";
+import {
+    isKilledSessionCode,
+    withTxnAndAutoRetry
+} from "jstests/concurrency/fsm_workload_helpers/auto_retry_transaction.js";
+import {Graph} from "jstests/libs/cycle_detection.js";
 
-// For withTxnAndAutoRetry, isKilledSessionCode.
-load('jstests/concurrency/fsm_workload_helpers/auto_retry_transaction.js');
-
-// For arrayEq.
-load("jstests/aggregation/extras/utils.js");
-
-var $config = (function() {
+export const $config = (function() {
     function checkTransactionCommitOrder(documents) {
         const graph = new Graph();
 
@@ -77,12 +73,12 @@ var $config = (function() {
                 return Object.assign({}, doc, {order: filteredCommitOrder});
             });
 
-            assertWhenOwnColl.eq([], result, tojson(filteredDocuments));
+            assert.eq([], result, tojson(filteredDocuments));
         }
     }
 
     function checkNumUpdatedByEachTransaction(documents) {
-        const updateCounts = new Map();
+        const updateCounts = new BSONAwareMap();
 
         for (let doc of documents) {
             for (let op of doc.order) {
@@ -129,11 +125,11 @@ var $config = (function() {
         const highestTxnNum =
             Math.max(updatedDocsServerHistory.length, data.updatedDocsClientHistory.length);
         for (let txnNum = 0; txnNum < highestTxnNum; ++txnNum) {
-            assertAlways((arrayEq(updatedDocsServerHistory[txnNum] || [],
-                                  data.updatedDocsClientHistory[txnNum] || [])),
-                         () => 'expected ' + tojson(data.updatedDocsClientHistory[txnNum]) +
-                             ' but instead have ' + tojson(updatedDocsServerHistory[txnNum]) +
-                             ' for txnNumber ' + txnNum);
+            assert((arrayEq(updatedDocsServerHistory[txnNum] || [],
+                            data.updatedDocsClientHistory[txnNum] || [])),
+                   () => 'expected ' + tojson(data.updatedDocsClientHistory[txnNum]) +
+                       ' but instead have ' + tojson(updatedDocsServerHistory[txnNum]) +
+                       ' for txnNumber ' + txnNum);
         }
     }
 
@@ -168,7 +164,7 @@ var $config = (function() {
             }
         }
 
-        assertWhenOwnColl.eq(allDocuments.length, numDocs * this.collections.length, () => {
+        assert.eq(allDocuments.length, numDocs * this.collections.length, () => {
             if (this.session) {
                 return "txnNumber: " + tojson(this.session.getTxnNumber_forTesting()) +
                     ", session id: " + tojson(this.session.getSessionId()) +
@@ -207,7 +203,7 @@ var $config = (function() {
 
                 withTxnAndAutoRetry(this.session, () => {
                     committedTxnInfo = [];
-                    for (let [i, docId] of docIds.entries()) {
+                    for (let [_, docId] of docIds.entries()) {
                         const collection =
                             this.collections[Random.randInt(this.collections.length)];
                         const txnDbName = collection.getDB().getName();
@@ -241,12 +237,11 @@ var $config = (function() {
                         const res = txnCollection.runCommand('update', {
                             updates: [{q: {_id: docId}, u: updateMods}],
                         });
-                        assertAlways.commandWorked(
-                            res,
-                            () => "Failed to update. result: " + tojson(res) +
-                                ", collection: " + tojson(coll));
-                        assertWhenOwnColl.eq(res.n, 1, () => tojson(res));
-                        assertWhenOwnColl.eq(res.nModified, 1, () => tojson(res));
+                        assert.commandWorked(res,
+                                             () => "Failed to update. result: " + tojson(res) +
+                                                 ", collection: " + tojson(coll));
+                        assert.eq(res.n, 1, () => tojson(res));
+                        assert.eq(res.nModified, 1, () => tojson(res));
                         committedTxnInfo.push(
                             {_id: docId, dbName: txnDbName, collName: txnCollName});
                     }
@@ -280,7 +275,7 @@ var $config = (function() {
 
                 try {
                     const cursor = sessionCollection.find();
-                    assertAlways.eq(cursor.itcount(), this.numDocs);
+                    assert.eq(cursor.itcount(), this.numDocs);
                 } catch (e) {
                     if (this.retryOnKilledSession && isKilledSessionCode(e.code)) {
                         // If the session is expected to be killed, ignore it.
@@ -350,8 +345,8 @@ var $config = (function() {
             }
 
             const res = bulk.execute({w: 'majority'});
-            assertWhenOwnColl.commandWorked(res);
-            assertWhenOwnColl.eq(this.numDocs, res.nInserted);
+            assert.commandWorked(res);
+            assert.eq(this.numDocs, res.nInserted);
         }
 
         if (cluster.isSharded()) {
