@@ -1,15 +1,15 @@
-'use strict';
-
 /**
  * Test transactions atomicity and isolation guarantees for transactions across multiple DBs.
  *
  * @tags: [uses_transactions, assumes_snapshot_transactions]
  */
 
-load('jstests/concurrency/fsm_libs/extend_workload.js');  // for extendWorkload
-load('jstests/concurrency/fsm_workloads/multi_statement_transaction_atomicity_isolation.js');
+import {extendWorkload} from "jstests/concurrency/fsm_libs/extend_workload.js";
+import {
+    $config as $baseConfig
+} from "jstests/concurrency/fsm_workloads/multi_statement_transaction_atomicity_isolation.js";
 
-var $config = extendWorkload($config, ($config, $super) => {
+export const $config = extendWorkload($baseConfig, ($config, $super) => {
     // Number of unique collections and number of unique databases. The square root is used
     // here to ensure the total number of namespaces (coll * db) is roughly equal to the
     // number of threads.
