@@ -4,7 +4,7 @@
  * Stresses $queryStats being toggled on and off during execution.
  *
  * @tags: [
- *  requires_fcv_70,
+ *  requires_fcv_72,
  *  # Cannot read query stats entries from a different node than issued the query.
  *  does_not_support_causal_consistency,
  *  # setParameter is not persistent.
@@ -16,9 +16,9 @@
  *
  */
 
-load("jstests/concurrency/fsm_workload_helpers/set_parameter.js");
+import {setParameterOnAllNodes} from "jstests/concurrency/fsm_workload_helpers/set_parameter.js";
 
-var $config = (function() {
+export const $config = (function() {
     function setCacheSize(db, cacheSize) {
         assert.commandWorked(
             db.adminCommand({setParameter: 1, internalQueryStatsCacheSize: cacheSize}));
