@@ -1,17 +1,14 @@
-'use strict';
-
 /**
  * Concurrently performs DDL commands and verifies guarantees are not broken.
  *
  * @tags: [
  *   requires_sharding,
- *   # TODO (SERVER-56879): Support add/remove shards in new DDL paths
- *   does_not_support_add_remove_shards,
  *  ]
  */
 
-load("jstests/concurrency/fsm_workload_helpers/state_transition_utils.js");
-load('jstests/libs/feature_flag_util.js');
+import {
+    uniformDistTransitions
+} from "jstests/concurrency/fsm_workload_helpers/state_transition_utils.js";
 
 const dbPrefix = jsTestName() + '_DB_';
 const dbCount = 2;
@@ -23,7 +20,7 @@ function getRandomShard(connCache) {
     return shards[Random.randInt(shards.length)];
 }
 
-var $config = (function() {
+export const $config = (function() {
     let data = {
         getRandomDb: function(db) {
             return db.getSiblingDB(dbPrefix + Random.randInt(dbCount));
@@ -31,6 +28,16 @@ var $config = (function() {
         getRandomCollection: function(db) {
             return db[collPrefix + Random.randInt(collCount)];
         },
+        movePrimaryAllowedErrorCodes: [
+            ErrorCodes.ConflictingOperationInProgress,
+            // The cloning phase has failed (e.g. as a result of a stepdown). When a failure
+            // occurs at this phase, the movePrimary operation does not recover.
+            7120202,
+            // In the FSM tests, there is a chance that there are still some User collections
+            // left to clone. This occurs when a MovePrimary joins an already existing MovePrimary
+            // command that has purposefully triggered a failpoint.
+            9046501,
+        ]
     };
 
     let states = {
@@ -40,7 +47,7 @@ var $config = (function() {
             const fullNs = coll.getFullName();
 
             jsTestLog('Executing create state: ' + fullNs);
-            assertAlways.commandWorked(
+            assert.commandWorked(
                 db.adminCommand({shardCollection: fullNs, key: {_id: 1}, unique: false}));
         },
         drop: function(db, collName, connCache) {
@@ -49,7 +56,7 @@ var $config = (function() {
 
             jsTestLog('Executing drop state: ' + coll.getFullName());
 
-            assert.commandWorked(db.runCommand({drop: coll.getName()}));
+            assert.eq(coll.drop(), true);
         },
         rename: function(db, collName, connCache) {
             db = this.getRandomDb(db);
@@ -59,7 +66,7 @@ var $config = (function() {
             const destCollName = destCollNS.split('.')[1];
 
             jsTestLog('Executing rename state:' + srcCollName + ' to ' + destCollNS);
-            assertAlways.commandWorkedOrFailedWithCode(
+            assert.commandWorkedOrFailedWithCode(
                 srcColl.renameCollection(destCollName, true /* dropTarget */), [
                     ErrorCodes.NamespaceNotFound,
                     ErrorCodes.ConflictingOperationInProgress,
@@ -67,61 +74,57 @@ var $config = (function() {
                 ]);
         },
         movePrimary: function(db, collName, connCache) {
-            if (this.skipMovePrimary) {
-                return;
-            }
-
             db = this.getRandomDb(db);
             const shardId = getRandomShard(connCache);
 
             jsTestLog('Executing movePrimary state: ' + db.getName() + ' to ' + shardId);
-            assertAlways.commandWorkedOrFailedWithCode(
-                db.adminCommand({movePrimary: db.getName(), to: shardId}), [
-                    ErrorCodes.ConflictingOperationInProgress,
-                    // The cloning phase has failed (e.g. as a result of a stepdown). When a failure
-                    // occurs at this phase, the movePrimary operation does not recover.
-                    7120202
-                ]);
+            assert.commandWorkedOrFailedWithCode(
+                db.adminCommand({movePrimary: db.getName(), to: shardId}),
+                this.movePrimaryAllowedErrorCodes);
         },
         collMod: function(db, collName, connCache) {
             db = this.getRandomDb(db);
             const coll = this.getRandomCollection(db);
 
             jsTestLog('Executing collMod state: ' + coll.getFullName());
-            assertAlways.commandWorkedOrFailedWithCode(
+            assert.commandWorkedOrFailedWithCode(
                 db.runCommand({collMod: coll.getName(), validator: {a: {$gt: 0}}}),
                 [ErrorCodes.NamespaceNotFound, ErrorCodes.ConflictingOperationInProgress]);
         },
         checkDatabaseMetadataConsistency: function(db, collName, connCache) {
-            if (this.skipMetadataChecks) {
-                return;
-            }
             db = this.getRandomDb(db);
             jsTestLog('Executing checkMetadataConsistency state for database: ' + db.getName());
             const inconsistencies = db.checkMetadataConsistency().toArray();
             assert.eq(0, inconsistencies.length, tojson(inconsistencies));
         },
         checkCollectionMetadataConsistency: function(db, collName, connCache) {
-            if (this.skipMetadataChecks) {
-                return;
-            }
             db = this.getRandomDb(db);
             const coll = this.getRandomCollection(db);
             jsTestLog('Executing checkMetadataConsistency state for collection: ' +
                       coll.getFullName());
             const inconsistencies = coll.checkMetadataConsistency().toArray();
             assert.eq(0, inconsistencies.length, tojson(inconsistencies));
+        },
+        untrackUnshardedCollection: function untrackUnshardedCollection(db, collName, connCache) {
+            // Note this command will behave as no-op in case the collection is not tracked.
+            const namespace = `${db}.${collName}`;
+            jsTestLog(`Started to untrack collection ${namespace}`);
+            assert.commandWorkedOrFailedWithCode(
+                db.adminCommand({untrackUnshardedCollection: namespace}), [
+                    // Handles the case where the collection is not located on its primary
+                    ErrorCodes.OperationFailed,
+                    // Handles the case where the collection is sharded
+                    ErrorCodes.InvalidNamespace,
+                    // Handles the case where the collection/db does not exist
+                    ErrorCodes.NamespaceNotFound,
+                    // The command does not exist in pre-8.0 versions
+                    ErrorCodes.CommandNotFound,
+                ]);
+            jsTestLog(`Untrack collection completed`);
         }
     };
 
     let setup = function(db, collName, cluster) {
-        // TODO (SERVER-71309): Remove once 7.0 becomes last LTS. Prevent non-resilient movePrimary
-        // operations from being executed in multiversion suites.
-        this.skipMovePrimary = !FeatureFlagUtil.isEnabled(db.getMongo(), 'ResilientMovePrimary');
-        this.skipMetadataChecks =
-            // TODO SERVER-70396: remove this flag
-            !FeatureFlagUtil.isEnabled(db.getMongo(), 'CheckMetadataConsistency');
-
         for (var i = 0; i < dbCount; i++) {
             const dbName = dbPrefix + i;
             const newDb = db.getSiblingDB(dbName);
@@ -131,7 +134,7 @@ var $config = (function() {
 
     let teardown = function(db, collName, cluster) {
         const configDB = db.getSiblingDB("config");
-        assertAlways(configDB.collections.countDocuments({allowMigrations: {$exists: true}}) == 0);
+        assert(configDB.collections.countDocuments({allowMigrations: {$exists: true}}) == 0);
     };
 
     return {
