@@ -1,10 +1,9 @@
-'use strict';
-
 /**
  * Same as the base workload, but refines to a nested shard key.
  *
  * @tags: [
  *   assumes_balancer_off,
+ *   resource_intensive,
  *   # The init() state function populates each document owned by a particular thread with a
  *   # "groupId" value. Doing so may take longer than the configured stepdown interval. It is
  *   # therefore unsafe to automatically run inside a multi-statement transaction because its
@@ -15,11 +14,13 @@
  *   uses_transactions,
  * ]
  */
-load('jstests/concurrency/fsm_libs/extend_workload.js');
-load(
-    'jstests/concurrency/fsm_workloads/random_moveChunk_refine_collection_shard_key_broadcast_delete_transaction.js');
+import {extendWorkload} from "jstests/concurrency/fsm_libs/extend_workload.js";
+import {
+    $config as $baseConfig
+} from
+    "jstests/concurrency/fsm_workloads/random_moveChunk_refine_collection_shard_key_broadcast_delete_transaction.js";
 
-var $config = extendWorkload($config, function($config, $super) {
+export const $config = extendWorkload($baseConfig, function($config, $super) {
     $config.data.newShardKey = {a: 1, "b.c": 1};
     $config.data.newShardKeyFields = ["a", "b.c"];
     return $config;
