@@ -1,20 +1,23 @@
-'use strict';
-
 /**
  * Performs updates that will change a document's shard key while migrating chunks. Uses both
  * retryable writes and multi-statement transactions.
  *
  * @tags: [
  *  requires_sharding,
+ *  resource_intensive,
  *  assumes_balancer_off,
  *  uses_transactions,
  * ]
  */
-load('jstests/concurrency/fsm_libs/extend_workload.js');
-load('jstests/concurrency/fsm_workloads/random_moveChunk_base.js');
-load('jstests/concurrency/fsm_workload_helpers/auto_retry_transaction.js');
+import {extendWorkload} from "jstests/concurrency/fsm_libs/extend_workload.js";
+import {fsm} from "jstests/concurrency/fsm_libs/fsm.js";
+import {
+    withTxnAndAutoRetry
+} from "jstests/concurrency/fsm_workload_helpers/auto_retry_transaction.js";
+import {$config as $baseConfig} from "jstests/concurrency/fsm_workloads/random_moveChunk_base.js";
+import {FeatureFlagUtil} from "jstests/libs/feature_flag_util.js";
 
-var $config = extendWorkload($config, function($config, $super) {
+export const $config = extendWorkload($baseConfig, function($config, $super) {
     $config.threadCount = 5;
     $config.iterations = 50;
 
@@ -82,8 +85,8 @@ var $config = extendWorkload($config, function($config, $super) {
             ErrorCodes.WriteConflict,
             ErrorCodes.LockTimeout,
             ErrorCodes.PreparedTransactionInProgress,
-            ErrorCodes.ShardInvalidatedForTargeting,
-            ErrorCodes.NoSuchTransaction
+            ErrorCodes.NoSuchTransaction,
+            ErrorCodes.ExceededTimeLimit
         ];
 
         // If we're running in a stepdown suite, then attempting to update the shard key may
@@ -96,6 +99,17 @@ var $config = extendWorkload($config, function($config, $super) {
             skippableErrors.push(...stepdownErrors);
         }
 
+        // If we're killing sessions, retrying an interrupted retryable write may fail with
+        // ConflictingOperationInProgress if the write's transaction number has already been used to
+        // start a transaction on the router. This should only happen if the original attempt was
+        // interrupted after starting its transaction on the router but *before* sending a
+        // transaction command to mongod, otherwise the retry will be rejected by mongod with
+        // IncompleteTransactionHistory. This also means we know the write must not have committed.
+        const interruptionRetryErrors = [ErrorCodes.ConflictingOperationInProgress];
+        if (this.retryOnKilledSession) {
+            skippableErrors.push(...interruptionRetryErrors);
+        }
+
         // Failed in the document shard key path, but not with a duplicate key error
         if (errMsg.includes(otherErrorsInChangeShardKeyMsg)) {
             return skippableErrors.includes(errCode);
@@ -108,7 +122,7 @@ var $config = extendWorkload($config, function($config, $super) {
         collection, collName, shardKeyField, moveAcrossChunks) {
         const idToUpdate = this.getIdForThread(collName);
         const randomDocToUpdate = collection.findOne({_id: idToUpdate});
-        assertWhenOwnColl.neq(randomDocToUpdate, null);
+        assert.neq(randomDocToUpdate, null);
 
         print("Updating the shard key field for this document: " + tojson(randomDocToUpdate));
 
@@ -193,9 +207,9 @@ var $config = extendWorkload($config, function($config, $super) {
 
     function assertDocWasUpdated(
         collection, idToUpdate, currentShardKey, newShardKey, newCounter, tid) {
-        assertWhenOwnColl.isnull(collection.findOne({_id: idToUpdate, skey: currentShardKey}));
-        assertWhenOwnColl.eq(collection.findOne({_id: idToUpdate, skey: newShardKey}),
-                             {_id: idToUpdate, skey: newShardKey, tid: tid, counter: newCounter});
+        assert.isnull(collection.findOne({_id: idToUpdate, skey: currentShardKey}));
+        assert.eq(collection.findOne({_id: idToUpdate, skey: newShardKey}),
+                  {_id: idToUpdate, skey: newShardKey, tid: tid, counter: newCounter});
     }
 
     function wasDocUpdated(collection, idToUpdate, currentShardKey) {
@@ -231,9 +245,8 @@ var $config = extendWorkload($config, function($config, $super) {
                     upsert: true,
                     new: true
                 });
-                assertWhenOwnColl.neq(modifiedDoc, null);
-                assertWhenOwnColl.eq(collection.findOne({_id: idToUpdate, skey: newShardKey}),
-                                     modifiedDoc);
+                assert.neq(modifiedDoc, null);
+                assert.eq(collection.findOne({_id: idToUpdate, skey: newShardKey}), modifiedDoc);
 
                 this.expectedCounters[idToUpdate] = counterForId + 1;
             } catch (e) {
@@ -272,10 +285,8 @@ var $config = extendWorkload($config, function($config, $super) {
                     print("Ignoring acceptable updateShardKey error attempting to update the" +
                           "document with _id: " + idToUpdate + " and shardKey: " + currentShardKey +
                           ": " + e);
-                    assertWhenOwnColl.neq(
-                        collection.findOne({_id: idToUpdate, skey: currentShardKey}), null);
-                    assertWhenOwnColl.eq(collection.findOne({_id: idToUpdate, skey: newShardKey}),
-                                         null);
+                    assert.neq(collection.findOne({_id: idToUpdate, skey: currentShardKey}), null);
+                    assert.eq(collection.findOne({_id: idToUpdate, skey: newShardKey}), null);
                     return;
                 }
                 throw e;
@@ -313,7 +324,7 @@ var $config = extendWorkload($config, function($config, $super) {
                 this.generateRandomUpdateStyle(idToUpdate, newShardKey, counterForId),
                 {multi: false});
             try {
-                assertWhenOwnColl.commandWorked(updateResult);
+                assert.commandWorked(updateResult);
                 this.expectedCounters[idToUpdate] = counterForId + 1;
             } catch (e) {
                 const err = updateResult instanceof WriteResult ? updateResult.getWriteError()
@@ -352,10 +363,8 @@ var $config = extendWorkload($config, function($config, $super) {
                     print("Ignoring acceptable updateShardKey error attempting to update the" +
                           "document with _id: " + idToUpdate + " and shardKey: " + currentShardKey +
                           ": " + tojson(updateResult));
-                    assertWhenOwnColl.neq(
-                        collection.findOne({_id: idToUpdate, skey: currentShardKey}), null);
-                    assertWhenOwnColl.eq(collection.findOne({_id: idToUpdate, skey: newShardKey}),
-                                         null);
+                    assert.neq(collection.findOne({_id: idToUpdate, skey: currentShardKey}), null);
+                    assert.eq(collection.findOne({_id: idToUpdate, skey: newShardKey}), null);
                     return;
                 }
 
@@ -475,29 +484,20 @@ var $config = extendWorkload($config, function($config, $super) {
                 bulk.insert({_id: i, skey: i, tid: tid});
             }
 
-            assertAlways.commandWorked(bulk.execute());
+            assert.commandWorked(bulk.execute());
 
             // Create a chunk with boundaries matching the partition's. The low chunk's lower bound
             // is minKey, so a split is not necessary.
             if (!partition.isLowChunk) {
-                assertAlways.commandWorked(
-                    db.adminCommand({split: ns, middle: {skey: partition.lower}}));
+                assert.commandWorked(db.adminCommand({split: ns, middle: {skey: partition.lower}}));
             }
-            assertAlways.commandWorked(
-                db.adminCommand({split: ns, middle: {skey: medianIdForThread}}));
+            assert.commandWorked(db.adminCommand({split: ns, middle: {skey: medianIdForThread}}));
         }
         db.printShardingStatus();
 
-        const parameterRes = db.adminCommand(
-            {getParameter: 1, featureFlagUpdateDocumentShardKeyUsingTransactionApi: 1});
-        if (!parameterRes.ok) {
-            assert.eq(parameterRes.errmsg, "no option found to get", parameterRes);
-            this.updateDocumentShardKeyUsingTransactionApiEnabled = false;
-        } else {
-            assert.commandWorked(parameterRes);
-            this.updateDocumentShardKeyUsingTransactionApiEnabled =
-                parameterRes.featureFlagUpdateDocumentShardKeyUsingTransactionApi.value;
-        }
+        this.updateDocumentShardKeyUsingTransactionApiEnabled =
+            FeatureFlagUtil.isPresentAndEnabled(db, "UpdateDocumentShardKeyUsingTransactionApi");
+
         print("Updating document shard key using transaction api enabled: " +
               this.updateDocumentShardKeyUsingTransactionApiEnabled);
     };
@@ -509,7 +509,7 @@ var $config = extendWorkload($config, function($config, $super) {
         const docs = db[collName].find({tid: this.tid}).toArray();
         docs.forEach(doc => {
             const expectedCounter = this.expectedCounters[doc._id];
-            assertWhenOwnColl.eq(expectedCounter, doc.counter, () => {
+            assert.eq(expectedCounter, doc.counter, () => {
                 return 'unexpected counter value, doc: ' + tojson(doc);
             });
         });
