@@ -1,5 +1,3 @@
-'use strict';
-
 /**
  * rename_capped_collection_droptarget.js
  *
@@ -9,8 +7,7 @@
  *
  * @tags: [requires_capped]
  */
-
-var $config = (function() {
+export const $config = (function() {
     var data = {
         // Use the workload name as a prefix for the collection name,
         // since the workload name is assumed to be unique.
@@ -27,8 +24,8 @@ var $config = (function() {
         function insert(db, collName, numDocs) {
             for (var i = 0; i < numDocs; ++i) {
                 var res = db[collName].insert({});
-                assertAlways.commandWorked(res);
-                assertAlways.eq(1, res.nInserted);
+                assert.commandWorked(res);
+                assert.eq(1, res.nInserted);
             }
         }
 
@@ -37,20 +34,20 @@ var $config = (function() {
             this.fromCollName = uniqueCollectionName(this.prefix, this.tid, num++);
             this.toCollName = uniqueCollectionName(this.prefix, this.tid, num++);
 
-            assertAlways.commandWorked(db.createCollection(this.fromCollName, options));
-            assertWhenOwnDB(db[this.fromCollName].isCapped());
+            assert.commandWorked(db.createCollection(this.fromCollName, options));
+            assert(db[this.fromCollName].isCapped());
         }
 
         function rename(db, collName) {
             // Clear out the "from" collection and insert 'fromCollCount' documents
             var fromCollCount = 7;
-            assertWhenOwnDB(db[this.fromCollName].drop());
-            assertAlways.commandWorked(db.createCollection(this.fromCollName, options));
-            assertWhenOwnDB(db[this.fromCollName].isCapped());
+            assert(db[this.fromCollName].drop());
+            assert.commandWorked(db.createCollection(this.fromCollName, options));
+            assert(db[this.fromCollName].isCapped());
             insert(db, this.fromCollName, fromCollCount);
 
             var toCollCount = 4;
-            assertAlways.commandWorked(db.createCollection(this.toCollName, options));
+            assert.commandWorked(db.createCollection(this.toCollName, options));
             insert(db, this.toCollName, toCollCount);
 
             // Verify that 'fromCollCount' documents exist in the "to" collection
@@ -60,11 +57,11 @@ var $config = (function() {
 
             // SERVER-57128: NamespaceNotFound is an acceptable error if the mongos retries
             // the rename after the coordinator has already fulfilled the original request
-            assertWhenOwnDB.commandWorkedOrFailedWithCode(res, ErrorCodes.NamespaceNotFound);
+            assert.commandWorkedOrFailedWithCode(res, ErrorCodes.NamespaceNotFound);
 
-            assertWhenOwnDB(db[this.toCollName].isCapped());
-            assertWhenOwnDB.eq(fromCollCount, db[this.toCollName].find().itcount());
-            assertWhenOwnDB.eq(0, db[this.fromCollName].find().itcount());
+            assert(db[this.toCollName].isCapped());
+            assert.eq(fromCollCount, db[this.toCollName].find().itcount());
+            assert.eq(0, db[this.fromCollName].find().itcount());
 
             // Swap "to" and "from" collections for next execution
             var temp = this.fromCollName;
