@@ -1,18 +1,14 @@
-'use strict';
-
 /**
  * server_status.js
  *
  * Simply checks that the serverStatus command works
  */
-var $config = (function() {
+export const $config = (function() {
     var states = {
         status: function status(db, collName) {
-            var opts =
-                {opcounterRepl: 1, oplog: 1, rangeDeleter: 1, repl: 1, security: 1, tcmalloc: 1};
             var res = db.serverStatus();
-            assertAlways.commandWorked(res);
-            assertAlways(res.hasOwnProperty('version'));
+            assert.commandWorked(res);
+            assert(res.hasOwnProperty('version'));
         }
     };
 
