@@ -1,5 +1,3 @@
-'use strict';
-
 /**
  * Extends sharded_base_partitioned.js.
  *
@@ -11,11 +9,14 @@
  *  assumes_balancer_off,
  * ]
  */
+import {extendWorkload} from "jstests/concurrency/fsm_libs/extend_workload.js";
+import {fsm} from "jstests/concurrency/fsm_libs/fsm.js";
+import {ChunkHelper} from "jstests/concurrency/fsm_workload_helpers/chunks.js";
+import {
+    $config as $baseConfig
+} from "jstests/concurrency/fsm_workloads/sharded_base_partitioned.js";
 
-load('jstests/concurrency/fsm_libs/extend_workload.js');                // for extendWorkload
-load('jstests/concurrency/fsm_workloads/sharded_base_partitioned.js');  // for $config
-
-var $config = extendWorkload($config, function($config, $super) {
+export const $config = extendWorkload($baseConfig, function($config, $super) {
     $config.iterations = 5;
     $config.threadCount = 5;
 
@@ -39,7 +40,7 @@ var $config = extendWorkload($config, function($config, $super) {
         var numShards = config.getDB('config').shards.find().itcount();
         var msg = 'There must be more than one shard when performing a moveChunks operation\n' +
             'shards: ' + tojson(config.getDB('config').shards.find().toArray());
-        assertAlways.gt(numShards, 1, msg);
+        assert.gt(numShards, 1, msg);
 
         // Choose a random chunk in our partition to move.
         var chunk = this.getRandomChunkInPartition(collName, config);
@@ -94,16 +95,14 @@ var $config = extendWorkload($config, function($config, $super) {
             const runningWithStepdowns =
                 TestData.runningWithConfigStepdowns || TestData.runningWithShardStepdowns;
 
-            // TODO SERVER-46669: The moveChunk command can succeed without waiting for the range
-            // deletion to complete if the replica set shard primary steps down.
             if (waitForDelete && !runningWithStepdowns) {
                 msg = 'moveChunk succeeded but original shard still had documents.\n' + msgBase +
                     ', waitForDelete: ' + waitForDelete + ', bounds: ' + tojson(bounds);
-                assertWhenOwnColl.eq(fromShardNumDocsAfter, 0, msg);
+                assert.eq(fromShardNumDocsAfter, 0, msg);
             }
             msg = 'moveChunk succeeded but new shard did not contain all documents.\n' + msgBase +
                 ', waitForDelete: ' + waitForDelete + ', bounds: ' + tojson(bounds);
-            assertWhenOwnColl.eq(toShardNumDocsAfter, numDocsBefore, msg);
+            assert.eq(toShardNumDocsAfter, numDocsBefore, msg);
         }
         // If the moveChunk operation failed, verify that the shard the chunk was
         // originally on returns all data for the chunk, and the shard the chunk
@@ -111,7 +110,7 @@ var $config = extendWorkload($config, function($config, $super) {
         else {
             msg = 'moveChunk failed but original shard did not contain all documents.\n' + msgBase +
                 ', waitForDelete: ' + waitForDelete + ', bounds: ' + tojson(bounds);
-            assertWhenOwnColl.eq(fromShardNumDocsAfter, numDocsBefore, msg);
+            assert.eq(fromShardNumDocsAfter, numDocsBefore, msg);
         }
 
         // Verify that all config servers have the correct after-state.
@@ -119,14 +118,13 @@ var $config = extendWorkload($config, function($config, $super) {
         // with the toShard. If the operation failed, verify that the config kept the chunk's shard
         // as the fromShard.
         var chunkAfter = config.getDB('config').chunks.findOne({_id: chunk._id});
-        var msg =
-            msgBase + '\nchunkBefore: ' + tojson(chunk) + '\nchunkAfter: ' + tojson(chunkAfter);
+        msg = msgBase + '\nchunkBefore: ' + tojson(chunk) + '\nchunkAfter: ' + tojson(chunkAfter);
         if (moveChunkRes.ok) {
             msg = "moveChunk succeeded but chunk's shard was not new shard.\n" + msg;
-            assertWhenOwnColl.eq(chunkAfter.shard, toShard, msg);
+            assert.eq(chunkAfter.shard, toShard, msg);
         } else {
             msg = "moveChunk failed but chunk's shard was not original shard.\n" + msg;
-            assertWhenOwnColl.eq(chunkAfter.shard, fromShard, msg);
+            assert.eq(chunkAfter.shard, fromShard, msg);
         }
 
         // Regardless of whether the operation succeeded or failed, verify that the number of chunks
@@ -134,7 +132,7 @@ var $config = extendWorkload($config, function($config, $super) {
         var numChunksAfter = ChunkHelper.getNumChunks(
             config, ns, this.partition.chunkLower, this.partition.chunkUpper);
         msg = 'Number of chunks in partition seen by config changed with moveChunk.\n' + msgBase;
-        assertWhenOwnColl.eq(numChunksBefore, numChunksAfter, msg);
+        assert.eq(numChunksBefore, numChunksAfter, msg);
 
         // Verify that all mongos processes see the correct after-state on the shards and configs.
         // (see comments below for specifics).
@@ -145,36 +143,36 @@ var $config = extendWorkload($config, function($config, $super) {
             var numDocsAfter = ChunkHelper.getNumDocs(mongos, ns, chunk.min._id, chunk.max._id);
             msg = 'Number of documents in range seen by mongos changed with moveChunk, range: ' +
                 tojson(bounds) + '.\n' + msgBase;
-            assertWhenOwnColl.eq(numDocsAfter, numDocsBefore, msg);
+            assert.eq(numDocsAfter, numDocsBefore, msg);
 
             // If the moveChunk operation succeeded, verify that each mongos sees all data in the
             // chunk's range on only the toShard. If the operation failed, verify that each mongos
             // sees all data in the chunk's range on only the fromShard.
             var shardsForChunk =
                 ChunkHelper.getShardsForRange(mongos, ns, chunk.min._id, chunk.max._id);
-            var msg =
+            msg =
                 msgBase + '\nMongos find().explain() results for chunk: ' + tojson(shardsForChunk);
-            assertWhenOwnColl.eq(shardsForChunk.shards.length, 1, msg);
+            assert.eq(shardsForChunk.shards.length, 1, msg);
             if (moveChunkRes.ok) {
                 msg = 'moveChunk succeeded but chunk was not on new shard.\n' + msg;
-                assertWhenOwnColl.eq(shardsForChunk.shards[0], toShard, msg);
+                assert.eq(shardsForChunk.shards[0], toShard, msg);
             } else {
                 msg = 'moveChunk failed but chunk was not on original shard.\n' + msg;
-                assertWhenOwnColl.eq(shardsForChunk.shards[0], fromShard, msg);
+                assert.eq(shardsForChunk.shards[0], fromShard, msg);
             }
 
             // If the moveChunk operation succeeded, verify that each mongos updated the chunk's
             // shard metadata with the toShard. If the operation failed, verify that each mongos
             // still sees the chunk's shard metadata as the fromShard.
-            var chunkAfter = mongos.getDB('config').chunks.findOne({_id: chunk._id});
-            var msg =
+            chunkAfter = mongos.getDB('config').chunks.findOne({_id: chunk._id});
+            msg =
                 msgBase + '\nchunkBefore: ' + tojson(chunk) + '\nchunkAfter: ' + tojson(chunkAfter);
             if (moveChunkRes.ok) {
                 msg = "moveChunk succeeded but chunk's shard was not new shard.\n" + msg;
-                assertWhenOwnColl.eq(chunkAfter.shard, toShard, msg);
+                assert.eq(chunkAfter.shard, toShard, msg);
             } else {
                 msg = "moveChunk failed but chunk's shard was not original shard.\n" + msg;
-                assertWhenOwnColl.eq(chunkAfter.shard, fromShard, msg);
+                assert.eq(chunkAfter.shard, fromShard, msg);
             }
         }
     };
