@@ -1,5 +1,3 @@
-'use strict';
-
 /**
  * Perform point-in-time snapshot reads that span a 'find' and multiple 'getmore's concurrently with
  * CRUD operations.
@@ -11,19 +9,19 @@
  *   requires_replication,
  * ]
  */
+import {interruptedQueryErrors} from "jstests/concurrency/fsm_libs/assert.js";
+import {
+    doSnapshotFindAtClusterTime,
+    doSnapshotGetMoreAtClusterTime
+} from "jstests/concurrency/fsm_workload_helpers/snapshot_read_utils.js";
 
-load('jstests/concurrency/fsm_workload_helpers/snapshot_read_utils.js');
-var $config = (function() {
+export const $config = (function() {
     const data = {numIds: 100, numDocsToInsertPerThread: 5, batchSize: 10};
 
     const states = {
 
         snapshotScan: function snapshotScan(db, collName) {
-            const readErrorCodes = [
-                ErrorCodes.ShutdownInProgress,
-                ErrorCodes.CursorNotFound,
-                ErrorCodes.QueryPlanKilled,
-            ];
+            const readErrorCodes = [...interruptedQueryErrors, ErrorCodes.ShutdownInProgress];
             if (!this.cursorId || this.cursorId == 0) {
                 doSnapshotFindAtClusterTime(db, collName, this, readErrorCodes, {a: 1});
             } else {
@@ -34,8 +32,8 @@ var $config = (function() {
         insertDocs: function insertDocs(db, collName) {
             for (let i = 0; i < this.numDocsToInsertPerThread; ++i) {
                 const res = db[collName].insert({x: 1});
-                assertWhenOwnColl.commandWorked(res);
-                assertWhenOwnColl.eq(1, res.nInserted);
+                assert.commandWorked(res);
+                assert.eq(1, res.nInserted);
             }
         },
 
@@ -45,9 +43,9 @@ var $config = (function() {
                     db[collName].update({a: i}, {$inc: {x: 1}});
                 } catch (e) {
                     // dropIndex can cause queries to throw if these queries yield.
-                    assertAlways.contains(e.code,
-                                          [ErrorCodes.QueryPlanKilled, ErrorCodes.OperationFailed],
-                                          'unexpected error code: ' + e.code + ': ' + e.message);
+                    assert.contains(e.code,
+                                    [ErrorCodes.QueryPlanKilled, ErrorCodes.OperationFailed],
+                                    'unexpected error code: ' + e.code + ': ' + e.message);
                 }
             }
         },
@@ -58,9 +56,9 @@ var $config = (function() {
                     db[collName].findOne({a: i});
                 } catch (e) {
                     // dropIndex can cause queries to throw if these queries yield.
-                    assertAlways.contains(e.code,
-                                          [ErrorCodes.QueryPlanKilled, ErrorCodes.OperationFailed],
-                                          'unexpected error code: ' + e.code + ': ' + e.message);
+                    assert.contains(e.code,
+                                    [ErrorCodes.QueryPlanKilled, ErrorCodes.OperationFailed],
+                                    'unexpected error code: ' + e.code + ': ' + e.message);
                 }
             }
         },
@@ -71,14 +69,14 @@ var $config = (function() {
                 db[collName].deleteOne({a: indexToDelete});
             } catch (e) {
                 // dropIndex can cause queries to throw if these queries yield.
-                assertAlways.contains(e.code,
-                                      [ErrorCodes.QueryPlanKilled, ErrorCodes.OperationFailed],
-                                      'unexpected error code: ' + e.code + ': ' + e.message);
+                assert.contains(e.code,
+                                [ErrorCodes.QueryPlanKilled, ErrorCodes.OperationFailed],
+                                'unexpected error code: ' + e.code + ': ' + e.message);
             }
         },
 
         createIndex: function createIndex(db, collName) {
-            db[collName].createIndex({a: 1}, {background: true});
+            db[collName].createIndex({a: 1});
         },
 
         dropIndex: function dropIndex(db, collName) {
@@ -122,7 +120,7 @@ var $config = (function() {
                     db.adminCommand({setParameter: 1, minSnapshotHistoryWindowInSeconds: 3600}));
             });
         }
-        assertWhenOwnColl.commandWorked(db.runCommand({create: collName}));
+        assert.commandWorked(db.runCommand({create: collName}));
         const docs = [...Array(this.numIds).keys()].map((i) => ({a: i, x: 1}));
         assert.commandWorked(db.runCommand({insert: collName, documents: docs}));
         assert.commandWorked(
