@@ -1,5 +1,3 @@
-'use strict';
-
 /**
  * Test a snapshot read spanning a find and getmore that runs concurrently with
  * killOp and txnNumber change.
@@ -9,10 +7,12 @@
  * @tags: [uses_transactions, state_functions_share_transaction]
  */
 
-load('jstests/concurrency/fsm_libs/extend_workload.js');                     // for extendWorkload
-load('jstests/concurrency/fsm_workloads/snapshot_read_kill_operations.js');  // for $config
+import {extendWorkload} from "jstests/concurrency/fsm_libs/extend_workload.js";
+import {
+    $config as $baseConfig
+} from "jstests/concurrency/fsm_workloads/snapshot_read_kill_operations.js";
 
-var $config = extendWorkload($config, function($config, $super) {
+export const $config = extendWorkload($baseConfig, function($config, $super) {
     $config.transitions = {
         init: {snapshotFind: 1.0},
         snapshotFind: {incrementTxnNumber: 0.33, killOp: 0.34, snapshotGetMore: 0.33},
