@@ -1,5 +1,3 @@
-'use strict';
-
 /**
  * view_catalog_cycle_lookup.js
  *
@@ -7,18 +5,21 @@
  * views against other eachother and the underlying collection. We are looking to expose situations
  * where a $lookup or $graphLookup view that forms a cycle is created successfully.
  *
- * @tags: [requires_fcv_51]
+ * TODO SERVER-90385 Enable this test in embedded router suites
+ * @tags: [
+ *     requires_fcv_51,
+ *     temp_disabled_embedded_router_uncategorized,
+ * ]
  */
 
-var $config = (function() {
+import {FeatureFlagUtil} from "jstests/libs/feature_flag_util.js";
+import {TransactionsUtil} from "jstests/libs/transactions_util.js";
+
+export const $config = (function() {
     // Use the workload name as a prefix for the view names, since the workload name is assumed
     // to be unique.
     const prefix = 'view_catalog_cycle_lookup_';
 
-    // Store the default value of the max sub pipeline view depth so it can be reset at the end of
-    // the test.
-    var oldMaxSubPipelineViewDepth;
-
     var data = {
         viewList: ['viewA', 'viewB', 'viewC', 'viewD', 'viewE'].map(viewName => prefix + viewName),
         getRandomView: function(viewList) {
@@ -71,7 +72,7 @@ var $config = (function() {
                     case 3:
                         return [];
                     default:
-                        assertAlways(false, "Invalid index: " + index);
+                        assert(false, "Invalid index: " + index);
                 }
             },
     };
@@ -83,6 +84,9 @@ var $config = (function() {
          * is expected at view create/modification time.
          */
         function remapViewToView(db, collName) {
+            if (this.shouldSkipTest) {
+                return;
+            }
             if (this.isSBELookupEnabled) {
                 return;
             }
@@ -91,12 +95,10 @@ var $config = (function() {
             const toName = this.getRandomView(this.viewList);
             const res = db.runCommand(
                 {collMod: fromName, viewOn: toName, pipeline: this.getRandomViewPipeline()});
-            assertAlways(res.ok === 1 ||
-                             [
-                                 ErrorCodes.GraphContainsCycle,
-                                 ErrorCodes.ConflictingOperationInProgress
-                             ].includes(res.code),
-                         tojson(res));
+            assert(res.ok === 1 ||
+                       [ErrorCodes.GraphContainsCycle, ErrorCodes.ConflictingOperationInProgress]
+                           .includes(res.code),
+                   tojson(res));
         }
 
         /**
@@ -105,6 +107,9 @@ var $config = (function() {
          * error as it is expected at view create/modification time.
          */
         function remapViewToCollection(db, collName) {
+            if (this.shouldSkipTest) {
+                return;
+            }
             if (this.isSBELookupEnabled) {
                 return;
             }
@@ -112,30 +117,43 @@ var $config = (function() {
             const fromName = this.getRandomView(this.viewList);
             const res = db.runCommand(
                 {collMod: fromName, viewOn: collName, pipeline: this.getRandomViewPipeline()});
-            assertAlways(res.ok === 1 ||
-                             [
-                                 ErrorCodes.GraphContainsCycle,
-                                 ErrorCodes.ConflictingOperationInProgress
-                             ].includes(res.code),
-                         tojson(res));
+            assert(res.ok === 1 ||
+                       [ErrorCodes.GraphContainsCycle, ErrorCodes.ConflictingOperationInProgress]
+                           .includes(res.code),
+                   tojson(res));
         }
 
         function readFromView(db, collName) {
+            if (this.shouldSkipTest) {
+                return;
+            }
             if (this.isSBELookupEnabled) {
                 return;
             }
 
             const viewName = this.getRandomView(this.viewList);
             const res = db.runCommand({find: viewName});
+
+            // If we encountered some transaction failure propagate this upwards so it gets
+            // automatically retried.
+            if (TransactionsUtil.isTransientTransactionError(res)) {
+                throw res;
+            }
+
             // When initializing an aggregation on a view, the server briefly releases its
             // collection lock before creating and iterating the cursor on the underlying namespace.
             // In this short window of time, it's possible that that namespace has been dropped and
             // replaced with a view.
+            //
             // TODO (SERVER-35635): It would be more appropriate for the server to return
             // OperationFailed, as CommandNotSupportedOnView is misleading.
-            assertAlways(res.ok === 1 || res.code === ErrorCodes.CommandNotSupportedOnView ||
-                             res.code === ErrorCodes.CommandOnShardedViewNotSupportedOnMongod,
-                         () => tojson(res));
+            // TODO SERVER-XXXXX: The mergeCursors sent by mongos to mongod might get killed during
+            // a remapping and return a somewhat misleading CursorNotFound error. Ideally it should
+            // be remapped to an OperationFailed error.
+            assert(res.ok === 1 || res.code === ErrorCodes.CommandNotSupportedOnView ||
+                       res.code === ErrorCodes.CursorNotFound ||
+                       res.code === ErrorCodes.CommandOnShardedViewNotSupportedOnMongod,
+                   () => tojson(res));
         }
 
         return {
@@ -153,15 +171,23 @@ var $config = (function() {
     };
 
     function setup(db, collName, cluster) {
+        // TODO SERVER-88936: Remove this field and associated checks once the flag is active on
+        // last-lts.
+        this.shouldSkipTest = TestData.runInsideTransaction && cluster.isSharded() &&
+            !FeatureFlagUtil.isPresentAndEnabled(db.getMongo(), 'AllowAdditionalParticipants');
+        if (this.shouldSkipTest) {
+            return;
+        }
+
         const coll = db[collName];
 
-        assertAlways.commandWorked(coll.insert({a: 1, b: 2}));
-        assertAlways.commandWorked(coll.insert({a: 2, b: 3}));
-        assertAlways.commandWorked(coll.insert({a: 3, b: 4}));
-        assertAlways.commandWorked(coll.insert({a: 4, b: 1}));
+        assert.commandWorked(coll.insert({a: 1, b: 2}));
+        assert.commandWorked(coll.insert({a: 2, b: 3}));
+        assert.commandWorked(coll.insert({a: 3, b: 4}));
+        assert.commandWorked(coll.insert({a: 4, b: 1}));
 
         for (let viewName of this.viewList) {
-            assertAlways.commandWorked(db.createView(viewName, collName, []));
+            assert.commandWorked(db.createView(viewName, collName, []));
         }
 
         // We need to increase the maximum sub-pipeline view depth for this test since sharded view
@@ -176,7 +202,7 @@ var $config = (function() {
             assert(maxSubPipelineViewDepthParam.hasOwnProperty("internalMaxSubPipelineViewDepth"));
             this.oldMaxSubPipelineViewDepth =
                 maxSubPipelineViewDepthParam.internalMaxSubPipelineViewDepth;
-            assertAlways.commandWorked(
+            assert.commandWorked(
                 db.adminCommand({setParameter: 1, internalMaxSubPipelineViewDepth: 100}));
         });
     }
@@ -185,7 +211,7 @@ var $config = (function() {
         // Restore the old max subpipeline view depth.
         if (this.oldMaxSubPipelineViewDepth) {
             cluster.executeOnMongodNodes((db) => {
-                assertAlways.commandWorked(db.adminCommand({
+                assert.commandWorked(db.adminCommand({
                     setParameter: 1,
                     internalMaxSubPipelineViewDepth: this.oldMaxSubPipelineViewDepth
                 }));
