@@ -1,7 +1,7 @@
 /*
- * The test runs commands that are not allowed with security token: applyOps.
  * @tags: [
- *   not_allowed_with_security_token,
+ *   # The test runs commands that are not allowed with security token: applyOps.
+ *   not_allowed_with_signed_security_token,
  *   assumes_superuser_permissions,
  *   requires_fastcount,
  *   requires_non_retryable_commands,
@@ -18,11 +18,6 @@
  * ]
  */
 
-(function() {
-"use strict";
-
-load("jstests/libs/index_catalog_helpers.js");
-
 var t = db.apply_ops1;
 t.drop();
 
@@ -534,4 +529,3 @@ assert.commandWorked(t.runCommand(
 assert.commandWorked(t.createIndex({a: 1}));
 assert.commandWorked(t.runCommand(
     {applyOps: [{op: 'c', ns: t.getFullName(), o: {dropIndexes: 'testColl', index: 'a_1'}}]}));
-})();
