@@ -1,8 +1,9 @@
 /**
  * Tests that applyOps can include operations on the system.views namespace.
- * The test runs commands that are not allowed with security token: applyOps.
+ *
  * @tags: [
- *   not_allowed_with_security_token,
+ *   # The test runs commands that are not allowed with security token: applyOps.
+ *   not_allowed_with_signed_security_token,
  *   # applyOps is not supported on mongos
  *   assumes_against_mongod_not_mongos,
  *   assumes_superuser_permissions,
@@ -14,9 +15,6 @@
  * ]
  */
 
-(function() {
-"use strict";
-
 const backingColl = db.apply_ops_system_dot_views_backing;
 backingColl.drop();
 const view = db.apply_ops_view;
@@ -44,4 +42,3 @@ const ops = [{
 
 assert.commandWorked(db.adminCommand({applyOps: ops}));
 assert.eq(view.find({}, {_id: 0, a: 1}).toArray(), [{a: 0}]);
-}());
