@@ -1,17 +1,15 @@
 // SERVER-16676 Make sure compact doesn't leave the collection with bad indexes
 // SERVER-16967 Make sure compact doesn't crash while collections are being dropped
 // in a different database.
-// The test runs commands that are not allowed with security token: compact.
+//
 // @tags: [
-//   not_allowed_with_security_token,
+//   # The test runs commands that are not allowed with security token: compact.
+//   not_allowed_with_signed_security_token,
 //   uses_multiple_connections,
 //   uses_parallel_shell,
 //   uses_compact,
 // ]
 
-(function() {
-'use strict';
-
 var coll = db.compact_keeps_indexes;
 
 coll.drop();
@@ -23,8 +21,8 @@ assert.eq(coll.getIndexes().length, 2);
 // force:true is for replset passthroughs
 var res = coll.runCommand('compact', {force: true});
 // Some storage engines (for example, inMemoryExperiment) do not support the compact command.
-if (res.code == 115) {  // CommandNotSupported
-    return;
+if (res.code == 115) {
+    quit();
 }
 assert.commandWorked(res);
 
@@ -44,4 +42,3 @@ for (var i = 0; i < 10; i++) {
     coll.runCommand('compact');
 }
 dropCollectionShell();
-}());
