@@ -1,11 +1,13 @@
-// The test runs commands that are not allowed with security token: getLog.
 // @tags: [
-//   not_allowed_with_security_token,does_not_support_stepdowns]
+//   # The test runs commands that are not allowed with security token: getLog.
+//   not_allowed_with_signed_security_token,
+//   does_not_support_stepdowns
+// ]
 
 // to run:
 //   ./mongo jstests/<this-file>
 
-contains = function(arr, obj) {
+let contains = function(arr, obj) {
     var i = arr.length;
     while (i--) {
         if (arr[i] === obj) {
