@@ -8,14 +8,11 @@
  *   assumes_against_mongod_not_mongos,
  * ]
  */
-(function() {
-"use strict";
-
-load("jstests/libs/clustered_collections/clustered_collection_util.js");
-load("jstests/libs/clustered_collections/clustered_collection_create_index_clustered_common.js");
+import {
+    CreateIndexesClusteredTest
+} from "jstests/libs/clustered_collections/clustered_collection_create_index_clustered_common.js";
 
 const replicatedDB = db.getSiblingDB("create_index_clustered");
 const collName = "coll";
 
-CreateIndexesClusteredTest.runBaseTests(replicatedDB, collName);
-})();
+CreateIndexesClusteredTest.runBaseTests(replicatedDB, collName);
\ No newline at end of file
