@@ -6,13 +6,9 @@
 //   does_not_support_transactions,
 //   tenant_migration_incompatible
 // ]
+import {assertArrayEq} from "jstests/aggregation/extras/utils.js";
 
-(function() {
-"use strict";
-
-load('jstests/aggregation/extras/utils.js');  // For assertArrayEq().
-
-const coll = db.getCollection('command_let_variables_expressions');
+const coll = db.getCollection(jsTestName());
 
 function setupColl() {
     coll.drop();
@@ -211,4 +207,3 @@ const literalLetParam = {
         ],
     });
 }
-}());
