@@ -1,17 +1,17 @@
-// The test runs commands that are not allowed with security token: dbhash.
 // @tags: [
-//   not_allowed_with_security_token,
+//   # The test runs commands that are not allowed with security token: dbhash.
+//   not_allowed_with_signed_security_token,
 // ]
 
-a = db.dbhasha;
-b = db.dbhashb;
+let a = db.dbhasha;
+let b = db.dbhashb;
 
 a.drop();
 b.drop();
 
 // debug SERVER-761
 db.getCollectionNames().forEach(function(x) {
-    v = db[x].validate();
+    let v = db[x].validate();
     if (!v.valid) {
         print(x);
         printjson(v);
@@ -45,8 +45,8 @@ assert.neq(gh(a), gh(b), "A2");
 b.insert({_id: 5});
 assert.eq(gh(a), gh(b), "A3");
 
-dba = db.getSiblingDB("dbhasha");
-dbb = db.getSiblingDB("dbhashb");
+let dba = db.getSiblingDB("dbhasha");
+let dbb = db.getSiblingDB("dbhashb");
 
 dba.dropDatabase();
 dbb.dropDatabase();
@@ -61,3 +61,6 @@ assert.neq(dbh(dba), dbh(dbb), "C2");
 dbb.foo.insert({_id: 5});
 assert.eq(gh(dba.foo, dba), gh(dbb.foo, dbb), "B3");
 assert.eq(dbh(dba), dbh(dbb), "C3");
+
+// Validate dbHash with an empty database does not trigger an fassert/invariant.
+assert.commandFailed(db.runCommand({"dbhash": ""}));
\ No newline at end of file
