@@ -5,8 +5,6 @@
  *   requires_non_retryable_commands,
  * ]
  */
-(function() {
-
 const collName = "collMod_index_invalid_option";
 
 assert.commandWorked(db.getCollection(collName).createIndex({a: 1}, {expireAfterSeconds: 100}));
@@ -15,5 +13,4 @@ assert.commandFailedWithCode(db.runCommand({
     collMod: collName,
     index: {keyPattern: {a: 1}, expireAfterSeconds: 200, invalidOption: 1}
 }),
-                             40415 /* IDL unknown field error */);
-})();
+                             ErrorCodes.IDLUnknownField);
