@@ -5,9 +5,6 @@
  *   requires_non_retryable_writes,
  * ]
  */
-(function() {
-"use strict";
-
 const viewNs = "test.view";
 const viewDefinition = {
     _id: viewNs,
@@ -18,10 +15,11 @@ const invalidField = {
     invalidField: true
 };
 
-assert.commandFailedWithCode(db.system.views.insert(viewDefinition), ErrorCodes.InvalidNamespace);
-assert.commandFailedWithCode(db.system.views.update({}, invalidField), ErrorCodes.InvalidNamespace);
+assert.commandFailedWithCode(db["system.views"].insert(viewDefinition),
+                             ErrorCodes.InvalidNamespace);
+assert.commandFailedWithCode(db["system.views"].update({}, invalidField),
+                             ErrorCodes.InvalidNamespace);
 assert.commandFailedWithCode(
     db.runCommand({findAndModify: "system.views", query: {}, update: invalidField}),
     ErrorCodes.InvalidNamespace);
-assert.commandFailedWithCode(db.system.views.remove({}), ErrorCodes.InvalidNamespace);
-})();
+assert.commandFailedWithCode(db["system.views"].remove({}), ErrorCodes.InvalidNamespace);
\ No newline at end of file
