@@ -11,9 +11,6 @@
  *   assumes_unsharded_collection,
  * ]
  */
-(function() {
-"use strict";
-
 const coll = db.field_name_validation;
 coll.drop();
 
@@ -265,5 +262,4 @@ assert.commandFailed(db.runCommand({
 assert.commandFailed(db.runCommand({
     aggregate: coll.getName(),
     pipeline: [{$group: {_id: "$_id.a", "$invalid": {$sum: "$_id.b"}}}]
-}));
-})();
+}));
\ No newline at end of file
