@@ -1,4 +1,4 @@
-t = db.geo9;
+let t = db.geo9;
 t.drop();
 
 t.save({_id: 1, a: [10, 10], b: [50, 50]});
@@ -15,7 +15,7 @@ t.createIndex({b: "2d"});
 function check(field) {
     var q = {};
     q[field] = {$near: [11, 11]};
-    arr = t.find(q).limit(3).map(function(z) {
+    let arr = t.find(q).limit(3).map(function(z) {
         return Geo.distance([11, 11], z[field]);
     });
     assert.eq(2 * Math.sqrt(2), Array.sum(arr), "test " + field);
