@@ -3,9 +3,6 @@
  * @tags: [
  * ]
  */
-(function() {
-"use strict";
-
 const coll = db.geooobsphere;
 coll.drop();
 
@@ -40,5 +37,4 @@ assert.commandFailedWithCode(coll.runCommand("aggregate", {
         }
     }]
 }),
-                             2);
-}());
+                             2);
\ No newline at end of file
