@@ -1,6 +1,6 @@
 // Make sure that we don't return several of the same result due to faulty
 // assumptions about the btree cursor.  That is, don't return duplicate results.
-t = db.geo_s2dedupnear;
+let t = db.geo_s2dedupnear;
 t.drop();
 
 t.createIndex({geo: "2dsphere"});
@@ -9,5 +9,5 @@ var x = {
     "coordinates": [[[100.0, 0.0], [101.0, 0.0], [101.0, 1.0], [100.0, 1.0], [100.0, 0.0]]]
 };
 t.insert({geo: x});
-res = t.find({geo: {$geoNear: {"type": "Point", "coordinates": [31, 41]}}});
+let res = t.find({geo: {$geoNear: {"type": "Point", "coordinates": [31, 41]}}});
 assert.eq(res.itcount(), 1);
