@@ -19,10 +19,10 @@
  * using the $planCacheStats aggregation metadata source on the same query shape with the index
  * filters.
  *
- * The test runs commands that are not allowed with security token: planCacheClearFilters,
- * planCacheListFilters, planCacheSetFilter.
  * @tags: [
- *   not_allowed_with_security_token,
+ *   # The test runs commands that are not allowed with security token: planCacheClearFilters,
+ *   # planCacheListFilters, planCacheSetFilter.
+ *   not_allowed_with_signed_security_token,
  *   # Cannot implicitly shard accessed collections because of collection existing when none
  *   # expected.
  *   assumes_no_implicit_collection_creation_after_drop,
@@ -33,22 +33,47 @@
  *   assumes_read_preference_unchanged,
  *   assumes_unsharded_collection,
  *   does_not_support_stepdowns,
+ *   # Plan cache state is node-local and will not get migrated alongside user data
+ *   assumes_balancer_off,
  *   # The SBE plan cache was first enabled in 6.3.
  *   requires_fcv_63,
  *   references_foreign_collection,
  * ]
  */
 
-(function() {
-load("jstests/libs/analyze_plan.js");
-load("jstests/libs/clustered_collections/clustered_collection_util.js");
-load("jstests/libs/fixture_helpers.js");  // For 'FixtureHelpers'.
-load("jstests/libs/sbe_util.js");         // For checkSbeRestrictedOrFullyEnabled.
+import {
+    getOptimizer,
+    getPlanCacheKeyFromPipeline,
+    getPlanCacheKeyFromShape,
+    getPlanStage,
+    getQueryPlanner,
+    getSingleNodeExplain,
+    getWinningPlan,
+    getWinningPlanFromExplain,
+    isClusteredIxscan,
+    isCollscan,
+    isExpress,
+    isIdhackOrExpress,
+    isIxscan,
+    planHasStage,
+} from "jstests/libs/analyze_plan.js";
+import {
+    ClusteredCollectionUtil
+} from "jstests/libs/clustered_collections/clustered_collection_util.js";
+import {FixtureHelpers} from "jstests/libs/fixture_helpers.js";
+import {
+    checkSbeFullFeatureFlagEnabled,
+    checkSbeRestrictedOrFullyEnabled
+} from "jstests/libs/sbe_util.js";
+
+// Flag indicating if index filter commands are running through the query settings interface.
+var isIndexFiltersToQuerySettings = TestData.isIndexFiltersToQuerySettings || false;
 
 const coll = db.jstests_index_filter_commands;
-
 coll.drop();
 
+const sbePlanCacheEnabled = checkSbeFullFeatureFlagEnabled(db);
+
 // Setup the data so that plans will not tie given the indices and query
 // below. Tying plans will not be cached, and we need cached shapes in
 // order to test the filter functionality.
@@ -133,7 +158,7 @@ function planCacheEntryForPipeline(pipeline, collection) {
         collection = coll;
     }
 
-    const planCacheKey = getPlanCacheKeyFromPipeline(pipeline, collection, db);
+    const planCacheKey = getPlanCacheKeyFromPipeline(pipeline, collection);
     const res = collection.getPlanCache().list([{$match: {planCacheKey: planCacheKey}}]);
     if (res.length === 0) {
         return null;
@@ -188,7 +213,7 @@ assert.eq(null, planCacheEntryForQuery(shape), coll.getPlanCache().list());
 
 // Check details of winning plan in plan cache after setting filter and re-executing query.
 assert.eq(1, coll.find(queryA1, projectionA1).sort(sortA1).itcount(), 'unexpected document count');
-planAfterSetFilter = planCacheEntryForQuery(shape);
+let planAfterSetFilter = planCacheEntryForQuery(shape);
 assert.neq(null, planAfterSetFilter, coll.getPlanCache().list());
 // Check 'indexFilterSet' field in plan details
 assert.eq(true, planAfterSetFilter.indexFilterSet, planAfterSetFilter);
@@ -204,13 +229,26 @@ assert.commandWorked(coll.runCommand('planCacheSetFilter', {query: queryID, inde
 var explain = coll.explain("executionStats").find(queryID).finish();
 assert.commandWorked(explain);
 
-const winningPlan = getWinningPlan(explain.queryPlanner);
+explain = getSingleNodeExplain(explain);
+const queryPlanner = getQueryPlanner(explain);
+const winningPlan = getWinningPlan(queryPlanner);
 const collectionIsClustered = ClusteredCollectionUtil.areAllCollectionsClustered(db.getMongo());
 if (collectionIsClustered) {
-    assert(isClusteredIxscan(db, getWinningPlan(explain.queryPlanner)),
-           "Expected clustered ixscan: " + tojson(explain));
+    assert(isExpress(db, getWinningPlan(queryPlanner)), "Expected Express: " + tojson(explain));
 } else {
-    assert(isIdhack(db, winningPlan), winningPlan);
+    switch (getOptimizer(explain)) {
+        case "classic":
+            assert(isIdhackOrExpress(db, winningPlan), winningPlan);
+            break;
+        case "CQF":
+            // TODO SERVER-70847, how to recognize the case of an IDHACK for Bonsai?
+            // TODO SERVER-77719: Ensure that the decision for using the scan lines up with CQF
+            // optimizer. M2: allow only collscans, M4: check bonsai behavior for index scan.
+            assert(isCollscan(db, getWinningPlanFromExplain(explain)));
+            break;
+        default:
+            break;
+    }
 }
 
 // Clearing filters on a missing collection should be a no-op.
@@ -229,26 +267,20 @@ if (!FixtureHelpers.isMongos(db)) {
 
         // No filter.
         coll.getPlanCache().clear();
-        assert.eq(
-            false, coll.find({z: 1}).explain(verbosity).queryPlanner.indexFilterSet, verbosity);
-        assert.eq(false,
-                  coll.find(queryA1, projectionA1)
-                      .sort(sortA1)
-                      .explain(verbosity)
-                      .queryPlanner.indexFilterSet,
-                  verbosity);
+        explain = getSingleNodeExplain(coll.find({z: 1}).explain(verbosity));
+        assert.eq(false, getQueryPlanner(explain).indexFilterSet, explain);
+        explain =
+            getSingleNodeExplain(coll.find(queryA1, projectionA1).sort(sortA1).explain(verbosity));
+        assert.eq(false, getQueryPlanner(explain).indexFilterSet, explain);
 
         // With one filter set.
         assert.commandWorked(
             coll.runCommand('planCacheSetFilter', {query: {z: 1}, indexes: [{z: 1}]}));
-        assert.eq(
-            true, coll.find({z: 1}).explain(verbosity).queryPlanner.indexFilterSet, verbosity);
-        assert.eq(false,
-                  coll.find(queryA1, projectionA1)
-                      .sort(sortA1)
-                      .explain(verbosity)
-                      .queryPlanner.indexFilterSet,
-                  verbosity);
+        explain = getSingleNodeExplain(coll.find({z: 1}).explain(verbosity));
+        assert.eq(true, getQueryPlanner(explain).indexFilterSet, explain);
+        explain =
+            getSingleNodeExplain(coll.find(queryA1, projectionA1).sort(sortA1).explain(verbosity));
+        assert.eq(false, getQueryPlanner(explain).indexFilterSet, verbosity);
 
         // With two filters set.
         assert.commandWorked(coll.runCommand('planCacheSetFilter', {
@@ -257,14 +289,11 @@ if (!FixtureHelpers.isMongos(db)) {
             sort: sortA1,
             indexes: [indexA1B1, indexA1C1]
         }));
-        assert.eq(
-            true, coll.find({z: 1}).explain(verbosity).queryPlanner.indexFilterSet, verbosity);
-        assert.eq(true,
-                  coll.find(queryA1, projectionA1)
-                      .sort(sortA1)
-                      .explain(verbosity)
-                      .queryPlanner.indexFilterSet,
-                  verbosity);
+        explain = getSingleNodeExplain(coll.find({z: 1}).explain(verbosity));
+        assert.eq(true, getQueryPlanner(explain).indexFilterSet, explain);
+        explain =
+            getSingleNodeExplain(coll.find(queryA1, projectionA1).sort(sortA1).explain(verbosity));
+        assert.eq(true, getQueryPlanner(explain).indexFilterSet, verbosity);
     });
 } else {
     clearFilters(coll, shape);
@@ -290,12 +319,12 @@ assert.commandWorked(coll.runCommand('planCacheSetFilter',
 // Ensure that index key patterns in planCacheSetFilter select any index with a matching key
 // pattern.
 
-explain = coll.find(queryAA).explain();
-assert(isIxscan(db, getWinningPlan(explain.queryPlanner)),
+explain = getSingleNodeExplain(coll.find(queryAA).explain());
+assert(isIxscan(db, getWinningPlan(getQueryPlanner(explain))),
        "Expected index scan: " + tojson(explain));
 
-explain = coll.find(queryAA).collation(collationEN).explain();
-assert(isIxscan(db, getWinningPlan(explain.queryPlanner)),
+explain = getSingleNodeExplain(coll.find(queryAA).collation(collationEN).explain());
+assert(isIxscan(db, getWinningPlan(getQueryPlanner(explain))),
        "Expected index scan: " + tojson(explain));
 
 // Ensure that index names in planCacheSetFilter only select matching names.
@@ -303,8 +332,8 @@ assert(isIxscan(db, getWinningPlan(explain.queryPlanner)),
 assert.commandWorked(coll.runCommand('planCacheSetFilter',
                                      {query: queryAA, collation: collationEN, indexes: ["a_1"]}));
 
-explain = coll.find(queryAA).collation(collationEN).explain();
-assert(isCollscan(db, getWinningPlan(explain.queryPlanner)),
+explain = getSingleNodeExplain(coll.find(queryAA).collation(collationEN).explain());
+assert(isCollscan(db, getWinningPlan(getQueryPlanner(explain))),
        "Expected collscan: " + tojson(explain));
 
 //
@@ -326,11 +355,16 @@ assert.commandWorked(
 filters = getFilters();
 assert.eq(0, filters.length, tojson(filters));
 
+// The code below tests specifics of index filter implementation and therefore are not run in
+// 'index_filters_to_query_settings' suite.
+if (isIndexFiltersToQuerySettings) {
+    quit();
+}
+
 //
 // Test that planCacheSetFilter and planCacheClearFilters do not allow queries containing $expr with
 // unbound variables.
 //
-
 assert(coll.drop());
 assert.commandWorked(coll.insert({a: "a"}));
 assert.commandWorked(coll.createIndex(indexA1, {name: "a_1"}));
@@ -384,95 +418,106 @@ if (checkSbeRestrictedOrFullyEnabled(db)) {
     let results = coll.aggregate(pipeline).toArray();
 
     // Check details of the cached plan.
-    assert.eq(1, results.length, results);
-    planAfterSetFilter = planCacheEntryForPipeline(pipeline);
-    assert.neq(null, planAfterSetFilter, coll.getPlanCache().list());
-    // Check 'indexFilterSet' field in plan details - no index filters should be applied.
-    assert.eq(false, planAfterSetFilter.indexFilterSet, planAfterSetFilter);
+    if (sbePlanCacheEnabled) {
+        assert.eq(1, results.length, results);
+        planAfterSetFilter = planCacheEntryForPipeline(pipeline);
+        assert.neq(null, planAfterSetFilter, coll.getPlanCache().list());
+        // Check 'indexFilterSet' field in plan details - no index filters should be applied.
+        assert.eq(false, planAfterSetFilter.indexFilterSet, planAfterSetFilter);
+    }
 
     // Ensure that despite an index filter being set on the foreign collection, we're still using
     // heuristics to select an INLJ plan. This can be proved by showing that the index being used is
     // the 'foreign_a_1' one, rather than 'foreign_a_1_c_1' specified in the index filter, as the
     // INLJ heuristics always prefer an index with a narrower key pattern.
     explain = coll.explain().aggregate(pipeline);
-    assert.commandWorked(explain);
 
-    let lookupStage = getPlanStage(explain, "EQ_LOOKUP");
-    assert.neq(null, lookupStage, explain);
-    assert.eq(lookupStage.strategy, "IndexedLoopJoin", explain);
-    assert.eq(lookupStage.indexName, "foreign_a_1");
+    // Cannot check when pipeline is split, because we won't push down to SBE in this case.`
+    if (!explain.splitPipeline) {
+        assert.commandWorked(explain);
+        explain = getSingleNodeExplain(explain);
 
-    // Now, add the same set of indexes to the main collection as defined on the foreign collection.
-    assert.commandWorked(coll.createIndex(indexA1, {name: "main_a_1"}));
-    assert.commandWorked(coll.createIndex(indexA1B1, {name: "main_a_1_b_1"}));
-    assert.commandWorked(coll.createIndex(indexA1C1, {name: "main_a_1_c_1"}));
+        let lookupStage = getPlanStage(explain, "EQ_LOOKUP");
+        assert.neq(null, lookupStage, explain);
+        assert.eq(lookupStage.strategy, "IndexedLoopJoin", explain);
+        assert.eq(lookupStage.indexName, "foreign_a_1");
 
-    // Add the same index filter on the main collection as defined on the foreign collection.
-    assert.commandWorked(
-        coll.runCommand("planCacheSetFilter", {query: queryA1, indexes: [indexA1C1]}));
-    filters = getFilters(coll);
-    assert.eq(1, filters.length, filters);
-    assert.eq(queryA1, filters[0].query, filters);
-    assert.eq(1, filters[0].indexes.length, filters);
-    assert.eq(indexA1C1, filters[0].indexes[0], filters);
+        // Now, add the same set of indexes to the main collection as defined on the foreign
+        // collection.
+        assert.commandWorked(coll.createIndex(indexA1, {name: "main_a_1"}));
+        assert.commandWorked(coll.createIndex(indexA1B1, {name: "main_a_1_b_1"}));
+        assert.commandWorked(coll.createIndex(indexA1C1, {name: "main_a_1_c_1"}));
 
-    // Make sure we still have one index filter defined on the foreign collection.
-    assert.commandWorked(
-        foreignColl.runCommand("planCacheSetFilter", {query: queryA1, indexes: [indexA1C1]}));
-    filters = getFilters(foreignColl);
-    assert.eq(1, filters.length, filters);
+        // Add the same index filter on the main collection as defined on the foreign collection.
+        assert.commandWorked(
+            coll.runCommand("planCacheSetFilter", {query: queryA1, indexes: [indexA1C1]}));
+        filters = getFilters(coll);
+        assert.eq(1, filters.length, filters);
+        assert.eq(queryA1, filters[0].query, filters);
+        assert.eq(1, filters[0].indexes.length, filters);
+        assert.eq(indexA1C1, filters[0].indexes[0], filters);
 
-    // Re-run the pipeline.
-    results = coll.aggregate(pipeline).toArray();
+        // Make sure we still have one index filter defined on the foreign collection.
+        assert.commandWorked(
+            foreignColl.runCommand("planCacheSetFilter", {query: queryA1, indexes: [indexA1C1]}));
+        filters = getFilters(foreignColl);
+        assert.eq(1, filters.length, filters);
 
-    // Check details of the cached plan.
-    assert.eq(1, results.length, results);
-    planAfterSetFilter = planCacheEntryForPipeline(pipeline);
-    assert.neq(null, planAfterSetFilter, coll.getPlanCache().list());
-    // Check 'indexFilterSet' field in plan details - an index filter should be applied.
-    assert.eq(true, planAfterSetFilter.indexFilterSet, planAfterSetFilter);
+        // Re-run the pipeline.
+        results = coll.aggregate(pipeline).toArray();
 
-    // Check that the inner side was still using the heursitics to select an INLJ plan, and the
-    // outer side honoured the index filter.
-    explain = coll.explain().aggregate(pipeline);
-    assert.commandWorked(explain);
+        // Check details of the cached plan, when SBE plan cache is enabled.
+        if (sbePlanCacheEnabled) {
+            assert.eq(1, results.length, results);
+            planAfterSetFilter = planCacheEntryForPipeline(pipeline);
+            assert.neq(null, planAfterSetFilter, coll.getPlanCache().list());
+            // Check 'indexFilterSet' field in plan details - an index filter should be applied.
+            assert.eq(true, planAfterSetFilter.indexFilterSet, planAfterSetFilter);
+        }
 
-    lookupStage = getPlanStage(explain, "EQ_LOOKUP");
-    assert.neq(null, lookupStage, explain);
-    assert.eq(lookupStage.strategy, "IndexedLoopJoin", explain);
-    assert.eq(lookupStage.indexName, "foreign_a_1");
+        // Check that the inner side was still using the heursitics to select an INLJ plan, and the
+        // outer side honoured the index filter.
+        explain = coll.explain().aggregate(pipeline);
+        assert.commandWorked(explain);
+        explain = getSingleNodeExplain(explain);
+        lookupStage = getPlanStage(explain, "EQ_LOOKUP");
+        assert.neq(null, lookupStage, explain);
+        assert.eq(lookupStage.strategy, "IndexedLoopJoin", explain);
+        assert.eq(lookupStage.indexName, "foreign_a_1");
 
-    ixscanStage = getPlanStage(explain, "IXSCAN");
-    assert.neq(null, ixscanStage, explain);
-    assert.eq(ixscanStage.indexName, "main_a_1_c_1", explain);
+        let ixscanStage = getPlanStage(explain, "IXSCAN");
+        assert.neq(null, ixscanStage, explain);
+        assert.eq(ixscanStage.indexName, "main_a_1_c_1", explain);
 
-    //
-    // Test that planCacheClearFilters only clears plan cache entries on the main collection.
-    //
+        //
+        // Test that planCacheClearFilters only clears plan cache entries on the main collection.
+        //
 
-    // Clear the index filter on the foreign collection and check that the plan cache entry for the
-    // $lookup still exists.
-    assert.commandWorked(foreignColl.runCommand("planCacheClearFilters", {query: queryA1}));
-    filters = getFilters(foreignColl);
-    assert.eq(0, filters.length, filters);
+        // Clear the index filter on the foreign collection and check that the plan cache entry for
+        // the $lookup still exists.
+        assert.commandWorked(foreignColl.runCommand("planCacheClearFilters", {query: queryA1}));
+        filters = getFilters(foreignColl);
+        assert.eq(0, filters.length, filters);
 
-    filters = getFilters(coll);
-    assert.eq(1, filters.length, filters);
-    assert.eq(queryA1, filters[0].query, filters);
-    assert.eq(1, filters[0].indexes.length, filters);
-    assert.eq(indexA1C1, filters[0].indexes[0], filters);
+        filters = getFilters(coll);
+        assert.eq(1, filters.length, filters);
+        assert.eq(queryA1, filters[0].query, filters);
+        assert.eq(1, filters[0].indexes.length, filters);
+        assert.eq(indexA1C1, filters[0].indexes[0], filters);
 
-    let planCacheEntry = planCacheEntryForPipeline(pipeline);
-    assert.neq(null, planCacheEntry, coll.getPlanCache().list());
-    assert.eq(true, planCacheEntry.indexFilterSet, planCacheEntry);
+        if (sbePlanCacheEnabled) {
+            let planCacheEntry = planCacheEntryForPipeline(pipeline);
+            assert.neq(null, planCacheEntry, coll.getPlanCache().list());
+            assert.eq(true, planCacheEntry.indexFilterSet, planCacheEntry);
 
-    // Clear the index filter on the main collection and ensure that the plan is no longer in the
-    // cache.
-    assert.commandWorked(coll.runCommand("planCacheClearFilters", {query: queryA1}));
-    filters = getFilters(coll);
-    assert.eq(0, filters.length, filters);
+            // Clear the index filter on the main collection and ensure that the plan is no longer
+            // in the cache.
+            assert.commandWorked(coll.runCommand("planCacheClearFilters", {query: queryA1}));
+            filters = getFilters(coll);
+            assert.eq(0, filters.length, filters);
 
-    planCacheEntry = planCacheEntryForPipeline(pipeline);
-    assert.eq(null, planCacheEntry, coll.getPlanCache().list());
+            planCacheEntry = planCacheEntryForPipeline(pipeline);
+            assert.eq(null, planCacheEntry, coll.getPlanCache().list());
+        }
+    }
 }
-}());
