@@ -2,9 +2,6 @@
 // Previously, we would disallow index creation with with too long a name.
 // @tags: [requires_non_retryable_commands, assumes_unsharded_collection]
 
-(function() {
-'use strict';
-
 const coll = db.long_index_rename;
 coll.drop();
 
@@ -18,5 +15,4 @@ assert.commandWorked(coll.createIndex({b: 1}, {name: 'a'.repeat(8192)}));
 // Before 4.2, index namespace lengths were checked while renaming collections.
 const dest = db.long_index_rename2;
 dest.drop();
-assert.commandWorked(coll.renameCollection(dest.getName()));
-})();
+assert.commandWorked(coll.renameCollection(dest.getName()));
\ No newline at end of file
