@@ -1,6 +1,6 @@
-// The test runs commands that are not allowed with security token: mapReduce.
 // @tags: [
-//   not_allowed_with_security_token,
+//   # The test runs commands that are not allowed with security token: mapReduce.
+//   not_allowed_with_signed_security_token,
 //   does_not_support_stepdowns,
 //   requires_fastcount,
 //   requires_getmore,
@@ -11,9 +11,7 @@
 //   requires_scripting,
 // ]
 
-load("jstests/aggregation/extras/utils.js");  // For resultsEq
-(function() {
-"use strict";
+import {resultsEq} from "jstests/aggregation/extras/utils.js";
 
 const coll = db.map_reduce_subplanning;
 coll.drop();
@@ -21,8 +19,9 @@ db.getCollection("mrOutput").drop();
 
 coll.createIndex({a: 1, c: 1});
 coll.createIndex({b: 1, c: 1});
-coll.createIndex({a: 1});
-coll.createIndex({b: 1});
+// Use descending indexes to avoid index deduplication.
+coll.createIndex({a: -1});
+coll.createIndex({b: -1});
 
 assert.commandWorked(coll.insert({a: 2}));
 assert.commandWorked(coll.insert({b: 3}));
@@ -44,4 +43,3 @@ assert.commandWorked(coll.mapReduce(
 
 assert(resultsEq([{"_id": "a", "value": 4}], db.getCollection("mrOutput").find().toArray()),
        db.getCollection("mrOutput").find().toArray());
-})();
