@@ -1,17 +1,11 @@
 // The {$set: {x: maxStr}}} update takes multiple seconds to execute.
-// @tags: [
-//   operations_longer_than_stepdown_interval,
-//   multiversion_incompatible,
-// ]
+// @tags: [operations_longer_than_stepdown_interval]
 
 /**
  * Confirms that:
  *  - Documents at the maximum BSON size limit can be written and read back.
  *  - Documents over the maximum BSON size limit cannot be written.
  */
-(function() {
-'use strict';
-
 const maxBsonObjectSize = db.hello().maxBsonObjectSize;
 const docOverhead = Object.bsonsize({_id: new ObjectId(), x: ''});
 const maxStrSize = maxBsonObjectSize - docOverhead;
@@ -54,7 +48,7 @@ coll.drop();
 assert.commandFailedWithCode(
     db.runCommand(
         {insert: coll.getName(), documents: [{_id: new ObjectId(), x: largerThanMaxString}]}),
-    ErrorCodes.BSONObjectTooLarge);
+    2);
 
 coll.drop();
 assert.commandFailedWithCode(db.runCommand({
@@ -71,5 +65,4 @@ assert.commandFailedWithCode(db.runCommand({
     ordered: true,
     updates: [{q: {_id: objectId}, u: {$set: {x: largerThanMaxString}}}]
 }),
-                             [17419, ErrorCodes.BSONObjectTooLarge]);
-})();
+                             [17419, 8375908]);
