@@ -6,9 +6,6 @@
  *   assumes_no_implicit_collection_creation_after_drop,
  * ]
  */
-(function() {
-"use strict";
-
 const collNamePrefix = 'merge_sort_collation_';
 let collCount = 0;
 
@@ -473,5 +470,4 @@ const testCases = [
     },
 ];
 
-testCases.forEach(executeQueryTestCase);
-}());
+testCases.forEach(executeQueryTestCase);
\ No newline at end of file
