@@ -1,4 +1,4 @@
-t = db.jstests_multi;
+let t = db.jstests_multi;
 t.drop();
 
 t.createIndex({a: 1});
