@@ -1,9 +1,7 @@
 // @tags: [requires_non_retryable_writes, requires_fcv_51]
 
-load("jstests/libs/analyze_plan.js");
+import {getWinningPlan, isIxscan} from "jstests/libs/analyze_plan.js";
 
-(function() {
-"use strict";
 const coll = db.partialFilterExpression_with_geoWithin;
 coll.drop();
 
@@ -163,4 +161,3 @@ results = command.toArray();
 // We expect to only find one matching result because we only have one point in our collection
 // inside the limits of our polygon (or in other words, inside the UWS of Manhattan ).
 assert.eq(results.length, 1);
-})();
