@@ -5,9 +5,6 @@
 //   operations_longer_than_stepdown_interval,  # large bulk inserts
 // ]
 
-(function() {
-'use strict';
-
 const t = db.jstests_explain5;
 t.drop();
 
@@ -40,5 +37,4 @@ const explain2 = t.find({a: {$gte: 0}, b: 2}).sort({a: 1}).hint({a: 1}).explain(
 jsTestLog('explain5 explain output after additional documents: ' + tojson(explain2));
 const stats2 = explain2.executionStats;
 assert.eq(666, stats2.nReturned, 'wrong nReturned for explain2');
-assert.eq(2000, stats2.totalKeysExamined, 'wrong totalKeysExamined for explain2');
-})();
+assert.eq(2000, stats2.totalKeysExamined, 'wrong totalKeysExamined for explain2');
\ No newline at end of file
