@@ -10,7 +10,6 @@
  * assumes_balancer_off,
  * ]
  */
-(function() {
 const coll = db.jstests_explain_large_bounds;
 coll.drop();
 
@@ -21,5 +20,4 @@ for (let i = 0; i < 1000000; i++) {
     inClause.push(i);
 }
 
-assert.commandWorked(coll.find({a: {$in: inClause}}).explain());
-}());
+assert.commandWorked(coll.find({a: {$in: inClause}}).explain());
\ No newline at end of file
