@@ -6,9 +6,6 @@
 //   requires_fcv_62,
 // ]
 
-(function() {
-"use strict";
-
 const coll = db.explain_server_params;
 coll.drop();
 
@@ -37,5 +34,4 @@ assert.hasFields(result.serverParameters, expectedParamList);
 
 result = coll.find({x: 1, y: 1}).explain('queryPlanner');
 assert(result.hasOwnProperty('serverParameters'), result);
-assert.hasFields(result.serverParameters, expectedParamList);
-})();
+assert.hasFields(result.serverParameters, expectedParamList);
\ No newline at end of file
