@@ -10,10 +10,7 @@
  *   assumes_read_concern_local,
  * ]
  */
-(function() {
-"use strict";
-
-load("jstests/libs/analyze_plan.js");
+import {getPlanStages, getWinningPlan} from "jstests/libs/analyze_plan.js";
 
 const testDB = db.getSiblingDB(jsTestName());
 assert.commandWorked(testDB.dropDatabase());
@@ -27,5 +24,4 @@ const explain = assert.commandWorked(
     coll.find({array: {$all: [[1], [2]]}}).sort({num: 1}).explain('executionStats'));
 assert.gt(
     getPlanStages(getWinningPlan(explain.queryPlanner), "SORT_MERGE").length, 0, tojson(explain));
-assert.eq(1, explain.executionStats.nReturned, tojson(explain));
-}());
+assert.eq(1, explain.executionStats.nReturned, tojson(explain));
\ No newline at end of file
