@@ -2,13 +2,12 @@
 // @tags: [
 //   assumes_read_concern_local,
 //   requires_fcv_63,
+//   # TODO SERVER-32311: re-enable test in balancer suites when the relative issue is solved.
+//   assumes_balancer_off,
 // ]
 
-(function() {
-"use strict";
-
-load("jstests/libs/analyze_plan.js");
-load("jstests/libs/sbe_util.js");  // For checkSbeFullyEnabled.
+import {getAggPlanStage, getPlanStage, hasRejectedPlans} from "jstests/libs/analyze_plan.js";
+import {checkSbeFullyEnabled} from "jstests/libs/sbe_util.js";
 
 const coll = db.expr_index_use;
 coll.drop();
@@ -330,4 +329,3 @@ confirmExpectedExprExecution({$lt: ["$w", {z: undefined, u: ["array"]}]},
                              {nReturned: 6, expectedIndex: {w: 1}});
 confirmExpectedExprExecution({$lte: ["$w", {z: undefined, u: ["array"]}]},
                              {nReturned: 7, expectedIndex: {w: 1}});
-})();
