@@ -1,6 +1,6 @@
 // SERVER-1932 Test unindexed matching of a range that is only valid in a multikey context.
 
-t = db.jstests_find8;
+let t = db.jstests_find8;
 t.drop();
 
 t.save({a: [1, 10]});
