@@ -1,10 +1,7 @@
 /**
  * Test a variety of predicates for the find command's filter, projection and sort expressions.
  */
-(function() {
-"use strict";
-
-load("jstests/aggregation/extras/utils.js");  // For arrayEq and orderedArrayEq.
+import {arrayEq, orderedArrayEq} from "jstests/aggregation/extras/utils.js";
 
 const coll = db.find_project_sort;
 coll.drop();
@@ -33,9 +30,6 @@ const documents = [
 ];
 assert.commandWorked(coll.insert(documents));
 
-assert.commandWorked(coll.createIndex({a: 1}));
-assert.commandWorked(coll.createIndex({z: 1}));
-
 function checkQuery(
     {expected = [], query = {}, proj = {}, sort = null, limit = null, skip = null, desc = null},
     hint) {
@@ -803,6 +797,9 @@ runIDHackTest();
 runCollScanTests();
 
 runFindTestsWithHint({$natural: 1});
+
+assert.commandWorked(coll.createIndex({a: 1}));
+assert.commandWorked(coll.createIndex({z: 1}));
+
 runFindTestsWithHint({a: 1});
-runFindTestsWithHint({z: 1});  // Multi-key
-}());
+runFindTestsWithHint({z: 1});  // Multi-key
\ No newline at end of file
