@@ -10,11 +10,7 @@
  *   references_foreign_collection,
  * ]
  */
-(function() {
-"use strict";
-
-load("jstests/libs/analyze_plan.js");         // For 'isCollscan()' and similar.
-load("jstests/aggregation/extras/utils.js");  // For 'resultsEq().'
+import {resultsEq} from "jstests/aggregation/extras/utils.js";
 
 const coll = db.lookup_using_hash_key;
 
@@ -46,5 +42,4 @@ let results = coll.aggregate(
 
 // We essentially just looked up ourselves for each document.
 let expected = allDocs.map(doc => Object.merge(doc, {relookup: [doc]}));
-assert(resultsEq(results, expected, true), [results, expected]);
-}());
+assert(resultsEq(results, expected, true), [results, expected]);
\ No newline at end of file
