@@ -3,7 +3,7 @@
  * correctly.
  */
 
-load("jstests/libs/collection_drop_recreate.js");  // For 'assertDropAndRecreateCollection'.
+import {assertDropAndRecreateCollection} from "jstests/libs/collection_drop_recreate.js";
 
 const coll = assertDropAndRecreateCollection(db, jsTestName());
 assert.commandWorked(db.runCommand({
