@@ -1,15 +1,14 @@
 // Tests that mapReduce preserves the indexes of the output collection, even when replacing it.
-// The test runs commands that are not allowed with security token: mapReduce.
+//
 // @tags: [
-//   not_allowed_with_security_token,
+//   # The test runs commands that are not allowed with security token: mapReduce.
+//   not_allowed_with_signed_security_token,
 //   # mapReduce does not support afterClusterTime.
 //   does_not_support_causal_consistency,
 //   does_not_support_stepdowns,
 //   uses_map_reduce_with_temp_collections,
 //   requires_scripting,
 // ]
-(function() {
-"use strict";
 const coll = db.mr_preserve_indexes;
 coll.drop();
 
@@ -45,4 +44,3 @@ assert.eq(2, out.getIndexes().length, () => tojson(out.getIndexes()));
 assert.commandWorked(coll.mapReduce(mapFn, reduceFn, {out: outName}));
 
 assert.eq(2, out.getIndexes().length, () => tojson(out.getIndexes()));
-}());
