@@ -1,7 +1,6 @@
-
-a = new ObjectId("4c1a478603eba73620000000");
-b = new ObjectId("4c1a478603eba73620000000");
-c = new ObjectId();
+let a = new ObjectId("4c1a478603eba73620000000");
+let b = new ObjectId("4c1a478603eba73620000000");
+let c = new ObjectId();
 
 assert.eq(a.toString(), b.toString(), "A");
 assert.eq(a.toString(), "ObjectId(\"4c1a478603eba73620000000\")", "B");
