@@ -11,17 +11,14 @@
  *   # Columnstore tests set server parameters to disable columnstore query planning heuristics -
  *   # server parameters are stored in-memory only so are not transferred onto the recipient.
  *   tenant_migration_incompatible,
- *   not_allowed_with_security_token,
+ *   not_allowed_with_signed_security_token,
  *   featureFlagSbeFull,
  * ]
  */
-(function() {
-"use strict";
-
-load("jstests/libs/columnstore_util.js");  // For setUpServerForColumnStoreIndexTest.
+import {setUpServerForColumnStoreIndexTest} from "jstests/libs/columnstore_util.js";
 
 if (!setUpServerForColumnStoreIndexTest(db)) {
-    return;
+    quit();
 }
 
 const coll = db.projection_semantics;
@@ -716,4 +713,3 @@ function testInputOutput({input, projection, expectedOutput, interestingIndexes
         expectedOutput: {measurements: {temperature: 20, pressure: 0.7}},
     });
 }());
-}());
