@@ -1,14 +1,13 @@
-(function() {
-t = db.push2;
+let t = db.push2;
 t.drop();
 
 t.save({_id: 1, a: []});
 
-s = new Array(700000).toString();
+let s = new Array(700000).toString();
 
-gotError = null;
+let gotError = null;
 
-for (x = 0; x < 100; x++) {
+for (let x = 0; x < 100; x++) {
     print(x + " pushes");
     var res = t.update({}, {$push: {a: s}});
     gotError = res.hasWriteError();
@@ -19,4 +18,3 @@ for (x = 0; x < 100; x++) {
 assert(gotError, "should have gotten error");
 
 t.drop();
-})();
