@@ -1,4 +1,4 @@
-t = db.regex_matches_self;
+let t = db.regex_matches_self;
 t.drop();
 
 t.insert({r: /^a/});
@@ -23,4 +23,4 @@ assert.eq(/^b/, t.findOne({r: /^b/}).r, '3 1 b');
 assert.eq(1, t.count({r: /^b/}), '3 2 b');
 
 t.insert({r: "a"});
-assert.eq(2, t.count({r: /^a/}), 'c');
\ No newline at end of file
+assert.eq(2, t.count({r: /^a/}), 'c');
