@@ -2,8 +2,6 @@
  * Test regex options in a find context.
  * @tags: [requires_fcv_51]
  */
-(function() {
-'use strict';
 const coll = db.jstests_regex_options;
 
 coll.drop();
@@ -35,5 +33,4 @@ assert.eq(1,
 assert.commandFailedWithCode(
     assert.throws(() => coll.find({}, {p: {$regexFind: {input: '$txt', regex: /^hello.*/g}}})
                             .itcount()),
-                 51108);
-})();
+                 51108);
\ No newline at end of file
