@@ -1,6 +1,6 @@
 // SERVER-1735 $type:10 matches null value, not missing value.
 
-t = db.jstests_type2;
+let t = db.jstests_type2;
 t.drop();
 
 t.save({a: null});
@@ -16,4 +16,4 @@ function test() {
 
 test();
 t.createIndex({a: 1});
-test();
\ No newline at end of file
+test();
