@@ -2,9 +2,6 @@
 //   no_selinux,
 // ]
 
-(function() {
-"use strict";
-
 // SERVER-8246 Min/MaxKey should be comparable
 //
 // make sure that the MinKey MaxKey JS types are comparable
@@ -18,5 +15,4 @@ function testType(t1, t2) {
     assert(!(doc._id instanceof t2), "Value for " + t1 + "shouldn't be instance of" + t2);
 }
 testType(MinKey, MaxKey);
-testType(MaxKey, MinKey);
-})();
+testType(MaxKey, MinKey);
\ No newline at end of file
