@@ -4,9 +4,6 @@
 //   assumes_standalone_mongod,
 // ]
 
-(function() {
-"use strict";
-
 var currentTime = new Date();
 
 var futureOpTime = new Timestamp((currentTime / 1000 + 3600), 0);
@@ -14,5 +11,4 @@ var futureOpTime = new Timestamp((currentTime / 1000 + 3600), 0);
 assert.commandFailedWithCode(
     db.runCommand(
         {find: 'user', filter: {x: 1}, readConcern: {afterOpTime: {ts: futureOpTime, t: 0}}}),
-    [ErrorCodes.NotAReplicaSet, ErrorCodes.NotImplemented]);
-})();
+    [ErrorCodes.NotAReplicaSet, ErrorCodes.NotImplemented]);
\ No newline at end of file
