@@ -2,14 +2,15 @@
 // the past. See SERVER-19614.
 //
 // @tags: [
+//   # The test runs commands that are not allowed with security token: mapReduce.
+//   not_allowed_with_signed_security_token,
 //   does_not_support_stepdowns,
 //   requires_non_retryable_commands,
 //   uses_map_reduce_with_temp_collections,
+//   # This test has statements that do not support non-local read concern.
+//   does_not_support_causal_consistency,
 // ]
 
-(function() {
-"use strict";
-
 db.recursion.drop();
 
 // Make sure the shell doesn't blow up.
@@ -32,4 +33,3 @@ function mapReduceRecursion() {
 
 assert.commandWorked(db.recursion.insert({}));
 assert.commandFailedWithCode(assert.throws(mapReduceRecursion), ErrorCodes.JSInterpreterFailure);
-}());
