@@ -9,17 +9,13 @@
 //   does_not_support_stepdowns,
 //   # The SBE plan cache was first enabled in 6.3.
 //   requires_fcv_63,
-//   # Plan cache state is node-local and will not get migrated alongside tenant data.
+//   # Plan cache state is node-local and will not get migrated alongside user data.
 //   tenant_migration_incompatible,
-//   # TODO SERVER-67607: Test plan cache with CQF enabled.
-//   cqf_incompatible,
+//   assumes_balancer_off,
 //   featureFlagSbeFull,
 // ]
 
-(function() {
-"use strict";
-
-load("jstests/libs/analyze_plan.js");
+import {getPlanCacheKeyFromShape, getWinningPlan, planHasStage} from "jstests/libs/analyze_plan.js";
 
 function getPlanCacheEntries(query, collection, db) {
     const planCacheKey = getPlanCacheKeyFromShape({query, collection, db});
@@ -54,4 +50,3 @@ assert.eq(true, planCacheEntries[0].isPinned, planCacheEntries);
 assert.eq(true, planCacheEntries[0].isActive, planCacheEntries);
 // Works is always 0 for pinned plan cache entries.
 assert.eq(0, planCacheEntries[0].works, planCacheEntries);
-}());
