@@ -1,9 +1,6 @@
 // Validation test for SERVER-14753. Note that the issue under test is a memory leak, so this
 // test would only be expected to fail when run under address sanitizer.
 
-(function() {
-
-"use strict";
 var t = db.jstests_server14753;
 
 t.drop();
@@ -14,5 +11,4 @@ for (var i = 0; i < 20; i++) {
 }
 for (var i = 0; i < 20; i++) {
     t.find({b: 1}).sort({a: 1}).next();
-}
-}());
+}
\ No newline at end of file
