@@ -2,9 +2,6 @@
  * Tests 'db.collection.save()' mongo shell method.
  *
  */
-(function() {
-'use strict';
-
 const testDB = db.getSiblingDB(jsTestName());
 assert.commandWorked(testDB.dropDatabase());
 
@@ -31,5 +28,4 @@ assert.throws(() => coll.save("The answer to life, the universe and everything")
               "saving a string must throw an error");
 assert.throws(() => coll.save([{"fruit": "mango"}, {"fruit": "orange"}]),
               [],
-              "saving an array must throw an error");
-})();
\ No newline at end of file
+              "saving an array must throw an error");
\ No newline at end of file
