@@ -7,11 +7,11 @@
  * but another's is still not multikey, they may need to use different plans for certain queries).
  * @tags: [
  *   assumes_unsharded_collection,
+ *   # In 8.0, we changed behavior for equality to null.
+ *   requires_fcv_80,
  * ]
  */
-(function() {
-"use strict";
-load("jstests/libs/analyze_plan.js");  // For getPlanStages.
+import {getPlanStages, getWinningPlan} from "jstests/libs/analyze_plan.js";
 
 const coll = db.sparse_index_supports_ne_null;
 coll.drop();
@@ -59,7 +59,7 @@ assert.commandWorked(coll.insert({a: undefined}));
 assert.commandWorked(coll.createIndex(keyPattern, {sparse: true}));
 
 // Be sure the index is used.
-checkQuery({query: query, shouldUseIndex: true, nResultsExpected: 2, indexKeyPattern: keyPattern});
+checkQuery({query: query, shouldUseIndex: true, nResultsExpected: 3, indexKeyPattern: keyPattern});
 checkQuery({
     query: elemMatchQuery,
     shouldUseIndex: true,
@@ -68,13 +68,13 @@ checkQuery({
 });
 
 // Be sure the index is not used.
-checkQuery({query: eqNullQuery, shouldUseIndex: false, nResultsExpected: 2});
-checkQuery({query: gteQuery, shouldUseIndex: false, nResultsExpected: 2});
-checkQuery({query: lteQuery, shouldUseIndex: false, nResultsExpected: 2});
+checkQuery({query: eqNullQuery, shouldUseIndex: false, nResultsExpected: 1});
+checkQuery({query: gteQuery, shouldUseIndex: false, nResultsExpected: 1});
+checkQuery({query: lteQuery, shouldUseIndex: false, nResultsExpected: 1});
 
 // When the index becomes multikey, it cannot support {$ne: null} queries.
 assert.commandWorked(coll.insert({a: [1, 2, 3]}));
-checkQuery({query: query, shouldUseIndex: false, nResultsExpected: 3, indexKeyPattern: keyPattern});
+checkQuery({query: query, shouldUseIndex: false, nResultsExpected: 4, indexKeyPattern: keyPattern});
 // But it can support queries with {$ne: null} within an $elemMatch.
 checkQuery({
     query: elemMatchQuery,
@@ -115,7 +115,7 @@ assert.commandWorked(coll.insert({a: undefined, b: 1}));
 assert.commandWorked(coll.createIndex(keyPattern, {sparse: true}));
 
 // Be sure the index is used.
-checkQuery({query: query, shouldUseIndex: true, nResultsExpected: 2, indexKeyPattern: keyPattern});
+checkQuery({query: query, shouldUseIndex: true, nResultsExpected: 3, indexKeyPattern: keyPattern});
 checkQuery({
     query: elemMatchQuery,
     shouldUseIndex: true,
@@ -124,13 +124,13 @@ checkQuery({
 });
 
 // Be sure the index is not used.
-checkQuery({query: eqNullQuery, shouldUseIndex: false, nResultsExpected: 2});
-checkQuery({query: gteQuery, shouldUseIndex: false, nResultsExpected: 2});
-checkQuery({query: lteQuery, shouldUseIndex: false, nResultsExpected: 2});
+checkQuery({query: eqNullQuery, shouldUseIndex: false, nResultsExpected: 1});
+checkQuery({query: gteQuery, shouldUseIndex: false, nResultsExpected: 1});
+checkQuery({query: lteQuery, shouldUseIndex: false, nResultsExpected: 1});
 
 // When the index becomes multikey on the second field, it should still be usable.
 assert.commandWorked(coll.insert({a: 1, b: [1, 2, 3]}));
-checkQuery({query: query, shouldUseIndex: true, nResultsExpected: 3, indexKeyPattern: keyPattern});
+checkQuery({query: query, shouldUseIndex: true, nResultsExpected: 4, indexKeyPattern: keyPattern});
 checkQuery({
     query: elemMatchQuery,
     shouldUseIndex: true,
@@ -140,7 +140,7 @@ checkQuery({
 
 // When the index becomes multikey on the first field, it should no longer be usable.
 assert.commandWorked(coll.insert({a: [1, 2, 3], b: 1}));
-checkQuery({query: query, shouldUseIndex: false, nResultsExpected: 4, indexKeyPattern: keyPattern});
+checkQuery({query: query, shouldUseIndex: false, nResultsExpected: 5, indexKeyPattern: keyPattern});
 // Queries which use a $elemMatch should still be able to use the index.
 checkQuery({
     query: elemMatchQuery,
@@ -148,6 +148,13 @@ checkQuery({
     nResultsExpected: 1,
     indexKeyPattern: keyPattern
 });
+assert.commandWorked(coll.insert({a: [undefined], b: 1}));
+checkQuery({
+    query: elemMatchQuery,
+    shouldUseIndex: true,
+    nResultsExpected: 2,
+    indexKeyPattern: keyPattern
+});
 })();
 
 // Nested field multikey with $elemMatch.
@@ -166,26 +173,15 @@ assert.commandWorked(coll.createIndex(keyPattern, {sparse: true}));
 const query = {
     "a.b.c.d": {$ne: null}
 };
-// $elemMatch object can only use the index when none of the paths below the $elemMatch is
-// not multikey.
-const elemMatchObjectQuery = {
-    "a.b": {$elemMatch: {"c.d": {$ne: null}}}
-};
 // $elemMatch value can always use the index.
 const elemMatchValueQuery = {
     "a.b.c.d": {$elemMatch: {$ne: null}}
 };
 
 // 'a.b' is multikey, so the index isn't used.
-checkQuery({query: query, shouldUseIndex: false, nResultsExpected: 2, indexKeyPattern: keyPattern});
+checkQuery({query: query, shouldUseIndex: false, nResultsExpected: 3, indexKeyPattern: keyPattern});
 // Since the multikey portion is above the $elemMatch, the $elemMatch query may use the
 // index.
-checkQuery({
-    query: elemMatchObjectQuery,
-    shouldUseIndex: true,
-    nResultsExpected: 2,
-    indexKeyPattern: keyPattern
-});
 checkQuery({
     query: elemMatchValueQuery,
     shouldUseIndex: true,
@@ -195,15 +191,9 @@ checkQuery({
 
 // Make the index become multikey on 'a' (another field above the $elemMatch).
 assert.commandWorked(coll.insert({a: [{b: [{c: {d: 1}}]}]}));
-checkQuery({query: query, shouldUseIndex: false, nResultsExpected: 3, indexKeyPattern: keyPattern});
+checkQuery({query: query, shouldUseIndex: false, nResultsExpected: 4, indexKeyPattern: keyPattern});
 // The only multikey paths are still above the $elemMatch, queries which use a $elemMatch
 // should still be able to use the index.
-checkQuery({
-    query: elemMatchObjectQuery,
-    shouldUseIndex: true,
-    nResultsExpected: 3,
-    indexKeyPattern: keyPattern
-});
 checkQuery({
     query: elemMatchValueQuery,
     shouldUseIndex: true,
@@ -213,13 +203,7 @@ checkQuery({
 
 // Make the index multikey for 'a.b.c'. Now the $elemMatch query may not use the index.
 assert.commandWorked(coll.insert({a: {b: [{c: [{d: 1}]}]}}));
-checkQuery({query: query, shouldUseIndex: false, nResultsExpected: 4, indexKeyPattern: keyPattern});
-checkQuery({
-    query: elemMatchObjectQuery,
-    shouldUseIndex: false,
-    nResultsExpected: 4,
-    indexKeyPattern: keyPattern
-});
+checkQuery({query: query, shouldUseIndex: false, nResultsExpected: 5, indexKeyPattern: keyPattern});
 checkQuery({
     query: elemMatchValueQuery,
     shouldUseIndex: true,
@@ -227,4 +211,3 @@ checkQuery({
     indexKeyPattern: keyPattern
 });
 })();
-})();
