@@ -1,13 +1,14 @@
-// The test runs commands that are not allowed with security token: stageDebug.
+// Test limit and skip
+//
 // @tags: [
-//   not_allowed_with_security_token,
+//   # The test runs commands that are not allowed with security token: stageDebug.
+//   not_allowed_with_signed_security_token,
 //   does_not_support_stepdowns,
 //   uses_testing_only_commands,
 //   no_selinux,
 // ]
 
-// Test limit and skip
-t = db.stages_limit_skip;
+let t = db.stages_limit_skip;
 t.drop();
 var collname = "stages_limit_skip";
 
@@ -20,7 +21,7 @@ t.createIndex({foo: 1});
 
 // foo <= 20, decreasing
 // Limit of 5 results.
-ixscan1 = {
+let ixscan1 = {
     ixscan: {
         args: {
             keyPattern: {foo: 1},
@@ -32,10 +33,8 @@ ixscan1 = {
         }
     }
 };
-limit1 = {
-    limit: {args: {node: ixscan1, num: 5}}
-};
-res = db.runCommand({stageDebug: {collection: collname, plan: limit1}});
+let limit1 = {limit: {args: {node: ixscan1, num: 5}}};
+let res = db.runCommand({stageDebug: {collection: collname, plan: limit1}});
 assert.eq(res.ok, 1);
 assert.eq(res.results.length, 5);
 assert.eq(res.results[0].foo, 20);
@@ -43,9 +42,7 @@ assert.eq(res.results[4].foo, 16);
 
 // foo <= 20, decreasing
 // Skip 5 results.
-skip1 = {
-    skip: {args: {node: ixscan1, num: 5}}
-};
+let skip1 = {skip: {args: {node: ixscan1, num: 5}}};
 res = db.runCommand({stageDebug: {collection: collname, plan: skip1}});
 assert.eq(res.ok, 1);
 assert.eq(res.results.length, 16);
