@@ -1,13 +1,14 @@
-// The test runs commands that are not allowed with security token: stageDebug.
+// Test basic OR functionality
+//
 // @tags: [
-//   not_allowed_with_security_token,
+//   # The test runs commands that are not allowed with security token: stageDebug.
+//   not_allowed_with_signed_security_token,
 //   does_not_support_stepdowns,
 //   uses_testing_only_commands,
 //   no_selinux,
 // ]
 
-// Test basic OR functionality
-t = db.stages_or;
+let t = db.stages_or;
 t.drop();
 var collname = "stages_or";
 
@@ -21,7 +22,7 @@ t.createIndex({bar: 1});
 t.createIndex({baz: 1});
 
 // baz >= 40
-ixscan1 = {
+let ixscan1 = {
     ixscan: {
         args: {
             keyPattern: {baz: 1},
@@ -34,7 +35,7 @@ ixscan1 = {
     }
 };
 // foo >= 40
-ixscan2 = {
+let ixscan2 = {
     ixscan: {
         args: {
             keyPattern: {foo: 1},
@@ -48,17 +49,13 @@ ixscan2 = {
 };
 
 // OR of baz and foo.  Baz == foo and we dedup.
-orix1ix2 = {
-    or: {args: {nodes: [ixscan1, ixscan2], dedup: true}}
-};
-res = db.runCommand({stageDebug: {collection: collname, plan: orix1ix2}});
+let orix1ix2 = {or: {args: {nodes: [ixscan1, ixscan2], dedup: true}}};
+let res = db.runCommand({stageDebug: {collection: collname, plan: orix1ix2}});
 assert.eq(res.ok, 1);
 assert.eq(res.results.length, 10);
 
 // No deduping, 2x the results.
-orix1ix2nodd = {
-    or: {args: {nodes: [ixscan1, ixscan2], dedup: false}}
-};
+let orix1ix2nodd = {or: {args: {nodes: [ixscan1, ixscan2], dedup: false}}};
 res = db.runCommand({stageDebug: {collection: collname, plan: orix1ix2nodd}});
 assert.eq(res.ok, 1);
 assert.eq(res.results.length, 20);
