@@ -1,16 +1,13 @@
 // sub1.js
 
-t = db.sub1;
+let t = db.sub1;
 t.drop();
 
-x = {
-    a: 1,
-    b: {c: {d: 2}}
-};
+let x = {a: 1, b: {c: {d: 2}}};
 
 t.save(x);
 
-y = t.findOne();
+let y = t.findOne();
 
 assert.eq(1, y.a);
 assert.eq(2, y.b.c.d);
