@@ -1,9 +1,6 @@
 /**
  * Test that verifies the javascript integration can handle large string exception messages.
  */
-(function() {
-'use strict';
-
 var len = 65 * 1024 * 1024;
 var str = new Array(len + 1).join('b');
 
@@ -11,5 +8,4 @@ var str = new Array(len + 1).join('b');
 // We do not want the mongo shell to terminate.
 assert.throws(function() {
     throw str;
-});
-})();
+});
\ No newline at end of file
