@@ -1,14 +1,44 @@
 // Helper functions for testing time-series collections.
 
-load("jstests/libs/feature_flag_util.js");
-load("jstests/aggregation/extras/utils.js");
+import {documentEq} from "jstests/aggregation/extras/utils.js";
+import {FeatureFlagUtil} from "jstests/libs/feature_flag_util.js";
+import {FixtureHelpers} from "jstests/libs/fixture_helpers.js";
 
-var TimeseriesTest = class {
-    /**
-     * Returns whether time-series bucket compression are supported.
-     */
-    static timeseriesBucketCompressionEnabled(conn) {
-        return FeatureFlagUtil.isEnabled(conn, "TimeseriesBucketCompression");
+export var TimeseriesTest = class {
+    static insertManyDocs(coll) {
+        jsTestLog("Inserting documents to a bucket.");
+        coll.insertMany(
+            [...Array(10).keys()].map(i => ({
+                                          "metadata": {"sensorId": 1, "type": "temperature"},
+                                          "timestamp": ISODate(),
+                                          "temp": i
+                                      })),
+            {ordered: false});
+    }
+    static getBucketMaxSpanSecondsFromGranularity(granularity) {
+        switch (granularity) {
+            case 'seconds':
+                return 60 * 60;
+            case 'minutes':
+                return 60 * 60 * 24;
+            case 'hours':
+                return 60 * 60 * 24 * 30;
+            default:
+                assert(false, 'Invalid granularity: ' + granularity);
+        }
+    }
+
+    static getBucketRoundingSecondsFromGranularity(granularity) {
+        switch (granularity) {
+            case 'seconds':
+                return 60;
+            case 'minutes':
+                return 60 * 60;
+            case 'hours':
+                return 60 * 60 * 24;
+            default:
+                assert(false, 'Invalid granularity: ' + granularity);
+        }
     }
 
     static bucketsMayHaveMixedSchemaData(coll) {
@@ -26,56 +56,16 @@ var TimeseriesTest = class {
     }
 
     /**
-     * Returns whether time-series updates and deletes are supported.
+     * Returns whether time-series always use compressed buckets are enabled.
+     * TODO SERVER-70605 remove this helper.
      */
-    static timeseriesUpdatesAndDeletesEnabled(conn) {
-        return assert
-            .commandWorked(
-                conn.adminCommand({getParameter: 1, featureFlagTimeseriesUpdatesAndDeletes: 1}))
-            .featureFlagTimeseriesUpdatesAndDeletes.value;
+    static timeseriesAlwaysUseCompressedBucketsEnabled(conn) {
+        return FeatureFlagUtil.isPresentAndEnabled(conn, "TimeseriesAlwaysUseCompressedBuckets");
     }
 
-    /**
-     * Returns whether sharded time-series updates and deletes are supported.
-     */
-    static shardedTimeseriesUpdatesAndDeletesEnabled(conn) {
-        return assert
-            .commandWorked(
-                conn.adminCommand({getParameter: 1, featureFlagShardedTimeSeriesUpdateDelete: 1}))
-            .featureFlagShardedTimeSeriesUpdateDelete.value;
-    }
-
-    static shardedtimeseriesCollectionsEnabled(conn) {
-        return assert
-            .commandWorked(conn.adminCommand({getParameter: 1, featureFlagShardedTimeSeries: 1}))
-            .featureFlagShardedTimeSeries.value;
-    }
-
-    static shardedTimeseriesUpdatesAndDeletesEnabled(conn) {
-        return assert
-            .commandWorked(
-                conn.adminCommand({getParameter: 1, featureFlagShardedTimeSeriesUpdateDelete: 1}))
-            .featureFlagShardedTimeSeriesUpdateDelete.value;
-    }
-
-    static timeseriesMetricIndexesEnabled(conn) {
-        return assert
-            .commandWorked(
-                conn.adminCommand({getParameter: 1, featureFlagTimeseriesMetricIndexes: 1}))
-            .featureFlagTimeseriesMetricIndexes.value;
-    }
-
-    static bucketUnpackWithSortEnabled(conn) {
-        const resp50 = conn.adminCommand({getParameter: 1, featureFlagBucketUnpackWithSort50: 1});
-        const resp = conn.adminCommand({getParameter: 1, featureFlagBucketUnpackWithSort: 1});
-        assert(Boolean(resp50.ok) ^ Boolean(resp.ok),
-               `Expected exactly one of these feature flags to exist: ${tojson(resp50)} vs ${
-                   tojson(resp)}`);
-        if (resp50.ok) {
-            return assert.commandWorked(resp50).featureFlagBucketUnpackWithSort50.value;
-        } else {
-            return assert.commandWorked(resp).featureFlagBucketUnpackWithSort.value;
-        }
+    // TODO SERVER-68058 remove this helper.
+    static arbitraryUpdatesEnabled(conn) {
+        return FeatureFlagUtil.isPresentAndEnabled(conn, "TimeseriesUpdatesSupport");
     }
 
     /**
@@ -90,6 +80,28 @@ var TimeseriesTest = class {
         }
     }
 
+    /**
+     * Decompresses a compressed bucket document. Replaces the compressed data in-place.
+     */
+    static decompressBucket(compressedBucket) {
+        assert.hasFields(
+            compressedBucket,
+            ["control"],
+            "TimeseriesTest.decompressBucket() should only be called on a bucket document");
+        if (compressedBucket.control.version == 1) {
+            // Bucket is already decompressed.
+            return;
+        }
+
+        for (const column in compressedBucket.data) {
+            compressedBucket.data[column] = decompressBSONColumn(compressedBucket.data[column]);
+        }
+
+        // The control object should reflect that the data is uncompressed.
+        compressedBucket.control.version = TimeseriesTest.BucketVersion.kUncompressed;
+        delete compressedBucket.control.count;
+    }
+
     /**
      * Returns a random element from an array.
      */
@@ -273,4 +285,29 @@ var TimeseriesTest = class {
     static getBucketsCollName(collName) {
         return `system.buckets.${collName}`;
     }
+
+    static assertInsertWorked(res) {
+        // TODO (SERVER-85548): Remove helper and revert to assert.commandWorked, no expected error
+        // codes
+        return assert.commandWorkedOrFailedWithCode(res, [8555700, 8555701]);
+    }
+
+    static isBucketCompressed(version) {
+        return (version == TimeseriesTest.BucketVersion.kCompressedSorted ||
+                version == TimeseriesTest.BucketVersion.kCompressedUnsorted);
+    }
+
+    // Timeseries stats are not returned if there are no timeseries collection. This is a helper to
+    // handle that case when tests drop their timeseries collections.
+    static getStat(stats, name) {
+        if (stats.hasOwnProperty(name))
+            return stats[name];
+        return 0;
+    }
+};
+
+TimeseriesTest.BucketVersion = {
+    kUncompressed: 1,
+    kCompressedSorted: 2,
+    kCompressedUnsorted: 3
 };
