@@ -1,9 +1,9 @@
-load("jstests/core/timeseries/libs/timeseries.js");
-
 /**
  * Helper class for aggregate tests with time-series collection.
  */
-var TimeseriesAggTests = class {
+import {TimeseriesTest} from "jstests/core/timeseries/libs/timeseries.js";
+
+export var TimeseriesAggTests = class {
     /**
      * Gets a test db object based on the test suite name.
      */
@@ -20,14 +20,15 @@ var TimeseriesAggTests = class {
      * @returns An array of a time-series collection and a non time-series collection,
      *     respectively in this order.
      */
-    static prepareInputCollections(numHosts, numIterations, includeIdleMeasurements = true) {
+    static prepareInputCollections(numHosts,
+                                   numIterations,
+                                   includeIdleMeasurements = true,
+                                   testDB = TimeseriesAggTests.getTestDb()) {
         const timeseriesCollOption = {timeseries: {timeField: "time", metaField: "tags"}};
 
         Random.setRandomSeed();
         const hosts = TimeseriesTest.generateHosts(numHosts);
 
-        const testDB = TimeseriesAggTests.getTestDb();
-
         // Creates a time-series input collection.
         const inColl = testDB.getCollection("in");
         inColl.drop();
@@ -37,7 +38,7 @@ var TimeseriesAggTests = class {
         const observerInColl = testDB.getCollection("observer_in");
         observerInColl.drop();
         assert.commandWorked(testDB.createCollection(observerInColl.getName()));
-        const currTime = new Date();
+        let currTime = new Date();
 
         // Inserts exactly the same random measurement to both inColl and observerInColl.
         for (let i = 0; i < numIterations; i++) {
@@ -45,7 +46,7 @@ var TimeseriesAggTests = class {
                 const userUsage = TimeseriesTest.getRandomUsage();
                 let newMeasurement = {
                     tags: host.tags,
-                    time: new Date(currTime + i),
+                    time: new Date(currTime++),
                     usage_guest: TimeseriesTest.getRandomUsage(),
                     usage_guest_nice: TimeseriesTest.getRandomUsage(),
                     usage_idle: TimeseriesTest.getRandomUsage(),
@@ -63,7 +64,7 @@ var TimeseriesAggTests = class {
                 if (includeIdleMeasurements && (i % 2)) {
                     let idleMeasurement = {
                         tags: host.tags,
-                        time: new Date(currTime + i),
+                        time: new Date(currTime++),
                         idle_user: 100 - userUsage
                     };
                     assert.commandWorked(inColl.insert(idleMeasurement));
@@ -78,11 +79,11 @@ var TimeseriesAggTests = class {
     /**
      * Gets an output collection object with the name 'outCollname'.
      */
-    static getOutputCollection(outCollName) {
-        const testDB = TimeseriesAggTests.getTestDb();
-
+    static getOutputCollection(outCollName, shouldDrop, testDB = TimeseriesAggTests.getTestDb()) {
         let outColl = testDB.getCollection(outCollName);
-        outColl.drop();
+        if (shouldDrop) {
+            outColl.drop();
+        }
 
         return outColl;
     }
@@ -96,21 +97,35 @@ var TimeseriesAggTests = class {
      * Executes 'prepareAction' before executing 'pipeline'. 'prepareAction' takes a collection
      * parameter and returns nothing.
      *
+     * If 'shouldDrop' is set to false, the output collection will not be dropped before executing
+     * 'pipeline'.
+     *
+     * If 'testDB' is set, that database will be used in the aggregation pipeline.
+     *
      * Returns sorted data by "time" field. The sorted result data will help simplify comparison
      * logic.
      */
-    static getOutputAggregateResults(inColl, pipeline, prepareAction = null) {
+    static getOutputAggregateResults(inColl,
+                                     pipeline,
+                                     prepareAction = null,
+                                     shouldDrop = true,
+                                     testDB = TimeseriesAggTests.getTestDb()) {
         // Figures out the output collection name from the last pipeline stage.
         var outCollName = "out";
         if (pipeline[pipeline.length - 1]["$out"] != undefined) {
-            // If the last stage is "$out", gets the output collection name from it.
-            outCollName = pipeline[pipeline.length - 1]["$out"];
+            // If the last stage is "$out", gets the output collection name from the string or
+            // object input.
+            if (typeof pipeline[pipeline.length - 1]["$out"] == 'string') {
+                outCollName = pipeline[pipeline.length - 1]["$out"];
+            } else {
+                outCollName = pipeline[pipeline.length - 1]["$out"]["coll"];
+            }
         } else if (pipeline[pipeline.length - 1]["$merge"] != undefined) {
             // If the last stage is "$merge", gets the output collection name from it.
             outCollName = pipeline[pipeline.length - 1]["$merge"].into;
         }
 
-        let outColl = TimeseriesAggTests.getOutputCollection(outCollName);
+        let outColl = TimeseriesAggTests.getOutputCollection(outCollName, shouldDrop, testDB);
         if (prepareAction != null) {
             prepareAction(outColl);
         }
@@ -122,4 +137,22 @@ var TimeseriesAggTests = class {
             .sort({"time": 1})
             .toArray();
     }
+
+    static verifyResults(actualResults, expectedResults) {
+        // Verifies that the number of measurements is same as expected.
+        assert.eq(actualResults.length, expectedResults.length, actualResults);
+
+        // Verifies that every measurement is same as expected.
+        for (var i = 0; i < expectedResults.length; ++i) {
+            assert.eq(actualResults[i], expectedResults[i], actualResults);
+        }
+    }
+
+    static generateOutPipeline(collName, dbName, options, aggStage = null) {
+        let outStage = {$out: {db: dbName, coll: collName, timeseries: options}};
+        if (aggStage) {
+            return [aggStage, outStage];
+        }
+        return [outStage];
+    }
 };
