@@ -11,11 +11,10 @@
  * ]
  */
 
-load("jstests/libs/analyze_plan.js");
-load("jstests/libs/fixture_helpers.js");  // For isSharded.
-load('jstests/noPassthrough/libs/index_build.js');
+import {getWinningPlan, isCollscan, isIxscan} from "jstests/libs/analyze_plan.js";
+import {FixtureHelpers} from "jstests/libs/fixture_helpers.js";
+import {IndexBuildTest} from "jstests/noPassthrough/libs/index_build.js";
 
-(function() {
 const timeFieldName = "timestamp";
 
 const coll = db.partialFilterExpression_with_internalBucketGeoWithin;
@@ -179,4 +178,3 @@ findAndExplain = assert.commandWorked(bucketsColl
                                           })
                                           .explain());
 assert(isCollscan(db, getWinningPlan(findAndExplain.queryPlanner)));
-})();
