@@ -3,15 +3,15 @@
  * collection.
  *
  * @tags: [
+ *   # This test depends on certain writes ending up in the same bucket. Stepdowns and tenant
+ *   # migrations may result in writes splitting between two primaries, and thus different buckets.
  *   does_not_support_stepdowns,
- *   does_not_support_transactions,
- *   requires_getmore,
+ *   tenant_migration_incompatible,
+ *   # We need a timeseries collection.
+ *   requires_timeseries,
  * ]
  */
-(function() {
-'use strict';
-
-load("jstests/core/timeseries/libs/timeseries.js");  // For 'TimeseriesTest'.
+import {TimeseriesTest} from "jstests/core/timeseries/libs/timeseries.js";
 
 TimeseriesTest.run((insert) => {
     const coll = db.timeseries_bucket_manual_removal;
@@ -46,25 +46,24 @@ TimeseriesTest.run((insert) => {
     ];
 
     assert.commandWorked(insert(coll, docs1));
-    assert.docEq(coll.find().toArray(), docs1);
+    assert.docEq(docs1, coll.find().toArray());
     let buckets = bucketsColl.find().toArray();
     assert.eq(buckets.length, 1, 'Expected one bucket but found ' + tojson(buckets));
     const bucketId = buckets[0]._id;
 
     assert.commandWorked(bucketsColl.remove({_id: bucketId}));
-    assert.docEq(coll.find().toArray(), []);
+    assert.docEq([], coll.find().toArray());
     buckets = bucketsColl.find().toArray();
     assert.eq(buckets.length, 0, 'Expected no buckets but found ' + tojson(buckets));
 
     assert.commandWorked(bucketsColl.remove({_id: bucketId}));
-    assert.docEq(coll.find().toArray(), []);
+    assert.docEq([], coll.find().toArray());
     buckets = bucketsColl.find().toArray();
     assert.eq(buckets.length, 0, 'Expected no buckets but found ' + tojson(buckets));
 
     assert.commandWorked(coll.insert(docs2, {ordered: false}));
-    assert.docEq(coll.find().toArray(), docs2);
+    assert.docEq(docs2, coll.find().toArray());
     buckets = bucketsColl.find().toArray();
     assert.eq(buckets.length, 1, 'Expected one bucket but found ' + tojson(buckets));
     assert.neq(buckets[0]._id, bucketId);
 });
-})();
