@@ -4,6 +4,8 @@
  *   requires_timeseries,
  *   does_not_support_stepdowns,
  *   directly_against_shardsvrs_incompatible,
+ *   # During fcv upgrade/downgrade the engine might not be what we expect.
+ *   cannot_run_during_upgrade_downgrade,
  *   # "Explain of a resolved view must be executed by mongos"
  *   directly_against_shardsvrs_incompatible,
  *   # Some suites use mixed-binary cluster setup where some nodes might have the flag enabled while
@@ -12,7 +14,8 @@
  *   assumes_standalone_mongod
  * ]
  */
-load("jstests/core/timeseries/libs/timeseries.js");
+
+import {TimeseriesTest} from "jstests/core/timeseries/libs/timeseries.js";
 
 TimeseriesTest.run((insert) => {
     const datePrefix = 1680912440;
@@ -376,9 +379,9 @@ TimeseriesTest.run((insert) => {
                         }])
                         .toArray();
         assert.eq(res.length, 3, res);
-        assert.docEq(res[0], {_id: 0, "t": NumberLong(50)}, res);
-        assert.docEq(res[1], {_id: 1, "t": NumberLong(-50)}, res);
-        assert.docEq(res[2], {_id: 2, "t": NumberLong(-150)}, res);
+        assert.docEq(res[0], {_id: 0, "t": 50}, res);
+        assert.docEq(res[1], {_id: 1, "t": -50}, res);
+        assert.docEq(res[2], {_id: 2, "t": -150}, res);
     }
 
     {
@@ -395,9 +398,9 @@ TimeseriesTest.run((insert) => {
                         }])
                         .toArray();
         assert.eq(res.length, 3, res);
-        assert.docEq(res[0], {_id: 0, "t": NumberLong(110)}, res);
-        assert.docEq(res[1], {_id: 1, "t": NumberLong(210)}, res);
-        assert.docEq(res[2], {_id: 2, "t": NumberLong(310)}, res);
+        assert.docEq(res[0], {_id: 0, "t": 110}, res);
+        assert.docEq(res[1], {_id: 1, "t": 210}, res);
+        assert.docEq(res[2], {_id: 2, "t": 310}, res);
     }
 
     {
