@@ -9,11 +9,8 @@
  * ]
  */
 
-(function() {
-"use strict";
-
-load("jstests/core/timeseries/libs/geo.js");
-load("jstests/core/timeseries/libs/timeseries.js");
+import {randomLongLat} from "jstests/core/timeseries/libs/geo.js";
+import {TimeseriesTest} from "jstests/core/timeseries/libs/timeseries.js";
 
 Random.setRandomSeed(7813223789272959000);
 
@@ -174,4 +171,3 @@ for (const minOrMax of ['maxDistance', 'minDistance']) {
     const tsResult = tsColl.aggregate(pipeline).toArray();
     assert.sameMembers(result, tsResult);
 }
-})();
