@@ -10,90 +10,108 @@
  *   requires_fcv_61,
  * ]
  */
-(function() {
-"use strict";
-
-load("jstests/libs/fixture_helpers.js");
-load("jstests/core/timeseries/libs/timeseries.js");
 
 const coll = db.timeseries_groupby_reorder;
 coll.drop();
 
-// We will only check correctness of the results here as checking the plan in JSTests is brittle and
-// is better done in document_source_internal_unpack_bucket_test/group_reorder_test.cpp. For the
-// cases when the re-write isn't applicable, the used datasets should yield wrong result if the
-// re-write is applied.
-function runGroupRewriteTest(docs, pipeline, expectedResults) {
-    coll.drop();
-    db.createCollection(coll.getName(), {timeseries: {metaField: "meta", timeField: "time"}});
-    coll.insertMany(docs);
-    assert.sameMembers(expectedResults, coll.aggregate(pipeline).toArray(), () => {
-        return `Pipeline: ${tojson(pipeline)}. Explain: ${
-            tojson(coll.explain().aggregate(pipeline))}`;
-    });
-}
+import {runGroupRewriteTest} from 'jstests/core/timeseries/timeseries_groupby_reorder_helpers.js';
 
 // Test with measurement group key -- a rewrite in this situation would be wrong.
 (function testNonMetaGroupKey() {
     const t = new Date();
     const docs = [
-        {time: t, meta: 1, key: 2, val: 1},  // global min
-        {time: t, meta: 1, key: 1, val: 3},  // min for key = 1
-        {time: t, meta: 1, key: 1, val: 5},  // max for key = 1
-        {time: t, meta: 1, key: 2, val: 7},  // global max
+        {time: t, myMeta: 1, key: 2, val: 1},  // global min
+        {time: t, myMeta: 1, key: 1, val: 3},  // min for key = 1
+        {time: t, myMeta: 1, key: 1, val: 5},  // max for key = 1
+        {time: t, myMeta: 1, key: 2, val: 7},  // global max
     ];
-    runGroupRewriteTest(docs,
+    runGroupRewriteTest(coll,
+                        docs,
                         [{$group: {_id: "$key", min: {$min: "$val"}}}, {$match: {_id: 1}}],
                         [{"_id": 1, "min": 3}]);
-    runGroupRewriteTest(docs,
+    runGroupRewriteTest(coll,
+                        docs,
                         [{$group: {_id: "$key", max: {$max: "$val"}}}, {$match: {_id: 1}}],
                         [{"_id": 1, "max": 5}]);
 })();
 
-// While a group with const group key can be re-written in terms of a group on the buckets, we don't
-// currently do it. However, when/if we start doing it, it should work.
+// Test with a constant group key. The $group rewrite applies.
 (function testConstGroupKey_NoFilter() {
     const t = new Date();
     const docs = [
-        {time: t, meta: 1, val: 0},
-        {time: t, meta: 1, val: 1},
-        {time: t, meta: 1, val: 5},
+        {time: t, myMeta: 1, val: 0},
+        {time: t, myMeta: 1, val: 1},
+        {time: t, myMeta: 1, val: 5},
     ];
     runGroupRewriteTest(
-        docs, [{$group: {_id: null, min: {$min: "$val"}}}], [{"_id": null, "min": 0}]);
+        coll, docs, [{$group: {_id: null, min: {$min: "$val"}}}], [{"_id": null, "min": 0}]);
     runGroupRewriteTest(
-        docs, [{$group: {_id: null, max: {$max: "$val"}}}], [{"_id": null, "max": 5}]);
+        coll, docs, [{$group: {_id: null, max: {$max: "$val"}}}], [{"_id": null, "max": 5}]);
 })();
 
-// With a filter on meta the group re-write would still apply if the group key is const.
+// Test with a group key that is optimized to a constant. The $group rewrite applies.
+(function testConstExprGroupKey_WithMinMaxAccumulator() {
+    const t = new Date();
+    const docs = [
+        {time: t, myMeta: 1, val: 3},
+        {time: t, myMeta: 3, val: 4},
+        {time: t, myMeta: 1, val: 5},
+    ];
+    runGroupRewriteTest(
+        coll,
+        docs,
+        [{$group: {_id: {$mod: [10, 5]}, min: {$min: "$val"}, max: {$max: "$val"}}}],
+        [{_id: 0, min: 3, max: 5}]);
+})();
+
+// Test with a null group key and the collection has no metaField. The $group rewrite applies.
+(function testNullGroupKey_WithMinMaxAccumulator_NoMetaField() {
+    const t = new Date();
+    const docs = [
+        {time: t, myMeta: 1, val: 3},
+        {time: t, myMeta: 3, val: 4},
+        {time: t, myMeta: 1, val: 5},
+    ];
+    runGroupRewriteTest(coll,
+                        docs,
+                        [{$group: {_id: null, min: {$min: "$val"}, max: {$max: "$val"}}}],
+                        [{_id: null, min: 3, max: 5}],
+                        true /* excludeMeta */);
+})();
+
+// With a filter on meta the group re-write does apply if the group key is const.
 (function testConstGroupKey_WithFilterOnMeta() {
     const t = new Date();
     const docs = [
-        {time: t, meta: 1, val: 0},
-        {time: t, meta: 2, val: 1},
-        {time: t, meta: 2, val: 3},
-        {time: t, meta: 1, val: 5},
+        {time: t, myMeta: 1, val: 0},
+        {time: t, myMeta: 2, val: 1},
+        {time: t, myMeta: 2, val: 3},
+        {time: t, myMeta: 1, val: 5},
     ];
-    runGroupRewriteTest(docs,
-                        [{$match: {meta: 2}}, {$group: {_id: null, min: {$min: "$val"}}}],
+    runGroupRewriteTest(coll,
+                        docs,
+                        [{$match: {myMeta: 2}}, {$group: {_id: null, min: {$min: "$val"}}}],
                         [{"_id": null, "min": 1}]);
-    runGroupRewriteTest(docs,
-                        [{$match: {meta: 2}}, {$group: {_id: null, max: {$max: "$val"}}}],
+    runGroupRewriteTest(coll,
+                        docs,
+                        [{$match: {myMeta: 2}}, {$group: {_id: null, max: {$max: "$val"}}}],
                         [{"_id": null, "max": 3}]);
 })();
 
-// In presense of a non-meta filter the group re-write doesn't apply even if the group key is const.
+// In presence of a non-meta filter the group re-write doesn't apply even if the group key is const.
 (function testConstGroupKey_WithFilterOnMeasurement() {
     const t = new Date();
     const docs = [
-        {time: t, meta: 1, val: 0, include: false},
-        {time: t, meta: 1, val: 1, include: true},
-        {time: t, meta: 1, val: 5, include: false},
+        {time: t, myMeta: 1, val: 0, include: false},
+        {time: t, myMeta: 1, val: 1, include: true},
+        {time: t, myMeta: 1, val: 5, include: false},
     ];
-    runGroupRewriteTest(docs,
+    runGroupRewriteTest(coll,
+                        docs,
                         [{$match: {include: true}}, {$group: {_id: null, min: {$min: "$val"}}}],
                         [{"_id": null, "min": 1}]);
-    runGroupRewriteTest(docs,
+    runGroupRewriteTest(coll,
+                        docs,
                         [{$match: {include: true}}, {$group: {_id: null, max: {$max: "$val"}}}],
                         [{"_id": null, "max": 1}]);
 })();
@@ -102,16 +120,18 @@ function runGroupRewriteTest(docs, pipeline, expectedResults) {
 (function testMetaGroupKey_NoFilter() {
     const t = new Date();
     const docs = [
-        {time: t, meta: 1, val: 5},
-        {time: t, meta: 2, val: 4},
-        {time: t, meta: 2, val: 3},
-        {time: t, meta: 1, val: 1},
+        {time: t, myMeta: 1, val: 5},
+        {time: t, myMeta: 2, val: 4},
+        {time: t, myMeta: 2, val: 3},
+        {time: t, myMeta: 1, val: 1},
     ];
-    runGroupRewriteTest(docs,
-                        [{$group: {_id: "$meta", min: {$min: "$val"}}}, {$match: {_id: 2}}],
+    runGroupRewriteTest(coll,
+                        docs,
+                        [{$group: {_id: "$myMeta", min: {$min: "$val"}}}, {$match: {_id: 2}}],
                         [{"_id": 2, "min": 3}]);
-    runGroupRewriteTest(docs,
-                        [{$group: {_id: "$meta", max: {$max: "$val"}}}, {$match: {_id: 2}}],
+    runGroupRewriteTest(coll,
+                        docs,
+                        [{$group: {_id: "$myMeta", max: {$max: "$val"}}}, {$match: {_id: 2}}],
                         [{"_id": 2, "max": 4}]);
 })();
 
@@ -119,16 +139,18 @@ function runGroupRewriteTest(docs, pipeline, expectedResults) {
 (function testMetaGroupKey_WithFilterOnMeta() {
     const t = new Date();
     const docs = [
-        {time: t, meta: 1, val: 5},
-        {time: t, meta: 2, val: 4},
-        {time: t, meta: 2, val: 3},
-        {time: t, meta: 1, val: 1},
+        {time: t, myMeta: 1, val: 5},
+        {time: t, myMeta: 2, val: 4},
+        {time: t, myMeta: 2, val: 3},
+        {time: t, myMeta: 1, val: 1},
     ];
-    runGroupRewriteTest(docs,
-                        [{$match: {meta: 2}}, {$group: {_id: "$meta", min: {$min: "$val"}}}],
+    runGroupRewriteTest(coll,
+                        docs,
+                        [{$match: {myMeta: 2}}, {$group: {_id: "$myMeta", min: {$min: "$val"}}}],
                         [{"_id": 2, "min": 3}]);
-    runGroupRewriteTest(docs,
-                        [{$match: {meta: 2}}, {$group: {_id: "$meta", max: {$max: "$val"}}}],
+    runGroupRewriteTest(coll,
+                        docs,
+                        [{$match: {myMeta: 2}}, {$group: {_id: "$myMeta", max: {$max: "$val"}}}],
                         [{"_id": 2, "max": 4}]);
 })();
 
@@ -136,16 +158,20 @@ function runGroupRewriteTest(docs, pipeline, expectedResults) {
 (function testMetaGroupKey_WithFilterOnMeasurement() {
     const t = new Date();
     const docs = [
-        {time: t, meta: 1, val: 3, include: false},
-        {time: t, meta: 1, val: 4, include: true},
-        {time: t, meta: 1, val: 5, include: false},
+        {time: t, myMeta: 1, val: 3, include: false},
+        {time: t, myMeta: 1, val: 4, include: true},
+        {time: t, myMeta: 1, val: 5, include: false},
     ];
-    runGroupRewriteTest(docs,
-                        [{$match: {include: true}}, {$group: {_id: "$meta", min: {$min: "$val"}}}],
-                        [{"_id": 1, "min": 4}]);
-    runGroupRewriteTest(docs,
-                        [{$match: {include: true}}, {$group: {_id: "$meta", max: {$max: "$val"}}}],
-                        [{"_id": 1, "max": 4}]);
+    runGroupRewriteTest(
+        coll,
+        docs,
+        [{$match: {include: true}}, {$group: {_id: "$myMeta", min: {$min: "$val"}}}],
+        [{"_id": 1, "min": 4}]);
+    runGroupRewriteTest(
+        coll,
+        docs,
+        [{$match: {include: true}}, {$group: {_id: "$myMeta", max: {$max: "$val"}}}],
+        [{"_id": 1, "max": 4}]);
 })();
 
 // Test SERVER-73822 fix: complex $min and $max (i.e. not just straight field refs) work correctly.
@@ -153,43 +179,33 @@ function runGroupRewriteTest(docs, pipeline, expectedResults) {
     const t = new Date();
     // min(a+b) != min(a) + min(b) and max(a+b) != max(a) + max(b)
     const docs = [
-        {time: t, meta: 1, a: 1, b: 20},  // max(a + b)
-        {time: t, meta: 1, a: 2, b: 10},
-        {time: t, meta: 1, a: 3, b: 1},  // min(a + b)
+        {time: t, myMeta: 1, a: 1, b: 20},  // max(a + b)
+        {time: t, myMeta: 1, a: 2, b: 10},
+        {time: t, myMeta: 1, a: 3, b: 1},  // min(a + b)
     ];
-    runGroupRewriteTest(docs,
-                        [{$group: {_id: "$meta", min: {$min: {$add: ["$a", "$b"]}}}}],
+    runGroupRewriteTest(coll,
+                        docs,
+                        [{$group: {_id: "$myMeta", min: {$min: {$add: ["$a", "$b"]}}}}],
                         [{"_id": 1, "min": 4}]);
-    runGroupRewriteTest(docs,
-                        [{$group: {_id: "$meta", max: {$max: {$add: ["$a", "$b"]}}}}],
+    runGroupRewriteTest(coll,
+                        docs,
+                        [{$group: {_id: "$myMeta", max: {$max: {$add: ["$a", "$b"]}}}}],
                         [{"_id": 1, "max": 21}]);
 })();
 
-// Test with meta group key and a non-min/max accumulator that doesn't use any fields. The buckets
-// still have to be unpacked because we don't know the number of events in uncompressed buckets.
-(function testMetaGroupKey_WithAccumulatorNotUsingAnyFields() {
-    const t = new Date();
-    const docs = [
-        {time: t, meta: 1, val: 3},
-        {time: t, meta: 3, val: 4},
-        {time: t, meta: 1, val: 5},
-    ];
-    runGroupRewriteTest(
-        docs, [{$group: {_id: "$meta", x: {$sum: 1}}}, {$match: {_id: 1}}], [{"_id": 1, "x": 2}]);
-})();
-
 // Test with meta group key and a non-min/max accumulator that uses only the meta field. This query
 // is _not_ eligible for the re-write w/o bucket unpacking because while it doesn't depend on any
 // fields of the individual events it still depends on the number of events in a bucket.
 (function testMetaGroupKey_WithNonMinMaxAccumulatorOnMeta() {
     const t = new Date();
     const docs = [
-        {time: t, meta: 1, val: 3},
-        {time: t, meta: 3, val: 4},
-        {time: t, meta: 1, val: 5},
+        {time: t, myMeta: 1, val: 3},
+        {time: t, myMeta: 3, val: 4},
+        {time: t, myMeta: 1, val: 5},
     ];
-    runGroupRewriteTest(docs,
-                        [{$group: {_id: "$meta", x: {$sum: "$meta"}}}, {$match: {_id: 1}}],
+    runGroupRewriteTest(coll,
+                        docs,
+                        [{$group: {_id: "$myMeta", x: {$sum: "$myMeta"}}}, {$match: {_id: 1}}],
                         [{"_id": 1, "x": 2}]);
 })();
 
@@ -198,37 +214,199 @@ function runGroupRewriteTest(docs, pipeline, expectedResults) {
 (function testMetaGroupKey_WithAccumulatorOnMeta_WithFilterOnMeasurement() {
     const t = new Date();
     const docs = [
-        {time: t, meta: 1, include: false},
+        {time: t, myMeta: 1, include: false},
     ];
     runGroupRewriteTest(
-        docs, [{$match: {include: true}}, {$group: {_id: "$meta", x: {$min: "$meta"}}}], []);
+        coll,
+        docs,
+        [{$match: {include: true}}, {$group: {_id: "$myMeta", x: {$min: "$myMeta"}}}],
+        []);
 })();
 
-// Test min/max on the time field (cannot rewrite $min because the control.time.min is rounded
+// Test min on the time field (cannot rewrite $min because the control.time.min is rounded
 // down).
-(function testMetaGroupKey_WithMinMaxOnTime() {
+(function testMetaGroupKey_WithMinOnTime() {
     const docs = [
-        {time: ISODate("2023-07-20T23:16:47.683Z"), meta: 1},
+        {time: ISODate("2023-07-20T23:16:47.683Z"), myMeta: 1},
     ];
-    runGroupRewriteTest(docs,
-                        [{$group: {_id: "$meta", min: {$min: "$time"}}}],
+    runGroupRewriteTest(coll,
+                        docs,
+                        [{$group: {_id: "$myMeta", min: {$min: "$time"}}}],
                         [{_id: 1, min: ISODate("2023-07-20T23:16:47.683Z")}]);
 })();
 
-// Test a group key which is an object with single field path. Ensure that the output perseveres the
-// user requested object structure for _id field.
-(function testMetaGroupKey_ObjectWithSingleFieldPathElement() {
+// Test max on the time field (can rewrite $max because the control.time.max is not rounded
+// down).
+(function testMetaGroupKey_WithMaxOnTime() {
+    const docs = [
+        {time: ISODate("2023-07-20T23:16:47.683Z"), myMeta: 1},
+    ];
+    runGroupRewriteTest(coll,
+                        docs,
+                        [{$group: {_id: "$myMeta", max: {$max: "$time"}}}],
+                        [{_id: 1, max: ISODate("2023-07-20T23:16:47.683Z")}]);
+})();
+
+// Test a group key that is a list of fields that are just referencing the metaField. The $group
+// rewrite applies.
+(function testListMetaFields_WithMinMaxAccumulator() {
     const t = new Date();
     const docs = [
-        {time: t, meta: {a: 1}, val: 5},
-        {time: t, meta: {a: 2}, val: 4},
-        {time: t, meta: {a: 2}, val: 3},
-        {time: t, meta: {a: 1}, val: 1},
+        {time: t, myMeta: {a: 2, b: 10}, val: 3},
+        {time: t, myMeta: {a: 3, b: 10}, val: 4},
+        {time: t, myMeta: {a: 2, b: 10}, val: 5},
     ];
     runGroupRewriteTest(
+        coll,
         docs,
-        [{$group: {_id: {d: "$meta.a"}, min: {$min: "$val"}}}, {$match: {_id: {d: 2}}}],
-        [{"_id": {d: 2}, "min": 3}]);
+        [{
+            $group:
+                {_id: {a: "$myMeta.a", b: "$myMeta.b"}, min: {$min: "$val"}, max: {$max: "$val"}}
+        }],
+        [{_id: {a: 3, b: 10}, min: 4, max: 4}, {_id: {a: 2, b: 10}, min: 3, max: 5}]);
+})();
+
+// Test a group key that is a list of fields with some fields referencing the metaField and some
+// fields are not. The $group rewrite does not apply.
+(function testListMetaAndOtherFields_WithMinMaxAccumulator() {
+    const t = new Date();
+    const docs = [
+        {time: t, myMeta: {a: 2, b: 10}, val: 3, string: "apple"},
+        {time: t, myMeta: {a: 3, b: 10}, val: 4, string: "pear"},
+        {time: t, myMeta: {a: 3, b: 10}, val: 6, string: "apple"},
+        {time: t, myMeta: {a: 2, b: 10}, val: 5, string: "apple"}
+    ];
+    runGroupRewriteTest(
+        coll,
+        docs,
+        [{$group: {_id: {a: "$myMeta.a", b: "$string"}, min: {$min: "$val"}, max: {$max: "$val"}}}],
+        [
+            {_id: {a: 2, b: "apple"}, min: 3, max: 5},
+            {_id: {a: 3, b: "apple"}, min: 6, max: 6},
+            {_id: {a: 3, b: "pear"}, min: 4, max: 4}
+        ]);
+})();
+
+// Test a group key that is a list of fields and the collection does not have a metaField. The
+// rewrite does not apply, since no fields are referencing the metaField.
+(function testListOfOtherFields_NoMetaField() {
+    const t = new Date();
+    const docs = [
+        {time: t, myMeta: {a: 2, b: 10}, val: 3, string: "apple"},
+        {time: t, myMeta: {a: 3, b: 10}, val: 4, string: "pear"},
+        {time: t, myMeta: {a: 3, b: 10}, val: 6, string: "apple"},
+        {time: t, myMeta: {a: 2, b: 10}, val: 5, string: "apple"}
+    ];
+    runGroupRewriteTest(
+        coll,
+        docs,
+        [{$group: {_id: {a: "$myMeta.a", b: "$string"}, min: {$min: "$val"}, max: {$max: "$val"}}}],
+        [
+            {_id: {a: 2, b: "apple"}, min: 3, max: 5},
+            {_id: {a: 3, b: "apple"}, min: 6, max: 6},
+            {_id: {a: 3, b: "pear"}, min: 4, max: 4}
+        ],
+        true /* excludeMeta */);
+})();
+
+//
+// The following tests validate the behavior of the $group rewrite with the $count accumulator.
+//
+
+// Test with a meta group key, and the $count accumulator. The rewrite should apply.
+(function testMetaGroupKey_WithCountMinMaxAccumulator() {
+    const t = new Date();
+    const docs = [
+        {time: t, myMeta: 1, val: 3},
+        {time: t, myMeta: 3, val: 4},
+        {time: t, myMeta: 1, val: 5},
+    ];
+    runGroupRewriteTest(
+        coll,
+        docs,
+        [
+            {$group: {_id: "$myMeta", x: {$count: {}}, y: {$min: "$val"}, z: {$max: "$val"}}},
+            {$match: {_id: 1}}
+        ],
+        [{"_id": 1, "x": 2, "y": 3, "z": 5}]);
+})();
+
+// Test with a meta group key, and {$sum: 1}. Since $count desugars to {$sum:1}, the rewrite should
+// apply.
+(function testMetaGroupKey_WithSum1Accumulator() {
+    const t = new Date();
+    const docs = [
+        {time: t, myMeta: 1, val: 3},
+        {time: t, myMeta: 3, val: 4},
+        {time: t, myMeta: 1, val: 5},
+    ];
+    runGroupRewriteTest(coll,
+                        docs,
+                        [{$group: {_id: "$myMeta", x: {$sum: 1}}}, {$match: {_id: 1}}],
+                        [{"_id": 1, "x": 2}]);
+})();
+
+// Test with a meta group key, and {$sum: 7}. The group rewrite doesn't apply in this case but we
+// do a different optimization that takes into account that no fields should be unpacked.
+(function testMetaGroupKey_WithSum7Accumulator() {
+    const t = new Date();
+    const docs = [
+        {time: t, meta: 1, val: 3},
+        {time: t, meta: 3, val: 4},
+        {time: t, meta: 1, val: 5},
+    ];
+    runGroupRewriteTest(coll,
+                        docs,
+                        [{$group: {_id: "$meta", x: {$sum: 7}}}, {$match: {_id: 1}}],
+                        [{"_id": 1, "x": 2 * 7}]);
+})();
+
+// Test with a constant group key with the $count accumulator when there is no metaField. The
+// rewrite should apply.
+(function testConstGroupKey_WithCountAccumulator_NoMetaField() {
+    const t = new Date();
+    const docs = [
+        {time: t, myMeta: 1, val: 3},
+        {time: t, myMeta: 3, val: 4},
+        {time: t, myMeta: 1, val: 5},
+    ];
+    runGroupRewriteTest(coll,
+                        docs,
+                        [{$group: {_id: null, x: {$sum: 1}}}],
+                        [{"_id": null, "x": 3}],
+                        true /* excludeMeta */);
+})();
+
+// Test with a meta group key with the $sum accumulator. Even though the rewrite applies for $count
+// which desugars to {$sum: 1}, for accumulators like {$sum: "$fieldPath"} the rewrite does not
+// apply.
+(function testMetaGroupKey_WithSumFieldPath() {
+    const t = new Date();
+    const docs = [
+        {time: t, myMeta: 1, val: 3},
+        {time: t, myMeta: 3, val: 4},
+        {time: t, myMeta: 1, val: 5},
+    ];
+    runGroupRewriteTest(coll,
+                        docs,
+                        [{$group: {_id: "$myMeta", x: {$sum: "$val"}}}],
+                        [{"_id": 1, "x": 8}, {"_id": 3, "x": 4}]);
+})();
+
+// Test the $count aggregation stage. Since this stage is rewritten to a $group and $project, the
+// rewrite will apply.
+(function testMetaGroupKey_WithCountStage() {
+    const t = new Date();
+    const docs = [
+        {time: t, myMeta: 1, val: 3},
+        {time: t, myMeta: 3, val: 4},
+        {time: t, myMeta: 1, val: 5},
+    ];
+    runGroupRewriteTest(coll,
+                        docs,
+                        [{$group: {_id: null, min: {$min: "$val"}}}, {$count: "groupCount"}],
+                        [{"groupCount": 1}],
+                        true /* excludeMeta */);
 })();
 
 // Validates the $project followed by $group scenarios, where the $project may project out/modify
@@ -244,29 +422,35 @@ function runGroupRewriteTest(docs, pipeline, expectedResults) {
     //
     // Tests with inclusion projections.
     //
-    runGroupRewriteTest(docs,
+    runGroupRewriteTest(coll,
+                        docs,
                         [{$project: {myMeta: 1}}, {$group: {_id: null, o: {$max: "$time"}}}],
                         [{_id: null, o: null}],
                         false /* excludeMeta */);
-    runGroupRewriteTest(docs,
+    runGroupRewriteTest(coll,
+                        docs,
                         [{$project: {myMeta: 1}}, {$group: {_id: null, o: {$max: "$val"}}}],
                         [{_id: null, o: null}],
                         false /* excludeMeta */);
-    runGroupRewriteTest(docs,
+    runGroupRewriteTest(coll,
+                        docs,
                         [{$project: {myMeta: 1}}, {$group: {_id: null, o: {$max: "$myMeta"}}}],
                         [{_id: null, o: 3}],
                         false /* excludeMeta */);
     runGroupRewriteTest(
+        coll,
         docs,
         [{$project: {myMeta: 1}}, {$group: {_id: {x: "$myMeta"}, o: {$max: "$myMeta"}}}],
         [{_id: {x: 1}, o: 1}, {_id: {x: 3}, o: 3}],
         false /* excludeMeta */);
     runGroupRewriteTest(
+        coll,
         docs,
         [{$project: {myMeta: 1}}, {$group: {_id: {x: "$myMeta"}, o: {$max: "$val"}}}],
         [{_id: {x: 1}, o: null}, {_id: {x: 3}, o: null}],
         false /* excludeMeta */);
     runGroupRewriteTest(
+        coll,
         docs,
         [{$project: {myMeta: 1}}, {$group: {_id: {meta: "$myMeta"}, c: {$count: {}}}}],
         [{_id: {meta: 1}, c: 2}, {_id: {meta: 3}, c: 1}],
@@ -275,7 +459,8 @@ function runGroupRewriteTest(docs, pipeline, expectedResults) {
     //
     // Tests with exclusion projections.
     //
-    runGroupRewriteTest(docs,
+    runGroupRewriteTest(coll,
+                        docs,
                         [
                             {$project: {myMeta: 0}},
                             {$addFields: {m: "$myMeta"}},
@@ -283,25 +468,30 @@ function runGroupRewriteTest(docs, pipeline, expectedResults) {
                         ],
                         [{_id: null, o: null}],
                         false /* excludeMeta */);
-    runGroupRewriteTest(docs,
+    runGroupRewriteTest(coll,
+                        docs,
                         [{$project: {myMeta: 0}}, {$group: {_id: "$myMeta", o: {$max: "$val"}}}],
                         [{_id: null, o: 5}],
                         false /* excludeMeta */);
     runGroupRewriteTest(
+        coll,
         docs,
         [{$project: {myMeta: 0}}, {$group: {_id: {x: "$myMeta"}, o: {$max: "$val"}}}],
         [{_id: {x: null}, o: 5}],
         false /* excludeMeta */);
     runGroupRewriteTest(
+        coll,
         docs,
         [{$project: {myMeta: 0, val: 0}}, {$group: {_id: {x: "$myMeta"}, o: {$max: "$val"}}}],
         [{_id: {x: null}, o: null}],
         false /* excludeMeta */);
-    runGroupRewriteTest(docs,
+    runGroupRewriteTest(coll,
+                        docs,
                         [{$project: {myMeta: 0}}, {$group: {_id: null, o: {$max: "$val"}}}],
                         [{_id: null, o: 5}],
                         false /* excludeMeta */);
-    runGroupRewriteTest(docs,
+    runGroupRewriteTest(coll,
+                        docs,
                         [{$project: {myMeta: 0}}, {$group: {_id: null, c: {$count: {}}}}],
                         [{_id: null, c: 3}],
                         false /* excludeMeta */);
@@ -316,25 +506,30 @@ function runGroupRewriteTest(docs, pipeline, expectedResults) {
         {time: t, myMeta: 3, val: 4},
         {time: t, myMeta: 1, val: 5},
     ];
-    runGroupRewriteTest(docs,
+    runGroupRewriteTest(coll,
+                        docs,
                         [{$project: {time: "$myMeta"}}, {$group: {_id: null, o: {$max: "$time"}}}],
                         [{_id: null, o: 3}],
                         false /* excludeMeta */);
-    runGroupRewriteTest(docs,
+    runGroupRewriteTest(coll,
+                        docs,
                         [{$project: {val: "$myMeta"}}, {$group: {_id: null, o: {$max: "$val"}}}],
                         [{_id: null, o: 3}],
                         false /* excludeMeta */);
-    runGroupRewriteTest(docs,
+    runGroupRewriteTest(coll,
+                        docs,
                         [{$addFields: {myMeta: "5"}}, {$group: {_id: null, o: {$max: "$myMeta"}}}],
                         [{_id: null, o: "5"}],
                         false /* excludeMeta */);
 
     runGroupRewriteTest(
+        coll,
         docs,
         [{$addFields: {myMeta: "x"}}, {$group: {_id: "$myMeta", o: {$max: "$val"}}}],
         [{_id: "x", o: 5}],
         false /* excludeMeta */);
-    runGroupRewriteTest(docs,
+    runGroupRewriteTest(coll,
+                        docs,
                         [
                             {$addFields: {myMeta: "y"}},
                             {$project: {val: "$myMeta"}},
@@ -342,7 +537,8 @@ function runGroupRewriteTest(docs, pipeline, expectedResults) {
                         ],
                         [{_id: {x: null}, o: "y"}],
                         false /* excludeMeta */);
-    runGroupRewriteTest(docs,
+    runGroupRewriteTest(coll,
+                        docs,
                         [
                             {$addFields: {myMeta: "$val"}},
                             {$project: {val: 0}},
@@ -351,4 +547,3 @@ function runGroupRewriteTest(docs, pipeline, expectedResults) {
                         [{_id: {x: 3}, o: null}, {_id: {x: 4}, o: null}, {_id: {x: 5}, o: null}],
                         false /* excludeMeta */);
 })();
-})();
