@@ -2,8 +2,6 @@
  * Test creating and using partial indexes, on a time-series collection.
  *
  * @tags: [
- *   # TODO (SERVER-73316): remove
- *   assumes_against_mongod_not_mongos,
  *   # Explain of a resolved view must be executed by mongos.
  *   directly_against_shardsvrs_incompatible,
  *   # Refusing to run a test that issues an aggregation command with explain because it may return
@@ -11,41 +9,64 @@
  *   does_not_support_stepdowns,
  *   # We need a timeseries collection.
  *   requires_timeseries,
+ *   # During fcv upgrade/downgrade the index created might not be what we expect.
+ *   # TODO SERVER-79304 remove this tag.
+ *   cannot_run_during_upgrade_downgrade,
+ *   # TODO SERVER-89705 a concurrent moveCollection can cause the default index to have the
+ *   # collation in the new collection causing some assertions to fail.
+ *   assumes_balancer_off,
  * ]
  */
-(function() {
-"use strict";
-
-load("jstests/libs/analyze_plan.js");
+import {
+    getAggPlanStage,
+    getPlanStages,
+    getRejectedPlan,
+    getRejectedPlans
+} from "jstests/libs/analyze_plan.js";
+import {FeatureFlagUtil} from "jstests/libs/feature_flag_util.js";
+import {FixtureHelpers} from "jstests/libs/fixture_helpers.js";
 
 const coll = db.timeseries_index_partial;
 const timeField = 'time';
 const metaField = 'm';
-
-coll.drop();
-assert.commandWorked(db.createCollection(coll.getName(), {timeseries: {timeField, metaField}}));
-
-const buckets = db.getCollection('system.buckets.' + coll.getName());
 let extraIndexes = [];
 let extraBucketIndexes = [];
-if (FixtureHelpers.isSharded(buckets)) {
-    // If the collection is sharded, expect an implicitly-created index on time.
-    // It will appear differently in listIndexes depending on whether you look at the time-series
-    // collection or the buckets collection.
-    extraIndexes.push({
-        "v": 2,
-        "key": {"time": 1},
-        "name": "control.min.time_1",
-    });
-    extraBucketIndexes.push({
-        "v": 2,
-        "key": {"control.min.time": 1},
-        "name": "control.min.time_1",
-    });
-}
+let buckets = [];
 
-// TODO SERVER-66438: Remove feature flag check.
-if (FeatureFlagUtil.isPresentAndEnabled(db, "TimeseriesScalabilityImprovements")) {
+function resetCollection(collation) {
+    coll.drop();
+    extraIndexes = [];
+    extraBucketIndexes = [];
+
+    if (collation) {
+        assert.commandWorked(db.createCollection(coll.getName(), {
+            timeseries: {timeField, metaField},
+            collation: collation,
+        }));
+    } else {
+        assert.commandWorked(
+            db.createCollection(coll.getName(), {timeseries: {timeField, metaField}}));
+    }
+    buckets = db.getCollection('system.buckets.' + coll.getName());
+    // If the collection is sharded, expect an implicitly-created index on time. It will appear
+    // differently in listIndexes depending on whether you look at the time-series collection or
+    // the buckets collection.
+    // TODO SERVER-79304 the test shouldn't rely on the feature flag.
+    const newShardKeyIndex =
+        FeatureFlagUtil.isPresentAndEnabled(db, "AuthoritativeShardCollection");
+    if (FixtureHelpers.isSharded(buckets)) {
+        extraIndexes.push({
+            "v": 2,
+            "key": {"time": 1},
+            "name": newShardKeyIndex ? "time_1" : "control.min.time_1",
+        });
+        extraBucketIndexes.push({
+            "v": 2,
+            "key": newShardKeyIndex ? {"control.min.time": 1, "control.max.time": 1}
+                                    : {"control.min.time": 1},
+            "name": newShardKeyIndex ? "time_1" : "control.min.time_1",
+        });
+    }
     // When enabled, the {meta: 1, time: 1} index gets built by default on the time-series
     // bucket collection.
     extraIndexes.push({
@@ -60,6 +81,8 @@ if (FeatureFlagUtil.isPresentAndEnabled(db, "TimeseriesScalabilityImprovements")
     });
 }
 
+resetCollection();
+
 assert.sameMembers(coll.getIndexes(), extraIndexes);
 assert.sameMembers(buckets.getIndexes(), extraBucketIndexes);
 
@@ -102,10 +125,16 @@ assert.commandFailedWithCode(coll.createIndex({a: 1}, {partialFilterExpression:
         // If scan is not present, check rejected plans
         if (scan === null) {
             const rejectedPlans = getRejectedPlans(getAggPlanStage(explain, "$cursor")["$cursor"]);
-            if (rejectedPlans.length === 1) {
-                const scans = getPlanStages(getRejectedPlan(rejectedPlans[0]), "IXSCAN");
-                if (scans.length === 1) {
-                    scan = scans[0];
+            if (rejectedPlans.length === 2) {
+                let firstScan = getPlanStages(getRejectedPlan(rejectedPlans[0]), "IXSCAN");
+                let secondScan = getPlanStages(getRejectedPlan(rejectedPlans[1]), "IXSCAN");
+                // Both plans should have an "IXSCAN" stage and one stage should scan the index on
+                // the 'a' field.
+                if (firstScan.length === 1 && secondScan.length === 1) {
+                    scan = firstScan[0];
+                    if (secondScan[0]["indexName"] == "a_1") {
+                        scan = secondScan[0];
+                    }
                 }
             }
         } else {
@@ -158,6 +187,15 @@ assert.commandFailedWithCode(coll.createIndex({a: 1}, {partialFilterExpression:
 
     // Test some predicates on the time field.
     {
+        // Note on implicitly sharded collections this index is already made and this operation is a
+        // no-op.
+        // TODO SERVER-79304 the test shouldn't rely on the feature flag.
+        if (FeatureFlagUtil.isPresentAndEnabled(db, "AuthoritativeShardCollection")) {
+            assert.commandWorked(coll.createIndex({[timeField]: 1}));
+        } else if (!FixtureHelpers.isSharded(buckets)) {
+            assert.commandWorked(coll.createIndex({[timeField]: 1}));
+        }
+
         const t0 = ISODate('2000-01-01T00:00:00Z');
         const t1 = ISODate('2000-01-01T00:00:01Z');
         const t2 = ISODate('2000-01-01T00:00:02Z');
@@ -185,6 +223,11 @@ assert.commandFailedWithCode(coll.createIndex({a: 1}, {partialFilterExpression:
             coll.createIndex({a: 1}, {partialFilterExpression: {[timeField]: {$gte: t1}}}));
         check({a: {$lt: 999}, [timeField]: {$gte: t1}});
         check({a: {$lt: 999}, [timeField]: {$gte: t2}});
+
+        // Drop the index, so it doesn't interfere with other tests.
+        if (!FixtureHelpers.isSharded(buckets)) {
+            assert.commandWorked(coll.dropIndex({[timeField]: 1}));
+        }
     }
 
     assert.commandWorked(coll.dropIndex({a: 1}));
@@ -243,12 +286,9 @@ assert.sameMembers(buckets.getIndexes(), extraBucketIndexes.concat([
 
 // Test how partialFilterExpression interacts with collation.
 {
+    // Recreate the collection with a collation.
     const numericCollation = {locale: "en_US", numericOrdering: true};
-    coll.drop();
-    assert.commandWorked(db.createCollection(coll.getName(), {
-        timeseries: {timeField, metaField},
-        collation: numericCollation,
-    }));
+    resetCollection(numericCollation);
 
     assert.commandWorked(coll.insert([
         {[timeField]: ISODate(), [metaField]: {x: "1000", y: 1}, a: "120"},
@@ -586,4 +626,3 @@ assert.sameMembers(buckets.getIndexes(), extraBucketIndexes.concat([
         }
     });
 }
-})();
