@@ -2,20 +2,25 @@
  * Tests index usage on meta and time fields for timeseries collections.
  *
  * @tags: [
- *   does_not_support_stepdowns,
- *   does_not_support_transactions,
- *   requires_fcv_51,
- *   requires_getmore,
- *   requires_pipeline_optimization,
  *   # Explain of a resolved view must be executed by mongos.
  *   directly_against_shardsvrs_incompatible,
+ *   # Refusing to run a test that issues an aggregation command with explain because it may return
+ *   # incomplete results if interrupted by a stepdown.
+ *   does_not_support_stepdowns,
+ *   # Tests that optimization produces expected query plans.
+ *   requires_pipeline_optimization,
+ *   # We need a timeseries collection.
+ *   requires_timeseries,
  * ]
  */
-(function() {
-"use strict";
-
-load("jstests/core/timeseries/libs/timeseries.js");
-load("jstests/libs/analyze_plan.js");
+import {TimeseriesTest} from "jstests/core/timeseries/libs/timeseries.js";
+import {
+    getAggPlanStage,
+    getPlanStages,
+    getRejectedPlan,
+    getRejectedPlans
+} from "jstests/libs/analyze_plan.js";
+import {FixtureHelpers} from "jstests/libs/fixture_helpers.js";
 
 const generateTest = (useHint) => {
     return (insert) => {
@@ -40,6 +45,10 @@ const generateTest = (useHint) => {
                 coll.getName(),
                 Object.assign({timeseries: {timeField: timeFieldName, metaField: metaFieldName}},
                               collOpts)));
+            // The {meta: 1, time: 1} index gets built by default on the time-series bucket
+            // collection. When this index is present, the query planner will use it, changing the
+            // expected behaviour of this test. Drop the index.
+            assert.commandWorked(coll.dropIndex({[metaFieldName]: 1, [timeFieldName]: 1}));
 
             const dbCollNames = testDB.getCollectionNames();
             assert.contains(bucketsColl.getName(),
@@ -395,8 +404,7 @@ const generateTest = (useHint) => {
                            {},
                            collation);
 
-        /*********************************** Tests $expr predicates
-         * *********************************/
+        /*********************************** Tests $expr predicates *******************************/
         resetCollections();
         assert.commandWorked(insert(coll, [
             {_id: 0, [timeFieldName]: ISODate('1990-01-01 00:00:00.000Z'), [metaFieldName]: 2},
@@ -404,9 +412,8 @@ const generateTest = (useHint) => {
             {_id: 2, [timeFieldName]: ISODate('2010-01-01 00:00:00.000Z'), [metaFieldName]: 2}
         ]));
 
-        if (!FixtureHelpers.isSharded(bucketsColl)) {
-            // Skip if the collection is implicitly sharded: it may use the implicitly created
-            // index.
+        if (useHint) {
+            // Skip if without hint: it may use the implicitly created index.
             testAggregationUsesIndex([{
                                          $match: {
                                              $expr: {
@@ -426,4 +433,3 @@ const generateTest = (useHint) => {
 // Run the test twice, once without hinting the index, and again hinting the index by spec.
 TimeseriesTest.run(generateTest(false));
 TimeseriesTest.run(generateTest(true));
-})();
