@@ -5,10 +5,7 @@
  *   requires_timeseries,
  * ]
  */
-(function() {
-"use strict";
-
-load("jstests/core/timeseries/libs/timeseries.js");
+import {TimeseriesTest} from "jstests/core/timeseries/libs/timeseries.js";
 
 const coll = db.timeseries_insert;
 coll.drop();
@@ -31,5 +28,4 @@ for (let i = 0; i < 100; i++) {
         fields: host.fields,
         tags: host.tags,
     }));
-}
-})();
\ No newline at end of file
+}
\ No newline at end of file
