@@ -4,8 +4,6 @@
  * collection.
  *
  * @tags: [
- *   # TODO (SERVER-73321): remove
- *   assumes_against_mongod_not_mongos,
  *   # Explain of a resolved view must be executed by mongos.
  *   directly_against_shardsvrs_incompatible,
  *   # Time series geo functionality requires optimization.
@@ -13,17 +11,14 @@
  *   # Refusing to run a test that issues an aggregation command with explain because it may return
  *   # incomplete results if interrupted by a stepdown.
  *   does_not_support_stepdowns,
- *   # Error-handling behavior updated in 6.2, will cause issues for multiversion testing
- *   requires_fcv_62,
  *   # We need a timeseries collection.
  *   requires_timeseries,
+ *   # Versions before 7.2 incorrectly allow $_internalBucketGeoWithin within $elemMatch.
+ *   requires_fcv_72,
  * ]
  */
 
-(function() {
-"use strict";
-
-load("jstests/libs/analyze_plan.js");
+import {getAggPlanStages} from "jstests/libs/analyze_plan.js";
 
 const coll = db.timeseries_internal_bucket_geo_within;
 coll.drop();
@@ -53,10 +48,6 @@ for (let collScanStage of collScanStages) {
             "field": "loc"
         }
     };
-    // TODO SERVER-60373 Fix duplicate predicates for sharded time-series collection
-    if (FixtureHelpers.isSharded(bucketsColl)) {
-        expectedPredicate = {$and: [expectedPredicate, expectedPredicate]};
-    }
     assert.docEq(expectedPredicate, collScanStage.filter, collScanStages);
 }
 
@@ -319,5 +310,27 @@ assert.sameMembers(results, [
     pipeline = [{$match: {loc: {$geoWithin: {$centerSphere: [[0, 80], 1], $center: [[0, 0], 5]}}}}];
     err = assert.throws(() => coll.explain().aggregate(pipeline));
     assert.eq(err.code, ErrorCodes.BadValue, err);
+
+    // $_internalBucketGeoWithin is not allowed in $elemMatch.
+    pipeline = [{
+        $match: {
+            abc: {
+                $elemMatch: {
+                    "$_internalBucketGeoWithin": {
+                        "withinRegion": {
+                            "$geometry": {
+                                "type": "Polygon",
+                                "coordinates": [[[0, 0], [3, 6], [6, 1], [0, 0]]]
+                            }
+                        },
+                        "field": "loc"
+                    }
+                }
+            }
+        }
+    }];
+    err = assert.throws(() => coll.explain().aggregate(pipeline));
+    assert.eq(err.code,
+              ErrorCodes.QueryFeatureNotAllowed,
+              {expectedCode: ErrorCodes.QueryFeatureNotAllowed, actualCode: err.code, err});
 }
-}());
