@@ -7,11 +7,6 @@
  *   requires_timeseries,
  * ]
  */
-(function() {
-'use strict';
-
-load("jstests/core/timeseries/libs/timeseries.js");
-
 const timeFieldName = 'time';
 
 const coll = db.timeseries_list_collections_filter_name;
@@ -36,5 +31,4 @@ const collectionOptions = [{
     info: {readOnly: false},
 }];
 
-assert.eq(collections, collectionOptions);
-})();
+assert.eq(collections, collectionOptions);
\ No newline at end of file
