@@ -9,10 +9,7 @@
  *   directly_against_shardsvrs_incompatible,
  * ]
  */
-(function() {
-"use strict";
-
-load("jstests/libs/analyze_plan.js");  // For getAggPlanStages
+import {getAggPlanStages} from "jstests/libs/analyze_plan.js";
 
 const coll = db.timeseries_match_pushdown;
 const timeField = 'time';
@@ -630,4 +627,3 @@ runTest({
         {[timeField]: ISODate('2022-01-01T00:00:04'), [metaField]: 2},
     ]
 });
-})();
