@@ -8,12 +8,8 @@
  *   requires_timeseries,
  * ]
  */
-(function() {
-"use strict";
-
-load("jstests/core/timeseries/libs/timeseries.js");
-load("jstests/libs/feature_flag_util.js");
-load("jstests/libs/fixture_helpers.js");
+import {TimeseriesTest} from "jstests/core/timeseries/libs/timeseries.js";
+import {FixtureHelpers} from "jstests/libs/fixture_helpers.js";
 
 TimeseriesTest.run((insert) => {
     const collName = "timeseries_metric_index_ascending_descending";
@@ -154,12 +150,10 @@ TimeseriesTest.run((insert) => {
     // There are more indexes for sharded collections because it includes the shard key index. When
     // time-series scalability improvements are enabled, the {meta: 1, time: 1} index gets built by
     // default on the time-series bucket collection.
-    const numExtraIndexes = (FixtureHelpers.isSharded(bucketsColl) ? 1 : 0) +
-        (FeatureFlagUtil.isPresentAndEnabled(db, "TimeseriesScalabilityImprovements") ? 1 : 0);
+    const numExtraIndexes = (FixtureHelpers.isSharded(bucketsColl) ? 1 : 0) + 1;
 
     userIndexes = coll.getIndexes();
     assert.eq(numExtraIndexes, userIndexes.length, tojson(userIndexes));
     bucketIndexes = bucketsColl.getIndexes();
     assert.eq(13 + numExtraIndexes, bucketIndexes.length, tojson(bucketIndexes));
 });
-}());
