@@ -6,10 +6,7 @@
  *   requires_timeseries,
  * ]
  */
-(function() {
-"use strict";
-
-load("jstests/core/timeseries/libs/timeseries.js");
+import {TimeseriesTest} from "jstests/core/timeseries/libs/timeseries.js";
 
 TimeseriesTest.run((insert) => {
     const collName = "timeseries_metric_index_hashed";
@@ -41,4 +38,3 @@ TimeseriesTest.run((insert) => {
     testIndex({[`${metaFieldName}.tag`]: 1, x: "hashed"});
     testIndex({x: 1, [`${metaFieldName}.tag`]: -1, y: "hashed"});
 });
-}());
