@@ -8,13 +8,17 @@
  *   does_not_support_stepdowns,
  *   # We need a timeseries collection.
  *   requires_timeseries,
- *   requires_fcv_70,
+ *   # TODO(mbroadst): Some bug here, appears to be double-prefixing
+ *   not_allowed_with_signed_security_token,
+ *   # TODO SERVER-88275: aggregation using internally a $mergeCursor stage can fail with
+ *   # QueryPlanKilled in suites with random migrations because moveCollection change the collection
+ *   # UUID by dropping and re-creating the collection. This specially happens on $out aggregations
+ *   # where the collection doesn't live on the primary shard.
+ *   assumes_balancer_off,
  * ]
  */
-(function() {
-"use strict";
-
-load("jstests/core/timeseries/libs/timeseries_agg_helpers.js");
+import {TimeseriesAggTests} from "jstests/core/timeseries/libs/timeseries_agg_helpers.js";
+import {FixtureHelpers} from "jstests/libs/fixture_helpers.js";
 
 const numHosts = 10;
 const numIterations = 20;
@@ -114,12 +118,13 @@ const destDB = testDB.getSiblingDB("outDifferentDB");
 assert.commandWorked(destDB.dropDatabase());
 timeseriesPipeline =
     TimeseriesAggTests.generateOutPipeline(targetCollName, destDB.getName(), {timeField: "time"});
-// TODO SERVER-75856 remove this conditional.
-if (FixtureHelpers.isMongos(testDB)) {  // this is not supported in mongos.
-    assert.throwsWithCode(() => inColl.aggregate(timeseriesPipeline), ErrorCodes.NamespaceNotFound);
-} else {
+// TODO (SERVER-75856): Support implicit database creation for $merge and $out when running
+// aggregate on a mongos.
+try {
     inColl.aggregate(timeseriesPipeline);
     assert.eq(300, destDB[targetCollName].find().itcount());
+} catch (e) {
+    assert.eq(e.code, ErrorCodes.NamespaceNotFound, e);
 }
 
 // Tests that an error is raised when trying to create a time-series collection from a non
@@ -131,8 +136,8 @@ assert.throwsWithCode(() => observerInColl.aggregate(pipeline), 7268700);
 // Tests that an error is raised for invalid timeseries options.
 pipeline = TimeseriesAggTests.generateOutPipeline(
     targetCollName, dbName, {timeField: "time", invalidField: "invalid"});
-assert.throwsWithCode(() => inColl.aggregate(pipeline), 40415);
-assert.throwsWithCode(() => observerInColl.aggregate(pipeline), 40415);
+assert.throwsWithCode(() => inColl.aggregate(pipeline), ErrorCodes.IDLUnknownField);
+assert.throwsWithCode(() => observerInColl.aggregate(pipeline), ErrorCodes.IDLUnknownField);
 
 // Tests that an error is raised if the user changes the 'timeField'.
 pipeline =
@@ -178,7 +183,6 @@ assert.throwsWithCode(() => observerInColl.aggregate(pipeline), ErrorCodes.BadVa
 if (!FixtureHelpers.isMongos(testDB)) {  // can not shard a view.
     assert.commandWorked(testDB.createCollection("view_out", {viewOn: "out"}));
     pipeline = TimeseriesAggTests.generateOutPipeline("view_out", dbName, {timeField: "time"});
-    assert.throwsWithCode(() => inColl.aggregate(pipeline), 7268703);
-    assert.throwsWithCode(() => observerInColl.aggregate(pipeline), 7268703);
+    assert.throws(() => inColl.aggregate(pipeline));
+    assert.throws(() => observerInColl.aggregate(pipeline));
 }
-})();
