@@ -2,15 +2,11 @@
  * Verifies that showRecordId() returns the ObjectId type for time-series collections.
  *
  * @tags: [
- *   does_not_support_stepdowns,
- *   does_not_support_transactions,
- *   requires_getmore,
+ *   # We need a timeseries collection.
+ *   requires_timeseries,
  * ]
  */
-(function() {
-"use strict";
-
-load("jstests/core/timeseries/libs/timeseries.js");
+import {TimeseriesTest} from "jstests/core/timeseries/libs/timeseries.js";
 
 TimeseriesTest.run((insert) => {
     const timeFieldName = "time";
@@ -59,5 +55,4 @@ TimeseriesTest.run((insert) => {
 
     const bucketsColl = db.getCollection("system.buckets." + coll.getName());
     checkRecordId(bucketsColl.find().showRecordId().toArray());
-});
-})();
+});
\ No newline at end of file
