@@ -2,17 +2,18 @@
  * Typically, time-series collections use measurements that always contain data for every field.
  * This test provides coverage for when this is not the case.
  * @tags: [
- *   # This test depends on certain writes ending up in the same bucket. Stepdowns may result in
- *   # writes splitting between two primaries, and thus different buckets.
+ *   # This test depends on certain writes ending up in the same bucket. Stepdowns and tenant
+ *   # migrations may result in writes splitting between two primaries, and thus different buckets.
  *   does_not_support_stepdowns,
+ *   tenant_migration_incompatible,
  *   # We need a timeseries collection.
  *   requires_timeseries,
+ *   # TODO SERVER-89764 a concurrent moveCollection during insertion can cause the bucket
+ *   # collection to insert more documents then expected by the test.
+ *   assumes_balancer_off,
  * ]
  */
-(function() {
-"use strict";
-
-load("jstests/core/timeseries/libs/timeseries.js");
+import {TimeseriesTest} from "jstests/core/timeseries/libs/timeseries.js";
 
 TimeseriesTest.run((insert) => {
     const collNamePrefix = 'timeseries_sparse_';
@@ -55,6 +56,10 @@ TimeseriesTest.run((insert) => {
         const bucketDocs = bucketsColl.find().sort({'control.min._id': 1}).toArray();
         assert.eq(1, bucketDocs.length, bucketDocs);
 
+        if (TimeseriesTest.timeseriesAlwaysUseCompressedBucketsEnabled(db)) {
+            TimeseriesTest.decompressBucket(bucketDocs[0]);
+        }
+
         // Check bucket.
         assert.eq(docs.length,
                   Object.keys(bucketDocs[0].data[timeFieldName]).length,
@@ -91,4 +96,3 @@ TimeseriesTest.run((insert) => {
             {_id: 3, time: t[3], c: 33, d: 30},
         ]);
 });
-})();
