@@ -4,14 +4,11 @@
  * @tags: [
  *   uses_transactions,
  *   requires_db_locking,
- *   # Creating sharded collections is not supported in multi-document transactions.
+ *   # TODO SERVER-86727 Can probably enable for sharded collections.
  *   assumes_unsharded_collection,
  * ]
  */
 
-(function() {
-"use strict";
-
 let dbName = 'create_collection_not_blocked_by_txn';
 let mydb = db.getSiblingDB(dbName);
 
@@ -34,4 +31,3 @@ assert.commandWorked(mydb.createCollection("b"));
 assert.commandWorked(mydb.runCommand({insert: "c", documents: [{x: 2}]}));
 
 assert.commandWorked(session.commitTransaction_forTesting());
-})();
