@@ -4,9 +4,6 @@
  * @tags: [uses_transactions, requires_db_locking, assumes_unsharded_collection]
  */
 
-(function() {
-"use strict";
-
 let dbName = 'drop_collection_not_blocked_by_txn';
 let mydb = db.getSiblingDB(dbName);
 
@@ -23,5 +20,4 @@ sessionDb.a.insert({y: 1});
 // This only requires database IX lock.
 assert.commandWorked(mydb.runCommand({drop: "b"}));
 
-assert.commandWorked(session.commitTransaction_forTesting());
-})();
+assert.commandWorked(session.commitTransaction_forTesting());
\ No newline at end of file
