@@ -1,9 +1,11 @@
 // Tests that the killCursors command is allowed in transactions.
-// The test runs commands that are not allowed with security token: endSession, killCursors.
+//
 // @tags: [
-//   not_allowed_with_security_token,uses_transactions, uses_parallel_shell]
-(function() {
-"use strict";
+//   # The test runs commands that are not allowed with security token: endSession, killCursors.
+//   not_allowed_with_signed_security_token,
+//   uses_transactions,
+//   uses_parallel_shell
+// ]
 
 const dbName = "test";
 const collName = "kill_cursors_in_transaction";
@@ -60,14 +62,7 @@ assert.soon(
                        {
                            $match: {
                                $or: [
-                                   {
-                                       $or: [
-                                           {'command.drop': collName},
-                                           // TODO SERVER-73627: Remove once 7.0 becomes last LTS.
-                                           {'command._shardsvrDropCollectionParticipant': collName}
-                                       ],
-                                       waitingForLock: true
-                                   },
+                                   {'command.drop': collName, waitingForLock: true},
                                    {'command._shardsvrParticipantBlock': collName},
                                ]
                            }
@@ -80,13 +75,27 @@ assert.soon(
             tojson(adminDB.aggregate([{$currentOp: {}}]).toArray());
     });
 
-// killCursors does not block behind the pending MODE_X lock.
-assert.commandWorked(sessionDb.runCommand({killCursors: collName, cursors: [res.cursor.id]}));
-
-assert.commandWorked(session.commitTransaction_forTesting());
+// killCursors does not block behind the pending MODE_X lock. It is possible that due to ticket
+// exhaustion we end up detecting a deadlocked state, where the drop operation is waiting for
+// an X collection lock but cannot acquire it because IX locks are being held by the killCursor
+// operation, in which case we fail the killCursor command. If there was an error running the
+// command below we should ensure that is a TransientTransactionError with a code of LockTimeOut,
+// and ensure that the transaction was successfully rolled back.
+res = sessionDb.runCommand({killCursors: collName, cursors: [res.cursor.id]});
+if (res.ok) {
+    assert.commandWorked(session.commitTransaction_forTesting());
+} else {
+    const isTransientTxnError =
+        res.hasOwnProperty("errorLabels") && res.errorLabels.includes("TransientTransactionError");
+    const isLockTimeout = res.hasOwnProperty("code") && ErrorCodes.LockTimeout === res.code;
+    assert(isTransientTxnError, res);
+    assert(isLockTimeout, res);
+    // The transaction should have implicitly been aborted.
+    assert.commandFailedWithCode(session.abortTransaction_forTesting(),
+                                 ErrorCodes.NoSuchTransaction);
+}
 
 // Once the transaction has committed, the drop can proceed.
 awaitDrop();
 
 session.endSession();
-}());
