@@ -1,13 +1,8 @@
 /**
  * Helper functions for testing prepared transactions.
- *
- * The test runs commands that are not allowed with security token: dataSize, prepareTransaction,
- * replSetGetStatus.
- * @tags: [
- *   not_allowed_with_security_token,uses_transactions]
- *
  */
-const PrepareHelpers = (function() {
+
+export const PrepareHelpers = (function() {
     /**
      * Prepares the active transaction on the session. This expects the 'prepareTransaction' command
      * to succeed and return a non-null 'prepareTimestamp'.
