@@ -1,14 +1,16 @@
-// The test runs commands that are not allowed with security token: endSession.
-// @tags: [
-//   not_allowed_with_security_token,uses_transactions, uses_snapshot_read_concern]
 // This test ensures that listCollections does not conflict with multi-statement transactions
 // as a result of taking MODE_S locks that are incompatible with MODE_IX needed for writes.
-(function() {
-"use strict";
+//
+// @tags: [
+//   # The test runs commands that are not allowed with security token: endSession.
+//   not_allowed_with_signed_security_token,
+//   uses_transactions,
+//   uses_snapshot_read_concern
+// ]
 
-// TODO (SERVER-39704): Remove the following load after SERVER-397074 is completed
-// For withTxnAndAutoRetryOnMongos.
-load('jstests/libs/auto_retry_transaction_in_sharding.js');
+// TODO (SERVER-39704): Remove the following load after SERVER-39704 is completed
+import {withTxnAndAutoRetryOnMongos} from "jstests/libs/auto_retry_transaction_in_sharding.js";
+import {FixtureHelpers} from "jstests/libs/fixture_helpers.js";
 
 var dbName = 'list_collections_not_blocked';
 var mydb = db.getSiblingDB(dbName);
@@ -19,8 +21,7 @@ mydb.foo.drop({writeConcern: {w: "majority"}});
 
 assert.commandWorked(mydb.createCollection("foo", {writeConcern: {w: "majority"}}));
 
-const isMongos = assert.commandWorked(db.runCommand("hello")).msg === "isdbgrid";
-if (isMongos) {
+if (FixtureHelpers.isMongos(db) || TestData.testingReplicaSetEndpoint) {
     // Before starting the transaction below, access the collection so it can be implicitly
     // sharded and force all shards to refresh their database versions because the refresh
     // requires an exclusive lock and would block behind the transaction.
@@ -50,4 +51,3 @@ withTxnAndAutoRetryOnMongos(session, () => {
 }, {readConcern: {level: "snapshot"}});
 
 session.endSession();
-}());
