@@ -11,10 +11,7 @@
  * ]
  */
 
-(function() {
-"use strict";
-load('jstests/libs/uuid_util.js');
-load("jstests/libs/fixture_helpers.js");  // For FixtureHelpers.
+import {FixtureHelpers} from "jstests/libs/fixture_helpers.js";
 
 // Makes assertions on commands run without logical session ids.
 TestData.disableImplicitSessions = true;
@@ -184,8 +181,9 @@ assert.commandFailedWithCode(sessionDb.runCommand({
 }),
                              ErrorCodes.InvalidOptions);
 
-// Mongos has special handling for commitTransaction to support commit recovery.
-if (!FixtureHelpers.isMongos(sessionDb)) {
+// The command above was run with autocommit: true, and commit recovery on a router requires
+// "recoveryToken".
+if (!FixtureHelpers.isMongos(sessionDb) && !TestData.testingReplicaSetEndpoint) {
     // Committing the transaction should fail.
     assert.commandFailedWithCode(sessionDb.adminCommand({
         commitTransaction: 1,
@@ -318,5 +316,4 @@ assert.commandFailedWithCode(
 
 // Aborting the transaction should succeed.
 assert.commandWorked(sessionDb.adminCommand(
-    {abortTransaction: 1, txnNumber: NumberLong(txnNumber), autocommit: false}));
-}());
+    {abortTransaction: 1, txnNumber: NumberLong(txnNumber), autocommit: false}));
\ No newline at end of file
