@@ -2,20 +2,19 @@
  * Tests that new transactions on a session block behind an existing prepared transaction on the
  * session.
  *
- * The test runs commands that are not allowed with security token: endSession, prepareTransaction.
  * @tags: [
- *   not_allowed_with_security_token,
+ *   # The test runs commands that are not allowed with security token: endSession,
+ *   # prepareTransaction.
+ *   not_allowed_with_signed_security_token,
  *   uses_transactions,
  *   uses_prepare_transaction,
  *   uses_parallel_shell
  * ]
  */
 
-(function() {
-"use strict";
-load("jstests/core/txns/libs/prepare_helpers.js");
-load("jstests/libs/curop_helpers.js");  // for waitForCurOpByFailPoint().
-load("jstests/libs/parallel_shell_helpers.js");
+import {PrepareHelpers} from "jstests/core/txns/libs/prepare_helpers.js";
+import {waitForCurOpByFailPointNoNS} from "jstests/libs/curop_helpers.js";
+import {funWithArgs} from "jstests/libs/parallel_shell_helpers.js";
 
 /**
  * Launches a parallel shell to start a new transaction on the session with the given lsid. It
@@ -158,4 +157,3 @@ assert.commandWorked(testDB.runCommand({create: collName, writeConcern: {w: "maj
 
     session.endSession();
 })();
-}());
