@@ -1,8 +1,11 @@
 // Tests that it is illegal to write to system collections within a transaction.
-// @tags: [uses_transactions, uses_snapshot_read_concern]
-(function() {
-"use strict";
-
+// @tags: [
+//   uses_transactions,
+//   uses_snapshot_read_concern,
+//   # system.js stored functions only work for collections that live on the db-primary shard so
+//   # we have to make sure it wont be moved anywhere by the balancer
+//   assumes_balancer_off
+// ]
 const session = db.getMongo().startSession();
 
 // Use a custom database, to avoid conflict with other tests that use the system.js collection.
@@ -52,7 +55,7 @@ assert.commandFailedWithCode(session.abortTransaction_forTesting(), ErrorCodes.N
 
 session.startTransaction({readConcern: {level: "snapshot"}});
 assert.commandFailedWithCode(systemDotViews.insert({_id: "new.view", viewOn: "bar", pipeline: []}),
-                             50791);
+                             [ErrorCodes.OperationNotSupportedInTransaction, 50791]);
 assert.commandFailedWithCode(session.abortTransaction_forTesting(), ErrorCodes.NoSuchTransaction);
 
 session.startTransaction({readConcern: {level: "snapshot"}});
@@ -70,4 +73,3 @@ assert.commandFailedWithCode(session.abortTransaction_forTesting(), ErrorCodes.N
 
 assert.commandWorked(systemColl.remove({_id: {$exists: true}}));
 assert.eq(systemColl.find().itcount(), 0);
-}());
