@@ -1,16 +1,13 @@
 // Verifies which read concern levels transactions support, with and without afterClusterTime.
 //
-// The test runs commands that are not allowed with security token: endSession.
 // @tags: [
-//   not_allowed_with_security_token,
+//   # The test runs commands that are not allowed with security token: endSession.
+//   not_allowed_with_signed_security_token,
 //   uses_transactions,
 //   uses_snapshot_read_concern,
 //   requires_majority_read_concern,
 // ]
-(function() {
-"use strict";
-
-load('jstests/libs/auto_retry_transaction_in_sharding.js');
+import {withTxnAndAutoRetryOnMongos} from "jstests/libs/auto_retry_transaction_in_sharding.js";
 
 const dbName = "test";
 const collName = "supported_read_concern_levels";
@@ -65,4 +62,3 @@ for (let level of kUnsupportedLevels) {
     runTest(level, {causalConsistency: false}, false /*supported*/);
     runTest(level, {causalConsistency: true}, false /*supported*/);
 }
-}());
