@@ -2,25 +2,20 @@
  * Tests that timestamped reads, reads with snapshot and afterClusterTime, wait for the prepare
  * transaction oplog entry to be visible before choosing a read timestamp.
  *
- * The test runs commands that are not allowed with security token: prepareTransaction.
  * @tags: [
- *  not_allowed_with_security_token,
+ *  # The test runs commands that are not allowed with security token: prepareTransaction.
+ *  not_allowed_with_signed_security_token,
  *  uses_transactions,
  *  uses_prepare_transaction,
  *  uses_parallel_shell,
  *  # 'setDefaultRWConcern' is not supposed to be run on shard nodes.
  *  command_not_supported_in_serverless,
- *   # TODO SERVER-70847: Snapshot reads do not succeed on non-conflicting documents while txn is
- *   # in prepare.
- *  cqf_incompatible,
  * ]
  */
-(function() {
-'use strict';
 
-load('jstests/core/txns/libs/prepare_helpers.js');
-load("jstests/libs/fail_point_util.js");
-load('jstests/libs/parallel_shell_helpers.js');
+import {PrepareHelpers} from "jstests/core/txns/libs/prepare_helpers.js";
+import {configureFailPoint} from "jstests/libs/fail_point_util.js";
+import {funWithArgs} from "jstests/libs/parallel_shell_helpers.js";
 
 TestData.dbName = 'test';
 const baseCollName = 'timestamped_reads_wait_for_prepare_oplog_visibility';
@@ -48,8 +43,8 @@ TestData.otherDocFilter = {
  * field. This function is run in a separate thread and tests that oplog visibility blocks
  * certain reads and that prepare conflicts block other types of reads.
  */
-const readThreadFunc = function(readFunc, _collName, hangTimesEntered, logTimesEntered) {
-    load("jstests/libs/fail_point_util.js");
+const readThreadFunc = async function(readFunc, _collName, hangTimesEntered, logTimesEntered) {
+    const {kDefaultWaitForFailPointTimeout} = await import("jstests/libs/fail_point_util.js");
 
     // Do not start reads until we are blocked in 'prepareTransaction'.
     assert.commandWorked(db.adminCommand({
@@ -141,8 +136,6 @@ function runTest(prefix, readFunc) {
 }
 
 const snapshotRead = function(_collName) {
-    const _db = db.getSiblingDB(TestData.dbName);
-
     const session = db.getMongo().startSession({causalConsistency: false});
     const sessionDB = session.getDatabase(TestData.dbName);
 
@@ -281,4 +274,3 @@ const normalRead = function(_collName) {
 runTest('normal_reads', normalRead);
 runTest('snapshot_reads', snapshotRead);
 runTest('afterClusterTime', afterClusterTime);
-})();
