@@ -1,7 +1,7 @@
-// The test runs a lot of commands that are not allowed with security token: addShard,
-// addShardToZone, appendOplogNote, applyOps, and so on.
 // @tags: [
-//   not_allowed_with_security_token,
+//   # The test runs a lot of commands that are not allowed with security token: addShard,
+//   # addShardToZone, appendOplogNote, applyOps, and so on.
+//   not_allowed_with_signed_security_token,
 //   assumes_unsharded_collection,
 //   assumes_superuser_permissions,
 //   does_not_support_stepdowns,
@@ -17,7 +17,8 @@
 //   directly_against_shardsvrs_incompatible,
 //   # This test has statements that do not support non-local read concern.
 //   does_not_support_causal_consistency,
-//   uses_compact
+//   uses_compact,
+//   creates_and_authenticates_user,
 // ]
 
 /*
@@ -72,11 +73,12 @@
  *      If true, do not run this command on a standalone mongod.
  */
 
-(function() {
-"use strict";
-
-load('jstests/sharding/libs/last_lts_mongod_commands.js');
-load('jstests/sharding/libs/last_lts_mongos_commands.js');
+import {
+    commandsRemovedFromMongodSinceLastLTS
+} from "jstests/sharding/libs/last_lts_mongod_commands.js";
+import {
+    commandsRemovedFromMongosSinceLastLTS
+} from "jstests/sharding/libs/last_lts_mongos_commands.js";
 
 // Pre-written reasons for skipping a test.
 const isAnInternalCommand = "internal command";
@@ -107,17 +109,17 @@ let viewsCommandTests = {
     _configsvrCommitMergeAllChunksOnShard: {skip: isAnInternalCommand},
     _configsvrCommitMovePrimary:
         {skip: isAnInternalCommand},  // Can be removed once 6.0 is last LTS
+    _configsvrCommitRefineCollectionShardKey: {skip: isAnInternalCommand},
     _configsvrCommitReshardCollection: {skip: isAnInternalCommand},
     _configsvrConfigureCollectionBalancing: {skip: isAnInternalCommand},
     _configsvrCreateDatabase: {skip: isAnInternalCommand},
     _configsvrDropIndexCatalogEntry: {skip: isAnInternalCommand},
     _configsvrEnsureChunkVersionIsGreaterThan: {skip: isAnInternalCommand},
-    _configsvrGetHistoricalPlacement: {skip: isAnInternalCommand},  // TODO SERVER-73029 remove
+    _configsvrGetHistoricalPlacement: {skip: isAnInternalCommand},
     _configsvrMovePrimary: {skip: isAnInternalCommand},
     _configsvrMoveRange: {skip: isAnInternalCommand},
     _configsvrRefineCollectionShardKey: {skip: isAnInternalCommand},
     _configsvrRenameCollection: {skip: isAnInternalCommand},
-    _configsvrRenameCollectionMetadata: {skip: isAnInternalCommand},
     _configsvrRemoveChunks: {skip: isAnInternalCommand},
     _configsvrRemoveShard: {skip: isAnInternalCommand},
     _configsvrRemoveShardFromZone: {skip: isAnInternalCommand},
@@ -148,9 +150,6 @@ let viewsCommandTests = {
     _migrateClone: {skip: isAnInternalCommand},
     _mongotConnPoolStats: {skip: isAnInternalCommand},
     _movePrimary: {skip: isAnInternalCommand},
-    _movePrimaryRecipientAbortMigration: {skip: isAnInternalCommand},
-    _movePrimaryRecipientForgetMigration: {skip: isAnInternalCommand},
-    _movePrimaryRecipientSyncData: {skip: isAnInternalCommand},
     _recvChunkAbort: {skip: isAnInternalCommand},
     _recvChunkCommit: {skip: isAnInternalCommand},
     _recvChunkReleaseCritSec: {skip: isAnInternalCommand},
@@ -158,18 +157,23 @@ let viewsCommandTests = {
     _recvChunkStatus: {skip: isAnInternalCommand},
     _refreshQueryAnalyzerConfiguration: {skip: isAnInternalCommand},
     _shardsvrAbortReshardCollection: {skip: isAnInternalCommand},
+    _shardsvrBeginMigrationBlockingOperation: {skip: isAnInternalCommand},
+    _shardsvrChangePrimary: {skip: isAnInternalCommand},
     _shardsvrCheckMetadataConsistency: {skip: isAnInternalCommand},
     _shardsvrCheckMetadataConsistencyParticipant: {skip: isAnInternalCommand},
+    _shardsvrCleanupStructuredEncryptionData: {skip: isAnInternalCommand},
     _shardsvrCloneCatalogData: {skip: isAnInternalCommand},
     _shardsvrCompactStructuredEncryptionData: {skip: isAnInternalCommand},
+    _shardsvrConvertToCapped: {skip: isAnInternalCommand},
+    _shardsvrCoordinateMultiUpdate: {skip: isAnInternalCommand},
     _shardsvrDropCollection: {skip: isAnInternalCommand},
-    // TODO SERVER-74324: deprecate _shardsvrDropCollectionIfUUIDNotMatching after 7.0 is lastLTS.
-    _shardsvrDropCollectionIfUUIDNotMatching: {skip: isUnrelated},
     _shardsvrDropCollectionIfUUIDNotMatchingWithWriteConcern: {skip: isUnrelated},
     _shardsvrDropCollectionParticipant: {skip: isAnInternalCommand},
     _shardsvrDropIndexCatalogEntryParticipant: {skip: isAnInternalCommand},
     _shardsvrDropIndexes: {skip: isAnInternalCommand},
+    _shardsvrEndMigrationBlockingOperation: {skip: isAnInternalCommand},
     _shardsvrInsertGlobalIndexKey: {skip: isAnInternalCommand},
+    _shardsvrJoinDDLCoordinators: {skip: isAnInternalCommand},
     _shardsvrDeleteGlobalIndexKey: {skip: isAnInternalCommand},
     _shardsvrWriteGlobalIndexKeys: {skip: isAnInternalCommand},
     _shardsvrCleanupReshardCollection: {skip: isAnInternalCommand},
@@ -206,16 +210,30 @@ let viewsCommandTests = {
     _shardsvrSetAllowMigrations: {skip: isAnInternalCommand},
     _shardsvrSetClusterParameter: {skip: isAnInternalCommand},
     _shardsvrSetUserWriteBlockMode: {skip: isAnInternalCommand},
+    _shardsvrUntrackUnsplittableCollection: {skip: isAnInternalCommand},
     _shardsvrValidateShardKeyCandidate: {skip: isAnInternalCommand},
     _shardsvrCollMod: {skip: isAnInternalCommand},
     _shardsvrCollModParticipant: {skip: isAnInternalCommand},
+    _shardsvrConvertToCappedParticipant: {skip: isAnInternalCommand},
     _shardsvrParticipantBlock: {skip: isAnInternalCommand},
     _shardsvrUnregisterIndex: {skip: isAnInternalCommand},
-    _startStreamProcessor: {skip: isAnInternalCommand},
+    streams_startStreamProcessor: {skip: isAnInternalCommand},
+    streams_startStreamSample: {skip: isAnInternalCommand},
+    streams_stopStreamProcessor: {skip: isAnInternalCommand},
+    streams_listStreamProcessors: {skip: isAnInternalCommand},
+    streams_getMoreStreamSample: {skip: isAnInternalCommand},
+    streams_getStats: {skip: isAnInternalCommand},
+    streams_testOnlyInsert: {skip: isAnInternalCommand},
+    streams_getMetrics: {skip: isAnInternalCommand},
+    streams_updateFeatureFlags: {skip: isAnInternalCommand},
+    streams_testOnlyGetFeatureFlags: {skip: isAnInternalCommand},
+    streams_writeCheckpoint: {skip: isAnInternalCommand},
     _transferMods: {skip: isAnInternalCommand},
     _vectorClockPersist: {skip: isAnInternalCommand},
+    abortMoveCollection: {skip: isUnrelated},
     abortReshardCollection: {skip: isUnrelated},
     abortTransaction: {skip: isUnrelated},
+    abortUnshardCollection: {skip: isUnrelated},
     addShard: {skip: isUnrelated},
     addShardToZone: {skip: isUnrelated},
     aggregate: {command: {aggregate: "view", pipeline: [{$match: {}}], cursor: {}}},
@@ -233,6 +251,7 @@ let viewsCommandTests = {
         skipSharded: true,
     },
     authenticate: {skip: isUnrelated},
+    autoCompact: {skip: isUnrelated},
     autoSplitVector: {
         command: {
             splitVector: "test.view",
@@ -243,9 +262,6 @@ let viewsCommandTests = {
     },
     balancerCollectionStatus: {
         command: {balancerCollectionStatus: "test.view"},
-        setup: function(conn) {
-            assert.commandWorked(conn.adminCommand({enableSharding: "test"}));
-        },
         skipStandalone: true,
         expectFailure: true,
         isAdminCommand: true,
@@ -260,12 +276,17 @@ let viewsCommandTests = {
         command: {captrunc: "view", n: 2, inc: false},
         expectFailure: true,
     },
-    checkMetadataConsistency: {skip: isUnrelated},
+    changePrimary: {skip: "Tested in sharding/change_primary.js"},
+    checkMetadataConsistency: {
+        command: {checkMetadataConsistency: "view"},
+        expectFailure: false,
+    },
     checkShardingIndex: {skip: isUnrelated},
     cleanupOrphaned: {
         skip: "Tested in views/views_sharded.js",
     },
     cleanupReshardCollection: {skip: isUnrelated},
+    cleanupStructuredEncryptionData: {skip: isUnrelated},
     clearJumboFlag: {
         command: {clearJumboFlag: "test.view"},
         skipStandalone: true,
@@ -280,6 +301,7 @@ let viewsCommandTests = {
     },
     clusterAbortTransaction: {skip: "already tested by 'abortTransaction' tests on mongos"},
     clusterAggregate: {skip: "already tested by 'aggregate' tests on mongos"},
+    clusterBulkWrite: {skip: "already tested by 'bulkWrite' tests on mongos"},
     clusterCommitTransaction: {skip: "already tested by 'commitTransaction' tests on mongos"},
     clusterCount: {skip: "already tested by 'count' tests on mongos"},
     clusterDelete: {skip: "already tested by 'delete' tests on mongos"},
@@ -322,8 +344,8 @@ let viewsCommandTests = {
             assert.commandWorked(conn.runCommand({dropAllRolesFromDatabase: 1}));
         }
     },
-    createSearchIndex: {skip: "present in v6.3 but renamed to createSearchIndexes in v7.0"},
     createSearchIndexes: {skip: isUnrelated},
+    createUnsplittableCollection: {skip: isUnrelated},
     createUser: {
         command: {createUser: "testuser", pwd: "testpass", roles: []},
         setup: function(conn) {
@@ -362,12 +384,8 @@ let viewsCommandTests = {
     delete: {command: {delete: "view", deletes: [{q: {x: 1}, limit: 1}]}, expectFailure: true},
     distinct: {command: {distinct: "view", key: "_id"}},
     donorAbortMigration: {skip: isUnrelated},
-    // TODO : remove overrides once possible SERVER-61845
-    donorAbortSplit: {skip: "has been removed from the server"},
     donorForgetMigration: {skip: isUnrelated},
-    donorForgetSplit: {skip: "has been removed from the server"},
     donorStartMigration: {skip: isUnrelated},
-    donorStartSplit: {skip: "has been removed from the server"},
     donorWaitForMigrationToCommit: {skip: isUnrelated},
     abortShardSplit: {skip: isUnrelated},
     commitShardSplit: {skip: isUnrelated},
@@ -456,12 +474,14 @@ let viewsCommandTests = {
     getShardMap: {skip: isUnrelated},
     getShardVersion: {
         command: {getShardVersion: "test.view"},
-        expectFailure: true,
-        expectedErrorCode: ErrorCodes.NoShardingEnabled,
+        // This command is only expected to fail with the errors below when it is run against
+        // a standalone replica set mongod.
+        expectFailure: !TestData.testingReplicaSetEndpoint,
+        expectedErrorCode:
+            [ErrorCodes.ShardingStateNotInitialized, ErrorCodes.NoShardingEnabled_OBSOLETE],
         isAdminCommand: true,
         skipSharded: true,  // mongos is tested in views/views_sharded.js
     },
-    getnonce: {skip: "removed in v6.3"},
     godinsert: {skip: isAnInternalCommand},
     grantPrivilegesToRole: {skip: "tested in auth/commands_user_defined_roles.js"},
     grantRolesToRole: {skip: isUnrelated},
@@ -547,6 +567,13 @@ let viewsCommandTests = {
         expectFailure: true,
         expectedErrorCode: ErrorCodes.NamespaceNotSharded,
     },
+    moveCollection: {
+        command: {moveCollection: "test.view", toShard: "move_collection-rs"},
+        expectedErrorCode: [ErrorCodes.NamespaceNotFound, ErrorCodes.IllegalOperation],
+        skipStandalone: true,
+        expectFailure: true,
+        isAdminCommand: true,
+    },
     movePrimary: {skip: "Tested in sharding/movePrimary1.js"},
     moveRange: {skip: isUnrelated},
     multicast: {skip: isUnrelated},
@@ -617,10 +644,7 @@ let viewsCommandTests = {
     resetPlacementHistory: {skip: isUnrelated},
     reshardCollection: {
         command: {reshardCollection: "test.view", key: {_id: 1}},
-        setup: function(conn) {
-            assert.commandWorked(conn.adminCommand({enableSharding: "test"}));
-        },
-        expectedErrorCode: ErrorCodes.NamespaceNotSharded,
+        expectedErrorCode: [ErrorCodes.NamespaceNotSharded, ErrorCodes.NamespaceNotFound],
         skipStandalone: true,
         expectFailure: true,
         isAdminCommand: true,
@@ -642,9 +666,6 @@ let viewsCommandTests = {
     revokeRolesFromUser: {skip: isUnrelated},
     setAllowMigrations: {
         command: {setAllowMigrations: "test.view", allowMigrations: false},
-        setup: function(conn) {
-            assert.commandWorked(conn.adminCommand({enableSharding: "test"}));
-        },
         expectedErrorCode: ErrorCodes.NamespaceNotSharded,
         skipStandalone: true,
         expectFailure: true,
@@ -652,6 +673,7 @@ let viewsCommandTests = {
     },
     rolesInfo: {skip: isUnrelated},
     rotateCertificates: {skip: isUnrelated},
+    rotateFTDC: {skip: isUnrelated},
     saslContinue: {skip: isUnrelated},
     saslStart: {skip: isUnrelated},
     sbe: {skip: isAnInternalCommand},
@@ -666,12 +688,11 @@ let viewsCommandTests = {
     setShardVersion: {skip: isUnrelated},
     setChangeStreamState: {skip: isUnrelated},
     setClusterParameter: {skip: isUnrelated},
+    setQuerySettings: {skip: isUnrelated},
+    removeQuerySettings: {skip: isUnrelated},
     setUserWriteBlockMode: {skip: isUnrelated},
     shardCollection: {
         command: {shardCollection: "test.view", key: {_id: 1}},
-        setup: function(conn) {
-            assert.commandWorked(conn.adminCommand({enableSharding: "test"}));
-        },
         skipStandalone: true,
         expectFailure: true,
         isAdminCommand: true,
@@ -698,7 +719,8 @@ let viewsCommandTests = {
         },
         skipSharded: true,
         expectFailure: true,
-        expectedErrorCode: ErrorCodes.NoShardingEnabled,
+        expectedErrorCode:
+            [ErrorCodes.ShardingStateNotInitialized, ErrorCodes.NoShardingEnabled_OBSOLETE],
         isAdminCommand: true,
     },
     splitVector: {
@@ -713,6 +735,7 @@ let viewsCommandTests = {
     startRecordingTraffic: {skip: isUnrelated},
     startSession: {skip: isAnInternalCommand},
     stopRecordingTraffic: {skip: isUnrelated},
+    sysprofile: {skip: isAnInternalCommand},
     testDeprecation: {skip: isAnInternalCommand},
     testDeprecationInVersion2: {skip: isAnInternalCommand},
     testInternalTransactions: {skip: isAnInternalCommand},
@@ -720,9 +743,24 @@ let viewsCommandTests = {
     testReshardCloneCollection: {skip: isAnInternalCommand},
     testVersion2: {skip: isAnInternalCommand},
     testVersions1And2: {skip: isAnInternalCommand},
+    timeseriesCatalogBucketParamsChanged: {skip: isAnInternalCommand},
     top: {skip: "tested in views/views_stats.js"},
     transitionFromDedicatedConfigServer: {skip: isUnrelated},
     transitionToDedicatedConfigServer: {skip: isUnrelated},
+    transitionToShardedCluster: {skip: isUnrelated},
+    unshardCollection: {
+        command: {unshardCollection: "test.view", toShard: "unshard_collection-rs"},
+        expectedErrorCode: [ErrorCodes.NamespaceNotSharded, ErrorCodes.NamespaceNotFound],
+        skipStandalone: true,
+        expectFailure: true,
+        isAdminCommand: true,
+    },
+    untrackUnshardedCollection: {
+        command: {untrackUnshardedCollection: "test.view"},
+        skipStandalone: true,
+        expectFailure: false,
+        isAdminCommand: true,
+    },
     update: {command: {update: "view", updates: [{q: {x: 1}, u: {x: 2}}]}, expectFailure: true},
     updateRole: {
         command: {
@@ -744,13 +782,13 @@ let viewsCommandTests = {
     validate: {command: {validate: "view"}, expectFailure: true},
     validateDBMetadata:
         {command: {validateDBMetadata: 1, apiParameters: {version: "1", strict: true}}},
-    waitForOngoingChunkSplits: {skip: isUnrelated},
     voteAbortIndexBuild: {skip: isUnrelated},
     voteCommitImportCollection: {skip: isUnrelated},
     voteCommitIndexBuild: {skip: isUnrelated},
     voteCommitTransaction: {skip: isUnrelated},
     voteAbortTransaction: {skip: isUnrelated},
     waitForFailPoint: {skip: isUnrelated},
+    getShardingReady: {skip: isAnInternalCommand},
     whatsmyuri: {skip: isUnrelated},
     whatsmysni: {skip: isUnrelated}
 };
@@ -852,4 +890,3 @@ for (let command of commands) {
             subtest.teardown(dbHandle);
     }
 }
-}());
