@@ -2,9 +2,6 @@
 // @tags: [
 //   requires_sharding,
 // ]
-(function() {
-"use strict";
-
 const coll = db.update_with_large_hint;
 coll.drop();
 
@@ -32,5 +29,4 @@ assert.commandFailedWithCode(coll.runCommand("update", {
 assert.commandFailedWithCode(coll.runCommand("delete", {
     deletes: [{q: {_id: 0}, limit: 1, hint: {[longHint]: 1}}],
 }),
-                             ErrorCodes.BadValue);
-}());
\ No newline at end of file
+                             ErrorCodes.BadValue);
\ No newline at end of file
