@@ -1,8 +1,5 @@
 // This test verifies writeConcern behavior on a standalone mongod.
 
-(function() {
-'use strict';
-
 var col = db.write_concern;
 col.drop();
 
@@ -12,5 +9,4 @@ assert.commandWorked(col.insert({_id: 1}, {writeConcern: {w: 1}}));
 assert.commandWorked(col.insert({_id: "majority"}, {writeConcern: {w: "majority"}}));
 
 // writeConcern: 2 should not work on standalone
-assert.writeError(col.insert({_id: 2}, {writeConcern: {w: 2}}), "expected writeConcern: 2 to fail");
-})();
+assert.writeError(col.insert({_id: 2}, {writeConcern: {w: 2}}), "expected writeConcern: 2 to fail");
\ No newline at end of file
