@@ -8,13 +8,10 @@
  * requires_fcv_70
  * ]
  */
-load("jstests/fle2/libs/encrypted_client_util.js");
-
-(function() {
-'use strict';
+import {isFLE2ReplicationEnabled} from "jstests/fle2/libs/encrypted_client_util.js";
 
 if (!isFLE2ReplicationEnabled()) {
-    return;
+    quit();
 }
 
 const dbTest = db.getSiblingDB('convert_encrypted_to_capped_db');
@@ -42,4 +39,3 @@ assert.commandFailedWithCode(
     dbTest.runCommand({cloneCollectionAsCapped: "basic", toCollection: "capped", size: 100000}),
     6367302,
     "Clone encrypted collection as capped passed");
-}());
