@@ -6,11 +6,6 @@
  * requires_fcv_70
  * ]
  */
-load("jstests/fle2/libs/encrypted_client_util.js");
-
-(function() {
-'use strict';
-
 let dbTest = db.getSiblingDB('create_encrypted_collection_db');
 
 dbTest.dropDatabase();
@@ -85,5 +80,4 @@ assert.commandWorked(dbTest.createCollection("basic_int32_cf", {
             "queries": {"queryType": "equality", contention: NumberInt(123)}
         }]
     }
-}));
-}());
+}));
\ No newline at end of file
