@@ -5,14 +5,16 @@
  * requires_fcv_70
  * ]
  */
-load("jstests/fle2/libs/encrypted_client_util.js");
-
-(function() {
-'use strict';
+import {isMongos} from "jstests/concurrency/fsm_workload_helpers/server_types.js";
+import {EncryptedClient, isEnterpriseShell} from "jstests/fle2/libs/encrypted_client_util.js";
 
 // Passthrough workaround
 if (!isMongos(db)) {
-    return;
+    quit();
+}
+
+if (!isEnterpriseShell()) {
+    quit();
 }
 
 let dbName = 'shard_state';
@@ -38,4 +40,3 @@ assert.commandFailedWithCode(
     db.adminCommand({shardCollection: 'shard_state.enxcol_.basic.ecc', key: {_id: 1}}), 6464401);
 assert.commandFailedWithCode(
     db.adminCommand({shardCollection: 'shard_state.enxcol_.basic.ecoc', key: {_id: 1}}), 6464401);
-}());
