@@ -1,16 +1,15 @@
-'use strict';
-
-(function() {
-load('jstests/libs/discover_topology.js');  // For Topology and DiscoverTopology.
-load('jstests/libs/fixture_helpers.js');
+import {DiscoverTopology, Topology} from "jstests/libs/discover_topology.js";
+import {getFailPointName} from "jstests/libs/fail_point_util.js";
+import {FixtureHelpers} from "jstests/libs/fixture_helpers.js";
 
 function cleanupHelloFailInjection(connection) {
     jsTestLog(`Cleanup Hello fail injection in ${connection}`);
     let adminDB = connection.getDB('admin');
-    assert.commandWorked(adminDB.runCommand({configureFailPoint: 'waitInHello', mode: "off"}));
-    const res =
-        assert.commandWorked(adminDB.runCommand({getParameter: 1, "failpoint.waitInHello": 1}));
-    assert.eq(res["failpoint.waitInHello"].mode, 0);
+    const fpName = getFailPointName("shardWaitInHello", connection.getMaxWireVersion());
+    assert.commandWorked(adminDB.runCommand({configureFailPoint: fpName, mode: "off"}));
+    const prefixedName = `failpoint.${fpName}`;
+    const res = assert.commandWorked(adminDB.runCommand({getParameter: 1, [prefixedName]: 1}));
+    assert.eq(res[`failpoint.${fpName}`].mode, 0);
 }
 
 function doFailInjectionCleanup(db) {
@@ -32,4 +31,3 @@ if (topology.type === Topology.kShardedCluster) {
     doFailInjectionCleanup(db);
 }
 jsTestLog(`Hello fail hook completed`);
-})();
