@@ -9,9 +9,6 @@
 // The test confirms this behavior for 1-node config servers, SCCC config servers and
 // CSRS config servers.
 
-(function() {
-"use strict";
-
 function getHostPart(hostAndPort) {
     return hostAndPort.substr(0, hostAndPort.lastIndexOf(':'));
 }
@@ -25,5 +22,4 @@ assert.commandWorked(c1.adminCommand("replSetInitiate"));
 c2 = MongoRunner.runMongod({configsvr: "", storageEngine: "wiredTiger"});
 c3 = MongoRunner.runMongod({configsvr: "", storageEngine: "wiredTiger"});
 
-assert(MongoRunner.runMongos({configdb: "csrs/" + getHostPart(c1.host)}));
-}());
+assert(MongoRunner.runMongos({configdb: "csrs/" + getHostPart(c1.host)}));
\ No newline at end of file
