@@ -8,7 +8,7 @@
  * command.
  * testCase.expectedErrorCode - an expected error do be produced by the "aggregate" command.
  */
-function executeAggregationTestCase(collection, testCase) {
+export function executeAggregationTestCase(collection, testCase) {
     jsTestLog(tojson(testCase));
     assert.commandWorked(collection.remove({}));
 
@@ -18,13 +18,15 @@ function executeAggregationTestCase(collection, testCase) {
     // Issue an aggregate command and verify the result.
     try {
         const actualResults = collection.aggregate(testCase.pipeline).toArray();
-        assert(testCase.expectedErrorCode === undefined,
-               `Expected an exception with code ${testCase.expectedErrorCode}`);
-        assert.docEq(actualResults, testCase.expectedResults);
+        if (testCase.expectedResults === undefined) {
+            assert(testCase.expectedErrorCode === undefined,
+                   `Expected an exception with code ${testCase.expectedErrorCode}`);
+        }
+        assert.docEq(testCase.expectedResults, actualResults);
     } catch (error) {
         if (testCase.expectedErrorCode === undefined) {
             throw error;
         }
         assert.commandFailedWithCode(error, testCase.expectedErrorCode);
     }
-}
\ No newline at end of file
+}
