@@ -2,12 +2,17 @@
  * Helper functions that are used in change streams rewrite test cases.
  */
 
-load("jstests/libs/collection_drop_recreate.js");  // For assertDropAndRecreateCollection.
-load("jstests/libs/fixture_helpers.js");           // For isMongos.
+import {
+    assertCreateCollection,
+    assertDropAndRecreateCollection,
+    assertDropCollection,
+} from "jstests/libs/collection_drop_recreate.js";
+import {FixtureHelpers} from "jstests/libs/fixture_helpers.js";
 
 // Function which generates a write workload on the specified collection, including all events that
 // a change stream may consume. Assumes that the specified collection does not already exist.
-function generateChangeStreamWriteWorkload(db, collName, numDocs, includInvalidatingEvents = true) {
+export function generateChangeStreamWriteWorkload(
+    db, collName, numDocs, includInvalidatingEvents = true) {
     // If this is a sharded passthrough, make sure we shard on something other than _id so that a
     // non-id field appears in the documentKey. This will generate 'create' and 'shardCollection'.
     if (FixtureHelpers.isMongos(db)) {
@@ -99,7 +104,8 @@ function generateChangeStreamWriteWorkload(db, collName, numDocs, includInvalida
 
 // Helper function to fully exhaust a change stream from the specified point and return all events.
 // Assumes that all relevant events can fit into a single 16MB batch.
-function getAllChangeStreamEvents(db, extraPipelineStages = [], csOptions = {}, resumeToken) {
+export function getAllChangeStreamEvents(
+    db, extraPipelineStages = [], csOptions = {}, resumeToken) {
     // Open a whole-cluster stream based on the supplied arguments.
     const csCursor = db.getMongo().watch(
         extraPipelineStages,
@@ -121,39 +127,56 @@ function getAllChangeStreamEvents(db, extraPipelineStages = [], csOptions = {},
 }
 
 // Helper function to check whether this value is a plain old javascript object.
-function isPlainObject(value) {
+export function isPlainObject(value) {
     return (value && typeof (value) == "object" && value.constructor === Object);
 }
 
 // Verifies the number of change streams events returned from a particular shard.
-function assertNumChangeStreamDocsReturnedFromShard(stats, shardName, expectedTotalReturned) {
+export function assertNumChangeStreamDocsReturnedFromShard(
+    stats, shardName, expectedTotalReturned) {
     assert(stats.shards.hasOwnProperty(shardName), stats);
     const stages = stats.shards[shardName].stages;
     const lastStage = stages[stages.length - 1];
     assert.eq(lastStage.nReturned, expectedTotalReturned, stages);
 }
 
-// Verifies the number of oplog events read by a particular shard.
-function assertNumMatchingOplogEventsForShard(stats, shardName, expectedTotalReturned) {
+export function getExecutionStatsForShard(stats, shardName) {
     assert(stats.shards.hasOwnProperty(shardName), stats);
     assert.eq(Object.keys(stats.shards[shardName].stages[0])[0], "$cursor", stats);
-    const executionStats = stats.shards[shardName].stages[0].$cursor.executionStats;
-    assert.eq(executionStats.nReturned,
+    return stats.shards[shardName].stages[0].$cursor.executionStats;
+}
+
+export function getUnwindStageForShard(stats, shardName) {
+    assert(stats.shards.hasOwnProperty(shardName), stats);
+    const stages = stats.shards[shardName].stages;
+    assert.eq(stages[1].$changeStream.stage, "internalUnwindTransaction");
+    return stages[1];
+}
+
+// Verifies the number of oplog events read by a particular shard.
+export function assertNumMatchingOplogEventsForShard(stats, shardName, expectedTotalReturned) {
+    const executionStats = getExecutionStatsForShard(stats, shardName);
+    // We verify the number of documents from the unwind stage instead of the oplog cursor, so we
+    // are testing that the filter is applied to the output of batched oplog entries as well.
+    const unwindStage = getUnwindStageForShard(stats, shardName);
+    assert.eq(unwindStage.nReturned,
               expectedTotalReturned,
               () => `Expected ${expectedTotalReturned} events on shard ${shardName} but got ` +
-                  `${executionStats.nReturned}. Execution stats:\n${tojson(executionStats)}`);
+                  `${executionStats.nReturned}. Execution stats:\n${tojson(executionStats)}\n` +
+                  `Unwind stage:\n${tojson(unwindStage)}`);
 }
 
 // Returns a newly created sharded collection sharded by caller provided shard key.
-function createShardedCollection(shardingTest, shardKey, dbName, collName, splitAt) {
+export function createShardedCollection(shardingTest, shardKey, dbName, collName, splitAt) {
+    assert.commandWorked(shardingTest.s.adminCommand(
+        {enableSharding: dbName, primaryShard: shardingTest.shard0.name}));
+
     const db = shardingTest.s.getDB(dbName);
     assertDropAndRecreateCollection(db, collName);
 
     const coll = db.getCollection(collName);
     assert.commandWorked(coll.createIndex({[shardKey]: 1}));
 
-    shardingTest.ensurePrimaryShard(dbName, shardingTest.shard0.shardName);
-
     // Shard the test collection and split it into two chunks: one that contains all {shardKey: <lt
     // splitAt>} documents and one that contains all {shardKey: <gte splitAt>} documents.
     shardingTest.shardColl(
@@ -173,7 +196,7 @@ function createShardedCollection(shardingTest, shardKey, dbName, collName, split
 // 2. There are no additional events being returned other than the ones in the 'expectedResult'.
 // 3. the filtering is been done at oplog level, and each of the shard read only the
 // 'expectedOplogNReturnedPerShard' documents.
-function verifyChangeStreamOnWholeCluster({
+export function verifyChangeStreamOnWholeCluster({
     st,
     changeStreamSpec,
     userMatchExpr,
