@@ -2,13 +2,12 @@
  * A class with helper functions which operate on change streams. The class maintains a list of
  * opened cursors and kills them on cleanup.
  */
-
-load("jstests/libs/fixture_helpers.js");  // For FixtureHelpers.
+import {getCollectionNameFromFullNamespace} from "jstests/libs/namespace_utils.js";
 
 /**
  * Enumeration of the possible types of change streams.
  */
-const ChangeStreamWatchMode = Object.freeze({
+export const ChangeStreamWatchMode = Object.freeze({
     kCollection: 1,
     kDb: 2,
     kCluster: 3,
@@ -17,29 +16,20 @@ const ChangeStreamWatchMode = Object.freeze({
 /**
  * Returns a truncated json object if the size of 'jsonObj' is greater than 'maxErrorSizeBytes'.
  */
-function tojsonMaybeTruncate(jsonObj) {
+export function tojsonMaybeTruncate(jsonObj) {
     // Maximum size for the json object.
     const maxErrorSizeBytes = 1024 * 1024;
 
     return tojson(jsonObj).substring(0, maxErrorSizeBytes);
 }
 
-/**
- * Returns true if feature flag 'featureFlagChangeStreamsRewrite' is enabled, false otherwise.
- */
-function isChangeStreamsRewriteEnabled(db) {
-    const getParam = db.adminCommand({getParameter: 1, featureFlagChangeStreamsRewrite: 1});
-    return getParam.hasOwnProperty("featureFlagChangeStreamsRewrite") &&
-        getParam.featureFlagChangeStreamsRewrite.value;
-}
-
 /**
  * Helper function used internally by ChangeStreamTest. If no passthrough is active, it is exactly
  * the same as calling db.runCommand. If a passthrough is active and has defined a function
  * 'changeStreamPassthroughAwareRunCommand', then this method will be overridden to allow individual
  * streams to explicitly exempt themselves from being modified by the passthrough.
  */
-function isChangeStreamPassthrough() {
+export function isChangeStreamPassthrough() {
     return typeof changeStreamPassthroughAwareRunCommand != 'undefined';
 }
 
@@ -47,15 +37,19 @@ function isChangeStreamPassthrough() {
  * Helper function to retrieve the type of change stream based on the passthrough suite in which the
  * test is being run. If no passthrough is active, this method returns the kCollection watch mode.
  */
-function changeStreamPassthroughType() {
-    return typeof ChangeStreamPassthroughHelpers === 'undefined'
+export function changeStreamPassthroughType() {
+    return typeof globalThis.ChangeStreamPassthroughHelpers === 'undefined'
         ? ChangeStreamWatchMode.kCollection
-        : ChangeStreamPassthroughHelpers.passthroughType();
+        : globalThis.ChangeStreamPassthroughHelpers.passthroughType();
 }
 
-const runCommandChangeStreamPassthroughAware =
-    (!isChangeStreamPassthrough() ? ((db, cmdObj) => db.runCommand(cmdObj))
-                                  : changeStreamPassthroughAwareRunCommand);
+export function runCommandChangeStreamPassthroughAware(db, cmdObj, noPassthrough) {
+    if (!isChangeStreamPassthrough()) {
+        return db.runCommand(cmdObj);
+    }
+
+    return globalThis.changeStreamPassthroughAwareRunCommand(db, cmdObj, noPassthrough);
+}
 
 /**
  * Asserts that the given opType triggers an invalidate entry depending on the type of change
@@ -65,7 +59,7 @@ const runCommandChangeStreamPassthroughAware =
  *     - whole cluster streams: none.
  * Returns the invalidate document if there was one, or null otherwise.
  */
-function assertInvalidateOp({cursor, opType}) {
+export function assertInvalidateOp({cursor, opType}) {
     if (!isChangeStreamPassthrough() ||
         (changeStreamPassthroughType() == ChangeStreamWatchMode.kDb && opType == "dropDatabase")) {
         assert.soon(() => cursor.hasNext());
@@ -79,14 +73,15 @@ function assertInvalidateOp({cursor, opType}) {
     return null;
 }
 
-function canonicalizeEventForTesting(event, expected) {
+export function canonicalizeEventForTesting(event, expected) {
     for (let fieldName of ["_id",
                            "clusterTime",
                            "txnNumber",
                            "lsid",
                            "collectionUUID",
                            "wallTime",
-                           "operationDescription"]) {
+                           "operationDescription",
+                           "nsType"]) {
         if (!expected.hasOwnProperty(fieldName)) {
             delete event[fieldName];
         }
@@ -117,8 +112,9 @@ function runInFixture(callback) {
  * the resume token, clusterTime, and other unknowable fields unless they are explicitly listed in
  * the expected event.
  */
-function isChangeStreamEventEq(actualEvent, expectedEvent) {
-    const testEvent = canonicalizeEventForTesting(Object.assign({}, actualEvent), expectedEvent);
+function isChangeStreamEventEq(
+    actualEvent, expectedEvent, eventModifier = canonicalizeEventForTesting) {
+    const testEvent = eventModifier(Object.assign({}, actualEvent), expectedEvent);
     return friendlyEqual(sortDoc(testEvent), sortDoc(expectedEvent));
 }
 
@@ -126,8 +122,8 @@ function isChangeStreamEventEq(actualEvent, expectedEvent) {
  * Helper to check whether a change event matches the given expected event. Throws assertion failure
  * if change events do not match.
  */
-function assertChangeStreamEventEq(actualEvent, expectedEvent) {
-    assert(isChangeStreamEventEq(actualEvent, expectedEvent),
+export function assertChangeStreamEventEq(actualEvent, expectedEvent, eventModifier) {
+    assert(isChangeStreamEventEq(actualEvent, expectedEvent, eventModifier),
            () => "Change events did not match. Expected: " + tojsonMaybeTruncate(expectedEvent) +
                ", Actual: " + tojsonMaybeTruncate(actualEvent));
 }
@@ -135,12 +131,12 @@ function assertChangeStreamEventEq(actualEvent, expectedEvent) {
 /**
  * Helper to check that there are no transaction operations past end of transaction event.
  */
-function assertEndOfTransaction(changes) {
+export function assertEndOfTransaction(changes) {
     const finishedTransactions = new Set();
     for (const change of changes) {
         assert.eq(change.hasOwnProperty("lsid"),
                   change.hasOwnProperty("txnNumber"),
-                  "Found change event with inconsistent lsid and txnNumber: " + tojson(change));
+                  "Found change eventy with inconsistent lsid and txnNumber: " + tojson(change));
         if (change.hasOwnProperty("lsid")) {
             const txnId = tojsononeline({lsid: change.lsid, txnNumber: change.txnNumber});
             if (change.operationType == "endOfTransaction") {
@@ -158,24 +154,22 @@ function isResumableChangeStreamError(error) {
     return "errorLabels" in error && error.errorLabels.includes("ResumableChangeStreamError");
 }
 
-function ChangeStreamTest(_db, options) {
-    load("jstests/libs/namespace_utils.js");  // For getCollectionNameFromFullNamespace.
-
+export function ChangeStreamTest(_db, options) {
     // Keeps track of cursors opened during the test so that we can be sure to
     // clean them up before the test completes.
     let _allCursors = [];
     // Map to store cursor-specific data
-    let _cursorData = {};
+    let _cursorData = new Map();
     let self = this;
     options = options || {};
     const eventModifier = options.eventModifier || canonicalizeEventForTesting;
 
     function updateResumeToken(cursor, changeEvents) {
         const cursorId = String(cursor.id);
-        if (!_cursorData[cursorId]) {
-            _cursorData[cursorId] = {};
+        if (!_cursorData.has(cursorId)) {
+            _cursorData.set(cursorId, {});
         }
-        const cursorInfo = _cursorData[cursorId];
+        const cursorInfo = _cursorData.get(cursorId);
 
         if (changeEvents && changeEvents.length > 0) {
             cursorInfo.resumeToken =
@@ -228,11 +222,11 @@ function ChangeStreamTest(_db, options) {
                 }
             }
             assert.neq(res.cursor.id, 0);
-            _cursorData[String(res.cursor.id)] = {
+            _cursorData.set(String(res.cursor.id), {
                 pipeline: pipeline,
                 collName: collName,
                 doNotModifyInPassthroughs: doNotModifyInPassthroughs
-            };
+            });
             updateResumeToken(res.cursor, res.cursor.firstBatch);
             _allCursors.push({db: _db.getName(), coll: collName, cursorId: res.cursor.id});
             return res.cursor;
@@ -255,7 +249,7 @@ function ChangeStreamTest(_db, options) {
 
     self.restartChangeStream = function(cursor) {
         const cursorId = String(cursor.id);
-        const cursorInfo = _cursorData[cursorId];
+        const cursorInfo = _cursorData.get(cursorId);
         if (!cursorInfo || !cursorInfo.resumeToken) {
             throw new Error("Cannot resume change stream - no resume token available for cursor");
         }
@@ -334,8 +328,9 @@ function ChangeStreamTest(_db, options) {
                     assert.neq(
                         cursor.id,
                         NumberLong(0),
-                        "Cursor has been closed unexpectedly. Observed change stream events: " +
-                            tojsonMaybeTruncate(changes));
+                        () => (
+                            "Cursor has been closed unexpectedly. Observed change stream events: " +
+                            tojsonMaybeTruncate(changes)));
                     cursor = self.getNextBatch(cursor);
                     changes[i] = getNextDocFromCursor(cursor);
                     return changes[i] !== null;
@@ -365,8 +360,8 @@ function ChangeStreamTest(_db, options) {
     function assertChangeIsExpected(
         expectedChanges, numChangesSeen, observedChanges, expectInvalidate) {
         if (expectedChanges) {
-            assertChangeStreamEventEq(observedChanges[numChangesSeen],
-                                      expectedChanges[numChangesSeen]);
+            assertChangeStreamEventEq(
+                observedChanges[numChangesSeen], expectedChanges[numChangesSeen], eventModifier);
         } else if (!expectInvalidate) {
             assert(!isInvalidated(observedChanges[numChangesSeen]),
                    "Change was invalidated when it should not have been. Number of changes seen: " +
@@ -422,7 +417,7 @@ function ChangeStreamTest(_db, options) {
             assert.eq(changes.length, expectedNumChanges, errMsgFunc);
             for (let i = 0; i < changes.length; i++) {
                 assert(expectedChanges.some(expectedChange => {
-                    return isChangeStreamEventEq(changes[i], expectedChange);
+                    return isChangeStreamEventEq(changes[i], expectedChange, eventModifier);
                 }),
                        errMsgFunc);
             }
@@ -474,6 +469,7 @@ function ChangeStreamTest(_db, options) {
         cursor = self.getNextBatch(cursor);
         assert.eq(
             0, cursor.nextBatch.length, () => "Cursor had changes: " + tojsonMaybeTruncate(cursor));
+        return cursor;
     };
 
     /**
@@ -481,7 +477,7 @@ function ChangeStreamTest(_db, options) {
      * If the current batch has a document in it, that one will be ignored.
      */
     self.getOneChange = function(cursor, expectInvalidate = false) {
-        changes = self.getNextChanges(cursor, 1, true);
+        const changes = self.getNextChanges(cursor, 1, true);
 
         if (expectInvalidate) {
             assert(isInvalidated(changes[changes.length - 1]),
@@ -600,17 +596,19 @@ function ChangeStreamTest(_db, options) {
 ChangeStreamTest.assertChangeStreamThrowsCode = function assertChangeStreamThrowsCode(
     {db, collName, pipeline, expectedCode, doNotModifyInPassthroughs}) {
     try {
-        // Run a passthrough-aware initial 'aggregate' command to open the change stream.
-        const res = assert.commandWorked(runCommandChangeStreamPassthroughAware(
-            db,
-            {aggregate: collName, pipeline: pipeline, cursor: {batchSize: 1}},
-            doNotModifyInPassthroughs));
+        runInFixture(() => {
+            // Run a passthrough-aware initial 'aggregate' command to open the change stream.
+            const res = assert.commandWorked(runCommandChangeStreamPassthroughAware(
+                db,
+                {aggregate: collName, pipeline: pipeline, cursor: {batchSize: 1}},
+                doNotModifyInPassthroughs));
 
-        // Create a cursor using the command response, and begin issuing getMores. We expect
-        // csCursor.hasNext() to throw the expected code before assert.soon() times out.
-        const csCursor = new DBCommandCursor(db, res, 1);
-        assert.soon(() => csCursor.hasNext());
-        assert(false, `Unexpected result from cursor: ${tojsonMaybeTruncate(csCursor.next())}`);
+            // Create a cursor using the command response, and begin issuing getMores. We expect
+            // csCursor.hasNext() to throw the expected code before assert.soon() times out.
+            const csCursor = new DBCommandCursor(db, res, 1);
+            assert.soon(() => csCursor.hasNext());
+            assert(false, `Unexpected result from cursor: ${tojsonMaybeTruncate(csCursor.next())}`);
+        });
     } catch (error) {
         assert.eq(
             error.code, expectedCode, `Caught unexpected error: ${tojsonMaybeTruncate(error)}`);
@@ -633,19 +631,23 @@ ChangeStreamTest.getDBForChangeStream = function(watchMode, dbObj) {
 /**
  * A set of functions to help validate the behaviour of $changeStreams for a given namespace.
  */
-function assertChangeStreamNssBehaviour(dbName, collName = "test", options, assertFunc) {
+export function assertChangeStreamNssBehaviour(dbName, collName = "test", options, assertFunc) {
     const testDb = db.getSiblingDB(dbName);
     options = (options || {});
-    const res = testDb.runCommand(
-        Object.assign({aggregate: collName, pipeline: [{$changeStream: options}], cursor: {}}));
-    return assertFunc(res);
+    return runInFixture(() => {
+        const res = testDb.runCommand(
+            Object.assign({aggregate: collName, pipeline: [{$changeStream: options}], cursor: {}}));
+        return assertFunc(res);
+    });
 }
-function assertValidChangeStreamNss(dbName, collName = "test", options) {
+
+export function assertValidChangeStreamNss(dbName, collName = "test", options) {
     const res = assertChangeStreamNssBehaviour(dbName, collName, options, assert.commandWorked);
     assert.commandWorked(db.getSiblingDB(dbName).runCommand(
         {killCursors: (collName == 1 ? "$cmd.aggregate" : collName), cursors: [res.cursor.id]}));
 }
-function assertInvalidChangeStreamNss(dbName, collName = "test", options) {
+
+export function assertInvalidChangeStreamNss(dbName, collName = "test", options) {
     assertChangeStreamNssBehaviour(
         dbName,
         collName,
@@ -654,14 +656,14 @@ function assertInvalidChangeStreamNss(dbName, collName = "test", options) {
             res, [ErrorCodes.InvalidNamespace, ErrorCodes.InvalidOptions]));
 }
 
-const kPreImagesCollectionDatabase = "config";
-const kPreImagesCollectionName = "system.preimages";
+export const kPreImagesCollectionDatabase = "config";
+export const kPreImagesCollectionName = "system.preimages";
 
 /**
  * Asserts that 'changeStreamPreAndPostImages' collection option is present and is enabled for
  * collection.
  */
-function assertChangeStreamPreAndPostImagesCollectionOptionIsEnabled(db, collName) {
+export function assertChangeStreamPreAndPostImagesCollectionOptionIsEnabled(db, collName) {
     const collectionInfos = db.getCollectionInfos({name: collName});
     assert(collectionInfos[0].options["changeStreamPreAndPostImages"]["enabled"] === true);
 }
@@ -669,17 +671,17 @@ function assertChangeStreamPreAndPostImagesCollectionOptionIsEnabled(db, collNam
 /**
  * Asserts that 'changeStreamPreAndPostImages' collection option is absent in the collection.
  */
-function assertChangeStreamPreAndPostImagesCollectionOptionIsAbsent(db, collName) {
+export function assertChangeStreamPreAndPostImagesCollectionOptionIsAbsent(db, collName) {
     const collectionInfos = db.getCollectionInfos({name: collName});
     assert(!collectionInfos[0].options.hasOwnProperty("changeStreamPreAndPostImages"));
 }
 
-function getPreImagesCollection(connection) {
+export function getPreImagesCollection(connection) {
     return connection.getDB(kPreImagesCollectionDatabase).getCollection(kPreImagesCollectionName);
 }
 
 // Returns the pre-images written while performing the write operations.
-function preImagesForOps(db, writeOps) {
+export function preImagesForOps(db, writeOps) {
     const preImagesColl = getPreImagesCollection(db.getMongo());
     const preImagesCollSortSpec = {"_id.ts": 1, "_id.applyOpsIndex": 1};
 
@@ -711,7 +713,7 @@ function preImagesForOps(db, writeOps) {
  * Returns documents from the pre-images collection from 'connection' ordered by _id.ts,
  * _id.applyOpsIndex ascending.
  */
-function getPreImages(connection) {
+export function getPreImages(connection) {
     return getPreImagesCollection(connection)
         .find()
         .sort({"_id.ts": 1, "_id.applyOpsIndex": 1})
@@ -719,7 +721,7 @@ function getPreImages(connection) {
         .toArray();
 }
 
-function findPreImagesCollectionDescriptions(db) {
+export function findPreImagesCollectionDescriptions(db) {
     return db.getSiblingDB(kPreImagesCollectionDatabase).runCommand("listCollections", {
         filter: {name: kPreImagesCollectionName}
     });
@@ -728,7 +730,7 @@ function findPreImagesCollectionDescriptions(db) {
 /**
  * Asserts that pre-images collection is absent in configDB.
  */
-function assertPreImagesCollectionIsAbsent(db) {
+export function assertPreImagesCollectionIsAbsent(db) {
     const result = findPreImagesCollectionDescriptions(db);
     assert.eq(result.cursor.firstBatch.length, 0);
 }
@@ -736,7 +738,7 @@ function assertPreImagesCollectionIsAbsent(db) {
 /**
  * Asserts that pre-images collection is created in the configDB and has clustered index on _id.
  */
-function assertPreImagesCollectionExists(db) {
+export function assertPreImagesCollectionExists(db) {
     const collectionInfos = findPreImagesCollectionDescriptions(db);
     assert.eq(collectionInfos.cursor.firstBatch.length, 1, collectionInfos);
     const preImagesCollectionDescription = collectionInfos.cursor.firstBatch[0];
