@@ -2,10 +2,7 @@
  * Utilities for testing clustered collections.
  */
 
-load("jstests/libs/analyze_plan.js");
-load("jstests/libs/collection_drop_recreate.js");
-
-var ClusteredCollectionUtil = class {
+export var ClusteredCollectionUtil = class {
     static areAllCollectionsClustered(conn) {
         const res =
             conn.adminCommand({getParameter: 1, "failpoint.clusterAllCollectionsByDefault": 1});
@@ -70,7 +67,7 @@ var ClusteredCollectionUtil = class {
             assert.commandWorked(db.runCommand({listCollections: 1, filter: {name: collName}}));
         const listCollsOptions = listColls.cursor.firstBatch[0].options;
         assert(listCollsOptions.clusteredIndex);
-        assert.docEq(listCollsOptions.clusteredIndex, fullCreateOptions.clusteredIndex);
+        assert.docEq(fullCreateOptions.clusteredIndex, listCollsOptions.clusteredIndex);
     }
 
     // The clusteredIndex should appear in listIndexes with additional "clustered" field.
@@ -84,9 +81,16 @@ var ClusteredCollectionUtil = class {
             createOptions.expireAfterSeconds !== null) {
             extraData.expireAfterSeconds = createOptions.expireAfterSeconds;
         }
-        const expectedListIndexesOutput =
-            Object.extend(extraData, fullCreateOptions.clusteredIndex);
-        assert.docEq(expectedListIndexesOutput, listIndexes.cursor.firstBatch[0]);
+        // TODO (SERVER-85312): Re-enable this after we branch for 8.1 so that multiversion
+        // will test 8.1 <> 8.0.
+        const version = db.version().split('.');
+        if (version[0] >= 8 || (version[0] == 7 && version[1] == 3) ||
+            typeof (extraData.expireAfterSeconds) === 'undefined' ||
+            extraData.expireAfterSeconds === null) {
+            const expectedListIndexesOutput =
+                Object.extend(extraData, fullCreateOptions.clusteredIndex);
+            assert.docEq(expectedListIndexesOutput, listIndexes.cursor.firstBatch[0]);
+        }
     }
 
     static testBasicClusteredCollection(db, collName, clusterKey) {
@@ -109,9 +113,8 @@ var ClusteredCollectionUtil = class {
         for (let len of lengths) {
             let id = 'x'.repeat(len);
 
-            // Validate the below for _id-clustered collection only until the following ticket is
-            // addressed:
-            // * TODO SERVER-60734 replacement updates should preserve the cluster key
+            // Validate the below for _id-clustered collection only given replacement updates only
+            // preserve cluster key '_id'.
             if (clusterKey == "_id") {
                 assert.commandWorked(coll.update({[clusterKey]: id}, {a: len}));
 
@@ -120,8 +123,6 @@ var ClusteredCollectionUtil = class {
             }
         }
 
-        // This section is based on jstests/core/timeseries/clustered_index_crud.js with
-        // specific additions for general-purpose (non-timeseries) clustered collections
         assert.commandWorked(coll.insert({[clusterKey]: 0, a: 1}));
         assert.commandWorked(coll.insert({[clusterKey]: 1, a: 1}));
         assert.eq(1, coll.find({[clusterKey]: 0}).itcount());
@@ -167,9 +168,8 @@ var ClusteredCollectionUtil = class {
         // Look up using the secondary index on {a: 1}
         assert.commandWorked(coll.createIndex({a: 1}));
 
-        // TODO remove the branch once SERVER-60734 "replacement updates should preserve the cluster
-        // key" is resolved.
         if (clusterKey == "_id") {
+            // Replacement updates only preserve the '_id' cluster key.
             assert.eq(1, coll.find({a: null}).itcount());
         } else {
             assert.eq(5, coll.find({a: null}).itcount());
@@ -183,9 +183,8 @@ var ClusteredCollectionUtil = class {
         assert.eq(1, coll.find({a: 10}).itcount());
         assert.eq(99, coll.findOne({a: 10})[clusterKeyString].length);
 
-        // TODO make it unconditional once SERVER-60734 "replacement updates should preserve the
-        // cluster key" is resolved.
         if (clusterKey == "_id") {
+            // Replacement updates only preserve the '_id' cluster key.
             for (let len of lengths) {
                 // Secondary index lookups for documents with large RecordId's.
                 assert.eq(1, coll.find({a: len}).itcount());
