@@ -3,7 +3,7 @@
  * collection did not exist. Avoids automatically recreating the collection in the case of test
  * suites where accessing or dropping the collection implicitly recreates it.
  */
-function assertDropCollection(db, collName) {
+export function assertDropCollection(db, collName) {
     var cmdRes = db.runCommand({drop: collName, writeConcern: {w: "majority"}});
     assert(cmdRes.ok === 1 || cmdRes.code === ErrorCodes.NamespaceNotFound, tojson(cmdRes));
 }
@@ -13,7 +13,7 @@ function assertDropCollection(db, collName) {
  * Returns the newly-created collection on success. When running under a sharded collections
  * passthrough, the new collection will be implicitly sharded.
  */
-function assertCreateCollection(db, collName, collOpts) {
+export function assertCreateCollection(db, collName, collOpts) {
     assert.commandWorked(db.createCollection(collName, collOpts));
     return db.getCollection(collName);
 }
@@ -23,7 +23,7 @@ function assertCreateCollection(db, collName, collOpts) {
  * any. Asserts if either step fails. Returns the newly-created collection on success. When running
  * under a sharded collections passthrough, the new collection will be implicitly sharded.
  */
-function assertDropAndRecreateCollection(db, collName, collOpts) {
+export function assertDropAndRecreateCollection(db, collName, collOpts) {
     assertDropCollection(db, collName);
     return assertCreateCollection(db, collName, collOpts);
 }
