@@ -1,7 +1,7 @@
 /**
  * Asserts that the given collection option is set to true.
  */
-function assertCollectionOptionIsEnabled(db, collName, option) {
+export function assertCollectionOptionIsEnabled(db, collName, option) {
     const collectionInfos = db.getCollectionInfos({name: collName});
     assert(collectionInfos[0].options[option] === true);
 }
@@ -9,7 +9,7 @@ function assertCollectionOptionIsEnabled(db, collName, option) {
 /**
  * Asserts that the given collection option is absent.
  */
-function assertCollectionOptionIsAbsent(db, collName, option) {
+export function assertCollectionOptionIsAbsent(db, collName, option) {
     const collectionInfos = db.getCollectionInfos({name: collName});
     assert(!collectionInfos[0].options.hasOwnProperty(option));
 }
