@@ -1,11 +1,9 @@
-"use strict";
-
-load("jstests/libs/fixture_helpers.js");
+import {FixtureHelpers} from "jstests/libs/fixture_helpers.js";
 
 /**
  * Utilities for feature flags.
  */
-var FeatureFlagUtil = (function() {
+export var FeatureFlagUtil = (function() {
     // A JS attempt at an enum.
     const FlagStatus = {
         kEnabled: 'kEnabled',
@@ -31,14 +29,21 @@ var FeatureFlagUtil = (function() {
         // In order to get an accurate answer for whether a feature flag is enabled, we need to ask
         // a mongod. If db represents a connection to mongos, or some other configuration, we need
         // to obtain the correct connection to a mongod.
-        let conn;
+        let conn = null;
         const setConn = (db) => {
-            if (FixtureHelpers.isMongos(db)) {
-                // For sharded cluster get a connection to the first replicaset
-                conn = new Mongo(FixtureHelpers.getAllReplicas(db)[0].getURL());
-            } else {
+            if (!FixtureHelpers.isMongos(db)) {
                 conn = db;
+                return;
             }
+
+            // For sharded cluster get a connection to the first replicaset through a Mongo
+            // object. We may fail to connect if we are in a stepdown/terminate passthrough, so
+            // retry on retryable errors. After the connection is established, runCommand overrides
+            // should guarantee that subsequent operations on the connection are retried in the
+            // event of network errors in suites where that possibility exists.
+            retryOnRetryableError(() => {
+                conn = new Mongo(FixtureHelpers.getAllReplicas(db)[0].getURL());
+            });
         };
         try {
             setConn(db);
@@ -62,6 +67,9 @@ var FeatureFlagUtil = (function() {
             conn.adminCommand({getParameter: 1, featureCompatibilityVersion: 1}));
         assert(fcvDoc.hasOwnProperty("featureCompatibilityVersion"), fcvDoc);
 
+        assert(!featureFlag.startsWith("featureFlag"),
+               `unexpected prefix in feature flag name: "${featureFlag}". Use "${
+                   featureFlag.replace(/^featureFlag/, '')}" instead.`);
         const fullFlagName = `featureFlag${featureFlag}`;
         const flagDoc = conn.adminCommand({getParameter: 1, [fullFlagName]: 1});
         if (!flagDoc.ok || !flagDoc.hasOwnProperty(fullFlagName)) {
@@ -77,7 +85,9 @@ var FeatureFlagUtil = (function() {
             MongoRunner.compareBinVersions(fcvDoc.featureCompatibilityVersion.version,
                                            flagDoc[fullFlagName].version) >= 0;
 
-        if (flagIsEnabled && (ignoreFCV || flagVersionIsValid)) {
+        const flagShouldBeFCVGated = flagDoc[fullFlagName].shouldBeFCVGated;
+
+        if (flagIsEnabled && (!flagShouldBeFCVGated || ignoreFCV || flagVersionIsValid)) {
             return FlagStatus.kEnabled;
         }
         return FlagStatus.kDisabled;
@@ -91,7 +101,7 @@ var FeatureFlagUtil = (function() {
      *
      * This wrapper checks for 'kEnabled' but raises an error for 'kNotFound' - if the flag is not
      * known. This can be useful if you want to gate an entire test on a feature flag like so:
-     *   if (!FeatureFlagUtils.isEnabled(db, "myFlag")) {
+     *   if (!FeatureFlagUtil.isEnabled(db, "myFlag")) {
      *       jsTestLog("Skipping test because my flag is not enabled");
      *       return;
      *   }
@@ -124,7 +134,7 @@ var FeatureFlagUtil = (function() {
      * conditionally change the assertion being made, like so:
      *
      *   // TODO SERVER-XYZ remove 'featureFlagMyFlag'.
-     *   if (FeatureFlagUtils.isPresentAndEnabled(db, "MyFlag")) {
+     *   if (FeatureFlagUtil.isPresentAndEnabled(db, "MyFlag")) {
      *       // I expect to see some new return value.
      *   } else {
      *       // I expect *not* to see some return value.
@@ -134,7 +144,7 @@ var FeatureFlagUtil = (function() {
      * example above. This is because it is all too easy to have a test like so which will silently
      * stop testing anything if we remove the feature flag without updating the test:
      *
-     *   if (FeatureFlagUtils.isPresentAndEnabled(db, "MyFlag")) {
+     *   if (FeatureFlagUtil.isPresentAndEnabled(db, "MyFlag")) {
      *       // Assert on something new.
      *   }
      *   // No else clause.
