@@ -1,5 +1,5 @@
 // Returns non-localhost ipaddr of host running the mongo shell process
-function get_ipaddr() {
+export function get_ipaddr() {
     // set temp path, if it exists
     var path = "";
     try {
