@@ -1,14 +1,11 @@
 /**
  * Enables causal consistency on the connections.
  */
-(function() {
-"use strict";
+import "jstests/libs/override_methods/set_read_preference_secondary.js";
 
-load("jstests/libs/override_methods/override_helpers.js");
-load('jstests/libs/override_methods/set_read_preference_secondary.js');
+import {OverrideHelpers} from "jstests/libs/override_methods/override_helpers.js";
 
 db.getMongo().setCausalConsistency();
 
 OverrideHelpers.prependOverrideInParallelShell(
     "jstests/libs/override_methods/enable_causal_consistency.js");
-})();
