@@ -4,7 +4,7 @@
  * Reads the collection entry for 'nss' from config.collections, asserts that such an entry exists,
  * and returns its 'uuid' field, which may be undefined.
  */
-function getUUIDFromConfigCollections(mongosConn, nss) {
+export function getUUIDFromConfigCollections(mongosConn, nss) {
     let collEntry = mongosConn.getDB("config").collections.findOne({_id: nss});
     assert.neq(undefined, collEntry);
     return collEntry.uuid;
@@ -14,7 +14,7 @@ function getUUIDFromConfigCollections(mongosConn, nss) {
  * Calls listCollections on a connection to 'db' with a filter for 'collName', asserts that a result
  * for 'collName' was returned, and returns its 'uuid' field, which may be undefined.
  */
-function getUUIDFromListCollections(db, collName) {
+export function getUUIDFromListCollections(db, collName) {
     let listCollsRes = db.runCommand({listCollections: 1, filter: {name: collName}});
     assert.commandWorked(listCollsRes);
     assert.neq(undefined, listCollsRes.cursor);
@@ -29,7 +29,7 @@ function getUUIDFromListCollections(db, collName) {
  * string of the form xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx.
  *
  */
-function extractUUIDFromObject(uuid) {
+export function extractUUIDFromObject(uuid) {
     const uuidString = uuid.toString();
     return uuidString.substring(6, uuidString.length - 2);
-}
\ No newline at end of file
+}
