@@ -7,11 +7,8 @@
 // stepdowns.
 TestData.skipCheckingUUIDsConsistentAcrossCluster = true;
 
-(function() {
-"use strict";
-
-load("jstests/multiVersion/libs/multi_rs.js");       // Used by upgradeSet.
-load("jstests/multiVersion/libs/multi_cluster.js");  // For upgradeCluster.
+await import("jstests/multiVersion/libs/multi_rs.js");
+await import("jstests/multiVersion/libs/multi_cluster.js");
 
 const dbName = "test";
 const collName = "change_streams_multi_version_sortkey";
@@ -34,8 +31,10 @@ function runTest(downgradeVersion) {
     });
 
     let mongosConn = st.s;
+    assert.commandWorked(
+        mongosConn.adminCommand({enableSharding: dbName, primaryShard: st.shard0.shardName}));
+
     assert.commandWorked(mongosConn.getDB(dbName).getCollection(collName).createIndex({shard: 1}));
-    st.ensurePrimaryShard(dbName, st.shard0.shardName);
 
     // Shard the test collection and split it into two chunks: one that contains all {shard: 1}
     // documents and one that contains all {shard: 2} documents.
@@ -84,7 +83,7 @@ function runTest(downgradeVersion) {
     //
     // Upgrade the config db and the shards to the "latest" binVersion.
     //
-    st.upgradeCluster(
+    st.upgradeBinariesWithBackCompat(
         "latest",
         {upgradeShards: true, upgradeConfigs: true, upgradeMongos: false, waitUntilStable: true});
 
@@ -96,9 +95,9 @@ function runTest(downgradeVersion) {
 
     //
     // Upgrade mongos to the "latest" binVersion and then open and read a change stream, this time
-    // with all cluster nodes upgraded but still in downgraded FCV.
+    // with all nodes upgraded but still in downgraded FCV and with upgradeBackCompat enabled.
     //
-    st.upgradeCluster(
+    st.upgradeBinariesWithBackCompat(
         "latest",
         {upgradeShards: false, upgradeConfigs: false, upgradeMongos: true, waitUntilStable: true});
     mongosConn = st.s;
@@ -106,11 +105,21 @@ function runTest(downgradeVersion) {
 
     resumeToken = insertAndValidateChanges(coll, coll.watch([], {resumeAfter: resumeToken}));
 
+    //
+    // Repeat with all nodes without upgradeBackCompat, but FCV still downgraded.
+    //
+    st.restartBinariesWithoutUpgradeBackCompat("latest", {waitUntilStable: true});
+    mongosConn = st.s;
+    coll = mongosConn.getDB(dbName)[collName];
+
+    resumeToken = insertAndValidateChanges(coll, coll.watch([], {resumeAfter: resumeToken}));
+
     //
     // Set the FCV to the "latest" version, and then open and read a change stream on the completely
     // upgraded cluster.
     //
-    assert.commandWorked(mongosConn.adminCommand({setFeatureCompatibilityVersion: latestFCV}));
+    assert.commandWorked(
+        mongosConn.adminCommand({setFeatureCompatibilityVersion: latestFCV, confirm: true}));
     checkFCV(st.configRS.getPrimary().getDB("admin"), latestFCV);
     checkFCV(st.rs0.getPrimary().getDB("admin"), latestFCV);
     checkFCV(st.rs1.getPrimary().getDB("admin"), latestFCV);
@@ -125,4 +134,3 @@ function runTest(downgradeVersion) {
 
 runTest("last-continuous");
 runTest("last-lts");
-}());
