@@ -1,9 +1,6 @@
 /*
  * Tests that an arbiter will always be on the latest FCV regardless of the FCV of the replica set.
  */
-(function() {
-"use strict";
-
 function runTest(FCV) {
     let rst = new ReplSetTest({nodes: [{}, {rsConfig: {arbiterOnly: true}}]});
     rst.startSet();
@@ -31,5 +28,4 @@ runTest(latestFCV);
 runTest(lastLTSFCV);
 if (lastContinuousFCV != lastLTSFCV) {
     runTest(lastContinuousFCV);
-}
-}());
+}
\ No newline at end of file
