@@ -3,8 +3,6 @@
  * prior version nodes can be initialized from the prior version node.
  */
 
-(function() {
-"use strict";
 var name = "initialize_from_old";
 // Test old version with both "last-lts" and "last-continuous".
 for (let oldVersion of ["last-lts", "last-continuous"]) {
@@ -24,5 +22,4 @@ for (let oldVersion of ["last-lts", "last-continuous"]) {
     // Wait for secondaries to finish their initial sync before shutting down the cluster.
     rst.awaitSecondaryNodes();
     rst.stopSet();
-}
-})();
+}
\ No newline at end of file
