@@ -1,7 +1,4 @@
-(function() {
-'use strict';
-
-load("jstests/replsets/libs/rename_across_dbs.js");
+import {RenameAcrossDatabasesTest} from "jstests/replsets/libs/rename_across_dbs.js";
 
 const nodes = [{binVersion: 'latest'}, {binVersion: 'last-lts'}, {}];
 const options = {
@@ -11,4 +8,3 @@ const options = {
 };
 
 new RenameAcrossDatabasesTest(options).run();
-}());
