@@ -39,7 +39,7 @@ ReplSetTest.prototype.upgradeSet = function(options, user, pwd) {
     }
 };
 
-function mergeNodeOptions(nodeOptions, options) {
+export function mergeNodeOptions(nodeOptions, options) {
     for (let nodeName in nodeOptions) {
         nodeOptions[nodeName] = Object.merge(nodeOptions[nodeName], options);
     }
@@ -91,6 +91,12 @@ ReplSetTest.prototype.upgradePrimary = function(primary, options, user, pwd) {
 
     authNode(primary);
 
+    // Can't step down if there is just one node
+    if (this.nodes.length == 1) {
+        this.upgradeNode(primary, options, user, pwd);
+        return primary;
+    }
+
     let oldPrimary = this.stepdown(primary);
     this.waitForState(oldPrimary, ReplSetTest.State.SECONDARY, undefined, authNode);
 
@@ -120,6 +126,7 @@ ReplSetTest.prototype.upgradePrimary = function(primary, options, user, pwd) {
         assert.eq(
             newPrimary, primary, "Primary changed unexpectedly after upgrading old primary node");
     }
+
     return newPrimary;
 };
 
@@ -131,7 +138,7 @@ ReplSetTest.prototype.upgradeNode = function(node, opts = {}, user, pwd) {
 
     var isMaster = node.getDB('admin').runCommand({isMaster: 1});
 
-    if (!isMaster.arbiterOnly) {
+    if (!isMaster.ismaster && !isMaster.arbiterOnly) {
         // Must retry this command, as it might return "currently running for election" and fail.
         // Node might still be running for an election that will fail because it lost the election
         // race with another node, at test initialization.  See SERVER-23133.
