@@ -6,9 +6,6 @@
  *   requires_sharding,
  * ]
  */
-(function() {
-"use strict";
-
 const dbName = jsTestName();
 const collName = "test";
 
@@ -25,10 +22,7 @@ function getShardCount(counts, shardName) {
  * on the i-th shard or no chunks assigned to that shard if shardDistribution[i] is null.
  */
 function runShardingTestExists(shardDistribution) {
-    const st = ShardingTest({
-        shards: shardDistribution.length,
-        setParameter: {receiveChunkWaitForRangeDeleterTimeoutMS: 90000}
-    });
+    const st = ShardingTest({shards: shardDistribution.length});
 
     const mongos = st.s0;
     const admin = mongos.getDB("admin");
@@ -37,8 +31,7 @@ function runShardingTestExists(shardDistribution) {
     const namespace = dbName + "." + collName;
 
     /* Shard the collection. */
-    assert.commandWorked(admin.runCommand({enableSharding: dbName}));
-    assert.commandWorked(admin.runCommand({movePrimary: dbName, to: shards[0]._id}));
+    assert.commandWorked(admin.runCommand({enableSharding: dbName, primaryShard: shards[0]._id}));
     assert.commandWorked(admin.runCommand({shardCollection: namespace, key: {a: 1}}));
 
     const coll = mongos.getCollection(namespace);
@@ -178,7 +171,7 @@ assert.commandFailedWithCode(
 assert.commandFailedWithCode(
     stDb.runCommand(
         {aggregate: doesNotExistName, pipeline: [{"$collStats": {"unknown": {}}}], cursor: {}}),
-    40415);
+    ErrorCodes.IDLUnknownField);
 
 assert.commandFailedWithCode(stDb.runCommand({
     aggregate: doesNotExistName,
@@ -212,7 +205,7 @@ assert.commandFailedWithCode(
 assert.commandFailedWithCode(
     standaloneDb.runCommand(
         {aggregate: doesNotExistName, pipeline: [{"$collStats": {"unknown": {}}}], cursor: {}}),
-    40415);
+    ErrorCodes.IDLUnknownField);
 
 assert.commandFailedWithCode(standaloneDb.runCommand({
     aggregate: doesNotExistName,
@@ -222,4 +215,3 @@ assert.commandFailedWithCode(standaloneDb.runCommand({
                              ErrorCodes.NamespaceNotFound);
 
 MongoRunner.stopMongod(conn);
-})();
