@@ -7,9 +7,7 @@
  * ]
  */
 
-(function() {
-"use strict";
-load("jstests/libs/fail_point_util.js");  // For 'configureFailPoint()'
+import {configureFailPoint} from "jstests/libs/fail_point_util.js";
 
 const rst = new ReplSetTest({
     nodes: 2,
@@ -150,5 +148,4 @@ validateTargetDocsPerBatch();
 validateTargetBatchTimeMS();
 validateTargetStagedDocsBytes();
 
-rst.stopSet();
-})();
+rst.stopSet();
\ No newline at end of file
