@@ -1,8 +1,5 @@
 // Log bound addresses at startup.
 
-(function() {
-'use strict';
-
 const mongo = MongoRunner.runMongod({ipv6: '', bind_ip: 'localhost', useLogFiles: true});
 assert.neq(mongo, null, "Database is not running");
 const log = cat(mongo.fullOptions.logFile);
@@ -11,5 +8,4 @@ assert(log.match(/Listening on.*127.0.0.1/), "Not listening on AF_INET");
 if (!_isWindows()) {
     assert(log.match(/Listening on.*\.sock/), "Not listening on AF_UNIX");
 }
-MongoRunner.stopMongod(mongo);
-}());
+MongoRunner.stopMongod(mongo);
\ No newline at end of file
