@@ -6,10 +6,7 @@
  *   requires_replication,
  * ]
  */
-(function() {
-"use strict";
-
-load("jstests/libs/fixture_helpers.js");  // For isReplSet().
+import {FixtureHelpers} from "jstests/libs/fixture_helpers.js";
 
 function retryableFindAndModify(database, collection, query, update, newImage, upsert, remove) {
     let cmd = {
@@ -82,27 +79,4 @@ function executeTest(db) {
     assert.neq(null, conn, "mongod was unable to start up");
     executeTest(conn.getDB("test"));
     MongoRunner.stopMongod(conn);
-}
-
-{
-    const rst = new ReplSetTest({
-        nodes: [
-            {},
-            {
-                // Disallow elections on secondary.
-                rsConfig: {
-                    priority: 0,
-                    votes: 0,
-                },
-            }
-        ]
-    });
-    rst.startSet();
-    rst.initiate();
-    // Test the modern default behavior where storeFindAndModifyImagesInSideCollection is true.
-    rst.getPrimary().adminCommand(
-        {setParameter: 1, storeFindAndModifyImagesInSideCollection: true});
-    executeTest(rst.getPrimary().getDB("test"));
-    rst.stopSet();
-}
-})();
+}
\ No newline at end of file
