@@ -5,10 +5,8 @@
 //   requires_majority_read_concern,
 //   requires_replication,
 // ]
-(function() {
-"use strict";
-load("jstests/libs/change_stream_util.js");        // For ChangeStreamTest.
-load("jstests/libs/collection_drop_recreate.js");  // For assert[Drop|Create]Collection.
+import {ChangeStreamTest, ChangeStreamWatchMode} from "jstests/libs/change_stream_util.js";
+import {assertDropAndRecreateCollection} from "jstests/libs/collection_drop_recreate.js";
 
 const rst = new ReplSetTest({nodes: 3});
 rst.startSet();
@@ -84,5 +82,4 @@ for (let key of Object.keys(ChangeStreamWatchMode)) {
     assert.commandWorked(primaryDB.adminCommand({replSetFreeze: 0}));
 }
 
-rst.stopSet();
-}());
+rst.stopSet();
\ No newline at end of file
