@@ -5,9 +5,6 @@
  *   requires_replication,
  * ]
  */
-(function() {
-'use strict';
-
 const rst = new ReplSetTest({
     nodes: 2,
     nodeOptions: {setParameter: {"aggregateOperationResourceConsumptionMetrics": true}}
@@ -111,20 +108,18 @@ let nextId = nDocs;
         // The first aggregate operation will read from the top of the oplog, size not guaranteed.
         assert.gt(metrics[dbName].primaryMetrics.docBytesRead, 0);
         assert.gt(metrics[dbName].primaryMetrics.docUnitsRead, 0);
-        assert.gt(metrics[dbName].primaryMetrics.cursorSeeks, 0);
         assert.eq(metrics[dbName].primaryMetrics.docUnitsReturned, 0);
     });
 
-    // Ensure that while nothing is returned from the change stream, the server still measures read
+    // Ensure that while nothing is returned from the change stream, the server measures no read
     // activity.
     clearMetrics(primary);
     assert(!cur.hasNext());
     assertMetrics(primary, (metrics) => {
         // Calling hasNext may perform many reads from the oplog. The oplog entry size is not
         // guaranteed.
-        assert.gt(metrics[dbName].primaryMetrics.docBytesRead, 0);
-        assert.gt(metrics[dbName].primaryMetrics.docUnitsRead, 0);
-        assert.gt(metrics[dbName].primaryMetrics.cursorSeeks, 0);
+        assert.eq(metrics[dbName].primaryMetrics.docBytesRead, 0);
+        assert.eq(metrics[dbName].primaryMetrics.docUnitsRead, 0);
         assert.eq(metrics[dbName].primaryMetrics.docUnitsReturned, 0);
     });
 
@@ -141,7 +136,6 @@ let nextId = nDocs;
         assert.eq(metrics[dbName].totalUnitsWritten, 1);
         assert.eq(metrics[dbName].primaryMetrics.docBytesRead, 0);
         assert.eq(metrics[dbName].primaryMetrics.docUnitsRead, 0);
-        assert.eq(metrics[dbName].primaryMetrics.cursorSeeks, 0);
         assert.eq(metrics[dbName].primaryMetrics.docUnitsReturned, 0);
     });
 
@@ -161,7 +155,6 @@ let nextId = nDocs;
         // Will read at least one document from the oplog.
         assert.gt(metrics[dbName].primaryMetrics.docBytesRead, 0);
         assert.gt(metrics[dbName].primaryMetrics.docUnitsRead, 0);
-        assert.gt(metrics[dbName].primaryMetrics.cursorSeeks, 0);
         // Returns one large document
         assert.eq(metrics[dbName].primaryMetrics.docUnitsReturned, 4);
     });
@@ -170,14 +163,13 @@ let nextId = nDocs;
     clearMetrics(primary);
     assert.commandWorked(primaryDB[collName].update(doc, {$set: {b: 0}}));
     assertMetrics(primary, (metrics) => {
-        assert.eq(metrics[dbName].docBytesWritten, 40);
+        assert.eq(metrics[dbName].docBytesWritten, 11);
         assert.eq(metrics[dbName].docUnitsWritten, 1);
         assert.eq(metrics[dbName].totalUnitsWritten, 1);
         assert.eq(metrics[dbName].primaryMetrics.docBytesRead, 29);
         assert.eq(metrics[dbName].primaryMetrics.docUnitsRead, 1);
         assert.eq(metrics[dbName].primaryMetrics.idxEntryBytesRead, 3);
         assert.eq(metrics[dbName].primaryMetrics.idxEntryUnitsRead, 1);
-        assert.gt(metrics[dbName].primaryMetrics.cursorSeeks, 0);
         assert.eq(metrics[dbName].primaryMetrics.docUnitsReturned, 0);
     });
 
@@ -202,12 +194,14 @@ let nextId = nDocs;
         assert.gte(metrics[dbName].primaryMetrics.docUnitsRead, 2);
         assert.eq(metrics[dbName].primaryMetrics.idxEntryBytesRead, 3);
         assert.eq(metrics[dbName].primaryMetrics.idxEntryUnitsRead, 1);
-        assert.gt(metrics[dbName].primaryMetrics.cursorSeeks, 0);
         assert.eq(metrics[dbName].primaryMetrics.docUnitsReturned, 4);
     });
 })();
 
 (function changeStreamSecondary() {
+    // Ensure change streams don't see previous writes.
+    rst.awaitReplication();
+
     clearMetrics(secondary);
     const cur = secondaryDB[collName].watch([], {fullDocument: "updateLookup"});
 
@@ -215,19 +209,16 @@ let nextId = nDocs;
         // The first aggregate operation will read one document from the oplog, size not guaranteed.
         assert.gt(metrics[dbName].secondaryMetrics.docBytesRead, 0);
         assert.gt(metrics[dbName].secondaryMetrics.docUnitsRead, 0);
-        assert.gt(metrics[dbName].secondaryMetrics.cursorSeeks, 0);
         assert.eq(metrics[dbName].secondaryMetrics.docUnitsReturned, 0);
     });
 
-    // Ensure that while nothing is returned from the change stream, the server still measures read
+    // Ensure that while nothing is returned from the change stream, the server measures no read
     // activity.
     clearMetrics(secondary);
     assert(!cur.hasNext());
     assertMetrics(secondary, (metrics) => {
-        // Calling hasNext may perform many reads from the oplog, and the size is not guaranteed.
-        assert.gt(metrics[dbName].secondaryMetrics.docBytesRead, 0);
-        assert.gt(metrics[dbName].secondaryMetrics.docUnitsRead, 0);
-        assert.gt(metrics[dbName].secondaryMetrics.cursorSeeks, 0);
+        assert.eq(metrics[dbName].secondaryMetrics.docBytesRead, 0);
+        assert.eq(metrics[dbName].secondaryMetrics.docUnitsRead, 0);
         assert.eq(metrics[dbName].secondaryMetrics.docUnitsReturned, 0);
     });
 
@@ -251,11 +242,9 @@ let nextId = nDocs;
     });
     assert.eq(doc, cur.next().fullDocument);
     assertMetrics(secondary, (metrics) => {
-        // Performs one seek on the oplog, read at least one entry, and then returns one large
-        // response document.
+        // Read at least one entry, and then returns one large response document.
         assert.gt(metrics[dbName].secondaryMetrics.docBytesRead, 0);
         assert.gt(metrics[dbName].secondaryMetrics.docUnitsRead, 0);
-        assert.gte(metrics[dbName].secondaryMetrics.cursorSeeks, 1);
         assert.gte(metrics[dbName].secondaryMetrics.docUnitsReturned, 3);
     });
 
@@ -280,16 +269,13 @@ let nextId = nDocs;
     let res = cur.next();
     assert.docEq(newDoc, res.fullDocument, res);
     assertMetrics(secondary, (metrics) => {
-        // Performs at least three seeks (oplog, _id index, collection), reads at least one entry
-        // from the oplog and once from the collection, and then returns one large response
-        // document.
+        // Reads at least one entry from the oplog and once from the collection, and then returns
+        // one large response document.
         assert.gt(metrics[dbName].secondaryMetrics.docBytesRead, 0);
         assert.gt(metrics[dbName].secondaryMetrics.docUnitsRead, 0);
         assert.eq(metrics[dbName].secondaryMetrics.idxEntryBytesRead, 3);
         assert.eq(metrics[dbName].secondaryMetrics.idxEntryUnitsRead, 1);
-        assert.gte(metrics[dbName].secondaryMetrics.cursorSeeks, 3);
         assert.gte(metrics[dbName].secondaryMetrics.docUnitsReturned, 4);
     });
 })();
 rst.stopSet();
-}());
