@@ -5,9 +5,8 @@
  *   requires_replication,
  * ]
  */
-(function() {
-load("jstests/libs/collection_drop_recreate.js");  // For assert[Drop|Create]Collection.
-load("jstests/libs/change_stream_util.js");        // For 'ChangeStreamTest'.
+import {ChangeStreamTest} from "jstests/libs/change_stream_util.js";
+import {assertDropAndRecreateCollection} from "jstests/libs/collection_drop_recreate.js";
 
 const st = new ShardingTest({
     shards: 2,
@@ -20,8 +19,8 @@ const st = new ShardingTest({
 const testDB = st.s.getDB(jsTestName());
 
 // Enable sharding on the test database and ensure that the primary is shard0.
-assert.commandWorked(testDB.adminCommand({enableSharding: testDB.getName()}));
-st.ensurePrimaryShard(testDB.getName(), st.shard0.shardName);
+assert.commandWorked(
+    testDB.adminCommand({enableSharding: testDB.getName(), primaryShard: st.shard0.shardName}));
 
 const caseInsensitiveCollectionName = "change_stream_case_insensitive";
 const caseInsensitive = {
@@ -65,5 +64,4 @@ assert.commandWorked(testDB.adminCommand({
 // Attempt to read from the change stream. We should see both inserts, without an invalidation.
 cst.assertNextChangesEqual({cursor: csCursor, expectedChanges: [{docId: 0}, {docId: 1}]});
 
-st.stop();
-})();
+st.stop();
\ No newline at end of file
