@@ -2,18 +2,12 @@
  * Checks that the oplog cap maintainer thread is started and the oplog truncate markers calculation
  * is performed under normal startup circumstances. Both of these operations should not be done when
  * starting up with any of the following modes:
- *     - skipOplogSampling
  *     - readonly
  *     - repair
  *     - recoverFromOplogAsStandalone
  *
  * @tags: [requires_replication, requires_persistence]
  */
-load("jstests/libs/oplog_truncation_util.js");
-
-(function() {
-"use strict";
-
 // Verify that the oplog cap maintainer thread is running under normal circumstances.
 jsTestLog("Testing single node replica set mode");
 const rst = new ReplSetTest(
@@ -28,23 +22,18 @@ checkLog.containsJson(primary, 22382);    // Oplog truncate markers calculated.
 rst.stopSet(/*signal=*/ null, /*forRestart=*/ true);
 
 // A subset of startup options prevent the oplog cap maintainer thread from being started. These
-// startup options are currently limited to skipOplogSampling, readOnly,
-// recoverFromOplogAsStandalone and repair.
+// startup options are currently limited to readOnly, recoverFromOplogAsStandalone and repair.
+function verifyOplogCapMaintainerThreadNotStarted(log) {
+    const threadRegex = new RegExp("\"id\":5295000");
+    const oplogTruncateMarkersRegex = new RegExp("\"id\":22382");
 
-jsTestLog("Testing with skipOplogSampling mode");
-clearRawMongoProgramOutput();
-let conn = MongoRunner.runMongod({
-    dbpath: primary.dbpath,
-    noCleanData: true,
-    setParameter: {skipOplogSampling: true, logComponentVerbosity: tojson({storage: 1})},
-});
-assert(conn);
-MongoRunner.stopMongod(conn);
-verifyOplogCapMaintainerThreadNotStarted(rawMongoProgramOutput(".*"));
+    assert(!threadRegex.test(log));
+    assert(!oplogTruncateMarkersRegex.test(log));
+}
 
 jsTestLog("Testing readOnly mode");
 clearRawMongoProgramOutput();
-conn = MongoRunner.runMongod({
+let conn = MongoRunner.runMongod({
     dbpath: primary.dbpath,
     noCleanData: true,
     queryableBackupMode: "",  // readOnly
@@ -74,5 +63,4 @@ conn = MongoRunner.runMongod({
     setParameter: {logComponentVerbosity: tojson({storage: 1})},
 });
 assert(!conn);
-verifyOplogCapMaintainerThreadNotStarted(rawMongoProgramOutput(".*"));
-}());
+verifyOplogCapMaintainerThreadNotStarted(rawMongoProgramOutput());
\ No newline at end of file
