@@ -6,9 +6,6 @@
  *   requires_sharding,
  * ]
  */
-(function() {
-"use strict";
-
 const st = new ShardingTest({mongos: 1, shards: 0, keyFile: "jstests/libs/key1"});
 
 assert.commandFailedWithCode(st.s.adminCommand({
@@ -17,5 +14,4 @@ assert.commandFailedWithCode(st.s.adminCommand({
 }),
                              ErrorCodes.ClientDisconnect);
 
-st.stop();
-})();
\ No newline at end of file
+st.stop();
\ No newline at end of file
