@@ -1,9 +1,6 @@
 /**
  * Test that verifies client metadata is logged as part of slow query logging in MongoD.
  */
-(function() {
-'use strict';
-
 let conn = MongoRunner.runMongod({useLogFiles: true});
 assert.neq(null, conn, 'mongod was unable to start up');
 
@@ -32,5 +29,4 @@ for (var a of log.split("\n")) {
 assert(predicate.test(log),
        "'Slow query' log line missing in mongod log file!\n" +
            "Log file contents: " + conn.fullOptions.logFile);
-MongoRunner.stopMongod(conn);
-})();
+MongoRunner.stopMongod(conn);
\ No newline at end of file
