@@ -2,9 +2,6 @@
  * @tags: [featureFlagSbeFull]
  */
 
-(function() {
-"use strict";
-
 const st = new ShardingTest({
     shards: 2,
     mongos: 1,
@@ -18,8 +15,8 @@ const db = st.getDB("test");
 const coll = db.analyze_coll;
 coll.drop();
 
-assert.commandWorked(st.s.adminCommand({enableSharding: db.getName()}));
-st.ensurePrimaryShard(db.getName(), st.shard1.shardName);
+assert.commandWorked(
+    st.s.adminCommand({enableSharding: db.getName(), primaryShard: st.shard1.shardName}));
 assert.commandWorked(
     st.s.adminCommand({shardCollection: coll.getFullName(), key: {_id: "hashed"}}));
 
@@ -35,4 +32,3 @@ res = db.runCommand({analyze: coll.getName(), writeConcern: {w: 1}});
 assert.commandWorked(res);
 
 st.stop();
-})();
