@@ -5,9 +5,6 @@
  * @tags: [requires_persistence, requires_fcv_52, requires_replication]
  */
 
-(function() {
-"use strict";
-
 const testName = "clustered_collection_at_startup";
 const dbpath = MongoRunner.dataPath + testName;
 const coll = "clusteredCollection";
@@ -43,5 +40,4 @@ replSet.initiate();
     assert(!/20322/.test(cmdRes.log));
 }
 
-replSet.stopSet();
-})();
+replSet.stopSet();
\ No newline at end of file
