@@ -7,10 +7,9 @@
  *   assumes_unsharded_collection,
  * ]
  */
-(function() {
-"use strict";
-load("jstests/libs/clustered_collections/clustered_collection_util.js");
-load("jstests/libs/clustered_collections/clustered_collection_hint_common.js");
+import {
+    testClusteredCollectionHint
+} from "jstests/libs/clustered_collections/clustered_collection_hint_common.js";
 
 const conn = MongoRunner.runMongod({setParameter: {supportArbitraryClusterKeyIndex: true}});
 
@@ -20,5 +19,4 @@ const nonReplicatedColl = nonReplicatedDB[collName];
 
 testClusteredCollectionHint(nonReplicatedColl, {ts: 1}, "ts_1");
 
-MongoRunner.stopMongod(conn);
-})();
+MongoRunner.stopMongod(conn);
\ No newline at end of file
