@@ -10,27 +10,17 @@
  * ]
  */
 
-(function() {
-'use strict';
-
-load('jstests/libs/fail_point_util.js');
-load('jstests/libs/parallel_shell_helpers.js');
+import {configureFailPoint} from "jstests/libs/fail_point_util.js";
+import {
+    assertCommandFailedWithCodeInParallelShell,
+    assertCommandWorkedInParallelShell,
+} from "jstests/libs/parallel_shell_helpers.js";
 
 const rst = new ReplSetTest({nodes: 1});
 rst.startSet();
 rst.initiate();
 
 const primary = rst.getPrimary();
-const collModIndexUniqueEnabled =
-    assert.commandWorked(primary.adminCommand({getParameter: 1, featureFlagCollModIndexUnique: 1}))
-        .featureFlagCollModIndexUnique.value;
-
-if (!collModIndexUniqueEnabled) {
-    jsTestLog('Skipping test because the collMod unique index feature flag is disabled');
-    rst.stopSet();
-    return;
-}
-
 let collCount = 0;
 const collPrefix = 'collmod_convert_to_unique_disallow_duplicates_';
 
@@ -253,5 +243,4 @@ testCollModConvertUniqueWithSideWrites(initialDocsDuplicate, (coll) => {
     jsTestLog('Failed to insert documents. Resuming collMod index conversion.');
 }, {_id: 1000, a: 100} /* duplicateDoc */, [{ids: [1, 2]}, {ids: [3, 4]}] /* expectedViolations */);
 
-rst.stopSet();
-})();
+rst.stopSet();
\ No newline at end of file
