@@ -3,9 +3,6 @@
  * Note: noPassthrough/commit_quorum.js - Verifies the commitQuorum behavior for replica sets.
  */
 
-(function() {
-'use strict';
-
 const standalone = MongoRunner.runMongod();
 const db = standalone.getDB("test");
 
@@ -15,5 +12,4 @@ assert.commandFailedWithCode(
         {createIndexes: "coll", indexes: [{name: "x_1", key: {x: 1}}], commitQuorum: "majority"}),
     ErrorCodes.BadValue);
 
-MongoRunner.stopMongod(standalone);
-})();
+MongoRunner.stopMongod(standalone);
\ No newline at end of file
