@@ -7,7 +7,6 @@
  * ]
  */
 
-(function() {
 const replSet = new ReplSetTest({
     nodes: [
         {
@@ -78,5 +77,4 @@ assert.commandFailedWithCode(coll.createIndex({a: 1}, {}, nNodes + 1),
                              ErrorCodes.UnsatisfiableCommitQuorum);
 coll.dropIndexes();
 
-replSet.stopSet();
-})();
+replSet.stopSet();
\ No newline at end of file
