@@ -3,9 +3,6 @@
  *
  * @tags: [requires_wiredtiger, requires_persistence]
  */
-(function() {
-"use strict";
-
 const conn = MongoRunner.runMongod({});
 const db = conn.getDB("test");
 const coll = db.getCollection(jsTest.name());
@@ -21,5 +18,4 @@ for (const failPoint of failPoints) {
     assert.commandWorked(db.adminCommand({configureFailPoint: failPoint, mode: "off"}));
 }
 
-MongoRunner.stopMongod(conn);
-}());
+MongoRunner.stopMongod(conn);
\ No newline at end of file
