@@ -1,8 +1,5 @@
 // Tests --networkMessageCompressors options.
 
-(function() {
-'use strict';
-
 var runTest = function(optionValue, expected) {
     jsTest.log("Testing with --networkMessageCompressors=\"" + optionValue +
                "\" expecting: " + expected);
@@ -38,5 +35,4 @@ var runTest = function(optionValue, expected) {
 assert.throws(() => MongoRunner.runMongod({networkMessageCompressors: "snappy,disabled"}));
 
 runTest("snappy", ["snappy"]);
-runTest("disabled", undefined);
-}());
+runTest("disabled", undefined);
\ No newline at end of file
