@@ -1,9 +1,10 @@
 // Test config file expansion using EXEC.
 
-(function() {
-'use strict';
-
-load('jstests/noPassthrough/libs/configExpand/lib.js');
+import {
+    configExpandFailure,
+    configExpandSuccess,
+    makeReflectionCmd
+} from "jstests/noPassthrough/libs/configExpand/lib.js";
 
 assert.eq(runNonMongoProgram.apply(null, makeReflectionCmd('12345', {sleep: 0}).split(" ")), 0);
 
@@ -29,5 +30,4 @@ configExpandFailure({
     }
 },
                     /Timeout expired/,
-                    {configExpandTimeoutSecs: 5});
-})();
+                    {configExpandTimeoutSecs: 5});
\ No newline at end of file
