@@ -1,10 +1,11 @@
 // Test config file expansion using REST at top level.
 // @tags: [requires_http_client]
 
-(function() {
-'use strict';
-
-load('jstests/noPassthrough/libs/configExpand/lib.js');
+import {
+    configExpandFailure,
+    ConfigExpandRestServer,
+    configExpandSuccess
+} from "jstests/noPassthrough/libs/configExpand/lib.js";
 
 const web = new ConfigExpandRestServer();
 web.start();
@@ -33,5 +34,4 @@ configExpandFailure({
                     /Timeout was reached/,
                     {configExpandTimeoutSecs: 5});
 
-web.stop();
-})();
+web.stop();
\ No newline at end of file
