@@ -7,9 +7,6 @@
  *   requires_replication,
  * ]
  */
-(function() {
-"use strict";
-
 const replSet = new ReplSetTest({nodes: 1});
 
 replSet.startSet();
@@ -33,5 +30,4 @@ assert.commandFailedWithCode(
         {find: "transactions", readConcern: {level: "snapshot", atClusterTime: operationTime}}),
     5557800);
 
-replSet.stopSet();
-})();
+replSet.stopSet();
\ No newline at end of file
