@@ -6,9 +6,6 @@
  * ]
  */
 
-(function() {
-'use strict';
-
 const conn = MongoRunner.runMongod();
 const db = conn.getDB("test");
 const collectionConfig = {
@@ -17,5 +14,4 @@ const collectionConfig = {
 const createResult = db.createCollection("coll", collectionConfig);
 assert.commandFailedWithCode(createResult, 6627201);
 
-MongoRunner.stopMongod(conn);
-})();
+MongoRunner.stopMongod(conn);
\ No newline at end of file
