@@ -4,10 +4,7 @@
  *   requires_replication,
  * ]
  */
-(function() {
-"use strict";
-
-load("jstests/libs/fail_point_util.js");
+import {configureFailPoint} from "jstests/libs/fail_point_util.js";
 
 const rst = new ReplSetTest({
     nodes: [
@@ -58,5 +55,4 @@ try {
     failPoint.off();
 }
 
-rst.stopSet();
-})();
+rst.stopSet();
\ No newline at end of file
