@@ -5,10 +5,10 @@
  *   requires_sharding,
  * ]
  */
-(function() {
-"use strict";
-
-const st = new ShardingTest({shards: 2});
+const st = new ShardingTest({
+    shards: 2,
+    mongosOptions: {setParameter: {'failpoint.skipClusterParameterRefresh': "{'mode':'alwaysOn'}"}}
+});
 st.stopBalancer();
 
 const db = st.s.getDB("test");
@@ -23,8 +23,8 @@ function getNumberOfCursorsMoreThanOneBatch() {
 }
 
 coll.drop();
-assert.commandWorked(db.adminCommand({enableSharding: db.getName()}));
-st.ensurePrimaryShard(db.getName(), st.shard0.shardName);
+assert.commandWorked(
+    db.adminCommand({enableSharding: db.getName(), primaryShard: st.shard0.shardName}));
 db.adminCommand({shardCollection: coll.getFullName(), key: {_id: 1}});
 assert.commandWorked(db.adminCommand({split: coll.getFullName(), middle: {_id: 0}}));
 
@@ -87,5 +87,4 @@ cmdRes = assert.commandWorked(
 assert.eq(getNumberOfCursorsOpened() - initialNumCursorsOpened, 3, cmdRes);
 assert.eq(getNumberOfCursorsMoreThanOneBatch() - initialNumCursorsMoreThanOneBatch, 2, cmdRes);
 
-st.stop();
-})();
+st.stop();
\ No newline at end of file
