@@ -7,9 +7,7 @@
  * ]
  */
 
-(function() {
-"use strict";
-load("jstests/libs/fail_point_util.js");
+import {configureFailPoint} from "jstests/libs/fail_point_util.js";
 
 const replTest = new ReplSetTest({name: "dbcheck_batch_deadline", nodes: 2});
 replTest.startSet();
@@ -29,10 +27,10 @@ for (let i = 0; i < collCount; i++) {
 }
 
 // Run dbCheck with a failpoint configured so that we're only ever able to process 1 document per
-// batch before hitting the 1-second default maxBatchTimeMillis.
+// batch before hitting the 1-second maxBatchTimeMillis.
 const fp = configureFailPoint(primary, 'SleepDbCheckInBatch', {sleepMs: 2000});
 const timesEntered = fp.count;
-assert.commandWorked(db.runCommand({dbCheck: coll.getName()}));
+assert.commandWorked(db.runCommand({dbCheck: coll.getName(), maxBatchTimeMillis: 1000}));
 
 // Wait for dbCheck to complete and disable the failpoint.
 assert.soon(function() {
@@ -67,5 +65,4 @@ if (debugBuild) {
 
 assert.eq(0, healthlog.find({"severity": {$ne: "info"}}).itcount());
 
-replTest.stopSet();
-})();
+replTest.stopSet();
\ No newline at end of file
