@@ -5,9 +5,6 @@
  * ]
  */
 
-(function() {
-"use strict";
-
 const rst = new ReplSetTest({nodes: 3});
 rst.startSet();
 rst.initiate();
@@ -55,5 +52,4 @@ assert.commandWorked(primary.adminCommand({replSetReconfig: cfg}));
 // Make sure all nodes, including the once-removed node, have the final config.
 rst.waitForConfigReplication(primary);
 
-rst.stopSet();
-})();
+rst.stopSet();
\ No newline at end of file
