@@ -7,11 +7,8 @@
  *   requires_replication,
  * ]
  */
-(function() {
-"use strict";
-
-load('jstests/noPassthrough/libs/index_build.js');
-load("jstests/libs/fail_point_util.js");
+import {configureFailPoint} from "jstests/libs/fail_point_util.js";
+import {IndexBuildTest} from "jstests/noPassthrough/libs/index_build.js";
 
 const rst = new ReplSetTest({
     nodes: [
@@ -66,5 +63,4 @@ rst.awaitReplication();
 // Have the new primary try to drop the database. The stepped down node must successfully replicate
 // this dropDatabase command.
 assert.commandWorked(rst.getPrimary().getDB('test').dropDatabase());
-rst.stopSet();
-})();
+rst.stopSet();
\ No newline at end of file
