@@ -3,10 +3,7 @@
  * command will only abort in-progress index builds if the user specifies all of the indexes that a
  * single builder is building together, as we can only abort at the index builder granularity level.
  */
-(function() {
-"use strict";
-
-load("jstests/noPassthrough/libs/index_build.js");
+import {IndexBuildTest} from "jstests/noPassthrough/libs/index_build.js";
 
 const mongodOptions = {};
 const conn = MongoRunner.runMongod(mongodOptions);
@@ -65,5 +62,4 @@ IndexBuildTest.assertIndexes(coll,
                              /*readyIndexes=*/["_id_", "a_1", "b_1", "c_1"],
                              /*notReadyIndexes=*/[]);
 
-MongoRunner.stopMongod(conn);
-}());
+MongoRunner.stopMongod(conn);
\ No newline at end of file
