@@ -4,10 +4,7 @@
  *
  * @tags: [requires_replication]
  */
-(function() {
-"use strict";
-
-load("jstests/noPassthrough/libs/index_build.js");
+import {IndexBuildTest} from "jstests/noPassthrough/libs/index_build.js";
 
 const rst = ReplSetTest({nodes: 3});
 rst.startSet();
@@ -70,5 +67,4 @@ IndexBuildTest.assertIndexes(
 IndexBuildTest.assertIndexes(
     secondaryColl, /*numIndexes=*/ 2, /*readyIndexes=*/["_id_", "y_1"], /*notReadyIndexes=*/[]);
 
-rst.stopSet();
-}());
+rst.stopSet();
\ No newline at end of file
