@@ -3,8 +3,6 @@
  * @tags: [requires_wiredtiger]
  */
 
-(function() {
-"use strict";
 var conn = MongoRunner.runMongod();
 var db = conn.getDB('test');
 
@@ -24,5 +22,4 @@ for (let i = 0; i < 100; i++) {
         db.adminCommand({configureFailPoint: 'WTWriteConflictException', mode: "off"}));
 }
 
-MongoRunner.stopMongod(conn);
-})();
+MongoRunner.stopMongod(conn);
\ No newline at end of file
