@@ -1,11 +1,8 @@
 /**
  * Test that the find command can spill to disk while executing a blocking sort.
  */
-(function() {
-"use strict";
-
-load("jstests/libs/analyze_plan.js");
-load("jstests/libs/sbe_util.js");  // For checkSbeFullyEnabled.
+import {getAggPlanStage, getPlanStage} from "jstests/libs/analyze_plan.js";
+import {checkSbeFullyEnabled} from "jstests/libs/sbe_util.js";
 
 // Only allow blocking sort execution to use 100 kB of memory.
 const kMaxMemoryUsageKB = 100;
@@ -430,4 +427,3 @@ assert.eq(aggregationExternalSortStatsForPipeline.spills,
           aggregationExternalSortStatsForPipeline);
 
 MongoRunner.stopMongod(conn);
-}());
