@@ -6,10 +6,12 @@
  * requires_sharding
  * ]
  */
-load("jstests/fle2/libs/encrypted_client_util.js");
+import {EncryptedClient, isEnterpriseShell} from "jstests/fle2/libs/encrypted_client_util.js";
 
-(function() {
-'use strict';
+if (!isEnterpriseShell()) {
+    jsTestLog("Skipping test as it requires the enterprise module");
+    quit();
+}
 
 function runTest(mongosConn, shardConn) {
     let dbName = 'testdb';
@@ -25,7 +27,9 @@ function runTest(mongosConn, shardConn) {
 
     let clientShard = new EncryptedClient(shardConn, dbName);
 
-    assert.commandFailedWithCode(clientShard.getDB().basic.compact(), 6583201);
+    clientShard.runEncryptionOperation(() => {
+        assert.commandFailedWithCode(clientShard.getDB().basic.compact(), 6583201);
+    });
 }
 
 jsTestLog("Sharding: Testing fle2 drop collection warning");
@@ -36,4 +40,3 @@ jsTestLog("Sharding: Testing fle2 drop collection warning");
 
     st.stop();
 }
-}());
