@@ -6,9 +6,6 @@
  * ]
  */
 
-(function() {
-'use strict';
-
 const conn = MongoRunner.runMongod();
 
 const coll = conn.getDB('test').getCollection('c');
@@ -43,5 +40,4 @@ assert.eq(
         })
         .itcount());
 
-MongoRunner.stopMongod(conn);
-}());
+MongoRunner.stopMongod(conn);
\ No newline at end of file
