@@ -6,10 +6,7 @@
  *   requires_fcv_62
  * ]
  */
-load('jstests/libs/ftdc.js');
-
-(function() {
-'use strict';
+import {verifyGetDiagnosticData} from "jstests/libs/ftdc.js";
 
 const kDbName = "mirrored_reads_ftdc_test";
 const kCollName = "test";
@@ -136,7 +133,7 @@ assert.commandWorked(primary.adminCommand({setParameter: 1, mirrorReads: {sampli
         }
         // There are two secondaries, so `kOperations * 2` reads must be resolved.
         return primaryResolvedBeforeReads + kOperations * 2 <= primaryResolvedAfterReads;
-    }, "Failed to update extended FTDC metrics within time limit", 10000);
+    }, "Failed to update extended FTDC metrics within time limit", 30000);
 
     const primaryDataAfterReads = getDiagnosticData(primary);
     const primarySentAfterReads = primaryDataAfterReads.sent;
@@ -151,7 +148,6 @@ assert.commandWorked(primary.adminCommand({setParameter: 1, mirrorReads: {sampli
         let mirroredReadsProcessedAfter = getMirroredReadsProcessedAsSecondary();
         return mirroredReadsSucceeded ==
             (mirroredReadsProcessedAfter - mirroredReadsProcessedBefore);
-    }, "Failed to wait for secondary mirrored reads stats to converge", 10000);
+    }, "Failed to wait for secondary mirrored reads stats to converge", 30000);
 }
 rst.stopSet();
-})();
