@@ -18,10 +18,7 @@
 // test fails, and hard-wiring that as the test number.
 //
 
-(function() {
-"use strict";
-
-load("jstests/libs/geo_math.js");
+import {deg2rad, rad2deg} from "jstests/libs/geo_math.js";
 
 const conn = MongoRunner.runMongod();
 assert.neq(null, conn, "mongod failed to start.");
@@ -537,5 +534,4 @@ for (var test = 0; test < numTests; test++) {
     assert.eq(results.poly.docsIn, res.nRemoved);
 }
 
-MongoRunner.stopMongod(conn);
-})();
+MongoRunner.stopMongod(conn);
\ No newline at end of file
