@@ -5,8 +5,6 @@
  *   uses_parallel_shell,
  * ]
  */
-(function() {
-
 const conn = MongoRunner.runMongod({setParameter: {internalQueryGlobalProfilingFilter: 1}});
 
 let handles = [];
@@ -27,5 +25,4 @@ for (const handle of handles) {
     handle();
 }
 
-MongoRunner.stopMongod(conn);
-})();
+MongoRunner.stopMongod(conn);
\ No newline at end of file
