@@ -1,16 +1,13 @@
 /**
  * Tests $group execution with increased spilling and a non-simple collation.
+ *
  * @tags: [featureFlagSbeFull]
  */
 
-load('jstests/aggregation/extras/utils.js');  // For assertArrayEq.
-
-(function() {
-"use strict";
+import {assertArrayEq} from "jstests/aggregation/extras/utils.js";
 
 const conn = MongoRunner.runMongod();
 const db = conn.getDB('test');
-
 const coll = db.group_pushdown_with_collation;
 coll.drop();
 for (let i = 0; i < 1000; i++) {
@@ -153,4 +150,3 @@ assertArrayEq({
 });
 
 MongoRunner.stopMongod(conn);
-}());
