@@ -7,15 +7,11 @@
  * ]
  */
 
-(function() {
-'use strict';
-
-load('jstests/noPassthrough/libs/hybrid_geo_index.js');
+import {HybridGeoIndexTest, Operation} from "jstests/noPassthrough/libs/hybrid_geo_index.js";
 
 const options = {};
 
 const invalidKey = 0;
 const validKey = 1;
 
-HybridGeoIndexTest.run(options, options, invalidKey, validKey, Operation.REMOVE);
-})();
+HybridGeoIndexTest.run(options, options, invalidKey, validKey, Operation.REMOVE);
\ No newline at end of file
