@@ -6,11 +6,10 @@
  *   requires_replication,
  * ]
  */
-(function() {
-"use strict";
-
-load('jstests/noPassthrough/libs/index_build.js');
-load("jstests/libs/fail_point_util.js");
+import {configureFailPoint} from "jstests/libs/fail_point_util.js";
+import {funWithArgs} from "jstests/libs/parallel_shell_helpers.js";
+import {extractUUIDFromObject} from "jstests/libs/uuid_util.js";
+import {IndexBuildTest} from "jstests/noPassthrough/libs/index_build.js";
 
 const rst = new ReplSetTest({
     nodes: [
@@ -34,32 +33,27 @@ primaryColl.drop();
 assert.commandWorked(primaryColl.insert({a: 1}));
 
 // Pause the index builds on the secondary, using the 'hangAfterStartingIndexBuild' failpoint.
+const failpointHangAfterInit = configureFailPoint(primaryDB, "hangAfterInitializingIndexBuild");
 const hangBeforeCleanup = configureFailPoint(primaryDB, 'hangIndexBuildBeforeAbortCleanUp');
-const hangAfterIndexBuildSecondDrain =
-    configureFailPoint(primaryDB, "hangAfterIndexBuildSecondDrain");
 
 // Block secondary to avoid commitQuorum being fullfilled.
 IndexBuildTest.pauseIndexBuilds(rst.getSecondary());
 
 jsTestLog("Waiting for index build to start");
 const createIdx = IndexBuildTest.startIndexBuild(
-    primary, primaryColl.getFullName(), {a: 1}, null, [ErrorCodes.Interrupted]);
+    primary, primaryColl.getFullName(), {a: 1}, null, [ErrorCodes.OutOfDiskSpace]);
 
-// Make sure we get the builder's opId, instead of the connection's.
-const filter = {
-    "desc": {$regex: /IndexBuildsCoordinatorMongod.*/}
-};
-const opId =
-    IndexBuildTest.waitForIndexBuildToStart(primaryDB, primaryColl.getName(), 'a_1', filter);
 const buildUUID =
     IndexBuildTest
         .assertIndexesSoon(primaryColl, 2, ['_id_'], ['a_1'], {includeBuildUUIDs: true})['a_1']
         .buildUUID;
 
-hangAfterIndexBuildSecondDrain.wait();
-jsTestLog("Running killOp to force self-abort on primary");
-assert.commandWorked(primaryDB.killOp(opId));
-hangAfterIndexBuildSecondDrain.off();
+configureFailPoint(primaryDB,
+                   "failIndexBuildWithErrorInSecondDrain",
+                   {buildUUID: buildUUID, error: ErrorCodes.OutOfDiskSpace});
+
+// Continue index build after preparing the artificial failure.
+failpointHangAfterInit.off();
 
 // Wait for the index build to be in clean up path.
 hangBeforeCleanup.wait();
@@ -91,4 +85,3 @@ collDrop();
 createIdx();
 
 rst.stopSet();
-})();
