@@ -7,11 +7,8 @@
  * ]
  */
 
-(function() {
-"use strict";
-
-load('jstests/libs/fail_point_util.js');
-load("jstests/noPassthrough/libs/index_build.js");
+import {configureFailPoint} from "jstests/libs/fail_point_util.js";
+import {IndexBuildTest} from "jstests/noPassthrough/libs/index_build.js";
 
 const rst = new ReplSetTest({nodes: 1});
 rst.startSet();
@@ -46,4 +43,3 @@ assert.commandFailedWithCode(
     ErrorCodes.CannotConvertIndexToUnique);
 
 rst.stopSet();
-}());
