@@ -1,10 +1,7 @@
 /**
- * Confirms that both foreground and background index builds can be aborted using killop.
+ * Confirms that index builds can be aborted using killop.
  */
-(function() {
-"use strict";
-
-load('jstests/noPassthrough/libs/index_build.js');
+import {IndexBuildTest} from "jstests/noPassthrough/libs/index_build.js";
 
 const conn = MongoRunner.runMongod();
 assert.neq(null, conn, "mongod was unable to start up");
@@ -14,11 +11,11 @@ assert.commandWorked(testDB.dropDatabase());
 assert.commandWorked(testDB.test.insert({a: 1}));
 const coll = testDB.test;
 
-// Test that building an index with 'options' can be aborted using killop.
-function testAbortIndexBuild(options) {
+// Test that building an index can be aborted using killop.
+function testAbortIndexBuild() {
     IndexBuildTest.pauseIndexBuilds(conn);
 
-    const createIdx = IndexBuildTest.startIndexBuild(conn, coll.getFullName(), {a: 1}, options);
+    const createIdx = IndexBuildTest.startIndexBuild(conn, coll.getFullName(), {a: 1});
 
     // When the index build starts, find its op id.
     const opId = IndexBuildTest.waitForIndexBuildToScanCollection(testDB, coll.getName(), 'a_1');
@@ -42,7 +39,5 @@ function testAbortIndexBuild(options) {
     IndexBuildTest.assertIndexes(coll, 1, ['_id_']);
 }
 
-testAbortIndexBuild({background: true});
-testAbortIndexBuild({background: false});
-MongoRunner.stopMongod(conn);
-})();
+testAbortIndexBuild();
+MongoRunner.stopMongod(conn);
\ No newline at end of file
