@@ -19,14 +19,11 @@
  *   uses_transactions,
  * ]
  */
-load('jstests/noPassthrough/libs/index_build.js');
-load("jstests/replsets/rslib.js");
-load("jstests/core/txns/libs/prepare_helpers.js");
-load("jstests/libs/fail_point_util.js");
-
-(function() {
-
-"use strict";
+import {PrepareHelpers} from "jstests/core/txns/libs/prepare_helpers.js";
+import {kDefaultWaitForFailPointTimeout} from "jstests/libs/fail_point_util.js";
+import {FeatureFlagUtil} from "jstests/libs/feature_flag_util.js";
+import {IndexBuildTest} from "jstests/noPassthrough/libs/index_build.js";
+import {waitForState} from "jstests/replsets/rslib.js";
 
 const dbName = "test";
 const collName = "coll";
@@ -45,7 +42,10 @@ assert.commandWorked(
 
 // Enable fail point which makes hybrid index build to hang before it aborts.
 var failPoint;
-if (TestData.setParameters.featureFlagIndexBuildGracefulErrorHandling) {
+
+const gracefulIndexBuildFeatureFlag =
+    FeatureFlagUtil.isEnabled(primary, "IndexBuildGracefulErrorHandling");
+if (gracefulIndexBuildFeatureFlag) {
     // If this feature flag is enabled, index builds fail immediately instead of suppressing errors
     // until the commit phase, and always signal the primary for abort (even if it is itself). Abort
     // is only ever performed in the command thread, which is interrupted by replication state
@@ -119,4 +119,3 @@ IndexBuildTest.assertIndexes(newPrimary.getDB(dbName).getCollection(collName), 1
 IndexBuildTest.assertIndexes(primaryColl, 1, ["_id_"], []);
 
 rst.stopSet();
-})();
