@@ -5,10 +5,7 @@
  *   requires_replication,
  * ]
  */
-(function() {
-"use strict";
-
-load('jstests/noPassthrough/libs/index_build.js');
+import {IndexBuildTest} from "jstests/noPassthrough/libs/index_build.js";
 
 const rst = new ReplSetTest({
     nodes: [
@@ -80,5 +77,4 @@ assert(res.valid, 'expected validation to succeed: ' + tojson(res));
 IndexBuildTest.assertIndexes(primaryColl, 2, ['_id_', 'a_1']);
 IndexBuildTest.assertIndexes(secondaryColl, 2, ['_id_', 'a_1']);
 
-rst.stopSet();
-})();
+rst.stopSet();
\ No newline at end of file
