@@ -2,11 +2,9 @@
  * Test that createCollection logs for a validator that can't match documents.
  */
 
-(function() {
-"use strict";
-
 const conn = MongoRunner.runMongod({});
 const db = conn.getDB(jsTestName());
+const collName = jsTestName();
 
 function containsValidatorWarning(logLine) {
     return logLine.includes("3216000");
@@ -16,8 +14,8 @@ let validator = {
     validator: {$jsonSchema: {bsonType: "object", required: ["val"], additionalProperties: false}}
 };
 
-db.createCollection("test", validator);
-let coll = db.getCollection("test");
+db.createCollection(collName, validator);
+let coll = db.getCollection(collName);
 let log = assert.commandWorked(db.adminCommand({getLog: "global"})).log;
 const logLine = log.filter(containsValidatorWarning);
 // The warning appears in two lines.
@@ -32,8 +30,8 @@ validator = {
     }
 };
 
-db.createCollection("test", validator);
-coll = db.getCollection("test");
+db.createCollection(collName, validator);
+coll = db.getCollection(collName);
 log = assert.commandWorked(db.adminCommand({getLog: "global"})).log;
 // Should be the same as above.
 assert.eq(logLine.length, 2, logLine);
@@ -45,11 +43,10 @@ validator = {
         {$jsonSchema: {bsonType: "object", required: ["val", "_id"], additionalProperties: false}}
 };
 
-db.createCollection("test", validator);
-coll = db.getCollection("test");
+db.createCollection(collName, validator);
+coll = db.getCollection(collName);
 log = assert.commandWorked(db.adminCommand({getLog: "global"})).log;
 // Should be the same as above.
 assert.eq(logLine.length, 2, logLine);
 
-MongoRunner.stopMongod(conn);
-})();
+MongoRunner.stopMongod(conn);
\ No newline at end of file
