@@ -5,10 +5,7 @@
  *   requires_replication,
  * ]
  */
-(function() {
-'use strict';
-
-load('jstests/noPassthrough/libs/index_build.js');
+import {IndexBuildTest} from "jstests/noPassthrough/libs/index_build.js";
 
 const dbName = "test";
 const collName = "coll";
@@ -53,19 +50,12 @@ replSet.awaitReplication();
 
 // Build and finish the first index.
 assert.commandWorked(primaryDB.runCommand(
-    {createIndexes: collName, indexes: [{key: {i: 1}, name: firstIndexName, background: true}]}));
+    {createIndexes: collName, indexes: [{key: {i: 1}, name: firstIndexName}]}));
 replSet.awaitReplication();
 
 // Start hanging index builds on the secondary.
 IndexBuildTest.pauseIndexBuilds(secondary);
 
-// With storage engines that do not support snapshot reads, the commitIndexBuild oplog entry may
-// block the listIndexes command on the secondary during oplog application because it will hold the
-// PBWM while waiting for the index build to complete in the backgroud. Therefore, we get the
-// primary to hold off on writing the commitIndexBuild oplog entry until we are ready to resume
-// index builds on the secondary.
-IndexBuildTest.pauseIndexBuilds(primary);
-
 // Build and hang on the second index. This should be run in the background if we pause index
 // builds on the primary because the createIndexes command will block.
 const coll = primaryDB.getCollection(collName);
@@ -102,10 +92,6 @@ assert(indexes[2].hasOwnProperty("buildUUID"));
 
 // Allow the replica set to finish the index build.
 IndexBuildTest.resumeIndexBuilds(secondary);
-
-// Wait for the index build to complete on the primary if we paused it.
-IndexBuildTest.resumeIndexBuilds(primary);
 createIdx();
 
-replSet.stopSet();
-}());
+replSet.stopSet();
\ No newline at end of file
