@@ -4,9 +4,6 @@
  * @tags: [requires_persistence]
  */
 
-(function() {
-'use strict';
-
 function waitFailedToStart(pid, exitCode) {
     assert.soon(function() {
         return !checkProgram(pid).alive;
@@ -77,5 +74,4 @@ if (_isWindows()) {
 // Check we have one log line
 let log = parseLogFile(backtraceLogFileFullPath);
 assert.eq(log["id"], 31380);
-assert.eq(log["msg"], "BACKTRACE");
-})();
+assert.eq(log["msg"], "BACKTRACE");
\ No newline at end of file
