@@ -7,15 +7,12 @@
  * ]
  */
 
-(function() {
-"use strict";
-
 // This test looks for exact matches in log output, which does not account for implicit
 // sessions.
 TestData.disableImplicitSessions = true;
 
-load("jstests/libs/fixture_helpers.js");  // For FixtureHelpers.
-load("jstests/libs/log.js");              // For findMatchingLogLine.
+import {FixtureHelpers} from "jstests/libs/fixture_helpers.js";
+import {findMatchingLogLine} from "jstests/libs/log.js";
 
 // Prevent the mongo shell from gossiping its cluster time, since this will increase the amount
 // of data logged for each op. For some of the testcases below, including the cluster time would
@@ -40,9 +37,9 @@ const dbName = "logtest";
 const mongosDB = mongosConn.getDB(dbName);
 const shardDB = shardConn.getDB(dbName);
 
-// Enable sharding on the the test database and ensure that the primary is on shard0.
-assert.commandWorked(mongosDB.adminCommand({enableSharding: mongosDB.getName()}));
-st.ensurePrimaryShard(mongosDB.getName(), shardConn.name);
+// Ensure that the primary shard for the test database is shard0.
+assert.commandWorked(
+    mongosDB.adminCommand({enableSharding: mongosDB.getName(), primaryShard: shardConn.name}));
 
 // Drops and re-shards the test collection, then splits at {_id: 0} and moves the upper chunk to
 // the second shard.
@@ -56,9 +53,13 @@ function dropAndRecreateTestCollection() {
 // corresponding output. Returns a pair of arrays [testsRun, logLines]; the former is the set of
 // test cases that were run, while the latter contains the logline for each test, or null if no
 // such logline was found.
-function runLoggingTests({db, slowMs, logLevel, sampleRate}) {
+function runLoggingTests({db, slowMs, logLevel, sampleRate, enableQueryStats = false}) {
     dropAndRecreateTestCollection();
 
+    // We either enable query stats on every query (rate limit -1) or no queries (rate limit 0).
+    assert.commandWorked(
+        db.adminCommand({setParameter: 1, internalQueryStatsRateLimit: enableQueryStats ? -1 : 0}));
+
     const coll = db.test;
 
     for (let i = 1; i <= 5; ++i) {
@@ -87,11 +88,24 @@ function runLoggingTests({db, slowMs, logLevel, sampleRate}) {
     // Certain fields in the log lines on mongoD are not applicable in their counterparts on
     // mongoS, and vice-versa. Ignore these fields when examining the logs of an instance on
     // which we do not expect them to appear.
-    const ignoreFields =
-            (isMongos
-                 ? ["docsExamined", "keysExamined", "keysInserted", "keysDeleted", "planSummary",
-					 "usedDisk", "hasSortStage"]
-                 : ["nShards"]);
+    const ignoreFields = (() => {
+        if (!isMongos) {
+            return ["nShards"];
+        } else if (!enableQueryStats) {
+            return [
+                "docsExamined",
+                "keysExamined",
+                "keysInserted",
+                "keysDeleted",
+                "planSummary",
+                "usedDisk",
+                "hasSortStage"
+            ];
+
+        } else {
+            return ["keysInserted", "keysDeleted", "planSummary"];
+        }
+    })();
 
     function confirmLogContents(db, {test, logFields}, testIndex) {
         // Clear the log before running the test, to guarantee that we do not match against any
@@ -179,7 +193,7 @@ function runLoggingTests({db, slowMs, logLevel, sampleRate}) {
                 command: "find",
                 find: coll.getName(),
                 comment: logFormatTestComment,
-                planSummary: "IDHACK",
+                planSummary: "EXPRESS_IXSCAN { _id: 1 }",
                 cursorExhausted: 1,
                 keysExamined: 1,
                 docsExamined: 1,
@@ -197,15 +211,16 @@ function runLoggingTests({db, slowMs, logLevel, sampleRate}) {
                           {_id: 1, a: 1, loc: {type: "Point", coordinates: [1, 1]}});
             },
             // TODO SERVER-34208: display FAM update metrics in mongoS logs.
-            logFields: Object.assign((isMongos ? {} : {nMatched: 1, nModified: 1}), {
-                command: "findAndModify",
-                findandmodify: coll.getName(),
-                planSummary: "IXSCAN { _id: 1 }",
-                keysExamined: 1,
-                docsExamined: 1,
-                $comment: logFormatTestComment,
-                collation: {locale: "fr"}
-            })
+            // For mongos with query stats enabled, findAndModify doesn't roll up data-bearing node
+            // metrics (keysExamined, docsExamined).
+            logFields: Object.assign(
+                (isMongos ? {} : {nMatched: 1, nModified: 1, keysExamined: 1, docsExamined: 1}), {
+                    command: "findAndModify",
+                    findandmodify: coll.getName(),
+                    planSummary: "IXSCAN { _id: 1 }",
+                    $comment: logFormatTestComment,
+                    collation: {locale: "fr"}
+                })
         },
         {
             test: function(db) {
@@ -214,15 +229,15 @@ function runLoggingTests({db, slowMs, logLevel, sampleRate}) {
                     out: {inline: 1},
                 }));
             },
-            logFields: {
+            // For mongos with query stats enabled, mapReduce doesn't roll up data-bearing node
+            // metrics (keysExamined, docsExamined).
+            logFields: Object.assign(isMongos ? {} : {keysExamined: 0, docsExamined: 10}, {
                 command: "mapReduce",
                 mapreduce: coll.getName(),
                 planSummary: "COLLSCAN",
-                keysExamined: 0,
-                docsExamined: 10,
                 $comment: logFormatTestComment,
                 out: {inline: 1}
-            }
+            })
         },
         {
             test: function(db) {
@@ -324,8 +339,10 @@ function runLoggingTests({db, slowMs, logLevel, sampleRate}) {
                         originalSortBytes.internalQueryMaxBlockingSortMemoryUsageBytes
                 }));
             },
+            // This query won't necessarily use disk in a sharded context.
             logFields:
-                {command: "aggregate", aggregate: coll.getName(), hasSortStage: 1, usedDisk: 1}
+                Object.assign(isMongos ? {} : {usedDisk: 1},
+                              {command: "aggregate", aggregate: coll.getName(), hasSortStage: 1})
         },
         {
             test: function(db) {
@@ -382,8 +399,9 @@ function runLoggingTests({db, slowMs, logLevel, sampleRate}) {
         const cmdObj = originatingCommands[cmdName];
         const cmdRes = assert.commandWorked(db.runCommand(cmdObj));
         const expectedCountOfDocuments = 14;
-        // Make sure queryHash field is present of getMore logs following a find command.
-        const extra = cmdName === "find" ? {queryHash: ""} : {};
+        // Make sure 'planCacheShapeHash' field is present of "getMore" logs following a find
+        // command.
+        const extra = cmdName === "find" ? {planCacheShapeHash: ""} : {};
 
         testList.push({
             test: function(db) {
@@ -428,7 +446,7 @@ for (let testDB of [shardDB, mongosDB]) {
     let [testsRun, logLines] =
         runLoggingTests({db: testDB, slowMs: -1, logLevel: 0, sampleRate: 1.0});
     let unlogged = getUnloggedTests(testsRun, logLines);
-    assert.eq(unlogged.length, 0, () => tojson(unlogged));
+    assert.eq(unlogged.length, 0, unlogged);
 
     // Test that only some operations are logged when sampleRate is < 1 at the default
     // logLevel, even when slowMs is < 0. The actual sample rate is probabilistic, and may
@@ -451,7 +469,7 @@ for (let testDB of [shardDB, mongosDB]) {
     [testsRun, logLines] =
         runLoggingTests({db: testDB, slowMs: 1000000, logLevel: 0, sampleRate: 1.0});
     unlogged = getUnloggedTests(testsRun, logLines);
-    assert.eq(unlogged.length, Math.floor(testsRun.length / 2), () => tojson(unlogged));
+    assert.eq(unlogged.length, Math.floor(testsRun.length / 2), unlogged);
 
     // Test that all operations are logged when logLevel is 1, regardless of sampleRate and
     // slowMs. We pass 'null' for slowMs to signify that a high threshold should be set
@@ -461,7 +479,13 @@ for (let testDB of [shardDB, mongosDB]) {
     [testsRun, logLines] =
         runLoggingTests({db: testDB, slowMs: null, logLevel: 1, sampleRate: 0.5});
     unlogged = getUnloggedTests(testsRun, logLines);
-    assert.eq(unlogged.length, 0, () => tojson(unlogged));
+    assert.eq(unlogged.length, 0, unlogged);
+
+    // Test with query stats enabled. This should only affect the log format in the mongos case,
+    // but we'll run against both the mongos and the shard to make sure.
+    [testsRun, logLines] = runLoggingTests(
+        {db: testDB, slowMs: -1, logLevel: 0, sampleRate: 1.0, enableQueryStats: true});
+    unlogged = getUnloggedTests(testsRun, logLines);
+    assert.eq(unlogged.length, 0, unlogged);
 }
 st.stop();
-})();
