@@ -4,11 +4,8 @@
 //   requires_sharding,
 // ]
 
-load("jstests/aggregation/extras/utils.js");  // For assertErrorCode.
-load("jstests/libs/sbe_util.js");             // For checkSbeRestrictedOrFullyEnabled.
-
-(function() {
-"use strict";
+import {assertErrorCode} from "jstests/aggregation/extras/utils.js";
+import {checkSbeRestrictedOrFullyEnabled} from "jstests/libs/sbe_util.js";
 
 // Used by testPipeline to sort result documents. All _ids must be primitives.
 function compareId(a, b) {
@@ -120,4 +117,3 @@ assert(sharded.adminCommand({shardCollection: "test.lookUp", key: {_id: 'hashed'
 runTest(sharded.getDB('test').lookUp, sharded.getDB('test').from, 4568);
 
 sharded.stop();
-}());
