@@ -1,30 +1,18 @@
 // Test that sharded $lookup can resolve sharded views correctly.
 // @tags: [requires_sharding, requires_fcv_51]
-(function() {
-"use strict";
-
-load("jstests/aggregation/extras/utils.js");  // For assertArrayEq.
-load("jstests/libs/profiler.js");             // For profilerHasSingleMatchingEntryOrThrow.
+import {assertArrayEq} from "jstests/aggregation/extras/utils.js";
+import {profilerHasSingleMatchingEntryOrThrow} from "jstests/libs/profiler.js";
 
 const sharded = new ShardingTest({
     mongos: 1,
     shards: [{verbose: 3}, {verbose: 3}, {verbose: 3}, {verbose: 3}],
     config: 1,
 });
-assert(sharded.adminCommand({enableSharding: "test"}));
-
-const isShardedLookupEnabled =
-    sharded.s.adminCommand({getParameter: 1, featureFlagShardedLookup: 1})
-        .featureFlagShardedLookup.value;
-if (!isShardedLookupEnabled) {
-    sharded.stop();
-    return;
-}
 
 const testDBName = "test";
 const testDB = sharded.getDB(testDBName);
 const primaryShard = sharded.shard0;
-sharded.ensurePrimaryShard(testDBName, primaryShard.shardName);
+assert(sharded.adminCommand({enableSharding: "test", primaryShard: primaryShard.shardName}));
 
 // Drop the 'profile' tables and then enable profiling on all shards.
 for (const shard of [sharded.shard0, sharded.shard1, sharded.shard2]) {
@@ -604,4 +592,3 @@ profilerHasSingleMatchingEntryOrThrow({
 });
 
 sharded.stop();
-}());
