@@ -1,9 +1,5 @@
 // Tests that match expression optimization works properly when the failpoint isn't triggered, and
 // is disabled properly when it is triggered.
-(function() {
-"use strict";
-
-load("jstests/libs/analyze_plan.js");  // For aggPlan functions.
 Random.setRandomSeed();
 
 const conn = MongoRunner.runMongod({});
@@ -38,5 +34,4 @@ const disabledResult = coll.aggregate(pipeline).toArray();
 // Test that the result is the same with and without optimizations enabled (result is sorted).
 assert.eq(enabledResult, disabledResult);
 
-MongoRunner.stopMongod(conn);
-}());
+MongoRunner.stopMongod(conn);
\ No newline at end of file
