@@ -3,10 +3,7 @@
 //
 // @tags: []
 
-(function() {
-"use strict";
-
-load("jstests/libs/fail_point_util.js");  // for 'configureFailPoint()'
+import {configureFailPoint} from "jstests/libs/fail_point_util.js";
 
 const st = new ShardingTest({shards: 1, mongos: 1, config: 1});
 
@@ -95,5 +92,4 @@ assertNoIdleCursors(st.shard0, collName);
     assertNoIdleCursors(st.shard0, collName, curs);
 }
 
-st.stop();
-})();
+st.stop();
\ No newline at end of file
