@@ -1,7 +1,4 @@
 // Tests that running mapReduce does not crash mongod if scripting is disabled.
-(function() {
-"use strict";
-
 const conn = MongoRunner.runMongod({noscripting: ''});
 const testDB = conn.getDB('foo');
 const coll = testDB.bar;
@@ -19,5 +16,4 @@ const reduce = function(key, values) {
 assert.commandFailedWithCode(
     testDB.runCommand({mapReduce: 'bar', map: map, reduce: reduce, out: {inline: 1}}), 31264);
 
-MongoRunner.stopMongod(conn);
-}());
+MongoRunner.stopMongod(conn);
\ No newline at end of file
