@@ -2,11 +2,8 @@
  * Tests that we can execute a query which survived a yield using an index scan on a path containing
  * a positional component. This test was designed to reproduce SERVER-52589.
  */
-(function() {
-"use strict";
-
-load("jstests/libs/analyze_plan.js");         // For explain helpers.
-load("jstests/aggregation/extras/utils.js");  // For assertArrayEq.
+import {assertArrayEq} from "jstests/aggregation/extras/utils.js";
+import {getPlanStage} from "jstests/libs/analyze_plan.js";
 
 // Configure 'internalQueryExecYieldIterations' such that operations will yield on each PlanExecutor
 // iteration.
@@ -59,5 +56,4 @@ assert.eq(ixscan.isMultiKey, false, explain);
 // Now execute the query and validate the result.
 assertArrayEq({actual: cursor.toArray(), expected: [doc]});
 
-MongoRunner.stopMongod(conn);
-}());
+MongoRunner.stopMongod(conn);
\ No newline at end of file
