@@ -1,5 +1,3 @@
-(function() {
-"use strict";
 const conn = MongoRunner.runMongod();
 assert.neq(null, conn, "mongod failed to start.");
 let mydb = conn.getDB("test_ns1");
@@ -48,4 +46,3 @@ for (let i = 0; i < its; i++) {
 print("yay");
 
 MongoRunner.stopMongod(conn);
-})();
