@@ -2,8 +2,6 @@
  * Tests counters for $group and $setWindowFields accumulator expressions.
  */
 
-(function() {
-"use strict";
 const mongod = MongoRunner.runMongod();
 const db = mongod.getDB(jsTest.name());
 const collName = jsTest.name();
@@ -122,5 +120,4 @@ for (const wAccInfo of windowAccumulators) {
     checkCounter(wAccInfo, "windowAccumulators");
 }
 
-MongoRunner.stopMongod(mongod);
-})();
+MongoRunner.stopMongod(mongod);
\ No newline at end of file
