@@ -6,14 +6,11 @@
  *   requires_replication,
  * ]
  */
-(function() {
-"use strict";
+import {aggPlanHasStage} from "jstests/libs/analyze_plan.js";
 
 // Deliberately inserts orphans outside of migration.
 TestData.skipCheckOrphans = true;
 
-load('jstests/libs/analyze_plan.js');  // For aggPlanHasStage().
-
 // Set up a 2-shard cluster.
 const st = new ShardingTest({name: jsTestName(), shards: 2, rs: {nodes: 1}});
 
@@ -25,6 +22,10 @@ const mongos = st.s;
 const configDB = mongos.getDB("config");
 
 const mongosDB = mongos.getDB(jsTestName());
+
+assert.commandWorked(
+    mongosDB.adminCommand({enableSharding: mongosDB.getName(), primaryShard: shard0.name}));
+
 const mongosColl = mongosDB.test;
 
 const shard0DB = shard0.getDB(jsTestName());
@@ -57,10 +58,6 @@ function runSampleAndConfirmResults({sampleSize, comment, expectedPlanSummaries}
     }
 }
 
-// Enable sharding on the the test database and ensure that the primary is shard0.
-assert.commandWorked(mongosDB.adminCommand({enableSharding: mongosDB.getName()}));
-st.ensurePrimaryShard(mongosDB.getName(), shard0.name);
-
 // Shard the collection on {_id: 1}, split at {_id: 0} and move the empty upper chunk to shard1.
 st.shardColl(mongosColl.getName(), {_id: 1}, {_id: 0}, {_id: 0}, mongosDB.getName());
 
@@ -150,4 +147,3 @@ runSampleAndConfirmResults({
 });
 
 st.stop();
-})();
