@@ -4,9 +4,6 @@
  *
  * Originally designed to reproduce SERVER-70597.
  */
-(function() {
-"use strict";
-
 const conn = MongoRunner.runMongod();
 const db = conn.getDB("test");
 
@@ -35,5 +32,4 @@ const results = coll.aggregate([
                     .toArray();
 assert.eq(results, [{a: 2, b: 3}]);
 
-MongoRunner.stopMongod(conn);
-})();
+MongoRunner.stopMongod(conn);
\ No newline at end of file
