@@ -1,12 +1,7 @@
 /**
  * SERVER-32001: Test that indexing paths for non-unique, partial, unique, partial&unique
  * crud operations correctly handle WriteConflictExceptions.
- *
- * SERVER-50359: Re-enable this test and fix the failure behind it on ephemeralForTest.
- * @tags: [incompatible_with_eft]
  */
-(function() {
-"strict";
 
 let conn = MongoRunner.runMongod();
 let testDB = conn.getDB("test");
@@ -52,7 +47,7 @@ assert.commandWorked(testDB.adminCommand(
     {configureFailPoint: 'WTWriteConflictExceptionForReads', mode: {activationProbability: 0.01}}));
 assert.commandWorked(testDB.adminCommand(
     {configureFailPoint: 'WTWriteConflictException', mode: {activationProbability: 0.01}}));
-res = benchRun(benchArgs);
+let res = benchRun(benchArgs);
 printjson({res});
 
 assert.commandWorked(
@@ -62,4 +57,3 @@ assert.commandWorked(
 res = t.validate();
 assert(res.valid, tojson(res));
 MongoRunner.stopMongod(conn);
-})();
