@@ -1,10 +1,8 @@
 /**
  * Test that applicationName and namespace appear in queryStats for the find command.
- * @tags: [requires_fcv_70]
+ * @tags: [requires_fcv_71]
  */
-load("jstests/libs/query_stats_utils.js");
-(function() {
-"use strict";
+import {getQueryStats, getQueryStatsFindCmd} from "jstests/libs/query_stats_utils.js";
 
 const kApplicationName = "MongoDB Shell";
 
@@ -34,4 +32,3 @@ assert.eq(1, queryStats.length, queryStats);
 assert.eq(kApplicationName, queryStats[0].key.client.application.name, queryStats);
 
 MongoRunner.stopMongod(conn);
-}());
