@@ -1,11 +1,8 @@
 /**
- * Test that the queryStats store can be cleared when the cache size is reset to 0.
- * @tags: [requires_fcv_70]
+ * Test that the query stats store can be cleared when the cache size is reset to 0.
+ * @tags: [requires_fcv_72]
  */
-load("jstests/libs/query_stats_utils.js");  // For verifyMetrics and getQueryStats.
-
-(function() {
-"use strict";
+import {getQueryStats} from "jstests/libs/query_stats_utils.js";
 
 // Turn on the collecting of queryStats metrics.
 let options = {
@@ -42,4 +39,3 @@ assert.eq(testDB.serverStatus().metrics.queryStats.queryStatsStoreSizeEstimateBy
 // Calling $queryStats should fail when the query stats store size is 0 bytes.
 assert.throwsWithCode(() => testDB.getSiblingDB("admin").aggregate([{$queryStats: {}}]), 6579000);
 MongoRunner.stopMongod(conn);
-}());
