@@ -2,11 +2,6 @@
  * Test that calls to read from query stats store fail when feature flag is turned off and sampling
  * rate > 0.
  */
-load('jstests/libs/analyze_plan.js');
-load("jstests/libs/feature_flag_util.js");
-
-(function() {
-"use strict";
 
 // This test specifically tests error handling when the feature flag is not on.
 // Set sampling rate to -1.
@@ -43,4 +38,3 @@ assert.commandFailedWithCode(testdb.adminCommand({
                              ErrorCodes.QueryFeatureNotAllowed);
 
 MongoRunner.stopMongod(conn);
-}());
\ No newline at end of file
