@@ -1,9 +1,6 @@
 // This test was designed to reproduce SERVER-77430. There was a mistaken assertion in a parser that
 // we are interested in proving will not fail here.
-// @tags: [requires_fcv_70]
-(function() {
-"use strict";
-
+// @tags: [requires_fcv_71]
 const st = new ShardingTest({
     mongos: 1,
     shards: 1,
@@ -19,4 +16,3 @@ const coll = st.s.getDB("test").geometry_without_coordinates;
 // This is a query that once mistakenly threw an error.
 assert.doesNotThrow(() => coll.find({geo: {$geoIntersects: {$geometry: {x: 40, y: 5}}}}).itcount());
 st.stop();
-}());
