@@ -7,7 +7,9 @@
  * ]
  */
 
-load("jstests/libs/query_stats_utils.js");
+import {
+    runCommandAndValidateQueryStats,
+} from "jstests/libs/query_stats_utils.js";
 const dbName = jsTestName();
 const collName = "coll";
 
@@ -49,6 +51,7 @@ const aggregateCommandObj = {
     maxTimeMS: 50 * 1000,
     bypassDocumentValidation: false,
     readConcern: {level: "majority"},
+    explain: true,
     collation: {locale: "en_US", strength: 2},
     hint: {"v": 1},
     comment: "",
@@ -76,6 +79,7 @@ const queryStatsAggregateKeyFields = [
     "client",
     "hint",
     "readConcern",
+    "explain",
     "cursor.batchSize",
     "$_passthroughToShard",
     "$_passthroughToShard.shard"
@@ -90,4 +94,4 @@ runCommandAndValidateQueryStats({
     keyFields: queryStatsAggregateKeyFields
 });
 
-st.stop();
\ No newline at end of file
+st.stop();
