@@ -1,9 +1,12 @@
 /**
  * This test confirms that query stats store key fields for an aggregate command are properly nested
  * and none are missing when running an explain query.
- * @tags: [requires_fcv_70]
+ * @tags: [requires_fcv_72]
  */
-load("jstests/libs/query_stats_utils.js");
+import {
+    runCommandAndValidateQueryStats,
+    withQueryStatsEnabled
+} from "jstests/libs/query_stats_utils.js";
 
 const collName = jsTestName();
 
