@@ -1,18 +1,14 @@
 /**
  *  Tests that query stats won't be collected for the 'analyze' command.
  *
- * @tags: [
- * requires_fcv_70
- * ]
+ * @tags: [featureFlagCommonQueryFramework]
  */
-load("jstests/libs/query_stats_utils.js");  // For getQueryStats
+import {
+    getQueryStats,
+} from "jstests/libs/query_stats_utils.js";
 
 const conn = MongoRunner.runMongod({
-    setParameter: {
-        internalQueryStatsRateLimit: -1,
-        internalQueryStatsErrorsAreCommandFatal: true,
-        featureFlagCommonQueryFramework: true
-    }
+    setParameter: {internalQueryStatsRateLimit: -1, internalQueryStatsErrorsAreCommandFatal: true}
 });
 assert.neq(null, conn, "mongod was unable to start up");
 
