@@ -2,11 +2,9 @@
  * Test that collectionType is returned properly in $queryStats. Checks for collection types
  * "collection", "view", "timeseries", "nonExistent", and "virtual". Type "changeStream" is covered
  * in query_stats_changeStreams.js.
- * @tags: [requires_fcv_70]
+ * @tags: [requires_fcv_72]
  */
-load("jstests/libs/query_stats_utils.js");
-(function() {
-"use strict";
+import {getQueryStats} from "jstests/libs/query_stats_utils.js";
 
 function runTest(conn) {
     const testDB = conn.getDB('test');
@@ -199,10 +197,10 @@ if (false) {
         mongosOptions: {
             setParameter: {
                 internalQueryStatsSamplingRate: -1,
+                'failpoint.skipClusterParameterRefresh': "{'mode':'alwaysOn'}"
             }
         },
     });
     runTest(st.s);
     st.stop();
 }
-}());
