@@ -1,15 +1,16 @@
 /**
- * Tests that QueryStats metrics aren't collected if the feature is enabled initially but is
- * disabled before the lifetime of the request is complete.
- * @tags: [requires_fcv_70]
+ * Tests that query stats aren't collected if the feature is enabled initially but is disabled
+ * before the lifetime of the request is complete.
+ * @tags: [
+ *   # Query Stats is new in 7.1.
+ *   requires_fcv_71,
+ * ]
  */
-load("jstests/libs/query_stats_utils.js");  // For getQueryStatsFindCmd.
-(function() {
-"use strict";
+import {getQueryStatsFindCmd} from "jstests/libs/query_stats_utils.js";
 
-// Test that no QueryStats entry is written when (1) dispatching an initial find query, (2)
-// disabling QueryStats, then (3) completing the command. Below, we run variations of this test
-// with combinations of different strategies to disable QueryStats and to end the command.
+// Test that no query stats entry is written when (1) dispatching an initial find query, (2)
+// disabling query stats, then (3) completing the command. Below, we run variations of this test
+// with combinations of different strategies to disable query stats and to end the command.
 function testStatsAreNotCollectedWhenDisabledBeforeCommandCompletion(
     {conn, coll, disableQueryStatsFn, endCommandFn, enableQueryStatsFn}) {
     // Issue a find commannd with a batchSize of 1 so that the query is not exhausted.
@@ -17,17 +18,17 @@ function testStatsAreNotCollectedWhenDisabledBeforeCommandCompletion(
     // Must run .next() to make sure the initial request is executed now.
     cursor.next();
 
-    // Disable QueryStats, then end the command,which triggers the path to writeQueryStats.
+    // Disable query stats, then end the command,which triggers the path to writeQueryStats.
     disableQueryStatsFn();
     endCommandFn(cursor);
 
-    // Must re-enable QueryStats in order to check via $queryStats that nothing was recorded.
+    // Must re-enable query stats in order to check via $queryStats that nothing was recorded.
     enableQueryStatsFn();
     const res = getQueryStatsFindCmd(conn);
     assert.eq(res.length, 0, res);
 }
 
-// Turn on the collecting of QueryStats metrics.
+// Turn on the collecting of queryStats metrics.
 let options = {setParameter: {internalQueryStatsRateLimit: -1}};
 
 const conn = MongoRunner.runMongod(options);
@@ -57,8 +58,8 @@ testStatsAreNotCollectedWhenDisabledBeforeCommandCompletion({
     enableQueryStatsFn: () => setQueryStatsCacheSize("10MB")
 });
 
-// Tests the scenario of disabling QueryStats by setting internalQueryStatsCacheSize to
-// 0 and ending the command by killing the cursor.
+// Tests the scenario of disabling query stats by setting internalQueryStatsCacheSize to 0 and
+// ending the command by killing the cursor.
 testStatsAreNotCollectedWhenDisabledBeforeCommandCompletion({
     conn: testDB,
     coll,
@@ -69,4 +70,3 @@ testStatsAreNotCollectedWhenDisabledBeforeCommandCompletion({
 });
 
 MongoRunner.stopMongod(conn);
-}());
\ No newline at end of file
