@@ -1,9 +1,14 @@
 /**
  * This test confirms that the correct verbosity levels are stored in the query stats key for
  * explain commands on an agg query.
- * @tags: [requires_fcv_70]
+ * @tags: [requires_fcv_72]
  */
-load("jstests/libs/query_stats_utils.js");
+import {FixtureHelpers} from "jstests/libs/fixture_helpers.js";
+import {
+    getLatestQueryStatsEntry,
+    getValueAtPath,
+    withQueryStatsEnabled,
+} from "jstests/libs/query_stats_utils.js";
 
 const testColl = jsTestName();
 
