@@ -1,11 +1,7 @@
 /**
  * Test that calls to read from query stats store fail when feature flag is turned off.
  */
-load('jstests/libs/analyze_plan.js');
-load("jstests/libs/feature_flag_util.js");
-
-(function() {
-"use strict";
+import {FeatureFlagUtil} from "jstests/libs/feature_flag_util.js";
 
 // This test specifically tests error handling when the feature flag is not on.
 // TODO SERVER-65800 this test can be removed when the feature flag is removed.
@@ -29,4 +25,3 @@ assert.commandFailedWithCode(testDB.adminCommand({
                              ErrorCodes.QueryFeatureNotAllowed);
 
 MongoRunner.stopMongod(conn);
-}());
\ No newline at end of file
