@@ -1,12 +1,9 @@
 /**
  * Test that the queryStats HMAC key is not logged.
- * @tags: [requires_fcv_70]
+ * @tags: [requires_fcv_71]
  */
 
-(function() {
-'use strict';
-
-load('jstests/libs/query_stats_utils.js');  // For getQueryStatsFindCmd
+import {getQueryStatsFindCmd} from "jstests/libs/query_stats_utils.js";
 
 const checkLogForHmacKey = function(conn) {
     const coll = conn.getCollection("test.foo");
@@ -57,4 +54,3 @@ const testMongoS = function() {
 
 testMongoD();
 testMongoS();
-})();
