@@ -2,11 +2,9 @@
  * This test confirms that queryStats store key fields specific to replica sets (readConcern and
  * readPreference) are included and correctly shapified. General command fields related to api
  * versioning are included for good measure.
- * @tags: [requires_fcv_70]
+ * @tags: [requires_fcv_71]
  */
-load("jstests/libs/query_stats_utils.js");  // For getLatestQueryStatsEntry
-(function() {
-"use strict";
+import {getLatestQueryStatsEntry} from "jstests/libs/query_stats_utils.js";
 
 const replTest = new ReplSetTest({
     name: 'queryStatsTest',
@@ -99,10 +97,9 @@ confirmCommandFieldsPresent(stats.key, commandObj);
 commandObj.readConcern = {
     level: "snapshot",
     atClusterTime: clusterTime
-};
-assert.commandWorked(replSetConn.getDB(dbName).runCommand(commandObj));
+},
+    assert.commandWorked(replSetConn.getDB(dbName).runCommand(commandObj));
 stats = getLatestQueryStatsEntry(replSetConn, {collName: collName});
 assert.eq(stats.key["readConcern"], {level: "snapshot", atClusterTime: "?timestamp"});
 
 replTest.stopSet();
-})();
