@@ -1,12 +1,9 @@
 /**
  * Test that the queryStats metrics are aggregated properly for queries run on a mongod where the
  * results fit into a single batch (and thus don't require a cursor), for find commands.
- * @tags: [requires_fcv_70]
+ * @tags: [requires_fcv_71]
  */
-load("jstests/libs/query_stats_utils.js");  // For verifyMetrics and getQueryStatsAggCmd.
-
-(function() {
-"use strict";
+import {getQueryStatsFindCmd, verifyMetrics} from "jstests/libs/query_stats_utils.js";
 
 // Turn on the collecting of queryStats metrics.
 let options = {
@@ -56,4 +53,3 @@ assert.eq(queryStatsEntry.metrics.firstResponseExecMicros, queryStatsEntry.metri
 verifyMetrics(queryStatsResults);
 
 MongoRunner.stopMongod(conn);
-}());
