@@ -1,10 +1,7 @@
 /**
  * Test query by namespace and uuid while collection changes are in the commit pending state.
  */
-(function() {
-"use strict";
-
-load("jstests/libs/fail_point_util.js");  // for 'configureFailPoint()'
+import {configureFailPoint} from "jstests/libs/fail_point_util.js";
 
 const afterWTCommitFP = "hangBeforePublishingCatalogUpdates";
 const beforeWTCommitFP = "hangAfterPreCommittingCatalogUpdates";
@@ -32,42 +29,25 @@ const nDocs = 5;
 // This ensures the database exists before performing any commands against it.
 assert.commandWorked(db.createCollection(dummyCollName));
 
-function listCollectionsEntryWithFilter(db, collName) {
-    return assert.commandWorked(db.runCommand({listCollections: 1, filter: {name: collName}}))
-        .cursor.firstBatch[0];
-}
-
-function listCollectionsEntryWithoutFilter(db, collName) {
-    let listCollectionsCursor = assert.commandWorked(db.runCommand({listCollections: 1}));
-    let collections = new DBCommandCursor(db, listCollectionsCursor).toArray();
-    return collections.find((coll) => coll.name == collName);
-}
-
-const listCollectionVariants = [listCollectionsEntryWithFilter, listCollectionsEntryWithoutFilter];
-
-function runTestCase(dbName,
-                     collName,
-                     parallelShellCommand,
-                     getUUIDBeforeCommand,
-                     fpName,
-                     getListCollectionsEntryFn) {
-    jsTest.log(`Test case with: ${getListCollectionsEntryFn.name}`);
-
+function runTestCase(dbName, collName, parallelShellCommand, getUUIDBeforeCommand, fpName) {
     let uuid;
     if (getUUIDBeforeCommand) {
-        uuid = getListCollectionsEntryFn(db, collName).info.uuid;
+        uuid = assert.commandWorked(db.runCommand({listCollections: 1, filter: {name: collName}}))
+                   .cursor.firstBatch[0]
+                   .info.uuid;
     }
 
-    // Pause before committing any catalog changes in the collection catalog (but after
-    // durably committed).
+    // Pause before committing any catalog changes in the collection catalog (but after durably
+    // committed).
     const failPoint = configureFailPoint(primary, fpName, {collectionNS: dbName + '.' + collName});
     const awaitResult = startParallelShell(parallelShellCommand, primary.port);
     failPoint.wait();
 
     if (!getUUIDBeforeCommand) {
-        let res = getListCollectionsEntryFn(db, collName);
-        if (res && res.info) {
-            uuid = res.info.uuid;
+        let res =
+            assert.commandWorked(db.runCommand({listCollections: 1, filter: {name: collName}}));
+        if (res.cursor.firstBatch[0] && res.cursor.firstBatch[0].info) {
+            uuid = res.cursor.firstBatch[0].info.uuid;
         }
     }
 
@@ -82,16 +62,14 @@ jsTest.log(
     "While a collection is commit pending for creation, we should return an empty version " +
     "of the collection. If done inside a transaction, we still return an empty version of the " +
     "collection since it's present in the transaction's WT snapshot.");
-
-function testDurableCommitPendingCreate(getListCollectionsEntryFn) {
+{
     setupCreateTest();
     let res =
         runTestCase(dbName,
                     createCollName,
                     `{db.getSiblingDB('${dbName}').runCommand({create: '${createCollName}'});}`,
                     false,
-                    afterWTCommitFP,
-                    getListCollectionsEntryFn);
+                    afterWTCommitFP);
 
     let nssRes = assert.commandWorked(db.runCommand({find: createCollName}));
     let nssDocCount = new DBCommandCursor(db, nssRes).itcount();
@@ -115,18 +93,16 @@ function testDurableCommitPendingCreate(getListCollectionsEntryFn) {
     res.fp.off();
     res.awaitResult();
 }
-listCollectionVariants.forEach(testDurableCommitPendingCreate);
 
 jsTest.log("Ensure that we cannot see the UUID and thus cannot find by UUID.");
-function testNonDurableCommitPendingCreate(getListCollectionsEntryFn) {
+{
     setupCreateTest();
     let res =
         runTestCase(dbName,
                     createCollName,
                     `{db.getSiblingDB('${dbName}').runCommand({create: '${createCollName}'});}`,
                     false,
-                    beforeWTCommitFP,
-                    getListCollectionsEntryFn);
+                    beforeWTCommitFP);
 
     let nssRes = assert.commandWorked(db.runCommand({find: createCollName}));
     let nssDocCount = new DBCommandCursor(db, nssRes).itcount();
@@ -136,7 +112,6 @@ function testNonDurableCommitPendingCreate(getListCollectionsEntryFn) {
     res.fp.off();
     res.awaitResult();
 }
-listCollectionVariants.forEach(testNonDurableCommitPendingCreate);
 
 function setupDropTest() {
     db[dropCollName].drop();
@@ -160,14 +135,13 @@ function setupRenameTest() {
 
 jsTest.log("Ensure that having a drop with a commit pending entry AFTER it has been made durable " +
            "makes the find fail with NamespaceNotFound.");
-function testDurableCommitPendingDrop(getListCollectionsEntryFn) {
+{
     setupDropTest();
     let res = runTestCase(dbName,
                           dropCollName,
                           `{db.getSiblingDB('${dbName}').runCommand({drop: '${dropCollName}'});}`,
                           true,
-                          afterWTCommitFP,
-                          getListCollectionsEntryFn);
+                          afterWTCommitFP);
 
     // Check that the results of queries are as expected.
     let nssRes = assert.commandWorked(db.runCommand({find: dropCollName}));
@@ -178,20 +152,18 @@ function testDurableCommitPendingDrop(getListCollectionsEntryFn) {
     res.fp.off();
     res.awaitResult();
 }
-listCollectionVariants.forEach(testDurableCommitPendingDrop);
 
 jsTest.log(
     "Check that having a commit pending entry BEFORE it has been made durable doesn't affect find " +
     "command when finding by UUID as it's still in a timeline before the drop has been made. That " +
     "is, it should see the collection before the drop.");
-function testNonDurableCommitPendingDrop(getListCollectionsEntryFn) {
+{
     setupDropTest();
     let res = runTestCase(dbName,
                           dropCollName,
                           `{db.getSiblingDB('${dbName}').runCommand({drop: '${dropCollName}'});}`,
                           true,
-                          beforeWTCommitFP,
-                          getListCollectionsEntryFn);
+                          beforeWTCommitFP);
 
     // Check that the results of queries are as expected.
     let nssRes = assert.commandWorked(db.runCommand({find: dropCollName}));
@@ -202,13 +174,12 @@ function testNonDurableCommitPendingDrop(getListCollectionsEntryFn) {
     res.fp.off();
     res.awaitResult();
 }
-listCollectionVariants.forEach(testNonDurableCommitPendingDrop);
 
 jsTest.log(
     "While a collection is commit pending for rename and it's been made durable, we should " +
     "find the new namespace (and the old one should be handled as a drop). By uuid, we should " +
     "find the new collection.");
-function testDurableCommitPendingRename(getListCollectionsEntryFn) {
+{
     setupRenameTest();
     let res =
         runTestCase(dbName,
@@ -216,8 +187,7 @@ function testDurableCommitPendingRename(getListCollectionsEntryFn) {
                     `{db.adminCommand({renameCollection: '${dbName}.${
                         renameSourceCollName}', to: '${dbName}.${renameDestinationCollName}'});}`,
                     true,
-                    afterWTCommitFP,
-                    getListCollectionsEntryFn);
+                    afterWTCommitFP);
 
     // Check that the results of queries are as expected.
     let sourceNssRes = assert.commandWorked(db.runCommand({find: renameSourceCollName}));
@@ -233,13 +203,12 @@ function testDurableCommitPendingRename(getListCollectionsEntryFn) {
     res.fp.off();
     res.awaitResult();
 }
-listCollectionVariants.forEach(testDurableCommitPendingRename);
 
 jsTest.log(
     "Ensure that a rename operation that's published changes that are not yet durable doesn't make " +
     "them visible to a snapshot taken before WT has committed the changes. Find should access the " +
     "old collection rather than the new one by UUID.");
-function testNonDurableCommitPendingRename(getListCollectionsEntryFn) {
+{
     setupRenameTest();
     let res =
         runTestCase(dbName,
@@ -247,8 +216,7 @@ function testNonDurableCommitPendingRename(getListCollectionsEntryFn) {
                     `{db.adminCommand({renameCollection: '${dbName}.${
                         renameSourceCollName}', to: '${dbName}.${renameDestinationCollName}'});}`,
                     true,
-                    beforeWTCommitFP,
-                    getListCollectionsEntryFn);
+                    beforeWTCommitFP);
 
     // Check that the results of queries are as expected as the snapshot should be from before
     // they've been made durable to WT.
@@ -265,7 +233,5 @@ function testNonDurableCommitPendingRename(getListCollectionsEntryFn) {
     res.fp.off();
     res.awaitResult();
 }
-listCollectionVariants.forEach(testNonDurableCommitPendingRename);
 
 replTest.stopSet();
-})();
