@@ -7,9 +7,6 @@
  *   requires_wiredtiger,
  * ]
  */
-(function() {
-'use strict';
-
 const rst = new ReplSetTest({
     nodes: [
         {
@@ -106,5 +103,4 @@ checkLog.containsJson(secondary, 21536, {
 // This ensures that the node is able to complete recovery and transition to SECONDARY.
 rst.awaitReplication();
 
-rst.stopSet();
-})();
+rst.stopSet();
\ No newline at end of file
