@@ -5,9 +5,6 @@
  *   requires_replication,
  * ]
  */
-(function() {
-"use strict";
-
 let replSet = new ReplSetTest({nodes: 2, oplogSize: 50});
 replSet.startSet();
 replSet.initiate();
@@ -22,26 +19,27 @@ const EB = 1024 * PB;
 assert.eq(primary.getDB('local').oplog.rs.stats().maxSize, 50 * MB);
 
 // Too small: 990MB
-assert.commandFailedWithCode(primary.getDB('admin').runCommand({replSetResizeOplog: 1, size: 900}),
-                             51024,  // IDL Error code for invalid fields
-                             "Expected replSetResizeOplog to fail because the size was too small");
+assert.commandFailedWithCode(
+    primary.getDB('admin').runCommand({replSetResizeOplog: 1, size: 900}),
+    [ErrorCodes.BadValue, 51024],  // getting BadValue when binary is > 7.1, else 51024
+    "Expected replSetResizeOplog to fail because the size was too small");
 
 // Way too small: -1GB
 assert.commandFailedWithCode(
     primary.getDB('admin').runCommand({replSetResizeOplog: 1, size: -1 * GB / MB}),
-    51024,  // IDL Error code for invalid fields
+    [ErrorCodes.BadValue, 51024],  // getting BadValue when binary is > 7.1, else 51024
     "Expected replSetResizeOplog to fail because the size was too small");
 
 // Too big: 8EB
 assert.commandFailedWithCode(
     primary.getDB('admin').runCommand({replSetResizeOplog: 1, size: 8 * EB / MB}),
-    51024,  // IDL Error code for invalid fields
+    [ErrorCodes.BadValue, 51024],  // getting BadValue when binary is > 7.1, else 51024
     "Expected replSetResizeOplog to fail because the size was too big");
 
 // Min Retention Hours not valid: -1hr
 assert.commandFailedWithCode(
     primary.getDB('admin').runCommand({replSetResizeOplog: 1, size: 990, minRetentionHours: -1}),
-    51024,  // IDL Error code for invalid fields
+    [ErrorCodes.BadValue, 51024],  // getting BadValue when binary is > 7.1, else 51024
     "Expected replSetResizeOplog to fail because the minimum retention hours was too low");
 
 // The maximum: 1PB
@@ -57,5 +55,4 @@ assert.commandWorked(
 
 assert.eq(primary.getDB('local').oplog.rs.stats().maxSize, 1 * PB);
 
-replSet.stopSet();
-})();
+replSet.stopSet();
\ No newline at end of file
