@@ -5,10 +5,7 @@
  *   uses_transactions,
  * ]
  */
-(function() {
-"use strict";
-
-load("jstests/libs/collection_drop_recreate.js");  // For assert[Drop|Create]Collection.
+import {assertDropAndRecreateCollection} from "jstests/libs/collection_drop_recreate.js";
 
 // Create a new single-node replica set, and ensure that it can support $changeStream.
 const rst = new ReplSetTest({nodes: 1});
@@ -115,5 +112,4 @@ csCursor = testCollection.watch([], {resumeAfter: resumePBRT});
 assert.docEq(csCursor.next(), txnEvent3);
 assert(!csCursor.hasNext());
 
-rst.stopSet();
-})();
+rst.stopSet();
\ No newline at end of file
