@@ -9,11 +9,8 @@
  *   requires_replication,
  * ]
  */
-(function() {
-"use strict";
-
-load("jstests/noPassthrough/libs/index_build.js");
-load("jstests/libs/columnstore_util.js");  // For setUpServerForColumnStoreIndexTest.
+import {setUpServerForColumnStoreIndexTest} from "jstests/libs/columnstore_util.js";
+import {ResumableIndexBuildTest} from "jstests/noPassthrough/libs/index_build.js";
 
 const dbName = "test";
 
@@ -61,5 +58,4 @@ if (columnstoreEnabled) {
     runTests([{a: 1, b: 1}, {a: 2, b: 2}], [{"$**": "columnstore"}, {b: 1}], "_columnstore");
 }
 
-rst.stopSet();
-})();
+rst.stopSet();
\ No newline at end of file
