@@ -8,11 +8,8 @@
  *   requires_replication,
  * ]
  */
-(function() {
-"use strict";
-
-load("jstests/noPassthrough/libs/index_build.js");
-load("jstests/libs/columnstore_util.js");  // For setUpServerForColumnStoreIndexTest.
+import {setUpServerForColumnStoreIndexTest} from "jstests/libs/columnstore_util.js";
+import {ResumableIndexBuildTest} from "jstests/noPassthrough/libs/index_build.js";
 
 const dbName = "test";
 
@@ -112,5 +109,4 @@ runTests(
     ],
     ["bulk load", "drain writes"],
     [{skippedPhaseLogID: 20391}, {skippedPhaseLogID: 20392}]);
-rst.stopSet();
-})();
\ No newline at end of file
+rst.stopSet();
\ No newline at end of file
