@@ -1,9 +1,6 @@
 /*
  * Verify behavior of retryable write commands on a standalone mongod.
  */
-(function() {
-"use strict";
-
 const standalone = MongoRunner.runMongod();
 const testDB = standalone.getDB("test");
 
@@ -14,5 +11,4 @@ assert.commandFailedWithCode(
     ErrorCodes.IllegalOperation,
     "expected command with transaction number to fail on standalone mongod");
 
-MongoRunner.stopMongod(standalone);
-}());
+MongoRunner.stopMongod(standalone);
\ No newline at end of file
