@@ -1,11 +1,9 @@
 // Verifies currentOp returns the expected fields for idle and active transactions in basic cases.
 // More cases are covered in unit tests.
 // @tags: [uses_transactions, uses_atclustertime]
-(function() {
-"use strict";
-
-load("jstests/libs/curop_helpers.js");   // For waitForCurOpByFailPoint().
-load("jstests/libs/parallelTester.js");  // for Thread.
+import {waitForCurOpByFailPointNoNS} from "jstests/libs/curop_helpers.js";
+import {configureFailPoint} from "jstests/libs/fail_point_util.js";
+import {Thread} from "jstests/libs/parallelTester.js";
 
 function verifyCurrentOpFields(res, isActive) {
     // Verify top level fields relevant to transactions. Note this does not include every field, so
@@ -126,8 +124,7 @@ jsTest.log("Inactive transaction.");
 
 jsTest.log("Active transaction.");
 (() => {
-    assert.commandWorked(st.rs0.getPrimary().adminCommand(
-        {configureFailPoint: "waitInFindBeforeMakingBatch", mode: "alwaysOn"}));
+    const fp = configureFailPoint(st.rs0.getPrimary(), "waitInFindBeforeMakingBatch");
 
     const txnThread = new Thread(function(host, dbName, collName) {
         const mongosConn = new Mongo(host);
@@ -150,11 +147,9 @@ jsTest.log("Active transaction.");
     const res = getCurrentOpForFilter(st, {"command.comment": "active_txn_find"});
     verifyCurrentOpFields(res, true /* isActive */);
 
-    assert.commandWorked(st.rs0.getPrimary().adminCommand(
-        {configureFailPoint: "waitInFindBeforeMakingBatch", mode: "off"}));
+    fp.off();
     txnThread.join();
 })();
 
 session.endSession();
 st.stop();
-}());
