@@ -1,8 +1,4 @@
 // Tests the _runningMongoChildProcessIds shell built-in.
-// @tags: [live_record_incompatible]
-
-(function() {
-'use strict';
 
 /**
  * @param {NumberLong[]} expected pids
@@ -47,5 +43,4 @@ try {
 })();
 
 // empty at the end
-assertRunningMongoChildProcessIds([]);
-})();
+assertRunningMongoChildProcessIds([]);
\ No newline at end of file
