@@ -3,13 +3,15 @@
  * demonstrates that unlike the classic multiplanner, the SBE multiplanner's end condition is by
  * default not proportional to the size of the collection.
  *
+ * TODO SERVER-83887 This entire test can be deleted when we remove the "Classic runtime planning
+ * for SBE" feature flag.
+ *
  * @tags: [
  *   # This test assumes that SBE is being used for most queries.
  *   featureFlagSbeFull,
  * ]
  */
-(function() {
-"use strict";
+import {FeatureFlagUtil} from "jstests/libs/feature_flag_util.js";
 
 const numDocs = 1000;
 const dbName = "sbe_multiplanner_db";
@@ -25,6 +27,13 @@ const trialLengthFromWorksKnob = 0.1 * numDocs;
 
 const conn = MongoRunner.runMongod({});
 assert.neq(conn, null, "mongod failed to start");
+
+if (FeatureFlagUtil.isPresentAndEnabled(conn, "ClassicRuntimePlanningForSbe")) {
+    jsTestLog("Skipping the test because SBE multi planner won't be used");
+    MongoRunner.stopMongod(conn);
+    quit();
+}
+
 const db = conn.getDB(dbName);
 const coll = db[collName];
 
@@ -116,4 +125,3 @@ allPlans = getAllPlansExecution("2");
 verifySbeNumReads(allPlans, trialLengthFromCollFrac);
 
 MongoRunner.stopMongod(conn);
-}());
