@@ -5,15 +5,11 @@
  * threshold is reached by adding entries to the SBE plan cache, the classic cache will start
  * stripping debug info even though the size of the classic cache may be below the threshold.
  * @tags: [
- *   # TODO SERVER-67607: Test plan cache with CQF enabled.
- *   cqf_incompatible,
  *   featureFlagSbeFull
  * ]
  */
-(function() {
-"use strict";
-
-load("jstests/libs/analyze_plan.js");  // For getPlanCacheKeyFromShape
+import {getPlanCacheKeyFromShape} from "jstests/libs/analyze_plan.js";
+import {getPlanCacheSize} from "jstests/libs/plan_cache_utils.js";
 
 const conn = MongoRunner.runMongod({});
 assert.neq(conn, null, "mongod failed to start");
@@ -23,7 +19,7 @@ function createTestCollection(collectionName) {
     const coll = db[collectionName];
     coll.drop();
     // Create multiple indexes to ensure we go through the multi-planner.
-    assert.commandWorked(coll.createIndexes([{a: 1}, {b: 1}, {a: 1, b: 1}, {b: 1, a: 1}]));
+    assert.commandWorked(coll.createIndexes([{a: 1}, {b: 1}, {a: -1, b: -1}, {b: -1, a: -1}]));
     return coll;
 }
 
@@ -52,23 +48,19 @@ function assertCacheEntryIsMissingDebugInfo(coll, queryShape) {
     }
 }
 
-function getPlanCacheSize() {
-    return db.runCommand({serverStatus: 1}).metrics.query.planCacheTotalSizeEstimateBytes;
-}
-
 // Set a large value to internalQueryCacheMaxSizeBytesBeforeStripDebugInfo to make sure that Debug
 // Info wouldn't be stripped off.
 assert.commandWorked(db.adminCommand(
     {setParameter: 1, internalQueryCacheMaxSizeBytesBeforeStripDebugInfo: 536870912}));
 
-const initialPlanCacheSize = getPlanCacheSize();
+const initialPlanCacheSize = getPlanCacheSize(db);
 
 // Add some entries to SBE Plan Cache and make sure that the global planCacheSize metric is affected
 // by the inserted entries.
 const sbeColl = createTestCollection("sbe");
 assert.eq(0, sbeColl.find({a: 0}).itcount());
 assert.eq(0, sbeColl.find({a: 2, b: 4}).itcount());
-const planCacheSizeAfterSbeStep = getPlanCacheSize();
+const planCacheSizeAfterSbeStep = getPlanCacheSize(db);
 assert.lt(initialPlanCacheSize, planCacheSizeAfterSbeStep);
 
 // Force classic plan cache.
@@ -82,7 +74,7 @@ const classicColl = createTestCollection("classic");
 // affected by the inserted entry as well as the entry contains DebugInfo.
 assert.eq(0, classicColl.find({a: 0}).itcount());
 assertCacheEntryHasDebugInfo(classicColl, {a: 0});
-const planCacheSizeAfterClassicStep = getPlanCacheSize();
+const planCacheSizeAfterClassicStep = getPlanCacheSize(db);
 assert.lt(planCacheSizeAfterSbeStep, planCacheSizeAfterClassicStep);
 
 // Set a smaller internalQueryCacheMaxSizeBytesBeforeStripDebugInfo to make sure that next inserted
@@ -97,4 +89,3 @@ assert.eq(0, classicColl.find({a: 2, b: 4}).itcount());
 assertCacheEntryIsMissingDebugInfo(classicColl, {a: 2, b: 4});
 
 MongoRunner.stopMongod(conn);
-}());
