@@ -1,8 +1,5 @@
 // Test --setShellParameter CLI switch.
 
-(function() {
-'use strict';
-
 function test(ssp, succeed) {
     const result = runMongoProgram('mongo', '--setShellParameter', ssp, '--nodb', '--eval', ';');
     assert.eq(
@@ -16,5 +13,4 @@ test('disabledSecureAllocatorDomains=foo', true);
 test('enableTestCommands=1', false);
 
 // Unknown
-test('theAnswerToTheQuestionOfLifeTheUniverseAndEverything=42', false);
-})();
+test('theAnswerToTheQuestionOfLifeTheUniverseAndEverything=42', false);
\ No newline at end of file
