@@ -4,8 +4,6 @@
  *
  * @tags: [requires_sharding]
  */
-(function() {
-"use strict";
 const st = new ShardingTest({shards: 1, mongos: 1});
 const dbName = "db";
 const db = st.getDB(dbName);
@@ -19,5 +17,4 @@ assert.commandWorked(st.s.adminCommand({shardCollection: coll.getFullName(), key
 assert.commandFailed(coll.runCommand("distinct", {help: helpFn, foo: 1}));
 assert.commandFailed(coll.runCommand(
     {explain: {distinct: coll.getName(), help: helpFn, foo: 1}, verbosity: 'queryPlanner'}));
-st.stop();
-})();
\ No newline at end of file
+st.stop();
\ No newline at end of file
