@@ -2,8 +2,6 @@
  * Tests that bsonObjToArray converts BSON objects to JS arrays.
  */
 
-(function() {
-'use strict';
 const conn = MongoRunner.runMongod();
 const db = conn.getDB('test');
 const tests = [];
@@ -29,5 +27,4 @@ tests.forEach((test) => {
     test();
 });
 
-MongoRunner.stopMongod(conn);
-})();
\ No newline at end of file
+MongoRunner.stopMongod(conn);
\ No newline at end of file
