@@ -2,9 +2,6 @@
  * Tests for the command assertion functions in mongo/shell/assert.js.
  */
 
-(function() {
-"use strict";
-
 const conn = MongoRunner.runMongod();
 const db = conn.getDB("commandAssertions");
 const kFakeErrCode = 1234567890;
@@ -409,5 +406,4 @@ tests.forEach((test) => {
 });
 
 /* cleanup */
-MongoRunner.stopMongod(conn);
-})();
+MongoRunner.stopMongod(conn);
\ No newline at end of file
