@@ -1,9 +1,6 @@
 /**
  * Ensure that we allow mongod to shutdown cleanly while being fsync locked.
  */
-(function() {
-"use strict";
-
 let conn = MongoRunner.runMongod();
 let db = conn.getDB("test");
 
@@ -11,5 +8,4 @@ for (let i = 0; i < 10; i++) {
     assert.commandWorked(db.adminCommand({fsync: 1, lock: 1}));
 }
 
-MongoRunner.stopMongod(conn, MongoRunner.EXIT_CLEAN, {skipValidation: true});
-}());
+MongoRunner.stopMongod(conn, MongoRunner.EXIT_CLEAN, {skipValidation: true});
\ No newline at end of file
