@@ -3,9 +3,7 @@
  *
  * This test was originally designed to reproduce SERVER-53760.
  */
-(function() {
-"use strict";
-load('jstests/libs/analyze_plan.js');  // For 'getAggPlanStage()'.
+import {getAggPlanStages} from "jstests/libs/analyze_plan.js";
 
 const conn = MongoRunner.runMongod();
 assert.neq(null, conn, "mongod was unable to start up");
@@ -58,16 +56,40 @@ function createPipeline(collection) {
 }
 
 const explain = createPipeline(coll.explain("executionStats"));
+
+// Returns the (first) value of the field named 'key' in the object 'obj' if it exists nested at any
+// depth, else returns "undefined".
+function findKey(key, obj) {
+    if ((typeof obj === "undefined") || (obj === null)) {
+        return;
+    }
+    if (obj.hasOwnProperty(key)) {
+        return obj[key];
+    }
+    for (let prop in obj) {
+        if (typeof obj[prop] === "object") {
+            let result = findKey(key, obj[prop]);
+            if (typeof result !== "undefined") {
+                return result;
+            }
+        }
+    }
+}  // function findKey
+
+let dataBytesSorted;
 const sortStages = getAggPlanStages(explain, "$sort");
-
-assert.eq(sortStages.length, 1, explain);
-const sort = sortStages[0];
-const dataBytesSorted = sort["totalDataSizeSortedBytesEstimate"];
-
+if (sortStages.length == 0) {
+    // SBE executed sbe::SortStage.
+    dataBytesSorted = findKey("totalDataSizeSorted", explain);
+} else {
+    // Classic engine executed DocumentSourceSort.
+    assert.eq(sortStages.length, 1, explain);
+    const sortStage = sortStages[0];
+    dataBytesSorted = sortStage["totalDataSizeSortedBytesEstimate"];
+}
 // The total data size sorted is no greater than 3x the total size of all documents sorted.
 assert.lt(dataBytesSorted, 3 * totalSize, explain);
 
 assert.eq(createPipeline(coll).toArray(), [{_id: 0, sumTop900UniqueValues: 94550}], explain);
 
-MongoRunner.stopMongod(conn);
-})();
+MongoRunner.stopMongod(conn);
\ No newline at end of file
