@@ -4,14 +4,10 @@
 // @tags: [
 //   requires_persistence,
 // ]
-
-(function() {
-"use strict";
-
-load("jstests/libs/analyze_plan.js");     // For getPlanStages, getQueryPlanner, getWinningPlan
-load("jstests/libs/fail_point_util.js");  // For configureFailPoint
-load("jstests/libs/parallel_shell_helpers.js");  // For funWithArgs
-load("jstests/libs/sbe_util.js");                // For checkSbeFullyEnabled
+import {getPlanStages, getQueryPlanner, getWinningPlan} from "jstests/libs/analyze_plan.js";
+import {configureFailPoint} from "jstests/libs/fail_point_util.js";
+import {funWithArgs} from "jstests/libs/parallel_shell_helpers.js";
+import {checkSbeFullyEnabled} from "jstests/libs/sbe_util.js";
 
 const conn = MongoRunner.runMongod();
 const db = conn.getDB("test");
@@ -25,6 +21,8 @@ const isSbeEnabled = checkSbeFullyEnabled(db);
 
 // Set up relevant query knobs so that the query will spill for every document.
 // No batching in document source cursor
+assert.commandWorked(
+    db.adminCommand({setParameter: 1, internalDocumentSourceCursorInitialBatchSize: 1}));
 assert.commandWorked(
     db.adminCommand({setParameter: 1, internalDocumentSourceCursorBatchSizeBytes: 1}));
 // Spilling memory threshold for $sort
@@ -36,8 +34,10 @@ assert.commandWorked(
 assert.commandWorked(db.adminCommand(
     {setParameter: 1, internalQuerySlotBasedExecutionHashAggApproxMemoryUseInBytesBeforeSpill: 1}));
 // Spilling memory threshold for $setWindowFields
-assert.commandWorked(
-    db.adminCommand({setParameter: 1, internalDocumentSourceSetWindowFieldsMaxMemoryBytes: 232}));
+assert.commandWorked(db.adminCommand({
+    setParameter: 1,
+    internalDocumentSourceSetWindowFieldsMaxMemoryBytes: isSbeEnabled ? 129 : 232
+}));
 // Spilling memory threshold for $lookup
 assert.commandWorked(db.adminCommand({
     setParameter: 1,
@@ -144,19 +144,18 @@ function testSpillingMetrics({stage, expectedSpillingMetrics, expectedSbeSpillin
 testSpillingMetrics({
     stage: stages['sort'],
     expectedSpillingMetrics: {spills: 19, spilledBytes: 654},
-    expectedSbeSpillingMetrics: {spills: 19, spilledBytes: 951}
+    expectedSbeSpillingMetrics: {spills: 19, spilledBytes: 935}
 });
 testSpillingMetrics({
     stage: stages['group'],
     expectedSpillingMetrics: {spills: 10, spilledBytes: 410},
     expectedSbeSpillingMetrics: {spills: 10, spilledBytes: 450}
 });
-if (!isSbeEnabled) {
-    testSpillingMetrics({
-        stage: stages['setWindowFields'],
-        expectedSpillingMetrics: {spills: 10, spilledBytes: 180},
-    });
-}
+testSpillingMetrics({
+    stage: stages['setWindowFields'],
+    expectedSpillingMetrics: {spills: 10, spilledBytes: 180},
+    expectedSbeSpillingMetrics: {spills: 9, spilledBytes: 500},
+});
 if (isSbeEnabled) {
     testSpillingMetrics({
         stage: stages['lookup'],
@@ -199,4 +198,3 @@ assert.commandFailedWithCode(
     ErrorCodes.OutOfDiskSpace);
 
 MongoRunner.stopMongod(conn);
-})();
