@@ -12,10 +12,7 @@
  * ]
  */
 
-(function() {
-"use strict";
-
-load("jstests/libs/fail_point_util.js");
+import {configureFailPoint} from "jstests/libs/fail_point_util.js";
 
 const rst = new ReplSetTest({name: jsTest.name(), nodes: 2});
 rst.startSet();
@@ -94,5 +91,4 @@ primary.setSecondaryOk();
 const find = primary.getDB(dbName).getCollection(collName).findOne({_id: "writeAfterHole"});
 assert.eq(find, null);
 
-rst.stopSet();
-})();
+rst.stopSet();
\ No newline at end of file
