@@ -1,9 +1,7 @@
 /**
  * This test makes makes sure Thread works with --enableJavaScriptProtection
  */
-(function() {
-'use strict';
-load('jstests/libs/parallelTester.js');
+import {Thread} from "jstests/libs/parallelTester.js";
 
 function testThread(threadType) {
     function threadFn(args) {
@@ -41,5 +39,4 @@ function testThread(threadType) {
 }
 
 // Test the Thread class
-testThread(Thread);
-}());
+testThread(Thread);
\ No newline at end of file
