@@ -2,9 +2,6 @@
 //   requires_replication,
 //   requires_sharding,
 // ]
-(function() {
-'use strict';
-
 // This test makes assertions about the number of sessions, which are not compatible with
 // implicit sessions.
 TestData.disableImplicitSessions = true;
@@ -39,6 +36,8 @@ function Sharding(lifetime) {
             rs: true,
             rsOptions: {setParameter: {TransactionRecordMinimumLifetimeMinutes: lifetime}},
             rs0: {nodes: 1},
+            mongosOptions:
+                {setParameter: {'failpoint.skipClusterParameterRefresh': "{'mode':'alwaysOn'}"}}
         },
     });
 
@@ -52,6 +51,7 @@ function Sharding(lifetime) {
 
     // Remove the session created by the above shardCollection
     this.st.s.getDB("config").system.sessions.remove({});
+    this.getTransactionConn().getDB("config").transactions.remove({});
 }
 
 Sharding.prototype.stop = function() {
@@ -162,4 +162,3 @@ Fixture.prototype.stop = function() {
         fixture.stop();
     }
 });
-})();
