@@ -2,11 +2,8 @@
  * Verifies the transitionFromDedicatedConfigServer feature flag guards running the config shard
  * transition commands.
  *
- * @tags: [requires_fcv_70]
+ * @tags: [requires_fcv_80]
  */
-(function() {
-"use strict";
-
 delete TestData.setParameters.featureFlagTransitionToCatalogShard;
 delete TestData.setParametersMongos.featureFlagTransitionToCatalogShard;
 
@@ -21,16 +18,13 @@ const st = new ShardingTest({
 });
 
 // None of the transition commands can be run on mongos or the config server.
-assert.commandFailedWithCode(st.s.adminCommand({transitionFromDedicatedConfigServer: 1}),
-                             ErrorCodes.CommandNotFound);
+assert.commandFailedWithCode(st.s.adminCommand({transitionFromDedicatedConfigServer: 1}), 8454804);
 assert.commandFailedWithCode(st.s.adminCommand({transitionToDedicatedConfigServer: 1}), 7368401);
 
 const configPrimary = st.configRS.getPrimary();
 assert.commandFailedWithCode(
-    configPrimary.adminCommand({_configsvrTransitionFromDedicatedConfigServer: 1}),
-    ErrorCodes.CommandNotFound);
+    configPrimary.adminCommand({_configsvrTransitionFromDedicatedConfigServer: 1}), 8454803);
 assert.commandFailedWithCode(
     configPrimary.adminCommand({_configsvrTransitionToDedicatedConfigServer: 1}), 7368402);
 
 st.stop();
-}());
