@@ -1,8 +1,6 @@
 // Test that the TTL monitor will correctly use TTL indexes that are also partial indexes.
 // SERVER-17984.
-(function() {
-"use strict";
-load("jstests/libs/ttl_util.js");
+import {TTLUtil} from "jstests/libs/ttl_util.js";
 
 // Launch mongod with shorter TTL monitor sleep interval.
 var runner = MongoRunner.runMongod({setParameter: "ttlMonitorSleepSecs=1"});
@@ -26,5 +24,4 @@ assert.eq(0,
           "Wrong number of documents in partial index, after TTL monitor run");
 assert.eq(
     1, coll.find().itcount(), "Wrong number of documents in collection, after TTL monitor run");
-MongoRunner.stopMongod(runner);
-})();
+MongoRunner.stopMongod(runner);
\ No newline at end of file
