@@ -6,9 +6,6 @@
  *   requires_replication,
  * ]
  */
-(function() {
-'use strict';
-
 const replTest = new ReplSetTest({nodes: 2});
 replTest.startSet();
 replTest.initiate();
@@ -70,5 +67,4 @@ db.c.drop();
     assert.commandFailedWithCode(s1.abortTransaction_forTesting(), ErrorCodes.NoSuchTransaction);
 }
 
-replTest.stopSet();
-})();
+replTest.stopSet();
\ No newline at end of file
