@@ -2,8 +2,6 @@
  * This tests that errors are logged when unindexing _id finds evidence of corruption, the server
  * does not crash, and the appropriate error is returned.
  */
-(function() {
-
 const replSet = new ReplSetTest({nodes: 1});
 replSet.startSet();
 replSet.initiate();
@@ -34,5 +32,4 @@ assert.soonNoExcept(() => {
     return true;
 });
 
-replSet.stopSet();
-})();
+replSet.stopSet();
\ No newline at end of file
