@@ -8,9 +8,6 @@
  *   requires_replication,
  * ]
  */
-(function() {
-"use strict";
-
 const rst = new ReplSetTest({name: jsTestName(), nodes: 1});
 rst.startSet();
 rst.initiate();
@@ -285,5 +282,4 @@ for (let result of results) {
     assert.eq(result.mergeCT, result.aggCT);
 }
 
-rst.stopSet();
-}());
+rst.stopSet();
\ No newline at end of file
