@@ -1,9 +1,6 @@
 // Verify that the update system correctly rejects invalid entries during post-image validation.
 // @tags: [
 // ]
-(function() {
-"use strict";
-
 const conn = MongoRunner.runMongod();
 assert.neq(null, conn, "mongod was unable to start up");
 
@@ -22,5 +19,4 @@ assert.commandWorked(testDB.coll.insert({_id: 1, a: [0]}));
 assert.commandWorked(testDB.coll.update({_id: 1}, {$set: {"a.00": {$ref: "coll", $db: "test"}}}));
 assert.docEq({_id: 1, a: [{$ref: "coll", $db: "test"}]}, testDB.coll.findOne({_id: 1}));
 
-MongoRunner.stopMongod(conn);
-}());
+MongoRunner.stopMongod(conn);
\ No newline at end of file
