@@ -1,9 +1,6 @@
 /**
  * Test that validate detects and properly reports multikey inconsistencies.
  */
-(function() {
-"use strict";
-
 const baseName = "validate_multikey_failures";
 const dbpath = MongoRunner.dataPath + baseName + "/";
 let conn = MongoRunner.runMongod({dbpath: dbpath});
@@ -24,6 +21,11 @@ const enableMultikeyUpdate = () => {
         conn.adminCommand({configureFailPoint: "skipUpdateIndexMultikey", mode: "off"}));
 };
 
+function checkValidateLogs() {
+    assert(checkLog.checkContainsWithAtLeastCountJson(
+        conn, 7463100, {"spec": {"v": 2, "key": {"_id": 1}, "name": "_id_"}}, 1));
+}
+
 // Test that multiple keys suggest index should be marked multikey.
 resetCollection();
 disableMultikeyUpdate();
@@ -69,5 +71,4 @@ assert(checkLog.checkContainsWithAtLeastCountJson(conn, 7556100, {"indexName": "
 assert(checkLog.checkContainsWithAtLeastCountJson(conn, 7556101, {"indexKey": {"a.b": 6}}, 1));
 assert(checkLog.checkContainsWithAtLeastCountJson(conn, 5367500, {"index": "a.b_1"}, 1));
 
-MongoRunner.stopMongod(conn, null, {skipValidation: true});
-})();
\ No newline at end of file
+MongoRunner.stopMongod(conn, null, {skipValidation: true});
\ No newline at end of file
