@@ -2,9 +2,6 @@
  * Tests that out-of-order keys are detected by validation during both the collection and index scan
  * phases.
  */
-(function() {
-"use strict";
-
 const rst = new ReplSetTest({nodes: 1});
 rst.startSet();
 rst.initiate();
@@ -33,5 +30,4 @@ assert(!res.valid);
 assert.commandWorked(
     primary.adminCommand({configureFailPoint: "failIndexKeyOrdering", mode: "off"}));
 
-rst.stopSet();
-})();
+rst.stopSet();
\ No newline at end of file
