@@ -1,10 +1,7 @@
 // Test collection validation related server parameter settings on server startup
 // and via the setParameter command.
 
-(function() {
-'use strict';
-
-load("jstests/noPassthrough/libs/server_parameter_helpers.js");
+import {testNumericServerParameter} from "jstests/noPassthrough/libs/server_parameter_helpers.js";
 
 // Valid parameter values are in the range [0, infinity).
 testNumericServerParameter('maxValidateMBperSec',
@@ -26,5 +23,4 @@ testNumericServerParameter('maxValidateMemoryUsageMB',
                            true /*hasLowerBound*/,
                            0 /*lowerOutOfBounds*/,
                            false /*hasUpperBound*/,
-                           "unused" /*upperOutOfBounds*/);
-})();
+                           "unused" /*upperOutOfBounds*/);
\ No newline at end of file
