@@ -14,9 +14,6 @@
 //     <repeat for all other views that had to be resolved for the query>
 //   }
 // ],
-(function() {
-'use strict';
-
 function resetProfiler(db) {
     assert.commandWorked(db.setProfilingLevel(0, {slowms: 0}));
     db.system.profile.drop();
@@ -96,5 +93,4 @@ assertResolvedView(resolvedViews[0], `${db.getName()}.a_view`, 2, 2);
 assertResolvedView(resolvedViews[1], `${db.getName()}.c_view`, 3, 2);
 
 assert.commandWorked(db.dropDatabase());
-MongoRunner.stopMongod(conn);
-})();
+MongoRunner.stopMongod(conn);
\ No newline at end of file
