@@ -7,9 +7,6 @@
  *  no_selinux
  * ]
  */
-(function() {
-'use strict';
-
 let dbTest = db.getSiblingDB('create_encrypted_collection_db');
 
 dbTest.basic.drop();
@@ -28,5 +25,4 @@ const sampleEncryptedFields = {
 assert.commandFailedWithCode(
     db.runCommand({create: "basic", encryptedFields: sampleEncryptedFields}),
     6346402,
-    "Create with encryptedFields passed on standalone");
-}());
+    "Create with encryptedFields passed on standalone");
\ No newline at end of file
