@@ -1,10 +1,10 @@
 // SERVER-2662 - drop client cursor in a context where query will yield frequently
 
-t = db.jstests_slowNightly_explain1;
+let t = db.jstests_slowNightly_explain1;
 t.drop();
 
 // Periodically drops the collection, invalidating client cursors for s2's operations.
-s1 = startParallelShell(function() {
+let s1 = startParallelShell(function() {
     t = db.jstests_slowNightly_explain1;
     for (var i = 0; i < 80; ++i) {
         t.drop();
@@ -17,11 +17,11 @@ s1 = startParallelShell(function() {
 });
 
 // Query repeatedly.
-s2 = startParallelShell(function() {
+let s2 = startParallelShell(function() {
     t = db.jstests_slowNightly_explain1;
     for (var i = 0; i < 500; ++i) {
         try {
-            z = t.find({x: {$gt: 0}, y: 1}).explain();
+            let z = t.find({x: {$gt: 0}, y: 1}).explain();
             t.count({x: {$gt: 0}, y: 1});
         } catch (e) {
         }
@@ -29,7 +29,7 @@ s2 = startParallelShell(function() {
 });
 
 // Put pressure on s2 to yield more often.
-s3 = startParallelShell(function() {
+let s3 = startParallelShell(function() {
     t = db.jstests_slowNightly_explain1;
     for (var i = 0; i < 200; ++i) {
         t.validate({scandata: true});
