@@ -8,9 +8,6 @@
  * ]
  */
 
-(function() {
-"use strict";
-
 const forceCheckpoint = () => {
     assert.commandWorked(db.fsyncLock());
     assert.commandWorked(db.fsyncUnlock());
@@ -44,5 +41,4 @@ assert.gt(collStats.freeStorageSize, 0);
 let dbStats = assert.commandWorked(testDB.stats({freeStorage: 1}));
 assert(dbStats.hasOwnProperty("freeStorageSize"), tojson(dbStats));
 assert.gt(dbStats.freeStorageSize, 0);
-assert.eq(dbStats.freeStorageSize, collStats.freeStorageSize);
-})();
+assert.eq(dbStats.freeStorageSize, collStats.freeStorageSize);
\ No newline at end of file
