@@ -1,8 +1,3 @@
-// Tests that the validate command works with MinKey and MaxKey.
-
-'use strict';
-
-(function() {
 var t = db.index_boundary_values_validate;
 t.drop();
 
@@ -27,4 +22,3 @@ assert.commandWorked(res);
 assert.eq(res.nrecords, 8, 'the collection had an unexpected number of records:\n' + tojson(res));
 assert.eq(res.nIndexes, 5, 'the collection had an unexpected number of indexes:\n' + tojson(res));
 assert.eq(res.valid, true, 'the collection failed validation:\n' + tojson(res));
-})();
