@@ -3,19 +3,21 @@
  * oscillates. It achieves this by creating two indexes, A and B, on a collection, and interleaving
  * queries which are "ideal" for index A with queries that are "ideal" for index B.
  */
-(function() {
-"use strict";
+import {
+    getCachedPlan,
+    getPlanCacheKeyFromShape,
+    getPlanCacheShapeHashFromObject,
+    getPlanStage
+} from "jstests/libs/analyze_plan.js";
+import {assertDropAndRecreateCollection} from "jstests/libs/collection_drop_recreate.js";
+import {checkSbeFullFeatureFlagEnabled} from "jstests/libs/sbe_util.js";
 
-load('jstests/libs/analyze_plan.js');              // For getPlanStage().
-load("jstests/libs/collection_drop_recreate.js");  // For assert[Drop|Create]Collection.
-load("jstests/libs/sbe_util.js");                  // For checkSbeFullyEnabled.
-
-const isSbeEnabled = checkSbeFullyEnabled(db);
+const isSbePlanCacheEnabled = checkSbeFullFeatureFlagEnabled(db);
 
 let coll = assertDropAndRecreateCollection(db, "plan_cache_replanning");
 
 function getReplannedMetric() {
-    const planCacheType = isSbeEnabled ? "sbe" : "classic";
+    const planCacheType = isSbePlanCacheEnabled ? "sbe" : "classic";
     return assert.commandWorked(db.serverStatus()).metrics.query.planCache[planCacheType].replanned;
 }
 
@@ -30,13 +32,13 @@ function getCachedPlanForQuery(filter) {
  * Asserts that the plan contained in the plan cache 'entry' is an index scan plan over the index
  * with the given 'indexName'.
  *
- * Also verifies that the query hash matches the provided 'expectedQueryHash'.
+ * Also verifies that the 'planCacheShapeHash' matches the provided 'expectedPlanCacheShapeHash'.
  */
-function assertPlanHasIxScanStage(entry, indexName, expectedQueryHash) {
-    assert.eq(entry.queryHash, expectedQueryHash, entry);
+function assertPlanHasIxScanStage(entry, indexName, expectedPlanCacheShapeHash) {
+    assert.eq(entry.planCacheShapeHash, expectedPlanCacheShapeHash, entry);
 
     const cachedPlan = getCachedPlan(entry.cachedPlan);
-    if (isSbeEnabled) {
+    if (isSbePlanCacheEnabled) {
         // The $planCacheStats output for the SBE plan cache only contains an debug string
         // representation of the execution plan. Rather than parse this string, we just check that
         // the index name appears somewhere in the plan.
@@ -84,17 +86,17 @@ assert.eq(1, coll.find(bIndexQuery).itcount());
 
 // The plan cache should now hold an inactive entry.
 let entry = getCachedPlanForQuery(bIndexQuery);
-let queryHash = entry.queryHash;
+let planCacheShapeHash = getPlanCacheShapeHashFromObject(entry);
 let entryWorks = entry.works;
 assert.eq(entry.isActive, false);
-assertPlanHasIxScanStage(entry, "b_1", queryHash);
+assertPlanHasIxScanStage(entry, "b_1", planCacheShapeHash);
 
 // Re-run the query. The inactive cache entry should be promoted to an active entry.
 assert.eq(1, coll.find(bIndexQuery).itcount());
 entry = getCachedPlanForQuery(bIndexQuery);
 assert.eq(entry.isActive, true);
 assert.eq(entry.works, entryWorks);
-assertPlanHasIxScanStage(entry, "b_1", queryHash);
+assertPlanHasIxScanStage(entry, "b_1", planCacheShapeHash);
 
 // Now we will attempt to oscillate the cache entry by interleaving queries which should use the
 // {a:1} and {b:1} index. When the plan using the {b: 1} index is in the cache, running a query
@@ -108,21 +110,21 @@ let replannedMetric = getReplannedMetric();
 assert.eq(1, coll.find(aIndexQuery).itcount());
 entry = getCachedPlanForQuery(aIndexQuery);
 assert.eq(entry.isActive, true);
-assertPlanHasIxScanStage(entry, "a_1", queryHash);
+assertPlanHasIxScanStage(entry, "a_1", planCacheShapeHash);
 assert.eq(replannedMetric + 1, getReplannedMetric());
 
 // Run the query which should use the {b: 1} index.
 assert.eq(1, coll.find(bIndexQuery).itcount());
 entry = getCachedPlanForQuery(bIndexQuery);
 assert.eq(entry.isActive, true);
-assertPlanHasIxScanStage(entry, "b_1", queryHash);
+assertPlanHasIxScanStage(entry, "b_1", planCacheShapeHash);
 assert.eq(replannedMetric + 2, getReplannedMetric());
 
 // The {b: 1} plan is again in the cache. Run the query which should use the {a: 1} index.
 assert.eq(1, coll.find(aIndexQuery).itcount());
 entry = getCachedPlanForQuery(aIndexQuery);
 assert.eq(entry.isActive, true);
-assertPlanHasIxScanStage(entry, "a_1", queryHash);
+assertPlanHasIxScanStage(entry, "a_1", planCacheShapeHash);
 assert.eq(replannedMetric + 3, getReplannedMetric());
 
 // The {a: 1} plan is back in the cache. Run the query which would perform better on the plan using
@@ -131,7 +133,7 @@ assert.eq(1, coll.find(bIndexQuery).itcount());
 entry = getCachedPlanForQuery(bIndexQuery);
 entryWorks = entry.works;
 assert.eq(entry.isActive, true);
-assertPlanHasIxScanStage(entry, "b_1", queryHash);
+assertPlanHasIxScanStage(entry, "b_1", planCacheShapeHash);
 assert.eq(replannedMetric + 4, getReplannedMetric());
 
 // Now run a plan that will perform poorly with both indices (it will be required to scan 500
@@ -146,7 +148,7 @@ assert.eq(500, coll.find({a: 3, b: 3}).itcount());
 // The cache entry should have been deactivated.
 entry = getCachedPlanForQuery({a: 3, b: 3});
 assert.eq(entry.isActive, false);
-assertPlanHasIxScanStage(entry, "a_1", queryHash);
+assertPlanHasIxScanStage(entry, "a_1", planCacheShapeHash);
 
 // The works value should have doubled.
 assert.eq(entry.works, entryWorks * 2);
@@ -205,18 +207,18 @@ coll = assertDropAndRecreateCollection(db, "plan_cache_replanning");
 
     // Now look at the cache entry and store the values for works, keysExamined.
     entry = getCachedPlanForQuery(filterOnSelectiveKeySpecialValue);
-    queryHash = entry.queryHash;
+    planCacheShapeHash = getPlanCacheShapeHashFromObject(entry);
     const specialValueCacheEntryWorks = entry.works;
 
     // Execution stats from when the plan cache entry was created are not exposed from the SBE plan
     // cache.
     let specialValueCacheEntryKeysExamined;
-    if (!isSbeEnabled) {
+    if (!isSbePlanCacheEnabled) {
         specialValueCacheEntryKeysExamined = entry.creationExecStats[0].totalKeysExamined;
     }
 
     assert.eq(entry.isActive, true, entry);
-    assertPlanHasIxScanStage(entry, "selectiveKey_1_tiebreak_1", queryHash);
+    assertPlanHasIxScanStage(entry, "selectiveKey_1_tiebreak_1", planCacheShapeHash);
 
     // Clear the plan cache for the collection.
     coll.getPlanCache().clear();
@@ -228,12 +230,12 @@ coll = assertDropAndRecreateCollection(db, "plan_cache_replanning");
 
     entry = getCachedPlanForQuery(filterOnSelectiveKey);
     assert.eq(entry.isActive, true, entry);
-    assertPlanHasIxScanStage(entry, "selectiveKey_1_tiebreak_1", queryHash);
+    assertPlanHasIxScanStage(entry, "selectiveKey_1_tiebreak_1", planCacheShapeHash);
 
     // The new cache entry's plan should have used fewer works (and examined fewer keys) compared
     // to the old cache entry's, since the query on the special value is slightly less efficient.
     assert.lt(entry.works, specialValueCacheEntryWorks, entry);
-    if (!isSbeEnabled) {
+    if (!isSbePlanCacheEnabled) {
         assert.lt(entry.creationExecStats[0].totalKeysExamined,
                   specialValueCacheEntryKeysExamined,
                   entry);
@@ -246,13 +248,12 @@ coll = assertDropAndRecreateCollection(db, "plan_cache_replanning");
     // Check that the cache entry hasn't changed.
     const entryAfterRunningSpecialQuery = getCachedPlanForQuery(filterOnSelectiveKey);
     assert.eq(entryAfterRunningSpecialQuery.isActive, true);
-    assertPlanHasIxScanStage(entry, "selectiveKey_1_tiebreak_1", queryHash);
+    assertPlanHasIxScanStage(entry, "selectiveKey_1_tiebreak_1", planCacheShapeHash);
 
     assert.eq(entry.works, entryAfterRunningSpecialQuery.works, entryAfterRunningSpecialQuery);
-    if (!isSbeEnabled) {
+    if (!isSbePlanCacheEnabled) {
         assert.eq(entryAfterRunningSpecialQuery.creationExecStats[0].totalKeysExamined,
                   entry.creationExecStats[0].totalKeysExamined,
                   entryAfterRunningSpecialQuery);
     }
 }
-})();
