@@ -3,9 +3,6 @@
  * collection.
  */
 
-(function() {
-"use strict";
-
 const collNamePrefix = "reindex_duplicate_keys_";
 let count = 0;
 
@@ -42,5 +39,4 @@ let runTest = function(doc) {
 };
 
 runTest();
-runTest({a: 1});
-})();
\ No newline at end of file
+runTest({a: 1});
\ No newline at end of file
