@@ -2,9 +2,6 @@
  * Tests that unique indexes can be built with a large number of non-unique values.
  */
 
-(function() {
-"use strict";
-
 let coll = db.stress_test_unique_index_notunique;
 coll.drop();
 
@@ -26,5 +23,4 @@ function loadCollectionWithDocs(collection, numDocs) {
 
 loadCollectionWithDocs(coll, kNumDocs);
 
-assert.commandFailedWithCode(coll.createIndex({a: 1}, {unique: true}), ErrorCodes.DuplicateKey);
-})();
+assert.commandFailedWithCode(coll.createIndex({a: 1}, {unique: true}), ErrorCodes.DuplicateKey);
\ No newline at end of file
