@@ -8,9 +8,6 @@
  *   requires_getmore,
  * ]
  */
-(function() {
-'use strict';
-
 const testDB = db.getSiblingDB(jsTestName());
 assert.commandWorked(testDB.dropDatabase());
 
@@ -52,5 +49,4 @@ assert.commandWorked(
     testDB.adminCommand({configureFailPoint: "allowSystemViewsDrop", mode: "alwaysOn"}));
 assert(testDB.system.views.drop());
 assert.commandWorked(
-    testDB.adminCommand({configureFailPoint: "allowSystemViewsDrop", mode: "off"}));
-})();
+    testDB.adminCommand({configureFailPoint: "allowSystemViewsDrop", mode: "off"}));
\ No newline at end of file
