@@ -3,15 +3,10 @@
  * time-series view is missing.
  *
  * @tags: [
- *     does_not_support_transactions,
- *     requires_fcv_49,
- *     requires_find_command,
- *     requires_getmore,
+ *   does_not_support_transactions,
+ *   requires_getmore,
  * ]
  */
-(function() {
-'use strict';
-
 const testDB = db.getSiblingDB(jsTestName());
 assert.commandWorked(testDB.dropDatabase());
 
@@ -34,5 +29,4 @@ assert.commandWorked(
 const collections = assert.commandWorked(testDB.runCommand({listCollections: 1})).cursor.firstBatch;
 jsTestLog('Checking listCollections result: ' + tojson(collections));
 assert.eq(collections.length, 1);
-assert(collections.find(entry => entry.name === 'system.buckets.' + coll.getName()));
-})();
+assert(collections.find(entry => entry.name === 'system.buckets.' + coll.getName()));
\ No newline at end of file
