@@ -8,10 +8,7 @@
  *   requires_getmore,
  * ]
  */
-(function() {
-"use strict";
-
-load("jstests/core/timeseries/libs/timeseries.js");
+import {TimeseriesTest} from "jstests/core/timeseries/libs/timeseries.js";
 
 const testDB = db.getSiblingDB("timeseries_system_views_drop");
 
@@ -38,5 +35,4 @@ TimeseriesTest.run((insert) => {
     assert.commandWorked(testDB.createView("myView", coll.getName(), []));
 
     assert(testDB.system.views.drop());
-});
-})();
+});
\ No newline at end of file
