@@ -3,10 +3,15 @@
 //   requires_http_client,
 // ]
 
-load("jstests/ocsp/lib/mock_ocsp.js");
-
-(function() {
-"use strict";
+import {FAULT_REVOKED, MockOCSPServer} from "jstests/ocsp/lib/mock_ocsp.js";
+import {
+    clearOCSPCache,
+    OCSP_CA_PEM,
+    OCSP_SERVER_CERT,
+    OCSP_SERVER_CERT_REVOKED,
+    waitForServer
+} from "jstests/ocsp/lib/ocsp_helpers.js";
+import {determineSSLProvider} from "jstests/ssl/libs/ssl_helpers.js";
 
 var ocsp_options = {
     sslMode: "requireSSL",
@@ -58,7 +63,7 @@ if (determineSSLProvider() === "apple") {
     const APPLE_OCSP_ERROR_CODE = "CSSMERR_TP_CERT_REVOKED";
     let output = rawMongoProgramOutput();
     assert(output.search(APPLE_OCSP_ERROR_CODE));
-    return;
+    quit();
 }
 
 clearOCSPCache();
@@ -100,4 +105,3 @@ MongoRunner.stopMongod(conn);
 // sleep to make sure that the threads don't interfere with each other.
 sleep(1000);
 mock_ocsp.stop();
-}());
