@@ -1,7 +1,16 @@
 // Check that OCSP verification may be used with invalid certificates
 // @tags: [requires_http_client, requires_ocsp_stapling]
 
-load("jstests/ocsp/lib/mock_ocsp.js");
+import {
+    FAULT_REVOKED,
+    MockOCSPServer,
+    OCSP_DELEGATE_RESPONDER
+} from "jstests/ocsp/lib/mock_ocsp.js";
+import {
+    OCSP_CA_PEM,
+    OCSP_SERVER_CERT_INVALID,
+    supportsStapling
+} from "jstests/ocsp/lib/ocsp_helpers.js";
 
 if (!supportsStapling()) {
     quit();
@@ -67,3 +76,4 @@ try {
     rstest.stopSet(15);
 } catch (e) {
 }
+mock_ocsp.stop();
