@@ -2,7 +2,8 @@
  * Tests $eq against a variety of BSON types and shapes.
  */
 
-load('jstests/query_golden/libs/example_data.js');
+import {show} from "jstests/libs/golden_test.js";
+import {leafs, smallDocs} from "jstests/query_golden/libs/example_data.js";
 
 const docs = smallDocs();
 
@@ -17,11 +18,8 @@ coll.insert(docs);
 print(`Collection count: ${coll.find().itcount()}`);
 
 for (const leaf of leafs()) {
-    // TODO SERVER-67550 Equality to null does not match undefined, in Bonsai.
-    if (tojson(leaf).match(/null|undefined/))
-        continue;
-    // TODO SERVER-67818 Bonsai NaN $eq NaN should be true.
-    if (tojson(leaf).match(/NaN/))
+    // Direct comparisons against undefined ({$eq: undefined}) are not allowed.
+    if (tojson(leaf).match(/undefined/))
         continue;
 
     const query = coll.find({a: {$eq: leaf}}, {_id: 0});
