@@ -10,14 +10,10 @@
  * ]
  */
 
-(function() {
-"use strict";
-
 // This will verify the completeness of our map and run all tests.
-load("jstests/libs/all_commands_test.js");
-load("jstests/libs/fixture_helpers.js");    // For isSharded and isReplSet
-load("jstests/libs/feature_flag_util.js");  // For isPresentAndEnabled
-load('jstests/replsets/rslib.js');
+import {AllCommandsTest} from "jstests/libs/all_commands_test.js";
+import {FeatureFlagUtil} from "jstests/libs/feature_flag_util.js";
+import {FixtureHelpers} from "jstests/libs/fixture_helpers.js";
 
 const name = jsTestName();
 const dbName = "alltestsdb";
@@ -29,6 +25,7 @@ const isAnInternalCommand = "internal command";
 const isDeprecated = "deprecated command";
 const commandIsDisabledOnLastLTS = "skip command on downgrading fcv";
 const requiresParallelShell = "requires parallel shell";
+const cannotRunWhileDowngrading = "cannot run command while downgrading";
 
 const allCommands = {
     _addShard: {skip: isAnInternalCommand},
@@ -52,12 +49,13 @@ const allCommands = {
     _configsvrCommitIndex: {skip: isAnInternalCommand},
     _configsvrCommitMergeAllChunksOnShard: {skip: isAnInternalCommand},
     _configsvrCommitMovePrimary: {skip: isAnInternalCommand},
+    _configsvrCommitRefineCollectionShardKey: {skip: isAnInternalCommand},
     _configsvrCommitReshardCollection: {skip: isAnInternalCommand},
     _configsvrConfigureCollectionBalancing: {skip: isAnInternalCommand},
     _configsvrCreateDatabase: {skip: isAnInternalCommand},
     _configsvrDropIndexCatalogEntry: {skip: isAnInternalCommand},
     _configsvrEnsureChunkVersionIsGreaterThan: {skip: isAnInternalCommand},
-    _configsvrGetHistoricalPlacement: {skip: isAnInternalCommand},  // TODO SERVER-73029 remove
+    _configsvrGetHistoricalPlacement: {skip: isAnInternalCommand},
     _configsvrMoveRange: {skip: isAnInternalCommand},
     _configsvrRefineCollectionShardKey: {skip: isAnInternalCommand},
     _configsvrRemoveChunks: {skip: isAnInternalCommand},
@@ -65,7 +63,6 @@ const allCommands = {
     _configsvrRemoveShardFromZone: {skip: isAnInternalCommand},
     _configsvrRemoveTags: {skip: isAnInternalCommand},
     _configsvrRepairShardedCollectionChunksHistory: {skip: isAnInternalCommand},
-    _configsvrRenameCollectionMetadata: {skip: isAnInternalCommand},
     _configsvrResetPlacementHistory: {skip: isAnInternalCommand},
     _configsvrReshardCollection: {skip: isAnInternalCommand},
     _configsvrRunRestore: {skip: isAnInternalCommand},
@@ -90,9 +87,6 @@ const allCommands = {
     _mergeAuthzCollections: {skip: isAnInternalCommand},
     _migrateClone: {skip: isAnInternalCommand},
     _mongotConnPoolStats: {skip: isAnInternalCommand},
-    _movePrimaryRecipientAbortMigration: {skip: isAnInternalCommand},
-    _movePrimaryRecipientForgetMigration: {skip: isAnInternalCommand},
-    _movePrimaryRecipientSyncData: {skip: isAnInternalCommand},
     _recvChunkAbort: {skip: isAnInternalCommand},
     _recvChunkCommit: {skip: isAnInternalCommand},
     _recvChunkReleaseCritSec: {skip: isAnInternalCommand},
@@ -100,25 +94,30 @@ const allCommands = {
     _recvChunkStatus: {skip: isAnInternalCommand},
     _refreshQueryAnalyzerConfiguration: {skip: isAnInternalCommand},
     _shardsvrAbortReshardCollection: {skip: isAnInternalCommand},
+    _shardsvrBeginMigrationBlockingOperation: {skip: isAnInternalCommand},
+    _shardsvrChangePrimary: {skip: isAnInternalCommand},
+    _shardsvrCleanupStructuredEncryptionData: {skip: isAnInternalCommand},
     _shardsvrCleanupReshardCollection: {skip: isAnInternalCommand},
     _shardsvrCloneCatalogData: {skip: isAnInternalCommand},
     _shardsvrCompactStructuredEncryptionData: {skip: isAnInternalCommand},
+    _shardsvrConvertToCapped: {skip: isAnInternalCommand},
     _shardsvrRegisterIndex: {skip: isAnInternalCommand},
     _shardsvrCommitIndexParticipant: {skip: isAnInternalCommand},
     _shardsvrCommitReshardCollection: {skip: isAnInternalCommand},
     _shardsvrDropCollection: {skip: isAnInternalCommand},
     _shardsvrCreateCollection: {skip: isAnInternalCommand},
     _shardsvrCreateGlobalIndex: {skip: isAnInternalCommand},
-    // TODO SERVER-74324: deprecate _shardsvrDropCollectionIfUUIDNotMatching after 7.0 is lastLTS.
-    _shardsvrDropCollectionIfUUIDNotMatching: {skip: isAnInternalCommand},
     _shardsvrDropCollectionIfUUIDNotMatchingWithWriteConcern: {skip: isAnInternalCommand},
     _shardsvrDropCollectionParticipant: {skip: isAnInternalCommand},
     _shardsvrDropGlobalIndex: {skip: isAnInternalCommand},
     _shardsvrDropIndexCatalogEntryParticipant: {skip: isAnInternalCommand},
     _shardsvrDropIndexes: {skip: isAnInternalCommand},
     _shardsvrCreateCollectionParticipant: {skip: isAnInternalCommand},
+    _shardsvrCoordinateMultiUpdate: {skip: isAnInternalCommand},
+    _shardsvrEndMigrationBlockingOperation: {skip: isAnInternalCommand},
     _shardsvrGetStatsForBalancing: {skip: isAnInternalCommand},
     _shardsvrInsertGlobalIndexKey: {skip: isAnInternalCommand},
+    _shardsvrJoinDDLCoordinators: {skip: isAnInternalCommand},
     _shardsvrDeleteGlobalIndexKey: {skip: isAnInternalCommand},
     _shardsvrWriteGlobalIndexKeys: {skip: isAnInternalCommand},
     _shardsvrJoinMigrations: {skip: isAnInternalCommand},
@@ -144,12 +143,28 @@ const allCommands = {
     _shardsvrValidateShardKeyCandidate: {skip: isAnInternalCommand},
     _shardsvrCollMod: {skip: isAnInternalCommand},
     _shardsvrCollModParticipant: {skip: isAnInternalCommand},
+    _shardsvrConvertToCappedParticipant: {skip: isAnInternalCommand},
     _shardsvrParticipantBlock: {skip: isAnInternalCommand},
+    _shardsvrUntrackUnsplittableCollection: {skip: isAnInternalCommand},
     _shardsvrCheckMetadataConsistency: {skip: isAnInternalCommand},
     _shardsvrCheckMetadataConsistencyParticipant: {skip: isAnInternalCommand},
-    _startStreamProcessor: {skip: isAnInternalCommand},
+    streams_startStreamProcessor: {skip: isAnInternalCommand},
+    streams_startStreamSample: {skip: isAnInternalCommand},
+    streams_stopStreamProcessor: {skip: isAnInternalCommand},
+    streams_listStreamProcessors: {skip: isAnInternalCommand},
+    streams_getMoreStreamSample: {skip: isAnInternalCommand},
+    streams_getStats: {skip: isAnInternalCommand},
+    streams_testOnlyInsert: {skip: isAnInternalCommand},
+    streams_getMetrics: {skip: isAnInternalCommand},
+    streams_updateFeatureFlags: {skip: isAnInternalCommand},
+    streams_testOnlyGetFeatureFlags: {skip: isAnInternalCommand},
+    streams_writeCheckpoint: {skip: isAnInternalCommand},
     _transferMods: {skip: isAnInternalCommand},
     _vectorClockPersist: {skip: isAnInternalCommand},
+    abortMoveCollection: {
+        // Skipping command because it requires testing through a parallel shell.
+        skip: requiresParallelShell,
+    },
     abortReshardCollection: {
         // Skipping command because it requires testing through a parallel shell.
         skip: requiresParallelShell,
@@ -181,6 +196,10 @@ const allCommands = {
             assert.commandWorked(conn.getDB(dbName).runCommand({drop: collName}));
         }
     },
+    abortUnshardCollection: {
+        // Skipping command because it requires testing through a parallel shell.
+        skip: requiresParallelShell,
+    },
     addShard: {
         skip: "cannot add shard while in downgrading FCV state",
     },
@@ -222,25 +241,17 @@ const allCommands = {
             assert.commandWorked(conn.getDB(dbName).runCommand({create: collName}));
         },
         command: {analyze: collName},
-        expectFailure: true,
-        expectedErrorCode: [
-            6660400,
-            6765500
-        ],  // Analyze command requires common query framework feature flag to be enabled.
+        checkFeatureFlag: "CommonQueryFramework",
         teardown: function(conn) {
             assert.commandWorked(conn.getDB(dbName).runCommand({drop: collName}));
         },
     },
     analyzeShardKey: {
-        // TODO SERVER-74867: Remove the skip once 7.0 is lastLTS.
-        skip: commandIsDisabledOnLastLTS,
-        // TODO SERVER-67966: Remove check when this feature flag is removed.
-        checkFeatureFlag: "AnalyzeShardKey",
         setUp: function(conn) {
             assert.commandWorked(conn.getDB(dbName).runCommand({create: collName}));
             assert.commandWorked(
                 conn.getDB('admin').runCommand({shardCollection: fullNs, key: {_id: 1}}));
-            for (let i = 0; i < 10; i++) {
+            for (let i = 0; i < 1000; i++) {
                 assert.commandWorked(conn.getCollection(fullNs).insert({a: i}));
             }
         },
@@ -265,7 +276,34 @@ const allCommands = {
         // Skipping command because it requires additional authentication setup.
         skip: "requires additional authentication setup"
     },
-    autoSplitVector: {skip: isAnInternalCommand},
+    autoCompact: {
+        checkFeatureFlag: "AutoCompact",
+        command: {autoCompact: false},
+        isAdminCommand: true,
+        isReplSetOnly: true,
+    },
+    autoSplitVector: {
+        setUp: function(conn) {
+            assert.commandWorked(conn.getDB(dbName).runCommand({create: collName}));
+            assert.commandWorked(
+                conn.getDB('admin').runCommand({shardCollection: fullNs, key: {a: 1}}));
+            for (let i = 0; i < 10; i++) {
+                assert.commandWorked(conn.getCollection(fullNs).insert({a: i}));
+            }
+        },
+        command: {
+            autoSplitVector: collName,
+            keyPattern: {a: 1},
+            min: {a: MinKey},
+            max: {a: MaxKey},
+            maxChunkSizeBytes: 1024 * 1024
+        },
+        teardown: function(conn) {
+            assert.commandWorked(conn.getDB(dbName).runCommand({drop: collName}));
+        },
+        isShardedOnly: true,
+        isAdminCommand: false,
+    },
     balancerCollectionStatus: {
         command: {balancerCollectionStatus: fullNs},
         isShardedOnly: true,
@@ -301,8 +339,6 @@ const allCommands = {
         isAdminCommand: true,
     },
     bulkWrite: {
-        // TODO SERVER-74867: Remove the skip once 7.0 is lastLTS.
-        skip: commandIsDisabledOnLastLTS,
         // TODO SERVER-67711: Remove check when this feature flag is removed.
         checkFeatureFlag: "BulkWriteCommand",
         isAdminCommand: true,
@@ -324,13 +360,10 @@ const allCommands = {
     captrunc: {
         skip: isAnInternalCommand,
     },
+    changePrimary: {skip: cannotRunWhileDowngrading},
     checkMetadataConsistency: {
         isAdminCommand: true,
         isShardedOnly: true,
-        // TODO SERVER-74867: Remove the skip once 7.0 is lastLTS.
-        skip: commandIsDisabledOnLastLTS,
-        // TODO SERVER-70396: Remove check when this feature flag is removed.
-        checkFeatureFlag: "CheckMetadataConsistency",
         command: {checkMetadataConsistency: 1},
     },
     checkShardingIndex: {
@@ -368,6 +401,7 @@ const allCommands = {
         // operation.
         skip: "requires additional setup through a failed resharding operation",
     },
+    cleanupStructuredEncryptionData: {skip: "requires additional encrypted collection setup"},
     clearJumboFlag: {
         isShardedOnly: true,
         fullScenario: function(conn, fixture) {
@@ -415,6 +449,7 @@ const allCommands = {
     },
     clusterAbortTransaction: {skip: "already tested by 'abortTransaction' tests on mongos"},
     clusterAggregate: {skip: "already tested by 'aggregate' tests on mongos"},
+    clusterBulkWrite: {skip: "already tested by 'bulkWrite' tests on mongos"},
     clusterCommitTransaction: {skip: "already tested by 'commitTransaction' tests on mongos"},
     clusterCount: {skip: "already tested by 'count' tests on mongos"},
     clusterDelete: {skip: "already tested by 'delete' tests on mongos"},
@@ -498,10 +533,6 @@ const allCommands = {
         isAdminCommand: true,
     },
     configureQueryAnalyzer: {
-        // TODO SERVER-74867: Remove the skip once 7.0 is lastLTS.
-        skip: commandIsDisabledOnLastLTS,
-        // TODO SERVER-67966: Remove check when this feature flag is removed.
-        checkFeatureFlag: "AnalyzeShardKey",
         setUp: function(conn) {
             assert.commandWorked(conn.getDB(dbName).runCommand({create: collName}));
             for (let i = 0; i < 10; i++) {
@@ -574,6 +605,7 @@ const allCommands = {
         // feature).
         skip: "requires mongot mock setup",
     },
+    createUnsplittableCollection: {skip: isAnInternalCommand},
     createUser: {
         command: {createUser: "foo", pwd: "bar", roles: []},
         teardown: function(conn) {
@@ -989,7 +1021,6 @@ const allCommands = {
     lockInfo: {
         isAdminCommand: true,
         command: {lockInfo: 1},
-        doesNotRunOnMongos: true,
     },
     logApplicationMessage: {
         isAdminCommand: true,
@@ -1006,9 +1037,7 @@ const allCommands = {
         skip: "requires additional authentication setup",
     },
     makeSnapshot: {
-        isAdminCommand: true,
-        command: {makeSnapshot: 1},
-        doesNotRunOnMongos: true,
+        skip: isAnInternalCommand,
     },
     mapReduce: {
         setUp: function(conn) {
@@ -1088,14 +1117,9 @@ const allCommands = {
             assert.commandWorked(conn.getDB(dbName).runCommand({drop: collName}));
         },
     },
+    moveCollection: {skip: cannotRunWhileDowngrading},
     movePrimary: {
-        isShardedOnly: true,
-        fullScenario: function(conn, fixture) {
-            assert.commandWorked(conn.getDB(dbName).runCommand({create: collName}));
-            assert.commandWorked(conn.getDB('admin').runCommand(
-                {movePrimary: dbName, to: fixture.shard0.shardName}));
-            assert.commandWorked(conn.getDB(dbName).runCommand({drop: collName}));
-        }
+        skip: cannotRunWhileDowngrading,
     },
     moveRange: {
         isShardedOnly: true,
@@ -1228,6 +1252,7 @@ const allCommands = {
         skip: isDeprecated,
     },
     removeShard: {
+        // We cannot test removeShard because we need to be able to run addShard during set up.
         // This will be tested in FCV upgrade/downgrade passthroughs in the sharding
         // directory.
         skip: "cannot add shard while in downgrading FCV state",
@@ -1314,41 +1339,11 @@ const allCommands = {
         command: {replSetResizeOplog: 1, minRetentionHours: 1},
     },
     resetPlacementHistory: {
-        skip: commandIsDisabledOnLastLTS,
-        checkFeatureFlag: "gHistoricalPlacementShardingCatalog",
         command: {resetPlacementHistory: 1},
         isShardedOnly: true,
         isAdminCommand: true,
-        expectFailure: true,
-        expectedErrorCode: ErrorCodes.TemporarilyUnavailable  // only works when FCV 7.0 is enabled.
-
-    },
-    reshardCollection: {
-        // TODO SERVER-74867: Remove the skip once 7.0 is lastLTS.
-        skip: commandIsDisabledOnLastLTS,
-        isShardedOnly: true,
-        isAdminCommand: true,
-        setUp: function(conn) {
-            assert.commandWorked(conn.getDB(dbName).runCommand({create: collName}));
-            const testColl = conn.getCollection(fullNs);
-            assert.commandWorked(
-                conn.getDB('admin').runCommand({shardCollection: fullNs, key: {_id: 1}}));
-
-            // Build an index on the collection to support the resharding operation.
-            assert.commandWorked(testColl.createIndex({a: 1}));
-
-            // Insert some documents that will be resharded.
-            assert.commandWorked(testColl.insert({_id: 0, a: 0}));
-            assert.commandWorked(testColl.insert({_id: 1, a: 1}));
-        },
-        command: {
-            reshardCollection: fullNs,
-            key: {a: 1},
-        },
-        teardown: function(conn) {
-            assert.commandWorked(conn.getDB(dbName).runCommand({drop: collName}));
-        },
     },
+    reshardCollection: {skip: cannotRunWhileDowngrading},
     revokePrivilegesFromRole: {
         setUp: function(conn) {
             assert.commandWorked(conn.getDB(dbName).runCommand({create: collName}));
@@ -1410,6 +1405,7 @@ const allCommands = {
         }
     },
     rotateCertificates: {skip: "requires additional authentication setup"},
+    rotateFTDC: {isAdminCommand: true, command: {rotateFTDC: 1}},
     saslContinue: {skip: "requires additional authentication setup"},
     saslStart: {skip: "requires additional authentication setup"},
     serverStatus: {
@@ -1447,8 +1443,6 @@ const allCommands = {
     setIndexCommitQuorum: {skip: requiresParallelShell},
     setFeatureCompatibilityVersion: {skip: "is tested through this test"},
     setProfilingFilterGlobally: {
-        // TODO SERVER-74867: Remove the skip once 7.0 is lastLTS.
-        skip: commandIsDisabledOnLastLTS,
         command: {setProfilingFilterGlobally: 1, filter: {nreturned: 0}},
         expectFailure: true,
         expectedErrorCode:
@@ -1474,6 +1468,27 @@ const allCommands = {
         doesNotRunOnStandalone: true,
         command: {setClusterParameter: {testIntClusterParameter: {intData: 2022}}}
     },
+    setQuerySettings: {
+        skip: commandIsDisabledOnLastLTS,
+        // TODO: SERVER-71537 Remove Feature Flag for PM-412.
+        checkFeatureFlag: "QuerySettings",
+        isAdminCommand: true,
+        doesNotRunOnStandalone: true,
+        command: {
+            setQuerySettings: {find: collName, $db: dbName, filter: {a: 1}},
+            settings: {indexHints: {ns: {db: dbName, coll: collName}, allowedIndexes: ["a_1"]}}
+        }
+    },
+    removeQuerySettings: {
+        skip: commandIsDisabledOnLastLTS,
+        // TODO: SERVER-71537 Remove Feature Flag for PM-412.
+        checkFeatureFlag: "QuerySettings",
+        isAdminCommand: true,
+        doesNotRunOnStandalone: true,
+        command: {
+            removeQuerySettings: {find: collName, $db: dbName, filter: {a: 1}},
+        }
+    },
     setUserWriteBlockMode: {
         command: {setUserWriteBlockMode: 1, global: true},
         teardown: function(conn) {
@@ -1534,6 +1549,7 @@ const allCommands = {
         // Skipping command because it requires an actual file path for recording traffic to.
         skip: "requires an actual file path to record traffic to",
     },
+    sysprofile: {skip: isAnInternalCommand},
     testDeprecation: {skip: isAnInternalCommand},
     testDeprecationInVersion2: {skip: isAnInternalCommand},
     testInternalTransactions: {skip: isAnInternalCommand},
@@ -1541,28 +1557,27 @@ const allCommands = {
     testReshardCloneCollection: {skip: isAnInternalCommand},
     testVersions1And2: {skip: isAnInternalCommand},
     testVersion2: {skip: isAnInternalCommand},
+    timeseriesCatalogBucketParamsChanged: {skip: isAnInternalCommand},
     top: {
         command: {top: 1},
         isAdminCommand: true,
         doesNotRunOnMongos: true,
     },
     transitionFromDedicatedConfigServer: {
-        // TODO SERVER-74867: Remove the skip once 7.0 is lastLTS.
-        skip: commandIsDisabledOnLastLTS,
-        // TODO SERVER-66060: Remove check when this feature flag is removed.
-        checkFeatureFlag: "CatalogShard",
-        command: {transitionFromDedicatedConfigServer: 1},
-        isShardedOnly: true,
-        isAdminCommand: true,
+        // This command uses addShard logic internally, so is not able to run with a transitionary
+        // FCV.
+        skip: cannotRunWhileDowngrading
     },
     transitionToDedicatedConfigServer: {
-        // TODO SERVER-74867: Remove the skip once 7.0 is lastLTS.
-        skip: commandIsDisabledOnLastLTS,
-        // TODO SERVER-66060: Remove check when this feature flag is removed.
-        checkFeatureFlag: "CatalogShard",
-        command: {transitionToDedicatedConfigServer: 1},
-        isShardedOnly: true,
-        isAdminCommand: true,
+        // In order to properly run this command, we'd need to add a shard or transition from a
+        // dedicated config server, which is not allowed in a transitionary FCV.
+        skip: cannotRunWhileDowngrading
+    },
+    transitionToShardedCluster: {skip: isAnInternalCommand},
+    unshardCollection: {skip: cannotRunWhileDowngrading},
+    untrackUnshardedCollection: {
+        // The command requires moveCollection on setup which is not allowed in fcv downgrading.
+        skip: cannotRunWhileDowngrading
     },
     update: {
         setUp: function(conn) {
@@ -1649,11 +1664,7 @@ const allCommands = {
     waitForFailPoint: {
         skip: isAnInternalCommand,
     },
-    waitForOngoingChunkSplits: {
-        command: {waitForOngoingChunkSplits: 1},
-        isShardedOnly: true,
-        isShardSvrOnly: true,
-    },
+    getShardingReady: {skip: isAnInternalCommand},
     whatsmysni: {
         command: {whatsmysni: 1},
         isAdminCommand: true,
@@ -1682,7 +1693,7 @@ let assertCommandOrWriteFailed = function(res, code, msg) {
 
 let runAllCommands = function(command, test, conn, fixture) {
     let cmdDb = conn.getDB(dbName);
-    const isShardedCluster = isMongos(cmdDb);
+    const isShardedCluster = FixtureHelpers.isMongos(cmdDb);
     const isReplSet = FixtureHelpers.isReplSet(cmdDb);
 
     // Skip command if it does not run on this type of cluster.
@@ -1772,17 +1783,13 @@ let runAllCommands = function(command, test, conn, fixture) {
 };
 
 let runTest = function(conn, adminDB, fixture) {
-    let runDowngradingToUpgrading = false;
-    if (FeatureFlagUtil.isEnabled(adminDB, "DowngradingToUpgrading")) {
-        runDowngradingToUpgrading = true;
-    }
-
-    assert.commandFailed(conn.adminCommand({setFeatureCompatibilityVersion: lastLTSFCV}));
+    assert.commandFailed(
+        conn.adminCommand({setFeatureCompatibilityVersion: lastLTSFCV, confirm: true}));
 
     jsTestLog("Running all commands in the downgradingToLastLTS FCV");
     // First check that the map contains all available commands.
     let commandsList = AllCommandsTest.checkCommandCoverage(conn, allCommands);
-    if (isMongos(adminDB)) {
+    if (FixtureHelpers.isMongos(adminDB)) {
         let shardCommandsList =
             AllCommandsTest.checkCommandCoverage(fixture.shard0.rs.getPrimary(), allCommands);
         commandsList = new Set(commandsList.concat(shardCommandsList));
@@ -1803,30 +1810,29 @@ let runTest = function(conn, adminDB, fixture) {
         runAllCommands(command, test, conn, fixture);
     }
 
-    if (runDowngradingToUpgrading) {
-        assert.commandWorked(conn.adminCommand({setFeatureCompatibilityVersion: latestFCV}));
+    assert.commandWorked(
+        conn.adminCommand({setFeatureCompatibilityVersion: latestFCV, confirm: true}));
 
-        jsTestLog("Running all commands after upgrading back to the latest FCV");
-        commandsList = AllCommandsTest.checkCommandCoverage(conn, allCommands);
-        if (isMongos(adminDB)) {
-            let shardCommandsList =
-                AllCommandsTest.checkCommandCoverage(fixture.shard0.rs.getPrimary(), allCommands);
-            commandsList = new Set(commandsList.concat(shardCommandsList));
+    jsTestLog("Running all commands after upgrading back to the latest FCV");
+    commandsList = AllCommandsTest.checkCommandCoverage(conn, allCommands);
+    if (FixtureHelpers.isMongos(adminDB)) {
+        let shardCommandsList =
+            AllCommandsTest.checkCommandCoverage(fixture.shard0.rs.getPrimary(), allCommands);
+        commandsList = new Set(commandsList.concat(shardCommandsList));
+    }
+
+    for (const command of commandsList) {
+        const test = allCommands[command];
+
+        // Coverage already guaranteed above, but check again just in case.
+        assert(test, "Coverage failure: must explicitly define a test for " + command);
+
+        if (test.skip !== undefined) {
+            jsTestLog("Skipping " + command + ": " + test.skip);
+            continue;
         }
 
-        for (const command of commandsList) {
-            const test = allCommands[command];
-
-            // Coverage already guaranteed above, but check again just in case.
-            assert(test, "Coverage failure: must explicitly define a test for " + command);
-
-            if (test.skip !== undefined) {
-                jsTestLog("Skipping " + command + ": " + test.skip);
-                continue;
-            }
-
-            runAllCommands(command, test, conn, fixture);
-        }
+        runAllCommands(command, test, conn, fixture);
     }
 };
 
@@ -1874,4 +1880,3 @@ let runShardedClusterTest = function() {
 runStandaloneTest();
 runReplicaSetTest();
 runShardedClusterTest();
-})();
