@@ -8,9 +8,6 @@
  *     requires_replication,
  * ]
  */
-(function() {
-"use strict";
-
 const rst = new ReplSetTest({nodes: 3});
 rst.startSet();
 rst.initiate();
@@ -38,5 +35,4 @@ assert.commandWorked(db.runCommand({applyOps: ops}));
 assert.commandWorked(db.runCommand(
     {applyOps: [{op: "d", ns: nss(dbName, collName), ts: Timestamp(20, 0), o: {_id: 19}}]}));
 
-rst.stopSet();
-}());
+rst.stopSet();
\ No newline at end of file
