@@ -11,9 +11,6 @@
  * @tags: [requires_fcv_53]
  */
 
-(function() {
-"use strict";
-
 const name = "arbiters_not_included_in_w2_wc";
 const rst = new ReplSetTest({name: name, nodes: 5});
 const nodes = rst.nodeList();
@@ -50,5 +47,4 @@ jsTestLog("Issuing a w:2 write and confirming that it times out");
 assert.commandFailedWithCode(testColl.insert({"b": 2}, {writeConcern: {w: 2, wtimeout: 5 * 1000}}),
                              ErrorCodes.WriteConcernFailed);
 
-rst.stopSet();
-})();
+rst.stopSet();
\ No newline at end of file
