@@ -4,9 +4,6 @@
  *  is renamed.
  */
 
-(function() {
-"use strict";
-
 // Bring up a 2 node replset.
 var name = "bg_index_rename";
 var rst = new ReplSetTest({name: name, nodes: 3});
@@ -23,7 +20,7 @@ for (var i = 0; i < 100; i++) {
 }
 
 // Add a background index.
-coll.createIndex({x: 1}, {background: true});
+coll.createIndex({x: 1});
 
 // Rename the collection.
 assert.commandWorked(
@@ -48,5 +45,4 @@ coll.insert({_id: 200, x: 600, str: "goodnight moon"});
 var indexes = barDB.runCommand({listIndexes: "test"});
 assert.eq(indexes.cursor.firstBatch.length, 2);
 
-rst.stopSet();
-}());
+rst.stopSet();
\ No newline at end of file
