@@ -7,10 +7,12 @@
  *   multiversion_incompatible
  *  ]
  */
-(function() {
-'use strict';
-
-load('jstests/libs/cluster_server_parameter_utils.js');
+import {
+    setupReplicaSet,
+    testGetClusterParameterStar,
+    testInvalidClusterParameterCommands,
+    testValidClusterParameterCommands,
+} from "jstests/libs/cluster_server_parameter_utils.js";
 
 // Tests that set/getClusterParameter works on a non-sharded replica set.
 const rst = new ReplSetTest({
@@ -33,4 +35,3 @@ testValidClusterParameterCommands(rst);
 testGetClusterParameterStar(rst);
 
 rst.stopSet();
-})();
