@@ -6,11 +6,8 @@
  * @tags: [multiversion_incompatible]
  */
 
-(function() {
-"use strict";
-
 // This will verify the completeness of our map and run all tests.
-load("jstests/libs/all_commands_test.js");
+import {AllCommandsTest} from "jstests/libs/all_commands_test.js";
 
 const name = jsTestName();
 const dbName = "alltestsdb";
@@ -34,8 +31,8 @@ const allCommands = {
     _configsvrBalancerStart: {skip: isPrimaryOnly},
     _configsvrBalancerStatus: {skip: isPrimaryOnly},
     _configsvrBalancerStop: {skip: isPrimaryOnly},
-    _configsvrCheckClusterMetadataConsistency: {skip: isPrimaryOnly},
-    _configsvrCheckMetadataConsistency: {skip: isPrimaryOnly},
+    _configsvrCheckClusterMetadataConsistency: {skip: isAnInternalCommand},
+    _configsvrCheckMetadataConsistency: {skip: isAnInternalCommand},
     _configsvrCleanupReshardCollection: {skip: isPrimaryOnly},
     _configsvrCollMod: {skip: isAnInternalCommand},
     _configsvrClearJumboFlag: {skip: isPrimaryOnly},
@@ -45,12 +42,13 @@ const allCommands = {
     _configsvrCommitIndex: {skip: isPrimaryOnly},
     _configsvrCommitMergeAllChunksOnShard: {skip: isPrimaryOnly},
     _configsvrCommitMovePrimary: {skip: isPrimaryOnly},
+    _configsvrCommitRefineCollectionShardKey: {skip: isPrimaryOnly},
     _configsvrCommitReshardCollection: {skip: isPrimaryOnly},
     _configsvrConfigureCollectionBalancing: {skip: isPrimaryOnly},
     _configsvrCreateDatabase: {skip: isPrimaryOnly},
     _configsvrDropIndexCatalogEntry: {skip: isPrimaryOnly},
     _configsvrEnsureChunkVersionIsGreaterThan: {skip: isPrimaryOnly},
-    _configsvrGetHistoricalPlacement: {skip: isAnInternalCommand},  // TODO SERVER-73029 remove
+    _configsvrGetHistoricalPlacement: {skip: isAnInternalCommand},
     _configsvrMoveRange: {skip: isPrimaryOnly},
     _configsvrRefineCollectionShardKey: {skip: isPrimaryOnly},
     _configsvrRemoveChunks: {skip: isPrimaryOnly},
@@ -58,7 +56,6 @@ const allCommands = {
     _configsvrRemoveShardFromZone: {skip: isPrimaryOnly},
     _configsvrRemoveTags: {skip: isPrimaryOnly},
     _configsvrRepairShardedCollectionChunksHistory: {skip: isPrimaryOnly},
-    _configsvrRenameCollectionMetadata: {skip: isPrimaryOnly},
     _configsvrResetPlacementHistory: {skip: isPrimaryOnly},
     _configsvrReshardCollection: {skip: isPrimaryOnly},
     _configsvrRunRestore: {skip: isPrimaryOnly},
@@ -83,9 +80,6 @@ const allCommands = {
     _mergeAuthzCollections: {skip: isPrimaryOnly},
     _migrateClone: {skip: isPrimaryOnly},
     _mongotConnPoolStats: {skip: isAnInternalCommand},
-    _movePrimaryRecipientAbortMigration: {skip: isAnInternalCommand},
-    _movePrimaryRecipientForgetMigration: {skip: isAnInternalCommand},
-    _movePrimaryRecipientSyncData: {skip: isAnInternalCommand},
     _recvChunkAbort: {skip: isPrimaryOnly},
     _recvChunkCommit: {skip: isPrimaryOnly},
     _recvChunkReleaseCritSec: {skip: isPrimaryOnly},
@@ -93,18 +87,19 @@ const allCommands = {
     _recvChunkStatus: {skip: isPrimaryOnly},
     _refreshQueryAnalyzerConfiguration: {skip: isPrimaryOnly},
     _shardsvrAbortReshardCollection: {skip: isPrimaryOnly},
+    _shardsvrChangePrimary: {skip: isAnInternalCommand},
+    _shardsvrCleanupStructuredEncryptionData: {skip: isPrimaryOnly},
     _shardsvrCleanupReshardCollection: {skip: isPrimaryOnly},
     _shardsvrCloneCatalogData: {skip: isPrimaryOnly},
     _shardsvrCompactStructuredEncryptionData: {skip: isPrimaryOnly},
     _shardsvrRegisterIndex: {skip: isPrimaryOnly},
     _shardsvrCommitIndexParticipant: {skip: isPrimaryOnly},
     _shardsvrCommitReshardCollection: {skip: isPrimaryOnly},
+    _shardsvrConvertToCapped: {skip: isPrimaryOnly},
     _shardsvrCreateGlobalIndex: {skip: isAnInternalCommand},
     _shardsvrDropGlobalIndex: {skip: isAnInternalCommand},
     _shardsvrDropCollection: {skip: isPrimaryOnly},
     _shardsvrCreateCollection: {skip: isPrimaryOnly},
-    // TODO SERVER-74324: deprecate _shardsvrDropCollectionIfUUIDNotMatching after 7.0 is lastLTS.
-    _shardsvrDropCollectionIfUUIDNotMatching: {skip: isNotAUserDataRead},
     _shardsvrDropCollectionIfUUIDNotMatchingWithWriteConcern: {skip: isNotAUserDataRead},
     _shardsvrDropCollectionParticipant: {skip: isPrimaryOnly},
     _shardsvrDropIndexCatalogEntryParticipant: {skip: isPrimaryOnly},
@@ -115,6 +110,7 @@ const allCommands = {
     _shardsvrDeleteGlobalIndexKey: {skip: isPrimaryOnly},
     _shardsvrWriteGlobalIndexKeys: {skip: isPrimaryOnly},
     _shardsvrJoinMigrations: {skip: isAnInternalCommand},
+    _shardsvrJoinDDLCoordinators: {skip: isPrimaryOnly},
     _shardsvrMergeAllChunksOnShard: {skip: isPrimaryOnly},
     _shardsvrMovePrimary: {skip: isPrimaryOnly},
     _shardsvrMovePrimaryEnterCriticalSection: {skip: isPrimaryOnly},
@@ -135,16 +131,33 @@ const allCommands = {
     _shardsvrSetUserWriteBlockMode: {skip: isPrimaryOnly},
     _shardsvrUnregisterIndex: {skip: isPrimaryOnly},
     _shardsvrValidateShardKeyCandidate: {skip: isPrimaryOnly},
+    _shardsvrCoordinateMultiUpdate: {skip: isAnInternalCommand},
     _shardsvrCollMod: {skip: isPrimaryOnly},
     _shardsvrCollModParticipant: {skip: isAnInternalCommand},
+    _shardsvrConvertToCappedParticipant: {skip: isAnInternalCommand},
     _shardsvrParticipantBlock: {skip: isAnInternalCommand},
     _shardsvrCheckMetadataConsistency: {skip: isAnInternalCommand},
     _shardsvrCheckMetadataConsistencyParticipant: {skip: isAnInternalCommand},
-    _startStreamProcessor: {skip: isAnInternalCommand},
+    _shardsvrBeginMigrationBlockingOperation: {skip: isAnInternalCommand},
+    _shardsvrUntrackUnsplittableCollection: {skip: isAnInternalCommand},
+    _shardsvrEndMigrationBlockingOperation: {skip: isAnInternalCommand},
+    streams_startStreamProcessor: {skip: isAnInternalCommand},
+    streams_startStreamSample: {skip: isAnInternalCommand},
+    streams_stopStreamProcessor: {skip: isAnInternalCommand},
+    streams_listStreamProcessors: {skip: isAnInternalCommand},
+    streams_getMoreStreamSample: {skip: isAnInternalCommand},
+    streams_getStats: {skip: isAnInternalCommand},
+    streams_testOnlyInsert: {skip: isAnInternalCommand},
+    streams_getMetrics: {skip: isAnInternalCommand},
+    streams_updateFeatureFlags: {skip: isAnInternalCommand},
+    streams_testOnlyGetFeatureFlags: {skip: isAnInternalCommand},
+    streams_writeCheckpoint: {skip: isAnInternalCommand},
     _transferMods: {skip: isPrimaryOnly},
     _vectorClockPersist: {skip: isPrimaryOnly},
+    abortMoveCollection: {skip: isPrimaryOnly},
     abortReshardCollection: {skip: isPrimaryOnly},
     abortTransaction: {skip: isPrimaryOnly},
+    abortUnshardCollection: {skip: isPrimaryOnly},
     aggregate: {
         command: {aggregate: collName, pipeline: [{$match: {}}], cursor: {}},
         expectFailure: true,
@@ -160,18 +173,22 @@ const allCommands = {
     appendOplogNote: {skip: isPrimaryOnly},
     applyOps: {skip: isPrimaryOnly},
     authenticate: {skip: isNotAUserDataRead},
+    autoCompact: {skip: isNotAUserDataRead},
     autoSplitVector: {skip: isPrimaryOnly},
     buildInfo: {skip: isNotAUserDataRead},
     bulkWrite: {skip: isPrimaryOnly},
     captrunc: {skip: isPrimaryOnly},
+    changePrimary: {skip: isPrimaryOnly},
     checkMetadataConsistency: {skip: isPrimaryOnly},
     checkShardingIndex: {skip: isPrimaryOnly},
     cleanupOrphaned: {skip: isPrimaryOnly},
     cleanupReshardCollection: {skip: isPrimaryOnly},
+    cleanupStructuredEncryptionData: {skip: isPrimaryOnly},
     clearLog: {skip: isNotAUserDataRead},
     cloneCollectionAsCapped: {skip: isPrimaryOnly},
     clusterAbortTransaction: {skip: "already tested by 'abortTransaction' tests on mongos"},
     clusterAggregate: {skip: "already tested by 'aggregate' tests on mongos"},
+    clusterBulkWrite: {skip: "already tested by 'bulkWrite' tests on mongos"},
     clusterCommitTransaction: {skip: "already tested by 'commitTransaction' tests on mongos"},
     clusterCount: {skip: "already tested by 'count' tests on mongos"},
     clusterDelete: {skip: "already tested by 'delete' tests on mongos"},
@@ -207,6 +224,7 @@ const allCommands = {
     createIndexes: {skip: isPrimaryOnly},
     createRole: {skip: isPrimaryOnly},
     createSearchIndexes: {skip: isNotAUserDataRead},
+    createUnsplittableCollection: {skip: isPrimaryOnly},
     createUser: {skip: isPrimaryOnly},
     currentOp: {skip: isNotAUserDataRead},
     dataSize: {
@@ -281,7 +299,6 @@ const allCommands = {
     getParameter: {skip: isNotAUserDataRead},
     getShardMap: {skip: isNotAUserDataRead},
     getShardVersion: {skip: isPrimaryOnly},
-    getnonce: {skip: "removed in v6.3"},
     godinsert: {skip: isAnInternalCommand},
     grantPrivilegesToRole: {skip: isPrimaryOnly},
     grantRolesToRole: {skip: isPrimaryOnly},
@@ -324,7 +341,7 @@ const allCommands = {
         expectedErrorCode: ErrorCodes.NotPrimaryOrSecondary
     },
     listSearchIndexes: {skip: isNotAUserDataRead},
-    lockInfo: {skip: isPrimaryOnly},
+    lockInfo: {skip: isAnInternalCommand},
     logApplicationMessage: {skip: isNotAUserDataRead},
     logMessage: {skip: isNotAUserDataRead},
     logRotate: {skip: isNotAUserDataRead},
@@ -386,6 +403,7 @@ const allCommands = {
     revokeRolesFromUser: {skip: isPrimaryOnly},
     rolesInfo: {skip: isPrimaryOnly},
     rotateCertificates: {skip: isNotAUserDataRead},
+    rotateFTDC: {skip: isNotAUserDataRead},
     saslContinue: {skip: isPrimaryOnly},
     saslStart: {skip: isPrimaryOnly},
     serverStatus: {skip: isNotAUserDataRead},
@@ -399,6 +417,8 @@ const allCommands = {
     setShardVersion: {skip: isNotAUserDataRead},
     setChangeStreamState: {skip: isNotAUserDataRead},
     setClusterParameter: {skip: isNotAUserDataRead},
+    setQuerySettings: {skip: isPrimaryOnly},
+    removeQuerySettings: {skip: isPrimaryOnly},
     setUserWriteBlockMode: {skip: isPrimaryOnly},
     shardingState: {skip: isNotAUserDataRead},
     shutdown: {skip: isNotAUserDataRead},
@@ -409,6 +429,7 @@ const allCommands = {
     startRecordingTraffic: {skip: isNotAUserDataRead},
     startSession: {skip: isNotAUserDataRead},
     stopRecordingTraffic: {skip: isNotAUserDataRead},
+    sysprofile: {skip: isAnInternalCommand},
     testDeprecation: {skip: isNotAUserDataRead},
     testDeprecationInVersion2: {skip: isNotAUserDataRead},
     testInternalTransactions: {skip: isNotAUserDataRead},
@@ -416,7 +437,9 @@ const allCommands = {
     testReshardCloneCollection: {skip: isNotAUserDataRead},
     testVersions1And2: {skip: isNotAUserDataRead},
     testVersion2: {skip: isNotAUserDataRead},
+    timeseriesCatalogBucketParamsChanged: {skip: isAnInternalCommand},
     top: {skip: isNotAUserDataRead},
+    transitionToShardedCluster: {skip: isNotAUserDataRead},
     update: {skip: isPrimaryOnly},
     updateRole: {skip: isPrimaryOnly},
     updateSearchIndex: {skip: isNotAUserDataRead},
@@ -432,7 +455,7 @@ const allCommands = {
     voteCommitImportCollection: {skip: isNotAUserDataRead},
     voteCommitIndexBuild: {skip: isNotAUserDataRead},
     waitForFailPoint: {skip: isNotAUserDataRead},
-    waitForOngoingChunkSplits: {skip: isNotAUserDataRead},
+    getShardingReady: {skip: isNotAUserDataRead},
     whatsmysni: {skip: isNotAUserDataRead},
     whatsmyuri: {skip: isNotAUserDataRead}
 };
@@ -489,4 +512,3 @@ AllCommandsTest.testAllCommands(secondary, allCommands, function(test) {
 // Turn off maintenance mode and stop the test.
 assert.commandWorked(secondary.adminCommand({replSetMaintenance: 0}));
 rst.stopSet();
-})();
