@@ -3,17 +3,11 @@
  * resources when reading a timestamp using the $_internalReadAtClusterTime option.
  *
  * @tags: [
- *   # Incompatible with all feature flags running on last continuous as dbHash runs lock-free in
- *   # v7.0.
- *   requires_fcv_70,
+ *   requires_fcv_73, # Renames FeatureCompatibilityVersion -> MultiDocumentTransactionsBarrier
  *   uses_transactions,
  * ]
  */
-(function() {
-"use strict";
-
-load("jstests/libs/feature_flag_util.js");
-load("jstests/libs/parallelTester.js");  // for Thread
+import {Thread} from "jstests/libs/parallelTester.js";
 
 const rst = new ReplSetTest({nodes: 1});
 rst.startSet();
@@ -22,13 +16,6 @@ rst.initiate();
 const primary = rst.getPrimary();
 const db = primary.getDB("test");
 
-if (FeatureFlagUtil.isEnabled(db, "PointInTimeCatalogLookups")) {
-    jsTestLog("Skipping test as dbHash is run lock-free with " +
-              "the point-in-time catalog lookups feature flag enabled");
-    rst.stopSet();
-    return;
-}
-
 const session = primary.startSession({causalConsistency: false});
 const sessionDB = session.getDatabase(db.getName());
 
@@ -44,7 +31,7 @@ const ops = db.currentOp({"lsid.id": session.getSessionId().id}).inprog;
 assert.eq(
     1, ops.length, () => "Failed to find session in currentOp() output: " + tojson(db.currentOp()));
 assert.eq(ops[0].locks, {
-    FeatureCompatibilityVersion: "w",
+    MultiDocumentTransactionsBarrier: "w",
     ReplicationStateTransition: "w",
     Global: "w",
     Database: "w",
@@ -97,7 +84,6 @@ assert.soon(() => {
         return false;
     }
     assert.eq(ops[0].locks, {
-        FeatureCompatibilityVersion: "r",
         ReplicationStateTransition: "w",
         Global: "r",
         Database: "r",
@@ -114,5 +100,4 @@ assert.commandWorked(threadCaptruncCmd.returnData());
 assert.commandWorked(threadDBHash.returnData());
 
 session.endSession();
-rst.stopSet();
-})();
+rst.stopSet();
\ No newline at end of file
