@@ -7,9 +7,6 @@
  *   uses_transactions,
  * ]
  */
-(function() {
-"use strict";
-
 const rst = new ReplSetTest({nodes: 2});
 rst.startSet();
 
@@ -100,6 +97,7 @@ assert.eq(atClusterTimeHashBefore,
 
     // Using $_internalReadAtClusterTime or snapshot read concern to read at the opTime
     // of the first insert should return the same md5sum on the secondary as it did on the primary.
+    rst.awaitReplication();
     res = assert.commandWorked(secondaryDB.runCommand({
         dbHash: 1,
         $_internalReadAtClusterTime: clusterTime,
@@ -172,16 +170,12 @@ assert.eq(atClusterTimeHashBefore,
     const otherDB = otherSession.getDatabase("test");
 
     // We create another collection inside a separate session to modify the collection catalog
-    // at an opTime later than 'clusterTime'. This prevents further usage of the snapshot
-    // associated with 'clusterTime' for snapshot reads.
+    // at an opTime later than 'clusterTime'.
     assert.commandWorked(otherDB.runCommand({create: "mycoll2"}));
-    assert.commandFailedWithCode(
-        db.runCommand({dbHash: 1, $_internalReadAtClusterTime: clusterTime}),
-        ErrorCodes.SnapshotUnavailable);
+    assert.commandWorked(db.runCommand({dbHash: 1, $_internalReadAtClusterTime: clusterTime}));
 
     otherSession.endSession();
 }
 
 session.endSession();
 rst.stopSet();
-})();
