@@ -7,9 +7,6 @@
  * ]
  */
 
-(function() {
-'use strict';
-
 const name = jsTestName();
 const replSetA = new ReplSetTest({
     name,
@@ -92,5 +89,4 @@ assert.eq(primaryB.host, statusB.members[0].name);
 assert.eq(ReplSetTest.State.PRIMARY, statusB.members[0].state);
 
 replSetB.stopSet();
-replSetA.stopSet();
-})();
+replSetA.stopSet();
\ No newline at end of file
