@@ -3,10 +3,7 @@
  * properly.
  */
 
-(function() {
-"use strict";
-
-load("jstests/replsets/libs/two_phase_drops.js");  // For TwoPhaseDropCollectionTest.
+import {TwoPhaseDropCollectionTest} from "jstests/replsets/libs/two_phase_drops.js";
 
 // Set up a two phase drop test.
 let testName = "drop_collection_two_phase";
@@ -21,7 +18,7 @@ let replTest = twoPhaseDropTest.initReplSet();
 if (!twoPhaseDropTest.supportsDropPendingNamespaces()) {
     jsTestLog('Drop pending namespaces not supported by storage engine. Skipping test.');
     twoPhaseDropTest.stop();
-    return;
+    quit();
 }
 
 // Create the collection that will be dropped.
@@ -33,5 +30,4 @@ twoPhaseDropTest.prepareDropCollection(collName);
 // COMMIT collection drop.
 twoPhaseDropTest.commitDropCollection(collName);
 
-twoPhaseDropTest.stop();
-}());
+twoPhaseDropTest.stop();
\ No newline at end of file
