@@ -3,9 +3,7 @@
  * then back to the first one.
  */
 
-(function() {
-"use strict";
-load("jstests/replsets/libs/election_handoff.js");
+import {ElectionHandoffTest} from "jstests/replsets/libs/election_handoff.js";
 
 const testName = "election_handoff_flip";
 const numNodes = 2;
@@ -21,5 +19,4 @@ ElectionHandoffTest.testElectionHandoff(rst, 0, 1);
 sleep(ElectionHandoffTest.stepDownPeriodSecs * 1000);
 ElectionHandoffTest.testElectionHandoff(rst, 1, 0);
 
-rst.stopSet();
-})();
\ No newline at end of file
+rst.stopSet();
\ No newline at end of file
