@@ -2,10 +2,7 @@
  * Tests that election handoff will not attempt to step up a node that is unelectable.
  */
 
-(function() {
-"use strict";
-
-load("jstests/replsets/libs/election_handoff.js");
+import {ElectionHandoffTest} from "jstests/replsets/libs/election_handoff.js";
 
 const rst = new ReplSetTest({name: jsTestName(), nodes: 3});
 rst.startSet();
@@ -21,5 +18,4 @@ assert.commandWorked(frozenSecondary.adminCommand({replSetFreeze: ReplSetTest.kF
 
 ElectionHandoffTest.testElectionHandoff(rst, 0, 2);
 
-rst.stopSet();
-})();
+rst.stopSet();
\ No newline at end of file
