@@ -4,9 +4,6 @@
  * @tags: [
  * ]
  */
-(function() {
-'use strict';
-
 jsTestLog("Test PSS configuration will set defaultWC to majority.");
 let replTest = new ReplSetTest({name: 'default_wc_majority', nodes: 3});
 replTest.startSet();
@@ -28,5 +25,4 @@ primary = replTest.getPrimary();
 res = assert.commandWorked(primary.adminCommand({getDefaultRWConcern: 1}));
 assert(!res.hasOwnProperty("defaultWriteConcern"));
 
-replTest.stopSet();
-})();
+replTest.stopSet();
\ No newline at end of file
