@@ -9,11 +9,8 @@
  * ]
  */
 
-(function() {
-"use strict";
-
-load("jstests/core/txns/libs/prepare_helpers.js");
-load("jstests/libs/fail_point_util.js");
+import {PrepareHelpers} from "jstests/core/txns/libs/prepare_helpers.js";
+import {kDefaultWaitForFailPointTimeout} from "jstests/libs/fail_point_util.js";
 
 const replTest = new ReplSetTest({nodes: 2});
 replTest.startSet();
@@ -123,4 +120,3 @@ res = newPrimary.getDB(dbName).getCollection(collName).findOne({_id: 4});
 assert.docEq({_id: 4}, res);
 
 replTest.stopSet();
-})();
