@@ -2,7 +2,6 @@
  * Tests that initial sync does not fail if it inserts documents which don't validate.
  */
 
-(function() {
 var name = 'initial_sync_document_validation';
 var replSet = new ReplSetTest({
     name: name,
@@ -25,5 +24,4 @@ replSet.awaitSecondaryNodes();
 assert.eq(1, secondary.getDB("test")[name].count());
 assert.docEq({_id: 0, x: 1}, secondary.getDB("test")[name].findOne());
 
-replSet.stopSet();
-})();
+replSet.stopSet();
\ No newline at end of file
