@@ -2,12 +2,9 @@
  * Test that CollectionCloner completes without error when a collection is dropped during cloning.
  */
 
-(function() {
-"use strict";
-
-load("jstests/libs/fail_point_util.js");
-load('jstests/replsets/libs/two_phase_drops.js');
-load("jstests/libs/uuid_util.js");
+import {kDefaultWaitForFailPointTimeout} from "jstests/libs/fail_point_util.js";
+import {extractUUIDFromObject, getUUIDFromListCollections} from "jstests/libs/uuid_util.js";
+import {TwoPhaseDropCollectionTest} from "jstests/replsets/libs/two_phase_drops.js";
 
 // Set up replica set. Disallow chaining so nodes always sync from primary.
 const testName = "initial_sync_drop_collection";
@@ -188,4 +185,3 @@ runDropTest({
 });
 
 replTest.stopSet();
-})();
