@@ -4,7 +4,6 @@
  * so we allow initial sync to retry 3 times.
  */
 
-(function() {
 var name = 'initial_sync_fail_insert_once';
 var replSet = new ReplSetTest(
     {name: name, nodes: 2, nodeOptions: {setParameter: "numInitialSyncAttempts=3"}});
@@ -31,5 +30,4 @@ assert.eq(1, secondary.getDB("test")[name].count());
 assert.docEq({_id: 0, x: 1}, secondary.getDB("test")[name].findOne());
 
 jsTest.log("Stopping repl set test; finished.");
-replSet.stopSet();
-})();
+replSet.stopSet();
\ No newline at end of file
