@@ -1,9 +1,6 @@
 // Test that if an afterClusterTime query is issued to a node in initial sync that has not yet
 // created its oplog, the node returns an error rather than crashing.
-(function() {
-'use strict';
-
-load("jstests/libs/fail_point_util.js");
+import {configureFailPoint} from "jstests/libs/fail_point_util.js";
 
 const replSet = new ReplSetTest({nodes: 1});
 
@@ -27,5 +24,4 @@ failPoint.off();
 replSet.awaitReplication();
 replSet.awaitSecondaryNodes();
 
-replSet.stopSet();
-})();
\ No newline at end of file
+replSet.stopSet();
\ No newline at end of file
