@@ -4,11 +4,6 @@
  * @tags: [requires_fcv_60]
  */
 
-(function() {
-'use strict';
-
-load('jstests/libs/fail_point_util.js');
-
 const rst = new ReplSetTest({nodes: 1, name: jsTestName()});
 rst.startSet();
 rst.initiate();
@@ -36,5 +31,4 @@ jsTestLog("Checking for message indicating sharding hook ran.");
 checkLog.containsJson(secondary, 6351912);
 
 jsTestLog("Done with test.");
-rst.stopSet();
-})();
+rst.stopSet();
\ No newline at end of file
