@@ -6,14 +6,11 @@
  * @tags: [
  * ]
  */
-(function() {
-"use strict";
-
-load("jstests/libs/fail_point_util.js");
+import {configureFailPoint, kDefaultWaitForFailPointTimeout} from "jstests/libs/fail_point_util.js";
 
 const testName = TestData.testName;
 const rst = new ReplSetTest({name: testName, nodes: [{}]});
-const nodes = rst.startSet();
+rst.startSet();
 rst.initiate();
 
 const primary = rst.getPrimary();
@@ -109,4 +106,3 @@ assert.commandWorked(primary.adminCommand({replSetReconfig: config, force: 1}));
 rst.waitForState(initialSyncNode, ReplSetTest.State.SECONDARY);
 
 rst.stopSet();
-})();
