@@ -2,10 +2,6 @@
  * Test initial sync with views present.
  */
 
-(function() {
-"use strict";
-
-load("jstests/replsets/rslib.js");
 let testName = "initial_sync_views";
 let hostName = getHostName();
 
@@ -37,5 +33,4 @@ let secondaryDB = secondary.getDB(testName);
 assert.eq(10, secondaryDB.coll.find().itcount());
 assert.eq(1, secondaryDB.view.find().itcount());
 
-replTest.stopSet();
-})();
+replTest.stopSet();
\ No newline at end of file
