@@ -3,8 +3,6 @@
  * configs, so this is just seeing if it fails when it's supposed to.
  * @tags: [multiversion_incompatible]
  */
-(function() {
-"use strict";
 var replTest = new ReplSetTest({name: 'testSet2', nodes: 1});
 var nodes = replTest.startSet();
 
@@ -21,5 +19,4 @@ assert.soon(function() {
     return false;
 });
 
-replTest.stopSet();
-}());
+replTest.stopSet();
\ No newline at end of file
