@@ -6,10 +6,10 @@
  * @tags: [requires_fcv_60, uses_transactions]
  */
 
-(function() {
-"use strict";
-
-load("jstests/sharding/libs/sharded_transactions_helpers.js");
+import {
+    makeCommitTransactionCmdObj,
+    makeLsidFilter
+} from "jstests/sharding/libs/sharded_transactions_helpers.js";
 
 // This test runs the reapLogicalSessionCacheNow command. That can lead to direct writes to the
 // config.transactions collection, which cannot be performed on a session.
@@ -22,7 +22,6 @@ const rst = new ReplSetTest({
             maxSessions: 1,
             // Make transaction records expire immediately.
             TransactionRecordMinimumLifetimeMinutes: 0,
-            storeFindAndModifyImagesInSideCollection: true,
             internalSessionsReapThreshold: 0
         }
     }
@@ -226,5 +225,4 @@ let numTransactionsCollEntriesReaped = 0;
 assert.eq(numTransactionsCollEntriesReaped, oplogColl.find({op: 'd', ns: kConfigTxnsNs}).itcount());
 assert.eq(0, oplogColl.find({op: {'$ne': 'd'}, ns: kConfigTxnsNs}).itcount());
 
-rst.stopSet();
-})();
+rst.stopSet();
\ No newline at end of file
