@@ -4,9 +4,6 @@
  *
  * @tags: [requires_fcv_60, uses_transactions]
  */
-(function() {
-"use strict";
-
 const logicalSessionRefreshMillis = 1000;
 const rst = new ReplSetTest({
     nodes: 2,
@@ -83,5 +80,4 @@ for (let isRetryableWriteSession of [true, false]) {
     }
 }
 
-rst.stopSet();
-})();
+rst.stopSet();
\ No newline at end of file
