@@ -6,11 +6,8 @@
  * ]
  */
 
-(function() {
-"use strict";
-
-load("jstests/libs/fail_point_util.js");
-load("jstests/replsets/rslib.js");
+import {kDefaultWaitForFailPointTimeout} from "jstests/libs/fail_point_util.js";
+import {waitForState} from "jstests/replsets/rslib.js";
 
 const rst = new ReplSetTest(
     {nodes: 1, nodeOptions: {setParameter: {logComponentVerbosity: tojson({replication: 3})}}});
@@ -42,4 +39,3 @@ assert.commandWorked(newNode.adminCommand({configureFailPoint: "failIsSelfCheck"
 jsTestLog("New node re-checks isSelf and becomes secondary");
 waitForState(newNode, ReplSetTest.State.SECONDARY);
 rst.stopSet();
-})();
