@@ -7,11 +7,7 @@
  * ]
  */
 
-(function() {
-"use strict";
-
-load("jstests/libs/fail_point_util.js");
-load("jstests/replsets/rslib.js");
+import {waitForState} from "jstests/replsets/rslib.js";
 
 const rst = new ReplSetTest({
     nodes: [{}, {rsConfig: {priority: 0, votes: 0}}],
@@ -39,5 +35,4 @@ assert.commandWorked(
 
 // Node 1 re-checks isSelf on next heartbeat and succeeds.
 waitForState(restartNode, ReplSetTest.State.SECONDARY);
-rst.stopSet();
-})();
+rst.stopSet();
\ No newline at end of file
