@@ -38,12 +38,12 @@
  * of each stage.
  */
 
-"use strict";
-
-load("jstests/replsets/rslib.js");
-load("jstests/replsets/libs/two_phase_drops.js");
-load("jstests/hooks/validate_collections.js");
-load('jstests/libs/fail_point_util.js');
+import {CollectionValidator} from "jstests/hooks/validate_collections.js";
+import {configureFailPoint} from "jstests/libs/fail_point_util.js";
+import {FeatureFlagUtil} from "jstests/libs/feature_flag_util.js";
+import {restartServerReplication, stopServerReplication} from "jstests/libs/write_concern_util.js";
+import {TwoPhaseDropCollectionTest} from "jstests/replsets/libs/two_phase_drops.js";
+import {waitForState} from "jstests/replsets/rslib.js";
 
 /**
  *
@@ -70,7 +70,7 @@ load('jstests/libs/fail_point_util.js');
  * @param {Object} [optional] nodeOptions command-line options to apply to all nodes in the replica
  *     set. Ignored if 'replSet' is provided.
  */
-function RollbackTest(name = "RollbackTest", replSet, nodeOptions) {
+export function RollbackTest(name = "RollbackTest", replSet, nodeOptions) {
     const State = {
         kStopped: "kStopped",
         kRollbackOps: "kRollbackOps",
@@ -107,6 +107,30 @@ function RollbackTest(name = "RollbackTest", replSet, nodeOptions) {
 
     // Make sure we have a replica set up and running.
     replSet = (replSet === undefined) ? performStandardSetup(nodeOptions) : replSet;
+
+    // Runs the passed in cmdObj on db with a security token if multitenancy support is activated.
+    let runWithTenantIfNeeded = (function() {
+        const adminDB = replSet.getPrimary().getDB("admin");
+        const featureFlagRequireTenantID = FeatureFlagUtil.isEnabled(adminDB, "RequireTenantID");
+        const featureFlagSecurityToken = FeatureFlagUtil.isEnabled(adminDB, "SecurityToken");
+        const multitenancyDoc =
+            assert.commandWorked(adminDB.adminCommand({getParameter: 1, multitenancySupport: 1}));
+        if (multitenancyDoc.hasOwnProperty("multitenancySupport") &&
+            multitenancyDoc.multitenancySupport && featureFlagRequireTenantID &&
+            featureFlagSecurityToken) {
+            return function(dbConn, cmdObj) {
+                const tenantId = ObjectId();
+                dbConn.getMongo()._setSecurityToken(_createTenantToken({tenant: tenantId}));
+                assert.commandWorked(dbConn.runCommand(cmdObj));
+                dbConn.getMongo()._setSecurityToken(undefined);
+            };
+        } else {
+            return function(dbConn, cmdObj) {
+                assert.commandWorked(dbConn.runCommand(cmdObj));
+            };
+        }
+    })();
+
     validateAndUseSetup(replSet);
 
     // Majority writes in the initial phase, before transitionToRollbackOperations(), should be
@@ -181,9 +205,12 @@ function RollbackTest(name = "RollbackTest", replSet, nodeOptions) {
         // ensure the insert was replicated and written to the on-disk journal of all 3
         // nodes, with the exception of ephemeral and in-memory storage engines where
         // journaling isn't supported.
-        assert.commandWorked(curPrimary.getDB(dbName).ensureSyncSource.insert(
-            {thisDocument: 'is inserted to ensure any node can sync from any other'},
-            {writeConcern: {w: 3, j: config.writeConcernMajorityJournalDefault}}));
+
+        runWithTenantIfNeeded(curPrimary.getDB(dbName), {
+            insert: "ensureSyncSource",
+            documents: [{thisDocument: 'is inserted to ensure any node can sync from any other'}],
+            writeConcern: {w: 3, j: config.writeConcernMajorityJournalDefault}
+        });
     }
 
     /**
@@ -194,26 +221,10 @@ function RollbackTest(name = "RollbackTest", replSet, nodeOptions) {
      * maximum of 50 milliseconds.
      */
     function setFastGetMoreEnabled(node) {
-        assert.soon(
-            () => {
-                try {
-                    assert.commandWorked(
-                        node.adminCommand({
-                            configureFailPoint: "setSmallOplogGetMoreMaxTimeMS",
-                            mode: "alwaysOn"
-                        }),
-                        `Failed to enable setSmallOplogGetMoreMaxTimeMS failpoint.`,
-                    );
-                    return true;
-                } catch (e) {
-                    if (isNetworkError(e)) {
-                        return false;
-                    }
-                    throw e;
-                }
-            },
-            `Failed to set failpoint "setSmallOplogGetMoreMaxTimeMS" on ${tojson(node.host)}`,
-        );
+        assert.commandWorked(
+            node.adminCommand(
+                {configureFailPoint: 'setSmallOplogGetMoreMaxTimeMS', mode: 'alwaysOn'}),
+            `Failed to enable setSmallOplogGetMoreMaxTimeMS failpoint.`);
     }
 
     /**
@@ -269,6 +280,7 @@ function RollbackTest(name = "RollbackTest", replSet, nodeOptions) {
 
         const name = rst.name;
         rst.checkOplogs(name);
+        rst.checkPreImageCollection(name);
         rst.checkReplicatedDataHashes(name);
         collectionValidator.validateNodes(rst.nodeList());
     };
@@ -470,10 +482,12 @@ function RollbackTest(name = "RollbackTest", replSet, nodeOptions) {
         // ensure that this document is not lost due to unclean shutdowns. Ephemeral and in-memory
         // storage engines are an exception because journaling isn't supported.
         let writeConcern = TestData.rollbackShutdowns ? {w: 1, j: true} : {w: 1};
-        let dbName = "EnsureThereIsAtLeastOneOperationToRollback";
-        assert.commandWorked(curPrimary.getDB(dbName).ensureRollback.insert(
-            {thisDocument: 'is inserted to ensure rollback is not skipped'},
-            {writeConcern: writeConcern}));
+        let dbName = "EnsureThereIsAtLeastOneOpToRollback";
+        runWithTenantIfNeeded(curPrimary.getDB(dbName), {
+            insert: "ensureRollback",
+            documents: [{thisDocument: 'is inserted to ensure rollback is not skipped'}],
+            writeConcern
+        });
 
         log(`Isolating the primary ${curPrimary.host} so it will step down`);
         // We should have already disconnected the primary from the secondary during the first stage
