@@ -35,11 +35,9 @@
  * of restarts.
  */
 
-"use strict";
-
-load("jstests/hooks/validate_collections.js");
-load("jstests/replsets/libs/two_phase_drops.js");
-load("jstests/replsets/rslib.js");
+import {CollectionValidator} from "jstests/hooks/validate_collections.js";
+import {TwoPhaseDropCollectionTest} from "jstests/replsets/libs/two_phase_drops.js";
+import {waitForState} from "jstests/replsets/rslib.js";
 
 Random.setRandomSeed();
 
@@ -58,7 +56,7 @@ Random.setRandomSeed();
  * @param {Object} [optional] nodeOptions command-line options to apply to all nodes in the replica
  *     set. Ignored if 'replSet' is provided.
  */
-function RollbackTestDeluxe(name = "FiveNodeDoubleRollbackTest", replSet, nodeOptions) {
+export function RollbackTestDeluxe(name = "FiveNodeDoubleRollbackTest", replSet, nodeOptions) {
     const State = {
         kStopped: "kStopped",
         kRollbackOps: "kRollbackOps",
@@ -297,12 +295,13 @@ function RollbackTestDeluxe(name = "FiveNodeDoubleRollbackTest", replSet, nodeOp
             case RollbackTestDeluxe.RoleCycleMode.kFixedRollbackSecondary:
                 [standbySecondary, curPrimary] = [curPrimary, standbySecondary];
                 break;
-            case RollbackTestDeluxe.RoleCycleMode.kRandom:
+            case RollbackTestDeluxe.RoleCycleMode.kRandom: {
                 let oldStandbySecondary = standbySecondary;
                 [standbySecondary, rollbackSecondary] =
                     Array.shuffle([curPrimary, rollbackSecondary]);
                 curPrimary = oldStandbySecondary;
                 break;
+            }
             default:
                 throw new Error(`Unknown role cycle mode ${curRoleCycleMode}`);
         }
