@@ -1,9 +1,6 @@
 // Test ReplSet default initiate with localhost-only binding
 // @tags: [multiversion_incompatible]
 
-(function() {
-'use strict';
-
 // Select localhost when binding to localhost
 const rt = new ReplSetTest({name: "rsLocal", nodes: 1});
 const primary = rt.startSet({bind_ip: undefined})[0];
@@ -13,5 +10,4 @@ assert(resp.me.startsWith('localhost:'), tojson(resp.me) + " should start with l
 
 // Wait for the primary to complete its election before shutting down the set.
 assert.soon(() => db.runCommand({hello: 1}).isWritablePrimary);
-rt.stopSet();
-})();
+rt.stopSet();
\ No newline at end of file
