@@ -8,9 +8,6 @@
  *    requires_fcv_53
  *  ]
  */
-(function() {
-'use strict';
-
 const rst = new ReplSetTest({nodes: 1});
 rst.startSet();
 rst.initiate();
@@ -124,5 +121,4 @@ try {
         {'configureFailPoint': 'hangBeforeMajorityReadTransactionStarted', 'mode': 'off'}));
 }
 
-rst.stopSet();
-})();
+rst.stopSet();
\ No newline at end of file
