@@ -6,10 +6,13 @@
  * ]
  */
 
-(function() {
-"use strict";
-load("jstests/libs/fail_point_util.js");
-load('jstests/replsets/rslib.js');
+import {configureFailPoint, kDefaultWaitForFailPointTimeout} from "jstests/libs/fail_point_util.js";
+import {
+    assertVoteCount,
+    isMemberNewlyAdded,
+    reconfig,
+    waitForNewlyAddedRemovalForNodeToBeCommitted,
+} from "jstests/replsets/rslib.js";
 
 const testName = jsTestName();
 const dbName = "testdb";
@@ -164,4 +167,3 @@ assert.commandWorked(primaryColl.insert({"steady": "state"}, {writeConcern: {w:
 
 rst.awaitReplication();
 rst.stopSet();
-})();
