@@ -6,12 +6,9 @@
  *  - via heartbeats:
  *      these nodes should include null lastApplied and lastDurable optimes in heartbeat responses
  */
-(function() {
-"use strict";
-
-load("jstests/libs/write_concern_util.js");
-load("jstests/libs/fail_point_util.js");
-load('jstests/replsets/rslib.js');
+import {kDefaultWaitForFailPointTimeout} from "jstests/libs/fail_point_util.js";
+import {checkWriteConcernTimedOut} from "jstests/libs/write_concern_util.js";
+import {reconfig} from "jstests/replsets/rslib.js";
 
 const testName = jsTestName();
 const rst = new ReplSetTest({name: testName, nodes: [{}, {rsConfig: {priority: 0}}]});
@@ -62,7 +59,9 @@ assert.commandWorked(secondary.adminCommand({
 
 jsTestLog("Inserting some docs on the primary to advance its lastApplied");
 
-assert.commandWorked(primaryDb.test.insert([{a: 1}, {b: 2}, {c: 3}, {d: 4}, {e: 5}]));
+// We insert these one at a time to avoid batching of inserts.
+[{a: 1}, {b: 2}, {c: 3}, {d: 4}, {e: 5}].forEach(
+    doc => assert.commandWorked(primaryDb.test.insert([doc])));
 
 jsTestLog("Resuming initial sync");
 
@@ -148,4 +147,3 @@ assert.commandWorked(
 
 rst.restart(1);
 rst.stopSet();
-})();
