@@ -6,10 +6,7 @@
  *   requires_persistence,
  * ]
  */
-(function() {
-"use strict";
-
-load("jstests/libs/global_snapshot_reads_util.js");
+import {SnapshotReadsTest} from "jstests/libs/global_snapshot_reads_util.js";
 
 const options = {
     // Set a large snapshot window of 10 minutes for the test.
@@ -48,5 +45,4 @@ assert(!cursor.hasOwnProperty("atClusterTime"));
 const distinctResult = assert.commandWorked(primaryDB.runCommand({distinct: "test", key: "_id"}));
 assert(!distinctResult.hasOwnProperty("atClusterTime"));
 
-replSet.stopSet();
-})();
+replSet.stopSet();
\ No newline at end of file
