@@ -5,9 +5,6 @@
  * secondary.
  */
 
-(function() {
-"use strict";
-
 const testName = "opcounters_repl";
 const dbName = testName;
 const rst = new ReplSetTest({name: testName, nodes: 2});
@@ -101,5 +98,4 @@ assert.eq(diff.secondary.command, 0);
 assert.eq(diff.primary.insert, 1);
 assert.eq(diff.secondary.insert, 1);
 
-rst.stopSet();
-})();
+rst.stopSet();
\ No newline at end of file
