@@ -1,7 +1,5 @@
 // oplog should contain the field "wt" with wallClock timestamps.
-(function() {
-'use strict';
-load('jstests/replsets/rslib.js');
+import {getLatestOp} from "jstests/replsets/rslib.js";
 
 var assertLastOplogHasWT = function(primary, msg) {
     const opLogEntry = getLatestOp(primary);
@@ -26,5 +24,4 @@ assertLastOplogHasWT(primary, 'update');
 assert.commandWorked(collection.remove({_id: 1}));
 assertLastOplogHasWT(primary, 'remove');
 
-replSet.stopSet();
-})();
+replSet.stopSet();
\ No newline at end of file
