@@ -1,6 +1,8 @@
 // Tests tracking of latestOptime and earliestOptime in serverStatus.oplog
 // Also tests tracking of wall clock times in replSetGetStatus
 
+import {FeatureFlagUtil} from "jstests/libs/feature_flag_util.js";
+
 function timestampCompare(o1, o2) {
     if (o1.t < o2.t) {
         return -1;
@@ -28,22 +30,34 @@ function wallTimeCompare(d1, d2) {
 }
 
 function optimesAndWallTimesAreEqual(replTest, isPersistent) {
+    const reduceMajorityWriteLatency =
+        FeatureFlagUtil.isPresentAndEnabled(replTest.getPrimary(), "ReduceMajorityWriteLatency");
     let prevReplStatus = replTest.nodes[0].getDB('admin').runCommand({replSetGetStatus: 1});
     let prevOptime = prevReplStatus.optimes.appliedOpTime.ts;
     let prevAppliedWallTime = prevReplStatus.optimes.lastAppliedWallTime;
     let prevDurableWallTime = prevReplStatus.optimes.lastDurableWallTime;
+    let prevWrittenWallTime =
+        (reduceMajorityWriteLatency) ? prevReplStatus.optimes.lastWrittenWallTime : null;
     for (var i = 1; i < replTest.nodes.length; i++) {
         let currentReplStatus = replTest.nodes[i].getDB('admin').runCommand({replSetGetStatus: 1});
         let currOptime = currentReplStatus.optimes.appliedOpTime.ts;
         let currAppliedWallTime = currentReplStatus.optimes.lastAppliedWallTime;
         let currDurableWallTime = currentReplStatus.optimes.lastDurableWallTime;
+        let currWrittenWallTime =
+            (reduceMajorityWriteLatency) ? currentReplStatus.optimes.lastWrittenWallTime : null;
         if (timestampCompare(prevOptime, currOptime) != 0 ||
             wallTimeCompare(prevAppliedWallTime, currAppliedWallTime) != 0 ||
+            // If ReduceMajorityWriteLatency is set, the prevWrittenWallTime and
+            // currWrittenWallTime will not be null, so they'll be truthy.
+            (prevWrittenWallTime && currWrittenWallTime &&
+             wallTimeCompare(prevWrittenWallTime, currWrittenWallTime) != 0) ||
             (isPersistent && wallTimeCompare(prevDurableWallTime, currDurableWallTime) != 0)) {
             jsTest.log("optimesAndWallTimesAreEqual returning false match, prevOptime: " +
                        tojson(prevOptime) + " latestOptime: " + tojson(currOptime) +
                        " prevAppliedWallTime: " + tojson(prevAppliedWallTime) +
-                       " latestWallTime: " + tojson(currAppliedWallTime) +
+                       " latestAppliedWallTime: " + tojson(currAppliedWallTime) +
+                       " prevWrittenAppliedWallTime: " + tojson(prevWrittenWallTime) +
+                       " latestWrittenWallTime: " + tojson(currWrittenWallTime) +
                        " prevDurableWallTime: " + tojson(prevDurableWallTime) +
                        " latestDurableWallTime: " + tojson(currDurableWallTime));
             replTest.dumpOplog(replTest.nodes[i], {}, 20);
