@@ -1,7 +1,5 @@
 // Tests the output of db.printSecondaryReplicationInfo() for unreachable secondaries.
 
-(function() {
-"use strict";
 const name = "printSecondaryReplicationInfo";
 const replSet = new ReplSetTest({name: name, nodes: 2});
 replSet.startSet();
@@ -21,5 +19,4 @@ joinShell();
 assert(
     rawMongoProgramOutput().match("no replication info, yet.  State: \\(not reachable/healthy\\)"));
 
-replSet.stopSet();
-})();
+replSet.stopSet();
\ No newline at end of file
