@@ -23,17 +23,11 @@
  *
  * @tags: [
  *   requires_majority_read_concern,
- *   # This test is incompatible with earlier implementations of point-in-time catalog lookups.
- *   requires_fcv_70,
  * ]
  */
 
-load("jstests/libs/parallelTester.js");  // For Thread.
-load("jstests/libs/write_concern_util.js");
-load("jstests/libs/feature_flag_util.js");
-
-(function() {
-"use strict";
+import {Thread} from "jstests/libs/parallelTester.js";
+import {restartServerReplication, stopServerReplication} from "jstests/libs/write_concern_util.js";
 
 // Each test case includes a 'prepare' method that sets up the initial state starting with a
 // database that has been dropped, a 'performOp' method that does some operation, and two
@@ -208,17 +202,9 @@ const testCases = {
 // Assertion helpers. These must get all state as arguments rather than through closure since
 // they may be passed in to a Thread.
 function assertReadsBlock(db, coll) {
-    var res = coll.runCommand('find', {"readConcern": {"level": "majority"}, "maxTimeMS": 5000});
-
-    // When point-in-time catalog reads are enabled, reads no longer block waiting for the majority
-    // commit point to advance and allow reading earlier than the minimum visible snapshot.
-    if (FeatureFlagUtil.isEnabled(db, "PointInTimeCatalogLookups")) {
-        assert.commandWorked(res);
-    } else {
-        assert.commandFailedWithCode(res,
-                                     ErrorCodes.MaxTimeMSExpired,
-                                     "Expected read of " + coll.getFullName() + " to block");
-    }
+    // With point-in-time catalog lookups, reads no longer block waiting for the majority commit
+    // point to advance.
+    assert.commandWorked(coll.runCommand('find', {"readConcern": {"level": "majority"}}));
 }
 
 function assertReadsSucceed(coll, timeoutMs = 20000) {
@@ -351,4 +337,3 @@ for (var testName in testCases) {
 }
 
 replTest.stopSet();
-}());
