@@ -4,9 +4,6 @@
  *
  * @tags: [requires_majority_read_concern, requires_fcv_53]
  */
-(function() {
-"use strict";
-
 function log(arg) {
     jsTest.log(tojson(arg));
 }
@@ -63,5 +60,4 @@ assert.eq(doDirtyRead(), 0);
 jsTest.log("doing committed read");
 assert.eq(doCommittedRead(), 0);
 jsTest.log("stopping replTest; test completed successfully");
-replTest.stopSet();
-}());
+replTest.stopSet();
\ No newline at end of file
