@@ -3,10 +3,8 @@
  * previous config to be committed in the current config even if we are exiting a config that was
  * installed via a 'force' reconfig.
  */
-(function() {
-"use strict";
-load("jstests/libs/write_concern_util.js");
-load("jstests/replsets/rslib.js");
+import {restartServerReplication, stopServerReplication} from "jstests/libs/write_concern_util.js";
+import {isConfigCommitted, reconnect} from "jstests/replsets/rslib.js";
 
 const dbName = "test";
 const collName = "coll";
@@ -77,5 +75,4 @@ assert.commandWorked(primary.adminCommand({replSetReconfig: twoNodeConfig}));
 assert.soon(() => isConfigCommitted(primary));
 rst.awaitReplication();
 
-rst.stopSet();
-}());
+rst.stopSet();
\ No newline at end of file
