@@ -16,11 +16,8 @@
  * ]
  */
 
-(function() {
-"use strict";
-
-load("jstests/core/txns/libs/prepare_helpers.js");
-load("jstests/libs/fail_point_util.js");
+import {PrepareHelpers} from "jstests/core/txns/libs/prepare_helpers.js";
+import {kDefaultWaitForFailPointTimeout} from "jstests/libs/fail_point_util.js";
 
 const replTest = new ReplSetTest({nodes: 3});
 replTest.startSet();
@@ -111,4 +108,3 @@ replTest.awaitReplication();
 assert.docEq({_id: 1, a: 1}, secondaryColl.findOne({_id: 1}));
 
 replTest.stopSet();
-})();
\ No newline at end of file
