@@ -4,9 +4,6 @@
  *
  * @tags: [requires_persistence, requires_replication]
  */
-(function() {
-"use strict";
-
 const rst = new ReplSetTest({
     name: "recoveryAfterCleanShutdown",
     nodes: 2,
@@ -77,5 +74,4 @@ assert.commandWorked(
     primaryDB[collCreatedAfterRestart].insert({_id: "insertedAfterRestart", wMajority}));
 
 // Fast metadata count should be correct after restart in the face of a clean shutdown.
-rst.stopSet();
-}());
+rst.stopSet();
\ No newline at end of file
