@@ -7,9 +7,6 @@
  * ]
  */
 
-(function() {
-"use strict";
-
 jsTestLog("Testing that the reindex command cannot be run on a primary or secondary");
 
 const replTest = new ReplSetTest({name: 'reindexTest', nodes: 2});
@@ -71,5 +68,4 @@ assert.commandFailedWithCode(nonExistentDb.getCollection('test').reIndex(),
 
 assert.eq(2, testColl.getIndexes().length, "Standalone didn't have proper indexes after reindex");
 
-MongoRunner.stopMongod(standalone);
-})();
+MongoRunner.stopMongod(standalone);
\ No newline at end of file
