@@ -1,8 +1,5 @@
 // Election when primary fails and remaining nodes are an arbiter and a secondary.
 
-(function() {
-"use strict";
-
 var replTest = new ReplSetTest({name: 'unicomplex', nodes: 3});
 var nodes = replTest.nodeList();
 
@@ -44,5 +41,4 @@ var new_primary = replTest.getPrimary();
 var newPrimaryId = replTest.getNodeId(new_primary);
 assert.neq(newPrimaryId, pId, "Secondary wasn't promoted to new primary");
 
-replTest.stopSet(15);
-}());
+replTest.stopSet(15);
\ No newline at end of file
