@@ -5,9 +5,7 @@
  * a single node against the performance with many nodes, to see how the fixture scales. See the
  * 'replsettest_control_12_nodes.js' test.
  */
-(function() {
 const replTest = new ReplSetTest({name: 'replsettest_control_1_node', nodes: 1});
 replTest.startSet();
 replTest.initiate();
-replTest.stopSet();
-})();
\ No newline at end of file
+replTest.stopSet();
\ No newline at end of file
