@@ -6,9 +6,6 @@
  * ]
  */
 
-(function() {
-"use strict";
-
 const rst = new ReplSetTest({
     nodes: 1,
     nodeOptions: {setParameter: {logComponentVerbosity: tojson({command: 2, query: 5})}}
@@ -22,9 +19,11 @@ const dbName = "test";
 const collName = jsTestName();
 
 jsTestLog("Inserting some data");
-// We will query the oplog for the entries corresponding to those inserts.
+// We will query the oplog for the entries corresponding to those inserts.  We insert one at time
+// to avoid batching of vectored inserts.
 const testData = [{_id: 0, ans: 42}, {_id: 1, ans: 42}, {_id: 2, ans: 42}];
-assert.commandWorked(node.getDB(dbName).getCollection(collName).insert(testData));
+testData.forEach(
+    doc => assert.commandWorked(node.getDB(dbName).getCollection(collName).insert([doc])));
 
 const localDb = node.getDB("local");
 const kNullTS = new Timestamp(0, 0);
@@ -301,4 +300,3 @@ jsTestLog("Running query on the oplog with an empty batch");
 }
 
 rst.stopSet();
-})();
