@@ -3,11 +3,7 @@
  *
  * @tags: [uses_transactions, uses_prepare_transaction]
  */
-(function() {
-
-"use strict";
-
-load("jstests/libs/write_concern_util.js");
+import {runWriteConcernRetryabilityTest} from "jstests/libs/write_concern_util.js";
 
 const kNodes = 2;
 
@@ -206,5 +202,4 @@ runWriteConcernRetryabilityTest(priConn,
                                 kNodes,
                                 'admin');
 
-replTest.stopSet();
-})();
+replTest.stopSet();
\ No newline at end of file
