@@ -19,9 +19,7 @@
  * ]
  */
 
-(function() {
-
-load("jstests/replsets/libs/rollback_test.js");
+import {RollbackTest} from "jstests/replsets/libs/rollback_test.js";
 
 const testName = "rollback_drop_database";
 
@@ -103,5 +101,4 @@ rollbackTest.stepUpNode(rollbackNode);
 assert.commandWorked(rollbackNode.getDB(conflictingDbName)["afterRollback"].insert(
     {"num": 2}, {writeConcern: {w: 2}}));
 
-rollbackTest.stop();
-})();
+rollbackTest.stop();
\ No newline at end of file
