@@ -1,10 +1,7 @@
 // When run with --majorityReadConcern=off, this test reproduces the bug described in SERVER-38925,
 // where rolling back a delete followed by a restart produces documents with duplicate _id.
 // @tags: [requires_persistence]
-(function() {
-"use strict";
-
-load("jstests/replsets/libs/rollback_test.js");
+import {RollbackTest} from "jstests/replsets/libs/rollback_test.js";
 
 TestData.rollbackShutdowns = true;
 TestData.allowUncleanShutdowns = true;
@@ -39,5 +36,4 @@ rollbackTest.transitionToSteadyStateOperations();
 rollbackTest.restartNode(0, 9);
 
 // Check the replica set.
-rollbackTest.stop();
-}());
+rollbackTest.stop();
\ No newline at end of file
