@@ -9,18 +9,14 @@
  *  @tags: [multiversion_incompatible]
  */
 
-(function() {
-"use strict";
-
-load("jstests/replsets/libs/rollback_test.js");
-load('jstests/libs/parallel_shell_helpers.js');
-load("jstests/libs/fail_point_util.js");
-load("jstests/replsets/rslib.js");
-load("jstests/libs/feature_flag_util.js");
+import {configureFailPoint} from "jstests/libs/fail_point_util.js";
+import {funWithArgs} from "jstests/libs/parallel_shell_helpers.js";
+import {RollbackTest} from "jstests/replsets/libs/rollback_test.js";
 
 function setFCV(fcv) {
-    assert.commandFailedWithCode(db.adminCommand({setFeatureCompatibilityVersion: fcv}),
-                                 ErrorCodes.InterruptedDueToReplStateChange);
+    assert.commandFailedWithCode(
+        db.adminCommand({setFeatureCompatibilityVersion: fcv, confirm: true}),
+        ErrorCodes.InterruptedDueToReplStateChange);
 }
 
 // Using getParameter results in waiting for the current FCV to be majority committed.  In this
@@ -48,7 +44,8 @@ function rollbackFCVFromDowngradingOrUpgrading(fromFCV, toFCV) {
     let secondaryAdminDB = secondary.getDB('admin');
 
     // Ensure the cluster starts at the correct FCV.
-    assert.commandWorked(primary.adminCommand({setFeatureCompatibilityVersion: toFCV}));
+    assert.commandWorked(
+        primary.adminCommand({setFeatureCompatibilityVersion: toFCV, confirm: true}));
     // Wait until the config has propagated to the other nodes and the primary has learned of it, so
     // that the config replication check in 'setFeatureCompatibilityVersion' is satisfied. This is
     // only important since 'setFeatureCompatibilityVersion' is known to implicitly call internal
@@ -96,7 +93,8 @@ function rollbackFCVFromDowngradingOrUpgrading(fromFCV, toFCV) {
     // As a rule, we forbid downgrading a node while a node is still in the upgrading state and
     // vice versa. Ensure that the in-memory and on-disk FCV are consistent by checking that we are
     // able to set the FCV back to the original version.
-    assert.commandWorked(newPrimary.adminCommand({setFeatureCompatibilityVersion: toFCV}));
+    assert.commandWorked(
+        newPrimary.adminCommand({setFeatureCompatibilityVersion: toFCV, confirm: true}));
 }
 
 // fromFCV refers to the FCV we will test rolling back from.
@@ -107,13 +105,9 @@ function rollbackFCVFromDowngradedOrUpgraded(fromFCV, toFCV, failPoint) {
     let primaryAdminDB = primary.getDB('admin');
     let secondaryAdminDB = secondary.getDB('admin');
 
-    const isDowngradingToUpgradingFlagOn = FeatureFlagUtil.isEnabled(primaryAdminDB,
-                                                                     "DowngradingToUpgrading",
-                                                                     null /* user not specified */,
-                                                                     true /* ignores FCV */);
-
     // Complete the upgrade/downgrade to ensure we are not in the upgrading/downgrading state.
-    assert.commandWorked(primary.adminCommand({setFeatureCompatibilityVersion: toFCV}));
+    assert.commandWorked(
+        primary.adminCommand({setFeatureCompatibilityVersion: toFCV, confirm: true}));
     // Wait for the majority commit point to be updated on the secondary, because checkFCV calls
     // getParameter for the featureCompatibilityVersion, which will wait until the FCV change makes
     // it into the node's majority committed snapshot.
@@ -137,7 +131,7 @@ function rollbackFCVFromDowngradedOrUpgraded(fromFCV, toFCV, failPoint) {
     }, "Failed waiting for server to unset the targetVersion or to set the FCV to " + fromFCV);
     rollbackTest.transitionToSyncSourceOperationsBeforeRollback();
     // The secondary should never have received the update to unset the targetVersion.
-    if (fromFCV == lastLTSFCV && isDowngradingToUpgradingFlagOn) {
+    if (fromFCV == lastLTSFCV) {
         // When downgrading, the secondary should still be in isCleaningServerMetadata.
         checkFCV(secondaryAdminDB, lastLTSFCV, fromFCV, true /* isCleaningServerMetadata */);
     } else {
@@ -159,7 +153,7 @@ function rollbackFCVFromDowngradedOrUpgraded(fromFCV, toFCV, failPoint) {
     assert.eq(topologyVersionBeforeRollback.counter + topologyVersionDiff,
               topologyVersionAfterRollback.counter);
     // The primary should have rolled back their FCV to contain the targetVersion.
-    if (fromFCV == lastLTSFCV && isDowngradingToUpgradingFlagOn) {
+    if (fromFCV == lastLTSFCV) {
         // Rolling back from downgraded to isCleaningServerMetadata state.
         checkFCV(primaryAdminDB, lastLTSFCV, fromFCV, true /* isCleaningServerMetadata */);
         checkFCV(secondaryAdminDB, lastLTSFCV, fromFCV, true /* isCleaningServerMetadata */);
@@ -175,12 +169,14 @@ function rollbackFCVFromDowngradedOrUpgraded(fromFCV, toFCV, failPoint) {
     // server metadata.
     // Ensure that the in-memory and on-disk FCV are consistent by checking that this rule is
     // upheld after rollback.
-    if (fromFCV === lastLTSFCV && toFCV === latestFCV && isDowngradingToUpgradingFlagOn) {
+    if (fromFCV === lastLTSFCV && toFCV === latestFCV) {
         assert.commandFailedWithCode(
-            newPrimary.adminCommand({setFeatureCompatibilityVersion: toFCV}), 7428200);
+            newPrimary.adminCommand({setFeatureCompatibilityVersion: toFCV, confirm: true}),
+            7428200);
     } else {
         assert.commandFailedWithCode(
-            newPrimary.adminCommand({setFeatureCompatibilityVersion: toFCV}), 5147403);
+            newPrimary.adminCommand({setFeatureCompatibilityVersion: toFCV, confirm: true}),
+            5147403);
     }
 }
 
@@ -196,13 +192,8 @@ function rollbackFCVFromUpgradingToDowngrading() {
     const syncSourceAdminDB = syncSource.getDB('admin');
 
     // Ensure the cluster starts at the correct FCV.
-    assert.commandWorked(rollbackNode.adminCommand({setFeatureCompatibilityVersion: latestFCV}));
-
-    if (!FeatureFlagUtil.isEnabled(rollbackNodeAdminDB, "DowngradingToUpgrading")) {
-        jsTestLog(
-            "Skipping rollbackFCVFromUpgradingToDowngrading as featureFlagDowngradingToUpgrading is not enabled");
-        return;
-    }
+    assert.commandWorked(
+        rollbackNode.adminCommand({setFeatureCompatibilityVersion: latestFCV, confirm: true}));
 
     fcvDoc = rollbackNodeAdminDB.system.version.findOne({_id: 'featureCompatibilityVersion'});
     jsTestLog(`rollbackNode's version at start: ${tojson(fcvDoc)}`);
@@ -215,8 +206,8 @@ function rollbackFCVFromUpgradingToDowngrading() {
         rollbackNode.adminCommand({configureFailPoint: "failUpgrading", mode: "alwaysOn"}));
 
     // Go to downgrading state (downgrading from latest to lastLTS).
-    assert.commandFailed(
-        rollbackNodeAdminDB.runCommand({setFeatureCompatibilityVersion: lastLTSFCV}));
+    assert.commandFailed(rollbackNodeAdminDB.runCommand(
+        {setFeatureCompatibilityVersion: lastLTSFCV, confirm: true}));
 
     fcvDoc = rollbackNodeAdminDB.system.version.findOne({_id: 'featureCompatibilityVersion'});
     jsTestLog(`rollbackNode's version after downgrading: ${tojson(fcvDoc)}`);
@@ -289,7 +280,8 @@ function rollbackFCVFromUpgradingToDowngrading() {
     const newPrimary = rollbackTest.getPrimary();
     const newPrimaryAdminDB = newPrimary.getDB('admin');
     // We should now be able to set the FCV from downgrading to upgrading to upgraded.
-    assert.commandWorked(newPrimary.adminCommand({setFeatureCompatibilityVersion: latestFCV}));
+    assert.commandWorked(
+        newPrimary.adminCommand({setFeatureCompatibilityVersion: latestFCV, confirm: true}));
     checkFCV(newPrimaryAdminDB, latestFCV);
 
     assert.commandWorked(
@@ -305,17 +297,9 @@ function rollbackFCVFromIsCleaningServerMetadataToDowngrading() {
     let primaryAdminDB = primary.getDB('admin');
     let secondaryAdminDB = secondary.getDB('admin');
 
-    if (!FeatureFlagUtil.isEnabled(primaryAdminDB,
-                                   "DowngradingToUpgrading",
-                                   null /* user not specified */,
-                                   true /* ignores FCV */)) {
-        jsTestLog(
-            "Skipping rollbackFCVFromIsCleaningServerMetadataToDowngrading test because isDowngradingToUpgrading is not enabled");
-        return;
-    }
-
     // Complete the upgrade/downgrade to ensure we are not in the upgrading/downgrading state.
-    assert.commandWorked(primary.adminCommand({setFeatureCompatibilityVersion: latestFCV}));
+    assert.commandWorked(
+        primary.adminCommand({setFeatureCompatibilityVersion: latestFCV, confirm: true}));
     // Wait for the majority commit point to be updated on the secondary, because checkFCV calls
     // getParameter for the featureCompatibilityVersion, which will wait until the FCV change makes
     // it into the node's majority committed snapshot.
@@ -360,7 +344,8 @@ function rollbackFCVFromIsCleaningServerMetadataToDowngrading() {
     let newPrimary = rollbackTest.getPrimary();
     // With the new downgrading to upgrading path, we can still go from downgrading -> upgrading
     // after rollback.
-    assert.commandWorked(newPrimary.adminCommand({setFeatureCompatibilityVersion: latestFCV}));
+    assert.commandWorked(
+        newPrimary.adminCommand({setFeatureCompatibilityVersion: latestFCV, confirm: true}));
 }
 
 const testName = jsTest.name();
@@ -387,4 +372,3 @@ rollbackFCVFromUpgradingToDowngrading();
 rollbackFCVFromIsCleaningServerMetadataToDowngrading();
 
 rollbackTest.stop();
-}());
