@@ -1,35 +1,33 @@
-var syncFrom;
-var wait;
-var occasionally;
-var reconnect;
-var getLatestOp;
-var waitForAllMembers;
-var reconfig;
-var safeReconfigShouldFail;
-var awaitOpTime;
-var waitUntilAllNodesCaughtUp;
-var waitForState;
-var reInitiateWithoutThrowingOnAbortedMember;
-var awaitRSClientHosts;
-var getLastOpTime;
-var getFirstOplogEntry;
-var setLogVerbosity;
-var stopReplicationAndEnforceNewPrimaryToCatchUp;
-var isConfigCommitted;
-var assertSameConfigContent;
-var isSameConfigContent;
-var getConfigWithNewlyAdded;
-var isMemberNewlyAdded;
-var waitForNewlyAddedRemovalForNodeToBeCommitted;
-var assertVoteCount;
-var disconnectSecondaries;
-var reconnectSecondaries;
-var createRstArgs;
-var createRst;
+import {restartServerReplication, stopServerReplication} from "jstests/libs/write_concern_util.js";
 
-(function() {
-"use strict";
-load("jstests/libs/write_concern_util.js");
+export var syncFrom;
+export var wait;
+export var occasionally;
+export var reconnect;
+export var getLatestOp;
+export var waitForAllMembers;
+export var reconfig;
+export var safeReconfigShouldFail;
+export var awaitOpTime;
+export var waitUntilAllNodesCaughtUp;
+export var waitForState;
+export var reInitiateWithoutThrowingOnAbortedMember;
+export var awaitRSClientHosts;
+export var getLastOpTime;
+export var getFirstOplogEntry;
+export var setLogVerbosity;
+export var stopReplicationAndEnforceNewPrimaryToCatchUp;
+export var isConfigCommitted;
+export var assertSameConfigContent;
+export var isSameConfigContent;
+export var getConfigWithNewlyAdded;
+export var isMemberNewlyAdded;
+export var waitForNewlyAddedRemovalForNodeToBeCommitted;
+export var assertVoteCount;
+export var disconnectSecondaries;
+export var reconnectSecondaries;
+export var createRstArgs;
+export var createRst;
 
 var count = 0;
 var w = 0;
@@ -129,13 +127,13 @@ reconnect = function(conn) {
         try {
             // Make this work with either dbs or connections.
             if (typeof (conn.getDB) == "function") {
-                db = conn.getDB('foo');
+                db = conn.getDB('config');
             } else {
                 db = conn;
             }
 
             // Run a simple command to re-establish connection.
-            db.bar.stats();
+            db.settings.stats();
 
             // SERVER-4241: Shell connections don't re-authenticate on reconnect.
             if (jsTest.options().keyFile) {
@@ -399,7 +397,6 @@ function autoReconfig(rst, targetConfig) {
  *     secondary, or arbiter states
  */
 reconfig = function(rst, config, force, doNotWaitForMembers) {
-    "use strict";
     var primary = rst.getPrimary();
     config = rst._updateConfigIfNotDurable(config);
 
@@ -897,4 +894,3 @@ createRst = function(rstArgs, retryOnRetryableErrors) {
         }
     }
 };
-}());
