@@ -7,10 +7,11 @@
  *   requires_persistence,
  *  ]
  */
-(function() {
-'use strict';
-
-load('jstests/libs/cluster_server_parameter_utils.js');
+import {
+    runGetClusterParameterNode,
+    runGetClusterParameterReplicaSet,
+    runSetClusterParameter,
+} from "jstests/libs/cluster_server_parameter_utils.js";
 
 // Checks that up-to-date cluster parameters are transferred over to newly-added replica set nodes
 // as part of initial sync.
@@ -28,6 +29,7 @@ function checkClusterParameterInitialSync(rst) {
     runSetClusterParameter(rst.getPrimary(), newStrParameter);
 
     // Check that the new values are visible on the majority of the nodes.
+    rst.awaitReplication();
     runGetClusterParameterReplicaSet(rst,
                                      ["testIntClusterParameter", "testStrClusterParameter"],
                                      [newIntParameter, newStrParameter]);
@@ -72,6 +74,7 @@ function checkClusterParameterRestart(rst) {
     runSetClusterParameter(rst.getPrimary(), newStrParameter);
 
     // Check that the new values are visible on the majority of the nodes.
+    rst.awaitReplication();
     runGetClusterParameterReplicaSet(rst,
                                      ["testIntClusterParameter", "testStrClusterParameter"],
                                      [newIntParameter, newStrParameter]);
@@ -97,4 +100,3 @@ checkClusterParameterInitialSync(rst);
 checkClusterParameterRestart(rst);
 
 rst.stopSet();
-})();
