@@ -4,9 +4,6 @@
  * @tags: [ requires_persistence ]
  */
 
-(function() {
-"use strict";
-
 // Start and configure mongod with invalid split horizons using override setting
 
 let startupConfig = {
@@ -69,5 +66,4 @@ jsTestLog("replSetReconfig output: " + tojson(output));
 assert.commandFailed(output);
 assert(output.errmsg.includes("Found split horizon configuration using IP"));
 
-MongoRunner.stopMongod(mongod);
-}());
\ No newline at end of file
+MongoRunner.stopMongod(mongod);
\ No newline at end of file
