@@ -9,9 +9,6 @@
  * ]
  */
 
-(function() {
-"use strict";
-
 const dbName = "test";
 const collName = jsTestName();
 
@@ -51,5 +48,4 @@ const primaryStandalone = MongoRunner.runMongod({
 // Test that the last insert is visible after replication recovery.
 assert.eq(primaryStandalone.getDB(dbName)[collName].find().toArray(), docs);
 
-MongoRunner.stopMongod(primaryStandalone);
-}());
+MongoRunner.stopMongod(primaryStandalone);
\ No newline at end of file
