@@ -6,9 +6,9 @@
  * @tags: [requires_persistence, requires_replication,
  * requires_majority_read_concern, uses_transactions, uses_prepare_transaction]
  */
-(function() {
-"use strict";
-load("jstests/replsets/libs/prepare_standalone_replication_recovery.js");
+import {
+    testPrepareRecoverFromOplogAsStandalone
+} from "jstests/replsets/libs/prepare_standalone_replication_recovery.js";
 
 const testName = "standalone_replication_recovery_prepare_only";
 
@@ -16,5 +16,4 @@ const testName = "standalone_replication_recovery_prepare_only";
 // fast count.
 TestData.skipEnforceFastCountOnValidate = true;
 
-testPrepareRecoverFromOplogAsStandalone(testName, /* commitBeforeRecovery */ false);
-})();
+testPrepareRecoverFromOplogAsStandalone(testName, /* commitBeforeRecovery */ false);
\ No newline at end of file
