@@ -6,11 +6,9 @@
  * @tags: [multiversion_incompatible]
  */
 
-(function() {
+import {configureFailPoint} from "jstests/libs/fail_point_util.js";
 
-load("jstests/libs/fail_point_util.js");
-
-rst = new ReplSetTest({nodes: 1});
+let rst = new ReplSetTest({nodes: 1});
 rst.startSet();
 rst.initiate();
 
@@ -40,5 +38,4 @@ secondary.setSecondaryOk();
 const secondaryAdminDb = secondary.getDB("admin");
 // Assert that the FCV document was cloned through initial sync on the secondary.
 checkFCV(secondaryAdminDb, latestFCV);
-rst.stopSet();
-}());
+rst.stopSet();
\ No newline at end of file
