@@ -266,6 +266,7 @@ export const TenantMigrationConcurrentWriteUtil = {
         _configsvrCommitMergeAllChunksOnShard: {skip: isNotRunOnUserDatabase},
         _configsvrCommitMovePrimary:
             {skip: isNotRunOnUserDatabase},  // Can be removed once 6.0 is last LTS
+        _configsvrCommitRefineCollectionShardKey: {skip: isNotRunOnUserDatabase},
         _configsvrCreateDatabase: {skip: isNotRunOnUserDatabase},
         _configsvrDropIndexCatalogEntry: {skip: isNotRunOnUserDatabase},
         _configsvrEnsureChunkVersionIsGreaterThan: {skip: isNotRunOnUserDatabase},
@@ -292,6 +293,8 @@ export const TenantMigrationConcurrentWriteUtil = {
         _recvChunkReleaseCritSec: {skip: isNotRunOnUserDatabase},
         _recvChunkStart: {skip: isNotRunOnUserDatabase},
         _recvChunkStatus: {skip: isNotRunOnUserDatabase},
+        _shardsvrChangePrimary: {skip: isNotRunOnUserDatabase},
+        _shardsvrCleanupStructuredEncryptionData: {skip: isOnlySupportedOnShardedCluster},
         _shardsvrCloneCatalogData: {skip: isNotRunOnUserDatabase},
         _shardsvrCommitIndexParticipant: {skip: isOnlySupportedOnShardedCluster},
         _shardsvrCompactStructuredEncryptionData: {skip: isOnlySupportedOnShardedCluster},
@@ -330,8 +333,10 @@ export const TenantMigrationConcurrentWriteUtil = {
         appendOplogNote: {skip: isNotRunOnUserDatabase},
         applyOps: {skip: isNotSupportedInServerless},
         authenticate: {skip: isAuthCommand},
+        autoCompact: {skip: isNotSupportedInServerless},
         buildInfo: {skip: isNotWriteCommand},
         captrunc: {skip: isNotSupportedInServerless},
+        changePrimary: {skip: isNotRunOnUserDatabase},
         checkShardingIndex: {skip: isNotRunOnUserDatabase},
         cleanupOrphaned: {skip: isNotRunOnUserDatabase},
         clearLog: {skip: isNotRunOnUserDatabase},
@@ -531,7 +536,6 @@ export const TenantMigrationConcurrentWriteUtil = {
         getQueryableEncryptionCountInfo: {skip: isNotWriteCommand},
         getShardMap: {skip: isNotRunOnUserDatabase},
         getShardVersion: {skip: isNotRunOnUserDatabase},
-        getnonce: {skip: "removed in v6.3"},
         godinsert: {skip: isNotRunOnUserDatabase},
         grantPrivilegesToRole: {skip: isAuthCommand},
         grantRolesToRole: {skip: isAuthCommand},
@@ -644,6 +648,7 @@ export const TenantMigrationConcurrentWriteUtil = {
         revokeRolesFromUser: {skip: isAuthCommand},
         rolesInfo: {skip: isNotWriteCommand},
         rotateCertificates: {skip: isAuthCommand},
+        rotateFTDC: {skip: isNotRunOnUserDatabase},
         saslContinue: {skip: isAuthCommand},
         saslStart: {skip: isAuthCommand},
         sbe: {skip: isNotRunOnUserDatabase},
@@ -710,7 +715,6 @@ export const TenantMigrationConcurrentWriteUtil = {
         voteAbortIndexBuild: {skip: isNotRunOnUserDatabase},
         voteCommitIndexBuild: {skip: isNotRunOnUserDatabase},
         waitForFailPoint: {skip: isNotRunOnUserDatabase},
-        waitForOngoingChunkSplits: {skip: isNotRunOnUserDatabase},
         whatsmysni: {skip: isNotRunOnUserDatabase},
         whatsmyuri: {skip: isNotRunOnUserDatabase}
     }
