@@ -4,7 +4,6 @@
  * We don't support unclean shutdowns with restarts into a last-lts binary.
  * @tags: [requires_persistence, multiversion_incompatible]
  */
-(function() {
 const rst = new ReplSetTest({
     nodes: 1,
     nodeOptions: {
@@ -29,5 +28,4 @@ rst.start(0, undefined, true /* restart */);
 jsTestLog("Waiting for the node to restart and become primary again.");
 rst.getPrimary();
 
-rst.stopSet();
-}());
+rst.stopSet();
\ No newline at end of file
