@@ -12,9 +12,10 @@
  * The second reconfig should then increase its priority to the desired level.
  */
 
-(function() {
-"use strict";
-load("jstests/replsets/rslib.js");
+import {
+    assertVoteCount,
+    waitForNewlyAddedRemovalForNodeToBeCommitted
+} from "jstests/replsets/rslib.js";
 
 {
     jsTestLog("Testing reconfig from PA set to PSA set fails");
@@ -163,5 +164,4 @@ load("jstests/replsets/rslib.js");
     });
 
     rst.stopSet();
-}
-})();
+}
\ No newline at end of file
