@@ -2,10 +2,10 @@
  * Tests that we are silently ignoring writeConcern when we write to local db.
  */
 
-(function() {
-'use strict';
-
-load("jstests/libs/write_concern_util.js");  // For stopReplicationOnSecondaries.
+import {
+    restartReplicationOnSecondaries,
+    stopReplicationOnSecondaries
+} from "jstests/libs/write_concern_util.js";
 
 const rst = new ReplSetTest(
     {nodes: [{}, {rsConfig: {priority: 0}}], nodeOptions: {setParameter: {logLevel: 1}}});
@@ -37,5 +37,4 @@ primaryColl.insertOne({x: 5}, {writeConcern: {w: 1}});
 primaryColl.insertOne({x: 6}, {writeConcern: {w: 2}});
 
 restartReplicationOnSecondaries(rst);
-rst.stopSet();
-})();
+rst.stopSet();
\ No newline at end of file
