@@ -2,9 +2,6 @@
  * Test that the server rejects extremely large values for write concern wtimeout.
  */
 
-(function() {
-"use strict";
-
 const rst = new ReplSetTest({name: jsTestName(), nodes: 2});
 rst.startSet();
 rst.initiateWithHighElectionTimeout();
@@ -28,5 +25,4 @@ assert.commandFailedWithCode(
     primaryColl.insert({b: 2}, {writeConcern: {w: 2, wtimeout: oneTrillionMS}}),
     ErrorCodes.FailedToParse);
 
-rst.stopSet();
-})();
+rst.stopSet();
\ No newline at end of file
