@@ -1,5 +1 @@
-(function() {
-'use strict';
-
 throw new Error("Explicitly forcing the JS test to fail.");
-})();
