@@ -9,10 +9,10 @@
  *   serverless
  *  ]
  */
-(function() {
-'use strict';
-
-load('jstests/libs/cluster_server_parameter_utils.js');
+import {
+    setupReplicaSet,
+    testDisabledClusterParameters
+} from "jstests/libs/cluster_server_parameter_utils.js";
 
 // Verifies that test-only parameters are disabled and excluded when enableTestCommands is false.
 TestData.enableTestCommands = false;
@@ -30,5 +30,4 @@ setupReplicaSet(rst);
 // getClusterParameter: '*' with and without a tenantId.
 testDisabledClusterParameters(rst);
 testDisabledClusterParameters(rst, ObjectId());
-rst.stopSet();
-}());
+rst.stopSet();
\ No newline at end of file
