@@ -1,9 +1,8 @@
+import {Thread} from "jstests/libs/parallelTester.js";
+import {extractUUIDFromObject} from "jstests/libs/uuid_util.js";
 import {TenantMigrationTest} from "jstests/replsets/libs/tenant_migration_test.js";
 import {makeX509OptionsForTest} from "jstests/replsets/libs/tenant_migration_util.js";
-
-load("jstests/replsets/rslib.js");
-load("jstests/libs/parallelTester.js");
-load("jstests/libs/uuid_util.js");
+import {createRst, createRstArgs} from "jstests/replsets/rslib.js";
 
 function runForgetShardSplitAsync(primaryHost, migrationIdString) {
     const primary = new Mongo(primaryHost);
@@ -20,8 +19,8 @@ function runAbortShardSplitAsync(primaryHost, migrationIdString) {
  * @param {rstArgs} replicaSetArgs for the replica set to connect to.
  * @param {tenantIds} perform a write operation for each tenantId.
  */
-export function doWriteOperations(rstArgs, tenantIds) {
-    load("jstests/replsets/rslib.js");
+export async function doWriteOperations(rstArgs, tenantIds) {
+    const {createRst} = await import("jstests/replsets/rslib.js");
 
     const donorRst = createRst(rstArgs, true);
     const donorPrimary = donorRst.getPrimary();
@@ -98,9 +97,8 @@ async function runCommitSplitThreadWrapper(rstArgs,
                                            tenantIds,
                                            recipientTagName,
                                            recipientSetName,
-                                           retryOnRetryableErrors,
-                                           enableDonorStartMigrationFsync) {
-    load("jstests/replsets/rslib.js");
+                                           retryOnRetryableErrors) {
+    const {createRst} = await import("jstests/replsets/rslib.js");
     const {runShardSplitCommand} = await import("jstests/serverless/libs/shard_split_test.js");
 
     const donorRst = createRst(rstArgs, true);
@@ -114,8 +112,7 @@ async function runCommitSplitThreadWrapper(rstArgs,
 
     jsTestLog(`Running async split command ${tojson(commitShardSplitCmdObj)}`);
 
-    return runShardSplitCommand(
-        donorRst, commitShardSplitCmdObj, retryOnRetryableErrors, enableDonorStartMigrationFsync);
+    return runShardSplitCommand(donorRst, commitShardSplitCmdObj, retryOnRetryableErrors);
 }
 
 /*
@@ -163,10 +160,8 @@ export function commitSplitAsync({
     recipientSetName,
     migrationId,
     retryOnRetryableErrors,
-    enableDonorStartMigrationFsync
 } = {
-    retryOnRetryableErrors: false,
-    enableDonorStartMigrationFsync: false
+    retryOnRetryableErrors: false
 }) {
     jsTestLog("Running commitAsync command");
 
@@ -179,20 +174,14 @@ export function commitSplitAsync({
                               tojson(tenantIds),
                               recipientTagName,
                               recipientSetName,
-                              retryOnRetryableErrors,
-                              enableDonorStartMigrationFsync);
+                              retryOnRetryableErrors);
     thread.start();
 
     return thread;
 }
 
-export function runShardSplitCommand(
-    replicaSet, cmdObj, retryOnRetryableErrors, enableDonorStartMigrationFsync) {
+export function runShardSplitCommand(replicaSet, cmdObj, retryOnRetryableErrors) {
     let res;
-    if (enableDonorStartMigrationFsync) {
-        replicaSet.awaitLastOpCommitted();
-        assert.commandWorked(replicaSet.getPrimary().adminCommand({fsync: 1}));
-    }
 
     assert.soon(() => {
         try {
@@ -248,8 +237,7 @@ class ShardSplitOperation {
      * Starts a shard split synchronously.
      */
 
-    commit({retryOnRetryableErrors} = {retryOnRetryableErrors: false},
-           {enableDonorStartMigrationFsync} = {enableDonorStartMigrationFsync: false}) {
+    commit({retryOnRetryableErrors} = {retryOnRetryableErrors: false}) {
         jsTestLog("Running commit command");
         const localCmdObj = {
             commitShardSplit: 1,
@@ -259,18 +247,14 @@ class ShardSplitOperation {
             recipientSetName: this.recipientSetName
         };
 
-        return runShardSplitCommand(
-            this.donorSet, localCmdObj, retryOnRetryableErrors, enableDonorStartMigrationFsync);
+        return runShardSplitCommand(this.donorSet, localCmdObj, retryOnRetryableErrors);
     }
 
     /**
      * Starts a shard split asynchronously and returns the Thread that runs it.
      * @returns the Thread running the commitShardSplit command.
      */
-    commitAsync({retryOnRetryableErrors, enableDonorStartMigrationFsync} = {
-        retryOnRetryableErrors: false,
-        enableDonorStartMigrationFsync: false
-    }) {
+    commitAsync({retryOnRetryableErrors} = {retryOnRetryableErrors: false}) {
         return commitSplitAsync({
             rst: this.donorSet,
             tenantIds: this.tenantIds,
@@ -278,7 +262,6 @@ class ShardSplitOperation {
             recipientSetName: this.recipientSetName,
             migrationId: this.migrationId,
             retryOnRetryableErrors,
-            enableDonorStartMigrationFsync
         });
     }
 
@@ -294,10 +277,7 @@ class ShardSplitOperation {
         const donorSet = createRst(donorRstArgs, true);
 
         const cmdObj = {forgetShardSplit: 1, migrationId: this.migrationId};
-        assert.commandWorked(runShardSplitCommand(donorSet,
-                                                  cmdObj,
-                                                  true /* retryableOnErrors */,
-                                                  false /*enableDonorStartMigrationFsync*/));
+        assert.commandWorked(runShardSplitCommand(donorSet, cmdObj, true /* retryableOnErrors */));
     }
 
     forgetAsync() {
@@ -355,7 +335,6 @@ export class ShardSplitTest {
         donorRst,
         nodeOptions,
         allowStaleReadsOnDonor = false,
-        initiateWithShortElectionTimeout = false
     } = {}) {
         nodeOptions = nodeOptions || {};
         if (quickGarbageCollection) {
@@ -372,11 +351,7 @@ export class ShardSplitTest {
         } else {
             this.donor = new ReplSetTest({name: "donor", nodes: 3, serverless: true, nodeOptions});
             this.donor.startSet();
-            if (initiateWithShortElectionTimeout) {
-                this.initiateWithShortElectionTimeout();
-            } else {
-                this.donor.initiate();
-            }
+            this.donor.initiate();
         }
 
         this.recipientTagName = recipientTagName;
@@ -384,13 +359,6 @@ export class ShardSplitTest {
         this.recipientNodes = [];
     }
 
-    initiateWithShortElectionTimeout() {
-        let config = this.donor.getReplSetConfig();
-        config.settings = config.settings || {};
-        config.settings["electionTimeoutMillis"] = 500;
-        this.donor.initiate(config);
-    }
-
     /*
      * Removes and stops the recipient nodes and then stops the donor nodes.
      * @param {shouldRestart} indicates whether stop() is being called with the intent to call
