@@ -3,9 +3,6 @@
  * @tags: [requires_fcv_63, serverless]
  */
 
-(function() {
-"use strict";
-
 const conn = MongoRunner.runMongod();
 const migrationId = UUID();
 const tenantIds = [ObjectId(), ObjectId()];
@@ -21,5 +18,4 @@ assert.commandFailedWithCode(conn.adminCommand({forgetShardSplit: 1, migrationId
                              ErrorCodes.CommandNotSupported);
 assert(!checkLog.checkContainsOnce(conn, "ShardSplitDonorService"),
        "Expected no mention of ShardSplitDonorService in logs");
-MongoRunner.stopMongod(conn);
-})();
+MongoRunner.stopMongod(conn);
\ No newline at end of file
