@@ -2,9 +2,6 @@
  * Basic integration tests for addShardToZone command. More detailed tests can be found
  * in sharding_catalog_add_shard_to_zone_test.cpp.
  */
-(function() {
-'use strict';
-
 let st = new ShardingTest({shards: 1});
 let mongos = st.s0;
 
@@ -36,5 +33,4 @@ assert.commandWorked(mongos.adminCommand({removeShardFromZone: shardName, zone:
 shardDoc = config.shards.findOne();
 assert.eq([], shardDoc.tags);
 
-st.stop();
-})();
+st.stop();
\ No newline at end of file
