@@ -1,8 +1,10 @@
 /**
  * Test that adding a config server replica set as a shard fails.
+ * @tags: [
+ *   # Test doesn't start enough mongods to have num_mongos routers
+ *   temp_disabled_embedded_router_num_routers,
+ * ]
  */
-(function() {
-
 var addShardRes;
 
 // Note: this method expects that the failure is *not* that the specified shardName is already
@@ -45,4 +47,3 @@ assertAddShardFailed(addShardRes, "nonConfig");
 configRS.stopSet();
 
 st.stop();
-})();
