@@ -1,10 +1,10 @@
 /**
  * Tests that the mongo shell can use a cluster time with a valid signature to advance a server's
  * cluster time.
+ * @tags: [
+ *   temp_disabled_embedded_router_mongo_bridge,
+ * ]
  */
-(function() {
-"use strict";
-
 // Setup 2 mongos processes with mongobridge.
 let st = new ShardingTest({shards: 1, mongos: 2, useBridge: true});
 
@@ -40,5 +40,4 @@ assert.eq(lt,
           "expected the disconnected mongos to send cluster time: " + tojson(lt) +
               ", received: " + tojson(res.$clusterTime));
 
-st.stop();
-})();
+st.stop();
\ No newline at end of file
