@@ -3,13 +3,10 @@
  *
  * @tags: [requires_fcv_70, featureFlagColumnstoreIndexes]
  */
-(function() {
-"use strict";
-
-load("jstests/libs/parallel_shell_helpers.js");
-load("jstests/libs/fail_point_util.js");
-load("jstests/sharding/analyze_shard_key/libs/analyze_shard_key_util.js");
-load("jstests/sharding/analyze_shard_key/libs/validation_common.js");
+import {configureFailPoint} from "jstests/libs/fail_point_util.js";
+import {funWithArgs} from "jstests/libs/parallel_shell_helpers.js";
+import {Thread} from "jstests/libs/parallelTester.js";
+import {ValidationTest} from "jstests/sharding/analyze_shard_key/libs/validation_common.js";
 
 const analyzeShardKeyNumRanges = 10;
 
@@ -183,7 +180,7 @@ const setParameterOpts = {analyzeShardKeyNumRanges};
     st.stop();
 }
 
-{
+if (!jsTestOptions().useAutoBootstrapProcedure) {  // TODO: SERVER-80318 Remove block
     const rst = new ReplSetTest({nodes: 2, nodeOptions: {setParameter: setParameterOpts}});
     rst.startSet();
     rst.initiate();
@@ -197,4 +194,3 @@ const setParameterOpts = {analyzeShardKeyNumRanges};
 
     rst.stopSet();
 }
-})();
