@@ -3,10 +3,7 @@
  *
  * @tags: [requires_fcv_70]
  */
-(function() {
-"use strict";
-
-load("jstests/sharding/analyze_shard_key/libs/validation_common.js");
+import {ValidationTest} from "jstests/sharding/analyze_shard_key/libs/validation_common.js";
 
 function runTest(conn) {
     const validationTest = ValidationTest(conn);
@@ -31,12 +28,11 @@ function runTest(conn) {
     st.stop();
 }
 
-{
+if (!jsTestOptions().useAutoBootstrapProcedure) {  // TODO: SERVER-80318 Remove block
     const rst = new ReplSetTest({nodes: 1});
     rst.startSet();
     rst.initiate();
     const primary = rst.getPrimary();
     runTest(primary);
     rst.stopSet();
-}
-})();
+}
\ No newline at end of file
