@@ -3,13 +3,8 @@
  *
  * @tags: [
  *   multiversion_incompatible,
- *   requires_find_command,
  * ]
  */
+import {MongosAPIParametersUtil} from "jstests/sharding/libs/mongos_api_params_util.js";
 
-(function() {
-'use strict';
-
-load('jstests/sharding/libs/mongos_api_params_util.js');
 MongosAPIParametersUtil.runTests({inTransaction: false, shardedCollection: false});
-})();
