@@ -5,10 +5,6 @@
  *   multiversion_incompatible,
  * ]
  */
+import {MongosAPIParametersUtil} from "jstests/sharding/libs/mongos_api_params_util.js";
 
-(function() {
-'use strict';
-
-load('jstests/sharding/libs/mongos_api_params_util.js');
 MongosAPIParametersUtil.runTests({inTransaction: true, shardedCollection: false});
-})();
