@@ -5,9 +5,6 @@
  *   uses_api_parameters,
  * ]
  */
-(function() {
-"use strict";
-
 const dbName = jsTestName();
 const collName = "test";
 const collForeignName = collName + "_foreign";
@@ -24,8 +21,7 @@ const namespace = dbName + "." + collName;
 const namespaceForeign = dbName + "." + collForeignName;
 
 /* Shard the collection. */
-assert.commandWorked(admin.runCommand({enableSharding: dbName}));
-assert.commandWorked(admin.runCommand({movePrimary: dbName, to: shards[0]._id}));
+assert.commandWorked(admin.runCommand({enableSharding: dbName, primaryShard: shards[0]._id}));
 assert.commandWorked(admin.runCommand({shardCollection: namespace, key: {a: 1}}));
 
 const coll = mongos.getCollection(namespace);
@@ -121,4 +117,3 @@ assert.commandWorked(db.runCommand(
 assert.commandWorked(db.runCommand({find: "view", apiStrict: true, apiVersion: "1"}));
 
 st.stop();
-})();
