@@ -5,8 +5,6 @@
  * @tags: [config_shard_incompatible]
  */
 
-(function() {
-"use strict";
 let st = new ShardingTest({
     shards: {
         rs0: {
@@ -41,5 +39,4 @@ for (let i = 0; i < secondaries.length; i++) {
     }
 }
 assert.eq(foundArbiter, true);
-st.stop();
-})();
+st.stop();
\ No newline at end of file
