@@ -4,13 +4,14 @@
  *
  * This test is labeled resource intensive because its total io_write is 30MB compared to a median
  * of 5MB across all sharding tests in wiredTiger.
- * @tags: [resource_intensive]
+ * @tags: [
+ *   resource_intensive,
+ *   # Test doesn't start enough mongods to have num_mongos routers
+ *   temp_disabled_embedded_router_num_routers,
+ * ]
  */
-(function() {
-'use strict';
-load("jstests/replsets/rslib.js");
-load("jstests/sharding/libs/find_chunks_util.js");
-load("jstests/libs/feature_flag_util.js");
+import {traceMissingDoc} from "jstests/libs/trace_missing_docs.js";
+import {awaitRSClientHosts} from "jstests/replsets/rslib.js";
 
 // Replica set nodes started with --shardsvr do not enable key generation until they are added
 // to a sharded cluster and reject commands with gossiped clusterTime from users without the
@@ -58,7 +59,7 @@ var s = new ShardingTest({
 
 if (s.getDB('admin').runCommand('buildInfo').bits < 64) {
     print('Skipping test on 32-bit platforms');
-    return;
+    quit();
 }
 
 print("Configuration: Add user " + tojson(adminUser));
@@ -108,9 +109,19 @@ assert(thrown);
 
 print("start rs w/correct key");
 
-d1.stopSet();
-d1.startSet({keyFile: "jstests/libs/key1", restart: true});
-d1.initiate();
+d1.stopSet(null /* signal */, true /* forRestart */);
+// If we are using the in-memory storage engine, we need to re-initiate the replica set
+// after restart before an election can occur, since the config does not persist. So
+// we must disable the auto stepup-on-restart behavior.
+if (jsTest.options().storageEngine == "inMemory") {
+    d1.startSet({keyFile: "jstests/libs/key1"},
+                true /* restart */,
+                false /* isMixedVersionCluster */,
+                true /* skipStepUpOnRestart */);
+    d1.initiate();
+} else {
+    d1.startSet({keyFile: "jstests/libs/key1", restart: true});
+}
 
 var primary = d1.getPrimary();
 
@@ -210,8 +221,6 @@ if (numDocs != num) {
     assert.eq(numDocs, numDocsSeen, "More docs discovered on second find()");
     assert.eq(num - numDocs, missingDocNumbers.length);
 
-    load('jstests/libs/trace_missing_docs.js');
-
     for (var i = 0; i < missingDocNumbers.length; i++) {
         jsTest.log("Tracing doc: " + missingDocNumbers[i]);
         traceMissingDoc(s.getDB("test").foo, {_id: missingDocNumbers[i], x: missingDocNumbers[i]});
@@ -326,4 +335,3 @@ assert.commandFailed(readOnlyDB.killOp(2000000000));
 s.stop();
 d1.stopSet();
 d2.stopSet();
-})();
