@@ -8,10 +8,7 @@
  *  ]
  * */
 
-(function() {
-"use strict";
-
-load("jstests/libs/parallel_shell_helpers.js");
+import {funWithArgs} from "jstests/libs/parallel_shell_helpers.js";
 
 const numShards = 2;
 Random.setRandomSeed();
@@ -113,4 +110,3 @@ for (let i = 0; i < numDatabases; i++) {
 }
 
 st.stop();
-}());
