@@ -5,9 +5,9 @@
  *  - Balancing is disabled for a collection, which has chunks on draining shards.
  */
 
-load("jstests/libs/fail_point_util.js");
-load("jstests/libs/parallel_shell_helpers.js");
-load('jstests/sharding/libs/find_chunks_util.js');
+import {configureFailPoint} from "jstests/libs/fail_point_util.js";
+import {funWithArgs} from "jstests/libs/parallel_shell_helpers.js";
+import {findChunksUtil} from "jstests/sharding/libs/find_chunks_util.js";
 
 const st = new ShardingTest({
     shards: 2,
@@ -44,10 +44,11 @@ assert.eq(2, chunksBeforeDrain.length);
 const csrsPrimary = st.configRS.getPrimary();
 configureFailPoint(csrsPrimary, "forceBalancerWarningChecks");
 
-let awaitRemoveShard = startParallelShell(funWithArgs(function(shardName) {
-                                              load("jstests/sharding/libs/remove_shard_util.js");
-                                              removeShard(db, shardName);
-                                          }, st.shard1.shardName), st.s.port);
+let awaitRemoveShard = startParallelShell(
+    funWithArgs(async function(shardName) {
+        const {removeShard} = await import("jstests/sharding/libs/remove_shard_util.js");
+        removeShard(db, shardName);
+    }, st.shard1.shardName), st.s.port);
 
 // Test warning when the balancer is disabled.
 // "Draining of removed shards cannot be completed because the balancer is disabled"
