@@ -1,10 +1,12 @@
 /**
  * Test of complex sharding initialization
+ * @tags: [
+ *   # This test is testing CMD parameters passed specifically to mongoS, which expects to be
+ *   # different from other nodes. That's not possible with an embedded router.
+ *   embedded_router_incompatible,
+ * ]
  */
 
-(function() {
-'use strict';
-
 function shardingTestUsingObjects() {
     var st = new ShardingTest({
         mongos: {s0: {verbose: 6}, s1: {verbose: 5}},
@@ -45,7 +47,12 @@ function shardingTestUsingObjects() {
 
     assert(s0.commandLine.hasOwnProperty("vvvvvv"));
     assert(s1.commandLine.hasOwnProperty("vvvvv"));
-    assert(c0.commandLine.hasOwnProperty("vvvv"));
+    if (!TestData.configShard) {
+        assert(c0.commandLine.hasOwnProperty("vvvv"));
+    } else {
+        // Same as shard 1.
+        assert(c0.commandLine.hasOwnProperty("vvv"));
+    }
     assert(rs0_d0.commandLine.hasOwnProperty("vvv"));
     assert(rs1_d0.commandLine.hasOwnProperty("vv"));
     assert(rs1_a1.commandLine.hasOwnProperty("v"));
@@ -81,7 +88,12 @@ function shardingTestUsingArrays() {
 
     assert(s0.commandLine.hasOwnProperty("vvvvv"));
     assert(s1.commandLine.hasOwnProperty("vvvv"));
-    assert(c0.commandLine.hasOwnProperty("vvv"));
+    if (!TestData.configShard) {
+        assert(c0.commandLine.hasOwnProperty("vvv"));
+    } else {
+        // Same as shard 1.
+        assert(c0.commandLine.hasOwnProperty("vv"));
+    }
     assert(rs0_d0.commandLine.hasOwnProperty("vv"));
     assert(rs1_d0.commandLine.hasOwnProperty("v"));
 
@@ -90,4 +102,3 @@ function shardingTestUsingArrays() {
 
 shardingTestUsingObjects();
 shardingTestUsingArrays();
-})();
