@@ -1,10 +1,7 @@
 /**
  * Perform basic tests for the split command against mongos.
  */
-(function() {
-'use strict';
-
-load("jstests/sharding/libs/find_chunks_util.js");
+import {findChunksUtil} from "jstests/sharding/libs/find_chunks_util.js";
 
 var st = new ShardingTest({mongos: 2, shards: 2, other: {chunkSize: 1}});
 var configDB = st.s0.getDB('config');
@@ -18,8 +15,7 @@ assert.commandFailed(configDB.adminCommand({split: 'user', key: {_id: 1}}));
 // split on unsharded collection (db is not sharding enabled).
 assert.commandFailed(configDB.adminCommand({split: 'test.user', key: {_id: 1}}));
 
-assert.commandWorked(configDB.adminCommand({enableSharding: 'test'}));
-st.ensurePrimaryShard('test', shard0);
+assert.commandWorked(configDB.adminCommand({enableSharding: 'test', primaryShard: shard0}));
 
 // split on unsharded collection (db is sharding enabled).
 assert.commandFailed(configDB.adminCommand({split: 'test.user', key: {_id: 1}}));
@@ -118,5 +114,4 @@ assert.gt(
     findChunksUtil.findChunksByNs(configDB, 'test.compound', {min: {$lt: {x: 0, y: 0}}}).itcount(),
     1);
 
-st.stop();
-})();
+st.stop();
\ No newline at end of file
