@@ -2,14 +2,10 @@
  * Tests that chunk migrations are blocked when there is no index on a hashed shard key.
  *
  * @tags: [
- *   requires_fcv_50,
+ *   requires_fcv_70,
  * ]
  */
-
-(function() {
-"use strict";
-
-load("jstests/sharding/libs/find_chunks_util.js");
+import {findChunksUtil} from "jstests/sharding/libs/find_chunks_util.js";
 
 const st = new ShardingTest({});
 
@@ -19,8 +15,7 @@ const nss = dbName + "." + collName;
 const configDB = st.s.getDB('config');
 
 const coll = st.getDB(dbName).getCollection(collName);
-const kDataString = "a".repeat(1024 * 1024);
-let docs = Array.from({length: 1000}, (_, i) => ({_id: i, field: kDataString}));
+let docs = Array.from({length: 1000}, (x, i) => ({_id: i, field: "a".repeat(1024 * 1024)}));
 
 assert.commandWorked(
     st.s.adminCommand({enablesharding: dbName, primaryShard: st.shard0.shardName}));
@@ -44,8 +39,11 @@ st.awaitBalancerRound();
 // During balancing, the balancer should catch the IndexNotFound and turn off the balancer for the
 // collection by setting {noBalance : true}.
 assert.soon(() => {
-    return configDB.getCollection('collections').findOne({_id: nss}).noBalance === true;
+    return configDB.getCollection('collections')
+               .findOne({_id: nss}, {}, {readConcern: "majority"})
+               .noBalance === true;
 });
+st.awaitBalancerRound();
 
 // Confirm all chunks remain on shard0.
 assert.eq(0, findChunksUtil.findChunksByNs(configDB, nss, {shard: st.shard1.shardName}).itcount());
@@ -55,18 +53,10 @@ assert.commandFailedWithCode(
     st.s.adminCommand(
         {moveChunk: nss, bounds: [chunks[0].min, chunks[0].max], to: st.shard1.shardName}),
     ErrorCodes.IndexNotFound);
-
-// moveRange is not present in 5.0 so skip this assertion if FCV is lesser than 6.0 in multi-version
-// test suite.
-if (st.rs0.getPrimary().getDB('admin').system.version.findOne(
-        {_id: 'featureCompatibilityVersion'}) == getFCVConstants().latest &&
-    st.rs1.getPrimary().getDB('admin').system.version.findOne(
-        {_id: 'featureCompatibilityVersion'}) == getFCVConstants().latest) {
-    assert.commandFailedWithCode(
-        st.s.adminCommand(
-            {moveRange: nss, toShard: st.shard1.shardName, min: chunks[0].min, max: chunks[0].max}),
-        ErrorCodes.IndexNotFound);
-}
+assert.commandFailedWithCode(
+    st.s.adminCommand(
+        {moveRange: nss, toShard: st.shard1.shardName, min: chunks[0].min, max: chunks[0].max}),
+    ErrorCodes.IndexNotFound);
 
 // Recreate the index and verify that we can re-enable balancing.
 assert.commandWorked(coll.createIndex({"_id": "hashed"}));
@@ -78,5 +68,6 @@ assert.soon(() => {
     return findChunksUtil.findChunksByNs(configDB, nss, {shard: st.shard1.shardName}).itcount() > 0;
 });
 
+st.awaitMigrations();
+
 st.stop();
-})();
