@@ -4,10 +4,7 @@
 //   requires_majority_read_concern,
 //   uses_change_streams,
 // ]
-(function() {
-'use strict';
-
-load('jstests/libs/profiler.js');  // For various profiler helpers.
+import {profilerHasSingleMatchingEntryOrThrow} from "jstests/libs/profiler.js";
 
 const st = new ShardingTest({
     shards: 3,
@@ -26,8 +23,7 @@ const shard1 = st.shard1;
 const shard2 = st.shard2;
 
 // Enable sharding to inform mongos of the database, and make sure all chunks start on shard 0.
-assert.commandWorked(mongos.adminCommand({enableSharding: dbName}));
-st.ensurePrimaryShard(dbName, shard0.shardName);
+assert.commandWorked(mongos.adminCommand({enableSharding: dbName, primaryShard: shard0.shardName}));
 
 // Create a change stream before any insert or update operations are performed.
 const csBeforeSharding = mongosColl.watch([], {fullDocument: "updateLookup"});
@@ -157,4 +153,3 @@ csAfterAddShard.close();
 
 st.stop();
 rs3.stopSet();
-})();
