@@ -1,17 +1,14 @@
 // Tests resuming change streams when shard key is changed.
 //
 // @tags: [
+//   requires_fcv_53,
 //   requires_majority_read_concern,
 //   uses_change_streams,
-//   requires_fcv_53,
 // ]
 
 // Cannot run the filtering metadata check on tests that run refineCollectionShardKey.
 TestData.skipCheckShardFilteringMetadata = true;
 
-(function() {
-"use strict";
-
 const dbName = 'testDB';
 const collName = 'testColl';
 
@@ -87,4 +84,3 @@ changes.forEach((change, i) => {
 });
 
 st.stop();
-})();
