@@ -11,10 +11,8 @@
 // stepdowns.
 TestData.skipCheckingUUIDsConsistentAcrossCluster = true;
 
-(function() {
-"use strict";
-load("jstests/libs/change_stream_util.js");        // For ChangeStreamTest.
-load("jstests/libs/collection_drop_recreate.js");  // For assert[Drop|Create]Collection.
+import {ChangeStreamWatchMode, ChangeStreamTest} from "jstests/libs/change_stream_util.js";
+import {assertDropAndRecreateCollection} from "jstests/libs/collection_drop_recreate.js";
 
 const st = new ShardingTest({
     shards: 2,
@@ -105,4 +103,3 @@ for (let key of Object.keys(ChangeStreamWatchMode)) {
 }
 
 st.stop();
-}());
