@@ -4,9 +4,6 @@
 //   requires_majority_read_concern,
 //   uses_change_streams,
 // ]
-(function() {
-"use strict";
-
 const st = new ShardingTest({
     shards: 2,
     rs: {
@@ -23,8 +20,8 @@ const mongosColl = mongosDB['coll'];
 assert.commandWorked(mongosDB.dropDatabase());
 
 // Enable sharding on the test DB and ensure its primary is st.shard0.shardName.
-assert.commandWorked(mongosDB.adminCommand({enableSharding: mongosDB.getName()}));
-st.ensurePrimaryShard(mongosDB.getName(), st.rs0.getURL());
+assert.commandWorked(
+    mongosDB.adminCommand({enableSharding: mongosDB.getName(), primaryShard: st.rs0.getURL()}));
 
 // Shard the test collection on _id.
 assert.commandWorked(
@@ -84,5 +81,4 @@ for (let nextId of [1000, -1000]) {
     assert.docEq({_id: nextId, updatedCount: 2}, next.fullDocument);
 }
 
-st.stop();
-})();
+st.stop();
\ No newline at end of file
