@@ -8,12 +8,11 @@
 //   denylist_from_rhel_67_s390x,
 //   requires_majority_read_concern,
 //   requires_persistence,
+//   # Test doesn't start enough mongods to have num_mongos routers
+//   temp_disabled_embedded_router_num_routers,
 //   uses_change_streams,
 // ]
-(function() {
-"use strict";
-
-load('jstests/libs/change_stream_util.js');  // For ChangeStreamTest.
+import {ChangeStreamTest} from "jstests/libs/change_stream_util.js";
 
 // Returns true if the shard is aware that the collection is sharded.
 function isShardAware(shard, coll) {
@@ -47,8 +46,8 @@ const st = new ShardingTest({
 const mongosDB = st.s0.getDB(testName);
 
 // Ensure that shard0 is the primary shard.
-assert.commandWorked(mongosDB.adminCommand({enableSharding: mongosDB.getName()}));
-st.ensurePrimaryShard(mongosDB.getName(), st.rs0.getURL());
+assert.commandWorked(
+    mongosDB.adminCommand({enableSharding: mongosDB.getName(), primaryShard: st.rs0.getURL()}));
 
 // Create unsharded collection on primary shard.
 const mongosColl = mongosDB[testName];
@@ -196,4 +195,3 @@ cstMongos2.assertNextChangesEqual({
 });
 
 st.stop();
-})();
