@@ -5,9 +5,6 @@
  *   uses_change_streams,
  * ]
  */
-(function() {
-"use strict";
-
 // Start a new sharded cluster with 2 nodes and obtain references to the test DB and collection.
 const st = new ShardingTest({
     shards: 2,
@@ -62,4 +59,3 @@ assert.soon(() => {
 });
 
 st.stop();
-})();
\ No newline at end of file
