@@ -2,9 +2,6 @@
 // Tests of cleanupOrphaned command permissions.
 //
 
-(function() {
-'use strict';
-
 // Multiple users cannot be authenticated on one connection within a session.
 TestData.disableImplicitSessions = true;
 
@@ -66,5 +63,4 @@ shardAdmin.logout();
 fooDB.auth('user', 'x');
 assertUnauthorized(shardAdmin.runCommand({cleanupOrphaned: 'foo.bar'}));
 
-st.stop();
-})();
+st.stop();
\ No newline at end of file
