@@ -1,9 +1,6 @@
 /**
  * Tests the collectionUUID parameter of the shardCollection command against capped collections.
  */
-(function() {
-'use strict';
-
 const st = new ShardingTest({shards: 1});
 const mongos = st.s0;
 
@@ -37,5 +34,4 @@ assert.commandFailedWithCode(
     mongos.adminCommand({shardCollection: coll.getFullName(), collectionUUID: uuid, key: {_id: 1}}),
     ErrorCodes.InvalidOptions);
 
-st.stop();
-})();
+st.stop();
\ No newline at end of file
