@@ -11,10 +11,12 @@
  *   resource_intensive,
  * ]
  */
-load('jstests/libs/write_concern_util.js');
+import {
+    assertWriteConcernError,
+    runCommandCheckAdmin,
+    shardCollectionWithChunks
+} from "jstests/libs/write_concern_util.js";
 
-(function() {
-"use strict";
 var st = new ShardingTest({
     // Set priority of secondaries to zero to prevent spurious elections.
     shards: {
@@ -41,7 +43,6 @@ function dropTestDatabase() {
     db.runCommand({dropDatabase: 1});
     db.extra.insert({a: 1});
     coll = db[collName];
-    st.ensurePrimaryShard(db.toString(), st.shard0.shardName);
     assert.eq(0, coll.find().itcount(), "test collection not empty");
     assert.eq(1, db.extra.find().itcount(), "extra collection should have 1 document");
 }
@@ -69,7 +70,6 @@ commands.push({
     req: {createIndexes: collName, indexes: [{key: {'type': 1}, name: 'type_index'}]},
     setupFunc: function() {
         coll.insert({type: 'oak'});
-        st.ensurePrimaryShard(db.toString(), st.shard0.shardName);
         assert.eq(coll.getIndexes().length, 1);
     },
     confirmFunc: function() {
@@ -98,7 +98,6 @@ commands.push({
         db = db.getSiblingDB("renameCollWC");
         // Ensure that database is created.
         db.leaves.insert({type: 'oak'});
-        st.ensurePrimaryShard(db.toString(), st.shard0.shardName);
         db.leaves.drop();
         db.pine_needles.drop();
         db.leaves.insert({type: 'oak'});
@@ -519,13 +518,12 @@ function testInvalidWriteConcern(cmd) {
     // sub-object detailing the failure. This is motivated by the fact that the number of documents
     // written is bounded only by the number of documents produced by the pipeline, which could lead
     // a writeConcernError object to exceed maximum BSON size.
-    if (cmd.req.aggregate !== undefined || cmd.req.mapReduce !== undefined) {
+    if (cmd.req.aggregate || cmd.req.mapReduce) {
         assert.commandFailedWithCode(
             res, [ErrorCodes.WriteConcernFailed, ErrorCodes.UnknownReplWriteConcern]);
 
         cmd.confirmFunc(cmd.isExpectedToWriteOnWriteConcernFailure);
-    } else if (cmd.req.renameCollection !== undefined &&
-               jsTestOptions().mongosBinVersion != 'last-lts') {
+    } else if (cmd.req.renameCollection) {
         // The renameCollection spans multiple nodes and potentially performs writes to the config
         // server, so the user-specified write concern has no effect.
         assert.commandWorked(res);
@@ -547,4 +545,3 @@ commands.forEach(function(cmd) {
 });
 
 st.stop();
-})();
