@@ -2,18 +2,14 @@
  * Test to verify the covering behaviour of compound hashed index on a cluster sharded with compound
  * hashed shard key.
  */
-(function() {
-"use strict";
-
-load("jstests/libs/analyze_plan.js");  // For assertStagesForExplainOfCommand().
+import {assertStagesForExplainOfCommand} from "jstests/libs/analyze_plan.js";
 
 const st = new ShardingTest({shards: 2});
 const kDbName = jsTestName();
+assert.commandWorked(
+    st.s.adminCommand({enableSharding: kDbName, primaryShard: st.shard0.shardName}));
 const coll = st.s.getDB(kDbName)["coll"];
 
-assert.commandWorked(st.s.adminCommand({enableSharding: kDbName}));
-st.ensurePrimaryShard(kDbName, st.shard0.shardName);
-
 /**
  * Runs find command with the 'filter', 'projection' and 'hint' parameters. Then validates that the
  * output returned matches 'expectedOutput'. Also runs explain() command on the same find command
@@ -199,5 +195,4 @@ validateFindCmdOutputAndPlan({
     stagesNotExpected: ["FETCH"]
 });
 
-st.stop();
-})();
\ No newline at end of file
+st.stop();
\ No newline at end of file
