@@ -4,10 +4,7 @@
  * ]
  */
 
-(function() {
-'use strict';
-
-load("jstests/libs/retryable_writes_util.js");
+import {RetryableWritesUtil} from "jstests/libs/retryable_writes_util.js";
 
 function runConfigsvrRemoveTagsWithRetries(conn, ns, lsid, txnNumber) {
     var res;
@@ -83,5 +80,4 @@ assert.commandFailedWithCode(
 assert.eq(2, configDB.tags.countDocuments({ns: ns}));
 assert.eq(2, configDB.tags.countDocuments({ns: anotherNs}));
 
-st.stop();
-})();
+st.stop();
\ No newline at end of file
