@@ -1,12 +1,29 @@
 /**
  * Specifies for each command whether it is expected to send a databaseVersion, and verifies that
  * the commands match the specification.
+ *
+ * Each command must have exactly one corresponding test defined. Each defined test case must
+ * correspond to an existing command. The allowable fields for the test cases are as follows:
+ *
+ *      - 'run': This is the specified test case that will be executed for each command.
+ *      - 'skip': Use this field to skip the execution of the test case, along with a justification.
+ *      It's important to note that this field doesn't bypass command validation; it only skips the
+ *      actual run.
+ *      - 'explain': This field is optional and is used to test the explain command on the specified
+ *      test case.
+ *      - 'conditional': If you set this field to true, the test case will skip the validation that
+ *      ensures all test cases match existing commands. This is useful for commands that only exist
+ *      in enterprise modules, for instance.
+ *      - 'skipMultiversion': If you set this field to true, the test case will skip running in
+ *      multiversion suites. This is useful if you have a command that existed behind a feature flag
+ *      in the previous version and is now enabled.
  */
-(function() {
-'use strict';
 
-load("jstests/libs/feature_flag_util.js");
-load('jstests/sharding/libs/last_lts_mongos_commands.js');
+import {FeatureFlagUtil} from "jstests/libs/feature_flag_util.js";
+import {
+    commandsAddedToMongosSinceLastLTS,
+    commandsRemovedFromMongosSinceLastLTS
+} from "jstests/sharding/libs/last_lts_mongos_commands.js";
 
 function getNewDbName(dbName) {
     if (!getNewDbName.counter) {
@@ -16,14 +33,14 @@ function getNewDbName(dbName) {
     return "db" + getNewDbName.counter;
 }
 
-function assertMongosDatabaseVersion(conn, dbName, dbVersion) {
-    let res = conn.adminCommand({getShardVersion: dbName});
-    assert.commandWorked(res);
-    assert.eq(dbVersion, res.version);
-}
-
-function assertShardDatabaseVersion(shard, dbName, dbVersion) {
-    let res = shard.adminCommand({getDatabaseVersion: dbName});
+function assertMatchingDatabaseVersion(conn, dbName, dbVersion) {
+    let res = conn.adminCommand({getDatabaseVersion: dbName});
+    // TODO (SERVER-81967): Remove once 8.0 becomes last LTS.
+    if (!res.ok && res.code === ErrorCodes.CommandNotFound) {
+        res = assert.commandWorked(conn.adminCommand({getShardVersion: dbName}));
+        assert.eq(dbVersion, res.version);
+        return;
+    }
     assert.commandWorked(res);
     assert.eq(dbVersion, res.dbVersion);
 }
@@ -104,27 +121,22 @@ function testCommandAfterMovePrimary(testCase, st, dbName, collName) {
 
     // Ensure all nodes know the dbVersion before the movePrimary.
     assert.commandWorked(st.s0.adminCommand({flushRouterConfig: 1}));
-    assertMongosDatabaseVersion(st.s0, dbName, dbVersionBefore);
+    assertMatchingDatabaseVersion(st.s0, dbName, dbVersionBefore);
     assert.commandWorked(primaryShardBefore.adminCommand({_flushDatabaseCacheUpdates: dbName}));
-    assertShardDatabaseVersion(primaryShardBefore, dbName, dbVersionBefore);
+    assertMatchingDatabaseVersion(primaryShardBefore, dbName, dbVersionBefore);
     assert.commandWorked(primaryShardAfter.adminCommand({_flushDatabaseCacheUpdates: dbName}));
-    assertShardDatabaseVersion(primaryShardAfter, dbName, dbVersionBefore);
+    assertMatchingDatabaseVersion(primaryShardAfter, dbName, dbVersionBefore);
 
     // Run movePrimary through the second mongos.
     assert.commandWorked(st.s1.adminCommand({movePrimary: dbName, to: primaryShardAfter.name}));
+
     const dbVersionAfter =
         st.s1.getDB("config").getCollection("databases").findOne({_id: dbName}).version;
 
     // After the movePrimary, both old and new primary shards should have cleared the dbVersion.
-    assertMongosDatabaseVersion(st.s0, dbName, dbVersionBefore);
-    assertShardDatabaseVersion(primaryShardBefore, dbName, {});
-    // TODO (SERVER-71309): Remove once 7.0 becomes last LTS.
-    if (FeatureFlagUtil.isEnabled(st.configRS.getPrimary().getDB('admin'),
-                                  "ResilientMovePrimary")) {
-        assertShardDatabaseVersion(primaryShardAfter, dbName, {});
-    } else {
-        assertShardDatabaseVersion(primaryShardAfter, dbName, dbVersionBefore);
-    }
+    assertMatchingDatabaseVersion(st.s0, dbName, dbVersionBefore);
+    assertMatchingDatabaseVersion(primaryShardBefore, dbName, {});
+    assertMatchingDatabaseVersion(primaryShardAfter, dbName, {});
 
     // Run the test case's command.
     const res = st.s0.getDB(testCase.runsAgainstAdminDb ? "admin" : dbName).runCommand(command);
@@ -142,23 +154,17 @@ function testCommandAfterMovePrimary(testCase, st, dbName, collName) {
         // 3. Which should have caused the mongos to refresh and retry against the new primary shard
         // 4. The new primary shard should have returned StaleDbVersion and refreshed
         // 5. Which should have caused the mongos to refresh and retry again, this time succeeding.
-        assertMongosDatabaseVersion(st.s0, dbName, dbVersionAfter);
-        assertShardDatabaseVersion(primaryShardBefore, dbName, dbVersionAfter);
-        assertShardDatabaseVersion(primaryShardAfter, dbName, dbVersionAfter);
+        assertMatchingDatabaseVersion(st.s0, dbName, dbVersionAfter);
+        assertMatchingDatabaseVersion(primaryShardBefore, dbName, dbVersionAfter);
+        assertMatchingDatabaseVersion(primaryShardAfter, dbName, dbVersionAfter);
     } else {
         // If the command does not participate in database versioning:
         // 1. The mongos should have targeted the old primary shard but not attached a dbVersion
         // 2. The old primary shard should have returned an ok response
         // 3. Both old and new primary shards should have cleared the dbVersion
-        assertMongosDatabaseVersion(st.s0, dbName, dbVersionBefore);
-        assertShardDatabaseVersion(primaryShardBefore, dbName, {});
-        // TODO (SERVER-71309): Remove once 7.0 becomes last LTS.
-        if (FeatureFlagUtil.isEnabled(st.configRS.getPrimary().getDB('admin'),
-                                      "ResilientMovePrimary")) {
-            assertShardDatabaseVersion(primaryShardAfter, dbName, {});
-        } else {
-            assertShardDatabaseVersion(primaryShardAfter, dbName, dbVersionBefore);
-        }
+        assertMatchingDatabaseVersion(st.s0, dbName, dbVersionBefore);
+        assertMatchingDatabaseVersion(primaryShardBefore, dbName, {});
+        assertMatchingDatabaseVersion(primaryShardAfter, dbName, {});
     }
 
     if (testCase.cleanUp) {
@@ -186,9 +192,9 @@ function testCommandAfterDropRecreateDatabase(testCase, st) {
                ", primary shard after: " + primaryShardAfter);
 
     // Ensure the router and primary shard know the dbVersion before the drop/recreate database.
-    assertMongosDatabaseVersion(st.s0, dbName, dbVersionBefore);
-    assertShardDatabaseVersion(primaryShardBefore, dbName, dbVersionBefore);
-    assertShardDatabaseVersion(primaryShardAfter, dbName, {});
+    assertMatchingDatabaseVersion(st.s0, dbName, dbVersionBefore);
+    assertMatchingDatabaseVersion(primaryShardBefore, dbName, dbVersionBefore);
+    assertMatchingDatabaseVersion(primaryShardAfter, dbName, {});
 
     // Drop and recreate the database through the second mongos. Insert the entry for the new
     // database explicitly to ensure it is assigned the other shard as the primary shard.
@@ -218,9 +224,9 @@ function testCommandAfterDropRecreateDatabase(testCase, st) {
 
     // The only change after the drop/recreate database should be that the old primary shard should
     // have cleared its dbVersion.
-    assertMongosDatabaseVersion(st.s0, dbName, dbVersionBefore);
-    assertShardDatabaseVersion(primaryShardBefore, dbName, {});
-    assertShardDatabaseVersion(primaryShardAfter, dbName, {});
+    assertMatchingDatabaseVersion(st.s0, dbName, dbVersionBefore);
+    assertMatchingDatabaseVersion(primaryShardBefore, dbName, {});
+    assertMatchingDatabaseVersion(primaryShardAfter, dbName, {});
 
     // Run the test case's command.
     const res = st.s0.getDB(testCase.runsAgainstAdminDb ? "admin" : dbName).runCommand(command);
@@ -238,17 +244,17 @@ function testCommandAfterDropRecreateDatabase(testCase, st) {
         // 3. Which should have caused the mongos to refresh and retry against the new primary shard
         // 4. The new primary shard should have returned StaleDbVersion and refreshed
         // 5. Which should have caused the mongos to refresh and retry again, this time succeeding.
-        assertMongosDatabaseVersion(st.s0, dbName, dbVersionAfter);
-        assertShardDatabaseVersion(primaryShardBefore, dbName, dbVersionAfter);
-        assertShardDatabaseVersion(primaryShardAfter, dbName, dbVersionAfter);
+        assertMatchingDatabaseVersion(st.s0, dbName, dbVersionAfter);
+        assertMatchingDatabaseVersion(primaryShardBefore, dbName, dbVersionAfter);
+        assertMatchingDatabaseVersion(primaryShardAfter, dbName, dbVersionAfter);
     } else {
         // If the command does not participate in database versioning, none of the nodes' view of
         // the dbVersion should have changed:
         // 1. The mongos should have targeted the old primary shard but not attached a dbVersion
         // 2. The old primary shard should have returned an ok response
-        assertMongosDatabaseVersion(st.s0, dbName, dbVersionBefore);
-        assertShardDatabaseVersion(primaryShardBefore, dbName, {});
-        assertShardDatabaseVersion(primaryShardAfter, dbName, {});
+        assertMatchingDatabaseVersion(st.s0, dbName, dbVersionBefore);
+        assertMatchingDatabaseVersion(primaryShardBefore, dbName, {});
+        assertMatchingDatabaseVersion(primaryShardAfter, dbName, {});
     }
 
     // Clean up.
@@ -272,8 +278,10 @@ let testCases = {
     _mergeAuthzCollections: {skip: "always targets the config server"},
     _mongotConnPoolStats: {skip: "not on a user database", conditional: true},
     _dropConnectionsToMongot: {skip: "not on a user database", conditional: true},
+    abortMoveCollection: {skip: "always targets the config server"},
     abortReshardCollection: {skip: "always targets the config server"},
     abortTransaction: {skip: "unversioned and uses special targetting rules"},
+    abortUnshardCollection: {skip: "always targets the config server"},
     addShard: {skip: "not on a user database"},
     addShardToZone: {skip: "not on a user database"},
     aggregate: {
@@ -313,14 +321,38 @@ let testCases = {
     },
     appendOplogNote: {skip: "unversioned and executes on all shards"},
     authenticate: {skip: "does not forward command to primary shard"},
+    autoSplitVector: {skip: "does not forward command to primary shard"},
     balancerCollectionStatus: {skip: "does not forward command to primary shard"},
     balancerStart: {skip: "not on a user database"},
     balancerStatus: {skip: "not on a user database"},
     balancerStop: {skip: "not on a user database"},
     buildInfo: {skip: "executes locally on mongos (not sent to any remote node)"},
-    bulkWrite: {skip: "not yet implemented"},
-    checkMetadataConsistency: {skip: "not yet implemented"},
+    bulkWrite: {
+        run: {
+            sendsDbVersion: true,
+            runsAgainstAdminDb: true,
+            command: function(dbName, collName) {
+                return {
+                    bulkWrite: 1,
+                    ops: [{insert: 0, document: {_id: 1}}],
+                    nsInfo: [{ns: dbName + "." + collName}]
+                };
+            },
+        },
+        skipMultiversion: true
+    },
+    changePrimary: {skip: "reads primary shard from sharding catalog with readConcern: local"},
+    checkMetadataConsistency: {
+        run: {
+            sendsDbVersion: true,
+            runsAgainstAdminDb: false,
+            command: function(dbName, collName) {
+                return {checkMetadataConsistency: 1};
+            }
+        }
+    },
     cleanupReshardCollection: {skip: "always targets the config server"},
+    cleanupStructuredEncryptionData: {skip: "requires encrypted collections"},
     clearJumboFlag: {skip: "does not forward command to primary shard"},
     clearLog: {skip: "executes locally on mongos (not sent to any remote node)"},
     collMod: {
@@ -375,6 +407,7 @@ let testCases = {
             }
         }
     },
+    cpuload: {skip: "executes locally on mongos (not sent to any remote node)"},
     create: {
         run: {
             sendsDbVersion: true,
@@ -394,6 +427,8 @@ let testCases = {
     },
     createSearchIndexes: {skip: "executes locally on mongos", conditional: true},
     createRole: {skip: "always targets the config server"},
+    createUnsplittableCollection:
+        {skip: "Test command that which functionality will be integrated into createCollection"},
     createUser: {skip: "always targets the config server"},
     currentOp: {skip: "not on a user database"},
     dataSize: {
@@ -462,7 +497,7 @@ let testCases = {
     echo: {skip: "does not forward command to primary shard"},
     enableSharding: {skip: "does not forward command to primary shard"},
     endSessions: {skip: "goes through the cluster write path"},
-    explain: {skip: "TODO SERVER-31226"},
+    explain: {skip: "already tested by each CRUD command through the 'explain' field"},
     features: {skip: "executes locally on mongos (not sent to any remote node)"},
     filemd5: {
         run: {
@@ -507,6 +542,7 @@ let testCases = {
     getAuditConfig: {skip: "not on a user database", conditional: true},
     getClusterParameter: {skip: "always targets the config server"},
     getCmdLineOpts: {skip: "executes locally on mongos (not sent to any remote node)"},
+    getDatabaseVersion: {skip: "executes locally on mongos (not sent to any remote node)"},
     getDefaultRWConcern: {skip: "executes locally on mongos (not sent to any remote node)"},
     getDiagnosticData: {skip: "executes locally on mongos (not sent to any remote node)"},
     getLog: {skip: "executes locally on mongos (not sent to any remote node)"},
@@ -573,6 +609,7 @@ let testCases = {
     },
     listSearchIndexes: {skip: "executes locally on mongos", conditional: true},
     listShards: {skip: "does not forward command to primary shard"},
+    lockInfo: {skip: "not on a user database"},
     logApplicationMessage: {skip: "not on a user database", conditional: true},
     logMessage: {skip: "not on a user database"},
     logRotate: {skip: "executes locally on mongos (not sent to any remote node)"},
@@ -614,12 +651,15 @@ let testCases = {
     mergeAllChunksOnShard: {skip: "does not forward command to primary shard"},
     mergeChunks: {skip: "does not forward command to primary shard"},
     moveChunk: {skip: "does not forward command to primary shard"},
+    moveCollection: {skip: "does not forward command to primary shard"},
     movePrimary: {skip: "reads primary shard from sharding catalog with readConcern: local"},
     moveRange: {skip: "does not forward command to primary shard"},
     multicast: {skip: "does not forward command to primary shard"},
     netstat: {skip: "executes locally on mongos (not sent to any remote node)"},
-    oidcListKeys: {skip: "executes locally on mongos (not sent to any remote node)"},
-    oidcRefreshKeys: {skip: "executes locally on mongos (not sent to any remote node)"},
+    oidcListKeys:
+        {skip: "executes locally on mongos (not sent to any remote node)", conditional: true},
+    oidcRefreshKeys:
+        {skip: "executes locally on mongos (not sent to any remote node)", conditional: true},
     ping: {skip: "executes locally on mongos (not sent to any remote node)"},
     planCacheClear: {
         run: {
@@ -688,6 +728,7 @@ let testCases = {
     revokeRolesFromUser: {skip: "always targets the config server"},
     rolesInfo: {skip: "always targets the config server"},
     rotateCertificates: {skip: "executes locally on mongos (not sent to any remote node)"},
+    rotateFTDC: {skip: "executes locally on mongos (not sent to any remote node)"},
     saslContinue: {skip: "not on a user database"},
     saslStart: {skip: "not on a user database"},
     serverStatus: {skip: "executes locally on mongos (not sent to any remote node)"},
@@ -713,6 +754,8 @@ let testCases = {
     setProfilingFilterGlobally: {skip: "executes locally on mongos (not sent to any remote node)"},
     setParameter: {skip: "executes locally on mongos (not sent to any remote node)"},
     setClusterParameter: {skip: "always targets the config server"},
+    setQuerySettings: {skip: "not on a user database"},
+    removeQuerySettings: {skip: "not on a user database"},
     setUserWriteBlockMode: {skip: "executes locally on mongos (not sent to any remote node)"},
     shardCollection: {skip: "does not forward command to primary shard"},
     shutdown: {skip: "does not forward command to primary shard"},
@@ -729,6 +772,8 @@ let testCases = {
     testVersions1And2: {skip: "executes locally on mongos (not sent to any remote node)"},
     transitionFromDedicatedConfigServer: {skip: "not on a user database"},
     transitionToDedicatedConfigServer: {skip: "not on a user database"},
+    unshardCollection: {skip: "does not forward command to primary shard"},
+    untrackUnshardedCollection: {skip: "does not forward command to primary shard"},
     update: {
         run: {
             sendsDbVersion: true,
@@ -785,6 +830,14 @@ commandsRemovedFromMongosSinceLastLTS.forEach(function(cmd) {
 
 const st = new ShardingTest({shards: 2, mongos: 2});
 
+// Database versioning tests only make sense when all collections are not tracked.
+const isTrackUnshardedUponCreationEnabled = FeatureFlagUtil.isPresentAndEnabled(
+    st.s.getDB('admin'), "TrackUnshardedCollectionsUponCreation");
+if (isTrackUnshardedUponCreationEnabled) {
+    st.stop();
+    quit();
+}
+
 const listCommandsRes = st.s0.adminCommand({listCommands: 1});
 assert.commandWorked(listCommandsRes);
 print("--------------------------------------------");
@@ -792,6 +845,9 @@ for (let command of Object.keys(listCommandsRes.commands)) {
     print(command);
 }
 
+const isMultiversion =
+    jsTest.options().shardMixedBinVersions || jsTest.options().useRandomBinVersionsWithinReplicaSet;
+
 (() => {
     // Validate test cases for all commands.
 
@@ -838,7 +894,7 @@ for (let command of Object.keys(listCommandsRes.commands)) {
 
     for (let command of Object.keys(listCommandsRes.commands)) {
         let testCase = testCases[command];
-        if (testCase.skip) {
+        if (testCase.skip || (isMultiversion && testCase.skipMultiversion)) {
             print("skipping " + command + ": " + testCase.skip);
             continue;
         }
@@ -877,7 +933,7 @@ for (let command of Object.keys(listCommandsRes.commands)) {
 
     for (let command of Object.keys(listCommandsRes.commands)) {
         let testCase = testCases[command];
-        if (testCase.skip) {
+        if (testCase.skip || (isMultiversion && testCase.skipMultiversion)) {
             print("skipping " + command + ": " + testCase.skip);
             continue;
         }
@@ -895,7 +951,7 @@ for (let command of Object.keys(listCommandsRes.commands)) {
 
     for (let command of Object.keys(listCommandsRes.commands)) {
         let testCase = testCases[command];
-        if (testCase.skip) {
+        if (testCase.skip || (isMultiversion && testCase.skipMultiversion)) {
             print("skipping " + command + ": " + testCase.skip);
             continue;
         }
@@ -908,4 +964,3 @@ for (let command of Object.keys(listCommandsRes.commands)) {
 })();
 
 st.stop();
-})();
