@@ -1,8 +1,5 @@
-(function() {
-'use strict';
-
-load("jstests/sharding/libs/find_chunks_util.js");
-load("jstests/sharding/libs/defragmentation_util.js");
+import {defragmentationUtil} from "jstests/sharding/libs/defragmentation_util.js";
+import {findChunksUtil} from "jstests/sharding/libs/find_chunks_util.js";
 
 Random.setRandomSeed();
 
@@ -40,7 +37,7 @@ let runTest = function(numCollections, dbName) {
     for (let i = 0; i < numCollections; ++i) {
         const numChunks = Random.randInt(28) + 2;
         const numZones = Random.randInt(numChunks / 2);
-        const docSizeBytes = Random.randInt(1024 * 1024) + 50;
+        const docSizeBytesRange = [50, 1024 * 1024];
 
         const coll = db[coll_prefix + i];
 
@@ -49,7 +46,7 @@ let runTest = function(numCollections, dbName) {
                                                        numChunks,
                                                        maxChunkFillMB,
                                                        numZones,
-                                                       docSizeBytes,
+                                                       docSizeBytesRange,
                                                        chunkSpacing,
                                                        true);
 
@@ -85,4 +82,3 @@ runTest(1, "singleCollection");
 runTest(3, "threeCollections");
 
 st.stop();
-})();
