@@ -1,14 +1,10 @@
 /**
  * Tests that direct shard connections are correctly allowed and disallowed using authentication.
  *
- * @tags: [featureFlagCheckForDirectShardOperations, featureFlagClusterCardinalityParameter,
- * requires_fcv_70]
+ * @tags: [featureFlagCheckForDirectShardOperations, requires_fcv_70]
  */
-(function() {
-'use strict';
-
-load("jstests/libs/feature_flag_util.js");
-load('jstests/sharding/libs/remove_shard_util.js');
+import {FeatureFlagUtil} from "jstests/libs/feature_flag_util.js";
+import {removeShard} from "jstests/sharding/libs/remove_shard_util.js";
 
 // Create a new sharded cluster for testing and enable auth.
 const st = new ShardingTest({name: jsTestName(), keyFile: "jstests/libs/key1", shards: 1});
@@ -25,36 +21,43 @@ assert(shardAdminDB.auth("admin", 'x'), "Authentication failed");
 assert.commandWorked(shardAdminDB.runCommand(
     {setParameter: 1, logComponentVerbosity: {sharding: {verbosity: 2}, assert: {verbosity: 1}}}));
 
+// TODO: (SERVER-87190) Remove once 8.0 becomes last lts.
+const failOnDirectOps =
+    FeatureFlagUtil.isPresentAndEnabled(shardAdminDB, "FailOnDirectShardOperations");
+const resetClusterCardinalityOnRemoveShard =
+    FeatureFlagUtil.isPresentAndEnabled(shardAdminDB, "RSEndpointClusterCardinalityParameter");
+
 function getUnauthorizedDirectWritesCount() {
     return assert.commandWorked(shardAdminDB.runCommand({serverStatus: 1}))
         .shardingStatistics.unauthorizedDirectShardOps;
 }
 
 function runTests(shouldBlockDirectConnections, directWriteCount) {
-    function assertUnauthorizedCountIncreased() {
-        const postDirectWriteCount = getUnauthorizedDirectWritesCount();
-        assert.gt(postDirectWriteCount,
-                  directWriteCount,
-                  `Number of direct write count didn't increased. Previous count: ${
-                      directWriteCount}, Current count: ${postDirectWriteCount}`);
-        directWriteCount = postDirectWriteCount;
-    }
-
     // Direct writes with root priviledges should always be authorized.
     assert.commandWorked(shardAdminTestDB.getCollection("coll").update(
         {x: {$exists: true}}, {$inc: {x: 1}}, {upsert: true}));
     assert.eq(getUnauthorizedDirectWritesCount(), directWriteCount);
 
-    // Direct writes with only read/write priviledges should depend on the cluster state.
+    // Direct writes with only read/write priviledges should depend on the cluster state and the
+    // failOnDirectShardOperations feature flag.
     shardAdminTestDB.createUser({user: "user", pwd: "y", roles: ["readWrite"]});
     assert(userTestDB.auth("user", "y"), "Authentication failed");
-    assert.commandWorked(userTestDB.getCollection("coll").update(
-        {x: {$exists: true}}, {$inc: {x: 1}}, {upsert: true}));
     if (!shouldBlockDirectConnections) {
         // In a single shard cluster, this should be allowed
+        assert.commandWorked(userTestDB.getCollection("coll").update(
+            {x: {$exists: true}}, {$inc: {x: 1}}, {upsert: true}));
         assert.eq(getUnauthorizedDirectWritesCount(), directWriteCount);
     } else {
-        assertUnauthorizedCountIncreased();
+        // In a 2 shard cluster, this will fail if the ff is enabled and warn otherwise.
+        if (failOnDirectOps) {
+            assert.commandFailedWithCode(userTestDB.getCollection("coll").update(
+                                             {x: {$exists: true}}, {$inc: {x: 1}}, {upsert: true}),
+                                         ErrorCodes.Unauthorized);
+        } else {
+            assert.commandWorked(userTestDB.getCollection("coll").update(
+                {x: {$exists: true}}, {$inc: {x: 1}}, {upsert: true}));
+        }
+        assert.eq(getUnauthorizedDirectWritesCount(), ++directWriteCount);
     }
 
     // Setting the server parameter should cause warnings to be emitted if the cluster only has one
@@ -64,8 +67,15 @@ function runTests(shouldBlockDirectConnections, directWriteCount) {
         ErrorCodes.InvalidOptions);
     // Skip this test in multiversion tests where the server parameter does not exist.
     if (res.ok) {
-        assert.commandWorked(userTestDB.getCollection("coll").update(
-            {x: {$exists: true}}, {$inc: {x: 1}}, {upsert: true}));
+        // In a 2 shard cluster, this will fail if the ff is enabled and warn otherwise.
+        if (failOnDirectOps && shouldBlockDirectConnections) {
+            assert.commandFailedWithCode(userTestDB.getCollection("coll").update(
+                                             {x: {$exists: true}}, {$inc: {x: 1}}, {upsert: true}),
+                                         ErrorCodes.Unauthorized);
+        } else {
+            assert.commandWorked(userTestDB.getCollection("coll").update(
+                {x: {$exists: true}}, {$inc: {x: 1}}, {upsert: true}));
+        }
         assert.eq(getUnauthorizedDirectWritesCount(), ++directWriteCount);
         assert.commandWorked(shardAdminDB.runCommand(
             {setParameter: 1, directConnectionChecksWithSingleShard: false}));
@@ -117,9 +127,13 @@ directWriteCount = runTests(true /* shouldBlockDirectConnections */, directWrite
 // Remove the second shard, this shouldn't affect the direct shard op checks.
 removeShard(mongosAdminUser, newShard.getURL());
 
-// With one shard again, direct shard operations should still be prevented.
+// With one shard again, direct shard operations should still be prevented assuming that
+// featureFlagRSEndpointClusterCardinalityParameter is disabled.
+// TODO (SERVER-67835) update this test to reflect the expected behavior for direct shard op checks
+// when RSEndpoint is enabled.
 jsTest.log("Running tests with one shard again.");
-directWriteCount = runTests(true /* shouldBlockDirectConnections */, directWriteCount);
+directWriteCount = runTests(
+    !resetClusterCardinalityOnRemoveShard /* shouldBlockDirectConnections */, directWriteCount);
 
 // Logout of final users
 mongosAdminUser.logout();
@@ -128,4 +142,3 @@ shardAdminDB.logout();
 // Stop the sharding test before the additional shard to ensure the test hooks run successfully.
 st.stop();
 newShard.stopSet();
-})();
