@@ -1,12 +1,9 @@
 /**
  * Tests that $documents stage continues even when the database does not exist
- * @tags: [multiversion_incompatible]
+ * @tags: [requires_fcv_62, multiversion_incompatible]
  *
  */
 
-(function() {
-"use strict";
-
 let st = new ShardingTest({shards: 3});
 
 function listDatabases(options) {
@@ -44,5 +41,4 @@ result = db.aggregate([
 ]);
 assert(result.toArray().length == 1);
 
-st.stop();
-})();
+st.stop();
\ No newline at end of file
