@@ -1,9 +1,6 @@
 // Tests changing the zones on a shard at runtime results in a correct distribution of chunks across
 // the cluster
-(function() {
-'use strict';
-
-load("jstests/sharding/libs/find_chunks_util.js");
+import {findChunksUtil} from "jstests/sharding/libs/find_chunks_util.js";
 
 const st = new ShardingTest({shards: 3, mongos: 1, other: {chunkSize: 1}});
 const dbName = 'test';
@@ -39,7 +36,10 @@ function assertBalanceCompleteAndStable(checkFunc, stepName) {
 
     assert.soon(checkFunc, 'Balance at step ' + stepName + ' did not happen', 3 * 60 * 1000, 2000);
 
-    st.awaitBalancerRound();
+    // When running with CSRS stepdowns, the balancer round might take longer to complete
+    const balancerRoundTimeout = TestData.runningWithConfigStepdowns ? 3 * 60 * 1000 : 60 * 1000;
+    st.awaitBalancerRound(balancerRoundTimeout);
+
     st.printShardingStatus(true);
     assert(checkFunc());
 
@@ -85,9 +85,9 @@ assertBalanceCompleteAndStable(function() {
 }, 'chunks to zones a and b');
 
 // Tag the entire collection to shard0 and wait for everything to move to that shard
-st.removeTagRange(ns, {_id: -100}, {_id: 100}, 'a');
-st.removeTagRange(ns, {_id: MinKey}, {_id: -100}, 'b');
-st.removeTagRange(ns, {_id: 100}, {_id: MaxKey}, 'b');
+st.removeTagRange(ns, {_id: -100}, {_id: 100});
+st.removeTagRange(ns, {_id: MinKey}, {_id: -100});
+st.removeTagRange(ns, {_id: 100}, {_id: MaxKey});
 
 st.removeShardTag(st.shard1.shardName, 'a');
 st.removeShardTag(st.shard2.shardName, 'b');
@@ -102,9 +102,8 @@ assertBalanceCompleteAndStable(function() {
 
 // Remove all zones and ensure collection is correctly redistributed
 st.removeShardTag(st.shard0.shardName, 'a');
-st.removeTagRange(ns, {_id: MinKey}, {_id: MaxKey}, 'a');
+st.removeTagRange(ns, {_id: MinKey}, {_id: MaxKey});
 
 assertBalanceCompleteAndStable(checkClusterEvenlyBalanced, 'final');
 
 st.stop();
-})();
