@@ -1,9 +1,6 @@
 /**
  * Test ClusterFindCmd with UUID for collection name fails (but does not crash)
  */
-(function() {
-"use strict";
-
 var cmdRes;
 var cursorId;
 
@@ -17,5 +14,4 @@ assert.commandWorked(db.adminCommand({enableSharding: db.getName()}));
 cmdRes = db.adminCommand({find: UUID()});
 assert.commandFailed(cmdRes);
 
-st.stop();
-})();
+st.stop();
\ No newline at end of file
