@@ -1,11 +1,11 @@
 /**
  * Tests the Config server observer.
  *
- *  @tags: [multiversion_incompatible]
+ * @tags: [
+ *   multiversion_incompatible,
+ *   temp_disabled_embedded_router_health_monitor,
+ * ]
  */
-(function() {
-'use strict';
-
 const kWaitForCompletedChecksCount = 20;
 const kWaitForPassedChecksCount = 10;
 
@@ -17,7 +17,6 @@ const params = {
             ]
         }),
         healthMonitoringIntervals: tojson({values: [{type: "configServer", interval: 200}]}),
-        featureFlagHealthMonitoring: true
     }
 };
 
@@ -44,5 +43,4 @@ assert.soon(() => {
     return false;
 }, 'Config server health check did not reach necessary count of passed checks', 60000, 1000);
 
-st.stop();
-})();
+st.stop();
\ No newline at end of file
