@@ -2,17 +2,17 @@
  * Tests that if the primary config server is blackholed from the point of view of mongos,
  * the Config server health checker will eventually crash the server.
  *
- *  @tags: [multiversion_incompatible]
+ *  @tags: [
+ *    multiversion_incompatible,
+ *    temp_disabled_embedded_router_health_monitor,
+ * ]
  */
-(function() {
-'use strict';
-
-load('jstests/replsets/rslib.js');
+import {reconfig} from "jstests/replsets/rslib.js";
 
 const kActiveFaultDurationSec = 12;
 
 // Crashed mongos will remain holding its socket as a zombie for some time.
-TestData.failIfUnterminatedProcesses = false;
+TestData.ignoreUnterminatedProcesses = true;
 
 // Checking index consistency involves talking to the primary config server which is blackholed from
 // the mongos in this test.
@@ -29,7 +29,6 @@ const mongosParams = {
         }),
         healthMonitoringIntervals: tojson({values: [{type: "configServer", interval: 1}]}),
         activeFaultDurationSecs: kActiveFaultDurationSec,
-        featureFlagHealthMonitoring: true
     }
 };
 
@@ -98,6 +97,8 @@ assert.soon(() => {
 
 // Mongos should not crash yet.
 assert.commandWorked(st.s0.adminCommand({"ping": 1}));
+let pidsBefore = _runningMongoChildProcessIds();
+let numPidsBefore = pidsBefore.length;
 
 jsTest.log('Partitioning the final config server replica from the mongos');
 st.config2.discardMessagesFrom(st.s, 1.0);
@@ -119,10 +120,22 @@ assert.soon(() => {
 
 try {
     // Refresh PIDs to force de-registration of the crashed mongos.
-    var pids = _runningMongoChildProcessIds();
-    jsTestLog(`Running processes: ${tojson(pids)}`);
+    assert.soon(
+        () => {
+            var numPidsNow = _runningMongoChildProcessIds().length;
+            return (numPidsBefore - numPidsNow) == 1;
+        },
+        () => {
+            var pids = _runningMongoChildProcessIds();
+            return `Encountered incorrect number of running processes. Expected: 11. Running processes: ${
+                tojson(pids)}`;
+        });
+    var pidsNow = _runningMongoChildProcessIds();
+    pidsBefore = pidsBefore.map((e) => e.toNumber());
+    pidsNow = pidsNow.map((e) => e.toNumber());
+    var difference = pidsBefore.filter((element) => !pidsNow.includes(element));
+    waitProgram(difference[0]);
     st.stop({skipValidatingExitCode: true, skipValidation: true});
 } catch (e) {
     jsTestLog(`Exception during shutdown: ${e}`);
 }
-}());
