@@ -1,11 +1,11 @@
 /**
  * Tests server status has correct fault/facet information.
  *
- *  @tags: [multiversion_incompatible]
+ * @tags: [
+ *   multiversion_incompatible,
+ *   temp_disabled_embedded_router_health_monitor,
+ * ]
  */
-(function() {
-'use strict';
-
 function changeObserverIntensity(observer, intensity) {
     let paramValue = {"values": [{"type": observer, "intensity": intensity}]};
     assert.commandWorked(
@@ -21,7 +21,6 @@ const params = {
                 {type: "dns", intensity: "off"}
             ]
         }),
-        featureFlagHealthMonitoring: true
     }
 };
 
@@ -94,5 +93,4 @@ assert(testObserver.totalChecksWithFailure >= 1);
 assert(testObserver.timeSinceLastCheckStartedMs >= 1);
 assert(testObserver.timeSinceLastCheckCompletedMs >= 1);
 
-st.stop();
-})();
+st.stop();
\ No newline at end of file
