@@ -1,9 +1,6 @@
 /**
  * This tests the basic cases for implicit database creation in a sharded cluster.
  */
-(function() {
-"use strict";
-
 const st = new ShardingTest({shards: 2});
 const configDB = st.s.getDB('config');
 
@@ -40,6 +37,5 @@ assert.commandWorked(st.s.adminCommand({addShard: newShard.getURL()}));
 
 assert.neq(null, configDB.databases.findOne({_id: 'unshardedDB'}));
 
-newShard.stopSet();
 st.stop();
-})();
+newShard.stopSet();
\ No newline at end of file
