@@ -1,9 +1,6 @@
 // Test SERVER-14306.  Do a query directly against a mongod with an in-memory sort and a limit that
 // doesn't cause the in-memory sort limit to be reached, then make sure the same limit also doesn't
 // cause the in-memory sort limit to be reached when running through a mongos.
-(function() {
-'use strict';
-
 var st = new ShardingTest({
     shards: 2,
     other: {
@@ -11,13 +8,9 @@ var st = new ShardingTest({
             {setParameter: {internalQueryMaxBlockingSortMemoryUsageBytes: 32 * 1024 * 1024}}
     }
 });
-assert.commandWorked(st.s.adminCommand({enableSharding: 'test'}));
-st.ensurePrimaryShard('test', st.shard0.shardName);
-
-// Make sure that at least 1 chunk is on another shard so that mongos doesn't treat this as a
-// single-shard query (which doesn't exercise the bug)
 assert.commandWorked(
-    st.s.adminCommand({shardCollection: 'test.skip', key: {_id: 'hashed'}, numInitialChunks: 64}));
+    st.s.adminCommand({enableSharding: 'test', primaryShard: st.shard0.shardName}));
+assert.commandWorked(st.s.adminCommand({shardCollection: 'test.skip', key: {_id: 'hashed'}}));
 
 var mongosCol = st.s.getDB('test').getCollection('skip');
 var shardCol = st.shard0.getDB('test').getCollection('skip');
@@ -52,4 +45,3 @@ assert.throwsWithCode(
     ErrorCodes.QueryExceededMemoryLimitNoDiskUseAllowed);
 
 st.stop();
-})();
