@@ -6,10 +6,7 @@
  * ]
  */
 
-(function() {
-"use strict";
-
-load("jstests/aggregation/extras/utils.js");  // For documentEq.
+import {documentEq} from "jstests/aggregation/extras/utils.js";
 
 // This test deliberately creates indexes in an inconsistent state.
 TestData.skipCheckingIndexesConsistentAcrossCluster = true;
@@ -73,8 +70,8 @@ const pipeline = [
     {$project: {_id: 0, indexName: "$$ROOT._id", inconsistentProperties: 1, missingFromShards: 1}}
 ];
 
-assert.commandWorked(st.s.adminCommand({enableSharding: dbName}));
-st.ensurePrimaryShard(dbName, st.shard0.shardName);
+assert.commandWorked(
+    st.s.adminCommand({enableSharding: dbName, primaryShard: st.shard0.shardName}));
 
 function shardCollectionWithChunkOnEachShard(collName) {
     const ns = dbName + "." + collName;
@@ -341,5 +338,4 @@ function shardCollectionWithChunkOnEachShard(collName) {
            tojson(res));
 })();
 
-st.stop();
-})();
+st.stop();
\ No newline at end of file
