@@ -5,22 +5,12 @@
  * @tags: [requires_fcv_60, uses_transactions]
  */
 
-(function() {
-"use strict";
-
 // This test makes assertions about the number of sessions, which are not compatible with
 // implicit sessions.
 TestData.disableImplicitSessions = true;
 
-const st = new ShardingTest({
-    shards: 1,
-    shardOptions: {
-        setParameter: {
-            TransactionRecordMinimumLifetimeMinutes: 0,
-            storeFindAndModifyImagesInSideCollection: true,
-        }
-    }
-});
+const st = new ShardingTest(
+    {shards: 1, shardOptions: {setParameter: {TransactionRecordMinimumLifetimeMinutes: 0}}});
 
 const shard0Rst = st.rs0;
 const shard0Primary = shard0Rst.getPrimary();
@@ -125,5 +115,4 @@ assert.eq(0,
           tojson(transactionsCollOnPrimary.find({txnNum: kInternalTxnNumber}).toArray()));
 assert.eq(0, imageCollOnPrimary.find().itcount());
 
-st.stop();
-})();
+st.stop();
\ No newline at end of file
