@@ -3,9 +3,6 @@
  *
  * @tags: [requires_fcv_60, uses_transactions]
  */
-(function() {
-"use strict";
-
 const kCollName = "testColl";
 
 function verifyInternalSessionsForExternalClients(testDB, {expectFail}) {
@@ -107,5 +104,4 @@ authutil.asCluster(st.s, keyFile, function() {
     verifyInternalSessionsForExternalClients(mongosDB, {expectFail: false});
 });
 
-st.stop();
-})();
+st.stop();
\ No newline at end of file
