@@ -1,12 +1,19 @@
 /*
  * Utilities for testing that internal transactions for retryable writes can be retried.
  */
-'use strict';
 
-load('jstests/sharding/internal_txns/libs/fixture_helpers.js');
-load('jstests/sharding/libs/sharded_transactions_helpers.js');
+import {runTxnRetryOnTransientError} from "jstests/sharding/internal_txns/libs/fixture_helpers.js";
+import {
+    getImageEntriesForTxn,
+    getImageEntriesForTxnOnNode,
+    getOplogEntriesForTxn,
+    getTxnEntriesForSession,
+    makeAbortTransactionCmdObj,
+    makeCommitTransactionCmdObj,
+    makePrepareTransactionCmdObj,
+} from "jstests/sharding/libs/sharded_transactions_helpers.js";
 
-function getOplogEntriesForTxnWithRetries(rs, lsid, txnNumber) {
+export function getOplogEntriesForTxnWithRetries(rs, lsid, txnNumber) {
     let oplogEntries;
     assert.soon(
         () => {
@@ -32,7 +39,7 @@ function getOplogEntriesForTxnWithRetries(rs, lsid, txnNumber) {
     return oplogEntries;
 }
 
-function RetryableInternalTransactionTest(collectionOptions = {}) {
+export function RetryableInternalTransactionTest(collectionOptions = {}) {
     // Transactions with more than two operations will have the behavior of large transactions and
     // span multiple oplog entries.
     const maxNumberOfTransactionOperationsInSingleOplogEntry = 2;
@@ -91,6 +98,7 @@ function RetryableInternalTransactionTest(collectionOptions = {}) {
     function assertConsistentImageEntries(lsid, txnNumber) {
         const imageEntriesOnPrimary =
             getImageEntriesForTxnOnNode(st.rs0.getPrimary(), lsid, txnNumber);
+        st.rs0.awaitReplication();
         st.rs0.getSecondaries().forEach(secondary => {
             const imageEntriesOnSecondary = getImageEntriesForTxnOnNode(secondary, lsid, txnNumber);
             assert.eq(imageEntriesOnSecondary, imageEntriesOnPrimary);
@@ -103,13 +111,13 @@ function RetryableInternalTransactionTest(collectionOptions = {}) {
                 remember: true,
                 startClean: false,
             });
-            const newPrimary = st.rs0.getPrimary();
+            st.rs0.waitForPrimary();
         } else if (mode == kTestMode.kFailover) {
             const oldPrimary = st.rs0.getPrimary();
             assert.commandWorked(
                 oldPrimary.adminCommand({replSetStepDown: ReplSetTest.kForeverSecs, force: true}));
             assert.commandWorked(oldPrimary.adminCommand({replSetFreeze: 0}));
-            const newPrimary = st.rs0.getPrimary();
+            st.rs0.waitForPrimary();
         }
     }
 
@@ -125,15 +133,10 @@ function RetryableInternalTransactionTest(collectionOptions = {}) {
         assert.commandWorked(mongosTestDB.adminCommand(commitCmdObj));
     }
 
-    function testNonRetryableBasic(cmdObj, {
-        txnOptions,
-        testMode,
-        expectFindAndModifyImageInOplog,
-        expectFindAndModifyImageInSideCollection
-    }) {
+    function testNonRetryableBasic(
+        cmdObj, {txnOptions, testMode, expectFindAndModifyImageInSideCollection}) {
         // A findAndModify write statement in a non-retryable transaction will not generate a
         // pre/post image.
-        assert(!expectFindAndModifyImageInOplog);
         assert(!expectFindAndModifyImageInSideCollection);
         jsTest.log("Testing retrying a non-retryable internal transaction");
         cmdObj.startTransaction = true;
@@ -150,7 +153,7 @@ function RetryableInternalTransactionTest(collectionOptions = {}) {
 
         const initialTxnStateBefore = getTransactionState(initialLsid, initialTxnNumber);
         assert.eq(initialTxnStateBefore.oplogEntries.length,
-                  (txnOptions.isPreparedTxn ? 2 : 1) + (expectFindAndModifyImageInOplog ? 1 : 0),
+                  (txnOptions.isPreparedTxn ? 2 : 1),
                   initialTxnStateBefore.oplogEntries);
         assert.eq(initialTxnStateBefore.imageEntries.length,
                   expectFindAndModifyImageInSideCollection ? 1 : 0,
@@ -171,14 +174,9 @@ function RetryableInternalTransactionTest(collectionOptions = {}) {
         assert.commandWorked(mongosTestColl.remove({}));
     }
 
-    function testRetryableBasic(cmdObj, {
-        txnOptions,
-        testMode,
-        expectFindAndModifyImageInOplog,
-        expectFindAndModifyImageInSideCollection,
-        checkRetryResponseFunc
-    }) {
-        assert(!expectFindAndModifyImageInOplog || !expectFindAndModifyImageInSideCollection);
+    function testRetryableBasic(
+        cmdObj,
+        {txnOptions, testMode, expectFindAndModifyImageInSideCollection, checkRetryResponseFunc}) {
         jsTest.log(
             "Testing retrying a retryable internal transaction with one applyOps oplog entry");
         cmdObj.startTransaction = true;
@@ -196,7 +194,7 @@ function RetryableInternalTransactionTest(collectionOptions = {}) {
 
         const initialTxnStateBefore = getTransactionState(initialLsid, initialTxnNumber);
         assert.eq(initialTxnStateBefore.oplogEntries.length,
-                  (txnOptions.isPreparedTxn ? 2 : 1) + (expectFindAndModifyImageInOplog ? 1 : 0),
+                  (txnOptions.isPreparedTxn ? 2 : 1),
                   initialTxnStateBefore.oplogEntries);
         assert.eq(initialTxnStateBefore.imageEntries.length,
                   expectFindAndModifyImageInSideCollection ? 1 : 0,
@@ -236,15 +234,9 @@ function RetryableInternalTransactionTest(collectionOptions = {}) {
         assert.commandWorked(mongosTestColl.remove({}));
     }
 
-    function testRetryableLargeTxn(cmdObj, {
-        txnOptions,
-        testMode,
-        expectFindAndModifyImageInOplog,
-        expectFindAndModifyImageInSideCollection,
-        checkRetryResponseFunc
-    }) {
-        assert(!expectFindAndModifyImageInOplog || !expectFindAndModifyImageInSideCollection);
-
+    function testRetryableLargeTxn(
+        cmdObj,
+        {txnOptions, testMode, expectFindAndModifyImageInSideCollection, checkRetryResponseFunc}) {
         jsTest.log(
             "Testing retrying a retryable internal transaction with more than one applyOps oplog entry");
 
@@ -315,8 +307,7 @@ function RetryableInternalTransactionTest(collectionOptions = {}) {
         const expectedOplogLength =
             Math.floor(stmtId / maxNumberOfTransactionOperationsInSingleOplogEntry);
         assert.eq(initialTxnStateBefore.oplogEntries.length,
-                  (txnOptions.isPreparedTxn ? expectedOplogLength + 1 : expectedOplogLength) +
-                      (expectFindAndModifyImageInOplog ? 1 : 0));
+                  (txnOptions.isPreparedTxn ? expectedOplogLength + 1 : expectedOplogLength));
         assert.eq(initialTxnStateBefore.imageEntries.length,
                   expectFindAndModifyImageInSideCollection ? 1 : 0,
                   initialTxnStateBefore.imageEntries);
@@ -365,7 +356,6 @@ function RetryableInternalTransactionTest(collectionOptions = {}) {
         txnOptions,
         testMode,
         expectRetryToSucceed,
-        expectFindAndModifyImageInOplog,
         expectFindAndModifyImageInSideCollection,
         checkRetryResponseFunc
     }) {
@@ -383,7 +373,6 @@ function RetryableInternalTransactionTest(collectionOptions = {}) {
             txnOptions,
             testMode,
             expectRetryToSucceed,
-            expectFindAndModifyImageInOplog,
             expectFindAndModifyImageInSideCollection,
             checkRetryResponseFunc
         });
@@ -452,15 +441,9 @@ function RetryableInternalTransactionTest(collectionOptions = {}) {
     function testRetryFindAndModify(findAndModifyCmdObj, {
         txnOptions,
         testMode,
-        enableFindAndModifyImageCollection,
         expectRetryToSucceed,
         expectFindAndModifyImage,
     }) {
-        const shard0Primary = st.rs0.getPrimary();
-        assert.commandWorked(shard0Primary.adminCommand({
-            setParameter: 1,
-            storeFindAndModifyImagesInSideCollection: enableFindAndModifyImageCollection
-        }));
         const checkRetryResponseFunc = (initialRes, retryRes) => {
             assert.eq(initialRes.lastErrorObject, retryRes.lastErrorObject);
             assert.eq(initialRes.value, retryRes.value);
@@ -470,10 +453,8 @@ function RetryableInternalTransactionTest(collectionOptions = {}) {
             txnOptions,
             testMode,
             expectRetryToSucceed,
-            expectFindAndModifyImageInOplog: expectRetryToSucceed && expectFindAndModifyImage &&
-                !enableFindAndModifyImageCollection,
-            expectFindAndModifyImageInSideCollection: expectRetryToSucceed &&
-                expectFindAndModifyImage && enableFindAndModifyImageCollection,
+            expectFindAndModifyImageInSideCollection:
+                expectRetryToSucceed && expectFindAndModifyImage,
             checkRetryResponseFunc
         });
     }
@@ -481,12 +462,9 @@ function RetryableInternalTransactionTest(collectionOptions = {}) {
     function testRetryFindAndModifyUpsert({
         txnOptions,
         testMode,
-        enableFindAndModifyImageCollection,
         expectRetryToSucceed,
     }) {
-        jsTest.log(
-            "Testing findAndModify upsert (i.e. no preImage or postImage) with enableFindAndModifyImageCollection: " +
-            enableFindAndModifyImageCollection);
+        jsTest.log("Testing findAndModify upsert (i.e. no preImage or postImage)");
 
         const findAndModifyCmdObj = {
             findAndModify: kCollName,
@@ -498,7 +476,6 @@ function RetryableInternalTransactionTest(collectionOptions = {}) {
         testRetryFindAndModify(findAndModifyCmdObj, {
             txnOptions,
             testMode,
-            enableFindAndModifyImageCollection,
             expectFindAndModifyImage,
             expectRetryToSucceed,
         });
@@ -507,12 +484,9 @@ function RetryableInternalTransactionTest(collectionOptions = {}) {
     function testRetryFindAndModifyUpdateWithPreImage({
         txnOptions,
         testMode,
-        enableFindAndModifyImageCollection,
         expectRetryToSucceed,
     }) {
-        jsTest.log(
-            "Testing findAndModify update with preImage with enableFindAndModifyImageCollection: " +
-            enableFindAndModifyImageCollection);
+        jsTest.log("Testing findAndModify update with preImage");
 
         assert.commandWorked(mongosTestColl.insert([{_id: -1, x: -1}]));
         const findAndModifyCmdObj = {
@@ -524,7 +498,6 @@ function RetryableInternalTransactionTest(collectionOptions = {}) {
         testRetryFindAndModify(findAndModifyCmdObj, {
             txnOptions,
             testMode,
-            enableFindAndModifyImageCollection,
             expectFindAndModifyImage,
             expectRetryToSucceed,
         });
@@ -533,12 +506,9 @@ function RetryableInternalTransactionTest(collectionOptions = {}) {
     function testRetryFindAndModifyUpdateWithPostImage({
         txnOptions,
         testMode,
-        enableFindAndModifyImageCollection,
         expectRetryToSucceed,
     }) {
-        jsTest.log(
-            "Testing findAndModify update with postImage with enableFindAndModifyImageCollection: " +
-            enableFindAndModifyImageCollection);
+        jsTest.log("Testing findAndModify update with postImage");
 
         assert.commandWorked(mongosTestColl.insert([{_id: -1, x: -1}]));
         const findAndModifyCmdObj = {
@@ -551,7 +521,6 @@ function RetryableInternalTransactionTest(collectionOptions = {}) {
         testRetryFindAndModify(findAndModifyCmdObj, {
             txnOptions,
             testMode,
-            enableFindAndModifyImageCollection,
             expectFindAndModifyImage,
             expectRetryToSucceed,
         });
@@ -560,12 +529,9 @@ function RetryableInternalTransactionTest(collectionOptions = {}) {
     function testRetryFindAndModifyRemove({
         txnOptions,
         testMode,
-        enableFindAndModifyImageCollection,
         expectRetryToSucceed,
     }) {
-        jsTest.log(
-            "Testing findAndModify remove (i.e. with preImage) with enableFindAndModifyImageCollection: " +
-            enableFindAndModifyImageCollection);
+        jsTest.log("Testing findAndModify remove (i.e. with preImage)");
 
         assert.commandWorked(mongosTestColl.insert([{_id: -1, x: -1}]));
         const findAndModifyCmdObj = {
@@ -577,7 +543,6 @@ function RetryableInternalTransactionTest(collectionOptions = {}) {
         testRetryFindAndModify(findAndModifyCmdObj, {
             txnOptions,
             testMode,
-            enableFindAndModifyImageCollection,
             expectFindAndModifyImage,
             expectRetryToSucceed,
         });
@@ -609,7 +574,6 @@ function RetryableInternalTransactionTest(collectionOptions = {}) {
     }
 
     this.runFindAndModifyTestsEnableImageCollection = function(testOptions) {
-        testOptions.enableFindAndModifyImageCollection = true;
         runFindAndModifyTests(testOptions);
     };
 
