@@ -2,12 +2,11 @@
  * Tests that retryable internal transactions for findAndModify are retryable and other kinds of
  * transactions for findAndModify are not retryable.
  *
- * @tags: [requires_fcv_60, uses_transactions]
+ * @tags: [requires_fcv_60, uses_transactions, exclude_from_large_txns]
  */
-(function() {
-'use strict';
-
-load('jstests/sharding/internal_txns/libs/retryable_internal_transaction_test.js');
+import {
+    RetryableInternalTransactionTest
+} from "jstests/sharding/internal_txns/libs/retryable_internal_transaction_test.js";
 
 const transactionTest = new RetryableInternalTransactionTest();
 
@@ -36,10 +35,6 @@ const transactionTest = new RetryableInternalTransactionTest();
     transactionTest.runTestsForAllRetryableInternalTransactionTypes(
         transactionTest.runFindAndModifyTestsEnableImageCollection,
         transactionTest.TestMode.kNonRecovery);
-    transactionTest.runTestsForAllRetryableInternalTransactionTypes(
-        transactionTest.runFindAndModifyTestsDisableImageCollection,
-        transactionTest.TestMode.kNonRecovery);
 }
 
-transactionTest.stop();
-})();
+transactionTest.stop();
\ No newline at end of file
