@@ -4,9 +4,6 @@
  *
  * @tags: [requires_fcv_60, uses_transactions, requires_persistence]
  */
-(function() {
-'use strict';
-
 // For the test case where we abort a prepared internal transaction for retryable findAndModify with
 // a pre/post image, the image collection on the primary is expected to be inconsistent with the
 // image collection on secondaries. The reason is that for prepared transactions, the pre/post image
@@ -16,14 +13,13 @@
 // the write to image collection only gets rolled back on secondaries.
 TestData.skipCheckDBHashes = true;
 
-load("jstests/replsets/rslib.js");
-load("jstests/sharding/libs/sharded_transactions_helpers.js");
+import {
+    makeAbortTransactionCmdObj,
+    makeCommitTransactionCmdObj,
+    makePrepareTransactionCmdObj,
+} from "jstests/sharding/libs/sharded_transactions_helpers.js";
 
-function runTest(st, stepDownShard0PrimaryFunc, testOpts = {
-    runFindAndModifyWithPreOrPostImage,
-    abortTxnAfterFailover,
-    enableFindAndModifyImageCollection
-}) {
+function runTest(st, stepDownShard0PrimaryFunc, testOpts) {
     jsTest.log("Testing with options " + tojson(testOpts));
 
     const sessionUUID = UUID();
@@ -38,11 +34,6 @@ function runTest(st, stepDownShard0PrimaryFunc, testOpts = {
     let testDB = st.rs0.getPrimary().getDB(kDbName);
     let testColl = testDB.getCollection(kCollName);
 
-    assert.commandWorked(testDB.adminCommand({
-        setParameter: 1,
-        storeFindAndModifyImagesInSideCollection: testOpts.enableFindAndModifyImageCollection
-    }));
-
     assert.commandWorked(testDB.createCollection(kCollName));
     if (testOpts.runFindAndModifyWithPreOrPostImage) {
         assert.commandWorked(testColl.insert({_id: 0, x: 0}));
@@ -198,5 +189,4 @@ function runTest(st, stepDownShard0PrimaryFunc, testOpts = {
     });
 
     st.stop();
-}
-})();
+}
\ No newline at end of file
