@@ -3,15 +3,13 @@
  * transactions that start and abort on the donor(s) before resharding are not retryable on the
  * recipient after resharding.
  *
- * @tags: [requires_fcv_60, uses_transactions, requires_persistence]
+ * @tags: [requires_fcv_60, uses_transactions, requires_persistence, exclude_from_large_txns]
  */
-(function() {
-"use strict";
-
-load("jstests/sharding/internal_txns/libs/resharding_test.js");
+import {
+    InternalTransactionReshardingTest
+} from "jstests/sharding/internal_txns/libs/resharding_test.js";
 
 const transactionTest = new InternalTransactionReshardingTest({reshardInPlace: false});
 transactionTest.runTestForInsertUpdateDeleteBeforeResharding(
     transactionTest.InternalTxnType.kRetryable, true /* abortOnInitialTry */);
-transactionTest.stop();
-})();
+transactionTest.stop();
\ No newline at end of file
