@@ -5,13 +5,11 @@
  *
  * @tags: [requires_fcv_60, uses_transactions, requires_persistence, exclude_from_large_txns]
  */
-(function() {
-"use strict";
-
-load("jstests/sharding/internal_txns/libs/chunk_migration_test.js");
+import {
+    InternalTransactionChunkMigrationTest
+} from "jstests/sharding/internal_txns/libs/chunk_migration_test.js";
 
 const transactionTest = new InternalTransactionChunkMigrationTest();
 transactionTest.runTestForInsertUpdateDeleteDuringChunkMigration(
     transactionTest.InternalTxnType.kRetryable, false /* abortOnInitialTry */);
-transactionTest.stop();
-})();
+transactionTest.stop();
\ No newline at end of file
