@@ -5,10 +5,9 @@
  *
  * @tags: [requires_fcv_60, uses_transactions, requires_persistence, exclude_from_large_txns]
  */
-(function() {
-"use strict";
-
-load("jstests/sharding/internal_txns/libs/resharding_test.js");
+import {
+    InternalTransactionReshardingTest
+} from "jstests/sharding/internal_txns/libs/resharding_test.js";
 
 const abortOnInitialTry = false;
 
@@ -24,5 +23,4 @@ const abortOnInitialTry = false;
     transactionTest.runTestForInsertUpdateDeleteDuringResharding(
         transactionTest.InternalTxnType.kRetryable, abortOnInitialTry);
     transactionTest.stop();
-}
-})();
+}
\ No newline at end of file
