@@ -8,13 +8,11 @@
  *
  * @tags: [requires_fcv_60, uses_transactions, exclude_from_large_txns]
  */
-(function() {
-'use strict';
-
-load("jstests/sharding/internal_txns/libs/retryable_internal_transaction_test.js");
+import {
+    RetryableInternalTransactionTest
+} from "jstests/sharding/internal_txns/libs/retryable_internal_transaction_test.js";
 
 const transactionTest = new RetryableInternalTransactionTest();
 transactionTest.runTestsForAllRetryableInternalTransactionTypes(
     transactionTest.runInsertUpdateDeleteTests, transactionTest.TestMode.kFailover);
-transactionTest.stop();
-})();
+transactionTest.stop();
\ No newline at end of file
