@@ -2,8 +2,6 @@
  * Test that the keys on config server are rotated according to the KeysRotationIntervalSec value
  */
 
-(function() {
-"use strict";
 const kRotationInterval = 30;
 let st = new ShardingTest({
     mongos: 1,
@@ -26,5 +24,4 @@ keys.toArray().forEach(function(key, i) {
                "key document " + i + ": " + tojson(key) +
                    "expiresAt value is greater than: " + tojson(maxExpireTime));
 });
-st.stop();
-})();
+st.stop();
\ No newline at end of file
