@@ -2,10 +2,7 @@
  * Run the kill_sessions tests against a sharded cluster.
  */
 
-load("jstests/libs/kill_sessions.js");
-
-(function() {
-'use strict';
+import {KillSessionsTestHelper} from "jstests/libs/kill_sessions.js";
 
 // This test involves killing all sessions, which will not work as expected if the kill command is
 // sent with an implicit session.
@@ -44,12 +41,6 @@ function runTests(needAuth) {
             host.getDB("local").auth("__system", "foopdedoop");
         }
         hosts.push(host);
-
-        assert.soon(function() {
-            var fcv =
-                host.getDB("admin").runCommand({getParameter: 1, featureCompatibilityVersion: 1});
-            return fcv["ok"] && fcv["featureCompatibilityVersion"] != "3.4";
-        });
     }
 
     var args = [forExec, forKill, hosts];
@@ -64,4 +55,3 @@ function runTests(needAuth) {
 
 runTests(true);
 runTests(false);
-})();
