@@ -6,21 +6,10 @@
  * monotonically increasing.
  */
 
-(function() {
-"use strict";
-
-load("jstests/libs/feature_flag_util.js");
-
 const numConnections = 10;
 const st = new ShardingTest({shards: 1, mongos: 1});
 const admin = st.s.getDB("admin");
 
-if (!FeatureFlagUtil.isEnabled(st.s.getDB("test"), "ConnHealthMetrics")) {
-    jsTestLog('Skipping test because the connection health metrics feature flag is disabled.');
-    st.stop();
-    return;
-}
-
 let previous = 0;
 for (var i = 0; i < numConnections; i++) {
     const conn = new Mongo(admin.getMongo().host);
@@ -34,5 +23,4 @@ for (var i = 0; i < numConnections; i++) {
 // and the metric should have increased for most if not all connections.
 assert.gt(previous, 0);
 
-st.stop();
-})();
+st.stop();
\ No newline at end of file
