@@ -1,9 +1,7 @@
 //
 // Test the listShards command by adding stand-alone and replica-set shards to a cluster
 //
-(function() {
-'use strict';
-load('jstests/sharding/libs/remove_shard_util.js');
+import {removeShard} from "jstests/sharding/libs/remove_shard_util.js";
 
 // TODO SERVER-50144 Remove this and allow orphan checking.
 // This test calls removeShard which can leave docs in config.rangeDeletions in state "pending",
@@ -55,5 +53,4 @@ assert(!checkShardName('repl', shardsArray),
        'listShards command returned removed replica set shard: ' + tojson(shardsArray));
 
 rs1.stopSet();
-st.stop();
-})();
+st.stop();
\ No newline at end of file
