@@ -6,10 +6,7 @@
  * @tags: [does_not_support_stepdowns, requires_streamable_rsm]
  */
 
-(function() {
-"use strict";
-
-load("jstests/sharding/libs/sharding_state_test.js");
+import {ShardingStateTest} from "jstests/sharding/libs/sharding_state_test.js";
 
 const st = new ShardingTest({config: 1, shards: {rs0: {nodes: 1}}});
 const rs = st.rs0;
@@ -32,5 +29,4 @@ jsTestLog("Checking sharding state after failover.");
 ShardingStateTest.failoverToMember(rs, newNode);
 ShardingStateTest.checkShardingState(st);
 
-st.stop();
-})();
+st.stop();
\ No newline at end of file
