@@ -6,10 +6,7 @@
  * @tags: [does_not_support_stepdowns, requires_streamable_rsm]
  */
 
-(function() {
-"use strict";
-
-load("jstests/sharding/libs/sharding_state_test.js");
+import {ShardingStateTest} from "jstests/sharding/libs/sharding_state_test.js";
 
 const st = new ShardingTest({
     config: 1,
@@ -17,7 +14,16 @@ const st = new ShardingTest({
 });
 const configRS = st.configRS;
 
-const newNode = ShardingStateTest.addReplSetNode({replSet: configRS, serverTypeFlag: "configsvr"});
+let newNode;
+if (TestData.configShard) {
+    newNode = ShardingStateTest.addReplSetNode({
+        replSet: configRS,
+        serverTypeFlag: "configsvr",
+        newNodeParams: "featureFlagTransitionToCatalogShard=true"
+    });
+} else {
+    newNode = ShardingStateTest.addReplSetNode({replSet: configRS, serverTypeFlag: "configsvr"});
+}
 
 jsTestLog("Checking sharding state before failover.");
 ShardingStateTest.checkShardingState(st);
@@ -26,5 +32,4 @@ jsTestLog("Checking sharding state after failover.");
 ShardingStateTest.failoverToMember(configRS, newNode);
 ShardingStateTest.checkShardingState(st);
 
-st.stop();
-})();
+st.stop();
\ No newline at end of file
