@@ -5,10 +5,7 @@
  * @tags: [requires_persistence, does_not_support_stepdowns, requires_streamable_rsm]
  */
 
-(function() {
-"use strict";
-
-load("jstests/sharding/libs/sharding_state_test.js");
+import {ShardingStateTest} from "jstests/sharding/libs/sharding_state_test.js";
 
 const st = new ShardingTest({config: 1, shards: {rs0: {nodes: 1}}});
 const rs = st.rs0;
@@ -20,5 +17,4 @@ jsTestLog("Ensuring node is up as a primary and checking sharding state");
 ShardingStateTest.failoverToMember(rs, primary);
 ShardingStateTest.checkShardingState(st);
 
-st.stop();
-})();
+st.stop();
\ No newline at end of file
