@@ -3,16 +3,14 @@
 // This test is to ensure that localhost authentication works correctly against a sharded
 // cluster whether they are hosted with "localhost" or a hostname.
 
-// Checking UUID and index consistency, which occurs on ShardingTest.stop, involves using a
-// mongos to read data on the config server, but this test uses a special shutdown function
-// which stops the mongoses before calling ShardingTest.stop.
+// The following checks, which occurs on ShardingTest.stop, involve using a mongos to read data on
+// the config server, but this test uses a special shutdown function which stops the mongoses before
+// calling ShardingTest.stop.
 TestData.skipCheckingUUIDsConsistentAcrossCluster = true;
 TestData.skipCheckingIndexesConsistentAcrossCluster = true;
 TestData.skipCheckOrphans = true;
 TestData.skipCheckShardFilteringMetadata = true;
-
-(function() {
-'use strict';
+TestData.skipCheckMetadataConsistency = true;
 
 var replSetName = "replsets_server-6591";
 var keyfile = "jstests/libs/key1";
@@ -38,7 +36,7 @@ var addUsersToEachShard = function(st) {
 
 var addShard = function(st, shouldPass) {
     adhocShard++;
-    var rs =
+    const rs =
         new ReplSetTest({nodes: 1, host: 'localhost', name: 'localhostAuthShard-' + adhocShard});
     rs.startSet({shardsvr: "", keyFile: keyfile, auth: ""});
     rs.initiate();
@@ -178,8 +176,7 @@ var setupSharding = function(shardingTest) {
     var mongo = shardingTest.s;
 
     print("============ enabling sharding on test.foo.");
-    mongo.getDB("admin").runCommand({enableSharding: "test"});
-    shardingTest.ensurePrimaryShard('test', st.shard1.shardName);
+    mongo.getDB("admin").runCommand({enableSharding: "test", primaryShard: st.shard1.shardName});
     mongo.getDB("admin").runCommand({shardCollection: "test.foo", key: {_id: 1}});
 
     var test = mongo.getDB("test");
@@ -243,4 +240,3 @@ extraShards.forEach(function(rs) {
     rs.stopSet();
 });
 st.stop();
-})();
