@@ -1,5 +1,5 @@
 /**
- * Tests the rewrite of NetworkInterfaceExceededTimeLimit exception coming from
+ * Tests the rewrite of PooledConnectionAcquisitionExceededTimeLimit coming from
  * `executor/connection_pool.cpp` into MaxTimeMSError when MaxTimeMS option is set for a given
  * sharding command.
  *
@@ -9,38 +9,33 @@
  * ]
  */
 
-(function() {
-"use strict";
-
-load("jstests/libs/fail_point_util.js");
+import {configureFailPoint} from "jstests/libs/fail_point_util.js";
 
 const databaseName = "my-database";
 const collectionName = "my-collection";
 
-function generateInsertCommand(maxTimeMS) {
-    return {insert: collectionName, documents: [{}], maxTimeMS: maxTimeMS};
+function generateFindCommand(maxTimeMS) {
+    return {find: collectionName, maxTimeMS: maxTimeMS};
 }
 
 const st = new ShardingTest({shards: 1, mongos: 1});
 const database = st.s0.getDB(databaseName);
 const collection = database.getCollection(collectionName);
-const session = database.getMongo().startSession({causalConsistency: false});
 
 assert.commandWorked(
     database.runCommand({create: collection.getName(), writeConcern: {w: "majority"}}));
 
-assert.commandWorked(database.runCommand(generateInsertCommand(1000)));
+assert.commandWorked(database.runCommand(generateFindCommand(1000)));
 
+// Mimic 1 second connection acquisition timeout via fail point.
 const failpoint =
     configureFailPoint(st.s, "forceExecutorConnectionPoolTimeout", {"timeout": 1000}, "alwaysOn");
 
-assert.commandFailedWithCode(database.runCommand(generateInsertCommand(1)),
+// We test that connection acquisition errors are rewritten to MaxTimeMSExpired due to the user
+// provided maxTimeMS.
+assert.commandFailedWithCode(database.runCommand(generateFindCommand(1000)),
                              ErrorCodes.MaxTimeMSExpired);
 
-assert.commandFailedWithCode(database.runCommand(generateInsertCommand(30000)),
-                             ErrorCodes.NetworkInterfaceExceededTimeLimit);
-
 failpoint.off();
 
 st.stop();
-}());
