@@ -5,10 +5,8 @@
  *   requires_fcv_70,
  * ]
  */
-(function() {
-'use strict';
-load("jstests/sharding/libs/find_chunks_util.js");
-load("jstests/libs/fail_point_util.js");
+import {configureFailPointForRS} from "jstests/libs/fail_point_util.js";
+import {findChunksUtil} from "jstests/sharding/libs/find_chunks_util.js";
 
 /* Create new sharded collection on testDB */
 let _collCounter = 0;
@@ -56,10 +54,13 @@ function setOnCurrentShardSince(mongoS, coll, extraQuery, refTimestamp, offsetIn
 }
 
 /* Set jumbo flag to true */
-function setJumboFlag(configDB, coll, chunkQuery) {
-    const collUuid = configDB.collections.findOne({_id: coll.getFullName()}).uuid;
+function setJumboFlag(mongoS, coll, chunkQuery) {
+    // Use 'retryWrites' when writing to the configsvr because they are not automatically retried.
+    const mongosSession = mongoS.startSession({retryWrites: true});
+    const sessionConfigDB = mongosSession.getDatabase('config');
+    const collUuid = sessionConfigDB.collections.findOne({_id: coll.getFullName()}).uuid;
     const query = Object.assign({uuid: collUuid}, chunkQuery);
-    assert.commandWorked(configDB.chunks.update(query, {$set: {jumbo: true}}));
+    assert.commandWorked(sessionConfigDB.chunks.update(query, {$set: {jumbo: true}}));
 }
 
 function setHistoryWindowInSecs(st, valueInSeconds) {
@@ -71,34 +72,47 @@ function resetHistoryWindowInSecs(st) {
     configureFailPointForRS(st.configRS.nodes, "overrideHistoryWindowInSecs", {}, "off");
 }
 
+function updateBalancerParameter(st, paramName, valueInMS) {
+    let prevParamValue = null;
+    st.forEachConfigServer((conn) => {
+        const res = conn.adminCommand({setParameter: 1, [paramName]: valueInMS});
+        assert.commandWorked(res);
+        prevParamValue = res.was;
+    });
+    return prevParamValue;
+}
+
 let defaultAutoMergerThrottlingMS = null;
 function setBalancerMergeThrottling(st, valueInMS) {
-    st.forEachConfigServer((conn) => {
-        const res = conn.adminCommand({setParameter: 1, autoMergerThrottlingMS: valueInMS});
-        assert.commandWorked(res);
-        defaultAutoMergerThrottlingMS = res.was;
-    });
+    const prevValue = updateBalancerParameter(st, "autoMergerThrottlingMS", valueInMS);
+    // Cache only the initial [default] parameter value
+    if (!defaultAutoMergerThrottlingMS)
+        defaultAutoMergerThrottlingMS = prevValue;
 }
 
 function resetBalancerMergeThrottling(st) {
-    if (!defaultAutoMergerThrottlingMS) {
-        // Default throttling param was never changed, hence no need to reset it
+    // No need to reset if parameter at initial state
+    if (!defaultAutoMergerThrottlingMS)
         return;
-    }
-
-    st.forEachConfigServer((conn) => {
-        assert.commandWorked(conn.adminCommand(
-            {setParameter: 1, autoMergerThrottlingMS: defaultAutoMergerThrottlingMS}));
-    });
+    updateBalancerParameter(st, "autoMergerThrottlingMS", defaultAutoMergerThrottlingMS);
+    defaultAutoMergerThrottlingMS = null;
 }
 
-function setBalanceRoundInterval(st, valueInMs) {
-    configureFailPointForRS(
-        st.configRS.nodes, "overrideBalanceRoundInterval", {intervalMs: valueInMs}, "alwaysOn");
+let defaultBalancerMigrationsThrottlingMs = null;
+function setBalancerMigrationsThrottling(st, valueInMS) {
+    const prevValue = updateBalancerParameter(st, "balancerMigrationsThrottlingMs", valueInMS);
+    // Cache only the initial [default] parameter value
+    if (!defaultBalancerMigrationsThrottlingMs)
+        defaultBalancerMigrationsThrottlingMs = prevValue;
 }
 
-function resetBalanceRoundInterval(st) {
-    configureFailPointForRS(st.configRS.nodes, "overrideBalanceRoundInterval", {}, "off");
+function resetBalancerMigrationsThrottling(st) {
+    // No need to reset if parameter at initial state
+    if (!defaultBalancerMigrationsThrottlingMs)
+        return;
+    updateBalancerParameter(
+        st, "balancerMigrationsThrottlingMs", defaultBalancerMigrationsThrottlingMs);
+    defaultBalancerMigrationsThrottlingMs = null;
 }
 
 function assertExpectedChunks(configDB, coll, expectedChunksPerShard) {
@@ -218,7 +232,8 @@ function mergeAllChunksOnShardTest(st, testDB) {
 
 function mergeAllChunksWithMaxNumberOfChunksTest(st, testDB) {
     // Skip this test if running in a suite with stepdowns
-    if (typeof ContinuousStepdown !== 'undefined') {
+    if (TestData.runningWithConfigStepdowns) {
+        jsTest.log(`Skipping mergeAllChunksWithMaxNumberOfChunksTest`);
         return;
     }
 
@@ -312,8 +327,8 @@ function mergeAllChunksOnShardConsideringJumboFlagTest(st, testDB) {
 
     // Set jumbo flag to a couple of chunks
     // Setting a chunks as jumbo must prevent it from being merged
-    setJumboFlag(configDB, coll, {min: {x: 3}});
-    setJumboFlag(configDB, coll, {min: {x: 8}});
+    setJumboFlag(st.s, coll, {min: {x: 3}});
+    setJumboFlag(st.s, coll, {min: {x: 8}});
 
     // Try to merge all mergeable chunks on shard0
     assert.commandWorked(
@@ -363,8 +378,8 @@ function balancerTriggersAutomergerWhenIsEnabledTest(st, testDB) {
         setOnCurrentShardSince(st.s, coll, {}, now, -historyWindowInSeconds - 1000);
     });
 
-    // Override balancer round interval and merge throttling to speed up the test
-    setBalanceRoundInterval(st, 100 /* ms */);
+    // Update balancer migration/merge throttling to speed up the test
+    setBalancerMigrationsThrottling(st, 100);
     setBalancerMergeThrottling(st, 0);
 
     // Enable the AutoMerger
@@ -386,7 +401,7 @@ function balancerTriggersAutomergerWhenIsEnabledTest(st, testDB) {
 
 function testConfigurableAutoMergerIntervalSecs(st, testDB) {
     // Override default configuration to speed up the test
-    setBalanceRoundInterval(st, 100 /* ms */);
+    setBalancerMigrationsThrottling(st, 100);
     setBalancerMergeThrottling(st, 0);
     setHistoryWindowInSecs(st, 0 /* seconds */);
 
@@ -520,7 +535,7 @@ function executeTestCase(testFunc) {
     // Teardown: stop the balancer, reset configuration and drop db
     st.stopBalancer();
     resetHistoryWindowInSecs(st);
-    resetBalanceRoundInterval(st);
+    resetBalancerMigrationsThrottling(st);
     resetBalancerMergeThrottling(st);
 
     // The auto-merger considers all collections, drop db so that collections from previous test
@@ -537,4 +552,3 @@ executeTestCase(testConfigurableAutoMergerIntervalSecs);
 executeTestCase(testMaxTimeProcessingChunksMSParameter);
 
 st.stop();
-})();
