@@ -1,9 +1,6 @@
 // Tests that the $merge aggregation stage is resilient to chunk migrations in both the source and
 // output collection during execution.
-(function() {
-'use strict';
-
-load("jstests/aggregation/extras/merge_helpers.js");  // For withEachMergeMode.
+import {withEachMergeMode} from "jstests/aggregation/extras/merge_helpers.js";
 
 const st = new ShardingTest({shards: 2, rs: {nodes: 1}});
 
@@ -153,5 +150,4 @@ withEachMergeMode(({whenMatchedMode, whenNotMatchedMode}) => {
     runMergeWithMode(whenMatchedMode, whenNotMatchedMode, targetColl);
 });
 
-st.stop();
-})();
+st.stop();
\ No newline at end of file
