@@ -2,24 +2,24 @@
  * Tests that serverStatus includes a migration status when called on the source shard of an active
  * migration.
  *
- * @tags: [requires_fcv_63]
+ * @tags: [requires_fcv_73]
  */
 
-load('./jstests/libs/chunk_manipulation_util.js');
-load("jstests/libs/feature_flag_util.js");
+import {
+    moveChunkParallel,
+    moveChunkStepNames,
+    pauseMoveChunkAtStep,
+    unpauseMoveChunkAtStep,
+    waitForMoveChunkStep,
+} from "jstests/libs/chunk_manipulation_util.js";
 
-(function() {
-'use strict';
-
-var staticMongod = MongoRunner.runMongod({});  // For startParallelOps.
+var staticMongod = MongoRunner.runMongod({});
 
 var st = new ShardingTest({shards: 2, mongos: 1});
 
 var mongos = st.s0;
 var admin = mongos.getDB("admin");
 var coll = mongos.getCollection("migration_server_status.coll");
-const usingSetClusterParameter =
-    FeatureFlagUtil.isPresentAndEnabled(st.config, "ClusterCardinalityParameter");
 
 assert.commandWorked(
     admin.runCommand({enableSharding: coll.getDB() + "", primaryShard: st.shard0.shardName}));
@@ -74,12 +74,8 @@ var assertSessionMigrationStatusSource = function(
     if (expectedEntriesSkippedLowerBound == null) {
         assert(migrationResult.sessionOplogEntriesSkippedSoFarLowerBound);
     } else {
-        // Running DDL operations increases this number by 1
-        let actualEntriesSkippedLowerBound = usingSetClusterParameter
-            ? expectedEntriesSkippedLowerBound + 1
-            : expectedEntriesSkippedLowerBound;
         assert.eq(migrationResult.sessionOplogEntriesSkippedSoFarLowerBound,
-                  actualEntriesSkippedLowerBound);
+                  expectedEntriesSkippedLowerBound);
     }
 };
 
@@ -173,4 +169,3 @@ assert(!shard1ServerStatus.sharding.migrations);
 
 st.stop();
 MongoRunner.stopMongod(staticMongod);
-})();
