@@ -6,8 +6,6 @@
  * In practice, drivers will send "helloOk: true" in the initial handshake when
  * opening a connection to the database.
  */
-(function() {
-"use strict";
 const st = new ShardingTest({shards: 1, mongos: 1});
 const mongos = st.s;
 
@@ -22,5 +20,4 @@ res = assert.commandWorked(mongos.adminCommand({isMaster: 1, helloOk: true}));
 assert.eq("boolean", typeof res.helloOk, "helloOk field is not a boolean" + tojson(res));
 assert.eq(res.helloOk, true);
 
-st.stop();
-})();
\ No newline at end of file
+st.stop();
\ No newline at end of file
