@@ -3,8 +3,8 @@
 //  multiversion_incompatible
 //  ]
 
-load("jstests/libs/noop_write_commands.js");
-load("jstests/libs/write_concern_util.js");
+import {getNoopWriteCommands} from "jstests/libs/noop_write_commands.js";
+import {assertWriteConcernError} from "jstests/libs/write_concern_util.js";
 
 // Create a shard with 3 nodes and stop one of the secondaries. This will allow majority write
 // concern to be met, but w: 3 will always time out.
