@@ -1,7 +1,4 @@
-load("jstests/sharding/move_chunk_with_session_helper.js");
-
-(function() {
-"use strict";
+import {testMoveChunkWithSession} from "jstests/sharding/move_chunk_with_session_helper.js";
 
 // Prevent unnecessary elections in the first shard replica set. Shard 'rs1' shard will need its
 // secondary to get elected, so we don't give it a zero priority.
@@ -12,8 +9,8 @@ var st = new ShardingTest({
         rs1: {nodes: [{rsConfig: {}}, {rsConfig: {}}]}
     }
 });
-assert.commandWorked(st.s.adminCommand({enableSharding: 'test'}));
-st.ensurePrimaryShard('test', st.shard0.shardName);
+assert.commandWorked(
+    st.s.adminCommand({enableSharding: 'test', primaryShard: st.shard0.shardName}));
 
 var coll = 'delete';
 var cmd = {
@@ -45,4 +42,3 @@ var checkDocuments = function(coll) {
 testMoveChunkWithSession(st, coll, cmd, setup, checkRetryResult, checkDocuments);
 
 st.stop();
-})();
