@@ -7,8 +7,7 @@ var admin = st.s0.getDB('admin');
 var testDb = 'test';
 var testNs = 'test.foo';
 
-assert.commandWorked(admin.runCommand({enableSharding: testDb}));
-st.ensurePrimaryShard(testDb, st.shard0.name);
+assert.commandWorked(admin.runCommand({enableSharding: testDb, primaryShard: st.shard0.name}));
 assert.commandWorked(admin.runCommand({shardCollection: testNs, key: {_id: 1}}));
 var curShardIndex = 0;
 
