@@ -2,10 +2,7 @@
 // Tests that the changelog entry for moveChunk.commit contains stats on the migration.
 //
 
-(function() {
-'use strict';
-
-load("jstests/sharding/libs/find_chunks_util.js");
+import {findChunksUtil} from "jstests/sharding/libs/find_chunks_util.js";
 
 var st = new ShardingTest({mongos: 1, shards: 2});
 var kDbName = 'db';
@@ -14,8 +11,7 @@ var mongos = st.s0;
 var shard0 = st.shard0.shardName;
 var shard1 = st.shard1.shardName;
 
-assert.commandWorked(mongos.adminCommand({enableSharding: kDbName}));
-st.ensurePrimaryShard(kDbName, shard0);
+assert.commandWorked(mongos.adminCommand({enableSharding: kDbName, primaryShard: shard0}));
 
 function assertCountsInChangelog() {
     let changeLog = st.s.getDB('config').changelog.find({what: 'moveChunk.commit'}).toArray();
@@ -38,5 +34,4 @@ assertCountsInChangelog();
 
 mongos.getDB(kDbName).fooHashed.drop();
 
-st.stop();
-})();
\ No newline at end of file
+st.stop();
\ No newline at end of file
