@@ -3,8 +3,8 @@
  * TransientTransactionError label) if a collection or database placement changes have occurred
  * later than the transaction data snapshot timestamp.
  */
-
-load("jstests/libs/fail_point_util.js");
+import {configureFailPoint} from "jstests/libs/fail_point_util.js";
+import {FeatureFlagUtil} from "jstests/libs/feature_flag_util.js";
 
 const st = new ShardingTest({mongos: 1, shards: 2});
 
@@ -99,8 +99,72 @@ const st = new ShardingTest({mongos: 1, shards: 2});
         assert.contains("TransientTransactionError", err.errorLabels, tojson(err));
     }
 
-    runTest('majority');
-    runTest('snapshot');
+    // These tests only make sense with untracked collections since movePrimary does not affect
+    // tracked collections
+    const isTrackUnshardedUponCreationEnabled = FeatureFlagUtil.isPresentAndEnabled(
+        st.s.getDB('admin'), "TrackUnshardedCollectionsUponCreation");
+    if (!isTrackUnshardedUponCreationEnabled) {
+        runTest('majority');
+
+        runTest('snapshot');
+    }
+}
+
+// Test transaction with concurrent move collection.
+{
+    const dbName1 = 'test';
+    const dbName2 = 'test2';
+    const collName1 = 'foo';
+    const collName2 = 'foo';
+    const ns2 = dbName2 + '.' + collName2;
+
+    function runTest(readConcernLevel) {
+        st.getDB(dbName1).dropDatabase();
+        st.getDB(dbName2).dropDatabase();
+        st.adminCommand({enableSharding: dbName1, primaryShard: st.shard0.shardName});
+        st.adminCommand({enableSharding: dbName2, primaryShard: st.shard1.shardName});
+
+        const coll1 = st.getDB(dbName1)[collName1];
+        coll1.insert({x: 1, c: 0});
+
+        const coll2 = st.getDB(dbName2)[collName2];
+        coll2.insert({x: 2, c: 0});
+
+        // Start a multi-document transaction. Execute one statement that will target shard0.
+        let session = st.s.startSession();
+        session.startTransaction({readConcern: {level: readConcernLevel}});
+        assert.eq(1, session.getDatabase(dbName1)[collName1].find().itcount());
+
+        // Run moveCollection to move dbName2.collName2 from shard1 to shard0.
+        assert.commandWorked(
+            st.s.adminCommand({moveCollection: ns2, toShard: st.shard0.shardName}));
+
+        // Make sure the router has fresh routing info to avoid causing the transaction to fail due
+        // to StaleConfig.
+        assert.eq(1, coll2.find().itcount());
+
+        // Execute a second statement, now on dbName2. With majority read concern, this is
+        // equivalent to the moveChunk scenario. However, since resharding changes the uuid, in the
+        // case of snapshot read concern, the router will fail to find chunk history for the
+        // timestamp before even sending the request to the shard.
+        const expectedError = readConcernLevel == 'snapshot' ? ErrorCodes.StaleChunkHistory
+                                                             : ErrorCodes.MigrationConflict;
+        let err = assert.throwsWithCode(() => {
+            session.getDatabase(dbName2)[collName2].find().itcount();
+        }, expectedError);
+
+        assert.contains("TransientTransactionError", err.errorLabels, tojson(err));
+    }
+
+    // These tests only make sense with tracked, unsharded collections since moveCollection does not
+    // affect untracked collections
+    const isTrackUnshardedUponCreationEnabled = FeatureFlagUtil.isPresentAndEnabled(
+        st.s.getDB('admin'), "TrackUnshardedCollectionsUponCreation");
+    if (isTrackUnshardedUponCreationEnabled) {
+        runTest('majority');
+
+        runTest('snapshot');
+    }
 }
 
 // Tests transactions with concurrent DDL operations.
@@ -164,6 +228,7 @@ const st = new ShardingTest({mongos: 1, shards: 2});
             assert.eq(2, coll3.find().itcount());
 
             // Now read coll3 within the transaction and expect to get a conflict.
+            // TODO (SERVER-86920): remove WriteConflict as an accepted error.
             let err = assert.throwsWithCode(() => {
                 if (command === 'find') {
                     sessionColl3.find().itcount();
@@ -321,6 +386,9 @@ const st = new ShardingTest({mongos: 1, shards: 2});
             assert.contains("TransientTransactionError", err.errorLabels, tojson(err));
 
             session.abortTransaction();
+            // Transaction abort can race 'abort' before 'new transaction' command
+            // on participant, so ensure there are no orphan transactions on participant.
+            st.shard1.getDB('config').runCommand({killSessions: [session.id]});
         }
 
         readConcerns.forEach((readConcern) => commands.forEach((command) => {
