@@ -1,11 +1,8 @@
 // This tests sharding an existing collection that both shards are aware of (SERVER-2828)
-(function() {
-'use strict';
-
 var st = new ShardingTest({shards: 2, mongos: 2});
 
-assert.commandWorked(st.s0.adminCommand({enablesharding: "test"}));
-st.ensurePrimaryShard('test', st.shard1.shardName);
+assert.commandWorked(
+    st.s0.adminCommand({enablesharding: "test", primaryShard: st.shard1.shardName}));
 
 assert.commandWorked(st.s0.adminCommand({shardcollection: "test.foo", key: {num: 1}}));
 
@@ -25,5 +22,4 @@ assert.commandWorked(st.s1.getDB("admin").runCommand({
 assert.eq(1, st.s0.getDB('test').existing.count({_id: 1}));
 assert.eq(1, st.s1.getDB('test').existing.count({_id: 1}));
 
-st.stop();
-})();
+st.stop();
\ No newline at end of file
