@@ -6,10 +6,7 @@
  *    requires_fcv_52
  * ]
  */
-(function() {
-"use strict";
-
-load("jstests/libs/fail_point_util.js");
+import {configureFailPoint} from "jstests/libs/fail_point_util.js";
 
 const st = new ShardingTest({mongos: 1, shards: 2, rs: {nodes: 1}});
 const dbName = "test";
@@ -17,8 +14,8 @@ const collName = "update_orphan_shard_key";
 const collection = st.s.getDB(dbName).getCollection(collName);
 
 // Create a sharded collection with two chunks on shard0, split at the key {x: -1}.
-assert.commandWorked(st.s.adminCommand({enableSharding: dbName}));
-assert.commandWorked(st.s.adminCommand({movePrimary: dbName, to: st.shard0.shardName}));
+assert.commandWorked(
+    st.s.adminCommand({enableSharding: dbName, primaryShard: st.shard0.shardName}));
 assert.commandWorked(st.s.adminCommand({shardCollection: collection.getFullName(), key: {x: 1}}));
 assert.commandWorked(st.s.adminCommand({split: collection.getFullName(), middle: {x: -1}}));
 
@@ -59,4 +56,3 @@ assert.soon(() => {
 assert.doesNotThrow(() => collection.aggregate([{$out: "output"}]));
 
 st.stop();
-})();
