@@ -1,9 +1,8 @@
 // SERVER-32255 This test ensures a node started with --shardsvr and added to a replica set receives
 // UUIDs upon re-initiation.
 // @tags: [multiversion_incompatible]
-(function() {
-"use strict";
-load("jstests/libs/check_uuids.js");
+import {checkCollectionUUIDs} from "jstests/libs/check_uuids.js";
+
 let st = new ShardingTest({shards: 1, rs: {nodes: 1}, mongos: 1});
 let mongos = st.s;
 let rs = st.rs0;
@@ -21,5 +20,4 @@ let secondaryAdminDB = newNode.getDB("admin");
 
 // Ensure the new node has UUIDs for all its collections.
 checkCollectionUUIDs(secondaryAdminDB);
-st.stop();
-})();
+st.stop();
\ No newline at end of file
