@@ -1,12 +1,8 @@
 // This test fails when run with authentication because benchRun with auth is broken: SERVER-6388
-(function() {
-"use strict";
-
 var numShards = 3;
 var s = new ShardingTest({name: "parallel", shards: numShards, mongos: 2});
 
-s.adminCommand({enablesharding: "test"});
-s.ensurePrimaryShard('test', s.shard1.shardName);
+s.adminCommand({enablesharding: "test", primaryShard: s.shard1.shardName});
 s.adminCommand({shardcollection: "test.foo", key: {_id: 1}});
 
 var db = s.getDB("test");
@@ -29,14 +25,14 @@ assert.commandWorked(bulk.execute());
 
 var doCommand = function(dbname, cmd) {
     x = benchRun({
-        ops: [{op: "findOne", ns: dbname + ".$cmd", query: cmd}],
+        ops: [{op: "findOne", ns: dbname + ".$cmd", query: cmd, readCmd: true}],
         host: db.getMongo().host,
         parallel: 2,
         seconds: 2
     });
     printjson(x);
     x = benchRun({
-        ops: [{op: "findOne", ns: dbname + ".$cmd", query: cmd}],
+        ops: [{op: "findOne", ns: dbname + ".$cmd", query: cmd, readCmd: true}],
         host: s._mongos[1].host,
         parallel: 2,
         seconds: 2
@@ -51,5 +47,4 @@ var x = s.getDB("config").stats();
 assert(x.ok, tojson(x));
 printjson(x);
 
-s.stop();
-}());
+s.stop();
\ No newline at end of file
