@@ -2,13 +2,14 @@
  * Ensures that if the primary config server is blackholed from the point of view of mongos, CRUD
  * and read-only config operations continue to work.
  *
- * @tags: [config_shard_incompatible]
+ * @tags: [
+ *    config_shard_incompatible,
+ *    temp_disabled_embedded_router_mongo_bridge,
+ * ]
  */
 
-(function() {
-'use strict';
-
-load('jstests/replsets/rslib.js');
+import {FixtureHelpers} from "jstests/libs/fixture_helpers.js";
+import {reconfig} from "jstests/replsets/rslib.js";
 
 // Checking index consistency involves talking to the primary config server which is blackholed from
 // the mongos in this test.
@@ -85,6 +86,7 @@ assert.throws(function() {
 
 // With secondary read pref config server reads should work
 st.s.setReadPref('secondary');
+FixtureHelpers.awaitReplication(configDB);
 assert.lt(0, configDB.chunks.find().itcount());
 assert.lt(0, configDB.chunks.find().count());
 assert.lt(0, configDB.chunks.aggregate().itcount());
@@ -93,15 +95,4 @@ jsTest.log('Remove network partition before tearing down');
 configPrimary.discardMessagesFrom(st.s, 0.0);
 st.s.discardMessagesFrom(configPrimary, 0.0);
 
-// The mongos may have some connections in its pool that were created during the partition. They
-// are waiting on hello responses that will never come and will eventually time out. Any operation
-// that is waiting for a connection to the config primary when that happens will have its request
-// failed, and this may fail the operation and the test. Operations that need a connection may
-// even be forced to wait for one of those doomed connections, which virtually guarantees that
-// it will fail as described above. To avoid that, drop the connections to the config primary.
-// For good measure, we'll also drop connections in the other direction in case they also exist.
-assert.commandWorked(st.s.adminCommand({dropConnections: 1, hostAndPort: [configPrimary.host]}));
-assert.commandWorked(configPrimary.adminCommand({dropConnections: 1, hostAndPort: [st.s.host]}));
-
 st.stop();
-}());
