@@ -1,9 +1,6 @@
 // Tests that an aggregation error which occurs on a sharded collection will send an error message
 // containing the host and port of the shard where the error occurred.
-(function() {
-"use strict";
-
-load("jstests/aggregation/extras/utils.js");  // For assertErrCodeAndErrMsgContains.
+import {assertErrCodeAndErrMsgContains} from "jstests/aggregation/extras/utils.js";
 
 const st = new ShardingTest({shards: 2});
 
@@ -11,8 +8,8 @@ const mongosDb = st.s.getDB(jsTestName());
 const coll = mongosDb.getCollection("foo");
 
 // Enable sharding on the test DB and ensure its primary is shard 0.
-assert.commandWorked(mongosDb.adminCommand({enableSharding: mongosDb.getName()}));
-st.ensurePrimaryShard(mongosDb.getName(), st.rs0.getURL());
+assert.commandWorked(
+    mongosDb.adminCommand({enableSharding: mongosDb.getName(), primaryShard: st.rs0.getURL()}));
 
 // Shard the collection.
 coll.drop();
@@ -30,5 +27,4 @@ const divideByZeroErrorCodes = [16608, 4848401, ErrorCodes.BadValue];
 
 assertErrCodeAndErrMsgContains(coll, pipe, divideByZeroErrorCodes, st.rs1.getPrimary().host);
 
-st.stop();
-}());
+st.stop();
\ No newline at end of file
