@@ -20,10 +20,15 @@
  *   requires_sharding,
  *   requires_spawning_own_processes,
  *   requires_profiling,
+ *   # This test uses a new $_internalSplitPipeline syntax introduced in 8.0.
+ *   requires_fcv_80,
  * ]
  */
-(function() {
-load("jstests/libs/profiler.js");  // For profilerHas*OrThrow helper functions.
+import {
+    profilerHasAtLeastOneAtMostNumMatchingEntriesOrThrow,
+    profilerHasSingleMatchingEntryOrThrow,
+    profilerHasZeroMatchingEntriesOrThrow,
+} from "jstests/libs/profiler.js";
 
 const st = new ShardingTest({shards: 2, mongos: 2});
 
@@ -35,7 +40,8 @@ const mongosForMove = st.s1;
 const mongosDB = mongosForAgg.getDB(jsTestName());
 const mongosColl = mongosDB.test;
 
-const shard0DB = primaryShardDB = st.shard0.getDB(jsTestName());
+const shard0DB = st.shard0.getDB(jsTestName());
+const primaryShardDB = shard0DB;
 const shard1DB = st.shard1.getDB(jsTestName());
 
 // Turn off best-effort recipient metadata refresh post-migration commit on both shards because
@@ -52,8 +58,8 @@ assert.commandWorked(mongosForAgg.getDB('admin').runCommand(
 assert.commandWorked(mongosDB.dropDatabase());
 
 // Enable sharding on the test DB and ensure its primary is st.shard0.shardName.
-assert.commandWorked(mongosDB.adminCommand({enableSharding: mongosDB.getName()}));
-st.ensurePrimaryShard(mongosDB.getName(), st.shard0.shardName);
+assert.commandWorked(
+    mongosDB.adminCommand({enableSharding: mongosDB.getName(), primaryShard: st.shard0.shardName}));
 
 // Shard the test collection on _id.
 assert.commandWorked(
@@ -81,8 +87,9 @@ const shardExceptions = [
     ErrorCodes.StaleEpoch,
 ];
 
-// Create an $_internalSplitPipeline stage that forces the merge to occur on the Primary shard.
-const forcePrimaryMerge = [{$_internalSplitPipeline: {mergeType: "primaryShard"}}];
+// Create an $_internalSplitPipeline stage that forces the merge to occur on the same shard.
+let forceOwningShardMerge =
+    [{$_internalSplitPipeline: {mergeType: {"specificShard": st.shard0.shardName}}}];
 
 function runAggShardTargetTest({splitPoint}) {
     // Ensure that both mongoS have up-to-date caches, and enable the profiler on both shards.
@@ -195,7 +202,7 @@ function runAggShardTargetTest({splitPoint}) {
     assert.eq(mongosColl
                   .aggregate([{$match: {_id: {$gte: -150, $lte: -50}}}]
                                  .concat(splitPoint)
-                                 .concat(forcePrimaryMerge),
+                                 .concat(forceOwningShardMerge),
                              {comment: testName})
                   .itcount(),
               2);
@@ -282,7 +289,7 @@ function runAggShardTargetTest({splitPoint}) {
     assert.eq(mongosColl
                   .aggregate([{$match: {_id: {$gte: -150, $lte: -50}}}]
                                  .concat(splitPoint)
-                                 .concat(forcePrimaryMerge),
+                                 .concat(forceOwningShardMerge),
                              {comment: testName})
                   .itcount(),
               2);
@@ -380,5 +387,4 @@ runAggShardTargetTest({
         ]
     });
 
-st.stop();
-})();
+st.stop();
\ No newline at end of file
