@@ -13,24 +13,28 @@
  * applicable.
  *
  * This test requires replica set configuration and user credentials to persist across a restart.
- * @tags: [requires_persistence, uses_transactions, uses_prepare_transaction, requires_fcv_60]
+ * @tags: [
+ *   requires_fcv_70,
+ *   requires_persistence,
+ *   temp_disabled_embedded_router_metrics,
+ *   uses_prepare_transaction,
+ *   uses_transactions,
+ * ]
  */
 
 // Restarts cause issues with authentication for awaiting replication.
 TestData.skipAwaitingReplicationOnShardsBeforeCheckingUUIDs = true;
 // Restarts shard nodes with no keyFile.
 TestData.skipCheckOrphans = true;
+TestData.skipCheckShardFilteringMetadata = true;
 
-(function() {
-"use strict";
-
-load("jstests/libs/fixture_helpers.js");  // For FixtureHelpers.
-load("jstests/libs/namespace_utils.js");  // For getCollectionNameFromFullNamespace.
+import {FixtureHelpers} from "jstests/libs/fixture_helpers.js";
+import {getCollectionNameFromFullNamespace} from "jstests/libs/namespace_utils.js";
 
 // Replica set nodes started with --shardsvr do not enable key generation until they are added
 // to a sharded cluster and reject commands with gossiped clusterTime from users without the
 // advanceClusterTime privilege. This causes ShardingTest setup to fail because the shell
-// briefly authenticates as __system and recieves clusterTime metadata then will fail trying to
+// briefly authenticates as __system and receives clusterTime metadata then will fail trying to
 // gossip that time later in setup.
 //
 
@@ -41,7 +45,11 @@ const stParams = {
     name: jsTestName(),
     keyFile: key,
     shards: 3,
-    rs: {nodes: 1, setParameter: {internalQueryExecYieldIterations: 1}}
+    rs: {nodes: 1, setParameter: {internalQueryExecYieldIterations: 1}},
+    other: {
+        mongosOptions:
+            {setParameter: {'failpoint.skipClusterParameterRefresh': "{'mode':'alwaysOn'}"}}
+    }
 };
 
 // Create a new sharded cluster for testing. We set the internalQueryExecYieldIterations
@@ -54,7 +62,6 @@ const st = new ShardingTest(stParams);
 // $currentOp behaviour.
 let shardConn = st.rs0.getPrimary();
 let mongosConn = st.s;
-let shardRS = st.rs0;
 
 let clusterTestDB = mongosConn.getDB(jsTestName());
 let clusterAdminDB = mongosConn.getDB("admin");
@@ -62,7 +69,7 @@ shardConn.waitForClusterTime(60);
 let shardTestDB = shardConn.getDB(jsTestName());
 let shardAdminDB = shardConn.getDB("admin");
 
-function createUsers(conn) {
+function createUsers(conn, grantDirectShardOperationsRole) {
     let adminDB = conn.getDB("admin");
 
     // Create an admin user, one user with the inprog privilege, and one without.
@@ -75,26 +82,41 @@ function createUsers(conn) {
         privileges: [{resource: {cluster: true}, actions: ["inprog"]}]
     }));
 
-    assert.commandWorked(adminDB.runCommand(
-        {createUser: "user_inprog", pwd: "pwd", roles: ["readWriteAnyDatabase", "role_inprog"]}));
+    let rolesUserInprog = ["readWriteAnyDatabase", "role_inprog"];
+    if (grantDirectShardOperationsRole)
+        rolesUserInprog.push("directShardOperations");
+    assert.commandWorked(
+        adminDB.runCommand({createUser: "user_inprog", pwd: "pwd", roles: rolesUserInprog}));
 
-    assert.commandWorked(adminDB.runCommand(
-        {createUser: "user_no_inprog", pwd: "pwd", roles: ["readWriteAnyDatabase"]}));
+    let rolesUserNoInprog = ["readWriteAnyDatabase"];
+    if (grantDirectShardOperationsRole)
+        rolesUserNoInprog.push("directShardOperations");
+
+    assert.commandWorked(
+        adminDB.runCommand({createUser: "user_no_inprog", pwd: "pwd", roles: rolesUserNoInprog}));
 }
 
 // Create necessary users at both cluster and shard-local level.
-createUsers(shardConn);
-createUsers(mongosConn);
+if (!TestData.configShard) {
+    // In config shard mode, the first shard is the config server, so creating the users via mongos
+    // below will also create them on the shard.
+    createUsers(shardConn, /* grantDirectShardOperationsRole */ true);
+    createUsers(mongosConn, /* grantDirectShardOperationsRole */ false);
+} else {
+    // Since this is making the user on both the config server and the shard, grant it direct shard
+    // operations role.
+    createUsers(mongosConn, /* grantDirectShardOperationsRole */ true);
+}
 
 // Create a test database and some dummy data on rs0.
 assert(clusterAdminDB.auth("admin", "pwd"));
 
+assert.commandWorked(clusterTestDB.adminCommand(
+    {enableSharding: clusterTestDB.getName(), primaryShard: st.shard0.shardName}));
 for (let i = 0; i < 5; i++) {
     assert.commandWorked(clusterTestDB.test.insert({_id: i, a: i}));
 }
 
-st.ensurePrimaryShard(clusterTestDB.getName(), shardRS.name);
-
 // Restarts a replset with a different set of parameters. Explicitly set the keyFile to null,
 // since if ReplSetTest#stopSet sees a keyFile property, it attempts to auth before dbhash
 // checks.
@@ -117,7 +139,6 @@ function restartCluster(st, newOpts) {
     // Re-link the cluster components.
     shardConn = st.rs0.getPrimary();
     mongosConn = st.s;
-    shardRS = st.rs0;
     clusterTestDB = mongosConn.getDB(jsTestName());
     clusterAdminDB = mongosConn.getDB("admin");
     shardTestDB = shardConn.getDB(jsTestName());
@@ -177,7 +198,7 @@ function assertCurrentOpHasSingleMatchingEntry({conn, currentOpAggFilter, curOpS
     return curOpResult[0];
 }
 
-function waitForParallelShell({conn, username, password, awaitShell}) {
+function waitForParallelShell({username, password, awaitShell}) {
     runCommandOnAllPrimaries({
         dbName: "admin",
         username: username,
@@ -415,16 +436,16 @@ function runCommonTests(conn, curOpSpec) {
         [{$currentOp: {idleConnections: true, allUsers: false}}, {$match: {desc: {$eq: "test"}}}];
 
     if (isRemoteShardCurOp) {
-        assert.docEq(explainPlan.splitPipeline.shardsPart, expectedStages);
+        assert.docEq(expectedStages, explainPlan.splitPipeline.shardsPart);
         for (let i = 0; i < stParams.shards; i++) {
-            let shardName = st["rs" + i].name;
-            assert.docEq(explainPlan.shards[shardName].stages, expectedStages);
+            let shardName = st["shard" + i].shardName;
+            assert.docEq(expectedStages, explainPlan.shards[shardName].stages);
         }
     } else if (isLocalMongosCurOp) {
         expectedStages[0].$currentOp.localOps = true;
-        assert.docEq(explainPlan.mongos.stages, expectedStages);
+        assert.docEq(expectedStages, explainPlan.mongos.stages);
     } else {
-        assert.docEq(explainPlan.stages, expectedStages);
+        assert.docEq(expectedStages, explainPlan.stages);
     }
 
     // Test that a user with the inprog privilege can run getMore on a $currentOp aggregation
@@ -474,18 +495,19 @@ assert.commandFailedWithCode(clusterAdminDB.currentOp({$ownOps: true}), ErrorCod
 assert(clusterAdminDB.logout());
 assert(clusterAdminDB.auth("user_inprog", "pwd"));
 
-assert.eq(clusterAdminDB
-              .aggregate([
-                  {$currentOp: {allUsers: true, idleConnections: true}},
-                  {$group: {_id: {shard: "$shard", host: "$host"}}},
-                  {$sort: {_id: 1}}
-              ])
-              .toArray(),
-          [
-              {_id: {shard: "aggregation_currentop-rs0", host: st.rs0.getPrimary().host}},
-              {_id: {shard: "aggregation_currentop-rs1", host: st.rs1.getPrimary().host}},
-              {_id: {shard: "aggregation_currentop-rs2", host: st.rs2.getPrimary().host}}
-          ]);
+const expectedOutput = [
+    {_id: {shard: st.shard0.shardName, host: st.rs0.getPrimary().host}},
+    {_id: {shard: st.shard1.shardName, host: st.rs1.getPrimary().host}},
+    {_id: {shard: st.shard2.shardName, host: st.rs2.getPrimary().host}}
+];
+assert.sameMembers(clusterAdminDB
+                       .aggregate([
+                           {$currentOp: {allUsers: true, idleConnections: true}},
+                           {$group: {_id: {shard: "$shard", host: "$host"}}},
+                           {$sort: {_id: 1}}
+                       ])
+                       .toArray(),
+                   expectedOutput);
 
 // Test that a $currentOp pipeline with {localOps:true} returns operations from the mongoS
 // itself rather than the shards.
@@ -866,12 +888,17 @@ runNoAuthTests(shardConn);
 runNoAuthTests(mongosConn);
 runNoAuthTests(mongosConn, {localOps: true});
 
+if (jsTestOptions().useAutoBootstrapProcedure) {  // TODO: SERVER-80318 Delete tests below
+    st.stop();
+    quit();
+}
+
 //
 // Replset specific tests.
 //
 
 // Take the replica set out of the cluster.
-shardConn = restartReplSet(st.rs0, {shardsvr: null});
+shardConn = restartReplSet(st.rs1, {shardsvr: null});
 shardTestDB = shardConn.getDB(jsTestName());
 shardAdminDB = shardConn.getDB("admin");
 
@@ -932,6 +959,5 @@ assert.commandWorked(shardAdminDB.killOp(op.opid));
 awaitShell();
 
 // Add the shard back into the replset so that it can be validated by st.stop().
-shardConn = restartReplSet(st.rs0, {shardsvr: ""});
+shardConn = restartReplSet(st.rs1, {shardsvr: ""});
 st.stop();
-})();
