@@ -1,9 +1,7 @@
-
 /**
  * Test shard targeting for queries on a collection with a non-simple collation and a hashed shard
  * key.
  */
-(function() {
 const st = new ShardingTest({mongos: 1, config: 1, shards: 2, rs: {nodes: 1}});
 
 function shardCollectionWithSplitsAndMoves(
@@ -11,10 +9,10 @@ function shardCollectionWithSplitsAndMoves(
     const collection = st.s.getCollection(ns);
     const db = collection.getDB();
 
-    assert.commandWorked(db.runCommand({create: collection.getName(), collation: collation}));
+    assert.commandWorked(
+        st.s.adminCommand({enableSharding: db.getName(), primaryShard: st.shard0.name}));
 
-    st.ensurePrimaryShard(db.getName(), st.shard0.shardName);
-    assert.commandWorked(st.s.adminCommand({enableSharding: db.getName()}));
+    assert.commandWorked(db.runCommand({create: collection.getName(), collation: collation}));
 
     assert.commandWorked(st.s.adminCommand({
         shardCollection: collection.getFullName(),
@@ -114,9 +112,6 @@ function findQueryWithCollation(collection, query, collation) {
     assert.commandWorked(
         db.runCommand({create: collection.getName(), collation: {locale: "en", strength: 2}}));
 
-    st.ensurePrimaryShard(db.getName(), st.shard0.shardName);
-    assert.commandWorked(st.s.adminCommand({enableSharding: db.getName()}));
-
     const res = assert.commandFailedWithCode(st.s.adminCommand({
         shardCollection: collection.getFullName(),
         key: {_id: 1},
@@ -410,4 +405,3 @@ function findQueryWithCollation(collection, query, collation) {
 }
 
 st.stop();
-})();
