@@ -3,9 +3,11 @@
  * set, and does not cause the mongoS floating point failure described in SERVER-30084.
  *
  * Requires no shards so there can't be a config shard.
- * @tags: [config_shard_incompatible]
+ * @tags: [
+ *   config_shard_incompatible,
+ *   temp_disabled_embedded_router_uncategorized,
+ * ]
  */
-(function() {
 const st = new ShardingTest({shards: 0});
 
 const adminDB = st.s.getDB("admin");
@@ -16,5 +18,4 @@ assert.commandWorked(adminDB.currentOp());
 assert.eq(adminDB.aggregate([{$currentOp: {}}]).itcount(), 0);
 assert.eq(adminDB.currentOp().inprog.length, 0);
 
-st.stop();
-})();
+st.stop();
\ No newline at end of file
