@@ -4,9 +4,6 @@
  * Always run on a fully upgraded cluster, so that {$meta: "sortKey"} projections use the newest
  * sort key format.
  */
-(function() {
-"use strict";
-
 var cmdRes;
 var cursorId;
 
@@ -15,9 +12,10 @@ st.stopBalancer();
 
 // Set up a collection sharded by "_id" with one chunk on each of the two shards.
 var db = st.s.getDB("test");
+assert.commandWorked(
+    db.adminCommand({enableSharding: db.getName(), primaryShard: st.shard0.shardName}));
 var coll = db.getCollection("find_getmore_cmd");
 
-coll.drop();
 assert.commandWorked(coll.insert({_id: -9, a: 4, b: "foo foo"}));
 assert.commandWorked(coll.insert({_id: -5, a: 8}));
 assert.commandWorked(coll.insert({_id: -1, a: 10, b: "foo"}));
@@ -27,8 +25,6 @@ assert.commandWorked(coll.insert({_id: 9, a: 3}));
 
 assert.commandWorked(coll.createIndex({b: "text"}));
 
-assert.commandWorked(db.adminCommand({enableSharding: db.getName()}));
-st.ensurePrimaryShard(db.getName(), st.shard0.shardName);
 db.adminCommand({shardCollection: coll.getFullName(), key: {_id: 1}});
 assert.commandWorked(db.adminCommand({split: coll.getFullName(), middle: {_id: 0}}));
 assert.commandWorked(
@@ -160,5 +156,4 @@ assert.eq(cmdRes.cursor.firstBatch[3], {key: [1]});
 assert.eq(cmdRes.cursor.firstBatch[4], {key: [5]});
 assert.eq(cmdRes.cursor.firstBatch[5], {key: [9]});
 
-st.stop();
-})();
+st.stop();
\ No newline at end of file
