@@ -2,10 +2,7 @@
  * This tests all points using $near.
  */
 
-load("jstests/libs/geo_near_random.js");
-
-(function() {
-'use strict';
+import {GeoNearRandomTest} from "jstests/libs/geo_near_random.js";
 
 var testName = "geo_near_random1";
 var s = new ShardingTest({shards: 3});
@@ -14,8 +11,7 @@ var db = s.getDB("test");
 
 var test = new GeoNearRandomTest(testName, db);
 
-assert.commandWorked(s.s0.adminCommand({enablesharding: 'test'}));
-s.ensurePrimaryShard('test', s.shard1.shardName);
+assert.commandWorked(s.s0.adminCommand({enablesharding: 'test', primaryShard: s.shard1.shardName}));
 assert.commandWorked(s.s0.adminCommand({shardcollection: ('test.' + testName), key: {_id: 1}}));
 
 test.insertPts(50);
@@ -48,4 +44,3 @@ test.testPt(test.mkPt(), opts);
 test.testPt(test.mkPt(), opts);
 
 s.stop();
-})();
