@@ -1,7 +1,4 @@
 // Tests that the sort specification is obeyed when the query contains $near/$nearSphere.
-(function() {
-'use strict';
-
 const st = new ShardingTest({shards: 2});
 const db = st.getDB("test");
 const coll = db.geo_near_sort;
@@ -10,8 +7,8 @@ const caseInsensitive = {
     strength: 2
 };
 
-assert.commandWorked(st.s0.adminCommand({enableSharding: db.getName()}));
-st.ensurePrimaryShard(db.getName(), st.shard0.shardName);
+assert.commandWorked(
+    st.s0.adminCommand({enableSharding: db.getName(), primaryShard: st.shard0.shardName}));
 assert.commandWorked(st.s0.adminCommand({shardCollection: coll.getFullName(), key: {_id: 1}}));
 
 // Split the data into 2 chunks and move the chunk with _id > 0 to shard 1.
@@ -107,5 +104,4 @@ testSortOrders({geo: {$near: {$geometry: {type: "Point", coordinates: [0, 0]}}}}
 testSortOrders({geo: {$nearSphere: {$geometry: {type: "Point", coordinates: [0, 0]}}}},
                {geo: "2dsphere"});
 
-st.stop();
-})();
+st.stop();
\ No newline at end of file
