@@ -6,11 +6,6 @@
  *
  * @tags: [requires_fcv_51]
  */
-(function() {
-"use strict";
-
-load("jstests/libs/discover_topology.js");  // For findDataBearingNodes.
-
 const testName = "lookup_stale_mongos";
 const st = new ShardingTest({
     shards: 2,
@@ -37,8 +32,8 @@ const expectedResults = [
 ];
 
 // Ensure that shard0 is the primary shard.
-assert.commandWorked(mongos0DB.adminCommand({enableSharding: mongos0DB.getName()}));
-st.ensurePrimaryShard(mongos0DB.getName(), st.shard0.shardName);
+assert.commandWorked(mongos0DB.adminCommand(
+    {enableSharding: mongos0DB.getName(), primaryShard: st.shard0.shardName}));
 
 assert.commandWorked(mongos0LocalColl.insert({_id: 0, a: 1}));
 assert.commandWorked(mongos0LocalColl.insert({_id: 1, a: null}));
@@ -129,5 +124,4 @@ assert.commandWorked(mongos0LocalColl.insert({_id: 2}));
 // unsharded.
 assert.eq(mongos1LocalColl.aggregate(pipeline).toArray(), expectedResults);
 
-st.stop();
-})();
+st.stop();
\ No newline at end of file
