@@ -3,13 +3,10 @@
 // inject metadata when communication results to mongos for merging. In addition, we want to ensure
 // that other fields beginning with a '$' are not modified before returning them to customers.
 // @tags: [
-//   requires_fcv_60
+//   requires_fcv_62
 // ]
 
-(function() {
-"use strict";
-
-function runTest(coll, keyword) {
+function runTest(coll) {
     const document = ({_id: 5, x: 1, $set: {$inc: {x: 5}}});
     assert.commandWorked(coll.insert(document));
 
@@ -35,12 +32,11 @@ function runTest(coll, keyword) {
 
 const st = new ShardingTest({shards: 2});
 try {
-    assert.commandWorked(st.s0.adminCommand({enableSharding: 'test'}));
-    st.ensurePrimaryShard('test', st.shard1.shardName);
+    assert.commandWorked(
+        st.s0.adminCommand({enableSharding: 'test', primaryShard: st.shard1.shardName}));
     assert.commandWorked(st.s0.adminCommand({shardCollection: 'test.coll', key: {x: 'hashed'}}));
 
     runTest(st.getDB("test").coll);
 } finally {
     st.stop();
 }
-})();
