@@ -1,15 +1,39 @@
 /**
  * Confirms that the limit on number of aggregragation pipeline stages is respected.
+ * @tags: [
+ *   requires_fcv_71,
+ * ]
  */
-(function() {
-"use strict";
-
-load("jstests/libs/fixture_helpers.js");
+import {FixtureHelpers} from "jstests/libs/fixture_helpers.js";
+import {getExpectedPipelineLimit} from "jstests/libs/optimizer_utils.js";
 
 function testLimits(testDB, lengthLimit) {
     let maxLength = lengthLimit;
     let tooLarge = lengthLimit + 1;
 
+    // Test that the enforced pre-parse length limit is the same as the post-parse limit.
+    // We use $count because it is desugared into two separate stages, so it will pass the pre-parse
+    // limit but fail after.
+    let kPreParseErrCode = 7749501;
+    let kPostParseErrCode = 5054701;
+
+    // 1. This test case will pass the pre-parse enforcer but fail after.
+    assert.commandFailedWithCode(testDB.runCommand({
+        aggregate: "test",
+        cursor: {},
+        pipeline: new Array(maxLength).fill({$count: "thecount"})
+    }),
+                                 kPostParseErrCode);
+
+    // 2. This test case should be caught by the pre-parse enforcer, and the error code reflects
+    // that.
+    assert.commandFailedWithCode(testDB.runCommand({
+        aggregate: "test",
+        cursor: {},
+        pipeline: new Array(tooLarge).fill({$count: "thecount"})
+    }),
+                                 kPreParseErrCode);
+
     assert.commandWorked(testDB.runCommand({
         aggregate: "test",
         cursor: {},
@@ -20,7 +44,7 @@ function testLimits(testDB, lengthLimit) {
         cursor: {},
         pipeline: new Array(tooLarge).fill({$project: {_id: 1}})
     }),
-                                 ErrorCodes.FailedToParse);
+                                 kPreParseErrCode);
     testDB.setLogLevel(1);
 
     assert.commandWorked(testDB.runCommand({
@@ -36,7 +60,7 @@ function testLimits(testDB, lengthLimit) {
         pipeline:
             [{$unionWith: {coll: "test", pipeline: new Array(tooLarge).fill({$project: {_id: 1}})}}]
     }),
-                                 ErrorCodes.FailedToParse);
+                                 kPreParseErrCode);
 
     assert.commandWorked(testDB.runCommand({
         aggregate: "test",
@@ -48,7 +72,7 @@ function testLimits(testDB, lengthLimit) {
         cursor: {},
         pipeline: [{$facet: {foo: new Array(tooLarge).fill({$project: {_id: 1}}), bar: []}}]
     }),
-                                 ErrorCodes.FailedToParse);
+                                 kPreParseErrCode);
 
     assert.commandWorked(testDB.runCommand(
         {update: "test", updates: [{q: {}, u: new Array(maxLength).fill({$project: {_id: 1}})}]}));
@@ -56,7 +80,7 @@ function testLimits(testDB, lengthLimit) {
         update: "test",
         updates: [{q: {}, u: new Array(tooLarge).fill({$project: {_id: 1}})}]
     }),
-                                 ErrorCodes.FailedToParse);
+                                 kPreParseErrCode);
 
     const collname = "test";
 
@@ -141,7 +165,7 @@ function testLimits(testDB, lengthLimit) {
                 {from: "test", as: "as", pipeline: new Array(tooLarge).fill({$project: {_id: 1}})}
         }]
     }),
-                                 ErrorCodes.FailedToParse);
+                                 [kPostParseErrCode, kPreParseErrCode]);
 }
 
 function runTest(lengthLimit, mongosConfig = {}, mongodConfig = {}) {
@@ -167,22 +191,18 @@ function runTest(lengthLimit, mongosConfig = {}, mongodConfig = {}) {
 // This is a sanity check to make sure that the default value is correct. If the limit is changed,
 // it will break for users and this check catches that.
 const st = new ShardingTest({shards: 1, rs: {nodes: 1}});
-let buildInfo = assert.commandWorked(st.s0.getDB("test").adminCommand("buildInfo"));
 let pipelineLimit =
     assert.commandWorked(st.s0.adminCommand({"getParameter": 1, "internalPipelineLengthLimit": 1}));
-let expectedPipelineLimit = buildInfo.debug ? 200 : 1000;
-assert.eq(expectedPipelineLimit, pipelineLimit["internalPipelineLengthLimit"]);
+assert.eq(getExpectedPipelineLimit(st.s0.getDB("test")),
+          pipelineLimit["internalPipelineLengthLimit"]);
 
 const shardPrimary = st.rs0.getPrimary().getDB("test");
-buildInfo = assert.commandWorked(shardPrimary.adminCommand("buildInfo"));
-expectedPipelineLimit = buildInfo.debug ? 200 : 1000;
 pipelineLimit = assert.commandWorked(
     shardPrimary.adminCommand({"getParameter": 1, "internalPipelineLengthLimit": 1}));
-assert.eq(expectedPipelineLimit, pipelineLimit["internalPipelineLengthLimit"]);
+assert.eq(getExpectedPipelineLimit(shardPrimary), pipelineLimit["internalPipelineLengthLimit"]);
 st.stop();
 
 // Test with modified pipeline length limit.
 runTest(50,
         {setParameter: {internalPipelineLengthLimit: 50}},
         {setParameter: {internalPipelineLengthLimit: 50}});
-})();
