@@ -1,8 +1,5 @@
 // Test query on shard key prefix with $in operator containing Regex.
 
-(function() {
-"use strict";
-
 const shardingTest = new ShardingTest({shards: 1});
 const db = shardingTest.getDB("test");
 const coll = db.shard_key_prefix_with_in_operator;
@@ -18,5 +15,4 @@ assert.commandWorked(db.adminCommand({shardCollection: coll.getFullName(), key:
 
 assert.doesNotThrow(() => coll.find({a: {$in: [/myRegex/, 1]}}).toArray());
 
-shardingTest.stop();
-}());
\ No newline at end of file
+shardingTest.stop();
\ No newline at end of file
