@@ -5,15 +5,20 @@
  * contains a nested $lookup stage. This also includes tests when the mongos has stale information
  * about the foreign collection.
  *
- * @tags: [requires_fcv_51]
+ * Shard targeting logic for $lookup changed in 7.3 and may not match the expected behavior in a
+ * multiversion environment.
+ * @tags: [
+ *   requires_fcv_73,
+ *   temp_disabled_embedded_router_uncategorized,
+ * ]
  */
 
-(function() {
-"use strict";
-
-load("jstests/aggregation/extras/utils.js");  // For arrayEq.
-load("jstests/libs/profiler.js");             // For profilerHas*OrThrow helper functions.
-load("jstests/libs/local_reads.js");          // For various local read helpers.
+import {resultsEq} from "jstests/aggregation/extras/utils.js";
+import {enableLocalReadLogs, getLocalReadCount} from "jstests/libs/local_reads.js";
+import {
+    profilerHasNumMatchingEntriesOrThrow,
+    profilerHasZeroMatchingEntriesOrThrow
+} from "jstests/libs/profiler.js";
 
 const st = new ShardingTest({shards: 2, mongos: 2});
 const testName = "sharded_lookup";
@@ -21,8 +26,8 @@ const testName = "sharded_lookup";
 const mongosDB = st.s0.getDB(testName);
 const shardList = [st.shard0.getDB(testName), st.shard1.getDB(testName)];
 
-assert.commandWorked(mongosDB.adminCommand({enableSharding: mongosDB.getName()}));
-st.ensurePrimaryShard(mongosDB.getName(), st.shard0.shardName);
+assert.commandWorked(
+    mongosDB.adminCommand({enableSharding: mongosDB.getName(), primaryShard: st.shard0.shardName}));
 
 // Turn on the profiler and increase the query log level for both shards.
 assert.commandWorked(st.shard0.getDB(testName).setProfilingLevel(2));
@@ -116,7 +121,8 @@ function assertLookupExecution(pipeline, opts, expected) {
                 filter: {
                     "command.aggregate": reviewsColl.getName(),
                     "command.comment": opts.comment,
-                    "command.fromMongos": expected.mongosMerger === true
+                    "command.fromMongos": expected.mongosMerger === true,
+                    "errName": {$ne: "StaleConfig"}
                 },
                 numExpectedMatches: expected.subpipelineExec[i]
             });
@@ -137,7 +143,8 @@ function assertLookupExecution(pipeline, opts, expected) {
                     filter: {
                         "command.aggregate": reviewsColl.getName(),
                         "command.comment": opts.comment,
-                        "command.fromMongos": expected.mongosMerger === true
+                        "command.fromMongos": expected.mongosMerger === true,
+                        "errName": {$ne: "StaleConfig"}
                     },
                     numExpectedMatches: expected.subpipelineExec[i]
                 });
@@ -150,12 +157,14 @@ function assertLookupExecution(pipeline, opts, expected) {
         // execution is as expected.
         if (expected.nestedExec) {
             // Confirm that a nested $lookup is never on the shards part of the pipeline split and
-            // doesn't get dispatched to a foreign shard.
+            // doesn't get dispatched to a foreign shard (that is, we should designate a merging
+            // shard to execute the nested $lookup).
             profilerHasZeroMatchingEntriesOrThrow({
                 profileDB: shardList[i],
                 filter: {
                     "command.aggregate": reviewsColl.getName(),
                     "command.comment": opts.comment,
+                    "errName": {$ne: "StaleConfig"},
                     "command.pipeline.$lookup": {$exists: true}
                 }
             });
@@ -163,8 +172,11 @@ function assertLookupExecution(pipeline, opts, expected) {
             // Confirm that the nested $lookup subpipeline execution is as expected.
             profilerHasNumMatchingEntriesOrThrow({
                 profileDB: shardList[i],
-                filter:
-                    {"command.aggregate": updatesColl.getName(), "command.comment": opts.comment},
+                filter: {
+                    "command.aggregate": updatesColl.getName(),
+                    "command.comment": opts.comment,
+                    "errName": {$ne: "StaleConfig"}
+                },
                 numExpectedMatches: expected.nestedExec[i]
             });
         }
@@ -821,4 +833,3 @@ assertLookupExecution(pipeline, {comment: "foreign_becomes_sharded", batchSize:
 });
 
 st.stop();
-}());
