@@ -2,26 +2,20 @@
  * Tests to verify that the current op output shows all the sub-pipelines of a $unionWith. In this
  * test we also validate that current op shows the expected stages and comment.
  *
- * In 5.1, new behavior was introduced regarding reading from an unsharded collection on the primary
- * shard. This behavior affects which operations are logged.
- *  @tags: [ requires_fcv_51 ]
+ * This test uses a new $_internalSplitPipeline syntax introduced in 8.0.
+ * @tags: [  requires_fcv_80, ]
  */
-(function() {
-"use strict";
-
-load("jstests/libs/fixture_helpers.js");  // For FixtureHelpers.
-load("jstests/libs/curop_helpers.js");    // For waitForCurOpByFailPoint.
+import {waitForCurOpByFailPoint} from "jstests/libs/curop_helpers.js";
+import {FixtureHelpers} from "jstests/libs/fixture_helpers.js";
 
 const st = new ShardingTest({shards: 2});
 const testDB = st.s.getDB(jsTestName());
 const shardedColl1 = testDB.shardedColl1;
 const shardedColl2 = testDB.shardedColl2;
 const unshardedColl = testDB.unsharded;
-const shard0DB = st.shard0.getDB(jsTestName());
-const shard1DB = st.shard1.getDB(jsTestName());
 
-assert.commandWorked(st.s0.adminCommand({enableSharding: testDB.getName()}));
-st.ensurePrimaryShard(testDB.getName(), st.shard0.shardName);
+assert.commandWorked(
+    st.s0.adminCommand({enableSharding: testDB.getName(), primaryShard: st.shard0.shardName}));
 
 function setupShardColl(shardedColl) {
     // Shard shardedColl on {x:1}, split it at {x:0}, and move chunk {x:1} to shard1.
@@ -147,7 +141,7 @@ runTest({
                     }]
                 }
             },
-            {$_internalSplitPipeline: {mergeType: "primaryShard"}}
+            {$_internalSplitPipeline: {mergeType: {"specificShard": st.shard0.shardName}}}
         ],
         cursor: {}
     },
@@ -194,4 +188,3 @@ runTest({
 });
 
 st.stop();
-})();
