@@ -4,33 +4,52 @@
  *  requires_fcv_63,
  * ]
  */
-(function() {
-"use strict";
 
-// For profilerHasSingleMatchingEntryOrThrow.
-load("jstests/libs/profiler.js");
-// For checkSbeFullyEnabled.
-load("jstests/libs/sbe_util.js");
-// For areAllCollectionsClustered.
-load("jstests/libs/clustered_collections/clustered_collection_util.js");
+import {profilerHasSingleMatchingEntryOrThrow} from "jstests/libs/profiler.js";
+
+// Legal values for the verifyExplainResult() 'optimizedAwayPipeline' argument.
+const kOptFalse = 0;
+const kOptTrue = 1;
+const kOptEither = 2;
 
 // Given sharded explain output in 'shardedExplain', verifies that the explain mode 'verbosity'
 // affected the output verbosity appropriately, and that the response has the expected format.
-// Set 'optimizedAwayPipeline' to true if the pipeline is expected to be optimized away.
+// Set 'optimizedAwayPipeline' to:
+//   kOptTrue if the pipeline is expected to be optimized away
+//   kOptFalse if the pipeline is expected to be present
+//   kOptEither if the call does not know so must accept either of the prior two cases
 function verifyExplainResult(
-    {shardedExplain = null, verbosity = "", optimizedAwayPipeline = false} = {}) {
+    {shardedExplain = null, verbosity = "", optimizedAwayPipeline = kOptFalse} = {}) {
     assert.commandWorked(shardedExplain);
     assert(shardedExplain.hasOwnProperty("shards"), tojson(shardedExplain));
+
+    // Verifies the explain for each shard.
     for (let elem in shardedExplain.shards) {
         let shard = shardedExplain.shards[elem];
         let root;
-        if (optimizedAwayPipeline) {
+
+        // Resolve 'kOptEither' to 'kOptTrue' or 'kOptFalse' for the current shard. If 'shard' has a
+        // "queryPlanner" property, this means the pipeline has been optimized away. (When the
+        // pipeline is present, "queryPlanner" is instead a property of shard.stages[0].$cursor.)
+        let optedAwayOnThisShard = optimizedAwayPipeline;
+        if (optedAwayOnThisShard == kOptEither) {
+            if (shard.hasOwnProperty("queryPlanner")) {
+                optedAwayOnThisShard = kOptTrue;
+            } else {
+                optedAwayOnThisShard = kOptFalse;
+            }
+        }
+
+        // Verify the explain output.
+        if (optedAwayOnThisShard == kOptTrue) {
             assert(shard.hasOwnProperty("queryPlanner"), tojson(shardedExplain));
             root = shard;
-        } else {
+        } else if (optedAwayOnThisShard == kOptFalse) {
             assert(shard.stages[0].hasOwnProperty("$cursor"), tojson(shardedExplain));
             assert(shard.stages[0].$cursor.hasOwnProperty("queryPlanner"), tojson(shardedExplain));
             root = shard.stages[0].$cursor;
+        } else {
+            assert(false, `Unsupported 'optimizedAwayPipeline' value ${optimizedAwayPipeline}`);
         }
         if (verbosity === "queryPlanner") {
             assert(!root.hasOwnProperty("executionStats"), tojson(shardedExplain));
@@ -53,12 +72,10 @@ let config = mongos.getDB("config");
 let db = mongos.getDB(jsTestName());
 db.dropDatabase();
 
-const isClustered = ClusteredCollectionUtil.areAllCollectionsClustered(st.rs0.getPrimary());
-
 let coll = db.getCollection("coll");
 
-assert.commandWorked(config.adminCommand({enableSharding: db.getName()}));
-st.ensurePrimaryShard(db.getName(), st.shard0.shardName);
+assert.commandWorked(
+    config.adminCommand({enableSharding: db.getName(), primaryShard: st.shard0.shardName}));
 assert.commandWorked(config.adminCommand({shardCollection: coll.getFullName(), key: {a: 1}}));
 
 assert.commandWorked(mongos.adminCommand({split: coll.getFullName(), middle: {a: 6}}));
@@ -81,11 +98,11 @@ assert.eq(5, view.find({a: {$lte: 8}}).itcount());
 
 let result = db.runCommand({explain: {find: "view", filter: {a: {$lte: 7}}}});
 verifyExplainResult(
-    {shardedExplain: result, verbosity: "allPlansExecution", optimizedAwayPipeline: true});
+    {shardedExplain: result, verbosity: "allPlansExecution", optimizedAwayPipeline: kOptTrue});
 for (let verbosity of explainVerbosities) {
     result = db.runCommand({explain: {find: "view", filter: {a: {$lte: 7}}}, verbosity: verbosity});
     verifyExplainResult(
-        {shardedExplain: result, verbosity: verbosity, optimizedAwayPipeline: true});
+        {shardedExplain: result, verbosity: verbosity, optimizedAwayPipeline: kOptTrue});
 }
 
 //
@@ -96,19 +113,19 @@ assert.eq(5, view.aggregate([{$match: {a: {$lte: 8}}}]).itcount());
 // Test that the explain:true flag for the aggregate command results in queryPlanner verbosity.
 result = db.runCommand({aggregate: "view", pipeline: [{$match: {a: {$lte: 8}}}], explain: true});
 verifyExplainResult(
-    {shardedExplain: result, verbosity: "queryPlanner", optimizedAwayPipeline: true});
+    {shardedExplain: result, verbosity: "queryPlanner", optimizedAwayPipeline: kOptTrue});
 
 result =
     db.runCommand({explain: {aggregate: "view", pipeline: [{$match: {a: {$lte: 8}}}], cursor: {}}});
 verifyExplainResult(
-    {shardedExplain: result, verbosity: "allPlansExecution", optimizedAwayPipeline: true});
+    {shardedExplain: result, verbosity: "allPlansExecution", optimizedAwayPipeline: kOptTrue});
 for (let verbosity of explainVerbosities) {
     result = db.runCommand({
         explain: {aggregate: "view", pipeline: [{$match: {a: {$lte: 8}}}], cursor: {}},
         verbosity: verbosity
     });
     verifyExplainResult(
-        {shardedExplain: result, verbosity: verbosity, optimizedAwayPipeline: true});
+        {shardedExplain: result, verbosity: verbosity, optimizedAwayPipeline: kOptTrue});
 }
 
 //
@@ -116,25 +133,18 @@ for (let verbosity of explainVerbosities) {
 //
 assert.eq(5, view.count({a: {$lte: 8}}));
 
-// If SBE is enabled on all nodes, then validate the explain results for a count command. We could
-// validate the explain results when the classic engine is enabled, but doing so is complicated for
-// multiversion scenarios (e.g. in the multiversion passthrough on the classic engine build variant
-// only some shards have SBE enabled, so the expected results differ across shards).
-if (checkSbeFullyEnabled(db, [], true /*checkAllNodes*/)) {
-    result = db.runCommand({explain: {count: "view", query: {a: {$lte: 8}}}});
-    // Allow success whether or not the pipeline is optimized away, as it differs based on test
-    // environment and execution engine used.
-    verifyExplainResult({
-        shardedExplain: result,
-        verbosity: "allPlansExecution",
-        optimizedAwayPipeline: !isClustered
-    });
-    for (let verbosity of explainVerbosities) {
-        result =
-            db.runCommand({explain: {count: "view", query: {a: {$lte: 8}}}, verbosity: verbosity});
-        verifyExplainResult(
-            {shardedExplain: result, verbosity: verbosity, optimizedAwayPipeline: !isClustered});
-    }
+// "count" on a view that is a $match will produce different explain output on Classic vs SBE, as
+// the query will be rewriten as a $group, but only SBE has a $group pushdown feature, which
+// optimizes away the pipeline. Depending on build variant and engine selection flags, as well as
+// specific configurations of individual nodes in multiversion clusters, we may get either the
+// Classic or SBE explain variant, so here we accept either one ('kOptEither').
+result = db.runCommand({explain: {count: "view", query: {a: {$lte: 8}}}});
+verifyExplainResult(
+    {shardedExplain: result, verbosity: "allPlansExecution", optimizedAwayPipeline: kOptEither});
+for (let verbosity of explainVerbosities) {
+    result = db.runCommand({explain: {count: "view", query: {a: {$lte: 8}}}, verbosity: verbosity});
+    verifyExplainResult(
+        {shardedExplain: result, verbosity: verbosity, optimizedAwayPipeline: kOptEither});
 }
 
 //
@@ -164,7 +174,7 @@ assert.commandFailedWithCode(result, ErrorCodes.CommandNotSupportedOnView);
 //  Confirm getShardVersion command fails.
 //
 assert.commandFailedWithCode(db.adminCommand({getShardVersion: view.getFullName()}),
-                             ErrorCodes.NamespaceNotSharded);
+                             [ErrorCodes.NamespaceNotSharded, ErrorCodes.NamespaceNotFound]);
 
 //
 // Confirm that the comment parameter on a find command is retained when rewritten as an
@@ -186,4 +196,3 @@ profilerHasSingleMatchingEntryOrThrow({
 });
 
 st.stop();
-})();
