@@ -20,6 +20,9 @@
  *   server, rather than to the shard.
  * - useLogs: Normally, profiling is used to check behavior.  Set this to true to use slow op log
  *   lines instead.
+ * - skipMultiversion: If this is set to true then the test will be skipped for multiversion suites
+ *   only. This is useful if the command was behind a feature flag in previous versions and is now
+ *   enabled.
  *
  * @tags: [
  *   does_not_support_stepdowns,
@@ -28,12 +31,13 @@
  *   uses_transactions,
  * ]
  */
-(function() {
-"use strict";
-
-load('jstests/libs/profiler.js');
-load('jstests/sharding/libs/last_lts_mongod_commands.js');
-load('jstests/sharding/libs/last_lts_mongos_commands.js');
+import {profilerHasSingleMatchingEntryOrThrow} from "jstests/libs/profiler.js";
+import {
+    commandsRemovedFromMongodSinceLastLTS
+} from "jstests/sharding/libs/last_lts_mongod_commands.js";
+import {
+    commandsRemovedFromMongosSinceLastLTS
+} from "jstests/sharding/libs/last_lts_mongos_commands.js";
 
 // TODO SERVER-50144 Remove this and allow orphan checking.
 // This test calls removeShard which can leave docs in config.rangeDeletions in state "pending",
@@ -99,12 +103,13 @@ let testCases = {
     _configsvrCommitIndex: {skip: "internal command"},
     _configsvrCommitMergeAllChunksOnShard: {skip: "internal command"},
     _configsvrCommitMovePrimary: {skip: "internal command"},  // Can be removed once 6.0 is last LTS
+    _configsvrCommitRefineCollectionShardKey: {skip: "internal command"},
     _configsvrCommitReshardCollection: {skip: "internal command"},
     _configsvrConfigureCollectionBalancing: {skip: "internal command"},
     _configsvrCreateDatabase: {skip: "internal command"},
     _configsvrDropIndexCatalogEntry: {skip: "internal command"},
     _configsvrEnsureChunkVersionIsGreaterThan: {skip: "internal command"},
-    _configsvrGetHistoricalPlacement: {skip: "internal command"},  // TODO SERVER-73029 remove
+    _configsvrGetHistoricalPlacement: {skip: "internal command"},
     _configsvrMovePrimary: {skip: "internal command"},  // Can be removed once 6.0 is last LTS
     _configsvrMoveRange: {skip: "internal command"},
     _configsvrRefineCollectionShardKey: {skip: "internal command"},
@@ -113,7 +118,6 @@ let testCases = {
     _configsvrRemoveShardFromZone: {skip: "internal command"},
     _configsvrRemoveTags: {skip: "internal command"},
     _configsvrRenameCollection: {skip: "internal command"},
-    _configsvrRenameCollectionMetadata: {skip: "internal command"},
     _configsvrRepairShardedCollectionChunksHistory: {skip: "internal command"},
     _configsvrResetPlacementHistory: {skip: "internal command"},
     _configsvrReshardCollection: {skip: "internal command"},
@@ -139,9 +143,6 @@ let testCases = {
     _mergeAuthzCollections: {skip: "internal command"},
     _migrateClone: {skip: "internal command"},
     _mongotConnPoolStats: {skip: "internal command"},
-    _movePrimaryRecipientSyncData: {skip: "internal command"},
-    _movePrimaryRecipientAbortMigration: {skip: "internal command"},
-    _movePrimaryRecipientForgetMigration: {skip: "internal command"},
     _recvChunkAbort: {skip: "internal command"},
     _recvChunkCommit: {skip: "internal command"},
     _recvChunkReleaseCritSec: {skip: "internal command"},
@@ -149,21 +150,24 @@ let testCases = {
     _recvChunkStatus: {skip: "internal command"},
     _refreshQueryAnalyzerConfiguration: {skip: "internal command"},
     _shardsvrAbortReshardCollection: {skip: "internal command"},
+    _shardsvrBeginMigrationBlockingOperation: {skip: "internal command"},
+    _shardsvrChangePrimary: {skip: "internal command"},
     _shardsvrCleanupReshardCollection: {skip: "internal command"},
     _shardsvrCloneCatalogData: {skip: "internal command"},
     _shardsvrRegisterIndex: {skip: "internal command"},
     _shardsvrCheckMetadataConsistency: {skip: "internal command"},
     _shardsvrCheckMetadataConsistencyParticipant: {skip: "internal command"},
+    _shardsvrCleanupStructuredEncryptionData: {skip: "internal command"},
     _shardsvrCommitIndexParticipant: {skip: "internal command"},
     _shardsvrCommitReshardCollection: {skip: "internal command"},
     _shardsvrCompactStructuredEncryptionData: {skip: "internal command"},
+    _shardsvrConvertToCapped: {skip: "internal command"},
+    _shardsvrCoordinateMultiUpdate: {skip: "internal command"},
     _shardsvrCreateCollection: {skip: "internal command"},
     _shardsvrCreateCollectionParticipant: {skip: "internal command"},
     _shardsvrCreateGlobalIndex: {skip: "internal command"},
     _shardsvrDropGlobalIndex: {skip: "internal command"},
     _shardsvrDropCollection: {skip: "internal command"},
-    // TODO SERVER-74324: deprecate _shardsvrDropCollectionIfUUIDNotMatching after 7.0 is lastLTS.
-    _shardsvrDropCollectionIfUUIDNotMatching: {skip: "internal command"},
     _shardsvrDropCollectionIfUUIDNotMatchingWithWriteConcern: {skip: "internal command"},
     _shardsvrDropCollectionParticipant: {skip: "internal command"},
     _shardsvrUnregisterIndex: {skip: "internal command"},
@@ -171,8 +175,10 @@ let testCases = {
     _shardsvrDropIndexes: {skip: "internal command"},
     _shardsvrDropDatabase: {skip: "internal command"},
     _shardsvrDropDatabaseParticipant: {skip: "internal command"},
+    _shardsvrEndMigrationBlockingOperation: {skip: "internal command"},
     _shardsvrGetStatsForBalancing: {skip: "internal command"},
     _shardsvrInsertGlobalIndexKey: {skip: "internal command"},
+    _shardsvrJoinDDLCoordinators: {skip: "internal command"},
     _shardsvrDeleteGlobalIndexKey: {skip: "internal command"},
     _shardsvrWriteGlobalIndexKeys: {skip: "internal command"},
     _shardsvrJoinMigrations: {skip: "internal command"},
@@ -198,10 +204,23 @@ let testCases = {
     _shardsvrValidateShardKeyCandidate: {skip: "internal command"},
     _shardsvrCollMod: {skip: "internal command"},
     _shardsvrCollModParticipant: {skip: "internal command"},
+    _shardsvrConvertToCappedParticipant: {skip: "internal command"},
     _shardsvrParticipantBlock: {skip: "internal command"},
-    _startStreamProcessor: {skip: "internal command"},
+    _shardsvrUntrackUnsplittableCollection: {skip: "internal command"},
+    streams_startStreamProcessor: {skip: "internal command"},
+    streams_startStreamSample: {skip: "internal command"},
+    streams_stopStreamProcessor: {skip: "internal command"},
+    streams_listStreamProcessors: {skip: "internal command"},
+    streams_getMoreStreamSample: {skip: "internal command"},
+    streams_getStats: {skip: "internal command"},
+    streams_testOnlyInsert: {skip: "internal command"},
+    streams_getMetrics: {skip: "internal command"},
+    streams_updateFeatureFlags: {skip: "internal command"},
+    streams_testOnlyGetFeatureFlags: {skip: "internal command"},
+    streams_writeCheckpoint: {skip: "internal command"},
     _transferMods: {skip: "internal command"},
     _vectorClockPersist: {skip: "internal command"},
+    abortMoveCollection: {skip: "does not accept read or write concern"},
     abortReshardCollection: {skip: "does not accept read or write concern"},
     abortTransaction: {
         setUp: function(conn) {
@@ -229,6 +248,7 @@ let testCases = {
         checkWriteConcern: true,
         useLogs: true,
     },
+    abortUnshardCollection: {skip: "does not accept read or write concern"},
     addShard: {skip: "does not accept read or write concern"},
     addShardToZone: {skip: "does not accept read or write concern"},
     aggregate: {
@@ -251,18 +271,37 @@ let testCases = {
     },
     applyOps: {skip: "internal command"},
     authenticate: {skip: "does not accept read or write concern"},
-    autoSplitVector: {skip: "internal command"},
+    autoCompact: {skip: "does not accept read or write concern"},
+    autoSplitVector: {skip: "does not accept read or write concern"},
     balancerCollectionStatus: {skip: "does not accept read or write concern"},
     balancerStart: {skip: "does not accept read or write concern"},
     balancerStatus: {skip: "does not accept read or write concern"},
     balancerStop: {skip: "does not accept read or write concern"},
     buildInfo: {skip: "does not accept read or write concern"},
-    bulkWrite: {skip: "not yet implemented"},
+    bulkWrite: {
+        setUp: function(conn) {
+            assert.commandWorked(conn.getDB(db).runCommand({create: coll, writeConcern: {w: 1}}));
+        },
+        db: "admin",
+        command: {
+            bulkWrite: 1,
+            ops: [{insert: 0, document: {_id: ObjectId()}}],
+            nsInfo: [{ns: db + "." + coll}]
+        },
+        checkReadConcern: false,
+        checkWriteConcern: true,
+        // TODO SERVER-23266: If the overall batch command if profiled, then it would be better
+        // to use profiling.  In the meantime, use logs.
+        useLogs: true,
+        skipMultiversion: true,
+    },
     captrunc: {skip: "test command"},
+    changePrimary: {skip: "does not accept read or write concern"},
     checkMetadataConsistency: {skip: "does not accept read or write concern"},
     checkShardingIndex: {skip: "does not accept read or write concern"},
     cleanupOrphaned: {skip: "only on shard server"},
     cleanupReshardCollection: {skip: "does not accept read or write concern"},
+    cleanupStructuredEncryptionData: {skip: "does not accept read or write concern"},
     clearJumboFlag: {skip: "does not accept read or write concern"},
     clearLog: {skip: "does not accept read or write concern"},
     clone: {skip: "deprecated"},
@@ -276,6 +315,7 @@ let testCases = {
     },
     clusterAbortTransaction: {skip: "already tested by 'abortTransaction' tests on mongos"},
     clusterAggregate: {skip: "already tested by 'aggregate' tests on mongos"},
+    clusterBulkWrite: {skip: "already tested by 'bulkWrite' tests on mongos"},
     clusterCommitTransaction: {skip: "already tested by 'commitTransaction' tests on mongos"},
     clusterCount: {skip: "already tested by 'count' tests on mongos"},
     clusterDelete: {skip: "already tested by 'delete' tests on mongos"},
@@ -346,9 +386,8 @@ let testCases = {
     },
     cpuload: {skip: "does not accept read or write concern"},
     create: {
-        command: {create: coll},
-        checkReadConcern: false,
-        checkWriteConcern: true,
+        skip:
+            "The create command is not passed through and instead it goes out to the shards as _shardsvrCreateCollection with the user-specified write concern"
     },
     createIndexes: {
         setUp: function(conn) {
@@ -365,8 +404,8 @@ let testCases = {
         shardedTargetsConfigServer: true,
         useLogs: true,
     },
-    createSearchIndex: {skip: "present in v6.3 but renamed to createSearchIndexes in v7.0"},
     createSearchIndexes: {skip: "does not accept read or write concern"},
+    createUnsplittableCollection: {skip: "does not accept read or write concern"},
     createUser: {
         command: {createUser: "foo", pwd: "bar", roles: []},
         checkReadConcern: false,
@@ -510,26 +549,9 @@ let testCases = {
     getLog: {skip: "does not accept read or write concern"},
     getMore: {skip: "does not accept read or write concern"},
     getParameter: {skip: "does not accept read or write concern"},
-    getQueryableEncryptionCountInfo: {
-        // TODO SERVER-75631 - Enable this test once the feature flag is gone
-        skip: "requires feature flag"
-        // setUp: function(conn) {
-        //     assert.commandWorked(conn.getCollection(nss).insert({x: 1}, {writeConcern: {w: 1}}));
-        // },
-        // command: {
-        //     getQueryableEncryptionCountInfo: coll,
-        //     tokens: [
-        //         {tokens: [{"s": BinData(0, "lUBO7Mov5Sb+c/D4cJ9whhhw/+PZFLCk/AQU2+BpumQ=")}]},
-        //     ],
-        //     "forInsert": true,
-        // },
-        // checkReadConcern: true,
-        // checkWriteConcern: false,
-        // useLogs: true,
-    },
+    getQueryableEncryptionCountInfo: {skip: "not profiled or logged"},
     getShardMap: {skip: "internal command"},
     getShardVersion: {skip: "internal command"},
-    getnonce: {skip: "removed in v6.3"},
     godinsert: {skip: "for testing only"},
     grantPrivilegesToRole: {
         setUp: function(conn) {
@@ -615,6 +637,7 @@ let testCases = {
         skip:
             "does not accept read or write concern (accepts writeConcern, but only explicitly and when _secondaryThrottle is true)"
     },
+    moveCollection: {skip: "does not accept read or write concern"},
     movePrimary: {skip: "does not accept read or write concern"},
     moveRange: {
         skip:
@@ -728,6 +751,7 @@ let testCases = {
     },
     rolesInfo: {skip: "does not accept read or write concern"},
     rotateCertificates: {skip: "does not accept read or write concern"},
+    rotateFTDC: {skip: "does not accept read or write concern"},
     saslContinue: {skip: "does not accept read or write concern"},
     saslStart: {skip: "does not accept read or write concern"},
     sbe: {skip: "internal command"},
@@ -743,6 +767,8 @@ let testCases = {
     setShardVersion: {skip: "internal command"},
     setChangeStreamState: {skip: "does not accept read or write concern"},
     setClusterParameter: {skip: "does not accept read or write concern"},
+    setQuerySettings: {skip: "does not accept read or write concern"},
+    removeQuerySettings: {skip: "does not accept read or write concern"},
     setUserWriteBlockMode: {skip: "does not accept read or write concern"},
     shardCollection: {skip: "does not accept read or write concern"},
     shardingState: {skip: "does not accept read or write concern"},
@@ -755,6 +781,7 @@ let testCases = {
     startRecordingTraffic: {skip: "does not accept read or write concern"},
     startSession: {skip: "does not accept read or write concern"},
     stopRecordingTraffic: {skip: "does not accept read or write concern"},
+    sysprofile: {skip: "internal command"},
     testDeprecation: {skip: "does not accept read or write concern"},
     testDeprecationInVersion2: {skip: "does not accept read or write concern"},
     testInternalTransactions: {skip: "internal command"},
@@ -762,9 +789,13 @@ let testCases = {
     testReshardCloneCollection: {skip: "internal command"},
     testVersions1And2: {skip: "does not accept read or write concern"},
     testVersion2: {skip: "does not accept read or write concern"},
+    timeseriesCatalogBucketParamsChanged: {skip: "internal command"},
     top: {skip: "does not accept read or write concern"},
     transitionFromDedicatedConfigServer: {skip: "does not accept read or write concern"},
     transitionToDedicatedConfigServer: {skip: "does not accept read or write concern"},
+    transitionToShardedCluster: {skip: "accepts only majority"},
+    unshardCollection: {skip: "does not accept read or write concern"},
+    untrackUnshardedCollection: {skip: "does not accept read or write concern"},
     update: {
         setUp: function(conn) {
             assert.commandWorked(conn.getCollection(nss).insert({x: 1}, {writeConcern: {w: 1}}));
@@ -807,7 +838,7 @@ let testCases = {
     voteCommitImportCollection: {skip: "internal command"},
     voteCommitIndexBuild: {skip: "internal command"},
     waitForFailPoint: {skip: "does not accept read or write concern"},
-    waitForOngoingChunkSplits: {skip: "does not accept read or write concern"},
+    getShardingReady: {skip: "internal command"},
     whatsmysni: {skip: "does not accept read or write concern"},
     whatsmyuri: {skip: "internal command"},
 };
@@ -936,6 +967,14 @@ function runScenario(
             print("skipping " + cmdName + ": " + test.skip);
             return;
         }
+
+        const isMultiversion = jsTest.options().shardMixedBinVersions ||
+            jsTest.options().useRandomBinVersionsWithinReplicaSet;
+        if (isMultiversion && test.skipMultiversion) {
+            print("skipping " + cmdName + " since we are in a multiversion suite.");
+            return;
+        }
+
         validateTestCase(test);
 
         let sharded = !!configSvrCheckConn;
@@ -1103,13 +1142,14 @@ function runTests(conn, regularCheckConn, configSvrCheckConn) {
                 {explicitRWC: true, explicitProvenance: true});
 }
 
-let rst = new ReplSetTest({nodes: 1});
-rst.startSet();
-rst.initiate();
-runTests(rst.getPrimary(), rst.getPrimary(), undefined, false);
-rst.stopSet();
+if (!jsTestOptions().useAutoBootstrapProcedure) {  // TODO: SERVER-80318 Delete block
+    let rst = new ReplSetTest({nodes: 1});
+    rst.startSet();
+    rst.initiate();
+    runTests(rst.getPrimary(), rst.getPrimary(), undefined, false);
+    rst.stopSet();
+}
 
 let st = new ShardingTest({mongos: 1, shards: {rs0: {nodes: 1}}});
 runTests(st.s0, st.rs0.getPrimary(), st.configRS.getPrimary(), true);
 st.stop();
-})();
