@@ -1,8 +1,11 @@
 // Tests propagation of RWC defaults across a sharded cluster.
-(function() {
-'use strict';
-
-load("jstests/libs/read_write_concern_defaults_propagation_common.js");
+// @tags: [
+//   # Test doesn't start enough mongods to have num_mongos routers
+//   temp_disabled_embedded_router_num_routers,
+// ]
+import {
+    ReadWriteConcernDefaultsPropagation
+} from "jstests/libs/read_write_concern_defaults_propagation_common.js";
 
 var st = new ShardingTest({
     shards: 1,
@@ -29,4 +32,3 @@ ReadWriteConcernDefaultsPropagation.runDropAndDeleteTests(st.configRS.getPrimary
                                                           mongosAndConfigNodes);
 
 st.stop();
-})();
