@@ -4,16 +4,15 @@
 
 // Cannot run the filtering metadata check on tests that run refineCollectionShardKey.
 TestData.skipCheckShardFilteringMetadata = true;
-
-(function() {
-'use strict';
-load("jstests/libs/config_shard_util.js");
-load('jstests/libs/fail_point_util.js');
-load('jstests/libs/profiler.js');
-load('jstests/sharding/libs/shard_versioning_util.js');
-load('jstests/sharding/libs/sharded_transactions_helpers.js');
-load("jstests/sharding/libs/find_chunks_util.js");
-load("jstests/sharding/updateOne_without_shard_key/libs/write_without_shard_key_test_util.js");
+import {configureFailPoint} from "jstests/libs/fail_point_util.js";
+import {
+    flushRoutersAndRefreshShardMetadata
+} from "jstests/sharding/libs/sharded_transactions_helpers.js";
+import {findChunksUtil} from "jstests/sharding/libs/find_chunks_util.js";
+import {
+    WriteWithoutShardKeyTestUtil
+} from "jstests/sharding/updateOne_without_shard_key/libs/write_without_shard_key_test_util.js";
+import {FeatureFlagUtil} from "jstests/libs/feature_flag_util.js";
 
 const st = new ShardingTest({
     mongos: 2,
@@ -31,18 +30,10 @@ const kDbName = 'db';
 const kCollName = 'foo';
 const kNsName = kDbName + '.' + kCollName;
 const kConfigCollections = 'config.collections';
-const kConfigChunks = 'config.chunks';
 const kConfigTags = 'config.tags';
-const kConfigChangelog = 'config.changelog';
 const kUnrelatedName = kDbName + '.bar';
 let oldEpoch = null;
 
-function enableShardingAndShardColl(keyDoc) {
-    assert.commandWorked(mongos.adminCommand({enableSharding: kDbName}));
-    st.ensurePrimaryShard(kDbName, primaryShard);
-    assert.commandWorked(mongos.adminCommand({shardCollection: kNsName, key: keyDoc}));
-}
-
 function dropAndRecreateColl(keyDoc) {
     assert.commandWorked(mongos.getDB(kDbName).runCommand({drop: kCollName}));
     assert.commandWorked(mongos.getCollection(kNsName).insert(keyDoc));
@@ -151,17 +142,8 @@ function validateCRUDAfterRefine() {
         mongos.setReadPref(null);
     }
 
-    if (jsTestOptions().mongosBinVersion !== "last-continuous") {
-        assert.commandWorked(sessionDB.getCollection(kCollName).remove({a: 1, b: 1}, true));
-        assert.commandWorked(sessionDB.getCollection(kCollName).remove({a: -1, b: -1}, true));
-    } else {
-        // The full shard key is required when removing documents when mongos is "last-continuous"
-        // (as SERVER-44422 is only backported to "last-lts" and "7.0").
-        assert.writeErrorWithCode(sessionDB.getCollection(kCollName).remove({a: 1, b: 1}, true),
-                                  ErrorCodes.ShardKeyNotFound);
-        assert.writeErrorWithCode(sessionDB.getCollection(kCollName).remove({a: -1, b: -1}, true),
-                                  ErrorCodes.ShardKeyNotFound);
-    }
+    assert.commandWorked(sessionDB.getCollection(kCollName).remove({a: 1, b: 1}, true));
+    assert.commandWorked(sessionDB.getCollection(kCollName).remove({a: -1, b: -1}, true));
 
     assert.commandWorked(sessionDB.getCollection(kCollName).remove({a: 1, b: 2, c: 1, d: 1}, true));
     assert.commandWorked(
@@ -303,19 +285,16 @@ assert.commandFailedWithCode(
 
 assert.commandWorked(mongos.getCollection(kNsName).insert({aKey: 1}));
 
-// Should fail because namespace 'db.foo' is not sharded. NOTE: This NamespaceNotSharded error
-// is thrown in RefineCollectionShardKeyCommand by 'getShardedCollectionRoutingInfoWithRefresh'.
+// Should fail because namespace 'db.foo' is not sharded.
 assert.commandFailedWithCode(
     mongos.adminCommand({refineCollectionShardKey: kNsName, key: {_id: 1, aKey: 1}}),
     ErrorCodes.NamespaceNotSharded);
 
-// Should fail because operation can't run on config server
-const isConfigShardEnabled = ConfigShardUtil.isEnabledIgnoringFCV(st);
 assert.commandFailedWithCode(
     mongos.adminCommand({refineCollectionShardKey: "config.collections", key: {_id: 1, aKey: 1}}),
-    isConfigShardEnabled ? ErrorCodes.NamespaceNotSharded : ErrorCodes.NoShardingEnabled);
+    ErrorCodes.NamespaceNotSharded);
 
-enableShardingAndShardColl({_id: 1});
+assert.commandWorked(mongos.adminCommand({shardCollection: kNsName, key: {_id: 1}}));
 
 // Should fail because shard key is invalid (i.e. bad values).
 assert.commandFailedWithCode(
@@ -330,7 +309,8 @@ assert.commandFailedWithCode(
     ErrorCodes.BadValue);
 
 // Should fail because shard key is not specified.
-assert.commandFailedWithCode(mongos.adminCommand({refineCollectionShardKey: kNsName}), 40414);
+assert.commandFailedWithCode(mongos.adminCommand({refineCollectionShardKey: kNsName}),
+                             ErrorCodes.IDLFailedToParse);
 assert.commandFailedWithCode(mongos.adminCommand({refineCollectionShardKey: kNsName, key: {}}),
                              ErrorCodes.BadValue);
 
@@ -349,7 +329,7 @@ assert.commandWorked(mongos.getDB(kDbName).dropDatabase());
 
 jsTestLog('********** NAMESPACE VALIDATION TESTS **********');
 
-enableShardingAndShardColl({_id: 1});
+assert.commandWorked(mongos.adminCommand({shardCollection: kNsName, key: {_id: 1}}));
 
 // Configure failpoint 'hangRefineCollectionShardKeyAfterRefresh' on staleMongos and run
 // refineCollectionShardKey against this mongos in a parallel thread.
@@ -374,7 +354,7 @@ assert.commandWorked(mongos.getDB(kDbName).dropDatabase());
 
 jsTestLog('********** SHARD KEY VALIDATION TESTS **********');
 
-enableShardingAndShardColl({_id: 1});
+assert.commandWorked(mongos.adminCommand({shardCollection: kNsName, key: {_id: 1}}));
 
 // Should fail because new shard key {aKey: 1} does not extend current shard key {_id: 1} of
 // namespace 'db.foo'.
@@ -591,7 +571,7 @@ assert.commandWorked(mongos.getDB(kDbName).dropDatabase());
 
 jsTestLog('********** UNIQUENESS PROPERTY TESTS **********');
 
-enableShardingAndShardColl({_id: 1});
+assert.commandWorked(mongos.adminCommand({shardCollection: kNsName, key: {_id: 1}}));
 assert.commandWorked(mongos.getCollection(kNsName).createIndex({_id: 1, aKey: 1}));
 
 // Verify that refineCollectionShardKey cannot modify a unique=false sharded collection.
@@ -631,7 +611,7 @@ const newKeyDoc = {
     d: 1
 };
 
-enableShardingAndShardColl(oldKeyDoc);
+assert.commandWorked(mongos.adminCommand({shardCollection: kNsName, key: oldKeyDoc}));
 assert.commandWorked(mongos.getCollection(kNsName).createIndex(newKeyDoc));
 
 // CRUD operations before and after refineCollectionShardKey should work as expected.
@@ -720,7 +700,7 @@ validateUnrelatedCollAfterRefine(oldCollArr, oldChunkArr, oldTagsArr);
 
 // Verify that all shards in the namespace 'db.foo' eventually refresh. NOTE: This will only succeed
 // in a linear jstest without failovers.
-const isStepdownSuite = typeof ContinuousStepdown !== 'undefined';
+const isStepdownSuite = TestData.runningWithConfigStepdowns;
 if (!isStepdownSuite) {
     dropAndReshardColl(oldKeyDoc);
     assert.commandWorked(mongos.getCollection(kNsName).createIndex(newKeyDoc));
@@ -745,74 +725,6 @@ if (!isStepdownSuite) {
                         .metadata.shardVersionEpoch.toString());
 }
 
-// TODO SERVER-72515: remove once 7.0 becomes last-lts.
-const fcvDoc = assert.commandWorked(
-    st.configRS.getPrimary().adminCommand({getParameter: 1, featureCompatibilityVersion: 1}));
-if (fcvDoc.featureCompatibilityVersion.version == lastLTSFCV &&
-    !jsTestOptions().shardMixedBinVersions) {
-    (() => {
-        //
-        // Verify listIndexes and checkShardingIndexes are retried on shard version errors and are
-        // sent with shard versions.
-        //
-
-        // Create a sharded collection with one chunk on shard0.
-        const dbName = "testShardVersions";
-        const collName = "fooShardVersions";
-        const ns = dbName + "." + collName;
-        assert.commandWorked(st.s.adminCommand({enableSharding: dbName}));
-        st.ensurePrimaryShard(dbName, st.shard0.shardName);
-        assert.commandWorked(st.s.adminCommand({shardCollection: ns, key: {x: 1}}));
-
-        const minKeyShardDB = st.rs0.getPrimary().getDB(dbName);
-        assert.commandWorked(minKeyShardDB.setProfilingLevel(2));
-
-        // Refining the shard key should internally retry on a stale epoch error for listIndexes and
-        // succeed.
-        assert.commandWorked(minKeyShardDB.adminCommand({
-            configureFailPoint: "failCommand",
-            mode: {times: 5},
-            data: {
-                errorCode: ErrorCodes.StaleEpoch,
-                failCommands: ["listIndexes"],
-                failInternalCommands: true
-            }
-        }));
-        assert.commandWorked(st.s.getCollection(ns).createIndex({x: 1, y: 1}));
-        assert.commandWorked(st.s.adminCommand({refineCollectionShardKey: ns, key: {x: 1, y: 1}}));
-
-        // Refining the shard key should internally retry on a stale epoch error for
-        // checkShardingIndex and succeed.
-        assert.commandWorked(minKeyShardDB.adminCommand({
-            configureFailPoint: "failCommand",
-            mode: {times: 5},
-            data: {
-                errorCode: ErrorCodes.StaleEpoch,
-                failCommands: ["checkShardingIndex"],
-                failInternalCommands: true
-            }
-        }));
-        assert.commandWorked(st.s.getCollection(ns).createIndex({x: 1, y: 1, z: 1}));
-        assert.commandWorked(
-            st.s.adminCommand({refineCollectionShardKey: ns, key: {x: 1, y: 1, z: 1}}));
-
-        // Verify both commands were sent with shard versions through the profiler.
-        profilerHasAtLeastOneMatchingEntryOrThrow({
-            profileDB: minKeyShardDB,
-            filter: {"command.listIndexes": collName, "command.shardVersion": {"$exists": true}}
-        });
-
-        profilerHasAtLeastOneMatchingEntryOrThrow({
-            profileDB: minKeyShardDB,
-            filter: {"command.checkShardingIndex": ns, "command.shardVersion": {"$exists": true}}
-        });
-
-        // Clean up.
-        assert.commandWorked(minKeyShardDB.setProfilingLevel(0));
-        assert(minKeyShardDB.system.profile.drop());
-    })();
-}
-
 // Assumes the given arrays are sorted by the max field.
 function compareMinAndMaxFields(shardedArr, refinedArr) {
     assert(shardedArr.length && refinedArr.length, tojson(shardedArr) + ", " + tojson(refinedArr));
@@ -853,8 +765,8 @@ function compareBoundaries(conn, shardedNs, refinedNs) {
     const shardedNs = dbName + ".shardedColl";
     const refinedNs = dbName + ".refinedColl";
 
-    assert.commandWorked(st.s.adminCommand({enableSharding: dbName}));
-    st.ensurePrimaryShard(dbName, st.shard0.shardName);
+    assert.commandWorked(
+        st.s.adminCommand({enableSharding: dbName, primaryShard: st.shard0.shardName}));
     assert.commandWorked(st.s.adminCommand({addShardToZone: st.shard0.shardName, zone: 'zone_1'}));
 
     assert.commandWorked(st.s.adminCommand({shardCollection: shardedNs, key: {a: 1, b: 1, c: 1}}));
@@ -887,14 +799,28 @@ function compareBoundaries(conn, shardedNs, refinedNs) {
     compareBoundaries(st.s, shardedNs, refinedNs);
 })();
 
-// For a shard key with nested fields.
+// Make sure split  is correctly disabled for unsplittable collection
+(() => {
+    if (FeatureFlagUtil.isPresentAndEnabled(mongos, "TrackUnshardedCollectionsUponCreation")) {
+        jsTest.log("Make sure refine shard key for unsplittable collection is correctly disabled");
+        const kCollNameUnsplittable = "unsplittable_bar";
+        const kNsNameUnsplittable = kDbName + "." + kCollNameUnsplittable;
+        assert.commandWorked(mongos.getDB(kDbName).runCommand(
+            {createUnsplittableCollection: kCollNameUnsplittable}));
+        assert.commandFailedWithCode(
+            mongos.adminCommand({refineCollectionShardKey: kNsNameUnsplittable, key: {a: 1, b: 1}}),
+            ErrorCodes.NamespaceNotSharded);
+    }
+})();
+
+// For a shard key with nested fields .
 (() => {
     const dbName = "compareDBNested";
     const shardedNs = dbName + ".shardedColl";
     const refinedNs = dbName + ".refinedColl";
 
-    assert.commandWorked(st.s.adminCommand({enableSharding: dbName}));
-    st.ensurePrimaryShard(dbName, st.shard0.shardName);
+    assert.commandWorked(
+        st.s.adminCommand({enableSharding: dbName, primaryShard: st.shard0.shardName}));
     assert.commandWorked(st.s.adminCommand({addShardToZone: st.shard0.shardName, zone: 'zone_1'}));
 
     assert.commandWorked(
@@ -929,4 +855,3 @@ function compareBoundaries(conn, shardedNs, refinedNs) {
 })();
 
 st.stop();
-})();
