@@ -1,10 +1,9 @@
 // This checks to make sure that sharded regex queries behave the same as unsharded regex queries.
 // Note, when the updateOneWithoutShardKey feature flag is enabled, upsert operations with queries
 // that do not match on the entire shard key are successful.
-(function() {
-'use strict';
-
-load("jstests/sharding/updateOne_without_shard_key/libs/write_without_shard_key_test_util.js");
+import {
+    WriteWithoutShardKeyTestUtil
+} from "jstests/sharding/updateOne_without_shard_key/libs/write_without_shard_key_test_util.js";
 
 var st = new ShardingTest({shards: 2});
 
@@ -21,8 +20,8 @@ var collCompound = mongos.getCollection("foo.barCompound");
 var collNested = mongos.getCollection("foo.barNested");
 var collHashed = mongos.getCollection("foo.barHashed");
 
-assert.commandWorked(admin.runCommand({enableSharding: coll.getDB().toString()}));
-st.ensurePrimaryShard(coll.getDB().toString(), st.shard0.shardName);
+assert.commandWorked(
+    admin.runCommand({enableSharding: coll.getDB().toString(), primaryShard: st.shard0.shardName}));
 
 //
 // Split the collection so that "abcde-0" and "abcde-1" go on different shards when possible
@@ -306,5 +305,4 @@ assert.eq(1, collNested.find({updated: true}).itcount());
 assert.commandWorked(collNested.remove({a: {b: /abcde.*/}}));
 assert.eq(2, collNested.find().itcount());
 
-st.stop();
-})();
+st.stop();
\ No newline at end of file
