@@ -1,8 +1,5 @@
 // Test for SERVER-31953 where secondaries crash when replicating an oplog entry where the document
 // identifier in the oplog entry contains a shard key value that contains an undefined value.
-(function() {
-"use strict";
-
 const st = new ShardingTest({mongos: 1, shard: 1, rs: {nodes: 2}});
 const mongosDB = st.s.getDB("test");
 const mongosColl = mongosDB.mycoll;
@@ -26,5 +23,4 @@ assert.commandWorked(mongosColl.update(
 assert.commandWorked(
     mongosColl.remove({}, {writeConcern: {w: 2, wtimeout: ReplSetTest.kDefaultTimeoutMS}}));
 
-st.stop();
-})();
\ No newline at end of file
+st.stop();
\ No newline at end of file
