@@ -3,9 +3,7 @@
  *
  * @tags: [uses_atclustertime]
  */
-(function() {
-"use strict";
-load("jstests/sharding/libs/resharding_test_fixture.js");
+import {ReshardingTest} from "jstests/sharding/libs/resharding_test_fixture.js";
 
 const sourceNs = "reshardingDb.coll";
 
@@ -43,5 +41,4 @@ reshardingTest.withReshardingInBackground(
         assert.commandWorked(mongos.adminCommand({commitReshardCollection: sourceNs}));
     });
 
-reshardingTest.teardown();
-})();
+reshardingTest.teardown();
\ No newline at end of file
