@@ -5,9 +5,7 @@
  *
  * @tags: [uses_atclustertime]
  */
-(function() {
-"use strict";
-load("jstests/sharding/libs/resharding_test_fixture.js");
+import {ReshardingTest} from "jstests/sharding/libs/resharding_test_fixture.js";
 
 const reshardingTest = new ReshardingTest({numDonors: 2, numRecipients: 2, reshardInPlace: true});
 reshardingTest.setup();
@@ -80,5 +78,4 @@ reshardingTest.withReshardingInBackground(
                       ', commit opTs: ' + tojson(commitOperationTS));
     });
 
-reshardingTest.teardown();
-})();
+reshardingTest.teardown();
\ No newline at end of file
