@@ -6,10 +6,7 @@
  *   uses_atclustertime
  * ]
  */
-(function() {
-"use strict";
-
-load("jstests/sharding/libs/resharding_test_fixture.js");
+import {ReshardingTest} from "jstests/sharding/libs/resharding_test_fixture.js";
 
 const reshardingTest = new ReshardingTest({numDonors: 3, numRecipients: 3, reshardInPlace: true});
 
@@ -70,5 +67,4 @@ assert.commandFailedWithCode(execRetryableInsert(lsidList[1], {oldKey: 0, newKey
 assert.commandFailedWithCode(execRetryableInsert(lsidList[2], {oldKey: 100, newKey: -10}),
                              ErrorCodes.IncompleteTransactionHistory);
 
-reshardingTest.teardown();
-})();
+reshardingTest.teardown();
\ No newline at end of file
