@@ -6,10 +6,7 @@
  *   uses_atclustertime
  * ]
  */
-(function() {
-"use strict";
-
-load("jstests/sharding/libs/resharding_test_fixture.js");
+import {ReshardingTest} from "jstests/sharding/libs/resharding_test_fixture.js";
 
 const reshardingTest = new ReshardingTest({numDonors: 2, reshardInPlace: false});
 reshardingTest.setup();
@@ -35,5 +32,4 @@ const newCollInfo = sourceCollection.exists();
 assert.neq(newCollInfo, null, "failed to find sharded collection after resharding");
 assert.neq(newCollInfo.info.uuid, originalCollInfo.info.uuid, {newCollInfo, originalCollInfo});
 
-reshardingTest.teardown();
-})();
+reshardingTest.teardown();
\ No newline at end of file
