@@ -6,9 +6,6 @@
  * ]
  */
 
-(function() {
-"use strict";
-
 const st = new ShardingTest({shards: 1});
 const db = st.getDB("test");
 
@@ -36,5 +33,4 @@ assert.commandFailedWithCode(db.runCommand({
 }),
                              ErrorCodes.BadValue);
 
-st.stop();
-})();
+st.stop();
\ No newline at end of file
