@@ -2,25 +2,26 @@
  * Test RetryableWriteError label in retryable writes and in transactions.
  *
  * @tags: [
+ *   temp_disabled_embedded_router_uncategorized,
  *   uses_transactions,
  * ]
  */
-(function() {
-"use strict";
-
-load("jstests/aggregation/extras/utils.js");
-load("jstests/libs/fail_point_util.js");
+import {anyEq} from "jstests/aggregation/extras/utils.js";
+import {configureFailPoint} from "jstests/libs/fail_point_util.js";
+import {Thread} from "jstests/libs/parallelTester.js";
 
 const dbName = "test";
 const collName = "retryable_write_error_labels";
 const ns = dbName + "." + collName;
+const acceptableErrorsDuringShutdown =
+    [ErrorCodes.InterruptedAtShutdown, ErrorCodes.CallbackCanceled, ErrorCodes.ShutdownInProgress];
 
 // Use ShardingTest because we need to test both mongod and mongos behaviors
 let overrideMaxAwaitTimeMS = {'mode': 'alwaysOn', 'data': {maxAwaitTimeMS: 1000}};
 const st = new ShardingTest({
     config: 1,
     mongos:
-        {s0: {setParameter: "failpoint.overrideMaxAwaitTimeMS=" + tojson(overrideMaxAwaitTimeMS)}},
+        {s0: {setParameter: {"failpoint.overrideMaxAwaitTimeMS": tojson(overrideMaxAwaitTimeMS)}}},
     shards: 1
 });
 
@@ -149,10 +150,11 @@ function testMongosError() {
     const retryableInsertThread = new Thread((mongosHost, dbName, collName) => {
         const mongos = new Mongo(mongosHost);
         const session = mongos.startSession();
+        session.startTransaction();
         return session.getDatabase(dbName).runCommand({
             insert: collName,
             documents: [{a: 0, b: "retryable"}],
-            txnNumber: NumberLong(0),
+            txnNumber: NumberLong(session.getTxnNumber_forTesting()),
         });
     }, st.s.host, dbName, collName);
     retryableInsertThread.start();
@@ -161,9 +163,7 @@ function testMongosError() {
     MongoRunner.stopMongos(st.s);
     try {
         const retryableInsertRes = retryableInsertThread.returnData();
-        checkErrorCode(retryableInsertRes,
-                       [ErrorCodes.InterruptedAtShutdown, ErrorCodes.CallbackCanceled],
-                       false /* isWCError */);
+        checkErrorCode(retryableInsertRes, acceptableErrorsDuringShutdown, false /* isWCError */);
         assertContainRetryableErrorLabel(retryableInsertRes);
     } catch (e) {
         if (!isNetworkError(e)) {
@@ -192,9 +192,8 @@ function testMongosError() {
     MongoRunner.stopMongos(st.s);
     try {
         const nonRetryableInsertRes = nonRetryableInsertThread.returnData();
-        checkErrorCode(nonRetryableInsertRes,
-                       [ErrorCodes.InterruptedAtShutdown, ErrorCodes.CallbackCanceled],
-                       false /* isWCError */);
+        checkErrorCode(
+            nonRetryableInsertRes, acceptableErrorsDuringShutdown, false /* isWCError */);
         assertNotContainErrorLabels(nonRetryableInsertRes);
     } catch (e) {
         if (!isNetworkError(e)) {
@@ -230,9 +229,7 @@ function testMongosError() {
 
     try {
         const commitTxnRes = commitTxnThread.returnData();
-        checkErrorCode(commitTxnRes,
-                       [ErrorCodes.InterruptedAtShutdown, ErrorCodes.CallbackCanceled],
-                       false /* isWCError */);
+        checkErrorCode(commitTxnRes, acceptableErrorsDuringShutdown, false /* isWCError */);
         assertContainRetryableErrorLabel(commitTxnRes);
     } catch (e) {
         if (!isNetworkError(e)) {
@@ -267,9 +264,7 @@ function testMongosError() {
 
     try {
         const abortTxnRes = abortTxnThread.returnData();
-        checkErrorCode(abortTxnRes,
-                       [ErrorCodes.InterruptedAtShutdown, ErrorCodes.CallbackCanceled],
-                       false /* isWCError */);
+        checkErrorCode(abortTxnRes, acceptableErrorsDuringShutdown, false /* isWCError */);
         assertContainRetryableErrorLabel(abortTxnRes);
     } catch (e) {
         if (!isNetworkError(e)) {
@@ -314,4 +309,3 @@ testMongosError();
 st.s.adminCommand({"configureFailPoint": "overrideMaxAwaitTimeMS", "mode": "off"});
 
 st.stop();
-}());
