@@ -2,9 +2,6 @@
  * Test basic retryable write without errors by checking that the resulting collection after the
  * retry is as expected and it does not create additional oplog entries.
  */
-(function() {
-"use strict";
-
 function checkFindAndModifyResult(expected, toCheck) {
     assert.eq(expected.ok, toCheck.ok);
     assert.eq(expected.value, toCheck.value);
@@ -329,12 +326,6 @@ function runFailpointTests(mainConn, priConn) {
     var lsid = UUID();
     var testDb = mainConn.getDB('TestDB');
 
-    if (TestData.configShard) {
-        // TODO SERVER-75821: Workaround for crash when executing the fail point while implicitly
-        // creating a collection in a transaction on the config server.
-        assert.commandWorked(testDb.createCollection("user"));
-    }
-
     // Test connection close (default behaviour). The connection will get closed, but the
     // inserts must succeed
     assert.commandWorked(priConn.adminCommand(
@@ -394,7 +385,7 @@ function runFailpointTests(mainConn, priConn) {
     assert.commandWorked(
         priConn.adminCommand({configureFailPoint: 'onPrimaryTransactionalWrite', mode: 'off'}));
 
-    var writeResult = testDb.runCommand(cmd);
+    writeResult = testDb.runCommand(cmd);
     assert.eq(2, writeResult.nModified);
 
     var collContents = testDb.user.find({}).sort({x: 1}).toArray();
@@ -557,4 +548,3 @@ runFailpointTests(st.s0, st.rs0.getPrimary());
 runMultiTests(st.s0);
 
 st.stop();
-})();
