@@ -18,15 +18,19 @@
  *
  * @tags: [
  *   # SERVER-56565 avoid CS stepdowns, since  an election may trigger a  refresh of stale metadata
- *   #              that form part of the test setup.
+ *   # that form part of the test setup.
  *   does_not_support_stepdowns,
- *  ]
+ *   temp_disabled_embedded_router_uncategorized,
+ * ]
  */
-(function() {
-"use strict";
-
-load('jstests/libs/profiler.js');
-load('jstests/sharding/libs/last_lts_mongos_commands.js');
+import {
+    buildCommandProfile,
+    profilerHasSingleMatchingEntryOrThrow,
+    profilerHasZeroMatchingEntriesOrThrow,
+} from "jstests/libs/profiler.js";
+import {
+    commandsRemovedFromMongosSinceLastLTS
+} from "jstests/sharding/libs/last_lts_mongos_commands.js";
 
 let db = "test";
 let coll = "foo";
@@ -52,6 +56,8 @@ let testCases = {
     _configsvrBalancerStart: {skip: "primary only"},
     _configsvrBalancerStatus: {skip: "primary only"},
     _configsvrBalancerStop: {skip: "primary only"},
+    _configsvrCheckClusterMetadataConsistency: {skip: "internal command"},
+    _configsvrCheckMetadataConsistency: {skip: "internal command"},
     _configsvrClearJumboFlag: {skip: "primary only"},
     _configsvrCommitChunksMerge: {skip: "primary only"},
     _configsvrCommitChunkMigration: {skip: "primary only"},
@@ -79,9 +85,14 @@ let testCases = {
     _mergeAuthzCollections: {skip: "primary only"},
     _migrateClone: {skip: "primary only"},
     _mongotConnPoolStats: {skip: "internal command"},
+    _shardsvrBeginMigrationBlockingOperation: {skip: "internal command"},
+    _shardsvrChangePrimary: {skip: "internal command"},
     _shardsvrCheckMetadataConsistency: {skip: "internal command"},
     _shardsvrCheckMetadataConsistencyParticipant: {skip: "internal command"},
+    _shardsvrCleanupStructuredEncryptionData: {skip: "primary only"},
     _shardsvrCompactStructuredEncryptionData: {skip: "primary only"},
+    _shardsvrCoordinateMultiUpdate: {skip: "primary only"},
+    _shardsvrEndMigrationBlockingOperation: {skip: "internal command"},
     _shardsvrMergeAllChunksOnShard: {skip: "primary only"},
     _shardsvrMovePrimary: {skip: "primary only"},
     _shardsvrMovePrimaryEnterCriticalSection: {skip: "primary only"},
@@ -93,8 +104,10 @@ let testCases = {
     _recvChunkStart: {skip: "primary only"},
     _recvChunkStatus: {skip: "primary only"},
     _transferMods: {skip: "primary only"},
+    abortMoveCollection: {skip: "primary only"},
     abortReshardCollection: {skip: "primary only"},
     abortTransaction: {skip: "primary only"},
+    abortUnshardCollection: {skip: "primary only"},
     addShard: {skip: "primary only"},
     addShardToZone: {skip: "primary only"},
     aggregate: {
@@ -126,23 +139,27 @@ let testCases = {
     applyOps: {skip: "primary only"},
     authSchemaUpgrade: {skip: "primary only"},
     authenticate: {skip: "does not return user data"},
+    autoSplitVector: {skip: "primary only"},
     balancerCollectionStatus: {skip: "primary only"},
     balancerStart: {skip: "primary only"},
     balancerStatus: {skip: "primary only"},
     balancerStop: {skip: "primary only"},
     buildInfo: {skip: "does not return user data"},
-    bulkWrite: {skip: "not yet implemented"},
+    bulkWrite: {skip: "primary only"},
     captrunc: {skip: "primary only"},
+    changePrimary: {skip: "primary only"},
     checkMetadataConsistency: {skip: "primary only"},
     checkShardingIndex: {skip: "primary only"},
     cleanupOrphaned: {skip: "primary only"},
     cleanupReshardCollection: {skip: "primary only"},
+    cleanupStructuredEncryptionData: {skip: "does not return user data"},
     clearJumboFlag: {skip: "primary only"},
     clearLog: {skip: "does not return user data"},
     clone: {skip: "primary only"},
     cloneCollectionAsCapped: {skip: "primary only"},
     clusterAbortTransaction: {skip: "already tested by 'abortTransaction' tests on mongos"},
     clusterAggregate: {skip: "already tested by 'aggregate' tests on mongos"},
+    clusterBulkWrite: {skip: "already tested by 'bulkWrite' tests on mongos"},
     clusterCommitTransaction: {skip: "already tested by 'commitTransaction' tests on mongos"},
     clusterCount: {skip: "already tested by 'count' tests on mongos"},
     clusterDelete: {skip: "already tested by 'delete' tests on mongos"},
@@ -180,6 +197,7 @@ let testCases = {
     createIndexes: {skip: "primary only"},
     createRole: {skip: "primary only"},
     createSearchIndexes: {skip: "primary only"},
+    createUnsplittableCollection: {skip: "primary only"},
     createUser: {skip: "primary only"},
     currentOp: {skip: "does not return user data"},
     dataSize: {skip: "does not return user data"},
@@ -234,6 +252,7 @@ let testCases = {
     getChangeStreamState: {skip: "does not return user data"},
     getClusterParameter: {skip: "does not return user data"},
     getCmdLineOpts: {skip: "does not return user data"},
+    getDatabaseVersion: {skip: "does not return user data"},
     getDefaultRWConcern: {skip: "does not return user data"},
     getDiagnosticData: {skip: "does not return user data"},
     getLog: {skip: "does not return user data"},
@@ -294,6 +313,7 @@ let testCases = {
     mergeAllChunksOnShard: {skip: "primary only"},
     mergeChunks: {skip: "primary only"},
     moveChunk: {skip: "primary only"},
+    moveCollection: {skip: "primary only"},
     movePrimary: {skip: "primary only"},
     moveRange: {skip: "primary only"},
     multicast: {skip: "does not return user data"},
@@ -339,6 +359,7 @@ let testCases = {
     revokeRolesFromUser: {skip: "primary only"},
     rolesInfo: {skip: "primary only"},
     rotateCertificates: {skip: "does not return user data"},
+    rotateFTDC: {skip: "does not return user data"},
     saslContinue: {skip: "primary only"},
     saslStart: {skip: "primary only"},
     sbe: {skip: "internal command"},
@@ -354,6 +375,8 @@ let testCases = {
     setShardVersion: {skip: "does not return user data"},
     setChangeStreamState: {skip: "does not return user data"},
     setClusterParameter: {skip: "does not return user data"},
+    setQuerySettings: {skip: "does not return user data"},
+    removeQuerySettings: {skip: "does not return user data"},
     setUserWriteBlockMode: {skip: "primary only"},
     shardCollection: {skip: "primary only"},
     shardingState: {skip: "does not return user data"},
@@ -366,6 +389,7 @@ let testCases = {
     startRecordingTraffic: {skip: "does not return user data"},
     startSession: {skip: "does not return user data"},
     stopRecordingTraffic: {skip: "does not return user data"},
+    sysprofile: {skip: "internal command"},
     testDeprecation: {skip: "does not return user data"},
     testDeprecationInVersion2: {skip: "does not return user data"},
     testInternalTransactions: {skip: "primary only"},
@@ -375,6 +399,9 @@ let testCases = {
     top: {skip: "does not return user data"},
     transitionFromDedicatedConfigServer: {skip: "primary only"},
     transitionToDedicatedConfigServer: {skip: "primary only"},
+    transitionToShardedCluster: {skip: "primary only"},
+    unshardCollection: {skip: "primary only"},
+    untrackUnshardedCollection: {skip: "primary only"},
     update: {skip: "primary only"},
     updateRole: {skip: "primary only"},
     updateSearchIndex: {skip: "primary only"},
@@ -384,7 +411,7 @@ let testCases = {
     validate: {skip: "does not return user data"},
     validateDBMetadata: {skip: "does not return user data"},
     waitForFailPoint: {skip: "does not return user data"},
-    waitForOngoingChunkSplits: {skip: "does not return user data"},
+    getShardingReady: {skip: "does not return user data"},
     whatsmyuri: {skip: "does not return user data"}
 };
 
@@ -394,7 +421,6 @@ commandsRemovedFromMongosSinceLastLTS.forEach(function(cmd) {
 
 let scenarios = {
     dropRecreateAsUnshardedOnSameShard: function(staleMongos, freshMongos, test, commandProfile) {
-        let primaryShardPrimary = st.rs0.getPrimary();
         let primaryShardSecondary = st.rs0.getSecondary();
 
         // Drop and recreate the collection.
@@ -446,7 +472,6 @@ let scenarios = {
         }
     },
     dropRecreateAsShardedOnSameShard: function(staleMongos, freshMongos, test, commandProfile) {
-        let primaryShardPrimary = st.rs0.getPrimary();
         let primaryShardSecondary = st.rs0.getSecondary();
 
         // Drop and recreate the collection as sharded.
@@ -503,8 +528,7 @@ let scenarios = {
             });
         }
     },
-    dropRecreateAsUnshardedOnDifferentShard: function(
-        staleMongos, freshMongos, test, commandProfile) {
+    dropRecreateAsUnshardedOnDifferentShard: function() {
         // There is no way to drop and recreate the collection as unsharded on a *different*
         // shard without calling movePrimary, and it is known that a stale mongos will not
         // refresh its notion of the primary shard after it loads it once.
@@ -512,7 +536,6 @@ let scenarios = {
     dropRecreateAsShardedOnDifferentShard: function(
         staleMongos, freshMongos, test, commandProfile) {
         let donorShardSecondary = st.rs0.getSecondary();
-        let recipientShardPrimary = st.rs1.getPrimary();
         let recipientShardSecondary = st.rs1.getSecondary();
 
         // Drop and recreate the collection as sharded, and move the chunk to the other shard.
@@ -609,8 +632,8 @@ for (let command of commands) {
         jsTest.log("testing command " + tojson(command) + " under scenario " + scenario);
 
         // Each scenario starts with a sharded collection with shard0 as the primary shard.
-        assert.commandWorked(staleMongos.adminCommand({enableSharding: db}));
-        st.ensurePrimaryShard(db, st.shard0.shardName);
+        assert.commandWorked(
+            staleMongos.adminCommand({enableSharding: db, primaryShard: st.shard0.shardName}));
         assert.commandWorked(staleMongos.adminCommand({shardCollection: nss, key: {x: 1}}));
 
         // We do this because we expect staleMongos to see that the collection is sharded, which
@@ -651,4 +674,3 @@ for (let command of commands) {
 }
 
 st.stop();
-})();
