@@ -1,15 +1,13 @@
 // shard6.js
-(function() {
-"use strict";
-
-load("jstests/sharding/updateOne_without_shard_key/libs/write_without_shard_key_test_util.js");
+import {
+    WriteWithoutShardKeyTestUtil
+} from "jstests/sharding/updateOne_without_shard_key/libs/write_without_shard_key_test_util.js";
 
 var summary = "";
 
 var s = new ShardingTest({name: "shard6", shards: 2});
 
-s.adminCommand({enablesharding: "test"});
-s.ensurePrimaryShard('test', s.shard1.shardName);
+s.adminCommand({enablesharding: "test", primaryShard: s.shard1.shardName});
 s.adminCommand({shardcollection: "test.data", key: {num: 1}});
 
 var version = s.getDB("admin").runCommand({buildinfo: 1}).versionArray;
@@ -122,5 +120,4 @@ assert.throws(function() {
     s.adminCommand({enablesharding: "admin"});
 });
 
-s.stop();
-})();
+s.stop();
\ No newline at end of file
