@@ -1,13 +1,19 @@
 /**
  * Tests that shard removal triggers an update of the catalog cache so that routers don't continue
  * to target shards that have been removed.
+ * @tags: [
+ *   temp_disabled_embedded_router_uncategorized,
+ * ]
  */
-(function() {
-'use strict';
+import {
+    moveDatabaseAndUnshardedColls
+} from "jstests/sharding/libs/move_database_and_unsharded_coll_helper.js";
+import {removeShard} from "jstests/sharding/libs/remove_shard_util.js";
 
 // Checking UUID consistency involves talking to shards, but this test shuts down shards.
 TestData.skipCheckingUUIDsConsistentAcrossCluster = true;
 TestData.skipCheckingIndexesConsistentAcrossCluster = true;
+TestData.skipCheckShardFilteringMetadata = true;
 
 // TODO SERVER-50144 Remove this and allow orphan checking.
 // This test calls removeShard which can leave docs in config.rangeDeletions in state "pending",
@@ -38,8 +44,8 @@ const dbName = 'TestDB';
     let router0ShardedColl = st.s0.getDB(dbName)[shardedCollName];
     let router1ShardedColl = st.s1.getDB(dbName)[shardedCollName];
 
-    assert.commandWorked(st.s0.adminCommand({enableSharding: dbName}));
-    st.ensurePrimaryShard(dbName, st.shard1.shardName);
+    assert.commandWorked(
+        st.s0.adminCommand({enableSharding: dbName, primaryShard: st.shard1.shardName}));
     assert.commandWorked(st.s0.adminCommand({shardCollection: shardedCollNs, key: {_id: 1}}));
 
     // Make sure data is inserted into shard0
@@ -63,28 +69,21 @@ const dbName = 'TestDB';
     st.startBalancer();
 
     // Remove shard0.
-    assert.soon(() => {
-        const removeRes = st.s0.adminCommand({removeShard: st.shard0.shardName});
-        if (!removeRes.ok && removeRes.code === ErrorCodes.ShardNotFound) {
-            // If the config server primary steps down right after removing the config.shards doc
-            // for the shard but before responding with "state": "completed", the mongos would retry
-            // the _configsvrRemoveShard command against the new config server primary, which would
-            // not find the removed shard in its ShardRegistry if it has done a ShardRegistry reload
-            // after the config.shards doc for the shard was removed. This would cause the command
-            // to fail with ShardNotFound.
-            return true;
-        }
-        return removeRes.state === 'completed';
-    });
+    removeShard(st, st.shard0.shardName);
 
-    // Stop the replica set so that future requests to this shard will be unsuccessful.
-    st.rs0.stopSet();
+    // Stop the replica set so that future requests to this shard will be unsuccessful. Skip this
+    // step for a config shard, since the config server must be up for the second router to
+    // refresh. The default read concern is local, so the router should eventually target a shard
+    // with chunks.
+    if (!TestData.configShard) {
+        st.rs0.stopSet();
+    }
 
     // Ensure that s1, the router which did not run removeShard, eventually stops targeting chunks
     // for the sharded collection which previously resided on a shard that no longer exists.
     assert.soon(() => {
         try {
-            const count = router1ShardedColl.count({_id: 1});
+            router1ShardedColl.count({_id: 1});
             return true;
         } catch (e) {
             print(e);
@@ -102,7 +101,7 @@ const dbName = 'TestDB';
  * 1. Create 2 shards and 2 routers. Make shard0 the primary shard for a database.
  * 2. Put data for an unsharded collection on shard0.
  * 3. Ensure both routers have up-to-date routing info.
- * 4. movePrimary for the database to shard1.
+ * 4. Move all data of the database to shard1.
  * 4. Remove shard0 by sending removeShard through router 0.
  * 5. Send a query through router 1 to target the sharded and unsharded collections. This should
  *    correctly target shard1.
@@ -112,15 +111,14 @@ const dbName = 'TestDB';
         "Test that entries for a database whose original primary shard gets removed are correctly invalidated in a router's catalog cache.");
 
     const unshardedCollName = 'UnshardedColl';
-    const unshardedCollNs = dbName + '.' + unshardedCollName;
 
     var st = new ShardingTest({shards: 2, mongos: 2, other: {enableBalancer: true}});
 
     let router0UnshardedColl = st.s0.getDB(dbName)[unshardedCollName];
     let router1UnshardedColl = st.s1.getDB(dbName)[unshardedCollName];
 
-    assert.commandWorked(st.s0.adminCommand({enableSharding: dbName}));
-    st.ensurePrimaryShard(dbName, st.shard0.shardName);
+    assert.commandWorked(
+        st.s0.adminCommand({enableSharding: dbName, primaryShard: st.shard0.shardName}));
 
     // Insert some documents into the unsharded collection whose primary is the to-be-removed
     // shard0.
@@ -131,32 +129,25 @@ const dbName = 'TestDB';
     assert.eq(1, router0UnshardedColl.find({}).itcount());
     assert.eq(1, router1UnshardedColl.find({}).itcount());
 
-    // Call movePrimary for the database so that shard0 can be removed.
-    assert.commandWorked(st.s0.adminCommand({movePrimary: dbName, to: st.shard1.shardName}));
+    moveDatabaseAndUnshardedColls(st.s0.getDB(dbName), st.shard1.shardName);
 
     // Remove shard0. We need assert.soon since chunks in the sessions collection may need to be
     // migrated off by the balancer.
-    assert.soon(() => {
-        const removeRes = st.s0.adminCommand({removeShard: st.shard0.shardName});
-        if (!removeRes.ok && removeRes.code === ErrorCodes.ShardNotFound) {
-            // If the config server primary steps down after removing the config.shards doc for the
-            // shard being removed but before completing the _configsvrRemoveShard command, the
-            // mongos would retry the command on the new config server primary which would not find
-            // the removed shard in its ShardRegistry causing the command to fail with
-            // ShardNotFound.
-            return true;
-        }
-        return removeRes.state === 'completed';
-    });
+    removeShard(st, st.shard0.shardName);
 
-    // Stop the replica set so that future requests to this shard will be unsuccessful.
-    st.rs0.stopSet();
+    // Stop the replica set so that future requests to this shard will be unsuccessful. Skip this
+    // step for a config shard, since the config server must be up for the second router to
+    // refresh. The default read concern is local, so the router should eventually target a shard
+    // with chunks.
+    if (!TestData.configShard) {
+        st.rs0.stopSet();
+    }
 
     // Ensure that s1, the router which did not run removeShard, eventually stops targeting data for
     // the unsharded collection which previously had as primary a shard that no longer exists.
     assert.soon(() => {
         try {
-            const count = router1UnshardedColl.count({_id: 1});
+            router1UnshardedColl.count({_id: 1});
             return true;
         } catch (e) {
             print(e);
@@ -165,4 +156,3 @@ const dbName = 'TestDB';
     });
     st.stop();
 })();
-})();
