@@ -6,9 +6,6 @@
  * ]
  */
 
-(function() {
-'use strict';
-
 function testShardedDataAggregationStage() {
     // Get all expected results in obj format
     const fooResults = fooColl.aggregate([{$collStats: {storageStats: {}}}]).toArray();
@@ -71,28 +68,64 @@ function testShardedDataAggregationStage() {
     }
 }
 
+// Skip orphans check because the range deleter is disabled in this test
+TestData.skipCheckOrphans = true;
+
 // Configure initial sharding cluster
-const st = new ShardingTest({shards: 2});
+// Disable the range deleter to be able to test `numOrphanDocs`
+const st = new ShardingTest({shards: 3, rs: {setParameter: {disableResumableRangeDeleter: true}}});
 const mongos = st.s;
 
 const ns1 = "test.foo";
 const ns2 = "bar.baz";
-const nsEmptyColl = "test.emptyColl";
+const ns3 = "test.unsharded";
+const ns4 = "bar.unsharded";
+const ns5 = "test.coll1";
+const ns6 = "bar.coll2";
+const ns7 = "test.timeseriesColl";
+const ns8 = "test.emptyColl";
 
 const adminDb = mongos.getDB("admin");
 const testDb = mongos.getDB("test");
 const barDb = mongos.getDB("bar");
 const fooColl = testDb.getCollection("foo");
 const bazColl = barDb.getCollection("baz");
+const unshardedColl1 = testDb.getCollection("unsharded");
+const unshardedColl2 = barDb.getCollection("unsharded");
+const barColl2 = barDb.getCollection("coll2");
 const timeseriesColl = testDb.getCollection("timeseriesColl");
+const emptyColl = testDb.getCollection("emptyColl");
 
 st.adminCommand({enablesharding: testDb.getName(), primaryShard: st.shard1.shardName});
+st.adminCommand({enablesharding: barDb.getName(), primaryShard: st.shard1.shardName});
 
 st.adminCommand({shardcollection: ns1, key: {skey: 1}});
 st.adminCommand({shardcollection: ns2, key: {skey: 1}});
-st.adminCommand({shardcollection: nsEmptyColl, key: {skey: 1}});
-st.adminCommand(
-    {shardcollection: timeseriesColl.getFullName(), timeseries: {timeField: "ts"}, key: {ts: 1}});
+st.adminCommand({shardcollection: ns5, key: {skey: 1}});
+st.adminCommand({shardcollection: ns6, key: {skey: 1}});
+st.adminCommand({shardcollection: ns7, timeseries: {timeField: "ts"}, key: {ts: 1}});
+st.adminCommand({shardcollection: ns8, key: {skey: 1}});
+
+// We create and move the unsharded collections to make sure they are tracked
+assert.commandWorked(testDb.runCommand({create: "unsharded"}));
+assert.commandWorked(barDb.runCommand({create: "unsharded"}));
+
+// We can only move collection under FCV v8.0
+const isMultiversion =
+    jsTest.options().shardMixedBinVersions || jsTest.options().useRandomBinVersionsWithinReplicaSet;
+if (!isMultiversion) {
+    assert.commandWorked(
+        barDb.adminCommand({moveCollection: "bar.unsharded", toShard: st.shard0.shardName}));
+    assert.commandWorked(
+        testDb.adminCommand({moveCollection: "test.unsharded", toShard: st.shard0.shardName}));
+}
+
+// Insert data to validate the aggregation stage
+for (let i = 0; i < 6; i++) {
+    assert.commandWorked(fooColl.insert({skey: i}));
+    assert.commandWorked(bazColl.insert({skey: (i + 5)}));
+    assert.commandWorked(barColl2.insert({skey: i}));
+}
 
 assert.commandWorked(timeseriesColl.insertOne({
     "metadata": {"sensorId": 5578, "type": "temperature"},
@@ -100,12 +133,6 @@ assert.commandWorked(timeseriesColl.insertOne({
     "temp": 12
 }));
 
-// Insert data to validate the aggregation stage
-for (let i = 0; i < 6; i++) {
-    assert.commandWorked(fooColl.insert({skey: i}));
-    assert.commandWorked(bazColl.insert({skey: (i + 5)}));
-}
-
 // Test before chunk migration
 testShardedDataAggregationStage();
 
@@ -114,6 +141,13 @@ st.adminCommand({moveChunk: ns1, find: {skey: 2}, to: st.shard0.name, _waitForDe
 st.adminCommand({split: ns2, middle: {skey: 7}});
 st.adminCommand({moveChunk: ns2, find: {skey: 7}, to: st.shard0.name, _waitForDelete: true});
 
+// Moving all the chunks outside the db primary shard for both collections.
+assert.commandWorked(st.s.adminCommand({moveChunk: ns5, find: {skey: 1}, to: st.shard0.shardName}));
+assert.commandWorked(st.s.adminCommand({moveChunk: ns6, find: {skey: 1}, to: st.shard0.shardName}));
+// Moving the chunk to another shard that is not the db primary to verify that shards that own
+// orphans but are not the primary are also shown
+assert.commandWorked(st.s.adminCommand({moveChunk: ns6, find: {skey: 1}, to: st.shard2.shardName}));
+
 // Test after chunk migration
 testShardedDataAggregationStage();
 
@@ -179,6 +213,47 @@ assert.neq(
     0,
     adminDb.aggregate([{$shardedDataDistribution: {}}, {$match: {shards: {$size: 2}}}]).itcount());
 
+// Test that verifies unsharded collections are not shown by $shardedDataDistribution
+assert.eq(0,
+          adminDb.aggregate([{$shardedDataDistribution: {}}, {$match: {ns: {$in: [ns3, ns4]}}}])
+              .itcount());
+
+// Test that verifies $shardedDataDistribution returns no information about a shard that does not
+// own any chunks or orphans.
+assert.eq(
+    1,
+    adminDb
+        .aggregate(
+            [{$shardedDataDistribution: {}}, {$match: {ns: ns5}}, {$unwind: {path: "$shards"}}])
+        .itcount());
+
+// Test that verifies $shardedDataDistribution still returns information about the a shard that
+// owned a chunk in the past and now only owns orphans.
+assert.eq(
+    3,
+    adminDb
+        .aggregate(
+            [{$shardedDataDistribution: {}}, {$match: {ns: ns6}}, {$unwind: {path: "$shards"}}])
+        .itcount());
+
+// Test that verifies $shardedDataDistribution returns no information about a shard that does not
+// own any chunks or orphans.
+assert.eq(
+    1,
+    adminDb
+        .aggregate(
+            [{$shardedDataDistribution: {}}, {$match: {ns: ns5}}, {$unwind: {path: "$shards"}}])
+        .itcount());
+
+// Test that verifies $shardedDataDistribution still returns information about the a shard that
+// owned a chunk in the past and now only owns orphans.
+assert.eq(
+    3,
+    adminDb
+        .aggregate(
+            [{$shardedDataDistribution: {}}, {$match: {ns: ns6}}, {$unwind: {path: "$shards"}}])
+        .itcount());
+
 // Test that verifies that the fields returned for timeseries collections are correct.
 assert.eq(1,
           adminDb
@@ -189,7 +264,7 @@ assert.eq(1,
                       $match: {
                           $and: [
                               {"shards.numOwnedDocuments": {$eq: 1}},
-                              {"shards.ownedSizeBytes": {$gt: 0}},
+                              {"shards.ownedSizeBytes": {$eq: 435}},
                               {"shards.orphanedSizeBytes": {$eq: 0}}
                           ]
                       }
@@ -197,13 +272,13 @@ assert.eq(1,
               ])
               .itcount());
 
-// Test that vierfies the corresponding fields returned by $shardedDataDistribution are properly set
-// to 0 on an empty collection.
+// Verify that the corresponding fields returned by $shardedDataDistribution are properly set to 0
+// on an empty collection
 assert.eq(1,
           adminDb
               .aggregate([
                   {$shardedDataDistribution: {}},
-                  {$match: {ns: nsEmptyColl}},
+                  {$match: {ns: ns8}},
                   {
                       $match: {
                           $and: [
@@ -216,6 +291,7 @@ assert.eq(1,
                   }
               ])
               .itcount());
+
 st.stop();
 
 // Test that verifies the behavior in unsharded deployments
@@ -231,6 +307,4 @@ const response2 = assert.commandFailedWithCode(
     6789101);
 assert.neq(
     -1, response2.errmsg.indexOf("The $shardedDataDistribution stage can only be run on mongoS"));
-
 rsTest.stopSet();
-})();
