@@ -6,9 +6,6 @@
  * ]
  */
 
-(function() {
-'use strict';
-
 // Test privileges
 function testPrivileges() {
     // Create new role with the exact privileges to execute $shardedDataDistribution
@@ -72,5 +69,4 @@ for (let i = 0; i < 6; i++) {
 
 testPrivileges();
 
-st.stop();
-})();
+st.stop();
\ No newline at end of file
