@@ -1,18 +1,13 @@
 /**
  * Tests that the global indexes API correctly creates and drops an index from the catalog.
  *
- * TODO SERVER-75274: Enable with a config shard.
  * @tags: [
  *   multiversion_incompatible,
  *   featureFlagGlobalIndexesShardingCatalog,
- *   config_shard_incompatible,
  *   does_not_support_stepdowns,
  * ]
  */
 
-(function() {
-'use strict';
-
 function registerIndex(rs, nss, pattern, name, uuid) {
     rs.getPrimary().adminCommand({
         _shardsvrRegisterIndex: nss,
@@ -39,6 +34,7 @@ function unregisterIndex(rs, nss, name, uuid) {
 // Assert a secondary contains the number of documents specified by count.
 function assertSecondaryCount(rst, collection, query, count) {
     let matchCount = false;
+    rst.awaitReplication();
     rst.getSecondaries().forEach(function(secondary) {
         matchCount =
             matchCount || secondary.getCollection(collection).countDocuments(query) == count;
@@ -119,14 +115,26 @@ assert.eq(0, st.rs1.getPrimary().getCollection(shardIndexCatalog).countDocuments
 }));
 
 jsTestLog("Ensure we committed in the right collection.");
-assert.eq(0, st.configRS.getPrimary().getCollection(shardIndexCatalog).countDocuments({
-    collectionUUID: collectionUUID,
-    name: index1Name
-}));
-assert.eq(0, st.rs0.getPrimary().getCollection(configsvrIndexCatalog).countDocuments({
-    collectionUUID: collectionUUID,
-    name: index1Name
-}));
+if (TestData.configShard) {
+    // The config server is shard0 in config shard mode, so it should have both collections.
+    assert.eq(1, st.configRS.getPrimary().getCollection(shardIndexCatalog).countDocuments({
+        collectionUUID: collectionUUID,
+        name: index1Name
+    }));
+    assert.eq(1, st.rs0.getPrimary().getCollection(configsvrIndexCatalog).countDocuments({
+        collectionUUID: collectionUUID,
+        name: index1Name
+    }));
+} else {
+    assert.eq(0, st.configRS.getPrimary().getCollection(shardIndexCatalog).countDocuments({
+        collectionUUID: collectionUUID,
+        name: index1Name
+    }));
+    assert.eq(0, st.rs0.getPrimary().getCollection(configsvrIndexCatalog).countDocuments({
+        collectionUUID: collectionUUID,
+        name: index1Name
+    }));
+}
 assert.eq(0, st.rs1.getPrimary().getCollection(configsvrIndexCatalog).countDocuments({
     collectionUUID: collectionUUID,
     name: index1Name
@@ -179,14 +187,26 @@ assertSecondaryCount(
     st.rs1, shardIndexCatalog, {collectionUUID: collectionUUID, name: index2Name}, 1);
 
 jsTestLog("Check we didn't commit in a wrong collection.");
-assert.eq(0, st.configRS.getPrimary().getCollection(shardIndexCatalog).countDocuments({
-    collectionUUID: collectionUUID,
-    name: index2Name
-}));
-assert.eq(0, st.rs0.getPrimary().getCollection(configsvrIndexCatalog).countDocuments({
-    collectionUUID: collectionUUID,
-    name: index2Name
-}));
+if (TestData.configShard) {
+    // The config server is shard0 in config shard mode, so it should have both collections.
+    assert.eq(1, st.configRS.getPrimary().getCollection(shardIndexCatalog).countDocuments({
+        collectionUUID: collectionUUID,
+        name: index2Name
+    }));
+    assert.eq(1, st.rs0.getPrimary().getCollection(configsvrIndexCatalog).countDocuments({
+        collectionUUID: collectionUUID,
+        name: index2Name
+    }));
+} else {
+    assert.eq(0, st.configRS.getPrimary().getCollection(shardIndexCatalog).countDocuments({
+        collectionUUID: collectionUUID,
+        name: index2Name
+    }));
+    assert.eq(0, st.rs0.getPrimary().getCollection(configsvrIndexCatalog).countDocuments({
+        collectionUUID: collectionUUID,
+        name: index2Name
+    }));
+}
 assert.eq(0, st.rs1.getPrimary().getCollection(configsvrIndexCatalog).countDocuments({
     collectionUUID: collectionUUID,
     name: index2Name
@@ -616,4 +636,3 @@ assert.eq(2, st.rs0.getPrimary().getCollection(shardIndexCatalog).countDocuments
 }));
 
 st.stop();
-})();
