@@ -7,12 +7,12 @@
  * ]
  */
 
-(function() {
-"use strict";
-
-load("jstests/libs/global_snapshot_reads_util.js");
-load("jstests/sharding/libs/sharded_transactions_helpers.js");
-load("jstests/sharding/libs/find_chunks_util.js");
+import {FeatureFlagUtil} from "jstests/libs/feature_flag_util.js";
+import {SnapshotReadsTest} from "jstests/libs/global_snapshot_reads_util.js";
+import {findChunksUtil} from "jstests/sharding/libs/find_chunks_util.js";
+import {
+    flushRoutersAndRefreshShardMetadata
+} from "jstests/sharding/libs/sharded_transactions_helpers.js";
 
 const nodeOptions = {
     // Set a large snapshot window of 10 minutes for the test.
@@ -31,10 +31,7 @@ function setUpAllScenarios(st) {
 
 let shardingScenarios = {
     singleShard: {
-        // Can't test unsharded collections because SERVER-86583 makes reads on unsharded collection
-        // on sharded clusters throw conflicts when reading at a point-in-time earlier than the its
-        // latest local catalog ddl.
-        compatibleCollections: [shardedCollName],
+        compatibleCollections: [shardedCollName, unshardedCollName],
         setUp: function() {
             const st = new ShardingTest({
                 mongos: 1,
@@ -148,6 +145,16 @@ for (let [scenarioName, scenario] of Object.entries(shardingScenarios)) {
             }
         }
 
+        // TODO SERVER-77915: remove once 8.0 becomes last-lts
+        if (!FeatureFlagUtil.isPresentAndEnabled(st.s, "TrackUnshardedCollectionsUponCreation") &&
+            collName == unshardedCollName) {
+            // Skip testing with untracked unsharded collections because reads at a point-in-time
+            // earlier than the latest catalog change (including index DDL) will throw conflicts.
+            jsTestLog('Skip unsharded collection scenario');
+            st.stop();
+            return;
+        }
+
         // Pass the same DB handle as "primaryDB" and "secondaryDB" params; the test functions will
         // send readPreference to mongos to target primary/secondary shard servers.
         let db = st.s.getDB(dbName);
@@ -168,4 +175,3 @@ for (let [scenarioName, scenario] of Object.entries(shardingScenarios)) {
         st.stop();
     });
 }
-})();
