@@ -1,9 +1,5 @@
-
-(function() {
-"use strict";
-
-load("jstests/libs/feature_flag_util.js");
-load('jstests/sharding/libs/remove_shard_util.js');  // For removeShard.
+import {FeatureFlagUtil} from "jstests/libs/feature_flag_util.js";
+import {removeShard} from "jstests/sharding/libs/remove_shard_util.js";
 
 const st = new ShardingTest({
     shards: 3,
@@ -18,6 +14,11 @@ const shard0 = st.shard0.shardName;
 const shard1 = st.shard1.shardName;
 const shard2 = st.shard2.shardName;
 
+// TODO SERVER-77915 Remove checkUnsplittableMetadata once 8.0 becomes last LTS. Update the test as
+// this variable is now always "true"
+const checkUnsplittableMetadata = FeatureFlagUtil.isPresentAndEnabled(
+    st.shard0.getDB('admin'), "TrackUnshardedCollectionsUponCreation");
+
 function getInfoFromConfigDatabases(dbName) {
     const configDBsQueryResults = configDB.databases.find({_id: dbName}).toArray();
     if (configDBsQueryResults.length === 0) {
@@ -55,7 +56,7 @@ function getLatestPlacementInfoFor(namespace) {
     return placementQueryResults[0];
 }
 
-function getValidatedPlacementInfoForDB(dbName, compareTimestamps = true) {
+function getValidatedPlacementInfoForDB(dbName) {
     const configDBInfo = getInfoFromConfigDatabases(dbName);
     const dbPlacementInfo = getLatestPlacementInfoFor(dbName);
     assert.neq(null, configDBInfo);
@@ -64,9 +65,7 @@ function getValidatedPlacementInfoForDB(dbName, compareTimestamps = true) {
     // config.databases.
     assert.sameMembers([configDBInfo.primary], dbPlacementInfo.shards);
 
-    if (compareTimestamps) {
-        assert(timestampCmp(configDBInfo.version.timestamp, dbPlacementInfo.timestamp) === 0);
-    }
+    assert(timestampCmp(configDBInfo.version.timestamp, dbPlacementInfo.timestamp) === 0);
 
     // No UUID field for DB namespaces
     assert.eq(undefined, dbPlacementInfo.uuid);
@@ -106,8 +105,7 @@ function testShardCollection(dbName, collName) {
     const nss = dbName + '.' + collName;
 
     // Shard the collection. Ensure enough chunks to cover all shards.
-    assert.commandWorked(
-        st.s.adminCommand({shardCollection: nss, key: {_id: "hashed"}, numInitialChunks: 20}));
+    assert.commandWorked(st.s.adminCommand({shardCollection: nss, key: {_id: "hashed"}}));
 
     // Verify that a consistent document has been added to config.placementHistory and that its list
     // of shards matches the current content of config.shards
@@ -198,9 +196,7 @@ function testMovePrimary(dbName, fromPrimaryShardName, toPrimaryShardName) {
     assert.commandWorked(st.s.adminCommand({movePrimary: dbName, to: toPrimaryShardName}));
 
     // Verify that the new primary shard is the one specified in the command.
-    // Suppress comparing timestamp assertion in v7.0. The issue is not severe and placementInfo has
-    // no practical use in v7.0. See SERVER-92842.
-    const newDbInfo = getValidatedPlacementInfoForDB(dbName, false /*compareTimestamps*/);
+    const newDbInfo = getValidatedPlacementInfoForDB(dbName);
     assert.sameMembers(newDbInfo.shards, [toPrimaryShardName]);
 }
 
@@ -229,13 +225,17 @@ function testDropCollection() {
     assert.commandWorked(db.runCommand({drop: collName}));
     assert.eq(numHistoryEntriesAfterFirstDrop, configDB.placementHistory.count({nss: nss}));
 
-    // Verify that no records get added in case an unsharded collection gets dropped
+    // Verify that records get added in case an unsharded collection gets dropped as well
     const unshardedCollName = 'unshardedColl';
     assert.commandWorked(db.createCollection(unshardedCollName));
 
     assert.commandWorked(db.runCommand({drop: unshardedCollName}));
 
-    assert.eq(0, configDB.placementHistory.count({nss: dbName + '.' + unshardedCollName}));
+    if (checkUnsplittableMetadata) {
+        assert.eq(2, configDB.placementHistory.count({nss: dbName + '.' + unshardedCollName}));
+    } else {
+        assert.eq(0, configDB.placementHistory.count({nss: dbName + '.' + unshardedCollName}));
+    }
 }
 
 function testRenameCollection() {
@@ -286,18 +286,35 @@ function testRenameCollection() {
     assert.sameMembers(initialPlacementForOldColl.shards,
                        targetCollPlacementInfoWhenRenamed.shards);
 
-    jsTest.log(
-        'Testing that no placement entries are added by rename() for unsharded collections involved in the DDL');
-    const unshardedOldCollName = 'unshardedOld';
-    const unshardedTargetCollName = 'unshardedTarget';
-    assert.commandWorked(db.createCollection(unshardedOldCollName));
-    assert.commandWorked(db.createCollection(unshardedTargetCollName));
+    if (checkUnsplittableMetadata) {
+        jsTest.log(
+            'Testing that placement entries are added by rename() for unsharded collections involved in the DDL');
+        const unshardedOldCollName = 'unshardedOld';
+        const unshardedTargetCollName = 'unshardedTarget';
+        assert.commandWorked(db.createCollection(unshardedOldCollName));
+        assert.commandWorked(db.createCollection(unshardedTargetCollName));
 
-    assert.commandWorked(
-        db[unshardedOldCollName].renameCollection(unshardedTargetCollName, true /*dropTarget*/));
+        assert.commandWorked(db[unshardedOldCollName].renameCollection(unshardedTargetCollName,
+                                                                       true /*dropTarget*/));
 
-    assert.eq(0, configDB.placementHistory.count({nss: dbName + '.' + unshardedOldCollName}));
-    assert.eq(0, configDB.placementHistory.count({nss: dbName + '.' + unshardedTargetCollName}));
+        assert.eq(2, configDB.placementHistory.count({nss: dbName + '.' + unshardedOldCollName}));
+        assert.eq(3,
+                  configDB.placementHistory.count({nss: dbName + '.' + unshardedTargetCollName}));
+    } else {
+        jsTest.log(
+            'Testing that no placement entries are added by rename() for unsharded collections involved in the DDL');
+        const unshardedOldCollName = 'unshardedOld';
+        const unshardedTargetCollName = 'unshardedTarget';
+        assert.commandWorked(db.createCollection(unshardedOldCollName));
+        assert.commandWorked(db.createCollection(unshardedTargetCollName));
+
+        assert.commandWorked(db[unshardedOldCollName].renameCollection(unshardedTargetCollName,
+                                                                       true /*dropTarget*/));
+
+        assert.eq(0, configDB.placementHistory.count({nss: dbName + '.' + unshardedOldCollName}));
+        assert.eq(0,
+                  configDB.placementHistory.count({nss: dbName + '.' + unshardedTargetCollName}));
+    }
 }
 
 function testDropDatabase(dbName, primaryShardName) {
@@ -333,8 +350,13 @@ function testDropDatabase(dbName, primaryShardName) {
     assert(timestampCmp(initialShardedCollPlacementInfo.timestamp,
                         finalShardedCollPlacementInfo.timestamp) < 0);
 
-    // ...And that unshardedCollName stays untracked.
-    assert.eq(null, getLatestPlacementInfoFor(unshardedCollNss));
+    if (checkUnsplittableMetadata) {
+        // ...And that unshardedCollName is also tracked.
+        assert.neq(null, getLatestPlacementInfoFor(unshardedCollNss));
+    } else {
+        // ...And that unshardedCollName stays untracked.
+        assert.eq(null, getLatestPlacementInfoFor(unshardedCollNss));
+    }
 }
 
 function testReshardCollection() {
@@ -398,7 +420,12 @@ function testAddShard() {
     assert.commandWorked(st.s.adminCommand({addShard: newReplicaSet.getURL(), name: newShardName}));
 
     for (const dbName of dbsOnNewReplicaSet) {
-        assert.eq(null, getLatestPlacementInfoFor(dbName + '.' + preExistingCollName));
+        if (checkUnsplittableMetadata) {
+            getValidatedPlacementInfoForCollection(
+                dbName, preExistingCollName, [newShardName], true);
+        } else {
+            assert.eq(null, getLatestPlacementInfoFor(dbName + '.' + preExistingCollName));
+        }
         const dbPlacementEntry = getValidatedPlacementInfoForDB(dbName);
         assert.sameMembers([newShardName], dbPlacementEntry.shards);
     }
@@ -411,15 +438,6 @@ function testAddShard() {
     newReplicaSet.stopSet();
 }
 
-// TODO SERVER-69106 remove the logic to skip the test execution
-const historicalPlacementDataFeatureFlag = FeatureFlagUtil.isEnabled(
-    st.configRS.getPrimary().getDB('admin'), "HistoricalPlacementShardingCatalog");
-if (!historicalPlacementDataFeatureFlag) {
-    jsTestLog("Skipping as featureFlagHistoricalPlacementShardingCatalog is disabled");
-    st.stop();
-    return;
-}
-
 jsTest.log('Testing placement entries added by explicit DB creation');
 testEnableSharding('explicitlyCreatedDB', shard0);
 
@@ -457,4 +475,3 @@ jsTest.log(
 testAddShard();
 
 st.stop();
-}());
