@@ -8,15 +8,13 @@
  * resharding test fixture, which doesn't work when the first shard is the config server. This only
  * tests the testing fixture, so it wouldn't add meaningful coverage for a config shard.
  * @tags: [
- *   requires_persistence,
- *   uses_atclustertime,
  *   config_shard_incompatible,
+ *   requires_persistence,
+ *   temp_disabled_embedded_router_uncategorized,
+ *   uses_atclustertime,
  * ]
  */
-(function() {
-"use strict";
-
-load("jstests/sharding/libs/resharding_test_fixture.js");
+import {ReshardingTest} from "jstests/sharding/libs/resharding_test_fixture.js";
 
 const reshardingTest = new ReshardingTest({enableElections: true});
 reshardingTest.setup();
@@ -54,5 +52,4 @@ reshardingTest.withReshardingInBackground(
         }
     });
 
-reshardingTest.teardown();
-})();
+reshardingTest.teardown();
\ No newline at end of file
