@@ -50,21 +50,15 @@
  * ]
  *
  * @tags: [
- *   requires_fcv_70
+ *   # TODO (SERVER-70605): Remove this tag once the time-series always compressed buckets feature
+ *   # flag can be removed.
+ *   multiversion_incompatible,
  * ]
  */
 
-(function() {
-load("jstests/core/timeseries/libs/timeseries.js");
 const numShards = 2;
 const st = new ShardingTest({shards: numShards});
 
-if (!TimeseriesTest.shardedtimeseriesCollectionsEnabled(st.shard0)) {
-    jsTestLog("Skipping test because the sharded time-series collection feature flag is disabled");
-    st.stop();
-    return;
-}
-
 const dbName = 'testDB';
 const collName = 'testColl';
 const mongosDB = st.s.getDB(dbName);
@@ -169,9 +163,6 @@ function assertTimeseriesAggregationCorrectness(total, shards) {
     assert.eq(total.numBucketsClosedDueToTimeForward,
               shards.map(x => x.numBucketsClosedDueToTimeForward)
                   .reduce((x, y) => x + y, 0 /* initial value */));
-    assert.eq(total.numBucketsClosedDueToTimeBackward,
-              shards.map(x => x.numBucketsClosedDueToTimeBackward)
-                  .reduce((x, y) => x + y, 0) /* initial value */);
     assert.eq(total.numBucketsClosedDueToMemoryThreshold,
               shards.map(x => x.numBucketsClosedDueToMemoryThreshold)
                   .reduce((x, y) => x + y, 0 /* initial value */));
@@ -253,4 +244,3 @@ verifyClusterCollStatsResult(
 );
 
 st.stop();
-})();
