@@ -7,9 +7,9 @@
  * ]
  */
 
-(function() {
-load("jstests/core/timeseries/libs/timeseries.js");
-load('jstests/libs/analyze_plan.js');
+import {TimeseriesTest} from "jstests/core/timeseries/libs/timeseries.js";
+import {getAggPlanStages} from "jstests/libs/analyze_plan.js";
+import {FeatureFlagUtil} from "jstests/libs/feature_flag_util.js";
 
 Random.setRandomSeed();
 
@@ -19,15 +19,6 @@ const timeField = 'time';
 const metaField = 'hostid';
 
 const st = new ShardingTest({shards: 2, rs: {nodes: 2}});
-const sDB = st.s.getDB(dbName);
-const shard0DB = st.shard0.getDB(dbName);
-const shard1DB = st.shard1.getDB(dbName);
-
-if (!TimeseriesTest.shardedtimeseriesCollectionsEnabled(st.shard0)) {
-    jsTestLog("Skipping test because the sharded time-series collection feature flag is disabled");
-    st.stop();
-    return;
-}
 
 // Helpers.
 let currentId = 0;
@@ -46,8 +37,8 @@ function generateDoc(time, metaValue) {
 // Shard key on just the time field.
 (function timeShardKey() {
     const mongosDB = st.s.getDB(dbName);
-    assert.commandWorked(mongosDB.adminCommand({enableSharding: dbName}));
-    st.ensurePrimaryShard(dbName, st.shard0.shardName);
+    assert.commandWorked(
+        mongosDB.adminCommand({enableSharding: dbName, primaryShard: st.shard0.shardName}));
 
     // Shard time-series collection.
     const shardKey = {[timeField]: 1};
@@ -59,7 +50,7 @@ function generateDoc(time, metaValue) {
 
     // Split the chunks such that primary shard has chunk: [MinKey, 2020-01-01) and other shard has
     // chunk [2020-01-01, MaxKey].
-    splitPoint = {[`control.min.${timeField}`]: ISODate(`2020-01-01`)};
+    let splitPoint = {[`control.min.${timeField}`]: ISODate(`2020-01-01`)};
     assert.commandWorked(
         mongosDB.adminCommand({split: `${dbName}.system.buckets.${collName}`, middle: splitPoint}));
 
@@ -79,6 +70,11 @@ function generateDoc(time, metaValue) {
     assert.eq(1, counts[otherShard.shardName], counts);
 
     const coll = mongosDB.getCollection(collName);
+
+    // The {meta: 1, time: 1} index gets built by default on the time-series bucket collection.
+    let extraIndexes = [{[metaField]: 1, [timeField]: 1}];
+    let extraBucketIndexes = [{"meta": 1, "control.min.time": 1, "control.max.time": 1}];
+
     for (let i = 0; i < 2; i++) {
         assert.commandWorked(coll.insert(generateDoc("2019-11-11", -1)));
         assert.commandWorked(coll.insert(generateDoc("2019-12-31", -1)));
@@ -101,21 +97,24 @@ function generateDoc(time, metaValue) {
     assert.eq(coll.getFullName(), listIndexesOutput.cursor.ns, listIndexesOutput);
 
     let indexKeys = listIndexesOutput.cursor.firstBatch.map(x => x.key);
-    assert.sameMembers([{[subField1]: 1}, {[subField2]: 1}, {[timeField]: 1}, {[metaField]: 1}],
-                       indexKeys);
+    assert.sameMembers(
+        [{[subField1]: 1}, {[subField2]: 1}, {[timeField]: 1}, {[metaField]: 1}].concat(
+            extraIndexes),
+        indexKeys);
 
     assert.commandWorked(coll.dropIndex({[subField2]: 1}));
     indexKeys = coll.getIndexes().map(x => x.key);
-    assert.sameMembers([{[subField1]: 1}, {[timeField]: 1}, {[metaField]: 1}], indexKeys);
+    assert.sameMembers([{[subField1]: 1}, {[timeField]: 1}, {[metaField]: 1}].concat(extraIndexes),
+                       indexKeys);
 
     plan = coll.find({[metaField]: 0}).explain();
     assert.eq(getAggPlanStages(plan, "IXSCAN").length, 2, plan);
 
     assert.commandWorked(coll.dropIndex({[metaField]: 1}));
     indexKeys = coll.getIndexes().map(x => x.key);
-    assert.sameMembers([{[subField1]: 1}, {[timeField]: 1}], indexKeys);
+    assert.sameMembers([{[subField1]: 1}, {[timeField]: 1}].concat(extraIndexes), indexKeys);
 
-    plan = coll.find({[metaField]: 0}).explain();
+    plan = coll.find({[subField2]: 0}).explain();
     assert.eq(getAggPlanStages(plan, "COLLSCAN").length, 2, plan);
 
     // Verify that running the commands on the buckets collection should work.
@@ -126,14 +125,18 @@ function generateDoc(time, metaValue) {
 
     assert.commandWorked(bucketsColl.dropIndex({'meta.subField1': 1}));
     indexKeys = bucketsColl.getIndexes().map(x => x.key);
-    assert.sameMembers([{'control.min.time': 1}], indexKeys);
+    let indexesToAdd = {'control.min.time': 1};
+    // TODO SERVER-79304 the test shouldn't rely on the feature flag.
+    if (FeatureFlagUtil.isPresentAndEnabled(mongosDB, "AuthoritativeShardCollection")) {
+        indexesToAdd = {'control.min.time': 1, "control.max.time": 1};
+    }
+    assert.sameMembers([indexesToAdd].concat(extraBucketIndexes), indexKeys);
 
     assert.commandWorked(bucketsColl.createIndex({'meta.subField2': 1}));
     indexKeys = bucketsColl.getIndexes().map(x => x.key);
-    assert.sameMembers([{'control.min.time': 1}, {'meta.subField2': 1}], indexKeys);
+    assert.sameMembers([indexesToAdd, {'meta.subField2': 1}].concat(extraBucketIndexes), indexKeys);
 
     assert(coll.drop());
 })();
 
 st.stop();
-})();
