@@ -7,9 +7,8 @@
  * ]
  */
 
-(function() {
-load("jstests/core/timeseries/libs/timeseries.js");
-load("jstests/libs/analyze_plan.js");
+import {TimeseriesTest} from "jstests/core/timeseries/libs/timeseries.js";
+import {isClusteredIxscan, isCollscan, isIxscan} from "jstests/libs/analyze_plan.js";
 
 Random.setRandomSeed();
 
@@ -21,16 +20,10 @@ const metaField = 'hostid';
 
 const st = new ShardingTest({shards: 2, rs: {nodes: 2}});
 const sDB = st.s.getDB(dbName);
-assert.commandWorked(sDB.adminCommand({enableSharding: dbName}));
+assert.commandWorked(sDB.adminCommand({enableSharding: dbName, primaryShard: st.shard0.shardName}));
 const shard0DB = st.shard0.getDB(dbName);
 const shard1DB = st.shard1.getDB(dbName);
 
-if (!TimeseriesTest.shardedtimeseriesCollectionsEnabled(st.shard0)) {
-    jsTestLog("Skipping test because the sharded time-series collection feature flag is disabled");
-    st.stop();
-    return;
-}
-
 // Helpers.
 let currentId = 0;
 function generateId() {
@@ -130,8 +123,6 @@ function runQuery(
 
 // Shard key on just the time field.
 (function timeShardKey() {
-    st.ensurePrimaryShard(dbName, st.shard0.shardName);
-
     // Shard time-series collection.
     const shardKey = {[timeField]: 1};
     assert.commandWorked(sDB.adminCommand({
@@ -142,7 +133,7 @@ function runQuery(
 
     // Split the chunks such that primary shard has chunk: [MinKey, 2020-01-01) and other shard has
     // chunk [2020-01-01, MaxKey].
-    splitPoint = {[`control.min.${timeField}`]: ISODate(`2020-01-01`)};
+    let splitPoint = {[`control.min.${timeField}`]: ISODate(`2020-01-01`)};
     assert.commandWorked(
         sDB.adminCommand({split: `${dbName}.system.buckets.${collName}`, middle: splitPoint}));
 
@@ -304,8 +295,6 @@ function runQuery(
 
 // Shard key on the metadata field and time fields.
 (function metaAndTimeShardKey() {
-    st.ensurePrimaryShard(dbName, st.shard0.shardName);
-
     assert.commandWorked(sDB.adminCommand({
         shardCollection: `${dbName}.${collName}`,
         key: {[metaField]: 1, 'time': 1},
@@ -463,8 +452,6 @@ function runQuery(
 
 // Shard key on the metadata fields.
 (function metaFieldShardKey() {
-    st.ensurePrimaryShard(dbName, st.shard0.shardName);
-
     // Shard timeseries collection.
     const metaPrefix = `${metaField}.prefix`;
     const metaSuffix = `${metaField}.suffix`;
@@ -475,7 +462,7 @@ function runQuery(
         timeseries: {timeField, metaField}
     }));
 
-    splitPoint = {'meta.prefix': 0, 'meta.suffix': 0};
+    let splitPoint = {'meta.prefix': 0, 'meta.suffix': 0};
     assert.commandWorked(
         sDB.adminCommand({split: `${dbName}.system.buckets.${collName}`, middle: splitPoint}));
 
@@ -525,5 +512,4 @@ function runQuery(
     assert(coll.drop());
 })();
 
-st.stop();
-})();
+st.stop();
\ No newline at end of file
