@@ -2,15 +2,13 @@
  * Test updates into sharded timeseries collection.
  *
  * @tags: [
- *   requires_fcv_51,
- *   requires_find_command,
+ *   # TODO (SERVER-70605): Remove this tag once the time-series always compressed buckets feature
+ *   # flag can be removed.
+ *   multiversion_incompatible,
  * ]
  */
 
-(function() {
-"use strict";
-
-load("jstests/core/timeseries/libs/timeseries.js");  // For 'TimeseriesTest' helpers.
+import {TimeseriesTest} from "jstests/core/timeseries/libs/timeseries.js";
 
 const st = new ShardingTest({shards: 2, rs: {nodes: 2}});
 const mongos = st.s0;
@@ -26,44 +24,11 @@ const metaField = "tag";
 const dateTime1 = ISODate("2021-07-12T16:00:00Z");
 const dateTime2 = ISODate("2021-07-13T16:00:00Z");
 
-//
-// Checks for feature flags.
-//
-
-if (!TimeseriesTest.shardedtimeseriesCollectionsEnabled(st.shard0)) {
-    jsTestLog("Skipping test because the sharded time-series collection feature flag is disabled");
-    st.stop();
-    return;
-}
-
-if (!TimeseriesTest.timeseriesUpdatesAndDeletesEnabled(st.shard0)) {
-    jsTestLog(
-        "Skipping test because the updates and deletes on time-series collection feature flag is disabled");
-    st.stop();
-    return;
-}
-
 const testDB = mongos.getDB(dbName);
 testDB.dropDatabase();
 assert.commandWorked(mongos.adminCommand({enableSharding: dbName}));
 
-if (!TimeseriesTest.shardedTimeseriesUpdatesAndDeletesEnabled(st.shard0)) {
-    // Ensure that the feature flag correctly prevents us from running an update on a sharded
-    // timeseries collection.
-    assert.commandWorked(testDB.createCollection(collName, {timeseries: {timeField, metaField}}));
-    const coll = testDB.getCollection(collName);
-    assert.commandWorked(coll.createIndex({[timeField]: 1}));
-    assert.commandWorked(mongos.adminCommand({
-        shardCollection: `${dbName}.${collName}`,
-        key: {[timeField]: 1},
-    }));
-    assert.commandFailedWithCode(
-        testDB.runCommand(
-            {update: coll.getName(), updates: [{q: {}, u: {[metaField]: 1}, multi: true}]}),
-        [ErrorCodes.NotImplemented, ErrorCodes.InvalidOptions]);
-    st.stop();
-    return;
-}
+const arbitraryUpdatesEnabled = TimeseriesTest.arbitraryUpdatesEnabled(st.shard0);
 
 const doc1 = {
     _id: 1,
@@ -232,11 +197,19 @@ function expectFailedUpdate(initialDocList) {
 //
 
 function testCaseMultiFalseUpdateFails({testUpdate}) {
+    if (arbitraryUpdatesEnabled) {
+        return;
+    }
+
     testUpdate({updates: [{q: {[metaField]: {b: "B"}}, u: {$set: {[metaField]: {b: "C"}}}}]},
                expectFailedUpdate([doc1]));
 }
 
 function testCaseReplacementAndPipelineUpdateFails({testUpdate}) {
+    if (arbitraryUpdatesEnabled) {
+        return;
+    }
+
     const expectFailedUpdateDoc = expectFailedUpdate([doc2]);
 
     // Replace a document to have no metaField, which should fail since updates with replacement
@@ -276,6 +249,10 @@ function testCaseReplacementAndPipelineUpdateFails({testUpdate}) {
 }
 
 function testCaseNoMetaFieldQueryUpdateFails({testUpdate}) {
+    if (arbitraryUpdatesEnabled) {
+        return;
+    }
+
     // Query on a field which is not the (nonexistent) metaField.
     testUpdate({
         updates: [{
@@ -340,6 +317,10 @@ function testCaseNoMetaFieldQueryUpdateFails({testUpdate}) {
 }
 
 function testCaseIllegalMetaFieldUpdateFails({testUpdate}) {
+    if (arbitraryUpdatesEnabled) {
+        return;
+    }
+
     // Query on the metaField and modify a field that is not the metaField.
     testUpdate({
         updates: [{
@@ -468,54 +449,56 @@ function testCaseBatchUpdates({testUpdate}) {
 
     // Multiple updates, ordered: query on the metaField and modify a field that is not the
     // metaField using dot notation.
-    testUpdate({
-        updates: [
-            {
-                q: {[metaField]: {c: "C", d: 2}},
-                u: {$set: {"f1.0": "f2"}},
-                multi: true,
-            },
-            {
-                q: {[metaField]: {c: "C", d: 2}},
-                u: {$inc: {[metaField + ".d"]: 6}},
-                multi: true,
-            }
-        ]
-    },
-               expectFailedUpdate([doc2]));
+    if (!arbitraryUpdatesEnabled) {
+        testUpdate({
+            updates: [
+                {
+                    q: {[metaField]: {c: "C", d: 2}},
+                    u: {$set: {"f1.0": "f2"}},
+                    multi: true,
+                },
+                {
+                    q: {[metaField]: {c: "C", d: 2}},
+                    u: {$inc: {[metaField + ".d"]: 6}},
+                    multi: true,
+                }
+            ]
+        },
+                   expectFailedUpdate([doc2]));
 
-    // Multiple updates, unordered: Modify the metaField, a field that is not the metaField, and the
-    // metaField. The first and last updates should succeed.
-    testUpdate({
-        initialDocList: [doc2],
-        updates: [
-            {
-                q: {[metaField]: {c: "C", d: 2}},
-                u: {$inc: {[metaField + ".d"]: 6}},
-                multi: true,
-            },
-            {
-                q: {[metaField]: {c: "C", d: 8}},
-                u: {$set: {"f1.0": "f2"}},
-                multi: true,
-            },
-            {
-                q: {[metaField]: {c: "C", d: 8}},
-                u: {$inc: {[metaField + ".d"]: 7}},
-                multi: true,
-            }
-        ],
-        resultDocList: [{
-            _id: 2,
-            [timeField]: dateTime2,
-            [metaField]: {c: "C", d: 15},
-            f: [{"k": "K", "v": "V"}],
-        }],
-        ordered: false,
-        n: 2,
-        pathToMetaFieldBeingUpdated: "d",
-        failCode: ErrorCodes.InvalidOptions,
-    });
+        // Multiple updates, unordered: Modify the metaField, a field that is not the metaField, and
+        // the metaField. The first and last updates should succeed.
+        testUpdate({
+            initialDocList: [doc2],
+            updates: [
+                {
+                    q: {[metaField]: {c: "C", d: 2}},
+                    u: {$inc: {[metaField + ".d"]: 6}},
+                    multi: true,
+                },
+                {
+                    q: {[metaField]: {c: "C", d: 8}},
+                    u: {$set: {"f1.0": "f2"}},
+                    multi: true,
+                },
+                {
+                    q: {[metaField]: {c: "C", d: 8}},
+                    u: {$inc: {[metaField + ".d"]: 7}},
+                    multi: true,
+                }
+            ],
+            resultDocList: [{
+                _id: 2,
+                [timeField]: dateTime1,
+                [metaField]: {c: "C", d: 15},
+                f: [{"k": "K", "v": "V"}],
+            }],
+            ordered: false,
+            n: 2,
+            pathToMetaFieldBeingUpdated: "d",
+            failCode: ErrorCodes.InvalidOptions,
+        });
+    }
 }
 
 function testCaseValidMetaFieldUpdates({testUpdate}) {
@@ -708,52 +691,55 @@ function testCaseValidMetaFieldUpdates({testUpdate}) {
         pathToMetaFieldBeingUpdated: "",
     });
 
-    // Query for documents using $jsonSchema with a field that is not the metaField required.
-    testUpdate({
-        updates: [{
-            q: {"$jsonSchema": {"required": [metaField, timeField]}},
-            u: {$set: {[metaField]: "a"}},
-            multi: true
-        }],
-    },
-               expectFailedUpdate([doc1, doc2, doc3]));
+    if (!arbitraryUpdatesEnabled) {
+        // Query for documents using $jsonSchema with a field that is not the metaField required.
+        testUpdate({
+            updates: [{
+                q: {"$jsonSchema": {"required": [metaField, timeField]}},
+                u: {$set: {[metaField]: "a"}},
+                multi: true
+            }],
+        },
+                   expectFailedUpdate([doc1, doc2, doc3]));
 
-    const nestedMetaObj = {_id: 6, [timeField]: dateTime1, [metaField]: {[metaField]: "A", a: 1}};
+        const nestedMetaObj =
+            {_id: 6, [timeField]: dateTime1, [metaField]: {[metaField]: "A", a: 1}};
 
-    // Query for documents using $jsonSchema with the metaField required and a required subfield of
-    // the metaField with the same name as the metaField.
-    testUpdate({
-        initialDocList: [doc1, nestedMetaObj],
-        updates: [{
-            q: {
-                "$jsonSchema": {
-                    "required": [metaField],
-                    "properties": {[metaField]: {"required": [metaField]}}
-                }
-            },
-            u: {$set: {[metaField]: "a"}},
-            multi: true
-        }],
-        resultDocList: [doc1, {_id: 6, [timeField]: dateTime1, [metaField]: "a", a: 1}],
-        n: 1,
-        pathToMetaFieldBeingUpdated: "",
-    });
+        // Query for documents using $jsonSchema with the metaField required and a required subfield
+        // of the metaField with the same name as the metaField.
+        testUpdate({
+            initialDocList: [doc1, nestedMetaObj],
+            updates: [{
+                q: {
+                    "$jsonSchema": {
+                        "required": [metaField],
+                        "properties": {[metaField]: {"required": [metaField]}}
+                    }
+                },
+                u: {$set: {[metaField]: "a"}},
+                multi: true
+            }],
+            resultDocList: [doc1, {_id: 6, [timeField]: dateTime1, [metaField]: "a", a: 1}],
+            n: 1,
+            pathToMetaFieldBeingUpdated: "",
+        });
 
-    // Query for documents using $jsonSchema with the metaField required and an optional field that
-    // is not the metaField.
-    testUpdate({
-        updates: [{
-            q: {
-                "$jsonSchema": {
-                    "required": [metaField],
-                    "properties": {"measurement": {description: "can be any value"}}
-                }
-            },
-            u: {$set: {[metaField]: "a"}},
-            multi: true
-        }]
-    },
-               expectFailedUpdate([doc1, nestedMetaObj]));
+        // Query for documents using $jsonSchema with the metaField required and an optional field
+        // that is not the metaField.
+        testUpdate({
+            updates: [{
+                q: {
+                    "$jsonSchema": {
+                        "required": [metaField],
+                        "properties": {"measurement": {description: "can be any value"}}
+                    }
+                },
+                u: {$set: {[metaField]: "a"}},
+                multi: true
+            }]
+        },
+                   expectFailedUpdate([doc1, nestedMetaObj]));
+    }
 
     // Query for documents on the metaField with the metaField nested within nested operators.
     testUpdate({
@@ -787,16 +773,18 @@ function testCaseValidMetaFieldUpdates({testUpdate}) {
         n: 0,
     });
 
-    // Do the same test case as above but with upsert:true, which should fail.
-    testUpdate({
-        updates: [{
-            q: {[metaField]: "Z"},
-            u: {$set: {[metaField]: 5}},
-            multi: true,
-            upsert: true,
-        }]
-    },
-               expectFailedUpdate([doc1, doc4, doc5]));
+    if (!arbitraryUpdatesEnabled) {
+        // Do the same test case as above but with upsert:true, which should fail.
+        testUpdate({
+            updates: [{
+                q: {[metaField]: "Z"},
+                u: {$set: {[metaField]: 5}},
+                multi: true,
+                upsert: true,
+            }]
+        },
+                   expectFailedUpdate([doc1, doc4, doc5]));
+    }
 }
 
 function testCaseUpdateWithLetDoc({testUpdate}) {
@@ -943,7 +931,9 @@ const tests = [
     testCaseBatchUpdates,
     testCaseValidMetaFieldUpdates,
 ];
-testUpdates({shardKeyTimeField: timeField, timeseriesOptions: {timeField}, tests});
+if (!arbitraryUpdatesEnabled) {
+    testUpdates({shardKeyTimeField: timeField, timeseriesOptions: {timeField}, tests});
+}
 testUpdates({shardKeyMetaFieldPath: metaField, timeseriesOptions, tests});
 testUpdates(
     {shardKeyTimeField: timeField, shardKeyMetaFieldPath: metaField, timeseriesOptions, tests});
@@ -969,5 +959,4 @@ testUpdates({
     tests: testsForMetaSubfieldShardKey,
 });
 
-st.stop();
-})();
+st.stop();
\ No newline at end of file
