@@ -7,11 +7,27 @@
  * ]
  */
 
-(function() {
-'use strict';
-
-load("jstests/sharding/libs/sharded_transactions_helpers.js");
-load("jstests/sharding/libs/update_shard_key_helpers.js");
+import {
+    enableCoordinateCommitReturnImmediatelyAfterPersistingDecision,
+    isUpdateDocumentShardKeyUsingTransactionApiEnabled,
+} from "jstests/sharding/libs/sharded_transactions_helpers.js";
+import {
+    assertCannotUpdate_id,
+    assertCannotUpdate_idDottedPath,
+    assertCannotUpdateInBulkOpWhenDocsMoveShards,
+    assertCannotUpdateSKToArray,
+    assertCannotUpdateWithMultiTrue,
+    assertCanUnsetSKField,
+    assertCanUpdateDottedPath,
+    assertCanUpdatePartialShardKey,
+    assertCanUpdatePrimitiveShardKey,
+    assertCanUpdatePrimitiveShardKeyHashedChangeShards,
+    runFindAndModifyCmdFail,
+    runFindAndModifyCmdSuccess,
+    runUpdateCmdFail,
+    runUpdateCmdSuccess,
+    shardCollectionMoveChunks,
+} from "jstests/sharding/libs/update_shard_key_helpers.js";
 
 const st = new ShardingTest({
     mongos: 1,
@@ -23,15 +39,13 @@ const st = new ShardingTest({
 const kDbName = 'db';
 const mongos = st.s0;
 const shard0 = st.shard0.shardName;
-const shard1 = st.shard1.shardName;
 const ns = kDbName + '.foo';
 
 const updateDocumentShardKeyUsingTransactionApiEnabled =
     isUpdateDocumentShardKeyUsingTransactionApiEnabled(st.s);
 
 enableCoordinateCommitReturnImmediatelyAfterPersistingDecision(st);
-assert.commandWorked(mongos.adminCommand({enableSharding: kDbName}));
-st.ensurePrimaryShard(kDbName, shard0);
+assert.commandWorked(mongos.adminCommand({enableSharding: kDbName, primaryShard: shard0}));
 
 function changeShardKeyWhenFailpointsSet(session,
                                          sessionDB,
@@ -414,6 +428,45 @@ assert.commandWorked(st.rs0.getPrimary().getDB(kDbName).adminCommand({
 
 mongos.getDB(kDbName).foo.drop();
 
+// ----Assert write result reports error when the internal transaction fails to commit----
+
+shardCollectionMoveChunks(st, kDbName, ns, {"x": 1}, docsToInsert, {"x": 100}, {"x": 300});
+
+// Set this failpoint on the coordinator shard so that sending prepareTransaction to
+// participating shards will fail.
+assert.commandWorked(st.rs0.getPrimary().getDB(kDbName).adminCommand({
+    configureFailPoint: "failRemoteTransactionCommand",
+    mode: "alwaysOn",
+    data: {command: "prepareTransaction", code: ErrorCodes.TransactionTooOld}
+}));
+
+res = sessionDB.foo.update({x: 4}, {$set: {x: 1000}});
+
+// For update, we should have reported the error in the write errors field and the top-level
+// fields should reflect that no update was performed.
+assert.writeErrorWithCode(res, ErrorCodes.TransactionTooOld);
+assert.eq(res.nModified, 0);
+assert.eq(res.nMatched, 0);
+assert.eq(res.nUpserted, 0);
+
+res = sessionDB.runCommand({
+    findAndModify: 'foo',
+    query: {x: 78},
+    update: {$set: {x: 250}},
+    lsid: {id: UUID()},
+    txnNumber: NumberLong(1),
+});
+
+// findAndModify reports the failure as a top-level error.
+assert.commandFailedWithCode(res, ErrorCodes.TransactionTooOld);
+
+assert.commandWorked(st.rs0.getPrimary().getDB(kDbName).adminCommand({
+    configureFailPoint: "failRemoteTransactionCommand",
+    mode: "off",
+}));
+
+mongos.getDB(kDbName).foo.drop();
+
 // ----Assert that updating the shard key in a batch with size > 1 fails----
 
 assertCannotUpdateInBulkOpWhenDocsMoveShards(st, kDbName, ns, session, sessionDB, false, true);
@@ -482,4 +535,3 @@ assert.eq(1, sessionDB.foo.find({"x": 1}).toArray().length);
 mongos.getDB(kDbName).foo.drop();
 
 st.stop();
-})();
