@@ -7,16 +7,17 @@
  *   uses_transactions,
  * ]
  */
-(function() {
-
-load("jstests/sharding/libs/sharded_transactions_helpers.js");
-load("jstests/sharding/updateOne_without_shard_key/libs/write_without_shard_key_test_util.js");
+import {
+    enableCoordinateCommitReturnImmediatelyAfterPersistingDecision
+} from "jstests/sharding/libs/sharded_transactions_helpers.js";
+import {
+    WriteWithoutShardKeyTestUtil
+} from "jstests/sharding/updateOne_without_shard_key/libs/write_without_shard_key_test_util.js";
 
 const s = new ShardingTest({name: "auto1", shards: 2, mongos: 1});
 
 enableCoordinateCommitReturnImmediatelyAfterPersistingDecision(s);
-s.adminCommand({enablesharding: "test"});
-s.ensurePrimaryShard('test', s.shard1.shardName);
+s.adminCommand({enablesharding: "test", primaryShard: s.shard1.shardName});
 
 const db = s.getDB("test");
 const sessionDb = s.s.startSession({retryWrites: true}).getDatabase("test");
@@ -266,4 +267,3 @@ testNestedShardKeys("update_nested", {"skey.skey": 1});
 testNestedShardKeys("update_nested_hashed", {"skey.skey": "hashed"});
 
 s.stop();
-})();
