@@ -3,19 +3,19 @@
 // operations with queries that do not match on the entire shard key are successful. NOTE: Generic
 // upsert behavior tests belong in the core suite
 //
-(function() {
-'use strict';
-
-load("jstests/sharding/updateOne_without_shard_key/libs/write_without_shard_key_test_util.js");
+import {
+    WriteWithoutShardKeyTestUtil
+} from "jstests/sharding/updateOne_without_shard_key/libs/write_without_shard_key_test_util.js";
 
 const st = new ShardingTest({shards: 2, mongos: 1});
 
 const mongos = st.s0;
 const admin = mongos.getDB("admin");
-const coll = mongos.getCollection("foo.bar");
+const kDbName = "test";
 
-assert(admin.runCommand({enableSharding: coll.getDB() + ""}).ok);
-st.ensurePrimaryShard(coll.getDB().getName(), st.shard1.shardName);
+assert(admin.runCommand({enableSharding: kDbName + "", primaryShard: st.shard0.name}).ok);
+
+const coll = mongos.getCollection(kDbName + ".bar");
 
 const upsertSuppliedResult = function(upsertColl, query, newDoc) {
     assert.commandWorked(upsertColl.remove({}));
@@ -48,7 +48,6 @@ const upsertedXVal = function(upsertColl, query, expr) {
 //
 // Tests for non-nested shard key.
 //
-st.ensurePrimaryShard(coll.getDB() + "", st.shard0.shardName);
 assert.commandWorked(admin.runCommand({shardCollection: coll + "", key: {x: 1}}));
 assert.commandWorked(admin.runCommand({split: coll + "", middle: {x: 0}}));
 assert.commandWorked(admin.runCommand(
@@ -173,7 +172,6 @@ coll.drop();
 //
 // Tests for nested shard key.
 //
-st.ensurePrimaryShard(coll.getDB() + "", st.shard0.shardName);
 assert.commandWorked(admin.runCommand({shardCollection: coll + "", key: {'x.x': 1}}));
 assert.commandWorked(admin.runCommand({split: coll + "", middle: {'x.x': 0}}));
 assert.commandWorked(admin.runCommand(
@@ -255,7 +253,6 @@ coll.drop();
 //
 // Tests for nested _id shard key.
 //
-st.ensurePrimaryShard(coll.getDB() + "", st.shard0.shardName);
 assert.commandWorked(admin.runCommand({shardCollection: coll + "", key: {'_id.x': 1}}));
 assert.commandWorked(admin.runCommand({split: coll + "", middle: {'_id.x': 0}}));
 assert.commandWorked(admin.runCommand(
@@ -400,4 +397,3 @@ assert.commandWorked(upsertedResult(coll, {"_id.x": -1}, {$set: {"_id.x": -1}}))
 assert.docEq({_id: {x: -1}}, coll.findOne({}));
 
 st.stop();
-})();
