@@ -1,10 +1,7 @@
-(function() {
-'use strict';
-
 // init with one shard with one node rs
 var st = new ShardingTest({shards: 1, rs: {nodes: 1}, mongos: 1});
 var mongos = st.s;
-var rs = st.rs0;
+const rs = st.rs0;
 
 assert.commandWorked(st.s0.adminCommand({enablesharding: "test"}));
 
@@ -38,4 +35,3 @@ jsTest.log("Issue find");
 assert.eq(db.foo.find({_id: 1}).readPref('secondary').next().x, 1);
 
 st.stop();
-})();
