@@ -5,9 +5,6 @@
 //   resource_intensive,
 // ]
 
-(function() {
-'use strict';
-
 const numLocations = 300;
 let locObj = [];
 // Add locations everywhere
@@ -63,5 +60,4 @@ const primary = rst.getPrimary();
 test(primary, /*index=*/ true);
 test(primary, /*index=*/ false);
 
-rst.stopSet();
-})();
+rst.stopSet();
\ No newline at end of file
