@@ -11,14 +11,12 @@
  *
  * @tags: [requires_wiredtiger, requires_persistence, requires_replication]
  */
-(function() {
-"use strict";
 
 // This test triggers an unclean shutdown, which may cause inaccurate fast counts.
 TestData.skipEnforceFastCountOnValidate = true;
 
-load("jstests/libs/parallel_shell_helpers.js");  // For startParallelShell
-load('jstests/libs/parallelTester.js');          // For Thread
+import {PrepareHelpers} from "jstests/core/txns/libs/prepare_helpers.js";
+import {Thread} from "jstests/libs/parallelTester.js";
 
 const kSeparator = "/";
 const copyDataFiles = function(src, dest) {
@@ -39,8 +37,6 @@ const copyDataFiles = function(src, dest) {
 
 // Multi-document prepared transaction workload executed by the client threads.
 const threadWorkload = function(host, threadId, start, end) {
-    load("jstests/core/txns/libs/prepare_helpers.js");
-
     try {
         const conn = new Mongo(host);
 
@@ -205,4 +201,3 @@ while (numUncleanShutdowns < kNumUncleanShutdowns) {
 }
 
 rst.stopSet();
-}());
