@@ -1,20 +1,18 @@
 // Test macOS refusing to start up with encrypted PEM file.
 
-load('jstests/ssl/libs/ssl_helpers.js');
-requireSSLProvider('apple', function() {
-    'use strict';
+import {requireSSLProvider} from "jstests/ssl/libs/ssl_helpers.js";
 
+requireSSLProvider('apple', function() {
     jsTest.log("Verifying that mongod will fail to start using an encrypted PEM file");
 
     const config = MongoRunner.mongodOptions({
-        sslPEMKeyFile: "jstests/libs/password_protected.pem",
-        sslMode: "requireSSL",
-        sslPEMKeyPassword: "qwerty",
-        sslCAFile: "jstests/libs/ca.pem",
+        tlsCertificateKeyFile: "jstests/libs/password_protected.pem",
+        tlsMode: "requireTLS",
+        tlsCertificateKeyFilePassword: "qwerty",
+        tlsCAFile: "jstests/libs/ca.pem",
     });
 
-    const mongod = MongoRunner.runMongod(config);
-    assert(mongod === null, "MongoD unexpectedly started up");
+    assert.throws(() => MongoRunner.runMongod(config), [], "MongoD unexpectedly started up");
 
     assert.eq(rawMongoProgramOutput().includes(
                   "Using encrypted PKCS#1/PKCS#8 PEM files is not supported on this platform"),
