@@ -1,8 +1,9 @@
 // Ensure the server counts the server TLS versions used
-(function() {
-'use strict';
-
-load("jstests/ssl/libs/ssl_helpers.js");
+import {
+    detectDefaultTLSProtocol,
+    sslProviderSupportsTLS1_0,
+    sslProviderSupportsTLS1_1,
+} from "jstests/ssl/libs/ssl_helpers.js";
 
 var SERVER_CERT = "jstests/libs/server.pem";
 var CLIENT_CERT = "jstests/libs/client.pem";
@@ -26,27 +27,24 @@ function runTestWithoutSubset(client) {
     print(tojson(expectedCounts));
 
     const conn = MongoRunner.runMongod({
-        sslMode: 'allowSSL',
-        sslPEMKeyFile: SERVER_CERT,
-        sslDisabledProtocols: 'none',
+        tlsMode: 'allowTLS',
+        tlsCertificateKeyFile: SERVER_CERT,
+        tlsDisabledProtocols: 'none',
         useLogFiles: true,
         tlsLogVersions: "TLS1_0,TLS1_1,TLS1_2,TLS1_3",
         tlsCAFile: CA_CERT
     });
 
-    print(disabledProtocols);
-    const version_number = client.replace(/TLS/, "").replace(/_/, ".");
-
     const exitStatus = runMongoProgram('mongo',
-                                       '--ssl',
-                                       '--sslAllowInvalidHostnames',
-                                       '--sslPEMKeyFile',
+                                       '--tls',
+                                       '--tlsAllowInvalidHostnames',
+                                       '--tlsCertificateKeyFile',
                                        CLIENT_CERT,
-                                       '--sslCAFile',
+                                       '--tlsCAFile',
                                        CA_CERT,
                                        '--port',
                                        conn.port,
-                                       '--sslDisabledProtocols',
+                                       '--tlsDisabledProtocols',
                                        disabledProtocols.join(","),
                                        '--eval',
                                        // The Javascript string "1.0" is implicitly converted to the
@@ -102,5 +100,4 @@ if (sslProviderSupportsTLS1_1()) {
     runTestWithoutSubset("TLS1_1");
 }
 runTestWithoutSubset("TLS1_2");
-runTestWithoutSubset("TLS1_3");
-})();
+runTestWithoutSubset("TLS1_3");
\ No newline at end of file
