@@ -1,9 +1,6 @@
 // Ensure that the shell may connect to servers running supporting restricted subsets of TLS
 // protocols.
 
-(function() {
-'use strict';
-
 var SERVER_CERT = "jstests/libs/server.pem";
 var CLIENT_CERT = "jstests/libs/client.pem";
 var CA_CERT = "jstests/libs/ca.pem";
@@ -11,18 +8,18 @@ var CA_CERT = "jstests/libs/ca.pem";
 function runTestWithoutSubset(subset) {
     const disabledProtocols = subset.join(",");
     const conn = MongoRunner.runMongod({
-        sslMode: 'allowSSL',
-        sslPEMKeyFile: SERVER_CERT,
-        sslDisabledProtocols: disabledProtocols,
+        tlsMode: 'allowTLS',
+        tlsCertificateKeyFile: SERVER_CERT,
+        tlsDisabledProtocols: disabledProtocols,
         tlsCAFile: CA_CERT
     });
 
     const exitStatus = runMongoProgram('mongo',
-                                       '--ssl',
-                                       '--sslAllowInvalidHostnames',
-                                       '--sslPEMKeyFile',
+                                       '--tls',
+                                       '--tlsAllowInvalidHostnames',
+                                       '--tlsCertificateKeyFile',
                                        CLIENT_CERT,
-                                       '--sslCAFile',
+                                       '--tlsCAFile',
                                        CA_CERT,
                                        '--port',
                                        conn.port,
@@ -36,5 +33,4 @@ function runTestWithoutSubset(subset) {
 
 runTestWithoutSubset(["TLS1_0"]);
 runTestWithoutSubset(["TLS1_2"]);
-runTestWithoutSubset(["TLS1_0", "TLS1_1"]);
-})();
+runTestWithoutSubset(["TLS1_0", "TLS1_1"]);
\ No newline at end of file
