@@ -1,8 +1,5 @@
 // Test setParameter tlsWithholdClientCertificate
 
-(function() {
-"use strict";
-
 function testRS(opts, expectWarning) {
     const rsOpts = {
         nodes: {node0: opts, node1: opts},
@@ -44,10 +41,9 @@ const test_options = Object.extend({
 testRS(test_options, true);
 
 const depr_options = Object.extend({
-    sslAllowConnectionsWithoutCertificates: '',
-    setParameter: 'sslWithholdClientCertificate=true',
+    tlsAllowConnectionsWithoutCertificates: '',
+    setParameter: 'tlsWithholdClientCertificate=true',
 },
                                    base_options);
 
-testRS(depr_options, true);
-}());
+testRS(depr_options, true);
\ No newline at end of file
