@@ -5,9 +5,17 @@
 // To install trusted-ca.pem for local testing on OSX, invoke the following at a console:
 //   security add-trusted-cert -d jstests/libs/trusted-ca.pem
 
-load('jstests/libs/python.js');
+import {getPython3Binary} from "jstests/libs/python.js";
 
 const HOST_TYPE = getBuildInfo().buildEnvironment.target_os;
+jsTest.log("HOST_TYPE = " + HOST_TYPE);
+
+if (HOST_TYPE == "macOS") {
+    // Ensure trusted-ca.pem is properly installed on MacOS hosts.
+    // (MacOS is the only OS where it is installed outside of this test)
+    let exitCode = runProgram("security", "verify-cert", "-c", "./jstests/libs/trusted-client.pem");
+    assert.eq(0, exitCode, 'Check for proper installation of Trusted CA on MacOS host');
+}
 if (HOST_TYPE == "windows") {
     assert.eq(0, runProgram(getPython3Binary(), "jstests/ssl_linear/windows_castore_cleanup.py"));
 
@@ -18,19 +26,19 @@ if (HOST_TYPE == "windows") {
     // Current User.
     runProgram("certutil.exe", "-addstore", "-f", "Root", "jstests\\libs\\trusted-ca.pem");
 }
+
 try {
     const x509Options = {
-        sslMode: 'requireSSL',
-        sslPEMKeyFile: 'jstests/libs/trusted-server.pem',
-        sslCAFile: 'jstests/libs/trusted-ca.pem',
-        sslClusterFile: 'jstests/libs/trusted-client.pem',
-        sslAllowInvalidCertificates: '',
-        sslWeakCertificateValidation: '',
+        tlsMode: 'requireTLS',
+        tlsCertificateKeyFile: 'jstests/libs/trusted-server.pem',
+        tlsCAFile: 'jstests/libs/trusted-ca.pem',
+        tlsAllowInvalidCertificates: '',
+        tlsWeakCertificateValidation: '',
     };
 
     const rst = new ReplSetTest({
         nodes: 2,
-        name: "sslSet",
+        name: "tlsSet",
         useHostName: false,
         nodeOptions: x509Options,
         waitForKeys: false
@@ -40,12 +48,12 @@ try {
 
     const subShellCommand = function(hosts) {
         var Ms = [];
-        for (var i = 0; i < 10; i++) {
+        for (let i = 0; i < 10; i++) {
             Ms.push(new Mongo("mongodb://" + hosts[0] + "," + hosts[1] +
-                              "/?ssl=true&replicaSet=sslSet"));
+                              "/?ssl=true&replicaSet=tlsSet"));
         }
 
-        for (var i = 0; i < 10; i++) {
+        for (let i = 0; i < 10; i++) {
             var db = Ms[i].getDB("test");
             db.setSecondaryOk();
             db.col.find().readPref("secondary").toArray();
@@ -57,13 +65,14 @@ try {
         replSet.nodes.forEach((node) => {
             hosts.push("localhost:" + node.port);
         });
+
         let command = `
                 (function () {
-                    'use strict';
                     let command = ${subShellCommand.toString()};
                     let hosts = ${tojson(hosts)};
                     command(hosts);
                 }());`;
+
         return command;
     };
 
@@ -80,8 +89,8 @@ try {
     rst.stopSet();
 } finally {
     if (HOST_TYPE == "windows") {
-        const trusted_ca_thumbprint = cat('jstests/libs/ca.pem.digest.sha1');
+        const trusted_ca_thumbprint = cat('jstests/libs/trusted-ca.pem.digest.sha1');
         runProgram("certutil.exe", "-delstore", "-f", "Root", trusted_ca_thumbprint);
         runProgram("certutil.exe", "-delstore", "-user", "-f", "CA", trusted_ca_thumbprint);
     }
-}
+}
\ No newline at end of file
