@@ -1,9 +1,7 @@
 // Storage Node Watchdog - validate watchdog monitors --auditpath
 //
-load("jstests/watchdog/lib/wd_test_common.js");
-
-(function() {
-'use strict';
+import {CharybdefsControl} from "jstests/watchdog/lib/charybdefs_lib.js";
+import {testFuseAndMongoD} from "jstests/watchdog/lib/wd_test_common.js";
 
 if (assert.commandWorked(db.runCommand({buildInfo: 1})).modules.includes("enterprise")) {
     let control = new CharybdefsControl("auditpath_hang");
@@ -17,4 +15,3 @@ if (assert.commandWorked(db.runCommand({buildInfo: 1})).modules.includes("enterp
         auditPath: auditPath + "/auditLog.json"
     });
 }
-})();
