@@ -31,9 +31,12 @@
 
 #include <functional>
 #include <memory>
+#include <string>
 #include <tuple>
 
-#include "mongo/unittest/unittest.h"
+#include "mongo/base/string_data.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
 
 
 namespace {
@@ -919,7 +922,7 @@ private:
     const std::string root;
     int generation = 0;
 
-    GeneratorImplementation* clone_impl() const {
+    GeneratorImplementation* clone_impl() const override {
         return new GeneratorImplementation{*this};
     }
 
@@ -937,7 +940,7 @@ class StorageImplementation : public Interface {
 private:
     std::string store;
 
-    StorageImplementation* clone_impl() const {
+    StorageImplementation* clone_impl() const override {
         return new StorageImplementation{*this};
     }
 
