@@ -29,18 +29,24 @@
 
 #pragma once
 
-#include "mongo/config.h"
-
+#include <array>
 #include <cstddef>
+#include <cstdlib>
 #include <limits>
 #include <memory>
+#include <new>
 #include <string>
 #include <type_traits>
+#include <utility>
 #include <vector>
 
 #include "mongo/base/static_assert.h"
+#include "mongo/base/string_data.h"
+#include "mongo/config.h"  // IWYU pragma: keep
 #include "mongo/db/server_options.h"
+#include "mongo/platform/atomic_word.h"
 #include "mongo/stdx/type_traits.h"
+#include "mongo/util/allocator.h"
 #include "mongo/util/assert_util.h"
 
 namespace mongo {
@@ -50,6 +56,33 @@ namespace secure_allocator_details {
 void* allocate(std::size_t bytes, std::size_t alignOf);
 void deallocate(void* ptr, std::size_t bytes);
 
+/**
+ * This class keeps track of secure allocation byte count and secure allocation bytes that are
+ * allocated in paged allocations.
+ */
+
+class SecureAllocCountInfo {
+public:
+    uint32_t getSecureAllocByteCount() {
+        return secureAllocByteCount.load();
+    }
+    uint32_t getSecureAllocBytesInPages() {
+        return secureAllocBytesInPages.load();
+    }
+    void updateSecureAllocByteCount(int32_t cnt) {
+        secureAllocByteCount.fetchAndAdd(cnt);
+    }
+    void updateSecureAllocBytesInPages(int32_t cnt) {
+        secureAllocBytesInPages.fetchAndAdd(cnt);
+    }
+
+private:
+    AtomicWord<uint32_t> secureAllocByteCount{0};
+    AtomicWord<uint32_t> secureAllocBytesInPages{0};
+};
+
+SecureAllocCountInfo& gSecureAllocCountInfo();
+
 inline void* allocateWrapper(std::size_t bytes, std::size_t alignOf, bool secure) {
     if (secure) {
         return allocate(bytes, alignOf);
