@@ -29,15 +29,20 @@
 
 #pragma once
 
+#include <boost/intrusive_ptr.hpp>
+#include <boost/smart_ptr.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <boost/smart_ptr/intrusive_ref_counter.hpp>
 #include <iosfwd>
+#include <memory>
+#include <new>
 #include <string>
 #include <type_traits>
-
-#include <boost/intrusive_ptr.hpp>
-#include <boost/smart_ptr/intrusive_ref_counter.hpp>
+#include <utility>
 
 #include "mongo/base/error_codes.h"
 #include "mongo/base/error_extra_info.h"
+#include "mongo/base/static_assert.h"
 #include "mongo/base/string_data.h"
 #include "mongo/bson/util/builder_fwd.h"
 #include "mongo/platform/compiler.h"
