@@ -30,14 +30,18 @@
 #pragma once
 
 #include <boost/optional.hpp>
+#include <boost/optional/optional.hpp>
 #include <iosfwd>
+#include <string>
 #include <type_traits>
 #include <utility>
 
+#include "mongo/base/error_codes.h"
 #include "mongo/base/static_assert.h"
 #include "mongo/base/status.h"
 #include "mongo/bson/util/builder_fwd.h"
 #include "mongo/platform/compiler.h"
+#include "mongo/unittest/stringify.h"
 #include "mongo/util/assert_util_core.h"
 
 namespace mongo {
@@ -177,6 +181,15 @@ private:
     boost::optional<T> _t;
 };
 
+template <typename T>
+std::string stringifyForAssert(const StatusWith<T>& sw) {
+    if (sw.isOK()) {
+        return unittest::stringify::invoke(sw.getValue());
+    } else {
+        return unittest::stringify::invoke(sw.getStatus());
+    }
+}
+
 template <typename T>
 auto operator<<(std::ostream& stream, const StatusWith<T>& sw)
     -> decltype(stream << sw.getValue())  // SFINAE on T streamability.
