@@ -29,18 +29,36 @@
 
 #include "mongo/bson/bson_validate.h"
 
+#include <algorithm>
+#include <array>
 #include <cstring>
+#include <fmt/format.h>
+#include <memory>
+#include <string>
+#include <type_traits>
+#include <utility>
 #include <vector>
 
+
+#include "mongo/base/data_type_endian.h"
 #include "mongo/base/data_view.h"
+#include "mongo/base/error_codes.h"
+#include "mongo/base/static_assert.h"
+#include "mongo/base/string_data.h"
 #include "mongo/bson/bson_depth.h"
+#include "mongo/bson/bson_validate_gen.h"
 #include "mongo/bson/bsonelement.h"
 #include "mongo/bson/bsonelementvalue.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/bson/bsontypes_util.h"
 #include "mongo/bson/util/bsoncolumn.h"
 #include "mongo/bson/util/bsoncolumn_util.h"
 #include "mongo/crypto/encryption_fields_util.h"
 #include "mongo/crypto/fle_field_schema_gen.h"
-#include "mongo/logv2/log.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/decimal_counter.h"
+#include "mongo/util/str.h"
 #include "mongo/util/str_escape.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
@@ -98,8 +116,8 @@ public:
 
     void popLevel() {}
 
-    BSONValidateMode validateMode() {
-        return BSONValidateMode::kDefault;
+    BSONValidateModeEnum validateMode() {
+        return BSONValidateModeEnum::kDefault;
     }
 };
 
@@ -183,8 +201,8 @@ public:
         }
     }
 
-    BSONValidateMode validateMode() {
-        return BSONValidateMode::kExtended;
+    BSONValidateModeEnum validateMode() {
+        return BSONValidateModeEnum::kExtended;
     }
 
 private:
@@ -317,9 +335,12 @@ public:
                         // Calling size() decompresses the entire column.
                         try {
                             BSONColumn(BSONElement(ptr)).size();
-                        } catch (...) {
-                            uasserted(NonConformantBSON,
-                                      "Exception ocurred while decompressing a BSON column.");
+                        } catch (DBException& e) {
+                            uasserted(
+                                NonConformantBSON,
+                                str::stream()
+                                    << "Exception occurred while decompressing a BSON column: "
+                                    << e.toString());
                         }
                         break;
                     }
@@ -357,8 +378,8 @@ public:
         checkDuplicateFieldName();
     }
 
-    BSONValidateMode validateMode() {
-        return BSONValidateMode::kFull;
+    BSONValidateModeEnum validateMode() {
+        return BSONValidateModeEnum::kFull;
     }
 
 private:
@@ -385,7 +406,7 @@ private:
 template <bool precise>
 Status _doValidateColumn(const char* originalBuffer,
                          uint64_t maxLength,
-                         BSONValidateMode mode,
+                         BSONValidateModeEnum mode,
                          ValidationVersion validationVersion);
 
 template <bool precise, typename BSONValidator>
@@ -538,7 +559,7 @@ private:
 
         if constexpr (precise) {
             auto nameLen = obj - _currElem;
-            _currFrame->elem = BSONElement(_currElem, nameLen, nameLen + len);
+            _currFrame->elem = BSONElement(_currElem, nameLen, BSONElement::TrustedInitTag{});
         }
         return cursor.ptr;
     }
@@ -730,7 +751,7 @@ class ColumnValidator {
 public:
     static Status doValidateBSONColumn(const char* originalBuffer,
                                        int maxLength,
-                                       BSONValidateMode mode,
+                                       BSONValidateModeEnum mode,
                                        ValidationVersion validationVersion) noexcept {
         // run control pointer through to end of buffer
         // run over literal data as directed by lengths from control
@@ -765,15 +786,15 @@ public:
                     }
                 } else if (bsoncolumn::isUncompressedLiteralControlByte(control)) {
                     int size;
-                    if (MONGO_likely(mode == BSONValidateMode::kDefault))
+                    if (MONGO_likely(mode == BSONValidateModeEnum::kDefault))
                         size = ValidateBuffer<precise, DefaultValidator>(
                                    ptr, end - ptr, DefaultValidator(), validationVersion)
                                    .validateAndMeasureElem();
-                    else if (mode == BSONValidateMode::kExtended)
+                    else if (mode == BSONValidateModeEnum::kExtended)
                         size = ValidateBuffer<precise, ExtendedValidator>(
                                    ptr, end - ptr, ExtendedValidator(), validationVersion)
                                    .validateAndMeasureElem();
-                    else if (mode == BSONValidateMode::kFull)
+                    else if (mode == BSONValidateModeEnum::kFull)
                         size = ValidateBuffer<precise, FullValidator>(
                                    ptr, end - ptr, FullValidator(), validationVersion)
                                    .validateAndMeasureElem();
@@ -816,7 +837,7 @@ public:
 template <bool precise>
 Status _doValidateColumn(const char* originalBuffer,
                          uint64_t maxLength,
-                         BSONValidateMode mode,
+                         BSONValidateModeEnum mode,
                          ValidationVersion validationVersion) {
     if constexpr (precise) {
         // First try validating using the fast but less precise version. That version will return
@@ -839,13 +860,13 @@ Status _doValidateColumn(const char* originalBuffer,
 
 Status validateBSON(const char* originalBuffer,
                     uint64_t maxLength,
-                    BSONValidateMode mode,
+                    BSONValidateModeEnum mode,
                     ValidationVersion validationVersion) noexcept {
-    if (MONGO_likely(mode == BSONValidateMode::kDefault))
+    if (MONGO_likely(mode == BSONValidateModeEnum::kDefault))
         return _doValidate(originalBuffer, maxLength, DefaultValidator(), validationVersion);
-    else if (mode == BSONValidateMode::kExtended)
+    else if (mode == BSONValidateModeEnum::kExtended)
         return _doValidate(originalBuffer, maxLength, ExtendedValidator(), validationVersion);
-    else if (mode == BSONValidateMode::kFull)
+    else if (mode == BSONValidateModeEnum::kFull)
         return ValidateBuffer<true, FullValidator>(
                    originalBuffer, maxLength, FullValidator(), validationVersion)
             .validate();
@@ -854,14 +875,14 @@ Status validateBSON(const char* originalBuffer,
 }
 
 Status validateBSON(const BSONObj& obj,
-                    BSONValidateMode mode,
-                    ValidationVersion validationVersion) {
+                    BSONValidateModeEnum mode,
+                    ValidationVersion validationVersion) noexcept {
     return validateBSON(obj.objdata(), obj.objsize(), mode, validationVersion);
 }
 
 Status validateBSONColumn(const char* originalBuffer,
                           int maxLength,
-                          BSONValidateMode mode,
+                          BSONValidateModeEnum mode,
                           ValidationVersion validationVersion) noexcept {
     return _doValidateColumn<true>(originalBuffer, maxLength, mode, validationVersion);
 }
