@@ -28,20 +28,44 @@
  */
 
 
-#include "mongo/base/status.h"
-#include "mongo/platform/basic.h"
+#include <cstdio>
+#include <cstring>
+#include <ctime>
+#include <memory>
+#include <ostream>
+#include <string>
+#include <utility>
 
+#include "mongo/base/data_type_endian.h"
 #include "mongo/base/data_view.h"
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/base/string_data.h"
 #include "mongo/bson/bson_depth.h"
 #include "mongo/bson/bson_validate.h"
-#include "mongo/bson/util/bsoncolumn_util.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/bson/bsontypes_util.h"
+#include "mongo/bson/oid.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/bson/util/bsoncolumn.h"
 #include "mongo/bson/util/bsoncolumnbuilder.h"
+#include "mongo/bson/util/builder.h"
 #include "mongo/crypto/fle_field_schema_gen.h"
-#include "mongo/db/jsobj.h"
 #include "mongo/db/matcher/expression_type.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/decimal128.h"
 #include "mongo/platform/random.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/stdx/type_traits.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/base64.h"
+#include "mongo/util/time_support.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
 
@@ -49,7 +73,6 @@
 namespace {
 
 using namespace mongo;
-using std::endl;
 using std::unique_ptr;
 
 void appendInvalidStringElement(const char* fieldName, BufBuilder* bb) {
@@ -217,15 +240,15 @@ TEST(BSONValidateExtended, MD5Size) {
     // 16 byte string.
     auto properSizeMD5 = "aaaaaaaaaaaaaaaa";
     BSONObj x1 = BSON("md5" << BSONBinData(properSizeMD5, 16, MD5Type));
-    ASSERT_OK(validateBSON(x1, mongo::BSONValidateMode::kExtended));
-    ASSERT_OK(validateBSON(x1, mongo::BSONValidateMode::kFull));
+    ASSERT_OK(validateBSON(x1, mongo::BSONValidateModeEnum::kExtended));
+    ASSERT_OK(validateBSON(x1, mongo::BSONValidateModeEnum::kFull));
 
     // 15 byte string.
     auto improperSizeMD5 = "aaaaaaaaaaaaaaa";
     BSONObj x2 = BSON("md5" << BSONBinData(improperSizeMD5, 15, MD5Type));
-    Status status = validateBSON(x2, mongo::BSONValidateMode::kExtended);
+    Status status = validateBSON(x2, mongo::BSONValidateModeEnum::kExtended);
     ASSERT_EQ(status.code(), ErrorCodes::NonConformantBSON);
-    status = validateBSON(x2, mongo::BSONValidateMode::kFull);
+    status = validateBSON(x2, mongo::BSONValidateModeEnum::kFull);
     ASSERT_EQ(status.code(), ErrorCodes::NonConformantBSON);
 }
 
@@ -235,15 +258,16 @@ TEST(BSONValidateExtended, BSONArrayIndexes) {
                        << "1"
                        << "b");
     BSONObj x1 = BSON("arr" << BSONArray(arr));
-    ASSERT_OK(validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateMode::kExtended));
-    ASSERT_OK(validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateMode::kFull));
+    ASSERT_OK(validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateModeEnum::kExtended));
+    ASSERT_OK(validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateModeEnum::kFull));
 
 
     arr = BSON("a" << 1 << "b" << 2);
     x1 = BSON("nonNumericalArray" << BSONArray(arr));
-    Status status = validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateMode::kExtended);
+    Status status =
+        validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateModeEnum::kExtended);
     ASSERT_EQ(status, ErrorCodes::NonConformantBSON);
-    status = validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateMode::kFull);
+    status = validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateModeEnum::kFull);
     ASSERT_EQ(status, ErrorCodes::NonConformantBSON);
 
     arr = BSON("1"
@@ -251,9 +275,9 @@ TEST(BSONValidateExtended, BSONArrayIndexes) {
                << "2"
                << "b");
     x1 = BSON("nonSequentialArray" << BSONArray(arr));
-    status = validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateMode::kExtended);
+    status = validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateModeEnum::kExtended);
     ASSERT_EQ(status, ErrorCodes::NonConformantBSON);
-    status = validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateMode::kFull);
+    status = validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateModeEnum::kFull);
     ASSERT_EQ(status, ErrorCodes::NonConformantBSON);
 
     x1 = BSON("nestedArraysAndObjects" << BSONArray(BSON("0"
@@ -265,9 +289,9 @@ TEST(BSONValidateExtended, BSONArrayIndexes) {
                                                                            << "b"))
                                                          << "2"
                                                          << "b")));
-    status = validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateMode::kExtended);
+    status = validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateModeEnum::kExtended);
     ASSERT_EQ(status, ErrorCodes::NonConformantBSON);
-    status = validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateMode::kFull);
+    status = validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateModeEnum::kFull);
     ASSERT_EQ(status, ErrorCodes::NonConformantBSON);
 
     x1 = BSON("longArray" << BSONArray(BSON("0"
@@ -292,8 +316,8 @@ TEST(BSONValidateExtended, BSONArrayIndexes) {
                                             << "j"
                                             << "10"
                                             << "k")));
-    ASSERT_OK(validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateMode::kExtended));
-    ASSERT_OK(validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateMode::kFull));
+    ASSERT_OK(validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateModeEnum::kExtended));
+    ASSERT_OK(validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateModeEnum::kFull));
 
     x1 = BSON("longNonSequentialArray" << BSONArray(BSON("0"
                                                          << "a"
@@ -317,9 +341,9 @@ TEST(BSONValidateExtended, BSONArrayIndexes) {
                                                          << "j"
                                                          << "11"
                                                          << "k")));
-    status = validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateMode::kExtended);
+    status = validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateModeEnum::kExtended);
     ASSERT_EQ(status, ErrorCodes::NonConformantBSON);
-    status = validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateMode::kFull);
+    status = validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateModeEnum::kFull);
     ASSERT_EQ(status, ErrorCodes::NonConformantBSON);
 
     x1 = BSON("validNestedArraysAndObjects"
@@ -330,8 +354,8 @@ TEST(BSONValidateExtended, BSONArrayIndexes) {
                                                                                 << "b"))
                                                               << "4"
                                                               << "b")))));
-    ASSERT_OK(validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateMode::kExtended));
-    ASSERT_OK(validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateMode::kFull));
+    ASSERT_OK(validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateModeEnum::kExtended));
+    ASSERT_OK(validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateModeEnum::kFull));
 
     x1 = BSON("invalidNestedArraysAndObjects"
               << BSON("arr" << BSONArray(BSON("0" << BSON("2" << 1 << "1" << 0 << "1"
@@ -341,9 +365,9 @@ TEST(BSONValidateExtended, BSONArrayIndexes) {
                                                                                 << "b"))
                                                               << "1"
                                                               << "b")))));
-    status = validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateMode::kExtended);
+    status = validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateModeEnum::kExtended);
     ASSERT_EQ(status, ErrorCodes::NonConformantBSON);
-    status = validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateMode::kFull);
+    status = validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateModeEnum::kFull);
     ASSERT_EQ(status, ErrorCodes::NonConformantBSON);
 }
 
@@ -352,55 +376,55 @@ TEST(BSONValidateExtended, BSONUTF8) {
                    << "\x00"
                    << "ValidString2"
                    << "str");
-    ASSERT_OK(validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateMode::kExtended));
-    ASSERT_OK(validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateMode::kFull));
+    ASSERT_OK(validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateModeEnum::kExtended));
+    ASSERT_OK(validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateModeEnum::kFull));
 
     // Invalid UTF-8 - 10000000; leading bit cannot be set for single byte UTF-8.
     x1 = BSON("InvalidOneByteString"
               << "\x80");
-    auto status = validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateMode::kExtended);
+    auto status = validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateModeEnum::kExtended);
     ASSERT_OK(status);
-    status = validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateMode::kFull);
+    status = validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateModeEnum::kFull);
     ASSERT_EQ(status, ErrorCodes::NonConformantBSON);
 
     x1 = BSON("ValidTwoByteString"
               << "\x40\x40");
-    ASSERT_OK(validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateMode::kExtended));
-    ASSERT_OK(validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateMode::kFull));
+    ASSERT_OK(validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateModeEnum::kExtended));
+    ASSERT_OK(validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateModeEnum::kFull));
 
     // Invalid UTF-8 - 11011111 11001111; second bit of second byte cannot be set.
     x1 = BSON("InvalidTwoByteString"
               << "\xDF\xCF");
-    status = validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateMode::kExtended);
+    status = validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateModeEnum::kExtended);
     ASSERT_OK(status);
-    status = validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateMode::kFull);
+    status = validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateModeEnum::kFull);
     ASSERT_EQ(status, ErrorCodes::NonConformantBSON);
 
     x1 = BSON("ValidThreeByteString"
               << "\x40\x40\x40");
-    ASSERT_OK(validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateMode::kExtended));
-    ASSERT_OK(validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateMode::kFull));
+    ASSERT_OK(validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateModeEnum::kExtended));
+    ASSERT_OK(validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateModeEnum::kFull));
 
     // Invalid UTF-8 - 11101111 10111111 11111111 - second bit of third byte cannot be set.
     x1 = BSON("InvalidThreeByteString"
               << "\xEF\xBF\xFF");
-    status = validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateMode::kExtended);
+    status = validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateModeEnum::kExtended);
     ASSERT_OK(status);
-    status = validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateMode::kFull);
+    status = validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateModeEnum::kFull);
     ASSERT_EQ(status, ErrorCodes::NonConformantBSON);
 
     x1 = BSON("ValidFourByteString"
               << "\x40\x40\x40\x40");
-    ASSERT_OK(validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateMode::kExtended));
-    ASSERT_OK(validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateMode::kFull));
+    ASSERT_OK(validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateModeEnum::kExtended));
+    ASSERT_OK(validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateModeEnum::kFull));
 
     // Invalid UTF-8 - 11110000 10011000 10011010 11111111 - second bit of fourth byte cannot be
     // set.
     x1 = BSON("InvalidFourByteString"
               << "\xF0\x98\x9A\xFF");
-    status = validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateMode::kExtended);
+    status = validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateModeEnum::kExtended);
     ASSERT_OK(status);
-    status = validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateMode::kFull);
+    status = validateBSON(x1.objdata(), x1.objsize(), mongo::BSONValidateModeEnum::kFull);
     ASSERT_EQ(status, ErrorCodes::NonConformantBSON);
 }
 
@@ -759,8 +783,9 @@ TEST(BSONValidateExtended, RegexOptions) {
     // order) throws a warning.
     std::pair<Status, Status> stats{Status::OK(), Status::OK()};
     auto fullyValidate = [&](BSONObj obj) {
-        return std::pair{validateBSON(obj.objdata(), obj.objsize(), BSONValidateMode::kExtended),
-                         validateBSON(obj.objdata(), obj.objsize(), BSONValidateMode::kFull)};
+        return std::pair{
+            validateBSON(obj.objdata(), obj.objsize(), BSONValidateModeEnum::kExtended),
+            validateBSON(obj.objdata(), obj.objsize(), BSONValidateModeEnum::kFull)};
     };
     BSONObj obj = BSON("a" << BSONRegEx("a*.conn", "ilmsux"));
     stats = fullyValidate(obj);
@@ -787,8 +812,8 @@ TEST(BSONValidateExtended, UUIDLength) {
     // Checks that an invalid UUID length (!= 16 bytes) throws a warning.
     std::pair<Status, Status> stats{Status::OK(), Status::OK()};
     auto fullyValidate = [&](BSONObj obj) {
-        return std::pair{validateBSON(obj, BSONValidateMode::kExtended),
-                         validateBSON(obj, BSONValidateMode::kFull)};
+        return std::pair{validateBSON(obj, BSONValidateModeEnum::kExtended),
+                         validateBSON(obj, BSONValidateModeEnum::kFull)};
     };
     BSONObj x = BSON("u" << BSONBinData("de", 2, BinDataType::newUUID));
     stats = fullyValidate(x);
@@ -808,47 +833,48 @@ TEST(BSONValidateExtended, UUIDLength) {
 
 TEST(BSONValidateExtended, DeprecatedTypes) {
     BSONObj obj = BSON("a" << BSONUndefined);
-    Status status = validateBSON(obj, BSONValidateMode::kExtended);
+    Status status = validateBSON(obj, BSONValidateModeEnum::kExtended);
     ASSERT_EQ(status.code(), ErrorCodes::NonConformantBSON);
-    status = validateBSON(obj, BSONValidateMode::kFull);
+    status = validateBSON(obj, BSONValidateModeEnum::kFull);
     ASSERT_EQ(status.code(), ErrorCodes::NonConformantBSON);
 
     obj = BSON("b" << BSONDBRef("db", OID("dbdbdbdbdbdbdbdbdbdbdbdb")));
-    status = validateBSON(obj, BSONValidateMode::kExtended);
+    status = validateBSON(obj, BSONValidateModeEnum::kExtended);
     ASSERT_EQ(status.code(), ErrorCodes::NonConformantBSON);
-    status = validateBSON(obj, BSONValidateMode::kFull);
+    status = validateBSON(obj, BSONValidateModeEnum::kFull);
     ASSERT_EQ(status.code(), ErrorCodes::NonConformantBSON);
 
     obj = BSON("c" << BSONSymbol("symbol"));
-    status = validateBSON(obj, BSONValidateMode::kExtended);
+    status = validateBSON(obj, BSONValidateModeEnum::kExtended);
     ASSERT_EQ(status.code(), ErrorCodes::NonConformantBSON);
-    status = validateBSON(obj, BSONValidateMode::kFull);
+    status = validateBSON(obj, BSONValidateModeEnum::kFull);
     ASSERT_EQ(status.code(), ErrorCodes::NonConformantBSON);
 
     obj = BSON("d" << BSONCodeWScope("(function(){})();", BSON("a" << 1)));
-    status = validateBSON(obj, BSONValidateMode::kExtended);
+    status = validateBSON(obj, BSONValidateModeEnum::kExtended);
     ASSERT_EQ(status.code(), ErrorCodes::NonConformantBSON);
-    status = validateBSON(obj, BSONValidateMode::kFull);
+    status = validateBSON(obj, BSONValidateModeEnum::kFull);
     ASSERT_EQ(status.code(), ErrorCodes::NonConformantBSON);
 
     obj = BSON("e" << BSONBinData("", 0, ByteArrayDeprecated));
-    status = validateBSON(obj, BSONValidateMode::kExtended);
+    status = validateBSON(obj, BSONValidateModeEnum::kExtended);
     ASSERT_EQ(status.code(), ErrorCodes::NonConformantBSON);
-    status = validateBSON(obj, BSONValidateMode::kFull);
+    status = validateBSON(obj, BSONValidateModeEnum::kFull);
     ASSERT_EQ(status.code(), ErrorCodes::NonConformantBSON);
 
     obj = BSON("f" << BSONBinData("", 0, bdtUUID));
-    status = validateBSON(obj, BSONValidateMode::kExtended);
+    status = validateBSON(obj, BSONValidateModeEnum::kExtended);
     ASSERT_EQ(status.code(), ErrorCodes::NonConformantBSON);
-    status = validateBSON(obj, BSONValidateMode::kFull);
+    status = validateBSON(obj, BSONValidateModeEnum::kFull);
     ASSERT_EQ(status.code(), ErrorCodes::NonConformantBSON);
 }
 
 TEST(BSONValidateExtended, DuplicateFieldNames) {
     std::pair<Status, Status> stats{Status::OK(), Status::OK()};
     auto fullyValidate = [&](BSONObj obj) {
-        return std::pair{validateBSON(obj.objdata(), obj.objsize(), BSONValidateMode::kExtended),
-                         validateBSON(obj.objdata(), obj.objsize(), BSONValidateMode::kFull)};
+        return std::pair{
+            validateBSON(obj.objdata(), obj.objsize(), BSONValidateModeEnum::kExtended),
+            validateBSON(obj.objdata(), obj.objsize(), BSONValidateModeEnum::kFull)};
     };
     BSONObj x = BSON("a" << 1 << "b" << 2);
     stats = fullyValidate(x);
@@ -881,7 +907,7 @@ TEST(BSONValidateExtended, BSONEncryptedValue) {
         fle = BSONBinData(
             reinterpret_cast<const void*>(&blob), sizeof(FleBlobHeader), BinDataType::Encrypt);
         obj = BSON("a" << fle);
-        status = validateBSON(obj, BSONValidateMode::kExtended);
+        status = validateBSON(obj, BSONValidateModeEnum::kExtended);
         ASSERT_OK(status);
     }
 
@@ -893,7 +919,7 @@ TEST(BSONValidateExtended, BSONEncryptedValue) {
         fle = BSONBinData(
             reinterpret_cast<const void*>(&blob), sizeof(FleBlobHeader), BinDataType::Encrypt);
         obj = BSON("a" << fle);
-        status = validateBSON(obj, BSONValidateMode::kExtended);
+        status = validateBSON(obj, BSONValidateModeEnum::kExtended);
         ASSERT_OK(status);
     }
 
@@ -905,7 +931,7 @@ TEST(BSONValidateExtended, BSONEncryptedValue) {
         fle = BSONBinData(
             reinterpret_cast<const void*>(&blob), sizeof(FleBlobHeader), BinDataType::Encrypt);
         obj = BSON("a" << fle);
-        status = validateBSON(obj, BSONValidateMode::kExtended);
+        status = validateBSON(obj, BSONValidateModeEnum::kExtended);
         ASSERT_OK(status);
     }
 
@@ -918,7 +944,7 @@ TEST(BSONValidateExtended, BSONEncryptedValue) {
         fle = BSONBinData(
             reinterpret_cast<const void*>(&blob), sizeof(FleBlobHeader), BinDataType::Encrypt);
         obj = BSON("a" << fle);
-        status = validateBSON(obj, BSONValidateMode::kExtended);
+        status = validateBSON(obj, BSONValidateModeEnum::kExtended);
         ASSERT_OK(status);
     }
 
@@ -930,7 +956,7 @@ TEST(BSONValidateExtended, BSONEncryptedValue) {
         fle = BSONBinData(
             reinterpret_cast<const void*>(&blob), sizeof(FleBlobHeader), BinDataType::Encrypt);
         obj = BSON("a" << fle);
-        status = validateBSON(obj, BSONValidateMode::kExtended);
+        status = validateBSON(obj, BSONValidateModeEnum::kExtended);
         ASSERT_OK(status);
     }
 
@@ -943,7 +969,7 @@ TEST(BSONValidateExtended, BSONEncryptedValue) {
         fle = BSONBinData(
             reinterpret_cast<const void*>(&blob), sizeof(FleBlobHeader), BinDataType::Encrypt);
         obj = BSON("a" << fle);
-        status = validateBSON(obj, BSONValidateMode::kExtended);
+        status = validateBSON(obj, BSONValidateModeEnum::kExtended);
         ASSERT_OK(status);
     }
 
@@ -951,9 +977,9 @@ TEST(BSONValidateExtended, BSONEncryptedValue) {
         // Empty Encrypted BSON Value.
         auto emptyBinData = "";
         obj = BSON("a" << BSONBinData(emptyBinData, 0, Encrypt));
-        status = validateBSON(obj, mongo::BSONValidateMode::kExtended);
+        status = validateBSON(obj, mongo::BSONValidateModeEnum::kExtended);
         ASSERT_EQ(status.code(), ErrorCodes::NonConformantBSON);
-        status = validateBSON(obj, mongo::BSONValidateMode::kFull);
+        status = validateBSON(obj, mongo::BSONValidateModeEnum::kFull);
         ASSERT_EQ(status.code(), ErrorCodes::NonConformantBSON);
     }
 
@@ -964,9 +990,9 @@ TEST(BSONValidateExtended, BSONEncryptedValue) {
         fle = BSONBinData(
             reinterpret_cast<const void*>(&blob), sizeof(FleBlobHeader), BinDataType::Encrypt);
         obj = BSON("a" << fle);
-        status = validateBSON(obj, mongo::BSONValidateMode::kExtended);
+        status = validateBSON(obj, mongo::BSONValidateModeEnum::kExtended);
         ASSERT_EQ(status.code(), ErrorCodes::NonConformantBSON);
-        status = validateBSON(obj, mongo::BSONValidateMode::kFull);
+        status = validateBSON(obj, mongo::BSONValidateModeEnum::kFull);
         ASSERT_EQ(status.code(), ErrorCodes::NonConformantBSON);
     }
 
@@ -978,9 +1004,9 @@ TEST(BSONValidateExtended, BSONEncryptedValue) {
         fle = BSONBinData(
             reinterpret_cast<const void*>(&blob), sizeof(FleBlobHeader) - 1, BinDataType::Encrypt);
         obj = BSON("a" << fle);
-        status = validateBSON(obj, mongo::BSONValidateMode::kExtended);
+        status = validateBSON(obj, mongo::BSONValidateModeEnum::kExtended);
         ASSERT_EQ(status.code(), ErrorCodes::NonConformantBSON);
-        status = validateBSON(obj, mongo::BSONValidateMode::kFull);
+        status = validateBSON(obj, mongo::BSONValidateModeEnum::kFull);
         ASSERT_EQ(status.code(), ErrorCodes::NonConformantBSON);
     }
 
@@ -992,9 +1018,9 @@ TEST(BSONValidateExtended, BSONEncryptedValue) {
         fle = BSONBinData(
             reinterpret_cast<const void*>(&blob), sizeof(FleBlobHeader), BinDataType::Encrypt);
         obj = BSON("a" << fle);
-        status = validateBSON(obj, mongo::BSONValidateMode::kExtended);
+        status = validateBSON(obj, mongo::BSONValidateModeEnum::kExtended);
         ASSERT_EQ(status.code(), ErrorCodes::NonConformantBSON);
-        status = validateBSON(obj, mongo::BSONValidateMode::kFull);
+        status = validateBSON(obj, mongo::BSONValidateModeEnum::kFull);
         ASSERT_EQ(status.code(), ErrorCodes::NonConformantBSON);
     }
 }
@@ -1002,9 +1028,9 @@ TEST(BSONValidateExtended, BSONEncryptedValue) {
 TEST(BSONValidateExtended, UnknownBinDataType) {
     BSONObj obj = BSON("unknownBinData" << BSONBinData("", 0, static_cast<BinDataType>(42)));
 
-    Status status = validateBSON(obj, BSONValidateMode::kExtended);
+    Status status = validateBSON(obj, BSONValidateModeEnum::kExtended);
     ASSERT_EQ(status.code(), ErrorCodes::NonConformantBSON);
-    status = validateBSON(obj, BSONValidateMode::kFull);
+    status = validateBSON(obj, BSONValidateModeEnum::kFull);
     ASSERT_EQ(status.code(), ErrorCodes::NonConformantBSON);
 }
 
@@ -1018,21 +1044,21 @@ TEST(BSONValidateColumn, BSONColumnInBSON) {
     cb.append(BSON("a" << 1).getField("a"));
     BSONBinData columnData = cb.finalize();
     BSONObj obj = BSON("a" << columnData);
-    Status status = validateBSON(obj, BSONValidateMode::kDefault);
+    Status status = validateBSON(obj, BSONValidateModeEnum::kDefault);
     ASSERT_OK(status);
-    status = validateBSON(obj, BSONValidateMode::kExtended);
+    status = validateBSON(obj, BSONValidateModeEnum::kExtended);
     ASSERT_OK(status);
-    status = validateBSON(obj, BSONValidateMode::kFull);
+    status = validateBSON(obj, BSONValidateModeEnum::kFull);
     ASSERT_OK(status);
 
     // Change one important byte.
     ((char*)columnData.data)[0] = '0';
     obj = BSON("a" << columnData);
-    status = validateBSON(obj, BSONValidateMode::kDefault);
+    status = validateBSON(obj, BSONValidateModeEnum::kDefault);
     ASSERT_EQ(status.code(), ErrorCodes::NonConformantBSON);
-    status = validateBSON(obj, BSONValidateMode::kExtended);
+    status = validateBSON(obj, BSONValidateModeEnum::kExtended);
     ASSERT_EQ(status.code(), ErrorCodes::NonConformantBSON);
-    status = validateBSON(obj, BSONValidateMode::kFull);
+    status = validateBSON(obj, BSONValidateModeEnum::kFull);
     ASSERT_EQ(status.code(), ErrorCodes::NonConformantBSON);
 }
 
@@ -1052,27 +1078,27 @@ TEST(BSONValidateColumn, BSONColumnInBSONRespectsVersion) {
     obj = BSON("a" << columnData);
 
     // Default refuses bad column
-    Status status = validateBSON(obj, BSONValidateMode::kDefault);
+    Status status = validateBSON(obj, BSONValidateModeEnum::kDefault);
     ASSERT_EQ(status.code(), ErrorCodes::NonConformantBSON);
-    status = validateBSON(obj, BSONValidateMode::kExtended);
+    status = validateBSON(obj, BSONValidateModeEnum::kExtended);
     ASSERT_EQ(status.code(), ErrorCodes::NonConformantBSON);
-    status = validateBSON(obj, BSONValidateMode::kFull);
+    status = validateBSON(obj, BSONValidateModeEnum::kFull);
     ASSERT_EQ(status.code(), ErrorCodes::NonConformantBSON);
 
     // V2 refuses bad column
-    status = validateBSON(obj, BSONValidateMode::kDefault, mongo::V2_Column);
+    status = validateBSON(obj, BSONValidateModeEnum::kDefault, mongo::V2_Column);
     ASSERT_EQ(status.code(), ErrorCodes::NonConformantBSON);
-    status = validateBSON(obj, BSONValidateMode::kExtended, mongo::V2_Column);
+    status = validateBSON(obj, BSONValidateModeEnum::kExtended, mongo::V2_Column);
     ASSERT_EQ(status.code(), ErrorCodes::NonConformantBSON);
-    status = validateBSON(obj, BSONValidateMode::kFull, mongo::V2_Column);
+    status = validateBSON(obj, BSONValidateModeEnum::kFull, mongo::V2_Column);
     ASSERT_EQ(status.code(), ErrorCodes::NonConformantBSON);
 
     // V1 passes bad column on default/extended, refuses on full
-    status = validateBSON(obj, BSONValidateMode::kDefault, mongo::V1_Original);
+    status = validateBSON(obj, BSONValidateModeEnum::kDefault, mongo::V1_Original);
     ASSERT_OK(status);
-    status = validateBSON(obj, BSONValidateMode::kExtended, mongo::V1_Original);
+    status = validateBSON(obj, BSONValidateModeEnum::kExtended, mongo::V1_Original);
     ASSERT_OK(status);
-    status = validateBSON(obj, BSONValidateMode::kFull, mongo::V1_Original);
+    status = validateBSON(obj, BSONValidateModeEnum::kFull, mongo::V1_Original);
     ASSERT_EQ(status.code(), ErrorCodes::NonConformantBSON);
 }
 
@@ -1121,7 +1147,8 @@ TEST(BSONValidateColumn, BSONColumnNoOverflowMissingAllEOOInColumn) {
         if (((char*)columnData.data)[i] == 0)
             ((char*)columnData.data)[i] = 1;
     BSONObj obj = BSON("a" << columnData);
-    ASSERT_EQ(validateBSON(obj, BSONValidateMode::kDefault).code(), ErrorCodes::NonConformantBSON);
+    ASSERT_EQ(validateBSON(obj, BSONValidateModeEnum::kDefault).code(),
+              ErrorCodes::NonConformantBSON);
 }
 
 TEST(BSONValidateColumn, BSONColumnTrailingGarbage) {
@@ -1308,17 +1335,40 @@ TEST(BSONValidateColumn, BSONColumnBadExtendedSelector) {
     ASSERT_OK(validateBSONColumn((char*)columnData.data, columnData.length));
 }
 
+TEST(BSONValidateColumn, BSONColumnInterestingFuzzerInputs) {
+    std::vector<std::string> base64s = {
+        "EI8AAAAADAA=",                  // fieldname not empty
+        "CgCwCrCwsLCwsLCwsAEAAAAAAAAA",  // non-zero delta off a jstNULL
+
+        // non-zero delta off of strlength > 16
+        "BQAjAAAAIwAAAAAAAABASHM/oaGhoaGhAAH1Tq9n//8AAAAAAABASHM/oaGhoaGhAAH1Tq9n//8AAAEA"};
+
+    for (auto& base64 : base64s) {
+        std::string data = base64::decode(base64.c_str());
+
+        // Fuzzer checks that validateBSONColumn keeps us from
+        // decoding strings with structural problems (incorrect strings
+        // that handle gracefully are not currently guaranteed
+        // to fail validation)
+        try {
+            if (validateBSONColumn(data.data(), data.size()).isOK())
+                BSONColumn(data.data(), data.size()).size();
+        } catch (...) {
+        }
+    }
+}
+
 TEST(BSONValidateColumn, BSONColumnWithCodeWScope) {
     BSONObj obj = BSON("a" << BSONCodeWScope("code", BSON("c" << 1)));
     BSONColumnBuilder cb;
     cb.append(obj.getField("a"));
     BSONBinData columnData = cb.finalize();
     ASSERT_OK(validateBSONColumn((char*)columnData.data, columnData.length));
+    ASSERT_FALSE(validateBSONColumn(
+                     (char*)columnData.data, columnData.length, BSONValidateModeEnum::kExtended)
+                     .isOK());
     ASSERT_FALSE(
-        validateBSONColumn((char*)columnData.data, columnData.length, BSONValidateMode::kExtended)
-            .isOK());
-    ASSERT_FALSE(
-        validateBSONColumn((char*)columnData.data, columnData.length, BSONValidateMode::kFull)
+        validateBSONColumn((char*)columnData.data, columnData.length, BSONValidateModeEnum::kFull)
             .isOK());
 }
 
@@ -1331,11 +1381,11 @@ TEST(BSONValidateColumn, BSONColumnWithArrayNestedCodeWScope) {
     cb.append(array.arr());
     BSONBinData columnData = cb.finalize();
     ASSERT_OK(validateBSONColumn((char*)columnData.data, columnData.length));
+    ASSERT_FALSE(validateBSONColumn(
+                     (char*)columnData.data, columnData.length, BSONValidateModeEnum::kExtended)
+                     .isOK());
     ASSERT_FALSE(
-        validateBSONColumn((char*)columnData.data, columnData.length, BSONValidateMode::kExtended)
-            .isOK());
-    ASSERT_FALSE(
-        validateBSONColumn((char*)columnData.data, columnData.length, BSONValidateMode::kFull)
+        validateBSONColumn((char*)columnData.data, columnData.length, BSONValidateModeEnum::kFull)
             .isOK());
 }
 
@@ -1345,11 +1395,11 @@ TEST(BSONValidateColumn, BSONColumnWithObjectNestedCodeWScope) {
     cb.append(BSON("a" << obj));
     BSONBinData columnData = cb.finalize();
     ASSERT_OK(validateBSONColumn((char*)columnData.data, columnData.length));
+    ASSERT_FALSE(validateBSONColumn(
+                     (char*)columnData.data, columnData.length, BSONValidateModeEnum::kExtended)
+                     .isOK());
     ASSERT_FALSE(
-        validateBSONColumn((char*)columnData.data, columnData.length, BSONValidateMode::kExtended)
-            .isOK());
-    ASSERT_FALSE(
-        validateBSONColumn((char*)columnData.data, columnData.length, BSONValidateMode::kFull)
+        validateBSONColumn((char*)columnData.data, columnData.length, BSONValidateModeEnum::kFull)
             .isOK());
 }
 
