@@ -29,8 +29,19 @@
 
 #pragma once
 
+#include <array>
 #include <bitset>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <cstddef>
+#include <cstring>
+#include <fmt/format.h>
+#include <functional>
+#include <iosfwd>
+#include <iterator>
+#include <limits>
 #include <list>
+#include <memory>
 #include <set>
 #include <string>
 #include <type_traits>
@@ -38,15 +49,22 @@
 #include <vector>
 
 #include "mongo/base/data_type.h"
+#include "mongo/base/data_type_endian.h"
+#include "mongo/base/data_view.h"
+#include "mongo/base/static_assert.h"
+#include "mongo/base/status.h"
 #include "mongo/base/string_data.h"
-#include "mongo/base/string_data_comparator_interface.h"
+#include "mongo/base/string_data_comparator.h"
 #include "mongo/bson/bson_comparator_interface_base.h"
 #include "mongo/bson/bsonelement.h"
 #include "mongo/bson/bsontypes.h"
 #include "mongo/bson/oid.h"
 #include "mongo/bson/timestamp.h"
 #include "mongo/bson/util/builder.h"
+#include "mongo/bson/util/builder_fwd.h"
 #include "mongo/platform/atomic_word.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/bufreader.h"
 #include "mongo/util/shared_buffer.h"
 #include "mongo/util/string_map.h"
@@ -54,6 +72,7 @@
 namespace mongo {
 
 class BSONObjBuilder;
+
 class BSONObjStlIterator;
 class ExtendedCanonicalV200Generator;
 class ExtendedRelaxedV200Generator;
@@ -370,8 +389,12 @@ public:
     /**
      * Get the field of the specified name. eoo() is true on the returned
      * element if not found.
+     *
+     * BSONElement will point to data which may only be owned by `this`.
+     * Thus, this BSONObj must outlive the returned BSONElement, hence the lifetime bound
+     * annotation.
      */
-    BSONElement getField(StringData name) const;
+    BSONElement getField(StringData name) const MONGO_COMPILER_LIFETIME_BOUND;
 
     /**
      * Get several fields at once. This is faster than separate getField() calls as the size of
@@ -537,12 +560,12 @@ public:
     int woCompare(const BSONObj& r,
                   const Ordering& o,
                   ComparisonRulesSet rules = ComparisonRules::kConsiderFieldName,
-                  const StringData::ComparatorInterface* comparator = nullptr) const;
+                  const StringDataComparator* comparator = nullptr) const;
 
     int woCompare(const BSONObj& r,
                   const BSONObj& ordering = BSONObj(),
                   ComparisonRulesSet rules = ComparisonRules::kConsiderFieldName,
-                  const StringData::ComparatorInterface* comparator = nullptr) const;
+                  const StringDataComparator* comparator = nullptr) const;
 
     DeferredComparison operator<(const BSONObj& other) const {
         return DeferredComparison(DeferredComparison::Type::kLT, *this, other);
@@ -701,6 +724,10 @@ public:
         return sizeof(BSONObj) + objsize();
     }
 
+    ConstDataRange asDataRange() const {
+        return ConstDataRange(objdata(), objsize());
+    }
+
 private:
     static constexpr char kEmptyObjectPrototype[] = {/*size*/ kMinBSONLength, 0, 0, 0, /*eoo*/ 0};
 
@@ -768,14 +795,20 @@ public:
     /**
      * Constructs an iterator pointing to the first element in obj or EOO if it is empty.
      */
-    explicit BSONObjStlIterator(const BSONObj& obj) : BSONObjStlIterator(obj.firstElement()) {}
+    explicit BSONObjStlIterator(const BSONObj& obj) {
+        auto ptr = obj.objdata() + sizeof(int32_t);
+        _cur._data = ptr;
+        while (*ptr)
+            ++ptr;
+        _cur._fieldNameSize = ptr - _cur._data;
+    }
 
     /**
      * Returns an iterator pointing to the EOO element in obj.
      */
     static BSONObjStlIterator endOf(const BSONObj& obj) {
         auto eooElem = BSONElement();
-        eooElem.data = (obj.objdata() + obj.objsize() - 1);
+        eooElem._data = obj.objdata() + obj.objsize() - 1;
         dassert(eooElem.eoo());  // This is checked in the BSONObj constructor.
         return BSONObjStlIterator(eooElem);
     }
@@ -784,8 +817,15 @@ public:
      * pre-increment
      */
     BSONObjStlIterator& operator++() {
-        dassert(!_cur.eoo());
-        *this = BSONObjStlIterator(BSONElement(_cur.rawdata() + _cur.size()));
+        dassert(*_cur._data);                          // Must not be EOO
+        auto ptr = _cur._data + _cur.fieldNameSize();  // ptr points to the 0 byte ending the name
+
+        ptr += _cur.valuesize() + 1;  // Skip the value and the 0 byte
+        _cur._data = ptr;
+        while (*ptr)
+            ++ptr;
+
+        _cur._fieldNameSize = ptr - _cur._data;
         return *this;
     }
 
@@ -883,7 +923,7 @@ public:
     }
 
     BSONElement next() {
-        MONGO_verify(_pos <= _theend);
+        dassert(_pos <= _theend);
         BSONElement e(_pos);
         _pos += e.size();
         return e;
@@ -907,7 +947,7 @@ public:
     }
 
     BSONElement operator*() {
-        MONGO_verify(_pos <= _theend);
+        dassert(_pos <= _theend);
         return BSONElement(_pos);
     }
 
@@ -933,37 +973,27 @@ class BSONIteratorSorted {
     BSONIteratorSorted& operator=(const BSONIteratorSorted&) = delete;
 
 public:
-    ~BSONIteratorSorted() {
-        MONGO_verify(_fields);
-    }
-
     bool more() {
-        return _cur < _nfields;
+        return _cur < static_cast<int>(_fields.size());
     }
 
     BSONElement next() {
-        MONGO_verify(_fields);
-        if (_cur < _nfields) {
-            const auto& element = _fields[_cur++];
-            return BSONElement(element.fieldName.rawData() - 1,  // Include type byte
-                               element.fieldName.size() + 1,     // Add null terminator
-                               element.totalSize);
+        if (_cur < static_cast<int>(_fields.size())) {
+            const auto& fieldName = _fields.at(_cur++);
+            return BSONElement(fieldName.rawData() - 1,  // Include type byte
+                               fieldName.size() + 1,     // Add null terminator
+                               BSONElement::TrustedInitTag{});
         }
 
         return BSONElement();
     }
 
 protected:
-    class ElementFieldCmp;
-    BSONIteratorSorted(const BSONObj& o, const ElementFieldCmp& cmp);
+    class FieldNameCmp;
+    BSONIteratorSorted(const BSONObj& o, const FieldNameCmp& cmp);
 
 private:
-    const int _nfields;
-    struct Field {
-        StringData fieldName;
-        int totalSize;
-    };
-    const std::unique_ptr<Field[]> _fields;
+    std::vector<StringData> _fields;
     int _cur;
 };
 
