@@ -29,9 +29,19 @@
 
 #pragma once
 
+#include <compare>
+#include <cstdint>
+#include <iosfwd>
+#include <string>
+#include <tuple>
+
 #include "mongo/base/data_view.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsontypes.h"
 #include "mongo/bson/util/builder.h"
+#include "mongo/bson/util/builder_fwd.h"
 #include "mongo/util/assert_util.h"
+#include "mongo/util/duration.h"
 #include "mongo/util/time_support.h"
 
 namespace mongo {
@@ -135,7 +145,7 @@ public:
     // name. This lives here because Timestamp manages its own serialization format.
 
     template <class Builder>
-    void append(Builder& builder, const StringData& fieldName) const {
+    void append(Builder& builder, StringData fieldName) const {
         // No endian conversions needed, since we store in-memory representation
         // in little endian format, regardless of target endian.
         builder.appendNum(static_cast<char>(bsonTimestamp));
