@@ -29,35 +29,40 @@
 
 #pragma once
 
+#include <boost/optional.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/static_assert.hpp>
 #include <cfloat>
 #include <cinttypes>
+#include <climits>
 #include <cstdint>
 #include <cstdio>
+#include <cstdlib>
 #include <cstring>
 #include <sstream>
 #include <string>
+#include <string_view>
 #include <type_traits>
-
-#include <boost/optional.hpp>
-
-#include "mongo/bson/util/builder_fwd.h"
+#include <utility>
 
 #include "mongo/base/data_type_endian.h"
 #include "mongo/base/data_view.h"
+#include "mongo/base/error_codes.h"
 #include "mongo/base/static_assert.h"
 #include "mongo/base/string_data.h"
 #include "mongo/bson/bsontypes.h"
+#include "mongo/bson/util/builder_fwd.h"
 #include "mongo/platform/bits.h"
 #include "mongo/platform/compiler.h"
 #include "mongo/platform/decimal128.h"
 #include "mongo/stdx/type_traits.h"
 #include "mongo/util/allocator.h"
 #include "mongo/util/assert_util.h"
-#include "mongo/util/concepts.h"
 #include "mongo/util/itoa.h"
 #include "mongo/util/shared_buffer.h"
 #include "mongo/util/shared_buffer_fragment.h"
 #include "mongo/util/str_basic.h"
+#include "mongo/util/tracking_allocator.h"
 
 namespace mongo {
 
@@ -89,17 +94,20 @@ static_assert(BufferMaxSize < (1 << 27));
  */
 const int kOpMsgReplyBSONBufferMaxSize = BSONObjMaxUserSize + (64 * 1024);
 
+namespace allocator_aware {
+template <class Allocator = std::allocator<void>>
 class SharedBufferAllocator {
     SharedBufferAllocator(const SharedBufferAllocator&) = delete;
     SharedBufferAllocator& operator=(const SharedBufferAllocator&) = delete;
 
 public:
     SharedBufferAllocator() = default;
-    SharedBufferAllocator(size_t sz) {
+    explicit SharedBufferAllocator(const Allocator& allocator) : _buf(allocator) {}
+    SharedBufferAllocator(size_t sz, const Allocator& allocator = {}) : _buf(allocator) {
         if (sz > 0)
             malloc(sz);
     }
-    SharedBufferAllocator(SharedBuffer buf) : _buf(std::move(buf)) {
+    SharedBufferAllocator(SharedBuffer<Allocator> buf) : _buf(std::move(buf)) {
         invariant(!_buf.isShared());
     }
 
@@ -109,7 +117,7 @@ public:
     SharedBufferAllocator& operator=(SharedBufferAllocator&&) = default;
 
     void malloc(size_t sz) {
-        _buf = SharedBuffer::allocate(sz);
+        _buf = SharedBuffer<Allocator>::allocate(sz, _buf.allocator());
     }
 
     void realloc(size_t sz) {
@@ -117,10 +125,10 @@ public:
     }
 
     void free() {
-        _buf = {};
+        _buf = SharedBuffer<Allocator>{_buf.allocator()};
     }
 
-    SharedBuffer release() {
+    SharedBuffer<Allocator> release() {
         return std::move(_buf);
     }
 
@@ -132,12 +140,18 @@ public:
         return _buf.get();
     }
 
+    Allocator allocator() const {
+        return _buf.allocator();
+    }
+
     // The buffer holder size for 'SharedBufferAllocator' comes from 'SharedBuffer'
-    static constexpr size_t kBuffHolderSize = SharedBuffer::kHolderSize;
+    static constexpr size_t kBuffHolderSize = SharedBuffer<Allocator>::kHolderSize;
 
 private:
-    SharedBuffer _buf;
+    SharedBuffer<Allocator> _buf;
 };
+}  // namespace allocator_aware
+using SharedBufferAllocator = allocator_aware::SharedBufferAllocator<>;
 
 class SharedBufferFragmentAllocator {
     SharedBufferFragmentAllocator(const SharedBufferFragmentAllocator&) = delete;
@@ -258,29 +272,31 @@ public:
 
     void malloc(size_t sz) {
         if (sz > SZ) {
-            _ptr = mongoMalloc(sz);
+            _ptr = ::operator new(sz);
             _capacity = sz;
         }
     }
 
     void realloc(size_t sz) {
-        if (_ptr == _buf) {
-            if (sz > SZ) {
-                _ptr = mongoMalloc(sz);
-                memcpy(_ptr, _buf, SZ);
-                _capacity = sz;
-            } else {
-                _capacity = SZ;
-            }
-        } else {
-            _ptr = mongoRealloc(_ptr, sz);
-            _capacity = sz;
+        if (_ptr == _buf && sz <= SZ) {
+            _capacity = SZ;
+            return;
+        }
+
+        auto oldCap = _capacity;
+        auto oldPtr = _ptr;
+        _ptr = ::operator new(sz);
+        _capacity = sz;
+        std::memcpy(_ptr, oldPtr, std::min(oldCap, sz));
+
+        if (oldPtr != _buf) {
+            ::operator delete(oldPtr, oldCap);
         }
     }
 
     void free() {
         if (_ptr != _buf)
-            ::free(_ptr);
+            ::operator delete(_ptr, _capacity);
         _ptr = _buf;
         _capacity = SZ;
     }
@@ -387,8 +403,8 @@ public:
         appendNumImpl(high);
     }
 
-    REQUIRES_FOR_NON_TEMPLATE(!std::is_same_v<int64_t, long long>)
-    void appendNum(int64_t j) {
+    template <int...>
+    requires(!std::is_same_v<int64_t, long long>) void appendNum(int64_t j) {
         appendNumImpl(j);
     }
 
@@ -499,7 +515,8 @@ public:
      * Replaces the buffer backing this BufBuilder with the passed in SharedBuffer.
      * Only legal to call when this builder is empty and when the SharedBuffer isn't shared.
      */
-    REQUIRES_FOR_NON_TEMPLATE(std::is_same_v<BufferAllocator, SharedBufferAllocator>)
+    template <int...>
+    requires std::is_same_v<BufferAllocator, SharedBufferAllocator>
     void useSharedBuffer(SharedBuffer buf) {
         invariant(len() == 0);  // Can only do this while empty.
         invariant(reservedBytes() == 0);
@@ -621,6 +638,36 @@ public:
     }
 };
 
+// The following extern template declarations must follow BasicBufBuilder and come before their
+// instantiations as base classes for BufBuilder. Do not remove or re-order these lines w.r.t those
+// types without being sure that you are not undoing the advantages of the extern template
+// declarations.
+extern template class BasicBufBuilder<allocator_aware::SharedBufferAllocator<std::allocator<void>>>;
+extern template class BasicBufBuilder<
+    allocator_aware::SharedBufferAllocator<TrackingAllocator<void>>>;
+
+namespace allocator_aware {
+template <class Allocator = std::allocator<void>>
+class BufBuilder : public BasicBufBuilder<SharedBufferAllocator<Allocator>> {
+public:
+    static constexpr size_t kDefaultInitSizeBytes = mongo::BufBuilder::kDefaultInitSizeBytes;
+    BufBuilder(size_t size = kDefaultInitSizeBytes, const Allocator& allocator = {})
+        : BasicBufBuilder<SharedBufferAllocator<Allocator>>(size, allocator) {}
+
+    Allocator allocator() const {
+        return BasicBufBuilder<SharedBufferAllocator<Allocator>>::_buf.allocator();
+    }
+
+    /**
+     * Assume ownership of the buffer.
+     * Note: There should not be any other method calls on this object after a call to 'release'.
+     */
+    SharedBuffer<Allocator> release() {
+        return BasicBufBuilder<SharedBufferAllocator<Allocator>>::_buf.release();
+    }
+};
+}  // namespace allocator_aware
+
 // The following extern template declaration must follow
 // BasicBufBuilder and come before its instantiation as a base class
 // for PooledFragmentBuilder. Do not remove or re-order these lines
@@ -771,8 +818,7 @@ public:
         const int prev = _buf.len();
         const int maxSize = 32;
         char* start = _buf.grow(maxSize);
-        int z = std::isnan(x) ? "nan\0"_sd.copy(start, maxSize) - 1
-                              : snprintf(start, maxSize, "%.16g", x);
+        int z = snprintf(start, maxSize, "%.16g", x);
         MONGO_verify(z >= 0);
         MONGO_verify(z < maxSize);
         _buf.setlen(prev + z);
