@@ -27,9 +27,12 @@
  *    it in the license file.
  */
 
-#include "mongo/unittest/unittest.h"
+#include <limits>
 
 #include "mongo/bson/util/builder.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/str.h"
 
 namespace mongo {
 TEST(Builder, String1) {
@@ -71,7 +74,7 @@ TEST(Builder, StringNulByteHandling) {
         // Since hasNulByte points to a string literal, we know that
         // *(hasNulByte.data() + hasNulByte.size()) is valid and == '\0'
         ASSERT_EQ(StringData(bb.buf(), bb.len()),
-                  StringData(hasNulByte.rawData(), hasNulByte.size() + 1));
+                  StringData(hasNulByte.data(), hasNulByte.size() + 1));
     }
 }
 
