@@ -27,36 +27,49 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kAccessControl
 
-#include "mongo/platform/basic.h"
+#include <boost/smart_ptr.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <utility>
+#include <vector>
 
-#include "mongo/client/authenticate.h"
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
 
+#include "mongo/base/error_codes.h"
 #include "mongo/base/status.h"
 #include "mongo/base/status_with.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
 #include "mongo/bson/bsonobj.h"
-#include "mongo/bson/json.h"
+#include "mongo/bson/bsontypes.h"
 #include "mongo/bson/util/bson_extract.h"
+#include "mongo/bson/util/builder.h"
+#include "mongo/bson/util/builder_fwd.h"
+#include "mongo/client/authenticate.h"
 #include "mongo/client/internal_auth.h"
 #include "mongo/client/sasl_client_authenticate.h"
-#include "mongo/config.h"
+#include "mongo/config.h"  // IWYU pragma: keep
 #include "mongo/db/auth/authorization_manager.h"
 #include "mongo/db/auth/sasl_command_constants.h"
+#include "mongo/db/auth/user.h"
 #include "mongo/db/server_options.h"
+#include "mongo/db/wire_version.h"
+#include "mongo/executor/remote_command_request.h"
+#include "mongo/executor/remote_command_response.h"
 #include "mongo/logv2/log.h"
-#include "mongo/platform/mutex.h"
-#include "mongo/rpc/get_status_from_command_result.h"
-#include "mongo/rpc/op_msg_rpc_impls.h"
-#include "mongo/util/net/ssl_manager.h"
-#include "mongo/util/net/ssl_options.h"
-#include "mongo/util/password_digest.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/future_impl.h"
+#include "mongo/util/str.h"
+
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kAccessControl
+
 
 namespace mongo {
 namespace auth {
 
 using executor::RemoteCommandRequest;
-using executor::RemoteCommandResponse;
 
 using AuthRequest = StatusWith<RemoteCommandRequest>;
 
@@ -101,7 +114,7 @@ StatusWith<OpMsgRequest> createX509AuthCmd(const BSONObj& params, StringData cli
     }
     auto db = extractDBField(params);
     if (!db.isOK())
-        return std::move(db.getStatus());
+        return db.getStatus();
 
     std::string username;
     auto response = bsonExtractStringFieldWithDefault(
@@ -118,10 +131,12 @@ StatusWith<OpMsgRequest> createX509AuthCmd(const BSONObj& params, StringData cli
         return {ErrorCodes::AuthenticationFailed, message.str()};
     }
 
-    return OpMsgRequest::fromDBAndBody(db.getValue(),
-                                       BSON("authenticate" << 1 << "mechanism"
-                                                           << "MONGODB-X509"
-                                                           << "user" << username));
+    return OpMsgRequestBuilder::create(
+        auth::ValidatedTenancyScope::kNotRequired /* db is not tenanted */,
+        AuthDatabaseNameUtil::deserialize(db.getValue()),
+        BSON("authenticate" << 1 << "mechanism"
+                            << "MONGODB-X509"
+                            << "user" << username));
 }
 
 // Use the MONGODB-X509 protocol to authenticate as "username." The certificate details
@@ -142,7 +157,7 @@ Future<void> authX509(RunCommandHook runCommand, const BSONObj& params, StringDa
 
 class DefaultInternalAuthParametersProvider : public InternalAuthParametersProvider {
 public:
-    ~DefaultInternalAuthParametersProvider() = default;
+    ~DefaultInternalAuthParametersProvider() override = default;
 
     BSONObj get(size_t index, StringData mechanism) final {
         return getInternalAuthParams(index, mechanism);
@@ -220,7 +235,10 @@ Future<std::string> negotiateSaslMechanism(RunCommandHook runCommand,
     }
     const auto request = builder.obj();
 
-    return runCommand(OpMsgRequest::fromDBAndBody("admin"_sd, std::move(request)))
+    return runCommand(OpMsgRequestBuilder::create(
+                          auth::ValidatedTenancyScope::kNotRequired /* admin is not per-tenant. */,
+                          DatabaseName::kAdmin,
+                          request))
         .then([](BSONObj reply) -> Future<std::string> {
             auto mechsArrayObj = reply.getField("saslSupportedMechs");
             if (mechsArrayObj.type() != Array) {
@@ -277,12 +295,17 @@ Future<void> authenticateInternalClient(
         });
 }
 
-BSONObj buildAuthParams(StringData dbname,
+BSONObj buildAuthParams(const DatabaseName& dbname,
                         StringData username,
                         StringData passwordText,
                         StringData mechanism) {
-
-    return BSON(saslCommandMechanismFieldName << mechanism << saslCommandUserDBFieldName << dbname
+    // Direct authentication expects no tenantId to be present.
+    fassert(8032000, dbname.tenantId() == boost::none);
+    // Because we assert above there is no TenantId, serialiazing the `dbname` will never include
+    // a tenantId as part of the dbName regardless of the serialization context.
+    SerializationContext sc = SerializationContext::stateDefault();
+    return BSON(saslCommandMechanismFieldName << mechanism << saslCommandUserDBFieldName
+                                              << DatabaseNameUtil::serialize(dbname, sc)
                                               << saslCommandUserFieldName << username
                                               << saslCommandPasswordFieldName << passwordText);
 }
@@ -378,7 +401,7 @@ SpeculativeAuthType speculateAuth(BSONObjBuilder* helloRequestBuilder,
         return SpeculativeAuthType::kNone;
     }
 
-    auto params = std::move(optParams.get());
+    auto params = std::move(optParams.value());
 
     auto ret = _speculateAuth(helloRequestBuilder,
                               mechanism,
