@@ -30,32 +30,45 @@
 // _ todo: reconnect?
 
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/client/connpool.h"
-
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <fmt/format.h>
+// IWYU pragma: no_include "cxxabi.h"
+#include <chrono>
+#include <cstddef>
+#include <exception>
+#include <functional>
 #include <limits>
+#include <ostream>
+#include <set>
 #include <string>
+#include <utility>
 
-#include "mongo/base/init.h"
+#include "mongo/base/error_codes.h"
+#include "mongo/base/init.h"  // IWYU pragma: keep
+#include "mongo/base/initializer.h"
+#include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonobj.h"
 #include "mongo/client/connection_string.h"
+#include "mongo/client/connpool.h"
 #include "mongo/client/dbclient_connection.h"
 #include "mongo/client/global_conn_pool.h"
-#include "mongo/client/replica_set_monitor.h"
-#include "mongo/config.h"
-#include "mongo/db/server_feature_flags_gen.h"
+#include "mongo/config.h"  // IWYU pragma: keep
 #include "mongo/executor/connection_pool_stats.h"
 #include "mongo/logv2/log.h"
-#include "mongo/stdx/chrono.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/compiler.h"
 #include "mongo/util/exit.h"
 #include "mongo/util/fail_point.h"
+#include "mongo/util/net/hostandport.h"
 #include "mongo/util/net/socket_exception.h"
+#include "mongo/util/str.h"
 #include "mongo/util/testing_proctor.h"
 
-#if !defined(__has_feature)
-#define __has_feature(x) 0
-#endif
-
 #if __has_feature(address_sanitizer)
 #include <sanitizer/lsan_interface.h>
 #endif
@@ -74,8 +87,7 @@ auto makeDuration(double secs) {
 }
 
 void recordWaitTime(PoolForHost& p, DBClientBase* conn, Date_t connRequestedAt) {
-    // (Ignore FCV check): This feature flag doesn't have any upgrade/downgrade concerns.
-    if (gFeatureFlagConnHealthMetrics.isEnabledAndIgnoreFCVUnsafe() && conn) {
+    if (conn) {
         p.recordConnectionWaitTime(connRequestedAt);
     }
 }
@@ -83,10 +95,7 @@ void recordWaitTime(PoolForHost& p, DBClientBase* conn, Date_t connRequestedAt)
 
 MONGO_FAIL_POINT_DEFINE(injectWaitTimeForConnpoolAcquisition);
 
-using std::endl;
 using std::list;
-using std::map;
-using std::set;
 using std::string;
 using std::vector;
 
@@ -110,8 +119,6 @@ PoolForHost::~PoolForHost() {
 void PoolForHost::clear() {
     if (!_parentDestroyed) {
         LOGV2(24124,
-              "Dropping all pooled connections to {connString} "
-              "(with timeout of {timeoutSecs} seconds)",
               "Dropping all pooled connections to a host",
               "connString"_attr = _hostName,
               "socketTimeout"_attr = makeDuration(_socketTimeoutSecs));
@@ -135,8 +142,6 @@ auto PoolForHost::done(DBConnectionPool* pool, DBClientBase* c) -> ConnectionHea
     if (isFailed || isBroken) {
         _badConns++;
         LOGV2(24125,
-              "Ending connection to {connString} (with timeout of {socketTimeout}) "
-              "due to bad connection status; {numOpenConns} connections to that host remain open",
               "Ending connection to a host due to a bad connection status",
               "connString"_attr = _hostName,
               "socketTimeout"_attr = makeDuration(_socketTimeoutSecs),
@@ -145,9 +150,6 @@ auto PoolForHost::done(DBConnectionPool* pool, DBClientBase* c) -> ConnectionHea
     } else if (_maxPoolSize >= 0 && static_cast<int>(_pool.size()) >= _maxPoolSize) {
         // We have a pool size that we need to enforce
         LOGV2(24126,
-              "Ending idle connection to {connString} (with timeout of {socketTimeout}) "
-              "because its pool meets constraints; "
-              "{numOpenConns} connections to that host remain open",
               "Ending idle connection to a host because its pool mees constraints",
               "connString"_attr = _hostName,
               "socketTimeout"_attr = makeDuration(_socketTimeoutSecs),
@@ -175,8 +177,6 @@ void PoolForHost::reportBadConnectionAt(uint64_t microSec) {
         microSec > _minValidCreationTimeMicroSec) {
         _minValidCreationTimeMicroSec = microSec;
         LOGV2(24127,
-              "Detected bad connection created at {currentTime}, "
-              "clearing pool for {connString} of {numOpenConns} connections",
               "Detected bad connection, clearing pool for host",
               "currentTime"_attr =
                   Microseconds(static_cast<Microseconds::rep>(_minValidCreationTimeMicroSec)),
@@ -325,8 +325,6 @@ public:
 
                 if (p.openConnections() >= _this->_maxInUse) {
                     LOGV2(20112,
-                          "Too many in-use connections; "
-                          "waiting until there are fewer than {maxInUseConns}",
                           "Too many in-use connections; waiting until there are fewer than maximum",
                           "maxInUseConns"_attr = _this->_maxInUse);
                     p.waitForFreeConnection(timeout, lk);
@@ -335,6 +333,15 @@ public:
                     // _finishCreate will take the lock again.
                     lk.unlock();
 
+                    if constexpr (kDebugBuild) {
+                        LOGV2_DEBUG(7591400,
+                                    1,
+                                    "Creating a new connection",
+                                    "target"_attr = host,
+                                    "timeout"_attr = timeout,
+                                    "requestedAt"_attr = connRequestedAt);
+                    }
+
                     // Create a new connection and return. All Connect functions
                     // should throw if they cannot create a connection.
                     auto c = connect();
@@ -420,9 +427,6 @@ DBClientBase* DBConnectionPool::_finishCreate(const string& ident,
     }
 
     LOGV2(20113,
-          "Successfully connected to {connString} "
-          "({numOpenConns} connections now open to that host "
-          "with a {socketTimeoutSecs} second timeout)",
           "Successfully connected to host",
           "connString"_attr = ident,
           "numOpenConns"_attr = openConnections(ident, socketTimeout),
@@ -477,11 +481,7 @@ Milliseconds DBConnectionPool::getPoolHostConnTime_forTest(const std::string& ho
                                                            double timeout) const {
     if (TestingProctor::instance().isEnabled()) {
         stdx::lock_guard<Latch> L(_mutex);
-        auto it = _pools.find(PoolKey(host, timeout));
-        iassert(9047201,
-                "Couldn't find a connection that matches the provided host and timeout pair!",
-                it != _pools.end());
-        return it->second._connTime;
+        return _pools.find(PoolKey(host, timeout))->second._connTime;
     }
     return Milliseconds();
 }
@@ -561,7 +561,6 @@ void DBConnectionPool::clear() {
     stdx::lock_guard<Latch> L(_mutex);
     LOGV2_DEBUG(20114,
                 2,
-                "Removing connections on all pools owned by {poolName}",
                 "Removing all connectionns associated with this set of pools",
                 "poolName"_attr = _name);
     for (PoolMap::iterator iter = _pools.begin(); iter != _pools.end(); ++iter) {
@@ -571,11 +570,8 @@ void DBConnectionPool::clear() {
 
 void DBConnectionPool::removeHost(const string& host) {
     stdx::lock_guard<Latch> L(_mutex);
-    LOGV2_DEBUG(20115,
-                2,
-                "Removing connections from all pools to {connString}",
-                "Removing connections from all pools to a host",
-                "connString"_attr = host);
+    LOGV2_DEBUG(
+        20115, 2, "Removing connections from all pools to a host", "connString"_attr = host);
     for (PoolMap::iterator i = _pools.begin(); i != _pools.end(); ++i) {
         const string& poolHost = i->first.ident;
         if (!serverNameCompare()(host, poolHost) && !serverNameCompare()(poolHost, host)) {
@@ -640,10 +636,7 @@ void DBConnectionPool::appendConnectionStats(executor::ConnectionPoolStats* stat
                                                    0,
                                                    0,
                                                    Milliseconds{0}};
-            // (Ignore FCV check): This feature flag doesn't have any upgrade/downgrade concerns.
-            if (gFeatureFlagConnHealthMetrics.isEnabledAndIgnoreFCVUnsafe()) {
-                hostStats.acquisitionWaitTimes = i->second.connectionWaitTimeStats();
-            }
+            hostStats.acquisitionWaitTimes = i->second.connectionWaitTimeStats();
             stats->updateStatsForHost("global", host, hostStats);
         }
     }
@@ -786,7 +779,6 @@ ScopedDbConnection::~ScopedDbConnection() {
         } else {
             /* see done() comments above for why we log this line */
             LOGV2(24128,
-                  "Scoped connection to {connString} not being returned to the pool",
                   "Scoped connection not being returned to the pool",
                   "connString"_attr = _conn->getServerAddress());
             kill();
