@@ -32,52 +32,61 @@
  */
 
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/client/dbclient_base.h"
-
-#include <algorithm>
+#include <boost/cstdint.hpp>
+#include <limits>
+#include <ostream>
 #include <utility>
 
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+
 #include "mongo/base/status.h"
 #include "mongo/base/status_with.h"
-#include "mongo/bson/util/bson_extract.h"
-#include "mongo/bson/util/builder.h"
+#include "mongo/bson/bsontypes.h"
 #include "mongo/client/authenticate.h"
 #include "mongo/client/client_api_version_parameters_gen.h"
-#include "mongo/client/constants.h"
+#include "mongo/client/dbclient_base.h"
 #include "mongo/client/dbclient_cursor.h"
-#include "mongo/config.h"
+#include "mongo/client/internal_auth.h"
+#include "mongo/config.h"  // IWYU pragma: keep
 #include "mongo/db/api_parameters_gen.h"
-#include "mongo/db/auth/validated_tenancy_scope.h"
-#include "mongo/db/commands.h"
-#include "mongo/db/json.h"
+#include "mongo/db/auth/user_name.h"
+#include "mongo/db/client.h"
+#include "mongo/db/cursor_id.h"
+#include "mongo/db/dbmessage.h"
+#include "mongo/db/logical_time.h"
 #include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/query/kill_cursors_gen.h"
-#include "mongo/db/server_feature_flags_gen.h"
+#include "mongo/db/repl/read_concern_args.h"
+#include "mongo/db/server_options.h"
+#include "mongo/db/service_context.h"
+#include "mongo/db/tenant_id.h"
 #include "mongo/db/wire_version.h"
 #include "mongo/executor/remote_command_request.h"
-#include "mongo/executor/remote_command_response.h"
 #include "mongo/logv2/log.h"
-#include "mongo/platform/mutex.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
 #include "mongo/rpc/factory.h"
 #include "mongo/rpc/get_status_from_command_result.h"
 #include "mongo/rpc/metadata.h"
-#include "mongo/rpc/metadata/client_metadata.h"
+#include "mongo/rpc/protocol.h"
 #include "mongo/rpc/reply_interface.h"
-#include "mongo/s/stale_exception.h"
 #include "mongo/util/assert_util.h"
-#include "mongo/util/concurrency/mutex.h"
-#include "mongo/util/debug_util.h"
-#include "mongo/util/net/ssl_manager.h"
-#include "mongo/util/net/ssl_options.h"
+#include "mongo/util/database_name_util.h"
+#include "mongo/util/future.h"
+#include "mongo/util/future_impl.h"
+#include "mongo/util/namespace_string_util.h"
+#include "mongo/util/net/hostandport.h"
+#include "mongo/util/str.h"
+#include "mongo/util/uuid.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kNetwork
 
 
 namespace mongo {
 
-using std::endl;
 using std::list;
 using std::string;
 using std::stringstream;
@@ -85,7 +94,6 @@ using std::unique_ptr;
 using std::vector;
 
 using executor::RemoteCommandRequest;
-using executor::RemoteCommandResponse;
 
 AtomicWord<long long> DBClientBase::ConnectionIdSequence;
 
@@ -129,10 +137,6 @@ rpc::UniqueReply DBClientBase::parseCommandReplyMessage(const std::string& host,
         uassertStatusOK(_metadataReader(opCtx, commandReply->getCommandReply(), host));
     }
 
-    // StaleConfig is thrown because clients acting as routers handle the exception at a higher
-    // level. Routing clients only expect StaleConfig from shards, so the exception should not be
-    // thrown when connected to a mongos, which allows StaleConfig to be returned to clients that
-    // connect to a mongos with DBClient, e.g. the shell.
     if (_shouldThrowOnStaleConfigError) {
         auto status = getStatusFromCommandResult(commandReply->getCommandReply());
         if (status == ErrorCodes::StaleConfig) {
@@ -192,9 +196,18 @@ void appendMetadata(OperationContext* opCtx,
 }
 }  // namespace
 
+auth::ValidatedTenancyScope DBClientBase::_createInnerRequestVTS(
+    const boost::optional<TenantId>& tenantId) const {
+    if (tenantId) {
+        return auth::ValidatedTenancyScopeFactory::create(
+            tenantId.get(), auth::ValidatedTenancyScopeFactory::TrustedForInnerOpMsgRequestTag{});
+    }
+    return auth::ValidatedTenancyScope::kNotRequired;
+}
+
 DBClientBase* DBClientBase::runFireAndForgetCommand(OpMsgRequest request) {
     // Make sure to reconnect if needed before building our request.
-    checkConnection();
+    ensureConnection();
 
     auto opCtx = haveClient() ? cc().getOperationContext() : nullptr;
     appendMetadata(opCtx, _metadataWriter, _apiParameters, request);
@@ -207,7 +220,7 @@ DBClientBase* DBClientBase::runFireAndForgetCommand(OpMsgRequest request) {
 std::pair<rpc::UniqueReply, DBClientBase*> DBClientBase::runCommandWithTarget(
     OpMsgRequest request) {
     // Make sure to reconnect if needed before building our request.
-    checkConnection();
+    ensureConnection();
 
     // call() oddly takes this by pointer, so we need to put it on the stack.
     auto host = getServerAddress();
@@ -219,7 +232,7 @@ std::pair<rpc::UniqueReply, DBClientBase*> DBClientBase::runCommandWithTarget(
     Message replyMsg;
 
     try {
-        call(requestMsg, replyMsg, &host);
+        replyMsg = call(requestMsg, &host);
     } catch (DBException& e) {
         e.addContext(str::stream() << str::stream() << "network error while attempting to run "
                                    << "command '" << request.getCommandName() << "' "
@@ -252,26 +265,14 @@ std::tuple<bool, DBClientBase*> DBClientBase::runCommandWithTarget(const Databas
     // TODO: This will be downconverted immediately if the underlying
     // requestBuilder is a legacyRequest builder. Not sure what the best
     // way to get around that is without breaking the abstraction.
-    auto request = _upconvertRequest(dbName, cmd, options);
+    auto request =
+        rpc::upconvertRequest(dbName, cmd, options, _createInnerRequestVTS(dbName.tenantId()));
     auto result = runCommandWithTarget(std::move(request));
 
     info = result.first->getCommandReply().getOwned();
     return std::make_tuple(isOk(info), result.second);
 }
 
-std::tuple<bool, std::shared_ptr<DBClientBase>> DBClientBase::runCommandWithTarget(
-    const DatabaseName& dbName,
-    BSONObj cmd,
-    BSONObj& info,
-    std::shared_ptr<DBClientBase> me,
-    int options) {
-    auto request = _upconvertRequest(dbName, cmd, options);
-    auto result = runCommandWithTarget(std::move(request), std::move(me));
-
-    info = result.first->getCommandReply().getOwned();
-    return std::make_tuple(isOk(info), result.second);
-}
-
 bool DBClientBase::runCommand(const DatabaseName& dbName, BSONObj cmd, BSONObj& info, int options) {
     auto res = runCommandWithTarget(dbName, std::move(cmd), info, options);
     return std::get<0>(res);
@@ -348,7 +349,7 @@ auth::RunCommandHook DBClientBase::_makeAuthRunCommandHook() {
             if (!status.isOK()) {
                 return status;
             }
-            return Future<BSONObj>::makeReady(std::move(ret->getCommandReply()));
+            return Future<BSONObj>::makeReady(ret->getCommandReply());
         } catch (const DBException& e) {
             return Future<BSONObj>::makeReady(e.toStatus());
         }
@@ -371,14 +372,14 @@ void DBClientBase::_auth(const BSONObj& params) {
     auth::authenticateClient(params, remote, clientName, _makeAuthRunCommandHook()).get();
 }
 
-Status DBClientBase::authenticateInternalUser(auth::StepDownBehavior stepDownBehavior) {
+void DBClientBase::authenticateInternalUser(auth::StepDownBehavior stepDownBehavior) {
     ScopedMetadataWriterRemover remover{this};
     if (!auth::isInternalAuthSet()) {
         if (!serverGlobalParams.quiet.load()) {
             LOGV2(20116, "ERROR: No authentication parameters set for internal user");
         }
-        return {ErrorCodes::AuthenticationFailed,
-                "No authentication parameters set for internal user"};
+        uasserted(ErrorCodes::AuthenticationFailed,
+                  "No authentication parameters set for internal user");
     }
 
     // We will only have a client name if SSL is enabled
@@ -391,37 +392,30 @@ Status DBClientBase::authenticateInternalUser(auth::StepDownBehavior stepDownBeh
 #endif
 
     auto authProvider = auth::createDefaultInternalAuthProvider();
-    auto status = auth::authenticateInternalClient(clientName,
-                                                   HostAndPort(getServerAddress()),
-                                                   boost::none,
-                                                   stepDownBehavior,
-                                                   _makeAuthRunCommandHook(),
-                                                   authProvider)
-                      .getNoThrow();
-    if (status.isOK()) {
-        return status;
+    try {
+        auth::authenticateInternalClient(clientName,
+                                         HostAndPort(getServerAddress()),
+                                         boost::none,
+                                         stepDownBehavior,
+                                         _makeAuthRunCommandHook(),
+                                         authProvider)
+            .get();
+    } catch (const DBException& e) {
+        if (!serverGlobalParams.quiet.load()) {
+            LOGV2(20117,
+                  "Can't authenticate as internal user",
+                  "connString"_attr = toString(),
+                  "error"_attr = e.toStatus());
+        }
+        throw;
     }
-
-    if (!serverGlobalParams.quiet.load()) {
-        LOGV2(20117,
-              "Can't authenticate to {connString} as internal user, error: {error}",
-              "Can't authenticate as internal user",
-              "connString"_attr = toString(),
-              "error"_attr = status);
-    }
-
-    return status;
 }
 
 void DBClientBase::auth(const BSONObj& params) {
     _auth(params);
 }
 
-bool DBClientBase::auth(const string& dbname,
-                        const string& username,
-                        const string& password_text,
-                        string& errmsg) {
-
+void DBClientBase::auth(const DatabaseName& dbname, StringData username, StringData password_text) {
     UserName user{username, dbname};
 
     StatusWith<string> mechResult =
@@ -435,33 +429,24 @@ bool DBClientBase::auth(const string& dbname,
     // negotiation does not succeeed for some reason.
     StringData mech = mechResult.isOK() ? mechResult.getValue() : "SCRAM-SHA-1"_sd;
 
-    try {
-        const auto authParams = auth::buildAuthParams(dbname, username, password_text, mech);
-        auth(authParams);
-        return true;
-    } catch (const AssertionException& ex) {
-        if (ex.code() != ErrorCodes::AuthenticationFailed)
-            throw;
-        errmsg = ex.what();
-        return false;
-    }
+    const auto authParams = auth::buildAuthParams(dbname, username, password_text, mech);
+    auth(authParams);
 }
 
-void DBClientBase::logout(const string& dbname, BSONObj& info) {
-    runCommand(DatabaseName(boost::none, dbname), BSON("logout" << 1), info);
+void DBClientBase::logout(const DatabaseName& dbName, BSONObj& info) {
+    runCommand(dbName, BSON("logout" << 1), info);
 }
 
 bool DBClientBase::isPrimary(bool& isPrimary, BSONObj* info) {
     BSONObjBuilder bob;
     bob.append("hello", 1);
-    if (auto wireSpec = WireSpec::instance().get(); wireSpec->isInternalClient) {
-        WireSpec::appendInternalClientWireVersion(wireSpec->outgoing, &bob);
-    }
+    ServiceContext* sc = haveClient() ? cc().getServiceContext() : getGlobalServiceContext();
+    WireSpec::getWireSpec(sc).appendInternalClientWireVersionIfNeeded(&bob);
 
     BSONObj o;
     if (info == nullptr)
         info = &o;
-    bool ok = runCommand(DatabaseName(boost::none, "admin"), bob.obj(), *info);
+    bool ok = runCommand(DatabaseName::kAdmin, bob.obj(), *info);
     isPrimary = info->getField("isWritablePrimary").trueValue();
     return ok;
 }
@@ -511,8 +496,8 @@ list<BSONObj> DBClientBase::getCollectionInfos(const DatabaseName& dbName, const
         const long long id = cursorObj["id"].Long();
 
         if (id != 0) {
-            const NamespaceString nss =
-                NamespaceStringUtil::deserialize(dbName.tenantId(), cursorObj["ns"].String());
+            const NamespaceString nss = NamespaceStringUtil::deserialize(
+                dbName.tenantId(), cursorObj["ns"].String(), SerializationContext::stateDefault());
             unique_ptr<DBClientCursor> cursor = getMore(nss, id);
             while (cursor->more()) {
                 infos.push_back(cursor->nextSafe().getOwned());
@@ -552,7 +537,7 @@ vector<BSONObj> DBClientBase::getDatabaseInfos(const BSONObj& filter,
     BSONObj cmd = bob.done();
 
     BSONObj res;
-    if (runCommand(DatabaseName(boost::none, "admin"), cmd, res, QueryOption_SecondaryOk)) {
+    if (runCommand(DatabaseName::kAdmin, cmd, res, QueryOption_SecondaryOk)) {
         BSONObj dbs = res["databases"].Obj();
         BSONObjIterator it(dbs);
         while (it.more()) {
@@ -623,7 +608,8 @@ unique_ptr<DBClientCursor> DBClientBase::getMore(const NamespaceString& nss, lon
 }
 
 namespace {
-OpMsgRequest createInsertRequest(const NamespaceString& nss,
+OpMsgRequest createInsertRequest(const auth::ValidatedTenancyScope& vts,
+                                 const NamespaceString& nss,
                                  const vector<BSONObj>& v,
                                  bool ordered,
                                  boost::optional<BSONObj> writeConcernObj) {
@@ -633,13 +619,14 @@ OpMsgRequest createInsertRequest(const NamespaceString& nss,
     if (writeConcernObj) {
         cmdBuilder.append(WriteConcernOptions::kWriteConcernField, *writeConcernObj);
     }
-    auto request = OpMsgRequestBuilder::create(nss.dbName(), cmdBuilder.obj());
+    auto request = OpMsgRequestBuilder::create(vts, nss.dbName(), cmdBuilder.obj());
     request.sequences.push_back({"documents", v});
 
     return request;
 }
 
-OpMsgRequest createUpdateRequest(const NamespaceString& nss,
+OpMsgRequest createUpdateRequest(const auth::ValidatedTenancyScope& vts,
+                                 const NamespaceString& nss,
                                  const BSONObj& filter,
                                  BSONObj updateSpec,
                                  bool upsert,
@@ -650,7 +637,7 @@ OpMsgRequest createUpdateRequest(const NamespaceString& nss,
     if (writeConcernObj) {
         cmdBuilder.append(WriteConcernOptions::kWriteConcernField, *writeConcernObj);
     }
-    auto request = OpMsgRequestBuilder::create(nss.dbName(), cmdBuilder.obj());
+    auto request = OpMsgRequestBuilder::create(vts, nss.dbName(), cmdBuilder.obj());
     request.sequences.push_back(
         {"updates",
          {BSON("q" << filter << "u" << updateSpec << "upsert" << upsert << "multi" << multi)}});
@@ -658,7 +645,8 @@ OpMsgRequest createUpdateRequest(const NamespaceString& nss,
     return request;
 }
 
-OpMsgRequest createRemoveRequest(const NamespaceString& nss,
+OpMsgRequest createRemoveRequest(const auth::ValidatedTenancyScope& vts,
+                                 const NamespaceString& nss,
                                  const BSONObj& filter,
                                  bool removeMany,
                                  boost::optional<BSONObj> writeConcernObj) {
@@ -669,7 +657,7 @@ OpMsgRequest createRemoveRequest(const NamespaceString& nss,
     if (writeConcernObj) {
         cmdBuilder.append(WriteConcernOptions::kWriteConcernField, *writeConcernObj);
     }
-    auto request = OpMsgRequestBuilder::create(nss.dbName(), cmdBuilder.obj());
+    auto request = OpMsgRequestBuilder::create(vts, nss.dbName(), cmdBuilder.obj());
     request.sequences.push_back({"deletes", {BSON("q" << filter << "limit" << limit)}});
 
     return request;
@@ -680,7 +668,8 @@ BSONObj DBClientBase::insertAcknowledged(const NamespaceString& nss,
                                          const vector<BSONObj>& v,
                                          bool ordered,
                                          boost::optional<BSONObj> writeConcernObj) {
-    OpMsgRequest request = createInsertRequest(nss, v, ordered, writeConcernObj);
+    OpMsgRequest request = createInsertRequest(
+        _createInnerRequestVTS(nss.tenantId()), nss, v, ordered, writeConcernObj);
     rpc::UniqueReply reply = runCommand(std::move(request));
     return reply->getCommandReply();
 }
@@ -696,7 +685,8 @@ void DBClientBase::insert(const NamespaceString& nss,
                           const vector<BSONObj>& v,
                           bool ordered,
                           boost::optional<BSONObj> writeConcernObj) {
-    auto request = createInsertRequest(nss, v, ordered, writeConcernObj);
+    auto request = createInsertRequest(
+        _createInnerRequestVTS(nss.tenantId()), nss, v, ordered, writeConcernObj);
     runFireAndForgetCommand(std::move(request));
 }
 
@@ -704,7 +694,8 @@ BSONObj DBClientBase::removeAcknowledged(const NamespaceString& nss,
                                          const BSONObj& filter,
                                          bool removeMany,
                                          boost::optional<BSONObj> writeConcernObj) {
-    OpMsgRequest request = createRemoveRequest(nss, filter, removeMany, writeConcernObj);
+    OpMsgRequest request = createRemoveRequest(
+        _createInnerRequestVTS(nss.tenantId()), nss, filter, removeMany, writeConcernObj);
     rpc::UniqueReply reply = runCommand(std::move(request));
     return reply->getCommandReply();
 }
@@ -713,7 +704,8 @@ void DBClientBase::remove(const NamespaceString& nss,
                           const BSONObj& filter,
                           bool removeMany,
                           boost::optional<BSONObj> writeConcernObj) {
-    auto request = createRemoveRequest(nss, filter, removeMany, writeConcernObj);
+    auto request = createRemoveRequest(
+        _createInnerRequestVTS(nss.tenantId()), nss, filter, removeMany, writeConcernObj);
     runFireAndForgetCommand(std::move(request));
 }
 
@@ -723,7 +715,13 @@ BSONObj DBClientBase::updateAcknowledged(const NamespaceString& nss,
                                          bool upsert,
                                          bool multi,
                                          boost::optional<BSONObj> writeConcernObj) {
-    auto request = createUpdateRequest(nss, filter, updateSpec, upsert, multi, writeConcernObj);
+    auto request = createUpdateRequest(_createInnerRequestVTS(nss.tenantId()),
+                                       nss,
+                                       filter,
+                                       updateSpec,
+                                       upsert,
+                                       multi,
+                                       writeConcernObj);
     rpc::UniqueReply reply = runCommand(std::move(request));
     return reply->getCommandReply();
 }
@@ -734,13 +732,21 @@ void DBClientBase::update(const NamespaceString& nss,
                           bool upsert,
                           bool multi,
                           boost::optional<BSONObj> writeConcernObj) {
-    auto request = createUpdateRequest(nss, filter, updateSpec, upsert, multi, writeConcernObj);
+    auto request = createUpdateRequest(_createInnerRequestVTS(nss.tenantId()),
+                                       nss,
+                                       filter,
+                                       updateSpec,
+                                       upsert,
+                                       multi,
+                                       writeConcernObj);
     runFireAndForgetCommand(std::move(request));
 }
 
 void DBClientBase::killCursor(const NamespaceString& nss, long long cursorId) {
-    runFireAndForgetCommand(OpMsgRequestBuilder::create(
-        nss.dbName(), KillCursorsCommandRequest(nss, {cursorId}).toBSON(BSONObj{})));
+    runFireAndForgetCommand(
+        OpMsgRequestBuilder::create(_createInnerRequestVTS(nss.tenantId()),
+                                    nss.dbName(),
+                                    KillCursorsCommandRequest(nss, {cursorId}).toBSON(BSONObj{})));
 }
 
 namespace {
@@ -793,8 +799,8 @@ std::list<BSONObj> DBClientBase::_getIndexSpecs(const NamespaceStringOrUUID& nsO
 
         const long long id = cursorObj["id"].Long();
         if (id != 0) {
-            const auto cursorNs =
-                NamespaceStringUtil::deserialize(dbName.tenantId(), cursorObj["ns"].String());
+            const auto cursorNs = NamespaceStringUtil::deserialize(
+                dbName.tenantId(), cursorObj["ns"].String(), SerializationContext::stateDefault());
             if (nsOrUuid.isNamespaceString()) {
                 invariant(nsOrUuid.nss() == cursorNs);
             }
@@ -841,11 +847,7 @@ void DBClientBase::dropIndex(const NamespaceString& nss,
 
     BSONObj info;
     if (!runCommand(nss.dbName(), cmdBuilder.obj(), info)) {
-        LOGV2_DEBUG(20118,
-                    _logLevel.toInt(),
-                    "dropIndex failed: {info}",
-                    "dropIndex failed",
-                    "info"_attr = info);
+        LOGV2_DEBUG(20118, _logLevel.toInt(), "dropIndex failed", "info"_attr = info);
         uassert(10007, "dropIndex failed", 0);
     }
 }
@@ -942,18 +944,6 @@ void DBClientBase::createIndexes(const NamespaceString& nss,
     }
 }
 
-OpMsgRequest DBClientBase::_upconvertRequest(const DatabaseName& dbName,
-                                             BSONObj legacyCmdObj,
-                                             int options) {
-    if (isAlwaysAppendDollarTenant_forTest() && dbName.tenantId()) {
-        BSONObjBuilder bob = BSONObjBuilder(std::move(legacyCmdObj));
-        dbName.tenantId()->serializeToBSON("$tenant", &bob);
-        legacyCmdObj = bob.obj();
-    }
-
-    return rpc::upconvertRequest(dbName, std::move(legacyCmdObj), options);
-}
-
 BSONElement getErrField(const BSONObj& o) {
     return o["$err"];
 }
