@@ -29,10 +29,25 @@
 
 #pragma once
 
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
 #include <cstdint>
 #include <functional>
+#include <list>
+#include <memory>
+#include <string>
+#include <tuple>
+#include <utility>
+#include <vector>
 
+#include "mongo/base/error_codes.h"
 #include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/timestamp.h"
 #include "mongo/client/authenticate.h"
 #include "mongo/client/client_api_version_parameters_gen.h"
 #include "mongo/client/connection_string.h"
@@ -40,9 +55,12 @@
 #include "mongo/client/index_spec.h"
 #include "mongo/client/mongo_uri.h"
 #include "mongo/client/read_preference.h"
-#include "mongo/config.h"
+#include "mongo/config.h"  // IWYU pragma: keep
+#include "mongo/db/database_name.h"
 #include "mongo/db/dbmessage.h"
 #include "mongo/db/jsobj.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/query/find_command.h"
 #include "mongo/db/write_concern_options.h"
 #include "mongo/logv2/log_severity.h"
 #include "mongo/platform/atomic_word.h"
@@ -52,7 +70,8 @@
 #include "mongo/rpc/unique_message.h"
 #include "mongo/transport/message_compressor_manager.h"
 #include "mongo/transport/session.h"
-#include "mongo/transport/transport_layer.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/net/ssl_types.h"
 #include "mongo/util/str.h"
 
 namespace mongo {
@@ -103,12 +122,10 @@ public:
 
     virtual std::string getServerAddress() const = 0;
 
-    virtual std::string getLocalAddress() const = 0;
-
     /**
      * Reconnect if needed and allowed.
      */
-    virtual void checkConnection() {}
+    virtual void ensureConnection() {}
 
     /**
      * If not checked recently, checks whether the underlying socket/sockets are still valid.
@@ -241,6 +258,7 @@ public:
      *  'info': The result object the database returns. Typically has { ok : ..., errmsg : ... }
      *          fields set.
      *  'options': See enum QueryOptions - normally not needed to run a command.
+     *  'vts': optional validated tenancy scope used to include tenancy information on a command
      *
      *  Returns true if the command returned "ok".
      */
@@ -258,21 +276,11 @@ public:
                                                          BSONObj& info,
                                                          int options = 0);
 
-    /**
-     * See the opMsg overload comment for why this function takes a shared_ptr ostensibly to this.
-     */
-    std::tuple<bool, std::shared_ptr<DBClientBase>> runCommandWithTarget(
-        const DatabaseName& dbName,
-        BSONObj cmd,
-        BSONObj& info,
-        std::shared_ptr<DBClientBase> me,
-        int options = 0);
-
     /**
      * Authenticates to another cluster member using appropriate authentication data.
-     * Returns true if the authentication was successful.
+     * Throws an exception if authentication fails.
      */
-    virtual Status authenticateInternalUser(
+    virtual void authenticateInternalUser(
         auth::StepDownBehavior stepDownBehavior = auth::StepDownBehavior::kKillConnection);
 
     /**
@@ -303,13 +311,8 @@ public:
      * Authentication is separate for each database on the server -- you may authenticate for any
      * number of databases on a single connection. The "admin" database is special and once
      * authenticated provides access to all databases on the server.
-     *
-     *   Returns true if successful.
      */
-    bool auth(const std::string& dbname,
-              const std::string& username,
-              const std::string& pwd,
-              std::string& errmsg);
+    void auth(const DatabaseName& dbname, StringData username, StringData pwd);
 
     /**
      * Logs out the connection for the given database.
@@ -318,7 +321,7 @@ public:
      * 'info': The result object for the logout command (provided for backwards compatibility with
      *         mongo shell).
      */
-    virtual void logout(const std::string& dbname, BSONObj& info);
+    virtual void logout(const DatabaseName& dbname, BSONObj& info);
 
     virtual bool authenticatedDuringConnect() const {
         return false;
@@ -483,11 +486,13 @@ public:
     static std::string genIndexName(const BSONObj& keys);
 
     /**
+     * Sends the provided message, returning the server's reponse.
+     *
      * 'actualServer' is set to the actual server where they call went if there was a choice (for
      * example SecondaryOk).
      */
-    void call(Message& toSend, Message& response, std::string* actualServer = nullptr) {
-        _call(toSend, response, actualServer);
+    Message call(Message& toSend, std::string* actualServer = nullptr) {
+        return _call(toSend, actualServer);
     };
 
     virtual void say(Message& toSend,
@@ -497,9 +502,9 @@ public:
     /**
      * Used by QueryOption_Exhaust. To use that your subclass must implement this.
      */
-    virtual Status recv(Message& m, int lastRequestId) {
+    virtual Message recv(int lastRequestId) {
         MONGO_verify(false);
-        return {ErrorCodes::NotImplemented, "recv() not implemented"};
+        uasserted(ErrorCodes::NotImplemented, "recv() not implemented");
     }
 
     /**
@@ -672,6 +677,10 @@ public:
     }
 #endif
 
+    virtual bool isGRPC() {
+        return false;
+    }
+
     const ClientAPIVersionParameters& getApiParameters() const {
         return _apiParameters;
     }
@@ -680,13 +689,13 @@ public:
         _shouldThrowOnStaleConfigError = value;
     }
 
-    void setAlwaysAppendDollarTenant_forTest() {
-        _alwaysAppendDollarTenant = true;
-    }
-
-    bool isAlwaysAppendDollarTenant_forTest() const {
-        return _alwaysAppendDollarTenant;
-    }
+protected:
+    /**
+     * Generates the validated tenancy scope for internal requests utilized within the server or
+     * across servers.
+     */
+    virtual auth::ValidatedTenancyScope _createInnerRequestVTS(
+        const boost::optional<TenantId>& tenantId) const;
 
 protected:
     /**
@@ -724,7 +733,7 @@ protected:
     bool _shouldThrowOnStaleConfigError = true;
 
 private:
-    virtual void _call(Message& toSend, Message& response, std::string* actualServer) = 0;
+    virtual Message _call(Message& toSend, std::string* actualServer) = 0;
 
     /**
      * Implementation for getIndexes() and getReadyIndexes().
@@ -735,12 +744,6 @@ private:
 
     auth::RunCommandHook _makeAuthRunCommandHook();
 
-    OpMsgRequest _upconvertRequest(const DatabaseName& dbName,
-                                   BSONObj legacyCmdObj,
-                                   int queryFlags = 0);
-
-    bool _alwaysAppendDollarTenant = false;
-
     rpc::RequestMetadataWriter _metadataWriter;
     rpc::ReplyMetadataReader _metadataReader;
 
@@ -767,8 +770,8 @@ public:
                                   rpc::RequestMetadataWriter writer,
                                   rpc::ReplyMetadataReader reader)
         : _conn(conn),
-          _oldWriter(std::move(conn->getRequestMetadataWriter())),
-          _oldReader(std::move(conn->getReplyMetadataReader())) {
+          _oldWriter(conn->getRequestMetadataWriter()),
+          _oldReader(conn->getReplyMetadataReader()) {
         _conn->setRequestMetadataWriter(std::move(writer));
         _conn->setReplyMetadataReader(std::move(reader));
     }
