@@ -29,28 +29,44 @@
 
 #include "mongo/client/dbclient_rs.h"
 
+#include <boost/move/utility_core.hpp>
+#include <boost/smart_ptr.hpp>
+#include <cstddef>
 #include <memory>
+#include <ostream>
+#include <set>
+#include <type_traits>
 #include <utility>
 
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status_with.h"
+#include "mongo/bson/bsonelement.h"
 #include "mongo/bson/util/builder.h"
+#include "mongo/bson/util/builder_fwd.h"
 #include "mongo/client/connpool.h"
 #include "mongo/client/dbclient_cursor.h"
 #include "mongo/client/global_conn_pool.h"
+#include "mongo/client/internal_auth.h"
 #include "mongo/client/read_preference.h"
 #include "mongo/client/replica_set_monitor.h"
-#include "mongo/config.h"
+#include "mongo/config.h"  // IWYU pragma: keep
 #include "mongo/db/auth/sasl_command_constants.h"
-#include "mongo/db/dbmessage.h"
-#include "mongo/db/jsobj.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/redaction.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/cancellation.h"
+#include "mongo/util/future.h"
+#include "mongo/util/str.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kNetwork
 
 
 namespace mongo {
 
-using std::endl;
-using std::map;
 using std::set;
 using std::shared_ptr;
 using std::string;
@@ -76,6 +92,7 @@ public:
         _secOkCmdList.insert("find");
         _secOkCmdList.insert("geoNear");
         _secOkCmdList.insert("group");
+        _secOkCmdList.insert("lockInfo");
     }
 } _populateReadPrefSecOkCmdList;
 
@@ -125,8 +142,6 @@ ReplicaSetMonitorPtr DBClientReplicaSet::_getMonitor() {
 string DBClientReplicaSet::getServerAddress() const {
     if (!_rsm) {
         LOGV2_WARNING(20147,
-                      "Trying to get server address for DBClientReplicaSet, "
-                      "but no ReplicaSetMonitor exists for {replicaSet}",
                       "Trying to get server address for DBClientReplicaSet, "
                       "but no ReplicaSetMonitor exists",
                       "replicaSet"_attr = _setName);
@@ -135,10 +150,6 @@ string DBClientReplicaSet::getServerAddress() const {
     return _rsm->getServerAddress();
 }
 
-string DBClientReplicaSet::getLocalAddress() const {
-    MONGO_UNIMPLEMENTED;
-}
-
 HostAndPort DBClientReplicaSet::getSuspectedPrimaryHostAndPort() const {
     if (!_primary) {
         return HostAndPort();
@@ -215,14 +226,14 @@ bool _isSecondaryCommand(StringData commandName, const BSONObj& commandArgs) {
 }
 
 // Internal implementation of isSecondaryQuery, takes previously-parsed read preference
-bool _isSecondaryQuery(StringData ns,
+bool _isSecondaryQuery(const NamespaceString& ns,
                        const BSONObj& filter,
                        const ReadPreferenceSetting& readPref) {
     // If the read pref is primary only, this is not a secondary query
     if (readPref.pref == ReadPreference::PrimaryOnly)
         return false;
 
-    if (ns.find(".$cmd") == string::npos) {
+    if (ns.coll().find("$cmd") == string::npos) {
         return true;
     }
 
@@ -313,43 +324,41 @@ bool DBClientReplicaSet::checkLastHost(const ReadPreferenceSetting* readPref) {
 
 void DBClientReplicaSet::_authConnection(DBClientConnection* conn) {
     if (_internalAuthRequested) {
-        auto status = conn->authenticateInternalUser();
-        if (!status.isOK()) {
+        try {
+            conn->authenticateInternalUser();
+        } catch (const DBException& e) {
             LOGV2_WARNING(20148,
-                          "Cached auth failed for set {replicaSet}: {error}",
                           "Cached auth failed",
                           "replicaSet"_attr = _setName,
-                          "error"_attr = status);
+                          "error"_attr = e.toStatus());
+            return;
         }
-        return;
     }
 
-    for (map<string, BSONObj>::const_iterator i = _auths.begin(); i != _auths.end(); ++i) {
+    for (const auto& kv : _auths) {
         try {
-            conn->auth(i->second);
+            conn->auth(kv.second);
         } catch (const AssertionException&) {
             LOGV2_WARNING(20149,
-                          "Cached auth failed for set: {replicaSet} db: {db} user: {user}",
                           "Cached auth failed",
                           "replicaSet"_attr = _setName,
-                          "db"_attr = i->second[saslCommandUserDBFieldName].str(),
-                          "user"_attr = i->second[saslCommandUserFieldName].str());
+                          "db"_attr = kv.first.toStringForErrorMsg(),
+                          "user"_attr = kv.second[saslCommandUserFieldName].str());
         }
     }
 }
 
 void DBClientReplicaSet::logoutAll(DBClientConnection* conn) {
     _internalAuthRequested = false;
-    for (map<string, BSONObj>::const_iterator i = _auths.begin(); i != _auths.end(); ++i) {
+    for (const auto& kv : _auths) {
         BSONObj response;
         try {
-            conn->logout(i->first, response);
+            conn->logout(kv.first, response);
         } catch (const AssertionException& ex) {
             LOGV2_WARNING(20150,
-                          "Failed to logout: {connString} on db: {db} with error: {error}",
                           "Failed to logout",
                           "connString"_attr = conn->getServerAddress(),
-                          "db"_attr = i->first,
+                          "db"_attr = kv.first.toStringForErrorMsg(),
                           "error"_attr = redact(ex));
         }
     }
@@ -381,7 +390,7 @@ Status DBClientReplicaSet::connect() {
 }
 
 template <typename Authenticate>
-Status DBClientReplicaSet::_runAuthLoop(Authenticate authCb) {
+void DBClientReplicaSet::_runAuthLoop(Authenticate authCb) {
     // We prefer to authenticate against a primary, but otherwise a secondary is ok too
     // Empty tag matches every secondary
     const auto readPref =
@@ -389,7 +398,6 @@ Status DBClientReplicaSet::_runAuthLoop(Authenticate authCb) {
 
     LOGV2_DEBUG(20132,
                 3,
-                "dbclient_rs attempting authentication of {replicaSet}",
                 "dbclient_rs attempting authentication",
                 "replicaSet"_attr = _getMonitor()->getName());
 
@@ -416,54 +424,53 @@ Status DBClientReplicaSet::_runAuthLoop(Authenticate authCb) {
                 resetPrimary();
             }
 
-            return Status::OK();
+            return;
         } catch (const DBException& e) {
-            auto status = e.toStatus();
-            if (status == ErrorCodes::AuthenticationFailed) {
-                return status;
+            if (e.code() == ErrorCodes::AuthenticationFailed) {
+                throw;
             }
-
-            lastNodeStatus =
-                status.withContext(str::stream() << "can't authenticate against replica set node "
-                                                 << _lastSecondaryOkHost);
+            lastNodeStatus = e.toStatus().withContext(
+                str::stream() << "can't authenticate against replica set node "
+                              << _lastSecondaryOkHost);
             _invalidateLastSecondaryOkCache(lastNodeStatus);
         }
     }
 
     if (lastNodeStatus.isOK()) {
-        return Status(ErrorCodes::HostNotFound,
-                      str::stream() << "Failed to authenticate, no good nodes in "
-                                    << _getMonitor()->getName());
+        uasserted(ErrorCodes::HostNotFound,
+                  str::stream() << "Failed to authenticate, no good nodes in "
+                                << _getMonitor()->getName());
     } else {
-        return lastNodeStatus;
+        uassertStatusOK(lastNodeStatus);  // always throws
     }
 }
 
-Status DBClientReplicaSet::authenticateInternalUser(auth::StepDownBehavior stepDownBehavior) {
-    if (!auth::isInternalAuthSet()) {
-        return {ErrorCodes::AuthenticationFailed,
-                "No authentication parameters set for internal user"};
-    }
+void DBClientReplicaSet::authenticateInternalUser(auth::StepDownBehavior stepDownBehavior) {
+    uassert(ErrorCodes::AuthenticationFailed,
+            "No authentication parameters set for internal user",
+            auth::isInternalAuthSet());
 
     _internalAuthRequested = true;
-    return _runAuthLoop([stepDownBehavior](DBClientConnection* conn) {
-        uassertStatusOK(conn->authenticateInternalUser(stepDownBehavior));
+    _runAuthLoop([stepDownBehavior](DBClientConnection* conn) {
+        conn->authenticateInternalUser(stepDownBehavior);
     });
 }
 
 void DBClientReplicaSet::_auth(const BSONObj& params) {
-    uassertStatusOK(_runAuthLoop([&](DBClientConnection* conn) {
+    _runAuthLoop([&](DBClientConnection* conn) {
         conn->auth(params);
         // Cache the new auth information since we now validated it's good
-        _auths[params[saslCommandUserDBFieldName].str()] = params.getOwned();
-    }));
+        const DatabaseName dbName = AuthDatabaseNameUtil::deserialize(
+            params[saslCommandUserDBFieldName].valueStringDataSafe());
+        _auths[dbName] = params.getOwned();
+    });
 }
 
-void DBClientReplicaSet::logout(const string& dbname, BSONObj& info) {
+void DBClientReplicaSet::logout(const DatabaseName& dbName, BSONObj& info) {
     DBClientConnection* priConn = checkPrimary();
 
-    priConn->logout(dbname, info);
-    _auths.erase(dbname);
+    priConn->logout(dbName, info);
+    _auths.erase(dbName);
 
     /* Also logout the cached secondary connection. Note that this is only
      * needed when we actually have something cached and is last known to be
@@ -472,7 +479,7 @@ void DBClientReplicaSet::logout(const string& dbname, BSONObj& info) {
     if (_lastSecondaryOkConn.get() != nullptr && !_lastSecondaryOkConn->isFailed()) {
         try {
             BSONObj dummy;
-            _lastSecondaryOkConn->logout(dbname, dummy);
+            _lastSecondaryOkConn->logout(dbName, dummy);
         } catch (const DBException&) {
             // Make sure we can't use this connection again.
             MONGO_verify(_lastSecondaryOkConn->isFailed());
@@ -508,7 +515,7 @@ std::unique_ptr<DBClientCursor> DBClientReplicaSet::find(FindCommandRequest find
                                                          ExhaustMode exhaustMode) {
     invariant(findRequest.getNamespaceOrUUID().isNamespaceString());
     if (_isSecondaryQuery(
-            findRequest.getNamespaceOrUUID().nss().ns(), findRequest.toBSON(BSONObj{}), readPref)) {
+            findRequest.getNamespaceOrUUID().nss(), findRequest.toBSON(BSONObj{}), readPref)) {
         LOGV2_DEBUG(5951202,
                     3,
                     "dbclient_rs query using secondary or tagged node selection",
@@ -615,7 +622,6 @@ DBClientConnection* DBClientReplicaSet::selectNodeUsingTags(
     if (checkLastHost(readPref.get())) {
         LOGV2_DEBUG(20137,
                     3,
-                    "dbclient_rs selecting compatible last used node {lastTagged}",
                     "dbclient_rs selecting compatible last used node",
                     "lastTagged"_attr = _lastSecondaryOkHost);
 
@@ -629,7 +635,6 @@ DBClientConnection* DBClientReplicaSet::selectNodeUsingTags(
     if (!selectedNodeStatus.isOK()) {
         LOGV2_DEBUG(20138,
                     3,
-                    "dbclient_rs no compatible node found: {error}",
                     "dbclient_rs no compatible node found",
                     "error"_attr = redact(selectedNodeStatus.getStatus()));
         return nullptr;
@@ -651,11 +656,8 @@ DBClientConnection* DBClientReplicaSet::selectNodeUsingTags(
     if (monitor->isPrimary(selectedNode)) {
         checkPrimary();
 
-        LOGV2_DEBUG(20139,
-                    3,
-                    "dbclient_rs selecting primary node {connString}",
-                    "dbclient_rs selecting primary node",
-                    "connString"_attr = selectedNode);
+        LOGV2_DEBUG(
+            20139, 3, "dbclient_rs selecting primary node", "connString"_attr = selectedNode);
 
         _lastSecondaryOkConn = _primary;
 
@@ -690,11 +692,7 @@ DBClientConnection* DBClientReplicaSet::selectNodeUsingTags(
         }
     }
 
-    LOGV2_DEBUG(20140,
-                3,
-                "dbclient_rs selecting node {connString}",
-                "dbclient_rs selecting node",
-                "connString"_attr = _lastSecondaryOkHost);
+    LOGV2_DEBUG(20140, 3, "dbclient_rs selecting node", "connString"_attr = _lastSecondaryOkHost);
 
     return _lastSecondaryOkConn.get();
 }
@@ -703,11 +701,8 @@ void DBClientReplicaSet::say(Message& toSend, bool isRetry, string* actualServer
     if (!isRetry)
         _lastClient = nullptr;
 
-    LOGV2_DEBUG(20142,
-                3,
-                "dbclient_rs say to primary node in {replicaSet}",
-                "dbclient_rs say to primary node",
-                "replicaSet"_attr = _getMonitor()->getName());
+    LOGV2_DEBUG(
+        20142, 3, "dbclient_rs say to primary node", "replicaSet"_attr = _getMonitor()->getName());
 
     DBClientConnection* primary = checkPrimary();
     if (actualServer)
@@ -719,18 +714,17 @@ void DBClientReplicaSet::say(Message& toSend, bool isRetry, string* actualServer
     return;
 }
 
-Status DBClientReplicaSet::recv(Message& m, int lastRequestId) {
+Message DBClientReplicaSet::recv(int lastRequestId) {
     MONGO_verify(_lastClient);
 
     try {
-        return _lastClient->recv(m, lastRequestId);
+        return _lastClient->recv(lastRequestId);
     } catch (DBException& e) {
         LOGV2(20143,
-              "Could not receive data from {connString}: {error}",
               "Could not receive data",
               "connString"_attr = _lastClient->toString(),
               "error"_attr = redact(e));
-        return e.toStatus();
+        throw;
     }
 }
 
@@ -800,18 +794,15 @@ std::pair<rpc::UniqueReply, std::shared_ptr<DBClientBase>> DBClientReplicaSet::r
     return {std::move(out.first), std::move(conn)};
 }
 
-void DBClientReplicaSet::_call(Message& toSend, Message& response, string* actualServer) {
-    LOGV2_DEBUG(20146,
-                3,
-                "dbclient_rs call to primary node in {replicaSet}",
-                "dbclient_rs call to primary node",
-                "replicaSet"_attr = _getMonitor()->getName());
+Message DBClientReplicaSet::_call(Message& toSend, string* actualServer) {
+    LOGV2_DEBUG(
+        20146, 3, "dbclient_rs call to primary node", "replicaSet"_attr = _getMonitor()->getName());
 
     DBClientConnection* m = checkPrimary();
     if (actualServer)
         *actualServer = m->getServerAddress();
 
-    m->call(toSend, response);
+    return m->call(toSend);
 }
 
 void DBClientReplicaSet::_invalidateLastSecondaryOkCache(const Status& status) {
