@@ -27,16 +27,24 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/client/fetcher.h"
-
+#include <mutex>
 #include <ostream>
 #include <utility>
 
-#include "mongo/db/jsobj.h"
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/client/dbclient_base.h"
+#include "mongo/client/fetcher.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/redaction.h"
 #include "mongo/rpc/get_status_from_command_result.h"
 #include "mongo/util/assert_util.h"
 #include "mongo/util/destructor_guard.h"
@@ -51,7 +59,6 @@ namespace mongo {
 namespace {
 
 using executor::RemoteCommandRequest;
-using executor::RemoteCommandResponse;
 
 using RemoteCommandCallbackArgs = executor::TaskExecutor::RemoteCommandCallbackArgs;
 const char* kCursorFieldName = "cursor";
@@ -113,7 +120,8 @@ Status parseCursorResponse(const BSONObj& obj,
                       str::stream() << "'" << kCursorFieldName << "." << kNamespaceFieldName
                                     << "' field must be a string: " << obj);
     }
-    const NamespaceString tempNss(namespaceElement.valueStringData());
+    const NamespaceString tempNss = NamespaceStringUtil::deserialize(
+        boost::none, namespaceElement.valueStringData(), SerializationContext::stateDefault());
     if (!tempNss.isValid()) {
         return Status(ErrorCodes::BadValue,
                       str::stream() << "'" << kCursorFieldName << "." << kNamespaceFieldName
@@ -167,7 +175,7 @@ Status parseCursorResponse(const BSONObj& obj,
 
 Fetcher::Fetcher(executor::TaskExecutor* executor,
                  const HostAndPort& source,
-                 StringData dbname,
+                 const DatabaseName& dbname,
                  const BSONObj& findCmdObj,
                  CallbackFn work,
                  const BSONObj& metadata,
@@ -222,7 +230,7 @@ std::string Fetcher::getDiagnosticString() const {
     str::stream output;
     output << "Fetcher";
     output << " source: " << _source.toString();
-    output << " database: " << _dbname;
+    output << " database: " << toStringForLogging(_dbname);
     output << " query: " << _cmdObj;
     output << " query metadata: " << _metadata;
     output << " active: " << _isActive_inlock();
@@ -384,7 +392,7 @@ void Fetcher::_callback(const RemoteCommandCallbackArgs& rcbd, const char* batch
         return;
     }
 
-    batchData.otherFields.metadata = std::move(rcbd.response.data);
+    batchData.otherFields.metadata = rcbd.response.data;
     batchData.elapsed = rcbd.response.elapsed.value_or(Microseconds{0});
     {
         stdx::lock_guard<Latch> lk(_mutex);
@@ -430,17 +438,13 @@ void Fetcher::_sendKillCursors(const CursorId id, const NamespaceString& nss) {
         auto logKillCursorsResult = [](const RemoteCommandCallbackArgs& args) {
             if (!args.response.isOK()) {
                 LOGV2_WARNING(23918,
-                              "killCursors command task failed: {error}",
                               "killCursors command task failed",
                               "error"_attr = redact(args.response.status));
                 return;
             }
             auto status = getStatusFromCommandResult(args.response.data);
             if (!status.isOK()) {
-                LOGV2_WARNING(23919,
-                              "killCursors command failed: {error}",
-                              "killCursors command failed",
-                              "error"_attr = redact(status));
+                LOGV2_WARNING(23919, "killCursors command failed", "error"_attr = redact(status));
             }
         };
 
@@ -451,7 +455,6 @@ void Fetcher::_sendKillCursors(const CursorId id, const NamespaceString& nss) {
         auto scheduleResult = _executor->scheduleRemoteCommand(request, logKillCursorsResult);
         if (!scheduleResult.isOK()) {
             LOGV2_WARNING(23920,
-                          "Failed to schedule killCursors command: {error}",
                           "Failed to schedule killCursors command",
                           "error"_attr = redact(scheduleResult.getStatus()));
         }
