@@ -29,21 +29,35 @@
 
 #pragma once
 
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
 #include <functional>
 #include <iosfwd>
 #include <memory>
 #include <string>
+#include <utility>
 #include <vector>
 
 #include "mongo/base/status.h"
 #include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
 #include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/client/read_preference.h"
 #include "mongo/client/remote_command_retry_scheduler.h"
 #include "mongo/db/clientcursor.h"
+#include "mongo/db/cursor_id.h"
 #include "mongo/db/namespace_string.h"
+#include "mongo/executor/remote_command_request.h"
 #include "mongo/executor/task_executor.h"
 #include "mongo/platform/mutex.h"
 #include "mongo/stdx/condition_variable.h"
+#include "mongo/transport/transport_layer.h"
+#include "mongo/util/duration.h"
+#include "mongo/util/future.h"
+#include "mongo/util/future_impl.h"
+#include "mongo/util/interruptible.h"
 #include "mongo/util/net/hostandport.h"
 
 namespace mongo {
@@ -130,7 +144,7 @@ public:
      */
     Fetcher(executor::TaskExecutor* executor,
             const HostAndPort& source,
-            StringData dbname,
+            const DatabaseName& dbname,
             const BSONObj& cmdObj,
             CallbackFn work,
             const BSONObj& metadata = ReadPreferenceSetting::secondaryPreferredMetadata(),
@@ -256,7 +270,7 @@ private:
     executor::TaskExecutor* _executor;
 
     HostAndPort _source;
-    std::string _dbname;
+    DatabaseName _dbname;
     BSONObj _cmdObj;
     BSONObj _metadata;
     CallbackFn _work;
