@@ -27,25 +27,45 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/client/mongo_uri.h"
-
-#include <utility>
-
 #include <boost/algorithm/string/case_conv.hpp>
 #include <boost/algorithm/string/classification.hpp>
+#include <boost/algorithm/string/compare.hpp>
 #include <boost/algorithm/string/find_iterator.hpp>
-#include <boost/algorithm/string/predicate.hpp>
 #include <boost/algorithm/string/split.hpp>
 #include <boost/range/algorithm/count.hpp>
+// IWYU pragma: no_include "boost/algorithm/string/detail/classification.hpp"
+// IWYU pragma: no_include "boost/algorithm/string/detail/finder.hpp"
+#include <algorithm>
+#include <array>
+#include <boost/algorithm/string/finder.hpp>
+#include <boost/core/addressof.hpp>
+#include <boost/function/function_base.hpp>
+#include <boost/iterator/iterator_facade.hpp>
+#include <boost/range/const_iterator.hpp>
+#include <boost/range/iterator_range_core.hpp>
+#include <boost/type_index/type_index_facade.hpp>
+#include <cstddef>
+#include <exception>
+#include <memory>
+#include <type_traits>
+#include <utility>
 
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
 #include "mongo/base/status_with.h"
+#include "mongo/bson/bsonelement.h"
 #include "mongo/bson/bsonobjbuilder.h"
-#include "mongo/client/sasl_client_authenticate.h"
+#include "mongo/bson/util/builder.h"
+#include "mongo/client/authenticate.h"
+#include "mongo/client/mongo_uri.h"
 #include "mongo/db/auth/sasl_command_constants.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/stdx/utility.h"
+#include "mongo/transport/transport_layer.h"
 #include "mongo/util/assert_util.h"
 #include "mongo/util/ctype.h"
 #include "mongo/util/dns_name.h"
@@ -444,8 +464,7 @@ MongoURI MongoURI::parseImpl(StringData url) {
     // slash ("/"), backslash ("\"), space (" "), double-quote ("""), or dollar sign ("$")
     // period (".") is also prohibited, but drivers MAY allow periods
     if (!database.empty() &&
-        !NamespaceString::validDBName(database,
-                                      NamespaceString::DollarInDbNameBehavior::Disallow)) {
+        !DatabaseName::validDBName(database, DatabaseName::DollarInDbNameBehavior::Disallow)) {
         uasserted(ErrorCodes::FailedToParse,
                   str::stream() << "Database name cannot have reserved "
                                    "characters for mongodb:// URL: "
@@ -471,47 +490,36 @@ MongoURI MongoURI::parseImpl(StringData url) {
                   str::stream() << "appName cannot exceed 128 characters: " << optIter->second);
     }
 
-    boost::optional<bool> retryWrites = boost::none;
-    optIter = options.find("retryWrites");
-    if (optIter != end(options)) {
-        if (optIter->second == "true") {
-            retryWrites.reset(true);
-        } else if (optIter->second == "false") {
-            retryWrites.reset(false);
-        } else {
-            uasserted(ErrorCodes::FailedToParse,
-                      str::stream() << "retryWrites must be either \"true\" or \"false\"");
+    auto extractBooleanOption =
+        [&options](CaseInsensitiveString optionName) -> boost::optional<bool> {
+        auto optIter = options.find(optionName);
+        if (optIter == end(options)) {
+            return boost::none;
         }
+        if (auto value = optIter->second; value == "true") {
+            return true;
+        } else if (value == "false") {
+            return false;
+        }
+        uasserted(ErrorCodes::FailedToParse,
+                  fmt::format("{} must be either \"true\" or \"false\"", optionName.original()));
+    };
+
+    const auto retryWrites = extractBooleanOption("retryWrites");
+    const auto helloOk = extractBooleanOption("helloOk");
+// TODO: SERVER-80343 Remove this ifdef once gRPC is compiled on all variants
+#ifdef MONGO_CONFIG_GRPC
+    const auto gRPC = extractBooleanOption("gRPC");
+#endif
+    auto tlsEnabled = extractBooleanOption("tls");
+    if (!tlsEnabled.has_value()) {
+        tlsEnabled = extractBooleanOption("ssl");
     }
 
-    const auto helloOk = [&options]() -> boost::optional<bool> {
-        if (auto optIter = options.find("helloOk"); optIter != end(options)) {
-            if (auto value = optIter->second; value == "true") {
-                return true;
-            } else if (value == "false") {
-                return false;
-            } else {
-                uasserted(ErrorCodes::FailedToParse,
-                          "helloOk must be either \"true\" or \"false\"");
-            }
-        }
-        return boost::none;
-    }();
-
-    transport::ConnectSSLMode sslMode = transport::kGlobalSSLMode;
-    auto sslModeIter = std::find_if(options.begin(), options.end(), [](auto pred) {
-        return pred.first == CaseInsensitiveString("ssl") ||
-            pred.first == CaseInsensitiveString("tls");
-    });
-    if (sslModeIter != options.end()) {
-        const auto& val = sslModeIter->second;
-        if (val == "true") {
-            sslMode = transport::kEnableSSL;
-        } else if (val == "false") {
-            sslMode = transport::kDisableSSL;
-        } else {
-            uasserted(51041, str::stream() << "tls must be either 'true' or 'false', not" << val);
-        }
+    auto tlsMode = transport::ConnectSSLMode::kGlobalSSLMode;
+    if (tlsEnabled.has_value()) {
+        tlsMode = *tlsEnabled ? transport::ConnectSSLMode::kEnableSSL
+                              : transport::ConnectSSLMode::kDisableSSL;
     }
 
     auto cs = replicaSetName.empty()
@@ -521,9 +529,13 @@ MongoURI MongoURI::parseImpl(StringData url) {
                     username,
                     password,
                     database,
-                    std::move(retryWrites),
-                    sslMode,
+                    retryWrites,
+                    tlsMode,
                     helloOk,
+// TODO: SERVER-80343 Remove this ifdef once gRPC is compiled on all variants
+#ifdef MONGO_CONFIG_GRPC
+                    gRPC,
+#endif
                     std::move(options));
 }
 
