@@ -29,7 +29,6 @@
 
 #ifdef _WIN32
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kNetwork
 
 #include "mongo/platform/basic.h"
 
@@ -44,6 +43,8 @@
 #include "mongo/util/str.h"
 #include "mongo/util/text.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kNetwork
+
 extern "C" int plain_client_plug_init(const sasl_utils_t* utils,
                                       int maxversion,
                                       int* out_version,
@@ -105,7 +106,7 @@ void HandleLastError(const sasl_utils_t* utils, DWORD errCode, const char* msg)
 
 int sspiClientMechNew(void* glob_context,
                       sasl_client_params_t* cparams,
-                      void** conn_context) throw() {
+                      void** conn_context) noexcept {
     // Prepare auth identity to pass to AcquireCredentialsHandle
     SEC_WINNT_AUTH_IDENTITY authIdentity;
     authIdentity.Flags = SEC_WINNT_AUTH_IDENTITY_UNICODE;
@@ -355,7 +356,7 @@ int sspiClientMechStep(void* conn_context,
                        sasl_interact_t** prompt_need,
                        const char** clientout,
                        unsigned* clientoutlen,
-                       sasl_out_params_t* oparams) throw() {
+                       sasl_out_params_t* oparams) noexcept {
     SspiConnContext* pcctx = static_cast<SspiConnContext*>(conn_context);
     *clientout = nullptr;
     *clientoutlen = 0;
