@@ -30,11 +30,11 @@
 
 #include "mongo/client/sdam/mock_topology_manager.h"
 
-#include <string>
+#include <memory>
+#include <mutex>
 
-#include "mongo/client/sdam/topology_state_machine.h"
-#include "mongo/logv2/log.h"
-#include "mongo/rpc/topology_version_gen.h"
+#include "mongo/client/sdam/topology_description.h"
+#include "mongo/util/assert_util.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kNetwork
 
@@ -44,7 +44,7 @@ namespace mongo::sdam {
 MockTopologyManager::MockTopologyManager() {}
 
 bool MockTopologyManager::onServerDescription(const HelloOutcome& helloOutcome) {
-    fassert(5429100, "MockTopologyManager does not support onServerDescription");
+    fasserted(5429100);  // MockTopologyManager does not support onServerDescription
     return true;
 }
 
