@@ -26,18 +26,31 @@
  *    exception statement from all source files in the program, then also delete
  *    it in the license file.
  */
-#include "mongo/client/sdam/sdam_test_base.h"
-
-#include <boost/algorithm/string.hpp>
-#include <ostream>
+#include <algorithm>
+#include <boost/none_t.hpp>
+#include <boost/optional.hpp>
+#include <cstddef>
+#include <iterator>
 #include <set>
+#include <vector>
 
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/client/sdam/sdam_test_base.h"
 #include "mongo/client/sdam/server_description.h"
 #include "mongo/client/sdam/server_description_builder.h"
-#include "mongo/db/jsobj.h"
 #include "mongo/db/repl/optime.h"
+#include "mongo/idl/idl_parser.h"
 #include "mongo/platform/random.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/duration.h"
 #include "mongo/util/str.h"
 #include "mongo/util/system_clock_source.h"
 
