@@ -27,29 +27,59 @@
  *    it in the license file.
  */
 
-#include <fstream>
+#include <boost/filesystem/directory.hpp>
+#include <boost/filesystem/path.hpp>
+#include <boost/filesystem/path_traits.hpp>
+#include <boost/iterator/iterator_facade.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+// IWYU pragma: no_include "ext/alloc_traits.h"
+#include <algorithm>
+#include <cstddef>
+#include <fstream>  // IWYU pragma: keep
 #include <iostream>
+#include <iterator>
 #include <memory>
+#include <ratio>
+#include <set>
+#include <string>
+#include <type_traits>
+#include <utility>
+#include <vector>
 
-#include <boost/algorithm/string.hpp>
-#include <boost/filesystem.hpp>
-#include <boost/filesystem/operations.hpp>
-#include <boost/format.hpp>
-
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/bson/json.h"
+#include "mongo/client/read_preference.h"
 #include "mongo/client/sdam/json_test_arg_parser.h"
+#include "mongo/client/sdam/sdam_configuration.h"
 #include "mongo/client/sdam/sdam_configuration_parameters_gen.h"
+#include "mongo/client/sdam/sdam_datatypes.h"
+#include "mongo/client/sdam/server_description.h"
 #include "mongo/client/sdam/server_description_builder.h"
 #include "mongo/client/sdam/server_selector.h"
-#include "mongo/client/sdam/topology_manager.h"
+#include "mongo/client/sdam/topology_description.h"
+#include "mongo/db/server_options.h"
 #include "mongo/logv2/log.h"
-#include "mongo/stdx/unordered_set.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/log_component_settings.h"
+#include "mongo/logv2/log_manager.h"
+#include "mongo/logv2/log_severity.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/clock_source_mock.h"
 #include "mongo/util/ctype.h"
-#include "mongo/util/optional_util.h"
-#include "mongo/util/options_parser/environment.h"
-#include "mongo/util/options_parser/option_section.h"
-#include "mongo/util/options_parser/options_parser.h"
+#include "mongo/util/duration.h"
+#include "mongo/util/net/hostandport.h"
+#include "mongo/util/options_parser/value.h"
+#include "mongo/util/time_support.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
 
@@ -116,9 +146,9 @@ public:
         parseTest(testFilePath);
     }
 
-    ~JsonRttTestCase() = default;
+    ~JsonRttTestCase() override = default;
 
-    TestCaseResult execute() {
+    TestCaseResult execute() override {
         LOGV2(4333500, "### Running Test ###", "testFilePath"_attr = _testFilePath);
 
         ServerDescriptionPtr updatedServerDescription;
@@ -146,7 +176,7 @@ public:
         return result;
     }
 
-    const std::string& FilePath() const {
+    const std::string& FilePath() const override {
         return _testFilePath;
     }
 
@@ -220,9 +250,9 @@ public:
                                          testFilePath.string()};
         }
     }
-    ~JsonServerSelectionTestCase() = default;
+    ~JsonServerSelectionTestCase() override = default;
 
-    TestCaseResult execute() {
+    TestCaseResult execute() override {
         LOGV2(4333504, "### Running Test ###", "testFilePath"_attr = _testFilePath);
         if (_parseError)
             return *_parseError;
@@ -259,11 +289,11 @@ public:
         return result;
     }
 
-    const std::string& FilePath() const {
+    const std::string& FilePath() const override {
         return _testFilePath;
     }
 
-    bool errorIsExpected() {
+    bool errorIsExpected() override {
         return _errorExpected;
     }
 
@@ -514,7 +544,7 @@ public:
         std::string result;
         for (size_t i = 0; i < errors.size(); ++i) {
             auto error = errors[i];
-            result = result + error.first + " - " + error.second;
+            result += error.first += " - "s += error.second;
             if (i != errors.size() - 1) {
                 result += "; ";
             }
