@@ -28,7 +28,15 @@
  */
 
 #include "mongo/client/sdam/topology_listener.h"
+
+#include <algorithm>
+#include <iterator>
+#include <mutex>
+#include <utility>
+
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/util/assert_util.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
 
