@@ -29,10 +29,20 @@
 
 #include "mongo/client/sdam/topology_manager.h"
 
-#include <string>
+#include <memory>
+#include <mutex>
+#include <utility>
 
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/bson/oid.h"
+#include "mongo/client/sdam/server_description.h"
+#include "mongo/client/sdam/topology_description.h"
 #include "mongo/client/sdam/topology_state_machine.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
 #include "mongo/rpc/topology_version_gen.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kNetwork
@@ -89,8 +99,6 @@ bool TopologyManagerImpl::onServerDescription(const HelloOutcome& helloOutcome)
     boost::optional<TopologyVersion> newTopologyVersion = helloOutcome.getTopologyVersion();
     if (isStaleTopologyVersion(lastTopologyVersion, newTopologyVersion)) {
         LOGV2(23930,
-              "Ignoring this hello response because our topologyVersion: {lastTopologyVersion} is "
-              "fresher than the provided topologyVersion: {newTopologyVersion}",
               "Ignoring this hello response because our last topologyVersion is fresher than the "
               "new topologyVersion provided",
               "lastTopologyVersion"_attr = lastTopologyVersion->toBSON(),
@@ -142,7 +150,6 @@ void TopologyManagerImpl::onServerRTTUpdated(HostAndPort hostAndPort, HelloRTT r
     }
     // otherwise, the server was removed from the topology. Nothing to do.
     LOGV2(4333201,
-          "Not updating RTT. Server {server} does not exist in {replicaSet}",
           "Not updating RTT. The server does not exist in the replica set",
           "server"_attr = hostAndPort,
           "replicaSet"_attr = getTopologyDescription()->getSetName());
