@@ -28,20 +28,42 @@
  */
 
 
-#include "mongo/platform/basic.h"
-
+#include <algorithm>
+#include <list>
 #include <memory>
+#include <ratio>
+#include <utility>
+#include <vector>
 
-#include "mongo/client/sdam/sdam.h"
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/client/sdam/sdam_configuration.h"
+#include "mongo/client/sdam/topology_description.h"
 #include "mongo/client/sdam/topology_listener_mock.h"
 #include "mongo/client/server_ping_monitor.h"
+#include "mongo/dbtests/mock/mock_remote_db_server.h"
 #include "mongo/dbtests/mock/mock_replica_set.h"
+#include "mongo/executor/network_connection_hook.h"
 #include "mongo/executor/network_interface_mock.h"
+#include "mongo/executor/remote_command_request.h"
+#include "mongo/executor/remote_command_response.h"
+#include "mongo/executor/task_executor_test_fixture.h"
 #include "mongo/executor/thread_pool_mock.h"
 #include "mongo/executor/thread_pool_task_executor.h"
 #include "mongo/executor/thread_pool_task_executor_test_fixture.h"
 #include "mongo/logv2/log.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/rpc/op_msg.h"
+#include "mongo/rpc/reply_interface.h"
+#include "mongo/rpc/unique_message.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
 #include "mongo/util/duration.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
@@ -53,7 +75,6 @@ namespace {
 const sdam::HelloRTT initialRTT = duration_cast<Milliseconds>(Milliseconds(100));
 using executor::NetworkInterfaceMock;
 using executor::RemoteCommandResponse;
-using executor::ThreadPoolExecutorTest;
 using InNetworkGuard = NetworkInterfaceMock::InNetworkGuard;
 
 class ServerPingMonitorTestFixture : public unittest::Test {
@@ -125,7 +146,7 @@ protected:
         node->setCommandReply("ping", BSON("ok" << 1));
 
         if (node->isRunning()) {
-            const auto opmsg = OpMsgRequest::fromDBAndBody(request.dbname, request.cmdObj);
+            const auto opmsg = static_cast<OpMsgRequest>(request);
             const auto reply = node->runCommand(request.id, opmsg)->getCommandReply();
             _net->scheduleSuccessfulResponse(noi, RemoteCommandResponse(reply, Milliseconds(0)));
         } else {
@@ -223,14 +244,14 @@ private:
 
 class SingleServerPingMonitorTest : public ServerPingMonitorTestFixture {
 protected:
-    void setUp() {
+    void setUp() override {
         ServerPingMonitorTestFixture::setUp();
         _replSet.reset(new MockReplicaSet(
             "test", 1, /* hasPrimary = */ false, /* dollarPrefixHosts = */ false));
         _hostAndPort = HostAndPort(_replSet->getSecondaries()[0]);
     }
 
-    void tearDown() {
+    void tearDown() override {
         _replSet.reset();
         ServerPingMonitorTestFixture::tearDown();
     }
