@@ -29,9 +29,16 @@
 #include "mongo/client/streamable_replica_set_monitor_discovery_time_processor.h"
 
 #include <memory>
+#include <string>
 
-#include "mongo/client/global_conn_pool.h"
+#include <boost/optional/optional.hpp>
+
+#include "mongo/client/sdam/server_description.h"
+#include "mongo/client/sdam/topology_description.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/util/net/hostandport.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kNetwork
 
