@@ -30,33 +30,44 @@
 #pragma once
 
 #include <array>
+#include <boost/move/utility_core.hpp>
 #include <boost/multiprecision/cpp_int.hpp>
 #include <boost/optional.hpp>
+#include <boost/optional/optional.hpp>
+#include <compare>
 #include <cstdint>
+#include <cstring>
+#include <functional>
+#include <memory>
 #include <string>
 #include <utility>
 #include <vector>
 
 #include "mongo/base/data_range.h"
+#include "mongo/base/data_range_cursor.h"
 #include "mongo/base/data_type_validated.h"
 #include "mongo/base/secure_allocator.h"
+#include "mongo/base/status.h"
 #include "mongo/base/status_with.h"
 #include "mongo/base/string_data.h"
 #include "mongo/bson/bsonelement.h"
 #include "mongo/bson/bsonmisc.h"
 #include "mongo/bson/bsonobj.h"
 #include "mongo/bson/bsontypes.h"
+#include "mongo/bson/bsontypes_util.h"
 #include "mongo/crypto/encryption_fields_gen.h"
 #include "mongo/crypto/fle_crypto_types.h"
 #include "mongo/crypto/fle_field_schema_gen.h"
 #include "mongo/crypto/fle_stats.h"
 #include "mongo/crypto/fle_stats_gen.h"
-#include "mongo/crypto/hash_block.h"
 #include "mongo/crypto/sha256_block.h"
 #include "mongo/crypto/symmetric_crypto.h"
+#include "mongo/crypto/symmetric_key.h"
 #include "mongo/db/exec/document_value/value.h"
 #include "mongo/db/namespace_string.h"
-#include "mongo/rpc/object_check.h"
+#include "mongo/idl/idl_parser.h"
+#include "mongo/platform/decimal128.h"
+#include "mongo/rpc/object_check.h"  // IWYU pragma: keep
 #include "mongo/util/uuid.h"
 
 namespace mongo {
@@ -222,6 +233,35 @@ public:
         ServerDerivedFromDataToken token);
 };
 
+/**
+ * Generate token anchor padding root
+ */
+class FLEAnchorPaddingGenerator {
+public:
+    /**
+     * AnchorPaddingRootToken = HMAC(ESCToken, d) = S^esc_f_d = Fs[f,1,2,d]
+     *  d = 136 bit blob of zero = 17 octets of 0
+     */
+    static AnchorPaddingRootToken generateAnchorPaddingRootToken(ESCToken token);
+};
+
+/**
+ * Generate token anchor padding derived tokens
+ */
+class FLEAnchorPaddingDerivedGenerator {
+public:
+    /**
+     * AnchorPaddingKeyToken = HMAC(AnchorPaddingRootToken, 1) = Fs[f,1,2,d,1]
+     */
+    static AnchorPaddingKeyToken generateAnchorPaddingKeyToken(AnchorPaddingRootToken token);
+
+    /**
+     * AnchorPaddingValueToken = HMAC(AnchorPaddingRootToken, 2) = Fs[f,1,2,d,2]
+     */
+    static AnchorPaddingValueToken generateAnchorPaddingValueToken(AnchorPaddingRootToken token);
+};
+
+
 /**
  * ESC Collection schema
  * {
@@ -334,7 +374,7 @@ struct ESCDocument {
  */
 class FLETagQueryInterface {
 public:
-    enum class TagQueryType { kInsert, kQuery, kCompact };
+    enum class TagQueryType { kInsert, kQuery, kCompact, kCleanup, kPadding };
 
     virtual ~FLETagQueryInterface();
 
@@ -393,117 +433,36 @@ public:
     virtual ECStats getStats() const = 0;
 };
 
-class ESCCollection {
+template <class TagToken, class ValueToken>
+class ESCCollectionCommon {
 public:
     /**
-     * Generate the _id value
+     * Decrypt a regular document.
      */
-    static PrfBlock generateId(HmacContext* context,
-                               ESCTwiceDerivedTagToken tagToken,
-                               boost::optional<uint64_t> index);
-
-    /**
-     * Generate a null document which will be the "first" document for a given field.
-     */
-    static BSONObj generateNullDocument(HmacContext* context,
-                                        ESCTwiceDerivedTagToken tagToken,
-                                        ESCTwiceDerivedValueToken valueToken,
-                                        uint64_t pos,
-                                        uint64_t count);
-
-    /**
-     * Generate a insert ESC document.
-     */
-    static BSONObj generateInsertDocument(HmacContext* context,
-                                          ESCTwiceDerivedTagToken tagToken,
-                                          ESCTwiceDerivedValueToken valueToken,
-                                          uint64_t index,
-                                          uint64_t count);
-
-    /**
-     * Generate a compaction placeholder ESC document.
-     */
-    static BSONObj generateCompactionPlaceholderDocument(HmacContext* context,
-                                                         ESCTwiceDerivedTagToken tagToken,
-                                                         ESCTwiceDerivedValueToken valueToken,
-                                                         uint64_t index,
-                                                         uint64_t count);
-    /**
-     * Decrypt the null document.
-     */
-    static StatusWith<ESCNullDocument> decryptNullDocument(ESCTwiceDerivedValueToken valueToken,
-                                                           BSONObj& doc);
-
-    /**
-     * Decrypt the null document.
-     */
-    static StatusWith<ESCNullDocument> decryptNullDocument(ESCTwiceDerivedValueToken valueToken,
-                                                           BSONObj&& doc);
+    static StatusWith<ESCDocument> decryptDocument(const ValueToken& valueToken, BSONObj& doc);
 
     /**
      * Decrypt a regular document.
      */
-    static StatusWith<ESCDocument> decryptDocument(ESCTwiceDerivedValueToken valueToken,
-                                                   BSONObj& doc);
-
-    /**
-     * Decrypt a regular document.
-     */
-    static StatusWith<ESCDocument> decryptDocument(ESCTwiceDerivedValueToken valueToken,
-                                                   BSONObj&& doc);
-
-    /**
-     * Search for the highest document id for a given field/value pair based on the token.
-     */
-    static boost::optional<uint64_t> emuBinary(const FLEStateCollectionReader& reader,
-                                               ESCTwiceDerivedTagToken tagToken,
-                                               ESCTwiceDerivedValueToken valueToken);
-
-    // ===== Protocol Version 2 =====
-    /**
-     * Generate the _id value for a non-anchor record
-     */
-    static PrfBlock generateNonAnchorId(HmacContext* context,
-                                        const ESCTwiceDerivedTagToken& tagToken,
-                                        uint64_t cpos);
+    static StatusWith<ESCDocument> decryptDocument(const ValueToken& valueToken, BSONObj&& doc);
 
     /**
      * Generate the _id value for an anchor record
      */
-    static PrfBlock generateAnchorId(HmacContext* context,
-                                     const ESCTwiceDerivedTagToken& tagToken,
-                                     uint64_t apos);
+    static PrfBlock generateAnchorId(const TagToken& tagToken, uint64_t apos);
 
     /**
      * Generate the _id value for a null anchor record
      */
-    static PrfBlock generateNullAnchorId(HmacContext* context,
-                                         const ESCTwiceDerivedTagToken& tagToken);
+    static PrfBlock generateNullAnchorId(const TagToken& tagToken);
 
     /**
-     * Generate a non-anchor ESC document for inserts.
+     * Calculate AnchorBinaryHops as described in OST.
      */
-    static BSONObj generateNonAnchorDocument(HmacContext* context,
-                                             const ESCTwiceDerivedTagToken& tagToken,
-                                             uint64_t cpos);
-
-    /**
-     * Generate an anchor ESC document for compacts.
-     */
-    static BSONObj generateAnchorDocument(HmacContext* context,
-                                          const ESCTwiceDerivedTagToken& tagToken,
-                                          const ESCTwiceDerivedValueToken& valueToken,
-                                          uint64_t apos,
-                                          uint64_t cpos);
-
-    /**
-     * Generate a null anchor ESC document for cleanups.
-     */
-    static BSONObj generateNullAnchorDocument(HmacContext* context,
-                                              const ESCTwiceDerivedTagToken& tagToken,
-                                              const ESCTwiceDerivedValueToken& valueToken,
-                                              uint64_t apos,
-                                              uint64_t cpos);
+    static boost::optional<uint64_t> anchorBinaryHops(const FLEStateCollectionReader& reader,
+                                                      const TagToken& tagToken,
+                                                      const ValueToken& valueToken,
+                                                      FLEStatusSection::EmuBinaryTracker& tracker);
 
     /**
      * Decrypts an anchor document (either null or non-null).
@@ -512,9 +471,114 @@ public:
      * If the input document is a null anchor, then ESCDocument.position is the non-zero
      * anchor position (apos), and ESCDocument.count is the cpos.
      */
-    static StatusWith<ESCDocument> decryptAnchorDocument(
+    static StatusWith<ESCDocument> decryptAnchorDocument(const ValueToken& valueToken,
+                                                         BSONObj& doc);
+
+    /**
+     * Reads the anchor document identified by anchorId, and if found, decrypts the value
+     * and returns the parsed positions as a pair. If the anchor is not found, returns none.
+     */
+    static boost::optional<ESCCountsPair> readAndDecodeAnchor(
+        const FLEStateCollectionReader& reader,
+        const ValueToken& valueToken,
+        const PrfBlock& anchorId);
+
+    /**
+     * Performs all the ESC reads required by the QE cleanup algorithm, for anchor cleanup or
+     * padding cleanup.
+     */
+    static FLEEdgeCountInfo getEdgeCountInfoForPaddingCleanupCommon(
+        const FLEStateCollectionReader& reader,
+        const TagToken& tagToken,
+        const ValueToken& valueToken,
+        const EmuBinaryResult& positions);
+};
+
+/**
+ * Specialization of ESCollectionCommon for ESCTwiceDerived(Tag|Value)Tokens
+ * with additional methods specific to encrypted data.
+ */
+class ESCCollection
+    : public ESCCollectionCommon<ESCTwiceDerivedTagToken, ESCTwiceDerivedValueToken> {
+public:
+    /**
+     * Generate the _id value
+     */
+    static PrfBlock generateId(const ESCTwiceDerivedTagToken& tagToken,
+                               boost::optional<uint64_t> index);
+
+    /**
+     * Generate a null document which will be the "first" document for a given field.
+     */
+    static BSONObj generateNullDocument(const ESCTwiceDerivedTagToken& tagToken,
+                                        const ESCTwiceDerivedValueToken& valueToken,
+                                        uint64_t pos,
+                                        uint64_t count);
+
+    /**
+     * Generate a insert ESC document.
+     */
+    static BSONObj generateInsertDocument(const ESCTwiceDerivedTagToken& tagToken,
+                                          const ESCTwiceDerivedValueToken& valueToken,
+                                          uint64_t index,
+                                          uint64_t count);
+
+    /**
+     * Generate a compaction placeholder ESC document.
+     */
+    static BSONObj generateCompactionPlaceholderDocument(
+        const ESCTwiceDerivedTagToken& tagToken,
+        const ESCTwiceDerivedValueToken& valueToken,
+        uint64_t index,
+        uint64_t count);
+
+    /**
+     * Decrypt the null document.
+     */
+    static StatusWith<ESCNullDocument> decryptNullDocument(
         const ESCTwiceDerivedValueToken& valueToken, BSONObj& doc);
 
+    /**
+     * Decrypt the null document.
+     */
+    static StatusWith<ESCNullDocument> decryptNullDocument(
+        const ESCTwiceDerivedValueToken& valueToken, BSONObj&& doc);
+
+    /**
+     * Search for the highest document id for a given field/value pair based on the token.
+     */
+    static boost::optional<uint64_t> emuBinary(const FLEStateCollectionReader& reader,
+                                               const ESCTwiceDerivedTagToken& tagToken,
+                                               const ESCTwiceDerivedValueToken& valueToken);
+
+    // ===== Protocol Version 2 =====
+    /**
+     * Generate the _id value for a non-anchor record
+     */
+    static PrfBlock generateNonAnchorId(const ESCTwiceDerivedTagToken& tagToken, uint64_t cpos);
+
+    /**
+     * Generate a non-anchor ESC document for inserts.
+     */
+    static BSONObj generateNonAnchorDocument(const ESCTwiceDerivedTagToken& tagToken,
+                                             uint64_t cpos);
+
+    /**
+     * Generate an anchor ESC document for compacts.
+     */
+    static BSONObj generateAnchorDocument(const ESCTwiceDerivedTagToken& tagToken,
+                                          const ESCTwiceDerivedValueToken& valueToken,
+                                          uint64_t apos,
+                                          uint64_t cpos);
+
+    /**
+     * Generate a null anchor ESC document for cleanups.
+     */
+    static BSONObj generateNullAnchorDocument(const ESCTwiceDerivedTagToken& tagToken,
+                                              const ESCTwiceDerivedValueToken& valueToken,
+                                              uint64_t apos,
+                                              uint64_t cpos);
+
     /*
      * Note on EmuBinaryV2 results:
      *    i = non-anchor position (cpos)
@@ -531,21 +595,10 @@ public:
      *    (x > 0) means non-null anchors exist without a null anchor OR new non-null anchors
      *            have been added since the last-recorded apos in the null anchor.
      */
-    struct EmuBinaryResult {
-        boost::optional<uint64_t> cpos;
-        boost::optional<uint64_t> apos;
-    };
-    static EmuBinaryResult emuBinaryV2(HmacContext* context,
-                                       const FLEStateCollectionReader& reader,
+    static EmuBinaryResult emuBinaryV2(const FLEStateCollectionReader& reader,
                                        const ESCTwiceDerivedTagToken& tagToken,
                                        const ESCTwiceDerivedValueToken& valueToken);
-    static boost::optional<uint64_t> anchorBinaryHops(HmacContext* context,
-                                                      const FLEStateCollectionReader& reader,
-                                                      const ESCTwiceDerivedTagToken& tagToken,
-                                                      const ESCTwiceDerivedValueToken& valueToken,
-                                                      FLEStatusSection::EmuBinaryTracker& tracker);
-    static boost::optional<uint64_t> binaryHops(HmacContext* context,
-                                                const FLEStateCollectionReader& reader,
+    static boost::optional<uint64_t> binaryHops(const FLEStateCollectionReader& reader,
                                                 const ESCTwiceDerivedTagToken& tagToken,
                                                 const ESCTwiceDerivedValueToken& valueToken,
                                                 boost::optional<uint64_t> x,
@@ -563,6 +616,25 @@ public:
         FLETagQueryInterface::TagQueryType type);
 };
 
+/**
+ * Specialization of ESCollectionCommon for AnchorPadding(Key|Value)Tokens
+ * with a custom anchor padding document generator.
+ */
+class ESCCollectionAnchorPadding
+    : public ESCCollectionCommon<AnchorPaddingKeyToken, AnchorPaddingValueToken> {
+public:
+    static PrfBlock generateNullAnchorId(const AnchorPaddingKeyToken& tagToken);
+    static PrfBlock generateAnchorId(const AnchorPaddingKeyToken& tagToken, uint64_t apos);
+
+    static BSONObj generateNullAnchorDocument(const AnchorPaddingKeyToken& keyToken,
+                                              const AnchorPaddingValueToken& valueToken,
+                                              uint64_t apos,
+                                              uint64_t /* cpos ignored */);
+
+    static BSONObj generatePaddingDocument(const AnchorPaddingKeyToken& keyToken,
+                                           const AnchorPaddingValueToken& valueToken,
+                                           uint64_t apos);
+};
 
 /**
  * ECC Collection
@@ -672,15 +744,12 @@ public:
     /**
      * Generate the _id value
      */
-    static PrfBlock generateId(HmacContext* context,
-                               ECCTwiceDerivedTagToken tagToken,
-                               boost::optional<uint64_t> index);
+    static PrfBlock generateId(ECCTwiceDerivedTagToken tagToken, boost::optional<uint64_t> index);
 
     /**
      * Generate a null document which will be the "first" document for a given field.
      */
-    static BSONObj generateNullDocument(HmacContext* context,
-                                        ECCTwiceDerivedTagToken tagToken,
+    static BSONObj generateNullDocument(ECCTwiceDerivedTagToken tagToken,
                                         ECCTwiceDerivedValueToken valueToken,
                                         uint64_t count);
 
@@ -689,8 +758,7 @@ public:
      *
      * Note: it is stored as (count, count)
      */
-    static BSONObj generateDocument(HmacContext* context,
-                                    ECCTwiceDerivedTagToken tagToken,
+    static BSONObj generateDocument(ECCTwiceDerivedTagToken tagToken,
                                     ECCTwiceDerivedValueToken valueToken,
                                     uint64_t index,
                                     uint64_t count);
@@ -698,8 +766,7 @@ public:
     /**
      * Generate a regular ECC document for (start, end)
      */
-    static BSONObj generateDocument(HmacContext* context,
-                                    ECCTwiceDerivedTagToken tagToken,
+    static BSONObj generateDocument(ECCTwiceDerivedTagToken tagToken,
                                     ECCTwiceDerivedValueToken valueToken,
                                     uint64_t index,
                                     uint64_t start,
@@ -708,8 +775,7 @@ public:
     /**
      * Generate a compaction ECC document.
      */
-    static BSONObj generateCompactionDocument(HmacContext* context,
-                                              ECCTwiceDerivedTagToken tagToken,
+    static BSONObj generateCompactionDocument(ECCTwiceDerivedTagToken tagToken,
                                               ECCTwiceDerivedValueToken valueToken,
                                               uint64_t index);
 
@@ -785,6 +851,7 @@ public:
                                                               FLEUserKeyAndId userKey,
                                                               const std::vector<std::string>& edges,
                                                               uint64_t maxContentionFactor,
+                                                              uint32_t sparsity,
                                                               const FLE2RangeFindSpec& spec);
 
     static FLE2FindRangePayloadV2 serializeFindRangeStubV2(const FLE2RangeFindSpec& spec);
@@ -835,18 +902,6 @@ public:
      * Validate the tags array exists and is of the right type.
      */
     static void validateTagsArray(const BSONObj& doc);
-
-    /**
-     * Validate document
-     *
-     * Checks performed
-     * 1. Fields, if present, are indexed the way specified
-     * 2. All fields can be decrypted successfully
-     * 3. There is a tag for each field and no extra tags
-     */
-    static void validateDocument(const BSONObj& doc,
-                                 const EncryptedFieldConfig& efc,
-                                 FLEKeyVault* keyVault);
 };
 
 /*
@@ -881,16 +936,28 @@ public:
  *
  * struct {
  *    uint8_t[32] esc;
+ *    uint8_t isLeaf; // Optional: 0 or 1 for range operations, absent for equality.
  * }
  */
 struct EncryptedStateCollectionTokensV2 {
 public:
-    EncryptedStateCollectionTokensV2(ESCDerivedFromDataTokenAndContentionFactorToken s) : esc(s) {}
+    EncryptedStateCollectionTokensV2(ESCDerivedFromDataTokenAndContentionFactorToken s,
+                                     boost::optional<bool> leaf)
+        : esc(s), isLeaf(std::move(leaf)) {}
     static StatusWith<EncryptedStateCollectionTokensV2> decryptAndParse(ECOCToken token,
                                                                         ConstDataRange cdr);
     StatusWith<std::vector<uint8_t>> serialize(ECOCToken token);
 
+    bool isEquality() const {
+        return isLeaf == boost::none;
+    }
+
+    bool isRange() const {
+        return isLeaf != boost::none;
+    }
+
     ESCDerivedFromDataTokenAndContentionFactorToken esc;
+    boost::optional<bool> isLeaf;
 };
 
 struct ECOCCompactionDocumentV2 {
@@ -903,9 +970,19 @@ struct ECOCCompactionDocumentV2 {
         return H::combine(std::move(h), doc.fieldName, doc.esc);
     }
 
+    bool isEquality() const {
+        return isLeaf == boost::none;
+    }
+
+    bool isRange() const {
+        return isLeaf != boost::none;
+    }
+
     // Id is not included as it unimportant
     std::string fieldName;
     ESCDerivedFromDataTokenAndContentionFactorToken esc;
+    boost::optional<bool> isLeaf;
+    boost::optional<AnchorPaddingRootToken> anchorPaddingRootToken;
 };
 
 /**
@@ -1238,10 +1315,9 @@ public:
      *
      * HMAC(EDCTwiceDerivedToken, count)
      */
-    static PrfBlock generateTag(HmacContext* obj,
-                                EDCTwiceDerivedToken edcTwiceDerived,
-                                FLECounter count);
+    static PrfBlock generateTag(EDCTwiceDerivedToken edcTwiceDerived, FLECounter count);
     static PrfBlock generateTag(const EDCServerPayloadInfo& payload);
+    static PrfBlock generateTag(const FLEEdgeToken& token, FLECounter count);
     static std::vector<PrfBlock> generateTags(const EDCServerPayloadInfo& rangePayload);
 
     /**
@@ -1339,6 +1415,15 @@ public:
 struct CompactionToken {
     std::string fieldPathName;
     ECOCToken token;
+    boost::optional<AnchorPaddingRootToken> anchorPaddingToken;
+
+    bool isEquality() const {
+        return anchorPaddingToken == boost::none;
+    }
+
+    bool isRange() const {
+        return anchorPaddingToken != boost::none;
+    }
 };
 
 class CompactionHelpers {
@@ -1355,6 +1440,15 @@ public:
      * in the encrypted field config
      */
     static void validateCompactionTokens(const EncryptedFieldConfig& efc, BSONObj compactionTokens);
+
+    /**
+     * Validates the compaction tokens BSON contains an element for each field
+     * in the encrypted field config
+     */
+    static void validateCleanupTokens(const EncryptedFieldConfig& efc, BSONObj cleanupTokens);
+
+private:
+    static void _validateTokens(const EncryptedFieldConfig& efc, BSONObj tokens, StringData cmd);
 };
 
 /**
@@ -1378,88 +1472,6 @@ struct ParsedFindEqualityPayload {
     explicit ParsedFindEqualityPayload(ConstDataRange cdr);
 };
 
-
-/**
- * FLE2 Range Utility functions
- */
-
-/**
- * Describe the encoding of an BSON int32
- *
- * NOTE: It is not a mistake that a int32 is encoded as uint32.
- */
-struct OSTType_Int32 {
-    OSTType_Int32(uint32_t v, uint32_t minP, uint32_t maxP) : value(v), min(minP), max(maxP) {}
-
-    uint32_t value;
-    uint32_t min;
-    uint32_t max;
-};
-
-OSTType_Int32 getTypeInfo32(int32_t value,
-                            boost::optional<int32_t> min,
-                            boost::optional<int32_t> max);
-
-/**
- * Describe the encoding of an BSON int64
- *
- * NOTE: It is not a mistake that a int64 is encoded as uint64.
- */
-struct OSTType_Int64 {
-    OSTType_Int64(uint64_t v, uint64_t minP, uint64_t maxP) : value(v), min(minP), max(maxP) {}
-
-    uint64_t value;
-    uint64_t min;
-    uint64_t max;
-};
-
-OSTType_Int64 getTypeInfo64(int64_t value,
-                            boost::optional<int64_t> min,
-                            boost::optional<int64_t> max);
-
-
-/**
- * Describe the encoding of an BSON double (i.e. IEEE 754 Binary64)
- *
- * NOTE: It is not a mistake that a double is encoded as uint64.
- */
-struct OSTType_Double {
-    OSTType_Double(uint64_t v, uint64_t minP, uint64_t maxP) : value(v), min(minP), max(maxP) {}
-
-    uint64_t value;
-    uint64_t min;
-    uint64_t max;
-};
-
-OSTType_Double getTypeInfoDouble(double value,
-                                 boost::optional<double> min,
-                                 boost::optional<double> max,
-                                 boost::optional<uint32_t> precision);
-/**
- * Describe the encoding of an BSON Decimal (i.e. IEEE 754 Decimal128)
- *
- * NOTE: It is not a mistake that a decimal is encoded as uint128.
- */
-
-struct OSTType_Decimal128 {
-    OSTType_Decimal128(boost::multiprecision::uint128_t v,
-                       boost::multiprecision::uint128_t minP,
-                       boost::multiprecision::uint128_t maxP)
-        : value(v), min(minP), max(maxP) {}
-
-    boost::multiprecision::uint128_t value;
-    boost::multiprecision::uint128_t min;
-    boost::multiprecision::uint128_t max;
-};
-
-boost::multiprecision::uint128_t toInt128FromDecimal128(Decimal128 dec);
-
-OSTType_Decimal128 getTypeInfoDecimal128(Decimal128 value,
-                                         boost::optional<Decimal128> min,
-                                         boost::optional<Decimal128> max,
-                                         boost::optional<uint32_t> precision);
-
-
 struct FLEFindEdgeTokenSet {
     EDCDerivedFromDataToken edc;
     ESCDerivedFromDataToken esc;
@@ -1475,6 +1487,11 @@ struct ParsedFindRangePayload {
     std::int32_t payloadId{};
 
     std::int64_t maxCounter{};
+    boost::optional<std::int32_t> sparsity{};
+    boost::optional<std::int32_t> trimFactor{};
+    boost::optional<std::int32_t> precision{};
+    boost::optional<IDLAnyType> indexMin{};
+    boost::optional<IDLAnyType> indexMax{};
 
     explicit ParsedFindRangePayload(BSONElement fleFindRangePayload);
     explicit ParsedFindRangePayload(const Value& fleFindRangePayload);
@@ -1492,36 +1509,49 @@ struct ParsedFindRangePayload {
 
 class Edges {
 public:
-    Edges(std::string leaf, int sparsity);
+    Edges(std::string leaf, int sparsity, const boost::optional<int>& trimFactor);
     std::vector<StringData> get();
+    std::size_t size() const;
+    const std::string& getLeaf() const {
+        return _leaf;
+    }
 
 private:
     std::string _leaf;
     int _sparsity;
+    int _trimFactor;
 };
 
 std::unique_ptr<Edges> getEdgesInt32(int32_t value,
                                      boost::optional<int32_t> min,
                                      boost::optional<int32_t> max,
-                                     int sparsity);
+                                     int sparsity,
+                                     const boost::optional<int>& trimFactor);
 
 std::unique_ptr<Edges> getEdgesInt64(int64_t value,
                                      boost::optional<int64_t> min,
                                      boost::optional<int64_t> max,
-                                     int sparsity);
+                                     int sparsity,
+                                     const boost::optional<int>& trimFactor);
 
 std::unique_ptr<Edges> getEdgesDouble(double value,
                                       boost::optional<double> min,
                                       boost::optional<double> max,
                                       boost::optional<uint32_t> precision,
-                                      int sparsity);
+                                      int sparsity,
+                                      const boost::optional<int>& trimFactor);
 
 std::unique_ptr<Edges> getEdgesDecimal128(Decimal128 value,
                                           boost::optional<Decimal128> min,
                                           boost::optional<Decimal128> max,
                                           boost::optional<uint32_t> precision,
+                                          int sparsity,
+                                          const boost::optional<int>& trimFactor);
+
+// Equivalent to a full edges calculation without creating an intemediate vector.
+// getEdgesT(min, min, max, precision, sparsity, trimFactor).size()
+std::uint64_t getEdgesLength(BSONType fieldType, StringData fieldPath, QueryTypeConfig config);
 
-                                          int sparsity);
 /**
  * Mincover calculator
  */
@@ -1532,7 +1562,8 @@ std::vector<std::string> minCoverInt32(int32_t lowerBound,
                                        bool includeUpperBound,
                                        boost::optional<int32_t> min,
                                        boost::optional<int32_t> max,
-                                       int sparsity);
+                                       int sparsity,
+                                       const boost::optional<int>& trimFactor);
 
 std::vector<std::string> minCoverInt64(int64_t lowerBound,
                                        bool includeLowerBound,
@@ -1540,7 +1571,8 @@ std::vector<std::string> minCoverInt64(int64_t lowerBound,
                                        bool includeUpperBound,
                                        boost::optional<int64_t> min,
                                        boost::optional<int64_t> max,
-                                       int sparsity);
+                                       int sparsity,
+                                       const boost::optional<int>& trimFactor);
 
 std::vector<std::string> minCoverDouble(double lowerBound,
                                         bool includeLowerBound,
@@ -1549,7 +1581,8 @@ std::vector<std::string> minCoverDouble(double lowerBound,
                                         boost::optional<double> min,
                                         boost::optional<double> max,
                                         boost::optional<uint32_t> precision,
-                                        int sparsity);
+                                        int sparsity,
+                                        const boost::optional<int>& trimFactor);
 
 std::vector<std::string> minCoverDecimal128(Decimal128 lowerBound,
                                             bool includeLowerBound,
@@ -1558,7 +1591,8 @@ std::vector<std::string> minCoverDecimal128(Decimal128 lowerBound,
                                             boost::optional<Decimal128> min,
                                             boost::optional<Decimal128> max,
                                             boost::optional<uint32_t> precision,
-                                            int sparsity);
+                                            int sparsity,
+                                            const boost::optional<int>& trimFactor);
 
 class FLEUtil {
 public:
@@ -1568,11 +1602,11 @@ public:
     /**
      * Compute HMAC-SHA-256
      */
-    static PrfBlock prf(HmacContext* hmacCtx, ConstDataRange key, uint64_t value, int64_t value2);
+    static PrfBlock prf(ConstDataRange key, ConstDataRange cdr);
 
-    static PrfBlock prf(HmacContext* hmacCtx, ConstDataRange key, ConstDataRange cdr);
+    static PrfBlock prf(ConstDataRange key, uint64_t value);
 
-    static PrfBlock prf(HmacContext* hmacCtx, ConstDataRange key, uint64_t value);
+    static void checkEFCForECC(const EncryptedFieldConfig& efc);
 
     /**
      * Decrypt AES-256-CTR encrypted data. Exposed for benchmarking purposes.
@@ -1586,6 +1620,11 @@ public:
  */
 PrfBlock PrfBlockfromCDR(const ConstDataRange& block);
 
+template <typename TokenT>
+TokenT FLETokenFromCDR(const ConstDataRange& block) {
+    return TokenT(PrfBlockfromCDR(block));
+}
+
 ConstDataRange binDataToCDR(BSONElement element);
 
 template <typename T>
@@ -1609,6 +1648,8 @@ boost::optional<EncryptedBinDataType> getEncryptedBinDataType(const BSONElement&
 bool hasQueryType(const EncryptedField& field, QueryTypeEnum queryType);
 bool hasQueryType(const EncryptedFieldConfig& config, QueryTypeEnum queryType);
 
+QueryTypeConfig getQueryType(const EncryptedField& field, QueryTypeEnum queryType);
+
 /**
  * Get the set of edges that minimally cover a range query specified by the given range spec and
  * sparsity.jj
