@@ -29,8 +29,13 @@
 
 #pragma once
 
-#include "mongo/crypto/hash_block.h"
+#include <cstddef>
+#include <cstdint>
+#include <initializer_list>
 
+#include "mongo/base/data_range.h"
+#include "mongo/base/string_data.h"
+#include "mongo/crypto/hash_block.h"
 #include "mongo/util/make_array_type.h"
 
 namespace mongo {
@@ -49,12 +54,6 @@ struct SHA1BlockTraits {
                             size_t keyLen,
                             std::initializer_list<ConstDataRange> input,
                             HashType* output);
-
-    static void computeHmacWithCtx(HmacContext* ctx,
-                                   const uint8_t* key,
-                                   size_t keyLen,
-                                   std::initializer_list<ConstDataRange> input,
-                                   HashType* output);
 };
 
 using SHA1Block = HashBlock<SHA1BlockTraits>;
