@@ -27,25 +27,27 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
+#include <algorithm>
+#include <initializer_list>
 #include <memory>
 
+#include "mongo/base/data_range.h"
+#include "mongo/config.h"  // IWYU pragma: keep
 #include "mongo/crypto/sha1_block.h"
 #include "mongo/crypto/sha256_block.h"
 #include "mongo/crypto/sha512_block.h"
-
-#include "mongo/config.h"
 #include "mongo/util/assert_util.h"
 
 #ifndef MONGO_CONFIG_SSL
 #error This file should only be included in SSL-enabled builds
 #endif
 
+#include <cstdint>
 #include <cstring>
 #include <openssl/evp.h>
 #include <openssl/hmac.h>
-#include <openssl/sha.h>
+#include <openssl/opensslv.h>
+#include <openssl/ossl_typ.h>
 
 #if OPENSSL_VERSION_NUMBER < 0x10100000L || \
     (defined(LIBRESSL_VERSION_NUMBER) && LIBRESSL_VERSION_NUMBER < 0x2070000fL)
@@ -167,25 +169,6 @@ void computeHashImpl(const EVP_MD* md,
                 EVP_DigestFinal_ex(digestCtx.get(), output->data(), nullptr) == 1);
 }
 
-template <typename HashType>
-void computeHmacImplWithCtx(HMAC_CTX* digestCtx,
-                            const EVP_MD* md,
-                            const uint8_t* key,
-                            size_t keyLen,
-                            std::initializer_list<ConstDataRange> input,
-                            HashType* const output) {
-    fassert(40380,
-            HMAC_Init_ex(digestCtx, key, keyLen, md, nullptr) == 1 &&
-                std::all_of(begin(input),
-                            end(input),
-                            [&](const auto& i) {
-                                return HMAC_Update(digestCtx,
-                                                   reinterpret_cast<const unsigned char*>(i.data()),
-                                                   i.length()) == 1;
-                            }) &&
-                HMAC_Final(digestCtx, output->data(), nullptr) == 1);
-}
-
 template <typename HashType>
 void computeHmacImpl(const EVP_MD* md,
                      const uint8_t* key,
@@ -193,7 +176,17 @@ void computeHmacImpl(const EVP_MD* md,
                      std::initializer_list<ConstDataRange> input,
                      HashType* const output) {
     std::unique_ptr<HMAC_CTX, decltype(&HMAC_CTX_free)> digestCtx(HMAC_CTX_new(), HMAC_CTX_free);
-    computeHmacImplWithCtx<HashType>(digestCtx.get(), md, key, keyLen, input, output);
+
+    fassert(40380,
+            HMAC_Init_ex(digestCtx.get(), key, keyLen, md, nullptr) == 1 &&
+                std::all_of(begin(input),
+                            end(input),
+                            [&](const auto& i) {
+                                return HMAC_Update(digestCtx.get(),
+                                                   reinterpret_cast<const unsigned char*>(i.data()),
+                                                   i.length()) == 1;
+                            }) &&
+                HMAC_Final(digestCtx.get(), output->data(), nullptr) == 1);
 }
 
 }  // namespace
@@ -221,15 +214,6 @@ void SHA1BlockTraits::computeHmac(const uint8_t* key,
         getOpenSSLHashLoader().getSHA1(), key, keyLen, input, output);
 }
 
-void SHA1BlockTraits::computeHmacWithCtx(HmacContext* ctx,
-                                         const uint8_t* key,
-                                         size_t keyLen,
-                                         std::initializer_list<ConstDataRange> input,
-                                         HashType* const output) {
-    return computeHmacImplWithCtx<SHA1BlockTraits::HashType>(
-        ctx->get(), getOpenSSLHashLoader().getSHA1(), key, keyLen, input, output);
-}
-
 void SHA256BlockTraits::computeHmac(const uint8_t* key,
                                     size_t keyLen,
                                     std::initializer_list<ConstDataRange> input,
@@ -238,15 +222,6 @@ void SHA256BlockTraits::computeHmac(const uint8_t* key,
         getOpenSSLHashLoader().getSHA256(), key, keyLen, input, output);
 }
 
-void SHA256BlockTraits::computeHmacWithCtx(HmacContext* ctx,
-                                           const uint8_t* key,
-                                           size_t keyLen,
-                                           std::initializer_list<ConstDataRange> input,
-                                           HashType* const output) {
-    return computeHmacImplWithCtx<SHA256BlockTraits::HashType>(
-        ctx->get(), getOpenSSLHashLoader().getSHA256(), key, keyLen, input, output);
-}
-
 void SHA512BlockTraits::computeHmac(const uint8_t* key,
                                     size_t keyLen,
                                     std::initializer_list<ConstDataRange> input,
@@ -255,13 +230,4 @@ void SHA512BlockTraits::computeHmac(const uint8_t* key,
         getOpenSSLHashLoader().getSHA512(), key, keyLen, input, output);
 }
 
-void SHA512BlockTraits::computeHmacWithCtx(HmacContext* ctx,
-                                           const uint8_t* key,
-                                           size_t keyLen,
-                                           std::initializer_list<ConstDataRange> input,
-                                           HashType* const output) {
-    return computeHmacImplWithCtx<SHA512BlockTraits::HashType>(
-        ctx->get(), getOpenSSLHashLoader().getSHA512(), key, keyLen, input, output);
-}
-
 }  // namespace mongo
