@@ -29,12 +29,30 @@
 
 
 #include "mongo/db/auth/authentication_session.h"
+
+#include <algorithm>
+#include <ratio>
+#include <vector>
+
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/bson/bsonmisc.h"
 #include "mongo/client/authenticate.h"
 #include "mongo/db/audit.h"
-#include "mongo/db/auth/authentication_metrics.h"
+#include "mongo/db/auth/auth_name.h"
 #include "mongo/db/client.h"
+#include "mongo/db/commands/test_commands_enabled.h"
 #include "mongo/db/connection_health_metrics_parameter_gen.h"
+#include "mongo/db/service_context.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/redaction.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/duration.h"
+#include "mongo/util/scopeguard.h"
+#include "mongo/util/str.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kAccessControl
 
@@ -261,8 +279,7 @@ void AuthenticationSession::_verifyUserNameFromSaslSupportedMechanisms(const Use
         // Reset _ssmUserName since we have found a conflict.
         auto ssmUserName = std::exchange(_ssmUserName, {});
         auto event = audit::AuthenticateEvent(auth::kSaslSupportedMechanisms,
-                                              ssmUserName.getDB(),
-                                              ssmUserName.getUser(),
+                                              ssmUserName,
                                               makeAppender(_mech.get()),
                                               ErrorCodes::AuthenticationAbandoned);
         audit::logAuthentication(_client, event);
@@ -339,22 +356,18 @@ void AuthenticationSession::markSuccessful() {
         _mechCounter->incSpeculativeAuthenticateSuccessful();
     }
 
-    auto event = audit::AuthenticateEvent(_mechName,
-                                          _userName.getDB(),
-                                          _userName.getUser(),
-                                          makeAppender(_mech.get()),
-                                          ErrorCodes::OK);
+    auto event =
+        audit::AuthenticateEvent(_mechName, _userName, makeAppender(_mech.get()), ErrorCodes::OK);
     audit::logAuthentication(_client, event);
 
     BSONObj metrics = _metricsRecorder.capture();
 
-    if (gEnableDetailedConnectionHealthMetricLogLines) {
+    if (gEnableDetailedConnectionHealthMetricLogLines.load()) {
         BSONObjBuilder extraInfoBob;
         if (_mech) {
             _mech->appendExtraInfo(&extraInfoBob);
         }
 
-        auto metadata = ClientMetadata::get(_client);
         LOGV2(5286306,
               "Successfully authenticated",
               "client"_attr = _client->getRemote(),
@@ -365,7 +378,6 @@ void AuthenticationSession::markSuccessful() {
               "db"_attr = _userName.getDB(),
               "result"_attr = Status::OK().code(),
               "metrics"_attr = metrics,
-              "doc"_attr = metadata ? metadata->getDocument() : BSONObj(),
               "extraInfo"_attr = extraInfoBob.obj());
     }
 }
@@ -377,23 +389,19 @@ void AuthenticationSession::markFailed(const Status& status) {
     // is effectively no longer speculative. We should continue auditing
     // in this case.
     if (!_isSpeculative || (_lastStep && _lastStep == StepType::kSaslContinue)) {
-        auto event = audit::AuthenticateEvent(_mechName,
-                                              _userName.getDB(),
-                                              _userName.getUser(),
-                                              makeAppender(_mech.get()),
-                                              status.code());
+        auto event = audit::AuthenticateEvent(
+            _mechName, _userName, makeAppender(_mech.get()), status.code());
         audit::logAuthentication(_client, event);
     }
 
     BSONObj metrics = _metricsRecorder.capture();
 
-    if (gEnableDetailedConnectionHealthMetricLogLines) {
+    if (gEnableDetailedConnectionHealthMetricLogLines.load()) {
         BSONObjBuilder extraInfoBob;
         if (_mech) {
             _mech->appendExtraInfo(&extraInfoBob);
         }
 
-        auto metadata = ClientMetadata::get(_client);
         LOGV2(5286307,
               "Failed to authenticate",
               "client"_attr = _client->getRemote(),
@@ -405,7 +413,6 @@ void AuthenticationSession::markFailed(const Status& status) {
               "error"_attr = redact(status),
               "result"_attr = status.code(),
               "metrics"_attr = metrics,
-              "doc"_attr = metadata ? metadata->getDocument() : BSONObj(),
               "extraInfo"_attr = extraInfoBob.obj());
     }
 }
