@@ -27,12 +27,21 @@
  *    it in the license file.
  */
 
-#include "mongo/db/auth/access_checks_gen.h"
-#include "mongo/db/auth/authorization_contract.h"
-
-#include "mongo/db/auth/privilege.h"
-#include "mongo/unittest/unittest.h"
 #include <initializer_list>
+#include <memory>
+
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+
+#include "mongo/base/string_data.h"
+#include "mongo/db/auth/access_checks_gen.h"
+#include "mongo/db/auth/action_type.h"
+#include "mongo/db/auth/authorization_contract.h"
+#include "mongo/db/auth/privilege.h"
+#include "mongo/db/auth/resource_pattern.h"
+#include "mongo/db/tenant_id.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
 
 namespace mongo {
 namespace {
@@ -47,7 +56,8 @@ TEST(AuthContractTest, Basic) {
     enableShardingActions.addAction(ActionType::enableSharding);
     enableShardingActions.addAction(ActionType::refineCollectionShardKey);
     enableShardingActions.addAction(ActionType::reshardCollection);
-    ac.addPrivilege(Privilege(ResourcePattern::forAnyNormalResource(), enableShardingActions));
+    ac.addPrivilege(
+        Privilege(ResourcePattern::forAnyNormalResource(boost::none), enableShardingActions));
 
     ASSERT_TRUE(ac.hasAccessCheck(AccessCheckEnum::kIsAuthenticated));
     ASSERT_TRUE(ac.hasAccessCheck(AccessCheckEnum::kIsCoAuthorized));
@@ -56,20 +66,20 @@ TEST(AuthContractTest, Basic) {
 
 
     ASSERT_TRUE(ac.hasPrivileges(
-        Privilege(ResourcePattern::forAnyNormalResource(), ActionType::enableSharding)));
-    ASSERT_TRUE(ac.hasPrivileges(
-        Privilege(ResourcePattern::forAnyNormalResource(), ActionType::refineCollectionShardKey)));
-    ASSERT_TRUE(ac.hasPrivileges(
-        Privilege(ResourcePattern::forAnyNormalResource(), ActionType::reshardCollection)));
+        Privilege(ResourcePattern::forAnyNormalResource(boost::none), ActionType::enableSharding)));
+    ASSERT_TRUE(ac.hasPrivileges(Privilege(ResourcePattern::forAnyNormalResource(boost::none),
+                                           ActionType::refineCollectionShardKey)));
+    ASSERT_TRUE(ac.hasPrivileges(Privilege(ResourcePattern::forAnyNormalResource(boost::none),
+                                           ActionType::reshardCollection)));
 
-    ASSERT_FALSE(
-        ac.hasPrivileges(Privilege(ResourcePattern::forAnyNormalResource(), ActionType::shutdown)));
     ASSERT_FALSE(ac.hasPrivileges(
-        Privilege(ResourcePattern::forClusterResource(), ActionType::enableSharding)));
+        Privilege(ResourcePattern::forAnyNormalResource(boost::none), ActionType::shutdown)));
+    ASSERT_FALSE(ac.hasPrivileges(
+        Privilege(ResourcePattern::forClusterResource(boost::none), ActionType::enableSharding)));
 
 
     ASSERT_TRUE(ac.hasPrivileges(
-        Privilege(ResourcePattern::forAnyNormalResource(), enableShardingActions)));
+        Privilege(ResourcePattern::forAnyNormalResource(boost::none), enableShardingActions)));
 
     ASSERT_TRUE(ac.contains(ac));
 }
@@ -103,7 +113,8 @@ TEST(AuthContractTest, DifferentAccessCheck) {
 TEST(AuthContractTest, SimplePrivilege) {
 
     AuthorizationContract ac;
-    ac.addPrivilege(Privilege(ResourcePattern::forAnyNormalResource(), ActionType::enableSharding));
+    ac.addPrivilege(
+        Privilege(ResourcePattern::forAnyNormalResource(boost::none), ActionType::enableSharding));
 
     AuthorizationContract empty;
 
@@ -116,10 +127,11 @@ TEST(AuthContractTest, DifferentResoucePattern) {
 
     AuthorizationContract ac1;
     ac1.addPrivilege(
-        Privilege(ResourcePattern::forAnyNormalResource(), ActionType::enableSharding));
+        Privilege(ResourcePattern::forAnyNormalResource(boost::none), ActionType::enableSharding));
 
     AuthorizationContract ac2;
-    ac2.addPrivilege(Privilege(ResourcePattern::forClusterResource(), ActionType::enableSharding));
+    ac2.addPrivilege(
+        Privilege(ResourcePattern::forClusterResource(boost::none), ActionType::enableSharding));
 
     ASSERT_FALSE(ac1.contains(ac2));
     ASSERT_FALSE(ac2.contains(ac1));
@@ -130,17 +142,16 @@ TEST(AuthContractTest, DifferentActionType) {
 
     AuthorizationContract ac1;
     ac1.addPrivilege(
-        Privilege(ResourcePattern::forAnyNormalResource(), ActionType::enableSharding));
+        Privilege(ResourcePattern::forAnyNormalResource(boost::none), ActionType::enableSharding));
 
     AuthorizationContract ac2;
-    ac2.addPrivilege(
-        Privilege(ResourcePattern::forAnyNormalResource(), ActionType::grantPrivilegesToRole));
+    ac2.addPrivilege(Privilege(ResourcePattern::forAnyNormalResource(boost::none),
+                               ActionType::grantPrivilegesToRole));
 
     ASSERT_FALSE(ac1.contains(ac2));
     ASSERT_FALSE(ac2.contains(ac1));
 }
 
-
 TEST(AuthContractTest, InitializerList) {
 
     AuthorizationContract ac1;
@@ -151,15 +162,17 @@ TEST(AuthContractTest, InitializerList) {
     enableShardingActions.addAction(ActionType::enableSharding);
     enableShardingActions.addAction(ActionType::refineCollectionShardKey);
     enableShardingActions.addAction(ActionType::reshardCollection);
-    ac1.addPrivilege(Privilege(ResourcePattern::forAnyNormalResource(), enableShardingActions));
+    ac1.addPrivilege(
+        Privilege(ResourcePattern::forAnyNormalResource(boost::none), enableShardingActions));
 
     AuthorizationContract ac2(
         std::initializer_list<AccessCheckEnum>{AccessCheckEnum::kIsAuthenticated,
                                                AccessCheckEnum::kIsCoAuthorized},
-        std::initializer_list<Privilege>{Privilege(ResourcePattern::forAnyNormalResource(),
-                                                   {ActionType::enableSharding,
-                                                    ActionType::refineCollectionShardKey,
-                                                    ActionType::reshardCollection})});
+        std::initializer_list<Privilege>{
+            Privilege(ResourcePattern::forAnyNormalResource(boost::none),
+                      {ActionType::enableSharding,
+                       ActionType::refineCollectionShardKey,
+                       ActionType::reshardCollection})});
 
     ASSERT_TRUE(ac1.contains(ac2));
     ASSERT_TRUE(ac2.contains(ac1));
@@ -174,18 +187,19 @@ TEST(AuthContractTest, NonTestModeCheck) {
     enableShardingActions.addAction(ActionType::enableSharding);
     enableShardingActions.addAction(ActionType::refineCollectionShardKey);
     enableShardingActions.addAction(ActionType::reshardCollection);
-    ac.addPrivilege(Privilege(ResourcePattern::forAnyNormalResource(), enableShardingActions));
+    ac.addPrivilege(
+        Privilege(ResourcePattern::forAnyNormalResource(boost::none), enableShardingActions));
 
     // Non-test mode will not keep accounting and will not take any mutex guard
     ASSERT_FALSE(ac.hasAccessCheck(AccessCheckEnum::kIsAuthenticated));
     ASSERT_FALSE(ac.hasAccessCheck(AccessCheckEnum::kIsCoAuthorized));
 
     ASSERT_FALSE(ac.hasPrivileges(
-        Privilege(ResourcePattern::forAnyNormalResource(), ActionType::enableSharding)));
-    ASSERT_FALSE(ac.hasPrivileges(
-        Privilege(ResourcePattern::forAnyNormalResource(), ActionType::refineCollectionShardKey)));
-    ASSERT_FALSE(ac.hasPrivileges(
-        Privilege(ResourcePattern::forAnyNormalResource(), ActionType::reshardCollection)));
+        Privilege(ResourcePattern::forAnyNormalResource(boost::none), ActionType::enableSharding)));
+    ASSERT_FALSE(ac.hasPrivileges(Privilege(ResourcePattern::forAnyNormalResource(boost::none),
+                                            ActionType::refineCollectionShardKey)));
+    ASSERT_FALSE(ac.hasPrivileges(Privilege(ResourcePattern::forAnyNormalResource(boost::none),
+                                            ActionType::reshardCollection)));
 }
 
 }  // namespace
