@@ -28,39 +28,53 @@
  */
 
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/auth/authorization_manager_impl.h"
-
+#include <absl/container/node_hash_map.h>
+#include <algorithm>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional.hpp>
+#include <boost/optional/optional.hpp>
+#include <iterator>
+#include <list>
 #include <memory>
+#include <mutex>
+#include <set>
 #include <string>
+#include <utility>
 #include <vector>
 
-#include "mongo/base/init.h"
-#include "mongo/base/shim.h"
+#include "mongo/base/error_codes.h"
+#include "mongo/base/init.h"  // IWYU pragma: keep
+#include "mongo/base/initializer.h"
 #include "mongo/base/status.h"
-#include "mongo/bson/util/bson_extract.h"
-#include "mongo/config.h"
-#include "mongo/crypto/mechanism_scram.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/config.h"  // IWYU pragma: keep
+#include "mongo/db/auth/action_set.h"
 #include "mongo/db/auth/address_restriction.h"
+#include "mongo/db/auth/auth_name.h"
 #include "mongo/db/auth/auth_types_gen.h"
-#include "mongo/db/auth/authorization_manager_global_parameters_gen.h"
-#include "mongo/db/auth/authorization_manager_impl_parameters_gen.h"
+#include "mongo/db/auth/authorization_manager_impl.h"
+#include "mongo/db/auth/authorization_session.h"
 #include "mongo/db/auth/authorization_session_impl.h"
 #include "mongo/db/auth/authz_manager_external_state.h"
-#include "mongo/db/auth/sasl_options.h"
-#include "mongo/db/auth/user_document_parser.h"
-#include "mongo/db/auth/user_management_commands_parser.h"
+#include "mongo/db/auth/builtin_roles.h"
+#include "mongo/db/auth/privilege.h"
+#include "mongo/db/auth/restriction_set.h"
 #include "mongo/db/commands/authentication_commands.h"
 #include "mongo/db/curop.h"
 #include "mongo/db/global_settings.h"
-#include "mongo/db/mongod_options.h"
+#include "mongo/idl/idl_parser.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/stdx/condition_variable.h"
 #include "mongo/util/assert_util.h"
+#include "mongo/util/duration.h"
 #include "mongo/util/fail_point.h"
-#include "mongo/util/net/ssl_peer_info.h"
-#include "mongo/util/net/ssl_types.h"
+#include "mongo/util/future.h"
 #include "mongo/util/str.h"
+#include "mongo/util/testing_proctor.h"
+#include "mongo/util/time_support.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kAccessControl
 
@@ -75,7 +89,7 @@ std::shared_ptr<UserHandle> createSystemUserHandle() {
     ActionSet allActions;
     allActions.addAllActions();
     PrivilegeVector privileges;
-    auth::generateUniversalPrivileges(&privileges);
+    auth::generateUniversalPrivileges(&privileges, boost::none /* tenantId */);
     (*user)->addPrivileges(privileges);
 
     if (internalSecurity.credentials) {
@@ -151,8 +165,12 @@ bool loggedCommandOperatesOnAuthzData(const NamespaceString& nss, const BSONObj&
     } else if (cmdName == "dropDatabase") {
         return true;
     } else if (cmdName == "renameCollection") {
-        const NamespaceString fromNamespace(cmdObj.firstElement().valueStringDataSafe());
-        const NamespaceString toNamespace(cmdObj.getStringField("to"));
+        auto context = SerializationContext::stateStorageRequest();
+
+        const NamespaceString fromNamespace = NamespaceStringUtil::deserialize(
+            nss.tenantId(), cmdObj.firstElement().valueStringDataSafe(), context);
+        const NamespaceString toNamespace =
+            NamespaceStringUtil::deserialize(nss.tenantId(), cmdObj.getStringField("to"), context);
 
         if (fromNamespace.isAdminDB() || toNamespace.isAdminDB()) {
             return isAuthzCollection(fromNamespace.coll()) || isAuthzCollection(toNamespace.coll());
@@ -190,15 +208,6 @@ bool appliesToAuthzData(StringData op, const NamespaceString& nss, const BSONObj
     }
 }
 
-std::unique_ptr<AuthorizationManager> authorizationManagerCreateImpl(
-    ServiceContext* serviceContext) {
-    return std::make_unique<AuthorizationManagerImpl>(serviceContext,
-                                                      AuthzManagerExternalState::create());
-}
-
-auto authorizationManagerCreateRegistration =
-    MONGO_WEAK_FUNCTION_REGISTRATION(AuthorizationManager::create, authorizationManagerCreateImpl);
-
 MONGO_FAIL_POINT_DEFINE(waitForUserCacheInvalidation);
 void handleWaitForUserCacheInvalidation(OperationContext* opCtx, const UserHandle& user) {
     auto fp = waitForUserCacheInvalidation.scopedIf([&](const auto& bsonData) {
@@ -240,7 +249,7 @@ void handleWaitForUserCacheInvalidation(OperationContext* opCtx, const UserHandl
 int authorizationManagerCacheSize;
 
 AuthorizationManagerImpl::AuthorizationManagerImpl(
-    ServiceContext* service, std::unique_ptr<AuthzManagerExternalState> externalState)
+    Service* service, std::unique_ptr<AuthzManagerExternalState> externalState)
     : _externalState(std::move(externalState)),
       _authSchemaVersionCache(service, _threadPool, _externalState.get()),
       _userCache(service,
@@ -290,6 +299,14 @@ OID AuthorizationManagerImpl::getCacheGeneration() {
 }
 
 void AuthorizationManagerImpl::setAuthEnabled(bool enabled) {
+    if (_authEnabled == enabled) {
+        return;
+    }
+
+    tassert(ErrorCodes::OperationFailed,
+            "Auth may not be disabled once enabled except for unit tests",
+            enabled || TestingProctor::instance().isEnabled());
+
     _authEnabled = enabled;
 }
 
@@ -315,7 +332,10 @@ bool AuthorizationManagerImpl::hasAnyPrivilegeDocuments(OperationContext* opCtx)
 Status AuthorizationManagerImpl::getUserDescription(OperationContext* opCtx,
                                                     const UserName& userName,
                                                     BSONObj* result) {
-    return _externalState->getUserDescription(opCtx, UserRequest(userName, boost::none), result);
+    return _externalState->getUserDescription(opCtx,
+                                              UserRequest(userName, boost::none),
+                                              result,
+                                              CurOp::get(opCtx)->getUserAcquisitionStats());
 }
 
 Status AuthorizationManagerImpl::hasValidAuthSchemaVersionDocumentForInitialSync(
@@ -412,14 +432,16 @@ StatusWith<UserHandle> AuthorizationManagerImpl::acquireUser(OperationContext* o
     // X.509 will give us our roles for initial acquire, but we have to lose them during
     // reacquire (for now) so reparse those roles into the request if not already present.
     if ((request.roles == boost::none) && request.mechanismData.empty() &&
-        (userName.getDB() == DatabaseName::kExternal.db())) {
+        (userName.getDatabaseName().isExternalDB())) {
         userRequest = getX509UserRequest(opCtx, std::move(userRequest));
     }
 #endif
 
+    auto userAcquisitionStats = CurOp::get(opCtx)->getUserAcquisitionStats();
     if (authUserCacheBypass.shouldFail()) {
         // Bypass cache and force a fresh load of the user.
-        auto loadedUser = uassertStatusOK(_externalState->getUserObject(opCtx, request));
+        auto loadedUser =
+            uassertStatusOK(_externalState->getUserObject(opCtx, request, userAcquisitionStats));
         // We have to inject into the cache in order to get a UserHandle.
         auto userHandle =
             _userCache.insertOrAssignAndGet(request, std::move(loadedUser), Date_t::now());
@@ -430,15 +452,16 @@ StatusWith<UserHandle> AuthorizationManagerImpl::acquireUser(OperationContext* o
 
     // Track wait time and user cache access statistics for the current op for logging. An extra
     // second of delay is added via the failpoint for testing.
-    UserAcquisitionStatsHandle userAcquisitionStatsHandle =
-        UserAcquisitionStatsHandle(CurOp::get(opCtx)->getMutableUserAcquisitionStats(),
-                                   opCtx->getServiceContext()->getTickSource(),
-                                   kCache);
+    UserAcquisitionStatsHandle userAcquisitionStatsHandle = UserAcquisitionStatsHandle(
+        userAcquisitionStats.get(), opCtx->getServiceContext()->getTickSource(), kCache);
     if (authUserCacheSleep.shouldFail()) {
         sleepsecs(1);
     }
 
-    auto cachedUser = _userCache.acquire(opCtx, userRequest);
+    auto cachedUser = _userCache.acquire(opCtx,
+                                         userRequest,
+                                         CacheCausalConsistency::kLatestCached,
+                                         CurOp::get(opCtx)->getUserAcquisitionStats());
 
     userAcquisitionStatsHandle.recordTimerEnd();
     invariant(cachedUser);
@@ -485,8 +508,7 @@ void AuthorizationManagerImpl::_updateCacheGeneration() {
     _cacheGeneration = OID::gen();
 }
 
-void AuthorizationManagerImpl::invalidateUserByName(OperationContext* opCtx,
-                                                    const UserName& userName) {
+void AuthorizationManagerImpl::invalidateUserByName(const UserName& userName) {
     LOGV2_DEBUG(20235, 2, "Invalidating user", "user"_attr = userName);
     _authSchemaVersionCache.invalidateAll();
     // There may be multiple entries in the cache with arbitrary other UserRequest data.
@@ -496,8 +518,7 @@ void AuthorizationManagerImpl::invalidateUserByName(OperationContext* opCtx,
     _updateCacheGeneration();
 }
 
-void AuthorizationManagerImpl::invalidateUsersFromDB(OperationContext* opCtx,
-                                                     const DatabaseName& dbname) {
+void AuthorizationManagerImpl::invalidateUsersFromDB(const DatabaseName& dbname) {
     LOGV2_DEBUG(20236, 2, "Invalidating all users from database", "database"_attr = dbname);
     _authSchemaVersionCache.invalidateAll();
     _userCache.invalidateKeyIf([&](const UserRequest& userRequest) {
@@ -506,10 +527,9 @@ void AuthorizationManagerImpl::invalidateUsersFromDB(OperationContext* opCtx,
     _updateCacheGeneration();
 }
 
-void AuthorizationManagerImpl::invalidateUsersByTenant(OperationContext* opCtx,
-                                                       const boost::optional<TenantId>& tenant) {
+void AuthorizationManagerImpl::invalidateUsersByTenant(const boost::optional<TenantId>& tenant) {
     if (!tenant) {
-        invalidateUserCache(opCtx);
+        invalidateUserCache();
         return;
     }
 
@@ -520,7 +540,7 @@ void AuthorizationManagerImpl::invalidateUsersByTenant(OperationContext* opCtx,
     _updateCacheGeneration();
 }
 
-void AuthorizationManagerImpl::invalidateUserCache(OperationContext* opCtx) {
+void AuthorizationManagerImpl::invalidateUserCache() {
     LOGV2_DEBUG(20237, 2, "Invalidating user cache");
     _authSchemaVersionCache.invalidateAll();
     _userCache.invalidateAll();
@@ -532,7 +552,7 @@ Status AuthorizationManagerImpl::refreshExternalUsers(OperationContext* opCtx) {
     // First, get a snapshot of the UserHandles in the cache.
     auto cachedUsers =
         _userCache.peekLatestCachedIf([&](const UserRequest& userRequest, const User&) {
-            return userRequest.name.getDB() == DatabaseName::kExternal.db();
+            return userRequest.name.getDatabaseName().isExternalDB();
         });
 
     // Then, retrieve the corresponding Users from the backing store for users in the $external
@@ -541,7 +561,8 @@ Status AuthorizationManagerImpl::refreshExternalUsers(OperationContext* opCtx) {
     bool isRefreshed{false};
     for (const auto& cachedUser : cachedUsers) {
         UserRequest request(cachedUser->getName(), boost::none);
-        auto storedUserStatus = _externalState->getUserObject(opCtx, request);
+        auto storedUserStatus = _externalState->getUserObject(
+            opCtx, request, CurOp::get(opCtx)->getUserAcquisitionStats());
         if (!storedUserStatus.isOK()) {
             // If the user simply is not found, then just invalidate the cached user and continue.
             if (storedUserStatus.getStatus().code() == ErrorCodes::UserNotFound) {
@@ -573,7 +594,7 @@ Status AuthorizationManagerImpl::initialize(OperationContext* opCtx) {
         return status;
     }
 
-    invalidateUserCache(opCtx);
+    invalidateUserCache();
     return Status::OK();
 }
 
@@ -601,9 +622,7 @@ std::vector<AuthorizationManager::CachedUserInfo> AuthorizationManagerImpl::getU
 }
 
 AuthorizationManagerImpl::AuthSchemaVersionCache::AuthSchemaVersionCache(
-    ServiceContext* service,
-    ThreadPoolInterface& threadPool,
-    AuthzManagerExternalState* externalState)
+    Service* service, ThreadPoolInterface& threadPool, AuthzManagerExternalState* externalState)
     : ReadThroughCache(
           _mutex,
           service,
@@ -627,7 +646,7 @@ AuthorizationManagerImpl::AuthSchemaVersionCache::_lookup(OperationContext* opCt
 }
 
 AuthorizationManagerImpl::UserCacheImpl::UserCacheImpl(
-    ServiceContext* service,
+    Service* service,
     ThreadPoolInterface& threadPool,
     int cacheSize,
     AuthSchemaVersionCache* authSchemaVersionCache,
@@ -636,17 +655,22 @@ AuthorizationManagerImpl::UserCacheImpl::UserCacheImpl(
           _mutex,
           service,
           threadPool,
-          [this](OperationContext* opCtx, const UserRequest& userReq, UserHandle cachedUser) {
-              return _lookup(opCtx, userReq, cachedUser);
+          [this](OperationContext* opCtx,
+                 const UserRequest& userReq,
+                 const UserHandle& cachedUser,
+                 const SharedUserAcquisitionStats& userAcquisitionStats) {
+              return _lookup(opCtx, userReq, cachedUser, userAcquisitionStats);
           },
           cacheSize),
       _authSchemaVersionCache(authSchemaVersionCache),
       _externalState(externalState) {}
 
 AuthorizationManagerImpl::UserCacheImpl::LookupResult
-AuthorizationManagerImpl::UserCacheImpl::_lookup(OperationContext* opCtx,
-                                                 const UserRequest& userReq,
-                                                 const UserHandle& unusedCachedUser) {
+AuthorizationManagerImpl::UserCacheImpl::_lookup(
+    OperationContext* opCtx,
+    const UserRequest& userReq,
+    const UserHandle& unusedCachedUser,
+    const SharedUserAcquisitionStats& userAcquisitionStats) {
     LOGV2_DEBUG(20238, 1, "Getting user record", "user"_attr = userReq.name);
 
     // Number of times to retry a user document that fetches due to transient AuthSchemaIncompatible
@@ -664,7 +688,8 @@ AuthorizationManagerImpl::UserCacheImpl::_lookup(OperationContext* opCtx,
             case schemaVersion28SCRAM:
             case schemaVersion26Final:
             case schemaVersion26Upgrade:
-                return LookupResult(uassertStatusOK(_externalState->getUserObject(opCtx, userReq)));
+                return LookupResult(uassertStatusOK(
+                    _externalState->getUserObject(opCtx, userReq, userAcquisitionStats)));
             case schemaVersion24:
                 _authSchemaVersionCache->invalidateAll();
 
