@@ -27,30 +27,14 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/auth/authorization_session.h"
-
+#include <boost/optional.hpp>
 #include <string>
 #include <vector>
 
+#include <boost/move/utility_core.hpp>
+
 #include "mongo/base/shim.h"
-#include "mongo/base/status.h"
-#include "mongo/db/auth/action_set.h"
-#include "mongo/db/auth/action_type.h"
-#include "mongo/db/auth/authz_session_external_state.h"
-#include "mongo/db/auth/privilege.h"
-#include "mongo/db/auth/restriction_environment.h"
-#include "mongo/db/auth/security_key.h"
-#include "mongo/db/auth/user_management_commands_parser.h"
-#include "mongo/db/bson/dotted_path_support.h"
-#include "mongo/db/catalog/document_validation.h"
-#include "mongo/db/client.h"
-#include "mongo/db/jsobj.h"
-#include "mongo/db/namespace_string.h"
-#include "mongo/db/pipeline/lite_parsed_pipeline.h"
-#include "mongo/util/assert_util.h"
-#include "mongo/util/str.h"
+#include "mongo/db/auth/authorization_session.h"
 
 namespace mongo {
 
@@ -59,14 +43,9 @@ AuthorizationSession::~AuthorizationSession() = default;
 void AuthorizationSession::ScopedImpersonate::swap() {
     using std::swap;
     auto impersonations = _authSession._getImpersonations();
-    swap(*std::get<0>(impersonations), _user);
+    auto originalUser = std::atomic_exchange(std::get<0>(impersonations), std::move(_user));
+    _user = std::move(originalUser);
     swap(*std::get<1>(impersonations), _roles);
 }
 
-std::unique_ptr<AuthorizationSession> AuthorizationSession::create(
-    AuthorizationManager* authzManager) {
-    static auto w = MONGO_WEAK_FUNCTION_DEFINITION(AuthorizationSession::create);
-    return w(authzManager);
-}
-
 }  // namespace mongo
