@@ -29,26 +29,44 @@
 
 #pragma once
 
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
 #include <memory>
 #include <string>
+#include <tuple>
 #include <vector>
 
 #include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
 #include "mongo/db/auth/action_set.h"
 #include "mongo/db/auth/action_type.h"
 #include "mongo/db/auth/authorization_manager.h"
 #include "mongo/db/auth/authz_session_external_state.h"
 #include "mongo/db/auth/privilege.h"
+#include "mongo/db/auth/resource_pattern.h"
+#include "mongo/db/auth/role_name.h"
+#include "mongo/db/auth/user.h"
 #include "mongo/db/auth/user_name.h"
-#include "mongo/db/auth/user_set.h"
+#include "mongo/db/client.h"
+#include "mongo/db/database_name.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/operation_context.h"
+#include "mongo/db/query/explain_verbosity_gen.h"
+#include "mongo/db/read_write_concern_provenance_base_gen.h"
+#include "mongo/db/session/logical_session_id_gen.h"
+#include "mongo/db/tenant_id.h"
+#include "mongo/util/concurrency/with_lock.h"
+#include "mongo/util/time_support.h"
 
 namespace mongo {
 
 class Client;
 class AuthorizationContract;
 
+class ListCollections;
+
 /**
  * Contains all the authorization logic for a single client connection.  It contains a set of
  * the users which have been authenticated, as well as a set of privileges that have been
@@ -79,9 +97,9 @@ public:
     class ScopedImpersonate {
     public:
         ScopedImpersonate(AuthorizationSession* authSession,
-                          std::vector<UserName>* users,
+                          std::shared_ptr<UserName>* user,
                           std::vector<RoleName>* roles)
-            : _authSession(*authSession), _users(*users), _roles(*roles) {
+            : _authSession(*authSession), _user(*user), _roles(*roles) {
             swap();
         }
 
@@ -93,7 +111,7 @@ public:
         void swap();
 
         AuthorizationSession& _authSession;
-        std::vector<UserName>& _users;
+        std::shared_ptr<UserName>& _user;
         std::vector<RoleName>& _roles;
     };
 
@@ -145,7 +163,9 @@ public:
      * Adds the User identified by "UserName" to the authorization session, acquiring privileges
      * for it in the process.
      */
-    virtual Status addAndAuthorizeUser(OperationContext* opCtx, const UserName& userName) = 0;
+    virtual Status addAndAuthorizeUser(OperationContext* opCtx,
+                                       const UserRequest& userRequest,
+                                       boost::optional<Date_t> expirationTime) = 0;
 
     // Returns the authenticated user with the given name.  Returns NULL
     // if no such user is found.
@@ -153,9 +173,11 @@ public:
     // and ownership of the user stays with the AuthorizationManager
     virtual User* lookupUser(const UserName& name) = 0;
 
-    // Returns the single user on this auth session. If no user is authenticated, or if
-    // multiple users are authenticated, this method will throw an exception.
-    virtual User* getSingleUser() = 0;
+    // Get the authenticated user's object handle, if any.
+    virtual boost::optional<UserHandle> getAuthenticatedUser() = 0;
+
+    // Get the authenticated user's tenant ID, if any.
+    virtual boost::optional<TenantId> getUserTenantId() const = 0;
 
     // Is auth disabled? Returns true if auth is disabled.
     virtual bool shouldIgnoreAuthChecks() = 0;
@@ -163,8 +185,8 @@ public:
     // Is authenticated as at least one user.
     virtual bool isAuthenticated() = 0;
 
-    // Gets an iterator over the names of all authenticated users stored in this manager.
-    virtual UserNameIterator getAuthenticatedUserNames() = 0;
+    // Gets the name of the currently authenticated user (if any).
+    virtual boost::optional<UserName> getAuthenticatedUserName() = 0;
 
     // Gets an iterator over the roles of all authenticated users stored in this manager.
     virtual RoleNameIterator getAuthenticatedRoleNames() = 0;
@@ -179,12 +201,13 @@ public:
     // Removes any authenticated principals whose authorization credentials came from the given
     // database, and revokes any privileges that were granted via that principal. This function
     // modifies state. Synchronizes with the Client lock.
-    virtual void logoutDatabase(Client* client, StringData dbname, StringData reason) = 0;
+    virtual void logoutDatabase(Client* client, const DatabaseName& dbname, StringData reason) = 0;
 
     // How the active session is authenticated.
     enum class AuthenticationMode {
         kNone,           // Not authenticated.
-        kConnection,     // For the duration of the connection, or until logged out.
+        kConnection,     // For the duration of the connection, or until logged out or
+                         // expiration.
         kSecurityToken,  // By operation scoped security token.
     };
     virtual AuthenticationMode getAuthenticationMode() const = 0;
@@ -198,8 +221,8 @@ public:
     // Checks if the current session is authorized to list the collections in the given
     // database. If it is, return a privilegeVector containing the privileges used to authorize
     // this command.
-    virtual StatusWith<PrivilegeVector> checkAuthorizedToListCollections(StringData dbname,
-                                                                         const BSONObj& cmdObj) = 0;
+    virtual StatusWith<PrivilegeVector> checkAuthorizedToListCollections(
+        const ListCollections&) = 0;
 
     // Checks if this connection is using the localhost bypass
     virtual bool isUsingLocalhostBypass() = 0;
@@ -253,18 +276,27 @@ public:
 
     // Returns true if the current session possesses a privilege which could apply to the
     // database resource, or a specific or arbitrary resource within the database.
-    virtual bool isAuthorizedForAnyActionOnAnyResourceInDB(StringData dbname) = 0;
+    virtual bool isAuthorizedForAnyActionOnAnyResourceInDB(const DatabaseName&) = 0;
 
     // Returns true if the current session possesses a privilege which applies to the resource.
     virtual bool isAuthorizedForAnyActionOnResource(const ResourcePattern& resource) = 0;
 
-    // Replaces the data for users that a system user is impersonating with new data.
-    // The auditing system adds these users and their roles to each audit record in the log.
-    virtual void setImpersonatedUserData(const std::vector<UserName>& usernames,
+    // Returns true if the current session possesses privileges on the cluster resource for the
+    // action(s) specified.
+    virtual bool isAuthorizedForClusterActions(const ActionSet& actionSet,
+                                               const boost::optional<TenantId>& tenantId) = 0;
+    bool isAuthorizedForClusterAction(ActionType action,
+                                      const boost::optional<TenantId>& tenantId) {
+        return isAuthorizedForClusterActions({action}, tenantId);
+    }
+
+    // Replaces the data for the user that a system user is impersonating with new data.
+    // The auditing system adds this user and their roles to each audit record in the log.
+    virtual void setImpersonatedUserData(const UserName& username,
                                          const std::vector<RoleName>& roles) = 0;
 
-    // Gets an iterator over the names of all users that the system user is impersonating.
-    virtual UserNameIterator getImpersonatedUserNames() = 0;
+    // Gets the name of the user, if any, that the system user is impersonating.
+    virtual boost::optional<UserName> getImpersonatedUserName() = 0;
 
     // Gets an iterator over the roles of all users that the system user is impersonating.
     virtual RoleNameIterator getImpersonatedRoleNames() = 0;
@@ -283,10 +315,11 @@ public:
     // in common.
     virtual bool isCoauthorizedWithClient(Client* opClient, WithLock opClientLock) = 0;
 
-    // Returns true if the session and 'userNameIter' share an authenticated user, or if both have
-    // no authenticated users. Impersonated users are not considered as 'authenticated' for the
-    // purpose of this check. This always returns true if auth is not enabled.
-    virtual bool isCoauthorizedWith(UserNameIterator userNameIter) = 0;
+    // Returns true if the specified userName is the currently authenticated user,
+    // or if the session is unauthenticated and `boost::none` is specified.
+    // Impersonated users are not considered as 'authenticated' for the purpose of this check.
+    // This always returns true if auth is not enabled.
+    virtual bool isCoauthorizedWith(const boost::optional<UserName>& userName) = 0;
 
     // Tells whether impersonation is active or not.  This state is set when
     // setImpersonatedUserData is called and cleared when clearImpersonatedUserData is
@@ -307,8 +340,16 @@ public:
     // resource.
     virtual bool mayBypassWriteBlockingMode() const = 0;
 
+    // Returns true if the authorization session is expired. When this returns true,
+    // isAuthenticated() is also expected to return false.
+    virtual bool isExpired() const = 0;
+
+    // When the current authorization will expire.
+    // boost::none indicates a non-expiring session.
+    virtual const boost::optional<Date_t>& getExpiration() const = 0;
+
 protected:
-    virtual std::tuple<std::vector<UserName>*, std::vector<RoleName>*> _getImpersonations() = 0;
+    virtual std::tuple<std::shared_ptr<UserName>*, std::vector<RoleName>*> _getImpersonations() = 0;
 };
 
 // Returns a status encoding whether the current session in the specified `opCtx` has privilege to
