@@ -27,24 +27,22 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/auth/authz_manager_external_state_d.h"
-
 #include <memory>
+#include <string>
+#include <utility>
 
+#include "mongo/base/error_codes.h"
 #include "mongo/base/shim.h"
 #include "mongo/base/status.h"
+#include "mongo/db/auth/authz_manager_external_state_d.h"
+#include "mongo/db/auth/authz_session_external_state.h"
 #include "mongo/db/auth/authz_session_external_state_d.h"
-#include "mongo/db/auth/user_name.h"
-#include "mongo/db/client.h"
 #include "mongo/db/db_raii.h"
 #include "mongo/db/dbdirectclient.h"
 #include "mongo/db/dbhelpers.h"
-#include "mongo/db/jsobj.h"
 #include "mongo/db/operation_context.h"
-#include "mongo/db/service_context.h"
-#include "mongo/db/storage/storage_engine.h"
+#include "mongo/db/query/find_command.h"
+#include "mongo/db/record_id.h"
 #include "mongo/util/assert_util.h"
 #include "mongo/util/str.h"
 
@@ -68,7 +66,7 @@ Status AuthzManagerExternalStateMongod::query(
         FindCommandRequest findRequest{collectionName};
         findRequest.setFilter(filter);
         findRequest.setProjection(projection);
-        client.find(std::move(findRequest), ReadPreferenceSetting{}, resultProcessor);
+        client.find(std::move(findRequest), resultProcessor);
         return Status::OK();
     } catch (const DBException& e) {
         return e.toStatus();
@@ -76,37 +74,26 @@ Status AuthzManagerExternalStateMongod::query(
 }
 
 Status AuthzManagerExternalStateMongod::findOne(OperationContext* opCtx,
-                                                const NamespaceString& collectionName,
+                                                const NamespaceString& nss,
                                                 const BSONObj& query,
                                                 BSONObj* result) {
-    AutoGetCollectionForReadCommandMaybeLockFree ctx(opCtx, collectionName);
+    AutoGetCollectionForReadCommandMaybeLockFree ctx(opCtx, nss);
 
     BSONObj found;
     if (Helpers::findOne(opCtx, ctx.getCollection(), query, found)) {
         *result = found.getOwned();
         return Status::OK();
     }
-    return Status(ErrorCodes::NoMatchingDocument,
-                  str::stream() << "No document in " << collectionName.ns() << " matches "
-                                << query);
+    return {ErrorCodes::NoMatchingDocument,
+            str::stream() << "No document in " << nss.toStringForErrorMsg() << " matches "
+                          << query};
 }
 
 bool AuthzManagerExternalStateMongod::hasOne(OperationContext* opCtx,
-                                             const NamespaceString& collectionName,
+                                             const NamespaceString& nss,
                                              const BSONObj& query) {
-    AutoGetCollectionForReadCommandMaybeLockFree ctx(opCtx, collectionName);
+    AutoGetCollectionForReadCommandMaybeLockFree ctx(opCtx, nss);
     return !Helpers::findOne(opCtx, ctx.getCollection(), query).isNull();
 }
 
-namespace {
-
-std::unique_ptr<AuthzManagerExternalState> authzManagerExternalStateCreateImpl() {
-    return std::make_unique<AuthzManagerExternalStateMongod>();
-}
-
-auto authzManagerExternalStateCreateRegistration = MONGO_WEAK_FUNCTION_REGISTRATION(
-    AuthzManagerExternalState::create, authzManagerExternalStateCreateImpl);
-
-}  // namespace
-
 }  // namespace mongo
