@@ -28,33 +28,53 @@
  */
 
 
-#include "mongo/db/storage/recovery_unit.h"
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/auth/authz_manager_external_state_local.h"
+#include <absl/container/node_hash_set.h>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional.hpp>
+#include <boost/optional/optional.hpp>
+#include <fmt/format.h>
+#include <set>
+#include <string>
+#include <type_traits>
+#include <utility>
 
+#include "mongo/base/error_codes.h"
 #include "mongo/base/status.h"
-#include "mongo/bson/util/bson_extract.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
 #include "mongo/db/auth/address_restriction.h"
-#include "mongo/db/auth/auth_options_gen.h"
+#include "mongo/db/auth/auth_name.h"
 #include "mongo/db/auth/auth_types_gen.h"
-#include "mongo/db/auth/privilege_parser.h"
+#include "mongo/db/auth/authz_manager_external_state_local.h"
+#include "mongo/db/auth/builtin_roles.h"
+#include "mongo/db/auth/parsed_privilege_gen.h"
+#include "mongo/db/auth/privilege.h"
+#include "mongo/db/auth/restriction_set.h"
 #include "mongo/db/auth/user_document_parser.h"
-#include "mongo/db/multitenancy.h"
+#include "mongo/db/auth/user_name.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
 #include "mongo/db/operation_context.h"
-#include "mongo/db/server_options.h"
-#include "mongo/db/storage/snapshot_manager.h"
+#include "mongo/db/storage/storage_options.h"
 #include "mongo/db/tenant_id.h"
+#include "mongo/idl/idl_parser.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/stdx/unordered_set.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/duration.h"
 #include "mongo/util/fail_point.h"
-#include "mongo/util/net/ssl_types.h"
 #include "mongo/util/str.h"
+#include "mongo/util/time_support.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kAccessControl
 
 
 namespace mongo {
+using namespace fmt::literals;
 
 using std::vector;
 using ResolveRoleOption = AuthzManagerExternalStateLocal::ResolveRoleOption;
@@ -228,15 +248,15 @@ ResolveRoleOption makeResolveRoleOption(PrivilegeFormat showPrivileges,
     return option;
 }
 
-MONGO_FAIL_POINT_DEFINE(authLocalGetUser);
-void handleAuthLocalGetUserFailPoint(const std::vector<RoleName>& directRoles) {
-    auto sfp = authLocalGetUser.scoped();
+MONGO_FAIL_POINT_DEFINE(authLocalGetSubRoles);
+void handleAuthLocalGetSubRolesFailPoint(const std::vector<RoleName>& directRoles) {
+    auto sfp = authLocalGetSubRoles.scoped();
     if (!sfp.isActive()) {
         return;
     }
 
-    IDLParserContext ctx("authLocalGetUser");
-    auto delay = AuthLocalGetUserFailPoint::parse(ctx, sfp.getData()).getResolveRolesDelayMS();
+    IDLParserContext ctx("authLocalGetSubRoles");
+    auto delay = AuthLocalGetSubRolesFailPoint::parse(ctx, sfp.getData()).getResolveRolesDelayMS();
 
     if (delay <= 0) {
         return;
@@ -307,8 +327,10 @@ AuthzManagerExternalStateLocal::RolesLocks AuthzManagerExternalStateLocal::_lock
     return AuthzManagerExternalStateLocal::RolesLocks(opCtx, tenant);
 }
 
-StatusWith<User> AuthzManagerExternalStateLocal::getUserObject(OperationContext* opCtx,
-                                                               const UserRequest& userReq) try {
+StatusWith<User> AuthzManagerExternalStateLocal::getUserObject(
+    OperationContext* opCtx,
+    const UserRequest& userReq,
+    const SharedUserAcquisitionStats& userAcquisitionStats) try {
     const UserName& userName = userReq.name;
     std::vector<RoleName> directRoles;
     User user(userReq);
@@ -362,7 +384,7 @@ StatusWith<User> AuthzManagerExternalStateLocal::getUserObject(OperationContext*
         }
     }
 
-    handleAuthLocalGetUserFailPoint(directRoles);
+    handleAuthLocalGetSubRolesFailPoint(directRoles);
 
     auto data = uassertStatusOK(resolveRoles(opCtx, directRoles, options));
     data.roles->insert(directRoles.cbegin(), directRoles.cend());
@@ -381,9 +403,11 @@ StatusWith<User> AuthzManagerExternalStateLocal::getUserObject(OperationContext*
     return ex.toStatus();
 }
 
-Status AuthzManagerExternalStateLocal::getUserDescription(OperationContext* opCtx,
-                                                          const UserRequest& userReq,
-                                                          BSONObj* result) try {
+Status AuthzManagerExternalStateLocal::getUserDescription(
+    OperationContext* opCtx,
+    const UserRequest& userReq,
+    BSONObj* result,
+    const SharedUserAcquisitionStats& userAcquisitionStats) try {
     const UserName& userName = userReq.name;
     std::vector<RoleName> directRoles;
     BSONObjBuilder resultBuilder;
@@ -436,7 +460,7 @@ Status AuthzManagerExternalStateLocal::getUserDescription(OperationContext* opCt
         }
     }
 
-    handleAuthLocalGetUserFailPoint(directRoles);
+    handleAuthLocalGetSubRolesFailPoint(directRoles);
 
     auto data = uassertStatusOK(resolveRoles(opCtx, directRoles, options));
     data.roles->insert(directRoles.cbegin(), directRoles.cend());
@@ -476,6 +500,7 @@ StatusWith<ResolvedRoleData> AuthzManagerExternalStateLocal::resolveRoles(
     const bool processRests = option.shouldMineRestrictions();
     const bool walkIndirect = !option.shouldMineDirectOnly();
     const bool skipUnknownRolesLog = option.shouldIgnoreUnknown();
+    IDLParserContext idlctx("resolveRoles");
 
     RoleNameSet inheritedRoles;
     PrivilegeVector inheritedPrivileges;
@@ -538,8 +563,15 @@ StatusWith<ResolvedRoleData> AuthzManagerExternalStateLocal::resolveRoles(
                                 << "Invalid 'privileges' field in role document '" << role << "'"};
                 }
                 for (const auto& privElem : elem.Obj()) {
-                    auto priv = Privilege::fromBSON(privElem);
-                    Privilege::addPrivilegeToPrivilegeVector(&inheritedPrivileges, priv);
+                    if (privElem.type() != Object) {
+                        return {ErrorCodes::UnsupportedFormat,
+                                "Expected privilege document as object, got {}"_format(
+                                    typeName(privElem.type()))};
+                    }
+                    auto pp = auth::ParsedPrivilege::parse(idlctx, privElem.Obj());
+                    Privilege::addPrivilegeToPrivilegeVector(
+                        &inheritedPrivileges,
+                        Privilege::resolvePrivilegeWithTenant(role.getTenant(), pp));
                 }
             }
 
@@ -722,7 +754,8 @@ Status AuthzManagerExternalStateLocal::getRoleDescriptionsForDB(
 
     return query(opCtx,
                  getRolesCollection(dbname.tenantId()),
-                 BSON(AuthorizationManager::ROLE_DB_FIELD_NAME << dbname.db()),
+                 BSON(AuthorizationManager::ROLE_DB_FIELD_NAME
+                      << dbname.serializeWithoutTenantPrefix_UNSAFE()),
                  BSONObj(),
                  [&](const BSONObj& roleDoc) {
                      try {
@@ -770,7 +803,7 @@ public:
         // invalidators will purge cache on a per-tenant basis as needed.
         auto db = nss.dbName();
         auto coll = nss.coll();
-        if (db.db() != DatabaseName::kAdmin.db()) {
+        if (!db.isAdminDB()) {
             return;
         }
 
@@ -823,7 +856,7 @@ constexpr auto kOpInsert = "i"_sd;
 constexpr auto kOpUpdate = "u"_sd;
 constexpr auto kOpDelete = "d"_sd;
 
-using InvalidateFn = std::function<void(OperationContext*, AuthorizationManager*)>;
+using InvalidateFn = std::function<void(AuthorizationManager*)>;
 
 /**
  * When we are currently in a WriteUnitOfWork, invalidation of the user cache must wait until
@@ -832,7 +865,7 @@ using InvalidateFn = std::function<void(OperationContext*, AuthorizationManager*
  * populated with stale data until the next invalidation.
  */
 void invalidateUserCacheOnCommit(OperationContext* opCtx, InvalidateFn invalidate) {
-    auto unit = opCtx->recoveryUnit();
+    auto unit = shard_role_details::getRecoveryUnit(opCtx);
     if (unit) {
         auto state = unit->getState();
         if (state == RecoveryUnit::State::kInactiveInUnitOfWork ||
@@ -844,13 +877,13 @@ void invalidateUserCacheOnCommit(OperationContext* opCtx, InvalidateFn invalidat
             unit->onCommit([invalidate = std::move(invalidate)](OperationContext* opCtx,
                                                                 boost::optional<Timestamp>) {
                 LOGV2_DEBUG(9349701, 3, "Invalidating user cache in onCommit handler");
-                invalidate(opCtx, AuthorizationManager::get(opCtx->getServiceContext()));
+                invalidate(AuthorizationManager::get(opCtx->getService()));
             });
             return;
         }
     }
     LOGV2_DEBUG(9349702, 3, "Not in WriteUnitOfWork, invalidating user cache immediately");
-    invalidate(opCtx, AuthorizationManager::get(opCtx->getServiceContext()));
+    invalidate(AuthorizationManager::get(opCtx->getService()));
 }
 
 void _invalidateUserCache(OperationContext* opCtx,
@@ -874,21 +907,19 @@ void _invalidateUserCache(OperationContext* opCtx,
                                                       "the form <dbname>.<username>.  Found: "
                                                    << id));
 
-            invalidateUserCacheOnCommit(
-                opCtx, [](auto* opCtx, auto* am) { am->invalidateUserCache(opCtx); });
+            invalidateUserCacheOnCommit(opCtx, [](auto* am) { am->invalidateUserCache(); });
             return;
         }
         UserName userName(id.substr(splitPoint + 1), id.substr(0, splitPoint), coll.getTenant());
-        invalidateUserCacheOnCommit(opCtx, [userName = std::move(userName)](auto* opCtx, auto* am) {
-            am->invalidateUserByName(opCtx, userName);
+        invalidateUserCacheOnCommit(opCtx, [userName = std::move(userName)](auto* am) {
+            am->invalidateUserByName(userName);
         });
     } else if (const auto& tenant = coll.getTenant()) {
-        invalidateUserCacheOnCommit(opCtx, [tenantId = tenant.value()](auto* opCtx, auto* am) {
-            am->invalidateUsersByTenant(opCtx, tenantId);
+        invalidateUserCacheOnCommit(opCtx, [tenantId = tenant.value()](auto* am) {
+            am->invalidateUsersByTenant(tenantId);
         });
     } else {
-        invalidateUserCacheOnCommit(opCtx,
-                                    [](auto* opCtx, auto* am) { am->invalidateUserCache(opCtx); });
+        invalidateUserCacheOnCommit(opCtx, [](auto* am) { am->invalidateUserCache(); });
     }
 }
 }  // namespace
