@@ -27,23 +27,36 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/auth/authz_manager_external_state_s.h"
-
+#include <absl/container/node_hash_set.h>
+#include <boost/move/utility_core.hpp>
+#include <boost/optional.hpp>
+#include <cstddef>
+#include <set>
 #include <string>
+#include <utility>
 #include <vector>
 
+#include <boost/optional/optional.hpp>
+
 #include "mongo/base/shim.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/db/auth/auth_name.h"
+#include "mongo/db/auth/authz_manager_external_state_s.h"
+#include "mongo/db/auth/authz_session_external_state.h"
 #include "mongo/db/auth/authz_session_external_state_s.h"
 #include "mongo/db/auth/user_document_parser.h"
-#include "mongo/db/auth/user_management_commands_parser.h"
 #include "mongo/db/auth/user_name.h"
 #include "mongo/db/multitenancy.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/rpc/get_status_from_command_result.h"
+#include "mongo/s/catalog/sharding_catalog_client.h"
 #include "mongo/s/grid.h"
-#include "mongo/util/net/ssl_types.h"
+#include "mongo/stdx/unordered_set.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/str.h"
 
 namespace mongo {
@@ -93,7 +106,7 @@ Status AuthzManagerExternalStateMongos::getStoredAuthorizationVersion(OperationC
     BSONObj getParameterCmd = BSON("getParameter" << 1 << "authSchemaVersion" << 1);
     BSONObjBuilder builder;
     const bool ok = Grid::get(opCtx)->catalogClient()->runUserManagementReadCommand(
-        opCtx, "admin", getParameterCmd, &builder);
+        opCtx, DatabaseName::kAdmin, getParameterCmd, &builder);
     BSONObj cmdResult = builder.obj();
     if (!ok) {
         return getStatusFromCommandResult(cmdResult);
@@ -108,17 +121,19 @@ Status AuthzManagerExternalStateMongos::getStoredAuthorizationVersion(OperationC
     return Status::OK();
 }
 
-StatusWith<User> AuthzManagerExternalStateMongos::getUserObject(OperationContext* opCtx,
-                                                                const UserRequest& userReq) {
+StatusWith<User> AuthzManagerExternalStateMongos::getUserObject(
+    OperationContext* opCtx,
+    const UserRequest& userReq,
+    const SharedUserAcquisitionStats& userAcquisitionStats) {
     // Marshalling to BSON and back is inevitable since the
     // source of truth is a system external to mongos.
     BSONObj userDoc;
-    auto status = getUserDescription(opCtx, userReq, &userDoc);
+    auto status = getUserDescription(opCtx, userReq, &userDoc, userAcquisitionStats);
     if (!status.isOK()) {
         return status;
     }
 
-    User user(userReq.name);
+    User user(userReq);
     V2UserDocumentParser dp;
     dp.setTenantId(getActiveTenant(opCtx));
     status = dp.initializeUserFromUserDocument(userDoc, &user);
@@ -129,9 +144,11 @@ StatusWith<User> AuthzManagerExternalStateMongos::getUserObject(OperationContext
     return std::move(user);
 }
 
-Status AuthzManagerExternalStateMongos::getUserDescription(OperationContext* opCtx,
-                                                           const UserRequest& user,
-                                                           BSONObj* result) {
+Status AuthzManagerExternalStateMongos::getUserDescription(
+    OperationContext* opCtx,
+    const UserRequest& user,
+    BSONObj* result,
+    const SharedUserAcquisitionStats& userAcquisitionStats) {
     const UserName& userName = user.name;
     if (!user.roles) {
         BSONObj usersInfoCmd = BSON("usersInfo" << userName.toBSON(true /* serialize tenant */)
@@ -140,7 +157,7 @@ Status AuthzManagerExternalStateMongos::getUserDescription(OperationContext* opC
                                                 << "showCustomData" << false);
         BSONObjBuilder builder;
         const bool ok = Grid::get(opCtx)->catalogClient()->runUserManagementReadCommand(
-            opCtx, "admin", usersInfoCmd, &builder);
+            opCtx, DatabaseName::kAdmin, usersInfoCmd, &builder);
         BSONObj cmdResult = builder.obj();
         if (!ok) {
             return getStatusFromCommandResult(cmdResult);
@@ -176,7 +193,7 @@ Status AuthzManagerExternalStateMongos::getUserDescription(OperationContext* opC
 
         BSONObjBuilder cmdResultBuilder;
         const bool cmdOk = Grid::get(opCtx)->catalogClient()->runUserManagementReadCommand(
-            opCtx, "admin", rolesInfoCmd, &cmdResultBuilder);
+            opCtx, DatabaseName::kAdmin, rolesInfoCmd, &cmdResultBuilder);
         BSONObj cmdResult = cmdResultBuilder.obj();
         if (!cmdOk || !cmdResult["userFragment"].ok()) {
             return Status(ErrorCodes::FailedToParse,
@@ -193,7 +210,7 @@ Status AuthzManagerExternalStateMongos::getUserDescription(OperationContext* opC
             !userInheritedPrivileges.isABSONObj()) {
             return Status(
                 ErrorCodes::UserDataInconsistent,
-                "Recieved malformed response to request for X509 roles from config server");
+                "Received malformed response to request for X509 roles from config server");
         }
 
         *result =
@@ -224,7 +241,7 @@ Status AuthzManagerExternalStateMongos::rolesExist(OperationContext* opCtx,
 
     BSONObjBuilder resultBuilder;
     if (!Grid::get(opCtx)->catalogClient()->runUserManagementReadCommand(
-            opCtx, "admin", rolesInfoCmd.obj(), &resultBuilder)) {
+            opCtx, DatabaseName::kAdmin, rolesInfoCmd.obj(), &resultBuilder)) {
         return {ErrorCodes::OperationFailed, "Failed running rolesInfo command on mongod"};
     }
 
@@ -261,7 +278,7 @@ bool AuthzManagerExternalStateMongos::hasAnyPrivilegeDocuments(OperationContext*
     BSONObj usersInfoCmd = BSON("usersInfo" << 1);
     BSONObjBuilder userBuilder;
     bool ok = Grid::get(opCtx)->catalogClient()->runUserManagementReadCommand(
-        opCtx, "admin", usersInfoCmd, &userBuilder);
+        opCtx, DatabaseName::kAdmin, usersInfoCmd, &userBuilder);
     if (!ok) {
         // If we were unable to complete the query,
         // it's best to assume that there _are_ privilege documents.  This might happen
@@ -279,7 +296,7 @@ bool AuthzManagerExternalStateMongos::hasAnyPrivilegeDocuments(OperationContext*
     BSONObj rolesInfoCmd = BSON("rolesInfo" << 1);
     BSONObjBuilder roleBuilder;
     ok = Grid::get(opCtx)->catalogClient()->runUserManagementReadCommand(
-        opCtx, "admin", rolesInfoCmd, &roleBuilder);
+        opCtx, DatabaseName::kAdmin, rolesInfoCmd, &roleBuilder);
     if (!ok) {
         return true;
     }
