@@ -28,17 +28,13 @@
  */
 
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/auth/authz_session_external_state_server_common.h"
-
 #include <mutex>
 
-#include "mongo/base/status.h"
+#include "mongo/db/auth/authz_session_external_state_server_common.h"
 #include "mongo/db/auth/enable_localhost_auth_bypass_parameter_gen.h"
 #include "mongo/db/client.h"
 #include "mongo/logv2/log.h"
-#include "mongo/util/debug_util.h"
+#include "mongo/logv2/log_component.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kAccessControl
 
