@@ -28,11 +28,13 @@
  */
 
 
-#include "mongo/platform/basic.h"
+#include <boost/move/utility_core.hpp>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
 #include "mongo/db/auth/cluster_auth_mode.h"
-
-#include "mongo/logv2/log.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/str.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kAccessControl
 
@@ -195,4 +197,8 @@ StringData ClusterAuthMode::toString() const {
     MONGO_UNREACHABLE;
 }
 
+bool ClusterAuthMode::equals(ClusterAuthMode& rhs) const {
+    return _value == rhs._value;
+}
+
 }  // namespace mongo
