@@ -27,23 +27,34 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
+#include <cstddef>
+#include <cstdint>
+#include <stdexcept>
 #include <string>
+#include <vector>
 
-#include "mongo/db/auth/sasl_plain_server_conversation.h"
+#include <boost/move/utility_core.hpp>
 
-#include "mongo/base/init.h"
+#include "mongo/base/error_codes.h"
+#include "mongo/base/init.h"  // IWYU pragma: keep
+#include "mongo/base/secure_allocator.h"
 #include "mongo/base/status.h"
 #include "mongo/base/string_data.h"
 #include "mongo/crypto/mechanism_scram.h"
+#include "mongo/db/auth/authorization_manager.h"
 #include "mongo/db/auth/sasl_mechanism_registry.h"
+#include "mongo/db/auth/sasl_plain_server_conversation.h"
 #include "mongo/db/auth/user.h"
 #include "mongo/db/connection_health_metrics_parameter_gen.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
 #include "mongo/util/base64.h"
+#include "mongo/util/duration.h"
 #include "mongo/util/password_digest.h"
-#include "mongo/util/text.h"
+#include "mongo/util/read_through_cache.h"
+#include "mongo/util/str.h"
+#include "mongo/util/text.h"  // IWYU pragma: keep
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kAccessControl
 
@@ -82,7 +93,7 @@ StatusWith<std::tuple<bool, std::string>> SASLPlainServerMechanism::stepImpl(
                       "PLAIN mechanism must be used with internal users");
     }
 
-    AuthorizationManager* authManager = AuthorizationManager::get(opCtx->getServiceContext());
+    AuthorizationManager* authManager = AuthorizationManager::get(opCtx->getService());
 
     // Expecting user input on the form: [authz-id]\0authn-id\0pwd
     std::string input = inputData.toString();
@@ -130,7 +141,7 @@ StatusWith<std::tuple<bool, std::string>> SASLPlainServerMechanism::stepImpl(
 
     // The authentication database is also the source database for the user.
     auto swUser = [&]() {
-        if (gEnableDetailedConnectionHealthMetricLogLines) {
+        if (gEnableDetailedConnectionHealthMetricLogLines.load()) {
             ScopedCallbackTimer timer([&](Microseconds elapsed) {
                 LOGV2(6788606,
                       "Auth metrics report",
