@@ -29,19 +29,35 @@
 
 #pragma once
 
+#include <algorithm>
+#include <boost/move/utility_core.hpp>
+#include <boost/optional.hpp>
+#include <boost/optional/optional.hpp>
+#include <cstdint>
+#include <set>
 #include <string>
+#include <tuple>
+#include <utility>
 #include <vector>
 
+#include "mongo/base/status.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/crypto/sha1_block.h"
 #include "mongo/crypto/sha256_block.h"
+#include "mongo/db/auth/action_set.h"
 #include "mongo/db/auth/privilege.h"
 #include "mongo/db/auth/resource_pattern.h"
 #include "mongo/db/auth/restriction_set.h"
 #include "mongo/db/auth/role_name.h"
+#include "mongo/db/auth/user_acquisition_stats.h"
 #include "mongo/db/auth/user_name.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/platform/atomic_word.h"
 #include "mongo/stdx/unordered_map.h"
 #include "mongo/stdx/unordered_set.h"
+#include "mongo/util/assert_util_core.h"
+#include "mongo/util/base64.h"
 #include "mongo/util/read_through_cache.h"
 
 namespace mongo {
@@ -399,7 +415,8 @@ private:
     RestrictionDocuments _indirectRestrictions;
 };
 
-using UserCache = ReadThroughCache<UserRequest, User>;
+using UserCache =
+    ReadThroughCache<UserRequest, User, CacheNotCausallyConsistent, SharedUserAcquisitionStats>;
 using UserHandle = UserCache::ValueHandle;
 
 }  // namespace mongo
