@@ -30,17 +30,34 @@
 
 #include "mongo/db/auth/user_document_parser.h"
 
+#include <algorithm>
+#include <array>
+#include <iterator>
 #include <string>
+#include <vector>
 
-#include "mongo/base/init.h"
+#include "mongo/base/error_codes.h"
+#include "mongo/base/init.h"  // IWYU pragma: keep
 #include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsontypes.h"
 #include "mongo/db/auth/address_restriction.h"
+#include "mongo/db/auth/auth_name.h"
 #include "mongo/db/auth/authorization_manager.h"
-#include "mongo/db/auth/privilege_parser.h"
+#include "mongo/db/auth/parsed_privilege_gen.h"
+#include "mongo/db/auth/privilege.h"
+#include "mongo/db/auth/restriction_set.h"
+#include "mongo/db/auth/role_name.h"
 #include "mongo/db/auth/user.h"
-#include "mongo/db/jsobj.h"
+#include "mongo/db/auth/user_name.h"
 #include "mongo/db/namespace_string.h"
+#include "mongo/idl/idl_parser.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/str.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kAccessControl
@@ -164,7 +181,7 @@ Status V2UserDocumentParser::checkValidUserDocument(const BSONObj& doc) const {
         return _badValue("User document needs 'db' field to be a non-empty string");
     }
     StringData userDBStr = userDBElement.valueStringData();
-    if (!NamespaceString::validDBName(userDBStr, NamespaceString::DollarInDbNameBehavior::Allow) &&
+    if (!DatabaseName::validDBName(userDBStr, DatabaseName::DollarInDbNameBehavior::Allow) &&
         userDBStr != "$external") {
         return _badValue(str::stream()
                          << "'" << userDBStr << "' is not a valid value for the db field.");
@@ -204,11 +221,11 @@ Status V2UserDocumentParser::checkValidUserDocument(const BSONObj& doc) const {
             return Status::OK();
         };
 
-        const auto sha1status = validateScram(SCRAMSHA1_CREDENTIAL_FIELD_NAME);
+        auto sha1status = validateScram(SCRAMSHA1_CREDENTIAL_FIELD_NAME);
         if (!sha1status.isOK() && (sha1status.code() != ErrorCodes::NoSuchKey)) {
             return sha1status;
         }
-        const auto sha256status = validateScram(SCRAMSHA256_CREDENTIAL_FIELD_NAME);
+        auto sha256status = validateScram(SCRAMSHA256_CREDENTIAL_FIELD_NAME);
         if (!sha256status.isOK() && (sha256status.code() != ErrorCodes::NoSuchKey)) {
             return sha256status;
         }
@@ -413,7 +430,7 @@ Status V2UserDocumentParser::initializeUserIndirectRolesFromUserDocument(const B
 }
 
 Status V2UserDocumentParser::initializeUserPrivilegesFromUserDocument(const BSONObj& doc,
-                                                                      User* user) const {
+                                                                      User* user) const try {
     BSONElement privilegesElement = doc[PRIVILEGES_FIELD_NAME];
     if (privilegesElement.eoo())
         return Status::OK();
@@ -422,40 +439,24 @@ Status V2UserDocumentParser::initializeUserPrivilegesFromUserDocument(const BSON
                       "User document 'inheritedPrivileges' element must be Array if present.");
     }
     PrivilegeVector privileges;
-    std::string errmsg;
-    for (BSONObjIterator it(privilegesElement.Obj()); it.more(); it.next()) {
-        if ((*it).type() != Object) {
+
+    for (const auto& element : privilegesElement.Obj()) {
+        if (element.type() != Object) {
             LOGV2_WARNING(23743,
                           "Wrong type of element in inheritedPrivileges array",
                           "user"_attr = user->getName(),
-                          "element"_attr = *it);
-            continue;
-        }
-        Privilege privilege;
-        ParsedPrivilege pp;
-        if (!pp.parseBSON((*it).Obj(), &errmsg)) {
-            LOGV2_WARNING(23744,
-                          "Could not parse privilege element in user document",
-                          "user"_attr = user->getName(),
-                          "error"_attr = errmsg);
+                          "element"_attr = element);
             continue;
         }
+
+        auto pp = auth::ParsedPrivilege::parse(IDLParserContext("userPrivilegeDoc"), element.Obj());
         std::vector<std::string> unrecognizedActions;
-        Status status =
-            ParsedPrivilege::parsedPrivilegeToPrivilege(pp, &privilege, &unrecognizedActions);
-        if (!status.isOK()) {
-            LOGV2_WARNING(23745,
-                          "Could not parse privilege element in user document",
-                          "user"_attr = user->getName(),
-                          "error"_attr = causedBy(status));
-            continue;
-        }
+        auto privilege = Privilege::resolvePrivilegeWithTenant(
+            user->getName().getTenant(), pp, &unrecognizedActions);
         if (unrecognizedActions.size()) {
             std::string unrecognizedActionsString;
             str::joinStringDelim(unrecognizedActions, &unrecognizedActionsString, ',');
             LOGV2_WARNING(23746,
-                          "Encountered unrecognized actions \"{action}\" while "
-                          "parsing user document for {user}",
                           "Encountered unrecognized actions while parsing user document",
                           "action"_attr = unrecognizedActionsString,
                           "user"_attr = user->getName());
@@ -464,6 +465,8 @@ Status V2UserDocumentParser::initializeUserPrivilegesFromUserDocument(const BSON
     }
     user->setPrivileges(privileges);
     return Status::OK();
+} catch (const DBException& ex) {
+    return ex.toStatus();
 }
 
 Status V2UserDocumentParser::initializeUserFromUserDocument(const BSONObj& privDoc,
