@@ -29,7 +29,13 @@
 
 #pragma once
 
+#include <utility>
+
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
 #include "mongo/base/status.h"
+#include "mongo/bson/bsonobj.h"
 #include "mongo/db/auth/user.h"
 #include "mongo/db/tenant_id.h"
 
