@@ -27,12 +27,17 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
+#include <cstddef>
+#include <cstdint>
 #include <set>
+#include <string>
 #include <vector>
 
-#include "mongo/base/simple_string_data_comparator.h"
+// IWYU pragma: no_include "boost/container/detail/flat_tree.hpp"
+#include <boost/container/flat_set.hpp>
+#include <boost/container/vector.hpp>
+
+#include "mongo/base/string_data_comparator.h"
 #include "mongo/bson/bson_depth.h"
 #include "mongo/bson/bsonelement.h"
 #include "mongo/bson/bsonelement_comparator.h"
@@ -40,8 +45,12 @@
 #include "mongo/bson/bsonobj.h"
 #include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/bson/json.h"
+#include "mongo/bson/util/builder.h"
+#include "mongo/bson/util/builder_fwd.h"
 #include "mongo/db/bson/dotted_path_support.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/bson_test_util.h"
+#include "mongo/unittest/framework.h"
 
 namespace mongo {
 namespace {
@@ -131,7 +140,7 @@ void assertBSONElementSetsAreEqual(const std::vector<BSONObj>& expectedObjs,
 
 
     BSONElementComparator eltCmp(BSONElementComparator::FieldNamesMode::kIgnore,
-                                 &SimpleStringDataComparator::kInstance);
+                                 &simpleStringDataComparator);
     for (size_t i = 0; i < expectedElements.size(); ++i) {
         if (eltCmp.evaluate(*expectedIt != *actualIt)) {
             StringBuilder sb;
@@ -189,9 +198,10 @@ TEST(ExtractAllElementsAlongPath, NestedObjectWithScalarValue) {
 TEST(ExtractAllElementsAlongPath, NestedMaxDepthObjectWithScalarValue) {
     BSONObj obj = BSON("a" << 1);
     std::string dotted_path = "a";
+    dotted_path.reserve(1 + BSONDepth::getMaxAllowableDepth() * 2);
     for (uint32_t i = 0; i < BSONDepth::getMaxAllowableDepth(); ++i) {
         obj = BSON("a" << obj);
-        dotted_path = "a." + dotted_path;
+        dotted_path.insert(0, "a.");
     }
 
     BSONElementSet actualElements;
