@@ -26,28 +26,48 @@
  *    exception statement from all source files in the program, then also delete
  *    it in the license file.
  */
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
 
 #define LOGV2_FOR_RECOVERY(ID, DLEVEL, MESSAGE, ...) \
     LOGV2_DEBUG_OPTIONS(ID, DLEVEL, {logv2::LogComponent::kStorageRecovery}, MESSAGE, ##__VA_ARGS__)
 
-#include "mongo/platform/basic.h"
+#include <algorithm>
+#include <memory>
+#include <string>
+#include <utility>
+#include <vector>
 
+#include <absl/container/node_hash_map.h>
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status_with.h"
+#include "mongo/bson/bsonobj.h"
 #include "mongo/db/catalog/catalog_control.h"
-
 #include "mongo/db/catalog/catalog_stats.h"
 #include "mongo/db/catalog/collection.h"
 #include "mongo/db/catalog/collection_catalog.h"
-#include "mongo/db/catalog/database.h"
 #include "mongo/db/catalog/database_holder.h"
-#include "mongo/db/ftdc/ftdc_mongod.h"
+#include "mongo/db/catalog/historical_catalogid_tracker.h"
+#include "mongo/db/database_name.h"
 #include "mongo/db/index_builds_coordinator.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/rebuild_indexes.h"
 #include "mongo/db/repl/oplog.h"
-#include "mongo/db/tenant_database_name.h"
+#include "mongo/db/service_context.h"
+#include "mongo/db/storage/storage_engine.h"
 #include "mongo/db/timeseries/timeseries_extended_range.h"
+#include "mongo/db/transaction_resources.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/stdx/unordered_map.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/scopeguard.h"
+#include "mongo/util/str.h"
+
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
 
 namespace mongo {
 namespace catalog {
@@ -68,37 +88,33 @@ void reopenAllDatabasesAndReloadCollectionCatalog(OperationContext* opCtx,
     boost::optional<BatchedCollectionCatalogWriter> catalogWriter(opCtx);
 
     auto databaseHolder = DatabaseHolder::get(opCtx);
-    std::vector<TenantDatabaseName> databasesToOpen = storageEngine->listDatabases();
-    for (auto&& tenantDbName : databasesToOpen) {
-        LOGV2_FOR_RECOVERY(
-            23992, 1, "openCatalog: dbholder reopening database", "db"_attr = tenantDbName);
-        auto db = databaseHolder->openDb(opCtx, tenantDbName);
-        invariant(db, str::stream() << "failed to reopen database " << tenantDbName.toString());
-        for (auto&& collNss :
-             catalogWriter.get()->getAllCollectionNamesFromDb(opCtx, tenantDbName)) {
+    std::vector<DatabaseName> databasesToOpen = storageEngine->listDatabases();
+    for (auto&& dbName : databasesToOpen) {
+        LOGV2_FOR_RECOVERY(23992, 1, "openCatalog: dbholder reopening database", logAttrs(dbName));
+        auto db = databaseHolder->openDb(opCtx, dbName);
+        invariant(db,
+                  str::stream() << "failed to reopen database " << dbName.toStringForErrorMsg());
+        for (auto&& collNss : catalogWriter.value()->getAllCollectionNamesFromDb(opCtx, dbName)) {
             // Note that the collection name already includes the database component.
-            auto collection = catalogWriter.get()->lookupCollectionByNamespace(opCtx, collNss);
+            auto collection = catalogWriter.value()->lookupCollectionByNamespace(opCtx, collNss);
             invariant(collection,
-                      str::stream()
-                          << "failed to get valid collection pointer for namespace " << collNss);
+                      str::stream() << "failed to get valid collection pointer for namespace "
+                                    << collNss.toStringForErrorMsg());
 
-            if (previousCatalogState.minVisibleTimestampMap.count(collection->uuid()) > 0) {
-                // After rolling back to a stable timestamp T, the minimum visible timestamp for
-                // each collection must be reset to (at least) its value at T. Additionally, there
-                // cannot exist a minimum visible timestamp greater than lastApplied. This allows us
-                // to upper bound what the minimum visible timestamp can be coming out of rollback.
-                //
-                // Because we only save the latest minimum visible timestamp for each collection, we
-                // bound the minimum visible timestamp (where necessary) to the stable timestamp.
-                // The benefit of fine grained tracking is assumed to be low-value compared to the
-                // cost/effort.
-                auto minVisible = std::min(
-                    stableTimestamp,
-                    previousCatalogState.minVisibleTimestampMap.find(collection->uuid())->second);
+            if (auto it = previousCatalogState.minValidTimestampMap.find(collection->uuid());
+                it != previousCatalogState.minValidTimestampMap.end()) {
+                // After rolling back to a stable timestamp T, the minimum valid timestamp for each
+                // collection must be reset to (at least) its value at T. When the min valid
+                // timestamp is clamped to the stable timestamp we may end up with a pessimistic
+                // minimum valid timestamp set where the last DDL operation occured earlier. This is
+                // fine as this is just an optimization when to avoid reading the catalog from WT.
+                auto minValid = std::min(stableTimestamp, it->second);
                 auto writableCollection =
-                    catalogWriter.get()->lookupCollectionByUUIDForMetadataWrite(opCtx,
-                                                                                collection->uuid());
-                writableCollection->setMinimumVisibleSnapshot(minVisible);
+                    catalogWriter.value()->lookupCollectionByUUIDForMetadataWrite(
+                        opCtx, collection->uuid());
+                writableCollection->setMinimumValidSnapshot(minValid);
+                // Update the pointer in the outer scope as we just requested a writable instance
+                collection = writableCollection;
             }
 
             if (collection->getTimeseriesOptions()) {
@@ -109,7 +125,7 @@ void reopenAllDatabasesAndReloadCollectionCatalog(OperationContext* opCtx,
                     extendedRangeSetting = it->second;
                 } else {
                     extendedRangeSetting =
-                        timeseries::collectionMayRequireExtendedRangeSupport(opCtx, collection);
+                        timeseries::collectionMayRequireExtendedRangeSupport(opCtx, *collection);
                 }
 
                 if (extendedRangeSetting) {
@@ -125,7 +141,8 @@ void reopenAllDatabasesAndReloadCollectionCatalog(OperationContext* opCtx,
                 // The oplog collection must be visible when establishing for repl. Finish our
                 // batched catalog write and continue on a new batch afterwards.
                 catalogWriter.reset();
-                collection->establishOplogCollectionForLogging(opCtx);
+
+                repl::establishOplogRecordStoreForLogging(opCtx, collection->getRecordStore());
                 catalogWriter.emplace(opCtx);
             }
         }
@@ -140,33 +157,32 @@ void reopenAllDatabasesAndReloadCollectionCatalog(OperationContext* opCtx,
 }  // namespace
 
 PreviousCatalogState closeCatalog(OperationContext* opCtx) {
-    invariant(opCtx->lockState()->isW());
+    invariant(shard_role_details::getLocker(opCtx)->isW());
 
     IndexBuildsCoordinator::get(opCtx)->assertNoIndexBuildInProgress();
 
     PreviousCatalogState previousCatalogState;
-    std::vector<TenantDatabaseName> allDbs =
+    std::vector<DatabaseName> allDbs =
         opCtx->getServiceContext()->getStorageEngine()->listDatabases();
 
     auto databaseHolder = DatabaseHolder::get(opCtx);
     auto catalog = CollectionCatalog::get(opCtx);
-    for (auto&& tenantDbName : allDbs) {
-        for (auto&& coll : catalog->range(tenantDbName)) {
+    for (auto&& dbName : allDbs) {
+        for (auto&& coll : catalog->range(dbName)) {
             if (!coll) {
                 break;
             }
 
-            boost::optional<Timestamp> minVisible = coll->getMinimumVisibleSnapshot();
-
-            // If there's a minimum visible, invariant there's also a UUID.
-            if (minVisible) {
-                LOGV2_DEBUG(20269,
+            // If there's a minimum valid, invariant there's also a UUID.
+            boost::optional<Timestamp> minValid = coll->getMinimumValidSnapshot();
+            if (minValid) {
+                LOGV2_DEBUG(6825500,
                             1,
-                            "closeCatalog: preserving min visible timestamp.",
-                            "coll_ns"_attr = coll->ns(),
+                            "closeCatalog: preserving min valid timestamp.",
+                            "ns"_attr = coll->ns(),
                             "uuid"_attr = coll->uuid(),
-                            "minVisible"_attr = minVisible);
-                previousCatalogState.minVisibleTimestampMap[coll->uuid()] = *minVisible;
+                            "minValid"_attr = minValid);
+                previousCatalogState.minValidTimestampMap[coll->uuid()] = *minValid;
             }
 
             if (coll->getTimeseriesOptions()) {
@@ -199,7 +215,7 @@ PreviousCatalogState closeCatalog(OperationContext* opCtx) {
 
     // Reset the stats counter for extended range time-series collections. This is maintained
     // outside the catalog itself.
-    catalog_stats::requiresTimeseriesExtendedRangeSupport.store(0);
+    catalog_stats::requiresTimeseriesExtendedRangeSupport.storeRelaxed(0);
 
     reopenOnFailure.dismiss();
     return previousCatalogState;
@@ -208,25 +224,32 @@ PreviousCatalogState closeCatalog(OperationContext* opCtx) {
 void openCatalog(OperationContext* opCtx,
                  const PreviousCatalogState& previousCatalogState,
                  Timestamp stableTimestamp) {
-    invariant(opCtx->lockState()->isW());
+    invariant(shard_role_details::getLocker(opCtx)->isW());
 
     // Load the catalog in the storage engine.
     LOGV2(20273, "openCatalog: loading storage engine catalog");
     auto storageEngine = opCtx->getServiceContext()->getStorageEngine();
+
+    // Remove catalogId mappings for larger timestamp than 'stableTimestamp'.
+    CollectionCatalog::write(opCtx, [stableTimestamp](CollectionCatalog& catalog) {
+        catalog.catalogIdTracker().rollback(stableTimestamp);
+    });
+
     // Ignore orphaned idents because this function is used during rollback and not at
     // startup recovery, when we may try to recover orphaned idents.
-    storageEngine->loadCatalog(opCtx, StorageEngine::LastShutdownState::kClean);
+    storageEngine->loadCatalog(opCtx, stableTimestamp, StorageEngine::LastShutdownState::kClean);
 
     LOGV2(20274, "openCatalog: reconciling catalog and idents");
-    auto reconcileResult = fassert(
-        40688,
-        storageEngine->reconcileCatalogAndIdents(opCtx, StorageEngine::LastShutdownState::kClean));
+    auto reconcileResult =
+        fassert(40688,
+                storageEngine->reconcileCatalogAndIdents(
+                    opCtx, stableTimestamp, StorageEngine::LastShutdownState::kClean));
 
     // Determine which indexes need to be rebuilt. rebuildIndexesOnCollection() requires that all
     // indexes on that collection are done at once, so we use a map to group them together.
-    StringMap<IndexNameObjs> nsToIndexNameObjMap;
+    stdx::unordered_map<NamespaceString, IndexNameObjs> nsToIndexNameObjMap;
     auto catalog = CollectionCatalog::get(opCtx);
-    for (StorageEngine::IndexIdentifier indexIdentifier : reconcileResult.indexesToRebuild) {
+    for (const StorageEngine::IndexIdentifier& indexIdentifier : reconcileResult.indexesToRebuild) {
         auto indexName = indexIdentifier.indexName;
         auto coll = catalog->lookupCollectionByNamespace(opCtx, indexIdentifier.nss);
         auto indexSpecs = getIndexNameObjs(
@@ -234,8 +257,9 @@ void openCatalog(OperationContext* opCtx,
         if (!indexSpecs.isOK() || indexSpecs.getValue().first.empty()) {
             fassert(40689,
                     {ErrorCodes::InternalError,
-                     str::stream() << "failed to get index spec for index " << indexName
-                                   << " in collection " << indexIdentifier.nss});
+                     str::stream()
+                         << "failed to get index spec for index " << indexName << " in collection "
+                         << indexIdentifier.nss.toStringForErrorMsg()});
         }
         auto indexesToRebuild = indexSpecs.getValue();
         invariant(
@@ -247,7 +271,7 @@ void openCatalog(OperationContext* opCtx,
             str::stream() << "expected to find a list containing exactly 1 index spec, but found "
                           << indexesToRebuild.second.size());
 
-        auto& ino = nsToIndexNameObjMap[indexIdentifier.nss.ns()];
+        auto& ino = nsToIndexNameObjMap[indexIdentifier.nss];
         ino.first.emplace_back(std::move(indexesToRebuild.first.back()));
         ino.second.emplace_back(std::move(indexesToRebuild.second.back()));
     }
@@ -256,17 +280,17 @@ void openCatalog(OperationContext* opCtx,
         NamespaceString collNss(entry.first);
 
         auto collection = catalog->lookupCollectionByNamespace(opCtx, collNss);
-        invariant(collection, str::stream() << "couldn't get collection " << collNss.toString());
+        invariant(collection,
+                  str::stream() << "couldn't get collection " << collNss.toStringForErrorMsg());
 
         for (const auto& indexName : entry.second.first) {
             LOGV2(20275,
-                  "openCatalog: rebuilding index: collection: {collNss}, index: {indexName}",
                   "openCatalog: rebuilding index",
-                  "namespace"_attr = collNss.toString(),
+                  logAttrs(collNss),
                   "index"_attr = indexName);
         }
 
-        std::vector<BSONObj> indexSpecs = entry.second.second;
+        const std::vector<BSONObj>& indexSpecs = entry.second.second;
         fassert(40690, rebuildIndexesOnCollection(opCtx, collection, indexSpecs, RepairData::kNo));
     }
 
@@ -283,7 +307,7 @@ void openCatalog(OperationContext* opCtx,
 
 
 void openCatalogAfterStorageChange(OperationContext* opCtx) {
-    invariant(opCtx->lockState()->isW());
+    invariant(shard_role_details::getLocker(opCtx)->isW());
     auto storageEngine = opCtx->getServiceContext()->getStorageEngine();
     reopenAllDatabasesAndReloadCollectionCatalog(opCtx, storageEngine, {}, {});
 }
