@@ -30,11 +30,19 @@
 
 #include "mongo/db/catalog/catalog_stats.h"
 
+#include <memory>
+
+#include <absl/container/flat_hash_set.h>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/db/catalog/collection_catalog.h"
-#include "mongo/db/catalog/database_holder.h"
+#include "mongo/db/catalog/views_for_database.h"
 #include "mongo/db/commands/server_status.h"
-#include "mongo/db/db_raii.h"
-#include "mongo/logv2/log.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/util/assert_util_core.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
 
@@ -46,7 +54,7 @@ AtomicWord<int> requiresTimeseriesExtendedRangeSupport;
 namespace {
 class CatalogStatsSSS : public ServerStatusSection {
 public:
-    CatalogStatsSSS() : ServerStatusSection("catalogStats") {}
+    using ServerStatusSection::ServerStatusSection;
 
     ~CatalogStatsSSS() override = default;
 
@@ -65,7 +73,6 @@ public:
         int timeseriesExtendedRange = 0;
         int csfle = 0;
         int queryableEncryption = 0;
-        int systemProfile = 0;
 
         void toBson(BSONObjBuilder* builder) const {
             builder->append("collections", collections);
@@ -80,7 +87,6 @@ public:
             }
             builder->append("csfle", csfle);
             builder->append("queryableEncryption", queryableEncryption);
-            builder->append("systemProfile", systemProfile);
         }
     };
 
@@ -94,10 +100,9 @@ public:
         stats.capped = catalogStats.userCapped;
         stats.clustered = catalogStats.userClustered;
         stats.internalCollections = catalogStats.internal;
-        stats.timeseriesExtendedRange = requiresTimeseriesExtendedRangeSupport.load();
+        stats.timeseriesExtendedRange = requiresTimeseriesExtendedRangeSupport.loadRelaxed();
         stats.csfle = catalogStats.csfle;
         stats.queryableEncryption = catalogStats.queryableEncryption;
-        stats.systemProfile = catalogStats.systemProfile;
 
         const auto viewCatalogDbNames = catalog->getViewCatalogDbNames(opCtx);
         for (const auto& dbName : viewCatalogDbNames) {
@@ -113,7 +118,7 @@ public:
         stats.toBson(&builder);
         return builder.obj();
     }
-
-} catalogStatsSSS;
+};
+auto catalogStatsSSS = *ServerStatusSectionBuilder<CatalogStatsSSS>("catalogStats").forShard();
 }  // namespace
 }  // namespace mongo::catalog_stats
