@@ -29,8 +29,12 @@
 
 #pragma once
 
+#include <utility>
+
 #include "mongo/db/operation_context.h"
+#include "mongo/db/repl/storage_interface.h"
 #include "mongo/db/repl/storage_interface_impl.h"
+#include "mongo/db/service_context.h"
 #include "mongo/db/service_context_d_test_fixture.h"
 
 namespace mongo {
@@ -54,7 +58,7 @@ protected:
     void tearDown() override;
 
 private:
-    std::unique_ptr<repl::StorageInterface> _storage;
+    repl::StorageInterface* _storage;
     ServiceContext::UniqueOperationContext _opCtx;
 };
 
