@@ -29,39 +29,85 @@
 
 #include "mongo/db/catalog/coll_mod.h"
 
-#include "mongo/db/stats/counters.h"
 #include <boost/optional.hpp>
+#include <cstdint>
+#include <fmt/format.h>
+#include <list>
+#include <memory>
+#include <string>
+#include <utility>
+#include <variant>
+#include <vector>
 
-#include "mongo/db/catalog/clustered_collection_util.h"
+#include <boost/cstdint.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/db/catalog/clustered_collection_options_gen.h"
 #include "mongo/db/catalog/coll_mod_index.h"
+#include "mongo/db/catalog/collection.h"
 #include "mongo/db/catalog/collection_catalog.h"
+#include "mongo/db/catalog/collection_options.h"
+#include "mongo/db/catalog/collection_options_gen.h"
 #include "mongo/db/catalog/collection_uuid_mismatch.h"
-#include "mongo/db/catalog/create_collection.h"
+#include "mongo/db/catalog/database.h"
 #include "mongo/db/catalog/index_catalog.h"
 #include "mongo/db/catalog/index_key_validate.h"
+#include "mongo/db/catalog_raii.h"
 #include "mongo/db/coll_mod_gen.h"
+#include "mongo/db/concurrency/d_concurrency.h"
 #include "mongo/db/concurrency/exception_util.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
 #include "mongo/db/curop_failpoint_helpers.h"
+#include "mongo/db/database_name.h"
 #include "mongo/db/db_raii.h"
+#include "mongo/db/feature_flag.h"
 #include "mongo/db/index/index_descriptor.h"
 #include "mongo/db/index_builds_coordinator.h"
+#include "mongo/db/matcher/expression_parser.h"
 #include "mongo/db/op_observer/op_observer.h"
+#include "mongo/db/pipeline/change_stream_pre_and_post_images_options_gen.h"
 #include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/s/collection_sharding_state.h"
 #include "mongo/db/s/database_sharding_state.h"
+#include "mongo/db/s/scoped_collection_metadata.h"
 #include "mongo/db/s/shard_key_index_util.h"
+#include "mongo/db/server_feature_flags_gen.h"
 #include "mongo/db/server_options.h"
 #include "mongo/db/service_context.h"
+#include "mongo/db/stats/counters.h"
+#include "mongo/db/storage/key_format.h"
+#include "mongo/db/storage/record_store.h"
 #include "mongo/db/storage/recovery_unit.h"
 #include "mongo/db/storage/storage_parameters_gen.h"
+#include "mongo/db/storage/write_unit_of_work.h"
+#include "mongo/db/timeseries/timeseries_gen.h"
 #include "mongo/db/timeseries/timeseries_options.h"
+#include "mongo/db/transaction_resources.h"
 #include "mongo/db/ttl_collection_cache.h"
+#include "mongo/db/views/view.h"
 #include "mongo/db/views/view_catalog_helpers.h"
-#include "mongo/idl/command_generic_argument.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/s/catalog/sharding_catalog_client.h"
+#include "mongo/s/catalog/type_collection.h"
 #include "mongo/s/grid.h"
+#include "mongo/s/shard_key_pattern.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/fail_point.h"
-#include "mongo/util/overloaded_visitor.h"
+#include "mongo/util/namespace_string_util.h"
+#include "mongo/util/overloaded_visitor.h"  // IWYU pragma: keep
+#include "mongo/util/str.h"
+#include "mongo/util/uuid.h"
 #include "mongo/util/version/releases.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
@@ -82,12 +128,14 @@ void assertNoMovePrimaryInProgress(OperationContext* opCtx, NamespaceString cons
         auto collDesc = scopedCss->getCollectionDescription(opCtx);
         collDesc.throwIfReshardingInProgress(nss);
 
-        if (!collDesc.isSharded()) {
+        // Only collections that are not registered in the sharding catalog are affected by
+        // movePrimary
+        if (!collDesc.hasRoutingTable()) {
             if (scopedDss->isMovePrimaryInProgress()) {
                 LOGV2(4945200, "assertNoMovePrimaryInProgress", logAttrs(nss));
 
                 uasserted(ErrorCodes::MovePrimaryInProgress,
-                          "movePrimary is in progress for namespace " + nss.toString());
+                          "movePrimary is in progress for namespace " + nss.toStringForErrorMsg());
             }
         }
     } catch (const DBException& ex) {
@@ -111,6 +159,7 @@ struct ParsedCollModRequest {
     boost::optional<long long> cappedSize;
     boost::optional<long long> cappedMax;
     boost::optional<bool> timeseriesBucketsMayHaveMixedSchemaData;
+    boost::optional<bool> recordIdsReplicated;
 };
 
 Status getNotSupportedOnViewError(StringData fieldName) {
@@ -133,18 +182,28 @@ Status getOnlySupportedOnTimeseriesError(StringData fieldName) {
             str::stream() << "option only supported on a time-series collection: " << fieldName};
 }
 
-boost::optional<ShardKeyPattern> getShardKeyPattern(OperationContext* opCtx,
-                                                    const NamespaceStringOrUUID& nsOrUUID,
-                                                    const CollMod& cmd) {
+boost::optional<ShardKeyPattern> getShardKeyPatternIfSharded(OperationContext* opCtx,
+                                                             const NamespaceStringOrUUID& nsOrUUID,
+                                                             const CollMod& cmd) {
+    if (!Grid::get(opCtx)->isInitialized()) {
+        return boost::none;
+    }
+
     try {
         const NamespaceString nss =
             CollectionCatalog::get(opCtx)->resolveNamespaceStringOrUUID(opCtx, nsOrUUID);
         if (auto catalogClient = Grid::get(opCtx)->catalogClient()) {
-            return ShardKeyPattern(catalogClient->getCollection(opCtx, nss).getKeyPattern());
+            auto coll = catalogClient->getCollection(opCtx, nss);
+            if (coll.getUnsplittable()) {
+                return boost::none;
+            } else {
+                return ShardKeyPattern(coll.getKeyPattern());
+            }
         }
     } catch (ExceptionFor<ErrorCodes::NamespaceNotFound>&) {
         // The collection is unsharded or doesn't exist.
     }
+
     return boost::none;
 }
 
@@ -230,15 +289,6 @@ StatusWith<std::pair<ParsedCollModRequest, BSONObj>> parseCollModRequest(
         }
 
         auto cmrIndex = &parsed.indexRequest;
-
-        if ((cmdIndex.getUnique() || cmdIndex.getPrepareUnique()) &&
-            !feature_flags::gCollModIndexUnique.isEnabled(
-                serverGlobalParams.featureCompatibility.acquireFCVSnapshot())) {
-            return {ErrorCodes::InvalidOptions,
-                    "collMod does not support converting an index to 'unique' or to "
-                    "'prepareUnique' mode in this FCV."};
-        }
-
         if (cmdIndex.getUnique() && cmdIndex.getForceNonUnique()) {
             return {ErrorCodes::InvalidOptions,
                     "collMod does not support 'unique' and 'forceNonUnique' options at the "
@@ -252,10 +302,6 @@ StatusWith<std::pair<ParsedCollModRequest, BSONObj>> parseCollModRequest(
                         "Please refer to the documentation and use the top-level "
                         "'expireAfterSeconds' option instead"};
             }
-            if (coll->isCapped()) {
-                return {ErrorCodes::InvalidOptions,
-                        "TTL indexes are not supported for capped collections."};
-            }
             if (auto status = index_key_validate::validateExpireAfterSeconds(
                     *cmdIndex.getExpireAfterSeconds(),
                     index_key_validate::ValidateExpireAfterSecondsMode::kSecondaryTTLIndex);
@@ -288,7 +334,8 @@ StatusWith<std::pair<ParsedCollModRequest, BSONObj>> parseCollModRequest(
             cmrIndex->idx = coll->getIndexCatalog()->findIndexByName(opCtx, indexName);
             if (!cmrIndex->idx) {
                 return {ErrorCodes::IndexNotFound,
-                        str::stream() << "cannot find index " << indexName << " for ns " << nss};
+                        str::stream() << "cannot find index " << indexName << " for ns "
+                                      << nss.toStringForErrorMsg()};
             }
         } else {
             std::vector<const IndexDescriptor*> indexes;
@@ -304,7 +351,8 @@ StatusWith<std::pair<ParsedCollModRequest, BSONObj>> parseCollModRequest(
                                       << indexes[1]->infoObj()};
             } else if (indexes.empty()) {
                 return {ErrorCodes::IndexNotFound,
-                        str::stream() << "cannot find index " << keyPattern << " for ns " << nss};
+                        str::stream() << "cannot find index " << keyPattern << " for ns "
+                                      << nss.toStringForErrorMsg()};
             }
 
             cmrIndex->idx = indexes[0];
@@ -394,7 +442,7 @@ StatusWith<std::pair<ParsedCollModRequest, BSONObj>> parseCollModRequest(
         }
 
         if (cmdIndex.getPrepareUnique()) {
-            // Check if prepareUnique is being set on a time-series collection.
+            // Check if prepareUnique is being set on a time-series collection index.
             if (isTimeseries) {
                 return {ErrorCodes::InvalidOptions,
                         "cannot set 'prepareUnique' for indexes of a time-series collection."};
@@ -550,7 +598,7 @@ StatusWith<std::pair<ParsedCollModRequest, BSONObj>> parseCollModRequest(
                                   << "' option is only supported on collections clustered by _id"};
         }
 
-        auto status = stdx::visit(
+        auto status = visit(
             OverloadedVisitor{
                 [&oplogEntryBuilder](const std::string& value) -> Status {
                     if (value != "off") {
@@ -597,6 +645,15 @@ StatusWith<std::pair<ParsedCollModRequest, BSONObj>> parseCollModRequest(
                                  *mixedSchema);
     }
 
+    if (auto recordIdsReplicated = cmr.getRecordIdsReplicated()) {
+        if (*recordIdsReplicated) {
+            return {ErrorCodes::InvalidOptions, "Cannot set recordIdsReplicated to true"};
+        }
+
+        parsed.recordIdsReplicated = recordIdsReplicated;
+        oplogEntryBuilder.append(CollMod::kRecordIdsReplicatedFieldName, false);
+    }
+
     if (const auto& dryRun = cmr.getDryRun()) {
         parsed.dryRun = *dryRun;
         // The dry run option should never be included in a collMod oplog entry.
@@ -615,43 +672,44 @@ void _setClusteredExpireAfterSeconds(
     OperationContext* opCtx,
     const CollectionOptions& oldCollOptions,
     Collection* coll,
-    const stdx::variant<std::string, std::int64_t>& clusteredIndexExpireAfterSeconds) {
+    const std::variant<std::string, std::int64_t>& clusteredIndexExpireAfterSeconds) {
     invariant(oldCollOptions.clusteredIndex);
 
     boost::optional<int64_t> oldExpireAfterSeconds = oldCollOptions.expireAfterSeconds;
 
-    stdx::visit(
-        OverloadedVisitor{
-            [&](const std::string& newExpireAfterSeconds) {
-                invariant(newExpireAfterSeconds == "off");
-                if (!oldExpireAfterSeconds) {
-                    // expireAfterSeconds is already disabled on the clustered index.
-                    return;
-                }
+    visit(OverloadedVisitor{
+              [&](const std::string& newExpireAfterSeconds) {
+                  invariant(newExpireAfterSeconds == "off");
+                  if (!oldExpireAfterSeconds) {
+                      // expireAfterSeconds is already disabled on the clustered index.
+                      return;
+                  }
 
-                coll->updateClusteredIndexTTLSetting(opCtx, boost::none);
-            },
-            [&](std::int64_t newExpireAfterSeconds) {
-                if (oldExpireAfterSeconds && *oldExpireAfterSeconds == newExpireAfterSeconds) {
-                    // expireAfterSeconds is already the requested value on the clustered index.
-                    return;
-                }
+                  coll->updateClusteredIndexTTLSetting(opCtx, boost::none);
+              },
+              [&](std::int64_t newExpireAfterSeconds) {
+                  if (oldExpireAfterSeconds && *oldExpireAfterSeconds == newExpireAfterSeconds) {
+                      // expireAfterSeconds is already the requested value on the clustered index.
+                      return;
+                  }
 
-                // If this collection was not previously TTL, inform the TTL monitor when we commit.
-                if (!oldExpireAfterSeconds) {
-                    auto ttlCache = &TTLCollectionCache::get(opCtx->getServiceContext());
-                    opCtx->recoveryUnit()->onCommit(
-                        [ttlCache, uuid = coll->uuid()](OperationContext*,
-                                                        boost::optional<Timestamp>) {
-                            ttlCache->registerTTLInfo(
-                                uuid, TTLCollectionCache::Info{TTLCollectionCache::ClusteredId{}});
-                        });
-                }
+                  // If this collection was not previously TTL, inform the TTL monitor when we
+                  // commit.
+                  if (!oldExpireAfterSeconds) {
+                      auto ttlCache = &TTLCollectionCache::get(opCtx->getServiceContext());
+                      shard_role_details::getRecoveryUnit(opCtx)->onCommit(
+                          [ttlCache, uuid = coll->uuid()](OperationContext*,
+                                                          boost::optional<Timestamp>) {
+                              ttlCache->registerTTLInfo(
+                                  uuid,
+                                  TTLCollectionCache::Info{TTLCollectionCache::ClusteredId{}});
+                          });
+                  }
 
-                invariant(newExpireAfterSeconds >= 0);
-                coll->updateClusteredIndexTTLSetting(opCtx, newExpireAfterSeconds);
-            }},
-        clusteredIndexExpireAfterSeconds);
+                  invariant(newExpireAfterSeconds >= 0);
+                  coll->updateClusteredIndexTTLSetting(opCtx, newExpireAfterSeconds);
+              }},
+          clusteredIndexExpireAfterSeconds);
 }
 
 Status _processCollModDryRunMode(OperationContext* opCtx,
@@ -697,9 +755,10 @@ Status _processCollModDryRunMode(OperationContext* opCtx,
     }
 
     // Throws exception if index contains duplicates.
-    auto violatingRecordsList = scanIndexForDuplicates(opCtx, collection, cmr.indexRequest.idx);
+    auto violatingRecordsList = scanIndexForDuplicates(opCtx, cmr.indexRequest.idx);
     if (!violatingRecordsList.empty()) {
-        uassertStatusOK(buildConvertUniqueErrorStatus(opCtx, collection, violatingRecordsList));
+        uassertStatusOK(
+            buildConvertUniqueErrorStatus(opCtx, collection.get(), violatingRecordsList));
     }
 
     return Status::OK();
@@ -720,7 +779,8 @@ StatusWith<const IndexDescriptor*> _setUpCollModIndexUnique(
     if (!collection) {
         checkCollectionUUIDMismatch(opCtx, nss, CollectionPtr(), cmd.getCollectionUUID());
         return Status(ErrorCodes::NamespaceNotFound,
-                      str::stream() << "ns does not exist for unique index conversion: " << nss);
+                      str::stream() << "ns does not exist for unique index conversion: "
+                                    << nss.toStringForErrorMsg());
     }
 
     // Scan index for duplicates without exclusive access.
@@ -731,7 +791,7 @@ StatusWith<const IndexDescriptor*> _setUpCollModIndexUnique(
     }
     const auto& cmr = statusW.getValue().first;
     auto idx = cmr.indexRequest.idx;
-    auto violatingRecordsList = scanIndexForDuplicates(opCtx, collection, idx);
+    auto violatingRecordsList = scanIndexForDuplicates(opCtx, idx);
 
     CurOpFailpointHelpers::waitWhileFailPointEnabled(
         &hangAfterCollModIndexUniqueFullIndexScan,
@@ -741,7 +801,8 @@ StatusWith<const IndexDescriptor*> _setUpCollModIndexUnique(
         nss);
 
     if (!violatingRecordsList.empty()) {
-        uassertStatusOK(buildConvertUniqueErrorStatus(opCtx, collection, violatingRecordsList));
+        uassertStatusOK(
+            buildConvertUniqueErrorStatus(opCtx, collection.get(), violatingRecordsList));
     }
 
     return idx;
@@ -750,6 +811,7 @@ StatusWith<const IndexDescriptor*> _setUpCollModIndexUnique(
 Status _collModInternal(OperationContext* opCtx,
                         const NamespaceStringOrUUID& nsOrUUID,
                         const CollMod& cmd,
+                        const CollectionAcquisition* acquisition,
                         BSONObjBuilder* result,
                         boost::optional<repl::OplogApplication::Mode> mode) {
     // Get key pattern from the config server if we may need it for parsing checks if on the primary
@@ -758,7 +820,7 @@ Status _collModInternal(OperationContext* opCtx,
     bool mayNeedKeyPatternForParsing =
         cmd.getIndex() && (cmd.getIndex()->getHidden() || cmd.getIndex()->getPrepareUnique());
     if (!mode && mayNeedKeyPatternForParsing) {
-        shardKeyPattern = getShardKeyPattern(opCtx, nsOrUUID, cmd);
+        shardKeyPattern = getShardKeyPatternIfSharded(opCtx, nsOrUUID, cmd);
     }
 
     if (cmd.getDryRun().value_or(false)) {
@@ -784,18 +846,20 @@ Status _collModInternal(OperationContext* opCtx,
             return cmd.getIndex() && cmd.getIndex()->getUnique().value_or(false) && !mode;
         });
 
-    AutoGetCollection coll(
-        opCtx,
-        nsOrUUID,
-        MODE_X,
-        AutoGetCollection::Options{}.viewMode(auto_get_collection::ViewMode::kViewsPermitted));
-    auto nss = coll.getNss();
+    boost::optional<AutoGetCollection> autoget;
+    if (!acquisition) {
+        autoget.emplace(
+            opCtx,
+            nsOrUUID,
+            MODE_X,
+            AutoGetCollection::Options{}.viewMode(auto_get_collection::ViewMode::kViewsPermitted));
+    }
+    auto nss = acquisition ? acquisition->nss() : autoget->getNss();
+    auto& coll = acquisition ? acquisition->getCollectionPtr() : autoget->getCollection();
     auto dbName = nss.dbName();
     Lock::CollectionLock systemViewsLock(
         opCtx, NamespaceString::makeSystemDotViewsNamespace(dbName), MODE_X);
 
-    Database* const db = coll.getDb();
-
     CurOpFailpointHelpers::waitWhileFailPointEnabled(
         &hangAfterDatabaseLock, opCtx, "hangAfterDatabaseLock", []() {}, nss);
 
@@ -812,12 +876,11 @@ Status _collModInternal(OperationContext* opCtx,
     // This can kill all cursors so don't allow running it while a background operation is in
     // progress.
     if (coll) {
-        assertNoMovePrimaryInProgress(opCtx, nss);
         IndexBuildsCoordinator::get(opCtx)->assertNoIndexBuildInProgForCollection(coll->uuid());
     }
 
-    // If db/collection/view does not exist, short circuit and return.
-    if (!db || (!coll && !view)) {
+    // If collection/view does not exist, short circuit and return.
+    if (!coll && !view) {
         if (nss.isTimeseriesBucketsCollection()) {
             // If a sharded time-series collection is dropped, it's possible that a stale mongos
             // sends the request on the buckets namespace instead of the view namespace. Ensure that
@@ -838,10 +901,11 @@ Status _collModInternal(OperationContext* opCtx,
 
     if (userInitiatedWritesAndNotPrimary) {
         return Status(ErrorCodes::NotWritablePrimary,
-                      str::stream() << "Not primary while setting collection options on " << nss);
+                      str::stream() << "Not primary while setting collection options on "
+                                    << nss.toStringForErrorMsg());
     }
 
-    auto statusW = parseCollModRequest(opCtx, nss, coll.getCollection(), cmd, shardKeyPattern);
+    auto statusW = parseCollModRequest(opCtx, nss, coll, cmd, shardKeyPattern);
     if (!statusW.isOK()) {
         return statusW.getStatus();
     }
@@ -862,7 +926,7 @@ Status _collModInternal(OperationContext* opCtx,
         LOGV2(5324200, "CMD: collMod", "cmdObj"_attr = cmd.toBSON(BSONObj()));
     }
 
-    return writeConflictRetry(opCtx, "collMod", nss.ns(), [&] {
+    return writeConflictRetry(opCtx, "collMod", nss, [&] {
         WriteUnitOfWork wunit(opCtx);
 
         // Handle collMod on a view and return early. The CollectionCatalog handles the creation of
@@ -872,7 +936,7 @@ Status _collModInternal(OperationContext* opCtx,
                 view->setPipeline(*cmd.getPipeline());
 
             if (!viewOn.empty())
-                view->setViewOn(NamespaceStringUtil::parseNamespaceFromRequest(dbName, viewOn));
+                view->setViewOn(NamespaceStringUtil::deserialize(dbName, viewOn));
 
             BSONArrayBuilder pipeline;
             for (auto& item : view->pipeline()) {
@@ -899,49 +963,63 @@ Status _collModInternal(OperationContext* opCtx,
 
         const CollectionOptions& oldCollOptions = coll->getCollectionOptions();
 
+        Collection* writableColl = acquisition
+            ? CollectionCatalog::get(opCtx)->lookupCollectionByUUIDForMetadataWrite(opCtx,
+                                                                                    coll->uuid())
+            : autoget->getWritableCollection(opCtx);
+
         if (cmrNew.cappedSize || cmrNew.cappedMax) {
             // If the current capped collection size exceeds the newly set limits, future document
             // inserts will prompt document deletion.
-            uassertStatusOK(coll.getWritableCollection(opCtx)->updateCappedSize(
-                opCtx, cmrNew.cappedSize, cmrNew.cappedMax));
+            uassertStatusOK(
+                writableColl->updateCappedSize(opCtx, cmrNew.cappedSize, cmrNew.cappedMax));
         }
 
         boost::optional<IndexCollModInfo> indexCollModInfo;
 
         // Handle collMod operation type appropriately.
         if (cmd.getExpireAfterSeconds()) {
-            _setClusteredExpireAfterSeconds(opCtx,
-                                            oldCollOptions,
-                                            coll.getWritableCollection(opCtx),
-                                            *cmd.getExpireAfterSeconds());
+            _setClusteredExpireAfterSeconds(
+                opCtx, oldCollOptions, writableColl, *cmd.getExpireAfterSeconds());
         }
 
         if (auto mixedSchema = cmrNew.timeseriesBucketsMayHaveMixedSchemaData) {
-            coll.getWritableCollection(opCtx)->setTimeseriesBucketsMayHaveMixedSchemaData(
-                opCtx, mixedSchema);
+            writableColl->setTimeseriesBucketsMayHaveMixedSchemaData(opCtx, mixedSchema);
+        }
+
+        if (auto recordIdsReplicated = cmrNew.recordIdsReplicated) {
+            // Must be false if present.
+            invariant(
+                !(*recordIdsReplicated),
+                fmt::format(
+                    "Unexpected true value for 'recordIdsReplicated' in collMod options for {}: {}",
+                    nss.toStringForErrorMsg(),
+                    cmd.toBSON({}).toString()));
+
+            writableColl->unsetRecordIdsReplicated(opCtx);
         }
 
         // Handle index modifications.
         processCollModIndexRequest(
-            opCtx, &coll, cmrNew.indexRequest, &indexCollModInfo, result, mode);
+            opCtx, writableColl, cmrNew.indexRequest, &indexCollModInfo, result, mode);
 
         if (cmrNew.collValidator) {
-            coll.getWritableCollection(opCtx)->setValidator(opCtx, *cmrNew.collValidator);
+            writableColl->setValidator(opCtx, *cmrNew.collValidator);
         }
         if (cmrNew.collValidationAction)
-            uassertStatusOKWithContext(coll.getWritableCollection(opCtx)->setValidationAction(
-                                           opCtx, *cmrNew.collValidationAction),
-                                       "Failed to set validationAction");
+            uassertStatusOKWithContext(
+                writableColl->setValidationAction(opCtx, *cmrNew.collValidationAction),
+                "Failed to set validationAction");
         if (cmrNew.collValidationLevel) {
-            uassertStatusOKWithContext(coll.getWritableCollection(opCtx)->setValidationLevel(
-                                           opCtx, *cmrNew.collValidationLevel),
-                                       "Failed to set validationLevel");
+            uassertStatusOKWithContext(
+                writableColl->setValidationLevel(opCtx, *cmrNew.collValidationLevel),
+                "Failed to set validationLevel");
         }
 
         if (cmrNew.changeStreamPreAndPostImagesOptions.has_value() &&
             *cmrNew.changeStreamPreAndPostImagesOptions !=
                 oldCollOptions.changeStreamPreAndPostImagesOptions) {
-            coll.getWritableCollection(opCtx)->setChangeStreamPreAndPostImages(
+            writableColl->setChangeStreamPreAndPostImages(
                 opCtx, *cmrNew.changeStreamPreAndPostImagesOptions);
         }
 
@@ -951,21 +1029,38 @@ Status _collModInternal(OperationContext* opCtx,
             uassertStatusOK(res);
             auto [newOptions, changed] = res.getValue();
             if (changed) {
-                coll.getWritableCollection(opCtx)->setTimeseriesOptions(opCtx, newOptions);
+                writableColl->setTimeseriesOptions(opCtx, newOptions);
+                if (feature_flags::gTSBucketingParametersUnchanged.isEnabled(
+                        serverGlobalParams.featureCompatibility.acquireFCVSnapshot())) {
+                    writableColl->setTimeseriesBucketingParametersChanged(opCtx, true);
+                };
             }
         }
 
+        // We involve an empty collMod command during a setFCV downgrade to clean timeseries
+        // bucketing parameters in the catalog. So if the FCV is in downgrading or downgraded stage,
+        // remove time-series bucketing parameters flag, as nodes older than 7.1 cannot understand
+        // this flag.
+        // (Generic FCV reference): This FCV check should exist across LTS binary versions.
+        // TODO SERVER-80003 remove special version handling when LTS becomes 8.0.
+        if (const auto version =
+                serverGlobalParams.featureCompatibility.acquireFCVSnapshot().getVersion();
+            cmrNew.numModifications == 0 && coll->timeseriesBucketingParametersHaveChanged() &&
+            version == multiversion::GenericFCV::kDowngradingFromLatestToLastLTS) {
+            writableColl->setTimeseriesBucketingParametersChanged(opCtx, boost::none);
+        }
+
         // Fix any invalid index options for indexes belonging to this collection.
         std::vector<std::string> indexesWithInvalidOptions =
-            coll.getWritableCollection(opCtx)->repairInvalidIndexOptions(opCtx);
+            writableColl->repairInvalidIndexOptions(opCtx);
         for (const auto& indexWithInvalidOptions : indexesWithInvalidOptions) {
             const IndexDescriptor* desc =
-                coll->getIndexCatalog()->findIndexByName(opCtx, indexWithInvalidOptions);
+                writableColl->getIndexCatalog()->findIndexByName(opCtx, indexWithInvalidOptions);
             invariant(desc);
 
             // Notify the index catalog that the definition of this index changed.
-            coll.getWritableCollection(opCtx)->getIndexCatalog()->refreshEntry(
-                opCtx, coll.getWritableCollection(opCtx), desc, CreateIndexEntryFlags::kIsReady);
+            writableColl->getIndexCatalog()->refreshEntry(
+                opCtx, writableColl, desc, CreateIndexEntryFlags::kIsReady);
         }
 
         // Only observe non-view collMods, as view operations are observed as operations on the
@@ -1017,8 +1112,9 @@ CollModRequest makeCollModDryRunRequest(const CollModRequest& request) {
 Status processCollModCommand(OperationContext* opCtx,
                              const NamespaceStringOrUUID& nsOrUUID,
                              const CollMod& cmd,
+                             const CollectionAcquisition* acquisition,
                              BSONObjBuilder* result) {
-    return _collModInternal(opCtx, nsOrUUID, cmd, result, boost::none);
+    return _collModInternal(opCtx, nsOrUUID, cmd, acquisition, result, boost::none);
 }
 
 Status processCollModCommandForApplyOps(OperationContext* opCtx,
@@ -1026,7 +1122,7 @@ Status processCollModCommandForApplyOps(OperationContext* opCtx,
                                         const CollMod& cmd,
                                         repl::OplogApplication::Mode mode) {
     BSONObjBuilder resultWeDontCareAbout;
-    return _collModInternal(opCtx, nsOrUUID, cmd, &resultWeDontCareAbout, mode);
+    return _collModInternal(opCtx, nsOrUUID, cmd, nullptr, &resultWeDontCareAbout, mode);
 }
 
 }  // namespace mongo
