@@ -30,18 +30,40 @@
 
 #include "mongo/db/catalog/coll_mod_index.h"
 
+#include <boost/move/utility_core.hpp>
+#include <boost/optional.hpp>
+#include <boost/optional/optional.hpp>
+#include <cstddef>
+#include <string>
+
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/bson/util/builder.h"
 #include "mongo/db/catalog/cannot_convert_index_to_unique_info.h"
+#include "mongo/db/catalog/index_catalog.h"
+#include "mongo/db/catalog/index_catalog_entry.h"
 #include "mongo/db/catalog/index_key_validate.h"
 #include "mongo/db/catalog/throttle_cursor.h"
+#include "mongo/db/catalog/validate_gen.h"
 #include "mongo/db/index/index_access_method.h"
 #include "mongo/db/storage/index_entry_comparison.h"
 #include "mongo/db/storage/key_string.h"
 #include "mongo/db/storage/recovery_unit.h"
+#include "mongo/db/storage/snapshot.h"
+#include "mongo/db/storage/sorted_data_interface.h"
+#include "mongo/db/transaction_resources.h"
 #include "mongo/db/ttl_collection_cache.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/compiler.h"
 #include "mongo/util/assert_util.h"
+#include "mongo/util/duration.h"
 #include "mongo/util/fail_point.h"
-#include "mongo/util/shared_buffer_fragment.h"
+#include "mongo/util/str.h"
+#include "mongo/util/uuid.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kIndex
 
@@ -56,7 +78,7 @@ MONGO_FAIL_POINT_DEFINE(assertAfterIndexUpdate);
  * Adjusts expiration setting on an index.
  */
 void _processCollModIndexRequestExpireAfterSeconds(OperationContext* opCtx,
-                                                   AutoGetCollection* autoColl,
+                                                   Collection* writableColl,
                                                    const IndexDescriptor* idx,
                                                    long long indexExpireAfterSeconds,
                                                    boost::optional<long long>* newExpireSecs,
@@ -66,22 +88,21 @@ void _processCollModIndexRequestExpireAfterSeconds(OperationContext* opCtx,
     // If this collection was not previously TTL, inform the TTL monitor when we commit.
     if (!oldExpireSecsElement) {
         auto ttlCache = &TTLCollectionCache::get(opCtx->getServiceContext());
-        const auto& coll = autoColl->getCollection();
         // Do not refer to 'idx' within this commit handler as it may be be invalidated by
         // IndexCatalog::refreshEntry().
-        opCtx->recoveryUnit()->onCommit([ttlCache,
-                                         uuid = coll->uuid(),
-                                         indexName = idx->indexName()](OperationContext*,
-                                                                       boost::optional<Timestamp>) {
-            // We assume the expireAfterSeconds field is valid, because we've already done
-            // validation of this field.
-            ttlCache->registerTTLInfo(
-                uuid, TTLCollectionCache::Info{indexName, /*isExpireAfterSecondsInvalid=*/false});
-        });
+        shard_role_details::getRecoveryUnit(opCtx)->onCommit(
+            [ttlCache, uuid = writableColl->uuid(), indexName = idx->indexName()](
+                OperationContext*, boost::optional<Timestamp>) {
+                // We assume the expireAfterSeconds field is valid, because we've already done
+                // validation of this field.
+                ttlCache->registerTTLInfo(
+                    uuid,
+                    TTLCollectionCache::Info{
+                        indexName, TTLCollectionCache::Info::ExpireAfterSecondsType::kInt});
+            });
 
         // Change the value of "expireAfterSeconds" on disk.
-        autoColl->getWritableCollection(opCtx)->updateTTLSetting(
-            opCtx, idx->indexName(), indexExpireAfterSeconds);
+        writableColl->updateTTLSetting(opCtx, idx->indexName(), indexExpireAfterSeconds);
         return;
     }
 
@@ -98,17 +119,16 @@ void _processCollModIndexRequestExpireAfterSeconds(OperationContext* opCtx,
         *oldExpireSecs = 0;
 
         // Change the value of "expireAfterSeconds" on disk.
-        autoColl->getWritableCollection(opCtx)->updateTTLSetting(
-            opCtx, idx->indexName(), indexExpireAfterSeconds);
+        writableColl->updateTTLSetting(opCtx, idx->indexName(), indexExpireAfterSeconds);
 
         // Keep the TTL information maintained by the TTLCollectionCache in sync so that we don't
         // try to fix up the TTL index during the next step-up.
         auto ttlCache = &TTLCollectionCache::get(opCtx->getServiceContext());
-        const auto& coll = autoColl->getCollection();
-        opCtx->recoveryUnit()->onCommit(
-            [ttlCache, uuid = coll->uuid(), indexName = idx->indexName()](
+        shard_role_details::getRecoveryUnit(opCtx)->onCommit(
+            [ttlCache, uuid = writableColl->uuid(), indexName = idx->indexName()](
                 OperationContext*, boost::optional<Timestamp>) {
-                ttlCache->unsetTTLIndexExpireAfterSecondsInvalid(uuid, indexName);
+                ttlCache->setTTLIndexExpireAfterSecondsType(
+                    uuid, indexName, TTLCollectionCache::Info::ExpireAfterSecondsType::kInt);
             });
         return;
     }
@@ -118,8 +138,7 @@ void _processCollModIndexRequestExpireAfterSeconds(OperationContext* opCtx,
     *oldExpireSecs = oldExpireSecsElement.safeNumberLong();
     if (**oldExpireSecs != indexExpireAfterSeconds) {
         // Change the value of "expireAfterSeconds" on disk.
-        autoColl->getWritableCollection(opCtx)->updateTTLSetting(
-            opCtx, idx->indexName(), indexExpireAfterSeconds);
+        writableColl->updateTTLSetting(opCtx, idx->indexName(), indexExpireAfterSeconds);
     }
 }
 
@@ -127,7 +146,7 @@ void _processCollModIndexRequestExpireAfterSeconds(OperationContext* opCtx,
  * Adjusts hidden setting on an index.
  */
 void _processCollModIndexRequestHidden(OperationContext* opCtx,
-                                       AutoGetCollection* autoColl,
+                                       Collection* writableColl,
                                        const IndexDescriptor* idx,
                                        bool indexHidden,
                                        boost::optional<bool>* newHidden,
@@ -136,8 +155,7 @@ void _processCollModIndexRequestHidden(OperationContext* opCtx,
     *oldHidden = idx->hidden();
     // Make sure when we set 'hidden' to false, we can remove the hidden field from catalog.
     if (*oldHidden != *newHidden) {
-        autoColl->getWritableCollection(opCtx)->updateHiddenSetting(
-            opCtx, idx->indexName(), indexHidden);
+        writableColl->updateHiddenSetting(opCtx, idx->indexName(), indexHidden);
     }
 }
 
@@ -145,35 +163,34 @@ void _processCollModIndexRequestHidden(OperationContext* opCtx,
  * Adjusts unique setting on an index to true.
  */
 void _processCollModIndexRequestUnique(OperationContext* opCtx,
-                                       AutoGetCollection* autoColl,
+                                       Collection* writableColl,
                                        const IndexDescriptor* idx,
                                        boost::optional<repl::OplogApplication::Mode> mode,
                                        boost::optional<bool>* newUnique) {
     invariant(!idx->unique(), str::stream() << "Index is already unique: " << idx->infoObj());
-    const auto& collection = autoColl->getCollection();
 
     // Checks for duplicates for the 'applyOps' command. In the tenant migration case, assumes
     // similarly to initial sync that we don't need to perform this check in the destination
     // cluster.
     if (mode && *mode == repl::OplogApplication::Mode::kApplyOpsCmd) {
-        auto duplicateRecordsList = scanIndexForDuplicates(opCtx, collection, idx);
+        auto duplicateRecordsList = scanIndexForDuplicates(opCtx, idx);
         if (!duplicateRecordsList.empty()) {
-            uassertStatusOK(buildConvertUniqueErrorStatus(opCtx, collection, duplicateRecordsList));
+            uassertStatusOK(
+                buildConvertUniqueErrorStatus(opCtx, writableColl, duplicateRecordsList));
         }
     }
 
     *newUnique = true;
-    autoColl->getWritableCollection(opCtx)->updateUniqueSetting(opCtx, idx->indexName(), true);
+    writableColl->updateUniqueSetting(opCtx, idx->indexName(), true);
     // Resets 'prepareUnique' to false after converting to unique index;
-    autoColl->getWritableCollection(opCtx)->updatePrepareUniqueSetting(
-        opCtx, idx->indexName(), false);
+    writableColl->updatePrepareUniqueSetting(opCtx, idx->indexName(), false);
 }
 
 /**
  * Adjusts prepareUnique setting on an index.
  */
 void _processCollModIndexRequestPrepareUnique(OperationContext* opCtx,
-                                              AutoGetCollection* autoColl,
+                                              Collection* writableColl,
                                               const IndexDescriptor* idx,
                                               bool indexPrepareUnique,
                                               boost::optional<bool>* newPrepareUnique,
@@ -181,8 +198,7 @@ void _processCollModIndexRequestPrepareUnique(OperationContext* opCtx,
     *newPrepareUnique = indexPrepareUnique;
     *oldPrepareUnique = idx->prepareUnique();
     if (*oldPrepareUnique != *newPrepareUnique) {
-        autoColl->getWritableCollection(opCtx)->updatePrepareUniqueSetting(
-            opCtx, idx->indexName(), indexPrepareUnique);
+        writableColl->updatePrepareUniqueSetting(opCtx, idx->indexName(), indexPrepareUnique);
     }
 }
 
@@ -190,19 +206,19 @@ void _processCollModIndexRequestPrepareUnique(OperationContext* opCtx,
  * Adjusts unique setting on an index to false.
  */
 void _processCollModIndexRequestForceNonUnique(OperationContext* opCtx,
-                                               AutoGetCollection* autoColl,
+                                               Collection* writableColl,
                                                const IndexDescriptor* idx,
                                                boost::optional<bool>* newForceNonUnique) {
     invariant(idx->unique(), str::stream() << "Index is already non-unique: " << idx->infoObj());
 
     *newForceNonUnique = true;
-    autoColl->getWritableCollection(opCtx)->updateUniqueSetting(opCtx, idx->indexName(), false);
+    writableColl->updateUniqueSetting(opCtx, idx->indexName(), false);
 }
 
 }  // namespace
 
 void processCollModIndexRequest(OperationContext* opCtx,
-                                AutoGetCollection* autoColl,
+                                Collection* writableColl,
                                 const ParsedCollModIndexRequest& collModIndexRequest,
                                 boost::optional<IndexCollModInfo>* indexCollModInfo,
                                 BSONObjBuilder* result,
@@ -232,30 +248,30 @@ void processCollModIndexRequest(OperationContext* opCtx,
     // TTL Index
     if (indexExpireAfterSeconds) {
         _processCollModIndexRequestExpireAfterSeconds(
-            opCtx, autoColl, idx, *indexExpireAfterSeconds, &newExpireSecs, &oldExpireSecs);
+            opCtx, writableColl, idx, *indexExpireAfterSeconds, &newExpireSecs, &oldExpireSecs);
     }
 
 
     // User wants to hide or unhide index.
     if (indexHidden) {
         _processCollModIndexRequestHidden(
-            opCtx, autoColl, idx, *indexHidden, &newHidden, &oldHidden);
+            opCtx, writableColl, idx, *indexHidden, &newHidden, &oldHidden);
     }
 
     // User wants to convert an index to be unique.
     if (indexUnique) {
         invariant(*indexUnique);
-        _processCollModIndexRequestUnique(opCtx, autoColl, idx, mode, &newUnique);
+        _processCollModIndexRequestUnique(opCtx, writableColl, idx, mode, &newUnique);
     }
 
     if (indexPrepareUnique) {
         _processCollModIndexRequestPrepareUnique(
-            opCtx, autoColl, idx, *indexPrepareUnique, &newPrepareUnique, &oldPrepareUnique);
+            opCtx, writableColl, idx, *indexPrepareUnique, &newPrepareUnique, &oldPrepareUnique);
     }
 
     // User wants to convert an index back to be non-unique.
     if (indexForceNonUnique) {
-        _processCollModIndexRequestForceNonUnique(opCtx, autoColl, idx, &newForceNonUnique);
+        _processCollModIndexRequestForceNonUnique(opCtx, writableColl, idx, &newForceNonUnique);
     }
 
     *indexCollModInfo =
@@ -282,44 +298,44 @@ void processCollModIndexRequest(OperationContext* opCtx,
 
     // Notify the index catalog that the definition of this index changed. This will invalidate the
     // local idx pointer. On rollback of this WUOW, the local var idx pointer will be valid again.
-    autoColl->getWritableCollection(opCtx)->getIndexCatalog()->refreshEntry(
-        opCtx, autoColl->getWritableCollection(opCtx), idx, flags);
+    writableColl->getIndexCatalog()->refreshEntry(opCtx, writableColl, idx, flags);
 
-    opCtx->recoveryUnit()->onCommit([oldExpireSecs,
-                                     newExpireSecs,
-                                     oldHidden,
-                                     newHidden,
-                                     newUnique,
-                                     oldPrepareUnique,
-                                     newPrepareUnique,
-                                     newForceNonUnique,
-                                     result](OperationContext*, boost::optional<Timestamp>) {
-        // add the fields to BSONObjBuilder result
-        if (oldExpireSecs) {
-            result->append("expireAfterSeconds_old", *oldExpireSecs);
-        }
-        if (newExpireSecs) {
-            result->append("expireAfterSeconds_new", *newExpireSecs);
-        }
-        if (newHidden) {
-            invariant(oldHidden);
-            result->append("hidden_old", *oldHidden);
-            result->append("hidden_new", *newHidden);
-        }
-        if (newUnique) {
-            invariant(*newUnique);
-            result->appendBool("unique_new", true);
-        }
-        if (newPrepareUnique) {
-            invariant(oldPrepareUnique);
-            result->append("prepareUnique_old", *oldPrepareUnique);
-            result->append("prepareUnique_new", *newPrepareUnique);
-        }
-        if (newForceNonUnique) {
-            invariant(*newForceNonUnique);
-            result->appendBool("forceNonUnique_new", true);
-        }
-    });
+    shard_role_details::getRecoveryUnit(opCtx)->onCommit(
+        [oldExpireSecs,
+         newExpireSecs,
+         oldHidden,
+         newHidden,
+         newUnique,
+         oldPrepareUnique,
+         newPrepareUnique,
+         newForceNonUnique,
+         result](OperationContext*, boost::optional<Timestamp>) {
+            // add the fields to BSONObjBuilder result
+            if (oldExpireSecs) {
+                result->append("expireAfterSeconds_old", *oldExpireSecs);
+            }
+            if (newExpireSecs) {
+                result->append("expireAfterSeconds_new", *newExpireSecs);
+            }
+            if (newHidden) {
+                invariant(oldHidden);
+                result->append("hidden_old", *oldHidden);
+                result->append("hidden_new", *newHidden);
+            }
+            if (newUnique) {
+                invariant(*newUnique);
+                result->appendBool("unique_new", true);
+            }
+            if (newPrepareUnique) {
+                invariant(oldPrepareUnique);
+                result->append("prepareUnique_old", *oldPrepareUnique);
+                result->append("prepareUnique_new", *newPrepareUnique);
+            }
+            if (newForceNonUnique) {
+                invariant(*newForceNonUnique);
+                result->appendBool("forceNonUnique_new", true);
+            }
+        });
 
     if (MONGO_unlikely(assertAfterIndexUpdate.shouldFail())) {
         LOGV2(20307, "collMod - assertAfterIndexUpdate fail point enabled");
@@ -327,36 +343,22 @@ void processCollModIndexRequest(OperationContext* opCtx,
     }
 }
 
-std::list<std::set<RecordId>> scanIndexForDuplicates(
-    OperationContext* opCtx,
-    const CollectionPtr& collection,
-    const IndexDescriptor* idx,
-    boost::optional<KeyString::Value> firstKeyString) {
+std::list<std::set<RecordId>> scanIndexForDuplicates(OperationContext* opCtx,
+                                                     const IndexDescriptor* idx) {
     auto entry = idx->getEntry();
     auto accessMethod = entry->accessMethod()->asSortedData();
-    // Only scans for the duplicates on one key if 'firstKeyString' is provided.
-    bool scanOneKey = static_cast<bool>(firstKeyString);
-
-    // Starting point of index traversal.
-    if (!firstKeyString) {
-        auto keyStringVersion = accessMethod->getSortedDataInterface()->getKeyStringVersion();
-        KeyString::Builder firstKeyStringBuilder(keyStringVersion,
-                                                 BSONObj(),
-                                                 entry->ordering(),
-                                                 KeyString::Discriminator::kExclusiveBefore);
-        firstKeyString = firstKeyStringBuilder.getValueCopy();
-    }
 
     // Scans index for duplicates, comparing consecutive index entries.
     // KeyStrings will be in strictly increasing order because all keys are sorted and they are
     // in the format (Key, RID), and all RecordIDs are unique.
-    DataThrottle dataThrottle(opCtx);
+    DataThrottle dataThrottle(opCtx, [&]() { return gMaxValidateMBperSec.load(); });
     dataThrottle.turnThrottlingOff();
     SortedDataInterfaceThrottleCursor indexCursor(opCtx, accessMethod, &dataThrottle);
     boost::optional<KeyStringEntry> prevIndexEntry;
     std::list<std::set<RecordId>> duplicateRecordsList;
     std::set<RecordId> duplicateRecords;
-    for (auto indexEntry = indexCursor.seekForKeyString(opCtx, *firstKeyString); indexEntry;
+
+    for (auto indexEntry = indexCursor.nextKeyString(opCtx); indexEntry;
          indexEntry = indexCursor.nextKeyString(opCtx)) {
         if (prevIndexEntry &&
             (indexEntry->loc.isLong()
@@ -372,9 +374,6 @@ std::list<std::set<RecordId>> scanIndexForDuplicates(
                 // Adds the current group of violations with the same duplicate value.
                 duplicateRecordsList.push_back(duplicateRecords);
                 duplicateRecords.clear();
-                if (scanOneKey) {
-                    break;
-                }
             }
         }
         prevIndexEntry = indexEntry;
@@ -386,7 +385,7 @@ std::list<std::set<RecordId>> scanIndexForDuplicates(
 }
 
 Status buildConvertUniqueErrorStatus(OperationContext* opCtx,
-                                     const CollectionPtr& collection,
+                                     const Collection* collection,
                                      const std::list<std::set<RecordId>>& duplicateRecordsList) {
     BSONArrayBuilder duplicateViolations;
     size_t violationsSize = 0;
