@@ -29,9 +29,8 @@
 
 #include "mongo/db/catalog/collection.h"
 
-#include <sstream>
-
-#include "mongo/logv2/log.h"
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
 
@@ -98,4 +97,34 @@ void Collection::Factory::set(ServiceContext* service,
     factory = std::move(newFactory);
 }
 
+std::pair<std::unique_ptr<CollatorInterface>, ExpressionContext::CollationMatchesDefault>
+resolveCollator(OperationContext* opCtx, BSONObj userCollation, const CollectionPtr& collection) {
+    if (!collection || !collection->getDefaultCollator()) {
+        if (userCollation.isEmpty()) {
+            return {nullptr, ExpressionContext::CollationMatchesDefault::kYes};
+        } else {
+            return {getUserCollator(opCtx, userCollation),
+                    // If the user explicitly provided a simple collation, we can still treat it as
+                    // 'CollationMatchesDefault::kYes', as no collation and simple collation are
+                    // functionally equivalent in the query code.
+                    (SimpleBSONObjComparator::kInstance.evaluate(userCollation ==
+                                                                 CollationSpec::kSimpleSpec))
+                        ? ExpressionContext::CollationMatchesDefault::kYes
+                        : ExpressionContext::CollationMatchesDefault::kNo};
+        }
+    }
+
+    auto defaultCollator = collection->getDefaultCollator()->clone();
+    if (userCollation.isEmpty()) {
+        return {std::move(defaultCollator), ExpressionContext::CollationMatchesDefault::kYes};
+    }
+    auto userCollator = getUserCollator(opCtx, userCollation);
+
+    if (CollatorInterface::collatorsMatch(defaultCollator.get(), userCollator.get())) {
+        return {std::move(defaultCollator), ExpressionContext::CollationMatchesDefault::kYes};
+    } else {
+        return {std::move(userCollator), ExpressionContext::CollationMatchesDefault::kNo};
+    }
+}
+
 }  // namespace mongo
