@@ -29,33 +29,61 @@
 
 #pragma once
 
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <cstddef>
+#include <cstdint>
 #include <functional>
+#include <iosfwd>
+#include <memory>
 #include <string>
+#include <utility>
 #include <vector>
 
 #include "mongo/base/status.h"
 #include "mongo/base/status_with.h"
 #include "mongo/base/string_data.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/bson/timestamp.h"
 #include "mongo/db/catalog/capped_visibility.h"
+#include "mongo/db/catalog/clustered_collection_options_gen.h"
 #include "mongo/db/catalog/collection_operation_source.h"
 #include "mongo/db/catalog/collection_options.h"
-#include "mongo/db/concurrency/d_concurrency.h"
+#include "mongo/db/catalog/collection_options_gen.h"
+#include "mongo/db/catalog/index_catalog.h"
+#include "mongo/db/catalog/index_catalog_entry.h"
+#include "mongo/db/index/index_descriptor.h"
+#include "mongo/db/index/multikey_paths.h"
+#include "mongo/db/matcher/expression.h"
+#include "mongo/db/matcher/expression_parser.h"
 #include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/pipeline/change_stream_pre_and_post_images_options_gen.h"
+#include "mongo/db/pipeline/expression_context.h"
+#include "mongo/db/query/collation/collator_factory_interface.h"
 #include "mongo/db/query/collation/collator_interface.h"
-#include "mongo/db/query/plan_executor.h"
 #include "mongo/db/record_id.h"
 #include "mongo/db/repl/oplog.h"
+#include "mongo/db/service_context.h"
 #include "mongo/db/session/logical_session_id.h"
 #include "mongo/db/storage/bson_collection_catalog_entry.h"
 #include "mongo/db/storage/durable_catalog_entry.h"
+#include "mongo/db/storage/ident.h"
 #include "mongo/db/storage/record_store.h"
 #include "mongo/db/storage/snapshot.h"
+#include "mongo/db/timeseries/timeseries_gen.h"
 #include "mongo/db/yieldable.h"
 #include "mongo/logv2/log_attr.h"
 #include "mongo/platform/mutex.h"
 #include "mongo/s/shard_key_pattern.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/decorable.h"
+#include "mongo/util/intrusive_counter.h"
+#include "mongo/util/uuid.h"
+#include "mongo/util/version/releases.h"
 
 namespace mongo {
 
@@ -96,6 +124,10 @@ struct CollectionUpdateArgs {
     // Set if the diff insert operation needs to check for the field's existence.
     bool mustCheckExistenceForInsertOperations = false;
 
+    // If not null, stores the replicated RecordId of the document - the RecordId is expected to
+    // replicated across nodes with the update.
+    RecordId replicatedRecordId;
+
     // Set if OpTimes were reserved for the update ahead of time.
     std::vector<OplogSlot> oplogSlots;
 };
@@ -135,17 +167,6 @@ public:
         static Factory* get(ServiceContext* service);
         static Factory* get(OperationContext* opCtx);
         static void set(ServiceContext* service, std::unique_ptr<Factory> factory);
-
-        /**
-         * Constructs a Collection object. This does not persist any state to the storage engine,
-         * only constructs an in-memory representation of what already exists on disk.
-         */
-        virtual std::shared_ptr<Collection> make(OperationContext* opCtx,
-                                                 const NamespaceString& nss,
-                                                 RecordId catalogId,
-                                                 const CollectionOptions& options,
-                                                 std::unique_ptr<RecordStore> rs) const = 0;
-
         /**
          * Constructs a Collection object. This does not persist any state to the storage engine,
          * only constructs an in-memory representation of what already exists on disk.
@@ -208,7 +229,7 @@ public:
     };
 
     Collection() = default;
-    virtual ~Collection() = default;
+    ~Collection() override = default;
 
     /**
      * Clones this Collection instance. Some members are deep copied and some are shallow copied.
@@ -237,16 +258,6 @@ public:
                                     const DurableCatalogEntry& catalogEntry,
                                     boost::optional<Timestamp> readTimestamp) = 0;
 
-    virtual bool isCommitted() const {
-        return true;
-    }
-
-    /**
-     * Update the visibility of this collection in the Collection Catalog. Updates to this value
-     * are not idempotent, as successive updates with the same `val` should not occur.
-     */
-    virtual void setCommitted(bool val) {}
-
     virtual bool isInitialized() const {
         return false;
     }
@@ -389,6 +400,17 @@ public:
     virtual void setTimeseriesBucketsMayHaveMixedSchemaData(OperationContext* opCtx,
                                                             boost::optional<bool> setting) = 0;
 
+    virtual boost::optional<bool> timeseriesBucketingParametersHaveChanged() const = 0;
+
+    /**
+     * Sets the 'timeseriesBucketingParametersHaveChanged' catalog entry flag to 'value' for this
+     * collection.
+     *
+     * Throws if this is not a time-series collection.
+     */
+    virtual void setTimeseriesBucketingParametersChanged(OperationContext* opCtx,
+                                                         boost::optional<bool> value) = 0;
+
     /**
      * Returns true if the passed in time-series bucket document contains mixed-schema data. Returns
      * a non-OK status if the bucket's min/max is malformed.
@@ -412,6 +434,12 @@ public:
      */
     virtual void setRequiresTimeseriesExtendedRangeSupport(OperationContext* opCtx) const = 0;
 
+    /**
+     * Returns true if the time-series collection was created with 'bucketRoundingSeconds' and
+     * 'bucketMaxSpanSeconds', and none of the time-series options have been changed. The value may
+     * only transition from true to false.
+     */
+    virtual bool areTimeseriesBucketsFixed() const = 0;
     /*
      * Returns true if this collection is clustered. That is, its RecordIds store the value of the
      * cluster key. If the collection is clustered on _id, there is no separate _id index.
@@ -427,10 +455,31 @@ public:
     virtual void updateClusteredIndexTTLSetting(OperationContext* opCtx,
                                                 boost::optional<int64_t> expireAfterSeconds) = 0;
 
+    static bool everUsesCappedSnapshots(const NamespaceString& nss) {
+        // The oplog tracks its visibility through support from the storage engine.
+        if (nss.isOplog()) {
+            return false;
+        }
+
+        // Only use the behavior for non-replicated capped collections (which can accept concurrent
+        // writes).
+        if (nss.isReplicated()) {
+            return false;
+        }
+        return true;
+    }
+
     virtual Status updateCappedSize(OperationContext* opCtx,
                                     boost::optional<long long> newCappedSize,
                                     boost::optional<long long> newCappedMax) = 0;
 
+    /**
+     * Updates the 'recordIdsReplicated' setting to false.
+     * The collection will no longer guarantee the same recordId for a given document across all
+     * nodes in the a replica set.
+     */
+    virtual void unsetRecordIdsReplicated(OperationContext* opCtx) = 0;
+
     //
     // Index
     //
@@ -617,6 +666,8 @@ public:
     // Stats
     //
 
+    virtual bool areRecordIdsReplicated() const = 0;
+
     virtual bool isCapped() const = 0;
     virtual long long getCappedMaxDocs() const = 0;
     virtual long long getCappedMaxSize() const = 0;
@@ -647,24 +698,13 @@ public:
      */
     virtual uint64_t getIndexFreeStorageBytes(OperationContext* opCtx) const = 0;
 
-    /**
-     * If return value is not boost::none, reads with majority read concern using an older snapshot
-     * must error.
-     */
-    virtual boost::optional<Timestamp> getMinimumVisibleSnapshot() const = 0;
-
-    virtual void setMinimumVisibleSnapshot(Timestamp name) = 0;
-
-
     /**
      * Get the timestamp this Collection instance was most recently changed at.
-     * TODO SERVER-68270: Should currently not be used until min visible snapshot is removed
      */
     virtual boost::optional<Timestamp> getMinimumValidSnapshot() const = 0;
 
     /**
      * Sets the timestamp this Collection instance was most recently changed at.
-     * TODO SERVER-68270: Should currently not be used until min visible snapshot is removed
      */
     virtual void setMinimumValidSnapshot(Timestamp name) = 0;
 
@@ -701,6 +741,8 @@ public:
      * specifications will match the form stored in the IndexCatalog should any of these indexes
      * already exist.
      */
+    virtual StatusWith<BSONObj> addCollationDefaultsToIndexSpecsForCreate(
+        OperationContext* opCtx, const BSONObj& indexSpec) const = 0;
     virtual StatusWith<std::vector<BSONObj>> addCollationDefaultsToIndexSpecsForCreate(
         OperationContext* opCtx, const std::vector<BSONObj>& indexSpecs) const = 0;
 
@@ -709,7 +751,7 @@ public:
     /**
      * Called when this Collection is deregistered from the catalog
      */
-    virtual void onDeregisterFromCatalog(ServiceContext* svcCtx) = 0;
+    virtual void onDeregisterFromCatalog(OperationContext* opCtx) = 0;
 
     friend auto logAttrs(const Collection& col) {
         return logv2::multipleAttrs(col.ns(), col.uuid());
@@ -737,7 +779,7 @@ public:
 
     CollectionPtr(const CollectionPtr&) = delete;
     CollectionPtr(CollectionPtr&&);
-    ~CollectionPtr();
+    ~CollectionPtr() override;
 
     CollectionPtr& operator=(const CollectionPtr&) = delete;
     CollectionPtr& operator=(CollectionPtr&&);
@@ -779,7 +821,8 @@ public:
     void setShardKeyPattern(const BSONObj& shardKeyPattern);
     const ShardKeyPattern& getShardKeyPattern() const;
 
-    bool isSharded() const {
+    // TODO SERVER-79296 remove sharding info from CollectionPtr
+    bool isSharded_DEPRECATED() const {
         return static_cast<bool>(_shardKeyPattern);
     }
 
@@ -814,4 +857,25 @@ inline ValidationLevelEnum validationLevelOrDefault(boost::optional<ValidationLe
     return level.value_or(ValidationLevelEnum::strict);
 }
 
+/**
+ * Returns a collator for the user-specified collation 'userCollation'.
+ *
+ * Note: The caller should check if 'userCollation' is not empty since the empty 'userCollation'
+ * has the special meaning that the query follows the collection default collation that exists.
+ */
+inline std::unique_ptr<CollatorInterface> getUserCollator(OperationContext* opCtx,
+                                                          const BSONObj& userCollation) {
+    tassert(7542402, "Empty user collation", !userCollation.isEmpty());
+    return uassertStatusOK(
+        CollatorFactoryInterface::get(opCtx->getServiceContext())->makeFromBSON(userCollation));
+}
+
+/**
+ * Resolves the collator to either the user-specified collation or, if none was specified, to
+ * the collection-default collation and also returns a flag indicating whether the user-provided
+ * collation matches the collection default collation.
+ */
+std::pair<std::unique_ptr<CollatorInterface>, ExpressionContext::CollationMatchesDefault>
+resolveCollator(OperationContext* opCtx, BSONObj userCollation, const CollectionPtr& collection);
+
 }  // namespace mongo
