@@ -29,26 +29,50 @@
 
 #pragma once
 
+#include <absl/container/flat_hash_set.h>
+#include <boost/optional.hpp>
+#include <boost/optional/optional.hpp>
+#include <cstdint>
 #include <functional>
+#include <immer/detail/util.hpp>
 #include <map>
+#include <memory>
 #include <set>
+#include <string>
+#include <utility>
+#include <vector>
 
+#include "mongo/base/status.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/timestamp.h"
 #include "mongo/db/catalog/collection.h"
+#include "mongo/db/catalog/historical_catalogid_tracker.h"
+#include "mongo/db/catalog/index_catalog_entry.h"
 #include "mongo/db/catalog/views_for_database.h"
 #include "mongo/db/database_name.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/profile_filter.h"
 #include "mongo/db/service_context.h"
 #include "mongo/db/storage/durable_catalog_entry.h"
+#include "mongo/db/tenant_id.h"
 #include "mongo/db/views/view.h"
 #include "mongo/stdx/unordered_map.h"
+#include "mongo/util/assert_util_core.h"
 #include "mongo/util/functional.h"
 #include "mongo/util/immutable/map.h"
 #include "mongo/util/immutable/unordered_map.h"
 #include "mongo/util/immutable/unordered_set.h"
+#include "mongo/util/str.h"
+#include "mongo/util/string_map.h"
 #include "mongo/util/uuid.h"
 
 namespace mongo {
 
+extern const SharedCollectionDecorations::Decoration<AtomicWord<bool>>
+    historicalIDTrackerAllowsMixedModeWrites;
+
 class CollectionCatalog {
     friend class iterator;
     using OrderedCollectionMap =
@@ -71,12 +95,9 @@ public:
         bool operator!=(const iterator& other) const;
 
     private:
-        void _skipUncommitted();
-
         const OrderedCollectionMap& _map;
         immutable::map<std::pair<DatabaseName, UUID>, std::shared_ptr<Collection>>::iterator
             _mapIter;
-        immutable::map<std::pair<DatabaseName, UUID>, std::shared_ptr<Collection>>::iterator _end;
     };
 
     class Range {
@@ -164,8 +185,8 @@ public:
      *
      * Must be in WriteUnitOfWork. View creation rolls back if the unit of work aborts.
      *
-     * Caller must ensure corresponding database exists. Expects db.system.views MODE_X lock and
-     * view namespace MODE_IX lock (unless 'durability' is set to kAlreadyDurable).
+     * Expects db.system.views MODE_X lock and view namespace MODE_IX lock (unless 'durability' is
+     * set to kAlreadyDurable).
      */
     Status createView(OperationContext* opCtx,
                       const NamespaceString& viewName,
@@ -180,8 +201,6 @@ public:
      * Drop the view named 'viewName'.
      *
      * Must be in WriteUnitOfWork. The drop rolls back if the unit of work aborts.
-     *
-     * Caller must ensure corresponding database exists.
      */
     Status dropView(OperationContext* opCtx, const NamespaceString& viewName) const;
 
@@ -189,8 +208,6 @@ public:
      * Modify the view named 'viewName' to have the new 'viewOn' and 'pipeline'.
      *
      * Must be in WriteUnitOfWork. The modification rolls back if the unit of work aborts.
-     *
-     * Caller must ensure corresponding database exists.
      */
     Status modifyView(OperationContext* opCtx,
                       const NamespaceString& viewName,
@@ -313,16 +330,6 @@ public:
                             std::shared_ptr<Collection> collection,
                             boost::optional<Timestamp> commitTime);
 
-    /**
-     * Like 'registerCollection' above but allows the Collection to be registered using just a
-     * MODE_IX lock on the namespace. The collection will be added to the catalog using a two-phase
-     * commit where it is marked as 'pending commit' internally. The user must call
-     * 'onCreateCollection' which sets up the necessary state for finishing the two-phase commit.
-     */
-    void registerCollectionTwoPhase(OperationContext* opCtx,
-                                    std::shared_ptr<Collection> collection,
-                                    boost::optional<Timestamp> commitTime);
-
     /**
      * Deregister the collection.
      *
@@ -421,23 +428,6 @@ public:
     Collection* lookupCollectionByNamespaceForMetadataWrite(OperationContext* opCtx,
                                                             const NamespaceString& nss) const;
 
-    /**
-     * Returns true if the collection has been registered in the CollectionCatalog but not yet made
-     * visible.
-     */
-    bool isCollectionAwaitingVisibility(UUID uuid) const;
-
-    // TODO SERVER-74468: Remove this function
-    std::shared_ptr<const Collection> lookupCollectionByNamespaceForRead_DONT_USE(
-        OperationContext* opCtx, const NamespaceString& nss) const {
-        return _getCollectionByNamespace(opCtx, nss);
-    }
-    // TODO SERVER-74468: Remove this function
-    std::shared_ptr<const Collection> lookupCollectionByUUIDForRead_DONT_USE(
-        OperationContext* opCtx, const UUID& uuid) const {
-        return _getCollectionByUUID(opCtx, uuid);
-    }
-
     /**
      * This function gets the NamespaceString from the collection catalog entry that
      * corresponds to UUID uuid. If no collection exists with the uuid, return
@@ -453,36 +443,22 @@ public:
                                           const NamespaceString& nss) const;
 
     /**
-     * Returns true if this CollectionCatalog contains the provided collection instance
+     * Checks if the provided instance is the latest version for this catalog version. This check
+     * should be used to determine if the collection instance is safe to perform CRUD writes on. For
+     * the check to be meaningful it should be performed against CollectionCatalog::latest.
      */
-    bool containsCollection(OperationContext* opCtx, const Collection* collection) const;
+    bool isLatestCollection(OperationContext* opCtx, const Collection* collection) const;
 
     /**
-     * Returns the CatalogId for a given 'nss' or 'uuid' at timestamp 'ts'.
+     * Verifies that the provided collection name doesn't exist in the catalog and is exclusively
+     * present in the uncommitted updates of the operation. For the check to be meaningful it should
+     * be performed against CollectionCatalog::latest.
      */
-    struct CatalogIdLookup {
-        enum class Existence {
-            // Namespace or UUID exists at time 'ts' and catalogId set in 'id'.
-            kExists,
-            // Namespace or UUID does not exist at time 'ts'.
-            kNotExists,
-            // Namespace or UUID existence at time 'ts' is unknown. The durable catalog must be
-            // scanned to determine.
-            kUnknown
-        };
-        RecordId id;
-        Existence result;
-    };
-    CatalogIdLookup lookupCatalogIdByNSS(const NamespaceString& nss,
-                                         boost::optional<Timestamp> ts = boost::none) const;
-    CatalogIdLookup lookupCatalogIdByUUID(const UUID& uuid,
-                                          boost::optional<Timestamp> ts = boost::none) const;
+    void ensureCollectionIsNew(OperationContext* opCtx, const NamespaceString& nss) const;
 
     /**
      * Iterates through the views in the catalog associated with database `dbName`, applying
      * 'callback' to each view.  If the 'callback' returns false, the iterator exits early.
-     *
-     * Caller must ensure corresponding database exists.
      */
     void iterateViews(OperationContext* opCtx,
                       const DatabaseName& dbName,
@@ -491,8 +467,6 @@ public:
     /**
      * Look up the 'nss' in the view catalog, returning a shared pointer to a View definition,
      * or nullptr if it doesn't exist.
-     *
-     * Caller must ensure corresponding database exists.
      */
     std::shared_ptr<const ViewDefinition> lookupView(OperationContext* opCtx,
                                                      const NamespaceString& nss) const;
@@ -500,8 +474,6 @@ public:
     /**
      * Same functionality as above, except this function skips validating durable views in the
      * view catalog.
-     *
-     * Caller must ensure corresponding database exists.
      */
     std::shared_ptr<const ViewDefinition> lookupViewWithoutValidatingDurable(
         OperationContext* opCtx, const NamespaceString& nss) const;
@@ -670,8 +642,6 @@ public:
         int csfle = 0;
         // Queryable Encryption collections on non-internal databases
         int queryableEncryption = 0;
-        // <>.system.profile collections on non-internal databases. Counted as `internal`.
-        int systemProfile = 0;
     };
 
     /**
@@ -727,23 +697,6 @@ public:
      */
     Range range(const DatabaseName& dbName) const;
 
-    /**
-     * Checks if 'cleanupForOldestTimestampAdvanced' should be called when the oldest timestamp
-     * advanced. Used to avoid a potentially expensive call to 'cleanupForOldestTimestampAdvanced'
-     * if no write is needed.
-     */
-    bool needsCleanupForOldestTimestamp(Timestamp oldest) const;
-
-    /**
-     * Cleans up internal structures when the oldest timestamp advances
-     */
-    void cleanupForOldestTimestampAdvanced(Timestamp oldest);
-
-    /**
-     * Cleans up internal structures after catalog reopen
-     */
-    void cleanupForCatalogReopen(Timestamp stable);
-
     /**
      * Ensures we have a MODE_X lock on a collection or MODE_IX lock for newly created collections.
      */
@@ -751,6 +704,15 @@ public:
                                                         const NamespaceString& nss);
     static bool hasExclusiveAccessToCollection(OperationContext* opCtx, const NamespaceString& nss);
 
+    /**
+     * Returns HistoricalCatalogIdTracker for historical namespace/uuid mappings to catalogId based
+     * on timestamp.
+     */
+    const HistoricalCatalogIdTracker& catalogIdTracker() const;
+    HistoricalCatalogIdTracker& catalogIdTracker();
+
+    class BatchedCollectionWrite;
+
 private:
     friend class CollectionCatalog::iterator;
     class PublishCatalogUpdates;
@@ -792,13 +754,9 @@ private:
 
     /**
      * Register the collection.
-     *
-     * If 'twoPhase' is true, this call must be followed by 'onCreateCollection' which continues the
-     * two-phase commit process.
      */
     void _registerCollection(OperationContext* opCtx,
                              std::shared_ptr<Collection> collection,
-                             bool twoPhase,
                              boost::optional<Timestamp> commitTime);
 
     std::shared_ptr<Collection> _lookupCollectionByUUID(UUID uuid) const;
@@ -882,43 +840,6 @@ private:
                                       const NamespaceString& nss,
                                       NamespaceType type) const;
 
-    /**
-     * CatalogId with Timestamp
-     */
-    struct TimestampedCatalogId {
-        boost::optional<RecordId> id;
-        Timestamp ts;
-    };
-
-    // Push a catalogId for namespace and UUID at given Timestamp. Timestamp needs to be larger than
-    // other entries for this namespace and UUID. boost::none for catalogId represent drop,
-    // boost::none for timestamp turns this operation into a no-op.
-    void _pushCatalogIdForNSSAndUUID(const NamespaceString& nss,
-                                     const UUID& uuid,
-                                     boost::optional<RecordId> catalogId,
-                                     boost::optional<Timestamp> ts);
-
-    // Push a catalogId for 'from' and 'to' for a rename operation at given Timestamp. Timestamp
-    // needs to be larger than other entries for these namespaces. boost::none for timestamp turns
-    // this operation into a no-op.
-    void _pushCatalogIdForRename(const NamespaceString& from,
-                                 const NamespaceString& to,
-                                 boost::optional<Timestamp> ts);
-
-    // Inserts a catalogId for namespace and UUID at given Timestamp, if not boost::none. Used after
-    // scanning the durable catalog for a correct mapping at the given timestamp.
-    void _insertCatalogIdForNSSAndUUIDAfterScan(boost::optional<NamespaceString> nss,
-                                                boost::optional<UUID> uuid,
-                                                boost::optional<RecordId> catalogId,
-                                                Timestamp ts);
-
-    // Helper to calculate if a namespace or UUID needs to be marked for cleanup for a set of
-    // timestamped catalogIds
-    template <class Key, class CatalogIdChangesContainer>
-    void _markForCatalogIdCleanupIfNeeded(const Key& key,
-                                          CatalogIdChangesContainer& catalogIdChangesContainer,
-                                          const std::vector<TimestampedCatalogId>& ids);
-
     /**
      * Returns true if catalog information about this namespace or UUID should be looked up from the
      * durable catalog rather than using the in-memory state of the catalog.
@@ -954,12 +875,6 @@ private:
         const NamespaceStringOrUUID& nssOrUUID,
         Timestamp readTimestamp) const;
 
-    // Helpers for 'lookupCatalogIdByNSS' and 'lookupCatalogIdByUUID'.
-    CatalogIdLookup _checkWithOldestCatalogIdTimestampMaintained(
-        boost::optional<Timestamp> ts) const;
-    CatalogIdLookup _findCatalogIdInRange(boost::optional<Timestamp> ts,
-                                          const std::vector<TimestampedCatalogId>& range) const;
-
     /**
      * When present, indicates that the catalog is in closed state, and contains a map from UUID
      * to pre-close NSS. See also onCloseCatalog.
@@ -985,21 +900,8 @@ private:
     immutable::unordered_map<NamespaceString, std::shared_ptr<Collection>> _pendingCommitNamespaces;
     immutable::unordered_map<UUID, std::shared_ptr<Collection>, UUID::Hash> _pendingCommitUUIDs;
 
-    // CatalogId mappings for all known namespaces and UUIDs for the CollectionCatalog. The vector
-    // is sorted on timestamp. UUIDs will have at most two entries. One for the create and another
-    // for the drop. UUIDs stay the same across collection renames.
-    immutable::unordered_map<NamespaceString, std::vector<TimestampedCatalogId>> _nssCatalogIds;
-    immutable::unordered_map<UUID, std::vector<TimestampedCatalogId>, UUID::Hash> _uuidCatalogIds;
-    // Set of namespaces and UUIDs that need cleanup when the oldest timestamp advances
-    // sufficiently.
-    immutable::unordered_set<NamespaceString> _nssCatalogIdChanges;
-    immutable::unordered_set<UUID, UUID::Hash> _uuidCatalogIdChanges;
-    // Point at which the oldest timestamp need to advance for there to be any catalogId namespace
-    // that can be cleaned up
-    Timestamp _lowestCatalogIdTimestampForCleanup = Timestamp::max();
-    // The oldest timestamp at which the catalog maintains catalogId mappings. Anything older than
-    // this is unknown and must be discovered by scanning the durable catalog.
-    Timestamp _oldestCatalogIdTimestampMaintained = Timestamp::max();
+    // Provides functionality to lookup catalogId by namespace/uuid for a given timestamp.
+    HistoricalCatalogIdTracker _catalogIdTracker;
 
     // Map of database names to their corresponding views and other associated state.
     ViewsForDatabaseMap _viewsForDatabase;
@@ -1036,48 +938,6 @@ private:
     Stats _stats;
 };
 
-/**
- * RAII style object to stash a versioned CollectionCatalog on the OperationContext.
- * Calls to CollectionCatalog::get(OperationContext*) will return this instance.
- *
- * Unstashes the CollectionCatalog at destruction if the OperationContext::isLockFreeReadsOp()
- * flag is no longer set. This is handling for the nested Stasher use case.
- */
-class CollectionCatalogStasher {
-public:
-    CollectionCatalogStasher(OperationContext* opCtx);
-    CollectionCatalogStasher(OperationContext* opCtx,
-                             std::shared_ptr<const CollectionCatalog> catalog);
-
-    /**
-     * Unstashes the catalog if _opCtx->isLockFreeReadsOp() is no longer set.
-     */
-    ~CollectionCatalogStasher();
-
-    /**
-     * Moves ownership of the stash to the new instance, and marks the old one unstashed.
-     */
-    CollectionCatalogStasher(CollectionCatalogStasher&& other);
-
-    CollectionCatalogStasher(const CollectionCatalogStasher&) = delete;
-    CollectionCatalogStasher& operator=(const CollectionCatalogStasher&) = delete;
-    CollectionCatalogStasher& operator=(CollectionCatalogStasher&&) = delete;
-
-    /**
-     * Stashes 'catalog' on the _opCtx.
-     */
-    void stash(std::shared_ptr<const CollectionCatalog> catalog);
-
-    /**
-     * Resets the OperationContext so CollectionCatalog::get() returns latest catalog again
-     */
-    void reset();
-
-private:
-    OperationContext* _opCtx;
-    bool _stashed;
-};
-
 /**
  * RAII class to perform multiple writes to the CollectionCatalog on a single copy of the
  * CollectionCatalog instance. Requires the global lock to be held in exclusive write mode (MODE_X)
