@@ -27,44 +27,34 @@
  *    it in the license file.
  */
 
-#include "mongo/db/tenant_database_name.h"
 #include <benchmark/benchmark.h>
+#include <cstdint>
+#include <memory>
+#include <mutex>
+#include <string>
+#include <utility>
 
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+
+#include "mongo/bson/timestamp.h"
+#include "mongo/db/catalog/collection.h"
 #include "mongo/db/catalog/collection_catalog.h"
 #include "mongo/db/catalog/collection_mock.h"
+#include "mongo/db/client.h"
 #include "mongo/db/concurrency/d_concurrency.h"
-#include "mongo/db/concurrency/lock_state.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
+#include "mongo/db/database_name.h"
+#include "mongo/db/namespace_string.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/service_context.h"
+#include "mongo/db/tenant_id.h"
+#include "mongo/util/assert_util_core.h"
 #include "mongo/util/uuid.h"
 
 namespace mongo {
-
 namespace {
 
-class LockerImplClientObserver : public ServiceContext::ClientObserver {
-public:
-    LockerImplClientObserver() = default;
-    ~LockerImplClientObserver() = default;
-
-    void onCreateClient(Client* client) final {}
-
-    void onDestroyClient(Client* client) final {}
-
-    void onCreateOperationContext(OperationContext* opCtx) override {
-        opCtx->setLockState(std::make_unique<LockerImpl>(opCtx->getServiceContext()));
-    }
-
-    void onDestroyOperationContext(OperationContext* opCtx) final {}
-};
-
-const ServiceContext::ConstructorActionRegisterer clientObserverRegisterer{
-    "CollectionCatalogBenchmarkClientObserver",
-    [](ServiceContext* service) {
-        service->registerClientObserver(std::make_unique<LockerImplClientObserver>());
-    },
-    [](ServiceContext* serviceContext) {}};
-
 ServiceContext* setupServiceContext() {
     auto serviceContext = ServiceContext::make();
     auto serviceContextPtr = serviceContext.get();
@@ -77,9 +67,12 @@ void createCollections(OperationContext* opCtx, int numCollections) {
     BatchedCollectionCatalogWriter batched(opCtx);
 
     for (auto i = 0; i < numCollections; i++) {
-        const NamespaceString nss("collection_catalog_bm", std::to_string(i));
+        const NamespaceString nss = NamespaceString::createNamespaceString_forTest(
+            "collection_catalog_bm", std::to_string(i));
         CollectionCatalog::write(opCtx, [&](CollectionCatalog& catalog) {
-            catalog.registerCollection(opCtx, std::make_shared<CollectionMock>(nss));
+            catalog.registerCollection(opCtx,
+                                       std::make_shared<CollectionMock>(nss),
+                                       /*ts=*/boost::none);
         });
     }
 }
@@ -88,7 +81,7 @@ void createCollections(OperationContext* opCtx, int numCollections) {
 
 void BM_CollectionCatalogWrite(benchmark::State& state) {
     auto serviceContext = setupServiceContext();
-    ThreadClient threadClient(serviceContext);
+    ThreadClient threadClient(serviceContext->getService());
     ServiceContext::UniqueOperationContext opCtx = threadClient->makeOperationContext();
 
     createCollections(opCtx.get(), state.range(0));
@@ -103,9 +96,15 @@ void BM_CollectionCatalogWrite(benchmark::State& state) {
 
 void BM_CollectionCatalogWriteBatchedWithGlobalExclusiveLock(benchmark::State& state) {
     auto serviceContext = setupServiceContext();
-    ThreadClient threadClient(serviceContext);
+    ThreadClient threadClient(serviceContext->getService());
     ServiceContext::UniqueOperationContext opCtx = threadClient->makeOperationContext();
 
+    // TODO(SERVER-74657): Please revisit if this thread could be made killable.
+    {
+        stdx::lock_guard<Client> lk(*threadClient.get());
+        threadClient.get()->setSystemOperationUnkillableByStepdown(lk);
+    }
+
     createCollections(opCtx.get(), state.range(0));
 
     Lock::GlobalLock globalLk(opCtx.get(), MODE_X);
@@ -119,7 +118,7 @@ void BM_CollectionCatalogWriteBatchedWithGlobalExclusiveLock(benchmark::State& s
 
 void BM_CollectionCatalogCreateDropCollection(benchmark::State& state) {
     auto serviceContext = setupServiceContext();
-    ThreadClient threadClient(serviceContext);
+    ThreadClient threadClient(serviceContext->getService());
     ServiceContext::UniqueOperationContext opCtx = threadClient->makeOperationContext();
     Lock::GlobalLock globalLk(opCtx.get(), MODE_X);
 
@@ -128,10 +127,12 @@ void BM_CollectionCatalogCreateDropCollection(benchmark::State& state) {
     for (auto _ : state) {
         benchmark::ClobberMemory();
         CollectionCatalog::write(opCtx.get(), [&](CollectionCatalog& catalog) {
-            const NamespaceString nss("collection_catalog_bm", std::to_string(state.range(0)));
+            const NamespaceString nss = NamespaceString::createNamespaceString_forTest(
+                "collection_catalog_bm", std::to_string(state.range(0)));
             const UUID uuid = UUID::gen();
-            catalog.registerCollection(opCtx.get(), std::make_shared<CollectionMock>(uuid, nss));
-            catalog.deregisterCollection(opCtx.get(), uuid);
+            catalog.registerCollection(
+                opCtx.get(), std::make_shared<CollectionMock>(uuid, nss), boost::none);
+            catalog.deregisterCollection(opCtx.get(), uuid, false, boost::none);
         });
     }
 }
@@ -141,7 +142,7 @@ void BM_CollectionCatalogCreateNCollectionsBatched(benchmark::State& state) {
         benchmark::ClobberMemory();
 
         auto serviceContext = setupServiceContext();
-        ThreadClient threadClient(serviceContext);
+        ThreadClient threadClient(serviceContext->getService());
         ServiceContext::UniqueOperationContext opCtx = threadClient->makeOperationContext();
 
         Lock::GlobalLock globalLk(opCtx.get(), MODE_X);
@@ -149,9 +150,11 @@ void BM_CollectionCatalogCreateNCollectionsBatched(benchmark::State& state) {
 
         auto numCollections = state.range(0);
         for (auto i = 0; i < numCollections; i++) {
-            const NamespaceString nss("collection_catalog_bm", std::to_string(i));
+            const NamespaceString nss = NamespaceString::createNamespaceString_forTest(
+                "collection_catalog_bm", std::to_string(i));
             CollectionCatalog::write(opCtx.get(), [&](CollectionCatalog& catalog) {
-                catalog.registerCollection(opCtx.get(), std::make_shared<CollectionMock>(nss));
+                catalog.registerCollection(
+                    opCtx.get(), std::make_shared<CollectionMock>(nss), boost::none);
             });
         }
     }
@@ -162,15 +165,17 @@ void BM_CollectionCatalogCreateNCollections(benchmark::State& state) {
         benchmark::ClobberMemory();
 
         auto serviceContext = setupServiceContext();
-        ThreadClient threadClient(serviceContext);
+        ThreadClient threadClient(serviceContext->getService());
         ServiceContext::UniqueOperationContext opCtx = threadClient->makeOperationContext();
         Lock::GlobalLock globalLk(opCtx.get(), MODE_X);
 
         auto numCollections = state.range(0);
         for (auto i = 0; i < numCollections; i++) {
-            const NamespaceString nss("collection_catalog_bm", std::to_string(i));
+            const NamespaceString nss = NamespaceString::createNamespaceString_forTest(
+                "collection_catalog_bm", std::to_string(i));
             CollectionCatalog::write(opCtx.get(), [&](CollectionCatalog& catalog) {
-                catalog.registerCollection(opCtx.get(), std::make_shared<CollectionMock>(nss));
+                catalog.registerCollection(
+                    opCtx.get(), std::make_shared<CollectionMock>(nss), boost::none);
             });
         }
     }
@@ -178,11 +183,12 @@ void BM_CollectionCatalogCreateNCollections(benchmark::State& state) {
 
 void BM_CollectionCatalogLookupCollectionByNamespace(benchmark::State& state) {
     auto serviceContext = setupServiceContext();
-    ThreadClient threadClient(serviceContext);
+    ThreadClient threadClient(serviceContext->getService());
     ServiceContext::UniqueOperationContext opCtx = threadClient->makeOperationContext();
 
     createCollections(opCtx.get(), state.range(0));
-    const NamespaceString nss("collection_catalog_bm", std::to_string(state.range(0) / 2));
+    const NamespaceString nss = NamespaceString::createNamespaceString_forTest(
+        "collection_catalog_bm", std::to_string(state.range(0) / 2));
 
     for (auto _ : state) {
         benchmark::ClobberMemory();
@@ -194,11 +200,12 @@ void BM_CollectionCatalogLookupCollectionByNamespace(benchmark::State& state) {
 
 void BM_CollectionCatalogLookupCollectionByUUID(benchmark::State& state) {
     auto serviceContext = setupServiceContext();
-    ThreadClient threadClient(serviceContext);
+    ThreadClient threadClient(serviceContext->getService());
     ServiceContext::UniqueOperationContext opCtx = threadClient->makeOperationContext();
 
     createCollections(opCtx.get(), state.range(0));
-    const NamespaceString nss("collection_catalog_bm", std::to_string(state.range(0) / 2));
+    const NamespaceString nss = NamespaceString::createNamespaceString_forTest(
+        "collection_catalog_bm", std::to_string(state.range(0) / 2));
     auto coll = CollectionCatalog::get(opCtx.get())->lookupCollectionByNamespace(opCtx.get(), nss);
     invariant(coll->ns() == nss);
     const UUID uuid = coll->uuid();
@@ -212,7 +219,7 @@ void BM_CollectionCatalogLookupCollectionByUUID(benchmark::State& state) {
 
 void BM_CollectionCatalogIterateCollections(benchmark::State& state) {
     auto serviceContext = setupServiceContext();
-    ThreadClient threadClient(serviceContext);
+    ThreadClient threadClient(serviceContext->getService());
     ServiceContext::UniqueOperationContext opCtx = threadClient->makeOperationContext();
 
     createCollections(opCtx.get(), state.range(0));
@@ -221,8 +228,8 @@ void BM_CollectionCatalogIterateCollections(benchmark::State& state) {
         benchmark::ClobberMemory();
         auto catalog = CollectionCatalog::get(opCtx.get());
         auto count = 0;
-        for ([[maybe_unused]] auto&& coll :
-             catalog->range(TenantDatabaseName(boost::none, "collection_catalog_bm"))) {
+        for ([[maybe_unused]] auto&& coll : catalog->range(
+                 DatabaseName::createDatabaseName_forTest(boost::none, "collection_catalog_bm"))) {
             benchmark::DoNotOptimize(count++);
         }
     }
