@@ -29,8 +29,52 @@
 
 #pragma once
 
+#include <cstddef>
+#include <cstdint>
+#include <memory>
+#include <string>
+#include <utility>
+#include <vector>
+
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/clonable_ptr.h"
+#include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/db/catalog/capped_visibility.h"
+#include "mongo/db/catalog/clustered_collection_options_gen.h"
 #include "mongo/db/catalog/collection.h"
+#include "mongo/db/catalog/collection_options.h"
+#include "mongo/db/catalog/collection_options_gen.h"
 #include "mongo/db/catalog/index_catalog.h"
+#include "mongo/db/catalog/index_catalog_entry.h"
+#include "mongo/db/index/index_descriptor.h"
+#include "mongo/db/index/multikey_paths.h"
+#include "mongo/db/matcher/expression_parser.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/pipeline/change_stream_pre_and_post_images_options_gen.h"
+#include "mongo/db/query/collation/collator_interface.h"
+#include "mongo/db/record_id.h"
+#include "mongo/db/storage/bson_collection_catalog_entry.h"
+#include "mongo/db/storage/durable_catalog_entry.h"
+#include "mongo/db/storage/ident.h"
+#include "mongo/db/storage/record_data.h"
+#include "mongo/db/storage/record_store.h"
+#include "mongo/db/storage/recovery_unit.h"
+#include "mongo/db/storage/snapshot.h"
+#include "mongo/db/timeseries/timeseries_gen.h"
+#include "mongo/db/transaction_resources.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/platform/mutex.h"
+#include "mongo/util/uuid.h"
+#include "mongo/util/version/releases.h"
 
 namespace mongo {
 
@@ -43,12 +87,6 @@ public:
                                                              const NamespaceString& nss,
                                                              BSONObj collationSpec);
 
-    // TODO SERVER-56999: We should just need one API to create Collections
-    explicit CollectionImpl(OperationContext* opCtx,
-                            const NamespaceString& nss,
-                            RecordId catalogId,
-                            const CollectionOptions& options,
-                            std::unique_ptr<RecordStore> recordStore);
 
     explicit CollectionImpl(OperationContext* opCtx,
                             const NamespaceString& nss,
@@ -56,19 +94,12 @@ public:
                             std::shared_ptr<BSONCollectionCatalogEntry::MetaData> metadata,
                             std::unique_ptr<RecordStore> recordStore);
 
-    ~CollectionImpl();
+    ~CollectionImpl() override;
 
     std::shared_ptr<Collection> clone() const final;
 
     class FactoryImpl : public Factory {
     public:
-        // TODO SERVER-56999: We should just need one API to create Collections
-        std::shared_ptr<Collection> make(OperationContext* opCtx,
-                                         const NamespaceString& nss,
-                                         RecordId catalogId,
-                                         const CollectionOptions& options,
-                                         std::unique_ptr<RecordStore> rs) const final;
-
         std::shared_ptr<Collection> make(
             OperationContext* opCtx,
             const NamespaceString& nss,
@@ -85,8 +116,6 @@ public:
                             const DurableCatalogEntry& catalogEntry,
                             boost::optional<Timestamp> readTimestamp) final;
     bool isInitialized() const final;
-    bool isCommitted() const final;
-    void setCommitted(bool val) final;
 
     const NamespaceString& ns() const final {
         return _ns;
@@ -135,7 +164,7 @@ public:
     bool requiresIdIndex() const final;
 
     Snapshotted<BSONObj> docFor(OperationContext* opCtx, const RecordId& loc) const final {
-        return Snapshotted<BSONObj>(opCtx->recoveryUnit()->getSnapshotId(),
+        return Snapshotted<BSONObj>(shard_role_details::getRecoveryUnit(opCtx)->getSnapshotId(),
                                     _shared->_recordStore->dataFor(opCtx, loc).releaseToBson());
     }
 
@@ -209,15 +238,23 @@ public:
     bool isTemporary() const final;
 
     boost::optional<bool> getTimeseriesBucketsMayHaveMixedSchemaData() const final;
+
     void setTimeseriesBucketsMayHaveMixedSchemaData(OperationContext* opCtx,
                                                     boost::optional<bool> setting) final;
 
+    boost::optional<bool> timeseriesBucketingParametersHaveChanged() const final;
+
+    void setTimeseriesBucketingParametersChanged(OperationContext* opCtx,
+                                                 boost::optional<bool> value) final;
+
     StatusWith<bool> doesTimeseriesBucketsDocContainMixedSchemaData(
         const BSONObj& bucketsDoc) const final;
 
     bool getRequiresTimeseriesExtendedRangeSupport() const final;
     void setRequiresTimeseriesExtendedRangeSupport(OperationContext* opCtx) const final;
 
+    bool areTimeseriesBucketsFixed() const final;
+
     /**
      * isClustered() relies on the object returned from getClusteredInfo(). If
      * ClusteredCollectionInfo exists, the collection is clustered.
@@ -231,10 +268,14 @@ public:
                             boost::optional<long long> newCappedSize,
                             boost::optional<long long> newCappedMax) final;
 
+    void unsetRecordIdsReplicated(OperationContext* opCtx) final;
+
     //
     // Stats
     //
 
+    bool areRecordIdsReplicated() const final;
+
     bool isCapped() const final;
     long long getCappedMaxDocs() const final;
     long long getCappedMaxSize() const final;
@@ -261,7 +302,7 @@ public:
      */
     bool isEmpty(OperationContext* opCtx) const final;
 
-    inline int averageObjectSize(OperationContext* opCtx) const {
+    inline int averageObjectSize(OperationContext* opCtx) const override {
         uint64_t n = numRecords(opCtx);
 
         if (n == 0)
@@ -275,22 +316,14 @@ public:
 
     uint64_t getIndexFreeStorageBytes(OperationContext* opCtx) const final;
 
-    /**
-     * If return value is not boost::none, reads with majority read concern using an older snapshot
-     * must error.
-     */
-    boost::optional<Timestamp> getMinimumVisibleSnapshot() const final {
-        return _minVisibleSnapshot;
-    }
     boost::optional<Timestamp> getMinimumValidSnapshot() const final {
         return _minValidSnapshot;
     }
 
     /**
-     * Updates the minimum visible snapshot. The 'newMinimumVisibleSnapshot' is ignored if it would
-     * set the minimum visible snapshot backwards in time.
+     * Updates the minimum valid snapshot. The 'newMinimumValidSnapshot' is ignored if it would
+     * set the minimum valid snapshot backwards in time.
      */
-    void setMinimumVisibleSnapshot(Timestamp newMinimumVisibleSnapshot) final;
     void setMinimumValidSnapshot(Timestamp newMinimumValidSnapshot) final;
 
     boost::optional<TimeseriesOptions> getTimeseriesOptions() const final;
@@ -304,6 +337,8 @@ public:
 
     const CollectionOptions& getCollectionOptions() const final;
 
+    StatusWith<BSONObj> addCollationDefaultsToIndexSpecsForCreate(
+        OperationContext* opCtx, const BSONObj& indexSpecs) const final;
     StatusWith<std::vector<BSONObj>> addCollationDefaultsToIndexSpecsForCreate(
         OperationContext* opCtx, const std::vector<BSONObj>& indexSpecs) const final;
 
@@ -395,17 +430,12 @@ private:
      */
     void _initCommon(OperationContext* opCtx);
 
-    /**
-     * Helper to set the _metadata field. Every set must check for the storageEngine option to work
-     * around the issue described in SERVER-91193 and SERVER-91194.
-     */
-    void _setMetadata(std::shared_ptr<BSONCollectionCatalogEntry::MetaData>&& metadata);
-
     /**
      * Holder of shared state between CollectionImpl clones
      */
     struct SharedState {
-        SharedState(CollectionImpl* collection,
+        SharedState(OperationContext* opCtx,
+                    CollectionImpl* collection,
                     std::unique_ptr<RecordStore> recordStore,
                     const CollectionOptions& options);
         ~SharedState();
@@ -428,8 +458,6 @@ private:
         const bool _isCapped;
         const bool _needCappedLock;
 
-        AtomicWord<bool> _committed{true};
-
         // Tracks in-progress capped inserts to inform visibility for forward scans so that no
         // uncommitted records are skipped.
         CappedVisibilityObserver _cappedObserver;
@@ -455,7 +483,6 @@ private:
     NamespaceString _ns;
     RecordId _catalogId;
     UUID _uuid;
-    bool _cachedCommitted = true;
     std::shared_ptr<SharedState> _shared;
 
     // Collection metadata cached from the DurableCatalog. Is kept separate from the SharedState
@@ -469,7 +496,6 @@ private:
     Validator _validator;
 
     // The earliest snapshot that is allowed to use this collection.
-    boost::optional<Timestamp> _minVisibleSnapshot;
     boost::optional<Timestamp> _minValidSnapshot;
 
     bool _initialized = false;
