@@ -27,18 +27,31 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/catalog/collection_options.h"
-
+#include <fmt/format.h>
 #include <limits>
+#include <memory>
+#include <vector>
 
-#include "mongo/db/json.h"
-#include "mongo/idl/server_parameter_test_util.h"
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/json.h"
+#include "mongo/db/catalog/collection_options.h"
+#include "mongo/db/server_options.h"
 #include "mongo/platform/decimal128.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/bson_test_util.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/duration.h"
+#include "mongo/util/str.h"
+#include "mongo/util/time_support.h"
+#include "mongo/util/version/releases.h"
 
 namespace mongo {
+using namespace fmt::literals;
 using unittest::assertGet;
 
 void checkRoundTrip(const CollectionOptions& options) {
@@ -115,21 +128,6 @@ TEST(CollectionOptions, CappedSizeNotRoundUpForAlignment) {
     ASSERT_EQUALS(options.cappedMaxDocs, 0);
 }
 
-TEST(CollectionOptions, CappedSizeRoundsUpForAlignment) {
-    serverGlobalParams.mutableFCV.setVersion(
-        multiversion::FeatureCompatibilityVersion::kVersion_6_0);
-    const long long kUnalignedCappedSize = 1000;
-    const long long kAlignedCappedSize = 1024;
-
-    // Check size rounds up to multiple of alignment.
-    auto options = assertGet(
-        CollectionOptions::parse((BSON("capped" << true << "size" << kUnalignedCappedSize))));
-
-    ASSERT_EQUALS(options.capped, true);
-    ASSERT_EQUALS(options.cappedSize, kAlignedCappedSize);
-    ASSERT_EQUALS(options.cappedMaxDocs, 0);
-}
-
 TEST(CollectionOptions, IgnoreSizeWrongType) {
     auto options =
         assertGet(CollectionOptions::parse(fromjson("{size: undefined, capped: undefined}")));
@@ -645,9 +643,6 @@ TEST(FLECollectionOptions, Equality_DisAllowedTypes) {
 
 
 TEST(FLECollectionOptions, Range_AllowedTypes) {
-    // TODO: SERVER-67760 remove feature flag
-    RAIIServerParameterControllerForTest featureFlagController("featureFlagFLE2Range", true);
-
     ASSERT_OK(CollectionOptions::parse(fromjson(str::stream() << R"({
         encryptedFields: {
             "fields": [
@@ -655,7 +650,7 @@ TEST(FLECollectionOptions, Range_AllowedTypes) {
                     "path": "firstName",
                     "keyId": { '$uuid': '5f34e99a-b214-451f-b6f6-d3d28e933d15' },
                     "bsonType": "int",
-                    "queries": {"queryType": "rangePreview", "sparsity" : 1, min : 1, max : 2}
+                    "queries": {"queryType": "range", "sparsity" : 1, min : 1, max : 2}
                 }
             ]
         }})"))
@@ -668,7 +663,7 @@ TEST(FLECollectionOptions, Range_AllowedTypes) {
                     "path": "firstName",
                     "keyId": { '$uuid': '5f34e99a-b214-451f-b6f6-d3d28e933d15' },
                     "bsonType": "long",
-                    "queries": {"queryType": "rangePreview", "sparsity" : 1, min : {$numberLong: "1"}, max : {$numberLong: "2"}}
+                    "queries": {"queryType": "range", "sparsity" : 1, min : {$numberLong: "1"}, max : {$numberLong: "2"}}
                 }
             ]
         }})"))
@@ -682,7 +677,7 @@ TEST(FLECollectionOptions, Range_AllowedTypes) {
                     "path": "firstName",
                     "keyId": { '$uuid': '5f34e99a-b214-451f-b6f6-d3d28e933d15' },
                     "bsonType": ")" << type << R"(",
-                    "queries": {"queryType": "rangePreview", "sparsity" : 1}
+                    "queries": {"queryType": "range", "sparsity" : 1}
                 }
             ]
         }})"))
@@ -698,7 +693,7 @@ TEST(FLECollectionOptions, Range_AllowedTypes) {
                     "bsonType": ")"
                                                               << "double"
                                                               << R"(",
-                    "queries": {"queryType": "rangePreview", "sparsity" : 1, min: 0.000, max: 1.000, precision: 3}
+                    "queries": {"queryType": "range", "sparsity" : 1, min: 0.000, max: 1.000, precision: 3}
                 }
             ]
         }})"))
@@ -713,7 +708,7 @@ TEST(FLECollectionOptions, Range_AllowedTypes) {
                     "bsonType": ")"
                                                               << "decimal"
                                                               << R"(",
-                    "queries": {"queryType": "rangePreview", "sparsity" : 1, min: NumberDecimal("0.000"), max: NumberDecimal("1.000"), precision: 3}
+                    "queries": {"queryType": "range", "sparsity" : 1, min: NumberDecimal("0.000"), max: NumberDecimal("1.000"), precision: 3}
                 }
             ]
         }})"))
@@ -727,7 +722,7 @@ TEST(FLECollectionOptions, Range_AllowedTypes) {
                     "path": "firstName",
                     "keyId": { '$uuid': '5f34e99a-b214-451f-b6f6-d3d28e933d15' },
                     "bsonType": "date",
-                    "queries": {"queryType": "rangePreview", "sparsity" : 1, min : {"$date": {"$numberLong": "12344"}}, max : {"$date": {"$numberLong": "12345"}}}
+                    "queries": {"queryType": "range", "sparsity" : 1, min : {"$date": {"$numberLong": "12344"}}, max : {"$date": {"$numberLong": "12345"}}}
                 }
             ]
         }})"))
@@ -736,9 +731,6 @@ TEST(FLECollectionOptions, Range_AllowedTypes) {
 
 
 TEST(FLECollectionOptions, Range_DisAllowedTypes) {
-    // TODO: SERVER-67760 remove feature flag
-    RAIIServerParameterControllerForTest featureFlagController("featureFlagFLE2Range", true);
-
     std::vector<std::string> typesDisallowedIndexed({
         "array",
         "binData",
@@ -765,58 +757,14 @@ TEST(FLECollectionOptions, Range_DisAllowedTypes) {
                     "path": "firstName",
                     "keyId": { '$uuid': '5f34e99a-b214-451f-b6f6-d3d28e933d15' },
                     "bsonType": ")" << type << R"(",
-                    "queries": {"queryType": "rangePreview"}
+                    "queries": {"queryType": "range"}
                 }
             ]
         }})")));
     }
 }
 
-TEST(FLECollectionOptions, Range_MissingFields) {
-    // TODO: SERVER-67760 remove feature flag
-    RAIIServerParameterControllerForTest featureFlagController("featureFlagFLE2Range", true);
-
-    ASSERT_STATUS_CODE(6775202, CollectionOptions::parse(fromjson(R"({
-        encryptedFields: {
-            "fields": [
-                {
-                    "path": "firstName",
-                    "keyId": { '$uuid': '5f34e99a-b214-451f-b6f6-d3d28e933d15' },
-                    "bsonType": "int",
-                    "queries": {"queryType": "rangePreview"}
-                }
-            ]
-    }})")));
-
-    ASSERT_STATUS_CODE(6775203, CollectionOptions::parse(fromjson(R"({
-        encryptedFields: {
-            "fields": [
-                {
-                    "path": "firstName",
-                    "keyId": { '$uuid': '5f34e99a-b214-451f-b6f6-d3d28e933d15' },
-                    "bsonType": "int",
-                    "queries": {"queryType": "rangePreview", sparsity: 1}
-                }
-            ]
-    }})")));
-
-    ASSERT_STATUS_CODE(6775204, CollectionOptions::parse(fromjson(R"({
-        encryptedFields: {
-            "fields": [
-                {
-                    "path": "firstName",
-                    "keyId": { '$uuid': '5f34e99a-b214-451f-b6f6-d3d28e933d15' },
-                    "bsonType": "int",
-                    "queries": {"queryType": "rangePreview", sparsity: 1, min : 1}
-                }
-            ]
-    }})")));
-}
-
 TEST(FLECollectionOptions, Equality_ExtraFields) {
-    // TODO: SERVER-67760 remove feature flag
-    RAIIServerParameterControllerForTest featureFlagController("featureFlagFLE2Range", true);
-
     ASSERT_STATUS_CODE(6775205, CollectionOptions::parse(fromjson(R"({
         encryptedFields: {
             "fields": [
@@ -856,10 +804,6 @@ TEST(FLECollectionOptions, Equality_ExtraFields) {
 
 
 TEST(FLECollectionOptions, Range_MinMax) {
-    // TODO: SERVER-67760 remove feature flag
-    RAIIServerParameterControllerForTest featureFlagController("featureFlagFLE2Range", true);
-
-
     {
         auto doc = BSON("encryptedFields"
                         << BSON("fields" << BSON_ARRAY(BSON("path"
@@ -868,7 +812,7 @@ TEST(FLECollectionOptions, Range_MinMax) {
                                                             << "int"
                                                             << "queries"
                                                             << BSON("queryType"
-                                                                    << "rangePreview"
+                                                                    << "range"
                                                                     << "sparsity" << 1 << "min" << 2
                                                                     << "max" << 1)))));
 
@@ -883,7 +827,7 @@ TEST(FLECollectionOptions, Range_MinMax) {
                                                             << "long"
                                                             << "queries"
                                                             << BSON("queryType"
-                                                                    << "rangePreview"
+                                                                    << "range"
                                                                     << "sparsity" << 1 << "min"
                                                                     << 2LL << "max" << 1LL)))));
 
@@ -899,7 +843,7 @@ TEST(FLECollectionOptions, Range_MinMax) {
                                                  << "double"
                                                  << "queries"
                                                  << BSON("queryType"
-                                                         << "rangePreview"
+                                                         << "range"
                                                          << "sparsity" << 1 << "min" << 2.0)))));
 
         ASSERT_STATUS_CODE(6967100, CollectionOptions::parse(doc));
@@ -911,7 +855,7 @@ TEST(FLECollectionOptions, Range_MinMax) {
                                                    << "double"
                                                    << "queries"
                                                    << BSON("queryType"
-                                                           << "rangePreview"
+                                                           << "range"
                                                            << "sparsity" << 1 << "max" << 2.0)))));
 
         ASSERT_STATUS_CODE(6967100, CollectionOptions::parse(doc));
@@ -924,7 +868,7 @@ TEST(FLECollectionOptions, Range_MinMax) {
                                               << "double"
                                               << "queries"
                                               << BSON("queryType"
-                                                      << "rangePreview"
+                                                      << "range"
                                                       << "sparsity" << 1 << "precision" << 2)))));
 
         ASSERT_STATUS_CODE(6967100, CollectionOptions::parse(doc));
@@ -937,7 +881,7 @@ TEST(FLECollectionOptions, Range_MinMax) {
                                     << "decimal"
                                     << "queries"
                                     << BSON("queryType"
-                                            << "rangePreview"
+                                            << "range"
                                             << "sparsity" << 1 << "min" << Decimal128(2.0))))));
 
         ASSERT_STATUS_CODE(6967100, CollectionOptions::parse(doc));
@@ -950,7 +894,7 @@ TEST(FLECollectionOptions, Range_MinMax) {
                                     << "decimal"
                                     << "queries"
                                     << BSON("queryType"
-                                            << "rangePreview"
+                                            << "range"
                                             << "sparsity" << 1 << "max" << Decimal128(2.0))))));
 
         ASSERT_STATUS_CODE(6967100, CollectionOptions::parse(doc));
@@ -963,7 +907,7 @@ TEST(FLECollectionOptions, Range_MinMax) {
                                               << "decimal"
                                               << "queries"
                                               << BSON("queryType"
-                                                      << "rangePreview"
+                                                      << "range"
                                                       << "sparsity" << 1 << "precision" << 2)))));
 
         ASSERT_STATUS_CODE(6967100, CollectionOptions::parse(doc));
@@ -980,7 +924,7 @@ TEST(FLECollectionOptions, Range_MinMax) {
                                                         << "date"
                                                         << "queries"
                                                         << BSON("queryType"
-                                                                << "rangePreview"
+                                                                << "range"
                                                                 << "sparsity" << 1 << "min" << end
                                                                 << "max" << start)))));
 
@@ -988,7 +932,6 @@ TEST(FLECollectionOptions, Range_MinMax) {
 }
 
 TEST(FLECollectionOptions, Range_BoundTypeMismatch) {
-    RAIIServerParameterControllerForTest featureFlagController("featureFlagFLE2Range", true);
     ASSERT_STATUS_CODE(7018200, CollectionOptions::parse(fromjson(str::stream() << R"({
         encryptedFields: {
             "fields": [
@@ -996,7 +939,7 @@ TEST(FLECollectionOptions, Range_BoundTypeMismatch) {
                     "path": "firstName",
                     "keyId": { '$uuid': '5f34e99a-b214-451f-b6f6-d3d28e933d15' },
                     "bsonType": "int",
-                    "queries": {"queryType": "rangePreview", "sparsity" : 1, min: {"$numberLong": "12344"}, max: {"$numberLong": "123440"}}
+                    "queries": {"queryType": "range", "sparsity" : 1, min: {"$numberLong": "12344"}, max: {"$numberLong": "123440"}}
                 }
             ]
         }})")));
@@ -1008,7 +951,7 @@ TEST(FLECollectionOptions, Range_BoundTypeMismatch) {
                     "path": "firstName",
                     "keyId": { '$uuid': '5f34e99a-b214-451f-b6f6-d3d28e933d15' },
                     "bsonType": "long",
-                    "queries": {"queryType": "rangePreview", "sparsity" : 1, min: 1, max: 2}
+                    "queries": {"queryType": "range", "sparsity" : 1, min: 1, max: 2}
                 }
             ]
         }})")));
@@ -1020,7 +963,7 @@ TEST(FLECollectionOptions, Range_BoundTypeMismatch) {
                     "path": "firstName",
                     "keyId": { '$uuid': '5f34e99a-b214-451f-b6f6-d3d28e933d15' },
                     "bsonType": "long",
-                    "queries": {"queryType": "rangePreview", "sparsity" : 1, min: {$numberLong: "1"}, max: 2}
+                    "queries": {"queryType": "range", "sparsity" : 1, min: {$numberLong: "1"}, max: 2}
                 }
             ]
         }})")));
@@ -1031,7 +974,68 @@ TEST(FLECollectionOptions, Range_BoundTypeMismatch) {
                     "path": "firstName",
                     "keyId": { '$uuid': '5f34e99a-b214-451f-b6f6-d3d28e933d15' },
                     "bsonType": "int",
-                    "queries": {"queryType": "rangePreview", "sparsity" : 1, min: 1, max: {"$numberLong": "123440"}}
+                    "queries": {"queryType": "range", "sparsity" : 1, min: 1, max: {"$numberLong": "123440"}}
+                }
+            ]
+        }})")));
+}
+
+TEST(FLECollectionOptions, Range_Sparsity) {
+    ASSERT(CollectionOptions::parse(fromjson(R"({
+        encryptedFields: {
+            "fields": [
+                {
+                    "path": "firstName",
+                    "keyId": { '$uuid': '5f34e99a-b214-451f-b6f6-d3d28e933d15' },
+                    "bsonType": "int",
+                    "queries": {"queryType": "range"}
+                }
+            ]
+        }})"))
+               .isOK());
+    ASSERT(CollectionOptions::parse(fromjson(R"({
+        encryptedFields: {
+            "fields": [
+                {
+                    "path": "firstName",
+                    "keyId": { '$uuid': '5f34e99a-b214-451f-b6f6-d3d28e933d15' },
+                    "bsonType": "int",
+                    "queries": {"queryType": "range", "sparsity" : 1}
+                }
+            ]
+        }})"))
+               .isOK());
+    ASSERT(CollectionOptions::parse(fromjson(R"({
+        encryptedFields: {
+            "fields": [
+                {
+                    "path": "firstName",
+                    "keyId": { '$uuid': '5f34e99a-b214-451f-b6f6-d3d28e933d15' },
+                    "bsonType": "int",
+                    "queries": {"queryType": "range", "sparsity" : 4}
+                }
+            ]
+        }})"))
+               .isOK());
+    ASSERT_STATUS_CODE(ErrorCodes::BadValue, CollectionOptions::parse(fromjson(str::stream() << R"({
+        encryptedFields: {
+            "fields": [
+                {
+                    "path": "firstName",
+                    "keyId": { '$uuid': '5f34e99a-b214-451f-b6f6-d3d28e933d15' },
+                    "bsonType": "int",
+                    "queries": {"queryType": "range", "sparsity" : 0}
+                }
+            ]
+        }})")));
+    ASSERT_STATUS_CODE(ErrorCodes::BadValue, CollectionOptions::parse(fromjson(str::stream() << R"({
+        encryptedFields: {
+            "fields": [
+                {
+                    "path": "firstName",
+                    "keyId": { '$uuid': '5f34e99a-b214-451f-b6f6-d3d28e933d15' },
+                    "bsonType": "int",
+                    "queries": {"queryType": "range", "sparsity" : 5}
                 }
             ]
         }})")));
