@@ -27,21 +27,63 @@
  *    it in the license file.
  */
 
-#include "mongo/bson/oid.h"
-#include "mongo/db/catalog/capped_utils.h"
+#include <memory>
+#include <mutex>
+#include <string>
+#include <vector>
+
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/json.h"
+#include "mongo/bson/timestamp.h"
 #include "mongo/db/catalog/catalog_test_fixture.h"
+#include "mongo/db/catalog/collection.h"
 #include "mongo/db/catalog/collection_mock.h"
-#include "mongo/db/catalog/collection_validation.h"
+#include "mongo/db/catalog/collection_options.h"
 #include "mongo/db/catalog/collection_write_path.h"
+#include "mongo/db/catalog/database.h"
+#include "mongo/db/catalog/index_catalog.h"
+#include "mongo/db/catalog/index_catalog_entry.h"
+#include "mongo/db/catalog_raii.h"
+#include "mongo/db/client.h"
+#include "mongo/db/concurrency/d_concurrency.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
 #include "mongo/db/db_raii.h"
 #include "mongo/db/index/index_access_method.h"
-#include "mongo/db/repl/storage_interface_impl.h"
+#include "mongo/db/index/index_descriptor.h"
+#include "mongo/db/index/multikey_paths.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/record_id.h"
+#include "mongo/db/repl/oplog.h"
+#include "mongo/db/repl/storage_interface.h"
+#include "mongo/db/service_context.h"
+#include "mongo/db/storage/record_data.h"
+#include "mongo/db/storage/record_store.h"
+#include "mongo/db/storage/recovery_unit.h"
+#include "mongo/db/storage/snapshot.h"
+#include "mongo/db/storage/write_unit_of_work.h"
+#include "mongo/db/timeseries/timeseries_gen.h"
+#include "mongo/db/transaction_resources.h"
 #include "mongo/db/update/document_diff_applier.h"
 #include "mongo/db/update/document_diff_calculator.h"
 #include "mongo/stdx/thread.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/bson_test_util.h"
+#include "mongo/unittest/framework.h"
 #include "mongo/util/assert_util.h"
+#include "mongo/util/duration.h"
 #include "mongo/util/fail_point.h"
+#include "mongo/util/shared_buffer.h"
+#include "mongo/util/time_support.h"
+#include "mongo/util/uuid.h"
 
 namespace mongo {
 namespace {
@@ -131,7 +173,7 @@ TEST_F(CollectionTest, CappedNotifierWaitUntilAsynchronousNotifyAll) {
 
     auto before = Date_t::now();
     stdx::thread thread([this, before, prevVersion, &notifier] {
-        ThreadClient client(getServiceContext());
+        ThreadClient client(getServiceContext()->getService());
         auto opCtx = cc().makeOperationContext();
         notifier->waitUntil(opCtx.get(), prevVersion, before + Milliseconds(25));
         auto after = Date_t::now();
@@ -156,7 +198,7 @@ TEST_F(CollectionTest, CappedNotifierWaitUntilAsynchronousKill) {
 
     auto before = Date_t::now();
     stdx::thread thread([this, before, prevVersion, &notifier] {
-        ThreadClient client(getServiceContext());
+        ThreadClient client(getServiceContext()->getService());
         auto opCtx = cc().makeOperationContext();
         notifier->waitUntil(opCtx.get(), prevVersion, before + Milliseconds(25));
         auto after = Date_t::now();
@@ -183,13 +225,13 @@ TEST_F(CollectionTest, CappedNotifierWaitUntilInterrupt) {
     auto& clientToInterrupt = cc();
     auto before = Date_t::now();
     stdx::thread thread([this, before, prevVersion, &notifier, &clientToInterrupt] {
-        ThreadClient client(getServiceContext());
+        ThreadClient client(getServiceContext()->getService());
         auto opCtx = cc().makeOperationContext();
         notifier->waitUntil(opCtx.get(), prevVersion, before + Milliseconds(25));
         auto after = Date_t::now();
         ASSERT_GTE(after - before, Milliseconds(25));
 
-        stdx::lock_guard<Client> lk(clientToInterrupt);
+        ClientLock lk(&clientToInterrupt);
         getServiceContext()->killOperation(
             lk, clientToInterrupt.getOperationContext(), ErrorCodes::Interrupted);
     });
@@ -312,7 +354,8 @@ TEST_F(CollectionTest, VerifyIndexIsUpdated) {
     auto newDoc = BSON("_id" << 1 << "a" << 5);
     {
         WriteUnitOfWork wuow(opCtx);
-        Snapshotted<BSONObj> oldSnap(opCtx->recoveryUnit()->getSnapshotId(), oldDoc);
+        Snapshotted<BSONObj> oldSnap(shard_role_details::getRecoveryUnit(opCtx)->getSnapshotId(),
+                                     oldDoc);
         CollectionUpdateArgs args{oldDoc};
         collection_internal::updateDocument(opCtx,
                                             coll,
@@ -363,7 +406,8 @@ TEST_F(CollectionTest, VerifyIndexIsUpdatedWithDamages) {
     auto damagesOutput = doc_diff::computeDamages(oldDoc, *diff, false);
     {
         WriteUnitOfWork wuow(opCtx);
-        Snapshotted<BSONObj> oldSnap(opCtx->recoveryUnit()->getSnapshotId(), oldDoc);
+        Snapshotted<BSONObj> oldSnap(shard_role_details::getRecoveryUnit(opCtx)->getSnapshotId(),
+                                     oldDoc);
         CollectionUpdateArgs args{oldDoc};
         auto newDocStatus =
             collection_internal::updateDocumentWithDamages(opCtx,
@@ -794,7 +838,7 @@ TEST_F(CatalogTestFixture, CappedVisibilityEmptyInitialState) {
         return uassertStatusOK(rs->insertRecord(opCtx, data.c_str(), data.size(), Timestamp()));
     };
 
-    auto longLivedClient = getServiceContext()->makeClient("longLived");
+    auto longLivedClient = getServiceContext()->getService()->makeClient("longLived");
     auto longLivedOpCtx = longLivedClient->makeOperationContext();
     WriteUnitOfWork longLivedWUOW(longLivedOpCtx.get());
 
@@ -855,7 +899,7 @@ TEST_F(CatalogTestFixture, CappedVisibilityNonEmptyInitialState) {
         return uassertStatusOK(rs->insertRecord(opCtx, data.c_str(), data.size(), Timestamp()));
     };
 
-    auto longLivedClient = getServiceContext()->makeClient("longLived");
+    auto longLivedClient = getServiceContext()->getService()->makeClient("longLived");
     auto longLivedOpCtx = longLivedClient->makeOperationContext();
 
     RecordId initialId;
@@ -947,12 +991,13 @@ TEST_F(CollectionTest, CappedCursorRollover) {
     }
 
     // Setup the cursor that should rollover.
-    auto otherClient = getServiceContext()->makeClient("otherClient");
+    auto otherClient = getServiceContext()->getService()->makeClient("otherClient");
     auto otherOpCtx = otherClient->makeOperationContext();
+    Lock::GlobalLock globalLock{otherOpCtx.get(), MODE_IS};
     auto cursor = rs->getCursor(otherOpCtx.get());
     ASSERT(cursor->next());
     cursor->save();
-    otherOpCtx->recoveryUnit()->abandonSnapshot();
+    shard_role_details::getRecoveryUnit(otherOpCtx.get())->abandonSnapshot();
 
     // Insert 10 documents which causes a rollover.
     {
@@ -970,6 +1015,73 @@ TEST_F(CollectionTest, CappedCursorRollover) {
     ASSERT(!cursor->next());
 }
 
+TEST_F(CollectionTest, BoundedSeek) {
+    NamespaceString nss = NamespaceString::createNamespaceString_forTest("test.t");
+    ASSERT_OK(storageInterface()->createCollection(operationContext(), nss, {}));
+
+    AutoGetCollection autoColl(operationContext(), nss, MODE_IX);
+    const CollectionPtr& coll = autoColl.getCollection();
+    RecordStore* rs = coll->getRecordStore();
+
+    auto doInsert = [&](OperationContext* opCtx) -> RecordId {
+        Lock::GlobalLock globalLock{opCtx, MODE_IX};
+        std::string data = "data";
+        return uassertStatusOK(rs->insertRecord(opCtx, data.c_str(), data.size(), Timestamp()));
+    };
+
+    // Insert 5 records and delete the first one.
+    const int numToInsert = 5;
+    RecordId recordIds[numToInsert];
+    {
+        WriteUnitOfWork wuow(operationContext());
+        for (int i = 0; i < numToInsert; ++i) {
+            recordIds[i] = doInsert(operationContext());
+        }
+        Lock::GlobalLock globalLock{operationContext(), MODE_IX};
+        rs->deleteRecord(operationContext(), recordIds[0]);
+        wuow.commit();
+    }
+
+    // Forward inclusive seek
+    ASSERT_ID_EQ(rs->getCursor(operationContext())
+                     ->seek(recordIds[1], SeekableRecordCursor::BoundInclusion::kInclude),
+                 recordIds[1]);
+    ASSERT_ID_EQ(rs->getCursor(operationContext())
+                     ->seek(recordIds[0], SeekableRecordCursor::BoundInclusion::kInclude),
+                 recordIds[1]);
+    ASSERT(!rs->getCursor(operationContext())
+                ->seek(RecordId(recordIds[numToInsert - 1].getLong() + 1),
+                       SeekableRecordCursor::BoundInclusion::kInclude));
+
+    // Forward exclusive seek
+    ASSERT_ID_EQ(rs->getCursor(operationContext())
+                     ->seek(recordIds[1], SeekableRecordCursor::BoundInclusion::kExclude),
+                 recordIds[2]);
+    ASSERT(!rs->getCursor(operationContext())
+                ->seek(RecordId(recordIds[numToInsert - 1]),
+                       SeekableRecordCursor::BoundInclusion::kExclude));
+
+    // Reverse inclusive seek
+    ASSERT_ID_EQ(
+        rs->getCursor(operationContext(), false)
+            ->seek(recordIds[numToInsert - 1], SeekableRecordCursor::BoundInclusion::kInclude),
+        recordIds[numToInsert - 1]);
+    ASSERT_ID_EQ(rs->getCursor(operationContext(), false)
+                     ->seek(RecordId(recordIds[numToInsert - 1].getLong() + 1),
+                            SeekableRecordCursor::BoundInclusion::kInclude),
+                 recordIds[numToInsert - 1]);
+    ASSERT(!rs->getCursor(operationContext(), false)
+                ->seek(recordIds[0], SeekableRecordCursor::BoundInclusion::kInclude));
+
+    // Reverse exclusive seek
+    ASSERT_ID_EQ(
+        rs->getCursor(operationContext(), false)
+            ->seek(recordIds[numToInsert - 1], SeekableRecordCursor::BoundInclusion::kExclude),
+        recordIds[numToInsert - 2]);
+    ASSERT(!rs->getCursor(operationContext(), false)
+                ->seek(RecordId(recordIds[1]), SeekableRecordCursor::BoundInclusion::kExclude));
+}
+
 TEST_F(CatalogTestFixture, CappedCursorYieldFirst) {
     NamespaceString nss = NamespaceString::createNamespaceString_forTest("test.t");
     CollectionOptions options;
@@ -995,7 +1107,7 @@ TEST_F(CatalogTestFixture, CappedCursorYieldFirst) {
 
     // See that things work if you yield before you first call next().
     cursor->save();
-    operationContext()->recoveryUnit()->abandonSnapshot();
+    shard_role_details::getRecoveryUnit(operationContext())->abandonSnapshot();
 
     ASSERT_TRUE(cursor->restore());
 
