@@ -29,19 +29,60 @@
 
 #include "mongo/db/catalog/collection_write_path.h"
 
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <cstddef>
+#include <cstdint>
+#include <iterator>
+#include <memory>
+#include <string>
+#include <type_traits>
+#include <utility>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
 #include "mongo/bson/simple_bsonelement_comparator.h"
-#include "mongo/crypto/fle_crypto.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/crypto/fle_crypto_types.h"
 #include "mongo/db/catalog/capped_collection_maintenance.h"
+#include "mongo/db/catalog/clustered_collection_options_gen.h"
+#include "mongo/db/catalog/collection_options.h"
+#include "mongo/db/catalog/collection_options_gen.h"
 #include "mongo/db/catalog/document_validation.h"
 #include "mongo/db/catalog/local_oplog_info.h"
+#include "mongo/db/concurrency/d_concurrency.h"
 #include "mongo/db/concurrency/exception_util.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
+#include "mongo/db/exec/document_value/document.h"
 #include "mongo/db/exec/write_stage_common.h"
+#include "mongo/db/feature_flag.h"
 #include "mongo/db/op_observer/op_observer.h"
+#include "mongo/db/record_id_helpers.h"
+#include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/s/operation_sharding_state.h"
+#include "mongo/db/server_options.h"
+#include "mongo/db/service_context.h"
+#include "mongo/db/storage/duplicate_key_error_info.h"
+#include "mongo/db/storage/index_entry_comparison.h"
+#include "mongo/db/storage/key_format.h"
+#include "mongo/db/storage/record_data.h"
+#include "mongo/db/storage/record_store.h"
+#include "mongo/db/storage/recovery_unit.h"
 #include "mongo/db/storage/storage_parameters_gen.h"
-#include "mongo/db/transaction/transaction_participant.h"
+#include "mongo/db/transaction_resources.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/decorable.h"
 #include "mongo/util/fail_point.h"
+#include "mongo/util/namespace_string_util.h"
+#include "mongo/util/str.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
 
@@ -73,6 +114,13 @@ MONGO_FAIL_POINT_DEFINE(hangAfterCollectionInserts);
 // This fail point introduces corruption to documents during insert.
 MONGO_FAIL_POINT_DEFINE(corruptDocumentOnInsert);
 
+// This fail point manually forces the RecordId to be of a given value during insert.
+MONGO_FAIL_POINT_DEFINE(explicitlySetRecordIdOnInsert);
+
+// This fail point skips deletion of the record, so that the deletion call would only delete the
+// index keys.
+MONGO_FAIL_POINT_DEFINE(skipDeleteRecord);
+
 bool compareSafeContentElem(const BSONObj& oldDoc, const BSONObj& newDoc) {
     if (newDoc.hasField(kSafeContent) != oldDoc.hasField(kSafeContent)) {
         return false;
@@ -96,7 +144,8 @@ std::vector<OplogSlot> reserveOplogSlotsForRetryableFindAndModify(OperationConte
     // entries and set the timestamp for these synthetic entries to be TS - 1.
     auto oplogInfo = LocalOplogInfo::get(opCtx);
     auto slots = oplogInfo->getNextOpTimes(opCtx, 2);
-    uassertStatusOK(opCtx->recoveryUnit()->setTimestamp(slots.back().getTimestamp()));
+    uassertStatusOK(
+        shard_role_details::getRecoveryUnit(opCtx)->setTimestamp(slots.back().getTimestamp()));
     return slots;
 }
 
@@ -155,6 +204,30 @@ std::vector<bool> makeFromMigrateForInserts(
     return fromMigrate;
 }
 
+bool isCappedCollectionWithIndex(const CollectionPtr& collection) {
+    return collection->isCapped() && collection->getIndexCatalog()->haveAnyIndexes();
+}
+
+Status allowedToInsertDocuments(OperationContext* opCtx,
+                                const CollectionPtr& collection,
+                                size_t batchSize) {
+    if (batchSize > 1 && isCappedCollectionWithIndex(collection)) {
+        // We require that inserts to indexed capped collections be done one-at-a-time to avoid the
+        // possibility that a later document causes an earlier document to be deleted before it can
+        // be indexed.
+        if (shouldDeferCappedDeletesToOplogApplication(opCtx, collection)) {
+            // However, the logic to do these deletes only runs when the inserts are originally
+            // performed (i.e. on the primary). When doing oplog application, the secondary will
+            // later apply those delete oplogs that were originally generated by the primary, so
+            // even batched inserts will be indexed before they can be deleted.
+            return Status::OK();
+        }
+        return {ErrorCodes::OperationCannotBeBatched,
+                "Can't batch inserts into indexed capped collections"};
+    }
+    return Status::OK();
+}
+
 Status insertDocumentsImpl(OperationContext* opCtx,
                            const CollectionPtr& collection,
                            const std::vector<InsertStatement>::const_iterator begin,
@@ -163,42 +236,29 @@ Status insertDocumentsImpl(OperationContext* opCtx,
                            bool fromMigrate) {
     const auto& nss = collection->ns();
 
-    dassert(opCtx->lockState()->isCollectionLockedForMode(nss, MODE_IX));
+    dassert(shard_role_details::getLocker(opCtx)->isCollectionLockedForMode(nss, MODE_IX) ||
+            (nss.isOplog() && shard_role_details::getLocker(opCtx)->isWriteLocked()) ||
+            (nss.isChangeCollection() && nss.tenantId() &&
+             shard_role_details::getLocker(opCtx)->isLockHeldForMode(
+                 {ResourceType::RESOURCE_TENANT, *nss.tenantId()}, MODE_IX)));
 
     const size_t count = std::distance(begin, end);
 
-    if (collection->isCapped() && collection->getIndexCatalog()->haveAnyIndexes() && count > 1) {
-        // We require that inserts to indexed capped collections be done one-at-a-time to avoid the
-        // possibility that a later document causes an earlier document to be deleted before it can
-        // be indexed.
-        // TODO SERVER-21512 It would be better to handle this here by just doing single inserts.
-        return {ErrorCodes::OperationCannotBeBatched,
-                "Can't batch inserts into indexed capped collections"};
+    auto allowStatus = allowedToInsertDocuments(opCtx, collection, count);
+    if (!allowStatus.isOK()) {
+        return allowStatus;
     }
 
     if (collection->needsCappedLock()) {
-        // Ensure that we have X-locked the metadata resource for this replicated, non-clustered
-        // capped collection. Non-clustered capped collections require writes to be serialized on
+        // X-lock the metadata resource for this replicated, non-clustered capped collection until
+        // the end of the WUOW. Non-clustered capped collections require writes to be serialized on
         // the secondary in order to guarantee insertion order (SERVER-21483); this exclusive access
         // to the metadata resource prevents the primary from executing with more concurrency than
         // secondaries - thus helping secondaries keep up - and protects '_cappedFirstRecord'. See
         // SERVER-21646. On the other hand, capped clustered collections with a monotonically
         // increasing cluster key natively guarantee preservation of the insertion order, and don't
         // need serialisation. We allow concurrent inserts for clustered capped collections.
-        bool oplogSlotsReserved = std::any_of(
-            begin, end, [](InsertStatement statement) { return !statement.oplogSlot.isNull(); });
-        if (oplogSlotsReserved) {
-            tassert(8218001,
-                    "Operation on non-clustered capped collection had reserved an oplog time but "
-                    "did not have an exclusive lock on "
-                    "the metadata resource",
-                    opCtx->lockState()->isLockHeldForMode(
-                        ResourceId(ResourceType::RESOURCE_METADATA, collection->ns()), MODE_X));
-        } else if (!opCtx->lockState()->isLockHeldForMode(
-                       ResourceId(ResourceType::RESOURCE_METADATA, collection->ns()), MODE_X)) {
-            Lock::ResourceLock heldUntilEndOfWUOW{
-                opCtx, ResourceId(RESOURCE_METADATA, nss), MODE_X};
-        }
+        Lock::ResourceLock heldUntilEndOfWUOW{opCtx, ResourceId(RESOURCE_METADATA, nss), MODE_X};
     }
 
     std::vector<Record> records;
@@ -207,7 +267,11 @@ Status insertDocumentsImpl(OperationContext* opCtx,
     timestamps.reserve(count);
 
     std::vector<RecordId> cappedRecordIds;
-    if (collection->usesCappedSnapshots()) {
+    // For capped collections requiring capped snapshots, usually RecordIds are reserved and
+    // registered here to handle visibility. If the RecordId is provided by the caller, it is
+    // assumed the caller already reserved and properly registered the inserts in the
+    // CappedVisibilityObserver.
+    if (collection->usesCappedSnapshots() && begin->recordId.isNull()) {
         cappedRecordIds = collection->reserveCappedRecordIds(opCtx, count);
     }
 
@@ -222,6 +286,10 @@ Status insertDocumentsImpl(OperationContext* opCtx,
                 record_id_helpers::keyForDoc(doc,
                                              collection->getClusteredInfo()->getIndexSpec(),
                                              collection->getDefaultCollator()));
+        } else if (!it->replicatedRecordId.isNull()) {
+            // The 'replicatedRecordId' being set indicates that this insert belongs to a replicated
+            // recordId collection, and we need to use the given recordId while inserting.
+            recordId = it->replicatedRecordId;
         } else if (!it->recordId.isNull()) {
             // This case would only normally be called in a testing circumstance to avoid
             // automatically generating record ids for capped collections.
@@ -238,11 +306,22 @@ Status insertDocumentsImpl(OperationContext* opCtx,
             continue;
         }
 
+        explicitlySetRecordIdOnInsert.execute([&](const BSONObj& data) {
+            const auto docToMatch = data["doc"].Obj();
+            if (doc.woCompare(docToMatch) == 0) {
+                {
+                    auto ridValue = data["rid"].safeNumberInt();
+                    recordId = RecordId(ridValue);
+                }
+            }
+        });
+
         records.emplace_back(Record{std::move(recordId), RecordData(doc.objdata(), doc.objsize())});
         timestamps.emplace_back(it->oplogSlot.getTimestamp());
     }
 
     Status status = collection->getRecordStore()->insertRecords(opCtx, &records, timestamps);
+
     if (!status.isOK()) {
         if (auto extraInfo = status.extraInfo<DuplicateKeyErrorInfo>();
             extraInfo && collection->isClustered()) {
@@ -265,6 +344,7 @@ Status insertDocumentsImpl(OperationContext* opCtx,
         }
         return status;
     }
+
     std::vector<BsonRecord> bsonRecords;
     bsonRecords.reserve(count);
     int recordIndex = 0;
@@ -280,6 +360,16 @@ Status insertDocumentsImpl(OperationContext* opCtx,
         bsonRecords.emplace_back(std::move(bsonRecord));
     }
 
+    // An empty vector of recordIds is ignored by the OpObserver. When non-empty,
+    // the OpObserver will add recordIds to the generated oplog entries.
+    std::vector<RecordId> recordIds;
+    if (collection->areRecordIdsReplicated()) {
+        recordIds.reserve(count);
+        for (const auto& r : records) {
+            recordIds.push_back(r.id);
+        }
+    }
+
     int64_t keysInserted = 0;
     status =
         collection->getIndexCatalog()->indexRecords(opCtx, collection, bsonRecords, &keysInserted);
@@ -291,9 +381,10 @@ Status insertDocumentsImpl(OperationContext* opCtx,
         opDebug->additiveMetrics.incrementKeysInserted(keysInserted);
         // 'opDebug' may be deleted at rollback time in case of multi-document transaction.
         if (!opCtx->inMultiDocumentTransaction()) {
-            opCtx->recoveryUnit()->onRollback([opDebug, keysInserted](OperationContext*) {
-                opDebug->additiveMetrics.incrementKeysInserted(-keysInserted);
-            });
+            shard_role_details::getRecoveryUnit(opCtx)->onRollback(
+                [opDebug, keysInserted](OperationContext*) {
+                    opDebug->additiveMetrics.incrementKeysInserted(-keysInserted);
+                });
         }
     }
 
@@ -303,6 +394,7 @@ Status insertDocumentsImpl(OperationContext* opCtx,
             collection,
             begin,
             end,
+            recordIds,
             /*fromMigrate=*/makeFromMigrateForInserts(opCtx, nss, begin, end, fromMigrate),
             /*defaultFromMigrate=*/fromMigrate);
     }
@@ -317,6 +409,7 @@ Status insertDocumentsImpl(OperationContext* opCtx,
 Status insertDocumentForBulkLoader(OperationContext* opCtx,
                                    const CollectionPtr& collection,
                                    const BSONObj& doc,
+                                   RecordId replicatedRecordId,
                                    const OnRecordInsertedFn& onRecordInserted) {
     const auto& nss = collection->ns();
 
@@ -330,9 +423,17 @@ Status insertDocumentForBulkLoader(OperationContext* opCtx,
         return status;
     }
 
-    dassert(opCtx->lockState()->isCollectionLockedForMode(nss, MODE_IX));
+    dassert(shard_role_details::getLocker(opCtx)->isCollectionLockedForMode(nss, MODE_IX) ||
+            (nss.isOplog() && shard_role_details::getLocker(opCtx)->isWriteLocked()));
 
-    RecordId recordId;
+    // The replicatedRecordId must be provided if the collection has recordIdsReplicated:true and it
+    // must not be provided if the collection has recordIdsReplicated:false
+    invariant(collection->areRecordIdsReplicated() != replicatedRecordId.isNull(),
+              str::stream() << "Unexpected recordId value for collection with ns: '"
+                            << collection->ns().toStringForErrorMsg() << "', uuid: '"
+                            << collection->uuid());
+
+    RecordId recordId = replicatedRecordId;
     if (collection->isClustered()) {
         invariant(collection->getRecordStore()->keyFormat() == KeyFormat::String);
         recordId = uassertStatusOK(record_id_helpers::keyForDoc(
@@ -368,11 +469,14 @@ Status insertDocumentForBulkLoader(OperationContext* opCtx,
     }
     inserts.emplace_back(kUninitializedStmtId, doc, slot);
 
+    // During initial sync, there are no recordIds to be passed to the OpObserver to
+    // include in oplog entries, as we don't generate oplog entries.
     opCtx->getServiceContext()->getOpObserver()->onInserts(
         opCtx,
         collection,
         inserts.begin(),
         inserts.end(),
+        /*recordIds=*/{},
         /*fromMigrate=*/std::vector<bool>(inserts.size(), false),
         /*defaultFromMigrate=*/false);
 
@@ -381,10 +485,11 @@ Status insertDocumentForBulkLoader(OperationContext* opCtx,
     // Capture the recordStore here instead of the CollectionPtr object itself, because the record
     // store's lifetime is controlled by the collection IX lock held on the write paths, whereas the
     // CollectionPtr is just a front to the collection and its lifetime is shorter
-    opCtx->recoveryUnit()->onCommit([recordStore = collection->getRecordStore()](
-                                        OperationContext*, boost::optional<Timestamp>) {
-        recordStore->notifyCappedWaitersIfNeeded();
-    });
+    shard_role_details::getRecoveryUnit(opCtx)->onCommit(
+        [recordStore = collection->getRecordStore()](OperationContext*,
+                                                     boost::optional<Timestamp>) {
+            recordStore->notifyCappedWaitersIfNeeded();
+        });
 
     return loc.getStatus();
 }
@@ -410,7 +515,7 @@ Status insertDocuments(OperationContext* opCtx,
             return Status(ErrorCodes::InternalError,
                           str::stream()
                               << "Collection::insertDocument got document without _id for ns:"
-                              << nss.toString());
+                              << nss.toStringForErrorMsg());
         }
 
         auto status = collection->checkValidationAndParseResult(opCtx, it->doc);
@@ -431,21 +536,22 @@ Status insertDocuments(OperationContext* opCtx,
         }
     }
 
-    const SnapshotId sid = opCtx->recoveryUnit()->getSnapshotId();
+    const SnapshotId sid = shard_role_details::getRecoveryUnit(opCtx)->getSnapshotId();
 
     status = insertDocumentsImpl(opCtx, collection, begin, end, opDebug, fromMigrate);
     if (!status.isOK()) {
         return status;
     }
-    invariant(sid == opCtx->recoveryUnit()->getSnapshotId());
+    invariant(sid == shard_role_details::getRecoveryUnit(opCtx)->getSnapshotId());
 
     // Capture the recordStore here instead of the CollectionPtr object itself, because the record
     // store's lifetime is controlled by the collection IX lock held on the write paths, whereas the
     // CollectionPtr is just a front to the collection and its lifetime is shorter
-    opCtx->recoveryUnit()->onCommit([recordStore = collection->getRecordStore()](
-                                        OperationContext*, boost::optional<Timestamp>) {
-        recordStore->notifyCappedWaitersIfNeeded();
-    });
+    shard_role_details::getRecoveryUnit(opCtx)->onCommit(
+        [recordStore = collection->getRecordStore()](OperationContext*,
+                                                     boost::optional<Timestamp>) {
+            recordStore->notifyCappedWaitersIfNeeded();
+        });
 
     hangAfterCollectionInserts.executeIf(
         [&](const BSONObj& data) {
@@ -463,10 +569,10 @@ Status insertDocuments(OperationContext* opCtx,
             hangAfterCollectionInserts.pauseWhileSet(opCtx);
         },
         [&](const BSONObj& data) {
-            const auto& collElem = data["collectionNS"];
+            const auto fpNss = NamespaceStringUtil::parseFailPointData(data, "collectionNS");
             const auto& firstIdElem = data["first_id"];
             // If the failpoint specifies no collection or matches the existing one, hang.
-            return (!collElem || nss.ns() == collElem.str()) &&
+            return (fpNss.isEmpty() || nss == fpNss) &&
                 (!firstIdElem ||
                  (begin != end && firstIdElem.type() == mongo::String &&
                   begin->doc["_id"].str() == firstIdElem.str()));
@@ -500,8 +606,8 @@ Status checkFailCollectionInsertsFailPoint(const NamespaceString& ns, const BSON
         },
         [&](const BSONObj& data) {
             // If the failpoint specifies no collection or matches the existing one, fail.
-            const auto collElem = data["collectionNS"];
-            return !collElem || ns.ns() == collElem.str();
+            const auto fpNss = NamespaceStringUtil::parseFailPointData(data, "collectionNS");
+            return fpNss.isEmpty() || ns == fpNss;
         });
     return s;
 }
@@ -544,8 +650,9 @@ void updateDocument(OperationContext* opCtx,
                 compareSafeContentElem(oldDoc.value(), newDoc));
     }
 
-    dassert(opCtx->lockState()->isCollectionLockedForMode(collection->ns(), MODE_IX));
-    invariant(oldDoc.snapshotId() == opCtx->recoveryUnit()->getSnapshotId());
+    dassert(
+        shard_role_details::getLocker(opCtx)->isCollectionLockedForMode(collection->ns(), MODE_IX));
+    invariant(oldDoc.snapshotId() == shard_role_details::getRecoveryUnit(opCtx)->getSnapshotId());
     invariant(newDoc.isOwned());
 
     if (collection->needsCappedLock()) {
@@ -553,15 +660,24 @@ void updateDocument(OperationContext* opCtx,
             opCtx, ResourceId(RESOURCE_METADATA, collection->ns()), MODE_X};
     }
 
-    SnapshotId sid = opCtx->recoveryUnit()->getSnapshotId();
+    SnapshotId sid = shard_role_details::getRecoveryUnit(opCtx)->getSnapshotId();
 
     BSONElement oldId = oldDoc.value()["_id"];
-    if (!oldId.eoo() && SimpleBSONElementComparator::kInstance.evaluate(oldId != newDoc["_id"]))
+    // We accept equivalent _id according to the collation defined in the collection. 'foo' and
+    // 'Foo' could be equivalent but not byte-identical according to the collation of the
+    // collection.
+    BSONElementComparator eltCmp{BSONElementComparator::FieldNamesMode::kConsider,
+                                 collection->getDefaultCollator()};
+    if (!oldId.eoo() && eltCmp.evaluate(oldId != newDoc["_id"]))
         uasserted(13596, "in Collection::updateDocument _id mismatch");
 
     args->changeStreamPreAndPostImagesEnabledForCollection =
         collection->isChangeStreamPreAndPostImagesEnabled();
 
+    if (collection->areRecordIdsReplicated()) {
+        args->replicatedRecordId = oldLocation;
+    }
+
     OplogUpdateEntryArgs onUpdateArgs(args, collection);
     const bool setNeedsRetryImageOplogField =
         args->storeDocOption != CollectionUpdateArgs::StoreDocOption::None;
@@ -608,7 +724,7 @@ void updateDocument(OperationContext* opCtx,
             opDebug->additiveMetrics.incrementKeysDeleted(keysDeleted);
             // 'opDebug' may be deleted at rollback time in case of multi-document transaction.
             if (!opCtx->inMultiDocumentTransaction()) {
-                opCtx->recoveryUnit()->onRollback(
+                shard_role_details::getRecoveryUnit(opCtx)->onRollback(
                     [opDebug, keysInserted, keysDeleted](OperationContext*) {
                         opDebug->additiveMetrics.incrementKeysInserted(-keysInserted);
                         opDebug->additiveMetrics.incrementKeysDeleted(-keysDeleted);
@@ -617,7 +733,7 @@ void updateDocument(OperationContext* opCtx,
         }
     }
 
-    invariant(sid == opCtx->recoveryUnit()->getSnapshotId());
+    invariant(sid == shard_role_details::getRecoveryUnit(opCtx)->getSnapshotId());
     args->updatedDoc = newDoc;
 
     opCtx->getServiceContext()->getOpObserver()->onUpdate(opCtx, onUpdateArgs);
@@ -633,10 +749,15 @@ StatusWith<BSONObj> updateDocumentWithDamages(OperationContext* opCtx,
                                               bool* indexesAffected,
                                               OpDebug* opDebug,
                                               CollectionUpdateArgs* args) {
-    dassert(opCtx->lockState()->isCollectionLockedForMode(collection->ns(), MODE_IX));
-    invariant(oldDoc.snapshotId() == opCtx->recoveryUnit()->getSnapshotId());
+    dassert(
+        shard_role_details::getLocker(opCtx)->isCollectionLockedForMode(collection->ns(), MODE_IX));
+    invariant(oldDoc.snapshotId() == shard_role_details::getRecoveryUnit(opCtx)->getSnapshotId());
     invariant(collection->updateWithDamagesSupported());
 
+    if (collection->areRecordIdsReplicated()) {
+        args->replicatedRecordId = loc;
+    }
+
     OplogUpdateEntryArgs onUpdateArgs(args, collection);
     const bool setNeedsRetryImageOplogField =
         args->storeDocOption != CollectionUpdateArgs::StoreDocOption::None;
@@ -691,7 +812,7 @@ StatusWith<BSONObj> updateDocumentWithDamages(OperationContext* opCtx,
             opDebug->additiveMetrics.incrementKeysDeleted(keysDeleted);
             // 'opDebug' may be deleted at rollback time in case of multi-document transaction.
             if (!opCtx->inMultiDocumentTransaction()) {
-                opCtx->recoveryUnit()->onRollback(
+                shard_role_details::getRecoveryUnit(opCtx)->onRollback(
                     [opDebug, keysInserted, keysDeleted](OperationContext*) {
                         opDebug->additiveMetrics.incrementKeysInserted(-keysInserted);
                         opDebug->additiveMetrics.incrementKeysDeleted(-keysDeleted);
@@ -749,50 +870,56 @@ void deleteDocument(OperationContext* opCtx,
         Lock::ResourceLock heldUntilEndOfWUOW{opCtx, ResourceId(RESOURCE_METADATA, nss), MODE_X};
     }
 
-    std::vector<OplogSlot> oplogSlots;
-    auto retryableFindAndModifyLocation = RetryableFindAndModifyLocation::kNone;
-    if (storeDeletedDoc == StoreDeletedDoc::On && retryableWrite == RetryableWrite::kYes) {
-        retryableFindAndModifyLocation = RetryableFindAndModifyLocation::kSideCollection;
-        oplogSlots = reserveOplogSlotsForRetryableFindAndModify(opCtx);
+    OplogDeleteEntryArgs deleteArgs;
+    if (collection->areRecordIdsReplicated()) {
+        deleteArgs.replicatedRecordId = loc;
     }
-    OplogDeleteEntryArgs deleteArgs{nullptr /* deletedDoc */,
-                                    fromMigrate,
-                                    collection->isChangeStreamPreAndPostImagesEnabled(),
-                                    retryableFindAndModifyLocation,
-                                    oplogSlots};
 
-    opCtx->getServiceContext()->getOpObserver()->aboutToDelete(opCtx, collection, doc.value());
+    // TODO(SERVER-80956): remove this call.
+    opCtx->getServiceContext()->getOpObserver()->aboutToDelete(
+        opCtx, collection, doc.value(), &deleteArgs);
 
-    boost::optional<BSONObj> deletedDoc;
-    const bool isRecordingPreImageForRetryableWrite =
-        retryableFindAndModifyLocation != RetryableFindAndModifyLocation::kNone;
-    const bool isTimeseriesCollection =
-        collection->getTimeseriesOptions() || nss.isTimeseriesBucketsCollection();
+    invariant(doc.value().isOwned(),
+              str::stream() << "Document to delete is not owned: snapshot id: " << doc.snapshotId()
+                            << " document: " << doc.value());
 
-    if (isRecordingPreImageForRetryableWrite ||
-        collection->isChangeStreamPreAndPostImagesEnabled() ||
-        (isTimeseriesCollection &&
-         feature_flags::gTimeseriesScalabilityImprovements.isEnabled(
-             serverGlobalParams.featureCompatibility.acquireFCVSnapshot()))) {
-        deletedDoc.emplace(doc.value().getOwned());
+    deleteArgs.fromMigrate = fromMigrate;
+    deleteArgs.changeStreamPreAndPostImagesEnabledForCollection =
+        collection->isChangeStreamPreAndPostImagesEnabled();
+
+    const bool shouldRecordPreImageForRetryableWrite =
+        storeDeletedDoc == StoreDeletedDoc::On && retryableWrite == RetryableWrite::kYes;
+    if (shouldRecordPreImageForRetryableWrite) {
+        deleteArgs.retryableFindAndModifyLocation = RetryableFindAndModifyLocation::kSideCollection;
+        deleteArgs.retryableFindAndModifyOplogSlots =
+            reserveOplogSlotsForRetryableFindAndModify(opCtx);
     }
+
     int64_t keysDeleted = 0;
     collection->getIndexCatalog()->unindexRecord(
         opCtx, collection, doc.value(), loc, noWarn, &keysDeleted, checkRecordId);
-    collection->getRecordStore()->deleteRecord(opCtx, loc);
-    if (deletedDoc) {
-        deleteArgs.deletedDoc = &(deletedDoc.value());
+
+    if (MONGO_unlikely(skipDeleteRecord.shouldFail())) {
+        LOGV2_DEBUG(8096000,
+                    3,
+                    "Skipping deleting record in deleteDocument",
+                    "recordId"_attr = loc,
+                    "doc"_attr = doc.value().toString());
+    } else {
+        collection->getRecordStore()->deleteRecord(opCtx, loc);
     }
 
-    opCtx->getServiceContext()->getOpObserver()->onDelete(opCtx, collection, stmtId, deleteArgs);
+    opCtx->getServiceContext()->getOpObserver()->onDelete(
+        opCtx, collection, stmtId, doc.value(), deleteArgs);
 
     if (opDebug) {
         opDebug->additiveMetrics.incrementKeysDeleted(keysDeleted);
         // 'opDebug' may be deleted at rollback time in case of multi-document transaction.
         if (!opCtx->inMultiDocumentTransaction()) {
-            opCtx->recoveryUnit()->onRollback([opDebug, keysDeleted](OperationContext*) {
-                opDebug->additiveMetrics.incrementKeysDeleted(-keysDeleted);
-            });
+            shard_role_details::getRecoveryUnit(opCtx)->onRollback(
+                [opDebug, keysDeleted](OperationContext*) {
+                    opDebug->additiveMetrics.incrementKeysDeleted(-keysDeleted);
+                });
         }
     }
 }
