@@ -27,16 +27,26 @@
  *    it in the license file.
  */
 
+#pragma once
+
+#include <boost/none.hpp>
+#include <boost/optional.hpp>
+#include <boost/optional/optional.hpp>
 #include <string>
 
 #include "mongo/base/status.h"
 #include "mongo/bson/bsonobj.h"
-#include "mongo/db/catalog/collection_catalog.h"
+#include "mongo/db/catalog/collection_options.h"
+#include "mongo/db/catalog/virtual_collection_options.h"
+#include "mongo/db/commands/create_gen.h"
+#include "mongo/db/database_name.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/util/uuid.h"
 
 namespace mongo {
-class BSONObj;
+
 class OperationContext;
-class BSONElement;
 
 /**
  * Creates a collection as described in "cmdObj" on the database "dbName". Creates the collection's
@@ -44,22 +54,29 @@ class BSONElement;
  * default _id index.
  */
 Status createCollection(OperationContext* opCtx,
-                        const std::string& dbName,
+                        const DatabaseName& dbName,
                         const BSONObj& cmdObj,
                         const BSONObj& idIndex = BSONObj());
 
 /**
  * Creates a collection as parsed in 'cmd'.
  */
-Status createCollection(OperationContext* opCtx,
-                        const NamespaceString& ns,
-                        const CreateCommand& cmd);
+Status createCollection(OperationContext* opCtx, const CreateCommand& cmd);
 
 /**
- * Creates the change stream pre-images collection. The collection is clustered by the primary key,
- * _id.
+ * Creates the collection or the view as described by 'options'.
  */
-void createChangeStreamPreImagesCollection(OperationContext* opCtx);
+Status createCollection(OperationContext* opCtx,
+                        const NamespaceString& ns,
+                        const CollectionOptions& options,
+                        const boost::optional<BSONObj>& idIndex);
+
+/**
+ * Creates a virtual collection as described by 'vopts'.
+ */
+Status createVirtualCollection(OperationContext* opCtx,
+                               const NamespaceString& ns,
+                               const VirtualCollectionOptions& vopts);
 
 /**
  * As above, but only used by replication to apply operations. This allows recreating collections
@@ -70,10 +87,9 @@ void createChangeStreamPreImagesCollection(OperationContext* opCtx);
  * error.
  */
 Status createCollectionForApplyOps(OperationContext* opCtx,
-                                   const std::string& dbName,
+                                   const DatabaseName& dbName,
                                    const boost::optional<UUID>& ui,
                                    const BSONObj& cmdObj,
                                    bool allowRenameOutOfTheWay,
                                    const boost::optional<BSONObj>& idIndex = boost::none);
-
 }  // namespace mongo
