@@ -28,27 +28,54 @@
  */
 
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/catalog/drop_database.h"
-
 #include <algorithm>
+#include <cstddef>
+#include <memory>
+#include <mutex>
+#include <string>
+#include <vector>
 
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/db/catalog/collection.h"
+#include "mongo/db/catalog/collection_catalog.h"
+#include "mongo/db/catalog/database.h"
 #include "mongo/db/catalog/database_holder.h"
+#include "mongo/db/catalog/drop_database.h"
 #include "mongo/db/catalog_raii.h"
 #include "mongo/db/client.h"
 #include "mongo/db/concurrency/exception_util.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
 #include "mongo/db/curop.h"
+#include "mongo/db/database_name.h"
 #include "mongo/db/index_builds_coordinator.h"
+#include "mongo/db/namespace_string.h"
 #include "mongo/db/op_observer/op_observer.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/repl/member_state.h"
+#include "mongo/db/repl/optime.h"
 #include "mongo/db/repl/repl_client_info.h"
 #include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/service_context.h"
+#include "mongo/db/storage/recovery_unit.h"
+#include "mongo/db/storage/write_unit_of_work.h"
+#include "mongo/db/transaction_resources.h"
 #include "mongo/db/write_concern_options.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/decorable.h"
 #include "mongo/util/duration.h"
 #include "mongo/util/fail_point.h"
 #include "mongo/util/scopeguard.h"
+#include "mongo/util/str.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
 
@@ -71,8 +98,8 @@ Status _checkNssAndReplState(OperationContext* opCtx, Database* db, const Databa
     }
 
     auto replCoord = repl::ReplicationCoordinator::get(opCtx);
-    bool userInitiatedWritesAndNotPrimary = opCtx->writesAreReplicated() &&
-        !replCoord->canAcceptWritesForDatabase(opCtx, dbName.toStringWithTenantId());
+    bool userInitiatedWritesAndNotPrimary =
+        opCtx->writesAreReplicated() && !replCoord->canAcceptWritesForDatabase(opCtx, dbName);
 
     if (userInitiatedWritesAndNotPrimary) {
         return Status(ErrorCodes::NotWritablePrimary,
@@ -94,8 +121,9 @@ void _finishDropDatabase(OperationContext* opCtx,
                          const DatabaseName& dbName,
                          Database* db,
                          std::size_t numCollections,
-                         bool abortIndexBuilds) {
-    invariant(opCtx->lockState()->isDbLockedForMode(dbName, MODE_X));
+                         bool abortIndexBuilds,
+                         bool fromMigrate) {
+    invariant(shard_role_details::getLocker(opCtx)->isDbLockedForMode(dbName, MODE_X));
 
     // If DatabaseHolder::dropDb() fails, we should reset the drop-pending state on Database.
     ScopeGuard dropPendingGuard([db, opCtx] { db->setDropPending(opCtx, false); });
@@ -111,12 +139,12 @@ void _finishDropDatabase(OperationContext* opCtx,
         dropDatabaseHangBeforeInMemoryDrop.pauseWhileSet(opCtx);
     }
 
-    writeConflictRetry(opCtx, "dropDatabase_database", dbName.toString(), [&] {
+    writeConflictRetry(opCtx, "dropDatabase_database", NamespaceString(dbName), [&] {
         // We need to replicate the dropDatabase oplog entry and clear the collection catalog in the
         // same transaction. This is to prevent stepdown from interrupting between these two
         // operations and leaving this node in an inconsistent state.
         WriteUnitOfWork wunit(opCtx);
-        opCtx->getServiceContext()->getOpObserver()->onDropDatabase(opCtx, dbName);
+        opCtx->getServiceContext()->getOpObserver()->onDropDatabase(opCtx, dbName, fromMigrate);
 
         auto databaseHolder = DatabaseHolder::get(opCtx);
         databaseHolder->dropDb(opCtx, db);
@@ -133,11 +161,14 @@ void _finishDropDatabase(OperationContext* opCtx,
     LOGV2(20336, "dropDatabase", logAttrs(dbName), "numCollectionsDropped"_attr = numCollections);
 }
 
-Status _dropDatabase(OperationContext* opCtx, const DatabaseName& dbName, bool abortIndexBuilds) {
+Status _dropDatabase(OperationContext* opCtx,
+                     const DatabaseName& dbName,
+                     bool abortIndexBuilds,
+                     bool markFromMigrate) {
     // As this code can potentially require replication we disallow holding locks entirely. Holding
     // of any locks is disallowed while awaiting replication because this can potentially block for
     // long time while doing network activity.
-    invariant(!opCtx->lockState()->isLocked());
+    invariant(!shard_role_details::getLocker(opCtx)->isLocked());
 
     uassert(ErrorCodes::IllegalOperation,
             "Cannot drop a database in read-only mode",
@@ -147,7 +178,7 @@ Status _dropDatabase(OperationContext* opCtx, const DatabaseName& dbName, bool a
     uassert(ErrorCodes::IllegalOperation,
             str::stream() << "Dropping the '" << dbName.toStringForErrorMsg()
                           << "' database is prohibited.",
-            dbName.db() != DatabaseName::kAdmin.db());
+            !dbName.isAdminDB());
 
     {
         CurOp::get(opCtx)->ensureStarted();
@@ -163,9 +194,11 @@ Status _dropDatabase(OperationContext* opCtx, const DatabaseName& dbName, bool a
     // collections to drop.
     repl::OpTime latestDropPendingOpTime;
 
+    const auto tenantLockMode{
+        boost::make_optional(dbName.tenantId() && dbName.isConfigDB(), MODE_X)};
     {
         boost::optional<AutoGetDb> autoDB;
-        autoDB.emplace(opCtx, dbName, MODE_X);
+        autoDB.emplace(opCtx, dbName, MODE_X /* database lock mode*/, tenantLockMode);
 
         Database* db = autoDB->getDb();
         Status status = _checkNssAndReplState(opCtx, db, dbName);
@@ -186,11 +219,10 @@ Status _dropDatabase(OperationContext* opCtx, const DatabaseName& dbName, bool a
             dropDatabaseHangHoldingLock.pauseWhileSet();
         }
 
-        LOGV2(
-            20337, "dropDatabase {dbName} - starting", "dropDatabase - starting", logAttrs(dbName));
+        LOGV2(20337, "dropDatabase - starting", logAttrs(dbName));
         db->setDropPending(opCtx, true);
 
-        // If Database::dropCollectionEventIfSystem() fails, we should reset the drop-pending state
+        // If Database::dropCollectionEvenIfSystem() fails, we should reset the drop-pending state
         // on Database.
         ScopeGuard dropPendingGuard([&db, opCtx] { db->setDropPending(opCtx, false); });
         auto indexBuildsCoord = IndexBuildsCoordinator::get(opCtx);
@@ -202,15 +234,18 @@ Status _dropDatabase(OperationContext* opCtx, const DatabaseName& dbName, bool a
                 // Create a scope guard to reset the drop-pending state on the database to false if
                 // there is a replica state change that kills this operation while the locks were
                 // yielded.
-                ScopeGuard dropPendingGuardWhileUnlocked([dbName, opCtx, &dropPendingGuard] {
-                    // TODO (SERVER-71610): Fix to be interruptible or document exception.
-                    UninterruptibleLockGuard noInterrupt(opCtx->lockState());  // NOLINT.
-                    AutoGetDb autoDB(opCtx, dbName, MODE_IX);
-                    if (auto db = autoDB.getDb()) {
-                        db->setDropPending(opCtx, false);
-                    }
-                    dropPendingGuard.dismiss();
-                });
+                ScopeGuard dropPendingGuardWhileUnlocked(
+                    [dbName, opCtx, &dropPendingGuard, tenantLockMode] {
+                        // This scope guard must succeed in acquiring locks and reverting the drop
+                        // pending state even when the failure is due to an interruption.
+                        UninterruptibleLockGuard noInterrupt(opCtx);  // NOLINT.
+                        AutoGetDb autoDB(
+                            opCtx, dbName, MODE_X /* database lock mode*/, tenantLockMode);
+                        if (auto db = autoDB.getDb()) {
+                            db->setDropPending(opCtx, false);
+                        }
+                        dropPendingGuard.dismiss();
+                    });
 
                 // Drop locks. The drop helper will acquire locks on our behalf.
                 autoDB = boost::none;
@@ -226,7 +261,7 @@ Status _dropDatabase(OperationContext* opCtx, const DatabaseName& dbName, bool a
                     dropDatabaseHangAfterWaitingForIndexBuilds.pauseWhileSet();
                 }
 
-                autoDB.emplace(opCtx, dbName, MODE_X);
+                autoDB.emplace(opCtx, dbName, MODE_X /* database lock mode*/, tenantLockMode);
                 db = autoDB->getDb();
 
                 dropPendingGuardWhileUnlocked.dismiss();
@@ -234,7 +269,7 @@ Status _dropDatabase(OperationContext* opCtx, const DatabaseName& dbName, bool a
                 // Abandon the snapshot as the index catalog will compare the in-memory state to the
                 // disk state, which may have changed when we released the collection lock
                 // temporarily.
-                opCtx->recoveryUnit()->abandonSnapshot();
+                shard_role_details::getRecoveryUnit(opCtx)->abandonSnapshot();
 
                 status = _checkNssAndReplState(opCtx, db, dbName);
                 if (!status.isOK()) {
@@ -255,19 +290,51 @@ Status _dropDatabase(OperationContext* opCtx, const DatabaseName& dbName, bool a
             ++numCollections;
             const auto& nss = viewCollPtr->ns();
             LOGV2(7193700,
-                  "dropDatabase {dbName} - dropping collection: {nss}",
                   "dropDatabase - dropping collection",
                   logAttrs(dbName),
                   "namespace"_attr = nss);
 
-            writeConflictRetry(opCtx, "dropDatabase_views_collection", nss.ns(), [&] {
+            writeConflictRetry(opCtx, "dropDatabase_views_collection", nss, [&] {
                 WriteUnitOfWork wunit(opCtx);
-                fassert(7193701, db->dropCollectionEvenIfSystem(opCtx, nss));
+                fassert(7193701, db->dropCollectionEvenIfSystem(opCtx, nss, {}, markFromMigrate));
                 wunit.commit();
             });
         }
 
-        // Refresh the catalog so the views collection isn't present.
+        // The system.profile collection is created using an untimestamped write to the catalog when
+        // enabling profiling on a database. So we drop it untimestamped as well to avoid mixed-mode
+        // timestamp usage.
+        auto systemProfilePtr = catalog->lookupCollectionByNamespace(
+            opCtx, NamespaceString::makeSystemDotProfileNamespace(dbName));
+        if (systemProfilePtr) {
+            const Timestamp commitTs =
+                shard_role_details::getRecoveryUnit(opCtx)->getCommitTimestamp();
+            if (!commitTs.isNull()) {
+                shard_role_details::getRecoveryUnit(opCtx)->clearCommitTimestamp();
+            }
+
+            // Ensure this block exits with the same commit timestamp state that it was called with.
+            ScopeGuard addCommitTimestamp([&opCtx, commitTs] {
+                if (!commitTs.isNull()) {
+                    shard_role_details::getRecoveryUnit(opCtx)->setCommitTimestamp(commitTs);
+                }
+            });
+
+            const auto& nss = systemProfilePtr->ns();
+            LOGV2(7574000,
+                  "dropDatabase - dropping collection",
+                  logAttrs(dbName),
+                  "namespace"_attr = nss);
+
+            invariant(!shard_role_details::getLocker(opCtx)->inAWriteUnitOfWork());
+            writeConflictRetry(opCtx, "dropDatabase_system.profile_collection", nss, [&] {
+                WriteUnitOfWork wunit(opCtx);
+                fassert(7574001, db->dropCollectionEvenIfSystem(opCtx, nss, {}, markFromMigrate));
+                wunit.commit();
+            });
+        }
+
+        // Refresh the catalog so the views and profile collections aren't present.
         catalog = CollectionCatalog::get(opCtx);
 
         std::vector<NamespaceString> collectionsToDrop;
@@ -280,15 +347,13 @@ Status _dropDatabase(OperationContext* opCtx, const DatabaseName& dbName, bool a
             numCollections++;
 
             LOGV2(20338,
-                  "dropDatabase {dbName} - dropping collection: {nss}",
                   "dropDatabase - dropping collection",
                   logAttrs(dbName),
                   "namespace"_attr = nss);
 
-            if (nss.isDropPendingNamespace() && replCoord->isReplEnabled() &&
+            if (nss.isDropPendingNamespace() && replCoord->getSettings().isReplSet() &&
                 opCtx->writesAreReplicated()) {
                 LOGV2(20339,
-                      "dropDatabase {dbName} - found drop-pending collection: {nss}",
                       "dropDatabase - found drop-pending collection",
                       logAttrs(dbName),
                       "namespace"_attr = nss);
@@ -308,9 +373,9 @@ Status _dropDatabase(OperationContext* opCtx, const DatabaseName& dbName, bool a
                 // Dropping a database on a primary replicates individual collection drops followed
                 // by a database drop oplog entry. When a secondary observes the database drop oplog
                 // entry, all of the replicated collections that were dropped must have been
-                // processed. Only non-replicated collections like `system.profile` should be left
-                // to remove. Collections with the `tmp.mr` namespace may or may not be getting
-                // replicated; be conservative and assume they are not.
+                // processed. Only non-replicated collections should be left to remove. Collections
+                // with the `tmp.mr` namespace may or may not be getting replicated; be conservative
+                // and assume they are not.
                 invariant(!nss.isReplicated() || nss.coll().startsWith("tmp.mr"));
             }
 
@@ -319,12 +384,12 @@ Status _dropDatabase(OperationContext* opCtx, const DatabaseName& dbName, bool a
                     catalog->lookupCollectionByNamespace(opCtx, nss)->uuid());
             }
 
-            writeConflictRetry(opCtx, "dropDatabase_collection", nss.ns(), [&] {
+            writeConflictRetry(opCtx, "dropDatabase_collection", nss, [&] {
                 WriteUnitOfWork wunit(opCtx);
                 // A primary processing this will assign a timestamp when the operation is written
                 // to the oplog. As stated above, a secondary processing must only observe
                 // non-replicated collections, thus this should not be timestamped.
-                fassert(40476, db->dropCollectionEvenIfSystem(opCtx, nss));
+                fassert(40476, db->dropCollectionEvenIfSystem(opCtx, nss, {}, markFromMigrate));
                 wunit.commit();
             });
         }
@@ -335,7 +400,8 @@ Status _dropDatabase(OperationContext* opCtx, const DatabaseName& dbName, bool a
 
         // If there are no collection drops to wait for, we complete the drop database operation.
         if (numCollectionsToDrop == 0U && latestDropPendingOpTime.isNull()) {
-            _finishDropDatabase(opCtx, dbName, db, numCollections, abortIndexBuilds);
+            _finishDropDatabase(
+                opCtx, dbName, db, numCollections, abortIndexBuilds, markFromMigrate);
             return Status::OK();
         }
     }
@@ -344,8 +410,9 @@ Status _dropDatabase(OperationContext* opCtx, const DatabaseName& dbName, bool a
     // any errors while we await the replication of any collection drops and then reacquire the
     // locks (which can throw) needed to finish the drop database.
     ScopeGuard dropPendingGuardWhileUnlocked([dbName, opCtx] {
-        // TODO (SERVER-71610): Fix to be interruptible or document exception.
-        UninterruptibleLockGuard noInterrupt(opCtx->lockState());  // NOLINT.
+        // This scope guard must succeed in acquiring locks and reverting the drop pending state
+        // even when the failure is due to an interruption.
+        UninterruptibleLockGuard noInterrupt(opCtx);  // NOLINT.
 
         AutoGetDb autoDB(opCtx, dbName, MODE_IX);
         if (auto db = autoDB.getDb()) {
@@ -354,7 +421,7 @@ Status _dropDatabase(OperationContext* opCtx, const DatabaseName& dbName, bool a
     });
 
     // Verify again that we haven't obtained any other locks before replication.
-    invariant(!opCtx->lockState()->isLocked());
+    invariant(!shard_role_details::getLocker(opCtx)->isLocked());
 
     auto awaitOpTime = [&]() {
         if (numCollectionsToDrop > 0U) {
@@ -368,8 +435,8 @@ Status _dropDatabase(OperationContext* opCtx, const DatabaseName& dbName, bool a
     // The user-supplied wTimeout should be used when waiting for majority write concern.
     const auto& userWriteConcern = opCtx->getWriteConcern();
     const auto wTimeout = !userWriteConcern.isImplicitDefaultWriteConcern()
-        ? Milliseconds{userWriteConcern.wTimeout}
-        : duration_cast<Milliseconds>(Minutes(10));
+        ? userWriteConcern.wTimeout
+        : WriteConcernOptions::Timeout(duration_cast<Milliseconds>(Minutes(10)));
 
     // This is used to wait for the collection drops to replicate to a majority of the replica
     // set. Note: Even though we're setting UNSET here, kMajority implies JOURNAL if journaling
@@ -379,9 +446,6 @@ Status _dropDatabase(OperationContext* opCtx, const DatabaseName& dbName, bool a
         WriteConcernOptions::kMajority, WriteConcernOptions::SyncMode::UNSET, wTimeout);
 
     LOGV2(20340,
-          "dropDatabase {dbName} waiting for {awaitOpTime} to be replicated at "
-          "{dropDatabaseWriteConcern}. Dropping {numCollectionsToDrop} collection(s), with "
-          "last collection drop at {latestDropPendingOpTime}",
           "dropDatabase waiting for replication and dropping collections",
           logAttrs(dbName),
           "awaitOpTime"_attr = awaitOpTime,
@@ -394,8 +458,6 @@ Status _dropDatabase(OperationContext* opCtx, const DatabaseName& dbName, bool a
     // If the user-provided write concern is weaker than majority, this is effectively a no-op.
     if (result.status.isOK() && !userWriteConcern.usedDefaultConstructedWC) {
         LOGV2(20341,
-              "dropDatabase {dbName} waiting for {awaitOpTime} to be replicated at "
-              "{userWriteConcern}",
               "dropDatabase waiting for replication",
               logAttrs(dbName),
               "awaitOpTime"_attr = awaitOpTime,
@@ -412,8 +474,6 @@ Status _dropDatabase(OperationContext* opCtx, const DatabaseName& dbName, bool a
     }
 
     LOGV2(20342,
-          "dropDatabase {dbName} - successfully dropped {numCollectionsToDrop} collection(s) "
-          "(most recent drop optime: {awaitOpTime}) after {result_duration}. dropping database",
           "dropDatabase - successfully dropped collections",
           logAttrs(dbName),
           "numCollectionsDropped"_attr = numCollectionsToDrop,
@@ -428,7 +488,7 @@ Status _dropDatabase(OperationContext* opCtx, const DatabaseName& dbName, bool a
         dropDatabaseHangAfterAllCollectionsDrop.pauseWhileSet();
     }
 
-    AutoGetDb autoDB(opCtx, dbName, MODE_X);
+    AutoGetDb autoDB(opCtx, dbName, MODE_X /* database lock mode*/, tenantLockMode);
     auto db = autoDB.getDb();
     if (!db) {
         return Status(ErrorCodes::NamespaceNotFound,
@@ -437,8 +497,8 @@ Status _dropDatabase(OperationContext* opCtx, const DatabaseName& dbName, bool a
                                     << numCollectionsToDrop << " collection(s).");
     }
 
-    bool userInitiatedWritesAndNotPrimary = opCtx->writesAreReplicated() &&
-        !replCoord->canAcceptWritesForDatabase(opCtx, dbName.toStringWithTenantId());
+    bool userInitiatedWritesAndNotPrimary =
+        opCtx->writesAreReplicated() && !replCoord->canAcceptWritesForDatabase(opCtx, dbName);
 
     if (userInitiatedWritesAndNotPrimary) {
         return Status(ErrorCodes::PrimarySteppedDown,
@@ -452,21 +512,23 @@ Status _dropDatabase(OperationContext* opCtx, const DatabaseName& dbName, bool a
     // _finishDropDatabase creates its own scope guard to ensure drop-pending is unset.
     dropPendingGuardWhileUnlocked.dismiss();
 
-    _finishDropDatabase(opCtx, dbName, db, numCollections, abortIndexBuilds);
+    _finishDropDatabase(opCtx, dbName, db, numCollections, abortIndexBuilds, markFromMigrate);
 
     return Status::OK();
 }
 
 }  // namespace
 
-Status dropDatabase(OperationContext* opCtx, const DatabaseName& dbName) {
+Status dropDatabase(OperationContext* opCtx, const DatabaseName& dbName, bool markFromMigrate) {
     const bool abortIndexBuilds = true;
-    return _dropDatabase(opCtx, dbName, abortIndexBuilds);
+    return _dropDatabase(opCtx, dbName, abortIndexBuilds, markFromMigrate);
 }
 
-Status dropDatabaseForApplyOps(OperationContext* opCtx, const DatabaseName& dbName) {
+Status dropDatabaseForApplyOps(OperationContext* opCtx,
+                               const DatabaseName& dbName,
+                               bool markFromMigrate) {
     const bool abortIndexBuilds = false;
-    return _dropDatabase(opCtx, dbName, abortIndexBuilds);
+    return _dropDatabase(opCtx, dbName, abortIndexBuilds, markFromMigrate);
 }
 
 }  // namespace mongo
