@@ -30,27 +30,57 @@
 #include "mongo/db/catalog/drop_indexes.h"
 
 #include <boost/algorithm/string/join.hpp>
+#include <boost/cstdint.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+// IWYU pragma: no_include "ext/alloc_traits.h"
+#include <algorithm>
+#include <cstdint>
+#include <memory>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/db/catalog/clustered_collection_options_gen.h"
+#include "mongo/db/catalog/collection.h"
 #include "mongo/db/catalog/collection_catalog.h"
-#include "mongo/db/catalog/collection_uuid_mismatch.h"
 #include "mongo/db/catalog/index_catalog.h"
-#include "mongo/db/client.h"
+#include "mongo/db/catalog/index_catalog_entry.h"
+#include "mongo/db/catalog_raii.h"
 #include "mongo/db/concurrency/exception_util.h"
-#include "mongo/db/curop.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
+#include "mongo/db/database_name.h"
 #include "mongo/db/db_raii.h"
 #include "mongo/db/index/index_descriptor.h"
 #include "mongo/db/index_builds_coordinator.h"
 #include "mongo/db/op_observer/op_observer.h"
+#include "mongo/db/repl/repl_settings.h"
 #include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/repl_set_member_in_standalone_mode.h"
 #include "mongo/db/s/collection_sharding_state.h"
 #include "mongo/db/s/database_sharding_state.h"
+#include "mongo/db/s/scoped_collection_metadata.h"
 #include "mongo/db/s/shard_key_index_util.h"
 #include "mongo/db/server_feature_flags_gen.h"
+#include "mongo/db/server_options.h"
 #include "mongo/db/service_context.h"
+#include "mongo/db/storage/recovery_unit.h"
 #include "mongo/db/storage/storage_parameters_gen.h"
+#include "mongo/db/storage/write_unit_of_work.h"
+#include "mongo/db/transaction_resources.h"
+#include "mongo/idl/idl_parser.h"
 #include "mongo/logv2/log.h"
-#include "mongo/util/overloaded_visitor.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/fail_point.h"
+#include "mongo/util/overloaded_visitor.h"  // IWYU pragma: keep
+#include "mongo/util/str.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
 
@@ -68,9 +98,11 @@ Status checkView(OperationContext* opCtx,
     if (!collection) {
         if (CollectionCatalog::get(opCtx)->lookupView(opCtx, nss)) {
             return Status(ErrorCodes::CommandNotSupportedOnView,
-                          str::stream() << "Cannot drop indexes on view " << nss);
+                          str::stream()
+                              << "Cannot drop indexes on view " << nss.toStringForErrorMsg());
         }
-        return Status(ErrorCodes::NamespaceNotFound, str::stream() << "ns not found " << nss);
+        return Status(ErrorCodes::NamespaceNotFound,
+                      str::stream() << "ns not found " << nss.toStringForErrorMsg());
     }
     return Status::OK();
 }
@@ -90,7 +122,7 @@ Status checkReplState(OperationContext* opCtx,
     }
 
     // Disallow index drops on drop-pending namespaces (system.drop.*) if we are primary.
-    auto isPrimary = replCoord->getSettings().usingReplSets() && canAcceptWrites;
+    auto isPrimary = replCoord->getSettings().isReplSet() && canAcceptWrites;
     const auto& nss = collection->ns();
     if (isPrimary && nss.isDropPendingNamespace()) {
         return Status(ErrorCodes::NamespaceNotFound,
@@ -154,31 +186,31 @@ bool containsClusteredIndex(const CollectionPtr& collection, const IndexArgument
     invariant(collection && collection->isClustered());
 
     auto clusteredIndexSpec = collection->getClusteredInfo()->getIndexSpec();
-    return stdx::visit(
-        OverloadedVisitor{[&](const std::string& indexName) -> bool {
-                              // While the clusteredIndex's name is optional during user
-                              // creation, it should always be filled in by default on the
-                              // collection object.
-                              auto clusteredIndexName = clusteredIndexSpec.getName();
-                              invariant(clusteredIndexName.has_value());
+    return visit(OverloadedVisitor{[&](const std::string& indexName) -> bool {
+                                       // While the clusteredIndex's name is optional during user
+                                       // creation, it should always be filled in by default on the
+                                       // collection object.
+                                       auto clusteredIndexName = clusteredIndexSpec.getName();
+                                       invariant(clusteredIndexName.has_value());
 
-                              return clusteredIndexName.value() == indexName;
-                          },
-                          [&](const std::vector<std::string>& indexNames) -> bool {
-                              // While the clusteredIndex's name is optional during user
-                              // creation, it should always be filled in by default on the
-                              // collection object.
-                              auto clusteredIndexName = clusteredIndexSpec.getName();
-                              invariant(clusteredIndexName.has_value());
+                                       return clusteredIndexName.value() == indexName;
+                                   },
+                                   [&](const std::vector<std::string>& indexNames) -> bool {
+                                       // While the clusteredIndex's name is optional during user
+                                       // creation, it should always be filled in by default on the
+                                       // collection object.
+                                       auto clusteredIndexName = clusteredIndexSpec.getName();
+                                       invariant(clusteredIndexName.has_value());
 
-                              return std::find(indexNames.begin(),
-                                               indexNames.end(),
-                                               clusteredIndexName.value()) != indexNames.end();
-                          },
-                          [&](const BSONObj& indexKey) -> bool {
-                              return clusteredIndexSpec.getKey().woCompare(indexKey) == 0;
-                          }},
-        index);
+                                       return std::find(indexNames.begin(),
+                                                        indexNames.end(),
+                                                        clusteredIndexName.value()) !=
+                                           indexNames.end();
+                                   },
+                                   [&](const BSONObj& indexKey) -> bool {
+                                       return clusteredIndexSpec.getKey().woCompare(indexKey) == 0;
+                                   }},
+                 index);
 }
 
 /**
@@ -188,9 +220,10 @@ bool containsClusteredIndex(const CollectionPtr& collection, const IndexArgument
 StatusWith<std::vector<std::string>> getIndexNames(OperationContext* opCtx,
                                                    const CollectionPtr& collection,
                                                    const IndexArgument& index) {
-    invariant(opCtx->lockState()->isCollectionLockedForMode(collection->ns(), MODE_IX));
+    invariant(
+        shard_role_details::getLocker(opCtx)->isCollectionLockedForMode(collection->ns(), MODE_IX));
 
-    return stdx::visit(
+    return visit(
         OverloadedVisitor{
             [](const std::string& arg) -> StatusWith<std::vector<std::string>> { return {{arg}}; },
             [](const std::vector<std::string>& arg) -> StatusWith<std::vector<std::string>> {
@@ -237,14 +270,14 @@ Status dropIndexByDescriptor(OperationContext* opCtx,
     // Support dropping unfinished indexes, but only if the index is 'frozen'. These indexes only
     // exist in standalone mode.
     if (entry->isFrozen()) {
-        invariant(!entry->isReady(opCtx));
+        invariant(!entry->isReady());
         invariant(getReplSetMemberInStandaloneMode(opCtx->getServiceContext()));
         // Return here. No need to fall through to op observer on standalone.
         return indexCatalog->dropUnfinishedIndex(opCtx, collection, entry);
     }
 
     // Do not allow dropping unfinished indexes that are not frozen.
-    if (!entry->isReady(opCtx)) {
+    if (!entry->isReady()) {
         return Status(ErrorCodes::IndexNotFound,
                       str::stream() << "can't drop unfinished index with name: "
                                     << entry->descriptor()->indexName());
@@ -292,7 +325,8 @@ void dropReadyIndexes(OperationContext* opCtx,
                       const std::vector<std::string>& indexNames,
                       DropIndexesReply* reply,
                       bool forceDropShardKeyIndex) {
-    invariant(opCtx->lockState()->isCollectionLockedForMode(collection->ns(), MODE_X));
+    invariant(
+        shard_role_details::getLocker(opCtx)->isCollectionLockedForMode(collection->ns(), MODE_X));
 
     if (indexNames.empty()) {
         return;
@@ -390,12 +424,14 @@ void assertNoMovePrimaryInProgress(OperationContext* opCtx, const NamespaceStrin
         auto collDesc = scopedCss->getCollectionDescription(opCtx);
         collDesc.throwIfReshardingInProgress(nss);
 
-        if (!collDesc.isSharded()) {
+        // Only collections that are not registered in the sharding catalog are affected by
+        // movePrimary
+        if (!collDesc.hasRoutingTable()) {
             if (scopedDss->isMovePrimaryInProgress()) {
                 LOGV2(4976500, "assertNoMovePrimaryInProgress", logAttrs(nss));
 
                 uasserted(ErrorCodes::MovePrimaryInProgress,
-                          "movePrimary is in progress for namespace " + nss.toString());
+                          "movePrimary is in progress for namespace " + nss.toStringForErrorMsg());
             }
         }
     } catch (const DBException& ex) {
@@ -429,15 +465,14 @@ DropIndexesReply dropIndexes(OperationContext* opCtx,
               "CMD: dropIndexes",
               logAttrs(nss),
               "uuid"_attr = collectionUUID,
-              "indexes"_attr =
-                  stdx::visit(OverloadedVisitor{[](const std::string& arg) { return arg; },
-                                                [](const std::vector<std::string>& arg) {
-                                                    return boost::algorithm::join(arg, ",");
-                                                },
-                                                [](const BSONObj& arg) {
-                                                    return arg.toString();
-                                                }},
-                              index));
+              "indexes"_attr = visit(OverloadedVisitor{[](const std::string& arg) { return arg; },
+                                                       [](const std::vector<std::string>& arg) {
+                                                           return boost::algorithm::join(arg, ",");
+                                                       },
+                                                       [](const BSONObj& arg) {
+                                                           return arg.toString();
+                                                       }},
+                                     index));
     }
 
     if ((*collection)->isClustered() &&
@@ -448,8 +483,7 @@ DropIndexesReply dropIndexes(OperationContext* opCtx,
     DropIndexesReply reply;
     reply.setNIndexesWas((*collection)->getIndexCatalog()->numIndexesTotal());
 
-    const bool isWildcard =
-        stdx::holds_alternative<std::string>(index) && stdx::get<std::string>(index) == "*";
+    const bool isWildcard = holds_alternative<std::string>(index) && get<std::string>(index) == "*";
 
     IndexBuildsCoordinator* indexBuildsCoord = IndexBuildsCoordinator::get(opCtx);
 
@@ -482,7 +516,7 @@ DropIndexesReply dropIndexes(OperationContext* opCtx,
 
         // Abandon the snapshot as the index catalog will compare the in-memory state to the
         // disk state, which may have changed when we released the lock temporarily.
-        opCtx->recoveryUnit()->abandonSnapshot();
+        shard_role_details::getRecoveryUnit(opCtx)->abandonSnapshot();
 
         // Take an exclusive lock on the collection now to be able to perform index catalog
         // writes when removing ready indexes from disk.
@@ -522,7 +556,7 @@ DropIndexesReply dropIndexes(OperationContext* opCtx,
         // The index catalog requires that no active index builders are running when dropping ready
         // indexes.
         IndexBuildsCoordinator::get(opCtx)->assertNoIndexBuildInProgForCollection(collectionUUID);
-        writeConflictRetry(opCtx, "dropIndexes", dbAndUUID.toString(), [&] {
+        writeConflictRetry(opCtx, "dropIndexes", dbAndUUID, [&] {
             WriteUnitOfWork wuow(opCtx);
 
             // This is necessary to check shard version.
@@ -575,16 +609,15 @@ DropIndexesReply dropIndexes(OperationContext* opCtx,
         invariant((*collection)->getIndexCatalog()->numIndexesInProgress() == 0);
     }
 
-    writeConflictRetry(
-        opCtx, "dropIndexes", dbAndUUID.toString(), [opCtx, &collection, &indexNames, &reply] {
-            WriteUnitOfWork wunit(opCtx);
+    writeConflictRetry(opCtx, "dropIndexes", dbAndUUID, [opCtx, &collection, &indexNames, &reply] {
+        WriteUnitOfWork wunit(opCtx);
 
-            // This is necessary to check shard version.
-            OldClientContext ctx(opCtx, (*collection)->ns());
-            dropReadyIndexes(
-                opCtx, collection->getWritableCollection(opCtx), indexNames, &reply, false);
-            wunit.commit();
-        });
+        // This is necessary to check shard version.
+        OldClientContext ctx(opCtx, (*collection)->ns());
+        dropReadyIndexes(
+            opCtx, collection->getWritableCollection(opCtx), indexNames, &reply, false);
+        wunit.commit();
+    });
 
     return reply;
 }
@@ -593,12 +626,16 @@ Status dropIndexesForApplyOps(OperationContext* opCtx,
                               const NamespaceString& nss,
                               const BSONObj& cmdObj) try {
     BSONObjBuilder bob(cmdObj);
-    bob.append("$db", nss.dbName().db());
+    bob.append("$db", nss.dbName().serializeWithoutTenantPrefix_UNSAFE());
     auto cmdObjWithDb = bob.obj();
-    auto parsed = DropIndexes::parse(
-        IDLParserContext{"dropIndexes", false /* apiStrict */, nss.tenantId()}, cmdObjWithDb);
+    auto parsed = DropIndexes::parse(IDLParserContext{"dropIndexes",
+                                                      false /* apiStrict */,
+                                                      auth::ValidatedTenancyScope::get(opCtx),
+                                                      nss.tenantId(),
+                                                      SerializationContext::stateStorageRequest()},
+                                     cmdObjWithDb);
 
-    return writeConflictRetry(opCtx, "dropIndexes", nss.db(), [opCtx, &nss, &cmdObj, &parsed] {
+    return writeConflictRetry(opCtx, "dropIndexes", nss, [opCtx, &nss, &cmdObj, &parsed] {
         AutoGetCollection collection(opCtx, nss, MODE_X);
 
         // If db/collection does not exist, short circuit and return.
