@@ -28,24 +28,41 @@
  */
 
 
-#include "mongo/db/catalog/multi_index_block.h"
-#include "mongo/platform/basic.h"
+#include <boost/move/utility_core.hpp>
+#include <mutex>
+#include <string>
+#include <type_traits>
 
-#include "mongo/db/catalog/index_builds_manager.h"
+#include <boost/optional/optional.hpp>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/bson/bson_validate.h"
 #include "mongo/db/catalog/collection.h"
 #include "mongo/db/catalog/collection_catalog.h"
-#include "mongo/db/catalog/index_catalog.h"
+#include "mongo/db/catalog/index_builds_manager.h"
 #include "mongo/db/catalog/index_repair.h"
+#include "mongo/db/catalog/multi_index_block.h"
 #include "mongo/db/catalog_raii.h"
+#include "mongo/db/client.h"
 #include "mongo/db/concurrency/exception_util.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
+#include "mongo/db/curop.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/operation_context.h"
+#include "mongo/db/storage/record_data.h"
+#include "mongo/db/storage/record_store.h"
+#include "mongo/db/storage/storage_parameters_gen.h"
 #include "mongo/db/storage/storage_repair_observer.h"
 #include "mongo/db/storage/write_unit_of_work.h"
+#include "mongo/db/transaction_resources.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/redaction.h"
+#include "mongo/platform/atomic_word.h"
 #include "mongo/util/assert_util.h"
 #include "mongo/util/progress_meter.h"
+#include "mongo/util/scopeguard.h"
 #include "mongo/util/str.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
@@ -92,9 +109,9 @@ Status IndexBuildsManager::setUpIndexBuild(OperationContext* opCtx,
     _registerIndexBuild(buildUUID);
 
     const auto& nss = collection->ns();
-    invariant(opCtx->lockState()->isCollectionLockedForMode(nss, MODE_X),
+    invariant(shard_role_details::getLocker(opCtx)->isCollectionLockedForMode(nss, MODE_X),
               str::stream() << "Unable to set up index build " << buildUUID << ": collection "
-                            << nss.ns() << " is not locked in exclusive mode.");
+                            << nss.toStringForErrorMsg() << " is not locked in exclusive mode.");
 
     auto builder = invariant(_getBuilder(buildUUID));
     if (options.protocol == IndexBuildProtocol::kTwoPhase) {
@@ -115,7 +132,7 @@ Status IndexBuildsManager::setUpIndexBuild(OperationContext* opCtx,
 
     std::vector<BSONObj> indexes;
     try {
-        indexes = writeConflictRetry(opCtx, "IndexBuildsManager::setUpIndexBuild", nss.ns(), [&]() {
+        indexes = writeConflictRetry(opCtx, "IndexBuildsManager::setUpIndexBuild", nss, [&]() {
             MultiIndexBlock::InitMode mode = options.forRecovery
                 ? MultiIndexBlock::InitMode::Recovery
                 : MultiIndexBlock::InitMode::SteadyState;
@@ -172,7 +189,7 @@ StatusWith<std::pair<long long, long long>> IndexBuildsManager::startBuildingInd
         opCtx->checkForInterrupt();
         // Cursor is left one past the end of the batch inside writeConflictRetry
         auto beginBatchId = record->id;
-        Status status = writeConflictRetry(opCtx, "repairDatabase", ns.ns(), [&] {
+        Status status = writeConflictRetry(opCtx, "repairDatabase", ns, [&] {
             // In the case of WCE in a partial batch, we need to go back to the beginning
             if (!record || (beginBatchId != record->id)) {
                 record = cursor->seekExact(beginBatchId);
@@ -186,13 +203,11 @@ StatusWith<std::pair<long long, long long>> IndexBuildsManager::startBuildingInd
                 if (!validStatus.isOK()) {
                     if (repair == RepairData::kNo) {
                         LOGV2_FATAL(31396,
-                                    "Invalid BSON detected at {id}: {validStatus}",
                                     "Invalid BSON detected",
                                     "id"_attr = id,
                                     "error"_attr = redact(validStatus));
                     }
                     LOGV2_WARNING(20348,
-                                  "Invalid BSON detected at {id}: {validStatus}. Deleting.",
                                   "Invalid BSON detected; deleting",
                                   "id"_attr = id,
                                   "error"_attr = redact(validStatus));
@@ -216,7 +231,7 @@ StatusWith<std::pair<long long, long long>> IndexBuildsManager::startBuildingInd
                             writeConflictRetry(
                                 opCtx,
                                 "insertSingleDocumentForInitialSyncOrRecovery-restoreCursor",
-                                ns.ns(),
+                                ns,
                                 [&cursor] { cursor->restore(); });
                         });
                     if (!insertStatus.isOK()) {
@@ -239,7 +254,7 @@ StatusWith<std::pair<long long, long long>> IndexBuildsManager::startBuildingInd
             ON_BLOCK_EXIT([opCtx, ns, &cursor]() {
                 // restore CAN throw WCE per API
                 writeConflictRetry(
-                    opCtx, "retryRestoreCursor", ns.ns(), [&cursor] { cursor->restore(); });
+                    opCtx, "retryRestoreCursor", ns, [&cursor] { cursor->restore(); });
             });
             wunit.commit();
             return Status::OK();
@@ -282,9 +297,9 @@ StatusWith<std::pair<long long, long long>> IndexBuildsManager::startBuildingInd
 
     if (recordsRemoved > 0) {
         StorageRepairObserver::get(opCtx->getServiceContext())
-            ->invalidatingModification(str::stream()
-                                       << "Moved " << recordsRemoved
-                                       << " records to lost and found: " << lostAndFoundNss.ns());
+            ->invalidatingModification(str::stream() << "Moved " << recordsRemoved
+                                                     << " records to lost and found: "
+                                                     << toStringForLogging(lostAndFoundNss));
 
         LOGV2(3956200,
               "Moved records to lost and found.",
@@ -336,7 +351,7 @@ Status IndexBuildsManager::commitIndexBuild(OperationContext* opCtx,
     return writeConflictRetry(
         opCtx,
         "IndexBuildsManager::commitIndexBuild",
-        nss.ns(),
+        nss,
         [this, builder, buildUUID, opCtx, &collection, nss, &onCreateEachFn, &onCommitFn] {
             WriteUnitOfWork wunit(opCtx);
             auto status = builder->commit(
