@@ -29,17 +29,30 @@
 
 #pragma once
 
+#include <boost/optional/optional.hpp>
+#include <cstdint>
+#include <functional>
 #include <memory>
+#include <string>
+#include <type_traits>
 #include <vector>
 
 #include "mongo/base/clonable_ptr.h"
+#include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/timestamp.h"
 #include "mongo/db/catalog/clustered_collection_options_gen.h"
 #include "mongo/db/catalog/index_catalog_entry.h"
 #include "mongo/db/index/multikey_paths.h"
 #include "mongo/db/jsobj.h"
+#include "mongo/db/namespace_string.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/record_id.h"
+#include "mongo/db/resumable_index_builds_gen.h"
 #include "mongo/db/server_options.h"
+#include "mongo/db/storage/key_string.h"
 #include "mongo/db/storage/record_store.h"
 
 namespace mongo {
@@ -47,8 +60,8 @@ namespace mongo {
 class Client;
 class Collection;
 class CollectionPtr;
-
 class IndexDescriptor;
+
 struct InsertDeleteOptions;
 
 struct BsonRecord {
@@ -186,15 +199,15 @@ public:
          * it should pass in a null value.
          */
         AllIndexesIterator(OperationContext* opCtx,
-                           std::unique_ptr<std::vector<IndexCatalogEntry*>> ownedContainer);
+                           std::unique_ptr<std::vector<const IndexCatalogEntry*>> ownedContainer);
 
     private:
         const IndexCatalogEntry* _advance() override;
 
         OperationContext* const _opCtx;
-        std::vector<IndexCatalogEntry*>::const_iterator _iterator;
-        std::vector<IndexCatalogEntry*>::const_iterator _endIterator;
-        std::unique_ptr<std::vector<IndexCatalogEntry*>> _ownedContainer;
+        std::vector<const IndexCatalogEntry*>::const_iterator _iterator;
+        std::vector<const IndexCatalogEntry*>::const_iterator _endIterator;
+        std::unique_ptr<std::vector<const IndexCatalogEntry*>> _ownedContainer;
     };
 
     enum class InclusionPolicy {
@@ -215,7 +228,9 @@ public:
     virtual std::unique_ptr<IndexCatalog> clone() const = 0;
 
     // Must be called before used.
-    virtual Status init(OperationContext* opCtx, Collection* collection) = 0;
+    virtual void init(OperationContext* opCtx,
+                      Collection* collection,
+                      bool isPointInTimeRead = false) = 0;
 
     // ---- accessors -----
 
@@ -223,12 +238,15 @@ public:
 
     virtual bool haveAnyIndexesInProgress() const = 0;
 
-    virtual int numIndexesTotal(OperationContext* opCtx) const = 0;
+    virtual int numIndexesTotal() const = 0;
 
-    virtual int numIndexesReady(OperationContext* opCtx) const = 0;
+    virtual int numIndexesReady() const = 0;
 
-    virtual int numIndexesInProgress(OperationContext* opCtx) const = 0;
+    virtual int numIndexesInProgress() const = 0;
 
+    /**
+     * Returns true if the _id index exists.
+     */
     virtual bool haveIdIndex(OperationContext* opCtx) const = 0;
 
     /**
@@ -276,6 +294,16 @@ public:
         std::vector<const IndexDescriptor*>& matches,
         InclusionPolicy inclusionPolicy = InclusionPolicy::kReady) const = 0;
 
+    /**
+     * Finds the index with the given ident. The ident uniquely identifies an index.
+     *
+     * Returns nullptr if the index is not found.
+     */
+    virtual const IndexDescriptor* findIndexByIdent(
+        OperationContext* opCtx,
+        StringData ident,
+        InclusionPolicy inclusionPolicy = InclusionPolicy::kReady) const = 0;
+
     /**
      * Reload the index definition for 'oldDesc' from the CollectionCatalogEntry.  'oldDesc'
      * must be a ready index that is already registered with the index catalog.  Returns an
@@ -299,6 +327,21 @@ public:
      */
     virtual const IndexCatalogEntry* getEntry(const IndexDescriptor* desc) const = 0;
 
+    /**
+     * Returns a writable IndexCatalogEntry copy that will be returned by current and future calls
+     * to this function. Any previous IndexCatalogEntry/IndexDescriptor pointers that were returned
+     * may be invalidated.
+     */
+    virtual IndexCatalogEntry* getWritableEntryByName(
+        OperationContext* opCtx,
+        StringData name,
+        InclusionPolicy inclusionPolicy = InclusionPolicy::kReady) = 0;
+    virtual IndexCatalogEntry* getWritableEntryByKeyPatternAndOptions(
+        OperationContext* opCtx,
+        const BSONObj& key,
+        const BSONObj& indexSpec,
+        InclusionPolicy inclusionPolicy = InclusionPolicy::kReady) = 0;
+
     /**
      * Returns a pointer to the index catalog entry associated with 'desc', where the caller assumes
      * shared ownership of the entry. Returns null if the entry does not exist.
@@ -327,7 +370,7 @@ public:
 
     virtual IndexCatalogEntry* createIndexEntry(OperationContext* opCtx,
                                                 Collection* collection,
-                                                std::unique_ptr<IndexDescriptor> descriptor,
+                                                IndexDescriptor&& descriptor,
                                                 CreateIndexEntryFlags flags) = 0;
 
     /**
@@ -382,11 +425,15 @@ public:
      *
      * This should only be used when we are confident in the specs, such as when specs are received
      * via replica set cloning or chunk migrations.
+     *
+     * 'removeInProgressIndexBuilds' controls whether in-progress index builds are also filtered
+     * out.
      */
     virtual std::vector<BSONObj> removeExistingIndexesNoChecks(
         OperationContext* opCtx,
         const CollectionPtr& collection,
-        const std::vector<BSONObj>& indexSpecsToBuild) const = 0;
+        const std::vector<BSONObj>& indexSpecsToBuild,
+        bool removeInProgressIndexBuilds = true) const = 0;
 
     /**
      * Drops indexes in the index catalog that returns true when it's descriptor returns true for
@@ -408,16 +455,6 @@ public:
                                 bool includingIdIndex,
                                 std::function<void(const IndexDescriptor*)> onDropFn) = 0;
 
-    /**
-     * Drops the index given its descriptor.
-     *
-     * The caller must hold the collection X lock and ensure no index builds are in progress on the
-     * collection.
-     */
-    virtual Status dropIndex(OperationContext* opCtx,
-                             Collection* collection,
-                             const IndexDescriptor* desc) = 0;
-
     /**
      * Resets the index given its descriptor.
      *
@@ -426,7 +463,7 @@ public:
      */
     virtual Status resetUnfinishedIndexForRecovery(OperationContext* opCtx,
                                                    Collection* collection,
-                                                   const IndexDescriptor* desc) = 0;
+                                                   IndexCatalogEntry* entry) = 0;
 
     /**
      * Drops an unfinished index given its descriptor.
@@ -435,7 +472,7 @@ public:
      */
     virtual Status dropUnfinishedIndex(OperationContext* opCtx,
                                        Collection* collection,
-                                       const IndexDescriptor* desc) = 0;
+                                       IndexCatalogEntry* entry) = 0;
 
     /**
      * Drops the index given its catalog entry.
@@ -482,6 +519,9 @@ public:
     /**
      * Both 'keysInsertedOut' and 'keysDeletedOut' are required and will be set to the number of
      * index keys inserted and deleted by this operation, respectively.
+     * The 'opDiff' argument specifies an optional document containing the differences between
+     * 'oldDoc' and 'newDoc' that can be used to decide which indexes have to be modified. If
+     * set to null, all indexes should be updated.
      *
      * This method may throw.
      */
@@ -489,6 +529,7 @@ public:
                                 const CollectionPtr& coll,
                                 const BSONObj& oldDoc,
                                 const BSONObj& newDoc,
+                                const BSONObj* opDiff,
                                 const RecordId& recordId,
                                 int64_t* keysInsertedOut,
                                 int64_t* keysDeletedOut) const = 0;
@@ -508,8 +549,10 @@ public:
     /*
      * Attempt compaction on all ready indexes to regain disk space, if the storage engine's index
      * supports compaction in-place.
+     * Returns an estimated number of bytes when doing a dry run.
      */
-    virtual Status compactIndexes(OperationContext* opCtx) const = 0;
+    virtual StatusWith<int64_t> compactIndexes(OperationContext* opCtx,
+                                               const CompactOptions& options) const = 0;
 
     virtual std::string getAccessMethodName(const BSONObj& keyPattern) = 0;
 
@@ -538,6 +581,24 @@ public:
     virtual void indexBuildSuccess(OperationContext* opCtx,
                                    Collection* coll,
                                    IndexCatalogEntry* index) = 0;
+
+    /**
+     * Helper function which normalizes index specs. This function will populate a complete
+     * collation spec in cases where the index spec specifies a collation, and will add the
+     * collection-default collation, if present, in cases where collation is omitted. If the index
+     * spec omits the collation and the collection does not have a default, the collation field is
+     * omitted from the spec.
+     *
+     * If 'collection' is null, no changes are made to the input specs.
+     *
+     * This function throws on error.
+     */
+    static BSONObj normalizeIndexSpecs(OperationContext* opCtx,
+                                       const CollectionPtr& collection,
+                                       const BSONObj& indexSpec);
+    static std::vector<BSONObj> normalizeIndexSpecs(OperationContext* opCtx,
+                                                    const CollectionPtr& collection,
+                                                    const std::vector<BSONObj>& indexSpecs);
 };
 
 inline IndexCatalog::InclusionPolicy operator|(IndexCatalog::InclusionPolicy lhs,
