@@ -27,42 +27,65 @@
  *    it in the license file.
  */
 
-
-#include "mongo/platform/basic.h"
-
 #include "mongo/db/catalog/index_catalog_entry_impl.h"
 
 #include <algorithm>
-#include <memory>
+#include <boost/container/flat_set.hpp>
+#include <boost/container/vector.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <cstddef>
+#include <utility>
 
-#include "mongo/base/init.h"
+#include "mongo/base/error_codes.h"
+#include "mongo/base/init.h"  // IWYU pragma: keep
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/timestamp.h"
 #include "mongo/db/catalog/collection.h"
-#include "mongo/db/concurrency/d_concurrency.h"
+#include "mongo/db/client.h"
 #include "mongo/db/concurrency/exception_util.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
 #include "mongo/db/index/index_access_method.h"
 #include "mongo/db/index/index_descriptor.h"
 #include "mongo/db/matcher/expression.h"
 #include "mongo/db/matcher/expression_parser.h"
+#include "mongo/db/matcher/extensions_callback_noop.h"
 #include "mongo/db/multi_key_path_tracker.h"
 #include "mongo/db/op_observer/op_observer.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/pipeline/expression_context.h"
 #include "mongo/db/query/collation/collator_factory_interface.h"
 #include "mongo/db/query/collection_query_info.h"
+#include "mongo/db/repl/optime.h"
 #include "mongo/db/service_context.h"
 #include "mongo/db/storage/durable_catalog.h"
+#include "mongo/db/storage/recovery_unit.h"
+#include "mongo/db/storage/storage_engine.h"
+#include "mongo/db/storage/write_unit_of_work.h"
+#include "mongo/db/timeseries/timeseries_gen.h"
 #include "mongo/db/timeseries/timeseries_index_schema_conversion_functions.h"
 #include "mongo/db/transaction/transaction_participant.h"
+#include "mongo/db/transaction_resources.h"
 #include "mongo/logv2/log.h"
-#include "mongo/util/scopeguard.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/redaction.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/fail_point.h"
+#include "mongo/util/namespace_string_util.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kIndex
 
-
 namespace mongo {
-MONGO_FAIL_POINT_DEFINE(skipUpdateIndexMultikey);
 
-using std::string;
+MONGO_FAIL_POINT_DEFINE(skipUpdateIndexMultikey);
 
 IndexCatalogEntryImpl::IndexCatalogEntryImpl(OperationContext* const opCtx,
                                              const CollectionPtr& collection,
@@ -74,7 +97,6 @@ IndexCatalogEntryImpl::IndexCatalogEntryImpl(OperationContext* const opCtx,
       _isReady(false),
       _isFrozen(isFrozen),
       _shouldValidateDocument(false),
-      _isDropped(false),
       _indexOffset(invariantStatusOK(
           collection->checkMetaDataForIndex(_descriptor.indexName(), _descriptor.infoObj()))) {
     _descriptor._entry = this;
@@ -127,27 +149,6 @@ void IndexCatalogEntryImpl::setAccessMethod(std::unique_ptr<IndexAccessMethod> a
         this, _shared->_accessMethod.get(), &_shared->_indexedPaths);
 }
 
-bool IndexCatalogEntryImpl::isReady(OperationContext* opCtx) const {
-    // For multi-document transactions, we can open a snapshot prior to checking the
-    // minimumSnapshotVersion on a collection.  This means we are unprotected from reading
-    // out-of-sync index catalog entries.  To fix this, we uassert if we detect that the
-    // in-memory catalog is out-of-sync with the on-disk catalog. This check is not necessary when
-    // point-in-time catalog lookups are enabled as the snapshot is always in sync.
-    // (Ignore FCV check): This feature flag doesn't have any upgrade/downgrade concerns.
-    if (opCtx->inMultiDocumentTransaction() &&
-        !feature_flags::gPointInTimeCatalogLookups.isEnabledAndIgnoreFCVUnsafe()) {
-        if (!isPresentInMySnapshot(opCtx) || isReadyInMySnapshot(opCtx) != _isReady) {
-            uasserted(ErrorCodes::SnapshotUnavailable,
-                      str::stream() << "Unable to read from a snapshot due to pending collection"
-                                       " catalog changes; please retry the operation.");
-        }
-    }
-
-    if (kDebugBuild)
-        invariant(_isReady == isReadyInMySnapshot(opCtx));
-    return _isReady;
-}
-
 bool IndexCatalogEntryImpl::isFrozen() const {
     invariant(!_isFrozen || !_isReady);
     return _isFrozen;
@@ -172,12 +173,6 @@ MultikeyPaths IndexCatalogEntryImpl::getMultikeyPaths(OperationContext* opCtx,
 
 // ---
 
-void IndexCatalogEntryImpl::setMinimumVisibleSnapshot(Timestamp newMinimumVisibleSnapshot) {
-    if (!_minVisibleSnapshot || (newMinimumVisibleSnapshot > _minVisibleSnapshot.value())) {
-        _minVisibleSnapshot = newMinimumVisibleSnapshot;
-    }
-}
-
 void IndexCatalogEntryImpl::setIsReady(bool newIsReady) {
     _isReady = newIsReady;
 }
@@ -285,7 +280,7 @@ void IndexCatalogEntryImpl::forceSetMultikey(OperationContext* const opCtx,
                                              const CollectionPtr& coll,
                                              bool isMultikey,
                                              const MultikeyPaths& multikeyPaths) const {
-    invariant(opCtx->lockState()->isCollectionLockedForMode(coll->ns(), MODE_X));
+    invariant(shard_role_details::getLocker(opCtx)->isCollectionLockedForMode(coll->ns(), MODE_X));
 
     // Don't check _indexTracksMultikeyPathsInCatalog because the caller may be intentionally trying
     // to bypass this check. That is, pre-3.4 indexes may be 'stuck' in a state where they are not
@@ -316,12 +311,13 @@ Status IndexCatalogEntryImpl::_setMultikeyInMultiDocumentTransaction(
     // If the index is not visible within the side transaction, the index may have been created,
     // but not committed, in the parent transaction. Therefore, we abandon the side transaction
     // and set the multikey flag in the parent transaction.
-    if (!isPresentInMySnapshot(opCtx)) {
+    if (!DurableCatalog::get(opCtx)->isIndexPresent(
+            opCtx, _shared->_catalogId, _descriptor.indexName())) {
         return {ErrorCodes::SnapshotUnavailable, "index not visible in side transaction"};
     }
 
     writeConflictRetry(
-        opCtx, "set index multikey", collection->ns().ns(), [&] {
+        opCtx, "set index multikey", collection->ns(), [&] {
             WriteUnitOfWork wuow(opCtx);
 
             // If we have a prepare optime for recovery, then we always use that. This is safe since
@@ -351,7 +347,7 @@ Status IndexCatalogEntryImpl::_setMultikeyInMultiDocumentTransaction(
                 // document that enables multikey hasn't enabled it yet but is present in the
                 // collection. In other words, the index is not set for multikey but there is
                 // already data present that relies on it.
-                auto status = opCtx->recoveryUnit()->setTimestamp(std::max(
+                auto status = shard_role_details::getRecoveryUnit(opCtx)->setTimestamp(std::max(
                     {recoveryPrepareOpTime.getTimestamp(),
                      opCtx->getServiceContext()->getStorageEngine()->getOldestTimestamp(),
                      opCtx->getServiceContext()->getStorageEngine()->getStableTimestamp() + 1}));
@@ -363,8 +359,9 @@ Status IndexCatalogEntryImpl::_setMultikeyInMultiDocumentTransaction(
 
                 auto msg = BSON("msg"
                                 << "Setting index to multikey"
-                                << "coll" << collection->ns().ns() << "index"
-                                << _descriptor.indexName());
+                                << "coll"
+                                << NamespaceStringUtil::serializeForCatalog(collection->ns())
+                                << "index" << _descriptor.indexName());
                 opCtx->getClient()->getServiceContext()->getOpObserver()->onOpMessage(opCtx, msg);
             }
 
@@ -390,22 +387,168 @@ void IndexCatalogEntryImpl::setIdent(std::shared_ptr<Ident> newIdent) {
     _shared->_accessMethod->setIdent(std::move(newIdent));
 }
 
+namespace {
+
+class NormalizedIndexCatalogEntry : public IndexCatalogEntry {
+public:
+    NormalizedIndexCatalogEntry(OperationContext* opCtx,
+                                const CollectionPtr& collection,
+                                const IndexCatalogEntry* entry)
+        : IndexCatalogEntry(),
+          _original(entry),
+          _indexDescriptor([&] {
+              auto desc = entry->descriptor();
+              auto normalizedSpec =
+                  IndexCatalog::normalizeIndexSpecs(opCtx, collection, desc->infoObj());
+              return IndexDescriptor{desc->getAccessMethodName(), std::move(normalizedSpec)};
+          }()),
+          _collator([&]() -> std::unique_ptr<CollatorInterface> {
+              const auto& collation = _indexDescriptor.collation();
+              if (!entry->getCollator() && !collation.isEmpty()) {
+                  auto statusWithCollator =
+                      CollatorFactoryInterface::get(opCtx->getServiceContext())
+                          ->makeFromBSON(collation);
+
+                  invariantStatusOK(statusWithCollator.getStatus());
+                  // Index spec should have already been validated.
+                  return std::move(statusWithCollator.getValue());
+              }
+              return nullptr;
+          }()) {}
+
+    NormalizedIndexCatalogEntry(const NormalizedIndexCatalogEntry& other)
+        : IndexCatalogEntry(other),
+          _original(other._original),
+          _indexDescriptor(other._indexDescriptor),
+          _collator(other._collator->clone()) {}
+
+    const std::string& getIdent() const final {
+        return _original->getIdent();
+    }
+
+    std::shared_ptr<Ident> getSharedIdent() const final {
+        return _original->getSharedIdent();
+    }
+
+    void setIdent(std::shared_ptr<Ident> newIdent) final {
+        MONGO_UNREACHABLE;
+    }
+
+    IndexDescriptor* descriptor() final {
+        MONGO_UNREACHABLE;
+    }
+
+    const IndexDescriptor* descriptor() const final {
+        return &_indexDescriptor;
+    }
+
+    IndexAccessMethod* accessMethod() const final {
+        return _original->accessMethod();
+    }
+
+    void setAccessMethod(std::unique_ptr<IndexAccessMethod> accessMethod) final {
+        MONGO_UNREACHABLE;
+    }
+
+    bool isHybridBuilding() const final {
+        return _original->isHybridBuilding();
+    }
+
+    IndexBuildInterceptor* indexBuildInterceptor() const final {
+        return _original->indexBuildInterceptor();
+    }
+
+    void setIndexBuildInterceptor(IndexBuildInterceptor* interceptor) final {
+        MONGO_UNREACHABLE;
+    }
+
+    const Ordering& ordering() const final {
+        return _original->ordering();
+    }
+
+    const MatchExpression* getFilterExpression() const final {
+        return _original->getFilterExpression();
+    }
+
+    const CollatorInterface* getCollator() const final {
+        return _collator ? _collator.get() : _original->getCollator();
+    }
+
+    NamespaceString getNSSFromCatalog(OperationContext* opCtx) const final {
+        return _original->getNSSFromCatalog(opCtx);
+    }
+
+    void setIsReady(bool newIsReady) final {
+        MONGO_UNREACHABLE;
+    }
+
+    void setIsFrozen(bool newIsFrozen) final {
+        MONGO_UNREACHABLE;
+    }
+
+    bool isMultikey(OperationContext* opCtx, const CollectionPtr& collection) const final {
+        return _original->isMultikey(opCtx, collection);
+    }
+
+    MultikeyPaths getMultikeyPaths(OperationContext* opCtx,
+                                   const CollectionPtr& collection) const final {
+        return _original->getMultikeyPaths(opCtx, collection);
+    }
+
+    void setMultikey(OperationContext* opCtx,
+                     const CollectionPtr& coll,
+                     const KeyStringSet& multikeyMetadataKeys,
+                     const MultikeyPaths& multikeyPaths) const final {
+        return _original->setMultikey(opCtx, coll, multikeyMetadataKeys, multikeyPaths);
+    }
+
+    void forceSetMultikey(OperationContext* opCtx,
+                          const CollectionPtr& coll,
+                          bool isMultikey,
+                          const MultikeyPaths& multikeyPaths) const final {
+        return _original->forceSetMultikey(opCtx, coll, isMultikey, multikeyPaths);
+    }
+
+    bool isReady() const final {
+        return _original->isReady();
+    }
+
+    bool isFrozen() const final {
+        return _original->isFrozen();
+    }
+
+    bool shouldValidateDocument() const final {
+        return _original->shouldValidateDocument();
+    }
+
+    const UpdateIndexData& getIndexedPaths() const final {
+        return _original->getIndexedPaths();
+    }
+
+    std::unique_ptr<const IndexCatalogEntry> getNormalizedEntry(
+        OperationContext* opCtx, const CollectionPtr& coll) const final {
+        return std::make_unique<NormalizedIndexCatalogEntry>(*this);
+    };
+
+private:
+    const IndexCatalogEntry* _original;
+    IndexDescriptor _indexDescriptor;
+    std::unique_ptr<CollatorInterface> _collator;
+};
+
+}  // namespace
+
+std::unique_ptr<const IndexCatalogEntry> IndexCatalogEntryImpl::getNormalizedEntry(
+    OperationContext* opCtx, const CollectionPtr& coll) const {
+    return std::make_unique<NormalizedIndexCatalogEntry>(opCtx, coll, this);
+}
+
 // ----
 
 NamespaceString IndexCatalogEntryImpl::getNSSFromCatalog(OperationContext* opCtx) const {
     return DurableCatalog::get(opCtx)->getNSSFromCatalog(opCtx, _shared->_catalogId);
 }
 
-bool IndexCatalogEntryImpl::isReadyInMySnapshot(OperationContext* opCtx) const {
-    return DurableCatalog::get(opCtx)->isIndexReady(
-        opCtx, _shared->_catalogId, _descriptor.indexName());
-}
-
-bool IndexCatalogEntryImpl::isPresentInMySnapshot(OperationContext* opCtx) const {
-    return DurableCatalog::get(opCtx)->isIndexPresent(
-        opCtx, _shared->_catalogId, _descriptor.indexName());
-}
-
 bool IndexCatalogEntryImpl::_catalogIsMultikey(OperationContext* opCtx,
                                                const CollectionPtr& collection,
                                                MultikeyPaths* multikeyPaths) const {
