@@ -29,22 +29,40 @@
 
 #include "mongo/db/catalog/index_catalog_impl.h"
 
+#include <boost/move/utility_core.hpp>
+#include <boost/numeric/conversion/converter_policies.hpp>
+#include <boost/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <cstddef>
+#include <initializer_list>
+#include <utility>
 #include <vector>
 
-#include "mongo/base/init.h"
-#include "mongo/bson/simple_bsonelement_comparator.h"
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/init.h"  // IWYU pragma: keep
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
 #include "mongo/bson/simple_bsonobj_comparator.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/db/aggregated_index_usage_tracker.h"
 #include "mongo/db/audit.h"
+#include "mongo/db/basic_types_gen.h"
+#include "mongo/db/catalog/clustered_collection_options_gen.h"
 #include "mongo/db/catalog/clustered_collection_util.h"
 #include "mongo/db/catalog/collection.h"
+#include "mongo/db/catalog/collection_catalog.h"
 #include "mongo/db/catalog/index_build_block.h"
 #include "mongo/db/catalog/index_catalog_entry_impl.h"
 #include "mongo/db/catalog/index_key_validate.h"
 #include "mongo/db/catalog/uncommitted_catalog_updates.h"
-#include "mongo/db/client.h"
-#include "mongo/db/clientcursor.h"
-#include "mongo/db/curop.h"
-#include "mongo/db/field_ref.h"
+#include "mongo/db/collection_index_usage_tracker.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
+#include "mongo/db/feature_flag.h"
 #include "mongo/db/fts/fts_spec.h"
 #include "mongo/db/global_settings.h"
 #include "mongo/db/index/index_access_method.h"
@@ -52,37 +70,53 @@
 #include "mongo/db/index/s2_access_method.h"
 #include "mongo/db/index/s2_bucket_access_method.h"
 #include "mongo/db/index_names.h"
-#include "mongo/db/jsobj.h"
-#include "mongo/db/keypattern.h"
 #include "mongo/db/matcher/expression.h"
+#include "mongo/db/matcher/expression_parser.h"
+#include "mongo/db/matcher/extensions_callback_noop.h"
 #include "mongo/db/multi_key_path_tracker.h"
 #include "mongo/db/operation_context.h"
-#include "mongo/db/ops/delete.h"
+#include "mongo/db/pipeline/expression_context.h"
+#include "mongo/db/query/collation/collation_spec.h"
 #include "mongo/db/query/collation/collator_factory_interface.h"
+#include "mongo/db/query/collation/collator_interface.h"
 #include "mongo/db/query/collection_index_usage_tracker_decoration.h"
 #include "mongo/db/query/collection_query_info.h"
-#include "mongo/db/query/internal_plans.h"
 #include "mongo/db/query/query_feature_flags_gen.h"
 #include "mongo/db/query/query_knobs_gen.h"
+#include "mongo/db/repl/repl_settings.h"
 #include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/repl_set_member_in_standalone_mode.h"
+#include "mongo/db/server_feature_flags_gen.h"
 #include "mongo/db/server_options.h"
 #include "mongo/db/service_context.h"
 #include "mongo/db/storage/durable_catalog.h"
-#include "mongo/db/storage/execution_context.h"
-#include "mongo/db/storage/historical_ident_tracker.h"
 #include "mongo/db/storage/kv/kv_engine.h"
+#include "mongo/db/storage/recovery_unit.h"
+#include "mongo/db/storage/storage_engine.h"
 #include "mongo/db/storage/storage_engine_init.h"
+#include "mongo/db/storage/storage_options.h"
 #include "mongo/db/storage/storage_parameters_gen.h"
 #include "mongo/db/storage/storage_util.h"
+#include "mongo/db/transaction_resources.h"
 #include "mongo/db/ttl_collection_cache.h"
 #include "mongo/db/update/document_diff_calculator.h"
-#include "mongo/db/vector_clock.h"
+#include "mongo/db/update_index_data.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/log_tag.h"
+#include "mongo/logv2/redaction.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/platform/compiler.h"
 #include "mongo/util/assert_util.h"
+#include "mongo/util/decorable.h"
 #include "mongo/util/fail_point.h"
+#include "mongo/util/intrusive_counter.h"
 #include "mongo/util/represent_as.h"
+#include "mongo/util/scopeguard.h"
+#include "mongo/util/shared_buffer_fragment.h"
 #include "mongo/util/str.h"
+#include "mongo/util/uuid.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kIndex
 
@@ -92,6 +126,10 @@ namespace mongo {
 MONGO_FAIL_POINT_DEFINE(skipUnindexingDocumentWhenDeleted);
 MONGO_FAIL_POINT_DEFINE(skipIndexNewRecords);
 
+MONGO_FAIL_POINT_DEFINE(skipUpdatingIndexDocument);
+
+// This failpoint causes the check for TTL indexes on capped collections to be ignored.
+MONGO_FAIL_POINT_DEFINE(ignoreTTLIndexCappedCollectionCheck);
 
 using std::string;
 using std::unique_ptr;
@@ -218,12 +256,12 @@ void IndexCatalogImpl::init(OperationContext* opCtx,
         if (spec.hasField(IndexDescriptor::kExpireAfterSecondsFieldName)) {
             // TTL indexes with an invalid 'expireAfterSeconds' field cause problems in multiversion
             // settings.
-            auto hasInvalidExpireAfterSeconds =
-                !index_key_validate::validateExpireAfterSeconds(
-                     spec[IndexDescriptor::kExpireAfterSecondsFieldName],
-                     index_key_validate::ValidateExpireAfterSecondsMode::kSecondaryTTLIndex)
-                     .isOK();
-            if (hasInvalidExpireAfterSeconds) {
+            auto swType = index_key_validate::validateExpireAfterSeconds(
+                spec[IndexDescriptor::kExpireAfterSecondsFieldName],
+                index_key_validate::ValidateExpireAfterSecondsMode::kSecondaryTTLIndex);
+            auto expireAfterSecondsType = index_key_validate::extractExpireAfterSecondsType(swType);
+            if (expireAfterSecondsType ==
+                TTLCollectionCache::Info::ExpireAfterSecondsType::kInvalid) {
                 LOGV2_OPTIONS(
                     6852200,
                     {logv2::LogTag::kStartupWarnings},
@@ -234,28 +272,21 @@ void IndexCatalogImpl::init(OperationContext* opCtx,
                     "index"_attr = indexName,
                     "spec"_attr = spec);
             }
-
-            // TTL indexes are not compatible with capped collections.
             // Note that TTL deletion is supported on capped clustered collections via bounded
             // collection scan, which does not use an index.
-            if (!collection->isCapped()) {
-                if (opCtx->lockState()->inAWriteUnitOfWork()) {
-                    opCtx->recoveryUnit()->onCommit(
-                        [svcCtx = opCtx->getServiceContext(),
-                         uuid = collection->uuid(),
-                         indexName,
-                         hasInvalidExpireAfterSeconds](OperationContext*,
-                                                       boost::optional<Timestamp>) {
-                            TTLCollectionCache::get(svcCtx).registerTTLInfo(
-                                uuid,
-                                TTLCollectionCache::Info{indexName, hasInvalidExpireAfterSeconds});
-                        });
-                } else {
-                    TTLCollectionCache::get(opCtx->getServiceContext())
-                        .registerTTLInfo(
-                            collection->uuid(),
-                            TTLCollectionCache::Info{indexName, hasInvalidExpireAfterSeconds});
-                }
+            if (shard_role_details::getLocker(opCtx)->inAWriteUnitOfWork()) {
+                shard_role_details::getRecoveryUnit(opCtx)->onCommit(
+                    [svcCtx = opCtx->getServiceContext(),
+                     uuid = collection->uuid(),
+                     indexName,
+                     expireAfterSecondsType](OperationContext*, boost::optional<Timestamp>) {
+                        TTLCollectionCache::get(svcCtx).registerTTLInfo(
+                            uuid, TTLCollectionCache::Info{indexName, expireAfterSecondsType});
+                    });
+            } else {
+                TTLCollectionCache::get(opCtx->getServiceContext())
+                    .registerTTLInfo(collection->uuid(),
+                                     TTLCollectionCache::Info{indexName, expireAfterSecondsType});
             }
         }
 
@@ -267,8 +298,8 @@ void IndexCatalogImpl::init(OperationContext* opCtx,
                 // single-phase index builds are dropped during startup and rollback.
                 auto buildUUID = collection->getIndexBuildUUID(indexName);
                 invariant(buildUUID,
-                          str::stream()
-                              << "collection: " << collection->ns() << "index:" << indexName);
+                          str::stream() << "collection: " << collection->ns().toStringForErrorMsg()
+                                        << "index:" << indexName);
             }
 
             // We intentionally do not drop or rebuild unfinished two-phase index builds before
@@ -282,28 +313,19 @@ void IndexCatalogImpl::init(OperationContext* opCtx,
                 auto flags = CreateIndexEntryFlags::kInitFromDisk | CreateIndexEntryFlags::kFrozen;
                 IndexCatalogEntry* entry =
                     createIndexEntry(opCtx, collection, std::move(descriptor), flags);
-                fassert(31433, !entry->isReady(opCtx));
+                fassert(31433, !entry->isReady());
             } else {
                 // Initializing with unfinished indexes may occur during rollback or startup.
                 auto flags = CreateIndexEntryFlags::kInitFromDisk;
                 IndexCatalogEntry* entry =
                     createIndexEntry(opCtx, collection, std::move(descriptor), flags);
-                fassert(4505500, !entry->isReady(opCtx));
+                fassert(4505500, !entry->isReady());
             }
         } else {
             auto flags = CreateIndexEntryFlags::kInitFromDisk | CreateIndexEntryFlags::kIsReady;
             IndexCatalogEntry* entry =
                 createIndexEntry(opCtx, collection, std::move(descriptor), flags);
-            fassert(17340, entry->isReady(opCtx));
-            // (Ignore FCV check): This feature flag doesn't have any upgrade/downgrade concerns.
-            if (!feature_flags::gPointInTimeCatalogLookups.isEnabledAndIgnoreFCVUnsafe() &&
-                recoveryTs && !entry->descriptor()->isIdIndex()) {
-                // When initializing indexes from disk, we conservatively set the
-                // minimumVisibleSnapshot to non _id indexes to the recovery timestamp. The _id
-                // index is left visible. It's assumed if the collection is visible, it's _id is
-                // valid to be used.
-                entry->setMinimumVisibleSnapshot(recoveryTs.value());
-            }
+            fassert(17340, entry->isReady());
         }
     }
 
@@ -312,10 +334,8 @@ void IndexCatalogImpl::init(OperationContext* opCtx,
     CollectionQueryInfo& info = CollectionQueryInfo::get(collection);
     if (isPointInTimeRead) {
         info.clearQueryCache(opCtx, CollectionPtr(collection));
-        info.rebuildIndexData(opCtx, CollectionPtr(collection));
-    } else {
-        info.init(opCtx, CollectionPtr(collection));
     }
+    info.init(opCtx, collection);
 }
 
 std::unique_ptr<IndexCatalog::IndexIterator> IndexCatalogImpl::getIndexIterator(
@@ -401,7 +421,7 @@ Status IndexCatalogImpl::_isNonIDIndexAndNotAllowedToBuild(OperationContext* opC
         return Status::OK();
     }
 
-    if (!getGlobalReplSettings().usingReplSets()) {
+    if (!getGlobalReplSettings().isReplSet()) {
         return Status::OK();
     }
 
@@ -421,7 +441,8 @@ void IndexCatalogImpl::_logInternalState(OperationContext* opCtx,
                                          const CollectionPtr& collection,
                                          long long numIndexesInCollectionCatalogEntry,
                                          const std::vector<std::string>& indexNamesToDrop) {
-    invariant(opCtx->lockState()->isCollectionLockedForMode(collection->ns(), MODE_X));
+    invariant(
+        shard_role_details::getLocker(opCtx)->isCollectionLockedForMode(collection->ns(), MODE_X));
 
     LOGV2_ERROR(20365,
                 "Internal Index Catalog state",
@@ -544,6 +565,7 @@ std::vector<BSONObj> IndexCatalogImpl::removeExistingIndexesNoChecks(
 
         // _doesSpecConflictWithExisting currently does more work than we require here: we are only
         // interested in the index already exists error.
+
         auto inclusionPolicy = removeInProgressIndexBuilds
             ? IndexCatalog::InclusionPolicy::kReady | IndexCatalog::InclusionPolicy::kUnfinished
             : IndexCatalog::InclusionPolicy::kReady;
@@ -583,6 +605,14 @@ IndexCatalogEntry* IndexCatalogImpl::createIndexEntry(OperationContext* opCtx,
 
     Status status = _isSpecOk(opCtx, CollectionPtr(collection), descriptor.infoObj());
     if (!status.isOK()) {
+        // If running inside a --repair operation, throw an error so the operation can attempt to
+        // remove any invalid options from the index specification. Any other types of invalid index
+        // specifications, e.g. not specifying a name for the index, will crash the server.
+        if (storageGlobalParams.repair &&
+            status.code() == ErrorCodes::InvalidIndexSpecificationOption) {
+            uasserted(ErrorCodes::InvalidIndexSpecificationOption, status.reason());
+        }
+
         LOGV2_FATAL(28782,
                     "Found an invalid index",
                     "descriptor"_attr = descriptor.infoObj(),
@@ -626,17 +656,6 @@ IndexCatalogEntry* IndexCatalogImpl::createIndexEntry(OperationContext* opCtx,
         _buildingIndexes.add(std::move(entry));
     }
 
-    bool initFromDisk = CreateIndexEntryFlags::kInitFromDisk & flags;
-    if (!initFromDisk && !UncommittedCatalogUpdates::isCreatedCollection(opCtx, collection->ns())) {
-        const std::string indexName = desc->indexName();
-        opCtx->recoveryUnit()->onRollback(
-            [collectionDecorations = collection->getSharedDecorations(),
-             indexName = std::move(indexName)](OperationContext*) {
-                CollectionIndexUsageTrackerDecoration::get(collectionDecorations)
-                    .unregisterIndex(indexName);
-            });
-    }
-
     return save;
 }
 
@@ -646,7 +665,8 @@ StatusWith<BSONObj> IndexCatalogImpl::createIndexOnEmptyCollection(OperationCont
     invariant(collection->uuid() == collection->uuid());
     CollectionCatalog::get(opCtx)->invariantHasExclusiveAccessToCollection(opCtx, collection->ns());
     invariant(collection->isEmpty(opCtx),
-              str::stream() << "Collection must be empty. Collection: " << collection->ns()
+              str::stream() << "Collection must be empty. Collection: "
+                            << collection->ns().toStringForErrorMsg()
                             << " UUID: " << collection->uuid()
                             << " Count (from size storer): " << collection->numRecords(opCtx));
 
@@ -936,16 +956,13 @@ Status IndexCatalogImpl::_isSpecOk(OperationContext* opCtx,
             return wildcardSpecStatus;
         }
     } else if (pluginName == IndexNames::COLUMN) {
-        // Be careful to only read the FCV one time to avoid race conditions.
         const auto fcvSnapshot = serverGlobalParams.featureCompatibility.acquireFCVSnapshot();
         uassert(ErrorCodes::NotImplemented,
                 str::stream() << pluginName
                               << " indexes are under development and cannot be used without "
                                  "enabling the feature flag",
                 // With our testing failpoint we may try to run this code before we've initialized
-                // the FCV. We also relax this check when a secondary is initial syncing, which may
-                // temporarily unset the FCV. In this case, the primary should have already checked
-                // that the feature flag is enabled.
+                // the FCV.
                 !fcvSnapshot.isVersionInitialized() ||
                     feature_flags::gFeatureFlagColumnstoreIndexes.isEnabled(fcvSnapshot));
         if (auto columnSpecStatus = validateColumnStoreSpec(collection, spec, indexVersion);
@@ -1077,9 +1094,9 @@ Status IndexCatalogImpl::_doesSpecConflictWithExisting(OperationContext* opCtx,
 
         if (desc) {
             // Index already exists with same name. Check whether the options are the same as well.
+            auto entry = getEntry(desc);
             IndexDescriptor candidate(_getAccessMethodName(key), spec);
-            auto indexComparison =
-                candidate.compareIndexOptions(opCtx, collection->ns(), getEntry(desc));
+            auto indexComparison = candidate.compareIndexOptions(opCtx, collection->ns(), entry);
 
             // Key pattern or another uniquely-identifying option differs. We can build this index,
             // but not with the specified (duplicate) name. User must specify another index name.
@@ -1107,7 +1124,6 @@ Status IndexCatalogImpl::_doesSpecConflictWithExisting(OperationContext* opCtx,
 
             // If an identical index exists, but it is frozen, return an error with a different
             // error code to the user, forcing the user to drop before recreating the index.
-            auto entry = getEntry(desc);
             if (entry->isFrozen()) {
                 return Status(ErrorCodes::CannotCreateIndex,
                               str::stream()
@@ -1131,7 +1147,6 @@ Status IndexCatalogImpl::_doesSpecConflictWithExisting(OperationContext* opCtx,
         if (desc) {
             LOGV2_DEBUG(20353,
                         2,
-                        "Index already exists with a different name: {name}, spec: {spec}",
                         "Index already exists with a different name",
                         "name"_attr = desc->indexName(),
                         "spec"_attr = desc->infoObj());
@@ -1139,9 +1154,10 @@ Status IndexCatalogImpl::_doesSpecConflictWithExisting(OperationContext* opCtx,
             // Index already exists with a different name. Check whether the options are identical.
             // We will return an error in either case, but this check allows us to generate a more
             // informative error message.
+            auto entry = getEntry(desc);
+
             IndexDescriptor candidate(_getAccessMethodName(key), spec);
-            auto indexComparison =
-                candidate.compareIndexOptions(opCtx, collection->ns(), getEntry(desc));
+            auto indexComparison = candidate.compareIndexOptions(opCtx, collection->ns(), entry);
 
             // The candidate's key and uniquely-identifying options are equivalent to an existing
             // index, but some other options are not identical. Return a message to that effect.
@@ -1162,8 +1178,8 @@ Status IndexCatalogImpl::_doesSpecConflictWithExisting(OperationContext* opCtx,
     }
 
     if (numIndexesTotal() >= kMaxNumIndexesAllowed) {
-        string s = str::stream() << "add index fails, too many indexes for " << collection->ns()
-                                 << " key:" << key;
+        string s = str::stream() << "add index fails, too many indexes for "
+                                 << collection->ns().toStringForErrorMsg() << " key:" << key;
         LOGV2(20354,
               "Exceeded maximum number of indexes",
               logAttrs(collection->ns()),
@@ -1293,8 +1309,9 @@ void IndexCatalogImpl::dropAllIndexes(OperationContext* opCtx,
 Status IndexCatalogImpl::resetUnfinishedIndexForRecovery(OperationContext* opCtx,
                                                          Collection* collection,
                                                          IndexCatalogEntry* entry) {
-    invariant(opCtx->lockState()->isCollectionLockedForMode(collection->ns(), MODE_X));
-    invariant(opCtx->lockState()->inAWriteUnitOfWork());
+    invariant(
+        shard_role_details::getLocker(opCtx)->isCollectionLockedForMode(collection->ns(), MODE_X));
+    invariant(shard_role_details::getLocker(opCtx)->inAWriteUnitOfWork());
 
     const std::string indexName = entry->descriptor()->indexName();
 
@@ -1325,7 +1342,8 @@ Status IndexCatalogImpl::resetUnfinishedIndexForRecovery(OperationContext* opCtx
     // Drop the ident if it exists. The storage engine will return OK if the ident is not found.
     auto engine = opCtx->getServiceContext()->getStorageEngine();
     const std::string ident = released->getIdent();
-    Status status = engine->getEngine()->dropIdent(opCtx->recoveryUnit(), ident);
+    Status status = engine->getEngine()->dropIdentSynchronous(
+        shard_role_details::getRecoveryUnit(opCtx), ident);
     if (!status.isOK()) {
         return status;
     }
@@ -1359,52 +1377,12 @@ Status IndexCatalogImpl::dropUnfinishedIndex(OperationContext* opCtx,
     if (!entry)
         return Status(ErrorCodes::InternalError, "cannot find index to delete");
 
-    if (entry->isReady(opCtx))
+    if (entry->isReady())
         return Status(ErrorCodes::InternalError, "expected unfinished index, but it is ready");
 
     return dropIndexEntry(opCtx, collection, entry);
 }
 
-namespace {
-class IndexRemoveChange final : public RecoveryUnit::Change {
-public:
-    IndexRemoveChange(const NamespaceString& nss,
-                      const UUID& uuid,
-                      std::shared_ptr<IndexCatalogEntry> entry,
-                      SharedCollectionDecorations* collectionDecorations)
-        : _nss(nss),
-          _uuid(uuid),
-          _entry(std::move(entry)),
-          _collectionDecorations(collectionDecorations) {}
-
-    void commit(OperationContext* opCtx, boost::optional<Timestamp> commitTime) final {
-        if (commitTime) {
-            HistoricalIdentTracker::get(opCtx).recordDrop(
-                _entry->getIdent(), _nss, _uuid, commitTime.value());
-        }
-
-        _entry->setDropped();
-    }
-
-    void rollback(OperationContext* opCtx) final {
-        auto indexDescriptor = _entry->descriptor();
-
-        // Refresh the CollectionIndexUsageTrackerDecoration's knowledge of what indices are
-        // present as it is shared state across Collection copies.
-        CollectionIndexUsageTrackerDecoration::get(_collectionDecorations)
-            .registerIndex(indexDescriptor->indexName(),
-                           indexDescriptor->keyPattern(),
-                           IndexFeatures::make(indexDescriptor, _nss.isOnInternalDb()));
-    }
-
-private:
-    const NamespaceString _nss;
-    const UUID _uuid;
-    std::shared_ptr<IndexCatalogEntry> _entry;
-    SharedCollectionDecorations* _collectionDecorations;
-};
-}  // namespace
-
 Status IndexCatalogImpl::dropIndexEntry(OperationContext* opCtx,
                                         Collection* collection,
                                         IndexCatalogEntry* entry) {
@@ -1429,16 +1407,9 @@ Status IndexCatalogImpl::dropIndexEntry(OperationContext* opCtx,
     }();
 
     invariant(released.get() == entry);
-    // TODO SERVER-77131: Remove index catalog entry instance in commit handler.
-    opCtx->recoveryUnit()->registerChange(
-        std::make_unique<IndexRemoveChange>(collection->ns(),
-                                            collection->uuid(),
-                                            entry->shared_from_this(),
-                                            collection->getSharedDecorations()));
 
-    CollectionQueryInfo::get(collection).rebuildIndexData(opCtx, CollectionPtr(collection));
-    CollectionIndexUsageTrackerDecoration::get(collection->getSharedDecorations())
-        .unregisterIndex(indexName);
+    CollectionQueryInfo::get(collection).rebuildIndexData(opCtx, collection);
+    CollectionIndexUsageTrackerDecoration::write(collection).unregisterIndex(indexName);
     _deleteIndexFromDisk(opCtx, collection, indexName, entry->shared_from_this());
 
     return Status::OK();
@@ -1506,13 +1477,7 @@ bool IndexCatalogImpl::haveIdIndex(OperationContext* opCtx) const {
 }
 
 const IndexDescriptor* IndexCatalogImpl::findIdIndex(OperationContext* opCtx) const {
-    auto ii = getIndexIterator(opCtx, InclusionPolicy::kReady);
-    while (ii->more()) {
-        const IndexDescriptor* desc = ii->next()->descriptor();
-        if (desc->isIdIndex())
-            return desc;
-    }
-    return nullptr;
+    return _readyIndexes.getIdIndex();
 }
 
 const IndexDescriptor* IndexCatalogImpl::findIndexByName(OperationContext* opCtx,
@@ -1593,7 +1558,7 @@ const IndexCatalogEntry* IndexCatalogImpl::getEntry(const IndexDescriptor* desc)
 IndexCatalogEntry* IndexCatalogImpl::getWritableEntryByName(OperationContext* opCtx,
                                                             StringData name,
                                                             InclusionPolicy inclusionPolicy) {
-    return _getWritableEntry(opCtx, findIndexByName(opCtx, name, inclusionPolicy));
+    return _getWritableEntry(findIndexByName(opCtx, name, inclusionPolicy));
 }
 
 IndexCatalogEntry* IndexCatalogImpl::getWritableEntryByKeyPatternAndOptions(
@@ -1602,11 +1567,10 @@ IndexCatalogEntry* IndexCatalogImpl::getWritableEntryByKeyPatternAndOptions(
     const BSONObj& indexSpec,
     InclusionPolicy inclusionPolicy) {
     return _getWritableEntry(
-        opCtx, findIndexByKeyPatternAndOptions(opCtx, key, indexSpec, inclusionPolicy));
+        findIndexByKeyPatternAndOptions(opCtx, key, indexSpec, inclusionPolicy));
 }
 
-IndexCatalogEntry* IndexCatalogImpl::_getWritableEntry(OperationContext* opCtx,
-                                                       const IndexDescriptor* descriptor) {
+IndexCatalogEntry* IndexCatalogImpl::_getWritableEntry(const IndexDescriptor* descriptor) {
     if (!descriptor) {
         return nullptr;
     }
@@ -1630,7 +1594,7 @@ IndexCatalogEntry* IndexCatalogImpl::_getWritableEntry(OperationContext* opCtx,
         return entryToReturn;
     };
 
-    if (descriptor->getEntry()->isReady(opCtx)) {
+    if (descriptor->getEntry()->isReady()) {
         return getWritableEntry(_readyIndexes);
     } else if (descriptor->getEntry()->isFrozen()) {
         return getWritableEntry(_frozenIndexes);
@@ -1658,23 +1622,20 @@ const IndexDescriptor* IndexCatalogImpl::refreshEntry(OperationContext* opCtx,
     const std::string indexName = oldDesc->indexName();
     invariant(collection->isIndexReady(indexName));
 
+    // The _id index should not be modified by a collMod.
+    tassert(9037800, "Should not be refreshing the _id index", !oldDesc->isIdIndex());
+
     // Delete the IndexCatalogEntry that owns this descriptor. After deletion, 'oldDesc' is invalid
     // and should not be dereferenced. Also, invalidate the index from the
     // CollectionIndexUsageTrackerDecoration (shared state among Collection instances).
-    IndexCatalogEntry* writableEntry = _getWritableEntry(opCtx, oldDesc);
+    IndexCatalogEntry* writableEntry = _getWritableEntry(oldDesc);
     invariant(writableEntry);
     std::shared_ptr<const IndexCatalogEntry> deletedEntry =
         _readyIndexes.release(writableEntry->descriptor());
     invariant(writableEntry == deletedEntry.get());
 
-    // TODO SERVER-77131: Remove index catalog entry instance in commit handler.
-    opCtx->recoveryUnit()->registerChange(
-        std::make_unique<IndexRemoveChange>(collection->ns(),
-                                            collection->uuid(),
-                                            writableEntry->shared_from_this(),
-                                            collection->getSharedDecorations()));
-    CollectionIndexUsageTrackerDecoration::get(collection->getSharedDecorations())
-        .unregisterIndex(indexName);
+    auto& indexUsageTracker = CollectionIndexUsageTrackerDecoration::write(collection);
+    indexUsageTracker.unregisterIndex(indexName);
 
     // Ask the CollectionCatalogEntry for the new index spec.
     BSONObj spec = collection->getIndexSpec(indexName).getOwned();
@@ -1684,22 +1645,14 @@ const IndexDescriptor* IndexCatalogImpl::refreshEntry(OperationContext* opCtx,
     // to the CollectionIndexUsageTrackerDecoration (shared state among Collection instances).
     auto newDesc = IndexDescriptor(_getAccessMethodName(keyPattern), spec);
     auto newEntry = createIndexEntry(opCtx, collection, std::move(newDesc), flags);
-    invariant(newEntry->isReady(opCtx));
+    invariant(newEntry->isReady());
     auto desc = newEntry->descriptor();
-    CollectionIndexUsageTrackerDecoration::get(collection->getSharedDecorations())
-        .registerIndex(desc->indexName(),
-                       desc->keyPattern(),
-                       IndexFeatures::make(desc, collection->ns().isOnInternalDb()));
+    indexUsageTracker.registerIndex(desc->indexName(),
+                                    desc->keyPattern(),
+                                    IndexFeatures::make(desc, collection->ns().isOnInternalDb()));
 
     // Last rebuild index data for CollectionQueryInfo for this Collection.
-    CollectionQueryInfo::get(collection).rebuildIndexData(opCtx, CollectionPtr(collection));
-
-    opCtx->recoveryUnit()->onCommit(
-        [newEntry](OperationContext*, boost::optional<Timestamp> commitTime) {
-            if (commitTime) {
-                newEntry->setMinimumVisibleSnapshot(*commitTime);
-            }
-        });
+    CollectionQueryInfo::get(collection).rebuildIndexData(opCtx, collection);
 
     // Return the new descriptor.
     return newEntry->descriptor();
@@ -1713,7 +1666,7 @@ Status IndexCatalogImpl::_indexFilteredRecords(OperationContext* opCtx,
                                                const IndexCatalogEntry* index,
                                                const std::vector<BsonRecord>& bsonRecords,
                                                int64_t* keysInsertedOut) const {
-    SharedBufferFragmentBuilder pooledBuilder(KeyString::HeapBuilder::kHeapAllocatorDefaultBytes);
+    SharedBufferFragmentBuilder pooledBuilder(key_string::HeapBuilder::kHeapAllocatorDefaultBytes);
 
     InsertDeleteOptions options;
     prepareInsertDeleteOptions(opCtx, coll->ns(), index->descriptor(), &options);
@@ -1727,9 +1680,17 @@ Status IndexCatalogImpl::_indexRecords(OperationContext* opCtx,
                                        const IndexCatalogEntry* index,
                                        const std::vector<BsonRecord>& bsonRecords,
                                        int64_t* keysInsertedOut) const {
-    if (MONGO_unlikely(skipIndexNewRecords.shouldFail())) {
+    bool skip = false;
+    skipIndexNewRecords.execute([&](const BSONObj& dataObj) {
+        // Skip the _id index by default.
+        if (!dataObj.hasField("skipIdIndex") || dataObj["skipIdIndex"].Bool() == true) {
+            skip = true;
+        } else if (!index->descriptor()->isIdIndex()) {
+            skip = true;
+        }
+    });
+    if (skip)
         return Status::OK();
-    }
 
     const MatchExpression* filter = index->getFilterExpression();
     if (!filter)
@@ -1752,7 +1713,22 @@ Status IndexCatalogImpl::_updateRecord(OperationContext* const opCtx,
                                        const RecordId& recordId,
                                        int64_t* const keysInsertedOut,
                                        int64_t* const keysDeletedOut) const {
-    SharedBufferFragmentBuilder pooledBuilder(KeyString::HeapBuilder::kHeapAllocatorDefaultBytes);
+    // TODO SERVER-80257: This failpoint was added to produce index corruption scenarios where an
+    // index has incorrect keys. Replace this failpoint with a test command instead.
+    if (auto failpoint = skipUpdatingIndexDocument.scoped(); MONGO_unlikely(failpoint.isActive()) &&
+        repl::feature_flags::gSecondaryIndexChecksInDbCheck.isEnabled(
+            serverGlobalParams.featureCompatibility.acquireFCVSnapshot())) {
+        auto indexName = failpoint.getData()["indexName"].valueStringDataSafe();
+        if (indexName == index->descriptor()->indexName()) {
+            LOGV2_DEBUG(
+                7844805,
+                3,
+                "Skipping updating index record because failpoint skipUpdatingIndexDocument is on",
+                "indexName"_attr = indexName);
+            return Status::OK();
+        }
+    }
+    SharedBufferFragmentBuilder pooledBuilder(key_string::HeapBuilder::kHeapAllocatorDefaultBytes);
 
     InsertDeleteOptions options;
     prepareInsertDeleteOptions(opCtx, coll->ns(), index->descriptor(), &options);
@@ -1794,11 +1770,17 @@ void IndexCatalogImpl::_unindexRecord(OperationContext* opCtx,
         MONGO_unlikely(failpoint.isActive())) {
         auto indexName = failpoint.getData()["indexName"].valueStringDataSafe();
         if (indexName == entry->descriptor()->indexName()) {
+            LOGV2_DEBUG(
+                7844806,
+                3,
+                "Skipping unindexing document because failpoint skipUnindexingDocumentWhenDeleted "
+                "is on",
+                "indexName"_attr = indexName);
             return;
         }
     }
 
-    SharedBufferFragmentBuilder pooledBuilder(KeyString::HeapBuilder::kHeapAllocatorDefaultBytes);
+    SharedBufferFragmentBuilder pooledBuilder(key_string::HeapBuilder::kHeapAllocatorDefaultBytes);
 
     InsertDeleteOptions options;
     prepareInsertDeleteOptions(opCtx, collection->ns(), entry->descriptor(), &options);
@@ -1873,7 +1855,8 @@ Status IndexCatalogImpl::indexRecords(OperationContext* opCtx,
         return Status::OK();
     }
 
-    if (Status status = opCtx->recoveryUnit()->setTimestamp(bsonRecords[0].ts); !status.isOK()) {
+    if (Status status = shard_role_details::getRecoveryUnit(opCtx)->setTimestamp(bsonRecords[0].ts);
+        !status.isOK()) {
         return status;
     }
 
@@ -1885,9 +1868,9 @@ Status IndexCatalogImpl::indexRecords(OperationContext* opCtx,
                                        IndexCatalog::InclusionPolicy::kUnfinished);
         if (!idx) {
             return Status(ErrorCodes::IndexNotFound,
-                          str::stream()
-                              << "Could not find index " << newPath.indexName << " in "
-                              << coll->ns() << " (" << coll->uuid() << ") to set to multikey.");
+                          str::stream() << "Could not find index " << newPath.indexName << " in "
+                                        << coll->ns().toStringForErrorMsg() << " (" << coll->uuid()
+                                        << ") to set to multikey.");
         }
         setMultikeyPaths(opCtx, coll, idx, newPath.multikeyMetadataKeys, newPath.multikeyPaths);
     }
@@ -1912,12 +1895,16 @@ Status IndexCatalogImpl::updateRecord(OperationContext* const opCtx,
         if (opDiff) {
             std::vector<const UpdateIndexData*> allIndexPaths;
             allIndexPaths.reserve(numIndexesToUpdate);
-            for (auto& indexes : {std::ref(_readyIndexes), std::ref(_buildingIndexes)}) {
-                for (const auto& indexEntry : indexes.get()) {
-                    dassert(!indexEntry->getIndexedPaths().isEmpty());
-                    allIndexPaths.push_back(&indexEntry->getIndexedPaths());
-                }
+
+            for (const auto& indexEntry : _readyIndexes) {
+                dassert(!indexEntry->getIndexedPaths().isEmpty());
+                allIndexPaths.push_back(&indexEntry->getIndexedPaths());
             }
+            for (const auto& indexEntry : _buildingIndexes) {
+                dassert(!indexEntry->getIndexedPaths().isEmpty());
+                allIndexPaths.push_back(&indexEntry->getIndexedPaths());
+            }
+
             toUpdate = mongo::doc_diff::anyIndexesMightBeAffected(*opDiff, allIndexPaths);
         } else {
             toUpdate = mongo::doc_diff::BitVector(numIndexesToUpdate);
@@ -1967,13 +1954,16 @@ void IndexCatalogImpl::unindexRecord(OperationContext* opCtx,
         const IndexCatalogEntry* entry = it->get();
 
         // If it's a background index, we DO NOT want to log anything.
-        bool logIfError = entry->isReady(opCtx) ? !noWarn : false;
+        bool logIfError = entry->isReady() ? !noWarn : false;
         _unindexRecord(
             opCtx, collection, entry, obj, loc, logIfError, keysDeletedOut, checkRecordId);
     }
 }
 
-Status IndexCatalogImpl::compactIndexes(OperationContext* opCtx) const {
+StatusWith<int64_t> IndexCatalogImpl::compactIndexes(OperationContext* opCtx,
+                                                     const CompactOptions& options) const {
+
+    int64_t estimatedBytes = 0;
     for (IndexCatalogEntryContainer::const_iterator it = _readyIndexes.begin();
          it != _readyIndexes.end();
          ++it) {
@@ -1983,16 +1973,17 @@ Status IndexCatalogImpl::compactIndexes(OperationContext* opCtx) const {
                     1,
                     "compacting index: {entry_descriptor}",
                     "entry_descriptor"_attr = *(entry->descriptor()));
-        Status status = entry->accessMethod()->compact(opCtx);
+        auto status = entry->accessMethod()->compact(opCtx, options);
         if (!status.isOK()) {
             LOGV2_ERROR(20377,
                         "Failed to compact index",
                         "index"_attr = *(entry->descriptor()),
-                        "error"_attr = redact(status));
+                        "error"_attr = redact(status.getStatus()));
             return status;
         }
+        estimatedBytes += status.getValue();
     }
-    return Status::OK();
+    return estimatedBytes;
 }
 
 std::string::size_type IndexCatalogImpl::getLongestIndexNameLength(OperationContext* opCtx) const {
