@@ -27,19 +27,21 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/catalog/index_key_validate.h"
-
+#include <boost/move/utility_core.hpp>
+#include <climits>
+#include <fmt/format.h>
 #include <limits>
+#include <memory>
 
+#include "mongo/base/error_codes.h"
 #include "mongo/bson/bsonmisc.h"
 #include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/bson/json.h"
+#include "mongo/db/catalog/index_key_validate.h"
 #include "mongo/db/index/index_descriptor.h"
-#include "mongo/db/query/query_knobs_gen.h"
 #include "mongo/idl/server_parameter_test_util.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
 
 namespace mongo {
 
@@ -246,18 +248,7 @@ TEST(IndexKeyValidateTest, WildcardIndexNumericKeyElementValueFailsIfZero) {
               validateKeyPattern(BSON("$**" << -0.0), IndexVersion::kV2));
 }
 
-TEST(IndexKeyValidateTest, WildcardIndexNumericKeyElementValueFailsIfNotPositive) {
-    RAIIServerParameterControllerForTest controller("featureFlagCompoundWildcardIndexes", false);
-    ASSERT_EQ(ErrorCodes::CannotCreateIndex,
-              validateKeyPattern(BSON("$**" << -0.1), IndexVersion::kV2));
-    ASSERT_EQ(ErrorCodes::CannotCreateIndex,
-              validateKeyPattern(BSON("$**" << -1), IndexVersion::kV2));
-    ASSERT_EQ(ErrorCodes::CannotCreateIndex,
-              validateKeyPattern(BSON("$**" << INT_MIN), IndexVersion::kV2));
-}
-
 TEST(IndexKeyValidateTest, WildcardIndexNumericKeyElementValueSucceedsIfNotPositive) {
-    RAIIServerParameterControllerForTest controller("featureFlagCompoundWildcardIndexes", true);
     ASSERT_OK(validateKeyPattern(BSON("$**" << -0.1), IndexVersion::kV2));
     ASSERT_OK(validateKeyPattern(BSON("$**" << -1), IndexVersion::kV2));
     ASSERT_OK(validateKeyPattern(BSON("$**" << INT_MIN), IndexVersion::kV2));
@@ -275,15 +266,7 @@ TEST(IndexKeyValidateTest, KeyElementNameWildcardFailsOnSubPathRepeat) {
     ASSERT_EQ(status, ErrorCodes::CannotCreateIndex);
 }
 
-TEST(IndexKeyValidateTest, KeyElementNameWildcardFailsOnCompound) {
-    RAIIServerParameterControllerForTest controller("featureFlagCompoundWildcardIndexes", false);
-    auto status = validateKeyPattern(BSON("$**" << 1 << "a" << 1), IndexVersion::kV2);
-    ASSERT_NOT_OK(status);
-    ASSERT_EQ(status, ErrorCodes::CannotCreateIndex);
-}
-
 TEST(IndexKeyValidateTest, KeyElementNameWildcardSucceedsOnCompound) {
-    RAIIServerParameterControllerForTest controller("featureFlagCompoundWildcardIndexes", true);
     ASSERT_OK(validateKeyPattern(BSON("$**" << 1 << "a" << 1), IndexVersion::kV2));
     ASSERT_OK(validateKeyPattern(BSON("a" << 1 << "$**" << 1), IndexVersion::kV2));
     ASSERT_OK(validateKeyPattern(BSON("a" << 1 << "$**" << -1 << "b" << 1), IndexVersion::kV2));
@@ -296,12 +279,10 @@ TEST(IndexKeyValidateTest, KeyElementNameWildcardFailsOnIncorrectValue) {
 }
 
 TEST(IndexKeyValidateTest, KeyElementNameWildcardFailsWhenValueIsPluginNameWithInvalidKeyName) {
-    RAIIServerParameterControllerForTest controller("featureFlagCompoundWildcardIndexes", false);
     auto status = validateKeyPattern(BSON("a"
                                           << "wildcard"),
                                      IndexVersion::kV2);
     ASSERT_NOT_OK(status);
-    ASSERT_EQ(status, ErrorCodes::CannotCreateIndex);
 }
 
 TEST(IndexKeyValidateTest, KeyElementNameWildcardFailsWhenValueIsPluginNameWithValidKeyName) {
@@ -438,6 +419,48 @@ TEST(IndexKeyValidateTest, ValidateAfterSecondsAcceptsFloatingPointNumber) {
     ASSERT_EQUALS(spec["expireAfterSeconds"].safeNumberLong(), 123LL);
 }
 
+TEST(IndexKeyValidateTest, NormalizeExpireAfterSecondsDetectsInvalidValues) {
+    auto specs = {
+        fromjson("{key: {a: 1}, name: 'index', expireAfterSeconds: NaN}"),
+        fromjson("{key: {a: 1}, name: 'index', expireAfterSeconds: 1E300}"),
+        fromjson("{key: {a: 1}, name: 'index', expireAfterSeconds: -1E300}"),
+        fromjson("{key: {a: 1}, name: 'index', expireAfterSeconds: -3600}"),
+    };
+
+    for (auto&& spec : specs) {
+        auto res = index_key_validate::normalizeExpireAfterSeconds(spec["expireAfterSeconds"]);
+        ASSERT_TRUE(res.has_value());
+        ASSERT_EQUALS(
+            res.value(),
+            durationCount<Seconds>(index_key_validate::kExpireAfterSecondsForInactiveTTLIndex));
+    }
+}
+
+TEST(IndexKeyValidateTest, NormalizeExpireAfterSecondsDetectsValidNonIntegerValues) {
+    auto specs = {
+        fromjson("{key: {a: 1}, name: 'index', expireAfterSeconds: 123.456}"),
+    };
+
+    for (auto&& spec : specs) {
+        auto res = index_key_validate::normalizeExpireAfterSeconds(spec["expireAfterSeconds"]);
+        ASSERT_TRUE(res.has_value());
+        ASSERT_EQUALS(res.value(), spec["expireAfterSeconds"].safeNumberInt());
+    }
+}
+
+TEST(IndexKeyValidateTest, NormalizeExpireAfterSecondsHandlesValidIntegerValues) {
+    auto specs = {
+        fromjson("{key: {a: 1}, name: 'index', expireAfterSeconds: 2147483646}"),
+        fromjson("{key: {a: 1}, name: 'index', expireAfterSeconds: 3600}"),
+        fromjson("{key: {a: 1}, name: 'index', expireAfterSeconds: 0}"),
+    };
+
+    for (auto&& spec : specs) {
+        auto res = index_key_validate::normalizeExpireAfterSeconds(spec["expireAfterSeconds"]);
+        ASSERT_FALSE(res.has_value());
+    }
+}
+
 TEST(IndexKeyValidateTest, RepairIndexSpecs) {
     ASSERT(fromjson("{key: {a: 1}, name: 'index'}")
                .binaryEqual(index_key_validate::repairIndexSpec(
