@@ -28,14 +28,43 @@
  */
 
 #include "mongo/db/catalog/index_repair.h"
+
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <cstdint>
+#include <memory>
+#include <string>
+#include <vector>
+
+#include <boost/none.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
 #include "mongo/base/status_with.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/db/catalog/collection_catalog.h"
+#include "mongo/db/catalog/collection_options.h"
 #include "mongo/db/catalog/collection_write_path.h"
-#include "mongo/db/catalog/collection_yield_restore.h"
-#include "mongo/db/catalog/validate_state.h"
+#include "mongo/db/catalog/database.h"
+#include "mongo/db/catalog/index_catalog.h"
 #include "mongo/db/catalog_raii.h"
 #include "mongo/db/concurrency/exception_util.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
 #include "mongo/db/index/index_access_method.h"
-#include "mongo/logv2/log_debug.h"
+#include "mongo/db/index/index_descriptor.h"
+#include "mongo/db/repl/oplog.h"
+#include "mongo/db/repl/read_concern_args.h"
+#include "mongo/db/session/logical_session_id.h"
+#include "mongo/db/shard_role.h"
+#include "mongo/db/storage/duplicate_key_error_info.h"
+#include "mongo/db/storage/snapshot.h"
+#include "mongo/db/storage/write_unit_of_work.h"
+#include "mongo/db/transaction_resources.h"
+#include "mongo/s/database_version.h"
+#include "mongo/s/shard_version.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/str.h"
+#include "mongo/util/uuid.h"
 
 namespace mongo {
 namespace index_repair {
@@ -47,28 +76,32 @@ StatusWith<int> moveRecordToLostAndFound(OperationContext* opCtx,
     AutoGetCollection autoColl(opCtx, lostAndFoundNss, MODE_IX);
     auto catalog = CollectionCatalog::get(opCtx);
     auto originalCollection = catalog->lookupCollectionByNamespace(opCtx, nss);
-    CollectionPtr localCollection(catalog->lookupCollectionByNamespace(opCtx, lostAndFoundNss));
+
+    auto localCollection = acquireCollection(
+        opCtx,
+        CollectionAcquisitionRequest(lostAndFoundNss,
+                                     PlacementConcern{boost::none, ShardVersion::UNSHARDED()},
+                                     repl::ReadConcernArgs::get(opCtx),
+                                     AcquisitionPrerequisites::kWrite),
+        MODE_IX);
 
     // Creates the collection if it doesn't exist.
-    if (!localCollection) {
+    if (!localCollection.exists()) {
         Status status =
-            writeConflictRetry(opCtx, "createLostAndFoundCollection", lostAndFoundNss.ns(), [&]() {
+            writeConflictRetry(opCtx, "createLostAndFoundCollection", lostAndFoundNss, [&]() {
                 // Ensure the database exists.
                 auto db = autoColl.ensureDbExists(opCtx);
-                invariant(db, lostAndFoundNss.ns());
+                invariant(db, lostAndFoundNss.toStringForErrorMsg());
 
                 WriteUnitOfWork wuow(opCtx);
+                ScopedLocalCatalogWriteFence scopedLocalCatalogWriteFence(opCtx, &localCollection);
 
                 // Since we are potentially deleting a document with duplicate _id values, we need
                 // to be able to insert into the lost and found collection without generating any
                 // duplicate key errors on the _id value.
                 CollectionOptions collOptions;
                 collOptions.setNoIdIndex();
-                localCollection =
-                    CollectionPtr(db->createCollection(opCtx, lostAndFoundNss, collOptions));
-
-                // Ensure the collection exists.
-                invariant(localCollection, lostAndFoundNss.ns());
+                db->createCollection(opCtx, lostAndFoundNss, collOptions);
 
                 wuow.commit();
                 return Status::OK();
@@ -78,10 +111,11 @@ StatusWith<int> moveRecordToLostAndFound(OperationContext* opCtx,
         }
     }
 
-    localCollection.makeYieldable(opCtx, LockedCollectionYieldRestore(opCtx, localCollection));
+    // Ensure the collection exists.
+    invariant(localCollection.exists(), lostAndFoundNss.toStringForErrorMsg());
 
     return writeConflictRetry(
-        opCtx, "writeDupDocToLostAndFoundCollection", nss.ns(), [&]() -> StatusWith<int> {
+        opCtx, "writeDupDocToLostAndFoundCollection", nss, [&]() -> StatusWith<int> {
             WriteUnitOfWork wuow(opCtx);
             Snapshotted<BSONObj> doc;
             int docSize = 0;
@@ -94,7 +128,7 @@ StatusWith<int> moveRecordToLostAndFound(OperationContext* opCtx,
 
             // Write document to lost_and_found collection and delete from original collection.
             Status status = collection_internal::insertDocument(
-                opCtx, localCollection, InsertStatement(doc.value()), nullptr);
+                opCtx, localCollection.getCollectionPtr(), InsertStatement(doc.value()), nullptr);
             if (!status.isOK()) {
                 return status;
             }
@@ -119,7 +153,7 @@ StatusWith<int> moveRecordToLostAndFound(OperationContext* opCtx,
 
 int repairMissingIndexEntry(OperationContext* opCtx,
                             const IndexCatalogEntry* index,
-                            const KeyString::Value& ks,
+                            const key_string::Value& ks,
                             const KeyFormat& keyFormat,
                             const NamespaceString& nss,
                             const CollectionPtr& coll,
@@ -130,7 +164,7 @@ int repairMissingIndexEntry(OperationContext* opCtx,
     int64_t numInserted = 0;
 
     Status insertStatus = Status::OK();
-    writeConflictRetry(opCtx, "insertingMissingIndexEntries", nss.ns(), [&] {
+    writeConflictRetry(opCtx, "insertingMissingIndexEntries", nss, [&] {
         WriteUnitOfWork wunit(opCtx);
         insertStatus =
             accessMethod->insertKeysAndUpdateMultikeyPaths(opCtx,
@@ -161,10 +195,10 @@ int repairMissingIndexEntry(OperationContext* opCtx,
 
         RecordId rid;
         if (keyFormat == KeyFormat::Long) {
-            rid = KeyString::decodeRecordIdLongAtEnd(ks.getBuffer(), ks.getSize());
+            rid = key_string::decodeRecordIdLongAtEnd(ks.getBuffer(), ks.getSize());
         } else {
             invariant(keyFormat == KeyFormat::String);
-            rid = KeyString::decodeRecordIdStrAtEnd(ks.getBuffer(), ks.getSize());
+            rid = key_string::decodeRecordIdStrAtEnd(ks.getBuffer(), ks.getSize());
         }
 
         auto dupKeyInfo = insertStatus.extraInfo<DuplicateKeyErrorInfo>();
@@ -196,7 +230,7 @@ int repairMissingIndexEntry(OperationContext* opCtx,
                 // duplicate records is in the index, so we need to add the newer record to the
                 // index.
                 if (dupKeyRid && ridToMove == *dupKeyRid) {
-                    writeConflictRetry(opCtx, "insertingMissingIndexEntries", nss.ns(), [&] {
+                    writeConflictRetry(opCtx, "insertingMissingIndexEntries", nss, [&] {
                         WriteUnitOfWork wunit(opCtx);
                         insertStatus = accessMethod->insertKeysAndUpdateMultikeyPaths(
                             opCtx, coll, index, {ks}, {}, {}, options, nullptr, nullptr);
@@ -209,7 +243,7 @@ int repairMissingIndexEntry(OperationContext* opCtx,
                 }
             } else {
                 results->errors.push_back(str::stream() << "unable to move record " << rid << " to "
-                                                        << lostAndFoundNss.ns());
+                                                        << lostAndFoundNss.toStringForErrorMsg());
             }
         } else {
             // If the missing index entry does not exist in the record store, then it has
