@@ -29,6 +29,9 @@
 
 #pragma once
 
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
 #include <functional>
 #include <iosfwd>
 #include <memory>
@@ -40,16 +43,23 @@
 #include "mongo/base/status_with.h"
 #include "mongo/base/string_data.h"
 #include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/db/catalog/collection.h"
 #include "mongo/db/catalog/collection_options.h"
 #include "mongo/db/catalog/index_build_block.h"
 #include "mongo/db/catalog/index_catalog.h"
+#include "mongo/db/catalog/index_catalog_entry.h"
 #include "mongo/db/catalog_raii.h"
 #include "mongo/db/index/index_access_method.h"
 #include "mongo/db/index/index_build_interceptor.h"
+#include "mongo/db/matcher/expression.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/record_id.h"
 #include "mongo/db/resumable_index_builds_gen.h"
+#include "mongo/db/storage/recovery_unit.h"
 #include "mongo/platform/mutex.h"
 #include "mongo/util/fail_point.h"
+#include "mongo/util/uuid.h"
 
 namespace mongo {
 
@@ -60,6 +70,7 @@ class CollectionPtr;
 class MatchExpression;
 class NamespaceString;
 class OperationContext;
+
 class ProgressMeterHolder;
 
 /**
@@ -109,10 +120,6 @@ public:
      * all indexes have been initialized. For callers that timestamp this write, use
      * 'makeTimestampedIndexOnInitFn', otherwise use 'kNoopOnInitFn'.
      *
-     * Callers can optionally limit the memory usage via 'maxMemoryUsageBytes'. If omitted, the
-     * default memory limit is controlled by the 'maxIndexBuildMemoryUsageMegabytes' server
-     * parameter.
-     *
      * Does not need to be called inside of a WriteUnitOfWork (but can be due to nesting).
      *
      * Requires holding an exclusive lock on the collection.
@@ -123,14 +130,11 @@ public:
         const std::vector<BSONObj>& specs,
         OnInitFn onInit,
         InitMode initMode = InitMode::SteadyState,
-        const boost::optional<ResumeIndexInfo>& resumeInfo = boost::none,
-        boost::optional<size_t> maxMemoryUsageBytes = boost::none);
-    StatusWith<std::vector<BSONObj>> init(
-        OperationContext* opCtx,
-        CollectionWriter& collection,
-        const BSONObj& spec,
-        OnInitFn onInit,
-        boost::optional<size_t> maxMemoryUsageBytes = boost::none);
+        const boost::optional<ResumeIndexInfo>& resumeInfo = boost::none);
+    StatusWith<std::vector<BSONObj>> init(OperationContext* opCtx,
+                                          CollectionWriter& collection,
+                                          const BSONObj& spec,
+                                          OnInitFn onInit);
     /**
      * Not all index initializations need an OnInitFn, in particular index builds that do not need
      * to timestamp catalog writes. This is a no-op.
