@@ -29,40 +29,76 @@
 
 #include "mongo/db/catalog/rename_collection.h"
 
+#include <algorithm>
+#include <boost/optional.hpp>
+#include <memory>
+#include <string>
+#include <vector>
+
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/bson/simple_bsonobj_comparator.h"
 #include "mongo/bson/unordered_fields_bsonobj_comparator.h"
+#include "mongo/db/auth/action_type.h"
+#include "mongo/db/auth/authorization_session.h"
+#include "mongo/db/auth/resource_pattern.h"
+#include "mongo/db/catalog/collection.h"
 #include "mongo/db/catalog/collection_catalog.h"
+#include "mongo/db/catalog/collection_options.h"
 #include "mongo/db/catalog/collection_uuid_mismatch.h"
 #include "mongo/db/catalog/collection_write_path.h"
+#include "mongo/db/catalog/database.h"
 #include "mongo/db/catalog/database_holder.h"
 #include "mongo/db/catalog/document_validation.h"
 #include "mongo/db/catalog/drop_collection.h"
 #include "mongo/db/catalog/index_catalog.h"
+#include "mongo/db/catalog/index_catalog_entry.h"
 #include "mongo/db/catalog/list_indexes.h"
 #include "mongo/db/catalog/local_oplog_info.h"
 #include "mongo/db/catalog/unique_collection_name.h"
-#include "mongo/db/client.h"
+#include "mongo/db/catalog_raii.h"
+#include "mongo/db/concurrency/d_concurrency.h"
 #include "mongo/db/concurrency/exception_util.h"
-#include "mongo/db/concurrency/lock_state.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
 #include "mongo/db/curop.h"
+#include "mongo/db/database_name.h"
 #include "mongo/db/db_raii.h"
 #include "mongo/db/index/index_descriptor.h"
 #include "mongo/db/index_builds_coordinator.h"
-#include "mongo/db/jsobj.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/op_observer/batched_write_policy.h"
 #include "mongo/db/op_observer/op_observer.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/ops/insert.h"
-#include "mongo/db/query/query_knobs_gen.h"
+#include "mongo/db/record_id.h"
+#include "mongo/db/repl/oplog.h"
 #include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/s/database_sharding_state.h"
 #include "mongo/db/s/operation_sharding_state.h"
-#include "mongo/db/server_options.h"
+#include "mongo/db/server_feature_flags_gen.h"
 #include "mongo/db/service_context.h"
+#include "mongo/db/stats/top.h"
+#include "mongo/db/storage/record_data.h"
+#include "mongo/db/storage/record_store.h"
 #include "mongo/db/storage/storage_parameters_gen.h"
+#include "mongo/db/storage/write_unit_of_work.h"
+#include "mongo/db/transaction_resources.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/fail_point.h"
 #include "mongo/util/namespace_string_util.h"
 #include "mongo/util/scopeguard.h"
+#include "mongo/util/str.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
 
@@ -94,8 +130,9 @@ Status checkSourceAndTargetNamespaces(OperationContext* opCtx,
     auto replCoord = repl::ReplicationCoordinator::get(opCtx);
     if (opCtx->writesAreReplicated() && !replCoord->canAcceptWritesFor(opCtx, source))
         return Status(ErrorCodes::NotWritablePrimary,
-                      str::stream() << "Not primary while renaming collection " << source << " to "
-                                    << target);
+                      str::stream() << "Not primary while renaming collection "
+                                    << source.toStringForErrorMsg() << " to "
+                                    << target.toStringForErrorMsg());
 
     if (isReplicatedChanged(opCtx, source, target))
         return {ErrorCodes::IllegalOperation,
@@ -112,9 +149,10 @@ Status checkSourceAndTargetNamespaces(OperationContext* opCtx,
     if (!sourceColl) {
         if (CollectionCatalog::get(opCtx)->lookupView(opCtx, source))
             return Status(ErrorCodes::CommandNotSupportedOnView,
-                          str::stream() << "cannot rename view: " << source);
+                          str::stream() << "cannot rename view: " << source.toStringForErrorMsg());
         return Status(ErrorCodes::NamespaceNotFound,
-                      str::stream() << "Source collection " << source.ns() << " does not exist");
+                      str::stream() << "Source collection " << source.toStringForErrorMsg()
+                                    << " does not exist");
     }
 
     // TODO SERVER-89089 move this check up in the rename stack execution once we have the guarantee
@@ -122,15 +160,17 @@ Status checkSourceAndTargetNamespaces(OperationContext* opCtx,
     if (source.isTimeseriesBucketsCollection() && sourceColl->getTimeseriesOptions() &&
         !target.isTimeseriesBucketsCollection()) {
         return Status(ErrorCodes::IllegalOperation,
-                      str::stream()
-                          << "Cannot rename timeseries buckets collection '" << source
-                          << "' to a namespace that is not timeseries buckets '" << target << "'.");
+                      str::stream() << "Cannot rename timeseries buckets collection '"
+                                    << source.toStringForErrorMsg()
+                                    << "' to a namespace that is not timeseries buckets '"
+                                    << target.toStringForErrorMsg() << "'.");
     }
 
     if (sourceColl->getCollectionOptions().encryptedFieldConfig &&
         !AuthorizationSession::get(opCtx->getClient())
-             ->isAuthorizedForActionsOnResource(ResourcePattern::forClusterResource(),
-                                                ActionType::setUserWriteBlockMode)) {
+             ->isAuthorizedForActionsOnResource(
+                 ResourcePattern::forClusterResource(target.tenantId()),
+                 ActionType::setUserWriteBlockMode)) {
         return Status(ErrorCodes::IllegalOperation, "Cannot rename an encrypted collection");
     }
 
@@ -141,12 +181,14 @@ Status checkSourceAndTargetNamespaces(OperationContext* opCtx,
     if (!targetColl) {
         if (CollectionCatalog::get(opCtx)->lookupView(opCtx, target))
             return Status(ErrorCodes::NamespaceExists,
-                          str::stream() << "a view already exists with that name: " << target);
+                          str::stream() << "a view already exists with that name: "
+                                        << target.toStringForErrorMsg());
     } else {
         if (targetColl->getCollectionOptions().encryptedFieldConfig &&
             !AuthorizationSession::get(opCtx->getClient())
-                 ->isAuthorizedForActionsOnResource(ResourcePattern::forClusterResource(),
-                                                    ActionType::setUserWriteBlockMode)) {
+                 ->isAuthorizedForActionsOnResource(
+                     ResourcePattern::forClusterResource(target.tenantId()),
+                     ActionType::setUserWriteBlockMode)) {
             return Status(ErrorCodes::IllegalOperation,
                           "Cannot rename to an existing encrypted collection");
         }
@@ -168,17 +210,20 @@ Status renameTargetCollectionToTmp(OperationContext* opCtx,
 
     // The generated unique collection name is only guaranteed to exist if the database is
     // exclusively locked.
-    invariant(opCtx->lockState()->isDbLockedForMode(targetDB->name(), LockMode::MODE_X));
+    invariant(shard_role_details::getLocker(opCtx)->isDbLockedForMode(targetDB->name(),
+                                                                      LockMode::MODE_X));
     auto tmpNameResult = makeUniqueCollectionName(opCtx, targetDB->name(), "tmp%%%%%.rename");
     if (!tmpNameResult.isOK()) {
         return tmpNameResult.getStatus().withContext(
             str::stream() << "Cannot generate a temporary collection name for the target "
-                          << targetNs << " (" << targetUUID << ") so that the source" << sourceNs
-                          << " (" << sourceUUID << ") could be renamed to " << targetNs);
+                          << targetNs.toStringForErrorMsg() << " (" << targetUUID
+                          << ") so that the source" << sourceNs.toStringForErrorMsg() << " ("
+                          << sourceUUID << ") could be renamed to "
+                          << targetNs.toStringForErrorMsg());
     }
     const auto& tmpName = tmpNameResult.getValue();
     const bool stayTemp = true;
-    return writeConflictRetry(opCtx, "renameCollection", targetNs.ns(), [&] {
+    return writeConflictRetry(opCtx, "renameCollection", targetNs, [&] {
         WriteUnitOfWork wunit(opCtx);
         auto status = targetDB->renameCollection(opCtx, targetNs, tmpName, stayTemp);
         if (!status.isOK())
@@ -187,8 +232,6 @@ Status renameTargetCollectionToTmp(OperationContext* opCtx,
         wunit.commit();
 
         LOGV2(20397,
-              "Successfully renamed the target {targetNs} ({targetUUID}) to {tmpName} so that the "
-              "source {sourceNs} ({sourceUUID}) could be renamed to {targetNs2}",
               "Successfully renamed the target so that the source could be renamed",
               "existingTargetNamespace"_attr = targetNs,
               "existingTargetUUID"_attr = targetUUID,
@@ -207,7 +250,7 @@ Status renameCollectionDirectly(OperationContext* opCtx,
                                 NamespaceString source,
                                 NamespaceString target,
                                 RenameCollectionOptions options) {
-    return writeConflictRetry(opCtx, "renameCollection", target.ns(), [&] {
+    return writeConflictRetry(opCtx, "renameCollection", target, [&] {
         WriteUnitOfWork wunit(opCtx);
 
         {
@@ -238,7 +281,7 @@ Status renameCollectionAndDropTarget(OperationContext* opCtx,
                                      const CollectionPtr& targetColl,
                                      RenameCollectionOptions options,
                                      repl::OpTime renameOpTimeFromApplyOps) {
-    return writeConflictRetry(opCtx, "renameCollection", target.ns(), [&] {
+    return writeConflictRetry(opCtx, "renameCollection", target, [&] {
         WriteUnitOfWork wunit(opCtx);
 
         // Target collection exists - drop it.
@@ -272,8 +315,6 @@ Status renameCollectionAndDropTarget(OperationContext* opCtx,
             if (!renameOpTime.isNull()) {
                 LOGV2_FATAL(
                     40616,
-                    "renameCollection: {from} to {to} (with dropTarget=true) - unexpected "
-                    "renameCollection oplog entry written to the oplog with optime {renameOpTime}",
                     "renameCollection (with dropTarget=true): unexpected renameCollection oplog "
                     "entry written to the oplog",
                     "from"_attr = source,
@@ -305,7 +346,7 @@ Status renameCollectionWithinDB(OperationContext* opCtx,
                                 const NamespaceString& source,
                                 const NamespaceString& target,
                                 RenameCollectionOptions options) {
-    invariant(source.db() == target.db());
+    invariant(source.isEqualDb(target));
     DisableDocumentValidation validationDisabler(opCtx);
 
     AutoGetDb autoDb(opCtx, source.dbName(), MODE_IX);
@@ -381,7 +422,7 @@ Status renameCollectionWithinDBForApplyOps(OperationContext* opCtx,
                                            const boost::optional<UUID>& uuidToDrop,
                                            repl::OpTime renameOpTimeFromApplyOps,
                                            const RenameCollectionOptions& options) {
-    invariant(source.db() == target.db());
+    invariant(source.isEqualDb(target));
     DisableDocumentValidation validationDisabler(opCtx);
 
     AutoGetDb autoDb(opCtx, source.dbName(), MODE_X);
@@ -402,7 +443,7 @@ Status renameCollectionWithinDBForApplyOps(OperationContext* opCtx,
         AutoStatsTracker::LogMode::kUpdateCurOp,
         CollectionCatalog::get(opCtx)->getDatabaseProfileLevel(source.dbName()));
 
-    return writeConflictRetry(opCtx, "renameCollection", target.ns(), [&] {
+    return writeConflictRetry(opCtx, "renameCollection", target, [&] {
         auto targetColl = CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, target);
         WriteUnitOfWork wuow(opCtx);
         if (targetColl) {
@@ -446,7 +487,7 @@ Status renameCollectionWithinDBForApplyOps(OperationContext* opCtx,
             invariant(options.dropTarget);
             auto collToDropBasedOnUUID = getNamespaceFromUUID(opCtx, uuidToDrop.value());
             if (collToDropBasedOnUUID && !collToDropBasedOnUUID->isDropPendingNamespace()) {
-                invariant(collToDropBasedOnUUID->db() == target.db());
+                invariant(collToDropBasedOnUUID->isEqualDb(target));
                 targetColl = CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(
                     opCtx, *collToDropBasedOnUUID);
             }
@@ -484,16 +525,17 @@ Status renameCollectionAcrossDatabases(OperationContext* opCtx,
                                        const NamespaceString& target,
                                        const RenameCollectionOptions& options) {
     invariant(
-        source.db() != target.db(),
+        !source.isEqualDb(target),
         str::stream()
             << "cannot rename within same database (use renameCollectionWithinDB instead): source: "
-            << source << "; target: " << target);
+            << source.toStringForErrorMsg() << "; target: " << target.toStringForErrorMsg());
 
     // Refer to txnCmdAllowlist in commands.cpp.
     invariant(!opCtx->inMultiDocumentTransaction(),
               str::stream() << "renameCollectionAcrossDatabases not supported in multi-document "
                                "transaction: source: "
-                            << source << "; target: " << target);
+                            << source.toStringForErrorMsg()
+                            << "; target: " << target.toStringForErrorMsg());
 
     uassert(ErrorCodes::InvalidOptions,
             "Cannot provide an expected collection UUID when renaming across databases",
@@ -501,7 +543,7 @@ Status renameCollectionAcrossDatabases(OperationContext* opCtx,
 
     boost::optional<Lock::DBLock> sourceDbLock;
     boost::optional<Lock::CollectionLock> sourceCollLock;
-    if (!opCtx->lockState()->isCollectionLockedForMode(source, MODE_S)) {
+    if (!shard_role_details::getLocker(opCtx)->isCollectionLockedForMode(source, MODE_S)) {
         // Lock the DB using MODE_IX to ensure we have the global lock in that mode, as to prevent
         // upgrade from MODE_IS to MODE_IX, which caused deadlock on systems not supporting Database
         // locking and should be avoided in general.
@@ -510,7 +552,7 @@ Status renameCollectionAcrossDatabases(OperationContext* opCtx,
     }
 
     boost::optional<Lock::DBLock> targetDBLock;
-    if (!opCtx->lockState()->isDbLockedForMode(target.dbName(), MODE_X)) {
+    if (!shard_role_details::getLocker(opCtx)->isDbLockedForMode(target.dbName(), MODE_X)) {
         targetDBLock.emplace(opCtx, target.dbName(), MODE_X);
     }
 
@@ -535,7 +577,7 @@ Status renameCollectionAcrossDatabases(OperationContext* opCtx,
     if (!sourceColl) {
         if (CollectionCatalog::get(opCtx)->lookupView(opCtx, source))
             return Status(ErrorCodes::CommandNotSupportedOnView,
-                          str::stream() << "cannot rename view: " << source);
+                          str::stream() << "cannot rename view: " << source.toStringForErrorMsg());
         return Status(ErrorCodes::NamespaceNotFound, "source namespace does not exist");
     }
 
@@ -544,9 +586,10 @@ Status renameCollectionAcrossDatabases(OperationContext* opCtx,
     if (source.isTimeseriesBucketsCollection() && sourceColl->getTimeseriesOptions() &&
         !target.isTimeseriesBucketsCollection()) {
         return Status(ErrorCodes::IllegalOperation,
-                      str::stream()
-                          << "Cannot rename timeseries buckets collection '" << source
-                          << "' to a namespace that is not timeseries buckets '" << target << "'.");
+                      str::stream() << "Cannot rename timeseries buckets collection '"
+                                    << source.toStringForErrorMsg()
+                                    << "' to a namespace that is not timeseries buckets '"
+                                    << target.toStringForErrorMsg() << "'.");
     }
 
     if (isReplicatedChanged(opCtx, source, target))
@@ -574,7 +617,8 @@ Status renameCollectionAcrossDatabases(OperationContext* opCtx,
 
     } else if (CollectionCatalog::get(opCtx)->lookupView(opCtx, target)) {
         return Status(ErrorCodes::NamespaceExists,
-                      str::stream() << "a view already exists with that name: " << target);
+                      str::stream() << "a view already exists with that name: "
+                                    << target.toStringForErrorMsg());
     }
 
     // Create a temporary collection in the target database. It will be removed if we fail to
@@ -585,7 +629,8 @@ Status renameCollectionAcrossDatabases(OperationContext* opCtx,
 
     // The generated unique collection name is only guaranteed to exist if the database is
     // exclusively locked.
-    invariant(opCtx->lockState()->isDbLockedForMode(targetDB->name(), LockMode::MODE_X));
+    invariant(shard_role_details::getLocker(opCtx)->isDbLockedForMode(targetDB->name(),
+                                                                      LockMode::MODE_X));
 
     // Note that this temporary collection name is used by MongoMirror and thus must not be changed
     // without consultation.
@@ -593,8 +638,9 @@ Status renameCollectionAcrossDatabases(OperationContext* opCtx,
         makeUniqueCollectionName(opCtx, target.dbName(), "tmp%%%%%.renameCollection");
     if (!tmpNameResult.isOK()) {
         return tmpNameResult.getStatus().withContext(
-            str::stream() << "Cannot generate temporary collection name to rename " << source
-                          << " to " << target);
+            str::stream() << "Cannot generate temporary collection name to rename "
+                          << source.toStringForErrorMsg() << " to "
+                          << target.toStringForErrorMsg());
     }
     const auto& tmpName = tmpNameResult.getValue();
 
@@ -604,13 +650,14 @@ Status renameCollectionAcrossDatabases(OperationContext* opCtx,
           "sourceCollection"_attr = source);
 
     // Renaming across databases will result in a new UUID.
-    NamespaceStringOrUUID tmpCollUUID{tmpName.dbName(), UUID::gen()};
+    NamespaceStringOrUUID tmpCollUUID{tmpName.dbName(),
+                                      options.newTargetCollectionUuid.get_value_or(UUID::gen())};
 
     {
         auto collectionOptions = sourceColl->getCollectionOptions();
         collectionOptions.uuid = tmpCollUUID.uuid();
 
-        writeConflictRetry(opCtx, "renameCollection", tmpName.ns(), [&] {
+        writeConflictRetry(opCtx, "renameCollection", tmpName, [&] {
             WriteUnitOfWork wunit(opCtx);
             targetDB->createCollection(opCtx, tmpName, collectionOptions);
             wunit.commit();
@@ -633,8 +680,6 @@ Status renameCollectionAcrossDatabases(OperationContext* opCtx,
             // Ignoring failure case when dropping the temporary collection during cleanup because
             // the rename operation has already failed for another reason.
             LOGV2(705521,
-                  "Unable to drop temporary collection {tmpName} while renaming from {source} to "
-                  "{target}: {error}",
                   "Unable to drop temporary collection while renaming",
                   "tempCollection"_attr = tmpName,
                   "source"_attr = source,
@@ -667,7 +712,7 @@ Status renameCollectionAcrossDatabases(OperationContext* opCtx,
     // index in an unfinished state. For more information on assigning timestamps to multiple index
     // builds, please see SERVER-35780 and SERVER-35070.
     if (!indexesToCopy.empty()) {
-        Status status = writeConflictRetry(opCtx, "renameCollection", tmpName.ns(), [&] {
+        Status status = writeConflictRetry(opCtx, "renameCollection", tmpName, [&] {
             WriteUnitOfWork wunit(opCtx);
             auto fromMigrate = false;
             try {
@@ -696,7 +741,7 @@ Status renameCollectionAcrossDatabases(OperationContext* opCtx,
         AutoGetCollection autoTmpColl(opCtx, tmpCollUUID, MODE_IX);
         if (!autoTmpColl) {
             return Status(ErrorCodes::NamespaceNotFound,
-                          str::stream() << "Temporary collection '" << tmpName
+                          str::stream() << "Temporary collection '" << tmpName.toStringForErrorMsg()
                                         << "' was removed while renaming collection across DBs");
         }
 
@@ -715,7 +760,7 @@ Status renameCollectionAcrossDatabases(OperationContext* opCtx,
             opCtx->checkForInterrupt();
             // Cursor is left one past the end of the batch inside writeConflictRetry.
             auto beginBatchId = record->id;
-            Status status = writeConflictRetry(opCtx, "renameCollection", tmpName.ns(), [&] {
+            Status status = writeConflictRetry(opCtx, "renameCollection", tmpName, [&] {
                 // Always reposition cursor in case it gets a WCE midway through.
                 record = cursor->seekExact(beginBatchId);
 
@@ -732,7 +777,9 @@ Status renameCollectionAcrossDatabases(OperationContext* opCtx,
                     canBeBatched);
 
                 bool isGroupedOplogEntries = stmts.size() > 1U;
-                WriteUnitOfWork wunit(opCtx, isGroupedOplogEntries);
+                WriteUnitOfWork wunit(opCtx,
+                                      isGroupedOplogEntries ? WriteUnitOfWork::kGroupForTransaction
+                                                            : WriteUnitOfWork::kDontGroup);
 
                 if (!isOplogDisabledForTmpColl && !isGroupedOplogEntries) {
                     auto oplogInfo = LocalOplogInfo::get(opCtx);
@@ -767,7 +814,7 @@ Status renameCollectionAcrossDatabases(OperationContext* opCtx,
 
                 cursor->save();
                 // When this exits via success or WCE, we need to restore the cursor.
-                ON_BLOCK_EXIT([opCtx, ns = tmpName.ns(), &cursor]() {
+                ON_BLOCK_EXIT([opCtx, ns = tmpName, &cursor]() {
                     writeConflictRetry(
                         opCtx, "retryRestoreCursor", ns, [&cursor] { cursor->restore(); });
                 });
@@ -782,7 +829,7 @@ Status renameCollectionAcrossDatabases(OperationContext* opCtx,
 
     // Getting here means we successfully built the target copy. We now do the final
     // in-place rename and remove the source collection.
-    invariant(tmpName.db() == target.db());
+    invariant(tmpName.isEqualDb(target));
     RenameCollectionOptions tempOptions(options);
     Status status = renameCollectionWithinDB(opCtx, tmpName, target, tempOptions);
     if (!status.isOK())
@@ -841,16 +888,16 @@ Status checkTargetCollectionIndexesMatch(const NamespaceString& targetNss,
 
 void validateNamespacesForRenameCollection(OperationContext* opCtx,
                                            const NamespaceString& source,
-                                           const NamespaceString& target) {
+                                           const NamespaceString& target,
+                                           const RenameCollectionOptions& options) {
     uassert(ErrorCodes::InvalidNamespace,
-            str::stream() << "Invalid source namespace: " << source.ns(),
+            str::stream() << "Invalid source namespace: " << source.toStringForErrorMsg(),
             source.isValid());
     uassert(ErrorCodes::InvalidNamespace,
-            str::stream() << "Invalid target namespace: " << target.ns(),
+            str::stream() << "Invalid target namespace: " << target.toStringForErrorMsg(),
             target.isValid());
 
-    if ((repl::ReplicationCoordinator::get(opCtx)->getReplicationMode() !=
-         repl::ReplicationCoordinator::modeNone)) {
+    if (repl::ReplicationCoordinator::get(opCtx)->getSettings().isReplSet()) {
         uassert(ErrorCodes::IllegalOperation,
                 "can't rename live oplog while replicating",
                 !source.isOplog());
@@ -882,7 +929,7 @@ void validateNamespacesForRenameCollection(OperationContext* opCtx,
     uassert(ErrorCodes::NamespaceNotFound,
             str::stream() << "renameCollection cannot accept a source collection that is in a "
                              "drop-pending state: "
-                          << source,
+                          << source.toStringForErrorMsg(),
             !source.isDropPendingNamespace());
 
     uassert(ErrorCodes::IllegalOperation,
@@ -901,8 +948,26 @@ void validateNamespacesForRenameCollection(OperationContext* opCtx,
         uassert(ErrorCodes::IllegalOperation,
                 "Renaming system.buckets collections is not allowed",
                 AuthorizationSession::get(opCtx->getClient())
-                    ->isAuthorizedForActionsOnResource(ResourcePattern::forClusterResource(),
-                                                       ActionType::setUserWriteBlockMode));
+                    ->isAuthorizedForActionsOnResource(
+                        ResourcePattern::forClusterResource(target.tenantId()),
+                        ActionType::setUserWriteBlockMode));
+    }
+
+    if (gFeatureFlagDisallowBucketCollectionWithoutTimeseriesOptions.isEnabled(
+            serverGlobalParams.featureCompatibility.acquireFCVSnapshot())) {
+        uassert(ErrorCodes::IllegalOperation,
+                str::stream() << "Cannot rename non timeseries buckets collection '"
+                              << source.toStringForErrorMsg()
+                              << "' to a timeseries buckets namespace '"
+                              << target.toStringForErrorMsg() << "'.",
+                source.isTimeseriesBucketsCollection() || !target.isTimeseriesBucketsCollection());
+
+        uassert(ErrorCodes::IllegalOperation,
+                str::stream() << "Cannot rename timeseries buckets collection '"
+                              << source.toStringForErrorMsg()
+                              << "' to a non timeseries buckets namespace '"
+                              << target.toStringForErrorMsg() << "'.",
+                !source.isTimeseriesBucketsCollection() || target.isTimeseriesBucketsCollection());
     }
 }
 
@@ -912,7 +977,7 @@ void validateAndRunRenameCollection(OperationContext* opCtx,
                                     const RenameCollectionOptions& options) {
     invariant(source != target, "Can't rename a collection to itself");
 
-    validateNamespacesForRenameCollection(opCtx, source, target);
+    validateNamespacesForRenameCollection(opCtx, source, target, options);
 
     OperationShardingState::ScopedAllowImplicitCollectionCreate_UNSAFE unsafeCreateCollection(
         opCtx);
@@ -927,7 +992,7 @@ Status renameCollection(OperationContext* opCtx,
         return Status(ErrorCodes::NamespaceNotFound,
                       str::stream() << "renameCollection() cannot accept a source "
                                        "collection that is in a drop-pending state: "
-                                    << source);
+                                    << source.toStringForErrorMsg());
     }
 
     if (source.isSystemDotViews() || target.isSystemDotViews()) {
@@ -948,13 +1013,12 @@ Status renameCollection(OperationContext* opCtx,
 
     StringData dropTargetMsg = options.dropTarget ? "yes"_sd : "no"_sd;
     LOGV2(20400,
-          "renameCollectionForCommand: rename {source} to {target}{dropTargetMsg}",
           "renameCollectionForCommand",
           "sourceNamespace"_attr = source,
           "targetNamespace"_attr = target,
           "dropTarget"_attr = dropTargetMsg);
 
-    if (source.db() == target.db())
+    if (source.isEqualDb(target))
         return renameCollectionWithinDB(opCtx, source, target, options);
     else {
         return renameCollectionAcrossDatabases(opCtx, source, target, options);
@@ -977,8 +1041,10 @@ Status renameCollectionForApplyOps(OperationContext* opCtx,
     const auto sourceNsElt = cmd["renameCollection"];
     const auto targetNsElt = cmd["to"];
 
-    NamespaceString sourceNss{NamespaceStringUtil::deserialize(tid, sourceNsElt.valueStringData())};
-    NamespaceString targetNss{NamespaceStringUtil::deserialize(tid, targetNsElt.valueStringData())};
+    NamespaceString sourceNss{NamespaceStringUtil::deserialize(
+        tid, sourceNsElt.valueStringData(), SerializationContext::stateDefault())};
+    NamespaceString targetNss{NamespaceStringUtil::deserialize(
+        tid, targetNsElt.valueStringData(), SerializationContext::stateDefault())};
 
     // TODO: not needed once we are no longer parsing for prefixed tenantIds
     uassert(ErrorCodes::IllegalOperation,
@@ -1016,18 +1082,18 @@ Status renameCollectionForApplyOps(OperationContext* opCtx,
                       str::stream() << "error with target namespace: " << targetStatus.reason());
     }
 
-    if ((repl::ReplicationCoordinator::get(opCtx)->getReplicationMode() ==
-         repl::ReplicationCoordinator::modeNone) &&
+    if (!repl::ReplicationCoordinator::get(opCtx)->getSettings().isReplSet() &&
         targetNss.isOplog()) {
         return Status(ErrorCodes::IllegalOperation,
                       str::stream() << "Cannot rename collection to the oplog");
     }
 
-    // Take global IX lock explicitly to avoid upgrading from IS later
-    Lock::GlobalLock globalLock(opCtx, MODE_IX);
-    AutoGetCollectionForRead sourceColl(
+    // Take strong database and collection locks in order to avoid upgrading later.
+    AutoGetDb sourceDb(opCtx, sourceNss.dbName(), MODE_X);
+    AutoGetCollection sourceColl(
         opCtx,
         sourceNss,
+        MODE_X,
         AutoGetCollection::Options{}.viewMode(auto_get_collection::ViewMode::kViewsPermitted));
 
     if (sourceNss.isDropPendingNamespace() || !sourceColl) {
@@ -1052,21 +1118,19 @@ Status renameCollectionForApplyOps(OperationContext* opCtx,
                       str::stream()
                           << "renameCollection() cannot accept a source "
                              "collection that does not exist or is in a drop-pending state: "
-                          << sourceNss.toString());
+                          << sourceNss.toStringForErrorMsg());
     }
 
     const std::string uuidToDropString = uuidToDrop ? uuidToDrop->toString() : "<none>";
     const std::string uuidString = uuidToRename ? uuidToRename->toString() : "UUID unknown";
     LOGV2(20401,
-          "renameCollectionForApplyOps: rename {sourceNss} ({uuidString}) to "
-          "{targetNss}{dropTargetMsg}",
           "renameCollectionForApplyOps",
           "sourceNamespace"_attr = sourceNss,
           "uuid"_attr = uuidString,
           "targetNamespace"_attr = targetNss,
           "uuidToDrop"_attr = uuidToDropString);
 
-    if (sourceNss.db() == targetNss.db()) {
+    if (sourceNss.isEqualDb(targetNss)) {
         return renameCollectionWithinDBForApplyOps(
             opCtx, sourceNss, targetNss, uuidToDrop, renameOpTime, options);
     } else {
@@ -1080,13 +1144,13 @@ Status renameCollectionForRollback(OperationContext* opCtx,
     // If the UUID we're targeting already exists, rename from there no matter what.
     auto source = getNamespaceFromUUID(opCtx, uuid);
     invariant(source);
-    invariant(source->db() == target.db(),
+    invariant(source->isEqualDb(target),
               str::stream() << "renameCollectionForRollback: source and target namespaces must "
                                "have the same database. source: "
-                            << *source << ". target: " << target);
+                            << (*source).toStringForErrorMsg()
+                            << ". target: " << target.toStringForErrorMsg());
 
     LOGV2(20402,
-          "renameCollectionForRollback: rename {source} ({uuid}) to {target}.",
           "renameCollectionForRollback",
           "source"_attr = *source,
           "uuid"_attr = uuid,
