@@ -82,9 +82,6 @@ std::map<StringData, boost::optional<bool>> getFlagsFromStorageEngineBson(
 
 boost::optional<bool> getFlagFromStorageEngineBson(const BSONObj& storageEngineOptions,
                                                    StringData flagName) {
-    if (storageEngineOptions.isEmpty()) {
-        return boost::none;
-    }
     return getFlagsFromStorageEngineBson(storageEngineOptions, {flagName})[flagName];
 }
 
@@ -94,7 +91,7 @@ static size_t findOrAddAppMetadataStructToConfigString(std::string& configString
     auto matches = appMetadataRegex->matchView(configString);
     if (!matches)
         configString += configString.empty() ? "app_metadata=()" : ",app_metadata=()";
-    return matches ? (matches[0].begin() + matches[0].size() - configString.data())
+    return matches ? (matches[0].data() + matches[0].size() - configString.data())
                    : configString.size() - 1;
 }
 
@@ -118,7 +115,7 @@ static void setFlagsToWtConfigStringAppMetadata(
         auto flagRegex = flagMatchRegex(flagName);
         auto matches = flagRegex.matchView(configString, {}, metadataPos);
         if (matches) {
-            size_t pos = matches[0].begin() - configString.data(), len = matches[0].size();
+            size_t pos = matches[0].data() - configString.data(), len = matches[0].size();
 
             if (flagValue.has_value()) {  // Replace existing flag
                 auto flagItem = fmt::format("{}={}", flagName, *flagValue);
