@@ -29,6 +29,21 @@
 
 #include "uncommitted_catalog_updates.h"
 
+#include <absl/container/node_hash_map.h>
+#include <algorithm>
+#include <boost/container/vector.hpp>
+#include <boost/none.hpp>
+#include <utility>
+
+#include <boost/container/small_vector.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/db/catalog/collection_catalog.h"
+#include "mongo/db/storage/recovery_unit.h"
+#include "mongo/db/transaction_resources.h"
+#include "mongo/util/decorable.h"
+
 namespace mongo {
 
 namespace {
@@ -40,7 +55,7 @@ const RecoveryUnit::Snapshot::Decoration<OpenedCollections> getOpenedCollections
 }  // namespace
 
 UncommittedCatalogUpdates& UncommittedCatalogUpdates::get(OperationContext* opCtx) {
-    return getUncommittedCatalogUpdates(opCtx->recoveryUnit()->getSnapshot());
+    return getUncommittedCatalogUpdates(shard_role_details::getRecoveryUnit(opCtx)->getSnapshot());
 }
 
 UncommittedCatalogUpdates::CollectionLookupResult UncommittedCatalogUpdates::lookupCollection(
@@ -105,41 +120,6 @@ void UncommittedCatalogUpdates::_createCollection(OperationContext* opCtx,
     const auto& nss = coll->ns();
     auto uuid = coll->uuid();
     _entries.push_back({action, coll, nss, uuid});
-
-    // When we create a collection after a drop we skip registering the collection in the
-    // preCommitHook and register it during the same commit handler that we unregister the
-    // collection.
-    if (action == Entry::Action::kCreatedCollection) {
-        opCtx->recoveryUnit()->registerPreCommitHook([uuid](OperationContext* opCtx) {
-            auto uncommittedCatalogUpdates = UncommittedCatalogUpdates::get(opCtx);
-            auto [found, createdColl, newColl] = lookupCollection(opCtx, uuid);
-            if (!createdColl) {
-                return;
-            }
-
-            // Invariant that a collection is found.
-            invariant(createdColl.get(), uuid.toString());
-
-            // This will throw when registering a namespace which is already in use.
-            CollectionCatalog::write(opCtx, [&, coll = createdColl](CollectionCatalog& catalog) {
-                catalog.registerCollectionTwoPhase(opCtx, coll, /*ts=*/boost::none);
-            });
-
-            opCtx->recoveryUnit()->onRollback([uuid](OperationContext* opCtx) {
-                CollectionCatalog::write(opCtx, [&](CollectionCatalog& catalog) {
-                    catalog.deregisterCollection(
-                        opCtx, uuid, /*isDropPending=*/false, /*ts=*/boost::none);
-                });
-            });
-
-            globalFailPointRegistry().find("hangAfterPreCommittingCatalogUpdates")->pauseWhileSet();
-        });
-    }
-
-    // We hold a reference to prevent the collection from being deleted when `PublishCatalogUpdates`
-    // runs its rollback handler as that happens first. Other systems may have setup some rollback
-    // handler that need to interact with this collection.
-    opCtx->recoveryUnit()->onRollback([coll](OperationContext*) {});
 }
 
 void UncommittedCatalogUpdates::writableCollection(std::shared_ptr<Collection> collection) {
@@ -225,12 +205,13 @@ void UncommittedCatalogUpdates::replaceViewsForDatabase(const DatabaseName& dbNa
 }
 
 void UncommittedCatalogUpdates::addView(OperationContext* opCtx, const NamespaceString& nss) {
-    opCtx->recoveryUnit()->registerPreCommitHook([nss](OperationContext* opCtx) {
-        CollectionCatalog::write(opCtx, [opCtx, nss](CollectionCatalog& catalog) {
-            catalog.registerUncommittedView(opCtx, nss);
+    shard_role_details::getRecoveryUnit(opCtx)->registerPreCommitHook(
+        [nss](OperationContext* opCtx) {
+            CollectionCatalog::write(opCtx, [opCtx, nss](CollectionCatalog& catalog) {
+                catalog.registerUncommittedView(opCtx, nss);
+            });
         });
-    });
-    opCtx->recoveryUnit()->onRollback([nss](OperationContext* opCtx) {
+    shard_role_details::getRecoveryUnit(opCtx)->onRollback([nss](OperationContext* opCtx) {
         CollectionCatalog::write(
             opCtx, [&](CollectionCatalog& catalog) { catalog.deregisterUncommittedView(nss); });
     });
@@ -271,7 +252,7 @@ bool UncommittedCatalogUpdates::isCreatedCollection(OperationContext* opCtx,
 }
 
 OpenedCollections& OpenedCollections::get(OperationContext* opCtx) {
-    return getOpenedCollections(opCtx->recoveryUnit()->getSnapshot());
+    return getOpenedCollections(shard_role_details::getRecoveryUnit(opCtx)->getSnapshot());
 }
 
 boost::optional<std::shared_ptr<const Collection>> OpenedCollections::lookupByNamespace(
