@@ -28,47 +28,72 @@
  */
 
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/catalog/validate_adaptor.h"
-
+#include <absl/container/flat_hash_map.h>
+#include <algorithm>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <climits>
 #include <fmt/format.h>
+#include <memory>
+#include <mutex>
+#include <stdexcept>
+#include <string>
+#include <utility>
+#include <vector>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bson_validate.h"
+#include "mongo/bson/bsonelement.h"
 #include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/bson/oid.h"
+#include "mongo/bson/timestamp.h"
 #include "mongo/bson/util/bson_extract.h"
 #include "mongo/bson/util/bsoncolumn.h"
-#include "mongo/db/catalog/backwards_compatible_collection_options_util.h"
+#include "mongo/db/catalog/clustered_collection_options_gen.h"
 #include "mongo/db/catalog/clustered_collection_util.h"
 #include "mongo/db/catalog/collection.h"
+#include "mongo/db/catalog/collection_impl.h"
 #include "mongo/db/catalog/column_index_consistency.h"
 #include "mongo/db/catalog/index_catalog.h"
 #include "mongo/db/catalog/index_consistency.h"
-#include "mongo/db/catalog/storage_engine_collection_options_flags_parser.h"
 #include "mongo/db/catalog/throttle_cursor.h"
+#include "mongo/db/catalog/validate_adaptor.h"
+#include "mongo/db/client.h"
 #include "mongo/db/concurrency/exception_util.h"
 #include "mongo/db/curop.h"
-#include "mongo/db/index/columns_access_method.h"
-#include "mongo/db/index/index_access_method.h"
 #include "mongo/db/index/index_descriptor.h"
-#include "mongo/db/index/wildcard_access_method.h"
+#include "mongo/db/index_names.h"
 #include "mongo/db/matcher/expression.h"
-#include "mongo/db/multi_key_path_tracker.h"
+#include "mongo/db/namespace_string.h"
 #include "mongo/db/operation_context.h"
+#include "mongo/db/query/collation/collator_interface.h"
 #include "mongo/db/record_id_helpers.h"
-#include "mongo/db/server_feature_flags_gen.h"
-#include "mongo/db/storage/execution_context.h"
 #include "mongo/db/storage/key_string.h"
 #include "mongo/db/storage/record_store.h"
-#include "mongo/db/storage/storage_parameters_gen.h"
+#include "mongo/db/storage/write_unit_of_work.h"
 #include "mongo/db/timeseries/bucket_catalog/flat_bson.h"
 #include "mongo/db/timeseries/timeseries_constants.h"
 #include "mongo/db/timeseries/timeseries_extended_range.h"
 #include "mongo/db/timeseries/timeseries_gen.h"
 #include "mongo/db/timeseries/timeseries_options.h"
 #include "mongo/logv2/log.h"
-#include "mongo/rpc/object_check.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/rpc/object_check.h"  // IWYU pragma: keep
+#include "mongo/util/assert_util.h"
+#include "mongo/util/concurrency/with_lock.h"
 #include "mongo/util/fail_point.h"
+#include "mongo/util/scopeguard.h"
+#include "mongo/util/shared_buffer_fragment.h"
+#include "mongo/util/str.h"
+#include "mongo/util/string_map.h"
 #include "mongo/util/testing_proctor.h"
+#include "mongo/util/time_support.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
 
@@ -92,6 +117,10 @@ static constexpr const char* kTimeseriesValidationInconsistencyReason =
 static constexpr const char* kBSONValidationNonConformantReason =
     "Detected one or more documents in this collection not conformant to BSON specifications. For "
     "more info, see logs with log id 6825900";
+static constexpr const char* kTimeseriesBucketingParametersChangedInconsistencyReason =
+    "A time series bucketing parameter was changed in this collection but "
+    "timeseriesBucketingParametersChanged is not true. For more info, see logs with log id "
+    "9175400.";
 
 /**
  * Validate that for each record in a clustered RecordStore the record key (RecordId) matches the
@@ -112,8 +141,8 @@ void _validateClusteredCollectionRecordId(OperationContext* opCtx,
     }
 
     const auto ksFromBSON =
-        KeyString::Builder(KeyString::Version::kLatestVersion, ridFromDoc.getValue());
-    const auto ksFromRid = KeyString::Builder(KeyString::Version::kLatestVersion, rid);
+        key_string::Builder(key_string::Version::kLatestVersion, ridFromDoc.getValue());
+    const auto ksFromRid = key_string::Builder(key_string::Version::kLatestVersion, rid);
 
     const auto clusterKeyField = clustered_util::getClusterKeyFieldName(indexSpec);
     if (ksFromRid != ksFromBSON) {
@@ -143,9 +172,48 @@ void schemaValidationFailed(CollectionValidation::ValidateState* state,
     }
 }
 
+
+void _timeseriesBucketingParametersChangedCheckFail(const CollectionPtr& coll,
+                                                    CollectionValidation::ValidateState* state,
+                                                    ValidateResults* results,
+                                                    bool isTimeseriesBucketingParameterChangedSet) {
+    if (state->isTimeseriesBucketingParametersChangedInconsistent()) {
+        // Only report the error message once.
+        return;
+    }
+    state->isTimeseriesBucketingParametersChangedInconsistent();
+
+    if (!isTimeseriesBucketingParameterChangedSet) {
+        // Get the original timeseries bucketing parameters.
+        auto originalTimeseriesOptions = coll->getTimeseriesOptions();
+        invariant(originalTimeseriesOptions != boost::none);
+        const auto options = originalTimeseriesOptions.get();
+        auto originalBucketMaxSpanSeconds = options.getBucketMaxSpanSeconds();
+        auto originalBucketRoundingSeconds = options.getBucketRoundingSeconds();
+        auto originalGranularity = options.getGranularity();
+
+        LOGV2_ERROR(9175400,
+                    "A time series bucketing parameter was changed",
+                    logAttrs(coll->ns()),
+                    "currentBucketMaxSpanSeconds"_attr = originalBucketMaxSpanSeconds,
+                    "currentBucketRoundingSeconds"_attr = originalBucketRoundingSeconds,
+                    "currentGranularity"_attr = originalGranularity);
+        results->errors.push_back(kTimeseriesBucketingParametersChangedInconsistencyReason);
+        results->valid = false;
+    }
+}
+
 // Checks that 'control.count' matches the actual number of measurements in a closed bucket.
-Status _validateTimeseriesCount(const BSONObj& control, int bucketCount, int version) {
-    if (version == 1) {
+Status _validateTimeseriesCount(const BSONObj& control,
+                                int bucketCount,
+                                int version,
+                                bool shouldDecompressBSON) {
+    // Skips the check if a bucket is compressed, but we are not in a validate mode that will
+    // decompress the bucket to actually go through the measurements.
+    if (version == timeseries::kTimeseriesControlUncompressedVersion ||
+        ((version == timeseries::kTimeseriesControlCompressedSortedVersion ||
+          timeseries::kTimeseriesControlCompressedUnsortedVersion) &&
+         !shouldDecompressBSON)) {
         return Status::OK();
     }
     long long controlCount;
@@ -195,9 +263,9 @@ Status _validateTimeSeriesIdTimestamp(OperationContext* opCtx,
  * Checks the value of the bucket's version.
  */
 Status _validateTimeseriesControlVersion(const BSONObj& recordBson, int bucketVersion) {
-    // Version 3 buckets are introduced in 8.0, but for backwards compatibility we recognize it as a
-    // valid version on 7.0 as well.
-    if (bucketVersion != 1 && bucketVersion != 2 && bucketVersion != 3) {
+    if (bucketVersion != timeseries::kTimeseriesControlUncompressedVersion &&
+        bucketVersion != timeseries::kTimeseriesControlCompressedSortedVersion &&
+        bucketVersion != timeseries::kTimeseriesControlCompressedUnsortedVersion) {
         return Status(
             ErrorCodes::BadValue,
             fmt::format("Invalid value for 'control.version'. Expected 1, 2, or 3, but got {}.",
@@ -210,10 +278,12 @@ Status _validateTimeseriesControlVersion(const BSONObj& recordBson, int bucketVe
  * Checks if the bucket's version matches the types of 'data' fields.
  */
 Status _validateTimeseriesDataFieldTypes(const BSONElement& dataField, int bucketVersion) {
-    auto dataType = bucketVersion == 1 ? BSONType::Object : BSONType::BinData;
+    auto dataType = (bucketVersion == timeseries::kTimeseriesControlUncompressedVersion)
+        ? BSONType::Object
+        : BSONType::BinData;
     // Checks that open buckets have 'Object' type and closed buckets have 'BinData Column' type.
     auto isCorrectType = [&](BSONElement el) {
-        if (bucketVersion == 1) {
+        if (bucketVersion == timeseries::kTimeseriesControlUncompressedVersion) {
             return el.type() == BSONType::Object;
         } else {
             return el.type() == BSONType::BinData && el.binDataType() == BinDataType::Column;
@@ -230,6 +300,53 @@ Status _validateTimeseriesDataFieldTypes(const BSONElement& dataField, int bucke
     return Status::OK();
 }
 
+
+/*
+ * Checks that only buckets that have timeSeriesBucketingParameters flag set have changed
+ * bucket parameters.
+ */
+Status _validateTimeseriesBucketingParametersChanged(const CollectionPtr& coll,
+                                                     timeseries::bucket_catalog::MinMax& minmax,
+                                                     const BSONElement& controlMin,
+                                                     const BSONElement& controlMax,
+                                                     StringData fieldName,
+                                                     CollectionValidation::ValidateState* state,
+                                                     ValidateResults* results,
+                                                     int version,
+                                                     bool shouldDecompressBSON) {
+    // Skips the check if a bucket is compressed, but we are not in a validate mode that will
+    // decompress the bucket to actually go through the measurements.
+    if ((version == timeseries::kTimeseriesControlCompressedSortedVersion ||
+         version == timeseries::kTimeseriesControlCompressedUnsortedVersion) &&
+        !shouldDecompressBSON) {
+        return Status::OK();
+    }
+
+    auto timeseriesBucketingParametersHaveChanged =
+        coll->timeseriesBucketingParametersHaveChanged();
+    auto timeseriesBucketingParametersHaveChangedVal =
+        timeseriesBucketingParametersHaveChanged == boost::none
+        ? true
+        : timeseriesBucketingParametersHaveChanged.value();
+
+    auto min = minmax.min();
+    auto max = minmax.max();
+
+    auto checkTimeSeriesBucketingParametersChanged = [&]() {
+        const auto options = coll->getTimeseriesOptions().value();
+        auto roundedDownTimeStamp =
+            timeseries::roundTimestampToGranularity(min.getField(fieldName).date(), options);
+        return roundedDownTimeStamp < controlMin.Date();
+    };
+
+    if (checkTimeSeriesBucketingParametersChanged()) {
+        _timeseriesBucketingParametersChangedCheckFail(
+            coll, state, results, timeseriesBucketingParametersHaveChangedVal);
+    }
+
+    return Status::OK();
+}
+
 /**
  * Checks whether the min and max values between 'control' and 'data' match, taking timestamp
  * granularity into account.
@@ -238,7 +355,16 @@ Status _validateTimeSeriesMinMax(const CollectionPtr& coll,
                                  timeseries::bucket_catalog::MinMax& minmax,
                                  const BSONElement& controlMin,
                                  const BSONElement& controlMax,
-                                 StringData fieldName) {
+                                 StringData fieldName,
+                                 int version,
+                                 bool shouldDecompressBSON) {
+    // Skips the check if a bucket is compressed, but we are not in a validate mode that will
+    // decompress the bucket to actually go through the measurements.
+    if ((version == timeseries::kTimeseriesControlCompressedSortedVersion ||
+         version == timeseries::kTimeseriesControlCompressedUnsortedVersion) &&
+        !shouldDecompressBSON) {
+        return Status::OK();
+    }
     auto min = minmax.min();
     auto max = minmax.max();
     auto checkMinAndMaxMatch = [&]() {
@@ -314,10 +440,14 @@ Status _validateTimeSeriesDataTimeField(const CollectionPtr& coll,
                                         const BSONElement& controlMin,
                                         const BSONElement& controlMax,
                                         StringData fieldName,
+                                        CollectionValidation::ValidateState* state,
+                                        ValidateResults* results,
                                         int version,
-                                        int* bucketCount) {
-    timeseries::bucket_catalog::MinMax minmax;
-    if (version == 1) {
+                                        int* bucketCount,
+                                        bool shouldDecompressBSON) {
+    TrackingContext trackingContext;
+    timeseries::bucket_catalog::MinMax minmax{trackingContext};
+    if (version == timeseries::kTimeseriesControlUncompressedVersion) {
         for (const auto& metric : timeField.Obj()) {
             if (metric.type() != BSONType::Date) {
                 return Status(ErrorCodes::BadValue,
@@ -325,53 +455,81 @@ Status _validateTimeSeriesDataTimeField(const CollectionPtr& coll,
             }
             // Checks that indices are consecutively increasing numbers starting from 0.
             if (auto idx = _idxInt(metric.fieldNameStringData()); idx != *bucketCount) {
-                return Status(ErrorCodes::BadValue,
-                              fmt::format("The index '{}' in time-series bucket data field '{}' is "
-                                          "not consecutively increasing from '0'",
-                                          metric.fieldNameStringData(),
-                                          fieldName));
+                return Status(
+                    ErrorCodes::BadValue,
+                    fmt::format("The indexes in time-series bucket data field '{}' is "
+                                "not consecutively increasing from '0'. Expected: {}, but got: {}",
+                                fieldName,
+                                *bucketCount,
+                                idx));
             }
             minmax.update(metric.wrap(fieldName), boost::none, coll->getDefaultCollator());
             ++(*bucketCount);
         }
-    } else {
-        BSONColumn col{timeField};
-        Date_t prevTimestamp = Date_t::min();
-        bool detectedOutOfOrder = false;
-        for (const auto& metric : col) {
-            if (!metric.eoo()) {
-                if (metric.type() != BSONType::Date) {
-                    return Status(
-                        ErrorCodes::BadValue,
-                        fmt::format("Time-series bucket '{}' field is not a Date", fieldName));
-                }
-                // Check the time values are sorted in increasing order for v2 buckets (compressed,
-                // sorted). Skip the check if the bucket is v3 (compressed, unsorted).
-                Date_t curTimestamp = metric.Date();
-                if (curTimestamp < prevTimestamp) {
-                    if (version == 2) {
+    } else if (shouldDecompressBSON) {
+        // Only decompress the bucket if we are in full validation mode, kBackgroundCheckBSON mode,
+        // or kForegroundCheckBSON mode since this is a relatively expensive operation.
+        try {
+            BSONColumn col{timeField};
+            Date_t prevTimestamp = Date_t::min();
+            bool detectedOutOfOrder = false;
+            for (const auto& metric : col) {
+                if (!metric.eoo()) {
+                    if (metric.type() != BSONType::Date) {
                         return Status(
                             ErrorCodes::BadValue,
-                            fmt::format("Time-series bucket '{}' field is not in ascending order",
-                                        fieldName));
-                    } else if (version == 3) {
-                        detectedOutOfOrder = true;
+                            fmt::format("Time-series bucket '{}' field is not a Date", fieldName));
                     }
+                    // Checks the time values are sorted in increasing order for v2 buckets
+                    // (compressed, sorted). Skip the check if the bucket is v3 (compressed,
+                    // unsorted).
+                    Date_t curTimestamp = metric.Date();
+                    if (curTimestamp < prevTimestamp) {
+                        if (version == timeseries::kTimeseriesControlCompressedSortedVersion) {
+                            return Status(
+                                ErrorCodes::BadValue,
+                                fmt::format(
+                                    "Time-series bucket '{}' field is not in ascending order",
+                                    fieldName));
+                        } else if (version ==
+                                   timeseries::kTimeseriesControlCompressedUnsortedVersion) {
+                            detectedOutOfOrder = true;
+                        }
+                    }
+                    prevTimestamp = curTimestamp;
+                    minmax.update(metric.wrap(fieldName), boost::none, coll->getDefaultCollator());
+                    ++(*bucketCount);
+                } else {
+                    return Status(ErrorCodes::BadValue,
+                                  "Time-series bucket has missing time fields");
                 }
-                prevTimestamp = curTimestamp;
-                minmax.update(metric.wrap(fieldName), boost::none, coll->getDefaultCollator());
-                ++(*bucketCount);
-            } else {
-                return Status(ErrorCodes::BadValue, "Time-series bucket has missing time fields");
             }
-        }
-        if (version == 3 && !detectedOutOfOrder) {
-            return Status(ErrorCodes::BadValue,
-                          "Time-series bucket is v3 but has its measurements in-order on time");
+            if (version == timeseries::kTimeseriesControlCompressedUnsortedVersion &&
+                !detectedOutOfOrder) {
+                return Status(ErrorCodes::BadValue,
+                              "Time-series bucket is v3 but has its measurements in-order on time");
+            }
+        } catch (DBException& e) {
+            return Status(ErrorCodes::InvalidBSON,
+                          str::stream() << "Exception occurred while decompressing a BSON column: "
+                                        << e.toString());
         }
     }
+    if (Status status = _validateTimeSeriesMinMax(
+            coll, minmax, controlMin, controlMax, fieldName, version, shouldDecompressBSON);
+        !status.isOK()) {
+        return status;
+    }
 
-    if (Status status = _validateTimeSeriesMinMax(coll, minmax, controlMin, controlMax, fieldName);
+    if (Status status = _validateTimeseriesBucketingParametersChanged(coll,
+                                                                      minmax,
+                                                                      controlMin,
+                                                                      controlMax,
+                                                                      fieldName,
+                                                                      state,
+                                                                      results,
+                                                                      version,
+                                                                      shouldDecompressBSON);
         !status.isOK()) {
         return status;
     }
@@ -388,10 +546,14 @@ Status _validateTimeSeriesDataField(const CollectionPtr& coll,
                                     const BSONElement& controlMin,
                                     const BSONElement& controlMax,
                                     StringData fieldName,
+                                    CollectionValidation::ValidateState* state,
+                                    ValidateResults* results,
                                     int version,
-                                    int bucketCount) {
-    timeseries::bucket_catalog::MinMax minmax;
-    if (version == 1) {
+                                    int bucketCount,
+                                    bool shouldDecompressBSON) {
+    TrackingContext trackingContext;
+    timeseries::bucket_catalog::MinMax minmax{trackingContext};
+    if (version == timeseries::kTimeseriesControlUncompressedVersion) {
         // Checks that indices are in increasing order and within the correct range.
         int prevIdx = INT_MIN;
         for (const auto& metric : dataField.Obj()) {
@@ -420,16 +582,25 @@ Status _validateTimeSeriesDataField(const CollectionPtr& coll,
             minmax.update(metric.wrap(fieldName), boost::none, coll->getDefaultCollator());
             prevIdx = idx;
         }
-    } else {
-        BSONColumn col{dataField};
-        for (const auto& metric : col) {
-            if (!metric.eoo()) {
-                minmax.update(metric.wrap(fieldName), boost::none, coll->getDefaultCollator());
+    } else if (shouldDecompressBSON) {
+        // Only decompress the bucket if we are in full validation mode, kBackgroundCheckBSON mode,
+        // or kForegroundCheckBSON mode since this is a relatively expensive operation.
+        try {
+            BSONColumn col{dataField};
+            for (const auto& metric : col) {
+                if (!metric.eoo()) {
+                    minmax.update(metric.wrap(fieldName), boost::none, coll->getDefaultCollator());
+                }
             }
+        } catch (DBException& e) {
+            return Status(ErrorCodes::InvalidBSON,
+                          str::stream() << "Exception occurred while decompressing a BSON column: "
+                                        << e.toString());
         }
     }
 
-    if (Status status = _validateTimeSeriesMinMax(coll, minmax, controlMin, controlMax, fieldName);
+    if (Status status = _validateTimeSeriesMinMax(
+            coll, minmax, controlMin, controlMax, fieldName, version, shouldDecompressBSON);
         !status.isOK()) {
         return status;
     }
@@ -439,7 +610,10 @@ Status _validateTimeSeriesDataField(const CollectionPtr& coll,
 
 Status _validateTimeSeriesDataFields(const CollectionPtr& coll,
                                      const BSONObj& recordBson,
-                                     int bucketVersion) {
+                                     CollectionValidation::ValidateState* state,
+                                     ValidateResults* results,
+                                     int bucketVersion,
+                                     bool shouldDecompressBSON) {
     BSONObj data = recordBson.getField(timeseries::kBucketDataFieldName).Obj();
     BSONObj control = recordBson.getField(timeseries::kBucketControlFieldName).Obj();
     BSONObj controlMin = control.getField(timeseries::kBucketControlMinFieldName).Obj();
@@ -486,13 +660,17 @@ Status _validateTimeSeriesDataFields(const CollectionPtr& coll,
                                                          controlMinFields[timeFieldName],
                                                          controlMaxFields[timeFieldName],
                                                          timeFieldName,
+                                                         state,
+                                                         results,
                                                          bucketVersion,
-                                                         &bucketCount);
+                                                         &bucketCount,
+                                                         shouldDecompressBSON);
         !status.isOK()) {
         return status;
     }
 
-    if (Status status = _validateTimeseriesCount(control, bucketCount, bucketVersion);
+    if (Status status =
+            _validateTimeseriesCount(control, bucketCount, bucketVersion, shouldDecompressBSON);
         !status.isOK()) {
         return status;
     }
@@ -511,8 +689,11 @@ Status _validateTimeSeriesDataFields(const CollectionPtr& coll,
                                                              controlMinFields[fieldName],
                                                              controlMaxFields[fieldName],
                                                              fieldName,
+                                                             state,
+                                                             results,
                                                              bucketVersion,
-                                                             bucketCount);
+                                                             bucketCount,
+                                                             shouldDecompressBSON);
                 !status.isOK()) {
                 return status;
             }
@@ -528,7 +709,9 @@ Status _validateTimeSeriesDataFields(const CollectionPtr& coll,
 Status _validateTimeSeriesBucketRecord(OperationContext* opCtx,
                                        const CollectionPtr& collection,
                                        const BSONObj& recordBson,
-                                       ValidateResults* results) {
+                                       CollectionValidation::ValidateState* state,
+                                       ValidateResults* results,
+                                       bool shouldDecompressBSON) {
     int bucketVersion = recordBson.getField(timeseries::kBucketControlFieldName)
                             .Obj()
                             .getIntField(timeseries::kBucketControlVersionFieldName);
@@ -543,7 +726,8 @@ Status _validateTimeSeriesBucketRecord(OperationContext* opCtx,
         return status;
     }
 
-    if (Status status = _validateTimeSeriesDataFields(collection, recordBson, bucketVersion);
+    if (Status status = _validateTimeSeriesDataFields(
+            collection, recordBson, state, results, bucketVersion, shouldDecompressBSON);
         !status.isOK()) {
         return status;
     }
@@ -560,8 +744,14 @@ void _timeseriesValidationFailed(CollectionValidation::ValidateState* state,
     }
     state->setTimeseriesDataInconsistent();
 
-    results->errors.push_back(kTimeseriesValidationInconsistencyReason);
-    results->valid = false;
+    if (TestingProctor::instance().isEnabled()) {
+        // In testing this is a fatal error. Some time-series checks are vital to test correctness,
+        // such as the time field being out-of-order for v: 2 buckets.
+        results->errors.push_back(kTimeseriesValidationInconsistencyReason);
+        results->valid = false;
+    } else {
+        results->warnings.push_back(kTimeseriesValidationInconsistencyReason);
+    }
 }
 
 void _BSONSpecValidationFailed(CollectionValidation::ValidateState* state,
@@ -583,14 +773,8 @@ Status ValidateAdaptor::validateRecord(OperationContext* opCtx,
                                        size_t* dataSize,
                                        ValidateResults* results,
                                        ValidationVersion validationVersion) {
-    auto validateBSONMode = BSONValidateMode::kDefault;
-    const auto fcvSnapshot = serverGlobalParams.featureCompatibility.acquireFCVSnapshot();
-    if (fcvSnapshot.isVersionInitialized() &&
-        feature_flags::gExtendValidateCommand.isEnabled(fcvSnapshot)) {
-        validateBSONMode = _validateState->getBSONValidateMode();
-    }
-    const Status status =
-        validateBSON(record.data(), record.size(), validateBSONMode, validationVersion);
+    Status status = validateBSON(
+        record.data(), record.size(), _validateState->getBSONValidateMode(), validationVersion);
     if (!status.isOK()) {
         if (status.code() != ErrorCodes::NonConformantBSON) {
             return status;
@@ -620,7 +804,7 @@ Status ValidateAdaptor::validateRecord(OperationContext* opCtx,
                                              results);
     }
 
-    SharedBufferFragmentBuilder pool(KeyString::HeapBuilder::kHeapAllocatorDefaultBytes);
+    SharedBufferFragmentBuilder pool(key_string::HeapBuilder::kHeapAllocatorDefaultBytes);
 
     for (const auto& indexIdent : _validateState->getIndexIdents()) {
         const IndexDescriptor* descriptor =
@@ -744,12 +928,11 @@ void ValidateAdaptor::traverseRecordStore(OperationContext* opCtx,
             }
 
             if (_validateState->fixErrors()) {
-                writeConflictRetry(
-                    opCtx, "corrupt record removal", _validateState->nss().ns(), [&] {
-                        WriteUnitOfWork wunit(opCtx);
-                        rs->deleteRecord(opCtx, record->id);
-                        wunit.commit();
-                    });
+                writeConflictRetry(opCtx, "corrupt record removal", _validateState->nss(), [&] {
+                    WriteUnitOfWork wunit(opCtx);
+                    rs->deleteRecord(opCtx, record->id);
+                    wunit.commit();
+                });
                 results->repaired = true;
                 results->numRemovedCorruptRecords++;
                 _numRecords--;
@@ -781,7 +964,7 @@ void ValidateAdaptor::traverseRecordStore(OperationContext* opCtx,
             // schema validator. Don't treat invalid documents as errors since documents can bypass
             // document validation when being inserted or updated.
             auto result = coll->checkValidation(opCtx, record->data.toBson());
-            const auto fcvSnapshot = serverGlobalParams.featureCompatibility.acquireFCVSnapshot();
+
             if (result.first != Collection::SchemaValidationResult::kPass) {
                 LOGV2_WARNING(5363500,
                               "Document is not compliant with the collection's schema",
@@ -791,12 +974,18 @@ void ValidateAdaptor::traverseRecordStore(OperationContext* opCtx,
 
                 nNonCompliantDocuments++;
                 schemaValidationFailed(_validateState, result.first, results);
-            } else if (fcvSnapshot.isVersionInitialized() &&
-                       feature_flags::gExtendValidateCommand.isEnabled(fcvSnapshot) &&
-                       coll->getTimeseriesOptions()) {
+            } else if (coll->getTimeseriesOptions()) {
+                BSONObj recordBson = record->data.toBson();
+                _enforceTimeseriesBucketsAreAlwaysCompressed(recordBson, results);
+
                 // Checks for time-series collection consistency.
                 Status bucketStatus =
-                    _validateTimeSeriesBucketRecord(opCtx, coll, record->data.toBson(), results);
+                    _validateTimeSeriesBucketRecord(opCtx,
+                                                    coll,
+                                                    recordBson,
+                                                    _validateState,
+                                                    results,
+                                                    _validateState->shouldDecompressBSONColumn());
                 // This log id should be kept in sync with the associated warning messages that are
                 // returned to the client.
                 if (!bucketStatus.isOK()) {
@@ -809,7 +998,7 @@ void ValidateAdaptor::traverseRecordStore(OperationContext* opCtx,
                     _timeseriesValidationFailed(_validateState, results);
                 } else {
                     auto containsMixedSchemaDataResponse =
-                        coll->doesTimeseriesBucketsDocContainMixedSchemaData(record->data.toBson());
+                        coll->doesTimeseriesBucketsDocContainMixedSchemaData(recordBson);
                     if (!containsMixedSchemaDataResponse.isOK() && !bucketMinMaxMalformedError) {
                         bucketMinMaxMalformedError = true;
                         LOGV2_WARNING(8469900,
@@ -823,16 +1012,8 @@ void ValidateAdaptor::traverseRecordStore(OperationContext* opCtx,
                         results->valid = false;
                     } else if (containsMixedSchemaDataResponse.isOK() &&
                                containsMixedSchemaDataResponse.getValue()) {
-                        // Only allow mixed-schema data if the new durable mixed-schema flag
-                        // introduced by SERVER-91195 (in `options.storageEngine`) is set.
-                        // Checking against `coll->getTimeseriesBucketsMayHaveMixedSchemaData()` is
-                        // unsafe, since only the legacy mixed-schema flag may be set, which can be
-                        // later lost on collection cloning due to SERVER-91194.
-                        bool mixedSchemaAllowed = getFlagFromStorageEngineBson(
-                                                      coll->getCollectionOptions().storageEngine,
-                                                      backwards_compatible_collection_options::
-                                                          kTimeseriesBucketsMayHaveMixedSchemaData)
-                                                      .value_or(false);
+                        bool mixedSchemaAllowed =
+                            coll->getTimeseriesBucketsMayHaveMixedSchemaData().get();
                         if (mixedSchemaAllowed && !bucketMixedSchemaDataWarning) {
                             bucketMixedSchemaDataWarning = true;
                             LOGV2_WARNING(8469901,
@@ -868,7 +1049,7 @@ void ValidateAdaptor::traverseRecordStore(OperationContext* opCtx,
             interruptIntervalNumBytes >= kInterruptIntervalNumBytes) {
             // Periodically checks for interrupts and yields.
             opCtx->checkForInterrupt();
-            _validateState->yield(opCtx);
+            _validateState->yieldCursors(opCtx);
 
             if (interruptIntervalNumBytes >= kInterruptIntervalNumBytes) {
                 interruptIntervalNumBytes = 0;
@@ -883,9 +1064,10 @@ void ValidateAdaptor::traverseRecordStore(OperationContext* opCtx,
 
     const auto fastCount = coll->numRecords(opCtx);
     if (_validateState->shouldEnforceFastCount() && fastCount != _numRecords) {
-        results->errors.push_back(
-            str::stream() << "fast count (" << fastCount << ") does not match number of records ("
-                          << _numRecords << ") for collection '" << coll->ns() << "'");
+        results->errors.push_back(str::stream() << "fast count (" << fastCount
+                                                << ") does not match number of records ("
+                                                << _numRecords << ") for collection '"
+                                                << coll->ns().toStringForErrorMsg() << "'");
         results->valid = false;
     }
 
@@ -982,4 +1164,26 @@ void ValidateAdaptor::addIndexEntryErrors(OperationContext* opCtx, ValidateResul
     _keyBasedIndexConsistency.addIndexEntryErrors(opCtx, results);
 }
 
+void ValidateAdaptor::_enforceTimeseriesBucketsAreAlwaysCompressed(const BSONObj& recordBson,
+                                                                   ValidateResults* results) {
+    if (!_validateState->enforceTimeseriesBucketsAreAlwaysCompressed()) {
+        return;
+    }
+
+    int bucketVersion = recordBson.getField(timeseries::kBucketControlFieldName)
+                            .Obj()
+                            .getIntField(timeseries::kBucketControlVersionFieldName);
+
+    if (bucketVersion != timeseries::kTimeseriesControlCompressedSortedVersion &&
+        bucketVersion != timeseries::kTimeseriesControlCompressedUnsortedVersion) {
+        LOGV2(7735100,
+              "Expected time-series bucket to be compressed",
+              "bucket"_attr = recordBson.toString());
+        results->errors.push_back(
+            "Expected time-series bucket to be compressed. Search logs for message "
+            "with id 7735100.");
+        results->valid = false;
+    }
+}
+
 }  // namespace mongo
