@@ -28,13 +28,33 @@
  */
 
 
-#include "mongo/platform/basic.h"
+#include <cstdint>
+#include <mutex>
+#include <string>
+#include <utility>
+#include <variant>
 
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/db/change_stream_options_manager.h"
 #include "mongo/db/change_stream_options_parameter_gen.h"
 #include "mongo/db/change_stream_serverless_helpers.h"
+#include "mongo/db/client.h"
+#include "mongo/db/cluster_role.h"
 #include "mongo/db/repl/replication_coordinator.h"
-#include "mongo/logv2/log.h"
+#include "mongo/db/server_options.h"
+#include "mongo/db/tenant_id.h"
+#include "mongo/idl/idl_parser.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/str.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
 
@@ -118,32 +138,31 @@ Status ChangeStreamOptionsParameter::validate(const BSONElement& newValueElement
 
         ChangeStreamOptions newOptions = ChangeStreamOptions::parse(ctxt, changeStreamOptionsObj);
         auto preAndPostImages = newOptions.getPreAndPostImages();
-        stdx::visit(
-            OverloadedVisitor{
-                [&](const std::string& expireAfterSeconds) {
-                    if (expireAfterSeconds != "off"_sd) {
-                        validateStatus = {
-                            ErrorCodes::BadValue,
-                            "Non-numeric value of 'expireAfterSeconds' should be 'off'"};
-                    }
-                },
-                [&](const std::int64_t& expireAfterSeconds) {
-                    if (change_stream_serverless_helpers::isServerlessEnvironment()) {
-                        validateStatus = {
-                            ErrorCodes::CommandNotSupported,
-                            "The 'changeStreamOptions.preAndPostImages.expireAfterSeconds' is "
-                            "unsupported in serverless, consider using "
-                            "'changeStreams.expireAfterSeconds' instead."};
-                        return;
-                    }
-                    if (expireAfterSeconds <= 0) {
-                        validateStatus = {
-                            ErrorCodes::BadValue,
-                            "Numeric value of 'expireAfterSeconds' should be positive"};
-                    }
-                },
-            },
-            preAndPostImages.getExpireAfterSeconds());
+        visit(OverloadedVisitor{
+                  [&](const std::string& expireAfterSeconds) {
+                      if (expireAfterSeconds != "off"_sd) {
+                          validateStatus = {
+                              ErrorCodes::BadValue,
+                              "Non-numeric value of 'expireAfterSeconds' should be 'off'"};
+                      }
+                  },
+                  [&](const std::int64_t& expireAfterSeconds) {
+                      if (change_stream_serverless_helpers::isServerlessEnvironment()) {
+                          validateStatus = {
+                              ErrorCodes::CommandNotSupported,
+                              "The 'changeStreamOptions.preAndPostImages.expireAfterSeconds' is "
+                              "unsupported in serverless, consider using "
+                              "'changeStreams.expireAfterSeconds' instead."};
+                          return;
+                      }
+                      if (expireAfterSeconds <= 0) {
+                          validateStatus = {
+                              ErrorCodes::BadValue,
+                              "Numeric value of 'expireAfterSeconds' should be positive"};
+                      }
+                  },
+              },
+              preAndPostImages.getExpireAfterSeconds());
 
         return validateStatus;
     } catch (const AssertionException& ex) {
