@@ -27,20 +27,32 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
+#include <string>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/db/auth/action_type.h"
 #include "mongo/db/auth/authorization_session.h"
+#include "mongo/db/auth/resource_pattern.h"
 #include "mongo/db/catalog/capped_utils.h"
-#include "mongo/db/client.h"
+#include "mongo/db/catalog/database.h"
+#include "mongo/db/catalog_raii.h"
 #include "mongo/db/commands.h"
-#include "mongo/db/db_raii.h"
+#include "mongo/db/concurrency/d_concurrency.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
+#include "mongo/db/database_name.h"
 #include "mongo/db/namespace_string.h"
-#include "mongo/db/op_observer/op_observer.h"
-#include "mongo/db/query/find.h"
-#include "mongo/db/query/internal_plans.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/service_context.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/namespace_string_util.h"
+#include "mongo/util/str.h"
 
 namespace mongo {
 namespace {
@@ -74,9 +86,9 @@ public:
                 "'toCollection' must be of type String",
                 nssElt.type() == BSONType::String);
         const NamespaceString nss(
-            NamespaceStringUtil::parseNamespaceFromRequest(dbName, nssElt.valueStringData()));
+            NamespaceStringUtil::deserialize(dbName, nssElt.valueStringData()));
         uassert(ErrorCodes::InvalidNamespace,
-                str::stream() << "Invalid target namespace: " << nss.ns(),
+                str::stream() << "Invalid target namespace: " << nss.toStringForErrorMsg(),
                 nss.isValid());
 
         if (!as->isAuthorizedForActionsOnResource(
@@ -115,10 +127,12 @@ public:
         auto size = cmdObj.getField("size").safeNumberLong();
         bool temp = cmdObj.getField("temp").trueValue();
 
-        uassert(ErrorCodes::InvalidOptions, "invalid command spec", size != 0);
+        uassert(ErrorCodes::InvalidOptions,
+                "Capped collection size must be greater than zero",
+                size > 0);
 
-        NamespaceString fromNs(NamespaceStringUtil::parseNamespaceFromRequest(dbName, from));
-        NamespaceString toNs(NamespaceStringUtil::parseNamespaceFromRequest(dbName, to));
+        NamespaceString fromNs(NamespaceStringUtil::deserialize(dbName, from));
+        NamespaceString toNs(NamespaceStringUtil::deserialize(dbName, to));
 
         AutoGetCollection autoColl(opCtx, fromNs, MODE_X);
         Lock::CollectionLock collLock(opCtx, toNs, MODE_X);
@@ -138,8 +152,8 @@ public:
         cloneCollectionAsCapped(opCtx, db, fromNs, toNs, size, temp);
         return true;
     }
-
-} cmdCloneCollectionAsCapped;
+};
+MONGO_REGISTER_COMMAND(CmdCloneCollectionAsCapped).forShard();
 
 /**
  * Converts the given collection to a capped collection w/ the specified size. This command is not
@@ -151,7 +165,7 @@ public:
     AllowedOnSecondary secondaryAllowed(ServiceContext*) const override {
         return AllowedOnSecondary::kNever;
     }
-    virtual bool supportsWriteConcern(const BSONObj& cmd) const override {
+    bool supportsWriteConcern(const BSONObj& cmd) const override {
         return true;
     }
 
@@ -182,13 +196,15 @@ public:
         const NamespaceString nss(CommandHelpers::parseNsCollectionRequired(dbName, cmdObj));
         auto size = cmdObj.getField("size").safeNumberLong();
 
-        uassert(ErrorCodes::InvalidOptions, "invalid command spec", size != 0);
+        uassert(ErrorCodes::InvalidOptions,
+                "Capped collection size must be greater than zero",
+                size > 0);
 
         convertToCapped(opCtx, nss, size);
         return true;
     }
-
-} cmdConvertToCapped;
+};
+MONGO_REGISTER_COMMAND(CmdConvertToCapped).forShard();
 
 }  // namespace
 }  // namespace mongo
