@@ -28,31 +28,59 @@
  */
 
 
-#include "mongo/platform/basic.h"
-
-#include <boost/optional.hpp>
 #include <map>
+#include <memory>
 #include <set>
 #include <string>
+#include <utility>
 
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/db/auth/action_type.h"
+#include "mongo/db/auth/authorization_session.h"
+#include "mongo/db/auth/resource_pattern.h"
 #include "mongo/db/catalog/collection.h"
-#include "mongo/db/catalog/collection_catalog_helper.h"
-#include "mongo/db/catalog/database.h"
+#include "mongo/db/catalog/collection_catalog.h"
 #include "mongo/db/catalog/index_catalog.h"
+#include "mongo/db/catalog_raii.h"
 #include "mongo/db/commands.h"
 #include "mongo/db/commands/test_commands_enabled.h"
-#include "mongo/db/db_raii.h"
-#include "mongo/db/exec/working_set_common.h"
+#include "mongo/db/concurrency/d_concurrency.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
+#include "mongo/db/database_name.h"
 #include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/query/index_bounds.h"
 #include "mongo/db/query/internal_plans.h"
+#include "mongo/db/query/plan_executor.h"
+#include "mongo/db/query/plan_yield_policy.h"
+#include "mongo/db/read_concern_support_result.h"
+#include "mongo/db/repl/optime.h"
+#include "mongo/db/repl/read_concern_level.h"
 #include "mongo/db/repl/replication_coordinator.h"
+#include "mongo/db/s/operation_sharding_state.h"
+#include "mongo/db/service_context.h"
+#include "mongo/db/storage/recovery_unit.h"
 #include "mongo/db/storage/storage_engine.h"
-#include "mongo/db/transaction/transaction_participant.h"
+#include "mongo/db/transaction_resources.h"
 #include "mongo/logv2/log.h"
-#include "mongo/platform/mutex.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/md5.h"
 #include "mongo/util/md5.hpp"
 #include "mongo/util/net/socket_utils.h"
+#include "mongo/util/str.h"
 #include "mongo/util/timer.h"
+#include "mongo/util/uuid.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
 
@@ -62,21 +90,42 @@ namespace mongo {
 namespace {
 
 constexpr char SKIP_TEMP_COLLECTION[] = "skipTempCollections";
-// TODO SERVER-106005: Remove this option once all versions tested in multiversion suites can scan
-// in natural order for capped collections.
-constexpr char USE_INDEX_SCAN_FOR_CAPPED_COLLECTIONS[] = "useIndexScanForCappedCollections";
+constexpr char EXCLUDE_RECORDIDS[] = "excludeRecordIds";
 
 std::shared_ptr<const CollectionCatalog> getConsistentCatalogAndSnapshot(OperationContext* opCtx) {
     // Loop until we get a consistent catalog and snapshot. This is only used for the lock-free
     // implementation of dbHash which skips acquiring database and collection locks.
     while (true) {
-        const auto catalogBeforeSnapshot = CollectionCatalog::get(opCtx);
-        opCtx->recoveryUnit()->preallocateSnapshot();
+        auto catalogBeforeSnapshot = CollectionCatalog::get(opCtx);
+        shard_role_details::getRecoveryUnit(opCtx)->preallocateSnapshot();
         const auto catalogAfterSnapshot = CollectionCatalog::get(opCtx);
         if (catalogBeforeSnapshot == catalogAfterSnapshot) {
             return catalogBeforeSnapshot;
         }
-        opCtx->recoveryUnit()->abandonSnapshot();
+        shard_role_details::getRecoveryUnit(opCtx)->abandonSnapshot();
+    }
+}
+
+// Includes Records and their RecordIds explicitly in the hash.
+void hashRecordsAndRecordIds(const CollectionPtr& collection, PlanExecutor* exec, md5_state_t* st) {
+    // Clustered collections implicitly replicate RecordIds. Clustered collections are also the only
+    // collections which have RecordIds that aren't of type "Long". This method assumes RecordIds
+    // are of type "Long" to optimize their translation into the hash.
+    invariant(!collection->isClustered());
+
+    BSONObj c;
+    RecordId rid;
+    while (exec->getNext(&c, &rid) == PlanExecutor::ADVANCED) {
+        md5_append(st, (const md5_byte_t*)c.objdata(), c.objsize());
+        const auto ridInt = rid.getLong();
+        md5_append(st, (const md5_byte_t*)&ridInt, sizeof(ridInt));
+    }
+}
+
+void hashRecordsOnly(PlanExecutor* exec, md5_state_t* st) {
+    BSONObj c;
+    while (exec->getNext(&c, nullptr) == PlanExecutor::ADVANCED) {
+        md5_append(st, (const md5_byte_t*)c.objdata(), c.objsize());
     }
 }
 
@@ -84,7 +133,7 @@ class DBHashCmd : public BasicCommand {
 public:
     DBHashCmd() : BasicCommand("dbHash", "dbhash") {}
 
-    virtual bool supportsWriteConcern(const BSONObj& cmd) const override {
+    bool supportsWriteConcern(const BSONObj& cmd) const override {
         return false;
     }
 
@@ -129,7 +178,7 @@ public:
                                  const DatabaseName& dbName,
                                  const BSONObj& cmdObj) const override {
         auto* as = AuthorizationSession::get(opCtx->getClient());
-        if (!as->isAuthorizedForActionsOnResource(ResourcePattern::forDatabaseName(dbName.db()),
+        if (!as->isAuthorizedForActionsOnResource(ResourcePattern::forDatabaseName(dbName),
                                                   ActionType::dbHash)) {
             return {ErrorCodes::Unauthorized, "unauthorized"};
         }
@@ -143,6 +192,11 @@ public:
              BSONObjBuilder& result) override {
         Timer timer;
 
+        // We want the dbCheck command to be allowed via direct shard connections despite the fact
+        // that it acquires a collection lock on a user collection. This is an exception, and should
+        // not be repeated elsewhere without sufficient need.
+        OperationShardingState::get(opCtx).setShouldSkipDirectShardConnectionChecks();
+
         std::set<std::string> desiredCollections;
         if (cmdObj["collections"].type() == Array) {
             BSONObjIterator i(cmdObj["collections"].Obj());
@@ -161,13 +215,10 @@ public:
             LOGV2(6859700, "Skipping hash computation for temporary collections");
         }
 
-        const bool useIndexScanForCappedCollections =
-            cmdObj.hasField(USE_INDEX_SCAN_FOR_CAPPED_COLLECTIONS) &&
-            cmdObj[USE_INDEX_SCAN_FOR_CAPPED_COLLECTIONS].trueValue();
-        if (useIndexScanForCappedCollections) {
-            LOGV2(8218000,
-                  "Performing index scan on the _id index instead of using natural order for "
-                  "capped collections");
+        const bool excludeRecordIds =
+            cmdObj.hasField(EXCLUDE_RECORDIDS) && cmdObj[EXCLUDE_RECORDIDS].trueValue();
+        if (excludeRecordIds) {
+            LOGV2(6859701, "Exclude recordIds in dbHash for recordIdsReplicated collections");
         }
 
         uassert(ErrorCodes::InvalidNamespace,
@@ -185,7 +236,7 @@ public:
             uassert(ErrorCodes::InvalidOptions,
                     "The '$_internalReadAtClusterTime' option is only supported when replication is"
                     " enabled",
-                    replCoord->isReplEnabled());
+                    replCoord->getSettings().isReplSet());
 
             uassert(ErrorCodes::TypeMismatch,
                     "The '$_internalReadAtClusterTime' option must be a Timestamp",
@@ -231,37 +282,25 @@ public:
             // The $_internalReadAtClusterTime option causes any storage-layer cursors created
             // during plan execution to read from a consistent snapshot of data at the supplied
             // clusterTime, even across yields.
-            opCtx->recoveryUnit()->setTimestampReadSource(RecoveryUnit::ReadSource::kProvided,
-                                                          targetClusterTime);
+            shard_role_details::getRecoveryUnit(opCtx)->setTimestampReadSource(
+                RecoveryUnit::ReadSource::kProvided, targetClusterTime);
 
             // The $_internalReadAtClusterTime option also causes any storage-layer cursors created
             // during plan execution to block on prepared transactions. Since the dbhash command
             // ignores prepare conflicts by default, change the behavior.
-            opCtx->recoveryUnit()->setPrepareConflictBehavior(PrepareConflictBehavior::kEnforce);
+            shard_role_details::getRecoveryUnit(opCtx)->setPrepareConflictBehavior(
+                PrepareConflictBehavior::kEnforce);
         }
 
         const bool isPointInTimeRead =
-            opCtx->recoveryUnit()->getTimestampReadSource() == RecoveryUnit::ReadSource::kProvided;
-
-        boost::optional<ShouldNotConflictWithSecondaryBatchApplicationBlock> shouldNotConflictBlock;
-        if (isPointInTimeRead) {
-            // If we are performing a read at a timestamp, then we allow oplog application to
-            // proceed concurrently with the dbHash command. This is done to ensure a prepare
-            // conflict is able to eventually be resolved by processing a later commitTransaction or
-            // abortTransaction oplog entry.
-            shouldNotConflictBlock.emplace(opCtx->lockState());
-        }
+            shard_role_details::getRecoveryUnit(opCtx)->getTimestampReadSource() ==
+            RecoveryUnit::ReadSource::kProvided;
 
         // We take the global lock here as dbHash runs lock-free with point-in-time catalog lookups.
         Lock::GlobalLock globalLock(opCtx, MODE_IS);
 
         // The CollectionCatalog to use for lock-free reads with point-in-time catalog lookups.
-        std::shared_ptr<const CollectionCatalog> catalog;
-        // (Ignore FCV check): This feature flag doesn't have any upgrade/downgrade concerns.
-        if (feature_flags::gPointInTimeCatalogLookups.isEnabledAndIgnoreFCVUnsafe()) {
-            // Make sure we get a CollectionCatalog in sync with our snapshot.
-            catalog = getConsistentCatalogAndSnapshot(opCtx);
-        }
+        std::shared_ptr<const CollectionCatalog> catalog = getConsistentCatalogAndSnapshot(opCtx);
 
         boost::optional<AutoGetDb> autoDb;
         if (isPointInTimeRead) {
@@ -275,7 +314,7 @@ public:
             autoDb.emplace(opCtx, dbName, MODE_S);
         }
 
-        result.append("host", prettyHostName());
+        result.append("host", prettyHostName(opCtx->getClient()->getLocalPort()));
 
         md5_state_t globalState;
         md5_init(&globalState);
@@ -288,8 +327,9 @@ public:
             auto collNss = collection->ns();
 
             uassert(ErrorCodes::BadValue,
-                    str::stream() << "weird fullCollectionName [" << collNss.toString() << "]",
-                    collNss.size() - 1 > dbName.db().size());
+                    str::stream() << "weird fullCollectionName [" << collNss.toStringForErrorMsg()
+                                  << "]",
+                    collNss.size() - 1 > dbName.size());
 
             if (repl::ReplicationCoordinator::isOplogDisabledForNS(collNss)) {
                 return true;
@@ -320,65 +360,59 @@ public:
             collectionToUUIDMap.emplace(collNss.coll().toString(), collection->uuid());
 
             // Compute the hash for this collection.
-            std::string hash =
-                _hashCollection(opCtx, CollectionPtr(collection), useIndexScanForCappedCollections);
+            std::string hash = _hashCollection(opCtx, CollectionPtr(collection), excludeRecordIds);
 
             collectionToHashMap[collNss.coll().toString()] = hash;
 
             return true;
         };
 
-        // (Ignore FCV check): This feature flag doesn't have any upgrade/downgrade concerns.
-        if (feature_flags::gPointInTimeCatalogLookups.isEnabledAndIgnoreFCVUnsafe()) {
-            for (auto&& coll : catalog->range(dbName)) {
-                UUID uuid = coll->uuid();
+        for (auto&& coll : catalog->range(dbName)) {
+            UUID uuid = coll->uuid();
 
-                // The namespace must be found as the UUID is fetched from the same
-                // CollectionCatalog instance.
-                boost::optional<NamespaceString> nss = catalog->lookupNSSByUUID(opCtx, uuid);
-                invariant(nss);
+            // The namespace must be found as the UUID is fetched from the same
+            // CollectionCatalog instance.
+            boost::optional<NamespaceString> nss = catalog->lookupNSSByUUID(opCtx, uuid);
+            invariant(nss);
 
-                // TODO:SERVER-75848 Make this lock-free
-                Lock::CollectionLock clk(opCtx, *nss, MODE_IS);
+            // TODO:SERVER-75848 Make this lock-free
+            Lock::CollectionLock clk(opCtx, *nss, MODE_IS);
 
-                const Collection* collection = nullptr;
-                if (nss->isGlobalIndex()) {
-                    // TODO SERVER-74209: Reading earlier than the minimum valid snapshot is not
-                    // supported for global indexes. It appears that the primary and secondaries
-                    // apply operations differently resulting in hash mismatches. This requires
-                    // further investigation. In the meantime, global indexes use the behaviour
-                    // prior to point-in-time lookups.
-                    collection = coll;
+            const Collection* collection = nullptr;
+            if (nss->isGlobalIndex()) {
+                // TODO SERVER-74209: Reading earlier than the minimum valid snapshot is not
+                // supported for global indexes. It appears that the primary and secondaries apply
+                // operations differently resulting in hash mismatches. This requires further
+                // investigation. In the meantime, global indexes use the behaviour prior to
+                // point-in-time lookups.
+                collection = coll;
 
-                    if (auto readTimestamp =
-                            opCtx->recoveryUnit()->getPointInTimeReadTimestamp(opCtx)) {
-                        auto minSnapshot = coll->getMinimumValidSnapshot();
-                        uassert(ErrorCodes::SnapshotUnavailable,
-                                str::stream()
-                                    << "Unable to read from a snapshot due to pending collection"
-                                       " catalog changes; please retry the operation. Snapshot"
-                                       " timestamp is "
-                                    << readTimestamp->toString()
-                                    << ". Collection minimum timestamp is "
-                                    << minSnapshot->toString(),
-                                !minSnapshot || *readTimestamp >= *minSnapshot);
-                    }
-                } else {
-                    collection = catalog->establishConsistentCollection(
-                        opCtx,
-                        {dbName, uuid},
-                        opCtx->recoveryUnit()->getPointInTimeReadTimestamp(opCtx));
-
-                    if (!collection) {
-                        // The collection did not exist at the read timestamp with the given UUID.
-                        continue;
-                    }
+                if (auto readTimestamp =
+                        shard_role_details::getRecoveryUnit(opCtx)->getPointInTimeReadTimestamp(
+                            opCtx)) {
+                    auto minSnapshot = coll->getMinimumValidSnapshot();
+                    uassert(ErrorCodes::SnapshotUnavailable,
+                            str::stream()
+                                << "Unable to read from a snapshot due to pending collection"
+                                   " catalog changes; please retry the operation. Snapshot"
+                                   " timestamp is "
+                                << readTimestamp->toString() << ". Collection minimum timestamp is "
+                                << minSnapshot->toString(),
+                            !minSnapshot || *readTimestamp >= *minSnapshot);
                 }
+            } else {
+                collection = catalog->establishConsistentCollection(
+                    opCtx,
+                    {dbName, uuid},
+                    shard_role_details::getRecoveryUnit(opCtx)->getPointInTimeReadTimestamp(opCtx));
 
-                (void)checkAndHashCollection(collection);
+                if (!collection) {
+                    // The collection did not exist at the read timestamp with the given UUID.
+                    continue;
+                }
             }
-        } else {
-            catalog::forEachCollectionFromDb(opCtx, dbName, MODE_IS, checkAndHashCollection);
+
+            (void)checkAndHashCollection(collection);
         }
 
         BSONObjBuilder bb(result.subobjStart("collections"));
@@ -420,35 +454,20 @@ public:
 private:
     std::string _hashCollection(OperationContext* opCtx,
                                 const CollectionPtr& collection,
-                                bool useIndexScanForCappedCollections) {
-        boost::optional<Lock::CollectionLock> collLock;
-        if (opCtx->recoveryUnit()->getTimestampReadSource() ==
-            RecoveryUnit::ReadSource::kProvided) {
-            // When performing a read at a timestamp, we are only holding the database lock in
-            // intent mode. We need to also acquire the collection lock in intent mode to ensure
-            // reading from the consistent snapshot doesn't overlap with any catalog operations on
-            // the collection.
-            invariant(opCtx->lockState()->isCollectionLockedForMode(collection->ns(), MODE_IS));
-
-            auto minSnapshot = collection->getMinimumVisibleSnapshot();
-            auto mySnapshot = opCtx->recoveryUnit()->getPointInTimeReadTimestamp(opCtx);
-            invariant(mySnapshot);
-
-            uassert(ErrorCodes::SnapshotUnavailable,
-                    str::stream() << "Unable to read from a snapshot due to pending collection"
-                                     " catalog changes; please retry the operation. Snapshot"
-                                     " timestamp is "
-                                  << mySnapshot->toString() << ". Collection minimum timestamp is "
-                                  << minSnapshot->toString(),
-                    !minSnapshot || *mySnapshot >= *minSnapshot);
-        } else {
-            invariant(opCtx->lockState()->isDbLockedForMode(collection->ns().dbName(), MODE_S));
-        }
-
+                                bool excludeRecordIds) {
         auto desc = collection->getIndexCatalog()->findIdIndex(opCtx);
 
         std::unique_ptr<PlanExecutor, PlanExecutor::Deleter> exec;
-        if (desc && !(collection->isCapped() && !useIndexScanForCappedCollections)) {
+
+        // Replicated RecordIds circumvent the need to perform an _id lookup because natural
+        // scan order is preserved across nodes. If to exclude recordIds, existing _id scan should
+        // be kept too. This way a customer will get the same hash with excludeRecordIds option as
+        // the one before upgrade.
+        bool includeRids = collection->areRecordIdsReplicated() && !excludeRecordIds;
+
+        // TODO SERVER-86692: This logic can be simplified once all capped, clustered, and
+        // replicated recordId collections always use a collection scan.
+        if (desc && !includeRids) {
             exec = InternalPlanner::indexScan(opCtx,
                                               &collection,
                                               desc,
@@ -458,8 +477,7 @@ private:
                                               PlanYieldPolicy::YieldPolicy::INTERRUPT_ONLY,
                                               InternalPlanner::FORWARD,
                                               InternalPlanner::IXSCAN_FETCH);
-        } else if ((collection->isCapped() && !useIndexScanForCappedCollections) ||
-                   collection->isClustered()) {
+        } else if (collection->isCapped() || collection->isClustered() || includeRids) {
             exec = InternalPlanner::collectionScan(
                 opCtx, &collection, PlanYieldPolicy::YieldPolicy::INTERRUPT_ONLY);
         } else {
@@ -471,10 +489,16 @@ private:
         md5_init(&st);
 
         try {
-            BSONObj c;
             MONGO_verify(nullptr != exec.get());
-            while (exec->getNext(&c, nullptr) == PlanExecutor::ADVANCED) {
-                md5_append(&st, (const md5_byte_t*)c.objdata(), c.objsize());
+
+            // It's unnecessary to explicitly include the RecordIds of a clustered collection in the
+            // hash. In a clustered collection, each RecordId is generated by the _id, which is
+            // already hashed as a part of the Record.
+            const bool explicitlyHashRecordIds = includeRids && !collection->isClustered();
+            if (explicitlyHashRecordIds) {
+                hashRecordsAndRecordIds(collection, exec.get(), &st);
+            } else {
+                hashRecordsOnly(exec.get(), &st);
             }
         } catch (DBException& exception) {
             LOGV2_WARNING(
@@ -489,8 +513,8 @@ private:
 
         return hash;
     }
-
-} dbhashCmd;
+};
+MONGO_REGISTER_COMMAND(DBHashCmd).forShard();
 
 }  // namespace
 }  // namespace mongo
