@@ -28,34 +28,64 @@
  */
 
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/commands/feature_compatibility_version.h"
-
+#include <absl/container/node_hash_map.h>
+#include <algorithm>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional.hpp>
 #include <fmt/format.h>
+#include <initializer_list>
+#include <memory>
+#include <string>
+#include <tuple>
+#include <utility>
+#include <vector>
 
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
 #include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/db/catalog/collection_catalog.h"
+#include "mongo/db/catalog/collection_options.h"
 #include "mongo/db/catalog_raii.h"
+#include "mongo/db/cluster_role.h"
+#include "mongo/db/commands/feature_compatibility_version.h"
 #include "mongo/db/commands/feature_compatibility_version_gen.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
+#include "mongo/db/database_name.h"
 #include "mongo/db/dbdirectclient.h"
+#include "mongo/db/feature_compatibility_version_document_gen.h"
 #include "mongo/db/feature_compatibility_version_documentation.h"
 #include "mongo/db/feature_compatibility_version_parser.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/repl/optime.h"
-#include "mongo/db/repl/repl_server_parameters_gen.h"
+#include "mongo/db/repl/replication_consistency_markers.h"
 #include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/repl/replication_process.h"
 #include "mongo/db/repl/storage_interface.h"
-#include "mongo/db/s/collection_sharding_state.h"
-#include "mongo/db/s/sharding_state.h"
 #include "mongo/db/service_context.h"
 #include "mongo/db/storage/storage_engine.h"
+#include "mongo/db/storage/storage_options.h"
+#include "mongo/db/tenant_id.h"
+#include "mongo/db/transaction_resources.h"
 #include "mongo/db/wire_version.h"
 #include "mongo/db/write_concern_options.h"
+#include "mongo/idl/idl_parser.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/log_tag.h"
 #include "mongo/rpc/get_status_from_command_result.h"
-#include "mongo/transport/service_entry_point.h"
+#include "mongo/stdx/unordered_map.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/concurrency/mutex.h"
+#include "mongo/util/str.h"
+#include "mongo/util/uuid.h"
 #include "mongo/util/version/releases.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
@@ -75,7 +105,6 @@ namespace {
  * Utility class for recording permitted transitions between feature compatibility versions and
  * their on-disk representation as FeatureCompatibilityVersionDocument objects.
  */
-// TODO SERVER-65269: Add back 'const' qualifier to FCVTransitions class declaration
 class FCVTransitions {
 public:
     FCVTransitions() {
@@ -132,6 +161,11 @@ public:
             _transitions[{GenericFCV::kDowngradingFromLatestToLastLTS,
                           GenericFCV::kLastLTS,
                           isFromConfigServer}] = GenericFCV::kLastLTS;
+
+            // Add transition from downgrading -> upgrading.
+            _transitions[{GenericFCV::kDowngradingFromLatestToLastLTS,
+                          GenericFCV::kLatest,
+                          isFromConfigServer}] = GenericFCV::kUpgradingFromLastLTSToLatest;
         }
         _fcvDocuments[GenericFCV::kDowngradingFromLatestToLastLTS] =
             makeFCVDoc(GenericFCV::kLastLTS /* effective */,
@@ -140,24 +174,6 @@ public:
             );
     }
 
-    /**
-     * If feature flag gDowngradingToUpgrading is enabled,
-     * we add the new downgrading->upgrading->latest path.
-     */
-    void featureFlaggedAddNewTransitionState() {
-        // (Ignore FCV check): This is intentional because we want to use this feature even if we
-        // are in downgrading fcv state.
-        if (repl::feature_flags::gDowngradingToUpgrading.isEnabledAndIgnoreFCVUnsafe()) {
-            for (auto&& isFromConfigServer : {false, true}) {
-                _transitions[{GenericFCV::kDowngradingFromLatestToLastLTS,
-                              GenericFCV::kLatest,
-                              isFromConfigServer}] = GenericFCV::kUpgradingFromLastLTSToLatest;
-            }
-        }
-    }
-
-    // TODO (SERVER-74847): Remove this transition once we remove testing around
-    // downgrading from latest to last continuous.
     void addTransitionFromLatestToLastContinuous() {
         for (auto&& isFromConfigServer : {false, true}) {
             _transitions[{GenericFCV::kLatest, GenericFCV::kLastContinuous, isFromConfigServer}] =
@@ -276,18 +292,15 @@ void runUpdateCommand(OperationContext* opCtx, const FeatureCompatibilityVersion
 
 }  // namespace
 
-boost::optional<BSONObj> FeatureCompatibilityVersion::findFeatureCompatibilityVersionDocument(
+StatusWith<BSONObj> FeatureCompatibilityVersion::findFeatureCompatibilityVersionDocument(
     OperationContext* opCtx) {
     AutoGetCollection autoColl(opCtx, NamespaceString::kServerConfigurationNamespace, MODE_IX);
-    invariant(autoColl.ensureDbExists(opCtx), NamespaceString::kServerConfigurationNamespace.ns());
+    invariant(autoColl.ensureDbExists(opCtx),
+              redactTenant(NamespaceString::kServerConfigurationNamespace));
 
     const auto query = BSON("_id" << multiversion::kParameterName);
-    const auto swFcv = repl::StorageInterface::get(opCtx)->findById(
+    return repl::StorageInterface::get(opCtx)->findById(
         opCtx, NamespaceString::kServerConfigurationNamespace, query["_id"]);
-    if (!swFcv.isOK()) {
-        return boost::none;
-    }
-    return swFcv.getValue();
 }
 
 void FeatureCompatibilityVersion::validateSetFeatureCompatibilityVersionRequest(
@@ -303,21 +316,26 @@ void FeatureCompatibilityVersion::validateSetFeatureCompatibilityVersionRequest(
         fcvTransitions.permitsTransition(fromVersion, newVersion, isFromConfigServer));
 
     auto fcvObj = findFeatureCompatibilityVersionDocument(opCtx);
-    auto fcvDoc = FeatureCompatibilityVersionDocument::parse(
-        IDLParserContext("featureCompatibilityVersionDocument"), fcvObj.value());
+    if (!fcvObj.isOK()) {
+        Status status = fcvObj.getStatus();
+        invariant(ErrorCodes::isRetriableError(status));
+        uasserted(
+            8531600,
+            str::stream() << "failed to validate setFCV request because the existing FCV document "
+                             "could not be found due to error: "
+                          << status.toString() << ". Retry the setFCV request.");
+    }
 
-    // (Ignore FCV check): This is intentional because we want to use this feature even if we are in
-    // downgrading fcv state.
-    if (repl::feature_flags::gDowngradingToUpgrading.isEnabledAndIgnoreFCVUnsafe()) {
-        auto isCleaningServerMetadata = fcvDoc.getIsCleaningServerMetadata();
-        uassert(
-            7428200,
+    auto fcvDoc = FeatureCompatibilityVersionDocument::parse(
+        IDLParserContext("featureCompatibilityVersionDocument"), fcvObj.getValue());
+
+    auto isCleaningServerMetadata = fcvDoc.getIsCleaningServerMetadata();
+    uassert(7428200,
             "Cannot upgrade featureCompatibilityVersion if a previous FCV downgrade stopped in the "
             "middle of cleaning up internal server metadata. Retry the FCV downgrade until it "
             "succeeds before attempting to upgrade the FCV.",
             !(newVersion > fromVersion &&
               (isCleaningServerMetadata.is_initialized() && *isCleaningServerMetadata)));
-    }
 
     auto setFCVPhase = setFCVRequest.getPhase();
     if (!isFromConfigServer || !setFCVPhase) {
@@ -374,14 +392,11 @@ void FeatureCompatibilityVersion::updateFeatureCompatibilityVersionDocument(
 
     // Only transition to fully upgraded or downgraded states when we have completed all required
     // upgrade/downgrade behavior, unless it is the newly added downgrading to upgrading path.
-    // (Ignore FCV check): This is intentional because we want to use this feature even if we are in
-    // downgrading fcv state.
     auto transitioningVersion = setTargetVersion &&
             serverGlobalParams.featureCompatibility.acquireFCVSnapshot().isUpgradingOrDowngrading(
                 fromVersion) &&
-            !(repl::feature_flags::gDowngradingToUpgrading.isEnabledAndIgnoreFCVUnsafe() &&
-              (fromVersion == GenericFCV::kDowngradingFromLatestToLastLTS &&
-               newVersion == GenericFCV::kLatest))
+            !(fromVersion == GenericFCV::kDowngradingFromLatestToLastLTS &&
+              newVersion == GenericFCV::kLatest)
         ? fromVersion
         : fcvTransitions.getTransitionalVersion(fromVersion, newVersion, isFromConfigServer);
 
@@ -391,42 +406,48 @@ void FeatureCompatibilityVersion::updateFeatureCompatibilityVersionDocument(
 
     newFCVDoc.setChangeTimestamp(changeTimestamp);
 
-    // (Ignore FCV check): This is intentional because we want to use this feature even if we are in
-    // downgrading fcv state.
-    if (repl::feature_flags::gDowngradingToUpgrading.isEnabledAndIgnoreFCVUnsafe()) {
-        // The setIsCleaningServerMetadata parameter can either be true, false, or boost::none.
-        // True indicates we want to set the isCleaningServerMetadata FCV document field to true.
-        // False indicates we want to remove the isCleaningServerMetadata FCV document field.
-        // boost::none indicates that we don't want to change the current value of the
-        // isCleaningServerMetadata field.
-        if (setIsCleaningServerMetadata.is_initialized()) {
-            // True case: set isCleaningServerMetadata doc field to true.
-            if (*setIsCleaningServerMetadata) {
-                newFCVDoc.setIsCleaningServerMetadata(true);
-            }
-            // Else, false case: don't set the isCleaningServerMetadata field in newFCVDoc. This is
-            // because runUpdateCommand overrides the current whole FCV document with what is in
-            // newFCVDoc so not setting the field is effectively removing it.
-        } else {
-            // boost::none case:
-            // If we don't want to update the isCleaningServerMetadata, we need to make sure not to
-            // override the existing field if it exists, so get the current isCleaningServerMetadata
-            // field value from the current FCV document and set it in newFCVDoc.
-            // This is to protect against the case where a previous FCV downgrade failed
-            // in the isCleaningServerMetadata phase, and the user runs setFCV again. In that
-            // case we do not want to remove the existing isCleaningServerMetadata FCV doc field
-            // because it would not be safe to upgrade the FCV.
-            auto currentFCVObj = findFeatureCompatibilityVersionDocument(opCtx);
-            auto currentFCVDoc = FeatureCompatibilityVersionDocument::parse(
-                IDLParserContext("featureCompatibilityVersionDocument"), currentFCVObj.value());
+    // The setIsCleaningServerMetadata parameter can either be true, false, or boost::none.
+    // True indicates we want to set the isCleaningServerMetadata FCV document field to true.
+    // False indicates we want to remove the isCleaningServerMetadata FCV document field.
+    // boost::none indicates that we don't want to change the current value of the
+    // isCleaningServerMetadata field.
+    if (setIsCleaningServerMetadata.is_initialized()) {
+        // True case: set isCleaningServerMetadata doc field to true.
+        if (*setIsCleaningServerMetadata) {
+            newFCVDoc.setIsCleaningServerMetadata(true);
+        }
+        // Else, false case: don't set the isCleaningServerMetadata field in newFCVDoc. This is
+        // because runUpdateCommand overrides the current whole FCV document with what is in
+        // newFCVDoc so not setting the field is effectively removing it.
+    } else {
+        // boost::none case:
+        // If we don't want to update the isCleaningServerMetadata, we need to make sure not to
+        // override the existing field if it exists, so get the current isCleaningServerMetadata
+        // field value from the current FCV document and set it in newFCVDoc.
+        // This is to protect against the case where a previous FCV downgrade failed
+        // in the isCleaningServerMetadata phase, and the user runs setFCV again. In that
+        // case we do not want to remove the existing isCleaningServerMetadata FCV doc field
+        // because it would not be safe to upgrade the FCV.
+        auto currentFCVObj = findFeatureCompatibilityVersionDocument(opCtx);
+        if (!currentFCVObj.isOK()) {
+            Status status = currentFCVObj.getStatus();
+            invariant(ErrorCodes::isRetriableError(status));
+            uasserted(8531601,
+                      str::stream()
+                          << "failed to update FCV document because the existing FCV document "
+                             "could not be found due to error: "
+                          << status.toString() << ". Retry the setFCV request.");
+        }
 
-            auto currentIsCleaningServerMetadata = currentFCVDoc.getIsCleaningServerMetadata();
-            if (currentIsCleaningServerMetadata.is_initialized() &&
-                *currentIsCleaningServerMetadata) {
-                newFCVDoc.setIsCleaningServerMetadata(*currentIsCleaningServerMetadata);
-            }
+        auto currentFCVDoc = FeatureCompatibilityVersionDocument::parse(
+            IDLParserContext("featureCompatibilityVersionDocument"), currentFCVObj.getValue());
+
+        auto currentIsCleaningServerMetadata = currentFCVDoc.getIsCleaningServerMetadata();
+        if (currentIsCleaningServerMetadata.is_initialized() && *currentIsCleaningServerMetadata) {
+            newFCVDoc.setIsCleaningServerMetadata(*currentIsCleaningServerMetadata);
         }
     }
+
     // Replace the current FCV document with newFCVDoc.
     runUpdateCommand(opCtx, newFCVDoc);
 }
@@ -441,13 +462,12 @@ void FeatureCompatibilityVersion::setIfCleanStartup(OperationContext* opCtx,
         return;
     }
 
-
     // If the server was not started with --shardsvr, the default featureCompatibilityVersion on
     // clean startup is the upgrade version. If it was started with --shardsvr, the default
     // featureCompatibilityVersion is the downgrade version, so that it can be safely added to a
     // downgrade version cluster. The config server will run setFeatureCompatibilityVersion as
     // part of addShard.
-    const bool storeUpgradeVersion = !serverGlobalParams.clusterRole.exclusivelyHasShardRole();
+    const bool storeUpgradeVersion = !serverGlobalParams.clusterRole.isShardOnly();
 
     UnreplicatedWritesBlock unreplicatedWritesBlock(opCtx);
     NamespaceString nss(NamespaceString::kServerConfigurationNamespace);
@@ -513,10 +533,17 @@ bool FeatureCompatibilityVersion::hasNoReplicatedCollections(OperationContext* o
     return true;
 }
 
-void FeatureCompatibilityVersion::updateMinWireVersion() {
-    WireSpec& wireSpec = WireSpec::instance();
+void FeatureCompatibilityVersion::updateMinWireVersion(OperationContext* opCtx) {
+    WireSpec& wireSpec = WireSpec::getWireSpec(opCtx->getServiceContext());
     const auto fcvSnapshot = serverGlobalParams.featureCompatibility.acquireFCVSnapshot();
     const auto currentFcv = fcvSnapshot.getVersion();
+    // The reason we set the minWireVersion to LATEST_WIRE_VERSION when downgrading from latest as
+    // well as on upgrading to latest is because we shouldn’t decrease the minWireVersion until we
+    // have fully downgraded to the lower FCV in case we get any backwards compatibility breakages,
+    // since during `kDowngradingFrom_X_to_Y` we may still be stopping/cleaning up any features from
+    // the upgraded FCV. In essence, a node with the upgraded FCV/binary should not be able to
+    // communicate with downgraded binary nodes until the FCV is completely downgraded to
+    // `kVersion_Y`.
     if (currentFcv == GenericFCV::kLatest ||
         (fcvSnapshot.isUpgradingOrDowngrading() &&
          currentFcv != GenericFCV::kUpgradingFromLastLTSToLastContinuous)) {
@@ -543,9 +570,9 @@ void FeatureCompatibilityVersion::updateMinWireVersion() {
 
 void FeatureCompatibilityVersion::initializeForStartup(OperationContext* opCtx) {
     // Global write lock must be held.
-    invariant(opCtx->lockState()->isW());
+    invariant(shard_role_details::getLocker(opCtx)->isW());
     auto featureCompatibilityVersion = findFeatureCompatibilityVersionDocument(opCtx);
-    if (!featureCompatibilityVersion) {
+    if (!featureCompatibilityVersion.isOK()) {
         serverGlobalParams.featureCompatibility.acquireFCVSnapshot().logFCVWithContext(
             "startup"_sd);
         return;
@@ -553,7 +580,8 @@ void FeatureCompatibilityVersion::initializeForStartup(OperationContext* opCtx)
 
     // If the server configuration collection already contains a valid
     // featureCompatibilityVersion document, cache it in-memory as a server parameter.
-    auto swVersion = FeatureCompatibilityVersionParser::parse(*featureCompatibilityVersion);
+    auto swVersion =
+        FeatureCompatibilityVersionParser::parse(featureCompatibilityVersion.getValue());
 
     // Note this error path captures all cases of an FCV document existing, but with any
     // unacceptable value. This includes unexpected cases with no path forward such as the FCV
@@ -572,9 +600,9 @@ void FeatureCompatibilityVersion::initializeForStartup(OperationContext* opCtx)
 
     auto version = swVersion.getValue();
     serverGlobalParams.mutableFCV.setVersion(version);
-    FeatureCompatibilityVersion::updateMinWireVersion();
-
+    FeatureCompatibilityVersion::updateMinWireVersion(opCtx);
     const auto fcvSnapshot = serverGlobalParams.featureCompatibility.acquireFCVSnapshot();
+
     fcvSnapshot.logFCVWithContext("startup"_sd);
 
     // On startup, if the version is in an upgrading or downgrading state, print a warning.
@@ -594,7 +622,8 @@ void FeatureCompatibilityVersion::initializeForStartup(OperationContext* opCtx)
 void FeatureCompatibilityVersion::fassertInitializedAfterStartup(OperationContext* opCtx) {
     Lock::GlobalWrite lk(opCtx);
     const auto replProcess = repl::ReplicationProcess::get(opCtx);
-    const bool usingReplication = repl::ReplicationCoordinator::get(opCtx)->isReplEnabled();
+    const bool usingReplication =
+        repl::ReplicationCoordinator::get(opCtx)->getSettings().isReplSet();
 
     // The node did not complete the last initial sync. If the initial sync flag is set and we
     // are part of a replica set, we expect the version to be initialized as part of initial
@@ -607,10 +636,6 @@ void FeatureCompatibilityVersion::fassertInitializedAfterStartup(OperationContex
 
     auto fcvDocument = findFeatureCompatibilityVersionDocument(opCtx);
 
-    // TODO SERVER-65269: Move downgrading->upgrading transition back to FCVTransitions
-    // constructor. Adding the new fcv downgrading -> upgrading path
-    fcvTransitions.featureFlaggedAddNewTransitionState();
-
     auto const storageEngine = opCtx->getServiceContext()->getStorageEngine();
     auto dbNames = storageEngine->listDatabases();
     bool nonLocalDatabases = std::any_of(
@@ -618,7 +643,7 @@ void FeatureCompatibilityVersion::fassertInitializedAfterStartup(OperationContex
 
     // Fail to start up if there is no featureCompatibilityVersion document and there are
     // non-local databases present.
-    if (!fcvDocument && nonLocalDatabases) {
+    if (!fcvDocument.isOK() && nonLocalDatabases) {
         LOGV2_FATAL_NOTRACE(40652,
                             "Unable to start up mongod due to missing featureCompatibilityVersion "
                             "document. Please run with --repair to restore the document.");
@@ -640,7 +665,7 @@ void FeatureCompatibilityVersion::addTransitionFromLatestToLastContinuous() {
 }
 
 Lock::ExclusiveLock FeatureCompatibilityVersion::enterFCVChangeRegion(OperationContext* opCtx) {
-    invariant(!opCtx->lockState()->isLocked());
+    invariant(!shard_role_details::getLocker(opCtx)->isLocked());
     return Lock::ExclusiveLock(opCtx, fcvDocumentLock);
 }
 
@@ -679,8 +704,7 @@ void FeatureCompatibilityVersionParameter::append(OperationContext* opCtx,
         // not part of the majority snapshot, the downgraded binary will see the upgrade FCV and
         // fail.)
         const auto replCoordinator = repl::ReplicationCoordinator::get(opCtx);
-        const bool isReplSet = replCoordinator &&
-            replCoordinator->getReplicationMode() == repl::ReplicationCoordinator::modeReplSet;
+        const bool isReplSet = replCoordinator && replCoordinator->getSettings().isReplSet();
         auto neededMajorityTimestamp = [] {
             stdx::lock_guard lk(lastFCVUpdateTimestampMutex);
             return lastFCVUpdateTimestamp;
@@ -707,8 +731,8 @@ Status FeatureCompatibilityVersionParameter::setFromString(StringData,
 
 FixedFCVRegion::FixedFCVRegion(OperationContext* opCtx)
     : _lk([&] {
-          invariant(!opCtx->lockState()->isLocked());
-          invariant(!opCtx->lockState()->isRSTLLocked());
+          invariant(!shard_role_details::getLocker(opCtx)->isLocked());
+          invariant(!shard_role_details::getLocker(opCtx)->isRSTLLocked());
           return Lock::SharedLock(opCtx, fcvDocumentLock);
       }()) {}
 
