@@ -27,51 +27,98 @@
  *    it in the license file.
  */
 
-#include <boost/optional.hpp>
+#include <boost/smart_ptr.hpp>
+#include <memory>
+#include <set>
+#include <string>
+#include <utility>
+#include <vector>
 
-#include "mongo/base/status_with.h"
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
 #include "mongo/bson/bsonobj.h"
 #include "mongo/bson/bsonobjbuilder.h"
-#include "mongo/db/catalog/collection_yield_restore.h"
+#include "mongo/crypto/fle_field_schema_gen.h"
+#include "mongo/db/auth/action_set.h"
+#include "mongo/db/auth/action_type.h"
+#include "mongo/db/auth/authorization_session.h"
+#include "mongo/db/auth/privilege.h"
+#include "mongo/db/auth/resource_pattern.h"
+#include "mongo/db/basic_types.h"
+#include "mongo/db/catalog/collection.h"
+#include "mongo/db/catalog/database_holder.h"
 #include "mongo/db/catalog/document_validation.h"
-#include "mongo/db/catalog_raii.h"
 #include "mongo/db/client.h"
 #include "mongo/db/commands.h"
 #include "mongo/db/commands/update_metrics.h"
 #include "mongo/db/concurrency/exception_util.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
+#include "mongo/db/curop.h"
 #include "mongo/db/curop_failpoint_helpers.h"
-#include "mongo/db/exec/update_stage.h"
+#include "mongo/db/database_name.h"
 #include "mongo/db/fle_crud.h"
-#include "mongo/db/matcher/extensions_callback_real.h"
 #include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/ops/delete_request_gen.h"
 #include "mongo/db/ops/insert.h"
 #include "mongo/db/ops/parsed_delete.h"
 #include "mongo/db/ops/parsed_update.h"
+#include "mongo/db/ops/parsed_writes_common.h"
+#include "mongo/db/ops/update_request.h"
 #include "mongo/db/ops/update_result.h"
 #include "mongo/db/ops/write_ops_exec.h"
+#include "mongo/db/ops/write_ops_gen.h"
+#include "mongo/db/ops/write_ops_parsers.h"
 #include "mongo/db/ops/write_ops_retryability.h"
-#include "mongo/db/query/collection_query_info.h"
+#include "mongo/db/pipeline/legacy_runtime_constants_gen.h"
+#include "mongo/db/pipeline/variables.h"
+#include "mongo/db/query/explain.h"
+#include "mongo/db/query/explain_options.h"
+#include "mongo/db/query/find_command.h"
+#include "mongo/db/query/find_command_gen.h"
 #include "mongo/db/query/get_executor.h"
-#include "mongo/db/query/plan_executor.h"
-#include "mongo/db/query/plan_summary_stats.h"
+#include "mongo/db/query/plan_yield_policy.h"
+#include "mongo/db/query/query_settings/query_settings_gen.h"
 #include "mongo/db/repl/repl_client_info.h"
 #include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/s/collection_sharding_state.h"
-#include "mongo/db/s/operation_sharding_state.h"
 #include "mongo/db/s/query_analysis_writer.h"
-#include "mongo/db/stats/counters.h"
-#include "mongo/db/stats/resource_consumption_metrics.h"
+#include "mongo/db/service_context.h"
+#include "mongo/db/session/logical_session_id.h"
+#include "mongo/db/shard_role.h"
 #include "mongo/db/stats/top.h"
 #include "mongo/db/storage/duplicate_key_error_info.h"
+#include "mongo/db/timeseries/timeseries_index_schema_conversion_functions.h"
+#include "mongo/db/timeseries/timeseries_update_delete_util.h"
+#include "mongo/db/timeseries/timeseries_write_util.h"
 #include "mongo/db/transaction/retryable_writes_stats.h"
 #include "mongo/db/transaction/transaction_participant.h"
+#include "mongo/db/transaction_resources.h"
 #include "mongo/db/transaction_validation.h"
 #include "mongo/db/update/update_util.h"
-#include "mongo/db/write_concern.h"
-#include "mongo/logv2/log.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/log_severity.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/rpc/op_msg.h"
+#include "mongo/rpc/reply_builder_interface.h"
+#include "mongo/s/analyze_shard_key_common_gen.h"
+#include "mongo/s/analyze_shard_key_role.h"
 #include "mongo/s/query_analysis_sampler_util.h"
 #include "mongo/s/would_change_owning_shard_exception.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/duration.h"
+#include "mongo/util/fail_point.h"
+#include "mongo/util/future.h"
 #include "mongo/util/log_and_backoff.h"
+#include "mongo/util/serialization_context.h"
+#include "mongo/util/str.h"
+#include "mongo/util/uuid.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
 
@@ -128,6 +175,7 @@ void makeDeleteRequest(OperationContext* opCtx,
     requestOut->setIsExplain(explain);
 
     requestOut->setYieldPolicy(PlanYieldPolicy::YieldPolicy::YIELD_AUTO);
+    requestOut->setIsTimeseriesNamespace(request.getIsTimeseriesNamespace());
 }
 
 write_ops::FindAndModifyCommandReply buildResponse(
@@ -158,7 +206,7 @@ write_ops::FindAndModifyCommandReply buildResponse(
 void assertCanWrite_inlock(OperationContext* opCtx, const NamespaceString& nss) {
     uassert(ErrorCodes::NotWritablePrimary,
             str::stream() << "Not primary while running findAndModify command on collection "
-                          << nss.ns(),
+                          << nss.toStringForErrorMsg(),
             repl::ReplicationCoordinator::get(opCtx)->canAcceptWritesFor(opCtx, nss));
 
     CollectionShardingState::assertCollectionLockedAndAcquire(opCtx, nss)
@@ -169,7 +217,7 @@ void recordStatsForTopCommand(OperationContext* opCtx) {
     auto curOp = CurOp::get(opCtx);
     Top::get(opCtx->getClient()->getServiceContext())
         .record(opCtx,
-                curOp->getNS(),
+                curOp->getNSS(),
                 curOp->getLogicalOp(),
                 Top::LockType::WriteLocked,
                 durationCount<Microseconds>(curOp->elapsedTimeExcludingPauses()),
@@ -181,7 +229,7 @@ void checkIfTransactionOnCappedColl(const CollectionPtr& coll, bool inTransactio
     if (coll && coll->isCapped()) {
         uassert(
             ErrorCodes::OperationNotSupportedInTransaction,
-            str::stream() << "Collection '" << coll->ns()
+            str::stream() << "Collection '" << coll->ns().toStringForErrorMsg()
                           << "' is a capped collection. Writes in transactions are not allowed on "
                              "capped collections.",
             !inTransaction);
@@ -211,8 +259,8 @@ public:
         return true;
     }
 
-    static void collectMetrics(const Request& request) {
-        CmdFindAndModify::_updateMetrics.collectMetrics(request);
+    void collectMetrics(const Request& request) const {
+        _updateMetrics->collectMetrics(request);
     }
 
     bool supportsRetryableWrite() const final {
@@ -239,6 +287,10 @@ public:
             return true;
         }
 
+        bool isSubjectToIngressAdmissionControl() const override {
+            return true;
+        }
+
         NamespaceString ns() const final {
             return this->request().getNamespace();
         }
@@ -254,12 +306,17 @@ public:
         void appendMirrorableRequest(BSONObjBuilder* bob) const final;
     };
 
+protected:
+    void doInitializeClusterRole(ClusterRole role) override {
+        write_ops::FindAndModifyCmdVersion1Gen<CmdFindAndModify>::doInitializeClusterRole(role);
+        _updateMetrics.emplace(getName(), role);
+    }
+
 private:
     // Update related command execution metrics.
-    static UpdateMetrics _updateMetrics;
-} cmdFindAndModify;
-
-UpdateMetrics CmdFindAndModify::_updateMetrics{"findAndModify"};
+    mutable boost::optional<UpdateMetrics> _updateMetrics;
+};
+MONGO_REGISTER_COMMAND(CmdFindAndModify).forShard();
 
 void CmdFindAndModify::Invocation::doCheckAuthorization(OperationContext* opCtx) const {
     std::vector<Privilege> privileges;
@@ -281,8 +338,7 @@ void CmdFindAndModify::Invocation::doCheckAuthorization(OperationContext* opCtx)
         actions.addAction(ActionType::bypassDocumentValidation);
     }
 
-    ResourcePattern resource(
-        CommandHelpers::resourcePatternForNamespace(request.getNamespace().toString()));
+    ResourcePattern resource(CommandHelpers::resourcePatternForNamespace(request.getNamespace()));
     uassert(17138,
             "Invalid target namespace " + resource.toString(),
             resource.isExactNamespacePattern());
@@ -290,7 +346,7 @@ void CmdFindAndModify::Invocation::doCheckAuthorization(OperationContext* opCtx)
 
     uassert(ErrorCodes::Unauthorized,
             str::stream() << "Not authorized to find and modify on database'"
-                          << this->request().getDbName() << "'",
+                          << this->request().getDbName().toStringForErrorMsg() << "'",
             AuthorizationSession::get(opCtx->getClient())->isAuthorizedForPrivileges(privileges));
 }
 
@@ -320,7 +376,8 @@ void CmdFindAndModify::Invocation::explain(OperationContext* opCtx,
     }();
     auto request = requestAndMsg.first;
 
-    const NamespaceString& nss = request.getNamespace();
+    auto [isTimeseriesViewRequest, nss] = timeseries::isTimeseriesViewRequest(opCtx, request);
+
     uassertStatusOK(userAllowedWriteNS(opCtx, nss));
     OpDebug* const opDebug = &curOp->debug();
     auto const dbName = request.getDbName();
@@ -331,50 +388,91 @@ void CmdFindAndModify::Invocation::explain(OperationContext* opCtx,
         const bool isExplain = true;
         makeDeleteRequest(opCtx, request, isExplain, &deleteRequest);
 
-        ParsedDelete parsedDelete(opCtx, &deleteRequest);
-        uassertStatusOK(parsedDelete.parseRequest());
-
         // Explain calls of the findAndModify command are read-only, but we take write
         // locks so that the timing information is more accurate.
-        AutoGetCollection collection(opCtx, nss, MODE_IX);
+        const auto collection =
+            acquireCollection(opCtx,
+                              CollectionAcquisitionRequest::fromOpCtx(
+                                  opCtx, nss, AcquisitionPrerequisites::OperationType::kWrite),
+                              MODE_IX);
         uassert(ErrorCodes::NamespaceNotFound,
                 str::stream() << "database " << dbName.toStringForErrorMsg() << " does not exist",
-                collection.getDb());
+                DatabaseHolder::get(opCtx)->getDb(opCtx, nss.dbName()));
+
+        if (isTimeseriesViewRequest) {
+            timeseries::timeseriesRequestChecks<DeleteRequest>(
+                collection.getCollectionPtr(),
+                &deleteRequest,
+                timeseries::deleteRequestCheckFunction);
+            timeseries::timeseriesHintTranslation<DeleteRequest>(collection.getCollectionPtr(),
+                                                                 &deleteRequest);
+        }
+
+        ParsedDelete parsedDelete(
+            opCtx, &deleteRequest, collection.getCollectionPtr(), isTimeseriesViewRequest);
+        uassertStatusOK(parsedDelete.parseRequest());
 
         CollectionShardingState::assertCollectionLockedAndAcquire(opCtx, nss)
             ->checkShardVersionOrThrow(opCtx);
 
-        const auto exec = uassertStatusOK(
-            getExecutorDelete(opDebug, &collection.getCollection(), &parsedDelete, verbosity));
+        const auto exec =
+            uassertStatusOK(getExecutorDelete(opDebug, collection, &parsedDelete, verbosity));
 
         auto bodyBuilder = result->getBodyBuilder();
         Explain::explainStages(
-            exec.get(), collection.getCollection(), verbosity, BSONObj(), cmdObj, &bodyBuilder);
+            exec.get(),
+            collection.getCollectionPtr(),
+            verbosity,
+            BSONObj(),
+            SerializationContext::stateCommandReply(request.getSerializationContext()),
+            cmdObj,
+            &bodyBuilder);
     } else {
         auto updateRequest = UpdateRequest();
         updateRequest.setNamespaceString(nss);
         update::makeUpdateRequest(opCtx, request, verbosity, &updateRequest);
 
-        const ExtensionsCallbackReal extensionsCallback(opCtx, &updateRequest.getNamespaceString());
-        ParsedUpdate parsedUpdate(opCtx, &updateRequest, extensionsCallback);
-        uassertStatusOK(parsedUpdate.parseRequest());
-
         // Explain calls of the findAndModify command are read-only, but we take write
         // locks so that the timing information is more accurate.
-        AutoGetCollection collection(opCtx, nss, MODE_IX);
+        const auto collection =
+            acquireCollection(opCtx,
+                              CollectionAcquisitionRequest::fromOpCtx(
+                                  opCtx, nss, AcquisitionPrerequisites::OperationType::kWrite),
+                              MODE_IX);
         uassert(ErrorCodes::NamespaceNotFound,
                 str::stream() << "database " << dbName.toStringForErrorMsg() << " does not exist",
-                collection.getDb());
+                DatabaseHolder::get(opCtx)->getDb(opCtx, nss.dbName()));
+        if (isTimeseriesViewRequest) {
+            timeseries::timeseriesRequestChecks<UpdateRequest>(
+                collection.getCollectionPtr(),
+                &updateRequest,
+                timeseries::updateRequestCheckFunction);
+            timeseries::timeseriesHintTranslation<UpdateRequest>(collection.getCollectionPtr(),
+                                                                 &updateRequest);
+        }
+
+        ParsedUpdate parsedUpdate(opCtx,
+                                  &updateRequest,
+                                  collection.getCollectionPtr(),
+                                  false /*forgoOpCounterIncrements*/,
+                                  isTimeseriesViewRequest);
+        uassertStatusOK(parsedUpdate.parseRequest());
 
         CollectionShardingState::assertCollectionLockedAndAcquire(opCtx, nss)
             ->checkShardVersionOrThrow(opCtx);
 
-        const auto exec = uassertStatusOK(
-            getExecutorUpdate(opDebug, &collection.getCollection(), &parsedUpdate, verbosity));
+        const auto exec =
+            uassertStatusOK(getExecutorUpdate(opDebug, collection, &parsedUpdate, verbosity));
 
         auto bodyBuilder = result->getBodyBuilder();
         Explain::explainStages(
-            exec.get(), collection.getCollection(), verbosity, BSONObj(), cmdObj, &bodyBuilder);
+            exec.get(),
+            collection.getCollectionPtr(),
+            verbosity,
+            BSONObj(),
+            SerializationContext::stateCommandReply(request.getSerializationContext()),
+            cmdObj,
+            &bodyBuilder);
     }
 }
 
@@ -385,13 +483,13 @@ write_ops::FindAndModifyCommandReply CmdFindAndModify::Invocation::typedRun(
     validate(req);
 
     // Start the query planning timer right after parsing.
-    auto const curOp = CurOp::get(opCtx);
-    curOp->beginQueryPlanningTimer();
+    auto& curOp = *CurOp::get(opCtx);
+    curOp.beginQueryPlanningTimer();
 
     if (req.getEncryptionInformation().has_value()) {
         {
             stdx::lock_guard<Client> lk(*opCtx->getClient());
-            CurOp::get(opCtx)->setShouldOmitDiagnosticInformation_inlock(lk, true);
+            curOp.setShouldOmitDiagnosticInformation_inlock(lk, true);
         }
         if (!req.getEncryptionInformation()->getCrudProcessed().get_value_or(false)) {
             return processFLEFindAndModify(opCtx, req);
@@ -400,14 +498,12 @@ write_ops::FindAndModifyCommandReply CmdFindAndModify::Invocation::typedRun(
 
     const NamespaceString& nsString = req.getNamespace();
     uassertStatusOK(userAllowedWriteNS(opCtx, nsString));
-    OpDebug* const opDebug = &curOp->debug();
 
-    // Collect metrics.
-    CmdFindAndModify::collectMetrics(req);
+    static_cast<const CmdFindAndModify*>(definition())->collectMetrics(req);
 
     auto disableDocumentValidation = req.getBypassDocumentValidation().value_or(false);
-    auto fleCrudProcessed =
-        write_ops_exec::getFleCrudProcessed(opCtx, req.getEncryptionInformation());
+    auto fleCrudProcessed = write_ops_exec::getFleCrudProcessed(
+        opCtx, req.getEncryptionInformation(), nsString.tenantId());
 
     DisableDocumentSchemaValidationIfTrue docSchemaValidationDisabler(opCtx,
                                                                       disableDocumentValidation);
@@ -442,11 +538,25 @@ write_ops::FindAndModifyCommandReply CmdFindAndModify::Invocation::typedRun(
         }
     }
 
+    // Initialize curOp information.
+    {
+        stdx::lock_guard<Client> lk(*opCtx->getClient());
+        if (req.getIsTimeseriesNamespace() && nsString.isTimeseriesBucketsCollection()) {
+            auto viewNss = nsString.getTimeseriesViewNamespace();
+            curOp.setNS_inlock(viewNss);
+            curOp.setOpDescription_inlock(timeseries::timeseriesViewCommand(
+                unparsedRequest().body, "findAndModify", viewNss.coll()));
+        } else {
+            curOp.setNS_inlock(nsString);
+        }
+        curOp.ensureStarted();
+    }
+
     auto sampleId = analyze_shard_key::getOrGenerateSampleId(
         opCtx, ns(), analyze_shard_key::SampledCommandNameEnum::kFindAndModify, req);
     if (sampleId) {
         analyze_shard_key::QueryAnalysisWriter::get(opCtx)
-            ->addFindAndModifyQuery(*sampleId, req)
+            ->addFindAndModifyQuery(opCtx, *sampleId, req)
             .getAsync([](auto) {});
     }
 
@@ -456,10 +566,7 @@ write_ops::FindAndModifyCommandReply CmdFindAndModify::Invocation::typedRun(
 
     const bool inTransaction = opCtx->inMultiDocumentTransaction();
 
-    // Although usually the PlanExecutor handles WCE internally, it will throw WCEs when it
-    // is executing a findAndModify. This is done to ensure that we can always match,
-    // modify, and return the document under concurrency, if a matching document exists.
-    return writeConflictRetry(opCtx, "findAndModify", nsString.ns(), [&] {
+    auto doWork = [&] {
         if (req.getRemove().value_or(false)) {
             DeleteRequest deleteRequest;
             makeDeleteRequest(opCtx, req, false, &deleteRequest);
@@ -468,8 +575,8 @@ write_ops::FindAndModifyCommandReply CmdFindAndModify::Invocation::typedRun(
                 deleteRequest.setStmtId(stmtId);
             }
             boost::optional<BSONObj> docFound;
-            write_ops_exec::writeConflictRetryRemove(
-                opCtx, nsString, &deleteRequest, curOp, opDebug, inTransaction, docFound);
+            write_ops_exec::performDelete(
+                opCtx, nsString, &deleteRequest, &curOp, inTransaction, boost::none, docFound);
             recordStatsForTopCommand(opCtx);
             return buildResponse(boost::none, true /* isRemove */, docFound);
         } else {
@@ -497,36 +604,26 @@ write_ops::FindAndModifyCommandReply CmdFindAndModify::Invocation::typedRun(
                 updateRequest.setAllowShardKeyUpdatesWithoutFullShardKeyInQuery(
                     req.getAllowShardKeyUpdatesWithoutFullShardKeyInQuery());
 
-                const ExtensionsCallbackReal extensionsCallback(
-                    opCtx, &updateRequest.getNamespaceString());
-                ParsedUpdate parsedUpdate(opCtx, &updateRequest, extensionsCallback);
-                uassertStatusOK(parsedUpdate.parseRequest());
-
                 try {
                     boost::optional<BSONObj> docFound;
                     auto updateResult =
-                        write_ops_exec::writeConflictRetryUpsert(opCtx,
-                                                                 nsString,
-                                                                 curOp,
-                                                                 opDebug,
-                                                                 inTransaction,
-                                                                 req.getRemove().value_or(false),
-                                                                 req.getUpsert().value_or(false),
-                                                                 docFound,
-                                                                 &parsedUpdate);
+                        write_ops_exec::performUpdate(opCtx,
+                                                      nsString,
+                                                      &curOp,
+                                                      inTransaction,
+                                                      req.getRemove().value_or(false),
+                                                      req.getUpsert().value_or(false),
+                                                      boost::none,
+                                                      docFound,
+                                                      &updateRequest);
                     recordStatsForTopCommand(opCtx);
                     return buildResponse(updateResult, req.getRemove().value_or(false), docFound);
 
                 } catch (const ExceptionFor<ErrorCodes::DuplicateKey>& ex) {
-                    if (!parsedUpdate.hasParsedQuery()) {
-                        uassertStatusOK(parsedUpdate.parseQueryToCQ());
-                    }
-
+                    auto cq = uassertStatusOK(
+                        parseWriteQueryToCQ(opCtx, nullptr /* expCtx */, updateRequest));
                     if (!write_ops_exec::shouldRetryDuplicateKeyException(
-                            opCtx,
-                            parsedUpdate,
-                            *ex.extraInfo<DuplicateKeyErrorInfo>(),
-                            retryAttempts)) {
+                            updateRequest, *cq, *ex.extraInfo<DuplicateKeyErrorInfo>())) {
                         throw;
                     }
 
@@ -560,7 +657,17 @@ write_ops::FindAndModifyCommandReply CmdFindAndModify::Invocation::typedRun(
                 }
             }
         }
-    });
+    };
+
+    // No need to call writeConflictRetry() since it does not retry if in a transaction,
+    // but calling it can cause WCE to be double counted.
+    if (inTransaction) {
+        return doWork();
+    }
+    // Although usually the PlanExecutor handles WCE internally, it will throw WCEs when it
+    // is executing a findAndModify. This is done to ensure that we can always match,
+    // modify, and return the document under concurrency, if a matching document exists.
+    return writeConflictRetry(opCtx, "findAndModify", nsString, doWork);
 }
 
 void CmdFindAndModify::Invocation::appendMirrorableRequest(BSONObjBuilder* bob) const {
