@@ -27,81 +27,56 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
-
-#include "mongo/platform/basic.h"
 
+#include <cstdint>
+#include <memory>
+#include <set>
 #include <sstream>
 #include <string>
+#include <utility>
 #include <vector>
 
-#include "mongo/base/init.h"
+#include <absl/container/node_hash_set.h>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/init.h"  // IWYU pragma: keep
+#include "mongo/base/initializer.h"
 #include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonobj_comparator_interface.h"
+#include "mongo/bson/bsontypes.h"
 #include "mongo/bson/simple_bsonobj_comparator.h"
+#include "mongo/db/auth/action_type.h"
 #include "mongo/db/auth/authorization_session.h"
+#include "mongo/db/auth/resource_pattern.h"
 #include "mongo/db/catalog/collection.h"
-#include "mongo/db/catalog/database.h"
-#include "mongo/db/client.h"
 #include "mongo/db/commands/index_filter_commands.h"
 #include "mongo/db/commands/plan_cache_commands.h"
 #include "mongo/db/db_raii.h"
-#include "mongo/db/jsobj.h"
 #include "mongo/db/matcher/expression_parser.h"
 #include "mongo/db/matcher/extensions_callback_real.h"
 #include "mongo/db/namespace_string.h"
+#include "mongo/db/pipeline/expression_context.h"
+#include "mongo/db/query/canonical_query.h"
 #include "mongo/db/query/canonical_query_encoder.h"
 #include "mongo/db/query/collection_query_info.h"
-#include "mongo/db/query/plan_cache_key_factory.h"
+#include "mongo/db/query/find_command.h"
 #include "mongo/db/query/query_settings_decoration.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/redaction.h"
+#include "mongo/stdx/type_traits.h"
 #include "mongo/stdx/unordered_set.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/intrusive_counter.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
-namespace {
-
-using std::string;
-using std::vector;
-using namespace mongo;
-
-//
-// Command instances.
-// Registers commands with the command system and make commands
-// available to the client.
-//
-
-MONGO_INITIALIZER_WITH_PREREQUISITES(SetupIndexFilterCommands, ())
-(InitializerContext* context) {
-    new ListFilters();
-    new ClearFilters();
-    new SetFilter();
-}
-
-/**
- * Remove the plan cache entries whose 'indexFilterKey' matches any key in 'indexFilterKeys'. Please
- * note that we do not handle 'indexFilterKey' hash collisions, namely it's fine to clear a plan
- * cache entry that we technically could have kept around.
- */
-void removePlanCacheEntriesByIndexFilterKeys(const stdx::unordered_set<uint32_t>& indexFilterKeys,
-                                             PlanCache* planCache) {
-    planCache->removeIf([&indexFilterKeys](const PlanCacheKey& key, const PlanCacheEntry& entry) {
-        return indexFilterKeys.contains(entry.indexFilterKey);
-    });
-}
-
-/**
- * Similar to removePlanCacheEntriesByIndexFilterKeys() above. This function clears cache entries in
- * a SBE plan cache. There is an extra check on the collection UUID because all collections share
- * one single 'sbe::PlanCache' instance.
- */
-void removePlanCacheEntriesByIndexFilterKeys(const stdx::unordered_set<uint32_t>& indexFilterKeys,
-                                             const UUID& collectionUuid,
-                                             sbe::PlanCache* planCache) {
-    planCache->removeIf([&](const sbe::PlanCacheKey& key, const sbe::PlanCacheEntry& entry) {
-        return indexFilterKeys.contains(entry.indexFilterKey) &&
-            key.getCollectionUuid() == collectionUuid;
-    });
-}
-}  // namespace
+// TODO: SERVER-88503 Remove Index Filters feature.
 
 namespace mongo {
 
@@ -113,10 +88,17 @@ IndexFilterCommand::IndexFilterCommand(const string& name, const string& helpTex
     : BasicCommand(name), helpText(helpText) {}
 
 bool IndexFilterCommand::run(OperationContext* opCtx,
-                             const string& dbname,
+                             const DatabaseName& dbName,
                              const BSONObj& cmdObj,
                              BSONObjBuilder& result) {
-    const NamespaceString nss(CommandHelpers::parseNsCollectionRequired(dbname, cmdObj));
+
+    static Rarely sampler;
+    if (sampler.tick()) {
+        LOGV2_WARNING(7923201,
+                      "Index filters are deprecated, consider using query settings instead. See "
+                      "https://www.mongodb.com/docs/manual/reference/command/setQuerySettings");
+    }
+    const NamespaceString nss(CommandHelpers::parseNsCollectionRequired(dbName, cmdObj));
     AutoGetCollectionForReadCommand ctx(opCtx, nss);
     uassertStatusOK(runIndexFilterCommand(opCtx, ctx.getCollection(), cmdObj, &result));
     return true;
@@ -134,11 +116,11 @@ std::string IndexFilterCommand::help() const {
     return helpText;
 }
 
-Status IndexFilterCommand::checkAuthForCommand(Client* client,
-                                               const std::string& dbname,
-                                               const BSONObj& cmdObj) const {
-    AuthorizationSession* authzSession = AuthorizationSession::get(client);
-    ResourcePattern pattern = parseResourcePattern(dbname, cmdObj);
+Status IndexFilterCommand::checkAuthForOperation(OperationContext* opCtx,
+                                                 const DatabaseName& dbName,
+                                                 const BSONObj& cmdObj) const {
+    AuthorizationSession* authzSession = AuthorizationSession::get(opCtx->getClient());
+    ResourcePattern pattern = parseResourcePattern(dbName, cmdObj);
 
     if (authzSession->isAuthorizedForActionsOnResource(pattern, ActionType::planCacheIndexFilter)) {
         return Status::OK();
@@ -184,6 +166,12 @@ Status ListFilters::list(const QuerySettings& querySettings, BSONObjBuilder* bob
     //  }
     BSONArrayBuilder hintsBuilder(bob->subarrayStart("filters"));
     std::vector<AllowedIndexEntry> entries = querySettings.getAllAllowedIndices();
+    if (entries.empty()) {
+        LOGV2_DEBUG(8403502,
+                    2,
+                    "Found no filters when listing index filters.",
+                    "nentries"_attr = entries.size());
+    }
     for (vector<AllowedIndexEntry>::const_iterator i = entries.begin(); i != entries.end(); ++i) {
         AllowedIndexEntry entry = *i;
 
@@ -228,12 +216,9 @@ Status ClearFilters::runIndexFilterCommand(OperationContext* opCtx,
     invariant(querySettings);
 
     PlanCache* planCacheClassic = CollectionQueryInfo::get(collection).getPlanCache();
-    sbe::PlanCache* planCacheSBE = nullptr;
     invariant(planCacheClassic);
-
-    if (feature_flags::gFeatureFlagSbePlanCache.isEnabledAndIgnoreFCV()) {
-        planCacheSBE = &sbe::getPlanCache(opCtx);
-    }
+    sbe::PlanCache* planCacheSBE = &sbe::getPlanCache(opCtx);
+    invariant(planCacheSBE);
 
     return clear(opCtx, collection, cmdObj, querySettings, planCacheClassic, planCacheSBE);
 }
@@ -249,21 +234,22 @@ Status ClearFilters::clear(OperationContext* opCtx,
     // - clear index filters for single query shape when a query shape is described in the
     //   command arguments.
     if (cmdObj.hasField("query")) {
-        auto statusWithCQ = plan_cache_commands::canonicalize(opCtx, collection->ns().ns(), cmdObj);
+        auto statusWithCQ = plan_cache_commands::canonicalize(opCtx, collection->ns(), cmdObj);
         if (!statusWithCQ.isOK()) {
             return statusWithCQ.getStatus();
         }
 
         std::unique_ptr<CanonicalQuery> cq = std::move(statusWithCQ.getValue());
 
-        querySettings->removeAllowedIndices(cq->encodeKeyForIndexFilters());
+        querySettings->removeAllowedIndices(cq->encodeKeyForPlanCacheCommand());
 
-        stdx::unordered_set<uint32_t> indexFilterKeys({canonical_query_encoder::computeHash(
-            canonical_query_encoder::encodeForIndexFilters(*cq))});
-        removePlanCacheEntriesByIndexFilterKeys(indexFilterKeys, planCacheClassic);
+        stdx::unordered_set<uint32_t> planCacheCommandKeys({canonical_query_encoder::computeHash(
+            canonical_query_encoder::encodeForPlanCacheCommand(*cq))});
+        plan_cache_commands::removePlanCacheEntriesByPlanCacheCommandKeys(planCacheCommandKeys,
+                                                                          planCacheClassic);
         if (planCacheSBE) {
-            removePlanCacheEntriesByIndexFilterKeys(
-                indexFilterKeys, collection->uuid(), planCacheSBE);
+            plan_cache_commands::removePlanCacheEntriesByPlanCacheCommandKeys(
+                planCacheCommandKeys, collection->uuid(), planCacheSBE);
         }
         LOGV2(20479, "Removed index filter on query", "query"_attr = redact(cq->toStringShort()));
 
@@ -282,6 +268,13 @@ Status ClearFilters::clear(OperationContext* opCtx,
     // cache shortly.
     std::vector<AllowedIndexEntry> entries = querySettings->getAllAllowedIndices();
 
+    if (entries.empty()) {
+        LOGV2_DEBUG(8403501,
+                    2,
+                    "Found no existing filters when clearing index filters.",
+                    "nentries"_attr = entries.size());
+    }
+
     // OK to proceed with clearing all the index filters stored in 'QuerySettings'.
     querySettings->clearAllowedIndices();
 
@@ -294,12 +287,13 @@ Status ClearFilters::clear(OperationContext* opCtx,
     // indexed solutions next time the query is run. Resolve plan cache key from (query, sort,
     // projection, and user-defined collation) in query settings entry. Concurrency note: There's no
     // harm in removing plan cache entries one at a time. Only way that
-    // removePlanCacheEntriesByIndexFilterKeys() can fail is when the query shape has been removed
-    // from the cache by some other means (re-index, collection info reset, ...). This is OK since
-    // that's the intended effect of calling the removePlanCacheEntriesByIndexFilterKeys() function
-    // with the key from the index filter entry.
-    stdx::unordered_set<uint32_t> indexFilterKeys;
-    for (auto entry : entries) {
+    // removePlanCacheEntriesByPlanCacheCommandKeys() can fail is when the query shape has been
+    // removed from the cache by some other means (re-index, collection info reset, ...). This is OK
+    // since that's the intended effect of calling the
+    // removePlanCacheEntriesByPlanCacheCommandKeys() function with the key from the index filter
+    // entry.
+    stdx::unordered_set<uint32_t> planCacheCommandKeys;
+    for (const auto& entry : entries) {
         // Create canonical query.
         auto findCommand = std::make_unique<FindCommandRequest>(nss);
         findCommand->setFilter(entry.query);
@@ -307,27 +301,24 @@ Status ClearFilters::clear(OperationContext* opCtx,
         findCommand->setProjection(entry.projection);
         findCommand->setCollation(entry.collation);
         const boost::intrusive_ptr<ExpressionContext> expCtx;
-        auto statusWithCQ =
-            CanonicalQuery::canonicalize(opCtx,
-                                         std::move(findCommand),
-                                         false,
-                                         expCtx,
-                                         extensionsCallback,
-                                         MatchExpressionParser::kAllowAllSpecialFeatures);
-        invariant(statusWithCQ.isOK());
-        std::unique_ptr<CanonicalQuery> cq = std::move(statusWithCQ.getValue());
+        auto cq = std::make_unique<CanonicalQuery>(CanonicalQueryParams{
+            .expCtx = makeExpressionContext(opCtx, *findCommand),
+            .parsedFind = ParsedFindCommandParams{
+                .findCommand = std::move(findCommand),
+                .extensionsCallback = extensionsCallback,
+                .allowedFeatures = MatchExpressionParser::kAllowAllSpecialFeatures}});
 
-        indexFilterKeys.insert(canonical_query_encoder::computeHash(
-            canonical_query_encoder::encodeForIndexFilters(*cq)));
+        planCacheCommandKeys.insert(canonical_query_encoder::computeHash(
+            canonical_query_encoder::encodeForPlanCacheCommand(*cq)));
     }
-    removePlanCacheEntriesByIndexFilterKeys(indexFilterKeys, planCacheClassic);
+    plan_cache_commands::removePlanCacheEntriesByPlanCacheCommandKeys(planCacheCommandKeys,
+                                                                      planCacheClassic);
     if (planCacheSBE) {
-        removePlanCacheEntriesByIndexFilterKeys(indexFilterKeys, collection->uuid(), planCacheSBE);
+        plan_cache_commands::removePlanCacheEntriesByPlanCacheCommandKeys(
+            planCacheCommandKeys, collection->uuid(), planCacheSBE);
     }
 
-    LOGV2(20480,
-          "Removed all index filters for collection",
-          "namespace"_attr = collection->ns().ns());
+    LOGV2(20480, "Removed all index filters for collection", logAttrs(collection->ns()));
 
     return Status::OK();
 }
@@ -348,12 +339,9 @@ Status SetFilter::runIndexFilterCommand(OperationContext* opCtx,
     invariant(querySettings);
 
     PlanCache* planCacheClassic = CollectionQueryInfo::get(collection).getPlanCache();
-    sbe::PlanCache* planCacheSBE = nullptr;
     invariant(planCacheClassic);
-
-    if (feature_flags::gFeatureFlagSbePlanCache.isEnabledAndIgnoreFCV()) {
-        planCacheSBE = &sbe::getPlanCache(opCtx);
-    }
+    sbe::PlanCache* planCacheSBE = &sbe::getPlanCache(opCtx);
+    invariant(planCacheSBE);
 
     return set(opCtx, collection, cmdObj, querySettings, planCacheClassic, planCacheSBE);
 }
@@ -393,7 +381,7 @@ Status SetFilter::set(OperationContext* opCtx,
         }
     }
 
-    auto statusWithCQ = plan_cache_commands::canonicalize(opCtx, collection->ns().ns(), cmdObj);
+    auto statusWithCQ = plan_cache_commands::canonicalize(opCtx, collection->ns(), cmdObj);
     if (!statusWithCQ.isOK()) {
         return statusWithCQ.getStatus();
     }
@@ -402,12 +390,14 @@ Status SetFilter::set(OperationContext* opCtx,
     // Add allowed indices to query settings, overriding any previous entries.
     querySettings->setAllowedIndices(*cq, indexes, indexNames);
 
-    // Remove entries that match 'indexFilterKeys' from both plan caches.
-    stdx::unordered_set<uint32_t> indexFilterKeys({canonical_query_encoder::computeHash(
-        canonical_query_encoder::encodeForIndexFilters(*cq))});
-    removePlanCacheEntriesByIndexFilterKeys(indexFilterKeys, planCacheClassic);
+    // Remove entries that match 'planCacheCommandKeys' from both plan caches.
+    stdx::unordered_set<uint32_t> planCacheCommandKeys({canonical_query_encoder::computeHash(
+        canonical_query_encoder::encodeForPlanCacheCommand(*cq))});
+    plan_cache_commands::removePlanCacheEntriesByPlanCacheCommandKeys(planCacheCommandKeys,
+                                                                      planCacheClassic);
     if (planCacheSBE) {
-        removePlanCacheEntriesByIndexFilterKeys(indexFilterKeys, collection->uuid(), planCacheSBE);
+        plan_cache_commands::removePlanCacheEntriesByPlanCacheCommandKeys(
+            planCacheCommandKeys, collection->uuid(), planCacheSBE);
     }
 
     LOGV2(20481,
@@ -418,4 +408,8 @@ Status SetFilter::set(OperationContext* opCtx,
     return Status::OK();
 }
 
+MONGO_REGISTER_COMMAND(ListFilters).forShard();
+MONGO_REGISTER_COMMAND(ClearFilters).forShard();
+MONGO_REGISTER_COMMAND(SetFilter).forShard();
+
 }  // namespace mongo
