@@ -27,30 +27,38 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/commands/internal_rename_if_options_and_indexes_match_gen.h"
+#include <list>
+#include <memory>
+#include <string>
+#include <vector>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/db/auth/action_type.h"
 #include "mongo/db/auth/authorization_session.h"
+#include "mongo/db/auth/resource_pattern.h"
 #include "mongo/db/catalog/rename_collection.h"
+#include "mongo/db/cluster_role.h"
 #include "mongo/db/commands.h"
-#include "mongo/db/db_raii.h"
-#include "mongo/db/s/collection_sharding_state.h"
-#include "mongo/db/s/database_sharding_state.h"
-#include "mongo/db/s/dist_lock_manager.h"
+#include "mongo/db/commands/internal_rename_if_options_and_indexes_match_gen.h"
+#include "mongo/db/dbdirectclient.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/server_options.h"
+#include "mongo/db/service_context.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/rpc/op_msg.h"
+#include "mongo/s/request_types/sharded_ddl_commands_gen.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/duration.h"
+#include "mongo/util/fail_point.h"
+#include "mongo/util/str.h"
 
 namespace mongo {
-MONGO_FAIL_POINT_DEFINE(blockBeforeInternalRenameIfOptionsAndIndexesMatch);
-
 namespace {
 
-bool isCollectionSharded(OperationContext* opCtx, const NamespaceString& nss) {
-    AutoGetCollectionForRead lock(opCtx, nss);
-    return opCtx->writesAreReplicated() &&
-        CollectionShardingState::get(opCtx, nss)->getCollectionDescription(opCtx).isSharded();
-}
-
-}  // namespace
+MONGO_FAIL_POINT_DEFINE(blockBeforeInternalRenameAndBeforeTakingDDLLocks);
 
 /**
  * Rename a collection while checking collection option and indexes.
@@ -73,30 +81,36 @@ public:
                 std::list<BSONObj>(originalIndexes.begin(), originalIndexes.end());
             const auto& collectionOptions = thisRequest.getCollectionOptions();
 
-            if (serverGlobalParams.clusterRole != ClusterRole::ShardServer) {
-                // No need to acquire additional locks in a non-sharded environment
-                _internalRun(opCtx, fromNss, toNss, indexList, collectionOptions);
-                return;
+            if (MONGO_unlikely(blockBeforeInternalRenameAndBeforeTakingDDLLocks.shouldFail())) {
+                blockBeforeInternalRenameAndBeforeTakingDDLLocks.pauseWhileSet();
             }
 
-            // Check if the receiving shard is still the primary for the database
-            DatabaseShardingState::checkIsPrimaryShardForDb(opCtx, fromNss.db());
+            if (serverGlobalParams.clusterRole.has(ClusterRole::None)) {
+                // No need to acquire additional locks in a non-sharded environment
+                _internalRun(opCtx, fromNss, toNss, indexList, collectionOptions);
+            } else {
+                // Sharded environment. Run the _shardsvrRenameCollection command, which will use
+                // the RenameCollectionCoordinator.
+                RenameCollectionRequest renameCollectionRequest(toNss);
+                renameCollectionRequest.setDropTarget(true);
+                renameCollectionRequest.setExpectedIndexes(originalIndexes);
+                renameCollectionRequest.setExpectedCollectionOptions(collectionOptions);
+                // TODO: SERVER-81975 (PM-1931) remove this once we enable support for $out to
+                // sharded collections.
+                renameCollectionRequest.setTargetMustNotBeSharded(true);
 
-            // Acquiring the local part of the distributed locks for involved namespaces allows:
-            // - Serialize with sharded DDLs, ensuring no concurrent modifications of the
-            // collections.
-            // - Check safely if the target collection is sharded or not.
-            auto distLockManager = DistLockManager::get(opCtx);
-            auto fromLocalDistlock = distLockManager->lockDirectLocally(
-                opCtx, fromNss.ns(), DistLockManager::kDefaultLockTimeout);
-            auto toLocalDistLock = distLockManager->lockDirectLocally(
-                opCtx, toNss.ns(), DistLockManager::kDefaultLockTimeout);
+                ShardsvrRenameCollection shardsvrRenameCollectionRequest(fromNss);
+                shardsvrRenameCollectionRequest.setRenameCollectionRequest(renameCollectionRequest);
 
-            uassert(ErrorCodes::IllegalOperation,
-                    str::stream() << "cannot rename to sharded collection '" << toNss << "'",
-                    !isCollectionSharded(opCtx, toNss));
+                // _shardsvrRenameCollecion requires majority write concern.
+                const auto cmd = CommandHelpers::appendMajorityWriteConcern(
+                    shardsvrRenameCollectionRequest.toBSON({}));
 
-            _internalRun(opCtx, fromNss, toNss, indexList, collectionOptions);
+                DBDirectClient client(opCtx);
+                BSONObj cmdResult;
+                client.runCommand(fromNss.dbName(), cmd, cmdResult);
+                uassertStatusOK(getStatusFromCommandResult(cmdResult));
+            }
         }
 
     private:
@@ -105,10 +119,6 @@ public:
                                  const NamespaceString& toNss,
                                  const std::list<BSONObj>& indexList,
                                  const BSONObj& collectionOptions) {
-            if (MONGO_unlikely(blockBeforeInternalRenameIfOptionsAndIndexesMatch.shouldFail())) {
-                blockBeforeInternalRenameIfOptionsAndIndexesMatch.pauseWhileSet();
-            }
-
             RenameCollectionOptions options;
             options.dropTarget = true;
             options.stayTemp = false;
@@ -129,17 +139,17 @@ public:
             auto from = thisRequest.getFrom();
             auto to = thisRequest.getTo();
             uassert(ErrorCodes::Unauthorized,
-                    str::stream() << "Unauthorized to rename " << from,
+                    str::stream() << "Unauthorized to rename " << from.toStringForErrorMsg(),
                     AuthorizationSession::get(opCtx->getClient())
                         ->isAuthorizedForActionsOnResource(ResourcePattern::forExactNamespace(from),
                                                            ActionType::renameCollection));
             uassert(ErrorCodes::Unauthorized,
-                    str::stream() << "Unauthorized to drop " << to,
+                    str::stream() << "Unauthorized to drop " << to.toStringForErrorMsg(),
                     AuthorizationSession::get(opCtx->getClient())
                         ->isAuthorizedForActionsOnResource(ResourcePattern::forExactNamespace(to),
                                                            ActionType::dropCollection));
             uassert(ErrorCodes::Unauthorized,
-                    str::stream() << "Unauthorized to insert into " << to,
+                    str::stream() << "Unauthorized to insert into " << to.toStringForErrorMsg(),
                     AuthorizationSession::get(opCtx->getClient())
                         ->isAuthorizedForActionsOnResource(ResourcePattern::forExactNamespace(to),
                                                            ActionType::insert));
@@ -162,6 +172,8 @@ public:
     AllowedOnSecondary secondaryAllowed(ServiceContext*) const override {
         return AllowedOnSecondary::kNever;
     }
+};
+MONGO_REGISTER_COMMAND(InternalRenameIfOptionsAndIndexesMatchCmd).forShard();
 
-} internalRenameIfOptionsAndIndexesMatchCmd;
+}  // namespace
 }  // namespace mongo
