@@ -27,11 +27,22 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <iosfwd>
+#include <string>
 
-#include "mongo/base/init.h"
+#include "mongo/base/init.h"  // IWYU pragma: keep
+#include "mongo/base/initializer.h"
+#include "mongo/base/status.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/db/admission/execution_admission_context.h"
 #include "mongo/db/commands.h"
+#include "mongo/db/database_name.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/repl/isself.h"
+#include "mongo/db/service_context.h"
+#include "mongo/db/transaction_resources.h"
 
 namespace mongo {
 
@@ -71,16 +82,15 @@ public:
     bool run(OperationContext* opCtx,
              const DatabaseName&,
              const BSONObj& cmdObj,
-             BSONObjBuilder& result) {
+             BSONObjBuilder& result) override {
+        // Critical to observability and diagnosability, annotate as immediate priority.
+        ScopedAdmissionPriority<ExecutionAdmissionContext> skipAdmissionControl(
+            opCtx, AdmissionContext::Priority::kExempt);
         result.append("id", repl::instanceId);
         return true;
     }
 };
 
-MONGO_INITIALIZER_WITH_PREREQUISITES(RegisterIsSelfCommand, ("GenerateInstanceId"))
-(InitializerContext* context) {
-    // Leaked intentionally: a Command registers itself when constructed
-    new IsSelfCommand();
-}
+MONGO_REGISTER_COMMAND(IsSelfCommand).forRouter().forShard();
 
 }  // namespace mongo
