@@ -27,23 +27,37 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <cstdint>
+#include <memory>
+#include <set>
+#include <string>
+#include <utility>
+#include <vector>
 
+#include <boost/cstdint.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/db/auth/action_type.h"
 #include "mongo/db/auth/authorization_session.h"
-#include "mongo/db/catalog/database_holder.h"
-#include "mongo/db/client.h"
+#include "mongo/db/auth/resource_pattern.h"
 #include "mongo/db/commands.h"
 #include "mongo/db/commands/list_databases_common.h"
 #include "mongo/db/commands/list_databases_gen.h"
+#include "mongo/db/concurrency/d_concurrency.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
 #include "mongo/db/curop_failpoint_helpers.h"
 #include "mongo/db/database_name.h"
 #include "mongo/db/matcher/expression.h"
-#include "mongo/db/multitenancy_gen.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/operation_context.h"
-#include "mongo/db/server_feature_flags_gen.h"
 #include "mongo/db/service_context.h"
 #include "mongo/db/storage/storage_engine.h"
+#include "mongo/rpc/op_msg.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/fail_point.h"
+#include "mongo/util/serialization_context.h"
 
 namespace mongo {
 
@@ -96,10 +110,12 @@ public:
             // {nameOnly: bool} - default false.
             const bool nameOnly = cmd.getNameOnly();
 
+            const auto& tenantId = cmd.getDbName().tenantId();
+
             // {authorizedDatabases: bool} - Dynamic default based on permissions.
-            const bool authorizedDatabases = ([as](const boost::optional<bool>& authDB) {
+            const bool authorizedDatabases = ([as, tenantId](const boost::optional<bool>& authDB) {
                 const bool mayListAllDatabases = as->isAuthorizedForActionsOnResource(
-                    ResourcePattern::forClusterResource(), ActionType::listDatabases);
+                    ResourcePattern::forClusterResource(tenantId), ActionType::listDatabases);
 
                 if (authDB) {
                     uassert(ErrorCodes::Unauthorized,
@@ -124,10 +140,16 @@ public:
 
                 CurOpFailpointHelpers::waitWhileFailPointEnabled(
                     &hangBeforeListDatabases, opCtx, "hangBeforeListDatabases", []() {});
-                dbNames =
-                    catalog->getAllConsistentDbNamesForTenant(opCtx, cmd.getDbName().tenantId());
+                dbNames = catalog->getAllConsistentDbNamesForTenant(opCtx, tenantId);
             }
             std::vector<ListDatabasesReplyItem> items;
+            SerializationContext scReply =
+                SerializationContext::stateCommandReply(cmd.getSerializationContext());
+            if (gMultitenancySupport && !tenantId) {
+                // During the multitenancy upgrade process a mongod might receive listDatabases from
+                // a non-multitenant node (with prefix and without token) during initial sync.
+                scReply.setPrefixState(true);
+            }
             int64_t totalSize =
                 list_databases::setReplyItems(opCtx,
                                               dbNames,
@@ -136,9 +158,11 @@ public:
                                               nameOnly,
                                               filter,
                                               false /* setTenantId */,
-                                              authorizedDatabases);
+                                              authorizedDatabases,
+                                              scReply);
 
-            ListDatabasesReply reply(items);
+            // We need to copy the serialization context from the request to the reply object
+            ListDatabasesReply reply(std::move(items), scReply);
             if (!nameOnly) {
                 reply.setTotalSize(totalSize);
                 reply.setTotalSizeMb(totalSize / (1024 * 1024));
@@ -147,6 +171,7 @@ public:
             return reply;
         }
     };
-} cmdListDatabases;
+};
+MONGO_REGISTER_COMMAND(CmdListDatabases).forShard();
 }  // namespace
 }  // namespace mongo
