@@ -28,36 +28,67 @@
  */
 
 
-#include "mongo/platform/basic.h"
-
+#include <cstdint>
+#include <fmt/format.h>
+#include <limits>
+#include <list>
 #include <memory>
+#include <set>
+#include <string>
 #include <utility>
+#include <vector>
 
-#include "mongo/bson/bsonobjbuilder.h"
+#include <boost/cstdint.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/client/read_preference.h"
+#include "mongo/db/api_parameters.h"
+#include "mongo/db/auth/action_type.h"
 #include "mongo/db/auth/authorization_session.h"
+#include "mongo/db/auth/privilege.h"
+#include "mongo/db/auth/resource_pattern.h"
+#include "mongo/db/basic_types_gen.h"
+#include "mongo/db/catalog/collection.h"
+#include "mongo/db/catalog/collection_catalog.h"
 #include "mongo/db/catalog/index_key_validate.h"
 #include "mongo/db/catalog/list_indexes.h"
 #include "mongo/db/clientcursor.h"
 #include "mongo/db/commands.h"
-#include "mongo/db/curop.h"
-#include "mongo/db/curop_failpoint_helpers.h"
 #include "mongo/db/cursor_manager.h"
 #include "mongo/db/db_raii.h"
+#include "mongo/db/exec/plan_stage.h"
 #include "mongo/db/exec/queued_data_stage.h"
 #include "mongo/db/exec/working_set.h"
-#include "mongo/db/index/index_descriptor.h"
 #include "mongo/db/list_indexes_gen.h"
-#include "mongo/db/query/cursor_request.h"
-#include "mongo/db/query/cursor_response.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/pipeline/expression_context.h"
+#include "mongo/db/query/collation/collator_interface.h"
 #include "mongo/db/query/find_common.h"
+#include "mongo/db/query/plan_executor.h"
 #include "mongo/db/query/plan_executor_factory.h"
+#include "mongo/db/query/plan_yield_policy.h"
+#include "mongo/db/record_id.h"
+#include "mongo/db/repl/read_concern_args.h"
 #include "mongo/db/service_context.h"
-#include "mongo/db/storage/durable_catalog.h"
-#include "mongo/db/storage/storage_engine.h"
+#include "mongo/db/storage/snapshot.h"
 #include "mongo/db/timeseries/catalog_helper.h"
 #include "mongo/db/timeseries/timeseries_index_schema_conversion_functions.h"
+#include "mongo/idl/idl_parser.h"
 #include "mongo/logv2/log.h"
-#include "mongo/util/uuid.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/rpc/op_msg.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/intrusive_counter.h"
+#include "mongo/util/serialization_context.h"
+#include "mongo/util/str.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
 
@@ -132,7 +163,7 @@ IndexSpecsWithNamespaceString getIndexSpecsWithNamespaceString(OperationContext*
 
             const CollectionPtr& coll = autoColl.getCollection();
             uassert(ErrorCodes::NamespaceNotFound,
-                    str::stream() << "ns does not exist: " << bucketsNss,
+                    str::stream() << "ns does not exist: " << bucketsNss.toStringForErrorMsg(),
                     coll);
 
             return std::make_pair(
@@ -147,8 +178,9 @@ IndexSpecsWithNamespaceString getIndexSpecsWithNamespaceString(OperationContext*
 
     const auto& nss = autoColl.getNss();
     const CollectionPtr& coll = autoColl.getCollection();
-    uassert(
-        ErrorCodes::NamespaceNotFound, str::stream() << "ns does not exist: " << nss.ns(), coll);
+    uassert(ErrorCodes::NamespaceNotFound,
+            str::stream() << "ns does not exist: " << nss.toStringForErrorMsg(),
+            coll);
 
     return std::make_pair(listIndexesInLock(opCtx, coll, nss, additionalInclude), nss);
 }
@@ -230,6 +262,10 @@ public:
             return false;
         }
 
+        bool isSubjectToIngressAdmissionControl() const override {
+            return true;
+        }
+
         NamespaceString ns() const final {
             auto nssOrUUID = request().getNamespaceOrUUID();
             if (nssOrUUID.isUUID()) {
@@ -252,7 +288,8 @@ public:
                 opCtx, cmd.getNamespaceOrUUID());
 
             uassert(ErrorCodes::Unauthorized,
-                    str::stream() << "Not authorized to list indexes on collection:" << nss.ns(),
+                    str::stream() << "Not authorized to list indexes on collection:"
+                                  << nss.toStringForErrorMsg(),
                     authzSession->isAuthorizedForActionsOnResource(
                         ResourcePattern::forExactNamespace(nss), ActionType::listIndexes));
         }
@@ -283,6 +320,9 @@ public:
                                            const NamespaceString& nss) {
             auto& cmd = request();
 
+            // We need to copy the serialization context from the request to the reply object
+            const auto serializationContext = cmd.getSerializationContext();
+
             long long batchSize = std::numeric_limits<long long>::max();
             if (cmd.getCursor() && cmd.getCursor()->getBatchSize()) {
                 batchSize = *cmd.getCursor()->getBatchSize();
@@ -309,7 +349,7 @@ public:
                                             std::move(ws),
                                             std::move(root),
                                             &CollectionPtr::null,
-                                            PlanYieldPolicy::YieldPolicy::NO_YIELD,
+                                            PlanYieldPolicy::YieldPolicy::INTERRUPT_ONLY,
                                             false, /* whether returned BSON must be owned */
                                             nss));
 
@@ -333,7 +373,13 @@ public:
 
                 try {
                     firstBatch.push_back(ListIndexesReplyItem::parse(
-                        IDLParserContext("ListIndexesReplyItem"), nextDoc));
+                        IDLParserContext(
+                            "ListIndexesReplyItem",
+                            false /* apiStrict */,
+                            auth::ValidatedTenancyScope::get(opCtx),
+                            nss.tenantId(),
+                            SerializationContext::stateCommandReply(serializationContext)),
+                        nextDoc));
                 } catch (const DBException& exc) {
                     LOGV2_ERROR(5254500,
                                 "Could not parse catalog entry while replying to listIndexes",
@@ -349,7 +395,11 @@ public:
             }
 
             if (exec->isEOF()) {
-                return ListIndexesReplyCursor(0 /* cursorId */, nss, std::move(firstBatch));
+                return ListIndexesReplyCursor(
+                    0 /* cursorId */,
+                    nss,
+                    std::move(firstBatch),
+                    SerializationContext::stateCommandReply(serializationContext));
             }
 
             exec->saveState();
@@ -372,10 +422,14 @@ public:
             pinnedCursor->incNReturnedSoFar(firstBatch.size());
 
             return ListIndexesReplyCursor(
-                pinnedCursor.getCursor()->cursorid(), nss, std::move(firstBatch));
+                pinnedCursor.getCursor()->cursorid(),
+                nss,
+                std::move(firstBatch),
+                SerializationContext::stateCommandReply(serializationContext));
         }
     };
-} cmdListIndexes;
+};
+MONGO_REGISTER_COMMAND(CmdListIndexes).forShard();
 
 }  // namespace
 }  // namespace mongo
