@@ -27,34 +27,70 @@
  *    it in the license file.
  */
 
+#include <cstdint>
+#include <fmt/format.h>
 #include <functional>
 #include <memory>
+#include <ostream>
 #include <string>
+#include <utility>
 #include <vector>
 
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/json.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/db/catalog/collection.h"
 #include "mongo/db/catalog/collection_catalog.h"
 #include "mongo/db/catalog/collection_options.h"
 #include "mongo/db/catalog/index_catalog.h"
 #include "mongo/db/catalog_raii.h"
 #include "mongo/db/client.h"
 #include "mongo/db/commands.h"
-#include "mongo/db/commands/map_reduce_gen.h"
+#include "mongo/db/commands/map_reduce_out_options.h"
 #include "mongo/db/commands/mr_common.h"
 #include "mongo/db/concurrency/exception_util.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
 #include "mongo/db/dbdirectclient.h"
-#include "mongo/db/json.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/op_observer/op_observer.h"
 #include "mongo/db/op_observer/op_observer_noop.h"
 #include "mongo/db/op_observer/op_observer_registry.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/repl/drop_pending_collection_reaper.h"
+#include "mongo/db/repl/member_state.h"
+#include "mongo/db/repl/oplog.h"
+#include "mongo/db/repl/optime.h"
+#include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/repl/replication_coordinator_mock.h"
+#include "mongo/db/repl/storage_interface.h"
 #include "mongo/db/repl/storage_interface_impl.h"
+#include "mongo/db/service_context.h"
 #include "mongo/db/service_context_d_test_fixture.h"
-#include "mongo/logv2/log.h"
+#include "mongo/db/storage/recovery_unit.h"
+#include "mongo/db/storage/write_unit_of_work.h"
+#include "mongo/db/transaction_resources.h"
 #include "mongo/rpc/factory.h"
 #include "mongo/rpc/get_status_from_command_result.h"
+#include "mongo/rpc/op_msg.h"
+#include "mongo/rpc/protocol.h"
 #include "mongo/scripting/dbdirectclient_factory.h"
 #include "mongo/scripting/engine.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/bson_test_util.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/duration.h"
+#include "mongo/util/str.h"
+#include "mongo/util/uuid.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
 
@@ -65,15 +101,16 @@ namespace {
  * Helper function to verify field of map_reduce_common::OutputOptions.
  */
 template <typename T>
-void _compareOutputOptionField(const std::string& dbname,
+void _compareOutputOptionField(const DatabaseName& dbName,
                                const std::string& cmdObjStr,
                                const std::string& fieldName,
                                const T& actual,
                                const T& expected) {
     if (actual == expected)
         return;
-    FAIL(str::stream() << "parseOutputOptions(\"" << dbname << ", " << cmdObjStr << "): "
-                       << fieldName << ": Expected: " << expected << ". Actual: " << actual);
+    FAIL(str::stream() << "parseOutputOptions(\"" << dbName.toStringForErrorMsg() << ", "
+                       << cmdObjStr << "): " << fieldName << ": Expected: " << expected
+                       << ". Actual: " << actual);
 }
 
 /**
@@ -96,7 +133,7 @@ std::string _getOutTypeString(OutputType outType) {
 /**
  * Test helper function to check expected result of parseOutputOptions.
  */
-void _testConfigParseOutputOptions(const std::string& dbname,
+void _testConfigParseOutputOptions(const DatabaseName& dbName,
                                    const std::string& cmdObjStr,
                                    const std::string& expectedOutDb,
                                    const std::string& expectedCollectionName,
@@ -105,18 +142,18 @@ void _testConfigParseOutputOptions(const std::string& dbname,
                                    OutputType expectedOutType) {
     const BSONObj cmdObj = fromjson(cmdObjStr);
     map_reduce_common::OutputOptions outputOptions =
-        map_reduce_common::parseOutputOptions(dbname, cmdObj);
-    _compareOutputOptionField(dbname, cmdObjStr, "outDb", outputOptions.outDB, expectedOutDb);
+        map_reduce_common::parseOutputOptions(dbName, cmdObj);
+    _compareOutputOptionField(dbName, cmdObjStr, "outDb", outputOptions.outDB, expectedOutDb);
     _compareOutputOptionField(
-        dbname, cmdObjStr, "collectionName", outputOptions.collectionName, expectedCollectionName);
-    _compareOutputOptionField(dbname,
+        dbName, cmdObjStr, "collectionName", outputOptions.collectionName, expectedCollectionName);
+    _compareOutputOptionField(dbName,
                               cmdObjStr,
                               "finalNamespace",
-                              outputOptions.finalNamespace.ns().toString(),
+                              outputOptions.finalNamespace.toString_forTest(),
                               expectedFinalNamespace);
     _compareOutputOptionField(
-        dbname, cmdObjStr, "outNonAtomic", outputOptions.outNonAtomic, expectedOutNonAtomic);
-    _compareOutputOptionField(dbname,
+        dbName, cmdObjStr, "outNonAtomic", outputOptions.outNonAtomic, expectedOutNonAtomic);
+    _compareOutputOptionField(dbName,
                               cmdObjStr,
                               "outType",
                               _getOutTypeString(outputOptions.outType),
@@ -127,33 +164,35 @@ void _testConfigParseOutputOptions(const std::string& dbname,
  * Tests for map_reduce_common::parseOutputOptions.
  */
 TEST(ConfigOutputOptionsTest, parseOutputOptions) {
+    DatabaseName dbName = DatabaseName::createDatabaseName_forTest(boost::none, "mydb");
+    DatabaseName dbName1 = DatabaseName::createDatabaseName_forTest(boost::none, "mydb1");
     // Missing 'out' field.
-    ASSERT_THROWS(map_reduce_common::parseOutputOptions("mydb", fromjson("{}")),
+    ASSERT_THROWS(map_reduce_common::parseOutputOptions(dbName, fromjson("{}")),
                   AssertionException);
     // 'out' must be either string or object.
-    ASSERT_THROWS(map_reduce_common::parseOutputOptions("mydb", fromjson("{out: 99}")),
+    ASSERT_THROWS(map_reduce_common::parseOutputOptions(dbName, fromjson("{out: 99}")),
                   AssertionException);
     // 'out.nonAtomic' false is not supported.
     ASSERT_THROWS(map_reduce_common::parseOutputOptions(
-                      "mydb", fromjson("{out: {normal: 'mycoll', nonAtomic: false}}")),
+                      dbName, fromjson("{out: {normal: 'mycoll', nonAtomic: false}}")),
                   AssertionException);
     ASSERT_THROWS(map_reduce_common::parseOutputOptions(
-                      "mydb", fromjson("{out: {replace: 'mycoll', nonAtomic: false}}")),
+                      dbName, fromjson("{out: {replace: 'mycoll', nonAtomic: false}}")),
                   AssertionException);
     ASSERT_THROWS(map_reduce_common::parseOutputOptions(
-                      "mydb", fromjson("{out: {inline: 'mycoll', nonAtomic: false}}")),
+                      dbName, fromjson("{out: {inline: 'mycoll', nonAtomic: false}}")),
                   AssertionException);
 
     // Unknown output specifer.
     ASSERT_THROWS(map_reduce_common::parseOutputOptions(
-                      "mydb", fromjson("{out: {no_such_out_type: 'mycoll'}}")),
+                      dbName, fromjson("{out: {no_such_out_type: 'mycoll'}}")),
                   AssertionException);
 
     // 'out' is string.
     _testConfigParseOutputOptions(
-        "mydb", "{out: 'mycoll'}", "", "mycoll", "mydb.mycoll", true, OutputType::Replace);
+        dbName, "{out: 'mycoll'}", "", "mycoll", "mydb.mycoll", true, OutputType::Replace);
     // 'out' is object.
-    _testConfigParseOutputOptions("mydb",
+    _testConfigParseOutputOptions(dbName,
                                   "{out: {normal: 'mycoll'}}",
                                   "",
                                   "mycoll",
@@ -161,7 +200,7 @@ TEST(ConfigOutputOptionsTest, parseOutputOptions) {
                                   true,
                                   OutputType::Replace);
     // 'out.db' overrides dbname parameter
-    _testConfigParseOutputOptions("mydb1",
+    _testConfigParseOutputOptions(dbName1,
                                   "{out: {replace: 'mycoll', db: 'mydb2'}}",
                                   "mydb2",
                                   "mycoll",
@@ -169,14 +208,14 @@ TEST(ConfigOutputOptionsTest, parseOutputOptions) {
                                   true,
                                   OutputType::Replace);
     // 'out.nonAtomic' is supported with merge and reduce.
-    _testConfigParseOutputOptions("mydb",
+    _testConfigParseOutputOptions(dbName,
                                   "{out: {merge: 'mycoll', nonAtomic: true}}",
                                   "",
                                   "mycoll",
                                   "mydb.mycoll",
                                   true,
                                   OutputType::Merge);
-    _testConfigParseOutputOptions("mydb",
+    _testConfigParseOutputOptions(dbName,
                                   "{out: {reduce: 'mycoll', nonAtomic: true}}",
                                   "",
                                   "mycoll",
@@ -184,7 +223,7 @@ TEST(ConfigOutputOptionsTest, parseOutputOptions) {
                                   true,
                                   OutputType::Reduce);
     // inline
-    _testConfigParseOutputOptions("mydb1",
+    _testConfigParseOutputOptions(dbName1,
                                   "{out: {inline: 'mycoll', db: 'mydb2'}}",
                                   "mydb2",
                                   "",
@@ -193,35 +232,35 @@ TEST(ConfigOutputOptionsTest, parseOutputOptions) {
                                   OutputType::InMemory);
 
     // Order should not matter in fields of 'out' object.
-    _testConfigParseOutputOptions("mydb1",
+    _testConfigParseOutputOptions(dbName1,
                                   "{out: {db: 'mydb2', normal: 'mycoll'}}",
                                   "mydb2",
                                   "mycoll",
                                   "mydb2.mycoll",
                                   true,
                                   OutputType::Replace);
-    _testConfigParseOutputOptions("mydb1",
+    _testConfigParseOutputOptions(dbName1,
                                   "{out: {db: 'mydb2', replace: 'mycoll'}}",
                                   "mydb2",
                                   "mycoll",
                                   "mydb2.mycoll",
                                   true,
                                   OutputType::Replace);
-    _testConfigParseOutputOptions("mydb1",
+    _testConfigParseOutputOptions(dbName1,
                                   "{out: {nonAtomic: true, merge: 'mycoll'}}",
                                   "",
                                   "mycoll",
                                   "mydb1.mycoll",
                                   true,
                                   OutputType::Merge);
-    _testConfigParseOutputOptions("mydb1",
+    _testConfigParseOutputOptions(dbName1,
                                   "{out: {nonAtomic: true, reduce: 'mycoll'}}",
                                   "",
                                   "mycoll",
                                   "mydb1.mycoll",
                                   true,
                                   OutputType::Reduce);
-    _testConfigParseOutputOptions("mydb1",
+    _testConfigParseOutputOptions(dbName1,
                                   "{out: {db: 'mydb2', inline: 'mycoll'}}",
                                   "mydb2",
                                   "",
@@ -264,8 +303,10 @@ public:
                    const CollectionPtr& coll,
                    std::vector<InsertStatement>::const_iterator begin,
                    std::vector<InsertStatement>::const_iterator end,
+                   const std::vector<RecordId>& recordIds,
                    std::vector<bool> fromMigrate,
-                   bool defaultFromMigrate) override;
+                   bool defaultFromMigrate,
+                   OpStateAccumulator* opAccumulator = nullptr) override;
 
     /**
      * Tracks the temporary collections mapReduces creates.
@@ -278,12 +319,12 @@ public:
                             const OplogSlot& createOpTime,
                             bool fromMigrate) override;
 
-    using OpObserver::onDropCollection;
     repl::OpTime onDropCollection(OperationContext* opCtx,
                                   const NamespaceString& collectionName,
                                   const UUID& uuid,
                                   std::uint64_t numRecords,
-                                  CollectionDropType dropType) override;
+                                  CollectionDropType dropType,
+                                  bool markFromMigrate) override;
 
     // Hook for onInserts. Defaults to a no-op function but may be overridden to inject exceptions
     // while mapReduce inserts its results into the temporary output collection.
@@ -322,8 +363,10 @@ void MapReduceOpObserver::onInserts(OperationContext* opCtx,
                                     const CollectionPtr& coll,
                                     std::vector<InsertStatement>::const_iterator begin,
                                     std::vector<InsertStatement>::const_iterator end,
+                                    const std::vector<RecordId>& recordIds,
                                     std::vector<bool> fromMigrate,
-                                    bool defaultFromMigrate) {
+                                    bool defaultFromMigrate,
+                                    OpStateAccumulator* opAccumulator) {
     onInsertsFn();
 }
 
@@ -344,7 +387,8 @@ repl::OpTime MapReduceOpObserver::onDropCollection(OperationContext* opCtx,
                                                    const NamespaceString& collectionName,
                                                    const UUID& uuid,
                                                    std::uint64_t numRecords,
-                                                   const CollectionDropType dropType) {
+                                                   const CollectionDropType dropType,
+                                                   bool markFromMigrate) {
     // If the oplog is not disabled for this namespace, then we need to reserve an op time for the
     // drop.
     if (!repl::ReplicationCoordinator::get(opCtx)->isOplogDisabledFor(opCtx, collectionName)) {
@@ -403,7 +447,7 @@ const NamespaceString MapReduceCommandTest::outputNss =
 
 void MapReduceCommandTest::setUp() {
     ServiceContextMongoDTest::setUp();
-    ScriptEngine::setup();
+    ScriptEngine::setup(ExecutionEnvironment::Server);
     auto service = getServiceContext();
     DBDirectClientFactory::get(service).registerImplementation(
         [](OperationContext* opCtx) { return std::make_unique<DBDirectClient>(opCtx); });
@@ -457,10 +501,12 @@ BSONObj MapReduceCommandTest::_makeCmdObj(StringData mapCode, StringData reduceC
 }
 
 Status MapReduceCommandTest::_runCommand(StringData mapCode, StringData reduceCode) {
-    auto command = CommandHelpers::findCommand("mapReduce");
+    auto command = CommandHelpers::findCommand(_opCtx.get(), "mapReduce");
     ASSERT(command) << "Unable to look up mapReduce command";
 
-    auto request = OpMsgRequest::fromDBAndBody(inputNss.db(), _makeCmdObj(mapCode, reduceCode));
+    auto request = OpMsgRequestBuilder::create(auth::ValidatedTenancyScope::get(_opCtx.get()),
+                                               inputNss.dbName(),
+                                               _makeCmdObj(mapCode, reduceCode));
     auto replyBuilder = rpc::makeReplyBuilder(rpc::Protocol::kOpMsg);
     auto result = CommandHelpers::runCommandDirectly(_opCtx.get(), request);
     auto status = getStatusFromCommandResult(result);
@@ -476,7 +522,7 @@ void MapReduceCommandTest::_assertTemporaryCollectionsAreDropped() {
     for (const auto& tempNss : _opObserver->tempNamespaces) {
         ASSERT_EQUALS(ErrorCodes::NamespaceNotFound,
                       _storage.getCollectionCount(_opCtx.get(), tempNss))
-            << "mapReduce did not remove temporary collection on success: " << tempNss.ns();
+            << "mapReduce did not remove temporary collection on success: " << tempNss.ns_forTest();
     }
 }
 
@@ -525,10 +571,11 @@ TEST_F(MapReduceCommandTest, PrimaryStepDownPreventsTemporaryCollectionDrops) {
 
     // Temporary collections should still be present because the server will not accept writes after
     // stepping down.
-    _opCtx->recoveryUnit()->setTimestampReadSource(RecoveryUnit::ReadSource::kLastApplied);
+    shard_role_details::getRecoveryUnit(_opCtx.get())
+        ->setTimestampReadSource(RecoveryUnit::ReadSource::kLastApplied);
     for (const auto& tempNss : _opObserver->tempNamespaces) {
         ASSERT_OK(_storage.getCollectionCount(_opCtx.get(), tempNss).getStatus())
-            << "missing mapReduce temporary collection: " << tempNss;
+            << "missing mapReduce temporary collection: " << tempNss.toStringForErrorMsg();
     }
 }
 
@@ -545,7 +592,7 @@ TEST_F(MapReduceCommandTest, ReplacingExistingOutputCollectionPreservesIndexes)
         AutoGetCollection coll(_opCtx.get(), outputNss, MODE_X);
         ASSERT(coll);
         writeConflictRetry(
-            _opCtx.get(), "ReplacingExistingOutputCollectionPreservesIndexes", outputNss.ns(), [&] {
+            _opCtx.get(), "ReplacingExistingOutputCollectionPreservesIndexes", outputNss, [&] {
                 WriteUnitOfWork wuow(_opCtx.get());
                 ASSERT_OK(
                     coll.getWritableCollection(_opCtx.get())
@@ -572,7 +619,7 @@ TEST_F(MapReduceCommandTest, ReplacingExistingOutputCollectionPreservesIndexes)
     ASSERT_NOT_EQUALS(
         *options.uuid,
         *CollectionCatalog::get(_opCtx.get())->lookupUUIDByNSS(_opCtx.get(), outputNss))
-        << "Output collection " << outputNss << " was not replaced";
+        << "Output collection " << outputNss.toStringForErrorMsg() << " was not replaced";
 
     _assertTemporaryCollectionsAreDropped();
 }
