@@ -27,23 +27,57 @@
  *    it in the license file.
  */
 
-#include <set>
+#include <compare>
+#include <map>
+#include <mutex>
+#include <string>
+#include <utility>
+#include <vector>
 
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/parse_number.h"
+#include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/bson/bsontypes.h"
 #include "mongo/bson/json.h"
 #include "mongo/bson/mutable/document.h"
-#include "mongo/client/replica_set_monitor.h"
-#include "mongo/config.h"
-#include "mongo/db/auth/authorization_manager.h"
+#include "mongo/bson/mutable/element.h"
+#include "mongo/config.h"  // IWYU pragma: keep
+#include "mongo/db/auth/action_type.h"
 #include "mongo/db/auth/authorization_session.h"
+#include "mongo/db/auth/resource_pattern.h"
+#include "mongo/db/cluster_role.h"
 #include "mongo/db/commands.h"
 #include "mongo/db/commands/parameters_gen.h"
 #include "mongo/db/commands/parse_log_component_settings.h"
+#include "mongo/db/database_name.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/server_options.h"
+#include "mongo/db/server_parameter.h"
 #include "mongo/db/server_parameter_gen.h"
+#include "mongo/db/service_context.h"
 #include "mongo/db/storage/kv/kv_engine.h"
-#include "mongo/db/storage/storage_options.h"
+#include "mongo/db/storage/storage_engine.h"
+#include "mongo/db/tenant_id.h"
 #include "mongo/idl/command_generic_argument.h"
+#include "mongo/idl/idl_parser.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/log_component_settings.h"
+#include "mongo/logv2/log_manager.h"
+#include "mongo/logv2/log_severity.h"
+#include "mongo/logv2/redaction.h"
+#include "mongo/platform/mutex.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/str.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
@@ -220,8 +254,8 @@ public:
                                  const DatabaseName& dbName,
                                  const BSONObj& cmdObj) const override {
         auto* as = AuthorizationSession::get(opCtx->getClient());
-        if (!as->isAuthorizedForActionsOnResource(ResourcePattern::forClusterResource(),
-                                                  ActionType::getParameter)) {
+        if (!as->isAuthorizedForActionsOnResource(
+                ResourcePattern::forClusterResource(dbName.tenantId()), ActionType::getParameter)) {
             return {ErrorCodes::Unauthorized, "unauthorized"};
         }
 
@@ -250,28 +284,42 @@ public:
         const auto options = parseGetParameterOptions(cmdObj.firstElement());
         const bool all = options.getAllParameters();
 
+        // If the "setAt" option has been set, then only include the parameter in the
+        // response if it matches the parameter's settability. If the "setAt" option has
+        // been omitted, then include all requested parameters.
+        boost::optional<SetAtOptionEnum> setAtOption = options.getSetAt();
+        bool isOptionRequestingRuntime =
+            setAtOption && (setAtOption.value() == SetAtOptionEnum::kRuntime);
+        bool isOptionRequestingStartup =
+            setAtOption && (setAtOption.value() == SetAtOptionEnum::kStartup);
+
         int before = result.len();
 
         const ServerParameter::Map& m = ServerParameterSet::getNodeParameterSet()->getMap();
         for (ServerParameter::Map::const_iterator i = m.begin(); i != m.end(); ++i) {
             if (i->second->isEnabled() && (all || cmdObj.hasElement(i->first.c_str()))) {
-                if (options.getShowDetails()) {
-                    BSONObjBuilder detailBob(result.subobjStart(i->second->name()));
-                    i->second->append(opCtx, &detailBob, "value", boost::none);
-                    detailBob.appendBool("settableAtRuntime",
-                                         i->second->allowedToChangeAtRuntime());
-                    detailBob.appendBool("settableAtStartup",
-                                         i->second->allowedToChangeAtStartup());
-                    detailBob.doneFast();
-                } else {
-                    i->second->append(opCtx, &result, i->second->name(), boost::none);
+                if (!setAtOption ||
+                    (isOptionRequestingRuntime && i->second->allowedToChangeAtRuntime()) ||
+                    (isOptionRequestingStartup && i->second->allowedToChangeAtStartup())) {
+                    if (options.getShowDetails()) {
+                        BSONObjBuilder detailBob(result.subobjStart(i->second->name()));
+                        i->second->append(opCtx, &detailBob, "value", boost::none);
+                        detailBob.appendBool("settableAtRuntime",
+                                             i->second->allowedToChangeAtRuntime());
+                        detailBob.appendBool("settableAtStartup",
+                                             i->second->allowedToChangeAtStartup());
+                        detailBob.doneFast();
+                    } else {
+                        i->second->append(opCtx, &result, i->second->name(), boost::none);
+                    }
                 }
             }
         }
         uassert(ErrorCodes::InvalidOptions, "no option found to get", before != result.len());
         return true;
     }
-} cmdGet;
+};
+MONGO_REGISTER_COMMAND(CmdGet).forRouter().forShard();
 
 class CmdSet : public BasicCommand {
 public:
@@ -292,8 +340,8 @@ public:
                                  const DatabaseName& dbName,
                                  const BSONObj& cmdObj) const override {
         auto* as = AuthorizationSession::get(opCtx->getClient());
-        if (!as->isAuthorizedForActionsOnResource(ResourcePattern::forClusterResource(),
-                                                  ActionType::setParameter)) {
+        if (!as->isAuthorizedForActionsOnResource(
+                ResourcePattern::forClusterResource(dbName.tenantId()), ActionType::setParameter)) {
             return {ErrorCodes::Unauthorized, "unauthorized"};
         }
 
@@ -338,15 +386,6 @@ public:
 
             ServerParameter::Map::const_iterator foundParameter = parameterMap.find(parameterName);
 
-            if (parameterName == "disableJavaScriptJIT") {
-                LOGV2_WARNING(
-                    10201700,
-                    "The Javascript Just In Time compiler is deprecated, and should be disabled "
-                    "for security reasons. The server parameter will be removed in a future major "
-                    "version. See https://www.mongodb.com/docs/manual/reference/parameters for "
-                    "more information.");
-            }
-
             // Check to see if this is actually a valid parameter
             uassert(ErrorCodes::InvalidOptions,
                     str::stream() << "attempted to set unrecognized parameter [" << parameterName
@@ -398,7 +437,8 @@ public:
                 str::stream()
                     << "Cannot set parameter requireApiVersion=true on a shard or config server",
                 parameterName != "requireApiVersion" || !parameter.trueValue() ||
-                    (serverGlobalParams.clusterRole.has(ClusterRole::None)));
+                    (!serverGlobalParams.clusterRole.has(ClusterRole::ShardServer) &&
+                     !serverGlobalParams.clusterRole.has(ClusterRole::ConfigServer)));
 
             auto oldValueObj = ([&] {
                 BSONObjBuilder bb;
@@ -417,7 +457,6 @@ public:
                 uassertStatusOK(foundParameter->second->set(parameter, boost::none));
             } catch (const DBException& ex) {
                 LOGV2(20496,
-                      "Error setting parameter {parameterName} to {newValue} errMsg: {error}",
                       "Error setting parameter to new value",
                       "parameterName"_attr = parameterName,
                       "newValue"_attr = redact(parameter.toString(false)),
@@ -436,15 +475,12 @@ public:
 
             if (oldValue) {
                 LOGV2(23435,
-                      "Successfully set parameter {parameterName} to {newValue} (was "
-                      "{oldValue})",
                       "Successfully set parameter to new value",
                       "parameterName"_attr = parameterName,
                       "newValue"_attr = redact(parameter.toString(false)),
                       "oldValue"_attr = redact(oldValue.toString(false)));
             } else {
                 LOGV2(23436,
-                      "Successfully set parameter {parameterName} to {newValue}",
                       "Successfully set parameter to new value",
                       "parameterName"_attr = parameterName,
                       "newValue"_attr = redact(parameter.toString(false)));
@@ -459,7 +495,8 @@ public:
 
         return true;
     }
-} cmdSet;
+};
+MONGO_REGISTER_COMMAND(CmdSet).forRouter().forShard();
 
 void LogLevelServerParameter::append(OperationContext*,
                                      BSONObjBuilder* builder,
