@@ -29,14 +29,18 @@
 
 #pragma once
 
+#include "mongo/base/status.h"
 #include "mongo/bson/bsonobj.h"
 #include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/db/auth/privilege.h"
+#include "mongo/db/catalog/external_data_source_scope_guard.h"
+#include "mongo/db/commands/server_status_metric.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/pipeline/aggregate_command_gen.h"
 #include "mongo/db/pipeline/lite_parsed_pipeline.h"
 #include "mongo/rpc/op_msg_rpc_impls.h"
+#include "mongo/rpc/reply_builder_interface.h"
 
 namespace mongo {
 
@@ -49,33 +53,20 @@ namespace mongo {
  * 'privileges' contains the privileges that were required to run this aggregation, to be used later
  * for re-checking privileges for getMore commands.
  *
- * If the query over a view that's already been resolved, the resolved view and the original
- * user-provided request both must be provided.
- *
  * On success, fills out 'result' with the command response.
  */
-Status runAggregate(OperationContext* opCtx,
-                    AggregateCommandRequest& request,
-                    const LiteParsedPipeline& liteParsedPipeline,
-                    const BSONObj& cmdObj,
-                    const PrivilegeVector& privileges,
-                    rpc::ReplyBuilderInterface* result,
-                    boost::optional<const ResolvedView&> resolvedView = boost::none,
-                    boost::optional<const AggregateCommandRequest&> origRequest = boost::none);
-
-/**
- * Convenience version that internally constructs the LiteParsedPipeline.
- */
-Status runAggregate(OperationContext* opCtx,
-                    AggregateCommandRequest& request,
-                    const BSONObj& cmdObj,
-                    const PrivilegeVector& privileges,
-                    rpc::ReplyBuilderInterface* result,
-                    boost::optional<const ResolvedView&> resolvedView = boost::none,
-                    boost::optional<const AggregateCommandRequest&> origRequest = boost::none);
+Status runAggregate(
+    OperationContext* opCtx,
+    AggregateCommandRequest& request,
+    const LiteParsedPipeline& liteParsedPipeline,
+    const BSONObj& cmdObj,
+    const PrivilegeVector& privileges,
+    rpc::ReplyBuilderInterface* result,
+    const std::vector<std::pair<NamespaceString, std::vector<ExternalDataSourceInfo>>>&
+        usedExternalDataSources = {});
 
 /**
  * Tracks explicit use of allowDiskUse:false with find and aggregate commands.
  */
-extern Counter64 allowDiskUseFalseCounter;
+extern Counter64& allowDiskUseFalseCounter;
 }  // namespace mongo
