@@ -28,12 +28,29 @@
  */
 
 #include "mongo/db/commands/set_profiling_filter_globally_cmd.h"
+
+#include <memory>
+
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/db/auth/action_type.h"
 #include "mongo/db/auth/authorization_session.h"
+#include "mongo/db/auth/resource_pattern.h"
 #include "mongo/db/catalog/collection_catalog.h"
+#include "mongo/db/commands/profile_common.h"
 #include "mongo/db/commands/profile_gen.h"
+#include "mongo/db/profile_filter.h"
 #include "mongo/db/profile_filter_impl.h"
+#include "mongo/db/query/query_knobs_gen.h"
+#include "mongo/idl/idl_parser.h"
 #include "mongo/logv2/log.h"
-#include "mongo/s/is_mongos.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/str.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
 
@@ -44,8 +61,8 @@ Status SetProfilingFilterGloballyCmd::checkAuthForOperation(OperationContext* op
                                                             const DatabaseName& dbName,
                                                             const BSONObj& cmdObj) const {
     AuthorizationSession* authSession = AuthorizationSession::get(opCtx->getClient());
-    return authSession->isAuthorizedForActionsOnResource(ResourcePattern::forAnyNormalResource(),
-                                                         ActionType::enableProfiler)
+    return authSession->isAuthorizedForActionsOnResource(
+               ResourcePattern::forAnyNormalResource(dbName.tenantId()), ActionType::enableProfiler)
         ? Status::OK()
         : Status(ErrorCodes::Unauthorized, "unauthorized");
 }
@@ -79,28 +96,14 @@ bool SetProfilingFilterGloballyCmd::run(OperationContext* opCtx,
     // explicitly stored for new databases.
     ProfileFilter::setDefault(newDefault);
 
+    // Writing to the CollectionCatalog requires holding the Global lock to avoid concurrent races
+    // with BatchedCollectionCatalogWriter.
+    Lock::GlobalLock lk{opCtx, MODE_IX};
 
-    if (!isMongos()) {
-        // Writing to the CollectionCatalog requires holding the Global lock to avoid concurrent
-        // races with BatchedCollectionCatalogWriter.
-        Lock::GlobalLock lk{opCtx, MODE_IX};
-
-        // Update all existing database settings.
-        CollectionCatalog::write(opCtx, [&](CollectionCatalog& catalog) {
-            catalog.setAllDatabaseProfileFilters(newDefault);
-        });
-    } else {
-        // Update all existing database settings.
-        //
-        // We use the ServiceContext overload since the OperationContext would crash the server
-        // due to not holding a Global Lock. As this is mongoS we know that there can't be a
-        // concurrent BatchedCollectionCatalogWriter since those are used only in mongoD. The
-        // Global lock is necessary to avoid conflicts with that class.
-        CollectionCatalog::write(opCtx->getServiceContext(), [&](CollectionCatalog& catalog) {
-            catalog.setAllDatabaseProfileFilters(newDefault);
-        });
-    }
-
+    // Update all existing database settings.
+    CollectionCatalog::write(opCtx, [&](CollectionCatalog& catalog) {
+        catalog.setAllDatabaseProfileFilters(newDefault);
+    });
 
     // Capture the old setting in the result object.
     if (oldDefault) {
@@ -112,10 +115,10 @@ bool SetProfilingFilterGloballyCmd::run(OperationContext* opCtx,
     // Log the change made to server's global profiling settings.
     LOGV2(72832,
           "Profiler settings changed globally",
-          "from"_attr = oldDefault ? BSON("filter" << redact(oldDefault->serialize()))
+          "from"_attr = oldDefault ? BSON("filter" << oldDefault->serialize())
                                    : BSON("filter"
                                           << "none"),
-          "to"_attr = newDefault ? BSON("filter" << redact(newDefault->serialize()))
+          "to"_attr = newDefault ? BSON("filter" << newDefault->serialize())
                                  : BSON("filter"
                                         << "none"));
     return true;
