@@ -27,16 +27,49 @@
  *    it in the license file.
  */
 
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <memory>
+#include <set>
+#include <string>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/db/auth/action_type.h"
 #include "mongo/db/auth/authorization_session.h"
+#include "mongo/db/auth/resource_pattern.h"
+#include "mongo/db/cluster_role.h"
 #include "mongo/db/commands.h"
-#include "mongo/db/commands/tenant_migration_donor_cmds_gen.h"
 #include "mongo/db/commands/tenant_migration_recipient_cmds_gen.h"
-#include "mongo/db/feature_compatibility_version_parser.h"
+#include "mongo/db/database_name.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/repl/optime.h"
 #include "mongo/db/repl/primary_only_service.h"
+#include "mongo/db/repl/repl_server_parameters_gen.h"
+#include "mongo/db/repl/repl_settings.h"
 #include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/repl/shard_merge_recipient_service.h"
 #include "mongo/db/repl/tenant_migration_recipient_service.h"
+#include "mongo/db/repl/tenant_migration_state_machine_gen.h"
+#include "mongo/db/repl/tenant_migration_util.h"
+#include "mongo/db/server_options.h"
+#include "mongo/db/serverless/serverless_types_gen.h"
+#include "mongo/db/service_context.h"
+#include "mongo/db/tenant_id.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/rpc/op_msg.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/fail_point.h"
+#include "mongo/util/future.h"
+#include "mongo/util/str.h"
+#include "mongo/util/uuid.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
 
@@ -52,10 +85,6 @@ public:
     using Request = RecipientSyncData;
     using Response = RecipientSyncDataResponse;
 
-    std::set<StringData> sensitiveFieldNames() const final {
-        return {Request::kRecipientCertificateForDonorFieldName};
-    }
-
     class Invocation : public InvocationBase {
 
     public:
@@ -66,6 +95,14 @@ public:
                     "tenant migrations are not available on config servers",
                     !serverGlobalParams.clusterRole.has(ClusterRole::ConfigServer));
 
+            uassert(ErrorCodes::IllegalOperation,
+                    "tenant migrations are only available if --serverless is enabled",
+                    repl::ReplicationCoordinator::get(opCtx)->getSettings().isServerless());
+
+            uassert(ErrorCodes::IllegalOperation,
+                    "Cannot run tenant migration with x509 authentication",
+                    repl::tenantMigrationDisableX509Auth);
+
             // (Generic FCV reference): This FCV reference should exist across LTS binary versions.
             uassert(
                 5356101,
@@ -113,15 +150,6 @@ public:
                                                       cmd.getTenantId()->toString(),
                                                       cmd.getStartMigrationDonorTimestamp(),
                                                       cmd.getReadPreference());
-
-            if (!repl::tenantMigrationDisableX509Auth) {
-                uassert(ErrorCodes::InvalidOptions,
-                        str::stream() << "'" << Request::kRecipientCertificateForDonorFieldName
-                                      << "' is a required field",
-                        cmd.getRecipientCertificateForDonor());
-                stateDoc.setRecipientCertificateForDonor(cmd.getRecipientCertificateForDonor());
-            }
-
             stateDoc.setProtocol(MigrationProtocolEnum::kMultitenantMigrations);
 
             auto recipientService =
@@ -147,14 +175,6 @@ public:
                                                  cmd.getStartMigrationDonorTimestamp(),
                                                  cmd.getReadPreference());
 
-            if (!repl::tenantMigrationDisableX509Auth) {
-                uassert(ErrorCodes::InvalidOptions,
-                        str::stream() << "'" << Request::kRecipientCertificateForDonorFieldName
-                                      << "' is a required field",
-                        cmd.getRecipientCertificateForDonor());
-                stateDoc.setRecipientCertificateForDonor(cmd.getRecipientCertificateForDonor());
-            }
-
             auto recipientService =
                 repl::PrimaryOnlyServiceRegistry::get(opCtx->getServiceContext())
                     ->lookupServiceByName(
@@ -174,19 +194,20 @@ public:
             uassert(ErrorCodes::Unauthorized,
                     "Unauthorized",
                     AuthorizationSession::get(opCtx->getClient())
-                        ->isAuthorizedForActionsOnResource(ResourcePattern::forClusterResource(),
-                                                           ActionType::runTenantMigration));
+                        ->isAuthorizedForActionsOnResource(
+                            ResourcePattern::forClusterResource(request().getDbName().tenantId()),
+                            ActionType::runTenantMigration));
         }
 
         bool supportsWriteConcern() const override {
             return false;
         }
-        NamespaceString ns() const {
+        NamespaceString ns() const override {
             return NamespaceString(request().getDbName());
         }
     };
 
-    std::string help() const {
+    std::string help() const override {
         return "Internal replica set command; instructs the recipient to sync data as part of a "
                "tenant migration.";
     }
@@ -198,9 +219,8 @@ public:
     BasicCommand::AllowedOnSecondary secondaryAllowed(ServiceContext*) const override {
         return BasicCommand::AllowedOnSecondary::kNever;
     }
-
-} recipientSyncDataCmd;
-
+};
+MONGO_REGISTER_COMMAND(RecipientSyncDataCmd).forShard();
 
 class RecipientVoteImportedFilesCommand final
     : public TypedCommand<RecipientVoteImportedFilesCommand> {
@@ -229,22 +249,26 @@ public:
             uassertStatusOK(
                 repl::ReplicationCoordinator::get(opCtx)->checkReplEnabledForCommand(&result));
 
+            uassert(ErrorCodes::IllegalOperation,
+                    "tenant migrations are only available if --serverless is enabled",
+                    repl::ReplicationCoordinator::get(opCtx)->getSettings().isServerless());
+
             const auto& cmd = request();
             LOGV2(6112805,
                   "Received RecipientVoteImportedFiles request",
                   "migrationId"_attr = cmd.getMigrationId(),
-                  "from"_attr = cmd.getFrom(),
-                  "success"_attr = cmd.getSuccess(),
-                  "reason"_attr = cmd.getReason());
-
+                  "from"_attr = cmd.getFrom());
             auto recipientService =
                 repl::PrimaryOnlyServiceRegistry::get(opCtx->getServiceContext())
                     ->lookupServiceByName(
                         repl::ShardMergeRecipientService::kShardMergeRecipientServiceName);
             auto [instance, _] = repl::ShardMergeRecipientService::Instance::lookup(
                 opCtx, recipientService, BSON("_id" << cmd.getMigrationId()));
-            uassert(8423340, "Unknown migrationId", instance);
-            (*instance)->onMemberImportedFiles(cmd.getFrom(), cmd.getSuccess(), cmd.getReason());
+            uassert(ErrorCodes::NoSuchTenantMigration,
+                    str::stream() << "Could not find tenant migration with id "
+                                  << cmd.getMigrationId(),
+                    instance);
+            (*instance)->onMemberImportedFiles(cmd.getFrom());
         }
 
     private:
@@ -260,11 +284,13 @@ public:
             uassert(ErrorCodes::Unauthorized,
                     "Unauthorized",
                     AuthorizationSession::get(opCtx->getClient())
-                        ->isAuthorizedForActionsOnResource(ResourcePattern::forClusterResource(),
-                                                           ActionType::internal));
+                        ->isAuthorizedForActionsOnResource(
+                            ResourcePattern::forClusterResource(request().getDbName().tenantId()),
+                            ActionType::internal));
         }
     };
-} recipientVoteImportedFilesCommand;
+};
+MONGO_REGISTER_COMMAND(RecipientVoteImportedFilesCommand).forShard();
 
 class RecipientForgetMigrationCmd : public TypedCommand<RecipientForgetMigrationCmd> {
 public:
@@ -276,10 +302,6 @@ public:
     // main chain).
     static inline const Timestamp kUnusedStartMigrationTimestamp{1, 1};
 
-    std::set<StringData> sensitiveFieldNames() const final {
-        return {Request::kRecipientCertificateForDonorFieldName};
-    }
-
     class Invocation : public InvocationBase {
 
     public:
@@ -290,6 +312,14 @@ public:
                     "tenant migrations are not available on config servers",
                     !serverGlobalParams.clusterRole.has(ClusterRole::ConfigServer));
 
+            uassert(ErrorCodes::IllegalOperation,
+                    "tenant migrations are only available if --serverless is enabled",
+                    repl::ReplicationCoordinator::get(opCtx)->getSettings().isServerless());
+
+            uassert(ErrorCodes::IllegalOperation,
+                    "Cannot run tenant migration with x509 authentication",
+                    repl::tenantMigrationDisableX509Auth);
+
             const auto& cmd = request();
             const auto migrationProtocol = cmd.getProtocol().value_or(kDefaultMigrationProtocol);
             const auto& tenantId = cmd.getTenantId();
@@ -331,14 +361,6 @@ public:
                                                       kUnusedStartMigrationTimestamp,
                                                       cmd.getReadPreference());
 
-            if (!repl::tenantMigrationDisableX509Auth) {
-                uassert(ErrorCodes::InvalidOptions,
-                        str::stream() << "'" << Request::kRecipientCertificateForDonorFieldName
-                                      << "' is a required field",
-                        cmd.getRecipientCertificateForDonor());
-                stateDoc.setRecipientCertificateForDonor(cmd.getRecipientCertificateForDonor());
-            }
-
             stateDoc.setProtocol(MigrationProtocolEnum::kMultitenantMigrations);
             // Set the state to 'kDone' so that we don't create a recipient access blocker
             // unnecessarily if this recipientForgetMigration command is received before a
@@ -366,14 +388,6 @@ public:
                                                  kUnusedStartMigrationTimestamp,
                                                  cmd.getReadPreference());
 
-            if (!repl::tenantMigrationDisableX509Auth) {
-                uassert(ErrorCodes::InvalidOptions,
-                        str::stream() << "'" << Request::kRecipientCertificateForDonorFieldName
-                                      << "' is a required field",
-                        cmd.getRecipientCertificateForDonor());
-                stateDoc.setRecipientCertificateForDonor(cmd.getRecipientCertificateForDonor());
-            }
-
             // Set 'startGarbageCollect' true to not start a migration (and install access blocker
             // or get serverless lock) unncessarily if this recipientForgetMigration command is
             // received before a recipientSyncData command or after the state doc is garbage
@@ -396,20 +410,21 @@ public:
             uassert(ErrorCodes::Unauthorized,
                     "Unauthorized",
                     AuthorizationSession::get(opCtx->getClient())
-                        ->isAuthorizedForActionsOnResource(ResourcePattern::forClusterResource(),
-                                                           ActionType::runTenantMigration));
+                        ->isAuthorizedForActionsOnResource(
+                            ResourcePattern::forClusterResource(request().getDbName().tenantId()),
+                            ActionType::runTenantMigration));
         }
 
         bool supportsWriteConcern() const override {
             return false;
         }
 
-        NamespaceString ns() const {
+        NamespaceString ns() const override {
             return NamespaceString(request().getDbName());
         }
     };
 
-    std::string help() const {
+    std::string help() const override {
         return "Interrupts tenant migration data sync and marks that the recipient's durable state "
                "machine may be garbage collected.";
     }
@@ -421,8 +436,8 @@ public:
     BasicCommand::AllowedOnSecondary secondaryAllowed(ServiceContext*) const override {
         return BasicCommand::AllowedOnSecondary::kNever;
     }
-
-} recipientForgetMigrationCmd;
+};
+MONGO_REGISTER_COMMAND(RecipientForgetMigrationCmd).forShard();
 
 }  // namespace
 }  // namespace mongo
