@@ -28,57 +28,85 @@
  */
 
 
-#include "mongo/base/checked_cast.h"
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <cstddef>
+#include <functional>
+#include <memory>
+#include <set>
+#include <string>
+#include <utility>
+#include <vector>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
 #include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/bson/mutable/document.h"
 #include "mongo/bson/mutable/element.h"
-#include "mongo/db/catalog/collection_catalog.h"
+#include "mongo/crypto/fle_field_schema_gen.h"
+#include "mongo/db/admission/execution_admission_context.h"
+#include "mongo/db/auth/authorization_session.h"
+#include "mongo/db/catalog/collection.h"
 #include "mongo/db/catalog/collection_operation_source.h"
-#include "mongo/db/catalog/database_holder.h"
-#include "mongo/db/catalog/document_validation.h"
-#include "mongo/db/client.h"
+#include "mongo/db/cluster_role.h"
 #include "mongo/db/commands.h"
 #include "mongo/db/commands/update_metrics.h"
 #include "mongo/db/commands/write_commands_common.h"
-#include "mongo/db/concurrency/exception_util.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
 #include "mongo/db/curop.h"
-#include "mongo/db/db_raii.h"
-#include "mongo/db/dbdirectclient.h"
 #include "mongo/db/fle_crud.h"
-#include "mongo/db/json.h"
-#include "mongo/db/matcher/doc_validation_error.h"
-#include "mongo/db/matcher/extensions_callback_real.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/not_primary_error_tracker.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/ops/delete_request_gen.h"
 #include "mongo/db/ops/parsed_delete.h"
 #include "mongo/db/ops/parsed_update.h"
+#include "mongo/db/ops/single_write_result_gen.h"
+#include "mongo/db/ops/update_request.h"
 #include "mongo/db/ops/write_ops.h"
 #include "mongo/db/ops/write_ops_exec.h"
+#include "mongo/db/ops/write_ops_gen.h"
+#include "mongo/db/ops/write_ops_parsers.h"
 #include "mongo/db/pipeline/aggregate_command_gen.h"
+#include "mongo/db/pipeline/legacy_runtime_constants_gen.h"
 #include "mongo/db/pipeline/lite_parsed_pipeline.h"
+#include "mongo/db/pipeline/process_interface/replica_set_node_process_interface.h"
+#include "mongo/db/pipeline/variables.h"
 #include "mongo/db/query/explain.h"
+#include "mongo/db/query/explain_options.h"
 #include "mongo/db/query/get_executor.h"
+#include "mongo/db/query/plan_yield_policy.h"
+#include "mongo/db/query/query_settings/query_settings_gen.h"
 #include "mongo/db/repl/repl_client_info.h"
 #include "mongo/db/repl/replication_coordinator.h"
-#include "mongo/db/repl/tenant_migration_conflict_info.h"
-#include "mongo/db/repl/tenant_migration_decoration.h"
-#include "mongo/db/s/collection_sharding_state.h"
-#include "mongo/db/s/operation_sharding_state.h"
-#include "mongo/db/stats/counters.h"
-#include "mongo/db/storage/duplicate_key_error_info.h"
-#include "mongo/db/storage/storage_parameters_gen.h"
+#include "mongo/db/server_options.h"
+#include "mongo/db/service_context.h"
+#include "mongo/db/shard_role.h"
 #include "mongo/db/timeseries/timeseries_index_schema_conversion_functions.h"
+#include "mongo/db/timeseries/timeseries_update_delete_util.h"
+#include "mongo/db/timeseries/timeseries_write_util.h"
 #include "mongo/db/transaction/retryable_writes_stats.h"
-#include "mongo/db/transaction/transaction_participant.h"
+#include "mongo/db/transaction_resources.h"
 #include "mongo/db/transaction_validation.h"
-#include "mongo/db/update/document_diff_applier.h"
-#include "mongo/db/write_concern.h"
-#include "mongo/logv2/log.h"
-#include "mongo/logv2/redaction.h"
-#include "mongo/s/stale_exception.h"
+#include "mongo/db/write_concern_options.h"
+#include "mongo/executor/task_executor_pool.h"
+#include "mongo/rpc/op_msg.h"
+#include "mongo/rpc/reply_builder_interface.h"
+#include "mongo/s/grid.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/concurrency/admission_context.h"
+#include "mongo/util/decorable.h"
 #include "mongo/util/fail_point.h"
-#include "mongo/util/string_map.h"
+#include "mongo/util/namespace_string_util.h"
+#include "mongo/util/safe_num.h"
+#include "mongo/util/serialization_context.h"
+#include "mongo/util/str.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
 
@@ -87,6 +115,7 @@ namespace mongo {
 namespace {
 
 MONGO_FAIL_POINT_DEFINE(hangInsertBeforeWrite);
+MONGO_FAIL_POINT_DEFINE(hangUpdateBeforeWrite);
 
 void redactTooLongLog(mutablebson::Document* cmdObj, StringData fieldName) {
     namespace mmb = mutablebson;
@@ -111,47 +140,6 @@ bool shouldSkipOutput(OperationContext* opCtx) {
          writeConcern.syncMode == WriteConcernOptions::SyncMode::UNSET);
 }
 
-/**
- * Returns true if 'ns' is a time-series collection. That is, this namespace is backed by a
- * time-series buckets collection.
- */
-template <class Request>
-bool isTimeseries(OperationContext* opCtx, const Request& request) {
-    uassert(5916400,
-            "'isTimeseriesNamespace' parameter can only be set when the request is sent on "
-            "system.buckets namespace",
-            !request.getIsTimeseriesNamespace() ||
-                request.getNamespace().isTimeseriesBucketsCollection());
-    const auto bucketNss = request.getIsTimeseriesNamespace()
-        ? request.getNamespace()
-        : request.getNamespace().makeTimeseriesBucketsNamespace();
-
-    // If the buckets collection exists now, the time-series insert path will check for the
-    // existence of the buckets collection later on with a lock.
-    // If this check is concurrent with the creation of a time-series collection and the buckets
-    // collection does not yet exist, this check may return false unnecessarily. As a result, an
-    // insert attempt into the time-series namespace will either succeed or fail, depending on who
-    // wins the race.
-    // Hold reference to the catalog for collection lookup without locks to be safe.
-    auto catalog = CollectionCatalog::get(opCtx);
-    auto coll = catalog->lookupCollectionByNamespace(opCtx, bucketNss);
-    if (!coll) {
-        return false;
-    }
-
-    if (auto options = coll->getTimeseriesOptions()) {
-        uassert(ErrorCodes::InvalidOptions,
-                "Time-series buckets collection is not clustered",
-                coll->isClustered());
-
-        uassertStatusOK(timeseries::validateBucketingParameters(*options));
-
-        return true;
-    }
-
-    return false;
-}
-
 /**
  * Contains hooks that are used by 'populateReply' method.
  */
@@ -187,7 +175,8 @@ void populateReply(OperationContext* opCtx,
         if (lastResult == ErrorCodes::StaleDbVersion ||
             lastResult == ErrorCodes::ShardCannotRefreshDueToLocksHeld ||
             ErrorCodes::isStaleShardVersionError(lastResult.getStatus()) ||
-            ErrorCodes::isTenantMigrationError(lastResult.getStatus())) {
+            ErrorCodes::isTenantMigrationError(lastResult.getStatus()) ||
+            lastResult == ErrorCodes::CannotImplicitlyCreateCollection) {
             // For ordered:false commands we need to duplicate these error results for all ops
             // after we stopped. See handleError() in write_ops_exec.cpp for more info.
             //
@@ -229,13 +218,9 @@ void populateReply(OperationContext* opCtx,
     {
         // Undocumented repl fields that mongos depends on.
         auto* replCoord = repl::ReplicationCoordinator::get(opCtx->getServiceContext());
-        const auto replMode = replCoord->getReplicationMode();
-        if (replMode != repl::ReplicationCoordinator::modeNone) {
+        if (replCoord->getSettings().isReplSet()) {
             replyBase.setOpTime(repl::ReplClientInfo::forClient(opCtx->getClient()).getLastOp());
-
-            if (replMode == repl::ReplicationCoordinator::modeReplSet) {
-                replyBase.setElectionId(replCoord->getElectionId());
-            }
+            replyBase.setElectionId(replCoord->getElectionId());
         }
     }
 
@@ -280,6 +265,7 @@ public:
     bool allowedInTransactions() const final {
         return true;
     }
+
     class Invocation final : public InvocationBaseGen {
     public:
         Invocation(OperationContext* opCtx,
@@ -293,6 +279,10 @@ public:
             return true;
         }
 
+        bool isSubjectToIngressAdmissionControl() const override {
+            return true;
+        }
+
         NamespaceString ns() const final {
             return request().getNamespace();
         }
@@ -320,26 +310,29 @@ public:
                 }
             }
 
-            if (isTimeseries(opCtx, request())) {
+            if (auto [isTimeseriesViewRequest, _] =
+                    timeseries::isTimeseriesViewRequest(opCtx, request());
+                isTimeseriesViewRequest) {
                 // Re-throw parsing exceptions to be consistent with CmdInsert::Invocation's
                 // constructor.
                 try {
                     return write_ops_exec::performTimeseriesWrites(opCtx, request());
                 } catch (DBException& ex) {
-                    ex.addContext(str::stream() << "time-series insert failed: " << ns().ns());
+                    ex.addContext(str::stream()
+                                  << "time-series insert failed: " << ns().toStringForErrorMsg());
                     throw;
                 }
             }
 
-            boost::optional<ScopedAdmissionPriorityForLock> priority;
+            boost::optional<ScopedAdmissionPriority<ExecutionAdmissionContext>> priority;
             if (request().getNamespace() == NamespaceString::kConfigSampledQueriesNamespace ||
                 request().getNamespace() == NamespaceString::kConfigSampledQueriesDiffNamespace) {
-                priority.emplace(opCtx->lockState(), AdmissionContext::Priority::kLow);
+                priority.emplace(opCtx, AdmissionContext::Priority::kLow);
             }
 
             if (hangInsertBeforeWrite.shouldFail([&](const BSONObj& data) {
-                    const auto ns = data.getStringField("ns");
-                    return ns == request().getNamespace().toString();
+                    const auto fpNss = NamespaceStringUtil::parseFailPointData(data, "ns"_sd);
+                    return fpNss == request().getNamespace();
                 })) {
                 hangInsertBeforeWrite.pauseWhileSet();
             }
@@ -369,7 +362,8 @@ public:
             throw;
         }
     };
-} cmdInsert;
+};
+MONGO_REGISTER_COMMAND(CmdInsert).forShard();
 
 class CmdUpdate final : public write_ops::UpdateCmdVersion1Gen<CmdUpdate> {
 public:
@@ -434,6 +428,10 @@ public:
             return true;
         }
 
+        bool isSubjectToIngressAdmissionControl() const override {
+            return true;
+        }
+
         NamespaceString ns() const final {
             return request().getNamespace();
         }
@@ -448,7 +446,7 @@ public:
 
         void appendMirrorableRequest(BSONObjBuilder* bob) const override {
             auto extractQueryDetails = [](const BSONObj& update, BSONObjBuilder* bob) -> void {
-                // "filter", "hint", and "collation" fields are optional.
+                // "filter", "sort", "hint", and "collation" fields are optional.
                 if (update.isEmpty())
                     return;
 
@@ -457,6 +455,8 @@ public:
 
                 if (update.hasField("q"))
                     bob->append("filter", update["q"].Obj());
+                if (update.hasField("sort") && !update["sort"].Obj().isEmpty())
+                    bob->append("sort", update["sort"].Obj());
                 if (update.hasField("hint") && !update["hint"].Obj().isEmpty())
                     bob->append("hint", update["hint"].Obj());
                 if (update.hasField("collation") && !update["collation"].Obj().isEmpty())
@@ -465,6 +465,7 @@ public:
 
             invariant(!_commandObj.isEmpty());
 
+
             bob->append("find", _commandObj["update"].String());
             extractQueryDetails(_updateOpObj, bob);
             bob->append("batchSize", 1);
@@ -474,12 +475,10 @@ public:
                 !shardVersion.eoo()) {
                 bob->append(shardVersion);
             }
-
             if (const auto& databaseVersion = _commandObj.getField("databaseVersion");
                 !databaseVersion.eoo()) {
                 bob->append(databaseVersion);
             }
-
             if (const auto& encryptionInfo = _commandObj.getField("encryptionInformation");
                 !encryptionInfo.eoo()) {
                 bob->append(encryptionInfo);
@@ -494,7 +493,6 @@ public:
 
             doTransactionValidationForWrites(opCtx, ns());
             write_ops::UpdateCommandReply updateReply;
-            OperationSource source = OperationSource::kStandard;
             if (request().getEncryptionInformation().has_value()) {
                 {
                     // Flag set here and in fle_crud.cpp since this only executes on a mongod.
@@ -506,14 +504,10 @@ public:
                 }
             }
 
-            if (isTimeseries(opCtx, request())) {
-                uassert(ErrorCodes::OperationNotSupportedInTransaction,
-                        str::stream() << "Cannot perform a multi-document transaction on a "
-                                         "time-series collection: "
-                                      << ns(),
-                        !opCtx->inMultiDocumentTransaction());
-                source = OperationSource::kTimeseriesUpdate;
-            }
+            auto [isTimeseriesViewRequest, bucketNs] =
+                timeseries::isTimeseriesViewRequest(opCtx, request());
+            OperationSource source = isTimeseriesViewRequest ? OperationSource::kTimeseriesUpdate
+                                                             : OperationSource::kStandard;
 
             long long nModified = 0;
 
@@ -521,7 +515,38 @@ public:
             // 'postProcessHandler' and should not be accessed afterwards.
             std::vector<write_ops::Upserted> upsertedInfoVec;
 
-            auto reply = write_ops_exec::performUpdates(opCtx, request(), source);
+            write_ops_exec::WriteResult reply;
+            // For retryable updates on time-series collections, we needs to run them in
+            // transactions to ensure the multiple writes are replicated atomically.
+            bool isTimeseriesRetryableUpdate = isTimeseriesViewRequest &&
+                opCtx->isRetryableWrite() && !opCtx->inMultiDocumentTransaction();
+            if (isTimeseriesRetryableUpdate) {
+                auto executor = serverGlobalParams.clusterRole.has(ClusterRole::None)
+                    ? ReplicaSetNodeProcessInterface::getReplicaSetNodeExecutor(
+                          opCtx->getServiceContext())
+                    : Grid::get(opCtx)->getExecutorPool()->getFixedExecutor();
+                ON_BLOCK_EXIT([&] {
+                    // Increments the counter if the command contains retries. This is normally done
+                    // within write_ops_exec::performUpdates. But for retryable timeseries updates,
+                    // we should handle the metrics only once at the caller since each statement
+                    // will be run as a separate update command through the internal transaction
+                    // API. See write_ops_exec::performUpdates for more details.
+                    if (!reply.retriedStmtIds.empty()) {
+                        RetryableWritesStats::get(opCtx)->incrementRetriedCommandsCount();
+                    }
+                });
+                write_ops_exec::runTimeseriesRetryableUpdates(
+                    opCtx, bucketNs, request(), executor, &reply);
+            } else {
+                if (hangUpdateBeforeWrite.shouldFail([&](const BSONObj& data) {
+                        const auto fpNss = NamespaceStringUtil::parseFailPointData(data, "ns"_sd);
+                        return fpNss == request().getNamespace();
+                    })) {
+                    hangUpdateBeforeWrite.pauseWhileSet();
+                }
+
+                reply = write_ops_exec::performUpdates(opCtx, request(), source);
+            }
 
             // Handler to process each 'SingleWriteResult'.
             auto singleWriteHandler = [&](const SingleWriteResult& opResult, int index) {
@@ -547,21 +572,14 @@ public:
                           PopulateReplyHooks{singleWriteHandler, postProcessHandler});
 
             // Collect metrics.
-            for (auto&& update : request().getUpdates()) {
-                // If this was a pipeline style update, record that pipeline-style was used and
-                // which stages were being used.
-                auto& updateMod = update.getU();
-                if (updateMod.type() == write_ops::UpdateModification::Type::kPipeline) {
-                    AggregateCommandRequest aggCmd(request().getNamespace(),
-                                                   updateMod.getUpdatePipeline());
-                    LiteParsedPipeline pipeline(aggCmd);
-                    pipeline.tickGlobalStageCounters();
-                    CmdUpdate::updateMetrics.incrementExecutedWithAggregationPipeline();
-                }
-
-                // If this command had arrayFilters option, record that it was used.
-                if (update.getArrayFilters()) {
-                    CmdUpdate::updateMetrics.incrementExecutedWithArrayFilters();
+            // For time-series retryable updates, the metrics are already incremented when running
+            // the internal transaction. Avoids updating them twice.
+            if (!isTimeseriesRetryableUpdate) {
+                for (auto&& update : request().getUpdates()) {
+                    incrementUpdateMetrics(update.getU(),
+                                           request().getNamespace(),
+                                           _getUpdateMetrics(),
+                                           update.getArrayFilters());
                 }
             }
 
@@ -572,6 +590,10 @@ public:
         }
 
     private:
+        UpdateMetrics& _getUpdateMetrics() const {
+            return *static_cast<const CmdUpdate&>(*definition())._updateMetrics;
+        }
+
         void doCheckAuthorization(OperationContext* opCtx) const final try {
             auth::checkAuthForUpdateCommand(AuthorizationSession::get(opCtx->getClient()),
                                             request().getBypassDocumentValidation(),
@@ -592,8 +614,11 @@ public:
                     "explained write batches must be of size 1",
                     request().getUpdates().size() == 1);
 
+            auto [isTimeseriesViewRequest, nss] =
+                timeseries::isTimeseriesViewRequest(opCtx, request());
+
             UpdateRequest updateRequest(request().getUpdates()[0]);
-            updateRequest.setNamespaceString(request().getNamespace());
+            updateRequest.setNamespaceString(nss);
             if (shouldDoFLERewrite(request())) {
                 {
                     stdx::lock_guard<Client> lk(*opCtx->getClient());
@@ -616,28 +641,13 @@ public:
             updateRequest.setYieldPolicy(PlanYieldPolicy::YieldPolicy::YIELD_AUTO);
             updateRequest.setExplain(verbosity);
 
-            const ExtensionsCallbackReal extensionsCallback(opCtx,
-                                                            &updateRequest.getNamespaceString());
-            ParsedUpdate parsedUpdate(opCtx, &updateRequest, extensionsCallback);
-            uassertStatusOK(parsedUpdate.parseRequest());
-
-            // Explains of write commands are read-only, but we take write locks so that timing
-            // info is more accurate.
-            AutoGetCollection collection(opCtx, request().getNamespace(), MODE_IX);
-
-            CurOp::get(opCtx)->beginQueryPlanningTimer();
-
-            auto exec = uassertStatusOK(getExecutorUpdate(&CurOp::get(opCtx)->debug(),
-                                                          &collection.getCollection(),
-                                                          &parsedUpdate,
-                                                          verbosity));
-            auto bodyBuilder = result->getBodyBuilder();
-            Explain::explainStages(exec.get(),
-                                   collection.getCollection(),
-                                   verbosity,
-                                   BSONObj(),
-                                   _commandObj,
-                                   &bodyBuilder);
+            write_ops_exec::explainUpdate(opCtx,
+                                          updateRequest,
+                                          isTimeseriesViewRequest,
+                                          request().getSerializationContext(),
+                                          _commandObj,
+                                          verbosity,
+                                          result);
         }
 
         BSONObj _commandObj;
@@ -646,11 +656,16 @@ public:
         BSONObj _updateOpObj;
     };
 
-    // Update related command execution metrics.
-    static UpdateMetrics updateMetrics;
-} cmdUpdate;
+protected:
+    void doInitializeClusterRole(ClusterRole role) override {
+        write_ops::UpdateCmdVersion1Gen<CmdUpdate>::doInitializeClusterRole(role);
+        _updateMetrics.emplace(getName(), role);
+    }
 
-UpdateMetrics CmdUpdate::updateMetrics{"update"};
+    // Update related command execution metrics.
+    mutable boost::optional<UpdateMetrics> _updateMetrics;
+};
+MONGO_REGISTER_COMMAND(CmdUpdate).forShard();
 
 class CmdDelete final : public write_ops::DeleteCmdVersion1Gen<CmdDelete> {
 public:
@@ -703,6 +718,10 @@ public:
             return true;
         }
 
+        bool isSubjectToIngressAdmissionControl() const override {
+            return true;
+        }
+
         NamespaceString ns() const final {
             return request().getNamespace();
         }
@@ -729,7 +748,9 @@ public:
                 }
             }
 
-            if (isTimeseries(opCtx, request())) {
+            if (auto [isTimeseriesViewRequest, _] =
+                    timeseries::isTimeseriesViewRequest(opCtx, request());
+                isTimeseriesViewRequest) {
                 source = OperationSource::kTimeseriesDelete;
             }
 
@@ -768,16 +789,10 @@ public:
                     "explained write batches must be of size 1",
                     request().getDeletes().size() == 1);
 
+            auto [isTimeseriesViewRequest, nss] =
+                timeseries::isTimeseriesViewRequest(opCtx, request());
+
             auto deleteRequest = DeleteRequest{};
-            auto isRequestToTimeseries = isTimeseries(opCtx, request());
-            auto nss = [&] {
-                auto nss = request().getNamespace();
-                if (!isRequestToTimeseries) {
-                    return nss;
-                }
-                return nss.isTimeseriesBucketsCollection() ? nss
-                                                           : nss.makeTimeseriesBucketsNamespace();
-            }();
             deleteRequest.setNsString(nss);
             deleteRequest.setLegacyRuntimeConstants(request().getLegacyRuntimeConstants().value_or(
                 Variables::generateRuntimeConstants(opCtx)));
@@ -804,52 +819,19 @@ public:
             deleteRequest.setHint(firstDelete.getHint());
             deleteRequest.setIsExplain(true);
 
-            // Explains of write commands are read-only, but we take write locks so that timing
-            // info is more accurate.
-            AutoGetCollection collection(opCtx, deleteRequest.getNsString(), MODE_IX);
-
-            if (isRequestToTimeseries) {
-                uassert(ErrorCodes::NamespaceNotFound,
-                        "Could not find time-series buckets collection for write explain",
-                        *collection);
-                auto timeseriesOptions = collection->getTimeseriesOptions();
-                uassert(ErrorCodes::InvalidOptions,
-                        "Time-series buckets collection is missing time-series options",
-                        timeseriesOptions);
-
-                if (timeseries::isHintIndexKey(firstDelete.getHint())) {
-                    deleteRequest.setHint(
-                        uassertStatusOK(timeseries::createBucketsIndexSpecFromTimeseriesIndexSpec(
-                            *timeseriesOptions, firstDelete.getHint())));
-                }
-            }
-
-            ParsedDelete parsedDelete(opCtx,
-                                      &deleteRequest,
-                                      isRequestToTimeseries && collection
-                                          ? collection->getTimeseriesOptions()
-                                          : boost::none);
-            uassertStatusOK(parsedDelete.parseRequest());
-
-            CurOp::get(opCtx)->beginQueryPlanningTimer();
-
-            // Explain the plan tree.
-            auto exec = uassertStatusOK(getExecutorDelete(&CurOp::get(opCtx)->debug(),
-                                                          &collection.getCollection(),
-                                                          &parsedDelete,
-                                                          verbosity));
-            auto bodyBuilder = result->getBodyBuilder();
-            Explain::explainStages(exec.get(),
-                                   collection.getCollection(),
-                                   verbosity,
-                                   BSONObj(),
-                                   _commandObj,
-                                   &bodyBuilder);
+            write_ops_exec::explainDelete(opCtx,
+                                          deleteRequest,
+                                          isTimeseriesViewRequest,
+                                          request().getSerializationContext(),
+                                          _commandObj,
+                                          verbosity,
+                                          result);
         }
 
         const BSONObj& _commandObj;
     };
-} cmdDelete;
+};
+MONGO_REGISTER_COMMAND(CmdDelete).forShard();
 
 }  // namespace
 }  // namespace mongo
