@@ -27,32 +27,33 @@
  *    it in the license file.
  */
 
-
-#include "mongo/platform/basic.h"
-
 #include "mongo/db/concurrency/lock_manager.h"
 
+#include <cstdint>
+#include <cstring>
 #include <fmt/format.h>
 #include <fmt/ostream.h>
+#include <string>
+#include <utility>
+#include <vector>
+
+#include <absl/container/node_hash_map.h>
+#include <absl/meta/type_traits.h>
 
-#include "mongo/base/data_type_endian.h"
-#include "mongo/base/data_view.h"
 #include "mongo/base/static_assert.h"
+#include "mongo/base/string_data.h"
 #include "mongo/bson/bsonobjbuilder.h"
-#include "mongo/config.h"
-#include "mongo/db/concurrency/d_concurrency.h"
+#include "mongo/db/concurrency/lock_request_list.h"
 #include "mongo/db/concurrency/locker.h"
-#include "mongo/db/concurrency/resource_catalog.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/service_context.h"
 #include "mongo/logv2/log.h"
+#include "mongo/stdx/thread.h"
 #include "mongo/util/assert_util.h"
 #include "mongo/util/decorable.h"
-#include "mongo/util/str.h"
-#include "mongo/util/timer.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
 
-
 namespace mongo {
 namespace {
 
@@ -126,7 +127,6 @@ static const char* LockRequestStatusNames[] = {
     "new",
     "granted",
     "waiting",
-    "converting",
 };
 
 // Ensure we do not add new status types without updating the names array
@@ -134,6 +134,7 @@ MONGO_STATIC_ASSERT((sizeof(LockRequestStatusNames) / sizeof(LockRequestStatusNa
                     LockRequest::StatusCount);
 
 auto getLockManager = ServiceContext::declareDecoration<LockManager>();
+
 }  // namespace
 
 /**
@@ -162,7 +163,6 @@ struct LockHead {
         memset(conflictCounts, 0, sizeof(conflictCounts));
         conflictModes = 0;
 
-        conversionsCount = 0;
         compatibleFirstCount = 0;
     }
 
@@ -287,15 +287,15 @@ struct LockHead {
     //
 
     // Doubly-linked list of requests, which have been granted. Newly granted requests go to
-    // the end of the queue. Conversion requests are granted from the beginning forward.
+    // the end of the queue.
     LockRequestList grantedList;
 
-    // Counts the grants and conversion counts for each of the supported lock modes. These
-    // counts should exactly match the aggregated modes on the granted list.
+    // Counts the grants for each of the supported lock modes. These counts should exactly match the
+    // aggregated modes on the granted list.
     uint32_t grantedCounts[LockModesCount];
 
-    // Bit-mask of the granted + converting modes on the granted queue. Maintained in lock-step
-    // with the grantedCounts array.
+    // Bit-mask of the granted modes on the granted queue. Maintained in lock-step with the
+    // grantedCounts array.
     uint32_t grantedModes;
 
     //
@@ -321,17 +321,6 @@ struct LockHead {
     // TODO: Remove this vector and make LockHead a POD
     std::vector<LockManager::Partition*> partitions;
 
-    //
-    // Conversion
-    //
-
-    // Counts the number of requests on the granted queue, which have requested any kind of
-    // conflicting conversion and are blocked (i.e. all requests which are currently
-    // STATUS_CONVERTING). This is an optimization for unlocking in that we do not need to
-    // check the granted queue for requests in STATUS_CONVERTING if this count is zero. This
-    // saves cycles in the regular case and only burdens the less-frequent lock upgrade case.
-    uint32_t conversionsCount;
-
     // Counts the number of requests on the granted queue, which have requested that the policy
     // be switched to compatible-first. As long as this value is > 0, the policy will stay
     // compatible-first.
@@ -415,13 +404,6 @@ void LockHead::migratePartitionedLockHeads() {
 // LockManager
 //
 
-// Have more buckets than CPUs to reduce contention on lock and caches
-const unsigned LockManager::_numLockBuckets(128);
-
-// Balance scalability of intent locks against potential added cost of conflicting locks.
-// The exact value doesn't appear very important, but should be power of two
-const unsigned LockManager::_numPartitions = 32;
-
 // static
 LockManager* LockManager::get(ServiceContext* service) {
     return &getLockManager(service);
@@ -432,38 +414,6 @@ LockManager* LockManager::get(ServiceContext& service) {
     return &getLockManager(service);
 }
 
-// static
-LockManager* LockManager::get(OperationContext* opCtx) {
-    return get(opCtx->getServiceContext());
-}
-
-
-// static
-std::map<LockerId, BSONObj> LockManager::getLockToClientMap(ServiceContext* serviceContext) {
-    std::map<LockerId, BSONObj> lockToClientMap;
-
-    for (ServiceContext::LockedClientsCursor cursor(serviceContext);
-         Client* client = cursor.next();) {
-        invariant(client);
-
-        stdx::lock_guard<Client> lk(*client);
-        const OperationContext* clientOpCtx = client->getOperationContext();
-
-        // Operation context specific information
-        if (clientOpCtx) {
-            BSONObjBuilder infoBuilder;
-            // The client information
-            client->reportState(infoBuilder);
-
-            infoBuilder.append("opid", static_cast<int>(clientOpCtx->getOpID()));
-            LockerId lockerId = clientOpCtx->lockState()->getId();
-            lockToClientMap.insert({lockerId, infoBuilder.obj()});
-        }
-    }
-
-    return lockToClientMap;
-}
-
 LockManager::LockManager() {
     _lockBuckets = new LockBucket[_numLockBuckets];
     _partitions = new Partition[_numPartitions];
@@ -473,7 +423,6 @@ LockManager::~LockManager() {
     cleanupUnusedLocks();
 
     for (unsigned i = 0; i < _numLockBuckets; i++) {
-        // TODO: dump more information about the non-empty bucket to see what locks were leaked
         invariant(_lockBuckets[i].data.empty());
     }
 
@@ -533,80 +482,6 @@ LockResult LockManager::lock(ResourceId resId, LockRequest* request, LockMode mo
     return lock->newRequest(request);
 }
 
-LockResult LockManager::convert(ResourceId resId, LockRequest* request, LockMode newMode) {
-    // If we are here, we already hold the lock in some mode. In order to keep it simple, we do
-    // not allow requesting a conversion while a lock is already waiting or pending conversion.
-    invariant(request->recursiveCount > 0);
-
-    request->recursiveCount++;
-
-    // Fast path for acquiring the same lock multiple times in modes, which are already covered
-    // by the current mode. It is safe to do this without locking, because 1) all calls for the
-    // same lock request must be done on the same thread and 2) if there are lock requests
-    // hanging off a given LockHead, then this lock will never disappear.
-    if ((LockConflictsTable[request->mode] | LockConflictsTable[newMode]) ==
-        LockConflictsTable[request->mode]) {
-        return LOCK_OK;
-    }
-
-    // TODO: For the time being we do not need conversions between unrelated lock modes (i.e.,
-    // modes which both add and remove to the conflicts set), so these are not implemented yet
-    // (e.g., S -> IX).
-    invariant((LockConflictsTable[request->mode] | LockConflictsTable[newMode]) ==
-              LockConflictsTable[newMode]);
-
-    LockBucket* bucket = _getBucket(resId);
-    stdx::lock_guard<SimpleMutex> scopedLock(bucket->mutex);
-    invariant(request->status == LockRequest::STATUS_GRANTED);
-
-    LockBucket::Map::iterator it = bucket->data.find(resId);
-    invariant(it != bucket->data.end());
-
-    LockHead* const lock = it->second;
-
-    if (lock->partitioned()) {
-        lock->migratePartitionedLockHeads();
-    }
-
-    // Construct granted mask without our current mode, so that it is not counted as
-    // conflicting
-    uint32_t grantedModesWithoutCurrentRequest = 0;
-
-    // We start the counting at 1 below, because LockModesCount also includes MODE_NONE
-    // at position 0, which can never be acquired/granted.
-    for (uint32_t i = 1; i < LockModesCount; i++) {
-        const uint32_t currentRequestHolds = (request->mode == static_cast<LockMode>(i) ? 1 : 0);
-
-        if (lock->grantedCounts[i] > currentRequestHolds) {
-            grantedModesWithoutCurrentRequest |= modeMask(static_cast<LockMode>(i));
-        }
-    }
-
-    // This check favours conversion requests over pending requests. For example:
-    //
-    // T1 requests lock L in IS
-    // T2 requests lock L in X
-    // T1 then upgrades L from IS -> S
-    //
-    // Because the check does not look into the conflict modes bitmap, it will grant L to
-    // T1 in S mode, instead of block, which would otherwise cause deadlock.
-    if (conflicts(newMode, grantedModesWithoutCurrentRequest)) {
-        request->status = LockRequest::STATUS_CONVERTING;
-        request->convertMode = newMode;
-
-        lock->conversionsCount++;
-        lock->incGrantedModeCount(request->convertMode);
-
-        return LOCK_WAITING;
-    } else {  // No conflict, existing request
-        lock->incGrantedModeCount(newMode);
-        lock->decGrantedModeCount(request->mode);
-        request->mode = newMode;
-
-        return LOCK_OK;
-    }
-}
-
 bool LockManager::unlock(LockRequest* request) {
     invariant(request->recursiveCount > 0);
     request->recursiveCount--;
@@ -617,8 +492,7 @@ bool LockManager::unlock(LockRequest* request) {
         // thorough the partition mutex. Migrations are expected to be rare.
         Partition* partition = _getPartition(request);
         stdx::lock_guard<SimpleMutex> scopedLock(partition->mutex);
-        invariant(request->status == LockRequest::STATUS_GRANTED ||
-                  request->status == LockRequest::STATUS_CONVERTING);
+        invariant(request->status == LockRequest::STATUS_GRANTED);
 
         if (request->status == LockRequest::STATUS_GRANTED && request->recursiveCount > 0)
             return false;
@@ -664,21 +538,6 @@ bool LockManager::unlock(LockRequest* request) {
         lock->decConflictModeCount(request->mode);
 
         _onLockModeChanged(lock, true);
-    } else if (request->status == LockRequest::STATUS_CONVERTING) {
-        // This cancels a pending convert request
-        invariant(request->recursiveCount > 0);
-        invariant(lock->conversionsCount > 0);
-
-        // Lock only goes from GRANTED to CONVERTING, so cancelling the conversion request
-        // brings it back to the previous granted mode.
-        request->status = LockRequest::STATUS_GRANTED;
-
-        lock->conversionsCount--;
-        lock->decGrantedModeCount(request->convertMode);
-
-        request->convertMode = MODE_NONE;
-
-        _onLockModeChanged(lock, lock->grantedCounts[request->convertMode] == 0);
     } else {
         // Invalid request status
         MONGO_UNREACHABLE;
@@ -719,7 +578,6 @@ void LockManager::cleanupUnusedLocks() {
 
 void LockManager::_cleanupUnusedLocksInBucket(LockBucket* bucket) {
     LockBucket::Map::iterator it = bucket->data.begin();
-    size_t deletedLockHeads = 0;
     while (it != bucket->data.end()) {
         LockHead* lock = it->second;
 
@@ -734,11 +592,9 @@ void LockManager::_cleanupUnusedLocksInBucket(LockBucket* bucket) {
             invariant(lock->conflictModes == 0);
             invariant(lock->conflictList._front == nullptr);
             invariant(lock->conflictList._back == nullptr);
-            invariant(lock->conversionsCount == 0);
             invariant(lock->compatibleFirstCount == 0);
 
             bucket->data.erase(it++);
-            deletedLockHeads++;
             delete lock;
         } else {
             it++;
@@ -747,48 +603,6 @@ void LockManager::_cleanupUnusedLocksInBucket(LockBucket* bucket) {
 }
 
 void LockManager::_onLockModeChanged(LockHead* lock, bool checkConflictQueue) {
-    // Unblock any converting requests (because conversions are still counted as granted and
-    // are on the granted queue).
-    for (LockRequest* iter = lock->grantedList._front;
-         (iter != nullptr) && (lock->conversionsCount > 0);
-         iter = iter->next) {
-        // Conversion requests are going in a separate queue
-        if (iter->status == LockRequest::STATUS_CONVERTING) {
-            invariant(iter->convertMode != 0);
-
-            // Construct granted mask without our current mode, so that it is not accounted as
-            // a conflict
-            uint32_t grantedModesWithoutCurrentRequest = 0;
-
-            // We start the counting at 1 below, because LockModesCount also includes
-            // MODE_NONE at position 0, which can never be acquired/granted.
-            for (uint32_t i = 1; i < LockModesCount; i++) {
-                const uint32_t currentRequestHolds =
-                    (iter->mode == static_cast<LockMode>(i) ? 1 : 0);
-
-                const uint32_t currentRequestWaits =
-                    (iter->convertMode == static_cast<LockMode>(i) ? 1 : 0);
-
-                // We cannot both hold and wait on the same lock mode
-                invariant(currentRequestHolds + currentRequestWaits <= 1);
-
-                if (lock->grantedCounts[i] > (currentRequestHolds + currentRequestWaits)) {
-                    grantedModesWithoutCurrentRequest |= modeMask(static_cast<LockMode>(i));
-                }
-            }
-
-            if (!conflicts(iter->convertMode, grantedModesWithoutCurrentRequest)) {
-                lock->conversionsCount--;
-                lock->decGrantedModeCount(iter->mode);
-                iter->status = LockRequest::STATUS_GRANTED;
-                iter->mode = iter->convertMode;
-                iter->convertMode = MODE_NONE;
-
-                iter->notify->notify(lock->resourceId, LOCK_OK);
-            }
-        }
-    }
-
     // Grant any conflicting requests, which might now be unblocked. Note that the loop below
     // slightly violates fairness in that it will grant *all* compatible requests on the line even
     // though there might be conflicting ones interspersed between them. For example, assume that an
@@ -869,23 +683,24 @@ bool LockManager::hasConflictingRequests(ResourceId resId, const LockRequest* re
     return request->lock ? !request->lock->conflictList.empty() : false;
 }
 
-void LockManager::dump() const {
-    BSONArrayBuilder locks;
-    _buildLocksArray(getLockToClientMap(getGlobalServiceContext()), true, nullptr, &locks);
-    LOGV2_OPTIONS(20521,
-                  logv2::LogTruncation::Disabled,
-                  "lock manager dump",
-                  "addr"_attr = formatPtr(this),
-                  "locks"_attr = locks.arr());
+std::vector<LogDegugInfo> LockManager::getLockInfoFromResourceHolders(ResourceId resId) {
+    std::vector<LogDegugInfo> locksInfo;
+    for (size_t i = 0; i < _numLockBuckets; ++i) {
+        LockBucket& bucket = _lockBuckets[i];
+        stdx::lock_guard<SimpleMutex> scopedLock{bucket.mutex};
+        const auto it = bucket.data.find(resId);
+        if (it == bucket.data.end()) {
+            continue;
+        }
+        LockHead* lock = it->second;
+        for (auto iter = lock->grantedList._front; iter != nullptr; iter = iter->next) {
+            locksInfo.emplace_back(iter->mode, iter->locker->getDebugInfo());
+        }
+    }
+    return locksInfo;
 }
 
-void LockManager::getLockInfoBSON(const std::map<LockerId, BSONObj>& lockToClientMap,
-                                  BSONObjBuilder* result) {
-    auto lockInfoArr = BSONArrayBuilder(result->subarrayStart("lockInfo"));
-    _buildLocksArray(lockToClientMap, false, this, &lockInfoArr);
-}
-
-void LockManager::_buildLocksArray(const std::map<LockerId, BSONObj>& lockToClientMap,
+void LockManager::getLockInfoArray(const std::map<LockerId, BSONObj>& lockToClientMap,
                                    bool forLogging,
                                    LockManager* mutableThis,
                                    BSONArrayBuilder* locks) const {
@@ -922,7 +737,6 @@ void LockManager::_buildLocksArray(const std::map<LockerId, BSONObj>& lockToClie
                         req.append("thread", formatThreadId(iter->locker->getThreadId()));
                     }
                     req.append("mode", modeName(iter->mode));
-                    req.append("convertMode", modeName(iter->convertMode));
                     req.append("enqueueAtFront", iter->enqueueAtFront);
                     req.append("compatibleFirst", iter->compatibleFirst);
                     req.append("debugInfo", iter->locker->getDebugInfo());
@@ -936,6 +750,28 @@ void LockManager::_buildLocksArray(const std::map<LockerId, BSONObj>& lockToClie
     }
 }
 
+//
+// LockBucket
+//
+
+LockHead* LockManager::LockBucket::findOrInsert(ResourceId resId) {
+    LockHead* lock;
+    Map::iterator it = data.find(resId);
+    if (it == data.end()) {
+        lock = new LockHead();
+        lock->initNew(resId);
+
+        data.insert(Map::value_type(resId, lock));
+    } else {
+        lock = it->second;
+    }
+    return lock;
+}
+
+//
+// Partition
+//
+
 PartitionedLockHead* LockManager::Partition::find(ResourceId resId) {
     Map::iterator it = data.find(resId);
     return it == data.end() ? nullptr : it->second;
@@ -955,66 +791,6 @@ PartitionedLockHead* LockManager::Partition::findOrInsert(ResourceId resId) {
     return lock;
 }
 
-LockHead* LockManager::LockBucket::findOrInsert(ResourceId resId) {
-    LockHead* lock;
-    Map::iterator it = data.find(resId);
-    if (it == data.end()) {
-        lock = new LockHead();
-        lock->initNew(resId);
-
-        data.insert(Map::value_type(resId, lock));
-    } else {
-        lock = it->second;
-    }
-    return lock;
-}
-
-//
-// ResourceId
-//
-std::string ResourceId::toString() const {
-    StringBuilder ss;
-    ss << "{" << _fullHash << ": " << resourceTypeName(getType()) << ", " << getHashId();
-    if (getType() == RESOURCE_MUTEX) {
-        ss << ", " << Lock::ResourceMutex::getName(*this);
-    }
-
-    if (getType() == RESOURCE_DATABASE || getType() == RESOURCE_COLLECTION) {
-        if (auto resourceName = ResourceCatalog::get(getGlobalServiceContext()).name(*this)) {
-            ss << ", " << *resourceName;
-        }
-    }
-
-    ss << "}";
-
-    return ss.str();
-}
-
-
-//
-// LockRequest
-//
-
-void LockRequest::initNew(Locker* locker, LockGrantNotification* notify) {
-    this->locker = locker;
-    this->notify = notify;
-
-    enqueueAtFront = false;
-    compatibleFirst = false;
-    recursiveCount = 1;
-
-    lock = nullptr;
-    partitionedLock = nullptr;
-    prev = nullptr;
-    next = nullptr;
-    status = STATUS_NEW;
-    partitioned = false;
-    mode = MODE_NONE;
-    convertMode = MODE_NONE;
-    unlockPending = 0;
-}
-
-
 //
 // Helper calls
 //
