@@ -27,13 +27,28 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
-#include <boost/algorithm/string.hpp>
+#include <absl/container/node_hash_map.h>
+#include <boost/algorithm/string/split.hpp>
+#include <boost/core/addressof.hpp>
+#include <boost/function/function_base.hpp>
+#include <boost/iterator/iterator_facade.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/type_index/type_index_facade.hpp>
+// IWYU pragma: no_include "ext/alloc_traits.h"
+#include <cstddef>
+#include <list>
+#include <memory>
+#include <string>
 
 #include "mongo/base/string_data.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/bson/bsontypes_util.h"
 #include "mongo/db/cst/bson_lexer.h"
+#include "mongo/db/cst/c_node.h"
 #include "mongo/db/cst/parser_gen.hpp"
+#include "mongo/platform/decimal128.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/string_map.h"
 
 namespace mongo {
@@ -334,7 +349,7 @@ void BSONLexer::tokenize(BSONElement elem, bool includeFieldName) {
     switch (elem.type()) {
         case BSONType::Array: {
             pushToken("start array", ParserGen::token::START_ARRAY);
-            auto index = 0;
+            auto index = 0U;
             for (auto&& nestedElem : elem.embeddedObject()) {
                 ScopedLocationTracker arrayCtx{this, index++};
                 // For arrays, do not tokenize the field names.
@@ -367,8 +382,8 @@ void BSONLexer::tokenize(BSONElement elem, bool includeFieldName) {
                 pushToken(elem.valueStringData(), it->second);
             } else {
                 // If we don't care about the keyword, then it's treated as a generic value.
-                if (elem.valueStringData()[0] == '$') {
-                    if (elem.valueStringData()[1] == '$') {
+                if (elem.valueStringData().starts_with('$')) {
+                    if (elem.valueStringData().starts_with("$$")) {
                         pushToken(elem.valueStringData(),
                                   ParserGen::token::DOLLAR_DOLLAR_STRING,
                                   elem.String());
@@ -483,7 +498,7 @@ BSONLexer::BSONLexer(BSONObj obj, ParserGen::token_type startingToken) {
     // array index. No need to tokenize the fieldname for that case.
     if (startingToken == ParserGen::token::START_PIPELINE) {
         pushToken("start array", ParserGen::token::START_ARRAY);
-        auto index = 0;
+        auto index = 0U;
         for (auto&& elem : obj) {
             ScopedLocationTracker stageCtx{this, index++};
             tokenize(elem, false);
