@@ -29,21 +29,39 @@
 
 #pragma once
 
+#include <absl/container/node_hash_map.h>
+#include <boost/optional/optional.hpp>
+#include <cstddef>
+#include <functional>
+#include <memory>
+#include <mutex>
 #include <utility>
+#include <vector>
 
+#include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
 #include "mongo/db/catalog/util/partitioned.h"
 #include "mongo/db/clientcursor.h"
 #include "mongo/db/cursor_id.h"
 #include "mongo/db/generic_cursor.h"
+#include "mongo/db/generic_cursor_gen.h"
 #include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/pipeline/process_interface/mongo_process_interface.h"
 #include "mongo/db/record_id.h"
+#include "mongo/db/service_context.h"
 #include "mongo/db/session/kill_sessions.h"
+#include "mongo/db/session/logical_session_id.h"
+#include "mongo/db/session/logical_session_id_gen.h"
 #include "mongo/db/session/session_killer.h"
+#include "mongo/platform/mutex.h"
+#include "mongo/platform/random.h"
 #include "mongo/stdx/unordered_map.h"
 #include "mongo/stdx/unordered_set.h"
 #include "mongo/util/clock_source.h"
 #include "mongo/util/concurrency/mutex.h"
 #include "mongo/util/duration.h"
+#include "mongo/util/time_support.h"
 #include "mongo/util/uuid.h"
 
 namespace mongo {
@@ -146,14 +164,6 @@ public:
      */
     Status killCursor(OperationContext* opCtx, CursorId id);
 
-    /**
-     * Same as 'killCursor' but with an 'authChecker' callback. This is used to recheck
-     * authorization before the cursor is killed.
-     */
-    Status killCursorWithAuthCheck(OperationContext* opCtx,
-                                   CursorId id,
-                                   const std::function<void(const ClientCursor&)>& authChecker);
-
     /**
      * Returns an OK status if we're authorized to erase the cursor. Otherwise, returns
      * ErrorCodes::Unauthorized.
@@ -217,7 +227,6 @@ public:
     }
 
 private:
-    static constexpr int kNumPartitions = 16;
     friend class ClientCursorPin;
 
     // deregisterAndDestroyCursor deregisters the cursor from the manager's cursorMap, then safely
@@ -254,9 +263,7 @@ private:
         map->erase(cursor->cursorid());
     }
 
-    Status _killCursor(OperationContext* opCtx,
-                       CursorId id,
-                       const std::function<void(const ClientCursor&)>& authChecker);
+    ClockSource* _preciseClockSource;
 
     // A CursorManager holds a pointer to all open ClientCursors. ClientCursors are owned by the
     // CursorManager, except when they are in use by a ClientCursorPin. When in use by a pin, an
@@ -282,7 +289,6 @@ private:
     // registering a cursor.
     mutable Mutex _opKeyMutex = MONGO_MAKE_LATCH("CursorManager::_opKeyMutex");
     stdx::unordered_map<OperationKey, std::set<CursorId>, UUID::Hash> _opKeyMap;
-
-    ClockSource* _preciseClockSource;
 };
+
 }  // namespace mongo
