@@ -29,11 +29,27 @@
 
 #pragma once
 
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <memory>
+#include <string>
+
+#include "mongo/bson/bsonobj.h"
+#include "mongo/client/connection_string.h"
 #include "mongo/client/dbclient_base.h"
-#include "mongo/config.h"
+#include "mongo/client/dbclient_cursor.h"
+#include "mongo/client/read_preference.h"
+#include "mongo/config.h"  // IWYU pragma: keep
 #include "mongo/db/dbmessage.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/ops/write_ops.h"
+#include "mongo/db/ops/write_ops_gen.h"
+#include "mongo/db/query/find_command.h"
+#include "mongo/rpc/message.h"
+#include "mongo/util/assert_util_core.h"
 #include "mongo/util/net/hostandport.h"
+#include "mongo/util/net/ssl_types.h"
 
 namespace mongo {
 
@@ -92,6 +108,10 @@ public:
     write_ops::FindAndModifyCommandReply findAndModify(
         const write_ops::FindAndModifyCommandRequest& findAndModify);
 
+protected:
+    auth::ValidatedTenancyScope _createInnerRequestVTS(
+        const boost::optional<TenantId>& tenantId) const override;
+
 private:
     bool isFailed() const override;
 
@@ -101,8 +121,6 @@ private:
 
     std::string getServerAddress() const override;
 
-    std::string getLocalAddress() const override;
-
     void say(Message& toSend, bool isRetry = false, std::string* actualServer = nullptr) override;
 
     ConnectionString::ConnectionType type() const override;
@@ -131,7 +149,7 @@ private:
 
     void _auth(const BSONObj& params) override;
 
-    void _call(Message& toSend, Message& response, std::string* actualServer) override;
+    Message _call(Message& toSend, std::string* actualServer) override;
 
     OperationContext* _opCtx;
 };
