@@ -27,35 +27,59 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
 
-#include "mongo/platform/basic.h"
+#include <set>
+#include <string>
+#include <utility>
 
-#include "mongo/db/dbhelpers.h"
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
 
+#include "mongo/base/status_with.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/db/catalog/clustered_collection_util.h"
+#include "mongo/db/catalog/collection.h"
+#include "mongo/db/catalog/collection_catalog.h"
+#include "mongo/db/catalog/collection_operation_source.h"
+#include "mongo/db/catalog/collection_write_path.h"
+#include "mongo/db/catalog/index_catalog.h"
+#include "mongo/db/catalog/index_catalog_entry.h"
+#include "mongo/db/catalog_raii.h"
+#include "mongo/db/curop.h"
 #include "mongo/db/db_raii.h"
-#include "mongo/db/exec/working_set_common.h"
-#include "mongo/db/index/btree_access_method.h"
-#include "mongo/db/json.h"
-#include "mongo/db/keypattern.h"
+#include "mongo/db/dbhelpers.h"
+#include "mongo/db/index/index_access_method.h"
+#include "mongo/db/index/index_descriptor.h"
+#include "mongo/db/matcher/expression_parser.h"
 #include "mongo/db/matcher/extensions_callback_real.h"
 #include "mongo/db/namespace_string.h"
-#include "mongo/db/op_observer.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/ops/delete.h"
 #include "mongo/db/ops/update.h"
 #include "mongo/db/ops/update_request.h"
+#include "mongo/db/ops/write_ops_parsers.h"
+#include "mongo/db/pipeline/expression_context.h"
+#include "mongo/db/query/canonical_query.h"
+#include "mongo/db/query/find_command.h"
 #include "mongo/db/query/get_executor.h"
 #include "mongo/db/query/index_bounds_builder.h"
 #include "mongo/db/query/internal_plans.h"
-#include "mongo/db/query/query_planner.h"
+#include "mongo/db/query/plan_executor.h"
+#include "mongo/db/query/plan_yield_policy.h"
 #include "mongo/db/record_id_helpers.h"
-#include "mongo/util/scopeguard.h"
-#include "mongo/util/str.h"
+#include "mongo/db/repl/oplog.h"
+#include "mongo/db/shard_role.h"
+#include "mongo/db/storage/snapshot.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/intrusive_counter.h"
+
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
+
 
 namespace mongo {
 
-using std::set;
 using std::string;
 using std::unique_ptr;
 
@@ -104,26 +128,18 @@ RecordId Helpers::findOne(OperationContext* opCtx,
     if (!collection)
         return RecordId();
 
-    const ExtensionsCallbackReal extensionsCallback(opCtx, &collection->ns());
+    auto cq = std::make_unique<CanonicalQuery>(CanonicalQueryParams{
+        .expCtx = makeExpressionContext(opCtx, *findCommand),
+        .parsedFind = ParsedFindCommandParams{
+            .findCommand = std::move(findCommand),
+            .extensionsCallback = ExtensionsCallbackReal(opCtx, &collection->ns()),
+            .allowedFeatures = MatchExpressionParser::kAllowAllSpecialFeatures}});
+    cq->setForceGenerateRecordId(true);
 
-    const boost::intrusive_ptr<ExpressionContext> expCtx;
-    auto statusWithCQ =
-        CanonicalQuery::canonicalize(opCtx,
-                                     std::move(findCommand),
-                                     false,
-                                     expCtx,
-                                     extensionsCallback,
-                                     MatchExpressionParser::kAllowAllSpecialFeatures);
-
-    massertStatusOK(statusWithCQ.getStatus());
-    unique_ptr<CanonicalQuery> cq = std::move(statusWithCQ.getValue());
-
-    auto exec = uassertStatusOK(getExecutor(opCtx,
-                                            &collection,
-                                            std::move(cq),
-                                            nullptr /* extractAndAttachPipelineStages */,
-                                            PlanYieldPolicy::YieldPolicy::NO_YIELD,
-                                            QueryPlannerParams::DEFAULT));
+    auto exec = uassertStatusOK(getExecutorFind(opCtx,
+                                                MultipleCollectionAccessor{collection},
+                                                std::move(cq),
+                                                PlanYieldPolicy::YieldPolicy::INTERRUPT_ONLY));
 
     PlanExecutor::ExecState state;
     BSONObj obj;
@@ -135,36 +151,20 @@ RecordId Helpers::findOne(OperationContext* opCtx,
 }
 
 bool Helpers::findById(OperationContext* opCtx,
-                       Database* database,
-                       StringData ns,
+                       const NamespaceString& nss,
                        BSONObj query,
-                       BSONObj& result,
-                       bool* nsFound,
-                       bool* indexFound) {
-    invariant(database);
-
-    // TODO ForRead?
-    NamespaceString nss{ns};
-    CollectionPtr collection =
-        CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, nss);
+                       BSONObj& result) {
+    auto collCatalog = CollectionCatalog::get(opCtx);
+    const Collection* collection = collCatalog->lookupCollectionByNamespace(opCtx, nss);
     if (!collection) {
         return false;
     }
 
-    if (nsFound)
-        *nsFound = true;
-
     const IndexCatalog* catalog = collection->getIndexCatalog();
     const IndexDescriptor* desc = catalog->findIdIndex(opCtx);
 
     if (!desc) {
         if (clustered_util::isClusteredOnId(collection->getClusteredInfo())) {
-            if (indexFound) {
-                // A collection clustered on _id implicitly has an _id index but no explicit
-                // IndexDescriptor tied to it.
-                *indexFound = 1;
-            }
-
             Snapshotted<BSONObj> doc;
             if (collection->findDoc(opCtx,
                                     record_id_helpers::keyForObj(IndexBoundsBuilder::objFromElement(
@@ -178,11 +178,9 @@ bool Helpers::findById(OperationContext* opCtx,
         return false;
     }
 
-    if (indexFound)
-        *indexFound = 1;
-
-    auto recordId = catalog->getEntry(desc)->accessMethod()->asSortedData()->findSingle(
-        opCtx, collection, query["_id"].wrap());
+    const IndexCatalogEntry* entry = catalog->getEntry(desc);
+    auto recordId = entry->accessMethod()->asSortedData()->findSingle(
+        opCtx, CollectionPtr(collection), entry, query["_id"].wrap());
     if (recordId.isNull())
         return false;
     result = collection->docFor(opCtx, recordId).value();
@@ -192,7 +190,7 @@ bool Helpers::findById(OperationContext* opCtx,
 RecordId Helpers::findById(OperationContext* opCtx,
                            const CollectionPtr& collection,
                            const BSONObj& idquery) {
-    verify(collection);
+    MONGO_verify(collection);
     const IndexCatalog* catalog = collection->getIndexCatalog();
     const IndexDescriptor* desc = catalog->findIdIndex(opCtx);
     if (!desc && clustered_util::isClusteredOnId(collection->getClusteredInfo())) {
@@ -203,17 +201,18 @@ RecordId Helpers::findById(OperationContext* opCtx,
     }
 
     uassert(13430, "no _id index", desc);
-    return catalog->getEntry(desc)->accessMethod()->asSortedData()->findSingle(
-        opCtx, collection, idquery["_id"].wrap());
+    const IndexCatalogEntry* entry = catalog->getEntry(desc);
+    return entry->accessMethod()->asSortedData()->findSingle(
+        opCtx, collection, entry, idquery["_id"].wrap());
 }
 
 // Acquires necessary locks to read the collection with the given namespace. If this is an oplog
-// read, use AutoGetOplog for simplified locking.
+// read, use AutoGetOplogFastPath for simplified locking.
 const CollectionPtr& getCollectionForRead(
     OperationContext* opCtx,
     const NamespaceString& ns,
     boost::optional<AutoGetCollectionForReadCommand>& autoColl,
-    boost::optional<AutoGetOplog>& autoOplog) {
+    boost::optional<AutoGetOplogFastPath>& autoOplog) {
     if (ns.isOplog()) {
         // Simplify locking rules for oplog collection.
         autoOplog.emplace(opCtx, OplogAccessMode::kRead);
@@ -224,16 +223,16 @@ const CollectionPtr& getCollectionForRead(
     }
 }
 
-bool Helpers::getSingleton(OperationContext* opCtx, const char* ns, BSONObj& result) {
+bool Helpers::getSingleton(OperationContext* opCtx, const NamespaceString& nss, BSONObj& result) {
     boost::optional<AutoGetCollectionForReadCommand> autoColl;
-    boost::optional<AutoGetOplog> autoOplog;
-    const auto& collection = getCollectionForRead(opCtx, NamespaceString(ns), autoColl, autoOplog);
+    boost::optional<AutoGetOplogFastPath> autoOplog;
+    const auto& collection = getCollectionForRead(opCtx, nss, autoColl, autoOplog);
     if (!collection) {
         return false;
     }
 
-    auto exec =
-        InternalPlanner::collectionScan(opCtx, &collection, PlanYieldPolicy::YieldPolicy::NO_YIELD);
+    auto exec = InternalPlanner::collectionScan(
+        opCtx, &collection, PlanYieldPolicy::YieldPolicy::INTERRUPT_ONLY);
     PlanExecutor::ExecState state = exec->getNext(&result, nullptr);
 
     CurOp::get(opCtx)->done();
@@ -249,16 +248,18 @@ bool Helpers::getSingleton(OperationContext* opCtx, const char* ns, BSONObj& res
     return false;
 }
 
-bool Helpers::getLast(OperationContext* opCtx, const char* ns, BSONObj& result) {
+bool Helpers::getLast(OperationContext* opCtx, const NamespaceString& nss, BSONObj& result) {
     boost::optional<AutoGetCollectionForReadCommand> autoColl;
-    boost::optional<AutoGetOplog> autoOplog;
-    const auto& collection = getCollectionForRead(opCtx, NamespaceString(ns), autoColl, autoOplog);
+    boost::optional<AutoGetOplogFastPath> autoOplog;
+    const auto& collection = getCollectionForRead(opCtx, nss, autoColl, autoOplog);
     if (!collection) {
         return false;
     }
 
-    auto exec = InternalPlanner::collectionScan(
-        opCtx, &collection, PlanYieldPolicy::YieldPolicy::NO_YIELD, InternalPlanner::BACKWARD);
+    auto exec = InternalPlanner::collectionScan(opCtx,
+                                                &collection,
+                                                PlanYieldPolicy::YieldPolicy::INTERRUPT_ONLY,
+                                                InternalPlanner::BACKWARD);
     PlanExecutor::ExecState state = exec->getNext(&result, nullptr);
 
     // Non-yielding collection scans from InternalPlanner will never error.
@@ -273,25 +274,24 @@ bool Helpers::getLast(OperationContext* opCtx, const char* ns, BSONObj& result)
 }
 
 UpdateResult Helpers::upsert(OperationContext* opCtx,
-                             const string& ns,
+                             CollectionAcquisition& coll,
                              const BSONObj& o,
                              bool fromMigrate) {
     BSONElement e = o["_id"];
-    verify(e.type());
+    MONGO_verify(e.type());
     BSONObj id = e.wrap();
-    return upsert(opCtx, ns, id, o, fromMigrate);
+    return upsert(opCtx, coll, id, o, fromMigrate);
 }
 
 UpdateResult Helpers::upsert(OperationContext* opCtx,
-                             const string& ns,
+                             CollectionAcquisition& coll,
                              const BSONObj& filter,
                              const BSONObj& updateMod,
                              bool fromMigrate) {
-    OldClientContext context(opCtx, ns);
+    OldClientContext context(opCtx, coll.nss());
 
-    const NamespaceString requestNs(ns);
     auto request = UpdateRequest();
-    request.setNamespaceString(requestNs);
+    request.setNamespaceString(coll.nss());
 
     request.setQuery(filter);
     request.setUpdateModification(write_ops::UpdateModification::parseFromClassicUpdate(updateMod));
@@ -300,21 +300,20 @@ UpdateResult Helpers::upsert(OperationContext* opCtx,
         request.setSource(OperationSource::kFromMigrate);
         request.setBypassEmptyTsReplacement(true);
     }
-    request.setYieldPolicy(PlanYieldPolicy::YieldPolicy::NO_YIELD);
+    request.setYieldPolicy(PlanYieldPolicy::YieldPolicy::INTERRUPT_ONLY);
 
-    return ::mongo::update(opCtx, context.db(), request);
+    return ::mongo::update(opCtx, coll, request);
 }
 
 void Helpers::update(OperationContext* opCtx,
-                     const string& ns,
+                     CollectionAcquisition& coll,
                      const BSONObj& filter,
                      const BSONObj& updateMod,
                      bool fromMigrate) {
-    OldClientContext context(opCtx, ns);
+    OldClientContext context(opCtx, coll.nss());
 
-    const NamespaceString requestNs(ns);
     auto request = UpdateRequest();
-    request.setNamespaceString(requestNs);
+    request.setNamespaceString(coll.nss());
 
     request.setQuery(filter);
     request.setUpdateModification(write_ops::UpdateModification::parseFromClassicUpdate(updateMod));
@@ -322,22 +321,29 @@ void Helpers::update(OperationContext* opCtx,
         request.setSource(OperationSource::kFromMigrate);
         request.setBypassEmptyTsReplacement(true);
     }
-    request.setYieldPolicy(PlanYieldPolicy::YieldPolicy::NO_YIELD);
+    request.setYieldPolicy(PlanYieldPolicy::YieldPolicy::INTERRUPT_ONLY);
 
-    ::mongo::update(opCtx, context.db(), request);
+    ::mongo::update(opCtx, coll, request);
 }
 
-void Helpers::putSingleton(OperationContext* opCtx, const char* ns, BSONObj obj) {
-    OldClientContext context(opCtx, ns);
+Status Helpers::insert(OperationContext* opCtx,
+                       const CollectionAcquisition& coll,
+                       const BSONObj& doc) {
+    OldClientContext context(opCtx, coll.nss());
+    return collection_internal::insertDocument(
+        opCtx, coll.getCollectionPtr(), InsertStatement{doc}, &CurOp::get(opCtx)->debug());
+}
+
+void Helpers::putSingleton(OperationContext* opCtx, CollectionAcquisition& coll, BSONObj obj) {
+    OldClientContext context(opCtx, coll.nss());
 
-    const NamespaceString requestNs(ns);
     auto request = UpdateRequest();
-    request.setNamespaceString(requestNs);
+    request.setNamespaceString(coll.nss());
 
     request.setUpdateModification(write_ops::UpdateModification::parseFromClassicUpdate(obj));
     request.setUpsert();
 
-    ::mongo::update(opCtx, context.db(), request);
+    ::mongo::update(opCtx, coll, request);
 
     CurOp::get(opCtx)->done();
 }
@@ -358,13 +364,10 @@ BSONObj Helpers::inferKeyPattern(const BSONObj& o) {
     return kpBuilder.obj();
 }
 
-void Helpers::emptyCollection(OperationContext* opCtx, const NamespaceString& nss) {
-    OldClientContext context(opCtx, nss.ns());
+void Helpers::emptyCollection(OperationContext* opCtx, const CollectionAcquisition& coll) {
+    OldClientContext context(opCtx, coll.nss());
     repl::UnreplicatedWritesBlock uwb(opCtx);
-    CollectionPtr collection = context.db()
-        ? CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, nss)
-        : nullptr;
-    deleteObjects(opCtx, collection, nss, BSONObj(), false);
+    deleteObjects(opCtx, coll, BSONObj(), false);
 }
 
 bool Helpers::findByIdAndNoopUpdate(OperationContext* opCtx,
@@ -390,10 +393,18 @@ bool Helpers::findByIdAndNoopUpdate(OperationContext* opCtx,
     // BSONObj because that's a second way OpObserverImpl::onUpdate() detects and ignores no-op
     // updates.
     repl::UnreplicatedWritesBlock uwb(opCtx);
-    CollectionUpdateArgs args;
+    CollectionUpdateArgs args(snapshottedDoc.value());
     args.criteria = idQuery;
     args.update = BSONObj();
-    collection->updateDocument(opCtx, recordId, snapshottedDoc, result, false, nullptr, &args);
+    collection_internal::updateDocument(opCtx,
+                                        collection,
+                                        recordId,
+                                        snapshottedDoc,
+                                        result,
+                                        collection_internal::kUpdateNoIndexes,
+                                        nullptr /* indexesAffected */,
+                                        nullptr /* opDebug */,
+                                        &args);
 
     return true;
 }
