@@ -32,12 +32,13 @@
 #include "mongo/db/auth/authorization_manager.h"
 #include "mongo/db/auth/authorization_session.h"
 #include "mongo/db/curop.h"
+#include "mongo/db/s/operation_sharding_state.h"
 #include "mongo/db/s/sharding_api_d_params_gen.h"
-#include "mongo/db/s/sharding_cluster_parameters_gen.h"
-#include "mongo/db/s/sharding_state.h"
 #include "mongo/db/s/sharding_statistics.h"
 #include "mongo/logv2/log.h"
+#include "mongo/s/sharding_cluster_parameters_gen.h"
 #include "mongo/s/sharding_feature_flags_gen.h"
+#include "mongo/s/sharding_state.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
@@ -54,12 +55,18 @@ void checkDirectShardOperationAllowed(OperationContext* opCtx, const NamespaceSt
     // Skip direct shard connection checks for namespaces which can have independent contents on
     // each shard. The direct shard connection check still applies to the sharding metadata
     // collection namespaces because those collections exist on a single, particular shard.
-    if ((nsIsDbOnly(nss.ns()) && nss.isOnInternalDb()) || nss.isShardLocalNamespace()) {
+    if ((nss.isDbOnly() && nss.dbName().isInternalDb()) || nss.isShardLocalNamespace()) {
+        return;
+    }
+    // Skip direct shard connection checks for commands which explicitly request skipping these
+    // checks. There should be very few cases in which this is true.
+    if (OperationShardingState::get(opCtx).shouldSkipDirectConnectionChecks()) {
         return;
     }
     const auto fcvSnapshot = serverGlobalParams.featureCompatibility.acquireFCVSnapshot();
     if (ShardingState::get(opCtx)->enabled() && fcvSnapshot.isVersionInitialized() &&
-        feature_flags::gCheckForDirectShardOperations.isEnabled(fcvSnapshot)) {
+        (feature_flags::gCheckForDirectShardOperations.isEnabled(fcvSnapshot) ||
+         feature_flags::gFailOnDirectShardOperations.isEnabled(fcvSnapshot))) {
         bool clusterHasTwoOrMoreShards = [&]() {
             auto* clusterParameters = ServerParameterSet::getClusterParameterSet();
             auto* clusterCardinalityParam =
@@ -67,31 +74,53 @@ void checkDirectShardOperationAllowed(OperationContext* opCtx, const NamespaceSt
                     "shardedClusterCardinalityForDirectConns");
             return clusterCardinalityParam->getValue(boost::none).getHasTwoOrMoreShards();
         }();
-
         if (clusterHasTwoOrMoreShards || directConnectionChecksWithSingleShard.load()) {
-            const bool authIsEnabled = AuthorizationManager::get(opCtx->getServiceContext()) &&
-                AuthorizationManager::get(opCtx->getServiceContext())->isAuthEnabled();
+            const bool authIsEnabled = AuthorizationManager::get(opCtx->getService()) &&
+                AuthorizationManager::get(opCtx->getService())->isAuthEnabled();
 
             const bool directShardOperationsAllowed = !authIsEnabled ||
                 (!CurOp::get(opCtx)->isStarted() || !CurOp::get(opCtx)->getCommand()) ||
                 (AuthorizationSession::exists(opCtx->getClient()) &&
                  AuthorizationSession::get(opCtx->getClient())
-                     ->isAuthorizedForActionsOnResource(ResourcePattern::forClusterResource(),
-                                                        ActionType::issueDirectShardOperations));
+                     ->isAuthorizedForActionsOnResource(
+                         ResourcePattern::forClusterResource(nss.tenantId()),
+                         ActionType::issueDirectShardOperations));
 
             if (!directShardOperationsAllowed) {
                 ShardingStatistics::get(opCtx).unauthorizedDirectShardOperations.addAndFetch(1);
-                // Atlas log ingestion requires a strict upper bound on the number of logs per
-                // hour. To abide by this, we log the lower verbosity messages with a different
-                // log ID to prevent log ingestion from picking them up.
-                const auto severity = ShardingState::get(opCtx)->directConnectionLogSeverity();
-                LOGV2_DEBUG(
-                    severity == logv2::LogSeverity::Warning() ? 7553700 : 8993900,
-                    severity.toInt(),
-                    "You are connecting to a sharded cluster improperly by connecting directly "
-                    "to a shard. Please connect to the cluster via a router (mongos).",
-                    logAttrs(nss),
-                    "command"_attr = CurOp::get(opCtx)->getCommand()->getName());
+                static constexpr char errorMsg[] =
+                    "You are connecting to a sharded cluster improperly by connecting directly to "
+                    "a shard. Please connect to the cluster via a router (mongos).";
+                if (feature_flags::gFailOnDirectShardOperations.isEnabled(fcvSnapshot) &&
+                    clusterHasTwoOrMoreShards) {
+                    // Atlas log ingestion requires a strict upper bound on the number of logs per
+                    // hour. To abide by this, we only log this message once per hour and rely on
+                    // the user assertion log (debug 1) otherwise.
+                    const auto severity = ShardingState::get(opCtx)->directConnectionLogSeverity();
+                    if (severity == logv2::LogSeverity::Warning()) {
+                        LOGV2_DEBUG(8679600,
+                                    logv2::LogSeverity::Error().toInt(),
+                                    errorMsg,
+                                    logAttrs(nss),
+                                    "command"_attr = CurOp::get(opCtx)->getCommand()->getName());
+                    }
+                    uasserted(ErrorCodes::Unauthorized,
+                              str::stream() << errorMsg << " Command: "
+                                            << CurOp::get(opCtx)->getCommand()->getName()
+                                            << ", Namespace: " << nss.toStringForErrorMsg());
+                } else if (feature_flags::gCheckForDirectShardOperations.isEnabled(fcvSnapshot) ||
+                           (!clusterHasTwoOrMoreShards &&
+                            directConnectionChecksWithSingleShard.load())) {
+                    // Atlas log ingestion requires a strict upper bound on the number of logs per
+                    // hour. To abide by this, we log the lower verbosity messages with a different
+                    // log ID to prevent log ingestion from picking them up.
+                    const auto severity = ShardingState::get(opCtx)->directConnectionLogSeverity();
+                    LOGV2_DEBUG(severity == logv2::LogSeverity::Warning() ? 7553700 : 8993900,
+                                severity.toInt(),
+                                errorMsg,
+                                logAttrs(nss),
+                                "command"_attr = CurOp::get(opCtx)->getCommand()->getName());
+                }
             }
         }
     }
