@@ -29,7 +29,16 @@
 
 #pragma once
 
+#include <boost/optional/optional.hpp>
+#include <string>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/db/repl/optime.h"
 #include "mongo/db/session/logical_session_id.h"
+#include "mongo/db/session/logical_session_id_gen.h"
 
 namespace mongo {
 static constexpr StringData kErrorLabelsFieldName = "errorLabels"_sd;
