@@ -27,16 +27,31 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <boost/cstdint.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <cstdint>
+#include <mutex>
 
+#include <boost/optional/optional.hpp>
+
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/db/client.h"
 #include "mongo/db/curop.h"
 #include "mongo/db/error_labels.h"
-#include "mongo/db/pipeline/aggregate_command_gen.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/pipeline/aggregation_request_helper.h"
 #include "mongo/db/pipeline/lite_parsed_pipeline.h"
+#include "mongo/db/service_context.h"
 #include "mongo/db/service_context_test_fixture.h"
-#include "mongo/db/session/logical_session_id.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/rpc/message.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/bson_test_util.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/fail_point.h"
 
 namespace mongo {
 namespace {
@@ -78,12 +93,6 @@ TEST(IsTransientTransactionErrorTest, ShardCannotRefreshDueToLocksHeldIsTransien
                                             false /* isCommitOrAbort */));
 }
 
-TEST(IsTransientTransactionErrorTest, ShardInvalidatedForTargetingIsTransient) {
-    ASSERT_TRUE(isTransientTransactionError(ErrorCodes::ShardInvalidatedForTargeting,
-                                            false /* hasWriteConcernError */,
-                                            false /* isCommitOrAbort */));
-}
-
 TEST(IsTransientTransactionErrorTest, StaleDbVersionIsTransient) {
     ASSERT_TRUE(isTransientTransactionError(
         ErrorCodes::StaleDbVersion, false /* hasWriteConcernError */, false /* isCommitOrAbort */));
@@ -125,7 +134,8 @@ public:
     ErrorLabelBuilderTest() : _opCtx(makeOperationContext()) {}
 
     void setCommand(BSONObj cmdObj) const {
-        CurOp::get(opCtx())->setGenericOpRequestDetails(
+        stdx::lock_guard<Client> clientLock(*opCtx()->getClient());
+        CurOp::get(opCtx())->setGenericOpRequestDetails_inlock(
             _testNss, nullptr, cmdObj, NetworkOp::dbMsg);
     }
 
@@ -150,7 +160,7 @@ private:
 };
 
 TEST_F(ErrorLabelBuilderTest, NonErrorCodesHaveNoLabel) {
-    OperationSessionInfoFromClient sessionInfo;
+    OperationSessionInfoFromClient sessionInfo{LogicalSessionFromClient(UUID::gen())};
     std::string commandName = "insert";
     ErrorLabelBuilder builder(opCtx(),
                               sessionInfo,
@@ -169,7 +179,7 @@ TEST_F(ErrorLabelBuilderTest, NonErrorCodesHaveNoLabel) {
 }
 
 TEST_F(ErrorLabelBuilderTest, NonTransactionsHaveNoTransientTransactionErrorLabel) {
-    OperationSessionInfoFromClient sessionInfo;
+    OperationSessionInfoFromClient sessionInfo{LogicalSessionFromClient(UUID::gen())};
     std::string commandName = "insert";
     ErrorLabelBuilder builder(opCtx(),
                               sessionInfo,
@@ -185,7 +195,7 @@ TEST_F(ErrorLabelBuilderTest, NonTransactionsHaveNoTransientTransactionErrorLabe
 }
 
 TEST_F(ErrorLabelBuilderTest, RetryableWritesHaveNoTransientTransactionErrorLabel) {
-    OperationSessionInfoFromClient sessionInfo;
+    OperationSessionInfoFromClient sessionInfo{LogicalSessionFromClient(UUID::gen())};
     sessionInfo.setTxnNumber(1);
     std::string commandName = "insert";
     ErrorLabelBuilder builder(opCtx(),
@@ -202,7 +212,7 @@ TEST_F(ErrorLabelBuilderTest, RetryableWritesHaveNoTransientTransactionErrorLabe
 }
 
 TEST_F(ErrorLabelBuilderTest, NonTransientTransactionErrorsHaveNoTransientTransactionErrorLabel) {
-    OperationSessionInfoFromClient sessionInfo;
+    OperationSessionInfoFromClient sessionInfo{LogicalSessionFromClient(UUID::gen())};
     sessionInfo.setTxnNumber(1);
     sessionInfo.setAutocommit(false);
     std::string commandName = "commitTransaction";
@@ -220,7 +230,7 @@ TEST_F(ErrorLabelBuilderTest, NonTransientTransactionErrorsHaveNoTransientTransa
 }
 
 TEST_F(ErrorLabelBuilderTest, TransientTransactionErrorsHaveTransientTransactionErrorLabel) {
-    OperationSessionInfoFromClient sessionInfo;
+    OperationSessionInfoFromClient sessionInfo{LogicalSessionFromClient(UUID::gen())};
     sessionInfo.setTxnNumber(1);
     sessionInfo.setAutocommit(false);
     std::string commandName = "commitTransaction";
@@ -240,7 +250,7 @@ TEST_F(ErrorLabelBuilderTest, TransientTransactionErrorsHaveTransientTransaction
 TEST_F(
     ErrorLabelBuilderTest,
     TransientTransactionErrorWithRetryableWriteConcernErrorHasTransientTransactionErrorLabelOnly) {
-    OperationSessionInfoFromClient sessionInfo;
+    OperationSessionInfoFromClient sessionInfo{LogicalSessionFromClient(UUID::gen())};
     sessionInfo.setTxnNumber(1);
     sessionInfo.setAutocommit(false);
     std::string commandName = "commitTransaction";
@@ -266,7 +276,7 @@ TEST_F(
 }
 
 TEST_F(ErrorLabelBuilderTest, NonRetryableWritesHaveNoRetryableWriteErrorLabel) {
-    OperationSessionInfoFromClient sessionInfo;
+    OperationSessionInfoFromClient sessionInfo{LogicalSessionFromClient(UUID::gen())};
     std::string commandName = "insert";
     ErrorLabelBuilder builder(opCtx(),
                               sessionInfo,
@@ -289,7 +299,7 @@ TEST_F(ErrorLabelBuilderTest, NonRetryableWritesHaveNoRetryableWriteErrorLabel)
 }
 
 TEST_F(ErrorLabelBuilderTest, NonRetryableWriteErrorsHaveNoRetryableWriteErrorLabel) {
-    OperationSessionInfoFromClient sessionInfo;
+    OperationSessionInfoFromClient sessionInfo{LogicalSessionFromClient(UUID::gen())};
     sessionInfo.setTxnNumber(1);
     std::string commandName = "update";
     ErrorLabelBuilder builder(opCtx(),
@@ -306,7 +316,7 @@ TEST_F(ErrorLabelBuilderTest, NonRetryableWriteErrorsHaveNoRetryableWriteErrorLa
 }
 
 TEST_F(ErrorLabelBuilderTest, RetryableWriteErrorsHaveRetryableWriteErrorLabel) {
-    OperationSessionInfoFromClient sessionInfo;
+    OperationSessionInfoFromClient sessionInfo{LogicalSessionFromClient(UUID::gen())};
     sessionInfo.setTxnNumber(1);
     std::string commandName = "update";
     ErrorLabelBuilder builder(opCtx(),
@@ -323,7 +333,7 @@ TEST_F(ErrorLabelBuilderTest, RetryableWriteErrorsHaveRetryableWriteErrorLabel)
 }
 
 TEST_F(ErrorLabelBuilderTest, NonLocalShutDownErrorsOnMongosDoNotHaveRetryableWriteErrorLabel) {
-    OperationSessionInfoFromClient sessionInfo;
+    OperationSessionInfoFromClient sessionInfo{LogicalSessionFromClient(UUID::gen())};
     sessionInfo.setTxnNumber(1);
     std::string commandName = "update";
     ErrorLabelBuilder builder(opCtx(),
@@ -341,7 +351,7 @@ TEST_F(ErrorLabelBuilderTest, NonLocalShutDownErrorsOnMongosDoNotHaveRetryableWr
 
 TEST_F(ErrorLabelBuilderTest,
        LocalShutDownErrorsOnMongosHaveRetryableWriteErrorLabelInterruptedAtShutdown) {
-    OperationSessionInfoFromClient sessionInfo;
+    OperationSessionInfoFromClient sessionInfo{LogicalSessionFromClient(UUID::gen())};
     sessionInfo.setTxnNumber(1);
     std::string commandName = "update";
     FailPointEnableBlock failPoint("errorLabelBuilderMockShutdown");
@@ -360,7 +370,7 @@ TEST_F(ErrorLabelBuilderTest,
 
 TEST_F(ErrorLabelBuilderTest,
        LocalShutDownErrorsOnMongosHaveRetryableWriteErrorLabelCallbackCanceled) {
-    OperationSessionInfoFromClient sessionInfo;
+    OperationSessionInfoFromClient sessionInfo{LogicalSessionFromClient(UUID::gen())};
     sessionInfo.setTxnNumber(1);
     std::string commandName = "update";
     FailPointEnableBlock failPoint("errorLabelBuilderMockShutdown");
@@ -379,7 +389,7 @@ TEST_F(ErrorLabelBuilderTest,
 
 TEST_F(ErrorLabelBuilderTest,
        RetryableWriteErrorsHaveNoRetryableWriteErrorLabelForInternalClients) {
-    OperationSessionInfoFromClient sessionInfo;
+    OperationSessionInfoFromClient sessionInfo{LogicalSessionFromClient(UUID::gen())};
     sessionInfo.setTxnNumber(1);
     std::string commandName = "update";
     ErrorLabelBuilder builder(opCtx(),
@@ -397,7 +407,7 @@ TEST_F(ErrorLabelBuilderTest,
 
 TEST_F(ErrorLabelBuilderTest,
        NonRetryableWriteErrorsInWriteConcernErrorsHaveNoRetryableWriteErrorLabel) {
-    OperationSessionInfoFromClient sessionInfo;
+    OperationSessionInfoFromClient sessionInfo{LogicalSessionFromClient(UUID::gen())};
     sessionInfo.setTxnNumber(1);
     std::string commandName = "update";
     ErrorLabelBuilder builder(opCtx(),
@@ -415,7 +425,7 @@ TEST_F(ErrorLabelBuilderTest,
 
 TEST_F(ErrorLabelBuilderTest,
        RetryableWriteErrorsInWriteConcernErrorsHaveRetryableWriteErrorLabel) {
-    OperationSessionInfoFromClient sessionInfo;
+    OperationSessionInfoFromClient sessionInfo{LogicalSessionFromClient(UUID::gen())};
     sessionInfo.setTxnNumber(1);
     std::string commandName = "update";
     ErrorLabelBuilder builder(opCtx(),
@@ -432,7 +442,7 @@ TEST_F(ErrorLabelBuilderTest,
 }
 
 TEST_F(ErrorLabelBuilderTest, RetryableWriteErrorsOnCommitAbortHaveRetryableWriteErrorLabel) {
-    OperationSessionInfoFromClient sessionInfo;
+    OperationSessionInfoFromClient sessionInfo{LogicalSessionFromClient(UUID::gen())};
     sessionInfo.setTxnNumber(1);
     sessionInfo.setAutocommit(false);
     std::string commandName;
@@ -510,7 +520,7 @@ TEST_F(ErrorLabelBuilderTest, RetryableWriteErrorsOnCommitAbortHaveRetryableWrit
 
 TEST_F(ErrorLabelBuilderTest,
        CommandsNotFromRouterWithStaleConfigErrorHaveRetryableWriteErrorLabel) {
-    OperationSessionInfoFromClient sessionInfo;
+    OperationSessionInfoFromClient sessionInfo{LogicalSessionFromClient(UUID::gen())};
     sessionInfo.setTxnNumber(1);
     std::string commandName = "update";
     ErrorLabelBuilder builder(opCtx(),
@@ -528,7 +538,7 @@ TEST_F(ErrorLabelBuilderTest,
 
 TEST_F(ErrorLabelBuilderTest,
        CommandsFromRouterWithStaleConfigErrorShouldNotHaveRetryableWriteErrorLabel) {
-    OperationSessionInfoFromClient sessionInfo;
+    OperationSessionInfoFromClient sessionInfo{LogicalSessionFromClient(UUID::gen())};
     sessionInfo.setTxnNumber(1);
     std::string commandName = "update";
     ErrorLabelBuilder builder(opCtx(),
@@ -545,7 +555,7 @@ TEST_F(ErrorLabelBuilderTest,
 }
 
 TEST_F(ErrorLabelBuilderTest, NonResumableChangeStreamError) {
-    OperationSessionInfoFromClient sessionInfo;
+    OperationSessionInfoFromClient sessionInfo{LogicalSessionFromClient(UUID::gen())};
     std::string commandName;
     ErrorLabelBuilder builder(opCtx(),
                               sessionInfo,
@@ -561,18 +571,18 @@ TEST_F(ErrorLabelBuilderTest, NonResumableChangeStreamError) {
 }
 
 TEST_F(ErrorLabelBuilderTest, ResumableChangeStreamErrorAppliesToChangeStreamAggregations) {
-    OperationSessionInfoFromClient sessionInfo;
+    OperationSessionInfoFromClient sessionInfo{LogicalSessionFromClient(UUID::gen())};
     // Build the aggregation command and confirm that it parses correctly, so we know that the error
     // is the only factor that determines the success or failure of isResumableChangeStreamError().
     auto cmdObj = BSON("aggregate" << nss().coll() << "pipeline"
                                    << BSON_ARRAY(BSON("$changeStream" << BSONObj())) << "cursor"
-                                   << BSONObj() << "$db" << nss().db());
-    auto aggRequest =
-        uassertStatusOK(aggregation_request_helper::parseFromBSONForTests(nss(), cmdObj));
+                                   << BSONObj() << "$db" << nss().db_forTest());
+    auto aggRequest = uassertStatusOK(aggregation_request_helper::parseFromBSONForTests(cmdObj));
     ASSERT_TRUE(LiteParsedPipeline(aggRequest).hasChangeStream());
 
     for (auto&& errorCode : {ErrorCodes::NetworkTimeout,
                              ErrorCodes::QueryPlanKilled,
+                             ErrorCodes::ResumeTenantChangeStream,
                              ErrorCodes::FailedToSatisfyReadPreference}) {
         // The label applies to a $changeStream "aggregate" command.
         std::string commandName = "aggregate";
@@ -607,14 +617,13 @@ TEST_F(ErrorLabelBuilderTest, ResumableChangeStreamErrorAppliesToChangeStreamAgg
 }
 
 TEST_F(ErrorLabelBuilderTest, ResumableChangeStreamErrorDoesNotApplyToNonResumableErrors) {
-    OperationSessionInfoFromClient sessionInfo;
+    OperationSessionInfoFromClient sessionInfo{LogicalSessionFromClient(UUID::gen())};
     // Build the aggregation command and confirm that it parses correctly, so we know that the error
     // is the only factor that determines the success or failure of isResumableChangeStreamError().
     auto cmdObj = BSON("aggregate" << nss().coll() << "pipeline"
                                    << BSON_ARRAY(BSON("$changeStream" << BSONObj())) << "cursor"
-                                   << BSONObj() << "$db" << nss().db());
-    auto aggRequest =
-        uassertStatusOK(aggregation_request_helper::parseFromBSONForTests(nss(), cmdObj));
+                                   << BSONObj() << "$db" << nss().db_forTest());
+    auto aggRequest = uassertStatusOK(aggregation_request_helper::parseFromBSONForTests(cmdObj));
     ASSERT_TRUE(LiteParsedPipeline(aggRequest).hasChangeStream());
 
     // The label does not apply to a ChangeStreamFatalError error on a $changeStream aggregation.
@@ -648,14 +657,13 @@ TEST_F(ErrorLabelBuilderTest, ResumableChangeStreamErrorDoesNotApplyToNonResumab
 }
 
 TEST_F(ErrorLabelBuilderTest, ResumableChangeStreamErrorDoesNotApplyToNonChangeStreamAggregations) {
-    OperationSessionInfoFromClient sessionInfo;
+    OperationSessionInfoFromClient sessionInfo{LogicalSessionFromClient(UUID::gen())};
     // Build the aggregation command and confirm that it parses correctly, so we know that the error
     // is the only factor that determines the success or failure of isResumableChangeStreamError().
     auto cmdObj =
         BSON("aggregate" << nss().coll() << "pipeline" << BSON_ARRAY(BSON("$match" << BSONObj()))
-                         << "cursor" << BSONObj() << "$db" << nss().db());
-    auto aggRequest =
-        uassertStatusOK(aggregation_request_helper::parseFromBSONForTests(nss(), cmdObj));
+                         << "cursor" << BSONObj() << "$db" << nss().db_forTest());
+    auto aggRequest = uassertStatusOK(aggregation_request_helper::parseFromBSONForTests(cmdObj));
     ASSERT_FALSE(LiteParsedPipeline(aggRequest).hasChangeStream());
 
     // The label does not apply to a non-$changeStream "aggregate" command.
@@ -689,7 +697,7 @@ TEST_F(ErrorLabelBuilderTest, ResumableChangeStreamErrorDoesNotApplyToNonChangeS
 }
 
 TEST_F(ErrorLabelBuilderTest, ResumableChangeStreamErrorDoesNotApplyToNonAggregations) {
-    OperationSessionInfoFromClient sessionInfo;
+    OperationSessionInfoFromClient sessionInfo{LogicalSessionFromClient(UUID::gen())};
     auto cmdObj = BSON("find" << nss().coll() << "filter" << BSONObj());
     // The label does not apply to a "find" command.
     std::string commandName = "find";
@@ -722,7 +730,7 @@ TEST_F(ErrorLabelBuilderTest, ResumableChangeStreamErrorDoesNotApplyToNonAggrega
 }
 
 TEST_F(ErrorLabelBuilderTest, NoWritesPerformedLabelApplied) {
-    OperationSessionInfoFromClient sessionInfo;
+    OperationSessionInfoFromClient sessionInfo{LogicalSessionFromClient(UUID::gen())};
     std::string commandName = "find";
     ErrorLabelBuilder builder(opCtx(),
                               sessionInfo,
@@ -738,7 +746,7 @@ TEST_F(ErrorLabelBuilderTest, NoWritesPerformedLabelApplied) {
 }
 
 TEST_F(ErrorLabelBuilderTest, NoWritesPerformedLabelNotAppliedAfterWrite) {
-    OperationSessionInfoFromClient sessionInfo;
+    OperationSessionInfoFromClient sessionInfo{LogicalSessionFromClient(UUID::gen())};
     std::string commandName = "update";
     ErrorLabelBuilder builder(opCtx(),
                               sessionInfo,
@@ -754,7 +762,7 @@ TEST_F(ErrorLabelBuilderTest, NoWritesPerformedLabelNotAppliedAfterWrite) {
 }
 
 TEST_F(ErrorLabelBuilderTest, NoWritesPerformedLabelNotAppliedIfUnknown) {
-    OperationSessionInfoFromClient sessionInfo;
+    OperationSessionInfoFromClient sessionInfo{LogicalSessionFromClient(UUID::gen())};
     std::string commandName = "update";
     ErrorLabelBuilder builder(opCtx(),
                               sessionInfo,
@@ -770,7 +778,7 @@ TEST_F(ErrorLabelBuilderTest, NoWritesPerformedLabelNotAppliedIfUnknown) {
 }
 
 TEST_F(ErrorLabelBuilderTest, NoWritesPerformedAndRetryableWriteAppliesBothLabels) {
-    OperationSessionInfoFromClient sessionInfo;
+    OperationSessionInfoFromClient sessionInfo{LogicalSessionFromClient(UUID::gen())};
     sessionInfo.setTxnNumber(1);
     std::string commandName = "update";
     auto actualErrorLabels = getErrorLabels(opCtx(),
@@ -790,7 +798,7 @@ TEST_F(ErrorLabelBuilderTest, NoWritesPerformedAndRetryableWriteAppliesBothLabel
 }
 
 TEST_F(ErrorLabelBuilderTest, NoWritesPerformedNotAppliedDuringOrdinaryUpdate) {
-    OperationSessionInfoFromClient sessionInfo;
+    OperationSessionInfoFromClient sessionInfo{LogicalSessionFromClient(UUID::gen())};
     std::string commandName = "update";
     auto actualErrorLabels = getErrorLabels(opCtx(),
                                             sessionInfo,
@@ -806,7 +814,7 @@ TEST_F(ErrorLabelBuilderTest, NoWritesPerformedNotAppliedDuringOrdinaryUpdate) {
 }
 
 TEST_F(ErrorLabelBuilderTest, NoWritesPerformedNotAppliedDuringTransientTransactionError) {
-    OperationSessionInfoFromClient sessionInfo;
+    OperationSessionInfoFromClient sessionInfo{LogicalSessionFromClient(UUID::gen())};
     sessionInfo.setTxnNumber(1);
     sessionInfo.setAutocommit(false);
     std::string commandName = "commitTransaction";
