@@ -28,25 +28,50 @@
  */
 
 
-#include "mongo/platform/basic.h"
+#include <cstdint>
+#include <exception>
+#include <string>
+#include <utility>
+#include <vector>
 
-#include "mongo/db/exec/batched_delete_stage.h"
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/util/builder.h"
 #include "mongo/db/catalog/collection.h"
 #include "mongo/db/catalog/collection_write_path.h"
+#include "mongo/db/catalog/index_catalog.h"
 #include "mongo/db/commands/server_status.h"
-#include "mongo/db/curop.h"
+#include "mongo/db/concurrency/exception_util.h"
+#include "mongo/db/exec/batched_delete_stage.h"
+#include "mongo/db/exec/document_value/document.h"
 #include "mongo/db/exec/plan_stage.h"
-#include "mongo/db/exec/scoped_timer.h"
 #include "mongo/db/exec/working_set.h"
-#include "mongo/db/exec/working_set_common.h"
 #include "mongo/db/exec/write_stage_common.h"
-#include "mongo/db/op_observer/op_observer.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/query/canonical_query.h"
+#include "mongo/db/query/plan_executor.h"
 #include "mongo/db/query/plan_executor_impl.h"
 #include "mongo/db/service_context.h"
-#include "mongo/logv2/log.h"
+#include "mongo/db/storage/recovery_unit.h"
+#include "mongo/db/storage/snapshot.h"
+#include "mongo/db/storage/write_unit_of_work.h"
+#include "mongo/db/transaction_resources.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/s/shard_version.h"
+#include "mongo/s/stale_exception.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/fail_point.h"
+#include "mongo/util/future.h"
+#include "mongo/util/namespace_string_util.h"
 #include "mongo/util/scopeguard.h"
+#include "mongo/util/str.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kWrite
 
@@ -55,7 +80,6 @@ namespace mongo {
 
 MONGO_FAIL_POINT_DEFINE(throwWriteConflictExceptionInBatchedDeleteStage);
 MONGO_FAIL_POINT_DEFINE(batchedDeleteStageSleepAfterNDocuments);
-MONGO_FAIL_POINT_DEFINE(batchedDeleteStageThrowTemporarilyUnavailableException);
 
 namespace {
 
@@ -85,12 +109,7 @@ void incrementSSSMetricNoOverflow(AtomicWord<long long>& metric, long long value
  * Reports globally-aggregated batch stats.
  */
 struct BatchedDeletesSSS : ServerStatusSection {
-    BatchedDeletesSSS()
-        : ServerStatusSection("batchedDeletes"),
-          batches(0),
-          docs(0),
-          stagedSizeBytes(0),
-          timeInBatchMillis(0) {}
+    using ServerStatusSection::ServerStatusSection;
 
     bool includeByDefault() const override {
         return true;
@@ -107,12 +126,13 @@ struct BatchedDeletesSSS : ServerStatusSection {
         return bob.obj();
     }
 
-    AtomicWord<long long> batches;
-    AtomicWord<long long> docs;
-    AtomicWord<long long> stagedSizeBytes;
-    AtomicWord<long long> timeInBatchMillis;
-    AtomicWord<long long> refetchesDueToYield;
-} batchedDeletesSSS;
+    AtomicWord<long long> batches{0};
+    AtomicWord<long long> docs{0};
+    AtomicWord<long long> stagedSizeBytes{0};
+    AtomicWord<long long> timeInBatchMillis{0};
+    AtomicWord<long long> refetchesDueToYield{0};
+};
+auto& batchedDeletesSSS = *ServerStatusSectionBuilder<BatchedDeletesSSS>("batchedDeletes");
 
 // Wrapper for write_stage_common::ensureStillMatches() which also updates the 'refetchesDueToYield'
 // serverStatus metric. As with ensureStillMatches, if false is returned, the WoringSetMember
@@ -123,7 +143,7 @@ bool ensureStillMatchesAndUpdateStats(const CollectionPtr& collection,
                                       WorkingSetID id,
                                       const CanonicalQuery* cq) {
     WorkingSetMember* member = ws->get(id);
-    if (opCtx->recoveryUnit()->getSnapshotId() != member->doc.snapshotId()) {
+    if (shard_role_details::getRecoveryUnit(opCtx)->getSnapshotId() != member->doc.snapshotId()) {
         incrementSSSMetricNoOverflow(batchedDeletesSSS.refetchesDueToYield, 1);
     }
     return write_stage_common::ensureStillMatches(collection, opCtx, ws, id, cq);
@@ -134,7 +154,7 @@ BatchedDeleteStage::BatchedDeleteStage(
     std::unique_ptr<DeleteStageParams> params,
     std::unique_ptr<BatchedDeleteStageParams> batchedDeleteParams,
     WorkingSet* ws,
-    const CollectionPtr& collection,
+    CollectionAcquisition collection,
     PlanStage* child)
     : DeleteStage::DeleteStage(
           kStageType.rawData(), expCtx, std::move(params), ws, collection, child),
@@ -229,22 +249,15 @@ PlanStage::StageState BatchedDeleteStage::doWork(WorkingSetID* out) {
         _commitStagedDeletes = _passStagingComplete || !_stagedDeletesBuffer.empty();
     }
 
-    // We need to check if 'planStateStage' is 'NEED_YIELD' earlier than we check 'isEOF()'.
-    // The reason is that 'isEOF()' returns true if we have completed staging and have an empty
-    // buffer of staged deletes. However, even if 'isEOF()' returns true, the 'planStateStage' can
-    // be 'NEED_YIELD' here if inside 'restoreState()' in '_deleteBatch()' a
-    // 'StorageEngineException' was thrown and caught. In this case, the 'planStateStage' is
-    // 'NEED_YIELD' and 'isEOF()' already returns true.
-    if (planStageState == PlanStage::NEED_YIELD) {
-        *out = idToReturn;
-        return PlanStage::NEED_YIELD;
-    }
-
     if (isEOF()) {
         invariant(planStageState != PlanStage::NEED_YIELD);
         return PlanStage::IS_EOF;
     }
 
+    if (planStageState == PlanStage::NEED_YIELD) {
+        *out = idToReturn;
+    }
+
     return planStageState;
 }
 
@@ -253,18 +266,20 @@ PlanStage::StageState BatchedDeleteStage::_deleteBatch(WorkingSetID* out) {
         return PlanStage::NEED_TIME;
     }
 
-    handlePlanStageYield(
+    const auto saveRet = handlePlanStageYield(
         expCtx(),
         "BatchedDeleteStage saveState",
-        collection()->ns().ns(),
         [&] {
             child()->saveState();
-            return PlanStage::NEED_TIME /* unused */;
+            return PlanStage::NEED_TIME;
         },
         [&] {
             // yieldHandler
-            std::terminate();
+            _prepareToRetryDrainAfterYield(out, {});
         });
+    if (saveRet != PlanStage::NEED_TIME) {
+        return saveRet;
+    }
 
     std::set<WorkingSetID> recordsToSkip;
     unsigned int docsDeleted = 0;
@@ -276,7 +291,6 @@ PlanStage::StageState BatchedDeleteStage::_deleteBatch(WorkingSetID* out) {
         const auto ret = handlePlanStageYield(
             expCtx(),
             "BatchedDeleteStage::_deleteBatch",
-            collection()->ns().ns(),
             [&] {
                 timeInBatch =
                     _commitBatch(out, &recordsToSkip, &docsDeleted, &bytesDeleted, &bufferOffset);
@@ -297,7 +311,8 @@ PlanStage::StageState BatchedDeleteStage::_deleteBatch(WorkingSetID* out) {
             // yield, wait for critical section to finish and then we'll resume the write from the
             // point we had left. We do this to prevent large multi-writes from repeatedly failing
             // due to StaleConfig and exhausting the mongos retry attempts.
-            planExecutorShardingCriticalSectionFuture(opCtx()) = ex->getCriticalSectionSignal();
+            planExecutorShardingState(opCtx()).criticalSectionFuture =
+                ex->getCriticalSectionSignal();
             _prepareToRetryDrainAfterYield(out, recordsToSkip);
             return PlanStage::NEED_YIELD;
         }
@@ -340,7 +355,8 @@ long long BatchedDeleteStage::_commitBatch(WorkingSetID* out,
     // Start a WUOW with 'groupOplogEntries' which groups a delete batch into a single timestamp
     // and oplog entry.
     WriteUnitOfWork wuow(opCtx(),
-                         _stagedDeletesBuffer.size() > 1U ? true : false /* groupOplogEntries */);
+                         _stagedDeletesBuffer.size() > 1U ? WriteUnitOfWork::kGroupForTransaction
+                                                          : WriteUnitOfWork::kDontGroup);
     for (; *bufferOffset < _stagedDeletesBuffer.size(); ++*bufferOffset) {
         if (MONGO_unlikely(throwWriteConflictExceptionInBatchedDeleteStage.shouldFail())) {
             throwWriteConflictException(
@@ -351,30 +367,28 @@ long long BatchedDeleteStage::_commitBatch(WorkingSetID* out,
         auto workingSetMemberID = _stagedDeletesBuffer.at(*bufferOffset);
         WorkingSetMember* member = _ws->get(workingSetMemberID);
 
-        // Determine whether the document being deleted is owned by this shard, and the action
-        // to undertake if it isn't.
         using write_stage_common::PreWriteFilter;
-        // Warning: on Action::kSkip, the WSM's underlaying document is no longer valid.
         const PreWriteFilter::Action action = [&]() {
             // The PlanExecutor YieldPolicy may change snapshots between calls to 'doWork()'.
             // Different documents may have different snapshots.
             const bool docStillMatches = ensureStillMatchesAndUpdateStats(
-                collection(), opCtx(), _ws, workingSetMemberID, _params->canonicalQuery);
+                collectionPtr(), opCtx(), _ws, workingSetMemberID, _params->canonicalQuery);
 
-            // Warning: if docStillMatches is false, the WSM's underlaying Document/BSONObj is
-            // no longer valid.
+            // Warning: if docStillMatches is false, the WSM's underlying Document/BSONObj is no
+            // longer valid.
             if (!docStillMatches) {
                 return PreWriteFilter::Action::kSkip;
             }
-            // Determine whether the document being deleted is owned by this shard, and the
-            // action to undertake if it isn't.
+
+            // Determine whether the document being deleted is owned by this shard, and the action
+            // to undertake if it isn't.
             return _preWriteFilter.computeActionAndLogSpecialCases(
-                member->doc.value(), "batched delete"_sd, collection()->ns());
+                member->doc.value(), "batched delete"_sd, collectionPtr()->ns());
         }();
 
         // Skip the document, as it either no longer exists, or has been filtered by the
         // PreWriteFilter.
-        if (PreWriteFilter::Action::kSkip == action) {
+        if (action == PreWriteFilter::Action::kSkip) {
             recordsToSkip->insert(workingSetMemberID);
             continue;
         }
@@ -388,27 +402,24 @@ long long BatchedDeleteStage::_commitBatch(WorkingSetID* out,
         tassert(
             6515700, "Expected document to have an _id field present", bsonObjDoc.hasField("_id"));
         applyOpsBytes += bsonObjDoc.getField("_id").size();
-        if (applyOpsBytes > BSONObjMaxUserSize && ((*bufferOffset) > 0)) {
+        if (applyOpsBytes > BSONObjMaxUserSize) {
             // There's no room to fit this deletion in the current batch, as doing so
             // would exceed 16MB of oplog entry: put this deletion back into the staging
-            // buffer and commit the batch. Very large _id fields may exceed this threshold. In that
-            // case, put them in their own batch.
+            // buffer and commit the batch.
+            invariant(*bufferOffset > 0);
             (*bufferOffset)--;
             wuow.commit();
             return batchTimer.millis();
         }
-        tassert(10118000,
-                "batch size may only exceed BSON cap for single, large documents",
-                applyOpsBytes <= BSONObjMaxUserSize || ((*bufferOffset) == 0));
 
         collection_internal::deleteDocument(
             opCtx(),
-            collection(),
+            collectionPtr(),
             Snapshotted(memberDoc.snapshotId(), bsonObjDoc),
             _params->stmtId,
             member->recordId,
             _params->opDebug,
-            _params->fromMigrate || writeToOrphan,
+            writeToOrphan,
             false,
             _params->returnDeleted ? collection_internal::StoreDeletedDoc::On
                                    : collection_internal::StoreDeletedDoc::Off,
@@ -429,9 +440,9 @@ long long BatchedDeleteStage::_commitBatch(WorkingSetID* out,
                 // committed + the number of documents deleted in the current unit of work.
 
                 // Assume nDocs is positive.
-                return data.hasField("sleepMs") && data.hasField("ns") &&
-                    data.getStringField("ns") == collection()->ns().toString() &&
-                    data.hasField("nDocs") &&
+                const auto fpNss = NamespaceStringUtil::parseFailPointData(data, "ns"_sd);
+                return data.hasField("sleepMs") && !fpNss.isEmpty() &&
+                    collectionPtr()->ns() == fpNss && data.hasField("nDocs") &&
                     _specificStats.docsDeleted + *docsDeleted >=
                     static_cast<unsigned int>(data.getIntField("nDocs"));
             });
@@ -499,16 +510,8 @@ PlanStage::StageState BatchedDeleteStage::_tryRestoreState(WorkingSetID* out) {
     return handlePlanStageYield(
         expCtx(),
         "BatchedDeleteStage::_tryRestoreState",
-        collection()->ns().ns(),
         [&] {
-            if (MONGO_unlikely(
-                    batchedDeleteStageThrowTemporarilyUnavailableException.shouldFail())) {
-                throwTemporarilyUnavailableException(
-                    str::stream()
-                    << "Hit failpoint '"
-                    << batchedDeleteStageThrowTemporarilyUnavailableException.getName() << "'.");
-            }
-            child()->restoreState(&collection());
+            child()->restoreState(&collectionPtr());
             return PlanStage::NEED_TIME;
         },
         [&] {
