@@ -28,13 +28,12 @@
  */
 
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/exec/batched_delete_stage_buffer.h"
-
+#include <algorithm>
 #include <fmt/format.h>
 
-#include "mongo/logv2/log.h"
+
+#include "mongo/db/exec/batched_delete_stage_buffer.h"
+#include "mongo/util/assert_util.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
