@@ -27,19 +27,42 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <cstdint>
+#include <iterator>
+#include <set>
+#include <string>
+#include <utility>
 
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/bson/json.h"
 #include "mongo/bson/util/bsoncolumn.h"
 #include "mongo/bson/util/bsoncolumnbuilder.h"
-#include "mongo/db/exec/bucket_unpacker.h"
+#include "mongo/db/exec/document_value/document.h"
 #include "mongo/db/exec/document_value/document_value_test_util.h"
+#include "mongo/db/exec/timeseries/bucket_unpacker.h"
+#include "mongo/db/query/timeseries/bucket_spec.h"
 #include "mongo/db/timeseries/bucket_compression.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/decimal_counter.h"
+#include "mongo/util/time_support.h"
 
 namespace mongo {
 namespace {
 
+using timeseries::BucketSpec;
+using timeseries::BucketUnpacker;
+
 constexpr auto kUserDefinedTimeName = "time"_sd;
 constexpr auto kUserDefinedMetaName = "myMeta"_sd;
 
@@ -90,7 +113,7 @@ public:
         BSONObjBuilder root;
         {
             BSONObjBuilder builder(root.subobjStart("control"_sd));
-            builder.append("version"_sd, 1);
+            builder.append("version"_sd, timeseries::kTimeseriesControlUncompressedVersion);
         }
         {
             BSONObjBuilder data(root.subobjStart("data"_sd));
