@@ -28,31 +28,41 @@
  */
 
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/exec/cached_plan.h"
-
 #include <memory>
+#include <type_traits>
+#include <utility>
+#include <vector>
 
-#include "mongo/db/catalog/collection.h"
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
 #include "mongo/db/concurrency/exception_util.h"
+#include "mongo/db/exec/cached_plan.h"
 #include "mongo/db/exec/multi_plan.h"
 #include "mongo/db/exec/plan_cache_util.h"
-#include "mongo/db/exec/scoped_timer.h"
 #include "mongo/db/exec/trial_period_utils.h"
-#include "mongo/db/exec/working_set_common.h"
 #include "mongo/db/query/classic_plan_cache.h"
 #include "mongo/db/query/collection_query_info.h"
-#include "mongo/db/query/explain.h"
+#include "mongo/db/query/plan_cache.h"
 #include "mongo/db/query/plan_cache_key_factory.h"
+#include "mongo/db/query/plan_explainer.h"
+#include "mongo/db/query/plan_explainer_factory.h"
 #include "mongo/db/query/plan_yield_policy.h"
 #include "mongo/db/query/query_knobs_gen.h"
 #include "mongo/db/query/query_planner.h"
 #include "mongo/db/query/stage_builder_util.h"
 #include "mongo/db/stats/counters.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/redaction.h"
+#include "mongo/platform/atomic_proxy.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/scopeguard.h"
 #include "mongo/util/str.h"
-#include "mongo/util/transitional_tools_do_not_use/vector_spooling.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
@@ -63,21 +73,20 @@ namespace mongo {
 const char* CachedPlanStage::kStageType = "CACHED_PLAN";
 
 CachedPlanStage::CachedPlanStage(ExpressionContext* expCtx,
-                                 const CollectionPtr& collection,
+                                 VariantCollectionPtrOrAcquisition collection,
                                  WorkingSet* ws,
                                  CanonicalQuery* cq,
-                                 const QueryPlannerParams& params,
                                  size_t decisionWorks,
                                  std::unique_ptr<PlanStage> root)
     : RequiresAllIndicesStage(kStageType, expCtx, collection),
       _ws(ws),
       _canonicalQuery(cq),
-      _plannerParams(params),
       _decisionWorks(decisionWorks) {
     _children.emplace_back(std::move(root));
 }
 
-Status CachedPlanStage::pickBestPlan(PlanYieldPolicy* yieldPolicy) {
+Status CachedPlanStage::pickBestPlan(const QueryPlannerParams& plannerParams,
+                                     PlanYieldPolicy* yieldPolicy) {
     // Adds the amount of time taken by pickBestPlan() to executionTime. There's lots of execution
     // work that happens here, so this is needed for the time accounting to make sense.
     auto optTimer = getOptTimer();
@@ -123,7 +132,8 @@ Status CachedPlanStage::pickBestPlan(PlanYieldPolicy* yieldPolicy) {
                         "status"_attr = redact(ex.toStatus()));
 
             const bool shouldCache = false;
-            return replan(yieldPolicy,
+            return replan(plannerParams,
+                          yieldPolicy,
                           shouldCache,
                           str::stream() << "cached plan returned: " << ex.toStatus());
         }
@@ -179,6 +189,7 @@ Status CachedPlanStage::pickBestPlan(PlanYieldPolicy* yieldPolicy) {
 
     const bool shouldCache = true;
     return replan(
+        plannerParams,
         yieldPolicy,
         shouldCache,
         str::stream()
@@ -195,14 +206,16 @@ Status CachedPlanStage::tryYield(PlanYieldPolicy* yieldPolicy) {
     // In all cases, the actual yielding happens here.
     if (yieldPolicy->shouldYieldOrInterrupt(expCtx()->opCtx)) {
         // Here's where we yield.
-        return yieldPolicy->yieldOrInterrupt(
-            expCtx()->opCtx, nullptr, RestoreContext::RestoreType::kYield);
+        return yieldPolicy->yieldOrInterrupt(expCtx()->opCtx);
     }
 
     return Status::OK();
 }
 
-Status CachedPlanStage::replan(PlanYieldPolicy* yieldPolicy, bool shouldCache, std::string reason) {
+Status CachedPlanStage::replan(const QueryPlannerParams& plannerParams,
+                               PlanYieldPolicy* yieldPolicy,
+                               bool shouldCache,
+                               std::string reason) {
     planCacheCounters.incrementClassicReplannedCounter();
     // We're going to start over with a new plan. Clear out info from our old plan.
     {
@@ -216,16 +229,16 @@ Status CachedPlanStage::replan(PlanYieldPolicy* yieldPolicy, bool shouldCache, s
 
     if (shouldCache) {
         // Deactivate the current cache entry.
-        const auto& coll = collection();
-        auto cache = CollectionQueryInfo::get(coll).getPlanCache();
-        cache->deactivate(plan_cache_key_factory::make<PlanCacheKey>(*_canonicalQuery, coll));
+        auto cache = CollectionQueryInfo::get(collectionPtr()).getPlanCache();
+        cache->deactivate(
+            plan_cache_key_factory::make<PlanCacheKey>(*_canonicalQuery, collectionPtr()));
     }
 
     // Use the query planning module to plan the whole query.
-    auto statusWithMultiPlanSolns = QueryPlanner::plan(*_canonicalQuery, _plannerParams);
+    auto statusWithMultiPlanSolns = QueryPlanner::plan(*_canonicalQuery, plannerParams);
     if (!statusWithMultiPlanSolns.isOK()) {
         return statusWithMultiPlanSolns.getStatus().withContext(
-            str::stream() << "error processing query: " << _canonicalQuery->toString()
+            str::stream() << "error processing query: " << _canonicalQuery->toStringForErrorMsg()
                           << " planner returned error");
     }
     auto solutions = std::move(statusWithMultiPlanSolns.getValue());
@@ -250,15 +263,22 @@ Status CachedPlanStage::replan(PlanYieldPolicy* yieldPolicy, bool shouldCache, s
         return Status::OK();
     }
 
-    // Many solutions. Create a MultiPlanStage to pick the best, update the cache,
-    // and so on. The working set will be shared by all candidate plans.
-    auto cachingMode = shouldCache ? PlanCachingMode::AlwaysCache : PlanCachingMode::NeverCache;
-    _children.emplace_back(
-        new MultiPlanStage(expCtx(), collection(), _canonicalQuery, cachingMode));
+    // Many solutions. Create a MultiPlanStage to pick the best, update the cache, and so on. The
+    // working set will be shared by all candidate plans.
+    _children.emplace_back(std::make_unique<MultiPlanStage>(
+        expCtx(),
+        collection(),
+        _canonicalQuery,
+        plan_cache_util::ConditionalClassicPlanCacheWriter{
+            plan_cache_util::ConditionalClassicPlanCacheWriter::alwaysOrNeverCacheMode(shouldCache),
+            opCtx(),
+            collection(),
+            false /* executeInSbe */},
+        _specificStats.replanReason));
     MultiPlanStage* multiPlanStage = static_cast<MultiPlanStage*>(child().get());
 
     for (size_t ix = 0; ix < solutions.size(); ++ix) {
-        solutions[ix]->indexFilterApplied = _plannerParams.indexFiltersApplied;
+        solutions[ix]->indexFilterApplied = plannerParams.indexFiltersApplied;
 
         auto&& nextPlanRoot = stage_builder::buildClassicExecutableTree(
             expCtx()->opCtx, collection(), *_canonicalQuery, *solutions[ix], _ws);
