@@ -27,26 +27,45 @@
  *    it in the license file.
  */
 
-#include "mongo/util/assert_util.h"
-
-#include "mongo/db/exec/collection_scan.h"
-
+#include <algorithm>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
 #include <memory>
+#include <utility>
+#include <vector>
 
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsontypes.h"
 #include "mongo/db/catalog/collection.h"
-#include "mongo/db/catalog/database.h"
+#include "mongo/db/client.h"
+#include "mongo/db/exec/collection_scan.h"
 #include "mongo/db/exec/collection_scan_common.h"
 #include "mongo/db/exec/filter.h"
-#include "mongo/db/exec/scoped_timer.h"
 #include "mongo/db/exec/working_set.h"
-#include "mongo/db/exec/working_set_common.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/query/plan_executor_impl.h"
 #include "mongo/db/query/query_knobs_gen.h"
-#include "mongo/db/record_id_helpers.h"
+#include "mongo/db/query/record_id_bound.h"
+#include "mongo/db/repl/oplog_entry.h"
+#include "mongo/db/repl/oplog_entry_gen.h"
 #include "mongo/db/repl/optime.h"
+#include "mongo/db/storage/record_data.h"
+#include "mongo/db/storage/recovery_unit.h"
+#include "mongo/db/transaction_resources.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/s/resharding/resharding_feature_flag_gen.h"
 #include "mongo/util/assert_util.h"
-#include "mongo/util/fail_point.h"
+#include "mongo/util/concurrency/admission_context.h"
+#include "mongo/util/str.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
@@ -57,18 +76,25 @@ MONGO_FAIL_POINT_DEFINE(hangCollScanDoWork);
 namespace mongo {
 
 using std::unique_ptr;
-using std::vector;
 
 namespace {
-const char* getStageName(const CollectionPtr& coll, const CollectionScanParams& params) {
-    return (!coll->ns().isOplog() && (params.minRecord || params.maxRecord)) ? "CLUSTERED_IXSCAN"
-                                                                             : "COLLSCAN";
+bool shouldIncludeStartRecord(const CollectionScanParams& params) {
+    return params.boundInclusion ==
+        CollectionScanParams::ScanBoundInclusion::kIncludeBothStartAndEndRecords ||
+        params.boundInclusion == CollectionScanParams::ScanBoundInclusion::kIncludeStartRecordOnly;
+}
+
+const char* getStageName(const VariantCollectionPtrOrAcquisition& coll,
+                         const CollectionScanParams& params) {
+    return (!coll.getCollectionPtr()->ns().isOplog() && (params.minRecord || params.maxRecord))
+        ? "CLUSTERED_IXSCAN"
+        : "COLLSCAN";
 }
 }  // namespace
 
 
 CollectionScan::CollectionScan(ExpressionContext* expCtx,
-                               const CollectionPtr& collection,
+                               VariantCollectionPtrOrAcquisition collection,
                                const CollectionScanParams& params,
                                WorkingSet* workingSet,
                                const MatchExpression* filter)
@@ -76,6 +102,7 @@ CollectionScan::CollectionScan(ExpressionContext* expCtx,
       _workingSet(workingSet),
       _filter((filter && !filter->isTriviallyTrue()) ? filter : nullptr),
       _params(params) {
+    const auto& collPtr = collection.getCollectionPtr();
     // Explain reports the direction of the collection scan.
     _specificStats.direction = params.direction;
     _specificStats.minRecord = params.minRecord;
@@ -84,14 +111,11 @@ CollectionScan::CollectionScan(ExpressionContext* expCtx,
     if (params.minRecord || params.maxRecord) {
         // The 'minRecord' and 'maxRecord' parameters are used for a special optimization that
         // applies only to forwards scans of the oplog and scans on clustered collections.
-        tassert(9049500,
-                "Cannot resume using '$_resumeAfter'/ '$_startAt' if 'minRecord' or 'maxRecord' is "
-                "used",
-                !params.resumeScanPoint);
-        if (collection->ns().isOplogOrChangeCollection()) {
+        invariant(!params.resumeAfterRecordId);
+        if (collPtr->ns().isOplogOrChangeCollection()) {
             invariant(params.direction == CollectionScanParams::FORWARD);
         } else {
-            invariant(collection->isClustered());
+            invariant(collPtr->isClustered());
         }
     }
 
@@ -101,7 +125,7 @@ CollectionScan::CollectionScan(ExpressionContext* expCtx,
         tassert(6125000,
                 "Only collection scans on clustered collections may specify recordId "
                 "BoundInclusion policies",
-                collection->isClustered());
+                collPtr->isClustered());
 
         if (filter) {
             // The filter is applied after the ScanBoundInclusion is considered.
@@ -115,12 +139,11 @@ CollectionScan::CollectionScan(ExpressionContext* expCtx,
     LOGV2_DEBUG(5400802,
                 5,
                 "collection scan bounds",
-                "min"_attr = (!_params.minRecord) ? "none" : redact(_params.minRecord->toString()),
-                "max"_attr = (!_params.maxRecord) ? "none" : redact(_params.maxRecord->toString()));
+                "min"_attr = (!_params.minRecord) ? "none" : _params.minRecord->toString(),
+                "max"_attr = (!_params.maxRecord) ? "none" : _params.maxRecord->toString());
     tassert(6521000,
             "Expected an oplog or a change collection with 'shouldTrackLatestOplogTimestamp'",
-            !_params.shouldTrackLatestOplogTimestamp ||
-                collection->ns().isOplogOrChangeCollection());
+            !_params.shouldTrackLatestOplogTimestamp || collPtr->ns().isOplogOrChangeCollection());
 
     if (params.assertTsHasNotFallenOff) {
         tassert(6521001,
@@ -131,15 +154,92 @@ CollectionScan::CollectionScan(ExpressionContext* expCtx,
                 params.direction == CollectionScanParams::FORWARD);
     }
 
-    if (params.resumeScanPoint) {
-        // The 'resumeScanPoint' parameter is used for resumable collection scans, which we
+    if (params.resumeAfterRecordId) {
+        // The 'resumeAfterRecordId' parameter is used for resumable collection scans, which we
         // only support in the forward direction.
         tassert(6521003,
-                "Expected forward collection scan with 'resumeScanPoint'",
+                "Expected forward collection scan with 'resumeAfterRecordId'",
                 params.direction == CollectionScanParams::FORWARD);
     }
 }
 
+namespace {
+
+/**
+ * Returns the first document (owned) in the collection using the cursor 'newCursor' assuming that
+ * the cursor has not been used and is unpositioned.
+ */
+BSONObj getFirstEntry(SeekableRecordCursor* newCursor) {
+    auto firstRecord = newCursor->next();
+    uassert(ErrorCodes::CollectionIsEmpty,
+            "Found collection empty when checking that the first record has not rolled over",
+            firstRecord);
+    auto entry = firstRecord->data.toBson().getOwned();
+
+    // If we use the cursor, unposition it so that it is ready for use by future callers.
+    newCursor->saveUnpositioned();
+    newCursor->restore();
+    return entry;
+};
+
+/**
+ * Asserts that the timestamp has not already fallen off the oplog or change collection and then
+ * returns an unpositioned cursor.
+ *
+ * Throws OplogQueryMinTsMissing if tsToCheck no longer exists in the oplog.
+ * Throws CollectionIsEmpty if the collection has no documents.
+ */
+std::unique_ptr<SeekableRecordCursor> initCursorAndAssertTsHasNotFallenOff(
+    OperationContext* opCtx, const CollectionPtr& coll, Timestamp tsToCheck) {
+    auto cursor = coll->getCursor(opCtx);
+    const auto firstEntry = getFirstEntry(cursor.get());
+    const repl::OplogEntryParserNonStrict oplogEntryParser{firstEntry};
+
+    // Indicates that 'firstEntry' means initialization of a replica set.
+    bool isNewRS{false};
+    try {
+        // Verify that the timestamp of the first observed oplog entry is earlier than or equal to
+        // timestamp that should not have fallen off the oplog.
+        if (oplogEntryParser.getOpTime().getTimestamp() <= tsToCheck) {
+            return cursor;
+        }
+
+        // If the first entry we see in the oplog is the replset initialization, then it doesn't
+        // matter if its timestamp is later than the timestamp that should not have fallen off the
+        // oplog; no events earlier can have fallen off this oplog.
+        // NOTE: A change collection can be created at any moment as such it might not have replset
+        // initialization message, as such this case is not fully applicable for the change
+        // collection.
+        isNewRS = oplogEntryParser.getOpType() == repl::OpTypeEnum::kNoop &&
+            oplogEntryParser.getObject().binaryEqual(BSON("msg" << repl::kInitiatingSetMsg));
+    } catch (const AssertionException& exception) {
+        uasserted(8881102,
+                  str::stream() << "Failed to parse the oldest oplog entry" << causedBy(exception));
+    }
+    uassert(ErrorCodes::OplogQueryMinTsMissing,
+            str::stream()
+                << "Specified timestamp has already fallen off the oplog for the input timestamp: "
+                << tsToCheck << ", first oplog entry: " << firstEntry.toString(),
+            isNewRS);
+    return cursor;
+}
+}  // namespace
+
+void CollectionScan::initCursor(OperationContext* opCtx,
+                                const CollectionPtr& collPtr,
+                                bool forward) {
+    if (_params.assertTsHasNotFallenOff) {
+        invariant(forward);
+        _cursor =
+            initCursorAndAssertTsHasNotFallenOff(opCtx, collPtr, *_params.assertTsHasNotFallenOff);
+
+        // We don't need to check this assertion again after we've confirmed the first oplog event.
+        _params.assertTsHasNotFallenOff = boost::none;
+    } else {
+        _cursor = collPtr->getCursor(opCtx, forward);
+    }
+}
+
 PlanStage::StageState CollectionScan::doWork(WorkingSetID* out) {
     if (MONGO_unlikely(hangCollScanDoWork.shouldFail())) {
         hangCollScanDoWork.pauseWhileSet();
@@ -152,17 +252,17 @@ PlanStage::StageState CollectionScan::doWork(WorkingSetID* out) {
 
     if (_params.lowPriority && !_priority && gDeprioritizeUnboundedUserCollectionScans.load() &&
         opCtx()->getClient()->isFromUserConnection() &&
-        opCtx()->lockState()->shouldWaitForTicket()) {
-        _priority.emplace(opCtx()->lockState(), AdmissionContext::Priority::kLow);
+        shard_role_details::getLocker(opCtx())->shouldWaitForTicket(opCtx())) {
+        _priority.emplace(opCtx(), AdmissionContext::Priority::kLow);
     }
 
     boost::optional<Record> record;
     const bool needToMakeCursor = !_cursor;
+    const auto& collPtr = collectionPtr();
 
     const auto ret = handlePlanStageYield(
         expCtx(),
         "CollectionScan",
-        collection()->ns().ns(),
         [&] {
             if (needToMakeCursor) {
                 const bool forward = _params.direction == CollectionScanParams::FORWARD;
@@ -179,14 +279,18 @@ PlanStage::StageState CollectionScan::doWork(WorkingSetID* out) {
                     // sure that we are using a fresh storage engine snapshot while waiting.
                     // Otherwise, we will end up reading from the snapshot where the oplog entries
                     // are not yet visible even after the wait.
-                    invariant(!_params.tailable && collection()->ns().isOplog());
+                    invariant(!_params.tailable && collPtr->ns().isOplog());
 
-                    opCtx()->recoveryUnit()->abandonSnapshot();
-                    collection()->getRecordStore()->waitForAllEarlierOplogWritesToBeVisible(
-                        opCtx());
+                    shard_role_details::getRecoveryUnit(opCtx())->abandonSnapshot();
+                    collPtr->getRecordStore()->waitForAllEarlierOplogWritesToBeVisible(opCtx());
                 }
 
-                _cursor = collection()->getCursor(opCtx(), forward);
+                try {
+                    initCursor(opCtx(), collPtr, forward);
+                } catch (const ExceptionFor<ErrorCodes::CollectionIsEmpty>&) {
+                    _commonStats.isEOF = true;
+                    return PlanStage::IS_EOF;
+                }
 
                 if (!_lastSeenId.isNull()) {
                     invariant(_params.tailable);
@@ -206,64 +310,46 @@ PlanStage::StageState CollectionScan::doWork(WorkingSetID* out) {
                     }
                 }
 
-                if (_params.resumeScanPoint && !_params.resumeScanPoint->recordId.isNull()) {
+                if (_params.resumeAfterRecordId) {
                     invariant(!_params.tailable);
                     invariant(_lastSeenId.isNull());
-                    // Seek to where we are trying to resume the scan from.
+                    // Seek to where we are trying to resume the scan from. Signal a KeyNotFound
+                    // error if the record no longer exists or if the recordId is null.
                     //
                     // Note that we want to return the record *after* this one since we have already
-                    // returned this one prior to the resume *unless* we are resuming from a deleted
-                    // record id in which case the cursor is already positioned on the next record
-                    // id.
-                    auto& resumeScanPoint = *_params.resumeScanPoint;
-                    auto& recordIdToSeek = resumeScanPoint.recordId;
-
-                    // Attempt to seek to the exact recordId. If it doesn't exist and we're using
-                    // '$_startAt' (tolerateKeyNotFound = true), we'll use seekNear() instead to
-                    // find the next highest recordId and return. In this case the cursor is already
-                    // positioned on the recordId we want to return so we don't always need to
-                    // advance it again below. If tolerateKeyNotFound = false, we throw.
-                    auto testRecord = _cursor->seekExact(recordIdToSeek);
-                    if (!testRecord) {
-                        if (resumeScanPoint.tolerateKeyNotFound) {
-                            // seekNear will by default return the greatest record id that is less
-                            // than or equal to 'recordIdToSeek' if one exists. In this case we're
-                            // interested in the next record id after 'recordIdToSeek' so we advance
-                            // the cursor.
-                            record = _cursor->seekNear(recordIdToSeek);
-                            if (record && record->id < recordIdToSeek) {
-                                record = _cursor->next();
-                            }
-                            return PlanStage::ADVANCED;
-                        } else {
-                            uasserted(
-                                ErrorCodes::KeyNotFound,
-                                str::stream()
-                                    << "Failed to resume collection scan: the recordId from "
-                                       "which we are "
-                                    << "attempting to resume no longer exists in the collection. "
-                                    << "recordId: " << recordIdToSeek);
-                        }
+                    // returned this one prior to the resume.
+                    auto& recordIdToSeek = *_params.resumeAfterRecordId;
+                    if (recordIdToSeek.isNull() || !_cursor->seekExact(recordIdToSeek)) {
+                        uasserted(ErrorCodes::KeyNotFound,
+                                  str::stream()
+                                      << "Failed to resume collection scan: the recordId from "
+                                         "which we are attempting to resume no longer exists in "
+                                         "the collection: "
+                                      << recordIdToSeek);
                     }
                 }
+
+                if (_lastSeenId.isNull() && _params.direction == CollectionScanParams::FORWARD &&
+                    _params.minRecord) {
+                    // Seek to the start location and return it.
+                    record = _cursor->seek(_params.minRecord->recordId(),
+                                           shouldIncludeStartRecord(_params)
+                                               ? SeekableRecordCursor::BoundInclusion::kInclude
+                                               : SeekableRecordCursor::BoundInclusion::kExclude);
+                    return PlanStage::ADVANCED;
+                } else if (_lastSeenId.isNull() &&
+                           _params.direction == CollectionScanParams::BACKWARD &&
+                           _params.maxRecord) {
+                    // Seek to the start location and return it.
+                    record = _cursor->seek(_params.maxRecord->recordId(),
+                                           shouldIncludeStartRecord(_params)
+                                               ? SeekableRecordCursor::BoundInclusion::kInclude
+                                               : SeekableRecordCursor::BoundInclusion::kExclude);
+                    return PlanStage::ADVANCED;
+                }
             }
 
-            if (_lastSeenId.isNull() && _params.direction == CollectionScanParams::FORWARD &&
-                _params.minRecord) {
-                // Seek to the approximate start location.
-                record = _cursor->seekNear(_params.minRecord->recordId());
-            }
-
-            if (_lastSeenId.isNull() && _params.direction == CollectionScanParams::BACKWARD &&
-                _params.maxRecord) {
-                // Seek to the approximate start location (at the end).
-                record = _cursor->seekNear(_params.maxRecord->recordId());
-            }
-
-            if (!record) {
-                record = _cursor->next();
-            }
-
+            record = _cursor->next();
             return PlanStage::ADVANCED;
         },
         [&] {
@@ -279,9 +365,9 @@ PlanStage::StageState CollectionScan::doWork(WorkingSetID* out) {
     }
 
     if (!record) {
-        // We hit EOF. If we are tailable and have already seen data, leave us in a state to pick up
-        // where we left off on the next call to work(). Otherwise, the EOF is permanent.
-        if (_params.tailable && !_lastSeenId.isNull()) {
+        // We hit EOF. If we are tailable, leave us in a state to pick up where we left off on the
+        // next call to work(). Otherwise, the EOF is permanent.
+        if (_params.tailable) {
             _cursor.reset();
         } else {
             _commonStats.isEOF = true;
@@ -289,7 +375,7 @@ PlanStage::StageState CollectionScan::doWork(WorkingSetID* out) {
 
         // For change collections, advance '_latestOplogEntryTimestamp' to the current snapshot
         // timestamp, i.e. the latest available timestamp in the global oplog.
-        if (_params.shouldTrackLatestOplogTimestamp && collection()->ns().isChangeCollection()) {
+        if (_params.shouldTrackLatestOplogTimestamp && collPtr->ns().isChangeCollection()) {
             setLatestOplogEntryTimestampToReadTimestamp();
         }
         _priority.reset();
@@ -297,9 +383,6 @@ PlanStage::StageState CollectionScan::doWork(WorkingSetID* out) {
     }
 
     _lastSeenId = record->id;
-    if (_params.assertTsHasNotFallenOff) {
-        assertTsHasNotFallenOff(*record);
-    }
     if (_params.shouldTrackLatestOplogTimestamp) {
         setLatestOplogEntryTimestamp(*record);
     }
@@ -307,14 +390,23 @@ PlanStage::StageState CollectionScan::doWork(WorkingSetID* out) {
     WorkingSetID id = _workingSet->allocate();
     WorkingSetMember* member = _workingSet->get(id);
     member->recordId = std::move(record->id);
-    member->resetDocument(opCtx()->recoveryUnit()->getSnapshotId(), record->data.releaseToBson());
+    member->resetDocument(shard_role_details::getRecoveryUnit(opCtx())->getSnapshotId(),
+                          record->data.releaseToBson());
     _workingSet->transitionToRecordIdAndObj(id);
 
     return returnIfMatches(member, id, out);
 }
 
 void CollectionScan::setLatestOplogEntryTimestampToReadTimestamp() {
-    const auto readTimestamp = opCtx()->recoveryUnit()->getPointInTimeReadTimestamp(opCtx());
+    // Since this method is only ever called when iterating a change collection, the following check
+    // effectively disables optime advancement in Serverless, for reasons outlined in SERVER-76288.
+    // TODO SERVER-76309: re-enable optime advancement to support sharding in Serverless.
+    if (collectionPtr()->ns().isChangeCollection()) {
+        return;
+    }
+
+    const auto readTimestamp =
+        shard_role_details::getRecoveryUnit(opCtx())->getPointInTimeReadTimestamp(opCtx());
 
     // If we don't have a read timestamp, we take no action here.
     if (!readTimestamp) {
@@ -346,50 +438,30 @@ void CollectionScan::setLatestOplogEntryTimestamp(const Record& record) {
     _latestOplogEntryTimestamp = std::max(_latestOplogEntryTimestamp, tsElem.timestamp());
 }
 
-void CollectionScan::assertTsHasNotFallenOff(const Record& record) {
-    const auto oplogEntry = record.data.toBson();
-    const repl::OplogEntryParserNonStrict oplogEntryParser{oplogEntry};
-    invariant(_specificStats.docsTested == 0);
-
-    // Indicates that 'oplogEntry' means initialization of a replica set.
-    bool isNewRS{false};
-
-    // Indicates that the timestamp of the observed oplog entry 'oplogEntry' is earlier than or
-    // equal to timestamp that should not have fallen off the oplog.
-    bool tsHasNotFallenOff{false};
-    try {
-        tsHasNotFallenOff =
-            oplogEntryParser.getOpTime().getTimestamp() <= *_params.assertTsHasNotFallenOff;
-
-        // If the first entry we see in the oplog is the replset initialization, then it doesn't
-        // matter if its timestamp is later than the timestamp that should not have fallen off the
-        // oplog; no events earlier can have fallen off this oplog.
-        // NOTE: A change collection can be created at any moment as such it might not have replset
-        // initialization message, as such this case is not fully applicable for the change
-        // collection.
-        isNewRS = oplogEntryParser.getOpType() == repl::OpTypeEnum::kNoop &&
-            oplogEntryParser.getObject().binaryEqual(BSON("msg" << repl::kInitiatingSetMsg));
-    } catch (const AssertionException& exception) {
-        uasserted(8881102,
-                  str::stream() << "Failed to parse the oldest oplog entry" << causedBy(exception));
+BSONObj CollectionScan::getPostBatchResumeToken() const {
+    // Return a resume token compatible with resumable initial sync.
+    if (_params.requestResumeToken) {
+        BSONObjBuilder builder;
+        _lastSeenId.serializeToken("$recordId", &builder);
+        if (resharding::gFeatureFlagReshardingImprovements.isEnabled(
+                serverGlobalParams.featureCompatibility.acquireFCVSnapshot())) {
+            auto initialSyncId =
+                repl::ReplicationCoordinator::get(opCtx())->getInitialSyncId(opCtx());
+            if (initialSyncId) {
+                initialSyncId.value().appendToBuilder(&builder, "$initialSyncId");
+            }
+        }
+        return builder.obj();
+    }
+    // Return a resume token compatible with resharding oplog sync.
+    if (_params.shouldTrackLatestOplogTimestamp) {
+        return ResumeTokenOplogTimestamp{_latestOplogEntryTimestamp}.toBSON();
     }
-    uassert(ErrorCodes::OplogQueryMinTsMissing,
-            str::stream()
-                << "Specified timestamp has already fallen off the oplog for the input timestamp: "
-                << *_params.assertTsHasNotFallenOff << ", oplog entry: " << oplogEntry.toString(),
-            isNewRS || tsHasNotFallenOff);
 
-    // We don't need to check this assertion again after we've confirmed the first oplog event.
-    _params.assertTsHasNotFallenOff = boost::none;
+    return {};
 }
 
 namespace {
-bool shouldIncludeStartRecord(const CollectionScanParams& params) {
-    return params.boundInclusion ==
-        CollectionScanParams::ScanBoundInclusion::kIncludeBothStartAndEndRecords ||
-        params.boundInclusion == CollectionScanParams::ScanBoundInclusion::kIncludeStartRecordOnly;
-}
-
 bool shouldIncludeEndRecord(const CollectionScanParams& params) {
     return params.boundInclusion ==
         CollectionScanParams::ScanBoundInclusion::kIncludeBothStartAndEndRecords ||
@@ -445,8 +517,8 @@ PlanStage::StageState CollectionScan::returnIfMatches(WorkingSetMember* member,
                                                       WorkingSetID* out) {
     ++_specificStats.docsTested;
 
-    // The 'minRecord' and 'maxRecord' bounds are always inclusive, even if the query predicate is
-    // an exclusive inequality like $gt or $lt. In such cases, we rely on '_filter' to either
+    // The 'maxRecord' bound is always inclusive, even if the query predicate is
+    // an exclusive inequality like $lt. In such cases, we rely on '_filter' to either
     // exclude or include the endpoints as required by the user's query.
     if (pastEndOfRange(_params, *member)) {
         _workingSet->free(memberID);
@@ -454,19 +526,6 @@ PlanStage::StageState CollectionScan::returnIfMatches(WorkingSetMember* member,
         return PlanStage::IS_EOF;
     }
 
-    // For clustered collections, seekNear() is allowed to return a record prior to the
-    // requested minRecord for a forward scan or after the requested maxRecord for a reverse
-    // scan. Ensure that we do not return a record out of the requested range. Require that the
-    // caller advance our cursor until it is positioned within the correct range.
-    //
-    // In the future, we could change seekNear() to always return a record after minRecord in the
-    // direction of the scan. However, tailable scans depend on the current behavior in order to
-    // mark their position for resuming the tailable scan later on.
-    if (beforeStartOfRange(_params, *member)) {
-        _workingSet->free(memberID);
-        return PlanStage::NEED_TIME;
-    }
-
     if (!Filter::passes(member, _filter)) {
         _workingSet->free(memberID);
         if (_params.shouldReturnEofOnFilterMismatch) {
@@ -519,8 +578,8 @@ void CollectionScan::doDetachFromOperationContext() {
 void CollectionScan::doReattachToOperationContext() {
     if (_params.lowPriority && gDeprioritizeUnboundedUserCollectionScans.load() &&
         opCtx()->getClient()->isFromUserConnection() &&
-        opCtx()->lockState()->shouldWaitForTicket()) {
-        _priority.emplace(opCtx()->lockState(), AdmissionContext::Priority::kLow);
+        shard_role_details::getLocker(opCtx())->shouldWaitForTicket(opCtx())) {
+        _priority.emplace(opCtx(), AdmissionContext::Priority::kLow);
     }
     if (_cursor)
         _cursor->reattachToOperationContext(opCtx());
