@@ -29,12 +29,21 @@
 
 #include "mongo/db/exec/document_value/document.h"
 
-#include <boost/functional/hash.hpp>
+#include <absl/container/node_hash_map.h>
+#include <boost/container_hash/extensions.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <cstdint>
+#include <memory>
 
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+
+#include "mongo/base/data_type_endian.h"
+#include "mongo/base/error_codes.h"
 #include "mongo/bson/bson_depth.h"
-#include "mongo/db/jsobj.h"
+#include "mongo/bson/util/builder_fwd.h"
 #include "mongo/db/pipeline/field_path.h"
-#include "mongo/db/pipeline/resume_token.h"
 #include "mongo/util/str.h"
 
 namespace mongo {
@@ -57,29 +66,32 @@ void assertFieldPathLengthOK(const std::vector<Position>& path) {
             path.size() < BSONDepth::getMaxAllowableDepth());
 }
 
-stdx::variant<BSONElement, Value, Document::TraversesArrayTag, stdx::monostate>
-getNestedFieldHelperBSON(BSONElement elt, const FieldPath& fp, size_t level) {
+/**
+ * Returns the BSONElement for path 'fp' and current nesting level.
+ *
+ * Returns EOO if the path does not exist or boost::none if an array is encountered along the path
+ * (but not at the end of the path).
+ */
+boost::optional<BSONElement> getNestedFieldHelperBSON(BSONElement elt,
+                                                      const FieldPath& fp,
+                                                      size_t level) {
     if (level == fp.getPathLength()) {
-        if (elt.ok()) {
-            return elt;
-        } else {
-            return stdx::monostate{};
-        }
+        return elt;
     }
 
     if (elt.type() == BSONType::Array) {
-        return Document::TraversesArrayTag{};
+        return boost::none;
     } else if (elt.type() == BSONType::Object) {
         auto subFieldElt = elt.embeddedObject()[fp.getFieldName(level)];
         return getNestedFieldHelperBSON(subFieldElt, fp, level + 1);
     }
 
     // The path continues "past" a scalar, and therefore does not exist.
-    return stdx::monostate{};
+    return BSONElement();
 }
 }  // namespace
 
-const DocumentStorage DocumentStorage::kEmptyDoc;
+const DocumentStorage DocumentStorage::kEmptyDoc{ConstructorTag::InitApproximateSize};
 
 const StringDataSet Document::allMetadataFieldNames{Document::metaFieldTextScore,
                                                     Document::metaFieldRandVal,
@@ -133,7 +145,7 @@ bool DocumentStorageIterator::shouldSkipDeleted() {
 
         // If we strip the metadata see if a field name matches the known list. All metadata fields
         // start with '$' so optimize for a quick bailout.
-        if (_storage->bsonHasMetadata() && !fieldName.empty() && fieldName[0] == '$' &&
+        if (_storage->bsonHasMetadata() && fieldName.starts_with('$') &&
             Document::allMetadataFieldNames.contains(fieldName)) {
             return true;
         }
@@ -294,9 +306,10 @@ void DocumentStorage::addFieldToHashTable(T field, Position pos) {
 }
 
 void DocumentStorage::alloc(unsigned newSize) {
+    const auto oldCapacity = allocatedBytes();
     const bool firstAlloc = !_cache;
     const bool doingRehash = needRehash();
-    const size_t oldCapacity = _cacheEnd - _cache;
+    const size_t oldSize = _cacheEnd - _cache;
 
     // make new bucket count big enough
     while (needRehash() || hashTabBuckets() < HASH_TAB_INIT_SIZE)
@@ -309,13 +322,18 @@ void DocumentStorage::alloc(unsigned newSize) {
 
     uassert(16490, "Tried to make oversized document", capacity <= size_t(BufferMaxSize));
 
-    std::unique_ptr<char[]> oldBuf(_cache);
-    _cache = new char[capacity];
+    auto oldCache = _cache;
+    ScopeGuard deleteOldCache([oldCache, oldCapacity] {
+        if (oldCache) {
+            ::operator delete(oldCache, oldCapacity);
+        }
+    });
+    _cache = static_cast<char*>(::operator new(capacity));
     _cacheEnd = _cache + capacity - hashTabBytes();
 
     if (!firstAlloc) {
         // This just copies the elements
-        memcpy(_cache, oldBuf.get(), _usedBytes);
+        memcpy(_cache, oldCache, _usedBytes);
 
         if (_numFields >= HASH_TAB_MIN) {
             // if we were hashing, deal with the hash table
@@ -323,7 +341,7 @@ void DocumentStorage::alloc(unsigned newSize) {
                 rehash();
             } else {
                 // no rehash needed so just slide table down to new position
-                memcpy(_hashTab, oldBuf.get() + oldCapacity, hashTabBytes());
+                memcpy(_hashTab, oldCache + oldSize, hashTabBytes());
             }
         }
     }
@@ -342,7 +360,7 @@ void DocumentStorage::reserveFields(size_t expectedFields) {
 
     uassert(16491, "Tried to make oversized document", newSize <= size_t(BufferMaxSize));
 
-    _cache = new char[newSize + hashTabBytes()];
+    _cache = static_cast<char*>(::operator new(newSize + hashTabBytes()));
     _cacheEnd = _cache + newSize;
 }
 
@@ -354,7 +372,7 @@ intrusive_ptr<DocumentStorage> DocumentStorage::clone() const {
         // Make a copy of the buffer with the fields.
         // It is very important that the positions of each field are the same after cloning.
         const size_t bufferBytes = allocatedBytes();
-        out->_cache = new char[bufferBytes];
+        out->_cache = static_cast<char*>(::operator new(bufferBytes));
         out->_cacheEnd = out->_cache + (_cacheEnd - _cache);
         memcpy(out->_cache, _cache, bufferBytes);
 
@@ -387,11 +405,12 @@ size_t DocumentStorage::getMetadataApproximateSize() const {
 }
 
 DocumentStorage::~DocumentStorage() {
-    std::unique_ptr<char[]> deleteBufferAtScopeEnd(_cache);
-
     for (auto it = iteratorCacheOnly(); !it.atEnd(); it.advance()) {
         it->val.~Value();  // explicit destructor call
     }
+    if (_cache) {
+        ::operator delete(_cache, allocatedBytes());
+    }
 }
 
 void DocumentStorage::reset(const BSONObj& bson, bool bsonHasMetadata) {
@@ -406,13 +425,17 @@ void DocumentStorage::reset(const BSONObj& bson, bool bsonHasMetadata) {
         it->val.~Value();  // explicit destructor call
     }
 
-    _cacheEnd = _cache;
+    if (_cache) {
+        ::operator delete(_cache, allocatedBytes());
+    }
+    _cacheEnd = _cache = nullptr;
     _usedBytes = 0;
     _numFields = 0;
     _hashTabMask = 0;
 
     // Clean metadata.
     _metadataFields = DocumentMetadataFields{};
+    _metadataFields.setModified(false);
 }
 
 Document DocumentStorage::shred() const {
@@ -450,7 +473,7 @@ void DocumentStorage::loadLazyMetadata() const {
     while (it.more()) {
         BSONElement elem(it.next());
         auto fieldName = elem.fieldNameStringData();
-        if (!fieldName.empty() && fieldName[0] == '$') {
+        if (fieldName.starts_with('$')) {
             if (fieldName == Document::metaFieldTextScore) {
                 _metadataFields.setTextScore(elem.Double());
             } else if (fieldName == Document::metaFieldSearchScore) {
@@ -502,7 +525,7 @@ void DocumentStorage::loadLazyMetadata() const {
 
 Document::Document(const BSONObj& bson) {
     MutableDocument md;
-    md.newStorageWithBson(bson, false);
+    md.reset(bson, false);
 
     *this = md.freeze();
 }
@@ -603,7 +626,7 @@ void Document::toBsonWithMetaData(BSONObjBuilder* builder) const {
 
 Document Document::fromBsonWithMetaData(const BSONObj& bson) {
     MutableDocument md;
-    md.newStorageWithBson(bson, true);
+    md.reset(bson, true);
 
     return md.freeze();
 }
@@ -667,43 +690,59 @@ MutableValue MutableDocument::getNestedField(const vector<Position>& positions)
 }
 
 
-stdx::variant<BSONElement, Value, Document::TraversesArrayTag, stdx::monostate>
-Document::getNestedFieldNonCachingHelper(const FieldPath& dottedField, size_t level) const {
+boost::optional<Value> Document::getNestedScalarFieldNonCachingHelper(const FieldPath& dottedField,
+                                                                      size_t level) const {
     if (!_storage) {
-        return stdx::monostate{};
+        return Value();
     }
 
     StringData fieldName = dottedField.getFieldName(level);
-    auto bsonEltOrValue = _storage->getFieldNonCaching(fieldName);
 
-    if (stdx::holds_alternative<BSONElement>(bsonEltOrValue)) {
-        return getNestedFieldHelperBSON(
-            stdx::get<BSONElement>(bsonEltOrValue), dottedField, level + 1);
-    }
+    // In many cases, the cache is empty and we can skip straight to reading from the backing BSON.
+    if (isModified()) {
+        if (auto val = _storage->getFieldCacheOnly(fieldName); val) {
+            // Whether landing on an array (level + 1 == dottedField.getPathLength) or traversing an
+            // array, return boost::none.
+            if (val->getType() == BSONType::Array)
+                return boost::none;
 
-    const Value& val = stdx::get<Value>(bsonEltOrValue);
+            if (level + 1 == dottedField.getPathLength()) {
+                return val;
+            }
 
-    if (level + 1 == dottedField.getPathLength()) {
-        if (val.missing()) {
-            return stdx::monostate{};
-        } else {
-            return val;
+            if (val->getType() == BSONType::Object) {
+                return val->getDocument().getNestedScalarFieldNonCachingHelper(dottedField,
+                                                                               level + 1);
+            }
+
+            // Returns missing when reading the sub-field of a scalar.
+            return Value();
         }
     }
 
-    if (val.getType() == BSONType::Array) {
-        return Document::TraversesArrayTag{};
-    } else if (val.getType() == BSONType::Object) {
-        return val.getDocument().getNestedFieldNonCachingHelper(dottedField, level + 1);
+    // Either the value does not exist in the cache or the cache is empty so the above block is
+    // skipped, now check the backing BSON.
+    if (auto bsonElt = _storage->getFieldBsonOnly(fieldName); !bsonElt.eoo()) {
+        auto maybeBsonElt = getNestedFieldHelperBSON(bsonElt, dottedField, level + 1);
+        // Take care to avoid needlessly constructing a Value. There are 4 possible states for
+        // 'maybeBsonElt':
+        // 1. Scalar BSONElement --> coerce to Value and return it.
+        // 2. Array BSONElement --> return boost::none per this function's contract.
+        // 3. BSONElement::eoo --> path does not exist, so return an empty Value via
+        // Value(BSONElement::eoo).
+        // 4. boost::none --> encountered an array along the path, return boost::none.
+        if (maybeBsonElt && maybeBsonElt->type() != BSONType::Array)
+            return Value(*maybeBsonElt);
+        return boost::none;
     }
 
-    // The path extends beyond a scalar, so it does not exist.
-    return stdx::monostate{};
+    // Path does not exist.
+    return Value();
 }
 
-stdx::variant<BSONElement, Value, Document::TraversesArrayTag, stdx::monostate>
-Document::getNestedFieldNonCaching(const FieldPath& dottedField) const {
-    return getNestedFieldNonCachingHelper(dottedField, 0);
+boost::optional<Value> Document::getNestedScalarFieldNonCaching(
+    const FieldPath& dottedField) const {
+    return getNestedScalarFieldNonCachingHelper(dottedField, 0);
 }
 
 static Value getNestedFieldHelper(const Document& doc,
@@ -748,8 +787,7 @@ size_t Document::memUsageForSorter() const {
         storage().nonCachedBsonObjSize();
 }
 
-void Document::hash_combine(size_t& seed,
-                            const StringData::ComparatorInterface* stringComparator) const {
+void Document::hash_combine(size_t& seed, const StringDataComparator* stringComparator) const {
     for (DocumentStorageIterator it = storage().iterator(); !it.atEnd(); it.advance()) {
         StringData name = it->nameSD();
         boost::hash_range(seed, name.rawData(), name.rawData() + name.size());
@@ -759,7 +797,7 @@ void Document::hash_combine(size_t& seed,
 
 int Document::compare(const Document& rL,
                       const Document& rR,
-                      const StringData::ComparatorInterface* stringComparator) {
+                      const StringDataComparator* stringComparator) {
 
     if (&rL.storage() == &rR.storage()) {
         // If the storage is the same (shared between the documents) then the documents must be
