@@ -29,23 +29,45 @@
 
 #pragma once
 
-#include "mongo/db/exec/document_value/document_internal.h"
-
 #include <boost/functional/hash.hpp>
 #include <boost/intrusive_ptr.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <cstring>
+#include <initializer_list>
+#include <iosfwd>
+#include <string>
+#include <typeinfo>
+#include <utility>
+#include <variant>
+#include <vector>
 
 #include "mongo/base/string_data.h"
-#include "mongo/base/string_data_comparator_interface.h"
+#include "mongo/base/string_data_comparator.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
 #include "mongo/bson/util/builder.h"
+#include "mongo/db/exec/document_value/document_internal.h"
+#include "mongo/db/exec/document_value/document_metadata_fields.h"
+#include "mongo/db/exec/document_value/value.h"
+#include "mongo/db/exec/document_value/value_internal.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/bufreader.h"
+#include "mongo/util/intrusive_counter.h"
 #include "mongo/util/string_map.h"
 
 namespace mongo {
 class BSONObj;
+
 class FieldIterator;
 class FieldPath;
 class Value;
 class MutableDocument;
-
 /** An internal class that represents the position of a field in a document.
  *
  *  This is a low-level class that you usually don't need to worry about.
@@ -161,19 +183,12 @@ public:
     Value getNestedField(const FieldPath& path, std::vector<Position>* positions = nullptr) const;
 
     /**
-     * Returns field at given path as either BSONElement or Value, depending on how it is
-     * stored. If an array is encountered in the middle of the path the TraversesArrayTag is
-     * returned.
+     * Returns field at given path coerced to a Value. If an array is encountered in the middle of
+     * the path or at the leaf then boost::none is returned.
      *
-     * It is possible, however, for the returned BSONElement/Value to be an array if the given path
-     * ends with an array. For example, the document {a: {b:[1,2]}} and the path "a.b" will return
-     * a BSONElement or Value for the array [1, 2].
-     *
-     * If the field is not found, stdx::monostate is returned.
+     * If the field is not found, an empty Value() is returned.
      */
-    struct TraversesArrayTag {};
-    stdx::variant<BSONElement, Value, TraversesArrayTag, stdx::monostate> getNestedFieldNonCaching(
-        const FieldPath& dottedField) const;
+    boost::optional<Value> getNestedScalarFieldNonCaching(const FieldPath& dottedField) const;
 
     // Number of fields in this document. Exp. runtime O(n).
     size_t computeSize() const {
@@ -225,7 +240,7 @@ public:
      *  as strings are compared, but comparing one field at a time instead
      *  of one character at a time.
      *
-     *  Pass a non-null StringData::ComparatorInterface if special string comparison semantics are
+     *  Pass a non-null StringDataComparator if special string comparison semantics are
      *  required. If the comparator is null, then a simple binary compare is used for strings. This
      *  comparator is only used for string *values*; field names are always compared using simple
      *  binary compare.
@@ -238,7 +253,7 @@ public:
      */
     static int compare(const Document& lhs,
                        const Document& rhs,
-                       const StringData::ComparatorInterface* stringComparator);
+                       const StringDataComparator* stringComparator);
 
     std::string toString() const;
 
@@ -265,7 +280,7 @@ public:
      * Meant to be used to create composite hashes suitable for
      * hashed container classes such as unordered_map.
      */
-    void hash_combine(size_t& seed, const StringData::ComparatorInterface* stringComparator) const;
+    void hash_combine(size_t& seed, const StringDataComparator* stringComparator) const;
 
     /**
      * Returns true, if this document is trivially convertible to BSON, meaning the underlying
@@ -425,8 +440,8 @@ private:
         return (_storage ? *_storage : DocumentStorage::emptyDoc());
     }
 
-    stdx::variant<BSONElement, Value, TraversesArrayTag, stdx::monostate>
-    getNestedFieldNonCachingHelper(const FieldPath& dottedField, size_t level) const;
+    boost::optional<Value> getNestedScalarFieldNonCachingHelper(const FieldPath& dottedField,
+                                                                size_t level) const;
 
     boost::intrusive_ptr<const DocumentStorage> _storage;
 };
@@ -751,16 +766,6 @@ public:
         storage().makeOwned();
     }
 
-    /**
-     * Creates a new document storage with the BSON object. Setting 'bsonHasMetadata' to true
-     * signals that the BSON object contains metadata fields (the complete list is in
-     * Document::allMetadataFieldNames).
-     */
-    DocumentStorage& newStorageWithBson(const BSONObj& bson, bool bsonHasMetadata) {
-        reset(make_intrusive<DocumentStorage>(bson, bsonHasMetadata, false, 0));
-        return const_cast<DocumentStorage&>(*storagePtr());
-    }
-
 private:
     friend class MutableValue;  // for access to next constructor
     explicit MutableDocument(MutableValue mv) : _storageHolder(nullptr), _storage(mv.getDocPtr()) {}
