@@ -31,11 +31,11 @@
 
 #include <bitset>
 #include <boost/intrusive_ptr.hpp>
+#include <variant>
 
 #include "mongo/base/static_assert.h"
 #include "mongo/db/exec/document_value/document_metadata_fields.h"
 #include "mongo/db/exec/document_value/value.h"
-#include "mongo/stdx/variant.h"
 #include "mongo/util/intrusive_counter.h"
 
 namespace mongo {
@@ -373,7 +373,7 @@ public:
           _bsonHasMetadata(bsonHasMetadata),
           _modified(modified) {}
 
-    ~DocumentStorage();
+    ~DocumentStorage() override;
 
     void reset(const BSONObj& bson, bool bsonHasMetadata);
 
@@ -452,23 +452,26 @@ public:
     }
 
     /**
-     * Given a field name either return a Value if the field resides in the cache, or a BSONElement
-     * if the field resides in the backing BSON.
+     * Retrieves the given field from the cache. Returns a boost::none if the field does not exist.
      */
-    stdx::variant<BSONElement, Value> getFieldNonCaching(StringData name) const {
+    boost::optional<Value> getFieldCacheOnly(StringData name) const {
         Position pos = findField(name, LookupPolicy::kCacheOnly);
         if (pos.found()) {
-            return {getField(pos).val};
+            return getField(pos).val;
         }
+        return boost::none;
+    }
 
+    /**
+     * Retrieves the given field from the backing BSON. Returns an EOO if the field does not exist.
+     */
+    BSONElement getFieldBsonOnly(StringData name) const {
         for (auto&& bsonElement : _bson) {
             if (name == bsonElement.fieldNameStringData()) {
-                return {bsonElement};
+                return bsonElement;
             }
         }
-
-        // Field not found. Return EOO Value.
-        return {Value()};
+        return BSONElement();
     }
 
     /// Adds a new field with missing Value at the end of the document
@@ -633,6 +636,16 @@ public:
     }
 
 private:
+    enum class ConstructorTag { InitApproximateSize = 0 };
+    DocumentStorage(ConstructorTag tag) : DocumentStorage() {
+        switch (tag) {
+            case ConstructorTag::InitApproximateSize:
+                snapshottedApproximateSize();
+                return;
+        }
+        MONGO_UNREACHABLE;
+    }
+
     /// Returns the position of the named field in the cache or Position()
     template <typename T>
     Position findFieldInCache(T name) const;
