@@ -27,22 +27,35 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/exec/document_value/value.h"
-
 #include <absl/hash/hash.h>
-#include <boost/functional/hash.hpp>
+#include <boost/container_hash/extensions.hpp>
+#include <boost/cstdint.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/numeric/conversion/converter_policies.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
 #include <cmath>
+#include <cstdint>
 #include <limits>
+#include <memory>
+#include <ostream>
+#include <type_traits>
+#include <typeinfo>
+
+#include <absl/strings/string_view.h>
 
 #include "mongo/base/compare_numbers.h"
 #include "mongo/base/data_type_endian.h"
-#include "mongo/base/simple_string_data_comparator.h"
+#include "mongo/base/data_view.h"
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data_comparator.h"
 #include "mongo/bson/bson_depth.h"
 #include "mongo/bson/simple_bsonobj_comparator.h"
 #include "mongo/db/exec/document_value/document.h"
-#include "mongo/db/jsobj.h"
+#include "mongo/db/exec/document_value/document_internal.h"
+#include "mongo/db/exec/document_value/value.h"
 #include "mongo/db/query/datetime/date_time_support.h"
 #include "mongo/platform/decimal128.h"
 #include "mongo/util/hex.h"
@@ -60,57 +73,6 @@ using std::stringstream;
 using std::vector;
 using namespace std::string_literals;
 
-void ValueStorage::verifyRefCountingIfShould() const {
-    switch (type) {
-        case MinKey:
-        case MaxKey:
-        case jstOID:
-        case Date:
-        case bsonTimestamp:
-        case EOO:
-        case jstNULL:
-        case Undefined:
-        case Bool:
-        case NumberInt:
-        case NumberLong:
-        case NumberDouble:
-            // the above types never reference external data
-            MONGO_verify(!refCounter);
-            break;
-
-        case String:
-        case RegEx:
-        case Code:
-        case Symbol:
-            // If this is using the short-string optimization, it must not have a ref-counted
-            // pointer.
-            invariant(!shortStr || !refCounter);
-
-            // If this is _not_ using the short string optimization, it must be storing a
-            // ref-counted pointer. One exception: in the BSONElement constructor of Value, it is
-            // possible for this ValueStorage to get constructed as a type but never initialized;
-            // the ValueStorage gets left as a nullptr and not marked as ref-counted, which is ok
-            // (SERVER-43205).
-            invariant(shortStr || (refCounter || !genericRCPtr));
-            break;
-
-        case NumberDecimal:
-        case BinData:  // TODO this should probably support short-string optimization
-        case Array:    // TODO this should probably support empty-is-NULL optimization
-        case DBRef:
-        case CodeWScope:
-            // the above types always reference external data.
-            invariant(refCounter);
-            invariant(bool(genericRCPtr));
-            break;
-
-        case Object:
-            // Objects either hold a NULL ptr or should be ref-counting
-            invariant(refCounter == bool(genericRCPtr));
-            break;
-    }
-}
-
 void ValueStorage::putString(StringData s) {
     // Note: this also stores data portion of BinData
     const size_t sizeNoNUL = s.size();
@@ -136,7 +98,7 @@ void ValueStorage::putDocument(Document&& d) {
     putRefCountable(std::move(d._storage));
 }
 
-void ValueStorage::putVector(boost::intrusive_ptr<RCVector>&& vec) {
+void ValueStorage::putVector(boost::intrusive_ptr<RCVector<Value>>&& vec) {
     fassert(16485, bool(vec));
     putRefCountable(std::move(vec));
 }
@@ -200,7 +162,7 @@ Value::Value(const BSONElement& elem) : _storage(elem.type()) {
         }
 
         case Array: {
-            auto vec = make_intrusive<RCVector>();
+            auto vec = make_intrusive<RCVector<Value>>();
             BSONForEach(sub, elem.embeddedObject()) {
                 vec->vec.push_back(Value(sub));
             }
@@ -262,7 +224,7 @@ Value::Value(const BSONElement& elem) : _storage(elem.type()) {
 }
 
 Value::Value(const BSONArray& arr) : _storage(Array) {
-    auto vec = make_intrusive<RCVector>();
+    auto vec = make_intrusive<RCVector<Value>>();
     BSONForEach(sub, arr) {
         vec->vec.push_back(Value(sub));
     }
@@ -270,7 +232,7 @@ Value::Value(const BSONArray& arr) : _storage(Array) {
 }
 
 Value::Value(const vector<BSONObj>& vec) : _storage(Array) {
-    auto storageVec = make_intrusive<RCVector>();
+    auto storageVec = make_intrusive<RCVector<Value>>();
     storageVec->vec.reserve(vec.size());
     for (auto&& obj : vec) {
         storageVec->vec.push_back(Value(obj));
@@ -279,7 +241,7 @@ Value::Value(const vector<BSONObj>& vec) : _storage(Array) {
 }
 
 Value::Value(const vector<Document>& vec) : _storage(Array) {
-    auto storageVec = make_intrusive<RCVector>();
+    auto storageVec = make_intrusive<RCVector<Value>>();
     storageVec->vec.reserve(vec.size());
     for (auto&& obj : vec) {
         storageVec->vec.push_back(Value(obj));
@@ -668,7 +630,7 @@ string Value::coerceToString() const {
 
         case Date:
             return uassertStatusOKWithContext(
-                TimeZoneDatabase::utcZone().formatDate(kISOFormatString, getDate()),
+                TimeZoneDatabase::utcZone().formatDate(kIsoFormatStringZ, getDate()),
                 "failed while coercing date to string");
 
         case EOO:
@@ -711,9 +673,7 @@ inline static int cmp(const T& left, const T& right) {
     }
 }
 
-int Value::compare(const Value& rL,
-                   const Value& rR,
-                   const StringData::ComparatorInterface* stringComparator) {
+int Value::compare(const Value& rL, const Value& rR, const StringDataComparator* stringComparator) {
     // Note, this function needs to behave identically to BSONElement::compareElements().
     // Additionally, any changes here must be replicated in hash_combine().
     BSONType lType = rL.getType();
@@ -899,8 +859,7 @@ size_t hashStringData(StringData sd, size_t seed) {
 }
 }  // namespace
 
-void Value::hash_combine(size_t& seed,
-                         const StringData::ComparatorInterface* stringComparator) const {
+void Value::hash_combine(size_t& seed, const StringDataComparator* stringComparator) const {
     BSONType type = getType();
 
     boost::hash_combine(seed, canonicalizeBSONType(type));
@@ -1016,7 +975,7 @@ void Value::hash_combine(size_t& seed,
 
         case CodeWScope: {
             intrusive_ptr<const RCCodeWScope> cws = _storage.getCodeWScope();
-            SimpleStringDataComparator::kInstance.hash_combine(seed, cws->code);
+            simpleStringDataComparator.hash_combine(seed, cws->code);
             SimpleBSONObjComparator::kInstance.hash_combine(seed, cws->scope);
             break;
         }
@@ -1171,7 +1130,7 @@ size_t Value::getApproximateSize() const {
 
         case Array: {
             size_t size = sizeof(Value);
-            size += sizeof(RCVector);
+            size += sizeof(RCVector<Value>);
             const size_t n = getArray().size();
             for (size_t i = 0; i < n; ++i) {
                 size += getArray()[i].getApproximateSize();
@@ -1248,7 +1207,7 @@ ostream& operator<<(ostream& out, const Value& val) {
             return out << "undefined";
         case Date:
             return out << [&] {
-                if (auto string = TimeZoneDatabase::utcZone().formatDate(kISOFormatString,
+                if (auto string = TimeZoneDatabase::utcZone().formatDate(kIsoFormatStringZ,
                                                                          val.coerceToDate());
                     string.isOK())
                     return string.getValue();
