@@ -29,11 +29,33 @@
 
 #pragma once
 
+#include <algorithm>
+#include <cstring>
+#include <initializer_list>
+#include <iosfwd>
+#include <string>
+#include <utility>
+#include <vector>
+
+#include "mongo/base/data_range.h"
 #include "mongo/base/static_assert.h"
 #include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/bson/bsontypes_util.h"
+#include "mongo/bson/oid.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/bson/util/builder.h"
 #include "mongo/db/exec/document_value/value_internal.h"
-#include "mongo/util/concepts.h"
+#include "mongo/platform/decimal128.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/bufreader.h"
+#include "mongo/util/intrusive_counter.h"
 #include "mongo/util/safe_num.h"
+#include "mongo/util/time_support.h"
 #include "mongo/util/uuid.h"
 
 namespace mongo {
@@ -110,7 +132,7 @@ public:
     explicit Value(const std::vector<BSONObj>& vec);
     explicit Value(const std::vector<Document>& vec);
     explicit Value(std::vector<Value> vec)
-        : _storage(Array, make_intrusive<RCVector>(std::move(vec))) {}
+        : _storage(Array, make_intrusive<RCVector<Value>>(std::move(vec))) {}
     explicit Value(const BSONBinData& bd) : _storage(BinData, bd) {}
     explicit Value(const BSONRegEx& re) : _storage(RegEx, re) {}
     explicit Value(const BSONCodeWScope& cws) : _storage(CodeWScope, cws) {}
@@ -306,7 +328,7 @@ public:
      * Compare two Values. Most Values should prefer to use ValueComparator instead. See
      * value_comparator.h for details.
      *
-     *  Pass a non-null StringData::ComparatorInterface if special string comparison semantics are
+     *  Pass a non-null StringDataComparator if special string comparison semantics are
      *  required. If the comparator is null, then a simple binary compare is used for strings. This
      *  comparator is only used for string *values*; field names are always compared using simple
      *  binary compare.
@@ -317,7 +339,7 @@ public:
      */
     static int compare(const Value& lhs,
                        const Value& rhs,
-                       const StringData::ComparatorInterface* stringComparator);
+                       const StringDataComparator* stringComparator);
 
     friend DeferredComparison operator==(const Value& lhs, const Value& rhs) {
         return DeferredComparison(DeferredComparison::Type::kEQ, lhs, rhs);
@@ -375,7 +397,7 @@ public:
      * Most callers should prefer the utilities in ValueComparator for hashing and creating function
      * objects for computing the hash. See value_comparator.h.
      */
-    void hash_combine(size_t& seed, const StringData::ComparatorInterface* stringComparator) const;
+    void hash_combine(size_t& seed, const StringDataComparator* stringComparator) const;
 
     /// Call this after memcpying to update ref counts if needed
     void memcpyed() const {
@@ -417,22 +439,15 @@ inline void swap(mongo::Value& lhs, mongo::Value& rhs) {
     lhs.swap(rhs);
 }
 
-MONGO_MAKE_BOOL_TRAIT(CanConstructValueFrom,
-                      (typename T),
-                      (T),
-                      (T val),
-                      //
-                      Value(std::forward<T>(val)));
-
 /**
  * This class is identical to Value, but supports implicit creation from any of the types explicitly
  * supported by Value.
  */
 class ImplicitValue : public Value {
 public:
-    TEMPLATE(typename T)
-    REQUIRES(CanConstructValueFrom<T>)
-    ImplicitValue(T&& arg) : Value(std::forward<T>(arg)) {}
+    template <typename T>
+    requires std::is_constructible_v<Value, T> ImplicitValue(T&& arg)
+        : Value(std::forward<T>(arg)) {}
 
     ImplicitValue(std::initializer_list<ImplicitValue> values) : Value(convertToValues(values)) {}
     ImplicitValue(std::vector<ImplicitValue> values) : Value(convertToValues(values)) {}
@@ -456,7 +471,7 @@ public:
         values.reserve(vec.size());
         for_each(
             vec.begin(), vec.end(), ([&](const ImplicitValue& val) { values.push_back(val); }));
-        return Value(values);
+        return Value(std::move(values));
     }
 
     /**
