@@ -28,21 +28,32 @@
  */
 
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/exec/index_scan.h"
-
+#include <absl/container/node_hash_map.h>
+#include <boost/container/small_vector.hpp>
+// IWYU pragma: no_include "boost/intrusive/detail/iterator.hpp"
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
 #include <memory>
+#include <vector>
 
-#include "mongo/db/catalog/index_catalog.h"
+#include <boost/optional/optional.hpp>
+
+#include "mongo/bson/ordering.h"
+#include "mongo/db/client.h"
+#include "mongo/db/exec/document_value/document_metadata_fields.h"
 #include "mongo/db/exec/filter.h"
-#include "mongo/db/exec/scoped_timer.h"
+#include "mongo/db/exec/index_scan.h"
 #include "mongo/db/index/index_access_method.h"
-#include "mongo/db/index_names.h"
 #include "mongo/db/query/index_bounds_builder.h"
 #include "mongo/db/query/plan_executor_impl.h"
 #include "mongo/db/query/query_knobs_gen.h"
+#include "mongo/db/storage/key_string.h"
+#include "mongo/db/storage/recovery_unit.h"
+#include "mongo/db/transaction_resources.h"
+#include "mongo/platform/atomic_word.h"
 #include "mongo/util/assert_util.h"
+#include "mongo/util/concurrency/admission_context.h"
+#include "mongo/util/debug_util.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
@@ -64,7 +75,7 @@ namespace mongo {
 const char* IndexScan::kStageType = "IXSCAN";
 
 IndexScan::IndexScan(ExpressionContext* expCtx,
-                     const CollectionPtr& collection,
+                     VariantCollectionPtrOrAcquisition collection,
                      IndexScanParams params,
                      WorkingSet* workingSet,
                      const MatchExpression* filter)
@@ -91,13 +102,21 @@ IndexScan::IndexScan(ExpressionContext* expCtx,
     _specificStats.collation = params.indexDescriptor->infoObj()
                                    .getObjectField(IndexDescriptor::kCollationFieldName)
                                    .getOwned();
+    if (_shouldDedup && internalUseRoaringBitmapsForRecordIDDeduplication.load()) {
+        const size_t threshold = static_cast<size_t>(internalRoaringBitmapsThreshold.load());
+        const size_t batchSize = static_cast<size_t>(internalRoaringBitmapsBatchSize.load());
+        const uint64_t universeSize =
+            static_cast<uint64_t>(threshold / internalRoaringBitmapsMinimalDensity.load());
+        _recordIdDeduplicator =
+            std::make_unique<RecordIdDeduplicator>(threshold, batchSize, universeSize);
+    }
 }
 
 boost::optional<IndexKeyEntry> IndexScan::initIndexScan() {
     if (_lowPriority && gDeprioritizeUnboundedUserIndexScans.load() &&
         opCtx()->getClient()->isFromUserConnection() &&
-        opCtx()->lockState()->shouldWaitForTicket()) {
-        _priority.emplace(opCtx()->lockState(), AdmissionContext::Priority::kLow);
+        shard_role_details::getLocker(opCtx())->shouldWaitForTicket(opCtx())) {
+        _priority.emplace(opCtx(), AdmissionContext::Priority::kLow);
     }
 
     // Perform the possibly heavy-duty initialization of the underlying index cursor.
@@ -112,12 +131,14 @@ boost::optional<IndexKeyEntry> IndexScan::initIndexScan() {
         _endKey = _bounds.endKey;
         _indexCursor->setEndPosition(_endKey, _endKeyInclusive);
 
-        KeyString::Value keyStringForSeek = IndexEntryComparison::makeKeyStringFromBSONKeyForSeek(
+        key_string::Builder builder(
+            indexAccessMethod()->getSortedDataInterface()->getKeyStringVersion());
+        auto keyStringForSeek = IndexEntryComparison::makeKeyStringFromBSONKeyForSeek(
             _startKey,
-            indexAccessMethod()->getSortedDataInterface()->getKeyStringVersion(),
             indexAccessMethod()->getSortedDataInterface()->getOrdering(),
             _forward,
-            _startKeyInclusive);
+            _startKeyInclusive,
+            builder);
         return _indexCursor->seek(keyStringForSeek);
     } else {
         // For single intervals, we can use an optimized scan which checks against the position
@@ -127,23 +148,25 @@ boost::optional<IndexKeyEntry> IndexScan::initIndexScan() {
                 _bounds, &_startKey, &_startKeyInclusive, &_endKey, &_endKeyInclusive)) {
             _indexCursor->setEndPosition(_endKey, _endKeyInclusive);
 
+            key_string::Builder builder(
+                indexAccessMethod()->getSortedDataInterface()->getKeyStringVersion());
             auto keyStringForSeek = IndexEntryComparison::makeKeyStringFromBSONKeyForSeek(
                 _startKey,
-                indexAccessMethod()->getSortedDataInterface()->getKeyStringVersion(),
                 indexAccessMethod()->getSortedDataInterface()->getOrdering(),
                 _forward,
-                _startKeyInclusive);
+                _startKeyInclusive,
+                builder);
             return _indexCursor->seek(keyStringForSeek);
         } else {
             _checker.reset(new IndexBoundsChecker(&_bounds, _keyPattern, _direction));
 
             if (!_checker->getStartSeekPoint(&_seekPoint))
                 return boost::none;
-            return _indexCursor->seek(IndexEntryComparison::makeKeyStringFromSeekPointForSeek(
-                _seekPoint,
+            key_string::Builder builder(
                 indexAccessMethod()->getSortedDataInterface()->getKeyStringVersion(),
-                indexAccessMethod()->getSortedDataInterface()->getOrdering(),
-                _forward));
+                indexAccessMethod()->getSortedDataInterface()->getOrdering());
+            return _indexCursor->seek(IndexEntryComparison::makeKeyStringFromSeekPointForSeek(
+                _seekPoint, _forward, builder));
         }
     }
 }
@@ -155,7 +178,6 @@ PlanStage::StageState IndexScan::doWork(WorkingSetID* out) {
     const auto ret = handlePlanStageYield(
         expCtx(),
         "IndexScan",
-        collection()->ns().ns(),
         [&] {
             switch (_scanState) {
                 case INITIALIZING:
@@ -164,14 +186,15 @@ PlanStage::StageState IndexScan::doWork(WorkingSetID* out) {
                 case GETTING_NEXT:
                     kv = _indexCursor->next();
                     break;
-                case NEED_SEEK:
+                case NEED_SEEK: {
                     ++_specificStats.seeks;
-                    kv = _indexCursor->seek(IndexEntryComparison::makeKeyStringFromSeekPointForSeek(
-                        _seekPoint,
+                    key_string::Builder builder(
                         indexAccessMethod()->getSortedDataInterface()->getKeyStringVersion(),
-                        indexAccessMethod()->getSortedDataInterface()->getOrdering(),
-                        _forward));
+                        indexAccessMethod()->getSortedDataInterface()->getOrdering());
+                    kv = _indexCursor->seek(IndexEntryComparison::makeKeyStringFromSeekPointForSeek(
+                        _seekPoint, _forward, builder));
                     break;
+                }
                 case HIT_END:
                     _priority.reset();
                     return PlanStage::IS_EOF;
@@ -237,7 +260,9 @@ PlanStage::StageState IndexScan::doWork(WorkingSetID* out) {
 
     if (_shouldDedup) {
         ++_specificStats.dupsTested;
-        if (!_returned.insert(kv->loc).second) {
+        const bool newRecordId = _recordIdDeduplicator ? _recordIdDeduplicator->insert(kv->loc)
+                                                       : _returned.insert(kv->loc).second;
+        if (!newRecordId) {
             // We've seen this RecordId before. Skip it this time.
             ++_specificStats.dupsDropped;
             return PlanStage::NEED_TIME;
@@ -255,8 +280,11 @@ PlanStage::StageState IndexScan::doWork(WorkingSetID* out) {
     WorkingSetID id = _workingSet->allocate();
     WorkingSetMember* member = _workingSet->get(id);
     member->recordId = std::move(kv->loc);
-    member->keyData.push_back(IndexKeyDatum(
-        _keyPattern, kv->key, workingSetIndexId(), opCtx()->recoveryUnit()->getSnapshotId()));
+    member->keyData.push_back(
+        IndexKeyDatum(_keyPattern,
+                      kv->key,
+                      workingSetIndexId(),
+                      shard_role_details::getRecoveryUnit(opCtx())->getSnapshotId()));
     _workingSet->transitionToRecordIdAndIdx(id);
 
     if (_addKeyMetadata) {
@@ -298,8 +326,8 @@ void IndexScan::doDetachFromOperationContext() {
 void IndexScan::doReattachToOperationContext() {
     if (_lowPriority && gDeprioritizeUnboundedUserIndexScans.load() &&
         opCtx()->getClient()->isFromUserConnection() &&
-        opCtx()->lockState()->shouldWaitForTicket()) {
-        _priority.emplace(opCtx()->lockState(), AdmissionContext::Priority::kLow);
+        shard_role_details::getLocker(opCtx())->shouldWaitForTicket(opCtx())) {
+        _priority.emplace(opCtx(), AdmissionContext::Priority::kLow);
     }
     if (_indexCursor)
         _indexCursor->reattachToOperationContext(opCtx());
