@@ -31,26 +31,42 @@
 #include "mongo/db/exec/multi_plan.h"
 
 #include <algorithm>
-#include <cmath>
+#include <boost/move/utility_core.hpp>
+#include <deque>
+#include <fmt/format.h>
 #include <memory>
+#include <string>
+#include <utility>
 
-#include "mongo/db/catalog/database.h"
-#include "mongo/db/client.h"
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/db/commands/server_status_metric.h"
 #include "mongo/db/concurrency/exception_util.h"
 #include "mongo/db/exec/histogram_server_status_metric.h"
-#include "mongo/db/exec/scoped_timer.h"
 #include "mongo/db/exec/trial_period_utils.h"
-#include "mongo/db/exec/working_set_common.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/query/classic_plan_cache.h"
 #include "mongo/db/query/collection_query_info.h"
-#include "mongo/db/query/explain.h"
 #include "mongo/db/query/multiple_collection_accessor.h"
 #include "mongo/db/query/plan_cache_key_factory.h"
+#include "mongo/db/query/plan_explainer.h"
+#include "mongo/db/query/plan_explainer_factory.h"
 #include "mongo/db/query/plan_ranker.h"
-#include "mongo/db/query/plan_ranker_util.h"
+#include "mongo/db/query/plan_ranking_decision.h"
+#include "mongo/db/query/query_knobs_gen.h"
+#include "mongo/db/service_context.h"
 #include "mongo/logv2/log.h"
-#include "mongo/util/histogram.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/redaction.h"
+#include "mongo/platform/atomic_proxy.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/duration.h"
 #include "mongo/util/str.h"
+#include "mongo/util/tick_source.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
@@ -73,50 +89,58 @@ void markShouldCollectTimingInfoOnSubtree(PlanStage* root) {
 /**
  * Aggregation of the total number of microseconds spent (in the classic multiplanner).
  */
-CounterMetric classicMicrosTotal("query.multiPlanner.classicMicros");
+auto& classicMicrosTotal = *MetricBuilder<Counter64>{"query.multiPlanner.classicMicros"};
 
 /**
  * Aggregation of the total number of "works" performed (in the classic multiplanner).
  */
-CounterMetric classicWorksTotal("query.multiPlanner.classicWorks");
+auto& classicWorksTotal = *MetricBuilder<Counter64>{"query.multiPlanner.classicWorks"};
 
 /**
  * Aggregation of the total number of invocations (of the classic multiplanner).
  */
-CounterMetric classicCount("query.multiPlanner.classicCount");
+auto& classicCount = *MetricBuilder<Counter64>{"query.multiPlanner.classicCount"};
 
 /**
  * An element in this histogram is the number of microseconds spent in an invocation (of the
  * classic multiplanner).
  */
-HistogramServerStatusMetric classicMicrosHistogram("query.multiPlanner.histograms.classicMicros",
-                                                   HistogramServerStatusMetric::pow(11, 1024, 4));
+auto& classicMicrosHistogram =
+    *MetricBuilder<HistogramServerStatusMetric>{"query.multiPlanner.histograms.classicMicros"}.bind(
+        HistogramServerStatusMetric::pow(11, 1024, 4));
 
 /**
  * An element in this histogram is the number of "works" performed during an invocation (of the
  * classic multiplanner).
  */
-HistogramServerStatusMetric classicWorksHistogram("query.multiPlanner.histograms.classicWorks",
-                                                  HistogramServerStatusMetric::pow(9, 128, 2));
+auto& classicWorksHistogram =
+    *MetricBuilder<HistogramServerStatusMetric>{"query.multiPlanner.histograms.classicWorks"}.bind(
+        HistogramServerStatusMetric::pow(9, 128, 2));
 
 /**
  * An element in this histogram is the number of plans in the candidate set of an invocation (of the
  * classic multiplanner).
  */
-HistogramServerStatusMetric classicNumPlansHistogram(
-    "query.multiPlanner.histograms.classicNumPlans", HistogramServerStatusMetric::pow(5, 2, 2));
+auto& classicNumPlansHistogram =
+    *MetricBuilder<HistogramServerStatusMetric>{"query.multiPlanner.histograms.classicNumPlans"}
+         .bind(HistogramServerStatusMetric::pow(5, 2, 2));
 
 }  // namespace
 
+MONGO_FAIL_POINT_DEFINE(sleepWhileMultiplanning);
+
 MultiPlanStage::MultiPlanStage(ExpressionContext* expCtx,
-                               const CollectionPtr& collection,
+                               VariantCollectionPtrOrAcquisition collection,
                                CanonicalQuery* cq,
-                               PlanCachingMode cachingMode)
+                               OnPickBestPlan onPickBestPlan,
+                               boost::optional<std::string> replanReason)
     : RequiresCollectionStage(kStageType, expCtx, collection),
-      _cachingMode(cachingMode),
       _query(cq),
+      _onPickBestPlan(std::move(onPickBestPlan)),
       _bestPlanIdx(kNoSuchPlan),
-      _backupPlanIdx(kNoSuchPlan) {}
+      _backupPlanIdx(kNoSuchPlan) {
+    _specificStats.replanReason = replanReason;
+}
 
 void MultiPlanStage::addPlan(std::unique_ptr<QuerySolution> solution,
                              std::unique_ptr<PlanStage> root,
@@ -166,9 +190,9 @@ PlanStage::StageState MultiPlanStage::doWork(WorkingSetID* out) {
 
         LOGV2_DEBUG(20588, 5, "Best plan errored, switching to backup plan");
 
-        CollectionQueryInfo::get(collection())
+        CollectionQueryInfo::get(collectionPtr())
             .getPlanCache()
-            ->remove(plan_cache_key_factory::make<PlanCacheKey>(*_query, collection()));
+            ->remove(plan_cache_key_factory::make<PlanCacheKey>(*_query, collectionPtr()));
 
         switchToBackupPlan();
         return _candidates[_bestPlanIdx].root->work(out);
@@ -189,12 +213,19 @@ void MultiPlanStage::tryYield(PlanYieldPolicy* yieldPolicy) {
     //   3) we need to yield and retry due to a WriteConflictException.
     // In all cases, the actual yielding happens here.
     if (yieldPolicy->shouldYieldOrInterrupt(expCtx()->opCtx)) {
-        uassertStatusOK(yieldPolicy->yieldOrInterrupt(
-            expCtx()->opCtx, nullptr, RestoreContext::RestoreType::kYield));
+        uassertStatusOK(yieldPolicy->yieldOrInterrupt(expCtx()->opCtx));
     }
 }
 
 Status MultiPlanStage::pickBestPlan(PlanYieldPolicy* yieldPolicy) {
+    if (bestPlanChosen()) {
+        return Status::OK();
+    }
+
+    if (MONGO_unlikely(sleepWhileMultiplanning.shouldFail())) {
+        sleepWhileMultiplanning.execute(
+            [&](const BSONObj& data) { sleepmillis(data["ms"].numberInt()); });
+    }
     // Adds the amount of time taken by pickBestPlan() to executionTime. There's lots of execution
     // work that happens here, so this is needed for the time accounting to make sense.
     auto optTimer = getOptTimer();
@@ -207,7 +238,7 @@ Status MultiPlanStage::pickBestPlan(PlanYieldPolicy* yieldPolicy) {
 
     const size_t numWorks =
         trial_period::getTrialPeriodMaxWorks(opCtx(),
-                                             collection(),
+                                             collectionPtr(),
                                              internalQueryPlanEvaluationWorks.load(),
                                              internalQueryPlanEvaluationCollFraction.load());
     size_t numResults = trial_period::getTrialPeriodNumToReturn(*_query);
@@ -252,8 +283,11 @@ Status MultiPlanStage::pickBestPlan(PlanYieldPolicy* yieldPolicy) {
     const auto& alreadyProduced = bestCandidate.results;
     const auto& bestSolution = bestCandidate.solution;
 
-    LOGV2_DEBUG(
-        20590, 5, "Winning solution", "bestSolution"_attr = redact(bestSolution->toString()));
+    LOGV2_DEBUG(20590,
+                5,
+                "Winning solution",
+                "bestSolution"_attr = redact(bestSolution->toString()),
+                "bestSolutionHash"_attr = bestSolution->hash());
 
     auto explainer =
         plan_explainer_factory::make(bestCandidate.root, bestSolution->_enumeratorExplainInfo);
@@ -271,12 +305,10 @@ Status MultiPlanStage::pickBestPlan(PlanYieldPolicy* yieldPolicy) {
         }
     }
 
-    plan_cache_util::updatePlanCache(expCtx()->opCtx,
-                                     MultipleCollectionAccessor(collection()),
-                                     _cachingMode,
-                                     *_query,
-                                     std::move(ranking),
-                                     _candidates);
+    // Invoke the callback provided on construction, passing 'ranking' and '_candidates' to describe
+    // the results of plan selection.
+    _onPickBestPlan(*_query, *this, std::move(ranking), _candidates);
+
     removeRejectedPlans();
 
     return Status::OK();
@@ -420,13 +452,20 @@ const QuerySolution* MultiPlanStage::bestSolution() const {
     return _candidates[_bestPlanIdx].solution.get();
 }
 
-std::unique_ptr<QuerySolution> MultiPlanStage::bestSolution() {
+std::unique_ptr<QuerySolution> MultiPlanStage::extractBestSolution() {
     if (_bestPlanIdx == kNoSuchPlan)
         return nullptr;
 
+    _bestPlanScore = _candidates[_bestPlanIdx].solution->score;
     return std::move(_candidates[_bestPlanIdx].solution);
 }
 
+bool MultiPlanStage::bestSolutionEof() const {
+    tassert(8523500, "The best plan is not chosen by the multi-planner", bestPlanChosen());
+    auto& bestPlan = _candidates[_bestPlanIdx];
+    return bestPlan.root->isEOF();
+}
+
 unique_ptr<PlanStageStats> MultiPlanStage::getStats() {
     _commonStats.isEOF = isEOF();
     unique_ptr<PlanStageStats> ret =
@@ -445,11 +484,22 @@ const SpecificStats* MultiPlanStage::getSpecificStats() const {
     return &_specificStats;
 }
 
+const plan_ranker::CandidatePlan& MultiPlanStage::getCandidate(size_t candidateIdx) const {
+    tassert(8223800,
+            str::stream() << "Invalid candidate plan index: " << candidateIdx
+                          << ", size: " << _candidates.size(),
+            candidateIdx < _candidates.size());
+    return _candidates[candidateIdx];
+}
+
 boost::optional<double> MultiPlanStage::getCandidateScore(size_t candidateIdx) const {
     tassert(5408301,
             str::stream() << "Invalid candidate plan index: " << candidateIdx
                           << ", size: " << _candidates.size(),
             candidateIdx < _candidates.size());
+    if (candidateIdx == static_cast<size_t>(_bestPlanIdx) && !_candidates[candidateIdx].solution) {
+        return _bestPlanScore;
+    }
     return _candidates[candidateIdx].solution->score;
 }
 
