@@ -27,24 +27,50 @@
  *    it in the license file.
  */
 
+#include <absl/container/flat_hash_map.h>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <cstddef>
+#include <cstdint>
+#include <initializer_list>
+#include <ostream>
+#include <utility>
+#include <vector>
+
+#include <boost/optional/optional.hpp>
+
 #include "mongo/base/string_data.h"
 #include "mongo/bson/timestamp.h"
 #include "mongo/db/exec/sbe/abt/abt_lower.h"
-#include "mongo/db/exec/sbe/abt/named_slots_mock.h"
+#include "mongo/db/exec/sbe/expressions/runtime_environment.h"
+#include "mongo/db/exec/sbe/util/debug_print.h"
+#include "mongo/db/query/optimizer/algebra/polyvalue.h"
+#include "mongo/db/query/optimizer/comparison_op.h"
+#include "mongo/db/query/optimizer/containers.h"
 #include "mongo/db/query/optimizer/defs.h"
 #include "mongo/db/query/optimizer/explain.h"
+#include "mongo/db/query/optimizer/index_bounds.h"
 #include "mongo/db/query/optimizer/metadata.h"
-#include "mongo/db/query/optimizer/node.h"
+#include "mongo/db/query/optimizer/node.h"  // IWYU pragma: keep
 #include "mongo/db/query/optimizer/node_defs.h"
 #include "mongo/db/query/optimizer/props.h"
 #include "mongo/db/query/optimizer/rewrites/const_eval.h"
 #include "mongo/db/query/optimizer/rewrites/path_lower.h"
 #include "mongo/db/query/optimizer/syntax/expr.h"
+#include "mongo/db/query/optimizer/syntax/path.h"
+#include "mongo/db/query/optimizer/utils/strong_alias.h"
+#include "mongo/db/query/optimizer/utils/unit_test_abt_literals.h"
 #include "mongo/db/query/optimizer/utils/unit_test_utils.h"
+#include "mongo/db/query/optimizer/utils/utils.h"
+#include "mongo/platform/decimal128.h"
+#include "mongo/unittest/assert.h"
 #include "mongo/unittest/death_test.h"
+#include "mongo/unittest/framework.h"
 #include "mongo/unittest/golden_test.h"
-#include "mongo/unittest/unittest.h"
-#include <vector>
+#include "mongo/unittest/golden_test_base.h"
+#include "mongo/util/str.h"
+#include "mongo/util/time_support.h"
+#include "mongo/util/uuid.h"
 
 namespace mongo::optimizer {
 namespace {
@@ -52,12 +78,14 @@ namespace {
 unittest::GoldenTestConfig goldenTestConfig{"src/mongo/db/test_output/exec/sbe"};
 using GoldenTestContext = unittest::GoldenTestContext;
 using GoldenTestConfig = unittest::GoldenTestConfig;
+using namespace unit_test_abt_literals;
 class ABTPlanGeneration : public unittest::Test {
 protected:
     ProjectionName scanLabel = ProjectionName{"scan0"_sd};
     NodeToGroupPropsMap _nodeMap;
     // This can be modified by tests that need other labels.
     FieldProjectionMap _fieldProjMap{{}, {scanLabel}, {}};
+    sbe::InputParamToSlotMap inputParamToSlotMap;
 
     void runExpressionVariation(GoldenTestContext& gctx, const std::string& name, const ABT& n) {
         auto& stream = gctx.outStream();
@@ -70,8 +98,14 @@ protected:
         stream << "-- OUTPUT:" << std::endl;
         auto env = VariableEnvironment::build(n);
         SlotVarMap map;
-        MockEmptyNamedSlotsProvider namedSlots;
-        auto expr = SBEExpressionLowering{env, map, namedSlots}.optimize(n);
+        sbe::RuntimeEnvironment runtimeEnv;
+        sbe::value::SlotIdGenerator ids;
+        Metadata metadata;
+        NodeProps np;
+        auto expr =
+            optimizer::SBEExpressionLowering{
+                env, map, runtimeEnv, ids, inputParamToSlotMap, &metadata, &np}
+                .optimize(n);
         stream << expr->toString() << std::endl;
     }
 
@@ -96,8 +130,12 @@ protected:
     void runNodeVariation(GoldenTestContext& gctx,
                           const std::string& name,
                           const ABT& n,
+                          sbe::RuntimeEnvironment* runtimeEnv,
+                          sbe::value::SlotIdGenerator* ids,
                           boost::optional<opt::unordered_map<std::string, IndexDefinition>>
-                              collIndexDefs = boost::none) {
+                              collIndexDefs = boost::none,
+                          opt::unordered_map<std::string, ScanDefinition> scanDefs =
+                              opt::unordered_map<std::string, ScanDefinition>()) {
         auto& stream = gctx.outStream();
         if (stream.tellp()) {
             stream << std::endl;
@@ -108,10 +146,7 @@ protected:
         stream << "-- OUTPUT:" << std::endl;
         auto env = VariableEnvironment::build(n);
         SlotVarMap map;
-        MockEmptyNamedSlotsProvider namedSlots;
         boost::optional<sbe::value::SlotId> ridSlot;
-        sbe::value::SlotIdGenerator ids;
-        opt::unordered_map<std::string, ScanDefinition> scanDefs;
 
         scanDefs.insert({"collName",
                          collIndexDefs.has_value() ? buildScanDefinition(collIndexDefs.value())
@@ -120,7 +155,7 @@ protected:
 
         Metadata md(scanDefs);
         auto planStage =
-            SBENodeLowering{env, namedSlots, ids, md, _nodeMap, ScanOrder::Forward}.optimize(
+            SBENodeLowering{env, *runtimeEnv, *ids, inputParamToSlotMap, md, _nodeMap}.optimize(
                 n, map, ridSlot);
         sbe::DebugPrinter printer;
         stream << stripUUIDs(printer.print(*planStage)) << std::endl;
@@ -132,18 +167,54 @@ protected:
         lastNodeGenerated = 0;
     }
 
+    void runNodeVariation(GoldenTestContext& gctx,
+                          const std::string& name,
+                          const ABT& n,
+                          boost::optional<opt::unordered_map<std::string, IndexDefinition>>
+                              collIndexDefs = boost::none,
+                          opt::unordered_map<std::string, ScanDefinition> scanDefs =
+                              opt::unordered_map<std::string, ScanDefinition>()) {
+        sbe::RuntimeEnvironment runtimeEnv;
+        sbe::value::SlotIdGenerator ids;
+        runNodeVariation(gctx, name, n, &runtimeEnv, &ids, collIndexDefs, scanDefs);
+    }
+
+    std::string autoUpdateExpressionVariation(const ABT& n) {
+        auto env = VariableEnvironment::build(n);
+        SlotVarMap map;
+        sbe::RuntimeEnvironment runtimeEnv;
+        sbe::value::SlotIdGenerator ids;
+        Metadata metadata;
+        NodeProps np;
+        auto expr =
+            optimizer::SBEExpressionLowering{
+                env, map, runtimeEnv, ids, inputParamToSlotMap, &metadata, &np}
+                .optimize(n);
+        return expr->toString();
+    }
+
     ScanDefinition buildScanDefinition(
-        opt::unordered_map<std::string, IndexDefinition> indexDefs = {}) {
+        opt::unordered_map<std::string, IndexDefinition> indexDefs = {},
+        DistributionAndPaths dnp = DistributionAndPaths(DistributionType::Centralized),
+        ShardingMetadata shardingMetadata = ShardingMetadata{}) {
         ScanDefOptions opts;
         opts.insert({"type", "mongod"});
-        opts.insert({"database", "test"});
-        opts.insert({"uuid", UUID::gen().toString()});
-
         MultikeynessTrie trie;
-        DistributionAndPaths dnp(DistributionType::Centralized);
+        IndexPathOccurrences indexPathOccurrences;
         bool exists = true;
-        CEType ce{false};
-        return ScanDefinition(opts, indexDefs, trie, dnp, exists, ce);
+        CEType ce{0.0};
+        return ScanDefinition(DatabaseNameUtil::deserialize(
+                                  boost::none, "test", SerializationContext::stateDefault()),
+                              UUID::gen(),
+                              opts,
+                              indexDefs,
+                              trie,
+                              dnp,
+                              exists,
+                              ce,
+                              shardingMetadata,
+                              indexPathOccurrences,
+                              ScanOrder::Forward /* scanOrder */);
     }
 
     // Does not add the node to the Node map, must be called inside '_node()'.
@@ -163,7 +234,7 @@ protected:
                     boost::none,
                     CostType::fromDouble(0),
                     CostType::fromDouble(0),
-                    {false}};
+                    CEType{0.0}};
         return n;
     }
     void runPathLowering(ABT& tree) {
@@ -206,7 +277,7 @@ protected:
         bool changed = false;
         do {
             changed = false;
-            if (PathLowering{prefixId, env}.optimize(tree)) {
+            if (PathLowering{prefixId}.optimize(tree)) {
                 changed = true;
             }
             if (ConstEval{env}.optimize(tree)) {
@@ -244,10 +315,193 @@ protected:
         return makeEquals(make<Variable>(ProjectionName(lhs)), rhs);
     }
 
+    boost::optional<sbe::value::SlotId> getSlotId(MatchExpression::InputParamId paramId) const {
+        auto it = inputParamToSlotMap.find(paramId);
+        if (it != inputParamToSlotMap.end()) {
+            return it->second;
+        }
+        return boost::none;
+    }
+
 private:
     int32_t lastNodeGenerated = 0;
 };
 
+TEST_F(ABTPlanGeneration, LowerShardFiltering) {
+    GoldenTestContext ctx(&goldenTestConfig);
+    ctx.printTestHeader(GoldenTestContext::HeaderFormat::Text);
+
+    const std::string shardKeyName = "SHARDKEYNAME";
+    {
+        // In shard filtering-related tests, mock the global and unconditional creation of a shard
+        // filtering slot in the runtime environment.
+        sbe::RuntimeEnvironment runtimeEnv;
+        sbe::value::SlotIdGenerator ids = sbe::value::SlotIdGenerator{};
+        runtimeEnv.registerSlot(
+            kshardFiltererSlotName, sbe::value::TypeTags::Nothing, 0, false, &ids);
+        // Create node properties which allow the lowering to access information about the Shard
+        // Key.
+        NodeProps filterNodeProps{getNextNodeID() /*_planNodeId*/,
+                                  {} /*_groupId*/,
+                                  {} /*_logicalProps*/,
+                                  {} /*_physicalProps*/,
+                                  boost::none /*_ridProjName*/,
+                                  CostType::fromDouble(0) /*_cost*/,
+                                  CostType::fromDouble(0) /*_localCost*/,
+                                  CEType{0.0} /*_adjustedCE*/};
+        // The IndexingAvailability logical property will provide information about where to look in
+        // the scanDefs for the Shard Key info.
+        properties::setPropertyOverwrite(
+            filterNodeProps._logicalProps,
+            properties::IndexingAvailability(10,
+                                             ProjectionName("testProjectionName"),
+                                             shardKeyName,
+                                             false,
+                                             false,
+                                             opt::unordered_set<std::string>()));
+
+        opt::unordered_map<std::string, ScanDefinition> scanDefs{std::make_pair(
+            shardKeyName,
+            buildScanDefinition(
+                {},
+                DistributionAndPaths{DistributionType::Centralized},
+                ShardingMetadata({IndexCollationEntry{_get("a", _id())._n, CollationOp::Ascending},
+                                  IndexCollationEntry{_get("b", _id())._n, CollationOp::Ascending},
+                                  IndexCollationEntry{_get("c", _id())._n, CollationOp::Ascending}},
+                                 true)))};
+
+        // Create the ABT to be lowered, starting with the shardFilter FunctionCall.
+        auto functionCallNode = make<FunctionCall>(
+            "shardFilter",
+            makeSeq(make<Variable>("proj0"), make<Variable>("proj1"), make<Variable>("proj2")));
+
+        auto filterNode =
+            _node(make<FilterNode>(
+                      std::move(functionCallNode),
+                      _node(make<PhysicalScanNode>(
+                          FieldProjectionMap{{},
+                                             {ProjectionName{"scan0"}},
+                                             {{FieldNameType{"a"}, ProjectionName{"proj0"}},
+                                              {FieldNameType{"b"}, ProjectionName{"proj1"}},
+                                              {FieldNameType{"c"}, ProjectionName{"proj2"}}}},
+                          "collName",
+                          false))),
+                  filterNodeProps);
+        runNodeVariation(ctx,
+                         "Shard Filtering with Top Level Fields",
+                         filterNode,
+                         &runtimeEnv,
+                         &ids,
+                         boost::none,
+                         scanDefs);
+    }
+
+    {
+        sbe::RuntimeEnvironment runtimeEnv;
+        sbe::value::SlotIdGenerator ids = sbe::value::SlotIdGenerator{};
+        runtimeEnv.registerSlot(
+            kshardFiltererSlotName, sbe::value::TypeTags::Nothing, 0, false, &ids);
+        NodeProps filterNodeProps{getNextNodeID() /*_planNodeId*/,
+                                  {} /*_groupId*/,
+                                  {} /*_logicalProps*/,
+                                  {} /*_physicalProps*/,
+                                  boost::none /*_ridProjName*/,
+                                  CostType::fromDouble(0) /*_cost*/,
+                                  CostType::fromDouble(0) /*_localCost*/,
+                                  CEType{0.0} /*_adjustedCE*/};
+        properties::setPropertyOverwrite(
+            filterNodeProps._logicalProps,
+            properties::IndexingAvailability(10,
+                                             ProjectionName("testProjectionName"),
+                                             shardKeyName,
+                                             false,
+                                             false,
+                                             opt::unordered_set<std::string>()));
+
+        // "a.b" is the shard Key in this variation of the test.
+        auto pathABT = _get("a", _get("b", _id()))._n;
+        opt::unordered_map<std::string, ScanDefinition> scanDefs{std::make_pair(
+            shardKeyName,
+            buildScanDefinition(
+                {},
+                DistributionAndPaths{DistributionType::Centralized},
+                ShardingMetadata({IndexCollationEntry{pathABT, CollationOp::Ascending}}, true)))};
+
+        auto evalPathABT =
+            make<EvalPath>(std::move(pathABT), make<Variable>(ProjectionName("scan0")));
+        auto evalNode = _node(make<EvaluationNode>(
+            ProjectionName("proj0"), _path(std::move(evalPathABT)), _node(scanForTest())));
+
+        // Create the ABT to be lowered, starting with the shardFilter FunctionCall.
+        auto functionCallNode = make<FunctionCall>("shardFilter", makeSeq(make<Variable>("proj0")));
+        auto filterNode = _node(make<FilterNode>(std::move(functionCallNode), std::move(evalNode)),
+                                filterNodeProps);
+        runNodeVariation(ctx,
+                         "Shard Filtering with Dotted Field Path",
+                         filterNode,
+                         &runtimeEnv,
+                         &ids,
+                         boost::none,
+                         scanDefs);
+    }
+
+    {
+        // Test lowering for a shardFilter with expressions other than Variables as children.
+        sbe::RuntimeEnvironment runtimeEnv;
+        sbe::value::SlotIdGenerator ids = sbe::value::SlotIdGenerator{};
+        runtimeEnv.registerSlot(
+            kshardFiltererSlotName, sbe::value::TypeTags::Nothing, 0, false, &ids);
+        NodeProps filterNodeProps{getNextNodeID() /*_planNodeId*/,
+                                  {} /*_groupId*/,
+                                  {} /*_logicalProps*/,
+                                  {} /*_physicalProps*/,
+                                  boost::none /*_ridProjName*/,
+                                  CostType::fromDouble(0) /*_cost*/,
+                                  CostType::fromDouble(0) /*_localCost*/,
+                                  CEType{0.0} /*_adjustedCE*/};
+        properties::setPropertyOverwrite(
+            filterNodeProps._logicalProps,
+            properties::IndexingAvailability(10,
+                                             ProjectionName("testProjectionName"),
+                                             shardKeyName,
+                                             false,
+                                             false,
+                                             opt::unordered_set<std::string>()));
+
+        opt::unordered_map<std::string, ScanDefinition> scanDefs{std::make_pair(
+            shardKeyName,
+            buildScanDefinition(
+                {},
+                DistributionAndPaths{DistributionType::Centralized},
+                ShardingMetadata({IndexCollationEntry{_get("a", _id())._n, CollationOp::Ascending},
+                                  IndexCollationEntry{_get("b", _id())._n, CollationOp::Ascending}},
+                                 true)))};
+
+        auto functionCallNode = make<FunctionCall>(
+            "shardFilter",
+            makeSeq(_path(make<EvalPath>(make<PathGet>("a", make<PathIdentity>()),
+                                         make<Variable>("scan0"))),
+                    make<Variable>("proj_b")));
+
+        auto filterNode =
+            _node(make<FilterNode>(
+                      std::move(functionCallNode),
+                      _node(make<PhysicalScanNode>(
+                          FieldProjectionMap{{},
+                                             {ProjectionName{"scan0"}},
+                                             {{FieldNameType{"b"}, ProjectionName{"proj_b"}}}},
+                          "collName",
+                          false))),
+                  filterNodeProps);
+        runNodeVariation(ctx,
+                         "Shard Filtering with Inlined path",
+                         filterNode,
+                         &runtimeEnv,
+                         &ids,
+                         boost::none,
+                         scanDefs);
+    }
+}
 
 TEST_F(ABTPlanGeneration, LowerConstantExpression) {
     GoldenTestContext ctx(&goldenTestConfig);
@@ -282,7 +536,7 @@ TEST_F(ABTPlanGeneration, LowerCollationNode) {
                                 boost::none,
                                 CostType::fromDouble(0),
                                 CostType::fromDouble(0),
-                                {false}};
+                                CEType{0.0}};
 
     auto node = _node(
         make<CollationNode>(properties::CollationRequirement({{"sortA", CollationOp::Ascending}}),
@@ -307,7 +561,7 @@ TEST_F(ABTPlanGeneration, LowerCollationNode) {
                                  boost::none,
                                  CostType::fromDouble(0),
                                  CostType::fromDouble(0),
-                                 {false}};
+                                 CEType{0.0}};
     auto node2 = _node(
         make<CollationNode>(properties::CollationRequirement({{"sortA", CollationOp::Ascending},
                                                               {"sortB", CollationOp::Descending}}),
@@ -359,7 +613,7 @@ TEST_F(ABTPlanGeneration, LowerExchangeNode) {
                                    boost::none,
                                    CostType::fromDouble(0),
                                    CostType::fromDouble(0),
-                                   {false}};
+                                   CEType{0.0}};
 
         properties::PhysProps evalPhysProps;
         properties::DistributionRequirement evalDistReq{properties::DistributionAndProjections(
@@ -373,7 +627,7 @@ TEST_F(ABTPlanGeneration, LowerExchangeNode) {
                                boost::none,
                                CostType::fromDouble(0),
                                CostType::fromDouble(0),
-                               {false}};
+                               CEType{0.0}};
 
 
         ABT evalNode =
@@ -394,8 +648,7 @@ TEST_F(ABTPlanGeneration, LowerExchangeNode) {
 
         runNodeVariation(
             ctx,
-            str::stream() << "Lower exchange node of type "
-                          << DistributionTypeEnum::toString[static_cast<int>(exchangeType)],
+            str::stream() << "Lower exchange node of type " << toStringData(exchangeType),
             _node(make<ExchangeNode>(distReq, std::move(evalNode2)), exchangeNodeProp));
     }
 }
@@ -421,6 +674,96 @@ TEST_F(ABTPlanGeneration, LowerFilterNode) {
                                _node(scanForTest()))));
 }
 
+TEST_F(ABTPlanGeneration, LowerFilterNodeParameterized) {
+    GoldenTestContext ctx(&goldenTestConfig);
+    ctx.printTestHeader(GoldenTestContext::HeaderFormat::Text);
+
+    runNodeVariation(
+        ctx,
+        "filter for: a >= Constant",
+        _node(make<FilterNode>(
+            _path(make<EvalFilter>(
+                make<PathGet>(
+                    "a",
+                    make<PathCompare>(
+                        Operations::Gte,
+                        make<FunctionCall>("getParam",
+                                           makeSeq(Constant::int32(0),
+                                                   Constant::int32(static_cast<int32_t>(
+                                                       sbe::value::TypeTags::NumberInt32)))))),
+                make<Variable>(scanLabel))),
+            _node(scanForTest()))));
+
+    // Check that correct sbe slot entry is added to inputParamToSlotMap
+    auto slotId0 = getSlotId(0);
+    ASSERT(slotId0 != boost::none);
+    ASSERT_EQUALS(*slotId0, 2);
+
+    runNodeVariation(
+        ctx,
+        "range conjunction filter for: a >= Constant1, a < Constant2",
+        _node(make<FilterNode>(
+            _path(make<EvalFilter>(
+                make<PathGet>(
+                    "a",
+                    make<PathComposeM>(
+                        make<PathCompare>(
+                            Operations::Gte,
+                            make<FunctionCall>("getParam",
+                                               makeSeq(Constant::int32(1),
+                                                       Constant::int32(static_cast<int32_t>(
+                                                           sbe::value::TypeTags::NumberInt32))))),
+                        make<PathCompare>(
+                            Operations::Lt,
+                            make<FunctionCall>("getParam",
+                                               makeSeq(Constant::int32(2),
+                                                       Constant::int32(static_cast<int32_t>(
+                                                           sbe::value::TypeTags::NumberInt32))))))),
+                make<Variable>(scanLabel))),
+            _node(scanForTest()))));
+
+    // Check that correct sbe slot entry is added to inputParamToSlotMap
+    auto slotId1 = getSlotId(1);
+    ASSERT(slotId1 != boost::none);
+    ASSERT_EQUALS(*slotId1, 2);
+
+    auto slotId2 = getSlotId(2);
+    ASSERT(slotId2 != boost::none);
+    ASSERT_EQUALS(*slotId2, 3);
+
+    runNodeVariation(
+        ctx,
+        "filter for getParam duplicated by the optimizer: a >= Constant1, a < Constant2",
+        _node(make<FilterNode>(
+            _path(make<EvalFilter>(
+                make<PathGet>(
+                    "a",
+                    make<PathComposeM>(
+                        make<PathCompare>(
+                            Operations::Gte,
+                            make<FunctionCall>("getParam",
+                                               makeSeq(Constant::int32(1),
+                                                       Constant::int32(static_cast<int32_t>(
+                                                           sbe::value::TypeTags::NumberInt32))))),
+                        make<PathCompare>(
+                            Operations::Lt,
+                            make<FunctionCall>("getParam",
+                                               makeSeq(Constant::int32(2),
+                                                       Constant::int32(static_cast<int32_t>(
+                                                           sbe::value::TypeTags::NumberInt32))))))),
+                make<Variable>(scanLabel))),
+            _node(scanForTest()))));
+
+    // Check that correct sbe slot entry is added to inputParamToSlotMap
+    slotId1 = getSlotId(1);
+    ASSERT(slotId1 != boost::none);
+    ASSERT_EQUALS(*slotId1, 2);
+
+    slotId2 = getSlotId(2);
+    ASSERT(slotId2 != boost::none);
+    ASSERT_EQUALS(*slotId2, 3);
+}
+
 TEST_F(ABTPlanGeneration, LowerGroupByNode) {
     GoldenTestContext ctx(&goldenTestConfig);
     ctx.printTestHeader(GoldenTestContext::HeaderFormat::Text);
@@ -431,8 +774,7 @@ TEST_F(ABTPlanGeneration, LowerGroupByNode) {
     for (const auto& groupType : groupTypes) {
         runNodeVariation(
             ctx,
-            str::stream() << "GroupByNode one output with type "
-                          << GroupNodeTypeEnum::toString[static_cast<int>(groupType)],
+            str::stream() << "GroupByNode one output with type " << toStringData(groupType),
             _node(make<GroupByNode>(
                 ProjectionNameVector{"key1", "key2"},
                 ProjectionNameVector{"outFunc1"},
@@ -442,8 +784,7 @@ TEST_F(ABTPlanGeneration, LowerGroupByNode) {
 
         runNodeVariation(
             ctx,
-            str::stream() << "GroupByNode multiple outputs with type "
-                          << GroupNodeTypeEnum::toString[static_cast<int>(groupType)],
+            str::stream() << "GroupByNode multiple outputs with type " << toStringData(groupType),
             _node(make<GroupByNode>(
                 ProjectionNameVector{"key1", "key2"},
                 ProjectionNameVector{"outFunc1", "outFunc2"},
@@ -601,7 +942,7 @@ TEST_F(ABTPlanGeneration, LowerMergeJoinNode) {
             "scan1");
         runNodeVariation(ctx,
                          str::stream() << "Lower merge join with one projection (collation="
-                                       << CollationOpEnum::toString[static_cast<int>(op)] << ")",
+                                       << toStringData(op) << ")",
                          _node(make<MergeJoinNode>(ProjectionNameVector{ProjectionName{"proj0"}},
                                                    ProjectionNameVector{ProjectionName{"proj1"}},
                                                    std::vector<CollationOp>{op},
@@ -626,8 +967,7 @@ TEST_F(ABTPlanGeneration, LowerMergeJoinNode) {
             runNodeVariation(
                 ctx,
                 str::stream() << "Lower merge join with two projections (collation="
-                              << CollationOpEnum::toString[static_cast<int>(op1)] << ", "
-                              << CollationOpEnum::toString[static_cast<int>(op2)] << ")",
+                              << toStringData(op1) << ", " << toStringData(op2) << ")",
                 _node(make<MergeJoinNode>(
                     ProjectionNameVector{ProjectionName{"proj0"}, ProjectionName{"proj2"}},
                     ProjectionNameVector{ProjectionName{"proj1"}, ProjectionName{"proj3"}},
@@ -651,7 +991,7 @@ TEST_F(ABTPlanGeneration, LowerMergeJoinNode) {
             {ProjectionName{"proj3"}});
         runNodeVariation(ctx,
                          str::stream() << "Lower merge join with required projection (collation="
-                                       << CollationOpEnum::toString[static_cast<int>(op1)] << ")",
+                                       << toStringData(op1) << ")",
                          _node(make<FilterNode>(makeEquals("proj3", Constant::str("NYC")),
                                                 _node(make<MergeJoinNode>(
                                                     ProjectionNameVector{ProjectionName{"proj0"}},
@@ -692,8 +1032,8 @@ TEST_F(ABTPlanGeneration, LowerNestedLoopJoinNode) {
 
         runNodeVariation(
             ctx,
-            str::stream() << "Nested loop join with equality predicate ("
-                          << JoinTypeEnum::toString[static_cast<int>(joinType)] << " join)",
+            str::stream() << "Nested loop join with equality predicate (" << toStringData(joinType)
+                          << " join)",
             _node(make<FilterNode>(makeEquals("proj2", Constant::int32(10024)), std::move(nlj))));
     }
 }
@@ -803,15 +1143,13 @@ TEST_F(ABTPlanGeneration, LowerSortedMergeNode) {
         runVariations(properties::CollationRequirement(
                           ProjectionCollationSpec({{ProjectionName{"proj0"}, op}})),
                       op,
-                      str::stream()
-                          << "sorted on `a` " << CollationOpEnum::toString[static_cast<int>(op)]);
+                      str::stream() << "sorted on `a` " << toStringData(op));
         for (auto& op2 : ops) {
             runVariations(properties::CollationRequirement(ProjectionCollationSpec(
                               {{ProjectionName{"proj0"}, op}, {ProjectionName{"proj1"}, op2}})),
                           op,
-                          str::stream()
-                              << "sorted on `a` " << CollationOpEnum::toString[static_cast<int>(op)]
-                              << " and `b` " << CollationOpEnum::toString[static_cast<int>(op2)]);
+                          str::stream() << "sorted on `a` " << toStringData(op) << " and `b` "
+                                        << toStringData(op2));
         }
     }
 }
@@ -835,11 +1173,8 @@ TEST_F(ABTPlanGeneration, LowerSpoolNodes) {
                 _node(make<SpoolConsumerNode>(spoolConsumeType, 1, ProjectionNameVector{"proj0"}));
             runNodeVariation(
                 ctx,
-                str::stream() << "Spool in union with "
-                              << SpoolProducerTypeEnum::toString[static_cast<int>(spoolProdType)]
-                              << " producer and "
-                              << SpoolConsumerTypeEnum::toString[static_cast<int>(spoolConsumeType)]
-                              << " consumer",
+                str::stream() << "Spool in union with " << toStringData(spoolProdType)
+                              << " producer and " << toStringData(spoolConsumeType) << " consumer",
                 _node(make<UnionNode>(ProjectionNameVector{"proj0"},
                                       makeSeq(std::move(leftTree), std::move(rightTree)))));
         }
@@ -952,5 +1287,15 @@ DEATH_TEST_F(ABTPlanGeneration,
     runNodeVariation(ctx, "Do not lower scan node", _node(make<ScanNode>("proj0", "scan0")));
 }
 
+TEST_F(ABTPlanGeneration, LowerBinaryOpEqMemberRHSArray) {
+    // Lower BinaryOp [EqMember] where the type of RHS is array.
+    std::string output = autoUpdateExpressionVariation(
+        _binary("EqMember", "hello"_cstr, _carray("1"_cdouble, "2"_cdouble, "3"_cdouble))._n);
+
+    ASSERT_STR_EQ_AUTO(  // NOLINT
+        "isMember(\"hello\", [1L, 2L, 3L]) ",
+        output);
+}
+
 }  // namespace
 }  // namespace mongo::optimizer
