@@ -27,22 +27,29 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/exec/sbe/expressions/expression.h"
-
-#include <iomanip>
+#include <absl/container/flat_hash_map.h>
+#include <absl/meta/type_traits.h>
+#include <functional>
 #include <sstream>
-#include <stack>
 #include <vector>
 
+#include <absl/container/inlined_vector.h>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/bson/ordering.h"
 #include "mongo/db/exec/sbe/expressions/compile_ctx.h"
+#include "mongo/db/exec/sbe/expressions/expression.h"
+#include "mongo/db/exec/sbe/expressions/runtime_environment.h"
 #include "mongo/db/exec/sbe/size_estimator.h"
-#include "mongo/db/exec/sbe/stages/spool.h"
 #include "mongo/db/exec/sbe/stages/stages.h"
+#include "mongo/db/exec/sbe/util/print_options.h"
 #include "mongo/db/exec/sbe/values/arith_common.h"
 #include "mongo/db/exec/sbe/values/value_printer.h"
 #include "mongo/db/exec/sbe/vm/datetime.h"
+#include "mongo/db/exec/sbe/vm/label.h"
+#include "mongo/db/query/datetime/date_time_support.h"
+#include "mongo/stdx/unordered_map.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/str.h"
 
 namespace mongo {
@@ -54,7 +61,7 @@ namespace sbe {
  */
 EVariable* getFrameVariable(EExpression* e) {
     auto var = e->as<EVariable>();
-    if (var && var->getFrameId() && !var->isMoveFrom()) {
+    if (var && var->getFrameId()) {
         return var;
     }
     return nullptr;
@@ -65,7 +72,7 @@ EVariable* getFrameVariable(EExpression* e) {
  */
 vm::Instruction::Parameter getParam(EVariable* var) {
     if (var) {
-        return {(int)var->getSlotId(), var->getFrameId()};
+        return {(int)var->getSlotId(), var->isMoveFrom(), var->getFrameId()};
     } else {
         return {};
     }
@@ -202,118 +209,88 @@ std::unique_ptr<EExpression> EPrimBinary::clone() const {
     }
 }
 
+
+/*
+ * Given a vector of clauses named [lhs1,...,lhsN-1, rhs], and a boolean isDisjunctive to indicate
+ * whether we are ANDing or ORing the clauses, we output the appropriate short circuiting
+ * CodeFragment. For AND (conjunctive) we compile them as following byte code:
+ * @true1:    lhs1
+ *            jumpNothing @end
+ *            jumpFalse @false
+ * ...
+ * @trueN-1:  lhsN-1
+ *            jumpNothing @end
+ *            jumpFalse @false
+ * @trueN:    rhs
+ *            jmp @end
+ * @false:    push false
+ * @end:
+ *
+ * For OR (disjunctive) we compile them as:
+ * @false1:   lhs1
+ *            jumpNothing @end
+ *            jumpTrue @true
+ * ...
+ * @falseN-1: lhsN-1
+ *            jumpNothing @end
+ *            jumpTrue @true
+ * @tfalseN:  rhs
+ *            jmp @end
+ * @true:     push true
+ * @end:
+ */
+vm::CodeFragment buildShortCircuitCode(CompileCtx& ctx,
+                                       const std::vector<const EExpression*>& clauses,
+                                       bool isDisjunction) {
+    return withNewLabels(ctx, [&](vm::LabelId endLabel, vm::LabelId resultLabel) {
+        // Build code fragment for all but the last clause, which is used for the final result
+        // branch.
+        tassert(7858700,
+                "There should be two or more clauses when compiling a logicAnd/logicOr.",
+                clauses.size() >= 2);
+        vm::CodeFragment code;
+        for (size_t i = 0; i < clauses.size() - 1; i++) {
+            auto clauseCode = clauses.at(i)->compileDirect(ctx);
+            clauseCode.appendLabelJumpNothing(endLabel);
+
+            if (isDisjunction) {
+                clauseCode.appendLabelJumpTrue(resultLabel);
+            } else {
+                clauseCode.appendLabelJumpFalse(resultLabel);
+            }
+
+            code.append(std::move(clauseCode));
+        }
+
+        // Build code fragment for final clause.
+        auto finalClause = clauses.back()->compileDirect(ctx);
+        finalClause.appendLabelJump(endLabel);
+
+        // Build code fragment for the short-circuited result.
+        vm::CodeFragment resultBranch;
+        resultBranch.appendLabel(resultLabel);
+        resultBranch.appendConstVal(value::TypeTags::Boolean,
+                                    value::bitcastFrom<bool>(isDisjunction));
+
+        // Only one of `finalClause` or `resultBranch` will execute, so the stack size adjustment
+        // should only be made one time here, rather than one adjustment for each CodeFragment.
+        code.append(std::move(finalClause), std::move(resultBranch));
+        code.appendLabel(endLabel);
+        return code;
+    });
+}
+
 vm::CodeFragment EPrimBinary::compileDirect(CompileCtx& ctx) const {
     const bool hasCollatorArg = (_nodes.size() == 3);
 
     invariant(!hasCollatorArg || isComparisonOp(_op));
 
     if (_op == EPrimBinary::logicAnd) {
-        /*
-         * We collect all connected AND clauses, named [lhs1,...,lhsN-1, rhs],
-         *  and compile them as following byte code:
-         *
-         * @true1:    lhs1
-         *            jumpNothing @end
-         *            jumpFalse @false
-         * ...
-         * @trueN-1:  lhsN-1
-         *            jumpNothing @end
-         *            jumpFalse @false
-         * @trueN:    rhs
-         *            jmp @end
-         * @false:    push false
-         * @end:
-         */
         auto clauses = collectAndClauses();
-        invariant(clauses.size() >= 2);
-
-        return withNewLabels(ctx, [&](vm::LabelId endLabel, vm::LabelId falseLabel) {
-            vm::CodeFragment code;
-
-            // Build code fragment for @false
-            vm::CodeFragment codeFalseBranch;
-            codeFalseBranch.appendLabel(falseLabel);
-            codeFalseBranch.appendConstVal(value::TypeTags::Boolean,
-                                           value::bitcastFrom<bool>(false));
-
-            // Build code fragment for @trueN
-            auto it = clauses.rbegin();
-            auto rhs = (*it)->compileDirect(ctx);
-            rhs.appendLabelJump(endLabel);
-
-            code.append(std::move(rhs), std::move(codeFalseBranch));
-
-            ++it;
-            invariant(it != clauses.rend());
-
-            // Build code fragment for @trueN-1 to @true1
-            for (; it != clauses.rend(); ++it) {
-                auto lhs = (*it)->compileDirect(ctx);
-                lhs.appendLabelJumpNothing(endLabel);
-                lhs.appendLabelJumpFalse(falseLabel);
-
-                lhs.append(std::move(code));
-                code = std::move(lhs);
-            }
-
-            // Append the end label
-            code.appendLabel(endLabel);
-
-            return code;
-        });
+        return buildShortCircuitCode(ctx, clauses, false /*isDisjunction*/);
     } else if (_op == EPrimBinary::logicOr) {
-        /*
-         * We collect all connected OR clauses, named [lhs1,...,lhsN-1, rhs],
-         * and compile them as following byte code:
-         *
-         * @false1:   lhs1
-         *            jumpNothing @end
-         *            jumpTrue @true
-         * ...
-         * @falseN-1: lhsN-1
-         *            jumpNothing @end
-         *            jumpTrue @true
-         * @tfalseN:  rhs
-         *            jmp @end
-         * @true:     push true
-         * @end:
-         */
         auto clauses = collectOrClauses();
-        invariant(clauses.size() >= 2);
-
-        return withNewLabels(ctx, [&](vm::LabelId endLabel, vm::LabelId trueLabel) {
-            vm::CodeFragment code;
-
-            auto it = clauses.rbegin();
-
-            // Build code fragment for @true
-            vm::CodeFragment codeTrueBranch;
-            codeTrueBranch.appendLabel(trueLabel);
-            codeTrueBranch.appendConstVal(value::TypeTags::Boolean, value::bitcastFrom<bool>(true));
-
-            // Build code fragment for @falseN
-            auto rhs = (*it)->compileDirect(ctx);
-            rhs.appendLabelJump(endLabel);
-            code.append(std::move(rhs), std::move(codeTrueBranch));
-
-            ++it;
-            invariant(it != clauses.rend());
-
-            // Build code fragment for @falseN-1 to @true1
-            for (; it != clauses.rend(); ++it) {
-                auto lhs = (*it)->compileDirect(ctx);
-                lhs.appendLabelJumpNothing(endLabel);
-                lhs.appendLabelJumpTrue(trueLabel);
-
-                lhs.append(std::move(code));
-                code = std::move(lhs);
-            }
-
-            // Append the end label
-            code.appendLabel(endLabel);
-
-            return code;
-        });
+        return buildShortCircuitCode(ctx, clauses, true /*isDisjunction*/);
     } else if (_op == EPrimBinary::fillEmpty) {
         // Special cases: rhs is trivial to evaluate -> avoid a jump
         if (EConstant* rhsConst = _nodes[1]->as<EConstant>()) {
@@ -586,6 +563,14 @@ struct BuiltinFn {
 
 /**
  * The map of recognized builtin functions.
+ *
+ * *************************************************************************************
+ * IMPORTANT:
+ *   Iff the third argument ('aggregate') to BuiltinFn{} is true, the actual arity
+ *   WILL BE INCREMENTED BY THE COMPILER (EFunction::compileDirect()) and it will push
+ *   an extra arg onto the stack (the accumulator value) for such fns. So an arityTest
+ *   fn with body {return n == 1} for the arity test really means an arity-2 function.
+ * *************************************************************************************
  */
 static stdx::unordered_map<std::string, BuiltinFn> kBuiltinFunctions = {
     {"dateDiff",
@@ -632,7 +617,7 @@ static stdx::unordered_map<std::string, BuiltinFn> kBuiltinFunctions = {
      BuiltinFn{[](size_t n) { return n == 3; }, vm::Builtin::newArrayFromRange, false}},
     {"newObj", BuiltinFn{[](size_t n) { return n % 2 == 0; }, vm::Builtin::newObj, false}},
     {"newBsonObj", BuiltinFn{[](size_t n) { return n % 2 == 0; }, vm::Builtin::newBsonObj, false}},
-    {"makeBsonObj", BuiltinFn{[](size_t n) { return n >= 2; }, vm::Builtin::makeBsonObj, false}},
+    {"makeBsonObj", BuiltinFn{[](size_t n) { return n >= 3; }, vm::Builtin::makeBsonObj, false}},
     {"ksToString", BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::ksToString, false}},
     {"ks",
      BuiltinFn{[](size_t n) { return n >= 3 && n <= Ordering::kMaxCompoundIndexKeys + 3; },
@@ -645,11 +630,12 @@ static stdx::unordered_map<std::string, BuiltinFn> kBuiltinFunctions = {
     {"abs", BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::abs, false}},
     {"ceil", BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::ceil, false}},
     {"floor", BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::floor, false}},
-    {"trunc", BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::trunc, false}},
+    {"trunc", BuiltinFn{[](size_t n) { return n == 1 || n == 2; }, vm::Builtin::trunc, false}},
     {"exp", BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::exp, false}},
     {"ln", BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::ln, false}},
     {"log10", BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::log10, false}},
     {"sqrt", BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::sqrt, false}},
+    {"pow", BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::pow, false}},
     {"addToArray", BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::addToArray, true}},
     {"addToArrayCapped",
      BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::addToArrayCapped, true}},
@@ -660,8 +646,13 @@ static stdx::unordered_map<std::string, BuiltinFn> kBuiltinFunctions = {
     {"collAddToSet", BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::collAddToSet, true}},
     {"collAddToSetCapped",
      BuiltinFn{[](size_t n) { return n == 3; }, vm::Builtin::collAddToSetCapped, true}},
+    {"setToArray", BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::setToArray, false}},
+    {"fillType", BuiltinFn{[](size_t n) { return n == 3; }, vm::Builtin::fillType, false}},
     {"doubleDoubleSum",
      BuiltinFn{[](size_t n) { return n > 0; }, vm::Builtin::doubleDoubleSum, false}},
+    {"convertSimpleSumToDoubleDoubleSum",
+     BuiltinFn{
+         [](size_t n) { return n == 1; }, vm::Builtin::convertSimpleSumToDoubleDoubleSum, false}},
     {"aggDoubleDoubleSum",
      BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggDoubleDoubleSum, true}},
     {"aggMergeDoubleDoubleSums",
@@ -682,8 +673,12 @@ static stdx::unordered_map<std::string, BuiltinFn> kBuiltinFunctions = {
     {"bitTestPosition",
      BuiltinFn{[](size_t n) { return n == 3; }, vm::Builtin::bitTestPosition, false}},
     {"bsonSize", BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::bsonSize, false}},
+    {"strLenBytes", BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::strLenBytes, false}},
     {"toLower", BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::toLower, false}},
     {"toUpper", BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::toUpper, false}},
+    {"trim", BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::trim, false}},
+    {"ltrim", BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::ltrim, false}},
+    {"rtrim", BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::rtrim, false}},
     {"coerceToBool", BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::coerceToBool, false}},
     {"coerceToString",
      BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::coerceToString, false}},
@@ -710,7 +705,6 @@ static stdx::unordered_map<std::string, BuiltinFn> kBuiltinFunctions = {
     {"aggConcatArraysCapped",
      BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::aggConcatArraysCapped, true}},
     {"isMember", BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::isMember, false}},
-    {"collIsMember", BuiltinFn{[](size_t n) { return n == 3; }, vm::Builtin::collIsMember, false}},
     {"indexOfBytes",
      BuiltinFn{[](size_t n) { return n == 3 || n == 4; }, vm::Builtin::indexOfBytes, false}},
     {"indexOfCP",
@@ -727,6 +721,7 @@ static stdx::unordered_map<std::string, BuiltinFn> kBuiltinFunctions = {
     {"setDifference",
      BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::setDifference, false}},
     {"setEquals", BuiltinFn{[](size_t n) { return n >= 2; }, vm::Builtin::setEquals, false}},
+    {"setIsSubset", BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::setIsSubset, false}},
     {"collSetUnion", BuiltinFn{[](size_t n) { return n >= 1; }, vm::Builtin::collSetUnion, false}},
     {"collSetIntersection",
      BuiltinFn{[](size_t n) { return n >= 1; }, vm::Builtin::collSetIntersection, false}},
@@ -734,9 +729,13 @@ static stdx::unordered_map<std::string, BuiltinFn> kBuiltinFunctions = {
      BuiltinFn{[](size_t n) { return n == 3; }, vm::Builtin::collSetDifference, false}},
     {"collSetEquals",
      BuiltinFn{[](size_t n) { return n >= 3; }, vm::Builtin::collSetEquals, false}},
+    {"collSetIsSubset",
+     BuiltinFn{[](size_t n) { return n == 3; }, vm::Builtin::collSetIsSubset, false}},
     {"aggSetUnion", BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggSetUnion, true}},
     {"aggSetUnionCapped",
      BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::aggSetUnionCapped, true}},
+    {"aggCollSetUnion",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::aggCollSetUnion, true}},
     {"aggCollSetUnionCapped",
      BuiltinFn{[](size_t n) { return n == 3; }, vm::Builtin::aggCollSetUnionCapped, true}},
     {"runJsPredicate",
@@ -768,18 +767,268 @@ static stdx::unordered_map<std::string, BuiltinFn> kBuiltinFunctions = {
     {"sortKeyComponentVectorGetElement",
      BuiltinFn{
          [](size_t n) { return n == 2; }, vm::Builtin::sortKeyComponentVectorGetElement, false}},
+    {"sortKeyComponentVectorToArray",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::sortKeyComponentVectorToArray, false}},
     {"tsSecond", BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::tsSecond, false}},
     {"tsIncrement", BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::tsIncrement, false}},
     {"typeMatch", BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::typeMatch, false}},
     {"dateTrunc", BuiltinFn{[](size_t n) { return n == 6; }, vm::Builtin::dateTrunc, false}},
-    {"_internalLeast",
-     BuiltinFn{[](size_t n) { return n == 1 || n == 2; }, vm::Builtin::internalLeast, false}},
-    {"_internalGreatest",
-     BuiltinFn{[](size_t n) { return n == 1 || n == 2; }, vm::Builtin::internalGreatest, false}},
+    {"getSortKeyAsc",
+     BuiltinFn{[](size_t n) { return n == 1 || n == 2; }, vm::Builtin::getSortKeyAsc, false}},
+    {"getSortKeyDesc",
+     BuiltinFn{[](size_t n) { return n == 1 || n == 2; }, vm::Builtin::getSortKeyDesc, false}},
+    {"getNonLeafSortKeyAsc",
+     BuiltinFn{
+         [](size_t n) { return n == 1 || n == 2; }, vm::Builtin::getNonLeafSortKeyAsc, false}},
+    {"getNonLeafSortKeyDesc",
+     BuiltinFn{
+         [](size_t n) { return n == 1 || n == 2; }, vm::Builtin::getNonLeafSortKeyDesc, false}},
     {"objectToArray",
      BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::objectToArray, false}},
     {"arrayToObject",
      BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::arrayToObject, false}},
+    {"array", BuiltinFn{kAnyNumberOfArgs, vm::Builtin::newArray, false}},
+    {"aggFirstNNeedsMoreInput",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggFirstNNeedsMoreInput, false}},
+    {"aggFirstN", BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::aggFirstN, false}},
+    {"aggFirstNMerge",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggFirstNMerge, true}},
+    {"aggFirstNFinalize",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggFirstNFinalize, false}},
+    {"aggLastN", BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggLastN, true}},
+    {"aggLastNMerge", BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggLastNMerge, true}},
+    {"aggLastNFinalize",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggLastNFinalize, false}},
+    {"aggTopN", BuiltinFn{[](size_t n) { return n >= 3; }, vm::Builtin::aggTopN, true}},
+    {"aggTopNArray", BuiltinFn{[](size_t n) { return n >= 2; }, vm::Builtin::aggTopNArray, true}},
+    {"aggTopNMerge", BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::aggTopNMerge, true}},
+    {"aggTopNFinalize",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::aggTopNFinalize, false}},
+    {"aggBottomN", BuiltinFn{[](size_t n) { return n >= 3; }, vm::Builtin::aggBottomN, true}},
+    {"aggBottomNArray",
+     BuiltinFn{[](size_t n) { return n >= 2; }, vm::Builtin::aggBottomNArray, true}},
+    {"aggBottomNMerge",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::aggBottomNMerge, true}},
+    {"aggBottomNFinalize",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::aggBottomNFinalize, false}},
+    {"aggMaxN", BuiltinFn{[](size_t n) { return n == 1 || n == 2; }, vm::Builtin::aggMaxN, true}},
+    {"aggMaxNMerge",
+     BuiltinFn{[](size_t n) { return n == 1 || n == 2; }, vm::Builtin::aggMaxNMerge, true}},
+    {"aggMaxNFinalize",
+     BuiltinFn{[](size_t n) { return n == 1 || n == 2; }, vm::Builtin::aggMaxNFinalize, false}},
+    {"aggMinN", BuiltinFn{[](size_t n) { return n == 1 || n == 2; }, vm::Builtin::aggMinN, true}},
+    {"aggMinNMerge",
+     BuiltinFn{[](size_t n) { return n == 1 || n == 2; }, vm::Builtin::aggMinNMerge, true}},
+    {"aggMinNFinalize",
+     BuiltinFn{[](size_t n) { return n == 1 || n == 2; }, vm::Builtin::aggMinNFinalize, false}},
+    {"aggRank", BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::aggRank, true}},
+    {"aggRankColl", BuiltinFn{[](size_t n) { return n == 3; }, vm::Builtin::aggRankColl, true}},
+    {"aggDenseRank", BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::aggDenseRank, true}},
+    {"aggDenseRankColl",
+     BuiltinFn{[](size_t n) { return n == 3; }, vm::Builtin::aggDenseRankColl, true}},
+    {"aggRankFinalize",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggRankFinalize, false}},
+    {"aggExpMovingAvg",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggExpMovingAvg, true}},
+    {"aggExpMovingAvgFinalize",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggExpMovingAvgFinalize, false}},
+    {"aggRemovableSumAdd",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggRemovableSumAdd, true}},
+    {"aggRemovableSumRemove",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggRemovableSumRemove, true}},
+    {"aggRemovableSumFinalize",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggRemovableSumFinalize, false}},
+    {"aggIntegralInit",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::aggIntegralInit, false}},
+    {"aggIntegralAdd",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::aggIntegralAdd, true}},
+    {"aggIntegralRemove",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::aggIntegralRemove, true}},
+    {"aggIntegralFinalize",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggIntegralFinalize, false}},
+    {"aggDerivativeFinalize",
+     BuiltinFn{[](size_t n) { return n == 5; }, vm::Builtin::aggDerivativeFinalize, false}},
+    {"aggCovarianceAdd",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::aggCovarianceAdd, true}},
+    {"aggCovarianceRemove",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::aggCovarianceRemove, true}},
+    {"aggCovarianceSampFinalize",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggCovarianceSampFinalize, false}},
+    {"aggCovariancePopFinalize",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggCovariancePopFinalize, false}},
+    {"aggRemovablePushAdd",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggRemovablePushAdd, true}},
+    {"aggRemovablePushRemove",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggRemovablePushRemove, true}},
+    {"aggRemovablePushFinalize",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggRemovablePushFinalize, false}},
+    {"aggRemovableStdDevAdd",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggRemovableStdDevAdd, true}},
+    {"aggRemovableStdDevRemove",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggRemovableStdDevRemove, true}},
+    {"aggRemovableStdDevSampFinalize",
+     BuiltinFn{
+         [](size_t n) { return n == 1; }, vm::Builtin::aggRemovableStdDevSampFinalize, false}},
+    {"aggRemovableStdDevPopFinalize",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggRemovableStdDevPopFinalize, false}},
+    {"aggRemovableAvgFinalize",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::aggRemovableAvgFinalize, false}},
+    {"aggLinearFillCanAdd",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggLinearFillCanAdd, false}},
+    {"aggLinearFillAdd",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::aggLinearFillAdd, true}},
+    {"aggLinearFillFinalize",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::aggLinearFillFinalize, false}},
+    {"aggRemovableFirstNInit",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggRemovableFirstNInit, false}},
+    {"aggRemovableFirstNAdd",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggRemovableFirstNAdd, true}},
+    {"aggRemovableFirstNRemove",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggRemovableFirstNRemove, true}},
+    {"aggRemovableFirstNFinalize",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggRemovableFirstNFinalize, false}},
+    {"aggRemovableLastNInit",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggRemovableLastNInit, false}},
+    {"aggRemovableLastNAdd",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggRemovableLastNAdd, true}},
+    {"aggRemovableLastNRemove",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggRemovableLastNRemove, true}},
+    {"aggRemovableLastNFinalize",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggRemovableLastNFinalize, false}},
+    {"aggRemovableAddToSetInit",
+     BuiltinFn{[](size_t n) { return n == 0; }, vm::Builtin::aggRemovableAddToSetInit, false}},
+    {"aggRemovableAddToSetCollInit",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggRemovableAddToSetCollInit, false}},
+    {"aggRemovableAddToSetAdd",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::aggRemovableAddToSetAdd, true}},
+    {"aggRemovableAddToSetRemove",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggRemovableAddToSetRemove, true}},
+    {"aggRemovableAddToSetFinalize",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggRemovableAddToSetFinalize, false}},
+    {"aggRemovableMinMaxNCollInit",
+     BuiltinFn{[](size_t n) { return n == 3; }, vm::Builtin::aggRemovableMinMaxNCollInit, false}},
+    {"aggRemovableMinMaxNInit",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::aggRemovableMinMaxNInit, false}},
+    {"aggRemovableMinMaxNAdd",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggRemovableMinMaxNAdd, true}},
+    {"aggRemovableMinMaxNRemove",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggRemovableMinMaxNRemove, true}},
+    {"aggRemovableMinNFinalize",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggRemovableMinNFinalize, false}},
+    {"aggRemovableMaxNFinalize",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggRemovableMaxNFinalize, false}},
+    {"aggRemovableTopNInit",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::aggRemovableTopNInit, false}},
+    {"aggRemovableTopNAdd",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::aggRemovableTopNAdd, true}},
+    {"aggRemovableTopNRemove",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::aggRemovableTopNRemove, true}},
+    {"aggRemovableTopNFinalize",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggRemovableTopNFinalize, false}},
+    {"aggRemovableBottomNInit",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::aggRemovableBottomNInit, false}},
+    {"aggRemovableBottomNAdd",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::aggRemovableBottomNAdd, true}},
+    {"aggRemovableBottomNRemove",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::aggRemovableBottomNRemove, true}},
+    {"aggRemovableBottomNFinalize",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::aggRemovableBottomNFinalize, false}},
+    {"valueBlockExists",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::valueBlockExists, false}},
+    {"valueBlockTypeMatch",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::valueBlockTypeMatch, false}},
+    {"valueBlockIsTimezone",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::valueBlockIsTimezone, false}},
+    {"valueBlockFillEmpty",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::valueBlockFillEmpty, false}},
+    {"valueBlockFillEmptyBlock",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::valueBlockFillEmptyBlock, false}},
+    {"valueBlockFillType",
+     BuiltinFn{[](size_t n) { return n == 3; }, vm::Builtin::valueBlockFillType, false}},
+    {"valueBlockAggMin",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::valueBlockAggMin, true}},
+    {"valueBlockAggMax",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::valueBlockAggMax, true}},
+    {"valueBlockAggCount",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::valueBlockAggCount, true}},
+    {"valueBlockAggSum",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::valueBlockAggSum, true}},
+    {"valueBlockAggDoubleDoubleSum",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::valueBlockAggDoubleDoubleSum, true}},
+    {"valueBlockAggTopN",
+     BuiltinFn{[](size_t n) { return n >= 4; }, vm::Builtin::valueBlockAggTopN, true}},
+    {"valueBlockAggTopNArray",
+     BuiltinFn{[](size_t n) { return n >= 3; }, vm::Builtin::valueBlockAggTopNArray, true}},
+    {"valueBlockAggBottomN",
+     BuiltinFn{[](size_t n) { return n >= 4; }, vm::Builtin::valueBlockAggBottomN, true}},
+    {"valueBlockAggBottomNArray",
+     BuiltinFn{[](size_t n) { return n >= 3; }, vm::Builtin::valueBlockAggBottomNArray, true}},
+    {"valueBlockDateDiff",
+     BuiltinFn{[](size_t n) { return n == 6 || n == 7; }, vm::Builtin::valueBlockDateDiff, false}},
+    {"valueBlockDateTrunc",
+     BuiltinFn{[](size_t n) { return n == 7; }, vm::Builtin::valueBlockDateTrunc, false}},
+    {"valueBlockDateAdd",
+     BuiltinFn{[](size_t n) { return n == 6; }, vm::Builtin::valueBlockDateAdd, false}},
+    {"valueBlockAdd",
+     BuiltinFn{[](size_t n) { return n == 3; }, vm::Builtin::valueBlockAdd, false}},
+    {"valueBlockSub",
+     BuiltinFn{[](size_t n) { return n == 3; }, vm::Builtin::valueBlockSub, false}},
+    {"valueBlockMult",
+     BuiltinFn{[](size_t n) { return n == 3; }, vm::Builtin::valueBlockMult, false}},
+    {"valueBlockDiv",
+     BuiltinFn{[](size_t n) { return n == 3; }, vm::Builtin::valueBlockDiv, false}},
+    {"valueBlockGtScalar",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::valueBlockGtScalar, false}},
+    {"valueBlockGteScalar",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::valueBlockGteScalar, false}},
+    {"valueBlockEqScalar",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::valueBlockEqScalar, false}},
+    {"valueBlockNeqScalar",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::valueBlockNeqScalar, false}},
+    {"valueBlockLtScalar",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::valueBlockLtScalar, false}},
+    {"valueBlockLteScalar",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::valueBlockLteScalar, false}},
+    {"valueBlockCmp3wScalar",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::valueBlockCmp3wScalar, false}},
+    {"valueBlockCombine",
+     BuiltinFn{[](size_t n) { return n == 3; }, vm::Builtin::valueBlockCombine, false}},
+    {"valueBlockLogicalAnd",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::valueBlockLogicalAnd, false}},
+    {"valueBlockLogicalOr",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::valueBlockLogicalOr, false}},
+    {"valueBlockLogicalNot",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::valueBlockLogicalNot, false}},
+    {"valueBlockNewFill",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::valueBlockNewFill, false}},
+    {"valueBlockSize",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::valueBlockSize, false}},
+    {"valueBlockNone",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::valueBlockNone, false}},
+    {"valueBlockIsMember",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::valueBlockIsMember, false}},
+    {"valueBlockCoerceToBool",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::valueBlockCoerceToBool, false}},
+    {"valueBlockTrunc",
+     BuiltinFn{[](size_t n) { return n == 1 || n == 2; }, vm::Builtin::valueBlockTrunc, false}},
+    {"valueBlockRound",
+     BuiltinFn{[](size_t n) { return n == 1 || n == 2; }, vm::Builtin::valueBlockRound, false}},
+    {"valueBlockMod",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::valueBlockMod, false}},
+    {"valueBlockConvert",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::valueBlockConvert, false}},
+    {"valueBlockGetSortKeyAsc",
+     BuiltinFn{
+         [](size_t n) { return n == 1 || n == 2; }, vm::Builtin::valueBlockGetSortKeyAsc, false}},
+    {"valueBlockGetSortKeyDesc",
+     BuiltinFn{
+         [](size_t n) { return n == 1 || n == 2; }, vm::Builtin::valueBlockGetSortKeyDesc, false}},
+    {"cellFoldValues_F",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::cellFoldValues_F, false}},
+    {"cellFoldValues_P",
+     BuiltinFn{[](size_t n) { return n == 2; }, vm::Builtin::cellFoldValues_P, false}},
+    {"cellBlockGetFlatValuesBlock",
+     BuiltinFn{[](size_t n) { return n == 1; }, vm::Builtin::cellBlockGetFlatValuesBlock, false}},
 };
 
 /**
@@ -991,6 +1240,7 @@ vm::CodeFragment generateTraverseCellTypes(CompileCtx& ctx,
  * The map of functions that resolve directly to instructions.
  */
 static stdx::unordered_map<std::string, InstrFn> kInstrFunctions = {
+    {"makeOwn", InstrFn{1, generator<1, &vm::CodeFragment::appendMakeOwn>, false}},
     {"getElement", InstrFn{2, generator<2, &vm::CodeFragment::appendGetElement>, false}},
     {"getField", InstrFn{2, generateGetField, false}},
     {"getArraySize", InstrFn{1, generator<1, &vm::CodeFragment::appendGetArraySize>, false}},
@@ -1000,10 +1250,12 @@ static stdx::unordered_map<std::string, InstrFn> kInstrFunctions = {
      InstrFn{2, generator<2, &vm::CodeFragment::appendGetFieldOrElement>, false}},
     {"traverseP", InstrFn{3, generateTraverseP, false}},
     {"traverseF", InstrFn{3, generateTraverseF, false}},
+    {"magicTraverseF", InstrFn{5, generatorLegacy<&vm::CodeFragment::appendMagicTraverseF>, false}},
     {"traverseCsiCellValues", InstrFn{2, generateTraverseCellValues, false}},
     {"traverseCsiCellTypes", InstrFn{2, generateTraverseCellTypes, false}},
     {"setField", InstrFn{3, generatorLegacy<&vm::CodeFragment::appendSetField>, false}},
     {"sum", InstrFn{1, generatorLegacy<&vm::CodeFragment::appendSum>, true}},
+    {"count", InstrFn{0, generatorLegacy<&vm::CodeFragment::appendCount>, true}},
     {"min", InstrFn{1, generatorLegacy<&vm::CodeFragment::appendMin>, true}},
     {"max", InstrFn{1, generatorLegacy<&vm::CodeFragment::appendMax>, true}},
     {"first", InstrFn{1, generatorLegacy<&vm::CodeFragment::appendFirst>, true}},
@@ -1017,6 +1269,7 @@ static stdx::unordered_map<std::string, InstrFn> kInstrFunctions = {
     {"isNull", InstrFn{1, generator<1, &vm::CodeFragment::appendIsNull>, false}},
     {"isObject", InstrFn{1, generator<1, &vm::CodeFragment::appendIsObject>, false}},
     {"isArray", InstrFn{1, generator<1, &vm::CodeFragment::appendIsArray>, false}},
+    {"isInList", InstrFn{1, generator<1, &vm::CodeFragment::appendIsInList>, false}},
     {"isString", InstrFn{1, generator<1, &vm::CodeFragment::appendIsString>, false}},
     {"isBinData", InstrFn{1, generator<1, &vm::CodeFragment::appendIsBinData>, false}},
     {"isNaN", InstrFn{1, generator<1, &vm::CodeFragment::appendIsNaN>, false}},
@@ -1025,10 +1278,14 @@ static stdx::unordered_map<std::string, InstrFn> kInstrFunctions = {
     {"isMinKey", InstrFn{1, generator<1, &vm::CodeFragment::appendIsMinKey>, false}},
     {"isMaxKey", InstrFn{1, generator<1, &vm::CodeFragment::appendIsMaxKey>, false}},
     {"isTimestamp", InstrFn{1, generator<1, &vm::CodeFragment::appendIsTimestamp>, false}},
+    {"isKeyString", InstrFn{1, generator<1, &vm::CodeFragment::appendIsKeyString>, false}},
+    {"valueBlockApplyLambda",
+     InstrFn{3, generatorLegacy<&vm::CodeFragment::appendValueBlockApplyLambda>, false}},
 };
 }  // namespace
 
 vm::CodeFragment EFunction::compileDirect(CompileCtx& ctx) const {
+    // Built-in function compilations.
     if (auto it = kBuiltinFunctions.find(_name); it != kBuiltinFunctions.end()) {
         auto arity = _nodes.size();
         if (!it->second.arityTest(arity)) {
@@ -1041,11 +1298,8 @@ vm::CodeFragment EFunction::compileDirect(CompileCtx& ctx) const {
         // specialized variants.
         if (_name == "typeMatch" && _nodes[1]->as<EConstant>()) {
             auto [tag, val] = _nodes[1]->as<EConstant>()->getConstant();
-            if (tag == value::TypeTags::NumberInt64) {
-                auto mask = value::bitcastTo<int64_t>(val);
-                uassert(6996901,
-                        "Second argument to typeMatch() must be a 32-bit integer constant",
-                        mask >> 32 == 0 || mask >> 32 == -1);
+            if (tag == value::TypeTags::NumberInt32) {
+                auto mask = static_cast<uint32_t>(value::bitcastTo<int32_t>(val));
                 auto param = appendParameter(code, ctx, _nodes[0].get());
                 code.appendTypeMatch(param, mask);
 
@@ -1102,8 +1356,9 @@ vm::CodeFragment EFunction::compileDirect(CompileCtx& ctx) const {
         code.appendFunction(it->second.builtin, arity);
 
         return code;
-    }
+    }  // if built-in function
 
+    // Instruction compilations from here down.
     if (auto it = kInstrFunctions.find(_name); it != kInstrFunctions.end()) {
         if (it->second.arity != _nodes.size()) {
             uasserted(4822845,
@@ -1120,8 +1375,20 @@ vm::CodeFragment EFunction::compileDirect(CompileCtx& ctx) const {
         return (it->second.generate)(ctx, _nodes, it->second.aggregate);
     }
 
+    if (_name == "aggState") {
+        uassert(7695204,
+                "aggregate function call: aggState occurs in the non-aggregate context.",
+                ctx.aggExpression && ctx.accumulator);
+        uassert(7695205,
+                str::stream() << "function call: aggState has wrong arity: " << _nodes.size(),
+                _nodes.size() == 0);
+        vm::CodeFragment code;
+        code.appendMoveVal(ctx.accumulator);
+        return code;
+    }
+
     uasserted(4822847, str::stream() << "unknown function call: " << _name);
-}
+}  // compileDirect
 
 std::vector<DebugPrinter::Block> EFunction::debugPrint() const {
     std::vector<DebugPrinter::Block> ret;
