@@ -27,7 +27,18 @@
  *    it in the license file.
  */
 
+#include <memory>
+#include <utility>
+
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
 #include "mongo/db/exec/sbe/expression_test_base.h"
+#include "mongo/db/exec/sbe/expressions/expression.h"
+#include "mongo/db/exec/sbe/values/slot.h"
+#include "mongo/db/exec/sbe/values/value.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/unittest/golden_test.h"
 
 namespace mongo::sbe {
 using SBELambdaTest = GoldenEExpressionTestFixture;
