@@ -27,8 +27,21 @@
  *    it in the license file.
  */
 
+#include <cstdint>
+#include <memory>
+#include <tuple>
+#include <utility>
+
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
 #include "mongo/db/exec/sbe/expression_test_base.h"
+#include "mongo/db/exec/sbe/values/slot.h"
+#include "mongo/db/exec/sbe/values/value.h"
+#include "mongo/db/exec/sbe/vm/vm.h"
 #include "mongo/db/query/sbe_stage_builder_helpers.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
 
 namespace mongo::sbe {
 
@@ -53,14 +66,22 @@ protected:
         ASSERT_EQUALS(tag, sbe::value::TypeTags::Nothing);
         ASSERT_EQUALS(val, 0);
     }
+
+    void runAndAssertBoolean(const vm::CodeFragment* compiledExpr, bool expected) {
+        auto [tag, val] = runCompiledExpression(compiledExpr);
+        value::ValueGuard guard(tag, val);
+
+        ASSERT(tag == value::TypeTags::Boolean);
+        ASSERT_EQUALS(value::bitcastTo<bool>(val), expected);
+    }
 };
 
 TEST_F(SBEBuiltinSetOpTest, ComputesSetUnion) {
     value::OwnedValueAccessor slotAccessor1, slotAccessor2;
     auto arrSlot1 = bindAccessor(&slotAccessor1);
     auto arrSlot2 = bindAccessor(&slotAccessor2);
-    auto setUnionExpr = sbe::makeE<sbe::EFunction>(
-        "setUnion", sbe::makeEs(makeE<EVariable>(arrSlot1), makeE<EVariable>(arrSlot2)));
+    auto setUnionExpr = stage_builder::makeFunction(
+        "setUnion", stage_builder::makeVariable(arrSlot1), stage_builder::makeVariable(arrSlot2));
     auto compiledExpr = compileExpression(*setUnionExpr);
 
     auto [arrTag1, arrVal1] = makeArray(BSON_ARRAY(1 << 2));
@@ -84,8 +105,8 @@ TEST_F(SBEBuiltinSetOpTest, ReturnsNothingSetUnion) {
     value::OwnedValueAccessor slotAccessor1, slotAccessor2;
     auto arrSlot1 = bindAccessor(&slotAccessor1);
     auto arrSlot2 = bindAccessor(&slotAccessor2);
-    auto setUnionExpr = sbe::makeE<sbe::EFunction>(
-        "setUnion", sbe::makeEs(makeE<EVariable>(arrSlot1), makeE<EVariable>(arrSlot2)));
+    auto setUnionExpr = stage_builder::makeFunction(
+        "setUnion", stage_builder::makeVariable(arrSlot1), stage_builder::makeVariable(arrSlot2));
     auto compiledExpr = compileExpression(*setUnionExpr);
 
     auto [arrTag1, arrVal1] = makeArray(BSON_ARRAY(1 << 2));
@@ -127,8 +148,9 @@ TEST_F(SBEBuiltinSetOpTest, ComputesSetIntersection) {
     value::OwnedValueAccessor slotAccessor1, slotAccessor2;
     auto arrSlot1 = bindAccessor(&slotAccessor1);
     auto arrSlot2 = bindAccessor(&slotAccessor2);
-    auto setIntersectionExpr = sbe::makeE<sbe::EFunction>(
-        "setIntersection", sbe::makeEs(makeE<EVariable>(arrSlot1), makeE<EVariable>(arrSlot2)));
+    auto setIntersectionExpr = stage_builder::makeFunction("setIntersection",
+                                                           stage_builder::makeVariable(arrSlot1),
+                                                           stage_builder::makeVariable(arrSlot2));
     auto compiledExpr = compileExpression(*setIntersectionExpr);
 
     auto [arrTag1, arrVal1] = makeArray(BSON_ARRAY(1 << 2 << 3));
@@ -153,8 +175,9 @@ TEST_F(SBEBuiltinSetOpTest, ReturnsNothingSetIntersection) {
     value::OwnedValueAccessor slotAccessor2;
     auto arrSlot1 = bindAccessor(&slotAccessor1);
     auto arrSlot2 = bindAccessor(&slotAccessor2);
-    auto setIntersectionExpr = sbe::makeE<sbe::EFunction>(
-        "setIntersection", sbe::makeEs(makeE<EVariable>(arrSlot1), makeE<EVariable>(arrSlot2)));
+    auto setIntersectionExpr = stage_builder::makeFunction("setIntersection",
+                                                           stage_builder::makeVariable(arrSlot1),
+                                                           stage_builder::makeVariable(arrSlot2));
     auto compiledExpr = compileExpression(*setIntersectionExpr);
 
     auto [arrTag1, arrVal1] = makeArray(BSON_ARRAY(1 << 2 << 3));
@@ -168,8 +191,9 @@ TEST_F(SBEBuiltinSetOpTest, ComputesSetDifference) {
     value::OwnedValueAccessor slotAccessor2;
     auto arrSlot1 = bindAccessor(&slotAccessor1);
     auto arrSlot2 = bindAccessor(&slotAccessor2);
-    auto setDiffExpr = sbe::makeE<sbe::EFunction>(
-        "setDifference", sbe::makeEs(makeE<EVariable>(arrSlot1), makeE<EVariable>(arrSlot2)));
+    auto setDiffExpr = stage_builder::makeFunction("setDifference",
+                                                   stage_builder::makeVariable(arrSlot1),
+                                                   stage_builder::makeVariable(arrSlot2));
     auto compiledExpr = compileExpression(*setDiffExpr);
 
     auto [arrTag1, arrVal1] = makeArray(BSON_ARRAY(1 << 2 << 3));
@@ -200,8 +224,9 @@ TEST_F(SBEBuiltinSetOpTest, ReturnsNothingSetDifference) {
     value::OwnedValueAccessor slotAccessor2;
     auto arrSlot1 = bindAccessor(&slotAccessor1);
     auto arrSlot2 = bindAccessor(&slotAccessor2);
-    auto setDiffExpr = sbe::makeE<sbe::EFunction>(
-        "setDifference", sbe::makeEs(makeE<EVariable>(arrSlot1), makeE<EVariable>(arrSlot2)));
+    auto setDiffExpr = stage_builder::makeFunction("setDifference",
+                                                   stage_builder::makeVariable(arrSlot1),
+                                                   stage_builder::makeVariable(arrSlot2));
     auto compiledExpr = compileExpression(*setDiffExpr);
 
     auto [arrTag1, arrVal1] = makeArray(BSON_ARRAY(1 << 2));
@@ -209,4 +234,113 @@ TEST_F(SBEBuiltinSetOpTest, ReturnsNothingSetDifference) {
     slotAccessor2.reset(value::TypeTags::NumberInt32, value::bitcastFrom<int32_t>(125));
     runAndAssertNothing(compiledExpr.get());
 }
+
+TEST_F(SBEBuiltinSetOpTest, ComputesSetEquals) {
+    value::OwnedValueAccessor slotAccessor1, slotAccessor2;
+    auto arrSlot1 = bindAccessor(&slotAccessor1);
+    auto arrSlot2 = bindAccessor(&slotAccessor2);
+    auto setEqualsExpr = stage_builder::makeFunction(
+        "setEquals", stage_builder::makeVariable(arrSlot1), stage_builder::makeVariable(arrSlot2));
+    auto compiledExpr = compileExpression(*setEqualsExpr);
+
+    auto [arrTag1, arrVal1] = makeArray(BSON_ARRAY(1 << 2 << 3));
+    slotAccessor1.reset(arrTag1, arrVal1);
+    auto [arrTag2, arrVal2] = makeArray(BSON_ARRAY(3 << 2 << 3 << 1));
+    slotAccessor2.reset(arrTag2, arrVal2);
+    runAndAssertBoolean(compiledExpr.get(), true);
+
+    std::tie(arrTag1, arrVal1) = makeArray(BSON_ARRAY(1 << 2 << 3));
+    slotAccessor1.reset(arrTag1, arrVal1);
+    std::tie(arrTag2, arrVal2) = value::makeNewArray();
+    slotAccessor2.reset(arrTag2, arrVal2);
+    runAndAssertBoolean(compiledExpr.get(), false);
+}
+
+TEST_F(SBEBuiltinSetOpTest, ReturnsNothingSetEquals) {
+    value::OwnedValueAccessor slotAccessor1, slotAccessor2;
+    auto arrSlot1 = bindAccessor(&slotAccessor1);
+    auto arrSlot2 = bindAccessor(&slotAccessor2);
+    auto setEqualsExpr = stage_builder::makeFunction(
+        "setEquals", stage_builder::makeVariable(arrSlot1), stage_builder::makeVariable(arrSlot2));
+    auto compiledExpr = compileExpression(*setEqualsExpr);
+
+    auto [arrTag1, arrVal1] = makeArray(BSON_ARRAY(1 << 2));
+    slotAccessor1.reset(arrTag1, arrVal1);
+    slotAccessor2.reset(value::TypeTags::NumberInt32, value::bitcastFrom<int32_t>(189));
+    runAndAssertNothing(compiledExpr.get());
+}
+
+TEST_F(SBEBuiltinSetOpTest, ComputesSetIsSubset) {
+    value::OwnedValueAccessor slotAccessor1;
+    value::OwnedValueAccessor slotAccessor2;
+    auto arrSlot1 = bindAccessor(&slotAccessor1);
+    auto arrSlot2 = bindAccessor(&slotAccessor2);
+    auto setIsSubsetExpr = stage_builder::makeFunction("setIsSubset",
+                                                       stage_builder::makeVariable(arrSlot1),
+                                                       stage_builder::makeVariable(arrSlot2));
+    auto compiledExpr = compileExpression(*setIsSubsetExpr);
+
+    // all elements are the same
+    auto [arrTag1, arrVal1] = makeArray(BSON_ARRAY(1 << 2 << 3));
+    slotAccessor1.reset(arrTag1, arrVal1);
+    auto [arrTag2, arrVal2] = makeArray(BSON_ARRAY(3 << 2 << 1));
+    slotAccessor2.reset(arrTag2, arrVal2);
+    runAndAssertBoolean(compiledExpr.get(), true);
+
+    // first array has the same elements multiple times
+    std::tie(arrTag1, arrVal1) = makeArray(BSON_ARRAY(1 << 2 << 3 << 1 << 2 << 3));
+    slotAccessor1.reset(arrTag1, arrVal1);
+    std::tie(arrTag2, arrVal2) = makeArray(BSON_ARRAY(1 << 2 << 3));
+    slotAccessor2.reset(arrTag2, arrVal2);
+    runAndAssertBoolean(compiledExpr.get(), true);
+
+    // first array is subset of the second array
+    std::tie(arrTag1, arrVal1) = makeArray(BSON_ARRAY(1 << 2 << 3));
+    slotAccessor1.reset(arrTag1, arrVal1);
+    std::tie(arrTag2, arrVal2) = makeArray(BSON_ARRAY(1 << 2 << 3 << 4 << 5));
+    slotAccessor2.reset(arrTag2, arrVal2);
+    runAndAssertBoolean(compiledExpr.get(), true);
+
+    // first array is not subset of the second array
+    std::tie(arrTag1, arrVal1) = makeArray(BSON_ARRAY(1 << 2 << 3));
+    slotAccessor1.reset(arrTag1, arrVal1);
+    std::tie(arrTag2, arrVal2) = makeArray(BSON_ARRAY(1 << 2 << 4 << 5));
+    slotAccessor2.reset(arrTag2, arrVal2);
+    runAndAssertBoolean(compiledExpr.get(), false);
+
+    // second array is empty
+    std::tie(arrTag1, arrVal1) = makeArray(BSON_ARRAY(1 << 2 << 3));
+    slotAccessor1.reset(arrTag1, arrVal1);
+    std::tie(arrTag2, arrVal2) = value::makeNewArray();
+    slotAccessor2.reset(arrTag2, arrVal2);
+    runAndAssertBoolean(compiledExpr.get(), false);
+
+    // first array is empty
+    std::tie(arrTag1, arrVal1) = value::makeNewArray();
+    slotAccessor1.reset(arrTag1, arrVal1);
+    std::tie(arrTag2, arrVal2) = makeArray(BSON_ARRAY(1 << 2 << 3));
+    slotAccessor2.reset(arrTag2, arrVal2);
+    runAndAssertBoolean(compiledExpr.get(), true);
+}
+
+TEST_F(SBEBuiltinSetOpTest, ReturnsNothingSetIsSubset) {
+    value::OwnedValueAccessor slotAccessor1;
+    value::OwnedValueAccessor slotAccessor2;
+    auto arrSlot1 = bindAccessor(&slotAccessor1);
+    auto arrSlot2 = bindAccessor(&slotAccessor2);
+    auto setIsSubsetExpr = stage_builder::makeFunction("setIsSubset",
+                                                       stage_builder::makeVariable(arrSlot1),
+                                                       stage_builder::makeVariable(arrSlot2));
+    auto compiledExpr = compileExpression(*setIsSubsetExpr);
+
+    auto [arrTag1, arrVal1] = makeArray(BSON_ARRAY(1 << 2));
+    slotAccessor1.reset(arrTag1, arrVal1);
+    slotAccessor2.reset(value::TypeTags::NumberInt32, value::bitcastFrom<int32_t>(125));
+    runAndAssertNothing(compiledExpr.get());
+
+    slotAccessor1.reset(value::TypeTags::NumberInt32, value::bitcastFrom<int32_t>(125));
+    auto [arrTag2, arrVal2] = makeArray(BSON_ARRAY(1 << 2));
+    slotAccessor2.reset(arrTag2, arrVal2);
+    runAndAssertNothing(compiledExpr.get());
+}
 }  // namespace mongo::sbe
