@@ -27,13 +27,20 @@
  *    it in the license file.
  */
 
+#include <cstdint>
+#include <memory>
+#include <ostream>
+#include <utility>
+
+#include "mongo/base/string_data.h"
 #include "mongo/db/exec/sbe/sbe_unittest.h"
-#include "mongo/db/exec/sbe/values/bson.h"
 #include "mongo/db/exec/sbe/values/value.h"
+#include "mongo/db/exec/sbe/values/value_printer.h"
+#include "mongo/db/exec/sbe/vm/label.h"
 #include "mongo/db/exec/sbe/vm/vm.h"
 #include "mongo/db/exec/sbe/vm/vm_printer.h"
+#include "mongo/unittest/framework.h"
 #include "mongo/unittest/golden_test.h"
-#include "mongo/unittest/unittest.h"
 
 namespace mongo::sbe {
 
@@ -120,7 +127,7 @@ TEST_F(SBECodeFragmentTest, AppendSimpleInstruction_Binary_LhsOnFrame) {
         code.appendConstVal(value::TypeTags::Nothing, 0);
 
         code.appendConstVal(rhsValue.first, rhsValue.second);
-        code.appendSub({0, frameId}, {});
+        code.appendSub({0, 0, frameId}, {});
         code.removeFrame(frameId);
 
         for (int i = 0; i < 2; i++) {
@@ -142,7 +149,7 @@ TEST_F(SBECodeFragmentTest, AppendSimpleInstruction_Binary_LhsOnFrame) {
 
         vm::CodeFragment code2;
         code2.appendConstVal(rhsValue.first, rhsValue.second);
-        code2.appendSub({0, frameId}, {});
+        code2.appendSub({0, 0, frameId}, {});
 
         code.append(std::move(code2));
         code.removeFrame(frameId);
@@ -168,7 +175,7 @@ TEST_F(SBECodeFragmentTest, AppendSimpleInstruction_Binary_LhsOnFrame) {
         rhs.appendConstVal(rhsValue.first, rhsValue.second);
 
         vm::CodeFragment instr;
-        instr.appendSub({0, frameId}, {});
+        instr.appendSub({0, 0, frameId}, {});
 
         vm::CodeFragment code;
         code.append(std::move(frame));
@@ -199,7 +206,7 @@ TEST_F(SBECodeFragmentTest, AppendSimpleInstruction_Binary_LhsOnFrame) {
 
         vm::CodeFragment instr;
         instr.append(std::move(rhs));
-        instr.appendSub({0, frameId}, {});
+        instr.appendSub({0, 0, frameId}, {});
 
         vm::CodeFragment code;
         code.append(std::move(frame));
@@ -231,7 +238,7 @@ TEST_F(SBECodeFragmentTest, AppendSimpleInstruction_Binary_RhsOnFrame) {
         code.appendConstVal(value::TypeTags::Nothing, 0);
 
         code.appendConstVal(lhsValue.first, lhsValue.second);
-        code.appendSub({}, {0, frameId});
+        code.appendSub({}, {0, 0, frameId});
         code.removeFrame(frameId);
 
         for (int i = 0; i < 2; i++) {
@@ -253,7 +260,7 @@ TEST_F(SBECodeFragmentTest, AppendSimpleInstruction_Binary_RhsOnFrame) {
 
         vm::CodeFragment code2;
         code2.appendConstVal(lhsValue.first, lhsValue.second);
-        code2.appendSub({}, {0, frameId});
+        code2.appendSub({}, {0, 0, frameId});
 
         code.append(std::move(code2));
         code.removeFrame(frameId);
@@ -279,7 +286,7 @@ TEST_F(SBECodeFragmentTest, AppendSimpleInstruction_Binary_RhsOnFrame) {
         lhs.appendConstVal(lhsValue.first, lhsValue.second);
 
         vm::CodeFragment instr;
-        instr.appendSub({}, {0, frameId});
+        instr.appendSub({}, {0, 0, frameId});
 
         vm::CodeFragment code;
         code.append(std::move(frame));
@@ -310,7 +317,7 @@ TEST_F(SBECodeFragmentTest, AppendSimpleInstruction_Binary_RhsOnFrame) {
 
         vm::CodeFragment instr;
         instr.append(std::move(lhs));
-        instr.appendSub({}, {0, frameId});
+        instr.appendSub({}, {0, 0, frameId});
 
         vm::CodeFragment code;
         code.append(std::move(frame));
@@ -343,7 +350,7 @@ TEST_F(SBECodeFragmentTest, AppendSimpleInstruction_Binary_BothOnFrame) {
         code.appendConstVal(rhsValue.first, rhsValue.second);
         code.appendConstVal(value::TypeTags::Nothing, 0);
 
-        code.appendSub({0, frameId}, {1, frameId});
+        code.appendSub({0, 0, frameId}, {1, 0, frameId});
         code.removeFrame(frameId);
 
         for (int i = 0; i < 3; i++) {
@@ -365,7 +372,7 @@ TEST_F(SBECodeFragmentTest, AppendSimpleInstruction_Binary_BothOnFrame) {
         code.appendConstVal(value::TypeTags::Nothing, 0);
 
         vm::CodeFragment code2;
-        code2.appendSub({0, frameId}, {1, frameId});
+        code2.appendSub({0, 0, frameId}, {1, 0, frameId});
 
         code.append(std::move(code2));
         code.removeFrame(frameId);
@@ -389,7 +396,7 @@ TEST_F(SBECodeFragmentTest, AppendSimpleInstruction_Binary_BothOnFrame) {
         frame.appendConstVal(value::TypeTags::Nothing, 0);
 
         vm::CodeFragment instr;
-        instr.appendSub({0, frameId}, {1, frameId});
+        instr.appendSub({0, 0, frameId}, {1, 0, frameId});
 
         vm::CodeFragment code;
         code.append(std::move(frame));
@@ -665,6 +672,42 @@ TEST_F(SBECodeFragmentTest, AppendLocalVal2) {
     }
 }
 
+TEST_F(SBECodeFragmentTest, AppendMakeOwn) {
+    auto lhsValue = value::makeBigString("one not too short string");
+    value::ValueGuard lhsGuard(lhsValue);
+    auto rhsValue = value::makeBigString("another string");
+    value::ValueGuard rhsGuard(rhsValue);
+    FrameId frameId = 10;
+
+    {
+        printVariation("append code 1");
+
+        vm::CodeFragment code;
+        code.declareFrame(frameId);
+        // Create frame with 3 variables to make variable resolution less trivial.
+        code.appendConstVal(lhsValue.first, lhsValue.second);
+        code.appendConstVal(rhsValue.first, rhsValue.second);
+        code.appendConstVal(value::TypeTags::Nothing, 0);
+
+        vm::CodeFragment code2;
+        code2.appendLocalVal(frameId, 0, false);
+        code2.appendMakeOwn({});
+        code2.appendLocalVal(frameId, 1, false);
+        code2.appendMakeOwn({});
+        code2.appendFunction(vm::Builtin::concat, 2);
+
+        code.append(std::move(code2));
+        code.removeFrame(frameId);
+
+        for (int i = 0; i < 3; i++) {
+            code.appendSwap();
+            code.appendPop();
+        }
+
+        runTest(code);
+    }
+}
+
 TEST_F(SBECodeFragmentTest, DeclareFrameNotEmptyStack) {
     auto value = makeInt32(10);
     FrameId frameId = 10;
