@@ -27,13 +27,34 @@
  *    it in the license file.
  */
 
+#include <cstdint>
+#include <cstring>
+#include <limits>
+#include <string_view>
+
+#include <absl/container/inlined_vector.h>
+#include <boost/cstdint.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/numeric/conversion/converter_policies.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/bson/bsontypes_util.h"
 #include "mongo/db/exec/sbe/sbe_unittest.h"
 #include "mongo/db/exec/sbe/values/bson.h"
 #include "mongo/db/exec/sbe/values/value.h"
 #include "mongo/db/exec/sbe/vm/vm.h"
 #include "mongo/db/exec/sbe/vm/vm_printer.h"
+#include "mongo/db/query/datetime/date_time_support.h"
+#include "mongo/platform/decimal128.h"
+#include "mongo/unittest/assert.h"
 #include "mongo/unittest/golden_test.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/util/represent_as.h"
 
 namespace mongo::sbe {
 
@@ -247,6 +268,27 @@ TEST(SBEValues, HashCompound) {
         set2->push_back(value::TypeTags::NumberDouble, value::bitcastFrom<double>(-5.0));
 
 
+        ASSERT_EQUALS(value::hashValue(tag1, val1), value::hashValue(tag2, val2));
+
+        value::releaseValue(tag1, val1);
+        value::releaseValue(tag2, val2);
+    }
+
+    {
+        auto [tag1, val1] = value::makeNewArrayMultiSet();
+        auto set1 = value::getArrayMultiSetView(val1);
+        set1->push_back(value::TypeTags::NumberInt32, value::bitcastFrom<int32_t>(-5));
+        set1->push_back(value::TypeTags::NumberInt32, value::bitcastFrom<int32_t>(-5));
+        set1->push_back(value::TypeTags::NumberInt32, value::bitcastFrom<int32_t>(-6));
+        set1->push_back(value::TypeTags::NumberInt32, value::bitcastFrom<int32_t>(-7));
+
+        auto [tag2, val2] = value::makeNewArrayMultiSet();
+        auto set2 = value::getArrayMultiSetView(val2);
+        set2->push_back(value::TypeTags::NumberDouble, value::bitcastFrom<double>(-7.0));
+        set2->push_back(value::TypeTags::NumberDouble, value::bitcastFrom<double>(-6.0));
+        set2->push_back(value::TypeTags::NumberDouble, value::bitcastFrom<double>(-5.0));
+        set2->push_back(value::TypeTags::NumberDouble, value::bitcastFrom<double>(-5.0));
+
         ASSERT_EQUALS(value::hashValue(tag1, val1), value::hashValue(tag2, val2));
 
         value::releaseValue(tag1, val1);
@@ -436,7 +478,7 @@ TEST(SBEVM, ConvertBinDataToBsonObj) {
     array.push_back(binDataTag, binDataVal);
 
     BSONArrayBuilder builder;
-    bson::convertToBsonObj(builder, &array);
+    bson::convertToBsonArr(builder, &array);
     auto convertedBinData = builder.done();
 
     ASSERT_EQ(originalBinData.woCompare(convertedBinData), 0);
