@@ -31,15 +31,40 @@
  * This file contains tests for sbe::HashAggStage.
  */
 
-#include "mongo/platform/basic.h"
+#include <cstddef>
+#include <cstdint>
+#include <limits>
+#include <memory>
+#include <string>
+#include <utility>
+#include <vector>
 
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/db/exec/sbe/expressions/expression.h"
 #include "mongo/db/exec/sbe/sbe_plan_stage_test.h"
+#include "mongo/db/exec/sbe/stages/co_scan.h"
 #include "mongo/db/exec/sbe/stages/hash_agg.h"
+#include "mongo/db/exec/sbe/stages/limit_skip.h"
 #include "mongo/db/exec/sbe/stages/scan.h"
 #include "mongo/db/exec/sbe/stages/sort.h"
+#include "mongo/db/exec/sbe/stages/stages.h"
 #include "mongo/db/exec/sbe/stages/union.h"
+#include "mongo/db/exec/sbe/values/slot.h"
+#include "mongo/db/exec/sbe/values/value.h"
+#include "mongo/db/exec/trial_run_tracker.h"
+#include "mongo/db/query/sbe_stage_builder_helpers.h"
+#include "mongo/db/query/stage_types.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
 #include "mongo/util/assert_util.h"
+#include "mongo/util/scopeguard.h"
+#include "mongo/util/uuid.h"
 
 namespace mongo::sbe {
 
@@ -47,28 +72,29 @@ using TrialRunTrackerTest = PlanStageTestFixture;
 
 TEST_F(TrialRunTrackerTest, TrackerAttachesToStreamingStage) {
     auto collUuid = UUID::parse("00000000-0000-0000-0000-000000000000").getValue();
-    auto scanStage = makeS<ScanStage>(collUuid,
-                                      DatabaseName(),
-                                      generateSlotId(),
-                                      generateSlotId(),
-                                      generateSlotId(),
-                                      generateSlotId(),
-                                      generateSlotId(),
-                                      generateSlotId(),
-                                      boost::none,
-                                      std::vector<std::string>{"field"},
-                                      makeSV(generateSlotId()),
-                                      generateSlotId(),
-                                      true,
-                                      nullptr,
-                                      kEmptyPlanNodeId,
-                                      ScanCallbacks());
+    auto scanStage = makeS<sbe::ScanStage>(collUuid,
+                                           generateSlotId() /* recordSlot */,
+                                           generateSlotId() /* recordIdSlot */,
+                                           generateSlotId() /* snapshotIdSlot */,
+                                           generateSlotId() /* indexIdSlot */,
+                                           generateSlotId() /* indexKeySlot */,
+                                           generateSlotId() /* indexKeyPatternSlot */,
+                                           boost::none /* oplogTsSlot */,
+                                           std::vector<std::string>{"field"} /* scanFieldNames */,
+                                           makeSV(generateSlotId()) /* scanFieldSlots */,
+                                           generateSlotId() /* seekRecordIdSlot */,
+                                           generateSlotId() /* minRecordIdSlot */,
+                                           generateSlotId() /* maxRecordIdSlot */,
+                                           true /* forward */,
+                                           nullptr /* yieldPolicy */,
+                                           kEmptyPlanNodeId /* nodeId */,
+                                           ScanCallbacks());
 
-    auto tracker = std::make_unique<TrialRunTracker>(size_t{0}, size_t{0});
+    auto tracker = std::make_unique<TrialRunTracker>(size_t{0}, size_t{0}, size_t{0});
     ON_BLOCK_EXIT([&]() { scanStage->detachFromTrialRunTracker(); });
 
     auto attachResult = scanStage->attachToTrialRunTracker(tracker.get());
-    ASSERT_EQ(attachResult, PlanStage::TrialRunTrackerAttachResultFlags::AttachedToStreamingStage);
+    ASSERT_EQ(attachResult, PlanStage::TrialRunTrackingType::TrackReads);
 }
 
 TEST_F(TrialRunTrackerTest, TrackerAttachesToBlockingStage) {
@@ -83,33 +109,35 @@ TEST_F(TrialRunTrackerTest, TrackerAttachesToBlockingStage) {
                          nullptr /*limit*/,
                          204857600,
                          false,
+                         nullptr /* yieldPolicy */,
                          kEmptyPlanNodeId);
 
-    auto tracker = std::make_unique<TrialRunTracker>(size_t{0}, size_t{0});
+    auto tracker = std::make_unique<TrialRunTracker>(size_t{0}, size_t{0}, size_t{0});
     ON_BLOCK_EXIT([&]() { sortStage->detachFromTrialRunTracker(); });
 
     auto attachResult = sortStage->attachToTrialRunTracker(tracker.get());
-    ASSERT_EQ(attachResult, PlanStage::TrialRunTrackerAttachResultFlags::AttachedToBlockingStage);
+    ASSERT_EQ(attachResult, PlanStage::TrialRunTrackingType::TrackResults);
 }
 
 TEST_F(TrialRunTrackerTest, TrackerAttachesToBothBlockingAndStreamingStages) {
     auto collUuid = UUID::parse("00000000-0000-0000-0000-000000000000").getValue();
-    auto scanStage = makeS<ScanStage>(collUuid,
-                                      DatabaseName(),
-                                      generateSlotId(),
-                                      generateSlotId(),
-                                      generateSlotId(),
-                                      generateSlotId(),
-                                      generateSlotId(),
-                                      generateSlotId(),
-                                      boost::none,
-                                      std::vector<std::string>{"field"},
-                                      makeSV(generateSlotId()),
-                                      generateSlotId(),
-                                      true,
-                                      nullptr,
-                                      kEmptyPlanNodeId,
-                                      ScanCallbacks());
+    auto scanStage = makeS<sbe::ScanStage>(collUuid,
+                                           generateSlotId() /* recordSlot */,
+                                           generateSlotId() /* recordIdSlot */,
+                                           generateSlotId() /* snapshotIdSlot */,
+                                           generateSlotId() /* indexIdSlot */,
+                                           generateSlotId() /* indexKeySlot */,
+                                           generateSlotId() /* indexKeyPatternSlot */,
+                                           boost::none /* oplogTsSlot */,
+                                           std::vector<std::string>{"field"} /* scanFieldNames */,
+                                           makeSV(generateSlotId()) /* scanFieldSlots */,
+                                           generateSlotId() /* seekRecordIdSlot */,
+                                           generateSlotId() /* minRecordIdSlot */,
+                                           generateSlotId() /* maxRecordIdSlot */,
+                                           true /* forward */,
+                                           nullptr /* yieldPolicy */,
+                                           kEmptyPlanNodeId /* nodeId */,
+                                           ScanCallbacks());
 
     auto rootSortStage = makeS<SortStage>(std::move(scanStage),
                                           makeSV(),
@@ -118,15 +146,16 @@ TEST_F(TrialRunTrackerTest, TrackerAttachesToBothBlockingAndStreamingStages) {
                                           nullptr /*limit*/,
                                           204857600,
                                           false,
+                                          nullptr /* yieldPolicy */,
                                           kEmptyPlanNodeId);
 
-    auto tracker = std::make_unique<TrialRunTracker>(size_t{0}, size_t{0});
+    auto tracker = std::make_unique<TrialRunTracker>(size_t{0}, size_t{0}, size_t{0});
     ON_BLOCK_EXIT([&]() { rootSortStage->detachFromTrialRunTracker(); });
 
     auto attachResult = rootSortStage->attachToTrialRunTracker(tracker.get());
     ASSERT_EQ(attachResult,
-              PlanStage::TrialRunTrackerAttachResultFlags::AttachedToStreamingStage |
-                  PlanStage::TrialRunTrackerAttachResultFlags::AttachedToBlockingStage);
+              PlanStage::TrialRunTrackingType::TrackReads |
+                  PlanStage::TrialRunTrackingType::TrackResults);
 }
 
 TEST_F(TrialRunTrackerTest, TrialEndsDuringOpenPhaseOfBlockingStage) {
@@ -156,14 +185,15 @@ TEST_F(TrialRunTrackerTest, TrialEndsDuringOpenPhaseOfBlockingStage) {
         boost::none,
         false /* allowDiskUse */,
         makeSlotExprPairVec(), /* mergingExprs */
+        nullptr /* yieldPolicy */,
         kEmptyPlanNodeId);
 
-    auto tracker = std::make_unique<TrialRunTracker>(numResultsLimit, size_t{0});
+    auto tracker = std::make_unique<TrialRunTracker>(numResultsLimit, size_t{0}, size_t{0});
     auto attachResult = hashAggStage->attachToTrialRunTracker(tracker.get());
 
     // Note: A scan is a streaming stage, but the "virtual scan" used here does not attach to the
     // tracker.
-    ASSERT_EQ(attachResult, PlanStage::TrialRunTrackerAttachResultFlags::AttachedToBlockingStage);
+    ASSERT_EQ(attachResult, PlanStage::TrialRunTrackingType::TrackResults);
 
     // The 'prepareTree()' function opens the HashAggStage, causing it to read documents from its
     // child. Because the child provides more documents than the 'numResults' limit, we expect the
@@ -201,6 +231,7 @@ TEST_F(TrialRunTrackerTest, OnlyDeepestNestedBlockingStageHasTrialRunTracker) {
                              nullptr /*limit*/,
                              204857600,
                              false,
+                             nullptr /* yieldPolicy */,
                              kEmptyPlanNodeId);
 
         return std::make_pair(scanSlot, std::move(sortStage));
@@ -228,6 +259,7 @@ TEST_F(TrialRunTrackerTest, OnlyDeepestNestedBlockingStageHasTrialRunTracker) {
         boost::none,
         false /* allowDiskUse */,
         makeSlotExprPairVec(), /* mergingExprs */
+        nullptr /* yieldPolicy */,
         kEmptyPlanNodeId);
 
     hashAggStage->prepare(*ctx);
@@ -235,14 +267,13 @@ TEST_F(TrialRunTrackerTest, OnlyDeepestNestedBlockingStageHasTrialRunTracker) {
 
     {
         // We expect the TrialRunTracker to attach to the SortStage but not the root HashAggStage.
-        auto tracker = std::make_unique<TrialRunTracker>(size_t{9}, size_t{0});
+        auto tracker = std::make_unique<TrialRunTracker>(size_t{9}, size_t{0}, size_t{0});
         ON_BLOCK_EXIT([&]() { hashAggStage->detachFromTrialRunTracker(); });
         auto attachResult = hashAggStage->attachToTrialRunTracker(tracker.get());
 
         // Note: A scan is a streaming stage, but the "virtual scan" used here does not attach to
         // the tracker.
-        ASSERT_EQ(attachResult,
-                  PlanStage::TrialRunTrackerAttachResultFlags::AttachedToBlockingStage);
+        ASSERT_EQ(attachResult, PlanStage::TrialRunTrackingType::TrackResults);
 
         // In this scenario, the HashAggStage will see 10+ documents (the 10 documents from the
         // "upperScan" plus the documents from the SortStage), which exceeds the 'numResults'
@@ -255,12 +286,11 @@ TEST_F(TrialRunTrackerTest, OnlyDeepestNestedBlockingStageHasTrialRunTracker) {
 
     {
         // We expect the TrialRunTracker to attach to the SortStage but not the root HashAggStage.
-        auto tracker = std::make_unique<TrialRunTracker>(size_t{2}, size_t{0});
+        auto tracker = std::make_unique<TrialRunTracker>(size_t{2}, size_t{0}, size_t{0});
         ON_BLOCK_EXIT([&]() { hashAggStage->detachFromTrialRunTracker(); });
         auto attachResult = hashAggStage->attachToTrialRunTracker(tracker.get());
 
-        ASSERT_EQ(attachResult,
-                  PlanStage::TrialRunTrackerAttachResultFlags::AttachedToBlockingStage);
+        ASSERT_EQ(attachResult, PlanStage::TrialRunTrackingType::TrackResults);
 
         // In this scenario, the SortStage will see more documents than the 2 permitted by the
         // 'numResults' requirement of the TrialRunTracker. The 'open()' call will _fail_, because
@@ -298,6 +328,7 @@ TEST_F(TrialRunTrackerTest, SiblingBlockingStagesBothGetTrialRunTracker) {
             boost::none,
             false /* allowDiskUse */,
             makeSlotExprPairVec(), /* mergingExprs */
+            nullptr /* yieldPolicy */,
             kEmptyPlanNodeId);
 
         return std::make_pair(countsSlot, std::move(hashAggStage));
@@ -324,15 +355,16 @@ TEST_F(TrialRunTrackerTest, SiblingBlockingStagesBothGetTrialRunTracker) {
                          nullptr /*limit*/,
                          204857600,
                          false,
+                         nullptr /* yieldPolicy */,
                          kEmptyPlanNodeId);
 
     // We expect the TrialRunTracker to attach to _both_ HashAgg stages but not to the SortStage.
-    auto tracker = std::make_unique<TrialRunTracker>(size_t{9}, size_t{0});
+    auto tracker = std::make_unique<TrialRunTracker>(size_t{9}, size_t{0}, size_t{0});
     auto attachResult = sortStage->attachToTrialRunTracker(tracker.get());
 
     // Note: A scan is a streaming stage, but the "virtual scan" used here does not attach to the
     // tracker.
-    ASSERT_EQ(attachResult, PlanStage::TrialRunTrackerAttachResultFlags::AttachedToBlockingStage);
+    ASSERT_EQ(attachResult, PlanStage::TrialRunTrackingType::TrackResults);
 
     // The 'prepareTree()' function opens the SortStage, causing it to read documents from its
     // child. If only one of the HashAgg stages were attached to the TrialRunTracker, it would not
@@ -346,47 +378,49 @@ TEST_F(TrialRunTrackerTest, SiblingBlockingStagesBothGetTrialRunTracker) {
 
 TEST_F(TrialRunTrackerTest, TrialRunTrackingCanBeDisabled) {
     auto scanStage =
-        makeS<ScanStage>(UUID::parse("00000000-0000-0000-0000-000000000000").getValue(),
-                         DatabaseName(),
-                         generateSlotId(),
-                         generateSlotId(),
-                         generateSlotId(),
-                         generateSlotId(),
-                         generateSlotId(),
-                         generateSlotId(),
-                         boost::none,
-                         std::vector<std::string>{"field"},
-                         makeSV(generateSlotId()),
-                         generateSlotId(),
-                         true,
-                         nullptr,
-                         kEmptyPlanNodeId,
-                         ScanCallbacks());
+        makeS<sbe::ScanStage>(UUID::parse("00000000-0000-0000-0000-000000000000").getValue(),
+                              generateSlotId() /* recordSlot */,
+                              generateSlotId() /* recordIdSlot */,
+                              generateSlotId() /* snapshotIdSlot */,
+                              generateSlotId() /* indexIdSlot */,
+                              generateSlotId() /* indexKeySlot */,
+                              generateSlotId() /* indexKeyPatternSlot */,
+                              boost::none /* oplogTsSlot */,
+                              std::vector<std::string>{"field"} /* scanFieldNames */,
+                              makeSV(generateSlotId()) /* scanFieldSlots */,
+                              generateSlotId() /* seekRecordIdSlot */,
+                              generateSlotId() /* minRecordIdSlot */,
+                              generateSlotId() /* maxRecordIdSlot */,
+                              true /* forward */,
+                              nullptr /* yieldPolicy */,
+                              kEmptyPlanNodeId /* nodeId */,
+                              ScanCallbacks());
 
     scanStage->disableTrialRunTracking();
-    auto tracker = std::make_unique<TrialRunTracker>(size_t{0}, size_t{0});
+    auto tracker = std::make_unique<TrialRunTracker>(size_t{0}, size_t{0}, size_t{0});
     auto attachResult = scanStage->attachToTrialRunTracker(tracker.get());
-    ASSERT_EQ(attachResult, PlanStage::TrialRunTrackerAttachResultFlags::NoAttachment);
+    ASSERT_EQ(attachResult, PlanStage::TrialRunTrackingType::NoTracking);
 }
 
 TEST_F(TrialRunTrackerTest, DisablingTrackingForChildDoesNotInhibitTrackingForParent) {
     auto scanStage =
-        makeS<ScanStage>(UUID::parse("00000000-0000-0000-0000-000000000000").getValue(),
-                         DatabaseName(),
-                         generateSlotId(),
-                         generateSlotId(),
-                         generateSlotId(),
-                         generateSlotId(),
-                         generateSlotId(),
-                         generateSlotId(),
-                         boost::none,
-                         std::vector<std::string>{"field"},
-                         makeSV(generateSlotId()),
-                         generateSlotId(),
-                         true,
-                         nullptr,
-                         kEmptyPlanNodeId,
-                         ScanCallbacks());
+        makeS<sbe::ScanStage>(UUID::parse("00000000-0000-0000-0000-000000000000").getValue(),
+                              generateSlotId() /* recordSlot */,
+                              generateSlotId() /* recordIdSlot */,
+                              generateSlotId() /* snapshotIdSlot */,
+                              generateSlotId() /* indexIdSlot */,
+                              generateSlotId() /* indexKeySlot */,
+                              generateSlotId() /* indexKeyPatternSlot */,
+                              boost::none /* oplogTsSlot */,
+                              std::vector<std::string>{"field"} /* scanFieldNames */,
+                              makeSV(generateSlotId()) /* scanFieldSlots */,
+                              generateSlotId() /* seekRecordIdSlot */,
+                              generateSlotId() /* minRecordIdSlot */,
+                              generateSlotId() /* maxRecordIdSlot */,
+                              true /* forward */,
+                              nullptr /* yieldPolicy */,
+                              kEmptyPlanNodeId /* nodeId */,
+                              ScanCallbacks());
 
     // Disable tracking for 'scanStage'. We should still attach the tracker for 'rootSortStage'.
     scanStage->disableTrialRunTracking();
@@ -398,14 +432,15 @@ TEST_F(TrialRunTrackerTest, DisablingTrackingForChildDoesNotInhibitTrackingForPa
                                           nullptr /*limit*/,
                                           204857600,
                                           false,
+                                          nullptr /* yieldPolicy */,
                                           kEmptyPlanNodeId);
 
 
-    auto tracker = std::make_unique<TrialRunTracker>(size_t{0}, size_t{0});
+    auto tracker = std::make_unique<TrialRunTracker>(size_t{0}, size_t{0}, size_t{0});
     ON_BLOCK_EXIT([&]() { rootSortStage->detachFromTrialRunTracker(); });
 
     auto attachResult = rootSortStage->attachToTrialRunTracker(tracker.get());
-    ASSERT_EQ(attachResult, PlanStage::TrialRunTrackerAttachResultFlags::AttachedToBlockingStage);
+    ASSERT_EQ(attachResult, PlanStage::TrialRunTrackingType::TrackResults);
 }
 
 TEST_F(TrialRunTrackerTest, DisablingTrackingForAChildStagePreventsEarlyExit) {
@@ -433,6 +468,7 @@ TEST_F(TrialRunTrackerTest, DisablingTrackingForAChildStagePreventsEarlyExit) {
             boost::none,
             false /* allowDiskUse */,
             makeSlotExprPairVec(), /* mergingExprs */
+            nullptr /* yieldPolicy */,
             kEmptyPlanNodeId);
 
         return std::make_pair(countsSlot, std::move(hashAggStage));
@@ -460,15 +496,16 @@ TEST_F(TrialRunTrackerTest, DisablingTrackingForAChildStagePreventsEarlyExit) {
                          nullptr /*limit*/,
                          204857600,
                          false,
+                         nullptr /* yieldPolicy */,
                          kEmptyPlanNodeId);
 
     // We expect the TrialRunTracker to attach to _only_ the left child.
-    auto tracker = std::make_unique<TrialRunTracker>(size_t{9}, size_t{0});
+    auto tracker = std::make_unique<TrialRunTracker>(size_t{9}, size_t{0}, size_t{0});
     auto attachResult = sortStage->attachToTrialRunTracker(tracker.get());
 
     // Note: A scan is a streaming stage, but the "virtual scan" used here does not attach to the
     // tracker.
-    ASSERT_EQ(attachResult, PlanStage::TrialRunTrackerAttachResultFlags::AttachedToBlockingStage);
+    ASSERT_EQ(attachResult, PlanStage::TrialRunTrackingType::TrackResults);
 
     // The 'prepareTree()' function opens the SortStage, causing it to read documents from its
     // child. Because only one of the HashAgg stages is attached to the TrialRunTracker, the
@@ -476,4 +513,38 @@ TEST_F(TrialRunTrackerTest, DisablingTrackingForAChildStagePreventsEarlyExit) {
     // 'prepareTree()' will not end the trial.
     prepareTree(ctx.get(), sortStage.get(), resultSlot);
 }
+
+TEST_F(TrialRunTrackerTest, TrackerAttachesToPlanningRootStageAndTracksTheDocumentsReturnedByIt) {
+    auto ctx = makeCompileCtx();
+
+    auto [inputTag, inputVal] = stage_builder::makeValue(
+        BSON_ARRAY(BSON_ARRAY(1 << 2) << BSON_ARRAY(3 << 4) << BSON_ARRAY(5 << 6)));
+    auto [scanSlot, scanStage] = generateVirtualScan(inputTag, inputVal, PlanNodeId{1});
+    auto unwindSlot = generateSlotId();
+    auto unwindStage = makeS<UnwindStage>(
+        std::move(scanStage), scanSlot, generateSlotId(), unwindSlot, true, PlanNodeId{2});
+    auto sortStage =
+        makeS<SortStage>(std::move(unwindStage),
+                         makeSV(unwindSlot),
+                         std::vector<value::SortDirection>{value::SortDirection::Ascending},
+                         makeSV(),
+                         nullptr /*limit*/,
+                         1024 * 1024,
+                         false,
+                         nullptr /* yieldPolicy */,
+                         PlanNodeId{3});
+
+    auto tracker = std::make_unique<TrialRunTracker>(size_t{6}, size_t{0}, size_t{3});
+    auto attachResult = sortStage->attachToTrialRunTracker(tracker.get(), PlanNodeId{1});
+    ASSERT_EQ(attachResult,
+              PlanStage::TrialRunTrackingType::TrackPlanningResults |
+                  PlanStage::TrialRunTrackingType::TrackResults);
+
+    // The 'prepareTree()' function opens the SortStage, causing it to read documents from its
+    // child. Only 3 documents should be returned by scan stage, which is marked as planning root,
+    // but sort should have 6 results as input because of unwind.
+    prepareTree(ctx.get(), sortStage.get(), unwindSlot);
+    ASSERT_EQ(tracker->getMetric<TrialRunTracker::kNumResults>(), 6);
+    ASSERT_EQ(tracker->getMetric<TrialRunTracker::kNumPlanningResults>(), 3);
+}
 }  // namespace mongo::sbe
