@@ -28,10 +28,28 @@
  */
 #include "mongo/db/exec/sbe/stages/column_scan.h"
 
+#include <absl/container/flat_hash_map.h>
+#include <absl/meta/type_traits.h>
+#include <list>
+
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/db/catalog/collection.h"
+#include "mongo/db/catalog/index_catalog.h"
 #include "mongo/db/exec/sbe/expressions/compile_ctx.h"
 #include "mongo/db/exec/sbe/expressions/expression.h"
 #include "mongo/db/exec/sbe/size_estimator.h"
 #include "mongo/db/index/columns_access_method.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/query/query_knobs_gen.h"
+#include "mongo/db/storage/record_data.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/str.h"
 
 namespace mongo {
 namespace sbe {
@@ -41,7 +59,6 @@ ColumnScanStage::RowstoreScanModeTracker::RowstoreScanModeTracker()
       _batchSize(_minBatchSize) {}
 
 ColumnScanStage::ColumnScanStage(UUID collectionUuid,
-                                 DatabaseName dbName,
                                  StringData columnIndexName,
                                  std::vector<std::string> paths,
                                  bool densePathIncludedInScan,
@@ -54,9 +71,12 @@ ColumnScanStage::ColumnScanStage(UUID collectionUuid,
                                  PlanYieldPolicy* yieldPolicy,
                                  PlanNodeId nodeId,
                                  bool participateInTrialRunTracking)
-    : PlanStage("columnscan"_sd, yieldPolicy, nodeId, participateInTrialRunTracking),
+    : PlanStage("columnscan"_sd,
+                yieldPolicy,
+                nodeId,
+                participateInTrialRunTracking,
+                TrialRunTrackingType::TrackReads),
       _collUuid(collectionUuid),
-      _dbName(dbName),
       _columnIndexName(columnIndexName),
       _paths(std::move(paths)),
       _includeInOutput(std::move(includeInOutput)),
@@ -77,7 +97,6 @@ std::unique_ptr<PlanStage> ColumnScanStage::clone() const {
         filteredPaths.emplace_back(fp.pathIndex, fp.filterExpr->clone(), fp.inputSlotId);
     }
     return std::make_unique<ColumnScanStage>(_collUuid,
-                                             _dbName,
                                              _columnIndexName,
                                              _paths,
                                              _densePathIncludedInScan,
@@ -89,7 +108,7 @@ std::unique_ptr<PlanStage> ColumnScanStage::clone() const {
                                              std::move(filteredPaths),
                                              _yieldPolicy,
                                              _commonStats.nodeId,
-                                             _participateInTrialRunTracking);
+                                             participateInTrialRunTracking());
 }
 
 void ColumnScanStage::prepare(CompileCtx& ctx) {
@@ -118,15 +137,16 @@ void ColumnScanStage::prepare(CompileCtx& ctx) {
     }
 
     tassert(6610200, "'_coll' should not be initialized prior to 'acquireCollection()'", !_coll);
-    std::tie(_coll, _collName, _catalogEpoch) = acquireCollection(_opCtx, _dbName, _collUuid);
+    _coll.acquireCollection(_opCtx, _collUuid);
 
-    auto indexCatalog = _coll->getIndexCatalog();
+    auto indexCatalog = _coll.getPtr()->getIndexCatalog();
     auto indexDesc = indexCatalog->findIndexByName(_opCtx, _columnIndexName);
     tassert(6610201,
             str::stream() << "could not find index named '" << _columnIndexName
-                          << "' in collection '" << _collName << "'",
+                          << "' in collection '" << _coll.getCollName()->toStringForErrorMsg()
+                          << "'",
             indexDesc);
-    _weakIndexCatalogEntry = indexCatalog->getEntryShared(indexDesc);
+    _columnIndexIdent = indexDesc->getEntry()->getIdent();
 }
 
 value::SlotAccessor* ColumnScanStage::getAccessor(CompileCtx& ctx, value::SlotId slot) {
@@ -180,17 +200,16 @@ void ColumnScanStage::doRestoreState(bool relinquishCursor) {
     invariant(!_coll);
 
     // If this stage has not been prepared, then yield recovery is a no-op.
-    if (!_collName) {
+    if (!_coll.getCollName()) {
         return;
     }
 
-    tassert(6610202, "Catalog epoch should be initialized", _catalogEpoch);
-    _coll = restoreCollection(_opCtx, *_collName, _dbName, _collUuid, *_catalogEpoch);
-
-    auto indexCatalogEntry = _weakIndexCatalogEntry.lock();
+    _coll.restoreCollection(_opCtx, _collUuid);
+    auto desc = _coll.getPtr()->getIndexCatalog()->findIndexByIdent(_opCtx, _columnIndexIdent);
     uassert(ErrorCodes::QueryPlanKilled,
             str::stream() << "query plan killed :: index '" << _columnIndexName << "' dropped",
-            indexCatalogEntry && !indexCatalogEntry->isDropped());
+            desc);
+
 
     if (_rowStoreCursor) {
         if (relinquishCursor) {
@@ -240,16 +259,6 @@ void ColumnScanStage::doAttachToOperationContext(OperationContext* opCtx) {
     }
 }
 
-void ColumnScanStage::doDetachFromTrialRunTracker() {
-    _tracker = nullptr;
-}
-
-PlanStage::TrialRunTrackerAttachResultMask ColumnScanStage::doAttachToTrialRunTracker(
-    TrialRunTracker* tracker, TrialRunTrackerAttachResultMask childrenAttachResult) {
-    _tracker = tracker;
-    return childrenAttachResult | TrialRunTrackerAttachResultFlags::AttachedToStreamingStage;
-}
-
 void ColumnScanStage::open(bool reOpen) {
     auto optTimer(getOptTimer(_opCtx));
 
@@ -267,22 +276,21 @@ void ColumnScanStage::open(bool reOpen) {
             // make some validity checks (the collection has not been dropped, renamed, etc.).
             tassert(
                 6610207, "ColumnScanStage is not open but have _rowStoreCursor", !_rowStoreCursor);
-            tassert(6610208, "Collection name should be initialized", _collName);
-            tassert(6610209, "Catalog epoch should be initialized", _catalogEpoch);
-            _coll = restoreCollection(_opCtx, *_collName, _dbName, _collUuid, *_catalogEpoch);
+            _coll.restoreCollection(_opCtx, _collUuid);
         }
     }
 
     if (!_rowStoreCursor) {
-        _rowStoreCursor = _coll->getCursor(_opCtx, true /* forward */);
+        _rowStoreCursor = _coll.getPtr()->getCursor(_opCtx, true /* forward */);
     }
 
     if (_columnCursors.empty()) {
-        auto entry = _weakIndexCatalogEntry.lock();
+        auto desc = _coll.getPtr()->getIndexCatalog()->findIndexByIdent(_opCtx, _columnIndexIdent);
         tassert(6610210,
                 str::stream() << "expected IndexCatalogEntry for index named: " << _columnIndexName,
-                static_cast<bool>(entry));
+                static_cast<bool>(desc));
 
+        auto entry = desc->getEntry();
         auto iam = static_cast<ColumnStoreAccessMethod*>(entry->accessMethod());
 
         for (size_t i = 0; i < _paths.size(); i++) {
@@ -306,8 +314,8 @@ void ColumnScanStage::open(bool reOpen) {
 }
 
 TranslatedCell ColumnScanStage::translateCell(PathView path, const SplitCellView& splitCellView) {
-    SplitCellView::Cursor<value::ColumnStoreEncoder> cellCursor =
-        splitCellView.subcellValuesGenerator<value::ColumnStoreEncoder>(&_encoder);
+    SplitCellView::Cursor<ColumnStoreEncoder> cellCursor =
+        splitCellView.subcellValuesGenerator<ColumnStoreEncoder>(&_encoder);
     return TranslatedCell{splitCellView.arrInfo, path, std::move(cellCursor)};
 }
 
@@ -336,10 +344,11 @@ void ColumnScanStage::readParentsIntoObj(StringData path,
     // If we inserted a new entry, replace the null with an actual cursor.
     if (inserted) {
         invariant(!it->second);
-        auto entry = _weakIndexCatalogEntry.lock();
+        auto desc = _coll.getPtr()->getIndexCatalog()->findIndexByIdent(_opCtx, _columnIndexIdent);
         tassert(6610211,
                 str::stream() << "expected IndexCatalogEntry for index named: " << _columnIndexName,
-                static_cast<bool>(entry));
+                static_cast<bool>(desc));
+        auto entry = desc->getEntry();
         auto iam = static_cast<ColumnStoreAccessMethod*>(entry->accessMethod());
 
         it->second =
@@ -544,7 +553,7 @@ PlanState ColumnScanStage::getNext() {
     // call.
     disableSlotAccess();
 
-    checkForInterrupt(_opCtx);
+    checkForInterruptAndYield(_opCtx);
 
     if (_scanTracker.isScanningRowstore()) {
         _scanTracker.track();
@@ -641,15 +650,7 @@ PlanState ColumnScanStage::getNext() {
             false, value::TypeTags::RecordId, value::bitcastFrom<RecordId*>(&_recordId));
     }
 
-    if (_tracker && _tracker->trackProgress<TrialRunTracker::kNumReads>(1)) {
-        // If we're collecting execution stats during multi-planning and reached the end of the
-        // trial period because we've performed enough physical reads, bail out from the trial
-        // run by raising a special exception to signal a runtime planner that this candidate
-        // plan has completed its trial run early. Note that a trial period is executed only
-        // once per a PlanStage tree, and once completed never run again on the same tree.
-        _tracker = nullptr;
-        uasserted(ErrorCodes::QueryTrialRunCompleted, "Trial run early exit in scan");
-    }
+    trackRead();
 
     return trackPlanState(PlanState::ADVANCED);
 }
