@@ -29,14 +29,39 @@
 
 #pragma once
 
-#include "mongo/config.h"
+#include <algorithm>
+#include <cstddef>
+#include <memory>
+#include <string>
+#include <utility>
+#include <vector>
+
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/string_data.h"
+#include "mongo/config.h"  // IWYU pragma: keep
+#include "mongo/db/catalog/index_catalog_entry.h"
+#include "mongo/db/exec/plan_stats.h"
+#include "mongo/db/exec/sbe/column_store_encoder.h"
+#include "mongo/db/exec/sbe/columnar.h"
 #include "mongo/db/exec/sbe/expressions/expression.h"
 #include "mongo/db/exec/sbe/stages/collection_helpers.h"
 #include "mongo/db/exec/sbe/stages/plan_stats.h"
 #include "mongo/db/exec/sbe/stages/stages.h"
-#include "mongo/db/exec/sbe/values/column_store_encoder.h"
-#include "mongo/db/exec/sbe/values/columnar.h"
+#include "mongo/db/exec/sbe/util/debug_print.h"
+#include "mongo/db/exec/sbe/values/slot.h"
+#include "mongo/db/exec/sbe/values/value.h"
+#include "mongo/db/exec/sbe/vm/vm.h"
+#include "mongo/db/field_ref.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/query/plan_yield_policy.h"
+#include "mongo/db/query/stage_types.h"
+#include "mongo/db/record_id.h"
 #include "mongo/db/storage/column_store.h"
+#include "mongo/db/storage/record_store.h"
+#include "mongo/util/string_map.h"
+#include "mongo/util/uuid.h"
 
 namespace mongo {
 namespace sbe {
@@ -72,7 +97,6 @@ public:
     };
 
     ColumnScanStage(UUID collectionUuid,
-                    DatabaseName dbName,
                     StringData columnIndexName,
                     std::vector<std::string> paths,
                     bool densePathIncludedInScan,
@@ -104,9 +128,6 @@ protected:
     void doRestoreState(bool relinquishCursor) override;
     void doDetachFromOperationContext() override;
     void doAttachToOperationContext(OperationContext* opCtx) override;
-    void doDetachFromTrialRunTracker() override;
-    TrialRunTrackerAttachResultMask doAttachToTrialRunTracker(
-        TrialRunTracker* tracker, TrialRunTrackerAttachResultMask childrenAttachResult) override;
 
 private:
     /**
@@ -234,16 +255,13 @@ private:
 
     void processRecordFromRowstore(const Record& record);
 
-    value::ColumnStoreEncoder _encoder{};
+    ColumnStoreEncoder _encoder{};
 
     // The columnar index this stage is scanning and the associated row store collection.
     const UUID _collUuid;
-    const DatabaseName _dbName;
     const std::string _columnIndexName;
-    CollectionPtr _coll;
-    boost::optional<NamespaceString> _collName;  // These two members are initialized in 'prepare()'
-    boost::optional<uint64_t> _catalogEpoch;     // and are not changed afterwards.
-    std::weak_ptr<const IndexCatalogEntry> _weakIndexCatalogEntry;
+    std::string _columnIndexIdent;
+    CollectionRef _coll;
 
     // Paths to be read from the index. '_includeInOutput' defines which of the fields should be
     // included into the reconstructed record and the order of paths in '_paths' defines the
@@ -362,10 +380,6 @@ private:
 
     bool _open{false};
 
-    // If provided, used during a trial run to accumulate certain execution stats. Once the trial
-    // run is complete, this pointer is reset to nullptr.
-    TrialRunTracker* _tracker{nullptr};
-
     ColumnScanStats _specificStats;
 };
 }  // namespace sbe
