@@ -27,22 +27,38 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/exec/sbe/stages/ix_scan.h"
+#include <absl/container/inlined_vector.h>
+#include <absl/container/node_hash_map.h>
+#include <absl/meta/type_traits.h>
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <utility>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/db/catalog/collection.h"
 #include "mongo/db/catalog/index_catalog.h"
+#include "mongo/db/client.h"
 #include "mongo/db/exec/sbe/expressions/compile_ctx.h"
 #include "mongo/db/exec/sbe/expressions/expression.h"
 #include "mongo/db/exec/sbe/size_estimator.h"
-#include "mongo/db/exec/sbe/values/bson.h"
-#include "mongo/db/exec/trial_run_tracker.h"
+#include "mongo/db/exec/sbe/stages/ix_scan.h"
 #include "mongo/db/index/index_access_method.h"
+#include "mongo/db/index/index_descriptor.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/query/query_knobs_gen.h"
+#include "mongo/db/record_id.h"
+#include "mongo/db/storage/recovery_unit.h"
+#include "mongo/db/storage/snapshot.h"
+#include "mongo/db/transaction_resources.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/concurrency/admission_context.h"
+#include "mongo/util/str.h"
 
 namespace mongo::sbe {
 IndexScanStageBase::IndexScanStageBase(StringData stageType,
                                        UUID collUuid,
-                                       DatabaseName dbName,
                                        StringData indexName,
                                        bool forward,
                                        boost::optional<value::SlotId> indexKeySlot,
@@ -55,9 +71,12 @@ IndexScanStageBase::IndexScanStageBase(StringData stageType,
                                        PlanNodeId nodeId,
                                        bool lowPriority,
                                        bool participateInTrialRunTracking)
-    : PlanStage(stageType, yieldPolicy, nodeId, participateInTrialRunTracking),
+    : PlanStage(stageType,
+                yieldPolicy,
+                nodeId,
+                participateInTrialRunTracking,
+                TrialRunTrackingType::TrackReads),
       _collUuid(collUuid),
-      _dbName(dbName),
       _indexName(indexName),
       _forward(forward),
       _indexKeySlot(indexKeySlot),
@@ -71,18 +90,6 @@ IndexScanStageBase::IndexScanStageBase(StringData stageType,
 }
 
 void IndexScanStageBase::prepareImpl(CompileCtx& ctx) {
-    if (_indexKeySlot) {
-        _recordAccessor = std::make_unique<value::OwnedValueAccessor>();
-    }
-
-    if (_recordIdSlot) {
-        _recordIdAccessor = std::make_unique<value::OwnedValueAccessor>();
-    }
-
-    if (_snapshotIdSlot) {
-        _snapshotIdAccessor = std::make_unique<value::OwnedValueAccessor>();
-    }
-
     _accessors.resize(_vars.size());
     for (size_t idx = 0; idx < _accessors.size(); ++idx) {
         auto [it, inserted] = _accessorMap.emplace(_vars[idx], &_accessors[idx]);
@@ -90,15 +97,16 @@ void IndexScanStageBase::prepareImpl(CompileCtx& ctx) {
     }
 
     tassert(5709602, "'_coll' should not be initialized prior to 'acquireCollection()'", !_coll);
-    std::tie(_coll, _collName, _catalogEpoch) = acquireCollection(_opCtx, _dbName, _collUuid);
+    _coll.acquireCollection(_opCtx, _collUuid);
 
-    auto indexCatalog = _coll->getIndexCatalog();
+    auto indexCatalog = _coll.getPtr()->getIndexCatalog();
     auto indexDesc = indexCatalog->findIndexByName(_opCtx, _indexName);
     tassert(4938500,
             str::stream() << "could not find index named '" << _indexName << "' in collection '"
-                          << _collName << "'",
+                          << _coll.getCollName()->toStringForErrorMsg() << "'",
             indexDesc);
 
+    _uniqueIndex = indexDesc->unique();
     _entry = indexCatalog->getEntry(indexDesc);
     tassert(4938503,
             str::stream() << "expected IndexCatalogEntry for index named: " << _indexName,
@@ -115,22 +123,27 @@ void IndexScanStageBase::prepareImpl(CompileCtx& ctx) {
         _indexIdentViewAccessor.reset();
     }
 
-    if (_snapshotIdAccessor) {
-        _latestSnapshotId = _opCtx->recoveryUnit()->getSnapshotId().toNumber();
+    if (_indexKeySlot) {
+        _recordAccessor.reset(
+            false, value::TypeTags::keyString, value::bitcastFrom<value::KeyStringEntry*>(&_key));
+    }
+
+    if (_snapshotIdSlot) {
+        _latestSnapshotId = shard_role_details::getRecoveryUnit(_opCtx)->getSnapshotId().toNumber();
     }
 }
 
 value::SlotAccessor* IndexScanStageBase::getAccessor(CompileCtx& ctx, value::SlotId slot) {
     if (_indexKeySlot && *_indexKeySlot == slot) {
-        return _recordAccessor.get();
+        return &_recordAccessor;
     }
 
     if (_recordIdSlot && *_recordIdSlot == slot) {
-        return _recordIdAccessor.get();
+        return &_recordIdAccessor;
     }
 
     if (_snapshotIdSlot && *_snapshotIdSlot == slot) {
-        return _snapshotIdAccessor.get();
+        return &_snapshotIdAccessor;
     }
 
     if (_indexIdentSlot && *_indexIdentSlot == slot) {
@@ -146,11 +159,11 @@ value::SlotAccessor* IndexScanStageBase::getAccessor(CompileCtx& ctx, value::Slo
 
 void IndexScanStageBase::doSaveState(bool relinquishCursor) {
     if (relinquishCursor) {
-        if (_recordAccessor) {
-            prepareForYielding(*_recordAccessor, slotsAccessible());
+        if (_indexKeySlot && !_key.owned() && slotsAccessible()) {
+            _key = std::move(*_key.makeCopy());
         }
-        if (_recordIdAccessor) {
-            prepareForYielding(*_recordIdAccessor, slotsAccessible());
+        if (_recordIdSlot) {
+            prepareForYielding(_recordIdAccessor, slotsAccessible());
         }
         for (auto& accessor : _accessors) {
             prepareForYielding(accessor, slotsAccessible());
@@ -172,18 +185,16 @@ void IndexScanStageBase::doSaveState(bool relinquishCursor) {
 }
 
 void IndexScanStageBase::restoreCollectionAndIndex() {
-    tassert(5777406, "Collection name should be initialized", _collName);
-    tassert(5777407, "Catalog epoch should be initialized", _catalogEpoch);
-    _coll = restoreCollection(_opCtx, *_collName, _dbName, _collUuid, *_catalogEpoch);
+    _coll.restoreCollection(_opCtx, _collUuid);
 
     auto [identTag, identVal] = _indexIdentAccessor.getViewOfValue();
     tassert(7566700, "Expected ident to be a string", value::isString(identTag));
 
     auto indexIdent = value::getStringView(identTag, identVal);
-    auto desc = _coll->getIndexCatalog()->findIndexByIdent(_opCtx, indexIdent);
+    auto desc = _coll.getPtr()->getIndexCatalog()->findIndexByIdent(_opCtx, indexIdent);
     uassert(ErrorCodes::QueryPlanKilled,
             str::stream() << "query plan killed :: index '" << _indexName << "' dropped",
-            desc && !desc->getEntry()->isDropped());
+            desc);
 
     // Re-obtain the index entry pointer that was set to null during yield preparation. It is safe
     // to access the index entry when the query is active, as its validity is protected by at least
@@ -198,7 +209,7 @@ void IndexScanStageBase::doRestoreState(bool relinquishCursor) {
     invariant(!_coll);
 
     // If this stage has not been prepared, then yield recovery is a no-op.
-    if (!_collName) {
+    if (!_coll.getCollName()) {
         return;
     }
     restoreCollectionAndIndex();
@@ -209,8 +220,8 @@ void IndexScanStageBase::doRestoreState(bool relinquishCursor) {
 
     // Yield is the only time during plan execution that the snapshotId can change. As such, we
     // update it accordingly as part of yield recovery.
-    if (_snapshotIdAccessor) {
-        _latestSnapshotId = _opCtx->recoveryUnit()->getSnapshotId().toNumber();
+    if (_snapshotIdSlot) {
+        _latestSnapshotId = shard_role_details::getRecoveryUnit(_opCtx)->getSnapshotId().toNumber();
     }
 }
 
@@ -223,24 +234,15 @@ void IndexScanStageBase::doDetachFromOperationContext() {
 
 void IndexScanStageBase::doAttachToOperationContext(OperationContext* opCtx) {
     if (_lowPriority && _open && gDeprioritizeUnboundedUserIndexScans.load() &&
-        _opCtx->getClient()->isFromUserConnection() && _opCtx->lockState()->shouldWaitForTicket()) {
-        _priority.emplace(opCtx->lockState(), AdmissionContext::Priority::kLow);
+        _opCtx->getClient()->isFromUserConnection() &&
+        shard_role_details::getLocker(_opCtx)->shouldWaitForTicket(_opCtx)) {
+        _priority.emplace(opCtx, AdmissionContext::Priority::kLow);
     }
     if (_cursor) {
         _cursor->reattachToOperationContext(opCtx);
     }
 }
 
-void IndexScanStageBase::doDetachFromTrialRunTracker() {
-    _tracker = nullptr;
-}
-
-PlanStage::TrialRunTrackerAttachResultMask IndexScanStageBase::doAttachToTrialRunTracker(
-    TrialRunTracker* tracker, TrialRunTrackerAttachResultMask childrenAttachResult) {
-    _tracker = tracker;
-    return childrenAttachResult | TrialRunTrackerAttachResultFlags::AttachedToStreamingStage;
-}
-
 void IndexScanStageBase::openImpl(bool reOpen) {
     _commonStats.opens++;
 
@@ -270,72 +272,64 @@ void IndexScanStageBase::openImpl(bool reOpen) {
     _scanState = ScanState::kNeedSeek;
 }
 
-void IndexScanStageBase::trackRead() {
+void IndexScanStageBase::trackIndexRead() {
     ++_specificStats.numReads;
-    if (_tracker && _tracker->trackProgress<TrialRunTracker::kNumReads>(1)) {
-        // If we're collecting execution stats during multi-planning and reached the end of the
-        // trial period because we've performed enough physical reads, bail out from the trial run
-        // by raising a special exception to signal a runtime planner that this candidate plan has
-        // completed its trial run early. Note that a trial period is executed only once per a
-        // PlanStage tree, and once completed never run again on the same tree.
-        _tracker = nullptr;
-        uasserted(ErrorCodes::QueryTrialRunCompleted, "Trial run early exit in ixscan");
-    }
+    trackRead();
 }
 
 PlanState IndexScanStageBase::getNext() {
     auto optTimer(getOptTimer(_opCtx));
 
     if (_lowPriority && !_priority && gDeprioritizeUnboundedUserIndexScans.load() &&
-        _opCtx->getClient()->isFromUserConnection() && _opCtx->lockState()->shouldWaitForTicket()) {
-        _priority.emplace(_opCtx->lockState(), AdmissionContext::Priority::kLow);
+        _opCtx->getClient()->isFromUserConnection() &&
+        shard_role_details::getLocker(_opCtx)->shouldWaitForTicket(_opCtx)) {
+        _priority.emplace(_opCtx, AdmissionContext::Priority::kLow);
     }
 
     // We are about to get next record from a storage cursor so do not bother saving our internal
     // state in case it yields as the state will be completely overwritten after the call.
     disableSlotAccess();
 
-    checkForInterrupt(_opCtx);
+    checkForInterruptAndYield(_opCtx);
 
     do {
         switch (_scanState) {
             case ScanState::kNeedSeek:
                 ++_specificStats.seeks;
-                trackRead();
+                trackIndexRead();
                 // Seek for key and establish the cursor position.
-                _nextRecord = seek();
+                _nextKeyString = seek();
                 break;
             case ScanState::kScanning:
-                trackRead();
-                _nextRecord = _cursor->nextKeyString();
+                trackIndexRead();
+                _nextKeyString = _cursor->nextKeyValueView();
                 break;
             case ScanState::kFinished:
                 _priority.reset();
                 return trackPlanState(PlanState::IS_EOF);
         }
-    } while (!validateKey(_nextRecord));
+    } while (!validateKey(_nextKeyString));
 
-    if (_recordAccessor) {
-        _recordAccessor->reset(false,
-                               value::TypeTags::ksValue,
-                               value::bitcastFrom<KeyString::Value*>(&_nextRecord->keyString));
+    if (_indexKeySlot) {
+        _key = value::KeyStringEntry{_nextKeyString};
     }
 
-    if (_recordIdAccessor) {
-        _recordIdAccessor->reset(
-            false, value::TypeTags::RecordId, value::bitcastFrom<RecordId*>(&_nextRecord->loc));
+    if (_recordIdSlot) {
+        auto nextRid = _nextKeyString.getRecordId();
+        _recordIdAccessor.reset(
+            false, value::TypeTags::RecordId, value::bitcastFrom<const RecordId*>(nextRid));
     }
 
-    if (_snapshotIdAccessor) {
+    if (_snapshotIdSlot) {
         // Copy the latest snapshot ID into the 'snapshotId' slot.
-        _snapshotIdAccessor->reset(value::TypeTags::NumberInt64,
-                                   value::bitcastFrom<uint64_t>(_latestSnapshotId));
+        _snapshotIdAccessor.reset(value::TypeTags::NumberInt64,
+                                  value::bitcastFrom<uint64_t>(_latestSnapshotId));
     }
 
     if (_accessors.size()) {
         _valuesBuffer.reset();
         readKeyStringValueIntoAccessors(
-            _nextRecord->keyString, *_ordering, &_valuesBuffer, &_accessors, _indexKeysToInclude);
+            _nextKeyString, *_ordering, &_valuesBuffer, &_accessors, _indexKeysToInclude);
     }
 
     return trackPlanState(PlanState::ADVANCED);
@@ -362,6 +356,10 @@ std::unique_ptr<PlanStageStats> IndexScanStageBase::getStats(bool includeDebugIn
         bob.appendNumber("keysExamined", static_cast<long long>(_specificStats.keysExamined));
         bob.appendNumber("seeks", static_cast<long long>(_specificStats.seeks));
         bob.appendNumber("numReads", static_cast<long long>(_specificStats.numReads));
+        if (_specificStats.keyCheckSkipped != 0) {
+            bob.appendNumber("keyCheckSkipped",
+                             static_cast<long long>(_specificStats.keyCheckSkipped));
+        }
         if (_indexKeySlot) {
             bob.appendNumber("indexKeySlot", static_cast<long long>(*_indexKeySlot));
         }
@@ -455,7 +453,6 @@ std::string IndexScanStageBase::getIndexName() const {
 }
 
 SimpleIndexScanStage::SimpleIndexScanStage(UUID collUuid,
-                                           DatabaseName dbName,
                                            StringData indexName,
                                            bool forward,
                                            boost::optional<value::SlotId> indexKeySlot,
@@ -472,7 +469,6 @@ SimpleIndexScanStage::SimpleIndexScanStage(UUID collUuid,
                                            bool participateInTrialRunTracking)
     : IndexScanStageBase(seekKeyLow ? "ixseek"_sd : "ixscan"_sd,
                          collUuid,
-                         dbName,
                          indexName,
                          forward,
                          indexKeySlot,
@@ -494,7 +490,6 @@ SimpleIndexScanStage::SimpleIndexScanStage(UUID collUuid,
 
 std::unique_ptr<PlanStage> SimpleIndexScanStage::clone() const {
     return std::make_unique<SimpleIndexScanStage>(_collUuid,
-                                                  _dbName,
                                                   _indexName,
                                                   _forward,
                                                   _indexKeySlot,
@@ -508,7 +503,7 @@ std::unique_ptr<PlanStage> SimpleIndexScanStage::clone() const {
                                                   _yieldPolicy,
                                                   _commonStats.nodeId,
                                                   _lowPriority,
-                                                  _participateInTrialRunTracking);
+                                                  participateInTrialRunTracking());
 }
 
 void SimpleIndexScanStage::prepare(CompileCtx& ctx) {
@@ -521,20 +516,21 @@ void SimpleIndexScanStage::prepare(CompileCtx& ctx) {
     if (_seekKeyHigh) {
         ctx.root = this;
         _seekKeyHighCode = _seekKeyHigh->compile(ctx);
-        _seekKeyHighHolder = std::make_unique<value::OwnedValueAccessor>();
     }
-    _seekKeyLowHolder = std::make_unique<value::OwnedValueAccessor>();
+
+    _seekKeyLowHolder.reset();
+    _seekKeyHighHolder.reset();
 }
 
 void SimpleIndexScanStage::doSaveState(bool relinquishCursor) {
     // Seek points are external to the index scan and must be accessible no matter what as long
     // as the index scan is opened.
     if (_open && relinquishCursor) {
-        if (_seekKeyLowHolder) {
-            prepareForYielding(*_seekKeyLowHolder, true);
+        if (_seekKeyLow) {
+            prepareForYielding(_seekKeyLowHolder, true);
         }
-        if (_seekKeyHighHolder) {
-            prepareForYielding(*_seekKeyHighHolder, true);
+        if (_seekKeyHigh) {
+            prepareForYielding(_seekKeyHighHolder, true);
         }
     }
 
@@ -551,46 +547,51 @@ void SimpleIndexScanStage::open(bool reOpen) {
         const auto msgTagLow = tagLow;
         uassert(4822851,
                 str::stream() << "seek key is wrong type: " << msgTagLow,
-                tagLow == value::TypeTags::ksValue);
-        _seekKeyLowHolder->reset(ownedLow, tagLow, valLow);
+                tagLow == value::TypeTags::keyString);
+        _seekKeyLowHolder.reset(ownedLow, tagLow, valLow);
 
         auto [ownedHi, tagHi, valHi] = _bytecode.run(_seekKeyHighCode.get());
         const auto msgTagHi = tagHi;
         uassert(4822852,
                 str::stream() << "seek key is wrong type: " << msgTagHi,
-                tagHi == value::TypeTags::ksValue);
+                tagHi == value::TypeTags::keyString);
 
-        _seekKeyHighHolder->reset(ownedHi, tagHi, valHi);
+        _seekKeyHighHolder.reset(ownedHi, tagHi, valHi);
+
+        // It is a point bound if the lowKey and highKey are same except discriminator.
+        auto& highKey = getSeekKeyHigh();
+        _pointBound = getSeekKeyLow().compareWithoutDiscriminator(highKey) == 0 &&
+            highKey.computeElementCount(*_ordering) == _entry->descriptor()->getNumFields();
+
+        _cursor->setEndPosition(highKey);
     } else if (_seekKeyLow) {
         auto [ownedLow, tagLow, valLow] = _bytecode.run(_seekKeyLowCode.get());
         const auto msgTagLow = tagLow;
         uassert(4822853,
                 str::stream() << "seek key is wrong type: " << msgTagLow,
-                tagLow == value::TypeTags::ksValue);
-        _seekKeyLowHolder->reset(ownedLow, tagLow, valLow);
+                tagLow == value::TypeTags::keyString);
+        _seekKeyLowHolder.reset(ownedLow, tagLow, valLow);
     } else {
         auto sdi = _entry->accessMethod()->asSortedData()->getSortedDataInterface();
-        KeyString::Builder kb(sdi->getKeyStringVersion(),
-                              sdi->getOrdering(),
-                              KeyString::Discriminator::kExclusiveBefore);
-        kb.appendDiscriminator(KeyString::Discriminator::kExclusiveBefore);
+        key_string::Builder kb(sdi->getKeyStringVersion(),
+                               sdi->getOrdering(),
+                               key_string::Discriminator::kExclusiveBefore);
+        kb.appendDiscriminator(key_string::Discriminator::kExclusiveBefore);
 
-        auto [copyTag, copyVal] = value::makeCopyKeyString(kb.getValueCopy());
-        _seekKeyLowHolder->reset(true, copyTag, copyVal);
+        auto [copyTag, copyVal] = value::makeKeyString(kb.getValueCopy());
+        _seekKeyLowHolder.reset(true, copyTag, copyVal);
     }
 }
 
-const KeyString::Value& SimpleIndexScanStage::getSeekKeyLow() const {
-    auto [tag, value] = _seekKeyLowHolder->getViewOfValue();
-    return *value::getKeyStringView(value);
+const key_string::Value& SimpleIndexScanStage::getSeekKeyLow() const {
+    auto [tag, value] = _seekKeyLowHolder.getViewOfValue();
+    return value::getKeyString(value)->getValue();
 }
 
-const KeyString::Value* SimpleIndexScanStage::getSeekKeyHigh() const {
-    if (!_seekKeyHighHolder) {
-        return nullptr;
-    }
-    auto [tag, value] = _seekKeyHighHolder->getViewOfValue();
-    return value::getKeyStringView(value);
+const key_string::Value& SimpleIndexScanStage::getSeekKeyHigh() const {
+    tassert(8843702, "HighKey must exist", _seekKeyHigh);
+    auto [tag, value] = _seekKeyHighHolder.getViewOfValue();
+    return value::getKeyString(value)->getValue();
 }
 
 std::unique_ptr<PlanStageStats> SimpleIndexScanStage::getStats(bool includeDebugInfo) const {
@@ -637,40 +638,27 @@ size_t SimpleIndexScanStage::estimateCompileTimeSize() const {
     return size;
 }
 
-boost::optional<KeyStringEntry> SimpleIndexScanStage::seek() {
-    return _cursor->seekForKeyString(getSeekKeyLow());
+SortedDataKeyValueView SimpleIndexScanStage::seek() {
+    auto& query = getSeekKeyLow();
+    return _cursor->seekForKeyValueView(StringData(query.getBuffer(), query.getSize()));
 }
 
-bool SimpleIndexScanStage::validateKey(const boost::optional<KeyStringEntry>& key) {
-    if (!key) {
+bool SimpleIndexScanStage::validateKey(const SortedDataKeyValueView& key) {
+    if (key.isEmpty()) {
         _scanState = ScanState::kFinished;
         return false;
     }
 
-    if (auto seekKeyHigh = getSeekKeyHigh(); seekKeyHigh) {
-        auto cmp = key->keyString.compare(*seekKeyHigh);
-
-        if (_forward) {
-            if (cmp > 0) {
-                _scanState = ScanState::kFinished;
-                return false;
-            }
-        } else {
-            if (cmp < 0) {
-                _scanState = ScanState::kFinished;
-                return false;
-            }
-        }
-    }
     // Note: we may in the future want to bump 'keysExamined' for comparisons to a key that result
     // in the stage returning EOF.
     ++_specificStats.keysExamined;
-    _scanState = ScanState::kScanning;
+
+    // For point bound on unique index, there's only one possible key.
+    _scanState = _pointBound && _uniqueIndex ? ScanState::kFinished : ScanState::kScanning;
     return true;
 }
 
 GenericIndexScanStage::GenericIndexScanStage(UUID collUuid,
-                                             DatabaseName dbName,
                                              StringData indexName,
                                              GenericIndexScanStageParams params,
                                              boost::optional<value::SlotId> indexKeySlot,
@@ -684,7 +672,6 @@ GenericIndexScanStage::GenericIndexScanStage(UUID collUuid,
                                              bool participateInTrialRunTracking)
     : IndexScanStageBase("ixscan_generic"_sd,
                          collUuid,
-                         dbName,
                          indexName,
                          params.direction == 1,
                          indexKeySlot,
@@ -696,7 +683,8 @@ GenericIndexScanStage::GenericIndexScanStage(UUID collUuid,
                          yieldPolicy,
                          planNodeId,
                          participateInTrialRunTracking),
-      _params{std::move(params)} {}
+      _params{std::move(params)},
+      _endKey{_params.version} {}
 
 std::unique_ptr<PlanStage> GenericIndexScanStage::clone() const {
     sbe::GenericIndexScanStageParams params{_params.indexBounds->clone(),
@@ -705,7 +693,6 @@ std::unique_ptr<PlanStage> GenericIndexScanStage::clone() const {
                                             _params.version,
                                             _params.ord};
     return std::make_unique<GenericIndexScanStage>(_collUuid,
-                                                   _dbName,
                                                    _indexName,
                                                    std::move(params),
                                                    _indexKeySlot,
@@ -716,7 +703,7 @@ std::unique_ptr<PlanStage> GenericIndexScanStage::clone() const {
                                                    _vars,
                                                    _yieldPolicy,
                                                    _commonStats.nodeId,
-                                                   _participateInTrialRunTracking);
+                                                   participateInTrialRunTracking());
 }
 
 void GenericIndexScanStage::prepare(CompileCtx& ctx) {
@@ -765,25 +752,44 @@ size_t GenericIndexScanStage::estimateCompileTimeSize() const {
     return size;
 }
 
-boost::optional<KeyStringEntry> GenericIndexScanStage::seek() {
-    return _cursor->seekForKeyString(IndexEntryComparison::makeKeyStringFromSeekPointForSeek(
-        _seekPoint, _params.version, _params.ord, _forward));
+SortedDataKeyValueView GenericIndexScanStage::seek() {
+    key_string::Builder builder(_params.version, _params.ord);
+    return _cursor->seekForKeyValueView(
+        IndexEntryComparison::makeKeyStringFromSeekPointForSeek(_seekPoint, _forward, builder));
 }
 
-bool GenericIndexScanStage::validateKey(const boost::optional<KeyStringEntry>& key) {
-    if (key && _checker) {
-        ++_specificStats.keysExamined;
+bool GenericIndexScanStage::validateKey(const SortedDataKeyValueView& key) {
+    if (key.isEmpty()) {
+        _scanState = ScanState::kFinished;
+        return false;
+    }
+    ++_specificStats.keysExamined;
+    auto keyStringData = key.getKeyStringWithoutRecordIdView();
 
+    if (!_endKey.isEmpty()) {
+        auto result = key_string::compare(
+            keyStringData.data(), _endKey.getBuffer(), keyStringData.size(), _endKey.getSize());
+        if ((_forward && result <= 0) || (!_forward && result >= 0)) {
+            ++_specificStats.keyCheckSkipped;
+            _scanState = ScanState::kScanning;
+            return true;
+        }
+    }
+
+    if (_checker) {
         _keyBuffer.reset();
         BSONObjBuilder keyBuilder(_keyBuffer);
-        KeyString::toBsonSafe(key->keyString.getBuffer(),
-                              key->keyString.getSize(),
-                              _params.ord,
-                              key->keyString.getTypeBits(),
-                              keyBuilder);
+
+        auto typeBits = key.getTypeBitsView();
+        BufReader typeBitsBr(typeBits.data(), typeBits.size());
+        auto reader = key_string::TypeBits::getReaderFromBuffer(key.getVersion(), &typeBitsBr);
+
+        key_string::toBsonSafe(
+            keyStringData.data(), keyStringData.size(), _params.ord, reader, keyBuilder);
         auto bsonKey = keyBuilder.done();
 
-        switch (_checker->checkKey(bsonKey, &_seekPoint)) {
+        switch (_checker->checkKeyWithEndPosition(
+            bsonKey, &_seekPoint, _endKey, _params.ord, _forward)) {
             case IndexBoundsChecker::VALID:
                 _scanState = ScanState::kScanning;
                 return true;
