@@ -29,14 +29,41 @@
 
 #pragma once
 
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <cstddef>
+#include <cstdint>
+#include <memory>
+#include <string>
+#include <vector>
+
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonobj.h"
 #include "mongo/bson/ordering.h"
+#include "mongo/bson/util/builder.h"
+#include "mongo/db/admission/execution_admission_context.h"
+#include "mongo/db/catalog/index_catalog_entry.h"
 #include "mongo/db/db_raii.h"
+#include "mongo/db/exec/plan_stats.h"
 #include "mongo/db/exec/sbe/expressions/expression.h"
 #include "mongo/db/exec/sbe/stages/collection_helpers.h"
+#include "mongo/db/exec/sbe/stages/plan_stats.h"
 #include "mongo/db/exec/sbe/stages/stages.h"
+#include "mongo/db/exec/sbe/util/debug_print.h"
+#include "mongo/db/exec/sbe/values/slot.h"
+#include "mongo/db/exec/sbe/values/value.h"
 #include "mongo/db/exec/sbe/vm/vm.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/query/index_bounds.h"
+#include "mongo/db/query/plan_yield_policy.h"
+#include "mongo/db/query/stage_types.h"
+#include "mongo/db/storage/index_entry_comparison.h"
+#include "mongo/db/storage/key_string.h"
 #include "mongo/db/storage/record_store.h"
 #include "mongo/db/storage/sorted_data_interface.h"
+#include "mongo/db/transaction_resources.h"
+#include "mongo/util/uuid.h"
 
 namespace mongo::sbe {
 
@@ -62,7 +89,6 @@ class IndexScanStageBase : public PlanStage {
 public:
     IndexScanStageBase(StringData stageType,
                        UUID collUuid,
-                       DatabaseName dbName,
                        StringData indexName,
                        bool forward,
                        boost::optional<value::SlotId> indexKeySlot,
@@ -99,27 +125,25 @@ protected:
     };
 
     // Seeks and returns the first/next index KeyStringEntry or boost::none if no such key exists.
-    virtual boost::optional<KeyStringEntry> seek() = 0;
+    virtual SortedDataKeyValueView seek() = 0;
     // Returns true if the 'key' is within the bounds and false otherwise. Implementations may set
     // state internally to reflect whether the scan is done, or whether a new seek point should be
     // used.
-    virtual bool validateKey(const boost::optional<KeyStringEntry>& key) = 0;
+    virtual bool validateKey(const SortedDataKeyValueView& key) = 0;
 
     void doSaveState(bool relinquishCursor) override;
     void doRestoreState(bool relinquishCursor) final;
     void doDetachFromOperationContext() final;
     void doAttachToOperationContext(OperationContext* opCtx) final;
-    void doDetachFromTrialRunTracker() final;
-    TrialRunTrackerAttachResultMask doAttachToTrialRunTracker(
-        TrialRunTracker* tracker, TrialRunTrackerAttachResultMask childrenAttachResult) final;
+
     /**
      * When this stage is re-opened after being closed, or during yield recovery, called to verify
      * that the index (and the index's collection) remain valid. If any validity check fails, throws
      * a UserException that terminates execution of the query.
      */
     void restoreCollectionAndIndex();
-    // Bumps '_specificStats.numReads' and calls trackProgress() if '_tracker' is non-null.
-    void trackRead();
+    // Bumps '_specificStats.numReads' and calls base class trackRead().
+    void trackIndexRead();
     // Shares the common code for PlanStage::prepare() implementation.
     void prepareImpl(CompileCtx& ctx);
     // Shares the common code for PlanStage::open() implementation.
@@ -130,7 +154,6 @@ protected:
     void debugPrintImpl(std::vector<DebugPrinter::Block>&) const;
 
     const UUID _collUuid;
-    const DatabaseName _dbName;
     const std::string _indexName;
     const bool _forward;
     const boost::optional<value::SlotId> _indexKeySlot;
@@ -142,23 +165,18 @@ protected:
 
     vm::ByteCode _bytecode;
 
-    // These members are default constructed to boost::none and are initialized when 'prepare()'
-    // is called. Once they are set, they are never modified again.
-    boost::optional<NamespaceString> _collName;
-    boost::optional<uint64_t> _catalogEpoch;
+    CollectionRef _coll;
 
-    CollectionPtr _coll;
-
-    std::unique_ptr<value::OwnedValueAccessor> _recordAccessor;
-    std::unique_ptr<value::OwnedValueAccessor> _recordIdAccessor;
-    std::unique_ptr<value::OwnedValueAccessor> _snapshotIdAccessor;
+    value::OwnedValueAccessor _recordAccessor;
+    value::OwnedValueAccessor _recordIdAccessor;
+    value::OwnedValueAccessor _snapshotIdAccessor;
 
     value::OwnedValueAccessor _indexIdentAccessor;
     value::ViewOfValueAccessor _indexIdentViewAccessor;
 
-    // This field holds the latest snapshot ID that we've received from _opCtx->recoveryUnit().
-    // This field gets initialized by prepare(), and it gets updated each time doRestoreState() is
-    // called.
+    // This field holds the latest snapshot ID that we've received from the recovery unit of the
+    // operation. This field gets initialized by prepare(), and it gets updated each time
+    // doRestoreState() is called.
     uint64_t _latestSnapshotId{0};
 
     // One accessor and slot for each key component that this stage will bind from an index entry's
@@ -169,22 +187,20 @@ protected:
     std::unique_ptr<SortedDataInterface::Cursor> _cursor;
     const IndexCatalogEntry* _entry{nullptr};
     boost::optional<Ordering> _ordering{boost::none};
-    boost::optional<KeyStringEntry> _nextRecord;
+    SortedDataKeyValueView _nextKeyString;
+    value::KeyStringEntry _key;
 
     // This buffer stores values that are projected out of the index entry. Values in the
     // '_accessors' list that are pointers point to data in this buffer.
     BufBuilder _valuesBuffer;
 
     bool _open{false};
+    bool _uniqueIndex{false};
     ScanState _scanState = ScanState::kNeedSeek;
     IndexScanStats _specificStats;
 
     bool _lowPriority;
-    boost::optional<ScopedAdmissionPriorityForLock> _priority;
-
-    // If provided, used during a trial run to accumulate certain execution stats. Once the trial
-    // run is complete, this pointer is reset to nullptr.
-    TrialRunTracker* _tracker{nullptr};
+    boost::optional<ScopedAdmissionPriority<ExecutionAdmissionContext>> _priority;
 };
 
 /**
@@ -207,7 +223,6 @@ protected:
 class SimpleIndexScanStage final : public IndexScanStageBase {
 public:
     SimpleIndexScanStage(UUID collUuid,
-                         DatabaseName dbName,
                          StringData indexName,
                          bool forward,
                          boost::optional<value::SlotId> indexKeySlot,
@@ -233,12 +248,12 @@ public:
 
 protected:
     void doSaveState(bool relinquishCursor) override;
-    boost::optional<KeyStringEntry> seek() override;
-    bool validateKey(const boost::optional<KeyStringEntry>& key) override;
+    SortedDataKeyValueView seek() override;
+    bool validateKey(const SortedDataKeyValueView& key) override;
 
 private:
-    const KeyString::Value& getSeekKeyLow() const;
-    const KeyString::Value* getSeekKeyHigh() const;
+    const key_string::Value& getSeekKeyLow() const;
+    const key_string::Value& getSeekKeyHigh() const;
 
     std::unique_ptr<EExpression> _seekKeyLow;
     std::unique_ptr<EExpression> _seekKeyHigh;
@@ -247,8 +262,10 @@ private:
     std::unique_ptr<vm::CodeFragment> _seekKeyLowCode;
     std::unique_ptr<vm::CodeFragment> _seekKeyHighCode;
 
-    std::unique_ptr<value::OwnedValueAccessor> _seekKeyLowHolder;
-    std::unique_ptr<value::OwnedValueAccessor> _seekKeyHighHolder;
+    value::OwnedValueAccessor _seekKeyLowHolder;
+    value::OwnedValueAccessor _seekKeyHighHolder;
+
+    bool _pointBound{false};
 };
 
 /**
@@ -270,13 +287,12 @@ struct GenericIndexScanStageParams {
     std::unique_ptr<EExpression> indexBounds;
     const BSONObj keyPattern;
     const int direction;
-    const KeyString::Version version;
+    const key_string::Version version;
     const Ordering ord;
 };
 class GenericIndexScanStage final : public IndexScanStageBase {
 public:
     GenericIndexScanStage(UUID collUuid,
-                          DatabaseName dbName,
                           StringData indexName,
                           GenericIndexScanStageParams params,
                           boost::optional<value::SlotId> indexKeySlot,
@@ -298,8 +314,8 @@ public:
     size_t estimateCompileTimeSize() const override;
 
 protected:
-    boost::optional<KeyStringEntry> seek() override;
-    bool validateKey(const boost::optional<KeyStringEntry>& key) override;
+    SortedDataKeyValueView seek() override;
+    bool validateKey(const SortedDataKeyValueView& key) override;
 
     const GenericIndexScanStageParams _params;
 
@@ -307,5 +323,7 @@ protected:
     IndexSeekPoint _seekPoint;
     std::unique_ptr<vm::CodeFragment> _indexBoundsCode;
     boost::optional<IndexBoundsChecker> _checker;
+    // The end position for current range, empty if currently not in a valid range.
+    key_string::Builder _endKey;
 };
 }  // namespace mongo::sbe
