@@ -29,8 +29,19 @@
 
 #pragma once
 
+#include <cstddef>
+#include <memory>
+#include <vector>
+
+#include <boost/optional/optional.hpp>
+
+#include "mongo/db/exec/plan_stats.h"
 #include "mongo/db/exec/sbe/expressions/expression.h"
+#include "mongo/db/exec/sbe/stages/plan_stats.h"
 #include "mongo/db/exec/sbe/stages/stages.h"
+#include "mongo/db/exec/sbe/util/debug_print.h"
+#include "mongo/db/exec/sbe/values/slot.h"
+#include "mongo/db/query/stage_types.h"
 
 namespace mongo::sbe {
 /**
@@ -68,9 +79,15 @@ public:
     std::vector<DebugPrinter::Block> debugPrint() const final;
     size_t estimateCompileTimeSize() const final;
 
+protected:
+    bool shouldOptimizeSaveState(size_t) const final {
+        return true;
+    }
+
 private:
-    boost::optional<int64_t> _runLimitOrSkipCode(const vm::CodeFragment* code,
-                                                 vm::ByteCode& bytecode);
+    boost::optional<int64_t> _runLimitOrSkipCode(const vm::CodeFragment* code);
+
+    vm::ByteCode _bytecode;
 
     std::unique_ptr<EExpression> _limitExpr;
     std::unique_ptr<EExpression> _skipExpr;
