@@ -27,13 +27,20 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <utility>
 
+#include <absl/container/flat_hash_map.h>
+#include <absl/container/inlined_vector.h>
+#include <absl/meta/type_traits.h>
+
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/db/exec/sbe/expressions/compile_ctx.h"
+#include "mongo/db/exec/sbe/size_estimator.h"
 #include "mongo/db/exec/sbe/stages/loop_join.h"
 #include "mongo/db/exec/sbe/stages/stage_visitors.h"
-
-#include "mongo/db/exec/sbe/size_estimator.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/str.h"
 
 namespace mongo::sbe {
@@ -63,7 +70,7 @@ LoopJoinStage::LoopJoinStage(std::unique_ptr<PlanStage> outer,
                              JoinType joinType,
                              PlanNodeId nodeId,
                              bool participateInTrialRunTracking)
-    : PlanStage("nlj"_sd, nodeId, participateInTrialRunTracking),
+    : PlanStage("nlj"_sd, nullptr /* yieldPolicy */, nodeId, participateInTrialRunTracking),
       _outerProjects(std::move(outerProjects)),
       _outerCorrelated(std::move(outerCorrelated)),
       _innerProjects(std::move(innerProjects)),
@@ -85,7 +92,7 @@ std::unique_ptr<PlanStage> LoopJoinStage::clone() const {
                                            _predicate ? _predicate->clone() : nullptr,
                                            _joinType,
                                            _commonStats.nodeId,
-                                           _participateInTrialRunTracking);
+                                           participateInTrialRunTracking());
 }
 
 void LoopJoinStage::prepare(CompileCtx& ctx) {
@@ -192,13 +199,6 @@ PlanState LoopJoinStage::getNext() {
     }
 }
 
-void LoopJoinStage::saveChildrenState(bool relinquishCursor, bool disableSlotAccess) {
-    // LoopJoinStage::getNext() only guarantees that the inner child's getNext() was called. Thus,
-    // it is safe to propagate disableSlotAccess to the inner child, but not to the outer child.
-    _children[1]->saveState(relinquishCursor, disableSlotAccess);
-    _children[0]->saveState(relinquishCursor, false);
-}
-
 void LoopJoinStage::close() {
     auto optTimer(getOptTimer(_opCtx));
 
