@@ -34,10 +34,13 @@
 
 #include "mongo/bson/util/builder.h"
 #include "mongo/db/exec/sbe/values/row.h"
+#include "mongo/db/exec/sbe/values/slot.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/record_id.h"
+#include "mongo/db/storage/key_string.h"
 #include "mongo/db/storage/record_store.h"
 #include "mongo/db/storage/temporary_record_store.h"
+#include "mongo/platform/basic.h"
 
 namespace mongo {
 namespace sbe {
@@ -49,11 +52,11 @@ namespace sbe {
 void assertIgnorePrepareConflictsBehavior(OperationContext* opCtx);
 
 // Encode key as a RecordId and TypeBits.
-std::pair<RecordId, KeyString::TypeBits> encodeKeyString(KeyString::Builder&,
-                                                         const value::MaterializedRow& value);
+std::pair<RecordId, key_string::TypeBits> encodeKeyString(key_string::Builder&,
+                                                          const value::MaterializedRow& value);
 
 // Reconstructs the KeyString carried in RecordId using 'typeBits'.
-KeyString::Value decodeKeyString(const RecordId& rid, KeyString::TypeBits typeBits);
+key_string::Value decodeKeyString(const RecordId& rid, key_string::TypeBits typeBits);
 
 /**
  * SpillingStore is a wrapper around a temporary record store than maintains its own transaction as
@@ -84,12 +87,12 @@ public:
     int upsertToRecordStore(OperationContext* opCtx,
                             const RecordId& key,
                             const value::MaterializedRow& val,
-                            const KeyString::TypeBits& typeBits,
+                            const key_string::TypeBits& typeBits,
                             bool update);
     int upsertToRecordStore(OperationContext* opCtx,
                             const RecordId& key,
                             BufBuilder& buf,
-                            const KeyString::TypeBits& typeBits,  // recover type of value.
+                            const key_string::TypeBits& typeBits,  // recover type of value.
                             bool update);
     int upsertToRecordStore(OperationContext* opCtx,
                             const RecordId& key,
