@@ -27,28 +27,47 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/exec/sbe/values/value.h"
+#include <absl/container/flat_hash_map.h>
+#include <absl/hash/hash.h>
+#include <absl/strings/string_view.h>
+#include <boost/cstdint.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/numeric/conversion/converter_policies.hpp>
+#include <boost/optional/optional.hpp>
+#include <cmath>
 
 #include "mongo/base/compare_numbers.h"
+#include "mongo/base/string_data_comparator.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/util/builder.h"
 #include "mongo/db/exec/js_function.h"
 #include "mongo/db/exec/sbe/makeobj_spec.h"
-#include "mongo/db/exec/sbe/size_estimator.h"
+#include "mongo/db/exec/sbe/sort_spec.h"
+#include "mongo/db/exec/sbe/util/print_options.h"
+#include "mongo/db/exec/sbe/values/block_interface.h"
 #include "mongo/db/exec/sbe/values/bson.h"
-#include "mongo/db/exec/sbe/values/sort_spec.h"
+#include "mongo/db/exec/sbe/values/cell_interface.h"
+#include "mongo/db/exec/sbe/values/generic_compare.h"
+#include "mongo/db/exec/sbe/values/slot.h"
+#include "mongo/db/exec/sbe/values/value.h"
 #include "mongo/db/exec/sbe/values/value_builder.h"
 #include "mongo/db/exec/sbe/values/value_printer.h"
+#include "mongo/db/index/btree_key_generator.h"
+#include "mongo/db/index/sort_key_generator.h"
 #include "mongo/db/query/collation/collator_interface.h"
 #include "mongo/db/query/datetime/date_time_support.h"
+#include "mongo/db/query/sort_pattern.h"
 #include "mongo/db/storage/key_string.h"
+#include "mongo/util/bufreader.h"
+#include "mongo/util/duration.h"
 #include "mongo/util/errno_util.h"
 #include "mongo/util/pcre_util.h"
 
 namespace mongo {
 namespace sbe {
 namespace value {
-
 namespace {
 template <typename T>
 auto abslHash(const T& val) {
@@ -62,6 +81,27 @@ auto abslHash(const T& val) {
 }
 }  // namespace
 
+constexpr size_t gTypeOpsSize =
+    size_t(TypeTags::TypeTagsMax) - size_t(TypeTags::EndOfNativeTypeTags);
+
+const ExtendedTypeOps* gTypeOps[gTypeOpsSize] = {0};
+
+const ExtendedTypeOps* getExtendedTypeOps(TypeTags tag) {
+    dassert(size_t(tag) > size_t(TypeTags::EndOfNativeTypeTags));
+
+    size_t typeOpsIdx = size_t(tag) - (size_t(TypeTags::EndOfNativeTypeTags) + 1);
+    return gTypeOps[typeOpsIdx];
+}
+
+void registerExtendedTypeOps(TypeTags tag, const ExtendedTypeOps* typeOps) {
+    tassert(7690414,
+            "Expected tag value to be within the range of extended types",
+            size_t(tag) > size_t(TypeTags::EndOfNativeTypeTags));
+
+    size_t typeOpsIdx = size_t(tag) - (size_t(TypeTags::EndOfNativeTypeTags) + 1);
+    gTypeOps[typeOpsIdx] = typeOps;
+}
+
 std::pair<TypeTags, Value> makeNewBsonRegex(StringData pattern, StringData flags) {
     // Add 2 to account NULL bytes after pattern and flags.
     auto totalSize = pattern.size() + flags.size() + 2;
@@ -131,21 +171,9 @@ std::pair<TypeTags, Value> makeNewBsonCodeWScope(StringData code, const char* sc
     return {TypeTags::bsonCodeWScope, bitcastFrom<char*>(buffer.release())};
 }
 
-std::pair<TypeTags, Value> makeCopyKeyString(const KeyString::Value& inKey) {
-    auto k = new KeyString::Value(inKey);
-    return {TypeTags::ksValue, bitcastFrom<KeyString::Value*>(k)};
-}
-
-std::pair<TypeTags, Value> makeNewPcreRegex(StringData pattern, StringData options) {
-    auto regex =
-        std::make_unique<pcre::Regex>(std::string{pattern}, pcre_util::flagsToOptions(options));
-    uassert(5073402, str::stream() << "Invalid Regex: " << errorMessage(regex->error()), *regex);
-    return {TypeTags::pcreRegex, bitcastFrom<pcre::Regex*>(regex.release())};
-}
-
-std::pair<TypeTags, Value> makeCopyPcreRegex(const pcre::Regex& regex) {
-    auto regexCopy = std::make_unique<pcre::Regex>(regex);
-    return {TypeTags::pcreRegex, bitcastFrom<pcre::Regex*>(regexCopy.release())};
+std::pair<TypeTags, Value> makeKeyString(const key_string::Value& inKey) {
+    return {TypeTags::keyString,
+            bitcastFrom<value::KeyStringEntry*>(new value::KeyStringEntry(inKey))};
 }
 
 std::pair<TypeTags, Value> makeCopyTimeZone(const TimeZone& tz) {
@@ -153,108 +181,14 @@ std::pair<TypeTags, Value> makeCopyTimeZone(const TimeZone& tz) {
     return {TypeTags::timeZone, bitcastFrom<TimeZone*>(tzCopy.release())};
 }
 
-KeyString::Value SortSpec::generateSortKey(const BSONObj& obj, const CollatorInterface* collator) {
-    _sortKeyGen.setCollator(collator);
-    return _sortKeyGen.computeSortKeyString(obj);
+std::pair<TypeTags, Value> makeCopyValueBlock(const ValueBlock& b) {
+    auto cpy = b.clone();
+    return {TypeTags::valueBlock, bitcastFrom<ValueBlock*>(cpy.release())};
 }
 
-value::SortKeyComponentVector* SortSpec::generateSortKeyComponentVector(
-    FastTuple<bool, value::TypeTags, value::Value> obj, const CollatorInterface* collator) {
-    auto [objOwned, objTag, objVal] = obj;
-    ValueGuard guard(objOwned, objTag, objVal);
-
-    // While this function accepts any type of object, for now we simply convert everything
-    // to BSON. In the future, we may change this function to avoid the conversion.
-    auto bsonObj = [&, objTag = objTag, objVal = objVal, objOwned = objOwned]() {
-        if (objTag == value::TypeTags::bsonObject) {
-            if (objOwned) {
-                // Take ownership of the temporary object here.
-                _tempVal.emplace(objTag, objVal);
-                guard.reset();
-            }
-            return BSONObj{value::bitcastTo<const char*>(objVal)};
-        } else if (objTag == value::TypeTags::Object) {
-            BSONObjBuilder objBuilder;
-            bson::convertToBsonObj(objBuilder, value::getObjectView(objVal));
-            _tempObj = objBuilder.obj();
-            return _tempObj;
-        } else {
-            MONGO_UNREACHABLE_TASSERT(7103703);
-        }
-    }();
-
-
-    _sortKeyGen.setCollator(collator);
-    // Use the generic API for getting an array of bson elements representing the
-    // sort key.
-    _sortKeyGen.generateSortKeyComponentVector(bsonObj, &_localBsonEltStorage);
-
-    // Convert this array of BSONElements into the SBE SortKeyComponentVector type.
-    {
-        size_t i = 0;
-        for (auto& elt : _localBsonEltStorage) {
-            _localSortKeyComponentStorage.elts[i++] = bson::convertFrom<true>(elt);
-        }
-    }
-    return &_localSortKeyComponentStorage;
-}
-
-BtreeKeyGenerator SortSpec::initKeyGen() const {
-    tassert(5037003,
-            "SortSpec should not be passed an empty sort pattern",
-            !_sortPatternBson.isEmpty());
-
-    std::vector<const char*> fields;
-    std::vector<BSONElement> fixed;
-    for (auto&& elem : _sortPatternBson) {
-        fields.push_back(elem.fieldName());
-
-        // BtreeKeyGenerator's constructor's first parameter (the 'fields' vector) and second
-        // parameter (the 'fixed' vector) are parallel vectors. The 'fixed' vector allows the
-        // caller to specify if the any sort keys have already been determined for one or more
-        // of the field paths from the 'fields' vector. In this case, we haven't determined what
-        // the sort keys are for any of the fields paths, so we populate the 'fixed' vector with
-        // EOO values to indicate this.
-        fixed.emplace_back();
-    }
-
-    const bool isSparse = false;
-    auto version = KeyString::Version::kLatestVersion;
-    auto ordering = Ordering::make(_sortPatternBson);
-
-    return {std::move(fields), std::move(fixed), isSparse, version, ordering};
-}
-
-size_t SortSpec::getApproximateSize() const {
-    auto size = sizeof(SortSpec);
-    size += _sortKeyGen.getApproximateSize();
-    size += _sortPatternBson.isOwned() ? _sortPatternBson.objsize() : 0;
-    return size;
-}
-
-std::pair<TypeTags, Value> makeCopyJsFunction(const JsFunction& jsFunction) {
-    auto ownedJsFunction = bitcastFrom<JsFunction*>(new JsFunction(jsFunction));
-    return {TypeTags::jsFunction, ownedJsFunction};
-}
-
-std::pair<TypeTags, Value> makeCopyShardFilterer(const ShardFilterer& filterer) {
-    auto filtererCopy = bitcastFrom<ShardFilterer*>(filterer.clone().release());
-    return {TypeTags::shardFilterer, filtererCopy};
-}
-
-std::pair<TypeTags, Value> makeCopyFtsMatcher(const fts::FTSMatcher& matcher) {
-    auto copy = bitcastFrom<fts::FTSMatcher*>(new fts::FTSMatcher(matcher.query(), matcher.spec()));
-    return {TypeTags::ftsMatcher, copy};
-}
-
-std::pair<TypeTags, Value> makeCopySortSpec(const SortSpec& ss) {
-    auto ssCopy = bitcastFrom<SortSpec*>(new SortSpec(ss));
-    return {TypeTags::sortSpec, ssCopy};
-}
-
-std::pair<TypeTags, Value> makeCopyMakeObjSpec(const MakeObjSpec& mos) {
-    auto mosCopy = bitcastFrom<MakeObjSpec*>(new MakeObjSpec(mos));
-    return {TypeTags::makeObjSpec, mosCopy};
+std::pair<TypeTags, Value> makeCopyCellBlock(const CellBlock& b) {
+    auto cpy = b.clone();
+    return {TypeTags::cellBlock, bitcastFrom<CellBlock*>(cpy.release())};
 }
 
 std::pair<TypeTags, Value> makeCopyCollator(const CollatorInterface& collator) {
@@ -282,7 +216,6 @@ std::pair<TypeTags, Value> makeCopyIndexBounds(const IndexBounds& bounds) {
     return {TypeTags::indexBounds, boundsCopy};
 }
 
-
 void releaseValueDeep(TypeTags tag, Value val) noexcept {
     switch (tag) {
         case TypeTags::RecordId:
@@ -297,9 +230,15 @@ void releaseValueDeep(TypeTags tag, Value val) noexcept {
         case TypeTags::ArraySet:
             delete getArraySetView(val);
             break;
+        case TypeTags::ArrayMultiSet:
+            delete getArrayMultiSetView(val);
+            break;
         case TypeTags::Object:
             delete getObjectView(val);
             break;
+        case TypeTags::MultiMap:
+            delete getMultiMapView(val);
+            break;
         case TypeTags::ObjectId:
             delete getObjectIdView(val);
             break;
@@ -317,36 +256,31 @@ void releaseValueDeep(TypeTags tag, Value val) noexcept {
         case TypeTags::bsonObject:
             UniqueBuffer::reclaim(getRawPointerView(val));
             break;
-        case TypeTags::ksValue:
-            delete getKeyStringView(val);
-            break;
-        case TypeTags::pcreRegex:
-            delete getPcreRegexView(val);
-            break;
-        case TypeTags::jsFunction:
-            delete getJsFunctionView(val);
-            break;
-        case TypeTags::shardFilterer:
-            delete getShardFiltererView(val);
-            break;
-        case TypeTags::ftsMatcher:
-            delete getFtsMatcherView(val);
-            break;
-        case TypeTags::sortSpec:
-            delete getSortSpecView(val);
-            break;
-        case TypeTags::makeObjSpec:
-            delete getMakeObjSpecView(val);
+        case TypeTags::keyString:
+            delete getKeyString(val);
             break;
         case TypeTags::collator:
             delete getCollatorView(val);
             break;
-        case TypeTags::indexBounds:
-            delete getIndexBoundsView(val);
-            break;
         case TypeTags::timeZone:
             delete getTimeZoneView(val);
             break;
+        case TypeTags::valueBlock:
+            delete getValueBlock(val);
+            break;
+        case TypeTags::cellBlock:
+            delete getCellBlock(val);
+            break;
+        case TypeTags::pcreRegex:
+        case TypeTags::jsFunction:
+        case TypeTags::shardFilterer:
+        case TypeTags::ftsMatcher:
+        case TypeTags::sortSpec:
+        case TypeTags::makeObjSpec:
+        case TypeTags::indexBounds:
+        case TypeTags::inList:
+            getExtendedTypeOps(tag)->release(val);
+            break;
         default:
             break;
     }
@@ -372,14 +306,28 @@ str::stream& operator<<(str::stream& str, const std::pair<TypeTags, Value>& valu
     return str;
 }
 
+std::string print(const std::pair<TypeTags, Value>& value) {
+    str::stream stream = str::stream();
+    stream << value;
+    return stream;
+}
+
+std::string printTagAndVal(const TypeTags tag, const Value value) {
+    return printTagAndVal(std::pair<TypeTags, Value>{tag, value});
+}
+
+std::string printTagAndVal(const std::pair<TypeTags, Value>& value) {
+    str::stream stream = str::stream();
+    stream << "tag: " << value.first << ", val: " << value;
+    return stream;
+}
+
 BSONType tagToType(TypeTags tag) noexcept {
     switch (tag) {
         case TypeTags::Nothing:
             return BSONType::EOO;
         case TypeTags::NumberInt32:
             return BSONType::NumberInt;
-        case TypeTags::RecordId:
-            return BSONType::EOO;
         case TypeTags::NumberInt64:
             return BSONType::NumberLong;
         case TypeTags::NumberDouble:
@@ -402,6 +350,8 @@ BSONType tagToType(TypeTags tag) noexcept {
             return BSONType::Array;
         case TypeTags::ArraySet:
             return BSONType::Array;
+        case TypeTags::ArrayMultiSet:
+            return BSONType::Array;
         case TypeTags::Object:
             return BSONType::Object;
         case TypeTags::ObjectId:
@@ -424,9 +374,6 @@ BSONType tagToType(TypeTags tag) noexcept {
             return BSONType::BinData;
         case TypeTags::bsonUndefined:
             return BSONType::Undefined;
-        case TypeTags::ksValue:
-            // This is completely arbitrary.
-            return BSONType::EOO;
         case TypeTags::bsonRegex:
             return BSONType::RegEx;
         case TypeTags::bsonJavascript:
@@ -436,7 +383,7 @@ BSONType tagToType(TypeTags tag) noexcept {
         case TypeTags::bsonCodeWScope:
             return BSONType::CodeWScope;
         default:
-            MONGO_UNREACHABLE;
+            return BSONType::EOO;
     }
 }
 
@@ -449,7 +396,7 @@ inline std::size_t hashObjectId(const uint8_t* objId) noexcept {
 std::size_t hashValue(TypeTags tag, Value val, const CollatorInterface* collator) noexcept {
     switch (tag) {
         case TypeTags::NumberInt32:
-            return abslHash(bitcastTo<int32_t>(val));
+            return abslHash(static_cast<int64_t>(bitcastTo<int32_t>(val)));
         case TypeTags::RecordId:
             return getRecordIdView(val)->hash();
         case TypeTags::NumberInt64:
@@ -507,8 +454,8 @@ std::size_t hashValue(TypeTags tag, Value val, const CollatorInterface* collator
                 tag == TypeTags::ObjectId ? getObjectIdView(val)->data() : bitcastTo<uint8_t*>(val);
             return hashObjectId(objId);
         }
-        case TypeTags::ksValue:
-            return getKeyStringView(val)->hash();
+        case TypeTags::keyString:
+            return getKeyString(val)->hash();
         case TypeTags::Array:
         case TypeTags::bsonArray: {
             auto arr = ArrayEnumerator{tag, val};
@@ -523,8 +470,10 @@ std::size_t hashValue(TypeTags tag, Value val, const CollatorInterface* collator
 
             return res;
         }
-        case TypeTags::ArraySet: {
-            size_t size = getArraySetView(val)->size();
+        case TypeTags::ArraySet:
+        case TypeTags::ArrayMultiSet: {
+            size_t size = tag == TypeTags::ArraySet ? getArraySetView(val)->size()
+                                                    : getArrayMultiSetView(val)->size();
             std::vector<size_t> valueHashes;
             valueHashes.reserve(size);
             for (ArrayEnumerator arr{tag, val}; !arr.atEnd(); arr.advance()) {
@@ -550,6 +499,16 @@ std::size_t hashValue(TypeTags tag, Value val, const CollatorInterface* collator
 
             return res;
         }
+        case TypeTags::MultiMap: {
+            auto multiMap = getMultiMapView(val);
+            auto res = hashInit();
+
+            for (const auto& [key, value] : multiMap->values()) {
+                res = hashCombine(res, hashValue(key.first, key.second, collator));
+                res = hashCombine(res, hashValue(value.first, value.second, collator));
+            }
+            return res;
+        }
         case TypeTags::bsonBinData: {
             auto size = getBSONBinDataSize(tag, val);
             if (size < 8) {
@@ -598,20 +557,6 @@ std::size_t hashValue(TypeTags tag, Value val, const CollatorInterface* collator
     return 0;
 }
 
-/**
- * Performs a three-way comparison for any type that has < and == operators. Additionally,
- * guarantees that the result will be exactlty -1, 0, or 1, which is important, because not all
- * comparison functions make that guarantee.
- *
- * The StringData::compare(basic_string_view s) function, for example, only promises that it
- * will return a value less than 0 in the case that 'this' is less than 's,' whereas we want to
- * return exactly -1.
- */
-template <typename T>
-int32_t compareHelper(const T lhs, const T rhs) noexcept {
-    return lhs < rhs ? -1 : (lhs == rhs ? 0 : 1);
-}
-
 /*
  * Three ways value comparison (aka spacehip operator).
  */
@@ -619,7 +564,7 @@ std::pair<TypeTags, Value> compareValue(TypeTags lhsTag,
                                         Value lhsValue,
                                         TypeTags rhsTag,
                                         Value rhsValue,
-                                        const StringData::ComparatorInterface* comparator) {
+                                        const StringDataComparator* comparator) {
     if (isNumber(lhsTag) && isNumber(rhsTag)) {
         switch (getWidestNumericalType(lhsTag, rhsTag)) {
             case TypeTags::NumberInt32: {
@@ -690,10 +635,11 @@ std::pair<TypeTags, Value> compareValue(TypeTags lhsTag,
     } else if (lhsTag == TypeTags::bsonUndefined && rhsTag == TypeTags::bsonUndefined) {
         return {TypeTags::NumberInt32, bitcastFrom<int32_t>(0)};
     } else if (isArray(lhsTag) && isArray(rhsTag)) {
-        // ArraySets carry semantics of an unordered set, so we cannot define a deterministic
-        // less or greater operations on them, but only compare for equality. Comparing an
-        // ArraySet with a regular Array is equivalent of converting the ArraySet to an Array
-        // and them comparing the two Arrays, so we can simply use a generic algorithm below.
+        // ArraySets and ArrayMultiSet carry semantics of an unordered set, so we cannot define a
+        // deterministic less or greater operations on them, but only compare for equality.
+        // Comparing an ArraySet or ArrayMultiSet with a regular Array is equivalent of converting
+        // the ArraySet/ArrayMultiSet to an Array and them comparing the two Arrays, so we can
+        // simply use a generic algorithm below.
         if (lhsTag == TypeTags::ArraySet && rhsTag == TypeTags::ArraySet) {
             auto lhsArr = getArraySetView(lhsValue);
             auto rhsArr = getArraySetView(rhsValue);
@@ -703,6 +649,16 @@ std::pair<TypeTags, Value> compareValue(TypeTags lhsTag,
             return {TypeTags::Nothing, 0};
         }
 
+        if (lhsTag == TypeTags::ArrayMultiSet && rhsTag == TypeTags::ArrayMultiSet) {
+            auto lhsArr = getArrayMultiSetView(lhsValue);
+            auto rhsArr = getArrayMultiSetView(rhsValue);
+            if (*lhsArr == *rhsArr) {
+                return {TypeTags::NumberInt32, bitcastFrom<int32_t>(0)};
+            }
+            // If they are not equal then we cannot say if one is smaller than the other.
+            return {TypeTags::Nothing, 0};
+        }
+
         auto lhsArr = ArrayEnumerator{lhsTag, lhsValue};
         auto rhsArr = ArrayEnumerator{rhsTag, rhsValue};
         while (!lhsArr.atEnd() && !rhsArr.atEnd()) {
@@ -723,6 +679,14 @@ std::pair<TypeTags, Value> compareValue(TypeTags lhsTag,
         } else {
             return {TypeTags::NumberInt32, bitcastFrom<int32_t>(1)};
         }
+    } else if (lhsTag == TypeTags::MultiMap && rhsTag == TypeTags::MultiMap) {
+        auto lhsMap = getMultiMapView(lhsValue);
+        auto rhsMap = getMultiMapView(rhsValue);
+        if (*lhsMap == *rhsMap) {
+            return {TypeTags::NumberInt32, bitcastFrom<int32_t>(0)};
+        }
+        // If they are not equal then we cannot say if one is smaller than the other.
+        return {TypeTags::Nothing, 0};
     } else if (isObject(lhsTag) && isObject(rhsTag)) {
         auto lhsObj = ObjectEnumerator{lhsTag, lhsValue};
         auto rhsObj = ObjectEnumerator{rhsTag, rhsValue};
@@ -778,15 +742,15 @@ std::pair<TypeTags, Value> compareValue(TypeTags lhsTag,
                              getRawPointerView(rhsValue) + sizeof(uint32_t),
                              lsz + 1);
         return {TypeTags::NumberInt32, bitcastFrom<int32_t>(compareHelper(result, 0))};
-    } else if (lhsTag == TypeTags::ksValue && rhsTag == TypeTags::ksValue) {
-        auto result = getKeyStringView(lhsValue)->compare(*getKeyStringView(rhsValue));
+    } else if (lhsTag == TypeTags::keyString && rhsTag == TypeTags::keyString) {
+        auto result = getKeyString(lhsValue)->compare(*getKeyString(rhsValue));
         return {TypeTags::NumberInt32, bitcastFrom<int32_t>(result)};
     } else if (lhsTag == TypeTags::Nothing && rhsTag == TypeTags::Nothing) {
         // Special case for Nothing in a hash table (group) and sort comparison.
         return {TypeTags::NumberInt32, bitcastFrom<int32_t>(0)};
     } else if (lhsTag == TypeTags::RecordId && rhsTag == TypeTags::RecordId) {
         int32_t result = getRecordIdView(lhsValue)->compare(*getRecordIdView(rhsValue));
-        return {TypeTags::NumberInt32, bitcastFrom<int32_t>(result)};
+        return {TypeTags::NumberInt32, bitcastFrom<int32_t>(compareHelper(result, 0))};
     } else if (lhsTag == TypeTags::bsonRegex && rhsTag == TypeTags::bsonRegex) {
         auto lhsRegex = getBsonRegexView(lhsValue);
         auto rhsRegex = getBsonRegexView(rhsValue);
@@ -842,7 +806,7 @@ std::pair<TypeTags, Value> compareValue(TypeTags lhsTag,
         auto result = canonicalizeBSONType(lhsType) - canonicalizeBSONType(rhsType);
         return {TypeTags::NumberInt32, bitcastFrom<int32_t>(compareHelper(result, 0))};
     }
-}
+}  // compareValue
 
 bool isNaN(TypeTags tag, Value val) noexcept {
     return (tag == TypeTags::NumberDouble && std::isnan(bitcastTo<double>(val))) ||
@@ -873,7 +837,9 @@ std::pair<TypeTags, Value> ArrayEnumerator::getViewOfValue() const {
     if (_array) {
         return _array->getAt(_index);
     } else if (_arraySet) {
-        return {_iter->first, _iter->second};
+        return {_arraySetIter->first, _arraySetIter->second};
+    } else if (_arrayMultiSet) {
+        return {_arrayMultiSetIter->first, _arrayMultiSetIter->second};
     } else {
         return bson::convertFrom<true>(_arrayCurrent, _arrayEnd, _fieldNameSize);
     }
@@ -887,11 +853,17 @@ bool ArrayEnumerator::advance() {
 
         return _index < _array->size();
     } else if (_arraySet) {
-        if (_iter != _arraySet->values().end()) {
-            ++_iter;
+        if (_arraySetIter != _arraySet->values().end()) {
+            ++_arraySetIter;
         }
 
-        return _iter != _arraySet->values().end();
+        return _arraySetIter != _arraySet->values().end();
+    } else if (_arrayMultiSet) {
+        if (_arrayMultiSetIter != _arrayMultiSet->values().end()) {
+            ++_arrayMultiSetIter;
+        }
+
+        return _arrayMultiSetIter != _arrayMultiSet->values().end();
     } else {
         if (_arrayCurrent != _arrayEnd - 1) {
             _arrayCurrent = bson::advance(_arrayCurrent, _fieldNameSize);
@@ -947,7 +919,7 @@ StringData ObjectEnumerator::getFieldName() const {
     }
 }
 
-void readKeyStringValueIntoAccessors(const KeyString::Value& keyString,
+void readKeyStringValueIntoAccessors(const SortedDataKeyValueView& keyString,
                                      const Ordering& ordering,
                                      BufBuilder* valueBufferBuilder,
                                      std::vector<OwnedValueAccessor>* accessors,
@@ -955,9 +927,13 @@ void readKeyStringValueIntoAccessors(const KeyString::Value& keyString,
     OwnedValueAccessorValueBuilder valBuilder(valueBufferBuilder);
     invariant(!indexKeysToInclude || indexKeysToInclude->count() == accessors->size());
 
-    BufReader reader(keyString.getBuffer(), keyString.getSize());
-    KeyString::TypeBits typeBits(keyString.getTypeBits());
-    KeyString::TypeBits::Reader typeBitsReader(typeBits);
+    auto ks = keyString.getKeyStringWithoutRecordIdView();
+    BufReader reader(ks.data(), ks.size());
+
+    auto typeBits = keyString.getTypeBitsView();
+    BufReader typeBitsBr(typeBits.data(), typeBits.size());
+    auto typeBitsReader =
+        key_string::TypeBits::getReaderFromBuffer(keyString.getVersion(), &typeBitsBr);
 
     bool keepReading = true;
     size_t componentIndex = 0;
@@ -969,15 +945,15 @@ void readKeyStringValueIntoAccessors(const KeyString::Value& keyString,
             ? (ordering.get(componentIndex) == -1)
             : false;
 
-        keepReading = KeyString::readSBEValue(
-            &reader, &typeBitsReader, inverted, typeBits.version, &valBuilder);
+        keepReading = key_string::readSBEValue(
+            &reader, &typeBitsReader, inverted, keyString.getVersion(), &valBuilder);
 
         invariant(componentIndex < Ordering::kMaxCompoundIndexKeys || !keepReading);
 
         // If 'indexKeysToInclude' indicates that this index key component is not part of the
         // projection, remove it from the list of values that will be fed to the 'accessors'
         // list. Note that, even when we are excluding a key component, we can't skip the call
-        // to 'KeyString::readSBEValue()' because it is needed to advance the 'reader' and
+        // to 'key_string::readSBEValue()' because it is needed to advance the 'reader' and
         // 'typeBitsReader' stream.
         if (indexKeysToInclude && (componentIndex < Ordering::kMaxCompoundIndexKeys) &&
             !(*indexKeysToInclude)[componentIndex]) {
@@ -1024,6 +1000,66 @@ bool operator==(const ArraySet& lhs, const ArraySet& rhs) {
 bool operator!=(const ArraySet& lhs, const ArraySet& rhs) {
     return !(lhs == rhs);
 }
+
+bool operator==(const MultiMap& lhs, const MultiMap& rhs) {
+    return lhs.values() == rhs.values();
+}
+
+bool operator!=(const MultiMap& lhs, const MultiMap& rhs) {
+    return !(lhs == rhs);
+}
+
+std::pair<TypeTags, Value> genericEq(TypeTags lhsTag,
+                                     Value lhsVal,
+                                     TypeTags rhsTag,
+                                     Value rhsVal,
+                                     const StringDataComparator* comparator) {
+    return genericCompare<std::equal_to<>>(lhsTag, lhsVal, rhsTag, rhsVal, comparator);
+}
+
+std::pair<TypeTags, Value> genericNeq(TypeTags lhsTag,
+                                      Value lhsVal,
+                                      TypeTags rhsTag,
+                                      Value rhsVal,
+                                      const StringDataComparator* comparator) {
+    auto [tag, val] = genericEq(lhsTag, lhsVal, rhsTag, rhsVal, comparator);
+    // genericEq() will return either Boolean or Nothing. If it returns Boolean, negate
+    // 'val' before returning it.
+    val = tag == TypeTags::Boolean ? bitcastFrom<bool>(!bitcastTo<bool>(val)) : val;
+    return {tag, val};
+}
+
+std::pair<TypeTags, Value> genericLt(TypeTags lhsTag,
+                                     Value lhsVal,
+                                     TypeTags rhsTag,
+                                     Value rhsVal,
+                                     const StringDataComparator* comparator) {
+    return genericCompare<std::less<>>(lhsTag, lhsVal, rhsTag, rhsVal, comparator);
+}
+
+std::pair<TypeTags, Value> genericLte(TypeTags lhsTag,
+                                      Value lhsVal,
+                                      TypeTags rhsTag,
+                                      Value rhsVal,
+                                      const StringDataComparator* comparator) {
+    return genericCompare<std::less_equal<>>(lhsTag, lhsVal, rhsTag, rhsVal, comparator);
+}
+
+std::pair<TypeTags, Value> genericGt(TypeTags lhsTag,
+                                     Value lhsVal,
+                                     TypeTags rhsTag,
+                                     Value rhsVal,
+                                     const StringDataComparator* comparator) {
+    return genericCompare<std::greater<>>(lhsTag, lhsVal, rhsTag, rhsVal, comparator);
+}
+
+std::pair<TypeTags, Value> genericGte(TypeTags lhsTag,
+                                      Value lhsVal,
+                                      TypeTags rhsTag,
+                                      Value rhsVal,
+                                      const StringDataComparator* comparator) {
+    return genericCompare<std::greater_equal<>>(lhsTag, lhsVal, rhsTag, rhsVal, comparator);
+}
 }  // namespace value
 }  // namespace sbe
 }  // namespace mongo
