@@ -27,12 +27,33 @@
  *    it in the license file.
  */
 #include <algorithm>
+#include <cstdint>
+#include <limits>
+#include <string>
+#include <utility>
+#include <vector>
 
+#include <fmt/format.h>
+
+#include "mongo/bson/bsontypes.h"
+#include "mongo/bson/oid.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/db/basic_types_gen.h"
 #include "mongo/db/exec/sbe/makeobj_spec.h"
-#include "mongo/db/exec/sbe/values/sort_spec.h"
+#include "mongo/db/exec/sbe/sort_spec.h"
+#include "mongo/db/exec/sbe/values/block_interface.h"
 #include "mongo/db/exec/sbe/values/value.h"
 #include "mongo/db/exec/sbe/values/value_printer.h"
-#include "mongo/platform/basic.h"
+#include "mongo/db/fts/fts_matcher.h"
+#include "mongo/db/fts/fts_query_impl.h"
+#include "mongo/db/query/datetime/date_time_support.h"
+#include "mongo/db/query/index_bounds.h"
+#include "mongo/db/record_id.h"
+#include "mongo/db/storage/key_string.h"
+#include "mongo/platform/decimal128.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/hex.h"
+#include "mongo/util/pcre.h"
 #include "mongo/util/pcre_util.h"
 
 namespace mongo::sbe::value {
@@ -82,9 +103,15 @@ void ValuePrinter<T>::writeTagToStream(TypeTags tag) {
         case TypeTags::ArraySet:
             stream << "ArraySet";
             break;
+        case TypeTags::ArrayMultiSet:
+            stream << "ArrayMultiSet";
+            break;
         case TypeTags::Object:
             stream << "Object";
             break;
+        case TypeTags::MultiMap:
+            stream << "MultiMap";
+            break;
         case TypeTags::ObjectId:
             stream << "ObjectId";
             break;
@@ -118,7 +145,7 @@ void ValuePrinter<T>::writeTagToStream(TypeTags tag) {
         case TypeTags::bsonUndefined:
             stream << "bsonUndefined";
             break;
-        case TypeTags::ksValue:
+        case TypeTags::keyString:
             stream << "KeyString";
             break;
         case TypeTags::pcreRegex:
@@ -163,6 +190,9 @@ void ValuePrinter<T>::writeTagToStream(TypeTags tag) {
         case TypeTags::indexBounds:
             stream << "indexBounds";
             break;
+        case TypeTags::inList:
+            stream << "inList";
+            break;
         case TypeTags::csiCell:
             stream << "csiCell";
             break;
@@ -172,6 +202,12 @@ void ValuePrinter<T>::writeTagToStream(TypeTags tag) {
         case TypeTags::timeZone:
             stream << "TimeZone";
             break;
+        case TypeTags::valueBlock:
+            stream << "ValueBlock";
+            break;
+        case TypeTags::cellBlock:
+            stream << "CellBlock";
+            break;
         default:
             stream << "unknown tag";
             break;
@@ -301,6 +337,51 @@ void ValuePrinter<T>::writeObjectToStream(const BSONObj& obj) {
     writeObjectToStream(TypeTags::bsonObject, bitcastFrom<const char*>(obj.objdata()));
 }
 
+template <typename T>
+void ValuePrinter<T>::writeMultiMapToStream(TypeTags tag, Value val, size_t depth) {
+    stream << '[';
+
+    auto multiMap = getMultiMapView(val);
+
+    auto first = true;
+    size_t iter = 0;
+    for (const auto& [key, value] : multiMap->values()) {
+        if (first) {
+            first = false;
+        } else {
+            stream << ", ";
+        }
+
+        if (depth > options.arrayObjectOrNestingMaxDepth()) {
+            stream << "...";
+            break;
+        }
+
+        auto keyDepth = depth;
+        auto valueDepth = depth;
+
+        stream << "{k : ";
+        if (value::isArray(key.first) || value::isObject(key.first)) {
+            keyDepth++;
+            depth = keyDepth;
+        }
+        writeValueToStream(key.first, key.second, keyDepth);
+        iter++;
+
+        stream << ", v : ";
+
+        if (value::isArray(value.first) || value::isObject(value.first)) {
+            valueDepth++;
+            depth = valueDepth;
+        }
+        writeValueToStream(value.first, value.second, valueDepth);
+        iter++;
+
+        stream << "}";
+    }
+    stream << ']';
+}
+
 template <typename T>
 void ValuePrinter<T>::writeObjectIdToStream(TypeTags tag, Value val) {
     auto objId =
@@ -403,6 +484,7 @@ void ValuePrinter<T>::writeValueToStream(TypeTags tag, Value val, size_t depth)
             writeArrayToStream(tag, val, depth);
             break;
         case TypeTags::ArraySet:
+        case TypeTags::ArrayMultiSet:
             if (options.normalizeOutput()) {
                 writeSortedArraySetToStream(tag, val, depth);
             } else {
@@ -413,6 +495,9 @@ void ValuePrinter<T>::writeValueToStream(TypeTags tag, Value val, size_t depth)
         case TypeTags::bsonObject:
             writeObjectToStream(tag, val, depth);
             break;
+        case TypeTags::MultiMap:
+            writeMultiMapToStream(tag, val, depth);
+            break;
         case TypeTags::ObjectId:
         case TypeTags::bsonObjectId:
             writeObjectIdToStream(tag, val);
@@ -456,9 +541,9 @@ void ValuePrinter<T>::writeValueToStream(TypeTags tag, Value val, size_t depth)
         case TypeTags::LocalLambda:
             stream << "LocalLambda";
             break;
-        case TypeTags::ksValue: {
-            auto ks = getKeyStringView(val);
-            stream << "KS(" << ks->toString() << ")";
+        case TypeTags::keyString: {
+            auto ks = getKeyString(val);
+            stream << "KS(" << hexblob::encode(ks->getKeyStringView()) << ")";
             break;
         }
         case TypeTags::Timestamp: {
@@ -466,12 +551,6 @@ void ValuePrinter<T>::writeValueToStream(TypeTags tag, Value val, size_t depth)
             stream << ts.toString();
             break;
         }
-        case TypeTags::pcreRegex: {
-            auto regex = getPcreRegexView(val);
-            stream << "PcreRegex(/" << regex->pattern() << "/"
-                   << pcre_util::optionsToFlags(regex->options()) << ")";
-            break;
-        }
         case TypeTags::timeZoneDB: {
             auto tzdb = getTimeZoneDBView(val);
             auto timeZones = tzdb->getTimeZoneStrings();
@@ -481,13 +560,6 @@ void ValuePrinter<T>::writeValueToStream(TypeTags tag, Value val, size_t depth)
         case TypeTags::RecordId:
             stream << "RecordId(" << getRecordIdView(val)->toString() << ")";
             break;
-        case TypeTags::jsFunction:
-            // TODO: Also include code.
-            stream << "jsFunction";
-            break;
-        case TypeTags::shardFilterer:
-            stream << "ShardFilterer";
-            break;
         case TypeTags::collator:
             writeCollatorToStream(getCollatorView(val));
             break;
@@ -516,32 +588,6 @@ void ValuePrinter<T>::writeValueToStream(TypeTags tag, Value val, size_t depth)
             stream << ')';
             break;
         }
-        case TypeTags::ftsMatcher: {
-            auto ftsMatcher = getFtsMatcherView(val);
-            stream << "FtsMatcher(";
-            writeObjectToStream(ftsMatcher->query().toBSON());
-            stream << ')';
-            break;
-        }
-        case TypeTags::sortSpec:
-            stream << "SortSpec(";
-            writeObjectToStream(getSortSpecView(val)->getPattern());
-            stream << ')';
-            break;
-        case TypeTags::makeObjSpec:
-            stream << "MakeObjSpec(" << getMakeObjSpecView(val)->toString() << ")";
-            break;
-        case TypeTags::indexBounds:
-            // When calling toString() we don't know if the index has a non-simple collation or
-            // not. Passing false could produce invalid UTF-8, which is not acceptable when we are
-            // going to put the resulting string into a BSON object and return it across the wire.
-            // While passing true may be misleading in cases when the index has no collation, it is
-            // safer to do so.
-            stream << "IndexBounds(";
-            writeStringDataToStream(
-                getIndexBoundsView(val)->toString(true /* hasNonSimpleCollation */));
-            stream << ")";
-            break;
         case TypeTags::csiCell:
             stream << "CsiCell(" << getCsiCellView(val) << ")";
             break;
@@ -555,6 +601,27 @@ void ValuePrinter<T>::writeValueToStream(TypeTags tag, Value val, size_t depth)
         case TypeTags::timeZone:
             stream << getTimeZoneView(val)->toString();
             break;
+        case TypeTags::valueBlock: {
+            auto* valueBlock = getValueBlock(val);
+            stream << valueBlock->extract();
+            break;
+        }
+        case TypeTags::cellBlock: {
+            auto* cellBlock = getCellBlock(val);
+            stream << "CellBlock: " << cellBlock->getValueBlock().extract();
+            break;
+        }
+
+        case TypeTags::pcreRegex:
+        case TypeTags::jsFunction:
+        case TypeTags::shardFilterer:
+        case TypeTags::ftsMatcher:
+        case TypeTags::sortSpec:
+        case TypeTags::makeObjSpec:
+        case TypeTags::indexBounds:
+        case TypeTags::inList:
+            stream << getExtendedTypeOps(tag)->print(val);
+            break;
         default:
             MONGO_UNREACHABLE;
     }
