@@ -26,12 +26,26 @@
  *    exception statement from all source files in the program, then also delete
  *    it in the license file.
  */
+#include <algorithm>
+#include <cstddef>
+#include <cstdint>
 #include <iomanip>
+#include <memory>
+#include <string>
+#include <utility>
+#include <vector>
 
+#include <absl/container/flat_hash_map.h>
+#include <absl/container/inlined_vector.h>
+
+#include "mongo/base/string_data.h"
+#include "mongo/db/exec/sbe/util/print_options.h"
+#include "mongo/db/exec/sbe/values/slot.h"
+#include "mongo/db/exec/sbe/values/value.h"
 #include "mongo/db/exec/sbe/values/value_printer.h"
 #include "mongo/db/exec/sbe/vm/vm.h"
 #include "mongo/db/exec/sbe/vm/vm_printer.h"
-#include "mongo/platform/basic.h"
+#include "mongo/db/query/datetime/date_time_support.h"
 
 namespace mongo::sbe::vm {
 
@@ -170,13 +184,18 @@ public:
                 case Instruction::collEq:
                 case Instruction::collNeq:
                 case Instruction::collCmp3w: {
-                    auto [popColl, offsetColl] = Instruction::Parameter::decodeParam(pcPointer);
-                    auto [popLhs, offsetLhs] = Instruction::Parameter::decodeParam(pcPointer);
-                    auto [popRhs, offsetRhs] = Instruction::Parameter::decodeParam(pcPointer);
+                    auto [popColl, moveFromColl, offsetColl] =
+                        Instruction::Parameter::decodeParam(pcPointer);
+                    auto [popLhs, moveFromLhs, offsetLhs] =
+                        Instruction::Parameter::decodeParam(pcPointer);
+                    auto [popRhs, moveFromRhs, offsetRhs] =
+                        Instruction::Parameter::decodeParam(pcPointer);
 
-                    os << "popLhs: " << popLhs << ", offsetLhs: " << offsetLhs
-                       << ", popRhs: " << popRhs << ", offsetRhs: " << offsetRhs
-                       << ", popColl: " << popColl << ", offsetColl: " << offsetColl;
+                    os << "popLhs: " << popLhs << ", moveFromLhs: " << moveFromLhs
+                       << ", offsetLhs: " << offsetLhs << ", popRhs: " << popRhs
+                       << ", moveFromRhs: " << moveFromRhs << ", offsetRhs: " << offsetRhs
+                       << ", popColl: " << popColl << ", moveFromColl: " << moveFromColl
+                       << ", offsetColl: " << offsetColl;
                     break;
                 }
                 // Instructions with 2 arguments.
@@ -197,14 +216,18 @@ public:
                 case Instruction::getElement:
                 case Instruction::collComparisonKey:
                 case Instruction::getFieldOrElement: {
-                    auto [popLhs, offsetLhs] = Instruction::Parameter::decodeParam(pcPointer);
-                    auto [popRhs, offsetRhs] = Instruction::Parameter::decodeParam(pcPointer);
+                    auto [popLhs, moveFromLhs, offsetLhs] =
+                        Instruction::Parameter::decodeParam(pcPointer);
+                    auto [popRhs, moveFromRhs, offsetRhs] =
+                        Instruction::Parameter::decodeParam(pcPointer);
 
-                    os << "popLhs: " << popLhs << ", offsetLhs: " << offsetLhs
-                       << ", popRhs: " << popRhs << ", offsetRhs: " << offsetRhs;
+                    os << "popLhs: " << popLhs << ", moveFromLhs: " << moveFromLhs
+                       << ", offsetLhs: " << offsetLhs << ", popRhs: " << popRhs
+                       << ", moveFromRhs: " << moveFromRhs << ", offsetRhs: " << offsetRhs;
                     break;
                 }
                 // Instructions with 1 argument.
+                case Instruction::makeOwn:
                 case Instruction::negate:
                 case Instruction::logicNot:
                 case Instruction::getArraySize:
@@ -212,6 +235,7 @@ public:
                 case Instruction::isNull:
                 case Instruction::isObject:
                 case Instruction::isArray:
+                case Instruction::isInList:
                 case Instruction::isString:
                 case Instruction::isNumber:
                 case Instruction::isBinData:
@@ -222,8 +246,10 @@ public:
                 case Instruction::isMinKey:
                 case Instruction::isMaxKey:
                 case Instruction::isTimestamp: {
-                    auto [popParam, offsetParam] = Instruction::Parameter::decodeParam(pcPointer);
-                    os << "popParam: " << popParam << ", offsetParam: " << offsetParam;
+                    auto [popParam, moveFromParam, offsetParam] =
+                        Instruction::Parameter::decodeParam(pcPointer);
+                    os << "popParam: " << popParam << ", moveFromParam: " << moveFromParam
+                       << ", offsetParam: " << offsetParam;
                     break;
                 }
                 // Instructions with a single integer argument.
@@ -275,14 +301,15 @@ public:
                     break;
                 }
                 case Instruction::getFieldImm: {
-                    auto [popParam, offsetParam] = Instruction::Parameter::decodeParam(pcPointer);
+                    auto [popParam, moveFromParam, offsetParam] =
+                        Instruction::Parameter::decodeParam(pcPointer);
                     auto size = readFromMemory<uint8_t>(pcPointer);
                     pcPointer += sizeof(size);
                     StringData fieldName(reinterpret_cast<const char*>(pcPointer), size);
                     pcPointer += size;
 
-                    os << "popParam: " << popParam << ", offsetParam: " << offsetParam
-                       << ", value: \"" << fieldName << "\"";
+                    os << "popParam: " << popParam << ", moveFromParam: " << moveFromParam
+                       << ", offsetParam: " << offsetParam << ", value: \"" << fieldName << "\"";
                     break;
                 }
                 case Instruction::pushConstVal: {
@@ -312,25 +339,29 @@ public:
                     break;
                 }
                 case Instruction::typeMatchImm: {
-                    auto [popParam, offsetParam] = Instruction::Parameter::decodeParam(pcPointer);
+                    auto [popParam, moveFromParam, offsetParam] =
+                        Instruction::Parameter::decodeParam(pcPointer);
                     auto mask = readFromMemory<uint32_t>(pcPointer);
                     pcPointer += sizeof(mask);
-                    os << "popParam: " << popParam << ", offsetParam: " << offsetParam
-                       << ", mask: " << mask;
+                    os << "popParam: " << popParam << ", moveFromParam: " << moveFromParam
+                       << ", offsetParam: " << offsetParam << ", mask: " << mask;
                     break;
                 }
-                case Instruction::function:
                 case Instruction::functionSmall: {
+                    auto f = readFromMemory<SmallBuiltinType>(pcPointer);
+                    pcPointer += sizeof(f);
+                    ArityType arity{0};
+                    arity = readFromMemory<SmallArityType>(pcPointer);
+                    pcPointer += sizeof(SmallArityType);
+                    os << "f: " << builtinToString(static_cast<Builtin>(f)) << ", arity: " << arity;
+                    break;
+                }
+                case Instruction::function: {
                     auto f = readFromMemory<Builtin>(pcPointer);
                     pcPointer += sizeof(f);
                     ArityType arity{0};
-                    if (i.tag == Instruction::function) {
-                        arity = readFromMemory<ArityType>(pcPointer);
-                        pcPointer += sizeof(ArityType);
-                    } else {
-                        arity = readFromMemory<SmallArityType>(pcPointer);
-                        pcPointer += sizeof(SmallArityType);
-                    }
+                    arity = readFromMemory<ArityType>(pcPointer);
+                    pcPointer += sizeof(ArityType);
                     os << "f: " << builtinToString(f) << ", arity: " << arity;
                     break;
                 }
