@@ -29,13 +29,21 @@
 
 #pragma once
 
+#include <cstdint>
+#include <memory>
+#include <string>
+#include <utility>
+
+#include "mongo/bson/bsonobj.h"
 #include "mongo/db/exec/document_value/document.h"
+#include "mongo/db/exec/document_value/value.h"
 #include "mongo/db/exec/plan_stats.h"
 #include "mongo/db/exec/sort_key_comparator.h"
 #include "mongo/db/exec/working_set.h"
 #include "mongo/db/pipeline/expression.h"
 #include "mongo/db/query/sort_pattern.h"
 #include "mongo/db/sorter/sorter.h"
+#include "mongo/db/sorter/sorter_stats.h"
 
 namespace mongo {
 /**
@@ -55,9 +63,8 @@ public:
     class Comparator {
     public:
         Comparator(const SortPattern& sortPattern) : _sortKeyComparator(sortPattern) {}
-        int operator()(const typename DocumentSorter::Data& lhs,
-                       const typename DocumentSorter::Data& rhs) const {
-            return _sortKeyComparator(lhs.first, rhs.first);
+        int operator()(const Value& lhs, const Value& rhs) const {
+            return _sortKeyComparator(lhs, rhs);
         }
 
     private:
@@ -71,14 +78,19 @@ public:
                  uint64_t limit,
                  uint64_t maxMemoryUsageBytes,
                  std::string tempDir,
-                 bool allowDiskUse)
+                 bool allowDiskUse,
+                 bool moveSortedDataIntoIterator = false)
         : _sortPattern(std::move(sortPattern)),
           _tempDir(std::move(tempDir)),
-          _diskUseAllowed(allowDiskUse) {
+          _diskUseAllowed(allowDiskUse),
+          _moveSortedDataIntoIterator(moveSortedDataIntoIterator) {
         _stats.sortPattern =
             _sortPattern.serialize(SortPattern::SortKeySerialization::kForExplain).toBson();
         _stats.limit = limit;
         _stats.maxMemoryUsageBytes = maxMemoryUsageBytes;
+        if (allowDiskUse) {
+            _sorterFileStats = std::make_unique<SorterFileStats>(nullptr);
+        }
     }
 
     const SortPattern& sortPattern() const {
@@ -115,17 +127,32 @@ public:
     }
 
     const SortStats& stats() const {
+        if (_sorter) {
+            _stats.memoryUsageBytes = _sorter->stats().memUsage();
+        }
         return _stats;
     }
 
+    SorterFileStats* getSorterFileStats() const {
+        if (!_sorterFileStats) {
+            return nullptr;
+        }
+        return _sorterFileStats.get();
+    }
+
+    long long spilledDataStorageSize() const {
+        if (!_sorterFileStats) {
+            return 0;
+        }
+        return _sorterFileStats->bytesSpilled();
+    }
+
     /**
      * Add data item to be sorted of type T with sort key specified by Value to the sort executor.
      * Should only be called before 'loadingDone()' is called.
      */
     void add(const Value& sortKey, const T& data) {
-        if (!_sorter) {
-            _sorter.reset(DocumentSorter::make(makeSortOptions(), Comparator(_sortPattern)));
-        }
+        ensureSorter();
         _sorter->add(sortKey, data);
     }
 
@@ -133,14 +160,13 @@ public:
      * Signals to the sort executor that there will be no more input documents.
      */
     void loadingDone() {
-        // This conditional should only pass if no documents were added to the sorter.
-        if (!_sorter) {
-            _sorter.reset(DocumentSorter::make(makeSortOptions(), Comparator(_sortPattern)));
-        }
+        ensureSorter();
         _output.reset(_sorter->done());
-        _stats.keysSorted += _sorter->numSorted();
+        _stats.keysSorted += _sorter->stats().numSorted();
         _stats.spills += _sorter->stats().spilledRanges();
-        _stats.totalDataSizeBytes += _sorter->totalDataSizeSorted();
+        _stats.totalDataSizeBytes += _sorter->stats().bytesSorted();
+        _stats.spilledDataStorageSize += spilledDataStorageSize();
+        _stats.memoryUsageBytes = 0;
         _sorter.reset();
     }
 
@@ -175,9 +201,35 @@ public:
         return _stats.maxMemoryUsageBytes;
     }
 
+    /**
+     * Pauses Loading and creates an iterator which can be used to get the current state in
+     * read-only mode. The stream code needs this to pause and get the current internal state which
+     * can be used to store it to a persistent storage which will constitute a checkpoint for
+     * streaming processing.
+     */
+    void pauseLoading() {
+        invariant(!_paused);
+        _paused = true;
+        ensureSorter();
+        _output.reset(_sorter->pause());
+    }
+
+    /**
+     * Resumes Loading. This will remove the iterator created in pauseLoading().
+     */
+    void resumeLoading() {
+        invariant(_paused);
+        _paused = false;
+        ensureSorter();
+        _output.reset();
+        _sorter->resume();
+        _isEOF = false;
+    }
+
 private:
     SortOptions makeSortOptions() const {
         SortOptions opts;
+        opts.moveSortedDataIntoIterator = _moveSortedDataIntoIterator;
         if (_stats.limit) {
             opts.limit = _stats.limit;
         }
@@ -186,20 +238,31 @@ private:
         if (_diskUseAllowed) {
             opts.extSortAllowed = true;
             opts.tempDir = _tempDir;
+            opts.sorterFileStats = _sorterFileStats.get();
         }
 
         return opts;
     }
 
+    void ensureSorter() {
+        // This conditional should only pass if no documents were added to the sorter.
+        if (!_sorter) {
+            _sorter.reset(DocumentSorter::make(makeSortOptions(), Comparator(_sortPattern)));
+        }
+    }
+
     const SortPattern _sortPattern;
     const std::string _tempDir;
     const bool _diskUseAllowed;
+    const bool _moveSortedDataIntoIterator;
 
+    std::unique_ptr<SorterFileStats> _sorterFileStats;
     std::unique_ptr<DocumentSorter> _sorter;
     std::unique_ptr<typename DocumentSorter::Iterator> _output;
 
-    SortStats _stats;
+    mutable SortStats _stats;
 
     bool _isEOF = false;
+    bool _paused = false;
 };
 }  // namespace mongo
