@@ -29,19 +29,30 @@
 
 #include "mongo/db/exec/write_stage_common.h"
 
-#include "mongo/base/shim.h"
+
 #include "mongo/db/catalog/collection.h"
+#include "mongo/db/cluster_role.h"
+#include "mongo/db/database_name.h"
 #include "mongo/db/exec/shard_filterer_impl.h"
 #include "mongo/db/exec/working_set.h"
 #include "mongo/db/exec/working_set_common.h"
+#include "mongo/db/keypattern.h"
+#include "mongo/db/matcher/expression.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/query/canonical_query.h"
 #include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/s/collection_sharding_state.h"
 #include "mongo/db/s/operation_sharding_state.h"
-#include "mongo/db/transaction/transaction_participant.h"
+#include "mongo/db/server_options.h"
+#include "mongo/db/storage/record_store.h"
+#include "mongo/db/storage/recovery_unit.h"
+#include "mongo/db/storage/snapshot.h"
+#include "mongo/db/transaction_resources.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
 #include "mongo/logv2/redaction.h"
+#include "mongo/util/str.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kWrite
 
@@ -59,8 +70,8 @@ PreWriteFilter::PreWriteFilter(OperationContext* opCtx, NamespaceString nss)
 
           // Only the primary node of a shard that is a replica set should run this filter.
           const auto replCoord = repl::ReplicationCoordinator::get(opCtx);
-          return !replCoord->getSettings().usingReplSets() ||
-              !replCoord->canAcceptWritesForDatabase(opCtx, DatabaseName::kAdmin.toString());
+          return !replCoord->getSettings().isReplSet() ||
+              !replCoord->canAcceptWritesForDatabase(opCtx, DatabaseName::kAdmin);
       }()) {}
 
 PreWriteFilter::Action PreWriteFilter::computeAction(const Document& doc) {
@@ -117,7 +128,7 @@ void PreWriteFilter::logSkippingDocument(const Document& doc,
                 "stream event",
                 "op"_attr = opKind,
                 logAttrs(collNs),
-                "record"_attr = redact(doc.toString()));
+                "record"_attr = doc);
 }
 
 void PreWriteFilter::logFromMigrate(const Document& doc,
@@ -129,22 +140,7 @@ void PreWriteFilter::logFromMigrate(const Document& doc,
                 "prevent a wrong change stream event",
                 "op"_attr = opKind,
                 logAttrs(collNs),
-                "record"_attr = redact(doc.toString()));
-}
-
-void CachedShardingDescription::restoreState() {
-    _collectionDescription.reset();
-}
-
-const ScopedCollectionDescription& CachedShardingDescription::getCollectionDescription(
-    OperationContext* opCtx) {
-    if (!_collectionDescription) {
-        const auto scopedCss =
-            CollectionShardingState::assertCollectionLockedAndAcquire(opCtx, _nss);
-        _collectionDescription = scopedCss->getCollectionDescription(opCtx);
-    }
-
-    return *_collectionDescription;
+                "record"_attr = doc);
 }
 
 bool ensureStillMatches(const CollectionPtr& collection,
@@ -155,7 +151,7 @@ bool ensureStillMatches(const CollectionPtr& collection,
     // If the snapshot changed, then we have to make sure we have the latest copy of the doc and
     // that it still matches.
     WorkingSetMember* member = ws->get(id);
-    if (opCtx->recoveryUnit()->getSnapshotId() != member->doc.snapshotId()) {
+    if (shard_role_details::getRecoveryUnit(opCtx)->getSnapshotId() != member->doc.snapshotId()) {
         std::unique_ptr<SeekableRecordCursor> cursor(collection->getCursor(opCtx));
 
         if (!WorkingSetCommon::fetch(opCtx, ws, id, cursor.get(), collection, collection->ns())) {
@@ -164,7 +160,8 @@ bool ensureStillMatches(const CollectionPtr& collection,
         }
 
         // Make sure the re-fetched doc still matches the predicate.
-        if (cq && !cq->root()->matchesBSON(member->doc.value().toBson(), nullptr)) {
+        if (cq &&
+            !cq->getPrimaryMatchExpression()->matchesBSON(member->doc.value().toBson(), nullptr)) {
             // No longer matches.
             return false;
         }
