@@ -27,14 +27,21 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <memory>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/base/string_data.h"
 #include "mongo/bson/bsonmisc.h"
 #include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/bson/json.h"
 #include "mongo/db/explain_gen.h"
 #include "mongo/db/query/explain_options.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/db/query/explain_verbosity_gen.h"
+#include "mongo/idl/idl_parser.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/bson_test_util.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/assert_util.h"
 
 namespace mongo {
 namespace {
@@ -102,7 +109,7 @@ TEST(ExplainTest, ParsingFailsIfFirstElementIsNotAnObject) {
         ExplainCommandRequest::parse(IDLParserContext("explain"),
                                      fromjson("{explain: 1, verbosity: 'queryPlanner'}")),
         DBException,
-        40414);
+        ErrorCodes::IDLFailedToParse);
 }
 
 TEST(ExplainTest, ParsingFailsIfUnknownFieldInCommandObject) {
@@ -111,7 +118,7 @@ TEST(ExplainTest, ParsingFailsIfUnknownFieldInCommandObject) {
             IDLParserContext("explain"),
             fromjson("{explain: {}, verbosity: 'queryPlanner', unknownField: true}")),
         DBException,
-        40415);
+        ErrorCodes::IDLUnknownField);
 }
 
 TEST(ExplainTest, CanParseGenericCommandArguments) {
