@@ -31,6 +31,11 @@
 
 #include <boost/container/small_vector.hpp>
 #include <boost/optional.hpp>
+// IWYU pragma: no_include "boost/intrusive/detail/iterator.hpp"
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <cstddef>
+#include <cstdint>
 #include <iosfwd>
 #include <set>
 #include <string>
@@ -38,7 +43,6 @@
 
 #include "mongo/base/string_data.h"
 #include "mongo/bson/bson_depth.h"
-#include "mongo/util/assert_util.h"
 #include "mongo/util/container_size_helper.h"
 
 namespace mongo {
@@ -46,9 +50,7 @@ namespace mongo {
 /**
  * A FieldPath represents a path in a document, starting from the root. The path
  * is made of "field parts" separated by dots. The class provides an efficient means to
- * "split" the dotted fields in its parts. It does not do UTF-8 validation but it does throw a
- * uassert() upon an attempt to construct a FieldRef in which any field path component contains an
- * embedded null byte.
+ * "split" the dotted fields in its parts, but no validation is done.
  *
  * Any field part may be replaced, after the "original" field reference was parsed. Any
  * part can be accessed through a StringData object.
@@ -93,12 +95,6 @@ public:
 
     explicit FieldRef(StringData path);
 
-    /**
-     * Field parts accessed through getPart() calls no longer would be valid, after the destructor
-     * ran.
-     */
-    ~FieldRef() {}
-
     /**
      * Builds a field path out of each field part in 'dottedField'.
      */
@@ -226,43 +222,16 @@ public:
     uint64_t estimateObjectSizeInBytes() const {
         return  // Add size of each element in '_replacements' vector.
             container_size_helper::estimateObjectSizeInBytes(
-                _replacements,
-                [](const ValidatedPathString& s) { return s.get().capacity(); },
-                true) +
+                _replacements, [](const std::string& s) { return s.capacity(); }, true) +
             // Add size of each element in '_parts' vector.
             container_size_helper::estimateObjectSizeInBytes(_parts) +
             // Add runtime size of '_dotted' string.
-            _dotted.get().capacity() +
+            _dotted.capacity() +
             // Add size of the object.
             sizeof(*this);
     }
 
 private:
-    /**
-     * Represents a string that has been validated to not contain embedded null bytes.
-     */
-    class ValidatedPathString {
-    public:
-        ValidatedPathString() = default;
-
-        explicit ValidatedPathString(std::string value) : _value(std::move(value)) {
-            uassert(9867600,
-                    "Field name can't contain null bytes",
-                    _value.find('\0') == std::string::npos);
-        }
-
-        void clear() {
-            _value.clear();
-        }
-
-        const std::string& get() const {
-            return _value;
-        }
-
-    private:
-        std::string _value;
-    };
-
     // Dotted fields are most often not longer than four parts. We use a mixed structure
     // here that will not require any extra memory allocation when that is the case. And
     // handle larger dotted fields if it is. The idea is not to penalize the common case
@@ -314,13 +283,13 @@ private:
      * Cached copy of the complete dotted name string. The StringView objects in "_parts" reference
      * this string.
      */
-    mutable ValidatedPathString _dotted;
+    mutable std::string _dotted;
 
     /**
      * String storage for path parts that have been replaced with setPart() or added with
      * appendPart() since the lasted time "_dotted" was materialized.
      */
-    mutable std::vector<ValidatedPathString> _replacements;
+    mutable std::vector<std::string> _replacements;
 };
 
 inline bool operator==(const FieldRef& lhs, const FieldRef& rhs) {
