@@ -27,25 +27,27 @@
  *    it in the license file.
  */
 
+#include <algorithm>
 #include <string>
 
-#include "mongo/base/error_codes.h"
-#include "mongo/base/status.h"
 #include "mongo/base/string_data.h"
+#include "mongo/bson/util/builder.h"
 #include "mongo/db/field_ref.h"
-#include "mongo/unittest/unittest.h"
-#include "mongo/util/str.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/death_test.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/assert_util.h"
 
 namespace mongo {
 namespace {
 
-TEST(FieldRefTest, NoFields) {
+TEST(Empty, NoFields) {
     FieldRef fieldRef("");
     ASSERT_EQUALS(fieldRef.numParts(), 0U);
     ASSERT_EQUALS(fieldRef.dottedField(), "");
 }
 
-TEST(FieldRefTest, NoFieldNames) {
+TEST(Empty, NoFieldNames) {
     std::string field = ".";
     FieldRef fieldRef(field);
     ASSERT_EQUALS(fieldRef.numParts(), 2U);
@@ -54,7 +56,7 @@ TEST(FieldRefTest, NoFieldNames) {
     ASSERT_EQUALS(fieldRef.dottedField(), field);
 }
 
-TEST(FieldRefTest, NoFieldNames2) {
+TEST(Empty, NoFieldNames2) {
     std::string field = "..";
     FieldRef fieldRef(field);
     ASSERT_EQUALS(fieldRef.numParts(), 3U);
@@ -64,7 +66,7 @@ TEST(FieldRefTest, NoFieldNames2) {
     ASSERT_EQUALS(fieldRef.dottedField(), field);
 }
 
-TEST(FieldRefTest, EmptyFieldName) {
+TEST(Empty, EmptyFieldName) {
     std::string field = ".b.";
     FieldRef fieldRef(field);
     ASSERT_EQUALS(fieldRef.numParts(), 3U);
@@ -74,7 +76,7 @@ TEST(FieldRefTest, EmptyFieldName) {
     ASSERT_EQUALS(fieldRef.dottedField(), field);
 }
 
-TEST(FieldRefTest, ReinitializeWithEmptyString) {
+TEST(Empty, ReinitializeWithEmptyString) {
     FieldRef fieldRef("a.b.c.d.e");
     ASSERT_EQUALS(fieldRef.numParts(), 5U);
 
@@ -83,7 +85,7 @@ TEST(FieldRefTest, ReinitializeWithEmptyString) {
     ASSERT_EQUALS(fieldRef.dottedField(), "");
 }
 
-TEST(FieldRefTest, SinglePart) {
+TEST(Normal, SinglePart) {
     std::string field = "a";
     FieldRef fieldRef(field);
     ASSERT_EQUALS(fieldRef.numParts(), 1U);
@@ -91,6 +93,17 @@ TEST(FieldRefTest, SinglePart) {
     ASSERT_EQUALS(fieldRef.dottedField(), field);
 }
 
+DEATH_TEST_REGEX(Normal, Overflow, "Tripwire assertion.*1589700") {
+    std::string field = "a";
+    for (size_t s = 1; s <= BSONObjMaxInternalSize / 2; s++) {
+        field.append(".a");
+    }
+    ASSERT_GT(field.size(), BSONObjMaxInternalSize);
+    FieldRef fieldRef;
+    ASSERT_THROWS_CODE(fieldRef.parse(field), AssertionException, 1589700);
+}
+
+
 TEST(Normal, ParseTwice) {
     std::string field = "a";
     FieldRef fieldRef;
@@ -102,7 +115,7 @@ TEST(Normal, ParseTwice) {
     }
 }
 
-TEST(FieldRefTest, MultiplePartsVariable) {
+TEST(Normal, MulitplePartsVariable) {
     const char* parts[] = {"a", "b", "c", "d", "e"};
     size_t size = sizeof(parts) / sizeof(char*);
     std::string field(parts[0]);
@@ -119,19 +132,7 @@ TEST(FieldRefTest, MultiplePartsVariable) {
     ASSERT_EQUALS(fieldRef.dottedField(), field);
 }
 
-TEST(FieldRefTest, ConstructorRejectsEmbeddedNullChars) {
-    const auto embeddedNull = "a\0"_sd;
-    FieldRef fieldRef;
-    ASSERT_THROWS_CODE(FieldRef(embeddedNull), AssertionException, 9867600);
-}
-
-TEST(FieldRefTest, ParseRejectsEmbeddedNullChars) {
-    const auto embeddedNull = "a\0"_sd;
-    FieldRef fieldRef;
-    ASSERT_THROWS_CODE(fieldRef.parse(embeddedNull), AssertionException, 9867600);
-}
-
-TEST(FieldRefTest, ReplaceSingleField) {
+TEST(Replacement, SingleField) {
     std::string field = "$";
     FieldRef fieldRef(field);
     ASSERT_EQUALS(fieldRef.numParts(), 1U);
@@ -144,7 +145,7 @@ TEST(FieldRefTest, ReplaceSingleField) {
     ASSERT_EQUALS(fieldRef.dottedField(), newField);
 }
 
-TEST(FieldRefTest, ReplaceInMultipleField) {
+TEST(Replacement, InMultipleField) {
     std::string field = "a.b.c.$.e";
     FieldRef fieldRef(field);
     ASSERT_EQUALS(fieldRef.numParts(), 5U);
@@ -157,7 +158,7 @@ TEST(FieldRefTest, ReplaceInMultipleField) {
     ASSERT_EQUALS(fieldRef.dottedField(), "a.b.c.d.e");
 }
 
-TEST(FieldRefTest, SameFieldMultipleReplacements) {
+TEST(Replacement, SameFieldMultipleReplacements) {
     std::string prefix = "a.";
     std::string field = prefix + "$";
     FieldRef fieldRef(field);
@@ -171,13 +172,7 @@ TEST(FieldRefTest, SameFieldMultipleReplacements) {
     }
 }
 
-TEST(FieldRefTest, SetPartRejectsEmbeddedNullChars) {
-    FieldRef fieldRef("f");
-    const auto embeddedNull = "a\0"_sd;
-    ASSERT_THROWS_CODE(fieldRef.setPart(0, embeddedNull), AssertionException, 9867600);
-}
-
-TEST(FieldRefTest, NormalPrefix) {
+TEST(Prefix, Normal) {
     FieldRef prefix, base("a.b.c");
 
     prefix.parse("a.b");
@@ -193,7 +188,7 @@ TEST(FieldRefTest, NormalPrefix) {
     ASSERT_TRUE(prefix.isPrefixOfOrEqualTo(base));
 }
 
-TEST(FieldRefTest, DottedPrefix) {
+TEST(Prefix, Dotted) {
     FieldRef prefix("a.0"), base("a.0.c");
     ASSERT_TRUE(prefix.isPrefixOf(base));
     ASSERT_TRUE(prefix.isPrefixOfOrEqualTo(base));
@@ -203,7 +198,7 @@ TEST(FieldRefTest, DottedPrefix) {
     ASSERT_TRUE(prefix.isPrefixOfOrEqualTo(base));
 }
 
-TEST(FieldRefTest, NoPrefixes) {
+TEST(Prefix, NoPrefixes) {
     FieldRef prefix("a.b"), base("a.b");
     ASSERT_FALSE(prefix.isPrefixOf(base));
     ASSERT_TRUE(prefix.isPrefixOfOrEqualTo(base));
@@ -217,14 +212,14 @@ TEST(FieldRefTest, NoPrefixes) {
     ASSERT_FALSE(prefix.isPrefixOfOrEqualTo(base));
 }
 
-TEST(FieldRefTest, EmptyPrefix) {
+TEST(Prefix, EmptyBase) {
     FieldRef field("a"), empty;
     ASSERT_FALSE(field.isPrefixOf(empty));
     ASSERT_FALSE(empty.isPrefixOf(field));
     ASSERT_FALSE(empty.isPrefixOf(empty));
 }
 
-TEST(FieldRefTest, CommonPrefixSizeNormal) {
+TEST(PrefixSize, Normal) {
     FieldRef fieldA("a.b"), fieldB("a");
     ASSERT_EQUALS(fieldA.commonPrefixSize(fieldB), 1U);
 
@@ -235,20 +230,20 @@ TEST(FieldRefTest, CommonPrefixSizeNormal) {
     ASSERT_EQUALS(fieldA.commonPrefixSize(fieldB), 2U);
 }
 
-TEST(FieldRefTest, CommonPrefixNoCommonality) {
+TEST(PrefixSize, NoCommonatility) {
     FieldRef fieldA, fieldB;
     fieldA.parse("a");
     fieldB.parse("b");
     ASSERT_EQUALS(fieldA.commonPrefixSize(fieldB), 0U);
 }
 
-TEST(FieldRefTest, CommonPrefixSizeEmpty) {
+TEST(PrefixSize, Empty) {
     FieldRef fieldA("a"), empty;
     ASSERT_EQUALS(fieldA.commonPrefixSize(empty), 0U);
     ASSERT_EQUALS(empty.commonPrefixSize(fieldA), 0U);
 }
 
-TEST(FieldRefTest, FullyOverlapsWithNormal) {
+TEST(FullyOverlapsWith, Normal) {
     FieldRef fieldA("a"), fieldB("a.b"), fieldC("a.b.c");
     FieldRef fieldD("a.b.d");
     ASSERT(fieldA.fullyOverlapsWith(fieldA));
@@ -267,19 +262,19 @@ TEST(FieldRefTest, FullyOverlapsWithNormal) {
     ASSERT_FALSE(fieldC.fullyOverlapsWith(fieldD));
 }
 
-TEST(FieldRefTest, FullyOverlapsWithNoCommonality) {
+TEST(FullyOverlapsWith, NoCommonality) {
     FieldRef fieldA("a.b.c"), fieldB("b.c.d");
     ASSERT_FALSE(fieldA.fullyOverlapsWith(fieldB));
     ASSERT_FALSE(fieldB.fullyOverlapsWith(fieldA));
 }
 
-TEST(FieldRefTest, FullyOverlapsWithEmpty) {
+TEST(FullyOverlapsWith, Empty) {
     FieldRef field("a"), empty;
     ASSERT_FALSE(field.fullyOverlapsWith(empty));
     ASSERT_FALSE(empty.fullyOverlapsWith(field));
 }
 
-TEST(FieldRefTest, EqualitySimple1) {
+TEST(Equality, Simple1) {
     FieldRef a("a.b");
     ASSERT(a.equalsDottedField("a.b"));
     ASSERT(!a.equalsDottedField("a"));
@@ -287,7 +282,7 @@ TEST(FieldRefTest, EqualitySimple1) {
     ASSERT(!a.equalsDottedField("a.b.c"));
 }
 
-TEST(FieldRefTest, EqualitySimple2) {
+TEST(Equality, Simple2) {
     FieldRef a("a");
     ASSERT(!a.equalsDottedField("a.b"));
     ASSERT(a.equalsDottedField("a"));
@@ -295,7 +290,7 @@ TEST(FieldRefTest, EqualitySimple2) {
     ASSERT(!a.equalsDottedField("a.b.c"));
 }
 
-TEST(FieldRefTest, ComparisonBothEmpty) {
+TEST(Comparison, BothEmpty) {
     FieldRef a;
     ASSERT_TRUE(a == a);
     ASSERT_FALSE(a != a);
@@ -305,7 +300,7 @@ TEST(FieldRefTest, ComparisonBothEmpty) {
     ASSERT_TRUE(a >= a);
 }
 
-TEST(FieldRefTest, ComparisonEqualInSize) {
+TEST(Comparison, EqualInSize) {
     FieldRef a("a.b.c"), b("a.d.c");
     ASSERT_FALSE(a == b);
     ASSERT_TRUE(a != b);
@@ -315,7 +310,7 @@ TEST(FieldRefTest, ComparisonEqualInSize) {
     ASSERT_FALSE(a >= b);
 }
 
-TEST(FieldRefTest, ComparisonNonEqual) {
+TEST(Comparison, NonEqual) {
     FieldRef a("a.b.c"), b("b.d");
     ASSERT_FALSE(a == b);
     ASSERT_TRUE(a != b);
@@ -325,7 +320,7 @@ TEST(FieldRefTest, ComparisonNonEqual) {
     ASSERT_FALSE(a >= b);
 }
 
-TEST(FieldRefTest, ComparisonMixedEmptyAndNot) {
+TEST(Comparison, MixedEmtpyAndNot) {
     FieldRef a("a"), b;
     ASSERT_FALSE(a == b);
     ASSERT_TRUE(a != b);
@@ -335,7 +330,7 @@ TEST(FieldRefTest, ComparisonMixedEmptyAndNot) {
     ASSERT_TRUE(a >= b);
 }
 
-TEST(FieldRefTest, DottedFieldSimple1) {
+TEST(DottedField, Simple1) {
     FieldRef a("a.b.c.d.e");
     ASSERT_EQUALS("a.b.c.d.e", a.dottedField());
     ASSERT_EQUALS("a.b.c.d.e", a.dottedField(0));
@@ -347,7 +342,7 @@ TEST(FieldRefTest, DottedFieldSimple1) {
     ASSERT_EQUALS("", a.dottedField(6));
 }
 
-TEST(FieldRefTest, DottedSubstringShort) {
+TEST(DottedSubstring, Short) {
     FieldRef path("a");
     ASSERT_EQUALS(1u, path.numParts());
     ASSERT_EQUALS("a", path.dottedSubstring(0, path.numParts()));
@@ -355,7 +350,7 @@ TEST(FieldRefTest, DottedSubstringShort) {
     ASSERT_EQUALS("", path.dottedSubstring(0, 0));
 }
 
-TEST(FieldRefTest, DottedSubstringEmpty) {
+TEST(DottedSubstring, Empty) {
     FieldRef path("");
     ASSERT_EQUALS(0u, path.numParts());
     ASSERT_EQUALS("", path.dottedSubstring(0, path.numParts()));
@@ -363,7 +358,7 @@ TEST(FieldRefTest, DottedSubstringEmpty) {
     ASSERT_EQUALS("", path.dottedSubstring(0, 0));
 }
 
-TEST(FieldRefTest, DottedSubstringNested) {
+TEST(DottedSubstring, Nested) {
     FieldRef path("a.b.c.d.e");
     ASSERT_EQUALS(5u, path.numParts());
 
@@ -390,7 +385,7 @@ TEST(FieldRefTest, DottedSubstringNested) {
 }
 
 // The "short" append tests operate entirely in "reserve" space.
-TEST(FieldRefTest, AppendShortSimple) {
+TEST(AppendShort, Simple) {
     FieldRef path("a.b");
     ASSERT_EQUALS(2u, path.numParts());
 
@@ -399,7 +394,7 @@ TEST(FieldRefTest, AppendShortSimple) {
     ASSERT_EQUALS("a.b.c", path.dottedField());
 }
 
-TEST(FieldRefTest, AppendAndReplaceShort1) {
+TEST(AppendShort, AppendAndReplace1) {
     FieldRef path("a.b");
     ASSERT_EQUALS(2u, path.numParts());
 
@@ -409,7 +404,7 @@ TEST(FieldRefTest, AppendAndReplaceShort1) {
     ASSERT_EQUALS("a.0.c", path.dottedField());
 }
 
-TEST(FieldRefTest, AppendAndReplaceShort2) {
+TEST(AppendShort, AppendAndReplace2) {
     FieldRef path("a.b");
     ASSERT_EQUALS(2u, path.numParts());
 
@@ -419,7 +414,7 @@ TEST(FieldRefTest, AppendAndReplaceShort2) {
     ASSERT_EQUALS("a.b.0", path.dottedField());
 }
 
-TEST(FieldRefTest, ReplaceAndAppendShort) {
+TEST(AppendShort, ReplaceAndAppend) {
     FieldRef path("a.b");
     ASSERT_EQUALS(2u, path.numParts());
 
@@ -430,7 +425,7 @@ TEST(FieldRefTest, ReplaceAndAppendShort) {
     ASSERT_EQUALS("a.0.c", path.dottedField());
 }
 
-TEST(FieldRefTest, AppendShortAndGetPart) {
+TEST(AppendShort, AppendAndGetPart) {
     FieldRef path("a.b");
     ASSERT_EQUALS(2u, path.numParts());
 
@@ -441,7 +436,7 @@ TEST(FieldRefTest, AppendShortAndGetPart) {
     ASSERT_EQUALS("c", path.getPart(2));
 }
 
-TEST(FieldRefTest, AppendEmptyPart) {
+TEST(AppendShort, AppendEmptyPart) {
     FieldRef path("a.b");
     ASSERT_EQUALS(2u, path.numParts());
 
@@ -451,7 +446,7 @@ TEST(FieldRefTest, AppendEmptyPart) {
     ASSERT_EQUALS("a.b.", path.dottedField());
 }
 
-TEST(FieldRefTest, SetEmptyPartThenAppendShort) {
+TEST(AppendShort, SetEmptyPartThenAppend) {
     FieldRef path("a.b");
     ASSERT_EQUALS(2u, path.numParts());
 
@@ -462,22 +457,9 @@ TEST(FieldRefTest, SetEmptyPartThenAppendShort) {
     ASSERT_EQUALS("", path.getPart(1));
 }
 
-TEST(FieldRefTest, AppendPartRejectsEmbeddedNullChars) {
-    FieldRef fieldRef("f");
-    const auto embeddedNull = "a\0"_sd;
-    ASSERT_THROWS_CODE(fieldRef.appendPart(embeddedNull), AssertionException, 9867600);
-    ASSERT_THROWS_CODE(
-        FieldRef::FieldRefTempAppend(fieldRef, embeddedNull), AssertionException, 9867600);
-}
-
-TEST(FieldRefTest, ContainsTooManyDots) {
-    std::string path(255, '.');
-    ASSERT_THROWS_CODE(FieldRef(path), AssertionException, 10396001);
-}
-
 // The "medium" append tests feature an append operation that spills out of reserve space (i.e.,
 // we append to a path that has _size == kReserveAhead).
-TEST(FieldRefTest, AppendMediumSimple) {
+TEST(AppendMedium, Simple) {
     FieldRef path("a.b.c.d");
     ASSERT_EQUALS(4u, path.numParts());
 
@@ -486,7 +468,7 @@ TEST(FieldRefTest, AppendMediumSimple) {
     ASSERT_EQUALS("a.b.c.d.e", path.dottedField());
 }
 
-TEST(FieldRefTest, AppendAndReplaceMedium1) {
+TEST(AppendMedium, AppendAndReplace1) {
     FieldRef path("a.b.c.d");
     ASSERT_EQUALS(4u, path.numParts());
 
@@ -496,7 +478,7 @@ TEST(FieldRefTest, AppendAndReplaceMedium1) {
     ASSERT_EQUALS("a.0.c.d.e", path.dottedField());
 }
 
-TEST(FieldRefTest, AppendAndReplaceMedium2) {
+TEST(AppendMedium, AppendAndReplace2) {
     FieldRef path("a.b.c.d");
     ASSERT_EQUALS(4u, path.numParts());
 
@@ -506,7 +488,7 @@ TEST(FieldRefTest, AppendAndReplaceMedium2) {
     ASSERT_EQUALS("a.b.c.d.0", path.dottedField());
 }
 
-TEST(FieldRefTest, ReplaceAndAppendMedium) {
+TEST(AppendMedium, ReplaceAndAppend) {
     FieldRef path("a.b.c.d");
     ASSERT_EQUALS(4u, path.numParts());
 
@@ -517,7 +499,7 @@ TEST(FieldRefTest, ReplaceAndAppendMedium) {
     ASSERT_EQUALS("a.0.c.d.e", path.dottedField());
 }
 
-TEST(FieldRefTest, AppendAndGetPartMedium) {
+TEST(AppendMedium, AppendAndGetPart) {
     FieldRef path("a.b.c.d");
     ASSERT_EQUALS(4u, path.numParts());
 
@@ -530,7 +512,7 @@ TEST(FieldRefTest, AppendAndGetPartMedium) {
     ASSERT_EQUALS("e", path.getPart(4));
 }
 
-TEST(FieldRefTest, AppendEmptyPartToMedium) {
+TEST(AppendMedium, AppendEmptyPart) {
     FieldRef path("a.b.c.d");
     ASSERT_EQUALS(4u, path.numParts());
 
@@ -540,7 +522,7 @@ TEST(FieldRefTest, AppendEmptyPartToMedium) {
     ASSERT_EQUALS("a.b.c.d.", path.dottedField());
 }
 
-TEST(FieldRefTest, SetEmptyPartThenAppendToMedium) {
+TEST(AppendMedium, SetEmptyPartThenAppend) {
     FieldRef path("a.b.c.d");
     ASSERT_EQUALS(4u, path.numParts());
 
@@ -553,7 +535,7 @@ TEST(FieldRefTest, SetEmptyPartThenAppendToMedium) {
 
 // The "long" append tests have paths that are bigger than the reserve space throughout their life
 // cycle.
-TEST(FieldRefTest, AppendLongSimple) {
+TEST(AppendLong, Simple) {
     FieldRef path("a.b.c.d.e.f");
     ASSERT_EQUALS(6u, path.numParts());
 
@@ -564,7 +546,7 @@ TEST(FieldRefTest, AppendLongSimple) {
     ASSERT_EQUALS("a.b.c.d.e.f.g.h.i", path.dottedField());
 }
 
-TEST(FieldRefTest, AppendAndReplaceLong1) {
+TEST(AppendLong, AppendAndReplace1) {
     FieldRef path("a.b.c.d.e.f");
     ASSERT_EQUALS(6u, path.numParts());
 
@@ -577,7 +559,7 @@ TEST(FieldRefTest, AppendAndReplaceLong1) {
     ASSERT_EQUALS("a.0.c.d.e.1.g.h.i", path.dottedField());
 }
 
-TEST(FieldRefTest, AppendAndReplaceLong2) {
+TEST(AppendLong, AppendAndReplace2) {
     FieldRef path("a.b.c.d.e.f");
     ASSERT_EQUALS(6u, path.numParts());
 
@@ -589,7 +571,7 @@ TEST(FieldRefTest, AppendAndReplaceLong2) {
     ASSERT_EQUALS("a.b.c.d.e.f.g.0.i", path.dottedField());
 }
 
-TEST(FieldRefTest, ReplaceAndAppendLong) {
+TEST(AppendLong, ReplaceAndAppend) {
     FieldRef path("a.b.c.d.e.f");
     ASSERT_EQUALS(6u, path.numParts());
 
@@ -604,7 +586,7 @@ TEST(FieldRefTest, ReplaceAndAppendLong) {
     ASSERT_EQUALS("a.0.c.d.e.1.g.h.i", path.dottedField());
 }
 
-TEST(FieldRefTest, AppendAndGetPartLong) {
+TEST(AppendLong, AppendAndGetPart) {
     FieldRef path("a.b.c.d.e.f");
     ASSERT_EQUALS(6u, path.numParts());
 
@@ -625,7 +607,7 @@ TEST(FieldRefTest, AppendAndGetPartLong) {
     ASSERT_EQUALS("i", path.getPart(8));
 }
 
-TEST(FieldRefTest, AppendEmptyPartToLong) {
+TEST(AppendLong, AppendEmptyPart) {
     FieldRef path("a.b.c.d.e.f");
     ASSERT_EQUALS(6u, path.numParts());
 
@@ -635,7 +617,7 @@ TEST(FieldRefTest, AppendEmptyPartToLong) {
     ASSERT_EQUALS("a.b.c.d.e.f.", path.dottedField());
 }
 
-TEST(FieldRefTest, SetEmptyPartThenAppendToLong) {
+TEST(AppendLong, SetEmptyPartThenAppend) {
     FieldRef path("a.b.c.d.e.f");
     ASSERT_EQUALS(6u, path.numParts());
 
@@ -649,7 +631,7 @@ TEST(FieldRefTest, SetEmptyPartThenAppendToLong) {
 }
 
 // The "short" removeLastPart tests operate entirely in "reserve" space.
-TEST(FieldRefTest, RemoveLastParthShortSimple) {
+TEST(RemoveLastPartShort, Simple) {
     FieldRef path("a.b");
     ASSERT_EQUALS(2u, path.numParts());
 
@@ -658,7 +640,7 @@ TEST(FieldRefTest, RemoveLastParthShortSimple) {
     ASSERT_EQUALS("a", path.dottedField());
 }
 
-TEST(FieldRefTest, RemoveLastPartShortUntilEmpty) {
+TEST(RemoveLastPartShort, RemoveUntilEmpty) {
     FieldRef path("a.b");
     ASSERT_EQUALS(2u, path.numParts());
 
@@ -668,7 +650,7 @@ TEST(FieldRefTest, RemoveLastPartShortUntilEmpty) {
     ASSERT_EQUALS("", path.dottedField());
 }
 
-TEST(FieldRefTest, AppendThenRemoveLastPartShort) {
+TEST(RemoveLastPartShort, AppendThenRemove) {
     FieldRef path("a.b");
     ASSERT_EQUALS(2u, path.numParts());
 
@@ -678,7 +660,7 @@ TEST(FieldRefTest, AppendThenRemoveLastPartShort) {
     ASSERT_EQUALS("a.b", path.dottedField());
 }
 
-TEST(FieldRefTest, RemoveLastPartShortThenAppend) {
+TEST(RemoveLastPartShort, RemoveThenAppend) {
     FieldRef path("a.b");
     ASSERT_EQUALS(2u, path.numParts());
 
@@ -688,7 +670,7 @@ TEST(FieldRefTest, RemoveLastPartShortThenAppend) {
     ASSERT_EQUALS("a.b", path.dottedField());
 }
 
-TEST(FieldRefTest, RemoveLastPartShortThenSetPart) {
+TEST(RemoveLastPartShort, RemoveThenSetPart) {
     FieldRef path("a.b");
     ASSERT_EQUALS(2u, path.numParts());
 
@@ -698,7 +680,7 @@ TEST(FieldRefTest, RemoveLastPartShortThenSetPart) {
     ASSERT_EQUALS("0", path.dottedField());
 }
 
-TEST(FieldRefTest, SetPartThenRemoveLastPartShort) {
+TEST(RemoveLastPartShort, SetPartThenRemove) {
     FieldRef path("a.b");
     ASSERT_EQUALS(2u, path.numParts());
 
@@ -709,7 +691,7 @@ TEST(FieldRefTest, SetPartThenRemoveLastPartShort) {
     ASSERT_EQUALS("0", path.dottedField());
 }
 
-TEST(FieldRefTest, AppendThenSetPartThenRemoveLastPartShort) {
+TEST(RemoveLastPartShort, AppendThenSetPartThenRemove) {
     FieldRef path("a.b");
     ASSERT_EQUALS(2u, path.numParts());
 
@@ -722,7 +704,7 @@ TEST(FieldRefTest, AppendThenSetPartThenRemoveLastPartShort) {
 
 // The "medium" removeLastPart tests feature paths that change in size during the test so that they
 // transition between fitting and not fitting in "reserve" space.
-TEST(FieldRefTest, RemoveLastPartMediumSimple) {
+TEST(RemoveLastPartMedium, Simple) {
     FieldRef path("a.b.c.d.e");
     ASSERT_EQUALS(5u, path.numParts());
 
@@ -731,7 +713,7 @@ TEST(FieldRefTest, RemoveLastPartMediumSimple) {
     ASSERT_EQUALS("a.b.c.d", path.dottedField());
 }
 
-TEST(FieldRefTest, RemoveLastPartMediumUntilEmpty) {
+TEST(RemoveLastPartMedium, RemoveUntilEmpty) {
     FieldRef path("a.b.c.d.e");
     ASSERT_EQUALS(5u, path.numParts());
 
@@ -742,7 +724,7 @@ TEST(FieldRefTest, RemoveLastPartMediumUntilEmpty) {
     ASSERT_EQUALS("", path.dottedField());
 }
 
-TEST(FieldRefTest, AppendThenRemoveLastPartMedium) {
+TEST(RemoveLastPartMedium, AppendThenRemove) {
     FieldRef path("a.b.c.d");
     ASSERT_EQUALS(4u, path.numParts());
 
@@ -752,7 +734,7 @@ TEST(FieldRefTest, AppendThenRemoveLastPartMedium) {
     ASSERT_EQUALS("a.b.c.d", path.dottedField());
 }
 
-TEST(FieldRefTest, RemoveLastPartMediumThenAppend) {
+TEST(RemoveLastPartMedium, RemoveThenAppend) {
     FieldRef path("a.b.c.d.e");
     ASSERT_EQUALS(5u, path.numParts());
 
@@ -762,7 +744,7 @@ TEST(FieldRefTest, RemoveLastPartMediumThenAppend) {
     ASSERT_EQUALS("a.b.c.d.e", path.dottedField());
 }
 
-TEST(FieldRefTest, RemoveLastPartMediumThenSetPart) {
+TEST(RemoveLastPartMedium, RemoveThenSetPart) {
     FieldRef path("a.b.c.d.e");
     ASSERT_EQUALS(5u, path.numParts());
 
@@ -772,7 +754,7 @@ TEST(FieldRefTest, RemoveLastPartMediumThenSetPart) {
     ASSERT_EQUALS("0.b.c.d", path.dottedField());
 }
 
-TEST(FieldRefTest, SetPartThenRemoveLastPartMedium) {
+TEST(RemoveLastPartMedium, SetPartThenRemove) {
     FieldRef path("a.b.c.d.e");
     ASSERT_EQUALS(5u, path.numParts());
 
@@ -783,7 +765,7 @@ TEST(FieldRefTest, SetPartThenRemoveLastPartMedium) {
     ASSERT_EQUALS("0.b.c.d", path.dottedField());
 }
 
-TEST(FieldRefTest, AppendThenSetPartThenRemoveLastPartMedium) {
+TEST(RemoveLastPartMedium, AppendThenSetPartThenRemove) {
     FieldRef path("a.b.c.d");
     ASSERT_EQUALS(4u, path.numParts());
 
@@ -797,7 +779,7 @@ TEST(FieldRefTest, AppendThenSetPartThenRemoveLastPartMedium) {
 
 // The "long" removeLastPart tests have paths that are bigger than the reserve space throughout
 // their life cycle (with the exception of RemoveUntilempty).
-TEST(FieldRefTest, RemoveLastPartLongSimple) {
+TEST(RemoveLastPartLong, Simple) {
     FieldRef path("a.b.c.d.e.f.g");
     ASSERT_EQUALS(7u, path.numParts());
 
@@ -806,7 +788,7 @@ TEST(FieldRefTest, RemoveLastPartLongSimple) {
     ASSERT_EQUALS("a.b.c.d.e.f", path.dottedField());
 }
 
-TEST(FieldRefTest, RemoveLastPartLongUntilEmpty) {
+TEST(RemoveLastPartLong, RemoveUntilEmpty) {
     FieldRef path("a.b.c.d.e.f.g");
     ASSERT_EQUALS(7u, path.numParts());
 
@@ -817,7 +799,7 @@ TEST(FieldRefTest, RemoveLastPartLongUntilEmpty) {
     ASSERT_EQUALS("", path.dottedField());
 }
 
-TEST(FieldRefTest, AppendThenRemoveLastPartLong) {
+TEST(RemoveLastPartLong, AppendThenRemove) {
     FieldRef path("a.b.c.d.e.f");
     ASSERT_EQUALS(6u, path.numParts());
 
@@ -827,7 +809,7 @@ TEST(FieldRefTest, AppendThenRemoveLastPartLong) {
     ASSERT_EQUALS("a.b.c.d.e.f", path.dottedField());
 }
 
-TEST(FieldRefTest, RemoveLastPartLongThenAppend) {
+TEST(RemoveLastPartLong, RemoveThenAppend) {
     FieldRef path("a.b.c.d.e.f.g");
     ASSERT_EQUALS(7u, path.numParts());
 
@@ -837,7 +819,7 @@ TEST(FieldRefTest, RemoveLastPartLongThenAppend) {
     ASSERT_EQUALS("a.b.c.d.e.f.g", path.dottedField());
 }
 
-TEST(FieldRefTest, RemoveLastPartLongThenSetPart) {
+TEST(RemoveLastPartLong, RemoveThenSetPart) {
     FieldRef path("a.b.c.d.e.f.g");
     ASSERT_EQUALS(7u, path.numParts());
 
@@ -848,7 +830,7 @@ TEST(FieldRefTest, RemoveLastPartLongThenSetPart) {
     ASSERT_EQUALS("0.b.c.d.1.f", path.dottedField());
 }
 
-TEST(FieldRefTest, SetPartThenRemoveLastPartLong) {
+TEST(RemoveLastPartLong, SetPartThenRemove) {
     FieldRef path("a.b.c.d.e.f.g");
     ASSERT_EQUALS(7u, path.numParts());
 
@@ -860,7 +842,7 @@ TEST(FieldRefTest, SetPartThenRemoveLastPartLong) {
     ASSERT_EQUALS("0.b.c.d.1.f", path.dottedField());
 }
 
-TEST(FieldRefTest, AppendThenSetPartThenRemoveLastPartLong) {
+TEST(RemoveLastPartLong, AppendThenSetPartThenRemove) {
     FieldRef path("a.b.c.d.e.f");
     ASSERT_EQUALS(6u, path.numParts());
 
@@ -873,13 +855,13 @@ TEST(FieldRefTest, AppendThenSetPartThenRemoveLastPartLong) {
     ASSERT_EQUALS("a.b.0.d.1.f", path.dottedField());
 }
 
-TEST(FieldRefTest, CopyConstructor) {
+TEST(RemoveLastPartLong, FieldRefCopyConstructor) {
     FieldRef original("a.b.c");
     FieldRef copy = original;
     ASSERT_EQ(original, copy);
 }
 
-TEST(FieldRefTest, CopyAssignment) {
+TEST(RemoveLastPartLong, FieldRefCopyAssignment) {
     FieldRef original("a.b.c");
     FieldRef other("x.y.z");
     ASSERT_NE(original, other);
@@ -887,7 +869,7 @@ TEST(FieldRefTest, CopyAssignment) {
     ASSERT_EQ(original, other);
 }
 
-TEST(FieldRefTest, FromCopyAssignmentIsValidAfterOriginalIsDeleted) {
+TEST(RemoveLastPartLong, FieldRefFromCopyAssignmentIsValidAfterOriginalIsDeleted) {
     FieldRef copy("x.y.z");
     {
         FieldRef original("a.b.c");
@@ -897,7 +879,7 @@ TEST(FieldRefTest, FromCopyAssignmentIsValidAfterOriginalIsDeleted) {
     ASSERT_EQ(copy, FieldRef("a.b.c"));
 }
 
-TEST(FieldRefTest, FromCopyAssignmentIsADeepCopy) {
+TEST(RemoveLastPartLong, FieldRefFromCopyAssignmentIsADeepCopy) {
     FieldRef original("a.b.c");
     FieldRef other("x.y.z");
     ASSERT_NE(original, other);
@@ -911,7 +893,7 @@ TEST(FieldRefTest, FromCopyAssignmentIsADeepCopy) {
     ASSERT_EQ(other, FieldRef("a.b.c"));
 }
 
-TEST(FieldRefTest, CanIdentifyNumericPathComponents) {
+TEST(NumericPathComponents, CanIdentifyNumericPathComponents) {
     FieldRef path("a.0.b.1.c");
     ASSERT(!path.isNumericPathComponentStrict(0));
     ASSERT(path.isNumericPathComponentStrict(1));
@@ -920,40 +902,40 @@ TEST(FieldRefTest, CanIdentifyNumericPathComponents) {
     ASSERT(!path.isNumericPathComponentStrict(4));
 }
 
-TEST(FieldRefTest, CanObtainAllNumericPathComponents) {
+TEST(NumericPathComponents, CanObtainAllNumericPathComponents) {
     FieldRef path("a.0.b.1.c.2.d");
     std::set<FieldIndex> expectedComponents{FieldIndex(1), FieldIndex(3), FieldIndex(5)};
     auto numericPathComponents = path.getNumericPathComponents();
     ASSERT(numericPathComponents == expectedComponents);
 }
 
-TEST(FieldRefTest, FieldsWithLeadingZeroesAreNotConsideredNumeric) {
+TEST(NumericPathComponents, FieldsWithLeadingZeroesAreNotConsideredNumeric) {
     FieldRef path("a.0.b.01.c.2.d");
     std::set<FieldIndex> expectedComponents{FieldIndex(1), FieldIndex(5)};
     auto numericPathComponents = path.getNumericPathComponents();
     ASSERT(numericPathComponents == expectedComponents);
 }
 
-TEST(FieldRefTest, RemoveFirstPartOnEmptyPathDoesNothing) {
+TEST(RemoveFirstPart, EmptyPathDoesNothing) {
     FieldRef path;
     path.removeFirstPart();
     ASSERT_EQ(path.numParts(), 0U);
 }
 
-TEST(FieldRefTest, RemoveFirstPartPathWithOneComponentBecomesEmpty) {
+TEST(RemoveFirstPart, PathWithOneComponentBecomesEmpty) {
     FieldRef path("first");
     path.removeFirstPart();
     ASSERT_EQ(path.numParts(), 0U);
 }
 
-TEST(FieldRefTest, RemoveFirstPartPathWithTwoComponentsOnlyHoldsSecond) {
+TEST(RemoveFirstPart, PathWithTwoComponentsOnlyHoldsSecond) {
     FieldRef path("remove.keep");
     path.removeFirstPart();
     ASSERT_EQ(path.numParts(), 1U);
     ASSERT_EQ(path, FieldRef("keep"));
 }
 
-TEST(FieldRefTest, RemovingFirstPartFromLongPathMultipleTimes) {
+TEST(RemoveFirstPart, RemovingFirstPartFromLongPathMultipleTimes) {
     FieldRef path("first.second.third.fourth.fifth.sixth.seventh.eigth.ninth.tenth");
     path.removeFirstPart();
     ASSERT_EQ(path, FieldRef("second.third.fourth.fifth.sixth.seventh.eigth.ninth.tenth"));
