@@ -28,40 +28,79 @@
  */
 
 
-#include <algorithm>
+#include <boost/cstdint.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/smart_ptr.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <cstddef>
+#include <map>
 #include <memory>
+#include <mutex>
+#include <string>
+#include <tuple>
+#include <type_traits>
+#include <variant>
+
+#include <boost/optional/optional.hpp>
 
 #include "mongo/base/data_range.h"
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/base/string_data.h"
 #include "mongo/bson/bsonelement.h"
 #include "mongo/bson/bsonmisc.h"
 #include "mongo/bson/bsonobj.h"
 #include "mongo/bson/bsonobjbuilder.h"
-#include "mongo/bson/bsontypes.h"
 #include "mongo/crypto/encryption_fields_gen.h"
 #include "mongo/crypto/fle_crypto.h"
 #include "mongo/crypto/fle_field_schema_gen.h"
+#include "mongo/crypto/fle_stats_gen.h"
 #include "mongo/db/auth/authorization_session.h"
+#include "mongo/db/auth/validated_tenancy_scope_factory.h"
+#include "mongo/db/basic_types.h"
+#include "mongo/db/client.h"
 #include "mongo/db/commands/fle2_get_count_info_command_gen.h"
+#include "mongo/db/curop.h"
+#include "mongo/db/database_name.h"
 #include "mongo/db/dbdirectclient.h"
+#include "mongo/db/error_labels.h"
 #include "mongo/db/fle_crud.h"
+#include "mongo/db/logical_time.h"
+#include "mongo/db/multitenancy_gen.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/operation_time_tracker.h"
+#include "mongo/db/ops/write_ops.h"
 #include "mongo/db/ops/write_ops_gen.h"
 #include "mongo/db/ops/write_ops_parsers.h"
 #include "mongo/db/query/collation/collator_factory_interface.h"
+#include "mongo/db/query/collation/collator_interface.h"
 #include "mongo/db/query/find_command.h"
+#include "mongo/db/query/fle/query_rewriter_interface.h"
 #include "mongo/db/query/fle/server_rewrite.h"
-#include "mongo/db/repl/repl_client_info.h"
+#include "mongo/db/repl/optime.h"
 #include "mongo/db/service_context.h"
+#include "mongo/db/tenant_id.h"
 #include "mongo/db/transaction/transaction_api.h"
+#include "mongo/executor/task_executor_pool.h"
 #include "mongo/idl/idl_parser.h"
 #include "mongo/logv2/log.h"
-#include "mongo/rpc/factory.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/rpc/get_status_from_command_result.h"
+#include "mongo/rpc/reply_interface.h"
+#include "mongo/rpc/unique_message.h"
 #include "mongo/s/grid.h"
 #include "mongo/s/transaction_router_resource_yielder.h"
 #include "mongo/s/write_ops/batch_write_exec.h"
+#include "mongo/s/write_ops/batched_upsert_detail.h"
 #include "mongo/util/assert_util.h"
-#include "mongo/util/concurrency/thread_pool.h"
+#include "mongo/util/fail_point.h"
+#include "mongo/util/future.h"
+#include "mongo/util/namespace_string_util.h"
+#include "mongo/util/out_of_line_executor.h"
+#include "mongo/util/str.h"
+#include "mongo/util/uuid.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kWrite
 
@@ -108,6 +147,23 @@ void appendPossibleWriteConcernErrorToReply(const WriteConcernErrorDetail& wce,
     }
 }
 
+// Throw transientTransaction errors as top-level errors instead of returning them as writeErrors.
+// By throwing them, the top-level error handling code kicks in and adds the error label. Also
+// discards the writeErrors.
+void throwTransientTransactionErrorIfNeeded(
+    bool hasWriteConcernError,
+    const boost::optional<std::vector<write_ops::WriteError>>& writeErrors) {
+    if (!writeErrors.has_value()) {
+        return;
+    }
+
+    for (const auto& e : writeErrors.value()) {
+        if (isTransientTransactionError(e.getStatus().code(), hasWriteConcernError, true)) {
+            uassertStatusOK(e.getStatus());
+        }
+    }
+}
+
 void replyToResponse(OperationContext* opCtx,
                      write_ops::WriteCommandReplyBase* replyBase,
                      BatchedCommandResponse* response) {
@@ -209,23 +265,28 @@ std::vector<QECountInfoRequestTokenSet> toTagSets(
 
 FLEEdgeCountInfo convertTokensToEdgeCount(const QECountInfoReplyTokens& token) {
 
-    boost::optional<EDCDerivedFromDataTokenAndContentionFactorToken> edc;
-    if (token.getEDCDerivedFromDataTokenAndContentionFactorToken()) {
-        edc = FLETokenFromCDR<FLETokenType::EDCDerivedFromDataTokenAndContentionFactorToken>(
-            token.getEDCDerivedFromDataTokenAndContentionFactorToken().value());
-    }
+    auto edc = token.getEDCDerivedFromDataTokenAndContentionFactorToken().map([](auto& t) {
+        return FLETokenFromCDR<FLETokenType::EDCDerivedFromDataTokenAndContentionFactorToken>(t);
+    });
 
-    boost::optional<uint64_t> cpos, apos;
-    auto& spos = token.getSearchedPositions();
-    if (spos) {
-        cpos = spos->getCpos();
-        apos = spos->getApos();
-    }
+    auto spos = token.getSearchedPositions().map([](auto& pair) {
+        EmuBinaryResult newPair;
+        newPair.cpos = pair.getCpos();
+        newPair.apos = pair.getApos();
+        return newPair;
+    });
+
+    auto npos = token.getNullAnchorPositions().map([](auto& pair) {
+        ESCCountsPair newPair;
+        newPair.cpos = pair.getCpos();
+        newPair.apos = pair.getApos();
+        return newPair;
+    });
 
     auto esc =
         FLETokenFromCDR<FLETokenType::ESCTwiceDerivedTagToken>(token.getESCTwiceDerivedTagToken());
 
-    return FLEEdgeCountInfo(token.getCount(), esc, cpos, apos, token.getStats(), edc);
+    return FLEEdgeCountInfo(token.getCount(), esc.data, spos, npos, token.getStats(), edc);
 }
 
 std::vector<std::vector<FLEEdgeCountInfo>> toEdgeCounts(
@@ -340,16 +401,14 @@ std::pair<mongo::StatusWith<mongo::txn_api::CommitResult>,
 insertSingleDocument(OperationContext* opCtx,
                      const write_ops::InsertCommandRequest& insertRequest,
                      BSONObj& document,
+                     const mongo::EncryptedFieldConfig& efc,
                      int32_t* stmtId,
                      GetTxnCallback getTxns) {
     auto edcNss = insertRequest.getNamespace();
-    auto ei = insertRequest.getEncryptionInformation().value();
 
     bool bypassDocumentValidation =
         insertRequest.getWriteCommandRequestBase().getBypassDocumentValidation();
 
-    auto efc = EncryptionInformationHelpers::getAndValidateSchema(edcNss, ei);
-
     EDCServerCollection::validateEncryptedFieldInfo(document, efc, bypassDocumentValidation);
     auto serverPayload = std::make_shared<std::vector<EDCServerPayloadInfo>>(
         EDCServerCollection::getEncryptedFieldInfo(document));
@@ -365,13 +424,14 @@ insertSingleDocument(OperationContext* opCtx,
     auto sharedInsertBlock = std::make_shared<decltype(insertBlock)>(insertBlock);
 
     auto reply = std::make_shared<write_ops::InsertCommandReply>();
+    auto service = opCtx->getService();
 
     auto baseStmtId = *stmtId;
     auto swResult = trun->runNoThrow(
         opCtx,
-        [sharedInsertBlock, reply, ownedDocument, bypassDocumentValidation, baseStmtId](
+        [service, sharedInsertBlock, reply, ownedDocument, bypassDocumentValidation, baseStmtId](
             const txn_api::TransactionClient& txnClient, ExecutorPtr txnExec) {
-            FLEQueryInterfaceImpl queryImpl(txnClient, getGlobalServiceContext());
+            FLEQueryInterfaceImpl queryImpl(txnClient, service);
 
             auto [edcNss2, efc2, serverPayload2, stmtId2] = *sharedInsertBlock.get();
 
@@ -430,7 +490,7 @@ std::pair<FLEBatchResult, write_ops::InsertCommandReply> processInsert(
     write_ops::WriteCommandReplyBase writeBase;
     std::vector<StmtId> retriedStmtIds;
 
-    // TODO: Remove with SERVER-73714
+    // This is an optimization for single-document unencrypted inserts.
     if (documents.size() == 1) {
         auto serverPayload = EDCServerCollection::getEncryptedFieldInfo(documents[0]);
         if (serverPayload.size() == 0) {
@@ -439,9 +499,12 @@ std::pair<FLEBatchResult, write_ops::InsertCommandReply> processInsert(
         }
     }
 
+    auto efc = EncryptionInformationHelpers::getAndValidateSchema(
+        insertRequest.getNamespace(), insertRequest.getEncryptionInformation().value());
+
     for (auto& document : documents) {
         const auto& [swResult, reply] =
-            insertSingleDocument(opCtx, insertRequest, document, &stmtId, getTxns);
+            insertSingleDocument(opCtx, insertRequest, document, efc, &stmtId, getTxns);
 
         writeBase.setElectionId(reply->getElectionId());
 
@@ -498,6 +561,9 @@ std::pair<FLEBatchResult, write_ops::InsertCommandReply> processInsert(
         writeBase.setRetriedStmtIds(std::move(retriedStmtIds));
     }
     appendPossibleWriteConcernErrorToReply(wcError, &writeBase);
+
+    throwTransientTransactionErrorIfNeeded(wcError.isValid(nullptr), writeErrors);
+
     returnReply.setWriteCommandReplyBase(writeBase);
 
     return {FLEBatchResult::kProcessed, returnReply};
@@ -519,29 +585,27 @@ write_ops::DeleteCommandReply processDelete(OperationContext* opCtx,
     auto reply = std::make_shared<write_ops::DeleteCommandReply>();
 
     auto ownedRequest = deleteRequest.serialize({});
-    const auto tenantId = deleteRequest.getDbName().tenantId();
-    if (tenantId && gMultitenancySupport) {
-        // `ownedRequest` is OpMsgRequest type which will parse the tenantId from ValidatedTenantId.
-        // Before parsing we should ensure that validatedTenancyScope is set in order not to lose
-        // the tenantId after the parsing.
-        ownedRequest.validatedTenancyScope =
-            VTS(tenantId.get(), VTS::TrustedForInnerOpMsgRequestTag{});
+
+    if (gMultitenancySupport) {
+        ownedRequest.validatedTenancyScope = auth::ValidatedTenancyScope::get(opCtx);
     }
 
     auto ownedDeleteRequest =
         write_ops::DeleteCommandRequest::parse(IDLParserContext("delete"), ownedRequest);
+
     auto ownedDeleteOpEntry = ownedDeleteRequest.getDeletes()[0];
     auto expCtx = makeExpCtx(opCtx, ownedDeleteRequest, ownedDeleteOpEntry);
     // The function that handles the transaction may outlive this function so we need to use
     // shared_ptrs
     auto deleteBlock = std::make_tuple(ownedDeleteRequest, expCtx);
     auto sharedDeleteBlock = std::make_shared<decltype(deleteBlock)>(deleteBlock);
+    auto service = opCtx->getService();
 
     auto swResult = trun->runNoThrow(
         opCtx,
-        [sharedDeleteBlock, ownedRequest, reply](const txn_api::TransactionClient& txnClient,
-                                                 ExecutorPtr txnExec) {
-            FLEQueryInterfaceImpl queryImpl(txnClient, getGlobalServiceContext());
+        [service, sharedDeleteBlock, ownedRequest, reply](
+            const txn_api::TransactionClient& txnClient, ExecutorPtr txnExec) {
+            FLEQueryInterfaceImpl queryImpl(txnClient, service);
 
             auto [deleteRequest2, expCtx2] = *sharedDeleteBlock.get();
 
@@ -570,6 +634,8 @@ write_ops::DeleteCommandReply processDelete(OperationContext* opCtx,
             return SemiFuture<void>::makeReady();
         });
 
+    bool hasWriteConcernError = false;
+
     if (!swResult.isOK()) {
         // FLETransactionAbort is used for control flow so it means we have a valid
         // InsertCommandReply with write errors so we should return that.
@@ -580,22 +646,28 @@ write_ops::DeleteCommandReply processDelete(OperationContext* opCtx,
         appendSingleStatusToWriteErrors(swResult.getStatus(), &reply->getWriteCommandReplyBase());
     } else if (!swResult.getValue().getEffectiveStatus().isOK()) {
         auto& commitResult = swResult.getValue();
+
+        hasWriteConcernError = commitResult.wcError.isValid(nullptr);
+
         appendPossibleWriteConcernErrorToReply(commitResult.wcError,
                                                &reply->getWriteCommandReplyBase());
         if (!commitResult.cmdStatus.isOK()) {
-            if (ErrorCodes::isA<ErrorCategory::WriteConcernError>(commitResult.cmdStatus)) {
+            if (commitResult.cmdStatus == ErrorCodes::UnsatisfiableWriteConcern) {
                 // On single-write-shard commits, the transaction API will abort the transaction and
-                // return a CommitResult with one of the WriteConcernErrors if any of the read-only
-                // shards fail to commit with the user requested write concern. This happens before
-                // any commits to the write shards are performed, so none of the changes are
-                // actually made durable.
-                reply->getWriteCommandReplyBase().setN(0);
+                // return a CommitResult with cmdStatus=UnsatisfiableWriteConcern if any of the
+                // read-only shards fail to commit with the user requested write concern. This
+                // happens before any commits to the write shards are performed, so none of the
+                // changes are actually made durable.
+                reply->setN(0);
             }
             appendSingleStatusToWriteErrors(commitResult.cmdStatus,
                                             &reply->getWriteCommandReplyBase());
         }
     }
 
+    throwTransientTransactionErrorIfNeeded(hasWriteConcernError,
+                                           reply->getWriteCommandReplyBase().getWriteErrors());
+
     return *reply;
 }
 
@@ -631,11 +703,11 @@ write_ops::UpdateCommandReply processUpdate(OperationContext* opCtx,
     auto reply = std::make_shared<write_ops::UpdateCommandReply>();
 
     auto ownedRequest = updateRequest.serialize({});
-    const auto tenantId = updateRequest.getDbName().tenantId();
-    if (tenantId && gMultitenancySupport) {
-        ownedRequest.validatedTenancyScope =
-            VTS(tenantId.get(), VTS::TrustedForInnerOpMsgRequestTag{});
+
+    if (gMultitenancySupport) {
+        ownedRequest.validatedTenancyScope = auth::ValidatedTenancyScope::get(opCtx);
     }
+
     auto ownedUpdateRequest =
         write_ops::UpdateCommandRequest::parse(IDLParserContext("update"), ownedRequest);
     auto ownedUpdateOpEntry = ownedUpdateRequest.getUpdates()[0];
@@ -643,12 +715,13 @@ write_ops::UpdateCommandReply processUpdate(OperationContext* opCtx,
     auto expCtx = makeExpCtx(opCtx, ownedUpdateRequest, ownedUpdateOpEntry);
     auto updateBlock = std::make_tuple(ownedUpdateRequest, expCtx);
     auto sharedupdateBlock = std::make_shared<decltype(updateBlock)>(updateBlock);
+    auto service = opCtx->getService();
 
     auto swResult = trun->runNoThrow(
         opCtx,
-        [sharedupdateBlock, reply, ownedRequest](const txn_api::TransactionClient& txnClient,
-                                                 ExecutorPtr txnExec) {
-            FLEQueryInterfaceImpl queryImpl(txnClient, getGlobalServiceContext());
+        [service, sharedupdateBlock, reply, ownedRequest](
+            const txn_api::TransactionClient& txnClient, ExecutorPtr txnExec) {
+            FLEQueryInterfaceImpl queryImpl(txnClient, service);
 
             auto [updateRequest2, expCtx2] = *sharedupdateBlock.get();
 
@@ -676,6 +749,7 @@ write_ops::UpdateCommandReply processUpdate(OperationContext* opCtx,
             return SemiFuture<void>::makeReady();
         });
 
+    bool hasWriteConcernError = false;
     if (!swResult.isOK()) {
         // FLETransactionAbort is used for control flow so it means we have a valid
         // InsertCommandReply with write errors so we should return that.
@@ -686,23 +760,30 @@ write_ops::UpdateCommandReply processUpdate(OperationContext* opCtx,
         appendSingleStatusToWriteErrors(swResult.getStatus(), &reply->getWriteCommandReplyBase());
     } else if (!swResult.getValue().getEffectiveStatus().isOK()) {
         auto& commitResult = swResult.getValue();
+
+        hasWriteConcernError = commitResult.wcError.isValid(nullptr);
+
         appendPossibleWriteConcernErrorToReply(commitResult.wcError,
                                                &reply->getWriteCommandReplyBase());
         if (!commitResult.cmdStatus.isOK()) {
-            if (ErrorCodes::isA<ErrorCategory::WriteConcernError>(commitResult.cmdStatus)) {
+            if (commitResult.cmdStatus == ErrorCodes::UnsatisfiableWriteConcern) {
                 // On single-write-shard commits, the transaction API will abort the transaction and
-                // return a CommitResult with one of the WriteConcernErrors if any of the read-only
-                // shards fail to commit with the user requested write concern. This happens before
-                // any commits to the write shards are performed, so none of the changes are
-                // actually made durable.
+                // return a CommitResult with cmdStatus=UnsatisfiableWriteConcern if any of the
+                // read-only shards fail to commit with the user requested write concern. This
+                // happens before any commits to the write shards are performed, so none of the
+                // changes are actually made durable.
                 reply->setNModified(0);
-                reply->getWriteCommandReplyBase().setN(0);
+                reply->setN(0);
             }
             appendSingleStatusToWriteErrors(commitResult.cmdStatus,
                                             &reply->getWriteCommandReplyBase());
         }
     }
 
+    throwTransientTransactionErrorIfNeeded(hasWriteConcernError,
+                                           reply->getWriteCommandReplyBase().getWriteErrors());
+
+
     return *reply;
 }
 
@@ -718,8 +799,8 @@ void processFieldsForInsertV2(FLEQueryInterface* queryImpl,
         return;
     }
 
-    const NamespaceString nssEsc = NamespaceStringUtil::parseNamespaceFromRequest(
-        edcNss.dbName(), efc.getEscCollection().value());
+    const NamespaceString nssEsc =
+        NamespaceStringUtil::deserialize(edcNss.dbName(), efc.getEscCollection().value());
 
     uint32_t totalTokens = 0;
 
@@ -762,8 +843,6 @@ void processFieldsForInsertV2(FLEQueryInterface* queryImpl,
     std::vector<BSONObj> escDocuments;
     escDocuments.reserve(totalTokens);
 
-    HmacContext hmacCtx;
-
     for (size_t i = 0; i < countInfoSets.size(); i++) {
         auto& countInfos = countInfoSets[i];
 
@@ -775,7 +854,7 @@ void processFieldsForInsertV2(FLEQueryInterface* queryImpl,
             serverPayload[i].counts.push_back(countInfo.count);
 
             escDocuments.push_back(ESCCollection::generateNonAnchorDocument(
-                &hmacCtx, countInfo.tagToken, countInfo.count));
+                ESCTwiceDerivedTagToken(countInfo.tagTokenData), countInfo.count));
         }
     }
 
@@ -783,8 +862,8 @@ void processFieldsForInsertV2(FLEQueryInterface* queryImpl,
         uassertStatusOK(queryImpl->insertDocuments(nssEsc, escDocuments, pStmtId, true));
     checkWriteErrors(escInsertReply);
 
-    NamespaceString nssEcoc = NamespaceStringUtil::parseNamespaceFromRequest(
-        edcNss.dbName(), efc.getEcocCollection().value());
+    NamespaceString nssEcoc =
+        NamespaceStringUtil::deserialize(edcNss.dbName(), efc.getEcocCollection().value());
     std::vector<BSONObj> ecocDocuments;
     ecocDocuments.reserve(totalTokens);
 
@@ -825,7 +904,7 @@ std::shared_ptr<ReplyType> constructDefaultReply() {
 
 template <>
 std::shared_ptr<write_ops::FindAndModifyCommandRequest> constructDefaultReply() {
-    return std::make_shared<write_ops::FindAndModifyCommandRequest>(NamespaceString());
+    return std::make_shared<write_ops::FindAndModifyCommandRequest>(NamespaceString::kEmpty);
 }
 
 template <typename ReplyType>
@@ -895,8 +974,7 @@ StatusWith<std::pair<ReplyType, OpMsgRequest>> processFindAndModifyRequest(
     OperationContext* opCtx,
     const write_ops::FindAndModifyCommandRequest& findAndModifyRequest,
     GetTxnCallback getTxns,
-    ProcessFindAndModifyCallback<ReplyType> processCallback,
-    ErrorWithWriteConcernErrorCallback wceCallback) {
+    ProcessFindAndModifyCallback<ReplyType> processCallback) {
 
     {
         stdx::lock_guard<Client> lk(*opCtx->getClient());
@@ -911,12 +989,13 @@ StatusWith<std::pair<ReplyType, OpMsgRequest>> processFindAndModifyRequest(
     // shared_ptrs
     std::shared_ptr<ReplyType> reply = constructDefaultReply<ReplyType>();
 
+
     auto ownedRequest = findAndModifyRequest.serialize({});
-    const auto tenantId = findAndModifyRequest.getDbName().tenantId();
-    if (tenantId && gMultitenancySupport) {
-        ownedRequest.validatedTenancyScope =
-            VTS(tenantId.get(), VTS::TrustedForInnerOpMsgRequestTag{});
+
+    if (gMultitenancySupport) {
+        ownedRequest.validatedTenancyScope = auth::ValidatedTenancyScope::get(opCtx);
     }
+
     auto ownedFindAndModifyRequest = write_ops::FindAndModifyCommandRequest::parse(
         IDLParserContext("findAndModify"), ownedRequest);
 
@@ -924,12 +1003,13 @@ StatusWith<std::pair<ReplyType, OpMsgRequest>> processFindAndModifyRequest(
     auto findAndModifyBlock = std::make_tuple(ownedFindAndModifyRequest, expCtx);
     auto sharedFindAndModifyBlock =
         std::make_shared<decltype(findAndModifyBlock)>(findAndModifyBlock);
+    auto service = opCtx->getService();
 
     auto swResult = trun->runNoThrow(
         opCtx,
-        [sharedFindAndModifyBlock, ownedRequest, reply, processCallback](
+        [service, sharedFindAndModifyBlock, ownedRequest, reply, processCallback](
             const txn_api::TransactionClient& txnClient, ExecutorPtr txnExec) {
-            FLEQueryInterfaceImpl queryImpl(txnClient, getGlobalServiceContext());
+            FLEQueryInterfaceImpl queryImpl(txnClient, service);
 
             auto [findAndModifyRequest2, expCtx] = *sharedFindAndModifyBlock.get();
 
@@ -952,15 +1032,8 @@ StatusWith<std::pair<ReplyType, OpMsgRequest>> processFindAndModifyRequest(
         return swResult.getStatus();
     } else if (!swResult.getValue().getEffectiveStatus().isOK()) {
         auto& commitResult = swResult.getValue();
-
-        if (commitResult.cmdStatus.isOK()) {
-            // commit encountered a write concern error, but succeeded with the write
-            addWriteConcernErrorInfoToReply(commitResult.wcError, reply.get());
-        } else {
-            // commit encountered an error, and maybe a write concern error as well
-            if (commitResult.wcError.isValid(nullptr) && wceCallback) {
-                wceCallback(commitResult.wcError);
-            }
+        addWriteConcernErrorInfoToReply(commitResult.wcError, reply.get());
+        if (!commitResult.cmdStatus.isOK()) {
             return commitResult.cmdStatus;
         }
     }
@@ -973,16 +1046,14 @@ processFindAndModifyRequest<write_ops::FindAndModifyCommandReply>(
     OperationContext* opCtx,
     const write_ops::FindAndModifyCommandRequest& findAndModifyRequest,
     GetTxnCallback getTxns,
-    ProcessFindAndModifyCallback<write_ops::FindAndModifyCommandReply> processCallback,
-    ErrorWithWriteConcernErrorCallback wceCallback);
+    ProcessFindAndModifyCallback<write_ops::FindAndModifyCommandReply> processCallback);
 
 template StatusWith<std::pair<write_ops::FindAndModifyCommandRequest, OpMsgRequest>>
 processFindAndModifyRequest<write_ops::FindAndModifyCommandRequest>(
     OperationContext* opCtx,
     const write_ops::FindAndModifyCommandRequest& findAndModifyRequest,
     GetTxnCallback getTxns,
-    ProcessFindAndModifyCallback<write_ops::FindAndModifyCommandRequest> processCallback,
-    ErrorWithWriteConcernErrorCallback wceCallback);
+    ProcessFindAndModifyCallback<write_ops::FindAndModifyCommandRequest> processCallback);
 
 StatusWith<write_ops::InsertCommandReply> processInsert(
     FLEQueryInterface* queryImpl,
@@ -1037,8 +1108,8 @@ bool hasIndexedFieldsInSchema(const std::vector<EncryptedField>& fields) {
     for (const auto& field : fields) {
         if (field.getQueries().has_value()) {
             const auto& queries = field.getQueries().get();
-            if (stdx::holds_alternative<std::vector<mongo::QueryTypeConfig>>(queries)) {
-                const auto& vec = stdx::get<0>(queries);
+            if (holds_alternative<std::vector<mongo::QueryTypeConfig>>(queries)) {
+                const auto& vec = get<0>(queries);
                 if (!vec.empty()) {
                     return true;
                 }
@@ -1535,16 +1606,8 @@ FLEBatchResult processFLEFindAndModify(OperationContext* opCtx,
         return FLEBatchResult::kNotProcessed;
     }
 
-    // This callback ensures that any write concern errors are set in the reply in the event
-    // that processFindAndModifyRequest returned a non-OK status, which is then thrown.
-    auto onErrorWithWCE = [&result](const WriteConcernErrorDetail& wce) {
-        if (!result.hasField("writeConcernError")) {
-            result.append("writeConcernError", wce.toBSON());
-        }
-    };
-
     auto swReply = processFindAndModifyRequest<write_ops::FindAndModifyCommandReply>(
-        opCtx, request, &getTransactionWithRetriesForMongoS, processFindAndModify, onErrorWithWCE);
+        opCtx, request, &getTransactionWithRetriesForMongoS);
 
     auto reply = uassertStatusOK(swReply).first;
 
@@ -1568,10 +1631,6 @@ BSONObj FLEQueryInterfaceImpl::getById(const NamespaceString& nss, BSONElement e
     FindCommandRequest find(nss);
     find.setFilter(BSON("_id" << element));
     find.setSingleBatch(true);
-    const auto tenantId = nss.tenantId();
-    if (tenantId && gMultitenancySupport) {
-        find.setDollarTenant(tenantId);
-    }
 
     find.setEncryptionInformation(makeEmptyProcessEncryptionInformation());
 
@@ -1592,17 +1651,16 @@ BSONObj FLEQueryInterfaceImpl::getById(const NamespaceString& nss, BSONElement e
 
 uint64_t FLEQueryInterfaceImpl::countDocuments(const NamespaceString& nss) {
     // Since count() does not work in a transaction, call count() by bypassing the transaction api
-    auto client = _serviceContext->makeClient("SEP-int-fle-crud");
+    // Allow the thread to be killable. If interrupted, the call to runCommand will fail with the
+    // interruption.
+    auto client = _service->makeClient("SEP-int-fle-crud");
+
     AlternativeClientRegion clientRegion(client);
     auto opCtx = cc().makeOperationContext();
     auto as = AuthorizationSession::get(cc());
     as->grantInternalAuthorization(opCtx.get());
 
     CountCommandRequest ccr(nss);
-    const auto tenantId = nss.tenantId();
-    if (tenantId && gMultitenancySupport) {
-        ccr.setDollarTenant(*tenantId);
-    }
     auto opMsgRequest = ccr.serialize(BSONObj());
 
     DBDirectClient directClient(opCtx.get());
@@ -1629,6 +1687,10 @@ QECountInfoQueryTypeEnum queryTypeTranslation(FLEQueryInterface::TagQueryType ty
             return QECountInfoQueryTypeEnum::Query;
         case FLEQueryInterface::TagQueryType::kCompact:
             return QECountInfoQueryTypeEnum::Compact;
+        case FLEQueryInterface::TagQueryType::kCleanup:
+            return QECountInfoQueryTypeEnum::Cleanup;
+        case FLEQueryInterface::TagQueryType::kPadding:
+            return QECountInfoQueryTypeEnum::Padding;
         default:
             uasserted(7517101, "Invalid TagQueryType value.");
     }
@@ -1641,11 +1703,6 @@ std::vector<std::vector<FLEEdgeCountInfo>> FLEQueryInterfaceImpl::getTags(
 
     GetQueryableEncryptionCountInfo getCountsCmd(nss);
 
-    const auto tenantId = nss.tenantId();
-    if (tenantId && gMultitenancySupport) {
-        getCountsCmd.setDollarTenant(tenantId);
-    }
-
     getCountsCmd.setTokens(toTagSets(tokensSets));
     getCountsCmd.setQueryType(queryTypeTranslation(type));
 
@@ -1670,11 +1727,6 @@ StatusWith<write_ops::InsertCommandReply> FLEQueryInterfaceImpl::insertDocuments
     dassert(documentCount > 0);
     insertRequest.setDocuments(std::move(objs));
 
-    const auto tenantId = nss.tenantId();
-    if (tenantId && gMultitenancySupport) {
-        insertRequest.setDollarTenant(tenantId);
-    }
-
     insertRequest.getWriteCommandRequestBase().setEncryptionInformation(
         makeEmptyProcessEncryptionInformation());
     insertRequest.getWriteCommandRequestBase().setBypassDocumentValidation(
@@ -1718,10 +1770,6 @@ std::pair<write_ops::DeleteCommandReply, BSONObj> FLEQueryInterfaceImpl::deleteW
     findAndModifyRequest.setCollation(deleteOpEntry.getCollation());
     findAndModifyRequest.setLet(deleteRequest.getLet());
     findAndModifyRequest.setStmtId(deleteRequest.getStmtId());
-    const auto tenantId = nss.tenantId();
-    if (tenantId && gMultitenancySupport) {
-        findAndModifyRequest.setDollarTenant(tenantId);
-    }
 
     auto ei2 = ei;
     ei2.setCrudProcessed(true);
@@ -1789,9 +1837,6 @@ std::pair<write_ops::UpdateCommandReply, BSONObj> FLEQueryInterfaceImpl::updateW
     findAndModifyRequest.setStmtId(updateRequest.getStmtId());
     findAndModifyRequest.setBypassDocumentValidation(updateRequest.getBypassDocumentValidation());
 
-    if (nss.tenantId() && gMultitenancySupport) {
-        findAndModifyRequest.setDollarTenant(nss.tenantId());
-    }
     auto ei2 = ei;
     ei2.setCrudProcessed(true);
     findAndModifyRequest.setEncryptionInformation(ei2);
@@ -1878,10 +1923,6 @@ write_ops::FindAndModifyCommandReply FLEQueryInterfaceImpl::findAndModify(
 std::vector<BSONObj> FLEQueryInterfaceImpl::findDocuments(const NamespaceString& nss,
                                                           BSONObj filter) {
     FindCommandRequest find(nss);
-    const auto tenantId = nss.tenantId();
-    if (tenantId && gMultitenancySupport) {
-        find.setDollarTenant(tenantId);
-    }
     find.setFilter(filter);
 
     find.setEncryptionInformation(makeEmptyProcessEncryptionInformation());
@@ -1931,25 +1972,39 @@ std::vector<std::vector<FLEEdgeCountInfo>> FLETagNoTXNQuery::getTags(
     invariant(!_opCtx->inMultiDocumentTransaction());
 
     // Pop off the current op context so we can get a fresh set of read concern settings
-    auto client = _opCtx->getServiceContext()->makeClient("FLETagNoTXNQuery");
+    // Allow the thread to be killable. If interrupted, the call to runCommand will fail with the
+    // interruption.
+    auto client = _opCtx->getService()->makeClient("FLETagNoTXNQuery");
+
     AlternativeClientRegion clientRegion(client);
     auto opCtx = cc().makeOperationContext();
     auto as = AuthorizationSession::get(cc());
     as->grantInternalAuthorization(opCtx.get());
 
-    GetQueryableEncryptionCountInfo getCountsCmd(nss);
+    const auto setDollarTenant = nss.tenantId() && gMultitenancySupport;
+    const auto vts = auth::ValidatedTenancyScope::get(_opCtx);
 
-    const auto tenantId = nss.tenantId();
-    if (tenantId && gMultitenancySupport) {
-        getCountsCmd.setDollarTenant(tenantId);
-    }
+    // We need to instruct the request object (via serialization context passed in when constructing
+    // getCountsCmd) that we do not ALSO prefix the $db field when serialize() is later called since
+    // we will already be setting the unsigned security token with default tenant protocol below.
+    // Providing both a tenant prefix and a default tenant protocol is unsupported and can lead to
+    // namespace errors.
+    auto sc = SerializationContext::stateCommandRequest(
+        setDollarTenant, vts != boost::none && vts->isFromAtlasProxy());
+
+    GetQueryableEncryptionCountInfo getCountsCmd(nss, sc);
 
     getCountsCmd.setTokens(toTagSets(tokensSets));
     getCountsCmd.setQueryType(queryTypeTranslation(type));
 
     DBDirectClient directClient(opCtx.get());
 
-    auto uniqueReply = directClient.runCommand(getCountsCmd.serialize({}));
+    auto request = getCountsCmd.serialize({});
+    if (vts) {
+        request.validatedTenancyScope = *vts;
+    }
+
+    auto uniqueReply = directClient.runCommand(request);
 
     auto response = uniqueReply->getCommandReply();
 
