@@ -30,24 +30,42 @@
 #pragma once
 
 #include <boost/smart_ptr/intrusive_ptr.hpp>
+// IWYU pragma: no_include "boost/container/detail/std_fwd.hpp"
+#include <boost/optional/optional.hpp>
 #include <cstdint>
+#include <functional>
+#include <memory>
+#include <utility>
+#include <vector>
 
+#include "mongo/base/status_with.h"
+#include "mongo/bson/bsonelement.h"
 #include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/bson/oid.h"
+#include "mongo/crypto/encryption_fields_gen.h"
 #include "mongo/crypto/fle_crypto.h"
+#include "mongo/crypto/fle_crypto_types.h"
 #include "mongo/crypto/fle_field_schema_gen.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/ops/write_ops_gen.h"
+#include "mongo/db/pipeline/aggregation_request_helper.h"
+#include "mongo/db/pipeline/expression_context.h"
+#include "mongo/db/pipeline/legacy_runtime_constants_gen.h"
 #include "mongo/db/pipeline/pipeline.h"
 #include "mongo/db/query/count_command_gen.h"
+#include "mongo/db/query/find_command.h"
 #include "mongo/db/server_options.h"
 #include "mongo/db/service_context.h"
+#include "mongo/db/session/logical_session_id.h"
 #include "mongo/db/transaction/transaction_api.h"
 #include "mongo/executor/inline_executor.h"
 #include "mongo/rpc/op_msg.h"
 #include "mongo/s/write_ops/batch_write_exec.h"
+#include "mongo/s/write_ops/batched_command_request.h"
 #include "mongo/s/write_ops/batched_command_response.h"
+#include "mongo/util/intrusive_counter.h"
 
 namespace mongo {
 class OperationContext;
@@ -157,6 +175,13 @@ std::pair<write_ops::FindAndModifyCommandRequest, OpMsgRequest>
 processFLEFindAndModifyExplainMongos(
     OperationContext* opCtx, const write_ops::FindAndModifyCommandRequest& findAndModifyRequest);
 
+/**
+ * Process a findAndModify request from a replica set.
+ */
+StatusWith<std::pair<write_ops::FindAndModifyCommandReply, OpMsgRequest>>
+processFLEFindAndModifyHelper(OperationContext* opCtx,
+                              const write_ops::FindAndModifyCommandRequest& findAndModifyRequest);
+
 /**
  * Process a findAndModify request from a replica set.
  */
@@ -305,9 +330,8 @@ public:
  */
 class FLEQueryInterfaceImpl : public FLEQueryInterface {
 public:
-    FLEQueryInterfaceImpl(const txn_api::TransactionClient& txnClient,
-                          ServiceContext* serviceContext)
-        : _txnClient(txnClient), _serviceContext(serviceContext) {}
+    FLEQueryInterfaceImpl(const txn_api::TransactionClient& txnClient, Service* service)
+        : _txnClient(txnClient), _service(service) {}
 
     BSONObj getById(const NamespaceString& nss, BSONElement element) final;
 
@@ -353,10 +377,43 @@ public:
 
 private:
     const txn_api::TransactionClient& _txnClient;
-    ServiceContext* _serviceContext;
+    Service* _service;
     std::shared_ptr<executor::InlineExecutor::SleepableExecutor> _executor;
 };
 
+/**
+ * FLEStateCollectionReader using an FLEQueryInterface.
+ */
+class FLEStateCollectionQueryInterfaceReader : public FLEStateCollectionReader {
+public:
+    FLEStateCollectionQueryInterfaceReader(FLEQueryInterface* iface, const NamespaceString& nss)
+        : _query(iface), _nss(nss) {}
+
+    uint64_t getDocumentCount() const override {
+        return _query->countDocuments(_nss);
+    }
+
+    BSONObj getById(PrfBlock block) const override {
+        BSONObjBuilder bob;
+        bob.appendBinData("_id"_sd, block.size(), BinDataType::BinDataGeneral, block.data());
+        auto docs = _query->findDocuments(_nss, bob.obj());
+        _stats.setRead(_stats.getRead() + 1);
+        if (docs.empty()) {
+            return {};
+        }
+        return std::move(docs[0]);
+    }
+
+    ECStats getStats() const override {
+        return _stats;
+    }
+
+private:
+    FLEQueryInterface* _query;
+    NamespaceString _nss;
+    mutable ECStats _stats;
+};
+
 /**
  * FLETagQueryInterface that does not use transaction_api.h to retrieve tags.
  */
@@ -459,32 +516,26 @@ using ProcessFindAndModifyCallback =
                             FLEQueryInterface* queryImpl,
                             const write_ops::FindAndModifyCommandRequest& findAndModifyRequest)>;
 
-// Callback function invoked when a write concern error occurs in addition to a command error.
-using ErrorWithWriteConcernErrorCallback = std::function<void(const WriteConcernErrorDetail&)>;
-
 template <typename ReplyType>
 StatusWith<std::pair<ReplyType, OpMsgRequest>> processFindAndModifyRequest(
     OperationContext* opCtx,
     const write_ops::FindAndModifyCommandRequest& findAndModifyRequest,
     GetTxnCallback getTxns,
-    ProcessFindAndModifyCallback<ReplyType> processCallback = processFindAndModify,
-    ErrorWithWriteConcernErrorCallback wceCallback = nullptr);
+    ProcessFindAndModifyCallback<ReplyType> processCallback = processFindAndModify);
 
 extern template StatusWith<std::pair<write_ops::FindAndModifyCommandReply, OpMsgRequest>>
 processFindAndModifyRequest<write_ops::FindAndModifyCommandReply>(
     OperationContext* opCtx,
     const write_ops::FindAndModifyCommandRequest& findAndModifyRequest,
     GetTxnCallback getTxns,
-    ProcessFindAndModifyCallback<write_ops::FindAndModifyCommandReply> processCallback,
-    ErrorWithWriteConcernErrorCallback wceCallback);
+    ProcessFindAndModifyCallback<write_ops::FindAndModifyCommandReply> processCallback);
 
 extern template StatusWith<std::pair<write_ops::FindAndModifyCommandRequest, OpMsgRequest>>
 processFindAndModifyRequest<write_ops::FindAndModifyCommandRequest>(
     OperationContext* opCtx,
     const write_ops::FindAndModifyCommandRequest& findAndModifyRequest,
     GetTxnCallback getTxns,
-    ProcessFindAndModifyCallback<write_ops::FindAndModifyCommandRequest> processCallback,
-    ErrorWithWriteConcernErrorCallback wceCallback);
+    ProcessFindAndModifyCallback<write_ops::FindAndModifyCommandRequest> processCallback);
 
 write_ops::UpdateCommandReply processUpdate(OperationContext* opCtx,
                                             const write_ops::UpdateCommandRequest& updateRequest,
