@@ -28,50 +28,70 @@
  */
 
 
-#include <MurmurHash3.h>
 #include <algorithm>
 #include <array>
-#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <cstring>
+#include <fmt/format.h>
+#include <iostream>
 #include <memory>
 #include <string>
-#include <unordered_map>
 #include <vector>
 
+#include <absl/container/node_hash_map.h>
+#include <boost/cstdint.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
 #include "mongo/base/data_range.h"
 #include "mongo/base/error_codes.h"
+#include "mongo/base/secure_allocator.h"
 #include "mongo/base/string_data.h"
 #include "mongo/bson/bsonelement.h"
 #include "mongo/bson/bsonmisc.h"
 #include "mongo/bson/bsonobj.h"
 #include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/bson/bsontypes.h"
+#include "mongo/bson/bsontypes_util.h"
 #include "mongo/bson/json.h"
+#include "mongo/crypto/aead_encryption.h"
 #include "mongo/crypto/encryption_fields_gen.h"
 #include "mongo/crypto/fle_crypto.h"
 #include "mongo/crypto/fle_field_schema_gen.h"
+#include "mongo/crypto/fle_stats_gen.h"
 #include "mongo/crypto/fle_tags.h"
-#include "mongo/crypto/hash_block.h"
 #include "mongo/crypto/symmetric_key.h"
+#include "mongo/db/basic_types.h"
+#include "mongo/db/catalog/clustered_collection_options_gen.h"
 #include "mongo/db/catalog/clustered_collection_util.h"
 #include "mongo/db/catalog/collection_options.h"
+#include "mongo/db/client.h"
 #include "mongo/db/fle_crud.h"
 #include "mongo/db/fle_query_interface_mock.h"
-#include "mongo/db/matcher/schema/encrypt_schema_gen.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/ops/write_ops_gen.h"
 #include "mongo/db/ops/write_ops_parsers.h"
+#include "mongo/db/query/collation/collator_interface.h"
+#include "mongo/db/query/query_knobs_gen.h"
+#include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/repl/replication_coordinator_mock.h"
 #include "mongo/db/repl/storage_interface.h"
 #include "mongo/db/repl/storage_interface_impl.h"
 #include "mongo/db/service_context.h"
 #include "mongo/db/service_context_d_test_fixture.h"
-#include "mongo/executor/network_interface_mock.h"
 #include "mongo/idl/idl_parser.h"
-#include "mongo/idl/server_parameter_test_util.h"
+#include "mongo/platform/atomic_word.h"
 #include "mongo/platform/random.h"
 #include "mongo/shell/kms_gen.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/stdx/unordered_map.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/bson_test_util.h"
+#include "mongo/unittest/framework.h"
 #include "mongo/util/assert_util.h"
+#include "mongo/util/hex.h"
+#include "mongo/util/intrusive_counter.h"
+#include "mongo/util/murmur3.h"
+#include "mongo/util/str.h"
+#include "mongo/util/time_support.h"
 #include "mongo/util/uuid.h"
 
 namespace mongo {
@@ -123,7 +143,7 @@ public:
 
     KeyMaterial getKey(const UUID& uuid) override;
     BSONObj getEncryptedKey(const UUID& uuid) override;
-    SymmetricKey& getKMSLocalKey() {
+    SymmetricKey& getKMSLocalKey() override {
         return _localKey;
     }
 
@@ -181,10 +201,8 @@ BSONObj TestKeyVault::getEncryptedKey(const UUID& uuid) {
 }
 
 UUID fieldNameToUUID(StringData field) {
-    std::array<uint8_t, UUID::kNumBytes> buf;
-
-    MurmurHash3_x86_128(field.rawData(), field.size(), 123456, buf.data());
-
+    std::array<char, UUID::kNumBytes> buf;
+    murmur3(field, 123456 /*seed*/, buf);
     return UUID::fromCDR(buf);
 }
 
@@ -205,12 +223,12 @@ int32_t getTestSeed() {
 
 class FleCrudTest : public ServiceContextMongoDTest {
 protected:
-    void setUp();
-    void tearDown();
+    void setUp() override;
+    void tearDown() override;
 
     void createCollection(const NamespaceString& ns);
 
-    void assertDocumentCounts(uint64_t edc, uint64_t esc, uint64_t ecc, uint64_t ecoc);
+    void assertDocumentCounts(uint64_t edc, uint64_t esc, uint64_t ecoc);
 
     void testValidateEncryptedFieldInfo(BSONObj obj, bool bypassValidation);
 
@@ -276,8 +294,6 @@ protected:
         NamespaceString::createNamespaceString_forTest("test.enxcol_.coll.esc");
     NamespaceString _ecocNs =
         NamespaceString::createNamespaceString_forTest("test.enxcol_.coll.ecoc");
-
-    HmacContext hmacCtx;
 };
 
 void FleCrudTest::setUp() {
@@ -439,7 +455,7 @@ EncryptedFieldConfig getTestEncryptedFieldConfig(
                         ,
             "path": "encrypted",
             "bsonType": "int",
-            "queries": {"queryType": "rangePreview", "min": 0, "max": 15, "sparsity": 1}
+            "queries": {"queryType": "range", "min": 0, "max": 15, "sparsity": 1, "trimFactor": 0}
 
         }
     ]
@@ -451,15 +467,11 @@ EncryptedFieldConfig getTestEncryptedFieldConfig(
     return EncryptedFieldConfig::parse(IDLParserContext("root"), fromjson(rangeSchemaV2));
 }
 
-void parseEncryptedInvalidFieldConfig(StringData esc, StringData ecc, StringData ecoc) {
+void parseEncryptedInvalidFieldConfig(StringData esc, StringData ecoc) {
 
     auto invalidCollectionNameSchema =
         // "{" +
-        fmt::format(
-            "{{\"escCollection\": \"{}\", \"eccCollection\": \"{}\", \"ecocCollection\": \"{}\", ",
-            esc,
-            ecc,
-            ecoc) +
+        fmt::format("{{\"escCollection\": \"{}\", \"ecocCollection\": \"{}\", ", esc, ecoc) +
         R"(
         "fields": [
             {
@@ -470,7 +482,7 @@ void parseEncryptedInvalidFieldConfig(StringData esc, StringData ecc, StringData
                             ,
                 "path": "encrypted",
                 "bsonType": "int",
-                "queries": {"queryType": "rangePreview", "min": 0, "max": 15, "sparsity": 1}
+                "queries": {"queryType": "range", "min": 0, "max": 15, "sparsity": 1, "trimFactor": 0}
 
             }
         ]
@@ -479,7 +491,7 @@ void parseEncryptedInvalidFieldConfig(StringData esc, StringData ecc, StringData
     EncryptedFieldConfig::parse(IDLParserContext("root"), fromjson(invalidCollectionNameSchema));
 }
 
-void FleCrudTest::assertDocumentCounts(uint64_t edc, uint64_t esc, uint64_t ecc, uint64_t ecoc) {
+void FleCrudTest::assertDocumentCounts(uint64_t edc, uint64_t esc, uint64_t ecoc) {
     ASSERT_EQ(_queryImpl->countDocuments(_edcNs), edc);
     ASSERT_EQ(_queryImpl->countDocuments(_escNs), esc);
     ASSERT_EQ(_queryImpl->countDocuments(_ecocNs), ecoc);
@@ -520,7 +532,6 @@ void FleCrudTest::validateDocument(int id, boost::optional<BSONObj> doc, Fle2Alg
     std::cout << "Updated Doc: " << updatedDoc << std::endl;
 
     auto efc = getTestEncryptedFieldConfig(alg);
-    FLEClientCrypto::validateDocument(updatedDoc, efc, &_keyVault);
 
     // Decrypt document
     auto decryptedDoc = FLEClientCrypto::decryptDocument(updatedDoc, &_keyVault);
@@ -542,6 +553,7 @@ BSONObj generateFLE2RangeInsertSpec(BSONElement value) {
     auto upperDoc = BSON("ub" << 15);
 
     spec.setMaxBound(boost::optional<IDLAnyType>(upperDoc.firstElement()));
+    spec.setTrimFactor(0);
     auto specDoc = BSON("s" << spec.toBSON());
 
     return specDoc;
@@ -552,17 +564,15 @@ std::vector<char> generateSinglePlaceholder(BSONElement value,
                                             Fle2AlgorithmInt alg = Fle2AlgorithmInt::kEquality,
                                             int64_t cm = 0) {
     FLE2EncryptionPlaceholder ep;
-
-    // Has to be generated outside of if statements to root the
-    // value until ep is finalized as an object.
-    BSONObj temp = generateFLE2RangeInsertSpec(value);
-
     ep.setAlgorithm(alg);
     ep.setUserKeyId(userKeyId);
     ep.setIndexKeyId(indexKeyId);
     ep.setType(mongo::Fle2PlaceholderType::kInsert);
 
+    // Keep definition outside of conditional to keep it alive until serialization.
+    BSONObj temp;
     if (alg == Fle2AlgorithmInt::kRange) {
+        temp = generateFLE2RangeInsertSpec(value);
         ep.setValue(temp.firstElement());
         ep.setSparsity(1);
     } else {
@@ -760,25 +770,25 @@ private:
 
 class FleTagsTest : public FleCrudTest {
 protected:
-    void setUp() {
+    void setUp() override {
         FleCrudTest::setUp();
     }
-    void tearDown() {
+    void tearDown() override {
         FleCrudTest::tearDown();
     }
 
     std::vector<std::vector<FLEEdgeCountInfo>> getCountInfoSets(BSONObj obj, uint64_t cm = 0) {
         auto s = getTestESCDataToken(obj);
         auto d = getTestEDCDataToken(obj);
-        auto nssEsc = NamespaceString("test.enxcol_.coll.esc");
+        auto nssEsc = NamespaceString::createNamespaceString_forTest("test.enxcol_.coll.esc");
         return mongo::fle::getCountInfoSets(_queryImpl.get(), nssEsc, s, d, cm);
     }
 
     std::vector<PrfBlock> readTags(BSONObj obj, uint64_t cm = 0) {
         auto s = getTestESCDataToken(obj);
         auto d = getTestEDCDataToken(obj);
-        auto nssEsc = NamespaceString("test.enxcol_.coll.esc");
-        auto nssEcc = NamespaceString("test.enxcol_.coll.ecc");
+        auto nssEsc = NamespaceString::createNamespaceString_forTest("test.enxcol_.coll.esc");
+        auto nssEcc = NamespaceString::createNamespaceString_forTest("test.enxcol_.coll.ecc");
 
         return mongo::fle::readTags(_queryImpl.get(), nssEsc, s, d, cm);
     }
@@ -792,13 +802,11 @@ TEST_F(FleCrudTest, InsertOne) {
 
     doSingleInsert(1, element, Fle2AlgorithmInt::kEquality);
 
-    assertDocumentCounts(1, 1, 0, 1);
+    assertDocumentCounts(1, 1, 1);
     assertECOCDocumentCountByField("encrypted", 1);
 
     ASSERT_FALSE(
-        _queryImpl
-            ->getById(_escNs,
-                      ESCCollection::generateNonAnchorId(&hmacCtx, getTestESCToken(element), 1))
+        _queryImpl->getById(_escNs, ESCCollection::generateNonAnchorId(getTestESCToken(element), 1))
             .isEmpty());
 }
 
@@ -807,7 +815,7 @@ TEST_F(FleCrudTest, InsertOneRange) {
     auto element = doc.firstElement();
 
     doSingleInsert(1, element, Fle2AlgorithmInt::kRange);
-    assertDocumentCounts(1, 5, 0, 5);
+    assertDocumentCounts(1, 5, 5);
     assertECOCDocumentCountByField("encrypted", 5);
 }
 
@@ -820,16 +828,14 @@ TEST_F(FleCrudTest, InsertTwoSame) {
     doSingleInsert(1, element, Fle2AlgorithmInt::kEquality);
     doSingleInsert(2, element, Fle2AlgorithmInt::kEquality);
 
-    assertDocumentCounts(2, 2, 0, 2);
+    assertDocumentCounts(2, 2, 2);
     assertECOCDocumentCountByField("encrypted", 2);
 
     auto escTagToken = getTestESCToken(element);
     ASSERT_FALSE(
-        _queryImpl->getById(_escNs, ESCCollection::generateNonAnchorId(&hmacCtx, escTagToken, 1))
-            .isEmpty());
+        _queryImpl->getById(_escNs, ESCCollection::generateNonAnchorId(escTagToken, 1)).isEmpty());
     ASSERT_FALSE(
-        _queryImpl->getById(_escNs, ESCCollection::generateNonAnchorId(&hmacCtx, escTagToken, 2))
-            .isEmpty());
+        _queryImpl->getById(_escNs, ESCCollection::generateNonAnchorId(escTagToken, 2)).isEmpty());
 }
 
 // Insert two documents with different values
@@ -842,22 +848,20 @@ TEST_F(FleCrudTest, InsertTwoDifferent) {
                    BSON("encrypted"
                         << "topsecret"));
 
-    assertDocumentCounts(2, 2, 0, 2);
+    assertDocumentCounts(2, 2, 2);
     assertECOCDocumentCountByField("encrypted", 2);
 
     ASSERT_FALSE(
         _queryImpl
             ->getById(_escNs,
-                      ESCCollection::generateNonAnchorId(&hmacCtx,
-                                                         getTestESCToken(BSON("encrypted"
+                      ESCCollection::generateNonAnchorId(getTestESCToken(BSON("encrypted"
                                                                               << "secret")),
                                                          1))
             .isEmpty());
     ASSERT_FALSE(
         _queryImpl
             ->getById(_escNs,
-                      ESCCollection::generateNonAnchorId(&hmacCtx,
-                                                         getTestESCToken(BSON("encrypted"
+                      ESCCollection::generateNonAnchorId(getTestESCToken(BSON("encrypted"
                                                                               << "topsecret")),
                                                          1))
             .isEmpty());
@@ -872,21 +876,20 @@ TEST_F(FleCrudTest, Insert100Fields) {
     };
     doSingleWideInsert(1, fieldCount, valueGenerator);
 
-    assertDocumentCounts(1, fieldCount, 0, fieldCount);
+    assertDocumentCounts(1, fieldCount, fieldCount);
 
     for (uint64_t field = 0; field < fieldCount; field++) {
         auto fieldName = fieldNameFromInt(field);
 
         assertECOCDocumentCountByField(fieldName, 1);
 
-        ASSERT_FALSE(_queryImpl
-                         ->getById(_escNs,
-                                   ESCCollection::generateNonAnchorId(
-                                       &hmacCtx,
-                                       getTestESCToken(fieldName,
-                                                       valueGenerator(fieldNameFromInt(field), 0)),
-                                       1))
-                         .isEmpty());
+        ASSERT_FALSE(
+            _queryImpl
+                ->getById(
+                    _escNs,
+                    ESCCollection::generateNonAnchorId(
+                        getTestESCToken(fieldName, valueGenerator(fieldNameFromInt(field), 0)), 1))
+                .isEmpty());
     }
 }
 
@@ -905,7 +908,7 @@ TEST_F(FleCrudTest, Insert20Fields50Rows) {
         doSingleWideInsert(row, fieldCount, valueGenerator);
     }
 
-    assertDocumentCounts(rowCount, rowCount * fieldCount, 0, rowCount * fieldCount);
+    assertDocumentCounts(rowCount, rowCount * fieldCount, rowCount * fieldCount);
 
     for (uint64_t row = 0; row < rowCount; row++) {
         for (uint64_t field = 0; field < fieldCount; field++) {
@@ -918,7 +921,6 @@ TEST_F(FleCrudTest, Insert20Fields50Rows) {
                 _queryImpl
                     ->getById(_escNs,
                               ESCCollection::generateNonAnchorId(
-                                  &hmacCtx,
                                   getTestESCToken(fieldName,
                                                   valueGenerator(fieldNameFromInt(field), row)),
                                   count))
@@ -1027,17 +1029,15 @@ TEST_F(FleCrudTest, InsertAndDeleteOne) {
 
     doSingleInsert(1, element, Fle2AlgorithmInt::kEquality);
 
-    assertDocumentCounts(1, 1, 0, 1);
+    assertDocumentCounts(1, 1, 1);
 
     ASSERT_FALSE(
-        _queryImpl
-            ->getById(_escNs,
-                      ESCCollection::generateNonAnchorId(&hmacCtx, getTestESCToken(element), 1))
+        _queryImpl->getById(_escNs, ESCCollection::generateNonAnchorId(getTestESCToken(element), 1))
             .isEmpty());
 
     doSingleDelete(1, Fle2AlgorithmInt::kEquality);
 
-    assertDocumentCounts(0, 1, 0, 1);
+    assertDocumentCounts(0, 1, 1);
     assertECOCDocumentCountByField("encrypted", 1);
 }
 
@@ -1048,11 +1048,11 @@ TEST_F(FleCrudTest, InsertAndDeleteOneRange) {
 
     doSingleInsert(1, element, Fle2AlgorithmInt::kRange);
 
-    assertDocumentCounts(1, 5, 0, 5);
+    assertDocumentCounts(1, 5, 5);
 
     doSingleDelete(1, Fle2AlgorithmInt::kRange);
 
-    assertDocumentCounts(0, 5, 0, 5);
+    assertDocumentCounts(0, 5, 5);
     assertECOCDocumentCountByField("encrypted", 5);
 }
 
@@ -1065,18 +1065,16 @@ TEST_F(FleCrudTest, InsertTwoSameAndDeleteTwo) {
     doSingleInsert(1, element, Fle2AlgorithmInt::kEquality);
     doSingleInsert(2, element, Fle2AlgorithmInt::kEquality);
 
-    assertDocumentCounts(2, 2, 0, 2);
+    assertDocumentCounts(2, 2, 2);
 
     ASSERT_FALSE(
-        _queryImpl
-            ->getById(_escNs,
-                      ESCCollection::generateNonAnchorId(&hmacCtx, getTestESCToken(element), 1))
+        _queryImpl->getById(_escNs, ESCCollection::generateNonAnchorId(getTestESCToken(element), 1))
             .isEmpty());
 
     doSingleDelete(2, Fle2AlgorithmInt::kEquality);
     doSingleDelete(1, Fle2AlgorithmInt::kEquality);
 
-    assertDocumentCounts(0, 2, 0, 2);
+    assertDocumentCounts(0, 2, 2);
     assertECOCDocumentCountByField("encrypted", 2);
 }
 
@@ -1089,12 +1087,12 @@ TEST_F(FleCrudTest, InsertTwoDifferentAndDeleteTwo) {
                    BSON("encrypted"
                         << "topsecret"));
 
-    assertDocumentCounts(2, 2, 0, 2);
+    assertDocumentCounts(2, 2, 2);
 
     doSingleDelete(2, Fle2AlgorithmInt::kEquality);
     doSingleDelete(1, Fle2AlgorithmInt::kEquality);
 
-    assertDocumentCounts(0, 2, 0, 2);
+    assertDocumentCounts(0, 2, 2);
     assertECOCDocumentCountByField("encrypted", 2);
 }
 
@@ -1103,11 +1101,11 @@ TEST_F(FleCrudTest, InsertOneButDeleteAnother) {
     doSingleInsert(1,
                    BSON("encrypted"
                         << "secret"));
-    assertDocumentCounts(1, 1, 0, 1);
+    assertDocumentCounts(1, 1, 1);
 
     doSingleDelete(2, Fle2AlgorithmInt::kEquality);
 
-    assertDocumentCounts(1, 1, 0, 1);
+    assertDocumentCounts(1, 1, 1);
     assertECOCDocumentCountByField("encrypted", 1);
 }
 
@@ -1118,13 +1116,13 @@ TEST_F(FleCrudTest, UpdateOne) {
                    BSON("encrypted"
                         << "secret"));
 
-    assertDocumentCounts(1, 1, 0, 1);
+    assertDocumentCounts(1, 1, 1);
 
     doSingleUpdate(1,
                    BSON("encrypted"
                         << "top secret"));
 
-    assertDocumentCounts(1, 2, 0, 2);
+    assertDocumentCounts(1, 2, 2);
     assertECOCDocumentCountByField("encrypted", 2);
 
     validateDocument(1,
@@ -1141,14 +1139,14 @@ TEST_F(FleCrudTest, UpdateOneRange) {
 
     doSingleInsert(1, element, Fle2AlgorithmInt::kRange);
 
-    assertDocumentCounts(1, 5, 0, 5);
+    assertDocumentCounts(1, 5, 5);
 
     auto doc2 = BSON("encrypted" << 2);
     auto elem2 = doc2.firstElement();
 
     doSingleUpdate(1, elem2, Fle2AlgorithmInt::kRange);
 
-    assertDocumentCounts(1, 10, 0, 10);
+    assertDocumentCounts(1, 10, 10);
 
     validateDocument(1,
                      BSON("_id" << 1 << "counter" << 2 << "plainText"
@@ -1163,13 +1161,13 @@ TEST_F(FleCrudTest, UpdateOneSameValue) {
                    BSON("encrypted"
                         << "secret"));
 
-    assertDocumentCounts(1, 1, 0, 1);
+    assertDocumentCounts(1, 1, 1);
 
     doSingleUpdate(1,
                    BSON("encrypted"
                         << "secret"));
 
-    assertDocumentCounts(1, 2, 0, 2);
+    assertDocumentCounts(1, 2, 2);
     assertECOCDocumentCountByField("encrypted", 2);
 
     validateDocument(1,
@@ -1186,7 +1184,7 @@ TEST_F(FleCrudTest, UpdateOneReplace) {
                    BSON("encrypted"
                         << "secret"));
 
-    assertDocumentCounts(1, 1, 0, 1);
+    assertDocumentCounts(1, 1, 1);
 
     auto replace = BSON("encrypted"
                         << "top secret");
@@ -1206,7 +1204,7 @@ TEST_F(FleCrudTest, UpdateOneReplace) {
         Fle2AlgorithmInt::kEquality);
 
 
-    assertDocumentCounts(1, 2, 0, 2);
+    assertDocumentCounts(1, 2, 2);
     assertECOCDocumentCountByField("encrypted", 2);
 
     validateDocument(1,
@@ -1223,7 +1221,7 @@ TEST_F(FleCrudTest, UpdateOneReplaceRange) {
 
     doSingleInsert(1, element, Fle2AlgorithmInt::kRange);
 
-    assertDocumentCounts(1, 5, 0, 5);
+    assertDocumentCounts(1, 5, 5);
 
     auto replace = BSON("encrypted" << 2);
     auto buf = generateSinglePlaceholder(replace.firstElement(), Fle2AlgorithmInt::kRange);
@@ -1240,7 +1238,7 @@ TEST_F(FleCrudTest, UpdateOneReplaceRange) {
         write_ops::UpdateModification(result, write_ops::UpdateModification::ReplacementTag{}),
         Fle2AlgorithmInt::kRange);
 
-    assertDocumentCounts(1, 10, 0, 10);
+    assertDocumentCounts(1, 10, 10);
 
     validateDocument(1,
                      BSON("_id" << 1 << "plaintext"
@@ -1256,7 +1254,7 @@ TEST_F(FleCrudTest, RenameSafeContent) {
                    BSON("encrypted"
                         << "secret"));
 
-    assertDocumentCounts(1, 1, 0, 1);
+    assertDocumentCounts(1, 1, 1);
 
     BSONObjBuilder builder;
     builder.append("$inc", BSON("counter" << 1));
@@ -1273,7 +1271,7 @@ TEST_F(FleCrudTest, SetSafeContent) {
                    BSON("encrypted"
                         << "secret"));
 
-    assertDocumentCounts(1, 1, 0, 1);
+    assertDocumentCounts(1, 1, 1);
 
     BSONObjBuilder builder;
     builder.append("$inc", BSON("counter" << 1));
@@ -1296,16 +1294,10 @@ TEST_F(FleCrudTest, testValidateEncryptedFieldConfig) {
 // Test that EDCServerCollection::validateEncryptedFieldInfo throws an error when collection names
 // do not match naming rules.
 TEST_F(FleCrudTest, testValidateEncryptedFieldConfigFields) {
-    ASSERT_THROWS_CODE(parseEncryptedInvalidFieldConfig(
-                           "enxcol_.coll.esc1", "enxcol_.coll.ecc", "enxcol_.coll.ecoc"),
+    ASSERT_THROWS_CODE(parseEncryptedInvalidFieldConfig("enxcol_.coll.esc1", "enxcol_.coll.ecoc"),
                        DBException,
                        7406900);
-    ASSERT_THROWS_CODE(parseEncryptedInvalidFieldConfig(
-                           "enxcol_.coll.esc", "enxcol_.coll.ecc1", "enxcol_.coll.ecoc"),
-                       DBException,
-                       7406901);
-    ASSERT_THROWS_CODE(parseEncryptedInvalidFieldConfig(
-                           "enxcol_.coll.esc", "enxcol_.coll.ecc", "enxcol_.coll.ecoc1"),
+    ASSERT_THROWS_CODE(parseEncryptedInvalidFieldConfig("enxcol_.coll.esc", "enxcol_.coll.ecoc1"),
                        DBException,
                        7406902);
 }
@@ -1316,7 +1308,7 @@ TEST_F(FleCrudTest, FindAndModify_UpdateOne) {
                    BSON("encrypted"
                         << "secret"));
 
-    assertDocumentCounts(1, 1, 0, 1);
+    assertDocumentCounts(1, 1, 1);
 
     auto doc = BSON("encrypted"
                     << "top secret");
@@ -1336,7 +1328,7 @@ TEST_F(FleCrudTest, FindAndModify_UpdateOne) {
         write_ops::UpdateModification(result, write_ops::UpdateModification::ModifierUpdateTag{}));
     doFindAndModify(req, Fle2AlgorithmInt::kEquality);
 
-    assertDocumentCounts(1, 2, 0, 2);
+    assertDocumentCounts(1, 2, 2);
     assertECOCDocumentCountByField("encrypted", 2);
 
     validateDocument(1,
@@ -1354,7 +1346,7 @@ TEST_F(FleCrudTest, FindAndModify_UpdateOneRange) {
 
     doSingleInsert(1, firstDoc.firstElement(), Fle2AlgorithmInt::kRange);
 
-    assertDocumentCounts(1, 5, 0, 5);
+    assertDocumentCounts(1, 5, 5);
 
     auto doc = BSON("encrypted" << 2);
     auto element = doc.firstElement();
@@ -1373,7 +1365,7 @@ TEST_F(FleCrudTest, FindAndModify_UpdateOneRange) {
         write_ops::UpdateModification(result, write_ops::UpdateModification::ModifierUpdateTag{}));
     doFindAndModify(req, Fle2AlgorithmInt::kRange);
 
-    assertDocumentCounts(1, 10, 0, 10);
+    assertDocumentCounts(1, 10, 10);
     assertECOCDocumentCountByField("encrypted", 10);
 
     validateDocument(1,
@@ -1391,14 +1383,14 @@ TEST_F(FleCrudTest, FindAndModify_InsertAndDeleteOne) {
 
     doSingleInsert(1, element, Fle2AlgorithmInt::kEquality);
 
-    assertDocumentCounts(1, 1, 0, 1);
+    assertDocumentCounts(1, 1, 1);
 
     write_ops::FindAndModifyCommandRequest req(_edcNs);
     req.setQuery(BSON("_id" << 1));
     req.setRemove(true);
     doFindAndModify(req, Fle2AlgorithmInt::kEquality);
 
-    assertDocumentCounts(0, 1, 0, 1);
+    assertDocumentCounts(0, 1, 1);
     assertECOCDocumentCountByField("encrypted", 1);
 }
 
@@ -1409,7 +1401,7 @@ TEST_F(FleCrudTest, FindAndModify_RenameSafeContent) {
                    BSON("encrypted"
                         << "secret"));
 
-    assertDocumentCounts(1, 1, 0, 1);
+    assertDocumentCounts(1, 1, 1);
 
     BSONObjBuilder builder;
     builder.append("$inc", BSON("counter" << 1));
@@ -1435,7 +1427,7 @@ TEST_F(FleCrudTest, FindAndModify_SetSafeContent) {
                    BSON("encrypted"
                         << "secret"));
 
-    assertDocumentCounts(1, 1, 0, 1);
+    assertDocumentCounts(1, 1, 1);
 
     BSONObjBuilder builder;
     builder.append("$inc", BSON("counter" << 1));
