@@ -29,9 +29,23 @@
 
 #pragma once
 
+#include <cstdint>
+#include <utility>
+#include <vector>
+
+#include "mongo/base/status_with.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonobj.h"
 #include "mongo/crypto/fle_crypto.h"
+#include "mongo/crypto/fle_crypto_types.h"
+#include "mongo/crypto/fle_field_schema_gen.h"
 #include "mongo/db/fle_crud.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/ops/write_ops_gen.h"
+#include "mongo/db/repl/storage_interface.h"
 #include "mongo/db/repl/storage_interface_impl.h"
+#include "mongo/db/session/logical_session_id.h"
 
 namespace mongo {
 
@@ -39,7 +53,7 @@ class FLEQueryInterfaceMock : public FLEQueryInterface {
 public:
     FLEQueryInterfaceMock(OperationContext* opCtx, repl::StorageInterface* storage)
         : _opCtx(opCtx), _storage(storage) {}
-    ~FLEQueryInterfaceMock() = default;
+    ~FLEQueryInterfaceMock() override = default;
 
     BSONObj getById(const NamespaceString& nss, BSONElement element) final;
 
