@@ -27,22 +27,35 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <boost/filesystem/path.hpp>
+#include <fmt/format.h>
+#include <memory>
 
-#include "mongo/db/ftdc/ftdc_mongod.h"
-
-#include <boost/filesystem.hpp>
+#include <boost/optional/optional.hpp>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
 #include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/db/cluster_role.h"
+#include "mongo/db/commands.h"
+#include "mongo/db/ftdc/collector.h"
 #include "mongo/db/ftdc/constants.h"
 #include "mongo/db/ftdc/controller.h"
+#include "mongo/db/ftdc/ftdc_mongod.h"
 #include "mongo/db/ftdc/ftdc_mongod_gen.h"
+#include "mongo/db/ftdc/ftdc_mongos.h"
 #include "mongo/db/ftdc/ftdc_server.h"
 #include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/storage/storage_options.h"
-#include "mongo/transport/transport_layer_ftdc_collector.h"
+#include "mongo/rpc/op_msg.h"
+#include "mongo/s/sharding_feature_flags_gen.h"
 #include "mongo/util/assert_util.h"
+#include "mongo/util/synchronized_value.h"
 
 namespace mongo {
 
@@ -50,7 +63,8 @@ Status validateCollectionStatsNamespaces(const std::vector<std::string> value,
                                          const boost::optional<TenantId>& tenantId) {
     try {
         for (const auto& nsStr : value) {
-            NamespaceString ns(nsStr);
+            const auto ns = NamespaceStringUtil::deserialize(
+                tenantId, nsStr, SerializationContext::stateDefault());
 
             if (!ns.isValid()) {
                 return Status(ErrorCodes::BadValue,
@@ -78,11 +92,14 @@ public:
         for (const auto& nsStr : namespaces) {
 
             try {
-                NamespaceString ns(nsStr);
+                // TODO SERVER-74464 tenantId needs to be passed.
+                const auto ns =
+                    NamespaceStringUtil::parseFromStringExpectTenantIdInMultitenancyMode(nsStr);
                 auto result = CommandHelpers::runCommandDirectly(
                     opCtx,
-                    OpMsgRequest::fromDBAndBody(
-                        ns.db(),
+                    OpMsgRequestBuilder::create(
+                        auth::ValidatedTenancyScope::get(opCtx),
+                        ns.dbName(),
                         BSON("aggregate" << ns.coll() << "cursor" << BSONObj{} << "pipeline"
                                          << BSON_ARRAY(BSON("$collStats" << BSON(
                                                                 "storageStats" << BSON(
@@ -115,18 +132,24 @@ static const BSONArray pipelineObj =
                     .obj())
         .arr();
 
+static const BSONObj getParameterQueryObj =
+    BSONObjBuilder{}
+        .append("getParameter",
+                BSONObjBuilder{}.append("allParameters", true).append("setAt", "runtime").obj())
+        .obj();
+
+static const BSONObj getClusterParameterQueryObj =
+    BSONObjBuilder{}.append("getClusterParameter", "*").append("omitInFTDC", true).obj();
+
 static const BSONObj replSetGetStatusObj =
     BSONObjBuilder{}.append("replSetGetStatus", 1).append("initialSync", 0).obj();
 
-static const BSONObj getDefaultRWConcernObj =
-    BSONObjBuilder{}.append("getDefaultRWConcern", 1).append("inMemory", true).obj();
-
 repl::ReplicationCoordinator* getGlobalRC() {
     return repl::ReplicationCoordinator::get(getGlobalServiceContext());
 }
 
 bool isRepl(const repl::ReplicationCoordinator& rc) {
-    return rc.getReplicationMode() != repl::ReplicationCoordinator::modeNone;
+    return rc.getSettings().isReplSet();
 }
 
 bool isArbiter(const repl::ReplicationCoordinator& rc) {
@@ -143,43 +166,65 @@ std::unique_ptr<FTDCCollectorInterface> makeFilteredCollector(
     return std::make_unique<FilteredFTDCCollector>(std::move(pred), std::move(collector));
 }
 
-void registerMongoDCollectors(FTDCController* controller) {
+void registerShardCollectors(FTDCController* controller) {
+    const auto role = ClusterRole::ShardServer;
+    registerServerCollectorsForRole(controller, role);
 
-    // These metrics are only collected if replication is enabled
     if (auto rc = getGlobalRC(); rc && isRepl(*rc)) {
         // CmdReplSetGetStatus
-        controller->addPeriodicCollector(std::make_unique<FTDCSimpleInternalCommandCollector>(
-            "replSetGetStatus", "replSetGetStatus", "", replSetGetStatusObj));
+        controller->addPeriodicCollector(
+            std::make_unique<FTDCSimpleInternalCommandCollector>(
+                "replSetGetStatus", "replSetGetStatus", DatabaseName::kEmpty, replSetGetStatusObj),
+            role);
+
 
         // CollectionStats
-        controller->addPeriodicCollector(
-            makeFilteredCollector(isDataStoringNode,
-                                  std::make_unique<FTDCSimpleInternalCommandCollector>(
-                                      "aggregate",
-                                      "local.oplog.rs.stats",
-                                      "local",
-                                      BSONObjBuilder{}
-                                          .append("aggregate", "oplog.rs")
-                                          .append("cursor", BSONObj{})
-                                          .append("pipeline", pipelineObj)
-                                          .obj())));
+        const struct {
+            StringData stat;
+            StringData coll;
+            const DatabaseName& db;
+        } specs[]{
+            {"local.oplog.rs.stats"_sd, "oplog.rs"_sd, DatabaseName::kLocal},
+            {"config.transactions.stats"_sd, "transactions"_sd, DatabaseName::kConfig},
+            {"config.image_collection.stats"_sd, "image_collection"_sd, DatabaseName::kConfig},
+        };
 
-        if (!serverGlobalParams.clusterRole.exclusivelyHasShardRole()) {
-            // GetDefaultRWConcern
-            controller->addOnRotateCollector(std::make_unique<FTDCSimpleInternalCommandCollector>(
-                "getDefaultRWConcern", "getDefaultRWConcern", "", getDefaultRWConcernObj));
+        for (const auto& spec : specs) {
+            controller->addPeriodicCollector(
+                makeFilteredCollector(isDataStoringNode,
+                                      std::make_unique<FTDCSimpleInternalCommandCollector>(
+                                          "aggregate",
+                                          spec.stat,
+                                          spec.db,
+                                          BSONObjBuilder{}
+                                              .append("aggregate", spec.coll)
+                                              .append("cursor", BSONObj{})
+                                              .append("pipeline", pipelineObj)
+                                              .obj())),
+                role);
         }
     }
 
-    controller->addPeriodicCollector(
-        makeFilteredCollector(isDataStoringNode, std::make_unique<FTDCCollectionStatsCollector>()));
+    controller->addPeriodicMetadataCollector(
+        std::make_unique<FTDCSimpleInternalCommandCollector>(
+            "getParameter", "getParameter", DatabaseName::kEmpty, getParameterQueryObj),
+        role);
 
-    controller->addPeriodicCollector(std::make_unique<transport::TransportLayerFTDCCollector>());
+    controller->addPeriodicMetadataCollector(
+        std::make_unique<FTDCSimpleInternalCommandCollector>("getClusterParameter",
+                                                             "getClusterParameter",
+                                                             DatabaseName::kEmpty,
+                                                             getClusterParameterQueryObj),
+        role);
+
+    controller->addPeriodicCollector(
+        makeFilteredCollector(isDataStoringNode, std::make_unique<FTDCCollectionStatsCollector>()),
+        role);
 }
 
 }  // namespace
 
-void startMongoDFTDC() {
+void startMongoDFTDC(ServiceContext* serviceContext) {
     auto dir = getFTDCDirectoryPathParameter();
 
     if (dir.empty()) {
@@ -187,7 +232,23 @@ void startMongoDFTDC() {
         dir /= kFTDCDefaultDirectory.toString();
     }
 
-    startFTDC(dir, FTDCStartMode::kStart, registerMongoDCollectors);
+    std::vector<RegisterCollectorsFunction> registerFns{
+        registerShardCollectors,
+    };
+
+    // (Ignore FCV check): this feature flag is not FCV-gated.
+    const bool multiServiceFTDCSchema =
+        feature_flags::gMultiServiceLogAndFTDCFormat.isEnabledAndIgnoreFCVUnsafe();
+
+    const UseMultiServiceSchema multiversionSchema{
+        serviceContext->getService(ClusterRole::RouterServer) && multiServiceFTDCSchema};
+
+    if (multiversionSchema) {
+        registerFns.emplace_back(registerRouterCollectors);
+    }
+
+    startFTDC(
+        serviceContext, dir, FTDCStartMode::kStart, std::move(registerFns), multiversionSchema);
 }
 
 void stopMongoDFTDC() {
