@@ -29,9 +29,14 @@
 
 #pragma once
 
+#include <boost/filesystem/path.hpp>
+#include <cstdint>
 #include <functional>
 #include <string>
 
+#include "mongo/base/status.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonobj.h"
 #include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/db/commands.h"
 #include "mongo/db/ftdc/collector.h"
@@ -39,6 +44,8 @@
 #include "mongo/db/ftdc/controller.h"
 #include "mongo/db/jsobj.h"
 #include "mongo/db/operation_context.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/rpc/op_msg.h"
 
 namespace mongo {
 
@@ -64,14 +71,16 @@ enum class FTDCStartMode {
 };
 
 /**
- * Start Full Time Data Capture
+ * Start Full Time Data Capture.
  * Starts 1 thread.
  *
  * See MongoD and MongoS specific functions.
  */
-void startFTDC(boost::filesystem::path& path,
+void startFTDC(ServiceContext* serviceContext,
+               boost::filesystem::path& path,
                FTDCStartMode startupMode,
-               RegisterCollectorsFunction registerCollectors);
+               std::vector<RegisterCollectorsFunction> registerCollectorsFns,
+               UseMultiServiceSchema multiServiceSchema);
 
 /**
  * Stop Full Time Data Capture
@@ -80,6 +89,11 @@ void startFTDC(boost::filesystem::path& path,
  */
 void stopFTDC();
 
+/**
+ * Register collectors wanted by all server roles.
+ */
+void registerServerCollectorsForRole(FTDCController* controller, ClusterRole clusterRole);
+
 /**
  * A simple FTDC Collector that runs Commands.
  */
@@ -87,7 +101,7 @@ class FTDCSimpleInternalCommandCollector : public FTDCCollectorInterface {
 public:
     FTDCSimpleInternalCommandCollector(StringData command,
                                        StringData name,
-                                       StringData ns,
+                                       const DatabaseName& db,
                                        BSONObj cmdObj);
 
     void collect(OperationContext* opCtx, BSONObjBuilder& builder) override;
@@ -106,6 +120,7 @@ private:
 struct FTDCStartupParams {
     AtomicWord<bool> enabled;
     AtomicWord<int> periodMillis;
+    AtomicWord<int> metadataCaptureFrequency;
 
     AtomicWord<int> maxDirectorySizeMB;
     AtomicWord<int> maxFileSizeMB;
@@ -115,6 +130,7 @@ struct FTDCStartupParams {
     FTDCStartupParams()
         : enabled(FTDCConfig::kEnabledDefault),
           periodMillis(FTDCConfig::kPeriodMillisDefault),
+          metadataCaptureFrequency(FTDCConfig::kMetadataCaptureFrequencyDefault),
           // Scale the values down since are defaults are in bytes, but the user interface is MB
           maxDirectorySizeMB(FTDCConfig::kMaxDirectorySizeBytesDefault / (1024 * 1024)),
           maxFileSizeMB(FTDCConfig::kMaxFileSizeBytesDefault / (1024 * 1024)),
@@ -129,6 +145,7 @@ extern FTDCStartupParams ftdcStartupParams;
  */
 Status onUpdateFTDCEnabled(bool value);
 Status onUpdateFTDCPeriod(std::int32_t value);
+Status onUpdateFTDCMetadataCaptureFrequency(std::int32_t value);
 Status onUpdateFTDCDirectorySize(std::int32_t value);
 Status onUpdateFTDCFileSize(std::int32_t value);
 Status onUpdateFTDCSamplesPerChunk(std::int32_t value);
