@@ -27,21 +27,37 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <absl/container/node_hash_map.h>
+#include <array>
+#include <boost/container/vector.hpp>
+#include <cstddef>
+#include <cstdint>
+#include <map>
+#include <utility>
+#include <vector>
 
-#include <MurmurHash3.h>
+#include <boost/optional/optional.hpp>
 
-#include "mongo/base/init.h"
+#include "mongo/base/error_codes.h"
+#include "mongo/base/init.h"  // IWYU pragma: keep
+#include "mongo/base/initializer.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonelement_comparator_interface.h"
 #include "mongo/db/bson/dotted_path_support.h"
 #include "mongo/db/fts/fts_index_format.h"
 #include "mongo/db/fts/fts_spec.h"
-#include "mongo/db/server_options.h"
+#include "mongo/db/index/multikey_paths.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/hex.h"
 #include "mongo/util/md5.hpp"
+#include "mongo/util/murmur3.h"
 #include "mongo/util/str.h"
 
 namespace mongo {
 
+MONGO_FAIL_POINT_DEFINE(enableCompoundTextIndexes);
+
 namespace fts {
 
 using std::string;
@@ -53,24 +69,18 @@ namespace {
 BSONObj nullObj;
 BSONElement nullElt;
 
-// New in textIndexVersion 2.
-// If the term is longer than 32 characters, it may
-// result in the generated key being too large
-// for the index. In that case, we generate a 64-character key
-// from the concatenation of the first 32 characters
-// and the hex string of the murmur3 hash value of the entire
-// term value.
+// New in textIndexVersion 2. If the term is longer than 32 characters, it may result in the
+// generated key being too large for the index. In that case, we generate a 64-character key from
+// the concatenation of the first 32 characters and the hex string of the murmur3 hash value of the
+// entire term value.
 const size_t termKeyPrefixLengthV2 = 32U;
 // 128-bit hash value expressed in hex = 32 characters
 const size_t termKeySuffixLengthV2 = 32U;
 const size_t termKeyLengthV2 = termKeyPrefixLengthV2 + termKeySuffixLengthV2;
 
-// TextIndexVersion 3.
-// If the term is longer than 256 characters, it may
-// result in the generated key being too large
-// for the index. In that case, we generate a 256-character key
-// from the concatenation of the first 224 characters
-// and the hex string of the md5 hash value of the entire
+// TextIndexVersion 3. If the term is longer than 256 characters, it may result in the generated key
+// being too large for the index. In that case, we generate a 256-character key from the
+// concatenation of the first 224 characters and the hex string of the md5 hash value of the entire
 // term value.
 const size_t termKeyPrefixLengthV3 = 224U;
 // 128-bit hash value expressed in hex = 32 characters
@@ -91,6 +101,10 @@ BSONElement extractNonFTSKeyElement(const BSONObj& obj, StringData path) {
     MultikeyComponents arrayComponents;
     dps::extractAllElementsAlongPath(
         obj, path, indexedElements, expandArrayOnTrailingField, &arrayComponents);
+
+    if (MONGO_unlikely(enableCompoundTextIndexes.shouldFail())) {
+        return nullElt;
+    }
     uassert(ErrorCodes::CannotBuildIndexKeys,
             str::stream() << "Field '" << path
                           << "' of text index contains an array in document: " << obj,
@@ -113,7 +127,7 @@ void FTSIndexFormat::getKeys(SharedBufferFragmentBuilder& pooledBufferBuilder,
                              const FTSSpec& spec,
                              const BSONObj& obj,
                              KeyStringSet* keys,
-                             KeyString::Version keyStringVersion,
+                             key_string::Version keyStringVersion,
                              Ordering ordering,
                              const boost::optional<RecordId>& id) {
     vector<BSONElement> extrasBefore;
@@ -139,7 +153,7 @@ void FTSIndexFormat::getKeys(SharedBufferFragmentBuilder& pooledBufferBuilder,
         const string& term = i->first;
         double weight = i->second;
 
-        KeyString::PooledBuilder keyString(pooledBufferBuilder, keyStringVersion, ordering);
+        key_string::PooledBuilder keyString(pooledBufferBuilder, keyStringVersion, ordering);
         for (const auto& elem : extrasBefore) {
             keyString.appendBSONElement(elem);
         }
@@ -168,9 +182,9 @@ BSONObj FTSIndexFormat::getIndexKey(double weight,
         b.appendAs(i.next(), "");
     }
 
-    KeyString::Builder keyString(KeyString::Version::kLatestVersion, KeyString::ALL_ASCENDING);
+    key_string::Builder keyString(key_string::Version::kLatestVersion, key_string::ALL_ASCENDING);
     _appendIndexKey(keyString, weight, term, textIndexVersion);
-    auto key = KeyString::toBson(keyString, KeyString::ALL_ASCENDING);
+    auto key = key_string::toBson(keyString, key_string::ALL_ASCENDING);
 
     return b.appendElements(key).obj();
 }
@@ -191,13 +205,10 @@ void FTSIndexFormat::_appendIndexKey(KeyStringBuilder& keyString,
         if (term.size() <= termKeyPrefixLengthV2) {
             keyString.appendString(term);
         } else {
-            union {
-                uint64_t hash[2];
-                char data[16];
-            } t;
+            std::array<char, 16> hash;
             uint32_t seed = 0;
-            MurmurHash3_x64_128(term.data(), term.size(), seed, t.hash);
-            string keySuffix = hexblob::encodeLower(t.data, sizeof(t.data));
+            murmur3(StringData{term}, seed, hash);
+            string keySuffix = hexblob::encodeLower(hash.data(), hash.size());
             invariant(termKeySuffixLengthV2 == keySuffix.size());
             keyString.appendString(term.substr(0, termKeyPrefixLengthV2) + keySuffix);
         }
