@@ -29,9 +29,14 @@
 
 #pragma once
 
+#include <boost/core/noncopyable.hpp>
 #include <boost/noncopyable.hpp>
 #include <boost/scoped_ptr.hpp>
+#include <boost/smart_ptr/scoped_ptr.hpp>
+#include <memory>
 #include <queue>
+#include <string>
+#include <utility>
 #include <vector>
 
 #include "mongo/db/geo/hash.h"
