@@ -28,7 +28,26 @@
  */
 
 #include "mongo/db/geo/shapes.h"
-#include "mongo/db/jsobj.h"
+
+#include <cstdlib>
+#include <r1interval.h>
+#include <s1angle.h>
+#include <s2.h>
+#include <s2cap.h>
+#include <s2cell.h>
+#include <s2latlng.h>
+#include <s2polygon.h>
+#include <s2polyline.h>
+#include <util/math/vector2-inl.h>
+#include <util/math/vector2.h>
+
+#include <s2cellid.h>
+#include <util/math/vector3-inl.h>
+
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/util/builder.h"
+#include "mongo/bson/util/builder_fwd.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/str.h"
 
 using std::abs;
@@ -456,6 +475,7 @@ std::unique_ptr<LineWithCRS> LineWithCRS::clone() const {
     cloned->crs = crs;
 
     std::vector<S2Point> vertices;
+    vertices.reserve(line.num_vertices());
     for (int i = 0; i < line.num_vertices(); ++i) {
         vertices.emplace_back(line.vertex(i));
     }
