@@ -28,16 +28,38 @@
  */
 
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/index/column_key_generator.h"
-
-#include <iomanip>
+#include <absl/container/flat_hash_map.h>
+#include <algorithm>
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <cstring>
 #include <ostream>
+#include <queue>
+#include <sys/types.h>
 #include <vector>
 
+#include <absl/container/node_hash_map.h>
+#include <absl/meta/type_traits.h>
+
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/db/exec/projection_executor.h"
+#include "mongo/db/exec/projection_executor_builder.h"
+#include "mongo/db/index/column_key_generator.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/pipeline/expression_context.h"
+#include "mongo/db/pipeline/transformer_interface.h"
+#include "mongo/db/query/projection_ast.h"
+#include "mongo/db/query/projection_parser.h"
+#include "mongo/db/query/projection_policies.h"
 #include "mongo/db/storage/column_store.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/decimal_counter.h"
 #include "mongo/util/functional.h"
+#include "mongo/util/intrusive_counter.h"
+#include "mongo/util/itoa.h"
+#include "mongo/util/scopeguard.h"
 #include "mongo/util/string_map.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kIndex
@@ -52,7 +74,7 @@ ColumnKeyGenerator::ColumnKeyGenerator(BSONObj keyPattern, BSONObj pathProjectio
 
 ColumnStoreProjection ColumnKeyGenerator::createProjectionExecutor(BSONObj keyPattern,
                                                                    BSONObj pathProjection) {
-    auto expCtx = make_intrusive<ExpressionContext>(nullptr, nullptr, NamespaceString());
+    auto expCtx = make_intrusive<ExpressionContext>(nullptr, nullptr, NamespaceString::kEmpty);
     auto projection = getASTProjection(keyPattern, pathProjection);
     auto policies = ProjectionPolicies::columnStoreIndexSpecProjectionPolicies();
     return ColumnStoreProjection{projection_executor::buildProjectionExecutor(
@@ -89,7 +111,7 @@ projection_ast::Projection ColumnKeyGenerator::getASTProjection(BSONObj keyPatte
     // ExpressionContext's OperationContext and CollatorInterface to 'nullptr' and the namespace
     // string to '' here; since we ban computed fields from the projection, the ExpressionContext
     // will never be used.
-    auto expCtx = make_intrusive<ExpressionContext>(nullptr, nullptr, NamespaceString());
+    auto expCtx = make_intrusive<ExpressionContext>(nullptr, nullptr, NamespaceString::kEmpty);
     auto policies = ProjectionPolicies::columnStoreIndexSpecProjectionPolicies();
     auto projection = projection_ast::parseAndAnalyze(expCtx, projSpec, policies);
     return projection;
@@ -586,7 +608,8 @@ private:
         }
 
         // Now put the rest of the new array info into the output buffer.
-        rcd.arrayInfoBuf += StringData(newIt, newPosition.end());
+        if (auto d = std::distance(newIt, newPosition.end()))
+            rcd.arrayInfoBuf += StringData(&*newIt, d);
         rcd.arrayInfoBuf += finalByte;
     }
 
@@ -682,7 +705,7 @@ private:
         static_assert(ColumnStore::kRowIdPath == "\xFF"_sd);
         uassert(6519200,
                 "Field name contains '\\xFF' which isn't valid in UTF-8",
-                fieldName[0] != '\xFF');  // We know field names are nul-terminated so this is safe.
+                !fieldName.starts_with('\xFF'));
 
         // Fields with dots are not illegal, but we skip them because a) the query optimizer can't
         // correctly track dependencies with them, and b) the current storage format uses dots as a
