@@ -27,18 +27,39 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/index/index_access_method.h"
-
+#include <boost/container/container_fwd.hpp>
+#include <fmt/format.h>
+// IWYU pragma: no_include "boost/container/detail/std_fwd.hpp"
+#include <boost/container/flat_set.hpp>
+#include <boost/container/small_vector.hpp>
+#include <boost/container/vector.hpp>
+#include <boost/cstdint.hpp>
+#include <boost/move/algo/move.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+// IWYU pragma: no_include "boost/intrusive/detail/iterator.hpp"
+// IWYU pragma: no_include "boost/move/algo/detail/set_difference.hpp"
+#include <algorithm>
+#include <deque>
+#include <exception>
+#include <iterator>
+#include <new>
+#include <string>
+#include <system_error>
+#include <tuple>
 #include <utility>
 #include <vector>
 
+#include <boost/optional/optional.hpp>
+
 #include "mongo/base/error_codes.h"
 #include "mongo/base/status.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/db/catalog/collection.h"
+#include "mongo/db/catalog/collection_options.h"
 #include "mongo/db/catalog/index_catalog.h"
-#include "mongo/db/catalog/index_consistency.h"
-#include "mongo/db/client.h"
+#include "mongo/db/catalog/validate_results.h"
 #include "mongo/db/commands/server_status.h"
 #include "mongo/db/concurrency/exception_util.h"
 #include "mongo/db/curop.h"
@@ -48,23 +69,40 @@
 #include "mongo/db/index/columns_access_method.h"
 #include "mongo/db/index/fts_access_method.h"
 #include "mongo/db/index/hash_access_method.h"
+#include "mongo/db/index/index_access_method.h"
 #include "mongo/db/index/index_build_interceptor.h"
 #include "mongo/db/index/index_descriptor.h"
 #include "mongo/db/index/s2_access_method.h"
 #include "mongo/db/index/s2_bucket_access_method.h"
 #include "mongo/db/index/wildcard_access_method.h"
-#include "mongo/db/jsobj.h"
-#include "mongo/db/keypattern.h"
+#include "mongo/db/index_names.h"
+#include "mongo/db/matcher/expression.h"
 #include "mongo/db/operation_context.h"
-#include "mongo/db/repl/timestamp_block.h"
+#include "mongo/db/service_context.h"
+#include "mongo/db/sorter/sorter.h"
+#include "mongo/db/sorter/sorter_gen.h"
 #include "mongo/db/storage/execution_context.h"
+#include "mongo/db/storage/index_entry_comparison.h"
+#include "mongo/db/storage/key_format.h"
 #include "mongo/db/storage/kv/kv_engine.h"
+#include "mongo/db/storage/recovery_unit.h"
+#include "mongo/db/storage/storage_engine.h"
 #include "mongo/db/storage/storage_options.h"
+#include "mongo/db/transaction_resources.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/redaction.h"
 #include "mongo/platform/atomic_word.h"
-#include "mongo/util/progress_meter.h"
-#include "mongo/util/scopeguard.h"
+#include "mongo/platform/random.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/bufreader.h"
+#include "mongo/util/database_name_util.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/fail_point.h"
+#include "mongo/util/namespace_string_util.h"
 #include "mongo/util/stacktrace.h"
+#include "mongo/util/str.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
 
@@ -116,10 +154,7 @@ std::unique_ptr<IndexAccessMethod> IndexAccessMethod::make(
         return std::make_unique<WildcardAccessMethod>(entry, makeSDI());
     else if (IndexNames::COLUMN == type)
         return std::make_unique<ColumnStoreAccessMethod>(entry, makeCS());
-    LOGV2(20688,
-          "Can't find index for keyPattern {keyPattern}",
-          "Can't find index for keyPattern",
-          "keyPattern"_attr = desc->keyPattern());
+    LOGV2(20688, "Can't find index for keyPattern", "keyPattern"_attr = desc->keyPattern());
     fassertFailed(31021);
 }
 
@@ -137,7 +172,7 @@ namespace {
  */
 class IndexBulkBuilderSSS : public ServerStatusSection {
 public:
-    IndexBulkBuilderSSS() : ServerStatusSection("indexBulkBuilder") {}
+    using ServerStatusSection::ServerStatusSection;
 
     bool includeByDefault() const final {
         return true;
@@ -174,8 +209,10 @@ public:
     // the external sorter and may be useful in diagnosing situations where the process is
     // close to exhausting this finite resource.
     SorterFileStats sorterFileStats = {&sorterTracker};
+};
 
-} indexBulkBuilderSSS;
+auto& indexBulkBuilderSSS =
+    *ServerStatusSectionBuilder<IndexBulkBuilderSSS>("indexBulkBuilder").forShard();
 
 /**
  * Returns true if at least one prefix of any of the indexed fields causes the index to be
@@ -188,7 +225,9 @@ bool isMultikeyFromPaths(const MultikeyPaths& multikeyPaths) {
                        [](const MultikeyComponents& components) { return !components.empty(); });
 }
 
-SortOptions makeSortOptions(size_t maxMemoryUsageBytes, StringData dbName, SorterFileStats* stats) {
+SortOptions makeSortOptions(size_t maxMemoryUsageBytes,
+                            const DatabaseName& dbName,
+                            SorterFileStats* stats) {
     return SortOptions()
         .TempDir(storageGlobalParams.dbpath + "/_tmp")
         .ExtSortAllowed()
@@ -196,7 +235,7 @@ SortOptions makeSortOptions(size_t maxMemoryUsageBytes, StringData dbName, Sorte
         .UseMemoryPool(true)
         .FileStats(stats)
         .Tracker(&indexBulkBuilderSSS.sorterTracker)
-        .DBName(dbName.toString());
+        .DBName(dbName);
 }
 
 MultikeyPaths createMultikeyPaths(const std::vector<MultikeyPath>& multikeyPathsVec) {
@@ -213,7 +252,7 @@ MultikeyPaths createMultikeyPaths(const std::vector<MultikeyPath>& multikeyPaths
 }  // namespace
 
 struct BtreeExternalSortComparison {
-    int operator()(const KeyString::Value& l, const KeyString::Value& r) const {
+    int operator()(const key_string::Value& l, const key_string::Value& r) const {
         return l.compare(r);
     }
 };
@@ -235,7 +274,7 @@ Status SortedDataIndexAccessMethod::insert(OperationContext* opCtx,
         invariant(bsonRecord.id != RecordId());
 
         if (!bsonRecord.ts.isNull()) {
-            Status status = opCtx->recoveryUnit()->setTimestamp(bsonRecord.ts);
+            Status status = shard_role_details::getRecoveryUnit(opCtx)->setTimestamp(bsonRecord.ts);
             if (!status.isOK())
                 return status;
         }
@@ -320,7 +359,7 @@ Status SortedDataIndexAccessMethod::update(OperationContext* opCtx,
     prepareUpdate(opCtx, coll, entry, oldDoc, newDoc, loc, options, &updateTicket);
 
     auto status = Status::OK();
-    if (entry->isHybridBuilding() || !entry->isReady(opCtx)) {
+    if (entry->isHybridBuilding() || !entry->isReady()) {
         bool logIfError = false;
         _unindexKeysOrWriteToSideTable(opCtx,
                                        coll->ns(),
@@ -440,8 +479,8 @@ Status SortedDataIndexAccessMethod::insertKeys(OperationContext* opCtx,
 
 void SortedDataIndexAccessMethod::removeOneKey(OperationContext* opCtx,
                                                const IndexCatalogEntry* entry,
-                                               const KeyString::Value& keyString,
-                                               bool dupsAllowed) {
+                                               const key_string::Value& keyString,
+                                               bool dupsAllowed) const {
 
     try {
         _newInterface->unindex(opCtx, keyString, dupsAllowed);
@@ -455,8 +494,6 @@ void SortedDataIndexAccessMethod::removeOneKey(OperationContext* opCtx,
 
         NamespaceString ns = entry->getNSSFromCatalog(opCtx);
         LOGV2(20683,
-              "Assertion failure: _unindex failed on: {namespace} for index: {indexName}. "
-              "{error}  KeyString:{keyString}",
               "Assertion failure: _unindex failed",
               "error"_attr = redact(e),
               "keyString"_attr = keyString,
@@ -475,7 +512,7 @@ Status SortedDataIndexAccessMethod::removeKeys(OperationContext* opCtx,
                                                const IndexCatalogEntry* entry,
                                                const KeyStringSet& keys,
                                                const InsertDeleteOptions& options,
-                                               int64_t* numDeleted) {
+                                               int64_t* numDeleted) const {
 
     for (const auto& key : keys) {
         removeOneKey(opCtx, entry, key, options.dupsAllowed);
@@ -493,12 +530,12 @@ RecordId SortedDataIndexAccessMethod::findSingle(OperationContext* opCtx,
                                                  const CollectionPtr& collection,
                                                  const IndexCatalogEntry* entry,
                                                  const BSONObj& requestedKey) const {
-    // Generate the key for this index.
-    KeyString::Value actualKey = [&]() {
+    boost::optional<RecordId> loc = [&]() {
+        // Generate the key for this index.
         if (entry->getCollator()) {
             // For performance, call get keys only if there is a non-simple collation.
             SharedBufferFragmentBuilder pooledBuilder(
-                KeyString::HeapBuilder::kHeapAllocatorDefaultBytes);
+                key_string::HeapBuilder::kHeapAllocatorDefaultBytes);
             auto& executionCtx = StorageExecutionContext::get(opCtx);
             auto keys = executionCtx.keys();
             KeyStringSet* multikeyMetadataKeys = nullptr;
@@ -516,17 +553,25 @@ RecordId SortedDataIndexAccessMethod::findSingle(OperationContext* opCtx,
                     multikeyPaths,
                     boost::none /* loc */);
             invariant(keys->size() == 1);
-            return *keys->begin();
+            const key_string::Value& actualKey = *keys->begin();
+            dassert(key_string::decodeDiscriminator(actualKey.getBuffer(),
+                                                    actualKey.getSize(),
+                                                    getSortedDataInterface()->getOrdering(),
+                                                    actualKey.getTypeBits()) ==
+                    key_string::Discriminator::kInclusive);
+            return _newInterface->findLoc(
+                opCtx,
+                StringData(actualKey.getBuffer(),
+                           static_cast<StringData::size_type>(actualKey.getSize())));
         } else {
-            KeyString::HeapBuilder requestedKeyString(
-                getSortedDataInterface()->getKeyStringVersion(),
-                BSONObj::stripFieldNames(requestedKey),
-                getSortedDataInterface()->getOrdering());
-            return requestedKeyString.release();
+            key_string::Builder requestedKeyString(getSortedDataInterface()->getKeyStringVersion(),
+                                                   requestedKey,
+                                                   getSortedDataInterface()->getOrdering());
+            return _newInterface->findLoc(opCtx, requestedKeyString.finishAndGetBuffer());
         }
     }();
 
-    if (auto loc = _newInterface->findLoc(opCtx, actualKey)) {
+    if (loc) {
         dassert(!loc->isNull());
         return std::move(*loc);
     }
@@ -604,7 +649,7 @@ void SortedDataIndexAccessMethod::prepareUpdate(OperationContext* opCtx,
                                                 const RecordId& record,
                                                 const InsertDeleteOptions& options,
                                                 UpdateTicket* ticket) const {
-    SharedBufferFragmentBuilder pooledBuilder(KeyString::HeapBuilder::kHeapAllocatorDefaultBytes);
+    SharedBufferFragmentBuilder pooledBuilder(key_string::HeapBuilder::kHeapAllocatorDefaultBytes);
     const MatchExpression* indexFilter = entry->getFilterExpression();
     if (!indexFilter || indexFilter->matchesBSON(from)) {
         // Override key constraints when generating keys for removal. This only applies to keys
@@ -698,8 +743,9 @@ Status SortedDataIndexAccessMethod::doUpdate(OperationContext* opCtx,
     return Status::OK();
 }
 
-Status SortedDataIndexAccessMethod::compact(OperationContext* opCtx) {
-    return this->_newInterface->compact(opCtx);
+StatusWith<int64_t> SortedDataIndexAccessMethod::compact(OperationContext* opCtx,
+                                                         const CompactOptions& options) {
+    return this->_newInterface->compact(opCtx, options);
 }
 
 std::shared_ptr<Ident> SortedDataIndexAccessMethod::getSharedIdent() const {
@@ -731,12 +777,12 @@ Status SortedDataIndexAccessMethod::applyIndexBuildSideWrite(OperationContext* o
         }
     }();
 
-    // Deserialize the encoded KeyString::Value.
+    // Deserialize the encoded key_string::Value.
     int keyLen;
     const char* binKey = operation["key"].binData(keyLen);
     BufReader reader(binKey, keyLen);
-    const KeyString::Value keyString =
-        KeyString::Value::deserialize(reader, getSortedDataInterface()->getKeyStringVersion());
+    const key_string::Value keyString =
+        key_string::Value::deserialize(reader, getSortedDataInterface()->getKeyStringVersion());
 
     const KeyStringSet keySet{keyString};
     if (opType == IndexBuildInterceptor::Op::kInsert) {
@@ -755,7 +801,7 @@ Status SortedDataIndexAccessMethod::applyIndexBuildSideWrite(OperationContext* o
         }
 
         *keysInserted += numInserted;
-        opCtx->recoveryUnit()->onRollback(
+        shard_role_details::getRecoveryUnit(opCtx)->onRollback(
             [keysInserted, numInserted](OperationContext*) { *keysInserted -= numInserted; });
     } else {
         invariant(opType == IndexBuildInterceptor::Op::kDelete);
@@ -766,7 +812,7 @@ Status SortedDataIndexAccessMethod::applyIndexBuildSideWrite(OperationContext* o
         }
 
         *keysDeleted += numDeleted;
-        opCtx->recoveryUnit()->onRollback(
+        shard_role_details::getRecoveryUnit(opCtx)->onRollback(
             [keysDeleted, numDeleted](OperationContext*) { *keysDeleted -= numDeleted; });
     }
     return Status::OK();
@@ -796,10 +842,10 @@ const IndexCatalogEntry* IndexAccessMethod::BulkBuilder::yield(OperationContext*
     const std::string indexIdent = entry->getIdent();
 
     // Releasing locks means a new snapshot should be acquired when restored.
-    opCtx->recoveryUnit()->abandonSnapshot();
+    shard_role_details::getRecoveryUnit(opCtx)->abandonSnapshot();
     collection.yield();
 
-    auto locker = opCtx->lockState();
+    auto locker = shard_role_details::getLocker(opCtx);
     Locker::LockSnapshot snapshot;
     locker->saveLockStateAndUnlock(&snapshot);
 
@@ -812,7 +858,9 @@ const IndexCatalogEntry* IndexAccessMethod::BulkBuilder::yield(OperationContext*
                 LOGV2(5180600, "Hanging index build during bulk load yield");
                 fp->pauseWhileSet();
             },
-            [opCtx, &ns](auto&& config) { return config.getStringField("namespace") == ns.ns(); });
+            [opCtx, &ns](auto&& config) {
+                return NamespaceStringUtil::parseFailPointData(config, "namespace") == ns;
+            });
     };
     failPointHang(&hangDuringIndexBuildBulkLoadYield);
     failPointHang(&hangDuringIndexBuildBulkLoadYieldSecond);
@@ -835,18 +883,18 @@ const IndexCatalogEntry* IndexAccessMethod::BulkBuilder::yield(OperationContext*
 class SortedDataIndexAccessMethod::BulkBuilderImpl final
     : public BulkBuilderCommon<SortedDataIndexAccessMethod::BulkBuilderImpl> {
 public:
-    using Sorter = mongo::Sorter<KeyString::Value, mongo::NullValue>;
+    using Sorter = mongo::Sorter<key_string::Value, mongo::NullValue>;
 
     BulkBuilderImpl(const IndexCatalogEntry* entry,
                     SortedDataIndexAccessMethod* iam,
                     size_t maxMemoryUsageBytes,
-                    StringData dbName);
+                    const DatabaseName& dbName);
 
     BulkBuilderImpl(const IndexCatalogEntry* entry,
                     SortedDataIndexAccessMethod* iam,
                     size_t maxMemoryUsageBytes,
                     const IndexStateInfo& stateInfo,
-                    StringData dbName);
+                    const DatabaseName& dbName);
 
     Status insert(OperationContext* opCtx,
                   const CollectionPtr& collection,
@@ -888,7 +936,7 @@ private:
 
     Sorter* _makeSorter(
         size_t maxMemoryUsageBytes,
-        StringData dbName,
+        const DatabaseName& dbName,
         boost::optional<StringData> fileName = boost::none,
         const boost::optional<std::vector<SorterRange>>& ranges = boost::none) const;
 
@@ -897,7 +945,7 @@ private:
     SortedDataIndexAccessMethod* _iam;
     std::unique_ptr<Sorter> _sorter;
 
-    KeyString::Value _previousKey;
+    key_string::Value _previousKey;
 
     // Set to true if any document added to the BulkBuilder causes the index to become multikey.
     bool _isMultiKey = false;
@@ -916,7 +964,7 @@ std::unique_ptr<IndexAccessMethod::BulkBuilder> SortedDataIndexAccessMethod::ini
     const IndexCatalogEntry* entry,
     size_t maxMemoryUsageBytes,
     const boost::optional<IndexStateInfo>& stateInfo,
-    StringData dbName) {
+    const DatabaseName& dbName) {
     return stateInfo
         ? std::make_unique<BulkBuilderImpl>(entry, this, maxMemoryUsageBytes, *stateInfo, dbName)
         : std::make_unique<BulkBuilderImpl>(entry, this, maxMemoryUsageBytes, dbName);
@@ -925,7 +973,7 @@ std::unique_ptr<IndexAccessMethod::BulkBuilder> SortedDataIndexAccessMethod::ini
 SortedDataIndexAccessMethod::BulkBuilderImpl::BulkBuilderImpl(const IndexCatalogEntry* entry,
                                                               SortedDataIndexAccessMethod* iam,
                                                               size_t maxMemoryUsageBytes,
-                                                              StringData dbName)
+                                                              const DatabaseName& dbName)
     : BulkBuilderCommon(0,
                         "Index Build: inserting keys from external sorter into index",
                         entry->descriptor()->indexName()),
@@ -938,7 +986,7 @@ SortedDataIndexAccessMethod::BulkBuilderImpl::BulkBuilderImpl(const IndexCatalog
                                                               SortedDataIndexAccessMethod* iam,
                                                               size_t maxMemoryUsageBytes,
                                                               const IndexStateInfo& stateInfo,
-                                                              StringData dbName)
+                                                              const DatabaseName& dbName)
     : BulkBuilderCommon(stateInfo.getNumKeys().value_or(0),
                         "Index Build: inserting keys from external sorter into index",
                         entry->descriptor()->indexName()),
@@ -1039,7 +1087,7 @@ void SortedDataIndexAccessMethod::BulkBuilderImpl::_insertMultikeyMetadataKeysIn
 
 SortedDataIndexAccessMethod::BulkBuilderImpl::Sorter::Settings
 SortedDataIndexAccessMethod::BulkBuilderImpl::_makeSorterSettings() const {
-    return std::pair<KeyString::Value::SorterDeserializeSettings,
+    return std::pair<key_string::Value::SorterDeserializeSettings,
                      mongo::NullValue::SorterDeserializeSettings>(
         {_iam->getSortedDataInterface()->getKeyStringVersion()}, {});
 }
@@ -1047,7 +1095,7 @@ SortedDataIndexAccessMethod::BulkBuilderImpl::_makeSorterSettings() const {
 SortedDataIndexAccessMethod::BulkBuilderImpl::Sorter*
 SortedDataIndexAccessMethod::BulkBuilderImpl::_makeSorter(
     size_t maxMemoryUsageBytes,
-    StringData dbName,
+    const DatabaseName& dbName,
     boost::optional<StringData> fileName,
     const boost::optional<std::vector<SorterRange>>& ranges) const {
     return fileName
@@ -1062,7 +1110,7 @@ SortedDataIndexAccessMethod::BulkBuilderImpl::_makeSorter(
                        _makeSorterSettings());
 }
 
-std::unique_ptr<mongo::Sorter<KeyString::Value, mongo::NullValue>::Iterator>
+std::unique_ptr<mongo::Sorter<key_string::Value, mongo::NullValue>::Iterator>
 SortedDataIndexAccessMethod::BulkBuilderImpl::finalizeSort() {
     _insertMultikeyMetadataKeysIntoSorter();
     return std::unique_ptr<Sorter::Iterator>(_sorter->done());
@@ -1227,7 +1275,7 @@ void SortedDataIndexAccessMethod::validateDocument(const CollectionPtr& collecti
  */
 std::string nextFileName() {
     static AtomicWord<unsigned> indexAccessMethodFileCounter;
-    static const int64_t randomSuffix = SecureRandom().nextInt64();
+    static const uint64_t randomSuffix = SecureRandom().nextUInt64();
     return str::stream() << "extsort-index." << indexAccessMethodFileCounter.fetchAndAdd(1) << '-'
                          << randomSuffix;
 }
@@ -1235,16 +1283,16 @@ std::string nextFileName() {
 Status SortedDataIndexAccessMethod::_handleDuplicateKey(
     OperationContext* opCtx,
     const IndexCatalogEntry* entry,
-    const KeyString::Value& dataKey,
+    const key_string::Value& dataKey,
     const RecordIdHandlerFn& onDuplicateRecord) {
     RecordId recordId = (KeyFormat::Long == _newInterface->rsKeyFormat())
-        ? KeyString::decodeRecordIdLongAtEnd(dataKey.getBuffer(), dataKey.getSize())
-        : KeyString::decodeRecordIdStrAtEnd(dataKey.getBuffer(), dataKey.getSize());
+        ? key_string::decodeRecordIdLongAtEnd(dataKey.getBuffer(), dataKey.getSize())
+        : key_string::decodeRecordIdStrAtEnd(dataKey.getBuffer(), dataKey.getSize());
     if (onDuplicateRecord) {
         return onDuplicateRecord(recordId);
     }
 
-    BSONObj dupKey = KeyString::toBson(dataKey, getSortedDataInterface()->getOrdering());
+    BSONObj dupKey = key_string::toBson(dataKey, getSortedDataInterface()->getOrdering());
     return buildDupKeyErrorStatus(dupKey.getOwned(),
                                   entry->getNSSFromCatalog(opCtx),
                                   entry->descriptor()->indexName(),
@@ -1285,20 +1333,6 @@ Status SortedDataIndexAccessMethod::_indexKeysOrWriteToSideTable(
             *keysInsertedOut += inserted;
         }
     } else {
-        // Ensure that our snapshot is compatible with the index's minimum visibile snapshot.
-        // (Ignore FCV check): This feature flag doesn't have any upgrade/downgrade concerns.
-        if (!feature_flags::gPointInTimeCatalogLookups.isEnabledAndIgnoreFCVUnsafe()) {
-            const auto minVisibleTimestamp = entry->getMinimumVisibleSnapshot();
-            const auto readTimestamp =
-                opCtx->recoveryUnit()->getPointInTimeReadTimestamp(opCtx).value_or(
-                    opCtx->recoveryUnit()->getCatalogConflictingTimestamp());
-            if (minVisibleTimestamp && !readTimestamp.isNull() &&
-                readTimestamp < *minVisibleTimestamp) {
-                throwWriteConflictException(
-                    "Unable to read from a snapshot due to pending catalog changes.");
-            }
-        }
-
         int64_t numInserted = 0;
         status = insertKeysAndUpdateMultikeyPaths(
             opCtx,
@@ -1357,20 +1391,6 @@ void SortedDataIndexAccessMethod::_unindexKeysOrWriteToSideTable(
     // We need to disable blind-deletes if 'checkRecordId' is explicitly set 'On'.
     options.dupsAllowed = options.dupsAllowed || checkRecordId == CheckRecordId::On;
 
-    // Ensure that our snapshot is compatible with the index's minimum visibile snapshot.
-    // (Ignore FCV check): This feature flag doesn't have any upgrade/downgrade concerns.
-    if (!feature_flags::gPointInTimeCatalogLookups.isEnabledAndIgnoreFCVUnsafe()) {
-        const auto minVisibleTimestamp = entry->getMinimumVisibleSnapshot();
-        const auto readTimestamp =
-            opCtx->recoveryUnit()->getPointInTimeReadTimestamp(opCtx).value_or(
-                opCtx->recoveryUnit()->getCatalogConflictingTimestamp());
-        if (minVisibleTimestamp && !readTimestamp.isNull() &&
-            readTimestamp < *minVisibleTimestamp) {
-            throwWriteConflictException(
-                "Unable to read from a snapshot due to pending catalog changes.");
-        }
-    }
-
     int64_t removed = 0;
     Status status = removeKeys(opCtx, entry, keys, options, &removed);
 
@@ -1391,5 +1411,6 @@ void SortedDataIndexAccessMethod::_unindexKeysOrWriteToSideTable(
 
 #undef MONGO_LOGV2_DEFAULT_COMPONENT
 #include "mongo/db/sorter/sorter.cpp"
+
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
-MONGO_CREATE_SORTER(mongo::KeyString::Value, mongo::NullValue, mongo::BtreeExternalSortComparison);
+MONGO_CREATE_SORTER(mongo::key_string::Value, mongo::NullValue, mongo::BtreeExternalSortComparison);
