@@ -27,10 +27,38 @@
  *    it in the license file.
  */
 
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/bson/bsontypes_util.h"
+#include "mongo/bson/json.h"
+#include "mongo/bson/util/builder.h"
+#include "mongo/crypto/encryption_fields_gen.h"
 #include "mongo/db/catalog/catalog_test_fixture.h"
+#include "mongo/db/catalog/clustered_collection_options_gen.h"
+#include "mongo/db/catalog/collection.h"
+#include "mongo/db/catalog/index_catalog.h"
 #include "mongo/db/catalog_raii.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
 #include "mongo/db/index/index_build_interceptor.h"
+#include "mongo/db/index/index_descriptor.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/repl/storage_interface.h"
+#include "mongo/db/service_context.h"
+#include "mongo/db/storage/record_data.h"
+#include "mongo/db/storage/storage_engine.h"
+#include "mongo/db/storage/write_unit_of_work.h"
+#include "mongo/db/timeseries/timeseries_gen.h"
 #include "mongo/idl/server_parameter_test_util.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/bson_test_util.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/uuid.h"
 
 namespace mongo {
 namespace {
@@ -63,7 +91,8 @@ protected:
         return operationContext()
             ->getServiceContext()
             ->getStorageEngine()
-            ->makeTemporaryRecordStoreFromExistingIdent(operationContext(), sideWritesIdent);
+            ->makeTemporaryRecordStoreFromExistingIdent(
+                operationContext(), sideWritesIdent, KeyFormat::Long);
     }
 
     std::vector<BSONObj> getSideWritesTableContents(
@@ -103,9 +132,9 @@ TEST_F(IndexBuilderInterceptorTest, SingleInsertIsSavedToSideWritesTable) {
     auto interceptor = createIndexBuildInterceptor(fromjson("{v: 2, name: 'a_1', key: {a: 1}}"));
     const IndexDescriptor* desc = getIndexDescriptor("a_1");
 
-    KeyString::HeapBuilder ksBuilder(KeyString::Version::kLatestVersion);
+    key_string::HeapBuilder ksBuilder(key_string::Version::kLatestVersion);
     ksBuilder.appendNumberLong(10);
-    KeyString::Value keyString(ksBuilder.release());
+    key_string::Value keyString(ksBuilder.release());
 
     WriteUnitOfWork wuow(operationContext());
     int64_t numKeys = 0;
