@@ -27,13 +27,30 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <boost/smart_ptr.hpp>
+// IWYU pragma: no_include "boost/container/detail/flat_tree.hpp"
+#include <boost/container/vector.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+// IWYU pragma: no_include "ext/alloc_traits.h"
+#include <algorithm>
+#include <utility>
+#include <variant>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/db/index/multikey_paths.h"
 #include "mongo/db/index/sort_key_generator.h"
-
-#include "mongo/bson/bsonobj_comparator.h"
+#include "mongo/db/pipeline/expression.h"
 #include "mongo/db/query/collation/collation_index_key.h"
-#include "mongo/util/overloaded_visitor.h"
+#include "mongo/db/storage/snapshot.h"
+#include "mongo/util/overloaded_visitor.h"  // IWYU pragma: keep
+#include "mongo/util/shared_buffer_fragment.h"
 
 namespace mongo {
 namespace {
@@ -75,18 +92,9 @@ SortKeyGenerator::SortKeyGenerator(SortPattern sortPattern, const CollatorInterf
 
     constexpr bool isSparse = false;
     _indexKeyGen = std::make_unique<BtreeKeyGenerator>(
-        fieldNames, fixed, isSparse, KeyString::Version::kLatestVersion, _ordering);
+        fieldNames, fixed, isSparse, key_string::Version::kLatestVersion, _ordering);
 
-    {
-        // TODO SERVER-74725: Remove this.
-        std::set<std::string> fieldNameSet;
-        for (auto& fn : fieldNames) {
-            fieldNameSet.insert(fn);
-        }
-        _sortHasRepeatKey = (fieldNameSet.size() != fieldNames.size());
-    }
-
-    if (!_sortHasMeta && !_sortHasRepeatKey) {
+    if (!_sortHasMeta) {
         size_t i = 0;
         for (auto&& keyPart : _sortPattern) {
             _sortKeyTreeRoot.addSortPatternPart(&keyPart, 0, i++);
@@ -103,10 +111,10 @@ Value SortKeyGenerator::computeSortKey(const WorkingSetMember& wsm) const {
     return computeSortKeyFromIndexKey(wsm);
 }
 
-KeyString::Value SortKeyGenerator::computeSortKeyString(const BSONObj& obj) {
+key_string::Value SortKeyGenerator::computeSortKeyString(const BSONObj& obj) {
     const bool fastPathSucceeded = fastFillOutSortKeyParts(obj, &_localEltStorage);
     if (fastPathSucceeded) {
-        KeyString::HeapBuilder builder(KeyString::Version::kLatestVersion, _ordering);
+        key_string::HeapBuilder builder(key_string::Version::kLatestVersion, _ordering);
         for (auto elt : _localEltStorage) {
             if (_collator) {
                 builder.appendBSONElement(elt, [&](StringData stringData) {
@@ -121,7 +129,7 @@ KeyString::Value SortKeyGenerator::computeSortKeyString(const BSONObj& obj) {
     }
 
     KeyStringSet keySet;
-    SharedBufferFragmentBuilder allocator(KeyString::HeapBuilder::kHeapAllocatorDefaultBytes);
+    SharedBufferFragmentBuilder allocator(key_string::HeapBuilder::kHeapAllocatorDefaultBytes);
     const bool skipMultikey = false;
     MultikeyPaths* multikeyPaths = nullptr;
     _indexKeyGen->getKeys(allocator, obj, skipMultikey, &keySet, multikeyPaths, _collator);
@@ -208,7 +216,7 @@ StatusWith<BSONObj> SortKeyGenerator::computeSortKeyFromDocumentWithoutMetadata(
     // corresponding collation keys. Therefore, we use the simple string comparator when comparing
     // the keys themselves.
     KeyStringSet keys;
-    SharedBufferFragmentBuilder allocator(KeyString::HeapBuilder::kHeapAllocatorDefaultBytes);
+    SharedBufferFragmentBuilder allocator(key_string::HeapBuilder::kHeapAllocatorDefaultBytes);
 
     try {
         // There's no need to compute the prefixes of the indexed fields that cause the index to be
@@ -231,7 +239,7 @@ StatusWith<BSONObj> SortKeyGenerator::computeSortKeyFromDocumentWithoutMetadata(
     invariant(!keys.empty());
 
     // The sort key is the first index key, ordered according to the pattern '_sortSpecWithoutMeta'.
-    return KeyString::toBson(*keys.begin(), Ordering::make(_sortSpecWithoutMeta));
+    return key_string::toBson(*keys.begin(), Ordering::make(_sortSpecWithoutMeta));
 }
 
 Value SortKeyGenerator::getCollationComparisonKey(const Value& val) const {
@@ -268,36 +276,10 @@ boost::optional<Value> SortKeyGenerator::extractKeyPart(
     Value plainKey;
     if (patternPart.fieldPath) {
         invariant(!patternPart.expression);
-        auto keyVariant = doc.getNestedFieldNonCaching(*patternPart.fieldPath);
-
-        auto key = stdx::visit(
-            OverloadedVisitor{
-                // In this case, the document has an array along the path given. This means the
-                // document is ineligible for taking the fast path for index key generation.
-                [](Document::TraversesArrayTag) -> boost::optional<Value> { return boost::none; },
-                // In this case the field was already in the cache (or may not have existed).
-                [](const Value& val) -> boost::optional<Value> {
-                    // The document may have an array at the given path.
-                    if (val.getType() == BSONType::Array) {
-                        return boost::none;
-                    }
-                    return val;
-                },
-                // In this case the field was in the backing BSON, and not in the cache.
-                [](BSONElement elt) -> boost::optional<Value> {
-                    // The document may have an array at the given path.
-                    if (elt.type() == BSONType::Array) {
-                        return boost::none;
-                    }
-                    return Value(elt);
-                },
-                [](stdx::monostate none) -> boost::optional<Value> { return Value(); },
-            },
-            keyVariant);
-
-        if (!key) {
+        auto key = doc.getNestedScalarFieldNonCaching(*patternPart.fieldPath);
+        if (!key)
             return boost::none;
-        }
+
         plainKey = std::move(*key);
     } else {
         invariant(patternPart.expression);
@@ -400,6 +382,9 @@ bool SortKeyGenerator::fastFillOutSortKeyPartsHelper(const BSONObj& bson,
             }
 
             if (childNode->part) {
+                tassert(8770401,
+                        "Expected partIdx to be less than the size of the output",
+                        childNode->partIdx < out->size());
                 (*out)[childNode->partIdx] = elt;
             }
 
@@ -422,7 +407,6 @@ bool SortKeyGenerator::fastFillOutSortKeyPartsHelper(const BSONObj& bson,
 void SortKeyGenerator::generateSortKeyComponentVector(const BSONObj& bson,
                                                       std::vector<BSONElement>* eltsOut) {
     tassert(7103704, "Sort cannot have meta", !_sortHasMeta);
-    tassert(7103701, "Sort cannot have repeat keys", !_sortHasRepeatKey);
     tassert(7103702, "Cannot pass null as eltsOut", eltsOut);
 
     const bool fastPathSucceeded = fastFillOutSortKeyParts(bson, eltsOut);
@@ -443,7 +427,6 @@ void SortKeyGenerator::generateSortKeyComponentVector(const BSONObj& bson,
                 }
             }
         }
-
         // Fast path succeeded, we're done.
         return;
     }
@@ -456,11 +439,23 @@ void SortKeyGenerator::generateSortKeyComponentVector(const BSONObj& bson,
 
     Document outDoc(std::vector<std::pair<StringData, Value>>{{""_sd, sortKeyVal}});
     _localObjStorage = outDoc.toBson();
-    if (_localObjStorage.firstElement().type() == BSONType::Array) {
+    tassert(8770400,
+            "Expected BSONElement array to be the same size as the sortPattern",
+            eltsOut->size() == _sortPattern.size());
+    if (_sortPattern.size() > 1) {
+        tassert(8770404,
+                "If the sort pattern size is > 1, the sortKey should be an array",
+                _localObjStorage.firstElement().type() == BSONType::Array);
+        BSONObjIterator sortKeyIt(_localObjStorage.firstElement().embeddedObject());
         size_t i = 0;
-        for (auto& elt : _localObjStorage.firstElement().embeddedObject()) {
-            (*eltsOut)[i++] = elt;
+        while (i < eltsOut->size() && sortKeyIt.more()) {
+            (*eltsOut)[i++] = sortKeyIt.next();
         }
+        tassert(8770402,
+                "Expected the number of elements in the sort key to the equal to eltsOut.size()",
+                i == eltsOut->size());
+        tassert(
+            8770403, "Expected to exhaust the elements in the sort key array", !sortKeyIt.more());
     } else {
         (*eltsOut)[0] = _localObjStorage.firstElement();
     }
