@@ -28,15 +28,28 @@
  */
 
 
-#include "mongo/platform/basic.h"
+#include <algorithm>
+#include <boost/container/flat_set.hpp>
+#include <boost/container/vector.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <initializer_list>
+#include <ostream>
+#include <string>
 
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/bson/json.h"
 #include "mongo/db/index/wildcard_key_generator.h"
 #include "mongo/db/query/collation/collator_interface_mock.h"
 #include "mongo/db/record_id_helpers.h"
 #include "mongo/idl/server_parameter_test_util.h"
 #include "mongo/logv2/log.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/assert_util.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
 
@@ -48,7 +61,7 @@ KeyStringSet makeKeySet(std::initializer_list<BSONObj> init = {}, RecordId id =
     KeyStringSet keys;
     Ordering ordering = Ordering::make(BSONObj());
     for (const auto& key : init) {
-        KeyString::HeapBuilder keyString(KeyString::Version::kLatestVersion, key, ordering);
+        key_string::HeapBuilder keyString(key_string::Version::kLatestVersion, key, ordering);
         if (!id.isNull()) {
             keyString.appendRecordId(id);
         }
@@ -61,7 +74,7 @@ std::string dumpKeyset(const KeyStringSet& keyStrings) {
     std::stringstream ss;
     ss << "[ ";
     for (auto& keyString : keyStrings) {
-        auto key = KeyString::toBson(keyString, Ordering::make(BSONObj()));
+        auto key = key_string::toBson(keyString, Ordering::make(BSONObj()));
         ss << key.toString() << " ";
     }
     ss << "]";
@@ -90,7 +103,7 @@ bool assertKeysetsEqual(const KeyStringSet& expectedKeys, const KeyStringSet& ac
 }
 
 struct WildcardKeyGeneratorTest : public unittest::Test {
-    SharedBufferFragmentBuilder allocator{KeyString::HeapBuilder::kHeapAllocatorDefaultBytes};
+    SharedBufferFragmentBuilder allocator{key_string::HeapBuilder::kHeapAllocatorDefaultBytes};
     KeyFormat rsKeyFormat = KeyFormat::Long;
 };
 
@@ -101,7 +114,7 @@ TEST_F(WildcardKeyGeneratorFullDocumentTest, ExtractTopLevelKey) {
     WildcardKeyGenerator keyGen{fromjson("{'$**': 1}"),
                                 {},
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
     auto inputDoc = fromjson("{a: 1}");
@@ -121,7 +134,7 @@ TEST_F(WildcardKeyGeneratorFullDocumentTest, ExtractKeysFromNestedObject) {
     WildcardKeyGenerator keyGen{fromjson("{'$**': 1}"),
                                 {},
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
     auto inputDoc = fromjson("{a: {b: 'one', c: 2}}");
@@ -143,7 +156,7 @@ TEST_F(WildcardKeyGeneratorFullDocumentTest, ShouldIndexEmptyObject) {
     WildcardKeyGenerator keyGen{fromjson("{'$**': 1}"),
                                 {},
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
     auto inputDoc = fromjson("{a: 1, b: {}}");
@@ -163,7 +176,7 @@ TEST_F(WildcardKeyGeneratorFullDocumentTest, ShouldIndexNonNestedEmptyArrayAsUnd
     WildcardKeyGenerator keyGen{fromjson("{'$**': 1}"),
                                 {},
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
     auto inputDoc = fromjson("{ a: [], b: {c: []}, d: [[], {e: []}]}");
@@ -192,7 +205,7 @@ TEST_F(WildcardKeyGeneratorFullDocumentTest, ExtractMultikeyPath) {
     WildcardKeyGenerator keyGen{fromjson("{'$**': 1}"),
                                 {},
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
     auto inputDoc = fromjson("{a: [1, 2, {b: 'one', c: 2}, {d: 3}]}");
@@ -220,7 +233,7 @@ TEST_F(WildcardKeyGeneratorFullDocumentTest, ExtractMultikeyPathsKeyFormatString
     WildcardKeyGenerator keyGen{fromjson("{'$**': 1}"),
                                 {},
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 KeyFormat::String};
     auto inputDoc = fromjson("{a: [1, 2, {b: 'one', c: 2}, {d: 3}]}");
@@ -248,7 +261,7 @@ TEST_F(WildcardKeyGeneratorFullDocumentTest, ExtractMultikeyPathAndDedupKeys) {
     WildcardKeyGenerator keyGen{fromjson("{'$**': 1}"),
                                 {},
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
     auto inputDoc = fromjson("{a: [1, 2, {b: 'one', c: 2}, {c: 2, d: 3}, {d: 3}]}");
@@ -276,7 +289,7 @@ TEST_F(WildcardKeyGeneratorFullDocumentTest, ExtractZeroElementMultikeyPath) {
     WildcardKeyGenerator keyGen{fromjson("{'$**': 1}"),
                                 {},
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
     auto inputDoc = fromjson("{a: [1, 2, {b: 'one', c: 2}, {c: 2, d: 3}, {d: 3}], e: []}");
@@ -305,7 +318,7 @@ TEST_F(WildcardKeyGeneratorFullDocumentTest, ExtractNestedMultikeyPaths) {
     WildcardKeyGenerator keyGen{fromjson("{'$**': 1}"),
                                 {},
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
 
@@ -343,7 +356,7 @@ TEST_F(WildcardKeyGeneratorFullDocumentTest, ExtractMixedPathTypesAndAllSubpaths
     WildcardKeyGenerator keyGen{fromjson("{'$**': 1}"),
                                 {},
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
 
@@ -391,7 +404,7 @@ TEST_F(WildcardKeyGeneratorSingleSubtreeTest, ExtractSubtreeWithSinglePathCompon
     WildcardKeyGenerator keyGen{fromjson("{'g.$**': 1}"),
                                 {},
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
 
@@ -422,7 +435,7 @@ TEST_F(WildcardKeyGeneratorSingleSubtreeTest, ExtractSubtreeWithMultiplePathComp
     WildcardKeyGenerator keyGen{fromjson("{'g.h.$**': 1}"),
                                 {},
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
 
@@ -453,7 +466,7 @@ TEST_F(WildcardKeyGeneratorSingleSubtreeTest, ExtractMultikeySubtree) {
     WildcardKeyGenerator keyGen{fromjson("{'g.h.j.$**': 1}"),
                                 {},
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
 
@@ -482,7 +495,7 @@ TEST_F(WildcardKeyGeneratorSingleSubtreeTest, ExtractNestedMultikeySubtree) {
     WildcardKeyGenerator keyGen{fromjson("{'a.e.$**': 1}"),
                                 {},
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
 
@@ -516,7 +529,7 @@ TEST_F(WildcardKeyGeneratorInclusionTest, InclusionProjectionSingleSubtree) {
     WildcardKeyGenerator keyGen{fromjson("{'$**': 1}"),
                                 fromjson("{g: 1}"),
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
 
@@ -547,7 +560,7 @@ TEST_F(WildcardKeyGeneratorInclusionTest, InclusionProjectionNestedSubtree) {
     WildcardKeyGenerator keyGen{fromjson("{'$**': 1}"),
                                 fromjson("{'g.h': 1}"),
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
 
@@ -578,7 +591,7 @@ TEST_F(WildcardKeyGeneratorInclusionTest, InclusionProjectionMultikeySubtree) {
     WildcardKeyGenerator keyGen{fromjson("{'$**': 1}"),
                                 fromjson("{'g.h.j': 1}"),
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
 
@@ -607,7 +620,7 @@ TEST_F(WildcardKeyGeneratorInclusionTest, InclusionProjectionNestedMultikeySubtr
     WildcardKeyGenerator keyGen{fromjson("{'$**': 1}"),
                                 fromjson("{'a.e': 1}"),
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
 
@@ -636,7 +649,7 @@ TEST_F(WildcardKeyGeneratorInclusionTest, InclusionProjectionMultipleSubtrees) {
     WildcardKeyGenerator keyGen{fromjson("{'$**': 1}"),
                                 fromjson("{'a.b': 1, 'a.c': 1, 'a.e': 1, 'g.h.i': 1}"),
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
 
@@ -671,7 +684,7 @@ TEST_F(WildcardKeyGeneratorExclusionTest, ExclusionProjectionSingleSubtree) {
     WildcardKeyGenerator keyGen{fromjson("{'$**': 1}"),
                                 fromjson("{g: 0}"),
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
 
@@ -707,7 +720,7 @@ TEST_F(WildcardKeyGeneratorExclusionTest, ExclusionProjectionNestedSubtree) {
     WildcardKeyGenerator keyGen{fromjson("{'$**': 1}"),
                                 fromjson("{'g.h': 0}"),
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
 
@@ -744,7 +757,7 @@ TEST_F(WildcardKeyGeneratorExclusionTest, ExclusionProjectionMultikeySubtree) {
     WildcardKeyGenerator keyGen{fromjson("{'$**': 1}"),
                                 fromjson("{'g.h.j': 0}"),
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
 
@@ -782,7 +795,7 @@ TEST_F(WildcardKeyGeneratorExclusionTest, ExclusionProjectionNestedMultikeySubtr
     WildcardKeyGenerator keyGen{fromjson("{'$**': 1}"),
                                 fromjson("{'a.e': 0}"),
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
 
@@ -823,7 +836,7 @@ TEST_F(WildcardKeyGeneratorExclusionTest, ExclusionProjectionMultipleSubtrees) {
     WildcardKeyGenerator keyGen{fromjson("{'$**': 1}"),
                                 fromjson("{'a.b': 0, 'a.c': 0, 'a.e': 0, 'g.h.i': 0}"),
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
 
@@ -864,7 +877,7 @@ TEST_F(WildcardKeyGeneratorIdTest, ExcludeIdFieldIfProjectionIsEmpty) {
     WildcardKeyGenerator keyGen{fromjson("{'$**': 1}"),
                                 {},
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
 
@@ -895,7 +908,7 @@ TEST_F(WildcardKeyGeneratorIdTest, ExcludeIdFieldForSingleSubtreeKeyPattern) {
     WildcardKeyGenerator keyGen{fromjson("{'a.$**': 1}"),
                                 {},
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
 
@@ -924,7 +937,7 @@ TEST_F(WildcardKeyGeneratorIdTest, PermitIdFieldAsSingleSubtreeKeyPattern) {
     WildcardKeyGenerator keyGen{fromjson("{'_id.$**': 1}"),
                                 {},
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
 
@@ -948,7 +961,7 @@ TEST_F(WildcardKeyGeneratorIdTest, PermitIdSubfieldAsSingleSubtreeKeyPattern) {
     WildcardKeyGenerator keyGen{fromjson("{'_id.id1.$**': 1}"),
                                 {},
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
 
@@ -971,7 +984,7 @@ TEST_F(WildcardKeyGeneratorIdTest, ExcludeIdFieldByDefaultForInclusionProjection
     WildcardKeyGenerator keyGen{fromjson("{'$**': 1}"),
                                 fromjson("{a: 1}"),
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
 
@@ -1000,7 +1013,7 @@ TEST_F(WildcardKeyGeneratorIdTest, PermitIdSubfieldInclusionInExplicitProjection
     WildcardKeyGenerator keyGen{fromjson("{'$**': 1}"),
                                 fromjson("{'_id.id1': 1}"),
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
 
@@ -1023,7 +1036,7 @@ TEST_F(WildcardKeyGeneratorIdTest, ExcludeIdFieldByDefaultForExclusionProjection
     WildcardKeyGenerator keyGen{fromjson("{'$**': 1}"),
                                 fromjson("{a: 0}"),
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
 
@@ -1047,7 +1060,7 @@ TEST_F(WildcardKeyGeneratorIdTest, PermitIdSubfieldExclusionInExplicitProjection
     WildcardKeyGenerator keyGen{fromjson("{'$**': 1}"),
                                 fromjson("{'_id.id1': 0}"),
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
 
@@ -1079,7 +1092,7 @@ TEST_F(WildcardKeyGeneratorIdTest, IncludeIdFieldIfExplicitlySpecifiedInProjecti
     WildcardKeyGenerator keyGen{fromjson("{'$**': 1}"),
                                 fromjson("{_id: 1, a: 1}"),
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
 
@@ -1110,7 +1123,7 @@ TEST_F(WildcardKeyGeneratorIdTest, ExcludeIdFieldIfExplicitlySpecifiedInProjecti
     WildcardKeyGenerator keyGen{fromjson("{'$**': 1}"),
                                 fromjson("{_id: 0, a: 1}"),
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
 
@@ -1139,7 +1152,7 @@ TEST_F(WildcardKeyGeneratorIdTest, IncludeIdFieldIfExplicitlySpecifiedInExclusio
     WildcardKeyGenerator keyGen{fromjson("{'$**': 1}"),
                                 fromjson("{_id: 1, a: 0}"),
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
 
@@ -1169,7 +1182,7 @@ TEST_F(WildcardKeyGeneratorCollationTest, CollationMixedPathAndKeyTypes) {
     WildcardKeyGenerator keyGen{fromjson("{'$**': 1}"),
                                 {},
                                 &collator,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
 
@@ -1223,7 +1236,7 @@ TEST_F(WildcardKeyGeneratorDottedFieldsTest, DoNotIndexDottedFields) {
     WildcardKeyGenerator keyGen{fromjson("{'$**': 1}"),
                                 {},
                                 {},
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
 
@@ -1253,7 +1266,7 @@ TEST_F(WildcardKeyGeneratorDottedFieldsTest, DoNotIndexDottedFieldsWithSimilarSu
     WildcardKeyGenerator keyGen{fromjson("{'a.b.$**': 1}"),
                                 {},
                                 {},
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
 
@@ -1274,17 +1287,13 @@ TEST_F(WildcardKeyGeneratorDottedFieldsTest, DoNotIndexDottedFieldsWithSimilarSu
 //
 // Following unit tests are for compound wildcard indexes.
 //
-struct WildcardKeyGeneratorCompoundTest : public WildcardKeyGeneratorTest {
-    WildcardKeyGeneratorCompoundTest() {}
-
-    RAIIServerParameterControllerForTest controllerCWI{"featureFlagCompoundWildcardIndexes", true};
-};
+struct WildcardKeyGeneratorCompoundTest : public WildcardKeyGeneratorTest {};
 
 TEST_F(WildcardKeyGeneratorCompoundTest, ExtractTopLevelKeyCompound) {
     WildcardKeyGenerator keyGen{fromjson("{'$**': 1, a: 1}"),
                                 fromjson("{a: 0}"),
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
     auto inputDoc = fromjson("{a: 1, b: 1}");
@@ -1304,7 +1313,7 @@ TEST_F(WildcardKeyGeneratorCompoundTest, ExtractKeysFromNestedObjectCompound) {
     WildcardKeyGenerator keyGen{fromjson("{c: 1, '$**': 1}"),
                                 fromjson("{c: 0}"),
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
     auto inputDoc = fromjson("{a: {b: 'one', c: 2}}");
@@ -1326,7 +1335,7 @@ TEST_F(WildcardKeyGeneratorCompoundTest, MiddleWildcardComponentCompound) {
     WildcardKeyGenerator keyGen{fromjson("{a: 1, '$**': 1, c: 1}"),
                                 fromjson("{a: 0, c: 0}"),
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
     auto inputDoc = fromjson("{a: 1, b: 2}");
@@ -1347,7 +1356,7 @@ TEST_F(WildcardKeyGeneratorCompoundTest, IndexSubTreeCompound) {
     WildcardKeyGenerator keyGen{fromjson("{a: 1, 'sub.$**': 1}"),
                                 {},
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
     auto inputDoc = fromjson("{a: 1, sub: {a: 1, b: 2}}");
@@ -1369,7 +1378,7 @@ TEST_F(WildcardKeyGeneratorCompoundTest, CompoundWildcardIndexShouldBeSparse) {
     WildcardKeyGenerator keyGen{fromjson("{'$**': 1, c: 1}"),
                                 fromjson("{c: 0}"),
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
     auto inputDoc = fromjson("{}");
@@ -1390,7 +1399,7 @@ TEST_F(WildcardKeyGeneratorCompoundTest, CanGenerateKeysForMultikeyFieldCompound
     WildcardKeyGenerator keyGen{fromjson("{a: 1, '$**': 1, c: 1}"),
                                 fromjson("{a: 0, c: 0}"),
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
     auto inputDoc = fromjson("{a: 1, b: [1, {c: [3]}]}");
@@ -1419,7 +1428,7 @@ TEST_F(WildcardKeyGeneratorCompoundTest, CannotCompoundWithMultikeyField) {
     WildcardKeyGenerator keyGen{fromjson("{'sub.$**': 1, arr: 1}"),
                                 {},
                                 nullptr,
-                                KeyString::Version::kLatestVersion,
+                                key_string::Version::kLatestVersion,
                                 Ordering::make(BSONObj()),
                                 rsKeyFormat};
     auto inputDoc = fromjson("{sub: {a: 1}, arr: [1, 2]}");
