@@ -29,9 +29,16 @@
 
 #pragma once
 
+#include <cstddef>
+#include <string>
+
 #include "mongo/base/status.h"
 #include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/bson/util/builder.h"
+#include "mongo/bson/util/builder_fwd.h"
 #include "mongo/db/jsobj.h"
 #include "mongo/db/query/query_shape/serialization_options.h"
 #include "mongo/util/str.h"
@@ -100,6 +107,14 @@ public:
         return str::stream() << *this;
     }
 
+    /**
+     * Returns a string representation of this BSONObj keypattern.
+     */
+    static std::string toString(const BSONObj& keyPattern) {
+        StringBuilder sb;
+        return addToStringBuilder(sb, keyPattern).str();
+    }
+
     /**
      * Writes to 'sb' a string representation of this KeyPattern.
      */
@@ -137,6 +152,7 @@ public:
     size_t getApproximateSize() const;
 
 private:
+    static StringBuilder& addToStringBuilder(StringBuilder& sb, const BSONObj& pattern);
     BSONObj _pattern;
 };
 
