@@ -29,19 +29,36 @@
 
 #pragma once
 
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
 #include <boost/optional.hpp>
+#include <boost/optional/optional.hpp>
+#include <cstddef>
+#include <cstdint>
 #include <functional>
 #include <memory>
+#include <string>
+#include <type_traits>
+#include <utility>
+#include <vector>
 
 #include "mongo/base/clonable_ptr.h"
 #include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
 #include "mongo/bson/bsonobj.h"
 #include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/util/builder_fwd.h"
+#include "mongo/db/field_ref.h"
 #include "mongo/db/matcher/expression_visitor.h"
 #include "mongo/db/matcher/match_details.h"
 #include "mongo/db/matcher/matchable.h"
 #include "mongo/db/pipeline/dependencies.h"
+#include "mongo/db/query/collation/collator_interface.h"
 #include "mongo/db/query/query_shape/serialization_options.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/fail_point.h"
 
 namespace mongo {
@@ -53,6 +70,7 @@ namespace mongo {
 extern FailPoint disableMatchExpressionOptimization;
 
 class CollatorInterface;
+
 class MatchExpression;
 class TreeMatchExpression;
 
@@ -295,23 +313,10 @@ public:
      *   - a pointer to a new MatchExpression.
      *
      * The value of 'expression' must not be nullptr.
+     * 'enableSimplification' parameter controls Boolean Expression Simplifier.
      */
-    static std::unique_ptr<MatchExpression> optimize(std::unique_ptr<MatchExpression> expression) {
-        // If the disableMatchExpressionOptimization failpoint is enabled, optimizations are skipped
-        // and the expression is left unmodified.
-        if (MONGO_unlikely(disableMatchExpressionOptimization.shouldFail())) {
-            return expression;
-        }
-
-        auto optimizer = expression->getOptimizer();
-
-        try {
-            return optimizer(std::move(expression));
-        } catch (DBException& ex) {
-            ex.addContext("Failed to optimize expression");
-            throw;
-        }
-    }
+    static std::unique_ptr<MatchExpression> optimize(std::unique_ptr<MatchExpression> expression,
+                                                     bool enableSimplification = true);
 
     /**
      * Traverses expression tree post-order. Sorts children at each non-leaf node by (MatchType,
@@ -322,22 +327,34 @@ public:
     /**
      * Convenience method which normalizes a MatchExpression tree by optimizing and then sorting it.
      */
-    static std::unique_ptr<MatchExpression> normalize(std::unique_ptr<MatchExpression> tree) {
-        tree = optimize(std::move(tree));
-        sortTree(tree.get());
-        return tree;
-    }
+    static std::unique_ptr<MatchExpression> normalize(std::unique_ptr<MatchExpression> tree,
+                                                      bool enableSimplification = true);
 
     /**
      * Assigns an optional input parameter ID to each node which is eligible for
      * auto-parameterization.
-     * - maxParameterCount - Maximum number of parameters that can be created. If the number of
-     *   parameters would exceed this value, no parameterization will be performed.
+     * - tree - The MatchExpression to be parameterized.
+     * - maxParamCount - Optional maximum number of parameters that can be created. If the
+     *   number of parameters would exceed this value, no parameterization will be performed.
+     * - startingParamId - Optional first parameter ID to use. This enables parameterizing a forest
+     *   of match expressions, where each tree continues IDs where the prior one left off.
+     * - parameterized - Optional output argument. If non-null, the method sets this output to
+     *   indicate whether parameterization was actually done.
      *
-     * Returns a map to a parameterized MatchExpression from assigned InputParamId.
+     * Returns a vector-form map to a parameterized MatchExpression from assigned InputParamId. (The
+     * vector index serves as the map key.)
      */
     static std::vector<const MatchExpression*> parameterize(
-        MatchExpression* tree, boost::optional<size_t> maxParameterCount = boost::none);
+        MatchExpression* tree,
+        boost::optional<size_t> maxParamCount = boost::none,
+        InputParamId startingParamId = 0,
+        bool* parameterized = nullptr);
+
+    /**
+     * Sets max param count in MatchExpression::parameterize to 0, clearing MatchExpression
+     * auto-parameterization before CanonicalQuery to ABT translation.
+     */
+    static std::vector<const MatchExpression*> unparameterize(MatchExpression* tree);
 
     MatchExpression(MatchType type, clonable_ptr<ErrorAnnotation> annotation = nullptr);
     virtual ~MatchExpression() {}
