@@ -27,17 +27,22 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <string>
 
-#include "mongo/db/jsobj.h"
-#include "mongo/db/json.h"
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/db/matcher/expression.h"
+#include "mongo/db/matcher/expression_always_boolean.h"
 #include "mongo/db/matcher/expression_array.h"
 #include "mongo/db/matcher/expression_leaf.h"
 #include "mongo/db/matcher/expression_tree.h"
 #include "mongo/db/query/collation/collator_interface_mock.h"
+#include "mongo/unittest/assert.h"
 #include "mongo/unittest/death_test.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/unittest/framework.h"
 
 namespace mongo {
 
@@ -275,6 +280,16 @@ DEATH_TEST_REGEX(ElemMatchValueMatchExpression,
     ASSERT_THROWS_CODE(op.getChild(numChildren), AssertionException, 6400205);
 }
 
+TEST(ElemMatchValueMatchExpression, IsReducedToAlwaysFalseIfContainsIt) {
+    auto baseOperand = BSON("$gt" << 6);
+    auto gt = std::make_unique<GTMatchExpression>(""_sd, baseOperand["$gt"]);
+    auto expr = std::make_unique<ElemMatchValueMatchExpression>("a"_sd, std::move(gt));
+    expr->add(std::make_unique<AlwaysFalseMatchExpression>());
+    ASSERT_FALSE(expr->isTriviallyFalse());
+    auto optimizedExpr = MatchExpression::optimize(std::move(expr), true);
+    ASSERT(optimizedExpr->isTriviallyFalse());
+}
+
 /**
 TEST(ElemMatchValueMatchExpression, MatchesIndexKey) {
     auto baseOperand = BSON("$lt" << 5);
