@@ -27,13 +27,24 @@
  *    it in the license file.
  */
 
-#include "mongo/db/matcher/expression_visitor.h"
-#include "mongo/db/pipeline/expression.h"
-#include "mongo/platform/basic.h"
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <utility>
 
+#include <boost/move/utility_core.hpp>
+
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/db/exec/document_value/document.h"
+#include "mongo/db/exec/document_value/value_comparator.h"
 #include "mongo/db/matcher/expression_expr.h"
 #include "mongo/db/matcher/expression_internal_eq_hashed_key.h"
-
+#include "mongo/db/matcher/expression_tree.h"
+#include "mongo/db/pipeline/expression.h"
+#include "mongo/db/pipeline/field_path.h"
+#include "mongo/db/pipeline/variables.h"
+#include "mongo/platform/compiler.h"
 #include "mongo/util/fail_point.h"
 
 namespace mongo {
@@ -150,7 +161,7 @@ std::unique_ptr<MatchExpression> attemptToRewriteEqHash(ExprMatchExpression& exp
     // Where "a" can be any field path and ? can be any number.
     if (auto eq = dynamic_cast<ExpressionCompare*>(childExpr.get());
         eq && eq->getOp() == ExpressionCompare::CmpOp::EQ) {
-        auto children = eq->getChildren();
+        const auto& children = eq->getChildren();
         tassert(7281406, "should have 2 $eq children", children.size() == 2ul);
 
         auto eqFirst = children[0].get();
@@ -209,7 +220,10 @@ MatchExpression::ExpressionOptimizerFunc ExprMatchExpression::getOptimizer() con
             andMatch->add(exprMatchExpr._rewriteResult->releaseMatchExpression());
             andMatch->add(std::move(expression));
             // Re-optimize the new AND in order to make sure that any AND children are absorbed.
-            expression = MatchExpression::optimize(std::move(andMatch));
+            // The Boolean simplifier is disabled since we don't want to simplify sub-expressions,
+            // but simplify the whole expression instead.
+            expression =
+                MatchExpression::optimize(std::move(andMatch), /* enableSimplification */ false);
         }
 
         // Replace trivially true expression with an empty AND since the planner doesn't always
