@@ -27,26 +27,38 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <functional>
+#include <limits>
+#include <utility>
+
+#include <boost/smart_ptr/intrusive_ptr.hpp>
 
 #include "mongo/base/checked_cast.h"
+#include "mongo/base/error_codes.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
 #include "mongo/bson/json.h"
+#include "mongo/bson/timestamp.h"
 #include "mongo/db/matcher/expression.h"
 #include "mongo/db/matcher/expression_expr.h"
 #include "mongo/db/matcher/expression_parser.h"
-#include "mongo/db/matcher/matcher.h"
+#include "mongo/db/matcher/expression_tree.h"
+#include "mongo/db/matcher/extensions_callback_noop.h"
 #include "mongo/db/pipeline/expression_context_for_test.h"
+#include "mongo/db/pipeline/variables.h"
 #include "mongo/db/query/collation/collator_interface_mock.h"
 #include "mongo/db/query/query_shape/serialization_options.h"
+#include "mongo/unittest/assert.h"
 #include "mongo/unittest/death_test.h"
-#include "mongo/unittest/inline_auto_update.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/fail_point.h"
+#include "mongo/util/str.h"
 
 namespace mongo {
 
 namespace {
 
-using unittest::assertGet;
 
 const double kNaN = std::numeric_limits<double>::quiet_NaN();
 
@@ -868,6 +880,16 @@ TEST_F(ExprMatchTest, ExprRedactsCorrectly) {
         R"({"$expr":{"$getField":{"field":"HASH<b>","input":"?object"}}})",
         serialize(opts));
 
+    createMatcher(fromjson("{$expr: {$getField: {field: \"$b\", input: {a: 1, b: 2}}}}"));
+    ASSERT_BSONOBJ_EQ_AUTO(  // NOLINT
+        R"({"$expr":{"$getField":{"field":"$HASH<b>","input":"?object"}}})",
+        serialize(opts));
+
+    createMatcher(fromjson("{$expr: {$getField: {field: {$const: \"$b\"}, input: {a: 1, b: 2}}}}"));
+    ASSERT_BSONOBJ_EQ_AUTO(  // NOLINT
+        R"({"$expr":{"$getField":{"field":{$const:"HASH<$b>"},"input":"?object"}}})",
+        serialize(opts));
+
     createMatcher(fromjson("{$expr: {$getField: {field: \"b\", input: \"$a\"}}}"));
     ASSERT_BSONOBJ_EQ_AUTO(  // NOLINT
         R"({"$expr":{"$getField":{"field":"HASH<b>","input":"$HASH<a>"}}})",
