@@ -30,12 +30,25 @@
 
 #include "mongo/db/matcher/expression_geo.h"
 
+#include <boost/move/utility_core.hpp>
+#include <boost/optional.hpp>
+#include <boost/optional/optional.hpp>
+#include <cmath>
+#include <limits>
+#include <s2cellid.h>
+#include <utility>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/bson/bsontypes.h"
 #include "mongo/bson/simple_bsonobj_comparator.h"
+#include "mongo/bson/util/builder.h"
 #include "mongo/db/geo/geoparser.h"
 #include "mongo/db/matcher/expression_geo_serializer.h"
 #include "mongo/db/matcher/expression_parser.h"
 #include "mongo/logv2/log.h"
-#include "mongo/platform/basic.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/redaction.h"
 #include "mongo/util/str.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
@@ -81,12 +94,12 @@ Status GeoExpression::parseQuery(const BSONObj& obj) {
     while (geoIt.more()) {
         BSONElement elt = geoIt.next();
         // $geoWithin doesn't accept multiple shapes.
-        if (geoContainer && queryElt.fieldNameStringData() == "$geoWithin"_sd) {
+        if (geoContainer && queryElt.fieldNameStringData() == kGeoWithinField) {
             return Status(ErrorCodes::BadValue,
                           str::stream() << "$geoWithin doesn't accept multiple shapes "
                                         << queryElt.toString());
         }
-        if (elt.fieldNameStringData() == "$uniqueDocs") {
+        if (elt.fieldNameStringData() == kUniqueDocsField) {
             // Deprecated "$uniqueDocs" field
             LOGV2_WARNING(23847, "Deprecated $uniqueDocs option", "query"_attr = redact(obj));
         } else {
@@ -182,7 +195,8 @@ bool GeoNearExpression::parseLegacyQuery(const BSONObj& obj) {
     while (it.more()) {
         BSONElement e = it.next();
         StringData fieldName = e.fieldNameStringData();
-        if ((fieldName == "$near") || (fieldName == "$geoNear") || (fieldName == "$nearSphere")) {
+        if ((fieldName == kNearField) || (fieldName == kGeoNearField) ||
+            (fieldName == kNearSphereField)) {
             if (!e.isABSONObj()) {
                 return false;
             }
@@ -192,17 +206,17 @@ bool GeoNearExpression::parseLegacyQuery(const BSONObj& obj) {
                 GeoParser::parsePointWithMaxDistance(embeddedObj, centroid.get(), &maxDistance)) {
                 uassert(18522, "max distance must be non-negative", maxDistance >= 0.0);
                 hasGeometry = true;
-                isNearSphere = (e.fieldNameStringData() == "$nearSphere");
+                isNearSphere = (e.fieldNameStringData() == kNearSphereField);
             }
-        } else if (fieldName == "$minDistance") {
+        } else if (fieldName == kMinDistanceField) {
             uassert(16893, "$minDistance must be a number", e.isNumber());
             minDistance = e.Number();
             uassert(16894, "$minDistance must be non-negative", minDistance >= 0.0);
-        } else if (fieldName == "$maxDistance") {
+        } else if (fieldName == kMaxDistanceField) {
             uassert(16895, "$maxDistance must be a number", e.isNumber());
             maxDistance = e.Number();
             uassert(16896, "$maxDistance must be non-negative", maxDistance >= 0.0);
-        } else if (fieldName == "$uniqueDocs") {
+        } else if (fieldName == kUniqueDocsField) {
             LOGV2_WARNING(23848, "Ignoring deprecated option $uniqueDocs");
         } else {
             // In a query document, $near queries can have no non-geo sibling parameters.
@@ -252,14 +266,10 @@ Status GeoNearExpression::parseNewQuery(const BSONObj& obj) {
     while (it.more()) {
         BSONElement e = it.next();
         StringData fieldName = e.fieldNameStringData();
-        if (fieldName == "$geometry") {
+        if (fieldName == kGeometryField) {
             if (e.isABSONObj()) {
                 BSONObj embeddedObj = e.embeddedObject();
-                // TODO SERVER-84598: $geoNear queries don't do a type check on the type of the
-                // geometry provided into the query. This is different than the behaviour we see for
-                // other GeoExpressions that use a GeometryContainer. GeoNear is a special case that
-                // only ever has type 'Point', however it may be a good idea to consider removing
-                // this silent acceptance of invalid "type" in the long term.
+                // TODO SERVER-86141: Refactor geoParsing.
                 Status status = GeoParser::parseQueryPoint(e, centroid.get());
                 if (!status.isOK()) {
                     return Status(ErrorCodes::BadValue,
@@ -274,7 +284,7 @@ Status GeoNearExpression::parseNewQuery(const BSONObj& obj) {
                 }
                 hasGeometry = true;
             }
-        } else if (fieldName == "$minDistance") {
+        } else if (fieldName == kMinDistanceField) {
             if (!e.isNumber()) {
                 return Status(ErrorCodes::BadValue, "$minDistance must be a number");
             }
@@ -282,7 +292,7 @@ Status GeoNearExpression::parseNewQuery(const BSONObj& obj) {
             if (!isValidNumericValue(minDistance)) {
                 return Status(ErrorCodes::BadValue, "$minDistance must be non-negative");
             }
-        } else if (fieldName == "$maxDistance") {
+        } else if (fieldName == kMaxDistanceField) {
             if (!e.isNumber()) {
                 return Status(ErrorCodes::BadValue, "$maxDistance must be a number");
             }
