@@ -29,15 +29,15 @@
 
 /** Unit tests for MatchExpression operator implementations in match_operators.{h,cpp}. */
 
-#include "mongo/unittest/unittest.h"
-
 #include <memory>
 
-#include "mongo/db/exec/document_value/document_value_test_util.h"
-#include "mongo/db/jsobj.h"
-#include "mongo/db/json.h"
-#include "mongo/db/matcher/expression.h"
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/bson/json.h"
 #include "mongo/db/matcher/expression_geo.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
 
 
 namespace mongo {
@@ -92,6 +92,10 @@ std::unique_ptr<GeoNearMatchExpression> makeGeoNearMatchExpression(const BSONObj
     return gne;
 }
 
+void assertGeoNearParseReturnsError(const BSONObj& locQuery) {
+    std::unique_ptr<GeoNearExpression> nq(new GeoNearExpression);
+    ASSERT_EQUALS(ErrorCodes::BadValue, nq->parseFrom(locQuery));
+}
 
 /**
  * A bunch of cases in which a geo expression is equivalent() to both itself or to another
@@ -509,10 +513,16 @@ TEST(ExpressionGeoTest, RoundTripSerializeGeoExpressions) {
         fromjson("{$geoNear: { $geometry: {coordinates: [0, 10]}}}"),
         fromjson("{$geoNear: { $geometry: {coordinates: [1, 1]}}}"));
 
-    // Test scenario with new $geometry query specifying invalid type.
+    // $geometry operator in $geoNear should accept only Point type.
     assertRepresentativeGeoNearShapeIsStable(
-        fromjson("{$geoNear: { $geometry: { type: 'b.c', coordinates: [0, 10]}}}"),
-        fromjson("{$geoNear: { $geometry: {type: 'b.c', coordinates: [1, 1]}}}"));
+        fromjson(R"({$geoNear: { $geometry: {"type": "Point", "coordinates": [0, 10]}}})"),
+        fromjson(R"({$geoNear: { $geometry: {"type": "Point", coordinates: [1, 1]}}})"));
+    assertGeoNearParseReturnsError(
+        fromjson(R"({$geoNear: { $geometry: {"type": "LineString", "coordinates": [0, 10]}}})"));
+    assertGeoNearParseReturnsError(fromjson(
+        R"({$geoNear: { $geometry: {"type": "LineString", "coordinates": [[1, 2], [3, 4]]}}})"));
+    assertGeoNearParseReturnsError(fromjson(
+        R"({$geoNear: { $geometry: {"type": "MultiPoint", "coordinates": [[0, 0], [1, 1]]}}})"));
 
     // Test scenario with $nearSphere without $geometry and no type specified
     assertRepresentativeGeoNearShapeIsStable(fromjson(R"({"$nearSphere":{"coordinates":[0,0]}})"),
