@@ -29,7 +29,21 @@
 
 #pragma once
 
+#include <memory>
+
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/db/exec/document_value/value.h"
+#include "mongo/db/matcher/expression.h"
 #include "mongo/db/matcher/expression_leaf.h"
+#include "mongo/db/matcher/expression_visitor.h"
+#include "mongo/db/matcher/match_details.h"
+#include "mongo/db/matcher/path.h"
+#include "mongo/util/assert_util.h"
 
 namespace mongo {
 
@@ -50,6 +64,9 @@ namespace mongo {
  *
  * - Comparison to an array is illegal. It is invalid usage to construct a
  *   InternalExprComparisonMatchExpression node which compares to an array.
+ *
+ * These expressions are always treated as "imprecise" and are to be used as the first level in a
+ * two-level filtering scheme. They must always be accompanied by a later filter which is precise.
  */
 template <typename T>
 class InternalExprComparisonMatchExpression : public ComparisonMatchExpressionBase {
@@ -66,7 +83,7 @@ public:
         invariant(_rhs.type() != BSONType::Array);
     }
 
-    virtual ~InternalExprComparisonMatchExpression() = default;
+    ~InternalExprComparisonMatchExpression() override = default;
 
     bool matchesSingleElement(const BSONElement& elem, MatchDetails* details) const final {
         // We use NonLeafArrayBehavior::kMatchSubpath traversal in
