@@ -29,17 +29,36 @@
 
 #pragma once
 
+#include <boost/move/utility_core.hpp>
 #include <boost/optional.hpp>
+#include <boost/optional/optional.hpp>
+#include <cstddef>
+#include <cstdint>
 #include <memory>
+#include <set>
+#include <string>
+#include <utility>
+#include <vector>
 
+#include "mongo/base/clonable_ptr.h"
+#include "mongo/base/status.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
 #include "mongo/bson/bsonelement_comparator.h"
 #include "mongo/bson/bsonmisc.h"
 #include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/bson/bsontypes.h"
+#include "mongo/bson/util/builder_fwd.h"
 #include "mongo/db/exec/document_value/value.h"
 #include "mongo/db/matcher/expression.h"
 #include "mongo/db/matcher/expression_path.h"
+#include "mongo/db/matcher/expression_visitor.h"
+#include "mongo/db/matcher/in_list_data.h"
+#include "mongo/db/matcher/match_details.h"
+#include "mongo/db/matcher/path.h"
 #include "mongo/db/query/collation/collator_interface.h"
+#include "mongo/db/query/query_shape/serialization_options.h"
 #include "mongo/db/query/util/make_data_structure.h"
 #include "mongo/stdx/unordered_map.h"
 #include "mongo/util/assert_util.h"
@@ -50,30 +69,29 @@ namespace mongo {
 class CollatorInterface;
 
 /**
- *  A struct primarily used to make input parameters for makePredicate() function. The
- * 'MatchExprType' helps in determining which MatchExpression to make in makePredicate() function.
+ * Makes a conjunction of the given predicates
  */
-template <typename MatchExprType, typename ValueType = BSONElement>
-struct MatchExprPredicate {
-    MatchExprPredicate(StringData path_, ValueType value_) : path(path_), value(value_){};
-    StringData path;
-    ValueType value;
-};
+template <typename... Ts>
+inline auto makeAnd(Ts&&... pack) {
+    auto predicates = makeVector<std::unique_ptr<MatchExpression>>(std::forward<Ts>(pack)...);
+    return std::make_unique<AndMatchExpression>(std::move(predicates));
+}
 
 /**
- * Helper function to make $and predicates based on the set of predicates passed as parameters.
+ * Makes a disjunction of the given predicates.
+ *
+ * - The result is non-null; it may be an OrMatchExpression with zero children.
+ * - Any trivially-false arguments are omitted.
+ * - If only one argument is nontrivial, returns that argument rather than adding an extra
+ *   OrMatchExpression around it.
  */
-template <typename T, typename ValueType, typename... Targs, typename... ValueTypeArgs>
-std::unique_ptr<MatchExpression> makePredicate(
-    MatchExprPredicate<T, ValueType> predicate,
-    MatchExprPredicate<Targs, ValueTypeArgs>... predicates) {
-    if constexpr (sizeof...(predicates) > 0) {
-        return std::make_unique<AndMatchExpression>(makeVector<std::unique_ptr<MatchExpression>>(
-            std::make_unique<T>(predicate.path, predicate.value),
-            (std::make_unique<Targs>(predicates.path, predicates.value))...));
-    } else {
-        return std::make_unique<T>(predicate.path, predicate.value);
-    }
+template <typename... Ts>
+inline auto makeOr(Ts&&... pack) {
+    auto predicates = makeVector<std::unique_ptr<MatchExpression>>(std::forward<Ts>(pack)...);
+    auto newEnd = std::remove_if(
+        predicates.begin(), predicates.end(), [](auto& node) { return node->isTriviallyFalse(); });
+    predicates.erase(newEnd, predicates.end());
+    return std::make_unique<OrMatchExpression>(std::move(predicates));
 }
 
 class LeafMatchExpression : public PathMatchExpression {
@@ -95,7 +113,7 @@ public:
         : PathMatchExpression(
               matchType, path, leafArrBehavior, nonLeafArrBehavior, std::move(annotation)) {}
 
-    virtual ~LeafMatchExpression() = default;
+    ~LeafMatchExpression() override = default;
 
     size_t numChildren() const override {
         return 0;
@@ -158,15 +176,15 @@ public:
                                   clonable_ptr<ErrorAnnotation> annotation = nullptr,
                                   const CollatorInterface* collator = nullptr);
 
-    virtual ~ComparisonMatchExpressionBase() = default;
+    ~ComparisonMatchExpressionBase() override = default;
 
-    virtual void debugString(StringBuilder& debug, int indentationLevel = 0) const;
+    void debugString(StringBuilder& debug, int indentationLevel = 0) const override;
 
-    virtual void appendSerializedRightHandSide(BSONObjBuilder* bob,
-                                               const SerializationOptions& opts = {},
-                                               bool includePath = true) const;
+    void appendSerializedRightHandSide(BSONObjBuilder* bob,
+                                       const SerializationOptions& opts = {},
+                                       bool includePath = true) const override;
 
-    virtual bool equivalent(const MatchExpression* other) const;
+    bool equivalent(const MatchExpression* other) const override;
 
     /**
      * Returns the name of this MatchExpression.
@@ -186,6 +204,21 @@ public:
         _rhs = elem;
     }
 
+    /**
+     * Populate the _backingBSON for this ComparisonMatchExpression. Typically, we want '_rhs' to be
+     * a BSONElement which points to FindCommandRequest owned BSON to avoid unnecessary copies.
+     * However, there are cases during optimization when we construct a new MatchExpression with a
+     * RHS that contains a value which isn't present in the original command. This function is
+     * useful in this case to avoid forcing callers to construct BSON and needing to keep it alive
+     * for the length of this MatchExpression. Callers should take care to avoid using this function
+     * in the hotpath.
+     */
+    void setBackingBSON(const BSONObj& obj) {
+        invariant(obj.isOwned());
+        _backingBSON = obj;
+        _backingBSONIsSet = true;
+    }
+
     const CollatorInterface* getCollator() const {
         return _collator;
     }
@@ -198,6 +231,10 @@ public:
         return _inputParamId;
     }
 
+    boost::optional<BSONObj> getOwnedBackingBSON() const {
+        return (_backingBSONIsSet ? boost::make_optional(_backingBSON) : boost::none);
+    }
+
 protected:
     /**
      * 'collator' must outlive the ComparisonMatchExpression and any clones made of it.
@@ -207,7 +244,14 @@ protected:
     }
 
     // BSON which holds the data referenced by _rhs.
+    // For performance, the _backingBSON in ComparisonMatchExpressionBase is optionally populated.
+    // The creator of the MatchExpression ensures the lifetime of the relevant BSONObj against which
+    // the comparison is performed. _rhs points into that relevant _backingBSON.
+    // The specific cases when _backingBSON is populated (non-exhaustive list):
+    // -- initialization using Value
+    // -- $in simplification to Equality where the original BSONObj is destroyed.
     BSONObj _backingBSON;
+    bool _backingBSONIsSet;
     BSONElement _rhs;
 
     // Collator used to compare elements. By default, simple binary comparison will be used.
@@ -220,6 +264,15 @@ private:
         };
     }
 
+    void setData(boost::optional<StringData>& path, BSONElement elem) {
+        _rhs = elem;
+    }
+
+    void setData(boost::optional<StringData>& path, Value elem) {
+        setBackingBSON(BSON((path ? *path : "") << elem));
+        setData(_backingBSON.firstElement());
+    }
+
     boost::optional<InputParamId> _inputParamId;
 };
 
@@ -258,7 +311,7 @@ public:
                               clonable_ptr<ErrorAnnotation> annotation = nullptr,
                               const CollatorInterface* collator = nullptr);
 
-    virtual ~ComparisonMatchExpression() = default;
+    ~ComparisonMatchExpression() override = default;
 
     bool matchesSingleElement(const BSONElement&, MatchDetails* details = nullptr) const final;
 };
@@ -287,6 +340,9 @@ public:
     std::unique_ptr<MatchExpression> clone() const final {
         std::unique_ptr<ComparisonMatchExpression> e =
             std::make_unique<EqualityMatchExpression>(path(), getData(), _errorAnnotation);
+        if (_backingBSONIsSet) {
+            e->setBackingBSON(_backingBSON);
+        }
         if (getTag()) {
             e->setTag(getTag()->clone());
         }
@@ -328,6 +384,9 @@ public:
     std::unique_ptr<MatchExpression> clone() const final {
         std::unique_ptr<ComparisonMatchExpression> e =
             std::make_unique<LTEMatchExpression>(path(), _rhs, _errorAnnotation);
+        if (_backingBSONIsSet) {
+            e->setBackingBSON(_backingBSON);
+        }
         if (getTag()) {
             e->setTag(getTag()->clone());
         }
@@ -369,6 +428,9 @@ public:
     std::unique_ptr<MatchExpression> clone() const final {
         std::unique_ptr<ComparisonMatchExpression> e =
             std::make_unique<LTMatchExpression>(path(), _rhs, _errorAnnotation);
+        if (_backingBSONIsSet) {
+            e->setBackingBSON(_backingBSON);
+        }
         if (getTag()) {
             e->setTag(getTag()->clone());
         }
@@ -415,6 +477,9 @@ public:
     std::unique_ptr<MatchExpression> clone() const final {
         std::unique_ptr<ComparisonMatchExpression> e =
             std::make_unique<GTMatchExpression>(path(), _rhs, _errorAnnotation);
+        if (_backingBSONIsSet) {
+            e->setBackingBSON(_backingBSON);
+        }
         if (getTag()) {
             e->setTag(getTag()->clone());
         }
@@ -460,6 +525,9 @@ public:
     std::unique_ptr<MatchExpression> clone() const final {
         std::unique_ptr<ComparisonMatchExpression> e =
             std::make_unique<GTEMatchExpression>(path(), _rhs, _errorAnnotation);
+        if (_backingBSONIsSet) {
+            e->setBackingBSON(_backingBSON);
+        }
         if (getTag()) {
             e->setTag(getTag()->clone());
         }
@@ -501,7 +569,7 @@ public:
                          StringData options,
                          clonable_ptr<ErrorAnnotation> annotation = nullptr);
 
-    ~RegexMatchExpression();
+    ~RegexMatchExpression() override;
 
     std::unique_ptr<MatchExpression> clone() const final {
         std::unique_ptr<RegexMatchExpression> e =
@@ -520,7 +588,7 @@ public:
 
     bool matchesSingleElement(const BSONElement&, MatchDetails* details = nullptr) const final;
 
-    virtual void debugString(StringBuilder& debug, int indentationLevel) const;
+    void debugString(StringBuilder& debug, int indentationLevel) const override;
 
     void appendSerializedRightHandSide(BSONObjBuilder* bob,
                                        const SerializationOptions& opts = {},
@@ -530,7 +598,7 @@ public:
 
     void shortDebugString(StringBuilder& debug) const;
 
-    virtual bool equivalent(const MatchExpression* other) const;
+    bool equivalent(const MatchExpression* other) const override;
 
     const std::string& getString() const {
         return _regex;
@@ -604,13 +672,13 @@ public:
 
     bool matchesSingleElement(const BSONElement&, MatchDetails* details = nullptr) const final;
 
-    virtual void debugString(StringBuilder& debug, int indentationLevel) const;
+    void debugString(StringBuilder& debug, int indentationLevel) const override;
 
     void appendSerializedRightHandSide(BSONObjBuilder* bob,
                                        const SerializationOptions& opts = {},
                                        bool includePath = true) const final;
 
-    virtual bool equivalent(const MatchExpression* other) const;
+    bool equivalent(const MatchExpression* other) const override;
 
     long long getDivisor() const {
         return _divisor;
@@ -662,7 +730,7 @@ public:
     explicit ExistsMatchExpression(boost::optional<StringData> path,
                                    clonable_ptr<ErrorAnnotation> annotation = nullptr);
 
-    virtual std::unique_ptr<MatchExpression> clone() const {
+    std::unique_ptr<MatchExpression> clone() const override {
         std::unique_ptr<ExistsMatchExpression> e =
             std::make_unique<ExistsMatchExpression>(path(), _errorAnnotation);
         if (getTag()) {
@@ -673,13 +741,13 @@ public:
 
     bool matchesSingleElement(const BSONElement&, MatchDetails* details = nullptr) const final;
 
-    virtual void debugString(StringBuilder& debug, int indentationLevel) const;
+    void debugString(StringBuilder& debug, int indentationLevel) const override;
 
     void appendSerializedRightHandSide(BSONObjBuilder* bob,
                                        const SerializationOptions& opts = {},
                                        bool includePath = true) const final;
 
-    virtual bool equivalent(const MatchExpression* other) const;
+    bool equivalent(const MatchExpression* other) const override;
 
     void acceptVisitor(MatchExpressionMutableVisitor* visitor) final {
         visitor->visit(this);
@@ -705,66 +773,127 @@ public:
     explicit InMatchExpression(boost::optional<StringData> path,
                                clonable_ptr<ErrorAnnotation> annotation = nullptr);
 
+    explicit InMatchExpression(boost::optional<StringData> path,
+                               clonable_ptr<ErrorAnnotation> annotation,
+                               std::shared_ptr<InListData> equalities);
+
     std::unique_ptr<MatchExpression> clone() const final;
 
     bool matchesSingleElement(const BSONElement&, MatchDetails* details = nullptr) const final;
 
-    virtual void debugString(StringBuilder& debug, int indentationLevel) const;
+    void debugString(StringBuilder& debug, int indentationLevel) const override;
 
     void appendSerializedRightHandSide(BSONObjBuilder* bob,
                                        const SerializationOptions& opts = {},
                                        bool includePath = true) const final;
 
-    virtual bool equivalent(const MatchExpression* other) const;
+    bool equivalent(const MatchExpression* other) const override;
 
     /**
      * 'collator' must outlive the InMatchExpression and any clones made of it.
      */
-    virtual void _doSetCollator(const CollatorInterface* collator);
-
-    Status setEqualities(std::vector<BSONElement> equalities);
-
-    void setBackingBSON(BSONObj equalityStorage);
-
-    Status addRegex(std::unique_ptr<RegexMatchExpression> expr);
+    void _doSetCollator(const CollatorInterface* collator) override;
 
     const std::vector<BSONElement>& getEqualities() const {
-        return _equalitySet;
+        return _equalities->getElements();
     }
 
-    bool contains(const BSONElement& e) const;
+    bool isBSONOwned() const {
+        return _equalities->isBSONOwned();
+    }
+
+    const BSONObj& getOwnedBSONStorage() const {
+        return _equalities->getOwnedBSONStorage();
+    }
+
+    void makeBSONOwned() {
+        cloneEqualitiesBeforeWriteIfNeeded();
+        _equalities->makeBSONOwned();
+    }
 
     const std::vector<std::unique_ptr<RegexMatchExpression>>& getRegexes() const {
         return _regexes;
     }
 
     const CollatorInterface* getCollator() const {
-        return _collator;
+        return _equalities->getCollator();
     }
 
-    bool hasNull() const {
-        return _hasNull;
+    /**
+     * Sets the equalities to 'bsonArray'. If 'bool(fn)' is true, then 'fn' will be invoked
+     * for each element in 'bsonArray'. If 'fn' returns a non-OK Status for any element, this
+     * function will immediately break and return that Status. If 'bool(fn)' is false, then
+     * 'fn' is ignored.
+     *
+     * This function will throw an error if any value in 'bsonArray' is Undefined. Any Regex
+     * values in 'bsonArray' will get passed to 'fn()' (assuming that 'bool(fn)' is true) but
+     * otherwise will be ignored.
+     */
+    Status setEqualitiesArray(BSONObj bsonArray,
+                              const std::function<Status(const BSONElement&)>& fn) {
+        cloneEqualitiesBeforeWriteIfNeeded();
+        constexpr bool errorOnRegex = false;
+        return _equalities->setElementsArray(std::move(bsonArray), errorOnRegex, fn);
+    }
+
+    /**
+     * Sets the equalities to 'bsonArray'. This function will throw an error if any value in
+     * 'bsonArray' is Regex or Undefined.
+     */
+    Status setEqualitiesArray(BSONObj bsonArray) {
+        cloneEqualitiesBeforeWriteIfNeeded();
+        return _equalities->setElementsArray(std::move(bsonArray));
+    }
+
+    /**
+     * Sets the equalities to 'equalities'. This function will throw an error if any value in
+     * 'equalities' is Regex or Undefined.
+     */
+    Status setEqualities(std::vector<BSONElement> equalities) {
+        cloneEqualitiesBeforeWriteIfNeeded();
+        return _equalities->setElements(std::move(equalities));
+    }
+
+    Status addRegex(std::unique_ptr<RegexMatchExpression> expr);
+
+    bool contains(const BSONElement& e) const {
+        return _equalities->contains(e);
     }
 
     bool hasRegex() const {
         return !_regexes.empty();
     }
 
+    bool hasNull() const {
+        return _equalities->hasNull();
+    }
+    bool hasArray() const {
+        return _equalities->hasArray();
+    }
+    bool hasObject() const {
+        return _equalities->hasObject();
+    }
     bool hasEmptyArray() const {
-        return _hasEmptyArray;
+        return _equalities->hasEmptyArray();
     }
-
     bool hasEmptyObject() const {
-        return _hasEmptyObject;
+        return _equalities->hasEmptyObject();
+    }
+    bool hasNonEmptyArray() const {
+        return _equalities->hasNonEmptyArray();
+    }
+    bool hasNonEmptyObject() const {
+        return _equalities->hasNonEmptyObject();
     }
-
     bool hasNonEmptyArrayOrObject() const {
-        return _hasNonEmptyArrayOrObject;
+        return hasNonEmptyArray() || hasNonEmptyObject();
     }
-
     bool hasNonScalarOrNonEmptyValues() const {
         return hasNonEmptyArrayOrObject() || hasNull() || hasRegex();
     }
+    bool equalitiesHasSingleElement() const {
+        return _equalities->hasSingleElement();
+    }
 
     void acceptVisitor(MatchExpressionMutableVisitor* visitor) final {
         visitor->visit(this);
@@ -782,55 +911,49 @@ public:
         return _inputParamId;
     }
 
+    // This method returns a 'shared_ptr<const InListData>' that points to this InMatchExpression's
+    // internal InListData object.
+    //
+    // You should only call this method if you specifically need a shared_ptr to the InListData
+    // for some reason. If you just need information about this InMatchExpression or its elements,
+    // you should use different InMatchExpression methods that provide the information you need.
+    std::shared_ptr<const InListData> getInListDataPtr() const {
+        // Mark '_equalities' as "shared" (if it's not already marked) before exposing a reference
+        // to '_equalities' outside of this class. This will prevent the InListData object from
+        // potentially being mutated in the future.
+        _equalities->setShared();
+
+        // We don't want to return a non-const reference to the contents of this InMatchExpression
+        // (because this is a const method), so it's important to use "shared_ptr<const InListData>"
+        // here instead of "shared_ptr<InListData>".
+        return std::shared_ptr<const InListData>{_equalities};
+    }
+
 private:
+    // If references to '_equalities' has been exposed outside this object (as indicated by the
+    // '_shared' flag), then this method will make a copy of '_equalities'. Otherwise this method
+    // will do nothing. After this method returns, the caller is guaranteed 'equalities->isShared()'
+    // will return false.
+    MONGO_COMPILER_ALWAYS_INLINE void cloneEqualitiesBeforeWriteIfNeeded() {
+        // If '_equalities' is marked as "shared" then it cannot be modified, so make a copy and
+        // then update '_equalities' to point to the copy.
+        if (_equalities->isShared()) {
+            _equalities = _equalities->clone();
+        }
+    }
+
     ExpressionOptimizerFunc getOptimizer() const final;
 
-    /**
-     * A helper to serialize to something like {$in: "?array<?number>"} or similar, depending on
-     * 'opts' and whether we have a mixed-type $in or not.
-     */
+    // A helper to serialize to something like {$in: "?array<?number>"} or similar, depending on
+    // 'opts' and whether we have a mixed-type $in or not.
     void serializeToShape(BSONObjBuilder* bob, const SerializationOptions& opts) const;
 
-    // Whether or not '_equalities' has a jstNULL element in it.
-    bool _hasNull = false;
-
-    // Whether or not '_equalities' has an empty array element in it.
-    bool _hasEmptyArray = false;
-
-    // Whether or not '_equalities' has an empty object element in it.
-    bool _hasEmptyObject = false;
-
-    // Whether or not '_equalities' has a non-empty array or object element in it.
-    bool _hasNonEmptyArrayOrObject = false;
-
-    // Collator used to construct '_eltCmp';
-    const CollatorInterface* _collator = nullptr;
-
-    // Comparator used to compare elements. By default, simple binary comparison will be used.
-    BSONElementComparator _eltCmp;
-
-    // Original container of equality elements, including duplicates. Needed for re-computing
-    // '_equalitySet' in case '_collator' changes after elements have been added.
-    //
-    // We keep the equalities in sorted order according to the current BSON element comparator. This
-    // enables a fast-path to avoid re-sorting if the expression is serialized and re-parsed.
-    std::vector<BSONElement> _originalEqualityVector;
-
-    // Deduped set of equality elements associated with this expression. Kept in sorted order to
-    // support std::binary_search. Because we need to sort the elements anyway for things like index
-    // bounds building, using binary search avoids the overhead of inserting into a hash table which
-    // doesn't pay for itself in the common case where lookups are done a few times if ever.
-    // TODO It may be worth dynamically creating a hashset after matchesSingleElement() has been
-    // called "many" times.
-    std::vector<BSONElement> _equalitySet;
+    // List of equalities (excluding regexes).
+    std::shared_ptr<InListData> _equalities;
 
     // Container of regex elements this object owns.
     std::vector<std::unique_ptr<RegexMatchExpression>> _regexes;
 
-    // When this $in is generated internally, e.g. via a rewrite, this is where we store the
-    // data of the corresponding equality elements.
-    BSONObj _equalityStorage;
-
     boost::optional<InputParamId> _inputParamId;
 };
 
@@ -856,17 +979,17 @@ public:
                                     const char* bitMaskBinary,
                                     uint32_t bitMaskLen,
                                     clonable_ptr<ErrorAnnotation> annotation);
-    virtual ~BitTestMatchExpression() {}
+    ~BitTestMatchExpression() override {}
 
     bool matchesSingleElement(const BSONElement&, MatchDetails* details = nullptr) const final;
 
-    virtual void debugString(StringBuilder& debug, int indentationLevel) const;
+    void debugString(StringBuilder& debug, int indentationLevel) const override;
 
     void appendSerializedRightHandSide(BSONObjBuilder* bob,
                                        const SerializationOptions& opts = {},
                                        bool includePath = true) const final;
 
-    virtual bool equivalent(const MatchExpression* other) const;
+    bool equivalent(const MatchExpression* other) const override;
 
     size_t numBitPositions() const {
         return _bitPositions.size();
@@ -932,7 +1055,7 @@ private:
     // Used to perform bit tests against numbers using a single bitwise operation.
     uint64_t _bitMask = 0;
 
-    // When this expression is parameterized, we require two parmeter markers, not one: a parameter
+    // When this expression is parameterized, we require two parameter markers, not one: a parameter
     // marker for the vector of bit positions and a second for the bitmask. The runtime plan
     // needs both values so that it can operate against either BinData or numerical inputs.
     boost::optional<InputParamId> _bitPositionsParamId;
@@ -944,7 +1067,8 @@ public:
     BitsAllSetMatchExpression(boost::optional<StringData> path,
                               std::vector<uint32_t> bitPositions,
                               clonable_ptr<ErrorAnnotation> annotation = nullptr)
-        : BitTestMatchExpression(BITS_ALL_SET, path, bitPositions, std::move(annotation)) {}
+        : BitTestMatchExpression(
+              BITS_ALL_SET, path, std::move(bitPositions), std::move(annotation)) {}
 
     BitsAllSetMatchExpression(boost::optional<StringData> path,
                               uint64_t bitMask,
@@ -988,7 +1112,8 @@ public:
     BitsAllClearMatchExpression(boost::optional<StringData> path,
                                 std::vector<uint32_t> bitPositions,
                                 clonable_ptr<ErrorAnnotation> annotation = nullptr)
-        : BitTestMatchExpression(BITS_ALL_CLEAR, path, bitPositions, std::move(annotation)) {}
+        : BitTestMatchExpression(
+              BITS_ALL_CLEAR, path, std::move(bitPositions), std::move(annotation)) {}
 
     BitsAllClearMatchExpression(boost::optional<StringData> path,
                                 uint64_t bitMask,
@@ -1032,7 +1157,8 @@ public:
     BitsAnySetMatchExpression(boost::optional<StringData> path,
                               std::vector<uint32_t> bitPositions,
                               clonable_ptr<ErrorAnnotation> annotation = nullptr)
-        : BitTestMatchExpression(BITS_ANY_SET, path, bitPositions, std::move(annotation)) {}
+        : BitTestMatchExpression(
+              BITS_ANY_SET, path, std::move(bitPositions), std::move(annotation)) {}
 
     BitsAnySetMatchExpression(boost::optional<StringData> path,
                               uint64_t bitMask,
@@ -1076,7 +1202,8 @@ public:
     BitsAnyClearMatchExpression(boost::optional<StringData> path,
                                 std::vector<uint32_t> bitPositions,
                                 clonable_ptr<ErrorAnnotation> annotation = nullptr)
-        : BitTestMatchExpression(BITS_ANY_CLEAR, path, bitPositions, std::move(annotation)) {}
+        : BitTestMatchExpression(
+              BITS_ANY_CLEAR, path, std::move(bitPositions), std::move(annotation)) {}
 
     BitsAnyClearMatchExpression(boost::optional<StringData> path,
                                 uint64_t bitMask,
