@@ -27,16 +27,40 @@
  *    it in the license file.
  */
 
-#include "mongo/db/pipeline/expression.h"
-
+#include <memory>
+#include <string>
+#include <utility>
 #include <vector>
 
+#include <boost/move/utility_core.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+
+#include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/json.h"
+#include "mongo/db/matcher/expression.h"
 #include "mongo/db/matcher/expression_always_boolean.h"
+#include "mongo/db/matcher/expression_leaf.h"
+#include "mongo/db/matcher/expression_parser.h"
+#include "mongo/db/matcher/expression_tree.h"
+#include "mongo/db/matcher/extensions_callback_noop.h"
+#include "mongo/db/namespace_string.h"
 #include "mongo/db/pipeline/expression_context_for_test.h"
 #include "mongo/db/query/canonical_query.h"
 #include "mongo/db/query/collation/collator_interface_mock.h"
+#include "mongo/db/query/find_command.h"
 #include "mongo/db/query/index_tag.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/db/query/parsed_find_command.h"
+#include "mongo/db/query/query_request_helper.h"
+#include "mongo/db/query/tailable_mode_gen.h"
+#include "mongo/idl/server_parameter_test_util.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/bson_test_util.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/str.h"
 
 namespace mongo {
 namespace {
@@ -44,7 +68,6 @@ namespace {
 static const NamespaceString nss =
     NamespaceString::createNamespaceString_forTest("testdb.testcoll");
 
-using unittest::assertGet;
 
 /**
  * Helper function to parse the given BSON object as a MatchExpression, checks the status,
@@ -105,8 +128,8 @@ TEST(ExpressionOptimizeTest, IsValidText) {
     // Invalid: TEXT inside NOR.
     ASSERT_NOT_OK(isValid("{$nor: [{$text: {$search: 's'}}, {a: 1}]}", *findCommand));
 
-    // Invalid: TEXT inside NOR.
-    ASSERT_NOT_OK(
+    // Valid: Boolean expression simplifier opens up $nor expressions.
+    ASSERT_OK(
         isValid("{$nor: ["
                 "    {$or: ["
                 "        {$text: {$search: 's'}},"
@@ -366,7 +389,6 @@ TEST(ExpressionOptimizeTest, AndWithSingleChildAlwaysTrueOptimizesToEmptyAnd) {
     BSONObj obj = fromjson("{$and: [{$alwaysTrue: 1}]}");
     std::unique_ptr<MatchExpression> matchExpression(parseMatchExpression(obj));
     auto optimizedMatchExpression = MatchExpression::optimize(std::move(matchExpression));
-    // TODO SERVER-34759 We want this to optimize to an AlwaysTrueMatchExpression.
     ASSERT_TRUE(dynamic_cast<AndMatchExpression*>(optimizedMatchExpression.get()));
     ASSERT_BSONOBJ_EQ(optimizedMatchExpression->serialize(), fromjson("{}"));
 }
@@ -375,7 +397,6 @@ TEST(ExpressionOptimizeTest, AndWithEachChildAlwaysTrueOptimizesToEmptyAnd) {
     BSONObj obj = fromjson("{$and: [{$alwaysTrue: 1}, {$alwaysTrue: 1}]}");
     std::unique_ptr<MatchExpression> matchExpression(parseMatchExpression(obj));
     auto optimizedMatchExpression = MatchExpression::optimize(std::move(matchExpression));
-    // TODO SERVER-34759 We want this to optimize to an AlwaysTrueMatchExpression.
     ASSERT_TRUE(dynamic_cast<AndMatchExpression*>(optimizedMatchExpression.get()));
     ASSERT_BSONOBJ_EQ(optimizedMatchExpression->serialize(), fromjson("{}"));
 }
@@ -391,7 +412,7 @@ TEST(ExpressionOptimizeTest, OrWithAlwaysTrueOptimizesToAlwaysTrue) {
     BSONObj obj = fromjson("{$or: [{a: 1}, {$alwaysTrue: 1}]}");
     std::unique_ptr<MatchExpression> matchExpression(parseMatchExpression(obj));
     auto optimizedMatchExpression = MatchExpression::optimize(std::move(matchExpression));
-    ASSERT_BSONOBJ_EQ(optimizedMatchExpression->serialize(), fromjson("{$alwaysTrue: 1}"));
+    ASSERT_BSONOBJ_EQ(optimizedMatchExpression->serialize(), fromjson("{}"));
 }
 
 TEST(ExpressionOptimizeTest, OrRemovesAlwaysFalseChildren) {
@@ -430,7 +451,7 @@ TEST(ExpressionOptimizeTest, NestedOrWithAlwaysTrueOptimizesToAlwaysTrue) {
     BSONObj obj = fromjson("{$or: [{$or: [{$alwaysTrue: 1}, {a: 1}]}, {b: 1}]}");
     std::unique_ptr<MatchExpression> matchExpression(parseMatchExpression(obj));
     auto optimizedMatchExpression = MatchExpression::optimize(std::move(matchExpression));
-    ASSERT_BSONOBJ_EQ(optimizedMatchExpression->serialize(), fromjson("{$alwaysTrue: 1}"));
+    ASSERT_BSONOBJ_EQ(optimizedMatchExpression->serialize(), fromjson("{}"));
 }
 
 TEST(ExpressionOptimizeTest, OrRewrittenToIn) {
@@ -476,15 +497,29 @@ TEST(ExpressionOptimizeTest, OrRewrittenToIn) {
 TEST(ExpressionOptimizeTest, OrRewrittenToInWithParameters) {
     BSONObj obj = fromjson("{$or: [{f1: {$eq: 3}}, {f1: {$eq: 4}}]}");
     std::unique_ptr<MatchExpression> matchExpr(parseMatchExpression(obj));
-    MatchExpression::parameterize(matchExpr.get());
+    bool parameterized;
+    MatchExpression::parameterize(
+        matchExpr.get(), boost::none /*maxParamCount=*/, 0 /*startingParamId=*/, &parameterized);
+    ASSERT_TRUE(parameterized);
     ASSERT_BSONOBJ_EQ(matchExpr->serialize(), obj);
 }
 
 TEST(ExpressionOptimizeTest, PartialOrToInRewriteDoesNotGenerateDirectlyNestedOr) {
-    BSONObj obj = fromjson("{$or: [{x: {$eq: 3}}, {x: {$eq: 4}}, {y: 5}, {z: 6}]}");
-    auto optimizedMatchExpression = MatchExpression::optimize(parseMatchExpression(obj));
-    ASSERT_BSONOBJ_EQ(optimizedMatchExpression->serialize(),
-                      fromjson("{$or: [{x: {$in: [3, 4]}}, {y: {$eq: 5}}, {z: {$eq: 6}}]}"));
+    // Test with the boolean simplifier on and off. In both cases, we should get the same result
+    // expression after optimization. This makes sure to test that 'MatchExpression::optimize()'
+    // itself does not unnecessarily generate intermediate nested $or nodes as described in
+    // SERVER-83602.
+    for (auto booleanSimplificationEnabled : {false, true}) {
+        RAIIServerParameterControllerForTest simplifierParamController(
+            "internalQueryEnableBooleanExpressionsSimplifier", booleanSimplificationEnabled);
+        BSONObj obj = fromjson("{$or: [{x: {$eq: 3}}, {x: {$eq: 4}}, {y: 5}, {z: 6}]}");
+        // We call MatchExpression::normalize() here to ensure the stability of the predicate order
+        // after optimizations.
+        auto optimizedAndNormalizedMatchExpression =
+            MatchExpression::normalize(parseMatchExpression(obj));
+        ASSERT_BSONOBJ_EQ(optimizedAndNormalizedMatchExpression->serialize(),
+                          fromjson("{$or: [{y: {$eq: 5}}, {z: {$eq: 6}}, {x: {$in: [3, 4]}}]}"));
+    }
 }
 
 TEST(ExpressionOptimizeTest, NorRemovesAlwaysFalseChildren) {
@@ -508,5 +543,20 @@ TEST(ExpressionOptimizeTest, NorWithAlwaysTrueChildOptimizesToAlwaysFalse) {
     auto optimizedMatchExpression = MatchExpression::optimize(std::move(matchExpression));
     ASSERT_BSONOBJ_EQ(optimizedMatchExpression->serialize(), fromjson("{$alwaysFalse: 1}"));
 }
+
+TEST(ExpressionOptimizeTest, EmptyInOptimizesToAlwaysFalse) {
+    BSONObj obj = fromjson("{x: {$in: []}}");
+    std::unique_ptr<MatchExpression> matchExpression(parseMatchExpression(obj));
+    auto optimizedMatchExpression = MatchExpression::optimize(std::move(matchExpression));
+    ASSERT_BSONOBJ_EQ(optimizedMatchExpression->serialize(), fromjson("{$alwaysFalse: 1}"));
+}
+
+TEST(ExpressionOptimizeTest, InWithJustRegexesIsNotOptimizedToAlwaysFalse) {
+    BSONObj obj = fromjson("{x: {$in: [/foo/, /bar/]}}");
+    std::unique_ptr<MatchExpression> matchExpression(parseMatchExpression(obj));
+    auto optimizedMatchExpression = MatchExpression::optimize(std::move(matchExpression));
+    ASSERT_BSONOBJ_EQ(optimizedMatchExpression->serialize(),
+                      fromjson("{x: {$in: [/foo/, /bar/]}}"));
+}
 }  // namespace
 }  // namespace mongo
