@@ -28,10 +28,36 @@
  */
 
 #include "mongo/db/matcher/expression_parameterization.h"
-#include "mongo/db/operation_context.h"
+
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <cstddef>
+#include <cstdint>
+#include <limits>
+#include <memory>
+#include <utility>
+
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/bson/bsontypes_util.h"
+#include "mongo/db/matcher/expression_parser.h"
+#include "mongo/db/matcher/expression_tree.h"
+#include "mongo/db/matcher/matcher_type_set.h"
+#include "mongo/db/matcher/schema/expression_internal_schema_allowed_properties.h"
+#include "mongo/db/pipeline/expression_context.h"
 #include "mongo/db/pipeline/expression_context_for_test.h"
-#include "mongo/db/query/query_planner_params.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/db/query/tree_walker.h"
+#include "mongo/platform/decimal128.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/intrusive_counter.h"
 
 namespace mongo {
 namespace {
@@ -42,12 +68,12 @@ void walkExpression(MatchExpressionParameterizationVisitorContext* context,
     tree_walker::walk<false, MatchExpression>(expression, &walker);
 }
 
-struct CountInputParamsContext {
-    size_t count{0};
+struct CollectInputParamsContext {
+    std::vector<size_t> paramIds;
 };
-class CountInputParamsVisitor : public MatchExpressionConstVisitor {
+class CollectInputParamsVisitor : public MatchExpressionConstVisitor {
 public:
-    explicit CountInputParamsVisitor(CountInputParamsContext* context) : _context(context) {}
+    explicit CollectInputParamsVisitor(CollectInputParamsContext* context) : _context(context) {}
 
     void visit(const AlwaysFalseMatchExpression* expr) final {}
     void visit(const AlwaysTrueMatchExpression* expr) final {}
@@ -117,7 +143,7 @@ public:
     }
     void visit(const ModMatchExpression* expr) final {
         countParam(expr->getDivisorInputParamId());
-        countParam(expr->getRemainder());
+        countParam(expr->getRemainderInputParamId());
     }
     void visit(const NorMatchExpression* expr) final {}
     void visit(const NotMatchExpression* expr) final {}
@@ -152,16 +178,16 @@ private:
 
     void countParam(boost::optional<MatchExpression::InputParamId> param) {
         if (param) {
-            _context->count++;
+            _context->paramIds.push_back(*param);
         }
     }
 
-    CountInputParamsContext* _context;
+    CollectInputParamsContext* _context;
 };
 
-class CountInputParamWalker {
+class CollectInputParamWalker {
 public:
-    explicit CountInputParamWalker(CountInputParamsVisitor* visitor) : _visitor{visitor} {
+    explicit CollectInputParamWalker(CollectInputParamsVisitor* visitor) : _visitor{visitor} {
         invariant(_visitor);
     }
 
@@ -174,18 +200,18 @@ public:
     void inVisit(long count, const MatchExpression* expr) {}
 
 private:
-    CountInputParamsVisitor* _visitor;
+    CollectInputParamsVisitor* _visitor;
 };
 
 /**
- * Return an number of parameters that are set within given expression tree.
+ * Return the list of parameter numbers that are set within given expression tree.
  */
-size_t countInputParams(const MatchExpression* expression) {
-    CountInputParamsContext context;
-    CountInputParamsVisitor visitor{&context};
-    CountInputParamWalker walker{&visitor};
+std::vector<size_t> collectInputParams(const MatchExpression* expression) {
+    CollectInputParamsContext context;
+    CollectInputParamsVisitor visitor{&context};
+    CollectInputParamWalker walker{&visitor};
     tree_walker::walk<true, MatchExpression>(expression, &walker);
-    return context.count;
+    return context.paramIds;
 }
 
 }  // namespace
@@ -356,6 +382,39 @@ TEST(MatchExpressionParameterizationVisitor, ComparisonMatchExpressionsWithNaNSe
     ASSERT_EQ(0, context.inputParamIdToExpressionMap.size());
 }
 
+TEST(MatchExpressionParameterizationVisitor, ComparisonMatchExpressionsWithBooleanSetsNoParamIds) {
+    std::vector<std::unique_ptr<MatchExpression>> expressions;
+    expressions.emplace_back(std::make_unique<LTMatchExpression>("a"_sd, Value(false)));
+    expressions.emplace_back(std::make_unique<GTMatchExpression>("b"_sd, Value(true)));
+    expressions.emplace_back(std::make_unique<EqualityMatchExpression>("c"_sd, Value(false)));
+    expressions.emplace_back(std::make_unique<LTEMatchExpression>("d"_sd, Value(true)));
+    expressions.emplace_back(std::make_unique<GTEMatchExpression>("e"_sd, Value(false)));
+
+    OrMatchExpression expr{std::move(expressions)};
+
+    MatchExpressionParameterizationVisitorContext context{};
+    walkExpression(&context, &expr);
+
+    ASSERT_EQ(0, context.inputParamIdToExpressionMap.size());
+}
+
+TEST(MatchExpressionParameterizationVisitor,
+     ComparisonMatchExpressionsWithEmptyStringSetsNoParamIds) {
+    std::vector<std::unique_ptr<MatchExpression>> expressions;
+    expressions.emplace_back(std::make_unique<LTMatchExpression>("a"_sd, Value(""_sd)));
+    expressions.emplace_back(std::make_unique<GTMatchExpression>("b"_sd, Value(""_sd)));
+    expressions.emplace_back(std::make_unique<EqualityMatchExpression>("c"_sd, Value(""_sd)));
+    expressions.emplace_back(std::make_unique<LTEMatchExpression>("d"_sd, Value(""_sd)));
+    expressions.emplace_back(std::make_unique<GTEMatchExpression>("e"_sd, Value(""_sd)));
+
+    OrMatchExpression expr{std::move(expressions)};
+
+    MatchExpressionParameterizationVisitorContext context{};
+    walkExpression(&context, &expr);
+
+    ASSERT_EQ(0, context.inputParamIdToExpressionMap.size());
+}
+
 TEST(MatchExpressionParameterizationVisitor, InMatchExpressionWithScalarsSetsOneParamId) {
     BSONObj operand = BSON_ARRAY(1 << "r" << true << 1.1);
     InMatchExpression expr{"a"_sd};
@@ -403,7 +462,7 @@ TEST(MatchExpressionParameterizationVisitor, InMatchExpressionWithNullSetsNoPara
 TEST(MatchExpressionParameterizationVisitor, InMatchExpressionWithRegexSetsNoParamIds) {
     BSONObj query = BSON("a" << BSON("$in" << BSON_ARRAY(BSONRegEx("/^regex/i"))));
 
-    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    auto expCtx = make_intrusive<ExpressionContextForTest>();
     StatusWithMatchExpression result = MatchExpressionParser::parse(query, expCtx);
     ASSERT_TRUE(result.isOK());
 
@@ -464,7 +523,7 @@ TEST(MatchExpressionParameterizationVisitor, ExprMatchExpressionSetsNoParamsIds)
     BSONObj query = BSON("$expr" << BSON("$gte" << BSON_ARRAY("$a"
                                                               << "$b")));
 
-    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    auto expCtx = make_intrusive<ExpressionContextForTest>();
     StatusWithMatchExpression result = MatchExpressionParser::parse(query, expCtx);
     ASSERT_TRUE(result.isOK());
 
@@ -476,7 +535,7 @@ TEST(MatchExpressionParameterizationVisitor, ExprMatchExpressionSetsNoParamsIds)
 TEST(MatchExpressionParameterizationVisitor, OrMatchExpressionSetsOneParam) {
     BSONObj query = BSON("$or" << BSON_ARRAY(BSON("a" << 1) << BSON("a" << 1)));
 
-    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    auto expCtx = make_intrusive<ExpressionContextForTest>();
     StatusWithMatchExpression result = MatchExpressionParser::parse(query, expCtx);
     ASSERT_TRUE(result.isOK());
 
@@ -489,7 +548,7 @@ TEST(MatchExpressionParameterizationVisitor, OrMatchExpressionSetsThreeParamWith
     BSONObj query =
         BSON("$or" << BSON_ARRAY(BSON("a" << 1) << BSON("b" << 2)) << "a" << 1 << "b" << 3);
 
-    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    auto expCtx = make_intrusive<ExpressionContextForTest>();
     StatusWithMatchExpression result = MatchExpressionParser::parse(query, expCtx);
     ASSERT_TRUE(result.isOK());
 
@@ -498,6 +557,69 @@ TEST(MatchExpressionParameterizationVisitor, OrMatchExpressionSetsThreeParamWith
     ASSERT_EQ(3, context.inputParamIdToExpressionMap.size());
 }
 
+TEST(MatchExpressionParameterizationVisitor,
+     MapIsUsedForParamIdLookupsAfterContextSizeExceedsThreshold) {
+    MatchExpressionParameterizationVisitorContext context{};
+    // Ensure lifetime of BSONObjs matches that of MatchExpressions.
+    std::vector<BSONObj> expressionBSONs;
+    std::vector<std::unique_ptr<MatchExpression>> expressions;
+
+    auto expectedSize = context.kUseMapThreshold;
+
+    auto addExpressions = [&]() {
+        for (size_t i = 0; i < expectedSize; i++) {
+            BSONObj gt = BSON("$gt" << static_cast<int>(i));
+            expressionBSONs.push_back(gt);
+            expressions.emplace_back(std::make_unique<GTMatchExpression>("a"_sd, gt["$gt"]));
+        }
+    };
+
+    // Add the same set of 50 expressions twice. For duplicate expressions, param ids will be reused
+    // by doing a lookup on the map.
+    addExpressions();
+    addExpressions();
+
+    OrMatchExpression expr{std::move(expressions)};
+    walkExpression(&context, &expr);
+
+    // Ensure param ids were reused for duplicate expressions.
+    ASSERT_EQ(expectedSize, context.inputParamIdToExpressionMap.size());
+    // The tasserts in MatchExpressionParameterizationVisitorContext::nextReusableInputParamId
+    // ensure the map is actually used for lookups once the amount of input param ids exceeds
+    // kUseMapThreshold.
+    ASSERT_TRUE(context.inputParamIdToExpressionMap.usingMap());
+}
+
+TEST(MatchExpressionParameterizationVisitor,
+     MapIsNotUsedForParamIdLookupsUntilContextSizeExceedsThreshold) {
+    MatchExpressionParameterizationVisitorContext context{};
+    // Ensure lifetime of BSONObjs matches that of MatchExpressions.
+    std::vector<BSONObj> expressionBSONs;
+    std::vector<std::unique_ptr<MatchExpression>> expressions;
+
+    auto expectedSize = context.kUseMapThreshold - 1;
+
+    auto addExpressions = [&]() {
+        for (size_t i = 0; i < expectedSize; i++) {
+            BSONObj gt = BSON("$gt" << static_cast<int>(i));
+            expressionBSONs.push_back(gt);
+            expressions.emplace_back(std::make_unique<GTMatchExpression>("a"_sd, gt["$gt"]));
+        }
+    };
+
+    // Add the same set of 49 expressions twice. For duplicate expressions, param ids will be reused
+    // by doing a lookup on the vector.
+    addExpressions();
+    addExpressions();
+
+    OrMatchExpression expr{std::move(expressions)};
+    walkExpression(&context, &expr);
+
+    // Ensure param ids were reused for duplicate expressions.
+    ASSERT_EQ(expectedSize, context.inputParamIdToExpressionMap.size());
+    ASSERT_FALSE(context.inputParamIdToExpressionMap.usingMap());
+}
+
 TEST(MatchExpressionParameterizationVisitor,
      AutoParametrizationWalkerSetsCorrectNumberOfParamsIds) {
     BSONObj equalityExpr = BSON("x" << 1);
@@ -513,36 +635,151 @@ TEST(MatchExpressionParameterizationVisitor,
                                              << BSON("$and" << BSON_ARRAY(regexExpr << sizeExpr))));
 
     {
-        boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+        auto expCtx = make_intrusive<ExpressionContextForTest>();
         StatusWithMatchExpression result = MatchExpressionParser::parse(query, expCtx);
         ASSERT_TRUE(result.isOK());
-        auto expression = result.getValue().get();
+        MatchExpression* expression = result.getValue().get();
 
         auto inputParamIdToExpressionMap = MatchExpression::parameterize(expression);
         ASSERT_EQ(6, inputParamIdToExpressionMap.size());
-        ASSERT_EQ(6, countInputParams(expression));
+        ASSERT_EQ(6, collectInputParams(expression).size());
     }
 
     {
-        boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+        auto expCtx = make_intrusive<ExpressionContextForTest>();
         StatusWithMatchExpression result = MatchExpressionParser::parse(query, expCtx);
         ASSERT_TRUE(result.isOK());
-        auto expression = result.getValue().get();
+        MatchExpression* expression = result.getValue().get();
 
         auto inputParamIdToExpressionMap = MatchExpression::parameterize(expression, 6);
         ASSERT_EQ(6, inputParamIdToExpressionMap.size());
-        ASSERT_EQ(6, countInputParams(expression));
+        ASSERT_EQ(6, collectInputParams(expression).size());
     }
 
     {
-        boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+        auto expCtx = make_intrusive<ExpressionContextForTest>();
         StatusWithMatchExpression result = MatchExpressionParser::parse(query, expCtx);
         ASSERT_TRUE(result.isOK());
-        auto expression = result.getValue().get();
+        MatchExpression* expression = result.getValue().get();
 
+        // This tests the optional limit on the maximum number of parameters allowed.
         auto inputParamIdToExpressionMap = MatchExpression::parameterize(expression, 5);
-        ASSERT_EQ(0, inputParamIdToExpressionMap.size());
-        ASSERT_EQ(0, countInputParams(expression));
+        ASSERT_EQ(5, inputParamIdToExpressionMap.size());
+        ASSERT_EQ(5, collectInputParams(expression).size());
+    }
+
+    {
+        // Test that the optional number of parameters limit is all-or-nothing for a binary op.
+        BSONObj query2 = BSON("a" << BSON("$mod" << BSON_ARRAY(1 << 2)) << "b"
+                                  << BSON("$mod" << BSON_ARRAY(3 << 4)));
+        auto expCtx = make_intrusive<ExpressionContextForTest>();
+        StatusWithMatchExpression result = MatchExpressionParser::parse(query2, expCtx);
+        ASSERT_TRUE(result.isOK());
+        MatchExpression* expression = result.getValue().get();
+
+        // Both operands of a binary op must be parameterized or not parameterized. There are two
+        // binary ops for a total of four possible parameters, each of which has a unique value so
+        // cannot share a prior parameter. With a limit of 3, the first two get parameterized but
+        // the last two do not.
+        auto inputParamIdToExpressionMap = MatchExpression::parameterize(expression, 3);
+        ASSERT_EQ(2, inputParamIdToExpressionMap.size());
+        ASSERT_EQ(2, collectInputParams(expression).size());
     }
 }
+
+TEST(MatchExpressionParameterizationVisitor, AutoParametrizationWalkerSetsCorrectReusedParamsIds) {
+    BSONObj repeatedEqualityExpr = BSON("x" << 1);
+
+    BSONObj query = BSON("$or" << BSON_ARRAY(repeatedEqualityExpr << repeatedEqualityExpr));
+
+    {
+        auto expCtx = make_intrusive<ExpressionContextForTest>();
+        StatusWithMatchExpression result = MatchExpressionParser::parse(query, expCtx);
+        ASSERT_TRUE(result.isOK());
+        MatchExpression* expression = result.getValue().get();
+
+        auto inputParamIdToExpressionMap = MatchExpression::parameterize(expression);
+        // Check that the two MatchExpression reuse the same parameter.
+        ASSERT_EQ(1, inputParamIdToExpressionMap.size());
+        auto parameters = collectInputParams(expression);
+        ASSERT_EQ(2, parameters.size());
+        ASSERT_EQ(parameters[0], parameters[1]);
+    }
+
+    {
+        auto expCtx = make_intrusive<ExpressionContextForTest>();
+        StatusWithMatchExpression result = MatchExpressionParser::parse(query, expCtx);
+        ASSERT_TRUE(result.isOK());
+        MatchExpression* expression = result.getValue().get();
+
+        auto inputParamIdToExpressionMap =
+            MatchExpression::parameterize(expression, boost::none, 15);
+        ASSERT_EQ(1, inputParamIdToExpressionMap.size());
+        auto parameters = collectInputParams(expression);
+        ASSERT_EQ(2, parameters.size());
+        ASSERT_EQ(parameters[0], parameters[1]);
+        // Check that the number of the parameter is indeed starting with parameter ID that was
+        // specified in the call to parameterize().
+        ASSERT_EQ(parameters[0], 15);
+    }
+
+    {
+        // Create a query where the reused expression is found a first time while the internal
+        // structure has not yet mutated to use a map, and another time when it is already using a
+        // map.
+        // Query will look like:
+        //
+        //  $and: [
+        //     {$or: [{x: 1}, {x: 1}]},
+        //     {y: 0},
+        //     {y: 1},
+        //     ....
+        //     {y: 25},
+        //     {$or: [{x: 1}, {x: 1}]},
+        //     {y: 26},
+        //     ....
+        //     {y: 49},
+        //     {$or: [{x: 1}, {x: 1}]}
+        //  ]
+        BSONArrayBuilder builder;
+        builder << query;
+        for (size_t i = 0; i < MatchExpressionParameterizationVisitorContext::kUseMapThreshold;
+             i++) {
+            builder << BSON("y" << static_cast<int>(i));
+            if (i == MatchExpressionParameterizationVisitorContext::kUseMapThreshold / 2) {
+                builder << query;
+            }
+        }
+        builder << query;
+        BSONObj longQuery = BSON("$and" << builder.arr());
+
+        auto expCtx = make_intrusive<ExpressionContextForTest>();
+        StatusWithMatchExpression result = MatchExpressionParser::parse(longQuery, expCtx);
+        ASSERT_TRUE(result.isOK());
+        MatchExpression* expression = result.getValue().get();
+
+        auto inputParamIdToExpressionMap =
+            MatchExpression::parameterize(expression, boost::none, 15);
+        // There should be kUseMapThreshold expressions plus 1.
+        ASSERT_EQ(MatchExpressionParameterizationVisitorContext::kUseMapThreshold + 1,
+                  inputParamIdToExpressionMap.size());
+        auto parameters = collectInputParams(expression);
+        // There should be kUseMapThreshold plus 3 x 2 references to input parameters.
+        ASSERT_EQ(MatchExpressionParameterizationVisitorContext::kUseMapThreshold + 6,
+                  parameters.size());
+        // The first two expressions and the last two should be the reused paramters.
+        ASSERT_EQ(parameters[0], parameters[1]);
+        ASSERT_EQ(parameters[0], parameters.back());
+        ASSERT_EQ(parameters[0], 15);
+        // Check that no parameter ID is lower than the one specified in the call to
+        // parameterize().
+        auto minIt = std::min_element(parameters.begin(), parameters.end());
+        ASSERT_EQ(*minIt, 15);
+        // Check that the span between lowest and highest parameter ID is equal to the number of
+        // unique parameters.
+        auto maxIt = std::max_element(parameters.begin(), parameters.end());
+        ASSERT_EQ(*maxIt, 15 + inputParamIdToExpressionMap.size() - 1);
+    }
+}
+
 }  // namespace mongo
