@@ -29,8 +29,16 @@
 
 #pragma once
 
+#include <memory>
+
+#include "mongo/bson/bsonelement.h"
+#include "mongo/db/fts/fts_query.h"
 #include "mongo/db/fts/fts_query_noop.h"
+#include "mongo/db/matcher/expression.h"
 #include "mongo/db/matcher/expression_text_base.h"
+#include "mongo/db/matcher/expression_visitor.h"
+#include "mongo/db/matcher/match_details.h"
+#include "mongo/util/assert_util.h"
 
 namespace mongo {
 
