@@ -27,9 +27,18 @@
  *    it in the license file.
  */
 
+#include <cstdint>
+#include <cstring>
+#include <set>
+
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/bson/bsontypes_util.h"
 #include "mongo/bson/json.h"
 #include "mongo/db/matcher/expression_type.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
 
 namespace mongo {
 namespace {
@@ -412,7 +421,7 @@ TEST(InternalSchemaBinDataFLE2EncryptedTypeTest, MatchesOnlyFLE2ServerSubtypes)
     memset(blob.keyUUID, 0, sizeof(blob.keyUUID));
     blob.originalBsonType = BSONType::String;
 
-    for (uint8_t i = 0; i < kNumEncryptedBinDataType; i++) {
+    for (uint8_t i = 0; i < idlEnumCount<EncryptedBinDataType>; i++) {
         blob.fleBlobSubtype = i;
         auto binData = BSONBinData(
             reinterpret_cast<const void*>(&blob), sizeof(FleBlobHeader), BinDataType::Encrypt);
