@@ -27,11 +27,12 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <memory>
+#include <string>
+#include <utility>
 
 #include "mongo/db/matcher/expression_where_noop.h"
-
-#include <memory>
+#include "mongo/util/assert_util.h"
 
 
 namespace mongo {
