@@ -29,7 +29,13 @@
 
 #pragma once
 
+#include <memory>
+
+#include "mongo/db/matcher/expression.h"
+#include "mongo/db/matcher/expression_visitor.h"
 #include "mongo/db/matcher/expression_where_base.h"
+#include "mongo/db/matcher/match_details.h"
+#include "mongo/db/matcher/matchable.h"
 
 namespace mongo {
 
