@@ -27,9 +27,8 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
 #include "mongo/db/matcher/schema/expression_internal_schema_num_properties.h"
+#include "mongo/bson/bsonobj.h"
 
 namespace mongo {
 
