@@ -30,18 +30,34 @@
 
 #include "mongo/db/mongod_options.h"
 
-#include <boost/filesystem.hpp>
+#include <algorithm>
+#include <boost/filesystem.hpp>  // IWYU pragma: keep
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <fmt/format.h>
+#include <initializer_list>
 #include <iostream>
+#include <iterator>
+#include <map>
+#include <memory>
 #include <string>
+#include <utility>
 #include <vector>
 
-#include "mongo/base/init.h"
+#include "mongo/base/error_codes.h"
+#include "mongo/base/init.h"  // IWYU pragma: keep
 #include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/bson/json.h"
+#include "mongo/bson/oid.h"
 #include "mongo/bson/util/builder.h"
-#include "mongo/config.h"
+#include "mongo/bson/util/builder_fwd.h"
+#include "mongo/config.h"  // IWYU pragma: keep
 #include "mongo/db/auth/cluster_auth_mode.h"
 #include "mongo/db/cluster_auth_mode_option_gen.h"
+#include "mongo/db/cluster_role.h"
 #include "mongo/db/global_settings.h"
 #include "mongo/db/keyfile_option_gen.h"
 #include "mongo/db/mongod_options_general_gen.h"
@@ -49,20 +65,33 @@
 #include "mongo/db/mongod_options_replication_gen.h"
 #include "mongo/db/mongod_options_sharding_gen.h"
 #include "mongo/db/mongod_options_storage_gen.h"
+#include "mongo/db/repl/repl_server_parameters_gen.h"
+#include "mongo/db/repl/repl_set_config_params_gen.h"
 #include "mongo/db/repl/repl_settings.h"
+#include "mongo/db/server_feature_flags_gen.h"
 #include "mongo/db/server_options.h"
 #include "mongo/db/server_options_base.h"
 #include "mongo/db/server_options_nongeneral_gen.h"
 #include "mongo/db/server_options_server_helpers.h"
-#include "mongo/db/storage/storage_parameters_gen.h"
+#include "mongo/db/server_options_upgrade_downgrade_gen.h"
+#include "mongo/db/service_context.h"
+#include "mongo/db/storage/storage_options.h"
 #include "mongo/logv2/log.h"
-#include "mongo/logv2/log_domain_global.h"
-#include "mongo/logv2/log_manager.h"
-#include "mongo/util/net/ssl_options.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/atomic_proxy.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/s/sharding_feature_flags_gen.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/options_parser/startup_options.h"
 #include "mongo/util/str.h"
 #include "mongo/util/version.h"
 
+#if defined(MONGO_CONFIG_HAVE_HEADER_UNISTD_H)
+#include <unistd.h>
+#endif
+
+
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kControl
 
 
@@ -95,6 +124,7 @@ Status addMongodOptions(moe::OptionSection* options) try {
     uassertStatusOK(addMongodLegacyOptions(options));
     uassertStatusOK(addKeyfileServerOption(options));
     uassertStatusOK(addClusterAuthModeServerOption(options));
+    uassertStatusOK(addServerUpgradeDowngradeOptions(options));
 
     return Status::OK();
 } catch (const AssertionException& ex) {
@@ -137,6 +167,13 @@ StatusWith<repl::ReplSettings> populateReplSettings(const moe::Environment& para
         // "replSetName" is previously removed if "replSet" and "replSetName" are both found to be
         // set by the user. Therefore, we only need to check for it if "replSet" in not found.
         replSettings.setReplSetString(params["replication.replSetName"].as<std::string>().c_str());
+    } else if (gFeatureFlagAllMongodsAreSharded.isEnabledUseLatestFCVWhenUninitialized(
+                   serverGlobalParams.featureCompatibility.acquireFCVSnapshot()) &&
+               serverGlobalParams.maintenanceMode != ServerGlobalParams::StandaloneMode) {
+        replSettings.setShouldAutoInitiate();
+        // Empty `replSet` in replSettings means that the replica set name will be auto-generated
+        // in `processReplSetInitiate` after auto-initiation occurs or loaded from the
+        // local replica set configuration document if already part of a replica set.
     }
 
     if (params.count("replication.oplogSizeMB")) {
@@ -212,6 +249,30 @@ Status validateMongodOptions(const moe::Environment& params) {
     }
 #endif
 
+    bool setConfigRole = params.count("configsvr");
+    bool setShardRole = params.count("shardsvr");
+    if (params.count("sharding.clusterRole")) {
+        auto clusterRole = params["sharding.clusterRole"].as<std::string>();
+        setConfigRole = setConfigRole || clusterRole == "configsvr";
+        setShardRole = setShardRole || clusterRole == "shardsvr";
+    }
+
+    bool setRouterPort = params.count("routerPort") || params.count("net.routerPort");
+
+    if (setRouterPort && !setConfigRole && !setShardRole) {
+        return Status(ErrorCodes::BadValue,
+                      "The embedded router requires the node to act as a shard or config server");
+    }
+
+    if (params.count("maintenanceMode")) {
+        auto maintenanceMode = params["maintenanceMode"].as<std::string>();
+        if (maintenanceMode == "standalone" &&
+            (params.count("replSet") || params.count("replication.replSetName"))) {
+            return Status(ErrorCodes::BadValue,
+                          "Cannot specify both standalone maintenance mode and replica set name");
+        }
+    }
+
     if (params.count("storage.queryableBackupMode")) {
         // Command line options that are disallowed when --queryableBackupMode is specified.
         for (const auto& disallowedOption :
@@ -223,13 +284,7 @@ Status validateMongodOptions(const moe::Environment& params) {
             }
         }
 
-        bool isClusterRoleShard = params.count("shardsvr");
-        if (params.count("sharding.clusterRole")) {
-            auto clusterRole = params["sharding.clusterRole"].as<std::string>();
-            isClusterRoleShard = isClusterRoleShard || (clusterRole == "shardsvr");
-        }
-
-        if (isClusterRoleShard && !params.count("sharding._overrideShardIdentity")) {
+        if (setShardRole && !params.count("sharding._overrideShardIdentity")) {
             return Status(
                 ErrorCodes::BadValue,
                 "shardsvr cluster role with queryableBackupMode requires _overrideShardIdentity");
@@ -240,7 +295,6 @@ Status validateMongodOptions(const moe::Environment& params) {
 }
 
 Status canonicalizeMongodOptions(moe::Environment* params) {
-
     Status ret = canonicalizeServerOptions(params);
     if (!ret.isOK()) {
         return ret;
@@ -262,31 +316,6 @@ Status canonicalizeMongodOptions(moe::Environment* params) {
         }
     }
 
-    // "sharding.archiveMovedChunks" comes from the config file, so override it if
-    // "noMoveParanoia" or "moveParanoia" are set since those come from the command line.
-    if (params->count("noMoveParanoia")) {
-        Status ret = params->set("sharding.archiveMovedChunks",
-                                 moe::Value(!(*params)["noMoveParanoia"].as<bool>()));
-        if (!ret.isOK()) {
-            return ret;
-        }
-        ret = params->remove("noMoveParanoia");
-        if (!ret.isOK()) {
-            return ret;
-        }
-    }
-    if (params->count("moveParanoia")) {
-        Status ret = params->set("sharding.archiveMovedChunks",
-                                 moe::Value((*params)["moveParanoia"].as<bool>()));
-        if (!ret.isOK()) {
-            return ret;
-        }
-        ret = params->remove("moveParanoia");
-        if (!ret.isOK()) {
-            return ret;
-        }
-    }
-
     // "sharding.clusterRole" comes from the config file, so override it if "configsvr" or
     // "shardsvr" are set since those come from the command line.
     if (params->count("configsvr")) {
@@ -322,6 +351,19 @@ Status canonicalizeMongodOptions(moe::Environment* params) {
         }
     }
 
+    // If the "--routerPort" option is passed from the command line, override "net.routerPort"
+    // (config file option) as it will be used later.
+    if (params->count("routerPort")) {
+        Status ret = params->set("net.routerPort", moe::Value((*params)["routerPort"].as<int>()));
+        if (!ret.isOK()) {
+            return ret;
+        }
+        ret = params->remove("routerPort");
+        if (!ret.isOK()) {
+            return ret;
+        }
+    }
+
     if (params->count("profile")) {
         int profilingMode;
         Status ret = params->get("profile", &profilingMode);
@@ -379,12 +421,46 @@ Status canonicalizeMongodOptions(moe::Environment* params) {
     return Status::OK();
 }
 
+namespace {
+constexpr char getPathSeparator() {
+#ifdef _WIN32
+    return '\\';
+#else
+    return '/';
+#endif
+}
+
+void removeTrailingPathSeparator(std::string& path, char separator) {
+    while (path.size() > 1 && path.ends_with(separator)) {
+        // size() check is for the unlikely possibility of --dbpath "/"
+        path.pop_back();
+    }
+}
+}  // namespace
+
 Status storeMongodOptions(const moe::Environment& params) {
     Status ret = storeServerOptions(params);
     if (!ret.isOK()) {
         return ret;
     }
 
+    boost::optional<std::map<std::string, std::string>> setParameterMap;
+    if (params.count("setParameter")) {
+        setParameterMap.emplace(params["setParameter"].as<std::map<std::string, std::string>>());
+    }
+
+    auto checkConflictWithSetParameter = [&setParameterMap](const std::string& configName,
+                                                            const std::string& parameterName) {
+        if (setParameterMap && setParameterMap->find(parameterName) != setParameterMap->end()) {
+            return Status(ErrorCodes::BadValue,
+                          fmt::format("Conflicting server setting and setParameter, only one of "
+                                      "the two should be used: config={}, setParameter={}",
+                                      configName,
+                                      parameterName));
+        }
+        return Status::OK();
+    };
+
     // TODO: Integrate these options with their setParameter counterparts
     if (params.count("security.authSchemaVersion")) {
         return Status(ErrorCodes::BadValue,
@@ -412,14 +488,9 @@ Status storeMongodOptions(const moe::Environment& params) {
             storageGlobalParams.dbpath = serverGlobalParams.cwd + "/" + storageGlobalParams.dbpath;
         }
     }
-#ifdef _WIN32
-    if (storageGlobalParams.dbpath.size() > 1 &&
-        storageGlobalParams.dbpath[storageGlobalParams.dbpath.size() - 1] == '/') {
-        // size() check is for the unlikely possibility of --dbpath "/"
-        storageGlobalParams.dbpath =
-            storageGlobalParams.dbpath.erase(storageGlobalParams.dbpath.size() - 1);
-    }
+    removeTrailingPathSeparator(storageGlobalParams.dbpath, getPathSeparator());
 
+#ifdef _WIN32
     StringData dbpath(storageGlobalParams.dbpath);
     if (dbpath.size() >= 2 && dbpath.startsWith("\\\\")) {
         // Check if the dbpath is on a Windows network share (eg. \\myserver\myshare)
@@ -447,6 +518,11 @@ Status storeMongodOptions(const moe::Environment& params) {
 
     if (params.count("storage.syncPeriodSecs")) {
         storageGlobalParams.syncdelay = params["storage.syncPeriodSecs"].as<double>();
+        Status conflictStatus =
+            checkConflictWithSetParameter("storage.syncPeriodSecs", "syncdelay");
+        if (!conflictStatus.isOK()) {
+            return conflictStatus;
+        }
     }
 
     if (params.count("storage.directoryPerDB")) {
@@ -465,12 +541,10 @@ Status storeMongodOptions(const moe::Environment& params) {
     if (params.count("storage.journal.commitIntervalMs")) {
         auto journalCommitIntervalMs = params["storage.journal.commitIntervalMs"].as<int>();
         storageGlobalParams.journalCommitIntervalMs.store(journalCommitIntervalMs);
-        if (journalCommitIntervalMs < 1 ||
-            journalCommitIntervalMs > StorageGlobalParams::kMaxJournalCommitIntervalMs) {
-            return Status(ErrorCodes::BadValue,
-                          str::stream()
-                              << "--journalCommitInterval out of allowed range (1-"
-                              << StorageGlobalParams::kMaxJournalCommitIntervalMs << "ms)");
+        Status conflictStatus = checkConflictWithSetParameter("storage.journal.commitIntervalMs",
+                                                              "journalCommitInterval");
+        if (!conflictStatus.isOK()) {
+            return conflictStatus;
         }
     }
 
@@ -509,8 +583,17 @@ Status storeMongodOptions(const moe::Environment& params) {
                           str::stream() << "Cannot specify both --repair and --restore");
         }
     }
-    if (params.count("magicRestore") && params["magicRestore"].as<bool>() == true) {
-        storageGlobalParams.magicRestore = 1;
+
+    if (params.count("maintenanceMode") &&
+        gFeatureFlagAllMongodsAreSharded.isEnabledUseLatestFCVWhenUninitialized(
+            serverGlobalParams.featureCompatibility.acquireFCVSnapshot())) {
+        // Setting maintenanceMode will disable sharding by setting 'clusterRole' to
+        // 'ClusterRole::None'. If maintenanceMode is set to 'standalone', replication will be
+        // disabled as well.
+        std::string value = params["maintenanceMode"].as<std::string>();
+        serverGlobalParams.maintenanceMode = (value == "replicaSet")
+            ? ServerGlobalParams::ReplicaSetMode
+            : ServerGlobalParams::StandaloneMode;
     }
 
     const auto replSettingsWithStatus = populateReplSettings(params);
@@ -518,7 +601,7 @@ Status storeMongodOptions(const moe::Environment& params) {
         return replSettingsWithStatus.getStatus();
     const repl::ReplSettings& replSettings(replSettingsWithStatus.getValue());
 
-    if (replSettings.usingReplSets()) {
+    if (replSettings.isReplSet()) {
         if ((params.count("security.authorization") &&
              params["security.authorization"].as<std::string>() == "enabled") &&
             !serverGlobalParams.startupClusterAuthMode.x509Only() &&
@@ -544,6 +627,27 @@ Status storeMongodOptions(const moe::Environment& params) {
         storageGlobalParams.allowOplogTruncation = false;
     }
 
+    if (params.count("magicRestore") && params["magicRestore"].as<bool>() == true) {
+        if (!replSettings.isReplSet()) {
+            return Status(ErrorCodes::BadValue,
+                          str::stream() << "Cannot start magic restore without --replSet.");
+        }
+
+        if (params.count("sharding.clusterRole")) {
+            return Status(ErrorCodes::BadValue,
+                          str::stream()
+                              << "Cannot start magic restore with --shardsvr or --configsvr. Magic "
+                                 "restore performs the restore procedure as a replica set node.");
+        }
+
+        storageGlobalParams.magicRestore = 1;
+
+        // Use an ephemeral port so that users don't connect to a node that is being restored.
+        if (!params.count("net.port")) {
+            serverGlobalParams.port = ServerGlobalParams::DefaultMagicRestorePort;
+        }
+    }
+
     serverGlobalParams.enableMajorityReadConcern = true;
 
     if (storageGlobalParams.engineSetByUser && (storageGlobalParams.engine == "devnull")) {
@@ -581,34 +685,20 @@ Status storeMongodOptions(const moe::Environment& params) {
         }
         return Status(ErrorCodes::BadValue, "--cacheSize option not currently supported");
     }
-    if (!params.count("net.port")) {
-        if (params.count("sharding.clusterRole")) {
-            std::string clusterRole = params["sharding.clusterRole"].as<std::string>();
-            if (clusterRole == "configsvr") {
-                serverGlobalParams.port = ServerGlobalParams::ConfigServerPort;
-            } else if (clusterRole == "shardsvr") {
-                serverGlobalParams.port = ServerGlobalParams::ShardServerPort;
-            } else {
-                StringBuilder sb;
-                sb << "Bad value for sharding.clusterRole: " << clusterRole
-                   << ".  Supported modes are: (configsvr|shardsvr)";
-                return Status(ErrorCodes::BadValue, sb.str());
-            }
-        }
-    }
+    const auto fcvSnapshot = serverGlobalParams.featureCompatibility.acquireFCVSnapshot();
     if (params.count("sharding.clusterRole")) {
         auto clusterRoleParam = params["sharding.clusterRole"].as<std::string>();
         // Force to set up the node as a replica set, unless we're a shard and we're using queryable
         // backup mode.
         if ((clusterRoleParam == "configsvr" || !params.count("storage.queryableBackupMode")) &&
-            !replSettings.usingReplSets()) {
+            !replSettings.isReplSet()) {
             return Status(ErrorCodes::BadValue,
                           str::stream() << "Cannot start a " << clusterRoleParam
                                         << " as a standalone server. Please use the option "
                                            "--replSet to start the node as a replica set.");
         }
         if (clusterRoleParam == "configsvr") {
-            serverGlobalParams.clusterRole = ClusterRole::ConfigServer;
+            serverGlobalParams.clusterRole = {ClusterRole::ShardServer, ClusterRole::ConfigServer};
             // Config server requires majority read concern.
             uassert(5324702,
                     str::stream() << "Cannot initialize config server with "
@@ -620,15 +710,39 @@ Status storeMongodOptions(const moe::Environment& params) {
             }
         } else if (clusterRoleParam == "shardsvr") {
             serverGlobalParams.clusterRole = ClusterRole::ShardServer;
+        } else {
+            return Status(ErrorCodes::BadValue,
+                          fmt::format("Bad value for sharding.clusterRole: {}. Supported modes "
+                                      "are: (configsvr|shardsvr)",
+                                      clusterRoleParam));
         }
+
+        // Every node in a sharded cluster will have by default the RouterServer role. As a
+        // consequence, the only possible combinations are:
+        // - { ShardServer, RouterServer }
+        // - { ShardServer, ConfigServer, RouterServer }
+        // - { RouterServer }
+        serverGlobalParams.clusterRole += ClusterRole::RouterServer;
+
+        if (params.count("net.routerPort")) {
+            if (feature_flags::gRouterPort.isEnabledUseLatestFCVWhenUninitialized(fcvSnapshot)) {
+                serverGlobalParams.routerPort = params["net.routerPort"].as<int>();
+            }
+        }
+    } else if (gFeatureFlagAllMongodsAreSharded.isEnabledUseLatestFCVWhenUninitialized(
+                   fcvSnapshot) &&
+               serverGlobalParams.maintenanceMode == ServerGlobalParams::MaintenanceMode::None) {
+        serverGlobalParams.doAutoBootstrapSharding = true;
+        serverGlobalParams.clusterRole = {
+            ClusterRole::ShardServer, ClusterRole::ConfigServer, ClusterRole::RouterServer};
     }
 
-    if (params.count("net.proxyPort")) {
-        serverGlobalParams.proxyPort = params["net.proxyPort"].as<int>();
-    }
-
-    if (params.count("sharding.archiveMovedChunks")) {
-        serverGlobalParams.moveParanoia = params["sharding.archiveMovedChunks"].as<bool>();
+    if (!params.count("net.port")) {
+        if (serverGlobalParams.clusterRole.has(ClusterRole::ConfigServer)) {
+            serverGlobalParams.port = ServerGlobalParams::ConfigServerPort;
+        } else if (serverGlobalParams.clusterRole.has(ClusterRole::ShardServer)) {
+            serverGlobalParams.port = ServerGlobalParams::ShardServerPort;
+        }
     }
 
     if (params.count("sharding._overrideShardIdentity")) {
@@ -696,19 +810,10 @@ Status storeMongodOptions(const moe::Environment& params) {
         }
     }
 
+    serverGlobalParams.upgradeBackCompat = params.count("upgradeBackCompat");
+    serverGlobalParams.downgradeBackCompat = params.count("downgradeBackCompat");
+
     setGlobalReplSettings(replSettings);
     return Status::OK();
 }
-
-namespace {
-std::function<ExitCode(ServiceContext* svcCtx)> _magicRestoreMainFn = nullptr;
-}
-
-void setMagicRestoreMain(std::function<ExitCode(ServiceContext* svcCtx)> magicRestoreMainFn) {
-    _magicRestoreMainFn = magicRestoreMainFn;
-}
-std::function<ExitCode(ServiceContext* svcCtx)> getMagicRestoreMain() {
-    return _magicRestoreMainFn;
-}
-
 }  // namespace mongo
