@@ -27,24 +27,25 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
 #include "mongo/db/namespace_string.h"
 
-#include <ostream>
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
 
 #include "mongo/base/parse_number.h"
 #include "mongo/base/status.h"
-#include "mongo/db/multitenancy_gen.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/bson/util/builder.h"
+#include "mongo/bson/util/builder_fwd.h"
 #include "mongo/db/server_options.h"
-#include "mongo/util/str.h"
+// IWYU pragma: no_include "mongo/db/namespace_string_reserved.def.h"
+#include "mongo/util/duration.h"
 
 namespace mongo {
 namespace {
 
 constexpr auto listCollectionsCursorCol = "$cmd.listCollections"_sd;
 constexpr auto bulkWriteCursorCol = "$cmd.bulkWrite"_sd;
-constexpr auto collectionlessAggregateCursorCol = "$cmd.aggregate"_sd;
 constexpr auto dropPendingNSPrefix = "system.drop."_sd;
 
 constexpr auto fle2Prefix = "enxcol_."_sd;
@@ -70,38 +71,12 @@ static const absl::flat_hash_set<NamespaceString> globallyUniqueConfigDbCollecti
 
 }  // namespace
 
-
-NamespaceString NamespaceString::parseFromStringExpectTenantIdInMultitenancyMode(StringData ns) {
-    if (!gMultitenancySupport) {
-        return NamespaceString(boost::none, ns);
-    }
-
-    auto tenantDelim = ns.find('_');
-    auto collDelim = ns.find('.');
-    // If the first '_' is after the '.' that separates the db and coll names, the '_' is part
-    // of the coll name and is not a db prefix.
-    if (tenantDelim == std::string::npos || collDelim < tenantDelim) {
-        return NamespaceString(boost::none, ns);
-    }
-
-    auto swOID = OID::parse(ns.substr(0, tenantDelim));
-    if (swOID.getStatus() == ErrorCodes::BadValue) {
-        // If we fail to parse an OID, either the size of the substring is incorrect, or there is an
-        // invalid character. This indicates that the db has the "_" character, but it does not act
-        // as a delimeter for a tenantId prefix.
-        return NamespaceString(boost::none, ns);
-    }
-
-    const TenantId tenantId(swOID.getValue());
-    return NamespaceString(tenantId, ns.substr(tenantDelim + 1, ns.size() - 1 - tenantDelim));
-}
-
 bool NamespaceString::isListCollectionsCursorNS() const {
     return coll() == listCollectionsCursorCol;
 }
 
 bool NamespaceString::isCollectionlessAggregateNS() const {
-    return coll() == collectionlessAggregateCursorCol;
+    return coll() == kCollectionlessAggregateCollection;
 }
 
 bool NamespaceString::isLegalClientSystemNS() const {
@@ -192,14 +167,15 @@ bool NamespaceString::mustBeAppliedInOwnOplogBatch() const {
         ns == kConfigsvrShardsNamespace.ns();
 }
 
-NamespaceString NamespaceString::makeBulkWriteNSS() {
-    return NamespaceString(DatabaseName::kAdmin, bulkWriteCursorCol);
+NamespaceString NamespaceString::makeBulkWriteNSS(const boost::optional<TenantId>& tenantId) {
+    return NamespaceString(tenantId, DatabaseName::kAdmin.db(omitTenant), bulkWriteCursorCol);
 }
 
 NamespaceString NamespaceString::makeClusterParametersNSS(
     const boost::optional<TenantId>& tenantId) {
-    return tenantId ? NamespaceString(tenantId, DatabaseName::kConfig.db(), "clusterParameters")
-                    : kClusterParametersNamespace;
+    return tenantId
+        ? NamespaceString(tenantId, DatabaseName::kConfig.db(omitTenant), "clusterParameters")
+        : kClusterParametersNamespace;
 }
 
 NamespaceString NamespaceString::makeSystemDotViewsNamespace(const DatabaseName& dbName) {
@@ -226,7 +202,7 @@ NamespaceString NamespaceString::makeLocalCollection(StringData collName) {
 }
 
 NamespaceString NamespaceString::makeCollectionlessAggregateNSS(const DatabaseName& dbName) {
-    NamespaceString nss(dbName, collectionlessAggregateCursorCol);
+    NamespaceString nss(dbName, kCollectionlessAggregateCollection);
     dassert(nss.isValid());
     dassert(nss.isCollectionlessAggregateNS());
     return nss;
@@ -234,7 +210,7 @@ NamespaceString NamespaceString::makeCollectionlessAggregateNSS(const DatabaseNa
 
 NamespaceString NamespaceString::makeChangeCollectionNSS(
     const boost::optional<TenantId>& tenantId) {
-    return NamespaceString{tenantId, DatabaseName::kConfig.db(), kChangeCollectionName};
+    return NamespaceString{tenantId, DatabaseName::kConfig.db(omitTenant), kChangeCollectionName};
 }
 
 NamespaceString NamespaceString::makeGlobalIndexNSS(const UUID& id) {
@@ -242,24 +218,10 @@ NamespaceString NamespaceString::makeGlobalIndexNSS(const UUID& id) {
                            NamespaceString::kGlobalIndexCollectionPrefix + id.toString());
 }
 
-NamespaceString NamespaceString::makeMovePrimaryOplogBufferNSS(const UUID& migrationId) {
-    return NamespaceString(DatabaseName::kConfig,
-                           "movePrimaryOplogBuffer." + migrationId.toString());
-}
-
-NamespaceString NamespaceString::makeMovePrimaryCollectionsToCloneNSS(const UUID& migrationId) {
-    return NamespaceString(DatabaseName::kConfig,
-                           "movePrimaryCollectionsToClone." + migrationId.toString());
-}
-
-NamespaceString NamespaceString::makeMovePrimaryTempCollectionsPrefix(const UUID& migrationId) {
-    return NamespaceString(DatabaseName::kConfig,
-                           "movePrimaryRecipient." + migrationId.toString() + ".willBeDeleted.");
-}
-
 NamespaceString NamespaceString::makePreImageCollectionNSS(
     const boost::optional<TenantId>& tenantId) {
-    return NamespaceString{tenantId, DatabaseName::kConfig.db(), kPreImagesCollectionName};
+    return NamespaceString{
+        tenantId, DatabaseName::kConfig.db(omitTenant), kPreImagesCollectionName};
 }
 
 NamespaceString NamespaceString::makeReshardingLocalOplogBufferNSS(
@@ -278,21 +240,27 @@ NamespaceString NamespaceString::makeReshardingLocalConflictStashNSS(
 
 NamespaceString NamespaceString::makeTenantUsersCollection(
     const boost::optional<TenantId>& tenantId) {
-    return NamespaceString(tenantId, DatabaseName::kAdmin.db(), NamespaceString::kSystemUsers);
+    return NamespaceString(
+        tenantId, DatabaseName::kAdmin.db(omitTenant), NamespaceString::kSystemUsers);
 }
 
 NamespaceString NamespaceString::makeTenantRolesCollection(
     const boost::optional<TenantId>& tenantId) {
-    return NamespaceString(tenantId, DatabaseName::kAdmin.db(), NamespaceString::kSystemRoles);
+    return NamespaceString(
+        tenantId, DatabaseName::kAdmin.db(omitTenant), NamespaceString::kSystemRoles);
 }
 
 NamespaceString NamespaceString::makeCommandNamespace(const DatabaseName& dbName) {
     return NamespaceString(dbName, "$cmd");
 }
 
+NamespaceString NamespaceString::makeDummyNamespace(const boost::optional<TenantId>& tenantId) {
+    return NamespaceString(tenantId, DatabaseName::kConfig.db(omitTenant), "dummy.namespace");
+}
+
 std::string NamespaceString::getSisterNS(StringData local) const {
     MONGO_verify(local.size() && local[0] != '.');
-    return db().toString() + "." + local.toString();
+    return db_deprecated().toString() + "." + local.toString();
 }
 
 void NamespaceString::serializeCollectionName(BSONObjBuilder* builder, StringData fieldName) const {
@@ -309,10 +277,10 @@ bool NamespaceString::isDropPendingNamespace() const {
 
 NamespaceString NamespaceString::makeDropPendingNamespace(const repl::OpTime& opTime) const {
     StringBuilder ss;
-    ss << db() << "." << dropPendingNSPrefix;
+    ss << db_deprecated() << "." << dropPendingNSPrefix;
     ss << opTime.getSecs() << "i" << opTime.getTimestamp().getInc() << "t" << opTime.getTerm();
     ss << "." << coll();
-    return NamespaceString(ss.stringData());
+    return NamespaceString(tenantId(), ss.stringData());
 }
 
 StatusWith<repl::OpTime> NamespaceString::getDropPendingNamespaceOpTime() const {
@@ -364,13 +332,13 @@ StatusWith<repl::OpTime> NamespaceString::getDropPendingNamespaceOpTime() const
     return repl::OpTime(Timestamp(Seconds(seconds), increment), term);
 }
 
-bool NamespaceString::isNamespaceAlwaysUnsharded() const {
+bool NamespaceString::isNamespaceAlwaysUntracked() const {
     // Local and admin never have sharded collections
-    if (db() == DatabaseName::kLocal.db() || db() == DatabaseName::kAdmin.db())
+    if (isLocalDB() || isAdminDB())
         return true;
 
     // Config can only have the system.sessions as sharded
-    if (db() == DatabaseName::kConfig.db())
+    if (isConfigDB())
         return *this != NamespaceString::kLogicalSessionsNamespace;
 
     if (isSystem()) {
@@ -403,19 +371,20 @@ bool NamespaceString::isShardLocalNamespace() const {
 }
 
 bool NamespaceString::isConfigDotCacheDotChunks() const {
-    return db() == "config" && coll().startsWith("cache.chunks.");
+    return db_deprecated() == "config" && coll().startsWith("cache.chunks.");
 }
 
 bool NamespaceString::isReshardingLocalOplogBufferCollection() const {
-    return db() == "config" && coll().startsWith(kReshardingLocalOplogBufferPrefix);
+    return db_deprecated() == "config" && coll().startsWith(kReshardingLocalOplogBufferPrefix);
 }
 
 bool NamespaceString::isReshardingConflictStashCollection() const {
-    return db() == "config" && coll().startsWith(kReshardingConflictStashPrefix);
+    return db_deprecated() == "config" && coll().startsWith(kReshardingConflictStashPrefix);
 }
 
 bool NamespaceString::isTemporaryReshardingCollection() const {
-    return coll().startsWith(kTemporaryReshardingCollectionPrefix);
+    return coll().startsWith(kTemporaryTimeseriesReshardingCollectionPrefix) ||
+        coll().startsWith(kTemporaryReshardingCollectionPrefix);
 }
 
 bool NamespaceString::isTimeseriesBucketsCollection() const {
@@ -444,6 +413,12 @@ bool NamespaceString::isFLE2StateCollection() const {
          coll().endsWith(fle2EcocSuffix) || coll().endsWith(fle2EcocCompactSuffix));
 }
 
+bool NamespaceString::isFLE2StateCollection(StringData coll) {
+    return coll.startsWith(fle2Prefix) &&
+        (coll.endsWith(fle2EscSuffix) || coll.endsWith(fle2EccSuffix) ||
+         coll.endsWith(fle2EcocSuffix));
+}
+
 bool NamespaceString::isOplogOrChangeCollection() const {
     return isOplog() || isChangeCollection();
 }
@@ -467,7 +442,7 @@ NamespaceString NamespaceString::getTimeseriesViewNamespace() const {
 }
 
 bool NamespaceString::isImplicitlyReplicated() const {
-    if (db() == DatabaseName::kConfig.db()) {
+    if (db_deprecated() == DatabaseName::kConfig.db(omitTenant)) {
         if (isChangeStreamPreImagesCollection() || isConfigImagesCollection() ||
             isChangeCollection()) {
             // Implicitly replicated namespaces are replicated, although they only replicate a
@@ -481,7 +456,7 @@ bool NamespaceString::isImplicitlyReplicated() const {
 }
 
 bool NamespaceString::isReplicated() const {
-    if (isLocal()) {
+    if (isLocalDB()) {
         return false;
     }
 
@@ -499,14 +474,22 @@ bool NamespaceString::isReplicated() const {
     return true;
 }
 
-std::string NamespaceStringOrUUID::toString() const {
+std::string NamespaceStringOrUUID::toStringForErrorMsg() const {
     if (isNamespaceString()) {
-        return nss().toString();
+        return nss().toStringForErrorMsg();
     }
 
     return uuid().toString();
 }
 
+std::string toStringForLogging(const NamespaceStringOrUUID& nssOrUUID) {
+    if (nssOrUUID.isNamespaceString()) {
+        return toStringForLogging(nssOrUUID.nss());
+    }
+
+    return nssOrUUID.uuid().toString();
+}
+
 void NamespaceStringOrUUID::serialize(BSONObjBuilder* builder, StringData fieldName) const {
     if (const NamespaceString* nss = get_if<NamespaceString>(&_nssOrUUID)) {
         builder->append(fieldName, nss->coll());
