@@ -30,13 +30,26 @@
 #pragma once
 
 #include <algorithm>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
 #include <boost/optional.hpp>
+#include <boost/optional/optional.hpp>
+#include <cstdint>
+#include <cstring>
+#include <fmt/format.h>
 #include <iosfwd>
 #include <mutex>
 #include <string>
+#include <tuple>
+#include <utility>
+#include <variant>
 
+#include "mongo/base/data_view.h"
+#include "mongo/base/error_codes.h"
 #include "mongo/base/status_with.h"
 #include "mongo/base/string_data.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/oid.h"
 #include "mongo/bson/util/builder.h"
 #include "mongo/db/database_name.h"
 #include "mongo/db/repl/optime.h"
@@ -44,96 +57,22 @@
 #include "mongo/db/tenant_id.h"
 #include "mongo/logv2/log_attr.h"
 #include "mongo/util/assert_util.h"
+#include "mongo/util/str.h"
 #include "mongo/util/uuid.h"
 
 namespace mongo {
-class NamespaceStringUtil;
-class IDLParserContext;
 
-class NamespaceString {
+class NamespaceString : private DatabaseName {
 public:
-    /**
-     * The NamespaceString reserved constants are actually this `ConstantProxy`
-     * type, which can be `constexpr` and can be used directly in place of
-     * `NamespaceString`, except in very rare cases. To work around those, use a
-     * `static_cast<const NamespaceString&>`. The first time it's used, a
-     * `ConstantProxy` produces a memoized `const NamespaceString*` and retains
-     * it for future uses.
-     */
-    class ConstantProxy {
-    public:
-        /**
-         * `ConstantProxy` objects can be copied, so that they behave more like
-         * `NamespaceString`. All copies will point to the same `SharedState`.
-         * The `SharedState` is meant to be defined constexpr, but has mutable
-         * data members to implement the on-demand memoization of the
-         * `NamespaceString`.
-         */
-        class SharedState {
-        public:
-            constexpr SharedState(DatabaseName::ConstantProxy dbName, StringData coll)
-                : _db{dbName}, _coll{coll} {}
-
-            const NamespaceString& get() const {
-                std::call_once(_once, [this] { _nss = new NamespaceString{_db, _coll}; });
-                return *_nss;
-            }
-
-        private:
-            DatabaseName::ConstantProxy _db;
-            StringData _coll;
-            mutable std::once_flag _once;
-            mutable const NamespaceString* _nss = nullptr;
-        };
-
-        constexpr explicit ConstantProxy(const SharedState* sharedState)
-            : _sharedState{sharedState} {}
-
-        operator const NamespaceString&() const {
-            return _get();
-        }
-
-        decltype(auto) ns() const {
-            return _get().ns();
-        }
-        decltype(auto) db() const {
-            return _get().db();
-        }
-        decltype(auto) coll() const {
-            return _get().coll();
-        }
-        decltype(auto) tenantId() const {
-            return _get().tenantId();
-        }
-        decltype(auto) dbName() const {
-            return _get().dbName();
-        }
-        decltype(auto) toString() const {
-            return _get().toString();
-        }
-
-        friend std::ostream& operator<<(std::ostream& stream, const ConstantProxy& nss) {
-            return stream << nss.toString();
-        }
-        friend StringBuilder& operator<<(StringBuilder& builder, const ConstantProxy& nss) {
-            return builder << nss.toString();
-        }
-
-    private:
-        const NamespaceString& _get() const {
-            return _sharedState->get();
-        }
-
-        const SharedState* _sharedState;
-    };
-
     constexpr static size_t MaxNSCollectionLenFCV42 = 120U;
-    constexpr static size_t MaxNsCollectionLen = 255;
+    constexpr static size_t MaxUserNsCollectionLen = 255;
+    constexpr static size_t MaxInternalNsCollectionLen = 511;
 
     // The maximum namespace length of sharded collections is less than that of unsharded ones since
     // the namespace of the cached chunks metadata, local to each shard, is composed by the
     // namespace of the related sharded collection (i.e., config.cache.chunks.<ns>).
-    constexpr static size_t MaxNsShardedCollectionLen = 235;  // 255 - len(ChunkType::ShardNSPrefix)
+    constexpr static size_t MaxUserNsShardedCollectionLen =
+        235;  // 255 - len(ChunkType::ShardNSPrefix)
 
     // Reserved system namespaces
 
@@ -161,7 +100,6 @@ public:
 
     // Prefix for orphan collections
     static constexpr StringData kOrphanCollectionPrefix = "orphan."_sd;
-    static constexpr StringData kOrphanCollectionDb = "local"_sd;
 
     // Prefix for collections that store the local resharding oplog buffer.
     static constexpr StringData kReshardingLocalOplogBufferPrefix =
@@ -173,6 +111,10 @@ public:
     // Prefix for temporary resharding collection.
     static constexpr StringData kTemporaryReshardingCollectionPrefix = "system.resharding."_sd;
 
+    // Prefix for temporary timeseries resharding collection.
+    static constexpr StringData kTemporaryTimeseriesReshardingCollectionPrefix =
+        "system.buckets.resharding."_sd;
+
     // Prefix for time-series buckets collection.
     static constexpr StringData kTimeseriesBucketsCollectionPrefix = "system.buckets."_sd;
 
@@ -180,6 +122,8 @@ public:
     // database.
     static constexpr StringData kGlobalIndexCollectionPrefix = "globalIndex."_sd;
 
+    // Collection name that is used for { aggregate: 1, ... } style aggregations.
+    static constexpr auto kCollectionlessAggregateCollection = "$cmd.aggregate"_sd;
 
     // Prefix for the temporary collection used by the $out stage.
     static constexpr StringData kOutTmpCollectionPrefix = "tmp.agg_out."_sd;
@@ -193,8 +137,8 @@ public:
     // definitions below. Because the `NamespaceString` class enclosing their
     // type is incomplete, they can't be _declared_ fully constexpr (a constexpr
     // limitation).
-#define NSS_CONSTANT(id, db, coll) static const ConstantProxy id;
-#include "namespace_string_reserved.def.h"
+#define NSS_CONSTANT(id, db, coll) static const NamespaceString id;
+#include "namespace_string_reserved.def.h"  // IWYU pragma: keep
 #undef NSS_CONSTANT
 
     /**
@@ -205,27 +149,29 @@ public:
     /**
      * Constructs a NamespaceString for the given database.
      */
-    explicit NamespaceString(DatabaseName dbName) : _data(std::move(dbName._data)) {}
+    explicit NamespaceString(DatabaseName dbName) : DatabaseName(std::move(dbName)) {}
 
-    // TODO SERVER-65920 Remove this constructor once all constructor call sites have been updated
-    // to pass tenantId explicitly
-    explicit NamespaceString(StringData ns, boost::optional<TenantId> tenantId = boost::none)
-        : NamespaceString(std::move(tenantId), ns) {}
-
-    // TODO SERVER-65920 Remove this constructor once all constructor call sites have been updated
-    // to pass tenantId explicitly
-    NamespaceString(StringData db,
-                    StringData collectionName,
-                    boost::optional<TenantId> tenantId = boost::none)
-        : NamespaceString(std::move(tenantId), db, collectionName) {}
+    constexpr NamespaceString(const char* data, size_t length) : DatabaseName(data, length) {}
 
     /**
-     * Constructs a NamespaceString from the string 'ns'. Should only be used when reading a
-     * namespace from disk. 'ns' is expected to contain a tenantId when running in Serverless mode.
+     * Construct a NamespaceString from a const reference. This constructor is required to avoid
+     * invoking DatabaseName(const DatabaseName&..) which would discard the collection from the
+     * underlying data.
      */
-    // TODO SERVER-70013 Move this function into NamespaceStringUtil, and delegate overlapping
-    // functionality to DatabaseNameUtil::parseDbNameFromStringExpectTenantIdInMultitenancyMode.
-    static NamespaceString parseFromStringExpectTenantIdInMultitenancyMode(StringData ns);
+    NamespaceString(NamespaceString&& other) = default;
+
+    NamespaceString(const NamespaceString& other) noexcept
+        : DatabaseName(other._data, other.sizeWithTenant() + kDataOffset, TrustedInitTag{}) {}
+    NamespaceString& operator=(NamespaceString&& other) = default;
+
+    /**
+     * Copy assignment operator. This cannot be defaulted as we must avoid calling DatabaseName copy
+     * assignment operator which would discard the collection from _data.
+     */
+    NamespaceString& operator=(const NamespaceString& other) noexcept {
+        _data = other._data;
+        return *this;
+    }
 
     /**
      * Constructs a NamespaceString in the global config db, "config.<collName>".
@@ -270,24 +216,6 @@ public:
         return NamespaceString(tenantId, db, coll);
     }
 
-    /**
-     * These functions construct a NamespaceString without checking for presence of TenantId. These
-     * must only be used by auth systems which are not yet tenant aware.
-     *
-     * TODO SERVER-74896 Remove this function. Any remaining call sites must be changed to use a
-     * function on NamespaceStringUtil.
-     */
-    static NamespaceString createNamespaceStringForAuth(const boost::optional<TenantId>& tenantId,
-                                                        StringData db,
-                                                        StringData coll) {
-        return NamespaceString(tenantId, db, coll);
-    }
-
-    static NamespaceString createNamespaceStringForAuth(const boost::optional<TenantId>& tenantId,
-                                                        StringData ns) {
-        return NamespaceString(tenantId, ns);
-    }
-
     /**
      * Constructs the namespace '<dbName>.$cmd.aggregate', which we use as the namespace for
      * aggregation commands with the format {aggregate: 1}.
@@ -336,7 +264,7 @@ public:
      * Constructs a NamespaceString representing a BulkWrite namespace. The format for this
      * namespace is admin.$cmd.bulkWrite".
      */
-    static NamespaceString makeBulkWriteNSS();
+    static NamespaceString makeBulkWriteNSS(const boost::optional<TenantId>& tenantId);
 
     /**
      * Constructs the oplog buffer NamespaceString for the given migration id for movePrimary op.
@@ -383,39 +311,43 @@ public:
     static NamespaceString makeCommandNamespace(const DatabaseName& dbName);
 
     /**
-     * NOTE: DollarInDbNameBehavior::allow is deprecated.
-     *
-     * Please use DollarInDbNameBehavior::disallow and check explicitly for any DB names that must
-     * contain a $.
+     * Constructs a dummy NamespaceString, "<tenantId>.config.dummy.namespace", to be used where a
+     * placeholder NamespaceString is needed. It must be acceptable for tenantId to be empty, so we
+     * use "config" as the db.
      */
-    enum class DollarInDbNameBehavior {
-        Disallow,
-        Allow,  // Deprecated
-    };
+    static NamespaceString makeDummyNamespace(const boost::optional<TenantId>& tenantId);
+
 
     boost::optional<TenantId> tenantId() const {
-        if (!_hasTenantId()) {
+        if (!hasTenantId()) {
             return boost::none;
         }
 
-        return TenantId{OID::from(&_data[kDataOffset])};
+        return TenantId{OID::from(_data.data() + kDataOffset)};
     }
 
-    StringData db() const {
-        // TODO SERVER-65456 Remove this function.
-        auto offset = _hasTenantId() ? kDataOffset + OID::kOIDSize : kDataOffset;
-        return StringData{_data.data() + offset, _dbNameOffsetEnd()};
+    using DatabaseName::db;
+
+    /**
+     * This function must only be used in sharding code (src/mongo/s and src/mongo/db/s).
+     */
+    StringData db_forSharding() const {
+        return db_deprecated();
     }
 
-    DatabaseName dbName() const {
-        auto offset = _hasTenantId() ? kDataOffset + OID::kOIDSize : kDataOffset;
-        return DatabaseName{_data.substr(0, offset + _dbNameOffsetEnd()),
-                            DatabaseName::TrustedInitTag{}};
+    /**
+     * This function must only be used in unit tests.
+     */
+    StringData db_forTest() const {
+        return db_deprecated();
+    }
+
+    const DatabaseName& dbName() const {
+        return *this;
     }
 
     StringData coll() const {
-        const auto offset =
-            kDataOffset + _dbNameOffsetEnd() + 1 + (_hasTenantId() ? OID::kOIDSize : 0);
+        const auto offset = kDataOffset + dbSize() + 1 + tenantIdSize();
         if (offset > _data.size()) {
             return {};
         }
@@ -423,21 +355,51 @@ public:
         return StringData{_data.data() + offset, _data.size() - offset};
     }
 
-    StringData ns() const {
-        auto offset = _hasTenantId() ? kDataOffset + OID::kOIDSize : kDataOffset;
-        return StringData{_data.data() + offset, _data.size() - offset};
+    ConstDataRange asDataRange() const {
+        auto nss = ns();
+        return ConstDataRange(nss.data(), nss.size());
     }
 
-    std::string toString() const {
-        return ns().toString();
+    StringData ns_forTest() const {
+        return ns();
     }
 
-    std::string toStringWithTenantId() const {
-        if (_hasTenantId()) {
-            return str::stream() << TenantId{OID::from(&_data[kDataOffset])} << "_" << ns();
-        }
+    /**
+     * Gets a namespace string without tenant id.
+     *
+     * MUST only be used for tests.
+     */
+    std::string toString_forTest() const {
+        return toString();
+    }
 
-        return ns().toString();
+    /**
+     * Returns a namespace string without tenant id.
+     * Please use the NamespaceStringUtil::serialize class instead to apply the proper serialization
+     * behavior.
+     * Only to be used when a tenant id cannot be tolerated in the serialized output, and should
+     * otherwise be avoided whenever possible.
+     *
+     * MUST only be used for very specific cases.
+     */
+    std::string serializeWithoutTenantPrefix_UNSAFE() const {
+        return toString();
+    }
+
+    /**
+     * Gets a namespace string with tenant id.
+     *
+     * MUST only be used for tests.
+     */
+    std::string toStringWithTenantId_forTest() const {
+        return toStringWithTenantId();
+    }
+
+    /**
+     * This function should only be used when creating a resouce id for nss.
+     */
+    std::string toStringForResourceId() const {
+        return toStringWithTenantId();
     }
 
     /**
@@ -455,13 +417,27 @@ public:
         return nss.toStringWithTenantId();
     }
 
+    /**
+     * Returns the size of the database and collection (including the 'dot').
+     */
     size_t size() const {
-        auto offset = _hasTenantId() ? kDataOffset + OID::kOIDSize : kDataOffset;
+        auto offset = kDataOffset + tenantIdSize();
         return _data.size() - offset;
     }
 
+    /**
+     * Returns the size of the tenant id, database and collection (including the 'dot').
+     */
+    size_t sizeWithTenant() const {
+        return _data.size() - kDataOffset;
+    }
+
+    size_t dbSize() const {
+        return DatabaseName::size();
+    }
+
     bool isEmpty() const {
-        return _data.size() == kDataOffset;
+        return size() == 0;
     }
 
     //
@@ -469,25 +445,22 @@ public:
     //
 
     bool isHealthlog() const {
-        return isLocal() && coll() == "system.healthlog";
+        return isLocalDB() && coll() == "system.healthlog";
     }
     bool isSystem() const {
         return coll().startsWith("system.");
     }
     bool isNormalCollection() const {
-        return !isSystem() && !(isLocal() && coll().startsWith("replset."));
+        return !isSystem() && !(isLocalDB() && coll().startsWith("replset."));
     }
     bool isGlobalIndex() const {
         return coll().startsWith(kGlobalIndexCollectionPrefix);
     }
     bool isAdminDB() const {
-        return db() == DatabaseName::kAdmin.db();
+        return db_deprecated() == DatabaseName::kAdmin.db(omitTenant);
     }
     bool isLocalDB() const {
-        return db() == DatabaseName::kLocal.db();
-    }
-    bool isLocal() const {
-        return db() == DatabaseName::kLocal.db();
+        return db_deprecated() == DatabaseName::kLocal.db(omitTenant);
     }
     bool isSystemDotProfile() const {
         return coll() == kSystemDotProfileCollectionName;
@@ -506,7 +479,7 @@ public:
         return coll() == kSystemUsers;
     }
     bool isServerConfigurationCollection() const {
-        return (db() == DatabaseName::kAdmin.db()) && (coll() == "system.version");
+        return isAdminDB() && (coll() == "system.version");
     }
     bool isPrivilegeCollection() const {
         if (!isAdminDB()) {
@@ -515,7 +488,7 @@ public:
         return (coll() == kSystemUsers) || (coll() == kSystemRoles);
     }
     bool isConfigDB() const {
-        return db() == DatabaseName::kConfig.db();
+        return db_deprecated() == DatabaseName::kConfig.db(omitTenant);
     }
     bool isCommand() const {
         return coll() == "$cmd";
@@ -528,14 +501,25 @@ public:
     }
 
     bool isOrphanCollection() const {
-        return db() == kOrphanCollectionDb && coll().startsWith(kOrphanCollectionPrefix);
+        return isLocalDB() && coll().startsWith(kOrphanCollectionPrefix);
     }
 
     /**
-     * Returns whether the specified namespace is used for internal purposes only and can
-     * never be marked as anything other than UNSHARDED.
+     * foo = true
+     * foo. = false
+     * foo.a = false
      */
-    bool isNamespaceAlwaysUnsharded() const;
+    bool isDbOnly() const {
+        return kDataOffset + DatabaseName::sizeWithTenant() == _data.size();
+    }
+
+    /**
+     * Returns whether the specified namespace is never tracked in the sharding catalog.
+     *
+     * These class of namespaces are used for internal purposes only and they are only registered in
+     * the local catalog but not tracked by the sharding catalog.
+     */
+    bool isNamespaceAlwaysUntracked() const;
 
     /**
      * Returns whether the specified namespace is shard-local, meaning it exists independently on
@@ -593,6 +577,8 @@ public:
      */
     bool isFLE2StateCollection() const;
 
+    static bool isFLE2StateCollection(StringData coll);
+
     /**
      * Returns true if the namespace is an oplog or a change collection, false otherwise.
      */
@@ -690,8 +676,16 @@ public:
      * Returns true if the namespace is valid. Special namespaces for internal use are considered as
      * valid.
      */
-    bool isValid(DollarInDbNameBehavior behavior = DollarInDbNameBehavior::Allow) const {
-        return validDBName(db(), behavior) && !coll().empty();
+    bool isValid(DatabaseName::DollarInDbNameBehavior behavior =
+                     DatabaseName::DollarInDbNameBehavior::Allow) const {
+        return DatabaseName::validDBName(db_deprecated(), behavior) && !coll().empty();
+    }
+
+    static bool isValid(StringData ns,
+                        DatabaseName::DollarInDbNameBehavior behavior =
+                            DatabaseName::DollarInDbNameBehavior::Allow) {
+        const auto nss = NamespaceString(boost::none, ns);
+        return nss.isValid(behavior);
     }
 
     /**
@@ -712,29 +706,6 @@ public:
         return ns.startsWith("local.oplog.");
     }
 
-    /**
-     * samples:
-     *   good
-     *      foo
-     *      bar
-     *      foo-bar
-     *   bad:
-     *      foo bar
-     *      foo.bar
-     *      foo"bar
-     *
-     * @param db - a possible database name
-     * @param DollarInDbNameBehavior - please do not change the default value. DB names that must
-     *                                 contain a $ should be checked explicitly.
-     * @return if db is an allowed database name
-     */
-    static bool validDBName(StringData dbName,
-                            DollarInDbNameBehavior behavior = DollarInDbNameBehavior::Disallow);
-
-    static bool validDBName(const DatabaseName& dbName,
-                            DollarInDbNameBehavior behavior = DollarInDbNameBehavior::Disallow) {
-        return validDBName(dbName.db(), behavior);
-    }
 
     /**
      * Takes a fully qualified namespace (ie dbname.collectionName), and returns true if
@@ -748,7 +719,7 @@ public:
      * @param ns - a full namespace (a.b)
      * @return if db.coll is an allowed collection name
      */
-    static bool validCollectionComponent(StringData ns);
+    static bool validCollectionComponent(const NamespaceString& ns);
 
     /**
      * Takes a collection name and returns true if it is a valid collection name.
@@ -763,20 +734,12 @@ public:
      */
     static bool validCollectionName(StringData coll);
 
-    friend std::ostream& operator<<(std::ostream& stream, const NamespaceString& nss) {
-        return stream << nss.toString();
-    }
-
-    friend StringBuilder& operator<<(StringBuilder& builder, const NamespaceString& nss) {
-        return builder << nss.toString();
-    }
-
     int compare(const NamespaceString& other) const {
-        if (_hasTenantId() && !other._hasTenantId()) {
+        if (hasTenantId() && !other.hasTenantId()) {
             return 1;
         }
 
-        if (other._hasTenantId() && !_hasTenantId()) {
+        if (other.hasTenantId() && !hasTenantId()) {
             return -1;
         }
 
@@ -784,12 +747,28 @@ public:
             StringData{other._data.data() + kDataOffset, other._data.size() - kDataOffset});
     }
 
-    friend bool operator==(const NamespaceString& lhs, const NamespaceString& rhs) {
-        return lhs._data == rhs._data;
+    /**
+     * Checks if a given tenant prefixes or matches the tenantId from this NamespaceString.
+     * TODO SERVER-63517 Since we are removing tenant migration code we might be able to remove this
+     * method from the codebase.
+     */
+    bool isNamespaceForTenant(StringData tenant) const {
+        if (auto tid = tenantId()) {
+            return tid->toString() == tenant;
+        }
+        return db_deprecated().startsWith(tenant + "_");
     }
 
-    friend bool operator!=(const NamespaceString& lhs, const NamespaceString& rhs) {
-        return lhs._data != rhs._data;
+    /**
+     * Use to compare the TenantId and `db` part of a NamespaceString.
+     */
+    bool isEqualDb(const NamespaceString& other) const {
+        return tenantId() == other.tenantId() && db_deprecated() == other.db_deprecated();
+    }
+
+    friend bool operator==(const NamespaceString& lhs, const NamespaceString& rhs) {
+        return StringData{lhs._data.data(), lhs._data.size()} ==
+            StringData{rhs._data.data(), rhs._data.size()};
     }
 
     friend bool operator<(const NamespaceString& lhs, const NamespaceString& rhs) {
@@ -810,18 +789,24 @@ public:
 
     template <typename H>
     friend H AbslHashValue(H h, const NamespaceString& nss) {
-        return H::combine(std::move(h), nss._data);
+        return H::combine(std::move(h), std::string_view{nss._data.data(), nss._data.size()});
     }
 
     friend auto logAttrs(const NamespaceString& nss) {
         return "namespace"_attr = nss;
     }
 
+    /**
+     * This function removes the tenant id and returns the namespace part of NamespaceString.
+     */
+    friend StringData redactTenant(const NamespaceString& nss) {
+        return nss.ns();
+    }
+
 private:
-    friend NamespaceStringUtil;
-    // TODO SERVER-74897 IDLParserContext should no longer be a friend once IDL generated commands
-    // call into NamespaceStringUtil directly to construct NamespaceStrings.
-    friend IDLParserContext;
+    friend class NamespaceStringUtil;
+    friend class NamespaceStringTest;
+    friend class AuthNamespaceStringUtil;
 
     /**
      * In order to construct NamespaceString objects, use NamespaceStringUtil. The functions
@@ -832,30 +817,15 @@ private:
      * Constructs a NamespaceString from the fully qualified namespace named in "ns" and the
      * tenantId. "ns" is NOT expected to contain the tenantId.
      */
-    explicit NamespaceString(boost::optional<TenantId> tenantId, StringData ns)
-        : _data(makeData(std::move(tenantId), ns)) {}
+    NamespaceString(boost::optional<TenantId> tenantId, StringData ns)
+        : DatabaseName(Storage::make(tenantId, ns), TrustedInitTag{}) {}
 
     /**
      * Constructs a NamespaceString for the given database and collection names.
      * "dbName" must not contain a ".", and "collectionName" must not start with one.
      */
-    NamespaceString(DatabaseName dbName, StringData collectionName) {
-        uassert(ErrorCodes::InvalidNamespace,
-                "Collection names cannot start with '.': " + collectionName,
-                collectionName.empty() || collectionName[0] != '.');
-        uassert(ErrorCodes::InvalidNamespace,
-                "namespaces cannot have embedded null characters",
-                collectionName.find('\0') == std::string::npos);
-
-        _data.resize(dbName._data.size() + 1 + collectionName.size());
-        std::memcpy(_data.data(), dbName._data.data(), dbName._data.size());
-        *reinterpret_cast<uint8_t*>(_data.data() + dbName._data.size()) = '.';
-        if (!collectionName.empty()) {
-            std::memcpy(_data.data() + dbName._data.size() + 1,
-                        collectionName.rawData(),
-                        collectionName.size());
-        }
-    }
+    NamespaceString(DatabaseName dbName, StringData collectionName)
+        : DatabaseName(Storage::make(dbName, collectionName), TrustedInitTag{}) {}
 
     /**
      * Constructs a NamespaceString for the given db name, collection name, and tenantId.
@@ -863,88 +833,43 @@ private:
      * NOT expected to contain a tenantId.
      */
     NamespaceString(boost::optional<TenantId> tenantId, StringData db, StringData collectionName)
-        : _data(makeData(std::move(tenantId), db, collectionName)) {}
+        : DatabaseName(Storage::make(tenantId, db, collectionName),
+                       DatabaseName::TrustedInitTag{}) {}
+
+    /**
+     * Please refer to NamespaceStringUtil::serialize method or use ns_forTest to satisfy any unit
+     * test needing access to ns().
+     */
+    StringData ns() const {
+        auto offset = kDataOffset + tenantIdSize();
+        return StringData{_data.data() + offset, _data.size() - offset};
+    }
+
+    std::string toString() const {
+        return ns().toString();
+    }
+
+    std::string toStringWithTenantId() const {
+        if (hasTenantId()) {
+            return str::stream() << TenantId{OID::from(_data.data() + kDataOffset)} << "_" << ns();
+        }
+
+        return ns().toString();
+    }
+
+    /**
+     * This method is deprecated and will be removed as part of SERVER-65456. We strongly
+     * encourage to make the use of `dbName`, which returns a DatabaseName object instead.
+     * In case you would need to a StringData object instead we strongly recommend taking a look
+     * at the DatabaseNameUtil::serialize method which takes in a DatabaseName object.
+     */
+    StringData db_deprecated() const {
+        return dbName().db(omitTenant);
+    }
 
     static constexpr size_t kDataOffset = sizeof(uint8_t);
     static constexpr uint8_t kTenantIdMask = 0x80;
     static constexpr uint8_t kDatabaseNameOffsetEndMask = 0x7F;
-
-    inline bool _hasTenantId() const {
-        return static_cast<uint8_t>(_data.front()) & kTenantIdMask;
-    }
-
-    inline size_t _dbNameOffsetEnd() const {
-        return static_cast<uint8_t>(_data.front()) & kDatabaseNameOffsetEndMask;
-    }
-
-    std::string makeData(boost::optional<TenantId> tenantId,
-                         StringData db,
-                         StringData collectionName) {
-        uassert(ErrorCodes::InvalidNamespace,
-                "namespaces cannot have embedded null characters",
-                db.find('\0') == std::string::npos &&
-                    collectionName.find('\0') == std::string::npos);
-        uassert(ErrorCodes::InvalidNamespace,
-                fmt::format("Collection names cannot start with '.': {}", collectionName),
-                collectionName.empty() || collectionName[0] != '.');
-        uassert(ErrorCodes::InvalidNamespace,
-                fmt::format("db name must be at most {} characters, found: {}",
-                            DatabaseName::kMaxDatabaseNameLength,
-                            db.size()),
-                db.size() <= DatabaseName::kMaxDatabaseNameLength);
-
-        uint8_t details = db.size() & kDatabaseNameOffsetEndMask;
-        size_t dbStartIndex = kDataOffset;
-        if (tenantId) {
-            dbStartIndex += OID::kOIDSize;
-            details |= kTenantIdMask;
-        }
-
-        std::string data;
-        data.resize(collectionName.empty() ? dbStartIndex + db.size()
-                                           : dbStartIndex + db.size() + 1 + collectionName.size());
-        *reinterpret_cast<uint8_t*>(data.data()) = details;
-        if (tenantId) {
-            std::memcpy(data.data() + kDataOffset, tenantId->_oid.view().view(), OID::kOIDSize);
-        }
-
-        if (!db.empty()) {
-            std::memcpy(data.data() + dbStartIndex, db.rawData(), db.size());
-        }
-
-        if (!collectionName.empty()) {
-            *reinterpret_cast<uint8_t*>(data.data() + dbStartIndex + db.size()) = '.';
-            std::memcpy(data.data() + dbStartIndex + db.size() + 1,
-                        collectionName.rawData(),
-                        collectionName.size());
-        }
-
-        return data;
-    }
-
-    std::string makeData(boost::optional<TenantId> tenantId, StringData ns) {
-        auto dotIndex = ns.find('.');
-        if (dotIndex == std::string::npos) {
-            return makeData(tenantId, ns, {});
-        }
-
-        return makeData(tenantId, ns.substr(0, dotIndex), ns.substr(dotIndex + 1, ns.size()));
-    }
-
-    // In order to reduce the size of a NamespaceString, we pack all possible namespace data
-    // into a single std::string with the following in-memory layout:
-    //
-    //      1 byte         12 byte optional tenant id               remaining bytes
-    //    discriminator       (see more below)                        namespace
-    //  |<------------->|<--------------------------->|<-------------------------------------->|
-    //  [---------------|----|----|----|----|----|----|----|----|----|----|----|----|----|----|]
-    //  0               1                            12                                       ??
-    //
-    // The MSB of the discriminator tells us whether a tenant id is present, and the remaining
-    // bits store the offset of end of the databaes component of the namespace. Database names
-    // must be 64 characters or shorter, so we can be confident the length will fit in three bits.
-
-    std::string _data{'\0'};
 };
 
 /**
@@ -955,70 +880,51 @@ class NamespaceStringOrUUID {
 public:
     NamespaceStringOrUUID() = delete;
     NamespaceStringOrUUID(NamespaceString nss) : _nssOrUUID(std::move(nss)) {}
-    NamespaceStringOrUUID(const NamespaceString::ConstantProxy& nss)
-        : NamespaceStringOrUUID{static_cast<const NamespaceString&>(nss)} {}
+    // NamespaceStringOrUUID(const NamespaceString& nss) : _nssOrUUID(nss) {}
     NamespaceStringOrUUID(DatabaseName dbname, UUID uuid)
         : _nssOrUUID(UUIDWithDbName{std::move(dbname), std::move(uuid)}) {}
-    NamespaceStringOrUUID(boost::optional<TenantId> tenantId, std::string db, UUID uuid)
-        : _nssOrUUID(
-              UUIDWithDbName{DatabaseName{std::move(tenantId), std::move(db)}, std::move(uuid)}) {}
-    // TODO SERVER-65920 Remove once all call sites have been changed to take tenantId explicitly
-    NamespaceStringOrUUID(std::string db,
-                          UUID uuid,
-                          boost::optional<TenantId> tenantId = boost::none)
-        : _nssOrUUID(
-              UUIDWithDbName{DatabaseName{std::move(tenantId), std::move(db)}, std::move(uuid)}) {}
 
     bool isNamespaceString() const {
-        return stdx::holds_alternative<NamespaceString>(_nssOrUUID);
+        return holds_alternative<NamespaceString>(_nssOrUUID);
     }
 
     const NamespaceString& nss() const {
-        invariant(stdx::holds_alternative<NamespaceString>(_nssOrUUID));
+        invariant(holds_alternative<NamespaceString>(_nssOrUUID));
         return get<NamespaceString>(_nssOrUUID);
     }
 
     bool isUUID() const {
-        return stdx::holds_alternative<UUIDWithDbName>(_nssOrUUID);
+        return holds_alternative<UUIDWithDbName>(_nssOrUUID);
     }
 
     const UUID& uuid() const {
-        invariant(stdx::holds_alternative<UUIDWithDbName>(_nssOrUUID));
+        invariant(holds_alternative<UUIDWithDbName>(_nssOrUUID));
         return get<1>(get<UUIDWithDbName>(_nssOrUUID));
     }
 
-    /**
-     * Returns database name as a string.
-     *
-     * TODO SERVER-66887 remove this function for better clarity once call sites have been changed
-     */
-    std::string dbname() const {
-        return dbName().db().toString();
-    }
-
     /**
      * Returns the database name.
      */
-    DatabaseName dbName() const {
-        if (stdx::holds_alternative<NamespaceString>(_nssOrUUID)) {
+    const DatabaseName& dbName() const {
+        if (holds_alternative<NamespaceString>(_nssOrUUID)) {
             return get<NamespaceString>(_nssOrUUID).dbName();
         }
 
         return get<0>(get<UUIDWithDbName>(_nssOrUUID));
     }
 
-    std::string toString() const;
+    /**
+     * This function should only be used when logging a NamespaceStringOrUUID in an error message.
+     */
+    std::string toStringForErrorMsg() const;
+
+    /**
+     * Method to be used only when logging a NamespaceStringOrUUID in a log message.
+     */
+    friend std::string toStringForLogging(const NamespaceStringOrUUID& nssOrUUID);
 
     void serialize(BSONObjBuilder* builder, StringData fieldName) const;
 
-    friend std::ostream& operator<<(std::ostream& stream, const NamespaceStringOrUUID& o) {
-        return stream << o.toString();
-    }
-
-    friend StringBuilder& operator<<(StringBuilder& builder, const NamespaceStringOrUUID& o) {
-        return builder << o.toString();
-    }
-
     template <typename H>
     friend H AbslHashValue(H h, const NamespaceStringOrUUID& nssOrUUID) {
         if (nssOrUUID.isNamespaceString()) {
@@ -1028,9 +934,16 @@ public:
         }
     }
 
+    ConstDataRange asDataRange() const {
+        if (isNamespaceString()) {
+            return nss().asDataRange();
+        }
+        return uuid().asDataRange();
+    }
+
 private:
     using UUIDWithDbName = std::tuple<DatabaseName, UUID>;
-    stdx::variant<NamespaceString, UUIDWithDbName> _nssOrUUID;
+    std::variant<NamespaceString, UUIDWithDbName> _nssOrUUID;
 };
 
 /**
@@ -1079,58 +992,14 @@ inline bool nsIsFull(StringData ns) {
     return true;
 }
 
-/**
- * foo = true
- * foo. = false
- * foo.a = false
- */
-inline bool nsIsDbOnly(StringData ns) {
-    size_t i = ns.find('.');
-    if (i == std::string::npos)
-        return true;
-    return false;
-}
 
-inline bool NamespaceString::validDBName(StringData db, DollarInDbNameBehavior behavior) {
-    if (db.size() == 0 || db.size() > DatabaseName::kMaxDatabaseNameLength)
-        return false;
-
-    for (StringData::const_iterator iter = db.begin(), end = db.end(); iter != end; ++iter) {
-        switch (*iter) {
-            case '\0':
-            case '/':
-            case '\\':
-            case '.':
-            case ' ':
-            case '"':
-                return false;
-            case '$':
-                if (behavior == DollarInDbNameBehavior::Disallow)
-                    return false;
-                continue;
-#ifdef _WIN32
-            // We prohibit all FAT32-disallowed characters on Windows
-            case '*':
-            case '<':
-            case '>':
-            case ':':
-            case '|':
-            case '?':
-                return false;
-#endif
-            default:
-                continue;
-        }
-    }
-    return true;
-}
-
-inline bool NamespaceString::validCollectionComponent(StringData ns) {
-    size_t idx = ns.find('.');
+inline bool NamespaceString::validCollectionComponent(const NamespaceString& ns) {
+    const auto nsStr = ns.ns();
+    size_t idx = nsStr.find('.');
     if (idx == std::string::npos)
         return false;
 
-    return validCollectionName(ns.substr(idx + 1)) || oplog(ns);
+    return validCollectionName(nsStr.substr(idx + 1)) || oplog(nsStr);
 }
 
 inline bool NamespaceString::validCollectionName(StringData coll) {
@@ -1153,22 +1022,44 @@ inline bool NamespaceString::validCollectionName(StringData coll) {
     return true;
 }
 
-// Here are the `constexpr` definitions for the `NamespaceString::ConstantProxy`
+inline std::string stringifyForAssert(const NamespaceString& nss) {
+    return toStringForLogging(nss);
+}
+
+// Here are the `constexpr` definitions for the
 // constant static data members of `NamespaceString`. They cannot be defined
 // `constexpr` inside the class definition, but they can be upgraded to
 // `constexpr` here below it. Each one needs to be initialized with the address
-// of their associated shared state, so those are all defined first, as
+// of their associated data, so those are all defined first, as
 // variables named by the same `id`, but in separate nested namespace.
-namespace nss_detail::const_proxy_shared_states {
-#define NSS_CONSTANT(id, db, coll) \
-    constexpr inline NamespaceString::ConstantProxy::SharedState id{db, coll};
+
+// cribbed from https://accu.org/journals/overload/30/172/wu/
+namespace namespace_string_data {
+
+template <size_t dbSize, size_t collSize>
+constexpr auto makeNsData(const char* db, const char* coll) {
+    // No dot if both db and coll are empty.
+    constexpr size_t dot = !!collSize;
+    std::array<char, 1 + dbSize + dot + collSize> result{};
+    auto p = result.begin();
+    *p++ = dbSize;
+    p = std::copy_n(db, dbSize, p);
+    if (dot)
+        *p++ = '.';
+    p = std::copy_n(coll, collSize, p);
+    return result;
+}
+
+#define NSS_CONSTANT(id, dbname, coll) \
+    constexpr inline auto id##_data =  \
+        makeNsData<dbname.size(), coll.size()>(dbname.db(OmitTenant{}).rawData(), coll.rawData());
 #include "namespace_string_reserved.def.h"
 #undef NSS_CONSTANT
-}  // namespace nss_detail::const_proxy_shared_states
+}  // namespace namespace_string_data
 
-#define NSS_CONSTANT(id, db, coll)                                       \
-    constexpr inline NamespaceString::ConstantProxy NamespaceString::id{ \
-        &nss_detail::const_proxy_shared_states::id};
+#define NSS_CONSTANT(id, db, coll)                                                                \
+    constexpr inline NamespaceString NamespaceString::id(namespace_string_data::id##_data.data(), \
+                                                         namespace_string_data::id##_data.size());
 #include "namespace_string_reserved.def.h"
 #undef NSS_CONSTANT
 
