@@ -27,29 +27,48 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <cstring>
+#include <string>
+#include <utility>
+#include <vector>
 
-#include "mongo/db/op_msg_fuzzer_fixture.h"
+#include <boost/smart_ptr/intrusive_ptr.hpp>
 
-#include "mongo/db/auth/authorization_session_for_test.h"
-#include "mongo/db/auth/authz_manager_external_state_local.h"
+#include "mongo/base/initializer.h"
+#include "mongo/base/status.h"
+#include "mongo/base/string_data.h"
+#include "mongo/db/auth/authorization_manager.h"
+#include "mongo/db/catalog/collection.h"
 #include "mongo/db/catalog/collection_impl.h"
+#include "mongo/db/catalog/database_holder.h"
 #include "mongo/db/catalog/database_holder_impl.h"
-#include "mongo/db/client.h"
-#include "mongo/db/index/index_access_method.h"
+#include "mongo/db/concurrency/d_concurrency.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
+#include "mongo/db/op_msg_fuzzer_fixture.h"
+#include "mongo/db/op_observer/op_observer.h"
 #include "mongo/db/op_observer/op_observer_registry.h"
-#include "mongo/db/operation_context.h"
-#include "mongo/db/repl/repl_client_info.h"
+#include "mongo/db/repl/repl_settings.h"
+#include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/repl/replication_coordinator_mock.h"
-#include "mongo/db/s/collection_sharding_state_factory_standalone.h"
-#include "mongo/db/service_entry_point_common.h"
+#include "mongo/db/s/collection_sharding_state.h"
+#include "mongo/db/s/collection_sharding_state_factory_shard.h"
+#include "mongo/db/server_options.h"
 #include "mongo/db/service_entry_point_mongod.h"
+#include "mongo/db/session_manager_mongod.h"
 #include "mongo/db/storage/control/storage_control.h"
+#include "mongo/db/storage/recovery_unit_noop.h"
+#include "mongo/db/storage/storage_engine.h"
 #include "mongo/db/storage/storage_engine_init.h"
 #include "mongo/db/storage/storage_options.h"
 #include "mongo/db/vector_clock_mutable.h"
-#include "mongo/transport/service_entry_point_impl.h"
+#include "mongo/rpc/message.h"
+#include "mongo/s/sharding_state.h"
+#include "mongo/transport/service_entry_point.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/future.h"
 #include "mongo/util/periodic_runner_factory.h"
+#include "mongo/util/shared_buffer.h"
+#include "mongo/util/version/releases.h"
 
 namespace mongo {
 
@@ -63,13 +82,13 @@ void OpMsgFuzzerFixture::_setAuthorizationManager() {
     auto localExternalState = std::make_unique<AuthzManagerExternalStateMock>();
     _externalState = localExternalState.get();
 
-    auto localAuthzManager =
-        std::make_unique<AuthorizationManagerImpl>(_serviceContext, std::move(localExternalState));
+    auto localAuthzManager = std::make_unique<AuthorizationManagerImpl>(
+        _serviceContext->getService(), std::move(localExternalState));
     _authzManager = localAuthzManager.get();
     _externalState->setAuthorizationManager(_authzManager);
     _authzManager->setAuthEnabled(true);
 
-    AuthorizationManager::set(_serviceContext, std::move(localAuthzManager));
+    AuthorizationManager::set(_serviceContext->getService(), std::move(localAuthzManager));
 }
 
 OpMsgFuzzerFixture::OpMsgFuzzerFixture(bool skipGlobalInitializers)
@@ -84,17 +103,16 @@ OpMsgFuzzerFixture::OpMsgFuzzerFixture(bool skipGlobalInitializers)
 
     _serviceContext = getGlobalServiceContext();
     _setAuthorizationManager();
-    _serviceContext->setServiceEntryPoint(
-        std::make_unique<ServiceEntryPointMongod>(_serviceContext));
+    _serviceContext->getService()->setServiceEntryPoint(
+        std::make_unique<ServiceEntryPointMongod>());
 
     auto observerRegistry = std::make_unique<OpObserverRegistry>();
     _serviceContext->setOpObserver(std::move(observerRegistry));
 
     _serviceContext->setPeriodicRunner(makePeriodicRunner(_serviceContext));
 
-    _clientStrand = ClientStrand::make(_serviceContext->makeClient("test", _session));
+    _clientStrand = ClientStrand::make(_serviceContext->getService()->makeClient("test", _session));
     auto clientGuard = _clientStrand->bind();
-    auto opCtx = _serviceContext->makeOperationContext(clientGuard.get());
 
     storageGlobalParams.dbpath = _dir.path();
     storageGlobalParams.engine = "wiredTiger";
@@ -104,14 +122,22 @@ OpMsgFuzzerFixture::OpMsgFuzzerFixture(bool skipGlobalInitializers)
     // (Generic FCV reference): Initialize FCV.
     serverGlobalParams.mutableFCV.setVersion(multiversion::GenericFCV::kLatest);
 
-    initializeStorageEngine(opCtx.get(),
-                            StorageEngineInitFlags::kAllowNoLockFile |
-                                StorageEngineInitFlags::kSkipMetadataFile);
-    StorageControl::startStorageControls(_serviceContext, true /*forTestOnly*/);
+    {
+        auto initializeStorageEngineOpCtx =
+            _serviceContext->makeOperationContext(clientGuard.get());
+        shard_role_details::setRecoveryUnit(initializeStorageEngineOpCtx.get(),
+                                            std::make_unique<RecoveryUnitNoop>(),
+                                            WriteUnitOfWork::RecoveryUnitState::kNotInUnitOfWork);
 
+        initializeStorageEngine(initializeStorageEngineOpCtx.get(),
+                                StorageEngineInitFlags::kAllowNoLockFile |
+                                    StorageEngineInitFlags::kSkipMetadataFile);
+        StorageControl::startStorageControls(_serviceContext, true /*forTestOnly*/);
+    }
+
+    ShardingState::create(_serviceContext);
     CollectionShardingStateFactory::set(
-        _serviceContext,
-        std::make_unique<CollectionShardingStateFactoryStandalone>(_serviceContext));
+        _serviceContext, std::make_unique<CollectionShardingStateFactoryShard>(_serviceContext));
     DatabaseHolder::set(_serviceContext, std::make_unique<DatabaseHolderImpl>());
     Collection::Factory::set(_serviceContext, std::make_unique<CollectionImpl::FactoryImpl>());
 
@@ -120,7 +146,7 @@ OpMsgFuzzerFixture::OpMsgFuzzerFixture(bool skipGlobalInitializers)
         _serviceContext,
         std::make_unique<repl::ReplicationCoordinatorMock>(_serviceContext, repl::ReplSettings()));
 
-    _serviceContext->getStorageEngine()->notifyStartupComplete();
+    _serviceContext->getStorageEngine()->notifyStorageStartupRecoveryComplete();
 }
 
 OpMsgFuzzerFixture::~OpMsgFuzzerFixture() {
@@ -134,7 +160,7 @@ OpMsgFuzzerFixture::~OpMsgFuzzerFixture() {
         databaseHolder->closeAll(opCtx.get());
     }
 
-    shutdownGlobalStorageEngineCleanly(_serviceContext, true /* memLeakAllowed */);
+    shutdownGlobalStorageEngineCleanly(_serviceContext);
 }
 
 int OpMsgFuzzerFixture::testOneInput(const char* Data, size_t Size) {
@@ -153,7 +179,10 @@ int OpMsgFuzzerFixture::testOneInput(const char* Data, size_t Size) {
     Message msg(std::move(sb));
 
     try {
-        _serviceContext->getServiceEntryPoint()->handleRequest(opCtx.get(), msg).get();
+        _serviceContext->getService()
+            ->getServiceEntryPoint()
+            ->handleRequest(opCtx.get(), msg)
+            .get();
     } catch (const AssertionException&) {
         // We need to catch exceptions caused by invalid inputs
     }
