@@ -29,20 +29,75 @@
 
 #pragma once
 
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <cstddef>
+#include <cstdint>
+#include <memory>
+#include <set>
 #include <string>
+#include <vector>
 
+#include "mongo/base/status.h"
+#include "mongo/bson/bsonobj.h"
 #include "mongo/bson/simple_bsonobj_comparator.h"
+#include "mongo/bson/timestamp.h"
 #include "mongo/db/catalog/collection.h"
+#include "mongo/db/catalog/collection_options.h"
 #include "mongo/db/catalog/commit_quorum_options.h"
+#include "mongo/db/database_name.h"
 #include "mongo/db/jsobj.h"
+#include "mongo/db/namespace_string.h"
 #include "mongo/db/operation_context.h"
+#include "mongo/db/repl/oplog.h"
+#include "mongo/db/repl/oplog_entry.h"
+#include "mongo/db/repl/optime.h"
 #include "mongo/db/repl/rollback.h"
+#include "mongo/db/service_context.h"
+#include "mongo/db/session/logical_session_id.h"
+#include "mongo/db/session/logical_session_id_gen.h"
 #include "mongo/db/transaction/transaction_operations.h"
+#include "mongo/stdx/unordered_map.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/duration.h"
+#include "mongo/util/string_map.h"
+#include "mongo/util/time_support.h"
+#include "mongo/util/uuid.h"
 
 namespace mongo {
 
 struct InsertStatement;
 
+struct OpTimeBundle {
+    repl::OpTime writeOpTime;
+    Date_t wallClockTime;
+};
+
+/**
+ * The generic container for onUpdate/onDelete/onUnpreparedTransactionCommit state-passing between
+ * OpObservers. Despite the naming, some OpObserver's don't strictly observe. This struct is written
+ * by OpObserverImpl and useful for later observers to inspect state they need.
+ *
+ * These structs are decorable to support the sharing of critical resources between OpObserverImpl
+ * and MigrationChunkClonerSourceOpObserver. No other decorations should be added to these structs.
+ */
+struct OpStateAccumulator : Decorable<OpStateAccumulator> {
+    OpStateAccumulator() = default;
+
+    // Use either 'opTime' for non-insert operations or 'insertOpTimes', but not both.
+    OpTimeBundle opTime;
+    std::vector<repl::OpTime> insertOpTimes;
+
+    // Temporary pre/post image information for a retryable findAndModify operation to be written
+    // to the image collection (config.image_collection).
+    boost::optional<repl::ReplOperation::ImageBundle> retryableFindAndModifyImageToWrite;
+
+private:
+    OpStateAccumulator(const OpStateAccumulator&) = delete;
+    OpStateAccumulator& operator=(const OpStateAccumulator&) = delete;
+};
+
 enum class RetryableFindAndModifyLocation {
     // The operation is not retryable, or not a "findAndModify" command. Do not record a
     // pre-image.
@@ -68,9 +123,15 @@ struct OplogUpdateEntryArgs {
         : updateArgs(updateArgs), coll(coll) {}
 };
 
-struct OplogDeleteEntryArgs {
-    const BSONObj* deletedDoc = nullptr;
-
+/**
+ * Holds supplementary information required for OpObserver::onDelete() to write out an
+ * oplog entry for deleting a single document from a collection.
+ *
+ * This struct is also passed to OpObserver::aboutToDelete() so that OpObserver
+ * implementations may include additional information (via decorations) to be shared with
+ * the onDelete() method within the same implementation.
+ */
+struct OplogDeleteEntryArgs : Decorable<OplogDeleteEntryArgs> {
     // "fromMigrate" indicates whether the delete was induced by a chunk migration, and so
     // should be ignored by the user as an internal maintenance operation and not a real delete.
     bool fromMigrate = false;
@@ -80,8 +141,15 @@ struct OplogDeleteEntryArgs {
     RetryableFindAndModifyLocation retryableFindAndModifyLocation =
         RetryableFindAndModifyLocation::kNone;
 
-    // Set if OpTime were reserved for the delete ahead of time.
-    std::vector<OplogSlot> oplogSlots;
+    // Non-null when the RecordId for the delete is / needs to be replicated.
+    RecordId replicatedRecordId;
+
+    // Set if OpTimes were reserved for the delete ahead of time for this retryable
+    // "findAndModify" operation.
+    // Implies 'retryableFindAndModifyLocation' is set to kSideCollection but the
+    // other way round (because of multi-doc transactions).
+    // See reserveOplogSlotsForRetryableFindAndModify() in collection_write_path.cpp.
+    std::vector<OplogSlot> retryableFindAndModifyOplogSlots;
 };
 
 struct IndexCollModInfo {
@@ -197,20 +265,28 @@ public:
                                    bool fromMigrate) = 0;
 
     /**
+     * 'recordIds' is a vector of recordIds corresponding to the inserted documents.
+     * The presence of a non-empty vector of recordIds indicates that the recordIds should
+     * be added to the oplog.
      * 'fromMigrate' array contains settings for each insert operation and takes into
      * account orphan documents.
      * 'defaultFromMigrate' is the initial 'fromMigrate' value for the 'fromMigrate' array
      * and is intended to be forwarded to downstream subsystems that expect a single
      * 'fromMigrate' to describe the entire set of inserts.
      * Examples: ShardServerOpObserver, UserWriteBlockModeOpObserver, and
-     * OpObserverShardingImpl::shardObserveInsertsOp().
+     * MigrationChunkClonerSourceOpObserver::onInserts().
+     *
+     * The 'defaultFromMigrate' value must be consistent with the 'fromMigrate' array; that is,
+     * if 'defaultFromMigrate' is true, all entries in the 'fromMigrate' array must be true.
      */
     virtual void onInserts(OperationContext* opCtx,
                            const CollectionPtr& coll,
                            std::vector<InsertStatement>::const_iterator begin,
                            std::vector<InsertStatement>::const_iterator end,
+                           const std::vector<RecordId>& recordIds,
                            std::vector<bool> fromMigrate,
-                           bool defaultFromMigrate) = 0;
+                           bool defaultFromMigrate,
+                           OpStateAccumulator* opAccumulator = nullptr) = 0;
 
     virtual void onInsertGlobalIndexKey(OperationContext* opCtx,
                                         const NamespaceString& globalIndexNss,
@@ -224,25 +300,33 @@ public:
                                         const BSONObj& key,
                                         const BSONObj& docKey) = 0;
 
-    virtual void onUpdate(OperationContext* opCtx, const OplogUpdateEntryArgs& args) = 0;
+    virtual void onUpdate(OperationContext* opCtx,
+                          const OplogUpdateEntryArgs& args,
+                          OpStateAccumulator* opAccumulator = nullptr) = 0;
 
     virtual void aboutToDelete(OperationContext* opCtx,
                                const CollectionPtr& coll,
-                               const BSONObj& doc) = 0;
+                               const BSONObj& doc,
+                               OplogDeleteEntryArgs* args,
+                               OpStateAccumulator* opAccumulator = nullptr) = 0;
 
     /**
      * Handles logging before document is deleted.
      *
      * "ns" name of the collection from which deleteState.idDoc will be deleted.
      *
+     * "doc" holds the pre-image of the document to be deleted.
+     *
      * "args" is a reference to information detailing whether the pre-image of the doc should be
-     * preserved with deletion. If `retryableWritePreImageRecordingType != kNotRetryable`, then the
-     * opObserver must store the `deletedDoc` in addition to the documentKey.
+     * preserved with deletion. OpObserverImpl::aboutToDelete() initializes the documentKey as a
+     * decoration on OplogDeleteEntryArgs.
      */
     virtual void onDelete(OperationContext* opCtx,
                           const CollectionPtr& coll,
                           StmtId stmtId,
-                          const OplogDeleteEntryArgs& args) = 0;
+                          const BSONObj& doc,
+                          const OplogDeleteEntryArgs& args,
+                          OpStateAccumulator* opAccumulator = nullptr) = 0;
 
     /**
      * Logs a no-op with "msgObj" in the o field into oplog.
@@ -319,7 +403,9 @@ public:
                            const BSONObj& collModCmd,
                            const CollectionOptions& oldCollOptions,
                            boost::optional<IndexCollModInfo> indexInfo) = 0;
-    virtual void onDropDatabase(OperationContext* opCtx, const DatabaseName& dbName) = 0;
+    virtual void onDropDatabase(OperationContext* opCtx,
+                                const DatabaseName& dbName,
+                                bool markFromMigrate) = 0;
 
     /**
      * This function logs an oplog entry when a 'drop' command on a collection is executed.
@@ -328,19 +414,12 @@ public:
      *
      * 'dropType' describes whether the collection drop is one-phase or two-phase.
      */
-    virtual repl::OpTime onDropCollection(OperationContext* opCtx,
-                                          const NamespaceString& collectionName,
-                                          const UUID& uuid,
-                                          std::uint64_t numRecords,
-                                          CollectionDropType dropType) = 0;
     virtual repl::OpTime onDropCollection(OperationContext* opCtx,
                                           const NamespaceString& collectionName,
                                           const UUID& uuid,
                                           std::uint64_t numRecords,
                                           CollectionDropType dropType,
-                                          bool markFromMigrate) {
-        return onDropCollection(opCtx, collectionName, uuid, numRecords, dropType);
-    }
+                                          bool markFromMigrate) = 0;
 
 
     /**
@@ -366,13 +445,6 @@ public:
      * Returns the optime of the oplog entry successfully written to the oplog.
      * Returns a null optime if an oplog entry was not written for this operation.
      */
-    virtual repl::OpTime preRenameCollection(OperationContext* opCtx,
-                                             const NamespaceString& fromCollection,
-                                             const NamespaceString& toCollection,
-                                             const UUID& uuid,
-                                             const boost::optional<UUID>& dropTargetUUID,
-                                             std::uint64_t numRecords,
-                                             bool stayTemp) = 0;
     virtual repl::OpTime preRenameCollection(OperationContext* opCtx,
                                              const NamespaceString& fromCollection,
                                              const NamespaceString& toCollection,
@@ -380,10 +452,8 @@ public:
                                              const boost::optional<UUID>& dropTargetUUID,
                                              std::uint64_t numRecords,
                                              bool stayTemp,
-                                             bool markFromMigrate) {
-        return preRenameCollection(
-            opCtx, fromCollection, toCollection, uuid, dropTargetUUID, numRecords, stayTemp);
-    }
+                                             bool markFromMigrate) = 0;
+
     /**
      * This function performs all op observer handling for a 'renameCollection' command except for
      * logging the oplog entry. It should be used specifically in instances where the optime is
@@ -401,13 +471,6 @@ public:
      * executed. It calls preRenameCollection to log the entry and postRenameCollection to do all
      * other handling.
      */
-    virtual void onRenameCollection(OperationContext* opCtx,
-                                    const NamespaceString& fromCollection,
-                                    const NamespaceString& toCollection,
-                                    const UUID& uuid,
-                                    const boost::optional<UUID>& dropTargetUUID,
-                                    std::uint64_t numRecords,
-                                    bool stayTemp) = 0;
     virtual void onRenameCollection(OperationContext* opCtx,
                                     const NamespaceString& fromCollection,
                                     const NamespaceString& toCollection,
@@ -415,10 +478,7 @@ public:
                                     const boost::optional<UUID>& dropTargetUUID,
                                     std::uint64_t numRecords,
                                     bool stayTemp,
-                                    bool markFromMigrate) {
-        onRenameCollection(
-            opCtx, fromCollection, toCollection, uuid, dropTargetUUID, numRecords, stayTemp);
-    }
+                                    bool markFromMigrate) = 0;
 
     virtual void onImportCollection(OperationContext* opCtx,
                                     const UUID& importUUID,
@@ -429,9 +489,6 @@ public:
                                     const BSONObj& storageMetadata,
                                     bool isDryRun) = 0;
 
-    virtual void onApplyOps(OperationContext* opCtx,
-                            const DatabaseName& dbName,
-                            const BSONObj& applyOpCmd) = 0;
     virtual void onEmptyCapped(OperationContext* opCtx,
                                const NamespaceString& collectionName,
                                const UUID& uuid) = 0;
@@ -447,11 +504,21 @@ public:
      * transaction, before the RecoveryUnit onCommit() is called.  It must not be called when no
      * transaction is active.
      *
+     * 'reservedSlots' is a list of oplog slots reserved for the oplog entries in a transaction.
+     *
      * The 'transactionOperations' contains the list of CRUD operations (formerly 'statements') to
      * be applied in this transaction.
+     *
+     * The 'applyOpsOperationAssignment' contains a representation of "applyOps" entries and oplog
+     * slots to be used for writing pre- and post- image oplog entries for a transaction.
      */
     virtual void onUnpreparedTransactionCommit(
-        OperationContext* opCtx, const TransactionOperations& transactionOperations) = 0;
+        OperationContext* opCtx,
+        const std::vector<OplogSlot>& reservedSlots,
+        const TransactionOperations& transactionOperations,
+        const ApplyOpsOplogSlotAndOperationAssignment& applyOpsOperationAssignment,
+        OpStateAccumulator* opAccumulator = nullptr) = 0;
+
     /**
      * The onPreparedTransactionCommit method is called on the commit of a prepared transaction,
      * after the RecoveryUnit onCommit() is called.  It must not be called when no transaction is
@@ -480,7 +547,9 @@ public:
      * are gathered in a single applyOps oplog entry, similar to multi-doc transactions, and written
      * to the oplog.
      */
-    virtual void onBatchedWriteCommit(OperationContext* opCtx) = 0;
+    virtual void onBatchedWriteCommit(OperationContext* opCtx,
+                                      WriteUnitOfWork::OplogEntryGroupType oplogGroupingFormat,
+                                      OpStateAccumulator* opStateAccumulator = nullptr) = 0;
 
     /**
      * Clears the accumulated write operations. No further writes is allowed in this storage
@@ -493,25 +562,24 @@ public:
      * This method is called before an atomic transaction is prepared. It must be called when a
      * transaction is active.
      *
-     * Optionally returns a representation of "applyOps" entries to be written and oplog slots to be
-     * used for writing pre- and post- image oplog entries for a transaction. Only one OpObserver in
-     * the system should return the representation of "applyOps" entries. The returned value is
-     * passed to 'onTransactionPrepare()'.
-     *
-     * The 'reservedSlots' is a list of oplog slots reserved for the oplog entries in a transaction.
-     * The last reserved slot represents the prepareOpTime used for the prepare oplog entry.
-     *
-     * The 'wallClockTime' is the time to record as wall clock time on oplog entries resulting from
-     * transaction preparation.
+     * 'reservedSlots' is a list of oplog slots reserved for the oplog entries in a transaction. The
+     * last reserved slot represents the prepareOpTime used for the prepare oplog entry.
      *
      * The 'transactionOperations' contains the list of CRUD operations to be applied in this
      * transaction. The operations may be modified by setting pre-image and post-image oplog entry
      * timestamps.
+     *
+     * The 'applyOpsOperationAssignment' contains a representation of "applyOps" entries and oplog
+     * slots to be used for writing pre- and post- image oplog entries for a transaction.
+     *
+     * The 'wallClockTime' is the time to record as wall clock time on oplog entries resulting from
+     * transaction preparation.
      */
-    virtual std::unique_ptr<ApplyOpsOplogSlotAndOperationAssignment> preTransactionPrepare(
+    virtual void preTransactionPrepare(
         OperationContext* opCtx,
         const std::vector<OplogSlot>& reservedSlots,
         const TransactionOperations& transactionOperations,
+        const ApplyOpsOplogSlotAndOperationAssignment& applyOpsOperationAssignment,
         Date_t wallClockTime) = 0;
 
     /**
@@ -525,8 +593,8 @@ public:
      * this transaction.
      *
      * The 'applyOpsOperationAssignment' contains a representation of "applyOps" entries and oplog
-     * slots to be used for writing pre- and post- image oplog entries for a transaction. A value
-     * returned by 'preTransactionPrepare()' should be passed as 'applyOpsOperationAssignment'.
+     * slots to be used for writing pre- and post- image oplog entries for a transaction.
+     * The same "applyOps" information should be passed to 'preTransactionPrepare()'.
      *
      * The 'numberOfPrePostImagesToWrite' is the number of CRUD operations that have a pre-image
      * to write as a noop oplog entry. The op observer will reserve oplog slots for these
@@ -541,7 +609,22 @@ public:
         const TransactionOperations& transactionOperations,
         const ApplyOpsOplogSlotAndOperationAssignment& applyOpsOperationAssignment,
         size_t numberOfPrePostImagesToWrite,
-        Date_t wallClockTime) = 0;
+        Date_t wallClockTime,
+        OpStateAccumulator* opAccumulator = nullptr) = 0;
+
+    /**
+     * The postTransactionPrepare method is called after an atomic transaction is prepared. It must
+     * be called when a transaction is active.
+     *
+     * 'reservedSlots' is a list of oplog slots reserved for the oplog entries in a transaction. The
+     * last reserved slot represents the prepareOpTime used for the prepare oplog entry.
+     *
+     * The 'transactionOperations' contains the list of CRUD operations to be applied in
+     * this transaction.
+     */
+    virtual void postTransactionPrepare(OperationContext* opCtx,
+                                        const std::vector<OplogSlot>& reservedSlots,
+                                        const TransactionOperations& transactionOperations) = 0;
 
     /**
      * This method is called when a transaction transitions into prepare while it is not primary,
@@ -583,6 +666,9 @@ public:
         // Set of all namespaces from ops being rolled back.
         std::set<NamespaceString> rollbackNamespaces = {};
 
+        // Set of all collection UUIDs from ops being rolled back.
+        std::set<UUID> rollbackUUIDs = {};
+
         // Set of all session ids from ops being rolled back.
         std::set<UUID> rollbackSessionIds = {};
 
@@ -616,19 +702,9 @@ public:
      *
      * This method is only applicable to the "rollback to a stable timestamp" algorithm, and is not
      * called when using any other rollback algorithm i.e "rollback via refetch".
-     *
-     * This function will call the private virtual '_onReplicationRollback' method. Any exceptions
-     * thrown indicates rollback failure that may have led us to some inconsistent on-disk or memory
-     * state, so we crash instead.
      */
-    void onReplicationRollback(OperationContext* opCtx,
-                               const RollbackObserverInfo& rbInfo) noexcept {
-        try {
-            _onReplicationRollback(opCtx, rbInfo);
-        } catch (const DBException& ex) {
-            fassert(6050902, ex.toStatus());
-        }
-    };
+    virtual void onReplicationRollback(OperationContext* opCtx,
+                                       const RollbackObserverInfo& rbInfo) = 0;
 
     /**
      * Called when the majority commit point is updated by replication.
@@ -642,10 +718,6 @@ public:
 
     struct Times;
 
-private:
-    virtual void _onReplicationRollback(OperationContext* opCtx,
-                                        const RollbackObserverInfo& rbInfo) = 0;
-
 protected:
     class ReservedTimes;
 };
