@@ -28,26 +28,33 @@
  */
 
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/operation_context.h"
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
 
+#include "mongo/base/error_extra_info.h"
+#include "mongo/base/string_data.h"
 #include "mongo/db/client.h"
+#include "mongo/db/concurrency/locker.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/operation_key_manager.h"
 #include "mongo/db/service_context.h"
+#include "mongo/db/storage/storage_engine.h"
 #include "mongo/logv2/log.h"
-#include "mongo/platform/mutex.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/compiler.h"
 #include "mongo/platform/random.h"
+#include "mongo/stdx/thread.h"
 #include "mongo/transport/baton.h"
 #include "mongo/util/assert_util.h"
 #include "mongo/util/clock_source.h"
 #include "mongo/util/fail_point.h"
-#include "mongo/util/scopeguard.h"
 #include "mongo/util/system_tick_source.h"
+#include "mongo/util/waitable.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
 
-
 namespace mongo {
 
 MONGO_FAIL_POINT_DEFINE(maxTimeAlwaysTimeOut);
@@ -74,11 +81,8 @@ const auto kNoWaiterThread = stdx::thread::id();
 }  // namespace
 
 OperationContext::OperationContext(Client* client, OperationId opId)
-    : OperationContext(client, OperationIdSlot(opId)) {}
-
-OperationContext::OperationContext(Client* client, OperationIdSlot&& opIdSlot)
     : _client(client),
-      _opId(std::move(opIdSlot)),
+      _opId(opId),
       _elapsedTime(client ? client->getServiceContext()->getTickSource()
                           : globalSystemTickSource()) {}
 
@@ -147,12 +151,6 @@ bool OperationContext::hasDeadlineExpired() const {
         return true;
     }
 
-    // TODO: Remove once all OperationContexts are properly connected to Clients and ServiceContexts
-    // in tests.
-    if (MONGO_unlikely(!getClient() || !getServiceContext())) {
-        return false;
-    }
-
     const auto now = getServiceContext()->getFastClockSource()->now();
     return now >= getDeadline();
 }
@@ -220,17 +218,11 @@ bool opShouldFail(Client* client, const BSONObj& failPointInfo) {
 }  // namespace
 
 Status OperationContext::checkForInterruptNoAssert() noexcept {
-    // TODO: Remove the MONGO_likely(hasClientAndServiceContext) once all operation contexts are
-    // constructed with clients.
-    const auto hasClientAndServiceContext = getClient() && getServiceContext();
-
-    if (MONGO_likely(hasClientAndServiceContext) && getClient()->getKilled() &&
-        !_isExecutingShutdown) {
+    if (getClient()->getKilled() && !_isExecutingShutdown) {
         return Status(ErrorCodes::ClientMarkedKilled, "client has been killed");
     }
 
-    if (MONGO_likely(hasClientAndServiceContext) && getServiceContext()->getKillAllOperations() &&
-        !_isExecutingShutdown) {
+    if (getServiceContext()->getKillAllOperations() && !_isExecutingShutdown) {
         return Status(ErrorCodes::InterruptedAtShutdown, "interrupted at shutdown");
     }
 
@@ -264,16 +256,42 @@ Status OperationContext::checkForInterruptNoAssert() noexcept {
     }
 
     if (_markKillOnClientDisconnect) {
-        const auto now = getServiceContext()->getFastClockSource()->now();
+        if (auto status = _checkClientConnected(); !status.isOK()) {
+            return status;
+        }
+    }
 
-        if (now > _lastClientCheck + Milliseconds(500)) {
-            _lastClientCheck = now;
+    return Status::OK();
+}
 
-            if (!getClient()->session()->isConnected()) {
-                markKilled(ErrorCodes::ClientDisconnect);
-                return Status(ErrorCodes::ClientDisconnect,
-                              "operation was interrupted because a client disconnected");
-            }
+void OperationContext::_schedulePeriodicClientConnectedCheck() {
+    if (!_baton) {
+        return;
+    }
+
+    auto nextCheck = _lastClientCheck + Seconds(1);
+    _baton->waitUntil(nextCheck, getCancellationToken()).getAsync([&](auto waitStatus) {
+        if (!waitStatus.isOK()) {
+            return;
+        }
+        if (!_checkClientConnected().isOK()) {
+            return;
+        }
+        _schedulePeriodicClientConnectedCheck();
+    });
+}
+
+Status OperationContext::_checkClientConnected() {
+    const auto now = getServiceContext()->getFastClockSource()->now();
+
+    if (now > _lastClientCheck + Milliseconds(500)) {
+        _lastClientCheck = now;
+
+        auto client = getClient();
+        if (!client->session()->isConnected()) {
+            markKilled(client->getDisconnectErrorCode());
+            return Status(client->getDisconnectErrorCode(),
+                          "operation was interrupted because a client disconnected");
         }
     }
 
@@ -353,17 +371,10 @@ void OperationContext::markKilled(ErrorCodes::Error killCode) {
         invariant(!ErrorExtraInfo::parserFor(killCode));
     }
 
-    if (killCode == ErrorCodes::ClientDisconnect) {
+    if (killCode == getClient()->getDisconnectErrorCode()) {
         LOGV2(20883, "Interrupted operation as its client disconnected", "opId"_attr = getOpID());
     }
 
-    // Record that a kill was requested on this operationContext due to replication state change
-    // since it is possible to call markKilled() multiple times but only the first killCode will
-    // be preserved.
-    if (killCode == ErrorCodes::InterruptedDueToReplStateChange) {
-        _killRequestedForReplStateChange.store(true);
-    }
-
     if (auto status = ErrorCodes::OK; _killCode.compareAndSwap(&status, killCode)) {
         _cancelSource.cancel();
         if (_baton) {
@@ -373,6 +384,10 @@ void OperationContext::markKilled(ErrorCodes::Error killCode) {
 }
 
 void OperationContext::markKillOnClientDisconnect() {
+    if (!getClient()) {
+        return;
+    }
+
     if (getClient()->isInDirectClient()) {
         return;
     }
@@ -381,13 +396,18 @@ void OperationContext::markKillOnClientDisconnect() {
         return;
     }
 
-    if (getClient() && getClient()->session()) {
+    if (auto session = getClient()->session()) {
         _lastClientCheck = getServiceContext()->getFastClockSource()->now();
 
         _markKillOnClientDisconnect = true;
 
         if (_baton) {
-            _baton->markKillOnClientDisconnect();
+            if (auto networkingBaton = _baton->networking(); networkingBaton &&
+                networkingBaton->getTransportLayer() == session->getTransportLayer()) {
+                networkingBaton->markKillOnClientDisconnect();
+            } else {
+                _schedulePeriodicClientConnectedCheck();
+            }
         }
     }
 }
@@ -412,7 +432,7 @@ void OperationContext::setOperationKey(OperationKey opKey) {
     invariant(!_opKey);
 
     _opKey.emplace(std::move(opKey));
-    OperationKeyManager::get(_client).add(*_opKey, _opId.getId());
+    OperationKeyManager::get(_client).add(*_opKey, _opId);
 }
 
 void OperationContext::releaseOperationKey() {
@@ -434,7 +454,7 @@ void OperationContext::setTxnRetryCounter(TxnRetryCounter txnRetryCounter) {
     _txnRetryCounter = txnRetryCounter;
 }
 
-std::unique_ptr<RecoveryUnit> OperationContext::releaseRecoveryUnit() {
+std::unique_ptr<RecoveryUnit> OperationContext::releaseRecoveryUnit_DO_NOT_USE() {
     if (_recoveryUnit) {
         _recoveryUnit->setOperationContext(nullptr);
     }
@@ -442,21 +462,21 @@ std::unique_ptr<RecoveryUnit> OperationContext::releaseRecoveryUnit() {
     return std::move(_recoveryUnit);
 }
 
-std::unique_ptr<RecoveryUnit> OperationContext::releaseAndReplaceRecoveryUnit() {
-    auto ru = releaseRecoveryUnit();
-    setRecoveryUnit(
+std::unique_ptr<RecoveryUnit> OperationContext::releaseAndReplaceRecoveryUnit_DO_NOT_USE() {
+    auto ru = releaseRecoveryUnit_DO_NOT_USE();
+    setRecoveryUnit_DO_NOT_USE(
         std::unique_ptr<RecoveryUnit>(getServiceContext()->getStorageEngine()->newRecoveryUnit()),
         WriteUnitOfWork::RecoveryUnitState::kNotInUnitOfWork);
     return ru;
 }
 
-void OperationContext::replaceRecoveryUnit() {
-    setRecoveryUnit(
+void OperationContext::replaceRecoveryUnit_DO_NOT_USE() {
+    setRecoveryUnit_DO_NOT_USE(
         std::unique_ptr<RecoveryUnit>(getServiceContext()->getStorageEngine()->newRecoveryUnit()),
         WriteUnitOfWork::RecoveryUnitState::kNotInUnitOfWork);
 }
 
-WriteUnitOfWork::RecoveryUnitState OperationContext::setRecoveryUnit(
+WriteUnitOfWork::RecoveryUnitState OperationContext::setRecoveryUnit_DO_NOT_USE(
     std::unique_ptr<RecoveryUnit> unit, WriteUnitOfWork::RecoveryUnitState state) {
     _recoveryUnit = std::move(unit);
     if (_recoveryUnit) {
@@ -468,13 +488,14 @@ WriteUnitOfWork::RecoveryUnitState OperationContext::setRecoveryUnit(
     return oldState;
 }
 
-void OperationContext::setLockState(std::unique_ptr<Locker> locker) {
+void OperationContext::setLockState_DO_NOT_USE(std::unique_ptr<Locker> locker) {
     invariant(!_locker);
     invariant(locker);
     _locker = std::move(locker);
 }
 
-std::unique_ptr<Locker> OperationContext::swapLockState(std::unique_ptr<Locker> locker, WithLock) {
+std::unique_ptr<Locker> OperationContext::swapLockState_DO_NOT_USE(std::unique_ptr<Locker> locker,
+                                                                   WithLock clientLock) {
     invariant(_locker);
     invariant(locker);
     _locker.swap(locker);
