@@ -29,30 +29,41 @@
 
 #pragma once
 
-#include "mongo/util/assert_util.h"
+#include <algorithm>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
 #include <boost/optional.hpp>
-#include <cstddef>
+#include <boost/optional/optional.hpp>
 #include <memory>
+#include <utility>
 
+#include "mongo/base/error_codes.h"
 #include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/db/baton.h"
 #include "mongo/db/client.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
 #include "mongo/db/concurrency/locker.h"
 #include "mongo/db/operation_id.h"
 #include "mongo/db/query/datetime/date_time_support.h"
+#include "mongo/db/service_context.h"
 #include "mongo/db/session/logical_session_id.h"
+#include "mongo/db/session/logical_session_id_gen.h"
 #include "mongo/db/session/logical_session_id_helpers.h"
 #include "mongo/db/storage/recovery_unit.h"
-#include "mongo/db/storage/storage_options.h"
 #include "mongo/db/storage/write_unit_of_work.h"
 #include "mongo/db/write_concern_options.h"
 #include "mongo/platform/atomic_word.h"
-#include "mongo/platform/mutex.h"
 #include "mongo/stdx/condition_variable.h"
-#include "mongo/transport/session.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/cancellation.h"
 #include "mongo/util/concurrency/with_lock.h"
 #include "mongo/util/decorable.h"
+#include "mongo/util/duration.h"
 #include "mongo/util/fail_point.h"
+#include "mongo/util/inline_memory.h"
 #include "mongo/util/interruptible.h"
 #include "mongo/util/lockable_adapter.h"
 #include "mongo/util/time_support.h"
@@ -60,10 +71,7 @@
 
 namespace mongo {
 
-class CurOp;
-class ProgressMeter;
 class ServiceContext;
-class StringData;
 
 namespace repl {
 class UnreplicatedWritesBlock;
@@ -82,6 +90,36 @@ extern FailPoint maxTimeAlwaysTimeOut;
 // This fail point cannot be used with the maxTimeAlwaysTimeOut fail point.
 extern FailPoint maxTimeNeverTimeOut;
 
+/**
+ * This class encapsulates the OperationContext state, that must be initialized before all the
+ * decorations and destroyed after all the decorations.
+ */
+class OperationContextBase {
+    // Initial size of the monotic buffer. This size may need to be adjusted when additional use
+    // cases for monotonic buffer are added.
+    static constexpr size_t kMonotonicBufferInlineSize = 32;
+
+public:
+    using MonotonicAllocator = inline_memory::ResourceAllocator<
+        void,
+        inline_memory::ExternalResource<inline_memory::MonotonicBufferResource<>>>;
+    /**
+     * Returns the memory buffer that can be used to monotonically allocate memory.
+     * The memory will be freed at the time when OperationContext is destroyed.
+     * Note:
+     * This buffer *should not* be used for ephemeral allocations, as they will not be reclaimed
+     * until OperationContext is destroyed. And similarly this buffer *must not* be used for
+     * allocations that may be referenced after OperationContext is destroyed.
+     */
+    MonotonicAllocator monotonicAllocator() {
+        return _monotonicBuffer.makeAllocator<void>();
+    }
+
+private:
+    inline_memory::MemoryBuffer<kMonotonicBufferInlineSize, alignof(std::max_align_t)>
+        _monotonicBuffer;
+};
+
 /**
  * This class encompasses the state required by an operation and lives from the time a network
  * operation is dispatched until its execution is finished. Note that each "getmore" on a cursor
@@ -93,7 +131,9 @@ extern FailPoint maxTimeNeverTimeOut;
  * (RecoveryUnitState) to reduce complexity and duplication in the storage-engine specific
  * RecoveryUnit and to allow better invariant checking.
  */
-class OperationContext : public Interruptible, public Decorable<OperationContext> {
+class OperationContext final : public OperationContextBase,
+                               public Interruptible,
+                               public Decorable<OperationContext> {
     OperationContext(const OperationContext&) = delete;
     OperationContext& operator=(const OperationContext&) = delete;
 
@@ -105,63 +145,66 @@ public:
      * CTOR if possible to avoid OperationId collisions.
      */
     OperationContext(Client* client, OperationId opId);
-    OperationContext(Client* client, OperationIdSlot&& opIdSlot);
-    virtual ~OperationContext();
+    ~OperationContext() override;
 
-    /**
-     * Interface for durability.  Caller DOES NOT own pointer.
-     */
-    RecoveryUnit* recoveryUnit() const {
+    // TODO (SERVER-77213): The RecoveryUnit ownership is being moved to the TransactionResources.
+    // Do not add any new usages to these methods as they will go away and will be folded as an
+    // implementation detail of the Shard Role API.
+    //
+    // Interface for durability.  Caller DOES NOT own pointer.
+    RecoveryUnit* recoveryUnit_DO_NOT_USE() const {
         return _recoveryUnit.get();
     }
 
-    /**
-     * Returns the RecoveryUnit (same return value as recoveryUnit()) but the caller takes
-     * ownership of the returned RecoveryUnit, and the OperationContext instance relinquishes
-     * ownership. Sets the RecoveryUnit to NULL.
-     */
-    std::unique_ptr<RecoveryUnit> releaseRecoveryUnit();
+    // TODO (SERVER-77213): The RecoveryUnit ownership is being moved to the TransactionResources.
+    // Do not add any new usages to these methods as they will go away and will be folded as an
+    // implementation detail of the Shard Role API.
+    //
+    // Returns the RecoveryUnit (same return value as recoveryUnit()) but the caller takes
+    // ownership of the returned RecoveryUnit, and the OperationContext instance relinquishes
+    // ownership. Sets the RecoveryUnit to NULL.
+    std::unique_ptr<RecoveryUnit> releaseRecoveryUnit_DO_NOT_USE();
 
-    /*
-     * Sets up a new, inactive RecoveryUnit in the OperationContext. Destroys any previous recovery
-     * unit and executes its rollback handlers.
-     */
-    void replaceRecoveryUnit();
+    // TODO (SERVER-77213): The RecoveryUnit ownership is being moved to the TransactionResources.
+    // Do not add any new usages to these methods as they will go away and will be folded as an
+    // implementation detail of the Shard Role API.
+    //
+    // Sets up a new, inactive RecoveryUnit in the OperationContext. Destroys any previous recovery
+    // unit and executes its rollback handlers.
+    void replaceRecoveryUnit_DO_NOT_USE();
 
-    /*
-     * Similar to replaceRecoveryUnit(), but returns the previous recovery unit like
-     * releaseRecoveryUnit().
-     */
-    std::unique_ptr<RecoveryUnit> releaseAndReplaceRecoveryUnit();
+    // TODO (SERVER-77213): The RecoveryUnit ownership is being moved to the TransactionResources.
+    // Do not add any new usages to these methods as they will go away and will be folded as an
+    // implementation detail of the Shard Role API.
+    //
+    // Similar to replaceRecoveryUnit(), but returns the previous recovery unit like
+    // releaseRecoveryUnit().
+    std::unique_ptr<RecoveryUnit> releaseAndReplaceRecoveryUnit_DO_NOT_USE();
 
 
-    /**
-     * Associates the OperatingContext with a different RecoveryUnit for getMore or
-     * subtransactions, see RecoveryUnitSwap. The new state is passed and the old state is
-     * returned separately even though the state logically belongs to the RecoveryUnit,
-     * as it is managed by the OperationContext.
-     */
-    WriteUnitOfWork::RecoveryUnitState setRecoveryUnit(std::unique_ptr<RecoveryUnit> unit,
-                                                       WriteUnitOfWork::RecoveryUnitState state);
+    // TODO (SERVER-77213): The RecoveryUnit ownership is being moved to the TransactionResources.
+    // Do not add any new usages to these methods as they will go away and will be folded as an
+    // implementation detail of the Shard Role API.
+    //
+    // Associates the OperatingContext with a different RecoveryUnit for getMore or
+    // subtransactions, see RecoveryUnitSwap. The new state is passed and the old state is
+    // returned separately even though the state logically belongs to the RecoveryUnit,
+    // as it is managed by the OperationContext.
+    WriteUnitOfWork::RecoveryUnitState setRecoveryUnit_DO_NOT_USE(
+        std::unique_ptr<RecoveryUnit> unit, WriteUnitOfWork::RecoveryUnitState state);
 
-    /**
-     * Interface for locking.  Caller DOES NOT own pointer.
-     */
-    Locker* lockState() const {
+    // TODO (SERVER-77213): The locker ownership is being moved to the TransactionResources. Do not
+    // add any new usages to these methods as they will go away and will be folded as an
+    // implementation detail of the Shard Role API.
+    //
+    // The way to access the locker associated with a given OperationContext is through the
+    // shard_role_details::getLocker methods.
+    Locker* lockState_DO_NOT_USE() const {
         return _locker.get();
     }
-
-    /**
-     * Sets the locker for use by this OperationContext. Call during OperationContext
-     * initialization, only.
-     */
-    void setLockState(std::unique_ptr<Locker> locker);
-
-    /**
-     * Swaps the locker, releasing the old locker to the caller.  The Client lock is required to
-     * call this function.
-     */
-    std::unique_ptr<Locker> swapLockState(std::unique_ptr<Locker> locker, WithLock);
+    void setLockState_DO_NOT_USE(std::unique_ptr<Locker> locker);
+    std::unique_ptr<Locker> swapLockState_DO_NOT_USE(std::unique_ptr<Locker> locker,
+                                                     WithLock clientLock);
 
     /**
      * Returns Status::OK() unless this operation is in a killed state.
@@ -180,6 +223,11 @@ public:
         return _client->getServiceContext();
     }
 
+    /** Returns the Service under which this operation operates. */
+    Service* getService() const {
+        return _client ? _client->getService() : nullptr;
+    }
+
     /**
      * Returns the client under which this context runs.
      */
@@ -191,7 +239,7 @@ public:
      * Returns the operation ID associated with this operation.
      */
     OperationId getOpID() const {
-        return _opId.getId();
+        return _opId;
     }
 
     /**
@@ -214,6 +262,9 @@ public:
      */
     void releaseOperationKey();
 
+    // TODO (SERVER-80523): BEGIN Expose OperationSessionInfoFromClient as a decoration instead of
+    // projecting all its fields as properties
+
     /**
      * Returns the session ID associated with this operation, if there is one.
      */
@@ -235,6 +286,12 @@ public:
         return _txnNumber;
     }
 
+    /**
+     * Associates a transaction number with this operation context. May only be called once for the
+     * lifetime of the operation and the operation must have a logical session id assigned.
+     */
+    void setTxnNumber(TxnNumber txnNumber);
+
     /**
      * Returns the txnRetryCounter associated with this operation.
      */
@@ -242,6 +299,49 @@ public:
         return _txnRetryCounter;
     }
 
+    /**
+     * Associates a txnRetryCounter with this operation context. May only be called once for the
+     * lifetime of the operation and the operation must have a logical session id and a transaction
+     * number assigned.
+     */
+    void setTxnRetryCounter(TxnRetryCounter txnRetryCounter);
+
+    /**
+     * Returns whether this operation is part of a multi-document transaction. Specifically, it
+     * indicates whether the user asked for a multi-document transaction.
+     */
+    bool inMultiDocumentTransaction() const {
+        return _inMultiDocumentTransaction;
+    }
+
+    /**
+     * Sets that this operation is part of a multi-document transaction. Once this is set, it cannot
+     * be unset.
+     */
+    void setInMultiDocumentTransaction() {
+        _inMultiDocumentTransaction = true;
+        if (!_txnRetryCounter.has_value()) {
+            _txnRetryCounter = 0;
+        }
+    }
+
+    bool isRetryableWrite() const {
+        return _txnNumber &&
+            (!_inMultiDocumentTransaction ||
+             isInternalSessionForRetryableWrite(*getLogicalSessionId()));
+    }
+
+    bool isCommandForwardedFromRouter() const {
+        return _isCommandForwardedFromRouter;
+    }
+
+    void setCommandForwardedFromRouter() {
+        _isCommandForwardedFromRouter = true;
+    }
+
+    // TODO (SERVER-80523): END Expose OperationSessionInfoFromClient as a decoration instead of
+    // projecting all its fields as properties
+
     /**
      * Returns a CancellationToken that will be canceled when the OperationContext is killed via
      * markKilled (including for internal reasons, like the OperationContext deadline being
@@ -265,23 +365,10 @@ public:
         return _baton;
     }
 
-    /**
-     * Associates a transaction number with this operation context. May only be called once for the
-     * lifetime of the operation and the operation must have a logical session id assigned.
-     */
-    void setTxnNumber(TxnNumber txnNumber);
-
-    /**
-     * Associates a txnRetryCounter with this operation context. May only be called once for the
-     * lifetime of the operation and the operation must have a logical session id and a transaction
-     * number assigned.
-     */
-    void setTxnRetryCounter(TxnRetryCounter txnRetryCounter);
-
     /**
      * Returns the top-level WriteUnitOfWork associated with this operation context, if any.
      */
-    WriteUnitOfWork* getWriteUnitOfWork() {
+    WriteUnitOfWork* getWriteUnitOfWork_DO_NOT_USE() {
         return _writeUnitOfWork.get();
     }
 
@@ -289,7 +376,7 @@ public:
      * Sets a top-level WriteUnitOfWork for this operation context, to be held for the duration
      * of the given network operation.
      */
-    void setWriteUnitOfWork(std::unique_ptr<WriteUnitOfWork> writeUnitOfWork) {
+    void setWriteUnitOfWork_DO_NOT_USE(std::unique_ptr<WriteUnitOfWork> writeUnitOfWork) {
         invariant(writeUnitOfWork || _writeUnitOfWork);
         invariant(!(writeUnitOfWork && _writeUnitOfWork));
 
@@ -445,31 +532,6 @@ public:
 
     bool isIgnoringInterrupts() const;
 
-    /**
-     * Returns whether this operation is part of a multi-document transaction. Specifically, it
-     * indicates whether the user asked for a multi-document transaction.
-     */
-    bool inMultiDocumentTransaction() const {
-        return _inMultiDocumentTransaction;
-    }
-
-    bool isRetryableWrite() const {
-        return _txnNumber &&
-            (!_inMultiDocumentTransaction ||
-             isInternalSessionForRetryableWrite(*getLogicalSessionId()));
-    }
-
-    /**
-     * Sets that this operation is part of a multi-document transaction. Once this is set, it cannot
-     * be unset.
-     */
-    void setInMultiDocumentTransaction() {
-        _inMultiDocumentTransaction = true;
-        if (!_txnRetryCounter.has_value()) {
-            _txnRetryCounter = 0;
-        }
-    }
-
     /**
      * Some operations coming into the system must be validated to ensure they meet constraints,
      * such as collection namespace length limits or unique index key constraints. However,
@@ -515,6 +577,17 @@ public:
         return _alwaysInterruptAtStepDownOrUp.load();
     }
 
+    /**
+     * Indicates that this operation should not receive interruptions while acquiring locks. Note
+     * that new usages of this require the use of UninterruptibleLockGuard and are subject to the
+     * same restrictions.
+     *
+     * TODO SERVER-68868: Remove this once UninterruptibleLockGuard is removed from the codebase.
+     */
+    bool uninterruptibleLocksRequested_DO_NOT_USE() const {
+        return _interruptibleLocksRequested < 0;
+    }
+
     /**
      * Clears metadata associated with a multi-document transaction.
      */
@@ -524,6 +597,7 @@ public:
         invariant(_ruState == WriteUnitOfWork::RecoveryUnitState::kNotInUnitOfWork);
         _inMultiDocumentTransaction = false;
         _isStartingMultiDocumentTransaction = false;
+        _isActiveTransactionParticipant = false;
         _lsid = boost::none;
         _txnNumber = boost::none;
         _txnRetryCounter = boost::none;
@@ -551,6 +625,22 @@ public:
         _isStartingMultiDocumentTransaction = isStartingMultiDocumentTransaction;
     }
 
+    /**
+     * Set if this op is being executed by an active transaction participant. Used to differentiate
+     * from an op being coordinated by a transaction router.
+     */
+    void setActiveTransactionParticipant() {
+        invariant(_inMultiDocumentTransaction || isRetryableWrite());
+        _isActiveTransactionParticipant = true;
+    }
+
+    /**
+     * Returns whether this op is being executed by an active transaction participant.
+     */
+    bool isActiveTransactionParticipant() const {
+        return _isActiveTransactionParticipant;
+    }
+
     /**
      * Sets '_comment'. The client lock must be acquired before calling this method.
      */
@@ -589,6 +679,20 @@ public:
         _storedMaxTime = maxTime;
     }
 
+    /**
+     * Sets whether the maxTime used by this operation is the default value.
+     */
+    void setUsesDefaultMaxTimeMS(bool usesDefaultMaxTimeMS) {
+        _usesDefaultMaxTimeMS = usesDefaultMaxTimeMS;
+    }
+
+    /**
+     * Gets whether the maxTime used by this operation is the default value.
+     */
+    bool usesDefaultMaxTimeMS() const {
+        return _usesDefaultMaxTimeMS;
+    }
+
     /**
      * Restore deadline to match the value stored in _storedMaxTime.
      */
@@ -651,6 +755,14 @@ public:
         _querySamplingOpts = option;
     }
 
+    void setRoutedByReplicaSetEndpoint(bool value) {
+        _routedByReplicaSetEndpoint = value;
+    }
+
+    bool routedByReplicaSetEndpoint() const {
+        return _routedByReplicaSetEndpoint;
+    }
+
     /**
      * Invokes the passed callback while ignoring interrupts. Note that this causes the deadline to
      * be reset to Date_t::max(), but that it can also subsequently be reduced in size after the
@@ -758,13 +870,34 @@ private:
         --_lockFreeReadOpCount;
     }
 
+    /**
+     * Schedule the client to be checked every second. If the client has disconnected, the operation
+     * will be killed. Periodic checks are not needed if the client's session is compatible with the
+     * networking baton associated with this opCtx.
+     *
+     * If there is no associated baton or it is not a networking baton, this method has no effect.
+     */
+    void _schedulePeriodicClientConnectedCheck();
+
+    /**
+     * If the client is networked, check that its underlying session is still connected. If the
+     * session is not connected, kill the operation. The status used to kill the operation will be
+     * returned.
+     *
+     * This will only actually check the underlying session every 500ms regardless of how often this
+     * is called, since doing so may be expensive.
+     */
+    Status _checkClientConnected();
+
     friend class WriteUnitOfWork;
     friend class repl::UnreplicatedWritesBlock;
     friend class LockFreeReadsBlock;
+    friend class InterruptibleLockGuard;
+    friend class UninterruptibleLockGuard;
 
     Client* const _client;
 
-    const OperationIdSlot _opId;
+    const OperationId _opId;
     boost::optional<OperationKey> _opKey;
 
     boost::optional<LogicalSessionId> _lsid;
@@ -804,6 +937,13 @@ private:
     Date_t _lastClientCheck;
     bool _isExecutingShutdown = false;
 
+    /**
+     * Contains the number of requesters for both InterruptibleLockGuard and
+     * UninterruptibleLockGuard. It is > 0 on the first case and < 0 in the other. The absolute
+     * number specifies how many requesters there are for each type.
+     */
+    int _interruptibleLocksRequested = 0;
+
     // Max operation time requested by the user or by the cursor in the case of a getMore with no
     // user-specified maxTimeMS. This is tracked with microsecond granularity for the purpose of
     // assigning unused execution time back to a cursor at the end of an operation, only. The
@@ -814,6 +954,8 @@ private:
     // The value of the maxTimeMS requested by user in the case it was overwritten.
     boost::optional<Microseconds> _storedMaxTime;
 
+    bool _usesDefaultMaxTimeMS = false;
+
     // Timer counting the elapsed time since the construction of this OperationContext.
     Timer _elapsedTime;
 
@@ -821,6 +963,8 @@ private:
     bool _shouldIncrementLatencyStats = true;
     bool _inMultiDocumentTransaction = false;
     bool _isStartingMultiDocumentTransaction = false;
+    bool _isActiveTransactionParticipant = false;
+    bool _isCommandForwardedFromRouter = false;
     // Commands from user applications must run validations and enforce constraints. Operations from
     // a trusted source, such as initial sync or consuming an oplog entry generated by a primary
     // typically desire to ignore constraints.
@@ -835,8 +979,6 @@ private:
     // of what locks it's taken.
     AtomicWord<bool> _alwaysInterruptAtStepDownOrUp{false};
 
-    AtomicWord<bool> _killRequestedForReplStateChange{false};
-
     // If populated, this is an owned singleton BSONObj whose only field, 'comment', is a copy of
     // the 'comment' field from the input command object.
     boost::optional<BSONObj> _comment;
@@ -857,6 +999,71 @@ private:
 
     // The query sampling options for operations on this opCtx.
     boost::optional<QuerySamplingOptions> _querySamplingOpts;
+
+    // Set to true if this operation is going through the router code paths because of the replica
+    // set endpoint.
+    bool _routedByReplicaSetEndpoint = false;
+};
+
+/**
+ * DO NOT USE THIS CLASS. USING THIS IS A PROGRAMMING ERROR AND WILL REQUIRE REVIEW FROM SERVICE
+ * ARCH. This class is here in order to transition the logic from Locker away into the
+ * OperationContext. It is here because multi-document transactions can migrate a locker across
+ * multiple living opCtx executions. Please see more details in SERVER-88292.
+ *
+ * This class prevents the given OperationContext from being interrupted while acquiring locks as
+ * long as it is in scope. The default behavior of acquisitions depends on the type of lock that is
+ * being requested. Use this in the unlikely case that waiting for a lock can't be interrupted.
+ *
+ * It is possible that multiple callers are requesting uninterruptible behavior, so the guard
+ * increments a counter on the OperationContext class to indicate how may guards are active.
+ *
+ * TODO SERVER-68868: Remove this class.
+ */
+class UninterruptibleLockGuard {
+public:
+    explicit UninterruptibleLockGuard(OperationContext* opCtx) : _opCtx(opCtx) {
+        invariant(_opCtx);
+        invariant(_opCtx->_interruptibleLocksRequested <= 0);
+        _opCtx->_interruptibleLocksRequested--;
+    }
+
+    UninterruptibleLockGuard(const UninterruptibleLockGuard& other) = delete;
+    UninterruptibleLockGuard& operator=(const UninterruptibleLockGuard&) = delete;
+
+    ~UninterruptibleLockGuard() {
+        invariant(_opCtx->_interruptibleLocksRequested < 0);
+        _opCtx->_interruptibleLocksRequested++;
+    }
+
+private:
+    OperationContext* const _opCtx;
+};
+
+/**
+ * This RAII type ensures that there are no UninterruptibleLockGuards while in scope. If an
+ * UninterruptibleLockGuard is held at a higher level, or taken at a lower level, an invariant will
+ * occur. This protects against UninterruptibleLockGuard uses on code paths that must be
+ * interruptible. Safe to nest InterruptibleLockGuard instances.
+ */
+class InterruptibleLockGuard {
+public:
+    explicit InterruptibleLockGuard(OperationContext* opCtx) : _opCtx(opCtx) {
+        invariant(_opCtx);
+        invariant(_opCtx->_interruptibleLocksRequested >= 0);
+        _opCtx->_interruptibleLocksRequested++;
+    }
+
+    InterruptibleLockGuard(const InterruptibleLockGuard& other) = delete;
+    InterruptibleLockGuard& operator=(const InterruptibleLockGuard&) = delete;
+
+    ~InterruptibleLockGuard() {
+        invariant(_opCtx->_interruptibleLocksRequested > 0);
+        _opCtx->_interruptibleLocksRequested--;
+    }
+
+private:
+    OperationContext* const _opCtx;
 };
 
 // Gets a TimeZoneDatabase pointer from the ServiceContext.
@@ -898,12 +1105,25 @@ class LockFreeReadsBlock {
     LockFreeReadsBlock& operator=(const LockFreeReadsBlock&) = delete;
 
 public:
+    // Allow move operators.
+    LockFreeReadsBlock(LockFreeReadsBlock&& rhs) : _opCtx(rhs._opCtx) {
+        rhs._opCtx = nullptr;
+    };
+    LockFreeReadsBlock& operator=(LockFreeReadsBlock&& rhs) {
+        _opCtx = rhs._opCtx;
+        rhs._opCtx = nullptr;
+
+        return *this;
+    };
+
     LockFreeReadsBlock(OperationContext* opCtx) : _opCtx(opCtx) {
         _opCtx->incrementLockFreeReadOpCount();
     }
 
     ~LockFreeReadsBlock() {
-        _opCtx->decrementLockFreeReadOpCount();
+        if (_opCtx) {
+            _opCtx->decrementLockFreeReadOpCount();
+        }
     }
 
 private:
