@@ -28,30 +28,51 @@
  */
 
 
-#include "mongo/platform/basic.h"
-
-#include <boost/optional.hpp>
+// IWYU pragma: no_include "cxxabi.h"
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <functional>
+#include <future>
+#include <initializer_list>
 #include <memory>
+#include <mutex>
+#include <ostream>
+#include <ratio>
+#include <string>
+#include <type_traits>
 
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/db/client.h"
 #include "mongo/db/curop.h"
-#include "mongo/db/json.h"
+#include "mongo/db/namespace_string.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/operation_context_group.h"
+#include "mongo/db/pipeline/expression_context.h"
+#include "mongo/db/query/collation/collator_interface.h"
 #include "mongo/db/service_context.h"
 #include "mongo/db/service_context_test_fixture.h"
 #include "mongo/db/session/logical_session_id.h"
 #include "mongo/logv2/log.h"
-#include "mongo/logv2/log_debug.h"
-#include "mongo/stdx/future.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/mutex.h"
+#include "mongo/stdx/future.h"  // IWYU pragma: keep
 #include "mongo/stdx/thread.h"
 #include "mongo/transport/session.h"
 #include "mongo/transport/transport_layer_mock.h"
+#include "mongo/unittest/assert.h"
 #include "mongo/unittest/barrier.h"
 #include "mongo/unittest/death_test.h"
+#include "mongo/unittest/framework.h"
 #include "mongo/unittest/join_thread.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/util/clock_source.h"
 #include "mongo/util/clock_source_mock.h"
+#include "mongo/util/future.h"
+#include "mongo/util/intrusive_counter.h"
+#include "mongo/util/tick_source.h"
 #include "mongo/util/tick_source_mock.h"
 #include "mongo/util/time_support.h"
 
@@ -71,7 +92,7 @@ class OperationContextTest : public ServiceContextTest {
 public:
     auto makeClient(std::string desc = "OperationContextTest",
                     std::shared_ptr<transport::Session> session = nullptr) {
-        return getServiceContext()->makeClient(desc, session);
+        return getServiceContext()->getService()->makeClient(desc, session);
     }
 };
 
@@ -143,12 +164,12 @@ TEST_F(OperationContextTest, OpCtxGroup) {
     ASSERT_TRUE(group1.isEmpty());
     {
         auto serviceCtx1 = ServiceContext::make();
-        auto client1 = serviceCtx1->makeClient("OperationContextTest1");
+        auto client1 = serviceCtx1->getService()->makeClient("OperationContextTest1");
         auto opCtx1 = group1.makeOperationContext(*client1);
         ASSERT_FALSE(group1.isEmpty());
 
         auto serviceCtx2 = ServiceContext::make();
-        auto client2 = serviceCtx2->makeClient("OperationContextTest2");
+        auto client2 = serviceCtx2->getService()->makeClient("OperationContextTest2");
         {
             auto opCtx2 = group1.makeOperationContext(*client2);
             opCtx1.discard();
@@ -169,7 +190,7 @@ TEST_F(OperationContextTest, OpCtxGroup) {
     OperationContextGroup group2;
     {
         auto serviceCtx = ServiceContext::make();
-        auto client = serviceCtx->makeClient("OperationContextTest");
+        auto client = serviceCtx->getService()->makeClient("OperationContextTest");
         auto opCtx = group2.adopt(client->makeOperationContext());
         ASSERT_FALSE(group2.isEmpty());
         ASSERT_TRUE(opCtx->checkForInterruptNoAssert().isOK());
@@ -184,7 +205,7 @@ TEST_F(OperationContextTest, OpCtxGroup) {
     OperationContextGroup group4;
     {
         auto serviceCtx = ServiceContext::make();
-        auto client = serviceCtx->makeClient("OperationContextTest");
+        auto client = serviceCtx->getService()->makeClient("OperationContextTest");
         auto opCtx1 = group3.makeOperationContext(*client);
         auto p1 = opCtx1.opCtx();
         auto opCtx2 = group4.take(std::move(opCtx1));
@@ -266,12 +287,12 @@ TEST_F(OperationContextTest, CancellationTokenIsCancelableAtFirst) {
 
 class OperationDeadlineTests : public OperationContextTest {
 public:
-    void setUp() {
+    void setUp() override {
         ServiceContext* serviceCtx = getServiceContext();
         serviceCtx->setFastClockSource(std::make_unique<SharedClockSourceAdapter>(mockClock));
         serviceCtx->setPreciseClockSource(std::make_unique<SharedClockSourceAdapter>(mockClock));
         serviceCtx->setTickSource(std::make_unique<TickSourceMock<>>());
-        client = serviceCtx->makeClient("OperationDeadlineTest");
+        client = serviceCtx->getService()->makeClient("OperationDeadlineTest");
     }
 
     void checkForInterruptForTimeout(OperationContext* opCtx) {
@@ -738,7 +759,7 @@ public:
                                  boost::optional<Date_t> maxTime,
                                  WaitFn waitFn) {
             auto barrier = std::make_shared<unittest::Barrier>(2);
-            task = stdx::packaged_task<bool()>([=] {
+            task = stdx::packaged_task<bool()>([=, this] {
                 if (maxTime)
                     opCtx->setDeadlineByDate(*maxTime, ErrorCodes::ExceededTimeLimit);
                 stdx::unique_lock<Latch> lk(mutex);
@@ -1068,44 +1089,13 @@ TEST_F(OperationContextTest, TestIsWaitingForConditionOrInterrupt) {
     ASSERT_FALSE(optCtx->isWaitingForConditionOrInterrupt());
 }
 
-TEST_F(OperationContextTest, TestActiveClientOperationsForClientsWithoutSession) {
-    auto serviceCtx = getServiceContext();
-    auto client = serviceCtx->makeClient("OperationContextTest");
-    ASSERT_EQ(serviceCtx->getActiveClientOperations(), 0);
-    {
-        auto opCtx = client->makeOperationContext();
-        ASSERT_EQ(serviceCtx->getActiveClientOperations(), 0);
-    }
-    ASSERT_EQ(serviceCtx->getActiveClientOperations(), 0);
-}
-
-TEST_F(OperationContextTest, TestActiveClientOperations) {
-    transport::TransportLayerMock transportLayer;
-    std::shared_ptr<transport::Session> session = transportLayer.createSession();
-
-    auto serviceCtx = getServiceContext();
-    auto client = serviceCtx->makeClient("OperationContextTest", session);
-    ASSERT_EQ(serviceCtx->getActiveClientOperations(), 0);
-
-    {
-        auto optCtx = client->makeOperationContext();
-        ASSERT_EQ(serviceCtx->getActiveClientOperations(), 1);
-    }
-    ASSERT_EQ(serviceCtx->getActiveClientOperations(), 0);
-
-    {
-        auto optCtx = client->makeOperationContext();
-        ASSERT_EQ(serviceCtx->getActiveClientOperations(), 1);
-        serviceCtx->killAndDelistOperation(optCtx.get());
-        ASSERT_EQ(serviceCtx->getActiveClientOperations(), 0);
-    }
-    ASSERT_EQ(serviceCtx->getActiveClientOperations(), 0);
-}
-
 TEST_F(OperationContextTest, CurrentOpExcludesKilledOperations) {
-    auto client = makeClient("MainClient");
+    auto client = getService()->makeClient("MainClient");
     auto opCtx = client->makeOperationContext();
 
+    const boost::intrusive_ptr<ExpressionContext> expCtx(new ExpressionContext(
+        opCtx.get(), nullptr, NamespaceString::createNamespaceString_forTest("foo.bar"_sd)));
+
     for (auto truncateOps : {true, false}) {
         for (auto backtraceMode : {true, false}) {
             BSONObjBuilder bobNoOpCtx, bobKilledOpCtx;
@@ -1115,18 +1105,18 @@ TEST_F(OperationContextTest, CurrentOpExcludesKilledOperations) {
             stdx::thread thread([&]() mutable {
                 stdx::lock_guard<Client> lk(*opCtx->getClient());
 
-                auto threadClient = makeClient("ThreadClient");
+                auto threadClient = getService()->makeClient("ThreadClient");
 
                 // Generate report in absence of any opCtx
                 CurOp::reportCurrentOpForClient(
-                    opCtx.get(), threadClient.get(), truncateOps, backtraceMode, &bobNoOpCtx);
+                    expCtx, threadClient.get(), truncateOps, backtraceMode, &bobNoOpCtx);
 
                 auto threadOpCtx = threadClient->makeOperationContext();
                 getServiceContext()->killAndDelistOperation(threadOpCtx.get());
 
                 // Generate report in presence of a killed opCtx
                 CurOp::reportCurrentOpForClient(
-                    opCtx.get(), threadClient.get(), truncateOps, backtraceMode, &bobKilledOpCtx);
+                    expCtx, threadClient.get(), truncateOps, backtraceMode, &bobKilledOpCtx);
             });
 
             thread.join();
