@@ -27,48 +27,51 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <boost/none.hpp>
+#include <memory>
 
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/db/curop.h"
 #include "mongo/db/ops/delete.h"
-
-#include "mongo/db/catalog/database.h"
 #include "mongo/db/ops/parsed_delete.h"
 #include "mongo/db/query/get_executor.h"
-#include "mongo/db/repl/repl_client_info.h"
+#include "mongo/db/shard_role.h"
+#include "mongo/util/assert_util.h"
 
 namespace mongo {
 
 long long deleteObjects(OperationContext* opCtx,
-                        const CollectionPtr& collection,
-                        const NamespaceString& ns,
+                        const CollectionAcquisition& collection,
                         BSONObj pattern,
                         bool justOne,
                         bool god,
                         bool fromMigrate) {
     auto request = DeleteRequest{};
-    request.setNsString(ns);
+    request.setNsString(collection.nss());
     request.setQuery(pattern);
     request.setMulti(!justOne);
     request.setGod(god);
     request.setFromMigrate(fromMigrate);
 
-    ParsedDelete parsedDelete(opCtx, &request);
+    ParsedDelete parsedDelete(opCtx, &request, collection.getCollectionPtr());
     uassertStatusOK(parsedDelete.parseRequest());
 
     auto exec = uassertStatusOK(getExecutorDelete(
-        &CurOp::get(opCtx)->debug(), &collection, &parsedDelete, boost::none /* verbosity */));
+        &CurOp::get(opCtx)->debug(), collection, &parsedDelete, boost::none /* verbosity */));
 
     return exec->executeDelete();
 }
 
 DeleteResult deleteObject(OperationContext* opCtx,
-                          const CollectionPtr& collection,
+                          const CollectionAcquisition& collection,
                           const DeleteRequest& request) {
-    ParsedDelete parsedDelete(opCtx, &request);
+    ParsedDelete parsedDelete(opCtx, &request, collection.getCollectionPtr());
     uassertStatusOK(parsedDelete.parseRequest());
 
     auto exec = uassertStatusOK(getExecutorDelete(
-        &CurOp::get(opCtx)->debug(), &collection, &parsedDelete, boost::none /* verbosity */));
+        &CurOp::get(opCtx)->debug(), collection, &parsedDelete, boost::none /* verbosity */));
 
     if (!request.getReturnDeleted()) {
         return {exec->executeDelete(), boost::none};
