@@ -26,29 +26,37 @@
  *    exception statement from all source files in the program, then also delete
  *    it in the license file.
  */
-#include "mongo/platform/basic.h"
 
 #include "mongo/db/ops/insert.h"
 
+#include <boost/move/utility_core.hpp>
+#include <cstddef>
+#include <cstdint>
+#include <memory>
+#include <string>
 #include <vector>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/base/string_data.h"
 #include "mongo/bson/bson_depth.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/bson/util/builder.h"
 #include "mongo/db/catalog/document_validation.h"
-#include "mongo/db/commands/feature_compatibility_version_parser.h"
+#include "mongo/db/logical_time.h"
 #include "mongo/db/mongod_options_storage_gen.h"
-#include "mongo/db/query/dbref.h"
-#include "mongo/db/query/query_feature_flags_gen.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/repl/replication_coordinator.h"
+#include "mongo/db/server_options.h"
+#include "mongo/db/service_context.h"
 #include "mongo/db/update/storage_validation.h"
 #include "mongo/db/vector_clock_mutable.h"
-#include "mongo/db/views/durable_view_catalog.h"
-#include "mongo/util/fail_point.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/util/decorable.h"
 #include "mongo/util/str.h"
 
 namespace mongo {
-
-using std::string;
-
 namespace {
 
 /**
@@ -121,21 +129,12 @@ StatusWith<BSONObj> fixDocumentForInsert(OperationContext* opCtx,
             BSONElement e = i.next();
 
             auto fieldName = e.fieldNameStringData();
-
-            if (fieldName[0] == '$') {
-                if (!serverGlobalParams.featureCompatibility.isVersionInitialized() ||
-                    !serverGlobalParams.featureCompatibility.isGreaterThanOrEqualTo(
-                        multiversion::FeatureCompatibilityVersion::kFullyDowngradedTo_5_0)) {
-                    return StatusWith<BSONObj>(ErrorCodes::BadValue,
-                                               str::stream()
-                                                   << "Document can't have $ prefixed field names: "
-                                                   << fieldName);
-                } else if (containsDotsAndDollarsField) {
-                    *containsDotsAndDollarsField = true;
-                    // If the internal validation is disabled and we confirm this doc contains
-                    // dots/dollars field name, we can skip other validations below.
-                    if (validationDisabled)
-                        return StatusWith<BSONObj>(BSONObj());
+            if (fieldName.starts_with('$') && containsDotsAndDollarsField) {
+                *containsDotsAndDollarsField = true;
+                // If the internal validation is disabled and we confirm this doc contains
+                // dots/dollars field name, we can skip other validations below.
+                if (validationDisabled) {
+                    return StatusWith<BSONObj>(BSONObj());
                 }
             }
 
@@ -209,8 +208,9 @@ Status userAllowedWriteNS(OperationContext* opCtx, const NamespaceString& ns) {
 
     if (ns.isSystemDotProfile() || ns.isSystemDotViews() ||
         (ns.isOplog() &&
-         repl::ReplicationCoordinator::get(getGlobalServiceContext())->isReplEnabled())) {
-        return Status(ErrorCodes::InvalidNamespace, str::stream() << "cannot write to " << ns);
+         repl::ReplicationCoordinator::get(getGlobalServiceContext())->getSettings().isReplSet())) {
+        return Status(ErrorCodes::InvalidNamespace,
+                      str::stream() << "cannot write to " << ns.toStringForErrorMsg());
     }
     return userAllowedCreateNS(opCtx, ns);
 }
@@ -222,8 +222,9 @@ Status userAllowedCreateNS(OperationContext* opCtx, const NamespaceString& ns) {
         return Status::OK();
     }
 
-    if (!ns.isValid(NamespaceString::DollarInDbNameBehavior::Disallow)) {
-        return Status(ErrorCodes::InvalidNamespace, str::stream() << "Invalid namespace: " << ns);
+    if (!ns.isValid(DatabaseName::DollarInDbNameBehavior::Disallow)) {
+        return Status(ErrorCodes::InvalidNamespace,
+                      str::stream() << "Invalid namespace: " << ns.toStringForErrorMsg());
     }
 
     if (!NamespaceString::validCollectionName(ns.coll())) {
@@ -231,25 +232,21 @@ Status userAllowedCreateNS(OperationContext* opCtx, const NamespaceString& ns) {
                       str::stream() << "Invalid collection name: " << ns.coll());
     }
 
-    if (serverGlobalParams.clusterRole == ClusterRole::ConfigServer && !ns.isOnInternalDb()) {
-        return Status(ErrorCodes::InvalidNamespace,
-                      str::stream()
-                          << "Can't create user databases on a --configsvr instance " << ns);
-    }
-
     if (ns.isSystemDotProfile()) {
         return Status::OK();
     }
 
-    if (ns.isSystem() && !ns.isLegalClientSystemNS(serverGlobalParams.featureCompatibility)) {
+    if (ns.isSystem() && !ns.isLegalClientSystemNS()) {
         return Status(ErrorCodes::InvalidNamespace,
-                      str::stream() << "Invalid system namespace: " << ns);
+                      str::stream() << "Invalid system namespace: " << ns.toStringForErrorMsg());
     }
 
-    if (ns.isNormalCollection() && ns.size() > NamespaceString::MaxNsCollectionLen) {
+    auto maxNsLen = ns.isOnInternalDb() ? NamespaceString::MaxInternalNsCollectionLen
+                                        : NamespaceString::MaxUserNsCollectionLen;
+    if (ns.isNormalCollection() && ns.size() > maxNsLen) {
         return Status(ErrorCodes::InvalidNamespace,
-                      str::stream() << "Fully qualified namespace is too long. Namespace: " << ns
-                                    << " Max: " << NamespaceString::MaxNsCollectionLen);
+                      str::stream() << "Fully qualified namespace is too long. Namespace: "
+                                    << ns.toStringForErrorMsg() << " Max: " << maxNsLen);
     }
 
     if (ns.coll().find(".system.") != std::string::npos) {
@@ -263,11 +260,12 @@ Status userAllowedCreateNS(OperationContext* opCtx, const NamespaceString& ns) {
             return Status::OK();
         }
 
-        if (ns.isConfigDB() && ns.isLegalClientSystemNS(serverGlobalParams.featureCompatibility)) {
+        if (ns.isConfigDB() && ns.isLegalClientSystemNS()) {
             return Status::OK();
         }
 
-        return Status(ErrorCodes::BadValue, str::stream() << "Invalid namespace: " << ns);
+        return Status(ErrorCodes::BadValue,
+                      str::stream() << "Invalid namespace: " << ns.toStringForErrorMsg());
     }
 
     return Status::OK();
