@@ -28,16 +28,18 @@
  */
 
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kWrite
-
-#include "mongo/platform/basic.h"
-
 #include "mongo/db/ops/update_result.h"
-
+#include "mongo/bson/bsonelement.h"
 #include "mongo/db/not_primary_error_tracker.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/redaction.h"
 #include "mongo/util/str.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kWrite
+
+
 namespace mongo {
 
 UpdateResult::UpdateResult(bool existing,
@@ -57,8 +59,6 @@ UpdateResult::UpdateResult(bool existing,
     }
     LOGV2_DEBUG(20885,
                 4,
-                "UpdateResult -- upserted: {upserted} modifiers: {modifiers} existing: {existing} "
-                "numDocsModified: {numModified} numMatched: {numMatched}",
                 "UpdateResult",
                 "numMatched"_attr = numMatched,
                 "numModified"_attr = numDocsModified,
