@@ -29,14 +29,41 @@
 
 #include "mongo/db/ops/write_ops.h"
 
+#include <algorithm>
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <memory>
+#include <utility>
+#include <variant>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/error_extra_info.h"
+#include "mongo/base/status.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/bson/oid.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/bson/util/builder.h"
+#include "mongo/crypto/fle_field_schema_gen.h"
+#include "mongo/db/basic_types.h"
+#include "mongo/db/commands/bulk_write_gen.h"
 #include "mongo/db/dbmessage.h"
+#include "mongo/db/namespace_string.h"
 #include "mongo/db/pipeline/aggregation_request_helper.h"
+#include "mongo/db/pipeline/legacy_runtime_constants_gen.h"
+#include "mongo/db/repl/optime.h"
+#include "mongo/db/update/document_diff_serialization.h"
 #include "mongo/db/update/update_oplog_entry_serialization.h"
 #include "mongo/db/update/update_oplog_entry_version.h"
+#include "mongo/idl/idl_parser.h"
 #include "mongo/rpc/get_status_from_command_result.h"
 #include "mongo/util/assert_util.h"
-#include "mongo/util/overloaded_visitor.h"
+#include "mongo/util/overloaded_visitor.h"  // IWYU pragma: keep
 #include "mongo/util/str.h"
+#include "mongo/util/time_support.h"
 
 namespace mongo {
 
@@ -65,6 +92,9 @@ static constexpr int kBoolSize = 1;
 // for the UUID itself.
 static const int kUUIDSize = 21;
 
+// This constant accounts for the size of a 32-bit integer.
+static const int kIntSize = 4;
+
 template <class T>
 void checkOpCountForCommand(const T& op, size_t numOps) {
     uassert(ErrorCodes::InvalidLength,
@@ -103,15 +133,15 @@ int getWriteCommandRequestBaseSize(const WriteCommandRequestBase& base) {
 
     if (auto stmtId = base.getStmtId(); stmtId) {
         estSize += write_ops::WriteCommandRequestBase::kStmtIdFieldName.size() +
-            sizeof(std::int32_t) + kPerElementOverhead;
+            write_ops::kStmtIdSize + kPerElementOverhead;
     }
 
     if (auto stmtIds = base.getStmtIds(); stmtIds) {
         estSize += write_ops::WriteCommandRequestBase::kStmtIdsFieldName.size();
         estSize += static_cast<int>(BSONObj::kMinBSONLength);
         estSize +=
-            ((sizeof(std::int32_t) + write_ops::kWriteCommandBSONArrayPerElementOverheadBytes) *
-             stmtIds->size());
+            (write_ops::kStmtIdSize + write_ops::kWriteCommandBSONArrayPerElementOverheadBytes) *
+            stmtIds->size();
         estSize += kPerElementOverhead;
     }
 
@@ -180,7 +210,7 @@ void opTimeSerializerWithTermCheck(repl::OpTime opTime, StringData fieldName, BS
     if (opTime.getTerm() == repl::OpTime::kUninitializedTerm) {
         bob->append(fieldName, opTime.getTimestamp());
     } else {
-        opTime.append(bob, fieldName.toString());
+        opTime.append(fieldName, bob);
     }
 }
 
@@ -240,12 +270,25 @@ int estimateRuntimeConstantsSize(const mongo::LegacyRuntimeConstants& constants)
     return size;
 }
 
+int getArrayFiltersFieldSize(const std::vector<mongo::BSONObj>& arrayFilters,
+                             const StringData arrayFiltersFieldName) {
+    auto size = BSONObj::kMinBSONLength + arrayFiltersFieldName.size() + kPerElementOverhead;
+    for (auto&& filter : arrayFilters) {
+        // For each filter, we not only need to account for the size of the filter itself,
+        // but also for the per array element overhead.
+        size += filter.objsize();
+        size += write_ops::kWriteCommandBSONArrayPerElementOverheadBytes;
+    }
+    return size;
+}
+
 int getUpdateSizeEstimate(const BSONObj& q,
                           const write_ops::UpdateModification& u,
                           const boost::optional<mongo::BSONObj>& c,
                           const bool includeUpsertSupplied,
                           const boost::optional<mongo::BSONObj>& collation,
                           const boost::optional<std::vector<mongo::BSONObj>>& arrayFilters,
+                          const boost::optional<mongo::BSONObj>& sort,
                           const mongo::BSONObj& hint,
                           const boost::optional<UUID>& sampleId,
                           const bool includeAllowShardKeyUpdatesWithoutFullShardKeyInQuery) {
@@ -278,17 +321,13 @@ int getUpdateSizeEstimate(const BSONObj& q,
 
     // Add the size of the 'arrayFilters' field, if present.
     if (arrayFilters) {
-        estSize += ([&]() {
-            auto size = BSONObj::kMinBSONLength + UpdateOpEntry::kArrayFiltersFieldName.size() +
-                kPerElementOverhead;
-            for (auto&& filter : *arrayFilters) {
-                // For each filter, we not only need to account for the size of the filter itself,
-                // but also for the per array element overhead.
-                size += filter.objsize();
-                size += write_ops::kWriteCommandBSONArrayPerElementOverheadBytes;
-            }
-            return size;
-        })();
+        estSize +=
+            getArrayFiltersFieldSize(arrayFilters.get(), UpdateOpEntry::kArrayFiltersFieldName);
+    }
+
+    // Add the size of the 'sort' field, if present.
+    if (sort) {
+        estSize += UpdateOpEntry::kSortFieldName.size() + sort->objsize() + kPerElementOverhead;
     }
 
     // Add the size of the 'hint' field, if present.
@@ -310,13 +349,86 @@ int getUpdateSizeEstimate(const BSONObj& q,
     return estSize;
 }
 
+/*
+ * Helper function to calculate the estimated size of an update operation in a bulkWrite command.
+ *
+ * Note: This helper function doesn't currently account for the size needed for the internal field
+ * '_allowShardKeyUpdatesWithoutFullShardKeyInQuery' for updateOne without shard key. This should be
+ * safe for now because each update operation without shard key is always executed in its own child
+ * batch. See `targetWriteOps` for more details.
+ * (This needs to change once we start batching multiple updateOne operations without shard key in
+ * the same batch.)
+ *
+ *  TODO (SERVER-82382): Fix sampleId size calculation.
+ */
+int getBulkWriteUpdateSizeEstimate(const BSONObj& filter,
+                                   const write_ops::UpdateModification& updateMods,
+                                   const boost::optional<mongo::BSONObj>& constants,
+                                   const bool includeUpsertSupplied,
+                                   const boost::optional<mongo::BSONObj>& collation,
+                                   const boost::optional<std::vector<mongo::BSONObj>>& arrayFilters,
+                                   const boost::optional<mongo::BSONObj>& sort,
+                                   const BSONObj& hint,
+                                   const boost::optional<UUID>& sampleId) {
+    int estSize = static_cast<int>(BSONObj::kMinBSONLength);
+
+    // Adds the size of the 'update' field which contains the index of the corresponding namespace.
+    estSize += BulkWriteUpdateOp::kUpdateFieldName.size() + kIntSize + kPerElementOverhead;
+
+    // Add the sizes of the 'multi' and 'upsert' fields.
+    estSize += BulkWriteUpdateOp::kUpsertFieldName.size() + kBoolSize + kPerElementOverhead;
+    estSize += BulkWriteUpdateOp::kMultiFieldName.size() + kBoolSize + kPerElementOverhead;
+
+    // Add the size of 'upsertSupplied' field if present.
+    if (includeUpsertSupplied) {
+        estSize +=
+            BulkWriteUpdateOp::kUpsertSuppliedFieldName.size() + kBoolSize + kPerElementOverhead;
+    }
+
+    // Add the sizes of the 'filter' and 'updateMods' fields.
+    estSize += (BulkWriteUpdateOp::kFilterFieldName.size() + filter.objsize() +
+                kPerElementOverhead + BulkWriteUpdateOp::kUpdateModsFieldName.size() +
+                updateMods.objsize() + kPerElementOverhead);
+
+    // Add the size of the 'constants' field, if present.
+    if (constants) {
+        estSize += (BulkWriteUpdateOp::kConstantsFieldName.size() + constants->objsize() +
+                    kPerElementOverhead);
+    }
+
+    // Add the size of the 'collation' field, if present.
+    if (collation) {
+        estSize += (BulkWriteUpdateOp::kCollationFieldName.size() + collation->objsize() +
+                    kPerElementOverhead);
+    }
+
+    // Add the size of the 'arrayFilters' field, if present.
+    if (arrayFilters) {
+        estSize +=
+            getArrayFiltersFieldSize(arrayFilters.get(), BulkWriteUpdateOp::kArrayFiltersFieldName);
+    }
+
+    // Add the size of the 'sort' field, if present.
+    if (sort) {
+        estSize += BulkWriteUpdateOp::kSortFieldName.size() + sort->objsize() + kPerElementOverhead;
+    }
+
+    // Add the size of the 'hint' field, if present.
+    if (!hint.isEmpty()) {
+        estSize += BulkWriteUpdateOp::kHintFieldName.size() + hint.objsize() + kPerElementOverhead;
+    }
+
+    // Add the size of the 'sampleId' field.
+    estSize += BulkWriteUpdateOp::kSampleIdFieldName.size() + kUUIDSize + kPerElementOverhead;
+
+    return estSize;
+}
+
 int getDeleteSizeEstimate(const BSONObj& q,
                           const boost::optional<mongo::BSONObj>& collation,
                           const mongo::BSONObj& hint,
                           const boost::optional<UUID>& sampleId) {
     using DeleteOpEntry = write_ops::DeleteOpEntry;
-
-    static const int kIntSize = 4;
     int estSize = static_cast<int>(BSONObj::kMinBSONLength);
 
     // Add the size of the 'q' field.
@@ -344,6 +456,53 @@ int getDeleteSizeEstimate(const BSONObj& q,
     return estSize;
 }
 
+// TODO (SERVER-82382): Fix sampleId size calculation.
+int getBulkWriteDeleteSizeEstimate(const BSONObj& filter,
+                                   const boost::optional<mongo::BSONObj>& collation,
+                                   const mongo::BSONObj& hint,
+                                   const boost::optional<UUID>& sampleId) {
+    int estSize = static_cast<int>(BSONObj::kMinBSONLength);
+
+    // Adds the size of the 'delete' field which contains the index of the corresponding namespace.
+    estSize += BulkWriteDeleteOp::kDeleteCommandFieldName.size() + kIntSize + kPerElementOverhead;
+
+    // Add the size of the 'filter' field.
+    estSize += BulkWriteDeleteOp::kFilterFieldName.size() + filter.objsize() + kPerElementOverhead;
+
+    // Add the size of the 'multi' field.
+    estSize += BulkWriteDeleteOp::kMultiFieldName.size() + kBoolSize + kPerElementOverhead;
+
+    // Add the size of the 'collation' field, if present.
+    if (collation) {
+        estSize += BulkWriteDeleteOp::kCollationFieldName.size() + collation->objsize() +
+            kPerElementOverhead;
+    }
+
+    // Add the size of the 'hint' field, if present.
+    if (!hint.isEmpty()) {
+        estSize +=
+            (BulkWriteDeleteOp::kHintFieldName.size() + hint.objsize() + kPerElementOverhead);
+    }
+
+    // Add the size of the 'sampleId' field.
+    estSize += BulkWriteUpdateOp::kSampleIdFieldName.size() + kUUIDSize + kPerElementOverhead;
+
+    return estSize;
+}
+
+int getBulkWriteInsertSizeEstimate(const mongo::BSONObj& document) {
+    int estSize = static_cast<int>(BSONObj::kMinBSONLength);
+
+    // Adds the size of the 'insert' field which contains the index of the corresponding namespace.
+    estSize += BulkWriteInsertOp::kInsertFieldName.size() + kIntSize + kPerElementOverhead;
+
+    // Add the size of the 'document' field.
+    estSize +=
+        BulkWriteInsertOp::kDocumentFieldName.size() + document.objsize() + kPerElementOverhead;
+
+    return estSize;
+}
+
 bool verifySizeEstimate(const write_ops::UpdateOpEntry& update) {
     return write_ops::getUpdateSizeEstimate(
                update.getQ(),
@@ -352,6 +511,7 @@ bool verifySizeEstimate(const write_ops::UpdateOpEntry& update) {
                update.getUpsertSupplied().has_value(),
                update.getCollation(),
                update.getArrayFilters(),
+               update.getSort(),
                update.getHint(),
                update.getSampleId(),
                update.getAllowShardKeyUpdatesWithoutFullShardKeyInQuery().has_value()) >=
@@ -385,6 +545,7 @@ bool verifySizeEstimate(const UpdateCommandRequest& updateReq,
                     update.getUpsertSupplied().has_value(),
                     update.getCollation(),
                     update.getArrayFilters(),
+                    update.getSort(),
                     update.getHint(),
                     update.getSampleId(),
                     update.getAllowShardKeyUpdatesWithoutFullShardKeyInQuery().has_value()) +
@@ -426,13 +587,6 @@ int getInsertHeaderSizeEstimate(const InsertCommandRequest& insertReq) {
 
     size += InsertCommandRequest::kCommandName.size() + kPerElementOverhead +
         insertReq.getNamespace().size() + 1 /* ns string null terminator */;
-
-    // Handle $tenant. Note that $tenant is injected as a hidden field into all IDL commands, unlike
-    // other passthrough fields.
-    if (auto tenant = insertReq.getDollarTenant(); tenant.has_value()) {
-        size += InsertCommandRequest::kDollarTenantFieldName.size() + OID::kOIDSize +
-            kPerElementOverhead;
-    }
     return size;
 }
 
@@ -445,13 +599,6 @@ int getUpdateHeaderSizeEstimate(const UpdateCommandRequest& updateReq) {
     size += write_ops::UpdateCommandRequest::kUpdatesFieldName.size() + kPerElementOverhead +
         static_cast<int>(BSONObj::kMinBSONLength);
 
-    // Handle $tenant. Note that $tenant is injected as a hidden field into all IDL commands, unlike
-    // other passthrough fields.
-    if (auto tenant = updateReq.getDollarTenant(); tenant.has_value()) {
-        size += UpdateCommandRequest::kDollarTenantFieldName.size() + OID::kOIDSize +
-            kPerElementOverhead;
-    }
-
     // Handle legacy runtime constants.
     if (auto runtimeConstants = updateReq.getLegacyRuntimeConstants();
         runtimeConstants.has_value()) {
@@ -475,13 +622,6 @@ int getDeleteHeaderSizeEstimate(const DeleteCommandRequest& deleteReq) {
     size += write_ops::DeleteCommandRequest::kDeletesFieldName.size() + kPerElementOverhead +
         static_cast<int>(BSONObj::kMinBSONLength);
 
-    // Handle $tenant. Note that $tenant is injected as a hidden field into all IDL commands, unlike
-    // other passthrough fields.
-    if (auto tenant = deleteReq.getDollarTenant(); tenant.has_value()) {
-        size += DeleteCommandRequest::kDollarTenantFieldName.size() + OID::kOIDSize +
-            kPerElementOverhead;
-    }
-
     // Handle legacy runtime constants.
     if (auto runtimeConstants = deleteReq.getLegacyRuntimeConstants();
         runtimeConstants.has_value()) {
@@ -595,7 +735,7 @@ UpdateModification UpdateModification::parseFromBSON(BSONElement elem) {
 }
 
 int UpdateModification::objsize() const {
-    return stdx::visit(
+    return visit(
         OverloadedVisitor{
             [](const ReplacementUpdate& replacement) -> int { return replacement.bson.objsize(); },
             [](const ModifierUpdate& modifier) -> int { return modifier.bson.objsize(); },
@@ -615,7 +755,7 @@ int UpdateModification::objsize() const {
 }
 
 UpdateModification::Type UpdateModification::type() const {
-    return stdx::visit(
+    return visit(
         OverloadedVisitor{[](const ReplacementUpdate& replacement) { return Type::kReplacement; },
                           [](const ModifierUpdate& modifier) { return Type::kModifier; },
                           [](const PipelineUpdate& pipelineUpdate) { return Type::kPipeline; },
@@ -632,24 +772,24 @@ UpdateModification::Type UpdateModification::type() const {
  */
 void UpdateModification::serializeToBSON(StringData fieldName, BSONObjBuilder* bob) const {
 
-    stdx::visit(OverloadedVisitor{
-                    [fieldName, bob](const ReplacementUpdate& replacement) {
-                        *bob << fieldName << replacement.bson;
-                    },
-                    [fieldName, bob](const ModifierUpdate& modifier) {
-                        *bob << fieldName << modifier.bson;
-                    },
-                    [fieldName, bob](const PipelineUpdate& pipeline) {
-                        BSONArrayBuilder arrayBuilder(bob->subarrayStart(fieldName));
-                        for (auto&& stage : pipeline) {
-                            arrayBuilder << stage;
-                        }
-                        arrayBuilder.doneFast();
-                    },
-                    [fieldName, bob](const DeltaUpdate& delta) { *bob << fieldName << delta.diff; },
-                    [](const TransformUpdate& transform) {
-                    }},
-                _update);
+    visit(OverloadedVisitor{
+              [fieldName, bob](const ReplacementUpdate& replacement) {
+                  *bob << fieldName << replacement.bson;
+              },
+              [fieldName, bob](const ModifierUpdate& modifier) {
+                  *bob << fieldName << modifier.bson;
+              },
+              [fieldName, bob](const PipelineUpdate& pipeline) {
+                  BSONArrayBuilder arrayBuilder(bob->subarrayStart(fieldName));
+                  for (auto&& stage : pipeline) {
+                      arrayBuilder << stage;
+                  }
+                  arrayBuilder.doneFast();
+              },
+              [fieldName, bob](const DeltaUpdate& delta) { *bob << fieldName << delta.diff; },
+              [](const TransformUpdate& transform) {
+              }},
+          _update);
 }
 
 WriteError::WriteError(int32_t index, Status status) : _index(index), _status(std::move(status)) {}
@@ -691,7 +831,9 @@ InsertCommandRequest InsertOp::parse(const OpMsgRequest& request) {
 InsertCommandRequest InsertOp::parseLegacy(const Message& msgRaw) {
     DbMessage msg(msgRaw);
 
-    InsertCommandRequest op(NamespaceString(msg.getns()));
+    // Passing boost::none since this is legacy code and should not be running in serverless.
+    InsertCommandRequest op(NamespaceStringUtil::deserialize(
+        boost::none, msg.getns(), SerializationContext::stateDefault()));
 
     {
         WriteCommandRequestBase writeCommandBase;
