@@ -29,26 +29,45 @@
 
 #pragma once
 
+#include <boost/move/utility_core.hpp>
 #include <boost/optional.hpp>
+#include <boost/optional/optional.hpp>
+#include <cstddef>
+#include <memory>
 #include <vector>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
 #include "mongo/base/status_with.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/crypto/fle_field_schema_gen.h"
 #include "mongo/db/catalog/collection_operation_source.h"
+#include "mongo/db/namespace_string.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/ops/single_write_result_gen.h"
 #include "mongo/db/ops/update_result.h"
 #include "mongo/db/ops/write_ops.h"
 #include "mongo/db/ops/write_ops_exec_util.h"
+#include "mongo/db/ops/write_ops_gen.h"
+#include "mongo/db/ops/write_ops_parsers.h"
+#include "mongo/db/query/query_settings/query_settings_gen.h"
 #include "mongo/db/repl/oplog.h"
 #include "mongo/db/repl/repl_client_info.h"
+#include "mongo/db/session/logical_session_id.h"
+#include "mongo/db/tenant_id.h"
+#include "mongo/executor/task_executor.h"
 #include "mongo/s/stale_exception.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/uuid.h"
 
 namespace mongo {
 
+class CanonicalQuery;
+class CurOp;
 class DeleteRequest;
 class OpDebug;
-class ParsedUpdate;
 class PlanExecutor;
+class UpdateRequest;
 
 namespace write_ops_exec {
 
@@ -81,7 +100,8 @@ bool handleError(OperationContext* opCtx,
                  WriteResult* out);
 
 bool getFleCrudProcessed(OperationContext* opCtx,
-                         const boost::optional<EncryptionInformation>& encryptionInfo);
+                         const boost::optional<EncryptionInformation>& encryptionInfo,
+                         const boost::optional<TenantId>& tenantId);
 
 /**
  * Returns true if caller should try to insert more documents. Does nothing else if batch is empty.
@@ -91,43 +111,47 @@ bool insertBatchAndHandleErrors(OperationContext* opCtx,
                                 const boost::optional<mongo::UUID>& collectionUUID,
                                 bool ordered,
                                 std::vector<InsertStatement>& batch,
+                                OperationSource source,
                                 LastOpFixer* lastOpFixer,
-                                WriteResult* out,
-                                OperationSource source);
+                                WriteResult* out);
 
 /**
  * If the operation succeeded, then returns either a document to return to the client, or
  * boost::none if no matching document to update/remove was found. If the operation failed, throws.
+ * Accepts the name of the operation (e.g. "delete", "update", "findAndModify") for use in the
+ * exception and log messages.
  */
 boost::optional<BSONObj> advanceExecutor(OperationContext* opCtx,
                                          PlanExecutor* exec,
-                                         bool isRemove);
+                                         bool isRemove,
+                                         StringData operationName);
 
 /**
- * Executes a findAndModify with remove:false, the returned document is placed into docFound (if
- * applicable). Should be called in a writeConflictRetry loop.
+ * Executes an update, supports returning a pre/post image. The returned document is placed into
+ * docFound (if applicable). Should be called in a writeConflictRetry loop.
  */
-UpdateResult writeConflictRetryUpsert(OperationContext* opCtx,
-                                      const NamespaceString& nsString,
-                                      CurOp* curOp,
-                                      OpDebug* opDebug,
-                                      bool inTransaction,
-                                      bool remove,
-                                      bool upsert,
-                                      boost::optional<BSONObj>& docFound,
-                                      ParsedUpdate* parsedUpdate);
+UpdateResult performUpdate(OperationContext* opCtx,
+                           const NamespaceString& nss,
+                           CurOp* curOp,
+                           bool inTransaction,
+                           bool remove,
+                           bool upsert,
+                           const boost::optional<mongo::UUID>& collectionUUID,
+                           boost::optional<BSONObj>& docFound,
+                           UpdateRequest* updateRequest);
 
 /**
- * Executes a findAndModify with remove:true, the returned document is placed into docFound (if
- * applicable). Should be called in a writeConflictRetry loop.
+ * Executes a delete, supports returning the deleted document. the returned document is placed into
+ * docFound (if applicable). Should be called in a writeConflictRetry loop. Returns the number of
+ * documents deleted as a long long to conform to OpDebug interface.
  */
-long long writeConflictRetryRemove(OperationContext* opCtx,
-                                   const NamespaceString& nsString,
-                                   DeleteRequest* deleteRequest,
-                                   CurOp* curOp,
-                                   OpDebug* opDebug,
-                                   bool inTransaction,
-                                   boost::optional<BSONObj>& docFound);
+long long performDelete(OperationContext* opCtx,
+                        const NamespaceString& nss,
+                        DeleteRequest* deleteRequest,
+                        CurOp* curOp,
+                        bool inTransaction,
+                        const boost::optional<mongo::UUID>& collectionUUID,
+                        boost::optional<BSONObj>& docFound);
 
 /**
  * Generates a WriteError for a given Status.
@@ -150,6 +174,16 @@ boost::optional<write_ops::WriteError> generateErrorNoTenantMigration(OperationC
                                                                       int index,
                                                                       size_t numErrors) noexcept;
 
+/**
+ * Updates the retryable write stats if the write op contains retry.
+ */
+void updateRetryStats(OperationContext* opCtx, bool containsRetry);
+
+/**
+ * Marks the op as complete, log it and profile if appropriate.
+ */
+void logOperationAndProfileIfNeeded(OperationContext* opCtx, CurOp* curOp);
+
 /**
  * Performs a batch of inserts, updates, or deletes.
  *
@@ -182,6 +216,18 @@ Status performAtomicTimeseriesWrites(OperationContext* opCtx,
                                      const std::vector<write_ops::InsertCommandRequest>& insertOps,
                                      const std::vector<write_ops::UpdateCommandRequest>& updateOps);
 
+/**
+ * Runs a time-series update command in a transaction and collects the write result from each
+ * statement.
+ *
+ * Assumes the update command is a retryable write and targeted on the time-series view namespace.
+ */
+void runTimeseriesRetryableUpdates(OperationContext* opCtx,
+                                   const NamespaceString& bucketNs,
+                                   const write_ops::UpdateCommandRequest& wholeOp,
+                                   std::shared_ptr<executor::TaskExecutor> executor,
+                                   write_ops_exec::WriteResult* reply);
+
 /**
  * Populate 'opDebug' with stats describing the execution of an update operation. Illegal to call
  * with a null OpDebug pointer.
@@ -190,12 +236,10 @@ void recordUpdateResultInOpDebug(const UpdateResult& updateResult, OpDebug* opDe
 
 /**
  * Returns true if an update failure due to a given DuplicateKey error is eligible for retry.
- * Requires that parsedUpdate.hasParsedQuery() is true.
  */
-bool shouldRetryDuplicateKeyException(OperationContext* opCtx,
-                                      const ParsedUpdate& parsedUpdate,
-                                      const DuplicateKeyErrorInfo& errorInfo,
-                                      int retryAttempts);
+bool shouldRetryDuplicateKeyException(const UpdateRequest& updateRequest,
+                                      const CanonicalQuery& cq,
+                                      const DuplicateKeyErrorInfo& errorInfo);
 
 /**
  * Returns an InsertCommandReply if the timeseries writes succeeded.
@@ -203,5 +247,27 @@ bool shouldRetryDuplicateKeyException(OperationContext* opCtx,
 write_ops::InsertCommandReply performTimeseriesWrites(
     OperationContext* opCtx, const write_ops::InsertCommandRequest& request);
 
+write_ops::InsertCommandReply performTimeseriesWrites(
+    OperationContext* opCtx, const write_ops::InsertCommandRequest& request, CurOp* curOp);
+
+/*
+ * Populates 'result' with the explain information for the write requests.
+ */
+void explainUpdate(OperationContext* opCtx,
+                   UpdateRequest& updateRequest,
+                   bool isTimeseriesViewRequest,
+                   const SerializationContext& serializationContext,
+                   const BSONObj& command,
+                   ExplainOptions::Verbosity verbosity,
+                   rpc::ReplyBuilderInterface* result);
+
+void explainDelete(OperationContext* opCtx,
+                   DeleteRequest& deleteRequest,
+                   bool isTimeseriesViewRequest,
+                   const SerializationContext& serializationContext,
+                   const BSONObj& command,
+                   ExplainOptions::Verbosity verbosity,
+                   rpc::ReplyBuilderInterface* result);
+
 }  // namespace write_ops_exec
 }  // namespace mongo
