@@ -29,11 +29,19 @@
 
 #pragma once
 
+#include <absl/container/node_hash_map.h>
+#include <cstddef>
 #include <stack>
+#include <utility>
+#include <vector>
 
+#include "mongo/db/matcher/expression.h"
 #include "mongo/db/matcher/expression_path.h"
-#include "mongo/db/query/optimizer/node.h"
+#include "mongo/db/query/optimizer/defs.h"
+#include "mongo/db/query/optimizer/node.h"  // IWYU pragma: keep
+#include "mongo/db/query/optimizer/syntax/syntax.h"
 #include "mongo/db/query/optimizer/utils/utils.h"
+#include "mongo/util/assert_util.h"
 
 namespace mongo::optimizer {
 
@@ -42,7 +50,8 @@ public:
     ExpressionAlgebrizerContext(bool assertExprSort,
                                 bool assertPathSort,
                                 const ProjectionName& rootProjection,
-                                PrefixId& prefixId);
+                                PrefixId& prefixId,
+                                QueryParameterMap& queryParameters);
 
     /**
      * Push an ABT onto the stack. Optionally perform a check on the type of the ABT based on
@@ -70,6 +79,8 @@ public:
 
     PrefixId& getPrefixId();
 
+    QueryParameterMap& getQueryParameters();
+
     /**
      * Returns a unique projection. It will be prefixed by 'uniqueIdPrefix'.
      */
@@ -130,6 +141,8 @@ private:
     // Used to track expressions contained under an $elemMatch. Each entry is either an
     // ELEM_MATCH_OBJECT or ELEM_MATCH_VALUE.
     std::vector<MatchExpression::MatchType> _elemMatchStack;
+
+    QueryParameterMap& _queryParameters;
 };
 
 }  // namespace mongo::optimizer
