@@ -28,7 +28,21 @@
  */
 
 #include "mongo/db/pipeline/abt/match_expression_visitor.h"
+
+#include <cstddef>
+#include <utility>
+#include <vector>
+
+#include <absl/container/node_hash_map.h>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/db/exec/document_value/value.h"
 #include "mongo/db/exec/docval_to_sbeval.h"
+#include "mongo/db/exec/sbe/values/value.h"
+#include "mongo/db/field_ref.h"
 #include "mongo/db/matcher/expression_always_boolean.h"
 #include "mongo/db/matcher/expression_array.h"
 #include "mongo/db/matcher/expression_expr.h"
@@ -37,6 +51,7 @@
 #include "mongo/db/matcher/expression_internal_eq_hashed_key.h"
 #include "mongo/db/matcher/expression_internal_expr_comparison.h"
 #include "mongo/db/matcher/expression_leaf.h"
+#include "mongo/db/matcher/expression_path.h"
 #include "mongo/db/matcher/expression_text.h"
 #include "mongo/db/matcher/expression_text_noop.h"
 #include "mongo/db/matcher/expression_tree.h"
@@ -45,6 +60,7 @@
 #include "mongo/db/matcher/expression_where.h"
 #include "mongo/db/matcher/expression_where_noop.h"
 #include "mongo/db/matcher/match_expression_walker.h"
+#include "mongo/db/matcher/matcher_type_set.h"
 #include "mongo/db/matcher/schema/expression_internal_schema_all_elem_match_from_index.h"
 #include "mongo/db/matcher/schema/expression_internal_schema_allowed_properties.h"
 #include "mongo/db/matcher/schema/expression_internal_schema_cond.h"
@@ -64,9 +80,32 @@
 #include "mongo/db/pipeline/abt/agg_expression_visitor.h"
 #include "mongo/db/pipeline/abt/expr_algebrizer_context.h"
 #include "mongo/db/pipeline/abt/utils.h"
+#include "mongo/db/query/optimizer/algebra/polyvalue.h"
+#include "mongo/db/query/optimizer/comparison_op.h"
+#include "mongo/db/query/optimizer/syntax/expr.h"
+#include "mongo/db/query/optimizer/syntax/path.h"
 #include "mongo/db/query/optimizer/utils/path_utils.h"
+#include "mongo/db/query/tree_walker.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/str.h"
 
 namespace mongo::optimizer {
+namespace {
+/**
+ * Return the minimum or maximum value for the "class" of values represented by the input
+ * constant. Used to support type bracketing. Takes into account both the type tag and value of
+ * the input constant.
+ * Return format is <min/max value, bool inclusive>
+ */
+std::pair<boost::optional<ABT>, bool> getMinMaxBoundForValue(const bool isMin,
+                                                             const sbe::value::TypeTags& tag,
+                                                             const sbe::value::Value& val) {
+    if (sbe::value::isNaN(tag, val)) {
+        return {Constant::fromDouble(std::numeric_limits<double>::quiet_NaN()), true};
+    }
+    return getMinMaxBoundForType(isMin, tag);
+}
+}  // namespace
 
 class ABTMatchExpressionPreVisitor : public SelectiveMatchExpressionVisitorBase<true> {
     using SelectiveMatchExpressionVisitorBase<true>::visit;
@@ -192,7 +231,6 @@ public:
         ABT result = make<PathIdentity>();
 
         const auto [tagTraverse, valTraverse] = sbe::value::makeNewArray();
-        sbe::value::ValueGuard arrGuard{tagTraverse, valTraverse};
         auto arrTraversePtr = sbe::value::getArrayView(valTraverse);
         arrTraversePtr->reserve(equalities.size());
 
@@ -214,16 +252,18 @@ public:
             }
         }
 
-        // If there is only one term in the match expression, we don't need to use EqMember
-        if (arrTraversePtr->size() == 1) {
-            const auto [tagSingle, valSingle] = sbe::value::copyValue(
-                arrTraversePtr->getAt(0).first, arrTraversePtr->getAt(0).second);
-            result = make<PathCompare>(Operations::Eq, make<Constant>(tagSingle, valSingle));
+        if (expr->getInputParamId()) {
+            result = make<FunctionCall>(
+                kParameterFunctionName,
+                makeSeq(make<Constant>(sbe::value::TypeTags::NumberInt32, *expr->getInputParamId()),
+                        make<Constant>(sbe::value::TypeTags::NumberInt32,
+                                       static_cast<int>(tagTraverse))));
+            _ctx.getQueryParameters().emplace(*expr->getInputParamId(),
+                                              Constant(tagTraverse, valTraverse));
         } else {
-            result =
-                make<PathCompare>(Operations::EqMember, make<Constant>(tagTraverse, valTraverse));
-            arrGuard.reset();
+            result = make<Constant>(tagTraverse, valTraverse);
         }
+        result = make<PathCompare>(Operations::EqMember, std::move(result));
 
         if (addNullPathDefault) {
             maybeComposePath<PathComposeA>(result, make<PathDefault>(Constant::boolean(true)));
@@ -425,13 +465,28 @@ public:
         assertSupportedPathExpression(expr);
 
         const ProjectionName lambdaProjName{_ctx.getNextId("lambda_sizeMatch")};
-        ABT result = make<PathLambda>(make<LambdaAbstraction>(
+        auto result = [&]() {
+            if (expr->getInputParamId()) {
+                _ctx.getQueryParameters().emplace(
+                    *expr->getInputParamId(),
+                    Constant(sbe::value::TypeTags::NumberInt64,
+                             sbe::value::bitcastFrom<int64_t>(*expr->getInputParamId())));
+                return make<FunctionCall>(
+                    kParameterFunctionName,
+                    makeSeq(
+                        make<Constant>(sbe::value::TypeTags::NumberInt32, *expr->getInputParamId()),
+                        make<Constant>(sbe::value::TypeTags::NumberInt32,
+                                       static_cast<int>(sbe::value::TypeTags::NumberInt32))));
+            } else {
+                return Constant::int64(expr->getData());
+            }
+        }();
+        result = make<PathLambda>(make<LambdaAbstraction>(
             lambdaProjName,
             make<BinaryOp>(
                 Operations::Eq,
                 make<FunctionCall>("getArraySize", makeSeq(make<Variable>(lambdaProjName))),
-                Constant::int64(expr->getData()))));
-
+                std::move(result))));
         if (shouldGeneratePath(expr)) {
             result = translateFieldRef(*(expr->fieldRef()), std::move(result));
         }
@@ -527,7 +582,17 @@ private:
         assertSupportedPathExpression(expr);
 
         auto [tag, val] = sbe::value::makeValue(Value(expr->getData()));
-        ABT result = make<PathCompare>(op, make<Constant>(tag, val));
+        auto result = ABT{make<PathIdentity>()};
+        if (expr->getInputParamId()) {
+            result = make<FunctionCall>(
+                kParameterFunctionName,
+                makeSeq(make<Constant>(sbe::value::TypeTags::NumberInt32, *expr->getInputParamId()),
+                        make<Constant>(sbe::value::TypeTags::NumberInt32, static_cast<int>(tag))));
+            _ctx.getQueryParameters().emplace(*expr->getInputParamId(), Constant(tag, val));
+        } else {
+            result = make<Constant>(tag, val);
+        }
+        result = make<PathCompare>(op, std::move(result));
 
         bool tagNullMatchMissingField =
             tag == sbe::value::TypeTags::Null && (op == Operations::Lte || op == Operations::Gte);
@@ -535,7 +600,7 @@ private:
         switch (op) {
             case Operations::Lt:
             case Operations::Lte: {
-                auto&& [constant, inclusive] = getMinMaxBoundForType(true /*isMin*/, tag);
+                auto&& [constant, inclusive] = getMinMaxBoundForValue(true /*isMin*/, tag, val);
                 if (constant) {
                     maybeComposePath(result,
                                      make<PathCompare>(inclusive ? Operations::Gte : Operations::Gt,
@@ -553,7 +618,7 @@ private:
 
             case Operations::Gt:
             case Operations::Gte: {
-                auto&& [constant, inclusive] = getMinMaxBoundForType(false /*isMin*/, tag);
+                auto&& [constant, inclusive] = getMinMaxBoundForValue(false /*isMin*/, tag, val);
                 if (constant) {
                     maybeComposePath(result,
                                      make<PathCompare>(inclusive ? Operations::Lte : Operations::Lt,
@@ -581,8 +646,8 @@ private:
 
             default:
                 tasserted(7021701,
-                          str::stream() << "Cannot generate comparison for operation: "
-                                        << OperationsEnum::toString[static_cast<int>(op)]);
+                          str::stream()
+                              << "Cannot generate comparison for operation: " << toStringData(op));
         }
 
         if (shouldGeneratePath(expr)) {
@@ -665,9 +730,13 @@ private:
 ABT generateMatchExpression(const MatchExpression* expr,
                             const bool allowAggExpressions,
                             const ProjectionName& rootProjection,
-                            PrefixId& prefixId) {
-    ExpressionAlgebrizerContext ctx(
-        false /*assertExprSort*/, true /*assertPathSort*/, rootProjection, prefixId);
+                            PrefixId& prefixId,
+                            QueryParameterMap& queryParameters) {
+    ExpressionAlgebrizerContext ctx(false /*assertExprSort*/,
+                                    true /*assertPathSort*/,
+                                    rootProjection,
+                                    prefixId,
+                                    queryParameters);
     ABTMatchExpressionPreVisitor preVisitor(ctx);
     ABTMatchExpressionVisitor postVisitor(ctx, allowAggExpressions);
     MatchExpressionWalker walker(&preVisitor, nullptr /*inVisitor*/, &postVisitor);
