@@ -30,7 +30,9 @@
 #pragma once
 
 #include "mongo/db/matcher/expression.h"
-#include "mongo/db/query/optimizer/node.h"
+#include "mongo/db/query/optimizer/defs.h"
+#include "mongo/db/query/optimizer/node.h"  // IWYU pragma: keep
+#include "mongo/db/query/optimizer/syntax/syntax.h"
 #include "mongo/db/query/optimizer/utils/utils.h"
 
 
@@ -42,6 +44,7 @@ namespace mongo::optimizer {
 ABT generateMatchExpression(const MatchExpression* expr,
                             bool allowAggExpressions,
                             const ProjectionName& rootProjection,
-                            PrefixId& prefixId);
+                            PrefixId& prefixId,
+                            QueryParameterMap& queryParameters);
 
 }  // namespace mongo::optimizer
