@@ -29,25 +29,31 @@
 
 #pragma once
 
-#include "mongo/base/string_data.h"
-#include "mongo/platform/basic.h"
-
 #include <boost/intrusive_ptr.hpp>
+#include <boost/none.hpp>
 #include <boost/optional.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
 #include <functional>
 #include <vector>
 
-#include "mongo/base/init.h"
+#include "mongo/base/init.h"  // IWYU pragma: keep
+#include "mongo/base/string_data.h"
 #include "mongo/bson/bsontypes.h"
+#include "mongo/db/exec/document_value/document.h"
 #include "mongo/db/exec/document_value/value.h"
 #include "mongo/db/exec/document_value/value_comparator.h"
 #include "mongo/db/pipeline/expression.h"
 #include "mongo/db/pipeline/expression_context.h"
-#include "mongo/db/pipeline/memory_usage_tracker.h"
 #include "mongo/db/query/query_shape/serialization_options.h"
 #include "mongo/db/query/stats/stats_gen.h"
 #include "mongo/db/query/stats/value_utils.h"
+#include "mongo/platform/basic.h"
+#include "mongo/platform/decimal128.h"
 #include "mongo/stdx/unordered_set.h"
+#include "mongo/util/assert_util_core.h"
+#include "mongo/util/intrusive_counter.h"
+#include "mongo/util/memory_usage_tracker.h"
 #include "mongo/util/summation.h"
 
 namespace mongo {
@@ -61,11 +67,19 @@ enum class AccumulatorDocumentsNeeded {
 
     // AccumulatorState only needs to see one document in a group, and when there is a sort order,
     // that document must be the first document.
-    kFirstDocument,
+    kFirstInputDocument,
 
     // AccumulatorState only needs to see one document in a group, and when there is a sort order,
     // that document must be the last document.
-    kLastDocument,
+    kLastInputDocument,
+
+    // AccumulatorState may only need to see the first document in a group if there is an index that
+    // matches the sort order.
+    kFirstOutputDocument,
+
+    // AccumulatorState may only need to see the last document in a group if there is an index that
+    // matches the sort order.
+    kLastOutputDocument,
 };
 
 class AccumulatorState : public RefCountable {
@@ -156,18 +170,6 @@ protected:
     /// Update subclass's internal state based on input
     virtual void processInternal(const Value& input, bool merging) = 0;
 
-    /**
-     * When accumulated values are merged, a different type of the input is expected compared to a
-     * normal accumulating pass.
-     */
-    MONGO_COMPILER_ALWAYS_INLINE void assertMergingInputType(const Value& input,
-                                                             BSONType requiredType) const {
-        uassert(9961600,
-                str::stream() << "Unexpected type on the merging pass of the " << getOpName()
-                              << ". This likely happened due to a malformed query.",
-                input.getType() == requiredType);
-    }
-
     auto getExpressionContext() const {
         return _expCtx;
     }
@@ -215,7 +217,7 @@ public:
     }
 
 private:
-    ValueUnorderedSet _set;
+    ValueFlatUnorderedSet _set;
 };
 
 class AccumulatorFirst final : public AccumulatorState {
@@ -235,7 +237,7 @@ public:
     static boost::intrusive_ptr<AccumulatorState> create(ExpressionContext* expCtx);
 
     AccumulatorDocumentsNeeded documentsNeeded() const final {
-        return AccumulatorDocumentsNeeded::kFirstDocument;
+        return AccumulatorDocumentsNeeded::kFirstInputDocument;
     }
 
 private:
@@ -288,7 +290,7 @@ public:
     static boost::intrusive_ptr<AccumulatorState> create(ExpressionContext* expCtx);
 
     AccumulatorDocumentsNeeded documentsNeeded() const final {
-        return AccumulatorDocumentsNeeded::kLastDocument;
+        return AccumulatorDocumentsNeeded::kLastInputDocument;
     }
 
 private:
@@ -299,17 +301,33 @@ class AccumulatorSum final : public AccumulatorState {
 public:
     static constexpr auto kName = "$sum"_sd;
 
+    /**
+     * These aliases represent two possible sum states in AcculumatorSum:
+     *  - ConstantSumState, which is used in the cases of sums over non-decimal constants such as
+     *    {$sum: 1}. It stores the current sum as a running total.
+     *  - NonConstantSumState which is used in all other cases. It stores the current sum using a
+     *    DoubleDoubleSummation and a DecimalTotal.
+     */
+    using NonConstantSumState = std::pair<DoubleDoubleSummation, Decimal128>;
+    using ConstantSumState = std::variant<int, long long, double>;
+
+
+    static boost::optional<Value> getConstantArgument(boost::intrusive_ptr<Expression> arg);
+
     const char* getOpName() const final {
         return kName.rawData();
     }
 
     explicit AccumulatorSum(ExpressionContext* expCtx);
+    explicit AccumulatorSum(ExpressionContext* expCtx, boost::optional<Value> constantAddend);
 
     void processInternal(const Value& input, bool merging) final;
     Value getValue(bool toBeMerged) final;
     void reset() final;
 
     static boost::intrusive_ptr<AccumulatorState> create(ExpressionContext* expCtx);
+    static boost::intrusive_ptr<AccumulatorState> create(ExpressionContext* expCtx,
+                                                         boost::optional<Value> constantAddend);
 
     ExpressionNary::Associativity getAssociativity() const final {
         return ExpressionNary::Associativity::kFull;
@@ -320,10 +338,31 @@ public:
     }
 
 private:
+    /**
+     * Helper function that converts this accumulator from tracking a constant sum to a non constant
+     * one.
+     */
+    DoubleDoubleSummation _constantSumToDoubleDoubleSummation();
+
+    /**
+     * Helper function that initializes this accumulator to track a constant sum.
+     */
+    void _initConstant(const BSONType& type);
+
+    /**
+     * Helper functions which implement the behavior for processing the desired sum type.
+     */
+    void _processInternalConstant(const Value& input,
+                                  AccumulatorSum::ConstantSumState& constantTotal);
+    void _processInternalNonConstant(const Value& input,
+                                     AccumulatorSum::NonConstantSumState& nonConstantTotal);
+
+    // Tracks the original constant addend argument.
+    boost::optional<Value> constantAddend = boost::none;
     BSONType totalType = NumberInt;
     BSONType nonDecimalTotalType = NumberInt;
-    DoubleDoubleSummation nonDecimalTotal;
-    Decimal128 decimalTotal;
+    std::variant<NonConstantSumState, ConstantSumState> sum =
+        std::make_pair<>(DoubleDoubleSummation(), Decimal128());
 };
 
 class AccumulatorMinMax : public AccumulatorState {
