@@ -27,12 +27,27 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <boost/none.hpp>
+#include <boost/smart_ptr.hpp>
+#include <type_traits>
 
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
 #include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/bson/util/builder.h"
 #include "mongo/db/pipeline/accumulator_js_reduce.h"
+#include "mongo/db/pipeline/javascript_execution.h"
 #include "mongo/db/pipeline/make_js_function.h"
 #include "mongo/db/pipeline/map_reduce_options_gen.h"
+#include "mongo/scripting/engine.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/intrusive_counter.h"
+#include "mongo/util/str.h"
 
 namespace mongo {
 
@@ -45,7 +60,7 @@ AccumulationExpression AccumulatorInternalJsReduce::parseInternalJsReduce(
             elem.type() == BSONType::Object);
     BSONObj obj = elem.embeddedObject();
 
-    expCtx->sbeGroupCompatible = false;
+    expCtx->sbeGroupCompatibility = SbeCompatibility::notCompatible;
     std::string funcSource;
     boost::intrusive_ptr<Expression> argument;
 
@@ -62,11 +77,11 @@ AccumulationExpression AccumulatorInternalJsReduce::parseInternalJsReduce(
     }
     uassert(31245,
             str::stream() << kName
-                          << " requires 'eval' argument, recieved input: " << obj.toString(false),
+                          << " requires 'eval' argument, received input: " << obj.toString(false),
             !funcSource.empty());
     uassert(31349,
             str::stream() << kName
-                          << " requires 'data' argument, recieved input: " << obj.toString(false),
+                          << " requires 'data' argument, received input: " << obj.toString(false),
             argument);
 
     auto factory = [expCtx, funcSource = funcSource]() {
@@ -113,7 +128,7 @@ void AccumulatorInternalJsReduce::processInternal(const Value& input, bool mergi
 
     _key = kField;
 
-    _memUsageBytes += vField.getApproximateSize();
+    _memUsageTracker.add(vField.getApproximateSize());
     _values.push_back(std::move(vField));
 }
 
@@ -189,7 +204,7 @@ boost::intrusive_ptr<AccumulatorState> AccumulatorInternalJsReduce::create(
 
 void AccumulatorInternalJsReduce::reset() {
     _values.clear();
-    _memUsageBytes = sizeof(*this);
+    _memUsageTracker.set(sizeof(*this));
     _key = Value{};
 }
 
@@ -254,6 +269,7 @@ Document AccumulatorJs::serialize(boost::intrusive_ptr<Expression> initializer,
 AccumulationExpression AccumulatorJs::parse(ExpressionContext* const expCtx,
                                             BSONElement elem,
                                             VariablesParseState vps) {
+    expCtx->hasServerSideJs.accumulator = true;
     /*
      * {$accumulator: {
      *   init: <code>,
@@ -274,7 +290,7 @@ AccumulationExpression AccumulatorJs::parse(ExpressionContext* const expCtx,
             elem.type() == BSONType::Object);
     BSONObj obj = elem.embeddedObject();
 
-    expCtx->sbeGroupCompatible = false;
+    expCtx->sbeGroupCompatibility = SbeCompatibility::notCompatible;
     std::string init, accumulate, merge;
     boost::optional<std::string> finalize;
     boost::intrusive_ptr<Expression> initArgs, accumulateArgs;
@@ -360,14 +376,12 @@ Value AccumulatorJs::getValue(bool toBeMerged) {
 }
 
 void AccumulatorJs::resetMemUsageBytes() {
-    _memUsageBytes = sizeof(*this) + _init.capacity() + _accumulate.capacity() + _merge.capacity();
+    _memUsageTracker.set(sizeof(*this) + _init.capacity() + _accumulate.capacity() +
+                         _merge.capacity());
     if (_finalize) {
-        _memUsageBytes += _finalize->capacity();
+        _memUsageTracker.add(_finalize->capacity());
     }
 }
-void AccumulatorJs::incrementMemUsageBytes(size_t bytes) {
-    _memUsageBytes += bytes;
-}
 
 void AccumulatorJs::startNewGroup(Value const& input) {
     // Between groups the _state should be empty: we initialize it to be empty it in the
@@ -394,7 +408,7 @@ void AccumulatorJs::startNewGroup(Value const& input) {
 
     // getApproximateSize includes sizeof(Value), but we already counted that in resetMemUsageBytes
     // as part of sizeof(*this).
-    incrementMemUsageBytes(_state->getApproximateSize() - sizeof(Value));
+    _memUsageTracker.add(_state->getApproximateSize() - sizeof(Value));
 }
 
 void AccumulatorJs::reset() {
@@ -421,8 +435,8 @@ void AccumulatorJs::processInternal(const Value& input, bool merging) {
 
     // getApproximateSize includes sizeof(Value), but we already counted that in resetMemUsageBytes
     // as part of sizeof(*this).
-    incrementMemUsageBytes(input.getApproximateSize() - sizeof(Value) +
-                           sizeof(std::pair<Value, bool>));
+    _memUsageTracker.add(input.getApproximateSize() - sizeof(Value) +
+                         sizeof(std::pair<Value, bool>));
 }
 
 void AccumulatorJs::reduceMemoryConsumptionIfAble() {
@@ -504,7 +518,7 @@ void AccumulatorJs::reduceMemoryConsumptionIfAble() {
     _pendingCalls.clear();
 
     resetMemUsageBytes();
-    incrementMemUsageBytes(_state->getApproximateSize());
+    _memUsageTracker.add(_state->getApproximateSize());
 }
 
 }  // namespace mongo
