@@ -28,8 +28,30 @@
  */
 
 #include "mongo/db/pipeline/accumulator_multi.h"
+
+#include <absl/container/node_hash_map.h>
+#include <boost/move/utility_core.hpp>
+#include <boost/smart_ptr.hpp>
+#include <iterator>
+#include <memory>
+#include <type_traits>
+
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/bson/util/builder.h"
+#include "mongo/bson/util/builder_fwd.h"
+#include "mongo/db/pipeline/field_path.h"
+#include "mongo/db/query/query_knobs_gen.h"
 #include "mongo/db/query/sort_pattern.h"
-#include "mongo/util/version/releases.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/intrusive_counter.h"
+#include "mongo/util/str.h"
 
 namespace mongo {
 using FirstLastSense = AccumulatorFirstLastN::Sense;
@@ -80,7 +102,7 @@ void AccumulatorN::processInternal(const Value& input, bool merging) {
 
     if (merging) {
         tassert(5787803, "input must be an array when 'merging' is true", input.isArray());
-        auto array = input.getArray();
+        const auto& array = input.getArray();
         for (auto&& val : array) {
             _processValue(val);
         }
@@ -166,7 +188,6 @@ template <MinMaxSense s>
 AccumulationExpression AccumulatorMinMaxN::parseMinMaxN(ExpressionContext* const expCtx,
                                                         BSONElement elem,
                                                         VariablesParseState vps) {
-    expCtx->sbeGroupCompatibility = SbeCompatibility::notCompatible;
     auto name = [] {
         if constexpr (s == MinMaxSense::kMin) {
             return AccumulatorMinN::getName();
@@ -213,7 +234,7 @@ void AccumulatorMinMaxN::_processValue(const Value& val) {
         }
     }
 
-    _set.emplace(SimpleMemoryToken{val.getApproximateSize(), &_memUsageTracker}, val);
+    _set.emplace(SimpleMemoryUsageToken{val.getApproximateSize(), &_memUsageTracker}, val);
     checkMemUsage();
 }
 
@@ -260,7 +281,6 @@ template <FirstLastSense v>
 AccumulationExpression AccumulatorFirstLastN::parseFirstLastN(ExpressionContext* const expCtx,
                                                               BSONElement elem,
                                                               VariablesParseState vps) {
-    expCtx->sbeGroupCompatibility = SbeCompatibility::notCompatible;
     auto name = [] {
         if constexpr (v == Sense::kFirst) {
             return AccumulatorFirstN::getName();
@@ -303,8 +323,9 @@ void AccumulatorFirstLastN::_processValue(const Value& val) {
         }
     }
 
-    _deque.emplace_back(SimpleMemoryToken{valToProcess.getApproximateSize(), &_memUsageTracker},
-                        std::move(valToProcess));
+    _deque.emplace_back(
+        SimpleMemoryUsageToken{valToProcess.getApproximateSize(), &_memUsageTracker},
+        std::move(valToProcess));
     checkMemUsage();
 }
 
@@ -338,7 +359,7 @@ boost::intrusive_ptr<Expression> AccumulatorFirstLastN::parseExpression(
 }
 
 void AccumulatorFirstLastN::reset() {
-    _deque = std::deque<SimpleMemoryTokenWith<Value>>();
+    _deque = std::deque<SimpleMemoryUsageTokenWith<Value>>();
 }
 
 Value AccumulatorFirstLastN::getValue(bool toBeMerged) {
@@ -501,12 +522,13 @@ Document AccumulatorTopBottomN<sense, single>::serialize(
 }
 
 template <TopBottomSense sense>
-std::pair<SortPattern, BSONArray> parseAccumulatorTopBottomNSortBy(ExpressionContext* const expCtx,
-                                                                   BSONObj sortBy) {
+std::tuple<SortPattern, BSONArray, bool> parseAccumulatorTopBottomNSortBy(
+    ExpressionContext* const expCtx, BSONObj sortBy) {
 
     SortPattern sortPattern(sortBy, expCtx);
     BSONArrayBuilder sortFieldsExpBab;
     BSONObjIterator sortByBoi(sortBy);
+    bool hasMeta = false;
     for (const auto& part : sortPattern) {
         const auto fieldName = sortByBoi.next().fieldNameStringData();
         if (part.expression) {
@@ -517,11 +539,12 @@ std::pair<SortPattern, BSONArray> parseAccumulatorTopBottomNSortBy(ExpressionCon
             // sortFields array contains the data we need for sorting.
             const auto serialized = part.expression->serialize();
             sortFieldsExpBab.append(serialized.getDocument().toBson());
+            hasMeta = true;
         } else {
             sortFieldsExpBab.append((StringBuilder() << "$" << fieldName).str());
         }
     }
-    return {sortPattern, sortFieldsExpBab.arr()};
+    return {sortPattern, sortFieldsExpBab.arr(), hasMeta};
 }
 
 template <TopBottomSense sense, bool single>
@@ -530,9 +553,12 @@ AccumulationExpression AccumulatorTopBottomN<sense, single>::parseTopBottomN(
     auto name = AccumulatorTopBottomN<sense, single>::getName();
     const auto [n, output, sortBy] =
         accumulatorNParseArgs<single>(expCtx, elem, name.rawData(), true, vps);
-    auto [sortPattern, sortFieldsExp] = parseAccumulatorTopBottomNSortBy<sense>(expCtx, *sortBy);
+    auto [sortPattern, sortFieldsExp, hasMeta] =
+        parseAccumulatorTopBottomNSortBy<sense>(expCtx, *sortBy);
 
-    expCtx->sbeGroupCompatibility = SbeCompatibility::notCompatible;
+    if (hasMeta) {
+        expCtx->sbeGroupCompatibility = SbeCompatibility::notCompatible;
+    }
 
     // Construct argument expression. If given sortBy: {field1: 1, field2: 1} it will be shaped like
     // {output: <output expression>, sortFields0: "$field1", sortFields1: "$field2"}. This projects
@@ -561,7 +587,7 @@ template <TopBottomSense sense, bool single>
 boost::intrusive_ptr<AccumulatorState> AccumulatorTopBottomN<sense, single>::create(
     ExpressionContext* expCtx, BSONObj sortBy, bool isRemovable) {
     return make_intrusive<AccumulatorTopBottomN<sense, single>>(
-        expCtx, parseAccumulatorTopBottomNSortBy<sense>(expCtx, sortBy).first, isRemovable);
+        expCtx, std::get<0>(parseAccumulatorTopBottomNSortBy<sense>(expCtx, sortBy)), isRemovable);
 }
 
 template <TopBottomSense sense, bool single>
@@ -628,8 +654,8 @@ void AccumulatorTopBottomN<sense, single>::_processValue(const Value& val) {
     const auto memUsage = keyOutPair.first.getApproximateSize() +
         keyOutPair.second.getApproximateSize() + sizeof(KeyOutPair);
     _map->emplace(keyOutPair.first,
-                  SimpleMemoryTokenWith<Value>{SimpleMemoryToken{memUsage, &_memUsageTracker},
-                                               keyOutPair.second});
+                  SimpleMemoryUsageTokenWith<Value>{
+                      SimpleMemoryUsageToken{memUsage, &_memUsageTracker}, keyOutPair.second});
     checkMemUsage();
 }
 
@@ -705,17 +731,17 @@ Value AccumulatorTopBottomN<sense, single>::getValueConst(bool toBeMerged) const
     };
 
     if constexpr (!single) {
-        return Value(result);
+        return Value(std::move(result));
     } else {
         if (toBeMerged) {
-            return Value(result);
+            return Value(std::move(result));
         } else {
             if (result.empty()) {
                 // This only occurs in a window function scenario, an accumulator will always have
                 // at least one value processed.
                 return Value(BSONNULL);
             }
-            return Value(result[0]);
+            return Value(std::move(result[0]));
         }
     }
 }
