@@ -28,33 +28,32 @@
  */
 
 #include "mongo/db/pipeline/accumulator_percentile.h"
+
+#include <type_traits>
+
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/db/basic_types.h"
 #include "mongo/db/exec/document_value/value.h"
+#include "mongo/db/pipeline/accumulator_percentile_gen.h"
 #include "mongo/idl/idl_parser.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/intrusive_counter.h"
+#include "mongo/util/str.h"
 
 namespace mongo {
 
 using boost::intrusive_ptr;
 
-REGISTER_ACCUMULATOR_WITH_FEATURE_FLAG(percentile,
-                                       AccumulatorPercentile::parseArgs,
-                                       feature_flags::gFeatureFlagApproxPercentiles);
+REGISTER_ACCUMULATOR(percentile, AccumulatorPercentile::parseArgs);
+REGISTER_STABLE_EXPRESSION(percentile, AccumulatorPercentile::parseExpression);
 
-REGISTER_EXPRESSION_WITH_FEATURE_FLAG(percentile,
-                                      AccumulatorPercentile::parseExpression,
-                                      AllowedWithApiStrict::kNeverInVersion1,
-                                      AllowedWithClientType::kAny,
-                                      feature_flags::gFeatureFlagApproxPercentiles);
-
-
-REGISTER_ACCUMULATOR_WITH_FEATURE_FLAG(median,
-                                       AccumulatorMedian::parseArgs,
-                                       feature_flags::gFeatureFlagApproxPercentiles);
-
-REGISTER_EXPRESSION_WITH_FEATURE_FLAG(median,
-                                      AccumulatorMedian::parseExpression,
-                                      AllowedWithApiStrict::kNeverInVersion1,
-                                      AllowedWithClientType::kAny,
-                                      feature_flags::gFeatureFlagApproxPercentiles);
+REGISTER_ACCUMULATOR(median, AccumulatorMedian::parseArgs);
+REGISTER_STABLE_EXPRESSION(median, AccumulatorMedian::parseExpression);
 
 Status AccumulatorPercentile::validatePercentileMethod(StringData method) {
     if (method != kApproximate) {
@@ -179,7 +178,6 @@ boost::intrusive_ptr<Expression> AccumulatorPercentile::parseExpression(
 
 void AccumulatorPercentile::processInternal(const Value& input, bool merging) {
     if (merging) {
-        assertMergingInputType(input, Array);
         dynamic_cast<PartialPercentile<Value>*>(_algo.get())->combine(input);
         return;
     }
