@@ -27,16 +27,26 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <vector>
 
-#include "mongo/db/pipeline/accumulator.h"
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/bson/bsontypes.h"
 #include "mongo/db/exec/document_value/value.h"
 #include "mongo/db/pipeline/accumulation_statement.h"
+#include "mongo/db/pipeline/accumulator.h"
 #include "mongo/db/pipeline/expression_context.h"
 #include "mongo/db/pipeline/window_function/window_function_expression.h"
 #include "mongo/db/pipeline/window_function/window_function_push.h"
 #include "mongo/db/query/query_knobs_gen.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/intrusive_counter.h"
+#include "mongo/util/str.h"
 
 namespace mongo {
 
@@ -50,7 +60,7 @@ void AccumulatorPush::processInternal(const Value& input, bool merging) {
     if (!merging) {
         if (!input.missing()) {
             _array.push_back(input);
-            _memUsageTracker.update(input.getApproximateSize());
+            _memUsageTracker.add(input.getApproximateSize());
             uassert(ErrorCodes::ExceededMemoryLimit,
                     str::stream()
                         << "$push used too much memory and cannot spill to disk. Memory limit: "
@@ -61,11 +71,11 @@ void AccumulatorPush::processInternal(const Value& input, bool merging) {
         // If we're merging, we need to take apart the arrays we receive and put their elements into
         // the array we are collecting.  If we didn't, then we'd get an array of arrays, with one
         // array from each merge source.
-        assertMergingInputType(input, Array);
+        invariant(input.getType() == Array);
 
         const vector<Value>& vec = input.getArray();
         for (auto&& val : vec) {
-            _memUsageTracker.update(val.getApproximateSize());
+            _memUsageTracker.add(val.getApproximateSize());
             uassert(ErrorCodes::ExceededMemoryLimit,
                     str::stream()
                         << "$push used too much memory and cannot spill to disk. Memory limit: "
