@@ -28,20 +28,39 @@
  */
 
 #include "mongo/db/pipeline/aggregation_request_helper.h"
+
+#include <boost/cstdint.hpp>
+#include <boost/move/utility_core.hpp>
+#include <cstdint>
+#include <memory>
+#include <string>
+
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+
 #include "mongo/base/error_codes.h"
 #include "mongo/base/status_with.h"
 #include "mongo/base/string_data.h"
+#include "mongo/bson/bsonmisc.h"
 #include "mongo/bson/bsonobjbuilder.h"
-#include "mongo/db/catalog/document_validation.h"
-#include "mongo/db/commands.h"
+#include "mongo/bson/simple_bsonobj_comparator.h"
+#include "mongo/db/api_parameters.h"
+#include "mongo/db/auth/validated_tenancy_scope.h"
+#include "mongo/db/auth/validated_tenancy_scope_factory.h"
+#include "mongo/db/basic_types.h"
+#include "mongo/db/client.h"
 #include "mongo/db/exec/document_value/document.h"
-#include "mongo/db/exec/document_value/value.h"
-#include "mongo/db/query/cursor_request.h"
+#include "mongo/db/feature_flag.h"
+#include "mongo/db/pipeline/aggregate_command_gen.h"
 #include "mongo/db/query/query_request_helper.h"
-#include "mongo/db/repl/read_concern_args.h"
-#include "mongo/db/storage/storage_options.h"
-#include "mongo/idl/command_generic_argument.h"
-#include "mongo/platform/basic.h"
+#include "mongo/db/server_options.h"
+#include "mongo/db/tenant_id.h"
+#include "mongo/db/write_concern_options.h"
+#include "mongo/idl/idl_parser.h"
+#include "mongo/s/resharding/resharding_feature_flag_gen.h"
+#include "mongo/transport/session.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/str.h"
 
 namespace mongo {
 namespace aggregation_request_helper {
@@ -49,64 +68,32 @@ namespace aggregation_request_helper {
 /**
  * Validate the aggregate command object.
  */
-void validate(OperationContext* opCtx,
-              const BSONObj& cmdObj,
+void validate(const BSONObj& cmdObj,
               const NamespaceString& nss,
               boost::optional<ExplainOptions::Verbosity> explainVerbosity);
 
-AggregateCommandRequest parseFromBSON(OperationContext* opCtx,
-                                      const DatabaseName& dbName,
-                                      const BSONObj& cmdObj,
-                                      boost::optional<ExplainOptions::Verbosity> explainVerbosity,
-                                      bool apiStrict) {
-    return parseFromBSON(opCtx, parseNs(dbName, cmdObj), cmdObj, explainVerbosity, apiStrict);
-}
-
 StatusWith<AggregateCommandRequest> parseFromBSONForTests(
-    NamespaceString nss,
-    const BSONObj& cmdObj,
-    boost::optional<ExplainOptions::Verbosity> explainVerbosity,
-    bool apiStrict) {
-    try {
-        return parseFromBSON(/*opCtx=*/nullptr, nss, cmdObj, explainVerbosity, apiStrict);
-    } catch (const AssertionException&) {
-        return exceptionToStatus();
-    }
-}
-
-StatusWith<AggregateCommandRequest> parseFromBSONForTests(
-    const DatabaseName& dbName,
     const BSONObj& cmdObj,
+    const boost::optional<auth::ValidatedTenancyScope>& vts,
     boost::optional<ExplainOptions::Verbosity> explainVerbosity,
     bool apiStrict) {
     try {
         return parseFromBSON(
-            /*opCtx=*/nullptr, dbName, cmdObj, explainVerbosity, apiStrict);
+            cmdObj, vts, explainVerbosity, apiStrict, SerializationContext::stateDefault());
     } catch (const AssertionException&) {
         return exceptionToStatus();
     }
 }
 
-AggregateCommandRequest parseFromBSON(OperationContext* opCtx,
-                                      NamespaceString nss,
-                                      const BSONObj& cmdObj,
+AggregateCommandRequest parseFromBSON(const BSONObj& cmdObj,
+                                      const boost::optional<auth::ValidatedTenancyScope>& vts,
                                       boost::optional<ExplainOptions::Verbosity> explainVerbosity,
-                                      bool apiStrict) {
-
-    // if the command object lacks field 'aggregate' or '$db', we will use the namespace in 'nss'.
-    bool cmdObjChanged = false;
-    auto cmdObjBob = BSONObjBuilder{BSON(AggregateCommandRequest::kCommandName << nss.coll())};
-    if (!cmdObj.hasField(AggregateCommandRequest::kCommandName) ||
-        !cmdObj.hasField(AggregateCommandRequest::kDbNameFieldName)) {
-        cmdObjBob.append("$db", nss.db());
-        cmdObjBob.appendElementsUnique(cmdObj);
-        cmdObjChanged = true;
-    }
-
-    AggregateCommandRequest request(nss);
-    request =
-        AggregateCommandRequest::parse(IDLParserContext("aggregate", apiStrict, nss.tenantId()),
-                                       cmdObjChanged ? cmdObjBob.obj() : cmdObj);
+                                      bool apiStrict,
+                                      const SerializationContext& serializationContext) {
+    auto tenantId = vts.has_value() ? boost::make_optional(vts->tenantId()) : boost::none;
+    auto request = AggregateCommandRequest::parse(
+        IDLParserContext("aggregate", apiStrict, vts, std::move(tenantId), serializationContext),
+        cmdObj);
 
     if (explainVerbosity) {
         uassert(ErrorCodes::FailedToParse,
@@ -116,47 +103,27 @@ AggregateCommandRequest parseFromBSON(OperationContext* opCtx,
         request.setExplain(explainVerbosity);
     }
 
-    validate(opCtx, cmdObj, nss, explainVerbosity);
+    validate(cmdObj, request.getNamespace(), explainVerbosity);
     return request;
 }
 
-NamespaceString parseNs(const DatabaseName& dbName, const BSONObj& cmdObj) {
-    auto firstElement = cmdObj.firstElement();
-
-    if (firstElement.isNumber()) {
-        uassert(ErrorCodes::FailedToParse,
-                str::stream() << "Invalid command format: the '"
-                              << firstElement.fieldNameStringData()
-                              << "' field must specify a collection name or 1",
-                firstElement.number() == 1);
-        return NamespaceString::makeCollectionlessAggregateNSS(dbName);
-    } else {
-        uassert(ErrorCodes::TypeMismatch,
-                str::stream() << "collection name has invalid type: "
-                              << typeName(firstElement.type()),
-                firstElement.type() == BSONType::String);
-
-        const NamespaceString nss(
-            NamespaceStringUtil::parseNamespaceFromRequest(dbName, firstElement.valueStringData()));
-
-        uassert(ErrorCodes::InvalidNamespace,
-                str::stream() << "Invalid namespace specified '" << nss.ns() << "'",
-                nss.isValid() && !nss.isCollectionlessAggregateNS());
-
-        return nss;
-    }
-}
-
 BSONObj serializeToCommandObj(const AggregateCommandRequest& request) {
     return request.toBSON(BSONObj());
 }
 
-Document serializeToCommandDoc(const AggregateCommandRequest& request) {
-    return Document(request.toBSON(BSONObj()).getOwned());
+Document serializeToCommandDoc(const boost::intrusive_ptr<ExpressionContext>& expCtx,
+                               const AggregateCommandRequest& request) {
+    MutableDocument doc(Document(request.toBSON(BSONObj()).getOwned()));
+
+    if (auto querySettingsBSON = expCtx->getQuerySettings().toBSON();
+        !querySettingsBSON.isEmpty()) {
+        doc.setField(AggregateCommandRequest::kQuerySettingsFieldName, Value(querySettingsBSON));
+    }
+
+    return doc.freeze();
 }
 
-void validate(OperationContext* opCtx,
-              const BSONObj& cmdObj,
+void validate(const BSONObj& cmdObj,
               const NamespaceString& nss,
               boost::optional<ExplainOptions::Verbosity> explainVerbosity) {
     bool hasCursorElem = cmdObj.hasField(AggregateCommandRequest::kCursorFieldName);
@@ -166,6 +133,12 @@ void validate(OperationContext* opCtx,
     bool hasFromMongosElem = cmdObj.hasField(AggregateCommandRequest::kFromMongosFieldName);
     bool hasNeedsMergeElem = cmdObj.hasField(AggregateCommandRequest::kNeedsMergeFieldName);
 
+    uassert(ErrorCodes::InvalidNamespace,
+            fmt::format("Invalid collection name specified '{}'",
+                        cmdObj.firstElement().valueStringDataSafe()),
+            cmdObj.firstElement().valueStringDataSafe() !=
+                NamespaceString::kCollectionlessAggregateCollection);
+
     // 'hasExplainElem' implies an aggregate command-level explain option, which does not require
     // a cursor argument.
     uassert(ErrorCodes::FailedToParse,
@@ -193,8 +166,38 @@ void validate(OperationContext* opCtx,
         requestReshardingResumeTokenElem && requestReshardingResumeTokenElem.boolean();
     uassert(ErrorCodes::FailedToParse,
             str::stream() << AggregateCommandRequest::kRequestReshardingResumeTokenFieldName
-                          << " must only be set for the oplog namespace, not " << nss,
+                          << " must only be set for the oplog namespace, not "
+                          << nss.toStringForErrorMsg(),
             !hasRequestReshardingResumeToken || nss.isOplog());
+
+    auto requestResumeTokenElem = cmdObj[AggregateCommandRequest::kRequestResumeTokenFieldName];
+    // We need to use isEnabledUseLastLTSFCVWhenUninitialized here because an aggregate
+    // command with $_requestResumeToken could be sent directly to an initial sync node with
+    // uninitialized FCV, and creating/parsing/validating this command invocation happens before
+    // any check that the node is a primary.
+    uassert(
+        90675,
+        "$_requestResumeToken is not supported without Resharding Improvements",
+        !requestResumeTokenElem ||
+            resharding::gFeatureFlagReshardingImprovements.isEnabledUseLastLTSFCVWhenUninitialized(
+                serverGlobalParams.featureCompatibility.acquireFCVSnapshot()));
+    uassert(ErrorCodes::FailedToParse,
+            str::stream() << AggregateCommandRequest::kRequestResumeTokenFieldName
+                          << " must be a boolean type",
+            !requestResumeTokenElem || requestResumeTokenElem.isBoolean());
+    bool hasRequestResumeToken = requestResumeTokenElem && requestResumeTokenElem.boolean();
+    uassert(ErrorCodes::FailedToParse,
+            str::stream() << AggregateCommandRequest::kRequestResumeTokenFieldName
+                          << " must be set for non-oplog namespace",
+            !hasRequestResumeToken || !nss.isOplog());
+    if (hasRequestResumeToken) {
+        auto hintElem = cmdObj[AggregateCommandRequest::kHintFieldName];
+        uassert(ErrorCodes::BadValue,
+                "hint must be {$natural:1} if 'requestResumeToken' is enabled",
+                hintElem && hintElem.isABSONObj() &&
+                    SimpleBSONObjComparator::kInstance.evaluate(
+                        hintElem.Obj() == BSON(query_request_helper::kNaturalSortField << 1)));
+    }
 }
 
 void validateRequestForAPIVersion(const OperationContext* opCtx,
@@ -209,9 +212,7 @@ void validateRequestForAPIVersion(const OperationContext* opCtx,
     // An internal client could be one of the following :
     //     - Does not have any transport session
     //     - The transport session tag is internal
-    bool isInternalClient =
-        !client->session() || (client->session()->getTags() & transport::Session::kInternalClient);
-
+    bool isInternalThreadOrClient = !client->session() || client->isInternalClient();
     // Checks that the 'exchange' or 'fromMongos' option can only be specified by the internal
     // client.
     if ((request.getExchange() || request.getFromMongos()) && apiStrict && apiVersion == "1") {
@@ -219,7 +220,7 @@ void validateRequestForAPIVersion(const OperationContext* opCtx,
                 str::stream() << "'exchange' and 'fromMongos' option cannot be specified with "
                                  "'apiStrict: true' in API Version "
                               << apiVersion,
-                isInternalClient);
+                isInternalThreadOrClient);
     }
 }
 
@@ -244,6 +245,9 @@ PlanExecutorPipeline::ResumableScanType getResumableScanType(const AggregateComm
     if (request.getRequestReshardingResumeToken()) {
         return PlanExecutorPipeline::ResumableScanType::kOplogScan;
     }
+    if (request.getRequestResumeToken()) {
+        return PlanExecutorPipeline::ResumableScanType::kNaturalOrderScan;
+    }
     return PlanExecutorPipeline::ResumableScanType::kNone;
 }
 }  // namespace aggregation_request_helper
