@@ -27,22 +27,47 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/pipeline/aggregation_request_helper.h"
+#include <boost/cstdint.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <cstdint>
+#include <fmt/format.h>
+#include <memory>
+#include <string>
+#include <vector>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
 #include "mongo/bson/bsonobj.h"
 #include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/bson/json.h"
-#include "mongo/db/catalog/document_validation.h"
+#include "mongo/db/basic_types.h"
+#include "mongo/db/basic_types_gen.h"
+#include "mongo/db/database_name.h"
 #include "mongo/db/exec/document_value/document.h"
 #include "mongo/db/exec/document_value/document_value_test_util.h"
 #include "mongo/db/exec/document_value/value.h"
 #include "mongo/db/namespace_string.h"
+#include "mongo/db/pipeline/aggregate_command_gen.h"
+#include "mongo/db/pipeline/aggregation_request_helper.h"
+#include "mongo/db/pipeline/expression_context_for_test.h"
+#include "mongo/db/query/explain_options.h"
 #include "mongo/db/query/query_request_helper.h"
 #include "mongo/db/repl/read_concern_args.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/db/tenant_id.h"
+#include "mongo/idl/idl_parser.h"
+#include "mongo/idl/server_parameter_test_util.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/bson_test_util.h"
+#include "mongo/unittest/framework.h"
 #include "mongo/util/assert_util.h"
+#include "mongo/util/str.h"
+#include "mongo/util/uuid.h"
 
 namespace mongo {
 namespace {
@@ -58,7 +83,6 @@ const Document kDefaultCursorOptionDocument{
 
 TEST(AggregationRequestTest, ShouldParseAllKnownOptions) {
     // Using oplog namespace so that validation of $_requestReshardingResumeToken succeeds.
-    NamespaceString nss = NamespaceString::createNamespaceString_forTest("local.oplog.rs");
     BSONObj inputBson = fromjson(
         "{aggregate: 'oplog.rs', pipeline: [{$match: {a: 'abc'}}], explain: false, allowDiskUse: "
         "true, fromMongos: true, "
@@ -66,14 +90,14 @@ TEST(AggregationRequestTest, ShouldParseAllKnownOptions) {
         "collation: {locale: 'en_US'}, cursor: {batchSize: 10}, hint: {a: 1}, maxTimeMS: 100, "
         "readConcern: {level: 'linearizable'}, $queryOptions: {$readPreference: 'nearest'}, "
         "exchange: {policy: 'roundrobin', consumers:NumberInt(2)}, isMapReduceCommand: true, $db: "
-        "'local', $_isClusterQueryWithoutShardKeyCmd: true}");
+        "'local', $_isClusterQueryWithoutShardKeyCmd: true, includeQueryStatsMetrics: true}");
     auto uuid = UUID::gen();
     BSONObjBuilder uuidBob;
     uuid.appendToBuilder(&uuidBob, AggregateCommandRequest::kCollectionUUIDFieldName);
     inputBson = inputBson.addField(uuidBob.obj().firstElement());
 
     auto request =
-        unittest::assertGet(aggregation_request_helper::parseFromBSONForTests(nss, inputBson));
+        unittest::assertGet(aggregation_request_helper::parseFromBSONForTests(inputBson));
     ASSERT_FALSE(request.getExplain());
     ASSERT_TRUE(request.getAllowDiskUse());
     ASSERT_TRUE(request.getFromMongos());
@@ -97,6 +121,7 @@ TEST(AggregationRequestTest, ShouldParseAllKnownOptions) {
                            << "nearest"));
     ASSERT_TRUE(request.getExchange().has_value());
     ASSERT_TRUE(request.getIsMapReduceCommand());
+    ASSERT_TRUE(request.getIncludeQueryStatsMetrics());
     ASSERT_EQ(*request.getCollectionUUID(), uuid);
 }
 
@@ -106,7 +131,7 @@ TEST(AggregationRequestTest, ShouldParseExplicitRequestReshardingResumeTokenFals
         "{aggregate: 'collection', pipeline: [], $_requestReshardingResumeToken: false, cursor: "
         "{}, $db: 'a'}");
     auto request =
-        unittest::assertGet(aggregation_request_helper::parseFromBSONForTests(nss, inputBson));
+        unittest::assertGet(aggregation_request_helper::parseFromBSONForTests(inputBson));
     ASSERT_FALSE(request.getRequestReshardingResumeToken());
 }
 
@@ -115,7 +140,7 @@ TEST(AggregationRequestTest, ShouldParseExplicitExplainTrue) {
     const BSONObj inputBson =
         fromjson("{aggregate: 'collection', pipeline: [], explain: true, cursor: {}, $db: 'a'}");
     auto request =
-        unittest::assertGet(aggregation_request_helper::parseFromBSONForTests(nss, inputBson));
+        unittest::assertGet(aggregation_request_helper::parseFromBSONForTests(inputBson));
     ASSERT_TRUE(request.getExplain());
     ASSERT(*request.getExplain() == ExplainOptions::Verbosity::kQueryPlanner);
 }
@@ -126,7 +151,7 @@ TEST(AggregationRequestTest, ShouldParseExplicitExplainFalseWithCursorOption) {
         "{aggregate: 'collection', pipeline: [], explain: false, cursor: {batchSize: 10}, $db: "
         "'a'}");
     auto request =
-        unittest::assertGet(aggregation_request_helper::parseFromBSONForTests(nss, inputBson));
+        unittest::assertGet(aggregation_request_helper::parseFromBSONForTests(inputBson));
     ASSERT_FALSE(request.getExplain());
     ASSERT_EQ(
         request.getCursor().getBatchSize().value_or(aggregation_request_helper::kDefaultBatchSize),
@@ -138,7 +163,7 @@ TEST(AggregationRequestTest, ShouldParseWithSeparateQueryPlannerExplainModeArg)
     const BSONObj inputBson =
         fromjson("{aggregate: 'collection', pipeline: [], cursor: {}, $db: 'a'}");
     auto request = unittest::assertGet(aggregation_request_helper::parseFromBSONForTests(
-        nss, inputBson, ExplainOptions::Verbosity::kQueryPlanner));
+        inputBson, boost::none, ExplainOptions::Verbosity::kQueryPlanner));
     ASSERT_TRUE(request.getExplain());
     ASSERT(*request.getExplain() == ExplainOptions::Verbosity::kQueryPlanner);
 }
@@ -148,7 +173,7 @@ TEST(AggregationRequestTest, ShouldParseWithSeparateQueryPlannerExplainModeArgAn
     const BSONObj inputBson =
         fromjson("{aggregate: 'collection', pipeline: [], cursor: {batchSize: 10}, $db: 'a'}");
     auto request = unittest::assertGet(aggregation_request_helper::parseFromBSONForTests(
-        nss, inputBson, ExplainOptions::Verbosity::kExecStats));
+        inputBson, boost::none, ExplainOptions::Verbosity::kExecStats));
     ASSERT_TRUE(request.getExplain());
     ASSERT(*request.getExplain() == ExplainOptions::Verbosity::kExecStats);
     ASSERT_EQ(
@@ -164,7 +189,7 @@ TEST(AggregationRequestTest, ShouldParseExplainFlagWithReadConcern) {
         "{aggregate: 'collection', pipeline: [], explain: true, readConcern: {level: 'majority'}, "
         "$db: 'a'}");
     auto request =
-        unittest::assertGet(aggregation_request_helper::parseFromBSONForTests(nss, inputBson));
+        unittest::assertGet(aggregation_request_helper::parseFromBSONForTests(inputBson));
     ASSERT_TRUE(request.getExplain());
     ASSERT_BSONOBJ_EQ(*request.getReadConcern(),
                       BSON("level"
@@ -183,7 +208,8 @@ TEST(AggregationRequestTest, ShouldOnlySerializeRequiredFieldsIfNoOptionalFields
         Document{{AggregateCommandRequest::kCommandName, nss.coll()},
                  {AggregateCommandRequest::kPipelineFieldName, std::vector<Value>{}},
                  {AggregateCommandRequest::kCursorFieldName, Value(kDefaultCursorOptionDocument)}};
-    ASSERT_DOCUMENT_EQ(aggregation_request_helper::serializeToCommandDoc(request),
+    ASSERT_DOCUMENT_EQ(aggregation_request_helper::serializeToCommandDoc(
+                           make_intrusive<ExpressionContextForTest>(nss), request),
                        expectedSerialization);
 }
 
@@ -218,6 +244,8 @@ TEST(AggregationRequestTest, ShouldSerializeOptionalValuesIfSet) {
     request.setCollectionUUID(uuid);
     request.setIsClusterQueryWithoutShardKeyCmd(true);
 
+    request.setIncludeQueryStatsMetrics(true);
+
     auto expectedSerialization = Document{
         {AggregateCommandRequest::kCommandName, nss.coll()},
         {AggregateCommandRequest::kPipelineFieldName, std::vector<Value>{}},
@@ -235,8 +263,10 @@ TEST(AggregationRequestTest, ShouldSerializeOptionalValuesIfSet) {
         {AggregateCommandRequest::kRequestReshardingResumeTokenFieldName, true},
         {AggregateCommandRequest::kIsMapReduceCommandFieldName, true},
         {AggregateCommandRequest::kCollectionUUIDFieldName, uuid},
-        {AggregateCommandRequest::kIsClusterQueryWithoutShardKeyCmdFieldName, true}};
-    ASSERT_DOCUMENT_EQ(aggregation_request_helper::serializeToCommandDoc(request),
+        {AggregateCommandRequest::kIsClusterQueryWithoutShardKeyCmdFieldName, true},
+        {AggregateCommandRequest::kIncludeQueryStatsMetricsFieldName, true}};
+    ASSERT_DOCUMENT_EQ(aggregation_request_helper::serializeToCommandDoc(
+                           make_intrusive<ExpressionContextForTest>(nss), request),
                        expectedSerialization);
 }
 
@@ -251,13 +281,14 @@ TEST(AggregationRequestTest, ShouldSerializeBatchSizeIfSetAndExplainFalse) {
         {AggregateCommandRequest::kCommandName, nss.coll()},
         {AggregateCommandRequest::kPipelineFieldName, std::vector<Value>{}},
         {AggregateCommandRequest::kCursorFieldName, Value(Document({{kBatchSizeFieldName, 10}}))}};
-    ASSERT_DOCUMENT_EQ(aggregation_request_helper::serializeToCommandDoc(request),
+    ASSERT_DOCUMENT_EQ(aggregation_request_helper::serializeToCommandDoc(
+                           make_intrusive<ExpressionContextForTest>(nss), request),
                        expectedSerialization);
 }
 
 TEST(AggregationRequestTest, ShouldSerialiseAggregateFieldToOneIfCollectionIsAggregateOneNSS) {
-    NamespaceString nss =
-        NamespaceString::makeCollectionlessAggregateNSS(DatabaseName(boost::none, "a"));
+    NamespaceString nss = NamespaceString::makeCollectionlessAggregateNSS(
+        DatabaseName::createDatabaseName_forTest(boost::none, "a"));
     AggregateCommandRequest request(nss, std::vector<mongo::BSONObj>());
 
     auto expectedSerialization =
@@ -267,15 +298,15 @@ TEST(AggregationRequestTest, ShouldSerialiseAggregateFieldToOneIfCollectionIsAgg
                   Value(Document({{aggregation_request_helper::kBatchSizeField,
                                    aggregation_request_helper::kDefaultBatchSize}}))}};
 
-    ASSERT_DOCUMENT_EQ(aggregation_request_helper::serializeToCommandDoc(request),
+    ASSERT_DOCUMENT_EQ(aggregation_request_helper::serializeToCommandDoc(
+                           make_intrusive<ExpressionContextForTest>(nss), request),
                        expectedSerialization);
 }
 
 TEST(AggregationRequestTest, ShouldSetBatchSizeToDefaultOnEmptyCursorObject) {
-    NamespaceString nss = NamespaceString::createNamespaceString_forTest("a.collection");
     const BSONObj inputBson = fromjson(
         "{aggregate: 'collection', pipeline: [{$match: {a: 'abc'}}], cursor: {}, $db: 'a'}");
-    auto request = aggregation_request_helper::parseFromBSONForTests(nss, inputBson);
+    auto request = aggregation_request_helper::parseFromBSONForTests(inputBson);
     ASSERT_OK(request.getStatus());
     ASSERT_EQ(request.getValue().getCursor().getBatchSize().value_or(
                   aggregation_request_helper::kDefaultBatchSize),
@@ -283,11 +314,10 @@ TEST(AggregationRequestTest, ShouldSetBatchSizeToDefaultOnEmptyCursorObject) {
 }
 
 TEST(AggregationRequestTest, ShouldAcceptHintAsString) {
-    NamespaceString nss = NamespaceString::createNamespaceString_forTest("a.collection");
     const BSONObj inputBson = fromjson(
         "{aggregate: 'collection', pipeline: [{$match: {a: 'abc'}}], hint: 'a_1', cursor: {}, $db: "
         "'a'}");
-    auto request = aggregation_request_helper::parseFromBSONForTests(nss, inputBson);
+    auto request = aggregation_request_helper::parseFromBSONForTests(inputBson);
     ASSERT_OK(request.getStatus());
     ASSERT_BSONOBJ_EQ(request.getValue().getHint().value_or(BSONObj()),
                       BSON("$hint"
@@ -306,7 +336,8 @@ TEST(AggregationRequestTest, ShouldNotSerializeBatchSizeWhenExplainSet) {
         Document{{AggregateCommandRequest::kCommandName, nss.coll()},
                  {AggregateCommandRequest::kPipelineFieldName, std::vector<Value>{}},
                  {AggregateCommandRequest::kCursorFieldName, Value(Document())}};
-    ASSERT_DOCUMENT_EQ(aggregation_request_helper::serializeToCommandDoc(request),
+    ASSERT_DOCUMENT_EQ(aggregation_request_helper::serializeToCommandDoc(
+                           make_intrusive<ExpressionContextForTest>(nss), request),
                        expectedSerialization);
 }
 
@@ -356,70 +387,67 @@ BSONObj constructInvalidRequest(const BSONObj& validRequest, const BSONObj& inva
  * 'invalidFields' are added to it.
  */
 void aggregationRequestParseFailureHelper(
-    const NamespaceString& nss,
     const BSONObj& validRequest,
     const BSONObj& invalidFields,
     ErrorCodes::Error expectedCode,
     boost::optional<ExplainOptions::Verbosity> explainVerbosity = boost::none) {
     // Verify that 'validRequest' parses correctly.
-    ASSERT_OK(aggregation_request_helper::parseFromBSONForTests(nss, validRequest, explainVerbosity)
+    ASSERT_OK(aggregation_request_helper::parseFromBSONForTests(
+                  validRequest, boost::none, explainVerbosity)
                   .getStatus());
 
     auto invalidRequest = constructInvalidRequest(validRequest, invalidFields);
 
     // Verify that the constructed invalid request fails to parse.
-    auto status =
-        aggregation_request_helper::parseFromBSONForTests(nss, invalidRequest, explainVerbosity)
-            .getStatus();
+    auto status = aggregation_request_helper::parseFromBSONForTests(
+                      invalidRequest, boost::none, explainVerbosity)
+                      .getStatus();
     ASSERT_NOT_OK(status);
     ASSERT_EQ(status.code(), expectedCode);
 }
 
+// This leverages the IDL generated aggregation namespace parsing.
+NamespaceString parseNS(const BSONObj& cmdObj) {
+    return uassertStatusOK(aggregation_request_helper::parseFromBSONForTests(cmdObj))
+        .getNamespace();
+}
+
 /**
  * Verifies that 'validRequest' parses correctly, but throws 'expectedCode' once 'invalidFields'
  * are added to it.
  */
-void parseNSHelper(const std::string& dbName,
-                   const BSONObj& validRequest,
+void parseNSHelper(const BSONObj& validRequest,
                    const BSONObj& invalidFields,
                    ErrorCodes::Error expectedCode) {
-    // Verify that 'validRequest' parses correctly.
-    auto shouldNotThrow =
-        aggregation_request_helper::parseNs(DatabaseName(boost::none, dbName), validRequest);
 
+    // Verify that 'validRequest' parses correctly.
+    auto shouldNotThrow = parseNS(validRequest);
     auto invalidRequest = constructInvalidRequest(validRequest, invalidFields);
 
     // Verify that the constructed invalid request fails to parse with 'expectedCode'.
-    ASSERT_THROWS_CODE(
-        aggregation_request_helper::parseNs(DatabaseName(boost::none, "a"), invalidRequest),
-        AssertionException,
-        expectedCode);
+    ASSERT_THROWS_CODE(parseNS(invalidRequest), AssertionException, expectedCode);
 }
 
 TEST(AggregationRequestTest, ShouldRejectNonArrayPipeline) {
-    NamespaceString nss = NamespaceString::createNamespaceString_forTest("a.collection");
     const BSONObj validRequest =
         fromjson("{aggregate: 'collection', pipeline: [], cursor: {}, $db: 'a'}");
     const BSONObj nonArrayPipeline = fromjson("{pipeline: {}}");
-    aggregationRequestParseFailureHelper(
-        nss, validRequest, nonArrayPipeline, ErrorCodes::TypeMismatch);
+    aggregationRequestParseFailureHelper(validRequest, nonArrayPipeline, ErrorCodes::TypeMismatch);
 }
 
 TEST(AggregationRequestTest, ShouldRejectPipelineArrayIfAnElementIsNotAnObject) {
-    NamespaceString nss = NamespaceString::createNamespaceString_forTest("a.collection");
     const BSONObj validRequest = fromjson(
         "{aggregate: 'collection', pipeline: [{$match: {a: 'abc'}}], cursor: {}, $db: 'a'}");
     BSONObj nonObjectPipelineElem = fromjson("{pipeline: [4]}");
     aggregationRequestParseFailureHelper(
-        nss, validRequest, nonObjectPipelineElem, ErrorCodes::TypeMismatch);
+        validRequest, nonObjectPipelineElem, ErrorCodes::TypeMismatch);
 
     nonObjectPipelineElem = fromjson("{pipeline: [{$match: {a: 'abc'}}, 4]}");
     aggregationRequestParseFailureHelper(
-        nss, validRequest, nonObjectPipelineElem, ErrorCodes::TypeMismatch);
+        validRequest, nonObjectPipelineElem, ErrorCodes::TypeMismatch);
 }
 
 TEST(AggregationRequestTest, ShouldRejectNonObjectCollation) {
-    NamespaceString nss = NamespaceString::createNamespaceString_forTest("a.collection");
     const BSONObj validRequest = fromjson(
         "{aggregate: 'collection', "
         "pipeline: [{$match: {a: 'abc'}}],"
@@ -428,7 +456,7 @@ TEST(AggregationRequestTest, ShouldRejectNonObjectCollation) {
         "$db: 'a'}");
     const BSONObj nonObjectCollation = fromjson("{collation: 1}");
     aggregationRequestParseFailureHelper(
-        nss, validRequest, nonObjectCollation, ErrorCodes::TypeMismatch);
+        validRequest, nonObjectCollation, ErrorCodes::TypeMismatch);
 }
 
 TEST(AggregationRequestTest, ShouldRejectNonStringNonObjectHint) {
@@ -440,8 +468,7 @@ TEST(AggregationRequestTest, ShouldRejectNonStringNonObjectHint) {
         "hint: {_id: 1},"
         "$db: 'a'}");
     const BSONObj nonObjectHint = fromjson("{hint: 1}");
-    aggregationRequestParseFailureHelper(
-        nss, validRequest, nonObjectHint, ErrorCodes::FailedToParse);
+    aggregationRequestParseFailureHelper(validRequest, nonObjectHint, ErrorCodes::FailedToParse);
 }
 
 TEST(AggregationRequestTest, ShouldRejectExplainIfNumber) {
@@ -453,8 +480,7 @@ TEST(AggregationRequestTest, ShouldRejectExplainIfNumber) {
         "explain: true, "
         "$db: 'a'}");
     const BSONObj numericExplain = fromjson("{explain: 1}");
-    aggregationRequestParseFailureHelper(
-        nss, validRequest, numericExplain, ErrorCodes::TypeMismatch);
+    aggregationRequestParseFailureHelper(validRequest, numericExplain, ErrorCodes::TypeMismatch);
 }
 
 TEST(AggregationRequestTest, ShouldRejectExplainIfObject) {
@@ -466,8 +492,7 @@ TEST(AggregationRequestTest, ShouldRejectExplainIfObject) {
         "explain: true, "
         "$db: 'a'}");
     const BSONObj objectExplain = fromjson("{explain: {}}");
-    aggregationRequestParseFailureHelper(
-        nss, validRequest, objectExplain, ErrorCodes::TypeMismatch);
+    aggregationRequestParseFailureHelper(validRequest, objectExplain, ErrorCodes::TypeMismatch);
 }
 
 TEST(AggregationRequestTest, ShouldRejectNonBoolFromMongos) {
@@ -479,8 +504,7 @@ TEST(AggregationRequestTest, ShouldRejectNonBoolFromMongos) {
         "fromMongos: true, "
         "$db: 'a'}");
     const BSONObj nonBoolFromMongos = fromjson("{fromMongos: 1}");
-    aggregationRequestParseFailureHelper(
-        nss, validRequest, nonBoolFromMongos, ErrorCodes::TypeMismatch);
+    aggregationRequestParseFailureHelper(validRequest, nonBoolFromMongos, ErrorCodes::TypeMismatch);
 }
 
 TEST(AggregationRequestTest, ShouldRejectNonBoolNeedsMerge) {
@@ -493,8 +517,20 @@ TEST(AggregationRequestTest, ShouldRejectNonBoolNeedsMerge) {
         "fromMongos: true,"
         "$db: 'a'}");
     const BSONObj nonBoolNeedsMerge = fromjson("{needsMerge: 1}");
+    aggregationRequestParseFailureHelper(validRequest, nonBoolNeedsMerge, ErrorCodes::TypeMismatch);
+}
+
+TEST(AggregationRequestTest, ShouldRejectNonBoolIncludeQueryStatsMetrics) {
+    NamespaceString nss = NamespaceString::createNamespaceString_forTest("a.collection");
+    const BSONObj validRequest = fromjson(
+        "{aggregate: 'collection',"
+        "pipeline: [{$match: {a: 'abc'}}], "
+        "cursor: {},"
+        "includeQueryStatsMetrics: false, "
+        "$db: 'a'}");
+    const BSONObj nonBoolIncludeQueryStatsMetrics = fromjson("{includeQueryStatsMetrics: 1}");
     aggregationRequestParseFailureHelper(
-        nss, validRequest, nonBoolNeedsMerge, ErrorCodes::TypeMismatch);
+        validRequest, nonBoolIncludeQueryStatsMetrics, ErrorCodes::TypeMismatch);
 }
 
 TEST(AggregationRequestTest, ShouldRejectNeedsMergeIfFromMongosNotPresent) {
@@ -506,7 +542,7 @@ TEST(AggregationRequestTest, ShouldRejectNeedsMergeIfFromMongosNotPresent) {
         "$db: 'a'}");
     const BSONObj needsMergeNoFromMongos = fromjson("{needsMerge: true}");
     aggregationRequestParseFailureHelper(
-        nss, validRequest, needsMergeNoFromMongos, ErrorCodes::FailedToParse);
+        validRequest, needsMergeNoFromMongos, ErrorCodes::FailedToParse);
 }
 
 TEST(AggregationRequestTest, ShouldRejectNonBoolAllowDiskUse) {
@@ -519,7 +555,7 @@ TEST(AggregationRequestTest, ShouldRejectNonBoolAllowDiskUse) {
         "$db: 'a'}");
     const BSONObj nonBoolAllowDiskUse = fromjson("{allowDiskUse: 1}");
     aggregationRequestParseFailureHelper(
-        nss, validRequest, nonBoolAllowDiskUse, ErrorCodes::TypeMismatch);
+        validRequest, nonBoolAllowDiskUse, ErrorCodes::TypeMismatch);
 }
 
 TEST(AggregationRequestTest, ShouldRejectNonBoolIsMapReduceCommand) {
@@ -532,7 +568,7 @@ TEST(AggregationRequestTest, ShouldRejectNonBoolIsMapReduceCommand) {
         "$db: 'a'}");
     const BSONObj nonBoolIsMapReduceCommand = fromjson("{isMapReduceCommand: 1}");
     aggregationRequestParseFailureHelper(
-        nss, validRequest, nonBoolIsMapReduceCommand, ErrorCodes::TypeMismatch);
+        validRequest, nonBoolIsMapReduceCommand, ErrorCodes::TypeMismatch);
 }
 
 TEST(AggregationRequestTest, ShouldRejectNoCursorNoExplain) {
@@ -543,22 +579,19 @@ TEST(AggregationRequestTest, ShouldRejectNoCursorNoExplain) {
         "{aggregate: 'collection',"
         "pipeline: [{$match: {a: 'abc'}}],"
         "$db: 'a'}");
-    auto status =
-        aggregation_request_helper::parseFromBSONForTests(nss, invalidRequest).getStatus();
+    auto status = aggregation_request_helper::parseFromBSONForTests(invalidRequest).getStatus();
     ASSERT_NOT_OK(status);
     ASSERT_EQ(status.code(), ErrorCodes::FailedToParse);
 
     // Adding explain should cause the aggregate to parse successfully.
     BSONObjBuilder explainRequest(invalidRequest);
     explainRequest.append("explain", true);
-    ASSERT_OK(
-        aggregation_request_helper::parseFromBSONForTests(nss, explainRequest.done()).getStatus());
+    ASSERT_OK(aggregation_request_helper::parseFromBSONForTests(explainRequest.done()).getStatus());
 
     // Adding cursor should cause the aggregate to parse successfully.
     BSONObjBuilder cursorRequest(invalidRequest);
     cursorRequest.append("cursor", BSONObj());
-    ASSERT_OK(
-        aggregation_request_helper::parseFromBSONForTests(nss, cursorRequest.done()).getStatus());
+    ASSERT_OK(aggregation_request_helper::parseFromBSONForTests(cursorRequest.done()).getStatus());
 }
 
 TEST(AggregationRequestTest, ShouldRejectNonObjectCursor) {
@@ -571,11 +604,11 @@ TEST(AggregationRequestTest, ShouldRejectNonObjectCursor) {
         "$db: 'a'}");
     const BSONObj nonObjCursorCommand = fromjson("{cursor: 1}");
     aggregationRequestParseFailureHelper(
-        nss, validRequest, nonObjCursorCommand, ErrorCodes::TypeMismatch);
+        validRequest, nonObjCursorCommand, ErrorCodes::TypeMismatch);
 
     const BSONObj arrayCursorCommand = fromjson("{cursor: []}");
     aggregationRequestParseFailureHelper(
-        nss, validRequest, arrayCursorCommand, ErrorCodes::TypeMismatch);
+        validRequest, arrayCursorCommand, ErrorCodes::TypeMismatch);
 }
 
 TEST(AggregationRequestTest, ShouldRejectExplainTrueWithSeparateExplainArg) {
@@ -586,8 +619,7 @@ TEST(AggregationRequestTest, ShouldRejectExplainTrueWithSeparateExplainArg) {
         "cursor: {},"
         "$db: 'a'}");
     const BSONObj explainTrue = fromjson("{explain: true}");
-    aggregationRequestParseFailureHelper(nss,
-                                         validRequest,
+    aggregationRequestParseFailureHelper(validRequest,
                                          explainTrue,
                                          ErrorCodes::FailedToParse,
                                          ExplainOptions::Verbosity::kExecStats);
@@ -601,8 +633,7 @@ TEST(AggregationRequestTest, ShouldRejectExplainFalseWithSeparateExplainArg) {
         "cursor: {},"
         "$db: 'a'}");
     const BSONObj explainFalse = fromjson("{explain: false}");
-    aggregationRequestParseFailureHelper(nss,
-                                         validRequest,
+    aggregationRequestParseFailureHelper(validRequest,
                                          explainFalse,
                                          ErrorCodes::FailedToParse,
                                          ExplainOptions::Verbosity::kExecStats);
@@ -613,7 +644,7 @@ TEST(AggregationRequestTest, ShouldRejectExplainWithWriteConcernMajority) {
     const BSONObj validRequest =
         fromjson("{aggregate: 'collection', pipeline: [], explain: true, $db: 'a'}");
     const BSONObj wcMajority = fromjson("{writeConcern: {w: 'majority'}}");
-    aggregationRequestParseFailureHelper(nss, validRequest, wcMajority, ErrorCodes::FailedToParse);
+    aggregationRequestParseFailureHelper(validRequest, wcMajority, ErrorCodes::FailedToParse);
 }
 
 TEST(AggregationRequestTest, ShouldRejectExplainExecStatsVerbosityWithWriteConcernMajority) {
@@ -621,43 +652,32 @@ TEST(AggregationRequestTest, ShouldRejectExplainExecStatsVerbosityWithWriteConce
     const BSONObj validRequest =
         fromjson("{aggregate: 'collection', pipeline: [], cursor: {}, $db: 'a'}");
     const BSONObj wcMajority = fromjson("{writeConcern: {w: 'majority'}}");
-    aggregationRequestParseFailureHelper(nss,
-                                         validRequest,
-                                         wcMajority,
-                                         ErrorCodes::FailedToParse,
-                                         ExplainOptions::Verbosity::kExecStats);
+    aggregationRequestParseFailureHelper(
+        validRequest, wcMajority, ErrorCodes::FailedToParse, ExplainOptions::Verbosity::kExecStats);
 }
 
 TEST(AggregationRequestTest, ShouldRejectRequestReshardingResumeTokenIfNonBooleanType) {
-    NamespaceString oplogNss = NamespaceString::createNamespaceString_forTest("local.oplog.rs");
     const BSONObj validRequest = fromjson(
-        "{aggregate: 'collection',"
+        "{aggregate: 'oplog.rs',"
         "pipeline: [],"
         "$_requestReshardingResumeToken: true,"
-        "$db: 'a', "
+        "$db: 'local', "
         "cursor: {}}");
     const BSONObj nonBoolReshardingResumeToken =
         fromjson("{$_requestReshardingResumeToken: 'yes'}");
     aggregationRequestParseFailureHelper(
-        oplogNss, validRequest, nonBoolReshardingResumeToken, ErrorCodes::TypeMismatch);
+        validRequest, nonBoolReshardingResumeToken, ErrorCodes::TypeMismatch);
 }
 
 TEST(AggregationRequestTest, ShouldRejectRequestReshardingResumeTokenIfNonOplogNss) {
-    NamespaceString oplogNss = NamespaceString::createNamespaceString_forTest("local.oplog.rs");
     const BSONObj validRequest = fromjson(
-        "{aggregate: 'collection',"
+        "{aggregate: 'oplog.rs',"
         "pipeline: [],"
         "$_requestReshardingResumeToken: true,"
-        "$db: 'a', "
+        "$db: 'local', "
         "cursor: {}}");
-    ASSERT_OK(
-        aggregation_request_helper::parseFromBSONForTests(oplogNss, validRequest).getStatus());
-
-    NamespaceString nonOplogNss = NamespaceString::createNamespaceString_forTest("a.collection");
-    auto status =
-        aggregation_request_helper::parseFromBSONForTests(nonOplogNss, validRequest).getStatus();
-    ASSERT_NOT_OK(status);
-    ASSERT_EQ(status, ErrorCodes::FailedToParse);
+    const BSONObj nonOplogNss = fromjson("{aggregate: 'collection', $db: 'a'}");
+    aggregationRequestParseFailureHelper(validRequest, nonOplogNss, ErrorCodes::FailedToParse);
 }
 
 TEST(AggregationRequestTest, ParseNSShouldReturnAggregateOneNSIfAggregateFieldIsOne) {
@@ -665,50 +685,34 @@ TEST(AggregationRequestTest, ParseNSShouldReturnAggregateOneNSIfAggregateFieldIs
         "1", "1.0", "NumberInt(1)", "NumberLong(1)", "NumberDecimal('1')"};
 
     for (auto& one : ones) {
-        const BSONObj inputBSON =
-            fromjson(str::stream() << "{aggregate: " << one << ", pipeline: [], $db: 'a'}");
-        ASSERT(aggregation_request_helper::parseNs(DatabaseName(boost::none, "a"), inputBSON)
-                   .isCollectionlessAggregateNS());
+        const BSONObj inputBSON = fromjson(
+            str::stream() << "{aggregate: " << one << ", pipeline: [], $db: 'a', cursor: {}}");
+        ASSERT(parseNS(inputBSON).isCollectionlessAggregateNS());
     }
 }
 
 TEST(AggregationRequestTest, ParseNSShouldRejectNumericNSIfAggregateFieldIsNotOne) {
-    const BSONObj validRequest = fromjson("{aggregate: 1, pipeline: [], $db: 'a'}");
+    const BSONObj validRequest = fromjson("{aggregate: 1, pipeline: [], $db: 'a', cursor: {}}");
     BSONObj nonOneAggregate = fromjson("{aggregate: 2}");
-    parseNSHelper("a", validRequest, nonOneAggregate, ErrorCodes::FailedToParse);
+    parseNSHelper(validRequest, nonOneAggregate, ErrorCodes::BadValue);
 }
 
 TEST(AggregationRequestTest, ParseNSShouldRejectNonStringNonNumericNS) {
-    const BSONObj validRequest = fromjson("{aggregate: 1, pipeline: [], $db: 'a'}");
+    const BSONObj validRequest = fromjson("{aggregate: 1, pipeline: [], $db: 'a', cursor: {}}");
     BSONObj nonStringNonNumericNS = fromjson("{aggregate: {}}");
-    parseNSHelper("a", validRequest, nonStringNonNumericNS, ErrorCodes::TypeMismatch);
+    parseNSHelper(validRequest, nonStringNonNumericNS, ErrorCodes::TypeMismatch);
 }
 
 TEST(AggregationRequestTest, ParseNSShouldRejectAggregateOneStringAsCollectionName) {
-    const BSONObj validRequest = fromjson("{aggregate: 1, pipeline: [], $db: 'a'}");
+    const BSONObj validRequest = fromjson("{aggregate: 1, pipeline: [], $db: 'a', cursor: {}}");
     BSONObj oneStringAsCollectionName = fromjson("{aggregate: '$cmd.aggregate'}");
-    parseNSHelper("a", validRequest, oneStringAsCollectionName, ErrorCodes::InvalidNamespace);
+    parseNSHelper(validRequest, oneStringAsCollectionName, ErrorCodes::InvalidNamespace);
 }
 
 TEST(AggregationRequestTest, ParseNSShouldRejectInvalidCollectionName) {
-    const BSONObj validRequest = fromjson("{aggregate: 1, pipeline: [], $db: 'a'}");
+    const BSONObj validRequest = fromjson("{aggregate: 1, pipeline: [], $db: 'a', cursor: {}}");
     BSONObj invalidCollectionName = fromjson("{aggregate: ''}");
-    parseNSHelper("a", validRequest, invalidCollectionName, ErrorCodes::InvalidNamespace);
-}
-
-TEST(AggregationRequestTest, ParseFromBSONOverloadsShouldProduceIdenticalRequests) {
-    const BSONObj inputBSON = fromjson(
-        "{aggregate: 'collection', pipeline: [{$match: {}}, {$project: {}}], cursor: {}, $db: "
-        "'a'}");
-    NamespaceString nss = NamespaceString::createNamespaceString_forTest("a.collection");
-
-    auto aggReqDBName = unittest::assertGet(
-        aggregation_request_helper::parseFromBSONForTests(nss.dbName(), inputBSON));
-    auto aggReqNSS =
-        unittest::assertGet(aggregation_request_helper::parseFromBSONForTests(nss, inputBSON));
-
-    ASSERT_DOCUMENT_EQ(aggregation_request_helper::serializeToCommandDoc(aggReqDBName),
-                       aggregation_request_helper::serializeToCommandDoc(aggReqNSS));
+    parseNSHelper(validRequest, invalidCollectionName, ErrorCodes::InvalidNamespace);
 }
 
 TEST(AggregationRequestTest, ShouldRejectExchangeNotObject) {
@@ -720,8 +724,7 @@ TEST(AggregationRequestTest, ShouldRejectExchangeNotObject) {
         "exchange: {policy: 'roundrobin', consumers: 2},"
         "$db: 'a'}");
     const BSONObj nonObjectExchange = fromjson("{exchange: '42'}");
-    aggregationRequestParseFailureHelper(
-        nss, validRequest, nonObjectExchange, ErrorCodes::TypeMismatch);
+    aggregationRequestParseFailureHelper(validRequest, nonObjectExchange, ErrorCodes::TypeMismatch);
 }
 
 TEST(AggregationRequestTest, ShouldRejectExchangeInvalidSpec) {
@@ -733,9 +736,8 @@ TEST(AggregationRequestTest, ShouldRejectExchangeInvalidSpec) {
         "exchange: {policy: 'roundrobin', consumers: 2}, "
         "$db: 'a'}");
     const BSONObj invalidExchangeSpec = fromjson("{exchange: {}}");
-    auto missingFieldErrorCode = ErrorCodes::duplicateCodeForTest(40414);
     aggregationRequestParseFailureHelper(
-        nss, validRequest, invalidExchangeSpec, missingFieldErrorCode);
+        validRequest, invalidExchangeSpec, ErrorCodes::IDLFailedToParse);
 }
 
 TEST(AggregationRequestTest, ShouldRejectInvalidWriteConcern) {
@@ -747,7 +749,7 @@ TEST(AggregationRequestTest, ShouldRejectInvalidWriteConcern) {
         "writeConcern: {w: 'majority'}, "
         "$db: 'a'}");
     const BSONObj invalidWC = fromjson("{writeConcern: 'invalid'}");
-    aggregationRequestParseFailureHelper(nss, validRequest, invalidWC, ErrorCodes::TypeMismatch);
+    aggregationRequestParseFailureHelper(validRequest, invalidWC, ErrorCodes::TypeMismatch);
 }
 
 TEST(AggregationRequestTest, ShouldRejectInvalidCollectionUUID) {
@@ -759,7 +761,7 @@ TEST(AggregationRequestTest, ShouldRejectInvalidCollectionUUID) {
     const BSONObj validRequest = validRequestBuilder.done();
     const BSONObj invalidCollectionUUID = fromjson("{collectionUUID: 2}");
     aggregationRequestParseFailureHelper(
-        nss, validRequest, invalidCollectionUUID, ErrorCodes::TypeMismatch);
+        validRequest, invalidCollectionUUID, ErrorCodes::TypeMismatch);
 }
 
 //
@@ -776,8 +778,7 @@ TEST(AggregationRequestTest, ShouldRejectUnknownField) {
     const BSONObj nonObjectExchange =
         fromjson("{thisIsNotARealField: 'this is not a real option'}");
     auto unknownFieldErrorCode = ErrorCodes::duplicateCodeForTest(40415);
-    aggregationRequestParseFailureHelper(
-        nss, validRequest, nonObjectExchange, unknownFieldErrorCode);
+    aggregationRequestParseFailureHelper(validRequest, nonObjectExchange, unknownFieldErrorCode);
 }
 
 TEST(AggregationRequestTest, ShouldIgnoreQueryOptions) {
@@ -785,7 +786,37 @@ TEST(AggregationRequestTest, ShouldIgnoreQueryOptions) {
     const BSONObj inputBson = fromjson(
         "{aggregate: 'collection', pipeline: [{$match: {a: 'abc'}}], cursor: {}, $queryOptions: "
         "{}, $db: 'a'}");
-    ASSERT_OK(aggregation_request_helper::parseFromBSONForTests(nss, inputBson).getStatus());
+    ASSERT_OK(aggregation_request_helper::parseFromBSONForTests(inputBson).getStatus());
+}
+
+TEST(AggregationRequestTest, ShouldRejectRequestResumeTokenIfNonBooleanType) {
+    RAIIServerParameterControllerForTest featureFlagController("featureFlagReshardingImprovements",
+                                                               true);
+    NamespaceString nss = NamespaceString::createNamespaceString_forTest("a.collection");
+    const BSONObj validRequest = fromjson(
+        "{aggregate: 'collection',"
+        "pipeline: [],"
+        "$_requestResumeToken: true,"
+        "hint: {$natural: 1},"
+        "$db: 'a', "
+        "cursor: {}}");
+    const BSONObj nonBoolReshardingResumeToken = fromjson("{$_requestResumeToken: 'yes'}");
+    aggregationRequestParseFailureHelper(
+        validRequest, nonBoolReshardingResumeToken, ErrorCodes::TypeMismatch);
+}
+
+TEST(AggregationRequestTest, ShouldRejectRequestResumeTokenIfOplogNss) {
+    RAIIServerParameterControllerForTest featureFlagController("featureFlagReshardingImprovements",
+                                                               true);
+    const BSONObj validRequest = fromjson(
+        "{aggregate: 'collection',"
+        "pipeline: [],"
+        "$_requestResumeToken: true,"
+        "hint: {$natural: 1},"
+        "$db: 'a', "
+        "cursor: {}}");
+    const BSONObj oplogNss = fromjson("{aggregate: 'oplog.rs', $db: 'local'}");
+    aggregationRequestParseFailureHelper(validRequest, oplogNss, ErrorCodes::FailedToParse);
 }
 
 }  // namespace
