@@ -29,19 +29,35 @@
 
 #include "mongo/db/pipeline/change_stream_event_transform.h"
 
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <cstddef>
+#include <initializer_list>
+#include <utility>
+#include <vector>
+
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+
+#include "mongo/bson/bsontypes.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/db/database_name.h"
+#include "mongo/db/exec/document_value/document_metadata_fields.h"
+#include "mongo/db/namespace_string.h"
 #include "mongo/db/pipeline/change_stream_document_diff_parser.h"
-#include "mongo/db/pipeline/change_stream_filter_helpers.h"
 #include "mongo/db/pipeline/change_stream_helpers.h"
-#include "mongo/db/pipeline/change_stream_helpers_legacy.h"
 #include "mongo/db/pipeline/change_stream_preimage_gen.h"
-#include "mongo/db/pipeline/document_path_support.h"
-#include "mongo/db/pipeline/document_source_change_stream_add_post_image.h"
+#include "mongo/db/pipeline/document_source_change_stream.h"
 #include "mongo/db/pipeline/resume_token.h"
-#include "mongo/db/repl/bson_extract_optime.h"
 #include "mongo/db/repl/oplog_entry.h"
 #include "mongo/db/repl/oplog_entry_gen.h"
-#include "mongo/db/server_feature_flags_gen.h"
+#include "mongo/db/tenant_id.h"
 #include "mongo/db/update/update_oplog_entry_serialization.h"
+#include "mongo/db/update/update_oplog_entry_version.h"
+#include "mongo/idl/idl_parser.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/namespace_string_util.h"
+#include "mongo/util/str.h"
 
 namespace mongo {
 namespace {
@@ -49,7 +65,57 @@ constexpr auto checkValueType = &DocumentSourceChangeStream::checkValueType;
 constexpr auto checkValueTypeOrMissing = &DocumentSourceChangeStream::checkValueTypeOrMissing;
 constexpr auto resolveResumeToken = &change_stream::resolveResumeTokenFromSpec;
 
-Document copyDocExceptFields(const Document& source, const std::set<StringData>& fieldNames) {
+const StringDataSet kOpsWithoutUUID = {
+    DocumentSourceChangeStream::kInvalidateOpType,
+    DocumentSourceChangeStream::kDropDatabaseOpType,
+    DocumentSourceChangeStream::kEndOfTransactionOpType,
+};
+
+const StringDataSet kOpsWithoutNs = {
+    DocumentSourceChangeStream::kEndOfTransactionOpType,
+};
+
+// Possible collection types, for the "type" field returned by collection / view create events.
+enum class CollectionType {
+    kCollection,
+    kView,
+    kTimeseries,
+};
+
+// Stringification for CollectionType.
+StringData toString(CollectionType type) {
+    switch (type) {
+        case CollectionType::kCollection:
+            return "collection"_sd;
+        case CollectionType::kView:
+            return "view"_sd;
+        case CollectionType::kTimeseries:
+            return "timeseries"_sd;
+    }
+    MONGO_UNREACHABLE_TASSERT(8814200);
+}
+
+// Determine type of collection / view created, based on oplog entry payload.
+// Defaults to 'kCollection', and is changed to kView if "viewOn" field is set, except if "viewOn"
+// indicates that it is a timeseries collection. In the latter case kTimeseries is returned.
+CollectionType determineCollectionType(const Document& data, const DatabaseName& dbName) {
+    Value viewOn = data.getField("viewOn");
+    tassert(8814203,
+            "'viewOn' should either be missing or a non-empty string",
+            viewOn.missing() || viewOn.getType() == BSONType::String);
+    if (viewOn.missing()) {
+        return CollectionType::kCollection;
+    }
+    StringData viewOnNss = viewOn.getStringData();
+    tassert(8814204, "'viewOn' should be a non-empty string", !viewOnNss.empty());
+    if (NamespaceString nss = NamespaceStringUtil::deserialize(dbName, viewOnNss);
+        nss.isTimeseriesBucketsCollection()) {
+        return CollectionType::kTimeseries;
+    }
+    return CollectionType::kView;
+}
+
+Document copyDocExceptFields(const Document& source, std::initializer_list<StringData> fieldNames) {
     MutableDocument doc(source);
     for (auto fieldName : fieldNames) {
         doc.remove(fieldName);
@@ -66,7 +132,7 @@ repl::OpTypeEnum getOplogOpType(const Document& oplog) {
 Value makeChangeStreamNsField(const NamespaceString& nss) {
     // For certain types, such as dropDatabase, the collection name may be empty and should be
     // omitted. We never report the NamespaceString's tenantId in change stream events.
-    return Value(Document{{"db", nss.dbName().db()},
+    return Value(Document{{"db", nss.dbName().serializeWithoutTenantPrefix_UNSAFE()},
                           {"coll", (nss.coll().empty() ? Value() : Value(nss.coll()))}});
 }
 
@@ -82,7 +148,23 @@ void setResumeTokenForEvent(const ResumeTokenData& resumeTokenData, MutableDocum
 
 NamespaceString createNamespaceStringFromOplogEntry(Value tid, StringData ns) {
     auto tenantId = tid.missing() ? boost::none : boost::optional<TenantId>{tid.getOid()};
-    return NamespaceStringUtil::deserialize(tenantId, ns);
+    return NamespaceStringUtil::deserialize(tenantId, ns, SerializationContext::stateDefault());
+}
+
+void addTransactionIdFieldsIfPresent(const Document& input, MutableDocument& output) {
+    // The lsid and txnNumber may be missing if this is a batched write.
+    auto lsid = input[DocumentSourceChangeStream::kLsidField];
+    checkValueTypeOrMissing(lsid, DocumentSourceChangeStream::kLsidField, BSONType::Object);
+    auto txnNumber = input[DocumentSourceChangeStream::kTxnNumberField];
+    checkValueTypeOrMissing(
+        txnNumber, DocumentSourceChangeStream::kTxnNumberField, BSONType::NumberLong);
+    // We are careful here not to overwrite existing lsid or txnNumber fields with MISSING.
+    if (!txnNumber.missing()) {
+        output.addField(DocumentSourceChangeStream::kTxnNumberField, txnNumber);
+    }
+    if (!lsid.missing()) {
+        output.addField(DocumentSourceChangeStream::kLsidField, lsid);
+    }
 }
 
 }  // namespace
@@ -168,8 +250,14 @@ Document ChangeStreamDefaultEventTransformation::applyTransformation(const Docum
     Value fullDocument;
     Value updateDescription;
     Value documentKey;
+    // Used to populate the 'operationDescription' output field and also to build the resumeToken
+    // for some events. Note that any change to the 'operationDescription' for existing events can
+    // break changestream resumability between different mongod versions and should thus be avoided!
     Value operationDescription;
     Value stateBeforeChange;
+    // Optional value containing the namespace type for changestream create events. This will be
+    // emitted as 'nsType' field.
+    Value nsType;
 
     switch (opType) {
         case repl::OpTypeEnum::kInsert: {
@@ -208,18 +296,17 @@ Document ChangeStreamDefaultEventTransformation::applyTransformation(const Docum
                 if (_changeStreamSpec.getShowRawUpdateDescription()) {
                     updateDescription = input[repl::OplogEntry::kObjectFieldName];
                 } else {
-                    const auto showDisambiguatedPaths = _changeStreamSpec.getShowExpandedEvents() &&
-                        feature_flags::gFeatureFlagChangeStreamsFurtherEnrichedEvents.isEnabled(
-                            serverGlobalParams.featureCompatibility.acquireFCVSnapshot());
-                    const auto& deltaDesc = change_stream_document_diff_parser::parseDiff(
+                    auto deltaDesc = change_stream_document_diff_parser::parseDiff(
                         diffObj.getDocument().toBson());
 
-                    updateDescription = Value(Document{
-                        {"updatedFields", deltaDesc.updatedFields},
-                        {"removedFields", std::move(deltaDesc.removedFields)},
-                        {"truncatedArrays", std::move(deltaDesc.truncatedArrays)},
-                        {"disambiguatedPaths",
-                         showDisambiguatedPaths ? Value(deltaDesc.disambiguatedPaths) : Value()}});
+                    updateDescription =
+                        Value(Document{{"updatedFields", std::move(deltaDesc.updatedFields)},
+                                       {"removedFields", std::move(deltaDesc.removedFields)},
+                                       {"truncatedArrays", std::move(deltaDesc.truncatedArrays)},
+                                       {"disambiguatedPaths",
+                                        _changeStreamSpec.getShowExpandedEvents()
+                                            ? Value(std::move(deltaDesc.disambiguatedPaths))
+                                            : Value()}});
                 }
             } else if (!oplogVersion.missing() || id.missing()) {
                 // This is not a replacement op, and we did not see a valid update version number.
@@ -245,8 +332,7 @@ Document ChangeStreamDefaultEventTransformation::applyTransformation(const Docum
                 operationType = DocumentSourceChangeStream::kDropCollectionOpType;
 
                 // The "o.drop" field will contain the actual collection name.
-                nss = NamespaceStringUtil::parseNamespaceFromDoc(nss.dbName(),
-                                                                 nssField.getStringData());
+                nss = NamespaceStringUtil::deserialize(nss.dbName(), nssField.getStringData());
             } else if (auto nssField = oField.getField("renameCollection"); !nssField.missing()) {
                 operationType = DocumentSourceChangeStream::kRenameCollectionOpType;
 
@@ -276,39 +362,40 @@ Document ChangeStreamDefaultEventTransformation::applyTransformation(const Docum
                 nss = NamespaceString(nss.dbName());
             } else if (auto nssField = oField.getField("create"); !nssField.missing()) {
                 operationType = DocumentSourceChangeStream::kCreateOpType;
-                nss = NamespaceStringUtil::parseNamespaceFromDoc(nss.dbName(),
-                                                                 nssField.getStringData());
-                operationDescription = Value(copyDocExceptFields(oField, {"create"_sd}));
+                nss = NamespaceStringUtil::deserialize(nss.dbName(), nssField.getStringData());
+                Document opDesc = copyDocExceptFields(oField, {"create"_sd});
+                operationDescription = Value(opDesc);
+
+                if (_changeStreamSpec.getShowExpandedEvents()) {
+                    // Populate 'nsType' field with collection type (always "collection" here).
+                    auto collectionType = determineCollectionType(oField, nss.dbName());
+                    tassert(8814201,
+                            "'operationDescription.type' should always resolve to 'collection' for "
+                            "collection create events",
+                            collectionType == CollectionType::kCollection);
+                    nsType = Value(toString(collectionType));
+                }
             } else if (auto nssField = oField.getField("createIndexes"); !nssField.missing()) {
                 operationType = DocumentSourceChangeStream::kCreateIndexesOpType;
-                nss = NamespaceStringUtil::parseNamespaceFromDoc(nss.dbName(),
-                                                                 nssField.getStringData());
+                nss = NamespaceStringUtil::deserialize(nss.dbName(), nssField.getStringData());
                 // Wrap the index spec in an "indexes" array for consistency with commitIndexBuild.
                 auto indexSpec = Value(copyDocExceptFields(oField, {"createIndexes"_sd}));
                 operationDescription = Value(Document{{"indexes", std::vector<Value>{indexSpec}}});
             } else if (auto nssField = oField.getField("commitIndexBuild"); !nssField.missing()) {
                 operationType = DocumentSourceChangeStream::kCreateIndexesOpType;
-                nss = NamespaceStringUtil::parseNamespaceFromDoc(nss.dbName(),
-                                                                 nssField.getStringData());
-                operationDescription = Value(Document{{"indexes", oField.getField("indexes")}});
-            } else if (auto nssField = oField.getField("startIndexBuild"); !nssField.missing()) {
-                operationType = DocumentSourceChangeStream::kStartIndexBuildOpType;
-                nss = NamespaceStringUtil::parseNamespaceFromDoc(nss.dbName(),
-                                                                 nssField.getStringData());
+                nss = NamespaceStringUtil::deserialize(nss.dbName(), nssField.getStringData());
                 operationDescription = Value(Document{{"indexes", oField.getField("indexes")}});
             } else if (auto nssField = oField.getField("dropIndexes"); !nssField.missing()) {
                 const auto o2Field = input[repl::OplogEntry::kObject2FieldName].getDocument();
                 operationType = DocumentSourceChangeStream::kDropIndexesOpType;
-                nss = NamespaceStringUtil::parseNamespaceFromDoc(nss.dbName(),
-                                                                 nssField.getStringData());
+                nss = NamespaceStringUtil::deserialize(nss.dbName(), nssField.getStringData());
                 // Wrap the index spec in an "indexes" array for consistency with createIndexes
                 // and commitIndexBuild.
                 auto indexSpec = Value(copyDocExceptFields(o2Field, {"dropIndexes"_sd}));
                 operationDescription = Value(Document{{"indexes", std::vector<Value>{indexSpec}}});
             } else if (auto nssField = oField.getField("collMod"); !nssField.missing()) {
                 operationType = DocumentSourceChangeStream::kModifyOpType;
-                nss = NamespaceStringUtil::parseNamespaceFromDoc(nss.dbName(),
-                                                                 nssField.getStringData());
+                nss = NamespaceStringUtil::deserialize(nss.dbName(), nssField.getStringData());
                 operationDescription = Value(copyDocExceptFields(oField, {"collMod"_sd}));
 
                 const auto o2Field = input[repl::OplogEntry::kObject2FieldName].getDocument();
@@ -325,11 +412,7 @@ Document ChangeStreamDefaultEventTransformation::applyTransformation(const Docum
             break;
         }
         case repl::OpTypeEnum::kNoop: {
-            // TODO SERVER-66138: The legacy oplog format for some no-op operations can include
-            // 'type' field, which was removed post 6.0. We can remove all the logic related to the
-            // 'type' field once 7.0 is release.
-            const auto o2Field = change_stream_legacy::convertFromLegacyOplogFormat(
-                input[repl::OplogEntry::kObject2FieldName].getDocument(), nss);
+            const auto o2Field = input[repl::OplogEntry::kObject2FieldName].getDocument();
 
             // Check whether this is a shardCollection oplog entry.
             if (!o2Field["shardCollection"].missing()) {
@@ -363,7 +446,7 @@ Document ChangeStreamDefaultEventTransformation::applyTransformation(const Docum
             }
 
             if (!o2Field["migrateChunkToNewShard"].missing()) {
-                operationType = change_stream_legacy::getNewShardDetectedOpName(_expCtx);
+                operationType = DocumentSourceChangeStream::kNewShardDetectedOpType;
                 operationDescription =
                     Value(copyDocExceptFields(o2Field, {"migrateChunkToNewShard"_sd}));
                 break;
@@ -386,6 +469,13 @@ Document ChangeStreamDefaultEventTransformation::applyTransformation(const Docum
                 break;
             }
 
+            if (!o2Field["endOfTransaction"].missing()) {
+                operationType = DocumentSourceChangeStream::kEndOfTransactionOpType;
+                operationDescription = Value{copyDocExceptFields(o2Field, {"endOfTransaction"_sd})};
+                addTransactionIdFieldsIfPresent(o2Field, doc);
+                break;
+            }
+
             // We should never see an unknown noop entry.
             MONGO_UNREACHABLE_TASSERT(5052201);
         }
@@ -394,11 +484,10 @@ Document ChangeStreamDefaultEventTransformation::applyTransformation(const Docum
         }
     }
 
-    // UUID should always be present except for invalidate and dropDatabase entries.
-    if (operationType != DocumentSourceChangeStream::kInvalidateOpType &&
-        operationType != DocumentSourceChangeStream::kDropDatabaseOpType) {
-        invariant(!uuid.missing(), "Saw a CRUD op without a UUID");
-    }
+    // UUID should always be present except for a known set of types.
+    tassert(7826901,
+            "Saw a CRUD op without a UUID",
+            !uuid.missing() || kOpsWithoutUUID.contains(operationType));
 
     // Extract the 'txnOpIndex' and 'applyOpsIndex' fields. These will be missing unless we are
     // unwinding a transaction.
@@ -408,14 +497,7 @@ Document ChangeStreamDefaultEventTransformation::applyTransformation(const Docum
 
     // Add some additional fields only relevant to transactions.
     if (!txnOpIndex.missing()) {
-        // The lsid and txnNumber may be missing if this is a batched write.
-        auto lsid = input[DocumentSourceChangeStream::kLsidField];
-        checkValueTypeOrMissing(lsid, DocumentSourceChangeStream::kLsidField, BSONType::Object);
-        auto txnNumber = input[DocumentSourceChangeStream::kTxnNumberField];
-        checkValueTypeOrMissing(
-            txnNumber, DocumentSourceChangeStream::kTxnNumberField, BSONType::NumberLong);
-        doc.addField(DocumentSourceChangeStream::kTxnNumberField, txnNumber);
-        doc.addField(DocumentSourceChangeStream::kLsidField, lsid);
+        addTransactionIdFieldsIfPresent(input, doc);
     }
 
     // Generate the resume token. Note that only 'ts' is always guaranteed to be present.
@@ -457,8 +539,10 @@ Document ChangeStreamDefaultEventTransformation::applyTransformation(const Docum
         doc.addField(DocumentSourceChangeStream::kPreImageIdField, Value(preImageId.toBSON()));
     }
 
-    // Add the 'ns' field to the change stream document, based on the final value of 'nss'.
-    doc.addField(DocumentSourceChangeStream::kNamespaceField, makeChangeStreamNsField(nss));
+    // If needed, add the 'ns' field to the change stream document, based on the final value of nss.
+    if (!kOpsWithoutNs.contains(operationType)) {
+        doc.addField(DocumentSourceChangeStream::kNamespaceField, makeChangeStreamNsField(nss));
+    }
 
     // The event may have a documentKey OR an operationDescription, but not both. We already
     // validated this while creating the resume token.
@@ -477,6 +561,10 @@ Document ChangeStreamDefaultEventTransformation::applyTransformation(const Docum
     // For a 'modify' event we add the state before modification if appropriate.
     doc.addField(DocumentSourceChangeStream::kStateBeforeChangeField, stateBeforeChange);
 
+    if (!nsType.missing()) {
+        doc.addField(DocumentSourceChangeStream::kNsTypeField, nsType);
+    }
+
     return doc.freeze();
 }
 
@@ -503,13 +591,38 @@ Document ChangeStreamViewDefinitionEventTransformation::applyTransformation(
     Value tenantId = input[repl::OplogEntry::kTidFieldName];
 
     StringData operationType;
+    // Used to populate the 'operationDescription' output field and also to build the resumeToken
+    // for some events. Note that any change to the 'operationDescription' for existing events can
+    // break changestream resumability between different mongod versions and should thus be avoided!
     Value operationDescription;
+    // Optional value containing the namespace type for changestream create events. This will be
+    // emitted as 'nsType' field.
+    Value nsType;
 
+    // For views, we are transforming the DML operation on the system.views
+    // collection into a DDL event as follows:
+    // - insert into system.views is turned into a create (collection) event.
+    // - update in system.views is turned into a (collection) modify event.
+    // - delete in system.views is turned into a drop (collection) event.
     Document oField = input[repl::OplogEntry::kObjectFieldName].getDocument();
+
+    // The 'o._id' is the full namespace string of the view.
+    const auto nss = createNamespaceStringFromOplogEntry(tenantId, oField["_id"].getStringData());
+
     switch (opType) {
         case repl::OpTypeEnum::kInsert: {
             operationType = DocumentSourceChangeStream::kCreateOpType;
-            operationDescription = Value(copyDocExceptFields(oField, {"_id"_sd}));
+            Document opDesc = copyDocExceptFields(oField, {"_id"_sd});
+            operationDescription = Value(opDesc);
+
+            // Populate 'nsType' field with either "view" or "timeseries".
+            auto collectionType = determineCollectionType(oField, nss.dbName());
+            tassert(8814202,
+                    "'operationDescription.type' should always resolve to 'view' or 'timeseries' "
+                    "for view creation event",
+                    collectionType == CollectionType::kView ||
+                        collectionType == CollectionType::kTimeseries);
+            nsType = Value(toString(collectionType));
             break;
         }
         case repl::OpTypeEnum::kUpdate: {
@@ -545,11 +658,13 @@ Document ChangeStreamViewDefinitionEventTransformation::applyTransformation(
     doc.addField(DocumentSourceChangeStream::kWallTimeField,
                  input[repl::OplogEntry::kWallClockTimeFieldName]);
 
-    // The 'o._id' is the full namespace string of the view.
-    const auto nss = createNamespaceStringFromOplogEntry(tenantId, oField["_id"].getStringData());
     doc.addField(DocumentSourceChangeStream::kNamespaceField, makeChangeStreamNsField(nss));
     doc.addField(DocumentSourceChangeStream::kOperationDescriptionField, operationDescription);
 
+    if (!nsType.missing()) {
+        doc.addField(DocumentSourceChangeStream::kNsTypeField, nsType);
+    }
+
     return doc.freeze();
 }
 
