@@ -27,26 +27,36 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <vector>
 
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/bson/json.h"
+#include "mongo/bson/oid.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/db/database_name.h"
 #include "mongo/db/exec/document_value/document.h"
 #include "mongo/db/exec/document_value/document_value_test_util.h"
 #include "mongo/db/exec/document_value/value.h"
-#include "mongo/db/exec/document_value/value_comparator.h"
-#include "mongo/db/matcher/schema/expression_internal_schema_object_match.h"
-#include "mongo/db/multitenancy_gen.h"
-#include "mongo/db/pipeline/aggregation_context_fixture.h"
+#include "mongo/db/namespace_string.h"
 #include "mongo/db/pipeline/change_stream_event_transform.h"
-#include "mongo/db/pipeline/change_stream_rewrite_helpers.h"
+#include "mongo/db/pipeline/change_stream_helpers.h"
 #include "mongo/db/pipeline/change_stream_test_helpers.h"
-#include "mongo/db/pipeline/document_source.h"
 #include "mongo/db/pipeline/document_source_change_stream.h"
+#include "mongo/db/pipeline/expression_context_for_test.h"
 #include "mongo/db/repl/oplog_entry.h"
+#include "mongo/db/repl/oplog_entry_gen.h"
+#include "mongo/db/tenant_id.h"
 #include "mongo/idl/server_parameter_test_util.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/unittest/framework.h"
 #include "mongo/util/intrusive_counter.h"
-#include "mongo/util/uuid.h"
+#include "mongo/util/time_support.h"
 
 namespace mongo {
 namespace {
@@ -85,7 +95,7 @@ TEST(ChangeStreamEventTransformTest, TestDefaultUpdateTransform) {
         {DocumentSourceChangeStream::kCollectionUuidField, testUuid()},
         {DocumentSourceChangeStream::kWallTimeField, Date_t()},
         {DocumentSourceChangeStream::kNamespaceField,
-         Document{{"db", nss.db()}, {"coll", nss.coll()}}},
+         Document{{"db", nss.db_forTest()}, {"coll", nss.coll()}}},
         {DocumentSourceChangeStream::kDocumentKeyField, documentKey},
         {
             "updateDescription",
@@ -99,9 +109,40 @@ TEST(ChangeStreamEventTransformTest, TestDefaultUpdateTransform) {
     ASSERT_DOCUMENT_EQ(applyTransformation(updateField), expectedUpdateField);
 }
 
+TEST(ChangeStreamEventTransformTest, TestCreateCollectionTransform) {
+    const NamespaceString nss =
+        NamespaceString::createNamespaceString_forTest(boost::none, "testDB.coll.name");
+    // Namespace for the command, i.e. "testDB.$cmd".
+    const NamespaceString commandNss = NamespaceString::makeCommandNamespace(nss.dbName());
+    const auto opDescription = Value(fromjson("{idIndex: {v: 2, key: {_id: 1}, name: '_id_'}}"));
+    const auto idIndex = Value(fromjson("{v: 2, key: {_id: 1}, name: '_id_'}"));
+    auto oplogEntry = makeOplogEntry(repl::OpTypeEnum::kCommand,  // op type
+                                     commandNss,                  // namespace
+                                     BSON("create" << nss.coll() << "idIndex" << idIndex),  // o
+                                     testUuid(),                                            // uuid
+                                     boost::none,   // fromMigrate
+                                     boost::none);  // o2
+
+    Document expectedDoc{
+        {DocumentSourceChangeStream::kIdField,
+         makeResumeToken(
+             kDefaultTs, testUuid(), opDescription, DocumentSourceChangeStream::kCreateOpType)},
+        {DocumentSourceChangeStream::kOperationTypeField,
+         DocumentSourceChangeStream::kCreateOpType},
+        {DocumentSourceChangeStream::kClusterTimeField, kDefaultTs},
+        {DocumentSourceChangeStream::kCollectionUuidField, testUuid()},
+        {DocumentSourceChangeStream::kWallTimeField, Date_t()},
+        {DocumentSourceChangeStream::kNamespaceField,
+         Document{{"db", nss.db_forTest()}, {"coll", nss.coll()}}},
+        {DocumentSourceChangeStream::kOperationDescriptionField, opDescription},
+        {DocumentSourceChangeStream::kNsTypeField, "collection"_sd}};
+
+    ASSERT_DOCUMENT_EQ(applyTransformation(oplogEntry), expectedDoc);
+}
+
 TEST(ChangeStreamEventTransformTest, TestCreateViewTransform) {
-    const NamespaceString systemViewNss =
-        NamespaceString::makeSystemDotViewsNamespace({boost::none, "viewDB"});
+    const NamespaceString systemViewNss = NamespaceString::makeSystemDotViewsNamespace(
+        DatabaseName::createDatabaseName_forTest(boost::none, "viewDB"));
     const NamespaceString viewNss =
         NamespaceString::createNamespaceString_forTest(boost::none, "viewDB.view.name");
     const auto viewPipeline =
@@ -109,7 +150,7 @@ TEST(ChangeStreamEventTransformTest, TestCreateViewTransform) {
     const auto opDescription = Document{{"viewOn", "baseColl"_sd}, {"pipeline", viewPipeline}};
     auto oplogEntry = makeOplogEntry(repl::OpTypeEnum::kInsert,  // op type
                                      systemViewNss,              // namespace
-                                     BSON("_id" << viewNss.toString() << "viewOn"
+                                     BSON("_id" << viewNss.toString_forTest() << "viewOn"
                                                 << "baseColl"
                                                 << "pipeline" << viewPipeline),  // o
                                      testUuid(),                                 // uuid
@@ -125,23 +166,64 @@ TEST(ChangeStreamEventTransformTest, TestCreateViewTransform) {
         {DocumentSourceChangeStream::kClusterTimeField, kDefaultTs},
         {DocumentSourceChangeStream::kWallTimeField, Date_t()},
         {DocumentSourceChangeStream::kNamespaceField,
-         Document{{"db", viewNss.db()}, {"coll", viewNss.coll()}}},
-        {DocumentSourceChangeStream::kOperationDescriptionField, opDescription}};
+         Document{{"db", viewNss.db_forTest()}, {"coll", viewNss.coll()}}},
+        {DocumentSourceChangeStream::kOperationDescriptionField, opDescription},
+        {DocumentSourceChangeStream::kNsTypeField, "view"_sd}};
+
+    ASSERT_DOCUMENT_EQ(
+        applyTransformation(oplogEntry,
+                            NamespaceString::makeCollectionlessAggregateNSS(
+                                DatabaseName::createDatabaseName_forTest(boost::none, "viewDB"))),
+        expectedDoc);
+}
+
+TEST(ChangeStreamEventTransformTest, TestCreateTimeseriesTransform) {
+    const NamespaceString systemViewNss = NamespaceString::makeSystemDotViewsNamespace(
+        DatabaseName::createDatabaseName_forTest(boost::none, "timeseriesDB"));
+    const NamespaceString collNss =
+        NamespaceString::createNamespaceString_forTest("timeseriesDB"_sd, "timeseriesColl"_sd);
+    const NamespaceString viewNss = collNss.makeTimeseriesBucketsNamespace();
+    const auto viewPipeline = Value(
+        fromjson("[{_internalUnpackBucket: {timeField: 'time', bucketMaxSpanSeconds: 3600}}]"));
+    const auto opDescription = Document{{"viewOn", viewNss.coll()}, {"pipeline", viewPipeline}};
+    auto oplogEntry = makeOplogEntry(
+        repl::OpTypeEnum::kInsert,  // op type
+        NamespaceString::makeSystemDotViewsNamespace(
+            DatabaseName::createDatabaseName_forTest(boost::none, "timeseriesDB"_sd)),  // namespace
+        BSON("_id" << collNss.toString_forTest() << "viewOn" << viewNss.coll() << "pipeline"
+                   << viewPipeline),  // o
+        testUuid(),                   // uuid
+        boost::none,                  // fromMigrate
+        BSON("_id" << collNss.toString_forTest()));
+
+    Document expectedDoc{
+        {DocumentSourceChangeStream::kIdField,
+         makeResumeToken(
+             kDefaultTs, testUuid(), opDescription, DocumentSourceChangeStream::kCreateOpType)},
+        {DocumentSourceChangeStream::kOperationTypeField,
+         DocumentSourceChangeStream::kCreateOpType},
+        {DocumentSourceChangeStream::kClusterTimeField, kDefaultTs},
+        {DocumentSourceChangeStream::kWallTimeField, Date_t()},
+        {DocumentSourceChangeStream::kNamespaceField,
+         Document{{"db", viewNss.db_forTest()}, {"coll", collNss.coll()}}},
+        {DocumentSourceChangeStream::kOperationDescriptionField, opDescription},
+        {DocumentSourceChangeStream::kNsTypeField, "timeseries"_sd}};
 
     ASSERT_DOCUMENT_EQ(applyTransformation(oplogEntry,
                                            NamespaceString::makeCollectionlessAggregateNSS(
-                                               DatabaseName(boost::none, "viewDB"))),
+                                               DatabaseName::createDatabaseName_forTest(
+                                                   boost::none, "timeseriesDB"))),
                        expectedDoc);
 }
 
 TEST(ChangeStreamEventTransformTest, TestCreateViewOnSingleCollection) {
-    const NamespaceString systemViewNss =
-        NamespaceString::makeSystemDotViewsNamespace({boost::none, "viewDB"});
+    const NamespaceString systemViewNss = NamespaceString::makeSystemDotViewsNamespace(
+        DatabaseName::createDatabaseName_forTest(boost::none, "viewDB"));
     const NamespaceString viewNss =
         NamespaceString::createNamespaceString_forTest(boost::none, "viewDB.view.name");
     const auto viewPipeline =
         Value(fromjson("[{$match: {field: 'value'}}, {$project: {field: 1}}]"));
-    const auto document = BSON("_id" << viewNss.toString() << "viewOn"
+    const auto document = BSON("_id" << viewNss.toString_forTest() << "viewOn"
                                      << "baseColl"
                                      << "pipeline" << viewPipeline);
     const auto documentKey = Value(Document{{"_id", document["_id"]}});
@@ -163,7 +245,7 @@ TEST(ChangeStreamEventTransformTest, TestCreateViewOnSingleCollection) {
         {DocumentSourceChangeStream::kWallTimeField, Date_t()},
         {DocumentSourceChangeStream::kFullDocumentField, Document(document)},
         {DocumentSourceChangeStream::kNamespaceField,
-         Document{{"db", systemViewNss.db()}, {"coll", systemViewNss.coll()}}},
+         Document{{"db", systemViewNss.db_forTest()}, {"coll", systemViewNss.coll()}}},
         {DocumentSourceChangeStream::kDocumentKeyField, documentKey}};
 
     ASSERT_DOCUMENT_EQ(applyTransformation(oplogEntry), expectedDoc);
@@ -190,7 +272,7 @@ TEST(ChangeStreamEventTransformTest, TestUpdateTransformWithTenantId) {
         );
 
     Document expectedNamespace =
-        Document{{"db", nssWithTenant.dbName().db()}, {"coll", nssWithTenant.coll()}};
+        Document{{"db", nssWithTenant.dbName().toString_forTest()}, {"coll", nssWithTenant.coll()}};
 
     auto changeStreamDoc = applyTransformation(updateField, nssWithTenant);
     auto outputNs = changeStreamDoc[DocumentSourceChangeStream::kNamespaceField].getDocument();
@@ -227,20 +309,22 @@ TEST(ChangeStreamEventTransformTest, TestRenameTransformWithTenantId) {
     NamespaceString renameTo =
         NamespaceString::createNamespaceString_forTest(tenantId, "unittests.rename_coll");
 
-    auto renameField = makeOplogEntry(
-        repl::OpTypeEnum::kCommand,  // op type
-        renameFrom.getCommandNS(),   // namespace
-        BSON("renameCollection" << renameFrom.toString() << "to" << renameTo.toString()),  // o
-        testUuid()                                                                         // uuid
-    );
+    auto renameField =
+        makeOplogEntry(repl::OpTypeEnum::kCommand,  // op type
+                       renameFrom.getCommandNS(),   // namespace
+                       BSON("renameCollection" << renameFrom.toString_forTest() << "to"
+                                               << renameTo.toString_forTest()),  // o
+                       testUuid()                                                // uuid
+        );
 
-    Document expectedDoc{{DocumentSourceChangeStream::kNamespaceField,
-                          Document{{"db", renameFrom.dbName().db()}, {"coll", renameFrom.coll()}}},
-                         {DocumentSourceChangeStream::kRenameTargetNssField,
-                          Document{{"db", renameTo.dbName().db()}, {"coll", renameTo.coll()}}},
-                         {DocumentSourceChangeStream::kOperationDescriptionField,
-                          Document{BSON("to" << BSON("db" << renameTo.dbName().db() << "coll"
-                                                          << renameTo.coll()))}}};
+    Document expectedDoc{
+        {DocumentSourceChangeStream::kNamespaceField,
+         Document{{"db", renameFrom.dbName().toString_forTest()}, {"coll", renameFrom.coll()}}},
+        {DocumentSourceChangeStream::kRenameTargetNssField,
+         Document{{"db", renameTo.dbName().toString_forTest()}, {"coll", renameTo.coll()}}},
+        {DocumentSourceChangeStream::kOperationDescriptionField,
+         Document{BSON("to" << BSON("db" << renameTo.dbName().toString_forTest() << "coll"
+                                         << renameTo.coll()))}}};
 
     auto changeStreamDoc = applyTransformation(renameField, renameFrom);
     auto renameDoc = Document{
@@ -257,12 +341,13 @@ TEST(ChangeStreamEventTransformTest, TestRenameTransformWithTenantId) {
     // in the oplog entry. It should still not be a part of the db name in the change event.
     RAIIServerParameterControllerForTest featureFlagController("featureFlagRequireTenantID", true);
 
-    auto oplogEntry = makeOplogEntry(
-        repl::OpTypeEnum::kCommand,  // op type
-        renameFrom.getCommandNS(),   // namespace
-        BSON("renameCollection" << renameFrom.toString() << "to" << renameTo.toString()),  // o
-        testUuid()                                                                         // uuid
-    );
+    auto oplogEntry =
+        makeOplogEntry(repl::OpTypeEnum::kCommand,  // op type
+                       renameFrom.getCommandNS(),   // namespace
+                       BSON("renameCollection" << renameFrom.toString_forTest() << "to"
+                                               << renameTo.toString_forTest()),  // o
+                       testUuid()                                                // uuid
+        );
 
     changeStreamDoc = applyTransformation(oplogEntry, renameFrom);
     renameDoc = Document{
@@ -292,7 +377,7 @@ TEST(ChangeStreamEventTransformTest, TestDropDatabaseTransformWithTenantId) {
                                       testUuid()                   // uuid
     );
 
-    Document expectedNamespace = Document{{"db", dbToDrop.dbName().db()}};
+    Document expectedNamespace = Document{{"db", dbToDrop.dbName().toString_forTest()}};
 
     auto changeStreamDoc = applyTransformation(dropDbField, dbToDrop);
     auto outputNs = changeStreamDoc[DocumentSourceChangeStream::kNamespaceField].getDocument();
@@ -332,7 +417,7 @@ TEST(ChangeStreamEventTransformTest, TestCreateTransformWithTenantId) {
     );
 
     Document expectedNamespace =
-        Document{{"db", nssWithTenant.dbName().db()}, {"coll", nssWithTenant.coll()}};
+        Document{{"db", nssWithTenant.dbName().toString_forTest()}, {"coll", nssWithTenant.coll()}};
 
     auto changeStreamDoc = applyTransformation(createField, nssWithTenant);
     auto outputNs = changeStreamDoc[DocumentSourceChangeStream::kNamespaceField].getDocument();
@@ -363,8 +448,8 @@ TEST(ChangeStreamEventTransformTest, TestCreateViewTransformWithTenantId) {
 
     const auto tenantId = TenantId(OID::gen());
 
-    const NamespaceString systemViewNss =
-        NamespaceString::makeSystemDotViewsNamespace({tenantId, "viewDB"});
+    const NamespaceString systemViewNss = NamespaceString::makeSystemDotViewsNamespace(
+        DatabaseName::createDatabaseName_forTest(tenantId, "viewDB"));
     const NamespaceString viewNss =
         NamespaceString::createNamespaceString_forTest(tenantId, "viewDB.view.name");
     const auto viewPipeline =
@@ -372,12 +457,13 @@ TEST(ChangeStreamEventTransformTest, TestCreateViewTransformWithTenantId) {
     const auto opDescription = Document{{"viewOn", "baseColl"_sd}, {"pipeline", viewPipeline}};
     auto createView = makeOplogEntry(repl::OpTypeEnum::kInsert,  // op type
                                      systemViewNss,              // namespace
-                                     BSON("_id" << viewNss.toString() << "viewOn"
+                                     BSON("_id" << viewNss.toString_forTest() << "viewOn"
                                                 << "baseColl"
                                                 << "pipeline" << viewPipeline),  // o
                                      testUuid());                                // uuid
 
-    Document expectedNamespace = Document{{"db", viewNss.dbName().db()}, {"coll", viewNss.coll()}};
+    Document expectedNamespace =
+        Document{{"db", viewNss.dbName().toString_forTest()}, {"coll", viewNss.coll()}};
 
     auto changeStreamDoc = applyTransformation(
         createView, NamespaceString::makeCollectionlessAggregateNSS(viewNss.dbName()));
@@ -391,7 +477,7 @@ TEST(ChangeStreamEventTransformTest, TestCreateViewTransformWithTenantId) {
 
     auto oplogEntry = makeOplogEntry(repl::OpTypeEnum::kInsert,  // op type
                                      systemViewNss,              // namespace
-                                     BSON("_id" << viewNss.toString() << "viewOn"
+                                     BSON("_id" << viewNss.toString_forTest() << "viewOn"
                                                 << "baseColl"
                                                 << "pipeline" << viewPipeline),  // o
                                      testUuid());
