@@ -30,10 +30,13 @@
 #pragma once
 
 #include <boost/optional.hpp>
+#include <cstddef>
 #include <set>
 #include <string>
 
+#include "mongo/bson/bsonobj.h"
 #include "mongo/db/exec/document_value/document.h"
+#include "mongo/db/exec/document_value/document_metadata_fields.h"
 #include "mongo/db/pipeline/variables.h"
 
 namespace mongo {
@@ -43,8 +46,10 @@ namespace mongo {
  * This ensures that a parent field is ordered directly before its children.
  */
 struct PathComparator {
+    using is_transparent = void;
+
     /* Returns true if the lhs value should sort before the rhs, false otherwise. */
-    bool operator()(const std::string& lhs, const std::string& rhs) const;
+    bool operator()(StringData lhs, StringData rhs) const;
 };
 
 /**
@@ -129,10 +134,11 @@ struct DepsTracker {
                                                TruncateToRootLevel truncation);
 
     /**
-     * Returns a projection object covering the non-metadata dependencies tracked by this class, or
-     * empty BSONObj if the entire document is required. By default, the resulting project will
-     * include the full, dotted field names of the dependencies. If 'truncationBehavior' is set to
-     * TruncateToRootLevel::yes, the project will contain only the root-level field names.
+     * Returns a projection object covering the non-metadata dependencies tracked by this class,
+     * or empty BSONObj if the entire document is required. By default, the resulting project
+     * will include the full, dotted field names of the dependencies. If 'truncationBehavior' is
+     * set to TruncateToRootLevel::yes, the project will contain only the root-level field
+     * names.
      */
     BSONObj toProjectionWithoutMetadata(
         TruncateToRootLevel truncationBehavior = TruncateToRootLevel::no) const;
@@ -147,16 +153,6 @@ struct DepsTracker {
         return fields.empty() && !needWholeDocument && !_metadataDeps.any();
     }
 
-    /**
-     * Returns 'true' if any of the DepsTracker's variables appear in the passed 'ids' set.
-     */
-    bool hasVariableReferenceTo(const std::set<Variables::Id>& ids) const {
-        std::vector<Variables::Id> match;
-        std::set_intersection(
-            vars.begin(), vars.end(), ids.begin(), ids.end(), std::back_inserter(match));
-        return !match.empty();
-    }
-
     /**
      * Returns a value with bits set indicating the types of metadata not available to the
      * pipeline.
@@ -197,6 +193,16 @@ struct DepsTracker {
         return _metadataDeps;
     }
 
+    /**
+     * Return all of the search metadata dependencies.
+     */
+    QueryMetadataBitSet& searchMetadataDeps() {
+        return _searchMetadataDeps;
+    }
+    const QueryMetadataBitSet& searchMetadataDeps() const {
+        return _searchMetadataDeps;
+    }
+
     /**
      * Request that all metadata in the given QueryMetadataBitSet be added as dependencies. Throws a
      * UserException if any of the requested metadata fields have been marked as unavailable.
@@ -215,7 +221,6 @@ struct DepsTracker {
      */
     OrderedPathSet fields;
 
-    std::set<Variables::Id> vars;    // IDs of referenced variables.
     bool needWholeDocument = false;  // If true, ignore 'fields'; the whole document is needed.
 
     // The output of some operators (such as $sample and $rand) depends on a source of fresh random
@@ -227,9 +232,12 @@ private:
     // Represents all metadata not available to the pipeline.
     QueryMetadataBitSet _unavailableMetadata;
 
-    // Represents which metadata is used by the pipeline. This is populated while performing
-    // dependency analysis.
+    // Represents which metadata stored in collection is used by the pipeline. This is populated
+    // while performing dependency analysis.
     QueryMetadataBitSet _metadataDeps;
+    // Represents which search metadata is used by the pipeline. This is populated while performing
+    // dependency analysis.
+    QueryMetadataBitSet _searchMetadataDeps;
 };
 
 }  // namespace mongo
