@@ -28,27 +28,35 @@
  */
 
 
-#include "mongo/platform/basic.h"
+#include <absl/meta/type_traits.h>
 
-#include "mongo/db/pipeline/document_source.h"
+#include <absl/container/node_hash_map.h>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
 
+#include "mongo/base/initializer.h"
 #include "mongo/db/exec/document_value/value.h"
 #include "mongo/db/feature_compatibility_version_documentation.h"
 #include "mongo/db/matcher/expression_algo.h"
 #include "mongo/db/pipeline/change_stream_constants.h"
-#include "mongo/db/pipeline/document_source_add_fields.h"
+#include "mongo/db/pipeline/document_source.h"
 #include "mongo/db/pipeline/document_source_group.h"
-#include "mongo/db/pipeline/document_source_internal_shard_filter.h"
 #include "mongo/db/pipeline/document_source_match.h"
-#include "mongo/db/pipeline/document_source_project.h"
 #include "mongo/db/pipeline/document_source_redact.h"
-#include "mongo/db/pipeline/document_source_replace_root.h"
 #include "mongo/db/pipeline/document_source_sample.h"
-#include "mongo/db/pipeline/document_source_sequential_document_cache.h"
+#include "mongo/db/pipeline/document_source_single_document_transformation.h"
 #include "mongo/db/pipeline/expression_context.h"
-#include "mongo/db/pipeline/field_path.h"
+#include "mongo/db/pipeline/lite_parsed_document_source.h"
+#include "mongo/db/query/allowed_contexts.h"
+#include "mongo/db/query/explain_options.h"
 #include "mongo/db/query/plan_summary_stats_visitor.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/redaction.h"
+#include "mongo/util/duration.h"
 #include "mongo/util/string_map.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
@@ -66,7 +74,11 @@ DocumentSource::DocumentSource(const StringData stageName,
                                const intrusive_ptr<ExpressionContext>& pCtx)
     : pSource(nullptr), pExpCtx(pCtx), _commonStats(stageName.rawData()) {
     if (pExpCtx->shouldCollectDocumentSourceExecStats()) {
-        _commonStats.executionTime.emplace(0);
+        if (internalMeasureQueryExecutionTimeInNanoseconds.load()) {
+            _commonStats.executionTime.precision = QueryExecTimerPrecision::kNanos;
+        } else {
+            _commonStats.executionTime.precision = QueryExecTimerPrecision::kMillis;
+        }
     }
 }
 
@@ -133,14 +145,8 @@ list<intrusive_ptr<DocumentSource>> DocumentSource::parse(
             str::stream() << "Unrecognized pipeline stage name: '" << stageName << "'",
             it != parserMap.end());
 
-    uassert(
-        ErrorCodes::QueryFeatureNotAllowed,
-        str::stream() << stageName
-                      << " is not allowed in the current feature compatibility version. See "
-                      << feature_compatibility_version_documentation::kCompatibilityLink
-                      << " for more information.",
-        !expCtx->maxFeatureCompatibilityVersion || !it->second.featureFlag ||
-            it->second.featureFlag->isEnabledOnVersion(*expCtx->maxFeatureCompatibilityVersion));
+    auto& entry = it->second;
+    expCtx->throwIfFeatureFlagIsNotEnabledOnFCV(stageName, entry.featureFlag);
 
     return it->second.parser(stageSpec, expCtx);
 }
@@ -174,8 +180,8 @@ bool groupMatchSwapVerified(const DocumentSourceMatch& nextMatch,
 }
 
 /**
- * Returns 'true' if the given stage is an internal change stream stage that can appear in a
- * router (mongoS) pipeline, or 'false' otherwise.
+ * Returns 'true' if the given stage is an internal change stream stage that can appear in a router
+ * (mongoS) pipeline, or 'false' otherwise.
  */
 bool isChangeStreamRouterPipelineStage(StringData stageName) {
     return change_stream_constants::kChangeStreamRouterPipelineStages.contains(stageName);
@@ -191,8 +197,8 @@ bool DocumentSource::pushMatchBefore(Pipeline::SourceContainer::iterator itr,
     auto nextStageAsMatch = dynamic_cast<DocumentSourceMatch*>((*std::next(itr)).get());
     if (!nextStageAsMatch || nextStageAsMatch->isTextQuery()) {
         // We do not need to attempt this optimization if the $match contains a text search
-        // predicate because, in that scenario, $match is already required to be the first stage
-        // in the pipeline.
+        // predicate because, in that scenario, $match is already required to be the first stage in
+        // the pipeline.
         return false;
     }
 
@@ -200,8 +206,8 @@ bool DocumentSource::pushMatchBefore(Pipeline::SourceContainer::iterator itr,
     // 1) The next stage after 'this' is $match.
     // 2) The $match stage does not contain a text search predicate.
 
-    // TODO SERVER-55492: Remove the following workaround when there are rename checks for
-    // 'other' match expressions.
+    // TODO SERVER-55492: Remove the following workaround when there are rename checks for 'other'
+    // match expressions.
     if (isChangeStreamRouterPipelineStage(this->getSourceName())) {
         // Always move the $match stage ahead of internal change stream stages appearing in the
         // router (mongoS) pipeline, because they do not access or modify any paths in the input
@@ -231,16 +237,15 @@ bool DocumentSource::pushMatchBefore(Pipeline::SourceContainer::iterator itr,
                     nonRenameableMatchPart ? nonRenameableMatchPart->serializeToBSONForDebug()
                                            : BSONObj()));
 
-    // At this point we know that at least part of the $match expression can be moved
-    // ahead of 'this'. So, we erase the original $match and move that renameable part
-    // ahead of 'this' stage.
+    // At this point we know that at least part of the $match expression can be moved ahead of
+    // 'this'. So, we erase the original $match and move that renameable part ahead of 'this' stage.
     container->erase(std::next(itr));
     container->insert(itr, std::move(renameableMatchPart));
 
-    // If 'nonRenameableMatchPart' is not null, the 'renameableMatchPart' of the $match
-    // expression was only one component of the original $match. So, we need to create a
-    // new $match stage for the remaining 'nonRenameableMatchPart' and insert it after
-    // 'this' - effectively keeping it in its original position in the pipeline.
+    // If 'nonRenameableMatchPart' is not null, the 'renameableMatchPart' of the $match expression
+    // was only one component of the original $match. So, we need to create a new $match stage for
+    // the remaining 'nonRenameableMatchPart' and insert it after 'this' - effectively keeping it in
+    // its original position in the pipeline.
     if (nonRenameableMatchPart) {
         container->insert(std::next(itr), std::move(nonRenameableMatchPart));
     }
@@ -335,23 +340,5 @@ std::unique_ptr<LiteParsedDocumentSource> throwOnParseLite(NamespaceString nss,
 MONGO_INITIALIZER_GROUP(BeginDocumentSourceRegistration,
                         ("default"),
                         ("EndDocumentSourceRegistration"))
-// Any remaining work on the parserMap should be done before finishing DocumentSource Registration.
-MONGO_INITIALIZER_WITH_PREREQUISITES(EndDocumentSourceRegistration,
-                                     ("BeginDocumentSourceRegistration"))
-(InitializerContext*) {
-    auto searchStageNames = {
-        "$vectorSearch"_sd, "$search"_sd, "$searchMeta"_sd, "$listSearchIndexes"_sd};
-    for (auto stageName : searchStageNames) {
-        auto searchIt = parserMap.find(stageName);
-        // If the stage has not been registered at this point, register a parser that errors
-        // with a useful error message on parsing a search stage.
-        if (searchIt == parserMap.end()) {
-            LiteParsedDocumentSource::registerParser(stageName.toString(),
-                                                     throwOnParseLite,
-                                                     AllowedWithApiStrict::kAlways,
-                                                     AllowedWithClientType::kAny);
-            DocumentSource::registerParser(stageName.toString(), throwOnParse, boost::none);
-        }
-    }
-}
+MONGO_INITIALIZER_GROUP(EndDocumentSourceRegistration, ("BeginDocumentSourceRegistration"), ())
 }  // namespace mongo
