@@ -27,14 +27,32 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/pipeline/document_source_bucket_auto.h"
+// IWYU pragma: no_include "ext/alloc_traits.h"
+#include <boost/smart_ptr.hpp>
+// IWYU pragma: no_include "boost/container/detail/std_fwd.hpp"
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <cmath>
+#include <cstddef>
+#include <deque>
+#include <string>
 
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/db/exec/document_value/value_comparator.h"
 #include "mongo/db/pipeline/accumulation_statement.h"
+#include "mongo/db/pipeline/accumulator_for_bucket_auto.h"
+#include "mongo/db/pipeline/accumulator_multi.h"
+#include "mongo/db/pipeline/document_source_bucket_auto.h"
 #include "mongo/db/pipeline/expression_dependencies.h"
 #include "mongo/db/pipeline/lite_parsed_document_source.h"
-#include "mongo/db/stats/resource_consumption_metrics.h"
+#include "mongo/db/query/allowed_contexts.h"
+#include "mongo/db/sorter/sorter_stats.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/str.h"
 
 namespace mongo {
 
@@ -92,7 +110,7 @@ const char* DocumentSourceBucketAuto::getSourceName() const {
 
 DocumentSource::GetNextResult DocumentSourceBucketAuto::doGetNext() {
     if (!_populated) {
-        const auto populationResult = populateSorter();
+        auto populationResult = populateSorter();
         if (populationResult.isPaused()) {
             return populationResult;
         }
@@ -167,10 +185,15 @@ DocumentSource::GetNextResult DocumentSourceBucketAuto::populateSorter() {
         _sorter.reset(Sorter<Value, Document>::make(opts, comparator));
     }
 
+    long long position = 0;
     auto next = pSource->getNext();
     for (; next.isAdvanced(); next = pSource->getNext()) {
         auto nextDoc = next.releaseDocument();
-        _sorter->add(extractKey(nextDoc), nextDoc);
+        auto key = extractKey(nextDoc);
+
+        auto doc = Document{{AccumulatorN::kFieldNameOutput, Value(std::move(nextDoc))},
+                            {AccumulatorN::kFieldNameGeneratedSortKey, Value(position++)}};
+        _sorter->add(std::move(key), std::move(doc));
         ++_nDocuments;
     }
     return next;
@@ -215,9 +238,20 @@ void DocumentSourceBucketAuto::addDocumentToBucket(const pair<Value, Document>&
     const size_t numAccumulators = _accumulatedFields.size();
     for (size_t k = 0; k < numAccumulators; k++) {
         if (bucket._accums[k]->needsInput()) {
-            bucket._accums[k]->process(
-                _accumulatedFields[k].expr.argument->evaluate(entry.second, &pExpCtx->variables),
-                false);
+            bool isPositionalAccum = isPositionalAccumulator(bucket._accums[k]->getOpName());
+            auto value = entry.second.getField(AccumulatorN::kFieldNameOutput);
+            auto evaluated = _accumulatedFields[k].expr.argument->evaluate(value.getDocument(),
+                                                                           &pExpCtx->variables);
+            if (isPositionalAccum) {
+                auto wrapped = Value(Document{
+                    {AccumulatorN::kFieldNameGeneratedSortKey,
+                     entry.second.getField(AccumulatorN::kFieldNameGeneratedSortKey)},
+                    {AccumulatorN::kFieldNameOutput, std::move(evaluated)},
+                });
+                bucket._accums[k]->process(Value(std::move(wrapped)), false);
+            } else {
+                bucket._accums[k]->process(std::move(evaluated), false);
+            }
         }
     }
 }
@@ -227,10 +261,6 @@ void DocumentSourceBucketAuto::initializeBucketIteration() {
     invariant(_sorter);
     _sortedInput.reset(_sorter->done());
 
-    auto& metricsCollector = ResourceConsumption::MetricsCollector::get(pExpCtx->opCtx);
-    metricsCollector.incrementKeysSorted(_sorter->stats().numSorted());
-    metricsCollector.incrementSorterSpills(_sorter->stats().spilledRanges());
-
     _sorter.reset();
 
     // If there are no buckets, then we don't need to populate anything.
@@ -430,7 +460,7 @@ intrusive_ptr<DocumentSourceBucketAuto> DocumentSourceBucketAuto::create(
     return new DocumentSourceBucketAuto(pExpCtx,
                                         groupByExpression,
                                         numBuckets,
-                                        accumulationStatements,
+                                        std::move(accumulationStatements),
                                         granularityRounder,
                                         maxMemoryUsageBytes);
 }
@@ -465,11 +495,12 @@ boost::intrusive_ptr<Expression>& DocumentSourceBucketAuto::getMutableGroupByExp
     return _groupByExpression;
 }
 
-const std::vector<AccumulationStatement>& DocumentSourceBucketAuto::getAccumulatedFields() const {
+const std::vector<AccumulationStatement>& DocumentSourceBucketAuto::getAccumulationStatements()
+    const {
     return _accumulatedFields;
 }
 
-std::vector<AccumulationStatement>& DocumentSourceBucketAuto::getMutableAccumulatedFields() {
+std::vector<AccumulationStatement>& DocumentSourceBucketAuto::getMutableAccumulationStatements() {
     tassert(7020502,
             "Cannot change accumulated field expression once execution has begun in BucketAuto",
             !_populated);
@@ -519,8 +550,10 @@ intrusive_ptr<DocumentSource> DocumentSourceBucketAuto::createFromBson(
                     argument.type() == BSONType::Object);
 
             for (auto&& outputField : argument.embeddedObject()) {
-                auto stmt = AccumulationStatement::parseAccumulationStatement(
+                auto parsedStmt = AccumulationStatement::parseAccumulationStatement(
                     pExpCtx.get(), outputField, vps);
+                auto stmt =
+                    replaceAccumulationStatementForBucketAuto(pExpCtx.get(), std::move(parsedStmt));
                 stmt.expr.initializer = stmt.expr.initializer->optimize();
                 uassert(4544714,
                         "Can't refer to the group key in $bucketAuto",
@@ -543,8 +576,11 @@ intrusive_ptr<DocumentSource> DocumentSourceBucketAuto::createFromBson(
             "$bucketAuto requires 'groupBy' and 'buckets' to be specified",
             groupByExpression && numBuckets);
 
-    return DocumentSourceBucketAuto::create(
-        pExpCtx, groupByExpression, numBuckets.value(), accumulationStatements, granularityRounder);
+    return DocumentSourceBucketAuto::create(pExpCtx,
+                                            groupByExpression,
+                                            numBuckets.value(),
+                                            std::move(accumulationStatements),
+                                            granularityRounder);
 }
 
 }  // namespace mongo
