@@ -29,12 +29,37 @@
 
 #pragma once
 
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <cstdint>
+#include <exception>
+#include <memory>
+#include <set>
+#include <system_error>
+#include <utility>
+#include <vector>
+
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/db/exec/document_value/document.h"
+#include "mongo/db/exec/document_value/value.h"
 #include "mongo/db/pipeline/accumulation_statement.h"
 #include "mongo/db/pipeline/accumulator.h"
+#include "mongo/db/pipeline/dependencies.h"
 #include "mongo/db/pipeline/document_source.h"
 #include "mongo/db/pipeline/expression.h"
+#include "mongo/db/pipeline/expression_context.h"
 #include "mongo/db/pipeline/granularity_rounder.h"
+#include "mongo/db/pipeline/pipeline.h"
+#include "mongo/db/pipeline/stage_constraints.h"
+#include "mongo/db/pipeline/variables.h"
+#include "mongo/db/query/query_shape/serialization_options.h"
 #include "mongo/db/sorter/sorter.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/util/intrusive_counter.h"
 
 namespace mongo {
 
@@ -45,7 +70,7 @@ namespace mongo {
 class DocumentSourceBucketAuto final : public DocumentSource {
 public:
     static constexpr StringData kStageName = "$bucketAuto"_sd;
-    Value serialize(const SerializationOptions& opts = SerializationOptions{}) const final override;
+    Value serialize(const SerializationOptions& opts = SerializationOptions{}) const final;
 
     DepsTracker::State getDependencies(DepsTracker* deps) const final;
 
@@ -102,11 +127,11 @@ public:
     boost::intrusive_ptr<Expression> getGroupByExpression() const;
     boost::intrusive_ptr<Expression>& getMutableGroupByExpression();
     /**
-     * Returns the accumulated fields expressions. The mutable getter can be used to alter
+     * Returns the AccumulationStatements. The mutable getter can be used to alter
      * the expression, but should not be used after execution has begun.
      */
-    const std::vector<AccumulationStatement>& getAccumulatedFields() const;
-    std::vector<AccumulationStatement>& getMutableAccumulatedFields();
+    const std::vector<AccumulationStatement>& getAccumulationStatements() const;
+    std::vector<AccumulationStatement>& getMutableAccumulationStatements();
 
 protected:
     GetNextResult doGetNext() final;
