@@ -29,17 +29,40 @@
 
 #include "mongo/db/pipeline/document_source_change_stream_handle_topology_change.h"
 
-#include <algorithm>
+// IWYU pragma: no_include "ext/alloc_traits.h"
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <memory>
+#include <string>
+#include <utility>
 
+#include "mongo/bson/bsontypes.h"
+#include "mongo/client/read_preference.h"
+#include "mongo/db/api_parameters.h"
+#include "mongo/db/logical_time.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/pipeline/aggregate_command_gen.h"
+#include "mongo/db/pipeline/aggregation_request_helper.h"
 #include "mongo/db/pipeline/change_stream_topology_change_info.h"
 #include "mongo/db/pipeline/document_source_change_stream.h"
+#include "mongo/db/pipeline/document_source_change_stream_gen.h"
+#include "mongo/db/pipeline/process_interface/mongo_process_interface.h"
+#include "mongo/db/pipeline/resume_token.h"
 #include "mongo/db/pipeline/sharded_agg_helpers.h"
-#include "mongo/db/query/query_feature_flags_gen.h"
+#include "mongo/db/shard_id.h"
+#include "mongo/platform/compiler.h"
 #include "mongo/s/catalog/type_shard.h"
 #include "mongo/s/client/shard_registry.h"
 #include "mongo/s/grid.h"
 #include "mongo/s/query/establish_cursors.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/decorable.h"
 #include "mongo/util/fail_point.h"
+#include "mongo/util/serialization_context.h"
+#include "mongo/util/str.h"
 
 namespace mongo {
 namespace {
@@ -85,7 +108,8 @@ bool isShardConfigEvent(const Document& eventDoc) {
     // Check whether this event occurred on the config.shards collection.
     auto nsObj = eventDoc[DocumentSourceChangeStream::kNamespaceField];
     const bool isConfigDotShardsEvent = nsObj["db"_sd].getType() == BSONType::String &&
-        nsObj["db"_sd].getStringData() == NamespaceString::kConfigsvrShardsNamespace.db() &&
+        nsObj["db"_sd].getStringData() ==
+            NamespaceString::kConfigsvrShardsNamespace.db(omitTenant) &&
         nsObj["coll"_sd].getType() == BSONType::String &&
         nsObj["coll"_sd].getStringData() == NamespaceString::kConfigsvrShardsNamespace.coll();
 
@@ -221,15 +245,14 @@ BSONObj DocumentSourceChangeStreamHandleTopologyChange::createUpdatedCommandForN
     // Create a new shard command object containing the new resume token.
     auto shardCommand = replaceResumeTokenInCommand(resumeTokenForNewShard.toDocument());
 
-    auto* opCtx = pExpCtx->opCtx;
-    bool apiStrict = APIParameters::get(opCtx).getAPIStrict().value_or(false);
-
-    // Create the 'AggregateCommandRequest' object which will help in creating the parsed pipeline.
-    auto aggCmdRequest = aggregation_request_helper::parseFromBSON(
-        opCtx, pExpCtx->ns, shardCommand, boost::none, apiStrict);
+    tassert(7663502,
+            str::stream() << "SerializationContext on the expCtx should not be empty, with ns: "
+                          << pExpCtx->ns.toStringForErrorMsg(),
+            pExpCtx->serializationCtxt != SerializationContext::stateDefault());
 
     // Parse and optimize the pipeline.
-    auto pipeline = Pipeline::parse(aggCmdRequest.getPipeline(), pExpCtx);
+    auto pipeline = Pipeline::parseFromArray(
+        shardCommand[AggregateCommandRequest::kPipelineFieldName], pExpCtx);
     pipeline->optimizePipeline();
 
     // Split the full pipeline to get the shard pipeline.
@@ -265,7 +288,7 @@ BSONObj DocumentSourceChangeStreamHandleTopologyChange::replaceResumeTokenInComm
     pipeline[0] =
         Value(Document{{DocumentSourceChangeStream::kStageName, changeStreamStage.freeze()}});
     MutableDocument newCmd(std::move(originalCmd));
-    newCmd[AggregateCommandRequest::kPipelineFieldName] = Value(pipeline);
+    newCmd[AggregateCommandRequest::kPipelineFieldName] = Value(std::move(pipeline));
     return newCmd.freeze().toBson();
 }
 
