@@ -28,16 +28,25 @@
  */
 
 
-#include "mongo/platform/basic.h"
+#include <boost/move/utility_core.hpp>
+#include <utility>
 
-#include "mongo/db/pipeline/document_source_change_stream_transform.h"
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
 
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/db/basic_types.h"
+#include "mongo/db/namespace_string.h"
 #include "mongo/db/pipeline/change_stream_helpers.h"
-#include "mongo/db/pipeline/expression.h"
-#include "mongo/db/pipeline/lite_parsed_document_source.h"
+#include "mongo/db/pipeline/document_source_change_stream.h"
+#include "mongo/db/pipeline/document_source_change_stream_transform.h"
 #include "mongo/db/pipeline/resume_token.h"
-#include "mongo/db/repl/bson_extract_optime.h"
+#include "mongo/idl/idl_parser.h"
 #include "mongo/util/assert_util.h"
+#include "mongo/util/intrusive_counter.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
 
@@ -111,7 +120,7 @@ namespace {
 template <typename T>
 void serializeSpecField(BSONObjBuilder* builder,
                         const SerializationOptions& opts,
-                        const StringData& fieldName,
+                        StringData fieldName,
                         const boost::optional<T>& value) {
     if (value) {
         opts.serializeLiteral((*value).toBSON()).addToBsonObj(builder, fieldName);
@@ -121,7 +130,7 @@ void serializeSpecField(BSONObjBuilder* builder,
 template <>
 void serializeSpecField(BSONObjBuilder* builder,
                         const SerializationOptions& opts,
-                        const StringData& fieldName,
+                        StringData fieldName,
                         const boost::optional<Timestamp>& value) {
     if (value) {
         opts.serializeLiteral(*value).addToBsonObj(builder, fieldName);
@@ -131,7 +140,7 @@ void serializeSpecField(BSONObjBuilder* builder,
 template <typename T>
 void serializeSpecField(BSONObjBuilder* builder,
                         const SerializationOptions& opts,
-                        const StringData& fieldName,
+                        StringData fieldName,
                         const T& value) {
     opts.appendLiteral(builder, fieldName, value);
 }
@@ -139,7 +148,7 @@ void serializeSpecField(BSONObjBuilder* builder,
 template <>
 void serializeSpecField(BSONObjBuilder* builder,
                         const SerializationOptions& opts,
-                        const StringData& fieldName,
+                        StringData fieldName,
                         const mongo::OptionalBool& value) {
     if (value.has_value()) {
         opts.appendLiteral(builder, fieldName, value.value_or(true));
