@@ -27,30 +27,68 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
-#include <boost/intrusive_ptr.hpp>
+#include <algorithm>
+#include <cstddef>
+#include <deque>
 #include <memory>
+#include <string>
+#include <utility>
+#include <vector>
 
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/string_data.h"
 #include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
 #include "mongo/bson/bsonobj.h"
 #include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/crypto/encryption_fields_gen.h"
+#include "mongo/db/catalog/clustered_collection_options_gen.h"
+#include "mongo/db/catalog/collection.h"
 #include "mongo/db/catalog/collection_mock.h"
 #include "mongo/db/exec/collection_scan.h"
+#include "mongo/db/exec/collection_scan_common.h"
+#include "mongo/db/exec/document_value/document.h"
+#include "mongo/db/exec/document_value/document_metadata_fields.h"
 #include "mongo/db/exec/document_value/document_value_test_util.h"
+#include "mongo/db/exec/document_value/value.h"
+#include "mongo/db/exec/document_value/value_comparator.h"
+#include "mongo/db/exec/plan_stage.h"
 #include "mongo/db/exec/plan_stats.h"
+#include "mongo/db/exec/working_set.h"
+#include "mongo/db/matcher/expression.h"
+#include "mongo/db/matcher/expression_parser.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/pipeline/aggregation_context_fixture.h"
+#include "mongo/db/pipeline/document_source.h"
 #include "mongo/db/pipeline/document_source_change_stream_check_resumability.h"
 #include "mongo/db/pipeline/document_source_change_stream_ensure_resume_token_present.h"
+#include "mongo/db/pipeline/document_source_change_stream_gen.h"
 #include "mongo/db/pipeline/document_source_mock.h"
 #include "mongo/db/pipeline/expression_context.h"
-#include "mongo/db/pipeline/process_interface/stub_mongo_process_interface.h"
+#include "mongo/db/pipeline/expression_context_for_test.h"
 #include "mongo/db/pipeline/resume_token.h"
+#include "mongo/db/query/collation/collator_interface.h"
 #include "mongo/db/query/collation/collator_interface_mock.h"
-#include "mongo/db/service_context.h"
+#include "mongo/db/query/record_id_bound.h"
+#include "mongo/db/record_id.h"
 #include "mongo/db/storage/devnull/devnull_kv_engine.h"
-#include "mongo/unittest/death_test.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/db/storage/record_data.h"
+#include "mongo/db/storage/record_store.h"
+#include "mongo/db/storage/snapshot.h"
+#include "mongo/db/timeseries/timeseries_gen.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/intrusive_counter.h"
+#include "mongo/util/time_support.h"
 #include "mongo/util/uuid.h"
 
 using boost::intrusive_ptr;
@@ -62,35 +100,64 @@ static constexpr StringData kTestNs = "test.ns"_sd;
 
 class ChangeStreamOplogCursorMock : public SeekableRecordCursor {
 public:
-    ChangeStreamOplogCursorMock(std::deque<Record>* records) : _records(records) {}
+    ChangeStreamOplogCursorMock(std::list<Record>* records) : _records(records) {}
 
-    virtual ~ChangeStreamOplogCursorMock() {}
+    ~ChangeStreamOplogCursorMock() override {}
 
     boost::optional<Record> next() override {
         if (_records->empty()) {
             return boost::none;
         }
-        auto& next = _records->front();
-        _records->pop_front();
-        return next;
+
+        if (!_initialized) {
+            _initialized = true;
+            _it = _records->cbegin();
+        } else {
+            _it++;
+        }
+
+        if (_it == _records->cend()) {
+            _initialized = false;
+            return boost::none;
+        }
+
+        return *_it;
     }
 
     boost::optional<Record> seekExact(const RecordId& id) override {
-        return Record{};
+        boost::optional<Record> result;
+        result.emplace();
+        return result;
     }
-    boost::optional<Record> seekNear(const RecordId& id) override {
-        return boost::none;
+    boost::optional<Record> seek(const RecordId& start, BoundInclusion boundInclusion) override {
+        invariant(boundInclusion == BoundInclusion::kInclude);
+        for (auto&& it = _records->cbegin(); it != _records->cend(); it++) {
+            if (it->id >= start) {
+                _initialized = true;
+                _it = it;
+                return *it;
+            }
+        }
+        _initialized = false;
+        return {};
     }
+
     void save() override {}
+
+    void saveUnpositioned() override {
+        _initialized = false;
+    }
     bool restore(bool tolerateCappedRepositioning) override {
         return true;
     }
     void detachFromOperationContext() override {}
     void reattachToOperationContext(OperationContext* opCtx) override {}
-    void setSaveStorageCursorOnDetachFromOperationContext(bool) {}
+    void setSaveStorageCursorOnDetachFromOperationContext(bool) override {}
 
 private:
-    std::deque<Record>* _records;
+    bool _initialized = false;
+    std::list<Record>::const_iterator _it;
+    std::list<Record>* _records;
 };
 
 class ChangeStreamOplogCollectionMock : public CollectionMock {
@@ -135,8 +202,8 @@ public:
 
 private:
     // We retain the owned record queue here because cursors may be destroyed and recreated.
-    mutable std::deque<Record> _records;
-    std::deque<BSONObj> _data;
+    mutable std::list<Record> _records;
+    std::list<BSONObj> _data;
 
     // These are no-op structures which are required by the CollectionScan.
     std::unique_ptr<RecordStore> _recordStore;
@@ -205,7 +272,7 @@ protected:
         // If this is the first call to doGetNext, we must create the COLLSCAN.
         if (!_collScan) {
             _collScan = std::make_unique<CollectionScan>(
-                pExpCtx.get(), _collectionPtr, _params, &_ws, _filter.get());
+                pExpCtx.get(), &_collectionPtr, _params, &_ws, _filter.get());
         }
         while (true) {
             // If the next result is a pause, return it and don't collscan.
@@ -213,19 +280,36 @@ protected:
             if (nextResult.isPaused()) {
                 return nextResult;
             }
+
             // Otherwise, retrieve the document via the CollectionScan stage.
             auto id = WorkingSet::INVALID_ID;
             switch (_collScan->doWork(&id)) {
                 case PlanStage::IS_EOF:
+                    // The CollectionScan can immediately return EOF, in which case document in the
+                    // pipeline should not be returned.
+                    while (nextResult.isAdvanced()) {
+                        nextResult = DocumentSourceMock::doGetNext();
+                    }
                     invariant(nextResult.isEOF());
                     return nextResult;
                 case PlanStage::ADVANCED: {
                     // We need to restore the _id field which was removed when we added this
                     // entry into the oplog. This is like a stripped-down DSCSTransform stage.
                     MutableDocument mutableDoc{_ws.get(id)->doc.value()};
+
+                    // The CollectionScan may immediately start returning our document, which means
+                    // we have to skip forward to get the _id of the correct document at the same
+                    // timestamp.
+                    while (nextResult.isAdvanced() &&
+                           nextResult.getDocument()["ts"].getTimestamp() <
+                               _ws.get(id)->doc.value()["ts"].getTimestamp()) {
+                        nextResult = DocumentSourceMock::doGetNext();
+                    }
+                    ASSERT(nextResult.isAdvanced());
                     mutableDoc["_id"] = nextResult.getDocument()["_id"];
                     mutableDoc.metadata().setSortKey(nextResult.getDocument()["_id"], true);
-                    return mutableDoc.freeze();
+                    auto frozen = mutableDoc.freeze();
+                    return frozen;
                 }
                 case PlanStage::NEED_TIME:
                     continue;
@@ -265,7 +349,7 @@ protected:
     void addOplogEntryOnTestNS(ResumeTokenData tokenData) {
         _mock->push_back(Document{{"ns", kTestNs},
                                   {"ts", tokenData.clusterTime},
-                                  {"_id", ResumeToken(std::move(tokenData)).toDocument()}});
+                                  {"_id", ResumeToken(tokenData).toDocument()}});
     }
 
     /**
@@ -916,8 +1000,8 @@ TEST_F(CheckResumabilityTest,
 TEST_F(CheckResumabilityTest, ShouldIgnoreOplogAfterFirstDoc) {
     Timestamp oplogTimestamp(100, 1);
     Timestamp resumeTimestamp(100, 2);
-    Timestamp oplogFutureTimestamp(100, 3);
-    Timestamp docTimestamp(100, 4);
+    Timestamp docTimestamp(100, 3);
+    Timestamp oplogFutureTimestamp(100, 4);
 
     auto dsCheckResumability = createDSCheckResumability(resumeTimestamp);
     addOplogEntryOnOtherNS(oplogTimestamp);
