@@ -27,16 +27,24 @@
  *    it in the license file.
  */
 
-#include "mongo/bson/bsonmisc.h"
+#include "mongo/util/assert_util.h"
+
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+
+#include "mongo/base/status_with.h"
+#include "mongo/bson/json.h"
 #include "mongo/db/exec/document_value/document_value_test_util.h"
 #include "mongo/db/exec/document_value/value.h"
 #include "mongo/db/pipeline/aggregation_context_fixture.h"
 #include "mongo/db/pipeline/document_source_densify.h"
 #include "mongo/db/pipeline/document_source_mock.h"
 #include "mongo/db/pipeline/pipeline.h"
+#include "mongo/unittest/assert.h"
 #include "mongo/unittest/death_test.h"
-#include "mongo/unittest/unittest.h"
-#include <utility>
+#include "mongo/unittest/framework.h"
 
 namespace mongo {
 namespace {
@@ -57,6 +65,50 @@ Date_t makeDate(std::string dateStr) {
     return statusDate.getValue();
 }
 
+// Helper to verify that generated values relate correctly to the step.
+bool isOnStepRelativeTo(DensifyValue testVal, DensifyValue base, RangeStatement range) {
+    if (testVal.isDate()) {
+
+        auto date = testVal.getDate();
+        auto unit = range.getUnit().value();
+        long long step = range.getStep().coerceToLong();
+        auto baseDate = base.getDate();
+
+        // Months, quarters and years have variable lengths depending on leap days
+        // and days-in-a-month, so a step is not a constant number of milliseconds
+        // across all dates. For these units, we need to iterate through rather
+        // than performing a calculation with modulo. As long as `baseDate` is not
+        // a large number of steps away from the value we're testing (as is true in
+        // our usage with _current as the base), this should not be a performance
+        // issue.
+        if (unit == TimeUnit::month || unit == TimeUnit::quarter || unit == TimeUnit::year) {
+
+            Date_t steppedDate = baseDate;
+            while (steppedDate < date) {
+                steppedDate = dateAdd(steppedDate, unit, step, TimeZoneDatabase::utcZone());
+            }
+            return steppedDate == date;
+        } else {
+            // Steps with units smaller than one month are always constant sized
+            // (because unix time does not have leap seconds), so we can perform
+            // modulo arithmetic.
+            auto testMillis = date.toMillisSinceEpoch();
+            auto baseMillis = baseDate.toMillisSinceEpoch();
+            auto stepDurationInMillis =
+                dateAdd(Date_t::fromMillisSinceEpoch(0), unit, step, TimeZoneDatabase::utcZone())
+                    .toMillisSinceEpoch();
+            auto diff = testMillis - baseMillis;
+            return diff % stepDurationInMillis == 0;
+        }
+    } else if (testVal.isNumber()) {
+        Value diff =
+            uassertStatusOK(ExpressionSubtract::apply(testVal.getNumber(), base.getNumber()));
+        Value remainder = uassertStatusOK(ExpressionMod::apply(diff, range.getStep()));
+        return remainder.getDouble() == 0.0;
+    }
+    return false;
+}
+
 DEATH_TEST(DensifyGeneratorTest, ErrorsIfMinOverMax, "lower or equal to max") {
     Document doc{{"a", 1}};
     size_t counter = 0;
@@ -66,8 +118,10 @@ DEATH_TEST(DensifyGeneratorTest, ErrorsIfMinOverMax, "lower or equal to max") {
                  "path",
                  doc,
                  doc,
+                 boost::none,
                  ValueComparator(),
-                 &counter),
+                 &counter,
+                 false),
         AssertionException,
         5733303);
 }
@@ -81,8 +135,10 @@ DEATH_TEST(DensifyGeneratorTest, ErrorsIfStepIsZero, "be positive") {
                  "path",
                  doc,
                  doc,
+                 boost::none,
                  ValueComparator(),
-                 &counter),
+                 &counter,
+                 false),
         AssertionException,
         5733305);
 }
@@ -96,8 +152,10 @@ DEATH_TEST(DensifyGeneratorTest, ErrorsOnMixedValues, "same type") {
                  "path",
                  doc,
                  doc,
+                 boost::none,
                  ValueComparator(),
-                 &counter),
+                 &counter,
+                 false),
         AssertionException,
         5733300);
 }
@@ -111,8 +169,10 @@ DEATH_TEST(DensifyGeneratorTest, ErrorsIfFieldExistsInDocument, "cannot include
                  "path",
                  doc,
                  doc,
+                 boost::none,
                  ValueComparator(),
-                 &counter),
+                 &counter,
+                 false),
         AssertionException,
         5733306);
 }
@@ -130,8 +190,10 @@ DEATH_TEST(DensifyGeneratorTest, ErrorsIfFieldExistsButIsArray, "cannot include
                  "arr",
                  preservedFields,
                  doc,
+                 boost::none,
                  ValueComparator(),
-                 &counter),
+                 &counter,
+                 false),
         AssertionException,
         5733306);
 }
@@ -149,8 +211,10 @@ TEST(DensifyGeneratorTest, ErrorsIfFieldIsInArray) {
                  "arr.path",
                  preservedFields,
                  doc,
+                 boost::none,
                  ValueComparator(),
-                 &counter),
+                 &counter,
+                 false),
         AssertionException,
         5733307);
 }
@@ -164,8 +228,10 @@ TEST(DensifyGeneratorTest, ErrorsIfPrefixOfFieldExists) {
                  "a.b",
                  doc,
                  doc,
+                 boost::none,
                  ValueComparator(),
-                 &counter),
+                 &counter,
+                 false),
         AssertionException,
         5733308);
 }
@@ -179,8 +245,10 @@ TEST(DensifyGeneratorTest, GeneratesNumericDocumentCorrectly) {
                  "a",
                  Document(),
                  doc,
+                 boost::none,
                  ValueComparator(),
-                 &counter);
+                 &counter,
+                 false);
     ASSERT_FALSE(generator.done());
     Document docOne{{"a", 1}};
     ASSERT_DOCUMENT_EQ(docOne, generator.getNextDocument());
@@ -197,8 +265,10 @@ TEST(DensifyGeneratorTest, GeneratesNumericDocumentCorrectlyWithoutFinalDoc) {
                  "a",
                  Document(),
                  boost::none,
+                 boost::none,
                  ValueComparator(),
-                 &counter);
+                 &counter,
+                 false);
     ASSERT_FALSE(generator.done());
     Document docOne{{"a", 1}};
     ASSERT_DOCUMENT_EQ(docOne, generator.getNextDocument());
@@ -215,8 +285,10 @@ TEST(DensifyGeneratorTest, PreservesIncludeFields) {
                  "a",
                  preserveFields,
                  doc,
+                 boost::none,
                  ValueComparator(),
-                 &counter);
+                 &counter,
+                 false);
     ASSERT_FALSE(generator.done());
     Document docOne{{"b", 1}, {"c", 1}, {"a", 1}};
     ASSERT_DOCUMENT_EQ(docOne, generator.getNextDocument());
@@ -234,8 +306,10 @@ TEST(DensifyGeneratorTest, GeneratesNumberOfNumericDocumentsCorrectly) {
                  "a",
                  Document(),
                  doc,
+                 boost::none,
                  ValueComparator(),
-                 &counter);
+                 &counter,
+                 false);
     for (int curVal = 0; curVal < 10; curVal += 2) {
         ASSERT_FALSE(generator.done());
         Document nextDoc{{"a", curVal}};
@@ -256,8 +330,10 @@ TEST(DensifyGeneratorTest, WorksWithNonIntegerStepAndPreserveFields) {
                  "a",
                  preserveFields,
                  doc,
+                 boost::none,
                  ValueComparator(),
-                 &counter);
+                 &counter,
+                 false);
     for (double curVal = 0; curVal < 10; curVal += 1.3) {
         ASSERT_FALSE(generator.done());
         Document nextDoc{{"b", 1}, {"c", 1}, {"a", curVal}};
@@ -277,8 +353,10 @@ TEST(DensifyGeneratorTest, GeneratesOffsetFromMaxDocsCorrectly) {
                  "a",
                  Document(),
                  doc,
+                 boost::none,
                  ValueComparator(),
-                 &counter);
+                 &counter,
+                 false);
     for (int curVal = 1; curVal < 11; curVal += 2) {
         ASSERT_FALSE(generator.done());
         Document nextDoc{{"a", curVal}};
@@ -299,8 +377,10 @@ TEST(DensifyGeneratorTest, GeneratesAtDottedPathCorrectly) {
                  "a.c",
                  preservedFields,
                  doc,
+                 boost::none,
                  ValueComparator(),
-                 &counter);
+                 &counter,
+                 false);
     for (int curVal = 1; curVal < 11; curVal += 2) {
         ASSERT_FALSE(generator.done());
         Document nextDoc{{"a", Document{{"b", 1}, {"c", curVal}}}};
@@ -317,8 +397,10 @@ TEST(DensifyGeneratorTest, GeneratesAtDottedPathCorrectly) {
                          "a.c.d",
                          secondPreservedFields,
                          doc,
+                         boost::none,
                          ValueComparator(),
-                         &counter);
+                         &counter,
+                         false);
     for (int curVal = 1; curVal < 11; curVal += 2) {
         ASSERT_FALSE(generator.done());
         Document nextDoc{{"a", Document{{"b", 1}, {"c", Document{{"d", curVal}}}}}};
@@ -339,8 +421,10 @@ DEATH_TEST(DensifyGeneratorTest, FailsIfDatesAndUnitNotProvided, "date step") {
                                 "a",
                                 Document(),
                                 Document(),
+                                boost::none,
                                 ValueComparator(),
-                                &counter),
+                                &counter,
+                                false),
                        AssertionException,
                        5733501);
 }
@@ -353,8 +437,10 @@ DEATH_TEST(DensifyGeneratorTest, FailsIfNumberAndUnitProvided, "non-date") {
                  "a",
                  Document(),
                  Document(),
+                 boost::none,
                  ValueComparator(),
-                 &counter),
+                 &counter,
+                 false),
         AssertionException,
         5733506);
 }
@@ -369,8 +455,10 @@ DEATH_TEST(DensifyGeneratorTest, DateMinMustBeLessThanMax, "lower or equal to")
                                 "a",
                                 Document(),
                                 Document(),
+                                boost::none,
                                 ValueComparator(),
-                                &counter),
+                                &counter,
+                                false),
                        AssertionException,
                        5733502);
 }
@@ -385,8 +473,10 @@ DEATH_TEST(DensifyGeneratorTest, DateStepMustBeInt, "whole number") {
                                 "a",
                                 Document(),
                                 Document(),
+                                boost::none,
                                 ValueComparator(),
-                                &counter),
+                                &counter,
+                                false),
                        AssertionException,
                        5733505);
 }
@@ -403,8 +493,10 @@ TEST(DensifyGeneratorTest, GeneratesDatesBySecondCorrectly) {
                               "a",
                               Document(),
                               doc,
+                              boost::none,
                               ValueComparator(),
-                              &counter);
+                              &counter,
+                              false);
     for (int curVal = 1; curVal < 11; curVal += 2) {
         auto appendStr = std::to_string(curVal);
         appendStr.insert(appendStr.begin(), 2 - appendStr.length(), '0');
@@ -429,8 +521,10 @@ TEST(DensifyGeneratorTest, GeneratesDatesByHourCorrectly) {
                               "a",
                               Document(),
                               doc,
+                              boost::none,
                               ValueComparator(),
-                              &counter);
+                              &counter,
+                              false);
     for (int curVal = 1; curVal < 15; curVal += 2) {
         auto appendStr = std::to_string(curVal);
         appendStr.insert(appendStr.begin(), 2 - appendStr.length(), '0');
@@ -455,8 +549,10 @@ TEST(DensifyGeneratorTest, GeneratesDatesByMonthCorrectly) {
                               "a",
                               Document(),
                               doc,
+                              boost::none,
                               ValueComparator(),
-                              &counter);
+                              &counter,
+                              false);
     for (int curVal = 1; curVal < 10; curVal += 2) {
         auto appendStr = std::to_string(curVal);
         appendStr.insert(appendStr.begin(), 2 - appendStr.length(), '0');
@@ -1392,12 +1488,12 @@ TEST_F(DensifyCloneTest, InternalDensifyCanBeCloned) {
 
 TEST(DensifyStepTest, InternalDensifyIsOnStepWithNumbers) {
     auto range = RangeStatement(Value(2), Full(), boost::none);
-    ASSERT_TRUE(DensifyValue(Value(5)).isOnStepRelativeTo(Value(1), range));
+    ASSERT_TRUE(isOnStepRelativeTo(DensifyValue(Value(5)), Value(1), range));
 }
 
 TEST(DensifyStepTest, InternalDensifyIsOffStepWithNumbers) {
     auto range = RangeStatement(Value(2), Full(), boost::none);
-    ASSERT_FALSE(DensifyValue(Value(5)).isOnStepRelativeTo(Value(2), range));
+    ASSERT_FALSE(isOnStepRelativeTo(DensifyValue(Value(5)), Value(2), range));
 }
 
 TEST(DensifyStepTest, InternalDensifyIsOnStepWithSmallDateStep) {
@@ -1405,7 +1501,7 @@ TEST(DensifyStepTest, InternalDensifyIsOnStepWithSmallDateStep) {
     auto val = DensifyValue(makeDate("2021-01-01T00:00:05.000Z"));
     auto base = DensifyValue(makeDate("2021-01-01T00:00:01.000Z"));
 
-    ASSERT_TRUE(val.isOnStepRelativeTo(base, range));
+    ASSERT_TRUE(isOnStepRelativeTo(val, base, range));
 }
 
 TEST(DensifyStepTest, InternalDensifyIsOffStepWithSmallDateStep) {
@@ -1413,7 +1509,7 @@ TEST(DensifyStepTest, InternalDensifyIsOffStepWithSmallDateStep) {
     auto val = DensifyValue(makeDate("2021-01-01T00:00:05.000Z"));
     auto base = DensifyValue(makeDate("2021-01-01T00:00:02.000Z"));
 
-    ASSERT_FALSE(val.isOnStepRelativeTo(base, range));
+    ASSERT_FALSE(isOnStepRelativeTo(val, base, range));
 }
 
 TEST(DensifyStepTest, InternalDensifyIsOnStepWithLargeDateStep) {
@@ -1421,7 +1517,7 @@ TEST(DensifyStepTest, InternalDensifyIsOnStepWithLargeDateStep) {
     auto val = DensifyValue(makeDate("2021-05-01T00:00:00.000Z"));
     auto base = DensifyValue(makeDate("2021-01-01T00:00:00.000Z"));
 
-    ASSERT_TRUE(val.isOnStepRelativeTo(base, range));
+    ASSERT_TRUE(isOnStepRelativeTo(val, base, range));
 }
 
 TEST(DensifyStepTest, InternalDensifyIsOffStepWithLargeDateStep) {
@@ -1429,7 +1525,7 @@ TEST(DensifyStepTest, InternalDensifyIsOffStepWithLargeDateStep) {
     auto val = DensifyValue(makeDate("2021-05-01T00:00:00.000Z"));
     auto base = DensifyValue(makeDate("2021-01-01T00:00:00.000Z"));
 
-    ASSERT_FALSE(val.isOnStepRelativeTo(base, range));
+    ASSERT_FALSE(isOnStepRelativeTo(val, base, range));
 }
 
 TEST(DensifyStepTest, InternalDensifyIsOffStepForDaysWithLargeDateStep) {
@@ -1437,7 +1533,7 @@ TEST(DensifyStepTest, InternalDensifyIsOffStepForDaysWithLargeDateStep) {
     auto val = DensifyValue(makeDate("2021-05-03T00:00:00.000Z"));
     auto base = DensifyValue(makeDate("2021-01-01T00:00:00.000Z"));
 
-    ASSERT_FALSE(val.isOnStepRelativeTo(base, range));
+    ASSERT_FALSE(isOnStepRelativeTo(val, base, range));
 }
 
 TEST_F(DensifyRedactionTest, RedactionDateBounds) {
