@@ -29,8 +29,22 @@
 
 #pragma once
 
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <list>
+#include <memory>
+#include <string>
+#include <utility>
+
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/db/auth/privilege.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/pipeline/document_source.h"
 #include "mongo/db/pipeline/document_source_single_document_transformation.h"
+#include "mongo/db/pipeline/expression_context.h"
+#include "mongo/db/pipeline/lite_parsed_document_source.h"
 #include "mongo/db/query/projection_parser.h"
+#include "mongo/stdx/unordered_set.h"
 
 namespace mongo {
 
@@ -55,13 +69,12 @@ public:
         return true;
     }
 
-    bool isDocuments() const final {
+    /**
+     * Returns true as the desugared pipeline begins with a $queue stage.
+     */
+    bool startsWithQueue() const final {
         return true;
     }
-
-    bool allowedToPassthroughFromMongos() const final {
-        return false;
-    }
 };
 
 static const std::string kGenFieldName = "_tempDocumentsField"s;
