@@ -28,17 +28,31 @@
  */
 
 
-#include "mongo/platform/basic.h"
-
+#include <boost/cstdint.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+// IWYU pragma: no_include "cxxabi.h"
 #include <algorithm>
+#include <compare>
 #include <iterator>
+#include <mutex>
 #include <set>
+#include <utility>
 
-#include "mongo/db/curop.h"
+#include "mongo/base/error_codes.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
 #include "mongo/db/hasher.h"
 #include "mongo/db/pipeline/document_source_exchange.h"
 #include "mongo/db/storage/key_string.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/util/fail_point.h"
+#include "mongo/util/str.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
@@ -164,7 +178,7 @@ std::vector<std::string> Exchange::extractBoundaries(
             kb << "" << elem;
         }
 
-        KeyString::Builder key{KeyString::Version::V1, kb.obj(), ordering};
+        key_string::Builder key{key_string::Version::V1, kb.obj(), ordering};
         std::string keyStr{key.getBuffer(), key.getSize()};
 
         ret.emplace_back(std::move(keyStr));
@@ -185,8 +199,8 @@ std::vector<std::string> Exchange::extractBoundaries(
         kbMax << "" << MAXKEY;
     }
 
-    KeyString::Builder minKey{KeyString::Version::V1, kbMin.obj(), ordering};
-    KeyString::Builder maxKey{KeyString::Version::V1, kbMax.obj(), ordering};
+    key_string::Builder minKey{key_string::Version::V1, kbMin.obj(), ordering};
+    key_string::Builder maxKey{key_string::Version::V1, kbMax.obj(), ordering};
     StringData minKeyStr{minKey.getBuffer(), minKey.getSize()};
     StringData maxKeyStr{maxKey.getBuffer(), maxKey.getSize()};
 
@@ -424,7 +438,7 @@ size_t Exchange::getTargetConsumer(const Document& input) {
         ++counter;
     }
 
-    KeyString::Builder key{KeyString::Version::V1, kb.obj(), _ordering};
+    key_string::Builder key{key_string::Version::V1, kb.obj(), _ordering};
     std::string keyStr{key.getBuffer(), key.getSize()};
 
     // Binary search for the consumer id.
