@@ -27,26 +27,43 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
 
-#include "mongo/platform/basic.h"
+#include <boost/move/utility_core.hpp>
+#include <cstdint>
+#include <mutex>
+#include <utility>
 
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+
+#include "mongo/base/status_with.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/db/hasher.h"
+#include "mongo/db/namespace_string.h"
 #include "mongo/db/pipeline/aggregation_context_fixture.h"
 #include "mongo/db/pipeline/document_source_exchange.h"
 #include "mongo/db/pipeline/document_source_mock.h"
-#include "mongo/db/storage/key_string.h"
+#include "mongo/db/pipeline/expression_context_for_test.h"
+#include "mongo/db/pipeline/process_interface/stub_mongo_process_interface.h"
+#include "mongo/db/query/collation/collator_interface.h"
+#include "mongo/db/service_context.h"
 #include "mongo/executor/network_interface_factory.h"
+#include "mongo/executor/task_executor.h"
 #include "mongo/executor/thread_pool_task_executor.h"
+#include "mongo/idl/idl_parser.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/atomic_word.h"
 #include "mongo/platform/random.h"
-#include "mongo/unittest/temp_dir.h"
-#include "mongo/unittest/unittest.h"
-#include "mongo/util/clock_source_mock.h"
+#include "mongo/stdx/mutex.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
 #include "mongo/util/concurrency/thread_pool.h"
-#include "mongo/util/system_clock_source.h"
 #include "mongo/util/time_support.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
+
 
 namespace mongo {
 
@@ -86,12 +103,13 @@ struct ThreadInfo {
 };
 }  // namespace
 
-const NamespaceString kTestNss = NamespaceString("test.docSourceExchange"_sd);
+const NamespaceString kTestNss =
+    NamespaceString::createNamespaceString_forTest("test.docSourceExchange"_sd);
 
 class DocumentSourceExchangeTest : public AggregationContextFixture {
 protected:
     std::unique_ptr<executor::TaskExecutor> _executor;
-    virtual void setUp() override {
+    void setUp() override {
         getExpCtx()->mongoProcessInterface = std::make_shared<StubMongoProcessInterface>();
 
         auto net = executor::makeNetworkInterface("ExchangeTest");
@@ -104,7 +122,7 @@ protected:
         _executor->startup();
     }
 
-    virtual void tearDown() override {
+    void tearDown() override {
         _executor->shutdown();
         _executor.reset();
     }
@@ -138,7 +156,7 @@ protected:
     }
 
     auto parseSpec(const BSONObj& spec) {
-        IDLParserErrorContext ctx("internalExchange");
+        IDLParserContext ctx("internalExchange");
         return ExchangeSpec::parse(ctx, spec);
     }
 
@@ -146,7 +164,7 @@ protected:
         std::vector<ThreadInfo> threads;
         for (size_t idx = 0; idx < nConsumers; ++idx) {
             ServiceContext::UniqueClient client =
-                getServiceContext()->makeClient("exchange client");
+                getServiceContext()->getService()->makeClient("exchange client");
             ServiceContext::UniqueOperationContext opCtxOwned =
                 getServiceContext()->makeOperationContext(client.get());
             OperationContext* opCtx = opCtxOwned.get();
@@ -525,7 +543,8 @@ TEST_F(DocumentSourceExchangeTest, RandomExchangeNConsumerResourceYielding) {
     std::vector<ThreadInfo> threads;
 
     for (size_t idx = 0; idx < nConsumers; ++idx) {
-        ServiceContext::UniqueClient client = getServiceContext()->makeClient("exchange client");
+        ServiceContext::UniqueClient client =
+            getServiceContext()->getService()->makeClient("exchange client");
         ServiceContext::UniqueOperationContext opCtxOwned =
             getServiceContext()->makeOperationContext(client.get());
         OperationContext* opCtx = opCtxOwned.get();
