@@ -30,7 +30,11 @@
 #pragma once
 
 #include <boost/intrusive_ptr.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
 #include <boost/optional.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
 #include <list>
 #include <memory>
 
@@ -78,12 +82,12 @@ private:
                                 boost::optional<FieldPath> locationField,
                                 double distanceMultiplier);
 
-    ~DocumentSourceGeoNearCursor() = default;
+    ~DocumentSourceGeoNearCursor() override = default;
 
     /**
      * Transforms 'obj' into a Document, calculating the distance.
      */
-    Document transformDoc(Document&& obj) const override final;
+    Document transformDoc(Document&& obj) const final;
 
     // The output field in which to store the computed distance.
     FieldPath _distanceField;
