@@ -28,7 +28,30 @@
  */
 
 #include "mongo/db/pipeline/document_source_internal_all_collection_stats.h"
-#include "mongo/db/pipeline/document_source_project.h"
+
+#include <boost/smart_ptr.hpp>
+#include <iterator>
+#include <list>
+#include <type_traits>
+
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/db/exec/document_value/document.h"
+#include "mongo/db/matcher/expression.h"
+#include "mongo/db/pipeline/document_source_coll_stats.h"
+#include "mongo/db/pipeline/document_source_single_document_transformation.h"
+#include "mongo/db/pipeline/process_interface/mongo_process_interface.h"
+#include "mongo/db/pipeline/transformer_interface.h"
+#include "mongo/db/query/allowed_contexts.h"
+#include "mongo/idl/idl_parser.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/str.h"
 
 namespace mongo {
 
@@ -52,13 +75,14 @@ DocumentSource::GetNextResult DocumentSourceInternalAllCollectionStats::doGetNex
 
     while (!_catalogDocs->empty()) {
         BSONObj obj(std::move(_catalogDocs->front()));
-        NamespaceString nss(obj["ns"].String());
+        const auto nss = NamespaceStringUtil::parseFromStringExpectTenantIdInMultitenancyMode(
+            obj["ns"].String());
 
         _catalogDocs->pop_front();
 
         // Avoid computing stats for collections that do not match the absorbed filter on the 'ns'
         // field.
-        if (_absorbedMatch && !_absorbedMatch->getMatchExpression()->matchesBSON(std::move(obj))) {
+        if (_absorbedMatch && !_absorbedMatch->getMatchExpression()->matchesBSON(obj)) {
             continue;
         }
 
@@ -111,7 +135,7 @@ Pipeline::SourceContainer::iterator DocumentSourceInternalAllCollectionStats::do
             _absorbedMatch = std::move(splitMatch.second);
         } else {
             // We have already absorbed a $match. We need to join it with splitMatch.second.
-            _absorbedMatch->joinMatchWith(std::move(splitMatch.second));
+            _absorbedMatch->joinMatchWith(std::move(splitMatch.second), "$and"_sd);
         }
     }
 
