@@ -29,31 +29,68 @@
 
 #include "mongo/db/pipeline/document_source_lookup.h"
 
-#include "mongo/base/init.h"
-#include "mongo/db/catalog_shard_feature_flag_gen.h"
+#include <absl/container/node_hash_set.h>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+// IWYU pragma: no_include "ext/alloc_traits.h"
+#include <algorithm>
+#include <iterator>
+#include <list>
+#include <tuple>
+#include <type_traits>
+
+#include "mongo/base/init.h"  // IWYU pragma: keep
+#include "mongo/base/status_with.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/db/auth/action_type.h"
+#include "mongo/db/auth/resource_pattern.h"
+#include "mongo/db/database_name.h"
 #include "mongo/db/exec/document_value/document.h"
 #include "mongo/db/exec/document_value/value.h"
-#include "mongo/db/jsobj.h"
+#include "mongo/db/feature_flag.h"
+#include "mongo/db/field_ref.h"
+#include "mongo/db/matcher/expression.h"
 #include "mongo/db/matcher/expression_algo.h"
+#include "mongo/db/matcher/expression_parser.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/pipeline/aggregation_request_helper.h"
 #include "mongo/db/pipeline/document_path_support.h"
 #include "mongo/db/pipeline/document_source_documents.h"
 #include "mongo/db/pipeline/document_source_merge_gen.h"
 #include "mongo/db/pipeline/document_source_queue.h"
-#include "mongo/db/pipeline/document_source_sort.h"
+#include "mongo/db/pipeline/document_source_sequential_document_cache.h"
 #include "mongo/db/pipeline/expression.h"
 #include "mongo/db/pipeline/expression_context.h"
 #include "mongo/db/pipeline/expression_dependencies.h"
+#include "mongo/db/pipeline/process_interface/mongo_process_interface.h"
+#include "mongo/db/pipeline/sharded_agg_helpers_targeting_policy.h"
 #include "mongo/db/pipeline/sort_reorder_helpers.h"
 #include "mongo/db/pipeline/variable_validation.h"
+#include "mongo/db/query/allowed_contexts.h"
 #include "mongo/db/query/collation/collator_factory_interface.h"
+#include "mongo/db/query/explain_options.h"
+#include "mongo/db/query/plan_summary_stats.h"
 #include "mongo/db/query/query_knobs_gen.h"
 #include "mongo/db/stats/counters.h"
 #include "mongo/db/views/resolved_view.h"
+#include "mongo/idl/idl_parser.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/atomic_word.h"
 #include "mongo/platform/overflow_arithmetic.h"
+#include "mongo/s/database_version.h"
+#include "mongo/s/shard_version.h"
+#include "mongo/s/sharding_state.h"
+#include "mongo/s/stale_exception.h"
 #include "mongo/util/fail_point.h"
+#include "mongo/util/namespace_string_util.h"
+#include "mongo/util/str.h"
+#include "mongo/util/uuid.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
@@ -97,30 +134,42 @@ void lookupPipeValidator(const Pipeline& pipeline) {
 // {from: {db: "local", coll: "tenantMigration.oplogView"}, ...} .
 NamespaceString parseLookupFromAndResolveNamespace(const BSONElement& elem,
                                                    const DatabaseName& defaultDb) {
-    // The object syntax only works for 'cache.chunks.*', 'local.oplog.rs' which are not user
-    // namespaces so object type is omitted from the error message below.
+    // The object syntax only works for 'cache.chunks.*', 'local.oplog.rs', and
+    // 'local.tenantMigration.oplogViewwhich' which are not user namespaces so object type is
+    // omitted from the error message below.
     uassert(ErrorCodes::FailedToParse,
             str::stream() << "$lookup 'from' field must be a string, but found "
                           << typeName(elem.type()),
             elem.type() == BSONType::String || elem.type() == BSONType::Object);
 
     if (elem.type() == BSONType::String) {
-        return NamespaceStringUtil::parseNamespaceFromRequest(defaultDb, elem.valueStringData());
+        return NamespaceStringUtil::deserialize(defaultDb, elem.valueStringData());
     }
 
     // Valdate the db and coll names.
-    auto spec = NamespaceSpec::parse(
-        IDLParserContext{elem.fieldNameStringData(), false /* apiStrict */, defaultDb.tenantId()},
-        elem.embeddedObject());
-    auto nss = NamespaceStringUtil::parseNamespaceFromRequest(spec.getDb().value_or(DatabaseName()),
-                                                              spec.getColl().value_or(""));
+    const auto tenantId = defaultDb.tenantId();
+    const auto vts = tenantId
+        ? boost::make_optional(auth::ValidatedTenancyScopeFactory::create(
+              *tenantId, auth::ValidatedTenancyScopeFactory::TrustedForInnerOpMsgRequestTag{}))
+        : boost::none;
+    auto spec = NamespaceSpec::parse(IDLParserContext{elem.fieldNameStringData(),
+                                                      false /* apiStrict */,
+                                                      vts,
+                                                      tenantId,
+                                                      SerializationContext::stateDefault()},
+                                     elem.embeddedObject());
+    auto nss = NamespaceStringUtil::deserialize(spec.getDb().value_or(DatabaseName()),
+                                                spec.getColl().value_or(""));
+    // In the cases nss == config.collections and nss == config.chunks we can proceed with the
+    // lookup as the merge will be done on the config server
+    bool isConfigSvrSupportedCollection = nss == NamespaceString::kConfigsvrCollectionsNamespace ||
+        nss == NamespaceString::kConfigsvrChunksNamespace;
     uassert(
         ErrorCodes::FailedToParse,
         str::stream() << "$lookup with syntax {from: {db:<>, coll:<>},..} is not supported for db: "
                       << nss.dbName().toStringForErrorMsg() << " and coll: " << nss.coll(),
         nss.isConfigDotCacheDotChunks() || nss == NamespaceString::kRsOplogNamespace ||
-            nss == NamespaceString::kTenantMigrationOplogView ||
-            nss == NamespaceString::kConfigsvrCollectionsNamespace);
+            nss == NamespaceString::kTenantMigrationOplogView || isConfigSvrSupportedCollection);
     return nss;
 }
 
@@ -370,7 +419,7 @@ std::unique_ptr<DocumentSourceLookUp::LiteParsed> DocumentSourceLookUp::LitePars
         fromNss = parseLookupFromAndResolveNamespace(fromElement, nss.dbName());
     }
     uassert(ErrorCodes::InvalidNamespace,
-            str::stream() << "invalid $lookup namespace: " << fromNss.ns(),
+            str::stream() << "invalid $lookup namespace: " << fromNss.toStringForErrorMsg(),
             fromNss.isValid());
 
     // Recursively lite parse the nested pipeline, if one exists.
@@ -402,8 +451,7 @@ PrivilegeVector DocumentSourceLookUp::LiteParsed::requiredPrivileges(
     if (!_pipelines.empty()) {
         const LiteParsedPipeline& pipeline = _pipelines[0];
         Privilege::addPrivilegesToPrivilegeVector(
-            &requiredPrivileges,
-            std::move(pipeline.requiredPrivileges(isMongos, bypassDocumentValidation)));
+            &requiredPrivileges, pipeline.requiredPrivileges(isMongos, bypassDocumentValidation));
     }
 
     return requiredPrivileges;
@@ -419,7 +467,9 @@ const char* DocumentSourceLookUp::getSourceName() const {
 }
 
 bool DocumentSourceLookUp::foreignShardedLookupAllowed() const {
-    return !pExpCtx->opCtx->inMultiDocumentTransaction();
+    const auto fcvSnapshot = serverGlobalParams.mutableFCV.acquireFCVSnapshot();
+    return !pExpCtx->opCtx->inMultiDocumentTransaction() ||
+        gFeatureFlagAllowAdditionalParticipants.isEnabled(fcvSnapshot);
 }
 
 void DocumentSourceLookUp::determineSbeCompatibility() {
@@ -445,33 +495,23 @@ void DocumentSourceLookUp::determineSbeCompatibility() {
 
 StageConstraints DocumentSourceLookUp::constraints(Pipeline::SplitState pipeState) const {
     HostTypeRequirement hostRequirement;
+    bool nominateMergingShard = false;
     if (_fromNs.isConfigDotCacheDotChunks()) {
         // $lookup from config.cache.chunks* namespaces is permitted to run on each individual
-        // shard, rather than just the primary, since each shard should have an identical copy of
-        // the namespace.
+        // shard, rather than just a merging shard, since each shard should have an identical copy
+        // of the namespace.
         hostRequirement = HostTypeRequirement::kAnyShard;
     } else if (pipeState == Pipeline::SplitState::kSplitForShards) {
         // This stage will only be on the shards pipeline if $lookup on sharded foreign collections
         // is allowed.
         hostRequirement = HostTypeRequirement::kAnyShard;
-    } else if (_fromNs == NamespaceString::kConfigsvrCollectionsNamespace &&
-               // (Ignore FCV check): If the config shard feature flag is enabled, the config
-               // server should have the components necessary to handle a merge. Config servers are
-               // upgraded first and downgraded last, so if any server is running the latest binary,
-               // we can assume the conifg servers are too.
-               !gFeatureFlagCatalogShard.isEnabledAndIgnoreFCVUnsafe()) {
-        // This is an unsharded collection, but the primary shard would be the config server, and
-        // the config servers are not prepared to take queries. Instead, we'll merge on any of the
-        // other shards.
-        hostRequirement = HostTypeRequirement::kAnyShard;
+    } else if (_fromNs.isCollectionlessAggregateNS()) {
+        // When the inner pipeline does not target a collection, it can run on any node.
+        hostRequirement = HostTypeRequirement::kRunOnceAnyNode;
     } else {
-        // If the pipeline is unsplit or this stage is on the merging part of the pipeline,
-        // when $lookup on sharded foreign collections is allowed, the foreign collection is
-        // sharded, and the stage is executing on mongos, the stage can run on mongos or any shard.
-        hostRequirement = (foreignShardedLookupAllowed() && pExpCtx->inMongos &&
-                           pExpCtx->mongoProcessInterface->isSharded(pExpCtx->opCtx, _fromNs))
-            ? HostTypeRequirement::kNone
-            : HostTypeRequirement::kPrimaryShard;
+        // If the pipeline is unsplit, then this $lookup can run anywhere.
+        hostRequirement = HostTypeRequirement::kNone;
+        nominateMergingShard = pipeState == Pipeline::SplitState::kSplitForMerge;
     }
 
     // By default, $lookup is allowed in a transaction and does not use disk.
@@ -491,12 +531,48 @@ StageConstraints DocumentSourceLookUp::constraints(Pipeline::SplitState pipeStat
             _resolvedIntrospectionPipeline->getSources(), constraints);
     }
 
+    if (nominateMergingShard) {
+        constraints.mergeShardId = getMergeShardId();
+    }
+
     constraints.canSwapWithMatch = true;
     constraints.canSwapWithSkippingOrLimitingStage = !_unwindSrc;
 
     return constraints;
 }
 
+boost::optional<ShardId> DocumentSourceLookUp::computeMergeShardId() const {
+    // If this $lookup is on the merging half of the pipeline and the inner collection isn't
+    // sharded (that is, it is either unsplittable or untracked), then we should merge on the shard
+    // which owns the inner collection.
+    if (auto msi =
+            pExpCtx->mongoProcessInterface->determineSpecificMergeShard(pExpCtx->opCtx, _fromNs)) {
+        return msi;
+    }
+
+    // If we have not yet designated a merging shard, and are either executing on mongod, the
+    // foreign collection is unsharded, or sharded $lookup is not allowed, designate the current
+    // shard as the merging shard. This is done to prevent pushing this $lookup to the shards part
+    // of the pipeline. This is an important optimization designating as this $lookup  as a merging
+    // stage allows us to execute a single $lookup (as opposed to executing one $lookup on each
+    // involved shard). When this stage is part of a deeply nested pipeline, it  prevents creating
+    // an exponential explosion of cursors/resources (proportional to the level of pipeline
+    // nesting).
+    if (!(pExpCtx->inMongos && pExpCtx->mongoProcessInterface->isSharded(pExpCtx->opCtx, _fromNs) &&
+          foreignShardedLookupAllowed())) {
+        auto shardId = ShardingState::get(pExpCtx->opCtx)->shardId();
+        // If the command is executed on a mongos, we might get an empty shardId. We should return a
+        // shardId only if it is valid (non-empty).
+        if (shardId.isValid()) {
+            return shardId;
+        } else {
+            return boost::none;
+        }
+    }
+
+    return boost::none;
+}
+
 DocumentSource::GetNextResult DocumentSourceLookUp::doGetNext() {
     if (_unwindSrc) {
         return unwindResult();
@@ -513,18 +589,10 @@ DocumentSource::GetNextResult DocumentSourceLookUp::doGetNext() {
     // '_unwindSrc' would be non-null, and we would not have made it here.
     invariant(!_matchSrc);
 
-    if (hasLocalFieldForeignFieldJoin()) {
-        auto matchStage =
-            makeMatchStageFromInput(inputDoc, *_localField, _foreignField->fullPath(), BSONObj());
-        // We've already allocated space for the trailing $match stage in '_resolvedPipeline'.
-        _resolvedPipeline[*_fieldMatchPipelineIdx] = matchStage;
-    }
-
     std::unique_ptr<Pipeline, PipelineDeleter> pipeline;
     try {
-        pipeline = buildPipeline(inputDoc);
-        LOGV2_DEBUG(
-            9497000, 5, "Built pipeline", "pipeline"_attr = pipeline->serializeForLogging());
+        pipeline = buildPipeline(_fromExpCtx, inputDoc);
+        LOGV2_DEBUG(9497000, 5, "Built pipeline", "pipeline"_attr = pipeline->serializeToBson());
     } catch (const ExceptionForCat<ErrorCategory::StaleShardVersionError>& ex) {
         // If lookup on a sharded collection is disallowed and the foreign collection is sharded,
         // throw a custom exception.
@@ -579,7 +647,7 @@ std::unique_ptr<Pipeline, PipelineDeleter> DocumentSourceLookUp::buildPipelineFr
 
     // Resolve the view definition.
     auto pipeline = Pipeline::makePipelineFromViewDefinition(
-        _fromExpCtx, resolvedNamespace, serializedPipeline, opts);
+        _fromExpCtx, resolvedNamespace, std::move(serializedPipeline), opts);
 
     // Store the pipeline with resolved namespaces so that we only trigger this exception on the
     // first input document.
@@ -599,39 +667,57 @@ std::unique_ptr<Pipeline, PipelineDeleter> DocumentSourceLookUp::buildPipelineFr
     return pipeline;
 }
 
-std::unique_ptr<Pipeline, PipelineDeleter> DocumentSourceLookUp::buildPipeline(
-    const Document& inputDoc) {
+template <bool isStreamsEngine>
+PipelinePtr DocumentSourceLookUp::buildPipeline(
+    const boost::intrusive_ptr<ExpressionContext>& fromExpCtx, const Document& inputDoc) {
+    if (hasLocalFieldForeignFieldJoin()) {
+        BSONObj filter =
+            !_unwindSrc || hasPipeline() ? BSONObj() : _additionalFilter.value_or(BSONObj());
+        auto matchStage =
+            makeMatchStageFromInput(inputDoc, *_localField, _foreignField->fullPath(), filter);
+        // We've already allocated space for the trailing $match stage in '_resolvedPipeline'.
+        _resolvedPipeline[*_fieldMatchPipelineIdx] = matchStage;
+    }
+
     // Copy all 'let' variables into the foreign pipeline's expression context.
-    _variables.copyToExpCtx(_variablesParseState, _fromExpCtx.get());
-    _fromExpCtx->forcePlanCache = true;
+    _variables.copyToExpCtx(_variablesParseState, fromExpCtx.get());
+    fromExpCtx->forcePlanCache = true;
+
+    // Query settings are looked up after parsing and therefore are not populated in the
+    // 'fromExpCtx' as part of DocumentSourceLookUp constructor. Assign query settings to the
+    // 'fromExpCtx' by copying them from the parent query ExpressionContext.
+    fromExpCtx->setQuerySettingsIfNotPresent(getContext()->getQuerySettings());
 
     // Resolve the 'let' variables to values per the given input document.
-    resolveLetVariables(inputDoc, &_fromExpCtx->variables);
+    resolveLetVariables(inputDoc, &fromExpCtx->variables);
 
     std::unique_ptr<MongoProcessInterface::ScopedExpectUnshardedCollection>
         expectUnshardedCollectionInScope;
 
     const auto allowForeignShardedColl = foreignShardedLookupAllowed();
-    if (!allowForeignShardedColl) {
+    if (!allowForeignShardedColl && !fromExpCtx->inMongos) {
         // Enforce that the foreign collection must be unsharded for lookup.
         expectUnshardedCollectionInScope =
-            _fromExpCtx->mongoProcessInterface->expectUnshardedCollectionInScope(
-                _fromExpCtx->opCtx, _fromExpCtx->ns, boost::none);
+            fromExpCtx->mongoProcessInterface->expectUnshardedCollectionInScope(
+                fromExpCtx->opCtx, fromExpCtx->ns, boost::none);
     }
 
-    // If we don't have a cache, build and return the pipeline immediately.
-    if (!_cache || _cache->isAbandoned()) {
+    // If we don't have a cache, build and return the pipeline immediately. We don't support caching
+    // for the streams engine.
+    if (isStreamsEngine || !_cache || _cache->isAbandoned()) {
         MakePipelineOptions pipelineOpts;
         pipelineOpts.alreadyOptimized = false;
         pipelineOpts.optimize = true;
-        pipelineOpts.attachCursorSource = true;
+        // The streams engine attaches its own remote cursor source, so we don't need to do it here.
+        pipelineOpts.attachCursorSource = !isStreamsEngine;
         pipelineOpts.validator = lookupPipeValidator;
-        // By default, $lookup doesnt support sharded 'from' collections.
-        pipelineOpts.shardTargetingPolicy = allowForeignShardedColl
+        // By default, $lookup does not support sharded 'from' collections. The streams engine does
+        // not care about sharding, and so it does not allow shard targeting.
+        pipelineOpts.shardTargetingPolicy = !isStreamsEngine && allowForeignShardedColl
             ? ShardTargetingPolicy::kAllowed
             : ShardTargetingPolicy::kNotAllowed;
         try {
-            return Pipeline::makePipeline(_resolvedPipeline, _fromExpCtx, pipelineOpts);
+            return Pipeline::makePipeline(_resolvedPipeline, fromExpCtx, pipelineOpts);
         } catch (const ExceptionFor<ErrorCodes::CommandOnShardedViewNotSupportedOnMongod>& e) {
             // This exception returns the information we need to resolve a sharded view. Update the
             // pipeline with the resolved view definition.
@@ -644,11 +730,11 @@ std::unique_ptr<Pipeline, PipelineDeleter> DocumentSourceLookUp::buildPipeline(
                         "$lookup found view definition. ns: {namespace}, pipeline: {pipeline}. New "
                         "$lookup sub-pipeline: {new_pipe}",
                         logAttrs(e->getNamespace()),
-                        "pipeline"_attr = Pipeline::serializePipelineForLogging(e->getPipeline()),
-                        "new_pipe"_attr = Pipeline::serializePipelineForLogging(_resolvedPipeline));
+                        "pipeline"_attr = Value(e->getPipeline()),
+                        "new_pipe"_attr = _resolvedPipeline);
 
             // We can now safely optimize and reattempt attaching the cursor source.
-            pipeline = Pipeline::makePipeline(_resolvedPipeline, _fromExpCtx, pipelineOpts);
+            pipeline = Pipeline::makePipeline(_resolvedPipeline, fromExpCtx, pipelineOpts);
 
             return pipeline;
         }
@@ -661,7 +747,7 @@ std::unique_ptr<Pipeline, PipelineDeleter> DocumentSourceLookUp::buildPipeline(
     pipelineOpts.optimize = false;
     pipelineOpts.attachCursorSource = false;
     pipelineOpts.validator = lookupPipeValidator;
-    auto pipeline = Pipeline::makePipeline(_resolvedPipeline, _fromExpCtx, pipelineOpts);
+    auto pipeline = Pipeline::makePipeline(_resolvedPipeline, fromExpCtx, pipelineOpts);
 
     // We can store the unoptimized serialization of the pipeline so that if we need to resolve
     // a sharded view later on, and we have a local-foreign field join, we will need to update
@@ -675,13 +761,13 @@ std::unique_ptr<Pipeline, PipelineDeleter> DocumentSourceLookUp::buildPipeline(
         auto shardTargetingPolicy = allowForeignShardedColl ? ShardTargetingPolicy::kAllowed
                                                             : ShardTargetingPolicy::kNotAllowed;
         try {
-            pipeline = pExpCtx->mongoProcessInterface->attachCursorSourceToPipeline(
+            pipeline = pExpCtx->mongoProcessInterface->preparePipelineForExecution(
                 pipeline.release(), shardTargetingPolicy);
         } catch (const ExceptionFor<ErrorCodes::CommandOnShardedViewNotSupportedOnMongod>& e) {
             // This exception returns the information we need to resolve a sharded view. Update the
             // pipeline with the resolved view definition.
             pipeline = buildPipelineFromViewDefinition(
-                serializedPipeline,
+                std::move(serializedPipeline),
                 ExpressionContext::ResolvedNamespace{e->getNamespace(), e->getPipeline()});
 
             // The serialized pipeline does not have a cache stage, so we will add it back to the
@@ -695,11 +781,11 @@ std::unique_ptr<Pipeline, PipelineDeleter> DocumentSourceLookUp::buildPipeline(
                         "$lookup found view definition. ns: {namespace}, pipeline: {pipeline}. New "
                         "$lookup sub-pipeline: {new_pipe}",
                         logAttrs(e->getNamespace()),
-                        "pipeline"_attr = Pipeline::serializePipelineForLogging(e->getPipeline()),
-                        "new_pipe"_attr = Pipeline::serializePipelineForLogging(_resolvedPipeline));
+                        "pipeline"_attr = Value(e->getPipeline()),
+                        "new_pipe"_attr = _resolvedPipeline);
 
             // Try to attach the cursor source again.
-            pipeline = pExpCtx->mongoProcessInterface->attachCursorSourceToPipeline(
+            pipeline = pExpCtx->mongoProcessInterface->preparePipelineForExecution(
                 pipeline.release(), shardTargetingPolicy);
         }
     }
@@ -713,6 +799,13 @@ std::unique_ptr<Pipeline, PipelineDeleter> DocumentSourceLookUp::buildPipeline(
     return pipeline;
 }
 
+// Explicit instantiations for buildPipeline().
+template PipelinePtr DocumentSourceLookUp::buildPipeline<false /*isStreamsEngine*/>(
+    const boost::intrusive_ptr<ExpressionContext>& fromExpCtx, const Document& inputDoc);
+
+template PipelinePtr DocumentSourceLookUp::buildPipeline<true /*isStreamsEngine*/>(
+    const boost::intrusive_ptr<ExpressionContext>& fromExpCtx, const Document& inputDoc);
+
 /**
  * Method that looks for a DocumentSourceSequentialDocumentCache stage and calls optimizeAt() on
  * it if it has yet to be optimized.
@@ -749,6 +842,9 @@ void DocumentSourceLookUp::addCacheStageAndOptimize(Pipeline& pipeline) {
         // itself, therefore we are adding it to the end of the pipeline, and calling
         // optimizeContainer on the pipeline to ensure the rest of the pipeline is in its correct
         // order before optimizing the cache.
+        // TODO SERVER-84113: We will no longer have separate logic based on if a cache is present
+        // in doOptimizeAt(), so we can instead only add and optimize the cache after
+        // optimizeContainer is called.
         pipeline.addFinalSource(
             DocumentSourceSequentialDocumentCache::create(_fromExpCtx, _cache.get_ptr()));
 
@@ -783,8 +879,8 @@ Pipeline::SourceContainer::iterator DocumentSourceLookUp::doOptimizeAt(
         return container->end();
     }
 
-    // If the following stage is $sort and there is no internal $unwind, consider pushing it ahead
-    // of $lookup.
+    // If the following stage is $sort and this $lookup has not absorbed a following $unwind, try to
+    // move the $sort ahead of the $lookup.
     if (!_unwindSrc) {
         itr = tryReorderingWithSort(itr, container);
         if (*itr != this) {
@@ -794,13 +890,22 @@ Pipeline::SourceContainer::iterator DocumentSourceLookUp::doOptimizeAt(
 
     auto nextUnwind = dynamic_cast<DocumentSourceUnwind*>((*std::next(itr)).get());
 
-    // If we are not already handling an $unwind stage internally, we can combine with the
-    // following $unwind stage.
+    // If we are not already handling an $unwind stage internally and the following stage is an
+    // $unwind of the $lookup "as" output array, subsume the $unwind into the current $lookup as an
+    // $LU ($lookup + $unwind) macro stage. The combined stage acts like a SQL join (one result
+    // record per LHS x RHS match instead of one result per LHS with an array of its RHS matches).
+    //
+    // Ideally for simplicity in the stage builder we would not absorb the downstream $unwind if the
+    // lookup strategy is kNonExistentForeignCollection, but that is not determined until later and
+    // would be hard to do so here as it requires several inputs we do not have. It is also hard to
+    // move that determination earlier as it occurs in the deep stack under createLegacyExecutor().
     if (nextUnwind && !_unwindSrc && nextUnwind->getUnwindPath() == _as.fullPath()) {
+        if (nextUnwind->preserveNullAndEmptyArrays() || nextUnwind->indexPath()) {
+            downgradeSbeCompatibility(SbeCompatibility::notCompatible);
+        } else {
+            downgradeSbeCompatibility(SbeCompatibility::requiresTrySbe);
+        }
         _unwindSrc = std::move(nextUnwind);
-
-        // We cannot push absorbed $unwind stages into SBE.
-        _sbeCompatibility = SbeCompatibility::notCompatible;
         container->erase(std::next(itr));
         return itr;
     }
@@ -887,17 +992,13 @@ Pipeline::SourceContainer::iterator DocumentSourceLookUp::doOptimizeAt(
         return std::next(itr);
     }
 
-    // We can internalize the $match. This $lookup should already be marked as SBE incompatible
-    // because a $match can only be internalized if an $unwind, which is SBE incompatible, was
-    // absorbed as well.
-    tassert(5843701,
-            "This $lookup cannot be compatible with SBE",
-            _sbeCompatibility == SbeCompatibility::notCompatible);
+    // We cannot yet lower $LUM (combined $lookup + $unwind + $match) stages to SBE.
+    _sbeCompatibility = SbeCompatibility::notCompatible;
     if (!_matchSrc) {
         _matchSrc = nextMatch;
     } else {
         // We have already absorbed a $match. We need to join it with 'dependent'.
-        _matchSrc->joinMatchWith(nextMatch);
+        _matchSrc->joinMatchWith(nextMatch, "$and"_sd);
     }
 
     // Remove the original $match.
@@ -921,7 +1022,7 @@ Pipeline::SourceContainer::iterator DocumentSourceLookUp::doOptimizeAt(
     // There may be further optimization between this $lookup and the new neighbor, so we return an
     // iterator pointing to ourself.
     return itr;
-}
+}  // doOptimizeAt
 
 bool DocumentSourceLookUp::usedDisk() {
     if (_pipeline)
@@ -987,18 +1088,7 @@ DocumentSource::GetNextResult DocumentSourceLookUp::unwindResult() {
 
         _input = nextInput.releaseDocument();
 
-        if (hasLocalFieldForeignFieldJoin()) {
-            // At this point, if there is a pipeline, '_additionalFilter' was added to the end of
-            // '_resolvedPipeline' in doOptimizeAt(). If there is no pipeline, we must add it to the
-            // $match stage created here.
-            BSONObj filter = hasPipeline() ? BSONObj() : _additionalFilter.value_or(BSONObj());
-            auto matchStage =
-                makeMatchStageFromInput(*_input, *_localField, _foreignField->fullPath(), filter);
-            // We've already allocated space for the trailing $match stage in '_resolvedPipeline'.
-            _resolvedPipeline[*_fieldMatchPipelineIdx] = matchStage;
-        }
-
-        _pipeline = buildPipeline(*_input);
+        _pipeline = buildPipeline(_fromExpCtx, *_input);
 
         // The $lookup stage takes responsibility for disposing of its Pipeline, since it will
         // potentially be used by multiple OperationContexts, and the $lookup stage is part of an
@@ -1073,10 +1163,12 @@ void DocumentSourceLookUp::serializeToArray(std::vector<Value>& array,
     // Support alternative $lookup from config.cache.chunks* namespaces.
     //
     // Do not include the tenantId in serialized 'from' namespace.
-    auto fromValue = (pExpCtx->ns.db() == _fromNs.db())
+    auto fromValue = pExpCtx->ns.isEqualDb(_fromNs)
         ? Value(opts.serializeIdentifier(_fromNs.coll()))
-        : Value(Document{{"db", opts.serializeIdentifier(_fromNs.dbName().db())},
-                         {"coll", opts.serializeIdentifier(_fromNs.coll())}});
+        : Value(Document{
+              {"db",
+               opts.serializeIdentifier(_fromNs.dbName().serializeWithoutTenantPrefix_UNSAFE())},
+              {"coll", opts.serializeIdentifier(_fromNs.coll())}});
 
     MutableDocument output(Document{
         {getSourceName(), Document{{"from", fromValue}, {"as", opts.serializeFieldPath(_as)}}}});
@@ -1095,12 +1187,14 @@ void DocumentSourceLookUp::serializeToArray(std::vector<Value>& array,
         }
         if (opts.transformIdentifiers ||
             opts.literalPolicy != LiteralSerializationPolicy::kUnchanged) {
-            return Pipeline::parse(*_userPipeline, _fromExpCtx, lookupPipeValidator)
-                ->serializeToBson(opts);
+            // TODO SERVER-94227 we don't need to do any validation as part of this parsing pass.
+            return Pipeline::parse(*_userPipeline, _fromExpCtx)->serializeToBson(opts);
         }
         if (opts.serializeForQueryAnalysis) {
             // If we are in query analysis, encrypted fields will have been marked in the
             // introspection pipeline, so we need to serialize that here.
+            // TODO SERVER-81802 always serialize the resolved pipeline for non-query-shapification
+            // cases.
             return _resolvedIntrospectionPipeline->serializeToBson(opts);
         }
         return *_userPipeline;
@@ -1226,6 +1320,11 @@ boost::optional<DocumentSource::DistributedPlanLogic> DocumentSourceLookUp::dist
     // either choice will work correctly; we are simply applying a heuristic optimization.
     if (foreignShardedLookupAllowed() && pExpCtx->subPipelineDepth == 0 &&
         pExpCtx->mongoProcessInterface->isSharded(_fromExpCtx->opCtx, _fromNs)) {
+        tassert(
+            8725000,
+            "Should not attempt to nominate merging shard when $lookup is not acting as a merger",
+            !mergeShardId.isInitialized() ||
+                (mergeShardId.isInitialized() && getMergeShardId() == boost::none));
         return boost::none;
     }
 
@@ -1233,6 +1332,11 @@ boost::optional<DocumentSource::DistributedPlanLogic> DocumentSourceLookUp::dist
         // When $lookup reads from config.cache.chunks.* namespaces, it should run on each
         // individual shard in parallel. This is a special case, and atypical for standard $lookup
         // since a full copy of config.cache.chunks.* collections exists on all shards.
+        tassert(
+            8725001,
+            "Should not attempt to nominate merging shard when $lookup is not acting as a merger",
+            !mergeShardId.isInitialized() ||
+                (mergeShardId.isInitialized() && getMergeShardId() == boost::none));
         return boost::none;
     }
 
@@ -1246,13 +1350,9 @@ void DocumentSourceLookUp::detachFromOperationContext() {
         // use Pipeline::detachFromOperationContext() to take care of updating '_fromExpCtx->opCtx'.
         _pipeline->detachFromOperationContext();
         invariant(_fromExpCtx->opCtx == nullptr);
-    }
-    if (_fromExpCtx) {
+    } else if (_fromExpCtx) {
         _fromExpCtx->opCtx = nullptr;
     }
-    if (_resolvedIntrospectionPipeline) {
-        _resolvedIntrospectionPipeline->detachFromOperationContext();
-    }
 }
 
 void DocumentSourceLookUp::reattachToOperationContext(OperationContext* opCtx) {
@@ -1261,13 +1361,9 @@ void DocumentSourceLookUp::reattachToOperationContext(OperationContext* opCtx) {
         // use Pipeline::reattachToOperationContext() to take care of updating '_fromExpCtx->opCtx'.
         _pipeline->reattachToOperationContext(opCtx);
         invariant(_fromExpCtx->opCtx == opCtx);
-    }
-    if (_fromExpCtx) {
+    } else if (_fromExpCtx) {
         _fromExpCtx->opCtx = opCtx;
     }
-    if (_resolvedIntrospectionPipeline) {
-        _resolvedIntrospectionPipeline->reattachToOperationContext(opCtx);
-    }
 }
 
 bool DocumentSourceLookUp::validateOperationContext(const OperationContext* opCtx) const {
@@ -1277,15 +1373,9 @@ bool DocumentSourceLookUp::validateOperationContext(const OperationContext* opCt
 
     if (_pipeline) {
         const auto& sources = _pipeline->getSources();
-        if (!std::all_of(sources.begin(), sources.end(), [opCtx](const auto& s) {
-                return s->validateOperationContext(opCtx);
-            })) {
-            return false;
-        }
-    }
-    if (_resolvedIntrospectionPipeline &&
-        !_resolvedIntrospectionPipeline->validateOperationContext(opCtx)) {
-        return false;
+        return std::all_of(sources.begin(), sources.end(), [opCtx](const auto& s) {
+            return s->validateOperationContext(opCtx);
+        });
     }
 
     return true;
@@ -1349,7 +1439,7 @@ boost::intrusive_ptr<DocumentSource> DocumentSourceLookUp::createFromBson(
         }
     }
 
-    if (fromNs.ns().empty()) {
+    if (fromNs.isEmpty()) {
         validateLookupCollectionlessPipeline(pipeline);
         fromNs = NamespaceString::makeCollectionlessAggregateNSS(pExpCtx->ns.dbName());
     }
@@ -1407,5 +1497,4 @@ void DocumentSourceLookUp::addInvolvedCollections(
         stage->addInvolvedCollections(collectionNames);
     }
 }
-
 }  // namespace mongo
