@@ -27,17 +27,24 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/pipeline/document_source_match.h"
-
+#include <absl/container/flat_hash_map.h>
+// IWYU pragma: no_include "boost/container/detail/std_fwd.hpp"
 #include <algorithm>
+#include <cstddef>
+#include <iterator>
+#include <list>
 #include <memory>
+#include <type_traits>
+#include <vector>
+
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
 
 #include "mongo/bson/bsonobj.h"
 #include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
 #include "mongo/db/exec/document_value/document.h"
-#include "mongo/db/jsobj.h"
+#include "mongo/db/exec/document_value/document_metadata_fields.h"
 #include "mongo/db/matcher/expression_algo.h"
 #include "mongo/db/matcher/expression_array.h"
 #include "mongo/db/matcher/expression_leaf.h"
@@ -45,14 +52,28 @@
 #include "mongo/db/matcher/extensions_callback_noop.h"
 #include "mongo/db/matcher/match_expression_dependencies.h"
 #include "mongo/db/pipeline/document_path_support.h"
-#include "mongo/db/pipeline/expression.h"
+#include "mongo/db/pipeline/document_source_match.h"
 #include "mongo/db/pipeline/lite_parsed_document_source.h"
 #include "mongo/db/pipeline/semantic_analysis.h"
+#include "mongo/db/query/allowed_contexts.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/ctype.h"
-#include "mongo/util/str.h"
 
 namespace mongo {
 
+namespace {
+
+bool containsTextOperator(const MatchExpression& expr) {
+    if (expr.matchType() == MatchExpression::MatchType::TEXT)
+        return true;
+    for (auto child : expr) {
+        if (containsTextOperator(*child))
+            return true;
+    }
+    return false;
+}
+
+}  // namespace
 using boost::intrusive_ptr;
 using std::pair;
 using std::string;
@@ -64,6 +85,41 @@ REGISTER_DOCUMENT_SOURCE(match,
                          DocumentSourceMatch::createFromBson,
                          AllowedWithApiStrict::kAlways);
 
+DocumentSourceMatch::DocumentSourceMatch(std::unique_ptr<MatchExpression> expr,
+                                         const boost::intrusive_ptr<ExpressionContext>& expCtx)
+    : DocumentSource(kStageName, expCtx) {
+    auto bsonObj = expr->serialize();
+    rebuild(std::move(bsonObj), std::move(expr));
+}
+
+DocumentSourceMatch::DocumentSourceMatch(const BSONObj& query,
+                                         const intrusive_ptr<ExpressionContext>& expCtx)
+    : DocumentSource(kStageName, expCtx) {
+    rebuild(query);
+}
+
+void DocumentSourceMatch::rebuild(BSONObj predicate) {
+    predicate = predicate.getOwned();
+    SbeCompatibility originalSbeCompatibility =
+        std::exchange(pExpCtx->sbeCompatibility, SbeCompatibility::noRequirements);
+    ON_BLOCK_EXIT([&] { pExpCtx->sbeCompatibility = originalSbeCompatibility; });
+    std::unique_ptr<MatchExpression> expr = uassertStatusOK(MatchExpressionParser::parse(
+        predicate, pExpCtx, ExtensionsCallbackNoop(), Pipeline::kAllowedMatcherFeatures));
+    _sbeCompatibility = pExpCtx->sbeCompatibility;
+    rebuild(std::move(predicate), std::move(expr));
+}
+
+void DocumentSourceMatch::rebuild(BSONObj predicate, std::unique_ptr<MatchExpression> expr) {
+    invariant(predicate.isOwned());
+    _predicate = std::move(predicate);
+    _isTextQuery = containsTextOperator(*expr);
+    DepsTracker dependencies =
+        DepsTracker(_isTextQuery ? DepsTracker::kAllMetadata & ~DepsTracker::kOnlyTextScore
+                                 : DepsTracker::kAllMetadata);
+    getDependencies(expr.get(), &dependencies);
+    _matchProcessor.emplace(MatchProcessor(std::move(expr), std::move(dependencies)));
+}
+
 const char* DocumentSourceMatch::getSourceName() const {
     return kStageName.rawData();
 }
@@ -71,7 +127,8 @@ const char* DocumentSourceMatch::getSourceName() const {
 Value DocumentSourceMatch::serialize(const SerializationOptions& opts) const {
     if (opts.verbosity || opts.transformIdentifiers ||
         opts.literalPolicy != LiteralSerializationPolicy::kUnchanged) {
-        return Value(DOC(getSourceName() << Document(_expression->serialize(opts))));
+        return Value(
+            DOC(getSourceName() << Document(_matchProcessor->getExpression()->serialize(opts))));
     }
     return Value(DOC(getSourceName() << Document(getQuery())));
 }
@@ -81,7 +138,8 @@ intrusive_ptr<DocumentSource> DocumentSourceMatch::optimize() {
         return nullptr;
     }
 
-    _expression = MatchExpression::optimize(std::move(_expression));
+    _matchProcessor->setExpression(MatchExpression::optimize(
+        std::move(_matchProcessor->getExpression()), /* enableSimplification */ false));
 
     return this;
 }
@@ -92,15 +150,7 @@ DocumentSource::GetNextResult DocumentSourceMatch::doGetNext() {
 
     auto nextInput = pSource->getNext();
     for (; nextInput.isAdvanced(); nextInput = pSource->getNext()) {
-        // MatchExpression only takes BSON documents, so we have to make one. As an optimization,
-        // only serialize the fields we need to do the match. Specify BSONObj::LargeSizeTrait so
-        // that matching against a large document mid-pipeline does not throw a BSON max-size error.
-        BSONObj toMatch = _dependencies.needWholeDocument
-            ? nextInput.getDocument().toBson<BSONObj::LargeSizeTrait>()
-            : document_path_support::documentToBsonWithPaths<BSONObj::LargeSizeTrait>(
-                  nextInput.getDocument(), _dependencies.fields);
-
-        if (_expression->matchesBSON(toMatch)) {
+        if (_matchProcessor->process(nextInput.getDocument())) {
             return nextInput;
         }
 
@@ -134,7 +184,7 @@ Pipeline::SourceContainer::iterator DocumentSourceMatch::doOptimizeAt(
         invariant(!nextMatch->_isTextQuery);
 
         // Merge 'nextMatch' into this stage.
-        joinMatchWith(nextMatch);
+        joinMatchWith(nextMatch, "$and"_sd);
 
         // Erase 'nextMatch'.
         container->erase(std::next(itr));
@@ -381,9 +431,13 @@ bool DocumentSourceMatch::isTextQuery(const BSONObj& query) {
     return false;
 }
 
-void DocumentSourceMatch::joinMatchWith(intrusive_ptr<DocumentSourceMatch> other) {
+void DocumentSourceMatch::joinMatchWith(intrusive_ptr<DocumentSourceMatch> other,
+                                        StringData joinPred) {
+    invariant(joinPred == "$and"_sd || joinPred == "$or"_sd,
+              str::stream() << "joinPred must be '$and' or '$or', was " << joinPred);
+
     BSONObjBuilder bob;
-    BSONArrayBuilder arrBob(bob.subarrayStart("$and"));
+    BSONArrayBuilder arrBob(bob.subarrayStart(joinPred));
 
     auto addPredicates = [&](const auto& predicates) {
         if (predicates.isEmpty()) {
@@ -391,9 +445,11 @@ void DocumentSourceMatch::joinMatchWith(intrusive_ptr<DocumentSourceMatch> other
         }
 
         for (auto&& pred : predicates) {
-            // If 'pred' is an $and, add its children directly to the new top-level $and to avoid
-            // nesting $and's. Otherwise, add 'pred' itself as a child.
-            if (pred.fieldNameStringData() == "$and") {
+            // For 'joinPred' == $and: If 'pred' is an $and, add its children directly to the new
+            // top-level $and to avoid nesting $and's. For 'joinPred' == $or: If 'pred' is a $or,
+            // add its children directly to the new top-level $or to avoid nesting $or's. Otherwise,
+            // add 'pred' itself as a child.
+            if (pred.fieldNameStringData() == joinPred) {
                 for (auto& child : pred.Array()) {
                     arrBob.append(child);
                 }
@@ -422,14 +478,15 @@ DocumentSourceMatch::splitSourceByFunc(const OrderedPathSet& fields,
                                        const StringMap<std::string>& renames,
                                        expression::ShouldSplitExprFunc func) && {
     pair<unique_ptr<MatchExpression>, unique_ptr<MatchExpression>> newExpr(
-        expression::splitMatchExpressionBy(std::move(_expression), fields, renames, func));
+        expression::splitMatchExpressionBy(
+            std::move(_matchProcessor->getExpression()), fields, renames, func));
 
     invariant(newExpr.first || newExpr.second);
 
     if (!newExpr.first) {
         // The entire $match depends on 'fields'. It cannot be split or moved, so we return this
         // stage without modification as the second stage in the pair.
-        _expression = std::move(newExpr.second);
+        _matchProcessor->setExpression(std::move(newExpr.second));
         return {nullptr, this};
     }
 
@@ -437,7 +494,7 @@ DocumentSourceMatch::splitSourceByFunc(const OrderedPathSet& fields,
         // This $match is entirely independent of 'fields' and there were no renames to apply. In
         // this case, the current stage can swap with its predecessor without modification. We
         // simply return this as the first stage in the pair.
-        _expression = std::move(newExpr.first);
+        _matchProcessor->setExpression(std::move(newExpr.first));
         return {this, nullptr};
     }
 
@@ -508,7 +565,7 @@ DocumentSourceMatch::splitMatchByModifiedFields(
             // This stage modifies all paths, so cannot be swapped with a $match at all.
             return {nullptr, match};
         case DocumentSource::GetModPathsReturn::Type::kFiniteSet:
-            modifiedPaths = std::move(modifiedPathsRet.paths);
+            modifiedPaths = modifiedPathsRet.paths;
             break;
         case DocumentSource::GetModPathsReturn::Type::kAllExcept: {
             DepsTracker depsTracker;
@@ -547,8 +604,13 @@ BSONObj DocumentSourceMatch::getQuery() const {
 }
 
 DepsTracker::State DocumentSourceMatch::getDependencies(DepsTracker* deps) const {
+    return getDependencies(_matchProcessor->getExpression().get(), deps);
+}
+
+DepsTracker::State DocumentSourceMatch::getDependencies(const MatchExpression* expr,
+                                                        DepsTracker* deps) const {
     // Get all field or variable dependencies.
-    match_expression::addDependencies(_expression.get(), deps);
+    match_expression::addDependencies(expr, deps);
 
     if (isTextQuery()) {
         // A $text aggregation field should return EXHAUSTIVE_FIELDS, since we don't necessarily
@@ -562,24 +624,7 @@ DepsTracker::State DocumentSourceMatch::getDependencies(DepsTracker* deps) const
 }
 
 void DocumentSourceMatch::addVariableRefs(std::set<Variables::Id>* refs) const {
-    match_expression::addVariableRefs(_expression.get(), refs);
-}
-
-DocumentSourceMatch::DocumentSourceMatch(const BSONObj& query,
-                                         const intrusive_ptr<ExpressionContext>& expCtx)
-    : DocumentSource(kStageName, expCtx) {
-    rebuild(query);
-}
-
-void DocumentSourceMatch::rebuild(BSONObj filter) {
-    _predicate = filter.getOwned();
-    _expression = uassertStatusOK(MatchExpressionParser::parse(
-        _predicate, pExpCtx, ExtensionsCallbackNoop(), Pipeline::kAllowedMatcherFeatures));
-    _isTextQuery = isTextQuery(_predicate);
-    _dependencies =
-        DepsTracker(_isTextQuery ? DepsTracker::kAllMetadata & ~DepsTracker::kOnlyTextScore
-                                 : DepsTracker::kAllMetadata);
-    getDependencies(&_dependencies);
+    match_expression::addVariableRefs(_matchProcessor->getExpression().get(), refs);
 }
 
 Value DocumentSourceInternalChangeStreamMatch::serialize(const SerializationOptions& opts) const {
