@@ -27,25 +27,37 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <absl/container/node_hash_set.h>
+#include <boost/smart_ptr.hpp>
+#include <initializer_list>
 
-#include "mongo/unittest/bson_test_util.h"
-#include <boost/intrusive_ptr.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
 
+#include "mongo/bson/json.h"
 #include "mongo/bson/unordered_fields_bsonobj_comparator.h"
+#include "mongo/db/database_name.h"
 #include "mongo/db/exec/document_value/document.h"
 #include "mongo/db/exec/document_value/document_value_test_util.h"
-#include "mongo/db/pipeline/aggregation_context_fixture.h"
+#include "mongo/db/exec/document_value/value_comparator.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/pipeline/document_source_merge.h"
-#include "mongo/db/pipeline/document_source_mock.h"
-#include "mongo/db/pipeline/process_interface/non_shardsvr_process_interface.h"
+#include "mongo/db/pipeline/document_source_merge_gen.h"
+#include "mongo/db/pipeline/expression_context_for_test.h"
+#include "mongo/db/pipeline/process_interface/stub_mongo_process_interface.h"
+#include "mongo/db/pipeline/serverless_aggregation_context_fixture.h"
+#include "mongo/db/query/query_shape/serialization_options.h"
+#include "mongo/db/tenant_id.h"
 #include "mongo/idl/server_parameter_test_util.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/str.h"
 
 namespace mongo {
 namespace {
 
-using boost::intrusive_ptr;
-
 constexpr StringData kWhenMatchedModeFieldName = DocumentSourceMergeSpec::kWhenMatchedFieldName;
 constexpr StringData kWhenNotMatchedModeFieldName =
     DocumentSourceMergeSpec::kWhenNotMatchedFieldName;
@@ -56,42 +68,11 @@ const StringData kDefaultWhenMatchedMode =
 const StringData kDefaultWhenNotMatchedMode =
     MergeWhenNotMatchedMode_serializer(MergeWhenNotMatchedModeEnum::kInsert);
 
-/**
- * For the purpsoses of this test, assume every collection is unsharded. Stages may ask this during
- * setup. For example, to compute its constraints, the $merge stage needs to know if the output
- * collection is sharded.
- */
-class MongoProcessInterfaceForTest : public StubMongoProcessInterface {
-public:
-    bool isSharded(OperationContext* opCtx, const NamespaceString& ns) override {
-        return false;
-    }
-
-    /**
-     * For the purposes of these tests, assume each collection is unsharded and has a document key
-     * of just "_id".
-     */
-    std::vector<FieldPath> collectDocumentKeyFieldsActingAsRouter(
-        OperationContext* opCtx, const NamespaceString& nss) const override {
-        return {"_id"};
-    }
-
-    void checkRoutingInfoEpochOrThrow(const boost::intrusive_ptr<ExpressionContext>& expCtx,
-                                      const NamespaceString&,
-                                      ChunkVersion) const override {
-        return;  // Assume it always matches for our tests here.
-    }
-};
-
 class DocumentSourceMergeTest : public AggregationContextFixture {
 public:
-    DocumentSourceMergeTest() : AggregationContextFixture() {
-        getExpCtx()->mongoProcessInterface = std::make_shared<MongoProcessInterfaceForTest>();
-    }
-
-    intrusive_ptr<DocumentSourceMerge> createMergeStage(BSONObj spec) {
+    boost::intrusive_ptr<DocumentSourceMerge> createMergeStage(BSONObj spec) {
         auto specElem = spec.firstElement();
-        intrusive_ptr<DocumentSourceMerge> mergeStage = dynamic_cast<DocumentSourceMerge*>(
+        boost::intrusive_ptr<DocumentSourceMerge> mergeStage = dynamic_cast<DocumentSourceMerge*>(
             DocumentSourceMerge::createFromBson(specElem, getExpCtx()).get());
         ASSERT_TRUE(mergeStage);
         return mergeStage;
@@ -99,39 +80,39 @@ public:
 };
 
 TEST_F(DocumentSourceMergeTest, CorrectlyParsesIfMergeSpecIsString) {
-    const auto& defaultDb = getExpCtx()->ns.db();
+    const auto& defaultDb = getExpCtx()->ns.db_forTest();
     const std::string targetColl = "target_collection";
     auto spec = BSON("$merge" << targetColl);
     auto mergeStage = createMergeStage(spec);
     ASSERT(mergeStage);
-    ASSERT_EQ(mergeStage->getOutputNs().db(), defaultDb);
+    ASSERT_EQ(mergeStage->getOutputNs().db_forTest(), defaultDb);
     ASSERT_EQ(mergeStage->getOutputNs().coll(), targetColl);
 }
 
 TEST_F(DocumentSourceMergeTest, CorrectlyParsesIfIntoIsString) {
-    const auto& defaultDb = getExpCtx()->ns.db();
+    const auto& defaultDb = getExpCtx()->ns.db_forTest();
     const std::string targetColl = "target_collection";
     auto spec = BSON("$merge" << BSON("into" << targetColl));
     auto mergeStage = createMergeStage(spec);
     ASSERT(mergeStage);
-    ASSERT_EQ(mergeStage->getOutputNs().db(), defaultDb);
+    ASSERT_EQ(mergeStage->getOutputNs().db_forTest(), defaultDb);
     ASSERT_EQ(mergeStage->getOutputNs().coll(), targetColl);
 }
 
 TEST_F(DocumentSourceMergeTest, CorrectlyParsesIfIntoIsObject) {
-    const auto& defaultDb = getExpCtx()->ns.db();
+    const auto& defaultDb = getExpCtx()->ns.db_forTest();
     const std::string targetDb = "target_db";
     const std::string targetColl = "target_collection";
     auto spec = BSON("$merge" << BSON("into" << BSON("coll" << targetColl)));
     auto mergeStage = createMergeStage(spec);
     ASSERT(mergeStage);
-    ASSERT_EQ(mergeStage->getOutputNs().db(), defaultDb);
+    ASSERT_EQ(mergeStage->getOutputNs().db_forTest(), defaultDb);
     ASSERT_EQ(mergeStage->getOutputNs().coll(), targetColl);
 
     spec = BSON("$merge" << BSON("into" << BSON("db" << targetDb << "coll" << targetColl)));
     mergeStage = createMergeStage(spec);
     ASSERT(mergeStage);
-    ASSERT_EQ(mergeStage->getOutputNs().db(), targetDb);
+    ASSERT_EQ(mergeStage->getOutputNs().db_forTest(), targetDb);
     ASSERT_EQ(mergeStage->getOutputNs().coll(), targetColl);
 }
 
@@ -151,7 +132,7 @@ TEST_F(DocumentSourceMergeTest, CorrectlyParsesIfWhenMatchedIsStringOrArray) {
 TEST_F(DocumentSourceMergeTest, CorrectlyParsesIfTargetAndAggregationNamespacesAreSame) {
     const auto targetNsSameAsAggregationNs = getExpCtx()->ns;
     const auto targetColl = targetNsSameAsAggregationNs.coll();
-    const auto targetDb = targetNsSameAsAggregationNs.db();
+    const auto targetDb = targetNsSameAsAggregationNs.db_forTest();
 
     auto spec = BSON("$merge" << BSON("into" << BSON("coll" << targetColl << "db" << targetDb)));
     ASSERT(createMergeStage(spec));
@@ -167,11 +148,11 @@ TEST_F(DocumentSourceMergeTest, FailsToParseIncorrectMergeSpecType) {
 
 TEST_F(DocumentSourceMergeTest, FailsToParseIfMergeSpecObjectWithoutInto) {
     auto spec = BSON("$merge" << BSONObj());
-    ASSERT_THROWS_CODE(createMergeStage(spec), AssertionException, 40414);
+    ASSERT_THROWS_CODE(createMergeStage(spec), AssertionException, ErrorCodes::IDLFailedToParse);
 
     spec = BSON("$merge" << BSON("whenMatched"
                                  << "replace"));
-    ASSERT_THROWS_CODE(createMergeStage(spec), AssertionException, 40414);
+    ASSERT_THROWS_CODE(createMergeStage(spec), AssertionException, ErrorCodes::IDLFailedToParse);
 }
 
 TEST_F(DocumentSourceMergeTest, FailsToParseIfIntoIsNotStringAndNotObject) {
@@ -199,19 +180,19 @@ TEST_F(DocumentSourceMergeTest, FailsToParseIfIntoIsAnEmptyString) {
 TEST_F(DocumentSourceMergeTest, FailsToParseIfIntoIsObjectWithInvalidFields) {
     auto spec = BSON("$merge" << BSON("into" << BSON("a"
                                                      << "b")));
-    ASSERT_THROWS_CODE(createMergeStage(spec), AssertionException, 40415);
+    ASSERT_THROWS_CODE(createMergeStage(spec), AssertionException, ErrorCodes::IDLUnknownField);
 
     spec = BSON("$merge" << BSON("into" << BSON("coll"
                                                 << "target_collection"
                                                 << "a"
                                                 << "b")));
-    ASSERT_THROWS_CODE(createMergeStage(spec), AssertionException, 40415);
+    ASSERT_THROWS_CODE(createMergeStage(spec), AssertionException, ErrorCodes::IDLUnknownField);
 
     spec = BSON("$merge" << BSON("into" << BSON("db"
                                                 << "target_db"
                                                 << "a"
                                                 << "b")));
-    ASSERT_THROWS_CODE(createMergeStage(spec), AssertionException, 40415);
+    ASSERT_THROWS_CODE(createMergeStage(spec), AssertionException, ErrorCodes::IDLUnknownField);
 }
 
 TEST_F(DocumentSourceMergeTest, FailsToParseIfIntoIsObjectWithEmptyCollectionName) {
@@ -399,13 +380,13 @@ TEST_F(DocumentSourceMergeTest, FailsToParseIfOnFieldIsNotStringOrArrayOfStrings
 }
 
 TEST_F(DocumentSourceMergeTest, CorrectlyUsesTargetDbThatMatchesAggregationDb) {
-    const auto targetDbSameAsAggregationDb = getExpCtx()->ns.db();
+    const auto targetDbSameAsAggregationDb = getExpCtx()->ns.db_forTest();
     const auto targetColl = "target_collection";
     auto spec = BSON("$merge" << BSON("into" << BSON("coll" << targetColl << "db"
                                                             << targetDbSameAsAggregationDb)));
 
     auto mergeStage = createMergeStage(spec);
-    ASSERT_EQ(mergeStage->getOutputNs().db(), targetDbSameAsAggregationDb);
+    ASSERT_EQ(mergeStage->getOutputNs().db_forTest(), targetDbSameAsAggregationDb);
     ASSERT_EQ(mergeStage->getOutputNs().coll(), targetColl);
 }
 
@@ -492,7 +473,7 @@ TEST_F(DocumentSourceMergeTest, SerializeDottedPathOnFieldsSharedPrefix) {
 }
 
 TEST_F(DocumentSourceMergeTest, SerializeIntoWhenMergeSpecIsStringNotDotted) {
-    const auto aggregationDb = getExpCtx()->ns.db();
+    const auto aggregationDb = getExpCtx()->ns.db_forTest();
     auto spec = BSON("$merge"
                      << "target_collection");
     auto mergeStage = createMergeStage(spec);
@@ -502,7 +483,7 @@ TEST_F(DocumentSourceMergeTest, SerializeIntoWhenMergeSpecIsStringNotDotted) {
 }
 
 TEST_F(DocumentSourceMergeTest, SerializeIntoWhenMergeSpecIsStringDotted) {
-    const auto aggregationDb = getExpCtx()->ns.db();
+    const auto aggregationDb = getExpCtx()->ns.db_forTest();
     auto spec = BSON("$merge"
                      << "my.target_collection");
     auto mergeStage = createMergeStage(spec);
@@ -512,7 +493,7 @@ TEST_F(DocumentSourceMergeTest, SerializeIntoWhenMergeSpecIsStringDotted) {
 }
 
 TEST_F(DocumentSourceMergeTest, SerializeIntoWhenIntoIsStringNotDotted) {
-    const auto aggregationDb = getExpCtx()->ns.db();
+    const auto aggregationDb = getExpCtx()->ns.db_forTest();
     auto spec = BSON("$merge" << BSON("into"
                                       << "target_collection"));
     auto mergeStage = createMergeStage(spec);
@@ -522,7 +503,7 @@ TEST_F(DocumentSourceMergeTest, SerializeIntoWhenIntoIsStringNotDotted) {
 }
 
 TEST_F(DocumentSourceMergeTest, SerializeIntoWhenIntoIsStringDotted) {
-    const auto aggregationDb = getExpCtx()->ns.db();
+    const auto aggregationDb = getExpCtx()->ns.db_forTest();
     auto spec = BSON("$merge" << BSON("into"
                                       << "my.target_collection"));
     auto mergeStage = createMergeStage(spec);
@@ -532,7 +513,7 @@ TEST_F(DocumentSourceMergeTest, SerializeIntoWhenIntoIsStringDotted) {
 }
 
 TEST_F(DocumentSourceMergeTest, SerializeIntoWhenIntoIsObjectWithCollNotDotted) {
-    const auto aggregationDb = getExpCtx()->ns.db();
+    const auto aggregationDb = getExpCtx()->ns.db_forTest();
     auto spec = BSON("$merge" << BSON("into" << BSON("coll"
                                                      << "target_collection")));
     auto mergeStage = createMergeStage(spec);
@@ -542,7 +523,7 @@ TEST_F(DocumentSourceMergeTest, SerializeIntoWhenIntoIsObjectWithCollNotDotted)
 }
 
 TEST_F(DocumentSourceMergeTest, SerializeIntoWhenIntoIsObjectWithCollDotted) {
-    const auto aggregationDb = getExpCtx()->ns.db();
+    const auto aggregationDb = getExpCtx()->ns.db_forTest();
     auto spec = BSON("$merge" << BSON("into" << BSON("coll"
                                                      << "my.target_collection")));
     auto mergeStage = createMergeStage(spec);
@@ -869,10 +850,10 @@ TEST_F(DocumentSourceMergeTest, SerializeLetArrayVariable) {
 }
 
 // This test verifies that when the 'let' argument is specified as 'null', the default 'new'
-// variable is still available. This is not a desirable behaviour but rather a limitation in the IDL
-// parser which cannot differentiate between an optional field specified explicitly as 'null', or
-// not specified at all. In both cases it will treat the field like it wasn't specified. So, this
-// test ensures that we're aware of this limitation. Once the limitation is addressed in
+// variable is still available. This is not a desirable behaviour but rather a limitation in the
+// IDL parser which cannot differentiate between an optional field specified explicitly as 'null',
+// or not specified at all. In both cases it will treat the field like it wasn't specified. So,
+// this test ensures that we're aware of this limitation. Once the limitation is addressed in
 // SERVER-41272, this test should be updated to accordingly.
 TEST_F(DocumentSourceMergeTest, SerializeNullLetVariablesAsDefault) {
     for (auto&& whenNotMatched : {"insert", "fail", "discard"}) {
@@ -905,14 +886,90 @@ TEST_F(DocumentSourceMergeTest, SerializeEmptyLetVariables) {
         auto mergeStage = createMergeStage(spec);
         ASSERT(mergeStage);
         auto serialized = mergeStage->serialize().getDocument();
-        ASSERT_VALUE_EQ(serialized["$merge"]["let"],
-                        (whenNotMatched == "insert"_sd ? Value(BSON("new"
-                                                                    << "$$ROOT"))
-                                                       : Value(BSONObj())));
+
+        if (whenNotMatched == "insert"_sd) {
+            ASSERT_VALUE_EQ(serialized["$merge"]["let"],
+                            Value(BSON("new"
+                                       << "$$ROOT")));
+        } else {
+            ASSERT_TRUE(serialized["$merge"]["let"].missing());
+        }
         ASSERT_VALUE_EQ(serialized["$merge"]["whenMatched"], Value(pipeline));
     }
 }
 
+TEST_F(DocumentSourceMergeTest, SerializeTargetCollectionVersion) {
+    auto pipeline = BSON_ARRAY(fromjson("{$project: {_id: true, x: '$$new'}}"));
+    auto spec =
+        BSON("$merge" << BSON("into"
+                              << "target_collection"
+                              << "whenMatched"
+                              << "merge"
+                              << "whenNotMatched"
+                              << "insert"
+                              << "targetCollectionVersion"
+                              << BSON("e" << OID() << "t" << Timestamp() << "v" << Timestamp())));
+    auto mergeStage = createMergeStage(spec);
+    ASSERT(mergeStage);
+    ASSERT_TRUE(mergeStage->getMergeProcessor()->getCollectionPlacementVersion().has_value());
+
+    // Ensure that 'targetCollectionVersion' attribute is not present in case of non-default literal
+    // policy.
+    {
+        SerializationOptions opts;
+        opts.literalPolicy = LiteralSerializationPolicy::kToRepresentativeParseableValue;
+        auto serialized = mergeStage->serialize(opts).getDocument();
+        ASSERT_BSONOBJ_EQ_AUTO(  // NOLINT
+            R"({
+            "$merge": {
+                "into": {
+                    "db": "unittests",
+                    "coll": "target_collection"
+                },
+                "on": "_id",
+                "whenMatched": "merge",
+                "whenNotMatched": "insert"
+            }
+        })",
+            serialized.toBson());
+    }
+
+    // Ensure that 'targetCollectionVersion' attribute is present in case of default literal policy.
+    {
+        auto serialized = mergeStage->serialize().getDocument();
+        ASSERT_BSONOBJ_EQ_AUTO(  // NOLINT
+            R"({
+            "$merge": {
+                "into": {
+                    "db": "unittests",
+                    "coll": "target_collection"
+                },
+                "on": "_id",
+                "whenMatched": "merge",
+                "whenNotMatched": "insert",
+                "targetCollectionVersion": {
+                    "e": {
+                        "$oid": "000000000000000000000000"
+                    },
+                    "t": {
+                        "$timestamp": {
+                            "t": 0,
+                            "i": 0
+                        }
+                    },
+                    "v": {
+                        "$timestamp": {
+                            "t": 0,
+                            "i": 0
+                        }
+                    }
+                }
+            }
+        })",
+            serialized.toBson());
+    }
+}
+
 TEST_F(DocumentSourceMergeTest, SerializeEmptyLetVariableMentionNew) {
     auto pipeline = BSON_ARRAY(fromjson("{$project: {_id: true, x: '$$new'}}"));
     auto spec =
@@ -927,7 +984,7 @@ TEST_F(DocumentSourceMergeTest, SerializeEmptyLetVariableMentionNew) {
         R"({
             "$merge": {
                 "into": {
-                    "db": "test",
+                    "db": "unittests",
                     "coll": "target_collection"
                 },
                 "on": "_id",
@@ -1054,8 +1111,8 @@ TEST_F(DocumentSourceMergeServerlessTest,
 
     // Pass collection name as a db + coll object.
     auto stageSpec =
-        BSON("$merge" << BSON("into" << BSON("db" << nss.dbName().toStringWithTenantId() << "coll"
-                                                  << _targetColl)));
+        BSON("$merge" << BSON("into" << BSON("db" << nss.dbName().toStringWithTenantId_forTest()
+                                                  << "coll" << _targetColl)));
     auto liteParsedLookup = DocumentSourceMerge::LiteParsed::parse(nss, stageSpec.firstElement());
     auto namespaceSet = liteParsedLookup->getInvolvedNamespaces();
     ASSERT_EQ(1, namespaceSet.size());
@@ -1084,8 +1141,8 @@ TEST_F(DocumentSourceMergeServerlessTest,
         ASSERT_EQ(*mergeSource->getOutputNs().tenantId(), *expCtx->ns.tenantId());
 
         // Assert the tenantId is not included in the serialized namespace.
-        auto dbField = flagStatus ? expCtx->ns.dbName().toString()
-                                  : expCtx->ns.dbName().toStringWithTenantId();
+        auto dbField = flagStatus ? expCtx->ns.dbName().toString_forTest()
+                                  : expCtx->ns.dbName().toStringWithTenantId_forTest();
         auto expectedDoc = Document{{"db", dbField}, {"coll", _targetColl}};
 
         auto serialized = mergeSource->serialize().getDocument();
@@ -1112,8 +1169,8 @@ TEST_F(DocumentSourceMergeServerlessTest,
         ASSERT(mergeSource->getOutputNs().tenantId());
         ASSERT_EQ(*mergeSource->getOutputNs().tenantId(), *expCtx->ns.tenantId());
 
-        auto dbField = flagStatus ? expCtx->ns.dbName().toString()
-                                  : expCtx->ns.dbName().toStringWithTenantId();
+        auto dbField = flagStatus ? expCtx->ns.dbName().toString_forTest()
+                                  : expCtx->ns.dbName().toStringWithTenantId_forTest();
         auto expectedDoc = Document{{"db", dbField}, {"coll", _targetColl}};
 
         auto serialized = mergeSource->serialize().getDocument();
@@ -1196,7 +1253,7 @@ TEST_F(DocumentSourceMergeTest, QueryShape) {
     auto expectedBson = fromjson(R"({
             "$merge": {
                 "into": {
-                    "db": "HASH<test>",
+                    "db": "HASH<unittests>",
                     "coll": "HASH<target_collection>"
                 },
                 "on": "HASH<_id>",
