@@ -27,21 +27,35 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <absl/container/node_hash_set.h>
+#include <boost/smart_ptr.hpp>
+#include <string>
+#include <vector>
 
-#include <boost/intrusive_ptr.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
 
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/json.h"
 #include "mongo/db/exec/document_value/document.h"
 #include "mongo/db/exec/document_value/document_value_test_util.h"
-#include "mongo/db/pipeline/aggregation_context_fixture.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/pipeline/document_source_out.h"
+#include "mongo/db/pipeline/expression_context_for_test.h"
+#include "mongo/db/pipeline/field_path.h"
+#include "mongo/db/pipeline/process_interface/stub_mongo_process_interface.h"
+#include "mongo/db/pipeline/serverless_aggregation_context_fixture.h"
+#include "mongo/db/tenant_id.h"
 #include "mongo/idl/server_parameter_test_util.h"
+#include "mongo/s/chunk_version.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
 
 namespace mongo {
 namespace {
 
-using boost::intrusive_ptr;
-
 /**
  * For the purpsoses of this test, assume every collection is unsharded. Stages may ask this during
  * setup. For example, to compute its constraints, the $out stage needs to know if the output
@@ -75,9 +89,9 @@ public:
         getExpCtx()->mongoProcessInterface = std::make_shared<MongoProcessInterfaceForTest>();
     }
 
-    intrusive_ptr<DocumentSourceOut> createOutStage(BSONObj spec) {
+    boost::intrusive_ptr<DocumentSourceOut> createOutStage(BSONObj spec) {
         auto specElem = spec.firstElement();
-        intrusive_ptr<DocumentSourceOut> outStage = dynamic_cast<DocumentSourceOut*>(
+        boost::intrusive_ptr<DocumentSourceOut> outStage = dynamic_cast<DocumentSourceOut*>(
             DocumentSourceOut::createFromBson(specElem, getExpCtx()).get());
         ASSERT_TRUE(outStage);
         return outStage;
@@ -92,7 +106,7 @@ TEST_F(DocumentSourceOutTest, FailsToParseIncorrectType) {
     ASSERT_THROWS_CODE(createOutStage(spec), AssertionException, 16990);
 
     spec = BSON("$out" << BSONObj());
-    ASSERT_THROWS_CODE(createOutStage(spec), AssertionException, 40414);
+    ASSERT_THROWS_CODE(createOutStage(spec), AssertionException, ErrorCodes::IDLFailedToParse);
 }
 
 TEST_F(DocumentSourceOutTest, AcceptsStringArgument) {
@@ -123,6 +137,7 @@ TEST_F(DocumentSourceOutTest, RedactionNoTimeseries) {
             }
         })");
     auto docSource = DocumentSourceOut::createFromBson(spec.firstElement(), getExpCtx());
+
     ASSERT_BSONOBJ_EQ_AUTO(  // NOLINT
         R"({
             $out: {
@@ -214,11 +229,23 @@ TEST_F(DocumentSourceOutServerlessTest, CreateFromBSONContainsExpectedNamespaces
     ASSERT_EQ(outSource->getOutputNs(),
               NamespaceString::createNamespaceString_forTest(defaultDb, targetColl));
 
+    // TODO SERVER-77000: update this test once the serialize function has been updated to use
+    // DatabaseNameUtil::serialize() instead.  We need to set the serialization context objs on the
+    // expCtx, and manipulate before calling outSource->serialize().
     // Assert the tenantId is not included in the serialized namespace.
     auto serialized = outSource->serialize().getDocument();
-    auto expectedDoc = Document{{"coll", targetColl}, {"db", expCtx->ns.dbName().db()}};
+    auto expectedDoc =
+        Document{{"coll", targetColl}, {"db", expCtx->ns.dbName().toString_forTest()}};
     ASSERT_DOCUMENT_EQ(serialized["$out"].getDocument(), expectedDoc);
 
+    // TODO SERVER-77000: uncomment the below
+    // expCtx->serializationCtxt.setPrefixState(true);
+    // std::string targetDb = str::stream()
+    //     << expCtx->ns.tenantId()->toString() << "_" << expCtx->ns.dbName().toString_forTest();
+    // serialized = outSource->serialize().getDocument();
+    // expectedDoc = Document{{"coll", targetColl}, {"db", targetDb}};
+    // ASSERT_DOCUMENT_EQ(serialized["$out"].getDocument(), expectedDoc);
+
     // The tenantId for the outputNs should be the same as that on the expCtx despite outputting
     // into different dbs.
     const std::string targetDb = "target_db";
@@ -228,7 +255,7 @@ TEST_F(DocumentSourceOutServerlessTest, CreateFromBSONContainsExpectedNamespaces
     ASSERT(outSource);
     ASSERT(outSource->getOutputNs().tenantId());
     ASSERT_EQ(*outSource->getOutputNs().tenantId(), *expCtx->ns.tenantId());
-    ASSERT_EQ(outSource->getOutputNs().dbName().db(), targetDb);
+    ASSERT_EQ(outSource->getOutputNs().dbName().toString_forTest(), targetDb);
 
     // Assert the tenantId is not included in the serialized namespace.
     serialized = outSource->serialize().getDocument();
