@@ -29,16 +29,40 @@
 
 #include "mongo/db/pipeline/document_source_query_stats.h"
 
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <cstddef>
+#include <list>
+
+#include "mongo/base/data_range.h"
+#include "mongo/base/error_codes.h"
+#include "mongo/bson/bsonobj.h"
 #include "mongo/bson/bsontypes.h"
+#include "mongo/db/commands/server_status_metric.h"
+#include "mongo/db/feature_flag.h"
+#include "mongo/db/pipeline/document_source_query_stats_gen.h"
+#include "mongo/db/query/allowed_contexts.h"
+#include "mongo/db/query/lru_key_value.h"
+#include "mongo/db/query/query_feature_flags_gen.h"
+#include "mongo/db/query/query_knobs_gen.h"
 #include "mongo/idl/idl_parser.h"
+#include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/atomic_word.h"
 #include "mongo/util/assert_util.h"
 #include "mongo/util/debug_util.h"
+#include "mongo/util/intrusive_counter.h"
+#include "mongo/util/str.h"
+#include "mongo/util/time_support.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQueryStats
 
 namespace mongo {
 namespace {
-CounterMetric queryStatsHmacApplicationErrors("queryStats.numHmacApplicationErrors");
+auto& queryStatsHmacApplicationErrors =
+    *MetricBuilder<Counter64>{"queryStats.numHmacApplicationErrors"};
 }
 
 REGISTER_DOCUMENT_SOURCE_WITH_FEATURE_FLAG(queryStats,
@@ -82,8 +106,9 @@ auto parseSpec(const BSONElement& spec, const Ctor& ctor) {
 
 }  // namespace
 
-BSONObj DocumentSourceQueryStats::computeQueryStatsKey(std::shared_ptr<const Key> key) const {
-    static const auto sha256HmacStringDataHasher = [](std::string key, const StringData& sd) {
+BSONObj DocumentSourceQueryStats::computeQueryStatsKey(
+    std::shared_ptr<const Key> key, const SerializationContext& serializationContext) const {
+    static const auto sha256HmacStringDataHasher = [](std::string key, StringData sd) {
         auto hashed = SHA256Block::computeHmac(
             (const uint8_t*)key.data(), key.size(), (const uint8_t*)sd.rawData(), sd.size());
         return hashed.toString();
@@ -97,14 +122,14 @@ BSONObj DocumentSourceQueryStats::computeQueryStatsKey(std::shared_ptr<const Key
             return sha256HmacStringDataHasher(_hmacKey, sd);
         };
     }
-    return key->toBson(pExpCtx->opCtx, opts);
+    return key->toBson(pExpCtx->opCtx, opts, serializationContext);
 }
 
 std::unique_ptr<DocumentSourceQueryStats::LiteParsed> DocumentSourceQueryStats::LiteParsed::parse(
     const NamespaceString& nss, const BSONElement& spec) {
     return parseSpec(spec, [&](TransformAlgorithmEnum algorithm, std::string hmacKey) {
         return std::make_unique<DocumentSourceQueryStats::LiteParsed>(
-            spec.fieldName(), algorithm, hmacKey);
+            spec.fieldName(), nss.tenantId(), algorithm, hmacKey);
     });
 }
 
@@ -114,7 +139,7 @@ boost::intrusive_ptr<DocumentSource> DocumentSourceQueryStats::createFromBson(
 
     uassert(ErrorCodes::InvalidNamespace,
             "$queryStats must be run against the 'admin' database with {aggregate: 1}",
-            nss.db() == DatabaseName::kAdmin.db() && nss.isCollectionlessAggregateNS());
+            nss.isAdminDB() && nss.isCollectionlessAggregateNS());
 
     LOGV2_DEBUG_OPTIONS(7808300,
                         1,
@@ -203,30 +228,40 @@ boost::optional<Document> DocumentSourceQueryStats::toDocument(
     const Date_t& partitionReadTime, const QueryStatsEntry& queryStatsEntry) const {
     const auto& key = queryStatsEntry.key;
     try {
-        auto queryStatsKey = computeQueryStatsKey(key);
-        // We use the representative shape to generate the key hash. This avoids returning duplicate
-        // hashes if we have bugs that cause two different representative shapes to re-parse into
-        // the same debug shape.
-        auto representativeShapeKey = key->toBson(
-            pExpCtx->opCtx, SerializationOptions::kRepresentativeQueryShapeSerializeOptions);
-
+        auto queryStatsKey = computeQueryStatsKey(key, SerializationContext::stateDefault());
+        // We use the representative shape to generate the key and shape hashes. This avoids
+        // returning duplicate hashes if we have bugs that cause two different representative shapes
+        // to re-parse into the same debug shape.
+        auto representativeShapeKey =
+            key->toBson(pExpCtx->opCtx,
+                        SerializationOptions::kRepresentativeQueryShapeSerializeOptions,
+                        SerializationContext::stateDefault());
         // This SHA256 version of the hash is output to aid in data analytics use cases. In these
         // cases, we often care about comparing hashes from different hosts, potentially on
         // different versions and platforms. The thinking here is that the SHA256 algorithm is more
         // stable across these different environments than the quicker 'absl::HashOf'
         // implementation.
-        auto hash = SHA256Block::computeHash((const uint8_t*)representativeShapeKey.objdata(),
-                                             representativeShapeKey.objsize())
-                        .toString();
-        return Document{{"key", std::move(queryStatsKey)},
-                        {"keyHash", hash},
-                        {"metrics", queryStatsEntry.toBSON()},
-                        {"asOf", partitionReadTime}};
+        auto keyHash = SHA256Block::computeHash((const uint8_t*)representativeShapeKey.objdata(),
+                                                representativeShapeKey.objsize())
+                           .toString();
+        auto queryShapeHash =
+            key->getQueryShapeHash(pExpCtx->opCtx, SerializationContext::stateDefault())
+                .toHexString();
+        return Document{
+            {"key", std::move(queryStatsKey)},
+            {"keyHash", keyHash},
+            {"queryShapeHash", queryShapeHash},
+            {"metrics",
+             queryStatsEntry.toBSON(feature_flags::gFeatureFlagQueryStatsDataBearingNodes.isEnabled(
+                 serverGlobalParams.featureCompatibility.acquireFCVSnapshot()))},
+            {"asOf", partitionReadTime}};
     } catch (const DBException& ex) {
         queryStatsHmacApplicationErrors.increment();
         const auto& hash = absl::HashOf(key);
         const auto queryShape = key->universalComponents()._queryShape->toBson(
-            pExpCtx->opCtx, SerializationOptions::kRepresentativeQueryShapeSerializeOptions);
+            pExpCtx->opCtx,
+            SerializationOptions::kRepresentativeQueryShapeSerializeOptions,
+            SerializationContext::stateDefault());
         LOGV2_DEBUG(7349403,
                     2,
                     "Error encountered when applying hmac to query shape, will not publish "
