@@ -29,10 +29,27 @@
 
 #pragma once
 
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <set>
+#include <string>
+#include <utility>
+
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
 #include "mongo/db/exec/document_value/document.h"
+#include "mongo/db/pipeline/dependencies.h"
+#include "mongo/db/pipeline/document_source.h"
+#include "mongo/db/pipeline/document_source_match.h"
 #include "mongo/db/pipeline/document_source_single_document_transformation.h"
 #include "mongo/db/pipeline/expression.h"
+#include "mongo/db/pipeline/expression_context.h"
 #include "mongo/db/pipeline/expression_dependencies.h"
+#include "mongo/db/pipeline/transformer_interface.h"
+#include "mongo/db/pipeline/variables.h"
+#include "mongo/db/query/explain_options.h"
+#include "mongo/db/query/query_shape/serialization_options.h"
+#include "mongo/util/intrusive_counter.h"
 
 namespace mongo {
 
@@ -45,20 +62,33 @@ public:
 
     ReplaceRootTransformation(const boost::intrusive_ptr<ExpressionContext>& expCtx,
                               boost::intrusive_ptr<Expression> newRootExpression,
-                              std::string errMsgContextForNonObject)
+                              std::string errMsgContextForNonObject,
+                              SbeCompatibility sbeCompatibility)
         : _expCtx(expCtx),
           _newRoot(std::move(newRootExpression)),
-          _errMsgContextForNonObject(std::move(errMsgContextForNonObject)) {}
+          _errMsgContextForNonObject(std::move(errMsgContextForNonObject)) {
+        _sbeCompatibility = sbeCompatibility;
+    }
 
     TransformerType getType() const final {
         return TransformerType::kReplaceRoot;
     }
 
-    Document applyTransformation(const Document& input) final;
+    Document applyTransformation(const Document& input) const final;
 
     // Optimize the newRoot expression.
     void optimize() final {
-        _newRoot->optimize();
+        // Optimization can sometimes modify a previously compatible expression so that it can no
+        // longer be executed in SBE. When that happens, the expression optimizer updates the
+        // 'sbeCompatibility' value in the ExpressionContext, which we can use to update the
+        // '_sbeCompatibility' value for this $replaceRoot operation.
+        SbeCompatibility originalSbeCompatibility =
+            std::exchange(_expCtx->sbeCompatibility, _sbeCompatibility);
+        ON_BLOCK_EXIT([&] { this->_expCtx->sbeCompatibility = originalSbeCompatibility; });
+
+        _newRoot = _newRoot->optimize();
+
+        _sbeCompatibility = _expCtx->sbeCompatibility;
     }
 
     Document serializeTransformation(boost::optional<ExplainOptions::Verbosity> explain,
@@ -82,6 +112,50 @@ public:
         return _newRoot;
     }
 
+    /**
+     * Detects if 'replaceRootTransform' represents the unnesting of a field path. If it does,
+     * returns the name of that field path. For example, if 'replaceRootTransform' represents the
+     * transformation associated with {$replaceWith: "$x"} or {$replaceRoot: {newRoot: "$x"}},
+     * returns "x".
+     */
+    boost::optional<std::string> unnestsPath() const;
+
+    /**
+     * This whole block adds
+     * {$or: [{"subDocument": {$type: "array"}}, {"subDocument": {$not: {$type: "object"}}}]}
+     * in order to match on documents which, when evaluated at $expression, don't resolve to
+     * objects. "array" is separately included because for MatchExpressions, arrays are evaluated as
+     * "object" types.
+     */
+    static boost::intrusive_ptr<DocumentSourceMatch> createTypeNEObjectPredicate(
+        const std::string& expression, const boost::intrusive_ptr<ExpressionContext>& expCtx);
+
+    /**
+     * Report renames from this stage to a MatchExpression. We only report a rename for a top-level
+     * field in a path to ensure that we only have at max one applicable rename per path in the ME.
+     */
+    void reportRenames(const MatchExpression* expr,
+                       const std::string& prefixPath,
+                       StringMap<std::string>& renames);
+
+    /**
+     * This optimization pushes a match stage before a replaceRoot stage to improve performance.
+     *
+     * Ex: [{$replaceWith: {"$subDocument"}}, {$match: {x: 2}}]
+     * ->
+     * [{$match: {"subDocument.x": 2}}, {$replaceWith: {"$subDocument"}}]
+     *
+     * We also append {$expr: {$ne: [{$type: "$expression"}, {$const: "object"}]}} to the pushed
+     * forward match stage to ensure that after we swap a $match stage before a $replaceRoot stage,
+     * we can still check that the structure of all documents in the collection can resolve to a
+     * document after `newRoot` is applied.
+     *
+     * Note: this optimization will not rename and push forward a match stage if there are
+     * dependencies between different paths contained in the match expression.
+     */
+    bool pushDotRenamedMatchBefore(Pipeline::SourceContainer::iterator itr,
+                                   Pipeline::SourceContainer* container);
+
     boost::intrusive_ptr<Expression>& getExpressionToModify() {
         return _newRoot;
     }
@@ -90,6 +164,10 @@ public:
         expression::addVariableRefs(_newRoot.get(), refs);
     }
 
+    SbeCompatibility sbeCompatibility() const {
+        return _sbeCompatibility;
+    }
+
 private:
     const boost::intrusive_ptr<ExpressionContext> _expCtx;
     boost::intrusive_ptr<Expression> _newRoot;
@@ -101,6 +179,8 @@ private:
     static constexpr StringData kErrorTemplate =
         "{} must evaluate to an object, but resulting value was: {}. Type of resulting value: "
         "'{}'. Input document: {}"_sd;
+
+    SbeCompatibility _sbeCompatibility = SbeCompatibility::notCompatible;
 };
 
 /*
@@ -127,7 +207,8 @@ public:
     static boost::intrusive_ptr<DocumentSource> create(
         const boost::intrusive_ptr<ExpressionContext>& expCtx,
         const boost::intrusive_ptr<Expression>& newRootExpression,
-        std::string errMsgContextForNonObjects);
+        std::string errMsgContextForNonObjects,
+        SbeCompatibility sbeCompatibility);
 
 private:
     // It is illegal to construct a DocumentSourceReplaceRoot directly, use createFromBson()
