@@ -27,9 +27,25 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <absl/container/flat_hash_map.h>
+#include <boost/container/small_vector.hpp>
+#include <boost/optional.hpp>
+// IWYU pragma: no_include "boost/intrusive/detail/iterator.hpp"
+#include <algorithm>
+#include <array>
+#include <iterator>
 
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/db/basic_types.h"
 #include "mongo/db/exec/add_fields_projection_executor.h"
+#include "mongo/db/exec/inclusion_projection_executor.h"
+#include "mongo/db/field_ref.h"
 #include "mongo/db/field_ref_set.h"
 #include "mongo/db/matcher/expression_algo.h"
 #include "mongo/db/pipeline/document_source_add_fields.h"
@@ -39,9 +55,21 @@
 #include "mongo/db/pipeline/document_source_sort.h"
 #include "mongo/db/pipeline/expression.h"
 #include "mongo/db/pipeline/lite_parsed_document_source.h"
+#include "mongo/db/pipeline/window_function/window_function_exec.h"
+#include "mongo/db/query/allowed_contexts.h"
+#include "mongo/db/query/explain_options.h"
+#include "mongo/db/query/projection_policies.h"
 #include "mongo/db/query/query_knobs_gen.h"
 #include "mongo/db/query/sort_pattern.h"
-#include "mongo/util/overloaded_visitor.h"
+#include "mongo/db/transaction_resources.h"
+#include "mongo/idl/idl_parser.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/base64.h"
+#include "mongo/util/fail_point.h"
+#include "mongo/util/overloaded_visitor.h"  // IWYU pragma: keep
+#include "mongo/util/str.h"
+#include "mongo/util/uuid.h"
 
 using boost::intrusive_ptr;
 using boost::optional;
@@ -51,6 +79,7 @@ using SortPatternPart = mongo::SortPattern::SortPatternPart;
 namespace mongo {
 
 namespace {
+MONGO_FAIL_POINT_DEFINE(overrideMemoryLimitForSpill);
 
 /**
  * Does a sort pattern contain a path that has been modified?
@@ -111,6 +140,7 @@ list<intrusive_ptr<DocumentSource>> document_source_set_window_fields::createFro
     FieldRefSet fieldSet;
     std::vector<FieldRef> backingRefs;
 
+    expCtx->sbeWindowCompatibility = SbeCompatibility::noRequirements;
     std::vector<WindowFunctionStatement> outputFields;
     const auto& output = spec.getOutput();
     backingRefs.reserve(output.nFields());
@@ -122,31 +152,21 @@ list<intrusive_ptr<DocumentSource>> document_source_set_window_fields::createFro
                 fieldSet.insert(&backingRefs.back(), &conflict));
         outputFields.push_back(WindowFunctionStatement::parse(outputElem, sortBy, expCtx.get()));
     }
+    auto sbeCompatibility = std::min(expCtx->sbeWindowCompatibility, expCtx->sbeCompatibility);
 
-    return create(expCtx, std::move(partitionBy), std::move(sortBy), std::move(outputFields));
-}
-
-WindowFunctionStatement WindowFunctionStatement::parse(BSONElement elem,
-                                                       const boost::optional<SortPattern>& sortBy,
-                                                       ExpressionContext* expCtx) {
-    // 'elem' is a statement like 'v: {$sum: {...}}', whereas the expression is '$sum: {...}'.
-    uassert(ErrorCodes::FailedToParse,
-            str::stream() << "The field '" << elem.fieldName() << "' must be an object",
-            elem.type() == BSONType::Object);
-    return WindowFunctionStatement(
-        elem.fieldName(),
-        window_function::Expression::parse(elem.embeddedObject(), sortBy, expCtx));
-}
-void WindowFunctionStatement::serialize(MutableDocument& outputFields,
-                                        const SerializationOptions& opts) const {
-    outputFields[opts.serializeFieldPathFromString(fieldName)] = expr->serialize(opts);
+    return create(expCtx,
+                  std::move(partitionBy),
+                  std::move(sortBy),
+                  std::move(outputFields),
+                  sbeCompatibility);
 }
 
 list<intrusive_ptr<DocumentSource>> document_source_set_window_fields::create(
     const intrusive_ptr<ExpressionContext>& expCtx,
     optional<intrusive_ptr<Expression>> partitionBy,
-    const optional<SortPattern>& sortBy,
-    std::vector<WindowFunctionStatement> outputFields) {
+    optional<SortPattern> sortBy,
+    std::vector<WindowFunctionStatement> outputFields,
+    SbeCompatibility sbeCompatibility) {
 
     // Starting with an input like this:
     //     {$setWindowFields: {partitionBy: {$foo: "$x"}, sortBy: {y: 1}, output: {...}}}
@@ -237,7 +257,14 @@ list<intrusive_ptr<DocumentSource>> document_source_set_window_fields::create(
     }
     if (sortBy) {
         for (const auto& part : *sortBy) {
-            combined.push_back(part);
+            // Check and filter out the partition by field within the sort field. The partition
+            // field is already added to the combined sort field variable. As the partition only
+            // contains documents that have the same value in the partition key an additional sort
+            // over that field does not change the order or the documents within the partition.
+            // Hence a sort by $partion does not change the sort order.
+            if (!simplePartitionBy || *simplePartitionBy != part.fieldPath) {
+                combined.push_back(part);
+            }
         }
     }
 
@@ -250,16 +277,17 @@ list<intrusive_ptr<DocumentSource>> document_source_set_window_fields::create(
     }
 
     if (!combined.empty()) {
-        result.push_back(DocumentSourceSort::create(expCtx, SortPattern{combined}));
+        result.push_back(DocumentSourceSort::create(expCtx, SortPattern{std::move(combined)}));
     }
 
     // $_internalSetWindowFields
     result.push_back(make_intrusive<DocumentSourceInternalSetWindowFields>(
         expCtx,
         simplePartitionByExpr,
-        sortBy,
-        outputFields,
-        internalDocumentSourceSetWindowFieldsMaxMemoryBytes.load()));
+        std::move(sortBy),
+        std::move(outputFields),
+        internalDocumentSourceSetWindowFieldsMaxMemoryBytes.load(),
+        sbeCompatibility));
 
     // $unset
     if (complexPartitionBy) {
@@ -273,9 +301,21 @@ intrusive_ptr<DocumentSource> DocumentSourceInternalSetWindowFields::optimize()
     // The _partitionBy is already optimized in create(), along with _iterator which initializes
     // with it. The _executableOutputs will be constructed using the expressions from the
     // '_outputFields' on the first call to doGetNext(). As a result, only expressions in the
-    // '_outputFeilds' are optimized here.
-    for (auto&& outputField : _outputFields) {
-        outputField.expr->optimize();
+    // '_outputFields' are optimized here.
+    if (_outputFields.size() > 0) {
+        // Calculate the new expression SBE compatibility after optimization without overwriting
+        // the previous SBE compatibility value. See the optimize() function for $group for a more
+        // detailed explanation.
+        auto expCtx = _outputFields[0].expr->expCtx();
+        auto origSbeCompatibility = expCtx->sbeCompatibility;
+        expCtx->sbeCompatibility = SbeCompatibility::noRequirements;
+
+        for (auto&& outputField : _outputFields) {
+            outputField.expr->optimize();
+        }
+
+        _sbeCompatibility = std::min(_sbeCompatibility, expCtx->sbeCompatibility);
+        expCtx->sbeCompatibility = origSbeCompatibility;
     }
     return this;
 }
@@ -339,17 +379,20 @@ boost::intrusive_ptr<DocumentSource> DocumentSourceInternalSetWindowFields::crea
         sortBy.emplace(*sortSpec, expCtx);
     }
 
+    expCtx->sbeWindowCompatibility = SbeCompatibility::noRequirements;
     std::vector<WindowFunctionStatement> outputFields;
     for (auto&& elem : spec.getOutput()) {
         outputFields.push_back(WindowFunctionStatement::parse(elem, sortBy, expCtx.get()));
     }
+    auto sbeCompatibility = std::min(expCtx->sbeWindowCompatibility, expCtx->sbeCompatibility);
 
     return make_intrusive<DocumentSourceInternalSetWindowFields>(
         expCtx,
         partitionBy,
         sortBy,
         outputFields,
-        internalDocumentSourceSetWindowFieldsMaxMemoryBytes.load());
+        internalDocumentSourceSetWindowFieldsMaxMemoryBytes.load(),
+        sbeCompatibility);
 }
 
 void DocumentSourceInternalSetWindowFields::initialize() {
@@ -481,7 +524,13 @@ DocumentSource::GetNextResult DocumentSourceInternalSetWindowFields::doGetNext()
             throw;
         }
 
-        if (!_memoryTracker.withinMemoryLimit() && _memoryTracker.allowDiskUse()) {
+        bool inMemoryLimit = _memoryTracker.withinMemoryLimit();
+        overrideMemoryLimitForSpill.execute([&](const BSONObj& data) {
+            _numDocsProcessed++;
+            inMemoryLimit = _numDocsProcessed <= data["maxDocsBeforeSpill"].numberInt();
+        });
+
+        if (!inMemoryLimit && _memoryTracker.allowDiskUse()) {
             // Attempt to spill where possible.
             _iterator.spillToDisk();
         }
