@@ -29,57 +29,49 @@
 
 #pragma once
 
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <cstddef>
+#include <cstdint>
+#include <list>
+#include <memory>
+#include <set>
+#include <string>
+#include <utility>
+#include <vector>
+
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/db/exec/document_value/document.h"
+#include "mongo/db/exec/document_value/value.h"
 #include "mongo/db/pipeline/accumulation_statement.h"
 #include "mongo/db/pipeline/accumulator.h"
+#include "mongo/db/pipeline/dependencies.h"
 #include "mongo/db/pipeline/document_source.h"
 #include "mongo/db/pipeline/document_source_set_window_fields_gen.h"
+#include "mongo/db/pipeline/expression.h"
+#include "mongo/db/pipeline/expression_context.h"
 #include "mongo/db/pipeline/expression_dependencies.h"
 #include "mongo/db/pipeline/field_path.h"
-#include "mongo/db/pipeline/memory_usage_tracker.h"
+#include "mongo/db/pipeline/pipeline.h"
+#include "mongo/db/pipeline/stage_constraints.h"
+#include "mongo/db/pipeline/variables.h"
 #include "mongo/db/pipeline/window_function/partition_iterator.h"
 #include "mongo/db/pipeline/window_function/window_bounds.h"
 #include "mongo/db/pipeline/window_function/window_function_exec.h"
-#include "mongo/db/pipeline/window_function/window_function_expression.h"
+#include "mongo/db/pipeline/window_function/window_function_statement.h"
+#include "mongo/db/query/query_shape/serialization_options.h"
+#include "mongo/db/query/sort_pattern.h"
+#include "mongo/util/intrusive_counter.h"
+#include "mongo/util/memory_usage_tracker.h"
+#include "mongo/util/string_map.h"
 
 namespace mongo {
 
-class WindowFunctionExec;
-
-struct WindowFunctionStatement {
-    std::string fieldName;  // top-level fieldname, not a path
-    boost::intrusive_ptr<window_function::Expression> expr;
-
-    WindowFunctionStatement(std::string fieldName,
-                            boost::intrusive_ptr<window_function::Expression> expr)
-        : fieldName(std::move(fieldName)), expr(std::move(expr)) {}
-
-    static WindowFunctionStatement parse(BSONElement elem,
-                                         const boost::optional<SortPattern>& sortBy,
-                                         ExpressionContext* expCtx);
-
-    void addDependencies(DepsTracker* deps) const {
-        if (expr) {
-            expr->addDependencies(deps);
-        }
-
-        const FieldPath path(fieldName);
-
-        // We do this because acting on "a.b" where a is an object also depends on "a" not being
-        // changed (e.g. to a non-object).
-        for (size_t i = 0; i < path.getPathLength() - 1; i++) {
-            deps->fields.insert(path.getSubpath(i).toString());
-        }
-    }
-
-    void addVariableRefs(std::set<Variables::Id>* refs) const {
-        if (expr) {
-            expr->addVariableRefs(refs);
-        }
-    }
-
-    void serialize(MutableDocument& outputFields, const SerializationOptions& opts) const;
-};
-
 /**
  * $setWindowFields is an alias: it desugars to some combination of projection, sorting,
  * and $_internalSetWindowFields.
@@ -93,8 +85,9 @@ std::list<boost::intrusive_ptr<DocumentSource>> createFromBson(
 std::list<boost::intrusive_ptr<DocumentSource>> create(
     const boost::intrusive_ptr<ExpressionContext>& expCtx,
     boost::optional<boost::intrusive_ptr<Expression>> partitionBy,
-    const boost::optional<SortPattern>& sortBy,
-    std::vector<WindowFunctionStatement> outputFields);
+    boost::optional<SortPattern> sortBy,
+    std::vector<WindowFunctionStatement> outputFields,
+    SbeCompatibility sbeCompatibility);
 
 }  // namespace document_source_set_window_fields
 
@@ -114,13 +107,15 @@ public:
         boost::optional<boost::intrusive_ptr<Expression>> partitionBy,
         boost::optional<SortPattern> sortBy,
         std::vector<WindowFunctionStatement> outputFields,
-        int64_t maxMemoryBytes)
+        int64_t maxMemoryBytes,
+        SbeCompatibility sbeCompatibility)
         : DocumentSource(kStageName, expCtx),
           _partitionBy(partitionBy),
           _sortBy(std::move(sortBy)),
           _outputFields(std::move(outputFields)),
           _memoryTracker{expCtx->allowDiskUse, maxMemoryBytes},
-          _iterator(expCtx.get(), pSource, &_memoryTracker, std::move(partitionBy), _sortBy){};
+          _iterator(expCtx.get(), pSource, &_memoryTracker, std::move(partitionBy), _sortBy),
+          _sbeCompatibility(sbeCompatibility){};
 
     GetModPathsReturn getModifiedPaths() const final {
         OrderedPathSet outputPaths;
@@ -143,7 +138,7 @@ public:
                                 UnionRequirement::kAllowed);
     }
 
-    const char* getSourceName() const {
+    const char* getSourceName() const override {
         return kStageName.rawData();
     };
 
@@ -176,16 +171,16 @@ public:
     Pipeline::SourceContainer::iterator doOptimizeAt(Pipeline::SourceContainer::iterator itr,
                                                      Pipeline::SourceContainer* container) final;
 
-    boost::optional<DistributedPlanLogic> distributedPlanLogic() {
+    boost::optional<DistributedPlanLogic> distributedPlanLogic() override {
         // Force to run on the merging half for now.
         return DistributedPlanLogic{nullptr, this, boost::none};
     }
 
     boost::intrusive_ptr<DocumentSource> optimize() final;
 
-    Value serialize(const SerializationOptions& opts = SerializationOptions{}) const final override;
+    Value serialize(const SerializationOptions& opts = SerializationOptions{}) const final;
 
-    DocumentSource::GetNextResult doGetNext();
+    DocumentSource::GetNextResult doGetNext() override;
 
     void setSource(DocumentSource* source) final {
         pSource = source;
@@ -196,6 +191,22 @@ public:
         return _iterator.usedDisk();
     };
 
+    SbeCompatibility sbeCompatibility() const {
+        return _sbeCompatibility;
+    }
+
+    boost::optional<boost::intrusive_ptr<Expression>> getPartitionBy() const {
+        return _partitionBy;
+    }
+
+    boost::optional<SortPattern> getSortBy() const {
+        return _sortBy;
+    }
+
+    const std::vector<WindowFunctionStatement>& getOutputFields() const {
+        return _outputFields;
+    }
+
 private:
     void initialize();
 
@@ -211,6 +222,9 @@ private:
     StringMap<std::unique_ptr<WindowFunctionExec>> _executableOutputs;
     bool _init = false;
     bool _eof = false;
+    // Used by the failpoint to determine when to spill to disk.
+    int32_t _numDocsProcessed = 0;
+    SbeCompatibility _sbeCompatibility = SbeCompatibility::noRequirements;
 };
 
 }  // namespace mongo
