@@ -29,16 +29,20 @@
 
 #include "mongo/db/pipeline/document_source_sharded_data_distribution.h"
 
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/bson/json.h"
 #include "mongo/db/pipeline/document_source_group.h"
 #include "mongo/db/pipeline/document_source_internal_all_collection_stats.h"
 #include "mongo/db/pipeline/document_source_lookup.h"
 #include "mongo/db/pipeline/document_source_match.h"
 #include "mongo/db/pipeline/document_source_project.h"
 #include "mongo/db/pipeline/expression_context.h"
-#include "mongo/db/pipeline/lite_parsed_document_source.h"
-#include "mongo/s/catalog/type_collection.h"
-
-#include "mongo/logv2/log.h"
+#include "mongo/db/query/allowed_contexts.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/intrusive_counter.h"
 
 namespace mongo {
 
@@ -74,16 +78,11 @@ list<intrusive_ptr<DocumentSource>> DocumentSourceShardedDataDistribution::creat
              "storageStats.numOrphanDocs": 1,
              "count": {$ifNull: ["$storageStats.count", 0]}, 
              "avgObjSize": {$ifNull: ["$storageStats.avgObjSize", 0]}, 
-             "numOrphanDocs": {$ifNull: ["$storageStats.numOrphanDocs", 0]} ,
+             "numOrphanDocs": {$ifNull: ["$storageStats.numOrphanDocs", 0]}, 
              "timeseries": {$ifNull: ["$storageStats.timeseries", null]}
          }
      })");
-
-    // Compute the `numOrphanedDocs` and `numOwnedDocuments` fields.
-    // Note that, for timeseries collections, these fields will report the number of buckets
-    // instead of the number of documents. We've decided to keep the field names as they are to
-    // avoid the downstream impact of having to check different fields depending on the collection
-    // time.
+    // We check if the collection is a timeseries in order to use the correct fields.
     static const BSONObj kGroupObj = fromjson(R"({
         $group: {
             _id: "$ns",
@@ -156,7 +155,7 @@ list<intrusive_ptr<DocumentSource>> DocumentSourceShardedDataDistribution::creat
             }
         }
     })");
-    static const BSONObj kLookupObj = fromjson(R"({
+    static const BSONObj kConfigCollectionsLookupObj = fromjson(R"({
          $lookup: {
             from: {
                 db: "config",
@@ -167,21 +166,73 @@ list<intrusive_ptr<DocumentSource>> DocumentSourceShardedDataDistribution::creat
             as: "matchingShardedCollection"
         }
     })");
-    static const BSONObj kMatchObj = fromjson("{$match: {matchingShardedCollection: {$ne: []}}}");
+    static const BSONObj kMatchObj = fromjson(R"({
+        $match: {
+            $and: [{
+                matchingShardedCollection: {$ne: []}
+            },
+            {
+                'matchingShardedCollection.unsplittable': {$ne: true}
+            }]
+            }
+    })");
+    // Adding a new field matchingShards that contains the shards that have a chunk
+    static const BSONObj kConfigChunksLookupObj = fromjson(R"({
+        "$lookup": {
+            "from": {
+                "db": "config",
+                "coll": "chunks"
+            },
+            "localField": "matchingShardedCollection.uuid",
+            "foreignField": "uuid",
+            "pipeline" : [
+                { "$project": { "shard": 1 } },
+                {
+                    "$group": {
+                        "_id": null,
+                        "shards": {
+                            "$addToSet": "$shard"
+                        }
+                    }
+                }
+            ],
+            "as": "matchingShards"
+        }    
+    })");
+
+    // We get rid of the db primary shards that do not contain a chunk and don't have any orphaned
+    // docs
     static const BSONObj kFinalProjectObj = fromjson(R"({
         $project: {
             _id: 0,
             ns: "$_id",
-            shards: "$shards"
+            shards: {
+                $filter: {
+                    input: "$shards",
+                    as: "shard",
+                    cond: {
+                        $or: [
+                            {
+                                $in: ["$$shard.shardName", {$first: "$matchingShards.shards"}]
+                            },
+                            {
+                                $ne: ["$$shard.numOrphanedDocs", 0]
+                            }
+                        ]
+
+                    }
+                }
+            }
         }
     })");
-
-    return {DocumentSourceInternalAllCollectionStats::createFromBsonInternal(
-                kAllCollStatsObj.firstElement(), expCtx),
-            DocumentSourceProject::createFromBson(kProjectObj.firstElement(), expCtx),
-            DocumentSourceGroup::createFromBson(kGroupObj.firstElement(), expCtx),
-            DocumentSourceLookUp::createFromBson(kLookupObj.firstElement(), expCtx),
-            DocumentSourceMatch::createFromBson(kMatchObj.firstElement(), expCtx),
-            DocumentSourceProject::createFromBson(kFinalProjectObj.firstElement(), expCtx)};
+    return {
+        DocumentSourceInternalAllCollectionStats::createFromBsonInternal(
+            kAllCollStatsObj.firstElement(), expCtx),
+        DocumentSourceProject::createFromBson(kProjectObj.firstElement(), expCtx),
+        DocumentSourceGroup::createFromBson(kGroupObj.firstElement(), expCtx),
+        DocumentSourceLookUp::createFromBson(kConfigCollectionsLookupObj.firstElement(), expCtx),
+        DocumentSourceMatch::createFromBson(kMatchObj.firstElement(), expCtx),
+        DocumentSourceLookUp::createFromBson(kConfigChunksLookupObj.firstElement(), expCtx),
+        DocumentSourceProject::createFromBson(kFinalProjectObj.firstElement(), expCtx)};
 }
 }  // namespace mongo
