@@ -27,23 +27,31 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
+#include <algorithm>
+#include <iterator>
 #include <memory>
+#include <string>
+#include <utility>
 
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsontypes.h"
 #include "mongo/db/exec/document_value/document.h"
 #include "mongo/db/exec/document_value/value.h"
 #include "mongo/db/exec/document_value/value_comparator.h"
 #include "mongo/db/pipeline/accumulation_statement.h"
-#include "mongo/db/pipeline/accumulator.h"
-#include "mongo/db/pipeline/document_source_group.h"
 #include "mongo/db/pipeline/document_source_streaming_group.h"
 #include "mongo/db/pipeline/expression.h"
 #include "mongo/db/pipeline/expression_context.h"
-#include "mongo/db/pipeline/expression_dependencies.h"
 #include "mongo/db/pipeline/lite_parsed_document_source.h"
-#include "mongo/db/stats/resource_consumption_metrics.h"
-#include "mongo/util/destructor_guard.h"
+#include "mongo/db/query/allowed_contexts.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/intrusive_counter.h"
 
 namespace mongo {
 
@@ -63,19 +71,24 @@ const char* DocumentSourceStreamingGroup::getSourceName() const {
 }
 
 DocumentSource::GetNextResult DocumentSourceStreamingGroup::doGetNext() {
-    auto getReadyResult = getNextReadyGroup();
-    if (!getReadyResult.isEOF()) {
-        return getReadyResult;
+    auto result = _groupProcessor.getNext();
+    if (result) {
+        return GetNextResult(std::move(*result));
     } else if (_sourceDepleted) {
         dispose();
-        return getReadyResult;
+        return GetNextResult::makeEOF();
     }
 
     auto prepareResult = readyNextBatch();
     if (prepareResult.isPaused()) {
         return prepareResult;
     }
-    return getNextReadyGroup();
+
+    result = _groupProcessor.getNext();
+    if (!result) {
+        return GetNextResult::makeEOF();
+    }
+    return GetNextResult(std::move(*result));
 }
 
 DocumentSourceStreamingGroup::DocumentSourceStreamingGroup(
@@ -91,9 +104,9 @@ boost::intrusive_ptr<DocumentSourceStreamingGroup> DocumentSourceStreamingGroup:
     boost::optional<int64_t> maxMemoryUsageBytes) {
     boost::intrusive_ptr<DocumentSourceStreamingGroup> groupStage =
         new DocumentSourceStreamingGroup(expCtx, maxMemoryUsageBytes);
-    groupStage->setIdExpression(groupByExpression);
+    groupStage->_groupProcessor.setIdExpression(groupByExpression);
     for (auto&& statement : accumulationStatements) {
-        groupStage->addAccumulator(statement);
+        groupStage->_groupProcessor.addAccumulationStatement(statement);
     }
     uassert(7026709,
             "streaming group must have at least one monotonic id expression",
@@ -102,7 +115,9 @@ boost::intrusive_ptr<DocumentSourceStreamingGroup> DocumentSourceStreamingGroup:
             "streaming group monotonic expression indexes must correspond to id expressions",
             std::all_of(monotonicExpressionIndexes.begin(),
                         monotonicExpressionIndexes.end(),
-                        [&](size_t i) { return i < groupStage->_idExpressions.size(); }));
+                        [&](size_t i) {
+                            return i < groupStage->_groupProcessor.getIdExpressions().size();
+                        }));
     groupStage->_monotonicExpressionIndexes = std::move(monotonicExpressionIndexes);
     return groupStage;
 }
@@ -120,13 +135,16 @@ boost::intrusive_ptr<DocumentSource> DocumentSourceStreamingGroup::createFromBso
         new DocumentSourceStreamingGroup(expCtx, maxMemoryUsageBytes);
     groupStage->initializeFromBson(elem);
 
-    const auto& monotonicIdFieldsElem = elem.Obj().getField(kMonotonicIdFieldsSpecField);
+    // BSONObj must outlive BSONElement. See BSONElement, BSONObj::getField().
+    auto obj = elem.Obj();
+    const auto& monotonicIdFieldsElem = obj.getField(kMonotonicIdFieldsSpecField);
     uassert(7026702,
             "streaming group must specify an array of monotonic id fields " +
                 kMonotonicIdFieldsSpecField,
             monotonicIdFieldsElem.type() == Array);
     const auto& monotonicIdFields = monotonicIdFieldsElem.Array();
-    if (groupStage->_idFieldNames.empty()) {
+    const auto& idFieldNames = groupStage->_groupProcessor.getIdFieldNames();
+    if (idFieldNames.empty()) {
         uassert(7026703,
                 "if there is no explicit id fields, " + kMonotonicIdFieldsSpecField +
                     " must contain a single \"_id\" string",
@@ -140,11 +158,10 @@ boost::intrusive_ptr<DocumentSource> DocumentSourceStreamingGroup::createFromBso
                     kMonotonicIdFieldsSpecField + " elements must be strings",
                     fieldNameElem.type() == String);
             StringData fieldName = fieldNameElem.valueStringData();
-            auto it = std::find(
-                groupStage->_idFieldNames.begin(), groupStage->_idFieldNames.end(), fieldName);
-            uassert(7026705, "id field not found", it != groupStage->_idFieldNames.end());
+            auto it = std::find(idFieldNames.begin(), idFieldNames.end(), fieldName);
+            uassert(7026705, "id field not found", it != idFieldNames.end());
             groupStage->_monotonicExpressionIndexes.push_back(
-                std::distance(groupStage->_idFieldNames.begin(), it));
+                std::distance(idFieldNames.begin(), it));
         }
         std::sort(groupStage->_monotonicExpressionIndexes.begin(),
                   groupStage->_monotonicExpressionIndexes.end());
@@ -156,11 +173,12 @@ boost::intrusive_ptr<DocumentSource> DocumentSourceStreamingGroup::createFromBso
 void DocumentSourceStreamingGroup::serializeAdditionalFields(
     MutableDocument& out, const SerializationOptions& opts) const {
     std::vector<Value> monotonicIdFields;
-    if (_idFieldNames.empty()) {
+    const auto& idFieldNames = _groupProcessor.getIdFieldNames();
+    if (idFieldNames.empty()) {
         monotonicIdFields.emplace_back(opts.serializeFieldPath("_id"));
     } else {
         for (size_t i : _monotonicExpressionIndexes) {
-            monotonicIdFields.emplace_back(opts.serializeFieldPathFromString(_idFieldNames[i]));
+            monotonicIdFields.emplace_back(opts.serializeFieldPathFromString(idFieldNames[i]));
         }
     }
     out[kMonotonicIdFieldsSpecField] = Value(std::move(monotonicIdFields));
@@ -180,7 +198,7 @@ DocumentSource::GetNextResult DocumentSourceStreamingGroup::getNextDocument() {
 }
 
 DocumentSource::GetNextResult DocumentSourceStreamingGroup::readyNextBatch() {
-    resetReadyGroups();
+    _groupProcessor.reset();
     GetNextResult input = getNextDocument();
     return readyNextBatchInner(input);
 }
@@ -189,23 +207,20 @@ DocumentSource::GetNextResult DocumentSourceStreamingGroup::readyNextBatch() {
 // and prevent stack overflows.
 MONGO_COMPILER_NOINLINE DocumentSource::GetNextResult
 DocumentSourceStreamingGroup::readyNextBatchInner(GetNextResult input) {
-    setExecutionStarted();
+    _groupProcessor.setExecutionStarted();
     // Calculate groups until we either exaust pSource or encounter change in monotonic id
     // expression, which means all current groups are finalized.
     for (; input.isAdvanced(); input = pSource->getNext()) {
-        if (shouldSpillWithAttemptToSaveMemory()) {
-            spill();
-        }
         auto root = input.releaseDocument();
-        Value id = computeId(root);
+        auto groupKey = _groupProcessor.computeGroupKey(root);
 
-        if (isBatchFinished(id)) {
+        if (isBatchFinished(groupKey)) {
             _firstDocumentOfNextBatch = std::move(root);
-            readyGroups();
+            _groupProcessor.readyGroups();
             return input;
         }
 
-        processDocument(id, root);
+        _groupProcessor.add(groupKey, root);
     }
 
     switch (input.getStatus()) {
@@ -216,7 +231,7 @@ DocumentSourceStreamingGroup::readyNextBatchInner(GetNextResult input) {
             return input;  // Propagate pause.
         }
         case DocumentSource::GetNextResult::ReturnStatus::kEOF: {
-            readyGroups();
+            _groupProcessor.readyGroups();
             _sourceDepleted = true;
             return input;
         }
@@ -225,7 +240,7 @@ DocumentSourceStreamingGroup::readyNextBatchInner(GetNextResult input) {
 }
 
 bool DocumentSourceStreamingGroup::isBatchFinished(const Value& id) {
-    if (_idExpressions.size() == 1) {
+    if (_groupProcessor.getIdExpressions().size() == 1) {
         tassert(7026706,
                 "if there are no explicit id fields, it is only one monotonic expression with id 0",
                 _monotonicExpressionIndexes.size() == 1 && _monotonicExpressionIndexes[0] == 0);
