@@ -28,20 +28,36 @@
  */
 
 
-#include "mongo/platform/basic.h"
-
+#include <absl/container/flat_hash_map.h>
 #include <iterator>
 
-#include "mongo/db/commands/test_commands_enabled.h"
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/db/auth/action_type.h"
+#include "mongo/db/auth/resource_pattern.h"
+#include "mongo/db/exec/document_value/document.h"
 #include "mongo/db/pipeline/document_source_documents.h"
 #include "mongo/db/pipeline/document_source_match.h"
 #include "mongo/db/pipeline/document_source_queue.h"
 #include "mongo/db/pipeline/document_source_single_document_transformation.h"
 #include "mongo/db/pipeline/document_source_union_with.h"
 #include "mongo/db/pipeline/document_source_union_with_gen.h"
-#include "mongo/db/pipeline/search_helper.h"
+#include "mongo/db/pipeline/process_interface/mongo_process_interface.h"
+#include "mongo/db/query/allowed_contexts.h"
+#include "mongo/db/query/plan_summary_stats.h"
 #include "mongo/db/views/resolved_view.h"
+#include "mongo/idl/idl_parser.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/util/namespace_string_util.h"
+#include "mongo/util/str.h"
+#include "mongo/util/uuid.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
@@ -54,29 +70,31 @@ REGISTER_DOCUMENT_SOURCE(unionWith,
                          AllowedWithApiStrict::kAlways);
 
 namespace {
-void validatorCallback(const Pipeline& pipeline) {
-    const auto& sources = pipeline.getSources();
-    std::for_each(sources.begin(), sources.end(), [](auto& src) {
-        uassert(31441,
-                str::stream() << src->getSourceName()
-                              << " is not allowed within a $unionWith's sub-pipeline",
-                src->constraints().isAllowedInUnionPipeline());
-    });
-}
-
 std::unique_ptr<Pipeline, PipelineDeleter> buildPipelineFromViewDefinition(
     const boost::intrusive_ptr<ExpressionContext>& expCtx,
     ExpressionContext::ResolvedNamespace resolvedNs,
     std::vector<BSONObj> currentPipeline) {
 
+    auto validatorCallback = [](const Pipeline& pipeline) {
+        const auto& sources = pipeline.getSources();
+        std::for_each(sources.begin(), sources.end(), [](auto& src) {
+            uassert(31441,
+                    str::stream() << src->getSourceName()
+                                  << " is not allowed within a $unionWith's sub-pipeline",
+                    src->constraints().isAllowedInUnionPipeline());
+        });
+    };
+
     MakePipelineOptions opts;
     opts.attachCursorSource = false;
     // Only call optimize() here if we actually have a pipeline to resolve in the view definition.
     opts.optimize = !resolvedNs.pipeline.empty();
     opts.validator = validatorCallback;
 
+    auto newExpCtx = expCtx->copyForSubPipeline(expCtx->ns, resolvedNs.uuid);
+    newExpCtx->inUnionWith = true;
     return Pipeline::makePipelineFromViewDefinition(
-        expCtx->copyForSubPipeline(expCtx->ns, resolvedNs.uuid), resolvedNs, currentPipeline, opts);
+        newExpCtx, resolvedNs, std::move(currentPipeline), opts);
 }
 
 }  // namespace
@@ -88,7 +106,6 @@ DocumentSourceUnionWith::DocumentSourceUnionWith(
     if (!_pipeline->getContext()->ns.isOnInternalDb()) {
         globalOpCounters.gotNestedAggregate();
     }
-    _pipeline->getContext()->inUnionWith = true;
 }
 
 DocumentSourceUnionWith::DocumentSourceUnionWith(
@@ -120,8 +137,8 @@ void validateUnionWithCollectionlessPipeline(
     LOGV2_DEBUG(5909700,
                 4,
                 "$unionWith validating collectionless pipeline",
-                "pipeline"_attr = Pipeline::serializePipelineForLogging(*pipeline),
-                "first"_attr = redact(firstStageBson));
+                "pipeline"_attr = pipeline,
+                "first"_attr = firstStageBson);
     uassert(ErrorCodes::FailedToParse,
             errMsg,
             // TODO SERVER-59628 replace with constraints check
@@ -149,14 +166,12 @@ std::unique_ptr<DocumentSourceUnionWith::LiteParsed> DocumentSourceUnionWith::Li
     NamespaceString unionNss;
     boost::optional<LiteParsedPipeline> liteParsedPipeline;
     if (spec.type() == BSONType::String) {
-        unionNss =
-            NamespaceStringUtil::parseNamespaceFromRequest(nss.dbName(), spec.valueStringData());
+        unionNss = NamespaceStringUtil::deserialize(nss.dbName(), spec.valueStringData());
     } else {
         auto unionWithSpec =
             UnionWithSpec::parse(IDLParserContext(kStageName), spec.embeddedObject());
         if (unionWithSpec.getColl()) {
-            unionNss = NamespaceStringUtil::parseNamespaceFromRequest(nss.dbName(),
-                                                                      *unionWithSpec.getColl());
+            unionNss = NamespaceStringUtil::deserialize(nss.dbName(), *unionWithSpec.getColl());
         } else {
             // If no collection specified, it must have $documents as first field in pipeline.
             validateUnionWithCollectionlessPipeline(unionWithSpec.getPipeline());
@@ -192,8 +207,7 @@ PrivilegeVector DocumentSourceUnionWith::LiteParsed::requiredPrivileges(
     if (!_pipelines.empty()) {
         const LiteParsedPipeline& pipeline = _pipelines[0];
         Privilege::addPrivilegesToPrivilegeVector(
-            &requiredPrivileges,
-            std::move(pipeline.requiredPrivileges(isMongos, bypassDocumentValidation)));
+            &requiredPrivileges, pipeline.requiredPrivileges(isMongos, bypassDocumentValidation));
     }
     return requiredPrivileges;
 }
@@ -209,14 +223,13 @@ boost::intrusive_ptr<DocumentSource> DocumentSourceUnionWith::createFromBson(
     NamespaceString unionNss;
     std::vector<BSONObj> pipeline;
     if (elem.type() == BSONType::String) {
-        unionNss = NamespaceStringUtil::parseNamespaceFromRequest(expCtx->ns.dbName(),
-                                                                  elem.valueStringData());
+        unionNss = NamespaceStringUtil::deserialize(expCtx->ns.dbName(), elem.valueStringData());
     } else {
         auto unionWithSpec =
             UnionWithSpec::parse(IDLParserContext(kStageName), elem.embeddedObject());
         if (unionWithSpec.getColl()) {
-            unionNss = NamespaceStringUtil::parseNamespaceFromRequest(expCtx->ns.dbName(),
-                                                                      *unionWithSpec.getColl());
+            unionNss =
+                NamespaceStringUtil::deserialize(expCtx->ns.dbName(), *unionWithSpec.getColl());
         } else {
             // if no collection specified, it must have $documents as first field in pipeline
             validateUnionWithCollectionlessPipeline(unionWithSpec.getPipeline());
@@ -248,42 +261,29 @@ DocumentSource::GetNextResult DocumentSourceUnionWith::doGetNext() {
         auto serializedPipe = _pipeline->serializeToBson();
         logStartingSubPipeline(serializedPipe);
         try {
+            // Query settings are looked up after parsing and therefore are not populated in the
+            // context of the unionWith '_pipeline' as part of DocumentSourceUnionWith constructor.
+            // Attach query settings to the '_pipeline->getContext()' by copying them from the
+            // parent query ExpressionContext.
+            _pipeline->getContext()->setQuerySettingsIfNotPresent(pExpCtx->getQuerySettings());
+
             LOGV2_DEBUG(9497002,
                         5,
                         "$unionWith before pipeline prep: ",
                         "pipeline"_attr = _pipeline->serializeToBson());
-            // We determine whether we should dismiss disposal for a $search subpipeline here for a
-            // combination of reasons:
-            // 1. The subpipeline of the $search stage will have an invalid opCtx on its
-            // PipelineDeleter during getMore operations. After the call to
-            // attachCursorSourceToPipeline(), the subpipeline goes out of scope, and during
-            // disposal, its invalid opCtx is set to the opCtx used for _pipeline execution. This
-            // results in the _pipeline executing with an invalid opCtx, which leads to errors.
-            // 2. To address an invalid opCtx on the pipelineDeleter, there is a dismissDisposal()
-            // to prevent pipelines from disposing of themselves. This allows an owner to dispose of
-            // a pipeline with a valid opCtx. The $search stage must call dismissDisposal() on its
-            // subpipeline and then ensure its proper disposal with a valid opCtx.
-            // 3. Calling dismissDisposal() for a subpipeline is only safe when its parent document
-            // source is part of a pipeline as the pipeline object handles calling disposal for its
-            // document sources. Generally, we can be sure that a document source is part of a
-            // pipeline during doGetNext(). Since attachCursorSourceToPipeline() and setup is done
-            // prior to _pipeline execution, we cannot dismiss disposal for the $search subpipeline
-            // during its doGetNext() - as that occurs after setup - so we add some $search logic
-            // here, where we know that the $search stage is in a pipeline.
-            _pipeline = getSearchHelpers(pExpCtx->opCtx->getServiceContext())
-                            ->dismissSearchSubpipelineDisposal(std::move(_pipeline));
             _pipeline =
-                pExpCtx->mongoProcessInterface->attachCursorSourceToPipeline(_pipeline.release());
+                pExpCtx->mongoProcessInterface->preparePipelineForExecution(_pipeline.release());
             LOGV2_DEBUG(9497003,
                         5,
                         "$unionWith POST pipeline prep: ",
                         "pipeline"_attr = _pipeline->serializeToBson());
+
             _executionState = ExecutionProgress::kIteratingSubPipeline;
         } catch (const ExceptionFor<ErrorCodes::CommandOnShardedViewNotSupportedOnMongod>& e) {
             _pipeline = buildPipelineFromViewDefinition(
                 pExpCtx,
                 ExpressionContext::ResolvedNamespace{e->getNamespace(), e->getPipeline()},
-                serializedPipe);
+                std::move(serializedPipe));
             logShardedViewFound(e);
             return doGetNext();
         }
@@ -316,7 +316,7 @@ MONGO_COMPILER_NOINLINE void DocumentSourceUnionWith::logStartingSubPipeline(
 }
 
 MONGO_COMPILER_NOINLINE void DocumentSourceUnionWith::logShardedViewFound(
-    const ExceptionFor<ErrorCodes::CommandOnShardedViewNotSupportedOnMongod>& e) {
+    const ExceptionFor<ErrorCodes::CommandOnShardedViewNotSupportedOnMongod>& e) const {
     LOGV2_DEBUG(4556300,
                 3,
                 "$unionWith found view definition. ns: {namespace}, pipeline: {pipeline}. New "
@@ -384,7 +384,8 @@ Value DocumentSourceUnionWith::serialize(const SerializationOptions& opts) const
         //  $limit stage after the $unionWith which results in only reading from the base collection
         //  branch and not the sub-pipeline.
         Pipeline* pipeCopy = nullptr;
-        if (*opts.verbosity == ExplainOptions::Verbosity::kQueryPlanner) {
+        if (*opts.verbosity == ExplainOptions::Verbosity::kQueryPlanner ||
+            *opts.verbosity == ExplainOptions::Verbosity::kQueryPlannerDebug) {
             pipeCopy = Pipeline::create(_pipeline->getSources(), _pipeline->getContext()).release();
         } else if (*opts.verbosity >= ExplainOptions::Verbosity::kExecStats &&
                    _executionState > ExecutionProgress::kIteratingSource) {
@@ -399,9 +400,9 @@ Value DocumentSourceUnionWith::serialize(const SerializationOptions& opts) const
             std::move(_pushedDownStages.begin(),
                       _pushedDownStages.end(),
                       std::back_inserter(recoveredPipeline));
-            pipeCopy =
-                Pipeline::parse(recoveredPipeline, _pipeline->getContext(), validatorCallback)
-                    .release();
+            // TODO SERVER-94227 we probably don't need to do any validation as part of this parsing
+            // pass?
+            pipeCopy = Pipeline::parse(recoveredPipeline, _pipeline->getContext()).release();
         } else {
             // The plan does not require reading from the sub-pipeline, so just include the
             // serialization in the explain output.
@@ -416,8 +417,36 @@ Value DocumentSourceUnionWith::serialize(const SerializationOptions& opts) const
         }
 
         invariant(pipeCopy);
-        BSONObj explainLocal =
-            pExpCtx->mongoProcessInterface->preparePipelineAndExplain(pipeCopy, *opts.verbosity);
+
+        auto preparePipelineAndExplain = [&](Pipeline* ownedPipeline) {
+            // Query settings are looked up after parsing and therefore are not populated in the
+            // context of the unionWith '_pipeline' as part of DocumentSourceUnionWith
+            // constructor. Attach query settings to the '_pipeline->getContext()' by copying
+            // them from the parent query ExpressionContext.
+            //
+            // NOTE: this is done here, as opposed to at the beginning of the serialize() method
+            // because serialize() is called when generating query shape, however, at that
+            // moment no query settings are present in the parent context.
+            _pipeline->getContext()->setQuerySettingsIfNotPresent(pExpCtx->getQuerySettings());
+
+            return pExpCtx->mongoProcessInterface->preparePipelineAndExplain(ownedPipeline,
+                                                                             *opts.verbosity);
+        };
+
+        BSONObj explainLocal = [&] {
+            auto serializedPipe = pipeCopy->serializeToBson();
+            try {
+                return preparePipelineAndExplain(pipeCopy);
+            } catch (const ExceptionFor<ErrorCodes::CommandOnShardedViewNotSupportedOnMongod>& e) {
+                logShardedViewFound(e);
+                auto resolvedPipeline = buildPipelineFromViewDefinition(
+                    pExpCtx,
+                    ExpressionContext::ResolvedNamespace{e->getNamespace(), e->getPipeline()},
+                    std::move(serializedPipe));
+                return preparePipelineAndExplain(resolvedPipeline.release());
+            }
+        }();
+
         LOGV2_DEBUG(4553501, 3, "$unionWith attached cursor to pipeline for explain");
         // We expect this to be an explanation of a pipeline -- there should only be one field.
         invariant(explainLocal.nFields() == 1);
@@ -434,7 +463,9 @@ Value DocumentSourceUnionWith::serialize(const SerializationOptions& opts) const
         auto serializedPipeline = [&]() -> std::vector<BSONObj> {
             if (opts.transformIdentifiers ||
                 opts.literalPolicy != LiteralSerializationPolicy::kUnchanged) {
-                return Pipeline::parse(_userPipeline, _pipeline->getContext(), validatorCallback)
+                // TODO SERVER-94227 we don't need to do any validation as part of this parsing
+                // pass.
+                return Pipeline::parse(_userPipeline, _pipeline->getContext())
                     ->serializeToBson(opts);
             }
             return _pipeline->serializeToBson(opts);
