@@ -29,10 +29,18 @@
 
 #include <utility>
 
+#include <absl/container/node_hash_map.h>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+
+#include "mongo/bson/timestamp.h"
+#include "mongo/db/basic_types.h"
+#include "mongo/db/feature_compatibility_version_documentation.h"
 #include "mongo/db/pipeline/aggregate_command_gen.h"
 #include "mongo/db/pipeline/expression_context.h"
 #include "mongo/db/pipeline/process_interface/stub_mongo_process_interface.h"
-#include "mongo/db/query/collation/collator_factory_interface.h"
 #include "mongo/db/stats/counters.h"
 #include "mongo/util/intrusive_counter.h"
 
@@ -84,13 +92,44 @@ ExpressionContext::ExpressionContext(OperationContext* opCtx,
               return boost::none;
           }(),
           findCmd.getLet(),
-          mayDbProfile) {}
+          mayDbProfile,
+          findCmd.getSerializationContext()) {}
+
+
+ExpressionContext::ExpressionContext(OperationContext* opCtx,
+                                     const DistinctCommandRequest& distinctCmd,
+                                     const NamespaceString& nss,
+                                     std::unique_ptr<CollatorInterface> collator,
+                                     bool mayDbProfile,
+                                     boost::optional<ExplainOptions::Verbosity> verbosity)
+    : ExpressionContext(
+          opCtx,
+          verbosity,
+          false,  // fromMongos
+          false,  // needsMerge
+          false,  // allowDiskUse
+          false,  // bypassDocumentValidation
+          false,  // isMapReduceCommand
+          nss,
+          boost::none,  // legacyRuntimeConstraints
+          std::move(collator),
+          nullptr,  // mongoProcessInterface
+          {},       // resolvedNamespaces
+          [&distinctCmd]() -> boost::optional<UUID> {
+              if (distinctCmd.getNamespaceOrUUID().isUUID()) {
+                  return distinctCmd.getNamespaceOrUUID().uuid();
+              }
+              return boost::none;
+          }(),
+          boost::none,  // letParameters
+          mayDbProfile,
+          distinctCmd.getSerializationContext()) {}
 
 ExpressionContext::ExpressionContext(OperationContext* opCtx,
                                      const AggregateCommandRequest& request,
                                      std::unique_ptr<CollatorInterface> collator,
                                      std::shared_ptr<MongoProcessInterface> processInterface,
-                                     ResolvedNamespaceMap resolvedNamespaces,
+                                     StringMap<ResolvedNamespace> resolvedNamespaces,
                                      boost::optional<UUID> collUUID,
                                      bool mayDbProfile,
                                      bool allowDiskUseByDefault)
@@ -104,11 +143,12 @@ ExpressionContext::ExpressionContext(OperationContext* opCtx,
                         request.getNamespace(),
                         request.getLegacyRuntimeConstants(),
                         std::move(collator),
-                        std::move(processInterface),
+                        processInterface,
                         std::move(resolvedNamespaces),
                         std::move(collUUID),
                         request.getLet(),
-                        mayDbProfile) {
+                        mayDbProfile,
+                        request.getSerializationContext()) {
 
     if (request.getIsMapReduceCommand()) {
         // mapReduce command JavaScript invocation is only subject to the server global
@@ -130,17 +170,21 @@ ExpressionContext::ExpressionContext(
     const boost::optional<LegacyRuntimeConstants>& runtimeConstants,
     std::unique_ptr<CollatorInterface> collator,
     const std::shared_ptr<MongoProcessInterface>& mongoProcessInterface,
-    ResolvedNamespaceMap resolvedNamespaces,
+    StringMap<ExpressionContext::ResolvedNamespace> resolvedNamespaces,
     boost::optional<UUID> collUUID,
     const boost::optional<BSONObj>& letParameters,
-    bool mayDbProfile)
+    bool mayDbProfile,
+    const SerializationContext& serializationCtx)
     : explain(explain),
       fromMongos(fromMongos),
       needsMerge(needsMerge),
-      allowDiskUse(allowDiskUse &&
-                   !(opCtx && opCtx->readOnly())),  // Disallow disk use if in read-only mode.
+      allowDiskUse(allowDiskUse && ([&]() {
+                       tassert(7738401, "opCtx null check", opCtx);
+                       return !(opCtx->readOnly());
+                   }())),  // Disallow disk use if in read-only mode.
       bypassDocumentValidation(bypassDocumentValidation),
       ns(ns),
+      serializationCtxt(serializationCtx),
       uuid(std::move(collUUID)),
       opCtx(opCtx),
       mongoProcessInterface(mongoProcessInterface),
@@ -148,10 +192,13 @@ ExpressionContext::ExpressionContext(
       variablesParseState(variables.useIdGenerator()),
       mayDbProfile(mayDbProfile),
       _collator(std::move(collator)),
-      _documentComparator(_collator.get()),
-      _valueComparator(_collator.get()),
+      _documentComparator(_collator.getCollator()),
+      _valueComparator(_collator.getCollator()),
       _resolvedNamespaces(std::move(resolvedNamespaces)) {
 
+    // Only initialize 'variables' if we are given a runtimeConstants object. We delay initializing
+    // variables and expect callers to invoke 'initializeReferencedSystemVariables()' after query
+    // parsing. This allows us to only initialize variables which are used in the query.
     if (runtimeConstants && runtimeConstants->getClusterTime().isNull()) {
         // Try to get a default value for clusterTime if a logical clock exists.
         auto genConsts = variables.generateRuntimeConstants(opCtx);
@@ -161,8 +208,6 @@ ExpressionContext::ExpressionContext(
         variables.setLegacyRuntimeConstants(genConsts);
     } else if (runtimeConstants) {
         variables.setLegacyRuntimeConstants(*runtimeConstants);
-    } else {
-        variables.setDefaultRuntimeConstants(opCtx);
     }
 
     if (!isMapReduce) {
@@ -178,9 +223,11 @@ ExpressionContext::ExpressionContext(
     const NamespaceString& nss,
     const boost::optional<LegacyRuntimeConstants>& runtimeConstants,
     const boost::optional<BSONObj>& letParameters,
+    bool allowDiskUse,
     bool mayDbProfile,
     boost::optional<ExplainOptions::Verbosity> explain)
     : explain(explain),
+      allowDiskUse(allowDiskUse),
       ns(nss),
       opCtx(opCtx),
       mongoProcessInterface(std::make_shared<StubMongoProcessInterface>()),
@@ -190,8 +237,8 @@ ExpressionContext::ExpressionContext(
       variablesParseState(variables.useIdGenerator()),
       mayDbProfile(mayDbProfile),
       _collator(std::move(collator)),
-      _documentComparator(_collator.get()),
-      _valueComparator(_collator.get()) {
+      _documentComparator(_collator.getCollator()),
+      _valueComparator(_collator.getCollator()) {
     if (runtimeConstants) {
         variables.setLegacyRuntimeConstants(*runtimeConstants);
     }
@@ -217,8 +264,8 @@ ExpressionContext::ExpressionContext(OperationContext* opCtx,
       maxFeatureCompatibilityVersion(boost::none),  // Ensure all features are allowed.
       mayDbProfile(true),
       _collator(nullptr),
-      _documentComparator(_collator.get()),
-      _valueComparator(_collator.get()) {
+      _documentComparator(_collator.getCollator()),
+      _valueComparator(_collator.getCollator()) {
     // This is a shortcut to avoid reading the clock and the vector clock, since we don't actually
     // care about their values for this 'blank' ExpressionContext codepath.
     variables.setLegacyRuntimeConstants({Date_t::min(), Timestamp()});
@@ -248,7 +295,7 @@ void ExpressionContext::checkForInterruptSlow() {
 
 ExpressionContext::CollatorStash::CollatorStash(ExpressionContext* const expCtx,
                                                 std::unique_ptr<CollatorInterface> newCollator)
-    : _expCtx(expCtx), _originalCollator(std::move(_expCtx->_collator)) {
+    : _expCtx(expCtx), _originalCollator(_expCtx->_collator.getCollatorShared()) {
     _expCtx->setCollator(std::move(newCollator));
 }
 
@@ -266,10 +313,15 @@ boost::intrusive_ptr<ExpressionContext> ExpressionContext::copyWith(
     NamespaceString ns,
     boost::optional<UUID> uuid,
     boost::optional<std::unique_ptr<CollatorInterface>> updatedCollator) const {
-
-    auto collator = updatedCollator
-        ? std::move(*updatedCollator)
-        : (_collator ? _collator->clone() : std::unique_ptr<CollatorInterface>{});
+    auto collator = [&]() {
+        if (updatedCollator) {
+            return std::move(*updatedCollator);
+        } else if (_collator.getCollator()) {
+            return _collator.getCollator()->clone();
+        } else {
+            return std::unique_ptr<CollatorInterface>();
+        }
+    }();
 
     auto expCtx = make_intrusive<ExpressionContext>(opCtx,
                                                     explain,
@@ -285,7 +337,12 @@ boost::intrusive_ptr<ExpressionContext> ExpressionContext::copyWith(
                                                     _resolvedNamespaces,
                                                     uuid,
                                                     boost::none /* letParameters */,
-                                                    mayDbProfile);
+                                                    mayDbProfile,
+                                                    SerializationContext());
+
+    if (_collator.getIgnore()) {
+        expCtx->setIgnoreCollator();
+    }
 
     expCtx->inMongos = inMongos;
     expCtx->maxFeatureCompatibilityVersion = maxFeatureCompatibilityVersion;
@@ -306,6 +363,9 @@ boost::intrusive_ptr<ExpressionContext> ExpressionContext::copyWith(
     expCtx->originalAggregateCommand = originalAggregateCommand.getOwned();
 
     expCtx->inLookup = inLookup;
+    expCtx->serializationCtxt = serializationCtxt;
+
+    expCtx->_querySettings = _querySettings;
 
     // Note that we intentionally skip copying the value of '_interruptCounter' because 'expCtx' is
     // intended to be used for executing a separate aggregation pipeline.
@@ -355,20 +415,55 @@ void ExpressionContext::stopExpressionCounters() {
     _expressionCounters.reset();
 }
 
-void ExpressionContext::setUserRoles() {
-    // Only set the value of $$USER_ROLES if it is referenced in the query.
-    // We need to check the FCV here because the $$USER_ROLES variable will always appear in the
-    // serialized command when one shard is sending a sub-query to another shard. The query will
-    // fail in the case where the shards are running different binVersions and one of them does not
-    // have a notion of this variable. This FCV check prevents this from happening, as the value of
-    // the variable is not set (and therefore not serialized) if the FCV is too old.
-    const auto fcvSnapshot = serverGlobalParams.featureCompatibility.acquireFCVSnapshot();
-    if (isSystemVarReferencedInQuery(Variables::kUserRolesId) &&
-        fcvSnapshot.isVersionInitialized() &&
-        feature_flags::gFeatureFlagUserRoles.isEnabled(fcvSnapshot) &&
-        enableAccessToUserRoles.load()) {
+void ExpressionContext::initializeReferencedSystemVariables() {
+    if (_systemVarsReferencedInQuery.contains(Variables::kNowId) &&
+        !variables.hasValue(Variables::kNowId)) {
+        variables.defineLocalNow();
+    }
+    if (_systemVarsReferencedInQuery.contains(Variables::kClusterTimeId) &&
+        !variables.hasValue(Variables::kClusterTimeId)) {
+        variables.defineClusterTime(opCtx);
+    }
+    if (_systemVarsReferencedInQuery.contains(Variables::kUserRolesId) &&
+        !variables.hasValue(Variables::kUserRolesId) && enableAccessToUserRoles.load()) {
         variables.defineUserRoles(opCtx);
     }
 }
 
+void ExpressionContext::throwIfFeatureFlagIsNotEnabledOnFCV(
+    StringData name, const boost::optional<FeatureFlag>& flag) {
+    if (!flag) {
+        return;
+    }
+
+    // If the FCV is not initialized yet, we check whether the feature flag is enabled on the last
+    // LTS FCV, which is the lowest FCV we can have on this server. If the FCV is set, then we
+    // should check if the flag is enabled on maxFeatureCompatibilityVersion or the current FCV. If
+    // both the FCV is uninitialized and maxFeatureCompatibilityVersion is set, to be safe, we
+    // should check the lowest FCV. We are guaranteed that maxFeatureCompatibilityVersion will
+    // always be greater than or equal to the last LTS. So we will check the last LTS.
+    const auto fcv = serverGlobalParams.featureCompatibility.acquireFCVSnapshot();
+    mongo::multiversion::FeatureCompatibilityVersion versionToCheck = fcv.getVersion();
+    if (!fcv.isVersionInitialized()) {
+        // (Generic FCV reference): This FCV reference should exist across LTS binary versions.
+        versionToCheck = multiversion::GenericFCV::kLastLTS;
+    } else if (maxFeatureCompatibilityVersion) {
+        versionToCheck = *maxFeatureCompatibilityVersion;
+    }
+
+    uassert(ErrorCodes::QueryFeatureNotAllowed,
+            // We would like to include the current version and the required minimum version in this
+            // error message, but using FeatureCompatibilityVersion::toString() would introduce a
+            // dependency cycle (see SERVER-31968).
+            str::stream() << name
+                          << " is not allowed in the current feature compatibility version. See "
+                          << feature_compatibility_version_documentation::kCompatibilityLink
+                          << " for more information.",
+            flag->isEnabledOnVersion(versionToCheck));
+}
+
+bool ExpressionContext::isFeatureFlagBinDataConvertEnabled() {
+    return _featureFlagBinDataConvertValue.get();
+}
+
 }  // namespace mongo
