@@ -29,16 +29,29 @@
 
 #pragma once
 
-#include <absl/container/node_hash_map.h>
+#include "mongo/db/query/query_feature_flags_gen.h"
+#include <absl/container/node_hash_set.h>
 #include <boost/intrusive_ptr.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
 #include <boost/optional.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <cstdint>
 #include <memory>
 #include <string>
+#include <utility>
 #include <vector>
 
+#include "mongo/base/error_codes.h"
 #include "mongo/base/string_data.h"
 #include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/client/dbclient_cursor.h"
+#include "mongo/db/basic_types_gen.h"
+#include "mongo/db/exec/document_value/document.h"
 #include "mongo/db/exec/document_value/document_comparator.h"
+#include "mongo/db/exec/document_value/value.h"
 #include "mongo/db/exec/document_value/value_comparator.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/operation_context.h"
@@ -46,16 +59,30 @@
 #include "mongo/db/pipeline/javascript_execution.h"
 #include "mongo/db/pipeline/legacy_runtime_constants_gen.h"
 #include "mongo/db/pipeline/process_interface/mongo_process_interface.h"
+#include "mongo/db/pipeline/resume_token.h"
 #include "mongo/db/pipeline/variables.h"
+#include "mongo/db/query/collation/collation_spec.h"
 #include "mongo/db/query/collation/collator_interface.h"
 #include "mongo/db/query/datetime/date_time_support.h"
+#include "mongo/db/query/distinct_command_gen.h"
 #include "mongo/db/query/explain_options.h"
+#include "mongo/db/query/find_command.h"
+#include "mongo/db/query/query_knob_configuration.h"
 #include "mongo/db/query/query_knobs_gen.h"
+#include "mongo/db/query/query_settings/query_settings_gen.h"
 #include "mongo/db/query/tailable_mode.h"
-#include "mongo/db/server_options.h"
+#include "mongo/db/query/tailable_mode_gen.h"
+#include "mongo/db/query/util/deferred.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/scripting/engine.h"
+#include "mongo/stdx/unordered_set.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/intrusive_counter.h"
+#include "mongo/util/serialization_context.h"
+#include "mongo/util/str.h"
 #include "mongo/util/string_map.h"
 #include "mongo/util/uuid.h"
+#include "mongo/util/version/releases.h"
 
 namespace mongo {
 
@@ -64,16 +91,17 @@ class AggregateCommandRequest;
 enum struct SbeCompatibility {
     // Not implemented in SBE.
     notCompatible,
-    // Implemented in SBE but behind the featureFlagSbeFull flag.
-    flagGuarded,
-    // Implemented in SBE and enabled by default.
-    fullyCompatible,
+    // Requires "featureFlagSbeFull" to be set. New SBE features which are under
+    // development can live under this feature flag until they are ready to be shipped.
+    requiresSbeFull,
+    // Requires the framework control knob to be "trySbeEngine". Fully tested, complete
+    // SBE features belong here.
+    requiresTrySbe,
+    // Used for the narrow feature set that we expose when "trySbeRestricted" is on. These features
+    // are always SBE compatible unless SBE is completely disabled with "forceClassicEngine".
+    noRequirements,
 };
 
-inline bool operator<(SbeCompatibility a, SbeCompatibility b) {
-    return static_cast<int>(a) < static_cast<int>(b);
-}
-
 class ExpressionContext : public RefCountable {
 public:
     struct ResolvedNamespace {
@@ -87,11 +115,6 @@ public:
         boost::optional<UUID> uuid = boost::none;
     };
 
-    // "ResolvedNamespaceMap" is used for external references to this type, but it needs
-    // ResolvedNamespace to be defined so it is declared after this class definition.
-    using InternalResolvedNamespaceMapType =
-        absl::flat_hash_map<NamespaceString, ResolvedNamespace>;
-
     /**
      * An RAII type that will temporarily change the ExpressionContext's collator. Resets the
      * collator to the previous value upon destruction.
@@ -118,7 +141,7 @@ public:
 
         boost::intrusive_ptr<ExpressionContext> _expCtx;
 
-        std::unique_ptr<CollatorInterface> _originalCollator;
+        std::shared_ptr<CollatorInterface> _originalCollator;
     };
 
     /**
@@ -142,6 +165,12 @@ public:
                       boost::optional<ExplainOptions::Verbosity> verbosity = boost::none,
                       bool allowDiskUseByDefault = false);
 
+    ExpressionContext(OperationContext* opCtx,
+                      const DistinctCommandRequest& distinctCmd,
+                      const NamespaceString& nss,
+                      std::unique_ptr<CollatorInterface> collator,
+                      bool mayDbProfile,
+                      boost::optional<ExplainOptions::Verbosity> verbosity);
     /**
      * Constructs an ExpressionContext to be used for Pipeline parsing and evaluation.
      * 'resolvedNamespaces' maps collection names (not full namespaces) to ResolvedNamespaces.
@@ -150,7 +179,7 @@ public:
                       const AggregateCommandRequest& request,
                       std::unique_ptr<CollatorInterface> collator,
                       std::shared_ptr<MongoProcessInterface> mongoProcessInterface,
-                      InternalResolvedNamespaceMapType resolvedNamespaces,
+                      StringMap<ExpressionContext::ResolvedNamespace> resolvedNamespaces,
                       boost::optional<UUID> collUUID,
                       bool mayDbProfile = true,
                       bool allowDiskUseByDefault = false);
@@ -171,10 +200,11 @@ public:
                       const boost::optional<LegacyRuntimeConstants>& runtimeConstants,
                       std::unique_ptr<CollatorInterface> collator,
                       const std::shared_ptr<MongoProcessInterface>& mongoProcessInterface,
-                      InternalResolvedNamespaceMapType resolvedNamespaces,
+                      StringMap<ExpressionContext::ResolvedNamespace> resolvedNamespaces,
                       boost::optional<UUID> collUUID,
                       const boost::optional<BSONObj>& letParameters = boost::none,
-                      bool mayDbProfile = true);
+                      bool mayDbProfile = true,
+                      const SerializationContext& serializationCtx = SerializationContext());
 
     /**
      * Constructs an ExpressionContext suitable for use outside of the aggregation system, including
@@ -187,6 +217,7 @@ public:
                       const NamespaceString& ns,
                       const boost::optional<LegacyRuntimeConstants>& runtimeConstants = boost::none,
                       const boost::optional<BSONObj>& letParameters = boost::none,
+                      bool allowDiskUse = false,
                       bool mayDbProfile = true,
                       boost::optional<ExplainOptions::Verbosity> explain = boost::none);
 
@@ -219,8 +250,8 @@ public:
     /**
      * Returns true if this is a collectionless aggregation on the specified database.
      */
-    bool isDBAggregation(StringData dbName) const {
-        return ns.db() == dbName && ns.isCollectionlessAggregateNS();
+    bool isDBAggregation(const NamespaceString& other) const {
+        return ns.isEqualDb(other) && ns.isCollectionlessAggregateNS();
     }
 
     /**
@@ -238,7 +269,11 @@ public:
     }
 
     const CollatorInterface* getCollator() const {
-        return _collator.get();
+        return _collator.getCollator();
+    }
+
+    std::shared_ptr<CollatorInterface> getCollatorShared() const {
+        return _collator.getCollatorShared();
     }
 
     /**
@@ -258,7 +293,11 @@ public:
      * the ExpressionContext.
      */
     BSONObj getCollatorBSON() const {
-        return _collator ? _collator->getSpec().toBSON() : CollationSpec::kSimpleSpec;
+        if (_collator.getIgnore()) {
+            return BSONObj();
+        }
+        auto* collator = _collator.getCollator();
+        return collator ? collator->getSpec().toBSON() : CollationSpec::kSimpleSpec;
     }
 
     /**
@@ -267,12 +306,13 @@ public:
      * Use with caution - '_collator' is used in the context of a Pipeline, and it is illegal
      * to change the collation once a Pipeline has been parsed with this ExpressionContext.
      */
-    void setCollator(std::unique_ptr<CollatorInterface> collator) {
-        _collator = std::move(collator);
+    void setCollator(std::shared_ptr<CollatorInterface> collator) {
+        _collator.setCollator(std::move(collator));
 
         // Document/Value comparisons must be aware of the collation.
-        _documentComparator = DocumentComparator(_collator.get());
-        _valueComparator = ValueComparator(_collator.get());
+        auto* ptr = _collator.getCollator();
+        _documentComparator = DocumentComparator(ptr);
+        _valueComparator = ValueComparator(ptr);
     }
 
     const DocumentComparator& getDocumentComparator() const {
@@ -304,11 +344,14 @@ public:
      * and 'needsMerge' fields.
      */
     boost::intrusive_ptr<ExpressionContext> copyForSubPipeline(
-        NamespaceString nss, boost::optional<UUID> uuid = boost::none) const {
+        NamespaceString nss, boost::optional<UUID> uuid = boost::none) {
         uassert(ErrorCodes::MaxSubPipelineDepthExceeded,
                 str::stream() << "Maximum number of nested sub-pipelines exceeded. Limit is "
                               << internalMaxSubPipelineViewDepth.load(),
                 subPipelineDepth < internalMaxSubPipelineViewDepth.load());
+        // Initialize any referenced system variables now so that the subpipeline has the same
+        // values for these variables.
+        this->initializeReferencedSystemVariables();
         auto newCopy = copyWith(std::move(nss), uuid);
         newCopy->subPipelineDepth += 1;
         // The original expCtx might have been attached to an aggregation pipeline running on the
@@ -323,7 +366,7 @@ public:
      * namespace not involved in the pipeline.
      */
     const ResolvedNamespace& getResolvedNamespace(const NamespaceString& nss) const {
-        auto it = _resolvedNamespaces.find(nss);
+        auto it = _resolvedNamespaces.find(nss.coll());
         tassert(9453000,
                 str::stream() << "No resolved namespace provided for " << nss.toStringForErrorMsg(),
                 it != _resolvedNamespaces.end());
@@ -339,6 +382,15 @@ public:
         return _resolvedNamespaces.empty();
     }
 
+    /**
+     * Returns true if the tailableMode indicates a tailable
+     * query.
+     */
+    bool isTailable() const {
+        return tailableMode == TailableModeEnum::kTailableAndAwaitData ||
+            tailableMode == TailableModeEnum::kTailable;
+    }
+
     /**
      * Convenience call that returns true if the tailableMode indicates a tailable and awaitData
      * query.
@@ -355,14 +407,14 @@ public:
         return !explain;
     }
 
-    void setResolvedNamespaces(InternalResolvedNamespaceMapType resolvedNamespaces) {
+    void setResolvedNamespaces(StringMap<ResolvedNamespace> resolvedNamespaces) {
         _resolvedNamespaces = std::move(resolvedNamespaces);
     }
 
     void addResolvedNamespaces(
-        mongo::stdx::unordered_set<mongo::NamespaceString> resolvedNamespaces) {
-        for (auto&& nss : resolvedNamespaces) {
-            _resolvedNamespaces.try_emplace(nss, nss, std::vector<BSONObj>{});
+        const mongo::stdx::unordered_set<mongo::NamespaceString>& resolvedNamespaces) {
+        for (const auto& nss : resolvedNamespaces) {
+            _resolvedNamespaces.try_emplace(nss.coll(), nss, std::vector<BSONObj>{});
         }
     }
 
@@ -416,7 +468,7 @@ public:
             invariant(scopeVar.isObject());
             scopeObj = scopeVar.getDocument().toBson();
         }
-        return JsExecution::get(opCtx, scopeObj, ns.db(), loadStoredProcedures, jsHeapLimitMB);
+        return JsExecution::get(opCtx, scopeObj, ns.dbName(), loadStoredProcedures, jsHeapLimitMB);
     }
 
     /**
@@ -455,18 +507,22 @@ public:
     }
 
     /**
-     * Sets the value of the $$USER_ROLES system variable.
+     * Initializes the value of system variables that are referenced by the query. This allows for
+     * lazy initialization of resources which may be expensive to construct (e.g. constructing
+     * cluster timestamp invokes taking a mutex). This function should be invoked after the parsing
+     * of all aggregation expressions in the query.
      */
-    void setUserRoles();
+    void initializeReferencedSystemVariables();
 
     /**
-     * Record that we have seen the given system variable in the query.
+     * Record that we have seen the given system variable in the query. Used for lazy initialization
+     * of variables.
      */
     void setSystemVarReferencedInQuery(Variables::Id var) {
         tassert(7612600,
                 "Cannot track references to user-defined variables.",
                 !Variables::isUserDefinedVariable(var));
-        _varsReferencedInQuery.insert(var);
+        _systemVarsReferencedInQuery.insert(var);
     }
 
     /**
@@ -477,9 +533,17 @@ public:
             7612601,
             "Cannot access whether a variable is referenced to or not for a user-defined variable.",
             !Variables::isUserDefinedVariable(var));
-        return _varsReferencedInQuery.count(var);
+        return _systemVarsReferencedInQuery.count(var);
     }
 
+    /**
+     * Throws if the provided feature flag is not enabled in the current FCV or
+     * 'maxFeatureCompatibilityVersion' if set. Will do nothing if the feature flag is enabled
+     * or boost::none.
+     */
+    void throwIfFeatureFlagIsNotEnabledOnFCV(StringData name,
+                                             const boost::optional<FeatureFlag>& flag);
+
     // The explain verbosity requested by the user, or boost::none if no explain was requested.
     boost::optional<ExplainOptions::Verbosity> explain;
 
@@ -490,9 +554,12 @@ public:
     bool allowDiskUse = false;
     bool bypassDocumentValidation = false;
     bool hasWhereClause = false;
+    bool isUpsert = false;
 
     NamespaceString ns;
 
+    SerializationContext serializationCtxt;
+
     // If known, the UUID of the execution namespace for this aggregation command.
     // TODO(SERVER-78226): Replace `ns` and `uuid` with a type which can express "nss and uuid".
     boost::optional<UUID> uuid;
@@ -548,12 +615,24 @@ public:
     const bool mayDbProfile = true;
 
     // The lowest SBE compatibility level of all expressions which use this expression context.
-    SbeCompatibility sbeCompatibility = SbeCompatibility::fullyCompatible;
+    SbeCompatibility sbeCompatibility = SbeCompatibility::noRequirements;
 
     // The lowest SBE compatibility level of all accumulators in the $group stage currently
     // being parsed using this expression context. This value is transient and gets
     // reset for every $group stage we parse. Each $group stage has its own per-stage flag.
-    SbeCompatibility sbeGroupCompatibility = SbeCompatibility::fullyCompatible;
+    SbeCompatibility sbeGroupCompatibility = SbeCompatibility::noRequirements;
+
+    // The lowest SBE compatibility level of all window functions in the $_internalSetWindowFields
+    // stage currently being parsed using this expression context. This value is transient and gets
+    // reset for every $_internalSetWindowFields stage we parse. Each $_internalSetWindowFields
+    // stage has its own per-stage flag.
+    SbeCompatibility sbeWindowCompatibility = SbeCompatibility::noRequirements;
+
+    // In some situations we could lower the collection access and, maybe, a prefix of a pipeline to
+    // SBE but doing so would prevent a specific optimization that exists in the classic engine from
+    // being applied. Until we implement the same optimization in SBE, we need to fallback to
+    // running the query in the classic engine entirely.
+    SbeCompatibility sbePipelineCompatibility = SbeCompatibility::noRequirements;
 
     // These fields can be used in a context when API version validations were not enforced during
     // parse time (Example creating a view or validator), but needs to be enforce while querying
@@ -562,9 +641,9 @@ public:
     bool exprDeprectedForApiV1 = false;
 
     // Tracks whether the collator to use for the aggregation matches the default collation of the
-    // collection or view. For collectionless aggregates this is set to 'kNoDefaultCollation'.
-    enum class CollationMatchesDefault { kNoDefault, kYes, kNo };
-    CollationMatchesDefault collationMatchesDefault = CollationMatchesDefault::kNoDefault;
+    // collection or view.
+    enum class CollationMatchesDefault { kYes, kNo };
+    CollationMatchesDefault collationMatchesDefault = CollationMatchesDefault::kYes;
 
     // When non-empty, contains the unmodified user provided aggregation command.
     BSONObj originalAggregateCommand;
@@ -580,6 +659,18 @@ public:
     // expression counting.
     bool enabledCounters = true;
 
+    // Forces the plan cache to be used even if there's only one solution available. Queries that
+    // are ineligible will still not be cached.
+    bool forcePlanCache = false;
+
+    // Keep track of the server-side Javascript operators used in the query. These flags will be set
+    // at parse time if applicable.
+    struct {
+        bool accumulator = false;
+        bool function = false;
+        bool where = false;
+    } hasServerSideJs;
+
     // Returns true if we've received a TemporarilyUnavailableException.
     bool getTemporarilyUnavailableException() {
         return _gotTemporarilyUnavailableException;
@@ -599,9 +690,29 @@ public:
         return _requiresTimeseriesExtendedRangeSupport;
     }
 
-    // Forces the plan cache to be used even if there's only one solution available. Queries that
-    // are ineligible will still not be cached.
-    bool forcePlanCache = false;
+    const query_settings::QuerySettings& getQuerySettings() const {
+        static const auto kEmptySettings = query_settings::QuerySettings();
+        return _querySettings.get_value_or(kEmptySettings);
+    }
+
+    /**
+     * Attaches 'querySettings' to context if they were not previously set.
+     */
+    void setQuerySettingsIfNotPresent(query_settings::QuerySettings querySettings) {
+        if (_querySettings.has_value()) {
+            return;
+        }
+
+        tassert(8827100,
+                "Query knobs shouldn't be initialized before query settings are set",
+                !_queryKnobConfiguration.isInitialized());
+
+        _querySettings = std::move(querySettings);
+    }
+
+    const QueryKnobConfiguration& getQueryKnobConfiguration() const {
+        return _queryKnobConfiguration.get(getQuerySettings());
+    }
 
     // This is state that is to be shared between the DocumentInternalSearchMongotRemote and
     // DocumentInternalSearchIdLookup stages (these stages are the result of desugaring $search)
@@ -636,6 +747,20 @@ public:
         long long _docsReturnedByIdLookup = 0;
     } sharedSearchState;
 
+    void setIgnoreCollator() {
+        _collator.setIgnore();
+    }
+
+    bool getIgnoreCollator() {
+        return _collator.getIgnore();
+    }
+
+    /**
+     * Returns true if featureFlagBinDataConvert is enabled. See the comment below on
+     * '_featureFlagBinDataConvertValue' for more information about this pattern.
+     */
+    bool isFeatureFlagBinDataConvertEnabled();
+
 protected:
     static const int kInterruptCheckPeriod = 128;
 
@@ -645,8 +770,50 @@ protected:
     // when _interruptCounter has been decremented to zero.
     void checkForInterruptSlow();
 
-    // Collator used for comparisons.
-    std::unique_ptr<CollatorInterface> _collator;
+    // Class responsible for tracking the collator used for comparisons. Specifically, this
+    // collator enforces the following contract:
+    // - When used in a replica set or a standalone, '_collator' will have the correct collation.
+    // - When routing to a tracked collection, '_collator' will have the correct collation according
+    // to the ChunkManager and can use it.
+    // - When routing to an untracked collection, '_collator' will be set incorrectly as there is
+    // no way to know the collation of an untracked collection on the router. However,
+    // 'ignore' will be set to 'true', which indicates that we will not attach '_collator' when
+    // routing commands to the target collection.
+    // TODO SERVER-81991: Delete this class once we branch for 8.0.
+    class RoutingCollator {
+    public:
+        RoutingCollator(std::shared_ptr<CollatorInterface> collator) : _ptr(std::move(collator)) {}
+        void setIgnore() {
+            _ignore = true;
+        }
+
+        bool getIgnore() const {
+            return _ignore;
+        }
+
+        void setCollator(std::shared_ptr<CollatorInterface> collator) {
+            _ptr = collator;
+            // If we are manually setting the collator, we shouldn't ignore it.
+            _ignore = false;
+        }
+        std::shared_ptr<CollatorInterface> getCollatorShared() const {
+            if (_ignore) {
+                return nullptr;
+            }
+            return _ptr;
+        }
+
+        CollatorInterface* getCollator() const {
+            if (_ignore) {
+                return nullptr;
+            }
+            return _ptr.get();
+        }
+
+    private:
+        std::shared_ptr<CollatorInterface> _ptr = nullptr;
+        bool _ignore = false;
+    } _collator;
 
     // Used for all comparisons of Document/Value during execution of the aggregation operation.
     // Must not be changed after parsing a Pipeline with this ExpressionContext.
@@ -654,7 +821,7 @@ protected:
     ValueComparator _valueComparator;
 
     // A map from namespace to the resolved namespace, in case any views are involved.
-    InternalResolvedNamespaceMapType _resolvedNamespaces;
+    StringMap<ResolvedNamespace> _resolvedNamespaces;
 
     int _interruptCounter = kInterruptCheckPeriod;
 
@@ -676,10 +843,39 @@ private:
 
     // We use this set to indicate whether or not a system variable was referenced in the query that
     // is being executed (if the variable was referenced, it is an element of this set).
-    stdx::unordered_set<Variables::Id> _varsReferencedInQuery;
+    stdx::unordered_set<Variables::Id> _systemVarsReferencedInQuery;
+
+    std::once_flag _querySettingsAttached;
+
+    boost::optional<query_settings::QuerySettings> _querySettings = boost::none;
+
+    DeferredFn<QueryKnobConfiguration, const query_settings::QuerySettings&>
+        _queryKnobConfiguration{[](const auto& querySettings) {
+            return QueryKnobConfiguration(querySettings);
+        }};
+
+    // We store the value of featureFlagBinDataConvert here. We can end up checking the value of the
+    // feature flag twice in certain scenarios; for example, if we have a $toString expression in a
+    // $project, we call makeConversionAlias() which checks the value of the feature flag to know
+    // whether we should add the 'format' argument to the ExpressionConvert. At some other point
+    // later we call the copy constructor ExpressionNodeAST, and since the copy constructor of
+    // ExpressionASTNode calls parseOperand(), we get into ExpressionConvert::parse(). This also
+    // checks the value of the feature flag to determine if we are allowed to parse the format
+    // argument. Since we want the value of the feature flag to be the same throughout an operation,
+    // we cache the value on the ExpressionContext. This is because if an FCV updgrade/downgrade
+    // happens in between the two checks of the feature flag's value, we can get different results
+    // and thus error due to partial execution of the feature. Caching the value here ensures that
+    // we will always see the same snapshot of the feature flag's value throughout the duration of
+    // one operation. Note that this is currently the only query feature flag that has this caching
+    // behavior. If it becomes a more common pattern that we need to check the value of a feature
+    // flag twice in an operation, we should strongly consider creating a class in which to cache
+    // the values of all the query feature flags in a more generalized manner, similar to
+    // QueryKnobConfiguration. Alternatively, we can store the FCV snapshot here and any query
+    // feature flag check would refer to the same FCV once it has been obtained.
+    Deferred<bool (*)()> _featureFlagBinDataConvertValue{[] {
+        return feature_flags::gFeatureFlagBinDataConvert.isEnabledUseLatestFCVWhenUninitialized(
+            serverGlobalParams.featureCompatibility.acquireFCVSnapshot());
+    }};
 };
 
-using ResolvedNamespaceMap =
-    absl::flat_hash_map<NamespaceString, ExpressionContext::ResolvedNamespace>;
-
 }  // namespace mongo
