@@ -26,17 +26,28 @@
  *    exception statement from all source files in the program, then also delete
  *    it in the license file.
  */
-#include "mongo/platform/basic.h"
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsontypes.h"
 #include "mongo/db/api_parameters.h"
 #include "mongo/db/pipeline/expression_test_api_version.h"
+#include "mongo/db/query/allowed_contexts.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/intrusive_counter.h"
+#include "mongo/util/str.h"
 
 namespace mongo {
 
 REGISTER_TEST_EXPRESSION(_testApiVersion,
+                         ExpressionTestApiVersion::parse,
                          AllowedWithApiStrict::kConditionally,
-                         AllowedWithClientType::kAny,
-                         ExpressionTestApiVersion::parse);
+                         AllowedWithClientType::kAny);
 
 ExpressionTestApiVersion::ExpressionTestApiVersion(ExpressionContext* const expCtx,
                                                    bool unstable,
@@ -99,6 +110,4 @@ Value ExpressionTestApiVersion::evaluate(const Document& root, Variables* variab
     return Value(1);
 }
 
-void ExpressionTestApiVersion::_doAddDependencies(DepsTracker* deps) const {}
-
 }  // namespace mongo
