@@ -29,8 +29,21 @@
 
 #pragma once
 
+#include <cmath>
+#include <limits>
+#include <string>
+#include <utility>
+
 #include "expression.h"
 
+#include "mongo/bson/bsontypes.h"
+#include "mongo/db/exec/document_value/value.h"
+#include "mongo/db/pipeline/expression_context.h"
+#include "mongo/db/pipeline/expression_visitor.h"
+#include "mongo/platform/decimal128.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/str.h"
+
 namespace mongo {
 
 /**
@@ -147,7 +160,7 @@ public:
     /**
      * evaluateNumericArg  evaluates the implented trig function on one numericArg.
      */
-    Value evaluateNumericArg(const Value& numericArg) const {
+    Value evaluateNumericArg(const Value& numericArg) const override {
         switch (numericArg.getType()) {
             case BSONType::NumberDouble: {
                 auto input = numericArg.getDouble();
@@ -192,7 +205,7 @@ public:
     /**
      * getOpName returns the name of the operation, e.g., $sin
      */
-    virtual const char* getOpName() const = 0;
+    const char* getOpName() const override = 0;
 };
 
 /**
@@ -235,7 +248,7 @@ public:
     /**
      * getOpName returns the name of the operation, e.g., $sinh
      */
-    virtual const char* getOpName() const = 0;
+    const char* getOpName() const override = 0;
 };
 
 class ExpressionArcTangent2 final : public ExpressionTwoNumericArgs<ExpressionArcTangent2> {
