@@ -29,10 +29,15 @@
 
 #include "mongo/db/pipeline/group_from_first_document_transformation.h"
 
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/db/exec/document_value/value.h"
 #include "mongo/db/pipeline/expression_dependencies.h"
 
 namespace mongo {
-Document GroupFromFirstDocumentTransformation::applyTransformation(const Document& input) {
+Document GroupFromFirstDocumentTransformation::applyTransformation(const Document& input) const {
     MutableDocument output(_accumulatorExprs.size());
 
     for (auto&& expr : _accumulatorExprs) {
@@ -88,9 +93,9 @@ std::unique_ptr<GroupFromFirstDocumentTransformation> GroupFromFirstDocumentTran
     const std::string& groupId,
     StringData originalStageName,
     std::vector<std::pair<std::string, boost::intrusive_ptr<Expression>>> accumulatorExprs,
-    ExpectedInput expectedInput) {
+    AccumulatorDocumentsNeeded docsNeeded) {
     return std::make_unique<GroupFromFirstDocumentTransformation>(
-        groupId, originalStageName, std::move(accumulatorExprs), expectedInput);
+        groupId, originalStageName, std::move(accumulatorExprs), docsNeeded);
 }
 
 }  // namespace mongo
