@@ -29,8 +29,27 @@
 
 #pragma once
 
+#include <memory>
+#include <set>
+#include <string>
+#include <utility>
+#include <vector>
+
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+
+#include "mongo/base/string_data.h"
+#include "mongo/db/exec/document_value/document.h"
+#include "mongo/db/pipeline/accumulator.h"
+#include "mongo/db/pipeline/dependencies.h"
+#include "mongo/db/pipeline/document_source.h"
 #include "mongo/db/pipeline/expression.h"
+#include "mongo/db/pipeline/expression_context.h"
 #include "mongo/db/pipeline/transformer_interface.h"
+#include "mongo/db/pipeline/variables.h"
+#include "mongo/db/query/explain_options.h"
+#include "mongo/db/query/query_shape/serialization_options.h"
+#include "mongo/util/intrusive_counter.h"
 
 namespace mongo {
 
@@ -42,17 +61,15 @@ namespace mongo {
  */
 class GroupFromFirstDocumentTransformation final : public TransformerInterface {
 public:
-    enum class ExpectedInput { kFirstDocument, kLastDocument };
-
     GroupFromFirstDocumentTransformation(
         const std::string& groupId,
         StringData originalStageName,
         std::vector<std::pair<std::string, boost::intrusive_ptr<Expression>>> accumulatorExprs,
-        ExpectedInput expectedInput = ExpectedInput::kFirstDocument)
+        AccumulatorDocumentsNeeded docsNeeded = AccumulatorDocumentsNeeded::kFirstInputDocument)
         : _accumulatorExprs(std::move(accumulatorExprs)),
           _groupId(groupId),
           _originalStageName(originalStageName),
-          _expectedInput(expectedInput) {}
+          _docsNeeded(docsNeeded) {}
 
     TransformerType getType() const final {
         return TransformerType::kGroupFromFirstDocument;
@@ -70,11 +87,11 @@ public:
         return _originalStageName;
     }
 
-    ExpectedInput expectedInput() const {
-        return _expectedInput;
+    AccumulatorDocumentsNeeded docsNeeded() const {
+        return _docsNeeded;
     }
 
-    Document applyTransformation(const Document& input) final;
+    Document applyTransformation(const Document& input) const final;
 
     void optimize() final;
 
@@ -92,13 +109,13 @@ public:
         const std::string& groupId,
         StringData originalStageName,
         std::vector<std::pair<std::string, boost::intrusive_ptr<Expression>>> accumulatorExprs,
-        ExpectedInput expectedInput);
+        AccumulatorDocumentsNeeded docsNeeded);
 
 private:
     std::vector<std::pair<std::string, boost::intrusive_ptr<Expression>>> _accumulatorExprs;
     std::string _groupId;
     StringData _originalStageName;
-    ExpectedInput _expectedInput;
+    AccumulatorDocumentsNeeded _docsNeeded;
 };
 
 }  // namespace mongo
