@@ -27,11 +27,17 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <absl/container/node_hash_map.h>
+#include <absl/container/node_hash_set.h>
+#include <absl/meta/type_traits.h>
+#include <algorithm>
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
 
 #include "mongo/db/pipeline/lite_parsed_document_source.h"
 #include "mongo/db/pipeline/lite_parsed_pipeline.h"
 #include "mongo/db/stats/counters.h"
+#include "mongo/util/string_map.h"
 
 namespace mongo {
 
@@ -52,7 +58,7 @@ void LiteParsedDocumentSource::registerParser(const std::string& name,
                                               AllowedWithClientType allowedWithClientType) {
     parserMap[name] = {parser, allowedWithApiStrict, allowedWithClientType};
     // Initialize a counter for this document source to track how many times it is used.
-    aggStageCounters.stageCounterMap[name] = std::make_unique<AggStageCounters::StageCounter>(name);
+    aggStageCounters.addMetric(name);
 }
 
 std::unique_ptr<LiteParsedDocumentSource> LiteParsedDocumentSource::parse(
@@ -101,7 +107,7 @@ LiteParsedDocumentSourceNestedPipelines::LiteParsedDocumentSourceNestedPipelines
     : LiteParsedDocumentSourceNestedPipelines(
           std::move(parseTimeName), std::move(foreignNss), std::vector<LiteParsedPipeline>{}) {
     if (pipeline)
-        _pipelines.emplace_back(std::move(pipeline.get()));
+        _pipelines.emplace_back(std::move(pipeline.value()));
 }
 
 stdx::unordered_set<NamespaceString>
@@ -111,7 +117,7 @@ LiteParsedDocumentSourceNestedPipelines::getInvolvedNamespaces() const {
         involvedNamespaces.insert(*_foreignNss);
 
     for (auto&& pipeline : _pipelines) {
-        auto involvedInSubPipe = pipeline.getInvolvedNamespaces();
+        const auto& involvedInSubPipe = pipeline.getInvolvedNamespaces();
         involvedNamespaces.insert(involvedInSubPipe.begin(), involvedInSubPipe.end());
     }
     return involvedNamespaces;
@@ -122,26 +128,29 @@ void LiteParsedDocumentSourceNestedPipelines::getForeignExecutionNamespaces(
     for (auto&& pipeline : _pipelines) {
         auto nssVector = pipeline.getForeignExecutionNamespaces();
         for (const auto& nssOrUUID : nssVector) {
-            auto nss = nssOrUUID.nss();
-            tassert(6458500, "nss expected to contain a NamespaceString", nss != boost::none);
-            nssSet.insert(*nss);
+            tassert(6458500,
+                    "nss expected to contain a NamespaceString",
+                    nssOrUUID.isNamespaceString());
+            nssSet.insert(nssOrUUID.nss());
         }
     }
 }
 
-bool LiteParsedDocumentSourceNestedPipelines::allowedToPassthroughFromMongos() const {
-    // If any of the sub-pipelines doesn't allow pass through, then return false.
-    return std::all_of(_pipelines.cbegin(), _pipelines.cend(), [](const auto& subPipeline) {
-        return subPipeline.allowedToPassthroughFromMongos();
+bool LiteParsedDocumentSourceNestedPipelines::isExemptFromIngressAdmissionControl() const {
+    return std::any_of(_pipelines.begin(), _pipelines.end(), [](auto&& pipeline) {
+        return pipeline.isExemptFromIngressAdmissionControl();
     });
 }
 
-bool LiteParsedDocumentSourceNestedPipelines::allowShardedForeignCollection(
-    NamespaceString nss, bool inMultiDocumentTransaction) const {
-    return std::all_of(
-        _pipelines.begin(), _pipelines.end(), [&nss, inMultiDocumentTransaction](auto&& pipeline) {
-            return pipeline.allowShardedForeignCollection(nss, inMultiDocumentTransaction);
-        });
+Status LiteParsedDocumentSourceNestedPipelines::checkShardedForeignCollAllowed(
+    const NamespaceString& nss, bool inMultiDocumentTransaction) const {
+    for (auto&& pipeline : _pipelines) {
+        if (auto status = pipeline.checkShardedForeignCollAllowed(nss, inMultiDocumentTransaction);
+            !status.isOK()) {
+            return status;
+        }
+    }
+    return Status::OK();
 }
 
 ReadConcernSupportResult LiteParsedDocumentSourceNestedPipelines::supportsReadConcern(
