@@ -29,28 +29,50 @@
 
 
 #include "mongo/db/pipeline/pipeline.h"
-#include "mongo/logv2/log.h"
 
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+// IWYU pragma: no_include "ext/alloc_traits.h"
 #include <algorithm>
+#include <bitset>
+#include <exception>
+#include <iterator>
+#include <ostream>
+#include <string>
+#include <utility>
 
 #include "mongo/base/error_codes.h"
 #include "mongo/base/exact_cast.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/bson/util/builder_fwd.h"
 #include "mongo/db/bson/dotted_path_support.h"
 #include "mongo/db/exec/document_value/document.h"
-#include "mongo/db/jsobj.h"
+#include "mongo/db/exec/plan_stats.h"
+#include "mongo/db/matcher/expression_algo.h"
 #include "mongo/db/operation_context.h"
-#include "mongo/db/pipeline/accumulator.h"
+#include "mongo/db/pipeline/aggregate_command_gen.h"
 #include "mongo/db/pipeline/change_stream_helpers.h"
 #include "mongo/db/pipeline/document_source.h"
+#include "mongo/db/pipeline/document_source_change_stream_gen.h"
 #include "mongo/db/pipeline/document_source_match.h"
 #include "mongo/db/pipeline/document_source_merge.h"
 #include "mongo/db/pipeline/document_source_out.h"
 #include "mongo/db/pipeline/document_source_single_document_transformation.h"
 #include "mongo/db/pipeline/expression_context.h"
 #include "mongo/db/pipeline/lite_parsed_pipeline.h"
-#include "mongo/db/pipeline/search_helper.h"
+#include "mongo/db/pipeline/resume_token.h"
+#include "mongo/db/pipeline/search/search_helper.h"
+#include "mongo/db/pipeline/stage_constraints.h"
+#include "mongo/db/pipeline/transformer_interface.h"
+#include "mongo/db/query/explain_options.h"
 #include "mongo/db/query/query_knobs_gen.h"
-#include "mongo/db/storage/storage_options.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/duration.h"
 #include "mongo/util/fail_point.h"
 #include "mongo/util/str.h"
 
@@ -69,9 +91,17 @@ Value appendCommonExecStats(Value docSource, const CommonStats& stats) {
     auto nReturned = static_cast<long long>(stats.advanced);
     doc.addField("nReturned", Value(nReturned));
 
-    invariant(stats.executionTime);
-    auto executionTimeMillisEstimate = durationCount<Milliseconds>(*stats.executionTime);
-    doc.addField("executionTimeMillisEstimate", Value(executionTimeMillisEstimate));
+    if (stats.executionTime.precision == QueryExecTimerPrecision::kMillis) {
+        doc.addField("executionTimeMillisEstimate",
+                     Value(durationCount<Milliseconds>(stats.executionTime.executionTimeEstimate)));
+    } else if (stats.executionTime.precision == QueryExecTimerPrecision::kNanos) {
+        doc.addField("executionTimeMillisEstimate",
+                     Value(durationCount<Milliseconds>(stats.executionTime.executionTimeEstimate)));
+        doc.addField("executionTimeMicros",
+                     Value(durationCount<Microseconds>(stats.executionTime.executionTimeEstimate)));
+        doc.addField("executionTimeNanos",
+                     Value(durationCount<Nanoseconds>(stats.executionTime.executionTimeEstimate)));
+    }
     return Value(doc.freeze());
 }
 
@@ -81,9 +111,10 @@ Value appendCommonExecStats(Value docSource, const CommonStats& stats) {
  */
 void validateTopLevelPipeline(const Pipeline& pipeline) {
     // Verify that the specified namespace is valid for the initial stage of this pipeline.
-    const NamespaceString& nss = pipeline.getContext()->ns;
+    auto expCtx = pipeline.getContext();
+    const NamespaceString& nss = expCtx->ns;
 
-    auto sources = pipeline.getSources();
+    const auto& sources = pipeline.getSources();
 
     if (sources.empty()) {
         uassert(ErrorCodes::InvalidNamespace,
@@ -129,7 +160,6 @@ void validateTopLevelPipeline(const Pipeline& pipeline) {
                 hasChangeStreamSplitLargeEventStage = true;
             }
         }
-        auto expCtx = pipeline.getContext();
         auto spec = isChangeStream ? expCtx->changeStreamSpec : boost::none;
         auto hasSplitEventResumeToken = spec &&
             change_stream::resolveResumeTokenFromSpec(expCtx, *spec).fragmentNum.has_value();
@@ -139,10 +169,10 @@ void validateTopLevelPipeline(const Pipeline& pipeline) {
                 !(hasSplitEventResumeToken && !hasChangeStreamSplitLargeEventStage));
     }
 
-    // Verify that usage of $searchMeta and $search is legal.
-    if (pipeline.getContext()->opCtx->getServiceContext()) {
-        getSearchHelpers(pipeline.getContext()->opCtx->getServiceContext())
-            ->assertSearchMetaAccessValid(sources, pipeline.getContext().get());
+    // Verify that usage of $searchMeta and $search is legal. Note that on mongos, we defer this
+    // check until after we've established cursors on the shards to resolve any views.
+    if (expCtx->opCtx->getServiceContext() && !expCtx->inMongos) {
+        search_helpers::assertSearchMetaAccessValid(sources, expCtx.get());
     }
 }
 
@@ -151,7 +181,6 @@ void validateTopLevelPipeline(const Pipeline& pipeline) {
 MONGO_FAIL_POINT_DEFINE(disablePipelineOptimization);
 
 using boost::intrusive_ptr;
-using std::endl;
 using std::ostringstream;
 using std::string;
 using std::vector;
@@ -184,7 +213,7 @@ std::unique_ptr<Pipeline, PipelineDeleter> Pipeline::clone(
     for (auto&& stage : _sources) {
         clonedStages.push_back(stage->clone(expCtx));
     }
-    return create(clonedStages, expCtx);
+    return create(std::move(clonedStages), expCtx);
 }
 
 template <class T>
@@ -192,10 +221,17 @@ std::unique_ptr<Pipeline, PipelineDeleter> Pipeline::parseCommon(
     const std::vector<T>& rawPipeline,
     const boost::intrusive_ptr<ExpressionContext>& expCtx,
     PipelineValidatorCallback validator,
+    bool isFacetPipeline,
     std::function<BSONObj(T)> getElemFunc) {
 
-    SourceContainer stages;
+    // Before parsing the pipeline, make sure it's not so long that it will make us run out of
+    // memory.
+    uassert(7749501,
+            str::stream() << "Pipeline length must be no longer than "
+                          << internalPipelineLengthLimit << " stages.",
+            static_cast<int>(rawPipeline.size()) <= internalPipelineLengthLimit);
 
+    SourceContainer stages;
     for (auto&& stageElem : rawPipeline) {
         auto parsedSources = DocumentSource::parse(expCtx, getElemFunc(stageElem));
         stages.insert(stages.end(), parsedSources.begin(), parsedSources.end());
@@ -204,13 +240,15 @@ std::unique_ptr<Pipeline, PipelineDeleter> Pipeline::parseCommon(
     std::unique_ptr<Pipeline, PipelineDeleter> pipeline(new Pipeline(std::move(stages), expCtx),
                                                         PipelineDeleter(expCtx->opCtx));
 
-    // First call the context-specific validator, which may be different for top-level pipelines
-    // versus nested pipelines.
-    if (validator)
-        validator(*pipeline);
-    else {
+    // First call the top level validator, unless this is a $facet
+    // (nested) pipeline. Then call the context-specific validator if one
+    // is provided.
+    if (!isFacetPipeline) {
         validateTopLevelPipeline(*pipeline);
     }
+    if (validator) {
+        validator(*pipeline);
+    }
 
     // Next run through the common validation rules that apply to every pipeline.
     constexpr bool alreadyOptimized = false;
@@ -230,7 +268,7 @@ std::unique_ptr<Pipeline, PipelineDeleter> Pipeline::parseFromArray(
             rawPipelineElement.type() == BSONType::Array);
     auto rawStages = rawPipelineElement.Array();
 
-    return parseCommon<BSONElement>(rawStages, expCtx, validator, [](BSONElement e) {
+    return parseCommon<BSONElement>(rawStages, expCtx, validator, false, [](BSONElement e) {
         uassert(6253720, "Pipeline array element must be an object", e.type() == BSONType::Object);
         return e.embeddedObject();
     });
@@ -240,8 +278,14 @@ std::unique_ptr<Pipeline, PipelineDeleter> Pipeline::parse(
     const std::vector<BSONObj>& rawPipeline,
     const intrusive_ptr<ExpressionContext>& expCtx,
     PipelineValidatorCallback validator) {
+    return parseCommon<BSONObj>(rawPipeline, expCtx, validator, false, [](BSONObj o) { return o; });
+}
 
-    return parseCommon<BSONObj>(rawPipeline, expCtx, validator, [](BSONObj o) { return o; });
+std::unique_ptr<Pipeline, PipelineDeleter> Pipeline::parseFacetPipeline(
+    const std::vector<BSONObj>& rawPipeline,
+    const intrusive_ptr<ExpressionContext>& expCtx,
+    PipelineValidatorCallback validator) {
+    return parseCommon<BSONObj>(rawPipeline, expCtx, validator, true, [](BSONObj o) { return o; });
 }
 
 std::unique_ptr<Pipeline, PipelineDeleter> Pipeline::create(
@@ -256,7 +300,7 @@ std::unique_ptr<Pipeline, PipelineDeleter> Pipeline::create(
 }
 
 void Pipeline::validateCommon(bool alreadyOptimized) const {
-    uassert(ErrorCodes::FailedToParse,
+    uassert(5054701,
             str::stream() << "Pipeline length must be no longer than "
                           << internalPipelineLengthLimit << " stages",
             static_cast<int>(_sources.size()) <= internalPipelineLengthLimit);
@@ -313,7 +357,7 @@ void Pipeline::validateCommon(bool alreadyOptimized) const {
 
         tassert(7355707,
                 "If a stage is broadcast to all shard servers then it must be a data source.",
-                constraints.hostRequirement != HostTypeRequirement::kAllShardServers ||
+                constraints.hostRequirement != HostTypeRequirement::kAllShardHosts ||
                     !constraints.requiresInputDocSource);
     }
 }
@@ -452,11 +496,41 @@ BSONObj Pipeline::getInitialQuery() const {
     return BSONObj{};
 }
 
-bool Pipeline::needsPrimaryShardMerger() const {
-    return std::any_of(_sources.begin(), _sources.end(), [&](const auto& stage) {
-        return stage->constraints(SplitState::kSplitForMerge).hostRequirement ==
-            HostTypeRequirement::kPrimaryShard;
-    });
+void Pipeline::parameterize() {
+    if (!_sources.empty()) {
+        if (auto matchStage = dynamic_cast<DocumentSourceMatch*>(_sources.front().get())) {
+            MatchExpression::parameterize(matchStage->getMatchExpression());
+            _isParameterized = true;
+        }
+    }
+}
+
+void Pipeline::unparameterize() {
+    if (!_sources.empty()) {
+        if (auto matchStage = dynamic_cast<DocumentSourceMatch*>(_sources.front().get())) {
+            // Sets max param count in MatchExpression::parameterize() to 0, clearing
+            // MatchExpression auto-parameterization before pipeline to ABT translation.
+            MatchExpression::unparameterize(matchStage->getMatchExpression());
+            _isParameterized = false;
+        }
+    }
+}
+
+bool Pipeline::canParameterize() {
+    if (!_sources.empty()) {
+        // First stage must be a DocumentSourceMatch.
+        return _sources.begin()->get()->getSourceName() == DocumentSourceMatch::kStageName;
+    }
+    return false;
+}
+
+boost::optional<ShardId> Pipeline::needsSpecificShardMerger() const {
+    for (const auto& stage : _sources) {
+        if (auto mergeShardId = stage->constraints(SplitState::kSplitForMerge).mergeShardId) {
+            return mergeShardId;
+        }
+    }
+    return boost::none;
 }
 
 bool Pipeline::needsMongosMerger() const {
@@ -466,10 +540,10 @@ bool Pipeline::needsMongosMerger() const {
     });
 }
 
-bool Pipeline::needsAllShardServers() const {
+bool Pipeline::needsAllShardHosts() const {
     return std::any_of(_sources.begin(), _sources.end(), [&](const auto& stage) {
         return stage->constraints().resolvedHostTypeRequirement(pCtx) ==
-            HostTypeRequirement::kAllShardServers;
+            HostTypeRequirement::kAllShardHosts;
     });
 }
 
@@ -477,15 +551,10 @@ bool Pipeline::needsShard() const {
     return std::any_of(_sources.begin(), _sources.end(), [&](const auto& stage) {
         auto hostType = stage->constraints().resolvedHostTypeRequirement(pCtx);
         return (hostType == HostTypeRequirement::kAnyShard ||
-                hostType == HostTypeRequirement::kPrimaryShard ||
-                hostType == HostTypeRequirement::kAllShardServers);
+                hostType == HostTypeRequirement::kAllShardHosts);
     });
 }
 
-bool Pipeline::canRunOnMongos() const {
-    return _pipelineCanRunOnMongoS().isOK();
-}
-
 bool Pipeline::requiredToRunOnMongos() const {
     invariant(_splitState != SplitState::kSplitForShards);
 
@@ -501,13 +570,10 @@ bool Pipeline::requiredToRunOnMongos() const {
         // If a mongoS-only stage occurs before a splittable stage, or if the pipeline is already
         // split, this entire pipeline must run on mongoS.
         if (hostRequirement == HostTypeRequirement::kMongoS) {
-            // Verify that the remainder of this pipeline can run on mongoS.
-            auto mongosRunStatus = _pipelineCanRunOnMongoS();
-
-            uassertStatusOKWithContext(mongosRunStatus,
-                                       str::stream() << stage->getSourceName()
-                                                     << " must run on mongoS, but cannot");
-
+            LOGV2_DEBUG(8346100,
+                        1,
+                        "stage {stage} is required to run on mongoS",
+                        "stage"_attr = stage->getSourceName());
             return true;
         }
     }
@@ -523,34 +589,9 @@ stdx::unordered_set<NamespaceString> Pipeline::getInvolvedCollections() const {
     return collectionNames;
 }
 
-std::vector<BSONObj> Pipeline::serializePipelineForLogging(const std::vector<BSONObj>& pipeline) {
-    std::vector<BSONObj> redacted;
-    for (auto&& b : pipeline) {
-        redacted.push_back(redact(b));
-    }
-    return redacted;
-}
-
-std::vector<BSONObj> Pipeline::serializeForLogging(
-    boost::optional<const SerializationOptions&> opts) const {
-    std::vector<BSONObj> serialized = serializeToBson(opts);
-    return serializePipelineForLogging(serialized);
-}
-
-std::vector<BSONObj> Pipeline::serializeContainerForLogging(
-    const SourceContainer& container, boost::optional<const SerializationOptions&> opts) {
-    std::vector<Value> serialized = serializeContainer(container, opts);
-    std::vector<BSONObj> redacted;
-    for (auto&& stage : serialized) {
-        invariant(stage.getType() == BSONType::Object);
-        redacted.push_back(redact(stage.getDocument().toBson()));
-    }
-    return redacted;
-}
-
-std::vector<Value> Pipeline::serializeContainer(const SourceContainer& container,
-                                                boost::optional<const SerializationOptions&> opts) {
-    std::vector<Value> serializedSources;
+vector<Value> Pipeline::serializeContainer(const SourceContainer& container,
+                                           boost::optional<const SerializationOptions&> opts) {
+    vector<Value> serializedSources;
     for (auto&& source : container) {
         source->serializeToArray(serializedSources, opts ? opts.get() : SerializationOptions());
     }
@@ -706,6 +747,7 @@ DepsTracker Pipeline::getDependenciesForContainer(
             deps.requestMetadata(localDeps.metadataDeps());
             knowAllMeta = status & DepsTracker::State::EXHAUSTIVE_META;
         }
+        deps.requestMetadata(localDeps.searchMetadataDeps());
     }
 
     if (!knowAllFields)
@@ -726,14 +768,13 @@ DepsTracker Pipeline::getDependenciesForContainer(
     return deps;
 }
 
-Status Pipeline::_pipelineCanRunOnMongoS() const {
+Status Pipeline::canRunOnMongos() const {
     for (auto&& stage : _sources) {
         auto constraints = stage->constraints(_splitState);
         auto hostRequirement = constraints.resolvedHostTypeRequirement(pCtx);
 
         const bool needsShard = (hostRequirement == HostTypeRequirement::kAnyShard ||
-                                 hostRequirement == HostTypeRequirement::kPrimaryShard ||
-                                 hostRequirement == HostTypeRequirement::kAllShardServers);
+                                 hostRequirement == HostTypeRequirement::kAllShardHosts);
 
         const bool mustWriteToDisk =
             (constraints.diskRequirement == DiskUseRequirement::kWritesPersistentData);
@@ -835,9 +876,11 @@ void Pipeline::appendPipeline(std::unique_ptr<Pipeline, PipelineDeleter> otherPi
 std::unique_ptr<Pipeline, PipelineDeleter> Pipeline::makePipeline(
     const std::vector<BSONObj>& rawPipeline,
     const boost::intrusive_ptr<ExpressionContext>& expCtx,
-    const MakePipelineOptions opts) {
+    MakePipelineOptions opts) {
     auto pipeline = Pipeline::parse(rawPipeline, expCtx, opts.validator);
 
+    expCtx->initializeReferencedSystemVariables();
+
     bool alreadyOptimized = opts.alreadyOptimized;
 
     if (opts.optimize) {
@@ -849,9 +892,9 @@ std::unique_ptr<Pipeline, PipelineDeleter> Pipeline::makePipeline(
 
     if (opts.attachCursorSource) {
         // Creating AggregateCommandRequest in order to pass all necessary 'opts' to the
-        // attachCursorSource().
+        // preparePipelineForExecution().
         AggregateCommandRequest aggRequest(expCtx->ns, pipeline->serializeToBson());
-        pipeline = expCtx->mongoProcessInterface->attachCursorSourceToPipeline(
+        pipeline = expCtx->mongoProcessInterface->preparePipelineForExecution(
             expCtx,
             aggRequest,
             pipeline.release(),
@@ -861,10 +904,6 @@ std::unique_ptr<Pipeline, PipelineDeleter> Pipeline::makePipeline(
             opts.useCollectionDefaultCollator);
     }
 
-    // After parsing the pipeline to detect if $$USER_ROLES is referenced, set the value of
-    // $$USER_ROLES for the pipeline.
-    expCtx->setUserRoles();
-
     return pipeline;
 }
 
@@ -895,7 +934,7 @@ std::unique_ptr<Pipeline, PipelineDeleter> Pipeline::makePipeline(
     pipeline->validateCommon(alreadyOptimized);
     aggRequest.setPipeline(pipeline->serializeToBson());
 
-    return expCtx->mongoProcessInterface->attachCursorSourceToPipeline(
+    return expCtx->mongoProcessInterface->preparePipelineForExecution(
         expCtx,
         aggRequest,
         pipeline.release(),
@@ -928,7 +967,7 @@ std::unique_ptr<Pipeline, PipelineDeleter> Pipeline::makePipelineFromViewDefinit
     subPipelineExpCtx->ns = resolvedNs.ns;
 
     if (resolvedNs.pipeline.empty()) {
-        return Pipeline::makePipeline(std::move(currentPipeline), subPipelineExpCtx, opts);
+        return Pipeline::makePipeline(currentPipeline, subPipelineExpCtx, opts);
     }
     auto resolvedPipeline = std::move(resolvedNs.pipeline);
 
@@ -942,7 +981,7 @@ std::unique_ptr<Pipeline, PipelineDeleter> Pipeline::makePipelineFromViewDefinit
                             std::make_move_iterator(currentPipeline.begin()),
                             std::make_move_iterator(currentPipeline.end()));
 
-    return Pipeline::makePipeline(std::move(resolvedPipeline), subPipelineExpCtx, opts);
+    return Pipeline::makePipeline(resolvedPipeline, subPipelineExpCtx, opts);
 }
 
 }  // namespace mongo
