@@ -29,24 +29,47 @@
 
 #pragma once
 
+#include <boost/intrusive_ptr.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
 #include <functional>
 #include <list>
+#include <memory>
+#include <set>
 #include <vector>
 
-#include <boost/intrusive_ptr.hpp>
-
+#include "mongo/base/status.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/db/exec/document_value/document.h"
+#include "mongo/db/exec/document_value/document_metadata_fields.h"
 #include "mongo/db/exec/document_value/value.h"
 #include "mongo/db/exec/plan_stats.h"
 #include "mongo/db/matcher/expression_parser.h"
 #include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/pipeline/dependencies.h"
+#include "mongo/db/pipeline/expression.h"
+#include "mongo/db/pipeline/expression_context.h"
+#include "mongo/db/pipeline/process_interface/mongo_process_interface.h"
 #include "mongo/db/pipeline/sharded_agg_helpers_targeting_policy.h"
+#include "mongo/db/pipeline/variables.h"
 #include "mongo/db/query/cursor_response_gen.h"
 #include "mongo/db/query/explain_options.h"
 #include "mongo/db/query/query_knobs_gen.h"
+#include "mongo/db/query/query_shape/serialization_options.h"
+#include "mongo/db/service_context.h"
 #include "mongo/executor/task_executor.h"
+#include "mongo/stdx/unordered_set.h"
+#include "mongo/util/assert_util_core.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/fail_point.h"
 #include "mongo/util/intrusive_counter.h"
 #include "mongo/util/timer.h"
+#include "mongo/util/uuid.h"
 
 namespace mongo {
 class BSONObj;
@@ -133,6 +156,15 @@ public:
         const boost::intrusive_ptr<ExpressionContext>& expCtx,
         PipelineValidatorCallback validator = nullptr);
 
+    /**
+     * Parses sub-pipelines from a $facet aggregation. Like parse(), but skips top-level
+     * validators.
+     */
+    static std::unique_ptr<Pipeline, PipelineDeleter> parseFacetPipeline(
+        const std::vector<BSONObj>& rawPipeline,
+        const boost::intrusive_ptr<ExpressionContext>& expCtx,
+        PipelineValidatorCallback validator = nullptr);
+
     /**
      * Like parse, but takes a BSONElement instead of a vector of objects. 'arrElem' must be an
      * array of objects.
@@ -265,9 +297,33 @@ public:
     BSONObj getInitialQuery() const;
 
     /**
-     * Returns 'true' if the pipeline must merge on the primary shard.
+     * Convenience wrapper that parameterizes a pipeline's match stage, if present.
      */
-    bool needsPrimaryShardMerger() const;
+    void parameterize();
+
+    /**
+     * Clear any parameterization in the pipeline.
+     */
+    void unparameterize();
+
+    /**
+     * Returns 'true' if a pipeline's structure is eligible for parameterization. It must have a
+     * $match first stage.
+     */
+    bool canParameterize();
+
+    /**
+     * Returns 'true' if a pipeline is parameterized.
+     */
+    bool isParameterized() {
+        return _isParameterized;
+    }
+
+    /**
+     * Returns a specific ShardId that should be merger for this pipeline or boost::none if it is
+     * not needed.
+     */
+    boost::optional<ShardId> needsSpecificShardMerger() const;
 
     /**
      * Returns 'true' if the pipeline must merge on mongoS.
@@ -280,15 +336,16 @@ public:
     bool needsShard() const;
 
     /**
-     * Returns 'true' if any stage in the pipeline requires being run on all shards.
+     * Returns 'true' if any stage in the pipeline requires being run on all hosts within targeted
+     * shards.
      */
-    bool needsAllShardServers() const;
+    bool needsAllShardHosts() const;
 
     /**
-     * Returns true if the pipeline can run on mongoS, but is not obliged to; that is, it can run
-     * either on mongoS or on a shard.
+     * Returns Status::OK() if the pipeline can run on mongoS, but is not obliged to; that is, it
+     * can run either on mongoS or on a shard.
      */
-    bool canRunOnMongos() const;
+    Status canRunOnMongos() const;
 
     /**
      * Returns true if this pipeline must only run on mongoS. Can be called on unsplit or merge
@@ -320,8 +377,7 @@ public:
     stdx::unordered_set<NamespaceString> getInvolvedCollections() const;
 
     /**
-     * Helpers to serialize a pipeline. If serializing for logging, use one of the "*forLogging"
-     * helpers, which handle redaction.
+     * Helpers to serialize a pipeline.
      */
     std::vector<Value> serialize(
         boost::optional<const SerializationOptions&> opts = boost::none) const;
@@ -331,13 +387,6 @@ public:
         const SourceContainer& container,
         boost::optional<const SerializationOptions&> opts = boost::none);
 
-    std::vector<BSONObj> serializeForLogging(
-        boost::optional<const SerializationOptions&> opts = boost::none) const;
-    static std::vector<BSONObj> serializeContainerForLogging(
-        const SourceContainer& container,
-        boost::optional<const SerializationOptions&> opts = boost::none);
-    static std::vector<BSONObj> serializePipelineForLogging(const std::vector<BSONObj>& pipeline);
-
     // The initial source is special since it varies between mongos and mongod.
     void addInitialSource(boost::intrusive_ptr<DocumentSource> source);
 
@@ -486,6 +535,7 @@ private:
         const std::vector<T>& rawPipeline,
         const boost::intrusive_ptr<ExpressionContext>& expCtx,
         PipelineValidatorCallback validator,
+        bool isFacetPipeline,
         std::function<BSONObj(T)> getElemFunc);
 
     /**
@@ -495,12 +545,6 @@ private:
      */
     void stitch();
 
-    /**
-     * Returns Status::OK if the pipeline can run on mongoS, or an error with a message explaining
-     * why it cannot.
-     */
-    Status _pipelineCanRunOnMongoS() const;
-
     /**
      * Asserts whether operation contexts associated with this pipeline are consistent across
      * sources.
@@ -512,6 +556,7 @@ private:
     SplitState _splitState = SplitState::kUnsplit;
     boost::intrusive_ptr<ExpressionContext> pCtx;
     bool _disposed = false;
+    bool _isParameterized = false;
 };
 
 /**
@@ -565,4 +610,6 @@ extern ServiceContext::Decoration<std::unique_ptr<Pipeline, PipelineDeleter> (*)
     Pipeline* origPipeline,
     boost::optional<UUID> uuid)>
     generateMetadataPipelineFunc;
+
+using PipelinePtr = std::unique_ptr<Pipeline, PipelineDeleter>;
 }  // namespace mongo
