@@ -30,60 +30,93 @@
 
 #include "mongo/db/pipeline/process_interface/common_mongod_process_interface.h"
 
+#include <absl/container/flat_hash_map.h>
 #include <algorithm>
+#include <boost/move/utility_core.hpp>
+#include <boost/smart_ptr.hpp>
+#include <cstddef>
+#include <limits>
 #include <vector>
 
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/db/auth/authorization_manager.h"
 #include "mongo/db/auth/authorization_session.h"
 #include "mongo/db/catalog/collection.h"
 #include "mongo/db/catalog/collection_catalog.h"
 #include "mongo/db/catalog/collection_options.h"
 #include "mongo/db/catalog/collection_uuid_mismatch.h"
-#include "mongo/db/catalog/create_collection.h"
 #include "mongo/db/catalog/database_holder.h"
-#include "mongo/db/catalog/document_validation.h"
-#include "mongo/db/catalog/drop_collection.h"
+#include "mongo/db/catalog/index_catalog.h"
 #include "mongo/db/catalog/index_catalog_entry.h"
-#include "mongo/db/catalog/list_indexes.h"
-#include "mongo/db/catalog/rename_collection.h"
+#include "mongo/db/catalog_raii.h"
+#include "mongo/db/collection_index_usage_tracker.h"
 #include "mongo/db/concurrency/d_concurrency.h"
 #include "mongo/db/concurrency/exception_util.h"
+#include "mongo/db/concurrency/fill_locker_info.h"
+#include "mongo/db/concurrency/flow_control_ticketholder.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
 #include "mongo/db/curop.h"
 #include "mongo/db/cursor_manager.h"
 #include "mongo/db/db_raii.h"
 #include "mongo/db/dbhelpers.h"
+#include "mongo/db/exec/document_value/value.h"
 #include "mongo/db/index/index_descriptor.h"
-#include "mongo/db/multitenancy_gen.h"
+#include "mongo/db/pipeline/aggregate_command_gen.h"
+#include "mongo/db/pipeline/document_source.h"
 #include "mongo/db/pipeline/document_source_cursor.h"
+#include "mongo/db/pipeline/initialize_auto_get_helper.h"
 #include "mongo/db/pipeline/lite_parsed_pipeline.h"
 #include "mongo/db/pipeline/pipeline_d.h"
+#include "mongo/db/pipeline/search/search_helper.h"
+#include "mongo/db/pipeline/stage_constraints.h"
+#include "mongo/db/pipeline/variables.h"
 #include "mongo/db/query/collection_index_usage_tracker_decoration.h"
 #include "mongo/db/query/collection_query_info.h"
-#include "mongo/db/query/query_knobs_gen.h"
+#include "mongo/db/query/explain.h"
+#include "mongo/db/query/multiple_collection_accessor.h"
+#include "mongo/db/query/plan_cache.h"
 #include "mongo/db/query/sbe_plan_cache.h"
 #include "mongo/db/repl/primary_only_service.h"
 #include "mongo/db/s/query_analysis_writer.h"
-#include "mongo/db/s/sharding_state.h"
 #include "mongo/db/s/transaction_coordinator_curop.h"
 #include "mongo/db/s/transaction_coordinator_worker_curop_repository.h"
-#include "mongo/db/server_feature_flags_gen.h"
+#include "mongo/db/service_context.h"
+#include "mongo/db/session/kill_sessions.h"
 #include "mongo/db/session/session_catalog.h"
-#include "mongo/db/stats/fill_locker_info.h"
+#include "mongo/db/shard_id.h"
 #include "mongo/db/stats/storage_stats.h"
+#include "mongo/db/stats/top.h"
 #include "mongo/db/storage/backup_cursor_hooks.h"
 #include "mongo/db/storage/durable_catalog.h"
+#include "mongo/db/storage/record_data.h"
+#include "mongo/db/storage/recovery_unit.h"
+#include "mongo/db/storage/write_unit_of_work.h"
 #include "mongo/db/timeseries/catalog_helper.h"
-#include "mongo/db/timeseries/timeseries_constants.h"
 #include "mongo/db/timeseries/timeseries_options.h"
 #include "mongo/db/transaction/transaction_history_iterator.h"
 #include "mongo/db/transaction/transaction_participant.h"
 #include "mongo/db/transaction/transaction_participant_resource_yielder.h"
+#include "mongo/db/transaction_resources.h"
+#include "mongo/db/views/view.h"
 #include "mongo/logv2/log.h"
-#include "mongo/s/cluster_commands_helpers.h"
-#include "mongo/s/query/document_source_merge_cursors.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/s/analyze_shard_key_common_gen.h"
+#include "mongo/s/analyze_shard_key_role.h"
 #include "mongo/s/query_analysis_sample_tracker.h"
 #include "mongo/s/query_analysis_sampler_util.h"
+#include "mongo/s/sharding_state.h"
 #include "mongo/util/database_name_util.h"
+#include "mongo/util/future.h"
 #include "mongo/util/namespace_string_util.h"
+#include "mongo/util/str.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
@@ -125,7 +158,7 @@ void assertIgnorePrepareConflictsBehavior(const boost::intrusive_ptr<ExpressionC
     tassert(5996900,
             "Expected operation to either be blocking on prepare conflicts or ignoring prepare "
             "conflicts and allowing writes",
-            expCtx->opCtx->recoveryUnit()->getPrepareConflictBehavior() !=
+            shard_role_details::getRecoveryUnit(expCtx->opCtx)->getPrepareConflictBehavior() !=
                 PrepareConflictBehavior::kIgnoreConflicts);
 }
 
@@ -153,14 +186,15 @@ void listDurableCatalog(OperationContext* opCtx,
             continue;
         }
 
-        NamespaceString ns(NamespaceString::parseFromStringExpectTenantIdInMultitenancyMode(
+        NamespaceString ns(NamespaceStringUtil::parseFromStringExpectTenantIdInMultitenancyMode(
             obj.getStringField("ns")));
         if (ns.isSystemDotViews()) {
             systemViewsNamespaces->push_back(ns);
         }
 
         BSONObjBuilder builder;
-        builder.append("db", DatabaseNameUtil::serialize(ns.dbName()));
+        builder.append(
+            "db", DatabaseNameUtil::serialize(ns.dbName(), SerializationContext::stateDefault()));
         builder.append("name", ns.coll());
         builder.append("type", "collection");
         if (!shardName.empty()) {
@@ -193,10 +227,10 @@ std::vector<Document> CommonMongodProcessInterface::getIndexStats(OperationConte
         return indexStats;
     }
 
-    auto indexStatsMap =
-        CollectionIndexUsageTrackerDecoration::get(collection->getSharedDecorations())
+    const auto& indexStatsMap =
+        CollectionIndexUsageTrackerDecoration::get(collection.getCollection().get())
             .getUsageStats();
-    for (auto&& indexStatsMapIter : *indexStatsMap) {
+    for (auto&& indexStatsMapIter : indexStatsMap) {
         auto indexName = indexStatsMapIter.first;
         auto stats = indexStatsMapIter.second;
         MutableDocument doc;
@@ -221,22 +255,8 @@ std::vector<Document> CommonMongodProcessInterface::getIndexStats(OperationConte
         auto entry = idxCatalog->getEntry(idx);
         doc["spec"] = Value(idx->infoObj());
 
-        // (Ignore FCV check): This feature flag doesn't have any upgrade/downgrade concerns.
-        if (feature_flags::gPointInTimeCatalogLookups.isEnabledAndIgnoreFCVUnsafe()) {
-            if (!entry->isReady(opCtx)) {
-                doc["building"] = Value(true);
-            }
-        } else {
-            // Not all indexes in the CollectionIndexUsageTracker may be visible or consistent with
-            // our snapshot. For this reason, it is unsafe to check `isReady` on the entry, which
-            // asserts that the index's in-memory state is consistent with our snapshot.
-            if (!entry->isPresentInMySnapshot(opCtx)) {
-                continue;
-            }
-
-            if (!entry->isReadyInMySnapshot(opCtx)) {
-                doc["building"] = Value(true);
-            }
+        if (!entry->isReady()) {
+            doc["building"] = Value(true);
         }
 
         indexStats.push_back(doc.freeze());
@@ -263,12 +283,11 @@ std::deque<BSONObj> CommonMongodProcessInterface::listCatalog(OperationContext*
 
         // We want to read all the system.views as well as _mdb_catalog (again) using a consistent
         // snapshot.
-        // TODO(SERVER-63754): Replace with a less verbose constructor overload when available.
         AutoGetCollectionForReadCommandMaybeLockFree collLock(
             opCtx,
             systemViewsNamespaces.front(),
-            AutoGetCollection::Options{}.secondaryNssOrUUIDs(
-                {++systemViewsNamespaces.cbegin(), systemViewsNamespaces.cend()}),
+            AutoGetCollection::Options{}.secondaryNssOrUUIDs(++systemViewsNamespaces.cbegin(),
+                                                             systemViewsNamespaces.cend()),
             AutoStatsTracker::LogMode::kUpdateTopAndCurOp);
 
         // If the primary collection is not available, it means the information from parsing
@@ -303,13 +322,17 @@ std::deque<BSONObj> CommonMongodProcessInterface::listCatalog(OperationContext*
             while (auto record = cursor->next()) {
                 BSONObj obj = record->data.releaseToBson();
 
-                NamespaceString ns(NamespaceStringUtil::deserialize(svns.nss().tenantId(),
-                                                                    obj.getStringField("_id")));
-                NamespaceString viewOnNs(NamespaceStringUtil::parseNamespaceFromDoc(
-                    ns.dbName(), obj.getStringField("viewOn")));
+                NamespaceString ns(
+                    NamespaceStringUtil::deserialize(svns.nss().tenantId(),
+                                                     obj.getStringField("_id"),
+                                                     SerializationContext::stateDefault()));
+                NamespaceString viewOnNs(
+                    NamespaceStringUtil::deserialize(ns.dbName(), obj.getStringField("viewOn")));
 
                 BSONObjBuilder builder;
-                builder.append("db", DatabaseNameUtil::serialize(ns.dbName()));
+                builder.append(
+                    "db",
+                    DatabaseNameUtil::serialize(ns.dbName(), SerializationContext::stateDefault()));
                 builder.append("name", ns.coll());
                 if (viewOnNs.isTimeseriesBucketsCollection()) {
                     builder.append("type", "timeseries");
@@ -334,21 +357,24 @@ boost::optional<BSONObj> CommonMongodProcessInterface::getCatalogEntry(
     const NamespaceString& ns,
     const boost::optional<UUID>& collUUID) const {
 
-    // Perform an AutoGetCollection. This will verify that the collection still exists at the given
+    // Perform an aquisition. This will verify that the collection still exists at the given
     // read concern. If it doesn't and the aggregation has specified a UUID then this acquisition
     // will fail.
-    AutoGetCollectionForRead coll{opCtx, ns, AutoGetCollection::Options{}.expectedUUID(collUUID)};
+    auto acquisition =
+        acquireCollectionMaybeLockFree(opCtx,
+                                       CollectionAcquisitionRequest::fromOpCtx(
+                                           opCtx, ns, AcquisitionPrerequisites::kRead, collUUID));
 
-    const auto& collPtr = coll.getCollection();
-
-    if (!collPtr) {
+    if (!acquisition.exists()) {
         return boost::none;
     }
 
-    auto obj = DurableCatalog::get(opCtx)->getCatalogEntry(opCtx, collPtr->getCatalogId());
+    auto obj = DurableCatalog::get(opCtx)->getCatalogEntry(
+        opCtx, acquisition.getCollectionPtr()->getCatalogId());
 
     BSONObjBuilder builder;
-    builder.append("db", DatabaseNameUtil::serialize(ns.dbName()));
+    builder.append("db",
+                   DatabaseNameUtil::serialize(ns.dbName(), SerializationContext::stateDefault()));
     builder.append("name", ns.coll());
     builder.append("type", "collection");
     if (auto shardName = getShardName(opCtx); !shardName.empty()) {
@@ -380,12 +406,13 @@ void CommonMongodProcessInterface::appendLatencyStats(OperationContext* opCtx,
 }
 
 Status CommonMongodProcessInterface::appendStorageStats(
-    OperationContext* opCtx,
+    const boost::intrusive_ptr<ExpressionContext>& expCtx,
     const NamespaceString& nss,
     const StorageStatsSpec& spec,
     BSONObjBuilder* builder,
     const boost::optional<BSONObj>& filterObj) const {
-    return appendCollectionStorageStats(opCtx, nss, spec, builder, filterObj);
+    return appendCollectionStorageStats(
+        expCtx->opCtx, nss, spec, expCtx->serializationCtxt, builder, filterObj);
 }
 
 Status CommonMongodProcessInterface::appendRecordCount(OperationContext* opCtx,
@@ -400,14 +427,14 @@ Status CommonMongodProcessInterface::appendQueryExecStats(OperationContext* opCt
     AutoGetCollectionForReadCommand collection(opCtx, nss);
     if (!collection) {
         return {ErrorCodes::NamespaceNotFound,
-                str::stream() << "Collection [" << nss.toString() << "] not found."};
+                str::stream() << "Collection [" << nss.toStringForErrorMsg() << "] not found."};
     }
 
     bool redactForQE =
         collection->getCollectionOptions().encryptedFieldConfig || nss.isFLE2StateCollection();
     if (!redactForQE) {
         auto collectionScanStats =
-            CollectionIndexUsageTrackerDecoration::get(collection->getSharedDecorations())
+            CollectionIndexUsageTrackerDecoration::get(collection.getCollection().get())
                 .getCollectionScanStats();
 
         dassert(collectionScanStats.collectionScans <=
@@ -427,14 +454,33 @@ Status CommonMongodProcessInterface::appendQueryExecStats(OperationContext* opCt
 
 BSONObj CommonMongodProcessInterface::getCollectionOptionsLocally(OperationContext* opCtx,
                                                                   const NamespaceString& nss) {
-    AutoGetCollectionForReadCommand collection(opCtx, nss);
-    BSONObj collectionOptions = {};
-    if (!collection) {
-        return collectionOptions;
+    auto acquisition =
+        acquireCollectionOrViewMaybeLockFree(opCtx,
+                                             CollectionOrViewAcquisitionRequest::fromOpCtx(
+                                                 opCtx,
+                                                 nss,
+                                                 AcquisitionPrerequisites::OperationType::kRead,
+                                                 AcquisitionPrerequisites::ViewMode::kCanBeView));
+
+    if (auto& collectionPtr = acquisition.getCollectionPtr(); collectionPtr) {
+        return collectionPtr->getCollectionOptions().toBSON();
     }
 
-    collectionOptions = collection->getCollectionOptions().toBSON();
-    return collectionOptions;
+    if (acquisition.isView()) {
+        const auto view = acquisition.getView().getViewDefinition();
+        if (view.timeseries()) {
+            return getCollectionOptionsLocally(opCtx, nss.makeTimeseriesBucketsNamespace());
+        }
+        BSONObjBuilder bob;
+        bob.append("viewOn", view.viewOn().coll());
+        bob.append("pipeline", view.pipeline());
+        if (view.defaultCollator()) {
+            bob.append("collation", view.defaultCollator()->getSpec().toBSON());
+        }
+        return bob.obj();
+    }
+
+    return {};
 }
 
 BSONObj CommonMongodProcessInterface::getCollectionOptions(OperationContext* opCtx,
@@ -442,21 +488,44 @@ BSONObj CommonMongodProcessInterface::getCollectionOptions(OperationContext* opC
     return getCollectionOptionsLocally(opCtx, nss);
 }
 
+query_shape::CollectionType CommonMongodProcessInterface::getCollectionTypeLocally(
+    OperationContext* opCtx, const NamespaceString& nss) {
+    return acquireCollectionOrViewMaybeLockFree(opCtx,
+                                                CollectionOrViewAcquisitionRequest::fromOpCtx(
+                                                    opCtx,
+                                                    nss,
+                                                    AcquisitionPrerequisites::OperationType::kRead,
+                                                    AcquisitionPrerequisites::ViewMode::kCanBeView))
+        .getCollectionType();
+}
+
+query_shape::CollectionType CommonMongodProcessInterface::getCollectionType(
+    OperationContext* opCtx, const NamespaceString& nss) {
+    return getCollectionTypeLocally(opCtx, nss);
+}
+
 std::unique_ptr<Pipeline, PipelineDeleter>
 CommonMongodProcessInterface::attachCursorSourceToPipelineForLocalRead(
     Pipeline* ownedPipeline,
     boost::optional<const AggregateCommandRequest&> aggRequest,
-    bool shouldUseCollectionDefaultCollator) {
+    bool shouldUseCollectionDefaultCollator,
+    ExecShardFilterPolicy shardFilterPolicy) {
     auto expCtx = ownedPipeline->getContext();
     std::unique_ptr<Pipeline, PipelineDeleter> pipeline(ownedPipeline,
                                                         PipelineDeleter(expCtx->opCtx));
 
-    boost::optional<DocumentSource*> firstStage = pipeline->getSources().empty()
-        ? boost::optional<DocumentSource*>{}
-        : pipeline->getSources().front().get();
+    Pipeline::SourceContainer& sources = pipeline->getSources();
+    boost::optional<DocumentSource*> firstStage =
+        sources.empty() ? boost::optional<DocumentSource*>{} : sources.front().get();
     invariant(!firstStage || !dynamic_cast<DocumentSourceCursor*>(*firstStage));
-    if (firstStage && !(*firstStage)->constraints().requiresInputDocSource) {
+
+    bool skipRequiresInputDocSourceCheck =
+        PipelineD::isSearchPresentAndEligibleForSbe(pipeline.get());
+
+    if (!skipRequiresInputDocSourceCheck && firstStage &&
+        !(*firstStage)->constraints().requiresInputDocSource) {
         // There's no need to attach a cursor here.
+        search_helpers::prepareSearchForNestedPipelineLegacyExecutor(pipeline.get());
         return pipeline;
     }
 
@@ -479,15 +548,25 @@ CommonMongodProcessInterface::attachCursorSourceToPipelineForLocalRead(
     // when constructing our query executor.
     auto lpp = LiteParsedPipeline(expCtx->ns, pipeline->serializeToBson());
     std::vector<NamespaceStringOrUUID> secondaryNamespaces = lpp.getForeignExecutionNamespaces();
-
-    AutoGetCollectionForReadCommandMaybeLockFree autoColl(
-        expCtx->opCtx,
-        expCtx->ns,
-        AutoGetCollection::Options{}.secondaryNssOrUUIDs(secondaryNamespaces),
-        AutoStatsTracker::LogMode::kUpdateTop);
-
     auto* opCtx = expCtx->opCtx;
-    const auto& collPtr = autoColl.getCollection();
+
+    boost::optional<AutoGetCollectionForReadCommandMaybeLockFree> autoColl = boost::none;
+    auto initAutoGetCallback = [&]() {
+        autoColl.emplace(opCtx,
+                         expCtx->ns,
+                         AutoGetCollection::Options{}.secondaryNssOrUUIDs(
+                             secondaryNamespaces.cbegin(), secondaryNamespaces.cend()),
+                         AutoStatsTracker::LogMode::kUpdateTop);
+    };
+
+    bool isAnySecondaryCollectionNotLocal =
+        intializeAutoGet(opCtx, expCtx->ns, secondaryNamespaces, initAutoGetCallback);
+
+    tassert(8322002,
+            "Should have initialized AutoGet* after calling 'initializeAutoGet'",
+            autoColl.has_value());
+
+    const auto& collPtr = autoColl->getCollection();
     if (aggRequest && aggRequest->getCollectionUUID() && collPtr) {
         checkCollectionUUIDMismatch(opCtx, expCtx->ns, collPtr, aggRequest->getCollectionUUID());
     }
@@ -499,26 +578,35 @@ CommonMongodProcessInterface::attachCursorSourceToPipelineForLocalRead(
         expCtx->setCollator(collPtr->getDefaultCollator()->clone());
     }
 
-    MultipleCollectionAccessor holder{opCtx,
-                                      &autoColl.getCollection(),
-                                      autoColl.getNss(),
-                                      autoColl.isAnySecondaryNamespaceAViewOrSharded(),
+    MultipleCollectionAccessor holder{expCtx->opCtx,
+                                      &autoColl->getCollection(),
+                                      autoColl->getNss(),
+                                      autoColl->isAnySecondaryNamespaceAView() ||
+                                          isAnySecondaryCollectionNotLocal,
                                       secondaryNamespaces};
     auto resolvedAggRequest = aggRequest ? &aggRequest.get() : nullptr;
     PipelineD::buildAndAttachInnerQueryExecutorToPipeline(
-        holder, expCtx->ns, resolvedAggRequest, pipeline.get());
+        holder, expCtx->ns, resolvedAggRequest, pipeline.get(), shardFilterPolicy);
 
     return pipeline;
 }
 
 std::string CommonMongodProcessInterface::getShardName(OperationContext* opCtx) const {
-    if (ShardingState::get(opCtx)->enabled()) {
-        return ShardingState::get(opCtx)->shardId().toString();
+    if (auto shardId = getShardId(opCtx)) {
+        return shardId->toString();
     }
 
     return std::string();
 }
 
+boost::optional<ShardId> CommonMongodProcessInterface::getShardId(OperationContext* opCtx) const {
+    if (ShardingState::get(opCtx)->enabled()) {
+        return ShardingState::get(opCtx)->shardId();
+    }
+
+    return {};
+}
+
 bool CommonMongodProcessInterface::inShardedEnvironment(OperationContext* opCtx) const {
     return ShardingState::get(opCtx)->enabled();
 }
@@ -553,14 +641,13 @@ boost::optional<Document> CommonMongodProcessInterface::doLookupSingleDocument(
         pipeline = Pipeline::makePipeline(
             aggRequest, foreignExpCtx, boost::none /* shardCursorsSortSpec */, opts);
     } catch (const ExceptionFor<ErrorCodes::NamespaceNotFound>& ex) {
-        LOGV2_DEBUG(
-            6726700, 1, "Namespace not found while looking up document", "error"_attr = redact(ex));
+        LOGV2_DEBUG(6726700, 1, "Namespace not found while looking up document", "error"_attr = ex);
         return boost::none;
     } catch (const ExceptionFor<ErrorCodes::CollectionUUIDMismatch>& ex) {
         LOGV2_DEBUG(9597600,
                     1,
                     "Target collection UUID is different from the expected UUID",
-                    "error"_attr = redact(ex));
+                    "error"_attr = ex);
         return boost::none;
     }
 
@@ -609,9 +696,12 @@ BackupCursorExtendState CommonMongodProcessInterface::extendBackupCursor(
 
 std::vector<BSONObj> CommonMongodProcessInterface::getMatchingPlanCacheEntryStats(
     OperationContext* opCtx, const NamespaceString& nss, const MatchExpression* matchExp) const {
-    const auto serializer = [](const auto& entry) {
+    const auto serializer = [](const auto& key, const auto& entry) {
         BSONObjBuilder out;
         Explain::planCacheEntryToBSON(entry, &out);
+        if (auto querySettings = key.querySettings().toBSON(); !querySettings.isEmpty()) {
+            out.append("querySettings"_sd, querySettings);
+        }
         return out.obj();
     };
 
@@ -623,8 +713,9 @@ std::vector<BSONObj> CommonMongodProcessInterface::getMatchingPlanCacheEntryStat
     };
 
     AutoGetCollection collection(opCtx, nss, MODE_IS);
-    uassert(
-        50933, str::stream() << "collection '" << nss.toString() << "' does not exist", collection);
+    uassert(50933,
+            str::stream() << "collection '" << nss.toStringForErrorMsg() << "' does not exist",
+            collection);
 
     const auto& collQueryInfo = CollectionQueryInfo::get(collection.getCollection());
     const auto planCache = collQueryInfo.getPlanCache();
@@ -683,13 +774,13 @@ bool CommonMongodProcessInterface::fieldsHaveSupportingUniqueIndex(
 }
 
 BSONObj CommonMongodProcessInterface::_reportCurrentOpForClient(
-    OperationContext* opCtx,
+    const boost::intrusive_ptr<ExpressionContext>& expCtx,
     Client* client,
     CurrentOpTruncateMode truncateOps,
     CurrentOpBacktraceMode backtraceMode) const {
     BSONObjBuilder builder;
 
-    CurOp::reportCurrentOpForClient(opCtx,
+    CurOp::reportCurrentOpForClient(expCtx,
                                     client,
                                     (truncateOps == CurrentOpTruncateMode::kTruncateOps),
                                     (backtraceMode == CurrentOpBacktraceMode::kIncludeBacktrace),
@@ -707,16 +798,16 @@ BSONObj CommonMongodProcessInterface::_reportCurrentOpForClient(
         }
 
         // Append lock stats before returning.
-        if (auto lockerInfo = clientOpCtx->lockState()->getLockerInfo(
-                CurOp::get(*clientOpCtx)->getLockStatsBase())) {
-            fillLockerInfo(*lockerInfo, builder);
-        }
+        auto lockerInfo = shard_role_details::getLocker(clientOpCtx)
+                              ->getLockerInfo(CurOp::get(*clientOpCtx)->getLockStatsBase());
+        fillLockerInfo(lockerInfo, builder);
+
 
         if (auto tcWorkerRepo = getTransactionCoordinatorWorkerCurOpRepository()) {
             tcWorkerRepo->reportState(clientOpCtx, &builder);
         }
 
-        auto flowControlStats = clientOpCtx->lockState()->getFlowControlStats();
+        auto flowControlStats = shard_role_details::getLocker(clientOpCtx)->getFlowControlStats();
         flowControlStats.writeToBuilder(builder);
     }
 
@@ -803,8 +894,10 @@ BSONObj CommonMongodProcessInterface::_convertRenameToInternalRename(
 
     BSONObjBuilder newCmd;
     newCmd.append("internalRenameIfOptionsAndIndexesMatch", 1);
-    newCmd.append("from", NamespaceStringUtil::serialize(sourceNs));
-    newCmd.append("to", NamespaceStringUtil::serialize(targetNs));
+    newCmd.append("from",
+                  NamespaceStringUtil::serialize(sourceNs, SerializationContext::stateDefault()));
+    newCmd.append("to",
+                  NamespaceStringUtil::serialize(targetNs, SerializationContext::stateDefault()));
     newCmd.append("collectionOptions", originalCollectionOptions);
     BSONArrayBuilder indexArrayBuilder(newCmd.subarrayStart("indexes"));
     for (auto&& index : originalIndexes) {
@@ -822,19 +915,28 @@ void CommonMongodProcessInterface::_handleTimeseriesCreateError(const DBExceptio
     // specification as the time-series view we wanted to create, we should not throw an
     // error. The user is allowed to overwrite an existing time-series collection when
     // entering this function.
-    auto view = CollectionCatalog::get(opCtx)->lookupView(opCtx, ns);
-    // Confirming the error is NamespaceExists and that there is a time-series view in that
-    // namespace.
-    if (ex.code() != ErrorCodes::NamespaceExists || !view || !view->timeseries()) {
+
+    // Confirming the error is NamespaceExists
+    if (ex.code() != ErrorCodes::NamespaceExists) {
         throw;
     }
-    // Confirming the time-series options of the existing view are the same as expected.
-    auto timeseriesOpts = mongo::timeseries::getTimeseriesOptions(opCtx, ns, true);
+    auto timeseriesOpts = _getTimeseriesOptions(opCtx, ns);
+    // Confirming there is a time-series view in that namespace and the time-series options of the
+    // existing view are the same as expected.
     if (!timeseriesOpts || !mongo::timeseries::optionsAreEqual(timeseriesOpts.value(), userOpts)) {
         throw;
     }
 }
 
+boost::optional<TimeseriesOptions> CommonMongodProcessInterface::_getTimeseriesOptions(
+    OperationContext* opCtx, const NamespaceString& ns) {
+    auto view = CollectionCatalog::get(opCtx)->lookupView(opCtx, ns);
+    if (!view || !view->timeseries()) {
+        return boost::none;
+    }
+    return mongo::timeseries::getTimeseriesOptions(opCtx, ns, true /*convertToBucketsNamespace*/);
+}
+
 void CommonMongodProcessInterface::writeRecordsToRecordStore(
     const boost::intrusive_ptr<ExpressionContext>& expCtx,
     RecordStore* rs,
@@ -842,7 +944,7 @@ void CommonMongodProcessInterface::writeRecordsToRecordStore(
     const std::vector<Timestamp>& ts) const {
     tassert(5643012, "Attempted to write to record store with nullptr", records);
     assertIgnorePrepareConflictsBehavior(expCtx);
-    writeConflictRetry(expCtx->opCtx, "MPI::writeRecordsToRecordStore", expCtx->ns.ns(), [&] {
+    writeConflictRetry(expCtx->opCtx, "MPI::writeRecordsToRecordStore", expCtx->ns, [&] {
         Lock::GlobalLock lk(expCtx->opCtx, MODE_IS);
         WriteUnitOfWork wuow(expCtx->opCtx);
         auto writeResult = rs->insertRecords(expCtx->opCtx, records, ts);
@@ -872,7 +974,7 @@ Document CommonMongodProcessInterface::readRecordFromRecordStore(
 void CommonMongodProcessInterface::deleteRecordFromRecordStore(
     const boost::intrusive_ptr<ExpressionContext>& expCtx, RecordStore* rs, RecordId rID) const {
     assertIgnorePrepareConflictsBehavior(expCtx);
-    writeConflictRetry(expCtx->opCtx, "MPI::deleteFromRecordStore", expCtx->ns.ns(), [&] {
+    writeConflictRetry(expCtx->opCtx, "MPI::deleteFromRecordStore", expCtx->ns, [&] {
         Lock::GlobalLock lk(expCtx->opCtx, MODE_IS);
         WriteUnitOfWork wuow(expCtx->opCtx);
         rs->deleteRecord(expCtx->opCtx, rID);
@@ -883,7 +985,7 @@ void CommonMongodProcessInterface::deleteRecordFromRecordStore(
 void CommonMongodProcessInterface::truncateRecordStore(
     const boost::intrusive_ptr<ExpressionContext>& expCtx, RecordStore* rs) const {
     assertIgnorePrepareConflictsBehavior(expCtx);
-    writeConflictRetry(expCtx->opCtx, "MPI::truncateRecordStore", expCtx->ns.ns(), [&] {
+    writeConflictRetry(expCtx->opCtx, "MPI::truncateRecordStore", expCtx->ns, [&] {
         Lock::GlobalLock lk(expCtx->opCtx, MODE_IS);
         WriteUnitOfWork wuow(expCtx->opCtx);
         auto status = rs->truncate(expCtx->opCtx);
@@ -904,9 +1006,4 @@ boost::optional<Document> CommonMongodProcessInterface::lookupSingleDocumentLoca
     return Document(document).getOwned();
 }
 
-std::unique_ptr<ResourceYielder> CommonMongodProcessInterface::getResourceYielder(
-    StringData cmdName) const {
-    return TransactionParticipantResourceYielder::make(cmdName);
-}
-
 }  // namespace mongo
