@@ -29,13 +29,54 @@
 
 #pragma once
 
+#include <boost/none.hpp>
+#include <boost/optional.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <deque>
+#include <list>
+#include <map>
+#include <memory>
+#include <set>
+#include <string>
+#include <utility>
+#include <vector>
+
+#include "mongo/base/status.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/timestamp.h"
 #include "mongo/db/client.h"
+#include "mongo/db/database_name.h"
+#include "mongo/db/exec/document_value/document.h"
 #include "mongo/db/exec/shard_filterer.h"
+#include "mongo/db/generic_cursor_gen.h"
+#include "mongo/db/matcher/expression.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/ops/write_ops_exec.h"
 #include "mongo/db/ops/write_ops_gen.h"
+#include "mongo/db/pipeline/expression_context.h"
+#include "mongo/db/pipeline/field_path.h"
 #include "mongo/db/pipeline/javascript_execution.h"
 #include "mongo/db/pipeline/pipeline.h"
 #include "mongo/db/pipeline/process_interface/common_process_interface.h"
+#include "mongo/db/pipeline/process_interface/mongo_process_interface.h"
+#include "mongo/db/pipeline/storage_stats_spec_gen.h"
+#include "mongo/db/query/collation/collator_interface.h"
+#include "mongo/db/record_id.h"
+#include "mongo/db/repl/optime.h"
+#include "mongo/db/resource_yielder.h"
+#include "mongo/db/storage/backup_cursor_state.h"
+#include "mongo/db/storage/key_format.h"
+#include "mongo/db/storage/record_store.h"
+#include "mongo/db/storage/storage_engine.h"
+#include "mongo/db/storage/temporary_record_store.h"
+#include "mongo/db/timeseries/timeseries_gen.h"
+#include "mongo/s/chunk_version.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/uuid.h"
 
 namespace mongo {
 
@@ -47,7 +88,7 @@ class CommonMongodProcessInterface : public CommonProcessInterface {
 public:
     using CommonProcessInterface::CommonProcessInterface;
 
-    virtual ~CommonMongodProcessInterface() = default;
+    ~CommonMongodProcessInterface() override = default;
 
     std::unique_ptr<TransactionHistoryIteratorBase> createTransactionHistoryIterator(
         repl::OpTime time) const final;
@@ -67,7 +108,7 @@ public:
                             const NamespaceString& nss,
                             bool includeHistograms,
                             BSONObjBuilder* builder) const final;
-    Status appendStorageStats(OperationContext* opCtx,
+    Status appendStorageStats(const boost::intrusive_ptr<ExpressionContext>& expCtx,
                               const NamespaceString& nss,
                               const StorageStatsSpec& spec,
                               BSONObjBuilder* builder,
@@ -77,16 +118,19 @@ public:
                              BSONObjBuilder* builder) const final;
     Status appendQueryExecStats(OperationContext* opCtx,
                                 const NamespaceString& nss,
-                                BSONObjBuilder* builder) const final override;
+                                BSONObjBuilder* builder) const final;
     BSONObj getCollectionOptions(OperationContext* opCtx, const NamespaceString& nss) override;
-
+    query_shape::CollectionType getCollectionType(OperationContext* opCtx,
+                                                  const NamespaceString& nss) override;
     std::unique_ptr<Pipeline, PipelineDeleter> attachCursorSourceToPipelineForLocalRead(
         Pipeline* pipeline,
         boost::optional<const AggregateCommandRequest&> aggRequest = boost::none,
-        bool shouldUseCollectionDefaultCollator = false) final;
-
+        bool shouldUseCollectionDefaultCollator = false,
+        ExecShardFilterPolicy shardFilterPolicy = AutomaticShardFiltering{}) final;
     std::string getShardName(OperationContext* opCtx) const final;
 
+    boost::optional<ShardId> getShardId(OperationContext* opCtx) const final;
+
     bool inShardedEnvironment(OperationContext* opCtx) const final;
 
     std::vector<GenericCursor> getIdleCursors(const boost::intrusive_ptr<ExpressionContext>& expCtx,
@@ -104,9 +148,7 @@ public:
 
     bool fieldsHaveSupportingUniqueIndex(const boost::intrusive_ptr<ExpressionContext>& expCtx,
                                          const NamespaceString& nss,
-                                         const std::set<FieldPath>& fieldPaths) const;
-
-    std::unique_ptr<ResourceYielder> getResourceYielder(StringData cmdName) const final;
+                                         const std::set<FieldPath>& fieldPaths) const override;
 
     std::pair<std::set<FieldPath>, boost::optional<ChunkVersion>>
     ensureFieldsUniqueOrResolveDocumentKey(
@@ -142,6 +184,9 @@ public:
 protected:
     BSONObj getCollectionOptionsLocally(OperationContext* opCtx, const NamespaceString& nss);
 
+    query_shape::CollectionType getCollectionTypeLocally(OperationContext* opCtx,
+                                                         const NamespaceString& nss);
+
     boost::optional<Document> doLookupSingleDocument(
         const boost::intrusive_ptr<ExpressionContext>& expCtx,
         const NamespaceString& nss,
@@ -149,7 +194,7 @@ protected:
         const Document& documentKey,
         MakePipelineOptions opts);
 
-    BSONObj _reportCurrentOpForClient(OperationContext* opCtx,
+    BSONObj _reportCurrentOpForClient(const boost::intrusive_ptr<ExpressionContext>& expCtx,
                                       Client* client,
                                       CurrentOpTruncateMode truncateOps,
                                       CurrentOpBacktraceMode backtraceMode) const final;
@@ -184,6 +229,13 @@ protected:
                                       const NamespaceString& ns,
                                       TimeseriesOptions userOpts);
 
+    /**
+     * If passed namespace is a timeseries, returns TimeseriesOptions. Otherwise, returns
+     * boost::none.
+     */
+    virtual boost::optional<TimeseriesOptions> _getTimeseriesOptions(OperationContext* opCtx,
+                                                                     const NamespaceString& ns);
+
 private:
     // Object which contains a JavaScript Scope, used for executing JS in pipeline stages and
     // expressions. Owned by the process interface so that there is one common scope for the
