@@ -30,40 +30,73 @@
 #pragma once
 
 #include <boost/intrusive_ptr.hpp>
+#include <boost/none.hpp>
 #include <boost/optional.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <cstdint>
 #include <deque>
 #include <list>
 #include <memory>
+#include <set>
 #include <string>
+#include <tuple>
+#include <utility>
 #include <vector>
 
+#include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/oid.h"
+#include "mongo/bson/timestamp.h"
 #include "mongo/client/dbclient_base.h"
+#include "mongo/client/dbclient_cursor.h"
 #include "mongo/db/collection_index_usage_tracker.h"
+#include "mongo/db/collection_type.h"
+#include "mongo/db/database_name.h"
 #include "mongo/db/exec/document_value/document.h"
 #include "mongo/db/exec/document_value/value.h"
+#include "mongo/db/exec/exec_shard_filter_policy.h"
 #include "mongo/db/generic_cursor.h"
+#include "mongo/db/generic_cursor_gen.h"
 #include "mongo/db/matcher/expression.h"
 #include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/ops/write_ops_exec.h"
+#include "mongo/db/ops/write_ops_gen.h"
+#include "mongo/db/ops/write_ops_parsers.h"
 #include "mongo/db/pipeline/field_path.h"
 #include "mongo/db/pipeline/lite_parsed_document_source.h"
 #include "mongo/db/pipeline/sharded_agg_helpers_targeting_policy.h"
 #include "mongo/db/pipeline/storage_stats_spec_gen.h"
+#include "mongo/db/pipeline/variables.h"
 #include "mongo/db/query/explain_options.h"
 #include "mongo/db/record_id.h"
 #include "mongo/db/repl/oplog_entry.h"
+#include "mongo/db/repl/optime.h"
 #include "mongo/db/resource_yielder.h"
 #include "mongo/db/storage/backup_cursor_hooks.h"
 #include "mongo/db/storage/backup_cursor_state.h"
+#include "mongo/db/storage/key_format.h"
+#include "mongo/db/storage/record_store.h"
+#include "mongo/db/storage/storage_engine.h"
 #include "mongo/db/storage/temporary_record_store.h"
+#include "mongo/db/timeseries/timeseries_gen.h"
+#include "mongo/db/write_concern_options.h"
 #include "mongo/executor/task_executor.h"
+#include "mongo/s/chunk_version.h"
+#include "mongo/s/database_version.h"
 #include "mongo/s/shard_version.h"
+#include "mongo/util/uuid.h"
 
 namespace mongo {
 
 class ShardFilterer;
 class ExpressionContext;
 class JsExecution;
+
 class Pipeline;
 class PipelineDeleter;
 class TransactionHistoryIteratorBase;
@@ -185,6 +218,15 @@ public:
         return true;
     }
 
+    /**
+     * Utility which determines which shard we should merge on. More precisely, if 'nss' is
+     * unsplittable or untracked, we should route to the shard which owns 'nss'. Note that this
+     * decision is inherently racy and subject to become stale. This is okay because either choice
+     * will work correctly, we are simply applying a heuristic optimization.
+     */
+    virtual boost::optional<ShardId> determineSpecificMergeShard(
+        OperationContext* opCtx, const NamespaceString& ns) const = 0;
+
     /**
      * Advances the proxied write time associated with the client in ReplClientInfo to
      * be at least as high as the one tracked by the OperationTimeTracker associated with the
@@ -268,7 +310,7 @@ public:
      * specific stats to be appended to parameter 'builder'. By passing 'boost::none' to
      * 'filterObj', the caller is requesting to append all possible storage stats.
      */
-    virtual Status appendStorageStats(OperationContext* opCtx,
+    virtual Status appendStorageStats(const boost::intrusive_ptr<ExpressionContext>& expCtx,
                                       const NamespaceString& nss,
                                       const StorageStatsSpec& spec,
                                       BSONObjBuilder* builder,
@@ -289,12 +331,20 @@ public:
                                         BSONObjBuilder* builder) const = 0;
 
     /**
-     * Gets the collection options for the collection given by 'nss'. Throws
-     * ErrorCodes::CommandNotSupportedOnView if 'nss' describes a view. Future callers may want to
+     * Gets the collection options for the collection given by 'nss'. If the nss represents a view,
+     * we will return the user spec that was used to create the view. Future callers may want to
      * parameterize this behavior.
      */
     virtual BSONObj getCollectionOptions(OperationContext* opCtx, const NamespaceString& nss) = 0;
 
+    /**
+     * Returns the query shape collection type in the namespace given by 'nss'. This function holds
+     * an acquisition that is freed right after execution, and can't guarantee if the collection or
+     * view exists in the future.
+     */
+    virtual query_shape::CollectionType getCollectionType(OperationContext* opCtx,
+                                                          const NamespaceString& nss) = 0;
+
     /**
      * Performs the given rename command if the collection given by 'targetNs' has the same options
      * as specified in 'originalCollectionOptions', and has the same indexes as 'originalIndexes'.
@@ -317,6 +367,23 @@ public:
     virtual void createCollection(OperationContext* opCtx,
                                   const DatabaseName& dbName,
                                   const BSONObj& cmdObj) = 0;
+
+    /**
+     * Creates a temporary collection 'nss', with the given 'collectionOptions'.
+     *
+     * On non-shard servers, temporary collections are collections with the 'temp' property set to
+     * true. On shard servers, temporary collections don't set the 'temp' property to true; instead,
+     * they are annotated on the 'kAggTempCollections' collection on this shard. In both cases,
+     * temporary collections are dropped (garbage-collected) on stepup (or startup in the case of
+     * standalone nodes).
+     *
+     * This function accepts an optional 'dataShard' parameter that indicates the shard where the
+     * temporary collection should be created.
+     */
+    virtual void createTempCollection(OperationContext* opCtx,
+                                      const NamespaceString& nss,
+                                      const BSONObj& collectionOptions,
+                                      boost::optional<ShardId> dataShard = boost::none) = 0;
     /**
      * Creates the view backing a time-series collection.
      */
@@ -335,6 +402,11 @@ public:
 
     virtual void dropCollection(OperationContext* opCtx, const NamespaceString& collection) = 0;
 
+    /**
+     * Drops a temporary collection that was previously created using 'createTempCollection'.
+     */
+    virtual void dropTempCollection(OperationContext* opCtx, const NamespaceString& nss) = 0;
+
     /**
      * Accepts a pipeline and returns a new one which will draw input from the underlying
      * collection. Performs no further optimization of the pipeline. NamespaceNotFound will be
@@ -348,7 +420,7 @@ public:
      * If `shardTargetingPolicy` is kNotAllowed, the cursor will only be for local reads regardless
      * of whether or not this function is called in a sharded environment.
      */
-    virtual std::unique_ptr<Pipeline, PipelineDeleter> attachCursorSourceToPipeline(
+    virtual std::unique_ptr<Pipeline, PipelineDeleter> preparePipelineForExecution(
         Pipeline* pipeline,
         ShardTargetingPolicy shardTargetingPolicy = ShardTargetingPolicy::kAllowed,
         boost::optional<BSONObj> readConcern = boost::none) = 0;
@@ -372,7 +444,7 @@ public:
      * This function takes ownership of the 'pipeline' argument, which is expected to be a valid
      * pointer to a Pipeline object.
      */
-    virtual std::unique_ptr<Pipeline, PipelineDeleter> attachCursorSourceToPipeline(
+    virtual std::unique_ptr<Pipeline, PipelineDeleter> preparePipelineForExecution(
         const boost::intrusive_ptr<ExpressionContext>& expCtx,
         const AggregateCommandRequest& aggRequest,
         Pipeline* pipeline,
@@ -409,7 +481,8 @@ public:
     virtual std::unique_ptr<Pipeline, PipelineDeleter> attachCursorSourceToPipelineForLocalRead(
         Pipeline* pipeline,
         boost::optional<const AggregateCommandRequest&> aggRequest = boost::none,
-        bool shouldUseCollectionDefaultCollator = false) = 0;
+        bool shouldUseCollectionDefaultCollator = false,
+        ExecShardFilterPolicy shardFilterPolicy = AutomaticShardFiltering{}) = 0;
 
     /**
      * Returns a vector of owned BSONObjs, each of which contains details of an in-progress
@@ -430,6 +503,11 @@ public:
      */
     virtual std::string getShardName(OperationContext* opCtx) const = 0;
 
+    /**
+     * Returns the the local shard if this process is a shardsvr, else boost::none.
+     */
+    virtual boost::optional<ShardId> getShardId(OperationContext* opCtx) const = 0;
+
     /**
      * Returns whether or not this process is running as part of a sharded cluster.
      */
@@ -558,8 +636,6 @@ public:
         const NamespaceString& nss,
         const boost::optional<DatabaseVersion>& dbVersion) = 0;
 
-    virtual std::unique_ptr<ResourceYielder> getResourceYielder(StringData cmdName) const = 0;
-
     /**
      * If the user did not provide the 'fieldPaths' set, a default unique key will be picked,
      * which can be either the "_id" field, or a shard key, depending on the 'outputNs' collection
