@@ -29,29 +29,56 @@
 
 #include "mongo/db/pipeline/process_interface/mongos_process_interface.h"
 
+#include <absl/container/node_hash_map.h>
+#include <algorithm>
+#include <iterator>
+#include <type_traits>
+#include <typeinfo>
+
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/client/read_preference.h"
+#include "mongo/db/auth/authorization_manager.h"
 #include "mongo/db/auth/authorization_session.h"
-#include "mongo/db/catalog/collection_catalog.h"
 #include "mongo/db/curop.h"
 #include "mongo/db/index/index_descriptor.h"
-#include "mongo/db/pipeline/document_source.h"
+#include "mongo/db/pipeline/aggregate_command_gen.h"
 #include "mongo/db/pipeline/document_source_merge.h"
+#include "mongo/db/pipeline/legacy_runtime_constants_gen.h"
 #include "mongo/db/pipeline/sharded_agg_helpers.h"
+#include "mongo/db/query/collation/collation_spec.h"
 #include "mongo/db/query/collation/collator_factory_interface.h"
+#include "mongo/db/query/collation/collator_interface.h"
+#include "mongo/db/query/cursor_response.h"
 #include "mongo/db/repl/read_concern_args.h"
+#include "mongo/db/service_context.h"
+#include "mongo/db/session/kill_sessions.h"
+#include "mongo/db/session/session_catalog.h"
 #include "mongo/executor/task_executor_pool.h"
 #include "mongo/logv2/redaction.h"
 #include "mongo/s/analyze_shard_key_role.h"
 #include "mongo/s/catalog_cache.h"
+#include "mongo/s/chunk_manager.h"
+#include "mongo/s/client/shard.h"
 #include "mongo/s/cluster_commands_helpers.h"
 #include "mongo/s/grid.h"
+#include "mongo/s/query/async_results_merger_params_gen.h"
 #include "mongo/s/query/cluster_cursor_manager.h"
 #include "mongo/s/query/document_source_merge_cursors.h"
 #include "mongo/s/query/establish_cursors.h"
-#include "mongo/s/query/router_exec_stage.h"
 #include "mongo/s/query_analysis_sample_tracker.h"
+#include "mongo/s/shard_version.h"
 #include "mongo/s/stale_shard_version_helpers.h"
 #include "mongo/s/transaction_router.h"
-#include "mongo/util/fail_point.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/net/hostandport.h"
+#include "mongo/util/str.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
@@ -67,10 +94,10 @@ StatusWith<CollectionRoutingInfo> getCollectionRoutingInfo(
     auto catalogCache = Grid::get(expCtx->opCtx)->catalogCache();
     auto swRoutingInfo = catalogCache->getCollectionRoutingInfo(expCtx->opCtx, expCtx->ns);
     // Additionally check that the ExpressionContext's UUID matches the collection routing info.
-    if (swRoutingInfo.isOK() && expCtx->uuid && swRoutingInfo.getValue().cm.isSharded()) {
+    if (swRoutingInfo.isOK() && expCtx->uuid && swRoutingInfo.getValue().cm.hasRoutingTable()) {
         if (!swRoutingInfo.getValue().cm.uuidMatches(*expCtx->uuid)) {
             return {ErrorCodes::NamespaceNotFound,
-                    str::stream() << "The UUID of collection " << expCtx->ns.ns()
+                    str::stream() << "The UUID of collection " << expCtx->ns.toStringForErrorMsg()
                                   << " changed; it may have been dropped and re-created."};
         }
     }
@@ -104,7 +131,7 @@ MongosProcessInterface::getWriteSizeEstimator(OperationContext* opCtx,
     return std::make_unique<TargetPrimaryWriteSizeEstimator>();
 }
 
-std::unique_ptr<Pipeline, PipelineDeleter> MongosProcessInterface::attachCursorSourceToPipeline(
+std::unique_ptr<Pipeline, PipelineDeleter> MongosProcessInterface::preparePipelineForExecution(
     Pipeline* ownedPipeline,
     ShardTargetingPolicy shardTargetingPolicy,
     boost::optional<BSONObj> readConcern) {
@@ -113,11 +140,11 @@ std::unique_ptr<Pipeline, PipelineDeleter> MongosProcessInterface::attachCursorS
             "shardTargetingPolicy cannot be kNotAllowed on mongos",
             shardTargetingPolicy != ShardTargetingPolicy::kNotAllowed);
 
-    return sharded_agg_helpers::attachCursorToPipeline(
+    return sharded_agg_helpers::preparePipelineForExecution(
         ownedPipeline, shardTargetingPolicy, std::move(readConcern));
 }
 
-std::unique_ptr<Pipeline, PipelineDeleter> MongosProcessInterface::attachCursorSourceToPipeline(
+std::unique_ptr<Pipeline, PipelineDeleter> MongosProcessInterface::preparePipelineForExecution(
     const boost::intrusive_ptr<ExpressionContext>& expCtx,
     const AggregateCommandRequest& aggRequest,
     Pipeline* pipeline,
@@ -187,7 +214,7 @@ boost::optional<Document> MongosProcessInterface::lookupSingleDocument(
                 auto cri = uassertStatusOK(getCollectionRoutingInfo(foreignExpCtx));
 
                 // Finalize the 'find' command object based on the routing table information.
-                if (findCmdIsByUuid && cri.cm.isSharded()) {
+                if (findCmdIsByUuid && cri.cm.hasRoutingTable()) {
                     // Find by UUID and shard versioning do not work together (SERVER-31946).  In
                     // the sharded case we've already checked the UUID, so find by namespace is
                     // safe.  In the unlikely case that the collection has been deleted and a new
@@ -260,13 +287,13 @@ boost::optional<Document> MongosProcessInterface::lookupSingleDocumentLocally(
 }
 
 BSONObj MongosProcessInterface::_reportCurrentOpForClient(
-    OperationContext* opCtx,
+    const boost::intrusive_ptr<ExpressionContext>& expCtx,
     Client* client,
     CurrentOpTruncateMode truncateOps,
     CurrentOpBacktraceMode backtraceMode) const {
     BSONObjBuilder builder;
 
-    CurOp::reportCurrentOpForClient(opCtx,
+    CurOp::reportCurrentOpForClient(expCtx,
                                     client,
                                     (truncateOps == CurrentOpTruncateMode::kTruncateOps),
                                     (backtraceMode == CurrentOpBacktraceMode::kIncludeBacktrace),
@@ -350,7 +377,9 @@ bool MongosProcessInterface::fieldsHaveSupportingUniqueIndex(
     // this is any shard that currently owns at least one chunk. This helper sends database and/or
     // shard versions to ensure this router is not stale, but will not automatically retry if either
     // version is stale.
-    auto response = loadIndexesFromAuthoritativeShard(expCtx->opCtx, nss);
+    const auto cri = uassertStatusOK(
+        Grid::get(expCtx->opCtx)->catalogCache()->getCollectionRoutingInfo(expCtx->opCtx, nss));
+    auto response = loadIndexesFromAuthoritativeShard(expCtx->opCtx, nss, cri);
 
     // If the namespace does not exist, then the field paths *must* be _id only.
     if (response.getStatus() == ErrorCodes::NamespaceNotFound) {
