@@ -46,7 +46,7 @@ class StubMongoProcessInterface : public MongoProcessInterface {
 public:
     StubMongoProcessInterface() : MongoProcessInterface(nullptr) {}
     using MongoProcessInterface::MongoProcessInterface;
-    virtual ~StubMongoProcessInterface() = default;
+    ~StubMongoProcessInterface() override = default;
 
     std::unique_ptr<TransactionHistoryIteratorBase> createTransactionHistoryIterator(
         repl::OpTime time) const override {
@@ -88,6 +88,11 @@ public:
         return false;
     }
 
+    boost::optional<ShardId> determineSpecificMergeShard(
+        OperationContext* opCtx, const NamespaceString& nss) const override {
+        return boost::none;
+    };
+
     void updateClientOperationTime(OperationContext* opCtx) const override {}
 
     Status insert(const boost::intrusive_ptr<ExpressionContext>& expCtx,
@@ -153,7 +158,7 @@ public:
         MONGO_UNREACHABLE;
     }
 
-    Status appendStorageStats(OperationContext* opCtx,
+    Status appendStorageStats(const boost::intrusive_ptr<ExpressionContext>& expCtx,
                               const NamespaceString& nss,
                               const StorageStatsSpec& spec,
                               BSONObjBuilder* builder,
@@ -177,6 +182,11 @@ public:
         MONGO_UNREACHABLE;
     }
 
+    query_shape::CollectionType getCollectionType(OperationContext* opCtx,
+                                                  const NamespaceString& nss) final {
+        MONGO_UNREACHABLE;
+    }
+
     void renameIfOptionsAndIndexesHaveNotChanged(
         OperationContext* opCtx,
         const NamespaceString& sourceNs,
@@ -194,23 +204,35 @@ public:
         MONGO_UNREACHABLE;
     }
 
+    void createTempCollection(OperationContext* opCtx,
+                              const NamespaceString& nss,
+                              const BSONObj& collectionOptions,
+                              boost::optional<ShardId> dataShard) override {
+        MONGO_UNREACHABLE;
+    }
+
     void createIndexesOnEmptyCollection(OperationContext* opCtx,
                                         const NamespaceString& ns,
                                         const std::vector<BSONObj>& indexSpecs) override {
         MONGO_UNREACHABLE;
     }
+
     void dropCollection(OperationContext* opCtx, const NamespaceString& ns) override {
         MONGO_UNREACHABLE;
     }
 
-    std::unique_ptr<Pipeline, PipelineDeleter> attachCursorSourceToPipeline(
+    void dropTempCollection(OperationContext* opCtx, const NamespaceString& nss) override {
+        MONGO_UNREACHABLE;
+    }
+
+    std::unique_ptr<Pipeline, PipelineDeleter> preparePipelineForExecution(
         Pipeline* pipeline,
         ShardTargetingPolicy shardTargetingPolicy = ShardTargetingPolicy::kAllowed,
         boost::optional<BSONObj> readConcern = boost::none) override {
         MONGO_UNREACHABLE;
     }
 
-    std::unique_ptr<Pipeline, PipelineDeleter> attachCursorSourceToPipeline(
+    std::unique_ptr<Pipeline, PipelineDeleter> preparePipelineForExecution(
         const boost::intrusive_ptr<mongo::ExpressionContext>& expCtx,
         const AggregateCommandRequest& aggRequest,
         Pipeline* pipeline,
@@ -229,7 +251,8 @@ public:
     std::unique_ptr<Pipeline, PipelineDeleter> attachCursorSourceToPipelineForLocalRead(
         Pipeline* pipeline,
         boost::optional<const AggregateCommandRequest&> aggRequest = boost::none,
-        bool shouldUseCollectionDefaultCollator = false) override {
+        bool shouldUseCollectionDefaultCollator = false,
+        ExecShardFilterPolicy shardFilterPolicy = AutomaticShardFiltering{}) override {
         MONGO_UNREACHABLE;
     }
 
@@ -247,6 +270,10 @@ public:
         MONGO_UNREACHABLE;
     }
 
+    boost::optional<ShardId> getShardId(OperationContext* opCtx) const override {
+        return boost::none;
+    }
+
     bool inShardedEnvironment(OperationContext* opCtx) const override {
         MONGO_UNREACHABLE;
     }
@@ -265,19 +292,19 @@ public:
         const NamespaceString& nss,
         boost::optional<UUID> collectionUUID,
         const Document& documentKey,
-        boost::optional<BSONObj> readConcern) {
+        boost::optional<BSONObj> readConcern) override {
         MONGO_UNREACHABLE;
     }
 
     boost::optional<Document> lookupSingleDocumentLocally(
         const boost::intrusive_ptr<ExpressionContext>& expCtx,
         const NamespaceString& nss,
-        const Document& documentKey) {
+        const Document& documentKey) override {
         MONGO_UNREACHABLE_TASSERT(6148002);
     }
 
     std::vector<GenericCursor> getIdleCursors(const boost::intrusive_ptr<ExpressionContext>& expCtx,
-                                              CurrentOpUserMode userMode) const {
+                                              CurrentOpUserMode userMode) const override {
         MONGO_UNREACHABLE;
     }
 
@@ -318,10 +345,6 @@ public:
         uasserted(51019, "Unexpected check of routing table");
     }
 
-    std::unique_ptr<ResourceYielder> getResourceYielder(StringData cmdName) const override {
-        return nullptr;
-    }
-
     std::pair<std::set<FieldPath>, boost::optional<ChunkVersion>>
     ensureFieldsUniqueOrResolveDocumentKey(
         const boost::intrusive_ptr<ExpressionContext>& expCtx,
@@ -348,31 +371,31 @@ public:
     }
 
     std::unique_ptr<TemporaryRecordStore> createTemporaryRecordStore(
-        const boost::intrusive_ptr<ExpressionContext>& expCtx, KeyFormat keyFormat) const {
+        const boost::intrusive_ptr<ExpressionContext>& expCtx, KeyFormat keyFormat) const override {
         MONGO_UNREACHABLE;
     }
 
     void writeRecordsToRecordStore(const boost::intrusive_ptr<ExpressionContext>& expCtx,
                                    RecordStore* rs,
                                    std::vector<Record>* records,
-                                   const std::vector<Timestamp>& ts) const {
+                                   const std::vector<Timestamp>& ts) const override {
         MONGO_UNREACHABLE;
     }
 
     Document readRecordFromRecordStore(const boost::intrusive_ptr<ExpressionContext>& expCtx,
                                        RecordStore* rs,
-                                       RecordId rID) const {
+                                       RecordId rID) const override {
         MONGO_UNREACHABLE;
     }
 
     void deleteRecordFromRecordStore(const boost::intrusive_ptr<ExpressionContext>& expCtx,
                                      RecordStore* rs,
-                                     RecordId rID) const {
+                                     RecordId rID) const override {
         MONGO_UNREACHABLE;
     }
 
     void truncateRecordStore(const boost::intrusive_ptr<ExpressionContext>& expCtx,
-                             RecordStore* rs) const {
+                             RecordStore* rs) const override {
         MONGO_UNREACHABLE;
     }
 };
