@@ -82,25 +82,24 @@ AggregateCommandRequest asAggregateCommandRequest(const FindCommandRequest& find
                           << " not supported in aggregation.",
             !findCommand.getReadOnce());
 
-    uassert(ErrorCodes::InvalidPipelineOperator,
-            str::stream() << "Option " << FindCommandRequest::kStartAtFieldName
-                          << " not supported in aggregation.",
-            findCommand.getStartAt().isEmpty());
-
     uassert(ErrorCodes::InvalidPipelineOperator,
             str::stream() << "Option " << FindCommandRequest::kAllowSpeculativeMajorityReadFieldName
                           << " not supported in aggregation.",
             !findCommand.getAllowSpeculativeMajorityRead());
 
-    uassert(ErrorCodes::InvalidPipelineOperator,
-            str::stream() << "Option " << FindCommandRequest::kRequestResumeTokenFieldName
-                          << " not supported in aggregation.",
-            !findCommand.getRequestResumeToken());
+    // Some options are disallowed when resharding improvements are disabled.
+    if (!resharding::gFeatureFlagReshardingImprovements.isEnabled(
+            serverGlobalParams.featureCompatibility.acquireFCVSnapshot())) {
+        uassert(ErrorCodes::InvalidPipelineOperator,
+                str::stream() << "Option " << FindCommandRequest::kRequestResumeTokenFieldName
+                              << " not supported in aggregation.",
+                !findCommand.getRequestResumeToken());
 
-    uassert(ErrorCodes::InvalidPipelineOperator,
-            str::stream() << "Option " << FindCommandRequest::kResumeAfterFieldName
-                          << " not supported in aggregation.",
-            findCommand.getResumeAfter().isEmpty());
+        uassert(ErrorCodes::InvalidPipelineOperator,
+                str::stream() << "Option " << FindCommandRequest::kResumeAfterFieldName
+                              << " not supported in aggregation.",
+                findCommand.getResumeAfter().isEmpty());
+    }
 
     // Now that we've successfully validated this QR, begin building the aggregation command.
     tassert(ErrorCodes::BadValue,
@@ -130,8 +129,18 @@ AggregateCommandRequest asAggregateCommandRequest(const FindCommandRequest& find
 
     // The aggregation 'cursor' option is always set, regardless of the presence of batchSize.
     SimpleCursorOptions cursor;
-    if (findCommand.getBatchSize()) {
-        cursor.setBatchSize(findCommand.getBatchSize());
+    if (auto batchSize = findCommand.getBatchSize()) {
+        // If the find command specifies `singleBatch`, 'limit' is required to be 1 (as checked
+        // above). If 'batchSize' is also 1, an open cursor will be returned, contradicting the
+        // 'singleBatch' option. We set 'batchSize' to 2 as a workaround to ensure no cursor is
+        // returned.
+        // TODO SERVER-83077 This workaround will be unnecessary if a full batch of size 1 doesn't
+        // open a cursor.
+        if (findCommand.getSingleBatch() && *batchSize == 1LL) {
+            cursor.setBatchSize(2);
+        } else {
+            cursor.setBatchSize(*batchSize);
+        }
     }
     result.setCursor(std::move(cursor));
 
@@ -145,6 +154,9 @@ AggregateCommandRequest asAggregateCommandRequest(const FindCommandRequest& find
     if (!findCommand.getHint().isEmpty()) {
         result.setHint(findCommand.getHint().getOwned());
     }
+    if (findCommand.getQuerySettings()) {
+        result.setQuerySettings(findCommand.getQuerySettings());
+    }
     if (findCommand.getReadConcern()) {
         result.setReadConcern(findCommand.getReadConcern()->getOwned());
     }
@@ -158,6 +170,15 @@ AggregateCommandRequest asAggregateCommandRequest(const FindCommandRequest& find
     if (findCommand.getLet()) {
         result.setLet(findCommand.getLet()->getOwned());
     }
+    if (resharding::gFeatureFlagReshardingImprovements.isEnabled(
+            serverGlobalParams.featureCompatibility.acquireFCVSnapshot())) {
+        result.setRequestResumeToken(findCommand.getRequestResumeToken());
+
+        if (!findCommand.getResumeAfter().isEmpty()) {
+            result.setResumeAfter(findCommand.getResumeAfter().getOwned());
+        }
+    }
+    result.setIncludeQueryStatsMetrics(findCommand.getIncludeQueryStatsMetrics());
 
     return result;
 }
