@@ -29,12 +29,18 @@
 
 #pragma once
 
+#include <boost/none.hpp>
 #include <boost/optional.hpp>
+#include <boost/optional/optional.hpp>
+#include <functional>
 #include <set>
 #include <string>
+#include <utility>
 
+#include "mongo/db/pipeline/dependencies.h"
 #include "mongo/db/pipeline/document_source.h"
 #include "mongo/db/pipeline/pipeline.h"
+#include "mongo/util/string_map.h"
 
 namespace mongo::semantic_analysis {
 
