@@ -29,12 +29,35 @@
 
 #pragma once
 
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <cstddef>
+#include <memory>
+#include <utility>
+#include <variant>
+#include <vector>
+
+#include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/db/exec/document_value/document.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/pipeline/aggregate_command_gen.h"
+#include "mongo/db/pipeline/exchange_spec_gen.h"
+#include "mongo/db/pipeline/expression_context.h"
 #include "mongo/db/pipeline/pipeline.h"
+#include "mongo/db/pipeline/sharded_agg_helpers_targeting_policy.h"
+#include "mongo/db/query/explain_options.h"
+#include "mongo/db/shard_id.h"
 #include "mongo/s/async_requests_sender.h"
 #include "mongo/s/catalog_cache.h"
+#include "mongo/s/client/shard.h"
 #include "mongo/s/query/owned_remote_cursor.h"
 #include "mongo/s/stale_shard_version_helpers.h"
-#include "mongo/stdx/variant.h"
 
 namespace mongo {
 namespace sharded_agg_helpers {
@@ -70,9 +93,9 @@ struct ShardedExchangePolicy {
 };
 
 struct DispatchShardPipelineResults {
-    // True if this pipeline was split, and the second half of the pipeline needs to be run on
-    // the primary shard for the database.
-    bool needsPrimaryShardMerge;
+    // Contains a value when the second half of the pipeline was requested to run on a specific
+    // shard.
+    boost::optional<ShardId> mergeShardId;
 
     // Populated if this *is not* an explain, this vector represents the cursors on the remote
     // shards.
@@ -116,6 +139,16 @@ boost::optional<ShardedExchangePolicy> checkIfEligibleForExchange(OperationConte
  */
 SplitPipeline splitPipeline(std::unique_ptr<Pipeline, PipelineDeleter> pipeline);
 
+/**
+ * Used to indicate if a pipeline contains any data source requiring extra handling for targeting
+ * shards.
+ */
+enum class PipelineDataSource {
+    kNormal,
+    kChangeStream,  // Indicates a pipeline has a $changeStream stage.
+    kQueue,         // Indicates the desugared pipeline starts with a $queue stage.
+};
+
 /**
  * Targets shards for the pipeline and returns a struct with the remote cursors or results, and the
  * pipeline that will need to be executed to merge the results from the remotes. If a stale shard
@@ -139,22 +172,26 @@ SplitPipeline splitPipeline(std::unique_ptr<Pipeline, PipelineDeleter> pipeline)
  */
 DispatchShardPipelineResults dispatchShardPipeline(
     Document serializedCommand,
-    bool hasChangeStream,
-    bool startsWithDocuments,
+    PipelineDataSource pipelineDataSource,
     bool eligibleForSampling,
     std::unique_ptr<Pipeline, PipelineDeleter> pipeline,
     boost::optional<ExplainOptions::Verbosity> explain,
+    bool requestQueryStatsFromRemotes = false,
+    boost::optional<CollectionRoutingInfo> cri = boost::none,
     ShardTargetingPolicy shardTargetingPolicy = ShardTargetingPolicy::kAllowed,
-    boost::optional<BSONObj> readConcern = boost::none);
+    boost::optional<BSONObj> readConcern = boost::none,
+    AsyncRequestsSender::ShardHostMap designatedHostsMap = {},
+    stdx::unordered_map<ShardId, BSONObj> resumeTokenMap = {},
+    std::set<ShardId> shardsToSkip = {});
 
 BSONObj createPassthroughCommandForShard(
     const boost::intrusive_ptr<ExpressionContext>& expCtx,
     Document serializedCommand,
     boost::optional<ExplainOptions::Verbosity> explainVerbosity,
     Pipeline* pipeline,
-    BSONObj collationObj,
     boost::optional<BSONObj> readConcern,
-    boost::optional<int> overrideBatchSize);
+    boost::optional<int> overrideBatchSize,
+    bool requestQueryStatsFromRemotes);
 
 BSONObj createCommandForTargetedShards(const boost::intrusive_ptr<ExpressionContext>& expCtx,
                                        Document serializedCommand,
@@ -162,7 +199,8 @@ BSONObj createCommandForTargetedShards(const boost::intrusive_ptr<ExpressionCont
                                        boost::optional<ShardedExchangePolicy> exchangeSpec,
                                        bool needsMerge,
                                        boost::optional<ExplainOptions::Verbosity> explain,
-                                       boost::optional<BSONObj> readConcern = boost::none);
+                                       boost::optional<BSONObj> readConcern = boost::none,
+                                       bool requestQueryStatsFromRemotes = false);
 
 /**
  * Convenience method for callers that want to do 'partitionCursors', 'injectMetaCursors', and
@@ -170,7 +208,8 @@ BSONObj createCommandForTargetedShards(const boost::intrusive_ptr<ExpressionCont
  */
 void partitionAndAddMergeCursorsSource(Pipeline* pipeline,
                                        std::vector<OwnedRemoteCursor> cursors,
-                                       boost::optional<BSONObj> shardCursorsSortSpec);
+                                       boost::optional<BSONObj> shardCursorsSortSpec,
+                                       bool requestQueryStatsFromRemotes);
 
 /**
  * Targets the shards with an aggregation command built from `ownedPipeline` and explain set to
@@ -199,9 +238,7 @@ StatusWith<CollectionRoutingInfo> getExecutionNsRoutingInfo(OperationContext* op
 /**
  * Returns true if an aggregation over 'nss' must run on all shards.
  */
-bool checkIfMustRunOnAllShards(const NamespaceString& nss,
-                               bool hasChangeStream,
-                               bool startsWithDocuments);
+bool checkIfMustRunOnAllShards(const NamespaceString& nss, PipelineDataSource pipelineDataSource);
 
 /**
  * Retrieves the desired retry policy based on whether the default writeConcern is set on 'opCtx'.
@@ -209,11 +246,18 @@ bool checkIfMustRunOnAllShards(const NamespaceString& nss,
 Shard::RetryPolicy getDesiredRetryPolicy(OperationContext* opCtx);
 
 /**
- * Uses sharded_agg_helpers to split the pipeline and dispatch half to the shards, leaving the
- * merging half executing in this process after attaching a $mergeCursors. Will retry on network
- * errors and also on StaleConfig errors to avoid restarting the entire operation.
+ * Prepares the given pipeline for execution. This involves:
+ * (1) Determining if the pipeline needs to have a cursor source attached.
+ * (2) If a cursor source is needed, attaching one. This may involve a local or remote cursor,
+ * depending on whether or not the pipeline's expression context permits local reads and a local
+ * read could be used to serve the pipeline. (3) Splitting the pipeline if required, and dispatching
+ * half to the shards, leaving the merging half executing in this process after attaching a
+ * $mergeCursors.
+ *
+ * Will retry on network errors and also on StaleConfig errors to avoid restarting the entire
+ * operation. Returns `ownedPipeline`, but made-ready for execution.
  */
-std::unique_ptr<Pipeline, PipelineDeleter> attachCursorToPipeline(
+std::unique_ptr<Pipeline, PipelineDeleter> preparePipelineForExecution(
     Pipeline* ownedPipeline,
     ShardTargetingPolicy shardTargetingPolicy = ShardTargetingPolicy::kAllowed,
     boost::optional<BSONObj> readConcern = boost::none);
@@ -241,9 +285,9 @@ std::unique_ptr<Pipeline, PipelineDeleter> attachCursorToPipeline(
  */
 std::unique_ptr<Pipeline, PipelineDeleter> targetShardsAndAddMergeCursors(
     const boost::intrusive_ptr<ExpressionContext>& expCtx,
-    stdx::variant<std::unique_ptr<Pipeline, PipelineDeleter>,
-                  AggregateCommandRequest,
-                  std::pair<AggregateCommandRequest, std::unique_ptr<Pipeline, PipelineDeleter>>>
+    std::variant<std::unique_ptr<Pipeline, PipelineDeleter>,
+                 AggregateCommandRequest,
+                 std::pair<AggregateCommandRequest, std::unique_ptr<Pipeline, PipelineDeleter>>>
         targetRequest,
     boost::optional<BSONObj> shardCursorsSortSpec = boost::none,
     ShardTargetingPolicy shardTargetingPolicy = ShardTargetingPolicy::kAllowed,
@@ -264,7 +308,8 @@ std::unique_ptr<Pipeline, PipelineDeleter> targetShardsAndAddMergeCursors(
 std::unique_ptr<Pipeline, PipelineDeleter> runPipelineDirectlyOnSingleShard(
     const boost::intrusive_ptr<ExpressionContext>& expCtx,
     AggregateCommandRequest request,
-    ShardId shardId);
+    ShardId shardId,
+    bool requestQueryStatsFromRemotes);
 
 }  // namespace sharded_agg_helpers
 }  // namespace mongo
