@@ -71,16 +71,18 @@ struct StageConstraints {
         // Indicates that the stage must run on the host to which it was originally sent and
         // cannot be forwarded from mongoS to the shards.
         kLocalOnly,
-        // Indicates that the stage must run on the primary shard.
-        kPrimaryShard,
+        // Indicates that the stage must run exactly once, but it is ok to forward it from the
+        // router to a shard to execute if some other stage in the pipeline needs to run on a
+        // shard. The stage provides its own data and is independent of any collection.
+        kRunOnceAnyNode,
         // Indicates that the stage must run on any participating shard.
         kAnyShard,
         // Indicates that the stage can only run on mongoS.
         kMongoS,
-        // Indicates that the stage should run on all data-bearing nodes, primary and seconday, for
+        // Indicates that the stage should run on all data-bearing hosts, primary and secondary, for
         // the participating shards. This is useful for stages like $currentOp which generate
         // node-specific metadata.
-        kAllShardServers,
+        kAllShardHosts,
     };
 
     /**
@@ -194,8 +196,7 @@ struct StageConstraints {
         // shard, since it needs to be able to run on mongoS in a cluster.
         invariant(!(changeStreamRequirement == ChangeStreamRequirement::kAllowlist &&
                     (hostRequirement == HostTypeRequirement::kAnyShard ||
-                     hostRequirement == HostTypeRequirement::kPrimaryShard ||
-                     hostRequirement == HostTypeRequirement::kAllShardServers)));
+                     hostRequirement == HostTypeRequirement::kAllShardHosts)));
 
         // A stage which is allowlisted for $changeStream cannot have a position requirement.
         invariant(!(changeStreamRequirement == ChangeStreamRequirement::kAllowlist &&
@@ -219,7 +220,7 @@ struct StageConstraints {
             7355706,
             "Stage can only broadcast to all shard servers if it must be the first stage in the "
             "pipeline.",
-            hostRequirement != HostTypeRequirement::kAllShardServers ||
+            hostRequirement != HostTypeRequirement::kAllShardHosts ||
                 (requiredPosition == PositionRequirement::kFirst));
     }
 
@@ -379,6 +380,9 @@ struct StageConstraints {
     // following merge sort.
     bool preservesOrderAndMetadata = false;
 
+    // If set, merge should be performed on the specified shard.
+    boost::optional<ShardId> mergeShardId = boost::none;
+
     bool operator==(const StageConstraints& other) const {
         return requiredPosition == other.requiredPosition &&
             hostRequirement == other.hostRequirement && diskRequirement == other.diskRequirement &&
@@ -394,7 +398,8 @@ struct StageConstraints {
             canAppearOnlyOnceInPipeline == other.canAppearOnlyOnceInPipeline &&
             isAllowedWithinUpdatePipeline == other.isAllowedWithinUpdatePipeline &&
             unionRequirement == other.unionRequirement &&
-            preservesOrderAndMetadata == other.preservesOrderAndMetadata;
+            preservesOrderAndMetadata == other.preservesOrderAndMetadata &&
+            mergeShardId == other.mergeShardId;
     }
 };
 }  // namespace mongo
