@@ -28,6 +28,19 @@
  */
 
 #include "mongo/db/pipeline/window_function/window_function_exec.h"
+
+#include <boost/none.hpp>
+#include <boost/optional.hpp>
+#include <boost/smart_ptr.hpp>
+#include <variant>
+#include <vector>
+
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+
+#include "mongo/db/pipeline/document_source_set_window_fields.h"
+#include "mongo/db/pipeline/field_path.h"
+#include "mongo/db/pipeline/window_function/window_bounds.h"
 #include "mongo/db/pipeline/window_function/window_function_exec_derivative.h"
 #include "mongo/db/pipeline/window_function/window_function_exec_first_last.h"
 #include "mongo/db/pipeline/window_function/window_function_exec_linear_fill.h"
@@ -35,6 +48,7 @@
 #include "mongo/db/pipeline/window_function/window_function_exec_non_removable_range.h"
 #include "mongo/db/pipeline/window_function/window_function_exec_removable_document.h"
 #include "mongo/db/pipeline/window_function/window_function_exec_removable_range.h"
+#include "mongo/db/pipeline/window_function/window_function_expression.h"
 #include "mongo/db/pipeline/window_function/window_function_shift.h"
 
 namespace mongo {
@@ -73,11 +87,11 @@ std::unique_ptr<WindowFunctionExec> translateDocumentWindow(
     boost::intrusive_ptr<window_function::Expression> expr,
     const boost::optional<SortPattern>& sortBy,
     const WindowBounds::DocumentBased& bounds,
-    MemoryUsageTracker::PerFunctionMemoryTracker* memTracker) {
+    MemoryUsageTracker::Impl* memTracker) {
     auto inputExpr = translateInputExpression(expr, sortBy);
 
-    return stdx::visit(
-        visit_helper::Overloaded{
+    return visit(
+        OverloadedVisitor{
             [&](const WindowBounds::Unbounded&) -> std::unique_ptr<WindowFunctionExec> {
                 // A left unbounded window will always be non-removable regardless of the upper
                 // bound.
@@ -95,7 +109,7 @@ std::unique_ptr<mongo::WindowFunctionExec> translateDerivative(
     window_function::ExpressionDerivative* expr,
     PartitionIterator* iter,
     const boost::optional<SortPattern>& sortBy,
-    MemoryUsageTracker::PerFunctionMemoryTracker* memTracker) {
+    MemoryUsageTracker::Impl* memTracker) {
     tassert(5490703,
             "$derivative requires a 1-field sortBy",
             sortBy && sortBy->size() == 1 && !sortBy->begin()->expression);
@@ -118,8 +132,7 @@ std::unique_ptr<WindowFunctionExec> WindowFunctionExec::create(
     const boost::optional<SortPattern>& sortBy,
     MemoryUsageTracker* memTracker) {
 
-    MemoryUsageTracker::PerFunctionMemoryTracker& functionMemTracker =
-        (*memTracker)[functionStmt.fieldName];
+    MemoryUsageTracker::Impl& functionMemTracker = (*memTracker)[functionStmt.fieldName];
     if (auto expr = dynamic_cast<window_function::ExpressionDerivative*>(functionStmt.expr.get())) {
         return translateDerivative(expr, iter, sortBy, &functionMemTracker);
     } else if (auto expr =
@@ -144,46 +157,45 @@ std::unique_ptr<WindowFunctionExec> WindowFunctionExec::create(
 
     WindowBounds bounds = functionStmt.expr->bounds();
 
-    return stdx::visit(
-        visit_helper::Overloaded{
-            [&](const WindowBounds::DocumentBased& docBounds) {
-                return translateDocumentWindow(
-                    iter, functionStmt.expr, sortBy, docBounds, &functionMemTracker);
-            },
-            [&](const WindowBounds::RangeBased& rangeBounds)
-                -> std::unique_ptr<WindowFunctionExec> {
-                // These checks should be enforced already during parsing.
-                tassert(5429401,
-                        "Range-based window needs a non-compound sortBy",
-                        sortBy != boost::none && sortBy->size() == 1);
-                SortPattern::SortPatternPart part = *sortBy->begin();
-                tassert(5429410,
-                        "Range-based window doesn't work on expression-sortBy",
-                        part.fieldPath != boost::none && !part.expression);
-                auto sortByExpr = ExpressionFieldPath::createPathFromString(
-                    expCtx, part.fieldPath->fullPath(), expCtx->variablesParseState);
+    return visit(OverloadedVisitor{
+                     [&](const WindowBounds::DocumentBased& docBounds) {
+                         return translateDocumentWindow(
+                             iter, functionStmt.expr, sortBy, docBounds, &functionMemTracker);
+                     },
+                     [&](const WindowBounds::RangeBased& rangeBounds)
+                         -> std::unique_ptr<WindowFunctionExec> {
+                         // These checks should be enforced already during parsing.
+                         tassert(5429401,
+                                 "Range-based window needs a non-compound sortBy",
+                                 sortBy != boost::none && sortBy->size() == 1);
+                         SortPattern::SortPatternPart part = *sortBy->begin();
+                         tassert(5429410,
+                                 "Range-based window doesn't work on expression-sortBy",
+                                 part.fieldPath != boost::none && !part.expression);
+                         auto sortByExpr = ExpressionFieldPath::createPathFromString(
+                             expCtx, part.fieldPath->fullPath(), expCtx->variablesParseState);
 
-                auto inputExpr = translateInputExpression(functionStmt.expr, sortBy);
-                if (stdx::holds_alternative<WindowBounds::Unbounded>(rangeBounds.lower)) {
-                    return std::make_unique<WindowFunctionExecNonRemovableRange>(
-                        iter,
-                        inputExpr,
-                        std::move(sortByExpr),
-                        functionStmt.expr->buildAccumulatorOnly(),
-                        bounds,
-                        &functionMemTracker);
-                } else {
-                    return std::make_unique<WindowFunctionExecRemovableRange>(
-                        iter,
-                        inputExpr,
-                        std::move(sortByExpr),
-                        functionStmt.expr->buildRemovable(),
-                        bounds,
-                        &functionMemTracker);
-                }
-            },
-        },
-        bounds.bounds);
+                         auto inputExpr = translateInputExpression(functionStmt.expr, sortBy);
+                         if (holds_alternative<WindowBounds::Unbounded>(rangeBounds.lower)) {
+                             return std::make_unique<WindowFunctionExecNonRemovableRange>(
+                                 iter,
+                                 inputExpr,
+                                 std::move(sortByExpr),
+                                 functionStmt.expr->buildAccumulatorOnly(),
+                                 bounds,
+                                 &functionMemTracker);
+                         } else {
+                             return std::make_unique<WindowFunctionExecRemovableRange>(
+                                 iter,
+                                 inputExpr,
+                                 std::move(sortByExpr),
+                                 functionStmt.expr->buildRemovable(),
+                                 bounds,
+                                 &functionMemTracker);
+                         }
+                     },
+                 },
+                 bounds.bounds);
 }
 
 }  // namespace mongo
