@@ -29,20 +29,58 @@
 
 #pragma once
 
+#include <absl/container/node_hash_map.h>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <functional>
+#include <memory>
+#include <set>
+#include <string>
+#include <utility>
+#include <variant>
+#include <vector>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/init.h"  // IWYU pragma: keep
 #include "mongo/base/initializer.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/db/exec/document_value/document.h"
+#include "mongo/db/exec/document_value/value.h"
+#include "mongo/db/feature_flag.h"
 #include "mongo/db/pipeline/accumulator.h"
 #include "mongo/db/pipeline/accumulator_for_window_functions.h"
 #include "mongo/db/pipeline/accumulator_multi.h"
 #include "mongo/db/pipeline/accumulator_percentile.h"
+#include "mongo/db/pipeline/dependencies.h"
 #include "mongo/db/pipeline/document_source.h"
 #include "mongo/db/pipeline/document_source_set_window_fields_gen.h"
+#include "mongo/db/pipeline/expression.h"
+#include "mongo/db/pipeline/expression_context.h"
 #include "mongo/db/pipeline/expression_dependencies.h"
+#include "mongo/db/pipeline/field_path.h"
 #include "mongo/db/pipeline/percentile_algo.h"
+#include "mongo/db/pipeline/variables.h"
 #include "mongo/db/pipeline/window_function/window_bounds.h"
 #include "mongo/db/pipeline/window_function/window_function.h"
+#include "mongo/db/pipeline/window_function/window_function_integral.h"
+#include "mongo/db/query/allowed_contexts.h"
 #include "mongo/db/query/datetime/date_time_support.h"
 #include "mongo/db/query/query_feature_flags_gen.h"
+#include "mongo/db/query/query_shape/serialization_options.h"
 #include "mongo/db/query/sort_pattern.h"
+#include "mongo/platform/decimal128.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/intrusive_counter.h"
+#include "mongo/util/str.h"
+#include "mongo/util/string_map.h"
 
 namespace mongo {
 class WindowFunctionExec;
@@ -71,7 +109,8 @@ class PartitionIterator;
         if (!__VA_ARGS__ ||                                                                    \
             (boost::optional<FeatureFlag>(featureFlag) != boost::none &&                       \
              !boost::optional<FeatureFlag>(featureFlag)                                        \
-                  ->isEnabledAndIgnoreFCVUnsafeAtStartup())) {                                 \
+                  ->isEnabledUseLatestFCVWhenUninitialized(                                    \
+                      serverGlobalParams.featureCompatibility.acquireFCVSnapshot()))) {        \
             return;                                                                            \
         }                                                                                      \
         ::mongo::window_function::Expression::registerParser(                                  \
@@ -149,7 +188,7 @@ public:
     /**
      * Is this a function that the parser knows about?
      */
-    inline static bool isFunction(const mongo::StringData& name) {
+    inline static bool isFunction(mongo::StringData name) {
         return parserMap.find(name) != parserMap.end();
     }
 
@@ -171,7 +210,7 @@ public:
           _input(std::move(input)),
           _bounds(std::move(bounds)) {}
 
-    std::string getOpName() const {
+    StringData getOpName() const {
         return _accumulatorName;
     }
 
@@ -273,7 +312,9 @@ public:
                               std::string accumulatorName,
                               boost::intrusive_ptr<::mongo::Expression> input,
                               WindowBounds bounds)
-        : Expression(expCtx, std::move(accumulatorName), std::move(input), std::move(bounds)) {}
+        : Expression(expCtx, std::move(accumulatorName), std::move(input), std::move(bounds)) {
+        expCtx->sbeWindowCompatibility = SbeCompatibility::notCompatible;
+    }
 };
 
 /**
@@ -407,10 +448,10 @@ public:
         auto arg = obj.firstElement();
         auto argName = arg.fieldNameStringData();
         if (isFunction(argName)) {
+            accumulatorName = argName;
             uassert(5371603,
                     str::stream() << accumulatorName << " must be specified with '{}' as the value",
                     arg.type() == BSONType::Object && arg.embeddedObject().nFields() == 0);
-            accumulatorName = argName;
         } else {
             tasserted(ErrorCodes::FailedToParse,
                       str::stream() << "Window function found an unknown argument: " << argName);
@@ -427,21 +468,31 @@ public:
             auto sortExpression = ExpressionFieldPath::createPathFromString(
                 expCtx, sortPatternPart.fieldPath->fullPath(), expCtx->variablesParseState);
             return make_intrusive<ExpressionFromRankAccumulator<RankType>>(
-                expCtx, accumulatorName->toString(), std::move(sortExpression), std::move(bounds));
+                expCtx,
+                accumulatorName->toString(),
+                std::move(sortExpression),
+                sortPatternPart.isAscending,
+                std::move(bounds));
         } else {
             return make_intrusive<ExpressionFromRankAccumulator<RankType>>(
-                expCtx, accumulatorName->toString(), sortPatternPart.expression, std::move(bounds));
+                expCtx,
+                accumulatorName->toString(),
+                sortPatternPart.expression,
+                sortPatternPart.isAscending,
+                std::move(bounds));
         }
     }
 
     ExpressionFromRankAccumulator(ExpressionContext* expCtx,
                                   std::string accumulatorName,
                                   boost::intrusive_ptr<::mongo::Expression> input,
+                                  bool isAscending,
                                   WindowBounds bounds)
-        : Expression(expCtx, std::move(accumulatorName), std::move(input), std::move(bounds)) {}
+        : Expression(expCtx, std::move(accumulatorName), std::move(input), std::move(bounds)),
+          _isAscending(isAscending) {}
 
     boost::intrusive_ptr<AccumulatorState> buildAccumulatorOnly() const final {
-        return RankType::create(_expCtx);
+        return RankType::create(_expCtx, _isAscending);
     }
 
     std::unique_ptr<WindowFunctionState> buildRemovable() const final {
@@ -455,6 +506,9 @@ public:
         args.addField(_accumulatorName, Value(Document()));
         return args.freezeToValue();
     }
+
+private:
+    bool _isAscending;
 };
 
 class ExpressionExpMovingAvg : public Expression {
@@ -515,6 +569,14 @@ public:
         return outerObj.freezeToValue();
     }
 
+    boost::optional<long long> getN() {
+        return _N;
+    }
+
+    boost::optional<Decimal128> getAlpha() {
+        return _alpha;
+    }
+
 protected:
     boost::optional<long long> _N;
     boost::optional<Decimal128> _alpha;
@@ -536,6 +598,18 @@ public:
         return _unit;
     }
 
+    boost::optional<long long> unitInMillis() const {
+        if (!_unit)
+            return boost::none;
+
+        auto milliseconds = timeUnitTypicalMilliseconds(*_unit);
+        tassert(7823402,
+                "TimeUnit must be less than or equal to a 'week' ",
+                milliseconds <= timeUnitTypicalMilliseconds(TimeUnit::week));
+
+        return milliseconds;
+    }
+
     Value serialize(const SerializationOptions& opts) const final {
         MutableDocument result;
         result[_accumulatorName][kArgInput] = _input->serialize(opts);
@@ -559,7 +633,7 @@ protected:
             unit = parseTimeUnit(arg.valueStringData());
             switch (*unit) {
                 // These larger time units vary so much, it doesn't make sense to define a
-                // fixed conversion from milliseconds. (See 'timeUnitTypicalMilliseconds'.)
+                // fixed conversion from milliseconds.
                 case TimeUnit::year:
                 case TimeUnit::quarter:
                 case TimeUnit::month:
@@ -590,21 +664,12 @@ protected:
                 !sortBy->begin()->expression);
     }
 
-    boost::optional<long long> convertTimeUnitToMillis(boost::optional<TimeUnit> unit) const {
-        if (!unit)
-            return boost::none;
-
-        auto status = timeUnitTypicalMilliseconds(*unit);
-        tassert(status);
-
-        return status.getValue();
-    }
-
     boost::optional<TimeUnit> _unit;
 };
 
 class ExpressionDerivative : public ExpressionWithUnit {
 public:
+    static constexpr StringData kName = "$derivative"_sd;
     ExpressionDerivative(ExpressionContext* expCtx,
                          boost::intrusive_ptr<::mongo::Expression> input,
                          WindowBounds bounds,
@@ -745,16 +810,17 @@ public:
     }
 
     boost::intrusive_ptr<AccumulatorState> buildAccumulatorOnly() const final {
-        return AccumulatorIntegral::create(_expCtx, convertTimeUnitToMillis(_unit));
+        return AccumulatorIntegral::create(_expCtx, unitInMillis());
     }
 
     std::unique_ptr<WindowFunctionState> buildRemovable() const final {
-        return WindowFunctionIntegral::create(_expCtx, convertTimeUnitToMillis(_unit));
+        return WindowFunctionIntegral::create(_expCtx, unitInMillis());
     }
 };
 
 class ExpressionLinearFill : public Expression {
 public:
+    static constexpr StringData kName = "$linearFill"_sd;
     ExpressionLinearFill(ExpressionContext* expCtx,
                          std::string accumulatorName,
                          boost::intrusive_ptr<::mongo::Expression> input,
@@ -885,10 +951,10 @@ public:
 /**
  * Describes a window function expression that accepts 'n' as a parameter. Templated by
  * 'WindowFunctionN', which corresponds to the 'WindowFunctionState' removable type associated
- * with this expression. It is also templated by AccumulatorType which is the AccumulatorState
+ * with this expression. It is also templated by 'AccumulatorNType' which is the AccumulatorState
  * associated with this expression.
  */
-template <typename WindowFunctionN, typename AcumulatorNType>
+template <typename WindowFunctionN, typename AccumulatorNType>
 class ExpressionN : public Expression {
 public:
     static boost::intrusive_ptr<Expression> parse(BSONObj obj,
@@ -953,7 +1019,9 @@ public:
         : Expression(expCtx, std::move(accumulatorName), std::move(input), std::move(bounds)),
           _ps(std::move(ps)),
           _method(method),
-          _intializeExpr(std::move(initializeExpr)) {}
+          _intializeExpr(std::move(initializeExpr)) {
+        expCtx->sbeWindowCompatibility = SbeCompatibility::notCompatible;
+    }
 
     Value serialize(const SerializationOptions& opts) const final;
 
