@@ -29,10 +29,22 @@
 
 #pragma once
 
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <cmath>
+#include <limits>
+#include <memory>
+#include <utility>
+
+#include "mongo/bson/bsontypes.h"
+#include "mongo/db/exec/document_value/value.h"
 #include "mongo/db/pipeline/accumulator.h"
 #include "mongo/db/pipeline/document_source.h"
 #include "mongo/db/pipeline/expression.h"
+#include "mongo/db/pipeline/expression_context.h"
 #include "mongo/db/pipeline/window_function/window_function.h"
+#include "mongo/platform/decimal128.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/intrusive_counter.h"
 
 namespace mongo {
 
@@ -64,7 +76,7 @@ public:
 
     Value getValue() const override;
 
-    void reset() {
+    void reset() override {
         _sumAcc->reset();
         _posInfiniteValueCount = 0;
         _negInfiniteValueCount = 0;
