@@ -28,7 +28,13 @@
  */
 
 #include "mongo/db/prepare_conflict_tracker.h"
-#include "mongo/platform/basic.h"
+
+#include <utility>
+
+
+#include "mongo/db/service_context.h"
+#include "mongo/util/assert_util_core.h"
+#include "mongo/util/decorable.h"
 
 namespace mongo {
 
@@ -52,7 +58,7 @@ void PrepareConflictTracker::endPrepareConflict(OperationContext* opCtx) {
         auto curTick = tickSource->getTicks();
 
         auto curConflictDuration =
-            tickSource->spanTo<Microseconds>(_prepareConflictStartTime, curTick);
+            tickSource->ticksTo<Microseconds>(curTick - _prepareConflictStartTime);
         _prepareConflictDuration.store(_prepareConflictDuration.load() + curConflictDuration);
         _prepareConflictStartTime = 0;
 
