@@ -28,33 +28,46 @@
  */
 
 
-#include "mongo/platform/basic.h"
+#include <boost/cstdint.hpp>
+#include <boost/none.hpp>
+#include <boost/smart_ptr.hpp>
+#include <cstddef>
+#include <cstdint>
 
-#include "mongo/db/query/canonical_query.h"
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
 
-#include "mongo/crypto/encryption_fields_gen.h"
-#include "mongo/db/catalog/collection.h"
-#include "mongo/db/commands/test_commands_enabled.h"
-#include "mongo/db/jsobj.h"
-#include "mongo/db/matcher/expression_array.h"
+#include "mongo/base/error_codes.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/db/basic_types.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/operation_context.h"
+#include "mongo/db/pipeline/document_source_match.h"
+#include "mongo/db/query/canonical_query.h"
 #include "mongo/db/query/canonical_query_encoder.h"
-#include "mongo/db/query/collation/collator_factory_interface.h"
+#include "mongo/db/query/cqf_command_utils.h"
 #include "mongo/db/query/indexability.h"
 #include "mongo/db/query/parsed_find_command.h"
+#include "mongo/db/query/projection_ast_util.h"
 #include "mongo/db/query/projection_parser.h"
-#include "mongo/db/query/query_decorations.h"
+#include "mongo/db/query/query_knob_configuration.h"
 #include "mongo/db/query/query_knobs_gen.h"
 #include "mongo/db/query/query_planner_common.h"
+#include "mongo/db/server_parameter.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/util/str.h"
+#include "mongo/util/synchronized_value.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
 namespace mongo {
 namespace {
 
-boost::optional<size_t> loadMaxParameterCount() {
+boost::optional<size_t> loadMaxMatchExpressionParams() {
     auto value = internalQueryAutoParameterizationMaxParameterCount.load();
     if (value > 0) {
         return value;
@@ -65,147 +78,112 @@ boost::optional<size_t> loadMaxParameterCount() {
 
 }  // namespace
 
-// static
-StatusWith<std::unique_ptr<CanonicalQuery>> CanonicalQuery::canonicalize(
+boost::intrusive_ptr<ExpressionContext> makeExpressionContext(
     OperationContext* opCtx,
-    std::unique_ptr<FindCommandRequest> findCommand,
-    bool explain,
-    const boost::intrusive_ptr<ExpressionContext>& givenExpCtx,
-    const ExtensionsCallback& extensionsCallback,
-    MatchExpressionParser::AllowedFeatureSet allowedFeatures,
-    const ProjectionPolicies& projectionPolicies,
-    std::vector<std::unique_ptr<InnerPipelineStageInterface>> pipeline,
-    bool isCountLike) {
-    if (givenExpCtx) {
-        // Caller provided an ExpressionContext, let's go ahead and use that.
-        auto swParsedFind = parsed_find_command::parse(givenExpCtx,
-                                                       std::move(findCommand),
-                                                       extensionsCallback,
-                                                       allowedFeatures,
-                                                       projectionPolicies);
-        if (!swParsedFind.isOK()) {
-            return swParsedFind.getStatus();
+    FindCommandRequest& findCommand,
+    boost::optional<ExplainOptions::Verbosity> verbosity) {
+    auto collator = [&]() -> std::unique_ptr<mongo::CollatorInterface> {
+        if (findCommand.getCollation().isEmpty()) {
+            return nullptr;
         }
-        return canonicalize(std::move(givenExpCtx),
-                            std::move(swParsedFind.getValue()),
-                            explain,
-                            std::move(pipeline),
-                            isCountLike);
-    } else {
-        // No ExpressionContext provided, let's call the override that makes one for us.
-        auto swResults = parsed_find_command::parse(
-            opCtx, std::move(findCommand), extensionsCallback, allowedFeatures, projectionPolicies);
-        if (!swResults.isOK()) {
-            return swResults.getStatus();
-        }
-        auto&& [expCtx, parsedFind] = std::move(swResults.getValue());
-        return canonicalize(
-            std::move(expCtx), std::move(parsedFind), explain, std::move(pipeline), isCountLike);
-    }
-}
-
-// static
-StatusWith<std::unique_ptr<CanonicalQuery>> CanonicalQuery::canonicalize(
-    boost::intrusive_ptr<ExpressionContext> expCtx,
-    std::unique_ptr<ParsedFindCommand> parsedFind,
-    bool explain,
-    std::vector<std::unique_ptr<InnerPipelineStageInterface>> pipeline,
-    bool isCountLike) {
-
-    // Make the CQ we'll hopefully return.
-    auto cq = std::make_unique<CanonicalQuery>();
-    cq->setExplain(explain);
-    if (auto initStatus = cq->init(std::move(expCtx),
-                                   std::move(parsedFind),
-                                   std::move(pipeline),
-                                   isCountLike,
-                                   true /*optimizeMatchExpression*/);
-        !initStatus.isOK()) {
-        return initStatus;
-    }
-    return {std::move(cq)};
+        return uassertStatusOKWithContext(CollatorFactoryInterface::get(opCtx->getServiceContext())
+                                              ->makeFromBSON(findCommand.getCollation()),
+                                          "unable to parse collation");
+    }();
+    return make_intrusive<ExpressionContext>(
+        opCtx, findCommand, std::move(collator), true /* mayDbProfile */, std::move(verbosity));
 }
 
 // static
 StatusWith<std::unique_ptr<CanonicalQuery>> CanonicalQuery::makeForSubplanner(
     OperationContext* opCtx, const CanonicalQuery& baseQuery, size_t i) {
+    try {
+        return std::make_unique<CanonicalQuery>(opCtx, baseQuery, i);
+    } catch (const DBException& e) {
+        return e.toStatus();
+    }
+}
+
+// static
+StatusWith<std::unique_ptr<CanonicalQuery>> CanonicalQuery::make(CanonicalQueryParams&& params) {
+    try {
+        return std::make_unique<CanonicalQuery>(std::move(params));
+    } catch (const DBException& e) {
+        return e.toStatus();
+    }
+}
+
+CanonicalQuery::CanonicalQuery(CanonicalQueryParams&& params) {
+    auto parsedFind = uassertStatusOK(
+        visit(OverloadedVisitor{[](std::unique_ptr<ParsedFindCommand> parsedFindRequest) {
+                                    return StatusWith(std::move(parsedFindRequest));
+                                },
+                                [&](ParsedFindCommandParams p) {
+                                    return parsed_find_command::parse(params.expCtx, std::move(p));
+                                }},
+              std::move(params.parsedFind)));
+
+    initCq(std::move(params.expCtx),
+           std::move(parsedFind),
+           std::move(params.pipeline),
+           params.isCountLike,
+           params.isSearchQuery,
+           true /*optimizeMatchExpression*/);
+}
+
+CanonicalQuery::CanonicalQuery(OperationContext* opCtx, const CanonicalQuery& baseQuery, size_t i) {
     tassert(8401301,
             "expected MatchExpression with rooted $or",
-            baseQuery.root()->matchType() == MatchExpression::OR);
+            baseQuery.getPrimaryMatchExpression()->matchType() == MatchExpression::OR);
     tassert(8401302,
             "attempted to get out of bounds child of $or",
-            baseQuery.root()->numChildren() > i);
-    auto root = baseQuery.root()->getChild(i);
+            baseQuery.getPrimaryMatchExpression()->numChildren() > i);
+    auto matchExpr = baseQuery.getPrimaryMatchExpression()->getChild(i);
+
     auto findCommand = std::make_unique<FindCommandRequest>(baseQuery.nss());
-    findCommand->setFilter(root->serialize());
+    findCommand->setFilter(matchExpr->serialize());
     findCommand->setProjection(baseQuery.getFindCommandRequest().getProjection().getOwned());
     findCommand->setSort(baseQuery.getFindCommandRequest().getSort().getOwned());
     findCommand->setCollation(baseQuery.getFindCommandRequest().getCollation().getOwned());
 
-    // Make the CQ we'll hopefully return.
-    auto cq = std::make_unique<CanonicalQuery>();
-    cq->setExplain(baseQuery.getExplain());
-    auto swParsedFind = ParsedFindCommand::withExistingFilter(
+    auto parsedFind = uassertStatusOK(ParsedFindCommand::withExistingFilter(
         baseQuery.getExpCtx(),
         baseQuery.getCollator() ? baseQuery.getCollator()->clone() : nullptr,
-        root->clone(),
-        std::move(findCommand));
-    if (!swParsedFind.isOK()) {
-        return swParsedFind.getStatus();
-    }
+        matchExpr->clone(),
+        std::move(findCommand),
+        ProjectionPolicies::findProjectionPolicies()));
+
+    _forSubPlanner = true;
+
     // Note: we do not optimize the MatchExpression representing the branch of the top-level $or
     // that we are currently examining. This is because repeated invocations of
     // MatchExpression::optimize() may change the order of predicates in the MatchExpression, due to
     // new rewrites being unlocked by previous ones. We need to preserve the order of predicates to
     // allow index tagging to work properly. See SERVER-84013 for more details.
-    Status initStatus =
-        cq->init(baseQuery.getExpCtx(),
-                 std::move(swParsedFind.getValue()),
-                 {} /* an empty pipeline */,
-                 false,  // The parent query countLike is independent from the subquery countLike.
-                 false /*optimizeMatchExpression*/);
-
-    invariant(initStatus.isOK());
-    return {std::move(cq)};
+    initCq(baseQuery.getExpCtx(),
+           std::move(parsedFind),
+           {} /* an empty cqPipeline */,
+           false,  // The parent query countLike is independent from the subquery countLike.
+           baseQuery.isSearchQuery(),
+           false /*optimizeMatchExpression*/);
 }
 
-void CanonicalQuery::parameterize() {
-    // Perform auto-parameterization only if the query is SBE-compatible and caching is enabled.
-    if (_expCtx->sbeCompatibility != SbeCompatibility::notCompatible &&
-        !internalQueryDisablePlanCache.load()) {
-        const bool hasNoTextNodes =
-            !QueryPlannerCommon::hasNode(_root.get(), MatchExpression::TEXT);
-        if (hasNoTextNodes) {
-            // When the SBE plan cache is enabled, we auto-parameterize queries in the hopes of
-            // caching a parameterized plan. Here we add parameter markers to the appropriate match
-            // expression leaf nodes.
-            _inputParamIdToExpressionMap =
-                MatchExpression::parameterize(_root.get(), loadMaxParameterCount());
-        } else {
-            LOGV2_DEBUG(6579310,
-                        5,
-                        "The query was not auto-parameterized since its match expression tree "
-                        "contains TEXT nodes");
-        }
-    }
-}
-
-Status CanonicalQuery::init(boost::intrusive_ptr<ExpressionContext> expCtx,
+void CanonicalQuery::initCq(boost::intrusive_ptr<ExpressionContext> expCtx,
                             std::unique_ptr<ParsedFindCommand> parsedFind,
-                            std::vector<std::unique_ptr<InnerPipelineStageInterface>> pipeline,
+                            std::vector<boost::intrusive_ptr<DocumentSource>> cqPipeline,
                             bool isCountLike,
+                            bool isSearchQuery,
                             bool optimizeMatchExpression) {
     _expCtx = expCtx;
+
     _findCommand = std::move(parsedFind->findCommandRequest);
 
-    _forceClassicEngine =
-        QueryKnobConfiguration::decoration(expCtx->opCtx).getInternalQueryFrameworkControlForOp() ==
-        QueryFrameworkControlEnum::kForceClassicEngine;
-
     if (optimizeMatchExpression) {
-        _root = MatchExpression::normalize(std::move(parsedFind->filter));
+        const bool enableSimplification = !_expCtx->inLookup && !_expCtx->isUpsert;
+        _primaryMatchExpression =
+            MatchExpression::normalize(std::move(parsedFind->filter), enableSimplification);
     } else {
-        _root = std::move(parsedFind->filter);
+        _primaryMatchExpression = std::move(parsedFind->filter);
     }
 
     if (parsedFind->proj) {
@@ -224,7 +202,7 @@ Status CanonicalQuery::init(boost::intrusive_ptr<ExpressionContext> expCtx,
             // MatchExpression, before we save this projection for later execution.
             _proj.emplace(projection_ast::parseAndAnalyze(expCtx,
                                                           _findCommand->getProjection(),
-                                                          _root.get(),
+                                                          _primaryMatchExpression.get(),
                                                           _findCommand->getFilter(),
                                                           *parsedFind->savedProjectionPolicies,
                                                           shouldOptimizeProj));
@@ -234,29 +212,17 @@ Status CanonicalQuery::init(boost::intrusive_ptr<ExpressionContext> expCtx,
                 _proj->optimize();
             }
         }
+
+        _metadataDeps = _proj->metadataDeps();
+        uassert(ErrorCodes::BadValue,
+                "cannot use sortKey $meta projection without a sort",
+                !(_proj->metadataDeps()[DocumentMetadataFields::kSortKey] &&
+                  _findCommand->getSort().isEmpty()));
     }
+
     if (parsedFind->sort) {
         _sortPattern = std::move(parsedFind->sort);
-    }
-    _pipeline = std::move(pipeline);
-    _isCountLike = isCountLike;
 
-    // The tree must always be valid after normalization.
-    dassert(parsed_find_command::isValid(_root.get(), *_findCommand).isOK());
-    if (auto status = isValidNormalized(_root.get()); !status.isOK()) {
-        return status;
-    }
-
-    if (_proj) {
-        _metadataDeps = _proj->metadataDeps();
-
-        if (_proj->metadataDeps()[DocumentMetadataFields::kSortKey] &&
-            _findCommand->getSort().isEmpty()) {
-            return {ErrorCodes::BadValue, "cannot use sortKey $meta projection without a sort"};
-        }
-    }
-
-    if (_sortPattern) {
         // Be sure to track and add any metadata dependencies from the sort (e.g. text score).
         _metadataDeps |= _sortPattern->metadataDeps(parsedFind->unavailableMetadata);
 
@@ -266,12 +232,37 @@ Status CanonicalQuery::init(boost::intrusive_ptr<ExpressionContext> expCtx,
             _metadataDeps.set(DocumentMetadataFields::kSortKey);
         }
     }
+    setCqPipeline(std::move(cqPipeline), false /* containsEntirePipeline */);
+    _isCountLike = isCountLike;
+    _isSearchQuery = isSearchQuery;
+
+    // Perform SBE auto-parameterization if there is not already a reason not to.
+    _disablePlanCache = internalQueryDisablePlanCache.load();
+    _maxMatchExpressionParams = loadMaxMatchExpressionParams();
+    if (expCtx->sbeCompatibility != SbeCompatibility::notCompatible &&
+        shouldParameterizeSbe(_primaryMatchExpression.get())) {
+        // When the SBE plan cache is enabled, we auto-parameterize queries in the hopes of caching
+        // a parameterized plan. Here we add parameter markers to the appropriate match expression
+        // leaf nodes unless it has too many predicates. If it did not actually get parameterized,
+        // we mark the query as uncacheable for SBE to avoid plan cache flooding.
+        bool parameterized;
+        _inputParamIdToExpressionMap = MatchExpression::parameterize(
+            _primaryMatchExpression.get(), _maxMatchExpressionParams, 0, &parameterized);
+        if (!parameterized) {
+            // Avoid plan cache flooding by not fully parameterized plans.
+            setUncacheableSbe();
+        }
+    }
+    // The tree must always be valid after normalization.
+    dassert(parsed_find_command::isValid(_primaryMatchExpression.get(), *_findCommand).isOK());
+    if (auto status = isValidNormalized(_primaryMatchExpression.get()); !status.isOK()) {
+        uasserted(status.code(), status.reason());
+    }
 
     // If the 'returnKey' option is set, then the plan should produce index key metadata.
     if (_findCommand->getReturnKey()) {
         _metadataDeps.set(DocumentMetadataFields::kIndexKey);
     }
-    return Status::OK();
 }
 
 void CanonicalQuery::setCollator(std::unique_ptr<CollatorInterface> collator) {
@@ -281,7 +272,28 @@ void CanonicalQuery::setCollator(std::unique_ptr<CollatorInterface> collator) {
 
     // The collator associated with the match expression tree is now invalid, since we have reset
     // the collator owned by the ExpressionContext.
-    _root->setCollator(collatorRaw);
+    _primaryMatchExpression->setCollator(collatorRaw);
+}
+
+void CanonicalQuery::serializeToBson(BSONObjBuilder* out) const {
+    // Display the filter.
+    auto filter = getPrimaryMatchExpression();
+    if (filter) {
+        out->append("filter", filter->serialize());
+    }
+
+    // Display the projection, if present.
+    auto proj = getProj();
+    if (proj) {
+        out->append("projection", projection_ast::serialize(*proj->root(), {}));
+    }
+
+    // Display the sort, if present.
+    auto sort = getSortPattern();
+    if (sort && !sort->empty()) {
+        out->append("sort",
+                    sort->serialize(SortPattern::SortKeySerialization::kForExplain).toBson());
+    }
 }
 
 // static
@@ -339,9 +351,13 @@ Status CanonicalQuery::isValidNormalized(const MatchExpression* root) {
     return Status::OK();
 }
 
-std::string CanonicalQuery::toString() const {
+std::string CanonicalQuery::toString(bool forErrMsg) const {
     str::stream ss;
-    ss << "ns=" << _findCommand->getNamespaceOrUUID().toString();
+    if (forErrMsg) {
+        ss << "ns=" << _findCommand->getNamespaceOrUUID().toStringForErrorMsg();
+    } else {
+        ss << "ns=" << toStringForLogging(_findCommand->getNamespaceOrUUID());
+    }
 
     if (_findCommand->getBatchSize()) {
         ss << " batchSize=" << *_findCommand->getBatchSize();
@@ -356,7 +372,7 @@ std::string CanonicalQuery::toString() const {
     }
 
     // The expression tree puts an endl on for us.
-    ss << "Tree: " << _root->debugString();
+    ss << "Tree: " << _primaryMatchExpression->debugString();
     ss << "Sort: " << _findCommand->getSort().toString() << '\n';
     ss << "Proj: " << _findCommand->getProjection().toString() << '\n';
     if (!_findCommand->getCollation().isEmpty()) {
@@ -365,10 +381,15 @@ std::string CanonicalQuery::toString() const {
     return ss;
 }
 
-std::string CanonicalQuery::toStringShort() const {
+std::string CanonicalQuery::toStringShort(bool forErrMsg) const {
     str::stream ss;
-    ss << "ns: " << _findCommand->getNamespaceOrUUID().toString()
-       << " query: " << _findCommand->getFilter().toString()
+    if (forErrMsg) {
+        ss << "ns: " << _findCommand->getNamespaceOrUUID().toStringForErrorMsg();
+    } else {
+        ss << "ns: " << toStringForLogging(_findCommand->getNamespaceOrUUID());
+    }
+
+    ss << " query: " << _findCommand->getFilter().toString()
        << " sort: " << _findCommand->getSort().toString()
        << " projection: " << _findCommand->getProjection().toString();
 
@@ -392,11 +413,52 @@ std::string CanonicalQuery::toStringShort() const {
 }
 
 CanonicalQuery::QueryShapeString CanonicalQuery::encodeKey() const {
-    return (!_forceClassicEngine && _sbeCompatible) ? canonical_query_encoder::encodeSBE(*this)
-                                                    : canonical_query_encoder::encodeClassic(*this);
+    return (!getExpCtx()->getQueryKnobConfiguration().isForceClassicEngineEnabled() &&
+            _sbeCompatible)
+        ? canonical_query_encoder::encodeSBE(*this,
+                                             canonical_query_encoder::Optimizer::kSbeStageBuilders)
+        : canonical_query_encoder::encodeClassic(*this);
 }
 
 CanonicalQuery::QueryShapeString CanonicalQuery::encodeKeyForPlanCacheCommand() const {
     return canonical_query_encoder::encodeForPlanCacheCommand(*this);
 }
+
+void CanonicalQuery::setCqPipeline(std::vector<boost::intrusive_ptr<DocumentSource>> cqPipeline,
+                                   bool containsEntirePipeline) {
+    _cqPipeline = std::move(cqPipeline);
+    _containsEntirePipeline = containsEntirePipeline;
+
+    // Find $match stages that weren't pushed down to find, but will be pushed to SBE. These
+    // need to be parameterized separately from the find layer filters.
+    for (auto& docSource : _cqPipeline) {
+        auto matchStage = dynamic_cast<DocumentSourceMatch*>(docSource.get());
+        if (matchStage) {
+            MatchExpression* matchExpr = matchStage->getMatchExpression();
+            if (shouldParameterizeSbe(matchExpr)) {
+                bool parameterized;
+                std::vector<const MatchExpression*> newParams = MatchExpression::parameterize(
+                    matchExpr, getMaxMatchExpressionParams(), numParams(), &parameterized);
+                if (parameterized) {
+                    addMatchParams(newParams);
+                } else {
+                    // Avoid plan cache flooding by not fully parameterized plans.
+                    setUncacheableSbe();
+                }
+            }
+        }
+    }
+}
+
+bool CanonicalQuery::shouldParameterizeSbe(MatchExpression* matchExpr) const {
+    if (_disablePlanCache || _isUncacheableSbe ||
+        QueryPlannerCommon::hasNode(matchExpr, MatchExpression::TEXT)) {
+        return false;
+    }
+    return true;
+}
+
+bool CanonicalQuery::shouldParameterizeLimitSkip() const {
+    return !_disablePlanCache && !_isUncacheableSbe;
+}
 }  // namespace mongo
