@@ -30,25 +30,77 @@
 
 #include "mongo/db/query/canonical_query_encoder.h"
 
+#include <algorithm>
 #include <boost/iterator/transform_iterator.hpp>
+#include <cstddef>
+#include <iterator>
+#include <memory>
+#include <set>
+#include <string>
+#include <type_traits>
+#include <vector>
 
-#include "mongo/base/simple_string_data_comparator.h"
+#include <boost/iterator/iterator_facade.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <s2cellid.h>
+
+#include "mongo/base/string_data_comparator.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/bson/util/builder.h"
+#include "mongo/bson/util/builder_fwd.h"
+#include "mongo/db/api_parameters.h"
+#include "mongo/db/basic_types.h"
+#include "mongo/db/basic_types_gen.h"
+#include "mongo/db/exec/document_value/document.h"
+#include "mongo/db/exec/document_value/value.h"
+#include "mongo/db/geo/geometry_container.h"
+#include "mongo/db/geo/shapes.h"
 #include "mongo/db/matcher/expression_array.h"
 #include "mongo/db/matcher/expression_expr.h"
 #include "mongo/db/matcher/expression_geo.h"
+#include "mongo/db/matcher/expression_internal_bucket_geo_within.h"
+#include "mongo/db/matcher/expression_leaf.h"
+#include "mongo/db/matcher/expression_path.h"
 #include "mongo/db/matcher/expression_text.h"
 #include "mongo/db/matcher/expression_text_noop.h"
+#include "mongo/db/matcher/expression_tree.h"
+#include "mongo/db/matcher/expression_type.h"
+#include "mongo/db/matcher/expression_visitor.h"
 #include "mongo/db/matcher/expression_where.h"
 #include "mongo/db/matcher/expression_where_noop.h"
+#include "mongo/db/matcher/schema/expression_internal_schema_allowed_properties.h"
+#include "mongo/db/pipeline/document_source.h"
 #include "mongo/db/pipeline/document_source_group.h"
+#include "mongo/db/pipeline/document_source_internal_projection.h"
 #include "mongo/db/pipeline/document_source_lookup.h"
+#include "mongo/db/pipeline/document_source_project.h"
+#include "mongo/db/pipeline/document_source_set_window_fields.h"
+#include "mongo/db/pipeline/document_source_single_document_transformation.h"
+#include "mongo/db/pipeline/expression_context.h"
+#include "mongo/db/pipeline/search/search_helper.h"
 #include "mongo/db/query/analyze_regex.h"
+#include "mongo/db/query/collation/collator_interface.h"
+#include "mongo/db/query/find_command.h"
 #include "mongo/db/query/projection.h"
-#include "mongo/db/query/query_decorations.h"
+#include "mongo/db/query/projection_ast.h"
+#include "mongo/db/query/projection_ast_util.h"
+#include "mongo/db/query/query_knob_configuration.h"
+#include "mongo/db/query/query_request_helper.h"
 #include "mongo/db/query/tree_walker.h"
+#include "mongo/db/repl/read_concern_args.h"
+#include "mongo/db/repl/read_concern_level.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
 #include "mongo/platform/overflow_arithmetic.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/base64.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/str.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
@@ -307,11 +359,11 @@ const char* encodeMatchType(MatchExpression::MatchType mt) {
  * - geometry type
  * - CRS (flat or spherical)
  */
-void encodeGeoMatchExpression(const GeoMatchExpression* tree, StringBuilder* keyBuilder) {
-    const GeoExpression& geoQuery = tree->getGeoExpression();
-
+void encodeGeoMatchExpression(const GeometryContainer& geo,
+                              GeoExpression::Predicate pred,
+                              StringBuilder* keyBuilder) {
     // Type of geo query.
-    switch (geoQuery.getPred()) {
+    switch (pred) {
         case GeoExpression::WITHIN:
             *keyBuilder << "wi";
             break;
@@ -325,20 +377,20 @@ void encodeGeoMatchExpression(const GeoMatchExpression* tree, StringBuilder* key
 
     // Geometry type.
     // Only one of the shared_ptrs in GeoContainer may be non-NULL.
-    *keyBuilder << geoQuery.getGeometry().getDebugType();
+    *keyBuilder << geo.getDebugType();
 
     // CRS (flat or spherical)
-    if (FLAT == geoQuery.getGeometry().getNativeCRS()) {
+    if (FLAT == geo.getNativeCRS()) {
         *keyBuilder << "fl";
-    } else if (SPHERE == geoQuery.getGeometry().getNativeCRS()) {
+    } else if (SPHERE == geo.getNativeCRS()) {
         *keyBuilder << "sp";
-    } else if (STRICT_SPHERE == geoQuery.getGeometry().getNativeCRS()) {
+    } else if (STRICT_SPHERE == geo.getNativeCRS()) {
         *keyBuilder << "ss";
     } else {
         LOGV2_ERROR(23849,
                     "Unknown CRS type in geometry",
-                    "crsType"_attr = (int)geoQuery.getGeometry().getNativeCRS(),
-                    "geometryType"_attr = geoQuery.getGeometry().getDebugType());
+                    "crsType"_attr = (int)geo.getNativeCRS(),
+                    "geometryType"_attr = geo.getDebugType());
         MONGO_UNREACHABLE;
     }
 }
@@ -407,30 +459,6 @@ void encodeCollation(const CollatorInterface* collation, StringBuilder* keyBuild
     // not be stable between versions.
 }
 
-void encodePipeline(const std::vector<std::unique_ptr<InnerPipelineStageInterface>>& pipeline,
-                    BufBuilder* bufBuilder) {
-    bufBuilder->appendChar(kEncodeSectionDelimiter);
-    for (auto& stage : pipeline) {
-        std::vector<Value> serializedArray;
-        if (auto lookupStage = dynamic_cast<DocumentSourceLookUp*>(stage->documentSource())) {
-            lookupStage->serializeToArray(serializedArray);
-            tassert(6443201,
-                    "$lookup stage isn't serialized to a single bson object",
-                    serializedArray.size() == 1 && serializedArray[0].getType() == Object);
-            const auto bson = serializedArray[0].getDocument().toBson();
-            bufBuilder->appendBuf(bson.objdata(), bson.objsize());
-        } else if (auto groupStage = dynamic_cast<DocumentSourceGroup*>(stage->documentSource())) {
-            auto serializedGroup = groupStage->serialize();
-            const auto bson = serializedGroup.getDocument().toBson();
-            bufBuilder->appendBuf(bson.objdata(), bson.objsize());
-        } else {
-            tasserted(6443200,
-                      str::stream() << "Pipeline stage cannot be encoded in plan cache key: "
-                                    << stage->documentSource()->getSourceName());
-        }
-    }
-}
-
 template <class RegexIterator>
 void encodeRegexFlagsForMatch(RegexIterator first, RegexIterator last, StringBuilder* keyBuilder) {
     // We sort the flags, so that queries with the same regex flags in different orders will have
@@ -485,23 +513,90 @@ void encodeKeyForMatch(const MatchExpression* tree, StringBuilder* keyBuilder) {
 
     encodeUserString(tree->path(), keyBuilder);
 
-    // GEO and GEO_NEAR require additional encoding.
-    if (MatchExpression::GEO == tree->matchType()) {
-        encodeGeoMatchExpression(static_cast<const GeoMatchExpression*>(tree), keyBuilder);
-    } else if (MatchExpression::GEO_NEAR == tree->matchType()) {
-        encodeGeoNearMatchExpression(static_cast<const GeoNearMatchExpression*>(tree), keyBuilder);
-    }
-
-    // We encode regular expression flags such that different options produce different shapes.
-    if (MatchExpression::REGEX == tree->matchType()) {
-        encodeRegexFlagsForMatch({static_cast<const RegexMatchExpression*>(tree)}, keyBuilder);
-    } else if (MatchExpression::MATCH_IN == tree->matchType()) {
-        const auto* inMatch = static_cast<const InMatchExpression*>(tree);
-        if (!inMatch->getRegexes().empty()) {
-            // Append '_re' to distinguish an $in without regexes from an $in with regexes.
-            encodeUserString("_re"_sd, keyBuilder);
-            encodeRegexFlagsForMatch(inMatch->getRegexes(), keyBuilder);
+    switch (tree->matchType()) {
+        // Geo operators require additional encoding.
+        case MatchExpression::GEO: {
+            auto geoTree = static_cast<const GeoMatchExpression*>(tree);
+            encodeGeoMatchExpression(geoTree->getGeoExpression().getGeometry(),
+                                     geoTree->getGeoExpression().getPred(),
+                                     keyBuilder);
+            break;
         }
+        case MatchExpression::GEO_NEAR:
+            encodeGeoNearMatchExpression(static_cast<const GeoNearMatchExpression*>(tree),
+                                         keyBuilder);
+            break;
+        case MatchExpression::INTERNAL_BUCKET_GEO_WITHIN: {
+            auto geoTree = static_cast<const InternalBucketGeoWithinMatchExpression*>(tree);
+            encodeGeoMatchExpression(geoTree->getGeoContainer(), GeoExpression::WITHIN, keyBuilder);
+            break;
+        }
+        case MatchExpression::REGEX:
+            // We encode regular expression flags such that different options produce different
+            // shapes.
+            encodeRegexFlagsForMatch({static_cast<const RegexMatchExpression*>(tree)}, keyBuilder);
+            break;
+        case MatchExpression::MATCH_IN: {
+            const auto* inMatch = static_cast<const InMatchExpression*>(tree);
+            if (!inMatch->getRegexes().empty()) {
+                // Append '_re' to distinguish an $in without regexes from an $in with regexes.
+                encodeUserString("_re"_sd, keyBuilder);
+                encodeRegexFlagsForMatch(inMatch->getRegexes(), keyBuilder);
+            }
+            break;
+        }
+        case MatchExpression::AND:
+        case MatchExpression::OR:
+        case MatchExpression::ELEM_MATCH_OBJECT:
+        case MatchExpression::ELEM_MATCH_VALUE:
+        case MatchExpression::SIZE:
+        case MatchExpression::EQ:
+        case MatchExpression::LTE:
+        case MatchExpression::LT:
+        case MatchExpression::GT:
+        case MatchExpression::GTE:
+        case MatchExpression::MOD:
+        case MatchExpression::EXISTS:
+        case MatchExpression::BITS_ALL_SET:
+        case MatchExpression::BITS_ALL_CLEAR:
+        case MatchExpression::BITS_ANY_SET:
+        case MatchExpression::BITS_ANY_CLEAR:
+        case MatchExpression::NOT:
+        case MatchExpression::NOR:
+        case MatchExpression::TYPE_OPERATOR:
+        case MatchExpression::WHERE:
+        case MatchExpression::EXPRESSION:
+        case MatchExpression::ALWAYS_FALSE:
+        case MatchExpression::ALWAYS_TRUE:
+        case MatchExpression::TEXT:
+        case MatchExpression::INTERNAL_2D_POINT_IN_ANNULUS:
+        case MatchExpression::INTERNAL_EXPR_EQ:
+        case MatchExpression::INTERNAL_EXPR_GT:
+        case MatchExpression::INTERNAL_EXPR_GTE:
+        case MatchExpression::INTERNAL_EXPR_LT:
+        case MatchExpression::INTERNAL_EXPR_LTE:
+        case MatchExpression::INTERNAL_EQ_HASHED_KEY:
+        case MatchExpression::INTERNAL_SCHEMA_ALLOWED_PROPERTIES:
+        case MatchExpression::INTERNAL_SCHEMA_ALL_ELEM_MATCH_FROM_INDEX:
+        case MatchExpression::INTERNAL_SCHEMA_BIN_DATA_ENCRYPTED_TYPE:
+        case MatchExpression::INTERNAL_SCHEMA_BIN_DATA_FLE2_ENCRYPTED_TYPE:
+        case MatchExpression::INTERNAL_SCHEMA_BIN_DATA_SUBTYPE:
+        case MatchExpression::INTERNAL_SCHEMA_COND:
+        case MatchExpression::INTERNAL_SCHEMA_EQ:
+        case MatchExpression::INTERNAL_SCHEMA_FMOD:
+        case MatchExpression::INTERNAL_SCHEMA_MATCH_ARRAY_INDEX:
+        case MatchExpression::INTERNAL_SCHEMA_MAX_ITEMS:
+        case MatchExpression::INTERNAL_SCHEMA_MAX_LENGTH:
+        case MatchExpression::INTERNAL_SCHEMA_MAX_PROPERTIES:
+        case MatchExpression::INTERNAL_SCHEMA_MIN_ITEMS:
+        case MatchExpression::INTERNAL_SCHEMA_MIN_LENGTH:
+        case MatchExpression::INTERNAL_SCHEMA_MIN_PROPERTIES:
+        case MatchExpression::INTERNAL_SCHEMA_OBJECT_MATCH:
+        case MatchExpression::INTERNAL_SCHEMA_ROOT_DOC_EQ:
+        case MatchExpression::INTERNAL_SCHEMA_TYPE:
+        case MatchExpression::INTERNAL_SCHEMA_UNIQUE_ITEMS:
+        case MatchExpression::INTERNAL_SCHEMA_XOR:
+            break;
     }
 
     // If the query predicate is minKey or maxKey it can't use the same plan as other GT/LT
@@ -588,7 +683,7 @@ void encodeKeyForProj(const projection_ast::Projection* proj, StringBuilder* key
         return;
     }
 
-    auto requiredFields = proj->getRequiredFields();
+    const auto& requiredFields = proj->getRequiredFields();
 
     // If the only requirement is that $sortKey be included with some value, we just act as if the
     // entire document is needed.
@@ -615,12 +710,27 @@ void encodeKeyForProj(const projection_ast::Projection* proj, StringBuilder* key
     }
 }
 
+void encodeKeyForPipelineStage(DocumentSource* docSource,
+                               std::vector<Value>& serializedArray,
+                               BufBuilder* bufBuilder) {
+    bufBuilder->appendChar(kEncodeSectionDelimiter);
+    serializedArray.clear();
+    docSource->serializeToArray(serializedArray);
+
+    for (const auto& value : serializedArray) {
+        tassert(
+            6443201, "Expected pipeline stage to serialize to objects", value.getType() == Object);
+        const BSONObj bson = value.getDocument().toBson();
+        bufBuilder->appendBuf(bson.objdata(), bson.objsize());
+    }
+}
+
 /**
  * Approximate the number of documents to be processed into a small, medium or large category. Best
  * plans for limit: 10 and limit: 1000 may be different. This allows us to cache different plans for
  * different cases without unbounded growth of plan cache for each skip and limit value.
  */
-char getLimitSkipCategory(OperationContext* opCtx,
+char getLimitSkipCategory(const boost::intrusive_ptr<ExpressionContext>& expCtx,
                           boost::optional<int64_t> skip,
                           boost::optional<int64_t> limit) {
     if (limit.value_or(0) == 1 && !skip) {
@@ -635,7 +745,7 @@ char getLimitSkipCategory(OperationContext* opCtx,
         return 'l';
     }
     size_t planEvaluationMaxResults =
-        QueryKnobConfiguration::decoration(opCtx).getPlanEvaluationMaxResultsForOp();
+        expCtx->getQueryKnobConfiguration().getPlanEvaluationMaxResultsForOp();
     if (limitSkipSum < planEvaluationMaxResults) {
         return 's';
     } else if (limitSkipSum < 10 * planEvaluationMaxResults) {
@@ -651,9 +761,16 @@ void encodeLimitSkip(const CanonicalQuery& cq, BufBuilder* bufBuilder) {
     if (!limit && !skip) {
         return;
     }
-    bufBuilder->appendChar(skip ? 1 : 0);
-    bufBuilder->appendChar(limit ? 1 : 0);
-    bufBuilder->appendChar(getLimitSkipCategory(cq.getOpCtx(), skip, limit));
+    if (cq.shouldParameterizeLimitSkip()) {
+        bufBuilder->appendChar('p');
+        bufBuilder->appendChar(skip ? 1 : 0);
+        bufBuilder->appendChar(limit ? 1 : 0);
+        bufBuilder->appendChar(getLimitSkipCategory(cq.getExpCtx(), skip, limit));
+    } else {
+        bufBuilder->appendChar('c');
+        bufBuilder->appendNum(skip.value_or(0));
+        bufBuilder->appendNum(limit.value_or(0));
+    }
 }
 
 void encodeFindCommandRequest(const CanonicalQuery& cq, BufBuilder* bufBuilder) {
@@ -682,43 +799,17 @@ void encodeFindCommandRequest(const CanonicalQuery& cq, BufBuilder* bufBuilder)
     };
     encodeBSONObj(findCommand.getResumeAfter());
 
-    encodeBSONObj(findCommand.getStartAt());
-
     // Read concern "available" results in SBE plans that do not perform shard filtering, so it must
     // be encoded differently from other read concerns.
     bool isAvailableReadConcern{false};
     if (const auto readConcern = findCommand.getReadConcern()) {
         isAvailableReadConcern =
             readConcern->getField(repl::ReadConcernArgs::kLevelFieldName).valueStringDataSafe() ==
-            repl::readConcernLevels::kAvailableName;
+            repl::readConcernLevels::toString(repl::ReadConcernLevel::kAvailableReadConcern);
     }
     bufBuilder->appendChar(isAvailableReadConcern ? 't' : 'f');
 }
-}  // namespace
 
-namespace canonical_query_encoder {
-
-CanonicalQuery::QueryShapeString encodeClassic(const CanonicalQuery& cq) {
-    StringBuilder keyBuilder;
-    encodeKeyForMatch(cq.root(), &keyBuilder);
-    encodeKeyForSort(cq.getFindCommandRequest().getSort(), &keyBuilder);
-    encodeKeyForProj(cq.getProj(), &keyBuilder);
-    encodeCollation(cq.getCollator(), &keyBuilder);
-
-    // This encoding can be removed once the classic query engine reaches EOL and SBE is used
-    // exclusively for all query execution.
-    keyBuilder << kEncodeSectionDelimiter << (cq.getForceClassicEngine() ? "f" : "t");
-
-    // The apiStrict flag can cause the query to see different set of indexes. For example, all
-    // sparse indexes will be ignored with apiStrict is used.
-    const bool apiStrict =
-        cq.getOpCtx() && APIParameters::get(cq.getOpCtx()).getAPIStrict().value_or(false);
-    keyBuilder << (apiStrict ? "t" : "f");
-
-    return keyBuilder.str();
-}
-
-namespace {
 /**
  * A visitor intended for use in combination with the corresponding walker class below to encode a
  * 'MatchExpression' into the SBE plan cache key.
@@ -729,10 +820,15 @@ namespace {
 class MatchExpressionSbePlanCacheKeySerializationVisitor final
     : public MatchExpressionConstVisitor {
 public:
-    explicit MatchExpressionSbePlanCacheKeySerializationVisitor(OperationContext* opCtx,
-                                                                BufBuilder* builder,
-                                                                bool hasSort)
-        : _opCtx(opCtx), _builder(builder), _hasSort(hasSort) {}
+    explicit MatchExpressionSbePlanCacheKeySerializationVisitor(
+        const boost::intrusive_ptr<ExpressionContext>& expCtx,
+        BufBuilder* builder,
+        bool encodeParameterTypes,
+        bool hasSort)
+        : _expCtx(expCtx),
+          _builder(builder),
+          _encodeParameterTypes(encodeParameterTypes),
+          _hasSort(hasSort) {}
 
     void visit(const BitsAllClearMatchExpression* expr) final {
         encodeBitTestExpression(expr);
@@ -773,13 +869,14 @@ public:
 
     void visit(const InMatchExpression* expr) final {
         encodeSingleParamPathNode(expr);
+
         // Encode the number of unique $in values as part of the plan cache key. If the query is
         // optimized by exploding for sort, the number of unique elements in $in determines how many
         // merge branches we get in the query plan. If there is no sort, this is not necessary.
         if (expr->getInputParamId() && _hasSort) {
             size_t maxScansToExplode =
-                QueryKnobConfiguration::decoration(_opCtx).getMaxScansToExplodeForOp();
-            // Assume that $in have n elements.
+                _expCtx->getQueryKnobConfiguration().getMaxScansToExplodeForOp();
+            // Assume that $in has n elements.
             // If n is less than or equal to maxScansToExplode, then it is possible that explode for
             // sort optimization will be used, so we need to add n to plan cache key.
             // If n is greater than maxScansToExplode, then we can't explode it for sort. So we can
@@ -973,6 +1070,20 @@ private:
     void encodeSingleParamPathNode(const T* expr) {
         if (expr->getInputParamId()) {
             encodeParamMarker(*expr->getInputParamId());
+            if (_encodeParameterTypes) {
+                if constexpr (std::is_base_of_v<ComparisonMatchExpressionBase, T>) {
+                    _builder->appendNum(canonicalizeBSONType(expr->getData().type()));
+                } else if constexpr (std::is_same_v<InMatchExpression, T>) {
+                    // Mimic ABT translation logic which treats $in with a single element as an Eq.
+                    // In this case, we need to encode the type of the operand.
+                    int type = canonicalizeBSONType(BSONType::Array);
+                    auto inExpr = static_cast<const InMatchExpression*>(expr);
+                    if (inExpr->getEqualities().size() == 1) {
+                        type = canonicalizeBSONType(inExpr->getEqualities().front().type());
+                    }
+                    _builder->appendNum(type);
+                }
+            }
         } else {
             encodeRhs(expr);
         }
@@ -1028,7 +1139,11 @@ private:
      * parameter marker.
      */
     void encodeRhs(const PathMatchExpression* expr) {
-        encodeHelper(expr->getSerializedRightHandSide());
+        // Call getSerializedRightHandSide() with 'inMatchExprSortAndDedupElements' set to false.
+        SerializationOptions opts;
+        opts.inMatchExprSortAndDedupElements = false;
+
+        encodeHelper(expr->getSerializedRightHandSide(std::move(opts)));
     }
 
     /**
@@ -1061,8 +1176,10 @@ private:
         _builder->appendBuf(elem.value(), elem.valuesize());
     }
 
-    OperationContext* const _opCtx;
+    const boost::intrusive_ptr<ExpressionContext>& _expCtx;
     BufBuilder* const _builder;
+    // Whether to encode the type of query parameter into the cache key.
+    bool _encodeParameterTypes{false};
     // Whether there is a sort absorbed by the Canonical query. Note: '_hasSort' is true only when
     // there is a sort that can be used for explode for sort optimization. For a $match stage,
     // '_hasSort' should be false since $match does not perform index selection. In cases where a
@@ -1080,10 +1197,12 @@ private:
  */
 class MatchExpressionSbePlanCacheKeySerializationWalker {
 public:
-    explicit MatchExpressionSbePlanCacheKeySerializationWalker(OperationContext* opCtx,
-                                                               BufBuilder* builder,
-                                                               bool hasSort)
-        : _builder{builder}, _visitor{opCtx, _builder, hasSort} {
+    explicit MatchExpressionSbePlanCacheKeySerializationWalker(
+        const boost::intrusive_ptr<ExpressionContext>& expCtx,
+        BufBuilder* builder,
+        bool encodeParameterTypes,
+        bool hasSort)
+        : _builder{builder}, _visitor{expCtx, _builder, encodeParameterTypes, hasSort} {
         invariant(_builder);
     }
 
@@ -1120,24 +1239,127 @@ private:
  * following property: Two match expression trees which are identical after auto-parameterization
  * have the same key, otherwise the keys must differ.
  */
-void encodeKeyForAutoParameterizedMatchSBE(OperationContext* opCtx,
+void encodeKeyForAutoParameterizedMatchSBE(const boost::intrusive_ptr<ExpressionContext>& expCtx,
                                            MatchExpression* matchExpr,
                                            BufBuilder* builder,
+                                           bool encodeParameterTypes,
                                            bool hasSort) {
-    MatchExpressionSbePlanCacheKeySerializationWalker walker{opCtx, builder, hasSort};
+    MatchExpressionSbePlanCacheKeySerializationWalker walker{
+        expCtx, builder, encodeParameterTypes, hasSort};
     tree_walker::walk<true, MatchExpression>(matchExpr, &walker);
 }
+
 }  // namespace
 
-std::string encodeSBE(const CanonicalQuery& cq) {
-    tassert(6142104,
-            "attempting to encode SBE plan cache key for SBE-incompatible query",
-            cq.isSbeCompatible());
+namespace canonical_query_encoder {
+
+/**
+ * Encode the stages pushed down to SBE via CanonicalQuery::cqPipeline.
+ * Also encodes pipelines that are eligible for the Bonsai plan cache.
+ */
+void encodePipeline(const boost::intrusive_ptr<ExpressionContext>& expCtx,
+                    const std::vector<boost::intrusive_ptr<DocumentSource>>& cqPipeline,
+                    BufBuilder* bufBuilder,
+                    const Optimizer optimizer) {
+    bufBuilder->appendChar(kEncodeSectionDelimiter);
+    std::vector<Value> serializedArray;
+    for (auto& stage : cqPipeline) {
+        auto documentSource = stage.get();
+        if (auto matchStage = dynamic_cast<DocumentSourceMatch*>(documentSource)) {
+            // Only encode parameter types in the MatchExpression if this key is being generated by
+            // Bonsai.
+            const bool encodeParameterTypes = optimizer == Optimizer::kBonsai;
+            // Match expressions are parameterized so need to be encoded differently.
+            encodeKeyForAutoParameterizedMatchSBE(
+                expCtx,
+                matchStage->getMatchExpression(),
+                bufBuilder,
+                encodeParameterTypes,
+                // We do not use explode for sort optimization for a $match stage, since it is not
+                // a part of index selection.
+                false /*hasSort*/);
+        } else if (!search_helpers::encodeSearchForSbeCache(expCtx, documentSource, bufBuilder)) {
+            encodeKeyForPipelineStage(documentSource, serializedArray, bufBuilder);
+        }
+    }  // for each stage in 'cqPipeline'
+    bufBuilder->appendChar(encodeEnum(optimizer));
+}
+
+CanonicalQuery::QueryShapeString encodePipeline(
+    const boost::intrusive_ptr<ExpressionContext>& expCtx,
+    const std::vector<boost::intrusive_ptr<DocumentSource>>& pipelineStages,
+    const Optimizer optimizer) {
+    static constexpr size_t bufferSize = 200;
+    BufBuilder bufBuilder(bufferSize);
+    canonical_query_encoder::encodePipeline(expCtx, pipelineStages, &bufBuilder, optimizer);
+    return base64::encode(StringData(bufBuilder.buf(), bufBuilder.len()));
+}
+
+CanonicalQuery::QueryShapeString encodeClassic(const CanonicalQuery& cq) {
+    StringBuilder keyBuilder;
+    encodeKeyForMatch(cq.getPrimaryMatchExpression(), &keyBuilder);
+    encodeKeyForSort(cq.getFindCommandRequest().getSort(), &keyBuilder);
+    encodeKeyForProj(cq.getProj(), &keyBuilder);
+    encodeCollation(cq.getCollator(), &keyBuilder);
+
+    // The apiStrict flag can cause the query to see different set of indexes. For example, all
+    // sparse indexes will be ignored with apiStrict is used.
+    const bool apiStrict =
+        cq.getOpCtx() && APIParameters::get(cq.getOpCtx()).getAPIStrict().value_or(false);
+    keyBuilder << (apiStrict ? "t" : "f");
+
+
+    // Encode a flag with three possible values:
+    // 1 ('c'): The cache entry is intended to use the classic code path completely. In this case
+    //            the entry stores 'works.'
+    // 2 ('s'): The cache entry was planned with the classic runtime planners, and we intend
+    //            to run it in SBE. In this case the entry stores 'reads.'
+    // 3 ('o'): The cache entry is used for sub-planning using the classic SubPlanner
+    //            and we intend to run it in SBE. In this case the entry stores 'works' but
+    //            since replanning doesn't happen for queries generated from subplanning,
+    //            the value has no meaning. TODO SERVER-90957: Update these entries to store a
+    //            placeholder instead.
+    //
+    // The third case is treated specially because the classic sub-planner stores and reads
+    // cache entries that cannot be executed with SBE. We use this extra flag to avoid the
+    // possibility of the sub planner writing a cache entry for a branch of $or which is then used
+    // by an independent query that doesn't subplan.
+    //
+    // For example, if the user runs an SBE-compatible query {$or: [{a:1,b:1}, {c:1,d:1}]} and a
+    // cache entry is written for each branch, we do NOT want thos cache entries to be re-used for
+    // a query {a:1,b:1} since the second query would expect the cache entry to store a "reads"
+    // value that can be used with SBE's cache recovery path.
+    //
+    // By incorporating 'forSubplanner' we eliminate this possibility.
+
+    if (cq.isSbeCompatible()) {
+        if (cq.forSubPlanner()) {
+            keyBuilder << "o";  // Case 3: 'o' for "OR."
+        } else {
+            keyBuilder << "s";  // Case 2: 's' for "SBE."
+        }
+    } else {
+        keyBuilder << "c";  // Case 1: 'c' for "classic."
+    }
+
+    return keyBuilder.str();
+}
+
+std::string encodeSBE(const CanonicalQuery& cq, const Optimizer optimizer) {
+    if (optimizer == Optimizer::kSbeStageBuilders) {
+        tassert(6142104,
+                "attempting to encode SBE plan cache key for SBE-incompatible query",
+                cq.isSbeCompatible());
+    }
 
     const auto& filter = cq.getQueryObj();
     const auto& proj = cq.getFindCommandRequest().getProjection();
     const auto& sort = cq.getFindCommandRequest().getSort();
-    const auto& hint = cq.getFindCommandRequest().getHint();
+
+    // Do not encode query's hint if query settings already has index hints.
+    const auto& hint = cq.getExpCtx()->getQuerySettings().getIndexHints()
+        ? BSONObj()
+        : cq.getFindCommandRequest().getHint();
 
     StringBuilder strBuilder;
     encodeKeyForSort(sort, &strBuilder);
@@ -1151,7 +1373,14 @@ std::string encodeSBE(const CanonicalQuery& cq) {
         kBufferSizeConstant;
 
     BufBuilder bufBuilder(bufSize);
-    encodeKeyForAutoParameterizedMatchSBE(cq.getOpCtx(), cq.root(), &bufBuilder, !sort.isEmpty());
+    // Only encode parameter types in the MatchExpression if this key is being generated by
+    // Bonsai.
+    const bool encodeParameterTypes = optimizer == Optimizer::kBonsai;
+    encodeKeyForAutoParameterizedMatchSBE(cq.getExpCtx(),
+                                          cq.getPrimaryMatchExpression(),
+                                          &bufBuilder,
+                                          encodeParameterTypes,
+                                          !sort.isEmpty());
     bufBuilder.appendBuf(proj.objdata(), proj.objsize());
     bufBuilder.appendStrBytes(strBuilderEncoded);
     bufBuilder.appendChar(kEncodeSectionDelimiter);
@@ -1175,14 +1404,16 @@ std::string encodeSBE(const CanonicalQuery& cq) {
 
     encodeFindCommandRequest(cq, &bufBuilder);
 
-    encodePipeline(cq.pipeline(), &bufBuilder);
-
+    encodePipeline(cq.getExpCtx(), cq.cqPipeline(), &bufBuilder, optimizer);
+    if (const auto& bitset = cq.searchMetadata(); bitset.any()) {
+        bufBuilder.appendStrBytes(bitset.to_string());
+    }
     return base64::encode(StringData(bufBuilder.buf(), bufBuilder.len()));
 }
 
 CanonicalQuery::PlanCacheCommandKey encodeForPlanCacheCommand(const CanonicalQuery& cq) {
     StringBuilder keyBuilder;
-    encodeKeyForMatch(cq.root(), &keyBuilder);
+    encodeKeyForMatch(cq.getPrimaryMatchExpression(), &keyBuilder);
     encodeKeyForSort(cq.getFindCommandRequest().getSort(), &keyBuilder);
     encodeKeyForProj(cq.getProj(), &keyBuilder);
 
@@ -1197,8 +1428,31 @@ CanonicalQuery::PlanCacheCommandKey encodeForPlanCacheCommand(const CanonicalQue
     return keyBuilder.str();
 }
 
+CanonicalQuery::PlanCacheCommandKey encodeForPlanCacheCommand(const Pipeline& pipeline) {
+    static constexpr size_t bufferSize = 200;
+    BufBuilder bufBuilder(bufferSize);
+
+    std::vector<Value> serializedArray;
+    for (auto& stage : pipeline.getSources()) {
+        auto documentSource = stage.get();
+        if (auto matchStage = dynamic_cast<DocumentSourceMatch*>(documentSource)) {
+            StringBuilder keyBuilder;
+            encodeKeyForMatch(matchStage->getMatchExpression(), &keyBuilder);
+            bufBuilder.appendCStr(keyBuilder.stringData());
+        } else if (!search_helpers::encodeSearchForSbeCache(
+                       pipeline.getContext(), documentSource, &bufBuilder)) {
+            encodeKeyForPipelineStage(documentSource, serializedArray, &bufBuilder);
+        }
+    }  // for each stage in 'pipeline'
+
+    std::string key(bufBuilder.buf(), bufBuilder.len());
+    return key;
+}
+
 uint32_t computeHash(StringData key) {
-    return SimpleStringDataComparator::kInstance.hash(key);
+    size_t seed = 0;
+    simpleStringDataComparator.hash_combine(seed, key);
+    return seed;
 }
 }  // namespace canonical_query_encoder
 }  // namespace mongo
