@@ -29,19 +29,46 @@
 
 #include "mongo/db/query/canonical_query_encoder.h"
 
-#include "mongo/db/jsobj.h"
-#include "mongo/db/json.h"
-#include "mongo/db/pipeline/document_source.h"
+#include <list>
+#include <memory>
+#include <ostream>
+#include <string>
+#include <utility>
+#include <vector>
+
+#include <boost/cstdint.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+
+#include "mongo/base/status_with.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/json.h"
+#include "mongo/db/api_parameters.h"
+#include "mongo/db/basic_types.h"
+#include "mongo/db/matcher/expression_parser.h"
+#include "mongo/db/matcher/extensions_callback_noop.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/pipeline/expression_context.h"
 #include "mongo/db/pipeline/expression_context_for_test.h"
-#include "mongo/db/pipeline/inner_pipeline_stage_impl.h"
+#include "mongo/db/pipeline/pipeline.h"
 #include "mongo/db/query/canonical_query.h"
 #include "mongo/db/query/canonical_query_test_util.h"
-#include "mongo/db/query/plan_cache_key_factory.h"
-#include "mongo/db/query/query_test_service_context.h"
+#include "mongo/db/query/collation/collator_factory_interface.h"
+#include "mongo/db/query/collation/collator_interface.h"
+#include "mongo/db/query/find_command.h"
+#include "mongo/db/query/projection_policies.h"
+#include "mongo/db/repl/read_concern_args.h"
+#include "mongo/db/repl/read_concern_level.h"
 #include "mongo/idl/server_parameter_test_util.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
 #include "mongo/unittest/golden_test.h"
-#include "mongo/unittest/unittest.h"
-#include "mongo/util/assert_util.h"
+#include "mongo/unittest/golden_test_base.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/intrusive_counter.h"
 
 namespace mongo {
 namespace {
@@ -49,17 +76,23 @@ namespace {
 using std::unique_ptr;
 
 static const NamespaceString foreignNss =
-    NamespaceString::createNamespaceString_forTest("test.foreigncoll");
+    NamespaceString::createNamespaceString_forTest("testdb.foreigncoll");
 
 unittest::GoldenTestConfig goldenTestConfig{"src/mongo/db/test_output/query"};
 
-std::vector<std::unique_ptr<InnerPipelineStageInterface>> parsePipeline(
-    const boost::intrusive_ptr<ExpressionContext> expCtx, const std::vector<BSONObj>& rawPipeline) {
+std::vector<boost::intrusive_ptr<DocumentSource>> parsePipeline(
+    const boost::intrusive_ptr<ExpressionContext> expCtx,
+    const std::vector<BSONObj>& rawPipeline,
+    bool shouldParameterize = false) {
     auto pipeline = Pipeline::parse(rawPipeline, expCtx);
 
-    std::vector<std::unique_ptr<InnerPipelineStageInterface>> stages;
+    if (shouldParameterize) {
+        pipeline->parameterize();
+    }
+
+    std::vector<boost::intrusive_ptr<DocumentSource>> stages;
     for (auto&& source : pipeline->getSources()) {
-        stages.emplace_back(std::make_unique<InnerPipelineStageImpl>(source));
+        stages.emplace_back(source);
     }
     return stages;
 }
@@ -95,18 +128,15 @@ protected:
         }
         auto pipeline = parsePipeline(expCtx, pipelineObj);
 
-        auto statusWithCQ =
-            CanonicalQuery::canonicalize(opCtx,
-                                         std::move(findCommand),
-                                         false,
-                                         expCtx,
-                                         ExtensionsCallbackNoop(),
-                                         MatchExpressionParser::kAllowAllSpecialFeatures,
-                                         ProjectionPolicies::findProjectionPolicies(),
-                                         std::move(pipeline),
-                                         isCountLike);
-        ASSERT_OK(statusWithCQ.getStatus());
-        return std::move(statusWithCQ.getValue());
+        return std::make_unique<CanonicalQuery>(CanonicalQueryParams{
+            .expCtx = expCtx,
+            .parsedFind =
+                ParsedFindCommandParams{.findCommand = std::move(findCommand),
+                                        .allowedFeatures =
+                                            MatchExpressionParser::kAllowAllSpecialFeatures},
+
+            .pipeline = std::move(pipeline),
+            .isCountLike = isCountLike});
     }
 
     unique_ptr<CanonicalQuery> canonicalize(OperationContext* opCtx, const char* queryStr) {
@@ -114,6 +144,12 @@ protected:
         return canonicalize(opCtx, queryObj, {}, {}, {});
     }
 
+    void testEncodeClassic(unittest::GoldenTestContext& gctx, const CanonicalQuery& cq) {
+        gctx.outStream() << "==== VARIATION: cq=" << cq.toString();
+        const auto key = canonical_query_encoder::encodeClassic(cq);
+        gctx.outStream() << key << std::endl << std::endl;
+    }
+
     /**
      * Test functions for computeKey, when no indexes are present. Cache keys are intentionally
      * obfuscated and are meaningful only within the current lifetime of the server process. Users
@@ -160,12 +196,6 @@ protected:
                    << ", allowDiskUse=" << findCommand->getAllowDiskUse()
                    << ", returnKey=" << findCommand->getReturnKey()
                    << ", requestResumeToken=" << findCommand->getRequestResumeToken();
-            if (!findCommand->getResumeAfter().isEmpty()) {
-                stream << ", resumeAfter=1";
-            }
-            if (!findCommand->getStartAt().isEmpty()) {
-                stream << ", startAt=1";
-            }
         }
         if (isCountLike) {
             stream << ", isCountLike=true";
@@ -185,10 +215,45 @@ protected:
                                                    isCountLike,
                                                    needsMerge));
         cq->setSbeCompatible(true);
-        cq->parameterize();
-        const auto key = canonical_query_encoder::encodeSBE(*cq);
+        const auto key = canonical_query_encoder::encodeSBE(
+            *cq, canonical_query_encoder::Optimizer::kSbeStageBuilders);
         gctx.outStream() << key << std::endl;
     }
+
+    void testComputeKeyForPipeline(unittest::GoldenTestContext& gctx,
+                                   StringData matchStr,
+                                   StringData projStr) {
+        auto& stream = gctx.outStream();
+        stream << "==== VARIATION: bonsai + sbe, " << matchStr << ", " << projStr;
+        stream << std::endl;
+
+        auto pipelineObj = [](StringData matchStr, StringData projStr) -> std::vector<BSONObj> {
+            auto matchObj = fromjson(matchStr);
+            if (projStr == "{}") {
+                return {matchObj};
+            }
+
+            auto projObj = fromjson(projStr);
+            return {matchObj, projObj};
+        };
+
+        const auto expCtx = make_intrusive<ExpressionContextForTest>(opCtx(), nss);
+        auto pipeline = parsePipeline(expCtx, pipelineObj(matchStr, projStr), true);
+
+        const auto key = canonical_query_encoder::encodePipeline(
+            expCtx.get(), pipeline, canonical_query_encoder::Optimizer::kBonsai);
+        gctx.outStream() << key << std::endl;
+    }
+
+    CanonicalQuery::QueryShapeString encodeBonsai(const char* queryStr) {
+        RAIIServerParameterControllerForTest cqf("featureFlagCommonQueryFramework", "true");
+        RAIIServerParameterControllerForTest tryBonsai("internalQueryFrameworkControl",
+                                                       "tryBonsai");
+        auto cqfQuery = canonicalize(opCtx(), queryStr);
+        cqfQuery->setSbeCompatible(true);
+        return canonical_query_encoder::encodeSBE(*cqfQuery,
+                                                  canonical_query_encoder::Optimizer::kBonsai);
+    }
 };
 
 TEST_F(CanonicalQueryEncoderTest, ComputeKey) {
@@ -343,6 +408,23 @@ TEST_F(CanonicalQueryEncoderTest, ComputeKeyGeoNear) {
                    "{}");
 }
 
+// Cache keys for $_internalBucketGeoWithin with flat and spherical geometry should
+// not be the same.
+TEST_F(CanonicalQueryEncoderTest, ComputeKeyTimeseriesGeoWithin) {
+    // Flat geometry.
+    unique_ptr<CanonicalQuery> cqFlat(
+        canonicalize(opCtx(),
+                     "{ $_internalBucketGeoWithin: { withinRegion: { $center: [ [ 180.0, 0.0 ], "
+                     "1.79 ] }, field: \"loc\" } }"));
+    // Spherical geometry.
+    unique_ptr<CanonicalQuery> cqSpherical(
+        canonicalize(opCtx(),
+                     "{ $_internalBucketGeoWithin: { withinRegion: { $centerSphere: [ [ 180.0, 0.0 "
+                     "], 1.79 ] }, field: \"loc\" } }"));
+    ASSERT_NOT_EQUALS(canonical_query_encoder::encodeClassic(*cqFlat),
+                      canonical_query_encoder::encodeClassic(*cqSpherical));
+}
+
 TEST_F(CanonicalQueryEncoderTest, ComputeKeyRegexDependsOnFlags) {
     unittest::GoldenTestContext gctx(&goldenTestConfig);
     // The computed key depends on which execution engine is enabled. As such, we enable SBE for
@@ -438,6 +520,71 @@ TEST_F(CanonicalQueryEncoderTest, CheckCollationIsEncoded) {
     testComputeKey(gctx, *cq);
 }
 
+TEST_F(CanonicalQueryEncoderTest, CheckSubplanningQueriesAreEncodedDifferentlyWhenSbeCompatible) {
+    unittest::GoldenTestContext gctx(&goldenTestConfig);
+    unique_ptr<CanonicalQuery> cqWithOr(canonicalize(opCtx(), "{$or: [{a: 1, x:1}, {b: 1, y:2}]}"));
+    cqWithOr->setSbeCompatible(true);
+
+    // Test how we compute the top level key.
+    testEncodeClassic(gctx, *cqWithOr);
+
+    // Test how we compute the keys when using the CanonicalQuery() constructor specifically used by
+    // subplanning.
+    CanonicalQuery subQuery1(opCtx(), *cqWithOr, 0 /* index of OR branch */);
+    subQuery1.setSbeCompatible(true);
+    CanonicalQuery subQuery2(opCtx(), *cqWithOr, 1 /* index of OR branch */);
+    subQuery2.setSbeCompatible(true);
+
+    testEncodeClassic(gctx, subQuery1);
+    testEncodeClassic(gctx, subQuery2);
+
+    // Test how we compute the key for an equivalent query, though not created with the subplanning
+    // constructor.
+    unique_ptr<CanonicalQuery> equivalentToSubQuery1(canonicalize(opCtx(), "{a: 1, x:1}"));
+    equivalentToSubQuery1->setSbeCompatible(true);
+    unique_ptr<CanonicalQuery> equivalentToSubQuery2(canonicalize(opCtx(), "{b: 1, y:2}"));
+    equivalentToSubQuery2->setSbeCompatible(true);
+
+    testEncodeClassic(gctx, *equivalentToSubQuery1);
+    testEncodeClassic(gctx, *equivalentToSubQuery2);
+
+    // Now check that the keys for subQueryX and equivalentSubQueryX are NOT equal.
+    ASSERT_NE(canonical_query_encoder::encodeClassic(subQuery1),
+              canonical_query_encoder::encodeClassic(*equivalentToSubQuery1));
+    ASSERT_NE(canonical_query_encoder::encodeClassic(subQuery2),
+              canonical_query_encoder::encodeClassic(*equivalentToSubQuery2));
+}
+
+TEST_F(CanonicalQueryEncoderTest, ComputeClassicKeyForSbeCompatibleQuery) {
+    unittest::GoldenTestContext gctx(&goldenTestConfig);
+
+    auto test = [&](const char* match, const char* sort, const char* proj) {
+        auto cq = canonicalize(opCtx(), fromjson(match), fromjson(sort), fromjson(proj), BSONObj());
+        cq->setSbeCompatible(true);
+
+        testEncodeClassic(gctx, *cq);
+    };
+
+    test("{}", "{}", "{}");
+    test("{$or: [{a: 1}, {b: 2}]}", "{}", "{}");
+    test("{a: 1}", "{}", "{}");
+    test("{b: 1}", "{}", "{}");
+    test("{a: 1, b: 1, c: 1}", "{}", "{}");
+
+    // With sort
+    test("{}", "{a: 1}", "{}");
+    test("{}", "{a: -1}", "{}");
+    test("{a: 1}", "{a: 1}", "{}");
+
+    // With projection
+    test("{a: 1}", "{a: 1}", "{a: 1}");
+    test("{}", "{a: 1}", "{a: 1}");
+    test("{}", "{a: 1}", "{a: 1}");
+    test("{}", "{}", "{a: 1}");
+    test("{}", "{}", "{a: true}");
+    test("{}", "{}", "{a: false}");
+}
+
 TEST_F(CanonicalQueryEncoderTest, ComputeKeySBE) {
     unittest::GoldenTestContext gctx(&goldenTestConfig);
     // Generated cache keys should be treated as opaque to the user.
@@ -503,7 +650,6 @@ TEST_F(CanonicalQueryEncoderTest, ComputeKeySBE) {
     findCommand = std::make_unique<FindCommandRequest>(nss);
     findCommand->setMax(mongo::fromjson("{ a : 1 }"));
     testComputeSBEKey(gctx, "{a: 1}", "{a: 1}", "{}", std::move(findCommand));
-
     findCommand = std::make_unique<FindCommandRequest>(nss);
     findCommand->setRequestResumeToken(true);
     // "hint" must be {$natural:1} if 'requestResumeToken' is enabled.
@@ -511,12 +657,6 @@ TEST_F(CanonicalQueryEncoderTest, ComputeKeySBE) {
     findCommand->setResumeAfter(mongo::fromjson("{ $recordId: NumberLong(1) }"));
     testComputeSBEKey(gctx, "{a: 1}", "{}", "{}", std::move(findCommand));
 
-    findCommand = std::make_unique<FindCommandRequest>(nss);
-    findCommand->setRequestResumeToken(true);
-    findCommand->setHint(fromjson("{$natural: 1}"));
-    findCommand->setStartAt(mongo::fromjson("{ $recordId: NumberLong(1) }"));
-    testComputeSBEKey(gctx, "{a: 1}", "{}", "{}", std::move(findCommand));
-
     findCommand = std::make_unique<FindCommandRequest>(nss);
     findCommand->setHint(fromjson("{a: 1}"));
     testComputeSBEKey(gctx, "{a: 1}", "{}", "{}", std::move(findCommand));
@@ -626,5 +766,64 @@ TEST_F(CanonicalQueryEncoderTest, ComputeKeyWithNeedsMerge) {
                       true /* needsMerge */);
 }
 
+TEST_F(CanonicalQueryEncoderTest, ComputeKeyForPipeline) {
+    unittest::GoldenTestContext gctx(&goldenTestConfig);
+    // SBE must be enabled in order to generate SBE plan cache keys.
+    // Bonsai plan cache
+    RAIIServerParameterControllerForTest controllerSBE("internalQueryFrameworkControl",
+                                                       "trySbeEngine");
+
+    testComputeKeyForPipeline(gctx, "{$match: {a: 1}}", "{}");
+    testComputeKeyForPipeline(gctx, "{$match: {a: 2}}", "{}");
+    testComputeKeyForPipeline(gctx, "{$match: {b: 1}}", "{}");
+    testComputeKeyForPipeline(gctx, "{$match: {$and: [{a: 1}, {b: 1}]}}", "{}");
+    testComputeKeyForPipeline(gctx, "{$match: {$or: [{a: 1}, {b: 1}]}}", "{}");
+
+    // with projection
+    testComputeKeyForPipeline(gctx, "{$match: {a: 1}}", "{$project: {a: 1}}");
+    testComputeKeyForPipeline(gctx, "{$match: {b: 1}}", "{$project: {b: 1}}");
+    testComputeKeyForPipeline(
+        gctx, "{$match: {$and: [{a: 1}, {b: 1}]}}", "{$project: {a: 1, b: 1}}");
+    testComputeKeyForPipeline(
+        gctx, "{$match: {$or: [{a: 1}, {b: 1}]}}", "{$project: {a: 1, b: 1}}");
+}
+
+TEST_F(CanonicalQueryEncoderTest, EncodeOptimizerType) {
+    auto query = canonicalize(opCtx(), "{a: 1}");
+    query->setSbeCompatible(true);
+    auto classicEncoding = canonical_query_encoder::encodeSBE(
+        *query, canonical_query_encoder::Optimizer::kSbeStageBuilders);
+    auto cqfEncoding =
+        canonical_query_encoder::encodeSBE(*query, canonical_query_encoder::Optimizer::kBonsai);
+    ASSERT_NE(classicEncoding, cqfEncoding);
+}
+
+TEST_F(CanonicalQueryEncoderTest, BonsaiComparisonOperationsEncodeTypes) {
+    ASSERT_NE(encodeBonsai("{a: 1}"), encodeBonsai("{a: 'str'}"));
+    ASSERT_NE(encodeBonsai("{a: {$gt: 1}}"), encodeBonsai("{a: {$gt: 'str'}}"));
+    ASSERT_NE(encodeBonsai("{a: {$gte: 1}}"), encodeBonsai("{a: {$gte: 'str'}}"));
+    ASSERT_NE(encodeBonsai("{a: {$lt: 1}}"), encodeBonsai("{a: {$lt: 'str'}}"));
+    ASSERT_NE(encodeBonsai("{a: {$lte: 1}}"), encodeBonsai("{a: {$lte: 'str'}}"));
+
+    // Different constants of the same canonical BSON type should have the same key.
+    ASSERT_EQ(encodeBonsai("{a: 1}"), encodeBonsai("{a: 5}"));
+    ASSERT_EQ(encodeBonsai("{a: 1}"), encodeBonsai("{a: 5.0}"));
+}
+
+TEST_F(CanonicalQueryEncoderTest, BonsaiInEncoding) {
+    // Single element $in's are translated as $eq's, which means that two single element $in's with
+    // different types shouldn't have the same key.
+    ASSERT_NE(encodeBonsai("{a: {$in: [1]}}"), encodeBonsai("{a: {$in: ['str']}}"));
+    // $in with length 1 is optimized to an $eq. It should have a different key than a $in with an
+    // arbitrary length.
+    ASSERT_NE(encodeBonsai("{a: {$in: [1]}}"), encodeBonsai("{a: {$in: [1, 2]}}"));
+    // $in's with different lengths should have the same key.
+    ASSERT_EQ(encodeBonsai("{a: {$in: [1, 2]}}"), encodeBonsai("{a: {$in: [1, 2, 3]}}"));
+    ASSERT_EQ(encodeBonsai("{a: {$in: [1, 2]}}"),
+              encodeBonsai("{a: {$in: [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]}}"));
+    // $in with same length but different types have the same key.
+    ASSERT_EQ(encodeBonsai("{a: {$in: [1, 2]}}"), encodeBonsai("{a: {$in: ['str1', 'str2']}}"));
+}
+
 }  // namespace
 }  // namespace mongo
