@@ -29,13 +29,23 @@
 
 #include "mongo/db/query/canonical_query.h"
 
-#include "mongo/db/json.h"
+#include <fmt/format.h>
+
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/json.h"
+#include "mongo/db/matcher/expression_hasher.h"
+#include "mongo/db/matcher/expression_leaf.h"
 #include "mongo/db/pipeline/expression_context_for_test.h"
 #include "mongo/db/query/collation/collator_factory_interface.h"
 #include "mongo/db/query/collation/collator_interface_mock.h"
+#include "mongo/db/query/query_request_helper.h"
 #include "mongo/db/query/query_test_service_context.h"
 #include "mongo/idl/server_parameter_test_util.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/bson_test_util.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/str.h"
 
 namespace mongo {
 namespace {
@@ -70,28 +80,48 @@ MatchExpression* parseMatchExpression(const BSONObj& obj) {
 
 void assertEquivalent(const char* queryStr,
                       const MatchExpression* expected,
-                      const MatchExpression* actual) {
-    if (actual->equivalent(expected)) {
-        return;
+                      const MatchExpression* actual,
+                      bool skipHashTest = false) {
+    str::stream stream;
+
+    const MatchExpressionHasher hash{};
+    if (!skipHashTest && hash(expected) != hash(actual)) {
+        stream << "MatchExpressions' hashes are not equal.\n";
+    }
+
+    if (!expected->equivalent(actual)) {
+        stream << "MatchExpressions are not equivalent.\n";
+    }
+
+    if (stream.ss.len() > 0) {
+        stream << "Original query: " << queryStr << "\nExpected: " << expected->debugString()
+               << "\nActual: " << actual->debugString();
+
+        FAIL(stream);
     }
-    str::stream ss;
-    ss << "Match expressions are not equivalent."
-       << "\nOriginal query: " << queryStr << "\nExpected: " << expected->debugString()
-       << "\nActual: " << actual->debugString();
-    FAIL(ss);
 }
 
 void assertNotEquivalent(const char* queryStr,
                          const MatchExpression* expected,
-                         const MatchExpression* actual) {
-    if (!actual->equivalent(expected)) {
-        return;
+                         const MatchExpression* actual,
+                         bool skipHashTest = false) {
+    str::stream stream;
+
+    const MatchExpressionHasher hash{};
+    if (!skipHashTest && hash(expected) == hash(actual)) {
+        stream << "MatchExpressions' hashes are equal.\n";
+    }
+
+    if (expected->equivalent(actual)) {
+        stream << "MatchExpressions are equivalent.\n";
+    }
+
+    if (stream.ss.len() > 0) {
+        stream << "Original query: " << queryStr << "\nExpected: " << expected->debugString()
+               << "\nActual: " << actual->debugString();
+
+        FAIL(stream);
     }
-    str::stream ss;
-    ss << "Match expressions are equivalent."
-       << "\nOriginal query: " << queryStr << "\nExpected: " << expected->debugString()
-       << "\nActual: " << actual->debugString();
-    FAIL(ss);
 }
 
 TEST(CanonicalQueryTest, IsValidSortKeyMetaProjection) {
@@ -102,8 +132,10 @@ TEST(CanonicalQueryTest, IsValidSortKeyMetaProjection) {
     {
         auto findCommand = query_request_helper::makeFromFindCommandForTests(
             fromjson("{find: 'testcoll', projection: {foo: {$meta: 'sortKey'}}, '$db': 'test'}"));
-        auto cq = CanonicalQuery::canonicalize(opCtx.get(), std::move(findCommand));
-        ASSERT_NOT_OK(cq.getStatus());
+        ASSERT_THROWS(std::make_unique<CanonicalQuery>(CanonicalQueryParams{
+                          .expCtx = makeExpressionContext(opCtx.get(), *findCommand),
+                          .parsedFind = ParsedFindCommandParams{std::move(findCommand)}}),
+                      DBException);
     }
 
     // Should be able to successfully create a CQ when there is a sort.
@@ -111,8 +143,9 @@ TEST(CanonicalQueryTest, IsValidSortKeyMetaProjection) {
         auto findCommand = query_request_helper::makeFromFindCommandForTests(
             fromjson("{find: 'testcoll', projection: {foo: {$meta: 'sortKey'}}, sort: {bar: 1}, "
                      "'$db': 'test'}"));
-        auto cq = CanonicalQuery::canonicalize(opCtx.get(), std::move(findCommand));
-        ASSERT_OK(cq.getStatus());
+        ASSERT_DOES_NOT_THROW(
+            CanonicalQuery({.expCtx = makeExpressionContext(opCtx.get(), *findCommand),
+                            .parsedFind = ParsedFindCommandParams{std::move(findCommand)}}));
     }
 }
 
@@ -184,15 +217,10 @@ unique_ptr<CanonicalQuery> canonicalize(const char* queryStr,
     auto findCommand = std::make_unique<FindCommandRequest>(nss);
     findCommand->setFilter(fromjson(queryStr));
 
-    auto statusWithCQ = CanonicalQuery::canonicalize(opCtx.get(),
-                                                     std::move(findCommand),
-                                                     false,
-                                                     nullptr,
-                                                     ExtensionsCallbackNoop(),
-                                                     allowedFeatures);
-
-    ASSERT_OK(statusWithCQ.getStatus());
-    return std::move(statusWithCQ.getValue());
+    return std::make_unique<CanonicalQuery>(CanonicalQueryParams{
+        .expCtx = makeExpressionContext(opCtx.get(), *findCommand),
+        .parsedFind = ParsedFindCommandParams{.findCommand = std::move(findCommand),
+                                              .allowedFeatures = allowedFeatures}});
 }
 
 std::unique_ptr<CanonicalQuery> canonicalize(const char* queryStr,
@@ -205,20 +233,22 @@ std::unique_ptr<CanonicalQuery> canonicalize(const char* queryStr,
     findCommand->setFilter(fromjson(queryStr));
     findCommand->setSort(fromjson(sortStr));
     findCommand->setProjection(fromjson(projStr));
-    auto statusWithCQ = CanonicalQuery::canonicalize(opCtx.get(), std::move(findCommand));
-    ASSERT_OK(statusWithCQ.getStatus());
-    return std::move(statusWithCQ.getValue());
+    return std::make_unique<CanonicalQuery>(
+        CanonicalQueryParams{.expCtx = makeExpressionContext(opCtx.get(), *findCommand),
+                             .parsedFind = ParsedFindCommandParams{std::move(findCommand)}});
 }
 
 /**
  * Test function for CanonicalQuery::normalize.
  */
-void testNormalizeQuery(const char* queryStr, const char* expectedExprStr) {
+void testNormalizeQuery(const char* queryStr,
+                        const char* expectedExprStr,
+                        bool skipHashTest = false) {
     unique_ptr<CanonicalQuery> cq(canonicalize(queryStr));
-    MatchExpression* me = cq->root();
+    MatchExpression* me = cq->getPrimaryMatchExpression();
     BSONObj expectedExprObj = fromjson(expectedExprStr);
     unique_ptr<MatchExpression> expectedExpr(parseMatchExpression(expectedExprObj));
-    assertEquivalent(queryStr, expectedExpr.get(), me);
+    assertEquivalent(queryStr, expectedExpr.get(), me, skipHashTest);
 }
 
 TEST(CanonicalQueryTest, NormalizeQuerySort) {
@@ -233,10 +263,12 @@ TEST(CanonicalQueryTest, NormalizeQuerySort) {
 TEST(CanonicalQueryTest, NormalizeQueryTree) {
     // Single-child $or elimination.
     testNormalizeQuery("{$or: [{b: 1}]}", "{b: 1}");
+    // $or deduplication for same-path children.
+    testNormalizeQuery("{$or: [{b: 1}, {b: 1}]}", "{b: 1}");
     // Single-child $and elimination.
     testNormalizeQuery("{$or: [{$and: [{a: 1}]}, {b: 1}]}", "{$or: [{a: 1}, {b: 1}]}");
     // Single-child $_internalSchemaXor elimination.
-    testNormalizeQuery("{$_internalSchemaXor: [{b: 1}]}", "{b: 1}");
+    testNormalizeQuery("{$_internalSchemaXor: [{b: 1}]}", "{b: 1}", /*skipHashTest*/ true);
     // $or absorbs $or children.
     testNormalizeQuery("{$or: [{a: 1}, {$or: [{b: 1}, {$or: [{c: 1}]}]}, {d: 1}]}",
                        "{$or: [{a: 1}, {b: 1}, {c: 1}, {d: 1}]}");
@@ -247,12 +279,15 @@ TEST(CanonicalQueryTest, NormalizeQueryTree) {
     testNormalizeQuery(
         "{$_internalSchemaXor: [{$and: [{a: 1}, {b:1}]}, {$_internalSchemaXor: [{c: 1}, {d: 1}]}]}",
         "{$_internalSchemaXor: [{$and: [{a: 1}, {b:1}]}, {$_internalSchemaXor: [{c: 1}, {d: "
-        "1}]}]}");
+        "1}]}]}",
+        /*skipHashTest*/ true);
+    // $in with 0 arguments is alwaysFalse
+    testNormalizeQuery("{a: {$in: []}}", "{$alwaysFalse: 1}");
     // $in with one argument is rewritten as an equality or regex predicate.
     testNormalizeQuery("{a: {$in: [1]}}", "{a: {$eq: 1}}");
     testNormalizeQuery("{a: {$in: [/./]}}", "{a: {$regex: '.'}}");
-    // $in with 0 or more than 1 argument is not modified.
-    testNormalizeQuery("{a: {$in: []}}", "{a: {$in: []}}");
+    // $in with two or more args is not rewritten
+    testNormalizeQuery("{a: {$in: [/foo/, /bar/]}}", "{a: {$in: [/foo/, /bar/]}}");
     testNormalizeQuery("{a: {$in: [/./, 3]}}", "{a: {$in: [/./, 3]}}");
     // Child of $elemMatch object expression is normalized.
     testNormalizeQuery("{a: {$elemMatch: {$or: [{b: 1}]}}}", "{a: {$elemMatch: {b: 1}}}");
@@ -264,23 +299,27 @@ TEST(CanonicalQueryTest, NormalizeQueryTree) {
         "  {$or: [{b: 1}]},"
         "  {$_internalSchemaXor: [{c: 1}]}"
         "]}",
-        "{$_internalSchemaCond: [{a: 1}, {b: 1}, {c: 1}]}");
+        "{$_internalSchemaCond: [{a: 1}, {b: 1}, {c: 1}]}",
+        /*skipHashTest*/ true);
 }
 
 TEST(CanonicalQueryTest, CanonicalizeFromBaseQuery) {
     QueryTestServiceContext serviceContext;
     auto opCtx = serviceContext.makeOperationContext();
 
-    const bool isExplain = true;
     const std::string cmdStr =
         "{find:'bogusns', filter:{$or:[{a:1,b:1},{a:1,c:1}]}, projection:{a:1}, sort:{b:1}, '$db': "
         "'test'}";
     auto findCommand = query_request_helper::makeFromFindCommandForTests(fromjson(cmdStr));
-    auto baseCq =
-        assertGet(CanonicalQuery::canonicalize(opCtx.get(), std::move(findCommand), isExplain));
+    auto expCtx = makeExpressionContext(opCtx.get(), *findCommand);
+    expCtx->explain = explain::VerbosityEnum::kQueryPlanner;
+    auto baseCq = std::make_unique<CanonicalQuery>(CanonicalQueryParams{
+        .expCtx = std::move(expCtx),
+        .parsedFind = ParsedFindCommandParams{std::move(findCommand)},
+    });
 
-    MatchExpression* firstClauseExpr = baseCq->root()->getChild(0);
-    auto childCq = assertGet(CanonicalQuery::makeForSubplanner(opCtx.get(), *baseCq, 0));
+    MatchExpression* firstClauseExpr = baseCq->getPrimaryMatchExpression()->getChild(0);
+    auto childCq = std::make_unique<CanonicalQuery>(opCtx.get(), *baseCq, 0);
 
     ASSERT_BSONOBJ_EQ(childCq->getFindCommandRequest().getFilter(), firstClauseExpr->serialize());
 
@@ -296,8 +335,6 @@ TEST(CanonicalQueryTest, CanonicalizeFromBaseQueryWithSpecialFeature) {
     // meant to reproduce SERVER-XYZ.
     QueryTestServiceContext serviceContext;
     auto opCtx = serviceContext.makeOperationContext();
-
-    const bool isExplain = true;
     const std::string cmdStr = R"({
         find:'bogusns',
         filter: {
@@ -311,19 +348,20 @@ TEST(CanonicalQueryTest, CanonicalizeFromBaseQueryWithSpecialFeature) {
         $db: 'test'
     })";
     auto findCommand = query_request_helper::makeFromFindCommandForTests(fromjson(cmdStr));
-    auto baseCq =
-        assertGet(CanonicalQuery::canonicalize(opCtx.get(),
-                                               std::move(findCommand),
-                                               isExplain,
-                                               nullptr,
-                                               ExtensionsCallbackNoop(),
-                                               MatchExpressionParser::kAllowAllSpecialFeatures));
+    auto expCtx = makeExpressionContext(opCtx.get(), *findCommand);
+    expCtx->explain = explain::VerbosityEnum::kQueryPlanner;
+    auto baseCq = std::make_unique<CanonicalQuery>(CanonicalQueryParams{
+        .expCtx = std::move(expCtx),
+        .parsedFind = ParsedFindCommandParams{.findCommand = std::move(findCommand),
+                                              .allowedFeatures =
+                                                  MatchExpressionParser::kAllowAllSpecialFeatures},
+    });
 
     // Note: be sure to use the second child to get $text, since we 'normalize' and sort the
     // MatchExpression tree as part of canonicalization. This will put the text search clause
     // second.
-    MatchExpression* secondClauseExpr = baseCq->root()->getChild(1);
-    auto childCq = assertGet(CanonicalQuery::makeForSubplanner(opCtx.get(), *baseCq, 1));
+    MatchExpression* secondClauseExpr = baseCq->getPrimaryMatchExpression()->getChild(1);
+    auto childCq = std::make_unique<CanonicalQuery>(opCtx.get(), *baseCq, 1);
 
     ASSERT_BSONOBJ_EQ(childCq->getFindCommandRequest().getFilter(), secondClauseExpr->serialize());
 
@@ -339,7 +377,9 @@ TEST(CanonicalQueryTest, CanonicalQueryFromQRWithNoCollation) {
     auto opCtx = serviceContext.makeOperationContext();
 
     auto findCommand = std::make_unique<FindCommandRequest>(nss);
-    auto cq = assertGet(CanonicalQuery::canonicalize(opCtx.get(), std::move(findCommand)));
+    auto cq = std::make_unique<CanonicalQuery>(
+        CanonicalQueryParams{.expCtx = makeExpressionContext(opCtx.get(), *findCommand),
+                             .parsedFind = ParsedFindCommandParams{std::move(findCommand)}});
     ASSERT_TRUE(cq->getCollator() == nullptr);
 }
 
@@ -350,7 +390,9 @@ TEST(CanonicalQueryTest, CanonicalQueryFromQRWithCollation) {
     auto findCommand = std::make_unique<FindCommandRequest>(nss);
     findCommand->setCollation(BSON("locale"
                                    << "reverse"));
-    auto cq = assertGet(CanonicalQuery::canonicalize(opCtx.get(), std::move(findCommand)));
+    auto cq = std::make_unique<CanonicalQuery>(
+        CanonicalQueryParams{.expCtx = makeExpressionContext(opCtx.get(), *findCommand),
+                             .parsedFind = ParsedFindCommandParams{std::move(findCommand)}});
     CollatorInterfaceMock collator(CollatorInterfaceMock::MockType::kReverseString);
     ASSERT_TRUE(CollatorInterface::collatorsMatch(cq->getCollator(), &collator));
 }
@@ -361,8 +403,10 @@ TEST(CanonicalQueryTest, CanonicalQueryFromBaseQueryWithNoCollation) {
 
     auto findCommand = std::make_unique<FindCommandRequest>(nss);
     findCommand->setFilter(fromjson("{$or:[{a:1,b:1},{a:1,c:1}]}"));
-    auto baseCq = assertGet(CanonicalQuery::canonicalize(opCtx.get(), std::move(findCommand)));
-    auto childCq = assertGet(CanonicalQuery::makeForSubplanner(opCtx.get(), *baseCq, 0));
+    auto baseCq = std::make_unique<CanonicalQuery>(
+        CanonicalQueryParams{.expCtx = makeExpressionContext(opCtx.get(), *findCommand),
+                             .parsedFind = ParsedFindCommandParams{std::move(findCommand)}});
+    auto childCq = std::make_unique<CanonicalQuery>(opCtx.get(), *baseCq, 0);
     ASSERT_TRUE(baseCq->getCollator() == nullptr);
     ASSERT_TRUE(childCq->getCollator() == nullptr);
 }
@@ -375,8 +419,10 @@ TEST(CanonicalQueryTest, CanonicalQueryFromBaseQueryWithCollation) {
     findCommand->setFilter(fromjson("{$or:[{a:1,b:1},{a:1,c:1}]}"));
     findCommand->setCollation(BSON("locale"
                                    << "reverse"));
-    auto baseCq = assertGet(CanonicalQuery::canonicalize(opCtx.get(), std::move(findCommand)));
-    auto childCq = assertGet(CanonicalQuery::makeForSubplanner(opCtx.get(), *baseCq, 0));
+    auto baseCq = std::make_unique<CanonicalQuery>(
+        CanonicalQueryParams{.expCtx = makeExpressionContext(opCtx.get(), *findCommand),
+                             .parsedFind = ParsedFindCommandParams{std::move(findCommand)}});
+    auto childCq = std::make_unique<CanonicalQuery>(opCtx.get(), *baseCq, 0);
     ASSERT(baseCq->getCollator());
     ASSERT(childCq->getCollator());
     ASSERT_TRUE(*(childCq->getCollator()) == *(baseCq->getCollator()));
@@ -388,10 +434,12 @@ TEST(CanonicalQueryTest, SettingCollatorUpdatesCollatorAndMatchExpression) {
 
     auto findCommand = std::make_unique<FindCommandRequest>(nss);
     findCommand->setFilter(fromjson("{a: 'foo', b: {$in: ['bar', 'baz']}}"));
-    auto cq = assertGet(CanonicalQuery::canonicalize(opCtx.get(), std::move(findCommand)));
-    ASSERT_EQUALS(2U, cq->root()->numChildren());
-    auto firstChild = cq->root()->getChild(0);
-    auto secondChild = cq->root()->getChild(1);
+    auto cq = std::make_unique<CanonicalQuery>(
+        CanonicalQueryParams{.expCtx = makeExpressionContext(opCtx.get(), *findCommand),
+                             .parsedFind = ParsedFindCommandParams{std::move(findCommand)}});
+    ASSERT_EQUALS(2U, cq->getPrimaryMatchExpression()->numChildren());
+    auto firstChild = cq->getPrimaryMatchExpression()->getChild(0);
+    auto secondChild = cq->getPrimaryMatchExpression()->getChild(1);
     auto equalityExpr = static_cast<EqualityMatchExpression*>(
         firstChild->matchType() == MatchExpression::EQ ? firstChild : secondChild);
     auto inExpr = static_cast<InMatchExpression*>(
@@ -416,21 +464,24 @@ TEST(CanonicalQueryTest, SettingCollatorUpdatesCollatorAndMatchExpression) {
 
 TEST(CanonicalQueryTest, NorWithOneChildNormalizedToNot) {
     unique_ptr<CanonicalQuery> cq(canonicalize("{$nor: [{a: 1}]}"));
-    auto root = cq->root();
+    auto root = cq->getPrimaryMatchExpression();
     ASSERT_EQ(MatchExpression::NOT, root->matchType());
     ASSERT_EQ(1U, root->numChildren());
     ASSERT_EQ(MatchExpression::EQ, root->getChild(0)->matchType());
 }
 
 TEST(CanonicalQueryTest, NorWithTwoChildrenNotNormalized) {
+    RAIIServerParameterControllerForTest controller(
+        "internalQueryEnableBooleanExpressionsSimplifier", false);
+
     unique_ptr<CanonicalQuery> cq(canonicalize("{$nor: [{a: 1}, {b: 1}]}"));
-    auto root = cq->root();
+    auto root = cq->getPrimaryMatchExpression();
     ASSERT_EQ(MatchExpression::NOR, root->matchType());
 }
 
 TEST(CanonicalQueryTest, NorWithOneChildNormalizedAfterNormalizingChild) {
     unique_ptr<CanonicalQuery> cq(canonicalize("{$nor: [{$or: [{a: 1}]}]}"));
-    auto root = cq->root();
+    auto root = cq->getPrimaryMatchExpression();
     ASSERT_EQ(MatchExpression::NOT, root->matchType());
     ASSERT_EQ(1U, root->numChildren());
     ASSERT_EQ(MatchExpression::EQ, root->getChild(0)->matchType());
@@ -443,14 +494,11 @@ void assertValidSortOrder(BSONObj sort, BSONObj filter = BSONObj{}) {
     auto findCommand = std::make_unique<FindCommandRequest>(nss);
     findCommand->setFilter(filter);
     findCommand->setSort(sort);
-    auto statusWithCQ =
-        CanonicalQuery::canonicalize(opCtx.get(),
-                                     std::move(findCommand),
-                                     false,
-                                     nullptr,
-                                     ExtensionsCallbackNoop(),
-                                     MatchExpressionParser::kAllowAllSpecialFeatures);
-    ASSERT_OK(statusWithCQ.getStatus());
+    ASSERT_DOES_NOT_THROW(
+        CanonicalQuery({.expCtx = makeExpressionContext(opCtx.get(), *findCommand),
+                        .parsedFind = ParsedFindCommandParams{
+                            .findCommand = std::move(findCommand),
+                            .allowedFeatures = MatchExpressionParser::kAllowAllSpecialFeatures}}));
 }
 
 TEST(CanonicalQueryTest, ValidSortOrdersCanonicalizeSuccessfully) {
@@ -468,8 +516,10 @@ void assertInvalidSortOrder(BSONObj sort) {
 
     auto findCommand = std::make_unique<FindCommandRequest>(nss);
     findCommand->setSort(sort);
-    auto statusWithCQ = CanonicalQuery::canonicalize(opCtx.get(), std::move(findCommand));
-    ASSERT_NOT_OK(statusWithCQ.getStatus());
+    ASSERT_THROWS(std::make_unique<CanonicalQuery>(CanonicalQueryParams{
+                      .expCtx = makeExpressionContext(opCtx.get(), *findCommand),
+                      .parsedFind = ParsedFindCommandParams{std::move(findCommand)}}),
+                  DBException);
 }
 
 TEST(CanonicalQueryTest, InvalidSortOrdersFailToCanonicalize) {
@@ -496,22 +546,14 @@ TEST(CanonicalQueryTest, InvalidSortOrdersFailToCanonicalize) {
 }
 
 TEST(CanonicalQueryTest, DoNotParameterizeTextExpressions) {
-    // We never parameterize unless SBE is enabled.
-    RAIIServerParameterControllerForTest sbeFullController("featureFlagSbeFull", true);
-
     auto cq =
         canonicalize("{$text: {$search: \"Hello World!\"}}",
                      MatchExpressionParser::kDefaultSpecialFeatures | MatchExpressionParser::kText);
-    cq->parameterize();
     ASSERT_FALSE(cq->isParameterized());
 }
 
 TEST(CanonicalQueryTest, DoParameterizeRegularExpressions) {
-    // SBE must be enabled in order to generate SBE plan cache keys.
-    RAIIServerParameterControllerForTest sbeFullController("featureFlagSbeFull", true);
-
     auto cq = canonicalize("{a: 1, b: {$lt: 5}}");
-    cq->parameterize();
     ASSERT_TRUE(cq->isParameterized());
 }
 }  // namespace
