@@ -27,8 +27,16 @@
  *    it in the license file.
  */
 
+#include <memory>
+
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/query/canonical_query.h"
 #include "mongo/db/query/query_test_service_context.h"
+#include "mongo/db/service_context.h"
+#include "mongo/unittest/framework.h"
 #include "mongo/unittest/unittest.h"
 
 namespace mongo {
@@ -41,9 +49,7 @@ protected:
         return _opCtx.get();
     }
 
-    std::unique_ptr<CanonicalQuery> canonicalize(OperationContext* opCtx, const BSONObj& queryObj);
     std::unique_ptr<CanonicalQuery> canonicalize(const BSONObj& queryObj);
-    std::unique_ptr<CanonicalQuery> canonicalize(OperationContext* opCtx, StringData queryObj);
     std::unique_ptr<CanonicalQuery> canonicalize(StringData queryStr);
     std::unique_ptr<CanonicalQuery> canonicalize(BSONObj query,
                                                  BSONObj sort,
