@@ -29,12 +29,17 @@
 
 #pragma once
 
+#include <memory>
+
+#include "mongo/bson/bsonobj.h"
+#include "mongo/db/auth/cluster_auth_mode.h"
 #include "mongo/db/query/collation/collator_interface.h"
 
 namespace mongo {
 
 class BSONObj;
 class ServiceContext;
+
 template <typename T>
 class StatusWith;
 
