@@ -27,37 +27,53 @@
  *    it in the license file.
  */
 
-
-#include "mongo/platform/basic.h"
-
 #include "mongo/db/query/collection_query_info.h"
 
-#include <memory>
+#include <map>
+#include <set>
+#include <string>
+#include <vector>
 
+#include <absl/container/node_hash_map.h>
+#include <absl/meta/type_traits.h>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/db/aggregated_index_usage_tracker.h"
 #include "mongo/db/catalog/collection.h"
 #include "mongo/db/catalog/index_catalog.h"
 #include "mongo/db/collection_index_usage_tracker.h"
-#include "mongo/db/concurrency/d_concurrency.h"
-#include "mongo/db/curop_metrics.h"
+#include "mongo/db/curop.h"
+#include "mongo/db/exec/index_path_projection.h"
 #include "mongo/db/exec/projection_executor.h"
-#include "mongo/db/exec/projection_executor_utils.h"
+#include "mongo/db/feature_flag.h"
+#include "mongo/db/field_ref.h"
 #include "mongo/db/fts/fts_spec.h"
 #include "mongo/db/index/columns_access_method.h"
+#include "mongo/db/index/index_access_method.h"
 #include "mongo/db/index/index_descriptor.h"
 #include "mongo/db/index/wildcard_access_method.h"
+#include "mongo/db/index_names.h"
+#include "mongo/db/matcher/expression.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/pipeline/transformer_interface.h"
 #include "mongo/db/query/classic_plan_cache.h"
 #include "mongo/db/query/collection_index_usage_tracker_decoration.h"
-#include "mongo/db/query/get_executor.h"
+#include "mongo/db/query/index_entry.h"
 #include "mongo/db/query/planner_ixselect.h"
-#include "mongo/db/service_context.h"
+#include "mongo/db/query/query_feature_flags_gen.h"
+#include "mongo/db/query/query_knobs_gen.h"
+#include "mongo/db/storage/storage_options.h"
 #include "mongo/logv2/log.h"
-#include "mongo/util/clock_source.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/util/assert_util_core.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
 
-
 namespace mongo {
-
 namespace {
 
 CoreIndexInfo indexInfoFromIndexCatalogEntry(const IndexCatalogEntry& ice) {
@@ -77,8 +93,23 @@ CoreIndexInfo indexInfoFromIndexCatalogEntry(const IndexCatalogEntry& ice) {
             IndexEntry::Identifier{desc->indexName()},
             ice.getFilterExpression(),
             ice.getCollator(),
-            projExec,
-            ice.shared_from_this()};
+            projExec};
+}
+
+void recordCollectionIndexUsage(const CollectionPtr& coll,
+                                long long collectionScans,
+                                long long collectionScansNonTailable,
+                                const std::set<std::string>& indexesUsed) {
+    const auto& collectionIndexUsageTracker =
+        CollectionIndexUsageTrackerDecoration::get(coll.get());
+
+    collectionIndexUsageTracker.recordCollectionScans(collectionScans);
+    collectionIndexUsageTracker.recordCollectionScansNonTailable(collectionScansNonTailable);
+
+    // Record indexes used to fulfill query.
+    for (auto it = indexesUsed.begin(); it != indexesUsed.end(); ++it) {
+        collectionIndexUsageTracker.recordIndexAccess(*it);
+    }
 }
 
 }  // namespace
@@ -87,7 +118,7 @@ CollectionQueryInfo::PlanCacheState::PlanCacheState()
     : classicPlanCache{static_cast<size_t>(internalQueryCacheMaxEntriesPerCollection.load())} {}
 
 CollectionQueryInfo::PlanCacheState::PlanCacheState(OperationContext* opCtx,
-                                                    const CollectionPtr& collection)
+                                                    const Collection* collection)
     : classicPlanCache{static_cast<size_t>(internalQueryCacheMaxEntriesPerCollection.load())},
       planCacheInvalidator{collection, opCtx->getServiceContext()} {
     std::vector<CoreIndexInfo> indexCores;
@@ -147,11 +178,7 @@ void CollectionQueryInfo::computeUpdateIndexData(const IndexCatalogEntry* entry,
             }
 
             // Handle regular index fields of Compound Wildcard Index.
-            // (Ignore FCV check): This is intentional because we want clusters which have wildcard
-            // indexes
-            // still be able to use the feature even if the FCV is downgraded.
-            if (isWildcard &&
-                feature_flags::gFeatureFlagCompoundWildcardIndexes.isEnabledAndIgnoreFCVUnsafe()) {
+            if (isWildcard) {
                 BSONObj key = descriptor->keyPattern();
                 BSONObjIterator j(key);
                 while (j.more()) {
@@ -203,8 +230,7 @@ void CollectionQueryInfo::computeUpdateIndexData(const IndexCatalogEntry* entry,
     }
 }
 
-void CollectionQueryInfo::computeUpdateIndexData(OperationContext* opCtx,
-                                                 const CollectionPtr& coll) {
+void CollectionQueryInfo::computeUpdateIndexData(OperationContext* opCtx, const Collection* coll) {
     _indexedPaths.clear();
 
     auto it = coll->getIndexCatalog()->getIndexIterator(
@@ -220,18 +246,15 @@ void CollectionQueryInfo::computeUpdateIndexData(OperationContext* opCtx,
 void CollectionQueryInfo::notifyOfQuery(OperationContext* opCtx,
                                         const CollectionPtr& coll,
                                         const PlanSummaryStats& summaryStats) const {
-    auto& collectionIndexUsageTracker =
-        CollectionIndexUsageTrackerDecoration::get(coll->getSharedDecorations());
+    recordCollectionIndexUsage(coll,
+                               summaryStats.collectionScans,
+                               summaryStats.collectionScansNonTailable,
+                               summaryStats.indexesUsed);
+}
 
-    collectionIndexUsageTracker.recordCollectionScans(summaryStats.collectionScans);
-    collectionIndexUsageTracker.recordCollectionScansNonTailable(
-        summaryStats.collectionScansNonTailable);
-
-    const auto& indexesUsed = summaryStats.indexesUsed;
-    // Record indexes used to fulfill query.
-    for (auto it = indexesUsed.begin(); it != indexesUsed.end(); ++it) {
-        collectionIndexUsageTracker.recordIndexAccess(*it);
-    }
+void CollectionQueryInfo::notifyOfQuery(const CollectionPtr& coll, const OpDebug& debug) const {
+    recordCollectionIndexUsage(
+        coll, debug.collectionScans, debug.collectionScansNonTailable, debug.indexesUsed);
 }
 
 void CollectionQueryInfo::clearQueryCache(OperationContext* opCtx, const CollectionPtr& coll) {
@@ -251,7 +274,7 @@ void CollectionQueryInfo::clearQueryCache(OperationContext* opCtx, const Collect
                     "Clearing plan cache - collection info cache reinstantiated",
                     logAttrs(coll->ns()));
 
-        updatePlanCacheIndexEntries(opCtx, coll);
+        updatePlanCacheIndexEntries(opCtx, coll.get());
     }
 }
 
@@ -264,25 +287,33 @@ void CollectionQueryInfo::clearQueryCacheForSetMultikey(const CollectionPtr& col
 }
 
 void CollectionQueryInfo::updatePlanCacheIndexEntries(OperationContext* opCtx,
-                                                      const CollectionPtr& coll) {
+                                                      const Collection* coll) {
     _planCacheState = std::make_shared<PlanCacheState>(opCtx, coll);
 }
 
-void CollectionQueryInfo::init(OperationContext* opCtx, const CollectionPtr& coll) {
+void CollectionQueryInfo::init(OperationContext* opCtx, Collection* coll) {
+    // Skip registering the index in a --repair, as the server will terminate after
+    // the repair operation completes.
+    if (storageGlobalParams.repair) {
+        LOGV2_DEBUG(7610901, 1, "In a repair, skipping registering indexes");
+        return;
+    }
+
     auto ii =
         coll->getIndexCatalog()->getIndexIterator(opCtx, IndexCatalog::InclusionPolicy::kReady);
+    auto& collectionIndexUsageTracker = CollectionIndexUsageTrackerDecoration::write(coll);
     while (ii->more()) {
         const IndexDescriptor* desc = ii->next()->descriptor();
-        CollectionIndexUsageTrackerDecoration::get(coll->getSharedDecorations())
-            .registerIndex(desc->indexName(),
-                           desc->keyPattern(),
-                           IndexFeatures::make(desc, coll->ns().isOnInternalDb()));
+        collectionIndexUsageTracker.registerIndex(
+            desc->indexName(),
+            desc->keyPattern(),
+            IndexFeatures::make(desc, coll->ns().isOnInternalDb()));
     }
 
     rebuildIndexData(opCtx, coll);
 }
 
-void CollectionQueryInfo::rebuildIndexData(OperationContext* opCtx, const CollectionPtr& coll) {
+void CollectionQueryInfo::rebuildIndexData(OperationContext* opCtx, const Collection* coll) {
     _keysComputed = false;
     computeUpdateIndexData(opCtx, coll);
     updatePlanCacheIndexEntries(opCtx, coll);
