@@ -29,12 +29,17 @@
 
 #pragma once
 
+#include <s2cellid.h>
+#include <s2region.h>
 #include <vector>
 
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonobj.h"
 #include "mongo/db/geo/hash.h"
 #include "mongo/db/geo/shapes.h"
 #include "mongo/db/index/s2_common.h"
 #include "mongo/db/jsobj.h"
+#include "mongo/db/query/index_bounds.h"
 #include "mongo/db/query/index_bounds_builder.h"  // For OrderedIntervalList
 
 class S2CellId;
