@@ -31,15 +31,19 @@
  * This file contains tests for mongo/db/query/index_bounds.cpp
  */
 
-#include "mongo/platform/basic.h"
+#include <memory>
 
-#include "mongo/db/jsobj.h"
-#include "mongo/db/json.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/json.h"
 #include "mongo/db/query/index_bounds.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/stdx/type_traits.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
 #include "mongo/util/assert_util.h"
 #include "mongo/util/str.h"
-#include "mongo/util/text.h"
+#include "mongo/util/text.h"  // IWYU pragma: keep
 
 using namespace mongo;
 
@@ -220,6 +224,16 @@ BSONObj allValues() {
     return bob.obj();
 }
 
+/**
+ * Returns an object representing the interval [MaxKey, MinKey].
+ */
+BSONObj allValuesReverse() {
+    BSONObjBuilder bob;
+    bob.appendMaxKey("");
+    bob.appendMinKey("");
+    return bob.obj();
+}
+
 /**
  * Test that if we complement the OIL twice,
  * we get back the original OIL.
@@ -653,6 +667,31 @@ TEST(IndexBoundsTest, BoundsDebugStringFormatTest) {
     ASSERT_EQ(nonStringInterval.toString(true), "['b']: [1, 1]");
 }
 
+TEST(IndexBoundsTest, Unbounded) {
+    IndexBounds bounds;
+
+    bounds.fields.push_back([] {
+        OrderedIntervalList oil;
+        oil.intervals.emplace_back(allValues(), true, true);
+        return oil;
+    }());
+    ASSERT_TRUE(bounds.isUnbounded());
+
+    bounds.fields.push_back([] {
+        OrderedIntervalList oil;
+        oil.intervals.emplace_back(allValuesReverse(), true, true);
+        return oil;
+    }());
+    ASSERT_TRUE(bounds.isUnbounded());
+
+    bounds.fields.push_back([] {
+        OrderedIntervalList oil;
+        oil.intervals.emplace_back(minKeyIntObj(1), true, true);
+        return oil;
+    }());
+    ASSERT_FALSE(bounds.isUnbounded());
+}
+
 //
 // Iteration over
 //
