@@ -27,23 +27,37 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <boost/container/small_vector.hpp>
+// IWYU pragma: no_include "boost/intrusive/detail/iterator.hpp"
+// IWYU pragma: no_include "boost/move/detail/iterator_to_raw_pointer.hpp"
+#include <boost/move/utility_core.hpp>
+#include <utility>
 
-#include "mongo/db/query/internal_plans.h"
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
 
+#include "mongo/base/status_with.h"
 #include "mongo/db/catalog/clustered_collection_util.h"
-#include "mongo/db/catalog/database.h"
-#include "mongo/db/client.h"
+#include "mongo/db/catalog/index_catalog_entry.h"
 #include "mongo/db/exec/collection_scan.h"
-#include "mongo/db/exec/eof.h"
 #include "mongo/db/exec/fetch.h"
 #include "mongo/db/exec/idhack.h"
 #include "mongo/db/exec/index_scan.h"
+#include "mongo/db/exec/limit.h"
+#include "mongo/db/exec/multi_iterator.h"
 #include "mongo/db/exec/update_stage.h"
 #include "mongo/db/exec/upsert_stage.h"
+#include "mongo/db/ops/update_request.h"
+#include "mongo/db/query/collation/collator_interface.h"
 #include "mongo/db/query/get_executor.h"
+#include "mongo/db/query/internal_plans.h"
 #include "mongo/db/query/plan_executor_factory.h"
 #include "mongo/db/record_id_helpers.h"
+#include "mongo/db/storage/record_store.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/intrusive_counter.h"
+#include "mongo/util/str.h"
 
 namespace mongo {
 
@@ -145,9 +159,41 @@ CollectionScanParams createCollectionScanParams(
 }
 }  // namespace
 
+std::unique_ptr<PlanExecutor, PlanExecutor::Deleter> InternalPlanner::sampleCollection(
+    OperationContext* opCtx,
+    VariantCollectionPtrOrAcquisition collection,
+    PlanYieldPolicy::YieldPolicy yieldPolicy,
+    boost::optional<int64_t> numSamples) {
+    const auto& collectionPtr = collection.getCollectionPtr();
+    invariant(collectionPtr);
+
+    std::unique_ptr<WorkingSet> ws = std::make_unique<WorkingSet>();
+
+    auto expCtx = make_intrusive<ExpressionContext>(
+        opCtx, std::unique_ptr<CollatorInterface>(nullptr), collectionPtr->ns());
+
+    auto rsRandCursor = collectionPtr->getRecordStore()->getRandomCursor(opCtx);
+    std::unique_ptr<PlanStage> root =
+        std::make_unique<MultiIteratorStage>(expCtx.get(), ws.get(), collection);
+    static_cast<MultiIteratorStage*>(root.get())->addIterator(std::move(rsRandCursor));
+
+    if (numSamples) {
+        auto samples = *numSamples;
+        invariant(samples >= 0,
+                  "Number of samples must be >= 0, otherwise LimitStage it will never end");
+        root = std::make_unique<LimitStage>(expCtx.get(), samples, ws.get(), std::move(root));
+    }
+
+    auto statusWithPlanExecutor = plan_executor_factory::make(
+        expCtx, std::move(ws), std::move(root), collection, yieldPolicy, false);
+
+    invariant(statusWithPlanExecutor.isOK());
+    return std::move(statusWithPlanExecutor.getValue());
+}
+
 std::unique_ptr<PlanExecutor, PlanExecutor::Deleter> InternalPlanner::collectionScan(
     OperationContext* opCtx,
-    const CollectionPtr* coll,
+    VariantCollectionPtrOrAcquisition collection,
     PlanYieldPolicy::YieldPolicy yieldPolicy,
     const Direction direction,
     const boost::optional<RecordId>& resumeAfterRecordId,
@@ -155,17 +201,17 @@ std::unique_ptr<PlanExecutor, PlanExecutor::Deleter> InternalPlanner::collection
     boost::optional<RecordIdBound> maxRecord,
     CollectionScanParams::ScanBoundInclusion boundInclusion,
     bool shouldReturnEofOnFilterMismatch) {
-    const auto& collection = *coll;
-    invariant(collection);
+    const auto& collectionPtr = collection.getCollectionPtr();
+    invariant(collectionPtr);
 
     std::unique_ptr<WorkingSet> ws = std::make_unique<WorkingSet>();
 
     auto expCtx = make_intrusive<ExpressionContext>(
-        opCtx, std::unique_ptr<CollatorInterface>(nullptr), collection->ns());
+        opCtx, std::unique_ptr<CollatorInterface>(nullptr), collectionPtr->ns());
 
     auto collScanParams = createCollectionScanParams(expCtx,
                                                      ws.get(),
-                                                     coll,
+                                                     &collectionPtr,
                                                      direction,
                                                      resumeAfterRecordId,
                                                      minRecord,
@@ -173,14 +219,14 @@ std::unique_ptr<PlanExecutor, PlanExecutor::Deleter> InternalPlanner::collection
                                                      boundInclusion,
                                                      shouldReturnEofOnFilterMismatch);
 
-    auto cs = _collectionScan(expCtx, ws.get(), &collection, collScanParams);
+    auto cs = _collectionScan(expCtx, ws.get(), &collectionPtr, collScanParams);
 
     // Takes ownership of 'ws' and 'cs'.
     auto statusWithPlanExecutor =
         plan_executor_factory::make(expCtx,
                                     std::move(ws),
                                     std::move(cs),
-                                    &collection,
+                                    collection,
                                     yieldPolicy,
                                     false /* whether owned BSON must be returned */);
     invariant(statusWithPlanExecutor.isOK());
@@ -215,7 +261,7 @@ std::unique_ptr<PlanExecutor, PlanExecutor::Deleter> InternalPlanner::collection
 
 std::unique_ptr<PlanExecutor, PlanExecutor::Deleter> InternalPlanner::deleteWithCollectionScan(
     OperationContext* opCtx,
-    const CollectionPtr* coll,
+    CollectionAcquisition coll,
     std::unique_ptr<DeleteStageParams> params,
     PlanYieldPolicy::YieldPolicy yieldPolicy,
     Direction direction,
@@ -225,8 +271,8 @@ std::unique_ptr<PlanExecutor, PlanExecutor::Deleter> InternalPlanner::deleteWith
     std::unique_ptr<BatchedDeleteStageParams> batchedDeleteParams,
     const MatchExpression* filter,
     bool shouldReturnEofOnFilterMismatch) {
-    const auto& collection = *coll;
-    invariant(collection);
+    const auto& collectionPtr = coll.getCollectionPtr();
+    invariant(collectionPtr);
     if (shouldReturnEofOnFilterMismatch) {
         tassert(7010801,
                 "MatchExpression filter must be provided when 'shouldReturnEofOnFilterMismatch' is "
@@ -236,15 +282,15 @@ std::unique_ptr<PlanExecutor, PlanExecutor::Deleter> InternalPlanner::deleteWith
 
     auto ws = std::make_unique<WorkingSet>();
     auto expCtx = make_intrusive<ExpressionContext>(
-        opCtx, std::unique_ptr<CollatorInterface>(nullptr), collection->ns());
+        opCtx, std::unique_ptr<CollatorInterface>(nullptr), collectionPtr->ns());
 
-    if (collection->isCapped()) {
+    if (collectionPtr->isCapped()) {
         expCtx->setIsCappedDelete();
     }
 
     auto collScanParams = createCollectionScanParams(expCtx,
                                                      ws.get(),
-                                                     coll,
+                                                     &collectionPtr,
                                                      direction,
                                                      boost::none /* resumeAfterId */,
                                                      minRecord,
@@ -252,24 +298,24 @@ std::unique_ptr<PlanExecutor, PlanExecutor::Deleter> InternalPlanner::deleteWith
                                                      boundInclusion,
                                                      shouldReturnEofOnFilterMismatch);
 
-    auto root = _collectionScan(expCtx, ws.get(), &collection, collScanParams, filter);
+    auto root = _collectionScan(expCtx, ws.get(), &collectionPtr, collScanParams, filter);
 
     if (batchedDeleteParams) {
         root = std::make_unique<BatchedDeleteStage>(expCtx.get(),
                                                     std::move(params),
                                                     std::move(batchedDeleteParams),
                                                     ws.get(),
-                                                    collection,
+                                                    coll,
                                                     root.release());
     } else {
         root = std::make_unique<DeleteStage>(
-            expCtx.get(), std::move(params), ws.get(), collection, root.release());
+            expCtx.get(), std::move(params), ws.get(), coll, root.release());
     }
 
     auto executor = plan_executor_factory::make(expCtx,
                                                 std::move(ws),
                                                 std::move(root),
-                                                &collection,
+                                                coll,
                                                 yieldPolicy,
                                                 false /* whether owned BSON must be returned */
     );
@@ -316,7 +362,7 @@ std::unique_ptr<PlanExecutor, PlanExecutor::Deleter> InternalPlanner::indexScan(
 
 std::unique_ptr<PlanExecutor, PlanExecutor::Deleter> InternalPlanner::deleteWithIndexScan(
     OperationContext* opCtx,
-    const CollectionPtr* coll,
+    CollectionAcquisition coll,
     std::unique_ptr<DeleteStageParams> params,
     const IndexDescriptor* descriptor,
     const BSONObj& startKey,
@@ -325,16 +371,16 @@ std::unique_ptr<PlanExecutor, PlanExecutor::Deleter> InternalPlanner::deleteWith
     PlanYieldPolicy::YieldPolicy yieldPolicy,
     Direction direction,
     std::unique_ptr<BatchedDeleteStageParams> batchedDeleteParams) {
-    const auto& collection = *coll;
-    invariant(collection);
+    const auto& collectionPtr = coll.getCollectionPtr();
+    invariant(collectionPtr);
     auto ws = std::make_unique<WorkingSet>();
 
     auto expCtx = make_intrusive<ExpressionContext>(
-        opCtx, std::unique_ptr<CollatorInterface>(nullptr), collection->ns());
+        opCtx, std::unique_ptr<CollatorInterface>(nullptr), collectionPtr->ns());
 
     std::unique_ptr<PlanStage> root = _indexScan(expCtx,
                                                  ws.get(),
-                                                 &collection,
+                                                 &collectionPtr,
                                                  descriptor,
                                                  startKey,
                                                  endKey,
@@ -347,17 +393,17 @@ std::unique_ptr<PlanExecutor, PlanExecutor::Deleter> InternalPlanner::deleteWith
                                                     std::move(params),
                                                     std::move(batchedDeleteParams),
                                                     ws.get(),
-                                                    collection,
+                                                    coll,
                                                     root.release());
     } else {
         root = std::make_unique<DeleteStage>(
-            expCtx.get(), std::move(params), ws.get(), collection, root.release());
+            expCtx.get(), std::move(params), ws.get(), coll, root.release());
     }
 
     auto executor = plan_executor_factory::make(expCtx,
                                                 std::move(ws),
                                                 std::move(root),
-                                                &collection,
+                                                coll,
                                                 yieldPolicy,
                                                 false /* whether owned BSON must be returned */
     );
@@ -394,7 +440,7 @@ std::unique_ptr<PlanExecutor, PlanExecutor::Deleter> InternalPlanner::shardKeyIn
 
 std::unique_ptr<PlanExecutor, PlanExecutor::Deleter> InternalPlanner::deleteWithShardKeyIndexScan(
     OperationContext* opCtx,
-    const CollectionPtr* coll,
+    CollectionAcquisition coll,
     std::unique_ptr<DeleteStageParams> params,
     const ShardKeyIndex& shardKeyIdx,
     const BSONObj& startKey,
@@ -413,25 +459,30 @@ std::unique_ptr<PlanExecutor, PlanExecutor::Deleter> InternalPlanner::deleteWith
                                    yieldPolicy,
                                    direction);
     }
-    auto collectionScanParams = convertIndexScanParamsToCollScanParams(
-        opCtx, coll, shardKeyIdx.keyPattern(), startKey, endKey, boundInclusion, direction);
+    auto collectionScanParams = convertIndexScanParamsToCollScanParams(opCtx,
+                                                                       &coll.getCollectionPtr(),
+                                                                       shardKeyIdx.keyPattern(),
+                                                                       startKey,
+                                                                       endKey,
+                                                                       boundInclusion,
+                                                                       direction);
 
-    const auto& collection = *coll;
-    invariant(collection);
+    const auto& collectionPtr = coll.getCollectionPtr();
+    invariant(collectionPtr);
 
     std::unique_ptr<WorkingSet> ws = std::make_unique<WorkingSet>();
 
     auto expCtx = make_intrusive<ExpressionContext>(
-        opCtx, std::unique_ptr<CollatorInterface>(nullptr), collection->ns());
+        opCtx, std::unique_ptr<CollatorInterface>(nullptr), collectionPtr->ns());
 
-    auto root = _collectionScan(expCtx, ws.get(), &collection, collectionScanParams);
+    auto root = _collectionScan(expCtx, ws.get(), &collectionPtr, collectionScanParams);
     root = std::make_unique<DeleteStage>(
-        expCtx.get(), std::move(params), ws.get(), collection, root.release());
+        expCtx.get(), std::move(params), ws.get(), coll, root.release());
 
     auto executor = plan_executor_factory::make(expCtx,
                                                 std::move(ws),
                                                 std::move(root),
-                                                &collection,
+                                                coll,
                                                 yieldPolicy,
                                                 false /* whether owned BSON must be returned */
     );
@@ -441,17 +492,17 @@ std::unique_ptr<PlanExecutor, PlanExecutor::Deleter> InternalPlanner::deleteWith
 
 std::unique_ptr<PlanExecutor, PlanExecutor::Deleter> InternalPlanner::updateWithIdHack(
     OperationContext* opCtx,
-    const CollectionPtr* coll,
+    CollectionAcquisition collection,
     const UpdateStageParams& params,
     const IndexDescriptor* descriptor,
     const BSONObj& key,
     PlanYieldPolicy::YieldPolicy yieldPolicy) {
-    const auto& collection = *coll;
-    invariant(collection);
+    const auto& collectionPtr = collection.getCollectionPtr();
+    invariant(collectionPtr);
     auto ws = std::make_unique<WorkingSet>();
 
     auto expCtx = make_intrusive<ExpressionContext>(
-        opCtx, std::unique_ptr<CollatorInterface>(nullptr), collection->ns());
+        opCtx, std::unique_ptr<CollatorInterface>(nullptr), collectionPtr->ns());
 
     auto idHackStage =
         std::make_unique<IDHackStage>(expCtx.get(), key, ws.get(), collection, descriptor);
@@ -465,10 +516,9 @@ std::unique_ptr<PlanExecutor, PlanExecutor::Deleter> InternalPlanner::updateWith
     auto executor = plan_executor_factory::make(expCtx,
                                                 std::move(ws),
                                                 std::move(root),
-                                                &collection,
+                                                collection,
                                                 yieldPolicy,
-                                                false /* whether owned BSON must be returned */
-    );
+                                                false /* whether owned BSON must be returned */);
     invariant(executor.getStatus());
     return std::move(executor.getValue());
 }
@@ -483,7 +533,7 @@ std::unique_ptr<PlanStage> InternalPlanner::_collectionScan(
     const auto& collection = *coll;
     invariant(collection);
 
-    return std::make_unique<CollectionScan>(expCtx.get(), collection, params, ws, filter);
+    return std::make_unique<CollectionScan>(expCtx.get(), coll, params, ws, filter);
 }
 
 std::unique_ptr<PlanStage> InternalPlanner::_indexScan(
@@ -509,10 +559,10 @@ std::unique_ptr<PlanStage> InternalPlanner::_indexScan(
     params.shouldDedup = descriptor->getEntry()->isMultikey(expCtx->opCtx, collection);
 
     std::unique_ptr<PlanStage> root =
-        std::make_unique<IndexScan>(expCtx.get(), collection, std::move(params), ws, nullptr);
+        std::make_unique<IndexScan>(expCtx.get(), coll, std::move(params), ws, nullptr);
 
     if (InternalPlanner::IXSCAN_FETCH & options) {
-        root = std::make_unique<FetchStage>(expCtx.get(), ws, std::move(root), nullptr, collection);
+        root = std::make_unique<FetchStage>(expCtx.get(), ws, std::move(root), nullptr, coll);
     }
 
     return root;
