@@ -29,6 +29,13 @@
 
 #pragma once
 
+#include <string>
+
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonelement_comparator.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/bson/simple_bsonobj_comparator.h"
 #include "mongo/db/jsobj.h"
 #include "mongo/util/hex.h"
 #include "mongo/util/str.h"
@@ -41,12 +48,19 @@ struct Interval {
     // 'start' may not point at the first field in _intervalData.
     // 'end' may not point at the last field in _intervalData.
     // 'start' and 'end' may point at the same field.
+    // This BSON may contain elements other than the start and end elements. We cannot make any
+    // assumptions about the order of elements in this object; we should also not make any
+    // assumptions about the field names of the elements, which are not guaranteed to be empty.
     BSONObj _intervalData;
 
     // Start and End must be ordered according to the index order.
+    // For the reasons mentioned above, comparisons to 'start' and 'end' should ignore the field
+    // names of the BSONElements.
     BSONElement start;
     bool startInclusive;
 
+    // For the reasons mentioned above, comparisons to 'start' and 'end' should ignore the field
+    // names of the BSONElements.
     BSONElement end;
     bool endInclusive;
 
@@ -98,6 +112,9 @@ struct Interval {
     /** Sets the current interval to the given values (see constructor) */
     void init(BSONObj base, bool startIncluded, bool endIncluded);
 
+    Interval(
+        BSONObj base, BSONElement start, bool startInclusive, BSONElement end, bool endInclusive);
+
     /**
      * Returns true if an empty-constructed interval hasn't been init()-ialized yet
      */
@@ -231,4 +248,14 @@ inline bool operator!=(const Interval& lhs, const Interval& rhs) {
     return !(lhs == rhs);
 }
 
+template <typename H>
+H AbslHashValue(H state, const Interval& c) {
+    // Ignore field names since we only care about the value of the intervals.
+    BSONElementComparator bec(BSONElementComparator::FieldNamesMode::kIgnore, nullptr);
+    size_t hash;
+    bec.hash_combine(hash, c.start);
+    bec.hash_combine(hash, c.end);
+    return H::combine(std::move(state), c.startInclusive, c.endInclusive, hash);
+}
+
 }  // namespace mongo
