@@ -29,8 +29,22 @@
 
 #include "mongo/db/query/interval_evaluation_tree.h"
 
-#include "mongo/db/matcher/expression_internal_expr_comparison.h"
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <cstddef>
+// IWYU pragma: no_include "ext/alloc_traits.h"
+#include <ostream>
+#include <type_traits>
+
+#include "mongo/base/checked_cast.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/db/matcher/expression_leaf.h"
+#include "mongo/db/matcher/expression_path.h"
+#include "mongo/db/matcher/expression_type.h"
 #include "mongo/db/query/index_bounds_builder.h"
+#include "mongo/db/query/interval.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/str.h"
 
 namespace mongo::interval_evaluation_tree {
 namespace {
@@ -300,17 +314,21 @@ bool Builder::isEmpty() const {
 }
 
 void Builder::pop() {
+    tassert(8917803, "Cannot call pop() after calling done()", !_doneHasBeenCalled);
     tassert(6944101, "Intervals list is empty", !_intervals.empty());
     _intervals.pop();
 }
 
-boost::optional<IET> Builder::done() const {
+boost::optional<IET> Builder::done() {
+    tassert(8917802, "Cannot call done() more than once", !_doneHasBeenCalled);
+    _doneHasBeenCalled = true;
+
     if (_intervals.empty()) {
         return boost::none;
     }
 
     tassert(6334807, "All intervals should be merged into one", _intervals.size() == 1);
-    return _intervals.top();
+    return std::move(_intervals.top());
 }
 
 OrderedIntervalList evaluateIntervals(const IET& iet,
