@@ -29,10 +29,25 @@
 
 #include "mongo/db/query/optimizer/cascades/physical_rewriter.h"
 
+#include <absl/container/node_hash_map.h>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+// IWYU pragma: no_include "ext/alloc_traits.h"
+#include <iostream>
+#include <string>
+#include <type_traits>
+#include <utility>
+#include <vector>
+
+#include "mongo/db/query/optimizer/algebra/polyvalue.h"
 #include "mongo/db/query/optimizer/cascades/enforcers.h"
 #include "mongo/db/query/optimizer/cascades/implementers.h"
+#include "mongo/db/query/optimizer/cascades/rewrite_queues.h"
 #include "mongo/db/query/optimizer/cascades/rewriter_rules.h"
 #include "mongo/db/query/optimizer/explain.h"
+#include "mongo/db/query/optimizer/node.h"  // IWYU pragma: keep
+#include "mongo/util/assert_util.h"
 
 namespace mongo::optimizer::cascades {
 
@@ -69,6 +84,10 @@ public:
         return true;
     }
 
+    bool operator()(const PhysProperty&, const DistributionRequirement& requiredProp) {
+        return getPropertyConst<DistributionRequirement>(_availableProps) == requiredProp;
+    }
+
     bool operator()(const PhysProperty&, const IndexingRequirement& requiredProp) {
         const auto& available = getPropertyConst<IndexingRequirement>(_availableProps);
         return available.getIndexReqTarget() == requiredProp.getIndexReqTarget() &&
@@ -77,9 +96,20 @@ public:
             requiredProp.getSatisfiedPartialIndexesGroupId();
     }
 
-    template <class T>
-    bool operator()(const PhysProperty&, const T& requiredProp) {
-        return getPropertyConst<T>(_availableProps) == requiredProp;
+    bool operator()(const PhysProperty&, const RepetitionEstimate& requiredProp) {
+        return getPropertyConst<RepetitionEstimate>(_availableProps) == requiredProp;
+    }
+
+    bool operator()(const PhysProperty&, const LimitEstimate& requiredProp) {
+        return getPropertyConst<LimitEstimate>(_availableProps) == requiredProp;
+    }
+
+    bool operator()(const PhysProperty&, const RemoveOrphansRequirement& requiredProp) {
+        const auto& available = getPropertyConst<RemoveOrphansRequirement>(_availableProps);
+        // If the winner's circle contains a plan that removes orphans, then it doesn't matter what
+        // the required property is. Otherwise, the required property must not require removing
+        // orphans.
+        return available.mustRemove() || !requiredProp.mustRemove();
     }
 
     static bool propertiesCompatible(const PhysProps& requiredProps,
@@ -160,10 +190,30 @@ void PhysicalRewriter::costAndRetainBestNode(std::unique_ptr<ABT> node,
         printCandidateInfo(*node, groupId, nodeCost, childProps, bestResult);
     }
 
-    const CostType childCostLimit =
-        bestResult._nodeInfo ? bestResult._nodeInfo->_cost : bestResult._costLimit;
+    auto& bestNode = bestResult._nodeInfo;
+    const CostType childCostLimit = bestNode ? bestNode->_cost : bestResult._costLimit;
     const auto cost = optimizeChildren(nodeCost, childProps, childCostLimit);
-    const bool improvement = cost && (!bestResult._nodeInfo || *cost < bestResult._nodeInfo->_cost);
+    boost::optional<size_t> numElements;
+
+    bool improvement = false;
+    if (cost) {
+        if (bestNode) {
+            if (*cost < bestNode->_cost) {
+                improvement = true;
+            } else if (bestNode->_cost < *cost) {
+                // No improvement.
+            } else {
+                // If the cost is identical, retain the plan which has fewer elements.
+                numElements = countElements(*node);
+                if (!bestNode->_numElements) {
+                    bestNode->_numElements = countElements(bestNode->_node);
+                }
+                improvement = numElements < bestNode->_numElements;
+            }
+        } else {
+            improvement = true;
+        }
+    }
 
     if (_debugInfo.hasDebugLevel(3)) {
         std::cout << (cost ? (improvement ? "Improved" : "Did not improve") : "Failed optimizing")
@@ -175,6 +225,7 @@ void PhysicalRewriter::costAndRetainBestNode(std::unique_ptr<ABT> node,
             "Retaining node with uninitialized rewrite rule",
             rule != cascades::PhysicalRewriteType::Uninitialized);
     PhysNodeInfo candidateNodeInfo{std::move(*node),
+                                   numElements,
                                    cost.value_or(CostType::kInfinity),
                                    nodeCost,
                                    nodeCostAndCE._ce,
@@ -182,10 +233,10 @@ void PhysicalRewriter::costAndRetainBestNode(std::unique_ptr<ABT> node,
                                    std::move(nodeCEMap)};
     const bool keepRejectedPlans = _hints._keepRejectedPlans;
     if (improvement) {
-        if (keepRejectedPlans && bestResult._nodeInfo) {
-            bestResult._rejectedNodeInfo.push_back(std::move(*bestResult._nodeInfo));
+        if (keepRejectedPlans && bestNode) {
+            bestResult._rejectedNodeInfo.push_back(std::move(*bestNode));
         }
-        bestResult._nodeInfo = std::move(candidateNodeInfo);
+        bestNode = std::move(candidateNodeInfo);
     } else if (keepRejectedPlans) {
         bestResult._rejectedNodeInfo.push_back(std::move(candidateNodeInfo));
     }
@@ -231,7 +282,7 @@ boost::optional<CostType> PhysicalRewriter::optimizeChildren(const CostType node
 PhysicalRewriter::OptimizeGroupResult::OptimizeGroupResult()
     : _success(false), _index(0), _cost(CostType::kInfinity) {}
 
-PhysicalRewriter::OptimizeGroupResult::OptimizeGroupResult(const size_t index, const CostType cost)
+PhysicalRewriter::OptimizeGroupResult::OptimizeGroupResult(const size_t index, CostType cost)
     : _success(true), _index(index), _cost(std::move(cost)) {
     uassert(6624347,
             "Cannot have successful optimization with infinite cost",
@@ -250,11 +301,22 @@ PhysicalRewriter::OptimizeGroupResult PhysicalRewriter::optimizeGroup(const Grou
 
     Group& group = _memo.getGroup(groupId);
     const LogicalProps& logicalProps = group._logicalProperties;
-    if (hasProperty<IndexingAvailability>(logicalProps) &&
-        !hasProperty<IndexingRequirement>(physProps)) {
-        // Re-optimize under complete scan indexing requirements.
-        setPropertyOverwrite(
-            physProps, IndexingRequirement{IndexReqTarget::Complete, true /*dedupRID*/, groupId});
+    if (hasProperty<IndexingAvailability>(logicalProps)) {
+        if (!hasProperty<IndexingRequirement>(physProps)) {
+            // Re-optimize under complete scan indexing requirements.
+            setPropertyOverwrite(
+                physProps,
+                IndexingRequirement{IndexReqTarget::Complete, true /*dedupRID*/, groupId});
+        }
+        if (!hasProperty<RemoveOrphansRequirement>(physProps)) {
+            // Re-optimize with RemoveOrphansRequirement. Only require orphan filtering if the
+            // metadata for the scan definition indicates that the collection may contain orphans.
+            auto& scanDef = _metadata._scanDefs.at(
+                getPropertyConst<IndexingAvailability>(logicalProps).getScanDefName());
+            setPropertyOverwrite(
+                physProps,
+                RemoveOrphansRequirement{scanDef.shardingMetadata().mayContainOrphans()});
+        }
     }
 
     auto& physicalNodes = group._physicalNodes;
@@ -348,8 +410,13 @@ PhysicalRewriter::OptimizeGroupResult PhysicalRewriter::optimizeGroup(const Grou
     // Enforcement rewrites run just once, and are independent of the logical nodes.
     if (groupId != _rootGroupId) {
         // Verify properties can be enforced and add enforcers if necessary.
-        addEnforcers(
-            groupId, _metadata, _ridProjections, queue._queue, bestResult._physProps, logicalProps);
+        addEnforcers(groupId,
+                     _metadata,
+                     _ridProjections,
+                     queue._queue,
+                     bestResult._physProps,
+                     logicalProps,
+                     _prefixId);
     }
 
     // Iterate until we perform all logical for the group and physical rewrites for our best plan.
