@@ -27,25 +27,36 @@
  *    it in the license file.
  */
 
-#include <functional>
-#include <stack>
+#include <boost/optional.hpp>
+#include <set>
+#include <type_traits>
 
-#include "mongo/db/query/optimizer/node.h"
+#include <absl/container/node_hash_map.h>
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/db/exec/sbe/values/value.h"
+#include "mongo/db/query/optimizer/algebra/polyvalue.h"
+#include "mongo/db/query/optimizer/containers.h"
+#include "mongo/db/query/optimizer/node.h"  // IWYU pragma: keep
+#include "mongo/db/query/optimizer/syntax/expr.h"
 #include "mongo/db/query/optimizer/utils/path_utils.h"
+#include "mongo/db/query/optimizer/utils/strong_alias.h"
 #include "mongo/db/query/optimizer/utils/utils.h"
+#include "mongo/util/str.h"
 
 namespace mongo::optimizer {
 
 /**
  * A simple helper that creates a vector of Sources and binds names.
  */
-static ABT buildSimpleBinder(const ProjectionNameVector& names) {
+static ABT buildSimpleBinder(ProjectionNameVector names) {
     ABTVector sources;
     for (size_t idx = 0; idx < names.size(); ++idx) {
         sources.emplace_back(make<Source>());
     }
 
-    return make<ExpressionBinder>(names, std::move(sources));
+    return make<ExpressionBinder>(std::move(names), std::move(sources));
 }
 
 /**
@@ -95,11 +106,13 @@ static ProjectionNameVector extractProjectionNamesForScan(
 
 PhysicalScanNode::PhysicalScanNode(FieldProjectionMap fieldProjectionMap,
                                    std::string scanDefName,
-                                   bool useParallelScan)
+                                   bool useParallelScan,
+                                   ScanOrder scanOrder)
     : Base(buildSimpleBinder(extractProjectionNamesForScan(fieldProjectionMap))),
       _fieldProjectionMap(std::move(fieldProjectionMap)),
       _scanDefName(std::move(scanDefName)),
-      _useParallelScan(useParallelScan) {}
+      _useParallelScan(useParallelScan),
+      _scanOrder(scanOrder) {}
 
 bool PhysicalScanNode::operator==(const PhysicalScanNode& other) const {
     return _fieldProjectionMap == other._fieldProjectionMap && _scanDefName == other._scanDefName &&
@@ -118,6 +131,10 @@ bool PhysicalScanNode::useParallelScan() const {
     return _useParallelScan;
 }
 
+ScanOrder PhysicalScanNode::getScanOrder() const {
+    return _scanOrder;
+}
+
 ValueScanNode::ValueScanNode(ProjectionNameVector projections,
                              boost::optional<properties::LogicalProps> props)
     : ValueScanNode(
@@ -144,13 +161,17 @@ ValueScanNode::ValueScanNode(ProjectionNameVector projections,
     for (size_t i = 0; i < _arraySize; i++) {
         const auto [tag1, val1] = arr->getAt(i);
         tassert(6624083,
-                "ValueScan must be initialized with an array",
+                "ValueScan must be initialized with an array of arrays: each subarray is a row, "
+                "with one element per projection",
                 tag1 == sbe::value::TypeTags::Array);
 
         const auto innerArray = sbe::value::getArrayView(val1);
+        size_t expectedSize = projectionCount + (_hasRID ? 1 : 0);
         tassert(6624084,
-                "Unexpected number of elements in inner array",
-                innerArray->size() == projectionCount + (_hasRID ? 1 : 0));
+                str::stream() << "ValueScanNode expected " << expectedSize
+                              << " elements in each subarray (one per projection) but got "
+                              << innerArray->size(),
+                innerArray->size() == expectedSize);
         tassert(6624177,
                 "First element must be a RecordId",
                 !_hasRID || innerArray->getAt(0).first == sbe::value::TypeTags::RecordId);
@@ -227,13 +248,12 @@ SeekNode::SeekNode(ProjectionName ridProjectionName,
                    FieldProjectionMap fieldProjectionMap,
                    std::string scanDefName)
     : Base(buildSimpleBinder(extractProjectionNamesForScan(fieldProjectionMap)),
-           make<References>(ProjectionNameVector{ridProjectionName})),
-      _ridProjectionName(std::move(ridProjectionName)),
+           make<References>(ProjectionNameVector{std::move(ridProjectionName)})),
       _fieldProjectionMap(std::move(fieldProjectionMap)),
       _scanDefName(std::move(scanDefName)) {}
 
 bool SeekNode::operator==(const SeekNode& other) const {
-    return _ridProjectionName == other._ridProjectionName &&
+    return getRIDProjectionName() == other.getRIDProjectionName() &&
         _fieldProjectionMap == other._fieldProjectionMap && _scanDefName == other._scanDefName;
 }
 
@@ -246,7 +266,7 @@ const std::string& SeekNode::getScanDefName() const {
 }
 
 const ProjectionName& SeekNode::getRIDProjectionName() const {
-    return _ridProjectionName;
+    return get<1>().cast<References>()->nodes()[0].cast<Variable>()->name();
 }
 
 MemoLogicalDelegatorNode::MemoLogicalDelegatorNode(const GroupIdType groupId)
@@ -339,9 +359,38 @@ const ProjectionName& RIDIntersectNode::getScanProjectionName() const {
     return _scanProjectionName;
 }
 
-RIDUnionNode::RIDUnionNode(ProjectionName scanProjectionName, ABT leftChild, ABT rightChild)
-    : Base(std::move(leftChild), std::move(rightChild)),
+/**
+ * A helper that builds References object of UnionNode or SortedMergeNode for reference tracking
+ * purposes.
+ *
+ * Example: union outputs 3 projections: A,B,C and it has 4 children. Then the References object is
+ * a vector of variables A,B,C,A,B,C,A,B,C,A,B,C. One group of variables per child.
+ */
+static ABT buildUnionTypeReferences(const ProjectionNameVector& names, const size_t numOfChildren) {
+    ABTVector variables;
+    for (size_t outerIdx = 0; outerIdx < numOfChildren; ++outerIdx) {
+        for (size_t idx = 0; idx < names.size(); ++idx) {
+            variables.emplace_back(make<Variable>(names[idx]));
+        }
+    }
+
+    return make<References>(std::move(variables));
+}
+
+RIDUnionNode::RIDUnionNode(ProjectionName scanProjectionName,
+                           ProjectionNameVector unionProjectionNames,
+                           ABT leftChild,
+                           ABT rightChild)
+    : Base(std::move(leftChild),
+           std::move(rightChild),
+           buildSimpleBinder(unionProjectionNames),
+           buildUnionTypeReferences(unionProjectionNames, 2)),
       _scanProjectionName(std::move(scanProjectionName)) {
+    tassert(7858803,
+            "Scan projection must exist in the RIDUnionNode projection list",
+            std::find(unionProjectionNames.cbegin(),
+                      unionProjectionNames.cend(),
+                      _scanProjectionName) != unionProjectionNames.cend());
     assertNodeSort(getLeftChild());
     assertNodeSort(getRightChild());
 }
@@ -362,6 +411,12 @@ ABT& RIDUnionNode::getRightChild() {
     return get<1>();
 }
 
+const ExpressionBinder& RIDUnionNode::binder() const {
+    const ABT& result = get<2>();
+    tassert(7858801, "Invalid binder type", result.is<ExpressionBinder>());
+    return *result.cast<ExpressionBinder>();
+}
+
 bool RIDUnionNode::operator==(const RIDUnionNode& other) const {
     return _scanProjectionName == other._scanProjectionName &&
         getLeftChild() == other.getLeftChild() && getRightChild() == other.getRightChild();
@@ -371,9 +426,9 @@ const ProjectionName& RIDUnionNode::getScanProjectionName() const {
     return _scanProjectionName;
 }
 
-static ProjectionNameVector createSargableBindings(const PartialSchemaRequirements& reqMap) {
+static ProjectionNameVector createSargableBindings(const PSRExpr::Node& reqMap) {
     ProjectionNameVector result;
-    PSRExpr::visitDNF(reqMap.getRoot(), [&](const PartialSchemaEntry& e) {
+    PSRExpr::visitDNF(reqMap, [&](const PartialSchemaEntry& e, const PSRExpr::VisitorContext&) {
         if (auto binding = e.second.getBoundProjectionName()) {
             result.push_back(*binding);
         }
@@ -381,15 +436,15 @@ static ProjectionNameVector createSargableBindings(const PartialSchemaRequiremen
     return result;
 }
 
-static ProjectionNameVector createSargableReferences(const PartialSchemaRequirements& reqMap) {
+static ProjectionNameVector createSargableReferences(const PSRExpr::Node& reqMap) {
     ProjectionNameOrderPreservingSet result;
-    PSRExpr::visitDNF(reqMap.getRoot(), [&](const PartialSchemaEntry& e) {
+    PSRExpr::visitDNF(reqMap, [&](const PartialSchemaEntry& e, const PSRExpr::VisitorContext&) {
         result.emplace_back(*e.first._projectionName);
     });
     return result.getVector();
 }
 
-SargableNode::SargableNode(PartialSchemaRequirements reqMap,
+SargableNode::SargableNode(PSRExpr::Node reqMap,
                            CandidateIndexes candidateIndexes,
                            boost::optional<ScanParams> scanParams,
                            const IndexReqTarget target,
@@ -402,11 +457,9 @@ SargableNode::SargableNode(PartialSchemaRequirements reqMap,
       _scanParams(std::move(scanParams)),
       _target(target) {
     assertNodeSort(getChild());
-    tassert(6624085, "SargableNode requires at least one predicate", !_reqMap.isNoop());
-    tassert(
-        7447500, "SargableNode requirements should be in DNF", PSRExpr::isDNF(_reqMap.getRoot()));
-    if (const size_t numLeaves = PSRExpr::numLeaves(_reqMap.getRoot());
-        numLeaves > kMaxPartialSchemaReqs) {
+    tassert(6624085, "SargableNode requires at least one predicate", !psr::isNoop(_reqMap));
+    tassert(7447500, "SargableNode requirements should be in DNF", PSRExpr::isDNF(_reqMap));
+    if (const size_t numLeaves = PSRExpr::numLeaves(_reqMap); numLeaves > kMaxPartialSchemaReqs) {
         tasserted(6624086,
                   str::stream() << "SargableNode has too many predicates: " << numLeaves
                                 << ". We allow at most " << kMaxPartialSchemaReqs);
@@ -415,7 +468,7 @@ SargableNode::SargableNode(PartialSchemaRequirements reqMap,
     auto bindings = createSargableBindings(_reqMap);
     tassert(7410100,
             "SargableNode with top-level OR cannot bind",
-            bindings.empty() || PSRExpr::isSingletonDisjunction(_reqMap.getRoot()));
+            bindings.empty() || PSRExpr::isSingletonDisjunction(_reqMap));
 
     ProjectionNameSet boundsProjectionNameSet(bindings.begin(), bindings.end());
 
@@ -423,44 +476,49 @@ SargableNode::SargableNode(PartialSchemaRequirements reqMap,
     // projections, or non-trivial multikey requirements which also bind. Further assert that under
     // a conjunction 1) non-multikey paths have at most one req and 2) there are no duplicate bound
     // projection names.
-    PSRExpr::visitDisjuncts(_reqMap.getRoot(), [&](const PSRExpr::Node& disjunct, const size_t) {
-        PartialSchemaKeySet seenKeys;
-        ProjectionNameSet seenProjNames;
-        PSRExpr::visitConjuncts(disjunct, [&](const PSRExpr::Node& conjunct, const size_t) {
-            PSRExpr::visitAtom(conjunct, [&](const PartialSchemaEntry& e) {
-                const auto& [key, req] = e;
-                if (auto projName = req.getBoundProjectionName()) {
-                    tassert(
-                        6624094,
-                        "SargableNode has a multikey requirement with a non-trivial interval which "
-                        "also binds",
-                        isIntervalReqFullyOpenDNF(req.getIntervals()) ||
-                            !checkPathContainsTraverse(key._path));
-                    tassert(6624095,
-                            "SargableNode has a perf only binding requirement",
-                            !req.getIsPerfOnly());
+    PSRExpr::visitDisjuncts(
+        _reqMap, [&](const PSRExpr::Node& disjunct, const PSRExpr::VisitorContext&) {
+            PartialSchemaKeySet seenKeys;
+            ProjectionNameSet seenProjNames;
+            PSRExpr::visitConjuncts(
+                disjunct, [&](const PSRExpr::Node& conjunct, const PSRExpr::VisitorContext&) {
+                    PSRExpr::visitAtom(
+                        conjunct, [&](const PartialSchemaEntry& e, const PSRExpr::VisitorContext&) {
+                            const auto& [key, req] = e;
+                            if (auto projName = req.getBoundProjectionName()) {
+                                tassert(6624094,
+                                        "SargableNode has a multikey requirement with a "
+                                        "non-trivial interval which "
+                                        "also binds",
+                                        isIntervalReqFullyOpenDNF(req.getIntervals()) ||
+                                            !checkPathContainsTraverse(key._path));
+                                tassert(6624095,
+                                        "SargableNode has a perf only binding requirement",
+                                        !req.getIsPerfOnly());
 
-                    auto insertedBoundProj = seenProjNames.insert(*projName).second;
-                    tassert(6624087,
-                            "PartialSchemaRequirements has duplicate bound projection names in "
-                            "a conjunction",
-                            insertedBoundProj);
-                }
+                                auto insertedBoundProj = seenProjNames.insert(*projName).second;
+                                tassert(6624087,
+                                        "PartialSchemaRequirements has duplicate bound projection "
+                                        "names in "
+                                        "a conjunction",
+                                        insertedBoundProj);
+                            }
 
-                tassert(6624088,
-                        "SargableNode cannot reference an internally bound projection",
-                        boundsProjectionNameSet.count(*key._projectionName) == 0);
+                            tassert(6624088,
+                                    "SargableNode cannot reference an internally bound projection",
+                                    boundsProjectionNameSet.count(*key._projectionName) == 0);
 
-                if (!checkPathContainsTraverse(key._path)) {
-                    auto insertedKey = seenKeys.insert(key).second;
-                    tassert(7155020,
-                            "PartialSchemaRequirements has two predicates on the same non-multikey "
-                            "path in a conjunction",
-                            insertedKey);
-                }
-            });
+                            if (!checkPathContainsTraverse(key._path)) {
+                                auto insertedKey = seenKeys.insert(key).second;
+                                tassert(7155020,
+                                        "PartialSchemaRequirements has two predicates on the same "
+                                        "non-multikey "
+                                        "path in a conjunction",
+                                        insertedKey);
+                            }
+                        });
+                });
         });
-    });
 }
 
 bool SargableNode::operator==(const SargableNode& other) const {
@@ -469,7 +527,7 @@ bool SargableNode::operator==(const SargableNode& other) const {
     return _reqMap == other._reqMap && _target == other._target && getChild() == other.getChild();
 }
 
-const PartialSchemaRequirements& SargableNode::getReqMap() const {
+const PSRExpr::Node& SargableNode::getReqMap() const {
     return _reqMap;
 }
 
@@ -703,24 +761,6 @@ const ABT& NestedLoopJoinNode::getFilter() const {
     return get<2>();
 }
 
-/**
- * A helper that builds References object of UnionNode or SortedMergeNode for reference tracking
- * purposes.
- *
- * Example: union outputs 3 projections: A,B,C and it has 4 children. Then the References object is
- * a vector of variables A,B,C,A,B,C,A,B,C,A,B,C. One group of variables per child.
- */
-static ABT buildUnionTypeReferences(const ProjectionNameVector& names, const size_t numOfChildren) {
-    ABTVector variables;
-    for (size_t outerIdx = 0; outerIdx < numOfChildren; ++outerIdx) {
-        for (size_t idx = 0; idx < names.size(); ++idx) {
-            variables.emplace_back(make<Variable>(names[idx]));
-        }
-    }
-
-    return make<References>(std::move(variables));
-}
-
 // Helper function to get the projection names from a CollationRequirement as a vector instead of a
 // set, since we would like to keep the order.
 static ProjectionNameVector getAffectedProjectionNamesOrdered(
@@ -797,7 +837,7 @@ GroupByNode::GroupByNode(ProjectionNameVector groupByProjectionNames,
                          GroupNodeType type,
                          ABT child)
     : Base(std::move(child),
-           buildSimpleBinder(aggregationProjectionNames),
+           buildSimpleBinder(std::move(aggregationProjectionNames)),
            make<References>(std::move(aggregationExpressions)),
            buildSimpleBinder(groupByProjectionNames),
            make<References>(groupByProjectionNames)),
@@ -930,7 +970,7 @@ ABT& SpoolProducerNode::getChild() {
 SpoolConsumerNode::SpoolConsumerNode(const SpoolConsumerType type,
                                      const int64_t spoolId,
                                      ProjectionNameVector projections)
-    : Base(buildSimpleBinder(projections)), _type(type), _spoolId(spoolId) {
+    : Base(buildSimpleBinder(std::move(projections))), _type(type), _spoolId(spoolId) {
     tassert(
         6624125, "Spool consumer must have a non-empty projection list", !binder().names().empty());
 }
@@ -999,7 +1039,7 @@ ABT& LimitSkipNode::getChild() {
     return get<0>();
 }
 
-ExchangeNode::ExchangeNode(const properties::DistributionRequirement distribution, ABT child)
+ExchangeNode::ExchangeNode(properties::DistributionRequirement distribution, ABT child)
     : Base(std::move(child), buildReferences(distribution.getAffectedProjectionNames())),
       _distribution(std::move(distribution)) {
     assertNodeSort(getChild());
