@@ -27,14 +27,35 @@
  *    it in the license file.
  */
 
+#include <memory>
+#include <utility>
+
+#include <absl/container/flat_hash_map.h>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/db/query/ce/hinted_estimator.h"
+#include "mongo/db/query/cost_model/cost_model_gen.h"
+#include "mongo/db/query/optimizer/cascades/memo.h"
+#include "mongo/db/query/optimizer/defs.h"
 #include "mongo/db/query/optimizer/explain.h"
+#include "mongo/db/query/optimizer/index_bounds.h"
+#include "mongo/db/query/optimizer/metadata.h"
 #include "mongo/db/query/optimizer/metadata_factory.h"
-#include "mongo/db/query/optimizer/node.h"
+#include "mongo/db/query/optimizer/node.h"  // IWYU pragma: keep
 #include "mongo/db/query/optimizer/opt_phase_manager.h"
+#include "mongo/db/query/optimizer/props.h"
 #include "mongo/db/query/optimizer/rewrites/const_eval.h"
+#include "mongo/db/query/optimizer/syntax/path.h"
+#include "mongo/db/query/optimizer/syntax/syntax.h"
 #include "mongo/db/query/optimizer/utils/unit_test_abt_literals.h"
 #include "mongo/db/query/optimizer/utils/unit_test_utils.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/db/query/optimizer/utils/utils.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
 
 
 namespace mongo::optimizer {
@@ -156,7 +177,7 @@ TEST(PhysRewriterParallel, IndexPartitioning0) {
                      {{makeNonMultikeyIndexPath("a"), CollationOp::Ascending}},
                      false /*isMultiKey*/,
                      {DistributionType::HashPartitioning, makeSeq(makeNonMultikeyIndexPath("a"))},
-                     {}}}},
+                     psr::makeNoOp()}}},
                ConstEval::constFold,
                {DistributionType::HashPartitioning, makeSeq(makeNonMultikeyIndexPath("b"))})}},
          5 /*numberOfPartitions*/},
@@ -166,7 +187,7 @@ TEST(PhysRewriterParallel, IndexPartitioning0) {
 
     ABT optimized = rootNode;
     phaseManager.optimize(optimized);
-    ASSERT_BETWEEN(60, 100, phaseManager.getMemo().getStats()._physPlanExplorationCount);
+    ASSERT_BETWEEN_AUTO(20, 40, phaseManager.getMemo().getStats()._physPlanExplorationCount);
 
     ASSERT_EXPLAIN_V2_AUTO(
         "Root [{pc}]\n"
@@ -226,7 +247,8 @@ TEST(PhysRewriterParallel, IndexPartitioning1) {
     auto phaseManager = makePhaseManager(
         {OptPhase::MemoSubstitutionPhase,
          OptPhase::MemoExplorationPhase,
-         OptPhase::MemoImplementationPhase},
+         OptPhase::MemoImplementationPhase,
+         OptPhase::ProjNormalize},
         prefixId,
         {{{"c1",
            createScanDef(
@@ -236,13 +258,13 @@ TEST(PhysRewriterParallel, IndexPartitioning1) {
                      {{makeNonMultikeyIndexPath("a"), CollationOp::Ascending}},
                      false /*isMultiKey*/,
                      {DistributionType::HashPartitioning, makeSeq(makeNonMultikeyIndexPath("a"))},
-                     {}}},
+                     psr::makeNoOp()}},
                 {"index2",
                  IndexDefinition{
                      {{makeNonMultikeyIndexPath("b"), CollationOp::Ascending}},
                      false /*isMultiKey*/,
                      {DistributionType::HashPartitioning, makeSeq(makeNonMultikeyIndexPath("b"))},
-                     {}}}},
+                     psr::makeNoOp()}}},
                ConstEval::constFold,
                {DistributionType::HashPartitioning, makeSeq(makeNonMultikeyIndexPath("c"))})}},
          5 /*numberOfPartitions*/},
@@ -252,28 +274,30 @@ TEST(PhysRewriterParallel, IndexPartitioning1) {
 
     ABT optimized = rootNode;
     phaseManager.optimize(optimized);
-    ASSERT_BETWEEN(110, 160, phaseManager.getMemo().getStats()._physPlanExplorationCount);
+    ASSERT_BETWEEN_AUTO(80, 140, phaseManager.getMemo().getStats()._physPlanExplorationCount);
 
-    const BSONObj& result = ExplainGenerator::explainBSONObj(optimized);
-
-    // Compare using BSON since the rid vars are currently unstable for this test.
-    ASSERT_BSON_PATH("\"Exchange\"", result, "child.nodeType");
-    ASSERT_BSON_PATH(
-        "{ type: \"Centralized\", disableExchanges: false }", result, "child.distribution");
-    ASSERT_BSON_PATH("\"GroupBy\"", result, "child.child.nodeType");
-    ASSERT_BSON_PATH("\"HashJoin\"", result, "child.child.child.nodeType");
-    ASSERT_BSON_PATH("\"Exchange\"", result, "child.child.child.leftChild.nodeType");
-    ASSERT_BSON_PATH("{ type: \"Replicated\", disableExchanges: false }",
-                     result,
-                     "child.child.child.leftChild.distribution");
-    ASSERT_BSON_PATH("\"IndexScan\"", result, "child.child.child.leftChild.child.nodeType");
-    ASSERT_BSON_PATH("\"index2\"", result, "child.child.child.leftChild.child.indexDefName");
-    ASSERT_BSON_PATH("\"Union\"", result, "child.child.child.rightChild.nodeType");
-    ASSERT_BSON_PATH("\"Evaluation\"", result, "child.child.child.rightChild.children.0.nodeType");
-    ASSERT_BSON_PATH(
-        "\"IndexScan\"", result, "child.child.child.rightChild.children.0.child.nodeType");
-    ASSERT_BSON_PATH(
-        "\"index1\"", result, "child.child.child.rightChild.children.0.child.indexDefName");
+    ASSERT_EXPLAIN_V2_AUTO(  // NOLINT
+        "Root [{renamed_7}]\n"
+        "Exchange []\n"
+        "|   |   distribution: \n"
+        "|   |       type: Centralized\n"
+        "GroupBy [{renamed_3}]\n"
+        "|   aggregations: \n"
+        "|       [renamed_7]\n"
+        "|           Variable [renamed_1]\n"
+        "HashJoin [joinType: Inner]\n"
+        "|   |   Condition\n"
+        "|   |       renamed_0 = renamed_4\n"
+        "|   Union [{renamed_3, renamed_4}]\n"
+        "|   Evaluation [{renamed_4} = Variable [renamed_0]]\n"
+        "|   IndexScan [{'<indexKey> 0': renamed_3, '<rid>': renamed_0}, scanDefName: c1, "
+        "indexDefName: index1, interval: {>Const [0]}]\n"
+        "Exchange []\n"
+        "|   |   distribution: \n"
+        "|   |       type: Replicated\n"
+        "IndexScan [{'<indexKey> 0': renamed_1, '<rid>': renamed_0}, scanDefName: c1, "
+        "indexDefName: index2, interval: {>Const [1]}]\n",
+        optimized);
 }
 
 TEST(PhysRewriterParallel, LocalGlobalAgg) {
@@ -392,7 +416,7 @@ TEST(PhysRewriterParallel, LocalLimitSkip) {
     ASSERT_BETWEEN(5, 15, phaseManager.getMemo().getStats()._physPlanExplorationCount);
 
     ASSERT_EXPLAIN_PROPS_V2_AUTO(
-        "Properties [cost: 0.00929774, localCost: 0, adjustedCE: 20]\n"
+        "Properties [cost: 0.00930515, localCost: 0, adjustedCE: 20]\n"
         "|   |   Logical:\n"
         "|   |       cardinalityEstimate: \n"
         "|   |           ce: 20\n"
@@ -409,7 +433,7 @@ TEST(PhysRewriterParallel, LocalLimitSkip) {
         "|       distribution: \n"
         "|           type: Centralized\n"
         "Root [{root}]\n"
-        "Properties [cost: 0.00929774, localCost: 0.00252777, adjustedCE: 30]\n"
+        "Properties [cost: 0.00930515, localCost: 0.00252964, adjustedCE: 30.03]\n"
         "|   |   Logical:\n"
         "|   |       cardinalityEstimate: \n"
         "|   |           ce: 1000\n"
@@ -432,8 +456,10 @@ TEST(PhysRewriterParallel, LocalLimitSkip) {
         "|           type: Centralized\n"
         "|       indexingRequirement: \n"
         "|           Complete, dedupRID\n"
+        "|       removeOrphans: \n"
+        "|           false\n"
         "LimitSkip [limit: 20, skip: 10]\n"
-        "Properties [cost: 0.00676997, localCost: 0.003001, adjustedCE: 30]\n"
+        "Properties [cost: 0.00677551, localCost: 0.003004, adjustedCE: 30.03]\n"
         "|   |   Logical:\n"
         "|   |       cardinalityEstimate: \n"
         "|   |           ce: 1000\n"
@@ -453,11 +479,14 @@ TEST(PhysRewriterParallel, LocalLimitSkip) {
         "|           type: Centralized\n"
         "|       indexingRequirement: \n"
         "|           Complete, dedupRID\n"
-        "|       limitEstimate: 30\n"
+        "|       limitEstimate: \n"
+        "|           30\n"
+        "|       removeOrphans: \n"
+        "|           false\n"
         "Exchange []\n"
         "|   |   distribution: \n"
         "|   |       type: Centralized\n"
-        "Properties [cost: 0.00376897, localCost: 0.00376897, adjustedCE: 30]\n"
+        "Properties [cost: 0.00377151, localCost: 0.00377151, adjustedCE: 30.03]\n"
         "|   |   Logical:\n"
         "|   |       cardinalityEstimate: \n"
         "|   |           ce: 1000\n"
@@ -477,7 +506,10 @@ TEST(PhysRewriterParallel, LocalLimitSkip) {
         "|           type: UnknownPartitioning, disableExchanges\n"
         "|       indexingRequirement: \n"
         "|           Complete, dedupRID\n"
-        "|       limitEstimate: 30\n"
+        "|       limitEstimate: \n"
+        "|           30\n"
+        "|       removeOrphans: \n"
+        "|           false\n"
         "PhysicalScan [{'<root>': root}, c1, parallel]\n",
         phaseManager);
 }
