@@ -27,20 +27,55 @@
  *    it in the license file.
  */
 
-#include <iostream>
+#include <utility>
+#include <vector>
 
+#include <absl/meta/type_traits.h>
+
+#include "mongo/db/query/optimizer/algebra/operator.h"
+#include "mongo/db/query/optimizer/containers.h"
+#include "mongo/db/query/optimizer/props.h"
 #include "mongo/db/query/optimizer/reference_tracker.h"
+#include "mongo/db/query/optimizer/utils/strong_alias.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/str.h"
 
 
 namespace mongo::optimizer {
 
-struct CollectedInfo {
-    using VarRefsMap = ProjectionNameMap<opt::unordered_map<const Variable*, bool>>;
+/**
+ * A Variable is 'resolved' when it is associated with a Definition.
+ */
+struct ResolvedVariable {
+    const Variable* var;
+    const Definition def;
+};
 
+/**
+ * While analyzing an ABT tree via the Collector transport class, there is a need
+ * for state that is 'global' for the duration of the analysis and is accessible to
+ * all Collector::transport methods. This class represents such a state.
+ * Notice:
+ * This is a struct instead of simple 'using' because at least one more future task
+ * will add more state to it - namely, SERVER-80954.
+ */
+struct CollectorState {
     /**
-     * All resolved variables so far, regardless of visibility in the ABT.
+     * All resolved variables, regardless of visibility in the ABT.
      */
-    opt::unordered_map<const Variable*, Definition> useMap;
+    std::unique_ptr<ResolvedVariablesMap> resolvedVariablesMap;
+};
+
+/**
+ * Information collected by each Collector::transport method for each ABT node in a tree.
+ * The Collector passes the CollectedInfo of a node's children to the parent's node
+ * transport method, where the child/children CollectedInfo is typically merged into
+ * the parent's CollectedInfo.
+ */
+struct CollectedInfo {
+    CollectedInfo(CollectorState& collr) : collector(collr){};
+
+    using VarRefsMap = ProjectionNameMap<opt::unordered_map<const Variable*, bool>>;
 
     /**
      * Current definitions available for use in ancestor nodes (projections).
@@ -50,7 +85,7 @@ struct CollectedInfo {
     /**
      * All free variables (i.e. so far not resolved) seen so far, regardless of visibility in the
      * ABT. Maps from projection name to all Variable instances referencing that name. Variables
-     * move from 'freeVars' to 'useMap' when they are resolved.
+     * move from 'freeVars' to 'Collector::resolvedVariables' when they are resolved.
      */
     ProjectionNameMap<std::vector<std::reference_wrapper<const Variable>>> freeVars;
 
@@ -60,11 +95,10 @@ struct CollectedInfo {
     opt::unordered_map<const Node*, DefinitionsMap> nodeDefs;
 
     /**
-     * Support for tracking last local variable reference. A Variable moves from 'varLastRefs' to
-     * 'lastRefs' while building the environment when we are sure it is a last reference.
+     * The collector transport class stores global information that is updated by some
+     * CollectedInfo methods. Hence we need a pointer to the collector.
      */
-    VarRefsMap varLastRefs;
-    opt::unordered_set<const Variable*> lastRefs;
+    CollectorState& collector;
 
     /**
      * This is a destructive merge, the 'other' will be siphoned out.
@@ -88,11 +122,6 @@ struct CollectedInfo {
             }
         }
 
-        // It should be impossible to have duplicate Variable pointer so every Variable should be
-        // moved from other.
-        useMap.merge(other.useMap);
-        tassert(6624024, "Found a duplicate Variable pointer", other.useMap.empty());
-
         // There should not be two projections of the same name propagated up by a single operator,
         // so every definition should be moved from other.
         defs.merge(other.defs);
@@ -108,81 +137,6 @@ struct CollectedInfo {
         // other.
         nodeDefs.merge(other.nodeDefs);
         tassert(6624026, "Found a duplicate Node pointer", other.nodeDefs.empty());
-
-        // Merge last references.
-        mergeLastRefs(std::move(other.varLastRefs));
-        lastRefs.merge(other.lastRefs);
-        tassert(6624027, "Found a duplicate Variable pointer in lastRefs", other.lastRefs.empty());
-    }
-
-    /**
-     * Merges variable references from 'other' and adjust last references as needed; i.e. it resets
-     * any last ref to false if it appears on both sides.
-     */
-    void mergeLastRefs(VarRefsMap&& other) {
-        mergeLastRefsImpl(std::move(other), false, true);
-    }
-
-    /**
-     * Merges variable references from 'other' but only keeps the last references from 'this'; i.e.
-     * it resets the 'other' side.
-     */
-    void mergeKeepLastRefs(VarRefsMap&& other) {
-        mergeLastRefsImpl(std::move(other), true, false);
-    }
-
-    /**
-     * Merges variable references from 'other' and keeps the last references from both sides.
-     */
-    void unionLastRefs(VarRefsMap&& other) {
-        mergeLastRefsImpl(std::move(other), false, false);
-    }
-
-    void mergeLastRefsImpl(VarRefsMap&& other, bool resetOther, bool resetBoth) {
-        for (auto otherIt = other.begin(), end = other.end(); otherIt != end;) {
-            if (auto localIt = varLastRefs.find(otherIt->first); localIt != varLastRefs.end()) {
-                // This variable is referenced in both sets. If requested, set other's last refs to
-                // false.
-                if (resetOther) {
-                    for (auto& [k, isLastRef] : otherIt->second) {
-                        isLastRef = false;
-                    }
-                }
-
-                // Merge the maps.
-                localIt->second.merge(otherIt->second);
-
-                // This variable is referenced in both sets. If requested, set all last refs for the
-                // variable to false.
-                if (resetBoth) {
-                    for (auto& [k, isLastRef] : localIt->second) {
-                        isLastRef = false;
-                    }
-                }
-                other.erase(otherIt++);
-            } else {
-                ++otherIt;
-            }
-        }
-        varLastRefs.merge(other);
-        tassert(6624098, "varLastRefs must be empty", other.empty());
-    }
-
-    /**
-     * Records collected last variable references for a specific variable. Should only be called
-     * when the variable is guaranteed not to be referenced again in the ABT.
-     */
-    void finalizeLastRefs(const ProjectionName& name) {
-        if (auto it = varLastRefs.find(name); it != varLastRefs.end()) {
-            for (auto& [var, isLastRef] : it->second) {
-                if (isLastRef) {
-                    lastRefs.emplace(var);
-                }
-            }
-
-            // After the finalization the map is not needed anymore.
-            varLastRefs.erase(it);
-        }
     }
 
     /**
@@ -224,7 +178,7 @@ struct CollectedInfo {
     void resolveFreeVars(const ProjectionName& name, const Definition& def) {
         if (auto it = freeVars.find(name); it != freeVars.end()) {
             for (const Variable& var : it->second) {
-                useMap.emplace(&var, def);
+                collector.resolvedVariablesMap->emplace(&var, def);
             }
             freeVars.erase(it);
         }
@@ -271,7 +225,9 @@ private:
 
 struct Collector {
     explicit Collector(const cascades::MemoGroupBinderInterface* memoInterface)
-        : _memoInterface(memoInterface) {}
+        : _memoInterface(memoInterface) {
+        collectorState.resolvedVariablesMap = std::make_unique<ResolvedVariablesMap>();
+    }
 
     template <typename T, typename... Ts>
     CollectedInfo transport(const ABT&, const T& op, Ts&&... ts) {
@@ -279,21 +235,18 @@ struct Collector {
 
         // The default behavior resolves free variables, merges known definitions and propagates
         // them up unmodified.
-        CollectedInfo result{};
+        CollectedInfo result{collectorState};
         (result.merge(std::forward<Ts>(ts)), ...);
 
         return result;
     }
 
     CollectedInfo transport(const ABT& n, const Variable& variable) {
-        CollectedInfo result{};
+        CollectedInfo result{collectorState};
 
         // Every variable starts as a free variable until it is resolved.
         result.freeVars[variable.name()].push_back(variable);
 
-        // Similarly, every variable starts as the last reference until proven otherwise.
-        result.varLastRefs[variable.name()].emplace(&variable, true);
-
         return result;
     }
 
@@ -301,13 +254,7 @@ struct Collector {
                             const Let& let,
                             CollectedInfo bindResult,
                             CollectedInfo inResult) {
-        CollectedInfo result{};
-
-        // The 'in' portion of the Let will execute after the bind, so its last refs should be kept
-        // over the last refs from the bind. Then, its safe to finalize the last ref for the
-        // variable defined by the Let here (we know it can't be referenced elsewhere in the ABT).
-        inResult.mergeKeepLastRefs(std::move(bindResult.varLastRefs));
-        inResult.finalizeLastRefs(let.varName());
+        CollectedInfo result{collectorState};
 
         result.merge(std::move(bindResult));
 
@@ -320,10 +267,7 @@ struct Collector {
     }
 
     CollectedInfo transport(const ABT& n, const LambdaAbstraction& lam, CollectedInfo inResult) {
-        CollectedInfo result{};
-
-        // As in the Let case, we can finalize the last ref for the local variable.
-        inResult.finalizeLastRefs(lam.varName());
+        CollectedInfo result{collectorState};
 
         // Local variables are not part of projections (i.e. we do not track them in defs) so
         // resolve any free variables manually.
@@ -333,32 +277,11 @@ struct Collector {
         return result;
     }
 
-    CollectedInfo transport(const ABT& n,
-                            const If&,
-                            CollectedInfo condResult,
-                            CollectedInfo thenResult,
-                            CollectedInfo elseResult) {
-        CollectedInfo result{};
-
-        // Only one of the 'then' or 'else' will be executed, so it's safe to union the last refs.
-        // Since the condition will be executed before either of the then/else, its last refs should
-        // be reset if there's a collision.
-        result.unionLastRefs(std::move(thenResult.varLastRefs));
-        result.unionLastRefs(std::move(elseResult.varLastRefs));
-        result.mergeKeepLastRefs(std::move(condResult.varLastRefs));
-
-        result.merge(std::move(condResult));
-        result.merge(std::move(thenResult));
-        result.merge(std::move(elseResult));
-
-        return result;
-    }
-
     static CollectedInfo collectForScan(const ABT& n,
                                         const Node& node,
                                         const ExpressionBinder& binder,
                                         CollectedInfo refs) {
-        CollectedInfo result{};
+        CollectedInfo result{refs.collector};
 
         // 'refs' should just track references to projections from any children of a Scan/Seek.
         result.mergeNoDefs(std::move(refs));
@@ -372,21 +295,21 @@ struct Collector {
     }
 
     CollectedInfo transport(const ABT& n, const ScanNode& node, CollectedInfo /*bindResult*/) {
-        return collectForScan(n, node, node.binder(), {});
+        return collectForScan(n, node, node.binder(), {collectorState});
     }
 
     CollectedInfo transport(const ABT& n, const ValueScanNode& node, CollectedInfo /*bindResult*/) {
-        return collectForScan(n, node, node.binder(), {});
+        return collectForScan(n, node, node.binder(), {collectorState});
     }
 
     CollectedInfo transport(const ABT& n,
                             const PhysicalScanNode& node,
                             CollectedInfo /*bindResult*/) {
-        return collectForScan(n, node, node.binder(), {});
+        return collectForScan(n, node, node.binder(), {collectorState});
     }
 
     CollectedInfo transport(const ABT& n, const IndexScanNode& node, CollectedInfo /*bindResult*/) {
-        return collectForScan(n, node, node.binder(), {});
+        return collectForScan(n, node, node.binder(), {collectorState});
     }
 
     CollectedInfo transport(const ABT& n,
@@ -397,14 +320,14 @@ struct Collector {
     }
 
     CollectedInfo transport(const ABT& n, const CoScanNode& node) {
-        CollectedInfo result{};
+        CollectedInfo result{collectorState};
         result.nodeDefs[&node] = result.defs;
         return result;
     }
 
     CollectedInfo transport(const ABT& n,
                             const MemoLogicalDelegatorNode& memoLogicalDelegatorNode) {
-        CollectedInfo result{};
+        CollectedInfo result{collectorState};
 
         tassert(6624029, "Uninitialized memo interface", _memoInterface);
         const auto& binder =
@@ -429,7 +352,7 @@ struct Collector {
                             const FilterNode& filterNode,
                             CollectedInfo childResult,
                             CollectedInfo exprResult) {
-        CollectedInfo result{};
+        CollectedInfo result{collectorState};
         result.merge(std::move(childResult));
         result.mergeNoDefs(std::move(exprResult));
         result.nodeDefs[&filterNode] = result.defs;
@@ -440,7 +363,7 @@ struct Collector {
                             const EvaluationNode& evaluationNode,
                             CollectedInfo childResult,
                             CollectedInfo exprResult) {
-        CollectedInfo result{};
+        CollectedInfo result{collectorState};
 
         tassert(6624030,
                 str::stream() << "Cannot overwrite project " << evaluationNode.getProjectionName(),
@@ -463,7 +386,7 @@ struct Collector {
                             CollectedInfo childResult,
                             CollectedInfo bindResult,
                             CollectedInfo /*refResult*/) {
-        CollectedInfo result{};
+        CollectedInfo result{collectorState};
 
         result.merge(std::move(childResult));
         result.mergeNoDefs(std::move(bindResult));
@@ -479,11 +402,11 @@ struct Collector {
         return result;
     }
 
-    template <class T>
-    CollectedInfo handleRIDNodeReferences(const T& node,
-                                          CollectedInfo leftChildResult,
-                                          CollectedInfo rightChildResult) {
-        CollectedInfo result{};
+    CollectedInfo transport(const ABT& n,
+                            const RIDIntersectNode& node,
+                            CollectedInfo leftChildResult,
+                            CollectedInfo rightChildResult) {
+        CollectedInfo result{collectorState};
 
         // This is a special case where both children of 'node' have a definition for the scan
         // projection. Remove the definition from one side to avoid running into the conflict of two
@@ -498,21 +421,75 @@ struct Collector {
         return result;
     }
 
-    CollectedInfo transport(const ABT& n,
-                            const RIDIntersectNode& node,
-                            CollectedInfo leftChildResult,
-                            CollectedInfo rightChildResult) {
-        return handleRIDNodeReferences(
-            node, std::move(leftChildResult), std::move(rightChildResult));
+    // Handle the collected info for UnionNode and RIDUnionNode, as both of them have their own
+    // binders and references.
+    CollectedInfo handleUnionCollectedInfo(ABT::reference_type nodeRef,
+                                           std::vector<CollectedInfo> childResults,
+                                           CollectedInfo bindResult,
+                                           CollectedInfo refsResult,
+                                           const ExpressionBinder& binder) {
+        CollectedInfo result{collectorState};
+
+        refsResult.assertEmptyDefs();
+
+        const auto& names = binder.names();
+
+        // Merge children but disregard any defined projections.
+        // Note that refsResult follows the structure as built by buildUnionTypeReferences, meaning
+        // it contains a free variable for each name for each child of the union and no other info.
+        size_t counter = 0;
+        for (auto& u : childResults) {
+            // Manually copy and resolve references of specific child. We do this manually
+            // because each Variable must be resolved by the appropriate child's definition.
+            for (const auto& name : names) {
+                tassert(7858802,
+                        str::stream() << "Union projection does not exist:  " << name,
+                        u.defs.count(name) != 0);
+                collectorState.resolvedVariablesMap->emplace(
+                    &refsResult.freeVars[name][counter].get(), u.defs[name]);
+            }
+            u.defs.clear();
+            result.merge(std::move(u));
+            ++counter;
+        }
+
+        result.mergeNoDefs(std::move(bindResult));
+
+        // Propagate union projections. Note that these are the only defs propagated, since we clear
+        // the child defs before merging above.
+        const auto& defs = binder.exprs();
+        for (size_t idx = 0; idx < names.size(); ++idx) {
+            result.defs[names[idx]] = Definition{nodeRef, defs[idx].ref()};
+        }
+
+        return result;
     }
 
     CollectedInfo transport(const ABT& n,
                             const RIDUnionNode& node,
                             CollectedInfo leftChildResult,
-                            CollectedInfo rightChildResult) {
-        // TODO SERVER-69026 should determine how the reference tracker for RIDUnionNode will work.
-        return handleRIDNodeReferences(
-            node, std::move(leftChildResult), std::move(rightChildResult));
+                            CollectedInfo rightChildResult,
+                            CollectedInfo bindResult,
+                            CollectedInfo refsResult) {
+        std::vector<CollectedInfo> childResults{std::move(leftChildResult),
+                                                std::move(rightChildResult)};
+
+        CollectedInfo result = handleUnionCollectedInfo(n.ref(),
+                                                        std::move(childResults),
+                                                        std::move(bindResult),
+                                                        std::move(refsResult),
+                                                        node.binder());
+
+        // We should always preserve the scanDef projection name.
+        const auto& scanProjName = node.getScanProjectionName();
+        tassert(7858800,
+                str::stream() << "The scanDef projection name has to be preserved: "
+                              << scanProjName,
+                result.defs.contains(scanProjName));
+
+        result.nodeDefs[&node] = result.defs;
+
+        return result;
     }
 
     template <class T>
@@ -520,7 +497,7 @@ struct Collector {
                                                 CollectedInfo leftChildResult,
                                                 CollectedInfo rightChildResult,
                                                 CollectedInfo filterResult) {
-        CollectedInfo result{};
+        CollectedInfo result{collectorState};
 
         // Note correlated projections might be coming either from the left child or from the
         // parent.
@@ -576,7 +553,7 @@ struct Collector {
                             CollectedInfo leftChildResult,
                             CollectedInfo rightChildResult,
                             CollectedInfo refsResult) {
-        CollectedInfo result{};
+        CollectedInfo result{collectorState};
 
         result.merge(std::move(leftChildResult));
         // Do not resolve further free variables.
@@ -593,7 +570,7 @@ struct Collector {
                             CollectedInfo leftChildResult,
                             CollectedInfo rightChildResult,
                             CollectedInfo refsResult) {
-        CollectedInfo result{};
+        CollectedInfo result{collectorState};
 
         result.merge(std::move(leftChildResult));
         // Do not resolve further free variables.
@@ -610,7 +587,7 @@ struct Collector {
                             std::vector<CollectedInfo> childResults,
                             CollectedInfo bindResult,
                             CollectedInfo refsResult) {
-        CollectedInfo result{};
+        CollectedInfo result{collectorState};
 
         const auto& names = node.binder().names();
 
@@ -628,7 +605,8 @@ struct Collector {
                 tassert(7063706,
                         str::stream() << "SortedMerge projection does not exist: " << name,
                         u.defs.count(name) != 0);
-                u.useMap.emplace(&refsResult.freeVars[name][counter].get(), u.defs[name]);
+                collectorState.resolvedVariablesMap->emplace(
+                    &refsResult.freeVars[name][counter].get(), u.defs[name]);
             }
             u.defs.clear();
             result.merge(std::move(u));
@@ -655,38 +633,11 @@ struct Collector {
                             std::vector<CollectedInfo> childResults,
                             CollectedInfo bindResult,
                             CollectedInfo refsResult) {
-        CollectedInfo result{};
-
-        const auto& names = unionNode.binder().names();
-
-        refsResult.assertEmptyDefs();
-
-        // Merge children but disregard any defined projections.
-        // Note that refsResult follows the structure as built by buildUnionTypeReferences, meaning
-        // it contains a free variable for each name for each child of the union and no other info.
-        size_t counter = 0;
-        for (auto& u : childResults) {
-            // Manually copy and resolve references of specific child. We do this manually because
-            // each Variable must be resolved by the appropriate child's definition.
-            for (const auto& name : names) {
-                tassert(6624031,
-                        str::stream() << "Union projection does not exist:  " << name,
-                        u.defs.count(name) != 0);
-                u.useMap.emplace(&refsResult.freeVars[name][counter].get(), u.defs[name]);
-            }
-            u.defs.clear();
-            result.merge(std::move(u));
-            ++counter;
-        }
-
-        result.mergeNoDefs(std::move(bindResult));
-
-        // Propagate union projections. Note that these are the only defs propagated, since we clear
-        // the child defs before merging above.
-        const auto& defs = unionNode.binder().exprs();
-        for (size_t idx = 0; idx < names.size(); ++idx) {
-            result.defs[names[idx]] = Definition{n.ref(), defs[idx].ref()};
-        }
+        CollectedInfo result = handleUnionCollectedInfo(n.ref(),
+                                                        std::move(childResults),
+                                                        std::move(bindResult),
+                                                        std::move(refsResult),
+                                                        unionNode.binder());
 
         result.nodeDefs[&unionNode] = result.defs;
 
@@ -700,7 +651,7 @@ struct Collector {
                             CollectedInfo refsAggResult,
                             CollectedInfo bindGbResult,
                             CollectedInfo refsGbResult) {
-        CollectedInfo result{};
+        CollectedInfo result{collectorState};
 
         // First resolve all variables from the inside point of view; i.e. agg expressions and group
         // by expressions reference variables from the input child.
@@ -744,7 +695,7 @@ struct Collector {
                             CollectedInfo childResult,
                             CollectedInfo bindResult,
                             CollectedInfo refsResult) {
-        CollectedInfo result{};
+        CollectedInfo result{collectorState};
 
         // First resolve all variables from the inside point of view.
         result.mergeNoDefs(std::move(refsResult));
@@ -772,7 +723,7 @@ struct Collector {
                             const UniqueNode& uniqueNode,
                             CollectedInfo childResult,
                             CollectedInfo refsResult) {
-        CollectedInfo result{};
+        CollectedInfo result{collectorState};
 
         result.merge(std::move(refsResult));
         result.merge(std::move(childResult));
@@ -792,7 +743,7 @@ struct Collector {
                             const CollationNode& collationNode,
                             CollectedInfo childResult,
                             CollectedInfo refsResult) {
-        CollectedInfo result{};
+        CollectedInfo result{collectorState};
 
         result.mergeNoDefs(std::move(refsResult));
         result.merge(std::move(childResult));
@@ -811,7 +762,7 @@ struct Collector {
     CollectedInfo transport(const ABT& n,
                             const LimitSkipNode& limitSkipNode,
                             CollectedInfo childResult) {
-        CollectedInfo result{};
+        CollectedInfo result{collectorState};
         result.merge(std::move(childResult));
         result.nodeDefs[&limitSkipNode] = result.defs;
         return result;
@@ -821,7 +772,7 @@ struct Collector {
                             const ExchangeNode& exchangeNode,
                             CollectedInfo childResult,
                             CollectedInfo refsResult) {
-        CollectedInfo result{};
+        CollectedInfo result{collectorState};
 
         result.mergeNoDefs(std::move(refsResult));
         result.merge(std::move(childResult));
@@ -840,7 +791,7 @@ struct Collector {
                             const RootNode& rootNode,
                             CollectedInfo childResult,
                             CollectedInfo refsResult) {
-        CollectedInfo result{};
+        CollectedInfo result{collectorState};
 
         result.mergeNoDefs(std::move(refsResult));
         result.merge(std::move(childResult));
@@ -862,7 +813,7 @@ struct Collector {
                             CollectedInfo filterResult,
                             CollectedInfo bindResult,
                             CollectedInfo refsResult) {
-        CollectedInfo result{};
+        CollectedInfo result{collectorState};
 
         result.merge(std::move(refsResult));
         result.merge(std::move(childResult));
@@ -887,42 +838,254 @@ struct Collector {
     }
 
     CollectedInfo transport(const ABT& n, const SpoolConsumerNode& node, CollectedInfo bindResult) {
-        return collectForScan(n, node, node.binder(), {});
+        return collectForScan(n, node, node.binder(), {collectorState});
     }
 
     CollectedInfo collect(const ABT& n) {
         return algebra::transport<true>(n, *this);
     }
+    /**
+     * The collector transport class stores here global information that is updated by
+     * some CollectedInfo methods. This object is passed to each CollectedInfo, so that
+     * it can update the collectorState.
+     */
+    CollectorState collectorState;
 
 private:
     const cascades::MemoGroupBinderInterface* _memoInterface;
 };
 
+/**
+ * Finds Variable references that are safe to mark moveFrom in SBE. See 'LastRefsSet'.
+ */
+struct LastRefsTransporter {
+    /**
+     * Maps each name that occurs free to the set of its last references.
+     * If a name has one or more free occurrences, but none are last, then the set is empty.
+     * If a name has no free occurrences at all, then there won't be a map entry.
+     *
+     * This distinction is important when combining results from two subtrees.
+     * For example, in 'let a = f(x) in g()', we know that 'x' does not occur in 'g()',
+     * so the 'x' in 'f(x)' is a last reference. On other hand in 'let a = f(x) in g(x, x)',
+     * 'x' does occur in 'g(x, x)', so the one in 'f(x)' is not a last reference--even if we don't
+     * know which use of 'x' in 'g(x, x)' is last.
+     */
+    using Result = ProjectionNameMap<LastRefsSet>;
+
+    explicit LastRefsTransporter(LastRefsSet& lastRefs) : _lastRefs(lastRefs) {}
+
+    /**
+     * Merge results, siphoning out r2 and putting results in r1.
+     *
+     * Variables that only occur on one side have their last-references preserved.
+     * Variables that occur on both sides have their last-references cleared, because we don't know
+     * which side is evaluated last.
+     */
+    static void merge(Result& result, Result& other) {
+        mergeImpl(result, other, false /*resetOther*/, true /*resetBoth*/);
+    }
+
+    /**
+     * Same as merge() but accepts a vector; this allows the generic transport() to handle Result
+     * and vector<Result> uniformly.
+     *
+     * Both of these overloads make no assumptions about evaluation order: if a variable occurs in
+     * more than one Result then all its last-references are cleared.
+     */
+    static void merge(Result& result, std::vector<Result>& others) {
+        for (auto& other : others) {
+            merge(result, other);
+        }
+    }
+
+    /**
+     * Like merge(), but assumes the left-hand side 'result' will be evaluated after 'other'.
+     *
+     * This means when a variable occurs on both sides, its last-references in 'result' can be
+     * preserved.
+     */
+    static void mergeKeepLastRefs(Result& result, Result& other) {
+        mergeImpl(result, other, true /*resetOther*/, false /*resetBoth*/);
+    }
+
+    /**
+     * Merges variable references from 'other' and keeps the last-references from both sides.
+     *
+     * This is appropriate when neither side is evaluated after the other. For example the two
+     * branches of a conditional.
+     */
+    static void unionLastRefs(Result& result, Result& other) {
+        mergeImpl(result, other, false /*resetOther*/, false /*resetBoth*/);
+    }
+
+    /**
+     * Combine all the occurrences from 'result' and 'other' into 'result'.
+     *
+     * The boolean flags control what to do when a variable occurs in both arguments:
+     * - 'resetOther' means preserve the last-references of 'result' but clear the ones in 'other'.
+     * - 'resetBoth' means clear the last-references flags of both 'result' and 'other'.
+     *
+     * If both flags are false then all last-references flags are preserved.
+     */
+    static void mergeImpl(Result& result, Result& other, bool resetOther, bool resetBoth) {
+        for (auto otherIt = other.begin(), end = other.end(); otherIt != end;) {
+            if (auto localIt = result.find(otherIt->first); localIt != result.end()) {
+                // This variable is referenced in both sets.
+
+                // If requested, stop treating occurrences in 'other' as last references.
+                if (resetOther) {
+                    otherIt->second.clear();
+                }
+
+                // Combine the last references from each side.
+                // Each side may contribute zero or more.
+                localIt->second.merge(otherIt->second);
+                other.erase(otherIt++);
+
+                // If requested, stop treating occurrences in the combined set as last references.
+                if (resetBoth) {
+                    localIt->second.clear();
+                }
+            } else {
+                // This variable is only referenced in 'other', so preserve its last-references.
+                // Preserve it by not erasing: we'll combine these not-erased entries at the end.
+                ++otherIt;
+            }
+        }
+        // Combine all the not-erased map entries.
+        result.merge(other);
+    }
+
+    /**
+     * Should be called once we know that we've seen all occurrences of a variable: removes all
+     * information about this variable from 'result', and records the last-references in the global
+     * set '_lastRefs'.
+     */
+    void finalizeLastRefs(Result& result, const ProjectionName& name) {
+        if (auto it = result.find(name); it != result.end()) {
+            _lastRefs.merge(it->second);
+            result.erase(it);
+        }
+    }
+
+    template <typename T>
+    Result transport(const ABT&, const T& op, auto... ts) {
+        if constexpr (std::is_base_of_v<Node, T>) {
+            // Since we currently only track last-references for local variables, we don't have to
+            // do anything for Nodes.
+            return {};
+        }
+
+        Result result{};
+        (merge(result, ts), ...);
+
+        return result;
+    }
+
+    Result transport(const ABT& n, const Variable& variable) {
+        Result result{};
+
+        // Every variable starts as a last reference until proven otherwise.
+        result[variable.name()].emplace(&variable);
+
+        return result;
+    }
+
+    Result transport(const ABT& n, const Let& let, Result bindResult, Result inResult) {
+        // The 'in' portion of the Let will execute after the bind, so its last refs should be kept
+        // over the last refs from the bind. Then, it's safe to finalize the last ref for the
+        // variable defined by the Let here (we know it can't be referenced elsewhere in the ABT).
+        mergeKeepLastRefs(inResult, bindResult);
+        finalizeLastRefs(inResult, let.varName());
+
+        return inResult;
+    }
+
+    Result transport(const ABT& n, const LambdaAbstraction& lam, Result inResult) {
+        // As in the Let case, we can finalize the last ref for the local variable.
+        finalizeLastRefs(inResult, lam.varName());
+
+        return inResult;
+    }
+
+    Result transport(
+        const ABT& n, const If&, Result condResult, Result thenResult, Result elseResult) {
+        Result result{};
+
+        // Only one of the 'then' or 'else' will be executed, so it's safe to union the last refs.
+        // Since the condition will be executed before either of the then/else, its last refs should
+        // be reset if there's a collision.
+        unionLastRefs(result, thenResult);
+        unionLastRefs(result, elseResult);
+        mergeKeepLastRefs(result, condResult);
+
+        return result;
+    }
+
+    void collect(const ABT& n) {
+        algebra::transport<true>(n, *this);
+    }
+
+private:
+    /**
+     * The set of Variable occurrences that are safe to move from.
+     *
+     * We only add to this map once a variable is resolved, because that's when we know we've seen
+     * all of its uses.
+     */
+    LastRefsSet& _lastRefs;
+};
+
 VariableEnvironment VariableEnvironment::build(
-    const ABT& root, const cascades::MemoGroupBinderInterface* memoInterface) {
+    const ABT& root,
+    const cascades::MemoGroupBinderInterface* memoInterface,
+    bool computeLastRefs) {
     Collector c(memoInterface);
     auto info = std::make_unique<CollectedInfo>(c.collect(root));
 
-    return VariableEnvironment{std::move(info), memoInterface};
+    boost::optional<LastRefsSet> lastRefs;
+    if (computeLastRefs) {
+        lastRefs.emplace();
+        LastRefsTransporter lrt(*lastRefs);
+        lrt.collect(root);
+    }
+
+    return VariableEnvironment{std::move(info),
+                               std::move(lastRefs),
+                               std::move(c.collectorState.resolvedVariablesMap),
+                               memoInterface};
 }
 
 void VariableEnvironment::rebuild(const ABT& root) {
-    _info = std::make_unique<CollectedInfo>(Collector{_memoInterface}.collect(root));
+    Collector c(_memoInterface);
+    _info = std::make_unique<CollectedInfo>(c.collect(root));
+
+    if (_lastRefs) {
+        _lastRefs->clear();
+        LastRefsTransporter lrt(*_lastRefs);
+        lrt.collect(root);
+    }
+
+    // Reset the Variable map to the newly computed one.
+    _resolvedVariablesMap = std::move(c.collectorState.resolvedVariablesMap);
 }
 
 VariableEnvironment::VariableEnvironment(std::unique_ptr<CollectedInfo> info,
+                                         boost::optional<LastRefsSet> lastRefs,
+                                         std::unique_ptr<ResolvedVariablesMap> resVarMap,
                                          const cascades::MemoGroupBinderInterface* memoInterface)
-    : _info(std::move(info)), _memoInterface(memoInterface) {}
+    : _info(std::move(info)),
+      _lastRefs(std::move(lastRefs)),
+      _resolvedVariablesMap(std::move(resVarMap)),
+      _memoInterface(memoInterface) {}
 
 VariableEnvironment::~VariableEnvironment() {}
 
 Definition VariableEnvironment::getDefinition(const Variable& var) const {
-    auto it = _info->useMap.find(&var);
-    if (it == _info->useMap.end()) {
-        return Definition();
+    if (auto it = _resolvedVariablesMap->find(&var); it != _resolvedVariablesMap->cend()) {
+        return it->second;
     }
-
-    return it->second;
+    return {};
 }
 
 const DefinitionsMap& VariableEnvironment::getDefinitions(const Node& node) const {
@@ -981,7 +1144,7 @@ size_t VariableEnvironment::freeOccurences(const ProjectionName& variable) const
 }
 
 bool VariableEnvironment::isLastRef(const Variable& var) const {
-    return _info->lastRefs.count(&var) > 0;
+    return _lastRefs && _lastRefs->contains(&var);
 }
 
 void VariableEnvironment::walkVariables(
