@@ -29,7 +29,13 @@
 
 #pragma once
 
+#include <vector>
+
+#include "mongo/db/query/optimizer/containers.h"
 #include "mongo/db/query/optimizer/reference_tracker.h"
+#include "mongo/db/query/optimizer/syntax/expr.h"
+#include "mongo/db/query/optimizer/syntax/path.h"
+#include "mongo/db/query/optimizer/syntax/syntax.h"
 
 namespace mongo::optimizer {
 
