@@ -29,11 +29,23 @@
 
 #pragma once
 
-#include "mongo/platform/decimal128.h"
+#include <absl/container/flat_hash_map.h>
+#include <cstdint>
 #include <ostream>
+#include <string>
+#include <type_traits>
+#include <utility>
+#include <vector>
 
+#include "mongo/base/string_data.h"
+#include "mongo/bson/timestamp.h"
 #include "mongo/db/exec/sbe/values/value.h"
+#include "mongo/db/query/optimizer/comparison_op.h"
+#include "mongo/db/query/optimizer/defs.h"
 #include "mongo/db/query/optimizer/syntax/syntax.h"
+#include "mongo/platform/decimal128.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/time_support.h"
 
 namespace mongo::optimizer {
 
@@ -63,6 +75,16 @@ public:
     static ABT emptyObject();
     static ABT emptyArray();
 
+    static ABT array(auto&&... elements) {
+        using namespace sbe::value;
+        auto [tag, val] = makeNewArray();
+        auto arr = getArrayView(val);
+        (arr->push_back(std::forward<decltype(elements)>(elements).first,
+                        std::forward<decltype(elements)>(elements).second),
+         ...);
+        return make<Constant>(tag, val);
+    }
+
     static ABT timestamp(const Timestamp& t);
     static ABT date(const Date_t& d);
 
@@ -206,9 +228,17 @@ public:
         return get<0>();
     }
 
+    ABT& getLeftChild() {
+        return get<0>();
+    }
+
     const ABT& getRightChild() const {
         return get<1>();
     }
+
+    ABT& getRightChild() {
+        return get<1>();
+    }
 };
 
 /**
@@ -234,13 +264,25 @@ public:
         return get<0>();
     }
 
+    ABT& getCondChild() {
+        return get<0>();
+    }
+
     const ABT& getThenChild() const {
         return get<1>();
     }
 
+    ABT& getThenChild() {
+        return get<1>();
+    }
+
     const ABT& getElseChild() const {
         return get<2>();
     }
+
+    ABT& getElseChild() {
+        return get<2>();
+    }
 };
 
 /**
@@ -271,9 +313,18 @@ public:
         return get<0>();
     }
 
+    ABT& bind() {
+        return get<0>();
+    }
+
+
     const ABT& in() const {
         return get<1>();
     }
+
+    ABT& in() {
+        return get<1>();
+    }
 };
 
 /**
@@ -331,9 +382,17 @@ public:
         return get<0>();
     }
 
+    ABT& getLambda() {
+        return get<0>();
+    }
+
     const ABT& getArgument() const {
         return get<1>();
     }
+
+    ABT& getArgument() {
+        return get<1>();
+    }
 };
 
 /**
@@ -393,6 +452,10 @@ public:
     const ABT& getInput() const {
         return get<1>();
     }
+
+    ABT& getInput() {
+        return get<1>();
+    }
 };
 
 /**
