@@ -29,8 +29,32 @@
 
 #include "mongo/db/query/optimizer/utils/abt_hash.h"
 
-#include "mongo/db/query/optimizer/node.h"
+#include <cstdint>
+#include <functional>
+#include <set>
+#include <string>
+#include <type_traits>
+#include <utility>
+#include <vector>
+
+#include <absl/container/node_hash_map.h>
+#include <absl/hash/hash.h>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/db/exec/sbe/makeobj_spec.h"
+#include "mongo/db/exec/sbe/values/value.h"
+#include "mongo/db/query/optimizer/algebra/operator.h"
+#include "mongo/db/query/optimizer/algebra/polyvalue.h"
+#include "mongo/db/query/optimizer/comparison_op.h"
+#include "mongo/db/query/optimizer/defs.h"
+#include "mongo/db/query/optimizer/index_bounds.h"
+#include "mongo/db/query/optimizer/node.h"  // IWYU pragma: keep
+#include "mongo/db/query/optimizer/partial_schema_requirements.h"
+#include "mongo/db/query/optimizer/syntax/expr.h"
+#include "mongo/db/query/optimizer/syntax/path.h"
+#include "mongo/db/query/optimizer/utils/strong_alias.h"
 #include "mongo/db/query/optimizer/utils/utils.h"
+#include "mongo/util/assert_util.h"
 
 namespace mongo::optimizer {
 
@@ -75,19 +99,12 @@ static void updateBoundHash(size_t& result, const BoundRequirement& bound) {
     updateHash(result, ABTHashGenerator::generate(bound.getBound()));
 };
 
-static void updateCompoundBoundHash(size_t& result, const CompoundBoundRequirement& bound) {
-    updateHash(result, std::hash<bool>()(bound.isInclusive()));
-    for (const auto& expr : bound.getBound()) {
-        updateHash(result, ABTHashGenerator::generate(expr));
-    }
-}
-
 template <class T>
 class BoolExprHasher {
 public:
     size_t transport(const typename T::Atom& node) {
         // Dispatch to one of the computeHash functions below.
-        return computeHash(node.getExpr());
+        return ABTHashGenerator::generate(node.getExpr());
     }
 
     size_t transport(const typename T::Conjunction& node, const std::vector<size_t> childResults) {
@@ -111,21 +128,14 @@ public:
     }
 };
 
-static size_t computeHash(const IntervalRequirement& req) {
+size_t ABTHashGenerator::generate(const IntervalRequirement& req) {
     size_t result = 17;
     updateBoundHash(result, req.getLowBound());
     updateBoundHash(result, req.getHighBound());
     return 17;
 }
 
-static size_t computeHash(const CompoundIntervalRequirement& req) {
-    size_t result = 19;
-    updateCompoundBoundHash(result, req.getLowBound());
-    updateCompoundBoundHash(result, req.getHighBound());
-    return result;
-}
-
-static size_t computeHash(const PartialSchemaEntry& entry) {
+size_t ABTHashGenerator::generate(const PartialSchemaEntry& entry) {
     const auto& [key, req] = entry;
 
     size_t result = 17;
@@ -146,9 +156,9 @@ static size_t computeHash(const PartialSchemaEntry& entry) {
     return result;
 }
 
-static size_t computePartialSchemaReqHash(const PartialSchemaRequirements& reqMap) {
+size_t ABTHashGenerator::generate(const PSRExpr::Node& reqMap) {
     BoolExprHasher<PSRExpr> psrHasher;
-    return psrHasher.compute(reqMap.getRoot());
+    return psrHasher.compute(reqMap);
 }
 
 /**
@@ -208,7 +218,7 @@ public:
                      size_t /*refResult*/) {
         // Specifically not hashing the candidate indexes and ScanParams. Those are derivative of
         // the requirements, and can have temp projection names.
-        return computeHashSeq<44>(computePartialSchemaReqHash(node.getReqMap()),
+        return computeHashSeq<44>(ABTHashGenerator::generate(node.getReqMap()),
                                   std::hash<IndexReqTarget>()(node.getTarget()),
                                   childResult);
     }
@@ -222,10 +232,16 @@ public:
                                   rightChildResult);
     }
 
-    size_t transport(const RIDUnionNode& node, size_t leftChildResult, size_t rightChildResult) {
+    size_t transport(const RIDUnionNode& node,
+                     size_t leftChildResult,
+                     size_t rightChildResult,
+                     size_t bindResult,
+                     size_t refsResult) {
         return computeHashSeq<47>(ProjectionName::Hasher()(node.getScanProjectionName()),
                                   leftChildResult,
-                                  rightChildResult);
+                                  rightChildResult,
+                                  bindResult,
+                                  refsResult);
     }
 
     size_t transport(const BinaryJoinNode& node,
@@ -292,6 +308,16 @@ public:
 
     size_t transport(const Constant& expr) {
         auto [tag, val] = expr.get();
+        // Extended types cannot be used directly with hashValue().
+        if (tag != sbe::value::TypeTags::Nothing && sbe::value::tagToType(tag) == BSONType::EOO) {
+            if (tag == sbe::value::TypeTags::makeObjSpec) {
+                return computeHashSeq<17>(
+                    absl::Hash<sbe::MakeObjSpec>{}(*sbe::value::getMakeObjSpecView(val)));
+            }
+
+            // Extended types must implement their own hash functions.
+            MONGO_UNREACHABLE_TASSERT(7936706);
+        }
         return computeHashSeq<17>(sbe::value::hashValue(tag, val));
     }
 
@@ -462,6 +488,11 @@ public:
         return computeHashSeq<7>(CEType::Hasher()(prop.getEstimate()));
     }
 
+    size_t operator()(const properties::PhysProperty&,
+                      const properties::RemoveOrphansRequirement& prop) {
+        return computeHashSeq<8>(std::hash<bool>()(prop.mustRemove()));
+    }
+
     static size_t computeHash(const properties::PhysProps& props) {
         PhysPropsHasher visitor;
         size_t result = 17;
