@@ -29,25 +29,30 @@
 
 #include "mongo/db/query/optimizer/utils/interval_utils.h"
 
-#include "mongo/db/exec/sbe/values/value.h"
-#include "mongo/db/query/optimizer/node.h"
+#include <absl/container/node_hash_map.h>
+#include <algorithm>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional.hpp>
+#include <iterator>
+#include <memory>
+#include <utility>
+
+#include <boost/optional/optional.hpp>
+
+#include "mongo/db/query/optimizer/algebra/operator.h"
+#include "mongo/db/query/optimizer/algebra/polyvalue.h"
+#include "mongo/db/query/optimizer/bool_expression.h"
+#include "mongo/db/query/optimizer/comparison_op.h"
+#include "mongo/db/query/optimizer/node.h"  // IWYU pragma: keep
+#include "mongo/db/query/optimizer/syntax/expr.h"
+#include "mongo/db/query/optimizer/syntax/syntax.h"
 #include "mongo/db/query/optimizer/utils/abt_compare.h"
+#include "mongo/util/assert_util.h"
 
 
 namespace mongo::optimizer {
 
-// Three way comparison of the two arguments. Returns boost::none if nothing can be determined
-// about the comparison between the two arguments.
-static boost::optional<int> cmpABT(const ABT& a1, const ABT& a2, const ConstFoldFn& constFold) {
-    ABT result = make<BinaryOp>(Operations::Cmp3w, a1, a2);
-    constFold(result);
-    if (const auto resultConst = result.cast<Constant>()) {
-        invariant(resultConst->isValueInt32());
-        return resultConst->getValueInt32();
-    }
-    return boost::none;
-};
-
 ABT minABT(const ABT& v1, const ABT& v2) {
     return make<If>(make<BinaryOp>(Operations::Lte, v1, v2), v1, v2);
 };
@@ -57,35 +62,31 @@ ABT maxABT(const ABT& v1, const ABT& v2) {
 };
 
 void constFoldInterval(IntervalRequirement& interval, const ConstFoldFn& constFold) {
-    ABT low = interval.getLowBound().getBound();
-    ABT high = interval.getHighBound().getBound();
-    constFold(low);
-    constFold(high);
-    interval = IntervalRequirement{
-        BoundRequirement{interval.getLowBound().isInclusive(), std::move(low)},
-        BoundRequirement{interval.getHighBound().isInclusive(), std::move(high)},
-    };
+    constFold(interval.getLowBound().getBound());
+    constFold(interval.getHighBound().getBound());
 }
 
 // Returns true if the interval can be proven to be empty. If no conclusion can be made, or the
 // interval is provably not empty, returns false.
-bool isIntervalEmpty(const IntervalRequirement& interval, const ConstFoldFn& constFold) {
+
+bool isIntervalEmpty(const IntervalRequirement& interval) {
     if (interval.getLowBound() == BoundRequirement{false, Constant::maxKey()} ||
         interval.getHighBound() == BoundRequirement{false, Constant::minKey()}) {
         return true;
     }
-    const auto boundsCmp =
-        cmpABT(interval.getLowBound().getBound(), interval.getHighBound().getBound(), constFold);
+
+    CmpResult boundsCmp = cmp3wFast(
+        Operations::Cmp3w, interval.getLowBound().getBound(), interval.getHighBound().getBound());
     // Can't make any conclusions about the comparison between the bounds. We don't know for sure
     // that it's empty.
-    if (!boundsCmp) {
+    if (boundsCmp == CmpResult::kIncomparable) {
         return false;
     }
     const bool hasExclusiveBound =
         !interval.getLowBound().isInclusive() || !interval.getHighBound().isInclusive();
     // If lower bound greater than upper bound, or the bounds are equal but the interval is
     // not completely inclusive, we have an empty interval.
-    return *boundsCmp == 1 || (*boundsCmp == 0 && hasExclusiveBound);
+    return boundsCmp == CmpResult::kGt || (boundsCmp == CmpResult::kEq && hasExclusiveBound);
 }
 
 std::vector<IntervalRequirement> unionTwoIntervals(const IntervalRequirement& int1,
@@ -154,10 +155,10 @@ std::vector<IntervalRequirement> unionTwoIntervals(const IntervalRequirement& in
         {cInc, c}, {dInc, make<If>(std::move(overlapAndNonEmptyCond), Constant::minKey(), d)}};
     constFoldInterval(primaryInt1, constFold);
     constFoldInterval(primaryInt2, constFold);
-    if (!isIntervalEmpty(primaryInt1, constFold)) {
+    if (!isIntervalEmpty(primaryInt1)) {
         result.push_back(std::move(primaryInt1));
     }
-    if (!isIntervalEmpty(primaryInt2, constFold)) {
+    if (!isIntervalEmpty(primaryInt2)) {
         result.push_back(std::move(primaryInt2));
     }
 
@@ -187,7 +188,7 @@ std::vector<IntervalRequirement> unionTwoIntervals(const IntervalRequirement& in
     // Analyze an aux interval for const-ness or emptiness, and add it to our result.
     const auto addAuxInterval = [&](IntervalRequirement auxInterval) {
         constFoldInterval(auxInterval, constFold);
-        if (!isIntervalEmpty(auxInterval, constFold)) {
+        if (!isIntervalEmpty(auxInterval)) {
             if (auxInterval.isConstant()) {
                 invariant(auxInterval.isEquality());
                 // Find the primary interval and merge with it.
@@ -267,7 +268,7 @@ std::vector<IntervalRequirement> unionTwoIntervals(const IntervalRequirement& in
 
 boost::optional<IntervalReqExpr::Node> unionDNFIntervals(const IntervalReqExpr::Node& intervalDNF,
                                                          const ConstFoldFn& constFold) {
-    IntervalReqExpr::Builder builder;
+    BoolExprBuilder<IntervalRequirement> builder;
     builder.pushDisj();
 
     // Since our input intervals are sorted, constDisjIntervals will be sorted as well.
@@ -291,7 +292,7 @@ boost::optional<IntervalReqExpr::Node> unionDNFIntervals(const IntervalReqExpr::
 
     // Remove empty intervals.
     for (auto it = constDisjIntervals.begin(); it != constDisjIntervals.end();) {
-        if (isIntervalEmpty(*it, constFold)) {
+        if (isIntervalEmpty(*it)) {
             it = constDisjIntervals.erase(it);
         } else {
             it++;
@@ -357,7 +358,7 @@ void combineIntervalsDNF(const bool intersect,
         return;
     }
 
-    IntervalReqExpr::Builder builder;
+    BoolExprBuilder<IntervalRequirement> builder;
     builder.pushDisj();
 
     const auto pushConjNodesFn = [&builder](const IntervalReqExpr::Node& conj) {
@@ -429,8 +430,8 @@ static std::vector<IntervalRequirement> intersectIntervals(const IntervalRequire
     // are inclusive.
     ABT maxLow = foldFn(maxABT(low1, low2));
     ABT minHigh = foldFn(minABT(high1, high2));
-    if (foldFn(make<BinaryOp>(Operations::Gt, maxLow, minHigh)) == Constant::boolean(true)) {
-        // Low bound is greater than high bound.
+    if (cmp3wFast(Operations::Gt, maxLow, minHigh) == CmpResult::kTrue) {
+        //   Low bound is greater than high bound.
         return {};
     }
 
@@ -444,10 +445,8 @@ static std::vector<IntervalRequirement> intersectIntervals(const IntervalRequire
     BoundRequirement lowBoundPrimary(low1Inc && low2Inc, maxLow);
     BoundRequirement highBoundPrimary(high1Inc && high2Inc, minHigh);
 
-    const bool boundsEqual =
-        foldFn(make<BinaryOp>(Operations::Eq, std::move(maxLow), std::move(minHigh))) ==
-        Constant::boolean(true);
-    if (boundsEqual) {
+    auto boundsEqual = cmpEqFast(maxLow, minHigh);
+    if (boundsEqual == CmpResult::kTrue) {
         if (low1Inc && high1Inc && low2Inc && high2Inc) {
             // Point interval.
             return {{std::move(lowBoundPrimary), std::move(highBoundPrimary)}};
@@ -481,16 +480,15 @@ static std::vector<IntervalRequirement> intersectIntervals(const IntervalRequire
     const auto addAuxInterval = [&](ABT low, ABT high, BoundRequirement& bound) {
         IntervalRequirement interval{{true, low}, {true, high}};
 
-        const ABT comparison =
-            foldFn(make<BinaryOp>(Operations::Lte, std::move(low), std::move(high)));
-        if (comparison == Constant::boolean(true)) {
+        CmpResult comparison = cmp3wFast(Operations::Lte, low, high);
+        if (comparison == CmpResult::kTrue) {
             if (interval.isEquality()) {
                 // We can determine the two bounds are equal.
                 bound = {true /*inclusive*/, bound.getBound()};
             } else {
                 result.push_back(std::move(interval));
             }
-        } else if (!comparison.is<Constant>()) {
+        } else if (comparison == CmpResult::kIncomparable) {
             // We cannot determine statically how the two bounds compare.
             result.push_back(std::move(interval));
         }
@@ -580,9 +578,10 @@ static std::vector<IntervalRequirement> intersectIntervals(const IntervalRequire
         addAuxInterval(std::move(low), std::move(high), highBoundPrimary);
     }
 
-    if (!boundsEqual || (lowBoundPrimary.isInclusive() && highBoundPrimary.isInclusive())) {
-        // We add the main interval to the result as long as it is a valid point interval, or the
-        // bounds are not equal.
+    if (boundsEqual == CmpResult::kIncomparable || boundsEqual == CmpResult::kFalse ||
+        (lowBoundPrimary.isInclusive() && highBoundPrimary.isInclusive())) {
+        //  We add the main interval to the result as long as it is a valid point interval, or
+        //  the bounds are not equal.
         result.emplace_back(std::move(lowBoundPrimary), std::move(highBoundPrimary));
     }
     return result;
@@ -590,7 +589,29 @@ static std::vector<IntervalRequirement> intersectIntervals(const IntervalRequire
 
 boost::optional<IntervalReqExpr::Node> intersectDNFIntervals(
     const IntervalReqExpr::Node& intervalDNF, const ConstFoldFn& constFold) {
-    IntervalReqExpr::Builder<false /*simplifyEmptyOrSingular*/, true /*removeDups*/> builder;
+
+    struct IntervalComparator {
+        bool operator()(const IntervalReqExpr::Node& i1, const IntervalReqExpr::Node& i2) const {
+            return compareIntervalExpr(i1, i2) < 0;
+        }
+    };
+    struct IntervalSimplifier {
+        using DefaultSimplifier = DefaultSimplifyAndCreateNode<IntervalRequirement>;
+
+        DefaultSimplifier::Result operator()(const BuilderNodeType type,
+                                             std::vector<IntervalReqExpr::Node> v,
+                                             const bool hasTrue,
+                                             const bool hasFalse) const {
+            // Deduplicate via sort + unique.
+            std::sort(v.begin(), v.end(), IntervalComparator{});
+            auto end = std::unique(v.begin(), v.end());
+            v.erase(end, v.end());
+
+            return DefaultSimplifier{}(type, std::move(v), hasTrue, hasFalse);
+        }
+    };
+
+    BoolExprBuilder<IntervalRequirement, IntervalSimplifier> builder;
     builder.pushDisj();
 
     for (const auto& disjunct : intervalDNF.cast<IntervalReqExpr::Disjunction>()->nodes()) {
@@ -650,7 +671,7 @@ boost::optional<IntervalReqExpr::Node> simplifyDNFIntervals(const IntervalReqExp
 bool combineCompoundIntervalsDNF(CompoundIntervalReqExpr::Node& targetIntervals,
                                  const IntervalReqExpr::Node& sourceIntervals,
                                  bool reverseSource) {
-    CompoundIntervalReqExpr::Builder builder;
+    BoolExprBuilder<CompoundIntervalRequirement> builder;
     builder.pushDisj();
 
     for (const auto& sourceConjunction :
@@ -694,7 +715,7 @@ bool combineCompoundIntervalsDNF(CompoundIntervalReqExpr::Node& targetIntervals,
 
 void padCompoundIntervalsDNF(CompoundIntervalReqExpr::Node& targetIntervals,
                              const bool reverseSource) {
-    CompoundIntervalReqExpr::Builder builder;
+    BoolExprBuilder<CompoundIntervalRequirement> builder;
     builder.pushDisj();
 
     for (const auto& targetConjunction :
@@ -776,56 +797,6 @@ void normalizeIntervals(IntervalReqExpr::Node& intervals) {
     IntervalNormalizer{}.normalize(intervals);
 }
 
-boost::optional<ABT> coerceIntervalToPathCompareEqMember(const IntervalReqExpr::Node& interval) {
-    // Create the array that EqMember will use to hold the members.
-    auto [eqMembersTag, eqMembersVal] = sbe::value::makeNewArray();
-    sbe::value::ValueGuard guard{eqMembersTag, eqMembersVal};
-    auto eqMembersArray = sbe::value::getArrayView(eqMembersVal);
-
-    // An EqMember is a disjunction of conjunctions of atoms (point intervals). For example [1, 1] U
-    // [2, 2] U [3, 3] However each conjunction should only have one atom child, so we can think of
-    // it as a disjunction of point intervals instead.
-    if (const auto disj = interval.cast<IntervalReqExpr::Disjunction>()) {
-        // We only make an EqMember if we have 2 or more comparisons.
-        if (disj->nodes().size() < 2) {
-            return boost::none;
-        }
-
-        for (const auto& child : disj->nodes()) {
-            if (!child.is<IntervalReqExpr::Conjunction>()) {
-                return boost::none;
-            }
-
-            // Check that the conjunction has one atom child.
-            const auto conjChild = child.cast<IntervalReqExpr::Conjunction>();
-            if (conjChild->nodes().size() != 1 ||
-                !conjChild->nodes().front().is<IntervalReqExpr::Atom>()) {
-                return boost::none;
-            }
-
-            // Check that the atom is a point interval, and the bound is a constant.
-            const auto atomChild = conjChild->nodes().front().cast<IntervalReqExpr::Atom>();
-            if (!atomChild->getExpr().isEquality() ||
-                !atomChild->getExpr().getLowBound().getBound().is<Constant>()) {
-                return boost::none;
-            }
-
-            const auto constAtomChildPair =
-                atomChild->getExpr().getLowBound().getBound().cast<Constant>()->get();
-
-            // Make a copy of the point bound, insert it into our EqMember members.
-            const auto newEqMember = copyValue(constAtomChildPair.first, constAtomChildPair.second);
-            eqMembersArray->push_back(newEqMember.first, newEqMember.second);
-        }
-
-        // If we got to this point, we have successfully coerced the interval into an EqMember!
-        // Reset the guard so the members array doesn't get deleted.
-        guard.reset();
-        return make<PathCompare>(Operations::EqMember, make<Constant>(eqMembersTag, eqMembersVal));
-    }
-    return boost::none;
-}
-
 bool isSimpleRange(const CompoundIntervalReqExpr::Node& interval) {
     if (const auto singularInterval = CompoundIntervalReqExpr::getSingularDNF(interval);
         singularInterval && !singularInterval->isEquality()) {
@@ -834,4 +805,170 @@ bool isSimpleRange(const CompoundIntervalReqExpr::Node& interval) {
     return false;
 }
 
+class CheckMultikeyness {
+public:
+    bool walk(const PathIdentity& identity, const MultikeynessTrie& trie, bool seenParentTraverse) {
+        return trie.isMultiKey;
+    }
+
+    bool walk(const PathTraverse& traverse,
+              const MultikeynessTrie& trie,
+              bool seenParentTraverse,
+              const ABT& child,
+              const ABT& refs) {
+        // If there is a traverse node, we assume that the key is already multikey.
+
+        // If the traverse node is single level, and its immediate parent isn't another traverse
+        // node, then we can check the next node.
+        if (traverse.getMaxDepth() == PathTraverse::kSingleLevel && !seenParentTraverse) {
+            return algebra::walk<false>(child, *this, trie, true /*seenParentTraverse*/);
+        }
+
+        // The multikeynessTrie contains metadata only on consecutively nested fields.
+        // e.g., {a: [[ {b: [5]} ]]} the multikeyness trie would *correctly* respond that path
+        // "Get [a] Traverse [1] Get [b] Id" is non-multikey.
+        // Traverse [1] flattens only one level of nesting, and field "b" is nested in two levels.
+        //
+        // Currently, if the traverse node is multi level (Traverse [N], N > 1), or we have multiple
+        // subsequent traverse nodes, we assume that the path is multikey.
+
+        // We have no information about multikeyness of the child path, i.e., it is multikey.
+        return true;
+    }
+
+    bool walk(const PathGet& get,
+              const MultikeynessTrie& trie,
+              bool seenParentTraverse,
+              const ABT& child) {
+        // If there is no trie node for the PathGet field, then the field is multikey.
+        auto it = trie.children.find(get.name());
+        if (it == trie.children.end()) {
+            return true;
+        }
+
+        return algebra::walk<false>(child, *this, it->second, false /*seenParentTraverse*/);
+    }
+
+    template <class N, class... Ts>
+    bool walk(const N& node, Ts&&...) {
+        return true;
+    }
+
+    static bool checkMultikeyness(const ABT& path, const MultikeynessTrie& multikeynessTrie) {
+        CheckMultikeyness instance;
+        return algebra::walk<false>(path, instance, multikeynessTrie, false);
+    }
+};
+
+static bool requiresArray(const IntervalReqExpr::Node& intervals,
+                          const PathToIntervalFn& pathToInterval) {
+
+    // Check whether the requirement interval is a requirement for an array.
+
+    // Create a single requirement interval for an array.
+    IntervalReqExpr::Node arrayIntervalDNF = pathToInterval(make<PathArr>()).get();
+
+    // Intersect the single requirement for an array with the input interval requirements.
+    // This will update in-place 'arrayIntervalDNF', which depending on the intervals, will
+    // result to either a subset of a single requirement for an array or an empty set.
+    combineIntervalsDNF(true /*intersect*/, arrayIntervalDNF, intervals);
+
+    // Comparing the result of the intersection with the array requirement will return
+    // true only if the input requirements contain a subset of array requirement.
+    return intervals == arrayIntervalDNF;
+}
+
+bool requiresArrayOnNonMultikeyPath(const ABT& path,
+                                    const IntervalReqExpr::Node& intervals,
+                                    const MultikeynessTrie& multikeynessTrie,
+                                    const PathToIntervalFn& pathToInterval) {
+    return requiresArray(intervals, pathToInterval) &&
+        !CheckMultikeyness::checkMultikeyness(path, multikeynessTrie);
+}
+
+bool mayContainNull(const IntervalReqExpr::Atom& node, const ConstFoldFn& constFold) {
+    const auto& interval = node.getExpr();
+
+    const auto foldFn = [&constFold](ABT expr) {
+        constFold(expr);
+        return expr;
+    };
+    if (const auto& lowBound = interval.getLowBound();
+        foldFn(make<BinaryOp>(lowBound.isInclusive() ? Operations::Gt : Operations::Gte,
+                              lowBound.getBound(),
+                              Constant::null())) == Constant::boolean(true)) {
+        // Lower bound is strictly larger than null, or equal to null but not inclusive.
+        return false;
+    }
+    if (const auto& highBound = interval.getHighBound();
+        foldFn(make<BinaryOp>(highBound.isInclusive() ? Operations::Lt : Operations::Lte,
+                              highBound.getBound(),
+                              Constant::null())) == Constant::boolean(true)) {
+        // Upper bound is strictly smaller than null, or equal to null but not inclusive.
+        return false;
+    }
+
+    return true;
+}
+
+/**
+ * Transport that replaces the intervals containing nulls in-place in order to preserve the DNF/CNF
+ * structure. If 'excludeNull' is true, the interval is updated to (null, HighBound] (i.e.
+ * intersecting {>Const [null]}). Otherwise, it is updated to [LowBound, null] (i.e. intersecting
+ * {<=Const [null]}).
+ */
+class SplitNullTransport {
+public:
+    void transport(IntervalReqExpr::Node& n,
+                   const IntervalReqExpr::Atom& atom,
+                   const ConstFoldFn& constFold,
+                   const bool excludeNull) {
+        if (mayContainNull(atom, constFold)) {
+            const IntervalRequirement& expr = atom.getExpr();
+            if (excludeNull) {
+                n.cast<IntervalReqExpr::Atom>()->getExpr() = IntervalRequirement{
+                    {false /*inclusive*/, Constant::null()}, expr.getHighBound()};
+            } else {
+                n.cast<IntervalReqExpr::Atom>()->getExpr() =
+                    IntervalRequirement{expr.getLowBound(), {true /*inclusive*/, Constant::null()}};
+            }
+        }
+    }
+    void transport(IntervalReqExpr::Node& n,
+                   const IntervalReqExpr::Conjunction&,
+                   const ConstFoldFn&,
+                   const bool excludeNull,
+                   std::vector<IntervalReqExpr::Node>&) {}
+    void transport(IntervalReqExpr::Node& n,
+                   const IntervalReqExpr::Disjunction&,
+                   const ConstFoldFn&,
+                   const bool excludeNull,
+                   std::vector<IntervalReqExpr::Node>&) {}
+
+
+    void split(IntervalReqExpr::Node& n, const ConstFoldFn& constFold, const bool excludeNull) {
+        algebra::transport<true>(n, *this, constFold, excludeNull);
+    }
+};
+
+boost::optional<std::pair<IntervalReqExpr::Node, IntervalReqExpr::Node>> splitNull(
+    const IntervalReqExpr::Node& interval, const ConstFoldFn& constFold) {
+    // Requires all the intervals to be constant and does not have a high bound with null
+    // inclusively. Having a low bound with null is acceptable because it's beneficial to split
+    // [null, HighBound] into [null, null] and (null, HighBound].
+    if (IntervalReqExpr::any(interval, [](const IntervalRequirement& requirement) {
+            return !requirement.isConstant() ||
+                requirement.getHighBound() == BoundRequirement{true, Constant::null()};
+        })) {
+        return boost::none;
+    }
+    IntervalReqExpr::Node nullExcluded = interval;
+    IntervalReqExpr::Node nullIncluded = interval;
+
+    SplitNullTransport{}.split(nullExcluded, constFold, true /* excludeNull */);
+    SplitNullTransport{}.split(nullIncluded, constFold, false /* excludeNull */);
+
+    return std::make_pair(nullExcluded, nullIncluded);
+}
+
 }  // namespace mongo::optimizer
