@@ -29,8 +29,11 @@
 
 #pragma once
 
-#include "mongo/db/query/optimizer/index_bounds.h"
+#include <boost/optional/optional.hpp>
+#include <vector>
 
+#include "mongo/db/query/optimizer/index_bounds.h"
+#include "mongo/db/query/optimizer/utils/const_fold_interface.h"
 #include "mongo/db/query/optimizer/utils/utils.h"
 
 namespace mongo::optimizer {
@@ -123,12 +126,6 @@ void padCompoundIntervalsDNF(CompoundIntervalReqExpr::Node& targetIntervals, boo
  */
 void normalizeIntervals(IntervalReqExpr::Node& intervals);
 
-/**
- * Analyze the given interval, and convert it into a PathCompare EqMember if possible.
- */
-boost::optional<ABT> coerceIntervalToPathCompareEqMember(const IntervalReqExpr::Node& interval);
-
-
 /**
  * Returns true if the interval corresponds to a simple range (e.g >10 as opposed to a point
  * equality or complex boolean expression of intervals).
@@ -138,6 +135,28 @@ bool isSimpleRange(const CompoundIntervalReqExpr::Node& interval);
 /**
  * Returns true if after constant folding we can tell the interval never returns true.
  */
-bool isIntervalEmpty(const IntervalRequirement& interval, const ConstFoldFn& constFold);
+bool isIntervalEmpty(const IntervalRequirement& interval);
 
+/**
+ * Returns true if (i) the given interval contains a requirement for the path to be an array AND
+ * (ii) the multikeyness trie returns that the given path is a non-multikey path.
+ */
+bool requiresArrayOnNonMultikeyPath(const ABT& path,
+                                    const IntervalReqExpr::Node& intervals,
+                                    const MultikeynessTrie& multikeynessTrie,
+                                    const PathToIntervalFn& pathToInterval);
+
+/**
+ * Returns true if the interval may contain nulls.
+ */
+bool mayContainNull(const IntervalReqExpr::Atom& node, const ConstFoldFn& constFold);
+
+/**
+ * Splits the interval into two mutually exclusive intervals to separate out nulls. Returns a pair
+ * of intervals where the first element has nulls excluded by intersecting {>Const [null]}, and the
+ * second element has nulls included by intersecting {<=Const [null]}. Returns boost::none if the
+ * interval contains variable.
+ */
+boost::optional<std::pair<IntervalReqExpr::Node, IntervalReqExpr::Node>> splitNull(
+    const IntervalReqExpr::Node& interval, const ConstFoldFn& constFold);
 }  // namespace mongo::optimizer
