@@ -29,7 +29,29 @@
 
 #include "mongo/db/query/optimizer/utils/memo_utils.h"
 
+#include <absl/container/node_hash_map.h>
+#include <absl/container/node_hash_set.h>
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <cstddef>
+#include <cstdint>
+// IWYU pragma: no_include "ext/alloc_traits.h"
+#include <algorithm>
+#include <iterator>
+#include <memory>
+#include <string>
+#include <utility>
+
+#include "mongo/db/query/optimizer/algebra/operator.h"
+#include "mongo/db/query/optimizer/algebra/polyvalue.h"
 #include "mongo/db/query/optimizer/cascades/memo.h"
+#include "mongo/db/query/optimizer/cascades/memo_defs.h"
+#include "mongo/db/query/optimizer/containers.h"
+#include "mongo/db/query/optimizer/node.h"  // IWYU pragma: keep
+#include "mongo/db/query/optimizer/reference_tracker.h"
+#include "mongo/db/query/optimizer/syntax/expr.h"
+#include "mongo/db/query/optimizer/utils/utils.h"
+#include "mongo/util/assert_util.h"
 
 
 namespace mongo::optimizer {
@@ -41,14 +63,16 @@ using namespace properties;
  */
 class MemoLatestPlanExtractor {
 public:
-    explicit MemoLatestPlanExtractor(const cascades::Memo& memo) : _memo(memo) {}
+    explicit MemoLatestPlanExtractor(const cascades::Memo& memo, bool collectProperties = false)
+        : _memo(memo), _collectProperties(collectProperties) {}
 
     /**
      * Logical delegator node.
      */
     void transport(ABT& n,
                    const MemoLogicalDelegatorNode& node,
-                   opt::unordered_set<GroupIdType>& visitedGroups) {
+                   opt::unordered_set<GroupIdType>& visitedGroups,
+                   bool collectProperties) {
         n = extractLatest(node.getGroupId(), visitedGroups);
     }
 
@@ -73,12 +97,27 @@ public:
         }
 
         ABT rootNode = _memo.getLogicalNodes(groupId).back();
-        algebra::transport<true>(rootNode, *this, visitedGroups);
+        algebra::transport<true>(rootNode, *this, visitedGroups, _collectProperties);
+
+        if (_collectProperties) {
+            NodeProps entry;
+            entry._logicalProps = _memo.getLogicalProps(groupId);
+            _properties.emplace(rootNode.cast<Node>(), std::move(entry));
+        }
+
         return rootNode;
     }
 
+    const NodeToGroupPropsMap& getProperties() const {
+        return _properties;
+    }
+
 private:
     const cascades::Memo& _memo;
+
+    const bool _collectProperties;
+
+    NodeToGroupPropsMap _properties;
 };
 
 ABT extractLatestPlan(const cascades::Memo& memo, const GroupIdType rootGroupId) {
@@ -87,6 +126,17 @@ ABT extractLatestPlan(const cascades::Memo& memo, const GroupIdType rootGroupId)
     return extractor.extractLatest(rootGroupId, visitedGroups);
 }
 
+PlanAndProps extractLatestPlanAndProps(const cascades::Memo& memo, const GroupIdType rootGroupId) {
+    MemoLatestPlanExtractor extractor(memo, true /*collecProperties*/);
+    opt::unordered_set<GroupIdType> visitedGroups;
+
+    ABT plan = extractor.extractLatest(rootGroupId, visitedGroups);
+    NodeToGroupPropsMap properties = extractor.getProperties();
+    PlanAndProps result(std::move(plan), std::move(properties));
+    return result;
+}
+
+
 template <class T, class Accessor = DefaultChildAccessor<T>>
 static void mergeNodeAndProps(const bool aggregateCost,
                               PlanAndProps& merged,
@@ -273,6 +323,31 @@ public:
         return result;
     }
 
+    PlanExtractorResult operator()(const ABT& n,
+                                   const EvaluationNode& node,
+                                   const bool isGroupRoot,
+                                   ProjectionNameOrderPreservingSet required) {
+        ProjectionNameOrderPreservingSet requiredForChild = required;
+        requiredForChild.erase(node.getProjectionName());
+        auto env = VariableEnvironment::build(
+            node.getProjection(), nullptr /*memoInterface*/, false /*computeLastRefs*/);
+        for (const auto& proj : env.freeVariableNames()) {
+            requiredForChild.emplace_back(proj);
+        }
+
+        auto result =
+            node.getChild().visit(*this, false /*isGroupRoot*/, std::move(requiredForChild));
+
+        PlanAndProps initial = createInitial(isGroupRoot, n, std::move(required));
+        for (size_t index = 0; index < result.size(); index++) {
+            auto& entry = result.at(index);
+            PlanAndProps merged = moveOrCopy(initial, index == result.size() - 1);
+            mergeNodeAndProps<EvaluationNode>(_includeRejected, merged, entry, true /*canMove*/);
+            std::swap(entry, merged);
+        }
+        return result;
+    }
+
     template <class T>
     PlanExtractorResult operator()(const ABT& n,
                                    const T& node,
@@ -381,6 +456,11 @@ public:
             const auto& nodeInfo =
                 (altIndex == 0) ? *result._nodeInfo : result._rejectedNodeInfo.at(altIndex - 1);
             const ABT& node = nodeInfo._node;
+            if (nodeInfo._cost.isInfinite()) {
+                // Skip the node with infinity cost as that indicates the failure of physical plan
+                // optimization.
+                continue;
+            }
 
             MemoPhysicalPlanExtractor instance(memo,
                                                metadata,
