@@ -30,13 +30,17 @@
 #pragma once
 
 #include <boost/core/demangle.hpp>
+#include <sstream>
+#include <string>
 #include <typeinfo>
 
+#include "mongo/db/exec/sbe/util/print_options.h"
 #include "mongo/db/query/optimizer/node.h"
 #include "mongo/db/query/optimizer/utils/utils.h"
 
 
 namespace mongo::optimizer::unit_test_abt_literals {
+using namespace sbe::value;
 
 /**
  * The functions in this file aim to simplify and shorten the manual construction of ABTs for
@@ -78,7 +82,7 @@ inline T getEnumByName(StringData str, const T1& toStr) {
 }
 
 inline Operations getOpByName(StringData str) {
-    return getEnumByName<Operations>(str, OperationsEnum::toString);
+    return getEnumByName<Operations>(str, OperationsEnumString::arr_);
 }
 
 template <class T>
@@ -110,19 +114,62 @@ inline auto operator"" _cdouble(const char* c, size_t len) {
     return ExprHolder{Constant::fromDouble(std::stod({c, len}))};
 }
 
+// Null constant.
+inline auto _cnull() {
+    return ExprHolder{Constant::null()};
+}
+
+// Nothing constant.
+inline auto _cnothing() {
+    return ExprHolder{Constant::nothing()};
+}
+
+// NaN constant
+inline auto _cNaN() {
+    return ExprHolder{Constant::fromDouble(std::numeric_limits<double>::quiet_NaN())};
+}
+
 // Boolean constant.
 inline auto _cbool(const bool val) {
     return ExprHolder{Constant::boolean(val)};
 }
 
-// Empty array constant.
-inline auto _cemptyarr() {
+// MinKey constant.
+inline auto _cminKey() {
+    return ExprHolder{Constant::minKey()};
+}
+
+// MaxKey constant.
+inline auto _cmaxKey() {
+    return ExprHolder{Constant::maxKey()};
+}
+
+// Array constant. We expect the arguments to be Constants.
+inline auto _carray(auto&&... elements) {
+    using namespace sbe::value;
+
+    auto [tag, val] = makeNewArray();
+    auto arr = getArrayView(val);
+    (arr->push_back(copyValue(elements._n.template cast<Constant>()->get().first,
+                              elements._n.template cast<Constant>()->get().second)),
+     ...);
+
+    return ExprHolder{make<Constant>(tag, val)};
+}
+
+// Empty Array constant.
+inline auto _cemparray() {
     return ExprHolder{Constant::emptyArray()};
 }
 
+// Empty Object constant.
+inline auto _cempobj() {
+    return ExprHolder{Constant::emptyObject()};
+}
+
 // Variable.
 inline auto operator"" _var(const char* c, size_t len) {
-    return ExprHolder{make<Variable>(ProjectionName{{c, len}})};
+    return ExprHolder{make<Variable>(ProjectionName{StringData{c, len}})};
 }
 
 // Vector of variable names.
@@ -278,8 +325,8 @@ inline auto _collation(std::vector<std::string> spec, NodeHolder input) {
     ProjectionCollationSpec pcspec;
     for (const auto& s : spec) {
         if (const auto pos = s.find(':'); pos < s.size() - 1) {
-            const auto& projName = s.substr(0, pos);
-            const auto& collSpec = s.substr(pos + 1);
+            std::string projName = s.substr(0, pos);
+            std::string collSpec = s.substr(pos + 1);
 
             CollationOp op;
             if (collSpec == "1") {
@@ -313,7 +360,7 @@ inline auto _spoolp(StringData type,
                     ExprHolder filter,
                     NodeHolder child) {
     return NodeHolder{make<SpoolProducerNode>(
-        getEnumByName<SpoolProducerType>(type, SpoolProducerTypeEnum::toString),
+        getEnumByName<SpoolProducerType>(type, SpoolProducerTypeEnumString::arr_),
         spoolId,
         std::move(pns),
         std::move(filter._n),
@@ -322,11 +369,16 @@ inline auto _spoolp(StringData type,
 
 inline auto _spoolc(StringData type, int64_t spoolId, ProjectionNameVector pns) {
     return NodeHolder{make<SpoolConsumerNode>(
-        getEnumByName<SpoolConsumerType>(type, SpoolConsumerTypeEnum::toString),
+        getEnumByName<SpoolConsumerType>(type, SpoolConsumerTypeEnumString::arr_),
         spoolId,
         std::move(pns))};
 }
 
+inline auto getParam(TypeTags typeTag) {
+    return _fn(
+        kParameterFunctionName, "0"_cint64, ExprHolder{Constant::int32(static_cast<int>(typeTag))});
+}
+
 /**
  * Note the root returns an ABT instead of a holder.
  */
@@ -483,23 +535,67 @@ public:
      * ABT Expressions.
      */
     std::string transport(const Constant& expr) {
+        using namespace sbe::value;
+        auto getVal = [&]() -> std::string {
+            return str::stream{} << "\"" << expr.get() << "\"";
+        };
+
+        auto out = str::stream{};
         if (expr.isValueBool()) {
-            return str::stream() << "_cbool(" << (expr.getValueBool() ? "true" : "false") << ")";
-        }
+            out << "_cbool(" << (expr.getValueBool() ? "true" : "false") << ")";
+        } else if (expr.isArray()) {
+            if (getArrayView(expr.get().second)->size() == 0) {
+                out << "_cemparray()";
+            } else {
+                out << "_carray(";
+                auto shouldTruncate = true;
+                size_t iter = 0;
 
-        str::stream out;
-        out << "\"" << expr.get() << "\"";
-
-        if (expr.isValueInt32()) {
-            out << "_cint32";
-        } else if (expr.isValueInt64()) {
-            out << "_cint64";
-        } else if (expr.isValueDouble()) {
-            out << "_cdouble";
+                if (auto ae = ArrayEnumerator{TypeTags::Array, expr.get().second}; !ae.atEnd()) {
+                    while (iter < sbe::PrintOptions::kDefaultArrayObjectOrNestingMaxDepth) {
+                        auto getMaker = [](auto&& arg) -> std::string {
+                            switch (arg.first) {
+                                case TypeTags::NumberDouble:
+                                    return (std::stringstream{}
+                                            << "sbe::value::bitcastFrom<double>(" << std::showpoint
+                                            << arg << ")")
+                                        .str();
+                                case TypeTags::StringSmall:
+                                    return str::stream{} << "sbe::value::makeSmallString(" << arg
+                                                         << "_sd).second";
+                                default:
+                                    return str::stream{} << "unimplemented(" << arg << ")";
+                            }
+                        };
+                        auto aeView = ae.getViewOfValue();
+                        out << "std::pair{sbe::value::TypeTags::" << aeView.first << ", "
+                            << getMaker(aeView) << "}";
+                        ae.advance();
+                        if (ae.atEnd()) {
+                            shouldTruncate = false;
+                            break;
+                        }
+                        out << ", ";
+                        ++iter;
+                    }
+                    if (shouldTruncate) {
+                        out << "...";
+                    }
+                }
+                out << ")";
+            }
+        } else if (expr.isObject() && getObjectView(expr.get().second)->size() == 0) {
+            out << "_cempobj()";
         } else if (expr.isString()) {
-            out << "_cstr";
+            out << expr.get() << "_cstr";
+        } else if (expr.isValueInt32()) {
+            out << getVal() << "_cint32";
+        } else if (expr.isValueInt64()) {
+            out << getVal() << "_cint64";
+        } else if (expr.isValueDouble()) {
+            out << getVal() << "_cdouble";
         } else {
-            out << "<non-standard constant>";
+            out << getVal() << "<non-standard constant>";
         }
 
         return out;
@@ -511,14 +607,12 @@ public:
     }
 
     std::string transport(const UnaryOp& expr, std::string inResult) {
-        return str::stream() << "_unary(\"" << OperationsEnum::toString[static_cast<int>(expr.op())]
-                             << "\", " << inResult << ")";
+        return str::stream() << "_unary(\"" << toStringData(expr.op()) << "\", " << inResult << ")";
     }
 
     std::string transport(const BinaryOp& expr, std::string leftResult, std::string rightResult) {
-        return str::stream() << "_binary(\""
-                             << OperationsEnum::toString[static_cast<int>(expr.op())] << "\", "
-                             << leftResult << ", " << rightResult << ")";
+        return str::stream() << "_binary(\"" << toStringData(expr.op()) << "\", " << leftResult
+                             << ", " << rightResult << ")";
     }
 
     std::string transport(const EvalPath& expr, std::string pathResult, std::string inputResult) {
@@ -545,8 +639,8 @@ public:
     }
 
     std::string transport(const PathCompare& path, std::string valueResult) {
-        return str::stream() << "_cmp(\"" << OperationsEnum::toString[static_cast<int>(path.op())]
-                             << "\", " << valueResult << ")";
+        return str::stream() << "_cmp(\"" << toStringData(path.op()) << "\", " << valueResult
+                             << ")";
     }
 
     std::string transport(const PathTraverse& path, std::string inResult) {
@@ -597,8 +691,7 @@ public:
 
     std::string transport(const SpoolConsumerNode& node, std::string /*bindResult*/) {
         str::stream os;
-        os << "_spoolc(\"" << SpoolConsumerTypeEnum::toString[static_cast<int>(node.getType())]
-           << "\""
+        os << "_spoolc(\"" << toStringData(node.getType()) << "\""
            << ", " << node.getSpoolId() << ", _varnames(";
         printProjNames(os, node.binder().names());
         os << "))";
@@ -690,8 +783,7 @@ public:
                           std::string /*bindResult*/,
                           std::string /*refsResult*/) {
         str::stream os;
-        os << ".spoolp(\"" << SpoolProducerTypeEnum::toString[static_cast<int>(node.getType())]
-           << "\""
+        os << ".spoolp(\"" << toStringData(node.getType()) << "\""
            << ", " << node.getSpoolId() << ", _varnames(";
         printProjNames(os, node.binder().names());
         return os << "), " << filterResult << ")" << _nodeSeparator << childResult;
