@@ -29,7 +29,21 @@
 
 #pragma once
 
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <memory>
+#include <utility>
+
+#include "mongo/base/status_with.h"
+#include "mongo/db/exec/document_value/document_metadata_fields.h"
 #include "mongo/db/matcher/expression.h"
+#include "mongo/db/matcher/expression_parser.h"
+#include "mongo/db/matcher/extensions_callback.h"
+#include "mongo/db/matcher/extensions_callback_noop.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/pipeline/expression_context.h"
+#include "mongo/db/query/collation/collator_interface.h"
+#include "mongo/db/query/find_command.h"
 #include "mongo/db/query/find_command_gen.h"
 #include "mongo/db/query/projection.h"
 #include "mongo/db/query/projection_policies.h"
@@ -37,6 +51,14 @@
 
 namespace mongo {
 
+struct ParsedFindCommandParams {
+    std::unique_ptr<FindCommandRequest> findCommand;
+    const ExtensionsCallback& extensionsCallback = ExtensionsCallbackNoop();
+    MatchExpressionParser::AllowedFeatureSet allowedFeatures =
+        MatchExpressionParser::kDefaultSpecialFeatures;
+    const ProjectionPolicies& projectionPolicies = ProjectionPolicies::findProjectionPolicies();
+};
+
 /**
  * Represents a find command request, but with more fully parsed ASTs for some fields which are
  * still raw BSONObj on the FindCommandRequest type.
@@ -53,7 +75,8 @@ struct ParsedFindCommand {
         const boost::intrusive_ptr<ExpressionContext>& expCtx,
         std::unique_ptr<CollatorInterface> collator,
         std::unique_ptr<MatchExpression> filter,
-        std::unique_ptr<FindCommandRequest> findCommandRequest);
+        std::unique_ptr<FindCommandRequest> findCommandRequest,
+        const ProjectionPolicies& projectionPolicies);
 
     std::unique_ptr<CollatorInterface> collator;
     std::unique_ptr<MatchExpression> filter;
@@ -70,7 +93,7 @@ struct ParsedFindCommand {
 
     // All other parameters to the find command which do not have AST-like types and can be
     // appropriately tracked as raw value types like ints. The fields above like 'filter' are all
-    // still present in their raw form on this FidnCommandRequest, but it is not expected that they
+    // still present in their raw form on this FindCommandRequest, but it is not expected that they
     // will be useful other than to keep the original BSON values around in-memory to avoid copying
     // large strings and such.
     std::unique_ptr<FindCommandRequest> findCommandRequest;
@@ -96,25 +119,8 @@ StatusWith<QueryMetadataBitSet> isValid(const MatchExpression* root,
 
 /**
  * Parses each big component of the input 'findCommand.' Throws exceptions if failing to parse.
- * Comes in one overload which will create an ExpressionContext for the caller, and one overload to
- * be used when the caller already has an ExpressionContext.
  */
-StatusWith<std::pair<boost::intrusive_ptr<ExpressionContext>, std::unique_ptr<ParsedFindCommand>>>
-parse(OperationContext* opCtx,
-      std::unique_ptr<FindCommandRequest> findCommand,
-      const ExtensionsCallback& extensionsCallback = ExtensionsCallbackNoop(),
-      MatchExpressionParser::AllowedFeatureSet allowedFeatures =
-          MatchExpressionParser::kDefaultSpecialFeatures,
-      const ProjectionPolicies& projectionPolicies = ProjectionPolicies::findProjectionPolicies());
-
-// Overload of the above for when the caller has an available ExpressionContext.
 StatusWith<std::unique_ptr<ParsedFindCommand>> parse(
-    const boost::intrusive_ptr<ExpressionContext>& expCtx,
-    std::unique_ptr<FindCommandRequest> findCommand,
-    const ExtensionsCallback& extensionsCallback = ExtensionsCallbackNoop(),
-    MatchExpressionParser::AllowedFeatureSet allowedFeatures =
-        MatchExpressionParser::kDefaultSpecialFeatures,
-    const ProjectionPolicies& projectionPolicies = ProjectionPolicies::findProjectionPolicies());
-
+    const boost::intrusive_ptr<ExpressionContext>& expCtx, ParsedFindCommandParams&& params);
 }  // namespace parsed_find_command
 }  // namespace mongo
