@@ -29,9 +29,14 @@
 
 #pragma once
 
+#include "mongo/bson/util/builder_fwd.h"
+#include "mongo/db/catalog/collection.h"
+#include "mongo/db/matcher/expression.h"
 #include "mongo/db/query/canonical_query.h"
 #include "mongo/db/query/classic_plan_cache.h"
 #include "mongo/db/query/collection_query_info.h"
+#include "mongo/db/query/multiple_collection_accessor.h"
+#include "mongo/db/query/plan_cache_indexability.h"
 #include "mongo/db/query/sbe_plan_cache.h"
 
 namespace mongo {
@@ -78,11 +83,19 @@ Key make(const CanonicalQuery& query, const CollectionPtr& collection) {
     return plan_cache_detail::make(query, collection, plan_cache_detail::PlanCacheKeyTag<Key>{});
 }
 
+/**
+ * Similar to above, a factory helper to make a SBE plan cache key, but used for find queries that
+ * might involve multiple collections.
+ */
+sbe::PlanCacheKey make(const CanonicalQuery& query,
+                       const MultipleCollectionAccessor& collections,
+                       canonical_query_encoder::Optimizer optimizer);
+
 /**
  * Similar to above, a factory helper to make a SBE plan cache key, but used for agg queries that
  * might involve multiple collections.
  */
-sbe::PlanCacheKey make(const CanonicalQuery& query, const MultipleCollectionAccessor& collections);
+sbe::PlanCacheKey make(const Pipeline& query, const MultipleCollectionAccessor& collections);
 
 }  // namespace plan_cache_key_factory
 }  // namespace mongo
