@@ -27,14 +27,37 @@
  *    it in the license file.
  */
 
+#include <iosfwd>
+#include <memory>
+#include <vector>
+
+#include <boost/move/utility_core.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+
+#include "mongo/base/status.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/json.h"
+#include "mongo/bson/util/builder_fwd.h"
 #include "mongo/db/index/wildcard_key_generator.h"
+#include "mongo/db/index_names.h"
+#include "mongo/db/matcher/expression.h"
+#include "mongo/db/matcher/expression_parser.h"
+#include "mongo/db/matcher/extensions_callback_noop.h"
+#include "mongo/db/pipeline/expression_context.h"
 #include "mongo/db/pipeline/expression_context_for_test.h"
 #include "mongo/db/query/canonical_query_test_util.h"
+#include "mongo/db/query/classic_plan_cache.h"
 #include "mongo/db/query/collation/collator_interface_mock.h"
+#include "mongo/db/query/index_entry.h"
+#include "mongo/db/query/plan_cache_indexability.h"
 #include "mongo/db/query/plan_cache_key_factory.h"
 #include "mongo/db/query/plan_cache_key_info.h"
-#include "mongo/idl/server_parameter_test_util.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/intrusive_counter.h"
+#include "mongo/util/str.h"
 
 namespace mongo {
 
@@ -42,25 +65,32 @@ using std::string;
 using std::unique_ptr;
 using std::vector;
 
-using unittest::assertGet;
 
 std::ostream& operator<<(std::ostream& stream, const PlanCacheKeyInfo& key) {
-    stream << key.toString();
+    stream << key.toString() << " settings: " << key.querySettings().toBSON().toString();
     return stream;
 }
 
+auto makeDbName(StringData dbName) {
+    return DatabaseNameUtil::deserialize(
+        boost::none /*tenantId=*/, dbName, SerializationContext::stateDefault());
+}
+
+
 namespace {
 using PlanCacheKeyInfoTest = CanonicalQueryTest;
 
 PlanCacheKeyInfo makeKey(const CanonicalQuery& cq,
-                         const std::vector<CoreIndexInfo>& indexCores = {}) {
+                         const std::vector<CoreIndexInfo>& indexCores = {},
+                         const query_settings::QuerySettings& settings = {}) {
     PlanCacheIndexabilityState indexabilityState;
     indexabilityState.updateDiscriminators(indexCores);
 
     StringBuilder indexabilityKeyBuilder;
-    plan_cache_detail::encodeIndexability(cq.root(), indexabilityState, &indexabilityKeyBuilder);
+    plan_cache_detail::encodeIndexability(
+        cq.getPrimaryMatchExpression(), indexabilityState, &indexabilityKeyBuilder);
 
-    return {cq.encodeKey(), indexabilityKeyBuilder.str()};
+    return {cq.encodeKey(), indexabilityKeyBuilder.str(), settings};
 }
 
 /**
@@ -99,38 +129,6 @@ std::pair<CoreIndexInfo, std::unique_ptr<WildcardProjection>> makeWildcardUpdate
             std::move(wcProj)};
 }
 
-/**
- * Check that the stable keys of 'a' and 'b' are not equal because of the last character.
- */
-void assertPlanCacheKeysUnequalDueToForceClassicEngineValue(const PlanCacheKeyInfo& a,
-                                                            const PlanCacheKeyInfo& b) {
-    auto aUnstablePart = a.getIndexabilityDiscriminators();
-    auto bUnstablePart = b.getIndexabilityDiscriminators();
-    auto aStablePart = a.getQueryShape();
-    auto bStablePart = b.getQueryShape();
-
-    ASSERT_EQ(aUnstablePart, bUnstablePart);
-    // The last 2 characters (plus separator) of the stable part encodes the engine that uses this
-    // PlanCacheKey and if apiStrict was used. So the stable parts except for the last two
-    // characters should be identical.
-    ASSERT_EQ(aStablePart.substr(0, aStablePart.size() - 2),
-              bStablePart.substr(0, bStablePart.size() - 2));
-
-    // Should have at least 2 byte to represent whether we must use the classic engine and stable
-    // API.
-    ASSERT_GTE(aStablePart.size(), 2);
-
-    // The indexability discriminators should match.
-    ASSERT_EQ(a.getIndexabilityDiscriminators(), b.getIndexabilityDiscriminators());
-
-    // The stable parts should not match because of the second character from the back, encoding the
-    // engine type.
-    ASSERT_NE(aStablePart, bStablePart);
-    ASSERT_NE(aStablePart[aStablePart.size() - 2], bStablePart[bStablePart.size() - 2]);
-    // Ensure that the the apiStrict values are equal.
-    ASSERT_EQ(aStablePart.back(), bStablePart.back());
-}
-
 /**
  * Check that the stable keys of 'a' and 'b' are equal, but the index discriminators are not.
  */
@@ -146,6 +144,68 @@ void assertPlanCacheKeysUnequalDueToDiscriminators(const PlanCacheKeyInfo& a,
 
 }  // namespace
 
+TEST_F(PlanCacheKeyInfoTest, EqualityOperator) {
+    unique_ptr<CanonicalQuery> cqEqZero(canonicalize("{a: 0}"));
+    unique_ptr<CanonicalQuery> cqEqOne(canonicalize("{a: 1}"));
+
+    NamespaceSpec nsSpec;
+    nsSpec.setDb(makeDbName("db"));
+    nsSpec.setColl("coll"_sd);
+    query_settings::QuerySettings settingsA1IndexKeyPattern;
+    settingsA1IndexKeyPattern.setIndexHints(
+        {{query_settings::IndexHintSpec(nsSpec, {IndexHint(BSON("a" << 1))})}});
+    query_settings::QuerySettings settingsA1IndexName;
+    settingsA1IndexName.setIndexHints(
+        {{query_settings::IndexHintSpec(nsSpec, {IndexHint("a_1")})}});
+    query_settings::QuerySettings settingsB1IndexKeyPattern;
+    settingsB1IndexKeyPattern.setIndexHints(
+        {{query_settings::IndexHintSpec(nsSpec, {IndexHint(BSON("b" << 1))})}});
+
+    auto keyWithoutSettingsZero = makeKey(*cqEqZero, {} /*indexCores=*/);
+    auto keyWithSettingsZeroA1IndexKeyPattern =
+        makeKey(*cqEqZero, {} /*indexCores=*/, settingsA1IndexKeyPattern);
+    auto keyWithSettingsZeroA1IndexName =
+        makeKey(*cqEqZero, {} /*indexCores=*/, settingsA1IndexName);
+    auto keyWithSettingsOneA1IndexKeyPattern =
+        makeKey(*cqEqOne, {} /*indexCores=*/, settingsA1IndexKeyPattern);
+    auto keyWithSettingsOneA1IndexName = makeKey(*cqEqOne, {} /*indexCores=*/, settingsA1IndexName);
+    auto keyWithSettingsOneB1IndexKeyPattern =
+        makeKey(*cqEqOne, {} /*indexCores=*/, settingsB1IndexKeyPattern);
+
+    // Ensure that presence of query settings results in different hash and makes equality fail.
+    ASSERT_NE(keyWithoutSettingsZero.planCacheKeyHash(),
+              keyWithSettingsZeroA1IndexKeyPattern.planCacheKeyHash());
+    ASSERT_NE(keyWithoutSettingsZero, keyWithSettingsZeroA1IndexKeyPattern);
+
+    ASSERT_NE(keyWithoutSettingsZero.planCacheKeyHash(),
+              keyWithSettingsOneA1IndexKeyPattern.planCacheKeyHash());
+    ASSERT_NE(keyWithoutSettingsZero, keyWithSettingsOneA1IndexKeyPattern);
+
+    // Ensure that keys with same query settings and equivalent 'planCacheShapeHash' results in
+    // same hash and passes equality check.
+    ASSERT_EQ(keyWithSettingsOneA1IndexName.planCacheKeyHash(),
+              keyWithSettingsZeroA1IndexName.planCacheKeyHash());
+    ASSERT_EQ(keyWithSettingsOneA1IndexName, keyWithSettingsZeroA1IndexName);
+
+    ASSERT_EQ(keyWithSettingsOneA1IndexKeyPattern.planCacheKeyHash(),
+              keyWithSettingsZeroA1IndexKeyPattern.planCacheKeyHash());
+    ASSERT_EQ(keyWithSettingsOneA1IndexKeyPattern, keyWithSettingsZeroA1IndexKeyPattern);
+
+    // Ensure that if settings are different, both hashes are different and keys are considered
+    // unequal.
+    ASSERT_NE(keyWithSettingsOneA1IndexKeyPattern.planCacheKeyHash(),
+              keyWithSettingsZeroA1IndexName.planCacheKeyHash());
+    ASSERT_NE(keyWithSettingsOneA1IndexKeyPattern, keyWithSettingsZeroA1IndexName);
+
+    ASSERT_NE(keyWithSettingsOneA1IndexKeyPattern.planCacheKeyHash(),
+              keyWithSettingsOneA1IndexName.planCacheKeyHash());
+    ASSERT_NE(keyWithSettingsOneA1IndexKeyPattern, keyWithSettingsOneA1IndexName);
+
+    ASSERT_NE(keyWithSettingsOneA1IndexKeyPattern.planCacheKeyHash(),
+              keyWithSettingsOneB1IndexKeyPattern.planCacheKeyHash());
+    ASSERT_NE(keyWithSettingsOneA1IndexKeyPattern, keyWithSettingsOneB1IndexKeyPattern);
+}
+
 // When a sparse index is present, computeKey() should generate different keys depending on
 // whether or not the predicates in the given query can use the index.
 TEST_F(PlanCacheKeyInfoTest, ComputeKeySparseIndex) {
@@ -281,8 +341,8 @@ TEST_F(PlanCacheKeyInfoTest, ComputeKeyPartialIndexDisjunction) {
 TEST_F(PlanCacheKeyInfoTest, ComputeKeyPartialIndexNestedDisjunction) {
     BSONObj filterObj = fromjson(R"(
         {$and: [
-            {$or: [{f: {$gt: 10}}, {f: {$lt: 0}}]},
-            {$or: [{f: {$gt: 11}}, {f: {$lt: 1}}]}
+            {$or: [{f: {$gt: 10}}, {f: {$lt: 0}}]}, 
+            {$or: [{f: {$gt: 11}}, {f: {$lt: 1}}]} 
         ]})");
     unique_ptr<MatchExpression> filterExpr(parseMatchExpression(filterObj));
 
@@ -368,170 +428,6 @@ TEST_F(PlanCacheKeyInfoTest, ComputeKeyCollationIndex) {
               makeKey(*inContainsStringHasCollation, indexCores));
     ASSERT_EQ(makeKey(*inNoStrings, indexCores).getIndexabilityDiscriminators(),
               makeKey(*inContainsStringHasCollation, indexCores).getIndexabilityDiscriminators());
-
-    unique_ptr<CanonicalQuery> elemMatchString(canonicalize("{a: {$elemMatch: {$eq: 'abc'}}}"));
-    unique_ptr<CanonicalQuery> elemMatchNoString(canonicalize("{a: {$elemMatch: {$eq: 5}}}"));
-
-    // 'elemMatchString' and 'elemMatchNoString' should have different discriminators.
-    assertPlanCacheKeysUnequalDueToDiscriminators(makeKey(*elemMatchString, indexCores),
-                                                  makeKey(*elemMatchNoString, indexCores));
-
-    unique_ptr<CanonicalQuery> elemMatchAndFirstBoundString(
-        canonicalize("{a: {$elemMatch: {$gt: 'A', $lt: 5}}}"));
-    unique_ptr<CanonicalQuery> elemMatchAndSecondBoundString(
-        canonicalize("{a: {$elemMatch: {$gt: 1, $lt: 'B'}}}"));
-    unique_ptr<CanonicalQuery> elemMatchAndNoString(
-        canonicalize("{a: {$elemMatch: {$gt: 1, $lt: 5}}}"));
-
-    // 'elemMatchAndFirstBoundString'/'elemMatchAndSecondBoundString' and 'elemMatchAndNoString'
-    // should have different discriminators.
-    assertPlanCacheKeysUnequalDueToDiscriminators(
-        makeKey(*elemMatchAndFirstBoundString, indexCores),
-        makeKey(*elemMatchAndNoString, indexCores));
-    assertPlanCacheKeysUnequalDueToDiscriminators(
-        makeKey(*elemMatchAndSecondBoundString, indexCores),
-        makeKey(*elemMatchAndNoString, indexCores));
-}
-
-TEST_F(PlanCacheKeyInfoTest, ComputeKeyElemMatchNoIndexOnPath) {
-    CollatorInterfaceMock collator(CollatorInterfaceMock::MockType::kReverseString);
-    const auto keyPattern = BSON("a" << 1);
-    const std::vector<CoreIndexInfo> indexCores = {
-        CoreIndexInfo(keyPattern,
-                      IndexNames::nameToType(IndexNames::findPluginName(keyPattern)),
-                      false,                       // sparse
-                      IndexEntry::Identifier{""},  // name
-                      nullptr,                     // filterExpr
-                      &collator)};                 // collation
-
-    // $elemMatch on "b.a" cannot use the index on field "a"; it should recognize that the field "a"
-    // being referred to in the query is not top-level.
-    unique_ptr<CanonicalQuery> valueElemMatchOnDiffField(
-        canonicalize("{b: {$elemMatch: {a: {$eq: 5}}}}"));
-    ASSERT_EQ(makeKey(*valueElemMatchOnDiffField, indexCores).getIndexabilityDiscriminators(), "");
-}
-
-TEST_F(PlanCacheKeyInfoTest, ComputeKeyObjectElemMatch) {
-    CollatorInterfaceMock collator(CollatorInterfaceMock::MockType::kReverseString);
-    const auto keyPattern = BSON("a.b" << 1);
-    const std::vector<CoreIndexInfo> indexCores = {
-        CoreIndexInfo(keyPattern,
-                      IndexNames::nameToType(IndexNames::findPluginName(keyPattern)),
-                      false,                       // sparse
-                      IndexEntry::Identifier{""},  // name
-                      nullptr,                     // filterExpr
-                      &collator)};                 // collation
-
-    // Object elemMatch on "a.b" should detect string comparison when query collation differs.
-    unique_ptr<CanonicalQuery> objElemMatchNoString(
-        canonicalize("{a: {$elemMatch: {b: {$eq: 5}}}}"));
-    unique_ptr<CanonicalQuery> objElemMatchString(
-        canonicalize("{a: {$elemMatch: {b: {$eq: 'str'}}}}"));
-    assertPlanCacheKeysUnequalDueToDiscriminators(makeKey(*objElemMatchNoString, indexCores),
-                                                  makeKey(*objElemMatchString, indexCores));
-
-    // Object elemMatch with multiple predicates on "a.b" should detect string comparison when query
-    // collation differs.
-    unique_ptr<CanonicalQuery> objElemMatchWithOrNoString(
-        canonicalize("{a: {$elemMatch: {$or: [{b: {$eq: 5}}, {b: {$eq: 10}}]}}}"));
-    unique_ptr<CanonicalQuery> objElemMatchWithOrString(
-        canonicalize("{a: {$elemMatch: {$or: [{b: {$eq: 'A'}}, {b: {$eq: 10}}]}}}"));
-
-    assertPlanCacheKeysUnequalDueToDiscriminators(makeKey(*objElemMatchWithOrNoString, indexCores),
-                                                  makeKey(*objElemMatchWithOrString, indexCores));
-}
-
-TEST_F(PlanCacheKeyInfoTest, ComputeKeyWithNot) {
-    const auto keyPattern = BSON("a.b" << 1);
-    const auto keyPattern2 = BSON("a" << 1);
-
-    const std::vector<CoreIndexInfo> indexCores = {
-        CoreIndexInfo(keyPattern,
-                      IndexNames::nameToType(IndexNames::findPluginName(keyPattern)),
-                      false,                       // sparse
-                      IndexEntry::Identifier{""},  // name
-                      nullptr,                     // filterExpr
-                      nullptr),                    // collation
-        CoreIndexInfo(keyPattern2,
-                      IndexNames::nameToType(IndexNames::findPluginName(keyPattern2)),
-                      false,                       // sparse
-                      IndexEntry::Identifier{""},  // name
-                      nullptr,                     // filterExpr
-                      nullptr),                    // collation
-    };
-
-    // $not with a child that does array matching should be differentiated from $not when the child
-    // does not do array matching.
-    unique_ptr<CanonicalQuery> dottedMatchNoArray(canonicalize("{'a.b': {$not: {$eq: 5}}}"));
-    unique_ptr<CanonicalQuery> dottedMatchArray(canonicalize("{'a.b': {$not: {$eq: []}}}"));
-    assertPlanCacheKeysUnequalDueToDiscriminators(makeKey(*dottedMatchNoArray, indexCores),
-                                                  makeKey(*dottedMatchArray, indexCores));
-
-    unique_ptr<CanonicalQuery> matchNoArray(canonicalize("{a: {$not: {$eq: 5}}}"));
-    unique_ptr<CanonicalQuery> matchArray(canonicalize("{a: {$not: {$eq: []}}}"));
-    assertPlanCacheKeysUnequalDueToDiscriminators(makeKey(*matchNoArray, indexCores),
-                                                  makeKey(*matchArray, indexCores));
-
-
-    // Tests the same as above, but where the $not is under $elemMatch.
-    unique_ptr<CanonicalQuery> objElemMatchNoArray(
-        canonicalize("{a: {$elemMatch: {b: {$not: {$eq: 5}}}}}"));
-    unique_ptr<CanonicalQuery> objElemMatchArray(
-        canonicalize("{a: {$elemMatch: {b: {$not: {$eq: []}}}}}"));
-    assertPlanCacheKeysUnequalDueToDiscriminators(makeKey(*objElemMatchNoArray, indexCores),
-                                                  makeKey(*objElemMatchArray, indexCores));
-
-    unique_ptr<CanonicalQuery> valueElemMatchNoArray(
-        canonicalize("{a: {$elemMatch: {$not: {$eq: 5}}}}"));
-    unique_ptr<CanonicalQuery> valueElemMatchArray(
-        canonicalize("{a: {$elemMatch: {$not: {$eq: []}}}}"));
-    assertPlanCacheKeysUnequalDueToDiscriminators(makeKey(*valueElemMatchNoArray, indexCores),
-                                                  makeKey(*valueElemMatchArray, indexCores));
-}
-
-TEST_F(PlanCacheKeyInfoTest, ComputeKeyWithLogicalNodes) {
-    // Test that we don't include discriminators for logical nodes $and/$or, since those have no
-    // effect on indexability.
-    CollatorInterfaceMock collator(CollatorInterfaceMock::MockType::kReverseString);
-    const auto keyPattern = BSON("a" << 1);
-    const auto dottedKeyPattern = BSON("a.b" << 1);
-    const std::vector<CoreIndexInfo> indexCores = {
-        CoreIndexInfo(keyPattern,
-                      IndexNames::nameToType(IndexNames::findPluginName(keyPattern)),
-                      false,                       // sparse
-                      IndexEntry::Identifier{""},  // name
-                      nullptr,                     // filterExpr
-                      &collator),                  // collation
-        CoreIndexInfo(dottedKeyPattern,
-                      IndexNames::nameToType(IndexNames::findPluginName(dottedKeyPattern)),
-                      false,                       // sparse
-                      IndexEntry::Identifier{""},  // name
-                      nullptr,                     // filterExpr
-                      &collator)};                 // collation
-
-    // Top-level logical nodes have no path, so they shouldn't be included.
-    unique_ptr<CanonicalQuery> simpleAnd(canonicalize("{$and: [{a: 1}, {b: 1}]}"));
-    ASSERT_EQ(makeKey(*simpleAnd, indexCores).getIndexabilityDiscriminators(), "<1>");
-    unique_ptr<CanonicalQuery> simpleOr(canonicalize("{$or: [{a: 1}, {b: 1}]}"));
-    ASSERT_EQ(makeKey(*simpleOr, indexCores).getIndexabilityDiscriminators(), "<1>");
-    unique_ptr<CanonicalQuery> orAnd(canonicalize("{$or: [{a: 1, b: 1}, {a: 5, b: 5}]}"));
-    ASSERT_EQ(makeKey(*orAnd, indexCores).getIndexabilityDiscriminators(), "<1><1>");
-    unique_ptr<CanonicalQuery> andOr(canonicalize("{a: 1, $or: [{c: 1}, {b: 1}]}"));
-    ASSERT_EQ(makeKey(*andOr, indexCores).getIndexabilityDiscriminators(), "<1>");
-
-    // Similar tests, but under an $elemMatch. Expect entries for the $elemMatch on "a" and each
-    // nested comparison on "a.b", and no other entries.
-    unique_ptr<CanonicalQuery> elemMatchSimpleAnd(
-        canonicalize("{a: {$elemMatch: {$and: [{c: 1}, {b: 1}]}}}"));
-    ASSERT_EQ(makeKey(*elemMatchSimpleAnd, indexCores).getIndexabilityDiscriminators(), "<1><1>");
-    unique_ptr<CanonicalQuery> elemMatchSimpleOr(
-        canonicalize("{a: {$elemMatch: {$or: [{c: 1}, {b: 1}]}}}"));
-    ASSERT_EQ(makeKey(*elemMatchSimpleOr, indexCores).getIndexabilityDiscriminators(), "<1><1>");
-    unique_ptr<CanonicalQuery> elemMatchWithLogicalNodes(
-        canonicalize("{a: {$elemMatch: {$or: [{$and: [{b: {$gte: 3}}, {b: {$lte: 5}}]}, {$and: "
-                     "[{b: {$gte: 1}}, {b: {$lte: 2}}]}]}}}"));
-    ASSERT_EQ(makeKey(*elemMatchWithLogicalNodes, indexCores).getIndexabilityDiscriminators(),
-              "<1><1><1><1><1>");
 }
 
 TEST_F(PlanCacheKeyInfoTest, ComputeKeyWildcardIndex) {
@@ -685,34 +581,6 @@ TEST_F(PlanCacheKeyInfoTest,
     }
 }
 
-TEST_F(PlanCacheKeyInfoTest, DifferentQueryEngines) {
-    const auto keyPattern = BSON("a" << 1);
-    const std::vector<CoreIndexInfo> indexCores = {
-        CoreIndexInfo(keyPattern,
-                      IndexNames::nameToType(IndexNames::findPluginName(keyPattern)),
-                      false,                         // sparse
-                      IndexEntry::Identifier{""})};  // name
-
-    // Helper to construct a plan cache key given the 'forceClassicEngine' flag.
-    auto constructPlanCacheKey = [&](bool forceClassicEngine) {
-        RAIIServerParameterControllerForTest controller{"internalQueryFrameworkControl",
-                                                        forceClassicEngine ? "forceClassicEngine"
-                                                                           : "trySbeEngine"};
-        QueryTestServiceContext serviceContext;
-        auto opCtx = serviceContext.makeOperationContext();
-        const auto queryStr = "{a: 0}";
-        unique_ptr<CanonicalQuery> cq(canonicalize(opCtx.get(), queryStr));
-        return makeKey(*cq, indexCores);
-    };
-
-    const auto classicEngineKey = constructPlanCacheKey(true);
-    const auto noClassicEngineKey = constructPlanCacheKey(false);
-
-    // Check that the two plan cache keys are not equal because the plans were created under
-    // different engines.
-    assertPlanCacheKeysUnequalDueToForceClassicEngineValue(classicEngineKey, noClassicEngineKey);
-}
-
 TEST_F(PlanCacheKeyInfoTest,
        ComputeKeyWildcardDiscriminatesCorrectlyWithPartialFilterAndExpression) {
     // Partial filter is an AND of multiple conditions.
