@@ -29,21 +29,84 @@
 
 #pragma once
 
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <exception>
+#include <string>
+#include <variant>
+#include <vector>
+
 #include "mongo/base/status.h"
+#include "mongo/base/string_data.h"
 #include "mongo/bson/bsonobj.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/db/exec/document_value/document.h"
 #include "mongo/db/exec/plan_stats.h"
+#include "mongo/db/namespace_string.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/ops/update_result.h"
 #include "mongo/db/query/canonical_query.h"
 #include "mongo/db/query/plan_explainer.h"
 #include "mongo/db/query/plan_summary_stats.h"
 #include "mongo/db/query/plan_yield_policy.h"
+#include "mongo/db/query/query_stats/data_bearing_node_metrics.h"
 #include "mongo/db/query/restore_context.h"
 #include "mongo/db/record_id.h"
+#include "mongo/db/repl/oplog.h"
+#include "mongo/db/repl/optime.h"
+#include "mongo/db/s/scoped_collection_metadata.h"
+#include "mongo/db/shard_role.h"
+#include "mongo/util/assert_util_core.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/future.h"
 
 namespace mongo {
 
-class PlanStage;
+// TODO: SERVER-76397 Remove this once we use CollectionAcquisition everywhere.
+class VariantCollectionPtrOrAcquisition {
+public:
+    VariantCollectionPtrOrAcquisition(const CollectionPtr* collectionPtr)
+        : _collectionPtrOrAcquisition(collectionPtr) {}
+    VariantCollectionPtrOrAcquisition(CollectionAcquisition collection)
+        : _collectionPtrOrAcquisition(collection) {}
+
+    const std::variant<const CollectionPtr*, CollectionAcquisition>& get() {
+        return _collectionPtrOrAcquisition;
+    };
+
+    const CollectionPtr& getCollectionPtr() const;
+
+    bool isCollectionPtr() const {
+        return holds_alternative<const CollectionPtr*>(_collectionPtrOrAcquisition);
+    }
+
+    bool isAcquisition() const {
+        return holds_alternative<CollectionAcquisition>(_collectionPtrOrAcquisition);
+    }
+
+    const CollectionAcquisition& getAcquisition() const {
+        return std::get<CollectionAcquisition>(_collectionPtrOrAcquisition);
+    }
+
+    boost::optional<ScopedCollectionFilter> getShardingFilter(OperationContext* opCtx) const;
+
+    const NamespaceString& nss() const {
+        return std::visit([](const auto& choice) -> const NamespaceString& { return nss(choice); },
+                          _collectionPtrOrAcquisition);
+    }
+
+private:
+    static const NamespaceString& nss(const CollectionPtr* collectionPtr) {
+        return (*collectionPtr)->ns();
+    }
+
+    static const NamespaceString& nss(const CollectionAcquisition& acquisition) {
+        return acquisition.nss();
+    }
+
+    std::variant<const CollectionPtr*, CollectionAcquisition> _collectionPtrOrAcquisition;
+};
 
 /**
  * If a getMore command specified a lastKnownCommittedOpTime (as secondaries do), we want to stop
@@ -59,14 +122,23 @@ class PlanStage;
 extern const OperationContext::Decoration<boost::optional<repl::OpTime>>
     clientsLastKnownCommittedOpTime;
 
-/**
- * If a plan yielded because it encountered a sharding critical section,
- * 'planExecutorShardingCriticalSectionFuture' will be set to a future that becomes ready when the
- * critical section ends. This future can be waited on to hold off resuming the plan execution while
- * the critical section is still active.
- */
-extern const OperationContext::Decoration<boost::optional<SharedSemiFuture<void>>>
-    planExecutorShardingCriticalSectionFuture;
+
+struct PlanExecutorShardingState {
+    /**
+     * If a plan yielded because it encountered a sharding critical section, 'criticalSectionFuture'
+     * will be set to a future that becomes ready when the critical section ends. This future can be
+     * waited on to hold off resuming the plan execution while the critical section is still active.
+     */
+    boost::optional<SharedSemiFuture<void>> criticalSectionFuture;
+    /**
+     * If a plan yielded because it needed to refresh a sharding catalog cache, then
+     * 'catalogCacheRefreshRequired' will be set to the nss for which the CatalogCache needs to be
+     * refreshed.
+     */
+    boost::optional<NamespaceString> catalogCacheRefreshRequired;
+};
+
+extern const OperationContext::Decoration<PlanExecutorShardingState> planExecutorShardingState;
 
 /**
  * A PlanExecutor is the abstraction that knows how to crank a tree of stages into execution.
@@ -173,6 +245,13 @@ public:
      */
     virtual CanonicalQuery* getCanonicalQuery() const = 0;
 
+    /**
+     * Get the pipeline that this executor is executing, without transferring ownership.
+     */
+    virtual Pipeline* getPipeline() const {
+        return nullptr;
+    }
+
     /**
      * Return the namespace that the query is running over.
      *
@@ -235,6 +314,12 @@ public:
      */
     virtual void reattachToOperationContext(OperationContext* opCtx) = 0;
 
+    /**
+     * Releases all storage engine resources that the plan executor has acquired. The plan will be
+     * left in the "saved" state so a call to restoreState() will be necessary afterwards.
+     */
+    void releaseAllAcquiredResources();
+
     /**
      * Produces the next document from the query execution plan. The caller can request that the
      * executor returns documents by passing a non-null pointer for the 'objOut' output parameter,
@@ -253,6 +338,33 @@ public:
      */
     virtual ExecState getNext(BSONObj* out, RecordId* dlOut) = 0;
 
+    /**
+     * This function allows the caller of an executor to specify how a batched execution should
+     * handle the next BSON object ('obj') produced by the executor. The field 'numAppended' is
+     * provided to indicate the number of results appended prior to 'obj'. On a successful call,
+     * 'pbrt' will be set to the latest postBatchResumeToken. This function returns whether or not
+     * the executor should continue appending; note that 'numAppended' is not incremented if
+     * append() returns false.
+     *
+     * See getNextBatch() below for use.
+     */
+    using AppendBSONObjFn =
+        std::function<bool(const BSONObj& obj, const BSONObj& pbrt, const size_t numAppended)>;
+
+    /**
+     * Looping version of getNext() which appends BSONObjs produced by the execution plan up to the
+     * 'batchSize', or until the provided append() function returns false. The 'pbrt' is an output
+     * parameter for storing the post-batch resume token for the last successful append(). Note that
+     * getNextBatch() will stash the current object on a failed append().
+     */
+    virtual size_t getNextBatch(size_t batchSize, AppendBSONObjFn append);
+
+    /**
+     * Looping version of getNext() which exhausts the execution plan, but does nothing with any
+     * resulting documents.
+     */
+    virtual void executeExhaustive();
+
     /**
      * Similar to 'getNext()', but returns a Document rather than a BSONObj.
      *
@@ -288,7 +400,7 @@ public:
      * If this plan executor has already executed an update operation, returns the an 'UpdateResult'
      * describing the outcome of the update. Illegal to call if either 1) the PlanExecutor is not
      * an update PlanExecutor, or 2) the PlanExecutor has not yet been executed either with
-     * 'executeUpdate()' or by calling 'getNext()' until end-of-stream.
+     * 'executeUpdate()' or by calling 'getNext()' until ADVANCED or end-of-stream.
      */
     virtual UpdateResult getUpdateResult() const = 0;
 
@@ -299,6 +411,14 @@ public:
      */
     virtual long long executeDelete() = 0;
 
+    /**
+     * If this plan executor has already executed a delete operation, returns the the number of
+     * documents that were deleted. Illegal to call if either 1) the PlanExecutor is not a delete
+     * PlanExecutor, or 2) the PlanExecutor has not yet been executed either with 'executeDelete()'
+     * or by calling 'getNext()' until ADVANCED or end-of-stream.
+     */
+    virtual long long getDeleteResult() const = 0;
+
     /**
      * If this plan executor has already executed a batched delete operation, returns the
      * 'BatchedDeleteStats' describing the outcome of the batched delete. Illegal to call if either
@@ -417,6 +537,13 @@ public:
      * Sets whether the executor needs to return owned data.
      */
     virtual void setReturnOwnedData(bool returnOwnedData){};
+
+    /** TODO: SERVER-76397 Remove this once we use acquisitions everywhere.
+     *
+     * Returns whether the plan executor uses shard role acquisitions or the legacy
+     * CollectionPtr/AutoGet approach.
+     */
+    virtual bool usesCollectionAcquisitions() const = 0;
 };
 
 }  // namespace mongo
