@@ -28,52 +28,62 @@
  */
 
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/query/plan_executor_impl.h"
-
-#include "mongo/util/duration.h"
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr.hpp>
+#include <cstddef>
+#include <fmt/format.h>
 #include <memory>
+#include <string>
+#include <utility>
+#include <variant>
 
-#include "mongo/bson/simple_bsonobj_comparator.h"
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+
+#include "mongo/base/error_codes.h"
 #include "mongo/db/catalog/collection.h"
 #include "mongo/db/concurrency/exception_util.h"
 #include "mongo/db/curop.h"
 #include "mongo/db/exec/cached_plan.h"
 #include "mongo/db/exec/collection_scan.h"
-#include "mongo/db/exec/count_scan.h"
-#include "mongo/db/exec/distinct_scan.h"
-#include "mongo/db/exec/idhack.h"
-#include "mongo/db/exec/index_scan.h"
-#include "mongo/db/exec/near.h"
+#include "mongo/db/exec/document_value/document_metadata_fields.h"
 #include "mongo/db/exec/plan_stage.h"
 #include "mongo/db/exec/plan_stats.h"
-#include "mongo/db/exec/sort.h"
 #include "mongo/db/exec/subplan.h"
+#include "mongo/db/exec/timeseries_modify.h"
 #include "mongo/db/exec/trial_stage.h"
 #include "mongo/db/exec/update_stage.h"
 #include "mongo/db/exec/working_set.h"
-#include "mongo/db/exec/working_set_common.h"
+#include "mongo/db/query/cursor_response.h"
+#include "mongo/db/query/find_command.h"
+#include "mongo/db/query/find_common.h"
 #include "mongo/db/query/mock_yield_policies.h"
+#include "mongo/db/query/plan_executor_impl.h"
 #include "mongo/db/query/plan_explainer_factory.h"
 #include "mongo/db/query/plan_explainer_impl.h"
 #include "mongo/db/query/plan_insert_listener.h"
 #include "mongo/db/query/plan_yield_policy_impl.h"
+#include "mongo/db/query/stage_types.h"
 #include "mongo/db/query/yield_policy_callbacks_impl.h"
-#include "mongo/db/repl/replication_coordinator.h"
+#include "mongo/db/repl/optime.h"
 #include "mongo/db/s/operation_sharding_state.h"
 #include "mongo/db/service_context.h"
+#include "mongo/db/shard_role.h"
+#include "mongo/db/transaction_resources.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/util/decorable.h"
 #include "mongo/util/fail_point.h"
-#include "mongo/util/scopeguard.h"
-#include "mongo/util/stacktrace.h"
+#include "mongo/util/future.h"
+#include "mongo/util/intrusive_counter.h"
+#include "mongo/util/namespace_string_util.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
 
 namespace mongo {
-
-using std::shared_ptr;
+using namespace fmt::literals;
 using std::string;
 using std::unique_ptr;
 using std::vector;
@@ -85,64 +95,38 @@ const OperationContext::Decoration<boost::optional<repl::OpTime>> clientsLastKno
 // namespace.
 MONGO_FAIL_POINT_DEFINE(planExecutorHangBeforeShouldWaitForInserts);
 
-namespace {
-
-/**
- * Constructs a PlanYieldPolicy based on 'policy'.
- */
-std::unique_ptr<PlanYieldPolicy> makeYieldPolicy(PlanExecutorImpl* exec,
-                                                 PlanYieldPolicy::YieldPolicy policy,
-                                                 const Yieldable* yieldable) {
-    switch (policy) {
-        case PlanYieldPolicy::YieldPolicy::YIELD_AUTO:
-        case PlanYieldPolicy::YieldPolicy::YIELD_MANUAL:
-        case PlanYieldPolicy::YieldPolicy::NO_YIELD:
-        case PlanYieldPolicy::YieldPolicy::WRITE_CONFLICT_RETRY_ONLY:
-        case PlanYieldPolicy::YieldPolicy::INTERRUPT_ONLY: {
-            return std::make_unique<PlanYieldPolicyImpl>(
-                exec, policy, yieldable, std::make_unique<YieldPolicyCallbacksImpl>(exec->nss()));
-        }
-        case PlanYieldPolicy::YieldPolicy::ALWAYS_TIME_OUT: {
-            return std::make_unique<AlwaysTimeOutYieldPolicy>(
-                exec->getOpCtx(), exec->getOpCtx()->getServiceContext()->getFastClockSource());
-        }
-        case PlanYieldPolicy::YieldPolicy::ALWAYS_MARK_KILLED: {
-            return std::make_unique<AlwaysPlanKilledYieldPolicy>(
-                exec->getOpCtx(), exec->getOpCtx()->getServiceContext()->getFastClockSource());
-        }
-        default:
-            MONGO_UNREACHABLE;
-    }
-}
-}  // namespace
-
 PlanExecutorImpl::PlanExecutorImpl(OperationContext* opCtx,
                                    unique_ptr<WorkingSet> ws,
                                    unique_ptr<PlanStage> rt,
                                    unique_ptr<QuerySolution> qs,
                                    unique_ptr<CanonicalQuery> cq,
                                    const boost::intrusive_ptr<ExpressionContext>& expCtx,
-                                   const CollectionPtr& collection,
+                                   VariantCollectionPtrOrAcquisition collection,
                                    bool returnOwnedBson,
                                    NamespaceString nss,
-                                   PlanYieldPolicy::YieldPolicy yieldPolicy)
+                                   PlanYieldPolicy::YieldPolicy yieldPolicy,
+                                   boost::optional<size_t> cachedPlanHash)
     : _opCtx(opCtx),
       _cq(std::move(cq)),
       _expCtx(_cq ? _cq->getExpCtx() : expCtx),
       _workingSet(std::move(ws)),
       _qs(std::move(qs)),
       _root(std::move(rt)),
-      _planExplainer(plan_explainer_factory::make(_root.get())),
+      _planExplainer(plan_explainer_factory::make(_root.get(), cachedPlanHash)),
       _mustReturnOwnedBson(returnOwnedBson),
       _nss(std::move(nss)) {
     invariant(!_expCtx || _expCtx->opCtx == _opCtx);
     invariant(!_cq || !_expCtx || _cq->getExpCtx() == _expCtx);
 
+    const CollectionPtr* collectionPtr = &collection.getCollectionPtr();
+    invariant(collectionPtr);
+    const bool collectionExists = static_cast<bool>(*collectionPtr);
+
     // If we don't yet have a namespace string, then initialize it from either 'collection' or
     // '_cq'.
     if (_nss.isEmpty()) {
-        if (collection) {
-            _nss = collection->ns();
+        if (collectionExists) {
+            _nss = (*collectionPtr)->ns();
         } else {
             invariant(_cq);
             if (_cq->getFindCommandRequest().getNamespaceOrUUID().isNamespaceString()) {
@@ -151,20 +135,20 @@ PlanExecutorImpl::PlanExecutorImpl(OperationContext* opCtx,
         }
     }
 
-    // There's no point in yielding if the collection doesn't exist.
-    _yieldPolicy =
-        makeYieldPolicy(this,
-                        collection ? yieldPolicy : PlanYieldPolicy::YieldPolicy::NO_YIELD,
-                        collection ? &collection : nullptr);
-
-    uassertStatusOK(_pickBestPlan());
+    _yieldPolicy = makeClassicYieldPolicy(
+        _opCtx,
+        _nss,
+        this,
+        collectionExists ? yieldPolicy : PlanYieldPolicy::YieldPolicy::INTERRUPT_ONLY,
+        collection);
 
     if (_qs) {
+        _planExplainer->setQuerySolution(_qs.get());
         _planExplainer->updateEnumeratorExplainInfo(_qs->_enumeratorExplainInfo);
     } else if (const MultiPlanStage* mps = getMultiPlanStage()) {
         const QuerySolution* soln = mps->bestSolution();
+        _planExplainer->setQuerySolution(soln);
         _planExplainer->updateEnumeratorExplainInfo(soln->_enumeratorExplainInfo);
-
     } else if (auto subplan = getStageByType(_root.get(), STAGE_SUBPLAN)) {
         auto subplanStage = static_cast<SubplanStage*>(subplan);
         _planExplainer->updateEnumeratorExplainInfo(
@@ -179,46 +163,6 @@ PlanExecutorImpl::PlanExecutorImpl(OperationContext* opCtx,
     }
 }
 
-Status PlanExecutorImpl::_pickBestPlan() {
-    invariant(_currentState == kUsable);
-
-    // First check if we need to do subplanning.
-    PlanStage* foundStage = getStageByType(_root.get(), STAGE_SUBPLAN);
-    if (foundStage) {
-        SubplanStage* subplan = static_cast<SubplanStage*>(foundStage);
-        return subplan->pickBestPlan(_yieldPolicy.get());
-    }
-
-    // If we didn't have to do subplanning, we might still have to do regular
-    // multi plan selection...
-    foundStage = getStageByType(_root.get(), STAGE_MULTI_PLAN);
-    if (foundStage) {
-        MultiPlanStage* mps = static_cast<MultiPlanStage*>(foundStage);
-        return mps->pickBestPlan(_yieldPolicy.get());
-    }
-
-    // ...or, we might have to run a plan from the cache for a trial period, falling back on
-    // regular planning if the cached plan performs poorly.
-    foundStage = getStageByType(_root.get(), STAGE_CACHED_PLAN);
-    if (foundStage) {
-        CachedPlanStage* cachedPlan = static_cast<CachedPlanStage*>(foundStage);
-        return cachedPlan->pickBestPlan(_yieldPolicy.get());
-    }
-
-    // Finally, we might have an explicit TrialPhase. This specifies exactly two candidate
-    // plans, one of which is to be evaluated. If it fails the trial, then the backup plan is
-    // adopted.
-    foundStage = getStageByType(_root.get(), STAGE_TRIAL);
-    if (foundStage) {
-        TrialStage* trialStage = static_cast<TrialStage*>(foundStage);
-        return trialStage->pickBestPlan(_yieldPolicy.get());
-    }
-
-    // Either we chose a plan, or no plan selection was required. In both cases,
-    // our work has been successfully completed.
-    return Status::OK();
-}
-
 PlanExecutorImpl::~PlanExecutorImpl() {
     invariant(_currentState == kDisposed);
 }
@@ -253,7 +197,10 @@ void PlanExecutorImpl::saveState() {
     if (!isMarkedAsKilled()) {
         _root->saveState();
     }
-    _yieldPolicy->setYieldable(nullptr);
+
+    if (!_yieldPolicy->usesCollectionAcquisitions()) {
+        _yieldPolicy->setYieldable(nullptr);
+    }
     _currentState = kSaved;
 }
 
@@ -273,7 +220,9 @@ void PlanExecutorImpl::restoreStateWithoutRetrying(const RestoreContext& context
                                                    const Yieldable* yieldable) {
     invariant(_currentState == kSaved);
 
-    _yieldPolicy->setYieldable(yieldable);
+    if (!_yieldPolicy->usesCollectionAcquisitions()) {
+        _yieldPolicy->setYieldable(yieldable);
+    }
     if (!isMarkedAsKilled()) {
         _root->restoreState(context);
     }
@@ -307,6 +256,41 @@ void PlanExecutorImpl::reattachToOperationContext(OperationContext* opCtx) {
     _currentState = kSaved;
 }
 
+namespace {
+/**
+ * Helper function used to determine if we need to hang before inserts.
+ */
+void hangBeforeShouldWaitForInsertsIfFailpointEnabled(PlanExecutorImpl* exec) {
+    if (MONGO_unlikely(
+            planExecutorHangBeforeShouldWaitForInserts.shouldFail([exec](const BSONObj& data) {
+                auto fpNss = NamespaceStringUtil::parseFailPointData(data, "namespace"_sd);
+                return fpNss.isEmpty() || fpNss == exec->nss();
+            }))) {
+        LOGV2(20946,
+              "PlanExecutor - planExecutorHangBeforeShouldWaitForInserts fail point "
+              "enabled. Blocking until fail point is disabled");
+        planExecutorHangBeforeShouldWaitForInserts.pauseWhileSet();
+    }
+}
+
+/**
+ * Helper function used to construct lambda passed into yielding logic.
+ */
+void doYield(OperationContext* opCtx) {
+    // If we yielded because we encountered a sharding critical section, wait for the critical
+    // section to end before continuing. By waiting for the critical section to be exited we avoid
+    // busy spinning immediately and encountering the same critical section again. It is important
+    // that this wait happens after having released the lock hierarchy -- otherwise deadlocks could
+    // happen, or the very least, locks would be unnecessarily held while waiting.
+    const auto& shardingCriticalSection = planExecutorShardingState(opCtx).criticalSectionFuture;
+    if (shardingCriticalSection) {
+        OperationShardingState::waitForCriticalSectionToComplete(opCtx, *shardingCriticalSection)
+            .ignore();
+        planExecutorShardingState(opCtx).criticalSectionFuture.reset();
+    }
+}
+}  // namespace
+
 PlanExecutor::ExecState PlanExecutorImpl::getNext(BSONObj* objOut, RecordId* dlOut) {
     const auto state = getNextDocument(&_docOutput, dlOut);
     if (objOut && state == ExecState::ADVANCED) {
@@ -354,11 +338,8 @@ PlanExecutor::ExecState PlanExecutorImpl::_getNextImpl(Snapshotted<Document>* ob
     // Capped insert data; declared outside the loop so we hold a shared pointer to the capped
     // insert notifier the entire time we are in the loop.  Holding a shared pointer to the
     // capped insert notifier is necessary for the notifierVersion to advance.
-    std::unique_ptr<insert_listener::Notifier> notifier;
-    if (insert_listener::shouldListenForInserts(_opCtx, _cq.get())) {
-        // We always construct the insert_listener::Notifier for awaitData cursors.
-        notifier = insert_listener::getCappedInsertNotifier(_opCtx, _nss, _yieldPolicy.get());
-    }
+    auto notifier = makeNotifier();
+
     for (;;) {
         // These are the conditions which can cause us to yield:
         //   1) The yield policy's timer elapsed, or
@@ -367,19 +348,7 @@ PlanExecutor::ExecState PlanExecutorImpl::_getNextImpl(Snapshotted<Document>* ob
         // In all cases, the actual yielding happens here.
 
         const auto whileYieldingFn = [&]() {
-            // If we yielded because we encountered a sharding critical section, wait for the
-            // critical section to end before continuing. By waiting for the critical section to be
-            // exited we avoid busy spinning immediately and encountering the same critical section
-            // again. It is important that this wait happens after having released the lock
-            // hierarchy -- otherwise deadlocks could happen, or the very least, locks would be
-            // unnecessarily held while waiting.
-            const auto& shardingCriticalSection = planExecutorShardingCriticalSectionFuture(_opCtx);
-            if (shardingCriticalSection) {
-                OperationShardingState::waitForCriticalSectionToComplete(_opCtx,
-                                                                         *shardingCriticalSection)
-                    .ignore();
-                planExecutorShardingCriticalSectionFuture(_opCtx).reset();
-            }
+            doYield(_opCtx);
         };
 
         if (_yieldPolicy->shouldYieldOrInterrupt(_opCtx)) {
@@ -439,73 +408,198 @@ PlanExecutor::ExecState PlanExecutorImpl::_getNextImpl(Snapshotted<Document>* ob
                 return PlanExecutor::ADVANCED;
             }
             // This result didn't have the data the caller wanted, try again.
+
         } else if (PlanStage::NEED_YIELD == code) {
-            invariant(id == WorkingSet::INVALID_ID);
-            invariant(_opCtx->recoveryUnit());
+            _handleNeedYield(writeConflictsInARow, tempUnavailErrorsInARow);
 
-            if (_expCtx->getTemporarilyUnavailableException()) {
-                _expCtx->setTemporarilyUnavailableException(false);
-
-                if (!_yieldPolicy->canAutoYield()) {
-                    throwTemporarilyUnavailableException(
-                        "got TemporarilyUnavailable exception on a plan that cannot auto-yield");
-                }
-
-                tempUnavailErrorsInARow++;
-                handleTemporarilyUnavailableException(
-                    _opCtx,
-                    tempUnavailErrorsInARow,
-                    "plan executor",
-                    _nss.ns(),
-                    ExceptionFor<ErrorCodes::TemporarilyUnavailable>(
-                        Status(ErrorCodes::TemporarilyUnavailable, "temporarily unavailable")),
-                    writeConflictsInARow);
-            } else {
-                // We're yielding because of a WriteConflictException.
-                if (!_yieldPolicy->canAutoYield() ||
-                    MONGO_unlikely(skipWriteConflictRetries.shouldFail())) {
-                    throwWriteConflictException(
-                        "Write conflict during plan execution and yielding is disabled.");
-                }
-
-                CurOp::get(_opCtx)->debug().additiveMetrics.incrementWriteConflicts(1);
-                writeConflictsInARow++;
-                logWriteConflictAndBackoff(writeConflictsInARow, "plan execution", _nss.ns());
-            }
-
-            // Yield next time through the loop.
-            invariant(_yieldPolicy->canAutoYield());
-            _yieldPolicy->forceYield();
         } else if (PlanStage::NEED_TIME == code) {
             // Fall through to yield check at end of large conditional.
-        } else {
-            invariant(PlanStage::IS_EOF == code);
-            if (MONGO_unlikely(planExecutorHangBeforeShouldWaitForInserts.shouldFail(
-                    [this](const BSONObj& data) {
-                        if (data.hasField("namespace") &&
-                            _nss != NamespaceString(data.getStringField("namespace"))) {
-                            return false;
-                        }
-                        return true;
-                    }))) {
-                LOGV2(20946,
-                      "PlanExecutor - planExecutorHangBeforeShouldWaitForInserts fail point "
-                      "enabled. Blocking until fail point is disabled");
-                planExecutorHangBeforeShouldWaitForInserts.pauseWhileSet();
-            }
 
-            if (!insert_listener::shouldWaitForInserts(_opCtx, _cq.get(), _yieldPolicy.get())) {
-                return PlanExecutor::IS_EOF;
-            }
-
-            insert_listener::waitForInserts(_opCtx, _yieldPolicy.get(), notifier);
-
-            // There may be more results, keep going.
-            continue;
+        } else if (_handleEOFAndExit(code, notifier)) {
+            return PlanExecutor::IS_EOF;
         }
     }
 }
 
+namespace {
+BSONObj makeBsonWithMetadata(Document& doc, WorkingSetMember* member) {
+    if (member->metadata()) {
+        MutableDocument md(std::move(doc));
+        md.setMetadata(member->releaseMetadata());
+        return md.freeze().toBsonWithMetaData();
+    }
+
+    return doc.toBsonWithMetaData();
+}
+}  // namespace
+
+std::unique_ptr<insert_listener::Notifier> PlanExecutorImpl::makeNotifier() {
+    if (insert_listener::shouldListenForInserts(_opCtx, _cq.get())) {
+        // We always construct the insert_listener::Notifier for awaitData cursors.
+        return insert_listener::getCappedInsertNotifier(_opCtx, _nss, _yieldPolicy.get());
+    }
+    return nullptr;
+}
+
+void PlanExecutorImpl::_handleNeedYield(size_t& writeConflictsInARow,
+                                        size_t& tempUnavailErrorsInARow) {
+    invariant(shard_role_details::getRecoveryUnit(_opCtx));
+
+    if (_expCtx->getTemporarilyUnavailableException()) {
+        _expCtx->setTemporarilyUnavailableException(false);
+
+        if (!_yieldPolicy->canAutoYield()) {
+            throwTemporarilyUnavailableException(
+                "got TemporarilyUnavailable exception on a plan that "
+                "cannot "
+                "auto-yield");
+        }
+
+        tempUnavailErrorsInARow++;
+        handleTemporarilyUnavailableException(
+            _opCtx,
+            tempUnavailErrorsInARow,
+            "plan executor",
+            NamespaceStringOrUUID(_nss),
+            ExceptionFor<ErrorCodes::TemporarilyUnavailable>(
+                Status(ErrorCodes::TemporarilyUnavailable, "temporarily unavailable")),
+            writeConflictsInARow);
+
+    } else {
+        // We're yielding because of a WriteConflictException.
+        if (!_yieldPolicy->canAutoYield() ||
+            MONGO_unlikely(skipWriteConflictRetries.shouldFail())) {
+            throwWriteConflictException(
+                "Write conflict during plan execution and yielding is "
+                "disabled.");
+        }
+
+        CurOp::get(_opCtx)->debug().additiveMetrics.incrementWriteConflicts(1);
+        writeConflictsInARow++;
+        logWriteConflictAndBackoff(
+            writeConflictsInARow, "plan execution", ""_sd, NamespaceStringOrUUID(_nss));
+    }
+
+    // Yield next time through the loop.
+    invariant(_yieldPolicy->canAutoYield());
+    _yieldPolicy->forceYield();
+}
+
+bool PlanExecutorImpl::_handleEOFAndExit(PlanStage::StageState code,
+                                         std::unique_ptr<insert_listener::Notifier>& notifier) {
+    invariant(PlanStage::IS_EOF == code);
+    hangBeforeShouldWaitForInsertsIfFailpointEnabled(this);
+
+    // The !notifier check is necessary because shouldWaitForInserts can return 'true' when
+    // shouldListenForInserts returned 'false' (above) in the case of a deadline becoming
+    // "unexpired" due to the system clock going backwards.
+    if (!notifier ||
+        !insert_listener::shouldWaitForInserts(_opCtx, _cq.get(), _yieldPolicy.get())) {
+        // Time to exit.
+        return true;
+    }
+
+    insert_listener::waitForInserts(_opCtx, _yieldPolicy.get(), notifier);
+    return false;
+}
+
+size_t PlanExecutorImpl::getNextBatch(size_t batchSize, AppendBSONObjFn append) {
+    const bool includeMetadata = _expCtx && _expCtx->needsMerge;
+    if (batchSize == 0) {
+        return 0;
+    }
+
+    checkFailPointPlanExecAlwaysFails();
+    _checkIfKilled();
+
+    const auto whileYieldingFn = [opCtx = _opCtx]() {
+        return doYield(opCtx);
+    };
+    auto notifier = makeNotifier();
+
+    WorkingSetID id = WorkingSet::INVALID_ID;
+    WorkingSetMember* member;
+    PlanStage::StageState code;
+
+    // The below are incremented on every WriteConflict or TemporarilyUnavailable error
+    // accordingly, and reset to 0 on any successful call to _root->work.
+    size_t writeConflictsInARow = 0;
+    size_t tempUnavailErrorsInARow = 0;
+
+    size_t numResults = 0;
+    BSONObj objOut;
+
+    // Handle case where previous execution stashed a result.
+    if (!_stash.empty()) {
+        objOut = includeMetadata ? _stash.front().toBson() : _stash.front().toBsonWithMetaData();
+        _stash.pop_front();
+        append(objOut, getPostBatchResumeToken(), numResults);
+        numResults++;
+    }
+
+    for (;;) {
+        _checkIfMustYield(whileYieldingFn);
+
+        code = _root->work(&id);
+
+        if (code != PlanStage::NEED_YIELD) {
+            writeConflictsInARow = 0;
+            tempUnavailErrorsInARow = 0;
+        }
+
+        if (code == PlanStage::ADVANCED) {
+            // Process working set member.
+            member = _workingSet->get(id);
+            if (MONGO_likely(member->hasObj())) {
+                if (includeMetadata) {
+                    objOut = makeBsonWithMetadata(member->doc.value(), member);
+                } else {
+                    objOut = member->doc.value().toBson();
+                }
+
+            } else if (member->keyData.size() >= 1) {
+                if (includeMetadata) {
+                    _docOutput = Document{member->keyData[0].keyData};
+                    objOut = makeBsonWithMetadata(_docOutput, member);
+                } else {
+                    objOut = member->keyData[0].keyData;
+                }
+
+            } else {
+                _workingSet->free(id);
+                continue;  // Try to call work() again- we didn't get what we needed.
+            }
+
+            _workingSet->free(id);
+
+            if (MONGO_unlikely(!append(objOut, getPostBatchResumeToken(), numResults))) {
+                stashResult(objOut);
+                break;
+            }
+            numResults++;
+
+            // Only check if the query has been killed or if we've filled up the batch once a result
+            // has been produced. Doing these checks every loop can impact the performace of queries
+            // that repeatedly return NEED_TIME.
+            if (MONGO_unlikely(numResults >= batchSize)) {
+                break;
+            }
+
+            _checkIfKilled();
+
+        } else if (code == PlanStage::NEED_YIELD) {
+            _handleNeedYield(writeConflictsInARow, tempUnavailErrorsInARow);
+
+        } else if (code == PlanStage::NEED_TIME) {
+            // Do nothing except reset counters; need more time.
+
+        } else if (_handleEOFAndExit(code, notifier)) {
+            break;
+        }
+    }
+    return numResults;
+}
+
 bool PlanExecutorImpl::isEOF() {
     invariant(_currentState == kUsable);
     return isMarkedAsKilled() || (_stash.empty() && _root->isEOF());
@@ -523,33 +617,88 @@ void PlanExecutorImpl::dispose(OperationContext* opCtx) {
     _currentState = kDisposed;
 }
 
-void PlanExecutorImpl::_executePlan() {
-    invariant(_currentState == kUsable);
-    Document obj;
-    PlanExecutor::ExecState state = PlanExecutor::ADVANCED;
-    while (PlanExecutor::ADVANCED == state) {
-        state = this->getNextDocument(&obj, nullptr);
-    }
+void PlanExecutorImpl::executeExhaustive() {
+    // We don't check batch size or do anything with returned BSON in exhaustDoWork().
+    checkFailPointPlanExecAlwaysFails();
+    _checkIfKilled();
 
-    if (isMarkedAsKilled()) {
-        uassertStatusOK(_killStatus);
-    }
+    const auto whileYieldingFn = [opCtx = _opCtx]() {
+        return doYield(opCtx);
+    };
+    auto notifier = makeNotifier();
 
-    invariant(!isMarkedAsKilled());
-    invariant(PlanExecutor::IS_EOF == state);
+    WorkingSetID id = WorkingSet::INVALID_ID;
+    PlanStage::StageState code;
+
+    // The below are incremented on every WriteConflict or TemporarilyUnavailable error
+    // accordingly, and reset to 0 on any successful call to _root->work.
+    size_t writeConflictsInARow = 0;
+    size_t tempUnavailErrorsInARow = 0;
+
+    for (;;) {
+        _checkIfMustYield(whileYieldingFn);
+
+        code = _root->work(&id);
+
+        if (code != PlanStage::NEED_YIELD) {
+            writeConflictsInARow = 0;
+            tempUnavailErrorsInARow = 0;
+        }
+
+        if (code == PlanStage::ADVANCED) {
+            // Free WSM.
+            _workingSet->free(id);
+
+            // Only check if the query has been killed or if we've filled up the batch once a result
+            // has been produced. Doing these checks every loop can impact the performace of queries
+            // that repeatedly return NEED_TIME.
+            _checkIfKilled();
+
+        } else if (code == PlanStage::NEED_YIELD) {
+            _handleNeedYield(writeConflictsInARow, tempUnavailErrorsInARow);
+
+        } else if (code == PlanStage::NEED_TIME) {
+            // Do nothing except reset counters; need more time.
+
+        } else if (_handleEOFAndExit(code, notifier)) {
+            break;
+        }
+    }
 }
 
 long long PlanExecutorImpl::executeCount() {
     invariant(_root->stageType() == StageType::STAGE_COUNT ||
               _root->stageType() == StageType::STAGE_RECORD_STORE_FAST_COUNT);
 
-    _executePlan();
+    executeExhaustive();
     auto countStats = static_cast<const CountStats*>(_root->getSpecificStats());
     return countStats->nCounted;
 }
 
 UpdateResult PlanExecutorImpl::executeUpdate() {
-    _executePlan();
+    try {
+        executeExhaustive();
+    } catch (ExceptionFor<ErrorCodes::StaleConfig>& ex) {
+        const auto updateResult = getUpdateResult();
+        tassert(
+            9146500,
+            fmt::format(
+                "An update plan should never yield after having performed an upsert; upsertId: {}",
+                redact(updateResult.upsertedId.toString())),
+            updateResult.upsertedId.isEmpty());
+        if (updateResult.numDocsModified > 0 && !_opCtx->isRetryableWrite() &&
+            !_opCtx->inMultiDocumentTransaction()) {
+            // An update plan can fail with StaleConfig error after having performed some writes but
+            // not completed. This can happen when the collection is moved. Routers consider
+            // StaleConfig as retryable. However, it is unsafe to retry, because if the update is
+            // not idempotent it would cause some documents to be updated twice. To prevent that, we
+            // rewrite the error code to QueryPlanKilled, which routers won't retry on.
+            ex.addContext("Update plan failed after having partially executed");
+            uasserted(ErrorCodes::QueryPlanKilled, ex.reason());
+        } else {
+            throw;
+        }
+    }
     return getUpdateResult();
 }
 
@@ -572,30 +721,54 @@ UpdateResult PlanExecutorImpl::getUpdateResult() const {
     }
 
     // If the collection exists, then we expect the root of the plan tree to either be an update
-    // stage, or (for findAndModify) a projection stage wrapping an update stage.
-    switch (_root->stageType()) {
-        case StageType::STAGE_PROJECTION_DEFAULT:
-        case StageType::STAGE_PROJECTION_COVERED:
-        case StageType::STAGE_PROJECTION_SIMPLE: {
-            invariant(_root->getChildren().size() == 1U);
-            invariant(StageType::STAGE_UPDATE == _root->child()->stageType());
-            const SpecificStats* stats = _root->child()->getSpecificStats();
+    // stage, or (for findAndModify) a projection stage wrapping an update / TS_MODIFY stage.
+    const auto updateStage = [&] {
+        switch (_root->stageType()) {
+            case StageType::STAGE_PROJECTION_DEFAULT:
+            case StageType::STAGE_PROJECTION_COVERED:
+            case StageType::STAGE_PROJECTION_SIMPLE: {
+                tassert(7314604,
+                        "Unexpected number of children: {}"_format(_root->getChildren().size()),
+                        _root->getChildren().size() == 1U);
+                auto childStage = _root->child().get();
+                tassert(7314605,
+                        "Unexpected child stage type: {}"_format(childStage->stageType()),
+                        StageType::STAGE_UPDATE == childStage->stageType() ||
+                            StageType::STAGE_TIMESERIES_MODIFY == childStage->stageType());
+                return childStage;
+            }
+            default:
+                return _root.get();
+        }
+    }();
+    switch (updateStage->stageType()) {
+        case StageType::STAGE_TIMESERIES_MODIFY: {
+            const auto& stats =
+                static_cast<const TimeseriesModifyStats&>(*updateStage->getSpecificStats());
+            return UpdateResult(
+                stats.nMeasurementsModified > 0 /* Did we update at least one obj? */,
+                stats.isModUpdate /* Is this a $mod update? */,
+                stats.nMeasurementsModified /* number of modified docs, no no-ops */,
+                stats.nMeasurementsMatched /* # of docs matched/updated, even no-ops */,
+                stats.objInserted /* objInserted */,
+                static_cast<TimeseriesModifyStage*>(updateStage)->containsDotsAndDollarsField());
+        }
+        case StageType::STAGE_UPDATE: {
+            const auto& stats = static_cast<const UpdateStats&>(*updateStage->getSpecificStats());
             return updateStatsToResult(
-                static_cast<const UpdateStats&>(*stats),
-                static_cast<UpdateStage*>(_root->child().get())->containsDotsAndDollarsField());
+                stats, static_cast<UpdateStage*>(updateStage)->containsDotsAndDollarsField());
         }
         default:
-            invariant(StageType::STAGE_UPDATE == _root->stageType());
-            const auto stats = _root->getSpecificStats();
-            return updateStatsToResult(
-                static_cast<const UpdateStats&>(*stats),
-                static_cast<UpdateStage*>(_root.get())->containsDotsAndDollarsField());
+            MONGO_UNREACHABLE_TASSERT(7314606);
     }
 }
 
 long long PlanExecutorImpl::executeDelete() {
-    _executePlan();
+    executeExhaustive();
+    return getDeleteResult();
+}
 
+long long PlanExecutorImpl::getDeleteResult() const {
     // If we're deleting from a non-existent collection, then the delete plan may have an EOF as
     // the root stage.
     if (_root->stageType() == STAGE_EOF) {
@@ -603,27 +776,40 @@ long long PlanExecutorImpl::executeDelete() {
     }
 
     // If the collection exists, the delete plan may either have a delete stage at the root, or
-    // (for findAndModify) a projection stage wrapping a delete stage.
-    switch (_root->stageType()) {
-        case StageType::STAGE_PROJECTION_DEFAULT:
-        case StageType::STAGE_PROJECTION_COVERED:
-        case StageType::STAGE_PROJECTION_SIMPLE: {
-            invariant(_root->getChildren().size() == 1U);
-            invariant(StageType::STAGE_DELETE == _root->child()->stageType());
-            const SpecificStats* stats = _root->child()->getSpecificStats();
-            return static_cast<const DeleteStats*>(stats)->docsDeleted;
+    // (for findAndModify) a projection stage wrapping a delete / TS_MODIFY stage.
+    const auto deleteStage = [&] {
+        switch (_root->stageType()) {
+            case StageType::STAGE_PROJECTION_DEFAULT:
+            case StageType::STAGE_PROJECTION_COVERED:
+            case StageType::STAGE_PROJECTION_SIMPLE: {
+                tassert(7308302,
+                        "Unexpected number of children: {}"_format(_root->getChildren().size()),
+                        _root->getChildren().size() == 1U);
+                auto childStage = _root->child().get();
+                tassert(7308303,
+                        "Unexpected child stage type: {}"_format(childStage->stageType()),
+                        StageType::STAGE_DELETE == childStage->stageType() ||
+                            StageType::STAGE_TIMESERIES_MODIFY == childStage->stageType());
+                return childStage;
+            }
+            default:
+                return _root.get();
         }
+    }();
+    switch (deleteStage->stageType()) {
         case StageType::STAGE_TIMESERIES_MODIFY: {
-            const auto* tsModifyStats =
-                static_cast<const TimeseriesModifyStats*>(_root->getSpecificStats());
-            return tsModifyStats->nMeasurementsDeleted;
+            const auto& tsModifyStats =
+                static_cast<const TimeseriesModifyStats&>(*deleteStage->getSpecificStats());
+            return tsModifyStats.nMeasurementsModified;
         }
-        default: {
-            invariant(StageType::STAGE_DELETE == _root->stageType() ||
-                      StageType::STAGE_BATCHED_DELETE == _root->stageType());
-            const auto* deleteStats = static_cast<const DeleteStats*>(_root->getSpecificStats());
-            return deleteStats->docsDeleted;
+        case StageType::STAGE_DELETE:
+        case StageType::STAGE_BATCHED_DELETE: {
+            const auto& deleteStats =
+                static_cast<const DeleteStats&>(*deleteStage->getSpecificStats());
+            return deleteStats.docsDeleted;
         }
+        default:
+            MONGO_UNREACHABLE_TASSERT(7308306);
     }
 }
 
@@ -648,10 +834,6 @@ void PlanExecutorImpl::stashResult(const BSONObj& obj) {
     _stash.push_front(Document{obj.getOwned()});
 }
 
-bool PlanExecutorImpl::isMarkedAsKilled() const {
-    return !_killStatus.isOK();
-}
-
 Status PlanExecutorImpl::getKillStatus() {
     invariant(isMarkedAsKilled());
     return _killStatus;
@@ -690,4 +872,8 @@ MultiPlanStage* PlanExecutorImpl::getMultiPlanStage() const {
     invariant(ps == nullptr || ps->stageType() == StageType::STAGE_MULTI_PLAN);
     return static_cast<MultiPlanStage*>(ps);
 }
+
+bool PlanExecutorImpl::usesCollectionAcquisitions() const {
+    return _yieldPolicy->usesCollectionAcquisitions();
+}
 }  // namespace mongo
