@@ -29,35 +29,85 @@
 
 #pragma once
 
-#include "mongo/util/duration.h"
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <deque>
+#include <memory>
 #include <queue>
+#include <utility>
+#include <vector>
 
+#include "mongo/base/status.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/db/exec/document_value/document.h"
+#include "mongo/db/exec/multi_plan.h"
+#include "mongo/db/exec/plan_stats.h"
+#include "mongo/db/exec/sbe/expressions/runtime_environment.h"
+#include "mongo/db/exec/sbe/stages/plan_stats.h"
 #include "mongo/db/exec/sbe/stages/stages.h"
+#include "mongo/db/exec/sbe/values/slot.h"
+#include "mongo/db/exec/sbe/values/value.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/ops/update_result.h"
+#include "mongo/db/query/canonical_query.h"
 #include "mongo/db/query/optimizer/explain_interface.h"
 #include "mongo/db/query/plan_executor.h"
+#include "mongo/db/query/plan_explainer.h"
 #include "mongo/db/query/plan_explainer_sbe.h"
 #include "mongo/db/query/plan_yield_policy_sbe.h"
+#include "mongo/db/query/query_solution.h"
+#include "mongo/db/query/restore_context.h"
 #include "mongo/db/query/sbe_plan_ranker.h"
 #include "mongo/db/query/sbe_runtime_planner.h"
-#include "mongo/db/query/sbe_stage_builder.h"
+#include "mongo/db/query/sbe_stage_builder_plan_data.h"
+#include "mongo/db/record_id.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/duration.h"
 
 namespace mongo {
 class PlanExecutorSBE final : public PlanExecutor {
 public:
+    struct MetaDataAccessor {
+        template <typename BSONTraits = BSONObj::DefaultSizeTrait>
+        BSONObj appendToBson(BSONObj doc) const;
+        Document appendToDocument(Document doc) const;
+        // Only for $search queries, holds the metadata returned from mongot.
+        sbe::value::SlotAccessor* metadataSearchScore{nullptr};
+        sbe::value::SlotAccessor* metadataSearchHighlights{nullptr};
+        sbe::value::SlotAccessor* metadataSearchDetails{nullptr};
+        sbe::value::SlotAccessor* metadataSearchSortValues{nullptr};
+        sbe::value::SlotAccessor* metadataSearchSequenceToken{nullptr};
+
+        sbe::value::SlotAccessor* sortKey{nullptr};
+        bool isSingleSortKey{true};
+    };
+
     PlanExecutorSBE(OperationContext* opCtx,
                     std::unique_ptr<CanonicalQuery> cq,
+                    std::unique_ptr<Pipeline, PipelineDeleter> pipeline,
                     std::unique_ptr<optimizer::AbstractABTPrinter> optimizerData,
                     sbe::CandidatePlans candidates,
                     bool returnOwnedBson,
                     NamespaceString nss,
                     bool isOpen,
                     std::unique_ptr<PlanYieldPolicySBE> yieldPolicy,
-                    bool generatedByBonsai);
+                    bool generatedByBonsai,
+                    boost::optional<size_t> cachedPlanHash,
+                    OptimizerCounterInfo optCounterInfo,
+                    std::unique_ptr<RemoteCursorMap> remoteCursors,
+                    std::unique_ptr<RemoteExplainVector> remoteExplains,
+                    std::unique_ptr<MultiPlanStage> classicRuntimePlannerStage);
 
     CanonicalQuery* getCanonicalQuery() const override {
         return _cq.get();
     }
 
+    Pipeline* getPipeline() const override {
+        return _pipeline.get();
+    }
+
     const NamespaceString& nss() const override {
         return _nss;
     }
@@ -70,11 +120,11 @@ public:
         return _opCtx;
     }
 
-    void saveState();
-    void restoreState(const RestoreContext& context);
+    void saveState() override;
+    void restoreState(const RestoreContext& context) override;
 
-    void detachFromOperationContext();
-    void reattachToOperationContext(OperationContext* opCtx);
+    void detachFromOperationContext() override;
+    void reattachToOperationContext(OperationContext* opCtx) override;
 
     ExecState getNext(BSONObj* out, RecordId* dlOut) override;
     ExecState getNextDocument(Document* objOut, RecordId* dlOut) override;
@@ -102,16 +152,21 @@ public:
         MONGO_UNREACHABLE;
     }
 
+    long long getDeleteResult() const override {
+        // Using SBE to execute a delete command is not yet supported.
+        MONGO_UNREACHABLE;
+    }
+
     BatchedDeleteStats getBatchedDeleteStats() override {
         // Using SBE to execute a batched delete command is not yet supported.
         MONGO_UNREACHABLE;
     }
 
-    void markAsKilled(Status killStatus);
+    void markAsKilled(Status killStatus) override;
 
-    void dispose(OperationContext* opCtx);
+    void dispose(OperationContext* opCtx) override;
 
-    void stashResult(const BSONObj& obj);
+    void stashResult(const BSONObj& obj) override;
 
     bool isMarkedAsKilled() const override {
         return !_killStatus.isOK();
@@ -149,19 +204,32 @@ public:
         return _isSaveRecoveryUnitAcrossCommandsEnabled;
     }
 
-    PlanExecutor::QueryFramework getQueryFramework() const override final {
+    PlanExecutor::QueryFramework getQueryFramework() const final {
         return _generatedByBonsai ? PlanExecutor::QueryFramework::kCQF
                                   : PlanExecutor::QueryFramework::kSBEOnly;
     }
 
-    void setReturnOwnedData(bool returnOwnedData) override final {
+    void setReturnOwnedData(bool returnOwnedData) final {
         _mustReturnOwnedBson = returnOwnedData;
     }
 
+    bool usesCollectionAcquisitions() const final;
+
+    /**
+     * For queries that have multiple executors, this can be used to differentiate between them.
+     */
+    boost::optional<StringData> getExecutorType() const final {
+        return CursorType_serializer(_cursorType);
+    }
+
 private:
     template <typename ObjectType>
     ExecState getNextImpl(ObjectType* out, RecordId* dlOut);
 
+    void initializeAccessors(MetaDataAccessor& accessor,
+                             const stage_builder::PlanStageMetadataSlots& metadataSlots,
+                             const QueryMetadataBitSet& metadataBit);
+
     enum class State { kClosed, kOpened };
 
     State _state{State::kClosed};
@@ -185,8 +253,18 @@ private:
     sbe::value::Value _valLastRecordId{0};
     sbe::RuntimeEnvironment::Accessor* _oplogTs{nullptr};
 
+    // Only for a resumed scan ("seek"). Slot holding the TypeTags::RecordId of the record to resume
+    // the scan from. '_seekRecordId' is the RecordId value, initialized from the slot at runtime.
     boost::optional<sbe::value::SlotId> _resumeRecordIdSlot;
 
+    // Only for clustered collection scans, holds the minimum record ID of the scan, if applicable.
+    boost::optional<sbe::value::SlotId> _minRecordIdSlot;
+
+    // Only for clustered collection scans, holds the maximum record ID of the scan, if applicable.
+    boost::optional<sbe::value::SlotId> _maxRecordIdSlot;
+
+    MetaDataAccessor _metadataAccessors;
+
     // NOTE: '_stash' stores documents as BSON. Currently, one of the '_stash' is usages is to store
     // documents received from the plan during multiplanning. This means that the documents
     // generated during multiplanning cannot exceed maximum BSON size. $group and $lookup CAN
@@ -209,6 +287,13 @@ private:
 
     std::unique_ptr<CanonicalQuery> _cq;
 
+    // For parameterized queries generated by Bonsai, we transfer ownership of the pipeline to the
+    // plan executor. This is to allow the slots in the runtime environment containing parameterized
+    // constants to be views over the MatchExpression contained in the pipeline; this allows us to
+    // avoid potentially expensive copies when binding constants into the plan. We also use this
+    // pipeline for explain purposes.
+    std::unique_ptr<Pipeline, PipelineDeleter> _pipeline;
+
     std::unique_ptr<PlanYieldPolicySBE> _yieldPolicy;
 
     std::unique_ptr<PlanExplainer> _planExplainer;
@@ -219,6 +304,15 @@ private:
 
     // Indicates whether this executor was constructed via Bonsai/CQF.
     bool _generatedByBonsai{false};
+
+    /**
+     * For commands that return multiple cursors, this value will contain the type of cursor.
+     * Default to a regular result cursor.
+     */
+    CursorTypeEnum _cursorType = CursorTypeEnum::DocumentResult;
+
+    std::unique_ptr<RemoteCursorMap> _remoteCursors;
+    std::unique_ptr<RemoteExplainVector> _remoteExplains;
 };
 
 /**
