@@ -27,19 +27,19 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <utility>
 
-#include "mongo/db/query/plan_explainer_factory.h"
+#include <boost/optional/optional.hpp>
 
 #include "mongo/db/exec/plan_cache_util.h"
+#include "mongo/db/query/plan_explainer_factory.h"
 #include "mongo/db/query/plan_explainer_impl.h"
 #include "mongo/db/query/plan_explainer_sbe.h"
-#include "mongo/util/duration.h"
-#include <ratio>
+#include "mongo/util/assert_util_core.h"
 
 namespace mongo::plan_explainer_factory {
-std::unique_ptr<PlanExplainer> make(PlanStage* root) {
-    return std::make_unique<PlanExplainerImpl>(root);
+std::unique_ptr<PlanExplainer> make(PlanStage* root, boost::optional<size_t> cachedPlanHash) {
+    return std::make_unique<PlanExplainerImpl>(root, cachedPlanHash);
 }
 
 std::unique_ptr<PlanExplainer> make(PlanStage* root, const PlanEnumeratorExplainInfo& explainInfo) {
@@ -67,6 +67,8 @@ std::unique_ptr<PlanExplainer> make(sbe::PlanStage* root,
                                               std::move(optimizerData),
                                               std::move(rejectedCandidates),
                                               isMultiPlan,
+                                              false,       /* isCachedPlan */
+                                              boost::none, /* cachedPlanHash */
                                               debugInfoSBE);
 }
 
@@ -77,21 +79,51 @@ std::unique_ptr<PlanExplainer> make(
     std::unique_ptr<optimizer::AbstractABTPrinter> optimizerData,
     std::vector<sbe::plan_ranker::CandidatePlan> rejectedCandidates,
     bool isMultiPlan,
-    std::shared_ptr<const plan_cache_debug_info::DebugInfoSBE> debugInfoSBE) {
-    // TODO SERVER-64882: Consider invariant(debugInfoSBE) as we may not need to create a
-    // DebugInfoSBE from QuerySolution after the feature flag is removed. We currently need it
-    // because debugInfoSBE can be null if the plan was recovered from the classic plan cache.
+    bool isFromPlanCache,
+    boost::optional<size_t> cachedPlanHash,
+    std::shared_ptr<const plan_cache_debug_info::DebugInfoSBE> debugInfoSBE,
+    OptimizerCounterInfo optCounterInfo,
+    RemoteExplainVector* remoteExplains) {
     if (!debugInfoSBE) {
         debugInfoSBE = std::make_shared<const plan_cache_debug_info::DebugInfoSBE>(
             plan_cache_util::buildDebugInfo(solution));
     }
-
     return std::make_unique<PlanExplainerSBE>(root,
                                               data,
                                               solution,
                                               std::move(optimizerData),
                                               std::move(rejectedCandidates),
                                               isMultiPlan,
-                                              debugInfoSBE);
+                                              isFromPlanCache,
+                                              cachedPlanHash,
+                                              debugInfoSBE,
+                                              std::move(optCounterInfo),
+                                              remoteExplains);
+}
+
+std::unique_ptr<PlanExplainer> make(
+    sbe::PlanStage* root,
+    const stage_builder::PlanStageData* data,
+    const QuerySolution* solution,
+    bool isMultiPlan,
+    bool isFromPlanCache,
+    boost::optional<size_t> cachedPlanHash,
+    std::shared_ptr<const plan_cache_debug_info::DebugInfoSBE> debugInfoSBE,
+    std::unique_ptr<PlanStage> classicRuntimePlannerStage,
+    RemoteExplainVector* remoteExplains) {
+    if (!debugInfoSBE) {
+        debugInfoSBE = std::make_shared<const plan_cache_debug_info::DebugInfoSBE>(
+            plan_cache_util::buildDebugInfo(solution));
+    }
+    return std::make_unique<PlanExplainerClassicRuntimePlannerForSBE>(
+        root,
+        data,
+        solution,
+        isMultiPlan,
+        isFromPlanCache,
+        cachedPlanHash,
+        debugInfoSBE,
+        std::move(classicRuntimePlannerStage),
+        remoteExplains);
 }
 }  // namespace mongo::plan_explainer_factory
