@@ -27,18 +27,26 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/query/plan_insert_listener.h"
-
 #include <memory>
+#include <string>
 
-#include "mongo/db/catalog/database.h"
-#include "mongo/db/catalog/database_holder.h"
+
+#include "mongo/db/basic_types.h"
+#include "mongo/db/catalog/collection.h"
+#include "mongo/db/catalog/collection_catalog.h"
 #include "mongo/db/curop.h"
+#include "mongo/db/query/find_command.h"
 #include "mongo/db/query/find_common.h"
+#include "mongo/db/query/plan_executor.h"
+#include "mongo/db/query/plan_insert_listener.h"
+#include "mongo/db/service_context.h"
+#include "mongo/db/transaction_resources.h"
 #include "mongo/logv2/log.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/util/clock_source.h"
+#include "mongo/util/decorable.h"
 #include "mongo/util/fail_point.h"
+#include "mongo/util/scopeguard.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
@@ -90,7 +98,8 @@ std::unique_ptr<Notifier> getCappedInsertNotifier(OperationContext* opCtx,
     //
     // We can only wait on the capped collection insert notifier if the collection is present,
     // otherwise we should retry immediately when we hit EOF.
-    if (opCtx->recoveryUnit()->getTimestampReadSource() == RecoveryUnit::kMajorityCommitted) {
+    if (shard_role_details::getRecoveryUnit(opCtx)->getTimestampReadSource() ==
+        RecoveryUnit::kMajorityCommitted) {
         return std::make_unique<MajorityCommittedPointNotifier>();
     } else {
         auto collection = CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, nss);
@@ -109,25 +118,20 @@ void waitForInserts(OperationContext* opCtx,
     // of the notifier at the time of the previous EOF, we require two EOFs in a row with no
     // notifier version change in order to wait.  This is sufficient to ensure we never wait
     // when data is available.
-    auto curOp = CurOp::get(opCtx);
-    curOp->pauseTimer();
-    ON_BLOCK_EXIT([curOp] { curOp->resumeTimer(); });
-
     notifier->prepareForWait(opCtx);
-    auto yieldResult = yieldPolicy->yieldOrInterrupt(
-        opCtx,
-        [opCtx, &notifier] {
-            const auto deadline = awaitDataState(opCtx).waitForInsertsDeadline;
-            notifier->waitUntil(opCtx, deadline);
-            if (MONGO_unlikely(planExecutorHangWhileYieldedInWaitForInserts.shouldFail())) {
-                LOGV2(4452903,
-                      "PlanExecutor - planExecutorHangWhileYieldedInWaitForInserts fail point "
-                      "enabled. "
-                      "Blocking until fail point is disabled");
-                planExecutorHangWhileYieldedInWaitForInserts.pauseWhileSet();
-            }
-        },
-        RestoreContext::RestoreType::kYield);
+    auto yieldResult = yieldPolicy->yieldOrInterrupt(opCtx, [opCtx, &notifier] {
+        const auto deadline = awaitDataState(opCtx).waitForInsertsDeadline;
+        auto curOp = CurOp::get(opCtx);
+        curOp->pauseTimer();
+        ON_BLOCK_EXIT([curOp] { curOp->resumeTimer(); });
+        notifier->waitUntil(opCtx, deadline);
+        if (MONGO_unlikely(planExecutorHangWhileYieldedInWaitForInserts.shouldFail())) {
+            LOGV2(4452903,
+                  "PlanExecutor - planExecutorHangWhileYieldedInWaitForInserts fail point enabled. "
+                  "Blocking until fail point is disabled");
+            planExecutorHangWhileYieldedInWaitForInserts.pauseWhileSet();
+        }
+    });
     notifier->doneWaiting(opCtx);
 
     uassertStatusOK(yieldResult);
