@@ -27,13 +27,25 @@
  *    it in the license file.
  */
 
+
+#include <queue>
+#include <vector>
+
+
+#include "mongo/base/string_data.h"
+#include "mongo/bson/util/builder.h"
+#include "mongo/bson/util/builder_fwd.h"
+#include "mongo/db/exec/plan_stats.h"
+#include "mongo/db/query/plan_ranker.h"
+#include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/redaction.h"
+#include "mongo/util/assert_util_core.h"
+#include "mongo/util/str.h"
+
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/query/plan_ranker.h"
-
-#include "mongo/logv2/log.h"
 
 namespace mongo::plan_ranker {
 namespace log_detail {
@@ -45,18 +57,19 @@ void logScoreFormula(std::function<std::string()> formula,
                      double noSortBonus,
                      double noIxisectBonus,
                      double tieBreakers) {
-    LOGV2_DEBUG(20961, 2, "Score formula", "formula"_attr = [&]() {
-        StringBuilder sb;
-        sb << "score(" << str::convertDoubleToString(score) << ") = baseScore("
-           << str::convertDoubleToString(baseScore) << ")"
-           << " + productivity(" << formula() << " = " << str::convertDoubleToString(productivity)
-           << ")"
-           << " + tieBreakers(" << str::convertDoubleToString(noFetchBonus) << " noFetchBonus + "
-           << str::convertDoubleToString(noSortBonus) << " noSortBonus + "
-           << str::convertDoubleToString(noIxisectBonus)
-           << " noIxisectBonus = " << str::convertDoubleToString(tieBreakers) << ")";
-        return sb.str();
-    }());
+    LOGV2_DEBUG(
+        20961, 2, "Score formula", "formula"_attr = [&]() {
+            StringBuilder sb;
+            sb << "score(" << str::convertDoubleToString(score) << ") = baseScore("
+               << str::convertDoubleToString(baseScore) << ")"
+               << " + productivity(" << formula() << " = "
+               << str::convertDoubleToString(productivity) << ")"
+               << " + tieBreakers(" << str::convertDoubleToString(noFetchBonus)
+               << " noFetchBonus + " << str::convertDoubleToString(noSortBonus) << " noSortBonus + "
+               << str::convertDoubleToString(noIxisectBonus)
+               << " noIxisectBonus = " << str::convertDoubleToString(tieBreakers) << ")";
+            return sb.str();
+        }());
 }
 
 void logScoreBoost(double score) {
@@ -98,15 +111,16 @@ void logTieBreaking(double score,
                     double docsExaminedBonus,
                     double indexPrefixBonus,
                     bool isPlanTied) {
-    LOGV2_DEBUG(8027500, 2, "Tie breaking heuristics", "formula"_attr = [&]() {
-        StringBuilder sb;
-        sb << "isPlanTied: " << isPlanTied << ". finalScore("
-           << str::convertDoubleToString(score + docsExaminedBonus + indexPrefixBonus)
-           << ") = score(" << str::convertDoubleToString(score) << ") + docsExaminedBonus("
-           << str::convertDoubleToString(docsExaminedBonus) << ") + indexPrefixBonus("
-           << str::convertDoubleToString(indexPrefixBonus) << ")";
-        return sb.str();
-    }());
+    LOGV2_DEBUG(
+        8027500, 2, "Tie breaking heuristics", "formula"_attr = [&]() {
+            StringBuilder sb;
+            sb << "isPlanTied: " << isPlanTied << ". finalScore("
+               << str::convertDoubleToString(score + docsExaminedBonus + indexPrefixBonus)
+               << ") = score(" << str::convertDoubleToString(score) << ") + docsExaminedBonus("
+               << str::convertDoubleToString(docsExaminedBonus) << ") + indexPrefixBonus("
+               << str::convertDoubleToString(indexPrefixBonus) << ")";
+            return sb.str();
+        }());
 }
 }  // namespace log_detail
 
@@ -124,7 +138,7 @@ protected:
             static_cast<double>(stats->common.works);
     }
 
-    std::string getProductivityFormula(const PlanStageStats* stats) const {
+    std::string getProductivityFormula(const PlanStageStats* stats) const override {
         StringBuilder sb;
         sb << "(" << stats->common.advanced << " advanced)/(" << stats->common.works << " works)";
         return sb.str();
@@ -418,7 +432,7 @@ std::vector<size_t> applyIndexPrefixHeuristic(const std::vector<const QuerySolut
             stack.emplace_back();
             stack.back().reserve(solutions.size());
             for (auto node : top) {
-                stack.back().emplace_back(node->children[childIndex]);
+                stack.back().emplace_back(node->children[childIndex].get());
             }
         }
 
