@@ -30,18 +30,27 @@
 #pragma once
 
 #include <functional>
+#include <memory>
+#include <string>
+#include <variant>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/base/string_data.h"
+#include "mongo/db/namespace_string.h"
 #include "mongo/db/operation_context.h"
-#include "mongo/db/query/restore_context.h"
+#include "mongo/db/query/query_knobs_gen.h"
 #include "mongo/util/assert_util.h"
 #include "mongo/util/clock_source.h"
 #include "mongo/util/duration.h"
 #include "mongo/util/elapsed_tracker.h"
+#include "mongo/util/str.h"
 #include "mongo/util/uuid.h"
 
 namespace mongo {
 
 class ClockSource;
+
 class Yieldable;
 
 class YieldPolicyCallbacks {
@@ -93,15 +102,14 @@ public:
         // (e.g. collection drop) during yield.
         YIELD_MANUAL,
 
-        // Can be used in one of the following scenarios:
-        //  - The caller will hold a lock continuously for the lifetime of this PlanExecutor.
-        //  - This PlanExecutor doesn't logically belong to a Collection, and so does not need to be
-        //    locked during execution. For example, this yield policy is used for PlanExecutors
-        //    which unspool queued metadata ("virtual collection scans") for listCollections and
-        //    listIndexes.
-        NO_YIELD,
-
-        // Will not yield locks or storage engine resources, but will check for interrupt.
+        // Will not yield locks or storage engine resources, either because the caller intends to
+        // hold the lock continuously for the lifetime of this PlanExecutor, or because this
+        // PlanExecutor doesn't logically belong to a Collection, and so does not need to be
+        // locked during execution. For example, this yield policy is used for PlanExecutors
+        // which unspool queued metadata ("virtual collection scans") for listCollections and
+        // listIndexes.
+        //
+        // Will still check for interrupt.
         INTERRUPT_ONLY,
 
         // Used for testing, this yield policy will cause the PlanExecutor to time out on the first
@@ -123,8 +131,6 @@ public:
                 return "WRITE_CONFLICT_RETRY_ONLY";
             case YieldPolicy::YIELD_MANUAL:
                 return "YIELD_MANUAL";
-            case YieldPolicy::NO_YIELD:
-                return "NO_YIELD";
             case YieldPolicy::INTERRUPT_ONLY:
                 return "INTERRUPT_ONLY";
             case YieldPolicy::ALWAYS_TIME_OUT:
@@ -135,7 +141,7 @@ public:
         MONGO_UNREACHABLE;
     }
 
-    static YieldPolicy parseFromBSON(const StringData& element) {
+    static YieldPolicy parseFromBSON(StringData element) {
         const std::string& yieldPolicy = element.toString();
         if (yieldPolicy == "YIELD_AUTO") {
             return YieldPolicy::YIELD_AUTO;
@@ -146,9 +152,6 @@ public:
         if (yieldPolicy == "YIELD_MANUAL") {
             return YieldPolicy::YIELD_MANUAL;
         }
-        if (yieldPolicy == "NO_YIELD") {
-            return YieldPolicy::NO_YIELD;
-        }
         if (yieldPolicy == "INTERRUPT_ONLY") {
             return YieldPolicy::INTERRUPT_ONLY;
         }
@@ -170,8 +173,9 @@ public:
                                             const NamespaceString& newNss,
                                             UUID collUuid) {
         uasserted(ErrorCodes::QueryPlanKilled,
-                  str::stream() << "collection renamed from '" << oldNss << "' to '" << newNss
-                                << "'. UUID " << collUuid);
+                  str::stream() << "collection renamed from '" << oldNss.toStringForErrorMsg()
+                                << "' to '" << newNss.toStringForErrorMsg() << "'. UUID "
+                                << collUuid);
     }
 
     /**
@@ -180,6 +184,8 @@ public:
      */
     static YieldPolicy getPolicyOverrideForOperation(OperationContext* opCtx, YieldPolicy desired);
 
+    class YieldThroughAcquisitions {};
+
     /**
      * Constructs a PlanYieldPolicy of the given 'policy' type. This class uses an ElapsedTracker
      * to keep track of elapsed time, which is initialized from the parameters 'cs',
@@ -189,14 +195,14 @@ public:
      * addition to releasing/restoring locks and the storage engine snapshot). The provided 'policy'
      * will be overridden depending on the nature of this operation. For example, multi-document
      * transactions will always downgrade to INTERRUPT_ONLY, and operations with recursively held
-     * locks will downgrade to NO_YIELD.
+     * locks will downgrade to INTERRUPT_ONLY.
      */
     PlanYieldPolicy(OperationContext* opCtx,
                     YieldPolicy policy,
                     ClockSource* cs,
                     int yieldIterations,
                     Milliseconds yieldPeriod,
-                    const Yieldable* yieldable,
+                    std::variant<const Yieldable*, YieldThroughAcquisitions> yieldable,
                     std::unique_ptr<const YieldPolicyCallbacks> callbacks);
 
     virtual ~PlanYieldPolicy() = default;
@@ -206,7 +212,14 @@ public:
      * YIELD_AUTO and INTERRUPT_ONLY) or release locks or storage engine state (in the case of
      * auto-yielding plans).
      */
-    virtual bool shouldYieldOrInterrupt(OperationContext* opCtx);
+    inline bool shouldYieldOrInterrupt(OperationContext* opCtx) {
+        if (auto t = _fastClock->now().toMillisSinceEpoch();
+            MONGO_unlikely(_forceYield || t > _nextYieldCheckpoint)) {
+            _nextYieldCheckpoint = t + _yieldIntervalMs;
+            return doShouldYieldOrInterrupt(opCtx);
+        }
+        return false;
+    }
 
     /**
      * Resets the yield timer so that we wait for a while before yielding/interrupting again.
@@ -227,15 +240,14 @@ public:
      * been relinquished.
      */
     virtual Status yieldOrInterrupt(OperationContext* opCtx,
-                                    std::function<void()> whileYieldingFn,
-                                    RestoreContext::RestoreType restoreType);
+                                    std::function<void()> whileYieldingFn = nullptr);
 
     /**
      * All calls to shouldYieldOrInterrupt() will return true until the next call to
      * yieldOrInterrupt(). This must only be called for auto-yielding plans, to force a yield. It
      * cannot be used to force an interrupt for INTERRUPT_ONLY plans.
      */
-    void forceYield() {
+    MONGO_COMPILER_ALWAYS_INLINE void forceYield() {
         dassert(canAutoYield());
         _forceYield = true;
     }
@@ -252,7 +264,6 @@ public:
             case YieldPolicy::ALWAYS_MARK_KILLED: {
                 return true;
             }
-            case YieldPolicy::NO_YIELD:
             case YieldPolicy::WRITE_CONFLICT_RETRY_ONLY:
             case YieldPolicy::INTERRUPT_ONLY: {
                 return false;
@@ -266,7 +277,7 @@ public:
      * either releasing storage engine resources via abandonSnapshot() OR yielding LockManager
      * locks.
      */
-    bool canAutoYield() const {
+    MONGO_COMPILER_ALWAYS_INLINE bool canAutoYield() const {
         switch (_policy) {
             case YieldPolicy::YIELD_AUTO:
             case YieldPolicy::WRITE_CONFLICT_RETRY_ONLY:
@@ -274,7 +285,6 @@ public:
             case YieldPolicy::ALWAYS_MARK_KILLED: {
                 return true;
             }
-            case YieldPolicy::NO_YIELD:
             case YieldPolicy::YIELD_MANUAL:
             case YieldPolicy::INTERRUPT_ONLY:
                 return false;
@@ -287,9 +297,20 @@ public:
     }
 
     void setYieldable(const Yieldable* yieldable) {
+        invariant(!usesCollectionAcquisitions());
         _yieldable = yieldable;
     }
 
+    bool usesCollectionAcquisitions() const {
+        return holds_alternative<YieldThroughAcquisitions>(_yieldable);
+    }
+
+protected:
+    /**
+     * The function that actually do check for interrupt or release locks or storage engine state.
+     */
+    MONGO_COMPILER_NOINLINE virtual bool doShouldYieldOrInterrupt(OperationContext* opCtx);
+
 private:
     /**
      * Functions to be implemented by derived classes which save and restore query execution state.
@@ -297,9 +318,7 @@ private:
      * specific query execution engines.
      */
     virtual void saveState(OperationContext* opCtx) = 0;
-    virtual void restoreState(OperationContext* opCtx,
-                              const Yieldable* yieldable,
-                              RestoreContext::RestoreType restoreType) = 0;
+    virtual void restoreState(OperationContext* opCtx, const Yieldable* yieldable) = 0;
 
     /**
      * TODO SERVER-59620: Remove this.
@@ -319,13 +338,18 @@ private:
     void performYield(OperationContext* opCtx,
                       const Yieldable& yieldable,
                       std::function<void()> whileYieldingFn);
+    void performYieldWithAcquisitions(OperationContext* opCtx,
+                                      std::function<void()> whileYieldingFn);
 
     const YieldPolicy _policy;
-    const Yieldable* _yieldable;
+    std::variant<const Yieldable*, YieldThroughAcquisitions> _yieldable;
     std::unique_ptr<const YieldPolicyCallbacks> _callbacks;
 
-    bool _forceYield = false;
     ElapsedTracker _elapsedTracker;
+    int64_t _yieldIntervalMs;
+    ClockSource* _fastClock;
+    int64_t _nextYieldCheckpoint{0};
+    bool _forceYield = false;
 };
 
 }  // namespace mongo
