@@ -30,30 +30,63 @@
 
 #include "mongo/db/query/planner_analysis.h"
 
+// IWYU pragma: no_include "boost/container/detail/std_fwd.hpp"
+#include <boost/cstdint.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <cstdint>
+#include <cstring>
+#include <s2cellid.h>
+// IWYU pragma: no_include "ext/alloc_traits.h"
+#include <algorithm>
+#include <ostream>
 #include <set>
+#include <type_traits>
 #include <vector>
 
-#include "mongo/base/simple_string_data_comparator.h"
+#include "mongo/base/checked_cast.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
 #include "mongo/bson/simple_bsonelement_comparator.h"
+#include "mongo/db/basic_types.h"
 #include "mongo/db/bson/dotted_path_support.h"
+#include "mongo/db/exec/document_value/document_metadata_fields.h"
 #include "mongo/db/index/expression_params.h"
+#include "mongo/db/index/multikey_paths.h"
 #include "mongo/db/index/s2_common.h"
-#include "mongo/db/jsobj.h"
+#include "mongo/db/index_names.h"
+#include "mongo/db/matcher/expression.h"
 #include "mongo/db/matcher/expression_geo.h"
-#include "mongo/db/matcher/match_expression_dependencies.h"
-#include "mongo/db/query/planner_ixselect.h"
+#include "mongo/db/pipeline/dependencies.h"
+#include "mongo/db/pipeline/field_path.h"
+#include "mongo/db/query/find_command.h"
+#include "mongo/db/query/index_bounds.h"
+#include "mongo/db/query/index_hint.h"
+#include "mongo/db/query/interval.h"
+#include "mongo/db/query/interval_evaluation_tree.h"
+#include "mongo/db/query/optimizer/algebra/polyvalue.h"
 #include "mongo/db/query/projection.h"
-#include "mongo/db/query/query_decorations.h"
+#include "mongo/db/query/query_knob_configuration.h"
 #include "mongo/db/query/query_planner_common.h"
+#include "mongo/db/query/query_request_helper.h"
 #include "mongo/db/query/query_solution.h"
+#include "mongo/db/query/stage_types.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/redaction.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/str.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
 
 namespace mongo {
 
-using std::endl;
 using std::pair;
 using std::string;
 using std::unique_ptr;
@@ -531,7 +564,7 @@ std::unique_ptr<QuerySolutionNode> analyzeProjection(const CanonicalQuery& query
             // too many fields output, so we need to trim them down.
             return std::make_unique<ProjectionNodeSimple>(
                 addSortKeyGeneratorStageIfNeeded(query, hasSortStage, std::move(solnRoot)),
-                *query.root(),
+                query.getPrimaryMatchExpression(),
                 projection);
         }
         if (isInclusionOnly) {
@@ -541,7 +574,7 @@ std::unique_ptr<QuerySolutionNode> analyzeProjection(const CanonicalQuery& query
                 // projection can cover.
                 return std::make_unique<ProjectionNodeCovered>(
                     addSortKeyGeneratorStageIfNeeded(query, hasSortStage, std::move(solnRoot)),
-                    *query.root(),
+                    query.getPrimaryMatchExpression(),
                     projection,
                     std::move(coveredKeyObj));
             }
@@ -551,7 +584,7 @@ std::unique_ptr<QuerySolutionNode> analyzeProjection(const CanonicalQuery& query
     // No fast path available, we need to add this generic projection node.
     return std::make_unique<ProjectionNodeDefault>(
         addSortKeyGeneratorStageIfNeeded(query, hasSortStage, std::move(solnRoot)),
-        *query.root(),
+        query.getPrimaryMatchExpression(),
         projection);
 }
 
@@ -658,9 +691,7 @@ std::unique_ptr<QuerySolutionNode> tryPushdownProjectBeneathSort(
     return ownedProjectionInput;
 }
 
-bool canUseSimpleSort(const QuerySolutionNode& solnRoot,
-                      const CanonicalQuery& cq,
-                      const QueryPlannerParams& plannerParams) {
+bool canUseSimpleSort(const QuerySolutionNode& solnRoot, const CanonicalQuery& cq) {
     // The simple sort stage discards any metadata other than sort key metadata. It can only be used
     // if there are no metadata dependencies, or the only metadata dependency is a 'kSortKey'
     // dependency.
@@ -691,7 +722,7 @@ boost::optional<const projection_ast::Projection*> attemptToGetProjectionFromQue
  * Returns true if 'setL' is a non-strict subset of 'setR'.
  *
  * The types of the sets are permitted to be different to allow checking something with compatible
- * but different types e.g. std::set<std::string> and StringDataUnorderedMap.
+ * but different types e.g. std::set<std::string>, StringMap.
  */
 template <typename SetL, typename SetR>
 bool isSubset(const SetL& setL, const SetR& setR) {
@@ -766,9 +797,37 @@ void deprioritizeUnboundedCollectionScan(QuerySolutionNode* solnRoot,
 
     collScan->lowPriority = true;
 }
+
+bool isShardedCollScan(QuerySolutionNode* solnRoot) {
+    return solnRoot->getType() == StageType::STAGE_SHARDING_FILTER &&
+        solnRoot->children.size() == 1 &&
+        solnRoot->children[0]->getType() == StageType::STAGE_COLLSCAN;
+}
+
+bool shouldReverseScanForSort(QuerySolutionNode* solnRoot,
+                              const CanonicalQuery& query,
+                              const QueryPlannerParams& params) {
+    // We cannot reverse this scan if its direction is specified by a $natural hint.
+    const FindCommandRequest& findCommand = query.getFindCommandRequest();
+    const bool isCollscan =
+        solnRoot->getType() == StageType::STAGE_COLLSCAN || isShardedCollScan(solnRoot);
+    const bool hasNaturalHint =
+        findCommand.getHint()[query_request_helper::kNaturalSortField].ok() &&
+        !params.querySettingsApplied;
+    const bool hasQuerySettingsEnforcedDirection =
+        params.mainCollectionInfo.collscanDirection.has_value();
+    if (isCollscan && (hasNaturalHint || hasQuerySettingsEnforcedDirection)) {
+        return false;
+    }
+
+    // The only collection scan that includes a sort order in 'providedSorts' is a scan on a
+    // clustered collection.
+    const BSONObj reverseSort = QueryPlannerCommon::reverseSortObj(findCommand.getSort());
+    return solnRoot->providedSorts().contains(reverseSort);
+}
 }  // namespace
 
-bool QueryPlannerAnalysis::isEligibleForHashJoin(const SecondaryCollectionInfo& foreignCollInfo) {
+bool QueryPlannerAnalysis::isEligibleForHashJoin(const CollectionInfo& foreignCollInfo) {
     return !internalQueryDisableLookupExecutionUsingHashJoin.load() && foreignCollInfo.exists &&
         foreignCollInfo.stats.noOfRecords <=
         internalQueryCollectionMaxNoOfDocumentsToChooseHashJoin.load() &&
@@ -806,8 +865,10 @@ void QueryPlannerAnalysis::removeUselessColumnScanRowStoreExpression(QuerySoluti
                 }
             }
             // Look for projection above column scan.
-            else if (root.getType() == STAGE_PROJECTION_SIMPLE ||
-                     root.getType() == STAGE_PROJECTION_DEFAULT) {
+            else if ((root.getType() == STAGE_PROJECTION_SIMPLE ||
+                      root.getType() == STAGE_PROJECTION_DEFAULT) &&
+                     static_cast<ProjectionNode&>(root).proj.type() ==
+                         projection_ast::ProjectType::kInclusion) {
                 auto& parentProjection = static_cast<ProjectionNode&>(root);
                 // A row store expression that preserves all fields used by the parent projection is
                 // redundant and can be removed.
@@ -824,18 +885,49 @@ void QueryPlannerAnalysis::removeUselessColumnScanRowStoreExpression(QuerySoluti
 }
 
 // static
-std::pair<EqLookupNode::LookupStrategy, boost::optional<IndexEntry>>
-QueryPlannerAnalysis::determineLookupStrategy(
+void QueryPlannerAnalysis::removeImpreciseInternalExprFilters(const QueryPlannerParams& params,
+                                                              QuerySolutionNode& root) {
+    if (params.mainCollectionInfo.stats.isTimeseries) {
+        // For timeseries collections, the imprecise filters are able to get rid of an entire
+        // bucket's worth of data, and save us from unpacking potentially thousands of documents.
+        // In this case, we decide not to prune the imprecise filters.
+        return;
+    }
+
+    // In principle we could do this optimization for TEXT queries, but for simplicity we do not
+    // today.
+    const auto stageType = root.getType();
+    if (stageType == StageType::STAGE_TEXT_OR || stageType == STAGE_TEXT_MATCH) {
+        return;
+    }
+
+    // Remove the imprecise predicates on nodes after a fetch. For nodes before a fetch, the
+    // imprecise filters may be able to save us the significant work of doing a fetch. In such
+    // cases, we assume the imprecise filter is always worth applying.
+    if (root.fetched() && root.filter) {
+        root.filter =
+            expression::assumeImpreciseInternalExprNodesReturnTrue(std::move(root.filter));
+    }
+
+    for (auto& child : root.children) {
+        removeImpreciseInternalExprFilters(params, *child);
+    }
+}
+
+// static
+QueryPlannerAnalysis::Strategy QueryPlannerAnalysis::determineLookupStrategy(
     const NamespaceString& foreignCollName,
     const std::string& foreignField,
-    const std::map<NamespaceString, SecondaryCollectionInfo>& collectionsInfo,
+    const std::map<NamespaceString, CollectionInfo>& collectionsInfo,
     bool allowDiskUse,
     const CollatorInterface* collator) {
     auto foreignCollItr = collectionsInfo.find(foreignCollName);
-    tassert(5842600,
-            str::stream() << "Expected collection info, but found none; target collection: "
-                          << foreignCollName,
-            foreignCollItr != collectionsInfo.end());
+    if (foreignCollItr == collectionsInfo.end() || !foreignCollItr->second.exists) {
+        return {EqLookupNode::LookupStrategy::kNonExistentForeignCollection, boost::none};
+    }
+
+    const auto scanDirection =
+        foreignCollItr->second.collscanDirection.value_or(NaturalOrderHint::Direction::kForward);
 
     // Check if an eligible index exists for indexed loop join strategy.
     const auto foreignIndex = [&]() -> boost::optional<IndexEntry> {
@@ -866,15 +958,21 @@ QueryPlannerAnalysis::determineLookupStrategy(
         return boost::none;
     }();
 
-    if (!foreignCollItr->second.exists) {
-        return {EqLookupNode::LookupStrategy::kNonExistentForeignCollection, boost::none};
-    } else if (foreignIndex) {
-        return {EqLookupNode::LookupStrategy::kIndexedLoopJoin, std::move(foreignIndex)};
-    } else if (allowDiskUse && isEligibleForHashJoin(foreignCollItr->second)) {
-        return {EqLookupNode::LookupStrategy::kHashJoin, boost::none};
-    } else {
-        return {EqLookupNode::LookupStrategy::kNestedLoopJoin, boost::none};
+    if (foreignIndex) {
+        // $natural hinted scan direction is not relevant for IndexedLoopJoin, but is passed here
+        // for consistency.
+        return {
+            EqLookupNode::LookupStrategy::kIndexedLoopJoin, std::move(foreignIndex), scanDirection};
     }
+    const bool tableScanForbidden = foreignCollItr->second.options &
+        (QueryPlannerParams::NO_TABLE_SCAN | QueryPlannerParams::STRICT_NO_TABLE_SCAN);
+    uassert(ErrorCodes::NoQueryExecutionPlans,
+            "No foreign index and table scan disallowed",
+            !tableScanForbidden);
+    if (allowDiskUse && isEligibleForHashJoin(foreignCollItr->second)) {
+        return {EqLookupNode::LookupStrategy::kHashJoin, boost::none, scanDirection};
+    }
+    return {EqLookupNode::LookupStrategy::kNestedLoopJoin, boost::none, scanDirection};
 }
 
 // static
@@ -882,7 +980,7 @@ void QueryPlannerAnalysis::analyzeGeo(const QueryPlannerParams& params,
                                       QuerySolutionNode* solnRoot) {
     // Get field names of all 2dsphere indexes with version >= 3.
     std::set<StringData> twoDSphereFields;
-    for (const IndexEntry& indexEntry : params.indices) {
+    for (const IndexEntry& indexEntry : params.mainCollectionInfo.indexes) {
         if (indexEntry.type != IndexType::INDEX_2DSPHERE) {
             continue;
         }
@@ -924,7 +1022,6 @@ BSONObj QueryPlannerAnalysis::getSortPattern(const BSONObj& indexKeyPattern) {
 
 // static
 bool QueryPlannerAnalysis::explodeForSort(const CanonicalQuery& query,
-                                          const QueryPlannerParams& params,
                                           std::unique_ptr<QuerySolutionNode>* solnRoot) {
     vector<QuerySolutionNode*> explodableNodes;
 
@@ -1051,7 +1148,7 @@ bool QueryPlannerAnalysis::explodeForSort(const CanonicalQuery& query,
 
     // Too many ixscans spoil the performance.
     if (totalNumScans >
-        QueryKnobConfiguration::decoration(query.getOpCtx()).getMaxScansToExplodeForOp()) {
+        query.getExpCtx()->getQueryKnobConfiguration().getMaxScansToExplodeForOp()) {
         (*solnRoot)->hitScanLimit = true;
         LOGV2_DEBUG(
             20950,
@@ -1114,12 +1211,6 @@ bool sortMatchesTraversalPreference(const TraversalPreference& traversalPreferen
     return true;
 }
 
-bool isShardedCollScan(QuerySolutionNode* solnRoot) {
-    return solnRoot->getType() == StageType::STAGE_SHARDING_FILTER &&
-        solnRoot->children.size() == 1 &&
-        solnRoot->children[0]->getType() == StageType::STAGE_COLLSCAN;
-}
-
 // static
 std::unique_ptr<QuerySolutionNode> QueryPlannerAnalysis::analyzeSort(
     const CanonicalQuery& query,
@@ -1174,13 +1265,7 @@ std::unique_ptr<QuerySolutionNode> QueryPlannerAnalysis::analyzeSort(
 
     // Sort is not provided.  See if we provide the reverse of our sort pattern.
     // If so, we can reverse the scan direction(s).
-    BSONObj reverseSort = QueryPlannerCommon::reverseSortObj(sortObj);
-    // The only collection scan that includes a sort order in 'providedSorts' is a scan on a
-    // clustered collection. However, we cannot reverse this scan if its direction is specified by a
-    // $natural hint.
-    const bool naturalCollScan = solnRoot->getType() == StageType::STAGE_COLLSCAN &&
-        query.getFindCommandRequest().getHint()[query_request_helper::kNaturalSortField];
-    if (providedSorts.contains(reverseSort) && !naturalCollScan) {
+    if (shouldReverseScanForSort(solnRoot.get(), query, params)) {
         QueryPlannerCommon::reverseScans(solnRoot.get());
         LOGV2_DEBUG(20951,
                     5,
@@ -1192,7 +1277,7 @@ std::unique_ptr<QuerySolutionNode> QueryPlannerAnalysis::analyzeSort(
     // Sort not provided, can't reverse scans to get the sort.  One last trick: We can "explode"
     // index scans over point intervals to an OR of sub-scans in order to pull out a sort.
     // Let's try this.
-    if (explodeForSort(query, params, &solnRoot)) {
+    if (explodeForSort(query, &solnRoot)) {
         return solnRoot;
     }
 
@@ -1221,10 +1306,18 @@ std::unique_ptr<QuerySolutionNode> QueryPlannerAnalysis::analyzeSort(
     size_t sortLimit = findCommand.getLimit() ? static_cast<size_t>(*findCommand.getLimit()) +
             static_cast<size_t>(findCommand.getSkip().value_or(0))
                                               : 0;
-    if (canUseSimpleSort(*solnRoot, query, params)) {
-        sortNode = std::make_unique<SortNodeSimple>(std::move(solnRoot), sortObj, sortLimit);
+    if (canUseSimpleSort(*solnRoot, query)) {
+        sortNode = std::make_unique<SortNodeSimple>(
+            std::move(solnRoot),
+            sortObj,
+            sortLimit,
+            LimitSkipParameterization{query.shouldParameterizeLimitSkip()});
     } else {
-        sortNode = std::make_unique<SortNodeDefault>(std::move(solnRoot), sortObj, sortLimit);
+        sortNode = std::make_unique<SortNodeDefault>(
+            std::move(solnRoot),
+            sortObj,
+            sortLimit,
+            LimitSkipParameterization{query.shouldParameterizeLimitSkip()});
     }
     sortNode->addSortKeyMetadata = query.metadataDeps()[DocumentMetadataFields::kSortKey];
     solnRoot = std::move(sortNode);
@@ -1239,6 +1332,7 @@ std::unique_ptr<QuerySolution> QueryPlannerAnalysis::analyzeDataAccess(
     const QueryPlannerParams& params,
     std::unique_ptr<QuerySolutionNode> solnRoot) {
     auto soln = std::make_unique<QuerySolution>();
+
     soln->indexFilterApplied = params.indexFiltersApplied;
 
     solnRoot->computeProperties();
@@ -1254,7 +1348,7 @@ std::unique_ptr<QuerySolution> QueryPlannerAnalysis::analyzeDataAccess(
 
     // If we're answering a query on a sharded system, we need to drop documents that aren't
     // logically part of our shard.
-    if (params.options & QueryPlannerParams::INCLUDE_SHARD_FILTER) {
+    if (params.mainCollectionInfo.options & QueryPlannerParams::INCLUDE_SHARD_FILTER) {
         if (!solnRoot->fetched()) {
             // See if we need to fetch information for our shard key.
             // NOTE: Solution nodes only list ordinary, non-transformed index keys for now
@@ -1304,7 +1398,10 @@ std::unique_ptr<QuerySolution> QueryPlannerAnalysis::analyzeDataAccess(
     soln->hasBlockingStage = hasSortStage || hasAndHashStage;
 
     if (findCommand.getSkip()) {
-        auto skip = std::make_unique<SkipNode>(std::move(solnRoot), *findCommand.getSkip());
+        auto skip = std::make_unique<SkipNode>(
+            std::move(solnRoot),
+            *findCommand.getSkip(),
+            LimitSkipParameterization{query.shouldParameterizeLimitSkip()});
         solnRoot = std::move(skip);
     }
 
@@ -1335,13 +1432,17 @@ std::unique_ptr<QuerySolution> QueryPlannerAnalysis::analyzeDataAccess(
     // sort. Otherwise, we will have to enforce the limit ourselves since it's not handled inside
     // SORT.
     if (!hasSortStage && findCommand.getLimit()) {
-        auto limit = std::make_unique<LimitNode>(std::move(solnRoot), *findCommand.getLimit());
+        auto limit = std::make_unique<LimitNode>(
+            std::move(solnRoot),
+            *findCommand.getLimit(),
+            LimitSkipParameterization{query.shouldParameterizeLimitSkip()});
         solnRoot = std::move(limit);
     }
 
     solnRoot = tryPushdownProjectBeneathSort(std::move(solnRoot));
 
     QueryPlannerAnalysis::removeUselessColumnScanRowStoreExpression(*solnRoot);
+    QueryPlannerAnalysis::removeImpreciseInternalExprFilters(params, *solnRoot);
 
     soln->setRoot(std::move(solnRoot));
     return soln;
