@@ -30,25 +30,44 @@
 
 #include "mongo/db/query/planner_ixselect.h"
 
+#include "mongo/db/query/analyze_regex.h"
+#include <absl/container/node_hash_map.h>
+#include <absl/container/node_hash_set.h>
+#include <boost/container/flat_set.hpp>
+#include <boost/container/vector.hpp>
+#include <s2cellid.h>
+// IWYU pragma: no_include "ext/alloc_traits.h"
+#include <algorithm>
+#include <functional>
+#include <memory>
+#include <set>
+#include <type_traits>
+#include <utility>
 #include <vector>
 
-#include "mongo/base/simple_string_data_comparator.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data_comparator.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/bson/simple_bsonobj_comparator.h"
+#include "mongo/db/field_ref.h"
+#include "mongo/db/geo/geometry_container.h"
 #include "mongo/db/geo/hash.h"
+#include "mongo/db/geo/shapes.h"
+#include "mongo/db/index/multikey_paths.h"
 #include "mongo/db/index/s2_common.h"
-#include "mongo/db/index/wildcard_key_generator.h"
 #include "mongo/db/index_names.h"
 #include "mongo/db/matcher/expression_algo.h"
 #include "mongo/db/matcher/expression_geo.h"
 #include "mongo/db/matcher/expression_internal_bucket_geo_within.h"
-#include "mongo/db/matcher/expression_internal_expr_comparison.h"
-#include "mongo/db/matcher/expression_text.h"
 #include "mongo/db/query/canonical_query_encoder.h"
 #include "mongo/db/query/collation/collator_interface.h"
 #include "mongo/db/query/index_tag.h"
 #include "mongo/db/query/indexability.h"
 #include "mongo/db/query/planner_wildcard_helpers.h"
-#include "mongo/db/query/query_planner_common.h"
 #include "mongo/logv2/log.h"
+#include "mongo/stdx/unordered_set.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/string_map.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
@@ -69,10 +88,7 @@ bool isComparisonWithArrayPred(const MatchExpression* me) {
         return static_cast<const ComparisonMatchExpression*>(me)->getData().type() ==
             BSONType::Array;
     } else if (type == MatchExpression::MATCH_IN) {
-        const auto& equalities = static_cast<const InMatchExpression*>(me)->getEqualities();
-        return std::any_of(equalities.begin(), equalities.end(), [](BSONElement elt) {
-            return elt.type() == BSONType::Array;
-        });
+        return static_cast<const InMatchExpression*>(me)->hasArray();
     }
     return false;
 }
@@ -121,14 +137,12 @@ bool QueryPlannerIXSelect::notEqualsNullCanUseIndex(const IndexEntry& index,
     // with a value of null if the index is multikey on one of the components of the path.
     //
     // This is quite subtle, and due to the semantics of null matching. For example, if the query is
-    // {a: {$ne: null}}, you might expect us to build index bounds of [MinKey, undefined) and
-    // (null, MaxKey] (or similar) on an 'a' index. However, with this query the document {a: []}
-    // should match (because it does not match {a: null}), but will have an index key of undefined.
-    // Similarly, the document {a: [null, null]} matches the query {'a.b': {$ne: null}}, but would
-    // have an index key of null in an index on 'a.b'. Since it's possible for a key of undefined to
-    // be included in the results and also possible for a value of null to be included, there are no
-    // restrictions on the bounds of the index for such a predicate. Further, such an index could
-    // not be used for covering, so would not provide any help to the query.
+    // {'a.b': {$ne: null}}, you might expect us to build index bounds of [MinKey, null) and
+    // (null, MaxKey] (or similar) on an 'a.b' index. However, the document {a: [null, null]}
+    // matches the query {'a.b': {$ne: null}}, but would have an index key of null. Since it's
+    // possible for a value of null to be included in the results, there are no restrictions on the
+    // bounds of the index for such a predicate. Further, such an index could not be used for
+    // covering, so would not provide any help to the query.
     //
     // There are two exceptions to this rule, both having to do with $elemMatch, see below.
     auto* parentElemMatch = elemMatchContext.innermostParentElemMatch;
@@ -139,12 +153,11 @@ bool QueryPlannerIXSelect::notEqualsNullCanUseIndex(const IndexEntry& index,
 
     if (MatchExpression::ELEM_MATCH_VALUE == parentElemMatch->matchType()) {
         // If this $ne clause is within a $elemMatch *value*, the semantics of $elemMatch guarantee
-        // that no matching values will be null or undefined, even if the index is multikey.
+        // that no matching values will be null, even if the index is multikey.
         //
         // For example, the document {a: []} does *not* match the query {a: {$elemMatch: {$ne:
         // null}} because there was no element within the array that matched. While the document {a:
-        // [[]]} *does* match that query, the index entry for that document would be [], not null or
-        // undefined.
+        // [[]]} *does* match that query, the index entry for that document would be [], not null.
         return true;
     } else {
         invariant(MatchExpression::ELEM_MATCH_OBJECT == parentElemMatch->matchType());
@@ -179,16 +192,8 @@ bool QueryPlannerIXSelect::notEqualsNullCanUseIndex(const IndexEntry& index,
 }
 
 bool QueryPlannerIXSelect::canUseIndexForNin(const InMatchExpression* ime) {
-    const std::vector<BSONElement>& inList = ime->getEqualities();
-    auto containsNull = [](const BSONElement& elt) {
-        return elt.type() == jstNULL;
-    };
-    auto containsEmptyArray = [](const BSONElement& elt) {
-        return elt.type() == Array && elt.embeddedObject().isEmpty();
-    };
-    return !ime->hasRegex() && inList.size() == 2 &&
-        std::any_of(inList.begin(), inList.end(), containsNull) &&
-        std::any_of(inList.begin(), inList.end(), containsEmptyArray);
+    return !ime->hasRegex() && ime->getEqualities().size() == 2 && ime->hasNull() &&
+        ime->hasEmptyArray();
 }
 
 /**
@@ -370,12 +375,11 @@ bool QueryPlannerIXSelect::_compatible(const BSONElement& keyPatternElt,
                                        std::size_t keyPatternIdx,
                                        MatchExpression* node,
                                        StringData fullPathToNode,
-                                       const CollatorInterface* collator,
-                                       const ElemMatchContext& elemMatchContext) {
+                                       const QueryContext& queryContext) {
     if ((boundsGeneratingNodeContainsComparisonToType(node, BSONType::String) ||
          boundsGeneratingNodeContainsComparisonToType(node, BSONType::Array) ||
          boundsGeneratingNodeContainsComparisonToType(node, BSONType::Object)) &&
-        !CollatorInterface::collatorsMatch(collator, index.collator)) {
+        !CollatorInterface::collatorsMatch(queryContext.collator, index.collator)) {
         return false;
     }
 
@@ -412,8 +416,9 @@ bool QueryPlannerIXSelect::_compatible(const BSONElement& keyPatternElt,
         indexedFieldType = keyPatternElt.String();
     }
 
-    const bool isChildOfElemMatchValue = elemMatchContext.innermostParentElemMatch &&
-        elemMatchContext.innermostParentElemMatch->matchType() == MatchExpression::ELEM_MATCH_VALUE;
+    const bool isChildOfElemMatchValue = queryContext.elemMatchContext.innermostParentElemMatch &&
+        queryContext.elemMatchContext.innermostParentElemMatch->matchType() ==
+            MatchExpression::ELEM_MATCH_VALUE;
 
     // We know keyPatternElt.fieldname() == node->path().
     MatchExpression::MatchType exprtype = node->matchType();
@@ -484,8 +489,8 @@ bool QueryPlannerIXSelect::_compatible(const BSONElement& keyPatternElt,
             // Most of the time we can't use a multikey index for a $ne: null query, however there
             // are a few exceptions around $elemMatch.
             const bool isNotEqualsNull = isQueryNegatingEqualToNull(node);
-            const bool canUseIndexForNeNull =
-                notEqualsNullCanUseIndex(index, keyPatternElt, keyPatternIdx, elemMatchContext);
+            const bool canUseIndexForNeNull = notEqualsNullCanUseIndex(
+                index, keyPatternElt, keyPatternIdx, queryContext.elemMatchContext);
             if (isNotEqualsNull && !canUseIndexForNeNull) {
                 return false;
             }
@@ -520,9 +525,13 @@ bool QueryPlannerIXSelect::_compatible(const BSONElement& keyPatternElt,
 
         // If this is an $elemMatch value, make sure _all_ of the children can use the index.
         if (node->matchType() == MatchExpression::ELEM_MATCH_VALUE) {
-            ElemMatchContext newContext;
-            newContext.fullPathToParentElemMatch = fullPathToNode;
-            newContext.innermostParentElemMatch = static_cast<ElemMatchValueMatchExpression*>(node);
+            ElemMatchContext newEMContext;
+            newEMContext.fullPathToParentElemMatch = fullPathToNode;
+            newEMContext.innermostParentElemMatch =
+                static_cast<ElemMatchValueMatchExpression*>(node);
+            QueryContext newContext;
+            newContext.elemMatchContext = newEMContext;
+            newContext.collator = queryContext.collator;
 
             FieldRef path(fullPathToNode);
             // If the index path has at least two components, and the last component of the path is
@@ -537,13 +546,8 @@ bool QueryPlannerIXSelect::_compatible(const BSONElement& keyPatternElt,
             auto&& children = node->getChildVector();
             if (!std::all_of(children->begin(), children->end(), [&](auto&& child) {
                     const auto newPath = fullPathToNode.toString() + child->path();
-                    return _compatible(keyPatternElt,
-                                       index,
-                                       keyPatternIdx,
-                                       child.get(),
-                                       newPath,
-                                       collator,
-                                       newContext);
+                    return _compatible(
+                        keyPatternElt, index, keyPatternIdx, child.get(), newPath, newContext);
                 })) {
                 return false;
             }
@@ -553,6 +557,22 @@ bool QueryPlannerIXSelect::_compatible(const BSONElement& keyPatternElt,
             return false;
         }
 
+        if (MatchExpression::REGEX == exprtype) {
+            // Indexes are only useful if have no collator since otherwise it's keys are ICU encoded
+            // and neither PCRE nor PCRE2 support such encoding.
+            //
+            // However we may still want to use the index if:
+            // 1. The query **must** use an indexed plan. (e.g: there are other predicates that
+            // require an index such as $text or geo) OR
+            // 2. The index has no collator OR
+            // 3. internalQueryPlannerIgnoreIndexWithCollationForRegex is set to false. This knob
+            // helps avoiding possible regressions when the index would still be better than
+            // COLLSCAN. See HELP-60129 for details.
+            return queryContext.mustUseIndexedPlan ||
+                CollatorInterface::isSimpleCollator(index.collator) ||
+                !internalQueryPlannerIgnoreIndexWithCollationForRegex.load();
+        }
+
         // We can only index EQ using text indices.  This is an artificial limitation imposed by
         // FTSSpec::getIndexPrefix() which will fail if there is not an EQ predicate on each
         // index prefix field of the text index.
@@ -729,10 +749,7 @@ bool QueryPlannerIXSelect::nodeIsSupportedByWildcardIndex(const MatchExpression*
     } else if (queryExpr->matchType() == MatchExpression::MATCH_IN) {
         const auto* queryExprIn = static_cast<const InMatchExpression*>(queryExpr);
 
-        return std::all_of(
-            queryExprIn->getEqualities().begin(),
-            queryExprIn->getEqualities().end(),
-            [](const BSONElement& elt) { return canUseWildcardIndex(elt, MatchExpression::EQ); });
+        return !queryExprIn->hasNonEmptyArray() && !queryExprIn->hasNonEmptyObject();
     }
 
     return true;
@@ -760,20 +777,10 @@ bool QueryPlannerIXSelect::nodeIsSupportedByHashedIndex(const MatchExpression* q
 }
 
 // static
-// This is the public method which does not accept an ElemMatchContext.
 void QueryPlannerIXSelect::rateIndices(MatchExpression* node,
                                        string prefix,
                                        const vector<IndexEntry>& indices,
-                                       const CollatorInterface* collator) {
-    return _rateIndices(node, prefix, indices, collator, ElemMatchContext{});
-}
-
-// static
-void QueryPlannerIXSelect::_rateIndices(MatchExpression* node,
-                                        string prefix,
-                                        const vector<IndexEntry>& indices,
-                                        const CollatorInterface* collator,
-                                        const ElemMatchContext& elemMatchCtx) {
+                                       const QueryContext& queryContext) {
     // Do not traverse tree beyond logical NOR node
     MatchExpression::MatchType exprtype = node->matchType();
     if (exprtype == MatchExpression::NOR) {
@@ -799,13 +806,8 @@ void QueryPlannerIXSelect::_rateIndices(MatchExpression* node,
             std::size_t keyPatternIndex = 0;
             for (auto&& keyPatternElt : index.keyPattern) {
                 if (keyPatternElt.fieldNameStringData() == fullPath &&
-                    _compatible(keyPatternElt,
-                                index,
-                                keyPatternIndex,
-                                node,
-                                fullPath,
-                                collator,
-                                elemMatchCtx)) {
+                    _compatible(
+                        keyPatternElt, index, keyPatternIndex, node, fullPath, queryContext)) {
                     if (keyPatternIndex == 0) {
                         rt->first.push_back(i);
                     } else {
@@ -825,22 +827,27 @@ void QueryPlannerIXSelect::_rateIndices(MatchExpression* node,
     } else if (Indexability::arrayUsesIndexOnChildren(node) && !node->path().empty()) {
         // Note we skip empty path components since they are not allowed in index key patterns.
         const auto newPath = prefix + node->path().toString();
-        ElemMatchContext newContext;
+        ElemMatchContext newEMContext;
         // Note this StringData is unowned and references the string declared on the stack here.
         // This should be fine since we are only ever reading from this in recursive calls as
         // context to help make planning decisions.
-        newContext.fullPathToParentElemMatch = newPath;
-        newContext.innermostParentElemMatch = static_cast<ElemMatchObjectMatchExpression*>(node);
+        newEMContext.fullPathToParentElemMatch = newPath;
+        newEMContext.innermostParentElemMatch = static_cast<ElemMatchObjectMatchExpression*>(node);
+        QueryContext newContext;
+        newContext.elemMatchContext = newEMContext;
+        newContext.collator = queryContext.collator;
 
         // If the array uses an index on its children, it's something like
         // {foo: {$elemMatch: {bar: 1}}}, in which case the predicate is really over foo.bar.
         prefix += node->path().toString() + ".";
         for (size_t i = 0; i < node->numChildren(); ++i) {
-            _rateIndices(node->getChild(i), prefix, indices, collator, newContext);
+            rateIndices(node->getChild(i), prefix, indices, newContext);
         }
-    } else if (node->getCategory() == MatchExpression::MatchCategory::kLogical) {
+    } else if (node->getCategory() ==
+               MatchExpression::MatchCategory::kLogical) {  // Por aqui deberia entrar en IN, no por
+                                                            // boundsGenerating arriba
         for (size_t i = 0; i < node->numChildren(); ++i) {
-            _rateIndices(node->getChild(i), prefix, indices, collator, elemMatchCtx);
+            rateIndices(node->getChild(i), prefix, indices, queryContext);
         }
     }
 }
@@ -1076,7 +1083,7 @@ void QueryPlannerIXSelect::stripInvalidAssignmentsToWildcardIndexes(
  */
 static void stripInvalidAssignmentsToTextIndex(MatchExpression* node,
                                                size_t idx,
-                                               const StringDataUnorderedSet& prefixPaths) {
+                                               const StringDataSet& prefixPaths) {
     // If we're here, there are prefixPaths and node is either:
     // 1. a text pred which we can't use as we have nothing over its prefix, or
     // 2. a non-text pred which we can't use as we don't have a text pred AND-related.
@@ -1111,7 +1118,7 @@ static void stripInvalidAssignmentsToTextIndex(MatchExpression* node,
     // The AND must have an EQ predicate for each prefix path.  When we encounter a child with a
     // tag we remove it from childrenPrefixPaths.  All children exist if this set is empty at
     // the end.
-    StringDataUnorderedSet childrenPrefixPaths = prefixPaths;
+    auto childrenPrefixPaths = prefixPaths;
 
     for (size_t i = 0; i < node->numChildren(); ++i) {
         MatchExpression* child = node->getChild(i);
@@ -1169,8 +1176,7 @@ void QueryPlannerIXSelect::stripInvalidAssignmentsToTextIndexes(MatchExpression*
         // Gather the set of paths that comprise the index prefix for this text index.
         // Each of those paths must have an equality assignment, otherwise we can't assign
         // *anything* to this index.
-        auto textIndexPrefixPaths =
-            SimpleStringDataComparator::kInstance.makeStringDataUnorderedSet();
+        StringDataSet textIndexPrefixPaths;
         BSONObjIterator it(index.keyPattern);
 
         // We stop when we see the first string in the key pattern.  We know that
