@@ -27,20 +27,37 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <list>
+#include <memory>
+#include <string>
+#include <utility>
+#include <vector>
 
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/json.h"
+#include "mongo/db/exec/index_path_projection.h"
 #include "mongo/db/index/column_key_generator.h"
-#include "mongo/db/pipeline/document_source.h"
-#include "mongo/db/pipeline/inner_pipeline_stage_impl.h"
-#include "mongo/db/pipeline/inner_pipeline_stage_interface.h"
+#include "mongo/db/index/index_descriptor.h"
+#include "mongo/db/index_names.h"
+#include "mongo/db/matcher/expression.h"
+#include "mongo/db/namespace_string.h"
 #include "mongo/db/pipeline/pipeline.h"
-#include "mongo/db/query/collation/collator_interface_mock.h"
+#include "mongo/db/query/canonical_query.h"
+#include "mongo/db/query/collation/collator_interface.h"
+#include "mongo/db/query/index_entry.h"
 #include "mongo/db/query/query_knobs_gen.h"
 #include "mongo/db/query/query_planner.h"
+#include "mongo/db/query/query_planner_params.h"
 #include "mongo/db/query/query_planner_test_fixture.h"
 #include "mongo/db/query/query_planner_test_lib.h"
+#include "mongo/db/query/query_solution.h"
 #include "mongo/idl/server_parameter_test_util.h"
-#include "mongo/unittest/death_test.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
 
 namespace mongo {
 const std::string kIndexName = "indexName";
@@ -61,11 +78,11 @@ protected:
 
         // We're interested in testing plans that use a columnar index, so don't generate collection
         // scans.
-        params.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
+        params.mainCollectionInfo.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
 
         // Initialize some made up collection stats but disable the column scan knobs by default.
-        params.collectionStats.noOfRecords = 12345;
-        params.collectionStats.approximateDataSizeBytes = 100000;
+        params.mainCollectionInfo.stats.noOfRecords = 12345;
+        params.mainCollectionInfo.stats.approximateDataSizeBytes = 100000;
         internalQueryColumnScanMinCollectionSizeBytes.store(0);
         internalQueryColumnScanMinAvgDocSizeBytes.store(0);
         internalQueryColumnScanMinNumColumnFilters.store(0);
@@ -94,25 +111,26 @@ protected:
                                                      BSONObj keyPattern = kKeyPattern,
                                                      MatchExpression* partialFilterExpr = nullptr,
                                                      CollatorInterface* collator = nullptr) {
-        params.columnStoreIndexes.emplace_back(keyPattern,
-                                               IndexType::INDEX_COLUMN,
-                                               IndexDescriptor::kLatestIndexVersion,
-                                               false /* sparse */,
-                                               false /* unique */,
-                                               IndexEntry::Identifier{indexName.toString()},
-                                               partialFilterExpr,
-                                               collator,
-                                               proj ? proj : &_defaultPathProj);
+        params.mainCollectionInfo.columnIndexes.emplace_back(
+            keyPattern,
+            IndexType::INDEX_COLUMN,
+            IndexDescriptor::kLatestIndexVersion,
+            false /* sparse */,
+            false /* unique */,
+            IndexEntry::Identifier{indexName.toString()},
+            partialFilterExpr,
+            collator,
+            proj ? proj : &_defaultPathProj);
         if (genPerColFilter) {
-            params.options |= QueryPlannerParams::GENERATE_PER_COLUMN_FILTERS;
+            params.mainCollectionInfo.options |= QueryPlannerParams::GENERATE_PER_COLUMN_FILTERS;
         }
     }
 
-    std::vector<std::unique_ptr<InnerPipelineStageInterface>> makeInnerPipelineStages(
+    std::vector<boost::intrusive_ptr<DocumentSource>> makeInnerPipelineStages(
         const Pipeline& pipeline) {
-        std::vector<std::unique_ptr<InnerPipelineStageInterface>> stages;
+        std::vector<boost::intrusive_ptr<DocumentSource>> stages;
         for (auto&& source : pipeline.getSources()) {
-            stages.emplace_back(std::make_unique<InnerPipelineStageImpl>(source));
+            stages.emplace_back(source);
         }
         return stages;
     }
@@ -124,12 +142,12 @@ protected:
     }
 
     double collectionSizeBytes() {
-        return params.collectionStats.approximateDataSizeBytes;
+        return params.mainCollectionInfo.stats.approximateDataSizeBytes;
     }
 
     double avgDocumentSizeBytes() {
-        return static_cast<double>(params.collectionStats.approximateDataSizeBytes) /
-            params.collectionStats.noOfRecords;
+        return static_cast<double>(params.mainCollectionInfo.stats.approximateDataSizeBytes) /
+            params.mainCollectionInfo.stats.noOfRecords;
     }
 
 private:
@@ -700,7 +718,7 @@ TEST_F(QueryPlannerColumnarTest, GroupTest) {
         }
     })");
 
-    ASSERT(!cq->pipeline().empty());
+    ASSERT(!cq->cqPipeline().empty());
     auto solution =
         QueryPlanner::extendWithAggPipeline(*cq, std::move(solns[0]), {} /* secondaryCollInfos */);
     ASSERT_OK(QueryPlannerTestLib::solutionMatches(R"({
@@ -739,7 +757,7 @@ TEST_F(QueryPlannerColumnarTest, MatchGroupTest) {
         }
     })");
 
-    ASSERT(!cq->pipeline().empty());
+    ASSERT(!cq->cqPipeline().empty());
     auto solution =
         QueryPlanner::extendWithAggPipeline(*cq, std::move(solns[0]), {} /* secondaryCollInfos */);
     ASSERT_OK(QueryPlannerTestLib::solutionMatches(R"({
@@ -779,7 +797,7 @@ TEST_F(QueryPlannerColumnarTest, MatchGroupWithOverlappingFieldsTest) {
         }
     })");
 
-    ASSERT(!cq->pipeline().empty());
+    ASSERT(!cq->cqPipeline().empty());
     auto solution =
         QueryPlanner::extendWithAggPipeline(*cq, std::move(solns[0]), {} /* secondaryCollInfos */);
     ASSERT_OK(QueryPlannerTestLib::solutionMatches(R"({
@@ -852,7 +870,7 @@ TEST_F(QueryPlannerColumnarTest, DottedFieldsWithGroupStageDoesNotRequireProject
         }
     })");
 
-    ASSERT(!cq->pipeline().empty());
+    ASSERT(!cq->cqPipeline().empty());
     auto solution =
         QueryPlanner::extendWithAggPipeline(*cq, std::move(solns[0]), {} /* secondaryCollInfos */);
     ASSERT_OK(QueryPlannerTestLib::solutionMatches(R"({
@@ -872,9 +890,49 @@ TEST_F(QueryPlannerColumnarTest, DottedFieldsWithGroupStageDoesNotRequireProject
         << solution->root()->toString();
 }
 
+TEST_F(QueryPlannerColumnarTest, ExtraFieldsNotPermittedWhenApplyingExclusionProjectionAfter) {
+    addColumnStoreIndexAndEnableFilterSplitting();
+
+    auto pipeline = Pipeline::parse(
+        {fromjson(
+            "{$_internalProjection: {spec: {newfield: {$const: 999}}, policies: 'addFields'}}")},
+        expCtx);
+
+    runQueryWithPipeline(BSONObj(), BSON("a" << 1), makeInnerPipelineStages(*pipeline));
+
+    assertNumSolutions(1U);
+    assertSolutionExists(R"({
+        column_scan: {
+            filtersByPath: {},
+            outputFields: ['_id', 'a'],
+            matchFields: []
+        }
+    })");
+
+    ASSERT(!cq->cqPipeline().empty());
+    auto solution =
+        QueryPlanner::extendWithAggPipeline(*cq, std::move(solns[0]), {} /* secondaryCollInfos */);
+    ASSERT_OK(QueryPlannerTestLib::solutionMatches(R"({
+        proj: {
+            spec: {newfield: { $const: 999 }},
+            isAddition: true,
+            node: {
+                 column_scan: {
+                     filtersByPath: {},
+                     outputFields: ['_id', 'a'],
+                     matchFields: [],
+                     extraFieldsPermitted: false
+                 }
+            }
+        }
+    })",
+                                                   solution->root()))
+        << solution->root()->toString();
+}
+
 TEST_F(QueryPlannerColumnarTest, ShardKeyFieldsIncluded) {
     addColumnStoreIndexAndEnableFilterSplitting();
-    params.options |= QueryPlannerParams::INCLUDE_SHARD_FILTER;
+    params.mainCollectionInfo.options |= QueryPlannerParams::INCLUDE_SHARD_FILTER;
     params.shardKey = BSON("sk1" << 1 << "sk2.nested" << 1);
 
     runQuerySortProj(BSON("name"
@@ -903,7 +961,7 @@ TEST_F(QueryPlannerColumnarTest, ShardKeyFieldsIncluded) {
 
 TEST_F(QueryPlannerColumnarTest, ShardKeyFieldsCountTowardsFieldLimit) {
     addColumnStoreIndexAndEnableFilterSplitting();
-    params.options |= QueryPlannerParams::INCLUDE_SHARD_FILTER;
+    params.mainCollectionInfo.options |= QueryPlannerParams::INCLUDE_SHARD_FILTER;
     params.shardKey = BSON("sk1" << 1 << "sk2.nested" << 1);
 
     // Lower the upper bound on number of fields for COLUMN_SCAN eligibility. This should cause us
@@ -1078,7 +1136,7 @@ TEST_F(QueryPlannerColumnarTest, UseColumnStoreWithAncestorField) {
         }
     })");
 
-    ASSERT(!cq->pipeline().empty());
+    ASSERT(!cq->cqPipeline().empty());
     auto solution =
         QueryPlanner::extendWithAggPipeline(*cq, std::move(solns[0]), {} /* secondaryCollInfos
         */);
@@ -1201,7 +1259,7 @@ TEST_F(QueryPlannerColumnarTest, ColumnIndexForCountIncludesShardFilter) {
     setIsCountLike();
     addColumnStoreIndexAndEnableFilterSplitting();
 
-    params.options |= QueryPlannerParams::INCLUDE_SHARD_FILTER;
+    params.mainCollectionInfo.options |= QueryPlannerParams::INCLUDE_SHARD_FILTER;
     params.shardKey = BSON("sk1" << 1 << "sk2.nested" << 1);
 
     runQuerySortProj(BSONObj(), BSONObj(), BSONObj());
@@ -1260,8 +1318,8 @@ TEST_F(QueryPlannerColumnarTest, PlanningHeuristics_NotMet) {
     params.availableMemoryBytes = 10 * 1024;
 
     // Update the collection's stats to just below the expected defaults.
-    params.collectionStats.approximateDataSizeBytes = params.availableMemoryBytes - 1;
-    params.collectionStats.noOfRecords =
+    params.mainCollectionInfo.stats.approximateDataSizeBytes = params.availableMemoryBytes - 1;
+    params.mainCollectionInfo.stats.noOfRecords =
         collectionSizeBytes() / internalQueryColumnScanMinAvgDocSizeBytes.load() + 1;
 
     runQuerySortProj(BSON("a" << 1), BSONObj(), BSON("a" << 1));
@@ -1275,8 +1333,8 @@ TEST_F(QueryPlannerColumnarTest, PlanningHeuristics_AvgDocSizeLargeEnough) {
     resetPlannerHeuristics();
     params.availableMemoryBytes = 10 * 1024;
 
-    params.collectionStats.approximateDataSizeBytes = params.availableMemoryBytes - 1;
-    params.collectionStats.noOfRecords =
+    params.mainCollectionInfo.stats.approximateDataSizeBytes = params.availableMemoryBytes - 1;
+    params.mainCollectionInfo.stats.noOfRecords =
         collectionSizeBytes() / internalQueryColumnScanMinAvgDocSizeBytes.load();
 
     runQuerySortProj(BSON("a" << 1), BSONObj(), BSON("a" << 1));
@@ -1296,8 +1354,8 @@ TEST_F(QueryPlannerColumnarTest, PlanningHeuristics_CollectionLargeEnough) {
     resetPlannerHeuristics();
     params.availableMemoryBytes = 10 * 1024;
 
-    params.collectionStats.approximateDataSizeBytes = params.availableMemoryBytes;
-    params.collectionStats.noOfRecords =
+    params.mainCollectionInfo.stats.approximateDataSizeBytes = params.availableMemoryBytes;
+    params.mainCollectionInfo.stats.noOfRecords =
         collectionSizeBytes() / internalQueryColumnScanMinAvgDocSizeBytes.load() + 1;
 
     runQuerySortProj(BSON("a" << 1), BSONObj(), BSON("a" << 1));
@@ -1317,8 +1375,8 @@ TEST_F(QueryPlannerColumnarTest, PlanningHeuristics_EnoughColumnFilters) {
     resetPlannerHeuristics();
     params.availableMemoryBytes = 10 * 1024;
 
-    params.collectionStats.approximateDataSizeBytes = params.availableMemoryBytes - 1;
-    params.collectionStats.noOfRecords =
+    params.mainCollectionInfo.stats.approximateDataSizeBytes = params.availableMemoryBytes - 1;
+    params.mainCollectionInfo.stats.noOfRecords =
         collectionSizeBytes() / internalQueryColumnScanMinAvgDocSizeBytes.load() + 1;
 
     runQuerySortProj(
@@ -1341,8 +1399,8 @@ TEST_F(QueryPlannerColumnarTest, PlanningHeuristics_EmptyCollection) {
     internalQueryColumnScanMinAvgDocSizeBytes.store(1);
     internalQueryColumnScanMinNumColumnFilters.store(1);
     // Update the collection's stats to be zero/empty.
-    params.collectionStats.noOfRecords = 0;
-    params.collectionStats.approximateDataSizeBytes = 0;
+    params.mainCollectionInfo.stats.noOfRecords = 0;
+    params.mainCollectionInfo.stats.approximateDataSizeBytes = 0;
 
     runQuerySortProj(BSONObj(), BSONObj(), BSON("a" << 1));
 
