@@ -27,20 +27,32 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <cstddef>
+#include <memory>
+#include <vector>
 
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/json.h"
+#include "mongo/db/index/multikey_paths.h"
 #include "mongo/db/matcher/expression_always_boolean.h"
 #include "mongo/db/query/collation/collator_interface_mock.h"
-#include "mongo/db/query/query_planner.h"
+#include "mongo/db/query/query_knobs_gen.h"
+#include "mongo/db/query/query_planner_params.h"
 #include "mongo/db/query/query_planner_test_fixture.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/db/query/query_solution.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
 
 namespace mongo {
 namespace {
 
 
 TEST_F(QueryPlannerTest, PlannerUsesCoveredIxscanForCountWhenIndexSatisfiesQuery) {
-    params.options = QueryPlannerParams::DEFAULT;
+    params.mainCollectionInfo.options = QueryPlannerParams::DEFAULT;
     setIsCountLike();
     addIndex(BSON("x" << 1));
     runQuery(BSON("x" << 5));
@@ -49,7 +61,7 @@ TEST_F(QueryPlannerTest, PlannerUsesCoveredIxscanForCountWhenIndexSatisfiesQuery
 }
 
 TEST_F(QueryPlannerTest, PlannerAddsFetchToIxscanForCountWhenFetchFilterNonempty) {
-    params.options = QueryPlannerParams::DEFAULT;
+    params.mainCollectionInfo.options = QueryPlannerParams::DEFAULT;
     setIsCountLike();
     addIndex(BSON("x" << 1));
     runQuery(BSON("y" << 3 << "x" << 5));
@@ -60,99 +72,131 @@ TEST_F(QueryPlannerTest, PlannerAddsFetchToIxscanForCountWhenFetchFilterNonempty
 }
 
 TEST_F(QueryPlannerTest, PlannerUsesCoveredIxscanForCountWhenIndexSatisfiesNullQuery) {
-    params.options = QueryPlannerParams::DEFAULT;
+    params.mainCollectionInfo.options = QueryPlannerParams::DEFAULT;
     setIsCountLike();
     addIndex(BSON("x" << 1));
     runQuery(fromjson("{x: null}"));
     ASSERT_EQUALS(getNumSolutions(), 1U);
     assertSolutionExists(
         "{ixscan: {pattern: {x: 1}, bounds: "
-        "{x: [[undefined,undefined,true,true],[null,null,true,true]]}}}");
+        "{x: [[null,null,true,true]]}}}");
 }
 
 TEST_F(QueryPlannerTest, PlannerUsesCoveredIxscanWhenIndexSatisfiesNullAndOtherQuery) {
-    params.options = QueryPlannerParams::DEFAULT;
+    params.mainCollectionInfo.options = QueryPlannerParams::DEFAULT;
     setIsCountLike();
     addIndex(BSON("x" << 1));
     runQuery(fromjson("{x: {$in: [null, 2]}}"));
     ASSERT_EQUALS(getNumSolutions(), 1U);
     assertSolutionExists(
         "{ixscan: {pattern: {x: 1}, bounds: "
-        "{x: [[undefined,undefined,true,true],[null,null,true,true], [2, 2, true, true]]}}}");
+        "{x: [[null,null,true,true], [2, 2, true, true]]}}}");
 }
 
-TEST_F(QueryPlannerTest, PlannerAddsFetchForCountWhenMultikeyIndexSatisfiesNullQuery) {
-    params.options = QueryPlannerParams::DEFAULT;
+TEST_F(QueryPlannerTest, PlannerUsesCoveredIxscanCountWhenMultikeyIndexSatisfiesNullQuery) {
+    params.mainCollectionInfo.options = QueryPlannerParams::DEFAULT;
     setIsCountLike();
     MultikeyPaths multikeyPaths{{0U}};
     addIndex(fromjson("{x: 1}"), multikeyPaths);
     runQuery(fromjson("{x: null}"));
     ASSERT_EQUALS(getNumSolutions(), 1U);
-    assertSolutionExists(
-        "{fetch: {filter: {x: null}, node: {ixscan: "
-        "{pattern: {x: 1}, bounds: "
-        "{x: [[undefined,undefined,true,true],[null,null,true,true]]}}}}}");
+    assertSolutionExists("{ixscan: {pattern: {x: 1}, bounds: {x: [[null,null,true,true]]}}}");
 }
 
-TEST_F(QueryPlannerTest, PlannerUsesCoveredIxscanForCountWhenMultikeyIndexSatisfiesNullEmptyQuery) {
-    params.options = QueryPlannerParams::DEFAULT;
+TEST_F(QueryPlannerTest, PlannerAddsFetchWhenMultikeyIndexSatisfiesDottedNullQuery) {
+    params.mainCollectionInfo.options = QueryPlannerParams::DEFAULT;
+    setIsCountLike();
+    MultikeyPaths multikeyPaths{{0U}};
+    addIndex(fromjson("{'x.y': 1}"), multikeyPaths);
+    runQuery(fromjson("{'x.y': null}"));
+    ASSERT_EQUALS(getNumSolutions(), 1U);
+    assertSolutionExists(
+        "{fetch: {filter: {'x.y': null}, node: {ixscan: {pattern: {'x.y': 1}, bounds: {'x.y': "
+        "[[null,null,true,true]]}}}}}");
+}
+
+TEST_F(QueryPlannerTest, PlannerAddsFetchForCountWhenMultikeyIndexSatisfiesNullEmptyQuery) {
+    params.mainCollectionInfo.options = QueryPlannerParams::DEFAULT;
     setIsCountLike();
     MultikeyPaths multikeyPaths{{0U}};
     addIndex(fromjson("{x: 1}"), multikeyPaths);
     runQuery(fromjson("{x: {$in: [null, []]}}"));
     ASSERT_EQUALS(getNumSolutions(), 1U);
     assertSolutionExists(
-        "{ixscan: {pattern: {x: 1}, bounds: "
-        "{x: [[undefined,undefined,true,true],[null,null,true,true], [[], [], true, true]]}}}");
+        "{fetch: {filter: {x: {$in: [null, []]}}, node: {ixscan: {pattern: {x: 1}, bounds: "
+        "{x: [[undefined,undefined,true,true], [null,null,true,true], [[], [], true, true]]}}}}}");
 }
 
-TEST_F(QueryPlannerTest, PlannerUsesCoveredIxscanWhenMultikeyIndexSatisfiesNullEmptyAndOtherQuery) {
-    params.options = QueryPlannerParams::DEFAULT;
+TEST_F(QueryPlannerTest, PlannerAddsFetchWhenMultikeyIndexSatisfiesNullEmptyAndOtherQuery) {
+    params.mainCollectionInfo.options = QueryPlannerParams::DEFAULT;
     setIsCountLike();
     MultikeyPaths multikeyPaths{{0U}};
     addIndex(fromjson("{x: 1}"), multikeyPaths);
     runQuery(fromjson("{x: {$in: [null, [], 2]}}"));
     ASSERT_EQUALS(getNumSolutions(), 1U);
+    assertSolutionExists(
+        "{fetch: {filter: {x: {$in: [null, [], 2]}}, node: {ixscan: {pattern: {x: 1}, bounds: "
+        "{x: [[undefined,undefined,true,true], [null,null,true,true], [2, 2, true, true], [[], [], "
+        "true, true]]}}}}}");
+}
+
+TEST_F(QueryPlannerTest, PlannerUsesCoveredIxscanWhenMultikeyIndexSatisfiesNullAndOtherQuery) {
+    params.mainCollectionInfo.options = QueryPlannerParams::DEFAULT;
+    setIsCountLike();
+    MultikeyPaths multikeyPaths{{0U}};
+    addIndex(fromjson("{x: 1}"), multikeyPaths);
+    runQuery(fromjson("{x: {$in: [null, 2]}}"));
+    ASSERT_EQUALS(getNumSolutions(), 1U);
     assertSolutionExists(
         "{ixscan: {pattern: {x: 1}, bounds: "
-        "{x: [[undefined,undefined,true,true],[null,null,true,true], [2, 2, true, true], [[], [], "
-        "true, true]]}}}");
+        "{x: [[null,null,true,true], [2, 2, true, true]]}}}");
 }
 
 TEST_F(QueryPlannerTest, PlannerUsesCoveredIxscanFindWhenIndexSatisfiesNullQuery) {
-    params.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
+    params.mainCollectionInfo.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
     addIndex(fromjson("{x: 1, _id: 1}"));
     runQuerySortProj(fromjson("{x: null}"), BSONObj(), fromjson("{_id: 1}"));
     ASSERT_EQUALS(getNumSolutions(), 1U);
     assertSolutionExists(
         "{proj: {spec: {_id: 1}, node:"
         "{ixscan: {pattern: {x: 1, _id: 1}, bounds: "
-        "{x: [[undefined,undefined,true,true],[null,null,true,true]]}}}}}");
+        "{x: [[null,null,true,true]]}}}}}");
 }
 
 TEST_F(QueryPlannerTest, PlannerAddsFetchForFindWhenMultikeyIndexSatisfiesNullQuery) {
-    params.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
+    params.mainCollectionInfo.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
     MultikeyPaths multikeyPaths{{0U}, MultikeyComponents{}};
     addIndex(fromjson("{x: 1, _id: 1}"), multikeyPaths);
     runQuerySortProj(fromjson("{x: null}"), BSONObj(), fromjson("{_id: 1}"));
     ASSERT_EQUALS(getNumSolutions(), 1U);
     assertSolutionExists(
-        "{proj: {spec: {_id: 1}, node:"
-        "{fetch: {filter: {x: null}, node: {ixscan: "
-        "{pattern: {x: 1, _id: 1}, bounds: "
-        "{x: [[undefined,undefined,true,true],[null,null,true,true]]}}}}}}}");
+        "{proj: {spec: {_id: 1}, node: {ixscan: {pattern: {x: 1, _id: 1}, bounds: "
+        "{x: [[null,null,true,true]]}}}}}");
 }
 
-TEST_F(QueryPlannerTest, PlannerUsesCoveredIxscanFindWhenMultikeyIndexSatisfiesNullEmptyQuery) {
-    params.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
+TEST_F(QueryPlannerTest, PlannerAddsFetchFindWhenMultikeyIndexSatisfiesNullEmptyQuery) {
+    params.mainCollectionInfo.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
     MultikeyPaths multikeyPaths{{0U}, MultikeyComponents{}};
     addIndex(fromjson("{x: 1, _id: 1}"), multikeyPaths);
     runQuerySortProj(fromjson("{x: {$in: [null, []]}}"), BSONObj(), fromjson("{_id: 1}"));
     ASSERT_EQUALS(getNumSolutions(), 1U);
+    assertSolutionExists(
+        "{proj: {spec: {_id: 1}, node:"
+        "{fetch: {filter: {x: {$in: [null, []]}}, node: {ixscan: {pattern: {x: 1, _id: 1}, bounds: "
+        "{x: [[undefined,undefined,true,true],[null,null,true,true], [[], [], true, true]]}}}}}}}");
+}
+
+TEST_F(QueryPlannerTest,
+       PlannerUsesCoveredIxscanForFindWhenMultikeyIndexSatisfiesNullAndOtherQuery) {
+    params.mainCollectionInfo.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
+    MultikeyPaths multikeyPaths{{0U}, MultikeyComponents{}};
+    addIndex(fromjson("{x: 1, _id: 1}"), multikeyPaths);
+    runQuerySortProj(fromjson("{x: {$in: [null, 2]}}"), BSONObj(), fromjson("{_id: 1}"));
+    ASSERT_EQUALS(getNumSolutions(), 1U);
     assertSolutionExists(
         "{proj: {spec: {_id: 1}, node:"
         "{ixscan: {pattern: {x: 1, _id: 1}, bounds: "
-        "{x: [[undefined,undefined,true,true],[null,null,true,true], [[], [], true, true]]}}}}}");
+        "{x: [[null,null,true,true], [2, 2, true, true]]}}}}}");
 }
 
 //
@@ -206,7 +250,7 @@ TEST_F(QueryPlannerTest, SparseIndexForQuery) {
 }
 
 TEST_F(QueryPlannerTest, ExprEqCannotUseSparseIndex) {
-    params.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
+    params.mainCollectionInfo.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
     addIndex(fromjson("{a: 1}"), false, true);
     runQuery(fromjson("{a: {$_internalExprEq: 1}}"));
 
@@ -214,7 +258,7 @@ TEST_F(QueryPlannerTest, ExprEqCannotUseSparseIndex) {
 }
 
 TEST_F(QueryPlannerTest, ExprEqCannotUseSparseIndexForEqualityToNull) {
-    params.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
+    params.mainCollectionInfo.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
     addIndex(fromjson("{a: 1}"), false, true);
     runQuery(fromjson("{a: {$_internalExprEq: null}}"));
 
@@ -250,7 +294,7 @@ TEST_F(QueryPlannerTest, NegationInElemMatchDoesNotUseSparseIndex) {
 }
 
 TEST_F(QueryPlannerTest, NinListWithOnlyNullAndEmptyArrayShouldUseMultikeyIndex) {
-    params.options = QueryPlannerParams::NO_TABLE_SCAN;
+    params.mainCollectionInfo.options = QueryPlannerParams::NO_TABLE_SCAN;
     // Use a multikey index.
     addIndex(fromjson("{a: 1}"), true);
     runQuery(fromjson("{a: { $nin: [[], null] }}"));
@@ -259,14 +303,14 @@ TEST_F(QueryPlannerTest, NinListWithOnlyNullAndEmptyArrayShouldUseMultikeyIndex)
         "{fetch: {filter: {a: {$not: {$in: [null, []]}}}, node: {ixscan: "
         "{pattern: {a: 1}, bounds: "
         "{a: ["
-        "['MinKey', undefined, true, false],"
+        "['MinKey', null, true, false],"
         "[null, [], false, false],"
         "[[], 'MaxKey', false, true]"
         "]}}}}}");
 }
 
 TEST_F(QueryPlannerTest, NinListWithOnlyNullAndEmptyArrayShouldUseIndex) {
-    params.options = QueryPlannerParams::NO_TABLE_SCAN;
+    params.mainCollectionInfo.options = QueryPlannerParams::NO_TABLE_SCAN;
     // Use an index which is not multikey.
     addIndex(fromjson("{a: 1}"));
     runQuery(fromjson("{a: { $nin: [[], null] }}"));
@@ -275,7 +319,7 @@ TEST_F(QueryPlannerTest, NinListWithOnlyNullAndEmptyArrayShouldUseIndex) {
         "{fetch: {node: {ixscan: "
         "{pattern: {a: 1}, bounds: "
         "{a: ["
-        "['MinKey', undefined, true, false],"
+        "['MinKey', null, true, false],"
         "[null, [], false, false],"
         "[[], 'MaxKey', false, true]"
         "]}}}}}");
@@ -576,7 +620,7 @@ TEST_F(QueryPlannerTest, CompoundIndexBoundsNotEqualsNull) {
     assertSolutionExists(
         "{fetch: {filter: null, node: {ixscan: {filter: null, pattern: "
         "{a: 1, b: 1}, bounds: {a: [['foo',{},false,false]], "
-        "b:[['MinKey',undefined,true,false],[null,'MaxKey',false,true]]}}}}}");
+        "b:[['MinKey',null,true,false],[null,'MaxKey',false,true]]}}}}}");
 }
 
 TEST_F(QueryPlannerTest, CompoundIndexBoundsDottedNotEqualsNull) {
@@ -588,7 +632,7 @@ TEST_F(QueryPlannerTest, CompoundIndexBoundsDottedNotEqualsNull) {
     assertSolutionExists(
         "{fetch: {filter: null, node: {ixscan: {filter: null, pattern: "
         "{'a.b': 1, 'c.d': 1}, bounds: {'a.b': [['foo',{},false,false]], "
-        "'c.d':[['MinKey',undefined,true,false],[null,'MaxKey',false,true]]}}}}}");
+        "'c.d':[['MinKey',null,true,false],[null,'MaxKey',false,true]]}}}}}");
 }
 
 TEST_F(QueryPlannerTest, CompoundIndexBoundsDottedNotEqualsNullWithProjection) {
@@ -603,7 +647,7 @@ TEST_F(QueryPlannerTest, CompoundIndexBoundsDottedNotEqualsNullWithProjection) {
         "{proj: {spec: {_id: 0, 'c.d': 1}, node: {"
         "  ixscan: {filter: null, pattern: {'a.b': 1, 'c.d': 1}, bounds: {"
         "    'a.b': [['foo',{},false,false]], "
-        "    'c.d':[['MinKey',undefined,true,false],[null,'MaxKey',false,true]]"
+        "    'c.d':[['MinKey',null,true,false],[null,'MaxKey',false,true]]"
         "}}}}}");
 }
 
@@ -693,7 +737,7 @@ TEST_F(QueryPlannerTest, IndexBoundsOrOfNegations) {
 }
 
 TEST_F(QueryPlannerTest, BoundsTypeMinKeyMaxKey) {
-    params.options = QueryPlannerParams::NO_TABLE_SCAN;
+    params.mainCollectionInfo.options = QueryPlannerParams::NO_TABLE_SCAN;
     addIndex(BSON("a" << 1));
 
     runQuery(fromjson("{a: {$type: -1}}"));
@@ -749,7 +793,7 @@ TEST_F(QueryPlannerTest, CompoundMultikeyBoundsNoIntersect) {
 //
 
 TEST_F(QueryPlannerTest, NoTableScanBasic) {
-    params.options = QueryPlannerParams::NO_TABLE_SCAN;
+    params.mainCollectionInfo.options = QueryPlannerParams::NO_TABLE_SCAN;
     runInvalidQuery(BSONObj());
     assertNoSolutions();
 
@@ -765,7 +809,7 @@ TEST_F(QueryPlannerTest, NoTableScanBasic) {
 }
 
 TEST_F(QueryPlannerTest, NoTableScanOrWithAndChild) {
-    params.options = QueryPlannerParams::NO_TABLE_SCAN;
+    params.mainCollectionInfo.options = QueryPlannerParams::NO_TABLE_SCAN;
     addIndex(BSON("a" << 1));
     runQuery(fromjson("{$or: [{a: 20}, {$and: [{a:1}, {b:7}]}]}"));
 
@@ -786,7 +830,8 @@ TEST_F(QueryPlannerTest, NoTableScanOrWithAndChild) {
 //
 
 TEST_F(QueryPlannerTest, IntersectBasicTwoPred) {
-    params.options = QueryPlannerParams::NO_TABLE_SCAN | QueryPlannerParams::INDEX_INTERSECTION;
+    params.mainCollectionInfo.options =
+        QueryPlannerParams::NO_TABLE_SCAN | QueryPlannerParams::INDEX_INTERSECTION;
     addIndex(BSON("a" << 1));
     addIndex(BSON("b" << 1));
     runQuery(fromjson("{a:1, b:{$gt: 1}}"));
@@ -798,7 +843,8 @@ TEST_F(QueryPlannerTest, IntersectBasicTwoPred) {
 }
 
 TEST_F(QueryPlannerTest, IntersectBasicTwoPredCompound) {
-    params.options = QueryPlannerParams::NO_TABLE_SCAN | QueryPlannerParams::INDEX_INTERSECTION;
+    params.mainCollectionInfo.options =
+        QueryPlannerParams::NO_TABLE_SCAN | QueryPlannerParams::INDEX_INTERSECTION;
     addIndex(BSON("a" << 1 << "c" << 1));
     addIndex(BSON("b" << 1));
     runQuery(fromjson("{a:1, b:1, c:1}"));
@@ -812,7 +858,8 @@ TEST_F(QueryPlannerTest, IntersectBasicTwoPredCompound) {
 
 // SERVER-12196
 TEST_F(QueryPlannerTest, IntersectBasicTwoPredCompoundMatchesIdxOrder1) {
-    params.options = QueryPlannerParams::NO_TABLE_SCAN | QueryPlannerParams::INDEX_INTERSECTION;
+    params.mainCollectionInfo.options =
+        QueryPlannerParams::NO_TABLE_SCAN | QueryPlannerParams::INDEX_INTERSECTION;
     addIndex(BSON("a" << 1));
     addIndex(BSON("b" << 1));
     runQuery(fromjson("{a:1, b:1}"));
@@ -833,7 +880,8 @@ TEST_F(QueryPlannerTest, IntersectBasicTwoPredCompoundMatchesIdxOrder1) {
 
 // SERVER-12196
 TEST_F(QueryPlannerTest, IntersectBasicTwoPredCompoundMatchesIdxOrder2) {
-    params.options = QueryPlannerParams::NO_TABLE_SCAN | QueryPlannerParams::INDEX_INTERSECTION;
+    params.mainCollectionInfo.options =
+        QueryPlannerParams::NO_TABLE_SCAN | QueryPlannerParams::INDEX_INTERSECTION;
     addIndex(BSON("b" << 1));
     addIndex(BSON("a" << 1));
     runQuery(fromjson("{a:1, b:1}"));
@@ -853,7 +901,8 @@ TEST_F(QueryPlannerTest, IntersectBasicTwoPredCompoundMatchesIdxOrder2) {
 }
 
 TEST_F(QueryPlannerTest, IntersectManySelfIntersections) {
-    params.options = QueryPlannerParams::NO_TABLE_SCAN | QueryPlannerParams::INDEX_INTERSECTION;
+    params.mainCollectionInfo.options =
+        QueryPlannerParams::NO_TABLE_SCAN | QueryPlannerParams::INDEX_INTERSECTION;
     // True means multikey.
     addIndex(BSON("a" << 1), true);
 
@@ -876,7 +925,8 @@ TEST_F(QueryPlannerTest, IntersectManySelfIntersections) {
 }
 
 TEST_F(QueryPlannerTest, CannotIntersectSubnodes) {
-    params.options = QueryPlannerParams::NO_TABLE_SCAN | QueryPlannerParams::INDEX_INTERSECTION;
+    params.mainCollectionInfo.options =
+        QueryPlannerParams::NO_TABLE_SCAN | QueryPlannerParams::INDEX_INTERSECTION;
     addIndex(BSON("a" << 1));
     addIndex(BSON("b" << 1));
     addIndex(BSON("c" << 1));
@@ -898,7 +948,8 @@ TEST_F(QueryPlannerTest, CannotIntersectSubnodes) {
 }
 
 TEST_F(QueryPlannerTest, IntersectSubtreeAndPred) {
-    params.options = QueryPlannerParams::NO_TABLE_SCAN | QueryPlannerParams::INDEX_INTERSECTION;
+    params.mainCollectionInfo.options =
+        QueryPlannerParams::NO_TABLE_SCAN | QueryPlannerParams::INDEX_INTERSECTION;
     addIndex(BSON("a" << 1));
     addIndex(BSON("b" << 1));
     addIndex(BSON("c" << 1));
@@ -925,7 +976,8 @@ TEST_F(QueryPlannerTest, IntersectSubtreeAndPred) {
 }
 
 TEST_F(QueryPlannerTest, IntersectElemMatch) {
-    params.options = QueryPlannerParams::NO_TABLE_SCAN | QueryPlannerParams::INDEX_INTERSECTION;
+    params.mainCollectionInfo.options =
+        QueryPlannerParams::NO_TABLE_SCAN | QueryPlannerParams::INDEX_INTERSECTION;
     addIndex(BSON("a.b" << 1));
     addIndex(BSON("a.c" << 1));
     runQuery(fromjson("{a : {$elemMatch: {b:1, c:1}}}"));
@@ -937,7 +989,8 @@ TEST_F(QueryPlannerTest, IntersectElemMatch) {
 }
 
 TEST_F(QueryPlannerTest, IntersectSortFromAndHash) {
-    params.options = QueryPlannerParams::NO_TABLE_SCAN | QueryPlannerParams::INDEX_INTERSECTION;
+    params.mainCollectionInfo.options =
+        QueryPlannerParams::NO_TABLE_SCAN | QueryPlannerParams::INDEX_INTERSECTION;
     addIndex(BSON("a" << 1));
     addIndex(BSON("b" << 1));
     runQuerySortProj(fromjson("{a: 1, b:{$gt: 1}}"), fromjson("{b:1}"), BSONObj());
@@ -957,7 +1010,8 @@ TEST_F(QueryPlannerTest, IntersectSortFromAndHash) {
 }
 
 TEST_F(QueryPlannerTest, IntersectCanBeVeryBig) {
-    params.options = QueryPlannerParams::NO_TABLE_SCAN | QueryPlannerParams::INDEX_INTERSECTION;
+    params.mainCollectionInfo.options =
+        QueryPlannerParams::NO_TABLE_SCAN | QueryPlannerParams::INDEX_INTERSECTION;
     addIndex(BSON("a" << 1));
     addIndex(BSON("b" << 1));
     addIndex(BSON("c" << 1));
@@ -980,7 +1034,8 @@ TEST_F(QueryPlannerTest, IntersectDisableAndHash) {
 
     // Turn index intersection on but disable hash-based intersection.
     internalQueryPlannerEnableHashIntersection.store(false);
-    params.options = QueryPlannerParams::NO_TABLE_SCAN | QueryPlannerParams::INDEX_INTERSECTION;
+    params.mainCollectionInfo.options =
+        QueryPlannerParams::NO_TABLE_SCAN | QueryPlannerParams::INDEX_INTERSECTION;
 
     addIndex(BSON("a" << 1));
     addIndex(BSON("b" << 1));
@@ -1016,7 +1071,8 @@ TEST_F(QueryPlannerTest, IntersectDisableAndHash) {
 
 // SERVER-12825
 TEST_F(QueryPlannerTest, IntersectCompoundInsteadBasic) {
-    params.options = QueryPlannerParams::NO_TABLE_SCAN | QueryPlannerParams::INDEX_INTERSECTION;
+    params.mainCollectionInfo.options =
+        QueryPlannerParams::NO_TABLE_SCAN | QueryPlannerParams::INDEX_INTERSECTION;
     addIndex(BSON("a" << 1));
     addIndex(BSON("b" << 1));
     addIndex(BSON("a" << 1 << "b" << 1));
@@ -1036,7 +1092,8 @@ TEST_F(QueryPlannerTest, IntersectCompoundInsteadBasic) {
 
 // SERVER-12825
 TEST_F(QueryPlannerTest, IntersectCompoundInsteadThreeCompoundIndices) {
-    params.options = QueryPlannerParams::NO_TABLE_SCAN | QueryPlannerParams::INDEX_INTERSECTION;
+    params.mainCollectionInfo.options =
+        QueryPlannerParams::NO_TABLE_SCAN | QueryPlannerParams::INDEX_INTERSECTION;
     addIndex(BSON("a" << 1 << "b" << 1));
     addIndex(BSON("c" << 1 << "d" << 1));
     addIndex(BSON("a" << 1 << "c" << -1 << "b" << -1 << "d" << 1));
@@ -1056,7 +1113,8 @@ TEST_F(QueryPlannerTest, IntersectCompoundInsteadThreeCompoundIndices) {
 
 // SERVER-12825
 TEST_F(QueryPlannerTest, IntersectCompoundInsteadUnusedField) {
-    params.options = QueryPlannerParams::NO_TABLE_SCAN | QueryPlannerParams::INDEX_INTERSECTION;
+    params.mainCollectionInfo.options =
+        QueryPlannerParams::NO_TABLE_SCAN | QueryPlannerParams::INDEX_INTERSECTION;
     addIndex(BSON("a" << 1));
     addIndex(BSON("b" << 1));
     addIndex(BSON("a" << 1 << "b" << 1 << "c" << 1));
@@ -1076,7 +1134,8 @@ TEST_F(QueryPlannerTest, IntersectCompoundInsteadUnusedField) {
 
 // SERVER-12825
 TEST_F(QueryPlannerTest, IntersectCompoundInsteadUnusedField2) {
-    params.options = QueryPlannerParams::NO_TABLE_SCAN | QueryPlannerParams::INDEX_INTERSECTION;
+    params.mainCollectionInfo.options =
+        QueryPlannerParams::NO_TABLE_SCAN | QueryPlannerParams::INDEX_INTERSECTION;
     addIndex(BSON("a" << 1 << "b" << 1));
     addIndex(BSON("c" << 1 << "d" << 1));
     addIndex(BSON("a" << 1 << "b" << 1 << "c" << 1));
@@ -1097,7 +1156,7 @@ TEST_F(QueryPlannerTest, IntersectCompoundInsteadUnusedField2) {
 // Make sure a top-level $or hits the limiting number
 // of solutions that we are willing to consider.
 TEST_F(QueryPlannerTest, OrEnumerationLimit) {
-    params.options = QueryPlannerParams::NO_TABLE_SCAN;
+    params.mainCollectionInfo.options = QueryPlannerParams::NO_TABLE_SCAN;
     addIndex(BSON("a" << 1));
     addIndex(BSON("b" << 1));
 
@@ -1115,7 +1174,7 @@ TEST_F(QueryPlannerTest, OrEnumerationLimit) {
 }
 
 TEST_F(QueryPlannerTest, OrEnumerationLimit2) {
-    params.options = QueryPlannerParams::NO_TABLE_SCAN;
+    params.mainCollectionInfo.options = QueryPlannerParams::NO_TABLE_SCAN;
     addIndex(BSON("a" << 1));
     addIndex(BSON("b" << 1));
     addIndex(BSON("c" << 1));
@@ -1133,7 +1192,7 @@ TEST_F(QueryPlannerTest, OrEnumerationLimit2) {
 
 // SERVER-13104: test that we properly enumerate all solutions for nested $or.
 TEST_F(QueryPlannerTest, EnumerateNestedOr) {
-    params.options = QueryPlannerParams::NO_TABLE_SCAN;
+    params.mainCollectionInfo.options = QueryPlannerParams::NO_TABLE_SCAN;
     addIndex(BSON("a" << 1));
     addIndex(BSON("b" << 1));
     addIndex(BSON("c" << 1));
@@ -1153,7 +1212,7 @@ TEST_F(QueryPlannerTest, EnumerateNestedOr) {
 
 // SERVER-13104: test that we properly enumerate all solutions for nested $or.
 TEST_F(QueryPlannerTest, EnumerateNestedOr2) {
-    params.options = QueryPlannerParams::NO_TABLE_SCAN;
+    params.mainCollectionInfo.options = QueryPlannerParams::NO_TABLE_SCAN;
     addIndex(BSON("a" << 1));
     addIndex(BSON("b" << 1));
     addIndex(BSON("c" << 1));
@@ -1194,7 +1253,7 @@ TEST_F(QueryPlannerTest, EnumerateNestedOr2) {
 
 
 TEST_F(QueryPlannerTest, FloatingPointInKeyPattern) {
-    params.options = QueryPlannerParams::NO_TABLE_SCAN;
+    params.mainCollectionInfo.options = QueryPlannerParams::NO_TABLE_SCAN;
 
     addIndex(BSON("a" << -0.1));
 
@@ -1207,7 +1266,7 @@ TEST_F(QueryPlannerTest, FloatingPointInKeyPattern) {
 }
 
 TEST_F(QueryPlannerTest, KeyPatternOverflowsInt) {
-    params.options = QueryPlannerParams::NO_TABLE_SCAN;
+    params.mainCollectionInfo.options = QueryPlannerParams::NO_TABLE_SCAN;
 
     addIndex(BSON("a" << -2147483649LL));
 
@@ -1221,7 +1280,7 @@ TEST_F(QueryPlannerTest, KeyPatternOverflowsInt) {
 
 
 TEST_F(QueryPlannerTest, NorWithSingleChildCanUseIndexAfterComplementingBounds) {
-    params.options = QueryPlannerParams::NO_TABLE_SCAN;
+    params.mainCollectionInfo.options = QueryPlannerParams::NO_TABLE_SCAN;
 
     addIndex(BSON("a" << 1));
     runQuery(fromjson("{$nor: [{a: {$lt: 3}}]}"));
@@ -1256,7 +1315,7 @@ TEST_F(QueryPlannerTest, EmptyQueryWithoutProjectionUsesCollscan) {
 }
 
 TEST_F(QueryPlannerTest, EmptyQueryWithProjectionUsesCoveredIxscanIfEnabled) {
-    params.options = QueryPlannerParams::GENERATE_COVERED_IXSCANS;
+    params.mainCollectionInfo.options = QueryPlannerParams::GENERATE_COVERED_IXSCANS;
     addIndex(BSON("a" << 1));
     runQueryAsCommand(fromjson("{find: 'testns', projection: {_id: 0, a: 1}}"));
     assertNumSolutions(1);
@@ -1267,7 +1326,7 @@ TEST_F(QueryPlannerTest, EmptyQueryWithProjectionUsesCoveredIxscanIfEnabled) {
 }
 
 TEST_F(QueryPlannerTest, EmptyQueryWithProjectionDoesNotUseCoveredIxscanIfDisabled) {
-    params.options &= ~QueryPlannerParams::GENERATE_COVERED_IXSCANS;
+    params.mainCollectionInfo.options &= ~QueryPlannerParams::GENERATE_COVERED_IXSCANS;
     addIndex(BSON("a" << 1));
     runQueryAsCommand(fromjson("{find: 'testns', projection: {_id: 0, a: 1}}"));
     assertNumSolutions(1);
@@ -1277,7 +1336,7 @@ TEST_F(QueryPlannerTest, EmptyQueryWithProjectionDoesNotUseCoveredIxscanIfDisabl
 }
 
 TEST_F(QueryPlannerTest, EmptyQueryWithProjectionUsesCoveredIxscanOnCompoundIndexIfEnabled) {
-    params.options = QueryPlannerParams::GENERATE_COVERED_IXSCANS;
+    params.mainCollectionInfo.options = QueryPlannerParams::GENERATE_COVERED_IXSCANS;
     addIndex(BSON("a" << 1 << "b" << 1 << "c" << 1));
     runQueryAsCommand(fromjson("{find: 'testns', projection: {_id: 0, a: 1, c: 1}}"));
     assertNumSolutions(1);
@@ -1289,7 +1348,7 @@ TEST_F(QueryPlannerTest, EmptyQueryWithProjectionUsesCoveredIxscanOnCompoundInde
 }
 
 TEST_F(QueryPlannerTest, EmptyQueryWithProjectionDoesNotUseCoveredIxscanOnCompoundIndexIfDisabled) {
-    params.options &= ~QueryPlannerParams::GENERATE_COVERED_IXSCANS;
+    params.mainCollectionInfo.options &= ~QueryPlannerParams::GENERATE_COVERED_IXSCANS;
     addIndex(BSON("a" << 1 << "b" << 1 << "c" << 1));
     runQueryAsCommand(fromjson("{find: 'testns', projection: {_id: 0, a: 1, c: 1}}"));
     assertNumSolutions(1);
@@ -1299,7 +1358,7 @@ TEST_F(QueryPlannerTest, EmptyQueryWithProjectionDoesNotUseCoveredIxscanOnCompou
 }
 
 TEST_F(QueryPlannerTest, EmptyQueryWithProjectionDoesNotConsiderNonHintedIndices) {
-    params.options = QueryPlannerParams::GENERATE_COVERED_IXSCANS;
+    params.mainCollectionInfo.options = QueryPlannerParams::GENERATE_COVERED_IXSCANS;
     addIndex(BSON("a" << 1));
     runQueryAsCommand(fromjson("{find: 'testns', projection: {_id: 0, a: 1}, hint: {_id: 1}}"));
     assertNumSolutions(1);
@@ -1311,7 +1370,7 @@ TEST_F(QueryPlannerTest, EmptyQueryWithProjectionDoesNotConsiderNonHintedIndices
 }
 
 TEST_F(QueryPlannerTest, EmptyQueryWithProjectionUsesCollscanIfNoCoveredIxscans) {
-    params.options = QueryPlannerParams::GENERATE_COVERED_IXSCANS;
+    params.mainCollectionInfo.options = QueryPlannerParams::GENERATE_COVERED_IXSCANS;
     addIndex(BSON("a" << 1));
     runQueryAsCommand(fromjson("{find: 'testns', projection: {a: 1}}"));
     assertNumSolutions(1);
@@ -1322,7 +1381,7 @@ TEST_F(QueryPlannerTest, EmptyQueryWithProjectionUsesCollscanIfNoCoveredIxscans)
 
 TEST_F(QueryPlannerTest,
        EmptyQueryWithProjectionUsesCoveredIxscanOnDotttedNonMultikeyIndexIfEnabled) {
-    params.options = QueryPlannerParams::GENERATE_COVERED_IXSCANS;
+    params.mainCollectionInfo.options = QueryPlannerParams::GENERATE_COVERED_IXSCANS;
     addIndex(BSON("a.b" << 1));
     runQueryAsCommand(fromjson("{find: 'testns', projection: {_id: 0, 'a.b': 1}}"));
     assertNumSolutions(1);
@@ -1334,7 +1393,7 @@ TEST_F(QueryPlannerTest,
 
 TEST_F(QueryPlannerTest,
        EmptyQueryWithProjectionDoesNotUseCoveredIxscanOnDotttedNonMultikeyIndexIfDisabled) {
-    params.options &= ~QueryPlannerParams::GENERATE_COVERED_IXSCANS;
+    params.mainCollectionInfo.options &= ~QueryPlannerParams::GENERATE_COVERED_IXSCANS;
     addIndex(BSON("a.b" << 1));
     runQueryAsCommand(fromjson("{find: 'testns', projection: {_id: 0, 'a.b': 1}}"));
     assertNumSolutions(1);
@@ -1344,7 +1403,7 @@ TEST_F(QueryPlannerTest,
 }
 
 TEST_F(QueryPlannerTest, EmptyQueryWithProjectionUsesCollscanIfIndexIsMultikey) {
-    params.options = QueryPlannerParams::GENERATE_COVERED_IXSCANS;
+    params.mainCollectionInfo.options = QueryPlannerParams::GENERATE_COVERED_IXSCANS;
     constexpr bool isMultikey = true;
     addIndex(BSON("a" << 1 << "b" << 1), isMultikey);
     runQueryAsCommand(fromjson("{find: 'testns', projection: {_id: 0, a: 1, b: 1}}"));
@@ -1355,7 +1414,7 @@ TEST_F(QueryPlannerTest, EmptyQueryWithProjectionUsesCollscanIfIndexIsMultikey)
 }
 
 TEST_F(QueryPlannerTest, EmptyQueryWithProjectionUsesCollscanIfIndexIsSparse) {
-    params.options = QueryPlannerParams::GENERATE_COVERED_IXSCANS;
+    params.mainCollectionInfo.options = QueryPlannerParams::GENERATE_COVERED_IXSCANS;
     constexpr bool isMultikey = false;
     constexpr bool isSparse = true;
     addIndex(BSON("a" << 1), isMultikey, isSparse);
@@ -1367,7 +1426,7 @@ TEST_F(QueryPlannerTest, EmptyQueryWithProjectionUsesCollscanIfIndexIsSparse) {
 }
 
 TEST_F(QueryPlannerTest, EmptyQueryWithProjectionUsesCollscanIfIndexIsPartial) {
-    params.options = QueryPlannerParams::GENERATE_COVERED_IXSCANS;
+    params.mainCollectionInfo.options = QueryPlannerParams::GENERATE_COVERED_IXSCANS;
     AlwaysFalseMatchExpression matchExpr;
     addIndex(BSON("a" << 1), &matchExpr);
     runQueryAsCommand(fromjson("{find: 'testns', projection: {_id: 0, a: 1}}"));
@@ -1378,7 +1437,7 @@ TEST_F(QueryPlannerTest, EmptyQueryWithProjectionUsesCollscanIfIndexIsPartial) {
 }
 
 TEST_F(QueryPlannerTest, EmptyQueryWithProjectionUsesCollscanIfIndexIsText) {
-    params.options = QueryPlannerParams::GENERATE_COVERED_IXSCANS;
+    params.mainCollectionInfo.options = QueryPlannerParams::GENERATE_COVERED_IXSCANS;
     addIndex(BSON("a"
                   << "text"));
     runQueryAsCommand(fromjson("{find: 'testns', projection: {_id: 0, a: 1}}"));
@@ -1389,7 +1448,7 @@ TEST_F(QueryPlannerTest, EmptyQueryWithProjectionUsesCollscanIfIndexIsText) {
 }
 
 TEST_F(QueryPlannerTest, EmptyQueryWithProjectionUsesCollscanIfIndexIsGeo) {
-    params.options = QueryPlannerParams::GENERATE_COVERED_IXSCANS;
+    params.mainCollectionInfo.options = QueryPlannerParams::GENERATE_COVERED_IXSCANS;
     addIndex(BSON("a"
                   << "2dsphere"));
     runQueryAsCommand(fromjson("{find: 'testns', projection: {_id: 0, a: 1}}"));
@@ -1400,7 +1459,7 @@ TEST_F(QueryPlannerTest, EmptyQueryWithProjectionUsesCollscanIfIndexIsGeo) {
 }
 
 TEST_F(QueryPlannerTest, EmptyQueryWithProjectionUsesCollscanIfIndexCollationDiffers) {
-    params.options = QueryPlannerParams::GENERATE_COVERED_IXSCANS;
+    params.mainCollectionInfo.options = QueryPlannerParams::GENERATE_COVERED_IXSCANS;
     CollatorInterfaceMock collator(CollatorInterfaceMock::MockType::kReverseString);
     addIndex(BSON("a" << 1), &collator);
     runQueryAsCommand(fromjson("{find: 'testns', projection: {_id: 0, a: 1}}"));
@@ -1412,7 +1471,7 @@ TEST_F(QueryPlannerTest, EmptyQueryWithProjectionUsesCollscanIfIndexCollationDif
 
 
 TEST_F(QueryPlannerTest, NoFetchStageWhenSingleFieldSortIsCoveredByIndex) {
-    params.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
+    params.mainCollectionInfo.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
     addIndex(fromjson("{a: 1, b: 1}"));
 
     // Sort on 'b'.
@@ -1427,7 +1486,7 @@ TEST_F(QueryPlannerTest, NoFetchStageWhenSingleFieldSortIsCoveredByIndex) {
 }
 
 TEST_F(QueryPlannerTest, NoFetchStageWhenTwoFieldAscendingSortIsCoveredByIndex) {
-    params.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
+    params.mainCollectionInfo.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
     addIndex(fromjson("{a: 1, b: 1}"));
 
     // Sort on 'b', 'a'.
@@ -1442,7 +1501,7 @@ TEST_F(QueryPlannerTest, NoFetchStageWhenTwoFieldAscendingSortIsCoveredByIndex)
 }
 
 TEST_F(QueryPlannerTest, NoFetchStageWhenTwoFieldMixedSortOrderSortIsCoveredByIndex) {
-    params.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
+    params.mainCollectionInfo.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
     addIndex(fromjson("{a: 1, b: 1}"));
 
     // Sort on 'b', 'a' descending.
@@ -1456,7 +1515,7 @@ TEST_F(QueryPlannerTest, NoFetchStageWhenTwoFieldMixedSortOrderSortIsCoveredByIn
 }
 
 TEST_F(QueryPlannerTest, MustFetchWhenNotAllSortKeysAreCoveredByIndex) {
-    params.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
+    params.mainCollectionInfo.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
     addIndex(fromjson("{a: 1, b: 1}"));
 
     runQueryAsCommand(
@@ -1472,7 +1531,7 @@ TEST_F(QueryPlannerTest, MustFetchWhenNotAllSortKeysAreCoveredByIndex) {
 }
 
 TEST_F(QueryPlannerTest, NoFetchStageWhenProjectionUsesExpressionWithCoveredDependency) {
-    params.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
+    params.mainCollectionInfo.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
     addIndex(fromjson("{a: 1, b: 1}"));
 
     runQueryAsCommand(fromjson(
@@ -1485,7 +1544,7 @@ TEST_F(QueryPlannerTest, NoFetchStageWhenProjectionUsesExpressionWithCoveredDepe
 
 TEST_F(QueryPlannerTest,
        NoFetchStageWhenProjectionAssignsExpressionWithCoveredDependencyToUnIndexedField) {
-    params.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
+    params.mainCollectionInfo.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
     addIndex(fromjson("{a: 1, b: 1}"));
 
     // Assign the result of the expression to 'x' which has no index.
@@ -1498,7 +1557,7 @@ TEST_F(QueryPlannerTest,
 }
 
 TEST_F(QueryPlannerTest, AddsFetchWhenProjectionAssignsToUnindexedDottedField) {
-    params.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
+    params.mainCollectionInfo.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
     addIndex(fromjson("{a: 1, b: 1}"));
 
     // Cannot be covered since 'x' may be an array.
@@ -1511,7 +1570,7 @@ TEST_F(QueryPlannerTest, AddsFetchWhenProjectionAssignsToUnindexedDottedField) {
 }
 
 TEST_F(QueryPlannerTest, NoFetchWhenProjectionAssignsToDottedIndexedNonArrayField) {
-    params.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
+    params.mainCollectionInfo.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
     addIndex(fromjson("{a: 1, b: 1, 'b.c': 1}"));
 
     // Can be covered since 'b' and 'b.c' are known to not be multikey.
@@ -1524,7 +1583,7 @@ TEST_F(QueryPlannerTest, NoFetchWhenProjectionAssignsToDottedIndexedNonArrayFiel
 }
 
 TEST_F(QueryPlannerTest, NoFetchWhenProjectionAssignsToIndexedNonArrayField) {
-    params.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
+    params.mainCollectionInfo.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
     addIndex(fromjson("{a: 1, b: 1}"));
 
     // Can be covered since 'b' is known to not be multikey.
@@ -1537,7 +1596,7 @@ TEST_F(QueryPlannerTest, NoFetchWhenProjectionAssignsToIndexedNonArrayField) {
 }
 
 TEST_F(QueryPlannerTest, MustFetchWhenExpressionUsesMultiKeyPath) {
-    params.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
+    params.mainCollectionInfo.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
     addIndex(fromjson("{a: 1, b: 1}"), {{}, {0}});
 
     // Cannot be covered since 'b' is multikey.
@@ -1550,7 +1609,7 @@ TEST_F(QueryPlannerTest, MustFetchWhenExpressionUsesMultiKeyPath) {
 }
 
 TEST_F(QueryPlannerTest, MustFetchWhenExpressionUsesDottedMultiKeyPath) {
-    params.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
+    params.mainCollectionInfo.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
     addIndex(fromjson("{a: 1, b: 1}"), {{}, {0}});
 
     // Cannot be covered since 'b' is multikey, meaning the result of the expression '$b.c' could
@@ -1564,7 +1623,7 @@ TEST_F(QueryPlannerTest, MustFetchWhenExpressionUsesDottedMultiKeyPath) {
 }
 
 TEST_F(QueryPlannerTest, MustFetchWhenExpressionUsesROOT) {
-    params.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
+    params.mainCollectionInfo.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
     addIndex(fromjson("{a: 1, b: 1}"), {{}, {0}});
 
     runQueryAsCommand(
@@ -1576,7 +1635,7 @@ TEST_F(QueryPlannerTest, MustFetchWhenExpressionUsesROOT) {
 }
 
 TEST_F(QueryPlannerTest, PlansForWholeIndexScanWithSortAreGenerated) {
-    params.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
+    params.mainCollectionInfo.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
     addIndex(BSON("a" << 1));
     addIndex(BSON("a" << 1 << "b" << 1));
 
