@@ -27,13 +27,40 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <cstddef>
+#include <map>
+#include <memory>
+#include <string>
+#include <utility>
+#include <vector>
 
-#include "mongo/db/query/collation/collator_interface_mock.h"
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/json.h"
+#include "mongo/db/field_ref.h"
+#include "mongo/db/index/index_descriptor.h"
+#include "mongo/db/index_names.h"
+#include "mongo/db/matcher/expression.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/query/canonical_query.h"
+#include "mongo/db/query/classic_plan_cache.h"
+#include "mongo/db/query/find_command.h"
+#include "mongo/db/query/index_entry.h"
 #include "mongo/db/query/index_tag.h"
 #include "mongo/db/query/query_planner.h"
+#include "mongo/db/query/query_planner_params.h"
 #include "mongo/db/query/query_planner_test_fixture.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/db/query/query_solution.h"
+#include "mongo/db/record_id.h"
+#include "mongo/db/service_context.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
 
 namespace mongo {
 namespace {
@@ -384,7 +411,7 @@ TEST_F(QueryPlannerTest, HintInvalid) {
 //
 
 TEST_F(QueryPlannerTest, ShardFilterCollScan) {
-    params.options = QueryPlannerParams::INCLUDE_SHARD_FILTER;
+    params.mainCollectionInfo.options = QueryPlannerParams::INCLUDE_SHARD_FILTER;
     params.shardKey = BSON("a" << 1);
     addIndex(BSON("a" << 1));
 
@@ -397,7 +424,7 @@ TEST_F(QueryPlannerTest, ShardFilterCollScan) {
 }
 
 TEST_F(QueryPlannerTest, ShardFilterBasicIndex) {
-    params.options = QueryPlannerParams::INCLUDE_SHARD_FILTER;
+    params.mainCollectionInfo.options = QueryPlannerParams::INCLUDE_SHARD_FILTER;
     params.shardKey = BSON("a" << 1);
     addIndex(BSON("a" << 1));
     addIndex(BSON("b" << 1));
@@ -412,7 +439,7 @@ TEST_F(QueryPlannerTest, ShardFilterBasicIndex) {
 }
 
 TEST_F(QueryPlannerTest, ShardFilterBasicCovered) {
-    params.options = QueryPlannerParams::INCLUDE_SHARD_FILTER;
+    params.mainCollectionInfo.options = QueryPlannerParams::INCLUDE_SHARD_FILTER;
     params.shardKey = BSON("a" << 1);
     addIndex(BSON("a" << 1));
 
@@ -426,7 +453,7 @@ TEST_F(QueryPlannerTest, ShardFilterBasicCovered) {
 }
 
 TEST_F(QueryPlannerTest, ShardFilterBasicProjCovered) {
-    params.options = QueryPlannerParams::INCLUDE_SHARD_FILTER;
+    params.mainCollectionInfo.options = QueryPlannerParams::INCLUDE_SHARD_FILTER;
     params.shardKey = BSON("a" << 1);
     addIndex(BSON("a" << 1));
 
@@ -440,7 +467,7 @@ TEST_F(QueryPlannerTest, ShardFilterBasicProjCovered) {
 }
 
 TEST_F(QueryPlannerTest, ShardFilterCompoundProjCovered) {
-    params.options = QueryPlannerParams::INCLUDE_SHARD_FILTER;
+    params.mainCollectionInfo.options = QueryPlannerParams::INCLUDE_SHARD_FILTER;
     params.shardKey = BSON("a" << 1 << "b" << 1);
     addIndex(BSON("a" << 1 << "b" << 1));
 
@@ -454,7 +481,7 @@ TEST_F(QueryPlannerTest, ShardFilterCompoundProjCovered) {
 }
 
 TEST_F(QueryPlannerTest, ShardFilterNestedProjCovered) {
-    params.options = QueryPlannerParams::INCLUDE_SHARD_FILTER;
+    params.mainCollectionInfo.options = QueryPlannerParams::INCLUDE_SHARD_FILTER;
     params.shardKey = BSON("a" << 1 << "b.c" << 1);
     addIndex(BSON("a" << 1 << "b.c" << 1));
 
@@ -468,7 +495,7 @@ TEST_F(QueryPlannerTest, ShardFilterNestedProjCovered) {
 }
 
 TEST_F(QueryPlannerTest, ShardFilterHashProjNotCovered) {
-    params.options = QueryPlannerParams::INCLUDE_SHARD_FILTER;
+    params.mainCollectionInfo.options = QueryPlannerParams::INCLUDE_SHARD_FILTER;
     params.shardKey = BSON("a"
                            << "hashed");
     addIndex(BSON("a"
@@ -485,7 +512,7 @@ TEST_F(QueryPlannerTest, ShardFilterHashProjNotCovered) {
 }
 
 TEST_F(QueryPlannerTest, ShardFilterKeyPrefixIndexCovered) {
-    params.options = QueryPlannerParams::INCLUDE_SHARD_FILTER;
+    params.mainCollectionInfo.options = QueryPlannerParams::INCLUDE_SHARD_FILTER;
     params.shardKey = BSON("a" << 1);
     addIndex(BSON("a" << 1 << "b" << 1 << "_id" << 1));
 
@@ -499,7 +526,7 @@ TEST_F(QueryPlannerTest, ShardFilterKeyPrefixIndexCovered) {
 }
 
 TEST_F(QueryPlannerTest, ShardFilterNoIndexNotCovered) {
-    params.options = QueryPlannerParams::INCLUDE_SHARD_FILTER;
+    params.mainCollectionInfo.options = QueryPlannerParams::INCLUDE_SHARD_FILTER;
     params.shardKey = BSON("a"
                            << "hashed");
     addIndex(BSON("b" << 1));
@@ -515,8 +542,8 @@ TEST_F(QueryPlannerTest, ShardFilterNoIndexNotCovered) {
 }
 
 TEST_F(QueryPlannerTest, CannotTrimIxisectParam) {
-    params.options = QueryPlannerParams::INDEX_INTERSECTION;
-    params.options |= QueryPlannerParams::NO_TABLE_SCAN;
+    params.mainCollectionInfo.options = QueryPlannerParams::INDEX_INTERSECTION;
+    params.mainCollectionInfo.options |= QueryPlannerParams::NO_TABLE_SCAN;
 
     addIndex(BSON("a" << 1));
     addIndex(BSON("b" << 1));
@@ -537,8 +564,8 @@ TEST_F(QueryPlannerTest, CannotTrimIxisectParam) {
 }
 
 TEST_F(QueryPlannerTest, CannotTrimIxisectParamBeneathOr) {
-    params.options = QueryPlannerParams::INDEX_INTERSECTION;
-    params.options |= QueryPlannerParams::NO_TABLE_SCAN;
+    params.mainCollectionInfo.options = QueryPlannerParams::INDEX_INTERSECTION;
+    params.mainCollectionInfo.options |= QueryPlannerParams::NO_TABLE_SCAN;
 
     addIndex(BSON("a" << 1));
     addIndex(BSON("b" << 1));
@@ -571,8 +598,8 @@ TEST_F(QueryPlannerTest, CannotTrimIxisectParamBeneathOr) {
 }
 
 TEST_F(QueryPlannerTest, CannotTrimIxisectAndHashWithOrChild) {
-    params.options = QueryPlannerParams::INDEX_INTERSECTION;
-    params.options |= QueryPlannerParams::NO_TABLE_SCAN;
+    params.mainCollectionInfo.options = QueryPlannerParams::INDEX_INTERSECTION;
+    params.mainCollectionInfo.options |= QueryPlannerParams::NO_TABLE_SCAN;
 
     addIndex(BSON("a" << 1));
     addIndex(BSON("b" << 1));
@@ -602,8 +629,8 @@ TEST_F(QueryPlannerTest, CannotTrimIxisectAndHashWithOrChild) {
 }
 
 TEST_F(QueryPlannerTest, CannotTrimIxisectParamSelfIntersection) {
-    params.options = QueryPlannerParams::INDEX_INTERSECTION;
-    params.options |= QueryPlannerParams::NO_TABLE_SCAN;
+    params.mainCollectionInfo.options = QueryPlannerParams::INDEX_INTERSECTION;
+    params.mainCollectionInfo.options |= QueryPlannerParams::NO_TABLE_SCAN;
 
     // true means multikey
     addIndex(BSON("a" << 1), true);
@@ -634,8 +661,8 @@ TEST_F(QueryPlannerTest, CannotTrimIxisectParamSelfIntersection) {
 // If a lookup against a unique index is available as a possible plan, then the planner
 // should not generate other possibilities.
 TEST_F(QueryPlannerTest, UniqueIndexLookup) {
-    params.options = QueryPlannerParams::INDEX_INTERSECTION;
-    params.options |= QueryPlannerParams::NO_TABLE_SCAN;
+    params.mainCollectionInfo.options = QueryPlannerParams::INDEX_INTERSECTION;
+    params.mainCollectionInfo.options |= QueryPlannerParams::NO_TABLE_SCAN;
 
     addIndex(BSON("a" << 1));
     addIndex(BSON("b" << 1),
@@ -652,7 +679,7 @@ TEST_F(QueryPlannerTest, UniqueIndexLookup) {
 }
 
 TEST_F(QueryPlannerTest, HintOnNonUniqueIndex) {
-    params.options = QueryPlannerParams::INDEX_INTERSECTION;
+    params.mainCollectionInfo.options = QueryPlannerParams::INDEX_INTERSECTION;
 
     addIndex(BSON("a" << 1));
     addIndex(BSON("b" << 1),
@@ -669,7 +696,7 @@ TEST_F(QueryPlannerTest, HintOnNonUniqueIndex) {
 }
 
 TEST_F(QueryPlannerTest, UniqueIndexLookupBelowOr) {
-    params.options = QueryPlannerParams::NO_TABLE_SCAN;
+    params.mainCollectionInfo.options = QueryPlannerParams::NO_TABLE_SCAN;
 
     addIndex(BSON("a" << 1));
     addIndex(BSON("b" << 1));
@@ -695,7 +722,7 @@ TEST_F(QueryPlannerTest, UniqueIndexLookupBelowOr) {
 }
 
 TEST_F(QueryPlannerTest, UniqueIndexLookupBelowOrBelowAnd) {
-    params.options = QueryPlannerParams::NO_TABLE_SCAN;
+    params.mainCollectionInfo.options = QueryPlannerParams::NO_TABLE_SCAN;
 
     addIndex(BSON("a" << 1));
     addIndex(BSON("b" << 1));
@@ -723,7 +750,7 @@ TEST_F(QueryPlannerTest, UniqueIndexLookupBelowOrBelowAnd) {
 }
 
 TEST_F(QueryPlannerTest, CoveredOrUniqueIndexLookup) {
-    params.options = QueryPlannerParams::NO_TABLE_SCAN;
+    params.mainCollectionInfo.options = QueryPlannerParams::NO_TABLE_SCAN;
 
     addIndex(BSON("a" << 1 << "b" << 1));
     addIndex(BSON("a" << 1),
@@ -791,13 +818,14 @@ TEST_F(QueryPlannerTest, CacheDataFromTaggedTreeFailsOnBadInput) {
     auto findCommand = std::make_unique<FindCommandRequest>(
         NamespaceString::createNamespaceString_forTest("test.collection"));
     findCommand->setFilter(BSON("a" << 3));
-    auto statusWithCQ = CanonicalQuery::canonicalize(opCtx.get(), std::move(findCommand));
-    ASSERT_OK(statusWithCQ.getStatus());
-    std::unique_ptr<CanonicalQuery> scopedCq = std::move(statusWithCQ.getValue());
-    scopedCq->root()->setTag(new IndexTag(1));
+    auto scopedCq = std::make_unique<CanonicalQuery>(
+        CanonicalQueryParams{.expCtx = makeExpressionContext(opCtx.get(), *findCommand),
+                             .parsedFind = ParsedFindCommandParams{std::move(findCommand)}});
+    scopedCq->getPrimaryMatchExpression()->setTag(new IndexTag(1));
 
-    ASSERT_NOT_OK(
-        QueryPlanner::cacheDataFromTaggedTree(scopedCq->root(), relevantIndices).getStatus());
+    ASSERT_NOT_OK(QueryPlanner::cacheDataFromTaggedTree(scopedCq->getPrimaryMatchExpression(),
+                                                        relevantIndices)
+                      .getStatus());
 }
 
 TEST_F(QueryPlannerTest, TagAccordingToCacheFailsOnBadInput) {
@@ -805,9 +833,9 @@ TEST_F(QueryPlannerTest, TagAccordingToCacheFailsOnBadInput) {
 
     auto findCommand = std::make_unique<FindCommandRequest>(nss);
     findCommand->setFilter(BSON("a" << 3));
-    auto statusWithCQ = CanonicalQuery::canonicalize(opCtx.get(), std::move(findCommand));
-    ASSERT_OK(statusWithCQ.getStatus());
-    std::unique_ptr<CanonicalQuery> scopedCq = std::move(statusWithCQ.getValue());
+    auto scopedCq = std::make_unique<CanonicalQuery>(
+        CanonicalQueryParams{.expCtx = makeExpressionContext(opCtx.get(), *findCommand),
+                             .parsedFind = ParsedFindCommandParams{std::move(findCommand)}});
 
     std::unique_ptr<PlanCacheIndexTree> indexTree(new PlanCacheIndexTree());
     indexTree->setIndexEntry(buildSimpleIndexEntry(BSON("a" << 1), "a_1"));
@@ -819,35 +847,38 @@ TEST_F(QueryPlannerTest, TagAccordingToCacheFailsOnBadInput) {
     ASSERT_NOT_OK(s);
 
     // Null indexTree.
-    s = QueryPlanner::tagAccordingToCache(scopedCq->root(), nullptr, indexMap);
+    s = QueryPlanner::tagAccordingToCache(scopedCq->getPrimaryMatchExpression(), nullptr, indexMap);
     ASSERT_NOT_OK(s);
 
     // Index not found.
-    s = QueryPlanner::tagAccordingToCache(scopedCq->root(), indexTree.get(), indexMap);
+    s = QueryPlanner::tagAccordingToCache(
+        scopedCq->getPrimaryMatchExpression(), indexTree.get(), indexMap);
     ASSERT_NOT_OK(s);
 
     // Index found once added to the map.
     indexMap[IndexEntry::Identifier{"a_1"}] = 0;
-    s = QueryPlanner::tagAccordingToCache(scopedCq->root(), indexTree.get(), indexMap);
+    s = QueryPlanner::tagAccordingToCache(
+        scopedCq->getPrimaryMatchExpression(), indexTree.get(), indexMap);
     ASSERT_OK(s);
 
     // Regenerate canonical query in order to clear tags.
     auto newQR = std::make_unique<FindCommandRequest>(nss);
     newQR->setFilter(BSON("a" << 3));
-    statusWithCQ = CanonicalQuery::canonicalize(opCtx.get(), std::move(newQR));
-    ASSERT_OK(statusWithCQ.getStatus());
-    scopedCq = std::move(statusWithCQ.getValue());
+    scopedCq = std::make_unique<CanonicalQuery>(CanonicalQueryParams{
+        .expCtx = makeExpressionContext(opCtx.get(), *newQR),
+        .parsedFind = ParsedFindCommandParams{.findCommand = std::move(newQR)}});
 
     // Mismatched tree topology.
     auto child = std::make_unique<PlanCacheIndexTree>();
     child->setIndexEntry(buildSimpleIndexEntry(BSON("a" << 1), "a_1"));
     indexTree->children.push_back(std::move(child));
-    s = QueryPlanner::tagAccordingToCache(scopedCq->root(), indexTree.get(), indexMap);
+    s = QueryPlanner::tagAccordingToCache(
+        scopedCq->getPrimaryMatchExpression(), indexTree.get(), indexMap);
     ASSERT_NOT_OK(s);
 }
 
 TEST_F(QueryPlannerTest, DollarResumeAfterFieldPropagatedFromQueryRequestToStageBuilder) {
-    BSONObj cmdObj = BSON("find" << nss.ns() << "hint" << BSON("$natural" << 1) << "sort"
+    BSONObj cmdObj = BSON("find" << nss.ns_forTest() << "hint" << BSON("$natural" << 1) << "sort"
                                  << BSON("$natural" << 1) << "$_requestResumeToken" << true
                                  << "$_resumeAfter" << BSON("$recordId" << 42LL));
 
@@ -856,22 +887,7 @@ TEST_F(QueryPlannerTest, DollarResumeAfterFieldPropagatedFromQueryRequestToStage
 
     const auto* node = solns.front()->root();
     const CollectionScanNode* csn = static_cast<const CollectionScanNode*>(node);
-    ASSERT_EQUALS(RecordId(42LL), csn->resumeScanPoint->recordId);
-    ASSERT_FALSE(csn->resumeScanPoint->tolerateKeyNotFound);
-}
-
-TEST_F(QueryPlannerTest, DollarStartAtFieldPropagatedFromQueryRequestToStageBuilder) {
-    BSONObj cmdObj = BSON("find" << nss.ns() << "hint" << BSON("$natural" << 1) << "sort"
-                                 << BSON("$natural" << 1) << "$_requestResumeToken" << true
-                                 << "$_startAt" << BSON("$recordId" << 42LL));
-
-    runQueryAsCommand(cmdObj);
-    assertHasOnlyCollscan();
-
-    const auto* node = solns.front()->root();
-    const CollectionScanNode* csn = static_cast<const CollectionScanNode*>(node);
-    ASSERT_EQUALS(RecordId(42LL), csn->resumeScanPoint->recordId);
-    ASSERT_TRUE(csn->resumeScanPoint->tolerateKeyNotFound);
+    ASSERT_EQUALS(RecordId(42LL), csn->resumeAfterRecordId.value());
 }
 
 TEST_F(QueryPlannerTest, PreserveRecordIdOptionPrecludesSimpleSort) {
