@@ -28,37 +28,54 @@
  */
 
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/query/query_planner_test_fixture.h"
-
 #include <algorithm>
+#include <boost/container/vector.hpp>
+#include <boost/cstdint.hpp>
+#include <boost/move/utility_core.hpp>
+#include <cstdint>
 
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status_with.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/json.h"
+#include "mongo/db/field_ref.h"
+#include "mongo/db/index/index_descriptor.h"
+#include "mongo/db/index_names.h"
 #include "mongo/db/matcher/expression_parser.h"
 #include "mongo/db/matcher/extensions_callback_noop.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/pipeline/expression_context_for_test.h"
+#include "mongo/db/query/canonical_query.h"
+#include "mongo/db/query/find_command.h"
+#include "mongo/db/query/projection_policies.h"
 #include "mongo/db/query/query_knobs_gen.h"
 #include "mongo/db/query/query_planner.h"
+#include "mongo/db/query/query_planner_test_fixture.h"
 #include "mongo/db/query/query_planner_test_lib.h"
+#include "mongo/db/query/query_request_helper.h"
 #include "mongo/logv2/log.h"
-#include "mongo/util/transitional_tools_do_not_use/vector_spooling.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/rpc/op_msg.h"
+#include "mongo/unittest/assert.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
 
 namespace mongo {
 
-using unittest::assertGet;
-
-const NamespaceString QueryPlannerTest::nss("test.collection");
 
 void QueryPlannerTest::setUp() {
+    nss = NamespaceString::createNamespaceString_forTest("test.collection");
     opCtx = serviceContext.makeOperationContext();
     expCtx = make_intrusive<ExpressionContext>(
         opCtx.get(), std::unique_ptr<CollatorInterface>(nullptr), nss);
     internalQueryPlannerEnableHashIntersection.store(true);
-    params.options = QueryPlannerParams::INCLUDE_COLLSCAN;
+    params.mainCollectionInfo.options = QueryPlannerParams::INCLUDE_COLLSCAN;
     addIndex(BSON("_id" << 1));
 }
 
@@ -71,35 +88,39 @@ void QueryPlannerTest::clearState() {
 }
 
 void QueryPlannerTest::addIndex(BSONObj keyPattern, bool multikey) {
-    params.indices.push_back({keyPattern,
-                              IndexNames::nameToType(IndexNames::findPluginName(keyPattern)),
-                              IndexDescriptor::kLatestIndexVersion,
-                              multikey,
-                              {},
-                              {},
-                              false,  // sparse
-                              false,  // unique
-                              IndexEntry::Identifier{"hari_king_of_the_stove"},
-                              nullptr,  // filterExpr
-                              BSONObj(),
-                              nullptr,
-                              nullptr});
+    params.mainCollectionInfo.indexes.push_back(
+        {keyPattern,
+         IndexNames::nameToType(IndexNames::findPluginName(keyPattern)),
+         IndexDescriptor::kLatestIndexVersion,
+         multikey,
+         {},
+         {},
+         false,  // sparse
+         false,  // unique
+         // Add the position to the name so we have a unique set of index names.
+         IndexEntry::Identifier{"hari_king_of_the_stove" +
+                                std::to_string(params.mainCollectionInfo.indexes.size())},
+         nullptr,  // filterExpr
+         BSONObj(),
+         nullptr,
+         nullptr});
 }
 
 void QueryPlannerTest::addIndex(BSONObj keyPattern, bool multikey, bool sparse) {
-    params.indices.push_back({keyPattern,
-                              IndexNames::nameToType(IndexNames::findPluginName(keyPattern)),
-                              IndexDescriptor::kLatestIndexVersion,
-                              multikey,
-                              {},
-                              {},
-                              sparse,
-                              false,  // unique
-                              IndexEntry::Identifier{"note_to_self_dont_break_build"},
-                              nullptr,  // filterExpr
-                              BSONObj(),
-                              nullptr,
-                              nullptr});
+    params.mainCollectionInfo.indexes.push_back(
+        {keyPattern,
+         IndexNames::nameToType(IndexNames::findPluginName(keyPattern)),
+         IndexDescriptor::kLatestIndexVersion,
+         multikey,
+         {},
+         {},
+         sparse,
+         false,  // unique
+         IndexEntry::Identifier{"note_to_self_dont_break_build"},
+         nullptr,  // filterExpr
+         BSONObj(),
+         nullptr,
+         nullptr});
 }
 
 void QueryPlannerTest::addIndex(BSONObj keyPattern, bool multikey, bool sparse, bool unique) {
@@ -109,51 +130,54 @@ void QueryPlannerTest::addIndex(BSONObj keyPattern, bool multikey, bool sparse,
 
 void QueryPlannerTest::addIndex(
     BSONObj keyPattern, bool multikey, bool sparse, bool unique, const std::string& name) {
-    params.indices.push_back({keyPattern,
-                              IndexNames::nameToType(IndexNames::findPluginName(keyPattern)),
-                              IndexDescriptor::kLatestIndexVersion,
-                              multikey,
-                              {},
-                              {},
-                              sparse,
-                              unique,
-                              IndexEntry::Identifier{name},
-                              nullptr,  // filterExpr
-                              BSONObj(),
-                              nullptr,
-                              nullptr});
+    params.mainCollectionInfo.indexes.push_back(
+        {keyPattern,
+         IndexNames::nameToType(IndexNames::findPluginName(keyPattern)),
+         IndexDescriptor::kLatestIndexVersion,
+         multikey,
+         {},
+         {},
+         sparse,
+         unique,
+         IndexEntry::Identifier{name},
+         nullptr,  // filterExpr
+         BSONObj(),
+         nullptr,
+         nullptr});
 }
 
 void QueryPlannerTest::addIndex(BSONObj keyPattern, BSONObj infoObj) {
-    params.indices.push_back({keyPattern,
-                              IndexNames::nameToType(IndexNames::findPluginName(keyPattern)),
-                              IndexDescriptor::kLatestIndexVersion,
-                              false,  // multikey
-                              {},
-                              {},
-                              false,  // sparse
-                              false,  // unique
-                              IndexEntry::Identifier{"foo"},
-                              nullptr,  // filterExpr
-                              infoObj,
-                              nullptr,
-                              nullptr});
+    params.mainCollectionInfo.indexes.push_back(
+        {keyPattern,
+         IndexNames::nameToType(IndexNames::findPluginName(keyPattern)),
+         IndexDescriptor::kLatestIndexVersion,
+         false,  // multikey
+         {},
+         {},
+         false,  // sparse
+         false,  // unique
+         IndexEntry::Identifier{"foo"},
+         nullptr,  // filterExpr
+         infoObj,
+         nullptr,
+         nullptr});
 }
 
 void QueryPlannerTest::addIndex(BSONObj keyPattern, MatchExpression* filterExpr) {
-    params.indices.push_back({keyPattern,
-                              IndexNames::nameToType(IndexNames::findPluginName(keyPattern)),
-                              IndexDescriptor::kLatestIndexVersion,
-                              false,  // multikey
-                              {},
-                              {},
-                              false,  // sparse
-                              false,  // unique
-                              IndexEntry::Identifier{"foo"},
-                              filterExpr,
-                              BSONObj(),
-                              nullptr,
-                              nullptr});
+    params.mainCollectionInfo.indexes.push_back(
+        {keyPattern,
+         IndexNames::nameToType(IndexNames::findPluginName(keyPattern)),
+         IndexDescriptor::kLatestIndexVersion,
+         false,  // multikey
+         {},
+         {},
+         false,  // sparse
+         false,  // unique
+         IndexEntry::Identifier{"foo"},
+         filterExpr,
+         BSONObj(),
+         nullptr,
+         nullptr});
 }
 
 void QueryPlannerTest::addIndex(BSONObj keyPattern, const MultikeyPaths& multikeyPaths) {
@@ -183,7 +207,7 @@ void QueryPlannerTest::addIndex(BSONObj keyPattern, const MultikeyPaths& multike
                      nullptr,
                      nullptr);
     entry.multikeyPaths = multikeyPaths;
-    params.indices.push_back(entry);
+    params.mainCollectionInfo.indexes.push_back(entry);
 }
 
 void QueryPlannerTest::addIndex(BSONObj keyPattern, const CollatorInterface* collator) {
@@ -208,7 +232,7 @@ void QueryPlannerTest::addIndex(BSONObj keyPattern, const CollatorInterface* col
                      nullptr,
                      nullptr);
     entry.collator = collator;
-    params.indices.push_back(entry);
+    params.mainCollectionInfo.indexes.push_back(entry);
 }
 
 void QueryPlannerTest::addIndex(BSONObj keyPattern,
@@ -235,7 +259,7 @@ void QueryPlannerTest::addIndex(BSONObj keyPattern,
                      nullptr,
                      nullptr);
     entry.collator = collator;
-    params.indices.push_back(entry);
+    params.mainCollectionInfo.indexes.push_back(entry);
 }
 
 void QueryPlannerTest::addIndex(BSONObj keyPattern,
@@ -261,11 +285,11 @@ void QueryPlannerTest::addIndex(BSONObj keyPattern,
                      nullptr,
                      nullptr);
     entry.collator = collator;
-    params.indices.push_back(entry);
+    params.mainCollectionInfo.indexes.push_back(entry);
 }
 
 void QueryPlannerTest::addIndex(const IndexEntry& ie) {
-    params.indices.push_back(ie);
+    params.mainCollectionInfo.indexes.push_back(ie);
 }
 
 void QueryPlannerTest::runQuery(BSONObj query) {
@@ -275,7 +299,7 @@ void QueryPlannerTest::runQuery(BSONObj query) {
 void QueryPlannerTest::runQueryWithPipeline(
     BSONObj query,
     BSONObj proj,
-    std::vector<std::unique_ptr<InnerPipelineStageInterface>> queryLayerPipeline) {
+    std::vector<boost::intrusive_ptr<DocumentSource>> queryLayerPipeline) {
     runQueryFull(query,
                  BSONObj(),
                  proj,
@@ -331,16 +355,15 @@ void QueryPlannerTest::runQuerySortProjSkipLimitHint(const BSONObj& query,
     runQueryFull(query, sort, proj, skip, limit, hint, BSONObj(), BSONObj());
 }
 
-void QueryPlannerTest::runQueryFull(
-    const BSONObj& query,
-    const BSONObj& sort,
-    const BSONObj& proj,
-    long long skip,
-    long long limit,
-    const BSONObj& hint,
-    const BSONObj& minObj,
-    const BSONObj& maxObj,
-    std::vector<std::unique_ptr<InnerPipelineStageInterface>> pipeline) {
+void QueryPlannerTest::runQueryFull(const BSONObj& query,
+                                    const BSONObj& sort,
+                                    const BSONObj& proj,
+                                    long long skip,
+                                    long long limit,
+                                    const BSONObj& hint,
+                                    const BSONObj& minObj,
+                                    const BSONObj& maxObj,
+                                    std::vector<boost::intrusive_ptr<DocumentSource>> pipeline) {
     clearState();
 
     auto findCommand = std::make_unique<FindCommandRequest>(nss);
@@ -356,22 +379,14 @@ void QueryPlannerTest::runQueryFull(
     findCommand->setHint(hint);
     findCommand->setMin(minObj);
     findCommand->setMax(maxObj);
-    auto statusWithCQ =
-        CanonicalQuery::canonicalize(opCtx.get(),
-                                     std::move(findCommand),
-                                     false,
-                                     expCtx,
-                                     ExtensionsCallbackNoop(),
-                                     MatchExpressionParser::kAllowAllSpecialFeatures,
-                                     ProjectionPolicies::findProjectionPolicies(),
-                                     std::move(pipeline),
-                                     isCountLike);
-    ASSERT_OK(statusWithCQ.getStatus());
-    cq = std::move(statusWithCQ.getValue());
+    cq = std::make_unique<CanonicalQuery>(CanonicalQueryParams{
+        .expCtx = makeExpressionContext(opCtx.get(), *findCommand),
+        .parsedFind = ParsedFindCommandParams{.findCommand = std::move(findCommand),
+                                              .allowedFeatures =
+                                                  MatchExpressionParser::kAllowAllSpecialFeatures},
+        .pipeline = std::move(pipeline),
+        .isCountLike = isCountLike});
     cq->setSbeCompatible(markQueriesSbeCompatible);
-    if (markQueriesSbeCompatible) {
-        cq->parameterize();
-    }
     cq->setForceGenerateRecordId(forceRecordId);
 
     auto statusWithMultiPlanSolns = QueryPlanner::plan(*cq, params);
@@ -440,18 +455,12 @@ void QueryPlannerTest::runInvalidQueryFull(const BSONObj& query,
     findCommand->setHint(hint);
     findCommand->setMin(minObj);
     findCommand->setMax(maxObj);
-    auto statusWithCQ =
-        CanonicalQuery::canonicalize(opCtx.get(),
-                                     std::move(findCommand),
-                                     false,
-                                     expCtx,
-                                     ExtensionsCallbackNoop(),
-                                     MatchExpressionParser::kAllowAllSpecialFeatures,
-                                     ProjectionPolicies::findProjectionPolicies(),
-                                     {},
-                                     isCountLike);
-    ASSERT_OK(statusWithCQ.getStatus());
-    cq = std::move(statusWithCQ.getValue());
+    cq = std::make_unique<CanonicalQuery>(CanonicalQueryParams{
+        .expCtx = makeExpressionContext(opCtx.get(), *findCommand),
+        .parsedFind = ParsedFindCommandParams{.findCommand = std::move(findCommand),
+                                              .allowedFeatures =
+                                                  MatchExpressionParser::kAllowAllSpecialFeatures},
+        .isCountLike = isCountLike});
     cq->setSbeCompatible(markQueriesSbeCompatible);
     cq->setForceGenerateRecordId(forceRecordId);
 
@@ -460,31 +469,23 @@ void QueryPlannerTest::runInvalidQueryFull(const BSONObj& query,
     ASSERT_NOT_OK(plannerStatus);
 }
 
-
 void QueryPlannerTest::runQueryAsCommand(const BSONObj& cmdObj) {
     clearState();
 
     invariant(nss.isValid());
-
-    const bool isExplain = false;
-
     // If there is no '$db', append it.
-    auto cmd = OpMsgRequest::fromDBAndBody(nss.db(), cmdObj).body;
+    auto cmd = OpMsgRequestBuilder::create(
+                   auth::ValidatedTenancyScope::get(opCtx.get()), nss.dbName(), cmdObj)
+                   .body;
     std::unique_ptr<FindCommandRequest> findCommand(
         query_request_helper::makeFromFindCommandForTests(cmd, nss));
 
-    auto statusWithCQ =
-        CanonicalQuery::canonicalize(opCtx.get(),
-                                     std::move(findCommand),
-                                     isExplain,
-                                     expCtx,
-                                     ExtensionsCallbackNoop(),
-                                     MatchExpressionParser::kAllowAllSpecialFeatures,
-                                     ProjectionPolicies::findProjectionPolicies(),
-                                     {},
-                                     isCountLike);
-    ASSERT_OK(statusWithCQ.getStatus());
-    cq = std::move(statusWithCQ.getValue());
+    cq = std::make_unique<CanonicalQuery>(CanonicalQueryParams{
+        .expCtx = makeExpressionContext(opCtx.get(), *findCommand),
+        .parsedFind = ParsedFindCommandParams{.findCommand = std::move(findCommand),
+                                              .allowedFeatures =
+                                                  MatchExpressionParser::kAllowAllSpecialFeatures},
+        .isCountLike = isCountLike});
     cq->setSbeCompatible(markQueriesSbeCompatible);
     cq->setForceGenerateRecordId(forceRecordId);
 
@@ -498,25 +499,19 @@ void QueryPlannerTest::runInvalidQueryAsCommand(const BSONObj& cmdObj) {
 
     invariant(nss.isValid());
 
-    const bool isExplain = false;
-
     // If there is no '$db', append it.
-    auto cmd = OpMsgRequest::fromDBAndBody(nss.db(), cmdObj).body;
+    auto cmd = OpMsgRequestBuilder::create(
+                   auth::ValidatedTenancyScope::get(opCtx.get()), nss.dbName(), cmdObj)
+                   .body;
     std::unique_ptr<FindCommandRequest> findCommand(
         query_request_helper::makeFromFindCommandForTests(cmd, nss));
 
-    auto statusWithCQ =
-        CanonicalQuery::canonicalize(opCtx.get(),
-                                     std::move(findCommand),
-                                     isExplain,
-                                     expCtx,
-                                     ExtensionsCallbackNoop(),
-                                     MatchExpressionParser::kAllowAllSpecialFeatures,
-                                     ProjectionPolicies::findProjectionPolicies(),
-                                     {},
-                                     isCountLike);
-    ASSERT_OK(statusWithCQ.getStatus());
-    cq = std::move(statusWithCQ.getValue());
+    cq = std::make_unique<CanonicalQuery>(CanonicalQueryParams{
+        .expCtx = makeExpressionContext(opCtx.get(), *findCommand),
+        .parsedFind = ParsedFindCommandParams{.findCommand = std::move(findCommand),
+                                              .allowedFeatures =
+                                                  MatchExpressionParser::kAllowAllSpecialFeatures},
+        .isCountLike = isCountLike});
     cq->setSbeCompatible(markQueriesSbeCompatible);
     cq->setForceGenerateRecordId(forceRecordId);
 
@@ -583,6 +578,10 @@ void QueryPlannerTest::assertSolutionExists(const std::string& solnJson, size_t
     FAIL(ss);
 }
 
+void QueryPlannerTest::assertSolutionDoesntExist(const std::string& solnJson) const {
+    assertSolutionExists(solnJson, 0 /* expect zero matches */);
+}
+
 void QueryPlannerTest::assertHasOneSolutionOf(const std::vector<std::string>& solnStrs) const {
     size_t matches = 0;
     for (std::vector<std::string>::const_iterator it = solnStrs.begin(); it != solnStrs.end();
