@@ -34,23 +34,55 @@
 
 #include "mongo/db/query/query_planner_test_lib.h"
 
-#include <ostream>
+#include <cstddef>
+#include <memory>
+#include <set>
+#include <utility>
+#include <vector>
 
+#include <absl/container/flat_hash_map.h>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
 #include "mongo/bson/simple_bsonobj_comparator.h"
-#include "mongo/db/jsobj.h"
-#include "mongo/db/json.h"
+#include "mongo/db/exec/document_value/value.h"
+#include "mongo/db/fts/fts_query.h"
+#include "mongo/db/matcher/expression.h"
 #include "mongo/db/matcher/expression_algo.h"
+#include "mongo/db/matcher/expression_hasher.h"
 #include "mongo/db/matcher/expression_parser.h"
-#include "mongo/db/pipeline/document_source_group.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/pipeline/accumulation_statement.h"
+#include "mongo/db/pipeline/expression.h"
+#include "mongo/db/pipeline/expression_context.h"
 #include "mongo/db/pipeline/expression_context_for_test.h"
+#include "mongo/db/pipeline/field_path.h"
 #include "mongo/db/query/collation/collator_factory_mock.h"
+#include "mongo/db/query/collation/collator_interface.h"
+#include "mongo/db/query/index_entry.h"
+#include "mongo/db/query/interval.h"
+#include "mongo/db/query/projection.h"
+#include "mongo/db/query/projection_ast.h"
 #include "mongo/db/query/projection_ast_util.h"
 #include "mongo/db/query/projection_parser.h"
-#include "mongo/db/query/query_planner.h"
+#include "mongo/db/query/projection_policies.h"
 #include "mongo/db/query/query_solution.h"
+#include "mongo/db/query/stage_types.h"
 #include "mongo/logv2/log.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/stdx/type_traits.h"
+#include "mongo/stdx/unordered_set.h"
 #include "mongo/util/assert_util.h"
+#include "mongo/util/intrusive_counter.h"
+#include "mongo/util/str.h"
+#include "mongo/util/string_map.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
 
@@ -84,13 +116,22 @@ Status filterMatches(const BSONObj& testFilter,
         root = MatchExpression::optimize(std::move(root));
     }
     MatchExpression::sortTree(root.get());
-    if (trueFilterClone->equivalent(root.get())) {
-        return Status::OK();
+    if (!trueFilterClone->equivalent(root.get())) {
+        return {ErrorCodes::Error{5619211},
+                str::stream()
+                    << "Provided filter did not match filter on query solution node. Expected: "
+                    << root->toString() << ". Found: " << trueFilter->toString()};
     }
-    return {
-        ErrorCodes::Error{5619211},
-        str::stream() << "Provided filter did not match filter on query solution node. Expected: "
-                      << root->toString() << ". Found: " << trueFilter->toString()};
+
+    const MatchExpressionHasher hash{};
+    if (hash(trueFilterClone.get()) != hash(root.get())) {
+        return {ErrorCodes::Error{7901821},
+                str::stream() << "Provided filter's hash did not match filter's hash on query "
+                                 "solution node. Expected: "
+                              << root->toString() << ". Found: " << trueFilter->toString()};
+    }
+
+    return Status::OK();
 }
 
 Status nodeHasMatchingFilter(const BSONObj& testFilter,
@@ -960,7 +1001,7 @@ Status QueryPlannerTestLib::solutionMatches(const BSONObj& testSoln,
                     "corresponding 'proj' object in the provided JSON"};
         }
         BSONObj projObj = el.Obj();
-        invariant(bsonObjFieldsAreInSet(projObj, {"type", "spec", "node"}));
+        invariant(bsonObjFieldsAreInSet(projObj, {"type", "spec", "node", "isAddition"}));
 
         BSONElement projType = projObj["type"];
         if (!projType.eoo()) {
@@ -1006,14 +1047,23 @@ Status QueryPlannerTestLib::solutionMatches(const BSONObj& testSoln,
                 "JSON"};
         }
 
+        // Extra flag which can be used to indicate whether the projection is an "addition" (adding
+        // fields) or not (excluding/including fields);
+        BSONElement isAdditionElt = projObj["isAddition"];
+        const bool isAddition = isAdditionElt.trueValue();
+
         // Create an empty/dummy expression context without access to the operation context and
         // collator. This should be sufficient to parse a projection.
-        auto expCtx =
-            make_intrusive<ExpressionContext>(nullptr, nullptr, NamespaceString("test.dummy"));
+        auto expCtx = make_intrusive<ExpressionContext>(
+            nullptr, nullptr, NamespaceString::createNamespaceString_forTest("test.dummy"));
         auto projection = projection_ast::parseAndAnalyze(
-            expCtx, spec.Obj(), ProjectionPolicies::findProjectionPolicies());
+            expCtx,
+            spec.Obj(),
+            isAddition ? ProjectionPolicies::addFieldsProjectionPolicies()
+                       : ProjectionPolicies::findProjectionPolicies());
         auto specProjObj = projection_ast::astToDebugBSON(projection.root());
         auto solnProjObj = projection_ast::astToDebugBSON(pn->proj.root());
+
         if (!SimpleBSONObjComparator::kInstance.evaluate(specProjObj == solnProjObj)) {
             return {ErrorCodes::Error{5619278},
                     str::stream() << "found a projection stage in the solution with "
@@ -1337,6 +1387,16 @@ Status QueryPlannerTestLib::solutionMatches(const BSONObj& testSoln,
             }
         }
 
+        if (auto extraFieldsPermittedElt = obj["extraFieldsPermitted"]) {
+            const bool extraFieldsPermitted = extraFieldsPermittedElt.trueValue();
+            if (actualColumnIxScanNode->extraFieldsPermitted != extraFieldsPermitted) {
+                return {ErrorCodes::Error{5842499},
+                        str::stream() << "Found mismatched extraFieldsPermitted. Expected "
+                                      << extraFieldsPermitted << " Found: "
+                                      << actualColumnIxScanNode->extraFieldsPermitted};
+            }
+        }
+
         if (auto matchFields = obj["matchFields"]) {
             if (auto matchStatus = stringSetsMatch(matchFields,
                                                    actualColumnIxScanNode->matchFields,
@@ -1421,13 +1481,14 @@ Status QueryPlannerTestLib::solutionMatches(const BSONObj& testSoln,
                                   << testSoln.toString()};
         }
 
-        if (expectedForeignCollection.str() != actualEqLookupNode->foreignCollection.toString()) {
+        if (expectedForeignCollection.str() !=
+            actualEqLookupNode->foreignCollection.toString_forTest()) {
             return {
                 ErrorCodes::Error{6267502},
                 str::stream() << "Test solution 'foreignCollection' does not match actual; test "
                                  ""
                               << expectedForeignCollection.str() << " != actual "
-                              << actualEqLookupNode->foreignCollection};
+                              << actualEqLookupNode->foreignCollection.toStringForErrorMsg()};
         }
 
         auto expectedLocalField = expectedEqLookupSoln["joinFieldLocal"];
@@ -1510,6 +1571,14 @@ Status QueryPlannerTestLib::solutionMatches(const BSONObj& testSoln,
         }
         return solutionMatches(child.Obj(), actualEqLookupNode->children[0].get(), relaxBoundsCheck)
             .withContext("mismatch below eq_lookup stage");
+    } else if (STAGE_EOF == trueSoln->getType()) {
+        auto eofElement = testSoln["eof"];
+        if (eofElement.eoo()) {
+            return {ErrorCodes::Error{8186300},
+                    "found an 'eof' object in the test solution but no corresponding "
+                    "'eof' object in the expected JSON"};
+        }
+        return Status::OK();
     }
     return {ErrorCodes::Error{5698301},
             str::stream() << "Unknown query solution node found: " << trueSoln->toString()};
