@@ -27,13 +27,38 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <memory>
+#include <set>
+#include <string>
+#include <utility>
+#include <vector>
 
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/json.h"
+#include "mongo/db/field_ref.h"
+#include "mongo/db/index/index_descriptor.h"
 #include "mongo/db/index/wildcard_key_generator.h"
+#include "mongo/db/index_names.h"
+#include "mongo/db/matcher/expression.h"
+#include "mongo/db/pipeline/dependencies.h"
+#include "mongo/db/query/collation/collator_interface.h"
 #include "mongo/db/query/collation/collator_interface_mock.h"
+#include "mongo/db/query/index_entry.h"
 #include "mongo/db/query/planner_wildcard_helpers.h"
+#include "mongo/db/query/query_knobs_gen.h"
+#include "mongo/db/query/query_planner_params.h"
 #include "mongo/db/query/query_planner_test_fixture.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/unittest/assert.h"
 #include "mongo/unittest/death_test.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/scopeguard.h"
 
 namespace mongo {
 
@@ -52,7 +77,7 @@ protected:
 
         // We're interested in testing plans that use a $** index, so don't generate collection
         // scans.
-        params.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
+        params.mainCollectionInfo.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
     }
 
     void addWildcardIndex(BSONObj keyPattern,
@@ -73,19 +98,19 @@ protected:
 
         _proj = WildcardKeyGenerator::createProjectionExecutor(keyPattern, wildcardProjection);
 
-        params.indices.push_back({std::move(keyPattern),
-                                  IndexType::INDEX_WILDCARD,
-                                  IndexDescriptor::kLatestIndexVersion,
-                                  isMultikey,
-                                  {},  // multikeyPaths
-                                  std::move(multikeyFieldRefs),
-                                  false,  // sparse
-                                  false,  // unique
-                                  IndexEntry::Identifier{indexName},
-                                  partialFilterExpr,
-                                  std::move(infoObj),
-                                  collator,
-                                  _proj.get_ptr()});
+        params.mainCollectionInfo.indexes.push_back({std::move(keyPattern),
+                                                     IndexType::INDEX_WILDCARD,
+                                                     IndexDescriptor::kLatestIndexVersion,
+                                                     isMultikey,
+                                                     {},  // multikeyPaths
+                                                     std::move(multikeyFieldRefs),
+                                                     false,  // sparse
+                                                     false,  // unique
+                                                     IndexEntry::Identifier{indexName},
+                                                     partialFilterExpr,
+                                                     std::move(infoObj),
+                                                     collator,
+                                                     _proj.get_ptr()});
     }
 
     boost::optional<WildcardProjection> _proj;
@@ -99,13 +124,13 @@ DEATH_TEST_F(QueryPlannerWildcardTest,
              CannotExpandPreExpandedWildcardIndexEntry,
              "Tripwire assertion") {
     addWildcardIndex(BSON("$**" << 1));
-    ASSERT_EQ(params.indices.size(), 2U);
+    ASSERT_EQ(params.mainCollectionInfo.indexes.size(), 2U);
 
     // Expand the $** index and add the expanded entry to the list of available indices.
     std::vector<IndexEntry> expandedIndex;
-    wcp::expandWildcardIndexEntry(params.indices.back(), {"a"}, &expandedIndex);
+    wcp::expandWildcardIndexEntry(params.mainCollectionInfo.indexes.back(), {"a"}, &expandedIndex);
     ASSERT_EQ(expandedIndex.size(), 1U);
-    params.indices.push_back(expandedIndex.front());
+    params.mainCollectionInfo.indexes.push_back(expandedIndex.front());
 
     // Now run a query. This will tassert when the planner expands the expanded index.
     runQuery(fromjson("{a: 1}"));
@@ -174,19 +199,6 @@ TEST_F(QueryPlannerWildcardTest, NotEqualsNullInElemMatchQueriesUseWildcardIndex
         "'x': [['MinKey', 'MaxKey',true,true]]}}}}}");
 }
 
-TEST_F(QueryPlannerWildcardTest, NotEqualsNullInElemMatchObjectSparseMultiKeyAboveElemMatch) {
-    addWildcardIndex(BSON("$**" << 1), {"a", "a.b"});
-
-    runQuery(fromjson("{'a.b': {$elemMatch: {'c.d': {$ne: null}}}}"));
-
-    assertNumSolutions(1U);
-    assertSolutionExists(
-        "{fetch: {node: {ixscan: {pattern: {'$_path': 1, 'a.b.c.d': 1},"
-        "bounds: {'$_path': [['a.b.c.d','a.b.c.d',true,true], ['a.b.c.d.','a.b.c.d/',true,false]],"
-        "'a.b.c.d': [['MinKey', 'MaxKey', true, true]]"
-        "}}}}}");
-}
-
 TEST_F(QueryPlannerWildcardTest, NotEqualsNullInElemMatchObjectSparseMultiKeyBelowElemMatch) {
     // "a.b.c" being multikey will prevent us from using the index since $elemMatch doesn't do
     // implicit array traversal.
@@ -624,7 +636,7 @@ TEST_F(QueryPlannerWildcardTest, DottedFieldCovering) {
 }
 
 TEST_F(QueryPlannerWildcardTest, CoveredIxscanForCountOnIndexedPath) {
-    params.options = QueryPlannerParams::DEFAULT;
+    params.mainCollectionInfo.options = QueryPlannerParams::DEFAULT;
     setIsCountLike();
     addWildcardIndex(BSON("$**" << 1));
     runQuery(fromjson("{a: 5}"));
@@ -817,7 +829,7 @@ TEST_F(QueryPlannerWildcardTest, PartialIndexWithExistsTrueFilterCanAnswerExiste
 
 TEST_F(QueryPlannerWildcardTest, WildcardIndexDoesNotParticipateInIndexIntersection) {
     // Enable both AND_SORTED and AND_HASH index intersection for this test.
-    params.options |= QueryPlannerParams::INDEX_INTERSECTION;
+    params.mainCollectionInfo.options |= QueryPlannerParams::INDEX_INTERSECTION;
     internalQueryPlannerEnableHashIntersection.store(true);
 
     // Add two standard single-field indexes.
@@ -1932,7 +1944,7 @@ TEST_F(QueryPlannerWildcardTest,
 }
 
 TEST_F(QueryPlannerWildcardTest, StringComparisonWithEqualCollatorsAndWildcardIndexUsesIndex) {
-    params.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
+    params.mainCollectionInfo.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
 
     CollatorInterfaceMock reverseStringCollator(CollatorInterfaceMock::MockType::kReverseString);
     addWildcardIndex(fromjson("{'$**': 1}"), {}, {}, {}, &reverseStringCollator);
