@@ -27,37 +27,69 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
 #include <algorithm>
-#include <boost/optional.hpp>
+#include <boost/cstdint.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <cstdint>
+#include <memory>
+#include <string>
+#include <utility>
+#include <vector>
 
 #include "mongo/base/error_codes.h"
-#include "mongo/db/catalog/collection_catalog.h"
-#include "mongo/db/catalog/collection_mock.h"
+#include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/json.h"
+#include "mongo/bson/simple_bsonobj_comparator.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/db/basic_types.h"
+#include "mongo/db/basic_types_gen.h"
 #include "mongo/db/cursor_id.h"
-#include "mongo/db/dbmessage.h"
-#include "mongo/db/json.h"
 #include "mongo/db/namespace_string.h"
+#include "mongo/db/pipeline/aggregate_command_gen.h"
 #include "mongo/db/pipeline/aggregation_request_helper.h"
+#include "mongo/db/pipeline/legacy_runtime_constants_gen.h"
 #include "mongo/db/pipeline/query_request_conversion.h"
+#include "mongo/db/query/cursor_response_gen.h"
+#include "mongo/db/query/find_command.h"
+#include "mongo/db/query/find_command_gen.h"
+#include "mongo/db/query/max_time_ms_parser.h"
 #include "mongo/db/query/query_request_helper.h"
 #include "mongo/db/query/query_test_service_context.h"
+#include "mongo/db/query/tailable_mode_gen.h"
+#include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/repl/replication_coordinator_mock.h"
 #include "mongo/db/service_context_test_fixture.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/db/tenant_id.h"
+#include "mongo/idl/idl_parser.h"
+#include "mongo/idl/server_parameter_test_util.h"
+#include "mongo/rpc/op_msg.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/bson_test_util.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/serialization_context.h"
+#include "mongo/util/time_support.h"
+#include "mongo/util/uuid.h"
 
 namespace mongo {
 namespace {
 
 using std::unique_ptr;
-using unittest::assertGet;
 
-static const NamespaceString testns("testdb.testcoll");
+static const NamespaceString testns =
+    NamespaceString::createNamespaceString_forTest("testdb.testcoll");
 
 TEST(QueryRequestTest, NegativeSkip) {
     FindCommandRequest findCommand(testns);
-    ASSERT_THROWS_CODE(findCommand.setSkip(-1), DBException, 51024);
+    ASSERT_THROWS_CODE(findCommand.setSkip(-1), DBException, ErrorCodes::BadValue);
 }
 
 TEST(QueryRequestTest, ZeroSkip) {
@@ -74,7 +106,7 @@ TEST(QueryRequestTest, PositiveSkip) {
 
 TEST(QueryRequestTest, NegativeLimit) {
     FindCommandRequest findCommand(testns);
-    ASSERT_THROWS_CODE(findCommand.setLimit(-1), DBException, 51024);
+    ASSERT_THROWS_CODE(findCommand.setLimit(-1), DBException, ErrorCodes::BadValue);
 }
 
 TEST(QueryRequestTest, ZeroLimit) {
@@ -91,7 +123,7 @@ TEST(QueryRequestTest, PositiveLimit) {
 
 TEST(QueryRequestTest, NegativeBatchSize) {
     FindCommandRequest findCommand(testns);
-    ASSERT_THROWS_CODE(findCommand.setBatchSize(-1), DBException, 51024);
+    ASSERT_THROWS_CODE(findCommand.setBatchSize(-1), DBException, ErrorCodes::BadValue);
 }
 
 TEST(QueryRequestTest, ZeroBatchSize) {
@@ -108,7 +140,7 @@ TEST(QueryRequestTest, PositiveBatchSize) {
 
 TEST(QueryRequestTest, NegativeMaxTimeMS) {
     FindCommandRequest findCommand(testns);
-    ASSERT_THROWS_CODE(findCommand.setMaxTimeMS(-1), DBException, 51024);
+    ASSERT_THROWS_CODE(findCommand.setMaxTimeMS(-1), DBException, ErrorCodes::BadValue);
 }
 
 TEST(QueryRequestTest, ZeroMaxTimeMS) {
@@ -243,21 +275,6 @@ TEST(QueryRequestTest, RequestResumeTokenWithSort) {
     ASSERT_OK(query_request_helper::validateFindCommandRequest(findCommand));
 }
 
-TEST(QueryRequestTest, InvalidResumeAfterStartAtTogether) {
-    QueryTestServiceContext serviceContext;
-    auto uniqueTxn = serviceContext.makeOperationContext();
-    OperationContext* opCtx = uniqueTxn.get();
-    FindCommandRequest findCommand(testns);
-
-    findCommand.setResumeAfter(fromjson("{$recordId: NumberLong(1)}"));
-    findCommand.setStartAt(fromjson("{$recordId: NumberLong(2)}"));
-    findCommand.setRequestResumeToken(true);
-    findCommand.setHint(fromjson("{$natural: 1}"));
-
-    ASSERT_NOT_OK(query_request_helper::validateResumeInput(
-        opCtx, findCommand.getResumeAfter(), findCommand.getStartAt(), false));
-}
-
 TEST(QueryRequestTest, InvalidResumeAfterWrongRecordIdType) {
     QueryTestServiceContext serviceContext;
     auto uniqueTxn = serviceContext.makeOperationContext();
@@ -268,30 +285,13 @@ TEST(QueryRequestTest, InvalidResumeAfterWrongRecordIdType) {
     findCommand.setRequestResumeToken(true);
     // Hint must be explicitly set for the query request to validate.
     findCommand.setHint(fromjson("{$natural: 1}"));
-    ASSERT_NOT_OK(query_request_helper::validateResumeInput(
-        opCtx, findCommand.getResumeAfter(), {} /* startAt */, false /* isClusteredCollection */));
+    ASSERT_NOT_OK(query_request_helper::validateResumeAfter(
+        opCtx, findCommand.getResumeAfter(), false /* isClusteredCollection */));
     resumeAfter = BSON("$recordId" << 1LL);
     findCommand.setResumeAfter(resumeAfter);
     ASSERT_OK(query_request_helper::validateFindCommandRequest(findCommand));
-    ASSERT_OK(query_request_helper::validateResumeInput(
-        opCtx, findCommand.getResumeAfter(), {} /* startAt */, false /* isClusteredCollection */));
-}
-
-TEST(QueryRequestTest, InvalidStartAtWrongRecordIdType) {
-    QueryTestServiceContext serviceContext;
-    auto uniqueTxn = serviceContext.makeOperationContext();
-    OperationContext* opCtx = uniqueTxn.get();
-    FindCommandRequest findCommand(testns);
-    findCommand.setStartAt(fromjson("{$recordId: NumberInt(1)}"));
-    findCommand.setRequestResumeToken(true);
-    // Hint must be explicitly set for the query request to validate.
-    findCommand.setHint(fromjson("{$natural: 1}"));
-    ASSERT_NOT_OK(query_request_helper::validateResumeInput(
-        opCtx, {} /* resumeAfter */, findCommand.getStartAt(), false /* isClusteredCollection */));
-    findCommand.setStartAt(fromjson("{$recordId: NumberLong(1)}"));
-    ASSERT_OK(query_request_helper::validateFindCommandRequest(findCommand));
-    ASSERT_OK(query_request_helper::validateResumeInput(
-        opCtx, {} /* resumeAfter */, findCommand.getStartAt(), false /* isClusteredCollection */));
+    ASSERT_OK(query_request_helper::validateResumeAfter(
+        opCtx, findCommand.getResumeAfter(), false /* isClusteredCollection */));
 }
 
 TEST(QueryRequestTest, InvalidResumeAfterExtraField) {
@@ -304,21 +304,8 @@ TEST(QueryRequestTest, InvalidResumeAfterExtraField) {
     findCommand.setRequestResumeToken(true);
     // Hint must be explicitly set for the query request to validate.
     findCommand.setHint(fromjson("{$natural: 1}"));
-    ASSERT_NOT_OK(query_request_helper::validateResumeInput(
-        opCtx, findCommand.getResumeAfter(), {} /* startAt */, false /* isClusteredCollection */));
-}
-
-TEST(QueryRequestTest, InvalidStartAtExtraField) {
-    QueryTestServiceContext serviceContext;
-    auto uniqueTxn = serviceContext.makeOperationContext();
-    OperationContext* opCtx = uniqueTxn.get();
-    FindCommandRequest findCommand(testns);
-    findCommand.setStartAt(fromjson("{$recordId: NumberLong(1), $extra: 1}"));
-    findCommand.setRequestResumeToken(true);
-    // Hint must be explicitly set for the query request to validate.
-    findCommand.setHint(fromjson("{$natural: 1}"));
-    ASSERT_NOT_OK(query_request_helper::validateResumeInput(
-        opCtx, {} /* resumeAfter */, findCommand.getStartAt(), false /* isClusteredCollection */));
+    ASSERT_NOT_OK(query_request_helper::validateResumeAfter(
+        opCtx, findCommand.getResumeAfter(), false /* isClusteredCollection */));
 }
 
 TEST(QueryRequestTest, ResumeAfterWithHint) {
@@ -333,17 +320,6 @@ TEST(QueryRequestTest, ResumeAfterWithHint) {
     ASSERT_OK(query_request_helper::validateFindCommandRequest(findCommand));
 }
 
-TEST(QueryRequestTest, StartAtWithHint) {
-    FindCommandRequest findCommand(testns);
-    findCommand.setStartAt(fromjson("{$recordId: NumberLong(1)}"));
-    findCommand.setRequestResumeToken(true);
-    ASSERT_NOT_OK(query_request_helper::validateFindCommandRequest(findCommand));
-    findCommand.setHint(fromjson("{a: 1}"));
-    ASSERT_NOT_OK(query_request_helper::validateFindCommandRequest(findCommand));
-    findCommand.setHint(fromjson("{$natural: 1}"));
-    ASSERT_OK(query_request_helper::validateFindCommandRequest(findCommand));
-}
-
 TEST(QueryRequestTest, ResumeAfterWithSort) {
     QueryTestServiceContext serviceContext;
     auto uniqueTxn = serviceContext.makeOperationContext();
@@ -355,25 +331,8 @@ TEST(QueryRequestTest, ResumeAfterWithSort) {
     // Hint must be explicitly set for the query request to validate.
     findCommand.setHint(fromjson("{$natural: 1}"));
     ASSERT_OK(query_request_helper::validateFindCommandRequest(findCommand));
-    ASSERT_OK(query_request_helper::validateResumeInput(
-        opCtx, findCommand.getResumeAfter(), {} /* startAt */, false /* isClusteredCollection */));
-    findCommand.setSort(fromjson("{a: 1}"));
-    ASSERT_NOT_OK(query_request_helper::validateFindCommandRequest(findCommand));
-    findCommand.setSort(fromjson("{$natural: 1}"));
-    ASSERT_OK(query_request_helper::validateFindCommandRequest(findCommand));
-}
-
-TEST(QueryRequestTest, StartAtWithSort) {
-    QueryTestServiceContext serviceContext;
-    auto uniqueTxn = serviceContext.makeOperationContext();
-    OperationContext* opCtx = uniqueTxn.get();
-    FindCommandRequest findCommand(testns);
-    findCommand.setStartAt(fromjson("{$recordId: NumberLong(1)}"));
-    findCommand.setRequestResumeToken(true);
-    findCommand.setHint(fromjson("{$natural: 1}"));
-    ASSERT_OK(query_request_helper::validateFindCommandRequest(findCommand));
-    ASSERT_OK(query_request_helper::validateResumeInput(
-        opCtx, {} /* resumeAfter */, findCommand.getStartAt(), false));
+    ASSERT_OK(query_request_helper::validateResumeAfter(
+        opCtx, findCommand.getResumeAfter(), false /* isClusteredCollection */));
     findCommand.setSort(fromjson("{a: 1}"));
     ASSERT_NOT_OK(query_request_helper::validateFindCommandRequest(findCommand));
     findCommand.setSort(fromjson("{$natural: 1}"));
@@ -392,22 +351,8 @@ TEST(QueryRequestTest, ResumeNoSpecifiedRequestResumeToken) {
     ASSERT_NOT_OK(query_request_helper::validateFindCommandRequest(findCommand));
     findCommand.setRequestResumeToken(true);
     ASSERT_OK(query_request_helper::validateFindCommandRequest(findCommand));
-    ASSERT_OK(query_request_helper::validateResumeInput(
-        opCtx, findCommand.getResumeAfter(), {} /* startAt */, false /* isClusteredCollection */));
-}
-
-TEST(QueryRequestTest, StartAtNoSpecifiedRequestResumeToken) {
-    QueryTestServiceContext serviceContext;
-    auto uniqueTxn = serviceContext.makeOperationContext();
-    OperationContext* opCtx = uniqueTxn.get();
-    FindCommandRequest findCommand(testns);
-    findCommand.setStartAt(fromjson("{$recordId: NumberLong(1)}"));
-    findCommand.setHint(fromjson("{$natural: 1}"));
-    ASSERT_NOT_OK(query_request_helper::validateFindCommandRequest(findCommand));
-    findCommand.setRequestResumeToken(true);
-    ASSERT_OK(query_request_helper::validateFindCommandRequest(findCommand));
-    ASSERT_OK(query_request_helper::validateResumeInput(
-        opCtx, {} /* resumeAfter */, findCommand.getStartAt(), false));
+    ASSERT_OK(query_request_helper::validateResumeAfter(
+        opCtx, findCommand.getResumeAfter(), false /* isClusteredCollection */));
 }
 
 TEST(QueryRequestTest, ExplicitEmptyResumeAfter) {
@@ -422,22 +367,8 @@ TEST(QueryRequestTest, ExplicitEmptyResumeAfter) {
     ASSERT_OK(query_request_helper::validateFindCommandRequest(findCommand));
     findCommand.setRequestResumeToken(true);
     ASSERT_OK(query_request_helper::validateFindCommandRequest(findCommand));
-    ASSERT_OK(query_request_helper::validateResumeInput(
-        opCtx, findCommand.getResumeAfter(), {} /* startAt */, false /* isClusteredCollection */));
-}
-
-TEST(QueryRequestTest, ExplicitEmptyStartAt) {
-    QueryTestServiceContext serviceContext;
-    auto uniqueTxn = serviceContext.makeOperationContext();
-    OperationContext* opCtx = uniqueTxn.get();
-    FindCommandRequest findCommand(NamespaceString::kRsOplogNamespace);
-    findCommand.setHint(fromjson("{$natural: 1}"));
-    findCommand.setStartAt(fromjson("{}"));
-    ASSERT_OK(query_request_helper::validateFindCommandRequest(findCommand));
-    findCommand.setRequestResumeToken(true);
-    ASSERT_OK(query_request_helper::validateFindCommandRequest(findCommand));
-    ASSERT_OK(query_request_helper::validateResumeInput(
-        opCtx, {} /* resumeAfter */, findCommand.getStartAt(), false));
+    ASSERT_OK(query_request_helper::validateResumeAfter(
+        opCtx, findCommand.getResumeAfter(), false /* isClusteredCollection */));
 }
 
 TEST(QueryRequestTest, ResumeAfterMismatchInitialSyncId) {
@@ -458,29 +389,8 @@ TEST(QueryRequestTest, ResumeAfterMismatchInitialSyncId) {
     ASSERT_NOT_OK(query_request_helper::validateFindCommandRequest(findCommand));
     findCommand.setRequestResumeToken(true);
     ASSERT_OK(query_request_helper::validateFindCommandRequest(findCommand));
-    ASSERT_NOT_OK(query_request_helper::validateResumeInput(
-        opCtx, findCommand.getResumeAfter(), {} /* startAt */, false /* isClusteredCollection */));
-}
-
-TEST(QueryRequestTest, StartAtMismatchInitialSyncId) {
-    QueryTestServiceContext serviceContext;
-    auto uniqueTxn = serviceContext.makeOperationContext();
-    OperationContext* opCtx = uniqueTxn.get();
-    auto replCoord =
-        std::make_unique<repl::ReplicationCoordinatorMock>(serviceContext.getServiceContext());
-    repl::ReplicationCoordinator::set(serviceContext.getServiceContext(), std::move(replCoord));
-
-    FindCommandRequest findCommand(testns);
-    BSONObj startAt =
-        BSON("$recordId" << 1LL << "$initialSyncId"
-                         << uassertStatusOK(UUID::parse("12345678-1234-9876-1234-000000000000")));
-    findCommand.setStartAt(startAt);
-    findCommand.setHint(fromjson("{$natural: 1}"));
-    ASSERT_NOT_OK(query_request_helper::validateFindCommandRequest(findCommand));
-    findCommand.setRequestResumeToken(true);
-    ASSERT_OK(query_request_helper::validateFindCommandRequest(findCommand));
-    ASSERT_NOT_OK(query_request_helper::validateResumeInput(
-        opCtx, {} /* resumeAfter */, findCommand.getStartAt(), false));
+    ASSERT_NOT_OK(query_request_helper::validateResumeAfter(
+        opCtx, findCommand.getResumeAfter(), false /* isClusteredCollection */));
 }
 
 //
@@ -576,6 +486,7 @@ TEST(QueryRequestTest, ParseFromCommandAllFlagsTrue) {
         "awaitData: true,"
         "allowPartialResults: true,"
         "readOnce: true,"
+        "includeQueryStatsMetrics: true,"
         "allowSpeculativeMajorityRead: true, '$db': 'test'}");
 
     unique_ptr<FindCommandRequest> findCommand(
@@ -587,6 +498,7 @@ TEST(QueryRequestTest, ParseFromCommandAllFlagsTrue) {
     ASSERT(findCommand->getTailable() && findCommand->getAwaitData());
     ASSERT(findCommand->getAllowPartialResults());
     ASSERT(findCommand->getReadOnce());
+    ASSERT(findCommand->getIncludeQueryStatsMetrics());
     ASSERT(findCommand->getAllowSpeculativeMajorityRead());
 }
 
@@ -614,6 +526,14 @@ TEST(QueryRequestTest, ParseFromCommandReadOnceDefaultsToFalse) {
     ASSERT(!findCommand->getReadOnce());
 }
 
+TEST(QueryRequestTest, ParseFromCommandIncludeQueryStatsMetricsDefaultsToFalse) {
+    BSONObj cmdObj = fromjson("{find: 'testns', '$db': 'test'}");
+
+    unique_ptr<FindCommandRequest> findCommand(
+        query_request_helper::makeFromFindCommandForTests(cmdObj));
+    ASSERT(!findCommand->getIncludeQueryStatsMetrics());
+}
+
 TEST(QueryRequestTest, ParseFromCommandValidMinMax) {
     BSONObj cmdObj = fromjson(
         "{find: 'testns',"
@@ -808,7 +728,7 @@ TEST(QueryRequestTest, ParseFromCommandMaxTimeMSWrongType) {
 
     ASSERT_THROWS_CODE(query_request_helper::makeFromFindCommandForTests(cmdObj),
                        DBException,
-                       ErrorCodes::BadValue);
+                       ErrorCodes::TypeMismatch);
 }
 
 
@@ -874,8 +794,9 @@ TEST(QueryRequestTest, ParseFromCommandSlaveOkWrongType) {
         "filter:  {a: 1},"
         "slaveOk: 3, '$db': 'test'}");
 
-    ASSERT_THROWS_CODE(
-        query_request_helper::makeFromFindCommandForTests(cmdObj), DBException, 40415);
+    ASSERT_THROWS_CODE(query_request_helper::makeFromFindCommandForTests(cmdObj),
+                       DBException,
+                       ErrorCodes::IDLUnknownField);
 }
 
 TEST(QueryRequestTest, ParseFromCommandOplogReplayWrongType) {
@@ -919,8 +840,9 @@ TEST(QueryRequestTest, ParseFromCommandExhaustWrongType) {
         "filter:  {a: 1},"
         "exhaust: 3, '$db': 'test'}");
 
-    ASSERT_THROWS_CODE(
-        query_request_helper::makeFromFindCommandForTests(cmdObj), DBException, 40415);
+    ASSERT_THROWS_CODE(query_request_helper::makeFromFindCommandForTests(cmdObj),
+                       DBException,
+                       ErrorCodes::IDLUnknownField);
 }
 
 
@@ -995,6 +917,14 @@ TEST(QueryRequestTest, ParseFromCommandLegacyRuntimeConstantsSubfieldsWrongType)
                        ErrorCodes::TypeMismatch);
 }
 
+TEST(QueryRequestTest, ParseFromCommandIncludeQueryStatsMetricsWrongType) {
+    BSONObj cmdObj = fromjson("{find: 'testns', '$db': 'test', 'includeQueryStatsMetrics': 42}");
+
+    ASSERT_THROWS_CODE(query_request_helper::makeFromFindCommandForTests(cmdObj),
+                       DBException,
+                       ErrorCodes::TypeMismatch);
+}
+
 //
 // Parsing errors where a field has the right type but a bad value.
 //
@@ -1004,8 +934,9 @@ TEST(QueryRequestTest, ParseFromCommandNegativeSkipError) {
         "{find: 'testns',"
         "skip: -3,"
         "filter: {a: 3}, '$db': 'test'}");
-    ASSERT_THROWS_CODE(
-        query_request_helper::makeFromFindCommandForTests(cmdObj), DBException, 51024);
+    ASSERT_THROWS_CODE(query_request_helper::makeFromFindCommandForTests(cmdObj),
+                       DBException,
+                       ErrorCodes::BadValue);
 }
 
 TEST(QueryRequestTest, ParseFromCommandSkipIsZero) {
@@ -1024,8 +955,9 @@ TEST(QueryRequestTest, ParseFromCommandNegativeLimitError) {
         "{find: 'testns',"
         "limit: -3,"
         "filter: {a: 3}, '$db': 'test'}");
-    ASSERT_THROWS_CODE(
-        query_request_helper::makeFromFindCommandForTests(cmdObj), DBException, 51024);
+    ASSERT_THROWS_CODE(query_request_helper::makeFromFindCommandForTests(cmdObj),
+                       DBException,
+                       ErrorCodes::BadValue);
 }
 
 TEST(QueryRequestTest, ParseFromCommandLimitIsZero) {
@@ -1044,8 +976,9 @@ TEST(QueryRequestTest, ParseFromCommandNegativeBatchSizeError) {
         "{find: 'testns',"
         "batchSize: -10,"
         "filter: {a: 3}, '$db': 'test'}");
-    ASSERT_THROWS_CODE(
-        query_request_helper::makeFromFindCommandForTests(cmdObj), DBException, 51024);
+    ASSERT_THROWS_CODE(query_request_helper::makeFromFindCommandForTests(cmdObj),
+                       DBException,
+                       ErrorCodes::BadValue);
 }
 
 TEST(QueryRequestTest, ParseFromCommandBatchSizeZero) {
@@ -1161,8 +1094,9 @@ TEST(QueryRequestTest, AsFindCommandRequestWithUuidAllNonOptionFields) {
 TEST(QueryRequestTest, AsFindCommandRequestWithUuidNoAvailableNamespace) {
     BSONObj cmdObj =
         fromjson("{find: { \"$binary\" : \"ASNFZ4mrze/ty6mHZUMhAQ==\", \"$type\" : \"04\" }}");
-    FindCommandRequest findCommand(NamespaceStringOrUUID(
-        "test", UUID::parse("01234567-89ab-cdef-edcb-a98765432101").getValue()));
+    FindCommandRequest findCommand(
+        NamespaceStringOrUUID(DatabaseName::createDatabaseName_forTest(boost::none, "test"),
+                              UUID::parse("01234567-89ab-cdef-edcb-a98765432101").getValue()));
     ASSERT_BSONOBJ_EQ(cmdObj.removeField("$db"), findCommand.toBSON(BSONObj()));
 }
 
@@ -1242,8 +1176,9 @@ TEST(QueryRequestTest, ParseCommandAllowMetaSortOnFieldWithoutMetaProject) {
 TEST(QueryRequestTest, ParseCommandForbidExhaust) {
     BSONObj cmdObj = fromjson("{find: 'testns', exhaust: true, '$db': 'test'}");
 
-    ASSERT_THROWS_CODE(
-        query_request_helper::makeFromFindCommandForTests(cmdObj), DBException, 40415);
+    ASSERT_THROWS_CODE(query_request_helper::makeFromFindCommandForTests(cmdObj),
+                       DBException,
+                       ErrorCodes::IDLUnknownField);
 }
 
 // Older versions of the server supported an "ntoreturn" parameter to the find command, which was
@@ -1254,7 +1189,7 @@ TEST(QueryRequestTest, NToReturnParamFailsToParse) {
     BSONObj cmdObj = fromjson("{find: 'testns', '$db': 'test', ntoreturn: 5}");
     ASSERT_THROWS_CODE_AND_WHAT(query_request_helper::makeFromFindCommandForTests(cmdObj),
                                 DBException,
-                                40415,
+                                ErrorCodes::IDLUnknownField,
                                 "BSON field 'FindCommandRequest.ntoreturn' is an unknown field.");
 }
 
@@ -1269,7 +1204,7 @@ TEST(QueryRequestTest, ParseCommandFirstFieldNotString) {
     BSONObj cmdObj = fromjson("{find: 1, '$db': 'test'}");
     ASSERT_THROWS_CODE(query_request_helper::makeFromFindCommandForTests(cmdObj),
                        DBException,
-                       ErrorCodes::BadValue);
+                       ErrorCodes::TypeMismatch);
 }
 
 TEST(QueryRequestTest, ParseCommandIgnoreShardVersionField) {
@@ -1323,8 +1258,9 @@ TEST(QueryRequestTest, ParseFromCommandForbidExtraField) {
         "{find: 'testns',"
         "foo: {a: 1}, '$db': 'test'}");
 
-    ASSERT_THROWS_CODE(
-        query_request_helper::makeFromFindCommandForTests(cmdObj), DBException, 40415);
+    ASSERT_THROWS_CODE(query_request_helper::makeFromFindCommandForTests(cmdObj),
+                       DBException,
+                       ErrorCodes::IDLUnknownField);
 }
 
 TEST(QueryRequestTest, ParseFromCommandForbidExtraOption) {
@@ -1332,80 +1268,73 @@ TEST(QueryRequestTest, ParseFromCommandForbidExtraOption) {
         "{find: 'testns',"
         "foo: true, '$db': 'test'}");
 
-    ASSERT_THROWS_CODE(
-        query_request_helper::makeFromFindCommandForTests(cmdObj), DBException, 40415);
+    ASSERT_THROWS_CODE(query_request_helper::makeFromFindCommandForTests(cmdObj),
+                       DBException,
+                       ErrorCodes::IDLUnknownField);
 }
 
 TEST(QueryRequestTest, ParseMaxTimeMSStringValueFails) {
     BSONObj maxTimeObj = BSON(query_request_helper::cmdOptionMaxTimeMS << "foo");
-    ASSERT_THROWS_CODE(parseMaxTimeMSForIDL(maxTimeObj[query_request_helper::cmdOptionMaxTimeMS]),
-                       DBException,
-                       ErrorCodes::BadValue);
+    ASSERT_EQ(parseMaxTimeMS(maxTimeObj[query_request_helper::cmdOptionMaxTimeMS]).getStatus(),
+              ErrorCodes::BadValue);
 }
 
 TEST(QueryRequestTest, ParseMaxTimeMSBoolValueFails) {
     BSONObj maxTimeObj = BSON(query_request_helper::cmdOptionMaxTimeMS << true);
-    ASSERT_THROWS_CODE(parseMaxTimeMSForIDL(maxTimeObj[query_request_helper::cmdOptionMaxTimeMS]),
-                       DBException,
-                       ErrorCodes::BadValue);
+    ASSERT_EQ(parseMaxTimeMS(maxTimeObj[query_request_helper::cmdOptionMaxTimeMS]).getStatus(),
+              ErrorCodes::BadValue);
 }
 
 TEST(QueryRequestTest, ParseMaxTimeMSNullValueFails) {
     BSONObj maxTimeObj = BSON(query_request_helper::cmdOptionMaxTimeMS << BSONNULL);
-    ASSERT_THROWS_CODE(parseMaxTimeMSForIDL(maxTimeObj[query_request_helper::cmdOptionMaxTimeMS]),
-                       DBException,
-                       ErrorCodes::BadValue);
+    ASSERT_EQ(parseMaxTimeMS(maxTimeObj[query_request_helper::cmdOptionMaxTimeMS]).getStatus(),
+              ErrorCodes::BadValue);
 }
 
 TEST(QueryRequestTest, ParseMaxTimeMSUndefinedValueFails) {
     BSONObj maxTimeObj = BSON(query_request_helper::cmdOptionMaxTimeMS << BSONUndefined);
-    ASSERT_THROWS_CODE(parseMaxTimeMSForIDL(maxTimeObj[query_request_helper::cmdOptionMaxTimeMS]),
-                       DBException,
-                       ErrorCodes::BadValue);
+    ASSERT_EQ(parseMaxTimeMS(maxTimeObj[query_request_helper::cmdOptionMaxTimeMS]).getStatus(),
+              ErrorCodes::BadValue);
 }
 
 TEST(QueryRequestTest, ParseMaxTimeMSEmptyObjectValueFails) {
     const auto emptyObj = BSONObjBuilder().obj();
     BSONObj maxTimeObj = BSON(query_request_helper::cmdOptionMaxTimeMS << emptyObj);
-    ASSERT_THROWS_CODE(parseMaxTimeMSForIDL(maxTimeObj[query_request_helper::cmdOptionMaxTimeMS]),
-                       DBException,
-                       ErrorCodes::BadValue);
+    ASSERT_EQ(parseMaxTimeMS(maxTimeObj[query_request_helper::cmdOptionMaxTimeMS]).getStatus(),
+              ErrorCodes::BadValue);
 }
 
 TEST(QueryRequestTest, ParseMaxTimeMSNonIntegralValueFails) {
     BSONObj maxTimeObj = BSON(query_request_helper::cmdOptionMaxTimeMS << 100.3);
-    ASSERT_THROWS_CODE(parseMaxTimeMSForIDL(maxTimeObj[query_request_helper::cmdOptionMaxTimeMS]),
-                       DBException,
-                       ErrorCodes::BadValue);
+    ASSERT_EQ(parseMaxTimeMS(maxTimeObj[query_request_helper::cmdOptionMaxTimeMS]).getStatus(),
+              ErrorCodes::BadValue);
 }
 
 TEST(QueryRequestTest, ParseMaxTimeMSOutOfRangeDoubleFails) {
     BSONObj maxTimeObj = BSON(query_request_helper::cmdOptionMaxTimeMS << 1e200);
-    ASSERT_THROWS_CODE(parseMaxTimeMSForIDL(maxTimeObj[query_request_helper::cmdOptionMaxTimeMS]),
-                       DBException,
-                       ErrorCodes::BadValue);
+    ASSERT_EQ(parseMaxTimeMS(maxTimeObj[query_request_helper::cmdOptionMaxTimeMS]).getStatus(),
+              ErrorCodes::BadValue);
 }
 
 TEST(QueryRequestTest, ParseMaxTimeMSNegativeValueFails) {
     BSONObj maxTimeObj = BSON(query_request_helper::cmdOptionMaxTimeMS << -400);
-    ASSERT_THROWS_CODE(parseMaxTimeMSForIDL(maxTimeObj[query_request_helper::cmdOptionMaxTimeMS]),
-                       DBException,
-                       ErrorCodes::BadValue);
+    ASSERT_EQ(parseMaxTimeMS(maxTimeObj[query_request_helper::cmdOptionMaxTimeMS]).getStatus(),
+              ErrorCodes::BadValue);
 }
 
 TEST(QueryRequestTest, ParseMaxTimeMSZeroSucceeds) {
     BSONObj maxTimeObj = BSON(query_request_helper::cmdOptionMaxTimeMS << 0);
-    ASSERT_EQ(parseMaxTimeMSForIDL(maxTimeObj[query_request_helper::cmdOptionMaxTimeMS]), 0);
+    ASSERT_EQ(parseMaxTimeMS(maxTimeObj[query_request_helper::cmdOptionMaxTimeMS]), 0);
 }
 
 TEST(QueryRequestTest, ParseMaxTimeMSEmptyElementSucceeds) {
     const auto emptyElem = BSONElement();
-    ASSERT_EQ(parseMaxTimeMSForIDL(emptyElem), 0);
+    ASSERT_EQ(parseMaxTimeMS(emptyElem), 0);
 }
 
 TEST(QueryRequestTest, ParseMaxTimeMSPositiveInRangeSucceeds) {
     BSONObj maxTimeObj = BSON(query_request_helper::cmdOptionMaxTimeMS << 300);
-    ASSERT_EQ(parseMaxTimeMSForIDL(maxTimeObj[query_request_helper::cmdOptionMaxTimeMS]), 300);
+    ASSERT_EQ(parseMaxTimeMS(maxTimeObj[query_request_helper::cmdOptionMaxTimeMS]), 300);
 }
 
 TEST(QueryRequestTest, ConvertToAggregationSucceeds) {
@@ -1528,15 +1457,21 @@ TEST(QueryRequestTest, ConvertToAggregationWithAllowPartialResultsFails) {
                        ErrorCodes::InvalidPipelineOperator);
 }
 
-TEST(QueryRequestTest, ConvertToAggregationWithRequestResumeTokenFails) {
+TEST(QueryRequestTest, ConvertToAggregationWithRequestResumeTokenSucceeds) {
+    RAIIServerParameterControllerForTest featureFlagController("featureFlagReshardingImprovements",
+                                                               true);
     FindCommandRequest findCommand(testns);
     findCommand.setRequestResumeToken(true);
-    ASSERT_THROWS_CODE(query_request_conversion::asAggregateCommandRequest(findCommand),
-                       DBException,
-                       ErrorCodes::InvalidPipelineOperator);
+    findCommand.setHint(BSON("$natural" << 1));
+
+    auto ar = query_request_conversion::asAggregateCommandRequest(findCommand);
+
+    ASSERT(ar.getRequestResumeToken());
 }
 
 TEST(QueryRequestTest, ConvertToAggregationWithResumeAfterFails) {
+    RAIIServerParameterControllerForTest featureFlagController("featureFlagReshardingImprovements",
+                                                               false);
     FindCommandRequest findCommand(testns);
     BSONObj resumeAfter = BSON("$recordId" << 1LL);
     findCommand.setResumeAfter(resumeAfter);
@@ -1659,6 +1594,22 @@ TEST(QueryRequestTest, ConvertToAggregationWithAllowDiskUseFalseSucceeds) {
     ASSERT_EQ(false, ar.getAllowDiskUse());
 }
 
+TEST(QueryRequestTest, ConvertToAggregationWithIncludeQueryStatsMetricsTrueSucceeds) {
+    FindCommandRequest findCommand(testns);
+    findCommand.setIncludeQueryStatsMetrics(true);
+    auto ar = query_request_conversion::asAggregateCommandRequest(findCommand);
+
+    ASSERT_TRUE(ar.getIncludeQueryStatsMetrics());
+}
+
+TEST(QueryRequestTest, ConvertToAggregationWithIncludeQueryStatsMetricsFalseSucceeds) {
+    FindCommandRequest findCommand(testns);
+    findCommand.setIncludeQueryStatsMetrics(false);
+    auto ar = query_request_conversion::asAggregateCommandRequest(findCommand);
+
+    ASSERT_FALSE(ar.getIncludeQueryStatsMetrics());
+}
+
 TEST(QueryRequestTest, ConvertToFindWithAllowDiskUseTrueSucceeds) {
     FindCommandRequest findCommand(testns);
     findCommand.setAllowDiskUse(true);
@@ -1679,8 +1630,14 @@ TEST(QueryRequestTest, ConvertToFindWithAllowDiskUseFalseSucceeds) {
 
 TEST(QueryRequestHelperTest, ValidateResponseMissingFields) {
     BSONObjBuilder builder;
+    QueryTestServiceContext serviceContext;
+    auto uniqueTxn = serviceContext.makeOperationContext();
+    OperationContext* opCtx = uniqueTxn.get();
     ASSERT_THROWS_CODE(
-        query_request_helper::validateCursorResponse(builder.asTempObj(), boost::none),
+        query_request_helper::validateCursorResponse(builder.asTempObj(),
+                                                     auth::ValidatedTenancyScope::get(opCtx),
+                                                     boost::none,
+                                                     SerializationContext()),
         DBException,
         6253507);
 }
@@ -1688,8 +1645,14 @@ TEST(QueryRequestHelperTest, ValidateResponseMissingFields) {
 TEST(QueryRequestHelperTest, ValidateResponseWrongDataType) {
     BSONObjBuilder builder;
     builder.append("cursor", 1);
+    QueryTestServiceContext serviceContext;
+    auto uniqueTxn = serviceContext.makeOperationContext();
+    OperationContext* opCtx = uniqueTxn.get();
     ASSERT_THROWS_CODE(
-        query_request_helper::validateCursorResponse(builder.asTempObj(), boost::none),
+        query_request_helper::validateCursorResponse(builder.asTempObj(),
+                                                     auth::ValidatedTenancyScope::get(opCtx),
+                                                     boost::none,
+                                                     SerializationContext()),
         DBException,
         ErrorCodes::TypeMismatch);
 }
@@ -1718,7 +1681,8 @@ class QueryRequestTest : public ServiceContextTest {};
 
 TEST_F(QueryRequestTest, ParseFromUUID) {
     const UUID uuid = UUID::gen();
-    NamespaceStringOrUUID nssOrUUID("test", uuid);
+    NamespaceStringOrUUID nssOrUUID(DatabaseName::createDatabaseName_forTest(boost::none, "test"),
+                                    uuid);
     FindCommandRequest findCommand(nssOrUUID);
     const NamespaceString nss = NamespaceString::createNamespaceString_forTest("test.testns");
     findCommand.setNss(nss);
