@@ -29,8 +29,19 @@
 
 #include "mongo/db/query/query_settings.h"
 
+#include <boost/move/utility_core.hpp>
+#include <mutex>
+#include <tuple>
+#include <utility>
+
+#include <absl/container/node_hash_map.h>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/bson/simple_bsonobj_comparator.h"
+#include "mongo/db/basic_types_gen.h"
 #include "mongo/db/query/canonical_query.h"
-#include "mongo/db/query/plan_cache.h"
+#include "mongo/db/query/collation/collator_interface.h"
+#include "mongo/db/query/find_command.h"
 
 namespace mongo {
 
@@ -77,7 +88,15 @@ AllowedIndexEntry::AllowedIndexEntry(const BSONObj& query,
 //
 
 boost::optional<AllowedIndicesFilter> QuerySettings::getAllowedIndicesFilter(
-    const CanonicalQuery::IndexFilterKey& key) const {
+    const CanonicalQuery& canonicalQuery) const {
+    // Fast-check if there is at least one allowed index in query settings.
+    if (!_someAllowedIndexEntriesPresent.loadRelaxed()) {
+        return {};
+    }
+
+    // Compute the key before entering the critical section.
+    const CanonicalQuery::PlanCacheCommandKey key = canonicalQuery.encodeKeyForPlanCacheCommand();
+
     stdx::lock_guard<Latch> cacheLock(_mutex);
     AllowedIndexEntryMap::const_iterator cacheIter = _allowedIndexEntryMap.find(key);
 
@@ -105,7 +124,7 @@ void QuerySettings::setAllowedIndices(const CanonicalQuery& canonicalQuery,
     const BSONObj& query = findCommand.getFilter();
     const BSONObj& sort = findCommand.getSort();
     const BSONObj& projection = findCommand.getProjection();
-    const auto key = canonicalQuery.encodeKeyForIndexFilters();
+    const auto key = canonicalQuery.encodeKeyForPlanCacheCommand();
     const BSONObj collation =
         canonicalQuery.getCollator() ? canonicalQuery.getCollator()->getSpec().toBSON() : BSONObj();
 
@@ -115,9 +134,10 @@ void QuerySettings::setAllowedIndices(const CanonicalQuery& canonicalQuery,
         std::piecewise_construct,
         std::forward_as_tuple(key),
         std::forward_as_tuple(query, sort, projection, collation, indexKeyPatterns, indexNames));
+    _updateSomeAllowedIndexEntriesPresent();
 }
 
-void QuerySettings::removeAllowedIndices(const CanonicalQuery::IndexFilterKey& key) {
+void QuerySettings::removeAllowedIndices(const CanonicalQuery::PlanCacheCommandKey& key) {
     stdx::lock_guard<Latch> cacheLock(_mutex);
     AllowedIndexEntryMap::iterator i = _allowedIndexEntryMap.find(key);
 
@@ -127,11 +147,17 @@ void QuerySettings::removeAllowedIndices(const CanonicalQuery::IndexFilterKey& k
     }
 
     _allowedIndexEntryMap.erase(i);
+    _updateSomeAllowedIndexEntriesPresent();
 }
 
 void QuerySettings::clearAllowedIndices() {
     stdx::lock_guard<Latch> cacheLock(_mutex);
     _allowedIndexEntryMap.clear();
+    _updateSomeAllowedIndexEntriesPresent();
+}
+
+void QuerySettings::_updateSomeAllowedIndexEntriesPresent() {
+    _someAllowedIndexEntriesPresent.store(!_allowedIndexEntryMap.empty());
 }
 
 }  // namespace mongo
