@@ -27,10 +27,23 @@
  *    it in the license file.
  */
 
+#include <boost/move/utility_core.hpp>
+#include <cstdint>
+#include <memory>
+#include <s2cellid.h>
+#include <utility>
+#include <variant>
+
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+
 #include "mongo/bson/bsonmisc.h"
 #include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/json.h"
 #include "mongo/db/matcher/expression_geo.h"
-#include "mongo/db/matcher/extensions_callback_real.h"
+#include "mongo/db/matcher/expression_text_base.h"
+#include "mongo/db/matcher/expression_tree.h"
+#include "mongo/db/matcher/extensions_callback_noop.h"
 #include "mongo/db/matcher/parsed_match_expression_for_test.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/pipeline/expression_context_for_test.h"
@@ -39,9 +52,10 @@
 #include "mongo/db/query/query_shape/serialization_options.h"
 #include "mongo/db/query/query_shape/shape_helpers.h"
 #include "mongo/db/service_context_test_fixture.h"
-#include "mongo/unittest/bson_test_util.h"
-#include "mongo/unittest/unittest.h"
-
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/intrusive_counter.h"
+#include "mongo/util/str.h"
 namespace mongo::query_shape {
 
 namespace {
@@ -539,8 +553,8 @@ TEST(QueryShapeIDL, ShapifyIDLStruct) {
                                {1, 2, 3, 4},
                                "field.path",
                                {"field.path.1", "fieldpath2"},
-                               NamespaceString{"db", "coll"},
-                               NamespaceString{"db", "coll"},
+                               NamespaceString::createNamespaceString_forTest("db", "coll"),
+                               NamespaceString::createNamespaceString_forTest("db", "coll"),
                                177,
                                true);
     ASSERT_BSONOBJ_EQ_AUTO(  // NOLINT
@@ -687,7 +701,7 @@ static const NamespaceString kDefaultTestNss =
 
 struct DummyShapeSpecificComponents : public query_shape::CmdSpecificShapeComponents {
     DummyShapeSpecificComponents(){};
-    void HashValue(absl::HashState state) const {}
+    void HashValue(absl::HashState state) const override {}
     size_t size() const final {
         return sizeof(DummyShapeSpecificComponents);
     }
@@ -726,7 +740,7 @@ public:
     }
 
     // Random number for testing purposes.
-    size_t extraSize() const final override {
+    size_t extraSize() const final {
         return 125;
     }
     void appendCmdSpecificShapeComponents(BSONObjBuilder&,
