@@ -28,20 +28,32 @@
  */
 
 #include "serialization_options.h"
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
 #include "mongo/db/query/query_shape/serialization_options.h"
 
-#include <boost/optional.hpp>
+#include <absl/container/node_hash_map.h>
 #include <string>
 
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+
 #include "mongo/base/string_data.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsontypes_util.h"
+#include "mongo/bson/oid.h"
 #include "mongo/bson/timestamp.h"
 #include "mongo/db/exec/document_value/document.h"
 #include "mongo/db/exec/document_value/value.h"
-#include "mongo/db/query/explain_options.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
 #include "mongo/util/assert_util.h"
+#include "mongo/util/string_map.h"
+#include "mongo/util/time_support.h"
+
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
+
 
 namespace mongo {
 
@@ -396,10 +408,12 @@ void appendDefaultOfNonArrayType(BSONObjBuilder* bob, StringData name, const BSO
 
 const SerializationOptions SerializationOptions::kRepresentativeQueryShapeSerializeOptions =
     SerializationOptions{.literalPolicy =
-                             LiteralSerializationPolicy::kToRepresentativeParseableValue};
+                             LiteralSerializationPolicy::kToRepresentativeParseableValue,
+                         .inMatchExprSortAndDedupElements = false};
 
 const SerializationOptions SerializationOptions::kDebugQueryShapeSerializeOptions =
-    SerializationOptions{.literalPolicy = LiteralSerializationPolicy::kToDebugTypeString};
+    SerializationOptions{.literalPolicy = LiteralSerializationPolicy::kToDebugTypeString,
+                         .inMatchExprSortAndDedupElements = false};
 
 const SerializationOptions SerializationOptions::kMarkIdentifiers_FOR_TEST = SerializationOptions{
     .transformIdentifiers = true, .transformIdentifiersCallback = applyHmacForTest};
