@@ -27,21 +27,39 @@
  *    it in the license file.
  */
 
+#include <algorithm>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <initializer_list>
+#include <memory>
+#include <ostream>
 #include <utility>
 
-#include "mongo/platform/basic.h"
-
-#include <memory>
-
-#include "mongo/bson/bsontypes.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/bson/json.h"
+#include "mongo/db/exec/document_value/value.h"
+#include "mongo/db/field_ref.h"
+#include "mongo/db/index/index_descriptor.h"
+#include "mongo/db/index/multikey_paths.h"
+#include "mongo/db/index_names.h"
+#include "mongo/db/matcher/expression_parser.h"
+#include "mongo/db/pipeline/expression_context.h"
 #include "mongo/db/query/collation/collator_interface_mock.h"
 #include "mongo/db/query/index_bounds_builder.h"
 #include "mongo/db/query/index_entry.h"
+#include "mongo/db/query/interval.h"
+#include "mongo/db/query/planner_wildcard_helpers.h"
 #include "mongo/db/query/projection_parser.h"
+#include "mongo/db/query/projection_policies.h"
 #include "mongo/db/query/query_solution.h"
 #include "mongo/db/query/query_test_service_context.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/db/query/wildcard_test_utils.h"
+#include "mongo/stdx/type_traits.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
 
 namespace mongo {
 
@@ -614,6 +632,421 @@ TEST(QuerySolutionTest, IndexScanNodeDoesNotRemoveCollatedFieldsFromSortsIfColla
     ASSERT_TRUE(sorts.contains(BSON("a" << 1)));
 }
 
+// Index: {a: 1, b: "hashed", c: 1}
+// Bounds: a: [1, 1], b: [MINKEY, MAXKEY], c: [1, 2]
+TEST(QuerySolutionTest, HashedIndexScanNodeTruncatesSort) {
+    IndexScanNode node{buildSimpleIndexEntry(BSON("a" << 1 << "b"
+                                                      << "hashed"
+                                                      << "c" << 1))};
+    OrderedIntervalList a{};
+    a.name = "a";
+    a.intervals.push_back(IndexBoundsBuilder::makePointInterval(BSON("" << 1)));
+    node.bounds.fields.push_back(a);
+
+    OrderedIntervalList b{};
+    b.name = "b";
+    b.intervals.push_back(IndexBoundsBuilder::makeRangeInterval(
+        BSON("" << MINKEY << "" << MAXKEY), BoundInclusion::kIncludeBothStartAndEndKeys));
+    node.bounds.fields.push_back(b);
+
+    OrderedIntervalList c{};
+    c.name = "c";
+    c.intervals.push_back(IndexBoundsBuilder::makeRangeInterval(
+        BSON("" << 1 << "" << 2), BoundInclusion::kIncludeBothStartAndEndKeys));
+    node.bounds.fields.push_back(c);
+
+    node.computeProperties();
+    auto sorts = node.providedSorts();
+
+    // The hashed field cannot be part of the provided sort and forces the rest of the sort to be
+    // truncated.
+    ASSERT_EQ(sorts, ProvidedSortSet(BSONObj(), {"a"} /* equality fields */));
+    ASSERT(sorts.contains(BSON("a" << 1)));
+    ASSERT_FALSE(sorts.contains(BSON("b" << 1)));
+    ASSERT_FALSE(sorts.contains(BSON("c" << 1)));
+    ASSERT_FALSE(sorts.contains(BSON("a" << 1 << "b" << 1)));
+    ASSERT_FALSE(sorts.contains(BSON("a" << 1 << "c" << 1)));
+    ASSERT_FALSE(sorts.contains(BSON("b" << 1 << "c" << 1)));
+}
+
+// Index: {a: 1, b: "hashed", c: 1}
+// Bounds: a: [1, 1], b: [MINKEY, MAXKEY], c: [1, 1]
+TEST(QuerySolutionTest, HashedIndexScanNodeTruncatesSortUnlessFollowedByEquality) {
+    IndexScanNode node{buildSimpleIndexEntry(BSON("a" << 1 << "b"
+                                                      << "hashed"
+                                                      << "c" << 1))};
+    OrderedIntervalList a{};
+    a.name = "a";
+    a.intervals.push_back(IndexBoundsBuilder::makePointInterval(BSON("" << 1)));
+    node.bounds.fields.push_back(a);
+
+    OrderedIntervalList b{};
+    b.name = "b";
+    b.intervals.push_back(IndexBoundsBuilder::makeRangeInterval(
+        BSON("" << MINKEY << "" << MAXKEY), BoundInclusion::kIncludeBothStartAndEndKeys));
+    node.bounds.fields.push_back(b);
+
+    OrderedIntervalList c{};
+    c.name = "c";
+    c.intervals.push_back(IndexBoundsBuilder::makePointInterval(BSON("" << 1)));
+    node.bounds.fields.push_back(c);
+
+    node.computeProperties();
+    auto sorts = node.providedSorts();
+
+    // The hashed field cannot be part of the provided sort and forces the rest of the sort to be
+    // truncated except for following equalities.
+    ASSERT_EQ(sorts, ProvidedSortSet(BSONObj(), {"a", "c"} /* equality fields */));
+    ASSERT(sorts.contains(BSON("a" << 1)));
+    ASSERT_FALSE(sorts.contains(BSON("b" << 1)));
+    ASSERT(sorts.contains(BSON("c" << 1)));
+    ASSERT_FALSE(sorts.contains(BSON("a" << 1 << "b" << 1)));
+    ASSERT(sorts.contains(BSON("a" << 1 << "c" << 1)));
+    ASSERT_FALSE(sorts.contains(BSON("b" << 1 << "c" << 1)));
+}
+
+// Index: {a: 1, b: "hashed", c: 1}
+// Bounds: a: [1, 1], b: [1, 1], c: [MINKEY, MAXKEY]
+TEST(QuerySolutionTest, HashedIndexScanNodeDoesNotTruncateSortWhenEquality) {
+    IndexScanNode node{buildSimpleIndexEntry(BSON("a" << 1 << "b"
+                                                      << "hashed"
+                                                      << "c" << 1))};
+    OrderedIntervalList a{};
+    a.name = "a";
+    a.intervals.push_back(IndexBoundsBuilder::makePointInterval(BSON("" << 1)));
+    node.bounds.fields.push_back(a);
+
+    OrderedIntervalList b{};
+    b.name = "b";
+    b.intervals.push_back(IndexBoundsBuilder::makePointInterval(BSON("" << 1)));
+    node.bounds.fields.push_back(b);
+
+    OrderedIntervalList c{};
+    c.name = "c";
+    c.intervals.push_back(IndexBoundsBuilder::makeRangeInterval(
+        BSON("" << MINKEY << "" << MAXKEY), BoundInclusion::kIncludeBothStartAndEndKeys));
+    node.bounds.fields.push_back(c);
+
+    node.computeProperties();
+    auto sorts = node.providedSorts();
+
+    // The hashed field can be part of the sort when its bounds are a point interval (equality). It
+    // does not cause truncation in this case.
+    ASSERT_EQ(sorts, ProvidedSortSet(BSON("c" << 1), {"a", "b"} /* equality fields */));
+    ASSERT(sorts.contains(BSON("a" << 1)));
+    ASSERT(sorts.contains(BSON("b" << 1)));
+    ASSERT(sorts.contains(BSON("c" << 1)));
+    ASSERT(sorts.contains(BSON("a" << 1 << "b" << 1)));
+    ASSERT(sorts.contains(BSON("a" << 1 << "c" << 1)));
+    ASSERT(sorts.contains(BSON("b" << 1 << "c" << 1)));
+}
+
+// Index: {a: 1, b: "hashed", c: 1}
+// Bounds: a: [1, 1], b: ["p", "p"], c: [MINKEY, MAXKEY]
+TEST(QuerySolutionTest, HashedIndexScanNodeDoesTruncatesSortWhenCollationDoesntMatch) {
+    IndexScanNode node{buildSimpleIndexEntry(BSON("a" << 1 << "b"
+                                                      << "hashed"
+                                                      << "c" << 1))};
+    CollatorInterfaceMock queryCollator(CollatorInterfaceMock::MockType::kToLowerString);
+    node.queryCollator = &queryCollator;
+
+    OrderedIntervalList a{};
+    a.name = "a";
+    a.intervals.push_back(IndexBoundsBuilder::makePointInterval(BSON("" << 1)));
+    node.bounds.fields.push_back(a);
+
+    OrderedIntervalList b{};
+    b.name = "b";
+    b.intervals.push_back(IndexBoundsBuilder::makePointInterval(BSON(""
+                                                                     << "p")));
+    node.bounds.fields.push_back(b);
+
+    OrderedIntervalList c{};
+    c.name = "c";
+    c.intervals.push_back(IndexBoundsBuilder::makeRangeInterval(
+        BSON("" << MINKEY << "" << MAXKEY), BoundInclusion::kIncludeBothStartAndEndKeys));
+    node.bounds.fields.push_back(c);
+
+    node.computeProperties();
+    auto sorts = node.providedSorts();
+
+    // The hashed field cannot be part of the sort when its bounds are a point interval (equality)
+    // that is affected by collation and the query and index collation do not match. It forces the
+    // rest of the sort to be truncated.
+    ASSERT_EQ(sorts, ProvidedSortSet(BSONObj(), {"a"} /* equality fields */));
+    ASSERT(sorts.contains(BSON("a" << 1)));
+    ASSERT_FALSE(sorts.contains(BSON("b" << 1)));
+    ASSERT_FALSE(sorts.contains(BSON("c" << 1)));
+    ASSERT_FALSE(sorts.contains(BSON("a" << 1 << "b" << 1)));
+    ASSERT_FALSE(sorts.contains(BSON("a" << 1 << "c" << 1)));
+    ASSERT_FALSE(sorts.contains(BSON("b" << 1 << "c" << 1)));
+}
+
+// Index: {a: 1, b: "hashed", c: 1}
+// Bounds: a: [1, 1], b: ["a", "b"], c: [MINKEY, MAXKEY]
+TEST(QuerySolutionTest,
+     HashedIndexScanNodeDoesTruncatesSortWhenCollationDoesntMatchWithRangeQuery) {
+    IndexScanNode node{buildSimpleIndexEntry(BSON("a" << 1 << "b"
+                                                      << "hashed"
+                                                      << "c" << 1))};
+    CollatorInterfaceMock queryCollator(CollatorInterfaceMock::MockType::kToLowerString);
+    node.queryCollator = &queryCollator;
+
+    OrderedIntervalList a{};
+    a.name = "a";
+    a.intervals.push_back(IndexBoundsBuilder::makePointInterval(BSON("" << 1)));
+    node.bounds.fields.push_back(a);
+
+    OrderedIntervalList b{};
+    b.name = "b";
+    b.intervals.push_back(
+        IndexBoundsBuilder::makeRangeInterval(BSON(""
+                                                   << "a"
+                                                   << ""
+                                                   << "b"),
+                                              BoundInclusion::kIncludeBothStartAndEndKeys));
+    node.bounds.fields.push_back(b);
+
+    OrderedIntervalList c{};
+    c.name = "c";
+    c.intervals.push_back(IndexBoundsBuilder::makeRangeInterval(
+        BSON("" << MINKEY << "" << MAXKEY), BoundInclusion::kIncludeBothStartAndEndKeys));
+    node.bounds.fields.push_back(c);
+
+    node.computeProperties();
+    auto sorts = node.providedSorts();
+
+    // The hashed field cannot be part of the sort when its bounds are a range interval that is
+    // affected by collation and the query and index collation do not match. It forces the rest of
+    // the sort to be truncated.
+    ASSERT_EQ(sorts, ProvidedSortSet(BSONObj(), {"a"} /* equality fields */));
+    ASSERT(sorts.contains(BSON("a" << 1)));
+    ASSERT_FALSE(sorts.contains(BSON("b" << 1)));
+    ASSERT_FALSE(sorts.contains(BSON("c" << 1)));
+    ASSERT_FALSE(sorts.contains(BSON("a" << 1 << "b" << 1)));
+    ASSERT_FALSE(sorts.contains(BSON("a" << 1 << "c" << 1)));
+    ASSERT_FALSE(sorts.contains(BSON("b" << 1 << "c" << 1)));
+}
+
+// Expanded wildcard index: {a: 1, b.$**: 1, c: 1}
+// Bounds: a: [1, 1], $_path: ["b", "b"], b: [1, 2], c: [2, 3]
+TEST(QuerySolutionTest, WildcardIndexSupportsSortWhenIndexOnlyNeedsToLookAtOnePath) {
+    mongo::wildcard_planning::WildcardIndexEntryMock wildcardIndex{
+        BSON("a" << 1 << "b.$**" << 1 << "c" << 1), BSONObj{}, {}};
+
+    // The following setup mimics a query that queries against fields "a", "b.d", and "c". This
+    // matches the bounds we generate for those fields below. However, only pass "b.d" as 'fields'
+    // here, because we only want to consider the expanded index with that field plugged in.
+    stdx::unordered_set<std::string> fields{"b.d"};
+    std::vector<IndexEntry> expandedIndexes{};
+    mongo::wildcard_planning::expandWildcardIndexEntry(
+        *wildcardIndex.indexEntry, fields, &expandedIndexes);
+
+    ASSERT_EQ(expandedIndexes.size(), 1);
+    IndexScanNode node{expandedIndexes.at(0)};
+
+    OrderedIntervalList a{};
+    a.name = "a";
+    a.intervals.push_back(IndexBoundsBuilder::makePointInterval(BSON("" << 1)));
+    node.bounds.fields.push_back(a);
+
+    OrderedIntervalList b{};
+    b.name = "b.d";
+    b.intervals.push_back(IndexBoundsBuilder::makeRangeInterval(
+        BSON("" << 1 << "" << 2), BoundInclusion::kIncludeBothStartAndEndKeys));
+
+    node.bounds.fields.push_back(b);
+
+    OrderedIntervalList c{};
+    c.name = "c";
+    c.intervals.push_back(IndexBoundsBuilder::makeRangeInterval(
+        BSON("" << 2 << "" << 3), BoundInclusion::kIncludeBothStartAndEndKeys));
+    node.bounds.fields.push_back(c);
+
+    std::vector<interval_evaluation_tree::Builder> ietBuilders;
+    mongo::wildcard_planning::finalizeWildcardIndexScanConfiguration(&node, &ietBuilders);
+
+    node.computeProperties();
+    auto sorts = node.providedSorts();
+
+    // When the wildcard path is only on one point, it can provide a sort on the wildcard field
+    // (b.d). "$_path" is a reserved key used for describing the wildcard path. It appears here in
+    // fields with equality bounds.
+    ASSERT_EQ(sorts,
+              ProvidedSortSet(BSON("b.d" << 1 << "c" << 1), {"a", "$_path"} /* equality fields */));
+    ASSERT(sorts.contains(BSON("a" << 1)));
+    ASSERT(sorts.contains(BSON("b.d" << 1)));
+    ASSERT_FALSE(sorts.contains(BSON("c" << 1)));
+    ASSERT(sorts.contains(BSON("a" << 1 << "b.d" << 1)));
+    ASSERT(sorts.contains(BSON("b.d" << 1 << "c" << 1)));
+    ASSERT(sorts.contains(BSON("a" << 1 << "b.d" << 1 << "c" << 1)));
+    ASSERT_FALSE(sorts.contains(BSON("a" << 1 << "c" << 1)));
+}
+
+// Index: {a: 1, b.$**: 1, c: 1}
+// Bounds: a: [1, 1], b: [MINKEY, MAXKEY], c: [2, 3]
+TEST(QuerySolutionTest, WildcardIndexDoesNotSupportSortWhenIndexNeedsToLookAtMultiplePaths) {
+    // The following setup mimics a query that queries against fields "a", "b", and "c". This
+    // matches the bounds we generate for those fields below. However, only pass "b" as 'fields'
+    // here, because we only want to consider the expanded index with that field plugged in.
+    mongo::wildcard_planning::WildcardIndexEntryMock wildcardIndex{
+        BSON("a" << 1 << "b.$**" << 1 << "c" << 1), BSONObj{}, {}};
+    stdx::unordered_set<std::string> fields{"b"};
+    std::vector<IndexEntry> expandedIndexes{};
+    mongo::wildcard_planning::expandWildcardIndexEntry(
+        *wildcardIndex.indexEntry, fields, &expandedIndexes);
+
+    ASSERT_EQ(expandedIndexes.size(), 1);
+    IndexScanNode node{expandedIndexes.at(0)};
+
+    OrderedIntervalList a{};
+    a.name = "a";
+    a.intervals.push_back(IndexBoundsBuilder::makePointInterval(BSON("" << 1)));
+    node.bounds.fields.push_back(a);
+
+    OrderedIntervalList b{};
+    b.name = "b";
+    b.intervals.push_back(IndexBoundsBuilder::makeRangeInterval(
+        BSON("" << MINKEY << "" << MAXKEY), BoundInclusion::kIncludeBothStartAndEndKeys));
+    node.bounds.fields.push_back(b);
+
+    OrderedIntervalList c{};
+    c.name = "c";
+    c.intervals.push_back(IndexBoundsBuilder::makeRangeInterval(
+        BSON("" << 2 << "" << 3), BoundInclusion::kIncludeBothStartAndEndKeys));
+    node.bounds.fields.push_back(c);
+
+    std::vector<interval_evaluation_tree::Builder> ietBuilders;
+    mongo::wildcard_planning::finalizeWildcardIndexScanConfiguration(&node, &ietBuilders);
+
+    node.computeProperties();
+    auto sorts = node.providedSorts();
+
+    // When the wildcard path is a union, it cannot provide a sort on any wildcard field.
+    ASSERT_EQ(sorts, ProvidedSortSet(BSONObj(), {"a"} /* equality fields */));
+    ASSERT(sorts.contains(BSON("a" << 1)));
+    ASSERT_FALSE(sorts.contains(BSON("b" << 1)));
+    ASSERT_FALSE(sorts.contains(BSON("b.d" << 1)));
+    ASSERT_FALSE(sorts.contains(BSON("c" << 1)));
+    ASSERT_FALSE(sorts.contains(BSON("a" << 1 << "b" << 1)));
+    ASSERT_FALSE(sorts.contains(BSON("a" << 1 << "c" << 1)));
+}
+
+// Index: {a: 1, b.$**: 1, c: 1}
+// Bounds: a: [1, 1], b: ["p", "p"], c: [2, 3]
+TEST(QuerySolutionTest, WildcardIndexDoesNotSupportSortWhenCollationDoesntMatch) {
+    // The following setup mimics a query that queries against fields "a", "b.d", and "c". This
+    // matches the bounds we generate for those fields below.  However, only pass "b.d" as 'fields'
+    // here, because we only want to consider the expanded index with that field plugged in.
+    mongo::wildcard_planning::WildcardIndexEntryMock wildcardIndex{
+        BSON("a" << 1 << "b.$**" << 1 << "c" << 1), BSONObj{}, {}};
+    stdx::unordered_set<std::string> fields{"b.d"};
+    std::vector<IndexEntry> expandedIndexes{};
+    mongo::wildcard_planning::expandWildcardIndexEntry(
+        *wildcardIndex.indexEntry, fields, &expandedIndexes);
+
+    ASSERT_EQ(expandedIndexes.size(), 1);
+    IndexScanNode node{expandedIndexes.at(0)};
+
+    CollatorInterfaceMock queryCollator(CollatorInterfaceMock::MockType::kToLowerString);
+    node.queryCollator = &queryCollator;
+
+    OrderedIntervalList a{};
+    a.name = "a";
+    a.intervals.push_back(IndexBoundsBuilder::makePointInterval(BSON("" << 1)));
+    node.bounds.fields.push_back(a);
+
+    OrderedIntervalList b{};
+    b.name = "b.d";
+    b.intervals.push_back(IndexBoundsBuilder::makePointInterval(BSON(""
+                                                                     << "p")));
+    node.bounds.fields.push_back(b);
+
+    OrderedIntervalList c{};
+    c.name = "c";
+    c.intervals.push_back(IndexBoundsBuilder::makeRangeInterval(
+        BSON("" << 2 << "" << 3), BoundInclusion::kIncludeBothStartAndEndKeys));
+    node.bounds.fields.push_back(c);
+
+
+    std::vector<interval_evaluation_tree::Builder> ietBuilders;
+    mongo::wildcard_planning::finalizeWildcardIndexScanConfiguration(&node, &ietBuilders);
+
+    node.computeProperties();
+
+    // When the wildcard path is only on one point, but the wildcard field matches string values
+    // and the query and indxex collation differ, then the index cannot provide a sort on the
+    // wildcard field (b.d). It does not cause truncation because the bounds for b.d are a point.
+    auto sorts = node.providedSorts();
+    ASSERT_EQ(sorts, ProvidedSortSet(BSON("c" << 1), {"a"} /* equality fields */));
+    ASSERT(sorts.contains(BSON("a" << 1)));
+    ASSERT_FALSE(sorts.contains(BSON("b.d" << 1)));
+    ASSERT(sorts.contains(BSON("c" << 1)));
+    ASSERT_FALSE(sorts.contains(BSON("a" << 1 << "b.d" << 1)));
+    ASSERT_FALSE(sorts.contains(BSON("b.d" << 1 << "c" << 1)));
+    ASSERT_FALSE(sorts.contains(BSON("a" << 1 << "b.d" << 1 << "c" << 1)));
+    ASSERT(sorts.contains(BSON("a" << 1 << "c" << 1)));
+}
+
+// Index: {a: 1, b.$**: 1, c: 1}
+// Bounds: a: [1, 1], b: ["a", "b"], c: [2, 3]
+TEST(QuerySolutionTest, WildcardIndexDoesNotSupportSortWhenCollationDoesntMatchWithRangeQuery) {
+    // The following setup mimics a query that queries against fields "a", "b.d", and "c". This
+    // matches the bounds we generate for those fields below. However, only pass "b.d" as 'fields'
+    // here, because we only want to consider the expanded index with that field plugged in.
+    mongo::wildcard_planning::WildcardIndexEntryMock wildcardIndex{
+        BSON("a" << 1 << "b.$**" << 1 << "c" << 1), BSONObj{}, {}};
+    stdx::unordered_set<std::string> fields{"b.d"};
+    std::vector<IndexEntry> expandedIndexes{};
+    mongo::wildcard_planning::expandWildcardIndexEntry(
+        *wildcardIndex.indexEntry, fields, &expandedIndexes);
+
+    ASSERT_EQ(expandedIndexes.size(), 1);
+    IndexScanNode node{expandedIndexes.at(0)};
+
+    CollatorInterfaceMock queryCollator(CollatorInterfaceMock::MockType::kToLowerString);
+    node.queryCollator = &queryCollator;
+
+    OrderedIntervalList a{};
+    a.name = "a";
+    a.intervals.push_back(IndexBoundsBuilder::makePointInterval(BSON("" << 1)));
+    node.bounds.fields.push_back(a);
+
+    OrderedIntervalList b{};
+    b.name = "b.d";
+    b.intervals.push_back(
+        IndexBoundsBuilder::makeRangeInterval(BSON(""
+                                                   << "a"
+                                                   << ""
+                                                   << "b"),
+                                              BoundInclusion::kIncludeBothStartAndEndKeys));
+    node.bounds.fields.push_back(b);
+
+    OrderedIntervalList c{};
+    c.name = "c";
+    c.intervals.push_back(IndexBoundsBuilder::makeRangeInterval(
+        BSON("" << 2 << "" << 3), BoundInclusion::kIncludeBothStartAndEndKeys));
+    node.bounds.fields.push_back(c);
+
+    std::vector<interval_evaluation_tree::Builder> ietBuilders;
+    mongo::wildcard_planning::finalizeWildcardIndexScanConfiguration(&node, &ietBuilders);
+
+    node.computeProperties();
+    auto sorts = node.providedSorts();
+
+    // When the wildcard path is only on one point, but the wildcard field matches string values
+    // and the query and indxex collation differ, then the index cannot provide a sort on the
+    // wildcard field (b.d). It also causes truncation because the bounds for b.d are a range.
+    ASSERT_EQ(sorts, ProvidedSortSet(BSONObj(), {"a"} /* equality fields */));
+    ASSERT(sorts.contains(BSON("a" << 1)));
+    ASSERT_FALSE(sorts.contains(BSON("b.d" << 1)));
+    ASSERT_FALSE(sorts.contains(BSON("c" << 1)));
+    ASSERT_FALSE(sorts.contains(BSON("a" << 1 << "b.d" << 1)));
+    ASSERT_FALSE(sorts.contains(BSON("b.d" << 1 << "c" << 1)));
+    ASSERT_FALSE(sorts.contains(BSON("a" << 1 << "b.d" << 1 << "c" << 1)));
+    ASSERT_FALSE(sorts.contains(BSON("a" << 1 << "c" << 1)));
+}
+
 // Index: {a: 1, b: 1, c: 1, d: 1, e: 1}
 // Intervals: a: [1,1], b: ['p','p'], c: [1,2], d: [MinKey, MaxKey], e: [1,2]
 TEST(QuerySolutionTest, CompoundIndexWithNonMatchingCollationFiltersAllSortsWithCollatedField) {
@@ -828,7 +1261,7 @@ TEST(QuerySolutionTest, InclusionProjectionPreservesSort) {
     auto matchExprAndProjection = createMatchExprAndProjection(match, projection);
 
     ProjectionNodeDefault proj{
-        std::move(node), *matchExprAndProjection.first, matchExprAndProjection.second};
+        std::move(node), matchExprAndProjection.first.get(), matchExprAndProjection.second};
     proj.computeProperties();
 
     ASSERT_EQ(proj.providedSorts(), ProvidedSortSet(BSON("a" << 1), {}));
@@ -845,7 +1278,7 @@ TEST(QuerySolutionTest, ExclusionProjectionDoesNotPreserveSort) {
     auto matchExprAndProjection = createMatchExprAndProjection(match, projection);
 
     ProjectionNodeDefault proj{
-        std::move(node), *matchExprAndProjection.first, matchExprAndProjection.second};
+        std::move(node), matchExprAndProjection.first.get(), matchExprAndProjection.second};
     proj.computeProperties();
 
     ASSERT_EQ(proj.providedSorts(), kEmptySet);
@@ -861,7 +1294,7 @@ TEST(QuerySolutionTest, InclusionProjectionTruncatesSort) {
     auto matchExprAndProjection = createMatchExprAndProjection(match, projection);
 
     ProjectionNodeDefault proj{
-        std::move(node), *matchExprAndProjection.first, matchExprAndProjection.second};
+        std::move(node), matchExprAndProjection.first.get(), matchExprAndProjection.second};
     proj.computeProperties();
 
     ASSERT_EQ(proj.providedSorts(), ProvidedSortSet(BSON("a" << 1), {}));
@@ -877,7 +1310,7 @@ TEST(QuerySolutionTest, ExclusionProjectionTruncatesSort) {
     auto matchExprAndProjection = createMatchExprAndProjection(match, projection);
 
     ProjectionNodeDefault proj{
-        std::move(node), *matchExprAndProjection.first, matchExprAndProjection.second};
+        std::move(node), matchExprAndProjection.first.get(), matchExprAndProjection.second};
     proj.computeProperties();
 
     ASSERT_EQ(proj.providedSorts(), ProvidedSortSet(BSON("a" << 1), {}));
@@ -1263,6 +1696,29 @@ TEST(QuerySolutionTest, GetSecondaryNamespaceVectorOverSingleEqLookupNode) {
     assertNamespaceVectorsAreEqual(qs.getAllSecondaryNamespaces(mainNss), expectedNssVector);
 }
 
+TEST(QuerySolutionTest, AssertSameHashes) {
+    auto makeQs = []() {
+        auto scanNode = std::make_unique<IndexScanNode>(buildSimpleIndexEntry(BSON("a" << 1)));
+        const NamespaceString mainNss = NamespaceString::createNamespaceString_forTest("db.main");
+        const NamespaceString foreignColl =
+            NamespaceString::createNamespaceString_forTest("db.col");
+        auto root = std::make_unique<EqLookupNode>(std::move(scanNode),
+                                                   foreignColl,
+                                                   "local",
+                                                   "remote",
+                                                   "b",
+                                                   EqLookupNode::LookupStrategy::kNestedLoopJoin,
+                                                   boost::none /* idxEntry */,
+                                                   false /* shouldProduceBson */);
+
+
+        auto qs = std::make_unique<QuerySolution>();
+        qs->setRoot(std::move(root));
+        return qs;
+    };
+    ASSERT(makeQs()->hash() == makeQs()->hash());
+}
+
 TEST(QuerySolutionTest, GetSecondaryNamespaceVectorDeduplicatesMainNss) {
     auto scanNode = std::make_unique<IndexScanNode>(buildSimpleIndexEntry(BSON("a" << 1)));
     const NamespaceString mainNss = NamespaceString::createNamespaceString_forTest("db.main");
@@ -1352,4 +1808,81 @@ TEST(QuerySolutionTest, GetSecondaryNamespaceVectorDeduplicatesNestedEqLookupNod
     std::vector<NamespaceStringOrUUID> expectedNssVector{foreignColl};
     assertNamespaceVectorsAreEqual(qs.getAllSecondaryNamespaces(mainNss), expectedNssVector);
 }
+
+TEST(QuerySolutionTest, GetFirstNodeByTypeFindsFirstNodeWhenNested) {
+    auto collScanNode = std::make_unique<CollectionScanNode>();
+    auto limitNode = std::make_unique<LimitNode>(
+        std::move(collScanNode), 10ll, LimitSkipParameterization::Disabled);
+
+    QuerySolution qs;
+    qs.setRoot(std::move(limitNode));
+
+    auto [foundNode, foundCount] = qs.getFirstNodeByType(StageType::STAGE_COLLSCAN);
+    ASSERT_EQ(qs.root()->children.at(0).get(), foundNode);
+    ASSERT_EQ(1, foundCount);
+}
+
+TEST(QuerySolutionTest, GetFirstNodeByTypeFindsFirstNodeWhenInRoot) {
+    auto collScanNode = std::make_unique<CollectionScanNode>();
+
+    QuerySolution qs;
+    qs.setRoot(std::move(collScanNode));
+
+    auto [foundNode, foundCount] = qs.getFirstNodeByType(StageType::STAGE_COLLSCAN);
+    ASSERT_EQ(qs.root(), foundNode);
+    ASSERT_EQ(1, foundCount);
+}
+
+TEST(QuerySolutionTest, GetFirstNodeByTypeReturnsNullIfNotFound) {
+    auto collScanNode = std::make_unique<CollectionScanNode>();
+
+    QuerySolution qs;
+    qs.setRoot(std::move(collScanNode));
+
+    auto [foundNode, foundCount] = qs.getFirstNodeByType(StageType::STAGE_IXSCAN);
+    ASSERT_EQ(nullptr, foundNode);
+    ASSERT_EQ(0, foundCount);
+}
+
+TEST(QuerySolutionTest, GetFirstNodeByTypeFindsFirstAndCountsWhenSeveral) {
+    auto collScanNode = std::make_unique<CollectionScanNode>();
+    auto firstLimitNode = std::make_unique<LimitNode>(
+        std::move(collScanNode), 10ll, LimitSkipParameterization::Disabled);
+    auto firstLimitNodeLimitValue = firstLimitNode->limit;
+    auto secondLimitNode = std::make_unique<LimitNode>(
+        std::move(firstLimitNode), 8ll, LimitSkipParameterization::Disabled);
+    // We use its 'limit' value to assert the first one was retrieved below hence cannot be equals
+    ASSERT(firstLimitNodeLimitValue != secondLimitNode->limit);
+    auto skipNode = std::make_unique<SkipNode>(
+        std::move(secondLimitNode), 9ll, LimitSkipParameterization::Disabled);
+
+    QuerySolution qs;
+    qs.setRoot(std::move(skipNode));
+
+    auto [foundNode, foundCount] = qs.getFirstNodeByType(StageType::STAGE_LIMIT);
+    const LimitNode* foundLimitNode = dynamic_cast<const LimitNode*>(foundNode);
+    ASSERT(foundLimitNode);
+    ASSERT_EQ(foundLimitNode->limit, 8ll);  // 8 is the value we assign to the first limit node
+    ASSERT_EQ(2, foundCount);
+}
+
+TEST(QuerySolutionTest, ShouldCacheEofPlanTree) {
+    // QuerySolutions with EOF nodes are eligible for the plan cache.
+
+    // QuerySolution with root EOF node is eligible for the plan cache.
+    auto solution1 = std::make_unique<QuerySolution>();
+    solution1->setRoot(std::make_unique<EofNode>());
+    ASSERT_TRUE(solution1->isEligibleForPlanCache());
+
+    // QuerySolution with child EOF node is eligible for the plan cache.
+    std::vector<std::unique_ptr<QuerySolutionNode>> indexScanList;
+    indexScanList.push_back(std::make_unique<EofNode>());
+    auto orNode = std::make_unique<OrNode>();
+    orNode->addChildren(std::move(indexScanList));
+
+    auto solution2 = std::make_unique<QuerySolution>();
+    solution2->setRoot(std::move(orNode));
+
+    ASSERT_TRUE(solution2->isEligibleForPlanCache());
+}
 }  // namespace
