@@ -29,15 +29,32 @@
 
 #pragma once
 
+#include <boost/container_hash/extensions.hpp>
 #include <boost/functional/hash.hpp>
+#include <boost/none.hpp>
+#include <boost/optional.hpp>
+#include <boost/optional/optional.hpp>
+#include <cstddef>
+#include <cstdint>
+#include <memory>
+#include <string>
+#include <utility>
+#include <vector>
 
+#include "mongo/bson/oid.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/db/catalog/util/partitioned.h"
 #include "mongo/db/exec/sbe/stages/stages.h"
 #include "mongo/db/hasher.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/query/plan_cache.h"
+#include "mongo/db/query/plan_cache_debug_info.h"
 #include "mongo/db/query/plan_cache_key_info.h"
-#include "mongo/db/query/sbe_stage_builder.h"
+#include "mongo/db/query/sbe_stage_builder_plan_data.h"
 #include "mongo/db/service_context.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/container_size_helper.h"
+#include "mongo/util/uuid.h"
 
 namespace mongo {
 namespace sbe {
@@ -59,16 +76,12 @@ struct PlanCacheKeyShardingEpoch {
 struct PlanCacheKeyCollectionState {
     bool operator==(const PlanCacheKeyCollectionState& other) const {
         return other.uuid == uuid && other.version == version &&
-            other.newestVisibleIndexTimestamp == newestVisibleIndexTimestamp &&
             other.collectionGeneration == collectionGeneration;
     }
 
     size_t hashCode() const {
         size_t hash = UUID::Hash{}(uuid);
         boost::hash_combine(hash, version);
-        if (newestVisibleIndexTimestamp) {
-            boost::hash_combine(hash, newestVisibleIndexTimestamp->asULL());
-        }
         if (collectionGeneration) {
             collectionGeneration->epoch.hash_combine(hash);
             boost::hash_combine(hash, collectionGeneration->ts.asULL());
@@ -88,20 +101,6 @@ struct PlanCacheKeyCollectionState {
     // all readers seeing this version of the collection have drained.
     size_t version;
 
-    // The '_collectionVersion' is not currently sufficient in order to ensure that the indexes
-    // visible to the reader are consistent with the indexes present in the cache entry. The reason
-    // is that all readers see the latest copy-on-write version of the 'Collection' object, even
-    // though they are allowed to read at an older timestamp, potentially at a time before an index
-    // build completed.
-    //
-    // To solve this problem, we incorporate the timestamp of the newest index visible to the reader
-    // into the plan cache key. This ensures that the set of indexes visible to the reader match
-    // those present in the plan cache entry, preventing a situation where the plan cache entry
-    // reflects a newer version of the index catalog than the one visible to the reader.
-    //
-    // In the future, this could instead be solved with point-in-time catalog lookups.
-    boost::optional<Timestamp> newestVisibleIndexTimestamp;
-
     // Ensures that a cached SBE plan cannot be reused if the collection has since become sharded or
     // changed its shard key. The cached plan may no longer be valid after sharding or shard key
     // refining since the structure of the plan depends on whether the collection is sharded, and if
@@ -148,8 +147,8 @@ public:
         return !(*this == other);
     }
 
-    uint32_t queryHash() const {
-        return _info.queryHash();
+    uint32_t planCacheShapeHash() const {
+        return _info.planCacheShapeHash();
     }
 
     uint32_t planCacheKeyHash() const {
@@ -165,6 +164,18 @@ public:
         return _info.toString();
     }
 
+    /**
+     * Returns the estimated size of the plan cache key in bytes.
+     */
+    uint64_t estimatedKeySizeBytes() const {
+        return sizeof(*this) + _info.keySizeInBytes() +
+            container_size_helper::estimateObjectSizeInBytes(_secondaryCollectionStates);
+    }
+
+    const query_settings::QuerySettings& querySettings() const {
+        return _info.querySettings();
+    }
+
 private:
     // Contains the actual encoding of the query shape as well as the index discriminators.
     const PlanCacheKeyInfo _info;
@@ -193,16 +204,19 @@ struct PlanCachePartitioner {
 
 /**
  * Represents the data cached in the SBE plan cache. This data holds an execution plan and necessary
- * auxiliary data for preparing and executing the PlanStage tree.
+ * auxiliary data for preparing and executing the PlanStage tree. In addition, holds the hash of the
+ * query solution that led to this plan.
  */
 struct CachedSbePlan {
-    CachedSbePlan(std::unique_ptr<sbe::PlanStage> root, stage_builder::PlanStageData data)
-        : root(std::move(root)), planStageData(std::move(data)) {
-        tassert(5968206, "The RuntimeEnvironment should not be null", planStageData.env);
+    CachedSbePlan(std::unique_ptr<sbe::PlanStage> root,
+                  stage_builder::PlanStageData data,
+                  size_t hash)
+        : root(std::move(root)), planStageData(std::move(data)), solutionHash(hash) {
+        tassert(5968206, "The RuntimeEnvironment should not be null", planStageData.env.runtimeEnv);
     }
 
     std::unique_ptr<CachedSbePlan> clone() const {
-        return std::make_unique<CachedSbePlan>(root->clone(), planStageData);
+        return std::make_unique<CachedSbePlan>(root->clone(), planStageData, solutionHash);
     }
 
     uint64_t estimateObjectSizeInBytes() const {
@@ -212,9 +226,13 @@ struct CachedSbePlan {
     std::unique_ptr<sbe::PlanStage> root;
     stage_builder::PlanStageData planStageData;
     bool indexFilterApplied = false;
+    // Hash of the QuerySolution that led this cache entry.
+    size_t solutionHash;
 };
 
 using PlanCacheEntry = PlanCacheEntryBase<CachedSbePlan, plan_cache_debug_info::DebugInfoSBE>;
+using CachedPlanHolder =
+    mongo::CachedPlanHolder<CachedSbePlan, plan_cache_debug_info::DebugInfoSBE>;
 
 struct BudgetEstimator {
     /**
@@ -223,9 +241,7 @@ struct BudgetEstimator {
      */
     size_t operator()(const sbe::PlanCacheKey& key,
                       const std::shared_ptr<const PlanCacheEntry>& entry) {
-        // TODO: SERVER-73649 include size of underlying query shape and size of int_32 key hash in
-        // total size estimation.
-        return entry->estimatedEntrySizeBytes;
+        return entry->estimatedEntrySizeBytes + key.estimatedKeySizeBytes();
     }
 };
 
