@@ -29,19 +29,27 @@
 
 #pragma once
 
+#include <boost/optional/optional.hpp>
+#include <string>
+#include <tuple>
+#include <utility>
+#include <vector>
+
 #include "mongo/db/exec/sbe/expressions/expression.h"
 #include "mongo/db/exec/sbe/stages/stages.h"
+#include "mongo/db/exec/sbe/values/slot.h"
 #include "mongo/db/exec/sbe/values/value.h"
 #include "mongo/db/matcher/expression.h"
 #include "mongo/db/matcher/expression_leaf.h"
-#include "mongo/db/query/sbe_stage_builder_eval_frame.h"
+#include "mongo/db/matcher/expression_visitor.h"
+#include "mongo/db/query/collation/collator_interface.h"
 #include "mongo/db/query/sbe_stage_builder_helpers.h"
 
 namespace mongo::stage_builder {
 class PlanStageSlots;
 
 /**
- * This function generates an EvalExpr that implements the filter expression represented by 'root'.
+ * This function generates an SbExpr that implements the filter expression represented by 'root'.
  * The 'inputSlot' and 'slots' parameters specify the input(s) that the filter should use.
  *
  * The 'isFilterOverIxscan' parameter controls if we should search for kField slots in 'slots' that
@@ -52,28 +60,15 @@ class PlanStageSlots;
  * key field names ('keyFields') that lists the names of all the kField slots that are needed by
  * 'root'.
  *
- * This function returns an EvalExpr. If 'root' is an AND with no children, this function will
- * return a null EvalExpr to indicate that there is no filter condition.
+ * This function returns an SbExpr. If 'root' is an AND with no children, this function will
+ * return a null SbExpr to indicate that there is no filter condition.
  */
-EvalExpr generateFilter(StageBuilderState& state,
-                        const MatchExpression* root,
-                        boost::optional<sbe::value::SlotId> inputSlot,
-                        const PlanStageSlots* slots,
-                        const std::vector<std::string>& keyFields = {},
-                        bool isFilterOverIxscan = false);
-
-/**
- * Converts the list of equalities inside the given $in expression ('expr') into an SBE array, which
- * is returned as a (typeTag, value) pair. The caller owns the resulting value.
- *
- * The returned tuple also includes three booleans, in this order:
- *  - 'hasArray': True if at least one of the values inside the $in equality list is an array.
- *  - 'hasObject': True if at least one of the values inside the $in equality list is an object.
- *  - 'hasNull': True if at least one of the values inside the $in equality list is a literal null
- * value.
- */
-std::tuple<sbe::value::TypeTags, sbe::value::Value, bool, bool, bool> convertInExpressionEqualities(
-    const InMatchExpression* expr, const PlanStageData& data);
+SbExpr generateFilter(StageBuilderState& state,
+                      const MatchExpression* root,
+                      boost::optional<SbSlot> inputSlot,
+                      const PlanStageSlots& slots,
+                      const std::vector<std::string>& keyFields = {},
+                      bool isFilterOverIxscan = false);
 
 /**
  * Converts the list of bit positions inside of any of the bit-test match expressions
@@ -87,24 +82,20 @@ std::pair<sbe::value::TypeTags, sbe::value::Value> convertBitTestBitPositions(
  * The following family of functions convert the given MatchExpression that consumes a single input
  * into an EExpression that consumes the input from the provided slot.
  */
-EvalExpr generateComparisonExpr(StageBuilderState& state,
-                                const ComparisonMatchExpression* expr,
-                                sbe::EPrimBinary::Op binaryOp,
-                                EvalExpr inputExpr);
-EvalExpr generateInExpr(StageBuilderState& state,
-                        const InMatchExpression* expr,
-                        EvalExpr inputExpr);
-EvalExpr generateBitTestExpr(StageBuilderState& state,
-                             const BitTestMatchExpression* expr,
-                             const sbe::BitTestBehavior& bitOp,
-                             EvalExpr inputExpr);
-EvalExpr generateModExpr(StageBuilderState& state,
-                         const ModMatchExpression* expr,
-                         EvalExpr inputExpr);
-EvalExpr generateRegexExpr(StageBuilderState& state,
-                           const RegexMatchExpression* expr,
-                           EvalExpr inputExpr);
-EvalExpr generateWhereExpr(StageBuilderState& state,
-                           const WhereMatchExpression* expr,
-                           EvalExpr inputExpr);
+SbExpr generateComparisonExpr(StageBuilderState& state,
+                              const ComparisonMatchExpression* expr,
+                              sbe::EPrimBinary::Op binaryOp,
+                              SbExpr inputExpr);
+SbExpr generateInExpr(StageBuilderState& state, const InMatchExpression* expr, SbExpr inputExpr);
+SbExpr generateBitTestExpr(StageBuilderState& state,
+                           const BitTestMatchExpression* expr,
+                           const sbe::BitTestBehavior& bitOp,
+                           SbExpr inputExpr);
+SbExpr generateModExpr(StageBuilderState& state, const ModMatchExpression* expr, SbExpr inputExpr);
+SbExpr generateRegexExpr(StageBuilderState& state,
+                         const RegexMatchExpression* expr,
+                         SbExpr inputExpr);
+SbExpr generateWhereExpr(StageBuilderState& state,
+                         const WhereMatchExpression* expr,
+                         SbExpr inputExpr);
 }  // namespace mongo::stage_builder
