@@ -28,34 +28,60 @@
  */
 
 
-#include "mongo/platform/basic.h"
+#include <absl/container/inlined_vector.h>
+#include <absl/container/node_hash_map.h>
+#include <algorithm>
+#include <bitset>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <cstddef>
+#include <deque>
+#include <iterator>
+#include <map>
+#include <string_view>
 
-#include "mongo/db/query/sbe_stage_builder_index_scan.h"
+#include <boost/optional/optional.hpp>
 
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/simple_bsonobj_comparator.h"
 #include "mongo/db/catalog/collection.h"
+#include "mongo/db/catalog/index_catalog.h"
+#include "mongo/db/catalog/index_catalog_entry.h"
+#include "mongo/db/exec/sbe/expressions/expression.h"
+#include "mongo/db/exec/sbe/expressions/runtime_environment.h"
 #include "mongo/db/exec/sbe/stages/branch.h"
-#include "mongo/db/exec/sbe/stages/co_scan.h"
 #include "mongo/db/exec/sbe/stages/filter.h"
-#include "mongo/db/exec/sbe/stages/hash_agg.h"
 #include "mongo/db/exec/sbe/stages/ix_scan.h"
-#include "mongo/db/exec/sbe/stages/limit_skip.h"
 #include "mongo/db/exec/sbe/stages/loop_join.h"
-#include "mongo/db/exec/sbe/stages/makeobj.h"
 #include "mongo/db/exec/sbe/stages/project.h"
-#include "mongo/db/exec/sbe/stages/spool.h"
-#include "mongo/db/exec/sbe/stages/union.h"
 #include "mongo/db/exec/sbe/stages/unique.h"
 #include "mongo/db/exec/sbe/stages/unwind.h"
 #include "mongo/db/index/index_access_method.h"
+#include "mongo/db/index/index_descriptor.h"
+#include "mongo/db/matcher/expression.h"
+#include "mongo/db/matcher/expression_leaf.h"
 #include "mongo/db/matcher/match_expression_dependencies.h"
+#include "mongo/db/pipeline/dependencies.h"
 #include "mongo/db/query/index_bounds_builder.h"
+#include "mongo/db/query/index_entry.h"
+#include "mongo/db/query/interval.h"
+#include "mongo/db/query/interval_evaluation_tree.h"
+#include "mongo/db/query/optimizer/algebra/polyvalue.h"
 #include "mongo/db/query/query_knobs_gen.h"
 #include "mongo/db/query/sbe_stage_builder.h"
 #include "mongo/db/query/sbe_stage_builder_filter.h"
-#include "mongo/db/query/util/make_data_structure.h"
+#include "mongo/db/query/sbe_stage_builder_index_scan.h"
+#include "mongo/db/storage/sorted_data_interface.h"
 #include "mongo/logv2/log.h"
-#include "mongo/util/overloaded_visitor.h"
-#include <boost/optional.hpp>
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/id_generator.h"
+#include "mongo/util/overloaded_visitor.h"  // IWYU pragma: keep
+#include "mongo/util/str.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
@@ -234,12 +260,16 @@ PlanStageSlots buildPlanStageSlots(StageBuilderState& state,
                 sbe::value::copyValue(sbe::value::TypeTags::bsonObject,
                                       sbe::value::bitcastFrom<const char*>(keyPattern.objdata()));
             auto slot =
-                state.data->env->registerSlot(bsonObjTag, bsonObjVal, true, state.slotIdGenerator);
+                state.env->registerSlot(bsonObjTag, bsonObjVal, true, state.slotIdGenerator);
             state.keyPatternToSlotMap[keyPattern] = slot;
             outputs.set(PlanStageSlots::kIndexKeyPattern, slot);
         }
     }
 
+    if (reqs.has(PlanStageSlots::kPrefetchedResult)) {
+        outputs.set(PlanStageSlots::kPrefetchedResult, state.getNothingSlot());
+    }
+
     return outputs;
 }
 
@@ -286,9 +316,9 @@ generateOptimizedMultiIntervalIndexScan(StageBuilderState& state,
     auto boundsSlot = [&] {
         if (intervals) {
             auto [boundsTag, boundsVal] = packIndexIntervalsInSbeArray(std::move(*intervals));
-            return state.data->env->registerSlot(boundsTag, boundsVal, true, state.slotIdGenerator);
+            return state.env->registerSlot(boundsTag, boundsVal, true, state.slotIdGenerator);
         } else {
-            return state.data->env->registerSlot(
+            return state.env->registerSlot(
                 sbe::value::TypeTags::Nothing, 0, true, state.slotIdGenerator);
         }
     }();
@@ -304,11 +334,12 @@ generateOptimizedMultiIntervalIndexScan(StageBuilderState& state,
         false /* Preserve null and empty arrays, in our case it cannot be empty anyway. */,
         planNodeId);
 
-    sbe::value::SlotMap<std::unique_ptr<sbe::EExpression>> projects;
-    projects.emplace(lowKeySlot,
-                     makeFunction("getField"_sd, makeVariable(unwindSlot), makeConstant("l"_sd)));
-    projects.emplace(highKeySlot,
-                     makeFunction("getField"_sd, makeVariable(unwindSlot), makeConstant("h"_sd)));
+    sbe::SlotExprPairVector projects;
+    projects.emplace_back(
+        lowKeySlot, makeFunction("getField"_sd, makeVariable(unwindSlot), makeStrConstant("l"_sd)));
+    projects.emplace_back(
+        highKeySlot,
+        makeFunction("getField"_sd, makeVariable(unwindSlot), makeStrConstant("h"_sd)));
 
     // Add another project stage to extract low and high keys from each value produced by unwind and
     // bind the keys to the 'lowKeySlot' and 'highKeySlot'.
@@ -326,7 +357,6 @@ generateOptimizedMultiIntervalIndexScan(StageBuilderState& state,
         : boost::none;
 
     auto stage = sbe::makeS<sbe::SimpleIndexScanStage>(collection->uuid(),
-                                                       collection->ns().dbName(),
                                                        indexName,
                                                        forward,
                                                        indexKeySlot,
@@ -378,7 +408,7 @@ generateGenericMultiIntervalIndexScan(StageBuilderState& state,
                                       const std::string& indexName,
                                       const IndexScanNode* ixn,
                                       const BSONObj& keyPattern,
-                                      KeyString::Version version,
+                                      key_string::Version version,
                                       Ordering ordering,
                                       sbe::IndexKeysInclusionSet indexKeysToInclude,
                                       sbe::value::SlotVector indexKeySlots,
@@ -391,12 +421,14 @@ generateGenericMultiIntervalIndexScan(StageBuilderState& state,
     std::unique_ptr<sbe::EExpression> boundsExpr;
 
     if (hasDynamicIndexBounds) {
-        boundsSlot.emplace(state.data->env->registerSlot(
+        boundsSlot.emplace(state.env->registerSlot(
             sbe::value::TypeTags::Nothing, 0, true /* owned */, state.slotIdGenerator));
         boundsExpr = makeVariable(*boundsSlot);
     } else {
         // 'sbe::EConstant' will take the ownership of the 'IndexBounds' pointer.
-        boundsExpr = makeConstant(sbe::value::TypeTags::indexBounds, new IndexBounds(ixn->bounds));
+        boundsExpr =
+            makeConstant(sbe::value::TypeTags::indexBounds,
+                         sbe::value::bitcastFrom<IndexBounds*>(new IndexBounds(ixn->bounds)));
     }
 
     sbe::GenericIndexScanStageParams params{
@@ -414,7 +446,6 @@ generateGenericMultiIntervalIndexScan(StageBuilderState& state,
 
     std::unique_ptr<sbe::PlanStage> stage =
         std::make_unique<sbe::GenericIndexScanStage>(collection->uuid(),
-                                                     collection->ns().dbName(),
                                                      indexName,
                                                      std::move(params),
                                                      indexKeySlot,
@@ -439,15 +470,23 @@ generateGenericMultiIntervalIndexScan(StageBuilderState& state,
 }
 
 /**
- * Checks if we can create a single interval index scan plan. Creation of the single interval index
- * scan plans is preferred due to lower query latency as a result of faster plan recovery from the
- * cache. The rule for checking if 'iets' resolve to a single interval is as follows:
+ * Represents the kinds of interval(s) required for an index scan: a single point interval, a single
+ * range interval, or multiple intervals.
+ */
+enum class IntervalsRequired { EqualityInterval, SingleRangeInterval, MultiInterval };
+
+/**
+ * Checks what kind of intervals are required for the index scan plan, i.e., if we can create a
+ * single interval index scan plan. Creation of the single interval index scan plans is preferred
+ * due to lower query latency as a result of faster plan recovery from the cache. The rule for
+ * checking if 'iets' resolve to a single interval is as follows:
  * - an optional sequence of '$eq' or constant point intervals followed by
  * - an optional single interval of a comparison match expression or a constant interval or an
  * intersection of two such nodes followed by
  * - an optional sequence of unbounded intervals [MinKey, MaxKey].
  */
-bool canGenerateSingleIntervalIndexScan(const std::vector<interval_evaluation_tree::IET>& iets) {
+IntervalsRequired canGenerateSingleIntervalIndexScan(
+    const std::vector<interval_evaluation_tree::IET>& iets) {
     // Represents different allowed states while checking if the 'iets' could be represented as a
     // single interval.
     enum class State { EqOrConstPoint, ComparisonOrConstRange, UnboundedInterval };
@@ -479,12 +518,12 @@ bool canGenerateSingleIntervalIndexScan(const std::vector<interval_evaluation_tr
                 } else if (isComparisonOrSingleConst(iet) || isSimpleIntersection) {
                     currentState = State::ComparisonOrConstRange;
                 } else {
-                    return false;
+                    return IntervalsRequired::MultiInterval;
                 }
                 break;
             case State::ComparisonOrConstRange:
                 if (!isMinToMax) {
-                    return false;
+                    return IntervalsRequired::MultiInterval;
                 }
 
                 // Transition to the next state as we allow only one bounded range, after that all
@@ -493,103 +532,78 @@ bool canGenerateSingleIntervalIndexScan(const std::vector<interval_evaluation_tr
                 break;
             case State::UnboundedInterval:
                 if (!isMinToMax) {
-                    return false;
+                    return IntervalsRequired::MultiInterval;
                 }
                 break;
         }
     }
 
-    return true;
+    return currentState == State::EqOrConstPoint ? IntervalsRequired::EqualityInterval
+                                                 : IntervalsRequired::SingleRangeInterval;
 }
 }  // namespace
 
-/**
- * Constructs the most simple version of an index scan from the single interval index bounds.
- *
- * In case when the 'lowKey' and 'highKey' are not specified, slots will be registered for them in
- * the runtime environment and their slot ids returned as a pair in the third element of the tuple.
- *
- * If 'indexKeySlot' is provided, than the corresponding slot will be filled out with each KeyString
- * in the index.
- */
+bool ietsArePointInterval(const std::vector<interval_evaluation_tree::IET>& iets) {
+    return canGenerateSingleIntervalIndexScan(iets) == IntervalsRequired::EqualityInterval;
+}
+
 std::tuple<std::unique_ptr<sbe::PlanStage>,
            PlanStageSlots,
            boost::optional<std::pair<sbe::value::SlotId, sbe::value::SlotId>>>
-generateSingleIntervalIndexScan(StageBuilderState& state,
-                                const CollectionPtr& collection,
-                                const std::string& indexName,
-                                const BSONObj& keyPattern,
-                                bool forward,
-                                std::unique_ptr<KeyString::Value> lowKey,
-                                std::unique_ptr<KeyString::Value> highKey,
-                                sbe::IndexKeysInclusionSet indexKeysToInclude,
-                                sbe::value::SlotVector indexKeySlots,
-                                const PlanStageReqs& reqs,
-                                PlanYieldPolicy* yieldPolicy,
-                                PlanNodeId planNodeId,
-                                bool lowPriority) {
+generateSingleIntervalIndexScanAndSlots(StageBuilderState& state,
+                                        const CollectionPtr& collection,
+                                        const std::string& indexName,
+                                        const BSONObj& keyPattern,
+                                        bool forward,
+                                        std::unique_ptr<key_string::Value> lowKey,
+                                        std::unique_ptr<key_string::Value> highKey,
+                                        sbe::IndexKeysInclusionSet indexKeysToInclude,
+                                        sbe::value::SlotVector indexKeySlots,
+                                        const PlanStageReqs& reqs,
+                                        PlanYieldPolicy* yieldPolicy,
+                                        PlanNodeId planNodeId,
+                                        bool lowPriority,
+                                        bool isPointInterval) {
     auto slotIdGenerator = state.slotIdGenerator;
-    auto recordIdSlot = slotIdGenerator->generate();
     tassert(6584701,
             "Either both lowKey and highKey are specified or none of them are",
             (lowKey && highKey) || (!lowKey && !highKey));
     const bool shouldRegisterLowHighKeyInRuntimeEnv = !lowKey;
 
-    auto lowKeySlot = !lowKey ? boost::make_optional(state.data->env->registerSlot(
+    auto lowKeySlot = !lowKey ? boost::make_optional(state.env->registerSlot(
                                     sbe::value::TypeTags::Nothing, 0, true, slotIdGenerator))
                               : boost::none;
-    auto highKeySlot = !highKey ? boost::make_optional(state.data->env->registerSlot(
+    auto highKeySlot = !highKey ? boost::make_optional(state.env->registerSlot(
                                       sbe::value::TypeTags::Nothing, 0, true, slotIdGenerator))
                                 : boost::none;
 
-    auto lowKeyExpr = !lowKey ? makeVariable(*lowKeySlot)
-                              : makeConstant(sbe::value::TypeTags::ksValue, lowKey.release());
-    auto highKeyExpr = !highKey ? makeVariable(*highKeySlot)
-                                : makeConstant(sbe::value::TypeTags::ksValue, highKey.release());
+    auto lowKeyExpr = !lowKey
+        ? makeVariable(*lowKeySlot)
+        : makeConstant(sbe::value::TypeTags::keyString, sbe::value::makeKeyString(*lowKey).second);
+    auto highKeyExpr = !highKey
+        ? makeVariable(*highKeySlot)
+        : makeConstant(sbe::value::TypeTags::keyString, sbe::value::makeKeyString(*highKey).second);
 
-    auto snapshotIdSlot = reqs.has(PlanStageSlots::kSnapshotId)
-        ? boost::make_optional(slotIdGenerator->generate())
-        : boost::none;
-    auto indexIdentSlot = reqs.has(PlanStageSlots::kIndexIdent)
-        ? boost::make_optional(slotIdGenerator->generate())
-        : boost::none;
-    auto indexKeySlot = reqs.has(PlanStageSlots::kIndexKey)
-        ? boost::make_optional(slotIdGenerator->generate())
-        : boost::none;
-
-    // Scan the index in the range {'lowKeySlot', 'highKeySlot'} (subject to inclusive or
-    // exclusive boundaries), and produce a single field recordIdSlot that can be used to
-    // position into the collection.
-    auto stage = sbe::makeS<sbe::SimpleIndexScanStage>(collection->uuid(),
-                                                       collection->ns().dbName(),
-                                                       indexName,
-                                                       forward,
-                                                       indexKeySlot,
-                                                       recordIdSlot,
-                                                       snapshotIdSlot,
-                                                       indexIdentSlot,
-                                                       indexKeysToInclude,
-                                                       std::move(indexKeySlots),
-                                                       lowKeyExpr->clone(),
-                                                       highKeyExpr->clone(),
-                                                       yieldPolicy,
-                                                       planNodeId,
-                                                       lowPriority);
-
-    auto outputs = buildPlanStageSlots(state,
-                                       reqs,
-                                       indexName,
-                                       keyPattern,
-                                       recordIdSlot,
-                                       snapshotIdSlot,
-                                       indexIdentSlot,
-                                       indexKeySlot);
+    auto [stage, outputs] = generateSingleIntervalIndexScan(state,
+                                                            collection,
+                                                            indexName,
+                                                            keyPattern,
+                                                            lowKeyExpr->clone(),
+                                                            highKeyExpr->clone(),
+                                                            std::move(indexKeysToInclude),
+                                                            std::move(indexKeySlots),
+                                                            reqs,
+                                                            yieldPolicy,
+                                                            planNodeId,
+                                                            forward,
+                                                            lowPriority);
 
     // If low and high keys are provided in the runtime environment, then we need to create
     // a cfilter stage on top of project in order to be sure that the single interval
     // exists (the interval may be empty), in which case the index scan plan should simply
-    // return EOF.
-    if (shouldRegisterLowHighKeyInRuntimeEnv) {
+    // return EOF. This does not apply when the interval is a point interval, since the interval
+    // should always exist in that case.
+    if (shouldRegisterLowHighKeyInRuntimeEnv && !isPointInterval) {
         stage = sbe::makeS<sbe::FilterStage<true /*IsConst*/, false /*IsEof*/>>(
             std::move(stage),
             makeBinaryOp(sbe::EPrimBinary::logicAnd,
@@ -605,6 +619,66 @@ generateSingleIntervalIndexScan(StageBuilderState& state,
                 : boost::none};
 }
 
+std::tuple<std::unique_ptr<sbe::PlanStage>, PlanStageSlots> generateSingleIntervalIndexScan(
+    StageBuilderState& state,
+    const CollectionPtr& collection,
+    const std::string& indexName,
+    const BSONObj& keyPattern,
+    std::unique_ptr<sbe::EExpression> lowKeyExpr,
+    std::unique_ptr<sbe::EExpression> highKeyExpr,
+    sbe::IndexKeysInclusionSet indexKeysToInclude,
+    sbe::value::SlotVector indexKeySlots,
+    const PlanStageReqs& reqs,
+    PlanYieldPolicy* yieldPolicy,
+    PlanNodeId planNodeId,
+    bool forward,
+    bool lowPriority) {
+    auto slotIdGenerator = state.slotIdGenerator;
+    auto recordIdSlot = slotIdGenerator->generate();
+    tassert(7856101,
+            "lowKeyExpr must be present if highKeyExpr is specified.",
+            lowKeyExpr || (!lowKeyExpr && !highKeyExpr));
+
+    auto snapshotIdSlot = reqs.has(PlanStageSlots::kSnapshotId)
+        ? boost::make_optional(slotIdGenerator->generate())
+        : boost::none;
+    auto indexIdentSlot = reqs.has(PlanStageSlots::kIndexIdent)
+        ? boost::make_optional(slotIdGenerator->generate())
+        : boost::none;
+    auto indexKeySlot = reqs.has(PlanStageSlots::kIndexKey)
+        ? boost::make_optional(slotIdGenerator->generate())
+        : boost::none;
+
+    // Scan the index in the range {'lowKeySlot', 'highKeySlot'} (subject to inclusive or
+    // exclusive boundaries), and produce a single field recordIdSlot that can be used to
+    // position into the collection.
+    auto stage = sbe::makeS<sbe::SimpleIndexScanStage>(collection->uuid(),
+                                                       indexName,
+                                                       forward,
+                                                       indexKeySlot,
+                                                       recordIdSlot,
+                                                       snapshotIdSlot,
+                                                       indexIdentSlot,
+                                                       indexKeysToInclude,
+                                                       std::move(indexKeySlots),
+                                                       std::move(lowKeyExpr),
+                                                       std::move(highKeyExpr),
+                                                       yieldPolicy,
+                                                       planNodeId,
+                                                       lowPriority);
+
+    auto outputs = buildPlanStageSlots(state,
+                                       reqs,
+                                       indexName,
+                                       keyPattern,
+                                       recordIdSlot,
+                                       snapshotIdSlot,
+                                       indexIdentSlot,
+                                       indexKeySlot);
+
+    return {std::move(stage), std::move(outputs)};
+}
+
 std::pair<std::unique_ptr<sbe::PlanStage>, PlanStageSlots> generateIndexScan(
     StageBuilderState& state,
     const CollectionPtr& collection,
@@ -612,8 +686,7 @@ std::pair<std::unique_ptr<sbe::PlanStage>, PlanStageSlots> generateIndexScan(
     const sbe::IndexKeysInclusionSet& originalFieldBitset,
     const sbe::IndexKeysInclusionSet& sortKeyBitset,
     PlanYieldPolicy* yieldPolicy,
-    bool doIndexConsistencyCheck,
-    bool needsCorruptionCheck) {
+    const PlanStageReqs& reqs) {
     auto indexName = ixn->index.identifier.catalogName;
     auto descriptor = collection->getIndexCatalog()->findIndexByName(state.opCtx, indexName);
     tassert(5483200,
@@ -648,15 +721,6 @@ std::pair<std::unique_ptr<sbe::PlanStage>, PlanStageSlots> generateIndexScan(
     auto fieldAndSortKeySlots =
         state.slotIdGenerator->generateMultiple(fieldAndSortKeyBitset.count());
 
-    // Generate the various slots needed for a consistency check and/or a corruption check if
-    // requested.
-    PlanStageReqs reqs;
-    reqs.set(PlanStageSlots::kRecordId)
-        .setIf(PlanStageSlots::kSnapshotId, doIndexConsistencyCheck)
-        .setIf(PlanStageSlots::kIndexIdent, doIndexConsistencyCheck)
-        .setIf(PlanStageSlots::kIndexKey, doIndexConsistencyCheck)
-        .setIf(PlanStageSlots::kIndexKeyPattern, needsCorruptionCheck);
-
     PlanStageSlots outputs;
 
     std::unique_ptr<sbe::PlanStage> stage;
@@ -664,20 +728,22 @@ std::pair<std::unique_ptr<sbe::PlanStage>, PlanStageSlots> generateIndexScan(
         // If we have just a single interval, we can construct a simplified sub-tree.
         auto&& [lowKey, highKey] = intervals[0];
 
+        const bool isPointInterval = *lowKey == *highKey;
         std::tie(stage, outputs, std::ignore) =
-            generateSingleIntervalIndexScan(state,
-                                            collection,
-                                            indexName,
-                                            keyPattern,
-                                            ixn->direction == 1,
-                                            std::move(lowKey),
-                                            std::move(highKey),
-                                            fieldAndSortKeyBitset,
-                                            fieldAndSortKeySlots,
-                                            reqs,
-                                            yieldPolicy,
-                                            ixn->nodeId(),
-                                            ixn->lowPriority);
+            generateSingleIntervalIndexScanAndSlots(state,
+                                                    collection,
+                                                    indexName,
+                                                    keyPattern,
+                                                    ixn->direction == 1,
+                                                    std::move(lowKey),
+                                                    std::move(highKey),
+                                                    fieldAndSortKeyBitset,
+                                                    fieldAndSortKeySlots,
+                                                    reqs,
+                                                    yieldPolicy,
+                                                    ixn->nodeId(),
+                                                    ixn->lowPriority,
+                                                    isPointInterval);
     } else if (intervals.size() > 1) {
         // If we were able to decompose multi-interval index bounds into a number of single-interval
         // bounds, we can also built an optimized sub-tree to perform an index scan.
@@ -727,14 +793,16 @@ std::pair<std::unique_ptr<sbe::PlanStage>, PlanStageSlots> generateIndexScan(
     }
 
     if (ixn->shouldDedup) {
-        stage = sbe::makeS<sbe::UniqueStage>(
-            std::move(stage), sbe::makeSV(outputs.get(PlanStageSlots::kRecordId)), ixn->nodeId());
+        stage =
+            sbe::makeS<sbe::UniqueStage>(std::move(stage),
+                                         sbe::makeSV(outputs.get(PlanStageSlots::kRecordId).slotId),
+                                         ixn->nodeId());
     }
 
     if (ixn->filter) {
         const bool isOverIxscan = true;
         auto filterExpr =
-            generateFilter(state, ixn->filter.get(), {}, &outputs, filterFields, isOverIxscan);
+            generateFilter(state, ixn->filter.get(), {}, outputs, filterFields, isOverIxscan);
         if (!filterExpr.isNull()) {
             stage = sbe::makeS<sbe::FilterStage<false>>(
                 std::move(stage), filterExpr.extractExpr(state), ixn->nodeId());
@@ -746,7 +814,7 @@ std::pair<std::unique_ptr<sbe::PlanStage>, PlanStageSlots> generateIndexScan(
 
 IndexIntervals makeIntervalsFromIndexBounds(const IndexBounds& bounds,
                                             bool forward,
-                                            KeyString::Version version,
+                                            key_string::Version version,
                                             Ordering ordering) {
     auto lowKeyInclusive{IndexBounds::isStartIncludedInBound(bounds.boundInclusion)};
     auto highKeyInclusive{IndexBounds::isEndIncludedInBound(bounds.boundInclusion)};
@@ -775,20 +843,8 @@ IndexIntervals makeIntervalsFromIndexBounds(const IndexBounds& bounds,
                     "Generated interval [lowKey, highKey]",
                     "lowKey"_attr = redact(lowKey),
                     "highKey"_attr = redact(highKey));
-        // Note that 'makeKeyFromBSONKeyForSeek()' is intended to compute the "start" key for an
-        // index scan. The logic for computing a "discriminator" for an "end" key is reversed, which
-        // is why we use 'makeKeyStringFromBSONKey()' to manually specify the discriminator for the
-        // end key.
-        result.push_back(
-            {std::make_unique<KeyString::Value>(
-                 IndexEntryComparison::makeKeyStringFromBSONKeyForSeek(
-                     lowKey, version, ordering, forward, lowKeyInclusive)),
-             std::make_unique<KeyString::Value>(IndexEntryComparison::makeKeyStringFromBSONKey(
-                 highKey,
-                 version,
-                 ordering,
-                 forward != highKeyInclusive ? KeyString::Discriminator::kExclusiveBefore
-                                             : KeyString::Discriminator::kExclusiveAfter))});
+        result.push_back(makeKeyStringPair(
+            lowKey, lowKeyInclusive, highKey, highKeyInclusive, version, ordering, forward));
     }
     return result;
 }
@@ -804,12 +860,10 @@ std::pair<sbe::value::TypeTags, sbe::value::Value> packIndexIntervalsInSbeArray(
         auto obj = sbe::value::getObjectView(val);
         sbe::value::ValueGuard guard{tag, val};
         obj->reserve(2);
-        obj->push_back("l"_sd,
-                       sbe::value::TypeTags::ksValue,
-                       sbe::value::bitcastFrom<KeyString::Value*>(lowKey.release()));
-        obj->push_back("h"_sd,
-                       sbe::value::TypeTags::ksValue,
-                       sbe::value::bitcastFrom<KeyString::Value*>(highKey.release()));
+        obj->push_back(
+            "l"_sd, sbe::value::TypeTags::keyString, sbe::value::makeKeyString(*lowKey).second);
+        obj->push_back(
+            "h"_sd, sbe::value::TypeTags::keyString, sbe::value::makeKeyString(*highKey).second);
         guard.reset();
         arr->push_back(tag, val);
     }
@@ -824,8 +878,7 @@ std::pair<std::unique_ptr<sbe::PlanStage>, PlanStageSlots> generateIndexScanWith
     const sbe::IndexKeysInclusionSet& originalFieldBitset,
     const sbe::IndexKeysInclusionSet& sortKeyBitset,
     PlanYieldPolicy* yieldPolicy,
-    bool doIndexConsistencyCheck,
-    bool needsCorruptionCheck) {
+    const PlanStageReqs& reqs) {
     const bool forward = ixn->direction == 1;
     auto indexName = ixn->index.identifier.catalogName;
     auto descriptor = collection->getIndexCatalog()->findIndexByName(state.opCtx, indexName);
@@ -859,36 +912,29 @@ std::pair<std::unique_ptr<sbe::PlanStage>, PlanStageSlots> generateIndexScanWith
     auto outputFieldAndSortKeySlots =
         state.slotIdGenerator->generateMultiple(fieldAndSortKeyBitset.count());
 
-    // Generate the various slots needed for a consistency check and/or a corruption check if
-    // requested.
-    PlanStageReqs reqs;
-    reqs.set(PlanStageSlots::kRecordId)
-        .setIf(PlanStageSlots::kSnapshotId, doIndexConsistencyCheck)
-        .setIf(PlanStageSlots::kIndexIdent, doIndexConsistencyCheck)
-        .setIf(PlanStageSlots::kIndexKey, doIndexConsistencyCheck)
-        .setIf(PlanStageSlots::kIndexKeyPattern, needsCorruptionCheck);
-
     PlanStageSlots outputs;
 
     // Whenever possible we should prefer building simplified single interval index scan plans in
     // order to get the best performance.
-    if (canGenerateSingleIntervalIndexScan(ixn->iets)) {
+    if (auto intervalsRequired = canGenerateSingleIntervalIndexScan(ixn->iets);
+        intervalsRequired != IntervalsRequired::MultiInterval) {
         boost::optional<std::pair<sbe::value::SlotId, sbe::value::SlotId>> indexScanBoundsSlots;
-        std::tie(stage, outputs, indexScanBoundsSlots) =
-            generateSingleIntervalIndexScan(state,
-                                            collection,
-                                            indexName,
-                                            keyPattern,
-                                            forward,
-                                            nullptr,
-                                            nullptr,
-                                            fieldAndSortKeyBitset,
-                                            outputFieldAndSortKeySlots,
-                                            reqs,
-                                            yieldPolicy,
-                                            ixn->nodeId(),
-                                            false /* lowPriority */);
-        recordIdSlot = outputs.get(PlanStageSlots::kRecordId);
+        std::tie(stage, outputs, indexScanBoundsSlots) = generateSingleIntervalIndexScanAndSlots(
+            state,
+            collection,
+            indexName,
+            keyPattern,
+            forward,
+            nullptr,
+            nullptr,
+            fieldAndSortKeyBitset,
+            outputFieldAndSortKeySlots,
+            reqs,
+            yieldPolicy,
+            ixn->nodeId(),
+            false /* lowPriority */,
+            intervalsRequired == IntervalsRequired::EqualityInterval /* isPointInterval */);
+        recordIdSlot = outputs.get(PlanStageSlots::kRecordId).slotId;
         tassert(6484702,
                 "lowKey and highKey runtime environment slots must be present",
                 indexScanBoundsSlots);
@@ -938,9 +984,9 @@ std::pair<std::unique_ptr<sbe::PlanStage>, PlanStageSlots> generateIndexScanWith
         tassert(6335204, "bounds slot for index scan is undefined", optimizedBoundsSlot);
         auto optimizedOutputs = std::move(optimizedOutSlots);
 
-        auto mergeThenElseBranches = [&](const PlanStageSlots::Name name) {
-            genericIndexScanSlots.push_back(genericOutputs.get(name));
-            optimizedIndexScanSlots.push_back(optimizedOutputs.get(name));
+        auto mergeThenElseBranches = [&](const PlanStageSlots::UnownedSlotName name) {
+            genericIndexScanSlots.push_back(genericOutputs.get(name).slotId);
+            optimizedIndexScanSlots.push_back(optimizedOutputs.get(name).slotId);
 
             const auto outputSlot = state.slotId();
             outputs.set(name, outputSlot);
@@ -948,21 +994,27 @@ std::pair<std::unique_ptr<sbe::PlanStage>, PlanStageSlots> generateIndexScanWith
         };
 
         mergeThenElseBranches(PlanStageSlots::kRecordId);
-        recordIdSlot = outputs.get(PlanStageSlots::kRecordId);
+        recordIdSlot = outputs.get(PlanStageSlots::kRecordId).slotId;
 
-        if (doIndexConsistencyCheck) {
+        if (reqs.has(PlanStageSlots::kSnapshotId)) {
             mergeThenElseBranches(PlanStageSlots::kSnapshotId);
+        }
+        if (reqs.has(PlanStageSlots::kIndexIdent)) {
             mergeThenElseBranches(PlanStageSlots::kIndexIdent);
+        }
+        if (reqs.has(PlanStageSlots::kIndexKey)) {
             mergeThenElseBranches(PlanStageSlots::kIndexKey);
         }
-
-        if (needsCorruptionCheck) {
+        if (reqs.has(PlanStageSlots::kIndexKeyPattern)) {
             mergeThenElseBranches(PlanStageSlots::kIndexKeyPattern);
         }
+        if (reqs.has(PlanStageSlots::kPrefetchedResult)) {
+            mergeThenElseBranches(PlanStageSlots::kPrefetchedResult);
+        }
 
         // Generate a branch stage that will either execute an optimized or a generic index scan
         // based on the condition in the slot 'isGenericScanSlot'.
-        auto isGenericScanSlot = state.data->env->registerSlot(
+        auto isGenericScanSlot = state.env->registerSlot(
             sbe::value::TypeTags::Nothing, 0, true /* owned */, state.slotIdGenerator);
         auto isGenericScanCondition = makeVariable(isGenericScanSlot);
         stage = sbe::makeS<sbe::BranchStage>(std::move(genericStage),
@@ -995,14 +1047,16 @@ std::pair<std::unique_ptr<sbe::PlanStage>, PlanStageSlots> generateIndexScanWith
     }
 
     if (ixn->shouldDedup) {
-        stage = sbe::makeS<sbe::UniqueStage>(
-            std::move(stage), sbe::makeSV(outputs.get(PlanStageSlots::kRecordId)), ixn->nodeId());
+        stage =
+            sbe::makeS<sbe::UniqueStage>(std::move(stage),
+                                         sbe::makeSV(outputs.get(PlanStageSlots::kRecordId).slotId),
+                                         ixn->nodeId());
     }
 
     if (ixn->filter) {
         const bool isOverIxscan = true;
         auto filterExpr =
-            generateFilter(state, ixn->filter.get(), {}, &outputs, filterFields, isOverIxscan);
+            generateFilter(state, ixn->filter.get(), {}, outputs, filterFields, isOverIxscan);
         if (!filterExpr.isNull()) {
             stage = sbe::makeS<sbe::FilterStage<false>>(
                 std::move(stage), filterExpr.extractExpr(state), ixn->nodeId());
@@ -1014,7 +1068,7 @@ std::pair<std::unique_ptr<sbe::PlanStage>, PlanStageSlots> generateIndexScanWith
                                   accessMethod->getSortedDataInterface()->getKeyStringVersion(),
                                   accessMethod->getSortedDataInterface()->getOrdering(),
                                   ixn->direction,
-                                  std::move(ixn->iets),
+                                  ixn->iets,
                                   std::move(parameterizedScanSlots)});
 
     return {std::move(stage), std::move(outputs)};
