@@ -31,18 +31,45 @@
  * This file contains tests for building execution stages that implement $lookup operator.
  */
 
+#include <cstddef>
+#include <iostream>
+#include <memory>
+#include <string>
+#include <type_traits>
+#include <utility>
+#include <vector>
+
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/json.h"
+#include "mongo/db/catalog/collection_options.h"
 #include "mongo/db/catalog/collection_write_path.h"
-#include "mongo/db/exec/sbe/sbe_plan_stage_test.h"
-#include "mongo/db/exec/sbe/stages/loop_join.h"
-#include "mongo/db/pipeline/document_source_lookup.h"
-#include "mongo/db/pipeline/expression_context.h"
-#include "mongo/db/pipeline/expression_context_for_test.h"
+#include "mongo/db/catalog_raii.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
+#include "mongo/db/exec/document_value/document.h"
+#include "mongo/db/exec/document_value/value.h"
+#include "mongo/db/exec/sbe/expressions/compile_ctx.h"
+#include "mongo/db/exec/sbe/stages/stages.h"
+#include "mongo/db/exec/sbe/util/debug_print.h"
+#include "mongo/db/exec/sbe/values/slot.h"
+#include "mongo/db/exec/sbe/values/value.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/query/collation/collator_interface.h"
+#include "mongo/db/query/multiple_collection_accessor.h"
 #include "mongo/db/query/query_solution.h"
-#include "mongo/db/query/query_test_service_context.h"
+#include "mongo/db/query/sbe_stage_builder.h"
 #include "mongo/db/query/sbe_stage_builder_test_fixture.h"
-#include "mongo/db/repl/replication_coordinator_mock.h"
-#include "mongo/db/repl/storage_interface_impl.h"
-#include "mongo/util/assert_util.h"
+#include "mongo/db/query/shard_filterer_factory_interface.h"
+#include "mongo/db/repl/oplog.h"
+#include "mongo/db/repl/storage_interface.h"
+#include "mongo/db/storage/key_string.h"
+#include "mongo/db/storage/write_unit_of_work.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/uuid.h"
 
 namespace mongo::sbe {
 namespace {
@@ -104,7 +131,7 @@ public:
                                     const std::string& asKey) {
         // Documents from the local collection are provided using collection scan.
         auto localScanNode = std::make_unique<CollectionScanNode>();
-        localScanNode->name = _nss.toString();
+        localScanNode->nss = _nss;
 
         // Construct logical query solution.
         auto lookupNode = std::make_unique<EqLookupNode>(std::move(localScanNode),
@@ -132,7 +159,7 @@ public:
         auto ctx = makeCompileCtx();
         prepareTree(ctx.get(), stage.get());
 
-        auto resultSlot = data.outputs.get(stage_builder::PlanStageSlots::kResult);
+        auto resultSlot = *data.staticData->resultSlot;
         SlotAccessor* resultSlotAccessor = stage->getAccessor(*ctx, resultSlot);
 
         return CompiledTree{std::move(stage), std::move(data), std::move(ctx), resultSlotAccessor};
@@ -150,16 +177,18 @@ public:
                       << std::endl;
         }
 
-        AutoGetCollection localColl(
-            operationContext(),
-            _nss,
-            LockMode::MODE_IS,
-            AutoGetCollection::Options{}.secondaryNssOrUUIDs({_foreignNss}));
+        std::vector<NamespaceStringOrUUID> foreignNssVec({_foreignNss});
+
+        AutoGetCollection localColl(operationContext(),
+                                    _nss,
+                                    LockMode::MODE_IS,
+                                    AutoGetCollection::Options{}.secondaryNssOrUUIDs(
+                                        foreignNssVec.cbegin(), foreignNssVec.cend()));
 
         MultipleCollectionAccessor colls(operationContext(),
                                          &localColl.getCollection(),
                                          _nss,
-                                         false /* isAnySecondaryNamespaceAViewOrSharded */,
+                                         false /* isAnySecondaryNamespaceAViewOrNotFullyLocal */,
                                          {_foreignNss});
 
         auto tree = buildLookupSbeTree(strategy, colls, localKey, foreignKey, asKey);
@@ -169,8 +198,7 @@ public:
         for (auto state = stage->getNext(); state == PlanState::ADVANCED;
              state = stage->getNext(), i++) {
             // Retrieve the result document from SBE plan.
-            auto [resultTag, resultValue] = tree.resultSlotAccessor->copyOrMoveValue();
-            ValueGuard resultGuard{resultTag, resultValue};
+            auto [resultTag, resultValue] = tree.resultSlotAccessor->getViewOfValue();
             if (enableDebugOutput) {
                 std::cout << "Actual document:   " << std::make_pair(resultTag, resultValue)
                           << std::endl;
