@@ -31,22 +31,43 @@
 
 #include "mongo/db/exec/sbe/expressions/expression.h"
 #include "mongo/db/exec/sbe/stages/stages.h"
+#include "mongo/db/exec/sbe/values/slot.h"
 #include "mongo/db/exec/sbe/values/value.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/query/projection.h"
-#include "mongo/db/query/sbe_stage_builder_eval_frame.h"
 #include "mongo/db/query/sbe_stage_builder_helpers.h"
 
 namespace mongo::stage_builder {
 class PlanStageSlots;
 
 /**
- * Generates an EvalExpr implementing a query projection. The 'inputSlot' defines a slot to read
+ * Generates an SbExpr implementing a query projection. The 'inputSlot' defines a slot to read
  * the input document from. 'slots' can optionaly be provided as well so that generateExpression()
  * can make use of kField slots when appropriate.
  */
-EvalExpr generateProjection(StageBuilderState& state,
-                            const projection_ast::Projection* proj,
-                            sbe::value::SlotId inputSlot,
-                            const PlanStageSlots* slots = nullptr);
+SbExpr generateProjection(StageBuilderState& state,
+                          const projection_ast::Projection* projection,
+                          SbExpr inputExpr,
+                          const PlanStageSlots* slots = nullptr);
+
+SbExpr generateProjection(StageBuilderState& state,
+                          projection_ast::ProjectType type,
+                          std::vector<std::string> paths,
+                          std::vector<ProjectNode> nodes,
+                          SbExpr inputExpr,
+                          const PlanStageSlots* slots = nullptr);
+
+SbExpr generateProjectionWithInputFields(StageBuilderState& state,
+                                         const projection_ast::Projection* projection,
+                                         SbExpr inputExpr,
+                                         const sbe::MakeObjInputPlan& inputPlan,
+                                         const PlanStageSlots* slots);
+
+SbExpr generateProjectionWithInputFields(StageBuilderState& state,
+                                         projection_ast::ProjectType type,
+                                         std::vector<std::string> paths,
+                                         std::vector<ProjectNode> nodes,
+                                         SbExpr inputExpr,
+                                         const sbe::MakeObjInputPlan& inputPlan,
+                                         const PlanStageSlots* slots);
 }  // namespace mongo::stage_builder
