@@ -27,15 +27,27 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <absl/container/inlined_vector.h>
+#include <boost/optional/optional.hpp>
 
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
 #include "mongo/db/catalog/collection.h"
 #include "mongo/db/catalog/collection_mock.h"
-#include "mongo/db/matcher/extensions_callback_noop.h"
+#include "mongo/db/exec/sbe/expressions/expression.h"
+#include "mongo/db/exec/sbe/expressions/runtime_environment.h"
+#include "mongo/db/exec/sbe/values/value.h"
+#include "mongo/db/exec/shard_filterer.h"
+#include "mongo/db/keypattern.h"
 #include "mongo/db/pipeline/expression_context_for_test.h"
-#include "mongo/db/query/mock_yield_policies.h"
+#include "mongo/db/query/canonical_query.h"
+#include "mongo/db/query/find_command.h"
 #include "mongo/db/query/sbe_stage_builder.h"
 #include "mongo/db/query/sbe_stage_builder_test_fixture.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/util/intrusive_counter.h"
 
 namespace mongo {
 std::unique_ptr<QuerySolution> SbeStageBuilderTestFixture::makeQuerySolution(
@@ -51,35 +63,37 @@ std::tuple<sbe::value::SlotVector,
            boost::intrusive_ptr<ExpressionContext>>
 SbeStageBuilderTestFixture::buildPlanStage(
     std::unique_ptr<QuerySolution> querySolution,
+    MultipleCollectionAccessor& colls,
     bool hasRecordId,
     std::unique_ptr<ShardFiltererFactoryInterface> shardFiltererInterface,
     std::unique_ptr<CollatorInterface> collator) {
     auto findCommand = std::make_unique<FindCommandRequest>(_nss);
     const boost::intrusive_ptr<ExpressionContext> expCtx(
-        new ExpressionContextForTest(opCtx(), _nss, std::move(collator)));
-    auto statusWithCQ =
-        CanonicalQuery::canonicalize(opCtx(), std::move(findCommand), false, expCtx);
-    ASSERT_OK(statusWithCQ.getStatus());
+        new ExpressionContextForTest(operationContext(), _nss, std::move(collator)));
+    auto cq = std::make_unique<CanonicalQuery>(CanonicalQueryParams{
+        .expCtx = expCtx, .parsedFind = ParsedFindCommandParams{std::move(findCommand)}});
+    if (hasRecordId) {
+        // Force the builder to generate the RecordId output even if it isn't needed by the plan.
+        cq->setForceGenerateRecordId(true);
+    }
 
     CollectionMock coll(_nss);
     CollectionPtr collPtr(&coll);
-    MultipleCollectionAccessor& colls = _collections;
     if (shardFiltererInterface) {
-        auto shardFilterer = shardFiltererInterface->makeShardFilterer(opCtx());
+        auto shardFilterer = shardFiltererInterface->makeShardFilterer(operationContext());
         collPtr.setShardKeyPattern(shardFilterer->getKeyPattern().toBSON());
         colls = MultipleCollectionAccessor(collPtr);
     }
 
     stage_builder::SlotBasedStageBuilder builder{
-        opCtx(), colls, *statusWithCQ.getValue(), *querySolution, nullptr /* YieldPolicy */};
+        operationContext(), colls, *cq, *querySolution, getYieldPolicy()};
 
-    auto stage = builder.build(querySolution->root());
-    auto data = builder.getPlanStageData();
+    auto [stage, data] = builder.build(querySolution->root());
 
     // Reset "shardFilterer".
     if (auto shardFiltererSlot = data.env->getSlotIfExists("shardFilterer"_sd);
         shardFiltererSlot && shardFiltererInterface) {
-        auto shardFilterer = shardFiltererInterface->makeShardFilterer(opCtx());
+        auto shardFilterer = shardFiltererInterface->makeShardFilterer(operationContext());
         data.env->resetSlot(*shardFiltererSlot,
                             sbe::value::TypeTags::shardFilterer,
                             sbe::value::bitcastFrom<ShardFilterer*>(shardFilterer.release()),
@@ -88,9 +102,9 @@ SbeStageBuilderTestFixture::buildPlanStage(
 
     auto slots = sbe::makeSV();
     if (hasRecordId) {
-        slots.push_back(data.outputs.get(stage_builder::PlanStageSlots::kRecordId));
+        slots.push_back(*data.staticData->recordIdSlot);
     }
-    slots.push_back(data.outputs.get(stage_builder::PlanStageSlots::kResult));
+    slots.push_back(*data.staticData->resultSlot);
 
     // 'expCtx' owns the collator and a collator slot is registered into the runtime environment
     // while creating 'builder'. So, the caller should retain the 'expCtx' until the execution is
