@@ -28,12 +28,28 @@
  */
 #pragma once
 
+#include <algorithm>
+#include <boost/move/utility_core.hpp>
+#include <boost/optional.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <cstddef>
+#include <set>
+#include <utility>
 #include <vector>
 
+#include "mongo/bson/bsonobj.h"
 #include "mongo/db/exec/document_value/document.h"
+#include "mongo/db/exec/document_value/document_metadata_fields.h"
+#include "mongo/db/pipeline/dependencies.h"
 #include "mongo/db/pipeline/document_path_support.h"
 #include "mongo/db/pipeline/expression.h"
+#include "mongo/db/pipeline/expression_context.h"
+#include "mongo/db/pipeline/field_path.h"
 #include "mongo/db/query/query_shape/serialization_options.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/intrusive_counter.h"
+#include "mongo/util/str.h"
 
 namespace mongo {
 class SortPattern {
@@ -65,8 +81,13 @@ public:
 
     SortPattern(std::vector<SortPatternPart> patterns) : _sortPattern(std::move(patterns)) {
         for (auto&& patternPart : _sortPattern) {
-            if (patternPart.fieldPath)
-                _paths.insert(patternPart.fieldPath->fullPath());
+            if (patternPart.fieldPath) {
+                const auto [_, inserted] = _paths.insert(patternPart.fieldPath->fullPath());
+                uassert(7472501,
+                        str::stream() << "$sort key must not contain duplicate keys (field: '"
+                                      << patternPart.fieldPath->fullPath() << "')",
+                        inserted);
+            }
         }
     }
 
