@@ -44,14 +44,17 @@ using PlanNodeId = uint32_t;
 static constexpr PlanNodeId kEmptyPlanNodeId = 0u;
 
 /**
- * These map to implementations of the PlanStage interface, all of which live in db/exec/
+ * These map to implementations of the PlanStage interface, all of which live in db/exec/. These
+ * stage types are shared between Classic and SBE.
  */
 enum StageType {
     STAGE_AND_HASH,
     STAGE_AND_SORTED,
     STAGE_BATCHED_DELETE,
     STAGE_CACHED_PLAN,
+
     STAGE_COLLSCAN,
+
     STAGE_COLUMN_SCAN,
 
     // A virtual scan stage that simulates a collection scan and doesn't depend on underlying
@@ -85,7 +88,7 @@ enum StageType {
 
     STAGE_IXSCAN,
     STAGE_LIMIT,
-
+    STAGE_MATCH,
     STAGE_MOCK,
 
     // Implements iterating over one or more RecordStore::Cursor.
@@ -101,6 +104,7 @@ enum StageType {
 
     STAGE_QUEUED_DATA,
     STAGE_RECORD_STORE_FAST_COUNT,
+    STAGE_REPLACE_ROOT,
     STAGE_RETURN_KEY,
     STAGE_SAMPLE_FROM_TIMESERIES_BUCKET,
     STAGE_SHARDING_FILTER,
@@ -128,14 +132,22 @@ enum StageType {
 
     STAGE_UNKNOWN,
 
-    STAGE_UNPACK_TIMESERIES_BUCKET,
+    // Stage for 'UnpackTimeseriesBucket' which is only used for $sample on a time-series bucket
+    // collection.
+    STAGE_UNPACK_SAMPLED_TS_BUCKET,
 
+    STAGE_UNWIND,
     STAGE_UPDATE,
 
     // Stages for DocumentSources.
     STAGE_GROUP,
     STAGE_EQ_LOOKUP,
+    STAGE_EQ_LOOKUP_UNWIND,
+    STAGE_SEARCH,
+    STAGE_WINDOW,
     STAGE_SENTINEL,
+    // Stage for the DocumentSource to unpack timeseries buckets.
+    STAGE_UNPACK_TS_BUCKET,
 };
 
 inline bool isProjectionStageType(StageType stageType) {
@@ -159,5 +171,8 @@ inline bool isSortStageType(StageType stageType) {
     }
 }
 
-StringData stageTypeToString(StageType stageType);
+struct QuerySolutionNode;
+
+StringData nodeStageTypeToString(const QuerySolutionNode* node);
+
 }  // namespace mongo
