@@ -27,10 +27,17 @@
  *    it in the license file.
  */
 #include "mongo/db/query/util/memory_util.h"
-#include "mongo/logv2/log.h"
+
+#include <algorithm>
+
+#include <boost/move/utility_core.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/base/string_data.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/pcre.h"
 #include "mongo/util/processinfo.h"
-#include <cstddef>
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
