@@ -27,15 +27,37 @@
  *    it in the license file.
  */
 
+
+#include <absl/container/node_hash_map.h>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional.hpp>
+#include <boost/optional/optional.hpp>
+#include <initializer_list>
+#include <list>
+#include <memory>
+#include <string>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/db/logical_time.h"
+#include "mongo/db/read_write_concern_defaults.h"
+#include "mongo/db/read_write_concern_provenance.h"
+#include "mongo/db/read_write_concern_provenance_base_gen.h"
+#include "mongo/db/storage/recovery_unit.h"
+#include "mongo/db/transaction_resources.h"
+#include "mongo/db/vector_clock.h"
+#include "mongo/idl/idl_parser.h"
+#include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/util/clock_source.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/future.h"
+#include "mongo/util/str.h"
+
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/read_write_concern_defaults.h"
-#include "mongo/db/repl/repl_server_parameters_gen.h"
-#include "mongo/db/server_options.h"
-#include "mongo/db/vector_clock.h"
-#include "mongo/logv2/log.h"
 
 namespace mongo {
 namespace {
@@ -88,7 +110,8 @@ void ReadWriteConcernDefaults::checkSuitabilityAsDefault(const ReadConcern& rc)
             !rc.getProvenance().hasSource());
 }
 
-void ReadWriteConcernDefaults::checkSuitabilityAsDefault(const WriteConcern& wc) {
+void ReadWriteConcernDefaults::checkSuitabilityAsDefault(const WriteConcern& wc,
+                                                         bool writeConcernMajorityShouldJournal) {
     uassert(ErrorCodes::BadValue,
             "Unacknowledged write concern is not suitable for the default write concern",
             !wc.isUnacknowledged());
@@ -96,6 +119,15 @@ void ReadWriteConcernDefaults::checkSuitabilityAsDefault(const WriteConcern& wc)
             str::stream() << "'" << ReadWriteConcernProvenance::kSourceFieldName
                           << "' must be unset in default write concern",
             !wc.getProvenance().hasSource());
+    if (writeConcernMajorityShouldJournal && wc.syncMode == WriteConcern::SyncMode::NONE &&
+        wc.isMajority()) {
+        LOGV2_WARNING(
+            8668501,
+            "Default write concern mode is majority but non-journaled, but the configuration has "
+            "'writeConcernMajorityJournalDefault' enabled.  The write concern journal setting will "
+            "be ignored; writes with default write concern will be journaled.",
+            "writeConcern"_attr = wc);
+    }
 }
 
 RWConcernDefault ReadWriteConcernDefaults::generateNewCWRWCToBeSavedOnDisk(
@@ -120,7 +152,8 @@ RWConcernDefault ReadWriteConcernDefaults::generateNewCWRWCToBeSavedOnDisk(
         rwc.setDefaultReadConcern(rc);
     }
     if (wc && !wc->usedDefaultConstructedWC) {
-        checkSuitabilityAsDefault(*wc);
+        auto const replCoord = repl::ReplicationCoordinator::get(opCtx);
+        checkSuitabilityAsDefault(*wc, replCoord->getWriteConcernMajorityShouldJournal());
         rwc.setDefaultWriteConcern(wc);
     }
 
@@ -167,14 +200,14 @@ void ReadWriteConcernDefaults::observeDirectWriteToConfigSettings(OperationConte
     // no new defaults document and the RWConcern will be default constructed, which matches the
     // behavior when lookup discovers a non-existent defaults document.
     auto newDefaultsDoc = newDoc
-        ? RWConcernDefault::parse(IDLParserErrorContext("RWDefaultsWriteObserver"),
-                                  newDoc->getOwned())
+        ? RWConcernDefault::parse(IDLParserContext("RWDefaultsWriteObserver"), newDoc->getOwned())
         : RWConcernDefault();
 
-    opCtx->recoveryUnit()->onCommit([this, opCtx, newDefaultsDoc = std::move(newDefaultsDoc)](
-                                        boost::optional<Timestamp> unusedCommitTime) mutable {
-        setDefault(opCtx, std::move(newDefaultsDoc));
-    });
+    shard_role_details::getRecoveryUnit(opCtx)->onCommit(
+        [this, newDefaultsDoc = std::move(newDefaultsDoc)](OperationContext* opCtx,
+                                                           boost::optional<Timestamp>) mutable {
+            setDefault(opCtx, std::move(newDefaultsDoc));
+        });
 }
 
 void ReadWriteConcernDefaults::invalidate() {
@@ -210,8 +243,8 @@ void ReadWriteConcernDefaults::refreshIfNecessary(OperationContext* opCtx) {
         // Log only if we updated the read- or write-concern defaults themselves.
         if (defaultsBefore.getDefaultWriteConcern() != defaultsAfter.getDefaultWriteConcern() ||
             (defaultsBefore.getDefaultReadConcern() && defaultsAfter.getDefaultReadConcern() &&
-             (defaultsBefore.getDefaultReadConcern().get().getLevel() !=
-              defaultsAfter.getDefaultReadConcern().get().getLevel()))) {
+             (defaultsBefore.getDefaultReadConcern().value().getLevel() !=
+              defaultsAfter.getDefaultReadConcern().value().getLevel()))) {
             LOGV2(20997, "Refreshed RWC defaults", "newDefaults"_attr = possibleNewDefaultsBSON);
         }
     }
@@ -241,7 +274,7 @@ ReadWriteConcernDefaults::RWConcernDefaultAndTime ReadWriteConcernDefaults::getD
 
     // Only overwrite the default read concern and its source if it has already been set on mongos.
     if (!cached.getDefaultReadConcernSource()) {
-        if (!cached.getDefaultReadConcern() || cached.getDefaultReadConcern().get().isEmpty()) {
+        if (!cached.getDefaultReadConcern() || cached.getDefaultReadConcern().value().isEmpty()) {
             auto rcDefault = getImplicitDefaultReadConcern();
             cached.setDefaultReadConcern(rcDefault);
             cached.setDefaultReadConcernSource(DefaultReadConcernSourceEnum::kImplicit);
@@ -258,13 +291,12 @@ ReadWriteConcernDefaults::RWConcernDefaultAndTime ReadWriteConcernDefaults::getD
     // already been set through the config server.
     if (!cached.getDefaultWriteConcernSource()) {
         const bool isCWWCSet = cached.getDefaultWriteConcern() &&
-            !cached.getDefaultWriteConcern().get().usedDefaultConstructedWC;
+            !cached.getDefaultWriteConcern().value().usedDefaultConstructedWC;
         if (isCWWCSet) {
             cached.setDefaultWriteConcernSource(DefaultWriteConcernSourceEnum::kGlobal);
         } else {
             cached.setDefaultWriteConcernSource(DefaultWriteConcernSourceEnum::kImplicit);
-            if (_implicitDefaultWriteConcernMajority &&
-                _implicitDefaultWriteConcernMajority.get()) {
+            if (_implicitDefaultWriteConcernMajority.loadRelaxed()) {
                 cached.setDefaultWriteConcern(
                     WriteConcernOptions(WriteConcernOptions::kMajority,
                                         WriteConcernOptions::SyncMode::UNSET,
@@ -278,13 +310,14 @@ ReadWriteConcernDefaults::RWConcernDefaultAndTime ReadWriteConcernDefaults::getD
 
 void ReadWriteConcernDefaults::setImplicitDefaultWriteConcernMajority(
     bool newImplicitDefaultWCMajority) {
-    invariant(!_implicitDefaultWriteConcernMajority ||
-              repl::enableDefaultWriteConcernUpdatesForInitiate.load());
-    _implicitDefaultWriteConcernMajority = newImplicitDefaultWCMajority;
+    LOGV2(7063400,
+          "Updating implicit default writeConcern majority",
+          "newImplicitDefaultWCMajority"_attr = newImplicitDefaultWCMajority);
+    _implicitDefaultWriteConcernMajority.store(newImplicitDefaultWCMajority);
 }
 
-boost::optional<bool> ReadWriteConcernDefaults::getImplicitDefaultWriteConcernMajority_forTest() {
-    return _implicitDefaultWriteConcernMajority;
+bool ReadWriteConcernDefaults::getImplicitDefaultWriteConcernMajority_forTest() {
+    return _implicitDefaultWriteConcernMajority.loadRelaxed();
 }
 
 boost::optional<ReadWriteConcernDefaults::ReadConcern>
@@ -302,9 +335,9 @@ ReadWriteConcernDefaults::getDefaultWriteConcern(OperationContext* opCtx) {
 boost::optional<ReadWriteConcernDefaults::WriteConcern> ReadWriteConcernDefaults::getCWWC(
     OperationContext* opCtx) {
     auto cached = _getDefaultCWRWCFromDisk(opCtx);
-    if (cached && cached.get().getDefaultWriteConcern() &&
-        !cached.get().getDefaultWriteConcern().get().usedDefaultConstructedWC) {
-        return cached.get().getDefaultWriteConcern().get();
+    if (cached && cached.value().getDefaultWriteConcern() &&
+        !cached.value().getDefaultWriteConcern().value().usedDefaultConstructedWC) {
+        return cached.value().getDefaultWriteConcern().value();
     }
 
     return boost::none;
@@ -333,7 +366,7 @@ ReadWriteConcernDefaults::ReadWriteConcernDefaults(ServiceContext* service,
 
           return options;
       }()),
-      _implicitDefaultWriteConcernMajority(boost::none) {
+      _implicitDefaultWriteConcernMajority(false) {
     _threadPool.startup();
 }
 
@@ -342,13 +375,14 @@ ReadWriteConcernDefaults::~ReadWriteConcernDefaults() = default;
 ReadWriteConcernDefaults::Cache::Cache(ServiceContext* service,
                                        ThreadPoolInterface& threadPool,
                                        FetchDefaultsFn fetchDefaultsFn)
-    : ReadThroughCache(_mutex,
-                       service,
-                       threadPool,
-                       [this](OperationContext* opCtx, Type, const ValueHandle& unusedCachedValue) {
-                           return LookupResult(lookup(opCtx));
-                       },
-                       1 /* cacheSize */),
+    : ReadThroughCache(
+          _mutex,
+          service->getService(),
+          threadPool,
+          [this](OperationContext* opCtx, Type, const ValueHandle& unusedCachedValue) {
+              return LookupResult(lookup(opCtx));
+          },
+          1 /* cacheSize */),
       _fetchDefaultsFn(std::move(fetchDefaultsFn)) {}
 
 boost::optional<RWConcernDefault> ReadWriteConcernDefaults::Cache::lookup(OperationContext* opCtx) {
