@@ -29,6 +29,8 @@
 
 #pragma once
 
+#include <array>
+#include <boost/container_hash/extensions.hpp>
 #include <boost/functional/hash.hpp>
 #include <boost/optional.hpp>
 #include <climits>
@@ -37,11 +39,23 @@
 #include <cstdint>
 #include <cstring>
 #include <fmt/format.h>
+#include <limits>
+#include <new>
 #include <ostream>
+#include <string>
+#include <string_view>
 #include <type_traits>
+#include <utility>
 
+#include "mongo/base/data_type_endian.h"
+#include "mongo/base/error_codes.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
 #include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
 #include "mongo/bson/util/builder.h"
+#include "mongo/bson/util/builder_fwd.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/bufreader.h"
 #include "mongo/util/hex.h"
 #include "mongo/util/shared_buffer.h"
@@ -55,12 +69,11 @@ class RecordIdChecks;
 /**
  * The key that uniquely identifies a Record in a Collection or RecordStore.
  */
-#pragma pack(push, 1)
 class alignas(int64_t) RecordId {
     // The alignas is necessary in order to comply with memory alignment. Internally we're using
     // 8-byte aligned data members (int64_t / ConstSharedBuffer) but as we're packing the structure
-    // the compiler will set the alignment to 1 due to the pragma so we must correct its alignment
-    // information for users of the class.
+    // the compiler will set the alignment to 1 so we must correct its alignment information for
+    // users of the class.
 
     // Class used for static assertions that can only happen when RecordId is completely defined.
     friend class details::RecordIdChecks;
@@ -90,92 +103,39 @@ public:
         return RecordId(kMaxRepr);
     }
 
-    RecordId() : _format(Format::kNull){};
+    RecordId() : _format(Format::kNull) {}
 
     ~RecordId() {
         if (_format == Format::kBigStr) {
-            _data.heapStr.buffer.~ConstSharedBuffer();
+            HeapStr::getBufferFrom(_data).~ConstSharedBuffer();
         }
     }
 
-    RecordId(RecordId&& other) {
-        switch (other._format) {
-            case kNull:
-                break;
-            case kLong:
-                _data.longId.id = other._data.longId.id;
-                break;
-            case kSmallStr:
-                _data.inlineStr = other._data.inlineStr;
-                break;
-            case kBigStr:
-                new (&_data.heapStr.buffer)
-                    ConstSharedBuffer(std::move(other._data.heapStr.buffer));
-                break;
-        }
-        _format = other._format;
-        other._format = Format::kNull;
-    };
+    RecordId(RecordId&& other) : _format(other._format), _data(other._data) {
+        other._format = kNull;
+    }
 
-    RecordId(const RecordId& other) {
-        switch (other._format) {
-            case kNull:
-                break;
-            case kLong:
-                _data.longId.id = other._data.longId.id;
-                break;
-            case kSmallStr:
-                _data.inlineStr = other._data.inlineStr;
-                break;
-            case kBigStr:
-                new (&_data.heapStr.buffer) ConstSharedBuffer(other._data.heapStr.buffer);
-                break;
+    RecordId(const RecordId& other) : _format(other._format), _data(other._data) {
+        if (_format == Format::kBigStr) {
+            // Re-initialize the SharedBuffer to get the correct reference count.
+            auto* buffer = &HeapStr::getBufferFrom(_data);
+            new (buffer) ConstSharedBuffer(HeapStr::getBufferFrom(other._data));
         }
-        _format = other._format;
     };
 
     RecordId& operator=(const RecordId& other) {
-        if (_format == Format::kBigStr) {
-            _data.heapStr.buffer.~ConstSharedBuffer();
-        }
-        switch (other._format) {
-            case kNull:
-                break;
-            case kLong:
-                _data.longId.id = other._data.longId.id;
-                break;
-            case kSmallStr:
-                _data.inlineStr = other._data.inlineStr;
-                break;
-            case kBigStr:
-                new (&_data.heapStr.buffer) ConstSharedBuffer(other._data.heapStr.buffer);
-                break;
-        }
-        _format = other._format;
+        RecordId tmp{other};
+        *this = std::move(tmp);
         return *this;
-    };
+    }
 
 
     RecordId& operator=(RecordId&& other) {
         if (_format == Format::kBigStr) {
-            _data.heapStr.buffer.~ConstSharedBuffer();
+            HeapStr::getBufferFrom(_data).~ConstSharedBuffer();
         }
-        switch (other._format) {
-            case kNull:
-                break;
-            case kLong:
-                _data.longId.id = other._data.longId.id;
-                break;
-            case kSmallStr:
-                _data.inlineStr = other._data.inlineStr;
-                break;
-            case kBigStr:
-                new (&_data.heapStr.buffer)
-                    ConstSharedBuffer(std::move(other._data.heapStr.buffer));
-                break;
-        }
-        _format = other._format;
-        other._format = Format::kNull;
+        std::memcpy(reinterpret_cast<void*>(this), &other, sizeof(RecordId));
+        other._format = kNull;
         return *this;
     }
 
@@ -185,7 +145,7 @@ public:
      */
     explicit RecordId(int64_t s) {
         _format = Format::kLong;
-        _data.longId.id = s;
+        LongId::getIdFrom(_data) = s;
     }
 
     /**
@@ -200,13 +160,15 @@ public:
             size <= kBigStrMaxSize);
         if (size <= kSmallStrMaxSize) {
             _format = Format::kSmallStr;
-            _data.inlineStr.size = static_cast<uint8_t>(size);
-            std::memcpy(_data.inlineStr.dataArr.data(), str, size);
+            InlineStr::getSizeFrom(_data) = static_cast<uint8_t>(size);
+            auto& arr = InlineStr::getArrayFrom(_data);
+            std::memcpy(arr.data(), str, size);
         } else {
             _format = Format::kBigStr;
             auto buffer = SharedBuffer::allocate(size);
             std::memcpy(buffer.get(), str, size);
-            new (&_data.heapStr.buffer) ConstSharedBuffer(std::move(buffer));
+            auto* bufferPtr = &HeapStr::getBufferFrom(_data);
+            new (bufferPtr) ConstSharedBuffer(std::move(buffer));
         }
     }
 
@@ -284,11 +246,16 @@ public:
      * Returns true if this RecordId is not suitable for storage in a RecordStore.
      */
     bool isNull() const {
-        // In the the int64_t format, null can also be represented by '0'.
-        if (_format == Format::kLong) {
-            return getLong() == 0;
+        switch (_format) {
+            case Format::kNull:
+                return true;
+            case Format::kLong:
+                // In the the int64_t format, null can also be represented by '0'.
+                return getLong() == 0;
+            default:
+                return false;
         }
-        return _format == Format::kNull;
+        MONGO_UNREACHABLE;
     }
 
     /**
@@ -362,6 +329,11 @@ public:
         return hash;
     }
 
+    /**
+     * Returns a string form of this RecordId, but in the cases where RecordId is either
+     * Format::kSmallStr or Format::kBigStr this is a raw hex dump. For a human-readable string call
+     * toStringHumanReadable().
+     */
     std::string toString() const {
         return withFormat(
             [](Null n) { return std::string("null"); },
@@ -369,12 +341,38 @@ public:
             [](const char* str, int size) { return hexblob::encodeLower(str, size); });
     }
 
+    /**
+     * Returns a human-readable string form of this RecordId. (Call toString() if you prefer a hex
+     * dump of string IDs.)
+     */
+    std::string toStringHumanReadable() const {
+        switch (_format) {
+            case Format::kNull:
+                return "null";
+            case Format::kLong:
+                return std::to_string(_getLongNoCheck());
+            case Format::kSmallStr: {
+                StringData str = _getSmallStrNoCheck();
+                return "kSmallStr size: " + std::to_string(str.size()) + " string: '" +
+                    std::string(str.rawData()) + "'";
+            }
+            case Format::kBigStr: {
+                StringData str = _getBigStrNoCheck();
+                return "kBigStr size: " + std::to_string(str.size()) + " string: '" +
+                    std::string(str.rawData()) + "'";
+            }
+            default:
+                MONGO_UNREACHABLE;
+        }
+    }
+
     /**
      * Returns the total amount of memory used by this RecordId, including itself and any heap
      * buffers.
      */
     size_t memUsage() const {
-        size_t largeStrSize = (_format == Format::kBigStr) ? _data.heapStr.buffer.capacity() : 0;
+        size_t largeStrSize =
+            (_format == Format::kBigStr) ? HeapStr::getBufferFrom(_data).capacity() : 0;
         return sizeof(RecordId) + largeStrSize;
     }
 
@@ -461,6 +459,24 @@ public:
      */
     static constexpr auto kSmallStrMaxSize = 30;
 
+    /// members for Sorter
+    struct SorterDeserializeSettings {};  // unused
+    void serializeForSorter(BufBuilder& buf) const {
+        serializeToken(buf);
+    }
+    static RecordId deserializeForSorter(BufReader& buf, const SorterDeserializeSettings&) {
+        return deserializeToken(buf);
+    }
+    int memUsageForSorter() const {
+        return memUsage();
+    }
+    RecordId getOwned() const {
+        MONGO_UNREACHABLE;
+    }
+    void makeOwned() {
+        MONGO_UNREACHABLE;
+    }
+
 private:
     /**
      * Format specifies the in-memory representation of this RecordId. This does not represent any
@@ -475,9 +491,9 @@ private:
          */
         kLong,
         /**
-         * Stores a variable-length binary string smaller than kSmallStrMaxSize. Data is stored in
-         * the InlineStr struct at '_data.inlineStr'. This RecordId may only be accessed using
-         * getStr().
+         * Stores a variable-length binary string less than or equal to kSmallStrMaxSize. Data is
+         * stored in the InlineStr struct at '_data.inlineStr'. This RecordId may only be accessed
+         * using getStr().
          */
         kSmallStr,
         /**
@@ -503,15 +519,16 @@ private:
     }
 
     int64_t _getLongNoCheck() const {
-        return _data.longId.id;
+        return LongId::getIdFrom(_data);
     }
 
     StringData _getSmallStrNoCheck() const {
-        return StringData(_data.inlineStr.dataArr.data(), _data.inlineStr.size);
+        return StringData(InlineStr::getArrayFrom(_data).data(), InlineStr::getSizeFrom(_data));
     }
 
     StringData _getBigStrNoCheck() const {
-        return StringData(_data.heapStr.buffer.get(), _data.heapStr.buffer.capacity());
+        const auto& buffer = HeapStr::getBufferFrom(_data);
+        return StringData(buffer.get(), buffer.capacity());
     }
 
     static constexpr auto kTargetSizeInBytes = 32;
@@ -524,41 +541,58 @@ private:
     static_assert(sizeof(Format) == 1);
     // All of this will work if and only if char size is 1 (std::byte) for the InlineString.
     static_assert(sizeof(std::byte) == sizeof(char));
+    using Content = std::array<std::byte, kTargetSizeInBytes - sizeof(Format)>;
+    Content _data;
     // Offsets/padding will be computed in respect to the whole class by taking into account the
     // Format data member.
     struct HeapStr {
         static_assert(std::alignment_of_v<ConstSharedBuffer> <= 8,
                       "ConstSharedBuffer is expected to have 8 bytes alignment at most. Having a "
                       "larger alignment requires changing the RecordId class alignment");
-        std::byte _padding[std::alignment_of_v<ConstSharedBuffer> -
-                           sizeof(Format)];  // offset = 1, size = 7
-        ConstSharedBuffer buffer;            // offset = 1 + 7, size = 8
+        static constexpr auto kOffset = std::alignment_of_v<ConstSharedBuffer> - sizeof(Format);
+        ConstSharedBuffer buffer;  // offset = 1 + 7, size = 8
+        static ConstSharedBuffer& getBufferFrom(Content& data) {
+            ConstSharedBuffer* ptr = (ConstSharedBuffer*)(data.data() + kOffset);
+            return *ptr;
+        }
+        static const ConstSharedBuffer& getBufferFrom(const Content& data) {
+            ConstSharedBuffer* ptr = (ConstSharedBuffer*)(data.data() + kOffset);
+            return *ptr;
+        }
     };
     struct InlineStr {
-        uint8_t size;  // offset = 1, size = 1
-        std::array<char, kTargetSizeInBytes - sizeof(Format) - sizeof(size)>
-            dataArr;  // offset = 1 + 1, size = 30
+        static constexpr auto kSizeOffset = 0;
+        static uint8_t& getSizeFrom(Content& data) {
+            uint8_t* ptr = (uint8_t*)(data.data() + kSizeOffset);
+            return *ptr;
+        };
+        static const uint8_t& getSizeFrom(const Content& data) {
+            uint8_t* ptr = (uint8_t*)(data.data() + kSizeOffset);
+            return *ptr;
+        };
+        static constexpr auto kArrayOffset = sizeof(uint8_t);
+        using arr = std::array<char, kTargetSizeInBytes - sizeof(Format) - sizeof(uint8_t)>;
+        static arr& getArrayFrom(Content& data) {
+            arr* ptr = (arr*)(data.data() + kArrayOffset);
+            return *ptr;
+        };
+        static const arr& getArrayFrom(const Content& data) {
+            arr* ptr = (arr*)(data.data() + kArrayOffset);
+            return *ptr;
+        };
     };
     struct LongId {
-        std::byte _padding[std::alignment_of_v<int64_t> - sizeof(Format)];  // offset = 1, size = 7
-        int64_t id;  // offset = 1 + 7, size = 8
+        static constexpr auto kOffset = std::alignment_of_v<int64_t> - sizeof(Format);
+        static int64_t& getIdFrom(Content& data) {
+            int64_t* result = (int64_t*)(data.data() + kOffset);
+            return *result;
+        }
+        static const int64_t& getIdFrom(const Content& data) {
+            int64_t* result = (int64_t*)(data.data() + kOffset);
+            return *result;
+        }
     };
-    union Content {
-        // Constructor and destructor are needed so that the union class is default constructible.
-        // Placement new will be used to initialize the correct data member in the RecordId
-        // constructor. This is necessary due to ConstSharedBuffer not being trivially
-        // constructible.
-        Content(){};
-        // Destructor is handled in the RecordId destructor as it handles knowing which data member
-        // is active.
-        ~Content(){};
-        HeapStr heapStr;
-        InlineStr inlineStr;
-        LongId longId;
-    };
-    Content _data;  // offset = 1, size = 31
 };
-#pragma pack(pop)
 
 namespace details {
 // Various assertions of RecordId that can only happen when the type is completely defined.
