@@ -27,10 +27,12 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+// IWYU pragma: no_include "cxxabi.h"
+#include <ostream>
+#include <utility>
 
+#include "mongo/base/status_with.h"
 #include "mongo/db/repl/abstract_async_component.h"
-
 #include "mongo/util/assert_util.h"
 #include "mongo/util/str.h"
 
