@@ -28,21 +28,44 @@
  */
 
 
-#include "mongo/platform/basic.h"
-
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+// IWYU pragma: no_include "ext/alloc_traits.h"
 #include <algorithm>
+#include <mutex>
 
+#include "mongo/base/error_codes.h"
 #include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
 #include "mongo/db/auth/authorization_manager.h"
+#include "mongo/db/client.h"
+#include "mongo/db/feature_flag.h"
 #include "mongo/db/multitenancy_gen.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/repl/all_database_cloner.h"
+#include "mongo/db/repl/member_data.h"
+#include "mongo/db/repl/member_state.h"
+#include "mongo/db/repl/replication_auth.h"
 #include "mongo/db/repl/replication_consistency_markers_gen.h"
 #include "mongo/db/repl/replication_consistency_markers_impl.h"
-#include "mongo/db/server_feature_flags_gen.h"
+#include "mongo/db/repl/replication_coordinator.h"
+#include "mongo/db/server_options.h"
+#include "mongo/db/service_context.h"
+#include "mongo/db/tenant_id.h"
+#include "mongo/idl/idl_parser.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/mutex.h"
 #include "mongo/rpc/get_status_from_command_result.h"
 #include "mongo/util/assert_util.h"
 #include "mongo/util/database_name_util.h"
+#include "mongo/util/net/ssl_options.h"
+#include "mongo/util/str.h"
+#include "mongo/util/uuid.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplicationInitialSync
 
@@ -119,9 +142,9 @@ BaseCloner::AfterStageBehavior AllDatabaseCloner::connectStage() {
             [this](const executor::RemoteCommandResponse& helloReply) {
                 return ensurePrimaryOrSecondary(helloReply);
             });
-        uassertStatusOK(client->connect(getSource(), StringData(), boost::none));
+        client->connect(getSource(), StringData(), boost::none);
     } else {
-        client->checkConnection();
+        client->ensureConnection();
     }
     uassertStatusOK(replAuthenticate(client).withContext(
         str::stream() << "Failed to authenticate to " << getSource()));
@@ -129,9 +152,8 @@ BaseCloner::AfterStageBehavior AllDatabaseCloner::connectStage() {
 }
 
 BaseCloner::AfterStageBehavior AllDatabaseCloner::getInitialSyncIdStage() {
-    auto initialSyncId = getClient()->findOne(
-        NamespaceString{ReplicationConsistencyMarkersImpl::kDefaultInitialSyncIdNamespace},
-        BSONObj{});
+    auto initialSyncId =
+        getClient()->findOne(NamespaceString::kDefaultInitialSyncIdNamespace, BSONObj{});
     uassert(ErrorCodes::InitialSyncFailure,
             "Cannot retrieve sync source initial sync ID",
             !initialSyncId.isEmpty());
@@ -162,8 +184,6 @@ BaseCloner::AfterStageBehavior AllDatabaseCloner::listDatabasesStage() {
             LOGV2_DEBUG(21055,
                         1,
                         "Excluding database due to the 'listDatabases' response not containing a "
-                        "'name' field for this entry: {db}",
-                        "Excluding database due to the 'listDatabases' response not containing a "
                         "'name' field for this entry",
                         "db"_attr = dbBSON);
             continue;
@@ -172,12 +192,12 @@ BaseCloner::AfterStageBehavior AllDatabaseCloner::listDatabasesStage() {
         boost::optional<TenantId> tenantId = dbBSON.hasField("tenantId")
             ? boost::make_optional<TenantId>(TenantId::parseFromBSON(dbBSON["tenantId"]))
             : boost::none;
-        DatabaseName dbName = DatabaseNameUtil::deserialize(tenantId, dbBSON["name"].str());
+        const DatabaseName dbName = DatabaseNameUtil::deserialize(
+            tenantId, dbBSON["name"].str(), SerializationContext::stateDefault());
 
-        if (dbName.db() == "local") {
+        if (dbName.isLocalDB()) {
             LOGV2_DEBUG(21056,
                         1,
-                        "Excluding database from the 'listDatabases' response: {db}",
                         "Excluding database from the 'listDatabases' response",
                         "db"_attr = dbBSON);
             continue;
@@ -185,7 +205,7 @@ BaseCloner::AfterStageBehavior AllDatabaseCloner::listDatabasesStage() {
             _databases.emplace_back(dbName);
 
             // Put admin dbs in the front of the vector.
-            if (dbName.db() == "admin") {
+            if (dbName.isAdminDB()) {
                 if (_databases.size() > 1) {
                     std::iter_swap(_databases.begin() + idxToInsertNextAdmin, _databases.end() - 1);
                 }
@@ -210,11 +230,7 @@ BaseCloner::AfterStageBehavior AllDatabaseCloner::listDatabasesStage() {
 }
 
 void AllDatabaseCloner::handleAdminDbNotValid(const Status& errorStatus) {
-    LOGV2_DEBUG(21059,
-                1,
-                "Validation failed on 'admin' db due to {error}",
-                "Validation failed on 'admin' db",
-                "error"_attr = errorStatus);
+    LOGV2_DEBUG(21059, 1, "Validation failed on 'admin' db", "error"_attr = errorStatus);
     setSyncFailedStatus(errorStatus);
 }
 
@@ -230,11 +246,6 @@ void AllDatabaseCloner::postStage() {
 
             BSONObj cmdObj = BSON("dbStats" << 1);
             BSONObjBuilder b(cmdObj);
-            const auto fcvSnapshot = serverGlobalParams.featureCompatibility.acquireFCVSnapshot();
-            if (gMultitenancySupport && fcvSnapshot.isVersionInitialized() &&
-                gFeatureFlagRequireTenantID.isEnabled(fcvSnapshot) && dbName.tenantId()) {
-                dbName.tenantId()->serializeToBSON("$tenant", &b);
-            }
 
             BSONObj res;
             getClient()->runCommand(dbName, b.obj(), res);
@@ -272,14 +283,11 @@ void AllDatabaseCloner::postStage() {
         if (dbStatus.isOK()) {
             LOGV2_DEBUG(21057,
                         1,
-                        "Database clone for '{dbName}' finished: {status}",
                         "Database clone finished",
                         "dbName"_attr = dbName,
                         "status"_attr = dbStatus);
         } else {
             LOGV2_WARNING(21060,
-                          "database '{dbName}' ({dbNumber} of {totalDbs}) "
-                          "clone failed due to {error}",
                           "Database clone failed",
                           "dbName"_attr = dbName,
                           "dbNumber"_attr = (_stats.databasesCloned + 1),
@@ -288,7 +296,7 @@ void AllDatabaseCloner::postStage() {
             setSyncFailedStatus(dbStatus);
             return;
         }
-        if (!foundUser && StringData(dbName.db()).equalCaseInsensitive("admin")) {
+        if (!foundUser && dbName.isAdminDB()) {
             LOGV2_DEBUG(21058, 1, "Finished the 'admin' db, now validating it");
             // Do special checks for the admin database because of auth. collections.
             {
@@ -298,7 +306,7 @@ void AllDatabaseCloner::postStage() {
                     opCtxPtr = cc().makeOperationContext();
                     opCtx = opCtxPtr.get();
                 }
-                auto authzManager = AuthorizationManager::get(opCtx->getServiceContext());
+                auto authzManager = AuthorizationManager::get(opCtx->getService());
 
                 // Check if global admin has a valid auth schema version document.
                 if (!dbName.tenantId() && !foundAuthSchemaDoc) {
@@ -322,9 +330,9 @@ void AllDatabaseCloner::postStage() {
             if (!foundAuthSchemaDoc && foundUser) {
                 std::string msg = str::stream()
                     << "During initial sync, found documents in "
-                    << NamespaceString::kAdminUsersNamespace.ns()
+                    << NamespaceString::kAdminUsersNamespace.toStringForErrorMsg()
                     << " but could not find an auth schema version document in "
-                    << NamespaceString::kServerConfigurationNamespace.ns() << ".  "
+                    << NamespaceString::kServerConfigurationNamespace.toStringForErrorMsg() << ".  "
                     << "This indicates that the primary of this replica set was not "
                        "successfully "
                        "upgraded to schema version "
@@ -364,10 +372,6 @@ std::string AllDatabaseCloner::toString() const {
                          << " db cloners completed:" << _stats.databasesCloned;
 }
 
-std::string AllDatabaseCloner::Stats::toString() const {
-    return toBSON().toString();
-}
-
 BSONObj AllDatabaseCloner::Stats::toBSON() const {
     BSONObjBuilder bob;
     append(&bob);
@@ -378,7 +382,8 @@ void AllDatabaseCloner::Stats::append(BSONObjBuilder* builder) const {
     builder->appendNumber("databasesToClone", static_cast<long long>(databasesToClone));
     builder->appendNumber("databasesCloned", static_cast<long long>(databasesCloned));
     for (auto&& db : databaseStats) {
-        BSONObjBuilder dbBuilder(builder->subobjStart(DatabaseNameUtil::serialize(db.dbname)));
+        BSONObjBuilder dbBuilder(builder->subobjStart(
+            DatabaseNameUtil::serialize(db.dbname, SerializationContext::stateDefault())));
         db.append(&dbBuilder);
         dbBuilder.doneFast();
     }
