@@ -30,33 +30,59 @@
 
 #include "mongo/db/repl/bgsync.h"
 
+#include <cstddef>
+// IWYU pragma: no_include "cxxabi.h"
+#include <chrono>
+#include <exception>
 #include <memory>
+#include <mutex>
+#include <string>
+#include <utility>
+#include <vector>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status_with.h"
 #include "mongo/base/string_data.h"
-#include "mongo/bson/util/bson_extract.h"
-#include "mongo/client/connection_pool.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/client/dbclient_base.h"
+#include "mongo/client/dbclient_connection.h"
 #include "mongo/db/auth/authorization_session.h"
 #include "mongo/db/client.h"
+#include "mongo/db/commands/server_status_metric.h"
 #include "mongo/db/concurrency/exception_util.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
 #include "mongo/db/concurrency/replication_state_transition_lock_guard.h"
 #include "mongo/db/dbhelpers.h"
-#include "mongo/db/index_builds_coordinator.h"
+#include "mongo/db/namespace_string.h"
 #include "mongo/db/repl/data_replicator_external_state_impl.h"
-#include "mongo/db/repl/oplog.h"
+#include "mongo/db/repl/member_state.h"
+#include "mongo/db/repl/oplog_buffer.h"
+#include "mongo/db/repl/oplog_entry.h"
 #include "mongo/db/repl/oplog_interface_local.h"
 #include "mongo/db/repl/oplog_interface_remote.h"
 #include "mongo/db/repl/repl_server_parameters_gen.h"
-#include "mongo/db/repl/replication_consistency_markers_impl.h"
 #include "mongo/db/repl/replication_coordinator.h"
-#include "mongo/db/repl/replication_coordinator_impl.h"
+#include "mongo/db/repl/replication_coordinator_external_state.h"
 #include "mongo/db/repl/replication_process.h"
 #include "mongo/db/repl/rollback_source_impl.h"
 #include "mongo/db/repl/rs_rollback.h"
 #include "mongo/db/repl/storage_interface.h"
+#include "mongo/db/repl/sync_source_selector.h"
+#include "mongo/db/service_context.h"
 #include "mongo/db/shutdown_in_progress_quiesce_info.h"
+#include "mongo/db/storage/recovery_unit.h"
+#include "mongo/db/storage/storage_engine.h"
+#include "mongo/db/transaction_resources.h"
 #include "mongo/logv2/log.h"
-#include "mongo/rpc/get_status_from_command_result.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/redaction.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/rpc/metadata/oplog_query_metadata.h"
 #include "mongo/rpc/metadata/repl_set_metadata.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/duration.h"
+#include "mongo/util/fail_point.h"
 #include "mongo/util/str.h"
 #include "mongo/util/testing_proctor.h"
 #include "mongo/util/time_support.h"
@@ -78,19 +104,21 @@ const Milliseconds kRollbackOplogSocketTimeout(10 * 60 * 1000);
 // The number of times a node attempted to choose a node to sync from among the available sync
 // source options. This occurs if we re-evaluate our sync source, receive an error from the source,
 // or step down.
-CounterMetric numSyncSourceSelections("repl.syncSource.numSelections");
+auto& numSyncSourceSelections = *MetricBuilder<Counter64>{"repl.syncSource.numSelections"};
 
 // The number of times a node kept it's original sync source after re-evaluating if its current sync
 // source was optimal.
-CounterMetric numTimesChoseSameSyncSource("repl.syncSource.numTimesChoseSame");
+auto& numTimesChoseSameSyncSource = *MetricBuilder<Counter64>{"repl.syncSource.numTimesChoseSame"};
 
 // The number of times a node chose a new sync source after re-evaluating if its current sync source
 // was optimal.
-CounterMetric numTimesChoseDifferentSyncSource("repl.syncSource.numTimesChoseDifferent");
+auto& numTimesChoseDifferentSyncSource =
+    *MetricBuilder<Counter64>{"repl.syncSource.numTimesChoseDifferent"};
 
 // The number of times a node could not find a sync source when choosing a node to sync from among
 // the available options.
-CounterMetric numTimesCouldNotFindSyncSource("repl.syncSource.numTimesCouldNotFind");
+auto& numTimesCouldNotFindSyncSource =
+    *MetricBuilder<Counter64>{"repl.syncSource.numTimesCouldNotFind"};
 
 /**
  * Extends DataReplicatorExternalStateImpl to be member state aware.
@@ -172,8 +200,10 @@ BackgroundSync::BackgroundSync(
     ReplicationCoordinator* replicationCoordinator,
     ReplicationCoordinatorExternalState* replicationCoordinatorExternalState,
     ReplicationProcess* replicationProcess,
+    OplogWriter* oplogWriter,
     OplogApplier* oplogApplier)
-    : _oplogApplier(oplogApplier),
+    : _oplogWriter(oplogWriter),
+      _oplogApplier(oplogApplier),
       _replCoord(replicationCoordinator),
       _replicationCoordinatorExternalState(replicationCoordinatorExternalState),
       _replicationProcess(replicationProcess) {}
@@ -223,9 +253,15 @@ bool BackgroundSync::_inShutdown_inlock() const {
 }
 
 void BackgroundSync::_run() {
-    Client::initThread("BackgroundSync");
+    Client::initThread("BackgroundSync",
+                       getGlobalServiceContext()->getService(ClusterRole::ShardServer));
     AuthorizationSession::get(cc())->grantInternalAuthorization(&cc());
 
+    {
+        stdx::lock_guard<Client> lk(cc());
+        cc().setSystemOperationUnkillableByStepdown(lk);
+    }
+
     while (!inShutdown()) {
         try {
             _runProducer();
@@ -236,10 +272,7 @@ void BackgroundSync::_run() {
             sleepmillis(100);  // sleep a bit to keep from hammering this thread with temp. errors.
         } catch (const std::exception& e2) {
             // redact(std::exception&) doesn't work
-            LOGV2_FATAL(28546,
-                        "sync producer exception: {error}",
-                        "Sync producer error",
-                        "error"_attr = redact(e2.what()));
+            LOGV2_FATAL(28546, "Sync producer error", "error"_attr = redact(e2.what()));
         }
     }
     // No need to reset optimes here because we are shutting down.
@@ -318,7 +351,6 @@ void BackgroundSync::_produce() {
         lastOpTimeFetched = _lastOpTimeFetched;
         if (!_syncSourceHost.empty()) {
             LOGV2(21080,
-                  "Clearing sync source {syncSource} to choose a new one.",
                   "Clearing sync source to choose a new one",
                   "syncSource"_attr = _syncSourceHost);
         }
@@ -365,11 +397,7 @@ void BackgroundSync::_produce() {
             auto status = _replCoord->abortCatchupIfNeeded(
                 ReplicationCoordinator::PrimaryCatchUpConclusionReason::kFailedWithError);
             if (!status.isOK()) {
-                LOGV2_DEBUG(21083,
-                            1,
-                            "Aborting catch-up failed with status: {error}",
-                            "Aborting catch-up failed",
-                            "error"_attr = status);
+                LOGV2_DEBUG(21083, 1, "Aborting catch-up failed", "error"_attr = status);
             }
             return;
         }
@@ -392,18 +420,14 @@ void BackgroundSync::_produce() {
         // Activate maintenance mode and transition to RECOVERING.
         auto status = _replCoord->setMaintenanceMode(opCtx.get(), true);
         if (!status.isOK()) {
-            LOGV2_WARNING(21116,
-                          "Failed to transition into maintenance mode: {error}",
-                          "Failed to transition into maintenance mode",
-                          "error"_attr = status);
+            LOGV2_WARNING(
+                21116, "Failed to transition into maintenance mode", "error"_attr = status);
             // Do not mark ourselves too stale on errors so we can try again next time.
             return;
         }
         status = _replCoord->setFollowerMode(MemberState::RS_RECOVERING);
         if (!status.isOK()) {
             LOGV2_WARNING(21117,
-                          "Failed to transition into {targetState}. "
-                          "Current state: {currentState}. Caused by: {error}",
                           "Failed to perform replica set state transition",
                           "targetState"_attr = MemberState(MemberState::RS_RECOVERING),
                           "currentState"_attr = _replCoord->getMemberState(),
@@ -434,7 +458,6 @@ void BackgroundSync::_produce() {
             _waitForNewSyncSourceSelectionData(sleepMS);
         } else {
             LOGV2(21088,
-                  "Changed sync source from {oldSyncSource} to {newSyncSource}",
                   "Changed sync source",
                   "oldSyncSource"_attr =
                       (oldSource.empty() ? std::string("empty") : oldSource.toString()),
@@ -444,10 +467,12 @@ void BackgroundSync::_produce() {
     } else {
         if (!syncSourceResp.isOK()) {
             LOGV2(21089,
-                  "failed to find sync source, received error "
-                  "{error}",
                   "Failed to find sync source",
                   "error"_attr = syncSourceResp.syncSourceStatus.getStatus());
+            // The code above makes the ReplicationCoordinator set its sync source, we clear it
+            // again here. Not doing this would give the impression that this node has a sync
+            // source, delaying other nodes switching away from it.
+            _replCoord->clearSyncSource();
         }
 
         long long sleepMS = _getRetrySleepMS();
@@ -464,18 +489,12 @@ void BackgroundSync::_produce() {
     // If we find a good sync source after having gone too stale, disable maintenance mode so we can
     // transition to SECONDARY.
     if (_tooStale.swap(false)) {
-        LOGV2(21091,
-              "No longer too stale. Able to sync from {syncSource}",
-              "No longer too stale. Able to start syncing",
-              "syncSource"_attr = source);
+        LOGV2(21091, "No longer too stale. Able to start syncing", "syncSource"_attr = source);
 
         auto opCtx = cc().makeOperationContext();
         auto status = _replCoord->setMaintenanceMode(opCtx.get(), false);
         if (!status.isOK()) {
-            LOGV2_WARNING(21118,
-                          "Failed to leave maintenance mode: {error}",
-                          "Failed to leave maintenance mode",
-                          "error"_attr = status);
+            LOGV2_WARNING(21118, "Failed to leave maintenance mode", "error"_attr = status);
         }
     }
 
@@ -538,15 +557,12 @@ void BackgroundSync::_produce() {
     const auto logLevel = TestingProctor::instance().isEnabled() ? 0 : 1;
     LOGV2_DEBUG(21092,
                 logLevel,
-                "scheduling fetcher to read remote oplog on {syncSource} starting at "
-                "{lastOpTimeFetched}",
                 "Scheduling fetcher to read remote oplog",
                 "syncSource"_attr = source,
                 "lastOpTimeFetched"_attr = oplogFetcher->getLastOpTimeFetched_forTest());
     auto scheduleStatus = oplogFetcher->startup();
     if (!scheduleStatus.isOK()) {
         LOGV2_WARNING(21119,
-                      "unable to schedule fetcher to read remote oplog on {syncSource}: {error}",
                       "Unable to schedule fetcher to read remote oplog",
                       "syncSource"_attr = source,
                       "error"_attr = scheduleStatus);
@@ -554,11 +570,7 @@ void BackgroundSync::_produce() {
     }
 
     oplogFetcher->join();
-    LOGV2_DEBUG(21093,
-                1,
-                "fetcher stopped reading remote oplog on {syncSource}",
-                "Fetcher stopped reading remote oplog",
-                "syncSource"_attr = source);
+    LOGV2_DEBUG(21093, 1, "Fetcher stopped reading remote oplog", "syncSource"_attr = source);
 
     // If the background sync is stopped after the fetcher is started, we need to
     // re-evaluate our sync source and oplog common point.
@@ -602,8 +614,6 @@ void BackgroundSync::_produce() {
     } else if (fetcherReturnStatus == ErrorCodes::InvalidBSON) {
         LOGV2_WARNING(
             5579701,
-            "Oplog fetcher got invalid BSON while querying oplog. Denylisting sync source "
-            "{syncSource} for {denylistDuration}.",
             "Oplog fetcher got invalid BSON while querying oplog. Denylisting sync source",
             "syncSource"_attr = source,
             "denylistDuration"_attr = denylistDuration);
@@ -619,17 +629,8 @@ void BackgroundSync::_produce() {
                 "denylistDuration"_attr = denylistDuration);
             _replCoord->denylistSyncSource(source, Date_t::now() + denylistDuration);
         }
-    } else if (fetcherReturnStatus.code() == ErrorCodes::BSONObjectTooLarge) {
-        // TODO: SERVER-99952 discovered that a large document near max size will exceed the limit
-        // when fetched with oplog metadata. Re-evaluate the handling of this error code after this
-        // ticket is resolved.
-        LOGV2_WARNING(10033600,
-                      "BSON document and metadata fetched by oplog fetcher exceeds the BSON limit"
-                      "sync source",
-                      "syncSource"_attr = source);
     } else if (!fetcherReturnStatus.isOK()) {
         LOGV2_WARNING(21122,
-                      "Oplog fetcher stopped querying remote oplog with error: {error}",
                       "Oplog fetcher stopped querying remote oplog with error",
                       "error"_attr = redact(fetcherReturnStatus));
     }
@@ -646,9 +647,15 @@ Status BackgroundSync::_enqueueDocuments(OplogFetcher::Documents::const_iterator
     }
 
     auto opCtx = cc().makeOperationContext();
+    OplogBuffer::Cost cost{info.toApplyDocumentBytes, info.toApplyDocumentCount};
 
     // Wait for enough space.
-    _oplogApplier->waitForSpace(opCtx.get(), info.toApplyDocumentBytes);
+    // This should be called outside of the mutex to avoid deadlocks.
+    if (_oplogWriter) {
+        _oplogWriter->waitForSpace(opCtx.get(), cost);
+    } else {
+        _oplogApplier->waitForSpace(opCtx.get(), cost);
+    }
 
     {
         // Don't add more to the buffer if we are in shutdown. Continue holding the lock until we
@@ -659,13 +666,17 @@ Status BackgroundSync::_enqueueDocuments(OplogFetcher::Documents::const_iterator
         }
 
         // Buffer docs for later application.
-        _oplogApplier->enqueue(opCtx.get(), begin, end);
+        // When _oplogWriter is not null, featureFlagReduceMajorityWriteLatency is enabled.
+        if (_oplogWriter) {
+            _oplogWriter->enqueue(opCtx.get(), begin, end, cost);
+        } else {
+            _oplogApplier->enqueue(opCtx.get(), begin, end, cost);
+        }
 
         // Update last fetched info.
         _lastOpTimeFetched = info.lastDocument;
         LOGV2_DEBUG(21096,
                     3,
-                    "batch resetting _lastOpTimeFetched: {lastOpTimeFetched}",
                     "Batch resetting _lastOpTimeFetched",
                     "lastOpTimeFetched"_attr = _lastOpTimeFetched);
     }
@@ -696,20 +707,14 @@ void BackgroundSync::_runRollback(OperationContext* opCtx,
         auto status = _replCoord->abortCatchupIfNeeded(
             ReplicationCoordinator::PrimaryCatchUpConclusionReason::kFailedWithError);
         if (!status.isOK()) {
-            LOGV2_DEBUG(21097,
-                        1,
-                        "Aborting catch-up failed with status: {error}",
-                        "Aborting catch-up failed",
-                        "error"_attr = status);
+            LOGV2_DEBUG(21097, 1, "Aborting catch-up failed", "error"_attr = status);
         }
         return;
     }
 
-    ShouldNotConflictWithSecondaryBatchApplicationBlock noConflict(opCtx->lockState());
-
     // Ensure future transactions read without a timestamp.
     invariant(RecoveryUnit::ReadSource::kNoTimestamp ==
-              opCtx->recoveryUnit()->getTimestampReadSource());
+              shard_role_details::getRecoveryUnit(opCtx)->getTimestampReadSource());
 
     // Rollback is a synchronous operation that uses the task executor and may not be
     // executed inside the fetcher callback.
@@ -732,8 +737,6 @@ void BackgroundSync::_runRollback(OperationContext* opCtx,
     auto lastApplied = _replCoord->getMyLastAppliedOpTime();
     if (lastApplied != lastOpTimeFetched) {
         LOGV2(21100,
-              "Waiting for all operations from {lastApplied} until {lastOpTimeFetched} to be "
-              "applied before starting rollback.",
               "Waiting for all operations from lastApplied until lastOpTimeFetched to be applied "
               "before starting rollback",
               "lastApplied"_attr = lastApplied,
@@ -758,15 +761,18 @@ void BackgroundSync::_runRollback(OperationContext* opCtx,
 
     OplogInterfaceLocal localOplog(opCtx);
 
-    const int messagingPortTags = 0;
-    ConnectionPool connectionPool(messagingPortTags);
-    std::unique_ptr<ConnectionPool::ConnectionPtr> connection;
-    auto getConnection = [&connection, &connectionPool, source]() -> DBClientBase* {
-        if (!connection.get()) {
-            connection.reset(new ConnectionPool::ConnectionPtr(
-                &connectionPool, source, Date_t::now(), kRollbackOplogSocketTimeout));
-        };
-        return connection->get();
+    std::unique_ptr<DBClientConnection> connection;
+    auto getConnection = [&connection, source]() -> DBClientBase* {
+        if (!connection) {
+            connection = std::make_unique<DBClientConnection>();
+            connection->setSoTimeout(durationCount<Milliseconds>(kRollbackOplogSocketTimeout) /
+                                     1000.0);
+            connection->connect(source, StringData(), boost::none);
+            if (auth::isInternalAuthSet()) {
+                connection->authenticateInternalUser();
+            }
+        }
+        return connection.get();
     };
 
     // Because oplog visibility is updated asynchronously, wait until all uncommitted oplog entries
@@ -803,10 +809,8 @@ void BackgroundSync::_runRollbackViaRecoverToCheckpoint(
     StorageInterface* storageInterface,
     OplogInterfaceRemote::GetConnectionFn getConnection) {
 
-    OplogInterfaceRemote remoteOplog(source,
-                                     getConnection,
-                                     NamespaceString::kRsOplogNamespace.ns(),
-                                     rollbackRemoteOplogQueryBatchSize.load());
+    OplogInterfaceRemote remoteOplog(
+        source, getConnection, rollbackRemoteOplogQueryBatchSize.load());
 
     {
         stdx::lock_guard<Latch> lock(_mutex);
@@ -817,25 +821,16 @@ void BackgroundSync::_runRollbackViaRecoverToCheckpoint(
             localOplog, &remoteOplog, storageInterface, _replicationProcess, _replCoord);
     }
 
-    LOGV2(21104,
-          "Scheduling rollback (sync source: {syncSource})",
-          "Scheduling rollback",
-          "syncSource"_attr = source);
-
+    LOGV2(21104, "Scheduling rollback", "syncSource"_attr = source);
     auto status = _rollback->runRollback(opCtx);
     if (status.isOK()) {
         LOGV2(21105, "Rollback successful");
     } else if (status == ErrorCodes::UnrecoverableRollbackError) {
-        LOGV2_FATAL_CONTINUE(21128,
-                             "Rollback failed with unrecoverable error: {error}",
-                             "Rollback failed with unrecoverable error",
-                             "error"_attr = status);
+        LOGV2_FATAL_CONTINUE(
+            21128, "Rollback failed with unrecoverable error", "error"_attr = status);
         fassertFailedWithStatusNoTrace(50666, status);
     } else {
-        LOGV2_WARNING(21124,
-                      "Rollback failed with retryable error: {error}",
-                      "Rollback failed with retryable error",
-                      "error"_attr = status);
+        LOGV2_WARNING(21124, "Rollback failed with retryable error", "error"_attr = status);
     }
 }
 
@@ -846,10 +841,8 @@ void BackgroundSync::_fallBackOnRollbackViaRefetch(
     OplogInterface* localOplog,
     OplogInterfaceRemote::GetConnectionFn getConnection) {
 
-    RollbackSourceImpl rollbackSource(getConnection,
-                                      source,
-                                      NamespaceString::kRsOplogNamespace.ns(),
-                                      rollbackRemoteOplogQueryBatchSize.load());
+    RollbackSourceImpl rollbackSource(
+        getConnection, source, rollbackRemoteOplogQueryBatchSize.load());
 
     rollback(opCtx, *localOplog, rollbackSource, requiredRBID, _replCoord, _replicationProcess);
 }
@@ -888,10 +881,7 @@ HostAndPort BackgroundSync::getSyncTarget() const {
 
 void BackgroundSync::clearSyncTarget() {
     stdx::unique_lock<Latch> lock(_mutex);
-    LOGV2(21106,
-          "Resetting sync source to empty, which was {previousSyncSource}",
-          "Resetting sync source to empty",
-          "previousSyncSource"_attr = _syncSourceHost);
+    LOGV2(21106, "Resetting sync source to empty", "previousSyncSource"_attr = _syncSourceHost);
     _syncSourceHost = HostAndPort();
     _notifySyncSourceSelectionDataChanged(lock);
 }
@@ -905,7 +895,12 @@ void BackgroundSync::_stop(WithLock lock, bool resetLastFetchedOptime) {
 
     _syncSourceHost = HostAndPort();
     if (resetLastFetchedOptime) {
-        invariant(_oplogApplier->getBuffer()->isEmpty());
+        // When _oplogWriter is not null, featureFlagReduceMajorityWriteLatency is enabled.
+        if (_oplogWriter) {
+            invariant(_oplogWriter->getBuffer()->isEmpty());
+        } else {
+            invariant(_oplogApplier->getBuffer()->isEmpty());
+        }
         _lastOpTimeFetched = OpTime();
         LOGV2(21108, "Resetting last fetched optimes in bgsync");
     }
@@ -926,11 +921,10 @@ void BackgroundSync::stop(bool resetLastFetchedOptime) {
 
 void BackgroundSync::start(OperationContext* opCtx) {
     OpTime lastAppliedOpTime;
-    ShouldNotConflictWithSecondaryBatchApplicationBlock noConflict(opCtx->lockState());
 
     // Ensure future transactions read without a timestamp.
     invariant(RecoveryUnit::ReadSource::kNoTimestamp ==
-              opCtx->recoveryUnit()->getTimestampReadSource());
+              shard_role_details::getRecoveryUnit(opCtx)->getTimestampReadSource());
 
     do {
         lastAppliedOpTime = _readLastAppliedOpTime(opCtx);
@@ -939,9 +933,11 @@ void BackgroundSync::start(OperationContext* opCtx) {
         if (_state != ProducerState::Starting) {
             return;
         }
-        // If a node steps down during drain mode, then the buffer may not be empty at the beginning
-        // of secondary state.
-        if (!_oplogApplier->getBuffer()->isEmpty()) {
+        // If a node steps down during drain mode, then the buffer may not be empty at the
+        // beginning of secondary state.
+        // When _oplogWriter is not null, featureFlagReduceMajorityWriteLatency is enabled.
+        auto buffer = _oplogWriter ? _oplogWriter->getBuffer() : _oplogApplier->getBuffer();
+        if (!buffer->isEmpty()) {
             LOGV2(21109, "Going to start syncing, but buffer is not empty");
         }
         setState(lk, ProducerState::Running);
@@ -951,8 +947,6 @@ void BackgroundSync::start(OperationContext* opCtx) {
         if (_lastOpTimeFetched <= lastAppliedOpTime) {
             LOGV2_DEBUG(21110,
                         1,
-                        "Setting bgsync _lastOpTimeFetched={lastAppliedOpTime}. Previous "
-                        "_lastOpTimeFetched: {previousLastOpTimeFetched}",
                         "Setting bgsync _lastOpTimeFetched to lastAppliedOpTime",
                         "lastAppliedOpTime"_attr = lastAppliedOpTime,
                         "previousLastOpTimeFetched"_attr = _lastOpTimeFetched);
@@ -963,7 +957,6 @@ void BackgroundSync::start(OperationContext* opCtx) {
 
     LOGV2_DEBUG(21111,
                 1,
-                "bgsync fetch queue set to: {lastOpTimeFetched}",
                 "bgsync fetch queue set to lastOpTimeFetched",
                 "lastOpTimeFetched"_attr = _lastOpTimeFetched);
 }
@@ -972,7 +965,7 @@ OpTime BackgroundSync::_readLastAppliedOpTime(OperationContext* opCtx) {
     BSONObj oplogEntry;
     try {
         bool success = writeConflictRetry(
-            opCtx, "readLastAppliedOpTime", NamespaceString::kRsOplogNamespace.ns(), [&] {
+            opCtx, "readLastAppliedOpTime", NamespaceString::kRsOplogNamespace, [&] {
                 return Helpers::getLast(opCtx, NamespaceString::kRsOplogNamespace, oplogEntry);
             });
 
@@ -984,7 +977,6 @@ OpTime BackgroundSync::_readLastAppliedOpTime(OperationContext* opCtx) {
         throw;
     } catch (const DBException& ex) {
         LOGV2_FATAL(18904,
-                    "Problem reading {namespace}: {error}",
                     "Problem reading from namespace",
                     logAttrs(NamespaceString::kRsOplogNamespace),
                     "error"_attr = redact(ex));
@@ -993,7 +985,6 @@ OpTime BackgroundSync::_readLastAppliedOpTime(OperationContext* opCtx) {
     OplogEntry parsedEntry(oplogEntry);
     LOGV2_DEBUG(21112,
                 1,
-                "Successfully read last entry of oplog while starting bgsync: {lastOplogEntry}",
                 "Successfully read last entry of oplog while starting bgsync",
                 "lastOplogEntry"_attr = redact(oplogEntry));
     return parsedEntry.getOpTime();
