@@ -28,26 +28,54 @@
  */
 
 
-#include "mongo/db/index/index_descriptor_fwd.h"
-#include "mongo/db/service_context.h"
-#include "mongo/platform/basic.h"
+#include <absl/container/node_hash_map.h>
+#include <boost/cstdint.hpp>
+#include <boost/none.hpp>
+#include <cstdint>
+#include <list>
+#include <mutex>
+#include <type_traits>
+
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
 
 #include "mongo/base/string_data.h"
-#include "mongo/db/catalog/clustered_collection_options_gen.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/client/dbclient_base.h"
+#include "mongo/client/read_preference.h"
 #include "mongo/db/catalog/clustered_collection_util.h"
-#include "mongo/db/commands/list_collections_filter.h"
-#include "mongo/db/index_build_entry_helpers.h"
+#include "mongo/db/client.h"
+#include "mongo/db/dbmessage.h"
+#include "mongo/db/feature_flag.h"
+#include "mongo/db/index/index_descriptor.h"
 #include "mongo/db/index_builds_coordinator.h"
+#include "mongo/db/multitenancy_gen.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/query/find_command.h"
 #include "mongo/db/repl/collection_bulk_loader.h"
 #include "mongo/db/repl/collection_cloner.h"
-#include "mongo/db/repl/database_cloner_gen.h"
+#include "mongo/db/repl/oplog_entry.h"
+#include "mongo/db/repl/read_concern_args.h"
 #include "mongo/db/repl/repl_server_parameters_gen.h"
 #include "mongo/db/server_feature_flags_gen.h"
-#include "mongo/db/wire_version.h"
+#include "mongo/db/server_options.h"
+#include "mongo/db/service_context.h"
+#include "mongo/db/storage/storage_engine.h"
+#include "mongo/db/tenant_id.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/platform/mutex.h"
 #include "mongo/rpc/get_status_from_command_result.h"
-
+#include "mongo/stdx/unordered_map.h"
 #include "mongo/util/assert_util.h"
+#include "mongo/util/clock_source.h"
+#include "mongo/util/duration.h"
+#include "mongo/util/fail_point.h"
+#include "mongo/util/namespace_string_util.h"
+#include "mongo/util/str.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplicationInitialSync
 
@@ -79,9 +107,7 @@ void waitWhileFailPointEnabled(FailPoint* failPoint,
             }
         },
         [&](const BSONObj& data) {
-            const auto ns = data.getStringField("nss"_sd);
-            const auto fpNss =
-                NamespaceStringUtil::deserialize(boost::none, ns, SerializationContext());
+            const auto fpNss = NamespaceStringUtil::parseFailPointData(data, "nss"_sd);
             // Only hang when cloning the specified collection, or if no collection was specified.
             return fpNss.isEmpty() || fpNss == sourceNss;
         });
@@ -99,7 +125,7 @@ CollectionCloner::CollectionCloner(const NamespaceString& sourceNss,
           "CollectionCloner"_sd, sharedData, source, client, storageInterface, dbPool),
       _sourceNss(sourceNss),
       _collectionOptions(collectionOptions),
-      _sourceDbAndUuid(NamespaceString("UNINITIALIZED")),
+      _sourceDbAndUuid(NamespaceString::kEmpty),
       _collectionClonerBatchSize(collectionClonerBatchSize),
       _countStage("count", this, &CollectionCloner::countStage),
       _listIndexesStage("listIndexes", this, &CollectionCloner::listIndexesStage),
@@ -113,7 +139,9 @@ CollectionCloner::CollectionCloner(const NamespaceString& sourceNss,
                      kProgressMeterSecondsBetween,
                      kProgressMeterCheckInterval,
                      "documents copied",
-                     str::stream() << _sourceNss.toString() << " collection clone progress"),
+                     str::stream() << NamespaceStringUtil::serialize(
+                                          _sourceNss, SerializationContext::stateDefault())
+                                   << " collection clone progress"),
       _scheduleDbWorkFn([this](executor::TaskExecutor::CallbackFn work) {
           auto task = [this, work = std::move(work)](
                           OperationContext* opCtx,
@@ -156,11 +184,6 @@ void CollectionCloner::preStage() {
     _stats.start = getSharedData()->getClock()->now();
 
     BSONObjBuilder b(BSON("collStats" << _sourceNss.coll().toString()));
-    const auto fcvSnapshot = serverGlobalParams.featureCompatibility.acquireFCVSnapshot();
-    if (gMultitenancySupport && fcvSnapshot.isVersionInitialized() &&
-        gFeatureFlagRequireTenantID.isEnabled(fcvSnapshot) && _sourceNss.tenantId()) {
-        _sourceNss.tenantId()->serializeToBSON("$tenant", &b);
-    }
 
     BSONObj res;
     getClient()->runCommand(_sourceNss.dbName(), b.obj(), res);
@@ -192,8 +215,6 @@ BaseCloner::AfterStageBehavior CollectionCloner::CollectionClonerStage::run() {
         return ClonerStage<CollectionCloner>::run();
     } catch (const ExceptionFor<ErrorCodes::NamespaceNotFound>&) {
         LOGV2(21132,
-              "CollectionCloner ns: '{namespace}' uuid: "
-              "UUID(\"{uuid}\") stopped because collection was dropped on source.",
               "CollectionCloner stopped because collection was dropped on source",
               logAttrs(getCloner()->getSourceNss()),
               "uuid"_attr = getCloner()->getSourceUuid());
@@ -238,7 +259,6 @@ BaseCloner::AfterStageBehavior CollectionCloner::listIndexesStage() {
         getClient()->getIndexSpecs(_sourceDbAndUuid, includeBuildUUIDs, QueryOption_SecondaryOk);
     if (indexSpecs.empty()) {
         LOGV2_WARNING(21143,
-                      "No indexes found for collection {namespace} while cloning from {source}",
                       "No indexes found for collection while cloning",
                       logAttrs(_sourceNss),
                       "source"_attr = getSource());
@@ -283,8 +303,6 @@ BaseCloner::AfterStageBehavior CollectionCloner::listIndexesStage() {
 
     if (!_idIndexSpec.isEmpty() && _collectionOptions.autoIndexId == CollectionOptions::NO) {
         LOGV2_WARNING(21144,
-                      "Found the _id_ index spec but the collection specified autoIndexId of false "
-                      "on ns:{namespace}",
                       "Found the _id index spec but the collection specified autoIndexId of false",
                       logAttrs(this->_sourceNss));
     }
@@ -388,6 +406,17 @@ void CollectionCloner::runQuery() {
 
     ExhaustMode exhaustMode = collectionClonerUsesExhaust ? ExhaustMode::kOn : ExhaustMode::kOff;
 
+    if (_collectionOptions.recordIdsReplicated) {
+        // The below projection returns a stream of documents in the format
+        // {r: <recordId>, d: <original document>}.
+        auto projection = BSON("_id" << 0 << "r"
+                                     << BSON("$meta"
+                                             << "recordId")
+                                     << "d"
+                                     << "$$ROOT");
+        findCmd.setProjection(std::move(projection));
+    }
+
     // We reset this every time we retry or resume a query.
     // We distinguish the first batch from the rest so that we only store the remote cursor id
     // the first time we get it.
@@ -441,12 +470,15 @@ void CollectionCloner::handleNextBatch(DBClientCursor& cursor) {
     }
 
     // Schedule the next document batch insertion.
-    auto&& scheduleResult = _scheduleDbWorkFn(
-        [=](const executor::TaskExecutor::CallbackArgs& cbd) { insertDocumentsCallback(cbd); });
+    auto&& scheduleResult =
+        _scheduleDbWorkFn([=, this](const executor::TaskExecutor::CallbackArgs& cbd) {
+            insertDocumentsCallback(cbd);
+        });
 
     if (!scheduleResult.isOK()) {
         Status newStatus = scheduleResult.getStatus().withContext(
-            str::stream() << "Error cloning collection '" << _sourceNss.ns() << "'");
+            str::stream() << "Error cloning collection '" << _sourceNss.toStringForErrorMsg()
+                          << "'");
         // We must throw an exception to terminate query.
         uassertStatusOK(newStatus);
     }
@@ -471,10 +503,8 @@ void CollectionCloner::insertDocumentsCallback(const executor::TaskExecutor::Cal
         // 'receivedBatches'.
         ++_stats.fetchedBatches;
         if (_documentsToInsert.size() == 0) {
-            LOGV2_WARNING(21145,
-                          "insertDocumentsCallback, but no documents to insert for ns:{namespace}",
-                          "insertDocumentsCallback, but no documents to insert",
-                          logAttrs(_sourceNss));
+            LOGV2_WARNING(
+                21145, "insertDocumentsCallback, but no documents to insert", logAttrs(_sourceNss));
             return;
         }
         _documentsToInsert.swap(docs);
@@ -483,9 +513,14 @@ void CollectionCloner::insertDocumentsCallback(const executor::TaskExecutor::Cal
         _progressMeter.hit(int(docs.size()));
         invariant(_collLoader);
 
+        CollectionBulkLoader::ParseRecordIdAndDocFunc fn = (_collectionOptions.recordIdsReplicated)
+            ? ([](const BSONObj& doc) {
+                  return std::make_pair(RecordId(doc["r"].Long()), doc["d"].Obj());
+              })
+            : ([](const BSONObj& doc) { return std::make_pair(RecordId(0), doc); });
         // The insert must be done within the lock, because CollectionBulkLoader is not
         // thread safe.
-        uassertStatusOK(_collLoader->insertDocuments(docs.cbegin(), docs.cend()));
+        uassertStatusOK(_collLoader->insertDocuments(docs.cbegin(), docs.cend(), fn));
     }
 
     initialSyncHangDuringCollectionClone.executeIf(
@@ -499,14 +534,14 @@ void CollectionCloner::insertDocumentsCallback(const executor::TaskExecutor::Cal
             }
         },
         [&](const BSONObj& data) {
-            return data["namespace"].String() == _sourceNss.ns() &&
+            return NamespaceStringUtil::parseFailPointData(data, "namespace") == _sourceNss &&
                 static_cast<int>(_stats.documentsCopied) >= data["numDocsToClone"].numberInt();
         });
 }
 
 bool CollectionCloner::isMyFailPoint(const BSONObj& data) const {
-    auto nss = data["nss"].str();
-    return (nss.empty() || nss == _sourceNss.toString()) && BaseCloner::isMyFailPoint(data);
+    const auto fpNss = NamespaceStringUtil::parseFailPointData(data, "nss"_sd);
+    return (fpNss.isEmpty() || fpNss == _sourceNss) && BaseCloner::isMyFailPoint(data);
 }
 
 void CollectionCloner::waitForDatabaseWorkToComplete() {
@@ -525,7 +560,7 @@ std::string CollectionCloner::Stats::toString() const {
 
 BSONObj CollectionCloner::Stats::toBSON() const {
     BSONObjBuilder bob;
-    bob.append("ns", NamespaceStringUtil::serialize(nss));
+    bob.append("ns", NamespaceStringUtil::serialize(nss, SerializationContext::stateDefault()));
     append(&bob);
     return bob.obj();
 }
