@@ -28,17 +28,33 @@
  */
 
 
-#include "mongo/db/service_context.h"
-#include "mongo/platform/basic.h"
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+// IWYU pragma: no_include "ext/alloc_traits.h"
+#include <list>
+#include <mutex>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
 #include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
 #include "mongo/db/commands/list_collections_filter.h"
 #include "mongo/db/repl/database_cloner.h"
-#include "mongo/db/repl/database_cloner_common.h"
 #include "mongo/db/repl/database_cloner_gen.h"
-#include "mongo/db/server_feature_flags_gen.h"
+#include "mongo/db/service_context.h"
+#include "mongo/db/storage/storage_engine.h"
+#include "mongo/idl/idl_parser.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/mutex.h"
+#include "mongo/stdx/unordered_set.h"
 #include "mongo/util/assert_util.h"
+#include "mongo/util/clock_source.h"
+#include "mongo/util/duration.h"
+#include "mongo/util/namespace_string_util.h"
+#include "mongo/util/str.h"
+#include "mongo/util/uuid.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplicationInitialSync
 
@@ -56,7 +72,7 @@ DatabaseCloner::DatabaseCloner(const DatabaseName& dbName,
           "DatabaseCloner"_sd, sharedData, source, client, storageInterface, dbPool),
       _dbName(dbName),
       _listCollectionsStage("listCollections", this, &DatabaseCloner::listCollectionsStage) {
-    invariant(!dbName.db().empty());
+    invariant(!dbName.isEmpty());
     _stats.dbname = dbName;
 }
 
@@ -90,20 +106,15 @@ BaseCloner::AfterStageBehavior DatabaseCloner::listCollectionsStage() {
         }
 
         NamespaceString collectionNamespace(
-            NamespaceStringUtil::parseNamespaceFromResponse(_dbName, result.getName()));
+            NamespaceStringUtil::deserialize(_dbName, result.getName()));
         if (collectionNamespace.isSystem() && !collectionNamespace.isReplicated()) {
             LOGV2_DEBUG(21146,
                         1,
-                        "Skipping 'system' collection: {namespace}",
                         "Database cloner skipping 'system' collection",
                         logAttrs(collectionNamespace));
             continue;
         }
-        LOGV2_DEBUG(21147,
-                    2,
-                    "Allowing cloning of collectionInfo: {info}",
-                    "Allowing cloning of collectionInfo",
-                    "info"_attr = info);
+        LOGV2_DEBUG(21147, 2, "Allowing cloning of collectionInfo", "info"_attr = info);
 
         bool isDuplicate = seen.insert(result.getName().toString()).second;
         uassert(51005,
@@ -128,8 +139,8 @@ BaseCloner::AfterStageBehavior DatabaseCloner::listCollectionsStage() {
 }
 
 bool DatabaseCloner::isMyFailPoint(const BSONObj& data) const {
-    return data["database"].str() == _dbName.toStringWithTenantId() &&
-        BaseCloner::isMyFailPoint(data);
+    const auto fpDbName = DatabaseNameUtil::parseFailPointData(data, "database"_sd);
+    return fpDbName == _dbName && BaseCloner::isMyFailPoint(data);
 }
 
 void DatabaseCloner::postStage() {
@@ -157,22 +168,18 @@ void DatabaseCloner::postStage() {
         }
         auto collStatus = _currentCollectionCloner->run();
         if (collStatus.isOK()) {
-            LOGV2_DEBUG(21148,
-                        1,
-                        "collection clone finished: {namespace}",
-                        "Collection clone finished",
-                        logAttrs(sourceNss));
+            LOGV2_DEBUG(21148, 1, "Collection clone finished", logAttrs(sourceNss));
         } else {
             LOGV2_ERROR(21149,
-                        "collection clone for '{namespace}' failed due to {error}",
                         "Collection clone failed",
                         logAttrs(sourceNss),
                         "error"_attr = collStatus.toString());
-            setSyncFailedStatus({ErrorCodes::InitialSyncFailure,
-                                 collStatus
-                                     .withContext(str::stream() << "Error cloning collection '"
-                                                                << sourceNss.toString() << "'")
-                                     .toString()});
+            setSyncFailedStatus(
+                {ErrorCodes::InitialSyncFailure,
+                 collStatus
+                     .withContext(str::stream() << "Error cloning collection '"
+                                                << sourceNss.toStringForErrorMsg() << "'")
+                     .toString()});
         }
         {
             stdx::lock_guard<Latch> lk(_mutex);
@@ -197,17 +204,6 @@ DatabaseCloner::Stats DatabaseCloner::getStats() const {
     return stats;
 }
 
-std::string DatabaseCloner::Stats::toString() const {
-    return toBSON().toString();
-}
-
-BSONObj DatabaseCloner::Stats::toBSON() const {
-    BSONObjBuilder bob;
-    bob.append("dbname", DatabaseNameUtil::serialize(dbname));
-    append(&bob);
-    return bob.obj();
-}
-
 void DatabaseCloner::Stats::append(BSONObjBuilder* builder) const {
     builder->appendNumber("collections", static_cast<long long>(collections));
     builder->appendNumber("clonedCollections", static_cast<long long>(clonedCollections));
@@ -222,8 +218,8 @@ void DatabaseCloner::Stats::append(BSONObjBuilder* builder) const {
     }
 
     for (auto&& collection : collectionStats) {
-        BSONObjBuilder collectionBuilder(
-            builder->subobjStart(NamespaceStringUtil::serialize(collection.nss)));
+        BSONObjBuilder collectionBuilder(builder->subobjStart(
+            NamespaceStringUtil::serialize(collection.nss, SerializationContext::stateDefault())));
         collection.append(&collectionBuilder);
         collectionBuilder.doneFast();
     }
