@@ -28,18 +28,27 @@
  */
 
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/repl/drop_pending_collection_reaper.h"
-
 #include <algorithm>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <type_traits>
 #include <utility>
 
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/status.h"
 #include "mongo/db/client.h"
 #include "mongo/db/operation_context.h"
+#include "mongo/db/repl/drop_pending_collection_reaper.h"
 #include "mongo/db/repl/storage_interface.h"
 #include "mongo/db/service_context.h"
+#include "mongo/db/storage/recovery_unit.h"
+#include "mongo/db/transaction_resources.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/decorable.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplication
 
@@ -93,16 +102,14 @@ void DropPendingCollectionReaper::addDropPendingNamespace(
     if (std::find_if(lowerBound, upperBound, matcher) != upperBound) {
         LOGV2_FATAL_NOTRACE(
             40448,
-            "Failed to add drop-pending collection {dropPendingNamespace} with drop optime "
-            "{dropOpTime}: duplicate optime and namespace pair.",
             "Failed to add drop-pending collection: duplicate optime and namespace pair",
             "dropPendingNamespace"_attr = dropPendingNamespace,
             "dropOpTime"_attr = dropOpTime);
     }
 
     _dropPendingNamespaces.insert(std::make_pair(dropOpTime, dropPendingNamespace));
-    if (opCtx->lockState()->inAWriteUnitOfWork()) {
-        opCtx->recoveryUnit()->onRollback(
+    if (shard_role_details::getLocker(opCtx)->inAWriteUnitOfWork()) {
+        shard_role_details::getRecoveryUnit(opCtx)->onRollback(
             [this, dropPendingNamespace, dropOpTime](OperationContext*) {
                 stdx::lock_guard<Latch> lock(_mutex);
 
@@ -146,8 +153,6 @@ bool DropPendingCollectionReaper::rollBackDropPendingCollection(
         auto it = std::find_if(lowerBound, upperBound, matcher);
         if (it == upperBound) {
             LOGV2_WARNING(21154,
-                          "Cannot find drop-pending namespace at OpTime {opTime} for collection "
-                          "{namespace} to roll back.",
                           "Cannot find drop-pending namespace to roll back",
                           "opTime"_attr = opTime,
                           logAttrs(collectionNamespace));
@@ -158,9 +163,6 @@ bool DropPendingCollectionReaper::rollBackDropPendingCollection(
     }
 
     LOGV2(21152,
-          "Rolling back collection drop for {pendingNamespace} with drop OpTime {dropOpTime} to "
-          "namespace "
-          "{namespace}",
           "Rolling back collection drop",
           "pendingNamespace"_attr = pendingNss,
           "dropOpTime"_attr = opTime,
@@ -194,8 +196,6 @@ void DropPendingCollectionReaper::dropCollectionsOlderThan(OperationContext* opC
             const auto& dropOpTime = opTimeAndNamespace.first;
             const auto& nss = opTimeAndNamespace.second;
             LOGV2(21153,
-                  "Completing collection drop for {namespace} with drop optime {dropOpTime} "
-                  "(notification optime: {notificationOpTime})",
                   "Completing collection drop",
                   logAttrs(nss),
                   "dropOpTime"_attr = dropOpTime,
@@ -208,15 +208,12 @@ void DropPendingCollectionReaper::dropCollectionsOlderThan(OperationContext* opC
                 status = exceptionToStatus();
             }
             if (!status.isOK()) {
-                LOGV2_WARNING(
-                    21155,
-                    "Failed to remove drop-pending collection {namespace} with drop optime "
-                    "{dropOpTime} (notification optime: {notificationOpTime}): {error}",
-                    "Failed to remove drop-pending collection ",
-                    logAttrs(nss),
-                    "dropOpTime"_attr = dropOpTime,
-                    "notificationOpTime"_attr = opTime,
-                    "error"_attr = status);
+                LOGV2_WARNING(21155,
+                              "Failed to remove drop-pending collection ",
+                              logAttrs(nss),
+                              "dropOpTime"_attr = dropOpTime,
+                              "notificationOpTime"_attr = opTime,
+                              "error"_attr = status);
             }
         }
     }
