@@ -29,22 +29,29 @@
 
 #include "mongo/db/repl/hello_auth.h"
 
+#include <boost/optional.hpp>
+
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonobj.h"
 #include "mongo/client/authenticate.h"
 #include "mongo/db/auth/authentication_session.h"
 #include "mongo/db/auth/sasl_command_constants.h"
 #include "mongo/db/auth/sasl_commands.h"
 #include "mongo/db/auth/sasl_mechanism_registry.h"
+#include "mongo/db/auth/user_name.h"
 #include "mongo/db/commands/authentication_commands.h"
-#include "mongo/db/connection_health_metrics_parameter_gen.h"
-#include "mongo/db/stats/counters.h"
+#include "mongo/db/tenant_id.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/str.h"
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kAccessControl
 namespace mongo {
 
 void handleHelloAuth(OperationContext* opCtx,
                      const DatabaseName& dbName,
                      const HelloCommand& cmd,
-                     const bool isInitialHandshake,
                      BSONObjBuilder* result) {
     // saslSupportedMechs: UserName -> List[String]
     if (auto userNameVariant = cmd.getSaslSupportedMechs()) {
@@ -53,17 +60,9 @@ void handleHelloAuth(OperationContext* opCtx,
             opCtx, AuthenticationSession::StepType::kSaslSupportedMechanisms, [&](auto session) {
                 session->setUserNameForSaslSupportedMechanisms(userName);
 
-                auto& saslMechanismRegistry =
-                    SASLServerMechanismRegistry::get(opCtx->getServiceContext());
+                auto& saslMechanismRegistry = SASLServerMechanismRegistry::get(opCtx->getService());
                 saslMechanismRegistry.advertiseMechanismNamesForUser(opCtx, userName, result);
             });
-    } else if (isInitialHandshake && gEnableDetailedConnectionHealthMetricLogLines) {
-        auto client = opCtx->getClient();
-        auto metadata = ClientMetadata::get(client);
-        LOGV2(10483900,
-              "Connection not authenticating",
-              "client"_attr = client->getRemote(),
-              "doc"_attr = metadata ? metadata->getDocument() : BSONObj());
     }
 
     // speculativeAuthenticate: SaslStart -> SaslReply or Authenticate -> AuthenticateReply
