@@ -30,34 +30,51 @@
 
 #pragma once
 
+#include <boost/move/utility_core.hpp>
+#include <cstddef>
 #include <cstdint>
 #include <functional>
 #include <iosfwd>
 #include <memory>
+#include <mutex>
+#include <string>
+#include <vector>
 
 #include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
 #include "mongo/bson/bsonobj.h"
 #include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/bson/timestamp.h"
+#include "mongo/client/dbclient_connection.h"
 #include "mongo/client/fetcher.h"
 #include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/repl/callback_completion_guard.h"
 #include "mongo/db/repl/data_replicator_external_state.h"
 #include "mongo/db/repl/initial_sync_shared_data.h"
+#include "mongo/db/repl/initial_syncer_interface.h"
 #include "mongo/db/repl/multiapplier.h"
 #include "mongo/db/repl/oplog_applier.h"
 #include "mongo/db/repl/oplog_buffer.h"
+#include "mongo/db/repl/oplog_entry.h"
 #include "mongo/db/repl/oplog_fetcher.h"
 #include "mongo/db/repl/optime.h"
+#include "mongo/db/repl/replication_process.h"
 #include "mongo/db/repl/rollback_checker.h"
+#include "mongo/db/repl/storage_interface.h"
 #include "mongo/db/repl/sync_source_selector.h"
 #include "mongo/dbtests/mock/mock_dbclient_connection.h"
 #include "mongo/executor/scoped_task_executor.h"
+#include "mongo/executor/task_executor.h"
+#include "mongo/platform/atomic_word.h"
 #include "mongo/platform/mutex.h"
 #include "mongo/stdx/condition_variable.h"
 #include "mongo/util/concurrency/thread_pool.h"
+#include "mongo/util/concurrency/with_lock.h"
+#include "mongo/util/duration.h"
 #include "mongo/util/fail_point.h"
 #include "mongo/util/net/hostandport.h"
+#include "mongo/util/time_support.h"
 
 namespace mongo {
 namespace repl {
@@ -77,49 +94,6 @@ struct MemberState;
 class ReplicationProcess;
 class StorageInterface;
 
-struct InitialSyncerOptions {
-    /** Function to return optime of last operation applied on this node */
-    using GetMyLastOptimeFn = std::function<OpTime()>;
-
-    /** Function to update optime of last operation applied on this node */
-    using SetMyLastOptimeFn = std::function<void(const OpTimeAndWallTime&)>;
-
-    /** Function to reset all optimes on this node (e.g. applied & durable). */
-    using ResetOptimesFn = std::function<void()>;
-
-    /** Function to sets this node into a specific follower mode. */
-    using SetFollowerModeFn = std::function<bool(const MemberState&)>;
-
-    // Retry values
-    Milliseconds syncSourceRetryWait{1000};
-    Milliseconds initialSyncRetryWait{1000};
-
-    // InitialSyncer waits this long before retrying getApplierBatchCallback() if there are
-    // currently no operations available to apply or if the 'rsSyncApplyStop' failpoint is active.
-    // This default value is based on the duration in OplogBatcher::run().
-    Milliseconds getApplierBatchCallbackRetryWait{1000};
-
-    // Replication settings
-    NamespaceString localOplogNS = NamespaceString::kRsOplogNamespace;
-    NamespaceString remoteOplogNS = NamespaceString::kRsOplogNamespace;
-
-    GetMyLastOptimeFn getMyLastOptime;
-    SetMyLastOptimeFn setMyLastOptime;
-    ResetOptimesFn resetOptimes;
-
-    SyncSourceSelector* syncSourceSelector = nullptr;
-
-    // The oplog fetcher will restart the oplog tailing query this many times on non-cancellation
-    // failures.
-    std::uint32_t oplogFetcherMaxFetcherRestarts = 0;
-
-    std::string toString() const {
-        return str::stream() << "InitialSyncerOptions -- "
-                             << " localOplogNs: " << localOplogNS.toString()
-                             << " remoteOplogNS: " << remoteOplogNS.toString();
-    }
-};
-
 /**
  * The initial syncer provides services to keep collection in sync by replicating
  * changes via an oplog source to the local system storage.
@@ -131,28 +105,16 @@ struct InitialSyncerOptions {
  * Entry Points:
  *      -- startup: Start initial sync.
  */
-class InitialSyncer {
+class InitialSyncer : public InitialSyncerInterface {
     InitialSyncer(const InitialSyncer&) = delete;
     InitialSyncer& operator=(const InitialSyncer&) = delete;
 
 public:
-    /**
-     * Callback function to report last applied optime of initial sync.
-     */
-    typedef std::function<void(const StatusWith<OpTimeAndWallTime>& lastApplied)> OnCompletionFn;
-
     /**
      * Callback completion guard for initial syncer.
      */
     using OnCompletionGuard = CallbackCompletionGuard<StatusWith<OpTimeAndWallTime>>;
 
-    /**
-     * Type of function to create a database client
-     *
-     * Used for testing only.
-     */
-    using CreateClientFn = std::function<std::unique_ptr<DBClientConnection>()>;
-
     struct InitialSyncAttemptInfo {
         int durationMillis;
         Status status;
@@ -201,50 +163,40 @@ public:
         void append(BSONObjBuilder* builder) const;
     };
 
-    InitialSyncer(InitialSyncerOptions opts,
+    InitialSyncer(InitialSyncerInterface::Options opts,
                   std::unique_ptr<DataReplicatorExternalState> dataReplicatorExternalState,
-                  ThreadPool* writerPool,
+                  ThreadPool* workerPool,
                   StorageInterface* storage,
                   ReplicationProcess* replicationProcess,
                   const OnCompletionFn& onCompletion);
 
-    virtual ~InitialSyncer();
+    ~InitialSyncer() override;
 
     /**
      * Returns true if an initial sync is currently running or in the process of shutting down.
      */
     bool isActive() const;
 
-    /**
-     * Starts initial sync process, with the provided number of attempts
-     */
-    Status startup(OperationContext* opCtx, std::uint32_t maxAttempts) noexcept;
+    std::string getInitialSyncMethod() const final;
 
-    /**
-     * Shuts down replication if "start" has been called, and blocks until shutdown has completed.
-     */
-    Status shutdown();
+    bool allowLocalDbAccess() const final {
+        return true;
+    }
 
-    /**
-     * Block until inactive.
-     */
-    void join();
+    Status startup(OperationContext* opCtx, std::uint32_t maxAttempts) noexcept final;
+
+    Status shutdown() final;
+
+    void join() final;
 
     /**
      * Returns internal state in a loggable format.
      */
     std::string getDiagnosticString() const;
 
-    /**
-     * Returns stats about the progress of initial sync. If initial sync is not in progress it
-     * returns an empty BSON object.
-     */
-    BSONObj getInitialSyncProgress() const;
+    BSONObj getInitialSyncProgress() const final;
 
-    /**
-     * Cancels the current initial sync attempt if the initial syncer is active.
-     */
-    void cancelCurrentAttempt();
+    void cancelCurrentAttempt() final;
 
     /**
      *
@@ -589,7 +541,7 @@ private:
     void _appendInitialSyncProgressMinimal_inlock(BSONObjBuilder* bob) const;
     BSONObj _getInitialSyncProgress_inlock() const;
 
-    StatusWith<std::vector<OplogEntry>> _getNextApplierBatch_inlock();
+    StatusWith<OplogApplierBatch> _getNextApplierBatch_inlock();
 
     /**
      * Schedules a fetcher to get the last oplog entry from the sync source.
@@ -693,7 +645,7 @@ private:
     //      _mutex or be in a callback in _exec to read.
 
     mutable Mutex _mutex = MONGO_MAKE_LATCH("InitialSyncer::_mutex");           // (S)
-    const InitialSyncerOptions _opts;                                           // (R)
+    const InitialSyncerInterface::Options _opts;                                // (R)
     std::unique_ptr<DataReplicatorExternalState> _dataReplicatorExternalState;  // (R)
     std::shared_ptr<executor::TaskExecutor> _exec;                              // (R)
     std::unique_ptr<executor::ScopedTaskExecutor> _attemptExec;                 // (X)
@@ -703,7 +655,7 @@ private:
     // NetworkInterfaceMock is unaware of this and this causes our unit tests to deadlock.
     std::shared_ptr<executor::TaskExecutor> _clonerExec;               // (R)
     std::unique_ptr<executor::ScopedTaskExecutor> _clonerAttemptExec;  // (X)
-    ThreadPool* _writerPool;                                           // (R)
+    ThreadPool* _workerPool;                                           // (R)
     StorageInterface* _storage;                                        // (R)
     ReplicationProcess* _replicationProcess;                           // (S)
 
