@@ -28,8 +28,14 @@
  */
 
 #include "mongo/db/repl/initial_syncer_common_stats.h"
+
+#include <memory>
+
 #include "mongo/db/commands/server_status_metric.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/redaction.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplicationInitialSync
 
@@ -38,16 +44,15 @@ namespace mongo {
 namespace repl {
 namespace initial_sync_common_stats {
 
-CounterMetric initialSyncFailedAttempts("repl.initialSync.failedAttempts");
-CounterMetric initialSyncFailures("repl.initialSync.failures");
-CounterMetric initialSyncCompletes("repl.initialSync.completed");
+Counter64& initialSyncFailedAttempts = *MetricBuilder<Counter64>{"repl.initialSync.failedAttempts"};
+Counter64& initialSyncFailures = *MetricBuilder<Counter64>{"repl.initialSync.failures"};
+Counter64& initialSyncCompletes = *MetricBuilder<Counter64>{"repl.initialSync.completed"};
 
 void LogInitialSyncAttemptStats(const StatusWith<OpTimeAndWallTime>& attemptResult,
                                 bool hasRetries,
                                 const BSONObj& stats) {
     // Don't remove or change this log id as it is ingested to Atlas.
     LOGV2(21192,
-          "Initial sync status: {status}, initial sync attempt statistics: {statistics}",
           "Initial sync status and statistics",
           "status"_attr =
               attemptResult.isOK() ? "successful" : (hasRetries ? "in_progress" : "failed"),
