@@ -28,12 +28,16 @@
  */
 
 
-#include "mongo/platform/basic.h"
+#include <utility>
 
-#include <climits>
 
+#include "mongo/base/string_data.h"
 #include "mongo/db/repl/member_data.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/redaction.h"
+#include "mongo/util/assert_util_core.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplication
 
@@ -45,6 +49,7 @@ MemberData::MemberData() : _health(-1), _authIssue(false), _configIndex(-1), _is
     _lastResponse.setState(MemberState::RS_UNKNOWN);
     _lastResponse.setElectionTime(Timestamp());
     _lastResponse.setAppliedOpTimeAndWallTime(OpTimeAndWallTime());
+    _lastResponse.setWrittenOpTimeAndWallTime(OpTimeAndWallTime());
 }
 
 MemberData::HeartbeatChanges MemberData::setUpValues(Date_t now,
@@ -69,11 +74,13 @@ MemberData::HeartbeatChanges MemberData::setUpValues(Date_t now,
     if (!hbResponse.hasAppliedOpTime()) {
         hbResponse.setAppliedOpTimeAndWallTime(_lastResponse.getAppliedOpTimeAndWallTime());
     }
+    if (!hbResponse.hasWrittenOpTime()) {
+        hbResponse.setWrittenOpTimeAndWallTime(_lastResponse.getWrittenOpTimeAndWallTime());
+    }
     // Log if the state changes
     const bool memberStateChanged = _lastResponse.getState() != hbResponse.getState();
     if (memberStateChanged) {
         LOGV2(21215,
-              "Member {hostAndPort} is now in state {newState}",
               "Member is in new state",
               "hostAndPort"_attr = _hostAndPort.toString(),
               "newState"_attr = hbResponse.getState().toString());
@@ -81,6 +88,9 @@ MemberData::HeartbeatChanges MemberData::setUpValues(Date_t now,
 
     bool opTimeAdvanced =
         advanceLastAppliedOpTimeAndWallTime(hbResponse.getAppliedOpTimeAndWallTime(), now);
+    opTimeAdvanced =
+        advanceLastWrittenOpTimeAndWallTime(hbResponse.getWrittenOpTimeAndWallTime(), now) ||
+        opTimeAdvanced;
     auto durableOpTimeAndWallTime = hbResponse.hasDurableOpTime()
         ? hbResponse.getDurableOpTimeAndWallTime()
         : OpTimeAndWallTime();
@@ -106,7 +116,6 @@ void MemberData::setDownValues(Date_t now, const std::string& heartbeatMessage)
 
     if (_lastResponse.getState() != MemberState::RS_DOWN) {
         LOGV2(21216,
-              "Member {hostAndPort} is now in state DOWN - {heartbeatMessage}",
               "Member is now in state DOWN",
               "hostAndPort"_attr = _hostAndPort.toString(),
               "heartbeatMessage"_attr = redact(heartbeatMessage));
@@ -116,6 +125,7 @@ void MemberData::setDownValues(Date_t now, const std::string& heartbeatMessage)
     _lastResponse.setState(MemberState::RS_DOWN);
     _lastResponse.setElectionTime(Timestamp());
     _lastResponse.setAppliedOpTimeAndWallTime(OpTimeAndWallTime());
+    _lastResponse.setWrittenOpTimeAndWallTime(OpTimeAndWallTime());
     _lastResponse.setSyncingTo(HostAndPort());
 
     // The _lastAppliedOpTime/_lastDurableOpTime fields don't get cleared merely by missing a
@@ -132,7 +142,6 @@ void MemberData::setAuthIssue(Date_t now) {
 
     if (_lastResponse.getState() != MemberState::RS_UNKNOWN) {
         LOGV2(21217,
-              "Member {hostAndPort} is now in state UNKNOWN due to authentication issue.",
               "Member is now in state UNKNOWN due to authentication issue",
               "hostAndPort"_attr = _hostAndPort.toString());
     }
@@ -141,9 +150,18 @@ void MemberData::setAuthIssue(Date_t now) {
     _lastResponse.setState(MemberState::RS_UNKNOWN);
     _lastResponse.setElectionTime(Timestamp());
     _lastResponse.setAppliedOpTimeAndWallTime(OpTimeAndWallTime());
+    _lastResponse.setWrittenOpTimeAndWallTime(OpTimeAndWallTime());
     _lastResponse.setSyncingTo(HostAndPort());
 }
 
+void MemberData::setLastWrittenOpTimeAndWallTime(OpTimeAndWallTime opTime, Date_t now) {
+    invariant(opTime.opTime.isNull() || opTime.wallTime > Date_t());
+    _lastUpdate = now;
+    _lastUpdateStale = false;
+    _lastWrittenOpTime = opTime.opTime;
+    _lastWrittenWallTime = opTime.wallTime;
+}
+
 void MemberData::setLastAppliedOpTimeAndWallTime(OpTimeAndWallTime opTime, Date_t now) {
     invariant(opTime.opTime.isNull() || opTime.wallTime > Date_t());
     _lastUpdate = now;
@@ -156,12 +174,20 @@ void MemberData::setLastDurableOpTimeAndWallTime(OpTimeAndWallTime opTime, Date_
     invariant(opTime.opTime.isNull() || opTime.wallTime > Date_t());
     _lastUpdate = now;
     _lastUpdateStale = false;
-    // Since _lastDurableOpTime is set asynchronously from _lastAppliedOpTime, it is possible that
-    // 'opTime' is ahead of _lastAppliedOpTime.
-    if (_lastAppliedOpTime >= opTime.opTime) {
-        _lastDurableOpTime = opTime.opTime;
-        _lastDurableWallTime = opTime.wallTime;
+    _lastDurableOpTime = opTime.opTime;
+    _lastDurableWallTime = opTime.wallTime;
+}
+
+
+bool MemberData::advanceLastWrittenOpTimeAndWallTime(OpTimeAndWallTime opTime, Date_t now) {
+    invariant(opTime.opTime.isNull() || opTime.wallTime > Date_t());
+    _lastUpdate = now;
+    _lastUpdateStale = false;
+    if (_lastWrittenOpTime < opTime.opTime) {
+        setLastWrittenOpTimeAndWallTime(opTime, now);
+        return true;
     }
+    return false;
 }
 
 bool MemberData::advanceLastAppliedOpTimeAndWallTime(OpTimeAndWallTime opTime, Date_t now) {
