@@ -27,62 +27,105 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
 #include <benchmark/benchmark.h>
+#include <chrono>
+#include <cstddef>
+#include <cstdint>
+#include <limits>
+#include <memory>
+#include <string>
+#include <utility>
 #include <vector>
 
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/status.h"
+#include "mongo/base/string_data.h"
 #include "mongo/bson/bsonmisc.h"
 #include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/bson/bsontypes_util.h"
+#include "mongo/bson/oid.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/db/catalog/collection.h"
 #include "mongo/db/catalog/collection_impl.h"
+#include "mongo/db/catalog/collection_options.h"
 #include "mongo/db/catalog/create_collection.h"
+#include "mongo/db/catalog/database_holder.h"
 #include "mongo/db/catalog/database_holder_impl.h"
 #include "mongo/db/client.h"
+#include "mongo/db/concurrency/d_concurrency.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
 #include "mongo/db/cursor_manager.h"
 #include "mongo/db/global_settings.h"
+#include "mongo/db/index_builds_coordinator.h"
 #include "mongo/db/index_builds_coordinator_mongod.h"
 #include "mongo/db/namespace_string.h"
+#include "mongo/db/op_observer/op_observer.h"
 #include "mongo/db/op_observer/op_observer_impl.h"
 #include "mongo/db/op_observer/op_observer_registry.h"
-#include "mongo/db/op_observer/oplog_writer_impl.h"
+#include "mongo/db/op_observer/operation_logger_impl.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/repl/drop_pending_collection_reaper.h"
+#include "mongo/db/repl/member_state.h"
+#include "mongo/db/repl/oplog.h"
+#include "mongo/db/repl/oplog_applier.h"
+#include "mongo/db/repl/oplog_applier_batcher.h"
 #include "mongo/db/repl/oplog_applier_impl.h"
+#include "mongo/db/repl/oplog_buffer.h"
 #include "mongo/db/repl/oplog_buffer_blocking_queue.h"
+#include "mongo/db/repl/oplog_entry.h"
+#include "mongo/db/repl/optime.h"
 #include "mongo/db/repl/repl_settings.h"
 #include "mongo/db/repl/replication_consistency_markers_mock.h"
 #include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/repl/replication_coordinator_mock.h"
+#include "mongo/db/repl/storage_interface.h"
 #include "mongo/db/repl/storage_interface_impl.h"
 #include "mongo/db/s/collection_sharding_state.h"
-#include "mongo/db/s/collection_sharding_state_factory_standalone.h"
+#include "mongo/db/s/collection_sharding_state_factory_shard.h"
+#include "mongo/db/server_options.h"
 #include "mongo/db/service_context.h"
 #include "mongo/db/service_entry_point_mongod.h"
+#include "mongo/db/session/session_catalog.h"
 #include "mongo/db/session/session_catalog_mongod.h"
-#include "mongo/db/storage/execution_control/concurrency_adjustment_parameters_gen.h"
+#include "mongo/db/session_manager_mongod.h"
+#include "mongo/db/storage/recovery_unit.h"
 #include "mongo/db/storage/recovery_unit_noop.h"
-#include "mongo/db/storage/storage_engine_parameters_gen.h"
+#include "mongo/db/storage/storage_engine_init.h"
 #include "mongo/db/storage/storage_options.h"
+#include "mongo/db/storage/write_unit_of_work.h"
 #include "mongo/db/transaction/session_catalog_mongod_transaction_interface_impl.h"
-#include "mongo/idl/server_parameter_test_util.h"
-#include "mongo/logv2/log_domain_global.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/log_component_settings.h"
 #include "mongo/logv2/log_manager.h"
+#include "mongo/logv2/log_severity.h"
+#include "mongo/s/sharding_state.h"
+#include "mongo/transport/service_entry_point.h"
 #include "mongo/unittest/temp_dir.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/clock_source.h"
 #include "mongo/util/clock_source_mock.h"
 #include "mongo/util/concurrency/thread_pool.h"
-#include "mongo/util/exit.h"
+#include "mongo/util/duration.h"
+#include "mongo/util/fail_point.h"
+#include "mongo/util/periodic_runner.h"
 #include "mongo/util/periodic_runner_factory.h"
 #include "mongo/util/time_support.h"
+#include "mongo/util/uuid.h"
+#include "mongo/util/version/releases.h"
 
 namespace mongo {
 namespace {
 
+constexpr std::size_t kOplogBufferSize = 256 * 1024 * 1024;
+constexpr std::size_t kOplogBufferCount = std::numeric_limits<std::size_t>::max();
+
 class TestServiceContext {
 public:
     TestServiceContext() {
-        // Disable execution control.
-        gStorageEngineConcurrencyAdjustmentAlgorithm = "fixedConcurrentTransactions";
-
         // Disable server info logging so that the benchmark output is cleaner.
         logv2::LogManager::global().getGlobalSettings().setMinimumLoggedSeverity(
             mongo::logv2::LogComponent::kDefault, mongo::logv2::LogSeverity::Error());
@@ -97,7 +140,7 @@ public:
         setGlobalServiceContext(ServiceContext::make());
         _svcCtx = getGlobalServiceContext();
 
-        _svcCtx->setServiceEntryPoint(std::make_unique<ServiceEntryPointMongod>(_svcCtx));
+        _svcCtx->getService()->setServiceEntryPoint(std::make_unique<ServiceEntryPointMongod>());
 
         auto fastClock = std::make_unique<ClockSourceMock>();
         // Timestamps are split into two 32-bit integers, seconds and "increments". Currently (but
@@ -125,7 +168,7 @@ public:
         storageGlobalParams.dbpath = _tempDir->path();
         storageGlobalParams.ephemeral = false;
 
-        Client::initThread("oplog application main");
+        Client::initThread("oplog application main", getGlobalServiceContext()->getService());
         _client = Client::getCurrent();
 
         repl::ReplSettings replSettings;
@@ -136,10 +179,21 @@ public:
         repl::ReplicationCoordinator::set(
             _svcCtx, std::unique_ptr<repl::ReplicationCoordinator>(_replCoord));
 
-        auto startupOpCtx = _svcCtx->makeOperationContext(&cc());
-        initializeStorageEngine(startupOpCtx.get(),
-                                StorageEngineInitFlags::kAllowNoLockFile |
-                                    StorageEngineInitFlags::kSkipMetadataFile);
+        // Disable fast shutdown so that WT can free memory.
+        globalFailPointRegistry().find("WTDisableFastShutDown")->setMode(FailPoint::alwaysOn);
+
+        {
+            auto initializeStorageEngineOpCtx = _svcCtx->makeOperationContext(&cc());
+            shard_role_details::setRecoveryUnit(
+                initializeStorageEngineOpCtx.get(),
+                std::make_unique<RecoveryUnitNoop>(),
+                WriteUnitOfWork::RecoveryUnitState::kNotInUnitOfWork);
+
+            initializeStorageEngine(initializeStorageEngineOpCtx.get(),
+                                    StorageEngineInitFlags::kAllowNoLockFile |
+                                        StorageEngineInitFlags::kSkipMetadataFile);
+        }
+
         DatabaseHolder::set(_svcCtx, std::make_unique<DatabaseHolderImpl>());
         repl::StorageInterface::set(_svcCtx, std::make_unique<repl::StorageInterfaceImpl>());
         auto storageInterface = repl::StorageInterface::get(_svcCtx);
@@ -150,26 +204,29 @@ public:
 
         auto registry = std::make_unique<OpObserverRegistry>();
         registry->addObserver(
-            std::make_unique<OpObserverImpl>(std::make_unique<OplogWriterImpl>()));
+            std::make_unique<OpObserverImpl>(std::make_unique<OperationLoggerImpl>()));
         _svcCtx->setOpObserver(std::move(registry));
+        ShardingState::create(_svcCtx);
         CollectionShardingStateFactory::set(
-            _svcCtx, std::make_unique<CollectionShardingStateFactoryStandalone>(_svcCtx));
+            _svcCtx, std::make_unique<CollectionShardingStateFactoryShard>(_svcCtx));
 
         MongoDSessionCatalog::set(
             _svcCtx,
             std::make_unique<MongoDSessionCatalog>(
                 std::make_unique<MongoDSessionCatalogTransactionInterfaceImpl>()));
 
-        _oplogApplierThreadPool = repl::makeReplWriterPool();
+        _oplogBuffer =
+            std::make_unique<repl::OplogBufferBlockingQueue>(kOplogBufferSize, kOplogBufferCount);
+        _oplogApplierThreadPool = repl::makeReplWorkerPool();
 
         // Act as a secondary to get optimizations due to parallizing 'prepare' oplog entries. But
         // do not include in the benchmark the time to write to the oplog.
         repl::OplogApplier::Options oplogApplierOptions(
             repl::OplogApplication::Mode::kSecondary,
-            false /*allowNamespaceNotFoundErrorsOnCrudOps*/,
-            true /*skipWritesToOplog*/);
+            false /* allowNamespaceNotFoundErrorsOnCrudOps */,
+            true /* skipWritesToOplog */);
         _oplogApplier = std::make_unique<repl::OplogApplierImpl>(nullptr,
-                                                                 &_oplogBuffer,
+                                                                 _oplogBuffer.get(),
                                                                  &repl::noopOplogApplierObserver,
                                                                  _replCoord,
                                                                  &_consistencyMarkers,
@@ -177,7 +234,7 @@ public:
                                                                  oplogApplierOptions,
                                                                  _oplogApplierThreadPool.get());
 
-        _svcCtx->notifyStartupComplete();
+        _svcCtx->notifyStorageStartupRecoveryComplete();
     }
 
     ~TestServiceContext() {
@@ -202,8 +259,8 @@ public:
         auto databaseHolder = DatabaseHolder::get(opCtx);
         databaseHolder->closeAll(opCtx);
 
-        // Shut down storage engine and free memory.
-        shutdownGlobalStorageEngineCleanly(_svcCtx, false /* memLeakAllowed */);
+        // Shut down storage engine.
+        shutdownGlobalStorageEngineCleanly(_svcCtx);
     }
 
     // Shut down the storage engine, clear the dbpath, and restart the storage engine with empty
@@ -219,10 +276,12 @@ public:
         storageGlobalParams.dbpath = _tempDir->path();
         storageGlobalParams.ephemeral = false;
 
-        auto uniqueOpCtx = _svcCtx->makeOperationContext(&cc());
-        uniqueOpCtx->setRecoveryUnit(std::make_unique<RecoveryUnitNoop>(),
-                                     WriteUnitOfWork::RecoveryUnitState::kNotInUnitOfWork);
-        initializeStorageEngine(uniqueOpCtx.get(),
+        auto initializeStorageEngineOpCtx = _svcCtx->makeOperationContext(&cc());
+        shard_role_details::setRecoveryUnit(initializeStorageEngineOpCtx.get(),
+                                            std::make_unique<RecoveryUnitNoop>(),
+                                            WriteUnitOfWork::RecoveryUnitState::kNotInUnitOfWork);
+
+        initializeStorageEngine(initializeStorageEngineOpCtx.get(),
                                 StorageEngineInitFlags::kAllowNoLockFile |
                                     StorageEngineInitFlags::kSkipMetadataFile |
                                     StorageEngineInitFlags::kForRestart);
@@ -251,7 +310,7 @@ private:
     std::unique_ptr<repl::OplogApplier> _oplogApplier;
 
     // This class also owns objects necessary for `_oplogApplier`.
-    repl::OplogBufferBlockingQueue _oplogBuffer;
+    std::unique_ptr<repl::OplogBufferBlockingQueue> _oplogBuffer;
     repl::ReplicationConsistencyMarkersMock _consistencyMarkers;
     std::unique_ptr<ThreadPool> _oplogApplierThreadPool;
     boost::optional<unittest::TempDir> _tempDir;
@@ -481,8 +540,9 @@ public:
 
     void applyOplog(OperationContext* opCtx) {
         while (!_testSvcCtx->getOplogApplier()->getBuffer()->isEmpty()) {
-            auto oplogBatch = invariantStatusOK(
-                _testSvcCtx->getOplogApplier()->getNextApplierBatch(opCtx, _batchLimits));
+            auto oplogBatch = invariantStatusOK(_testSvcCtx->getOplogApplier()->getNextApplierBatch(
+                                                    opCtx, _batchLimits))
+                                  .releaseBatch();
 
             const auto lastOpInBatch = oplogBatch.back();
             const auto lastOpTimeInBatch = lastOpInBatch.getOpTime();
@@ -506,9 +566,9 @@ private:
 
     std::vector<BSONObj> _oplogEntries;
     UUID _foobarUUID;
-    NamespaceString _foobarNs{"foo.bar"_sd};
-    repl::OplogBatcher::BatchLimits _batchLimits{std::numeric_limits<std::size_t>::max(),
-                                                 std::numeric_limits<std::size_t>::max()};
+    NamespaceString _foobarNs = NamespaceString::createNamespaceString_forTest("foo.bar"_sd);
+    repl::OplogApplierBatcher::BatchLimits _batchLimits{std::numeric_limits<std::size_t>::max(),
+                                                        std::numeric_limits<std::size_t>::max()};
 };
 
 void runBMTest(TestServiceContext& testSvcCtx, Fixture& fixture, benchmark::State& state) {
