@@ -29,31 +29,81 @@
 
 #include "mongo/db/repl/oplog_applier_impl.h"
 
-#include "mongo/db/catalog/collection_catalog.h"
+// IWYU pragma: no_include "cxxabi.h"
+#include <absl/container/node_hash_map.h>
+#include <absl/meta/type_traits.h>
+#include <boost/cstdint.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional.hpp>
+#include <boost/optional/optional.hpp>
+#include <cstddef>
+#include <cstdint>
+#include <iterator>
+#include <memory>
+#include <mutex>
+#include <string>
+#include <utility>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/db/admission/execution_admission_context.h"
+#include "mongo/db/catalog/collection.h"
 #include "mongo/db/catalog/collection_write_path.h"
-#include "mongo/db/catalog/database.h"
-#include "mongo/db/catalog/database_holder.h"
-#include "mongo/db/catalog/document_validation.h"
 #include "mongo/db/catalog_raii.h"
 #include "mongo/db/change_stream_change_collection_manager.h"
 #include "mongo/db/change_stream_serverless_helpers.h"
 #include "mongo/db/client.h"
 #include "mongo/db/commands/fsync.h"
-#include "mongo/db/db_raii.h"
-#include "mongo/db/repl/apply_ops.h"
+#include "mongo/db/commands/server_status_metric.h"
+#include "mongo/db/concurrency/d_concurrency.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/repl/apply_ops_command_info.h"
+#include "mongo/db/repl/initial_syncer.h"
+#include "mongo/db/repl/member_state.h"
+#include "mongo/db/repl/oplog_applier_batcher.h"
 #include "mongo/db/repl/oplog_applier_utils.h"
-#include "mongo/db/repl/oplog_batcher.h"
-#include "mongo/db/repl/repl_server_parameters_gen.h"
+#include "mongo/db/repl/oplog_entry_gen.h"
+#include "mongo/db/repl/oplog_writer_impl.h"
+#include "mongo/db/repl/replication_metrics.h"
+#include "mongo/db/repl/split_prepare_session_manager.h"
 #include "mongo/db/repl/transaction_oplog_application.h"
-#include "mongo/db/session/logical_session_id.h"
+#include "mongo/db/server_options.h"
+#include "mongo/db/service_context.h"
 #include "mongo/db/stats/counters.h"
 #include "mongo/db/stats/timer_stats.h"
 #include "mongo/db/storage/control/journal_flusher.h"
+#include "mongo/db/storage/recovery_unit.h"
+#include "mongo/db/storage/storage_engine.h"
 #include "mongo/db/storage/storage_util.h"
+#include "mongo/db/storage/write_unit_of_work.h"
+#include "mongo/db/tenant_id.h"
+#include "mongo/db/transaction_resources.h"
+#include "mongo/logv2/attribute_storage.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/redaction.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/platform/mutex.h"
+#include "mongo/stdx/condition_variable.h"
+#include "mongo/stdx/thread.h"
 #include "mongo/stdx/unordered_map.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/clock_source.h"
+#include "mongo/util/concurrency/admission_context.h"
+#include "mongo/util/concurrency/mutex.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/duration.h"
 #include "mongo/util/fail_point.h"
 #include "mongo/util/log_with_sampling.h"
+#include "mongo/util/scopeguard.h"
+#include "mongo/util/str.h"
+#include "mongo/util/time_support.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplication
 
@@ -66,13 +116,13 @@ MONGO_FAIL_POINT_DEFINE(pauseBatchApplicationAfterWritingOplogEntries);
 MONGO_FAIL_POINT_DEFINE(hangAfterRecordingOpApplicationStartTime);
 
 // The oplog entries applied
-CounterMetric opsAppliedStats("repl.apply.ops");
+auto& opsAppliedStats = *MetricBuilder<Counter64>{"repl.apply.ops"};
 
 // Tracks the oplog application batch size.
-CounterMetric oplogApplicationBatchSize("repl.apply.batchSize");
+auto& oplogApplicationBatchSize = *MetricBuilder<Counter64>{"repl.apply.batchSize"};
 
 // Number and time of each ApplyOps worker pool round
-auto& applyBatchStats = makeServerStatusMetric<TimerStats>("repl.apply.batches");
+auto& applyBatchStats = *MetricBuilder<TimerStats>("repl.apply.batches");
 
 /**
  * Used for logging a report of ops that take longer than "slowMS" to apply. This is called
@@ -107,7 +157,8 @@ Status finishAndLogApply(OperationContext* opCtx,
 
             // Obtain storage specific statistics and log them if they exist.
             CurOp::get(opCtx)->debug().storageStats =
-                opCtx->recoveryUnit()->computeOperationStatisticsSinceLastCall();
+                shard_role_details::getRecoveryUnit(opCtx)
+                    ->computeOperationStatisticsSinceLastCall();
             CurOp::get(opCtx)->debug().reportStorageStats(&attrs);
 
             LOGV2(51801, "Applied op", attrs);
@@ -282,10 +333,8 @@ void _addOplogChainOpsToWriterVectors(OperationContext* opCtx,
     partialTxnList->clear();
 
     if (op->shouldPrepare()) {
-        // Prepared transaction operations should not have commands.
-        invariant(!shouldSerialize);
         OplogApplierUtils::addDerivedPrepares(
-            opCtx, op, &extractedOps, writerVectors, collPropertiesCache);
+            opCtx, op, &extractedOps, writerVectors, collPropertiesCache, shouldSerialize);
         return;
     }
 
@@ -293,72 +342,21 @@ void _addOplogChainOpsToWriterVectors(OperationContext* opCtx,
         opCtx, &extractedOps, writerVectors, collPropertiesCache, shouldSerialize);
 }
 
-Status _insertDocumentsToOplogAndChangeCollections(
-    OperationContext* opCtx,
-    std::vector<InsertStatement>::const_iterator begin,
-    std::vector<InsertStatement>::const_iterator end,
-    bool skipWritesToOplog) {
-    WriteUnitOfWork wunit(opCtx);
-
-    if (!skipWritesToOplog) {
-        AutoGetOplog autoOplog(opCtx, OplogAccessMode::kWrite);
-        auto& oplogColl = autoOplog.getCollection();
-        if (!oplogColl) {
-            return {ErrorCodes::NamespaceNotFound, "Oplog collection does not exist"};
-        }
-
-        auto status = collection_internal::insertDocuments(
-            opCtx, oplogColl, begin, end, nullptr /* OpDebug */, false /* fromMigrate */);
-        if (!status.isOK()) {
-            return status;
-        }
-    }
-
-    // Write the corresponding oplog entries to tenants respective change
-    // collections in the serverless.
-    if (change_stream_serverless_helpers::isChangeCollectionsModeActive()) {
-        auto status =
-            ChangeStreamChangeCollectionManager::get(opCtx).insertDocumentsToChangeCollection(
-                opCtx,
-                begin,
-                end,
-                !skipWritesToOplog /* hasAcquiredGlobalIXLock */,
-                nullptr /* OpDebug */);
-        if (!status.isOK()) {
-            return status;
-        }
-    }
-
-    wunit.commit();
-
-    return Status::OK();
-}
-
 void _setOplogApplicationWorkerOpCtxStates(OperationContext* opCtx) {
     // Do not enforce constraints.
     opCtx->setEnforceConstraints(false);
 
-    // Since we swap the locker in stash / unstash transaction resources,
-    // ShouldNotConflictWithSecondaryBatchApplicationBlock will touch the locker that has been
-    // destroyed by unstash in its destructor. Thus we set the flag explicitly.
-    opCtx->lockState()->setShouldConflictWithSecondaryBatchApplication(false);
-
     // When querying indexes, we return the record matching the key if it exists, or an adjacent
     // document. This means that it is possible for us to hit a prepare conflict if we query for an
     // incomplete key and an adjacent key is prepared.
     // We ignore prepare conflicts on secondaries because they may encounter prepare conflicts that
     // did not occur on the primary.
-    opCtx->recoveryUnit()->setPrepareConflictBehavior(
+    shard_role_details::getRecoveryUnit(opCtx)->setPrepareConflictBehavior(
         PrepareConflictBehavior::kIgnoreConflictsAllowWrites);
 
-    // Applying an Oplog batch is crucial to the stability of the Replica Set. We
-    // mark it as having Immediate priority so that it skips waiting for ticket
-    // acquisition and flow control.
-    opCtx->lockState()->setAdmissionPriority(AdmissionContext::Priority::kImmediate);
-
     // Ensure future transactions read without a timestamp.
     invariant(RecoveryUnit::ReadSource::kNoTimestamp ==
-              opCtx->recoveryUnit()->getTimestampReadSource());
+              shard_role_details::getRecoveryUnit(opCtx)->getTimestampReadSource());
 }
 }  // namespace
 
@@ -376,6 +374,7 @@ public:
 
 protected:
     void _recordApplied(const OpTimeAndWallTime& newOpTimeAndWallTime) {
+        _replCoord->setMyLastWrittenOpTimeAndWallTimeForward(newOpTimeAndWallTime);
         // We have to use setMyLastAppliedOpTimeAndWallTimeForward since this thread races with
         // ReplicationExternalStateImpl::onTransitionToPrimary.
         _replCoord->setMyLastAppliedOpTimeAndWallTimeForward(newOpTimeAndWallTime);
@@ -384,12 +383,6 @@ protected:
         signalOplogWaiters();
     }
 
-    void _recordDurable(const OpTimeAndWallTime& newOpTimeAndWallTime) {
-        // We have to use setMyLastDurableOpTimeAndWallTimeForward since this thread races with
-        // ReplicationExternalStateImpl::onTransitionToPrimary.
-        _replCoord->setMyLastDurableOpTimeAndWallTimeForward(newOpTimeAndWallTime);
-    }
-
 private:
     // Used to update the replication system's progress.
     ReplicationCoordinator* _replCoord;
@@ -400,7 +393,7 @@ public:
     ApplyBatchFinalizerForJournal(ReplicationCoordinator* replCoord)
         : ApplyBatchFinalizer(replCoord),
           _waiterThread{&ApplyBatchFinalizerForJournal::_run, this} {};
-    ~ApplyBatchFinalizerForJournal();
+    ~ApplyBatchFinalizerForJournal() override;
 
     void record(const OpTimeAndWallTime& newOpTimeAndWallTime) override;
 
@@ -442,11 +435,15 @@ void ApplyBatchFinalizerForJournal::record(const OpTimeAndWallTime& newOpTimeAnd
 }
 
 void ApplyBatchFinalizerForJournal::_run() {
-    Client::initThread("ApplyBatchFinalizerForJournal");
+    Client::initThread("ApplyBatchFinalizerForJournal",
+                       getGlobalServiceContext()->getService(ClusterRole::ShardServer));
+
+    {
+        stdx::lock_guard<Client> lk(cc());
+        cc().setSystemOperationUnkillableByStepdown(lk);
+    }
 
     while (true) {
-        OpTimeAndWallTime latestOpTimeAndWallTime = {OpTime(), Date_t()};
-
         {
             stdx::unique_lock<Latch> lock(_mutex);
             while (_latestOpTimeAndWallTime.opTime.isNull() && !_shutdownSignaled) {
@@ -457,13 +454,11 @@ void ApplyBatchFinalizerForJournal::_run() {
                 return;
             }
 
-            latestOpTimeAndWallTime = _latestOpTimeAndWallTime;
             _latestOpTimeAndWallTime = {OpTime(), Date_t()};
         }
 
         auto opCtx = cc().makeOperationContext();
         JournalFlusher::get(opCtx.get())->waitForJournalFlush();
-        _recordDurable(latestOpTimeAndWallTime);
     }
 }
 
@@ -476,13 +471,29 @@ OplogApplierImpl::OplogApplierImpl(executor::TaskExecutor* executor,
                                    ReplicationConsistencyMarkers* consistencyMarkers,
                                    StorageInterface* storageInterface,
                                    const OplogApplier::Options& options,
-                                   ThreadPool* writerPool)
+                                   ThreadPool* workerPool)
     : OplogApplier(executor, oplogBuffer, observer, options),
       _replCoord(replCoord),
-      _writerPool(writerPool),
+      _workerPool(workerPool),
       _storageInterface(storageInterface),
-      _consistencyMarkers(consistencyMarkers),
-      _beginApplyingOpTime(options.beginApplyingOpTime) {}
+      _consistencyMarkers(consistencyMarkers) {
+
+    // Change collections are always written as part of oplog application, even though
+    // in steady state mode where a separate OplogWriter thread is responsible for
+    // writing oplog entries. This is because the OplogWriter thread can be ahead of
+    // oplog application, but DDL ops on change collections (e.g. create/drop) must be
+    // first applied before we can perform writes on those collections.
+    _oplogWriter = std::make_unique<OplogWriterImpl>(
+        nullptr /* executor */,
+        nullptr /* writeBuffer */,
+        nullptr /* applyBuffer */,
+        workerPool,
+        replCoord,
+        storageInterface,
+        consistencyMarkers,
+        &noopOplogWriterObserver,
+        OplogWriter::Options(options.skipWritesToOplog, false /* skipWritesToChangeColl */));
+}
 
 void OplogApplierImpl::_run(OplogBuffer* oplogBuffer) {
     // Start up a thread from the batcher to pull from the oplog buffer into the batcher's oplog
@@ -495,12 +506,17 @@ void OplogApplierImpl::_run(OplogBuffer* oplogBuffer) {
     // arbiterOnly field for any member.
     invariant(!_replCoord->getMemberState().arbiter());
 
+    const auto useOplogWriter = feature_flags::gReduceMajorityWriteLatency.isEnabled(
+        serverGlobalParams.featureCompatibility.acquireFCVSnapshot());
+
+    // The OplogWriter will take care of journaling when featureFlagReduceMajorityWriteLatency
+    // is enabled.
     std::unique_ptr<ApplyBatchFinalizer> finalizer{
-        getGlobalServiceContext()->getStorageEngine()->isEphemeral()
+        (useOplogWriter || getGlobalServiceContext()->getStorageEngine()->isEphemeral())
             ? new ApplyBatchFinalizer(_replCoord)
             : new ApplyBatchFinalizerForJournal(_replCoord)};
 
-    while (true) {  // Exits on message from OplogBatcher.
+    while (true) {  // Exits on message from OplogApplierBatcher.
         // Use a new operation context each iteration, as otherwise we may appear to use a single
         // collection name to refer to collections with different UUIDs.
         const ServiceContext::UniqueOperationContext opCtxPtr = cc().makeOperationContext();
@@ -509,29 +525,26 @@ void OplogApplierImpl::_run(OplogBuffer* oplogBuffer) {
         // The oplog applier is crucial for stability of the replica set. As a result we mark it as
         // having Immediate priority. This makes the operation skip waiting for ticket acquisition
         // and flow control.
-        ScopedAdmissionPriorityForLock priority(opCtx.lockState(),
-                                                AdmissionContext::Priority::kImmediate);
+        ScopedAdmissionPriority<ExecutionAdmissionContext> priority(
+            &opCtx, AdmissionContext::Priority::kExempt);
 
         // For pausing replication in tests.
-        if (MONGO_unlikely(rsSyncApplyStop.shouldFail())) {
+        if (MONGO_unlikely(!useOplogWriter && rsSyncApplyStop.shouldFail())) {
             LOGV2(21229,
                   "Oplog Applier - rsSyncApplyStop fail point enabled. Blocking until fail "
                   "point is disabled");
             rsSyncApplyStop.pauseWhileSet(&opCtx);
         }
 
-        // Transition to SECONDARY state, if possible.
-        _replCoord->finishRecoveryIfEligible(&opCtx);
-
         // Blocks up to a second waiting for a batch to be ready to apply. If one doesn't become
         // ready in time, we'll loop again so we can do the above checks periodically.
-        OplogBatch ops = _oplogBatcher->getNextBatch(Seconds(1));
+        OplogApplierBatch ops = _oplogBatcher->getNextBatch(Seconds(1));
         if (ops.empty()) {
             if (ops.mustShutdown()) {
                 // Shut down and exit oplog application loop.
                 return;
             }
-            if (MONGO_unlikely(rsSyncApplyStop.shouldFail())) {
+            if (MONGO_unlikely(!useOplogWriter && rsSyncApplyStop.shouldFail())) {
                 continue;
             }
             if (ops.termWhenExhausted()) {
@@ -540,7 +553,7 @@ void OplogApplierImpl::_run(OplogBuffer* oplogBuffer) {
                 // they can be stale. We make sure the applier is still draining in the given term
                 // before and after the check, so that if the oplog buffer was exhausted, then
                 // it still will be.
-                _replCoord->signalDrainComplete(&opCtx, *ops.termWhenExhausted());
+                _replCoord->signalApplierDrainComplete(&opCtx, *ops.termWhenExhausted());
             }
             continue;  // Try again.
         }
@@ -554,10 +567,6 @@ void OplogApplierImpl::_run(OplogBuffer* oplogBuffer) {
 
         // Make sure the oplog doesn't go back in time or repeat an entry.
         if (firstOpTimeInBatch <= lastAppliedOpTimeAtStartOfBatch) {
-            LOGV2(10180701,
-                  "Oplog entry that is less than our last applied OpTime",
-                  "entry"_attr = redact(ops.front().toBSONForLogging()));
-
             fassert(34361,
                     Status(ErrorCodes::OplogOutOfOrder,
                            str::stream() << "Attempted to apply an oplog entry ("
@@ -567,7 +576,7 @@ void OplogApplierImpl::_run(OplogBuffer* oplogBuffer) {
         }
 
         // Don't allow the fsync+lock thread to see intermediate states of batch application.
-        stdx::lock_guard<SimpleMutex> fsynclk(filesLockedFsync);
+        stdx::lock_guard<SimpleMutex> fsynclk(oplogApplierLockedFsync);
 
         // Apply the operations in this batch. '_applyOplogBatch' returns the optime of the
         // last op that was applied, which should be the last optime in the batch.
@@ -594,115 +603,26 @@ void OplogApplierImpl::_run(OplogBuffer* oplogBuffer) {
                                 << " in the middle of batch application");
 
         // 2. Update oplog visibility by notifying the storage engine of the new oplog entries.
-        const bool orderedCommit = true;
-        _storageInterface->oplogDiskLocRegister(
-            &opCtx, lastOpTimeInBatch.getTimestamp(), orderedCommit);
+        // The OplogWriter will take care of visibility when featureFlagReduceMajorityWriteLatency
+        // is enabled.
+        if (!useOplogWriter) {
+            const bool orderedCommit = true;
+            _storageInterface->oplogDiskLocRegister(
+                &opCtx, lastOpTimeInBatch.getTimestamp(), orderedCommit);
+        }
 
         // 3. Finalize this batch. The finalizer advances the global timestamp to lastOpTimeInBatch.
         finalizer->record({lastOpTimeInBatch, lastWallTimeInBatch});
     }
 }
 
-
-// Schedules the writes to the oplog and the change collection for 'ops' into threadPool. The caller
-// must guarantee that 'ops' stays valid until all scheduled work in the thread pool completes.
-void scheduleWritesToOplogAndChangeCollection(OperationContext* opCtx,
-                                              StorageInterface* storageInterface,
-                                              ThreadPool* writerPool,
-                                              const std::vector<OplogEntry>& ops,
-                                              bool skipWritesToOplog) {
-    // Skip performing any writes during the startup recovery when running in the non-serverless
-    // environment.
-    if (skipWritesToOplog && !change_stream_serverless_helpers::isChangeCollectionsModeActive()) {
-        return;
-    }
-
-    auto makeOplogWriterForRange = [storageInterface, &ops, skipWritesToOplog](size_t begin,
-                                                                               size_t end) {
-        // The returned function will be run in a separate thread after this returns. Therefore
-        // all captures other than 'ops' must be by value since they will not be available. The
-        // caller guarantees that 'ops' will stay in scope until the spawned threads complete.
-        return [storageInterface, &ops, begin, end, skipWritesToOplog](auto status) {
-            invariant(status);
-            auto opCtx = cc().makeOperationContext();
-
-            // Oplog writes are crucial to the stability of the replica set. We mark the operations
-            // as having Immediate priority so that it skips waiting for ticket acquisition and flow
-            // control.
-            ScopedAdmissionPriorityForLock priority(opCtx->lockState(),
-                                                    AdmissionContext::Priority::kImmediate);
-
-            UnreplicatedWritesBlock uwb(opCtx.get());
-            ShouldNotConflictWithSecondaryBatchApplicationBlock shouldNotConflictBlock(
-                opCtx->lockState());
-
-            std::vector<InsertStatement> docs;
-            docs.reserve(end - begin);
-            for (size_t i = begin; i < end; i++) {
-                docs.emplace_back(InsertStatement{ops[i].getEntry().getRaw(),
-                                                  ops[i].getOpTime().getTimestamp(),
-                                                  ops[i].getOpTime().getTerm()});
-            }
-
-            // TODO SERVER-67168 the 'nsOrUUID' is used only to log the debug message when retrying
-            // inserts on the oplog and change collections. The 'writeConflictRetry' assumes
-            // operations are done on a single namespace. But the method
-            // '_insertDocumentsToOplogAndChangeCollections' can perform inserts on the oplog and
-            // multiple change collections, ie. several namespaces. As such 'writeConflictRetry'
-            // will not log the correct namespace when retrying. Refactor this code to log the
-            // correct namespace in the log message.
-            NamespaceStringOrUUID nsOrUUID = !skipWritesToOplog
-                ? NamespaceString::kRsOplogNamespace
-                : NamespaceString::makeChangeCollectionNSS(boost::none /* tenantId */);
-
-            fassert(6663400,
-                    storage_helpers::insertBatchAndHandleRetry(
-                        opCtx.get(), nsOrUUID, docs, [&](auto* opCtx, auto begin, auto end) {
-                            return _insertDocumentsToOplogAndChangeCollections(
-                                opCtx, begin, end, skipWritesToOplog);
-                        }));
-        };
-    };
-
-    // We want to be able to take advantage of bulk inserts so we don't use multiple threads if it
-    // would result too little work per thread. This also ensures that we can amortize the
-    // setup/teardown overhead across many writes.
-    const size_t kMinOplogEntriesPerThread = 16;
-    const bool enoughToMultiThread =
-        ops.size() >= kMinOplogEntriesPerThread * writerPool->getStats().options.maxThreads;
-
-    // Storage engines support parallel writes to the oplog because they are required to ensure that
-    // oplog entries are ordered correctly, even if inserted out-of-order.
-    if (!enoughToMultiThread) {
-        writerPool->schedule(makeOplogWriterForRange(0, ops.size()));
-        return;
-    }
-
-    const size_t numOplogThreads = writerPool->getStats().options.maxThreads;
-    const size_t numOpsPerThread = ops.size() / numOplogThreads;
-    for (size_t thread = 0; thread < numOplogThreads; thread++) {
-        size_t begin = thread * numOpsPerThread;
-        size_t end = (thread == numOplogThreads - 1) ? ops.size() : begin + numOpsPerThread;
-        writerPool->schedule(makeOplogWriterForRange(begin, end));
-    }
-}
-
 StatusWith<OpTime> OplogApplierImpl::_applyOplogBatch(OperationContext* opCtx,
                                                       std::vector<OplogEntry> ops) {
     invariant(!ops.empty());
 
-    LOGV2_DEBUG(21230,
-                2,
-                "replication batch size is {size}",
-                "Replication batch size",
-                "size"_attr = ops.size());
+    LOGV2_DEBUG(21230, 2, "Oplog application batch size", "size"_attr = ops.size());
 
-    // Stop all readers until we're done. This also prevents doc-locking engines from deleting old
-    // entries from the oplog until we finish writing.
-    Lock::ParallelBatchWriterMode pbwm(opCtx);
-
-    invariant(_replCoord);
-    if (_replCoord->getApplierState() == ReplicationCoordinator::ApplierState::Stopped) {
+    if (_replCoord->getOplogSyncState() == ReplicationCoordinator::OplogSyncState::Stopped) {
         LOGV2_FATAL_CONTINUE(21234, "Attempting to replicate ops while primary");
         return {ErrorCodes::CannotApplyOplogWhilePrimary,
                 "attempting to replicate ops while primary"};
@@ -711,23 +631,28 @@ StatusWith<OpTime> OplogApplierImpl::_applyOplogBatch(OperationContext* opCtx,
     // Increment the batch size stat.
     oplogApplicationBatchSize.increment(ops.size());
 
-    std::vector<WorkerMultikeyPathInfo> multikeyVector(_writerPool->getStats().options.maxThreads);
+    std::vector<WorkerMultikeyPathInfo> multikeyVector(_workerPool->getStats().options.maxThreads);
     {
         // Each node records cumulative batch application stats for itself using this timer.
         TimerHolder timer(&applyBatchStats);
 
         // We must wait for the all work we've dispatched to complete before leaving this block
         // because the spawned threads refer to objects on the stack
-        ON_BLOCK_EXIT([&] { _writerPool->waitForIdle(); });
+        ON_BLOCK_EXIT([&] { _workerPool->waitForIdle(); });
 
-        // Write batch of ops into oplog.
-        if (!getOptions().skipWritesToOplog) {
-            _consistencyMarkers->setOplogTruncateAfterPoint(
-                opCtx, _replCoord->getMyLastAppliedOpTime().getTimestamp());
-        }
-
-        scheduleWritesToOplogAndChangeCollection(
-            opCtx, _storageInterface, _writerPool, ops, getOptions().skipWritesToOplog);
+        // Write ops into the oplog collection and change collections if needed.
+        //
+        // - In steady state mode, a separate OplogWriter is responsible for writing the oplog
+        // collection, but the change collections must be written as part of oplog application
+        // because DDL ops (e.g. create/drop) on change collections are explicitly replicated,
+        // and so must be first applied before writes are performed to those collections.
+        //
+        // - In initial sync mode, the applier is responsible for writing the oplog collection
+        // as well as the change collections.
+        //
+        // - In recovering modes, there is no need to write to the oplog collection, and the
+        // applier is responsible for writing the change collections.
+        bool writtenOplog = _oplogWriter->scheduleWriteOplogBatch(opCtx, ops);
 
         // Holds 'pseudo operations' generated by secondaries to aid in replication.
         // Keep in scope until all operations in 'ops' and 'derivedOps' have been applied.
@@ -739,14 +664,16 @@ StatusWith<OpTime> OplogApplierImpl::_applyOplogBatch(OperationContext* opCtx,
         std::vector<std::vector<OplogEntry>> derivedOps;
 
         std::vector<std::vector<ApplierOperation>> writerVectors(
-            _writerPool->getStats().options.maxThreads);
+            _workerPool->getStats().options.maxThreads);
         _fillWriterVectors(opCtx, &ops, &writerVectors, &derivedOps);
 
-        // Wait for writes to finish before applying ops.
-        _writerPool->waitForIdle();
+        // Wait for oplog writes to finish.
+        if (writtenOplog) {
+            _oplogWriter->waitForScheduledWrites(opCtx);
+        }
 
-        // Use this fail point to hold the PBWM lock after we have written the oplog entries but
-        // before we have applied them.
+        // Use this fail point to hang after we have written the oplog entries but before we have
+        // applied them.
         if (MONGO_unlikely(pauseBatchApplicationAfterWritingOplogEntries.shouldFail())) {
             LOGV2(21231,
                   "pauseBatchApplicationAfterWritingOplogEntries fail point enabled. Blocking "
@@ -758,13 +685,8 @@ StatusWith<OpTime> OplogApplierImpl::_applyOplogBatch(OperationContext* opCtx,
         const bool isDataConsistent =
             _consistencyMarkers->getMinValid(opCtx) < ops.front().getOpTime();
 
-        // Reset consistency markers in case the node fails while applying ops.
-        if (!getOptions().skipWritesToOplog) {
-            _consistencyMarkers->setOplogTruncateAfterPoint(opCtx, Timestamp());
-        }
-
         {
-            std::vector<Status> statusVector(_writerPool->getStats().options.maxThreads,
+            std::vector<Status> statusVector(_workerPool->getStats().options.maxThreads,
                                              Status::OK());
             // Doles out all the work to the writer pool threads. writerVectors is not modified,
             // but applyOplogBatchPerWorker will modify the vectors that it contains.
@@ -773,7 +695,7 @@ StatusWith<OpTime> OplogApplierImpl::_applyOplogBatch(OperationContext* opCtx,
                 if (writerVectors[i].empty())
                     continue;
 
-                _writerPool->schedule([this,
+                _workerPool->schedule([this,
                                        &writer = writerVectors.at(i),
                                        &status = statusVector.at(i),
                                        &multikeyVector = multikeyVector.at(i),
@@ -788,7 +710,7 @@ StatusWith<OpTime> OplogApplierImpl::_applyOplogBatch(OperationContext* opCtx,
                 });
             }
 
-            _writerPool->waitForIdle();
+            _workerPool->waitForIdle();
 
             // If any of the statuses is not ok, return error.
             for (auto it = statusVector.cbegin(); it != statusVector.cend(); ++it) {
@@ -796,11 +718,6 @@ StatusWith<OpTime> OplogApplierImpl::_applyOplogBatch(OperationContext* opCtx,
                 if (!status.isOK()) {
                     LOGV2_FATAL_CONTINUE(
                         21235,
-                        "Failed to apply batch of operations. Number of operations in "
-                        "batch: {numOperationsInBatch}. First operation: {firstOperation}. "
-                        "Last operation: "
-                        "{lastOperation}. Oplog application failed in writer thread "
-                        "{failedWriterThread}: {error}",
                         "Failed to apply batch of operations",
                         "numOperationsInBatch"_attr = ops.size(),
                         "firstOperation"_attr = redact(ops.front().toBSONForLogging()),
@@ -822,7 +739,7 @@ StatusWith<OpTime> OplogApplierImpl::_applyOplogBatch(OperationContext* opCtx,
         }
     }
 
-    // Use this fail point to hold the PBWM lock and prevent the batch from completing.
+    // Use this fail point to prevent the batch from completing.
     if (MONGO_unlikely(pauseBatchApplicationBeforeCompletion.shouldFail())) {
         LOGV2(21232,
               "pauseBatchApplicationBeforeCompletion fail point enabled. Blocking until fail "
@@ -839,8 +756,7 @@ StatusWith<OpTime> OplogApplierImpl::_applyOplogBatch(OperationContext* opCtx,
     }
 
     Timestamp firstTimeInBatch = ops.front().getTimestamp();
-    // Set any indexes to multikey that this batch ignored. This must be done while holding the
-    // parallel batch writer mode lock.
+    // Set any indexes to multikey that this batch ignored.
     for (const WorkerMultikeyPathInfo& infoVector : multikeyVector) {
         for (const MultikeyPathInfo& info : infoVector) {
             // We timestamp every multikey write with the first timestamp in the batch. It is always
@@ -975,8 +891,7 @@ void OplogApplierImpl::_deriveOpsAndFillWriterVectors(
 
         // Extract applyOps operations and fill writers with extracted operations.
         if (op.isTerminalApplyOps()) {
-            // applyOps entries generated by a transaction must have a prevOpTime.
-            if (auto prevOpTime = op.getPrevWriteOpTimeInTransaction()) {
+            if (op.applyOpsIsLinkedTransactionally()) {
                 // On commit of unprepared transactions, get transactional operations from the
                 // oplog and fill writers with those operations.
                 // Flush partialTxnList operations for current transaction.
@@ -991,8 +906,6 @@ void OplogApplierImpl::_deriveOpsAndFillWriterVectors(
                 invariant(partialTxnList->empty(), op.toStringForLogging());
             } else {
                 // The applyOps entry was not generated as part of a transaction.
-                invariant(!op.getPrevWriteOpTimeInTransaction());
-
                 auto& extractedOps = retryImageRectifier.storeExtractedOpsAndDeletes(
                     ApplyOps::extractOperations(op), derivedOps, false /* isPrepared */);
 
@@ -1003,30 +916,27 @@ void OplogApplierImpl::_deriveOpsAndFillWriterVectors(
             continue;
         }
 
-        // (Ignore FCV check): This feature flag doesn't have any upgrade/downgrade concerns.
-        if (repl::feature_flags::gApplyPreparedTxnsInParallel.isEnabledAndIgnoreFCVUnsafe()) {
-            // Prepare entries in secondary mode do not come in their own batch, extract applyOps
-            // operations and fill writers with the extracted operations.
-            if (op.shouldPrepare() && (getOptions().mode == OplogApplication::Mode::kSecondary)) {
-                auto* partialTxnList = getPartialTxnList(op);
-                _addOplogChainOpsToWriterVectors(opCtx,
-                                                 &op,
-                                                 partialTxnList,
-                                                 derivedOps,
-                                                 writerVectors,
-                                                 &collPropertiesCache,
-                                                 &retryImageRectifier);
-                continue;
-            }
+        // Prepare entries in secondary mode do not come in their own batch, extract applyOps
+        // operations and fill writers with the extracted operations.
+        if (op.shouldPrepare() && (getOptions().mode == OplogApplication::Mode::kSecondary)) {
+            auto* partialTxnList = getPartialTxnList(op);
+            _addOplogChainOpsToWriterVectors(opCtx,
+                                             &op,
+                                             partialTxnList,
+                                             derivedOps,
+                                             writerVectors,
+                                             &collPropertiesCache,
+                                             &retryImageRectifier);
+            continue;
+        }
 
-            // Fill the writers with commit or abort operation. Depending on whether the operation
-            // refers to a split prepare, it might also be split into multiple ops.
-            if (op.isPreparedCommitOrAbort() &&
-                (getOptions().mode == OplogApplication::Mode::kSecondary)) {
-                OplogApplierUtils::addDerivedCommitsOrAborts(
-                    opCtx, &op, writerVectors, &collPropertiesCache);
-                continue;
-            }
+        // Fill the writers with commit or abort operation. Depending on whether the operation
+        // refers to a split prepare, it might also be split into multiple ops.
+        if (op.isPreparedCommitOrAbort() &&
+            (getOptions().mode == OplogApplication::Mode::kSecondary)) {
+            OplogApplierUtils::addDerivedCommitsOrAborts(
+                opCtx, &op, writerVectors, &collPropertiesCache);
+            continue;
         }
 
         // If we see a commitTransaction command that is a part of a prepared transaction during
@@ -1080,6 +990,12 @@ Status applyOplogEntryOrGroupedInserts(OperationContext* opCtx,
                                        const OplogEntryOrGroupedInserts& entryOrGroupedInserts,
                                        OplogApplication::Mode oplogApplicationMode,
                                        const bool isDataConsistent) {
+    // Applying an Oplog batch is crucial to the stability of the Replica Set. We
+    // mark it as having Immediate priority so that it skips waiting for ticket
+    // acquisition and flow control.
+    ScopedAdmissionPriority<ExecutionAdmissionContext> skipTicketAcquisition(
+        opCtx, AdmissionContext::Priority::kExempt);
+
     // Certain operations like prepareTransaction might reset the recovery unit or lock state
     // due to doing things like stashTransactionResources. So we restore the necessary states
     // here every time before applying a new entry or grouped inserts.
@@ -1133,6 +1049,12 @@ Status OplogApplierImpl::applyOplogBatchPerWorker(OperationContext* opCtx,
                                                   std::vector<ApplierOperation>* ops,
                                                   WorkerMultikeyPathInfo* workerMultikeyPathInfo,
                                                   const bool isDataConsistent) {
+    // Applying an Oplog batch is crucial to the stability of the Replica Set. We
+    // mark it as having Immediate priority so that it skips waiting for ticket
+    // acquisition and flow control.
+    ScopedAdmissionPriority<ExecutionAdmissionContext> skipTicketAcquisition(
+        opCtx, AdmissionContext::Priority::kExempt);
+
     UnreplicatedWritesBlock uwb(opCtx);
     _setOplogApplicationWorkerOpCtxStates(opCtx);
 
