@@ -27,28 +27,68 @@
  *    it in the license file.
  */
 
+#include <absl/container/node_hash_map.h>
 #include <absl/hash/hash.h>
+#include <absl/meta/type_traits.h>
+#include <algorithm>
+#include <boost/cstdint.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/optional.hpp>
+#include <cstddef>
+#include <cstdint>
+#include <functional>
+#include <memory>
+#include <set>
+#include <string>
+#include <utility>
 
-#include "mongo/db/catalog_raii.h"
-#include "mongo/platform/basic.h"
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonelement_comparator.h"
+#include "mongo/bson/bsonobj.h"
 #include "mongo/db/catalog/collection.h"
 #include "mongo/db/catalog/collection_catalog.h"
+#include "mongo/db/catalog/database.h"
 #include "mongo/db/catalog/document_validation.h"
+#include "mongo/db/catalog_raii.h"
 #include "mongo/db/concurrency/exception_util.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
 #include "mongo/db/curop.h"
 #include "mongo/db/curop_metrics.h"
+#include "mongo/db/database_name.h"
 #include "mongo/db/db_raii.h"
+#include "mongo/db/feature_flag.h"
 #include "mongo/db/global_index.h"
 #include "mongo/db/multitenancy_gen.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/repl/oplog_applier_utils.h"
+#include "mongo/db/repl/oplog_constraint_violation_logger.h"
+#include "mongo/db/repl/oplog_entry_gen.h"
+#include "mongo/db/repl/read_concern_args.h"
 #include "mongo/db/repl/repl_server_parameters_gen.h"
+#include "mongo/db/repl/replication_coordinator.h"
+#include "mongo/db/repl/split_prepare_session_manager.h"
 #include "mongo/db/server_feature_flags_gen.h"
+#include "mongo/db/server_options.h"
+#include "mongo/db/session/logical_session_id.h"
+#include "mongo/db/shard_role.h"
 #include "mongo/db/stats/counters.h"
-#include "mongo/util/fail_point.h"
-
+#include "mongo/db/tenant_id.h"
+#include "mongo/db/transaction_resources.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/redaction.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/fail_point.h"
+#include "mongo/util/str.h"
+#include "mongo/util/uuid.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplication
 
@@ -119,18 +159,7 @@ uint32_t getWriterId(OperationContext* opCtx,
                      CachedCollectionProperties* collPropertiesCache,
                      uint32_t numWriters,
                      boost::optional<uint32_t> forceWriterId = boost::none) {
-    boost::optional<size_t> idHash;
-    NamespaceString nss = op->isGlobalIndexCrudOpType()
-        ? NamespaceString::makeGlobalIndexNSS(op->getUuid().value())
-        : op->getNss();
-
-    if (op->isCrudOpType()) {
-        auto collProperties = collPropertiesCache->getCollectionProperties(opCtx, nss);
-        processCrudOp(opCtx, op, collProperties, idHash);
-    }
-
-    auto hash = idHash ? absl::HashOf(nss, *idHash) : absl::HashOf(nss);
-
+    auto hash = OplogApplierUtils::getOplogEntryHash(opCtx, op, collPropertiesCache);
     return (forceWriterId ? *forceWriterId : hash) % numWriters;
 }
 
@@ -195,6 +224,22 @@ void addTopLevelCommitOrAbort(OperationContext* opCtx,
 }
 }  // namespace
 
+uint32_t OplogApplierUtils::getOplogEntryHash(OperationContext* opCtx,
+                                              OplogEntry* op,
+                                              CachedCollectionProperties* collPropertiesCache) {
+    boost::optional<size_t> idHash;
+    NamespaceString nss = op->isGlobalIndexCrudOpType()
+        ? NamespaceString::makeGlobalIndexNSS(op->getUuid().value())
+        : op->getNss();
+
+    if (op->isCrudOpType()) {
+        auto collProperties = collPropertiesCache->getCollectionProperties(opCtx, nss);
+        processCrudOp(opCtx, op, collProperties, idHash);
+    }
+
+    return idHash ? absl::HashOf(nss, *idHash) : absl::HashOf(nss);
+}
+
 uint32_t OplogApplierUtils::addToWriterVector(
     OperationContext* opCtx,
     OplogEntry* op,
@@ -255,7 +300,8 @@ void OplogApplierUtils::addDerivedPrepares(
     OplogEntry* prepareOp,
     std::vector<OplogEntry>* derivedOps,
     std::vector<std::vector<ApplierOperation>>* writerVectors,
-    CachedCollectionProperties* collPropertiesCache) {
+    CachedCollectionProperties* collPropertiesCache,
+    bool serial) {
 
     // Get the SplitPrepareSessionManager to be used to create split sessions.
     auto splitSessManager = ReplicationCoordinator::get(opCtx)->getSplitPrepareSessionManager();
@@ -289,10 +335,19 @@ void OplogApplierUtils::addDerivedPrepares(
         // split session when moving the ops to the real writer vector.
         std::set<uint32_t> writerIds;
         std::vector<std::vector<const OplogEntry*>> bufWriterVectors(writerVectors->size());
+        boost::optional<uint32_t> serialWriterId;
         for (auto&& op : *derivedOps) {
             auto writerId = getWriterId(opCtx, &op, collPropertiesCache, writerVectors->size());
-            addToWriterVectorImpl(writerId, &bufWriterVectors, &op);
-            writerIds.emplace(writerId);
+            if (serial && !serialWriterId) {
+                serialWriterId.emplace(writerId);
+                writerIds.emplace(*serialWriterId);
+            }
+            if (serialWriterId) {
+                addToWriterVectorImpl(*serialWriterId, &bufWriterVectors, &op);
+            } else {
+                addToWriterVectorImpl(writerId, &bufWriterVectors, &op);
+                writerIds.emplace(writerId);
+            }
         }
 
         const auto& sessionInfos = splitSessFunc({writerIds.begin(), writerIds.end()});
@@ -393,9 +448,9 @@ Status OplogApplierUtils::applyOplogEntryOrGroupedInsertsCommon(
     const NamespaceString nss(op->getNss());
     auto opType = op->getOpType();
 
-    const auto fcvSnapshot = serverGlobalParams.featureCompatibility.acquireFCVSnapshot();
-    if ((gMultitenancySupport && fcvSnapshot.isVersionInitialized() &&
-         gFeatureFlagRequireTenantID.isEnabled(fcvSnapshot))) {
+    if (gMultitenancySupport &&
+        gFeatureFlagRequireTenantID.isEnabled(
+            serverGlobalParams.featureCompatibility.acquireFCVSnapshot())) {
         invariant(op->getTid() == nss.tenantId());
     } else {
         invariant(op->getTid() == boost::none);
@@ -405,97 +460,130 @@ Status OplogApplierUtils::applyOplogEntryOrGroupedInsertsCommon(
         incrementOpsAppliedStats();
         return Status::OK();
     } else if (DurableOplogEntry::isCrudOpType(opType)) {
-        auto status =
-            writeConflictRetry(opCtx, "applyOplogEntryOrGroupedInserts_CRUD", nss.ns(), [&] {
-                // Need to throw instead of returning a status for it to be properly ignored.
+        auto status = writeConflictRetry(opCtx, "applyOplogEntryOrGroupedInserts_CRUD", nss, [&] {
+            // Need to throw instead of returning a status for it to be properly ignored.
+            try {
+                boost::optional<CollectionAcquisition> coll;
+                Database* db = nullptr;
+
+                // If the collection UUID does not resolve, acquire the collection using the
+                // namespace. This is so we reach `applyOperation_inlock` below and invalidate
+                // the preimage / postimage for the op if applicable.
+
+                // TODO SERVER-41371 / SERVER-73661 this code is difficult to maintain and
+                // needs to be done everywhere this situation is possible. We should try
+                // to consolidate this into applyOperation_inlock.
                 try {
-                    boost::optional<AutoGetCollection> autoColl;
-                    Database* db = nullptr;
+                    coll.emplace(
+                        acquireCollection(opCtx,
+                                          {getNsOrUUID(nss, *op),
+                                           AcquisitionPrerequisites::kPretendUnsharded,
+                                           repl::ReadConcernArgs::get(opCtx),
+                                           AcquisitionPrerequisites::kWrite},
+                                          fixLockModeForSystemDotViewsChanges(nss, MODE_IX)));
 
-                    // If the collection UUID does not resolve, acquire the collection using the
-                    // namespace. This is so we reach `applyOperation_inlock` below and invalidate
-                    // the preimage / postimage for the op if applicable.
-
-                    // TODO SERVER-41371 / SERVER-73661 this code is difficult to maintain and
-                    // needs to be done everywhere this situation is possible. We should try
-                    // to consolidate this into applyOperation_inlock.
-                    try {
-                        autoColl.emplace(opCtx,
-                                         getNsOrUUID(nss, *op),
-                                         fixLockModeForSystemDotViewsChanges(nss, MODE_IX));
-                        db = autoColl->getDb();
-                    } catch (ExceptionFor<ErrorCodes::NamespaceNotFound>& ex) {
-                        if (!isDataConsistent) {
-                            autoColl.emplace(
-                                opCtx, nss, fixLockModeForSystemDotViewsChanges(nss, MODE_IX));
-                            db = autoColl->ensureDbExists(opCtx);
-                        } else {
-                            throw ex;
-                        }
-                    }
-
-                    uassert(ErrorCodes::NamespaceNotFound,
-                            str::stream() << "missing database ("
-                                          << nss.dbName().toStringForErrorMsg() << ")",
-                            db);
-                    OldClientContext ctx(opCtx, autoColl->getNss(), db);
-
-                    // We convert updates to upserts in secondary mode when the
-                    // oplogApplicationEnforcesSteadyStateConstraints parameter is false, to avoid
-                    // failing on the constraint that updates in steady state mode always update
-                    // an existing document.
-                    //
-                    // In initial sync and recovery modes we always ignore errors about missing
-                    // documents on update, so there is no reason to convert the updates to upsert.
-
-                    bool shouldAlwaysUpsert = !oplogApplicationEnforcesSteadyStateConstraints &&
-                        oplogApplicationMode == OplogApplication::Mode::kSecondary;
-                    Status status = applyOperation_inlock(opCtx,
-                                                          db,
-                                                          entryOrGroupedInserts,
-                                                          shouldAlwaysUpsert,
-                                                          oplogApplicationMode,
-                                                          isDataConsistent,
-                                                          incrementOpsAppliedStats);
-                    if (!status.isOK() && status.code() == ErrorCodes::WriteConflict) {
-                        throwWriteConflictException(
-                            str::stream() << "WriteConflict caught when applying operation."
-                                          << " Original error: " << status.reason());
-                    }
-                    return status;
+                    AutoGetDb autoDb(opCtx, coll->nss().dbName(), MODE_IX);
+                    db = autoDb.getDb();
                 } catch (ExceptionFor<ErrorCodes::NamespaceNotFound>& ex) {
-                    // This can happen in initial sync or recovery modes (when a delete of the
-                    // namespace appears later in the oplog), but we will ignore it in the caller.
-                    //
-                    // When we're not enforcing steady-state constraints, the error is ignored
-                    // only for deletes, on the grounds that deleting from a non-existent collection
-                    // is a no-op.
-                    if (opType == OpTypeEnum::kDelete &&
-                        !oplogApplicationEnforcesSteadyStateConstraints &&
-                        oplogApplicationMode == OplogApplication::Mode::kSecondary) {
-                        if (opCounters) {
-                            const auto& opObj = redact(op->toBSONForLogging());
-                            opCounters->gotDeleteFromMissingNamespace();
-                            logOplogConstraintViolation(
-                                opCtx,
-                                op->getNss(),
-                                OplogConstraintViolationEnum::kDeleteOnMissingNs,
-                                "delete",
-                                opObj,
-                                boost::none /* status */);
-                        }
-                        return Status::OK();
-                    }
+                    if (!isDataConsistent) {
+                        coll.emplace(
+                            acquireCollection(opCtx,
+                                              {nss,
+                                               AcquisitionPrerequisites::kPretendUnsharded,
+                                               repl::ReadConcernArgs::get(opCtx),
+                                               AcquisitionPrerequisites::kWrite},
+                                              fixLockModeForSystemDotViewsChanges(nss, MODE_IX)));
 
-                    ex.addContext(str::stream() << "Failed to apply operation: "
-                                                << redact(entryOrGroupedInserts.toBSON()));
-                    throw;
+                        AutoGetDb autoDb(opCtx, coll->nss().dbName(), MODE_IX);
+                        db = autoDb.ensureDbExists(opCtx);
+                    } else {
+                        throw ex;
+                    }
                 }
-            });
+
+                invariant(coll);
+                uassert(ErrorCodes::NamespaceNotFound,
+                        str::stream()
+                            << "missing database (" << nss.dbName().toStringForErrorMsg() << ")",
+                        db);
+                OldClientContext ctx(opCtx, coll->nss(), db);
+
+                // We convert updates to upserts in secondary mode when the
+                // oplogApplicationEnforcesSteadyStateConstraints parameter is false, to avoid
+                // failing on the constraint that updates in steady state mode always update
+                // an existing document.
+                //
+                // In initial sync and recovery modes we always ignore errors about missing
+                // documents on update, so there is no reason to convert the updates to upsert.
+
+                bool shouldAlwaysUpsert = !oplogApplicationEnforcesSteadyStateConstraints &&
+                    oplogApplicationMode == OplogApplication::Mode::kSecondary;
+                Status status = applyOperation_inlock(opCtx,
+                                                      *coll,
+                                                      entryOrGroupedInserts,
+                                                      shouldAlwaysUpsert,
+                                                      oplogApplicationMode,
+                                                      isDataConsistent,
+                                                      incrementOpsAppliedStats);
+                if (!status.isOK() && status.code() == ErrorCodes::WriteConflict) {
+                    throwWriteConflictException(str::stream()
+                                                << "WriteConflict caught when applying operation."
+                                                << " Original error: " << status.reason());
+                }
+                return status;
+            } catch (ExceptionFor<ErrorCodes::NamespaceNotFound>& ex) {
+                // This can happen in initial sync or recovery modes (when a delete of the
+                // namespace appears later in the oplog), but we will ignore it in the caller.
+                //
+                // When we're not enforcing steady-state constraints, the error is ignored
+                // only for deletes, on the grounds that deleting from a non-existent collection
+                // is a no-op.
+                if (opType == OpTypeEnum::kDelete &&
+                    !oplogApplicationEnforcesSteadyStateConstraints &&
+                    oplogApplicationMode == OplogApplication::Mode::kSecondary) {
+                    LOGV2_DEBUG(8994800,
+                                1,
+                                "Acceptable error when applying CRUD op",
+                                "error"_attr = redact(ex),
+                                "op"_attr = redact(entryOrGroupedInserts.toBSON()));
+                    if (opCounters) {
+                        const auto& opObj = redact(op->toBSONForLogging());
+                        opCounters->gotDeleteFromMissingNamespace();
+                        logOplogConstraintViolation(
+                            opCtx,
+                            op->getNss(),
+                            OplogConstraintViolationEnum::kDeleteOnMissingNs,
+                            "delete",
+                            opObj,
+                            boost::none /* status */);
+                    }
+                    return Status::OK();
+                }
+
+                ex.addContext(str::stream() << "Failed to apply operation: "
+                                            << redact(entryOrGroupedInserts.toBSON()));
+                throw;
+            } catch (ExceptionFor<ErrorCodes::CommandNotSupportedOnView>& ex) {
+                // This can happen in initial sync or unstable recovery mode when a time-series
+                // collection is created in place of a dropped regular collection and oplog entries
+                // are being applied that were originally performed on the regular collection.
+                if (oplogApplicationMode == OplogApplication::Mode::kInitialSync ||
+                    oplogApplicationMode == OplogApplication::Mode::kUnstableRecovering) {
+                    LOGV2_DEBUG(8994801,
+                                1,
+                                "Acceptable error when applying CRUD op",
+                                "error"_attr = redact(ex),
+                                "op"_attr = redact(entryOrGroupedInserts.toBSON()));
+                    return Status::OK();
+                }
+
+                throw;
+            }
+        });
         return status;
     } else if (opType == OpTypeEnum::kCommand) {
         auto status =
-            writeConflictRetry(opCtx, "applyOplogEntryOrGroupedInserts_command", nss.ns(), [&] {
+            writeConflictRetry(opCtx, "applyOplogEntryOrGroupedInserts_command", nss, [&] {
                 // A special case apply for commands to avoid implicit database creation.
                 Status status = applyCommand_inlock(opCtx, op, oplogApplicationMode);
                 incrementOpsAppliedStats();
@@ -508,7 +596,10 @@ Status OplogApplierUtils::applyOplogEntryOrGroupedInsertsCommon(
                     LOGV2(5863600,
                           "Hanging due to 'hangAfterApplyingCollectionDropOplogEntry' failpoint.");
                 },
-                [&](const BSONObj& data) { return (nss.db() == data["dbName"].str()); });
+                [&](const BSONObj& data) {
+                    const auto fpDbName = DatabaseNameUtil::parseFailPointData(data, "dbName");
+                    return nss.dbName() == fpDbName;
+                });
         }
         return status;
     }
@@ -547,7 +638,7 @@ Status OplogApplierUtils::applyOplogBatchCommon(
 
         // If we didn't create a group, try to apply the op individually.
         try {
-            const Status status =
+            Status status =
                 applyOplogEntryOrGroupedInserts(opCtx, op, oplogApplicationMode, isDataConsistent);
 
             if (!status.isOK()) {
@@ -566,12 +657,20 @@ Status OplogApplierUtils::applyOplogBatchCommon(
                 }
 
                 LOGV2_FATAL_CONTINUE(21237,
-                                     "Error applying operation ({oplogEntry}): {error}",
                                      "Error applying operation",
                                      "oplogEntry"_attr = redact(op->toBSONForLogging()),
                                      "error"_attr = causedBy(redact(status)));
                 return status;
             }
+        } catch (const ExceptionFor<ErrorCodes::DuplicateKey>& e) {
+            auto info = e.extraInfo<DuplicateKeyErrorInfo>();
+            LOGV2_FATAL_CONTINUE(5689600,
+                                 "Writer worker caught duplicate key exception",
+                                 "keyPattern"_attr = info->getKeyPattern(),
+                                 "keyValue"_attr = redact(info->getDuplicatedKeyValue()),
+                                 "error"_attr = redact(e.reason()),
+                                 "oplogEntry"_attr = redact(op->toBSONForLogging()));
+            return e.toStatus();
         } catch (const DBException& e) {
             // SERVER-24927 If we have a NamespaceNotFound exception, then this document will be
             // dropped before initial sync or recovery ends anyways and we should ignore it.
@@ -587,7 +686,6 @@ Status OplogApplierUtils::applyOplogBatchCommon(
             }
 
             LOGV2_FATAL_CONTINUE(21238,
-                                 "writer worker caught exception: {error} on: {oplogEntry}",
                                  "Writer worker caught exception",
                                  "error"_attr = redact(e),
                                  "oplogEntry"_attr = redact(op->toBSONForLogging()));
