@@ -26,41 +26,60 @@
  *    exception statement from all source files in the program, then also delete
  *    it in the license file.
  */
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplication
-
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/repl/oplog_buffer_collection.h"
 
 #include <algorithm>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
 #include <iterator>
+#include <mutex>
 #include <numeric>
+#include <utility>
+#include <vector>
 
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
 #include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/bson/simple_bsonobj_comparator.h"
 #include "mongo/bson/util/bson_extract.h"
+#include "mongo/db/catalog/clustered_collection_options_gen.h"
+#include "mongo/db/catalog/clustered_collection_util.h"
 #include "mongo/db/catalog/collection_options.h"
 #include "mongo/db/catalog/document_validation.h"
+#include "mongo/db/catalog_raii.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
 #include "mongo/db/dbdirectclient.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/ops/single_write_result_gen.h"
 #include "mongo/db/ops/write_ops_exec.h"
+#include "mongo/db/ops/write_ops_gen.h"
+#include "mongo/db/query/index_bounds.h"
+#include "mongo/db/repl/oplog_buffer_collection.h"
 #include "mongo/db/repl/storage_interface.h"
 #include "mongo/util/assert_util.h"
+#include "mongo/util/str.h"
+
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplication
+
 
 namespace mongo {
 namespace repl {
 
 namespace {
 
-const StringData kDefaultOplogCollectionNamespace = "local.temp_oplog_buffer"_sd;
 const StringData kOplogEntryFieldName = "entry"_sd;
 const StringData kIdFieldName = "_id"_sd;
 const StringData kTimestampFieldName = "ts"_sd;
 const StringData kIdIdxName = "_id_"_sd;
 
+const Timestamp kInvalidLastPushedTimestamp(0, 1);
 }  // namespace
 
 NamespaceString OplogBufferCollection::getDefaultNamespace() {
-    return NamespaceString(kDefaultOplogCollectionNamespace);
+    return NamespaceString::kDefaultOplogCollectionNamespace;
 }
 
 std::tuple<BSONObj, Timestamp> OplogBufferCollection::addIdToDocument(const BSONObj& orig) {
@@ -104,11 +123,11 @@ void OplogBufferCollection::startup(OperationContext* opCtx) {
     stdx::lock_guard<Latch> lk(_mutex);
     // If we are starting from an existing collection, we must populate the in memory state of the
     // buffer.
-    auto sizeResult = _storageInterface->getCollectionSize(opCtx, _nss);
-    fassert(40403, sizeResult);
-    _size = sizeResult.getValue();
+    _size = uassertStatusOK(_storageInterface->getCollectionSize(opCtx, _nss));
     _sizeIsValid = true;
 
+    _count = uassertStatusOK(_storageInterface->getCollectionCount(opCtx, _nss));
+
     // We always start from the beginning, with _lastPoppedKey being empty. This is safe because
     // it is always safe to replay old oplog entries in order. We explicitly reset all fields
     // since nothing prevents reusing an OplogBufferCollection, and the underlying collection may
@@ -121,15 +140,6 @@ void OplogBufferCollection::startup(OperationContext* opCtx) {
 
 void OplogBufferCollection::_updateLastPushedTimestampFromCollection(WithLock,
                                                                      OperationContext* opCtx) {
-    auto countResult = _storageInterface->getCollectionCount(opCtx, _nss);
-    fassert(40404, countResult);
-    _count = countResult.getValue();
-
-    if (_count == 0) {
-        _lastPushedTimestamp = {};
-        return;
-    }
-
     auto lastPushedObj = _lastDocumentPushed_inlock(opCtx);
     if (lastPushedObj) {
         auto lastPushedId = lastPushedObj->getObjectField(kIdFieldName);
@@ -155,7 +165,8 @@ void OplogBufferCollection::shutdown(OperationContext* opCtx) {
 
 void OplogBufferCollection::push(OperationContext* opCtx,
                                  Batch::const_iterator begin,
-                                 Batch::const_iterator end) {
+                                 Batch::const_iterator end,
+                                 boost::optional<const Cost&> cost) {
     if (begin == end) {
         return;
     }
@@ -181,10 +192,18 @@ void OplogBufferCollection::preload(OperationContext* opCtx,
     if (begin == end) {
         return;
     }
+
+    ScopeGuard failToPreloadGuard([this] {
+        stdx::unique_lock lk(_mutex);
+        _lastPushedTimestamp = kInvalidLastPushedTimestamp;
+    });
+
     stdx::lock_guard<Latch> lk(_mutex);
     invariant(_lastPoppedKey.isEmpty());
     _push(lk, opCtx, begin, end);
     _updateLastPushedTimestampFromCollection(lk, opCtx);
+
+    failToPreloadGuard.dismiss();
 }
 
 void OplogBufferCollection::_push(WithLock,
@@ -230,17 +249,13 @@ void OplogBufferCollection::_push(WithLock,
     _cvNoLongerEmpty.notify_all();
 }
 
-void OplogBufferCollection::waitForSpace(OperationContext* opCtx, std::size_t size) {}
+void OplogBufferCollection::waitForSpace(OperationContext* opCtx, const Cost& cost) {}
 
 bool OplogBufferCollection::isEmpty() const {
     stdx::lock_guard<Latch> lk(_mutex);
     return _count == 0;
 }
 
-std::size_t OplogBufferCollection::getMaxSize() const {
-    return 0;
-}
-
 std::size_t OplogBufferCollection::getSize() const {
     stdx::lock_guard<Latch> lk(_mutex);
     uassert(4940100, "getSize() called on OplogBufferCollection after seek", _sizeIsValid);
@@ -254,6 +269,9 @@ std::size_t OplogBufferCollection::getCount() const {
 
 void OplogBufferCollection::clear(OperationContext* opCtx) {
     stdx::lock_guard<Latch> lk(_mutex);
+    // We acquire the appropriate locks for the temporary oplog buffer collection here,
+    // so that we perform the drop and create under the same locks.
+    AutoGetCollection autoColl(opCtx, NamespaceString::kDefaultOplogCollectionNamespace, MODE_X);
     _dropCollection(opCtx);
     _createCollection(opCtx);
     _size = 0;
@@ -372,14 +390,13 @@ boost::optional<OplogBuffer::Value> OplogBufferCollection::_lastDocumentPushed_i
         return boost::none;
     }
     const auto docs =
-        fassert(40348,
-                _storageInterface->findDocuments(opCtx,
-                                                 _nss,
-                                                 kIdIdxName,
-                                                 StorageInterface::ScanDirection::kBackward,
-                                                 {},
-                                                 BoundInclusion::kIncludeStartKeyOnly,
-                                                 1U));
+        uassertStatusOK(_storageInterface->findDocuments(opCtx,
+                                                         _nss,
+                                                         kIdIdxName,
+                                                         StorageInterface::ScanDirection::kBackward,
+                                                         {},
+                                                         BoundInclusion::kIncludeStartKeyOnly,
+                                                         1U));
     invariant(1U == docs.size());
     return docs.front();
 }
@@ -421,15 +438,14 @@ BSONObj OplogBufferCollection::_peek_inlock(OperationContext* opCtx, PeekMode pe
     // when size of read ahead cache is greater than zero in the options.
     if (_peekCache.empty()) {
         std::size_t limit = isPeekCacheEnabled ? _options.peekCacheSize : 1U;
-        const auto docs =
-            fassert(40163,
-                    _storageInterface->findDocuments(opCtx,
-                                                     _nss,
-                                                     kIdIdxName,
-                                                     StorageInterface::ScanDirection::kForward,
-                                                     startKey,
-                                                     boundInclusion,
-                                                     limit));
+        const auto docs = uassertStatusOK(
+            _storageInterface->findDocuments(opCtx,
+                                             _nss,
+                                             kIdIdxName,
+                                             StorageInterface::ScanDirection::kForward,
+                                             startKey,
+                                             boundInclusion,
+                                             limit));
         invariant(!docs.empty());
         for (const auto& doc : docs) {
             _peekCache.push(doc);
@@ -453,7 +469,10 @@ BSONObj OplogBufferCollection::_peek_inlock(OperationContext* opCtx, PeekMode pe
 void OplogBufferCollection::_createCollection(OperationContext* opCtx) {
     CollectionOptions options;
     options.temp = _options.useTemporaryCollection;
-    UninterruptibleLockGuard noInterrupt(opCtx->lockState());
+    // This oplog-like collection will benefit from clustering by _id to reduce storage engine
+    // overhead and improve _id query efficiency.
+    options.clusteredIndex = clustered_util::makeDefaultClusteredIdIndex();
+
     auto status = _storageInterface->createCollection(opCtx, _nss, options);
     if (status.code() == ErrorCodes::NamespaceExists)
         return;
@@ -461,12 +480,15 @@ void OplogBufferCollection::_createCollection(OperationContext* opCtx) {
 }
 
 void OplogBufferCollection::_dropCollection(OperationContext* opCtx) {
-    UninterruptibleLockGuard noInterrupt(opCtx->lockState());
     uassertStatusOK(_storageInterface->dropCollection(opCtx, _nss));
 }
 
 Timestamp OplogBufferCollection::getLastPushedTimestamp() const {
     stdx::lock_guard<Latch> lk(_mutex);
+    uassert(8359601,
+            "preload() might have failed. So clear() should be called before reading "
+            "'lastPushedTimestamp'",
+            _lastPushedTimestamp != kInvalidLastPushedTimestamp);
     return _lastPushedTimestamp;
 }
 
