@@ -27,7 +27,13 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <mutex>
+#include <utility>
+#include <vector>
+
+#include <boost/optional/optional.hpp>
 
 #include "mongo/db/repl/oplog_buffer_proxy.h"
 #include "mongo/util/assert_util.h"
@@ -63,27 +69,24 @@ void OplogBufferProxy::shutdown(OperationContext* opCtx) {
 
 void OplogBufferProxy::push(OperationContext* opCtx,
                             Batch::const_iterator begin,
-                            Batch::const_iterator end) {
+                            Batch::const_iterator end,
+                            boost::optional<const Cost&> cost) {
     if (begin == end) {
         return;
     }
     stdx::lock_guard<Latch> lk(_lastPushedMutex);
     _lastPushed = *(end - 1);
-    _target->push(opCtx, begin, end);
+    _target->push(opCtx, begin, end, cost);
 }
 
-void OplogBufferProxy::waitForSpace(OperationContext* opCtx, std::size_t size) {
-    _target->waitForSpace(opCtx, size);
+void OplogBufferProxy::waitForSpace(OperationContext* opCtx, const Cost& cost) {
+    _target->waitForSpace(opCtx, cost);
 }
 
 bool OplogBufferProxy::isEmpty() const {
     return _target->isEmpty();
 }
 
-std::size_t OplogBufferProxy::getMaxSize() const {
-    return _target->getMaxSize();
-}
-
 std::size_t OplogBufferProxy::getSize() const {
     return _target->getSize();
 }
