@@ -29,6 +29,15 @@
 
 #include "mongo/db/repl/oplog_constraint_violation_logger.h"
 
+#include <boost/optional/optional.hpp>
+
+
+// IWYU pragma: no_include "ext/alloc_traits.h"
+
+#include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplication
 
 namespace mongo {
