@@ -30,16 +30,29 @@
 
 #include "mongo/db/repl/oplog_entry.h"
 
+#include <array>
+#include <boost/cstdint.hpp>
+#include <boost/optional.hpp>
 #include <fmt/format.h>
+#include <string_view>
 
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/db/feature_flag.h"
 #include "mongo/db/global_index.h"
 #include "mongo/db/index/index_descriptor.h"
 #include "mongo/db/multitenancy_gen.h"
 #include "mongo/db/namespace_string.h"
-#include "mongo/db/server_feature_flags_gen.h"
+#include "mongo/db/server_options.h"
 #include "mongo/logv2/redaction.h"
 #include "mongo/s/catalog/type_index_catalog.h"
 #include "mongo/util/namespace_string_util.h"
+#include "mongo/util/str.h"
 #include "mongo/util/time_support.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplication
@@ -81,12 +94,13 @@ BSONObj makeOplogEntryDoc(OpTime opTime,
     builder.append(OplogEntryBase::kTermFieldName, opTime.getTerm());
     builder.append(OplogEntryBase::kVersionFieldName, version);
     builder.append(OplogEntryBase::kOpTypeFieldName, OpType_serializer(opType));
-    const auto fcvSnapshot = serverGlobalParams.featureCompatibility.acquireFCVSnapshot();
-    if (nss.tenantId() && gMultitenancySupport && fcvSnapshot.isVersionInitialized() &&
-        gFeatureFlagRequireTenantID.isEnabled(fcvSnapshot)) {
+    if (nss.tenantId() && gMultitenancySupport &&
+        gFeatureFlagRequireTenantID.isEnabled(
+            serverGlobalParams.featureCompatibility.acquireFCVSnapshot())) {
         nss.tenantId()->serializeToBSON(OplogEntryBase::kTidFieldName, &builder);
     }
-    builder.append(OplogEntryBase::kNssFieldName, NamespaceStringUtil::serialize(nss));
+    builder.append(OplogEntryBase::kNssFieldName,
+                   NamespaceStringUtil::serialize(nss, SerializationContext::stateDefault()));
     builder.append(OplogEntryBase::kWallClockTimeFieldName, wallClockTime);
     if (uuid) {
         uuid->appendToBuilder(&builder, OplogEntryBase::kUuidFieldName);
@@ -137,54 +151,9 @@ BSONObj makeOplogEntryDoc(OpTime opTime,
 }
 }  // namespace
 
-DurableOplogEntry::CommandType parseCommandType(const BSONObj& objectField) {
-    StringData commandString(objectField.firstElementFieldName());
-    if (commandString == "create") {
-        return DurableOplogEntry::CommandType::kCreate;
-    } else if (commandString == "renameCollection") {
-        return DurableOplogEntry::CommandType::kRenameCollection;
-    } else if (commandString == "drop") {
-        return DurableOplogEntry::CommandType::kDrop;
-    } else if (commandString == "collMod") {
-        return DurableOplogEntry::CommandType::kCollMod;
-    } else if (commandString == "applyOps") {
-        return DurableOplogEntry::CommandType::kApplyOps;
-    } else if (commandString == "dbCheck") {
-        return DurableOplogEntry::CommandType::kDbCheck;
-    } else if (commandString == "dropDatabase") {
-        return DurableOplogEntry::CommandType::kDropDatabase;
-    } else if (commandString == "emptycapped") {
-        return DurableOplogEntry::CommandType::kEmptyCapped;
-    } else if (commandString == "createIndexes") {
-        return DurableOplogEntry::CommandType::kCreateIndexes;
-    } else if (commandString == "startIndexBuild") {
-        return DurableOplogEntry::CommandType::kStartIndexBuild;
-    } else if (commandString == "commitIndexBuild") {
-        return DurableOplogEntry::CommandType::kCommitIndexBuild;
-    } else if (commandString == "abortIndexBuild") {
-        return DurableOplogEntry::CommandType::kAbortIndexBuild;
-    } else if (commandString == "dropIndexes") {
-        return DurableOplogEntry::CommandType::kDropIndexes;
-    } else if (commandString == "deleteIndexes") {
-        return DurableOplogEntry::CommandType::kDropIndexes;
-    } else if (commandString == "commitTransaction") {
-        return DurableOplogEntry::CommandType::kCommitTransaction;
-    } else if (commandString == "abortTransaction") {
-        return DurableOplogEntry::CommandType::kAbortTransaction;
-    } else if (commandString == "importCollection") {
-        return DurableOplogEntry::CommandType::kImportCollection;
-    } else if (commandString == kShardingIndexCatalogOplogEntryName) {
-        return DurableOplogEntry::CommandType::kModifyCollectionShardingIndexCatalog;
-    } else if (commandString == "createGlobalIndex") {
-        return DurableOplogEntry::CommandType::kCreateGlobalIndex;
-    } else if (commandString == "dropGlobalIndex") {
-        return DurableOplogEntry::CommandType::kDropGlobalIndex;
-    } else {
-        uasserted(ErrorCodes::BadValue,
-                  str::stream() << "Unknown oplog entry command type: " << commandString
-                                << " Object field: " << redact(objectField));
-    }
-    MONGO_UNREACHABLE;
+CommandTypeEnum parseCommandType(const BSONObj& objectField) {
+    return CommandType_parse(IDLParserContext("commandString"),
+                             objectField.firstElementFieldNameStringData());
 }
 
 void ReplOperation::extractPrePostImageForTransaction(boost::optional<ImageBundle>* image) const {
@@ -196,7 +165,7 @@ void ReplOperation::extractPrePostImageForTransaction(boost::optional<ImageBundl
     uassert(6054001,
             fmt::format("{} can only store the pre or post image of one findAndModify operation "
                         "for each transaction",
-                        NamespaceString::kConfigImagesNamespace.toString()),
+                        NamespaceString::kConfigImagesNamespace.toStringForErrorMsg()),
             !(*image));
 
     switch (*needsRetryImage) {
@@ -216,9 +185,9 @@ void ReplOperation::extractPrePostImageForTransaction(boost::optional<ImageBundl
 }
 
 void ReplOperation::setTid(boost::optional<mongo::TenantId> value) & {
-    const auto fcvSnapshot = serverGlobalParams.featureCompatibility.acquireFCVSnapshot();
-    if (gMultitenancySupport && fcvSnapshot.isVersionInitialized() &&
-        gFeatureFlagRequireTenantID.isEnabled(fcvSnapshot))
+    if (gMultitenancySupport &&
+        gFeatureFlagRequireTenantID.isEnabled(
+            serverGlobalParams.featureCompatibility.acquireFCVSnapshot()))
         DurableReplOperation::setTid(value);
 }
 
@@ -342,13 +311,18 @@ MutableOplogEntry MutableOplogEntry::makeGlobalIndexCrudOperation(const OpTypeEn
 }
 
 StatusWith<MutableOplogEntry> MutableOplogEntry::parse(const BSONObj& object) {
-    boost::optional<TenantId> tid;
-    if (object.hasElement("tid"))
-        tid = TenantId::parseFromBSON(object["tid"]);
-
+    const auto tid = OplogEntry::parseTid(object);
     try {
         MutableOplogEntry oplogEntry;
-        oplogEntry.parseProtected(IDLParserContext("OplogEntryBase", false /* apiStrict */, tid),
+        const auto vts = tid
+            ? boost::make_optional(auth::ValidatedTenancyScopeFactory::create(
+                  *tid, auth::ValidatedTenancyScopeFactory::TrustedForInnerOpMsgRequestTag{}))
+            : boost::none;
+        oplogEntry.parseProtected(IDLParserContext("OplogEntryBase",
+                                                   false /* apiStrict */,
+                                                   vts,
+                                                   tid,
+                                                   SerializationContext::stateDefault()),
                                   object);
         return oplogEntry;
     } catch (...) {
@@ -362,9 +336,10 @@ ReplOperation MutableOplogEntry::toReplOperation() const noexcept {
 }
 
 void MutableOplogEntry::setTid(boost::optional<mongo::TenantId> value) & {
-    const auto fcvSnapshot = serverGlobalParams.featureCompatibility.acquireFCVSnapshot();
-    if (gMultitenancySupport && fcvSnapshot.isVersionInitialized() &&
-        gFeatureFlagRequireTenantID.isEnabled(fcvSnapshot))
+    // Only set Tid if we have a TenantId value and the server parameter and feature flag are on.
+    if (value && gMultitenancySupport &&
+        gFeatureFlagRequireTenantID.isEnabled(
+            serverGlobalParams.featureCompatibility.acquireFCVSnapshot()))
         getDurableReplOperation().setTid(std::move(value));
 }
 
@@ -383,7 +358,7 @@ OpTime MutableOplogEntry::getOpTime() const {
 }
 
 size_t DurableOplogEntry::getDurableReplOperationSize(const DurableReplOperation& op) {
-    const auto stmtIds = variant_util::toVector<StmtId>(op.getStatementIds());
+    const auto& stmtIds = op.getStatementIds();
     return sizeof(op) + (op.getTid() ? op.getTid()->toString().size() : 0) + op.getNss().size() +
         op.getObject().objsize() + (op.getObject2() ? op.getObject2()->objsize() : 0) +
         (sizeof(std::vector<StmtId>) + (sizeof(StmtId) * stmtIds.size()));
@@ -401,11 +376,18 @@ StatusWith<DurableOplogEntry> DurableOplogEntry::parse(const BSONObj& object) {
 DurableOplogEntry::DurableOplogEntry(BSONObj rawInput) : _raw(std::move(rawInput)) {
     _raw = _raw.getOwned();
 
-    boost::optional<TenantId> tid;
-    if (_raw.hasElement("tid"))
-        tid = TenantId::parseFromBSON(_raw["tid"]);
+    const auto tid = OplogEntry::parseTid(_raw);
 
-    parseProtected(IDLParserContext("OplogEntryBase", false /* apiStrict */, tid), _raw);
+    const auto vts = tid
+        ? boost::make_optional(auth::ValidatedTenancyScopeFactory::create(
+              *tid, auth::ValidatedTenancyScopeFactory::TrustedForInnerOpMsgRequestTag{}))
+        : boost::none;
+    parseProtected(IDLParserContext("OplogEntryBase",
+                                    false /* apiStrict */,
+                                    vts,
+                                    tid,
+                                    SerializationContext::stateDefault()),
+                   _raw);
 
     // Parse command type from 'o' and 'o2' fields.
     if (isCommand()) {
@@ -502,17 +484,38 @@ bool DurableOplogEntry::isUpdateOrDelete() const {
 }
 
 bool DurableOplogEntry::shouldPrepare() const {
-    return getCommandType() == CommandType::kApplyOps &&
+    return getCommandType() == CommandTypeEnum::kApplyOps &&
         getObject()[ApplyOpsCommandInfoBase::kPrepareFieldName].booleanSafe();
 }
 
+bool DurableOplogEntry::applyOpsIsLinkedTransactionally() const {
+    // An applyOps with a prevWriteOpTime is part of a transaction, unless multiOpType is
+    // kApplyOpsAppliedSeparately.
+    return bool(getPrevWriteOpTimeInTransaction()) &&
+        getMultiOpType().value_or(MultiOplogEntryType::kLegacyMultiOpType) !=
+        MultiOplogEntryType::kApplyOpsAppliedSeparately;
+}
+
+bool DurableOplogEntry::isInTransaction() const {
+    if (getCommandType() == CommandTypeEnum::kAbortTransaction ||
+        getCommandType() == CommandTypeEnum::kCommitTransaction)
+        return true;
+    if (!getTxnNumber() || !getSessionId())
+        return false;
+    if (getCommandType() != CommandTypeEnum::kApplyOps)
+        return false;
+    return applyOpsIsLinkedTransactionally();
+}
+
 bool DurableOplogEntry::isSingleOplogEntryTransaction() const {
-    if (getCommandType() != CommandType::kApplyOps || !getTxnNumber() || !getSessionId() ||
+    if (getCommandType() != CommandTypeEnum::kApplyOps || !getTxnNumber() || !getSessionId() ||
         getObject()[ApplyOpsCommandInfoBase::kPartialTxnFieldName].booleanSafe()) {
         return false;
     }
     auto prevOptimeOpt = getPrevWriteOpTimeInTransaction();
-    if (!prevOptimeOpt) {
+    if (!prevOptimeOpt ||
+        getMultiOpType().value_or(MultiOplogEntryType::kLegacyMultiOpType) ==
+            MultiOplogEntryType::kApplyOpsAppliedSeparately) {
         // If there is no prevWriteOptime, then this oplog entry is not a part of a transaction.
         return false;
     }
@@ -520,21 +523,23 @@ bool DurableOplogEntry::isSingleOplogEntryTransaction() const {
 }
 
 bool DurableOplogEntry::isEndOfLargeTransaction() const {
-    if (getCommandType() != CommandType::kApplyOps) {
-        // If the oplog entry is neither commit nor abort, then it must be an applyOps. Otherwise,
-        // it cannot be a termainal oplog entry of a large transaction.
-        return false;
-    }
-    auto prevOptimeOpt = getPrevWriteOpTimeInTransaction();
-    if (!prevOptimeOpt) {
+    if (getCommandType() != CommandTypeEnum::kApplyOps) {
         // If the oplog entry is neither commit nor abort, then it must be an applyOps. Otherwise,
         // it cannot be a terminal oplog entry of a large transaction.
         return false;
     }
+    auto prevOptimeOpt = getPrevWriteOpTimeInTransaction();
+    if (!prevOptimeOpt) {
+        // If there is no prevWriteOptime, then this oplog entry is not a part of a transaction.
+        return false;
+    }
     // There should be a previous oplog entry in a multiple oplog entry transaction if this is
     // supposed to be the last one. The first oplog entry in a large transaction will have a null
-    // ts.
-    return !prevOptimeOpt->isNull() && !isPartialTransaction();
+    // ts.  The end of a large transaction should not have a partialTxn field, nor should
+    // multiOpType be set to kApplyOpsAppliedSeparately
+    return !prevOptimeOpt->isNull() && !isPartialTransaction() &&
+        getMultiOpType().value_or(MultiOplogEntryType::kLegacyMultiOpType) !=
+        MultiOplogEntryType::kApplyOpsAppliedSeparately;
 }
 
 bool DurableOplogEntry::isSingleOplogEntryTransactionWithCommand() const {
@@ -548,16 +553,9 @@ bool DurableOplogEntry::isSingleOplogEntryTransactionWithCommand() const {
     // optimization, if necessary, could be to ensure the primary always constructs applyOps oplog
     // entries with commands at the beginning.
     for (BSONElement e : applyOps.Array()) {
-        auto ns = e.Obj().getField("ns");
-        if (!ns.eoo()) {
-            auto tid = e.Obj().getField("tid");
-            auto tenantId = tid.eoo()
-                ? boost::none
-                : boost::make_optional<TenantId>(TenantId::parseFromBSON(tid));
-
-            if (NamespaceStringUtil::deserialize(tenantId, ns.String()).isCommand()) {
-                return true;
-            }
+        auto const opType = e.Obj().getStringField(OplogEntry::kOpTypeFieldName);
+        if (opType == "c"_sd) {
+            return true;
         }
     }
     return false;
@@ -565,11 +563,12 @@ bool DurableOplogEntry::isSingleOplogEntryTransactionWithCommand() const {
 
 bool DurableOplogEntry::isIndexCommandType() const {
     return getOpType() == OpTypeEnum::kCommand &&
-        ((getCommandType() == CommandType::kCreateIndexes) ||
-         (getCommandType() == CommandType::kStartIndexBuild) ||
-         (getCommandType() == CommandType::kCommitIndexBuild) ||
-         (getCommandType() == CommandType::kAbortIndexBuild) ||
-         (getCommandType() == CommandType::kDropIndexes));
+        ((getCommandType() == CommandTypeEnum::kCreateIndexes) ||
+         (getCommandType() == CommandTypeEnum::kStartIndexBuild) ||
+         (getCommandType() == CommandTypeEnum::kCommitIndexBuild) ||
+         (getCommandType() == CommandTypeEnum::kAbortIndexBuild) ||
+         (getCommandType() == CommandTypeEnum::kDropIndexes) ||
+         (getCommandType() == CommandTypeEnum::kDeleteIndexes));
 }
 
 BSONElement DurableOplogEntry::getIdElement() const {
@@ -598,7 +597,7 @@ BSONObj DurableOplogEntry::getObjectContainingDocumentKey() const {
     }
 }
 
-DurableOplogEntry::CommandType DurableOplogEntry::getCommandType() const {
+CommandTypeEnum DurableOplogEntry::getCommandType() const {
     return _commandType;
 }
 
@@ -648,6 +647,18 @@ StatusWith<OplogEntry> OplogEntry::parse(const BSONObj& object) {
 
     return OplogEntry(std::move(parseStatus.getValue()));
 }
+
+boost::optional<TenantId> OplogEntry::parseTid(const BSONObj& object) {
+    if (!gMultitenancySupport) {
+        return boost::none;
+    }
+    BSONElement tidElem = object["tid"];
+    if (tidElem.eoo()) {
+        return boost::none;
+    }
+    return TenantId::parseFromBSON(tidElem);
+}
+
 std::string OplogEntry::toStringForLogging() const {
     return toBSONForLogging().toString();
 }
@@ -676,7 +687,7 @@ const boost::optional<mongo::Value>& OplogEntry::get_id() const& {
     return _entry.get_id();
 }
 
-std::vector<StmtId> OplogEntry::getStatementIds() const& {
+const std::vector<StmtId>& OplogEntry::getStatementIds() const& {
     return _entry.getStatementIds();
 }
 
@@ -743,10 +754,6 @@ const mongo::Date_t& OplogEntry::getWallClockTime() const {
     return _entry.getWallClockTime();
 }
 
-boost::optional<std::int64_t> OplogEntry::getHash() const& {
-    return _entry.getHash();
-}
-
 std::int64_t OplogEntry::getVersion() const {
     return _entry.getVersion();
 }
@@ -763,6 +770,14 @@ const boost::optional<mongo::UUID>& OplogEntry::getFromTenantMigration() const&
     return _entry.getFromTenantMigration();
 }
 
+const boost::optional<mongo::repl::OpTime>& OplogEntry::getDonorOpTime() const& {
+    return _entry.getDonorOpTime();
+}
+
+boost::optional<std::int64_t> OplogEntry::getDonorApplyOpsIndex() const& {
+    return _entry.getDonorApplyOpsIndex();
+}
+
 const boost::optional<mongo::repl::OpTime>& OplogEntry::getPrevWriteOpTimeInTransaction() const& {
     return _entry.getPrevWriteOpTimeInTransaction();
 }
@@ -771,6 +786,10 @@ const boost::optional<mongo::repl::OpTime>& OplogEntry::getPostImageOpTime() con
     return _entry.getPostImageOpTime();
 }
 
+boost::optional<mongo::repl::MultiOplogEntryType> OplogEntry::getMultiOpType() const& {
+    return _entry.getMultiOpType();
+}
+
 boost::optional<RetryImageEnum> OplogEntry::getNeedsRetryImage() const {
     return _needsRetryImage;
 }
@@ -787,6 +806,14 @@ bool OplogEntry::isCommand() const {
     return _entry.isCommand();
 }
 
+bool OplogEntry::applyOpsIsLinkedTransactionally() const {
+    return _entry.applyOpsIsLinkedTransactionally();
+}
+
+bool OplogEntry::isInTransaction() const {
+    return _entry.isInTransaction();
+}
+
 bool OplogEntry::isPartialTransaction() const {
     return _entry.isPartialTransaction();
 }
