@@ -30,22 +30,62 @@
 
 #include "mongo/db/repl/oplog_fetcher.h"
 
-#include "mongo/bson/mutable/document.h"
+#include <boost/cstdint.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <cstdint>
+#include <fmt/format.h>
+// IWYU pragma: no_include "ext/alloc_traits.h"
+#include <algorithm>
+#include <mutex>
+
+#include "mongo/base/counter.h"
+#include "mongo/base/error_codes.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/client/read_preference.h"
+#include "mongo/db/basic_types_gen.h"
+#include "mongo/db/client.h"
 #include "mongo/db/commands/server_status_metric.h"
-#include "mongo/db/jsobj.h"
+#include "mongo/db/exec/document_value/document.h"
 #include "mongo/db/matcher/matcher.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/pipeline/document_source_find_and_modify_image_lookup.h"
 #include "mongo/db/pipeline/document_source_match.h"
+#include "mongo/db/pipeline/expression_context.h"
+#include "mongo/db/pipeline/pipeline.h"
+#include "mongo/db/pipeline/process_interface/mongo_process_interface.h"
+#include "mongo/db/query/collation/collator_interface.h"
+#include "mongo/db/repl/optime_with.h"
 #include "mongo/db/repl/repl_server_parameters_gen.h"
 #include "mongo/db/repl/replication_auth.h"
-#include "mongo/db/repl/replication_coordinator.h"
+#include "mongo/db/repl/replication_process.h"
+#include "mongo/db/repl/sync_source_selector.h"
+#include "mongo/db/service_context.h"
 #include "mongo/db/stats/timer_stats.h"
+#include "mongo/db/write_concern_options.h"
+#include "mongo/idl/idl_parser.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/redaction.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/platform/compiler.h"
 #include "mongo/rpc/metadata/oplog_query_metadata.h"
+#include "mongo/rpc/metadata/repl_set_metadata.h"
 #include "mongo/s/resharding/resume_token_gen.h"
 #include "mongo/util/assert_util.h"
 #include "mongo/util/fail_point.h"
+#include "mongo/util/intrusive_counter.h"
+#include "mongo/util/net/ssl_options.h"
+#include "mongo/util/str.h"
+#include "mongo/util/string_map.h"
 #include "mongo/util/time_support.h"
+#include "mongo/util/timer.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplication
 
@@ -90,15 +130,15 @@ BSONObj OplogBatchStats::getReport() const {
 }
 
 // The number and time spent reading batches off the network
-auto& oplogBatchStats = makeServerStatusMetric<OplogBatchStats>("repl.network.getmores");
+auto& oplogBatchStats = *MetricBuilder<OplogBatchStats>("repl.network.getmores");
 // The oplog entries read via the oplog reader
-CounterMetric opsReadStats("repl.network.ops");
+auto& opsReadStats = *MetricBuilder<Counter64>{"repl.network.ops"};
 // The bytes read via the oplog reader
-CounterMetric networkByteStats("repl.network.bytes");
+auto& networkByteStats = *MetricBuilder<Counter64>{"repl.network.bytes"};
 
-CounterMetric readersCreatedStats("repl.network.readersCreated");
+auto& readersCreatedStats = *MetricBuilder<Counter64>{"repl.network.readersCreated"};
 
-const Milliseconds maximumAwaitDataTimeoutMS(30 * 1000);
+const Milliseconds maximumAwaitDataTimeoutMS = Seconds{30};
 
 /**
  * Calculates await data timeout based on the current replica set configuration.
@@ -132,6 +172,24 @@ StatusWith<OplogFetcher::DocumentsInfo> OplogFetcher::validateDocuments(
     info.networkDocumentBytes = 0;
     info.networkDocumentCount = 0;
     for (auto&& doc : documents) {
+        if (feature_flags::gReduceMajorityWriteLatency.isEnabled(
+                serverGlobalParams.featureCompatibility.acquireFCVSnapshot())) {
+            // Check for oplog version change.
+            auto version = doc[OplogEntry::kVersionFieldName].numberLong();
+            if (version != OplogEntry::kOplogVersion) {
+                static constexpr char message[] = "Unexpected oplog version";
+                LOGV2_FATAL_CONTINUE(8539101,
+                                     message,
+                                     "expectedVersion"_attr = OplogEntry::kOplogVersion,
+                                     "foundVersion"_attr = version,
+                                     "oplogEntry"_attr = redact(doc));
+                return Status(ErrorCodes::BadValue,
+                              str::stream() << message << ", expected oplog version "
+                                            << OplogEntry::kOplogVersion << ", found version "
+                                            << version << ", oplog entry: " << redact(doc));
+            }
+        }
+
         info.networkDocumentBytes += doc.objsize();
         ++info.networkDocumentCount;
 
@@ -240,7 +298,7 @@ std::string OplogFetcher::toString() {
     output << "OplogFetcher -";
     output << " last optime fetched: " << _lastFetched.toString();
     output << " source: " << _config.source.toString();
-    output << " namespace: " << _nss.toString();
+    output << " namespace: " << toStringForLogging(_nss);
     output << " active: " << _isActive_inlock();
     output << " shutting down?:" << _isShuttingDown_inlock();
     output << " first batch: " << _firstBatch;
@@ -469,12 +527,11 @@ Status OplogFetcher::_connect() {
                     // If this is a retry, let the DBClientConnection handle the reconnect itself
                     // for proper backoff behavior.
                     LOGV2(23437,
-                          "OplogFetcher reconnecting due to error: {error}",
                           "OplogFetcher reconnecting due to error",
                           "error"_attr = connectStatus);
-                    _conn->checkConnection();
+                    _conn->ensureConnection();
                 } else {
-                    uassertStatusOK(_conn->connect(_config.source, "OplogFetcher", boost::none));
+                    _conn->connect(_config.source, "OplogFetcher", boost::none);
                 }
                 uassertStatusOK(replAuthenticate(_conn.get())
                                     .withContext(str::stream()
@@ -519,7 +576,7 @@ void OplogFetcher::_setMetadataWriterAndReader() {
 
             // Run VectorClockMetadataHook on reply metadata so this matches the behavior of the
             // connections in the replication coordinator thread pool.
-            return _vectorClockMetadataHook->readReplyMetadata(opCtx, source, _metadataObj);
+            return _vectorClockMetadataHook->readReplyMetadata(opCtx, _metadataObj);
         });
 }
 
@@ -527,7 +584,7 @@ void OplogFetcher::_setMetadataWriterAndReader() {
 AggregateCommandRequest OplogFetcher::_makeAggregateCommandRequest(long long maxTimeMs,
                                                                    Timestamp startTs) const {
     auto opCtx = cc().makeOperationContext();
-    ResolvedNamespaceMap resolvedNamespaces;
+    StringMap<ExpressionContext::ResolvedNamespace> resolvedNamespaces;
     boost::intrusive_ptr<ExpressionContext> expCtx =
         make_intrusive<ExpressionContext>(opCtx.get(),
                                           boost::none, /* explain */
@@ -555,7 +612,7 @@ AggregateCommandRequest OplogFetcher::_makeAggregateCommandRequest(long long max
     BSONObjBuilder secondMatchBuilder(BSON("ts" << BSON("$gte" << startTs)));
     stages.emplace_back(DocumentSourceMatch::createFromBson(
         Document{{"$match", Document{secondMatchBuilder.obj()}}}.toBson().firstElement(), expCtx));
-    const auto serializedPipeline = Pipeline::create(std::move(stages), expCtx)->serializeToBson();
+    auto serializedPipeline = Pipeline::create(std::move(stages), expCtx)->serializeToBson();
 
     AggregateCommandRequest aggRequest(_nss, std::move(serializedPipeline));
     aggRequest.setReadConcern(_config.queryReadConcern.toBSONInner());
@@ -756,13 +813,7 @@ Status OplogFetcher::_onSuccessfulBatch(const Documents& documents) {
     if (_isShuttingDown()) {
         return Status(ErrorCodes::CallbackCanceled, "oplog fetcher shutting down");
     }
-    // We call fetchSuccessful after any batch succeeds and it will reset _numRestarts to 0. Note
-    // that if the first batch is succesful and the second batch due to an error that is not handled
-    // in shouldContinue(), it will create an indefinite loop. The loop occurs because the restart
-    // after the second batch will recreate the cursor and repeat the successful first batch and
-    // reset _numRestarts, never letting it increment enough to exceed maxRestarts. This can be
-    // resolved by adding the error to shouldContinue() and returning false in that case so that it
-    // doesn't infinitely loop.
+
     _oplogFetcherRestartDecision->fetchSuccessful(this);
 
     // Stop fetching and return on fail point.
@@ -806,8 +857,6 @@ Status OplogFetcher::_onSuccessfulBatch(const Documents& documents) {
     if (!documents.empty()) {
         LOGV2_DEBUG(21270,
                     2,
-                    "oplog fetcher read {batchSize} operations from remote oplog starting at "
-                    "{firstTimestamp} and ending at {lastTimestamp}",
                     "Oplog fetcher read batch from remote oplog",
                     "batchSize"_attr = documents.size(),
                     "firstTimestamp"_attr = documents.front()["ts"],
@@ -819,8 +868,6 @@ Status OplogFetcher::_onSuccessfulBatch(const Documents& documents) {
     auto oqMetadataResult = rpc::OplogQueryMetadata::readFromMetadata(_metadataObj);
     if (!oqMetadataResult.isOK()) {
         LOGV2_ERROR(21278,
-                    "invalid oplog query metadata from sync source {syncSource}: "
-                    "{error}: {metadata}",
                     "Invalid oplog query metadata from sync source",
                     "syncSource"_attr = _config.source,
                     "error"_attr = oqMetadataResult.getStatus(),
@@ -839,7 +886,6 @@ Status OplogFetcher::_onSuccessfulBatch(const Documents& documents) {
 
         LOGV2_DEBUG(21272,
                     1,
-                    "oplog fetcher successfully fetched from {syncSource}",
                     "Oplog fetcher successfully fetched from sync source",
                     "syncSource"_attr = _config.source);
 
@@ -886,8 +932,6 @@ Status OplogFetcher::_onSuccessfulBatch(const Documents& documents) {
     auto metadataResult = rpc::ReplSetMetadata::readFromMetadata(_metadataObj);
     if (!metadataResult.isOK()) {
         LOGV2_ERROR(21279,
-                    "invalid replication metadata from sync source {syncSource}: "
-                    "{error}: {metadata}",
                     "Invalid replication metadata from sync source",
                     "syncSource"_attr = _config.source,
                     "error"_attr = metadataResult.getStatus(),
@@ -899,18 +943,22 @@ Status OplogFetcher::_onSuccessfulBatch(const Documents& documents) {
     // Determine if we should stop syncing from our current sync source.
     auto changeSyncSourceAction = _dataReplicatorExternalState->shouldStopFetching(
         _config.source, replSetMetadata, oqMetadata, previousOpTimeFetched, lastDocOpTime);
-    str::stream errMsg;
-    errMsg << "sync source " << _config.source.toString();
-    errMsg << " (config version: " << replSetMetadata.getConfigVersion();
-    errMsg << "; last applied optime: " << oqMetadata.getLastOpApplied().toString();
-    errMsg << "; sync source index: " << oqMetadata.getSyncSourceIndex();
-    errMsg << "; has primary index: " << oqMetadata.hasPrimaryIndex();
-    errMsg << ") is no longer valid";
-    errMsg << " previous batch last fetched optime: " << previousOpTimeFetched.toString();
-    errMsg << " current batch last fetched optime: " << lastDocOpTime.toString();
+
+    auto getErrMsg = [&] {
+        str::stream errMsg;
+        errMsg << "sync source " << _config.source.toString();
+        errMsg << " (config version: " << replSetMetadata.getConfigVersion();
+        errMsg << "; last applied optime: " << oqMetadata.getLastOpApplied().toString();
+        errMsg << "; sync source index: " << oqMetadata.getSyncSourceIndex();
+        errMsg << "; has primary index: " << oqMetadata.hasPrimaryIndex();
+        errMsg << ") is no longer valid";
+        errMsg << " previous batch last fetched optime: " << previousOpTimeFetched.toString();
+        errMsg << " current batch last fetched optime: " << lastDocOpTime.toString();
+        return errMsg;
+    };
 
     if (changeSyncSourceAction == ChangeSyncSourceAction::kStopSyncingAndDropLastBatchIfPresent) {
-        return Status(ErrorCodes::InvalidSyncSource, errMsg);
+        return Status(ErrorCodes::InvalidSyncSource, getErrMsg());
     }
 
     _dataReplicatorExternalState->processMetadata(replSetMetadata, oqMetadata);
@@ -938,7 +986,7 @@ Status OplogFetcher::_onSuccessfulBatch(const Documents& documents) {
     }
 
     if (changeSyncSourceAction == ChangeSyncSourceAction::kStopSyncingAndEnqueueLastBatch) {
-        return Status(ErrorCodes::InvalidSyncSource, errMsg);
+        return Status(ErrorCodes::InvalidSyncSource, getErrMsg());
     }
 
     if (MONGO_unlikely(hangOplogFetcherBeforeAdvancingLastFetched.shouldFail())) {
@@ -954,7 +1002,6 @@ Status OplogFetcher::_onSuccessfulBatch(const Documents& documents) {
     if (lastDocOpTime != previousOpTimeFetched) {
         LOGV2_DEBUG(21273,
                     3,
-                    "Oplog fetcher setting last fetched optime ahead after batch: {lastDocOpTime}",
                     "Oplog fetcher setting last fetched optime ahead after batch",
                     "lastDocOpTime"_attr = lastDocOpTime);
 
@@ -1118,23 +1165,13 @@ bool OplogFetcher::OplogFetcherRestartDecisionDefault::shouldContinue(OplogFetch
               "error"_attr = redact(status));
         return false;
     }
-    // If the oplog batch fetched exceeds the size limit, do not attempt to restart.
-    else if (status.code() == ErrorCodes::BSONObjectTooLarge) {
-        LOGV2(10033601,
-              "BSON document and metadata fetched by oplog fetcher exceeds the BSON limit",
-              "error"_attr = redact(status));
-        return false;
-    }
     if (_numRestarts == _maxRestarts) {
         LOGV2(21274,
-              "Error returned from oplog query (no more query restarts left): {error}",
               "Error returned from oplog query (no more query restarts left)",
               "error"_attr = redact(status));
         return false;
     }
     LOGV2(21275,
-          "Recreating cursor for oplog fetcher due to error: {error}. Last fetched optime: "
-          "{lastOpTimeFetched}. Attempts remaining: {attemptsRemaining}",
           "Recreating cursor for oplog fetcher due to error",
           "lastOpTimeFetched"_attr = fetcher->_getLastOpTimeFetched(),
           "attemptsRemaining"_attr = (_maxRestarts - _numRestarts),
