@@ -30,8 +30,19 @@
 
 #include "mongo/db/repl/oplog_fetcher_mock.h"
 
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+// IWYU pragma: no_include "cxxabi.h"
+#include <functional>
+#include <mutex>
 #include <utility>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status_with.h"
+#include "mongo/db/client.h"
+#include "mongo/db/repl/replication_process.h"
+#include "mongo/util/assert_util.h"
+
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplication
 
 
@@ -104,7 +115,7 @@ void OplogFetcherMock::receiveBatch(CursorId cursorId,
     // is to avoid interfering the thread local client in the test thread.
     Status status = Status::OK();
     stdx::thread enqueueDocumentThread([&]() {
-        Client::initThread("enqueueDocumentThread");
+        Client::initThread("enqueueDocumentThread", getGlobalServiceContext()->getService());
         status = _enqueueDocumentsFn(documents.cbegin(), documents.cend(), info);
     });
     // Wait until the enqueue finishes.
@@ -182,7 +193,7 @@ void OplogFetcherMock::_doStartup_inlock() {
     // _finishCallback is called only once (outside of the mutex) on shutdown.
     _waitForFinishThread = stdx::thread([this]() {
         auto future = [&] {
-            Client::initThread("OplogFetcherMock");
+            Client::initThread("OplogFetcherMock", getGlobalServiceContext()->getService());
             stdx::lock_guard<Latch> lock(_mutex);
             return _finishPromise->getFuture();
         }();
