@@ -28,32 +28,52 @@
  */
 
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/repl/primary_only_service.h"
-
+#include <absl/container/flat_hash_map.h>
+#include <absl/container/node_hash_map.h>
+#include <absl/container/node_hash_set.h>
+#include <absl/meta/type_traits.h>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <fmt/format.h>
+#include <functional>
+#include <mutex>
+#include <tuple>
 #include <utility>
 
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/client/dbclient_cursor.h"
 #include "mongo/db/auth/authorization_session.h"
-#include "mongo/db/catalog_raii.h"
 #include "mongo/db/client.h"
 #include "mongo/db/dbdirectclient.h"
-#include "mongo/db/ops/write_ops.h"
-#include "mongo/db/repl/repl_client_info.h"
+#include "mongo/db/query/find_command.h"
+#include "mongo/db/repl/primary_only_service.h"
 #include "mongo/db/repl/repl_server_parameters_gen.h"
 #include "mongo/db/repl/replica_set_aware_service.h"
 #include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/repl/wait_for_majority_service.h"
 #include "mongo/db/service_context.h"
+#include "mongo/db/transaction_resources.h"
 #include "mongo/db/vector_clock_metadata_hook.h"
 #include "mongo/executor/network_connection_hook.h"
-#include "mongo/executor/network_interface.h"
 #include "mongo/executor/network_interface_factory.h"
 #include "mongo/executor/thread_pool_task_executor.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/compiler.h"
 #include "mongo/rpc/metadata/egress_metadata_hook_list.h"
-#include "mongo/s/write_ops/batched_command_response.h"
+#include "mongo/rpc/metadata/metadata_hook.h"
 #include "mongo/util/concurrency/thread_pool.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/future_impl.h"
+#include "mongo/util/scopeguard.h"
+#include "mongo/util/str.h"
+#include "mongo/util/time_support.h"
+#include "mongo/util/timer.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplication
 
@@ -140,6 +160,9 @@ PrimaryOnlyServiceRegistry* PrimaryOnlyServiceRegistry::get(ServiceContext* serv
 
 void PrimaryOnlyServiceRegistry::registerService(std::unique_ptr<PrimaryOnlyService> service) {
     auto ns = service->getStateDocumentsNS();
+    invariant(
+        ns.isConfigDB(),
+        "PrimaryOnlyServices can only register a state documents namespace in the 'config' db");
     auto name = service->getServiceName();
     auto servicePtr = service.get();
 
@@ -148,19 +171,15 @@ void PrimaryOnlyServiceRegistry::registerService(std::unique_ptr<PrimaryOnlyServ
               str::stream() << "Attempted to register PrimaryOnlyService (" << name
                             << ") that is already registered");
 
-    auto [existingServiceIt, inserted2] = _servicesByNamespace.emplace(ns.toString(), servicePtr);
+    auto [existingServiceIt, inserted2] = _servicesByNamespace.emplace(ns, servicePtr);
     auto existingService = existingServiceIt->second;
     invariant(inserted2,
               str::stream() << "Attempted to register PrimaryOnlyService (" << name
-                            << ") with state document namespace \"" << ns
+                            << ") with state document namespace \"" << ns.toStringForErrorMsg()
                             << "\" that is already in use by service "
                             << existingService->getServiceName());
-    LOGV2_INFO(5123008,
-               "Successfully registered PrimaryOnlyService {service} with state documents stored "
-               "in {namespace}",
-               "Successfully registered PrimaryOnlyService",
-               "service"_attr = name,
-               logAttrs(ns));
+    LOGV2_INFO(
+        5123008, "Successfully registered PrimaryOnlyService", "service"_attr = name, logAttrs(ns));
 }
 
 PrimaryOnlyService* PrimaryOnlyServiceRegistry::lookupServiceByName(StringData serviceName) {
@@ -173,7 +192,7 @@ PrimaryOnlyService* PrimaryOnlyServiceRegistry::lookupServiceByName(StringData s
 
 PrimaryOnlyService* PrimaryOnlyServiceRegistry::lookupServiceByNamespace(
     const NamespaceString& ns) {
-    auto it = _servicesByNamespace.find(ns.toString());
+    auto it = _servicesByNamespace.find(ns);
     if (it == _servicesByNamespace.end()) {
         return nullptr;
     }
@@ -197,7 +216,7 @@ void PrimaryOnlyServiceRegistry::onShutdown() {
 void PrimaryOnlyServiceRegistry::onStepUpComplete(OperationContext* opCtx, long long term) {
     auto replCoord = ReplicationCoordinator::get(opCtx);
 
-    if (!replCoord || !replCoord->isReplEnabled()) {
+    if (!replCoord || !replCoord->getSettings().isReplSet()) {
         // Unit tests may not have replication coordinator set up.
         return;
     }
@@ -209,8 +228,8 @@ void PrimaryOnlyServiceRegistry::onStepUpComplete(OperationContext* opCtx, long
 
 
     // Since this method is called before we mark the node writable in
-    // ReplicationCoordinatorImpl::signalDrainComplete and therefore can block the new primary from
-    // starting to receive writes, generate a warning if we are spending too long here.
+    // ReplicationCoordinatorImpl::signalApplierDrainComplete and therefore can block the new
+    // primary from starting to receive writes, generate a warning if we are spending too long here.
     Timer totalTime{};
     ON_BLOCK_EXIT([&] {
         auto timeSpent = totalTime.millis();
@@ -331,13 +350,11 @@ void PrimaryOnlyService::startup(OperationContext* opCtx) {
     threadPoolOptions.threadNamePrefix = getServiceName() + "-";
     threadPoolOptions.poolName = getServiceName() + "ThreadPool";
     threadPoolOptions.onCreateThread = [this](const std::string& threadName) {
-        Client::initThread(threadName.c_str());
+        Client::initThread(threadName.c_str(),
+                           getGlobalServiceContext()->getService(ClusterRole::ShardServer));
         auto client = Client::getCurrent();
         AuthorizationSession::get(*client)->grantInternalAuthorization(&cc());
 
-        stdx::lock_guard<Client> lk(*client);
-        client->setSystemOperationKillableByStepdown(lk);
-
         // Associate this Client with this PrimaryOnlyService
         primaryOnlyServiceStateForClient(client).primaryOnlyService = this;
     };
@@ -359,6 +376,7 @@ void PrimaryOnlyService::startup(OperationContext* opCtx) {
 }
 
 void PrimaryOnlyService::onStepUp(const OpTime& stepUpOpTime) {
+    using namespace fmt::literals;
     SimpleBSONObjUnorderedMap<ActiveInstance> savedInstances;
     invariant(_getHasExecutor());
     auto newThenOldScopedExecutor =
@@ -418,7 +436,7 @@ void PrimaryOnlyService::onStepUp(const OpTime& stepUpOpTime) {
                 "service"_attr = getServiceName(),
                 "stepUpOpTime"_attr = stepUpOpTime);
     WaitForMajorityService::get(_serviceContext)
-        .waitUntilMajority(stepUpOpTime, _source.token())
+        .waitUntilMajorityForWrite(stepUpOpTime, _source.token())
         .thenRunOn(**newScopedExecutor)
         .then([this, newScopedExecutor, newTerm] {
             // Note that checking both the state and the term are optimizations and are
@@ -445,6 +463,30 @@ void PrimaryOnlyService::onStepUp(const OpTime& stepUpOpTime) {
             }
             _rebuildInstances(newTerm);
         })
+        .onError([this, newTerm](Status s) {
+            LOGV2_ERROR(5165001,
+                        "Failed to rebuild PrimaryOnlyService on stepup.",
+                        "service"_attr = getServiceName(),
+                        "error"_attr = s);
+
+            stdx::lock_guard lk(_mutex);
+            if (_state != State::kRebuilding || _term != newTerm) {
+                // We've either stepped or shut down, or advanced to a new term.
+                // In either case, we rely on the stepdown/shutdown logic or the
+                // step-up of the new term to set _state and do nothing here.
+                bool steppedDown = _state == State::kPaused;
+                bool shutDown = _state == State::kShutdown;
+                bool termAdvanced = _term > newTerm;
+                invariant(
+                    steppedDown || shutDown || termAdvanced,
+                    "Unexpected _state or _term; _state is {}, _term is {}, term was {} "_format(
+                        _getStateString(lk), _term, newTerm));
+                return;
+            }
+            invariant(_state == State::kRebuilding);
+            _rebuildStatus = s;
+            _setState(State::kRebuildFailed, lk);
+        })
         .getAsync([](auto&&) {});  // Ignore the result Future
     lk.unlock();
 }
@@ -461,7 +503,7 @@ void PrimaryOnlyService::_interruptInstances(WithLock, Status status) {
     }
 
     for (auto opCtx : _opCtxs) {
-        stdx::lock_guard<Client> clientLock(*opCtx->getClient());
+        ClientLock clientLock(opCtx->getClient());
         _serviceContext->killOperation(clientLock, opCtx, status.code());
     }
 }
@@ -473,8 +515,6 @@ void PrimaryOnlyService::onStepDown() {
     }
 
     LOGV2_INFO(5123007,
-               "Interrupting (due to stepDown) PrimaryOnlyService {service} with {numInstances} "
-               "currently running instances and {numOperationContexts} associated operations",
                "Interrupting PrimaryOnlyService due to stepDown",
                "service"_attr = getServiceName(),
                "numInstances"_attr = _activeInstances.size(),
@@ -497,14 +537,11 @@ void PrimaryOnlyService::shutdown() {
     bool hasExecutor;
     {
         stdx::lock_guard lk(_mutex);
-        LOGV2_INFO(
-            5123006,
-            "Shutting down PrimaryOnlyService {service} with {numInstances} currently running "
-            "instances and {numOperationContexts} associated operations",
-            "Shutting down PrimaryOnlyService",
-            "service"_attr = getServiceName(),
-            "numInstances"_attr = _activeInstances.size(),
-            "numOperationContexts"_attr = _opCtxs.size());
+        LOGV2_INFO(5123006,
+                   "Shutting down PrimaryOnlyService",
+                   "service"_attr = getServiceName(),
+                   "numInstances"_attr = _activeInstances.size(),
+                   "numOperationContexts"_attr = _opCtxs.size());
 
         // If the _state is already kPaused, the instances have already been interrupted.
         if (_state != State::kPaused) {
@@ -593,8 +630,10 @@ std::pair<boost::optional<std::shared_ptr<PrimaryOnlyService::Instance>>, bool>
 PrimaryOnlyService::lookupInstance(OperationContext* opCtx, const InstanceID& id) {
     // If this operation is holding any database locks, then it must have opted into getting
     // interrupted at stepdown to prevent deadlocks.
-    invariant(!opCtx->lockState()->isLocked() || opCtx->shouldAlwaysInterruptAtStepDownOrUp() ||
-              opCtx->lockState()->wasGlobalLockTakenInModeConflictingWithWrites());
+    invariant(
+        !shard_role_details::getLocker(opCtx)->isLocked() ||
+        opCtx->shouldAlwaysInterruptAtStepDownOrUp() ||
+        shard_role_details::getLocker(opCtx)->wasGlobalLockTakenInModeConflictingWithWrites());
 
     stdx::unique_lock lk(_mutex);
     _waitForStateNotRebuilding(opCtx, lk);
@@ -620,8 +659,10 @@ std::vector<std::shared_ptr<PrimaryOnlyService::Instance>> PrimaryOnlyService::g
     OperationContext* opCtx) {
     // If this operation is holding any database locks, then it must have opted into getting
     // interrupted at stepdown to prevent deadlocks.
-    invariant(!opCtx->lockState()->isLocked() || opCtx->shouldAlwaysInterruptAtStepDownOrUp() ||
-              opCtx->lockState()->wasGlobalLockTakenInModeConflictingWithWrites());
+    invariant(
+        !shard_role_details::getLocker(opCtx)->isLocked() ||
+        opCtx->shouldAlwaysInterruptAtStepDownOrUp() ||
+        shard_role_details::getLocker(opCtx)->wasGlobalLockTakenInModeConflictingWithWrites());
 
     std::vector<std::shared_ptr<PrimaryOnlyService::Instance>> instances;
 
@@ -685,25 +726,20 @@ bool PrimaryOnlyService::_getHasExecutor() const {
     return _hasExecutor.load();
 }
 
-void PrimaryOnlyService::_rebuildInstances(long long term) noexcept {
+void PrimaryOnlyService::_rebuildInstances(long long term) {
     std::vector<BSONObj> stateDocuments;
 
     auto serviceName = getServiceName();
-    LOGV2_INFO(5123005,
-               "Rebuilding PrimaryOnlyService {service} due to stepUp",
-               "Rebuilding PrimaryOnlyService due to stepUp",
-               "service"_attr = serviceName);
+    LOGV2_INFO(
+        5123005, "Rebuilding PrimaryOnlyService due to stepUp", "service"_attr = serviceName);
 
     if (!MONGO_unlikely(PrimaryOnlyServiceSkipRebuildingInstances.shouldFail())) {
         auto ns = getStateDocumentsNS();
-        LOGV2_DEBUG(
-            5123004,
-            2,
-            "Querying {namespace} to look for state documents while rebuilding PrimaryOnlyService "
-            "{service}",
-            "Querying to look for state documents while rebuilding PrimaryOnlyService",
-            logAttrs(ns),
-            "service"_attr = serviceName);
+        LOGV2_DEBUG(5123004,
+                    2,
+                    "Querying to look for state documents while rebuilding PrimaryOnlyService",
+                    logAttrs(ns),
+                    "service"_attr = serviceName);
 
         // The PrimaryOnlyServiceClientObserver will make any OpCtx created as part of a
         // PrimaryOnlyService immediately get interrupted if the service is not in state kRunning.
@@ -724,65 +760,45 @@ void PrimaryOnlyService::_rebuildInstances(long long term) noexcept {
             while (cursor->more()) {
                 stateDocuments.push_back(cursor->nextSafe().getOwned());
             }
-        } catch (const DBException& e) {
-            LOGV2_ERROR(
-                4923601,
-                "Failed to start PrimaryOnlyService {service} because the query on {namespace} "
-                "for state documents failed due to {error}",
-                "Failed to start PrimaryOnlyService because the query for state documents failed",
-                "service"_attr = serviceName,
-                logAttrs(ns),
-                "error"_attr = e);
-
-            Status status = e.toStatus();
-            status.addContext(str::stream()
-                              << "Failed to start PrimaryOnlyService \"" << serviceName
-                              << "\" because the query for state documents on ns \"" << ns
-                              << "\" failed");
-
-            stdx::lock_guard lk(_mutex);
-            if (_state != State::kRebuilding || _term != term) {
-                _stateChangeCV.notify_all();
-                return;
-            }
-            _setState(State::kRebuildFailed, lk);
-            _rebuildStatus = std::move(status);
-            return;
+        } catch (DBException& e) {
+            e.addContext(str::stream()
+                         << "Error querying the state document collection "
+                         << ns.toStringForErrorMsg() << " for service " << serviceName);
+            throw;
         }
     }
 
+    LOGV2_DEBUG(5123003,
+                2,
+                "Found state documents while rebuilding PrimaryOnlyService that correspond to "
+                "instances of that service",
+                "service"_attr = serviceName,
+                "numDocuments"_attr = stateDocuments.size());
+
     while (MONGO_unlikely(PrimaryOnlyServiceHangBeforeRebuildingInstances.shouldFail())) {
         {
             stdx::lock_guard lk(_mutex);
             if (_state != State::kRebuilding || _term != term) {  // Node stepped down
-                _stateChangeCV.notify_all();
                 return;
             }
         }
         sleepmillis(100);
     }
 
-    // Must create opCtx before taking _mutex to avoid deadlock.
-    AllowOpCtxWhenServiceRebuildingBlock allowOpCtxBlock(Client::getCurrent());
-    auto opCtx = cc().makeOperationContext();
     stdx::lock_guard lk(_mutex);
     if (_state != State::kRebuilding || _term != term) {
         // Node stepped down before finishing rebuilding service from previous stepUp.
-        _stateChangeCV.notify_all();
         return;
     }
     invariant(_activeInstances.empty());
     invariant(_term == term);
 
-    LOGV2_DEBUG(5123003,
+    // Construct new instances using the state documents and add to _activeInstances.
+    LOGV2_DEBUG(5165000,
                 2,
-                "While rebuilding PrimaryOnlyService {service}, found {numDocuments} state "
-                "documents corresponding to instances of that service",
-                "Found state documents while rebuilding PrimaryOnlyService that correspond to "
-                "instances of that service",
+                "Starting to construct and run instances for service",
                 "service"_attr = serviceName,
-                "numDocuments"_attr = stateDocuments.size());
-
+                "numInstances"_attr = stateDocuments.size());
     for (auto&& doc : stateDocuments) {
         auto idElem = doc["_id"];
         fassert(4923602, !idElem.eoo());
@@ -791,6 +807,7 @@ void PrimaryOnlyService::_rebuildInstances(long long term) noexcept {
         [[maybe_unused]] auto newInstance =
             _insertNewInstance(lk, std::move(instance), std::move(instanceID));
     }
+
     _setState(State::kRunning, lk);
 }
 
@@ -803,16 +820,14 @@ std::shared_ptr<PrimaryOnlyService::Instance> PrimaryOnlyService::_insertNewInst
                    instance,
                    scopedExecutor = _scopedExecutor,
                    token = instanceSource.token(),
-                   instanceID] {
+                   instanceID]() mutable {
                 LOGV2_DEBUG(5123002,
                             3,
-                            "Starting instance of PrimaryOnlyService {service} with InstanceID "
-                            "{instanceID}",
                             "Starting instance of PrimaryOnlyService",
                             "service"_attr = serviceName,
                             "instanceID"_attr = instanceID);
 
-                return instance->run(std::move(scopedExecutor), std::move(token));
+                return instance->run(std::move(scopedExecutor), token);
             })
             // TODO SERVER-61717 remove this error handler once instance are automatically released
             // at the end of run()
