@@ -27,21 +27,25 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/repl/primary_only_service_op_observer.h"
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <utility>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/timestamp.h"
 #include "mongo/db/repl/primary_only_service.h"
+#include "mongo/db/repl/primary_only_service_op_observer.h"
+#include "mongo/db/storage/recovery_unit.h"
+#include "mongo/db/transaction_resources.h"
+#include "mongo/util/assert_util_core.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/str.h"
 
 namespace mongo {
 namespace repl {
 
-namespace {
-
-const auto documentIdDecoration = OperationContext::declareDecoration<BSONObj>();
-
-}  // namespace
-
 PrimaryOnlyServiceOpObserver::PrimaryOnlyServiceOpObserver(ServiceContext* serviceContext) {
     _registry = PrimaryOnlyServiceRegistry::get(serviceContext);
 }
@@ -49,29 +53,25 @@ PrimaryOnlyServiceOpObserver::PrimaryOnlyServiceOpObserver(ServiceContext* servi
 PrimaryOnlyServiceOpObserver::~PrimaryOnlyServiceOpObserver() = default;
 
 
-void PrimaryOnlyServiceOpObserver::aboutToDelete(OperationContext* opCtx,
-                                                 NamespaceString const& nss,
-                                                 const UUID& uuid,
-                                                 BSONObj const& doc) {
+void PrimaryOnlyServiceOpObserver::onDelete(OperationContext* opCtx,
+                                            const CollectionPtr& coll,
+                                            StmtId stmtId,
+                                            const BSONObj& doc,
+                                            const OplogDeleteEntryArgs& args,
+                                            OpStateAccumulator* opAccumulator) {
+    const auto& nss = coll->ns();
+
     // Extract the _id field from the document. If it does not have an _id, use the
     // document itself as the _id.
-    documentIdDecoration(opCtx) = doc["_id"] ? doc["_id"].wrap() : doc;
-}
-
-void PrimaryOnlyServiceOpObserver::onDelete(OperationContext* opCtx,
-                                            const NamespaceString& nss,
-                                            const UUID& uuid,
-                                            StmtId stmtId,
-                                            const OplogDeleteEntryArgs& args) {
-    auto& documentId = documentIdDecoration(opCtx);
+    auto documentId = doc["_id"] ? doc["_id"].wrap() : doc;
     invariant(!documentId.isEmpty());
 
     auto service = _registry->lookupServiceByNamespace(nss);
     if (!service) {
         return;
     }
-    opCtx->recoveryUnit()->onCommit(
-        [service, documentId, nss](boost::optional<Timestamp> unusedCommitTime) {
+    shard_role_details::getRecoveryUnit(opCtx)->onCommit(
+        [service, documentId, nss](OperationContext*, boost::optional<Timestamp>) {
             // Release the instance without interrupting it since for some primary-only services
             // there is still work to be done after the state document is removed.
             service->releaseInstance(documentId, Status::OK());
@@ -83,15 +83,17 @@ repl::OpTime PrimaryOnlyServiceOpObserver::onDropCollection(OperationContext* op
                                                             const NamespaceString& collectionName,
                                                             const UUID& uuid,
                                                             std::uint64_t numRecords,
-                                                            const CollectionDropType dropType) {
+                                                            const CollectionDropType dropType,
+                                                            bool markFromMigrate) {
     auto service = _registry->lookupServiceByNamespace(collectionName);
     if (service) {
-        opCtx->recoveryUnit()->onCommit(
-            [service, collectionName](boost::optional<Timestamp> unusedCommitTime) {
+        shard_role_details::getRecoveryUnit(opCtx)->onCommit(
+            [service, collectionName](OperationContext*, boost::optional<Timestamp>) {
                 // Release and interrupt all the instances since the state document collection is
                 // not supposed to be dropped.
-                service->releaseAllInstances(Status(
-                    ErrorCodes::Interrupted, str::stream() << collectionName << " is dropped"));
+                service->releaseAllInstances(
+                    Status(ErrorCodes::Interrupted,
+                           str::stream() << collectionName.toStringForErrorMsg() << " is dropped"));
             });
     }
     return {};
