@@ -27,27 +27,40 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplication
-
-#include "mongo/platform/basic.h"
-
 #include "mongo/db/repl/read_concern_args.h"
 
+#include <utility>
+
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsontypes.h"
 #include "mongo/bson/util/bson_extract.h"
-#include "mongo/db/jsobj.h"
 #include "mongo/db/logical_time.h"
 #include "mongo/db/operation_context.h"
+#include "mongo/db/read_write_concern_provenance_base_gen.h"
 #include "mongo/db/repl/bson_extract_optime.h"
+#include "mongo/db/repl/read_concern_gen.h"
+#include "mongo/db/repl/read_concern_level.h"
+#include "mongo/idl/idl_parser.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/decorable.h"
 #include "mongo/util/str.h"
 
-using std::string;
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplication
 
 namespace mongo {
 namespace repl {
+namespace {
 
 const OperationContext::Decoration<ReadConcernArgs> handle =
     OperationContext::declareDecoration<ReadConcernArgs>();
 
+}  // namespace
+
 ReadConcernArgs& ReadConcernArgs::get(OperationContext* opCtx) {
     return handle(opCtx);
 }
@@ -56,7 +69,6 @@ const ReadConcernArgs& ReadConcernArgs::get(const OperationContext* opCtx) {
     return handle(opCtx);
 }
 
-
 // The "kImplicitDefault" read concern, used by internal operations, is deliberately empty (no
 // 'level' specified). This allows internal operations to specify a read concern, while still
 // allowing it to be either local or available on sharded secondaries.
@@ -64,7 +76,8 @@ const BSONObj ReadConcernArgs::kImplicitDefault;
 
 // The "kLocal" read concern just specifies that the read concern is local. This is used for
 // internal operations intended to be run only on a certain target cluster member.
-const BSONObj ReadConcernArgs::kLocal = BSON(kLevelFieldName << readConcernLevels::kLocalName);
+const BSONObj ReadConcernArgs::kLocal =
+    BSON(kLevelFieldName << readConcernLevels::toString(ReadConcernLevel::kLocalReadConcern));
 
 ReadConcernArgs::ReadConcernArgs() : _specified(false) {}
 
@@ -81,6 +94,90 @@ ReadConcernArgs::ReadConcernArgs(boost::optional<LogicalTime> afterClusterTime,
       _level(std::move(level)),
       _specified(_afterClusterTime || _level) {}
 
+Status ReadConcernArgs::parse(ReadConcernIdl inner) {
+    if (auto& opTime = inner.getAfterOpTime()) {
+        _opTime = *opTime;
+    }
+    _afterClusterTime = inner.getAfterClusterTime();
+    _atClusterTime = inner.getAtClusterTime();
+    if (auto level = inner.getLevel()) {
+        _level = *level;
+    }
+    if (auto allowTxnTableSnapshot = inner.getAllowTransactionTableSnapshot()) {
+        _allowTransactionTableSnapshot = *allowTxnTableSnapshot;
+    }
+    if (auto wait = inner.getWaitLastStableRecoveryTimestamp()) {
+        _waitLastStableRecoveryTimestamp = *wait;
+    }
+    if (auto provenance = inner.getProvenance()) {
+        _provenance = *provenance;
+    }
+
+    if (_afterClusterTime && _opTime) {
+        return Status(ErrorCodes::InvalidOptions,
+                      str::stream()
+                          << "Can not specify both " << ReadConcernIdl::kAfterClusterTimeFieldName
+                          << " and " << ReadConcernIdl::kAfterOpTimeFieldName);
+    }
+
+    if (_afterClusterTime && _atClusterTime) {
+        return Status(ErrorCodes::InvalidOptions,
+                      "Specifying a timestamp for readConcern snapshot in a causally consistent "
+                      "session is not allowed. See "
+                      "https://docs.mongodb.com/manual/core/read-isolation-consistency-recency/"
+                      "#causal-consistency");
+    }
+
+    // Note: 'available' should not be used with after cluster time, as cluster time can wait for
+    // replication whereas the premise of 'available' is to avoid waiting. 'linearizable' should not
+    // be used with after cluster time, since linearizable reads are inherently causally consistent.
+    if (_afterClusterTime && getLevel() != ReadConcernLevel::kMajorityReadConcern &&
+        getLevel() != ReadConcernLevel::kLocalReadConcern &&
+        getLevel() != ReadConcernLevel::kSnapshotReadConcern) {
+        return Status(ErrorCodes::InvalidOptions,
+                      str::stream()
+                          << kAfterClusterTimeFieldName << " field can be set only if "
+                          << kLevelFieldName << " is equal to "
+                          << readConcernLevels::toString(ReadConcernLevel::kMajorityReadConcern)
+                          << ", "
+                          << readConcernLevels::toString(ReadConcernLevel::kLocalReadConcern)
+                          << ", or "
+                          << readConcernLevels::toString(ReadConcernLevel::kSnapshotReadConcern));
+    }
+
+    if (_opTime && getLevel() == ReadConcernLevel::kSnapshotReadConcern) {
+        return Status(ErrorCodes::InvalidOptions,
+                      str::stream()
+                          << kAfterOpTimeFieldName << " field cannot be set if " << kLevelFieldName
+                          << " is equal to "
+                          << readConcernLevels::toString(ReadConcernLevel::kSnapshotReadConcern));
+    }
+
+    if (_atClusterTime && getLevel() != ReadConcernLevel::kSnapshotReadConcern) {
+        return Status(ErrorCodes::InvalidOptions,
+                      str::stream()
+                          << kAtClusterTimeFieldName << " field can be set only if "
+                          << kLevelFieldName << " is equal to "
+                          << readConcernLevels::toString(ReadConcernLevel::kSnapshotReadConcern));
+    }
+
+    // Make sure that atClusterTime wasn't specified with zero seconds.
+    if (_atClusterTime && _atClusterTime->asTimestamp().isNull()) {
+        return Status(ErrorCodes::InvalidOptions,
+                      str::stream() << kAtClusterTimeFieldName << " cannot be a null timestamp");
+    }
+
+    // It's okay for afterClusterTime to be specified with zero seconds, but not an uninitialized
+    // timestamp.
+    if (_afterClusterTime && _afterClusterTime == LogicalTime::kUninitialized) {
+        return Status(ErrorCodes::InvalidOptions,
+                      str::stream() << kAfterClusterTimeFieldName << " cannot be a null timestamp");
+    }
+
+    _specified = true;
+    return Status::OK();
+}
+
 std::string ReadConcernArgs::toString() const {
     return toBSON().toString();
 }
@@ -101,34 +198,10 @@ bool ReadConcernArgs::isEmpty() const {
     return !_afterClusterTime && !_opTime && !_atClusterTime && !_level;
 }
 
-bool ReadConcernArgs::isSpecified() const {
-    return _specified;
-}
-
 bool ReadConcernArgs::isImplicitDefault() const {
     return getProvenance().isImplicitDefault();
 }
 
-ReadConcernLevel ReadConcernArgs::getLevel() const {
-    return _level.value_or(ReadConcernLevel::kLocalReadConcern);
-}
-
-bool ReadConcernArgs::hasLevel() const {
-    return _level.is_initialized();
-}
-
-boost::optional<OpTime> ReadConcernArgs::getArgsOpTime() const {
-    return _opTime;
-}
-
-boost::optional<LogicalTime> ReadConcernArgs::getArgsAfterClusterTime() const {
-    return _afterClusterTime;
-}
-
-boost::optional<LogicalTime> ReadConcernArgs::getArgsAtClusterTime() const {
-    return _atClusterTime;
-}
-
 Status ReadConcernArgs::initialize(const BSONElement& readConcernElem) {
     invariant(isEmpty());  // only legal to call on uninitialized object.
     _specified = false;
@@ -148,130 +221,13 @@ Status ReadConcernArgs::initialize(const BSONElement& readConcernElem) {
 
 Status ReadConcernArgs::parse(const BSONObj& readConcernObj) {
     invariant(isEmpty());  // only legal to call on uninitialized object.
-    for (auto&& field : readConcernObj) {
-        auto fieldName = field.fieldNameStringData();
-        if (fieldName == kAfterOpTimeFieldName) {
-            OpTime opTime;
-            // TODO pass field in rather than scanning again.
-            auto opTimeStatus =
-                bsonExtractOpTimeField(readConcernObj, kAfterOpTimeFieldName, &opTime);
-            if (!opTimeStatus.isOK()) {
-                return opTimeStatus;
-            }
-            _opTime = opTime;
-        } else if (fieldName == kAfterClusterTimeFieldName) {
-            Timestamp afterClusterTime;
-            auto afterClusterTimeStatus = bsonExtractTimestampField(
-                readConcernObj, kAfterClusterTimeFieldName, &afterClusterTime);
-            if (!afterClusterTimeStatus.isOK()) {
-                return afterClusterTimeStatus;
-            }
-            _afterClusterTime = LogicalTime(afterClusterTime);
-        } else if (fieldName == kAtClusterTimeFieldName) {
-            Timestamp atClusterTime;
-            auto atClusterTimeStatus =
-                bsonExtractTimestampField(readConcernObj, kAtClusterTimeFieldName, &atClusterTime);
-            if (!atClusterTimeStatus.isOK()) {
-                return atClusterTimeStatus;
-            }
-            _atClusterTime = LogicalTime(atClusterTime);
-        } else if (fieldName == kLevelFieldName) {
-            std::string levelString;
-            // TODO pass field in rather than scanning again.
-            auto readCommittedStatus =
-                bsonExtractStringField(readConcernObj, kLevelFieldName, &levelString);
 
-            if (!readCommittedStatus.isOK()) {
-                return readCommittedStatus;
-            }
-
-            _level = readConcernLevels::fromString(levelString);
-            if (!_level) {
-                return Status(ErrorCodes::FailedToParse,
-                              str::stream() << kReadConcernFieldName << '.' << kLevelFieldName
-                                            << " must be either '" << readConcernLevels::kLocalName
-                                            << "', '" << readConcernLevels::kMajorityName << "', '"
-                                            << readConcernLevels::kLinearizableName << "', '"
-                                            << readConcernLevels::kAvailableName << "', or '"
-                                            << readConcernLevels::kSnapshotName << "'");
-            }
-        } else if (fieldName == kAllowTransactionTableSnapshot) {
-            auto status = bsonExtractBooleanField(
-                readConcernObj, kAllowTransactionTableSnapshot, &_allowTransactionTableSnapshot);
-            if (!status.isOK()) {
-                return status;
-            }
-        } else if (fieldName == ReadWriteConcernProvenance::kSourceFieldName) {
-            try {
-                _provenance = ReadWriteConcernProvenance::parse(
-                    IDLParserErrorContext("ReadConcernArgs::parse"), readConcernObj);
-            } catch (const DBException&) {
-                return exceptionToStatus();
-            }
-        } else {
-            return Status(ErrorCodes::InvalidOptions,
-                          str::stream() << "Unrecognized option in " << kReadConcernFieldName
-                                        << ": " << fieldName);
-        }
+    try {
+        auto inner = ReadConcernIdl::parse(IDLParserContext("readConcern"), readConcernObj);
+        return parse(std::move(inner));
+    } catch (const DBException& ex) {
+        return ex.toStatus();
     }
-
-    if (_afterClusterTime && _opTime) {
-        return Status(ErrorCodes::InvalidOptions,
-                      str::stream() << "Can not specify both " << kAfterClusterTimeFieldName
-                                    << " and " << kAfterOpTimeFieldName);
-    }
-
-    if (_afterClusterTime && _atClusterTime) {
-        return Status(ErrorCodes::InvalidOptions,
-                      "Specifying a timestamp for readConcern snapshot in a causally consistent "
-                      "session is not allowed. See "
-                      "https://docs.mongodb.com/manual/core/read-isolation-consistency-recency/"
-                      "#causal-consistency");
-    }
-
-    // Note: 'available' should not be used with after cluster time, as cluster time can wait for
-    // replication whereas the premise of 'available' is to avoid waiting. 'linearizable' should not
-    // be used with after cluster time, since linearizable reads are inherently causally consistent.
-    if (_afterClusterTime && getLevel() != ReadConcernLevel::kMajorityReadConcern &&
-        getLevel() != ReadConcernLevel::kLocalReadConcern &&
-        getLevel() != ReadConcernLevel::kSnapshotReadConcern) {
-        return Status(ErrorCodes::InvalidOptions,
-                      str::stream()
-                          << kAfterClusterTimeFieldName << " field can be set only if "
-                          << kLevelFieldName << " is equal to " << readConcernLevels::kMajorityName
-                          << ", " << readConcernLevels::kLocalName << ", or "
-                          << readConcernLevels::kSnapshotName);
-    }
-
-    if (_opTime && getLevel() == ReadConcernLevel::kSnapshotReadConcern) {
-        return Status(ErrorCodes::InvalidOptions,
-                      str::stream()
-                          << kAfterOpTimeFieldName << " field cannot be set if " << kLevelFieldName
-                          << " is equal to " << readConcernLevels::kSnapshotName);
-    }
-
-    if (_atClusterTime && getLevel() != ReadConcernLevel::kSnapshotReadConcern) {
-        return Status(ErrorCodes::InvalidOptions,
-                      str::stream() << kAtClusterTimeFieldName << " field can be set only if "
-                                    << kLevelFieldName << " is equal to "
-                                    << readConcernLevels::kSnapshotName);
-    }
-
-    // Make sure that atClusterTime wasn't specified with zero seconds.
-    if (_atClusterTime && _atClusterTime->asTimestamp().isNull()) {
-        return Status(ErrorCodes::InvalidOptions,
-                      str::stream() << kAtClusterTimeFieldName << " cannot be a null timestamp");
-    }
-
-    // It's okay for afterClusterTime to be specified with zero seconds, but not an uninitialized
-    // timestamp.
-    if (_afterClusterTime && _afterClusterTime == LogicalTime::kUninitialized) {
-        return Status(ErrorCodes::InvalidOptions,
-                      str::stream() << kAfterClusterTimeFieldName << " cannot be a null timestamp");
-    }
-
-    _specified = true;
-    return Status::OK();
 }
 
 ReadConcernArgs ReadConcernArgs::fromBSONThrows(const BSONObj& readConcernObj) {
@@ -280,6 +236,12 @@ ReadConcernArgs ReadConcernArgs::fromBSONThrows(const BSONObj& readConcernObj) {
     return rc;
 }
 
+ReadConcernArgs ReadConcernArgs::fromIDLThrows(ReadConcernIdl readConcern) {
+    ReadConcernArgs rc;
+    uassertStatusOK(rc.parse(std::move(readConcern)));
+    return rc;
+}
+
 void ReadConcernArgs::setMajorityReadMechanism(MajorityReadMechanism mechanism) {
     invariant(*_level == ReadConcernLevel::kMajorityReadConcern);
     _majorityReadMechanism = mechanism;
@@ -295,28 +257,25 @@ bool ReadConcernArgs::isSpeculativeMajority() const {
         _majorityReadMechanism == MajorityReadMechanism::kSpeculative;
 }
 
-void ReadConcernArgs::_appendInfoInner(BSONObjBuilder* builder) const {
-    if (_level) {
-        builder->append(kLevelFieldName, readConcernLevels::toString(_level.get()));
-    }
-
-    if (_opTime) {
-        _opTime->append(builder, kAfterOpTimeFieldName.toString());
-    }
-
-    if (_afterClusterTime) {
-        builder->append(kAfterClusterTimeFieldName, _afterClusterTime->asTimestamp());
-    }
-
-    if (_atClusterTime) {
-        builder->append(kAtClusterTimeFieldName, _atClusterTime->asTimestamp());
-    }
-
+ReadConcernIdl ReadConcernArgs::toReadConcernIdl() const {
+    ReadConcernIdl rc;
+    rc.setLevel(_level);
+    rc.setAfterOpTime(_opTime);
+    rc.setAfterClusterTime(_afterClusterTime);
+    rc.setAtClusterTime(_atClusterTime);
     if (_allowTransactionTableSnapshot) {
-        builder->append(kAllowTransactionTableSnapshot, _allowTransactionTableSnapshot);
+        rc.setAllowTransactionTableSnapshot(_allowTransactionTableSnapshot);
     }
+    if (_waitLastStableRecoveryTimestamp) {
+        rc.setWaitLastStableRecoveryTimestamp(_waitLastStableRecoveryTimestamp);
+    }
+    rc.setProvenance(_provenance.getSource());
 
-    _provenance.serialize(builder);
+    return rc;
+}
+
+void ReadConcernArgs::_appendInfoInner(BSONObjBuilder* builder) const {
+    toReadConcernIdl().serialize(builder);
 }
 
 void ReadConcernArgs::appendInfo(BSONObjBuilder* builder) const {
@@ -325,42 +284,8 @@ void ReadConcernArgs::appendInfo(BSONObjBuilder* builder) const {
     rcBuilder.done();
 }
 
-boost::optional<ReadConcernLevel> readConcernLevels::fromString(StringData levelString) {
-    if (levelString == readConcernLevels::kLocalName) {
-        return ReadConcernLevel::kLocalReadConcern;
-    } else if (levelString == readConcernLevels::kMajorityName) {
-        return ReadConcernLevel::kMajorityReadConcern;
-    } else if (levelString == readConcernLevels::kLinearizableName) {
-        return ReadConcernLevel::kLinearizableReadConcern;
-    } else if (levelString == readConcernLevels::kAvailableName) {
-        return ReadConcernLevel::kAvailableReadConcern;
-    } else if (levelString == readConcernLevels::kSnapshotName) {
-        return ReadConcernLevel::kSnapshotReadConcern;
-    } else {
-        return boost::none;
-    }
-}
-
 StringData readConcernLevels::toString(ReadConcernLevel level) {
-    switch (level) {
-        case ReadConcernLevel::kLocalReadConcern:
-            return kLocalName;
-
-        case ReadConcernLevel::kMajorityReadConcern:
-            return kMajorityName;
-
-        case ReadConcernLevel::kLinearizableReadConcern:
-            return kLinearizableName;
-
-        case ReadConcernLevel::kAvailableReadConcern:
-            return kAvailableName;
-
-        case ReadConcernLevel::kSnapshotReadConcern:
-            return kSnapshotName;
-
-        default:
-            MONGO_UNREACHABLE;
-    }
+    return ReadConcernLevel_serializer(level);
 }
 
 }  // namespace repl
