@@ -29,33 +29,42 @@
 
 #pragma once
 
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
 #include <boost/optional.hpp>
+#include <boost/optional/optional.hpp>
 #include <string>
 
 #include "mongo/base/status.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/timestamp.h"
 #include "mongo/db/json.h"
 #include "mongo/db/logical_time.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/read_write_concern_provenance.h"
 #include "mongo/db/repl/optime.h"
+#include "mongo/db/repl/read_concern_gen.h"
 #include "mongo/db/repl/read_concern_level.h"
-#include "mongo/util/time_support.h"
+#include "mongo/util/assert_util_core.h"
 
 namespace mongo {
-
-class BSONObj;
-class OperationContext;
-
 namespace repl {
 
 class ReadConcernArgs {
 public:
     static constexpr StringData kReadConcernFieldName = "readConcern"_sd;
-    static constexpr StringData kAfterOpTimeFieldName = "afterOpTime"_sd;
-    static constexpr StringData kAfterClusterTimeFieldName = "afterClusterTime"_sd;
-    static constexpr StringData kAtClusterTimeFieldName = "atClusterTime"_sd;
-    static constexpr StringData kLevelFieldName = "level"_sd;
+    static constexpr StringData kAfterOpTimeFieldName = ReadConcernIdl::kAfterOpTimeFieldName;
+    static constexpr StringData kAfterClusterTimeFieldName =
+        ReadConcernIdl::kAfterClusterTimeFieldName;
+    static constexpr StringData kAtClusterTimeFieldName = ReadConcernIdl::kAtClusterTimeFieldName;
+    static constexpr StringData kLevelFieldName = ReadConcernIdl::kLevelFieldName;
     static constexpr StringData kAllowTransactionTableSnapshot =
-        "$_allowTransactionTableSnapshot"_sd;
+        ReadConcernIdl::kAllowTransactionTableSnapshotFieldName;
+    static constexpr StringData kWaitLastStableRecoveryTimestamp =
+        ReadConcernIdl::kWaitLastStableRecoveryTimestampFieldName;
 
     static const BSONObj kImplicitDefault;
     static const BSONObj kLocal;
@@ -85,6 +94,7 @@ public:
 
     ReadConcernArgs(boost::optional<LogicalTime> afterClusterTime,
                     boost::optional<ReadConcernLevel> level);
+
     /**
      * Format:
      * {
@@ -114,7 +124,14 @@ public:
      */
     Status parse(const BSONObj& readConcernObj);
 
+    /**
+     * Initializes the object by parsing the IDL representation of the actual readConcern
+     * sub-object.
+     */
+    Status parse(ReadConcernIdl inner);
+
     static ReadConcernArgs fromBSONThrows(const BSONObj& readConcernObj);
+    static ReadConcernArgs fromIDLThrows(ReadConcernIdl readConcern);
 
     /**
      * Sets the mechanism we should use to satisfy 'majority' reads.
@@ -151,7 +168,9 @@ public:
      * If the RC was specified as an empty BSON object this will still be true (unlike isEmpty()).
      * False represents an absent or missing read concern, ie. one which wasn't present at all.
      */
-    bool isSpecified() const;
+    bool isSpecified() const {
+        return _specified;
+    }
 
     /**
      * Returns true if this ReadConcernArgs represents an implicit default read concern.
@@ -161,20 +180,31 @@ public:
     /**
      *  Returns default kLocalReadConcern if _level is not set.
      */
-    ReadConcernLevel getLevel() const;
+    ReadConcernLevel getLevel() const {
+        return _level.value_or(ReadConcernLevel::kLocalReadConcern);
+    }
+
     /**
      * Checks whether _level is explicitly set.
      */
-    bool hasLevel() const;
+    bool hasLevel() const {
+        return _level.has_value();
+    }
 
     /**
      * Returns the opTime. Deprecated: will be replaced with getArgsAfterClusterTime.
      */
-    boost::optional<OpTime> getArgsOpTime() const;
+    boost::optional<OpTime> getArgsOpTime() const {
+        return _opTime;
+    }
 
-    boost::optional<LogicalTime> getArgsAfterClusterTime() const;
+    boost::optional<LogicalTime> getArgsAfterClusterTime() const {
+        return _afterClusterTime;
+    }
 
-    boost::optional<LogicalTime> getArgsAtClusterTime() const;
+    boost::optional<LogicalTime> getArgsAtClusterTime() const {
+        return _atClusterTime;
+    }
 
     /**
      * Returns a BSON object of the form:
@@ -193,6 +223,11 @@ public:
     BSONObj toBSONInner() const;
     std::string toString() const;
 
+    /**
+     * Returns the IDL-struct representation of this object's inner BSON serialized form.
+     */
+    ReadConcernIdl toReadConcernIdl() const;
+
     ReadWriteConcernProvenance& getProvenance() {
         return _provenance;
     }
@@ -225,6 +260,14 @@ public:
         return _allowTransactionTableSnapshot;
     }
 
+    bool waitLastStableRecoveryTimestamp() const {
+        return _waitLastStableRecoveryTimestamp;
+    }
+
+    void setWaitLastStableRecoveryTimestamp(bool wait) {
+        _waitLastStableRecoveryTimestamp = wait;
+    }
+
 private:
     /**
      * Appends level, afterOpTime, and the other "inner" fields of the read concern args.
@@ -263,6 +306,8 @@ private:
     bool _atClusterTimeSelected = false;
 
     bool _allowTransactionTableSnapshot = false;
+
+    bool _waitLastStableRecoveryTimestamp = false;
 };
 
 }  // namespace repl
