@@ -31,9 +31,15 @@
 
 #include <string>
 
+#include "mongo/base/status.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/timestamp.h"
 #include "mongo/db/repl/member_state.h"
 #include "mongo/db/repl/optime.h"
 #include "mongo/db/repl/repl_set_config.h"
+#include "mongo/util/net/hostandport.h"
 #include "mongo/util/time_support.h"
 
 namespace mongo {
@@ -110,6 +116,11 @@ public:
     }
     OpTime getAppliedOpTime() const;
     OpTimeAndWallTime getAppliedOpTimeAndWallTime() const;
+    bool hasWrittenOpTime() const {
+        return _writtenOpTimeSet;
+    }
+    OpTime getWrittenOpTime() const;
+    OpTimeAndWallTime getWrittenOpTimeAndWallTime() const;
     bool hasDurableOpTime() const {
         return _durableOpTimeSet;
     }
@@ -181,6 +192,11 @@ public:
         _appliedOpTime = time.opTime;
         _appliedWallTime = time.wallTime;
     }
+    void setWrittenOpTimeAndWallTime(OpTimeAndWallTime time) {
+        _writtenOpTimeSet = true;
+        _writtenOpTime = time.opTime;
+        _writtenWallTime = time.wallTime;
+    }
     void setDurableOpTimeAndWallTime(OpTimeAndWallTime time) {
         _durableOpTimeSet = true;
         _durableOpTime = time.opTime;
@@ -207,6 +223,10 @@ private:
     OpTime _appliedOpTime;
     Date_t _appliedWallTime;
 
+    bool _writtenOpTimeSet = false;
+    OpTime _writtenOpTime;
+    Date_t _writtenWallTime;
+
     bool _durableOpTimeSet = false;
     OpTime _durableOpTime;
     Date_t _durableWallTime;
