@@ -41,7 +41,7 @@ namespace repl {
 class ReplSettings {
 public:
     std::string ourSetName() const;
-    bool usingReplSets() const;
+    bool isReplSet() const;
 
     /**
      * Getters
@@ -49,6 +49,7 @@ public:
     long long getOplogSizeBytes() const;
     std::string getReplSetString() const;
     bool isServerless() const;
+    bool shouldAutoInitiate() const;
 
     /**
      * Static getter for the 'recoverFromOplogAsStandalone' server parameter.
@@ -61,11 +62,13 @@ public:
     void setOplogSizeBytes(long long oplogSizeBytes);
     void setReplSetString(std::string replSetString);
     void setServerlessMode();
+    void setShouldAutoInitiate();
 
 private:
     long long _oplogSizeBytes = 0;  // --oplogSize
 
     bool _isServerless = false;
+    bool _shouldAutoInitiate = false;
     std::string _replSetString;  // --replSet[/<seedlist>]
 };
 
