@@ -27,15 +27,29 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
 #include <memory>
+#include <ratio>
 
+#include <boost/move/utility_core.hpp>
+
+#include "mongo/base/string_data.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/db/catalog/database_holder.h"
 #include "mongo/db/catalog/database_holder_mock.h"
+#include "mongo/db/repl/optime.h"
+#include "mongo/db/repl/repl_server_parameters_gen.h"
+#include "mongo/db/repl/repl_set_config.h"
 #include "mongo/db/repl/replica_set_aware_service.h"
+#include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/repl/replication_coordinator_mock.h"
 #include "mongo/db/service_context_test_fixture.h"
-#include "mongo/unittest/log_test.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/duration.h"
+#include "mongo/util/fail_point.h"
+#include "mongo/util/time_support.h"
 
 namespace mongo {
 
@@ -50,6 +64,7 @@ public:
     int numCallsOnStepUpBegin{0};
     int numCallsOnStepUpComplete{0};
     int numCallsOnStepDown{0};
+    int numCallsOnRollback{0};
     int numCallsOnBecomeArbiter{0};
 
 protected:
@@ -77,6 +92,10 @@ protected:
         numCallsOnStepDown++;
     }
 
+    void onRollback() override {
+        numCallsOnRollback++;
+    }
+
     void onBecomeArbiter() override {
         numCallsOnBecomeArbiter++;
     }
@@ -96,7 +115,7 @@ private:
         return false;
     }
 
-    std::string getServiceName() const final override {
+    std::string getServiceName() const final {
         return "ServiceA";
     }
 };
@@ -122,7 +141,7 @@ private:
         return true;
     }
 
-    std::string getServiceName() const final override {
+    std::string getServiceName() const final {
         return "ServiceB";
     }
 };
@@ -150,7 +169,7 @@ private:
         return true;
     }
 
-    std::string getServiceName() const final override {
+    std::string getServiceName() const final {
         return "ServiceC";
     }
 
@@ -188,6 +207,11 @@ private:
         TestService::onStepDown();
     }
 
+    void onRollback() final {
+        ASSERT_EQ(numCallsOnRollback, ServiceB::get(getServiceContext())->numCallsOnRollback - 1);
+        TestService::onRollback();
+    }
+
     void onBecomeArbiter() final {
         ASSERT_EQ(numCallsOnBecomeArbiter,
                   ServiceB::get(getServiceContext())->numCallsOnBecomeArbiter - 1);
@@ -233,7 +257,7 @@ private:
         return true;
     }
 
-    std::string getServiceName() const final override {
+    std::string getServiceName() const final {
         return "SlowService";
     }
 
@@ -266,7 +290,7 @@ public:
 
         auto serviceContext = getServiceContext();
         auto replCoord = std::make_unique<repl::ReplicationCoordinatorMock>(serviceContext);
-        replCoord->setMyLastAppliedOpTimeAndWallTime(
+        replCoord->setMyLastAppliedOpTimeAndWallTimeForward(
             repl::OpTimeAndWallTime(repl::OpTime(Timestamp(1, 1), _term), Date_t()));
         repl::ReplicationCoordinator::set(serviceContext, std::move(replCoord));
 
@@ -308,6 +332,7 @@ TEST_F(ReplicaSetAwareServiceTest, ReplicaSetAwareService) {
     ASSERT_EQ(0, a->numCallsOnStepUpBegin);
     ASSERT_EQ(0, a->numCallsOnStepUpComplete);
     ASSERT_EQ(0, a->numCallsOnStepDown);
+    ASSERT_EQ(0, a->numCallsOnRollback);
     ASSERT_EQ(0, a->numCallsOnBecomeArbiter);
 
     ASSERT_EQ(0, b->numCallsOnStartup);
@@ -316,6 +341,7 @@ TEST_F(ReplicaSetAwareServiceTest, ReplicaSetAwareService) {
     ASSERT_EQ(0, b->numCallsOnStepUpBegin);
     ASSERT_EQ(0, b->numCallsOnStepUpComplete);
     ASSERT_EQ(0, b->numCallsOnStepDown);
+    ASSERT_EQ(0, b->numCallsOnRollback);
     ASSERT_EQ(0, b->numCallsOnBecomeArbiter);
 
     ASSERT_EQ(0, c->numCallsOnStartup);
@@ -324,6 +350,7 @@ TEST_F(ReplicaSetAwareServiceTest, ReplicaSetAwareService) {
     ASSERT_EQ(0, c->numCallsOnStepUpBegin);
     ASSERT_EQ(0, c->numCallsOnStepUpComplete);
     ASSERT_EQ(0, c->numCallsOnStepDown);
+    ASSERT_EQ(0, c->numCallsOnRollback);
     ASSERT_EQ(0, c->numCallsOnBecomeArbiter);
 
     ReplicaSetAwareServiceRegistry::get(sc).onStartup(opCtx);
@@ -337,6 +364,7 @@ TEST_F(ReplicaSetAwareServiceTest, ReplicaSetAwareService) {
     ReplicaSetAwareServiceRegistry::get(sc).onStepUpComplete(opCtx, _term);
     ReplicaSetAwareServiceRegistry::get(sc).onStepUpComplete(opCtx, _term);
     ReplicaSetAwareServiceRegistry::get(sc).onStepDown();
+    ReplicaSetAwareServiceRegistry::get(sc).onRollback();
     ReplicaSetAwareServiceRegistry::get(sc).onBecomeArbiter();
 
     ASSERT_EQ(0, a->numCallsOnStartup);
@@ -345,6 +373,7 @@ TEST_F(ReplicaSetAwareServiceTest, ReplicaSetAwareService) {
     ASSERT_EQ(0, a->numCallsOnStepUpBegin);
     ASSERT_EQ(0, a->numCallsOnStepUpComplete);
     ASSERT_EQ(0, a->numCallsOnStepDown);
+    ASSERT_EQ(0, a->numCallsOnRollback);
     ASSERT_EQ(0, a->numCallsOnBecomeArbiter);
 
     ASSERT_EQ(1, b->numCallsOnStartup);
@@ -353,6 +382,7 @@ TEST_F(ReplicaSetAwareServiceTest, ReplicaSetAwareService) {
     ASSERT_EQ(3, b->numCallsOnStepUpBegin);
     ASSERT_EQ(2, b->numCallsOnStepUpComplete);
     ASSERT_EQ(1, b->numCallsOnStepDown);
+    ASSERT_EQ(1, b->numCallsOnRollback);
     ASSERT_EQ(1, b->numCallsOnBecomeArbiter);
 
     ASSERT_EQ(1, c->numCallsOnStartup);
@@ -361,6 +391,7 @@ TEST_F(ReplicaSetAwareServiceTest, ReplicaSetAwareService) {
     ASSERT_EQ(3, c->numCallsOnStepUpBegin);
     ASSERT_EQ(2, c->numCallsOnStepUpComplete);
     ASSERT_EQ(1, c->numCallsOnStepDown);
+    ASSERT_EQ(1, c->numCallsOnRollback);
     ASSERT_EQ(1, c->numCallsOnBecomeArbiter);
 }
 
