@@ -29,30 +29,57 @@
 
 #pragma once
 
-#include "mongo/db/repl/replication_coordinator_fwd.h"
-
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <cstddef>
+#include <cstdint>
+#include <functional>
+#include <memory>
+#include <string>
 #include <vector>
 
 #include "mongo/base/status.h"
 #include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/oid.h"
 #include "mongo/bson/timestamp.h"
+#include "mongo/client/connection_string.h"
+#include "mongo/db/common_request_args_gen.h"
+#include "mongo/db/database_name.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/repl/member_config.h"
 #include "mongo/db/repl/member_data.h"
 #include "mongo/db/repl/member_state.h"
+#include "mongo/db/repl/optime.h"
+#include "mongo/db/repl/repl_set_config.h"
+#include "mongo/db/repl/repl_set_heartbeat_response.h"
 #include "mongo/db/repl/repl_settings.h"
+#include "mongo/db/repl/replication_coordinator_fwd.h"
 #include "mongo/db/repl/split_horizon.h"
 #include "mongo/db/repl/split_prepare_session_manager.h"
 #include "mongo/db/repl/sync_source_selector.h"
+#include "mongo/db/service_context.h"
+#include "mongo/db/storage/storage_engine.h"
 #include "mongo/db/storage/storage_engine_init.h"
+#include "mongo/db/write_concern_options.h"
 #include "mongo/executor/task_executor.h"
+#include "mongo/platform/compiler.h"
 #include "mongo/rpc/topology_version_gen.h"
 #include "mongo/util/duration.h"
+#include "mongo/util/future.h"
+#include "mongo/util/interruptible.h"
 #include "mongo/util/net/hostandport.h"
 #include "mongo/util/time_support.h"
+#include "mongo/util/uuid.h"
 
 namespace mongo {
 
 class BSONObj;
 class BSONObjBuilder;
+
 class CommitQuorumOptions;
 class IndexDescriptor;
 class NamespaceString;
@@ -75,13 +102,17 @@ class ReplSetMetadata;
 namespace repl {
 
 class BackgroundSync;
+
 class HelloResponse;
 class OpTime;
 class OpTimeAndWallTime;
+
 class ReadConcernArgs;
 class ReplSetConfig;
+
 class ReplSetHeartbeatArgsV1;
 class ReplSetHeartbeatResponse;
+
 class ReplSetRequestVotesArgs;
 class ReplSetRequestVotesResponse;
 class UpdatePositionArgs;
@@ -111,7 +142,7 @@ public:
         StatusAndDuration(const Status& stat, Milliseconds ms) : status(stat), duration(ms) {}
     };
 
-    virtual ~ReplicationCoordinator();
+    ~ReplicationCoordinator() override;
 
     /**
      * Does any initial bookkeeping needed to start replication, and instructs the other
@@ -161,22 +192,9 @@ public:
      */
     virtual const ReplSettings& getSettings() const = 0;
 
-    enum Mode { modeNone = 0, modeReplSet };
-
-    /**
-     * Returns a value indicating whether this node was configured at start-up to run standalone or
-     * as a member of a replica set.
-     */
-    virtual Mode getReplicationMode() const = 0;
-
-    /**
-     * Returns true if this node is configured to be a member of a replica set.
-     */
-    virtual bool isReplEnabled() const = 0;
-
     /**
      * Returns the current replica set state of this node (PRIMARY, SECONDARY, STARTUP, etc).
-     * It is invalid to call this unless getReplicationMode() == modeReplSet.
+     * It is invalid to call this unless getSettings().isReplSet() returns true.
      */
     virtual MemberState getMemberState() const = 0;
 
@@ -199,7 +217,7 @@ public:
     /**
      * Returns true if this node is in state PRIMARY or SECONDARY.
      *
-     * It is invalid to call this unless getReplicationMode() == modeReplSet.
+     * It is invalid to call this unless getSettings().isReplSet() returns true.
      *
      * This method may be optimized to reduce synchronization overhead compared to
      * reading the current member state with getMemberState().
@@ -272,13 +290,15 @@ public:
      * NOTE: This function can only be meaningfully called while the caller holds the
      * ReplicationStateTransitionLock in some mode other than MODE_NONE.
      */
-    virtual bool canAcceptWritesForDatabase(OperationContext* opCtx, StringData dbName) = 0;
+    virtual bool canAcceptWritesForDatabase(OperationContext* opCtx,
+                                            const DatabaseName& dbName) = 0;
 
     /**
      * Version which does not check for the RSTL.  Do not use in new code. Without the RSTL, the
      * return value may be inaccurate by the time the function returns.
      */
-    virtual bool canAcceptWritesForDatabase_UNSAFE(OperationContext* opCtx, StringData dbName) = 0;
+    virtual bool canAcceptWritesForDatabase_UNSAFE(OperationContext* opCtx,
+                                                   const DatabaseName& dbName) = 0;
 
     /**
      * Returns true if it is valid for this node to accept writes on the given namespace.
@@ -351,38 +371,19 @@ public:
                                              const NamespaceString& ns) = 0;
 
     /**
-     * Updates our internal tracking of the last OpTime applied to this node. Also updates our
-     * internal tracking of the wall clock time corresponding to that operation.
-     *
-     * The new value of "opTime" must be no less than any prior value passed to this method, and
-     * it is the caller's job to properly synchronize this behavior.  The exception to this rule
-     * is that after calls to resetLastOpTimesFromOplog(), the minimum acceptable value for
-     * "opTime" is reset based on the contents of the oplog, and may go backwards due to
-     * rollback. Additionally, the optime given MUST represent a consistent database state.
+     * Updates our internal tracking of the last OpTime that an oplog entry is written into memory
+     * on this node if the supplied optime is later than the current lastWritten OpTime.
      */
-    virtual void setMyLastAppliedOpTimeAndWallTime(const OpTimeAndWallTime& opTimeAndWallTime) = 0;
+    virtual void setMyLastWrittenOpTimeAndWallTimeForward(
+        const OpTimeAndWallTime& opTimeAndWallTime) = 0;
 
     /**
-     * Updates our internal tracking of the last OpTime durable to this node. Also updates our
-     * internal tracking of the wall clock time corresponding to that operation.
-     *
-     * The new value of "opTime" must be no less than any prior value passed to this method, and
-     * it is the caller's job to properly synchronize this behavior.  The exception to this rule
-     * is that after calls to resetLastOpTimesFromOplog(), the minimum acceptable value for
-     * "opTime" is reset based on the contents of the oplog, and may go backwards due to
-     * rollback.
-     */
-    virtual void setMyLastDurableOpTimeAndWallTime(const OpTimeAndWallTime& opTimeAndWallTime) = 0;
-
-    /**
-     * Updates our internal tracking of the last OpTime applied to this node, but only
-     * if the supplied optime is later than the current last OpTime known to the replication
+     * Updates our internal tracking of the last OpTime applied to this node, but only if the
+     * supplied optime is later than the current lastApplied OpTime known to the replication
      * coordinator.
      *
-     * This function is used by logOp() on a primary, since the ops in the oplog do not
-     * necessarily commit in sequential order. It is also used when we finish oplog batch
-     * application on secondaries, to avoid any potential race conditions around setting the
-     * applied optime from more than one thread.
+     * This function is only used on secondary and the caller needs to make sure the input opTime is
+     * not greater than the current lastWritten timestamp.
      *
      * Since the last applied op time and wall time might not be visible (i.e. there may be
      * "oplog holes" from oplog entries with earlier timestamps which commit after this one)
@@ -393,16 +394,33 @@ public:
         const OpTimeAndWallTime& opTimeAndWallTime) = 0;
 
     /**
-     * Updates our internal tracking of the last OpTime durable to this node, but only
-     * if the supplied optime is later than the current last OpTime known to the replication
+     * Updates our internal tracking of the last OpTime durable to this node, but only if the
+     * supplied optime is later than the current lastDurable OpTime known to the replication
      * coordinator. Also updates the wall clock time corresponding to that operation.
      *
-     * This function is used by logOp() on a primary, since the ops in the oplog do not
-     * necessarily commit in sequential order.
+     * This function is used by onDurable() hook on secondary, while primary should use
+     * setMyLastDurableAndLastWrittenOpTimeAndWallTimeForward() instead. The caller needs to make
+     * sure the input opTime is not greater than the current lastWritten timestamp.
      */
     virtual void setMyLastDurableOpTimeAndWallTimeForward(
         const OpTimeAndWallTime& opTimeAndWallTime) = 0;
-    // virtual void setMyLastDurableOpTimeForward(const OpTimeAndWallTime& opTimeAndWallTime) = 0;
+
+    /**
+     * Update lastApplied and lastWritten under the same lock guard. This is used by the oplog
+     * onCommit() hook on the primary.
+     */
+    virtual void setMyLastAppliedAndLastWrittenOpTimeAndWallTimeForward(
+        const OpTimeAndWallTime& opTimeAndWallTime) = 0;
+
+    /**
+     * Update lastDurable and lastWritten under the same lock guard. This should only be used on
+     * primary due to an edge case that the onCommit() hook is stuck at somewhere and lastWritten
+     * has not been updated yet. On secondary, we always write oplog into in-memory collection first
+     * before journal flush, which means that lastWritten should already be updated, so we should
+     * use setMyLastDurableOpTimeAndWallTimeForward() instead.
+     */
+    virtual void setMyLastDurableAndLastWrittenOpTimeAndWallTimeForward(
+        const OpTimeAndWallTime& opTimeAndWallTime) = 0;
 
     /**
      * Same as above, but used during places we need to zero our last optime.
@@ -414,6 +432,22 @@ public:
      */
     virtual void setMyHeartbeatMessage(const std::string& msg) = 0;
 
+    /**
+     * Returns the last optime recorded by setMyLastWrittenOpTimeAndWallTimeForward.
+     */
+    virtual OpTime getMyLastWrittenOpTime() const = 0;
+
+    /*
+     * Returns the same as getMyLastWrittenOpTime() and additionally returns the wall clock time
+     * corresponding to that OpTime.
+     *
+     * When rollbackSafe is true, this returns an empty OpTimeAndWallTime if the node is in ROLLBACK
+     * state. The lastWrittenOpTime during ROLLBACK might be temporarily pointing to an oplog entry
+     * in the divergent branch of history which would become invalid after the rollback finishes.
+     */
+    virtual OpTimeAndWallTime getMyLastWrittenOpTimeAndWallTime(
+        bool rollbackSafe = false) const = 0;
+
     /**
      * Returns the last optime recorded by setMyLastAppliedOpTime.
      */
@@ -422,13 +456,8 @@ public:
     /*
      * Returns the same as getMyLastAppliedOpTime() and additionally returns the wall clock time
      * corresponding to that OpTime.
-     *
-     * When rollbackSafe is true, this returns an empty OpTimeAndWallTime if the node is in ROLLBACK
-     * state. The lastAppliedOpTime during ROLLBACK might be temporarily pointing to an oplog entry
-     * in the divergent branch of history which would become invalid after the rollback finishes.
      */
-    virtual OpTimeAndWallTime getMyLastAppliedOpTimeAndWallTime(
-        bool rollbackSafe = false) const = 0;
+    virtual OpTimeAndWallTime getMyLastAppliedOpTimeAndWallTime() const = 0;
 
     /**
      * Returns the last optime recorded by setMyLastDurableOpTime.
@@ -465,6 +494,9 @@ public:
     virtual Status waitUntilOpTimeForReadUntil(OperationContext* opCtx,
                                                const ReadConcernArgs& settings,
                                                boost::optional<Date_t> deadline) = 0;
+    virtual Status waitUntilOpTimeWrittenUntil(OperationContext* opCtx,
+                                               LogicalTime clusterTime,
+                                               boost::optional<Date_t> deadline) = 0;
 
     /**
      * Waits until the timestamp of this node's lastCommittedOpTime is >= the given timestamp.
@@ -529,78 +561,109 @@ public:
      * Step-up
      * =======
      * On stepup, repl coord enters catch-up mode. It's the same as the secondary mode from
-     * the perspective of producer and applier, so there's nothing to do with them.
-     * When a node enters drain mode, producer state = Stopped, applier state = Draining.
+     * the perspective of producer and writer/applier, so there's nothing to do with them.
      *
-     * If the applier state is Draining, it will signal repl coord when there's nothing to apply.
-     * The applier goes into Stopped state at the same time.
+     * When entering drain mode, producer state = Stopped, writer state = Draining, applier
+     * state = Running.
+     *
+     * Once writer is done writing everything, applier state = Draining. Applier will later
+     * signal repl coord when there's nothing to apply. After that both writer and applier
+     * go to the Stopped state.
      *
      * The states go like the following:
-     * - secondary and during catchup mode
-     * (producer: Running, applier: Running)
+     *
+     * - Secondary and during catchup mode:
+     * (producer: Running, writer: Running, applier: Running)
      *      |
-     *      | finish catch-up, enter drain mode
+     *      | finishes catch-up, writer enters drain mode
      *      V
-     * - drain mode
-     * (producer: Stopped, applier: Draining)
+     * - Writer drain mode:
+     * (producer: Stopped, writer: Draining, applier: Running)
      *      |
-     *      | applier signals drain is complete
+     *      | writer signals drain complete, applier enters drain mode
+     *      V
+     * - Applier drain mode:
+     * (producer: Stopped, writer: Draining, applier: Draining)
+     *      |
+     *      | applier signals drain complete
      *      V
      * - primary is in writable mode
-     * (producer: Stopped, applier: Stopped)
+     * (producer: Stopped, writer: Stopped, applier: Stopped)
      *
      *
      * Step-down
      * =========
      * The state transitions become:
-     * - primary is in writable mode
-     * (producer: Stopped, applier: Stopped)
+     *
+     * - Primary is in writable mode:
+     * (producer: Stopped, writer: Stopped, applier: Stopped)
      *      |
-     *      | step down
+     *      | steps down
      *      V
-     * - secondary mode, starting bgsync
-     * (producer: Starting, applier: Running)
+     * - Secondary mode, starting bgsync:
+     * (producer: Starting, writer: Running, applier: Running)
      *      |
      *      | bgsync runs start()
      *      V
-     * - secondary mode, normal
-     * (producer: Running, applier: Running)
+     * - Secondary mode, normal:
+     * (producer: Running, Writer: Running, applier: Running)
      *
      * When a node steps down during draining mode, it's OK to change from (producer: Stopped,
-     * applier: Draining) to (producer: Starting, applier: Running).
-     *
-     * When a node steps down during catchup mode, the states remain the same (producer: Running,
+     * writer: Draining, applier: Running/Draining) to (producer: Starting, writer: Running,
      * applier: Running).
      *
-     * DrainingForShardSplit follows the same state diagram as Draining, it only exists to hint the
-     * signalDrainModeComplete method that it should not follow the primary step-up logic.
+     * When a node steps down during catchup mode, the states remain the same (producer: Running,
+     * writer: Running, applier: Running).
+     *
+     * WriterDrainingForShardSplit and ApplierDrainingForShardSplit follow the same state diagram
+     * as Draining, they only exist to hint the signalApplierDrainComplete method that it should
+     * not follow the primary step-up logic.
      */
-    enum class ApplierState { Running, Draining, DrainingForShardSplit, Stopped };
+    enum class OplogSyncState {
+        Running,
+        WriterDraining,
+        WriterDrainingForShardSplit,
+        ApplierDraining,
+        ApplierDrainingForShardSplit,
+        Stopped,
+    };
 
     /**
-     * In normal cases: Running -> Draining -> Stopped -> Running.
-     * Draining -> Running is also possible if a node steps down during drain mode.
+     * In normal cases: Running -> WriterDraining -> ApplierDraining -> Stopped -> Running.
+     * WriterDraining/ApplierDraining -> Running is also possible if the node steps down
+     * during drain mode.
      *
-     * Only the applier can make the transition from Draining to Stopped by calling
-     * signalDrainComplete().
+     * Only the applier can make the transition from ApplierDraining to Stopped by calling
+     * signalApplierDrainComplete().
      */
-    virtual ApplierState getApplierState() = 0;
+    virtual OplogSyncState getOplogSyncState() = 0;
+
+    /**
+     * Signals that a previously requested pause and drain of the writer buffer
+     * has completed.
+     *
+     * This is an interface that allows the writer to notify the applier buffer
+     * to start draining.
+     */
+    virtual void signalWriterDrainComplete(OperationContext* opCtx,
+                                           long long termWhenExhausted) = 0;
 
     /**
      * Signals that a previously requested pause and drain of the applier buffer
      * has completed.
      *
-     * This is an interface that allows the applier to reenable writes after
-     * a successful election triggers the draining of the applier buffer.
+     * This is an interface that allows the applier to reenable writes after a
+     * successful election triggers the draining of the applier buffer.
      *
-     * The applier signals drain complete when the buffer is empty and it's in Draining
+     * The applier signals drain complete when its buffer is empty and it is in Draining
      * state. We need to make sure the applier checks both conditions in the same term.
      * Otherwise, it's possible that the applier confirms the empty buffer, but the node
-     * steps down and steps up so quickly that the applier signals drain complete in the wrong
-     * term.
+     * steps down and steps up so quickly that the applier signals drain complete in the
+     * wrong term.
      */
-    virtual void signalDrainComplete(OperationContext* opCtx,
-                                     long long termWhenBufferIsEmpty) noexcept = 0;
+    virtual void signalApplierDrainComplete(OperationContext* opCtx,
+                                            long long termWhenExhausted) noexcept = 0;
+
 
     /**
      * Signals the sync source feedback thread to wake up and send a handshake and
@@ -697,10 +760,16 @@ public:
     virtual Status validateWriteConcern(const WriteConcernOptions& writeConcern) const = 0;
 
     /**
-     * Returns a pointer to the MemberConfig corresponding to the member with the given
-     * HostAndPort in the current ReplSetConfig, or NULL if there is no member with that address.
+     * Returns a copy of the MemberConfig corresponding to the member with the given
+     * HostAndPort in the current ReplSetConfig, or boost::none if there is no member with that
+     * address.
+     *
+     * This is deprecated because the previous version of this method returned a pointer to an
+     * internal structure that could change at any time, and getting member information is
+     * inherently racy; member configuration can change at any time.
      */
-    virtual const MemberConfig* findConfigMemberByHostAndPort(const HostAndPort& hap) const = 0;
+    virtual boost::optional<MemberConfig> findConfigMemberByHostAndPort_deprecated(
+        const HostAndPort& hap) const = 0;
 
     /**
      * Returns whether all members of the current ReplSetConfig set have hostname localhost.
@@ -930,7 +999,7 @@ public:
      * Prepares a metadata object with the ReplSetMetadata and the OplogQueryMetadata depending
      * on what has been requested.
      */
-    virtual void prepareReplMetadata(const BSONObj& metadataRequestObj,
+    virtual void prepareReplMetadata(const CommonRequestArgs& requestArgs,
                                      const OpTime& lastOpTimeFromClient,
                                      BSONObjBuilder* builder) const = 0;
 
@@ -980,8 +1049,9 @@ public:
 
     /**
      * Appends diagnostics about the replication subsystem.
+     * Places it under a subobject called `leafName`.
      */
-    virtual void appendDiagnosticBSON(BSONObjBuilder* bob) = 0;
+    virtual void appendDiagnosticBSON(BSONObjBuilder* bob, StringData leafName) = 0;
 
     /**
      * Appends connection information to the provided BSONObjBuilder.
@@ -1026,11 +1096,6 @@ public:
      */
     virtual void incrementNumCatchUpOpsIfCatchingUp(long numOps) = 0;
 
-    /**
-     * Signals that drop pending collections have been removed from storage.
-     */
-    virtual void signalDropPendingCollectionsRemovedFromStorage() = 0;
-
     /**
      * Returns true if logOp() should not append an entry to the oplog for this operation.
      */
@@ -1154,7 +1219,7 @@ public:
      * - 'OnRemoteCmdComplete' will be called once the response for the remote command is received.
      */
     virtual BSONObj runCmdOnPrimaryAndAwaitResponse(OperationContext* opCtx,
-                                                    const std::string& dbName,
+                                                    const DatabaseName& dbName,
                                                     const BSONObj& cmdObj,
                                                     OnRemoteCmdScheduledFn onRemoteCmdScheduled,
                                                     OnRemoteCmdCompleteFn onRemoteCmdComplete) = 0;
@@ -1205,6 +1270,22 @@ public:
      */
     virtual bool isRetryableWrite(OperationContext* opCtx) const = 0;
 
+    /**
+     * Returns the in-memory initialSyncId from last initial sync. boost::none will be returned if
+     * there is no initial sync.
+     */
+    virtual boost::optional<UUID> getInitialSyncId(OperationContext* opCtx) = 0;
+
+    /*
+     * Clear this node's sync source.
+     */
+    virtual void clearSyncSource() = 0;
+
+    /**
+     * Returns true if the node undergoes initial sync or rollback.
+     */
+    bool isDataRecovering() const;
+
 protected:
     ReplicationCoordinator();
 };
