@@ -27,15 +27,26 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
 #include "replication_coordinator_noop.h"
 
+#include <boost/move/utility_core.hpp>
+
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/util/assert_util.h"
+
 namespace mongo {
 namespace repl {
 
+ReplSettings makeDefaultReplSettings() {
+    ReplSettings settings;
+    settings.setReplSetString("ReplicationCoordinatorNoOpDefaultSetName");
+    return settings;
+}
+
 ReplicationCoordinatorNoOp::ReplicationCoordinatorNoOp(ServiceContext* service)
-    : _service(service) {}
+    : _service(service), _settings(makeDefaultReplSettings()) {}
 
 void ReplicationCoordinatorNoOp::startup(OperationContext* opCtx,
                                          StorageEngine::LastShutdownState lastShutdownState) {}
@@ -53,14 +64,6 @@ bool ReplicationCoordinatorNoOp::inQuiesceMode() const {
 void ReplicationCoordinatorNoOp::shutdown(OperationContext* opCtx,
                                           BSONObjBuilder* shutdownTimeElapsedBuilder) {}
 
-ReplicationCoordinator::Mode ReplicationCoordinatorNoOp::getReplicationMode() const {
-    return modeReplSet;
-}
-
-bool ReplicationCoordinatorNoOp::isReplEnabled() const {
-    return getReplicationMode() == modeReplSet;
-}
-
 MemberState ReplicationCoordinatorNoOp::getMemberState() const {
     return MemberState::RS_PRIMARY;
 }
@@ -70,7 +73,7 @@ OpTime ReplicationCoordinatorNoOp::getMyLastAppliedOpTime() const {
 }
 
 const ReplSettings& ReplicationCoordinatorNoOp::getSettings() const {
-    MONGO_UNREACHABLE;
+    return _settings;
 }
 
 bool ReplicationCoordinatorNoOp::isWritablePrimaryForReportingPurposes() {
@@ -78,12 +81,12 @@ bool ReplicationCoordinatorNoOp::isWritablePrimaryForReportingPurposes() {
 }
 
 bool ReplicationCoordinatorNoOp::canAcceptWritesForDatabase(OperationContext* opCtx,
-                                                            StringData dbName) {
+                                                            const DatabaseName& dbName) {
     MONGO_UNREACHABLE;
 }
 
 bool ReplicationCoordinatorNoOp::canAcceptWritesForDatabase_UNSAFE(OperationContext* opCtx,
-                                                                   StringData dbName) {
+                                                                   const DatabaseName& dbName) {
     MONGO_UNREACHABLE;
 }
 
@@ -134,8 +137,7 @@ bool ReplicationCoordinatorNoOp::buildsIndexes() {
     MONGO_UNREACHABLE;
 }
 
-OpTimeAndWallTime ReplicationCoordinatorNoOp::getMyLastAppliedOpTimeAndWallTime(
-    bool rollbackSafe) const {
+OpTimeAndWallTime ReplicationCoordinatorNoOp::getMyLastAppliedOpTimeAndWallTime() const {
     MONGO_UNREACHABLE;
 }
 
@@ -148,7 +150,7 @@ OpTime ReplicationCoordinatorNoOp::getCurrentCommittedSnapshotOpTime() const {
     MONGO_UNREACHABLE;
 }
 
-void ReplicationCoordinatorNoOp::appendDiagnosticBSON(mongo::BSONObjBuilder*) {
+void ReplicationCoordinatorNoOp::appendDiagnosticBSON(mongo::BSONObjBuilder*, StringData) {
     MONGO_UNREACHABLE;
 }
 
@@ -184,11 +186,15 @@ Status ReplicationCoordinatorNoOp::setFollowerModeRollback(OperationContext* opC
     MONGO_UNREACHABLE;
 }
 
-ReplicationCoordinator::ApplierState ReplicationCoordinatorNoOp::getApplierState() {
+ReplicationCoordinator::OplogSyncState ReplicationCoordinatorNoOp::getOplogSyncState() {
     MONGO_UNREACHABLE;
 }
 
-void ReplicationCoordinatorNoOp::signalDrainComplete(OperationContext*, long long) noexcept {
+void ReplicationCoordinatorNoOp::signalWriterDrainComplete(OperationContext*, long long) noexcept {
+    MONGO_UNREACHABLE;
+}
+
+void ReplicationCoordinatorNoOp::signalApplierDrainComplete(OperationContext*, long long) noexcept {
     MONGO_UNREACHABLE;
 }
 
@@ -200,6 +206,11 @@ void ReplicationCoordinatorNoOp::setMyHeartbeatMessage(const std::string&) {
     MONGO_UNREACHABLE;
 }
 
+void ReplicationCoordinatorNoOp::setMyLastWrittenOpTimeAndWallTimeForward(
+    const OpTimeAndWallTime&) {
+    MONGO_UNREACHABLE;
+}
+
 void ReplicationCoordinatorNoOp::setMyLastAppliedOpTimeAndWallTimeForward(
     const OpTimeAndWallTime&) {
     MONGO_UNREACHABLE;
@@ -210,11 +221,13 @@ void ReplicationCoordinatorNoOp::setMyLastDurableOpTimeAndWallTimeForward(
     MONGO_UNREACHABLE;
 }
 
-void ReplicationCoordinatorNoOp::setMyLastAppliedOpTimeAndWallTime(const OpTimeAndWallTime&) {
+void ReplicationCoordinatorNoOp::setMyLastAppliedAndLastWrittenOpTimeAndWallTimeForward(
+    const OpTimeAndWallTime&) {
     MONGO_UNREACHABLE;
 }
 
-void ReplicationCoordinatorNoOp::setMyLastDurableOpTimeAndWallTime(const OpTimeAndWallTime&) {
+void ReplicationCoordinatorNoOp::setMyLastDurableAndLastWrittenOpTimeAndWallTimeForward(
+    const OpTimeAndWallTime&) {
     MONGO_UNREACHABLE;
 }
 
@@ -247,6 +260,12 @@ Status ReplicationCoordinatorNoOp::waitUntilOpTimeForReadUntil(OperationContext*
     MONGO_UNREACHABLE;
 }
 
+Status ReplicationCoordinatorNoOp::waitUntilOpTimeWrittenUntil(OperationContext*,
+                                                               LogicalTime,
+                                                               boost::optional<Date_t>) {
+    MONGO_UNREACHABLE;
+}
+
 Status ReplicationCoordinatorNoOp::awaitTimestampCommitted(OperationContext* opCtx, Timestamp ts) {
     MONGO_UNREACHABLE;
 }
@@ -338,7 +357,7 @@ BSONObj ReplicationCoordinatorNoOp::getConfigBSON() const {
     MONGO_UNREACHABLE;
 }
 
-const MemberConfig* ReplicationCoordinatorNoOp::findConfigMemberByHostAndPort(
+boost::optional<MemberConfig> ReplicationCoordinatorNoOp::findConfigMemberByHostAndPort_deprecated(
     const HostAndPort& hap) const {
     MONGO_UNREACHABLE;
 }
@@ -496,7 +515,7 @@ Status ReplicationCoordinatorNoOp::processReplSetRequestVotes(OperationContext*,
     MONGO_UNREACHABLE;
 }
 
-void ReplicationCoordinatorNoOp::prepareReplMetadata(const BSONObj&,
+void ReplicationCoordinatorNoOp::prepareReplMetadata(const CommonRequestArgs&,
                                                      const OpTime&,
                                                      BSONObjBuilder*) const {
     MONGO_UNREACHABLE;
@@ -535,10 +554,6 @@ Status ReplicationCoordinatorNoOp::stepUpIfEligible(bool skipDryRun) {
     MONGO_UNREACHABLE;
 }
 
-void ReplicationCoordinatorNoOp::signalDropPendingCollectionsRemovedFromStorage() {
-    MONGO_UNREACHABLE;
-}
-
 boost::optional<Timestamp> ReplicationCoordinatorNoOp::getRecoveryTimestamp() {
     MONGO_UNREACHABLE;
 }
@@ -602,7 +617,7 @@ void ReplicationCoordinatorNoOp::cancelCbkHandle(
 
 BSONObj ReplicationCoordinatorNoOp::runCmdOnPrimaryAndAwaitResponse(
     OperationContext* opCtx,
-    const std::string& dbName,
+    const DatabaseName& dbName,
     const BSONObj& cmdObj,
     OnRemoteCmdScheduledFn onRemoteCmdScheduled,
     OnRemoteCmdCompleteFn onRemoteCmdComplete) {
@@ -630,5 +645,22 @@ bool ReplicationCoordinatorNoOp::isRetryableWrite(OperationContext* opCtx) const
     MONGO_UNREACHABLE;
 }
 
+boost::optional<UUID> ReplicationCoordinatorNoOp::getInitialSyncId(OperationContext* opCtx) {
+    MONGO_UNREACHABLE;
+}
+
+OpTime ReplicationCoordinatorNoOp::getMyLastWrittenOpTime() const {
+    MONGO_UNREACHABLE;
+}
+
+OpTimeAndWallTime ReplicationCoordinatorNoOp::getMyLastWrittenOpTimeAndWallTime(
+    bool rollbackSafe) const {
+    MONGO_UNREACHABLE;
+}
+
+void ReplicationCoordinatorNoOp::clearSyncSource() {
+    MONGO_UNREACHABLE;
+}
+
 }  // namespace repl
 }  // namespace mongo
