@@ -28,36 +28,83 @@
  */
 
 
-#include "mongo/platform/basic.h"
+#include <algorithm>
+#include <boost/cstdint.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <cstddef>
+#include <cstdint>
+#include <fmt/format.h>
+#include <iterator>
+#include <limits>
+#include <memory>
+#include <mutex>
+#include <set>
+#include <string>
+#include <vector>
 
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/checked_cast.h"
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/json.h"
+#include "mongo/bson/simple_bsonobj_comparator.h"
+#include "mongo/db/catalog/collection_options.h"
 #include "mongo/db/client.h"
 #include "mongo/db/db_raii.h"
+#include "mongo/db/exec/document_value/value.h"
 #include "mongo/db/namespace_string.h"
+#include "mongo/db/op_observer/op_observer.h"
 #include "mongo/db/op_observer/op_observer_noop.h"
 #include "mongo/db/op_observer/op_observer_registry.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/repl/drop_pending_collection_reaper.h"
+#include "mongo/db/repl/member_state.h"
+#include "mongo/db/repl/oplog.h"
 #include "mongo/db/repl/oplog_applier_impl_test_fixture.h"
 #include "mongo/db/repl/oplog_entry.h"
-#include "mongo/db/repl/oplog_interface_local.h"
+#include "mongo/db/repl/oplog_entry_gen.h"
+#include "mongo/db/repl/replication_consistency_markers.h"
 #include "mongo/db/repl/replication_consistency_markers_mock.h"
+#include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/repl/replication_coordinator_mock.h"
 #include "mongo/db/repl/replication_recovery.h"
+#include "mongo/db/repl/storage_interface.h"
 #include "mongo/db/repl/storage_interface_impl.h"
+#include "mongo/db/server_options.h"
+#include "mongo/db/service_context.h"
 #include "mongo/db/service_context_d_test_fixture.h"
+#include "mongo/db/session/logical_session_id.h"
+#include "mongo/db/session/logical_session_id_gen.h"
 #include "mongo/db/session/session_catalog_mongod.h"
 #include "mongo/db/session/session_txn_record_gen.h"
+#include "mongo/db/shard_id.h"
 #include "mongo/db/storage/durable_history_pin.h"
-#include "mongo/db/storage/storage_options.h"
 #include "mongo/db/storage/storage_parameters_gen.h"
 #include "mongo/db/transaction/session_catalog_mongod_transaction_interface_impl.h"
 #include "mongo/db/transaction/transaction_participant.h"
+#include "mongo/db/update/document_diff_serialization.h"
 #include "mongo/db/update/update_oplog_entry_serialization.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/log_severity.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/platform/mutex.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/bson_test_util.h"
 #include "mongo/unittest/death_test.h"
+#include "mongo/unittest/framework.h"
 #include "mongo/unittest/log_test.h"
-#include "mongo/unittest/unittest.h"
 #include "mongo/util/assert_util.h"
-#include "mongo/util/str.h"
+#include "mongo/util/duration.h"
+#include "mongo/util/fail_point.h"
+#include "mongo/util/scopeguard.h"
+#include "mongo/util/time_support.h"
+#include "mongo/util/uuid.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
 
@@ -67,7 +114,7 @@ namespace repl {
 namespace {
 
 const auto& oplogNs = NamespaceString::kRsOplogNamespace;
-const NamespaceString testNs("a.a");
+const NamespaceString testNs = NamespaceString::createNamespaceString_forTest("a.a");
 
 class StorageInterfaceRecovery : public StorageInterfaceImpl {
 public:
@@ -111,6 +158,17 @@ public:
         return _pointInTimeReadTimestamp;
     }
 
+    void setInitialDataTimestamp(ServiceContext* serviceCtx, Timestamp snapshotName) override {
+        stdx::lock_guard<Latch> lock(_mutex);
+        _initialDataTimestamp = snapshotName;
+    }
+
+    Timestamp getInitialDataTimestamp(ServiceContext* serviceCtx) const override {
+        stdx::lock_guard<Latch> lock(_mutex);
+        return _initialDataTimestamp;
+    };
+
+
 private:
     mutable Mutex _mutex = MONGO_MAKE_LATCH("StorageInterfaceRecovery::_mutex");
     Timestamp _initialDataTimestamp = Timestamp::min();
@@ -122,12 +180,12 @@ private:
 
 class ReplicationRecoveryTestObObserver : public OpObserverNoop {
 public:
-    using OpObserver::onDropCollection;
     repl::OpTime onDropCollection(OperationContext* opCtx,
                                   const NamespaceString& collectionName,
                                   const UUID& uuid,
                                   std::uint64_t numRecords,
-                                  const CollectionDropType dropType) override {
+                                  const CollectionDropType dropType,
+                                  bool markFromMigrate) override {
         // If the oplog is not disabled for this namespace, then we need to reserve an op time for
         // the drop.
         if (!repl::ReplicationCoordinator::get(opCtx)->isOplogDisabledFor(opCtx, collectionName)) {
@@ -136,17 +194,6 @@ public:
         return {};
     }
 
-    /**
-     * Called when we prepare a multi-doc transaction using the TransactionParticipant.
-     */
-    std::unique_ptr<ApplyOpsOplogSlotAndOperationAssignment> preTransactionPrepare(
-        OperationContext* opCtx,
-        const std::vector<OplogSlot>& reservedSlots,
-        const TransactionOperations& transactionOperations,
-        Date_t wallClockTime) override {
-        return std::make_unique<ApplyOpsOplogSlotAndOperationAssignment>(/*prepare=*/false);
-    }
-
     const repl::OpTime dropOpTime = {Timestamp(Seconds(100), 1U), 1LL};
 };
 
@@ -303,7 +350,7 @@ repl::OplogEntry _makeTransactionOplogEntry(repl::OpTime opTime,
     builder.append("t", opTime.getTerm());
     builder.append("v", repl::OplogEntry::kOplogVersion);
     builder.append("op", "c");
-    builder.append("ns", testNs.toString());
+    builder.append("ns", testNs.toString_forTest());
     builder.append("o", object);
     builder.append("wall", wallTime);
     builder.append("stmtId", stmtId);
@@ -1135,7 +1182,7 @@ TEST_F(ReplicationRecoveryTest, CommitTransactionOplogEntryCorrectlyUpdatesConfi
 
     const auto txnOperations = BSON_ARRAY(BSON("op"
                                                << "i"
-                                               << "ns" << testNs.toString() << "o"
+                                               << "ns" << testNs.toString_forTest() << "o"
                                                << BSON("_id" << 1)));
     const auto prepareDate = Date_t::now();
     const auto prepareOp =
@@ -1210,7 +1257,7 @@ TEST_F(ReplicationRecoveryTest,
 
     const auto txnOperations = BSON_ARRAY(BSON("op"
                                                << "i"
-                                               << "ns" << testNs.toString() << "o"
+                                               << "ns" << testNs.toString_forTest() << "o"
                                                << BSON("_id" << 1)));
     const auto prepareDate = Date_t::now();
     const auto prepareOp =
@@ -1690,6 +1737,96 @@ TEST_F(ReplicationRecoveryTest, StartupRecoveryRunsCompletionHook) {
     ASSERT_EQ(getConsistencyMarkers()->getOplogTruncateAfterPoint(opCtx), Timestamp());
 }
 
+TEST_F(ReplicationRecoveryTest, TruncateOplogToTimestamp) {
+    ReplicationRecoveryImpl recovery(getStorageInterface(), getConsistencyMarkers());
+    auto opCtx = getOperationContext();
+
+    _setUpOplog(opCtx, getStorageInterface(), {1, 2, 3, 4, 5});
+    // We should truncate the oplog to Timestamp(3, 3) without ever setting the truncate after
+    // point. The truncate after point should remain as the default value.
+    ASSERT_EQUALS(Timestamp(), getConsistencyMarkers()->getOplogTruncateAfterPoint(opCtx));
+    recovery.truncateOplogToTimestamp(opCtx, Timestamp(3, 3));
+
+    ASSERT_EQUALS(Timestamp(), getConsistencyMarkers()->getOplogTruncateAfterPoint(opCtx));
+    _assertDocsInOplog(opCtx, {1, 2, 3});
+    _assertDocsInTestCollection(opCtx, {});
+}
+
+DEATH_TEST_REGEX_F(ReplicationRecoveryTest,
+                   TruncateOplogToTimestampOplogDoesntExist,
+                   "Fatal assertion.*34418") {
+    ReplicationRecoveryImpl recovery(getStorageInterface(), getConsistencyMarkers());
+    auto opCtx = getOperationContext();
+    ASSERT_OK(getStorageInterface()->dropCollection(opCtx, NamespaceString::kRsOplogNamespace));
+
+    // The truncate after point should remain as the default value.
+    ASSERT_EQUALS(Timestamp(), getConsistencyMarkers()->getOplogTruncateAfterPoint(opCtx));
+    // Without an oplog, the 'truncateOplogToTimestamp' function should hit a fatal assertion.
+    recovery.truncateOplogToTimestamp(opCtx, Timestamp(3, 3));
+}
+
+
+TEST_F(ReplicationRecoveryTest, ApplyOplogEntriesForRestore) {
+    storageGlobalParams.magicRestore = true;
+    auto opCtx = getOperationContext();
+    getStorageInterface()->setInitialDataTimestamp(opCtx->getServiceContext(),
+                                                   Timestamp::kAllowUnstableCheckpointsSentinel);
+    getStorageInterfaceRecovery()->setRecoveryTimestamp(Timestamp(1, 1));
+    ReplicationRecoveryImpl recovery(getStorageInterface(), getConsistencyMarkers());
+    _setUpOplog(opCtx, getStorageInterface(), {1, 2, 3});
+
+    recovery.applyOplogEntriesForRestore(opCtx, Timestamp(1, 1));
+    ASSERT_EQ(getConsistencyMarkers()->getAppliedThrough(opCtx), OpTime());
+    ASSERT_EQ(getStorageInterface()->getInitialDataTimestamp(opCtx->getServiceContext()),
+              Timestamp::kAllowUnstableCheckpointsSentinel);
+}
+
+DEATH_TEST_REGEX_F(ReplicationRecoveryTest,
+                   ApplyOplogEntriesForRestoreStorageMustSupportRts,
+                   "Invariant failure") {
+    storageGlobalParams.magicRestore = true;
+    auto opCtx = getOperationContext();
+    getStorageInterface()->setInitialDataTimestamp(opCtx->getServiceContext(),
+                                                   Timestamp::kAllowUnstableCheckpointsSentinel);
+    getStorageInterfaceRecovery()->setSupportsRecoveryTimestamp(false);
+    ReplicationRecoveryImpl recovery(getStorageInterface(), getConsistencyMarkers());
+    _setUpOplog(opCtx, getStorageInterface(), {1, 2, 3, 4, 5});
+    recovery.applyOplogEntriesForRestore(opCtx, Timestamp(1, 1));
+}
+
+DEATH_TEST_REGEX_F(ReplicationRecoveryTest,
+                   ApplyOplogEntriesForRestoreNoOplog,
+                   "Fatal assertion.*8290703") {
+    storageGlobalParams.magicRestore = true;
+    auto opCtx = getOperationContext();
+    getStorageInterface()->setInitialDataTimestamp(opCtx->getServiceContext(),
+                                                   Timestamp::kAllowUnstableCheckpointsSentinel);
+    ReplicationRecoveryImpl recovery(getStorageInterface(), getConsistencyMarkers());
+    recovery.applyOplogEntriesForRestore(opCtx, Timestamp(1, 1));
+}
+
+TEST_F(ReplicationRecoveryTest, ApplyOplogEntriesForRestoreStartPointIsAfterOplog) {
+    storageGlobalParams.magicRestore = true;
+    auto opCtx = getOperationContext();
+    getStorageInterface()->setInitialDataTimestamp(opCtx->getServiceContext(),
+                                                   Timestamp::kAllowUnstableCheckpointsSentinel);
+    ReplicationRecoveryImpl recovery(getStorageInterface(), getConsistencyMarkers());
+    _setUpOplog(opCtx, getStorageInterface(), {1, 2, 3, 4, 5});
+    getStorageInterfaceRecovery()->setRecoveryTimestamp(Timestamp(7, 7));
+    // The function will adjust the Timestamp(7, 7) start point to the top of the oplog.
+    startCapturingLogMessages();
+    ASSERT_DOES_NOT_THROW(recovery.applyOplogEntriesForRestore(opCtx, Timestamp(7, 7)));
+    stopCapturingLogMessages();
+    ASSERT_EQUALS(
+        1,
+        countTextFormatLogLinesContaining("Start point for recovery oplog application not found in "
+                                          "oplog. Adjusting start point to earlier entry"));
+    ASSERT_EQ(getConsistencyMarkers()->getAppliedThrough(opCtx), OpTime());
+    ASSERT_EQ(getStorageInterface()->getInitialDataTimestamp(opCtx->getServiceContext()),
+              Timestamp::kAllowUnstableCheckpointsSentinel);
+}
+
+
 }  // namespace
 }  // namespace repl
 }  // namespace mongo
