@@ -30,10 +30,16 @@
 
 #include "mongo/db/repl/reporter.h"
 
-#include "mongo/bson/util/bson_extract.h"
+#include <boost/move/utility_core.hpp>
+// IWYU pragma: no_include "cxxabi.h"
+#include <mutex>
+
+#include "mongo/base/error_codes.h"
 #include "mongo/db/commands/server_status_metric.h"
-#include "mongo/db/repl/update_position_args.h"
+#include "mongo/executor/remote_command_request.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
 #include "mongo/rpc/get_status_from_command_result.h"
 #include "mongo/util/assert_util.h"
 #include "mongo/util/destructor_guard.h"
@@ -47,7 +53,7 @@ namespace repl {
 namespace {
 
 // The number of replSetUpdatePosition commands a node sent to its sync source.
-CounterMetric numUpdatePosition("repl.network.replSetUpdatePosition.num");
+auto& numUpdatePosition = *MetricBuilder<Counter64>{"repl.network.replSetUpdatePosition.num"};
 
 }  // namespace
 
@@ -97,32 +103,44 @@ void Reporter::shutdown() {
 
     _status = Status(ErrorCodes::CallbackCanceled, "Reporter no longer valid");
 
-    if (!_isActive_inlock()) {
-        return;
+    _requestWaitingStatus = RequestWaitingStatus::kNoWaiting;
+
+    if (_isActive_inlock()) {
+        executor::TaskExecutor::CallbackHandle handle;
+        if (_remoteCommandCallbackHandle.isValid()) {
+            invariant(!_prepareAndSendCommandCallbackHandle.isValid());
+            handle = _remoteCommandCallbackHandle;
+        } else {
+            invariant(!_remoteCommandCallbackHandle.isValid());
+            invariant(_prepareAndSendCommandCallbackHandle.isValid());
+            handle = _prepareAndSendCommandCallbackHandle;
+        }
+
+        _executor->cancel(handle);
     }
 
-    _isWaitingToSendReporter = false;
+    if (_isBackupActive_inlock()) {
+        executor::TaskExecutor::CallbackHandle handle;
+        if (_backupRemoteCommandCallbackHandle.isValid()) {
+            invariant(!_backupPrepareAndSendCommandCallbackHandle.isValid());
+            handle = _backupRemoteCommandCallbackHandle;
+        } else {
+            invariant(!_backupRemoteCommandCallbackHandle.isValid());
+            invariant(_backupPrepareAndSendCommandCallbackHandle.isValid());
+            handle = _backupPrepareAndSendCommandCallbackHandle;
+        }
 
-    executor::TaskExecutor::CallbackHandle handle;
-    if (_remoteCommandCallbackHandle.isValid()) {
-        invariant(!_prepareAndSendCommandCallbackHandle.isValid());
-        handle = _remoteCommandCallbackHandle;
-    } else {
-        invariant(!_remoteCommandCallbackHandle.isValid());
-        invariant(_prepareAndSendCommandCallbackHandle.isValid());
-        handle = _prepareAndSendCommandCallbackHandle;
+        _executor->cancel(handle);
     }
-
-    _executor->cancel(handle);
 }
 
 Status Reporter::join() {
     stdx::unique_lock<Latch> lk(_mutex);
-    _condition.wait(lk, [this]() { return !_isActive_inlock(); });
+    _condition.wait(lk, [this]() { return !_isActive_inlock() && !_isBackupActive_inlock(); });
     return _status;
 }
 
-Status Reporter::trigger() {
+Status Reporter::trigger(bool allowOneMore) {
     stdx::lock_guard<Latch> lk(_mutex);
 
     // If these was a previous error then the reporter is dead and return that error.
@@ -130,6 +148,7 @@ Status Reporter::trigger() {
         return _status;
     }
 
+    bool useBackupChannel = false;
     if (_keepAliveTimeoutWhen != Date_t()) {
         // Reset keep alive expiration to signal handler that it was canceled internally.
         invariant(_prepareAndSendCommandCallbackHandle.isValid());
@@ -137,27 +156,39 @@ Status Reporter::trigger() {
         _executor->cancel(_prepareAndSendCommandCallbackHandle);
         return Status::OK();
     } else if (_isActive_inlock()) {
-        _isWaitingToSendReporter = true;
-        return Status::OK();
+        if (!allowOneMore) {
+            // If it is already scheduled to be prioritized request, keep it as it is.
+            if (_requestWaitingStatus == RequestWaitingStatus::kNoWaiting) {
+                _requestWaitingStatus = RequestWaitingStatus::kNormalWaiting;
+            }
+            return Status::OK();
+        } else if (_isBackupActive_inlock()) {
+            _requestWaitingStatus = RequestWaitingStatus::kPrioritizedWaiting;
+            return Status::OK();
+        } else {
+            useBackupChannel = true;
+        }
     }
 
     auto scheduleResult =
-        _executor->scheduleWork([=](const executor::TaskExecutor::CallbackArgs& args) {
-            _prepareAndSendCommandCallback(args, true);
+        _executor->scheduleWork([=, this](const executor::TaskExecutor::CallbackArgs& args) {
+            _prepareAndSendCommandCallback(args, true, useBackupChannel);
         });
 
     _status = scheduleResult.getStatus();
     if (!_status.isOK()) {
-        LOGV2_DEBUG(
-            21585,
-            2,
-            "Reporter failed to schedule callback to prepare and send update command: {error}",
-            "Reporter failed to schedule callback to prepare and send update command",
-            "error"_attr = _status);
+        LOGV2_DEBUG(21585,
+                    2,
+                    "Reporter failed to schedule callback to prepare and send update command",
+                    "error"_attr = _status);
         return _status;
     }
 
-    _prepareAndSendCommandCallbackHandle = scheduleResult.getValue();
+    if (!useBackupChannel) {
+        _prepareAndSendCommandCallbackHandle = scheduleResult.getValue();
+    } else {
+        _backupPrepareAndSendCommandCallbackHandle = scheduleResult.getValue();
+    }
 
     return _status;
 }
@@ -176,7 +207,6 @@ StatusWith<BSONObj> Reporter::_prepareCommand() {
     if (!prepareResult.isOK()) {
         LOGV2_DEBUG(21586,
                     2,
-                    "Reporter failed to prepare update command with status: {error}",
                     "Reporter failed to prepare update command",
                     "error"_attr = prepareResult.getStatus());
         _status = prepareResult.getStatus();
@@ -186,27 +216,25 @@ StatusWith<BSONObj> Reporter::_prepareCommand() {
     return prepareResult.getValue();
 }
 
-void Reporter::_sendCommand_inlock(BSONObj commandRequest, Milliseconds netTimeout) {
+void Reporter::_sendCommand_inlock(BSONObj commandRequest,
+                                   Milliseconds netTimeout,
+                                   bool useBackupChannel) {
     LOGV2_DEBUG(21587,
                 2,
-                "Reporter sending oplog progress to upstream updater {target}: {commandRequest}",
                 "Reporter sending oplog progress to upstream updater",
                 "target"_attr = _target,
                 "commandRequest"_attr = commandRequest);
 
     auto scheduleResult = _executor->scheduleRemoteCommand(
-        executor::RemoteCommandRequest(_target, "admin", commandRequest, nullptr, netTimeout),
-        [this](const executor::TaskExecutor::RemoteCommandCallbackArgs& rcbd) {
-            _processResponseCallback(rcbd);
+        executor::RemoteCommandRequest(
+            _target, DatabaseName::kAdmin, commandRequest, nullptr, netTimeout),
+        [this, useBackupChannel](const executor::TaskExecutor::RemoteCommandCallbackArgs& rcbd) {
+            _processResponseCallback(rcbd, useBackupChannel);
         });
 
     _status = scheduleResult.getStatus();
     if (!_status.isOK()) {
-        LOGV2_DEBUG(21588,
-                    2,
-                    "Reporter failed to schedule with status: {error}",
-                    "Reporter failed to schedule",
-                    "error"_attr = _status);
+        LOGV2_DEBUG(21588, 2, "Reporter failed to schedule", "error"_attr = _status);
         if (_status != ErrorCodes::ShutdownInProgress) {
             fassert(34434, _status);
         }
@@ -215,25 +243,28 @@ void Reporter::_sendCommand_inlock(BSONObj commandRequest, Milliseconds netTimeo
 
     numUpdatePosition.increment(1);
 
-    _remoteCommandCallbackHandle = scheduleResult.getValue();
+    if (useBackupChannel) {
+        _backupRemoteCommandCallbackHandle = scheduleResult.getValue();
+    } else {
+        _remoteCommandCallbackHandle = scheduleResult.getValue();
+    }
 }
 
 void Reporter::_processResponseCallback(
-    const executor::TaskExecutor::RemoteCommandCallbackArgs& rcbd) {
+    const executor::TaskExecutor::RemoteCommandCallbackArgs& rcbd, bool useBackupChannel) {
     {
         stdx::lock_guard<Latch> lk(_mutex);
 
         // If the reporter was shut down before this callback is invoked,
         // return the canceled "_status".
         if (!_status.isOK()) {
-            invariant(_status == ErrorCodes::CallbackCanceled);
-            _onShutdown_inlock();
+            _onShutdown_inlock(useBackupChannel);
             return;
         }
 
         _status = rcbd.response.status;
         if (!_status.isOK()) {
-            _onShutdown_inlock();
+            _onShutdown_inlock(useBackupChannel);
             return;
         }
 
@@ -242,22 +273,30 @@ void Reporter::_processResponseCallback(
         _status = getStatusFromCommandResult(commandResult);
 
         if (!_status.isOK()) {
-            _onShutdown_inlock();
+            _onShutdown_inlock(useBackupChannel);
             return;
         }
 
-        if (!_isWaitingToSendReporter) {
+        if (useBackupChannel &&
+            _requestWaitingStatus != RequestWaitingStatus::kPrioritizedWaiting) {
+            _backupRemoteCommandCallbackHandle = executor::TaskExecutor::CallbackHandle();
+            return;
+        }
+
+
+        if (_requestWaitingStatus == RequestWaitingStatus::kNoWaiting) {
             // Since we are also on a timer, schedule a report for that interval, or until
             // triggered.
             auto when = _executor->now() + _keepAliveInterval;
             bool fromTrigger = false;
             auto scheduleResult = _executor->scheduleWorkAt(
-                when, [=](const executor::TaskExecutor::CallbackArgs& args) {
-                    _prepareAndSendCommandCallback(args, fromTrigger);
+                when, [=, this](const executor::TaskExecutor::CallbackArgs& args) {
+                    _prepareAndSendCommandCallback(args, fromTrigger, false
+                                                   /*useBackupChannel*/);
                 });
             _status = scheduleResult.getStatus();
             if (!_status.isOK()) {
-                _onShutdown_inlock();
+                _onShutdown_inlock(useBackupChannel);
                 return;
             }
 
@@ -272,28 +311,34 @@ void Reporter::_processResponseCallback(
     // Must call without holding the lock.
     auto prepareResult = _prepareCommand();
 
+    // Since we unlock above, there is a chance that the main channel and backup channel reach this
+    // point at about the same time and the updatePosition request would be almost the same. We may
+    // save one request in that case but for now we leave it for future optimization.
     stdx::lock_guard<Latch> lk(_mutex);
     if (!_status.isOK()) {
-        _onShutdown_inlock();
+        _onShutdown_inlock(useBackupChannel);
         return;
     }
 
-    _sendCommand_inlock(prepareResult.getValue(), _updatePositionTimeout);
+    _sendCommand_inlock(prepareResult.getValue(), _updatePositionTimeout, useBackupChannel);
     if (!_status.isOK()) {
-        _onShutdown_inlock();
+        _onShutdown_inlock(useBackupChannel);
         return;
     }
 
-    invariant(_remoteCommandCallbackHandle.isValid());
-    _isWaitingToSendReporter = false;
+    auto& remoteCommandCallbackHandle =
+        useBackupChannel ? _backupRemoteCommandCallbackHandle : _remoteCommandCallbackHandle;
+    invariant(remoteCommandCallbackHandle.isValid());
+    _requestWaitingStatus = RequestWaitingStatus::kNoWaiting;
 }
 
 void Reporter::_prepareAndSendCommandCallback(const executor::TaskExecutor::CallbackArgs& args,
-                                              bool fromTrigger) {
+                                              bool fromTrigger,
+                                              bool useBackupChannel) {
     {
         stdx::lock_guard<Latch> lk(_mutex);
         if (!_status.isOK()) {
-            _onShutdown_inlock();
+            _onShutdown_inlock(useBackupChannel);
             return;
         }
 
@@ -306,7 +351,7 @@ void Reporter::_prepareAndSendCommandCallback(const executor::TaskExecutor::Call
         }
 
         if (!_status.isOK()) {
-            _onShutdown_inlock();
+            _onShutdown_inlock(useBackupChannel);
             return;
         }
     }
@@ -316,26 +361,39 @@ void Reporter::_prepareAndSendCommandCallback(const executor::TaskExecutor::Call
 
     stdx::lock_guard<Latch> lk(_mutex);
     if (!_status.isOK()) {
-        _onShutdown_inlock();
+        _onShutdown_inlock(useBackupChannel);
         return;
     }
 
-    _sendCommand_inlock(prepareResult.getValue(), _updatePositionTimeout);
+    _sendCommand_inlock(prepareResult.getValue(), _updatePositionTimeout, useBackupChannel);
     if (!_status.isOK()) {
-        _onShutdown_inlock();
+        _onShutdown_inlock(useBackupChannel);
         return;
     }
 
-    invariant(_remoteCommandCallbackHandle.isValid());
-    _prepareAndSendCommandCallbackHandle = executor::TaskExecutor::CallbackHandle();
-    _keepAliveTimeoutWhen = Date_t();
+    auto& remoteCommandCallbackHandle =
+        useBackupChannel ? _backupRemoteCommandCallbackHandle : _remoteCommandCallbackHandle;
+    auto& prepareAndSendCommandCallbackHandle = useBackupChannel
+        ? _backupPrepareAndSendCommandCallbackHandle
+        : _prepareAndSendCommandCallbackHandle;
+    invariant(remoteCommandCallbackHandle.isValid());
+    prepareAndSendCommandCallbackHandle = executor::TaskExecutor::CallbackHandle();
+    if (!useBackupChannel) {
+        // Only reset keepAliveTimeout when it is triggered for the main channel.
+        _keepAliveTimeoutWhen = Date_t();
+    }
 }
 
-void Reporter::_onShutdown_inlock() {
-    _isWaitingToSendReporter = false;
-    _remoteCommandCallbackHandle = executor::TaskExecutor::CallbackHandle();
-    _prepareAndSendCommandCallbackHandle = executor::TaskExecutor::CallbackHandle();
-    _keepAliveTimeoutWhen = Date_t();
+void Reporter::_onShutdown_inlock(bool useBackupChannel) {
+    _requestWaitingStatus = RequestWaitingStatus::kNoWaiting;
+    if (!useBackupChannel) {
+        _remoteCommandCallbackHandle = executor::TaskExecutor::CallbackHandle();
+        _prepareAndSendCommandCallbackHandle = executor::TaskExecutor::CallbackHandle();
+        _keepAliveTimeoutWhen = Date_t();
+    } else {
+        _backupRemoteCommandCallbackHandle = executor::TaskExecutor::CallbackHandle();
+        _backupPrepareAndSendCommandCallbackHandle = executor::TaskExecutor::CallbackHandle();
+    }
     _condition.notify_all();
 }
 
@@ -344,13 +402,23 @@ bool Reporter::isActive() const {
     return _isActive_inlock();
 }
 
+bool Reporter::isBackupActive() const {
+    stdx::lock_guard<Latch> lk(_mutex);
+    return _isBackupActive_inlock();
+}
+
 bool Reporter::_isActive_inlock() const {
     return _remoteCommandCallbackHandle.isValid() || _prepareAndSendCommandCallbackHandle.isValid();
 }
 
+bool Reporter::_isBackupActive_inlock() const {
+    return _backupRemoteCommandCallbackHandle.isValid() ||
+        _backupPrepareAndSendCommandCallbackHandle.isValid();
+}
+
 bool Reporter::isWaitingToSendReport() const {
     stdx::lock_guard<Latch> lk(_mutex);
-    return _isWaitingToSendReporter;
+    return _requestWaitingStatus != RequestWaitingStatus::kNoWaiting;
 }
 
 Date_t Reporter::getKeepAliveTimeoutWhen_forTest() const {
