@@ -27,22 +27,37 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
+#include <boost/move/utility_core.hpp>
+#include <map>
 #include <memory>
+#include <ratio>
+#include <utility>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/db/baton.h"
+#include "mongo/db/repl/optime.h"
 #include "mongo/db/repl/reporter.h"
 #include "mongo/db/repl/update_position_args.h"
+#include "mongo/executor/network_connection_hook.h"
 #include "mongo/executor/network_interface_mock.h"
+#include "mongo/executor/remote_command_request.h"
+#include "mongo/executor/remote_command_response.h"
+#include "mongo/executor/task_executor_test_fixture.h"
 #include "mongo/executor/thread_pool_task_executor_test_fixture.h"
+#include "mongo/stdx/type_traits.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
 #include "mongo/unittest/task_executor_proxy.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/util/assert_util.h"
 
 namespace {
 
 using namespace mongo;
 using namespace mongo::repl;
-using executor::NetworkInterfaceMock;
 using executor::RemoteCommandRequest;
 using executor::RemoteCommandResponse;
 
@@ -71,12 +86,12 @@ public:
             cmdBuilder.subarrayStart(UpdatePositionArgs::kUpdateArrayFieldName));
         for (auto&& itr : _progressMap) {
             BSONObjBuilder entry(arrayBuilder.subobjStart());
-            itr.second.lastDurableOpTime.append(&entry,
-                                                UpdatePositionArgs::kDurableOpTimeFieldName);
+            itr.second.lastDurableOpTime.append(UpdatePositionArgs::kDurableOpTimeFieldName,
+                                                &entry);
             entry.appendDate(UpdatePositionArgs::kDurableWallTimeFieldName,
                              Date_t() + Seconds(itr.second.lastDurableOpTime.getSecs()));
-            itr.second.lastAppliedOpTime.append(&entry,
-                                                UpdatePositionArgs::kAppliedOpTimeFieldName);
+            itr.second.lastAppliedOpTime.append(UpdatePositionArgs::kAppliedOpTimeFieldName,
+                                                &entry);
             entry.appendDate(UpdatePositionArgs::kAppliedWallTimeFieldName,
                              Date_t() + Seconds(itr.second.lastAppliedOpTime.getSecs()));
             entry.append(UpdatePositionArgs::kMemberIdFieldName, itr.first);
@@ -139,7 +154,7 @@ protected:
 
 class ReporterTestNoTriggerAtSetUp : public ReporterTest {
 private:
-    virtual bool triggerAtSetUp() const override;
+    bool triggerAtSetUp() const override;
 };
 
 ReporterTest::ReporterTest() {}
@@ -251,9 +266,11 @@ TEST(UpdatePositionArgs, AcceptsUnknownFieldInUpdateInfo) {
     auto updateInfo =
         BSON(UpdatePositionArgs::kConfigVersionFieldName
              << 1 << UpdatePositionArgs::kMemberIdFieldName << 1
-             << UpdatePositionArgs::kDurableOpTimeFieldName << OpTime()
              << UpdatePositionArgs::kAppliedOpTimeFieldName << OpTime()
+             << UpdatePositionArgs::kWrittenOpTimeFieldName << OpTime()
+             << UpdatePositionArgs::kDurableOpTimeFieldName << OpTime()
              << UpdatePositionArgs::kAppliedWallTimeFieldName << now
+             << UpdatePositionArgs::kWrittenWallTimeFieldName << now
              << UpdatePositionArgs::kDurableWallTimeFieldName << now << "unknownField" << 1);
     bob.append(UpdatePositionArgs::kUpdateArrayFieldName, BSON_ARRAY(updateInfo));
     BSONObj cmdObj = bob.obj();
@@ -267,6 +284,7 @@ TEST(UpdatePositionArgs, AcceptsUnknownFieldInUpdateInfo) {
     ASSERT_EQ(updatesArr.nFields(), 1);
     bob2.appendElements(updatesArr[0].Obj());
     bob2.append(UpdatePositionArgs::kAppliedWallTimeFieldName, now);
+    bob2.append(UpdatePositionArgs::kWrittenWallTimeFieldName, now);
     bob2.append(UpdatePositionArgs::kDurableWallTimeFieldName, now);
     bob2.append("unknownField", 1);
     ASSERT_EQ(bob2.obj().woCompare(updateInfo), 0);
@@ -568,8 +586,8 @@ TEST_F(ReporterTestNoTriggerAtSetUp,
     public:
         TaskExecutorWithFailureInScheduleWork(executor::TaskExecutor* executor)
             : unittest::TaskExecutorProxy(executor) {}
-        virtual StatusWith<executor::TaskExecutor::CallbackHandle> scheduleWork(
-            CallbackFn&& override) {
+        StatusWith<executor::TaskExecutor::CallbackHandle> scheduleWork(
+            CallbackFn&& override) override {
             return Status(ErrorCodes::OperationFailed, "failed to schedule work");
         }
     };
@@ -591,7 +609,7 @@ TEST_F(ReporterTestNoTriggerAtSetUp, FailingToScheduleRemoteCommandTaskShouldMak
     public:
         TaskExecutorWithFailureInScheduleRemoteCommand(executor::TaskExecutor* executor)
             : unittest::TaskExecutorProxy(executor) {}
-        virtual StatusWith<executor::TaskExecutor::CallbackHandle> scheduleRemoteCommandOnAny(
+        StatusWith<executor::TaskExecutor::CallbackHandle> scheduleRemoteCommandOnAny(
             const executor::RemoteCommandRequestOnAny& request,
             const RemoteCommandOnAnyCallbackFn& cb,
             const BatonHandle& baton = nullptr) override {
@@ -620,8 +638,8 @@ TEST_F(ReporterTest, FailingToScheduleTimeoutShouldMakeReporterInactive) {
     public:
         TaskExecutorWithFailureInScheduleWorkAt(executor::TaskExecutor* executor)
             : unittest::TaskExecutorProxy(executor) {}
-        virtual StatusWith<executor::TaskExecutor::CallbackHandle> scheduleWorkAt(
-            Date_t when, CallbackFn&&) override {
+        StatusWith<executor::TaskExecutor::CallbackHandle> scheduleWorkAt(Date_t when,
+                                                                          CallbackFn&&) override {
             return Status(ErrorCodes::OperationFailed, "failed to schedule work");
         }
     };
@@ -720,4 +738,70 @@ TEST_F(ReporterTest, ShutdownImmediatelyAfterTriggerWhileKeepAliveTimeoutIsSched
     assertReporterDone();
 }
 
+TEST_F(ReporterTest, AllowUsingBackupChannel) {
+    ASSERT_FALSE(reporter->isBackupActive());
+    ASSERT_OK(reporter->trigger());
+    ASSERT_FALSE(reporter->isBackupActive());
+    ASSERT_OK(reporter->trigger(true /*allowOneMore*/));
+    ASSERT_TRUE(reporter->isBackupActive());
+
+    processNetworkResponse(BSON("ok" << 1), true);
+    processNetworkResponse(BSON("ok" << 1), true);
+    processNetworkResponse(BSON("ok" << 1), false);
+    ASSERT_FALSE(reporter->isBackupActive());
+    ASSERT_TRUE(reporter->isActive());
+
+    reporter->shutdown();
+    ASSERT_EQUALS(ErrorCodes::CallbackCanceled, reporter->join());
+    assertReporterDone();
+}
+
+TEST_F(ReporterTest, BackupChannelFailureAlsoCauseReporterFailure) {
+    ASSERT_FALSE(reporter->isBackupActive());
+    ASSERT_OK(reporter->trigger(true /*allowOneMore*/));
+    ASSERT_TRUE(reporter->isBackupActive());
+
+    processNetworkResponse(BSON("ok" << 1), true);
+    processNetworkResponse({ErrorCodes::OperationFailed, "update failed", Milliseconds(0)}, false);
+    ASSERT_EQUALS(ErrorCodes::OperationFailed, reporter->getStatus_forTest().code());
+
+    // We need to explicitly shutdown the reporter here because we need to ask the main channel to
+    // quit without processing a network response.
+    reporter->shutdown();
+    ASSERT_EQUALS(ErrorCodes::CallbackCanceled, reporter->join());
+    assertReporterDone();
+}
+
+TEST_F(ReporterTest, BackupChannelSendAnotherOneAfterResponseIfReceiveTwo) {
+    ASSERT_FALSE(reporter->isBackupActive());
+    ASSERT_OK(reporter->trigger(true /*allowOneMore*/));
+    ASSERT_TRUE(reporter->isBackupActive());
+    // Trigger on the backup channel one more time so reporter will send another request immediately
+    // after one channel becomes free.
+    ASSERT_OK(reporter->trigger(true /*allowOneMore*/));
+    ASSERT_TRUE(reporter->isWaitingToSendReport());
+    processNetworkResponse(BSON("ok" << 1), true);
+    processNetworkResponse(BSON("ok" << 1), true);
+    ASSERT_TRUE(reporter->isActive() || reporter->isBackupActive());
+    ASSERT_FALSE(reporter->isWaitingToSendReport());
+
+    processNetworkResponse(BSON("ok" << 1), false);
+
+    reporter->shutdown();
+    ASSERT_EQUALS(ErrorCodes::CallbackCanceled, reporter->join());
+    assertReporterDone();
+}
+
+TEST_F(ReporterTestNoTriggerAtSetUp, NotUsingBackupChannelWhenFree) {
+    ASSERT_FALSE(reporter->isBackupActive());
+    ASSERT_FALSE(reporter->isActive());
+    ASSERT_OK(reporter->trigger(true /*allowOneMore*/));
+    ASSERT_FALSE(reporter->isBackupActive());
+    ASSERT_TRUE(reporter->isActive());
+
+    reporter->shutdown();
+    ASSERT_EQUALS(ErrorCodes::CallbackCanceled, reporter->join());
+    assertReporterDone();
+}
+
 }  // namespace
