@@ -28,9 +28,18 @@
  */
 
 
-#include "mongo/platform/basic.h"
+#include <memory>
+#include <mutex>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/db/repl/rollback_checker.h"
+#include "mongo/executor/remote_command_request.h"
+#include "mongo/util/assert_util.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplication
 
@@ -137,7 +146,7 @@ bool RollbackChecker::_checkForRollback_inlock(int remoteRBID) {
 StatusWith<RollbackChecker::CallbackHandle> RollbackChecker::_scheduleGetRollbackId(
     const RemoteCommandCallbackFn& nextAction) {
     executor::RemoteCommandRequest getRollbackIDReq(
-        _syncSource, "admin", BSON("replSetGetRBID" << 1), nullptr);
+        _syncSource, DatabaseName::kAdmin, BSON("replSetGetRBID" << 1), nullptr);
     return _executor->scheduleRemoteCommand(getRollbackIDReq, nextAction);
 }
 
