@@ -29,16 +29,40 @@
 
 #include "mongo/db/repl/rollback_impl.h"
 
+#include <absl/container/flat_hash_map.h>
+#include <absl/container/node_hash_map.h>
+#include <absl/container/node_hash_set.h>
+#include <absl/meta/type_traits.h>
+#include <algorithm>
+#include <boost/cstdint.hpp>
+#include <boost/filesystem/path.hpp>
+#include <boost/none.hpp>
+#include <cstdint>
 #include <fmt/format.h>
+#include <limits>
+#include <memory>
+#include <mutex>
+#include <utility>
 
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/bson/util/bson_extract.h"
+#include "mongo/client/dbclient_cursor.h"
+#include "mongo/db/catalog/collection.h"
 #include "mongo/db/catalog/collection_catalog.h"
-#include "mongo/db/catalog/database_holder.h"
 #include "mongo/db/catalog/drop_collection.h"
 #include "mongo/db/catalog/import_collection_oplog_entry_gen.h"
+#include "mongo/db/client.h"
+#include "mongo/db/cluster_role.h"
 #include "mongo/db/commands.h"
 #include "mongo/db/concurrency/d_concurrency.h"
 #include "mongo/db/concurrency/exception_util.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
 #include "mongo/db/concurrency/replication_state_transition_lock_guard.h"
 #include "mongo/db/database_name.h"
 #include "mongo/db/db_raii.h"
@@ -46,30 +70,64 @@
 #include "mongo/db/dbhelpers.h"
 #include "mongo/db/global_index.h"
 #include "mongo/db/index_builds_coordinator.h"
-#include "mongo/db/logical_time_validator.h"
 #include "mongo/db/operation_context.h"
+#include "mongo/db/query/find_command.h"
 #include "mongo/db/query/get_executor.h"
-#include "mongo/db/repl/apply_ops.h"
+#include "mongo/db/query/plan_executor.h"
+#include "mongo/db/query/plan_yield_policy.h"
+#include "mongo/db/repl/apply_ops_command_info.h"
 #include "mongo/db/repl/drop_pending_collection_reaper.h"
+#include "mongo/db/repl/member_state.h"
+#include "mongo/db/repl/oplog.h"
+#include "mongo/db/repl/oplog_entry_gen.h"
+#include "mongo/db/repl/read_concern_args.h"
+#include "mongo/db/repl/repl_settings.h"
+#include "mongo/db/repl/replica_set_aware_service.h"
+#include "mongo/db/repl/replication_consistency_markers.h"
 #include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/repl/replication_process.h"
+#include "mongo/db/repl/replication_recovery.h"
 #include "mongo/db/repl/roll_back_local_operations.h"
 #include "mongo/db/repl/rollback_impl_gen.h"
+#include "mongo/db/repl/split_prepare_session_manager.h"
 #include "mongo/db/repl/storage_interface.h"
 #include "mongo/db/repl/tenant_migration_access_blocker_util.h"
 #include "mongo/db/repl/transaction_oplog_application.h"
 #include "mongo/db/s/type_shard_identity.h"
+#include "mongo/db/server_options.h"
 #include "mongo/db/server_recovery.h"
 #include "mongo/db/serverless/serverless_operation_lock_registry.h"
+#include "mongo/db/service_context.h"
 #include "mongo/db/session/kill_sessions_local.h"
+#include "mongo/db/session/logical_session_id_gen.h"
 #include "mongo/db/session/session_catalog_mongod.h"
 #include "mongo/db/session/session_txn_record_gen.h"
-#include "mongo/db/storage/historical_ident_tracker.h"
+#include "mongo/db/shard_role.h"
+#include "mongo/db/storage/record_store.h"
+#include "mongo/db/storage/recovery_unit.h"
 #include "mongo/db/storage/remove_saver.h"
+#include "mongo/db/storage/storage_engine.h"
 #include "mongo/db/transaction/transaction_history_iterator.h"
+#include "mongo/db/transaction_resources.h"
+#include "mongo/idl/idl_parser.h"
+#include "mongo/logv2/attribute_storage.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/redaction.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/platform/compiler.h"
 #include "mongo/s/catalog/type_config_version.h"
+#include "mongo/s/database_version.h"
+#include "mongo/s/shard_version.h"
+#include "mongo/util/clock_source.h"
+#include "mongo/util/duration.h"
+#include "mongo/util/fail_point.h"
+#include "mongo/util/namespace_string_util.h"
+#include "mongo/util/net/hostandport.h"
 #include "mongo/util/scopeguard.h"
+#include "mongo/util/str.h"
+#include "mongo/util/string_map.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplicationRollback
 
@@ -200,6 +258,7 @@ Status RollbackImpl::runRollback(OperationContext* opCtx) {
         return status;
     }
     _listener->onTransitionToRollback();
+    ReplicaSetAwareServiceRegistry::get(getGlobalServiceContext()).onRollback();
 
     if (MONGO_unlikely(rollbackHangAfterTransitionToRollback.shouldFail())) {
         LOGV2(21591,
@@ -288,7 +347,7 @@ void RollbackImpl::_killAllUserOperations(OperationContext* opCtx) {
     int numOpsRunning = 0;
 
     for (ServiceContext::LockedClientsCursor cursor(serviceCtx); Client* client = cursor.next();) {
-        stdx::lock_guard<Client> lk(*client);
+        ClientLock lk(client);
         if (client->isFromSystemConnection() && !client->canKillSystemOperationInStepdown(lk)) {
             continue;
         }
@@ -377,8 +436,6 @@ void RollbackImpl::_stopAndWaitForIndexBuilds(OperationContext* opCtx) {
         if (numInProg > 0) {
             LOGV2_DEBUG(21596,
                         1,
-                        "Waiting for {numBackgroundOperationsInProgress} "
-                        "background operations to complete on database '{db}'",
                         "Waiting for background operations to complete",
                         "numBackgroundOperationsInProgress"_attr = numInProg,
                         logAttrs(dbName));
@@ -389,20 +446,26 @@ void RollbackImpl::_stopAndWaitForIndexBuilds(OperationContext* opCtx) {
     LOGV2(21597, "Finished waiting for background operations to complete before rollback");
 }
 
-StatusWith<std::set<NamespaceString>> RollbackImpl::_namespacesForOp(const OplogEntry& oplogEntry) {
+StatusWith<std::pair<std::set<NamespaceString>, std::set<UUID>>>
+RollbackImpl::_namespacesAndUUIDsForOp(const OplogEntry& oplogEntry) {
     NamespaceString opNss = oplogEntry.getNss();
+
     OpTypeEnum opType = oplogEntry.getOpType();
     std::set<NamespaceString> namespaces;
+    std::set<UUID> uuids;
+    if (auto opUUID = oplogEntry.getUuid()) {
+        uuids.insert(opUUID.get());
+    }
 
     // No namespaces for a no-op.
     if (opType == OpTypeEnum::kNoop) {
-        return std::set<NamespaceString>();
+        return std::make_pair(std::set<NamespaceString>(), std::set<UUID>());
     }
 
     // CRUD ops have the proper namespace in the operation 'ns' field.
     if (opType == OpTypeEnum::kInsert || opType == OpTypeEnum::kUpdate ||
         opType == OpTypeEnum::kDelete) {
-        return std::set<NamespaceString>({opNss});
+        return std::make_pair(std::set<NamespaceString>({opNss}), uuids);
     }
 
     // If the operation is a command, then we need to extract the appropriate namespaces from the
@@ -413,15 +476,22 @@ StatusWith<std::set<NamespaceString>> RollbackImpl::_namespacesForOp(const Oplog
 
         // Does not handle 'applyOps' entries.
         invariant(oplogEntry.getCommandType() != OplogEntry::CommandType::kApplyOps,
-                  "_namespacesForOp does not handle 'applyOps' oplog entries.");
+                  "_namespacesAndUUIDsForOp does not handle 'applyOps' oplog entries.");
 
         switch (oplogEntry.getCommandType()) {
             case OplogEntry::CommandType::kRenameCollection: {
                 // Add both the 'from' and 'to' namespaces.
-                namespaces.insert(NamespaceStringUtil::deserialize(
-                    opNss.tenantId(), firstElem.valueStringDataSafe()));
                 namespaces.insert(
-                    NamespaceStringUtil::deserialize(opNss.tenantId(), obj.getStringField("to")));
+                    NamespaceStringUtil::deserialize(opNss.tenantId(),
+                                                     firstElem.valueStringDataSafe(),
+                                                     SerializationContext::stateDefault()));
+                namespaces.insert(
+                    NamespaceStringUtil::deserialize(opNss.tenantId(),
+                                                     obj.getStringField("to"),
+                                                     SerializationContext::stateDefault()));
+                if (auto uuidSW = UUID::parse(obj.getField("dropTarget")); uuidSW.isOK()) {
+                    uuids.insert(uuidSW.getValue());
+                }
                 break;
             }
             case OplogEntry::CommandType::kDropDatabase: {
@@ -432,7 +502,7 @@ StatusWith<std::set<NamespaceString>> RollbackImpl::_namespacesForOp(const Oplog
             case OplogEntry::CommandType::kEmptyCapped: {
                 // These commands do not need to be supported by rollback. 'convertToCapped' should
                 // always be converted to lower level DDL operations, and 'emptycapped' is a
-                // testing-only command.
+                // test-only command.
                 std::string message = str::stream()
                     << "Encountered unsupported command type '" << firstElem.fieldName()
                     << "' during rollback.";
@@ -446,6 +516,7 @@ StatusWith<std::set<NamespaceString>> RollbackImpl::_namespacesForOp(const Oplog
             case OplogEntry::CommandType::kImportCollection:
             case OplogEntry::CommandType::kCreateIndexes:
             case OplogEntry::CommandType::kDropIndexes:
+            case OplogEntry::CommandType::kDeleteIndexes:
             case OplogEntry::CommandType::kStartIndexBuild:
             case OplogEntry::CommandType::kAbortIndexBuild:
             case OplogEntry::CommandType::kCommitIndexBuild:
@@ -471,7 +542,7 @@ StatusWith<std::set<NamespaceString>> RollbackImpl::_namespacesForOp(const Oplog
         }
     }
 
-    return namespaces;
+    return std::make_pair(namespaces, uuids);
 }
 
 void RollbackImpl::_restoreTxnsTableEntryFromRetryableWrites(OperationContext* opCtx,
@@ -530,23 +601,32 @@ void RollbackImpl::_restoreTxnsTableEntryFromRetryableWrites(OperationContext* o
             sessionTxnRecord.setLastWriteDate(wallClockTime);
         }
         const auto nss = NamespaceString::kSessionTransactionsTableNamespace;
-        writeConflictRetry(opCtx, "updateSessionTransactionsTableInRollback", nss.ns(), [&] {
-            opCtx->recoveryUnit()->allowOneUntimestampedWrite();
-            AutoGetCollection collection(opCtx, nss, MODE_IX);
+        writeConflictRetry(opCtx, "updateSessionTransactionsTableInRollback", nss, [&] {
+            shard_role_details::getRecoveryUnit(opCtx)->allowOneUntimestampedWrite();
+            auto collection =
+                acquireCollection(opCtx,
+                                  CollectionAcquisitionRequest(
+                                      nss,
+                                      PlacementConcern{boost::none, ShardVersion::UNSHARDED()},
+                                      repl::ReadConcernArgs::get(opCtx),
+                                      AcquisitionPrerequisites::kWrite),
+                                  MODE_IX);
             auto filter = BSON(SessionTxnRecord::kSessionIdFieldName << sessionId.toBSON());
             UnreplicatedWritesBlock uwb(opCtx);
             // Perform an untimestamped write so that it will not be rolled back on recovering
             // to the 'stableTimestamp' if we were to crash. This is safe because this update is
             // meant to be consistent with the 'stableTimestamp' and not the common point.
-            Helpers::upsert(opCtx, nss, filter, sessionTxnRecord.toBSON(), /*fromMigrate=*/false);
+            Helpers::upsert(
+                opCtx, collection, filter, sessionTxnRecord.toBSON(), /*fromMigrate=*/false);
         });
     }
     // Take a stable checkpoint so that writes to the 'config.transactions' table are
     // persisted to disk before truncating the oplog. If we were to take an unstable checkpoint, we
     // would have to update replication metadata like 'minValid.appliedThrough' to be consistent
     // with the oplog.
-    opCtx->recoveryUnit()->waitUntilUnjournaledWritesDurable(opCtx,
-                                                             /*stableCheckpoint=*/true);
+    shard_role_details::getRecoveryUnit(opCtx)->waitUntilUnjournaledWritesDurable(
+        opCtx,
+        /*stableCheckpoint=*/true);
 }
 
 void RollbackImpl::_runPhaseFromAbortToReconstructPreparedTxns(
@@ -603,9 +683,6 @@ void RollbackImpl::_runPhaseFromAbortToReconstructPreparedTxns(
     _rollbackStats.stableTimestamp = stableTimestamp;
     _listener->onRecoverToStableTimestamp(stableTimestamp);
 
-    // Rollback historical ident entries.
-    HistoricalIdentTracker::get(opCtx).rollbackTo(stableTimestamp);
-
     // Log the total number of insert and update operations that have been rolled back as a
     // result of recovering to the stable timestamp.
     auto getCommandCount = [&](StringData key) {
@@ -631,8 +708,6 @@ void RollbackImpl::_runPhaseFromAbortToReconstructPreparedTxns(
     // oplog truncate after point. If we entered rollback, we are guaranteed to have at least one
     // oplog entry after the common point.
     LOGV2(21600,
-          "Marking to truncate all oplog entries with timestamps greater than "
-          "{commonPoint}",
           "Marking to truncate all oplog entries with timestamps greater than common point",
           "commonPoint"_attr = commonPoint.getOpTime().getTimestamp());
     Timestamp truncatePoint = commonPoint.getOpTime().getTimestamp();
@@ -664,7 +739,9 @@ void RollbackImpl::_runPhaseFromAbortToReconstructPreparedTxns(
     // rollback.
     _correctRecordStoreCounts(opCtx);
 
-    tenant_migration_access_blocker::recoverTenantMigrationAccessBlockers(opCtx);
+    if (_replicationCoordinator->getSettings().isServerless()) {
+        tenant_migration_access_blocker::recoverTenantMigrationAccessBlockers(opCtx);
+    }
     ServerlessOperationLockRegistry::recoverLocks(opCtx);
 
     // Reconstruct prepared transactions after counts have been adjusted. Since prepared
@@ -701,8 +778,6 @@ void RollbackImpl::_correctRecordStoreCounts(OperationContext* opCtx) {
             LOGV2_DEBUG(
                 21601,
                 2,
-                "Not setting collection count to {newCount} for {namespace} ({uuid}) "
-                "[{ident}] because it is marked for size adjustment.",
                 "Not setting collection count because namespace is marked for size adjustment",
                 "newCount"_attr = newCount,
                 logAttrs(nss),
@@ -716,14 +791,13 @@ void RollbackImpl::_correctRecordStoreCounts(OperationContext* opCtx) {
         // determine the correct count here post-recovery using a collection scan.
         if (kCollectionScanRequired == newCount) {
             LOGV2(21602,
-                  "Scanning collection {namespace} ({uuid}) to fix collection count.",
                   "Scanning collection to fix collection count",
                   logAttrs(nss),
                   "uuid"_attr = uuid.toString());
             AutoGetCollectionForRead collToScan(opCtx, nss);
             invariant(coll == collToScan.getCollection().get(),
-                      str::stream() << "Catalog returned invalid collection: " << nss.ns() << " ("
-                                    << uuid.toString() << ")");
+                      str::stream() << "Catalog returned invalid collection: "
+                                    << nss.toStringForErrorMsg() << " (" << uuid.toString() << ")");
             auto exec = getCollectionScanExecutor(opCtx,
                                                   collToScan.getCollection(),
                                                   PlanYieldPolicy::YieldPolicy::INTERRUPT_ONLY,
@@ -738,8 +812,6 @@ void RollbackImpl::_correctRecordStoreCounts(OperationContext* opCtx) {
                 // We ignore errors here because crashing or leaving rollback would only leave
                 // collection counts more inaccurate.
                 LOGV2_WARNING(21637,
-                              "Failed to set count of {namespace} ({uuid}) [{ident}] due to failed "
-                              "collection scan: {error}",
                               "Failed to set count of namespace due to failed collection scan",
                               logAttrs(nss),
                               "uuid"_attr = uuid.toString(),
@@ -750,14 +822,11 @@ void RollbackImpl::_correctRecordStoreCounts(OperationContext* opCtx) {
             newCount = countFromScan;
         }
 
-        auto status =
-            _storageInterface->setCollectionCount(opCtx, {nss.db().toString(), uuid}, newCount);
+        auto status = _storageInterface->setCollectionCount(opCtx, {nss.dbName(), uuid}, newCount);
         if (!status.isOK()) {
             // We ignore errors here because crashing or leaving rollback would only leave
             // collection counts more inaccurate.
             LOGV2_WARNING(21638,
-                          "Failed to set count of {namespace} ({uuid}) [{ident}] to {newCount}. "
-                          "Received: {error}",
                           "Failed to set count of namespace",
                           logAttrs(nss),
                           "uuid"_attr = uuid.toString(),
@@ -767,7 +836,6 @@ void RollbackImpl::_correctRecordStoreCounts(OperationContext* opCtx) {
         } else {
             LOGV2_DEBUG(21603,
                         2,
-                        "Set collection count of {namespace} ({uuid}) [{ident}] to {newCount}.",
                         "Set collection count of namespace",
                         logAttrs(nss),
                         "uuid"_attr = uuid.toString(),
@@ -822,9 +890,6 @@ Status RollbackImpl::_findRecordStoreCounts(OperationContext* opCtx) {
 
         if (oldCount > static_cast<uint64_t>(std::numeric_limits<long long>::max())) {
             LOGV2_WARNING(21639,
-                          "Count for {namespace} ({uuid}) was {oldCount} which is larger than the "
-                          "maximum int64_t value. Not attempting to fix "
-                          "count during rollback.",
                           "Count for namespace was larger than the maximum int64_t value. Not "
                           "attempting to fix count during rollback",
                           logAttrs(*nss),
@@ -839,10 +904,6 @@ Status RollbackImpl::_findRecordStoreCounts(OperationContext* opCtx) {
         if (newCount < 0) {
             LOGV2_WARNING(
                 21640,
-                "Attempted to set count for {namespace} ({uuid}) to {newCount} but set it to 0 "
-                "instead. This is likely due to the count previously "
-                "becoming inconsistent from an unclean shutdown or a rollback that could "
-                "not fix the count correctly. Old count: {oldCount}. Count change: {countDiff}",
                 "Attempted to set count for namespace but set it to 0 instead. This is likely due "
                 "to the count previously becoming inconsistent from an unclean shutdown or a "
                 "rollback that could not fix the count correctly",
@@ -853,17 +914,14 @@ Status RollbackImpl::_findRecordStoreCounts(OperationContext* opCtx) {
                 "countDiff"_attr = countDiff);
             newCount = 0;
         }
-        LOGV2_DEBUG(
-            21605,
-            2,
-            "Record count of {namespace} ({uuid}) before rollback is {oldCount}. Setting it "
-            "to {newCount}, due to change of {countDiff}",
-            "Setting record count for namespace after rollback",
-            logAttrs(*nss),
-            "uuid"_attr = uuid.toString(),
-            "oldCount"_attr = oldCount,
-            "newCount"_attr = newCount,
-            "countDiff"_attr = countDiff);
+        LOGV2_DEBUG(21605,
+                    2,
+                    "Setting record count for namespace after rollback",
+                    logAttrs(*nss),
+                    "uuid"_attr = uuid.toString(),
+                    "oldCount"_attr = oldCount,
+                    "newCount"_attr = newCount,
+                    "countDiff"_attr = countDiff);
         _newCounts[uuid] = newCount;
     }
 
@@ -899,7 +957,8 @@ Status RollbackImpl::_processRollbackOp(OperationContext* opCtx, const OplogEntr
         // Follow chain on applyOps oplog entries to process entire unprepared transaction.
         // The beginning of the applyOps chain may precede the common point.
         auto status = _processRollbackOpForApplyOps(opCtx, oplogEntry);
-        if (const auto prevOpTime = oplogEntry.getPrevWriteOpTimeInTransaction()) {
+        if (oplogEntry.applyOpsIsLinkedTransactionally()) {
+            const auto prevOpTime = oplogEntry.getPrevWriteOpTimeInTransaction();
             for (TransactionHistoryIterator iter(*prevOpTime); status.isOK() && iter.hasNext();) {
                 status = _processRollbackOpForApplyOps(opCtx, iter.next(opCtx));
             }
@@ -912,13 +971,15 @@ Status RollbackImpl::_processRollbackOp(OperationContext* opCtx, const OplogEntr
         return Status::OK();
     }
 
-    // Extract the appropriate namespaces from the oplog operation.
-    auto namespacesSW = _namespacesForOp(oplogEntry);
-    if (!namespacesSW.isOK()) {
-        return namespacesSW.getStatus();
+    // Extract the appropriate namespaces and UUIDs from the oplog operation.
+    auto namespacesAndUUIDsSW = _namespacesAndUUIDsForOp(oplogEntry);
+    if (!namespacesAndUUIDsSW.isOK()) {
+        return namespacesAndUUIDsSW.getStatus();
     } else {
-        _observerInfo.rollbackNamespaces.insert(namespacesSW.getValue().begin(),
-                                                namespacesSW.getValue().end());
+        _observerInfo.rollbackNamespaces.insert(namespacesAndUUIDsSW.getValue().first.begin(),
+                                                namespacesAndUUIDsSW.getValue().first.end());
+        _observerInfo.rollbackUUIDs.insert(namespacesAndUUIDsSW.getValue().second.begin(),
+                                           namespacesAndUUIDsSW.getValue().second.end());
     }
 
     // If the operation being rolled back has a session id, then we add it to the set of
@@ -964,7 +1025,6 @@ Status RollbackImpl::_processRollbackOp(OperationContext* opCtx, const OplogEntr
             // Check if the creation of the shard identity document is being rolled back.
             _observerInfo.shardIdentityRolledBack = true;
             LOGV2_WARNING(21641,
-                          "Shard identity document rollback detected. oplog op: {oplogEntry}",
                           "Shard identity document rollback detected",
                           "oplogEntry"_attr = redact(oplogEntry.toBSONForLogging()));
         } else if (serverGlobalParams.clusterRole.has(ClusterRole::ConfigServer) &&
@@ -973,7 +1033,6 @@ Status RollbackImpl::_processRollbackOp(OperationContext* opCtx, const OplogEntr
             // back.
             _observerInfo.configServerConfigVersionRolledBack = true;
             LOGV2_WARNING(21642,
-                          "Config version document rollback detected. oplog op: {oplogEntry}",
                           "Config version document rollback detected",
                           "oplogEntry"_attr = redact(oplogEntry.toBSONForLogging()));
         }
@@ -1050,8 +1109,10 @@ Status RollbackImpl::_processRollbackOp(OperationContext* opCtx, const OplogEntr
             if (auto countResult = _parseDroppedCollectionCount(oplogEntry)) {
                 PendingDropInfo info;
                 info.count = *countResult;
-                info.nss = NamespaceStringUtil::deserialize(
-                    opNss.tenantId(), oplogEntry.getObject()[kToFieldName].String());
+                info.nss =
+                    NamespaceStringUtil::deserialize(opNss.tenantId(),
+                                                     oplogEntry.getObject()[kToFieldName].String(),
+                                                     SerializationContext::stateDefault());
                 _pendingDrops[dropTargetUUID] = info;
                 _newCounts[dropTargetUUID] = info.count;
             } else {
@@ -1154,10 +1215,7 @@ StatusWith<RollBackLocalOperations::RollbackCommonPoint> RollbackImpl::_findComm
     auto stableTimestamp =
         _storageInterface->getLastStableRecoveryTimestamp(opCtx->getServiceContext());
 
-    LOGV2(21607,
-          "Rollback common point is {commonPointOpTime}",
-          "Rollback common point",
-          "commonPointOpTime"_attr = commonPointOpTime);
+    LOGV2(21607, "Rollback common point", "commonPointOpTime"_attr = commonPointOpTime);
 
     // This failpoint is used for testing the invariant below.
     if (MONGO_unlikely(rollbackToTimestampHangCommonPointBeforeReplCommitPoint.shouldFail()) &&
@@ -1181,8 +1239,6 @@ StatusWith<RollBackLocalOperations::RollbackCommonPoint> RollbackImpl::_findComm
         // This is an fassert rather than an invariant, since it can happen if the server was
         // recently upgraded to enableMajorityReadConcern=true.
         LOGV2_FATAL_NOTRACE(51121,
-                            "Common point must be at least stable timestamp, common point: "
-                            "{commonPoint}, stable timestamp: {stableTimestamp}",
                             "Common point must be at least stable timestamp",
                             "commonPoint"_attr = commonPointOpTime.getTimestamp(),
                             "stableTimestamp"_attr = *stableTimestamp);
@@ -1232,9 +1288,6 @@ Status RollbackImpl::_checkAgainstTimeLimit(
     } else {
         LOGV2_WARNING(
             21643,
-            "Wall clock times on oplog entries not monotonically increasing. This "
-            "might indicate a backward clock skew. Time at first oplog after common point: "
-            "{firstOpWallClockTimeAfterCommonPoint}. Time at top of oplog: {topOfOplogWallTime}",
             "Wall clock times on oplog entries not monotonically increasing. This might indicate a "
             "backward clock skew",
             "firstOpWallClockTimeAfterCommonPoint"_attr = firstOpWallClockTimeAfterCommonPoint,
@@ -1254,15 +1307,12 @@ boost::optional<BSONObj> RollbackImpl::_findDocumentById(OperationContext* opCtx
     } else if (document.getStatus().code() == ErrorCodes::NoSuchKey) {
         return boost::none;
     } else {
-        LOGV2_FATAL_CONTINUE(
-            21645,
-            "Rollback failed to read document with {id} in namespace {namespace} with uuid "
-            "{uuid}{error}",
-            "Rollback failed to read document",
-            "id"_attr = redact(id),
-            logAttrs(nss),
-            "uuid"_attr = uuid.toString(),
-            "error"_attr = causedBy(document.getStatus()));
+        LOGV2_FATAL_CONTINUE(21645,
+                             "Rollback failed to read document",
+                             "id"_attr = redact(id),
+                             logAttrs(nss),
+                             "uuid"_attr = uuid.toString(),
+                             "error"_attr = causedBy(document.getStatus()));
         fassert(50751, document.getStatus());
     }
 
@@ -1280,9 +1330,6 @@ Status RollbackImpl::_writeRollbackFiles(OperationContext* opCtx) {
         // managed by the storage engine. See StorageEngine::supportsPendingDrops().
         if (!nss && storageEngine->supportsPendingDrops()) {
             LOGV2(21608,
-                  "The collection with UUID {uuid} is missing in the CollectionCatalog. This could "
-                  "be due to a dropped "
-                  " collection. Not writing rollback file for uuid",
                   "Collection is missing in the CollectionCatalog. This could be due to a dropped "
                   "collection. Not writing rollback file for uuid",
                   "uuid"_attr = uuid);
@@ -1305,8 +1352,6 @@ void RollbackImpl::_writeRollbackFileForNamespace(OperationContext* opCtx,
                                                   const SimpleBSONObjUnorderedSet& idSet) {
     RemoveSaver removeSaver(kRollbackRemoveSaverType, uuid.toString(), kRollbackRemoveSaverWhy);
     LOGV2(21609,
-          "Preparing to write deleted documents to a rollback file for collection {namespace} with "
-          "uuid {uuid} to {file}",
           "Preparing to write deleted documents to a rollback file",
           logAttrs(nss),
           "uuid"_attr = uuid.toString(),
@@ -1367,7 +1412,15 @@ Status RollbackImpl::_triggerOpObserver(OperationContext* opCtx) {
         return Status(ErrorCodes::ShutdownInProgress, "rollback shutting down");
     }
     LOGV2(21610, "Triggering the rollback op observer");
-    opCtx->getServiceContext()->getOpObserver()->onReplicationRollback(opCtx, _observerInfo);
+
+    // Any exceptions thrown from onReplicationRollback() indicates a rollback failure that may
+    // have led us to some inconsistent on-disk or memory state, so we crash instead.
+    try {
+        opCtx->getServiceContext()->getOpObserver()->onReplicationRollback(opCtx, _observerInfo);
+    } catch (const DBException& ex) {
+        fassert(6050902, ex.toStatus());
+    }
+
     return Status::OK();
 }
 
@@ -1386,9 +1439,6 @@ void RollbackImpl::_transitionFromRollbackToSecondary(OperationContext* opCtx) {
     auto status = _replicationCoordinator->setFollowerMode(MemberState::RS_SECONDARY);
     if (!status.isOK()) {
         LOGV2_FATAL_NOTRACE(40408,
-                            "Failed to transition into {targetState}; expected to be in "
-                            "state {expectedState}; found self in "
-                            "{actualState} {error}",
                             "Failed to perform replica set state transition",
                             "targetState"_attr = MemberState(MemberState::RS_SECONDARY),
                             "expectedState"_attr = MemberState(MemberState::RS_ROLLBACK),
@@ -1452,6 +1502,7 @@ void RollbackImpl::_summarizeRollback(OperationContext* opCtx) const {
               _observerInfo.configServerConfigVersionRolledBack);
     attrs.add("affectedSessions", _observerInfo.rollbackSessionIds);
     attrs.add("affectedNamespaces", _observerInfo.rollbackNamespaces);
+    attrs.add("affectedUUIDs", _observerInfo.rollbackUUIDs);
     attrs.add("rollbackCommandCounts", _observerInfo.rollbackCommandCounts);
     attrs.add("totalEntriesRolledBackIncludingNoops", _observerInfo.numberOfEntriesObserved);
     LOGV2(21612, "Rollback summary", attrs);
