@@ -29,14 +29,34 @@
 
 #pragma once
 
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
 #include <functional>
+#include <set>
+#include <string>
+#include <vector>
 
+#include "mongo/base/status.h"
 #include "mongo/base/status_with.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/simple_bsonobj_comparator.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/db/namespace_string.h"
 #include "mongo/db/op_observer/op_observer.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/repl/oplog_entry.h"
+#include "mongo/db/repl/oplog_interface.h"
+#include "mongo/db/repl/optime.h"
+#include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/repl/roll_back_local_operations.h"
 #include "mongo/db/repl/rollback.h"
 #include "mongo/db/repl/storage_interface.h"
+#include "mongo/platform/mutex.h"
+#include "mongo/stdx/unordered_map.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/time_support.h"
+#include "mongo/util/uuid.h"
 
 namespace mongo {
 
@@ -46,6 +66,7 @@ namespace repl {
 
 class OplogInterface;
 class ReplicationCoordinator;
+
 class ReplicationProcess;
 
 /**
@@ -247,7 +268,7 @@ public:
                  ReplicationProcess* replicationProcess,
                  ReplicationCoordinator* replicationCoordinator);
 
-    virtual ~RollbackImpl();
+    ~RollbackImpl() override;
 
     /**
      * Runs the rollback algorithm.
@@ -266,8 +287,9 @@ public:
     /**
      * Wrappers to expose private methods for testing.
      */
-    StatusWith<std::set<NamespaceString>> _namespacesForOp_forTest(const OplogEntry& oplogEntry) {
-        return _namespacesForOp(oplogEntry);
+    StatusWith<std::pair<std::set<NamespaceString>, std::set<UUID>>>
+    _namespacesAndUUIDsForOp_forTest(const OplogEntry& oplogEntry) {
+        return _namespacesAndUUIDsForOp(oplogEntry);
     }
 
     /**
@@ -434,7 +456,8 @@ private:
      * handle 'applyOps' oplog entries, since it assumes their sub operations have already been
      * extracted at a higher layer.
      */
-    StatusWith<std::set<NamespaceString>> _namespacesForOp(const OplogEntry& oplogEntry);
+    StatusWith<std::pair<std::set<NamespaceString>, std::set<UUID>>> _namespacesAndUUIDsForOp(
+        const OplogEntry& oplogEntry);
 
     /**
      * Persists rollback files to disk for each namespace that contains documents inserted or
