@@ -85,7 +85,7 @@ public:
      * Fetch a single document from the sync source using the UUID. Returns the namespace matching
      * the UUID on the sync source as well.
      */
-    virtual std::pair<BSONObj, NamespaceString> findOneByUUID(const std::string& db,
+    virtual std::pair<BSONObj, NamespaceString> findOneByUUID(const DatabaseName& db,
                                                               UUID uuid,
                                                               const BSONObj& filter) const = 0;
 
