@@ -27,16 +27,24 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/repl/split_horizon.h"
-
 #include <algorithm>
-#include <boost/optional.hpp>
+#include <boost/none.hpp>
+#include <cstddef>
 #include <iterator>
+#include <ostream>
+#include <vector>
 
-#include "mongo/stdx/utility.h"
-#include "mongo/unittest/unittest.h"
+#include <absl/container/flat_hash_map.h>
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/status.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/db/repl/split_horizon.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
 
 using namespace std::literals::string_literals;
 
