@@ -27,22 +27,42 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
+#include <boost/none.hpp>
+#include <fmt/format.h>
 #include <functional>
 #include <memory>
+#include <vector>
 
+#include <boost/move/utility_core.hpp>
+
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/db/baton.h"
 #include "mongo/db/cursor_id.h"
+#include "mongo/db/exec/document_value/value.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/repl/oplog_entry.h"
+#include "mongo/db/repl/oplog_entry_gen.h"
 #include "mongo/db/repl/sync_source_resolver.h"
-#include "mongo/db/repl/sync_source_selector.h"
 #include "mongo/db/repl/sync_source_selector_mock.h"
+#include "mongo/db/session/logical_session_id.h"
+#include "mongo/db/shard_id.h"
 #include "mongo/executor/network_interface_mock.h"
+#include "mongo/executor/remote_command_request.h"
 #include "mongo/executor/thread_pool_task_executor_test_fixture.h"
+#include "mongo/stdx/type_traits.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/bson_test_util.h"
+#include "mongo/unittest/framework.h"
 #include "mongo/unittest/task_executor_proxy.h"
-#include "mongo/unittest/unittest.h"
 #include "mongo/util/assert_util.h"
+#include "mongo/util/time_support.h"
+#include "mongo/util/uuid.h"
 
 namespace {
 
@@ -154,7 +174,7 @@ BSONObj makeCursorResponse(CursorId cursorId,
     {
         BSONObjBuilder cursorBob(bob.subobjStart("cursor"));
         cursorBob.append("id", cursorId);
-        cursorBob.append("ns", nss.toString());
+        cursorBob.append("ns", nss.toString_forTest());
         {
             BSONArrayBuilder batchBob(
                 cursorBob.subarrayStart(isFirstBatch ? "firstBatch" : "nextBatch"));
@@ -300,7 +320,7 @@ void _scheduleFirstOplogEntryFetcherResponse(executor::NetworkInterfaceMock* net
     ASSERT_TRUE(net->hasReadyRequests());
     auto request = net->scheduleSuccessfulResponse(makeCursorResponse(0, nss, docs));
     ASSERT_EQUALS(currentSyncSource, request.target);
-    ASSERT_EQUALS(NamespaceString::kRsOplogNamespace.db(), request.dbname);
+    ASSERT_EQUALS(NamespaceString::kRsOplogNamespace.dbName(), request.dbname);
     ASSERT_EQUALS(SyncSourceResolver::kFetcherTimeout, request.timeout);
     auto firstElement = request.cmdObj.firstElement();
     ASSERT_EQUALS("find"_sd, firstElement.fieldNameStringData());
@@ -356,7 +376,7 @@ void _scheduleRBIDResponse(executor::NetworkInterfaceMock* net,
     ASSERT_TRUE(net->hasReadyRequests());
     auto request = net->scheduleSuccessfulResponse(reply);
     ASSERT_EQUALS(currentSyncSource, request.target);
-    ASSERT_EQUALS("admin", request.dbname);
+    ASSERT_EQUALS(DatabaseName::kAdmin, request.dbname);
     ASSERT_EQUALS(SyncSourceResolver::kFetcherTimeout, request.timeout);
     ASSERT_BSONOBJ_EQ(BSON("replSetGetRBID" << 1), request.cmdObj);
     net->runReadyNetworkOperations();
@@ -682,7 +702,7 @@ void _scheduleRequiredOpTimeFetcherResponse(executor::NetworkInterfaceMock* net,
     ASSERT_TRUE(net->hasReadyRequests());
     auto request = net->scheduleSuccessfulResponse(makeCursorResponse(0, nss, docs));
     ASSERT_EQUALS(currentSyncSource, request.target);
-    ASSERT_EQUALS(NamespaceString::kRsOplogNamespace.db(), request.dbname);
+    ASSERT_EQUALS(NamespaceString::kRsOplogNamespace.dbName(), request.dbname);
     ASSERT_EQUALS(SyncSourceResolver::kFetcherTimeout, request.timeout);
     auto firstElement = request.cmdObj.firstElement();
     ASSERT_EQUALS("find"_sd, firstElement.fieldNameStringData());
