@@ -29,7 +29,14 @@
 
 #pragma once
 
+#include <functional>
+
+#include "mongo/base/status_with.h"
+#include "mongo/db/baton.h"
+#include "mongo/executor/remote_command_request.h"
+#include "mongo/executor/task_executor.h"
 #include "mongo/unittest/task_executor_proxy.h"
+#include "mongo/util/time_support.h"
 
 namespace mongo {
 namespace repl {
@@ -80,6 +87,10 @@ public:
         [](const executor::RemoteCommandRequestOnAny&) {
             return false;
         };
+
+    // Override to specify the ErrorCode calls to schedule should fail with when configured to do
+    // so.
+    ErrorCodes::Error failureCode = ErrorCodes::OperationFailed;
 };
 
 }  // namespace repl
