@@ -28,18 +28,39 @@
  */
 
 
-#include "mongo/platform/basic.h"
-
+#include <absl/container/node_hash_map.h>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+// IWYU pragma: no_include "ext/alloc_traits.h"
 #include <algorithm>
+#include <mutex>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/db/client.h"
+#include "mongo/db/database_name.h"
 #include "mongo/db/dbdirectclient.h"
+#include "mongo/db/dbmessage.h"
 #include "mongo/db/repl/cloner_utils.h"
+#include "mongo/db/repl/optime.h"
 #include "mongo/db/repl/tenant_all_database_cloner.h"
 #include "mongo/db/repl/tenant_database_cloner.h"
 #include "mongo/db/repl/tenant_migration_decoration.h"
+#include "mongo/db/tenant_id.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/platform/mutex.h"
 #include "mongo/rpc/get_status_from_command_result.h"
 #include "mongo/util/assert_util.h"
+#include "mongo/util/clock_source.h"
+#include "mongo/util/database_name_util.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/fail_point.h"
+#include "mongo/util/str.h"
 #include "mongo/util/string_map.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTenantMigration
@@ -99,8 +120,7 @@ BaseCloner::AfterStageBehavior TenantAllDatabaseCloner::listDatabasesStage() {
 
     BSONObj readResult;
     BSONObj cmd = ClonerUtils::buildMajorityWaitRequest(_operationTime);
-    getClient()->runCommand(
-        DatabaseName(boost::none, "admin"), cmd, readResult, QueryOption_SecondaryOk);
+    getClient()->runCommand(DatabaseName::kAdmin, cmd, readResult, QueryOption_SecondaryOk);
     uassertStatusOKWithContext(
         getStatusFromCommandResult(readResult),
         "TenantAllDatabaseCloner failed to get listDatabases result majority-committed");
@@ -163,7 +183,10 @@ BaseCloner::AfterStageBehavior TenantAllDatabaseCloner::listExistingDatabasesSta
         clonedDatabases.emplace_back(dbName);
 
         BSONObj res;
-        client.runCommand(DatabaseName(boost::none, dbName), BSON("dbStats" << 1), res);
+        client.runCommand(DatabaseNameUtil::deserialize(
+                              boost::none, dbName, SerializationContext::stateDefault()),
+                          BSON("dbStats" << 1),
+                          res);
         if (auto status = getStatusFromCommandResult(res); !status.isOK()) {
             LOGV2_WARNING(5522900,
                           "Skipping recording of data size metrics for database due to failure "
@@ -233,7 +256,10 @@ BaseCloner::AfterStageBehavior TenantAllDatabaseCloner::initializeStatsStage() {
     long long approxTotalDataSizeLeftOnRemote = 0;
     for (const auto& dbName : _databases) {
         BSONObj res;
-        getClient()->runCommand(DatabaseName(boost::none, dbName), BSON("dbStats" << 1), res);
+        getClient()->runCommand(DatabaseNameUtil::deserialize(
+                                    boost::none, dbName, SerializationContext::stateDefault()),
+                                BSON("dbStats" << 1),
+                                res);
         if (auto status = getStatusFromCommandResult(res); !status.isOK()) {
             LOGV2_WARNING(5426600,
                           "Skipping recording of data size metrics for database due to failure "
