@@ -28,10 +28,7 @@
  */
 
 
-#include "mongo/platform/basic.h"
-
 #include "mongo/db/repl/tenant_base_cloner.h"
-#include "mongo/logv2/log.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTenantMigration
 
