@@ -28,25 +28,62 @@
  */
 
 
-#include "mongo/platform/basic.h"
+#include <absl/container/node_hash_map.h>
+#include <boost/cstdint.hpp>
+#include <boost/none.hpp>
+#include <cstdint>
+#include <fmt/format.h>
+#include <list>
+#include <memory>
+#include <mutex>
+#include <utility>
 
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/status_with.h"
 #include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/client/dbclient_base.h"
+#include "mongo/client/read_preference.h"
+#include "mongo/db/catalog/collection.h"
 #include "mongo/db/catalog/collection_catalog.h"
 #include "mongo/db/catalog/document_validation.h"
-#include "mongo/db/commands/list_collections_filter.h"
-#include "mongo/db/db_raii.h"
+#include "mongo/db/client.h"
+#include "mongo/db/database_name.h"
 #include "mongo/db/dbdirectclient.h"
+#include "mongo/db/dbmessage.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/ops/single_write_result_gen.h"
 #include "mongo/db/ops/write_ops_exec.h"
+#include "mongo/db/ops/write_ops_gen.h"
+#include "mongo/db/query/find_command.h"
 #include "mongo/db/repl/cloner_utils.h"
-#include "mongo/db/repl/database_cloner_gen.h"
+#include "mongo/db/repl/read_concern_args.h"
+#include "mongo/db/repl/read_concern_level.h"
 #include "mongo/db/repl/repl_server_parameters_gen.h"
 #include "mongo/db/repl/tenant_collection_cloner.h"
 #include "mongo/db/repl/tenant_migration_decoration.h"
 #include "mongo/db/s/operation_sharding_state.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/redaction.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/platform/mutex.h"
 #include "mongo/rpc/get_status_from_command_result.h"
+#include "mongo/rpc/metadata.h"
 #include "mongo/rpc/metadata/repl_set_metadata.h"
 #include "mongo/util/assert_util.h"
+#include "mongo/util/clock_source.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/duration.h"
+#include "mongo/util/fail_point.h"
+#include "mongo/util/namespace_string_util.h"
+#include "mongo/util/scopeguard.h"
+#include "mongo/util/str.h"
+#include "mongo/util/string_map.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTenantMigration
 
@@ -83,7 +120,7 @@ TenantCollectionCloner::TenantCollectionCloner(const NamespaceString& sourceNss,
           "TenantCollectionCloner"_sd, sharedData, source, client, storageInterface, dbPool),
       _sourceNss(sourceNss),
       _collectionOptions(collectionOptions),
-      _sourceDbAndUuid(NamespaceString("UNINITIALIZED")),
+      _sourceDbAndUuid(NamespaceString::kEmpty),
       _collectionClonerBatchSize(collectionClonerBatchSize),
       _countStage("count", this, &TenantCollectionCloner::countStage),
       _checkIfDonorCollectionIsEmptyStage(
@@ -98,13 +135,15 @@ TenantCollectionCloner::TenantCollectionCloner(const NamespaceString& sourceNss,
                      kProgressMeterSecondsBetween,
                      kProgressMeterCheckInterval,
                      "documents copied",
-                     str::stream() << _sourceNss.toString() << " tenant collection clone progress"),
+                     str::stream() << NamespaceStringUtil::serialize(
+                                          _sourceNss, SerializationContext::stateDefault())
+                                   << " tenant collection clone progress"),
       _tenantId(tenantId) {
     invariant(sourceNss.isValid());
-    invariant(ClonerUtils::isNamespaceForTenant(sourceNss, tenantId));
+    invariant(sourceNss.isNamespaceForTenant(tenantId));
     invariant(collectionOptions.uuid);
     _sourceDbAndUuid = NamespaceStringOrUUID(sourceNss.dbName(), *collectionOptions.uuid);
-    _stats.ns = _sourceNss.ns().toString();
+    _stats.ns = NamespaceStringUtil::serialize(sourceNss, SerializationContext::stateDefault());
 }
 
 BaseCloner::ClonerStages TenantCollectionCloner::getStages() {
@@ -237,15 +276,14 @@ BaseCloner::AfterStageBehavior TenantCollectionCloner::listIndexesStage() {
             }
         },
         [&](const BSONObj& data) {
+            const auto fpNss = NamespaceStringUtil::parseFailPointData(data, "nss"_sd);
             // Only hang when cloning the specified collection, or if no collection was specified.
-            auto nss = data["nss"].str();
-            return nss.empty() || nss == _sourceNss.toString();
+            return fpNss.isEmpty() || fpNss == _sourceNss;
         });
 
     BSONObj readResult;
     BSONObj cmd = ClonerUtils::buildMajorityWaitRequest(_operationTime);
-    getClient()->runCommand(
-        DatabaseName(boost::none, "admin"), cmd, readResult, QueryOption_SecondaryOk);
+    getClient()->runCommand(DatabaseName::kAdmin, cmd, readResult, QueryOption_SecondaryOk);
     uassertStatusOKWithContext(
         getStatusFromCommandResult(readResult),
         "TenantCollectionCloner failed to get listIndexes result majority-committed");
@@ -277,7 +315,7 @@ BaseCloner::AfterStageBehavior TenantCollectionCloner::listIndexesStage() {
         ErrorCodes::IllegalOperation,
         str::stream() << "Found empty '_id' index spec but the collection is not specified with "
                          "'autoIndexId' as false, tenantId: "
-                      << _tenantId << ", namespace: " << this->_sourceNss,
+                      << _tenantId << ", namespace: " << this->_sourceNss.toStringForErrorMsg(),
         _collectionOptions.clusteredIndex || !_idIndexSpec.isEmpty() ||
             _collectionOptions.autoIndexId == CollectionOptions::NO);
 
@@ -301,13 +339,14 @@ BaseCloner::AfterStageBehavior TenantCollectionCloner::createCollectionStage() {
         uassert(5342500,
                 str::stream() << "Collection uuid" << getSourceUuid()
                               << " already exists but does not belong to tenant",
-                ClonerUtils::isNamespaceForTenant(collection->ns(), _tenantId));
+                collection->ns().isNamespaceForTenant(_tenantId));
         uassert(5342501,
                 str::stream() << "Collection uuid" << getSourceUuid()
                               << " already exists but does not belong to the same database",
-                collection->ns().db() == _sourceNss.db());
+                collection->ns().isEqualDb(_sourceNss));
         uassert(ErrorCodes::NamespaceExists,
-                str::stream() << "Tenant '" << _tenantId << "': collection '" << collection->ns()
+                str::stream() << "Tenant '" << _tenantId << "': collection '"
+                              << collection->ns().toStringForErrorMsg()
                               << "' already exists prior to data sync",
                 getSharedData()->getResumePhase() == ResumePhase::kDataSync);
 
@@ -513,17 +552,18 @@ void TenantCollectionCloner::handleNextBatch(DBClientCursor& cursor) {
             }
         },
         [&](const BSONObj& data) {
+            const auto fpNss = NamespaceStringUtil::parseFailPointData(data, "nss"_sd);
             // Only hang when cloning the specified collection, or if no collection was specified.
-            auto nss = data["nss"].str();
-            return nss.empty() || nss == _sourceNss.toString();
+            return fpNss.isEmpty() || fpNss == _sourceNss;
         });
 }
 
 
 void TenantCollectionCloner::insertDocuments(std::vector<BSONObj> docsToInsert) {
+    using namespace fmt::literals;
     invariant(docsToInsert.size(),
               "Document size can't be non-zero:: namespace: {}, tenantId: {}"_format(
-                  _sourceNss.toString(), _tenantId));
+                  toStringForLogging(_sourceNss), _tenantId));
 
     {
         stdx::lock_guard<Latch> lk(_mutex);
@@ -567,8 +607,8 @@ void TenantCollectionCloner::insertDocuments(std::vector<BSONObj> docsToInsert)
 }
 
 bool TenantCollectionCloner::isMyFailPoint(const BSONObj& data) const {
-    auto nss = data["nss"].str();
-    return (nss.empty() || nss == _sourceNss.toString()) && BaseCloner::isMyFailPoint(data);
+    const auto fpNss = NamespaceStringUtil::parseFailPointData(data, "nss"_sd);
+    return (fpNss.isEmpty() || fpNss == _sourceNss) && BaseCloner::isMyFailPoint(data);
 }
 
 TenantCollectionCloner::Stats TenantCollectionCloner::getStats() const {
