@@ -31,22 +31,47 @@
 
 #include <boost/filesystem.hpp>
 #include <boost/filesystem/fstream.hpp>
+#include <boost/filesystem/path.hpp>
+#include <boost/move/utility_core.hpp>
+#include <cstddef>
+#include <iosfwd>
 #include <memory>
+#include <string>
+#include <sys/types.h>
 #include <vector>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/client/dbclient_connection.h"
+#include "mongo/client/dbclient_cursor.h"
 #include "mongo/db/repl/base_cloner.h"
+#include "mongo/db/repl/storage_interface.h"
 #include "mongo/db/repl/task_runner.h"
 #include "mongo/db/repl/tenant_base_cloner.h"
 #include "mongo/db/repl/tenant_migration_shared_data.h"
+#include "mongo/executor/task_executor.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/concurrency/thread_pool.h"
+#include "mongo/util/functional.h"
+#include "mongo/util/net/hostandport.h"
 #include "mongo/util/progress_meter.h"
+#include "mongo/util/time_support.h"
+#include "mongo/util/uuid.h"
 
 namespace mongo::repl {
 
 class TenantFileCloner final : public TenantBaseCloner {
 public:
     struct Stats {
-        std::string filePath;
-        size_t fileSize;
+
+        explicit Stats(const std::string& filePath, size_t fileSize)
+            : filePath(filePath), fileSize(fileSize) {}
+
+        const std::string filePath;
+        const size_t fileSize;
         Date_t start;
         Date_t end;
         size_t receivedBatches{0};
@@ -85,7 +110,7 @@ public:
                      StorageInterface* storageInterface,
                      ThreadPool* dbPool);
 
-    virtual ~TenantFileCloner() = default;
+    ~TenantFileCloner() override = default;
 
     /**
      * Waits for any file system work to finish or fail.
@@ -191,8 +216,8 @@ private:
     const UUID _backupId;                    // (R)
     const UUID _migrationId;                 // (R)
     const std::string _remoteFileName;       // (R)
-    size_t _remoteFileSize;                  // (R)
     const std::string _relativePathString;   // (R)
+    Stats _stats;                            // (M)
     boost::filesystem::path _localFilePath;  // (X)
 
     TenantFileClonerQueryStage _queryStage;  // (R)
@@ -211,7 +236,6 @@ private:
     ScheduleFsWorkFn _scheduleFsWorkFn;  // (R)
 
     ProgressMeter _progressMeter;  // (X) progress meter for this instance.
-    Stats _stats;                  // (M)
 
     static constexpr int kProgressMeterSecondsBetween = 60;
     static constexpr int kProgressMeterCheckInterval = 128;
