@@ -50,8 +50,8 @@ public:
      */
     enum class BlockerType { kDonor, kRecipient };
 
-    TenantMigrationAccessBlocker(BlockerType type, MigrationProtocolEnum protocol)
-        : _type(type), _protocol(protocol) {}
+    TenantMigrationAccessBlocker(BlockerType type, const UUID& migrationId)
+        : _type(type), _migrationId(migrationId) {}
     virtual ~TenantMigrationAccessBlocker() = default;
 
     //
@@ -63,8 +63,8 @@ public:
 
 
     virtual Status checkIfLinearizableReadWasAllowed(OperationContext* opCtx) = 0;
-    virtual SharedSemiFuture<void> getCanReadFuture(OperationContext* opCtx,
-                                                    StringData command) = 0;
+    virtual SharedSemiFuture<void> getCanRunCommandFuture(OperationContext* opCtx,
+                                                          StringData command) = 0;
 
     //
     // Called by index build user threads before acquiring an index build slot, and again right
@@ -72,6 +72,16 @@ public:
     //
     virtual Status checkIfCanBuildIndex() = 0;
 
+    /**
+     * Checks if opening a new change stream should block.
+     */
+    virtual Status checkIfCanOpenChangeStream() = 0;
+
+    /**
+     * Checks if getMores for change streams should fail.
+     */
+    virtual Status checkIfCanGetMoreChangeStream() = 0;
+
     // We suspend TTL deletions at the recipient side to avoid the race when a document is updated
     // at the donor side, which may prevent it from being garbage collected by TTL, while the
     // recipient side document is deleted by the TTL. The donor side update will fail to propagate
@@ -90,9 +100,11 @@ public:
     virtual void appendInfoForServerStatus(BSONObjBuilder* builder) const = 0;
 
     /**
-     * Returns structured info with tenant id and connection string.
+     * Returns structured info.
      */
-    virtual BSONObj getDebugInfo() const = 0;
+    BSONObj getDebugInfo() const {
+        return BSON("migrationId" << _migrationId.toString());
+    }
 
     /**
      * Updates the runtime statistics for the number of tenant migration errors that have been
@@ -108,15 +120,15 @@ public:
     }
 
     /**
-     * Returns the protocol of the access blocker.
+     * Returns the migration id of access blocker.
      */
-    MigrationProtocolEnum getProtocol() {
-        return _protocol;
+    const UUID& getMigrationId() const {
+        return _migrationId;
     }
 
 private:
     const BlockerType _type;
-    const MigrationProtocolEnum _protocol;
+    const UUID _migrationId;
 };
 
 }  // namespace mongo
