@@ -28,19 +28,58 @@
  */
 
 
-#include "mongo/platform/basic.h"
+#include <algorithm>
+#include <boost/optional.hpp>
+#include <boost/smart_ptr.hpp>
+#include <memory>
+#include <mutex>
+#include <string>
+#include <utility>
+#include <vector>
 
-#include "mongo/db/client.h"
-#include "mongo/db/commands/tenant_migration_donor_cmds_gen.h"
+#include <absl/container/node_hash_map.h>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/db/logical_time.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/repl/optime.h"
 #include "mongo/db/repl/read_concern_args.h"
+#include "mongo/db/repl/read_concern_level.h"
 #include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/repl/storage_interface.h"
+#include "mongo/db/repl/tenant_migration_access_blocker.h"
+#include "mongo/db/repl/tenant_migration_access_blocker_registry.h"
 #include "mongo/db/repl/tenant_migration_access_blocker_util.h"
 #include "mongo/db/repl/tenant_migration_conflict_info.h"
+#include "mongo/db/repl/tenant_migration_donor_access_blocker.h"
+#include "mongo/db/service_context.h"
+#include "mongo/executor/task_executor.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/platform/mutex.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/cancellation.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/duration.h"
 #include "mongo/util/fail_point.h"
+#include "mongo/util/future.h"
+#include "mongo/util/future_impl.h"
 #include "mongo/util/future_util.h"
+#include "mongo/util/str.h"
+#include "mongo/util/string_map.h"
+#include "mongo/util/time_support.h"
+#include "mongo/util/uuid.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTenantMigration
 
@@ -135,13 +174,13 @@ Status TenantMigrationDonorAccessBlocker::waitUntilCommittedOrAborted(OperationC
     MONGO_UNREACHABLE;
 }
 
-SharedSemiFuture<void> TenantMigrationDonorAccessBlocker::getCanReadFuture(OperationContext* opCtx,
-                                                                           StringData command) {
+SharedSemiFuture<void> TenantMigrationDonorAccessBlocker::getCanRunCommandFuture(
+    OperationContext* opCtx, StringData command) {
     // Exclude internal client requests
-    if (tenant_migration_access_blocker::shouldExcludeRead(opCtx)) {
+    if (tenant_migration_access_blocker::shouldExclude(opCtx)) {
         LOGV2_DEBUG(6397500,
                     1,
-                    "Internal tenant read got excluded from the MTAB filtering",
+                    "Internal tenant command got excluded from the MTAB filtering",
                     "migrationId"_attr = getMigrationId(),
                     "opId"_attr = opCtx->getOpID());
         return SharedSemiFuture<void>();
@@ -190,7 +229,7 @@ SharedSemiFuture<void> TenantMigrationDonorAccessBlocker::getCanReadFuture(Opera
 
             return SharedSemiFuture<void>(
                 Status(ErrorCodes::TenantMigrationCommitted,
-                       "Read must be re-routed to the new owner of this tenant"));
+                       "Command must be re-routed to the new owner of this tenant"));
 
         default:
             MONGO_UNREACHABLE;
@@ -224,6 +263,41 @@ Status TenantMigrationDonorAccessBlocker::checkIfCanBuildIndex() {
     MONGO_UNREACHABLE;
 }
 
+Status TenantMigrationDonorAccessBlocker::checkIfCanOpenChangeStream() {
+    stdx::lock_guard<Latch> lg(_mutex);
+    switch (_state.getState()) {
+        case BlockerState::State::kAllow:
+            return Status::OK();
+        case BlockerState::State::kBlockWrites:
+        case BlockerState::State::kBlockWritesAndReads:
+            return {TenantMigrationConflictInfo(getMigrationId(), shared_from_this()),
+                    "Change stream must wait for commit or abort to get a safe start time"};
+        case BlockerState::State::kReject:
+            // At this point checkIfCanReadOrBlock should have blocked this at the command level.
+            return {ErrorCodes::TenantMigrationCommitted,
+                    "Change stream must be resumed on the new owner of this tenant"};
+        case BlockerState::State::kAborted:
+            return Status::OK();
+    }
+    MONGO_UNREACHABLE;
+}
+
+Status TenantMigrationDonorAccessBlocker::checkIfCanGetMoreChangeStream() {
+    stdx::lock_guard<Latch> lg(_mutex);
+    switch (_state.getState()) {
+        case BlockerState::State::kAllow:
+        case BlockerState::State::kBlockWrites:
+        case BlockerState::State::kBlockWritesAndReads:
+            return Status::OK();
+        case BlockerState::State::kReject:
+            return {ErrorCodes::ResumeTenantChangeStream,
+                    "Change stream must be resumed on the new owner of this tenant"};
+        case BlockerState::State::kAborted:
+            return Status::OK();
+    }
+    MONGO_UNREACHABLE;
+}
+
 void TenantMigrationDonorAccessBlocker::startBlockingWrites() {
     stdx::lock_guard<Latch> lg(_mutex);
 
