@@ -30,36 +30,100 @@
 
 #include "mongo/db/repl/tenant_migration_donor_service.h"
 
+#include <boost/none.hpp>
+#include <boost/optional.hpp>
+#include <boost/smart_ptr.hpp>
+#include <functional>
+#include <mutex>
+#include <tuple>
+#include <type_traits>
+
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/checked_cast.h"
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status_with.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/timestamp.h"
 #include "mongo/client/async_remote_command_targeter_adapter.h"
 #include "mongo/client/connection_string.h"
-#include "mongo/client/replica_set_monitor.h"
-#include "mongo/config.h"
+#include "mongo/client/remote_command_retry_scheduler.h"
+#include "mongo/client/remote_command_targeter_rs.h"
+#include "mongo/config.h"  // IWYU pragma: keep
+#include "mongo/db//shard_role.h"
+#include "mongo/db/auth/authorization_session.h"
+#include "mongo/db/catalog/collection.h"
 #include "mongo/db/catalog/collection_write_path.h"
-#include "mongo/db/commands/tenant_migration_donor_cmds_gen.h"
+#include "mongo/db/catalog/local_oplog_info.h"
+#include "mongo/db/catalog_raii.h"
+#include "mongo/db/client.h"
 #include "mongo/db/commands/tenant_migration_recipient_cmds_gen.h"
 #include "mongo/db/concurrency/exception_util.h"
-#include "mongo/db/db_raii.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
+#include "mongo/db/database_name.h"
+#include "mongo/db/dbdirectclient.h"
 #include "mongo/db/dbhelpers.h"
 #include "mongo/db/index_builds_coordinator.h"
+#include "mongo/db/keys_collection_document_gen.h"
 #include "mongo/db/keys_collection_util.h"
+#include "mongo/db/ops/update_result.h"
 #include "mongo/db/persistent_task_store.h"
 #include "mongo/db/query/find_command.h"
+#include "mongo/db/record_id.h"
+#include "mongo/db/repl/oplog.h"
+#include "mongo/db/repl/read_concern_args.h"
+#include "mongo/db/repl/read_concern_level.h"
+#include "mongo/db/repl/repl_client_info.h"
 #include "mongo/db/repl/repl_server_parameters_gen.h"
 #include "mongo/db/repl/repl_set_config.h"
+#include "mongo/db/repl/replication_coordinator.h"
+#include "mongo/db/repl/tenant_migration_access_blocker_registry.h"
 #include "mongo/db/repl/tenant_migration_access_blocker_util.h"
 #include "mongo/db/repl/tenant_migration_donor_access_blocker.h"
 #include "mongo/db/repl/tenant_migration_state_machine_gen.h"
 #include "mongo/db/repl/tenant_migration_statistics.h"
+#include "mongo/db/repl/tenant_migration_util.h"
 #include "mongo/db/repl/wait_for_majority_service.h"
+#include "mongo/db/server_options.h"
+#include "mongo/db/storage/recovery_unit.h"
+#include "mongo/db/storage/snapshot.h"
+#include "mongo/db/storage/storage_engine.h"
+#include "mongo/db/storage/write_unit_of_work.h"
+#include "mongo/db/transaction_resources.h"
+#include "mongo/db/write_concern_options.h"
 #include "mongo/executor/async_rpc.h"
+#include "mongo/executor/async_rpc_error_info.h"
 #include "mongo/executor/async_rpc_retry_policy.h"
+#include "mongo/executor/async_rpc_targeter.h"
 #include "mongo/executor/connection_pool.h"
+#include "mongo/executor/network_connection_hook.h"
 #include "mongo/executor/network_interface_factory.h"
+#include "mongo/executor/remote_command_request.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/platform/compiler.h"
 #include "mongo/rpc/get_status_from_command_result.h"
 #include "mongo/rpc/metadata/egress_metadata_hook_list.h"
+#include "mongo/rpc/metadata/metadata_hook.h"
+#include "mongo/s/database_version.h"
+#include "mongo/s/shard_version.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/cancellation.h"
+#include "mongo/util/clock_source.h"
+#include "mongo/util/concurrency/with_lock.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/duration.h"
+#include "mongo/util/fail_point.h"
 #include "mongo/util/future_util.h"
+#include "mongo/util/net/hostandport.h"
+#include "mongo/util/net/ssl_options.h"
+#include "mongo/util/out_of_line_executor.h"
+#include "mongo/util/str.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTenantMigration
 
@@ -131,7 +195,7 @@ public:
         : RetryWithBackoffOnErrorCategories(b), _protocol{p} {}
 
     /** Returns true if we should retry sending SyncData given the error */
-    bool recordAndEvaluateRetry(Status status) {
+    bool recordAndEvaluateRetry(Status status) override {
         if (_protocol == MigrationProtocolEnum::kShardMerge || status.isOK()) {
             return false;
         }
@@ -227,7 +291,8 @@ void TenantMigrationDonorService::checkIfConflictsWithOtherInstances(
                 existingIsAborted);
 
         // Any existing migration for this tenant must be aborted and garbage-collectable.
-        if (existingTypedInstance->getTenantId() == stateDoc.getTenantId()) {
+        if (stateDoc.getTenantId() &&
+            existingTypedInstance->getTenantId() == *stateDoc.getTenantId()) {
             uassert(ErrorCodes::ConflictingOperationInProgress,
                     str::stream() << "tenant " << stateDoc.getTenantId() << " is already migrating",
                     existingIsAborted);
@@ -326,19 +391,12 @@ TenantMigrationDonorService::Instance::Instance(ServiceContext* const serviceCon
       _instanceName(kServiceName + "-" + _stateDoc.getId().toString()),
       _recipientUri(
           uassertStatusOK(MongoURI::parse(_stateDoc.getRecipientConnectionString().toString()))),
-      _tenantId(_stateDoc.getTenantId()),
+      _tenantId(_stateDoc.getTenantId() ? *_stateDoc.getTenantId() : ""),
       _tenantIds(_stateDoc.getTenantIds() ? *_stateDoc.getTenantIds() : std::vector<TenantId>()),
       _protocol(_stateDoc.getProtocol().value_or(MigrationProtocolEnum::kMultitenantMigrations)),
       _recipientConnectionString(_stateDoc.getRecipientConnectionString()),
       _readPreference(_stateDoc.getReadPreference()),
-      _migrationUuid(_stateDoc.getId()),
-      _donorCertificateForRecipient(_stateDoc.getDonorCertificateForRecipient()),
-      _recipientCertificateForDonor(_stateDoc.getRecipientCertificateForDonor()),
-      _sslMode(repl::tenantMigrationDisableX509Auth ? transport::kGlobalSSLMode
-                                                    : transport::kEnableSSL) {
-
-    _recipientCmdExecutor = _makeRecipientCmdExecutor();
-    _recipientCmdExecutor->startup();
+      _migrationUuid(_stateDoc.getId()) {
 
     if (_stateDoc.getState() > TenantMigrationDonorStateEnum::kUninitialized) {
         // The migration was resumed on stepup.
@@ -349,7 +407,10 @@ TenantMigrationDonorService::Instance::Instance(ServiceContext* const serviceCon
             auto errmsg = abortReasonBson["errmsg"].String();
             _abortReason = Status(ErrorCodes::Error(code), errmsg);
         }
-        _durableState = DurableState{_stateDoc.getState(), _abortReason, _stateDoc.getExpireAt()};
+        _durableState = DurableState{_stateDoc.getState(),
+                                     _stateDoc.getAbortReason(),
+                                     _stateDoc.getExpireAt(),
+                                     _stateDoc.getBlockTimestamp()};
 
         _initialDonorStateDurablePromise.emplaceValue();
 
@@ -364,70 +425,6 @@ TenantMigrationDonorService::Instance::~Instance() {
     stdx::lock_guard<Latch> lg(_mutex);
     invariant(_initialDonorStateDurablePromise.getFuture().isReady());
     invariant(_receiveDonorForgetMigrationPromise.getFuture().isReady());
-
-    // Unlike the TenantMigrationDonorService's scoped task executor which is shut down on stepdown
-    // and joined on stepup, _recipientCmdExecutor is only shut down and joined when the Instance
-    // is destroyed. This is safe since ThreadPoolTaskExecutor::shutdown() only cancels the
-    // outstanding work on the task executor which the cancellation token will already do, and the
-    // Instance will be destroyed on stepup so this is equivalent to joining the task executor on
-    // stepup.
-    _recipientCmdExecutor->shutdown();
-    _recipientCmdExecutor->join();
-}
-
-std::shared_ptr<executor::ThreadPoolTaskExecutor>
-TenantMigrationDonorService::Instance::_makeRecipientCmdExecutor() {
-    ThreadPool::Options threadPoolOptions(_getRecipientCmdThreadPoolLimits());
-    threadPoolOptions.threadNamePrefix = _instanceName + "-";
-    threadPoolOptions.poolName = _instanceName + "ThreadPool";
-    threadPoolOptions.onCreateThread = [this](const std::string& threadName) {
-        Client::initThread(threadName.c_str());
-        auto client = Client::getCurrent();
-        AuthorizationSession::get(*client)->grantInternalAuthorization(&cc());
-
-        // Ideally, we should also associate the client created by _recipientCmdExecutor with the
-        // TenantMigrationDonorService to make the opCtxs created by the task executor get
-        // registered in the TenantMigrationDonorService, and killed on stepdown. But that would
-        // require passing the pointer to the TenantMigrationService into the Instance and making
-        // constructInstance not const so we can set the client's decoration here. Right now there
-        // is no need for that since the task executor is only used with scheduleRemoteCommand and
-        // no opCtx will be created (the cancellation token is responsible for canceling the
-        // outstanding work on the task executor).
-        stdx::lock_guard<Client> lk(*client);
-        client->setSystemOperationKillableByStepdown(lk);
-    };
-
-    auto hookList = std::make_unique<rpc::EgressMetadataHookList>();
-
-    auto connPoolOptions = executor::ConnectionPool::Options();
-    if (_donorCertificateForRecipient) {
-        invariant(!repl::tenantMigrationDisableX509Auth);
-        invariant(_recipientCertificateForDonor);
-        invariant(_sslMode == transport::kEnableSSL);
-#ifdef MONGO_CONFIG_SSL
-        uassert(ErrorCodes::IllegalOperation,
-                "Cannot run tenant migration with x509 authentication as SSL is not enabled",
-                getSSLGlobalParams().sslMode.load() != SSLParams::SSLMode_disabled);
-        auto donorSSLClusterPEMPayload =
-            _donorCertificateForRecipient->getCertificate().toString() + "\n" +
-            _donorCertificateForRecipient->getPrivateKey().toString();
-        connPoolOptions.transientSSLParams = TransientSSLParams{
-            _recipientUri.connectionString(), std::move(donorSSLClusterPEMPayload)};
-#else
-        // If SSL is not supported, the donorStartMigration command should have failed certificate
-        // field validation.
-        MONGO_UNREACHABLE;
-#endif
-    } else {
-        invariant(repl::tenantMigrationDisableX509Auth);
-        invariant(!_recipientCertificateForDonor);
-        invariant(_sslMode == transport::kGlobalSSLMode);
-    }
-
-    return std::make_shared<executor::ThreadPoolTaskExecutor>(
-        std::make_unique<ThreadPool>(threadPoolOptions),
-        executor::makeNetworkInterface(
-            _instanceName + "-Network", nullptr, std::move(hookList), connPoolOptions));
 }
 
 boost::optional<BSONObj> TenantMigrationDonorService::Instance::reportForCurrentOp(
@@ -444,7 +441,14 @@ boost::optional<BSONObj> TenantMigrationDonorService::Instance::reportForCurrent
     _migrationUuid.appendToBuilder(&bob, "instanceID"_sd);
     if (getProtocol() == MigrationProtocolEnum::kMultitenantMigrations) {
         bob.append("tenantId", _tenantId);
+    } else {
+        invariant(_stateDoc.getTenantIds());
+        BSONArrayBuilder arrayBuilder(bob.subarrayStart("tenantIds"));
+        for (const auto& tenantId : *_stateDoc.getTenantIds()) {
+            tenantId.serializeToBSON(&arrayBuilder);
+        }
     }
+
     bob.append("recipientConnectionString", _recipientConnectionString);
     bob.append("readPreference", _readPreference.toInnerBSON());
     bob.append("receivedCancellation", _abortRequested);
@@ -467,7 +471,7 @@ boost::optional<BSONObj> TenantMigrationDonorService::Instance::reportForCurrent
         bob.append("blockTimestamp", *_stateDoc.getBlockTimestamp());
     }
     if (_stateDoc.getCommitOrAbortOpTime()) {
-        _stateDoc.getCommitOrAbortOpTime()->append(&bob, "commitOrAbortOpTime");
+        _stateDoc.getCommitOrAbortOpTime()->append("commitOrAbortOpTime", &bob);
     }
     if (_stateDoc.getAbortReason()) {
         bob.append("abortReason", *_stateDoc.getAbortReason());
@@ -475,21 +479,36 @@ boost::optional<BSONObj> TenantMigrationDonorService::Instance::reportForCurrent
     return bob.obj();
 }
 
+TenantMigrationDonorDocument TenantMigrationDonorService::Instance::getStateDoc() const {
+    stdx::lock_guard lk(_mutex);
+    return _stateDoc;
+}
+
 void TenantMigrationDonorService::Instance::checkIfOptionsConflict(const BSONObj& options) const {
     auto stateDoc = tenant_migration_access_blocker::parseDonorStateDocument(options);
 
     invariant(stateDoc.getId() == _migrationUuid);
     invariant(stateDoc.getProtocol());
 
-    if (stateDoc.getProtocol().value() != _protocol || stateDoc.getTenantId() != _tenantId ||
+    auto tenantIdsMatch = [&] {
+        switch (_protocol) {
+            case MigrationProtocolEnum::kShardMerge:
+                invariant(stateDoc.getTenantIds());
+                return *stateDoc.getTenantIds() == _tenantIds;
+            case MigrationProtocolEnum::kMultitenantMigrations:
+                invariant(stateDoc.getTenantId());
+                return *stateDoc.getTenantId() == _tenantId;
+        }
+        MONGO_UNREACHABLE;
+    };
+
+    if (stateDoc.getProtocol().value() != _protocol || !tenantIdsMatch() ||
         stateDoc.getRecipientConnectionString() != _recipientConnectionString ||
-        !stateDoc.getReadPreference().equals(_readPreference) ||
-        stateDoc.getDonorCertificateForRecipient() != _donorCertificateForRecipient ||
-        stateDoc.getRecipientCertificateForDonor() != _recipientCertificateForDonor) {
+        !stateDoc.getReadPreference().equals(_readPreference)) {
         uasserted(ErrorCodes::ConflictingOperationInProgress,
                   str::stream() << "Found active migration for migrationId \""
                                 << _migrationUuid.toBSON() << "\" with different options "
-                                << tenant_migration_util::redactStateDoc(_stateDoc.toBSON()));
+                                << tenant_migration_util::redactStateDoc(getStateDoc().toBSON()));
     }
 }
 
@@ -541,10 +560,17 @@ ExecutorFuture<repl::OpTime> TenantMigrationDonorService::Instance::_insertState
 
                pauseTenantMigrationBeforeInsertingDonorStateDoc.pauseWhileSet(opCtx);
 
-               AutoGetCollection collection(opCtx, _stateDocumentsNS, MODE_IX);
+               auto collection =
+                   acquireCollection(opCtx,
+                                     CollectionAcquisitionRequest(
+                                         _stateDocumentsNS,
+                                         PlacementConcern{boost::none, ShardVersion::UNSHARDED()},
+                                         repl::ReadConcernArgs::get(opCtx),
+                                         AcquisitionPrerequisites::kWrite),
+                                     MODE_IX);
 
                writeConflictRetry(
-                   opCtx, "TenantMigrationDonorInsertStateDoc", _stateDocumentsNS.ns(), [&] {
+                   opCtx, "TenantMigrationDonorInsertStateDoc", _stateDocumentsNS, [&] {
                        const auto filter =
                            BSON(TenantMigrationDonorDocument::kIdFieldName << _migrationUuid);
                        const auto updateMod = [&]() {
@@ -552,7 +578,7 @@ ExecutorFuture<repl::OpTime> TenantMigrationDonorService::Instance::_insertState
                            return BSON("$setOnInsert" << _stateDoc.toBSON());
                        }();
                        auto updateResult = Helpers::upsert(
-                           opCtx, _stateDocumentsNS, filter, updateMod, /*fromMigrate=*/false);
+                           opCtx, collection, filter, updateMod, /*fromMigrate=*/false);
 
                        // '$setOnInsert' update operator can never modify an existing on-disk state
                        // doc.
@@ -564,7 +590,7 @@ ExecutorFuture<repl::OpTime> TenantMigrationDonorService::Instance::_insertState
         .until([&](StatusWith<repl::OpTime> swOpTime) {
             if (swOpTime.getStatus().code() == ErrorCodes::ConflictingServerlessOperation) {
                 LOGV2(6531508,
-                      "Tenant migration completed due to serverless lock error",
+                      "Tenant migration failed to start due to serverless lock error",
                       "id"_attr = _migrationUuid,
                       "status"_attr = swOpTime.getStatus());
                 uassertStatusOK(swOpTime);
@@ -594,17 +620,19 @@ ExecutorFuture<repl::OpTime> TenantMigrationDonorService::Instance::_updateState
                AutoGetCollection collection(opCtx, _stateDocumentsNS, MODE_IX);
 
                uassert(ErrorCodes::NamespaceNotFound,
-                       str::stream() << _stateDocumentsNS.ns() << " does not exist",
+                       str::stream()
+                           << _stateDocumentsNS.toStringForErrorMsg() << " does not exist",
                        collection);
 
                writeConflictRetry(
-                   opCtx, "TenantMigrationDonorUpdateStateDoc", _stateDocumentsNS.ns(), [&] {
+                   opCtx, "TenantMigrationDonorUpdateStateDoc", _stateDocumentsNS, [&] {
                        WriteUnitOfWork wuow(opCtx);
 
                        const auto originalRecordId = Helpers::findOne(
                            opCtx, collection.getCollection(), originalStateDocBson);
                        const auto originalSnapshot = Snapshotted<BSONObj>(
-                           opCtx->recoveryUnit()->getSnapshotId(), originalStateDocBson);
+                           shard_role_details::getRecoveryUnit(opCtx)->getSnapshotId(),
+                           originalStateDocBson);
                        invariant(!originalRecordId.isNull());
 
                        if (nextState == TenantMigrationDonorStateEnum::kBlocking) {
@@ -619,11 +647,12 @@ ExecutorFuture<repl::OpTime> TenantMigrationDonorService::Instance::_updateState
                                mtab->startBlockingWrites();
                            }
 
-                           opCtx->recoveryUnit()->onRollback([mtabVector](OperationContext*) {
-                               for (auto& mtab : mtabVector) {
-                                   mtab->rollBackStartBlocking();
-                               }
-                           });
+                           shard_role_details::getRecoveryUnit(opCtx)->onRollback(
+                               [mtabVector](OperationContext*) {
+                                   for (auto& mtab : mtabVector) {
+                                       mtab->rollBackStartBlocking();
+                                   }
+                               });
                        }
 
                        // Reserve an opTime for the write.
@@ -710,12 +739,19 @@ TenantMigrationDonorService::Instance::_markStateDocAsGarbageCollectable(
 
                pauseTenantMigrationDonorBeforeMarkingStateGarbageCollectable.pauseWhileSet(opCtx);
 
-               AutoGetCollection collection(opCtx, _stateDocumentsNS, MODE_IX);
+               auto collection =
+                   acquireCollection(opCtx,
+                                     CollectionAcquisitionRequest(
+                                         _stateDocumentsNS,
+                                         PlacementConcern{boost::none, ShardVersion::UNSHARDED()},
+                                         repl::ReadConcernArgs::get(opCtx),
+                                         AcquisitionPrerequisites::kWrite),
+                                     MODE_IX);
 
                writeConflictRetry(
                    opCtx,
                    "TenantMigrationDonorMarkStateDocAsGarbageCollectable",
-                   _stateDocumentsNS.ns(),
+                   _stateDocumentsNS,
                    [&] {
                        const auto filter =
                            BSON(TenantMigrationDonorDocument::kIdFieldName << _migrationUuid);
@@ -724,7 +760,7 @@ TenantMigrationDonorService::Instance::_markStateDocAsGarbageCollectable(
                            return _stateDoc.toBSON();
                        }();
                        auto updateResult = Helpers::upsert(
-                           opCtx, _stateDocumentsNS, filter, updateMod, /*fromMigrate=*/false);
+                           opCtx, collection, filter, updateMod, /*fromMigrate=*/false);
 
                        invariant(updateResult.numDocsModified == 1);
                    });
@@ -760,11 +796,10 @@ ExecutorFuture<void> TenantMigrationDonorService::Instance::_waitForMajorityWrit
     repl::OpTime opTime,
     const CancellationToken& token) {
     return WaitForMajorityService::get(_serviceContext)
-        .waitUntilMajority(std::move(opTime), token)
+        .waitUntilMajorityForWrite(std::move(opTime), token)
         .thenRunOn(**executor)
         .then([this, self = shared_from_this()] {
             stdx::lock_guard<Latch> lg(_mutex);
-            boost::optional<Status> abortReason;
             switch (_stateDoc.getState()) {
                 case TenantMigrationDonorStateEnum::kAbortingIndexBuilds:
                     setPromiseOkIfNotReady(lg, _initialDonorStateDurablePromise);
@@ -772,17 +807,16 @@ ExecutorFuture<void> TenantMigrationDonorService::Instance::_waitForMajorityWrit
                 case TenantMigrationDonorStateEnum::kDataSync:
                 case TenantMigrationDonorStateEnum::kBlocking:
                 case TenantMigrationDonorStateEnum::kCommitted:
-                    break;
                 case TenantMigrationDonorStateEnum::kAborted:
-                    invariant(_abortReason);
-                    abortReason = _abortReason;
                     break;
                 default:
                     MONGO_UNREACHABLE;
             }
 
-            _durableState =
-                DurableState{_stateDoc.getState(), std::move(abortReason), _stateDoc.getExpireAt()};
+            _durableState = DurableState{_stateDoc.getState(),
+                                         _stateDoc.getAbortReason(),
+                                         _stateDoc.getExpireAt(),
+                                         _stateDoc.getBlockTimestamp()};
         });
 }
 
@@ -798,7 +832,6 @@ ExecutorFuture<void> TenantMigrationDonorService::Instance::_sendRecipientSyncDa
 
     MigrationRecipientCommonData commonData(
         _migrationUuid, donorConnString.toString(), _readPreference);
-    commonData.setRecipientCertificateForDonor(_recipientCertificateForDonor);
     if (_protocol == MigrationProtocolEnum::kMultitenantMigrations) {
         commonData.setTenantId(boost::optional<StringData>(_tenantId));
     } else {
@@ -820,12 +853,28 @@ ExecutorFuture<void> TenantMigrationDonorService::Instance::_sendRecipientSyncDa
     auto retryPolicy =
         std::make_shared<RecipientSyncDataRetryPolicy>(getProtocol(), kExponentialBackoff);
     auto options =
-        std::make_shared<RecipientSyncDataRPCOptions>(request, **exec, token, retryPolicy);
+        std::make_shared<RecipientSyncDataRPCOptions>(**exec, token, request, retryPolicy);
     auto cmdRes = async_rpc::sendCommand(options, _serviceContext, std::move(asyncTargeter));
-    return std::move(cmdRes).ignoreValue().onError([](Status status) {
-        return async_rpc::unpackRPCStatusIgnoringWriteConcernAndWriteErrors(status).addContext(
-            "Tenant migration recipient command failed");
-    });
+    return std::move(cmdRes).ignoreValue().onError(
+        [returnAfterReachingDonorTs =
+             request.getReturnAfterReachingDonorTimestamp().has_value()](Status status) {
+            std::stringstream errMsg;
+            errMsg << "'recipientSyncData' command";
+            if (returnAfterReachingDonorTs)
+                errMsg << " with "
+                       << RecipientSyncData::kReturnAfterReachingDonorTimestampFieldName;
+            errMsg << " failed";
+
+            auto underlyingError =
+                async_rpc::unpackRPCStatusIgnoringWriteConcernAndWriteErrors(status);
+            if (ErrorCodes::isNotPrimaryError(underlyingError) ||
+                ErrorCodes::isShutdownError(underlyingError)) {
+                errMsg << " :: Merge recipient service interrupted; not resilient to stepdowns or "
+                          "node restarts";
+                return Status(ErrorCodes::TenantMigrationAborted, errMsg.str());
+            }
+            return underlyingError.addContext(errMsg.str());
+        });
 }
 
 ExecutorFuture<void> TenantMigrationDonorService::Instance::_sendRecipientForgetMigrationCommand(
@@ -841,7 +890,6 @@ ExecutorFuture<void> TenantMigrationDonorService::Instance::_sendRecipientForget
 
     MigrationRecipientCommonData commonData(
         _migrationUuid, donorConnString.toString(), _readPreference);
-    commonData.setRecipientCertificateForDonor(_recipientCertificateForDonor);
     {
         stdx::lock_guard<Latch> lg(_mutex);
         if (_protocol == MigrationProtocolEnum::kMultitenantMigrations) {
@@ -863,11 +911,11 @@ ExecutorFuture<void> TenantMigrationDonorService::Instance::_sendRecipientForget
         kPrimaryOnlyReadPreference, recipientTargeterRS);
     auto retryPolicy = std::make_shared<RecipientForgetMigrationRetryPolicy>(kExponentialBackoff);
     auto options =
-        std::make_shared<RecipientForgetMigrationRPCOptions>(request, **exec, token, retryPolicy);
+        std::make_shared<RecipientForgetMigrationRPCOptions>(**exec, token, request, retryPolicy);
     auto cmdRes = async_rpc::sendCommand(options, _serviceContext, std::move(asyncTargeter));
     return std::move(cmdRes).ignoreValue().onError([](Status status) {
         return async_rpc::unpackRPCStatusIgnoringWriteConcernAndWriteErrors(status).addContext(
-            "Tenant migration recipient command failed");
+            "'recipientForgetMigration' command failed");
     });
 }
 
@@ -907,6 +955,13 @@ SemiFuture<void> TenantMigrationDonorService::Instance::run(
     const CancellationToken& token) noexcept {
     pauseTenantMigrationBeforeEnteringFutureChain.pauseWhileSet();
 
+    LOGV2(7559500,
+          "Starting tenant migration donor instance: ",
+          "migrationId"_attr = _migrationUuid,
+          "protocol"_attr = MigrationProtocol_serializer(_protocol),
+          "recipientConnectionString"_attr = _recipientConnectionString,
+          "readPreference"_attr = _readPreference);
+
     {
         stdx::lock_guard<Latch> lg(_mutex);
         if (!_stateDoc.getMigrationStart()) {
@@ -950,48 +1005,38 @@ SemiFuture<void> TenantMigrationDonorService::Instance::run(
 
             LOGV2(6104900,
                   "Entering 'aborting index builds' state.",
-                  "migrationId"_attr = _migrationUuid,
-                  "tenantId"_attr = _tenantId);
+                  "migrationId"_attr = _migrationUuid);
             // Note we do not use the abort migration token here because the donorAbortMigration
             // command waits for a decision to be persisted which will not happen if inserting
             // the initial state document fails.
             return _enterAbortingIndexBuildsState(executor, token);
         })
         .then([this, self = shared_from_this(), executor, abortToken] {
-            LOGV2(6104901,
-                  "Aborting index builds.",
-                  "migrationId"_attr = _migrationUuid,
-                  "tenantId"_attr = _tenantId);
+            LOGV2(6104901, "Aborting index builds.", "migrationId"_attr = _migrationUuid);
             _abortIndexBuilds(abortToken);
         })
         .then([this, self = shared_from_this(), executor, recipientTargeterRS, abortToken] {
             LOGV2(6104902,
                   "Fetching cluster time key documents from recipient.",
-                  "migrationId"_attr = _migrationUuid,
-                  "tenantId"_attr = _tenantId);
+                  "migrationId"_attr = _migrationUuid);
             return _fetchAndStoreRecipientClusterTimeKeyDocs(
                 executor, recipientTargeterRS, abortToken);
         })
         .then([this, self = shared_from_this(), executor, abortToken] {
-            LOGV2(6104903,
-                  "Entering 'data sync' state.",
-                  "migrationId"_attr = _migrationUuid,
-                  "tenantId"_attr = _tenantId);
+            LOGV2(6104903, "Entering 'data sync' state.", "migrationId"_attr = _migrationUuid);
             return _enterDataSyncState(executor, abortToken);
         })
         .then([this, self = shared_from_this(), executor, recipientTargeterRS, abortToken] {
             LOGV2(6104904,
                   "Waiting for recipient to finish data sync and become consistent.",
-                  "migrationId"_attr = _migrationUuid,
-                  "tenantId"_attr = _tenantId);
+                  "migrationId"_attr = _migrationUuid);
             return _waitForRecipientToBecomeConsistentAndEnterBlockingState(
                 executor, recipientTargeterRS, abortToken);
         })
         .then([this, self = shared_from_this(), executor, recipientTargeterRS, abortToken, token] {
             LOGV2(6104905,
                   "Waiting for recipient to reach the block timestamp.",
-                  "migrationId"_attr = _migrationUuid,
-                  "tenantId"_attr = _tenantId);
+                  "migrationId"_attr = _migrationUuid);
             return _waitForRecipientToReachBlockTimestampAndEnterCommittedState(
                 executor, recipientTargeterRS, abortToken, token);
         })
@@ -1042,10 +1087,15 @@ SemiFuture<void> TenantMigrationDonorService::Instance::run(
 
             setPromiseFromStatusIfNotReady(lg, _forgetMigrationDurablePromise, status);
 
+            // If a ConflictingServerlessOperation was thrown, ensure a valid _abortReason exists.
+            if (!_abortReason &&
+                isNotDurableAndServerlessConflict(lg, _initialDonorStateDurablePromise)) {
+                _abortReason.emplace(_initialDonorStateDurablePromise.getFuture().getNoThrow());
+            }
+
             LOGV2(5006601,
                   "Tenant migration completed",
                   "migrationId"_attr = _migrationUuid,
-                  "tenantId"_attr = _tenantId,
                   "status"_attr = status,
                   "abortReason"_attr = _abortReason);
 
@@ -1166,9 +1216,9 @@ TenantMigrationDonorService::Instance::_fetchAndStoreRecipientClusterTimeKeyDocs
                        };
 
                        auto fetcher = std::make_shared<Fetcher>(
-                           _recipientCmdExecutor.get(),
+                           *executor,
                            host,
-                           nss.db().toString(),
+                           nss.dbName(),
                            cmdObj,
                            fetcherCallback,
                            kPrimaryOnlyReadPreference.toContainingBSON(),
@@ -1177,8 +1227,7 @@ TenantMigrationDonorService::Instance::_fetchAndStoreRecipientClusterTimeKeyDocs
                            RemoteCommandRetryScheduler::makeRetryPolicy<
                                ErrorCategory::RetriableError>(
                                kMaxRecipientKeyDocsFindAttempts,
-                               executor::RemoteCommandRequest::kNoTimeout),
-                           _sslMode);
+                               executor::RemoteCommandRequest::kNoTimeout));
 
                        {
                            stdx::lock_guard<Latch> lg(_mutex);
@@ -1294,7 +1343,7 @@ TenantMigrationDonorService::Instance::_waitUntilStartMigrationDonorTimestampIsC
                auto opCtx = opCtxHolder.get();
                auto storageEngine = opCtx->getServiceContext()->getStorageEngine();
                if (storageEngine->getLastStableRecoveryTimestamp() < startMigrationDonorTimestamp) {
-                   opCtx->recoveryUnit()->waitUntilUnjournaledWritesDurable(
+                   shard_role_details::getRecoveryUnit(opCtx)->waitUntilUnjournaledWritesDurable(
                        opCtx,
                        /*stableCheckpoint*/ true);
                }
@@ -1336,8 +1385,7 @@ TenantMigrationDonorService::Instance::_waitForRecipientToBecomeConsistentAndEnt
             // Enter "blocking" state.
             LOGV2(6104907,
                   "Updating its state doc to enter 'blocking' state.",
-                  "migrationId"_attr = _migrationUuid,
-                  "tenantId"_attr = _tenantId);
+                  "migrationId"_attr = _migrationUuid);
             return _updateStateDoc(executor, TenantMigrationDonorStateEnum::kBlocking, abortToken)
                 .then([this, self = shared_from_this(), executor, abortToken](repl::OpTime opTime) {
                     return _waitForMajorityWriteConcern(executor, std::move(opTime), abortToken);
@@ -1419,10 +1467,7 @@ TenantMigrationDonorService::Instance::_waitForRecipientToReachBlockTimestampAnd
             checkForTokenInterrupt(abortToken);
 
             // Enter "commit" state.
-            LOGV2(6104908,
-                  "Entering 'committed' state.",
-                  "migrationId"_attr = _migrationUuid,
-                  "tenantId"_attr = _tenantId);
+            LOGV2(6104908, "Entering 'committed' state.", "migrationId"_attr = _migrationUuid);
             // Ignore the abort token once we've entered the committed state
             return _updateStateDoc(executor, TenantMigrationDonorStateEnum::kCommitted, token)
                 .then([this, self = shared_from_this(), executor, token](repl::OpTime opTime) {
@@ -1484,7 +1529,6 @@ ExecutorFuture<void> TenantMigrationDonorService::Instance::_handleErrorOrEnterA
         LOGV2(6104912,
               "Entering 'aborted' state.",
               "migrationId"_attr = _migrationUuid,
-              "tenantId"_attr = _tenantId,
               "status"_attr = status);
         // Enter "abort" state.
         _abortReason.emplace(status);
@@ -1523,8 +1567,7 @@ TenantMigrationDonorService::Instance::_waitForForgetMigrationThenMarkMigrationG
 
     LOGV2(6104909,
           "Waiting to receive 'donorForgetMigration' command.",
-          "migrationId"_attr = _migrationUuid,
-          "tenantId"_attr = _tenantId);
+          "migrationId"_attr = _migrationUuid);
     auto expiredAt = [&]() {
         stdx::lock_guard<Latch> lg(_mutex);
         return _stateDoc.getExpireAt();
@@ -1558,15 +1601,13 @@ TenantMigrationDonorService::Instance::_waitForForgetMigrationThenMarkMigrationG
 
             LOGV2(6104910,
                   "Waiting for recipientForgetMigration response.",
-                  "migrationId"_attr = _migrationUuid,
-                  "tenantId"_attr = _tenantId);
+                  "migrationId"_attr = _migrationUuid);
             return _sendRecipientForgetMigrationCommand(executor, recipientTargeterRS, token);
         })
         .then([this, self = shared_from_this(), executor, token] {
             LOGV2(6104911,
                   "Marking external keys as garbage collectable.",
-                  "migrationId"_attr = _migrationUuid,
-                  "tenantId"_attr = _tenantId);
+                  "migrationId"_attr = _migrationUuid);
             // Note marking the keys as garbage collectable is not atomic with marking the
             // state document garbage collectable, so an interleaved failover can lead the
             // keys to be deleted before the state document has an expiration date. This is
@@ -1581,8 +1622,7 @@ TenantMigrationDonorService::Instance::_waitForForgetMigrationThenMarkMigrationG
         .then([this, self = shared_from_this(), executor, token] {
             LOGV2(6523600,
                   "Marking state document as garbage collectable.",
-                  "migrationId"_attr = _migrationUuid,
-                  "tenantId"_attr = _tenantId);
+                  "migrationId"_attr = _migrationUuid);
             return _markStateDocAsGarbageCollectable(executor, token);
         })
         .then([this, self = shared_from_this(), executor, token](repl::OpTime opTime) {
@@ -1603,16 +1643,12 @@ TenantMigrationDonorService::Instance::_waitForGarbageCollectionDelayThenDeleteS
     LOGV2(8423362,
           "Waiting for garbage collection delay before deleting state document",
           "migrationId"_attr = _migrationUuid,
-          "tenantId"_attr = _tenantId,
           "expireAt"_attr = *_stateDoc.getExpireAt());
 
     return (*executor)
         ->sleepUntil(*_stateDoc.getExpireAt(), token)
         .then([this, self = shared_from_this(), executor, token]() {
-            LOGV2(8423363,
-                  "Deleting state document",
-                  "migrationId"_attr = _migrationUuid,
-                  "tenantId"_attr = _tenantId);
+            LOGV2(8423363, "Deleting state document", "migrationId"_attr = _migrationUuid);
             return _removeStateDoc(executor, token);
         });
 }
