@@ -29,6 +29,10 @@
 
 #pragma once
 
+#include <functional>
+#include <memory>
+
+#include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/db/service_context.h"
 #include "mongo/platform/atomic_word.h"
 #include "mongo/util/scopeguard.h"
