@@ -28,24 +28,68 @@
  */
 
 
-#include "mongo/platform/basic.h"
+#include <algorithm>
+#include <boost/optional.hpp>
+#include <cstdint>
+#include <iterator>
+#include <memory>
+#include <mutex>
+#include <string>
 
-#include "mongo/db/repl/transaction_oplog_application.h"
+#include <boost/cstdint.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/client/dbclient_cursor.h"
 #include "mongo/db/catalog/document_validation.h"
-#include "mongo/db/catalog_raii.h"
+#include "mongo/db/client.h"
 #include "mongo/db/commands/txn_cmds_gen.h"
+#include "mongo/db/concurrency/d_concurrency.h"
 #include "mongo/db/concurrency/exception_util.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
 #include "mongo/db/dbdirectclient.h"
 #include "mongo/db/index_builds_coordinator.h"
+#include "mongo/db/namespace_string.h"
 #include "mongo/db/op_observer/op_observer.h"
-#include "mongo/db/repl/apply_ops.h"
-#include "mongo/db/repl/storage_interface_impl.h"
+#include "mongo/db/query/find_command.h"
+#include "mongo/db/repl/apply_ops_command_info.h"
+#include "mongo/db/repl/oplog_entry_gen.h"
+#include "mongo/db/repl/optime.h"
+#include "mongo/db/repl/read_concern_args.h"
 #include "mongo/db/repl/timestamp_block.h"
+#include "mongo/db/repl/transaction_oplog_application.h"
+#include "mongo/db/repl_index_build_state.h"
+#include "mongo/db/server_options.h"
+#include "mongo/db/service_context.h"
+#include "mongo/db/session/internal_session_pool.h"
+#include "mongo/db/session/logical_session_id.h"
+#include "mongo/db/session/logical_session_id_gen.h"
+#include "mongo/db/session/logical_session_id_helpers.h"
 #include "mongo/db/session/session_catalog_mongod.h"
+#include "mongo/db/session/session_txn_record_gen.h"
+#include "mongo/db/shard_role.h"
+#include "mongo/db/storage/recovery_unit.h"
+#include "mongo/db/storage/write_unit_of_work.h"
 #include "mongo/db/transaction/transaction_history_iterator.h"
 #include "mongo/db/transaction/transaction_participant.h"
+#include "mongo/db/transaction_resources.h"
+#include "mongo/idl/idl_parser.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/redaction.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/s/sharding_feature_flags_gen.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/fail_point.h"
+#include "mongo/util/scopeguard.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplication
 
@@ -68,6 +112,31 @@ MONGO_FAIL_POINT_DEFINE(applyPrepareTxnOpsFailsWithWriteConflict);
 
 MONGO_FAIL_POINT_DEFINE(hangBeforeSessionCheckOutForApplyPrepare);
 
+class ScopedSetTxnInfoOnOperationContext {
+public:
+    ScopedSetTxnInfoOnOperationContext(OperationContext* opCtx,
+                                       const LogicalSessionId& lsid,
+                                       const TxnNumber& txnNumber,
+                                       boost::optional<TxnRetryCounter> txnRetryCounter)
+        : _opCtx(opCtx) {
+        stdx::lock_guard<Client> lk(*opCtx->getClient());
+        opCtx->setLogicalSessionId(lsid);
+        opCtx->setTxnNumber(txnNumber);
+        if (txnRetryCounter.has_value()) {
+            opCtx->setTxnRetryCounter(*txnRetryCounter);
+        }
+        opCtx->setInMultiDocumentTransaction();
+    }
+
+    // The opCtx may be used to apply later operations in a batch, clean up before reusing.
+    ~ScopedSetTxnInfoOnOperationContext() {
+        stdx::lock_guard<Client> lk(*_opCtx->getClient());
+        _opCtx->resetMultiDocumentTransactionState();
+    }
+
+    OperationContext* _opCtx;
+};
+
 // Given a vector of OplogEntry pointers, copy and return a vector of OplogEntry's.
 std::vector<OplogEntry> _copyOps(const std::vector<const OplogEntry*>& ops) {
     std::vector<OplogEntry> res;
@@ -114,6 +183,10 @@ boost::optional<std::vector<StmtId>> _getCommittedStmtIds(const LogicalSessionId
 Status _applyOperationsForTransaction(OperationContext* opCtx,
                                       const std::vector<OplogEntry>& txnOps,
                                       repl::OplogApplication::Mode oplogApplicationMode) {
+
+    const bool allowCollectionCreatinInPreparedTransactions =
+        feature_flags::gCreateCollectionInPreparedTransactions.isEnabled(
+            serverGlobalParams.featureCompatibility.acquireFCVSnapshot());
     // Apply each the operations via repl::applyOperation.
     for (const auto& op : txnOps) {
         try {
@@ -121,17 +194,36 @@ Status _applyOperationsForTransaction(OperationContext* opCtx,
                 continue;
             }
 
-            // Presently, it is not allowed to run a prepared transaction with a command
-            // inside. TODO(SERVER-46105)
-            invariant(!op.isCommand());
-            AutoGetCollection coll(opCtx, op.getNss(), MODE_IX);
+            if (!allowCollectionCreatinInPreparedTransactions) {
+                // Presently, it is not allowed to run a prepared transaction with a command inside.
+                // TODO(SERVER-46105)
+                invariant(!op.isCommand());
+            }
+
+            auto coll = acquireCollection(
+                opCtx,
+                CollectionAcquisitionRequest(op.getNss(),
+                                             AcquisitionPrerequisites::kPretendUnsharded,
+                                             repl::ReadConcernArgs::get(opCtx),
+                                             AcquisitionPrerequisites::kWrite),
+                MODE_IX);
             const bool isDataConsistent = true;
-            auto status = repl::applyOperation_inlock(opCtx,
-                                                      coll.getDb(),
-                                                      ApplierOperation{&op},
-                                                      false /*alwaysUpsert*/,
-                                                      oplogApplicationMode,
-                                                      isDataConsistent);
+            auto status = [&] {
+                if (op.isCommand()) {
+                    invariant(op.getCommandType() == OplogEntry::CommandType::kCreate ||
+                                  op.getCommandType() == OplogEntry::CommandType::kCreateIndexes,
+                              "Multi-document transactions only support 'create' and "
+                              "'createIndexes' command");
+                    return repl::applyCommand_inlock(
+                        opCtx, ApplierOperation{&op}, oplogApplicationMode);
+                }
+                return repl::applyOperation_inlock(opCtx,
+                                                   coll,
+                                                   ApplierOperation{&op},
+                                                   false /*alwaysUpsert*/,
+                                                   oplogApplicationMode,
+                                                   isDataConsistent);
+            }();
             if (!status.isOK()) {
                 return status;
             }
@@ -162,20 +254,15 @@ Status _applyOperationsForTransaction(OperationContext* opCtx,
             }
 
             if (!ignoreException) {
-                LOGV2_DEBUG(
-                    21845,
-                    1,
-                    "Error applying operation in transaction. {error}- oplog entry: {oplogEntry}",
-                    "Error applying operation in transaction",
-                    "error"_attr = redact(ex),
-                    "oplogEntry"_attr = redact(op.toBSONForLogging()));
+                LOGV2_DEBUG(21845,
+                            1,
+                            "Error applying operation in transaction",
+                            "error"_attr = redact(ex),
+                            "oplogEntry"_attr = redact(op.toBSONForLogging()));
                 return exceptionToStatus();
             }
             LOGV2_DEBUG(21846,
                         1,
-                        "Encountered but ignoring error: {error} while applying operations for "
-                        "transaction because we are either in initial "
-                        "sync or recovering mode - oplog entry: {oplogEntry}",
                         "Encountered but ignoring error while applying operations for transaction "
                         "because we are either in initial sync or recovering mode",
                         "error"_attr = redact(ex),
@@ -200,36 +287,48 @@ Status _applyTransactionFromOplogChain(OperationContext* opCtx,
                                        Timestamp commitTimestamp,
                                        Timestamp durableTimestamp) {
     invariant(repl::OplogApplication::inRecovering(mode));
+    invariant(entry.getSessionId().has_value(),
+              "A transaction operation must always have a session id");
+    invariant(entry.getTxnNumber().has_value(),
+              "A transaction operation must always have a txn number");
+    ScopedSetTxnInfoOnOperationContext scopedTxnInfo(
+        opCtx,
+        *entry.getSessionId(),
+        *entry.getTxnNumber(),
+        entry.getOperationSessionInfo().getTxnRetryCounter());
 
     auto ops = readTransactionOperationsFromOplogChain(opCtx, entry, {});
 
     const auto dbName = entry.getNss().dbName();
     Status status = Status::OK();
 
-    writeConflictRetry(opCtx, "replaying prepared transaction", dbName.db(), [&] {
-        WriteUnitOfWork wunit(opCtx);
+    repl::writeConflictRetryWithLimit(
+        opCtx, "replaying prepared transaction", NamespaceString(dbName), [&] {
+            WriteUnitOfWork wunit(opCtx);
 
-        // We might replay a prepared transaction behind oldest timestamp.
-        opCtx->recoveryUnit()->setRoundUpPreparedTimestamps(true);
+            // We might replay a prepared transaction behind oldest timestamp.
+            shard_role_details::getRecoveryUnit(opCtx)->setRoundUpPreparedTimestamps(true);
 
-        BSONObjBuilder resultWeDontCareAbout;
+            BSONObjBuilder resultWeDontCareAbout;
 
-        status = _applyOperationsForTransaction(opCtx, ops, mode);
-        if (status.isOK()) {
-            // If the transaction was empty then we have no locks, ensure at least Global IX.
-            Lock::GlobalLock lk(opCtx, MODE_IX);
-            opCtx->recoveryUnit()->setPrepareTimestamp(commitTimestamp);
-            wunit.prepare();
+            status = _applyOperationsForTransaction(opCtx, ops, mode);
+            if (status.isOK()) {
+                // If the transaction was empty then we have no locks, ensure at least Global
+                // IX.
+                Lock::GlobalLock lk(opCtx, MODE_IX);
+                shard_role_details::getRecoveryUnit(opCtx)->setPrepareTimestamp(commitTimestamp);
+                wunit.prepare();
+
+                // Calls setCommitTimestamp() to set commit timestamp of the transaction and
+                // clears the commit timestamp in the recovery unit when tsBlock goes out of the
+                // scope. It is necessary that we clear the commit timestamp because there can
+                // be another transaction in the same recovery unit calling setTimestamp().
+                TimestampBlock tsBlock(opCtx, commitTimestamp);
+                shard_role_details::getRecoveryUnit(opCtx)->setDurableTimestamp(durableTimestamp);
+                wunit.commit();
+            }
+        });
 
-            // Calls setCommitTimestamp() to set commit timestamp of the transaction and
-            // clears the commit timestamp in the recovery unit when tsBlock goes out of the
-            // scope. It is necessary that we clear the commit timestamp because there can be
-            // another transaction in the same recovery unit calling setTimestamp().
-            TimestampBlock tsBlock(opCtx, commitTimestamp);
-            opCtx->recoveryUnit()->setDurableTimestamp(durableTimestamp);
-            wunit.commit();
-        }
-    });
     return status;
 }
 
@@ -242,20 +341,8 @@ Status _applyCommitTransaction(OperationContext* opCtx,
                                const LogicalSessionId& lsid,
                                TxnNumber txnNumber,
                                Timestamp commitTimestamp) {
-    {
-        stdx::lock_guard<Client> lk(*opCtx->getClient());
-        opCtx->setLogicalSessionId(lsid);
-        opCtx->setTxnNumber(txnNumber);
-        if (auto txnRetryCounter = commitOp.getOperationSessionInfo().getTxnRetryCounter()) {
-            opCtx->setTxnRetryCounter(*txnRetryCounter);
-        }
-        opCtx->setInMultiDocumentTransaction();
-    }
-    // This opCtx can be used to apply later operations in the batch, clean up before reusing.
-    ON_BLOCK_EXIT([opCtx]() {
-        stdx::lock_guard<Client> lk(*opCtx->getClient());
-        opCtx->resetMultiDocumentTransactionState();
-    });
+    ScopedSetTxnInfoOnOperationContext scopedTxnInfo(
+        opCtx, lsid, txnNumber, commitOp.getOperationSessionInfo().getTxnRetryCounter());
 
     // The write on transaction table may be applied concurrently, so refreshing state
     // from disk may read that write, causing starting a new transaction on an existing
@@ -279,20 +366,8 @@ Status _applyAbortTransaction(OperationContext* opCtx,
                               const OplogEntry& abortOp,
                               const LogicalSessionId& lsid,
                               TxnNumber txnNumber) {
-    {
-        stdx::lock_guard<Client> lk(*opCtx->getClient());
-        opCtx->setLogicalSessionId(lsid);
-        opCtx->setTxnNumber(txnNumber);
-        if (auto txnRetryCounter = abortOp.getOperationSessionInfo().getTxnRetryCounter()) {
-            opCtx->setTxnRetryCounter(*txnRetryCounter);
-        }
-        opCtx->setInMultiDocumentTransaction();
-    }
-    // This opCtx can be used to apply later operations in the batch, clean up before reusing.
-    ON_BLOCK_EXIT([opCtx]() {
-        stdx::lock_guard<Client> lk(*opCtx->getClient());
-        opCtx->resetMultiDocumentTransactionState();
-    });
+    ScopedSetTxnInfoOnOperationContext scopedTxnInfo(
+        opCtx, lsid, txnNumber, abortOp.getOperationSessionInfo().getTxnRetryCounter());
 
     // The write on transaction table may be applied concurrently, so refreshing state
     // from disk may read that write, causing starting a new transaction on an existing
@@ -409,7 +484,7 @@ std::pair<std::vector<OplogEntry>, bool> _readTransactionOperationsFromOplogChai
     bool isTransactionWithCommand = false;
     // Ensure future transactions read without a timestamp.
     invariant(RecoveryUnit::ReadSource::kNoTimestamp ==
-              opCtx->recoveryUnit()->getTimestampReadSource());
+              shard_role_details::getRecoveryUnit(opCtx)->getTimestampReadSource());
 
     std::vector<OplogEntry> ops;
 
@@ -519,6 +594,8 @@ Status _applyPrepareTransaction(OperationContext* opCtx,
                                 const std::vector<OplogEntry>& txnOps,
                                 repl::OplogApplication::Mode mode,
                                 boost::optional<std::vector<StmtId>> stmtIds = boost::none) {
+    ScopedSetTxnInfoOnOperationContext scopedTxnInfo(
+        opCtx, lsid, txnNumber, prepareOp.getOperationSessionInfo().getTxnRetryCounter());
 
     // Block application of prepare oplog entries on secondaries when a concurrent
     // background index build is running. This will prevent hybrid index builds from
@@ -551,103 +628,98 @@ Status _applyPrepareTransaction(OperationContext* opCtx,
         }
     }
 
-    {
-        stdx::lock_guard<Client> lk(*opCtx->getClient());
-        opCtx->setLogicalSessionId(lsid);
-        opCtx->setTxnNumber(txnNumber);
-        if (auto txnRetryCounter = prepareOp.getOperationSessionInfo().getTxnRetryCounter()) {
-            opCtx->setTxnRetryCounter(*txnRetryCounter);
-        }
-        opCtx->setInMultiDocumentTransaction();
-    }
-    // This opCtx can be used to apply later operations in the batch, clean up before
-    // reusing.
-    ON_BLOCK_EXIT([opCtx]() {
-        stdx::lock_guard<Client> lk(*opCtx->getClient());
-        opCtx->resetMultiDocumentTransactionState();
-    });
+    return repl::writeConflictRetryWithLimit(
+        opCtx, "applying prepare transaction", prepareOp.getNss(), [&] {
+            // The write on transaction table may be applied concurrently, so refreshing
+            // state from disk may read that write, causing starting a new transaction
+            // on an existing txnNumber. Thus, we start a new transaction without
+            // refreshing state from disk.
+            hangBeforeSessionCheckOutForApplyPrepare.pauseWhileSet();
+            auto mongoDSessionCatalog = MongoDSessionCatalog::get(opCtx);
+            auto sessionCheckout = mongoDSessionCatalog->checkOutSessionWithoutRefresh(opCtx);
+            auto txnParticipant = TransactionParticipant::get(opCtx);
 
-    return writeConflictRetry(opCtx, "applying prepare transaction", prepareOp.getNss().ns(), [&] {
-        // The write on transaction table may be applied concurrently, so refreshing
-        // state from disk may read that write, causing starting a new transaction
-        // on an existing txnNumber. Thus, we start a new transaction without
-        // refreshing state from disk.
-        hangBeforeSessionCheckOutForApplyPrepare.pauseWhileSet();
-        auto mongoDSessionCatalog = MongoDSessionCatalog::get(opCtx);
-        auto sessionCheckout = mongoDSessionCatalog->checkOutSessionWithoutRefresh(opCtx);
-        auto txnParticipant = TransactionParticipant::get(opCtx);
+            // We reset the recovery unit on retries, so make sure that we set the
+            // necessary states.
 
-        // We reset the recovery unit on retries, so make sure that we set the
-        // necessary states.
+            // When querying indexes, we return the record matching the key if it exists,
+            // or an adjacent document. This means that it is possible for us to hit a
+            // prepare conflict if we query for an incomplete key and an adjacent key is
+            // prepared. We ignore prepare conflicts on recovering nodes because they may
+            // may encounter prepare conflicts that did not occur on the primary.
+            shard_role_details::getRecoveryUnit(opCtx)->setPrepareConflictBehavior(
+                PrepareConflictBehavior::kIgnoreConflictsAllowWrites);
 
-        // When querying indexes, we return the record matching the key if it exists,
-        // or an adjacent document. This means that it is possible for us to hit a
-        // prepare conflict if we query for an incomplete key and an adjacent key is
-        // prepared. We ignore prepare conflicts on recovering nodes because they may
-        // may encounter prepare conflicts that did not occur on the primary.
-        opCtx->recoveryUnit()->setPrepareConflictBehavior(
-            PrepareConflictBehavior::kIgnoreConflictsAllowWrites);
-        // We might replay a prepared transaction behind oldest timestamp.
-        if (repl::OplogApplication::inRecovering(mode) ||
-            mode == repl::OplogApplication::Mode::kInitialSync) {
-            opCtx->recoveryUnit()->setRoundUpPreparedTimestamps(true);
-        }
+            // We might replay a prepared transaction behind oldest timestamp.
+            if (repl::OplogApplication::inRecovering(mode) ||
+                mode == repl::OplogApplication::Mode::kInitialSync) {
+                shard_role_details::getRecoveryUnit(opCtx)->setRoundUpPreparedTimestamps(true);
+            }
 
-        // Release WUOW, transaction lock resources and abort storage transaction
-        // so that the writeConflictRetry loop will be able to retry applying the
-        // transactional ops on WCE error.
-        ScopeGuard abortOnError([&txnParticipant, opCtx] {
-            // Abort transaction and invalidate the session it is associated with.
-            txnParticipant.abortTransaction(opCtx);
-            txnParticipant.invalidate(opCtx);
+            // Release WUOW, transaction lock resources and abort storage transaction
+            // so that the writeConflictRetry loop will be able to retry applying the
+            // transactional ops on WCE error.
+            ScopeGuard abortOnError([&txnParticipant, opCtx] {
+                // Abort transaction and invalidate the session it is associated with.
+                txnParticipant.abortTransaction(opCtx);
+                txnParticipant.invalidate(opCtx);
+            });
+
+            // Starts the WUOW.
+            txnParticipant.unstashTransactionResources(
+                opCtx, "prepareTransaction", repl::OplogApplication::inRecovering(mode));
+
+            // Set this in case the application of any ops needs to use the prepare timestamp
+            // of this transaction. It should be cleared automatically when the txn finishes.
+            if (repl::OplogApplication::inRecovering(mode) ||
+                mode == repl::OplogApplication::Mode::kInitialSync) {
+                txnParticipant.setPrepareOpTimeForRecovery(opCtx, prepareOp.getOpTime());
+            }
+
+            auto status = _applyOperationsForTransaction(opCtx, txnOps, mode);
+
+            // Add committed statement IDs if this is a retryable internal transaction.
+            // They are used when this node becomes primary to avoid re-executing
+            // committed txn statements.
+            const auto& committedStmtIds = stmtIds ? stmtIds : _getCommittedStmtIds(lsid, txnOps);
+            if (committedStmtIds) {
+                txnParticipant.addCommittedStmtIds(opCtx, *committedStmtIds, prepareOp.getOpTime());
+            }
+
+            if (MONGO_unlikely(applyPrepareTxnOpsFailsWithWriteConflict.shouldFail())) {
+                LOGV2(4947101, "Hit applyPrepareTxnOpsFailsWithWriteConflict failpoint");
+                throwWriteConflictException(
+                    "Prepare transaction apply ops failed due to write conflict");
+            }
+
+            fassert(31137, status);
+
+            if (MONGO_unlikely(applyOpsHangBeforePreparingTransaction.shouldFail())) {
+                LOGV2(21847, "Hit applyOpsHangBeforePreparingTransaction failpoint");
+                applyOpsHangBeforePreparingTransaction.pauseWhileSet(opCtx);
+            }
+
+            // If we are in a recovery mode, we should have set roundUpPreparedTimestamps to true
+            // above, and that setting should have persisted after we unstashed the transaction
+            // resources and started the WUOW above. This setting is necessary for the subsequent
+            // call to prepareTransaction to succeed in the case where the prepare timestamp is
+            // older than the stable timestamp.
+            invariant(!repl::OplogApplication::inRecovering(mode) ||
+                      shard_role_details::getRecoveryUnit(opCtx)->getRoundUpPreparedTimestamps());
+
+            txnParticipant.prepareTransaction(opCtx, prepareOp.getOpTime());
+
+            auto opObserver = opCtx->getServiceContext()->getOpObserver();
+            invariant(opObserver);
+            opObserver->onTransactionPrepareNonPrimary(
+                opCtx, *prepareOp.getSessionId(), txnOps, prepareOp.getOpTime());
+
+            // Prepare transaction success.
+            abortOnError.dismiss();
+
+            txnParticipant.stashTransactionResources(opCtx);
+            return Status::OK();
         });
-
-        // Starts the WUOW.
-        txnParticipant.unstashTransactionResources(opCtx, "prepareTransaction");
-
-        // Set this in case the application of any ops needs to use the prepare timestamp
-        // of this transaction. It should be cleared automatically when the txn finishes.
-        if (repl::OplogApplication::inRecovering(mode) ||
-            mode == repl::OplogApplication::Mode::kInitialSync) {
-            txnParticipant.setPrepareOpTimeForRecovery(opCtx, prepareOp.getOpTime());
-        }
-
-        auto status = _applyOperationsForTransaction(opCtx, txnOps, mode);
-
-        // Add committed statement IDs if this is a retryable internal transaction.
-        // They are used when this node becomes primary to avoid re-executing
-        // committed txn statements.
-        const auto& committedStmtIds = stmtIds ? stmtIds : _getCommittedStmtIds(lsid, txnOps);
-        if (committedStmtIds) {
-            txnParticipant.addCommittedStmtIds(opCtx, *committedStmtIds, prepareOp.getOpTime());
-        }
-
-        if (MONGO_unlikely(applyPrepareTxnOpsFailsWithWriteConflict.shouldFail())) {
-            LOGV2(4947101, "Hit applyPrepareTxnOpsFailsWithWriteConflict failpoint");
-            throwWriteConflictException(
-                "Prepare transaction apply ops failed due to write conflict");
-        }
-
-        fassert(31137, status);
-
-        if (MONGO_unlikely(applyOpsHangBeforePreparingTransaction.shouldFail())) {
-            LOGV2(21847, "Hit applyOpsHangBeforePreparingTransaction failpoint");
-            applyOpsHangBeforePreparingTransaction.pauseWhileSet(opCtx);
-        }
-
-        txnParticipant.prepareTransaction(opCtx, prepareOp.getOpTime());
-
-        auto opObserver = opCtx->getServiceContext()->getOpObserver();
-        invariant(opObserver);
-        opObserver->onTransactionPrepareNonPrimary(
-            opCtx, *prepareOp.getSessionId(), txnOps, prepareOp.getOpTime());
-
-        // Prepare transaction success.
-        abortOnError.dismiss();
-
-        txnParticipant.stashTransactionResources(opCtx);
-        return Status::OK();
-    });
 }
 
 /**
@@ -660,9 +732,6 @@ void _reconstructPreparedTransaction(OperationContext* opCtx,
     // The transaction may have been prepared originally with document validation bypassed.
     DisableDocumentValidation validationDisabler(opCtx);
 
-    // Snapshot transaction can never conflict with the PBWM lock.
-    opCtx->lockState()->setShouldConflictWithSecondaryBatchApplication(false);
-
     // The operations here are reconstructed at their prepare time. However, that time
     // will be ignored because there is an outer write unit of work during their
     // application. The prepare time of the transaction is set explicitly below.
@@ -715,14 +784,8 @@ Status applyPrepareTransaction(OperationContext* opCtx,
         case repl::OplogApplication::Mode::kSecondary: {
             switch (op.instruction) {
                 case repl::ApplicationInstruction::applyOplogEntry: {
-                    // Checkout the session and apply non-split prepare op.
-                    // TODO (SERVER-70578): This can no longer happen once the feature flag
-                    // is removed.
-                    invariant(!op.subSession);
-                    invariant(!op.preparedTxnOps);
-                    auto ops = readTransactionOperationsFromOplogChain(opCtx, *op, {});
-                    return _applyPrepareTransaction(
-                        opCtx, *op, *op->getSessionId(), *op->getTxnNumber(), ops, mode);
+                    // Not possible for secondary when applying prepare oplog entries.
+                    MONGO_UNREACHABLE;
                 }
                 case repl::ApplicationInstruction::applySplitPreparedTxnOp: {
                     // Checkout the session and apply split prepare op.
@@ -767,7 +830,7 @@ void reconstructPreparedTransactions(OperationContext* opCtx, repl::OplogApplica
 
     // Ensure future transactions read without a timestamp.
     invariant(RecoveryUnit::ReadSource::kNoTimestamp ==
-              opCtx->recoveryUnit()->getTimestampReadSource());
+              shard_role_details::getRecoveryUnit(opCtx)->getTimestampReadSource());
 
     DBDirectClient client(opCtx);
     FindCommandRequest findRequest{NamespaceString::kSessionTransactionsTableNamespace};
@@ -795,8 +858,10 @@ void reconstructPreparedTransactions(OperationContext* opCtx, repl::OplogApplica
         {
             // Make a new opCtx so that we can set the lsid when applying the prepare
             // transaction oplog entry.
-            auto newClient =
-                opCtx->getServiceContext()->makeClient("reconstruct-prepared-transactions");
+            auto newClient = opCtx->getServiceContext()
+                                 ->getService(ClusterRole::ShardServer)
+                                 ->makeClient("reconstruct-prepared-transactions");
+
             AlternativeClientRegion acr(newClient);
             const auto newOpCtx = cc().makeOperationContext();
 
