@@ -27,20 +27,27 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
-#include <functional>
+#include <algorithm>
+#include <cstdint>
 #include <memory>
+#include <string>
 
+#include "mongo/base/error_codes.h"
 #include "mongo/base/status.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/timestamp.h"
 #include "mongo/db/commands.h"
-#include "mongo/db/jsobj.h"
 #include "mongo/db/repl/repl_set_request_votes_args.h"
 #include "mongo/db/repl/vote_requester.h"
 #include "mongo/executor/remote_command_request.h"
 #include "mongo/executor/remote_command_response.h"
-#include "mongo/unittest/unittest.h"
-#include "mongo/util/str.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/bson_test_util.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/duration.h"
 
 namespace mongo {
 namespace repl {
@@ -48,7 +55,6 @@ namespace {
 
 using executor::RemoteCommandRequest;
 using executor::RemoteCommandResponse;
-using unittest::assertGet;
 
 bool stringContains(const std::string& haystack, const std::string& needle) {
     return haystack.find(needle) != std::string::npos;
@@ -87,7 +93,7 @@ TEST(ReplSetRequestVotes, ResponseAcceptsUnknownField) {
 
 class VoteRequesterTest : public mongo::unittest::Test {
 public:
-    virtual void setUp() {
+    void setUp() override {
         ReplSetConfig config(
             ReplSetConfig::parse(BSON("_id"
                                       << "rs0"
@@ -115,10 +121,11 @@ public:
                                                       term,
                                                       false,  // not a dryRun
                                                       lastOplogEntry,
+                                                      lastOplogEntry,
                                                       -1));
     }
 
-    virtual void tearDown() {
+    void tearDown() override {
         _requester.reset(nullptr);
     }
 
@@ -157,7 +164,7 @@ protected:
 
     RemoteCommandRequest requestFrom(std::string hostname) {
         return RemoteCommandRequest(HostAndPort(hostname),
-                                    "",  // fields do not matter in VoteRequester
+                                    DatabaseName::kEmpty,  // fields do not matter in VoteRequester
                                     BSONObj(),
                                     nullptr,
                                     Milliseconds(0));
@@ -235,7 +242,7 @@ protected:
 
 class VoteRequesterDryRunTest : public VoteRequesterTest {
 public:
-    virtual void setUp() {
+    void setUp() override {
         ReplSetConfig config(
             ReplSetConfig::parse(BSON("_id"
                                       << "rs0"
@@ -263,13 +270,14 @@ public:
                                                       term,
                                                       true,  // dryRun
                                                       lastOplogEntry,
+                                                      lastOplogEntry,
                                                       -1));
     }
 };
 
 class VoteRequesterCatchupTakeoverDryRunTest : public VoteRequesterTest {
 public:
-    virtual void setUp() {
+    void setUp() override {
         ReplSetConfig config(
             ReplSetConfig::parse(BSON("_id"
                                       << "rs0"
@@ -295,6 +303,7 @@ public:
                                                       term,
                                                       true,  // dryRun
                                                       lastOplogEntry,
+                                                      lastOplogEntry,
                                                       primaryIndex));
     }
 };
