@@ -29,21 +29,31 @@
 
 #pragma once
 
+#include <boost/move/utility_core.hpp>
+#include <boost/optional.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr.hpp>
+#include <boost/utility/in_place_factory.hpp>  // IWYU pragma: keep
 #include <map>
 #include <memory>
+#include <utility>
 #include <vector>
 
-#include <boost/optional.hpp>
-#include <boost/utility/in_place_factory.hpp>
-
+#include "mongo/base/status.h"
+#include "mongo/base/string_data.h"
 #include "mongo/db/client_strand.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/repl/optime.h"
 #include "mongo/db/service_context.h"
 #include "mongo/executor/task_executor.h"
+#include "mongo/platform/atomic_word.h"
 #include "mongo/platform/mutex.h"
 #include "mongo/stdx/thread.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/cancellation.h"
 #include "mongo/util/concurrency/thread_pool.h"
 #include "mongo/util/future.h"
+#include "mongo/util/future_impl.h"
 
 namespace mongo {
 
@@ -183,14 +193,6 @@ public:
      */
     void shutDown();
 
-    /**
-     * Identical to waitUntilMajorityForWrite, which should be used instead.
-     *
-     * TODO(SERVER-74754): Remove this method and replace all uses with waitUntilMajorityForWrite.
-     */
-    SemiFuture<void> waitUntilMajority(const repl::OpTime& opTime,
-                                       const CancellationToken& cancelToken);
-
     /**
      * Enqueue a request to wait for the given opTime to be majority committed.
      */
