@@ -27,16 +27,33 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
 
-#include "mongo/db/repl_index_build_state.h"
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
 
-#include "mongo/db/query/index_bounds_builder.h"
+#include "mongo/base/error_codes.h"
+#include "mongo/base/string_data.h"
+#include "mongo/db/catalog/collection_catalog.h"
+#include "mongo/db/client.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
+#include "mongo/db/feature_flag.h"
+#include "mongo/db/index/index_descriptor.h"
+#include "mongo/db/repl/member_state.h"
+#include "mongo/db/repl/repl_settings.h"
 #include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/repl/tenant_migration_access_blocker_util.h"
+#include "mongo/db/repl_index_build_state.h"
+#include "mongo/db/server_options.h"
+#include "mongo/db/service_context.h"
+#include "mongo/db/storage/recovery_unit.h"
 #include "mongo/db/storage/storage_parameters_gen.h"
+#include "mongo/db/transaction_resources.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
 #include "mongo/util/assert_util.h"
+#include "mongo/util/str.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
 
@@ -95,6 +112,11 @@ void IndexBuildState::setState(State state,
                   str::stream() << "current state :" << toString(_state)
                                 << ", new state: " << toString(state));
     }
+    LOGV2_DEBUG(6826201,
+                1,
+                "Index build: transitioning state",
+                "current"_attr = toString(_state),
+                "new"_attr = toString(state));
     _state = state;
     if (timestamp)
         _timestamp = timestamp;
@@ -106,9 +128,8 @@ void IndexBuildState::setState(State state,
 
 bool IndexBuildState::_checkIfValidTransition(IndexBuildState::State currentState,
                                               IndexBuildState::State newState) const {
-    // (Ignore FCV check): This feature flag doesn't have any upgrade/downgrade concerns.
-    const auto graceful =
-        feature_flags::gIndexBuildGracefulErrorHandling.isEnabledAndIgnoreFCVUnsafe();
+    const auto graceful = feature_flags::gIndexBuildGracefulErrorHandling.isEnabled(
+        serverGlobalParams.featureCompatibility.acquireFCVSnapshot());
     switch (currentState) {
         case IndexBuildState::State::kSetup:
             return
@@ -245,9 +266,21 @@ void ReplIndexBuildState::setPostFailureState(const Status& status) {
         IndexBuildState::kFailureCleanUp, false /* skipCheck */, boost::none, status);
 }
 
+void ReplIndexBuildState::setVotedForCommitReadiness(OperationContext* opCtx) {
+    stdx::lock_guard lk(_mutex);
+    invariant(!_votedForCommitReadiness);
+    opCtx->checkForInterrupt();
+    _votedForCommitReadiness = true;
+}
+
+bool ReplIndexBuildState::canVoteForAbort() const {
+    stdx::lock_guard lk(_mutex);
+    return !_votedForCommitReadiness;
+}
+
 void ReplIndexBuildState::commit(OperationContext* opCtx) {
     auto skipCheck = _shouldSkipIndexBuildStateTransitionCheck(opCtx);
-    opCtx->recoveryUnit()->onCommit(
+    shard_role_details::getRecoveryUnit(opCtx)->onCommit(
         [this, skipCheck](OperationContext*, boost::optional<Timestamp>) {
             stdx::lock_guard lk(_mutex);
             _indexBuildState.setState(IndexBuildState::kCommitted, skipCheck);
@@ -291,7 +324,8 @@ void ReplIndexBuildState::abortForShutdown(OperationContext* opCtx) {
 void ReplIndexBuildState::onOplogAbort(OperationContext* opCtx, const NamespaceString& nss) const {
     auto replCoord = repl::ReplicationCoordinator::get(opCtx);
     bool isPrimary = replCoord->canAcceptWritesFor(opCtx, nss) &&
-        !replCoord->getSettings().shouldRecoverFromOplogAsStandalone();
+        !replCoord->getSettings().shouldRecoverFromOplogAsStandalone() &&
+        !storageGlobalParams.magicRestore;
     invariant(!isPrimary, str::stream() << "Index build: " << buildUUID);
 
     stdx::lock_guard lk(_mutex);
@@ -372,8 +406,6 @@ void ReplIndexBuildState::setCommitQuorumSatisfied(OperationContext* opCtx) {
         auto action = _waitForNextAction->getFuture().get(opCtx);
 
         LOGV2(3856200,
-              "Not signaling \"{skippedAction}\" as it was previously signaled with "
-              "\"{previousAction}\" for index build: {buildUUID}",
               "Skipping signaling as it was previously signaled for index build",
               "skippedAction"_attr =
                   indexBuildActionToString(IndexBuildAction::kCommitQuorumSatisfied),
@@ -430,7 +462,7 @@ bool ReplIndexBuildState::tryCommit(OperationContext* opCtx) {
 
     _indexBuildState.setState(IndexBuildState::kApplyCommitOplogEntry,
                               skipCheck,
-                              opCtx->recoveryUnit()->getCommitTimestamp());
+                              shard_role_details::getRecoveryUnit(opCtx)->getCommitTimestamp());
     // Promise can be set only once.
     // We can't skip signaling here if a signal is already set because the previous commit or
     // abort signal might have been sent to handle for primary case.
@@ -446,8 +478,9 @@ ReplIndexBuildState::TryAbortResult ReplIndexBuildState::tryAbort(OperationConte
     // MODE_X lock is held and there cannot be concurrent external aborters.
     auto nssOptional = CollectionCatalog::get(opCtx)->lookupNSSByUUID(opCtx, collectionUUID);
     invariant(!_indexBuildState.isExternalAbort());
-    invariant(nssOptional &&
-              opCtx->lockState()->isCollectionLockedForMode(nssOptional.get(), MODE_X));
+    invariant(
+        nssOptional &&
+        shard_role_details::getLocker(opCtx)->isCollectionLockedForMode(nssOptional.get(), MODE_X));
 
     // Wait until the build is done setting up. This indicates that all required state is
     // initialized to attempt an abort.
@@ -506,9 +539,9 @@ ReplIndexBuildState::TryAbortResult ReplIndexBuildState::tryAbort(OperationConte
     LOGV2(4656003, "Aborting index build", "buildUUID"_attr = buildUUID, "error"_attr = reason);
 
     // Set the state on replState. Once set, the calling thread must complete the abort process.
-    auto abortTimestamp =
-        boost::make_optional<Timestamp>(!opCtx->recoveryUnit()->getCommitTimestamp().isNull(),
-                                        opCtx->recoveryUnit()->getCommitTimestamp());
+    auto abortTimestamp = boost::make_optional<Timestamp>(
+        !shard_role_details::getRecoveryUnit(opCtx)->getCommitTimestamp().isNull(),
+        shard_role_details::getRecoveryUnit(opCtx)->getCommitTimestamp());
     auto skipCheck = _shouldSkipIndexBuildStateTransitionCheck(opCtx);
     Status abortStatus = signalAction == IndexBuildAction::kTenantMigrationAbort
         ? tenant_migration_access_blocker::checkIfCanBuildIndex(opCtx, dbName)
@@ -532,7 +565,7 @@ ReplIndexBuildState::TryAbortResult ReplIndexBuildState::tryAbort(OperationConte
     auto serviceContext = opCtx->getServiceContext();
     if (auto target = serviceContext->getLockedClient(*_opId)) {
         auto targetOpCtx = target->getOperationContext();
-        serviceContext->killOperation(target, targetOpCtx, ErrorCodes::IndexBuildAborted);
+        serviceContext->killOperation(target, targetOpCtx);
     }
 
     // Set the signal. Because we have already interrupted the index build, it will not observe
@@ -546,10 +579,15 @@ bool ReplIndexBuildState::forceSelfAbort(OperationContext* opCtx, const Status&
     stdx::lock_guard lk(_mutex);
     if (_indexBuildState.isSettingUp() || _indexBuildState.isAborted() ||
         _indexBuildState.isCommitted() || _indexBuildState.isAborting() ||
-        _indexBuildState.isApplyingCommitOplogEntry()) {
+        _indexBuildState.isApplyingCommitOplogEntry() || _votedForCommitReadiness) {
         // If the build is setting up, it is not yet abortable. If the index build has already
         // passed a point of no return, interrupting will not be productive. If the index build is
         // already in the process of aborting, it cannot be aborted again.
+        LOGV2(7617000,
+              "Index build: cannot force abort",
+              "buildUUID"_attr = buildUUID,
+              "state"_attr = _indexBuildState,
+              "votedForCommit"_attr = _votedForCommitReadiness);
         return false;
     }
 
@@ -565,9 +603,8 @@ bool ReplIndexBuildState::forceSelfAbort(OperationContext* opCtx, const Status&
 
         LOGV2(7419400, "Forcefully aborting index build", "buildUUID"_attr = buildUUID);
 
-        // We don't pass IndexBuildAborted as the interruption error code because that would imply
-        // that we are taking responsibility for cleaning up the index build, when in fact the index
-        // builder thread is responsible.
+        // The index builder thread is responsible for cleaning up, as indicated by the
+        // kFailureCleanUp state.
         serviceContext->killOperation(target, targetOpCtx);
     }
     return true;
@@ -698,7 +735,18 @@ void ReplIndexBuildState::appendBuildInfo(BSONObjBuilder* builder) const {
 
 bool ReplIndexBuildState::_shouldSkipIndexBuildStateTransitionCheck(OperationContext* opCtx) const {
     const auto replCoord = repl::ReplicationCoordinator::get(opCtx);
-    if (replCoord->isReplEnabled() && protocol == IndexBuildProtocol::kTwoPhase) {
+    if (replCoord->getSettings().isReplSet() && protocol == IndexBuildProtocol::kTwoPhase) {
+        if (replCoord->getMemberState() == repl::MemberState::RS_STARTUP2 &&
+            !serverGlobalParams.featureCompatibility.acquireFCVSnapshot().isVersionInitialized()) {
+            // We're likely at the initial stages of a new logical initial sync attempt, and we
+            // haven't yet replicated the FCV from the sync source. Skip the index build state
+            // transition checks because they rely on the FCV.
+            LOGV2_DEBUG(6826202,
+                        2,
+                        "Index build: skipping index build state transition checks because the FCV "
+                        "isn't known yet");
+            return true;
+        }
         return false;
     }
     return true;
