@@ -29,24 +29,36 @@
 
 #pragma once
 
-#include "mongo/stdx/mutex.h"
-#include "mongo/util/concurrency/with_lock.h"
 #include <algorithm>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
 #include <list>
+#include <memory>
 #include <string>
+#include <utility>
 #include <vector>
 
+#include "mongo/base/status.h"
 #include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/bson/timestamp.h"
 #include "mongo/db/catalog/commit_quorum_options.h"
 #include "mongo/db/concurrency/d_concurrency.h"
+#include "mongo/db/database_name.h"
 #include "mongo/db/index/index_descriptor.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/operation_context.h"
+#include "mongo/db/operation_id.h"
 #include "mongo/db/repl/optime.h"
 #include "mongo/executor/task_executor.h"
+#include "mongo/platform/mutex.h"
 #include "mongo/stdx/condition_variable.h"
+#include "mongo/stdx/mutex.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/concurrency/with_lock.h"
 #include "mongo/util/future.h"
+#include "mongo/util/future_impl.h"
 #include "mongo/util/net/hostandport.h"
 #include "mongo/util/uuid.h"
 
@@ -331,6 +343,20 @@ public:
      */
     void setPostFailureState(const Status& status);
 
+    /**
+     * Indicate that the index build has attempted to vote for commit readiness. After calling this,
+     * the index build cannot vote for abort. Performs an interrupt check, in case the build was
+     * concurrently forced to self abort or received a killop, in which case the vote for abort is
+     * necessary.
+     */
+    void setVotedForCommitReadiness(OperationContext* opCtx);
+
+    /**
+     * Returns true if this index build can still vote for abort. Voting for abort is not possible
+     * after the index build has voted for commit.
+     */
+    bool canVoteForAbort() const;
+
     /**
      * This index build has completed successfully and there is no further work to be done.
      */
@@ -413,7 +439,7 @@ public:
     std::string getAbortReason() const;
 
     /**
-     * Returns abort status. Invariants if not in aborted state.
+     * Returns abort status. Returns Status::OK() if not in aborted state.
      */
     Status getAbortStatus() const;
 
@@ -603,6 +629,9 @@ private:
 
     // Set once setup is complete, indicating that a clean up is required in case of abort.
     bool _cleanUpRequired = false;
+
+    // Set once before attempting to vote for commit readiness.
+    bool _votedForCommitReadiness = false;
 };
 
 }  // namespace mongo
