@@ -29,14 +29,30 @@
 
 #include "mongo/db/s/active_migrations_registry.h"
 
+#include <absl/container/node_hash_map.h>
+#include <boost/none.hpp>
+#include <fmt/format.h>
+#include <mutex>
+
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
 #include "mongo/db/catalog_raii.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
+#include "mongo/db/database_name.h"
+#include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/s/collection_sharding_runtime.h"
 #include "mongo/db/s/migration_destination_manager.h"
-#include "mongo/db/s/migration_session_id.h"
 #include "mongo/db/s/migration_source_manager.h"
 #include "mongo/db/service_context.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/decorable.h"
 #include "mongo/util/scopeguard.h"
+#include "mongo/util/str.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kShardingMigration
 
@@ -89,7 +105,7 @@ void ActiveMigrationsRegistry::lock(OperationContext* opCtx, StringData reason)
         uassert(ErrorCodes::NotWritablePrimary,
                 "Cannot lock the registry while the node is in draining mode",
                 repl::ReplicationCoordinator::get(opCtx)->canAcceptWritesForDatabase(
-                    opCtx, DatabaseName::kAdmin.toString()));
+                    opCtx, DatabaseName::kAdmin));
     }
 
     unblockMigrationsOnError.dismiss();
@@ -282,7 +298,7 @@ Status ActiveMigrationsRegistry::ActiveMoveChunkState::constructErrorStatus() co
         "'{}{}' for namespace {} to shard {}",
         (args.getMin() ? "min: " + args.getMin()->toString() + " - " : ""),
         (args.getMax() ? "max: " + args.getMax()->toString() : ""),
-        args.getCommandParameter().ns(),
+        args.getCommandParameter().toStringForErrorMsg(),
         args.getToShard().toString());
     return {ErrorCodes::ConflictingOperationInProgress, std::move(errMsg)};
 }
@@ -291,8 +307,8 @@ Status ActiveMigrationsRegistry::ActiveReceiveChunkState::constructErrorStatus()
     return {ErrorCodes::ConflictingOperationInProgress,
             str::stream() << "Unable to start new balancer operation because this shard is "
                              "currently receiving chunk "
-                          << range.toString() << " for namespace " << nss.ns() << " from "
-                          << fromShardId};
+                          << range.toString() << " for namespace " << nss.toStringForErrorMsg()
+                          << " from " << fromShardId};
 }
 
 ScopedDonateChunk::ScopedDonateChunk(ActiveMigrationsRegistry* registry,
