@@ -28,14 +28,52 @@
  */
 
 #include "mongo/db/s/balancer/auto_merger_policy.h"
+
+#include <absl/container/node_hash_map.h>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <memory>
+#include <mutex>
+#include <string>
+#include <utility>
+#include <variant>
+
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/bson/bson_field.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/client/dbclient_cursor.h"
 #include "mongo/db/dbdirectclient.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/pipeline/aggregate_command_gen.h"
 #include "mongo/db/pipeline/document_source_lookup.h"
+#include "mongo/db/pipeline/document_source_match.h"
+#include "mongo/db/pipeline/document_source_unwind.h"
+#include "mongo/db/pipeline/expression_context.h"
+#include "mongo/db/pipeline/pipeline.h"
+#include "mongo/db/repl/read_concern_args.h"
+#include "mongo/db/repl/read_concern_level.h"
 #include "mongo/db/s/config/sharding_catalog_manager.h"
 #include "mongo/db/s/sharding_config_server_parameters_gen.h"
+#include "mongo/db/s/sharding_logging.h"
+#include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/redaction.h"
+#include "mongo/s/catalog/type_chunk.h"
 #include "mongo/s/catalog/type_collection.h"
+#include "mongo/s/catalog/type_collection_gen.h"
 #include "mongo/s/client/shard_registry.h"
 #include "mongo/s/grid.h"
-#include "mongo/s/sharding_feature_flags_gen.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/intrusive_counter.h"
+#include "mongo/util/string_map.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
@@ -43,23 +81,38 @@ namespace mongo {
 
 namespace {
 const std::string kPolicyName{"AutoMergerPolicy"};
+
+void logAutoMergeChange(OperationContext* opCtx, const std::string& what) {
+    auto catalogManager = ShardingCatalogManager::get(opCtx);
+    ShardingLogging::get(opCtx)->logChange(opCtx,
+                                           what,
+                                           NamespaceString::kEmpty,
+                                           BSONObj(),
+                                           WriteConcernOptions(),
+                                           std::move(catalogManager->localConfigShard()),
+                                           catalogManager->localCatalogClient());
+}
 }  // namespace
 
 
-void AutoMergerPolicy::enable() {
+void AutoMergerPolicy::enable(OperationContext* opCtx) {
     stdx::lock_guard<Latch> lk(_mutex);
     if (!_enabled) {
         _enabled = true;
-        _init(lk);
+        logAutoMergeChange(opCtx, "autoMerge.enable");
+        _init(opCtx, lk);
     }
 }
 
-void AutoMergerPolicy::disable() {
+void AutoMergerPolicy::disable(OperationContext* opCtx) {
     stdx::lock_guard<Latch> lk(_mutex);
-    _enabled = false;
-    _collectionsToMergePerShard.clear();
-    _maxHistoryTimeCurrentRound = Timestamp(0, 0);
-    _maxHistoryTimePreviousRound = Timestamp(0, 0);
+    if (_enabled) {
+        _enabled = false;
+        _collectionsToMergePerShard.clear();
+        _maxHistoryTimeCurrentRound = Timestamp(0, 0);
+        _maxHistoryTimePreviousRound = Timestamp(0, 0);
+        logAutoMergeChange(opCtx, "autoMerge.disable");
+    }
 }
 
 bool AutoMergerPolicy::isEnabled() {
@@ -67,14 +120,12 @@ bool AutoMergerPolicy::isEnabled() {
     return _enabled;
 }
 
-void AutoMergerPolicy::checkInternalUpdates() {
+void AutoMergerPolicy::checkInternalUpdates(OperationContext* opCtx) {
     stdx::lock_guard<Latch> lk(_mutex);
-    if (!feature_flags::gAutoMerger.isEnabled(
-            serverGlobalParams.featureCompatibility.acquireFCVSnapshot()) ||
-        !_enabled) {
+    if (!_enabled) {
         return;
     }
-    _checkInternalUpdatesWithLock(lk);
+    _checkInternalUpdatesWithLock(opCtx, lk);
 }
 
 StringData AutoMergerPolicy::getName() const {
@@ -85,13 +136,11 @@ boost::optional<BalancerStreamAction> AutoMergerPolicy::getNextStreamingAction(
     OperationContext* opCtx) {
     stdx::unique_lock<Latch> lk(_mutex);
 
-    if (!feature_flags::gAutoMerger.isEnabled(
-            serverGlobalParams.featureCompatibility.acquireFCVSnapshot()) ||
-        !_enabled) {
+    if (!_enabled) {
         return boost::none;
     }
 
-    _checkInternalUpdatesWithLock(lk);
+    _checkInternalUpdatesWithLock(opCtx, lk);
 
     bool applyThrottling = false;
 
@@ -138,6 +187,11 @@ boost::optional<BalancerStreamAction> AutoMergerPolicy::getNextStreamingAction(
         }
     }
 
+    if (_withinRound) {
+        // All mergeable chunks have been processed for the current round.
+        _withinRound = false;
+        logAutoMergeChange(opCtx, "autoMerge.end");
+    }
     return boost::none;
 }
 
@@ -153,9 +207,9 @@ void AutoMergerPolicy::applyActionResult(OperationContext* opCtx,
         }
     });
 
-    const auto& mergeAction = stdx::get<MergeAllChunksOnShardInfo>(action);
+    const auto& mergeAction = get<MergeAllChunksOnShardInfo>(action);
 
-    const auto& swResponse = stdx::get<StatusWith<NumMergedChunks>>(response);
+    const auto& swResponse = get<StatusWith<NumMergedChunks>>(response);
     if (swResponse.isOK()) {
         auto numMergedChunks = swResponse.getValue();
         if (numMergedChunks > 0) {
@@ -165,7 +219,7 @@ void AutoMergerPolicy::applyActionResult(OperationContext* opCtx,
         return;
     }
 
-    const auto status = std::move(swResponse.getStatus());
+    const auto status = swResponse.getStatus();
     if (status.code() == ErrorCodes::ConflictingOperationInProgress) {
         // Reschedule auto-merge for <shard, nss> because commit overlapped with other chunk ops
         _rescheduledCollectionsToMergePerShard[mergeAction.shardId].push_back(mergeAction.nss);
@@ -187,23 +241,25 @@ void AutoMergerPolicy::applyActionResult(OperationContext* opCtx,
     }
 }
 
-void AutoMergerPolicy::_init(WithLock lk) {
+void AutoMergerPolicy::_init(OperationContext* opCtx, WithLock lk) {
     _maxHistoryTimePreviousRound = _maxHistoryTimeCurrentRound;
     _intervalTimer.reset();
     _collectionsToMergePerShard.clear();
     _firstAction = true;
+    _withinRound = true;
     _outstandingActions = 0;
     _onStateUpdated();
+    logAutoMergeChange(opCtx, "autoMerge.start");
 }
 
-void AutoMergerPolicy::_checkInternalUpdatesWithLock(WithLock lk) {
+void AutoMergerPolicy::_checkInternalUpdatesWithLock(OperationContext* opCtx, WithLock lk) {
     if (!_enabled || !_collectionsToMergePerShard.empty() || _outstandingActions) {
         return;
     }
 
     // Trigger Automerger every `autoMergerIntervalSecs` seconds
     if (_intervalTimer.seconds() > autoMergerIntervalSecs) {
-        _init(lk);
+        _init(opCtx, lk);
     }
 }
 
@@ -218,22 +274,25 @@ AutoMergerPolicy::_getNamespacesWithMergeableChunksPerShard(OperationContext* op
         Pipeline::SourceContainer stages;
 
         auto expCtx = make_intrusive<ExpressionContext>(opCtx, nullptr, ChunkType::ConfigNS);
-        ResolvedNamespaceMap resolvedNamespaces;
-        resolvedNamespaces[ChunkType::ConfigNS] = {ChunkType::ConfigNS, std::vector<BSONObj>()};
-        resolvedNamespaces[CollectionType::ConfigNS] = {CollectionType::ConfigNS,
-                                                        std::vector<BSONObj>()};
+        StringMap<ExpressionContext::ResolvedNamespace> resolvedNamespaces;
+        resolvedNamespaces[ChunkType::ConfigNS.coll()] = {ChunkType::ConfigNS,
+                                                          std::vector<BSONObj>()};
+        resolvedNamespaces[CollectionType::ConfigNS.coll()] = {CollectionType::ConfigNS,
+                                                               std::vector<BSONObj>()};
         expCtx->setResolvedNamespaces(resolvedNamespaces);
 
         // 1. Match all collections where `automerge` is enabled and `defragmentation` is disabled
         // {
         //     $match : {
         //         enableAutoMerge : { $ne : false },
-        //         defragmentCollection : { $ne : true }
+        //         defragmentCollection : { $ne : true },
+        //         unsplittable : { $ne : true }
         //     }
         // }
         stages.emplace_back(DocumentSourceMatch::create(
             BSON(CollectionType::kEnableAutoMergeFieldName
                  << BSON("$ne" << false) << CollectionType::kDefragmentCollectionFieldName
+                 << BSON("$ne" << true) << CollectionType::kUnsplittableFieldName
                  << BSON("$ne" << true)),
             expCtx));
 
@@ -298,7 +357,10 @@ AutoMergerPolicy::_getNamespacesWithMergeableChunksPerShard(OperationContext* op
 
         while (cursor->more()) {
             const auto doc = cursor->nextSafe();
-            const auto nss = NamespaceString(doc.getStringField(CollectionType::kNssFieldName));
+            const auto nss =
+                NamespaceStringUtil::deserialize(boost::none,
+                                                 doc.getStringField(CollectionType::kNssFieldName),
+                                                 SerializationContext::stateDefault());
             collectionsToMerge[shard].push_back(nss);
         }
     }
