@@ -29,11 +29,21 @@
 
 #pragma once
 
+#include <cstdint>
+
 #include "mongo/bson/bsonobj.h"
+#include "mongo/db/keypattern.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/s/balancer/balancer_policy.h"
 #include "mongo/db/shard_id.h"
+#include "mongo/db/write_concern_options.h"
 #include "mongo/executor/task_executor.h"
+#include "mongo/s/catalog/type_chunk.h"
+#include "mongo/s/chunk_version.h"
 #include "mongo/s/request_types/move_range_request_gen.h"
+#include "mongo/s/shard_version.h"
+#include "mongo/util/future.h"
 
 namespace mongo {
 
@@ -88,6 +98,12 @@ public:
     virtual SemiFuture<NumMergedChunks> requestMergeAllChunksOnShard(OperationContext* opCtx,
                                                                      const NamespaceString& nss,
                                                                      const ShardId& shardId) = 0;
+
+    virtual SemiFuture<void> requestMoveCollection(OperationContext* opCtx,
+                                                   const NamespaceString& nss,
+                                                   const ShardId& toShardId,
+                                                   const ShardId& dbPrimaryShardId,
+                                                   const DatabaseVersion& dbVersion) = 0;
 };
 
 }  // namespace mongo
