@@ -29,17 +29,42 @@
 
 #include "mongo/db/s/balancer/balancer_commands_scheduler_impl.h"
 
+#include <absl/container/node_hash_map.h>
+#include <absl/meta/type_traits.h>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+// IWYU pragma: no_include "cxxabi.h"
+#include <mutex>
+#include <type_traits>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/client/read_preference.h"
+#include "mongo/client/remote_command_targeter.h"
 #include "mongo/db/client.h"
-#include "mongo/db/dbdirectclient.h"
+#include "mongo/db/s/config/sharding_catalog_manager.h"
 #include "mongo/db/s/sharding_util.h"
 #include "mongo/db/shard_id.h"
+#include "mongo/executor/remote_command_request.h"
+#include "mongo/executor/task_executor.h"
+#include "mongo/executor/task_executor_pool.h"
+#include "mongo/idl/idl_parser.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/redaction.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/rpc/get_status_from_command_result.h"
+#include "mongo/s/async_requests_sender.h"
 #include "mongo/s/catalog/type_chunk.h"
+#include "mongo/s/client/shard.h"
+#include "mongo/s/client/shard_registry.h"
 #include "mongo/s/grid.h"
-#include "mongo/s/request_types/migration_secondary_throttle_options.h"
 #include "mongo/s/request_types/shardsvr_join_migrations_request_gen.h"
-#include "mongo/s/shard_key_pattern.h"
 #include "mongo/util/fail_point.h"
+#include "mongo/util/scopeguard.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
@@ -57,7 +82,7 @@ void waitForQuiescedCluster(OperationContext* opCtx) {
 
     const auto responses =
         sharding_util::sendCommandToShards(opCtx,
-                                           DatabaseName::kAdmin.toString(),
+                                           DatabaseName::kAdmin,
                                            joinShardOnMigrationsRequest.toBSON({}),
                                            unquiescedShardIds,
                                            executor,
@@ -82,11 +107,7 @@ Status processRemoteResponse(const executor::RemoteCommandResponse& remoteRespon
         return remoteResponse.status;
     }
     auto remoteStatus = getStatusFromCommandResult(remoteResponse.data);
-    return Shard::shouldErrorBePropagated(remoteStatus.code())
-        ? remoteStatus
-        : Status(ErrorCodes::OperationFailed,
-                 str::stream() << "Command request failed on source shard. "
-                               << causedBy(remoteStatus));
+    return remoteStatus.withContext("Command request failed on source shard.");
 }
 
 }  // namespace
@@ -237,16 +258,25 @@ SemiFuture<NumMergedChunks> BalancerCommandsSchedulerImpl::requestMergeAllChunks
                 return responseStatus;
             }
 
-            try {
-                return MergeAllChunksOnShardResponse::parse(
-                           IDLParserContext{"MergeAllChunksOnShardResponse"}, remoteResponse.data)
-                    .getNumMergedChunks();
-            } catch (const DBException&) {
-                // TODO SERVER-74573 remove try-catch once 7.0 branches out
-                // It may happen in multiversion scenarios for the command not to return a
-                // MergeAllChunksOnShardResponse (in v6.3 the reply was empty)
-                return 0;
-            }
+            return MergeAllChunksOnShardResponse::parse(
+                       IDLParserContext{"MergeAllChunksOnShardResponse"}, remoteResponse.data)
+                .getNumMergedChunks();
+        })
+        .semi();
+}
+
+SemiFuture<void> BalancerCommandsSchedulerImpl::requestMoveCollection(
+    OperationContext* opCtx,
+    const NamespaceString& nss,
+    const ShardId& toShardId,
+    const ShardId& dbPrimaryShardId,
+    const DatabaseVersion& dbVersion) {
+    auto commandInfo =
+        std::make_shared<MoveCollectionCommandInfo>(nss, toShardId, dbPrimaryShardId, dbVersion);
+
+    return _buildAndEnqueueNewRequest(opCtx, std::move(commandInfo))
+        .then([](const executor::RemoteCommandResponse& remoteResponse) {
+            return processRemoteResponse(remoteResponse);
         })
         .semi();
 }
@@ -326,7 +356,9 @@ CommandSubmissionResult BalancerCommandsSchedulerImpl::_submit(
 void BalancerCommandsSchedulerImpl::_applySubmissionResult(
     WithLock, CommandSubmissionResult&& submissionResult) {
     auto submittedRequestIt = _requests.find(submissionResult.id);
-    invariant(submittedRequestIt != _requests.end());
+    tassert(8245206,
+            "Submission result ID not found in the requests",
+            submittedRequestIt != _requests.end());
     auto& submittedRequest = submittedRequestIt->second;
     auto submissionOutcome = submittedRequest.applySubmissionResult(std::move(submissionResult));
     if (!submissionOutcome.isOK()) {
@@ -343,9 +375,9 @@ void BalancerCommandsSchedulerImpl::_applyCommandResponse(
     UUID requestId, const executor::RemoteCommandResponse& response) {
     {
         stdx::lock_guard<Latch> lg(_mutex);
-        invariant(_state != SchedulerState::Stopped);
+        tassert(8245207, "Scheduler is stopped", _state != SchedulerState::Stopped);
         auto requestIt = _requests.find(requestId);
-        invariant(requestIt != _requests.end());
+        tassert(8245208, "Request ID is already in use", requestIt != _requests.end());
         auto& request = requestIt->second;
         request.setOutcome(response);
         _recentlyCompletedRequestIds.emplace_back(request.getId());
@@ -370,18 +402,8 @@ void BalancerCommandsSchedulerImpl::_workerThread() {
         _stateUpdatedCV.notify_all();
     });
 
-    Client::initThread("BalancerCommandsScheduler");
-    {
-        stdx::lock_guard<Client> lk(cc());
-        cc().setSystemOperationKillableByStepdown(lk);
-    }
-
-    // This worker thread may perform remote request, so that its operation context must be
-    // interruptible. Marking it so here is safe, since the replica set changes are also notified by
-    // the Balancer (a PrimaryOnlyService) and tracked through the _state field (which is checked
-    // right after).
-    auto opCtxHolder = cc().makeOperationContext();
-    opCtxHolder.get()->setAlwaysInterruptAtStepDownOrUp_UNSAFE();
+    Client::initThread("BalancerCommandsScheduler",
+                       getGlobalServiceContext()->getService(ClusterRole::ShardServer));
 
     bool stopWorkerRequested = false;
     LOGV2(5847205, "Balancer scheduler thread started");
@@ -393,7 +415,7 @@ void BalancerCommandsSchedulerImpl::_workerThread() {
         // 1. Check the internal state and plan for the actions to be taken ont this round.
         {
             stdx::unique_lock<Latch> ul(_mutex);
-            invariant(_state != SchedulerState::Stopped);
+            tassert(8245209, "Scheduler is stopped", _state != SchedulerState::Stopped);
             _stateUpdatedCV.wait(ul, [this] {
                 return ((!_unsubmittedRequestIds.empty() &&
                          !MONGO_likely(pauseSubmissionsFailPoint.shouldFail())) ||
@@ -424,6 +446,7 @@ void BalancerCommandsSchedulerImpl::_workerThread() {
 
         // 2. Serve the picked up requests, submitting their related commands.
         for (auto& submissionInfo : commandsToSubmit) {
+            auto opCtxHolder = cc().makeOperationContext();
             if (submissionInfo.commandInfo) {
                 submissionInfo.commandInfo.get()->attachOperationMetadataTo(opCtxHolder.get());
             }
