@@ -29,22 +29,42 @@
 
 #pragma once
 
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <cstddef>
+#include <cstdint>
+#include <fmt/format.h>
+#include <map>
 #include <set>
+#include <string>
+#include <tuple>
+#include <utility>
 #include <variant>
 #include <vector>
 
+#include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
 #include "mongo/bson/bsonobj.h"
-#include "mongo/bson/simple_bsonobj_comparator.h"
+#include "mongo/bson/bsonobj_comparator_interface.h"
+#include "mongo/db/auth/validated_tenancy_scope.h"
+#include "mongo/db/keypattern.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/s/balancer/cluster_statistics.h"
 #include "mongo/db/shard_id.h"
 #include "mongo/s/catalog/type_chunk.h"
 #include "mongo/s/chunk_manager.h"
+#include "mongo/s/chunk_version.h"
+#include "mongo/s/request_types/get_stats_for_balancing_gen.h"
 #include "mongo/s/request_types/move_range_request_gen.h"
 #include "mongo/s/shard_version.h"
-#include "mongo/util/concurrency/with_lock.h"
+#include "mongo/stdx/unordered_set.h"
+#include "mongo/util/uuid.h"
 
 namespace mongo {
+
+using namespace fmt::literals;
+
 struct ZoneRange {
     ZoneRange(const BSONObj& a_min, const BSONObj& a_max, const std::string& _zone);
 
@@ -201,27 +221,39 @@ struct ShardZoneInfo {
 
 typedef int NumMergedChunks;
 
-typedef stdx::variant<MergeInfo, DataSizeInfo, MigrateInfo, MergeAllChunksOnShardInfo>
+typedef std::variant<MergeInfo, DataSizeInfo, MigrateInfo, MergeAllChunksOnShardInfo>
     BalancerStreamAction;
 
-typedef stdx::variant<Status, StatusWith<DataSizeResponse>, StatusWith<NumMergedChunks>>
+typedef std::variant<Status, StatusWith<DataSizeResponse>, StatusWith<NumMergedChunks>>
     BalancerStreamActionResponse;
 
 typedef std::vector<ClusterStatistics::ShardStatistics> ShardStatisticsVector;
 typedef StringMap<StringMap<ShardZoneInfo>> ShardZoneInfoMap;
 
+using ShardDataSizeMap = std::map<ShardId, int64_t>;
+using NamespaceStringToShardDataSizeMap = stdx::unordered_map<NamespaceString, ShardDataSizeMap>;
 /*
  * Keeps track of info needed for data size aware balancing.
  */
 struct CollectionDataSizeInfoForBalancing {
-    CollectionDataSizeInfoForBalancing(std::map<ShardId, int64_t>&& shardToDataSizeMap,
+    CollectionDataSizeInfoForBalancing(ShardDataSizeMap&& shardToDataSizeMap,
                                        long maxChunkSizeBytes)
         : shardToDataSizeMap(std::move(shardToDataSizeMap)), maxChunkSizeBytes(maxChunkSizeBytes) {}
 
-    std::map<ShardId, int64_t> shardToDataSizeMap;
+    ShardDataSizeMap shardToDataSizeMap;
     const int64_t maxChunkSizeBytes;
 };
 
+NamespaceStringToShardDataSizeMap getStatsForBalancing(
+    OperationContext* opCtx,
+    const std::vector<ShardId>& shardIds,
+    const std::vector<NamespaceWithOptionalUUID>& namespacesWithUUIDsForStatsRequest);
+
+ShardDataSizeMap getStatsForBalancing(
+    OperationContext* opCtx,
+    const std::vector<ShardId>& shardIds,
+    const NamespaceWithOptionalUUID& namespaceWithUUIDsForStatsRequest);
+
 /**
  * Keeps track of zones for a collection.
  */
@@ -258,6 +290,13 @@ public:
         return _zoneRanges;
     }
 
+    /**
+     * Retrieves the collection zones from the catalog client
+     */
+    static StatusWith<ZoneInfo> getZonesForCollection(OperationContext* opCtx,
+                                                      const NamespaceString& nss,
+                                                      const KeyPattern& keyPattern);
+
 private:
     // Map of zone max key to the zone description
     BSONObjIndexedMap<ZoneRange> _zoneRanges;
@@ -266,12 +305,6 @@ private:
     std::set<std::string> _allZones;
 };
 
-/**
- * read all tags for collection via the catalog client and add to the zoneInfo
- */
-StatusWith<ZoneInfo> createCollectionZoneInfo(OperationContext* opCtx,
-                                              const NamespaceString& nss,
-                                              const KeyPattern& keyPattern);
 class ChunkManager;
 
 /**
@@ -445,16 +478,6 @@ public:
         stdx::unordered_set<ShardId>* availableShards,
         bool forceJumbo);
 
-    /**
-     * Using the specified distribution information, returns a suggested better location for the
-     * specified chunk if one is available.
-     */
-    static boost::optional<MigrateInfo> balanceSingleChunk(
-        const ChunkType& chunk,
-        const ShardStatisticsVector& shardStats,
-        const DistributionStatus& distribution,
-        const CollectionDataSizeInfoForBalancing& collDataSizeInfo);
-
 private:
     /*
      * Only considers shards with the specified zone, all shards in case the zone is empty.
