@@ -30,27 +30,55 @@
 
 #include "mongo/db/s/collmod_coordinator.h"
 
+#include <absl/container/node_hash_map.h>
+#include <algorithm>
+#include <boost/smart_ptr.hpp>
+#include <set>
+#include <string>
+#include <tuple>
+#include <utility>
+
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/bson/simple_bsonobj_comparator.h"
+#include "mongo/client/connection_string.h"
+#include "mongo/client/read_preference.h"
 #include "mongo/db/catalog/coll_mod.h"
-#include "mongo/db/catalog/collection_catalog.h"
-#include "mongo/db/catalog/collection_uuid_mismatch.h"
-#include "mongo/db/catalog/database_holder.h"
+#include "mongo/db/catalog_raii.h"
+#include "mongo/db/client.h"
 #include "mongo/db/coll_mod_gen.h"
-#include "mongo/db/db_raii.h"
-#include "mongo/db/ops/insert.h"
+#include "mongo/db/commands.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
+#include "mongo/db/s/forwardable_operation_metadata.h"
 #include "mongo/db/s/participant_block_gen.h"
-#include "mongo/db/s/sharded_collmod_gen.h"
 #include "mongo/db/s/sharding_ddl_coordinator_gen.h"
 #include "mongo/db/s/sharding_ddl_util.h"
-#include "mongo/db/s/sharding_state.h"
+#include "mongo/db/session/logical_session_id_gen.h"
 #include "mongo/db/timeseries/catalog_helper.h"
 #include "mongo/db/timeseries/timeseries_collmod.h"
 #include "mongo/db/timeseries/timeseries_options.h"
+#include "mongo/db/write_concern_options.h"
+#include "mongo/executor/async_rpc.h"
+#include "mongo/executor/async_rpc_util.h"
 #include "mongo/idl/idl_parser.h"
-#include "mongo/logv2/log.h"
 #include "mongo/s/async_requests_sender.h"
+#include "mongo/s/catalog/sharding_catalog_client.h"
+#include "mongo/s/catalog/type_collection.h"
+#include "mongo/s/catalog/type_database_gen.h"
+#include "mongo/s/catalog_cache.h"
+#include "mongo/s/chunk_manager.h"
+#include "mongo/s/client/shard.h"
+#include "mongo/s/client/shard_registry.h"
 #include "mongo/s/cluster_commands_helpers.h"
 #include "mongo/s/grid.h"
+#include "mongo/s/sharding_state.h"
 #include "mongo/util/fail_point.h"
+#include "mongo/util/future_impl.h"
+#include "mongo/util/read_through_cache.h"
+#include "mongo/util/str.h"
+#include "mongo/util/uuid.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
@@ -61,15 +89,6 @@ MONGO_FAIL_POINT_DEFINE(collModBeforeConfigServerUpdate);
 
 namespace {
 
-bool isShardedColl(OperationContext* opCtx, const NamespaceString& nss) {
-    try {
-        auto coll = Grid::get(opCtx)->catalogClient()->getCollection(opCtx, nss);
-        return true;
-    } catch (const ExceptionFor<ErrorCodes::NamespaceNotFound>&) {
-        // The collection is not sharded or doesn't exist.
-        return false;
-    }
-}
 
 bool hasTimeSeriesBucketingUpdate(const CollModRequest& request) {
     if (!request.getTimeseries().has_value()) {
@@ -79,6 +98,32 @@ bool hasTimeSeriesBucketingUpdate(const CollModRequest& request) {
     return ts->getGranularity() || ts->getBucketMaxSpanSeconds() || ts->getBucketRoundingSeconds();
 }
 
+template <typename CommandType>
+std::vector<AsyncRequestsSender::Response> sendAuthenticatedCommandWithOsiToShards(
+    OperationContext* opCtx,
+    std::shared_ptr<async_rpc::AsyncRPCOptions<CommandType>> opts,
+    const std::vector<ShardId>& shardIds,
+    const OperationSessionInfo& osi,
+    bool ignoreResponses,
+    WriteConcernOptions wc = WriteConcernOptions()) {
+    async_rpc::AsyncRPCCommandHelpers::appendMajorityWriteConcern(opts->genericArgs, wc);
+    async_rpc::AsyncRPCCommandHelpers::appendOSI(opts->genericArgs, osi);
+    return sharding_ddl_util::sendAuthenticatedCommandToShards(
+        opCtx, opts, shardIds, ignoreResponses);
+}
+
+// Extract the first response from the list of shardResponses, and propagate to the global level of
+// the response
+void _appendResponseCollModIndexChanges(
+    const std::vector<AsyncRequestsSender::Response>& shardResponses, BSONObjBuilder& result) {
+    if (shardResponses.empty() || !shardResponses[0].swResponse.isOK()) {
+        return;
+    }
+
+    auto& firstShardResponse = shardResponses[0].swResponse.getValue().data;
+    result.appendElements(CommandHelpers::filterCommandReplyForPassthrough(firstShardResponse));
+}
+
 }  // namespace
 
 CollModCoordinator::CollModCoordinator(ShardingDDLCoordinatorService* service,
@@ -94,7 +139,7 @@ void CollModCoordinator::checkIfOptionsConflict(const BSONObj& doc) const {
     const auto& otherReq = otherDoc.getCollModRequest().toBSON();
 
     uassert(ErrorCodes::ConflictingOperationInProgress,
-            str::stream() << "Another collMod for namespace " << originalNss()
+            str::stream() << "Another collMod for namespace " << originalNss().toStringForErrorMsg()
                           << " is being executed with different parameters: " << selfReq,
             SimpleBSONObjComparator::kInstance.evaluate(selfReq == otherReq));
 }
@@ -103,11 +148,6 @@ void CollModCoordinator::appendCommandInfo(BSONObjBuilder* cmdInfoBuilder) const
     cmdInfoBuilder->appendElements(_request.toBSON());
 };
 
-// TODO SERVER-68008 Remove once 7.0 becomes last LTS
-bool CollModCoordinator::_isPre61Compatible() const {
-    return operationType() == DDLCoordinatorTypeEnum::kCollModPre61Compatible;
-}
-
 void CollModCoordinator::_performNoopRetryableWriteOnParticipants(
     OperationContext* opCtx, const std::shared_ptr<executor::TaskExecutor>& executor) {
     auto shardsAndConfigsvr = [&] {
@@ -131,7 +171,10 @@ void CollModCoordinator::_saveCollectionInfoOnCoordinatorIfNecessary(OperationCo
         info.timeSeriesOptions = timeseries::getTimeseriesOptions(opCtx, originalNss(), true);
         info.nsForTargeting =
             info.timeSeriesOptions ? originalNss().makeTimeseriesBucketsNamespace() : originalNss();
-        info.isSharded = isShardedColl(opCtx, info.nsForTargeting);
+        const auto optColl =
+            sharding_ddl_util::getCollectionFromConfigServer(opCtx, info.nsForTargeting);
+        info.isTracked = (bool)optColl;
+        info.isSharded = optColl && !optColl->getUnsplittable();
         _collInfo = std::move(info);
     }
 }
@@ -139,7 +182,7 @@ void CollModCoordinator::_saveCollectionInfoOnCoordinatorIfNecessary(OperationCo
 void CollModCoordinator::_saveShardingInfoOnCoordinatorIfNecessary(OperationContext* opCtx) {
     tassert(
         6522700, "Sharding information must be gathered after collection information", _collInfo);
-    if (!_shardingInfo && _collInfo->isSharded) {
+    if (!_shardingInfo && _collInfo->isTracked) {
         ShardingInfo info;
         info.isPrimaryOwningChunks = false;
         const auto [chunkManager, _] = uassertStatusOK(
@@ -180,44 +223,59 @@ void CollModCoordinator::_saveShardingInfoOnCoordinatorIfNecessary(OperationCont
 std::vector<AsyncRequestsSender::Response> CollModCoordinator::_sendCollModToPrimaryShard(
     OperationContext* opCtx,
     ShardsvrCollModParticipant& request,
-    const std::shared_ptr<executor::ScopedTaskExecutor>& executor) {
+    const std::shared_ptr<executor::ScopedTaskExecutor>& executor,
+    const CancellationToken& token) {
     // A view definition will only be present on the primary shard. So we pass an addition
     // 'performViewChange' flag only to the primary shard.
     request.setPerformViewChange(true);
+    auto opts = std::make_shared<async_rpc::AsyncRPCOptions<ShardsvrCollModParticipant>>(
+        **executor, token, request, GenericArguments());
 
-    return sharding_ddl_util::sendAuthenticatedCommandToShards(
-        opCtx,
-        nss().db(),
-        CommandHelpers::appendMajorityWriteConcern(request.toBSON({})),
-        {_shardingInfo->primaryShard},
-        **executor,
-        _shardingInfo->isPrimaryOwningChunks);
+    try {
+        return sendAuthenticatedCommandWithOsiToShards(opCtx,
+                                                       opts,
+                                                       {_shardingInfo->primaryShard},
+                                                       getNewSession(opCtx),
+                                                       false /* ignoreResponses */);
+    } catch (const DBException& ex) {
+        // For a db primary shard not owning chunks only throw retriable errors for the coordinator.
+        if (_shardingInfo->isPrimaryOwningChunks ||
+            _isRetriableErrorForDDLCoordinator(ex.toStatus())) {
+            throw;
+        }
+        return {};
+    }
 }
 
 std::vector<AsyncRequestsSender::Response> CollModCoordinator::_sendCollModToParticipantShards(
     OperationContext* opCtx,
     ShardsvrCollModParticipant& request,
-    const std::shared_ptr<executor::ScopedTaskExecutor>& executor) {
+    const std::shared_ptr<executor::ScopedTaskExecutor>& executor,
+    const CancellationToken& token) {
     request.setPerformViewChange(false);
+    auto opts = std::make_shared<async_rpc::AsyncRPCOptions<ShardsvrCollModParticipant>>(
+        **executor, token, request, GenericArguments());
 
     // The collMod command targets all shards, regardless of whether they have chunks. The shards
-    // that have no chunks for the collection will not throw nor will be included in the responses.
+    // that have no chunks for the collection will not be included in the responses.
 
-    sharding_ddl_util::sendAuthenticatedCommandToShards(
-        opCtx,
-        nss().db(),
-        CommandHelpers::appendMajorityWriteConcern(request.toBSON({})),
-        _shardingInfo->participantsNotOwningChunks,
-        **executor,
-        false /* throwOnError */);
+    for (auto&& shard : _shardingInfo->participantsNotOwningChunks) {
+        try {
+            sendAuthenticatedCommandWithOsiToShards(
+                opCtx, opts, {shard}, getNewSession(opCtx), false /* ignoreResponses */);
+        } catch (const DBException& ex) {
+            // For shards not owning chunks only throw retriable errors for the coordinator.
+            if (_isRetriableErrorForDDLCoordinator(ex.toStatus())) {
+                throw;
+            }
+        }
+    }
 
-    return sharding_ddl_util::sendAuthenticatedCommandToShards(
-        opCtx,
-        nss().db(),
-        CommandHelpers::appendMajorityWriteConcern(request.toBSON({})),
-        _shardingInfo->participantsOwningChunks,
-        **executor,
-        true /* throwOnError */);
+    return sendAuthenticatedCommandWithOsiToShards(opCtx,
+                                                   opts,
+                                                   _shardingInfo->participantsOwningChunks,
+                                                   getNewSession(opCtx),
+                                                   false /* ignoreResponses */);
 }
 
 ExecutorFuture<void> CollModCoordinator::_runImpl(
@@ -234,8 +292,12 @@ ExecutorFuture<void> CollModCoordinator::_runImpl(
             }
 
             {
+                // Implicitly check for collection UUID mismatch - use the 'originalNss()' provided
+                // in the DDL command. Timeseries collections will always throw
+                // CollectionUUIDMismatch, as the collection name is on a view, which doesn't have a
+                // UUID.
                 AutoGetCollection coll{opCtx,
-                                       nss(),
+                                       originalNss(),
                                        MODE_IS,
                                        AutoGetCollection::Options{}
                                            .viewMode(auto_get_collection::ViewMode::kViewsPermitted)
@@ -254,9 +316,6 @@ ExecutorFuture<void> CollModCoordinator::_runImpl(
             }
         })
         .then([this, executor = executor, anchor = shared_from_this()] {
-            if (_isPre61Compatible()) {
-                return;
-            }
             _buildPhaseHandler(
                 Phase::kFreezeMigrations, [this, executor = executor, anchor = shared_from_this()] {
                     auto opCtxHolder = cc().makeOperationContext();
@@ -265,54 +324,38 @@ ExecutorFuture<void> CollModCoordinator::_runImpl(
 
                     _saveCollectionInfoOnCoordinatorIfNecessary(opCtx);
 
-                    if (_collInfo->isSharded) {
-                        _doc.setCollUUID(
-                            sharding_ddl_util::getCollectionUUID(opCtx, _collInfo->nsForTargeting));
+                    if (_collInfo->isTracked) {
+                        const auto& collUUID =
+                            sharding_ddl_util::getCollectionUUID(opCtx, _collInfo->nsForTargeting);
+                        _doc.setCollUUID(collUUID);
                         sharding_ddl_util::stopMigrations(
-                            opCtx, _collInfo->nsForTargeting, _doc.getCollUUID());
+                            opCtx, _collInfo->nsForTargeting, collUUID, getNewSession(opCtx));
                     }
                 })();
         })
         .then(_buildPhaseHandler(
             Phase::kBlockShards,
-            [this, executor = executor, anchor = shared_from_this()] {
+            [this, token, executor = executor, anchor = shared_from_this()] {
                 auto opCtxHolder = cc().makeOperationContext();
                 auto* opCtx = opCtxHolder.get();
                 getForwardableOpMetadata().setOn(opCtx);
 
                 _saveCollectionInfoOnCoordinatorIfNecessary(opCtx);
-
-                if (_isPre61Compatible() && _collInfo->isSharded) {
-                    const auto migrationsAlreadyBlockedForBucketNss =
-                        hasTimeSeriesBucketingUpdate(_request) &&
-                        _doc.getMigrationsAlreadyBlockedForBucketNss();
-
-                    if (!migrationsAlreadyBlockedForBucketNss) {
-                        _doc.setCollUUID(sharding_ddl_util::getCollectionUUID(
-                            opCtx, _collInfo->nsForTargeting, true /* allowViews */));
-                        sharding_ddl_util::stopMigrations(
-                            opCtx, _collInfo->nsForTargeting, _doc.getCollUUID());
-                    }
-                }
-
                 _saveShardingInfoOnCoordinatorIfNecessary(opCtx);
 
-                if (_collInfo->isSharded && hasTimeSeriesBucketingUpdate(_request)) {
-                    if (_isPre61Compatible()) {
-                        auto newDoc = _doc;
-                        newDoc.setMigrationsAlreadyBlockedForBucketNss(true);
-                        _updateStateDocument(opCtx, std::move(newDoc));
-                    }
-
+                if (_collInfo->isTracked && hasTimeSeriesBucketingUpdate(_request)) {
                     ShardsvrParticipantBlock blockCRUDOperationsRequest(_collInfo->nsForTargeting);
-                    const auto cmdObj = CommandHelpers::appendMajorityWriteConcern(
-                        blockCRUDOperationsRequest.toBSON({}));
+                    blockCRUDOperationsRequest.setBlockType(
+                        CriticalSectionBlockTypeEnum::kReadsAndWrites);
+                    auto opts =
+                        std::make_shared<async_rpc::AsyncRPCOptions<ShardsvrParticipantBlock>>(
+                            **executor, token, blockCRUDOperationsRequest, GenericArguments());
                     std::vector<ShardId> shards = _shardingInfo->participantsOwningChunks;
                     if (_shardingInfo->isPrimaryOwningChunks) {
                         shards.push_back(_shardingInfo->primaryShard);
                     }
-                    sharding_ddl_util::sendAuthenticatedCommandToShards(
-                        opCtx, nss().db(), cmdObj, shards, **executor, true /* throwOnError */);
+                    sendAuthenticatedCommandWithOsiToShards(
+                        opCtx, opts, shards, getNewSession(opCtx), false /* ignoreResponses */);
                 }
             }))
         .then(_buildPhaseHandler(
@@ -327,7 +370,7 @@ ExecutorFuture<void> CollModCoordinator::_runImpl(
                 _saveCollectionInfoOnCoordinatorIfNecessary(opCtx);
                 _saveShardingInfoOnCoordinatorIfNecessary(opCtx);
 
-                if (_collInfo->isSharded && _collInfo->timeSeriesOptions &&
+                if (_collInfo->isTracked && _collInfo->timeSeriesOptions &&
                     hasTimeSeriesBucketingUpdate(_request)) {
                     ConfigsvrCollMod request(_collInfo->nsForTargeting, _request);
                     const auto cmdObj =
@@ -337,14 +380,13 @@ ExecutorFuture<void> CollModCoordinator::_runImpl(
                     uassertStatusOK(Shard::CommandResponse::getEffectiveStatus(
                         configShard->runCommand(opCtx,
                                                 ReadPreferenceSetting(ReadPreference::PrimaryOnly),
-                                                nss().db().toString(),
+                                                nss().dbName(),
                                                 cmdObj,
                                                 Shard::RetryPolicy::kIdempotent)));
                 }
             }))
         .then(_buildPhaseHandler(
-            Phase::kUpdateShards,
-            [this, executor = executor, anchor = shared_from_this()] {
+            Phase::kUpdateShards, [this, token, executor = executor, anchor = shared_from_this()] {
                 auto opCtxHolder = cc().makeOperationContext();
                 auto* opCtx = opCtxHolder.get();
                 getForwardableOpMetadata().setOn(opCtx);
@@ -352,9 +394,9 @@ ExecutorFuture<void> CollModCoordinator::_runImpl(
                 _saveCollectionInfoOnCoordinatorIfNecessary(opCtx);
                 _saveShardingInfoOnCoordinatorIfNecessary(opCtx);
 
-                if (_collInfo->isSharded) {
+                if (_collInfo->isTracked) {
                     try {
-                        if (!_firstExecution) {
+                        if (!_firstExecution && _collInfo->isSharded) {
                             bool allowMigrations = sharding_ddl_util::checkAllowMigrations(
                                 opCtx, _collInfo->nsForTargeting);
                             if (_result.is_initialized() && allowMigrations) {
@@ -362,24 +404,16 @@ ExecutorFuture<void> CollModCoordinator::_runImpl(
                                 return;
                             } else if (allowMigrations) {
                                 // Previous run on a different node completed, but we lost the
-                                // result in the stepdown. Restart from stage in which we disallow
-                                // migrations.
-                                auto newPhase = _isPre61Compatible() ? Phase::kBlockShards
-                                                                     : Phase::kFreezeMigrations;
-                                _enterPhase(newPhase);
+                                // result in the stepdown. Restart from kFreezeMigrations.
+                                _enterPhase(Phase::kFreezeMigrations);
                                 uasserted(ErrorCodes::Interrupted,
                                           "Retriable error to move to previous stage");
                             }
                         }
 
-                        ShardsvrCollModParticipant request(originalNss(), _request);
-                        bool needsUnblock =
-                            _collInfo->timeSeriesOptions && hasTimeSeriesBucketingUpdate(_request);
-                        request.setNeedsUnblock(needsUnblock);
-
-                        // If trying to convert an index to unique, executes a dryRun first to find
-                        // any duplicates without actually changing the indexes to avoid
-                        // inconsistent index specs on different shards. Example:
+                        // If trying to convert an index to unique on a sharded collection, executes
+                        // a dryRun first to find any duplicates without actually changing the
+                        // indexes to avoid inconsistent index specs on different shards. Example:
                         //   Shard0: {_id: 0, a: 1}
                         //   Shard1: {_id: 1, a: 2}, {_id: 2, a: 2}
                         //   When trying to convert index {a: 1} to unique, the dry run will return
@@ -389,28 +423,31 @@ ExecutorFuture<void> CollModCoordinator::_runImpl(
                             // strip out other incompatible options.
                             auto dryRunRequest = ShardsvrCollModParticipant{
                                 originalNss(), makeCollModDryRunRequest(_request)};
+                            GenericArguments args;
+                            async_rpc::AsyncRPCCommandHelpers::appendMajorityWriteConcern(args);
+                            auto optsDryRun = std::make_shared<
+                                async_rpc::AsyncRPCOptions<ShardsvrCollModParticipant>>(
+                                **executor, token, dryRunRequest, args);
                             std::vector<ShardId> shards = _shardingInfo->participantsOwningChunks;
                             if (_shardingInfo->isPrimaryOwningChunks) {
                                 shards.push_back(_shardingInfo->primaryShard);
                             }
                             sharding_ddl_util::sendAuthenticatedCommandToShards(
-                                opCtx,
-                                nss().db(),
-                                CommandHelpers::appendMajorityWriteConcern(
-                                    dryRunRequest.toBSON({})),
-                                shards,
-                                **executor);
+                                opCtx, optsDryRun, shards);
                         }
 
+                        ShardsvrCollModParticipant request(originalNss(), _request);
+                        bool needsUnblock =
+                            _collInfo->timeSeriesOptions && hasTimeSeriesBucketingUpdate(_request);
+                        request.setNeedsUnblock(needsUnblock);
+
                         std::vector<AsyncRequestsSender::Response> responses;
 
-                        // In the case of the participants, we are broadcasting the collMod to all
-                        // the shards. On one hand, if the shard contains chunks for the
-                        // collections, we parse all the responses. On the other hand, if the shard
-                        // does not contain chunks, we make a best effort to not process the
-                        // returned responses or throw any errors.
+                        // We are broadcasting the collMod to all the shards, but only appending the
+                        // participants' responses from those owning chunks.
 
-                        auto primaryResponse = _sendCollModToPrimaryShard(opCtx, request, executor);
+                        auto primaryResponse =
+                            _sendCollModToPrimaryShard(opCtx, request, executor, token);
                         if (_shardingInfo->isPrimaryOwningChunks) {
                             responses.insert(responses.end(),
                                              std::make_move_iterator(primaryResponse.begin()),
@@ -418,7 +455,7 @@ ExecutorFuture<void> CollModCoordinator::_runImpl(
                         }
 
                         auto participantsResponses =
-                            _sendCollModToParticipantShards(opCtx, request, executor);
+                            _sendCollModToParticipantShards(opCtx, request, executor, token);
                         responses.insert(responses.end(),
                                          std::make_move_iterator(participantsResponses.begin()),
                                          std::make_move_iterator(participantsResponses.end()));
@@ -426,18 +463,33 @@ ExecutorFuture<void> CollModCoordinator::_runImpl(
 
                         BSONObjBuilder builder;
                         std::string errmsg;
-                        auto ok =
-                            appendRawResponses(opCtx, &errmsg, &builder, responses).responseOK;
+                        bool ok = [&]() {
+                            BSONObjBuilder rawBuilder;
+                            bool ok = appendRawResponses(opCtx, &errmsg, &rawBuilder, responses)
+                                          .responseOK;
+                            BSONObj extractedObjFromRaw = rawBuilder.obj();
+                            if (ok) {
+                                extractedObjFromRaw = extractedObjFromRaw.removeField("raw");
+                                _appendResponseCollModIndexChanges(responses, builder);
+                            }
+                            builder.appendElements(extractedObjFromRaw);
+                            return ok;
+                        }();
+
                         if (!errmsg.empty()) {
                             CommandHelpers::appendSimpleCommandStatus(builder, ok, errmsg);
                         }
+
                         _result = builder.obj();
+
+                        const auto collUUID = _doc.getCollUUID();
                         sharding_ddl_util::resumeMigrations(
-                            opCtx, _collInfo->nsForTargeting, _doc.getCollUUID());
+                            opCtx, _collInfo->nsForTargeting, collUUID, getNewSession(opCtx));
                     } catch (DBException& ex) {
                         if (!_isRetriableErrorForDDLCoordinator(ex.toStatus())) {
+                            const auto collUUID = _doc.getCollUUID();
                             sharding_ddl_util::resumeMigrations(
-                                opCtx, _collInfo->nsForTargeting, _doc.getCollUUID());
+                                opCtx, _collInfo->nsForTargeting, collUUID, getNewSession(opCtx));
                         }
                         throw;
                     }
@@ -449,28 +501,13 @@ ExecutorFuture<void> CollModCoordinator::_runImpl(
                         opCtx, originalNss(), cmd, true, &collModResBuilder));
                     auto collModRes = collModResBuilder.obj();
 
-                    const auto dbInfo = uassertStatusOK(
-                        Grid::get(opCtx)->catalogCache()->getDatabase(opCtx, nss().db()));
-                    const auto shard = uassertStatusOK(
-                        Grid::get(opCtx)->shardRegistry()->getShard(opCtx, dbInfo->getPrimary()));
+                    const auto shard = uassertStatusOK(Grid::get(opCtx)->shardRegistry()->getShard(
+                        opCtx, ShardingState::get(opCtx)->shardId()));
                     BSONObjBuilder builder;
                     builder.appendElements(collModRes);
-                    BSONObjBuilder subBuilder(builder.subobjStart("raw"));
-                    subBuilder.append(shard->getConnString().toString(), collModRes);
-                    subBuilder.doneFast();
                     _result = builder.obj();
                 }
-            }))
-        .onError([this, anchor = shared_from_this()](const Status& status) {
-            if (!status.isA<ErrorCategory::NotPrimaryError>() &&
-                !status.isA<ErrorCategory::ShutdownError>()) {
-                LOGV2_ERROR(5757002,
-                            "Error running collMod",
-                            logAttrs(nss()),
-                            "error"_attr = redact(status));
-            }
-            return status;
-        });
+            }));
 }
 
 }  // namespace mongo
