@@ -29,66 +29,33 @@
 
 #pragma once
 
+#include <boost/move/utility_core.hpp>
 #include <boost/optional.hpp>
+#include <boost/optional/optional.hpp>
 #include <memory>
+#include <set>
 
+#include "mongo/base/status.h"
+#include "mongo/base/string_data.h"
 #include "mongo/bson/bsonobj.h"
+#include "mongo/crypto/fle_stats_gen.h"
 #include "mongo/db/commands/fle2_compact.h"
+#include "mongo/db/commands/fle2_compact_gen.h"
 #include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/ops/write_ops.h"
+#include "mongo/db/pipeline/process_interface/mongo_process_interface.h"
 #include "mongo/db/s/compact_structured_encryption_data_coordinator_gen.h"
 #include "mongo/db/s/sharding_ddl_coordinator.h"
+#include "mongo/db/s/sharding_ddl_coordinator_service.h"
+#include "mongo/executor/scoped_task_executor.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/cancellation.h"
+#include "mongo/util/future.h"
+#include "mongo/util/uuid.h"
 
 namespace mongo {
 
-// TODO: SERVER-68373 remove once 7.0 becomes last LTS
-class CompactStructuredEncryptionDataCoordinatorPre70Compatible final
-    : public RecoverableShardingDDLCoordinator<CompactStructuredEncryptionDataStatePre70Compatible,
-                                               CompactStructuredEncryptionDataPhaseEnum> {
-public:
-    static constexpr auto kStateContext = "CompactStructuredEncryptionDataStatePre70Compatible"_sd;
-    using StateDoc = CompactStructuredEncryptionDataStatePre70Compatible;
-    using Phase = CompactStructuredEncryptionDataPhaseEnum;
-
-    CompactStructuredEncryptionDataCoordinatorPre70Compatible(
-        ShardingDDLCoordinatorService* service, const BSONObj& doc)
-        : RecoverableShardingDDLCoordinator(
-              service, "CompactStructuredEncryptionDataCoordinatorPre70Compatible", doc) {}
-
-    boost::optional<BSONObj> reportForCurrentOp(
-        MongoProcessInterface::CurrentOpConnectionsMode connMode,
-        MongoProcessInterface::CurrentOpSessionsMode sessionMode) noexcept final;
-
-    CompactStructuredEncryptionDataCommandReply getResponse(OperationContext* opCtx) {
-        getCompletionFuture().get(opCtx);
-        invariant(_response);
-        return *_response;
-    }
-
-    void checkIfOptionsConflict(const BSONObj& stateDoc) const final {}
-
-private:
-    StringData serializePhase(const Phase& phase) const override {
-        return CompactStructuredEncryptionDataPhase_serializer(phase);
-    }
-
-    ExecutorFuture<void> _runImpl(std::shared_ptr<executor::ScopedTaskExecutor> executor,
-                                  const CancellationToken& token) noexcept final;
-
-    // TODO SERVER-68373 remove once 7.0 becomes last LTS
-    bool _isPre61Compatible() const {
-        return operationType() ==
-            DDLCoordinatorTypeEnum::kCompactStructuredEncryptionDataPre61Compatible;
-    }
-
-    // TODO SERVER-68373 remove once 7.0 becomes last LTS
-    void _enterPhase(const Phase& newPhase) override;
-
-private:
-    boost::optional<CompactStructuredEncryptionDataCommandReply> _response;
-    bool _skipCompact{false};
-    boost::optional<UUID> _ecocRenameUuid;
-};
-
 class CompactStructuredEncryptionDataCoordinator final
     : public RecoverableShardingDDLCoordinator<CompactStructuredEncryptionDataState,
                                                CompactStructuredEncryptionDataPhaseEnum> {
@@ -122,6 +89,8 @@ private:
     ExecutorFuture<void> _runImpl(std::shared_ptr<executor::ScopedTaskExecutor> executor,
                                   const CancellationToken& token) noexcept final;
 
+    std::set<NamespaceString> _getAdditionalLocksToAcquire(OperationContext* opCtx) override;
+
 private:
     // The response to the compact command
     boost::optional<CompactStructuredEncryptionDataCommandReply> _response;
