@@ -28,12 +28,58 @@
  */
 
 
+#include <boost/cstdint.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <cstdint>
+#include <iterator>
+#include <memory>
+#include <string>
+#include <utility>
+#include <vector>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bson_field.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/client/read_preference.h"
+#include "mongo/db/api_parameters.h"
+#include "mongo/db/auth/action_type.h"
 #include "mongo/db/auth/authorization_session.h"
+#include "mongo/db/auth/privilege.h"
+#include "mongo/db/auth/resource_pattern.h"
+#include "mongo/db/basic_types_gen.h"
+#include "mongo/db/clientcursor.h"
+#include "mongo/db/cluster_role.h"
 #include "mongo/db/commands.h"
+#include "mongo/db/database_name.h"
+#include "mongo/db/metadata_consistency_types_gen.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/pipeline/aggregate_command_gen.h"
+#include "mongo/db/query/cursor_response_gen.h"
+#include "mongo/db/query/plan_executor.h"
+#include "mongo/db/query/query_request_helper.h"
+#include "mongo/db/repl/read_concern_args.h"
+#include "mongo/db/repl/read_concern_level.h"
 #include "mongo/db/s/config/sharding_catalog_manager.h"
 #include "mongo/db/s/metadata_consistency_util.h"
-#include "mongo/s/client/shard_remote_gen.h"
+#include "mongo/db/server_options.h"
+#include "mongo/db/service_context.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/rpc/op_msg.h"
+#include "mongo/s/catalog/sharding_catalog_client.h"
+#include "mongo/s/catalog/type_chunk.h"
+#include "mongo/s/catalog/type_collection.h"
+#include "mongo/s/catalog/type_tags.h"
 #include "mongo/s/request_types/sharded_ddl_commands_gen.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/duration.h"
+#include "mongo/util/str.h"
+#include "mongo/util/uuid.h"
 
 namespace mongo {
 namespace {
@@ -78,10 +124,9 @@ public:
             std::vector<MetadataInconsistencyItem> inconsistenciesMerged;
             const auto catalogClient = ShardingCatalogManager::get(opCtx)->localCatalogClient();
 
-            const auto commandLevel = metadata_consistency_util::getCommandLevel(nss);
-            switch (commandLevel) {
+            switch (metadata_consistency_util::getCommandLevel(nss)) {
                 case MetadataConsistencyCommandLevelEnum::kDatabaseLevel: {
-                    const auto collections = catalogClient->getCollections(opCtx, nss.db());
+                    const auto collections = catalogClient->getCollections(opCtx, nss.dbName());
 
                     for (const auto& coll : collections) {
                         _runChecksForCollection(opCtx, coll, inconsistenciesMerged);
@@ -98,14 +143,10 @@ public:
                     break;
                 }
                 default:
-                    tasserted(1011702,
+                    uasserted(ErrorCodes::IllegalOperation,
                               str::stream()
-                                  << "Unexpected parameter during the internal execution of "
-                                     "checkMetadataConsistency command. The config server was "
-                                     "expecting to receive a database or collection level "
-                                     "parameter, but received "
-                                  << MetadataConsistencyCommandLevel_serializer(commandLevel)
-                                  << " with namespace " << nss.toStringForErrorMsg());
+                                  << Request::kCommandName
+                                  << " can only be run over a specific collection or database");
             }
 
             auto exec = metadata_consistency_util::makeQueuedPlanExecutor(
@@ -120,14 +161,15 @@ public:
                 repl::ReadConcernArgs::get(opCtx),
                 ReadPreferenceSetting::get(opCtx),
                 request().toBSON({}),
-                {Privilege(ResourcePattern::forClusterResource(), ActionType::internal)}};
+                {Privilege(ResourcePattern::forClusterResource(nss.tenantId()),
+                           ActionType::internal)}};
 
             const auto batchSize = [&]() -> long long {
                 const auto& cursorOpts = request().getCursor();
                 if (cursorOpts && cursorOpts->getBatchSize()) {
                     return *cursorOpts->getBatchSize();
                 } else {
-                    return query_request_helper::kDefaultBatchSize;
+                    return query_request_helper::getDefaultBatchSize();
                 }
             }();
 
@@ -140,14 +182,22 @@ public:
             OperationContext* opCtx,
             const CollectionType& coll,
             std::vector<MetadataInconsistencyItem>& inconsistenciesMerged) {
-            auto chunksInconsistencies = metadata_consistency_util::checkChunksInconsistencies(
+            auto chunksInconsistencies = metadata_consistency_util::checkChunksConsistency(
                 opCtx, coll, _getCollectionChunks(opCtx, coll));
 
             inconsistenciesMerged.insert(inconsistenciesMerged.end(),
                                          std::make_move_iterator(chunksInconsistencies.begin()),
                                          std::make_move_iterator(chunksInconsistencies.end()));
 
-            auto zonesInconsistencies = metadata_consistency_util::checkZonesInconsistencies(
+            auto collectionsInconsistencies =
+                metadata_consistency_util::checkCollectionShardingMetadataConsistency(opCtx, coll);
+
+            inconsistenciesMerged.insert(
+                inconsistenciesMerged.end(),
+                std::make_move_iterator(collectionsInconsistencies.begin()),
+                std::make_move_iterator(collectionsInconsistencies.end()));
+
+            auto zonesInconsistencies = metadata_consistency_util::checkZonesConsistency(
                 opCtx, coll, _getCollectionZones(opCtx, coll.getNss()));
 
             inconsistenciesMerged.insert(inconsistenciesMerged.end(),
@@ -196,12 +246,13 @@ public:
             uassert(ErrorCodes::Unauthorized,
                     "Unauthorized",
                     AuthorizationSession::get(opCtx->getClient())
-                        ->isAuthorizedForActionsOnResource(ResourcePattern::forClusterResource(),
-                                                           ActionType::internal));
+                        ->isAuthorizedForActionsOnResource(
+                            ResourcePattern::forClusterResource(request().getDbName().tenantId()),
+                            ActionType::internal));
         }
     };
-
-} configsvrCheckMetadataConsistencyCommand;
+};
+MONGO_REGISTER_COMMAND(ConfigsvrCheckMetadataConsistencyCommand).forShard();
 
 }  // namespace
 }  // namespace mongo
