@@ -28,14 +28,31 @@
  */
 
 
-#include "mongo/base/status_with.h"
+#include <memory>
+#include <string>
+#include <utility>
+
+#include <boost/move/utility_core.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/string_data.h"
 #include "mongo/db/auth/action_type.h"
 #include "mongo/db/auth/authorization_session.h"
+#include "mongo/db/auth/resource_pattern.h"
+#include "mongo/db/cluster_role.h"
 #include "mongo/db/commands.h"
+#include "mongo/db/database_name.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/repl/read_concern_args.h"
+#include "mongo/db/repl/read_concern_level.h"
 #include "mongo/db/s/config/sharding_catalog_manager.h"
-#include "mongo/s/grid.h"
+#include "mongo/db/server_options.h"
+#include "mongo/db/service_context.h"
+#include "mongo/rpc/op_msg.h"
 #include "mongo/s/request_types/merge_chunk_request_gen.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/str.h"
 
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
@@ -89,11 +106,9 @@ public:
                     request().getMaxNumberOfChunksToMerge(),
                     request().getMaxTimeProcessingChunksMS()));
 
-            auto shardAndCollVers = response.first;
-            auto numMergedChunks = response.second;
-            auto res = MergeAllChunksOnShardResponse{shardAndCollVers.shardPlacementVersion};
-            res.setNumMergedChunks(numMergedChunks);
-            return res;
+            const auto& [shardAndCollVers, numMergedChunks] = response;
+            return MergeAllChunksOnShardResponse{shardAndCollVers.shardPlacementVersion,
+                                                 numMergedChunks};
         }
 
     private:
@@ -109,12 +124,13 @@ public:
             uassert(ErrorCodes::Unauthorized,
                     "Unauthorized",
                     AuthorizationSession::get(opCtx->getClient())
-                        ->isAuthorizedForActionsOnResource(ResourcePattern::forClusterResource(),
-                                                           ActionType::internal));
+                        ->isAuthorizedForActionsOnResource(
+                            ResourcePattern::forClusterResource(request().getDbName().tenantId()),
+                            ActionType::internal));
         }
     };
-
-} configSvrCommitMergeAllChunksOnShard;
+};
+MONGO_REGISTER_COMMAND(ConfigSvrCommitMergeAllChunksOnShardCommand).forShard();
 
 }  // namespace
 }  // namespace mongo
