@@ -28,16 +28,59 @@
  */
 
 
+#include <cstddef>
+#include <cstdint>
+#include <list>
+#include <memory>
+#include <set>
+#include <string>
+#include <utility>
+#include <vector>
+
+#include <absl/container/node_hash_map.h>
+#include <boost/cstdint.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/none_t.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/client/dbclient_cursor.h"
+#include "mongo/db/auth/action_type.h"
 #include "mongo/db/auth/authorization_session.h"
-#include "mongo/db/catalog/create_collection.h"
-#include "mongo/db/catalog_raii.h"
+#include "mongo/db/auth/resource_pattern.h"
+#include "mongo/db/catalog/collection.h"
+#include "mongo/db/catalog/collection_catalog.h"
 #include "mongo/db/commands.h"
+#include "mongo/db/database_name.h"
 #include "mongo/db/dbdirectclient.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/query/find_command.h"
+#include "mongo/db/repl/repl_settings.h"
 #include "mongo/db/repl/replication_coordinator.h"
-#include "mongo/db/repl/storage_interface_impl.h"
+#include "mongo/db/repl/storage_interface.h"
+#include "mongo/db/s/config/known_collections.h"
+#include "mongo/db/service_context.h"
+#include "mongo/db/storage/storage_options.h"
+#include "mongo/db/tenant_id.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
 #include "mongo/stdx/unordered_map.h"
 #include "mongo/util/assert_util.h"
+#include "mongo/util/database_name_util.h"
+#include "mongo/util/namespace_string_util.h"
+#include "mongo/util/str.h"
+#include "mongo/util/string_map.h"
+#include "mongo/util/testing_proctor.h"
 #include "mongo/util/uuid.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
@@ -63,9 +106,12 @@ ShouldRestoreDocument shouldRestoreDocument(OperationContext* opCtx,
 
     auto findRequest = FindCommandRequest(NamespaceString::kConfigsvrRestoreNamespace);
     if (nss && uuid) {
-        findRequest.setFilter(BSON("ns" << nss->toString() << "uuid" << *uuid));
+        findRequest.setFilter(
+            BSON("ns" << NamespaceStringUtil::serialize(*nss, SerializationContext::stateDefault())
+                      << "uuid" << *uuid));
     } else if (nss) {
-        findRequest.setFilter(BSON("ns" << nss->toString()));
+        findRequest.setFilter(BSON(
+            "ns" << NamespaceStringUtil::serialize(*nss, SerializationContext::stateDefault())));
     } else if (uuid) {
         findRequest.setFilter(BSON("uuid" << *uuid));
     }
@@ -86,9 +132,10 @@ ShouldRestoreDocument shouldRestoreDocument(OperationContext* opCtx,
                 (void)UUID::parse(doc);
             } catch (const AssertionException&) {
                 uasserted(ErrorCodes::BadValue,
-                          str::stream() << "The uuid field of '" << doc.toString() << "' in '"
-                                        << NamespaceString::kConfigsvrRestoreNamespace.toString()
-                                        << "' needs to be of type UUID");
+                          str::stream()
+                              << "The uuid field of '" << doc.toString() << "' in '"
+                              << NamespaceString::kConfigsvrRestoreNamespace.toStringForErrorMsg()
+                              << "' needs to be of type UUID");
             }
         }
     }
@@ -108,8 +155,9 @@ std::set<std::string> getDatabasesToRestore(OperationContext* opCtx) {
             continue;
         }
 
-        NamespaceString nss(doc.getStringField("ns"));
-        databasesToRestore.emplace(nss.db());
+        NamespaceString nss = NamespaceStringUtil::deserialize(
+            boost::none, doc.getStringField("ns"), SerializationContext::stateDefault());
+        databasesToRestore.emplace(nss.db_forSharding());
     }
 
     return databasesToRestore;
@@ -145,7 +193,7 @@ public:
         return AllowedOnSecondary::kNever;
     }
 
-    bool adminOnly() const {
+    bool adminOnly() const override {
         return true;
     }
 
@@ -154,11 +202,12 @@ public:
     }
 
     Status checkAuthForOperation(OperationContext* opCtx,
-                                 const DatabaseName&,
+                                 const DatabaseName& dbName,
                                  const BSONObj&) const override {
         if (!AuthorizationSession::get(opCtx->getClient())
-                 ->isAuthorizedForActionsOnResource(ResourcePattern::forClusterResource(),
-                                                    ActionType::internal)) {
+                 ->isAuthorizedForActionsOnResource(
+                     ResourcePattern::forClusterResource(dbName.tenantId()),
+                     ActionType::internal)) {
             return Status(ErrorCodes::Unauthorized, "Unauthorized");
         }
         return Status::OK();
@@ -169,8 +218,9 @@ public:
              const BSONObj& cmdObj,
              BSONObjBuilder& result) override {
         uassert(ErrorCodes::CommandFailed,
-                "This command can only be used in standalone mode",
-                !repl::ReplicationCoordinator::get(opCtx)->getSettings().usingReplSets());
+                "This command can only be used in standalone mode or for magicRestore",
+                !repl::ReplicationCoordinator::get(opCtx)->getSettings().isReplSet() ||
+                    storageGlobalParams.magicRestore);
 
         uassert(ErrorCodes::CommandFailed,
                 "This command can only be run during a restore procedure",
@@ -181,10 +231,30 @@ public:
             // this command.
             CollectionPtr restoreColl(CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(
                 opCtx, NamespaceString::kConfigsvrRestoreNamespace));
-            uassert(ErrorCodes::NamespaceNotFound,
-                    str::stream() << "Collection " << NamespaceString::kConfigsvrRestoreNamespace
-                                  << " is missing",
-                    restoreColl);
+            uassert(
+                ErrorCodes::NamespaceNotFound,
+                str::stream() << "Collection "
+                              << NamespaceString::kConfigsvrRestoreNamespace.toStringForErrorMsg()
+                              << " is missing",
+                restoreColl);
+        }
+
+        DBDirectClient client(opCtx);
+
+        if (TestingProctor::instance().isEnabled()) {
+            // All collections in the config server must be defined in kConfigCollections.
+            // Collections to restore should be defined in kCollectionEntries.
+            auto collInfos = client.getCollectionInfos(DatabaseName::kConfig);
+            for (auto&& info : collInfos) {
+                StringData collName = info.getStringField("name");
+                // Ignore cache collections as they will be dropped later in the restore procedure.
+                if (kConfigCollections.find(collName) == kConfigCollections.end() &&
+                    !collName.startsWith("cache")) {
+                    LOGV2_FATAL(6863300,
+                                "Identified unknown collection in config server.",
+                                "collName"_attr = collName);
+                }
+            }
         }
 
         for (const auto& collectionEntry : kCollectionEntries) {
@@ -201,7 +271,6 @@ public:
                 continue;
             }
 
-            DBDirectClient client(opCtx);
             auto findRequest = FindCommandRequest(nss);
             auto cursor = client.find(findRequest);
 
@@ -217,10 +286,15 @@ public:
                         // Handles the "_id.namespace" case for collection
                         // "config.system.sharding_ddl_coordinators".
                         const auto obj = doc.getField(nssFieldName->substr(0, dotPosition)).Obj();
-                        docNss = NamespaceString(
-                            obj.getStringField(nssFieldName->substr(dotPosition + 1)));
+                        docNss = NamespaceStringUtil::deserialize(
+                            boost::none,
+                            obj.getStringField(nssFieldName->substr(dotPosition + 1)),
+                            SerializationContext::stateDefault());
                     } else {
-                        docNss = NamespaceString(doc.getStringField(*nssFieldName));
+                        docNss =
+                            NamespaceStringUtil::deserialize(boost::none,
+                                                             doc.getStringField(*nssFieldName),
+                                                             SerializationContext::stateDefault());
                     }
                 }
 
@@ -236,6 +310,12 @@ public:
                                 "docUUID"_attr = docUUID);
                 }
 
+                // Time-series bucket collection namespaces are reported using the view namespace in
+                // $backupCursor.
+                if (docNss && docNss->isTimeseriesBucketsCollection()) {
+                    docNss = docNss->getTimeseriesViewNamespace();
+                }
+
                 ShouldRestoreDocument shouldRestore = shouldRestoreDocument(opCtx, docNss, docUUID);
 
                 LOGV2_DEBUG(6261302,
@@ -258,7 +338,7 @@ public:
                             logAttrs(coll->ns().dbName()),
                             "uuid"_attr = coll->uuid(),
                             "_id"_attr = doc.getField("_id"));
-                NamespaceStringOrUUID nssOrUUID(coll->ns().db().toString(), coll->uuid());
+                NamespaceStringOrUUID nssOrUUID(coll->ns().dbName(), coll->uuid());
                 uassertStatusOK(repl::StorageInterface::get(opCtx)->deleteById(
                     opCtx, nssOrUUID, doc.getField("_id")));
             }
@@ -290,14 +370,18 @@ public:
                 while (cursor->more()) {
                     auto doc = cursor->next();
 
-                    const NamespaceString dbNss = NamespaceString(doc.getStringField("_id"));
+                    const NamespaceString dbNss =
+                        NamespaceStringUtil::deserialize(boost::none,
+                                                         doc.getStringField("_id"),
+                                                         SerializationContext::stateDefault());
                     if (!dbNss.coll().empty()) {
                         // We want to handle database only namespaces.
                         continue;
                     }
 
                     bool shouldRestore =
-                        databasesRestored.find(dbNss.db().toString()) != databasesRestored.end();
+                        databasesRestored.find(dbNss.db_forSharding().toString()) !=
+                        databasesRestored.end();
 
                     LOGV2_DEBUG(6261305,
                                 1,
@@ -319,7 +403,7 @@ public:
                                 logAttrs(coll->ns().dbName()),
                                 "uuid"_attr = coll->uuid(),
                                 "_id"_attr = doc.getField("_id"));
-                    NamespaceStringOrUUID nssOrUUID(coll->ns().db().toString(), coll->uuid());
+                    NamespaceStringOrUUID nssOrUUID(coll->ns().dbName(), coll->uuid());
                     uassertStatusOK(repl::StorageInterface::get(opCtx)->deleteById(
                         opCtx, nssOrUUID, doc.getField("_id")));
                 }
@@ -328,8 +412,8 @@ public:
 
         return true;
     }
-
-} runRestoreCmd;
+};
+MONGO_REGISTER_COMMAND(ConfigSvrRunRestoreCommand).forShard();
 
 }  // namespace
 }  // namespace mongo
