@@ -29,21 +29,56 @@
 
 #include "mongo/db/s/config/initial_split_policy.h"
 
+#include <absl/container/flat_hash_map.h>
+#include <absl/container/node_hash_map.h>
+#include <absl/container/node_hash_set.h>
+#include <absl/meta/type_traits.h>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <limits>
+#include <numeric>
+#include <random>
+#include <set>
+#include <string>
+#include <type_traits>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/bson/oid.h"
+#include "mongo/bson/simple_bsonobj_comparator.h"
 #include "mongo/client/read_preference.h"
-#include "mongo/db/bson/dotted_path_support.h"
+#include "mongo/db/cluster_role.h"
 #include "mongo/db/curop.h"
-#include "mongo/db/pipeline/document_source.h"
-#include "mongo/db/pipeline/lite_parsed_pipeline.h"
+#include "mongo/db/exec/document_value/document.h"
+#include "mongo/db/exec/document_value/value.h"
+#include "mongo/db/field_ref.h"
+#include "mongo/db/logical_time.h"
+#include "mongo/db/pipeline/expression_context.h"
+#include "mongo/db/pipeline/process_interface/mongo_process_interface.h"
 #include "mongo/db/pipeline/process_interface/shardsvr_process_interface.h"
-#include "mongo/db/pipeline/sharded_agg_helpers.h"
+#include "mongo/db/repl/read_concern_level.h"
 #include "mongo/db/s/balancer/balancer_policy.h"
-#include "mongo/db/s/sharding_state.h"
+#include "mongo/db/server_options.h"
+#include "mongo/db/storage/storage_options.h"
 #include "mongo/db/vector_clock.h"
-#include "mongo/logv2/log.h"
-#include "mongo/s/balancer_configuration.h"
+#include "mongo/executor/task_executor_pool.h"
 #include "mongo/s/catalog/type_shard.h"
+#include "mongo/s/client/shard.h"
+#include "mongo/s/client/shard_registry.h"
 #include "mongo/s/grid.h"
+#include "mongo/s/sharding_feature_flags_gen.h"
 #include "mongo/stdx/unordered_map.h"
+#include "mongo/stdx/unordered_set.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/intrusive_counter.h"
+#include "mongo/util/str.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
@@ -52,7 +87,6 @@ namespace {
 
 using ChunkDistributionMap = stdx::unordered_map<ShardId, size_t>;
 using ZoneShardMap = StringMap<std::vector<ShardId>>;
-using boost::intrusive_ptr;
 
 std::vector<ShardId> getAllNonDrainingShardIdsShuffled(OperationContext* opCtx) {
     const auto shardsAndOpTime = uassertStatusOKWithContext(
@@ -122,25 +156,113 @@ ShardId selectBestShard(const ChunkDistributionMap& chunkMap,
 
     for (const auto& shard : shards) {
         auto candidateIter = chunkMap.find(shard);
+        // If limitedShardIds is provided, only pick shard in that set.
         if (bestShardIter == chunkMap.end() || candidateIter->second < bestShardIter->second) {
             bestShardIter = candidateIter;
         }
     }
 
-    invariant(bestShardIter != chunkMap.end());
+    uassert(ErrorCodes::InvalidOptions,
+            str::stream() << "No shards found for chunk: " << chunkRange.toString()
+                          << " in zone: " << zone,
+            bestShardIter != chunkMap.end());
+
     return bestShardIter->first;
 }
 
 /*
  * Returns a map mapping each tag name to a vector of shard ids with that tag name
  */
-StringMap<std::vector<ShardId>> buildTagsToShardIdsMap(OperationContext* opCtx,
-                                                       const std::vector<TagsType>& tags) {
+StringMap<std::vector<ShardId>> buildTagsToShardIdsMap(
+    OperationContext* opCtx,
+    const std::vector<TagsType>& tags,
+    const boost::optional<std::vector<ShardId>>& availableShardIds = boost::none) {
     StringMap<std::vector<ShardId>> tagToShardIds;
     if (tags.empty()) {
         return tagToShardIds;
     }
 
+    // Get all docs in config.shards through a query instead of going through the shard registry
+    // because we need the zones as well. Filter out the shards not contained at availableShardIds,
+    // if set.
+    auto filter = [&] {
+        if (!availableShardIds) {
+            return BSONObj();
+        }
+
+        BSONArrayBuilder availableShardsBuilder;
+        for (auto&& shard : *availableShardIds) {
+            availableShardsBuilder.append(shard);
+        }
+        return BSON(ShardType::name() << BSON("$in" << availableShardsBuilder.arr()));
+    }();
+    const auto configServer = Grid::get(opCtx)->shardRegistry()->getConfigShard();
+    const auto shardDocs = uassertStatusOK(
+        configServer->exhaustiveFindOnConfig(opCtx,
+                                             ReadPreferenceSetting(ReadPreference::Nearest),
+                                             repl::ReadConcernLevel::kMajorityReadConcern,
+                                             NamespaceString::kConfigsvrShardsNamespace,
+                                             filter,
+                                             BSONObj(),
+                                             boost::none));
+    uassert(50986, str::stream() << "Could not find any shard documents", !shardDocs.docs.empty());
+
+    for (const auto& tag : tags) {
+        tagToShardIds[tag.getTag()] = {};
+    }
+
+    for (const auto& shardDoc : shardDocs.docs) {
+        auto parsedShard = uassertStatusOK(ShardType::fromBSON(shardDoc));
+
+        for (const auto& tag : parsedShard.getTags()) {
+            tagToShardIds[tag].push_back(parsedShard.getName());
+        }
+    }
+
+    for (const auto& tag : tags) {
+        uassert(ErrorCodes::CannotCreateChunkDistribution,
+                "The initial split policy is not able to find a chunk distribution that satisfies "
+                "the given set of zones with the current shards",
+                !tagToShardIds[tag.getTag()].empty());
+    }
+
+    return tagToShardIds;
+}
+
+/**
+ * Returns a set of split points to ensure that chunk boundaries will align with the zone
+ * ranges.
+ */
+BSONObjSet extractSplitPointsFromZones(const ShardKeyPattern& shardKey,
+                                       const boost::optional<std::vector<TagsType>>& zones) {
+    auto splitPoints = SimpleBSONObjComparator::kInstance.makeBSONObjSet();
+
+    if (!zones) {
+        return splitPoints;
+    }
+
+    for (const auto& zone : *zones) {
+        splitPoints.insert(zone.getMinKey());
+        splitPoints.insert(zone.getMaxKey());
+    }
+
+    const auto keyPattern = shardKey.getKeyPattern();
+    splitPoints.erase(keyPattern.globalMin());
+    splitPoints.erase(keyPattern.globalMax());
+
+    return splitPoints;
+}
+
+/*
+ * Returns a map mapping shard id to a set of zone tags.
+ */
+stdx::unordered_map<ShardId, stdx::unordered_set<std::string>> buildShardIdToTagsMap(
+    OperationContext* opCtx, const std::vector<ShardKeyRange>& shards) {
+    stdx::unordered_map<ShardId, stdx::unordered_set<std::string>> shardIdToTags;
+    if (shards.empty()) {
+        return shardIdToTags;
+    }
+
     // Get all docs in config.shards through a query instead of going through the shard registry
     // because we need the zones as well
     const auto configServer = Grid::get(opCtx)->shardRegistry()->getConfigShard();
@@ -152,26 +274,26 @@ StringMap<std::vector<ShardId>> buildTagsToShardIdsMap(OperationContext* opCtx,
                                              BSONObj(),
                                              BSONObj(),
                                              boost::none));
-    uassert(50986, str::stream() << "Could not find any shard documents", !shardDocs.docs.empty());
+    uassert(
+        7661502, str::stream() << "Could not find any shard documents", !shardDocs.docs.empty());
 
-    for (const auto& tag : tags) {
-        tagToShardIds[tag.getTag()] = {};
+    for (const auto& shard : shards) {
+        shardIdToTags[shard.getShard()] = {};
     }
 
     for (const auto& shardDoc : shardDocs.docs) {
         auto parsedShard = uassertStatusOK(ShardType::fromBSON(shardDoc));
         for (const auto& tag : parsedShard.getTags()) {
-            tagToShardIds[tag].push_back(parsedShard.getName());
+            shardIdToTags[ShardId(parsedShard.getName())].insert(tag);
         }
     }
 
-    return tagToShardIds;
+    return shardIdToTags;
 }
-
 }  // namespace
 
 std::vector<BSONObj> InitialSplitPolicy::calculateHashedSplitPoints(
-    const ShardKeyPattern& shardKeyPattern, BSONObj prefix, size_t numInitialChunks) {
+    const ShardKeyPattern& shardKeyPattern, BSONObj prefix, int numInitialChunks) {
     invariant(shardKeyPattern.isHashedPattern());
     invariant(numInitialChunks > 0);
 
@@ -216,7 +338,7 @@ std::vector<BSONObj> InitialSplitPolicy::calculateHashedSplitPoints(
         current += intervalSize / 2;
     }
 
-    for (size_t i = 0; i < (numInitialChunks - 1) / 2; i++) {
+    for (int i = 0; i < (numInitialChunks - 1) / 2; i++) {
         splitPoints.push_back(buildSplitPoint(current));
         splitPoints.push_back(buildSplitPoint(-current));
         current += intervalSize;
@@ -231,10 +353,16 @@ InitialSplitPolicy::ShardCollectionConfig InitialSplitPolicy::generateShardColle
     const ShardKeyPattern& shardKeyPattern,
     const Timestamp& validAfter,
     const std::vector<BSONObj>& splitPoints,
-    const std::vector<ShardId>& allShardIds,
-    const int numContiguousChunksPerShard) {
+    const std::vector<ShardId>& allShardIds) {
     invariant(!allShardIds.empty());
 
+    size_t numInitialChunksPerShard = 1;
+    // TODO SERVER-81884: update once 8.0 becomes last LTS.
+    if (!feature_flags::gOneChunkPerShardEmptyCollectionWithHashedShardKey.isEnabled(
+            serverGlobalParams.featureCompatibility.acquireFCVSnapshot())) {
+        numInitialChunksPerShard = 2;
+    }
+
     std::vector<BSONObj> finalSplitPoints;
 
     // Make sure points are unique and ordered
@@ -257,7 +385,8 @@ InitialSplitPolicy::ShardCollectionConfig InitialSplitPolicy::generateShardColle
         const BSONObj min = (i == 0) ? keyPattern.globalMin() : finalSplitPoints[i - 1];
         const BSONObj max =
             (i < finalSplitPoints.size()) ? finalSplitPoints[i] : keyPattern.globalMax();
-        const ShardId shardId = allShardIds[(i / numContiguousChunksPerShard) % allShardIds.size()];
+        // TODO SERVER-81884: simplify once numInitialChunksPerShard is always 1
+        const ShardId shardId = allShardIds[(i / numInitialChunksPerShard) % allShardIds.size()];
 
         appendChunk(params, min, max, &version, shardId, &chunks);
     }
@@ -265,52 +394,6 @@ InitialSplitPolicy::ShardCollectionConfig InitialSplitPolicy::generateShardColle
     return {std::move(chunks)};
 }
 
-std::unique_ptr<InitialSplitPolicy> InitialSplitPolicy::calculateOptimizationStrategy(
-    OperationContext* opCtx,
-    const ShardKeyPattern& shardKeyPattern,
-    const std::int64_t numInitialChunks,
-    const bool presplitHashedZones,
-    const std::vector<TagsType>& tags,
-    size_t numShards,
-    bool collectionIsEmpty) {
-    uassert(ErrorCodes::InvalidOptions,
-            str::stream() << "numInitialChunks is only supported when the collection is empty "
-                             "and has a hashed field in the shard key pattern",
-            !numInitialChunks || (shardKeyPattern.isHashedPattern() && collectionIsEmpty));
-    uassert(ErrorCodes::InvalidOptions,
-            str::stream()
-                << "When the prefix of the hashed shard key is a range field, "
-                   "'numInitialChunks' can only be used when the 'presplitHashedZones' is true",
-            !numInitialChunks || shardKeyPattern.hasHashedPrefix() || presplitHashedZones);
-
-    // If 'presplitHashedZones' flag is set, we always use 'PresplitHashedZonesSplitPolicy', to make
-    // sure we throw the correct assertion if further validation fails.
-    if (presplitHashedZones) {
-        return std::make_unique<PresplitHashedZonesSplitPolicy>(
-            opCtx, shardKeyPattern, tags, numInitialChunks, collectionIsEmpty);
-    }
-
-    // The next preference is to use split points based strategy. This is only possible if the
-    // collection is empty with shard key having a hashed prefix.
-    if (tags.empty() && shardKeyPattern.hasHashedPrefix() && collectionIsEmpty) {
-        return std::make_unique<SplitPointsBasedSplitPolicy>(
-            shardKeyPattern, numShards, numInitialChunks);
-    }
-
-    if (!tags.empty()) {
-        if (collectionIsEmpty) {
-            return std::make_unique<SingleChunkPerTagSplitPolicy>(opCtx, tags);
-        }
-        return std::make_unique<SingleChunkOnPrimarySplitPolicy>();
-    }
-
-    if (collectionIsEmpty) {
-        return std::make_unique<SingleChunkOnPrimarySplitPolicy>();
-    }
-
-    return std::make_unique<SingleChunkOnPrimarySplitPolicy>();
-}
-
 InitialSplitPolicy::ShardCollectionConfig SingleChunkOnPrimarySplitPolicy::createFirstChunks(
     OperationContext* opCtx,
     const ShardKeyPattern& shardKeyPattern,
@@ -331,24 +414,65 @@ InitialSplitPolicy::ShardCollectionConfig SingleChunkOnPrimarySplitPolicy::creat
     return {std::move(chunks)};
 }
 
+SingleChunkOnShardSplitPolicy::SingleChunkOnShardSplitPolicy(OperationContext* opCtx,
+                                                             ShardId dataShard) {
+    uassertStatusOK(Grid::get(opCtx)->shardRegistry()->getShard(opCtx, dataShard));
+    _dataShard = dataShard;
+}
+
+InitialSplitPolicy::ShardCollectionConfig SingleChunkOnShardSplitPolicy::createFirstChunks(
+    OperationContext* opCtx,
+    const ShardKeyPattern& shardKeyPattern,
+    const SplitPolicyParams& params) {
+    const auto currentTime = VectorClock::get(opCtx)->getTime();
+    const auto validAfter = currentTime.clusterTime().asTimestamp();
+
+    ChunkVersion version({OID::gen(), validAfter}, {1, 0});
+    const auto& keyPattern = shardKeyPattern.getKeyPattern();
+    std::vector<ChunkType> chunks;
+    appendChunk(
+        params, keyPattern.globalMin(), keyPattern.globalMax(), &version, _dataShard, &chunks);
+
+    return {std::move(chunks)};
+}
+
+SplitPointsBasedSplitPolicy::SplitPointsBasedSplitPolicy(
+    const ShardKeyPattern& shardKeyPattern,
+    size_t numShards,
+    boost::optional<std::vector<ShardId>> availableShardIds)
+    : _availableShardIds(std::move(availableShardIds)) {
+
+    size_t numInitialChunksPerShard = 1;
+    // TODO SERVER-81884: update once 8.0 becomes last LTS.
+    if (!feature_flags::gOneChunkPerShardEmptyCollectionWithHashedShardKey.isEnabled(
+            serverGlobalParams.featureCompatibility.acquireFCVSnapshot())) {
+        numInitialChunksPerShard = 2;
+    }
+
+    _splitPoints = calculateHashedSplitPoints(
+        shardKeyPattern, BSONObj(), numShards * numInitialChunksPerShard);
+}
+
 InitialSplitPolicy::ShardCollectionConfig SplitPointsBasedSplitPolicy::createFirstChunks(
     OperationContext* opCtx,
     const ShardKeyPattern& shardKeyPattern,
     const SplitPolicyParams& params) {
-
-    // On which shards are the generated chunks allowed to be placed.
-    const auto shardIds = getAllNonDrainingShardIdsShuffled(opCtx);
-
     const auto currentTime = VectorClock::get(opCtx)->getTime();
     const auto validAfter = currentTime.clusterTime().asTimestamp();
     return generateShardCollectionInitialChunks(
-        params, shardKeyPattern, validAfter, _splitPoints, shardIds, _numContiguousChunksPerShard);
+        params,
+        shardKeyPattern,
+        validAfter,
+        _splitPoints,
+        _availableShardIds ? *_availableShardIds : getAllNonDrainingShardIdsShuffled(opCtx));
 }
 
-AbstractTagsBasedSplitPolicy::AbstractTagsBasedSplitPolicy(OperationContext* opCtx,
-                                                           std::vector<TagsType> tags)
-    : _tags(tags) {
-    _tagToShardIds = buildTagsToShardIdsMap(opCtx, tags);
+AbstractTagsBasedSplitPolicy::AbstractTagsBasedSplitPolicy(
+    OperationContext* opCtx,
+    std::vector<TagsType> tags,
+    boost::optional<std::vector<ShardId>> availableShardIds)
+    : _tags(std::move(tags)), _availableShardIds(std::move(availableShardIds)) {
+    _tagToShardIds = buildTagsToShardIdsMap(opCtx, _tags, _availableShardIds);
 }
 
 AbstractTagsBasedSplitPolicy::SplitInfo SingleChunkPerTagSplitPolicy::buildSplitInfoForTag(
@@ -368,7 +492,7 @@ InitialSplitPolicy::ShardCollectionConfig AbstractTagsBasedSplitPolicy::createFi
     const SplitPolicyParams& params) {
     invariant(!_tags.empty());
 
-    const auto shardIds = getAllNonDrainingShardIdsShuffled(opCtx);
+    const auto shardIds = _availableShardIds.value_or(getAllNonDrainingShardIdsShuffled(opCtx));
     const auto currentTime = VectorClock::get(opCtx)->getTime();
     const auto validAfter = currentTime.clusterTime().asTimestamp();
     const auto& keyPattern = shardKeyPattern.getKeyPattern();
@@ -391,11 +515,6 @@ InitialSplitPolicy::ShardCollectionConfig AbstractTagsBasedSplitPolicy::createFi
         // Create chunk for the actual tag - [tag.getMinKey, tag.getMaxKey)
         const auto it = tagToShards.find(tag.getTag());
         invariant(it != tagToShards.end());
-        uassert(50973,
-                str::stream()
-                    << "Cannot shard collection " << tag.getNS() << " due to zone " << tag.getTag()
-                    << " which is not assigned to a shard. Please assign this zone to a shard.",
-                !it->second.empty());
 
         // The buildSplitInfoForTag() should provide split points which are in sorted order. So we
         // don't need to sort them again while generating chunks.
@@ -440,20 +559,24 @@ InitialSplitPolicy::ShardCollectionConfig AbstractTagsBasedSplitPolicy::createFi
 
 AbstractTagsBasedSplitPolicy::SplitInfo PresplitHashedZonesSplitPolicy::buildSplitInfoForTag(
     TagsType tag, const ShardKeyPattern& shardKeyPattern) {
+
+    // This strategy presplits each tag such that at least 'minNumInitialChunksPerShard' chunks are
+    // placed on every shard to which the tag is assigned. We distribute the chunks in an
+    // best-effort attempt to ensure that an equal number of chunks are created on each shard
+    // regardless of how the zones are laid out.
+
+    size_t minNumInitialChunksPerShard = 1;
+    // TODO SERVER-81884: update once 8.0 becomes last LTS.
+    if (!feature_flags::gOneChunkPerShardEmptyCollectionWithHashedShardKey.isEnabled(
+            serverGlobalParams.featureCompatibility.acquireFCVSnapshot())) {
+        minNumInitialChunksPerShard = 2;
+    }
+
     // Returns the ceiling number for the decimal value of x/y.
     auto ceilOfXOverY = [](auto x, auto y) {
         return (x / y) + (x % y != 0);
     };
 
-    // This strategy presplits each tag such that at least 1 chunk is placed on every shard to which
-    // the tag is assigned. We distribute the chunks such that at least '_numInitialChunks' are
-    // created across the cluster, and we make a best-effort attempt to ensure that an equal number
-    // of chunks are created on each shard regardless of how the zones are laid out.
-
-    //  We take the ceiling when the number is not divisible so that the final number of chunks
-    //  we generate are at least '_numInitialChunks'.
-    auto numChunksPerShard = ceilOfXOverY(_numInitialChunks, _numTagsPerShard.size());
-
     const auto& tagsToShardsMap = getTagsToShardIds();
     invariant(tagsToShardsMap.find(tag.getTag()) != tagsToShardsMap.end());
     const auto& shardsForCurrentTag = tagsToShardsMap.find(tag.getTag())->second;
@@ -464,10 +587,12 @@ AbstractTagsBasedSplitPolicy::SplitInfo PresplitHashedZonesSplitPolicy::buildSpl
     chunkDistribution.reserve((shardsForCurrentTag.size()));
     auto numChunksForCurrentTag = 0;
     for (auto&& shard : shardsForCurrentTag) {
+        // TODO SERVER-81884: update once 8.0 becomes last LTS.
+        // numChunksForCurrentTagOnShard will always be 1 when minNumInitialChunksPerShard == 1
         auto numChunksForCurrentTagOnShard =
-            ceilOfXOverY(numChunksPerShard, _numTagsPerShard[shard.toString()]);
+            ceilOfXOverY(minNumInitialChunksPerShard, _numTagsPerShard[shard.toString()]);
         chunkDistribution.push_back({shard, numChunksForCurrentTagOnShard});
-        numChunksForCurrentTag += (numChunksForCurrentTagOnShard);
+        numChunksForCurrentTag += numChunksForCurrentTagOnShard;
     }
 
     // Extract the fields preceding the hashed field. We use this object as a base for building
@@ -489,9 +614,9 @@ PresplitHashedZonesSplitPolicy::PresplitHashedZonesSplitPolicy(
     OperationContext* opCtx,
     const ShardKeyPattern& shardKeyPattern,
     std::vector<TagsType> tags,
-    size_t numInitialChunks,
-    bool isCollectionEmpty)
-    : AbstractTagsBasedSplitPolicy(opCtx, tags) {
+    bool isCollectionEmpty,
+    boost::optional<std::vector<ShardId>> availableShardIds)
+    : AbstractTagsBasedSplitPolicy(opCtx, tags, std::move(availableShardIds)) {
     // Verify that tags have been set up correctly for this split policy.
     _validate(shardKeyPattern, isCollectionEmpty);
 
@@ -506,9 +631,6 @@ PresplitHashedZonesSplitPolicy::PresplitHashedZonesSplitPolicy(
     // If we are here, we have confirmed that at least one tag is already set up. A tag can only be
     // created if they are associated with a zone and the zone has to be assigned to a shard.
     invariant(!_numTagsPerShard.empty());
-
-    // If 'numInitialChunks' was not specified, use default value.
-    _numInitialChunks = numInitialChunks ? numInitialChunks : _numTagsPerShard.size() * 2;
 }
 
 /**
@@ -615,7 +737,7 @@ std::vector<BSONObj> SamplingBasedSplitPolicy::createRawPipeline(const ShardKeyP
     for (auto&& fieldRef : shardKeyFields) {
         // If the shard key includes a hashed field and current fieldRef is the hashed field.
         if (shardKey.isHashedPattern() &&
-            fieldRef->dottedField().compare(shardKey.getHashedField().fieldNameStringData()) == 0) {
+            fieldRef->dottedField() == shardKey.getHashedField().fieldNameStringData()) {
             arrayToObjectBuilder.emplace_back(
                 Doc{{"k", V{fieldRef->dottedField()}},
                     {"v", Doc{{"$toHashedIndexKey", V{"$" + fieldRef->dottedField()}}}}});
@@ -639,20 +761,30 @@ SamplingBasedSplitPolicy SamplingBasedSplitPolicy::make(
     const ShardKeyPattern& shardKey,
     int numInitialChunks,
     boost::optional<std::vector<TagsType>> zones,
+    boost::optional<std::vector<ShardId>> availableShardIds,
     int samplesPerChunk) {
     uassert(4952603, "samplesPerChunk should be > 0", samplesPerChunk > 0);
     return SamplingBasedSplitPolicy(
         numInitialChunks,
         zones,
-        _makePipelineDocumentSource(opCtx, nss, shardKey, numInitialChunks, samplesPerChunk));
+        _makePipelineDocumentSource(opCtx, nss, shardKey, numInitialChunks, samplesPerChunk),
+        availableShardIds);
 }
 
-SamplingBasedSplitPolicy::SamplingBasedSplitPolicy(int numInitialChunks,
-                                                   boost::optional<std::vector<TagsType>> zones,
-                                                   std::unique_ptr<SampleDocumentSource> samples)
-    : _numInitialChunks(numInitialChunks), _zones(std::move(zones)), _samples(std::move(samples)) {
+SamplingBasedSplitPolicy::SamplingBasedSplitPolicy(
+    int numInitialChunks,
+    boost::optional<std::vector<TagsType>> zones,
+    std::unique_ptr<SampleDocumentSource> samples,
+    boost::optional<std::vector<ShardId>> availableShardIds)
+    : _numInitialChunks(numInitialChunks),
+      _zones(std::move(zones)),
+      _samples(std::move(samples)),
+      _availableShardIds(std::move(availableShardIds)) {
     uassert(4952602, "numInitialChunks should be > 0", numInitialChunks > 0);
     uassert(4952604, "provided zones should not be empty", !_zones || _zones->size());
+    uassert(7679103,
+            "provided availableShardIds should not be empty",
+            !_availableShardIds || !_availableShardIds->empty());
 }
 
 BSONObjSet SamplingBasedSplitPolicy::createFirstSplitPoints(OperationContext* opCtx,
@@ -665,7 +797,7 @@ BSONObjSet SamplingBasedSplitPolicy::createFirstSplitPoints(OperationContext* op
         }
     }
 
-    auto splitPoints = _extractSplitPointsFromZones(shardKey);
+    auto splitPoints = extractSplitPointsFromZones(shardKey, _zones);
     if (splitPoints.size() < static_cast<size_t>(_numInitialChunks - 1)) {
         // The BlockingResultsMerger underlying the $mergeCursors stage records how long was
         // spent waiting for samples from the donor shards. It doing so requires the CurOp
@@ -703,8 +835,13 @@ InitialSplitPolicy::ShardCollectionConfig SamplingBasedSplitPolicy::createFirstC
         }
     }
 
-    {
-        auto shardIds = getAllNonDrainingShardIdsShuffled(opCtx);
+    if (_availableShardIds) {
+        for (const auto& shardId : *_availableShardIds) {
+            chunkDistribution.emplace(shardId, 0);
+        }
+        zoneToShardMap.emplace("", *_availableShardIds);
+    } else {
+        const auto shardIds = getAllNonDrainingShardIdsShuffled(opCtx);
         for (const auto& shard : shardIds) {
             chunkDistribution.emplace(shard, 0);
         }
@@ -737,25 +874,6 @@ InitialSplitPolicy::ShardCollectionConfig SamplingBasedSplitPolicy::createFirstC
     return {std::move(chunks)};
 }
 
-BSONObjSet SamplingBasedSplitPolicy::_extractSplitPointsFromZones(const ShardKeyPattern& shardKey) {
-    auto splitPoints = SimpleBSONObjComparator::kInstance.makeBSONObjSet();
-
-    if (!_zones) {
-        return splitPoints;
-    }
-
-    for (const auto& zone : *_zones) {
-        splitPoints.insert(zone.getMinKey());
-        splitPoints.insert(zone.getMaxKey());
-    }
-
-    const auto keyPattern = shardKey.getKeyPattern();
-    splitPoints.erase(keyPattern.globalMin());
-    splitPoints.erase(keyPattern.globalMax());
-
-    return splitPoints;
-}
-
 void SamplingBasedSplitPolicy::_appendSplitPointsFromSample(BSONObjSet* splitPoints,
                                                             const ShardKeyPattern& shardKey,
                                                             int nToAppend) {
@@ -792,8 +910,8 @@ SamplingBasedSplitPolicy::_makePipelineDocumentSource(OperationContext* opCtx,
                                                       int samplesPerChunk,
                                                       MakePipelineOptions opts) {
     auto rawPipeline = createRawPipeline(shardKey, numInitialChunks, samplesPerChunk);
-    ResolvedNamespaceMap resolvedNamespaces;
-    resolvedNamespaces[ns] = {ns, std::vector<BSONObj>{}};
+    StringMap<ExpressionContext::ResolvedNamespace> resolvedNamespaces;
+    resolvedNamespaces[ns.coll()] = {ns, std::vector<BSONObj>{}};
 
     auto pi = [&]() -> std::shared_ptr<MongoProcessInterface> {
         if (serverGlobalParams.clusterRole.has(ClusterRole::ConfigServer) ||
@@ -855,4 +973,126 @@ boost::optional<BSONObj> SamplingBasedSplitPolicy::PipelineDocumentSource::getNe
     return val->toBson();
 }
 
+ShardDistributionSplitPolicy ShardDistributionSplitPolicy::make(
+    OperationContext* opCtx,
+    const ShardKeyPattern& shardKey,
+    std::vector<ShardKeyRange> shardDistribution,
+    boost::optional<std::vector<TagsType>> zones) {
+    uassert(7661501, "ShardDistribution should not be empty", shardDistribution.size() > 0);
+    return ShardDistributionSplitPolicy(shardDistribution, zones);
+}
+
+ShardDistributionSplitPolicy::ShardDistributionSplitPolicy(
+    std::vector<ShardKeyRange>& shardDistribution, boost::optional<std::vector<TagsType>> zones)
+    : _shardDistribution(std::move(shardDistribution)), _zones(std::move(zones)) {}
+
+InitialSplitPolicy::ShardCollectionConfig ShardDistributionSplitPolicy::createFirstChunks(
+    OperationContext* opCtx,
+    const ShardKeyPattern& shardKeyPattern,
+    const SplitPolicyParams& params) {
+    const auto& keyPattern = shardKeyPattern.getKeyPattern();
+    if (_zones) {
+        for (auto& zone : *_zones) {
+            zone.setMinKey(keyPattern.extendRangeBound(zone.getMinKey(), false));
+            zone.setMaxKey(keyPattern.extendRangeBound(zone.getMaxKey(), false));
+        }
+    }
+
+    auto splitPoints = extractSplitPointsFromZones(shardKeyPattern, _zones);
+    std::vector<ChunkType> chunks;
+    uassert(7679102,
+            "ShardDistribution without min/max must not use this split policy.",
+            _shardDistribution[0].getMin());
+
+    unsigned long shardDistributionIdx = 0;
+    const auto currentTime = VectorClock::get(opCtx)->getTime();
+    const auto validAfter = currentTime.clusterTime().asTimestamp();
+    ChunkVersion version({OID::gen(), validAfter}, {1, 0});
+    for (const auto& splitPoint : splitPoints) {
+        _appendChunks(params, splitPoint, keyPattern, shardDistributionIdx, version, chunks);
+    }
+    _appendChunks(
+        params, keyPattern.globalMax(), keyPattern, shardDistributionIdx, version, chunks);
+
+    if (_zones) {
+        _checkShardsMatchZones(opCtx, chunks, *_zones);
+    }
+
+    return {std::move(chunks)};
+}
+
+void ShardDistributionSplitPolicy::_appendChunks(const SplitPolicyParams& params,
+                                                 const BSONObj& splitPoint,
+                                                 const KeyPattern& keyPattern,
+                                                 unsigned long& shardDistributionIdx,
+                                                 ChunkVersion& version,
+                                                 std::vector<ChunkType>& chunks) {
+    while (shardDistributionIdx < _shardDistribution.size()) {
+        auto shardMin =
+            keyPattern.extendRangeBound(*_shardDistribution[shardDistributionIdx].getMin(), false);
+        auto shardMax =
+            keyPattern.extendRangeBound(*_shardDistribution[shardDistributionIdx].getMax(), false);
+        auto lastChunkMax =
+            chunks.empty() ? keyPattern.globalMin() : chunks.back().getRange().getMax();
+        /* When we compare a defined shard range with a splitPoint, there are three cases:
+         * 1. The whole shard range is on the left side of the splitPoint -> Add this shard as a
+         * whole chunk and move to next shard.
+         * 2. The splitPoint is in the middle of the shard range. -> Append (shardMin,
+         * splitPoint) as a chunk and move to next split point.
+         * 3. The whole shard range is on the right side of the splitPoint -> Move to the next
+         * splitPoint.
+         * This algorithm relies on the shardDistribution is continuous and complete to be
+         * correct, which is validated in the cmd handler.
+         */
+        if (SimpleBSONObjComparator::kInstance.evaluate(shardMin < splitPoint)) {
+            // The whole shard range is on the left side of the splitPoint.
+            if (SimpleBSONObjComparator::kInstance.evaluate(shardMax <= splitPoint)) {
+                appendChunk(params,
+                            lastChunkMax,
+                            shardMax,
+                            &version,
+                            _shardDistribution[shardDistributionIdx].getShard(),
+                            &chunks);
+                lastChunkMax = shardMax;
+                shardDistributionIdx++;
+            } else {  // The splitPoint is in the middle of the shard range.
+                appendChunk(params,
+                            lastChunkMax,
+                            splitPoint,
+                            &version,
+                            _shardDistribution[shardDistributionIdx].getShard(),
+                            &chunks);
+                lastChunkMax = splitPoint;
+                return;
+            }
+        } else {  // The whole shard range is on the right side of the splitPoint.
+            return;
+        }
+    }
+}
+
+void ShardDistributionSplitPolicy::_checkShardsMatchZones(
+    OperationContext* opCtx,
+    const std::vector<ChunkType>& chunks,
+    const std::vector<mongo::TagsType>& zones) {
+    ZoneInfo zoneInfo;
+    auto shardIdToTags = buildShardIdToTagsMap(opCtx, _shardDistribution);
+    for (const auto& zone : zones) {
+        uassertStatusOK(
+            zoneInfo.addRangeToZone({zone.getMinKey(), zone.getMaxKey(), zone.getTag()}));
+    }
+
+    for (const auto& chunk : chunks) {
+        auto zoneFromCmdParameter = zoneInfo.getZoneForRange({chunk.getMin(), chunk.getMax()});
+        auto iter = shardIdToTags.find(chunk.getShard());
+        uassert(ErrorCodes::InvalidOptions,
+                str::stream() << "Specified zones and shardDistribution are conflicting with the "
+                                 "existing shard/zone, shard "
+                              << chunk.getShard() << "doesn't belong to zone "
+                              << zoneFromCmdParameter,
+                iter != shardIdToTags.end() &&
+                    iter->second.find(zoneFromCmdParameter) != iter->second.end());
+    }
+}
+
 }  // namespace mongo
