@@ -29,17 +29,32 @@
 
 #pragma once
 
+#include <algorithm>
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <cstddef>
+#include <cstdint>
+#include <memory>
+#include <utility>
 #include <vector>
 
 #include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobj_comparator_interface.h"
+#include "mongo/bson/timestamp.h"
 #include "mongo/db/catalog/collection_options.h"
+#include "mongo/db/keypattern.h"
 #include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/pipeline/pipeline.h"
 #include "mongo/db/shard_id.h"
 #include "mongo/s/catalog/type_chunk.h"
 #include "mongo/s/catalog/type_tags.h"
+#include "mongo/s/chunk_version.h"
+#include "mongo/s/resharding/common_types_gen.h"
 #include "mongo/s/shard_key_pattern.h"
 #include "mongo/util/string_map.h"
+#include "mongo/util/uuid.h"
+
 namespace mongo {
 
 struct SplitPolicyParams {
@@ -49,19 +64,6 @@ struct SplitPolicyParams {
 
 class InitialSplitPolicy {
 public:
-    /**
-     * Returns the optimization strategy for building initial chunks based on the input parameters
-     * and the collection state.
-     */
-    static std::unique_ptr<InitialSplitPolicy> calculateOptimizationStrategy(
-        OperationContext* opCtx,
-        const ShardKeyPattern& shardKeyPattern,
-        std::int64_t numInitialChunks,
-        bool presplitHashedZones,
-        const std::vector<TagsType>& tags,
-        size_t numShards,
-        bool collectionIsEmpty);
-
     virtual ~InitialSplitPolicy() {}
 
     /**
@@ -78,14 +80,6 @@ public:
                                                     const ShardKeyPattern& shardKeyPattern,
                                                     const SplitPolicyParams& params) = 0;
 
-    /**
-     * Returns whether the chunk generation strategy being used is optimized or not. Since there is
-     * only a single unoptimized policy, we return true by default here.
-     */
-    virtual bool isOptimized() {
-        return true;
-    }
-
     /**
      * Returns split points to use for creating chunks in cases where the shard key contains a
      * hashed field. For new collections which use hashed shard keys, we can can pre-split the range
@@ -97,7 +91,7 @@ public:
      */
     static std::vector<BSONObj> calculateHashedSplitPoints(const ShardKeyPattern& shardKeyPattern,
                                                            BSONObj prefix,
-                                                           size_t numInitialChunks);
+                                                           int numInitialChunks);
 
     /**
      * Produces the initial chunks that need to be written for an *empty* collection which is being
@@ -120,8 +114,7 @@ public:
         const ShardKeyPattern& shardKeyPattern,
         const Timestamp& validAfter,
         const std::vector<BSONObj>& splitPoints,
-        const std::vector<ShardId>& allShardIds,
-        int numContiguousChunksPerShard);
+        const std::vector<ShardId>& allShardIds);
 };
 
 /**
@@ -134,6 +127,21 @@ public:
                                             const SplitPolicyParams& params) override;
 };
 
+/**
+ * Create a single chunk on a specified shard.
+ */
+class SingleChunkOnShardSplitPolicy : public InitialSplitPolicy {
+public:
+    SingleChunkOnShardSplitPolicy(OperationContext* opCtx, ShardId dataShard);
+
+    ShardCollectionConfig createFirstChunks(OperationContext* opCtx,
+                                            const ShardKeyPattern& shardKeyPattern,
+                                            const SplitPolicyParams& params) override;
+
+private:
+    ShardId _dataShard;
+};
+
 /**
  * Split point building strategy to be used when the appropriate splitpoints can be trivially
  * deduced from the shard key.
@@ -143,15 +151,10 @@ public:
     /**
      * Constructor used when generating split points for a hashed-prefix shard key.
      */
-    SplitPointsBasedSplitPolicy(const ShardKeyPattern& shardKeyPattern,
-                                size_t numShards,
-                                size_t numInitialChunks) {
-        // If 'numInitialChunks' was not specified, use default value.
-        numInitialChunks = numInitialChunks ? numInitialChunks : numShards * 2;
-        _splitPoints = calculateHashedSplitPoints(shardKeyPattern, BSONObj(), numInitialChunks);
-        _numContiguousChunksPerShard =
-            std::max(numInitialChunks / numShards, static_cast<size_t>(1));
-    }
+    SplitPointsBasedSplitPolicy(
+        const ShardKeyPattern& shardKeyPattern,
+        size_t numShards,
+        boost::optional<std::vector<ShardId>> availableShardIds = boost::none);
 
     ShardCollectionConfig createFirstChunks(OperationContext* opCtx,
                                             const ShardKeyPattern& shardKeyPattern,
@@ -161,13 +164,10 @@ public:
     const std::vector<BSONObj>& getSplitPoints() const {
         return _splitPoints;
     }
-    size_t getNumContiguousChunksPerShard() const {
-        return _numContiguousChunksPerShard;
-    }
 
 private:
     std::vector<BSONObj> _splitPoints;
-    size_t _numContiguousChunksPerShard;
+    boost::optional<std::vector<ShardId>> _availableShardIds;
 };
 
 /**
@@ -184,7 +184,9 @@ public:
         std::vector<std::pair<ShardId, size_t>> chunkDistribution;
     };
 
-    AbstractTagsBasedSplitPolicy(OperationContext* opCtx, std::vector<TagsType> tags);
+    AbstractTagsBasedSplitPolicy(OperationContext* opCtx,
+                                 std::vector<TagsType> tags,
+                                 boost::optional<std::vector<ShardId>> availableShardIds);
 
     ShardCollectionConfig createFirstChunks(OperationContext* opCtx,
                                             const ShardKeyPattern& shardKeyPattern,
@@ -207,6 +209,7 @@ public:
 private:
     const std::vector<TagsType> _tags;
     StringMap<std::vector<ShardId>> _tagToShardIds;
+    boost::optional<std::vector<ShardId>> _availableShardIds;
 };
 
 /**
@@ -214,8 +217,11 @@ private:
  */
 class SingleChunkPerTagSplitPolicy : public AbstractTagsBasedSplitPolicy {
 public:
-    SingleChunkPerTagSplitPolicy(OperationContext* opCtx, std::vector<TagsType> tags)
-        : AbstractTagsBasedSplitPolicy(opCtx, tags) {}
+    SingleChunkPerTagSplitPolicy(
+        OperationContext* opCtx,
+        std::vector<TagsType> tags,
+        boost::optional<std::vector<ShardId>> availableShardIds = boost::none)
+        : AbstractTagsBasedSplitPolicy(opCtx, tags, availableShardIds) {}
 
     SplitInfo buildSplitInfoForTag(TagsType tag, const ShardKeyPattern& shardKeyPattern) override;
 
@@ -231,11 +237,12 @@ private:
  */
 class PresplitHashedZonesSplitPolicy : public AbstractTagsBasedSplitPolicy {
 public:
-    PresplitHashedZonesSplitPolicy(OperationContext* opCtx,
-                                   const ShardKeyPattern& shardKeyPattern,
-                                   std::vector<TagsType> tags,
-                                   size_t numInitialChunks,
-                                   bool isCollectionEmpty);
+    PresplitHashedZonesSplitPolicy(
+        OperationContext* opCtx,
+        const ShardKeyPattern& shardKeyPattern,
+        std::vector<TagsType> tags,
+        bool isCollectionEmpty,
+        boost::optional<std::vector<ShardId>> availableShardIds = boost::none);
 
     SplitInfo buildSplitInfoForTag(TagsType tag, const ShardKeyPattern& shardKeyPattern) override;
 
@@ -244,7 +251,6 @@ private:
      * Validates that each of tags are set up correctly so that the tags can be split further.
      */
     void _validate(const ShardKeyPattern& shardKeyPattern, bool isCollectionEmpty);
-    size_t _numInitialChunks;
     StringMap<size_t> _numTagsPerShard;
 };
 
@@ -287,11 +293,13 @@ public:
                                          const ShardKeyPattern& shardKey,
                                          int numInitialChunks,
                                          boost::optional<std::vector<TagsType>> zones,
+                                         boost::optional<std::vector<ShardId>> availableShardIds,
                                          int samplesPerChunk = kDefaultSamplesPerChunk);
 
     SamplingBasedSplitPolicy(int numInitialChunks,
                              boost::optional<std::vector<TagsType>> zones,
-                             std::unique_ptr<SampleDocumentSource> samples);
+                             std::unique_ptr<SampleDocumentSource> samples,
+                             boost::optional<std::vector<ShardId>> availableShardIds);
 
     /**
      * Generates the initial split points and returns them in ascending shard key order. Does not
@@ -330,12 +338,6 @@ private:
         int samplesPerChunk,
         MakePipelineOptions opts = {});
 
-    /**
-     * Returns a set of split points to ensure that chunk boundaries will align with the zone
-     * ranges.
-     */
-    BSONObjSet _extractSplitPointsFromZones(const ShardKeyPattern& shardKey);
-
     /**
      * Append split points based from the samples taken from the collection.
      */
@@ -343,9 +345,47 @@ private:
                                       const ShardKeyPattern& shardKey,
                                       int nToAppend);
 
-    const NamespaceString _ns;
     const int _numInitialChunks;
     boost::optional<std::vector<TagsType>> _zones;
     std::unique_ptr<SampleDocumentSource> _samples;
+    // If provided, only pick shard that is in this vector.
+    boost::optional<std::vector<ShardId>> _availableShardIds;
 };
+
+class ShardDistributionSplitPolicy : public InitialSplitPolicy {
+public:
+    static ShardDistributionSplitPolicy make(OperationContext* opCtx,
+                                             const ShardKeyPattern& shardKey,
+                                             std::vector<ShardKeyRange> shardDistribution,
+                                             boost::optional<std::vector<TagsType>> zones);
+
+    ShardDistributionSplitPolicy(std::vector<ShardKeyRange>& shardDistribution,
+                                 boost::optional<std::vector<TagsType>> zones);
+
+    ShardCollectionConfig createFirstChunks(OperationContext* opCtx,
+                                            const ShardKeyPattern& shardKeyPattern,
+                                            const SplitPolicyParams& params) override;
+
+private:
+    /**
+     * Given a splitPoint, create chunks from _shardDistribution until passing the splitPoint.
+     */
+    void _appendChunks(const SplitPolicyParams& params,
+                       const BSONObj& splitPoint,
+                       const KeyPattern& keyPattern,
+                       unsigned long& shardDistributionIdx,
+                       ChunkVersion& version,
+                       std::vector<ChunkType>& chunks);
+
+    /**
+     * Check the chunks created from command parameter "zones" and "shardDistribution" are
+     * satisfying the existing zone mapping rules in config.
+     */
+    void _checkShardsMatchZones(OperationContext* opCtx,
+                                const std::vector<ChunkType>& chunks,
+                                const std::vector<mongo::TagsType>& zones);
+    std::vector<ShardKeyRange> _shardDistribution;
+    boost::optional<std::vector<TagsType>> _zones;
+};
+
 }  // namespace mongo
