@@ -29,51 +29,108 @@
 
 #include "mongo/db/s/config/sharding_catalog_manager.h"
 
+#include <absl/container/node_hash_map.h>
 #include <algorithm>
+#include <boost/cstdint.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <cstddef>
+#include <iterator>
+#include <mutex>
 #include <tuple>
 #include <vector>
 
-#include "mongo/db/auth/authorization_session_impl.h"
+#include "mongo/base/error_codes.h"
+#include "mongo/bson/bson_field.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/util/builder.h"
+#include "mongo/db/auth/authorization_session.h"
+#include "mongo/db/cancelable_operation_context.h"
 #include "mongo/db/catalog/coll_mod.h"
+#include "mongo/db/catalog/collection_options_gen.h"
+#include "mongo/db/client.h"
+#include "mongo/db/cluster_role.h"
 #include "mongo/db/coll_mod_gen.h"
 #include "mongo/db/dbdirectclient.h"
+#include "mongo/db/dbmessage.h"
 #include "mongo/db/error_labels.h"
+#include "mongo/db/exec/document_value/document.h"
+#include "mongo/db/feature_flag.h"
+#include "mongo/db/logical_time.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/ops/write_ops.h"
+#include "mongo/db/ops/write_ops_gen.h"
 #include "mongo/db/ops/write_ops_parsers.h"
+#include "mongo/db/pipeline/aggregate_command_gen.h"
+#include "mongo/db/pipeline/document_source.h"
 #include "mongo/db/pipeline/document_source_group.h"
 #include "mongo/db/pipeline/document_source_lookup.h"
+#include "mongo/db/pipeline/document_source_match.h"
 #include "mongo/db/pipeline/document_source_merge.h"
 #include "mongo/db/pipeline/document_source_project.h"
 #include "mongo/db/pipeline/document_source_union_with.h"
+#include "mongo/db/pipeline/expression_context.h"
 #include "mongo/db/pipeline/pipeline.h"
 #include "mongo/db/query/cursor_response.h"
-#include "mongo/db/query/query_request_helper.h"
+#include "mongo/db/query/find_command.h"
 #include "mongo/db/read_write_concern_defaults.h"
+#include "mongo/db/repl/optime_with.h"
+#include "mongo/db/repl/read_concern_args.h"
 #include "mongo/db/repl/repl_client_info.h"
+#include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/repl/tenant_migration_access_blocker_util.h"
+#include "mongo/db/resource_yielder.h"
 #include "mongo/db/s/config/index_on_config.h"
 #include "mongo/db/s/config/placement_history_cleaner.h"
 #include "mongo/db/s/sharding_util.h"
+#include "mongo/db/server_options.h"
+#include "mongo/db/session/logical_session_cache.h"
+#include "mongo/db/session/logical_session_id_gen.h"
 #include "mongo/db/vector_clock.h"
+#include "mongo/executor/connection_pool_stats.h"
+#include "mongo/executor/inline_executor.h"
+#include "mongo/executor/task_executor_pool.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/log_severity.h"
+#include "mongo/logv2/redaction.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/rpc/get_status_from_command_result.h"
 #include "mongo/rpc/metadata/impersonated_user_metadata.h"
+#include "mongo/rpc/op_msg.h"
+#include "mongo/s/async_requests_sender.h"
 #include "mongo/s/balancer_configuration.h"
 #include "mongo/s/catalog/sharding_catalog_client.h"
 #include "mongo/s/catalog/type_chunk.h"
 #include "mongo/s/catalog/type_collection.h"
+#include "mongo/s/catalog/type_collection_gen.h"
 #include "mongo/s/catalog/type_config_version.h"
 #include "mongo/s/catalog/type_namespace_placement_gen.h"
 #include "mongo/s/catalog/type_shard.h"
 #include "mongo/s/catalog/type_tags.h"
 #include "mongo/s/client/shard_registry.h"
-#include "mongo/s/database_version.h"
 #include "mongo/s/grid.h"
 #include "mongo/s/sharding_feature_flags_gen.h"
 #include "mongo/s/write_ops/batched_command_request.h"
 #include "mongo/s/write_ops/batched_command_response.h"
 #include "mongo/transport/service_entry_point.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/duration.h"
+#include "mongo/util/fail_point.h"
+#include "mongo/util/future.h"
+#include "mongo/util/future_impl.h"
+#include "mongo/util/intrusive_counter.h"
 #include "mongo/util/log_and_backoff.h"
+#include "mongo/util/namespace_string_util.h"
+#include "mongo/util/out_of_line_executor.h"
+#include "mongo/util/scopeguard.h"
+#include "mongo/util/string_map.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
@@ -91,7 +148,7 @@ const auto getShardingCatalogManager =
     ServiceContext::declareDecoration<boost::optional<ShardingCatalogManager>>();
 
 OpMsg runCommandInLocalTxn(OperationContext* opCtx,
-                           StringData db,
+                           const DatabaseName& db,
                            bool startTransaction,
                            TxnNumber txnNumber,
                            BSONObj cmdObj) {
@@ -100,17 +157,19 @@ OpMsg runCommandInLocalTxn(OperationContext* opCtx,
         bob.append("startTransaction", true);
     }
     bob.append("autocommit", false);
-    bob.append(OperationSessionInfo::kTxnNumberFieldName, txnNumber);
+    bob.append(OperationSessionInfoFromClient::kTxnNumberFieldName, txnNumber);
 
     BSONObjBuilder lsidBuilder(bob.subobjStart("lsid"));
     opCtx->getLogicalSessionId()->serialize(&bob);
     lsidBuilder.doneFast();
 
     return OpMsg::parseOwned(
-        opCtx->getServiceContext()
+        opCtx->getService()
             ->getServiceEntryPoint()
-            ->handleRequest(opCtx,
-                            OpMsgRequest::fromDBAndBody(db.toString(), bob.obj()).serialize())
+            ->handleRequest(
+                opCtx,
+                OpMsgRequestBuilder::create(auth::ValidatedTenancyScope::get(opCtx), db, bob.obj())
+                    .serialize())
             .get()
             .response);
 }
@@ -136,10 +195,12 @@ void startTransactionWithNoopFind(OperationContext* opCtx,
     findCommand.setBatchSize(0);
     findCommand.setSingleBatch(true);
 
-    auto res =
-        runCommandInLocalTxn(
-            opCtx, nss.db(), true /*startTransaction*/, txnNumber, findCommand.toBSON(BSONObj()))
-            .body;
+    auto res = runCommandInLocalTxn(opCtx,
+                                    nss.dbName(),
+                                    true /*startTransaction*/,
+                                    txnNumber,
+                                    findCommand.toBSON(BSONObj()))
+                   .body;
     uassertStatusOK(getStatusFromCommandResult(res));
 }
 
@@ -150,11 +211,9 @@ BSONObj commitOrAbortTransaction(OperationContext* opCtx,
     // Swap out the clients in order to get a fresh opCtx. Previous operations in this transaction
     // that have been run on this opCtx would have set the timeout in the locker on the opCtx, but
     // commit should not have a lock timeout.
-    auto newClient = getGlobalServiceContext()->makeClient("ShardingCatalogManager");
-    {
-        stdx::lock_guard<Client> lk(*newClient);
-        newClient->setSystemOperationKillableByStepdown(lk);
-    }
+    auto newClient = getGlobalServiceContext()
+                         ->getService(ClusterRole::ShardServer)
+                         ->makeClient("ShardingCatalogManager");
     AlternativeClientRegion acr(newClient);
     auto newOpCtx = cc().makeOperationContext();
     newOpCtx->setAlwaysInterruptAtStepDownOrUp_UNSAFE();
@@ -169,7 +228,7 @@ BSONObj commitOrAbortTransaction(OperationContext* opCtx,
     BSONObjBuilder bob;
     bob.append(cmdName, true);
     bob.append("autocommit", false);
-    bob.append(OperationSessionInfo::kTxnNumberFieldName, txnNumber);
+    bob.append(OperationSessionInfoFromClient::kTxnNumberFieldName, txnNumber);
     bob.append(WriteConcernOptions::kWriteConcernField, writeConcern.toBSON());
 
     BSONObjBuilder lsidBuilder(bob.subobjStart("lsid"));
@@ -179,11 +238,13 @@ BSONObj commitOrAbortTransaction(OperationContext* opCtx,
     const auto cmdObj = bob.obj();
 
     const auto replyOpMsg = OpMsg::parseOwned(
-        newOpCtx->getServiceContext()
+        newOpCtx->getService()
             ->getServiceEntryPoint()
-            ->handleRequest(
-                newOpCtx.get(),
-                OpMsgRequest::fromDBAndBody(DatabaseName::kAdmin.toString(), cmdObj).serialize())
+            ->handleRequest(newOpCtx.get(),
+                            OpMsgRequestBuilder::create(auth::ValidatedTenancyScope::kNotRequired,
+                                                        DatabaseName::kAdmin,
+                                                        cmdObj)
+                                .serialize())
             .get()
             .response);
     return replyOpMsg.body;
@@ -300,10 +361,10 @@ public:
                     std::vector<NamespaceString>&& resolvedNamespaces)
         : _expCtx{make_intrusive<ExpressionContext>(opCtx, nullptr /*collator*/, nss)} {
 
-        ResolvedNamespaceMap resolvedNamespacesMap;
+        StringMap<ExpressionContext::ResolvedNamespace> resolvedNamespacesMap;
 
         for (const auto& collNs : resolvedNamespaces) {
-            resolvedNamespacesMap[collNs] = {collNs, std::vector<BSONObj>() /* pipeline */};
+            resolvedNamespacesMap[collNs.coll()] = {collNs, std::vector<BSONObj>() /* pipeline */};
         }
 
         _expCtx->setResolvedNamespaces(resolvedNamespacesMap);
@@ -537,8 +598,9 @@ void setInitializationTimeOnPlacementHistory(
             NamespaceString::kConfigsvrPlacementHistoryNamespace);
         write_ops::DeleteOpEntry entryDelMarker;
         entryDelMarker.setQ(
-            BSON(NamespacePlacementType::kNssFieldName
-                 << ShardingCatalogClient::kConfigPlacementHistoryInitializationMarker.ns()));
+            BSON(NamespacePlacementType::kNssFieldName << NamespaceStringUtil::serialize(
+                     ShardingCatalogClient::kConfigPlacementHistoryInitializationMarker,
+                     SerializationContext::stateDefault())));
         entryDelMarker.setMulti(true);
         deleteRequest.setDeletes({entryDelMarker});
 
@@ -619,7 +681,9 @@ ShardingCatalogManager::ShardingCatalogManager(
       _executorForAddShard(std::move(addShardExecutor)),
       _localConfigShard(std::move(localConfigShard)),
       _localCatalogClient(std::move(localCatalogClient)),
+      _kAddRemoveShardLock("addRemoveShardLock"),
       _kShardMembershipLock("shardMembershipLock"),
+      _kClusterCardinalityParameterLock("clusterCardinalityParameterLock"),
       _kChunkOpLock("chunkOpLock"),
       _kZoneOpLock("zoneOpLock"),
       _kPlacementHistoryInitializationLock("placementHistoryInitializationOpLock") {
@@ -669,12 +733,9 @@ Status ShardingCatalogManager::initializeConfigDatabaseIfNeeded(OperationContext
         return status;
     }
 
-    if (feature_flags::gConfigSettingsSchema.isEnabled(
-            serverGlobalParams.featureCompatibility.acquireFCVSnapshot())) {
-        status = _initConfigSettings(opCtx);
-        if (!status.isOK()) {
-            return status;
-        }
+    status = _initConfigSettings(opCtx);
+    if (!status.isOK()) {
+        return status;
     }
 
     // Make sure to write config.version last since we detect rollbacks of config.version and
@@ -691,10 +752,6 @@ Status ShardingCatalogManager::initializeConfigDatabaseIfNeeded(OperationContext
     return Status::OK();
 }
 
-Status ShardingCatalogManager::upgradeConfigSettings(OperationContext* opCtx) {
-    return _initConfigSettings(opCtx);
-}
-
 ShardingCatalogClient* ShardingCatalogManager::localCatalogClient() {
     invariant(serverGlobalParams.clusterRole.has(ClusterRole::ConfigServer));
     return _localCatalogClient.get();
@@ -720,11 +777,6 @@ Status ShardingCatalogManager::_initConfigVersion(OperationContext* opCtx) {
     VersionType newVersion;
     newVersion.setClusterId(OID::gen());
 
-    if (!feature_flags::gStopUsingConfigVersion.isEnabled(
-            serverGlobalParams.featureCompatibility.acquireFCVSnapshot())) {
-        newVersion.setCurrentVersion(VersionType::CURRENT_CONFIG_VERSION);
-        newVersion.setMinCompatibleVersion(VersionType::MIN_COMPATIBLE_CONFIG_VERSION);
-    }
     auto insertStatus = _localCatalogClient->insertConfigDocument(
         opCtx, VersionType::ConfigNS, newVersion.toBSON(), kNoWaitWriteConcern);
     return insertStatus;
@@ -738,6 +790,12 @@ Status ShardingCatalogManager::_initConfigIndexes(OperationContext* opCtx) {
         return result;
     }
 
+    result = createIndexOnConfigCollection(
+        opCtx, NamespaceString::kConfigDatabasesNamespace, BSON("_id" << 1), unique);
+    if (!result.isOK()) {
+        return result.withContext("couldn't create _id_ index on config db");
+    }
+
     result = createIndexOnConfigCollection(
         opCtx, NamespaceString::kConfigsvrShardsNamespace, BSON(ShardType::host() << 1), unique);
     if (!result.isOK()) {
@@ -758,7 +816,8 @@ Status ShardingCatalogManager::_initConfigIndexes(OperationContext* opCtx) {
 
     if (feature_flags::gGlobalIndexesShardingCatalog.isEnabled(
             serverGlobalParams.featureCompatibility.acquireFCVSnapshot())) {
-        result = sharding_util::createShardingIndexCatalogIndexes(opCtx);
+        result = sharding_util::createShardingIndexCatalogIndexes(
+            opCtx, NamespaceString::kConfigsvrIndexCatalogNamespace);
         if (!result.isOK()) {
             return result;
         }
@@ -800,39 +859,42 @@ Status ShardingCatalogManager::_initConfigCollections(OperationContext* opCtx) {
     return Status::OK();
 }
 
+// TODO (SERVER-83264): Move new validator to _initConfigSettings and remove old validator once 8.0
+// becomes last LTS.
+BSONObj createConfigSettingsValidator() {
+    // (Generic FCV reference): on versions where the balancerSettingsSchema feature flag is
+    // enabled, install an extended validator. This must be the case even for transitional FCV
+    // states, as the validator is installed during FCV upgrade.
+    auto fcvSnapshot = serverGlobalParams.featureCompatibility.acquireFCVSnapshot();
+    auto targetVersion = fcvSnapshot.isUpgradingOrDowngrading()
+        ? getTransitionFCVFromAndTo(fcvSnapshot.getVersion()).second
+        : fcvSnapshot.getVersion();
+
+    if (feature_flags::gBalancerSettingsSchema.isEnabledOnVersion(targetVersion)) {
+        const auto noopValidator = BSON(
+            "properties" << BSON(
+                "_id" << BSON("enum" << BSON_ARRAY(AutoMergeSettingsType::kKey
+                                                   << ReadWriteConcernDefaults::kPersistedDocumentId
+                                                   << "audit"))));
+        return BSON("$jsonSchema" << BSON("oneOf" << BSON_ARRAY(BalancerSettingsType::kSchema
+                                                                << ChunkSizeSettingsType::kSchema
+                                                                << noopValidator)));
+    } else {
+        const auto noopValidator = BSON(
+            "properties" << BSON(
+                "_id" << BSON("enum" << BSON_ARRAY(BalancerSettingsType::kKey
+                                                   << AutoMergeSettingsType::kKey
+                                                   << ReadWriteConcernDefaults::kPersistedDocumentId
+                                                   << "audit"))));
+        return BSON("$jsonSchema" << BSON(
+                        "oneOf" << BSON_ARRAY(ChunkSizeSettingsType::kSchema << noopValidator)));
+    }
+}
+
 Status ShardingCatalogManager::_initConfigSettings(OperationContext* opCtx) {
     DBDirectClient client(opCtx);
 
-    /**
-     * $jsonSchema: {
-     *   oneOf: [
-     *       {"properties": {_id: {enum: ["chunksize"]}},
-     *                      {value: {bsonType: "number", minimum: 1, maximum: 1024}}},
-     *       {"properties": {_id: {enum: ["balancer", "automerge" "ReadWriteConcernDefaults",
-     * "audit"]}}}
-     *   ]
-     * }
-     *
-     * Note: the schema uses "number" for the chunksize instead of "int" because "int" requires the
-     * user to pass NumberInt(x) as the value rather than x (as all of our docs recommend). Non-
-     * integer values will be handled as they were before the schema, by the balancer failing until
-     * a new value is set.
-     */
-    const auto chunkSizeValidator =
-        BSON("properties" << BSON("_id" << BSON("enum" << BSON_ARRAY(ChunkSizeSettingsType::kKey))
-                                        << "value"
-                                        << BSON("bsonType"
-                                                << "number"
-                                                << "minimum" << 1 << "maximum" << 1024))
-                          << "additionalProperties" << false);
-    const auto noopValidator =
-        BSON("properties" << BSON(
-                 "_id" << BSON("enum" << BSON_ARRAY(
-                                   BalancerSettingsType::kKey
-                                   << AutoMergeSettingsType::kKey
-                                   << ReadWriteConcernDefaults::kPersistedDocumentId << "audit"))));
-    const auto fullValidator =
-        BSON("$jsonSchema" << BSON("oneOf" << BSON_ARRAY(chunkSizeValidator << noopValidator)));
+    const auto fullValidator = createConfigSettingsValidator();
 
     BSONObj cmd = BSON("create" << NamespaceString::kConfigSettingsNamespace.coll());
     BSONObj result;
@@ -851,7 +913,7 @@ Status ShardingCatalogManager::_initConfigSettings(OperationContext* opCtx) {
     collModCmd.getCollModRequest().setValidationLevel(ValidationLevelEnum::strict);
     BSONObjBuilder builder;
     return processCollModCommand(
-        opCtx, {NamespaceString::kConfigSettingsNamespace}, collModCmd, &builder);
+        opCtx, {NamespaceString::kConfigSettingsNamespace}, collModCmd, nullptr, &builder);
 }
 
 Status ShardingCatalogManager::setFeatureCompatibilityVersionOnShards(OperationContext* opCtx,
@@ -883,7 +945,7 @@ Status ShardingCatalogManager::setFeatureCompatibilityVersionOnShards(OperationC
         auto response = shard->runCommandWithFixedRetryAttempts(
             opCtx,
             ReadPreferenceSetting{ReadPreference::PrimaryOnly},
-            "admin",
+            DatabaseName::kAdmin,
             cmdObj,
             Shard::RetryPolicy::kIdempotent);
         if (!response.isOK()) {
@@ -966,7 +1028,14 @@ Status ShardingCatalogManager::_notifyClusterOnNewDatabases(
     try {
         // Setup an AlternativeClientRegion and a non-interruptible Operation Context to ensure that
         // the notification may be also sent out while the node is stepping down.
-        auto altClient = opCtx->getServiceContext()->makeClient("_notifyClusterOnNewDatabases");
+        auto altClient = opCtx->getServiceContext()
+                             ->getService(ClusterRole::ShardServer)
+                             ->makeClient("_notifyClusterOnNewDatabases");
+        // TODO(SERVER-74658): Please revisit if this thread could be made killable.
+        {
+            mongo::stdx::lock_guard<mongo::Client> lk(*altClient.get());
+            altClient.get()->setSystemOperationUnkillableByStepdown(lk);
+        }
         AlternativeClientRegion acr(altClient);
         auto altOpCtxHolder = cc().makeOperationContext();
         auto altOpCtx = altOpCtxHolder.get();
@@ -982,7 +1051,7 @@ Status ShardingCatalogManager::_notifyClusterOnNewDatabases(
         // send cmd
         auto executor = Grid::get(altOpCtx)->getExecutorPool()->getFixedExecutor();
         auto responses = sharding_util::sendCommandToShards(altOpCtx,
-                                                            DatabaseName::kAdmin.db(),
+                                                            DatabaseName::kAdmin,
                                                             bob.obj(),
                                                             recipients,
                                                             executor,
@@ -1053,9 +1122,10 @@ BSONObj ShardingCatalogManager::writeToConfigDocumentInTxn(OperationContext* opC
                                                            const BatchedCommandRequest& request,
                                                            TxnNumber txnNumber) {
     invariant(nss.dbName() == DatabaseName::kConfig);
-    auto response = runCommandInLocalTxn(
-                        opCtx, nss.db(), false /* startTransaction */, txnNumber, request.toBSON())
-                        .body;
+    auto response =
+        runCommandInLocalTxn(
+            opCtx, nss.dbName(), false /* startTransaction */, txnNumber, request.toBSON())
+            .body;
 
     uassertStatusOK(getStatusFromWriteCommandReply(response));
 
@@ -1104,10 +1174,12 @@ boost::optional<BSONObj> ShardingCatalogManager::findOneConfigDocumentInTxn(
     findCommand.setSingleBatch(true);
     findCommand.setLimit(1);
 
-    auto res =
-        runCommandInLocalTxn(
-            opCtx, nss.db(), false /*startTransaction*/, txnNumber, findCommand.toBSON(BSONObj()))
-            .body;
+    auto res = runCommandInLocalTxn(opCtx,
+                                    nss.dbName(),
+                                    false /*startTransaction*/,
+                                    txnNumber,
+                                    findCommand.toBSON(BSONObj()))
+                   .body;
     uassertStatusOK(getStatusFromCommandResult(res));
 
     auto cursor = uassertStatusOK(CursorResponse::parseFromBSON(res));
@@ -1123,7 +1195,7 @@ boost::optional<BSONObj> ShardingCatalogManager::findOneConfigDocumentInTxn(
 BSONObj ShardingCatalogManager::findOneConfigDocument(OperationContext* opCtx,
                                                       const NamespaceString& nss,
                                                       const BSONObj& query) {
-    invariant(nss.dbName().db() == DatabaseName::kConfig.db());
+    invariant(nss.isConfigDB());
 
     FindCommandRequest findCommand(nss);
     findCommand.setFilter(query);
@@ -1132,6 +1204,121 @@ BSONObj ShardingCatalogManager::findOneConfigDocument(OperationContext* opCtx,
     return client.findOne(findCommand);
 }
 
+Status ShardingCatalogManager::checkTimeseriesShardKeys(OperationContext* opCtx,
+                                                        const DatabaseName& dbName) {
+
+    // The following aggregation pipeline collects all the collections that are timeseries and are
+    // sharded based on the timeField key. This supposed to run on the config.collections
+    // collection.
+    //
+    // The stages are the following:
+    //
+    // 1. Match all the documents that has timeseriesField property. This is only true for the
+    // timeseries collections
+    //
+    // 2. Add shardKeys and viewName fields.
+    // The shardKeys will be an array with the keys of the elements in the original `key` field.
+    // Eg: { meta.sensorID: 1, timestamp: 1 } -> [ "meta.sensorID", "timestamp" ]
+    // The viewName will be the value of the `_id` field without the `.system.buckets` part.
+    // Eg: `test.system.buckets.shardedCollectionName` -> `test.shardedCollectionName`
+    //
+    // 3. Project the timeShardKey, timeField and viewName into a new document
+    // The timeShardKey is the previously added shardKeys where the element matches the
+    // `.*\.$timeseriesFields.timeField` pattern where $timeseriesFields.timeField is the timeField
+    // of the original timeseries collection.
+    // Eg: [ "meta.sensorID", "timestamp" ] -> [ "timestamp" ]
+    // The timeField is the field name of the timeField in the original timeseries collection
+    // The viewName is the same as previously explained
+    //
+    // 4. Match all documents where the timeShardKey is not empty
+    static const auto rawPipelineStages = [] {
+        auto rawPipelineBSON = fromjson(R"({pipeline: [
+            {
+                "$match":{
+                    "timeseriesFields":{
+                        "$ne":null
+                    }
+                }
+            },
+            {
+                "$addFields":{
+                    "shardKeys":{
+                        "$map":{
+                            "input":{
+                                "$objectToArray":"$key"
+                            },
+                            "in":"$$this.k"
+                        }
+                    },
+                    "viewName":{
+                        "$replaceOne":{
+                            "input":"$_id",
+                            "find":".system.buckets",
+                            "replacement":""
+                        }
+                    }
+                }
+            },
+            {
+                "$project":{
+                    "timeShardKey":{
+                        "$filter":{
+                            "input":"$shardKeys",
+                            "cond":{
+                                "$regexMatch":{
+                                    "input":"$$this",
+                                    "regex":{
+                                        "$concat":[
+                                            ".*\\.",
+                                            "$timeseriesFields.timeField"
+                                        ]
+                                    }
+                                }
+                            }
+                        }
+                    },
+                    "timeField":"$timeseriesFields.timeField",
+                    "viewName":"$viewName"
+                }
+            },
+            {
+                "$match":{
+                    "timeShardKey":{
+                        "$ne":[]
+                    }
+                }
+            }
+        ]})");
+        return parsePipelineFromBSON(rawPipelineBSON.firstElement());
+    }();
+
+    AggregateCommandRequest timeFieldIndexedTimeSeriesAggRequest{
+        NamespaceString::kConfigsvrCollectionsNamespace, rawPipelineStages};
+    auto documents =
+        _localCatalogClient->runCatalogAggregation(opCtx,
+                                                   timeFieldIndexedTimeSeriesAggRequest,
+                                                   {repl::ReadConcernLevel::kMajorityReadConcern});
+
+    for (auto&& document : documents) {
+        const auto viewNameField = document.getField("viewName");
+        const auto viewName = NamespaceStringUtil::deserialize(
+            boost::none, viewNameField.String(), SerializationContext::stateDefault());
+
+        const auto timeFieldField = document.getField("timeField");
+        const auto timeField = timeFieldField.String();
+
+        LOGV2_WARNING(
+            8864701,
+            "The time-series collection is currently using timeField as a shard key. Sharding on "
+            "time will be disabled in future versions. Please reshard your collection using "
+            "metaField as recommended in our time-series sharding documentation.",
+            logAttrs(viewName),
+            "timeField"_attr = timeField);
+    }
+
+    return Status::OK();
+}
+
 void ShardingCatalogManager::withTransactionAPI(OperationContext* opCtx,
                                                 const NamespaceString& namespaceForInitialFind,
                                                 txn_api::Callback callback) {
@@ -1175,10 +1362,6 @@ void ShardingCatalogManager::withTransaction(
 
     AlternativeSessionRegion asr(opCtx);
     auto* const client = asr.opCtx()->getClient();
-    {
-        stdx::lock_guard<Client> lk(*client);
-        client->setSystemOperationKillableByStepdown(lk);
-    }
     asr.opCtx()->setAlwaysInterruptAtStepDownOrUp_UNSAFE();
     AuthorizationSession::get(client)->grantInternalAuthorization(client);
     TxnNumber txnNumber = 0;
@@ -1285,8 +1468,26 @@ void ShardingCatalogManager::initializePlacementHistory(OperationContext* opCtx)
 
     // Delete any existing document that has been already majority committed.
     {
-        repl::ReadConcernArgs::get(opCtx) =
-            repl::ReadConcernArgs(repl::ReadConcernLevel::kMajorityReadConcern);
+        // Set the needed read concern for the operation; since its execution through
+        // _localConfigShard involves the DBDirectClient, RecoveryUnit::ReadSource also needs to
+        // be restored upon exit.
+        auto originalReadConcern =
+            std::exchange(repl::ReadConcernArgs::get(opCtx),
+                          repl::ReadConcernArgs(repl::ReadConcernLevel::kMajorityReadConcern));
+
+        auto originalReadSource =
+            shard_role_details::getRecoveryUnit(opCtx)->getTimestampReadSource();
+        boost::optional<Timestamp> originalReadTimestamp;
+        if (originalReadSource == RecoveryUnit::ReadSource::kProvided) {
+            originalReadTimestamp =
+                shard_role_details::getRecoveryUnit(opCtx)->getPointInTimeReadTimestamp(opCtx);
+        }
+
+        ScopeGuard resetopCtxStateGuard([&] {
+            repl::ReadConcernArgs::get(opCtx) = std::move(originalReadConcern);
+            shard_role_details::getRecoveryUnit(opCtx)->setTimestampReadSource(
+                originalReadSource, originalReadTimestamp);
+        });
 
         write_ops::DeleteCommandRequest deleteOp(
             NamespaceString::kConfigsvrPlacementHistoryNamespace);
@@ -1300,7 +1501,7 @@ void ShardingCatalogManager::initializePlacementHistory(OperationContext* opCtx)
         uassertStatusOK(_localConfigShard->runCommandWithFixedRetryAttempts(
             opCtx,
             ReadPreferenceSetting{ReadPreference::PrimaryOnly},
-            NamespaceString::kConfigsvrPlacementHistoryNamespace.db().toString(),
+            NamespaceString::kConfigsvrPlacementHistoryNamespace.dbName(),
             deleteOp.toBSON(BSON(WriteConcernOptions::kWriteConcernField
                                  << ShardingCatalogClient::kLocalWriteConcern.toBSON())),
             Shard::RetryPolicy::kNotIdempotent));
@@ -1347,7 +1548,14 @@ void ShardingCatalogManager::initializePlacementHistory(OperationContext* opCtx)
     // (This operation includes a $merge stage writing into the config database, which requires
     // internal client credentials).
     {
-        auto altClient = opCtx->getServiceContext()->makeClient("initializePlacementHistory");
+        auto altClient = opCtx->getServiceContext()
+                             ->getService(ClusterRole::ShardServer)
+                             ->makeClient("initializePlacementHistory");
+        // TODO(SERVER-74658): Please revisit if this thread could be made killable.
+        {
+            stdx::lock_guard<Client> lk(*altClient.get());
+            altClient.get()->setSystemOperationUnkillableByStepdown(lk);
+        }
         AuthorizationSession::get(altClient.get())->grantInternalAuthorization(altClient.get());
         AlternativeClientRegion acr(altClient);
         auto executor =
@@ -1433,9 +1641,10 @@ void ShardingCatalogManager::cleanUpPlacementHistory(OperationContext* opCtx,
              << "$" + NamespacePlacementType::kNssFieldName << "mostRecentTimestamp"
              << BSON("$max"
                      << "$" + NamespacePlacementType::kTimestampFieldName)));
-    pipeline.addStage<DocumentSourceMatch>(BSON(
-        "_id" << BSON(
-            "$ne" << ShardingCatalogClient::kConfigPlacementHistoryInitializationMarker.ns())));
+    pipeline.addStage<DocumentSourceMatch>(
+        BSON("_id" << BSON("$ne" << NamespaceStringUtil::serialize(
+                               ShardingCatalogClient::kConfigPlacementHistoryInitializationMarker,
+                               SerializationContext::stateDefault()))));
 
     auto aggRequest = pipeline.buildAsAggregateCommandRequest();
 
@@ -1450,14 +1659,17 @@ void ShardingCatalogManager::cleanUpPlacementHistory(OperationContext* opCtx,
                      &earliestClusterTime](const std::vector<BSONObj>& batch,
                                            const boost::optional<BSONObj>& postBatchResumeToken) {
         for (const auto& obj : batch) {
-            const auto nss = NamespaceString(obj["_id"].String());
+            const auto nss = NamespaceStringUtil::deserialize(
+                boost::none, obj["_id"].String(), SerializationContext::stateDefault());
             const auto timeOfMostRecentDoc = obj["mostRecentTimestamp"].timestamp();
             write_ops::DeleteOpEntry stmt;
 
             const auto minTimeToPreserve = std::min(timeOfMostRecentDoc, earliestClusterTime);
-            stmt.setQ(BSON(NamespacePlacementType::kNssFieldName
-                           << nss.ns() << NamespacePlacementType::kTimestampFieldName
-                           << BSON("$lt" << minTimeToPreserve)));
+            stmt.setQ(
+                BSON(NamespacePlacementType::kNssFieldName
+                     << NamespaceStringUtil::serialize(nss, SerializationContext::stateDefault())
+                     << NamespacePlacementType::kTimestampFieldName
+                     << BSON("$lt" << minTimeToPreserve)));
             stmt.setMulti(true);
             deleteStatements.emplace_back(std::move(stmt));
         }
@@ -1481,13 +1693,37 @@ void ShardingCatalogManager::cleanUpPlacementHistory(OperationContext* opCtx,
     uassertStatusOK(_localConfigShard->runCommandWithFixedRetryAttempts(
         opCtx,
         ReadPreferenceSetting{ReadPreference::PrimaryOnly},
-        NamespaceString::kConfigsvrPlacementHistoryNamespace.db().toString(),
+        NamespaceString::kConfigsvrPlacementHistoryNamespace.dbName(),
         deleteRequest.toBSON({}),
         Shard::RetryPolicy::kIdempotent));
 
     LOGV2_DEBUG(7068808, 2, "Cleaning up placement history - done deleting entries");
 }
 
+int ShardingCatalogManager::deleteMaxSizeMbFromShardEntries(OperationContext* opCtx) {
+    auto unsetMaxSizeMbReq = BSON("$unset" << BSON("maxSize"
+                                                   << ""));
+    DBDirectClient client(opCtx);
+    write_ops::UpdateCommandRequest updateOp(NamespaceString::kConfigsvrShardsNamespace);
+    updateOp.setUpdates({[&] {
+        write_ops::UpdateOpEntry entry;
+        entry.setU(unsetMaxSizeMbReq);
+        entry.setQ({});
+        entry.setMulti(true);
+        entry.setUpsert(false);
+        return entry;
+    }()});
+    updateOp.getWriteCommandRequestBase().setOrdered(false);
+    auto updateReply = client.update(updateOp);
+    write_ops::checkWriteErrors(updateReply);
+    return updateReply.getN();
+}
+
+Status ShardingCatalogManager::upgradeDowngradeConfigSettings(OperationContext* opCtx) {
+    return _initConfigSettings(opCtx);
+}
+
+
 void ShardingCatalogManager::_performLocalNoopWriteWithWAllWriteConcern(OperationContext* opCtx,
                                                                         StringData msg) {
     tenant_migration_access_blocker::performNoopWrite(opCtx, msg);
