@@ -29,16 +29,44 @@
 
 #pragma once
 
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <climits>
+#include <cstdint>
+#include <functional>
+#include <memory>
+#include <string>
+#include <utility>
+#include <vector>
+
+#include "mongo/base/status.h"
 #include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
 #include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/oid.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/client/connection_string.h"
 #include "mongo/client/fetcher.h"
+#include "mongo/client/read_preference.h"
+#include "mongo/client/remote_command_targeter.h"
 #include "mongo/db/auth/authorization_session.h"
 #include "mongo/db/commands/notify_sharding_event_gen.h"
 #include "mongo/db/concurrency/d_concurrency.h"
+#include "mongo/db/database_name.h"
+#include "mongo/db/namespace_string.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/repl/optime_with.h"
+#include "mongo/db/repl/read_concern_level.h"
+#include "mongo/db/server_parameter.h"
+#include "mongo/db/service_context.h"
 #include "mongo/db/session/logical_session_cache.h"
+#include "mongo/db/session/logical_session_id.h"
+#include "mongo/db/shard_id.h"
+#include "mongo/db/tenant_id.h"
+#include "mongo/db/timeseries/timeseries_gen.h"
 #include "mongo/db/transaction/transaction_api.h"
+#include "mongo/db/write_concern_options.h"
 #include "mongo/executor/task_executor.h"
 #include "mongo/platform/mutex.h"
 #include "mongo/s/catalog/sharding_catalog_client.h"
@@ -46,10 +74,14 @@
 #include "mongo/s/catalog/type_collection.h"
 #include "mongo/s/catalog/type_database_gen.h"
 #include "mongo/s/catalog/type_shard.h"
+#include "mongo/s/chunk_version.h"
 #include "mongo/s/client/shard.h"
 #include "mongo/s/client/shard_registry.h"
+#include "mongo/s/database_version.h"
 #include "mongo/s/shard_key_pattern.h"
 #include "mongo/s/write_ops/batched_command_request.h"
+#include "mongo/util/functional.h"
+#include "mongo/util/uuid.h"
 
 namespace mongo {
 
@@ -57,13 +89,13 @@ struct RemoveShardProgress {
     /**
      * Used to indicate to the caller of the removeShard method whether draining of chunks for
      * a particular shard has started, is ongoing, or has been completed. When removing a catalog
-     * shard, there is a new state when waiting for range deletions of all moved away chunks.
-     * Removing other shards will skip this state.
+     * shard, there is a new state when waiting for range deletions of all moved away chunks and any
+     * in progress drops of user collections. Removing other shards will skip this state.
      */
     enum DrainingShardStatus {
         STARTED,
         ONGOING,
-        PENDING_RANGE_DELETIONS,
+        PENDING_DATA_CLEANUP,
         COMPLETED,
     };
 
@@ -72,14 +104,15 @@ struct RemoveShardProgress {
      * jumbo chunks and databases within the shard
      */
     struct DrainingShardUsage {
-        const long long totalChunks;
-        const long long databases;
-        const long long jumboChunks;
+        long long totalChunks;
+        long long databases;
+        long long jumboChunks;
     };
 
     DrainingShardStatus status;
     boost::optional<DrainingShardUsage> remainingCounts;
     boost::optional<long long> pendingRangeDeletions;
+    boost::optional<NamespaceString> firstNonEmptyCollection;
 };
 
 /**
@@ -249,6 +282,13 @@ public:
                                   const NamespaceString& nss,
                                   const BSONObj& query);
 
+    /**
+     * Checks if the shard keys are valid for the timeseries collections in the given database.
+     * If dbName is DatabaseName::kEmpty it checks in all database.
+     */
+    Status checkTimeseriesShardKeys(OperationContext* opCtx,
+                                    const DatabaseName& dbName = DatabaseName::kEmpty);
+
     //
     // Chunk Operations
     //
@@ -427,8 +467,9 @@ public:
      * exists and matches all the prerequisites returns success, otherwise throws NamespaceNotFound.
      */
     DatabaseType createDatabase(OperationContext* opCtx,
-                                StringData dbName,
-                                const boost::optional<ShardId>& optPrimaryShard);
+                                const DatabaseName& dbName,
+                                const boost::optional<ShardId>& optPrimaryShard,
+                                const SerializationContext& serializationContext);
 
     /**
      * Updates the metadata in config.databases collection with the new primary shard for the given
@@ -437,7 +478,8 @@ public:
     void commitMovePrimary(OperationContext* opCtx,
                            const DatabaseName& dbName,
                            const DatabaseVersion& expectedDbVersion,
-                           const ShardId& toShardId);
+                           const ShardId& toShardId,
+                           const SerializationContext& serializationContext);
 
     //
     // Collection Operations
@@ -448,10 +490,25 @@ public:
      * denotes the new shard key, which must contain the old shard key as a prefix.
      *
      * Throws exception on errors.
+     * TODO SERVER-79064: Remove once 8.0 becomes last LTS
      */
-    void refineCollectionShardKey(OperationContext* opCtx,
-                                  const NamespaceString& nss,
-                                  const ShardKeyPattern& newShardKey);
+    void refineCollectionShardKeyDEPRECATED(OperationContext* opCtx,
+                                            const NamespaceString& nss,
+                                            const ShardKeyPattern& newShardKey);
+
+    /**
+     * Executes the commit of the refine collection shard key of an existing collection with
+     * namespace 'nss' using the new transaction API. Here, 'shardKey' denotes the new shard key,
+     * which must contain the old shard key as a prefix.
+     *
+     * Throws exception on transaction errors.
+     */
+    void commitRefineCollectionShardKey(OperationContext* opCtx,
+                                        const NamespaceString& nss,
+                                        const ShardKeyPattern& newShardKey,
+                                        const Timestamp& newTimestamp,
+                                        const OID& newEpoch,
+                                        const Timestamp& oldTimestamp);
 
     void configureCollectionBalancing(OperationContext* opCtx,
                                       const NamespaceString& nss,
@@ -488,6 +545,15 @@ public:
                                      const ConnectionString& shardConnectionString,
                                      bool isConfigShard);
 
+
+    /**
+     *
+     * Adds the config server as a shard. Thin wrapper around 'addShard' that retrieves and uses the
+     * config server connection string as the shard connection string.
+     *
+     */
+    void addConfigShard(OperationContext* opCtx);
+
     /**
      * Inserts the config server shard identity document using a sentinel shard id. Requires the
      * config server's ShardingState has not already been enabled. Throws on errors.
@@ -510,6 +576,13 @@ public:
      */
     Lock::SharedLock enterStableTopologyRegion(OperationContext* opCtx);
 
+    /**
+     * Updates the "hasTwoOrMoreShard" cluster cardinality parameter based on the given number of
+     * shards. Cannot be called while holding the _kShardMembershipLock in exclusive mode since
+     * setting cluster parameters requires taking this lock in shared mode.
+     */
+    Status updateClusterCardinalityParameter(OperationContext* opCtx, int numShards);
+
     //
     // Cluster Upgrade Operations
     //
@@ -519,18 +592,6 @@ public:
      */
     Status setFeatureCompatibilityVersionOnShards(OperationContext* opCtx, const BSONObj& cmdObj);
 
-    /*
-     * Rename collection metadata as part of a renameCollection operation.
-     *
-     * - Updates the FROM collection entry if the source collection is sharded
-     * - Removes the TO collection entry if the target collection was sharded
-     */
-    void renameShardedMetadata(OperationContext* opCtx,
-                               const NamespaceString& from,
-                               const NamespaceString& to,
-                               const WriteConcernOptions& writeConcern,
-                               boost::optional<CollectionType> optFromCollType);
-
     //
     // For Diagnostics
     //
@@ -566,19 +627,6 @@ public:
                               bool force,
                               const Timestamp& validAfter);
 
-    /**
-     * Creates config.settings (if needed) and adds a schema to the collection.
-     */
-    Status upgradeConfigSettings(OperationContext* opCtx);
-
-    /**
-     * Set `onCurrentShardSince` to the same value as `history[0].validAfter` for all config.chunks
-     * entries.
-     * Only called on the FCV upgrade
-     * TODO (SERVER-72791): Remove the method once FCV 7.0 becomes last-lts.
-     */
-    void setOnCurrentShardSinceFieldOnChunks(OperationContext* opCtx);
-
     /**
      * Returns a catalog client that will always run commands locally. Can only be used on a
      * config server node.
@@ -610,6 +658,25 @@ public:
      **/
     void cleanUpPlacementHistory(OperationContext* opCtx, const Timestamp& earliestClusterTime);
 
+    /**
+     * Remove unused `maxSizeMb` field from `config.shards` documents on upgrade to 8.0
+     * returns the number of documents updated
+     * TODO SERVER-80266 delete once 8.0 becomes last lts
+     */
+    int deleteMaxSizeMbFromShardEntries(OperationContext* opCtx);
+
+    /**
+     * Updates the config.settings schema for FCV upgrade and downgrade.
+     * TODO (SERVER-83264): Remove once 8.0 becomes last LTS.
+     */
+    Status upgradeDowngradeConfigSettings(OperationContext* opCtx);
+
+    /**
+     * Schedules an asynchronous unset of the addOrRemoveShardInProgress cluster parameter, in case
+     * a previous addShard/removeShard left it enabled after a failure or crash.
+     */
+    void scheduleAsyncUnblockDDLCoordinators(OperationContext* opCtx);
+
 private:
     /**
      * Performs the necessary checks for version compatibility and creates a new config.version
@@ -680,16 +747,26 @@ private:
      * it returns excluding those named local, config and admin, since they serve administrative
      * purposes.
      */
-    StatusWith<std::vector<std::string>> _getDBNamesListFromShard(
+    StatusWith<std::vector<DatabaseName>> _getDBNamesListFromShard(
         OperationContext* opCtx, std::shared_ptr<RemoteCommandTargeter> targeter);
 
+
+    /**
+     * Runs the listCollections command for every database provided on the specified host and
+     * returns the namespaces of all collections.
+     */
+    StatusWith<std::vector<CollectionType>> _getCollListFromShard(
+        OperationContext* opCtx,
+        const std::vector<DatabaseName>& dbNames,
+        std::shared_ptr<RemoteCommandTargeter> targeter);
+
     /**
      * Runs a command against a "shard" that is not yet in the cluster and thus not present in the
      * ShardRegistry.
      */
     StatusWith<Shard::CommandResponse> _runCommandForAddShard(OperationContext* opCtx,
                                                               RemoteCommandTargeter* targeter,
-                                                              StringData dbName,
+                                                              const DatabaseName& dbName,
                                                               const BSONObj& cmdObj);
 
     /**
@@ -762,7 +839,6 @@ private:
      * Given a vector of cluster parameters in disk format, sets them locally.
      */
     void _setClusterParametersLocally(OperationContext* opCtx,
-                                      const boost::optional<TenantId>& tenantId,
                                       const std::vector<BSONObj>& parameters);
 
     /**
@@ -795,13 +871,12 @@ private:
     /**
      * Execute the migration chunk updates using the internal transaction API.
      */
-    void _commitChunkMigrationInTransaction(
-        OperationContext* opCtx,
-        const NamespaceString& nss,
-        std::shared_ptr<const ChunkType> migratedChunk,
-        std::shared_ptr<const std::vector<ChunkType>> splitChunks,
-        std::shared_ptr<ChunkType> controlChunk,
-        const ShardId& donorShardId);
+    void _commitChunkMigrationInTransaction(OperationContext* opCtx,
+                                            const NamespaceString& nss,
+                                            const ChunkType& migratedChunk,
+                                            const std::vector<ChunkType>& splitChunks,
+                                            const boost::optional<ChunkType>& controlChunk,
+                                            const ShardId& donorShardId);
 
     /**
      * Inserts new entries into the config catalog to describe the shard being added (and the
@@ -809,7 +884,8 @@ private:
      */
     void _addShardInTransaction(OperationContext* opCtx,
                                 const ShardType& newShard,
-                                std::vector<std::string>&& databasesInNewShard);
+                                std::vector<DatabaseName>&& databasesInNewShard,
+                                std::vector<CollectionType>&& collectionsInNewShard);
     /**
      * Use the internal transaction API to remove a shard.
      */
@@ -847,9 +923,28 @@ private:
                                                            const std::vector<BSONObj>& splitPoints);
 
     /**
-     * Performs a noop write locally on the current process and waits for all nodes to replicate it.
+     * Updates the "hasTwoOrMoreShard" cluster cardinality parameter after an add or remove shard
+     * operation if needed. Can only be called after refreshing the shard registry and while holding
+     * _kClusterCardinalityParameterLock lock in exclusive mode to avoid interleaving with other
+     * add/remove shard operation and its set cluster cardinality parameter operation.
+     */
+    Status _updateClusterCardinalityParameterAfterAddShardIfNeeded(const Lock::ExclusiveLock&,
+                                                                   OperationContext* opCtx);
+    Status _updateClusterCardinalityParameterAfterRemoveShardIfNeeded(const Lock::ExclusiveLock&,
+                                                                      OperationContext* opCtx);
+    /*
+     * Commits the new database metadata for a createDatabase operation.
      *
-     * TODO SERVER-75391: Remove.
+     * Throws ShardNotFound if the proposed 'primaryShard' is found to not exist or be draining.
+     * This check (and the actual) commit, is done under the _kShardMembershipLock to ensure
+     * synchronization with removeShard operations.
+     */
+    DatabaseType _commitCreateDatabase(OperationContext* opCtx,
+                                       const DatabaseName& dbName,
+                                       const ShardId& primaryShard);
+
+    /**
+     * Performs a noop write locally on the current process and waits for all nodes to replicate it.
      */
     void _performLocalNoopWriteWithWAllWriteConcern(OperationContext* opCtx, StringData msg);
 
@@ -887,11 +982,21 @@ private:
     // _kZoneOpLock
 
     /**
-     * Lock that guards changes to the set of shards in the cluster (ie addShard and removeShard
-     * requests).
+     * Lock that is held for the entire duration of an add/remove shard operation so that only one
+     * command can execute at a given time.
+     */
+    Lock::ResourceMutex _kAddRemoveShardLock;
+
+    /**
+     * Lock that is held in exclusive mode during the commit phase of an add/remove shard operation.
      */
     Lock::ResourceMutex _kShardMembershipLock;
 
+    /**
+     * Lock that guards changes to the cluster cardinality parameter.
+     */
+    Lock::ResourceMutex _kClusterCardinalityParameterLock;
+
     /**
      * Lock for chunk split/merge/move operations. This should be acquired when doing split/merge/
      * move operations that can affect the config.chunks collection.
