@@ -28,25 +28,82 @@
  */
 
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/s/config/sharding_catalog_manager.h"
-
+#include <boost/smart_ptr.hpp>
+#include <cstdint>
+#include <iterator>
+#include <memory>
 #include <set>
+#include <string>
+#include <tuple>
+#include <utility>
+#include <vector>
 
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bson_field.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/oid.h"
+#include "mongo/bson/timestamp.h"
 #include "mongo/client/read_preference.h"
-#include "mongo/db/commands/feature_compatibility_version.h"
+#include "mongo/db/concurrency/d_concurrency.h"
+#include "mongo/db/database_name.h"
+#include "mongo/db/logical_time.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/ops/write_ops.h"
+#include "mongo/db/ops/write_ops_gen.h"
+#include "mongo/db/repl/optime_with.h"
+#include "mongo/db/repl/read_concern_level.h"
 #include "mongo/db/s/balancer/balancer.h"
-#include "mongo/db/s/sharding_ddl_util.h"
+#include "mongo/db/s/config/sharding_catalog_manager.h"
 #include "mongo/db/s/sharding_logging.h"
 #include "mongo/db/s/sharding_util.h"
+#include "mongo/db/session/logical_session_id.h"
+#include "mongo/db/shard_id.h"
+#include "mongo/db/timeseries/timeseries_gen.h"
 #include "mongo/db/timeseries/timeseries_options.h"
+#include "mongo/db/transaction/transaction_api.h"
 #include "mongo/db/vector_clock.h"
+#include "mongo/db/write_concern_options.h"
+#include "mongo/executor/task_executor_pool.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/compiler.h"
 #include "mongo/s/balancer_configuration.h"
+#include "mongo/s/catalog/sharding_catalog_client.h"
+#include "mongo/s/catalog/type_chunk.h"
+#include "mongo/s/catalog/type_collection.h"
+#include "mongo/s/catalog/type_collection_gen.h"
+#include "mongo/s/catalog/type_shard.h"
 #include "mongo/s/catalog/type_tags.h"
+#include "mongo/s/catalog_cache.h"
 #include "mongo/s/chunk_constraints.h"
+#include "mongo/s/chunk_manager.h"
+#include "mongo/s/client/shard.h"
+#include "mongo/s/client/shard_registry.h"
 #include "mongo/s/grid.h"
+#include "mongo/s/routing_information_cache.h"
+#include "mongo/s/shard_key_pattern.h"
+#include "mongo/s/type_collection_common_types_gen.h"
+#include "mongo/s/write_ops/batched_command_request.h"
+#include "mongo/s/write_ops/batched_command_response.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/fail_point.h"
+#include "mongo/util/future.h"
+#include "mongo/util/future_impl.h"
+#include "mongo/util/namespace_string_util.h"
+#include "mongo/util/out_of_line_executor.h"
+#include "mongo/util/scopeguard.h"
+#include "mongo/util/str.h"
+#include "mongo/util/timer.h"
+#include "mongo/util/uuid.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
@@ -90,14 +147,13 @@ void triggerFireAndForgetShardRefreshes(OperationContext* opCtx,
             shard->runFireAndForgetCommand(
                 opCtx,
                 ReadPreferenceSetting{ReadPreference::PrimaryOnly},
-                DatabaseName::kAdmin.toString(),
-                BSON("_flushRoutingTableCacheUpdates" << coll.getNss().ns()));
+                DatabaseName::kAdmin,
+                BSON("_flushRoutingTableCacheUpdates" << NamespaceStringUtil::serialize(
+                         coll.getNss(), SerializationContext::stateDefault())));
         }
     }
 }
 
-}  // namespace
-
 // Returns the pipeline updates to be used for updating a refined collection's chunk and tag
 // documents.
 //
@@ -208,9 +264,172 @@ std::pair<std::vector<BSONObj>, std::vector<BSONObj>> makeChunkAndTagUpdatesForR
     return std::make_pair(std::move(chunkUpdates), std::move(tagUpdates));
 }
 
-void ShardingCatalogManager::refineCollectionShardKey(OperationContext* opCtx,
-                                                      const NamespaceString& nss,
-                                                      const ShardKeyPattern& newShardKeyPattern) {
+void refineCollectionShardKeyInTxn(OperationContext* opCtx,
+                                   const NamespaceString& nss,
+                                   const ShardKeyPattern& newShardKeyPattern,
+                                   const Timestamp& newTimestamp,
+                                   const OID& newEpoch,
+                                   boost::optional<Timestamp> oldTimestamp) {
+    Timer executionTimer;
+    Timer totalTimer;
+
+    auto updateCollectionAndChunksFn = [&](const txn_api::TransactionClient& txnClient,
+                                           ExecutorPtr txnExec) -> SemiFuture<void> {
+        FindCommandRequest collQuery{CollectionType::ConfigNS};
+        BSONObjBuilder builder;
+        builder.append(CollectionType::kNssFieldName,
+                       NamespaceStringUtil::serialize(nss, SerializationContext::stateDefault()));
+        if (oldTimestamp.is_initialized()) {
+            builder.append(CollectionType::kTimestampFieldName, *oldTimestamp);
+        }
+        collQuery.setFilter(builder.obj());
+        collQuery.setLimit(1);
+        const auto findCollResponse = txnClient.exhaustiveFindSync(collQuery);
+        // TODO SERVER-79064: Remove once 8.0 becomes last LTS
+        uassert(ErrorCodes::NamespaceNotFound,
+                str::stream() << "Expected to find collection " << nss.toStringForErrorMsg(),
+                findCollResponse.size() == 1);
+
+        CollectionType collType(findCollResponse[0]);
+        const auto oldShardKeyPattern = ShardKeyPattern(collType.getKeyPattern());
+        const auto oldFields = oldShardKeyPattern.toBSON();
+        const auto newFields =
+            newShardKeyPattern.toBSON().filterFieldsUndotted(oldFields, false /* inFilter */);
+
+        collType.setEpoch(newEpoch);
+        collType.setTimestamp(newTimestamp);
+        collType.setKeyPattern(newShardKeyPattern.getKeyPattern());
+
+        // Update the config.collections entry for the given namespace.
+        auto catalogUpdateRequest = BatchedCommandRequest::buildUpdateOp(
+            CollectionType::ConfigNS,
+            BSON(CollectionType::kNssFieldName
+                 << NamespaceStringUtil::serialize(nss, SerializationContext::stateDefault())),
+            collType.toBSON(),
+            false /* upsert */,
+            false /* multi */);
+        auto updateCollResponse = txnClient.runCRUDOpSync(catalogUpdateRequest, {});
+        uassertStatusOK(updateCollResponse.toStatus());
+
+        LOGV2(7648601,
+              "refineCollectionShardKey updated collection entry",
+              logAttrs(nss),
+              "durationMillis"_attr = executionTimer.millis(),
+              "totalTimeMillis"_attr = totalTimer.millis());
+        executionTimer.reset();
+        if (MONGO_unlikely(hangRefineCollectionShardKeyBeforeUpdatingChunks.shouldFail())) {
+            LOGV2(7648602, "Hit hangRefineCollectionShardKeyBeforeUpdatingChunks failpoint");
+            hangRefineCollectionShardKeyBeforeUpdatingChunks.pauseWhileSet();
+        }
+
+        auto [chunkUpdates, tagUpdates] = makeChunkAndTagUpdatesForRefine(newFields);
+
+        const auto chunksQuery = BSON(ChunkType::collectionUUID << collType.getUuid());
+        auto chunksUpdateRequest =
+            BatchedCommandRequest::buildPipelineUpdateOp(ChunkType::ConfigNS,
+                                                         chunksQuery,
+                                                         chunkUpdates,
+                                                         false /* upsert */,
+                                                         true /* useMultiUpdate */);
+
+        auto updateChunksResponse = txnClient.runCRUDOpSync(chunksUpdateRequest, {});
+        uassertStatusOK(updateChunksResponse.toStatus());
+        uassert(ErrorCodes::ConflictingOperationInProgress,
+                str::stream() << "Expected to match at least one doc but matched "
+                              << updateChunksResponse.getN(),
+                updateChunksResponse.getN() > 0);
+
+        uassert(ErrorCodes::ConflictingOperationInProgress,
+                str::stream() << "Expected to match one doc but matched "
+                              << updateCollResponse.getN(),
+                updateCollResponse.getN() == 1);
+        LOGV2(7648603,
+              "refineCollectionShardKey: updated chunk entries",
+              logAttrs(nss),
+              "durationMillis"_attr = executionTimer.millis(),
+              "totalTimeMillis"_attr = totalTimer.millis());
+        executionTimer.reset();
+        auto tagUpdateRequest = BatchedCommandRequest::buildPipelineUpdateOp(
+            TagsType::ConfigNS,
+            BSON("ns" << NamespaceStringUtil::serialize(nss, SerializationContext::stateDefault())),
+            tagUpdates,
+            false /* upsert */,
+            true /* useMultiUpdate */);
+        auto updateTagResponse = txnClient.runCRUDOpSync(tagUpdateRequest, {});
+        uassertStatusOK(updateTagResponse.toStatus());
+
+        LOGV2(7648604,
+              "refineCollectionShardKey: updated zone entries",
+              logAttrs(nss),
+              "durationMillis"_attr = executionTimer.millis(),
+              "totalTimeMillis"_attr = totalTimer.millis());
+
+        if (MONGO_unlikely(hangRefineCollectionShardKeyBeforeCommit.shouldFail())) {
+            LOGV2(7648605, "Hit hangRefineCollectionShardKeyBeforeCommit failpoint");
+            hangRefineCollectionShardKeyBeforeCommit.pauseWhileSet();
+        }
+
+        return SemiFuture<void>::makeReady();
+    };
+    auto& executor = Grid::get(opCtx)->getExecutorPool()->getFixedExecutor();
+    auto inlineExecutor = std::make_shared<executor::InlineExecutor>();
+
+    txn_api::SyncTransactionWithRetries txn(opCtx, executor, nullptr, inlineExecutor);
+
+    txn.run(opCtx, updateCollectionAndChunksFn);
+}
+
+}  // namespace
+
+void ShardingCatalogManager::commitRefineCollectionShardKey(
+    OperationContext* opCtx,
+    const NamespaceString& nss,
+    const ShardKeyPattern& newShardKeyPattern,
+    const Timestamp& newTimestamp,
+    const OID& newEpoch,
+    const Timestamp& oldTimestamp) {
+    // Take _kChunkOpLock in exclusive mode to prevent concurrent chunk modifications and generate
+    // strictly monotonously increasing collection placement versions
+    Lock::ExclusiveLock chunkLk(opCtx, _kChunkOpLock);
+    Lock::ExclusiveLock zoneLk(opCtx, _kZoneOpLock);
+
+    // Idempotency check: if the shard key is already the one requested, there is nothing to do
+    // except waiting for majority, in case the write haven't been majority written.
+    auto collType = _localCatalogClient->getCollection(opCtx, nss);
+    if (newTimestamp == collType.getTimestamp()) {
+        uassert(7648607,
+                str::stream() << "Expected refined key " << newShardKeyPattern.toBSON() << " but "
+                              << collType.getKeyPattern().toBSON() << " provided",
+                SimpleBSONObjComparator::kInstance.evaluate(collType.getKeyPattern().toBSON() ==
+                                                            newShardKeyPattern.toBSON()));
+        repl::ReplClientInfo::forClient(opCtx->getClient()).setLastOpToSystemLastOpTime(opCtx);
+        return;
+    }
+
+    // In order to proceed, the timestamp must match.
+    uassert(7648608,
+            str::stream() << "Expected to find collection " << nss.toStringForErrorMsg()
+                          << " with timestamp " << oldTimestamp.toStringPretty(),
+            collType.getTimestamp() == oldTimestamp);
+
+    // The transaction API will use the write concern on the opCtx, which will have the default
+    // sharding wTimeout of 60 seconds. Refining a shard key may involve writing many more
+    // documents than a normal operation, so we override the write concern to not use a
+    // wTimeout, matching the behavior before the API was introduced.
+    WriteConcernOptions originalWC = opCtx->getWriteConcern();
+    opCtx->setWriteConcern(WriteConcernOptions{WriteConcernOptions::kMajority,
+                                               WriteConcernOptions::SyncMode::UNSET,
+                                               WriteConcernOptions::kNoTimeout});
+    ON_BLOCK_EXIT([opCtx, originalWC] { opCtx->setWriteConcern(originalWC); });
+
+    refineCollectionShardKeyInTxn(
+        opCtx, nss, newShardKeyPattern, newTimestamp, newEpoch, oldTimestamp);
+}
+
+void ShardingCatalogManager::refineCollectionShardKeyDEPRECATED(
+    OperationContext* opCtx,
+    const NamespaceString& nss,
+    const ShardKeyPattern& newShardKeyPattern) {
     // Mark opCtx as interruptible to ensure that all reads and writes to the metadata collections
     // under the exclusive _kChunkOpLock happen on the same term.
     opCtx->setAlwaysInterruptAtStepDownOrUp_UNSAFE();
@@ -234,116 +453,13 @@ void ShardingCatalogManager::refineCollectionShardKey(OperationContext* opCtx,
     uassertStatusOK(ShardingLogging::get(opCtx)->logChangeChecked(
         opCtx,
         "refineCollectionShardKey.start",
-        nss.ns(),
+        nss,
         BSON("oldKey" << oldShardKeyPattern.toBSON() << "newKey" << newShardKeyPattern.toBSON()
                       << "oldEpoch" << collType.getEpoch() << "newEpoch" << newEpoch),
         ShardingCatalogClient::kLocalWriteConcern,
         _localConfigShard,
         _localCatalogClient.get()));
 
-    const auto oldFields = oldShardKeyPattern.toBSON();
-    const auto newFields =
-        newShardKeyPattern.toBSON().filterFieldsUndotted(oldFields, false /* inFilter */);
-
-    collType.setEpoch(newEpoch);
-    collType.setKeyPattern(newShardKeyPattern.getKeyPattern());
-
-    auto now = VectorClock::get(opCtx)->getTime();
-    Timestamp newTimestamp = now.clusterTime().asTimestamp();
-    collType.setTimestamp(newTimestamp);
-
-    auto updateCollectionAndChunksWithAPIFn =
-        [collType, newFields, nss, timers](const txn_api::TransactionClient& txnClient,
-                                           ExecutorPtr txnExec) -> SemiFuture<void> {
-        auto [chunkUpdates, tagUpdates] = makeChunkAndTagUpdatesForRefine(newFields);
-
-        // Update the config.collections entry for the given namespace.
-        auto catalogUpdateRequest =
-            BatchedCommandRequest::buildUpdateOp(CollectionType::ConfigNS,
-                                                 BSON(CollectionType::kNssFieldName << nss.ns()),
-                                                 collType.toBSON(),
-                                                 false /* upsert */,
-                                                 false /* multi */);
-        return txnClient.runCRUDOp(catalogUpdateRequest, {})
-            .thenRunOn(txnExec)
-            .then([&txnClient, timers, collType, nss, chunkUpdates = std::move(chunkUpdates)](
-                      auto catalogResponse) {
-                uassertStatusOK(catalogResponse.toStatus());
-
-                LOGV2(5875906,
-                      "refineCollectionShardKey updated collection entry for {namespace}: took "
-                      "{durationMillis} ms. Total time taken: {totalTimeMillis} ms.",
-                      "refineCollectionShardKey updated collection entry",
-                      logAttrs(nss),
-                      "durationMillis"_attr = timers->executionTimer.millis(),
-                      "totalTimeMillis"_attr = timers->totalTimer.millis());
-                timers->executionTimer.reset();
-
-                if (MONGO_unlikely(hangRefineCollectionShardKeyBeforeUpdatingChunks.shouldFail())) {
-                    LOGV2(5875907,
-                          "Hit hangRefineCollectionShardKeyBeforeUpdatingChunks failpoint");
-                    hangRefineCollectionShardKeyBeforeUpdatingChunks.pauseWhileSet();
-                }
-
-                // Update all config.chunks entries for the given namespace by setting (i) their
-                // bounds for each new field in the refined key to MinKey (except for the global max
-                // chunk where the max bounds are set to MaxKey), and unsetting (ii) their jumbo
-                // field.
-                const auto chunksQuery = BSON(ChunkType::collectionUUID << collType.getUuid());
-                auto chunkUpdateRequest =
-                    BatchedCommandRequest::buildPipelineUpdateOp(ChunkType::ConfigNS,
-                                                                 chunksQuery,
-                                                                 chunkUpdates,
-                                                                 false /* upsert */,
-                                                                 true /* useMultiUpdate */);
-
-                return txnClient.runCRUDOp(chunkUpdateRequest, {});
-            })
-            .thenRunOn(txnExec)
-            .then(
-                [&txnClient, timers, nss, tagUpdates = std::move(tagUpdates)](auto chunksResponse) {
-                    uassertStatusOK(chunksResponse.toStatus());
-
-                    LOGV2(5875908,
-                          "refineCollectionShardKey: updated chunk entries for {namespace}: took "
-                          "{durationMillis} ms. Total time taken: {totalTimeMillis} ms.",
-                          "refineCollectionShardKey: updated chunk entries",
-                          logAttrs(nss),
-                          "durationMillis"_attr = timers->executionTimer.millis(),
-                          "totalTimeMillis"_attr = timers->totalTimer.millis());
-                    timers->executionTimer.reset();
-
-                    // Update all config.tags entries for the given namespace by setting their
-                    // bounds for each new field in the refined key to MinKey (except for the global
-                    // max tag where the max bounds are set to MaxKey).
-                    auto tagUpdateRequest =
-                        BatchedCommandRequest::buildPipelineUpdateOp(TagsType::ConfigNS,
-                                                                     BSON("ns" << nss.ns()),
-                                                                     tagUpdates,
-                                                                     false /* upsert */,
-                                                                     true /* useMultiUpdate */);
-                    return txnClient.runCRUDOp(tagUpdateRequest, {});
-                })
-            .thenRunOn(txnExec)
-            .then([&txnClient, timers, nss](auto tagsResponse) {
-                uassertStatusOK(tagsResponse.toStatus());
-
-                LOGV2(5875909,
-                      "refineCollectionShardKey: updated zone entries for {namespace}: took "
-                      "{durationMillis} ms. Total time taken: {totalTimeMillis} ms.",
-                      "refineCollectionShardKey: updated zone entries",
-                      logAttrs(nss),
-                      "durationMillis"_attr = timers->executionTimer.millis(),
-                      "totalTimeMillis"_attr = timers->totalTimer.millis());
-
-                if (MONGO_unlikely(hangRefineCollectionShardKeyBeforeCommit.shouldFail())) {
-                    LOGV2(5875910, "Hit hangRefineCollectionShardKeyBeforeCommit failpoint");
-                    hangRefineCollectionShardKeyBeforeCommit.pauseWhileSet();
-                }
-            })
-            .semi();
-    };
-
     // The transaction API will use the write concern on the opCtx, which will have the default
     // sharding wTimeout of 60 seconds. Refining a shard key may involve writing many more
     // documents than a normal operation, so we override the write concern to not use a
@@ -354,11 +470,14 @@ void ShardingCatalogManager::refineCollectionShardKey(OperationContext* opCtx,
                                                WriteConcernOptions::kNoTimeout});
     ON_BLOCK_EXIT([opCtx, originalWC] { opCtx->setWriteConcern(originalWC); });
 
-    withTransactionAPI(opCtx, nss, std::move(updateCollectionAndChunksWithAPIFn));
+    auto now = VectorClock::get(opCtx)->getTime();
+    Timestamp newTimestamp = now.clusterTime().asTimestamp();
+    refineCollectionShardKeyInTxn(
+        opCtx, nss, newShardKeyPattern, newTimestamp, newEpoch, boost::none);
 
     ShardingLogging::get(opCtx)->logChange(opCtx,
                                            "refineCollectionShardKey.end",
-                                           nss.ns(),
+                                           nss,
                                            BSONObj(),
                                            ShardingCatalogClient::kLocalWriteConcern,
                                            _localConfigShard,
@@ -372,8 +491,6 @@ void ShardingCatalogManager::refineCollectionShardKey(OperationContext* opCtx,
     } catch (const DBException& ex) {
         LOGV2(
             51798,
-            "refineCollectionShardKey: failed to best-effort refresh all shards containing chunks "
-            "in {namespace}",
             "refineCollectionShardKey: failed to best-effort refresh all shards containing chunks",
             "error"_attr = ex.toStatus(),
             logAttrs(nss));
@@ -407,7 +524,7 @@ void ShardingCatalogManager::configureCollectionBalancing(
 
         ShardingLogging::get(opCtx)->logChange(opCtx,
                                                "configureCollectionBalancing",
-                                               nss.ns(),
+                                               nss,
                                                logChangeDetail.obj(),
                                                ShardingCatalogClient::kMajorityWriteConcern,
                                                _localConfigShard,
@@ -473,7 +590,9 @@ void ShardingCatalogManager::configureCollectionBalancing(
         withTransaction(opCtx,
                         CollectionType::ConfigNS,
                         [this, &nss, &update](OperationContext* opCtx, TxnNumber txnNumber) {
-                            const auto query = BSON(CollectionType::kNssFieldName << nss.ns());
+                            const auto query = BSON(
+                                CollectionType::kNssFieldName << NamespaceStringUtil::serialize(
+                                    nss, SerializationContext::stateDefault()));
                             const auto res = writeToConfigDocumentInTxn(
                                 opCtx,
                                 CollectionType::ConfigNS,
@@ -496,8 +615,8 @@ void ShardingCatalogManager::configureCollectionBalancing(
     }
 
     const auto [cm, _] = uassertStatusOK(
-        Grid::get(opCtx)->catalogCache()->getShardedCollectionRoutingInfoWithPlacementRefresh(opCtx,
-                                                                                              nss));
+        RoutingInformationCache::get(opCtx)->getShardedCollectionRoutingInfoWithPlacementRefresh(
+            opCtx, nss));
     std::set<ShardId> shardsIds;
     cm.getAllShardIds(&shardsIds);
 
@@ -513,50 +632,6 @@ void ShardingCatalogManager::configureCollectionBalancing(
     logConfigureCollectionBalancing();
 }
 
-void ShardingCatalogManager::renameShardedMetadata(
-    OperationContext* opCtx,
-    const NamespaceString& from,
-    const NamespaceString& to,
-    const WriteConcernOptions& writeConcern,
-    boost::optional<CollectionType> optFromCollType) {
-    // Take _kChunkOpLock in exclusive mode to prevent concurrent chunk modifications and generate
-    // strictly monotonously increasing collection placement versions
-    Lock::ExclusiveLock chunkLk(opCtx, _kChunkOpLock);
-    Lock::ExclusiveLock zoneLk(opCtx, _kZoneOpLock);
-
-    std::string logMsg = str::stream() << from << " to " << to;
-    if (optFromCollType) {
-        // Rename CSRS metadata in case the source collection is sharded
-        auto collType = *optFromCollType;
-        sharding_ddl_util::shardedRenameMetadata(
-            opCtx, _localConfigShard, _localCatalogClient.get(), collType, to, writeConcern);
-        ShardingLogging::get(opCtx)->logChange(
-            opCtx,
-            "renameCollection.metadata",
-            str::stream() << logMsg << ": dropped target collection and renamed source collection",
-            BSON("newCollMetadata" << collType.toBSON()),
-            ShardingCatalogClient::kLocalWriteConcern,
-            _localConfigShard,
-            _localCatalogClient.get());
-    } else {
-        // Remove stale CSRS metadata in case the source collection is unsharded and the
-        // target collection was sharded
-        // throws if the provided UUID does not match
-        sharding_ddl_util::removeCollAndChunksMetadataFromConfig_notIdempotent(
-            opCtx, _localConfigShard, _localCatalogClient.get(), to, writeConcern);
-        sharding_ddl_util::removeTagsMetadataFromConfig_notIdempotent(
-            opCtx, _localConfigShard, to, writeConcern);
-        ShardingLogging::get(opCtx)->logChange(opCtx,
-                                               "renameCollection.metadata",
-                                               str::stream()
-                                                   << logMsg << " : dropped target collection.",
-                                               BSONObj(),
-                                               ShardingCatalogClient::kLocalWriteConcern,
-                                               _localConfigShard,
-                                               _localCatalogClient.get());
-    }
-}
-
 void ShardingCatalogManager::updateTimeSeriesBucketingParameters(
     OperationContext* opCtx,
     const NamespaceString& nss,
@@ -566,62 +641,63 @@ void ShardingCatalogManager::updateTimeSeriesBucketingParameters(
     Lock::ExclusiveLock lk(opCtx, _kChunkOpLock);
 
     const auto [cm, _] = uassertStatusOK(
-        Grid::get(opCtx)->catalogCache()->getShardedCollectionRoutingInfoWithPlacementRefresh(opCtx,
-                                                                                              nss));
+        RoutingInformationCache::get(opCtx)->getCollectionRoutingInfoWithPlacementRefresh(opCtx,
+                                                                                          nss));
     std::set<ShardId> shardIds;
     cm.getAllShardIds(&shardIds);
 
-    withTransaction(opCtx,
+    withTransaction(
+        opCtx,
+        CollectionType::ConfigNS,
+        [this, &nss, &timeseriesParameters, &shardIds](OperationContext* opCtx,
+                                                       TxnNumber txnNumber) {
+            auto granularityFieldName = CollectionType::kTimeseriesFieldsFieldName + "." +
+                TypeCollectionTimeseriesFields::kGranularityFieldName;
+            auto bucketSpanFieldName = CollectionType::kTimeseriesFieldsFieldName + "." +
+                TypeCollectionTimeseriesFields::kBucketMaxSpanSecondsFieldName;
+            auto bucketRoundingFieldName = CollectionType::kTimeseriesFieldsFieldName + "." +
+                TypeCollectionTimeseriesFields::kBucketRoundingSecondsFieldName;
+
+            BSONObjBuilder updateCmd;
+            BSONObj bucketUp;
+            if (timeseriesParameters.getGranularity().has_value()) {
+                auto bucketSpan = timeseries::getMaxSpanSecondsFromGranularity(
+                    timeseriesParameters.getGranularity().get());
+                updateCmd.append("$unset", BSON(bucketRoundingFieldName << ""));
+                bucketUp = BSON(
+                    granularityFieldName
+                    << BucketGranularity_serializer(timeseriesParameters.getGranularity().get())
+                    << bucketSpanFieldName << bucketSpan);
+            } else {
+                invariant(timeseriesParameters.getBucketMaxSpanSeconds().has_value() &&
+                          timeseriesParameters.getBucketRoundingSeconds().has_value());
+                updateCmd.append("$unset", BSON(granularityFieldName << ""));
+                bucketUp = BSON(bucketSpanFieldName
+                                << timeseriesParameters.getBucketMaxSpanSeconds().get()
+                                << bucketRoundingFieldName
+                                << timeseriesParameters.getBucketRoundingSeconds().get());
+            }
+            updateCmd.append("$set", bucketUp);
+
+            writeToConfigDocumentInTxn(
+                opCtx,
+                CollectionType::ConfigNS,
+                BatchedCommandRequest::buildUpdateOp(
                     CollectionType::ConfigNS,
-                    [this, &nss, &timeseriesParameters, &shardIds](OperationContext* opCtx,
-                                                                   TxnNumber txnNumber) {
-                        auto granularityFieldName = CollectionType::kTimeseriesFieldsFieldName +
-                            "." + TypeCollectionTimeseriesFields::kGranularityFieldName;
-                        auto bucketSpanFieldName = CollectionType::kTimeseriesFieldsFieldName +
-                            "." + TypeCollectionTimeseriesFields::kBucketMaxSpanSecondsFieldName;
-                        auto bucketRoundingFieldName = CollectionType::kTimeseriesFieldsFieldName +
-                            "." + TypeCollectionTimeseriesFields::kBucketRoundingSecondsFieldName;
+                    BSON(CollectionType::kNssFieldName << NamespaceStringUtil::serialize(
+                             nss, SerializationContext::stateDefault())) /* query */,
+                    updateCmd.obj() /* update */,
+                    false /* upsert */,
+                    false /* multi */),
+                txnNumber);
 
-                        BSONObjBuilder updateCmd;
-                        BSONObj bucketUp;
-                        if (timeseriesParameters.getGranularity().has_value()) {
-                            auto bucketSpan = timeseries::getMaxSpanSecondsFromGranularity(
-                                timeseriesParameters.getGranularity().get());
-                            updateCmd.append("$unset", BSON(bucketRoundingFieldName << ""));
-                            bucketUp = BSON(granularityFieldName
-                                            << BucketGranularity_serializer(
-                                                   timeseriesParameters.getGranularity().get())
-                                            << bucketSpanFieldName << bucketSpan);
-                        } else {
-                            invariant(timeseriesParameters.getBucketMaxSpanSeconds().has_value() &&
-                                      timeseriesParameters.getBucketRoundingSeconds().has_value());
-                            updateCmd.append("$unset", BSON(granularityFieldName << ""));
-                            bucketUp =
-                                BSON(bucketSpanFieldName
-                                     << timeseriesParameters.getBucketMaxSpanSeconds().get()
-                                     << bucketRoundingFieldName
-                                     << timeseriesParameters.getBucketRoundingSeconds().get());
-                        }
-                        updateCmd.append("$set", bucketUp);
-
-                        writeToConfigDocumentInTxn(
-                            opCtx,
-                            CollectionType::ConfigNS,
-                            BatchedCommandRequest::buildUpdateOp(
-                                CollectionType::ConfigNS,
-                                BSON(CollectionType::kNssFieldName << nss.ns()) /* query */,
-                                updateCmd.obj() /* update */,
-                                false /* upsert */,
-                                false /* multi */),
-                            txnNumber);
-
-                        // Bump the chunk version for shards.
-                        bumpMajorVersionOneChunkPerShard(opCtx,
-                                                         nss,
-                                                         txnNumber,
-                                                         {std::make_move_iterator(shardIds.begin()),
-                                                          std::make_move_iterator(shardIds.end())});
-                    });
+            // Bump the chunk version for shards.
+            bumpMajorVersionOneChunkPerShard(opCtx,
+                                             nss,
+                                             txnNumber,
+                                             {std::make_move_iterator(shardIds.begin()),
+                                              std::make_move_iterator(shardIds.end())});
+        });
 }
 
 }  // namespace mongo
