@@ -29,25 +29,60 @@
 
 #include "mongo/db/s/drop_collection_coordinator.h"
 
+#include <absl/container/node_hash_map.h>
+#include <algorithm>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/smart_ptr.hpp>
+#include <fmt/format.h>
+#include <tuple>
+#include <utility>
+#include <vector>
+
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
 #include "mongo/db/cancelable_operation_context.h"
-#include "mongo/db/catalog/collection_uuid_mismatch.h"
-#include "mongo/db/concurrency/exception_util.h"
-#include "mongo/db/db_raii.h"
+#include "mongo/db/catalog/collection.h"
+#include "mongo/db/catalog/collection_catalog.h"
+#include "mongo/db/catalog/drop_collection.h"
+#include "mongo/db/catalog_raii.h"
+#include "mongo/db/client.h"
+#include "mongo/db/concurrency/d_concurrency.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
+#include "mongo/db/drop_gen.h"
+#include "mongo/db/repl/repl_client_info.h"
 #include "mongo/db/s/collection_sharding_runtime.h"
+#include "mongo/db/s/forwardable_operation_metadata.h"
 #include "mongo/db/s/participant_block_gen.h"
 #include "mongo/db/s/range_deletion_util.h"
-#include "mongo/db/s/sharded_index_catalog_commands_gen.h"
+#include "mongo/db/s/shard_filtering_metadata_refresh.h"
+#include "mongo/db/s/sharding_ddl_coordinator.h"
 #include "mongo/db/s/sharding_ddl_util.h"
 #include "mongo/db/s/sharding_index_catalog_ddl_util.h"
 #include "mongo/db/s/sharding_logging.h"
-#include "mongo/db/s/sharding_state.h"
+#include "mongo/db/service_context.h"
+#include "mongo/db/session/logical_session_id_gen.h"
+#include "mongo/db/shard_id.h"
+#include "mongo/db/vector_clock_mutable.h"
+#include "mongo/executor/async_rpc.h"
+#include "mongo/executor/async_rpc_util.h"
+#include "mongo/executor/task_executor_pool.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
 #include "mongo/s/analyze_shard_key_documents_gen.h"
 #include "mongo/s/catalog/sharding_catalog_client.h"
+#include "mongo/s/catalog/type_collection.h"
+#include "mongo/s/catalog_cache.h"
+#include "mongo/s/catalog_cache_loader.h"
 #include "mongo/s/client/shard_registry.h"
 #include "mongo/s/grid.h"
-#include "mongo/s/request_types/sharded_ddl_commands_gen.h"
-#include "mongo/s/sharding_feature_flags_gen.h"
+#include "mongo/s/sharding_state.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/future_impl.h"
+#include "mongo/util/out_of_line_executor.h"
+#include "mongo/util/uuid.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
@@ -56,7 +91,9 @@ namespace mongo {
 void DropCollectionCoordinator::dropCollectionLocally(OperationContext* opCtx,
                                                       const NamespaceString& nss,
                                                       bool fromMigrate,
-                                                      bool dropSystemCollections) {
+                                                      bool dropSystemCollections,
+                                                      const boost::optional<UUID>& expectedUUID,
+                                                      bool requireCollectionEmpty) {
 
     boost::optional<UUID> collectionUUID;
     {
@@ -73,9 +110,37 @@ void DropCollectionCoordinator::dropCollectionLocally(OperationContext* opCtx,
             return boost::none;
         }();
 
-        // Clear CollectionShardingRuntime entry.
-        CollectionShardingRuntime::assertCollectionLockedAndAcquireExclusive(opCtx, nss)
-            ->clearFilteringMetadataForDroppedCollection(opCtx);
+        if (collectionUUID && expectedUUID && *collectionUUID != *expectedUUID) {
+            // Ignore the drop collection if the collections exists locally and there is a mismatch
+            // between the current uuid and the expected one.
+            LOGV2_DEBUG(8363400,
+                        1,
+                        "Skipping dropping the collection due to mismatched collection uuid",
+                        "nss"_attr = nss,
+                        "uuid"_attr = *collectionUUID,
+                        "expectedUUID"_attr = *expectedUUID);
+            return;
+        }
+
+        if (requireCollectionEmpty) {
+            bool isEmpty = [&]() {
+                auto localCatalog = CollectionCatalog::get(opCtx);
+                const auto coll = localCatalog->lookupCollectionByNamespace(opCtx, nss);
+                if (coll) {
+                    return coll->isEmpty(opCtx);
+                }
+                return true;
+            }();
+            if (!isEmpty) {
+                // Ignore the drop collection if the collection has records locally and it's
+                // explicitely required to skip non-empty collections.
+                LOGV2_DEBUG(9525700,
+                            1,
+                            "Skipping dropping the collection because it is not empty",
+                            "nss"_attr = nss);
+                return;
+            }
+        }
     }
 
     dropCollectionShardingIndexCatalog(opCtx, nss);
@@ -86,12 +151,8 @@ void DropCollectionCoordinator::dropCollectionLocally(OperationContext* opCtx,
     if (collectionUUID) {
         // The multi-document remove command cannot be run in  transactions, so run it using
         // an alternative client.
-        auto newClient = opCtx->getServiceContext()->makeClient("removeRangeDeletions-" +
-                                                                collectionUUID->toString());
-        {
-            stdx::lock_guard<Client> lk(*newClient.get());
-            newClient->setSystemOperationKillableByStepdown(lk);
-        }
+        auto newClient =
+            opCtx->getService()->makeClient("removeRangeDeletions-" + collectionUUID->toString());
         AlternativeClientRegion acr{newClient};
         auto executor =
             Grid::get(opCtx->getServiceContext())->getExecutorPool()->getFixedExecutor();
@@ -100,7 +161,8 @@ void DropCollectionCoordinator::dropCollectionLocally(OperationContext* opCtx,
             cc().makeOperationContext(), opCtx->getCancellationToken(), executor);
 
         try {
-            removePersistentRangeDeletionTasksByUUID(alternativeOpCtx.get(), *collectionUUID);
+            rangedeletionutil::removePersistentRangeDeletionTasksByUUID(alternativeOpCtx.get(),
+                                                                        *collectionUUID);
         } catch (const DBException& e) {
             LOGV2_ERROR(6501601,
                         "Failed to remove persistent range deletion tasks on drop collection",
@@ -130,9 +192,9 @@ void DropCollectionCoordinator::dropCollectionLocally(OperationContext* opCtx,
     }
 
     // Force the refresh of the catalog cache to purge outdated information. Note also that this
-    // code is indirectly used to notify to secondary nodes to clear their filtering information.
-    const auto catalog = Grid::get(opCtx)->catalogCache();
-    uassertStatusOK(catalog->getCollectionRoutingInfoWithRefresh(opCtx, nss));
+    // code is indirectly used to notify to secondary nodes to clear their filtering information
+    // once the data flushed to disk get replicated.
+    forceShardFilteringMetadataRefresh(opCtx, nss);
     CatalogCacheLoader::get(opCtx).waitForCollectionFlush(opCtx, nss);
 
     // Ensures the remove of range deletions and the refresh of the catalog cache will be waited for
@@ -145,9 +207,6 @@ ExecutorFuture<void> DropCollectionCoordinator::_runImpl(
     const CancellationToken& token) noexcept {
     return ExecutorFuture<void>(**executor)
         .then([this, executor = executor, anchor = shared_from_this()] {
-            if (_isPre70Compatible())
-                return;
-
             if (_doc.getPhase() < Phase::kFreezeCollection)
                 _checkPreconditionsAndSaveArgumentsOnDoc();
         })
@@ -156,49 +215,18 @@ ExecutorFuture<void> DropCollectionCoordinator::_runImpl(
                                      _freezeMigrations(executor);
                                  }))
 
-        .then([this, executor = executor, anchor = shared_from_this()] {
-            if (_isPre70Compatible())
-                return;
-
-            _buildPhaseHandler(Phase::kEnterCriticalSection,
-                               [this, executor = executor, anchor = shared_from_this()] {
-                                   _enterCriticalSection(executor);
-                               })();
-        })
+        .then(_buildPhaseHandler(Phase::kEnterCriticalSection,
+                                 [this, token, executor = executor, anchor = shared_from_this()] {
+                                     _enterCriticalSection(executor, token);
+                                 }))
         .then(_buildPhaseHandler(Phase::kDropCollection,
                                  [this, executor = executor, anchor = shared_from_this()] {
                                      _commitDropCollection(executor);
                                  }))
-        .then([this, executor = executor, anchor = shared_from_this()] {
-            if (_isPre70Compatible())
-                return;
-
-            _buildPhaseHandler(Phase::kReleaseCriticalSection,
-                               [this, executor = executor, anchor = shared_from_this()] {
-                                   _exitCriticalSection(executor);
-                               })();
-        })
-        .onError([this, anchor = shared_from_this()](const Status& status) {
-            if (!status.isA<ErrorCategory::NotPrimaryError>() &&
-                !status.isA<ErrorCategory::ShutdownError>()) {
-                LOGV2_ERROR(5280901,
-                            "Error running drop collection",
-                            logAttrs(nss()),
-                            "error"_attr = redact(status));
-            }
-            return status;
-        });
-}
-
-void DropCollectionCoordinator::_saveCollInfo(OperationContext* opCtx) {
-
-    try {
-        auto coll = Grid::get(opCtx)->catalogClient()->getCollection(opCtx, nss());
-        _doc.setCollInfo(std::move(coll));
-    } catch (const ExceptionFor<ErrorCodes::NamespaceNotFound>&) {
-        // The collection is not sharded or doesn't exist.
-        _doc.setCollInfo(boost::none);
-    }
+        .then(_buildPhaseHandler(Phase::kReleaseCriticalSection,
+                                 [this, token, executor = executor, anchor = shared_from_this()] {
+                                     _exitCriticalSection(executor, token);
+                                 }));
 }
 
 void DropCollectionCoordinator::_checkPreconditionsAndSaveArgumentsOnDoc() {
@@ -221,6 +249,7 @@ void DropCollectionCoordinator::_checkPreconditionsAndSaveArgumentsOnDoc() {
         // at this level, such as the time series collection must be resolved to remove the
         // corresponding bucket collection, or tag documents associated to non-existing collections
         // must be cleaned up.
+        using namespace fmt::literals;
         if (nss().isSystem()) {
             uassert(ErrorCodes::NamespaceNotFound,
                     "namespace {} does not exist"_format(nss().toStringForErrorMsg()),
@@ -230,7 +259,13 @@ void DropCollectionCoordinator::_checkPreconditionsAndSaveArgumentsOnDoc() {
         }
     }
 
-    _saveCollInfo(opCtx);
+    try {
+        auto coll = Grid::get(opCtx)->catalogClient()->getCollection(opCtx, nss());
+        _doc.setCollInfo(std::move(coll));
+    } catch (const ExceptionFor<ErrorCodes::NamespaceNotFound>&) {
+        // The collection is not sharded or doesn't exist.
+        _doc.setCollInfo(boost::none);
+    }
 }
 
 void DropCollectionCoordinator::_freezeMigrations(
@@ -239,41 +274,22 @@ void DropCollectionCoordinator::_freezeMigrations(
     auto* opCtx = opCtxHolder.get();
     getForwardableOpMetadata().setOn(opCtx);
 
-    if (_isPre70Compatible()) {
-        _saveCollInfo(opCtx);
-
-        // TODO SERVER-73627: Remove once 7.0 becomes last LTS.
-        // This check can be removed since it is also present in _checkPreconditions.
-        {
-            AutoGetCollection coll{opCtx,
-                                   nss(),
-                                   MODE_IS,
-                                   AutoGetCollection::Options{}
-                                       .viewMode(auto_get_collection::ViewMode::kViewsPermitted)
-                                       .expectedUUID(_doc.getCollectionUUID())};
-        }
-
-        // Persist the collection info before sticking to using it's uuid. This ensures this node is
-        // still the RS primary, so it was also the primary at the moment we read the collection
-        // metadata.
-        _updateStateDocument(opCtx, StateDoc(_doc));
-    }
-
     BSONObjBuilder logChangeDetail;
     if (_doc.getCollInfo()) {
         logChangeDetail.append("collectionUUID", _doc.getCollInfo()->getUuid().toBSON());
     }
 
     ShardingLogging::get(opCtx)->logChange(
-        opCtx, "dropCollection.start", nss().ns(), logChangeDetail.obj());
+        opCtx, "dropCollection.start", nss(), logChangeDetail.obj());
 
     if (_doc.getCollInfo()) {
-        sharding_ddl_util::stopMigrations(opCtx, nss(), _doc.getCollInfo()->getUuid());
+        const auto collUUID = _doc.getCollInfo()->getUuid();
+        sharding_ddl_util::stopMigrations(opCtx, nss(), collUUID, getNewSession(opCtx));
     }
 }
 
 void DropCollectionCoordinator::_enterCriticalSection(
-    std::shared_ptr<executor::ScopedTaskExecutor> executor) {
+    std::shared_ptr<executor::ScopedTaskExecutor> executor, const CancellationToken& token) {
     LOGV2_DEBUG(7038100, 2, "Acquiring critical section", logAttrs(nss()));
 
     auto opCtxHolder = cc().makeOperationContext();
@@ -283,17 +299,14 @@ void DropCollectionCoordinator::_enterCriticalSection(
     ShardsvrParticipantBlock blockCRUDOperationsRequest(nss());
     blockCRUDOperationsRequest.setBlockType(mongo::CriticalSectionBlockTypeEnum::kReadsAndWrites);
     blockCRUDOperationsRequest.setReason(_critSecReason);
-    blockCRUDOperationsRequest.setAllowViews(true);
 
-    const auto cmdObj =
-        CommandHelpers::appendMajorityWriteConcern(blockCRUDOperationsRequest.toBSON({}))
-            .addFields(getNewSession(opCtx).toBSON());
+    GenericArguments args;
+    async_rpc::AsyncRPCCommandHelpers::appendMajorityWriteConcern(args);
+    async_rpc::AsyncRPCCommandHelpers::appendOSI(args, getNewSession(opCtx));
+    auto opts = std::make_shared<async_rpc::AsyncRPCOptions<ShardsvrParticipantBlock>>(
+        **executor, token, blockCRUDOperationsRequest, args);
     sharding_ddl_util::sendAuthenticatedCommandToShards(
-        opCtx,
-        nss().db(),
-        cmdObj,
-        Grid::get(opCtx)->shardRegistry()->getAllShardIds(opCtx),
-        **executor);
+        opCtx, opts, Grid::get(opCtx)->shardRegistry()->getAllShardIds(opCtx));
 
     LOGV2_DEBUG(7038101, 2, "Acquired critical section", logAttrs(nss()));
 }
@@ -310,7 +323,9 @@ void DropCollectionCoordinator::_commitDropCollection(
 
     // Remove the query sampling configuration document for this collection, if it exists.
     sharding_ddl_util::removeQueryAnalyzerMetadataFromConfig(
-        opCtx, BSON(analyze_shard_key::QueryAnalyzerDocument::kNsFieldName << nss().toString()));
+        opCtx,
+        BSON(analyze_shard_key::QueryAnalyzerDocument::kNsFieldName
+             << NamespaceStringUtil::serialize(nss(), SerializationContext::stateDefault())));
 
     if (collIsSharded) {
         invariant(_doc.getCollInfo());
@@ -333,6 +348,11 @@ void DropCollectionCoordinator::_commitDropCollection(
     // Remove tags even if the collection is not sharded or didn't exist
     sharding_ddl_util::removeTagsMetadataFromConfig(opCtx, nss(), getNewSession(opCtx));
 
+    // Checkpoint the configTime to ensure that, in the case of a stepdown, the new primary will
+    // start-up from a configTime that is inclusive of the metadata removable that was committed
+    // during the critical section.
+    VectorClockMutable::get(opCtx)->waitForDurableConfigTime().get(opCtx);
+
     const auto primaryShardId = ShardingState::get(opCtx)->shardId();
 
     // We need to send the drop to all the shards because both movePrimary and
@@ -363,12 +383,12 @@ void DropCollectionCoordinator::_commitDropCollection(
         false /* fromMigrate */,
         false /* dropSystemCollections */);
 
-    ShardingLogging::get(opCtx)->logChange(opCtx, "dropCollection", nss().ns());
+    ShardingLogging::get(opCtx)->logChange(opCtx, "dropCollection", nss());
     LOGV2(5390503, "Collection dropped", logAttrs(nss()));
 }
 
 void DropCollectionCoordinator::_exitCriticalSection(
-    std::shared_ptr<executor::ScopedTaskExecutor> executor) {
+    std::shared_ptr<executor::ScopedTaskExecutor> executor, const CancellationToken& token) {
     LOGV2_DEBUG(7038102, 2, "Releasing critical section", logAttrs(nss()));
 
     auto opCtxHolder = cc().makeOperationContext();
@@ -378,17 +398,14 @@ void DropCollectionCoordinator::_exitCriticalSection(
     ShardsvrParticipantBlock unblockCRUDOperationsRequest(nss());
     unblockCRUDOperationsRequest.setBlockType(CriticalSectionBlockTypeEnum::kUnblock);
     unblockCRUDOperationsRequest.setReason(_critSecReason);
-    unblockCRUDOperationsRequest.setAllowViews(true);
 
-    const auto cmdObj =
-        CommandHelpers::appendMajorityWriteConcern(unblockCRUDOperationsRequest.toBSON({}))
-            .addFields(getNewSession(opCtx).toBSON());
+    GenericArguments args;
+    async_rpc::AsyncRPCCommandHelpers::appendMajorityWriteConcern(args);
+    async_rpc::AsyncRPCCommandHelpers::appendOSI(args, getNewSession(opCtx));
+    auto opts = std::make_shared<async_rpc::AsyncRPCOptions<ShardsvrParticipantBlock>>(
+        **executor, token, unblockCRUDOperationsRequest, args);
     sharding_ddl_util::sendAuthenticatedCommandToShards(
-        opCtx,
-        nss().db(),
-        cmdObj,
-        Grid::get(opCtx)->shardRegistry()->getAllShardIds(opCtx),
-        **executor);
+        opCtx, opts, getAllShardsAndConfigServerIds(opCtx));
 
     LOGV2_DEBUG(7038103, 2, "Released critical section", logAttrs(nss()));
 }
