@@ -27,10 +27,35 @@
  *    it in the license file.
  */
 
+#include <list>
+#include <ostream>
+#include <string>
+
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/init.h"  // IWYU pragma: keep
+#include "mongo/base/initializer.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/oid.h"
+#include "mongo/client/dbclient_cursor.h"
+#include "mongo/db/client.h"
 #include "mongo/db/commands/list_collections_filter.h"
+#include "mongo/db/create_indexes_gen.h"
+#include "mongo/db/database_name.h"
 #include "mongo/db/dbdirectclient.h"
-#include "mongo/db/op_observer/op_observer_noop.h"
+#include "mongo/db/exec/document_value/value_comparator.h"
+#include "mongo/db/keypattern.h"
+#include "mongo/db/op_observer/op_observer.h"
 #include "mongo/db/op_observer/op_observer_registry.h"
+#include "mongo/db/ops/write_ops_gen.h"
+#include "mongo/db/query/collation/collator_interface.h"
+#include "mongo/db/query/find_command.h"
+#include "mongo/db/read_write_concern_defaults.h"
 #include "mongo/db/read_write_concern_defaults_cache_lookup_mock.h"
 #include "mongo/db/repl/database_cloner_gen.h"
 #include "mongo/db/repl/primary_only_service_test_fixture.h"
@@ -40,12 +65,31 @@
 #include "mongo/db/s/global_index/global_index_cloning_service.h"
 #include "mongo/db/s/global_index/global_index_util.h"
 #include "mongo/db/s/resharding/resharding_service_test_helpers.h"
+#include "mongo/db/s/sharding_test_helpers.h"
+#include "mongo/db/server_parameter.h"
+#include "mongo/db/session/logical_session_cache.h"
 #include "mongo/db/session/logical_session_cache_noop.h"
 #include "mongo/db/session/session_catalog_mongod.h"
+#include "mongo/db/shard_id.h"
+#include "mongo/db/tenant_id.h"
 #include "mongo/db/transaction/session_catalog_mongod_transaction_interface_impl.h"
+#include "mongo/idl/idl_parser.h"
 #include "mongo/idl/server_parameter_test_util.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/s/catalog/type_chunk.h"
+#include "mongo/s/chunk_manager.h"
+#include "mongo/s/chunk_version.h"
+#include "mongo/s/database_version.h"
 #include "mongo/s/request_types/sharded_ddl_commands_gen.h"
+#include "mongo/s/resharding/type_collection_fields_gen.h"
+#include "mongo/s/sharding_test_fixture_common.h"
+#include "mongo/s/type_collection_common_types_gen.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/uuid.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
 
@@ -60,6 +104,7 @@ using OpObserverForTest =
                                                                          GlobalIndexClonerDoc>;
 using PauseDuringStateTransitions =
     resharding_service_test_helpers::PauseDuringStateTransitions<GlobalIndexClonerStateEnum>;
+using Fault = sharding_test_helpers::Fault;
 
 const ShardId kRecipientShardId{"myShardId"};
 const NamespaceString kSourceNss =
@@ -86,52 +131,28 @@ public:
         auto rt = RoutingTableHistory::makeNew(kSourceNss,
                                                _sourceUUID,
                                                BSON(kSourceShardKey << 1),
+                                               false, /* unsplittable */
                                                nullptr /* defaultCollator */,
                                                false /* unique */,
-                                               std::move(epoch),
+                                               epoch,
                                                Timestamp(1, 1),
                                                boost::none /* timeseriesFields */,
                                                boost::none /* reshardingFields */,
                                                true /* allowMigrations */,
                                                chunks);
 
-        return ChunkManager(_someDonorId,
-                            DatabaseVersion(UUID::gen(), Timestamp(1, 1)),
-                            _makeStandaloneRoutingTableHistory(std::move(rt)),
-                            boost::none /* clusterTime */);
+        return ChunkManager(
+            _someDonorId,
+            DatabaseVersion(UUID::gen(), Timestamp(1, 1)),
+            ShardingTestFixtureCommon::makeStandaloneRoutingTableHistory(std::move(rt)),
+            boost::none /* clusterTime */);
     }
 
 private:
-    RoutingTableHistoryValueHandle _makeStandaloneRoutingTableHistory(
-        RoutingTableHistory rt) const {
-        const auto version = rt.getVersion();
-        return RoutingTableHistoryValueHandle(
-            std::make_shared<RoutingTableHistory>(std::move(rt)),
-            ComparableChunkVersion::makeComparableChunkVersion(version));
-    }
-
     const UUID _sourceUUID{UUID::gen()};
     const ShardId _someDonorId{"otherShardId"};
 };
 
-class Fault {
-public:
-    Fault(Status error, int triggerCount = 1)
-        : _error(std::move(error)), _remainingTriggerCount(triggerCount) {}
-
-    void throwIfEnabled() {
-        if (_remainingTriggerCount == 0 || _remainingTriggerCount-- == 0) {
-            return;
-        }
-
-        uassertStatusOK(_error);
-    }
-
-private:
-    const Status _error;
-    int _remainingTriggerCount{0};
-};
-
 template <typename T>
 struct FaultOrData {
 public:
@@ -310,7 +331,7 @@ public:
         createGlobalIndex.setDbName(DatabaseName::kAdmin);
         BSONObj cmdResult;
         auto success =
-            client.runCommand({boost::none, "admin"}, createGlobalIndex.toBSON({}), cmdResult);
+            client.runCommand(DatabaseName::kAdmin, createGlobalIndex.toBSON({}), cmdResult);
         ASSERT(success) << "createGlobalIndex cmd failed with result: " << cmdResult;
     }
 
