@@ -28,10 +28,17 @@
  */
 
 
-#include "mongo/platform/basic.h"
+#include <utility>
 
+
+#include "mongo/base/error_codes.h"
+#include "mongo/db/cluster_role.h"
 #include "mongo/db/s/global_user_write_block_state.h"
+#include "mongo/db/server_options.h"
+#include "mongo/db/transaction_resources.h"
 #include "mongo/db/write_block_bypass.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/decorable.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
@@ -60,7 +67,7 @@ void GlobalUserWriteBlockState::disableUserWriteBlocking(OperationContext* opCtx
 
 void GlobalUserWriteBlockState::checkUserWritesAllowed(OperationContext* opCtx,
                                                        const NamespaceString& nss) const {
-    invariant(opCtx->lockState()->isLocked());
+    invariant(shard_role_details::getLocker(opCtx)->isLocked());
     uassert(ErrorCodes::UserWritesBlocked,
             "User writes blocked",
             !_globalUserWritesBlocked.load() ||
@@ -69,7 +76,7 @@ void GlobalUserWriteBlockState::checkUserWritesAllowed(OperationContext* opCtx,
 }
 
 bool GlobalUserWriteBlockState::isUserWriteBlockingEnabled(OperationContext* opCtx) const {
-    invariant(opCtx->lockState()->isLocked());
+    invariant(shard_role_details::getLocker(opCtx)->isLocked());
     return _globalUserWritesBlocked.load();
 }
 
