@@ -27,13 +27,23 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <algorithm>
+#include <fmt/format.h>
+#include <functional>
+#include <utility>
+#include <vector>
 
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/db/s/metrics/sharding_data_transform_cumulative_metrics.h"
 #include "mongo/db/s/metrics/sharding_data_transform_instance_metrics.h"
 #include "mongo/db/s/metrics/sharding_data_transform_metrics_test_fixture.h"
-#include "mongo/logv2/log.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/clock_source_mock.h"
 #include "mongo/util/duration.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
@@ -105,7 +115,7 @@ public:
                                                std::move(observer)},
           _scopedObserver(registerInstanceMetrics()) {}
 
-    boost::optional<Milliseconds> getRecipientHighEstimateRemainingTimeMillis() const {
+    boost::optional<Milliseconds> getRecipientHighEstimateRemainingTimeMillis() const override {
         return boost::none;
     }
 
@@ -168,7 +178,7 @@ public:
     void createMetricsAndAssertIncrementsCumulativeMetricsField(
         const std::function<void(ShardingDataTransformInstanceMetrics*)>& mutate,
         Section section,
-        const StringData& fieldName) {
+        StringData fieldName) {
         auto metrics = createInstanceMetrics(UUID::gen(), Role::kCoordinator);
         assertIncrementsCumulativeMetricsField(metrics.get(), mutate, section, fieldName);
     }
