@@ -29,38 +29,85 @@
 
 #include "mongo/db/s/migration_source_manager.h"
 
+#include <absl/container/node_hash_map.h>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <mutex>
+#include <string>
+#include <tuple>
+#include <type_traits>
+#include <vector>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
 #include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/client/read_preference.h"
+#include "mongo/db/catalog/collection.h"
 #include "mongo/db/catalog_raii.h"
+#include "mongo/db/client.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
+#include "mongo/db/database_name.h"
+#include "mongo/db/keypattern.h"
 #include "mongo/db/operation_context.h"
+#include "mongo/db/persistent_task_store.h"
 #include "mongo/db/read_concern.h"
+#include "mongo/db/repl/optime.h"
+#include "mongo/db/repl/read_concern_args.h"
+#include "mongo/db/repl/read_concern_level.h"
+#include "mongo/db/repl/repl_client_info.h"
 #include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/s/auto_split_vector.h"
 #include "mongo/db/s/chunk_operation_precondition_checks.h"
 #include "mongo/db/s/commit_chunk_migration_gen.h"
 #include "mongo/db/s/migration_chunk_cloner_source.h"
 #include "mongo/db/s/migration_coordinator.h"
+#include "mongo/db/s/migration_coordinator_document_gen.h"
 #include "mongo/db/s/migration_util.h"
 #include "mongo/db/s/range_deletion_util.h"
 #include "mongo/db/s/shard_filtering_metadata_refresh.h"
 #include "mongo/db/s/shard_metadata_util.h"
 #include "mongo/db/s/sharding_logging.h"
 #include "mongo/db/s/sharding_runtime_d_params_gen.h"
-#include "mongo/db/s/sharding_state.h"
-#include "mongo/db/s/sharding_state_recovery.h"
 #include "mongo/db/s/sharding_statistics.h"
 #include "mongo/db/s/type_shard_collection.h"
+#include "mongo/db/s/type_shard_collection_gen.h"
+#include "mongo/db/service_context.h"
+#include "mongo/db/storage/recovery_unit.h"
 #include "mongo/db/timeseries/bucket_catalog/bucket_catalog.h"
-#include "mongo/db/vector_clock.h"
-#include "mongo/db/vector_clock_mutable.h"
+#include "mongo/db/transaction_resources.h"
+#include "mongo/db/write_concern.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/redaction.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/s/catalog/sharding_catalog_client.h"
 #include "mongo/s/catalog/type_chunk.h"
 #include "mongo/s/catalog_cache_loader.h"
+#include "mongo/s/chunk.h"
+#include "mongo/s/chunk_manager.h"
+#include "mongo/s/client/shard.h"
+#include "mongo/s/client/shard_registry.h"
 #include "mongo/s/grid.h"
 #include "mongo/s/shard_key_pattern.h"
 #include "mongo/s/shard_version_factory.h"
+#include "mongo/s/sharding_state.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/clock_source.h"
+#include "mongo/util/decorable.h"
 #include "mongo/util/duration.h"
 #include "mongo/util/fail_point.h"
+#include "mongo/util/namespace_string_util.h"
 #include "mongo/util/scopeguard.h"
+#include "mongo/util/str.h"
+#include "mongo/util/time_support.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kShardingMigration
 
@@ -129,7 +176,6 @@ MONGO_FAIL_POINT_DEFINE(hangBeforeEnteringCriticalSection);
 MONGO_FAIL_POINT_DEFINE(hangBeforeLeavingCriticalSection);
 MONGO_FAIL_POINT_DEFINE(migrationCommitNetworkError);
 MONGO_FAIL_POINT_DEFINE(hangBeforePostMigrationCommitRefresh);
-MONGO_FAIL_POINT_DEFINE(overrideDefaultShardingCatalogClientWriteConcernTimeout);
 
 }  // namespace
 
@@ -162,25 +208,23 @@ MigrationSourceManager::MigrationSourceManager(OperationContext* opCtx,
                           << _args.getToShard())),
       _moveTimingHelper(_opCtx,
                         "from",
-                        _args.getCommandParameter().ns(),
+                        _args.getCommandParameter(),
                         _args.getMin(),
                         _args.getMax(),
                         6,  // Total number of steps
                         _args.getToShard(),
                         _args.getFromShard()) {
-    invariant(!_opCtx->lockState()->isLocked());
+    invariant(!shard_role_details::getLocker(_opCtx)->isLocked());
     // Since the MigrationSourceManager is registered on the CSR from the constructor, another
     // thread can get it and abort the migration (and get a reference to the completion promise's
     // future). When this happens, since we throw an exception from the constructor, the destructor
     // will not run, so we have to do complete it here, otherwise we get a BrokenPromise
+    // TODO (SERVER-92531): Use existing clean up infrastructure when aborting in early stages
     ScopeGuard scopedGuard([&] { _completion.emplaceValue(); });
 
     LOGV2(22016,
-          "Starting chunk migration donation {requestParameters} with expected collection epoch "
-          "{collectionEpoch}",
           "Starting chunk migration donation",
-          "requestParameters"_attr = redact(_args.toBSON({})),
-          "collectionEpoch"_attr = _args.getEpoch());
+          "requestParameters"_attr = redact(_args.toBSON({})));
 
     _moveTimingHelper.done(1);
     moveChunkHangAtStep1.pauseWhileSet();
@@ -210,8 +254,8 @@ MigrationSourceManager::MigrationSourceManager(OperationContext* opCtx,
             AutoGetCollection autoColl(_opCtx, nss(), MODE_IS);
             const auto scopedCsr =
                 CollectionShardingRuntime::assertCollectionLockedAndAcquireShared(opCtx, nss());
-            const auto [metadata, _] =
-                checkCollectionIdentity(_opCtx, nss(), _args.getEpoch(), boost::none);
+            const auto [metadata, _] = checkCollectionIdentity(
+                _opCtx, nss(), _args.getEpoch(), _args.getCollectionTimestamp());
             return metadata;
         }();
 
@@ -254,13 +298,13 @@ MigrationSourceManager::MigrationSourceManager(OperationContext* opCtx,
     // MigrationSourceManager on the CSR.
     const auto [collectionMetadata, collectionIndexInfo, collectionUUID] = [&] {
         // TODO (SERVER-71444): Fix to be interruptible or document exception.
-        UninterruptibleLockGuard noInterrupt(_opCtx->lockState());  // NOLINT.
+        UninterruptibleLockGuard noInterrupt(_opCtx);  // NOLINT.
         AutoGetCollection autoColl(_opCtx, nss(), MODE_IS);
         auto scopedCsr =
             CollectionShardingRuntime::assertCollectionLockedAndAcquireExclusive(opCtx, nss());
 
-        const auto [metadata, indexInfo] =
-            checkCollectionIdentity(_opCtx, nss(), _args.getEpoch(), boost::none);
+        auto [metadata, indexInfo] = checkCollectionIdentity(
+            _opCtx, nss(), _args.getEpoch(), _args.getCollectionTimestamp());
 
         UUID collectionUUID = autoColl.getCollection()->uuid();
 
@@ -287,7 +331,7 @@ MigrationSourceManager::MigrationSourceManager(OperationContext* opCtx,
     // CollectionShardingRuntime::waitForClean() allows to sync on tasks already registered on the
     // RangeDeleterService, but may miss pending ones in case this code runs after a failover. The
     // enclosing while loop allows to address such a gap.
-    while (migrationutil::checkForConflictingDeletions(opCtx, range, collectionUUID)) {
+    while (rangedeletionutil::checkForConflictingDeletions(opCtx, range, collectionUUID)) {
         uassert(ErrorCodes::ResumableRangeDeleterDisabled,
                 "Failing migration because the disableResumableRangeDeleter server "
                 "parameter is set to true on the donor shard, which contains range "
@@ -332,6 +376,7 @@ MigrationSourceManager::MigrationSourceManager(OperationContext* opCtx,
 
     _collectionEpoch = _args.getEpoch();
     _collectionUUID = collectionUUID;
+    _collectionTimestamp = _args.getCollectionTimestamp();
 
     _chunkVersion = collectionMetadata.getChunkManager()
                         ->findIntersectingChunkWithSimpleCollation(*_args.getMin())
@@ -360,38 +405,26 @@ MigrationSourceManager::~MigrationSourceManager() {
 }
 
 void MigrationSourceManager::startClone() {
-    invariant(!_opCtx->lockState()->isLocked());
+    invariant(!shard_role_details::getLocker(_opCtx)->isLocked());
     invariant(_state == kCreated);
     ScopeGuard scopedGuard([&] { _cleanupOnError(); });
     _stats.countDonorMoveChunkStarted.addAndFetch(1);
 
-    auto wcOptions = [&]() {
-        auto wcOptions = ShardingCatalogClient::kMajorityWriteConcern;
-
-        overrideDefaultShardingCatalogClientWriteConcernTimeout.execute([&](const BSONObj& data) {
-            auto elem = data.getField("wTimeoutMillis");
-            auto wcTimeout = Milliseconds(elem.exactNumberLong());
-            wcOptions = WriteConcernOptions(
-                WriteConcernOptions::kMajority, WriteConcernOptions::SyncMode::UNSET, wcTimeout);
-        });
-        return wcOptions;
-    }();
-
     uassertStatusOK(ShardingLogging::get(_opCtx)->logChangeChecked(
         _opCtx,
         "moveChunk.start",
-        nss().ns(),
+        nss(),
         BSON("min" << *_args.getMin() << "max" << *_args.getMax() << "from" << _args.getFromShard()
                    << "to" << _args.getToShard()),
-        wcOptions));
+        ShardingCatalogClient::kMajorityWriteConcern));
 
     _cloneAndCommitTimer.reset();
 
     auto replCoord = repl::ReplicationCoordinator::get(_opCtx);
-    auto replEnabled = replCoord->isReplEnabled();
+    auto replEnabled = replCoord->getSettings().isReplSet();
 
     {
-        const auto metadata = _getCurrentMetadataAndCheckEpoch();
+        const auto metadata = _getCurrentMetadataAndCheckForConflictingErrors();
 
         AutoGetCollection autoColl(_opCtx,
                                    nss(),
@@ -458,7 +491,7 @@ void MigrationSourceManager::startClone() {
 }
 
 void MigrationSourceManager::awaitToCatchUp() {
-    invariant(!_opCtx->lockState()->isLocked());
+    invariant(!shard_role_details::getLocker(_opCtx)->isLocked());
     invariant(_state == kCloning);
     ScopeGuard scopedGuard([&] { _cleanupOnError(); });
     _stats.totalDonorChunkCloneTimeMillis.addAndFetch(_cloneAndCommitTimer.millis());
@@ -475,7 +508,7 @@ void MigrationSourceManager::awaitToCatchUp() {
 }
 
 void MigrationSourceManager::enterCriticalSection() {
-    invariant(!_opCtx->lockState()->isLocked());
+    invariant(!shard_role_details::getLocker(_opCtx)->isLocked());
     invariant(_state == kCloneCaughtUp);
     ScopeGuard scopedGuard([&] { _cleanupOnError(); });
     _stats.totalDonorChunkCloneTimeMillis.addAndFetch(_cloneAndCommitTimer.millis());
@@ -491,16 +524,13 @@ void MigrationSourceManager::enterCriticalSection() {
     if (!cm.getVersion(_args.getToShard()).isSet()) {
         migrationutil::notifyChangeStreamsOnRecipientFirstChunk(
             _opCtx, nss(), _args.getFromShard(), _args.getToShard(), _collectionUUID);
-    }
 
-    // Mark the shard as running critical operation, which requires recovery on crash.
-    //
-    // NOTE: The 'migrateChunkToNewShard' oplog message written by the above call to
-    // 'notifyChangeStreamsOnRecipientFirstChunk' depends on this majority write to carry its local
-    // write to majority committed.
-    // TODO (SERVER-60110): Remove once 7.0 becomes last LTS.
-    uassertStatusOKWithContext(ShardingStateRecovery_DEPRECATED::startMetadataOp(_opCtx),
-                               "Start metadata op");
+        // Wait for the above 'migrateChunkToNewShard' oplog message to be majority acknowledged.
+        WriteConcernResult ignoreResult;
+        auto latestOpTime = repl::ReplClientInfo::forClient(_opCtx->getClient()).getLastOp();
+        uassertStatusOK(waitForWriteConcern(
+            _opCtx, latestOpTime, WriteConcerns::kMajorityWriteConcernNoTimeout, &ignoreResult));
+    }
 
     LOGV2_DEBUG_OPTIONS(4817402,
                         2,
@@ -521,7 +551,8 @@ void MigrationSourceManager::enterCriticalSection() {
     uassertStatusOKWithContext(
         shardmetadatautil::updateShardCollectionsEntry(
             _opCtx,
-            BSON(ShardCollectionType::kNssFieldName << nss().ns()),
+            BSON(ShardCollectionType::kNssFieldName
+                 << NamespaceStringUtil::serialize(nss(), SerializationContext::stateDefault())),
             BSON("$inc" << BSON(ShardCollectionType::kEnterCriticalSectionCounterFieldName << 1)),
             false /*upsert*/),
         "Persist critical section signal for secondaries");
@@ -534,7 +565,7 @@ void MigrationSourceManager::enterCriticalSection() {
 }
 
 void MigrationSourceManager::commitChunkOnRecipient() {
-    invariant(!_opCtx->lockState()->isLocked());
+    invariant(!shard_role_details::getLocker(_opCtx)->isLocked());
     invariant(_state == kCriticalSection);
     ScopeGuard scopedGuard([&] {
         _cleanupOnError();
@@ -560,7 +591,7 @@ void MigrationSourceManager::commitChunkOnRecipient() {
 }
 
 void MigrationSourceManager::commitChunkMetadataOnConfig() {
-    invariant(!_opCtx->lockState()->isLocked());
+    invariant(!shard_role_details::getLocker(_opCtx)->isLocked());
     invariant(_state == kCloneCompleted);
 
     ScopeGuard scopedGuard([&] {
@@ -574,7 +605,7 @@ void MigrationSourceManager::commitChunkMetadataOnConfig() {
     BSONObjBuilder builder;
 
     {
-        const auto metadata = _getCurrentMetadataAndCheckEpoch();
+        const auto metadata = _getCurrentMetadataAndCheckForConflictingErrors();
 
         auto migratedChunk = MigratedChunkType(*_chunkVersion, *_args.getMin(), *_args.getMax());
 
@@ -600,7 +631,7 @@ void MigrationSourceManager::commitChunkMetadataOnConfig() {
         Grid::get(_opCtx)->shardRegistry()->getConfigShard()->runCommandWithFixedRetryAttempts(
             _opCtx,
             ReadPreferenceSetting{ReadPreference::PrimaryOnly},
-            "admin",
+            DatabaseName::kAdmin,
             builder.obj(),
             Shard::RetryPolicy::kIdempotent);
 
@@ -615,7 +646,7 @@ void MigrationSourceManager::commitChunkMetadataOnConfig() {
     if (!migrationCommitStatus.isOK()) {
         {
             // TODO (SERVER-71444): Fix to be interruptible or document exception.
-            UninterruptibleLockGuard noInterrupt(_opCtx->lockState());  // NOLINT.
+            UninterruptibleLockGuard noInterrupt(_opCtx);  // NOLINT.
             AutoGetCollection autoColl(_opCtx, nss(), MODE_IX);
             CollectionShardingRuntime::assertCollectionLockedAndAcquireExclusive(_opCtx, nss())
                 ->clearFilteringMetadata(_opCtx);
@@ -654,7 +685,7 @@ void MigrationSourceManager::commitChunkMetadataOnConfig() {
                             "error"_attr = redact(ex));
         {
             // TODO (SERVER-71444): Fix to be interruptible or document exception.
-            UninterruptibleLockGuard noInterrupt(_opCtx->lockState());  // NOLINT.
+            UninterruptibleLockGuard noInterrupt(_opCtx);  // NOLINT.
             AutoGetCollection autoColl(_opCtx, nss(), MODE_IX);
             CollectionShardingRuntime::assertCollectionLockedAndAcquireExclusive(_opCtx, nss())
                 ->clearFilteringMetadata(_opCtx);
@@ -668,7 +699,7 @@ void MigrationSourceManager::commitChunkMetadataOnConfig() {
 
     // Migration succeeded
 
-    const auto refreshedMetadata = _getCurrentMetadataAndCheckEpoch();
+    const auto refreshedMetadata = _getCurrentMetadataAndCheckForConflictingErrors();
     // Check if there are no chunks left on donor shard. Write an oplog event for change streams if
     // the last chunk migrated off the donor.
     if (!refreshedMetadata.getChunkManager()->getVersion(_args.getFromShard()).isSet()) {
@@ -678,8 +709,6 @@ void MigrationSourceManager::commitChunkMetadataOnConfig() {
 
 
     LOGV2(22018,
-          "Migration succeeded and updated collection placement version to "
-          "{updatedCollectionPlacementVersion}",
           "Migration succeeded and updated collection placement version",
           "updatedCollectionPlacementVersion"_attr = refreshedMetadata.getCollPlacementVersion(),
           "migrationId"_attr = _coordinator->getMigrationId());
@@ -688,7 +717,7 @@ void MigrationSourceManager::commitChunkMetadataOnConfig() {
     // out are no longer updatable.
     if (nss().isTimeseriesBucketsCollection()) {
         auto& bucketCatalog = timeseries::bucket_catalog::BucketCatalog::get(_opCtx);
-        clear(bucketCatalog, nss().getTimeseriesViewNamespace());
+        clear(bucketCatalog, _collectionUUID.get());
     }
 
     _coordinator->setMigrationDecision(DecisionEnum::kCommitted);
@@ -711,7 +740,7 @@ void MigrationSourceManager::commitChunkMetadataOnConfig() {
     ShardingLogging::get(_opCtx)->logChange(
         _opCtx,
         "moveChunk.commit",
-        nss().ns(),
+        nss(),
         BSON("min" << *_args.getMin() << "max" << *_args.getMax() << "from" << _args.getFromShard()
                    << "to" << _args.getToShard() << "counts" << *_recipientCloneCounts),
         ShardingCatalogClient::kMajorityWriteConcern);
@@ -719,13 +748,11 @@ void MigrationSourceManager::commitChunkMetadataOnConfig() {
     const ChunkRange range(*_args.getMin(), *_args.getMax());
 
     std::string orphanedRangeCleanUpErrMsg = str::stream()
-        << "Moved chunks successfully but failed to clean up " << nss() << " range "
-        << redact(range.toString()) << " due to: ";
+        << "Moved chunks successfully but failed to clean up " << nss().toStringForErrorMsg()
+        << " range " << redact(range.toString()) << " due to: ";
 
     if (_args.getWaitForDelete()) {
         LOGV2(22019,
-              "Waiting for migration cleanup after chunk commit for the namespace {namespace} "
-              "and range {range}",
               "Waiting for migration cleanup after chunk commit",
               logAttrs(nss()),
               "range"_attr = redact(range.toString()),
@@ -754,7 +781,7 @@ void MigrationSourceManager::_cleanupOnError() noexcept {
     ShardingLogging::get(_opCtx)->logChange(
         _opCtx,
         "moveChunk.error",
-        _args.getCommandParameter().ns(),
+        _args.getCommandParameter(),
         BSON("min" << *_args.getMin() << "max" << *_args.getMax() << "from" << _args.getFromShard()
                    << "to" << _args.getToShard()),
         ShardingCatalogClient::kMajorityWriteConcern);
@@ -770,10 +797,10 @@ SharedSemiFuture<void> MigrationSourceManager::abort() {
     return _completion.getFuture();
 }
 
-CollectionMetadata MigrationSourceManager::_getCurrentMetadataAndCheckEpoch() {
+CollectionMetadata MigrationSourceManager::_getCurrentMetadataAndCheckForConflictingErrors() {
     auto metadata = [&] {
         // TODO (SERVER-71444): Fix to be interruptible or document exception.
-        UninterruptibleLockGuard noInterrupt(_opCtx->lockState());  // NOLINT.
+        UninterruptibleLockGuard noInterrupt(_opCtx);  // NOLINT.
         AutoGetCollection autoColl(_opCtx, _args.getCommandParameter(), MODE_IS);
         const auto scopedCsr = CollectionShardingRuntime::assertCollectionLockedAndAcquireShared(
             _opCtx, _args.getCommandParameter());
@@ -784,16 +811,27 @@ CollectionMetadata MigrationSourceManager::_getCurrentMetadataAndCheckEpoch() {
                 optMetadata);
         return *optMetadata;
     }();
-
-    uassert(ErrorCodes::ConflictingOperationInProgress,
-            str::stream() << "The collection's epoch has changed since the migration began. "
-                             "Expected collection epoch: "
-                          << _collectionEpoch->toString() << ", but found: "
-                          << (metadata.isSharded()
-                                  ? metadata.getCollPlacementVersion().epoch().toString()
-                                  : "unsharded collection"),
-            metadata.isSharded() &&
-                metadata.getCollPlacementVersion().epoch() == *_collectionEpoch);
+    if (_collectionTimestamp) {
+        uassert(ErrorCodes::ConflictingOperationInProgress,
+                str::stream()
+                    << "The collection's timestamp has changed since the migration began. Expected "
+                       "timestamp: "
+                    << _collectionTimestamp->toStringPretty() << ", but found: "
+                    << (metadata.isSharded()
+                            ? metadata.getCollPlacementVersion().getTimestamp().toStringPretty()
+                            : "unsharded collection"),
+                metadata.isSharded() &&
+                    *_collectionTimestamp == metadata.getCollPlacementVersion().getTimestamp());
+    } else {
+        uassert(
+            ErrorCodes::ConflictingOperationInProgress,
+            str::stream()
+                << "The collection's epoch has changed since the migration began. Expected epoch: "
+                << _collectionEpoch->toString() << ", but found: "
+                << (metadata.isSharded() ? metadata.getCollPlacementVersion().toString()
+                                         : "unsharded collection"),
+            metadata.isSharded() && metadata.getCollPlacementVersion().epoch() == _collectionEpoch);
+    }
 
     return metadata;
 }
@@ -804,7 +842,7 @@ void MigrationSourceManager::_cleanup(bool completeMigration) noexcept {
     auto cloneDriver = [&]() {
         // Unregister from the collection's sharding state and exit the migration critical section.
         // TODO (SERVER-71444): Fix to be interruptible or document exception.
-        UninterruptibleLockGuard noInterrupt(_opCtx->lockState());  // NOLINT.
+        UninterruptibleLockGuard noInterrupt(_opCtx);  // NOLINT.
         AutoGetCollection autoColl(_opCtx, nss(), MODE_IX);
         auto scopedCsr =
             CollectionShardingRuntime::assertCollectionLockedAndAcquireExclusive(_opCtx, nss());
@@ -844,11 +882,9 @@ void MigrationSourceManager::_cleanup(bool completeMigration) noexcept {
                 _coordinator->setMigrationDecision(DecisionEnum::kAborted);
             }
 
-            auto newClient = _opCtx->getServiceContext()->makeClient("MigrationCoordinator");
-            {
-                stdx::lock_guard<Client> lk(*newClient.get());
-                newClient->setSystemOperationKillableByStepdown(lk);
-            }
+            auto newClient = _opCtx->getServiceContext()
+                                 ->getService(ClusterRole::ShardServer)
+                                 ->makeClient("MigrationCoordinator");
             AlternativeClientRegion acr(newClient);
             auto newOpCtxPtr = cc().makeOperationContext();
             auto newOpCtx = newOpCtxPtr.get();
@@ -856,24 +892,15 @@ void MigrationSourceManager::_cleanup(bool completeMigration) noexcept {
             if (_state >= kCriticalSection && _state <= kCommittingOnConfig) {
                 _stats.totalCriticalSectionTimeMillis.addAndFetch(_cloneAndCommitTimer.millis());
 
-                // NOTE: The order of the operations below is important and the comments explain the
-                // reasoning behind it.
-                //
                 // Wait for the updates to the cache of the routing table to be fully written to
-                // disk before clearing the 'minOpTime recovery' document. This way, we ensure that
-                // all nodes from a shard, which donated a chunk will always be at the placement
-                // version of the last migration it performed.
+                // disk. This way, we ensure that all nodes from a shard which donated a chunk will
+                // always be at the placement version of the last migration it performed.
                 //
                 // If the metadata is not persisted before clearing the 'inMigration' flag below, it
                 // is possible that the persisted metadata is rolled back after step down, but the
                 // write which cleared the 'inMigration' flag is not, a secondary node will report
                 // itself at an older placement version.
                 CatalogCacheLoader::get(newOpCtx).waitForCollectionFlush(newOpCtx, nss());
-
-                // Clear the 'minOpTime recovery' document so that the next time a node from this
-                // shard becomes a primary, it won't have to recover the config server optime.
-                // TODO (SERVER-60110): Remove once 7.0 becomes last LTS.
-                ShardingStateRecovery_DEPRECATED::endMetadataOp(newOpCtx);
             }
             if (completeMigration) {
                 // This can be called on an exception path after the OperationContext has been
@@ -886,8 +913,6 @@ void MigrationSourceManager::_cleanup(bool completeMigration) noexcept {
         _state = kDone;
     } catch (const DBException& ex) {
         LOGV2_WARNING(5089001,
-                      "Failed to complete the migration {migrationId} with "
-                      "{chunkMigrationRequestParameters} due to: {error}",
                       "Failed to complete the migration",
                       "chunkMigrationRequestParameters"_attr = redact(_args.toBSON({})),
                       "error"_attr = redact(ex),
@@ -895,7 +920,7 @@ void MigrationSourceManager::_cleanup(bool completeMigration) noexcept {
         // Something went really wrong when completing the migration just unset the metadata and let
         // the next op to recover.
         // TODO (SERVER-71444): Fix to be interruptible or document exception.
-        UninterruptibleLockGuard noInterrupt(_opCtx->lockState());  // NOLINT.
+        UninterruptibleLockGuard noInterrupt(_opCtx);  // NOLINT.
         AutoGetCollection autoColl(_opCtx, nss(), MODE_IX);
         CollectionShardingRuntime::assertCollectionLockedAndAcquireExclusive(_opCtx, nss())
             ->clearFilteringMetadata(_opCtx);
@@ -938,7 +963,7 @@ MigrationSourceManager::ScopedRegisterer::ScopedRegisterer(MigrationSourceManage
 
 MigrationSourceManager::ScopedRegisterer::~ScopedRegisterer() {
     // TODO (SERVER-71444): Fix to be interruptible or document exception.
-    UninterruptibleLockGuard noInterrupt(_msm->_opCtx->lockState());  // NOLINT.
+    UninterruptibleLockGuard noInterrupt(_msm->_opCtx);  // NOLINT.
     AutoGetCollection autoColl(_msm->_opCtx, _msm->_args.getCommandParameter(), MODE_IX);
     auto scopedCsr = CollectionShardingRuntime::assertCollectionLockedAndAcquireExclusive(
         _msm->_opCtx, _msm->_args.getCommandParameter());
