@@ -30,28 +30,97 @@
 #include "mongo/db/s/move_primary_coordinator.h"
 
 #include <algorithm>
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr.hpp>
+#include <fmt/format.h>
+#include <iterator>
+#include <list>
+#include <tuple>
+#include <utility>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status_with.h"
+#include "mongo/bson/bson_field.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsontypes.h"
 #include "mongo/bson/util/bson_extract.h"
-#include "mongo/client/connpool.h"
+#include "mongo/client/connection_string.h"
+#include "mongo/client/read_preference.h"
 #include "mongo/db/catalog/drop_collection.h"
 #include "mongo/db/catalog_raii.h"
+#include "mongo/db/client.h"
+#include "mongo/db/commands.h"
 #include "mongo/db/commands/list_collections_filter.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
+#include "mongo/db/dbdirectclient.h"
 #include "mongo/db/repl/change_stream_oplog_notification.h"
+#include "mongo/db/repl/read_concern_level.h"
+#include "mongo/db/s/collection_sharding_runtime.h"
 #include "mongo/db/s/database_sharding_state.h"
+#include "mongo/db/s/forwardable_operation_metadata.h"
 #include "mongo/db/s/sharding_ddl_util.h"
 #include "mongo/db/s/sharding_logging.h"
 #include "mongo/db/s/sharding_recovery_service.h"
-#include "mongo/db/s/sharding_state.h"
+#include "mongo/db/server_options.h"
+#include "mongo/db/session/logical_session_id_gen.h"
+#include "mongo/db/shard_id.h"
+#include "mongo/db/transaction_resources.h"
 #include "mongo/db/vector_clock_mutable.h"
 #include "mongo/db/write_block_bypass.h"
+#include "mongo/idl/idl_parser.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/redaction.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/rpc/get_status_from_command_result.h"
+#include "mongo/s/catalog/sharding_catalog_client.h"
+#include "mongo/s/catalog/type_database_gen.h"
+#include "mongo/s/catalog/type_shard.h"
+#include "mongo/s/client/shard.h"
+#include "mongo/s/client/shard_registry.h"
 #include "mongo/s/grid.h"
 #include "mongo/s/request_types/move_primary_gen.h"
+#include "mongo/s/resharding/resharding_feature_flag_gen.h"
+#include "mongo/s/sharding_state.h"
+#include "mongo/util/database_name_util.h"
+#include "mongo/util/duration.h"
+#include "mongo/util/fail_point.h"
+#include "mongo/util/future_impl.h"
+#include "mongo/util/namespace_string_util.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
 namespace mongo {
+using namespace fmt::literals;
+
+namespace {
 
 MONGO_FAIL_POINT_DEFINE(hangBeforeCloningData);
+MONGO_FAIL_POINT_DEFINE(movePrimaryFailIfNeedToCloneMovableCollections);
+
+/**
+ * Returns true if this unsharded collection can be moved by a moveCollection command.
+ */
+bool isMovableUnshardedCollection(const NamespaceString& nss, bool timeseriesReshardingSupported) {
+    if (nss.isFLE2StateCollection()) {
+        // TODO (SERVER-83713): Reconsider isFLE2StateCollection check.
+        return false;
+    }
+
+    if (nss.isTimeseriesBucketsCollection()) {
+        return timeseriesReshardingSupported;
+    }
+
+    if (nss.isNamespaceAlwaysUntracked()) {
+        return false;
+    }
+
+    return true;
+}
+
+}  // namespace
 
 MovePrimaryCoordinator::MovePrimaryCoordinator(ShardingDDLCoordinatorService* service,
                                                const BSONObj& initialState)
@@ -60,7 +129,8 @@ MovePrimaryCoordinator::MovePrimaryCoordinator(ShardingDDLCoordinatorService* se
       _csReason([&] {
           BSONObjBuilder builder;
           builder.append("command", "movePrimary");
-          builder.append("db", _dbName.toString());
+          builder.append(
+              "db", DatabaseNameUtil::serialize(_dbName, SerializationContext::stateDefault()));
           builder.append("to", _doc.getToShardId());
           return builder.obj();
       }()) {}
@@ -69,6 +139,11 @@ bool MovePrimaryCoordinator::canAlwaysStartWhenUserWritesAreDisabled() const {
     return true;
 }
 
+logv2::DynamicAttributes MovePrimaryCoordinator::getCoordinatorLogAttrs() const {
+    return logv2::DynamicAttributes{getBasicCoordinatorAttrs(),
+                                    "toShardId"_attr = _doc.getToShardId()};
+}
+
 StringData MovePrimaryCoordinator::serializePhase(const Phase& phase) const {
     return MovePrimaryCoordinatorPhase_serializer(phase);
 }
@@ -170,7 +245,7 @@ ExecutorFuture<void> MovePrimaryCoordinator::runMovePrimaryWorkflow(
                     uasserted(
                         7120202,
                         "movePrimary operation on database {} failed cloning data to recipient {}"_format(
-                            _dbName.toString(), toShardId.toString()));
+                            _dbName.toStringForErrorMsg(), toShardId.toString()));
                 }
 
                 LOGV2(7120201,
@@ -182,7 +257,7 @@ ExecutorFuture<void> MovePrimaryCoordinator::runMovePrimaryWorkflow(
 
                 ScopeGuard unblockWritesLegacyOnExit([&] {
                     // TODO (SERVER-71444): Fix to be interruptible or document exception.
-                    UninterruptibleLockGuard noInterrupt(opCtx->lockState());  // NOLINT
+                    UninterruptibleLockGuard noInterrupt(opCtx);  // NOLINT
                     unblockWritesLegacy(opCtx);
                 });
 
@@ -195,8 +270,9 @@ ExecutorFuture<void> MovePrimaryCoordinator::runMovePrimaryWorkflow(
 
                 cloneData(opCtx);
 
-                // TODO (SERVER-71566): Temporary solution to cover the case of stepping down before
-                // actually entering the `kCatchup` phase.
+                // Hack to cover the case of stepping down before actually entering the `kCatchup`
+                // phase. Once the time required by the `kClone` phase will be reduced, this
+                // synchronization mechanism can be replaced using a critical section.
                 blockWrites(opCtx);
             }))
         .then(_buildPhaseHandler(Phase::kCatchup,
@@ -303,7 +379,7 @@ ExecutorFuture<void> MovePrimaryCoordinator::runMovePrimaryWorkflow(
 }
 
 void MovePrimaryCoordinator::cloneData(OperationContext* opCtx) {
-    const auto& collectionsToClone = getUnshardedCollections(opCtx);
+    const auto& collectionsToClone = getCollectionsToClone(opCtx);
     assertNoOrphanedDataOnRecipient(opCtx, collectionsToClone);
 
     _doc.setCollectionsToClone(collectionsToClone);
@@ -329,6 +405,18 @@ ExecutorFuture<void> MovePrimaryCoordinator::_cleanupOnAbort(
             const auto& failedPhase = _doc.getPhase();
             const auto& toShardId = _doc.getToShardId();
 
+            unblockReadsAndWrites(opCtx);
+            try {
+                // Even if the error is `ShardNotFound`, the recipient may still be in draining
+                // mode, so try to exit the critical section anyway.
+                exitCriticalSectionOnRecipient(opCtx);
+            } catch (const ExceptionFor<ErrorCodes::ShardNotFound>&) {
+                LOGV2_INFO(7392902,
+                           "Failed to exit critical section on recipient as it has been removed",
+                           logAttrs(_dbName),
+                           "to"_attr = toShardId);
+            }
+
             if (failedPhase <= Phase::kCommit) {
                 // A non-retryable error occurred before the new primary shard was actually
                 // committed, so any cloned data on the recipient must be dropped.
@@ -345,18 +433,6 @@ ExecutorFuture<void> MovePrimaryCoordinator::_cleanupOnAbort(
                 }
             }
 
-            unblockReadsAndWrites(opCtx);
-            try {
-                // Even if the error is `ShardNotFound`, the recipient may still be in draining
-                // mode, so try to exit the critical section anyway.
-                exitCriticalSectionOnRecipient(opCtx);
-            } catch (const ExceptionFor<ErrorCodes::ShardNotFound>&) {
-                LOGV2_INFO(7392902,
-                           "Failed to exit critical section on recipient as it has been removed",
-                           logAttrs(_dbName),
-                           "to"_attr = toShardId);
-            }
-
             LOGV2_ERROR(7392903,
                         "Failed movePrimary operation",
                         logAttrs(_dbName),
@@ -378,10 +454,10 @@ void MovePrimaryCoordinator::logChange(OperationContext* opCtx,
         details.append("error", status.toString());
     }
     ShardingLogging::get(opCtx)->logChange(
-        opCtx, "movePrimary.{}"_format(what), _dbName.toString(), details.obj());
+        opCtx, "movePrimary.{}"_format(what), NamespaceString(_dbName), details.obj());
 }
 
-std::vector<NamespaceString> MovePrimaryCoordinator::getUnshardedCollections(
+std::vector<NamespaceString> MovePrimaryCoordinator::getCollectionsToClone(
     OperationContext* opCtx) const {
     const auto allCollections = [&] {
         DBDirectClient dbClient(opCtx);
@@ -393,8 +469,8 @@ std::vector<NamespaceString> MovePrimaryCoordinator::getUnshardedCollections(
             std::string collName;
             uassertStatusOK(bsonExtractStringField(collInfo, "name", &collName));
 
-            const NamespaceString nss(
-                NamespaceStringUtil::parseNamespaceFromDoc(_dbName, collName));
+            const NamespaceString nss(NamespaceStringUtil::deserialize(_dbName, collName));
+
             if (!nss.isSystem() || nss.isLegalClientSystemNS()) {
                 colls.push_back(nss);
             }
@@ -404,22 +480,58 @@ std::vector<NamespaceString> MovePrimaryCoordinator::getUnshardedCollections(
         return colls;
     }();
 
-    const auto shardedCollections = [&] {
-        auto colls = Grid::get(opCtx)->catalogClient()->getAllShardedCollectionsForDb(
-            opCtx, _dbName.toString(), repl::ReadConcernLevel::kMajorityReadConcern);
+    const auto collectionsToIgnore = [&] {
+        auto catalogClient = Grid::get(opCtx)->catalogClient();
+        auto colls = catalogClient->getShardedCollectionNamespacesForDb(
+            opCtx, _dbName, repl::ReadConcernLevel::kMajorityReadConcern, {});
+        auto unshardedTrackedColls = catalogClient->getUnsplittableCollectionNamespacesForDb(
+            opCtx, _dbName, repl::ReadConcernLevel::kMajorityReadConcern, {});
+
+        std::move(
+            unshardedTrackedColls.begin(), unshardedTrackedColls.end(), std::back_inserter(colls));
 
         std::sort(colls.begin(), colls.end());
+
         return colls;
     }();
 
-    std::vector<NamespaceString> unshardedCollections;
+    std::vector<NamespaceString> collectionsToClone;
     std::set_difference(allCollections.cbegin(),
                         allCollections.cend(),
-                        shardedCollections.cbegin(),
-                        shardedCollections.cend(),
-                        std::back_inserter(unshardedCollections));
+                        collectionsToIgnore.cbegin(),
+                        collectionsToIgnore.cend(),
+                        std::back_inserter(collectionsToClone));
 
-    return unshardedCollections;
+    const auto fcvSnapshot = serverGlobalParams.featureCompatibility.acquireFCVSnapshot();
+    bool timeseriesReshardingSupported = fcvSnapshot.isVersionInitialized() &&
+        resharding::gFeatureFlagReshardingForTimeseries.isEnabled(fcvSnapshot);
+
+    for (const auto& nss : collectionsToClone) {
+        movePrimaryFailIfNeedToCloneMovableCollections.executeIf(
+            [&](const BSONObj& data) {
+                if (isMovableUnshardedCollection(nss, timeseriesReshardingSupported)) {
+                    AutoGetCollection autoColl(opCtx, nss, MODE_IS);
+                    uassert(9046501,
+                            str::stream() << "Found a user collection to clone: "
+                                          << nss.toStringForErrorMsg(),
+                            !autoColl);
+                }
+            },
+            [&](const BSONObj& data) {
+                if (!data.hasField("comment")) {
+                    return true;
+                }
+                // If this failpoint is configured with a "comment", only fail the command if
+                // its "comment" matches the failpoint's "comment".
+                if (!opCtx->getComment()) {
+                    return false;
+                }
+                return opCtx->getComment()->checkAndGetStringData() ==
+                    data.getStringField("comment");
+            });
+    }
+
+    return collectionsToClone;
 }
 
 void MovePrimaryCoordinator::assertNoOrphanedDataOnRecipient(
@@ -440,7 +552,7 @@ void MovePrimaryCoordinator::assertNoOrphanedDataOnRecipient(
         const auto listResponse = uassertStatusOK(
             toShard->runExhaustiveCursorCommand(opCtx,
                                                 ReadPreferenceSetting(ReadPreference::PrimaryOnly),
-                                                _dbName.toString(),
+                                                _dbName,
                                                 listCommand,
                                                 Milliseconds(-1)));
 
@@ -448,7 +560,7 @@ void MovePrimaryCoordinator::assertNoOrphanedDataOnRecipient(
         for (const auto& bsonColl : listResponse.docs) {
             std::string collName;
             uassertStatusOK(bsonExtractStringField(bsonColl, "name", &collName));
-            colls.push_back(NamespaceStringUtil::parseNamespaceFromResponse(_dbName, collName));
+            colls.push_back(NamespaceStringUtil::deserialize(_dbName, collName));
         }
 
         std::sort(colls.begin(), colls.end());
@@ -457,7 +569,7 @@ void MovePrimaryCoordinator::assertNoOrphanedDataOnRecipient(
 
     for (const auto& nss : collectionsToClone) {
         uassert(ErrorCodes::NamespaceExists,
-                "Found orphaned collection {} on recipient {}"_format(nss.toString(),
+                "Found orphaned collection {} on recipient {}"_format(nss.toStringForErrorMsg(),
                                                                       toShardId.toString()),
                 !std::binary_search(allCollections.cbegin(), allCollections.cend(), nss));
     };
@@ -476,7 +588,9 @@ std::vector<NamespaceString> MovePrimaryCoordinator::cloneDataToRecipient(Operat
 
     auto cloneCommand = [&](boost::optional<OperationSessionInfo> osi) {
         BSONObjBuilder commandBuilder;
-        commandBuilder.append("_shardsvrCloneCatalogData", _dbName.toString());
+        commandBuilder.append(
+            "_shardsvrCloneCatalogData",
+            DatabaseNameUtil::serialize(_dbName, SerializationContext::stateDefault()));
         commandBuilder.append("from", fromShard->getConnString().toString());
         if (osi.is_initialized()) {
             commandBuilder.appendElements(osi->toBSON());
@@ -488,7 +602,7 @@ std::vector<NamespaceString> MovePrimaryCoordinator::cloneDataToRecipient(Operat
         const auto cloneResponse =
             toShard->runCommand(opCtx,
                                 ReadPreferenceSetting(ReadPreference::PrimaryOnly),
-                                DatabaseName::kAdmin.db().toString(),
+                                DatabaseName::kAdmin,
                                 command,
                                 Shard::RetryPolicy::kNoRetry);
 
@@ -500,7 +614,8 @@ std::vector<NamespaceString> MovePrimaryCoordinator::cloneDataToRecipient(Operat
         std::vector<NamespaceString> colls;
         for (const auto& bsonElem : cloneResponse.getValue().response["clonedColls"].Obj()) {
             if (bsonElem.type() == String) {
-                colls.push_back(NamespaceString(bsonElem.String()));
+                colls.push_back(NamespaceStringUtil::deserialize(
+                    boost::none, bsonElem.String(), SerializationContext::stateDefault()));
             }
         }
 
@@ -543,33 +658,34 @@ void MovePrimaryCoordinator::commitMetadataToConfig(
     const auto commitResponse =
         config->runCommandWithFixedRetryAttempts(opCtx,
                                                  ReadPreferenceSetting(ReadPreference::PrimaryOnly),
-                                                 DatabaseName::kAdmin.db().toString(),
+                                                 DatabaseName::kAdmin,
                                                  commitCommand,
                                                  Shard::RetryPolicy::kIdempotent);
 
     uassertStatusOKWithContext(
         Shard::CommandResponse::getEffectiveStatus(commitResponse),
         "movePrimary operation on database {} failed to commit metadata changes"_format(
-            _dbName.toString()));
+            _dbName.toStringForErrorMsg()));
 }
 
 void MovePrimaryCoordinator::assertChangedMetadataOnConfig(
     OperationContext* opCtx, const DatabaseVersion& preCommitDbVersion) const {
     const auto postCommitDbType = [&]() {
         const auto config = Grid::get(opCtx)->shardRegistry()->getConfigShard();
-        auto findResponse = uassertStatusOK(
-            config->exhaustiveFindOnConfig(opCtx,
-                                           ReadPreferenceSetting{ReadPreference::PrimaryOnly},
-                                           repl::ReadConcernLevel::kMajorityReadConcern,
-                                           NamespaceString::kConfigDatabasesNamespace,
-                                           BSON(DatabaseType::kNameFieldName << _dbName.toString()),
-                                           BSONObj(),
-                                           1));
+        auto findResponse = uassertStatusOK(config->exhaustiveFindOnConfig(
+            opCtx,
+            ReadPreferenceSetting{ReadPreference::PrimaryOnly},
+            repl::ReadConcernLevel::kMajorityReadConcern,
+            NamespaceString::kConfigDatabasesNamespace,
+            BSON(DatabaseType::kDbNameFieldName
+                 << DatabaseNameUtil::serialize(_dbName, SerializationContext::stateDefault())),
+            BSONObj(),
+            1));
 
         const auto databases = std::move(findResponse.docs);
         uassert(ErrorCodes::IncompatibleShardingMetadata,
                 "Tried to find version for database {}, but found no databases"_format(
-                    _dbName.toString()),
+                    _dbName.toStringForErrorMsg()),
                 !databases.empty());
 
         return DatabaseType::parse(IDLParserContext("DatabaseType"), databases.front());
@@ -593,21 +709,27 @@ void MovePrimaryCoordinator::dropStaleDataOnDonor(OperationContext* opCtx) const
     WriteBlockBypass::get(opCtx).set(true);
 
     invariant(_doc.getCollectionsToClone());
-    for (const auto& nss : *_doc.getCollectionsToClone()) {
+
+    const auto dropColl = [&](const NamespaceString& nssToDrop) {
         DropReply unusedDropReply;
         try {
             uassertStatusOK(
                 dropCollection(opCtx,
-                               nss,
+                               nssToDrop,
                                &unusedDropReply,
                                DropCollectionSystemCollectionMode::kAllowSystemCollectionDrops,
-                               false /* fromMigrate */));
+                               true /* fromMigrate */));
         } catch (const DBException& e) {
             LOGV2_WARNING(7120210,
                           "Failed to drop stale collection on donor",
-                          logAttrs(nss),
-                          "error"_attr = redact(e));
+                          logv2::DynamicAttributes{getCoordinatorLogAttrs(),
+                                                   logAttrs(nssToDrop),
+                                                   "error"_attr = redact(e)});
         }
+    };
+
+    for (const auto& nss : *_doc.getCollectionsToClone()) {
+        dropColl(nss);
     }
 }
 
@@ -628,7 +750,7 @@ void MovePrimaryCoordinator::dropOrphanedDataOnRecipient(
             {_doc.getToShardId()},
             **executor,
             getNewSession(opCtx),
-            false /* fromMigrate */,
+            true /* fromMigrate */,
             true /* dropSystemCollections */);
     }
 }
@@ -665,6 +787,7 @@ void MovePrimaryCoordinator::unblockReadsAndWrites(OperationContext* opCtx) cons
         NamespaceString(_dbName),
         _csReason,
         ShardingCatalogClient::kLocalWriteConcern,
+        ShardingRecoveryService::NoCustomAction(),
         false /*throwIfReasonDiffers*/);
 }
 
@@ -687,7 +810,7 @@ void MovePrimaryCoordinator::enterCriticalSectionOnRecipient(OperationContext* o
         return toShard->runCommandWithFixedRetryAttempts(
             opCtx,
             ReadPreferenceSetting(ReadPreference::PrimaryOnly),
-            DatabaseName::kAdmin.toString(),
+            DatabaseName::kAdmin,
             enterCriticalSectionCommand,
             Shard::RetryPolicy::kIdempotent);
     }();
@@ -695,7 +818,7 @@ void MovePrimaryCoordinator::enterCriticalSectionOnRecipient(OperationContext* o
     uassertStatusOKWithContext(
         Shard::CommandResponse::getEffectiveStatus(enterCriticalSectionResponse),
         "movePrimary operation on database {} failed to block read/write operations on recipient {}"_format(
-            _dbName.toString(), toShardId.toString()));
+            _dbName.toStringForErrorMsg(), toShardId.toString()));
 }
 
 void MovePrimaryCoordinator::exitCriticalSectionOnRecipient(OperationContext* opCtx) {
@@ -717,7 +840,7 @@ void MovePrimaryCoordinator::exitCriticalSectionOnRecipient(OperationContext* op
         return toShard->runCommandWithFixedRetryAttempts(
             opCtx,
             ReadPreferenceSetting(ReadPreference::PrimaryOnly),
-            DatabaseName::kAdmin.toString(),
+            DatabaseName::kAdmin,
             exitCriticalSectionCommand,
             Shard::RetryPolicy::kIdempotent);
     }();
@@ -725,7 +848,7 @@ void MovePrimaryCoordinator::exitCriticalSectionOnRecipient(OperationContext* op
     uassertStatusOKWithContext(
         Shard::CommandResponse::getEffectiveStatus(exitCriticalSectionResponse),
         "movePrimary operation on database {} failed to unblock read/write operations on recipient {}"_format(
-            _dbName.toString(), toShardId.toString()));
+            _dbName.toStringForErrorMsg(), toShardId.toString()));
 }
 
 }  // namespace mongo
