@@ -28,14 +28,31 @@
  */
 
 #include "mongo/db/s/operation_sharding_state.h"
+
+#include <boost/none.hpp>
+#include <memory>
+
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/string_data.h"
+#include "mongo/bson/oid.h"
+#include "mongo/bson/timestamp.h"
 #include "mongo/db/s/shard_server_test_fixture.h"
+#include "mongo/s/chunk_version.h"
+#include "mongo/s/index_version.h"
 #include "mongo/s/shard_version_factory.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/uuid.h"
 
 namespace mongo {
 namespace {
 
-const NamespaceString kNss("TestDB", "TestColl");
-const NamespaceString kAnotherNss("TestDB", "AnotherColl");
+const NamespaceString kNss = NamespaceString::createNamespaceString_forTest("TestDB", "TestColl");
+const NamespaceString kAnotherNss =
+    NamespaceString::createNamespaceString_forTest("TestDB", "AnotherColl");
 
 using OperationShardingStateTest = ShardServerTestFixture;
 
@@ -44,7 +61,7 @@ TEST_F(OperationShardingStateTest, ScopedSetShardRoleDbVersion) {
     ScopedSetShardRole scopedSetShardRole(operationContext(), kNss, boost::none, dbv);
 
     auto& oss = OperationShardingState::get(operationContext());
-    ASSERT_EQ(dbv, *oss.getDbVersion(kNss.db()));
+    ASSERT_EQ(dbv, *oss.getDbVersion(kNss.dbName()));
 }
 
 TEST_F(OperationShardingStateTest, ScopedSetShardRoleShardVersion) {
@@ -100,7 +117,7 @@ TEST_F(OperationShardingStateTest, ScopedSetShardRoleIgnoresFixedDbVersion) {
     ScopedSetShardRole scopedSetShardRole(operationContext(), kNss, boost::none, dbv);
 
     auto& oss = OperationShardingState::get(operationContext());
-    ASSERT_FALSE(oss.getDbVersion(kNss.db()));
+    ASSERT_FALSE(oss.getDbVersion(kNss.dbName()));
 }
 
 TEST_F(OperationShardingStateTest, ScopedSetShardRoleAllowedShardVersionsWithFixedDbVersion) {
@@ -132,31 +149,5 @@ TEST_F(OperationShardingStateTest, ScopedSetShardRoleAllowedShardVersionsWithFix
     }
 }
 
-TEST_F(OperationShardingStateTest, ScopeSetShardRoleHasStrongExceptionGuarantee) {
-    CollectionGeneration gen(OID::gen(), Timestamp(1, 0));
-    ShardVersion shardVersion =
-        ShardVersionFactory::make({gen, {1, 0}}, boost::optional<CollectionIndexes>(boost::none));
-    auto uuid = UUID::gen();
-    DatabaseVersion dbv1{uuid, Timestamp(1, 0)};
-    DatabaseVersion dbv2{uuid, Timestamp(2, 0)};
-
-    auto& oss = OperationShardingState::get(operationContext());
-    {
-        ScopedSetShardRole scopedSetShardRole1(operationContext(), kNss, shardVersion, dbv1);
-        ASSERT_THROWS_CODE(
-            [&] {
-                ScopedSetShardRole scopedSetShardRole2(
-                    operationContext(), kNss, shardVersion, dbv2);
-            }(),
-            DBException,
-            640571);
-        ASSERT_EQ(shardVersion, *oss.getShardVersion(kNss));
-        ASSERT_EQ(dbv1, *oss.getDbVersion(kNss.db()));
-    }
-
-    ASSERT_EQ(boost::none, oss.getShardVersion(kNss));
-    ASSERT_EQ(boost::none, oss.getDbVersion(kNss.db()));
-}
-
 }  // namespace
 }  // namespace mongo
