@@ -29,17 +29,32 @@
 
 #include "mongo/db/s/query_analysis_coordinator.h"
 
+#include <absl/container/flat_hash_map.h>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <memory>
+#include <utility>
+
+#include <absl/container/node_hash_map.h>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/checked_cast.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/db/op_observer/op_observer.h"
 #include "mongo/db/op_observer/op_observer_registry.h"
 #include "mongo/db/s/config/config_server_test_fixture.h"
-#include "mongo/db/s/query_analysis_op_observer.h"
+#include "mongo/db/s/query_analysis_op_observer_configsvr.h"
+#include "mongo/db/service_context_d_test_fixture.h"
 #include "mongo/idl/server_parameter_test_util.h"
-#include "mongo/logv2/log.h"
 #include "mongo/s/analyze_shard_key_documents_gen.h"
 #include "mongo/s/catalog/type_mongos.h"
-#include "mongo/unittest/death_test.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
 #include "mongo/util/assert_util.h"
 #include "mongo/util/clock_source_mock.h"
+#include "mongo/util/duration.h"
+#include "mongo/util/uuid.h"
 #include "mongo/util/version.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
@@ -56,7 +71,7 @@ public:
         ConfigServerTestFixture::setupOpObservers();
         auto opObserverRegistry =
             checked_cast<OpObserverRegistry*>(getServiceContext()->getOpObserver());
-        opObserverRegistry->addObserver(std::make_unique<QueryAnalysisOpObserver>());
+        opObserverRegistry->addObserver(std::make_unique<QueryAnalysisOpObserverConfigSvr>());
     }
 
 protected:
@@ -173,7 +188,6 @@ protected:
 private:
     const std::shared_ptr<ClockSourceMock> _mockClock = std::make_shared<ClockSourceMock>();
 
-    RAIIServerParameterControllerForTest _featureFlagController{"featureFlagAnalyzeShardKey", true};
     RAIIServerParameterControllerForTest _inactiveThresholdController{
         "queryAnalysisSamplerInActiveThresholdSecs", inActiveThresholdSecs};
 };
@@ -232,12 +246,12 @@ TEST_F(QueryAnalysisCoordinatorTest, UpdateConfigurationsSameCollectionUUid) {
 
     auto analyzerDocPostUpdate =
         makeConfigQueryAnalyzersDocument(nss0, collUuid0, QueryAnalyzerModeEnum::kFull, 1.5, now());
-    uassertStatusOK(
-        updateToConfigCollection(operationContext(),
-                                 NamespaceString::kConfigQueryAnalyzersNamespace,
-                                 BSON(QueryAnalyzerDocument::kNsFieldName << nss0.toString()),
-                                 analyzerDocPostUpdate.toBSON(),
-                                 false /* upsert */));
+    uassertStatusOK(updateToConfigCollection(
+        operationContext(),
+        NamespaceString::kConfigQueryAnalyzersNamespace,
+        BSON(QueryAnalyzerDocument::kNsFieldName << nss0.toString_forTest()),
+        analyzerDocPostUpdate.toBSON(),
+        false /* upsert */));
 
     // The update should cause the configuration to have the new sample rate.
     configurations = coordinator->getConfigurationsForTest();
@@ -266,12 +280,12 @@ TEST_F(QueryAnalysisCoordinatorTest, UpdateConfigurationDifferentCollectionUUid)
 
     auto analyzerDocPostUpdate = makeConfigQueryAnalyzersDocument(
         nss0, UUID::gen(), QueryAnalyzerModeEnum::kFull, 1.5, now());
-    uassertStatusOK(
-        updateToConfigCollection(operationContext(),
-                                 NamespaceString::kConfigQueryAnalyzersNamespace,
-                                 BSON(QueryAnalyzerDocument::kNsFieldName << nss0.toString()),
-                                 analyzerDocPostUpdate.toBSON(),
-                                 false /* upsert */));
+    uassertStatusOK(updateToConfigCollection(
+        operationContext(),
+        NamespaceString::kConfigQueryAnalyzersNamespace,
+        BSON(QueryAnalyzerDocument::kNsFieldName << nss0.toString_forTest()),
+        analyzerDocPostUpdate.toBSON(),
+        false /* upsert */));
 
     // The update should cause the configuration to have the new collection uuid, sample rate and
     // start time.
@@ -299,12 +313,12 @@ TEST_F(QueryAnalysisCoordinatorTest, RemoveOrCreateConfigurationsOnModeUpdate) {
 
     auto analyzerDocPostUpdate0 =
         makeConfigQueryAnalyzersDocument(nss0, collUuid0, QueryAnalyzerModeEnum::kOff);
-    uassertStatusOK(
-        updateToConfigCollection(operationContext(),
-                                 NamespaceString::kConfigQueryAnalyzersNamespace,
-                                 BSON(QueryAnalyzerDocument::kNsFieldName << nss0.toString()),
-                                 analyzerDocPostUpdate0.toBSON(),
-                                 false /* upsert */));
+    uassertStatusOK(updateToConfigCollection(
+        operationContext(),
+        NamespaceString::kConfigQueryAnalyzersNamespace,
+        BSON(QueryAnalyzerDocument::kNsFieldName << nss0.toString_forTest()),
+        analyzerDocPostUpdate0.toBSON(),
+        false /* upsert */));
 
     // The update to mode "off" should cause the configuration to get removed.
     configurations = coordinator->getConfigurationsForTest();
@@ -312,12 +326,12 @@ TEST_F(QueryAnalysisCoordinatorTest, RemoveOrCreateConfigurationsOnModeUpdate) {
 
     auto analyzerDocPostUpdate1 =
         makeConfigQueryAnalyzersDocument(nss0, collUuid0, QueryAnalyzerModeEnum::kFull, 15);
-    uassertStatusOK(
-        updateToConfigCollection(operationContext(),
-                                 NamespaceString::kConfigQueryAnalyzersNamespace,
-                                 BSON(QueryAnalyzerDocument::kNsFieldName << nss0.toString()),
-                                 analyzerDocPostUpdate1.toBSON(),
-                                 false /* upsert */));
+    uassertStatusOK(updateToConfigCollection(
+        operationContext(),
+        NamespaceString::kConfigQueryAnalyzersNamespace,
+        BSON(QueryAnalyzerDocument::kNsFieldName << nss0.toString_forTest()),
+        analyzerDocPostUpdate1.toBSON(),
+        false /* upsert */));
 
     // The update to mode "on" should cause the configuration to get recreated.
     configurations = coordinator->getConfigurationsForTest();
