@@ -29,18 +29,50 @@
 
 #include "mongo/db/s/query_analysis_writer.h"
 
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+// IWYU pragma: no_include "cxxabi.h"
+#include <cstdint>
+#include <future>
+#include <new>
+#include <set>
+#include <system_error>
+#include <tuple>
+#include <type_traits>
+#include <utility>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/bson/unordered_fields_bsonobj_comparator.h"
-#include "mongo/db/db_raii.h"
+#include "mongo/bson/util/builder.h"
+#include "mongo/client/dbclient_cursor.h"
+#include "mongo/crypto/fle_field_schema_gen.h"
+#include "mongo/db/catalog/collection_catalog.h"
+#include "mongo/db/client.h"
 #include "mongo/db/dbdirectclient.h"
+#include "mongo/db/ops/write_ops_parsers.h"
+#include "mongo/db/query/find_command.h"
 #include "mongo/db/s/shard_server_test_fixture.h"
 #include "mongo/db/update/document_diff_calculator.h"
+#include "mongo/idl/idl_parser.h"
 #include "mongo/idl/server_parameter_test_util.h"
-#include "mongo/logv2/log.h"
 #include "mongo/platform/random.h"
 #include "mongo/s/analyze_shard_key_documents_gen.h"
 #include "mongo/s/query_analysis_sample_tracker.h"
+#include "mongo/stdx/future.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/bson_test_util.h"
 #include "mongo/unittest/death_test.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/fail_point.h"
+#include "mongo/util/static_immortal.h"
+#include "mongo/util/synchronized_value.h"
+#include "mongo/util/time_support.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
 
@@ -170,7 +202,7 @@ void assertBsonObjEqualUnordered(const BSONObj& lhs, const BSONObj& rhs) {
 
 struct QueryAnalysisWriterTest : public ShardServerTestFixture {
 public:
-    void setUp() {
+    void setUp() override {
         ShardServerTestFixture::setUp();
         QueryAnalysisWriter::get(operationContext())->onStartup(operationContext());
 
@@ -186,7 +218,7 @@ public:
         tracker.refreshConfigurations({configuration0, configuration1});
     }
 
-    void tearDown() {
+    void tearDown() override {
         QueryAnalysisWriter::get(operationContext())->onShutdown();
         ShardServerTestFixture::tearDown();
     }
@@ -265,6 +297,7 @@ protected:
                 write_ops::UpdateModification(BSON("$set" << BSON("b.$[element]" << i))));
             updateOp.setC(BSON("x" << i));
             updateOp.setArrayFilters(std::vector<BSONObj>{BSON("element" << BSON("$gt" << i))});
+            updateOp.setSort(BSON("_id" << 1));
             updateOp.setMulti(_getRandomBool());
             updateOp.setUpsert(_getRandomBool());
             updateOp.setUpsertSupplied(_getRandomBool());
@@ -490,7 +523,7 @@ private:
     int _getConfigDocumentsCount(const NamespaceString& configNss,
                                  const NamespaceString& collNss) const {
         DBDirectClient client(operationContext());
-        return client.count(configNss, BSON("ns" << collNss.toString()));
+        return client.count(configNss, BSON("ns" << collNss.toString_forTest()));
     }
 
     /**
@@ -506,7 +539,6 @@ private:
         return cursor->next();
     }
 
-    RAIIServerParameterControllerForTest _featureFlagController{"featureFlagAnalyzeShardKey", true};
     // This fixture manually flushes sampled queries and diffs.
     FailPointEnableBlock _fp{"disableQueryAnalysisWriterFlusher"};
     PseudoRandom _random{SecureRandom{}.nextInt64()};
@@ -530,7 +562,7 @@ TEST_F(QueryAnalysisWriterTest, CreateTTLIndexesWhenSampledQueriesIndexExists) {
         FailPoint::alwaysOn,
         0,
         BSON("failCommands" << BSON_ARRAY("createIndexes") << "namespace"
-                            << NamespaceString::kConfigSampledQueriesNamespace.toString()
+                            << NamespaceString::kConfigSampledQueriesNamespace.toStringForErrorMsg()
                             << "errorCode" << ErrorCodes::IndexAlreadyExists
                             << "failInternalCommands" << true << "failLocalClients" << true));
     auto& writer = *QueryAnalysisWriter::get(operationContext());
@@ -544,13 +576,14 @@ TEST_F(QueryAnalysisWriterTest, CreateTTLIndexesWhenSampledQueriesIndexExists) {
 
 TEST_F(QueryAnalysisWriterTest, CreateTTLIndexesWhenSampledQueriesDiffIndexExists) {
     auto failCreateIndexes = globalFailPointRegistry().find("failCommand");
-    failCreateIndexes->setMode(
-        FailPoint::alwaysOn,
-        0,
-        BSON("failCommands" << BSON_ARRAY("createIndexes") << "namespace"
-                            << NamespaceString::kConfigSampledQueriesDiffNamespace.toString()
-                            << "errorCode" << ErrorCodes::IndexAlreadyExists
-                            << "failInternalCommands" << true << "failLocalClients" << true));
+    failCreateIndexes
+        ->setMode(FailPoint::alwaysOn,
+                  0,
+                  BSON("failCommands"
+                       << BSON_ARRAY("createIndexes") << "namespace"
+                       << NamespaceString::kConfigSampledQueriesDiffNamespace.toStringForErrorMsg()
+                       << "errorCode" << ErrorCodes::IndexAlreadyExists << "failInternalCommands"
+                       << true << "failLocalClients" << true));
     auto& writer = *QueryAnalysisWriter::get(operationContext());
     auto future = writer.createTTLIndexes(operationContext());
     future.get();
@@ -565,11 +598,11 @@ TEST_F(QueryAnalysisWriterTest, CreateTTLIndexesWhenAnalyzeShardKeySplitPointsIn
     failCreateIndexes->setMode(
         FailPoint::alwaysOn,
         0,
-        BSON(
-            "failCommands" << BSON_ARRAY("createIndexes") << "namespace"
-                           << NamespaceString::kConfigAnalyzeShardKeySplitPointsNamespace.toString()
-                           << "errorCode" << ErrorCodes::IndexAlreadyExists
-                           << "failInternalCommands" << true << "failLocalClients" << true));
+        BSON("failCommands"
+             << BSON_ARRAY("createIndexes") << "namespace"
+             << NamespaceString::kConfigAnalyzeShardKeySplitPointsNamespace.toStringForErrorMsg()
+             << "errorCode" << ErrorCodes::IndexAlreadyExists << "failInternalCommands" << true
+             << "failLocalClients" << true));
     auto& writer = *QueryAnalysisWriter::get(operationContext());
     auto future = writer.createTTLIndexes(operationContext());
     future.get();
@@ -740,7 +773,7 @@ TEST_F(QueryAnalysisWriterTest, AggregateQuery) {
 DEATH_TEST_F(QueryAnalysisWriterTest, UpdateQueryNotMarkedForSampling, "invariant") {
     auto& writer = *QueryAnalysisWriter::get(operationContext());
     auto [originalCmd, _] = makeUpdateCommandRequest(nss0, 1, {} /* markForSampling */);
-    writer.addUpdateQuery(originalCmd, 0).get();
+    writer.addUpdateQuery(operationContext(), originalCmd, 0).get();
 }
 
 TEST_F(QueryAnalysisWriterTest, UpdateQueriesMarkedForSampling) {
@@ -750,8 +783,8 @@ TEST_F(QueryAnalysisWriterTest, UpdateQueriesMarkedForSampling) {
         makeUpdateCommandRequest(nss0, 3, {0, 2} /* markForSampling */);
     ASSERT_EQ(expectedSampledCmds.size(), 2U);
 
-    writer.addUpdateQuery(originalCmd, 0).get();
-    writer.addUpdateQuery(originalCmd, 2).get();
+    writer.addUpdateQuery(operationContext(), originalCmd, 0).get();
+    writer.addUpdateQuery(operationContext(), originalCmd, 2).get();
     ASSERT_EQ(writer.getQueriesCountForTest(), 2);
     writer.flushQueriesForTest(operationContext());
     ASSERT_EQ(writer.getQueriesCountForTest(), 0);
@@ -768,7 +801,7 @@ TEST_F(QueryAnalysisWriterTest, UpdateQueriesMarkedForSampling) {
 DEATH_TEST_F(QueryAnalysisWriterTest, DeleteQueryNotMarkedForSampling, "invariant") {
     auto& writer = *QueryAnalysisWriter::get(operationContext());
     auto [originalCmd, _] = makeDeleteCommandRequest(nss0, 1, {} /* markForSampling */);
-    writer.addDeleteQuery(originalCmd, 0).get();
+    writer.addDeleteQuery(operationContext(), originalCmd, 0).get();
 }
 
 TEST_F(QueryAnalysisWriterTest, DeleteQueriesMarkedForSampling) {
@@ -778,8 +811,8 @@ TEST_F(QueryAnalysisWriterTest, DeleteQueriesMarkedForSampling) {
         makeDeleteCommandRequest(nss0, 3, {1, 2} /* markForSampling */);
     ASSERT_EQ(expectedSampledCmds.size(), 2U);
 
-    writer.addDeleteQuery(originalCmd, 1).get();
-    writer.addDeleteQuery(originalCmd, 2).get();
+    writer.addDeleteQuery(operationContext(), originalCmd, 1).get();
+    writer.addDeleteQuery(operationContext(), originalCmd, 2).get();
     ASSERT_EQ(writer.getQueriesCountForTest(), 2);
     writer.flushQueriesForTest(operationContext());
     ASSERT_EQ(writer.getQueriesCountForTest(), 0);
@@ -797,7 +830,7 @@ DEATH_TEST_F(QueryAnalysisWriterTest, FindAndModifyQueryNotMarkedForSampling, "i
     auto& writer = *QueryAnalysisWriter::get(operationContext());
     auto [originalCmd, _] =
         makeFindAndModifyCommandRequest(nss0, true /* isUpdate */, false /* markForSampling */);
-    writer.addFindAndModifyQuery(originalCmd).get();
+    writer.addFindAndModifyQuery(operationContext(), originalCmd).get();
 }
 
 TEST_F(QueryAnalysisWriterTest, FindAndModifyQueryUpdateMarkedForSampling) {
@@ -808,7 +841,7 @@ TEST_F(QueryAnalysisWriterTest, FindAndModifyQueryUpdateMarkedForSampling) {
     ASSERT_EQ(expectedSampledCmds.size(), 1U);
     auto [sampleId, expectedSampledCmd] = *expectedSampledCmds.begin();
 
-    writer.addFindAndModifyQuery(originalCmd).get();
+    writer.addFindAndModifyQuery(operationContext(), originalCmd).get();
     ASSERT_EQ(writer.getQueriesCountForTest(), 1);
     writer.flushQueriesForTest(operationContext());
     ASSERT_EQ(writer.getQueriesCountForTest(), 0);
@@ -828,7 +861,7 @@ TEST_F(QueryAnalysisWriterTest, FindAndModifyQueryRemoveMarkedForSampling) {
     ASSERT_EQ(expectedSampledCmds.size(), 1U);
     auto [sampleId, expectedSampledCmd] = *expectedSampledCmds.begin();
 
-    writer.addFindAndModifyQuery(originalCmd).get();
+    writer.addFindAndModifyQuery(operationContext(), originalCmd).get();
     ASSERT_EQ(writer.getQueriesCountForTest(), 1);
     writer.flushQueriesForTest(operationContext());
     ASSERT_EQ(writer.getQueriesCountForTest(), 0);
@@ -859,8 +892,8 @@ TEST_F(QueryAnalysisWriterTest, MultipleQueriesAndCollections) {
     auto originalCountFilter = makeNonEmptyFilter();
     auto originalCountCollation = makeNonEmptyCollation();
 
-    writer.addDeleteQuery(originalDeleteCmd, 1).get();
-    writer.addUpdateQuery(originalUpdateCmd, 0).get();
+    writer.addDeleteQuery(operationContext(), originalDeleteCmd, 1).get();
+    writer.addUpdateQuery(operationContext(), originalUpdateCmd, 0).get();
     writer.addCountQuery(countSampleId, nss1, originalCountFilter, originalCountCollation).get();
     ASSERT_EQ(writer.getQueriesCountForTest(), 3);
     writer.flushQueriesForTest(operationContext());
@@ -917,7 +950,7 @@ TEST_F(QueryAnalysisWriterTest, DuplicateQueries) {
                                    originalFindFilter,
                                    originalFindCollation);
 
-    writer.addUpdateQuery(originalUpdateCmd, 0).get();
+    writer.addUpdateQuery(operationContext(), originalUpdateCmd, 0).get();
     writer
         .addFindQuery(findSampleId,
                       nss0,
@@ -974,7 +1007,6 @@ TEST_F(QueryAnalysisWriterTest, QueriesMultipleBatches_MaxBatchSize) {
 }
 
 TEST_F(QueryAnalysisWriterTest, QueriesMultipleBatchesFewQueries_MaxBSONObjSize) {
-    RAIIServerParameterControllerForTest featureFlagController("featureFlagAnalyzeShardKey", true);
     auto& writer = *QueryAnalysisWriter::get(operationContext());
 
     auto numQueries = 3;
@@ -999,7 +1031,6 @@ TEST_F(QueryAnalysisWriterTest, QueriesMultipleBatchesFewQueries_MaxBSONObjSize)
 }
 
 TEST_F(QueryAnalysisWriterTest, QueriesMultipleBatchesManyQueries_MaxBSONObjSize) {
-    RAIIServerParameterControllerForTest featureFlagController("featureFlagAnalyzeShardKey", true);
     auto& writer = *QueryAnalysisWriter::get(operationContext());
 
     auto numQueries = 75'000;
@@ -1067,10 +1098,10 @@ TEST_F(QueryAnalysisWriterTest, FlushAfterAddUpdateIfExceedsSizeLimit) {
                                  std::string(maxMemoryUsageBytes / 2, 'a') /* filterFieldName */);
     ASSERT_EQ(expectedSampledCmds.size(), 2U);
 
-    writer.addUpdateQuery(originalCmd, 0).get();
+    writer.addUpdateQuery(operationContext(), originalCmd, 0).get();
     ASSERT_EQ(writer.getQueriesCountForTest(), 1);
     // Adding the next query causes the size to exceed the limit.
-    writer.addUpdateQuery(originalCmd, 2).get();
+    writer.addUpdateQuery(operationContext(), originalCmd, 2).get();
     ASSERT_EQ(writer.getQueriesCountForTest(), 0);
 
     ASSERT_EQ(getSampledQueryDocumentsCount(nss0), 2);
@@ -1095,10 +1126,10 @@ TEST_F(QueryAnalysisWriterTest, FlushAfterAddDeleteIfExceedsSizeLimit) {
                                  std::string(maxMemoryUsageBytes / 2, 'a') /* filterFieldName */);
     ASSERT_EQ(expectedSampledCmds.size(), 2U);
 
-    writer.addDeleteQuery(originalCmd, 0).get();
+    writer.addDeleteQuery(operationContext(), originalCmd, 0).get();
     ASSERT_EQ(writer.getQueriesCountForTest(), 1);
     // Adding the next query causes the size to exceed the limit.
-    writer.addDeleteQuery(originalCmd, 1).get();
+    writer.addDeleteQuery(operationContext(), originalCmd, 1).get();
     ASSERT_EQ(writer.getQueriesCountForTest(), 0);
 
     ASSERT_EQ(getSampledQueryDocumentsCount(nss0), 2);
@@ -1133,10 +1164,10 @@ TEST_F(QueryAnalysisWriterTest, FlushAfterAddFindAndModifyIfExceedsSizeLimit) {
     ASSERT_EQ(expectedSampledCmds0.size(), 1U);
     auto [sampleId1, expectedSampledCmd1] = *expectedSampledCmds1.begin();
 
-    writer.addFindAndModifyQuery(originalCmd0).get();
+    writer.addFindAndModifyQuery(operationContext(), originalCmd0).get();
     ASSERT_EQ(writer.getQueriesCountForTest(), 1);
     // Adding the next query causes the size to exceed the limit.
-    writer.addFindAndModifyQuery(originalCmd1).get();
+    writer.addFindAndModifyQuery(operationContext(), originalCmd1).get();
     ASSERT_EQ(writer.getQueriesCountForTest(), 0);
 
     ASSERT_EQ(getSampledQueryDocumentsCount(nss0), 1);
@@ -1179,7 +1210,7 @@ TEST_F(QueryAnalysisWriterTest, AddQueriesBackAfterWriteError) {
     auto hangTimesEntered = hangFp->setMode(FailPoint::alwaysOn, 0);
 
     auto future = stdx::async(stdx::launch::async, [&] {
-        ThreadClient tc(getServiceContext());
+        ThreadClient tc(getServiceContext()->getService());
         auto opCtx = makeOperationContext();
         writer.flushQueriesForTest(opCtx.get());
     });
@@ -1274,7 +1305,7 @@ TEST_F(QueryAnalysisWriterTest, RemoveDuplicatesFromBufferAfterWriteError) {
     auto hangTimesEntered = hangFp->setMode(FailPoint::alwaysOn, 0);
 
     auto future = stdx::async(stdx::launch::async, [&] {
-        ThreadClient tc(getServiceContext());
+        ThreadClient tc(getServiceContext()->getService());
         auto opCtx = makeOperationContext();
         writer.flushQueriesForTest(opCtx.get());
     });
@@ -1539,16 +1570,16 @@ void QueryAnalysisWriterTest::assertNoSampling(const NamespaceString& nss, const
     ASSERT_EQ(writer.getQueriesCountForTest(), 0);
 
     auto originalUpdateCmd = makeUpdateCommandRequest(nss, 1, {0} /* markForSampling */).first;
-    writer.addUpdateQuery(originalUpdateCmd, 0).get();
+    writer.addUpdateQuery(operationContext(), originalUpdateCmd, 0).get();
     ASSERT_EQ(writer.getQueriesCountForTest(), 0);
 
     auto originalDeleteCmd = makeDeleteCommandRequest(nss, 1, {0} /* markForSampling */).first;
-    writer.addDeleteQuery(originalDeleteCmd, 0).get();
+    writer.addDeleteQuery(operationContext(), originalDeleteCmd, 0).get();
     ASSERT_EQ(writer.getQueriesCountForTest(), 0);
 
     auto originalFindAndModifyCmd =
         makeFindAndModifyCommandRequest(nss, true /* isUpdate */, true /* markForSampling */).first;
-    writer.addFindAndModifyQuery(originalFindAndModifyCmd).get();
+    writer.addFindAndModifyQuery(operationContext(), originalFindAndModifyCmd).get();
 
     writer
         .addDiff(UUID::gen() /* sampleId */,
