@@ -29,19 +29,60 @@
 
 #include "mongo/db/s/range_deleter_service.h"
 
+#include <absl/container/node_hash_map.h>
+#include <absl/meta/type_traits.h>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <iterator>
+#include <tuple>
+#include <type_traits>
+#include <vector>
+
+#include <boost/move/utility_core.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+
+#include "mongo/base/checked_cast.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/client/dbclient_cursor.h"
 #include "mongo/db/catalog_raii.h"
+#include "mongo/db/client.h"
+#include "mongo/db/database_name.h"
 #include "mongo/db/dbdirectclient.h"
+#include "mongo/db/keypattern.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/op_observer/op_observer.h"
 #include "mongo/db/op_observer/op_observer_registry.h"
+#include "mongo/db/query/find_command.h"
 #include "mongo/db/repl/repl_client_info.h"
 #include "mongo/db/repl/wait_for_majority_service.h"
-#include "mongo/db/s/balancer_stats_registry.h"
+#include "mongo/db/s/collection_metadata.h"
 #include "mongo/db/s/collection_sharding_runtime.h"
 #include "mongo/db/s/range_deleter_service_op_observer.h"
 #include "mongo/db/s/range_deletion_util.h"
 #include "mongo/db/s/shard_filtering_metadata_refresh.h"
+#include "mongo/db/s/sharding_runtime_d_params_gen.h"
+#include "mongo/executor/network_interface_factory.h"
+#include "mongo/executor/network_interface_thread_pool.h"
+#include "mongo/executor/thread_pool_task_executor.h"
+#include "mongo/idl/idl_parser.h"
 #include "mongo/logv2/log.h"
-#include "mongo/s/sharding_feature_flags_gen.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/redaction.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/s/chunk_version.h"
+#include "mongo/s/shard_key_pattern.h"
+#include "mongo/util/cancellation.h"
+#include "mongo/util/database_name_util.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/duration.h"
 #include "mongo/util/future_util.h"
+#include "mongo/util/scopeguard.h"
+#include "mongo/util/str.h"
+#include "mongo/util/time_support.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kShardingRangeDeleter
 
@@ -57,7 +98,7 @@ BSONObj getShardKeyPattern(OperationContext* opCtx,
         boost::optional<NamespaceString> optNss;
         {
             AutoGetCollection collection(
-                opCtx, NamespaceStringOrUUID{dbName.toString(), collectionUuid}, MODE_IS);
+                opCtx, NamespaceStringOrUUID{dbName, collectionUuid}, MODE_IS);
 
             auto optMetadata = CollectionShardingRuntime::assertCollectionLockedAndAcquireShared(
                                    opCtx, collection.getNss())
@@ -88,7 +129,7 @@ RangeDeleterService* RangeDeleterService::get(OperationContext* opCtx) {
 
 RangeDeleterService::ReadyRangeDeletionsProcessor::ReadyRangeDeletionsProcessor(
     OperationContext* opCtx)
-    : _thread([this] { _runRangeDeletions(); }) {}
+    : _service(opCtx->getServiceContext()), _thread([this] { _runRangeDeletions(); }) {}
 
 RangeDeleterService::ReadyRangeDeletionsProcessor::~ReadyRangeDeletionsProcessor() {
     shutdown();
@@ -133,11 +174,8 @@ void RangeDeleterService::ReadyRangeDeletionsProcessor::_completedRangeDeletion(
 }
 
 void RangeDeleterService::ReadyRangeDeletionsProcessor::_runRangeDeletions() {
-    Client::initThread(kRangeDeletionThreadName);
-    {
-        stdx::lock_guard<Client> lk(cc());
-        cc().setSystemOperationKillableByStepdown(lk);
-    }
+    ThreadClient threadClient(rangedeletionutil::kRangeDeletionThreadName,
+                              _service->getService(ClusterRole::ShardServer));
 
     {
         stdx::lock_guard<Latch> lock(_mutex);
@@ -195,7 +233,7 @@ void RangeDeleterService::ReadyRangeDeletionsProcessor::_runRangeDeletions() {
                             (optKeyPattern ? (*optKeyPattern).toBSON()
                                            : getShardKeyPattern(opCtx, dbName, collectionUuid));
 
-                        uassertStatusOK(deleteRangeInBatches(
+                        uassertStatusOK(rangedeletionutil::deleteRangeInBatches(
                             opCtx, dbName, collectionUuid, shardKeyPattern, range));
                         orphansRemovalCompleted = true;
                     } catch (ExceptionFor<ErrorCodes::NamespaceNotFound>&) {
@@ -247,14 +285,15 @@ void RangeDeleterService::ReadyRangeDeletionsProcessor::_runRangeDeletions() {
                     // out of order the deletions of orphans and the removal of the
                     // entry persisted in `config.rangeDeletions`
                     WaitForMajorityService::get(opCtx->getServiceContext())
-                        .waitUntilMajority(clientOpTime, CancellationToken::uncancelable())
+                        .waitUntilMajorityForWrite(clientOpTime, CancellationToken::uncancelable())
                         .get(opCtx);
                 }
 
                 // Remove persistent range deletion task
                 try {
                     RangeDeleterService::get(opCtx)->deregisterTask(collectionUuid, range);
-                    removePersistentRangeDeletionTask(opCtx, collectionUuid, range);
+                    rangedeletionutil::removePersistentRangeDeletionTask(
+                        opCtx, collectionUuid, range);
 
                     LOGV2_DEBUG(6872504,
                                 2,
@@ -299,10 +338,7 @@ void RangeDeleterService::ReadyRangeDeletionsProcessor::_runRangeDeletions() {
 }
 
 void RangeDeleterService::onStartup(OperationContext* opCtx) {
-    // (Ignore FCV check): This feature doesn't have any upgrade/downgrade concerns. The feature
-    // flag is used to turn on new range deleter on startup.
-    if (disableResumableRangeDeleter.load() ||
-        !feature_flags::gRangeDeleterService.isEnabledAndIgnoreFCVUnsafe()) {
+    if (disableResumableRangeDeleter.load()) {
         return;
     }
 
@@ -312,12 +348,6 @@ void RangeDeleterService::onStartup(OperationContext* opCtx) {
 }
 
 void RangeDeleterService::onStepUpComplete(OperationContext* opCtx, long long term) {
-    // (Ignore FCV check): This feature doesn't have any upgrade/downgrade concerns. The feature
-    // flag is used to turn on new range deleter on startup.
-    if (!feature_flags::gRangeDeleterService.isEnabledAndIgnoreFCVUnsafe()) {
-        return;
-    }
-
     if (disableResumableRangeDeleter.load()) {
         LOGV2_INFO(
             6872508,
@@ -331,7 +361,7 @@ void RangeDeleterService::onStepUpComplete(OperationContext* opCtx, long long te
     auto lock = _acquireMutexUnconditionally();
     dassert(_state == kDown, "Service expected to be down before stepping up");
 
-    _state = kInitializing;
+    _state = kReadyForInitialization;
 
     const std::string kExecName("RangeDeleterServiceExecutor");
     auto net = executor::makeNetworkInterface(kExecName);
@@ -348,17 +378,18 @@ void RangeDeleterService::onStepUpComplete(OperationContext* opCtx, long long te
 }
 
 void RangeDeleterService::_recoverRangeDeletionsOnStepUp(OperationContext* opCtx) {
-
     _stepUpCompletedFuture =
         ExecutorFuture<void>(_executor)
             .then([serviceContext = opCtx->getServiceContext(), this] {
-                ThreadClient tc("ResubmitRangeDeletionsOnStepUp", serviceContext);
+                ThreadClient tc("ResubmitRangeDeletionsOnStepUp",
+                                serviceContext->getService(ClusterRole::ShardServer));
 
                 {
                     auto lock = _acquireMutexUnconditionally();
-                    if (_state != kInitializing) {
+                    if (_state != kReadyForInitialization) {
                         return;
                     }
+                    _state = kInitializing;
                     _initOpCtxHolder = tc->makeOperationContext();
                 }
 
