@@ -27,14 +27,44 @@
  *    it in the license file.
  */
 
-#include "mongo/db/s/range_deleter_service_test.h"
+#include <ostream>
+
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/bson/oid.h"
+#include "mongo/bson/timestamp.h"
 #include "mongo/bson/unordered_fields_bsonobj_comparator.h"
+#include "mongo/db/catalog/collection.h"
 #include "mongo/db/catalog/create_collection.h"
 #include "mongo/db/catalog_raii.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
+#include "mongo/db/dbdirectclient.h"
 #include "mongo/db/persistent_task_store.h"
+#include "mongo/db/query/collation/collator_interface.h"
 #include "mongo/db/repl/wait_for_majority_service.h"
+#include "mongo/db/s/collection_metadata.h"
 #include "mongo/db/s/collection_sharding_runtime.h"
 #include "mongo/db/s/operation_sharding_state.h"
+#include "mongo/db/s/range_deleter_service.h"
+#include "mongo/db/s/range_deleter_service_test.h"
+#include "mongo/db/s/sharding_runtime_d_params_gen.h"
+#include "mongo/db/shard_id.h"
+#include "mongo/idl/server_parameter_test_util.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/platform/random.h"
+#include "mongo/s/chunk_manager.h"
+#include "mongo/s/chunk_version.h"
+#include "mongo/s/database_version.h"
+#include "mongo/s/resharding/type_collection_fields_gen.h"
+#include "mongo/s/type_collection_common_types_gen.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/duration.h"
+#include "mongo/util/time_support.h"
 
 namespace mongo {
 
@@ -59,13 +89,13 @@ void RangeDeleterServiceTest::setUp() {
     }
 
     {
-        AutoGetCollection autoColl(opCtx, nsCollA, MODE_IX);
+        AutoGetCollection autoColl(opCtx, nsCollA, MODE_X);
         uuidCollA = autoColl.getCollection()->uuid();
         nssWithUuid[uuidCollA] = nsCollA;
         _setFilteringMetadataByUUID(opCtx, uuidCollA);
     }
     {
-        AutoGetCollection autoColl(opCtx, nsCollB, MODE_IX);
+        AutoGetCollection autoColl(opCtx, nsCollB, MODE_X);
         uuidCollB = autoColl.getCollection()->uuid();
         nssWithUuid[uuidCollB] = nsCollB;
         _setFilteringMetadataByUUID(opCtx, uuidCollB);
@@ -102,6 +132,7 @@ void RangeDeleterServiceTest::_setFilteringMetadataByUUID(OperationContext* opCt
                             RoutingTableHistory::makeNew(nss,
                                                          uuid,
                                                          kShardKeyPattern,
+                                                         false, /* unsplittable */
                                                          nullptr,
                                                          false,
                                                          epoch,
@@ -116,7 +147,6 @@ void RangeDeleterServiceTest::_setFilteringMetadataByUUID(OperationContext* opCt
         return CollectionMetadata(std::move(cm), ShardId("this"));
     }();
 
-    AutoGetCollection autoColl(opCtx, nss, LockMode::MODE_X);
     CollectionShardingRuntime::assertCollectionLockedAndAcquireExclusive(opCtx, nss)
         ->setFilteringMetadata(opCtx, metadata);
 }
@@ -588,7 +618,7 @@ TEST_F(RangeDeleterServiceTest, TotalNumOfRegisteredTasks) {
 }
 
 TEST_F(RangeDeleterServiceTest, RegisterTaskWithDisableResumableRangeDeleterFlagEnabled) {
-    RAIIServerParameterControllerForTest enableFeatureFlag{"disableResumableRangeDeleter", true};
+    RAIIServerParameterControllerForTest disableRangeDeleter{"disableResumableRangeDeleter", true};
 
     auto rds = RangeDeleterService::get(opCtx);
     auto taskWithOngoingQueries = rangeDeletionTask0ForCollA;
@@ -615,7 +645,7 @@ TEST_F(RangeDeleterServiceTest,
         uuidCollA, taskWithOngoingQueries->getTask().getRange());
     ASSERT(!overlappingRangeFuture.isReady());
 
-    RAIIServerParameterControllerForTest enableFeatureFlag{"disableResumableRangeDeleter", true};
+    RAIIServerParameterControllerForTest disableRangeDeleter{"disableResumableRangeDeleter", true};
     auto overlappingRangeFutureWhenDisabled = rds->getOverlappingRangeDeletionsFuture(
         uuidCollA, taskWithOngoingQueries->getTask().getRange());
     ASSERT(overlappingRangeFutureWhenDisabled.isReady());
