@@ -27,23 +27,73 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <cstddef>
+#include <cstdint>
+#include <deque>
+#include <iostream>
+#include <iterator>
+#include <map>
+#include <memory>
+#include <string>
+#include <utility>
+#include <vector>
 
+#include <boost/cstdint.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <fmt/format.h>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/json.h"
+#include "mongo/bson/oid.h"
+#include "mongo/bson/timestamp.h"
 #include "mongo/db/dbdirectclient.h"
+#include "mongo/db/exec/document_value/document.h"
 #include "mongo/db/exec/document_value/document_value_test_util.h"
+#include "mongo/db/exec/document_value/value.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/pipeline/aggregate_command_gen.h"
 #include "mongo/db/pipeline/aggregation_context_fixture.h"
+#include "mongo/db/pipeline/document_source.h"
 #include "mongo/db/pipeline/document_source_mock.h"
+#include "mongo/db/pipeline/expression_context.h"
 #include "mongo/db/pipeline/expression_context_for_test.h"
+#include "mongo/db/pipeline/pipeline.h"
+#include "mongo/db/pipeline/process_interface/stub_mongo_process_interface.h"
+#include "mongo/db/pipeline/sharded_agg_helpers_targeting_policy.h"
 #include "mongo/db/repl/apply_ops_command_info.h"
+#include "mongo/db/repl/apply_ops_gen.h"
 #include "mongo/db/repl/image_collection_entry_gen.h"
 #include "mongo/db/repl/mock_repl_coord_server_fixture.h"
 #include "mongo/db/repl/oplog_entry.h"
+#include "mongo/db/repl/oplog_entry_gen.h"
+#include "mongo/db/repl/optime.h"
+#include "mongo/db/s/resharding/donor_oplog_id_gen.h"
 #include "mongo/db/s/resharding/resharding_donor_oplog_iterator.h"
 #include "mongo/db/s/resharding/resharding_util.h"
-#include "mongo/db/service_context_d_test_fixture.h"
+#include "mongo/db/session/logical_session_id.h"
+#include "mongo/db/session/logical_session_id_gen.h"
+#include "mongo/db/shard_id.h"
 #include "mongo/db/transaction/transaction_history_iterator.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/idl/idl_parser.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/bson_test_util.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/intrusive_counter.h"
 #include "mongo/util/str.h"
+#include "mongo/util/string_map.h"
+#include "mongo/util/time_support.h"
+#include "mongo/util/uuid.h"
 
 namespace mongo {
 namespace {
@@ -62,13 +112,13 @@ public:
           _oplogIt(_oplogContents.rbegin()),
           _nextOpTime(std::move(startTime)) {}
 
-    virtual ~MockTransactionHistoryIterator() = default;
+    ~MockTransactionHistoryIterator() override = default;
 
-    bool hasNext() const {
+    bool hasNext() const override {
         return !_nextOpTime.isNull();
     }
 
-    repl::OplogEntry next(OperationContext* opCtx) {
+    repl::OplogEntry next(OperationContext* opCtx) override {
         BSONObj oplogBSON = findOneOplogEntry(_nextOpTime);
 
         auto oplogEntry = uassertStatusOK(repl::OplogEntry::parse(oplogBSON));
@@ -82,7 +132,7 @@ public:
         return oplogEntry;
     }
 
-    repl::OpTime nextOpTime(OperationContext* opCtx) {
+    repl::OpTime nextOpTime(OperationContext* opCtx) override {
         BSONObj oplogBSON = findOneOplogEntry(_nextOpTime);
 
         auto prevOpTime = oplogBSON[repl::OplogEntry::kPrevWriteOpTimeInTransactionFieldName];
@@ -122,7 +172,7 @@ public:
     MockMongoInterface(std::deque<DocumentSource::GetNextResult> mockResults)
         : _mockResults(std::move(mockResults)) {}
 
-    std::unique_ptr<Pipeline, PipelineDeleter> attachCursorSourceToPipeline(
+    std::unique_ptr<Pipeline, PipelineDeleter> preparePipelineForExecution(
         Pipeline* ownedPipeline,
         ShardTargetingPolicy shardTargetingPolicy = ShardTargetingPolicy::kAllowed,
         boost::optional<BSONObj> readConcern = boost::none) final {
@@ -134,7 +184,7 @@ public:
         return pipeline;
     }
 
-    std::unique_ptr<Pipeline, PipelineDeleter> attachCursorSourceToPipeline(
+    std::unique_ptr<Pipeline, PipelineDeleter> preparePipelineForExecution(
         const boost::intrusive_ptr<ExpressionContext>& expCtx,
         const AggregateCommandRequest& aggRequest,
         Pipeline* pipeline,
@@ -142,11 +192,11 @@ public:
         ShardTargetingPolicy shardTargetingPolicy = ShardTargetingPolicy::kAllowed,
         boost::optional<BSONObj> readConcern = boost::none,
         bool shouldUseDefaultCollectionCollator = false) final {
-        return attachCursorSourceToPipeline(pipeline, shardTargetingPolicy, std::move(readConcern));
+        return preparePipelineForExecution(pipeline, shardTargetingPolicy, std::move(readConcern));
     }
 
     std::unique_ptr<TransactionHistoryIteratorBase> createTransactionHistoryIterator(
-        repl::OpTime time) const {
+        repl::OpTime time) const override {
         return std::unique_ptr<TransactionHistoryIteratorBase>(
             new MockTransactionHistoryIterator(_mockResults, time));
     }
@@ -154,7 +204,7 @@ public:
     BSONObj getCollectionOptions(OperationContext* opCtx, const NamespaceString& nss) override {
         auto optionIter = _collectionOptions.find(nss);
         invariant(optionIter != _collectionOptions.end(),
-                  str::stream() << nss.ns() << " was not registered");
+                  str::stream() << nss.toStringForErrorMsg() << " was not registered");
 
         return optionIter->second;
     }
@@ -164,7 +214,7 @@ public:
         const NamespaceString& nss,
         boost::optional<UUID> collectionUUID,
         const Document& documentKey,
-        boost::optional<BSONObj> readConcern) {
+        boost::optional<BSONObj> readConcern) override {
         DBDirectClient client(expCtx->opCtx);
         auto result = client.findOne(nss, documentKey.toBson());
         if (result.isEmpty()) {
@@ -384,7 +434,7 @@ protected:
         return pipeline;
     }
 
-    const NamespaceString _crudNss{"test.foo"};
+    const NamespaceString _crudNss = NamespaceString::createNamespaceString_forTest("test.foo");
     // Use a constant value so unittests can store oplog entries as extended json strings in code.
     const UUID _reshardingCollUUID =
         fassert(5074001, UUID::parse("8926ba8e-611a-42c2-bb1a-3b7819f610ed"));
@@ -1305,6 +1355,107 @@ TEST_F(ReshardingAggTest, VerifyPipelineLargeTxn) {
     ASSERT(!doc);
 }
 
+TEST_F(ReshardingAggTest, VerifyPipelineLargeBatchedRetryableWrite) {
+    std::deque<DocumentSource::GetNextResult> pipelineSource = {Document(fromjson(R"({
+        "lsid": {
+          "id": { "$binary": "+0TxuFyBSeqjfJzju2Xl+w==", "$type": "04" },
+          "uid": { "$binary": "47DEQpj8HBSa+/TImW+5JCeuQeRkm5NMpJWZG3hSuFU=", "$type": "00" }
+        },
+        "txnNumber": { "$numberLong": "0" },
+        "op": "c",
+        "ns": "admin.$cmd",
+        "o": {
+          "applyOps": [ {
+              "op": "i",
+              "ns": "test.foo",
+              "ui": { "$binary": "iSa6jmEaQsK7Gjt4GfYQ7Q==", "$type": "04" },
+              "o": { "_id": 4, "x": -20, "y": 4 },
+              "destinedRecipient": "shard0"
+            },
+            {
+              "op": "i",
+              "ns": "test.foo",
+              "ui": { "$binary": "iSa6jmEaQsK7Gjt4GfYQ7Q==", "$type": "04" },
+              "o": { "_id": 5, "x": -30, "y": 11 },
+              "destinedRecipient": "shard1"
+            }
+          ]
+        },
+        "ts": { "$timestamp": { "t": 1609800491, "i": 1 } },
+        "t": { "$numberLong": "1" },
+        "wall": { "$date": "2021-01-04T17:48:11.237-05:00" },
+        "v": { "$numberLong": "2" },
+        "prevOpTime": {
+          "ts": { "$timestamp": { "t": 0, "i": 0 } },
+          "t": { "$numberLong": "-1" }
+        },
+        "multiOpType": 1
+    })")),
+                                                                Document(fromjson(R"({
+        "lsid": {
+          "id": { "$binary": "+0TxuFyBSeqjfJzju2Xl+w==", "$type": "04" },
+          "uid": { "$binary": "47DEQpj8HBSa+/TImW+5JCeuQeRkm5NMpJWZG3hSuFU=", "$type": "00" }
+        },
+        "txnNumber": { "$numberLong": "0" },
+        "op": "c",
+        "ns": "admin.$cmd",
+        "o": {
+          "applyOps": [ {
+              "op": "i",
+              "ns": "test.foo",
+              "ui": { "$binary": "iSa6jmEaQsK7Gjt4GfYQ7Q==", "$type": "04" },
+              "o": { "_id": 6, "x": -40, "y": 11 },
+              "destinedRecipient": "shard1"
+            }
+          ],
+          "count": { "$numberLong": "3" }
+        },
+        "ts": { "$timestamp": { "t": 1609800491, "i": 2 } },
+        "t": { "$numberLong": "1" },
+        "wall": { "$date": "2021-01-04T17:48:11.240-05:00" },
+        "v": { "$numberLong": "2" },
+        "prevOpTime": {
+          "ts": { "$timestamp": { "t": 1609800491, "i": 1 } },
+          "t": { "$numberLong": "1" }
+        },
+        "multiOpType": 1
+    })"))};
+
+    auto pipeline = createPipeline(pipelineSource);
+
+    auto doc = pipeline->getNext();
+    ASSERT(doc);
+
+    auto oplogEntry = uassertStatusOK(repl::OplogEntry::parse(doc->toBson()));
+
+    ASSERT(oplogEntry.isCommand());
+    ASSERT(repl::OplogEntry::CommandType::kApplyOps == oplogEntry.getCommandType());
+    ASSERT_FALSE(oplogEntry.shouldPrepare());
+    ASSERT_FALSE(oplogEntry.isPartialTransaction());
+    ASSERT_EQ(repl::MultiOplogEntryType::kApplyOpsAppliedSeparately, oplogEntry.getMultiOpType());
+    ASSERT_EQ(1, oplogEntry.getObject()["applyOps"].Obj().nFields());
+    ASSERT_EQ(pipelineSource[0].getDocument()["ts"].getTimestamp(), oplogEntry.getTimestamp());
+    auto timestamp0 = pipelineSource[0].getDocument()["ts"].getTimestamp();
+    ASSERT(validateOplogId(timestamp0, pipelineSource[0].getDocument(), oplogEntry));
+
+    doc = pipeline->getNext();
+    ASSERT(doc);
+
+    oplogEntry = uassertStatusOK(repl::OplogEntry::parse(doc->toBson()));
+    ASSERT(oplogEntry.isCommand());
+    ASSERT(repl::OplogEntry::CommandType::kApplyOps == oplogEntry.getCommandType());
+    ASSERT_FALSE(oplogEntry.shouldPrepare());
+    ASSERT_FALSE(oplogEntry.isPartialTransaction());
+    ASSERT_EQ(repl::MultiOplogEntryType::kApplyOpsAppliedSeparately, oplogEntry.getMultiOpType());
+    ASSERT_EQ(1, oplogEntry.getObject()["applyOps"].Obj().nFields());
+    ASSERT_EQ(pipelineSource[1].getDocument()["ts"].getTimestamp(), oplogEntry.getTimestamp());
+    auto timestamp1 = pipelineSource[1].getDocument()["ts"].getTimestamp();
+    ASSERT(validateOplogId(timestamp1, pipelineSource[1].getDocument(), oplogEntry));
+
+    doc = pipeline->getNext();
+    ASSERT(!doc);
+}
+
 // This case can only happen if a primary locally commits the transaction, but fails before the
 // commit txn is replicated. A new node will step up, and then it will see the in-progress entry +
 // txn state, and then abort it. This means the new primary will have a a partialTxn applyOps entry
@@ -1432,7 +1583,7 @@ using ReshardingAggWithStorageTest = MockReplCoordServerFixture;
 // with no-op pre/post image oplog.
 TEST_F(ReshardingAggWithStorageTest, RetryableFindAndModifyWithImageLookup) {
     repl::OpTime opTime(Timestamp(43, 56), 1);
-    const NamespaceString kCrudNs("foo", "bar");
+    const NamespaceString kCrudNs = NamespaceString::createNamespaceString_forTest("foo", "bar");
     const UUID kCrudUUID = UUID::gen();
     const ShardId kMyShardId{"shard1"};
     ReshardingDonorOplogId id(opTime.getTimestamp(), opTime.getTimestamp());
@@ -1528,7 +1679,7 @@ TEST_F(ReshardingAggWithStorageTest, RetryableFindAndModifyWithImageLookup) {
 
 TEST_F(ReshardingAggWithStorageTest,
        RetryableFindAndModifyInsideInternalTransactionWithImageLookup) {
-    const NamespaceString kCrudNs("foo", "bar");
+    const NamespaceString kCrudNs = NamespaceString::createNamespaceString_forTest("foo", "bar");
     const UUID kCrudUUID = UUID::gen();
     const ShardId kMyShardId{"shard1"};
 
