@@ -29,18 +29,45 @@
 
 #pragma once
 
+#include <memory>
+#include <utility>
+#include <vector>
+
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/status.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/oid.h"
+#include "mongo/bson/timestamp.h"
 #include "mongo/db/cancelable_operation_context.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/pipeline/process_interface/mongo_process_interface.h"
 #include "mongo/db/repl/primary_only_service.h"
 #include "mongo/db/s/resharding/coordinator_document_gen.h"
 #include "mongo/db/s/resharding/resharding_coordinator_observer.h"
 #include "mongo/db/s/resharding/resharding_metrics.h"
 #include "mongo/db/service_context.h"
+#include "mongo/db/session/logical_session_id.h"
 #include "mongo/db/shard_id.h"
+#include "mongo/executor/async_rpc.h"
+#include "mongo/executor/scoped_task_executor.h"
+#include "mongo/executor/task_executor.h"
 #include "mongo/platform/mutex.h"
 #include "mongo/s/catalog/type_chunk.h"
 #include "mongo/s/catalog/type_collection.h"
 #include "mongo/s/catalog/type_tags.h"
+#include "mongo/s/chunk_version.h"
+#include "mongo/s/index_version.h"
+#include "mongo/s/resharding/common_types_gen.h"
+#include "mongo/util/cancellation.h"
+#include "mongo/util/concurrency/thread_pool.h"
 #include "mongo/util/future.h"
+#include "mongo/util/future_impl.h"
+#include "mongo/util/uuid.h"
 
 namespace mongo {
 namespace resharding {
@@ -51,7 +78,8 @@ CollectionType createTempReshardingCollectionType(
     const ReshardingCoordinatorDocument& coordinatorDoc,
     const ChunkVersion& chunkVersion,
     const BSONObj& collation,
-    boost::optional<CollectionIndexes> indexVersion);
+    boost::optional<CollectionIndexes> indexVersion,
+    boost::optional<bool> isUnsplittable);
 
 void removeChunkDocs(OperationContext* opCtx, const UUID& collUUID);
 
@@ -76,14 +104,15 @@ void writeParticipantShardsAndTempCollInfo(OperationContext* opCtx,
                                            const ReshardingCoordinatorDocument& coordinatorDoc,
                                            std::vector<ChunkType> initialChunks,
                                            std::vector<BSONObj> zones,
-                                           boost::optional<CollectionIndexes> indexVersion);
+                                           boost::optional<CollectionIndexes> indexVersion,
+                                           boost::optional<bool> isUnsplittable);
 
 void writeStateTransitionAndCatalogUpdatesThenBumpCollectionPlacementVersions(
     OperationContext* opCtx,
     ReshardingMetrics* metrics,
     const ReshardingCoordinatorDocument& coordinatorDoc);
 
-ReshardingCoordinatorDocument removeCoordinatorDocAndReshardingFields(
+ReshardingCoordinatorDocument removeOrQuiesceCoordinatorDocAndRemoveReshardingFields(
     OperationContext* opCtx,
     ReshardingMetrics* metrics,
     const ReshardingCoordinatorDocument& coordinatorDoc,
@@ -109,26 +138,21 @@ public:
                                                               const NamespaceString& nss,
                                                               const UUID& uuid);
 
+    bool getIsUnsplittable(OperationContext* opCtx, const NamespaceString& nss);
+
     boost::optional<CollectionIndexes> getCatalogIndexVersionForCommit(OperationContext* opCtx,
                                                                        const NamespaceString& nss);
 
-    virtual void sendCommandToShards(OperationContext* opCtx,
-                                     StringData dbName,
-                                     const BSONObj& command,
-                                     const std::vector<ShardId>& shardIds,
-                                     const std::shared_ptr<executor::TaskExecutor>& executor) = 0;
+    template <typename CommandType>
+    void sendCommandToShards(OperationContext* opCtx,
+                             std::shared_ptr<async_rpc::AsyncRPCOptions<CommandType>> opts,
+                             const std::vector<ShardId>& shardIds);
 };
 
 class ReshardingCoordinatorExternalStateImpl final : public ReshardingCoordinatorExternalState {
 public:
     ParticipantShardsAndChunks calculateParticipantShardsAndChunks(
         OperationContext* opCtx, const ReshardingCoordinatorDocument& coordinatorDoc) override;
-
-    void sendCommandToShards(OperationContext* opCtx,
-                             StringData dbName,
-                             const BSONObj& command,
-                             const std::vector<ShardId>& shardIds,
-                             const std::shared_ptr<executor::TaskExecutor>& executor) override;
 };
 
 /**
@@ -140,7 +164,8 @@ public:
         : _stepdownToken(stepdownToken),
           _abortSource(CancellationSource(stepdownToken)),
           _abortToken(_abortSource.token()),
-          _commitMonitorCancellationSource(CancellationSource(_abortToken)) {}
+          _commitMonitorCancellationSource(CancellationSource(_abortToken)),
+          _quiesceCancellationSource(CancellationSource(_stepdownToken)) {}
 
     /**
      * Returns whether the any token has been canceled.
@@ -177,6 +202,10 @@ public:
         _commitMonitorCancellationSource.cancel();
     }
 
+    void cancelQuiescePeriod() {
+        _quiesceCancellationSource.cancel();
+    }
+
     const CancellationToken& getStepdownToken() {
         return _stepdownToken;
     }
@@ -189,6 +218,10 @@ public:
         return _commitMonitorCancellationSource.token();
     }
 
+    CancellationToken getCancelQuiesceToken() {
+        return _quiesceCancellationSource.token();
+    }
+
 private:
     // The token passed in by the PrimaryOnlyService runner that is canceled when this shard's
     // underlying replica set node is stepping down or shutting down.
@@ -204,6 +237,10 @@ private:
     // The source created by inheriting from the abort token.
     // Provides the means to cancel the commit monitor (e.g., due to receiving the commit command).
     CancellationSource _commitMonitorCancellationSource;
+
+    // A source created by inheriting from the stepdown token.
+    // Provides the means to cancel the quiesce period.
+    CancellationSource _quiesceCancellationSource;
 };
 
 class ReshardingCoordinator;
@@ -214,7 +251,7 @@ public:
 
     explicit ReshardingCoordinatorService(ServiceContext* serviceContext)
         : PrimaryOnlyService(serviceContext), _serviceContext(serviceContext) {}
-    ~ReshardingCoordinatorService() = default;
+    ~ReshardingCoordinatorService() override = default;
 
     friend ReshardingCoordinator;
 
@@ -246,6 +283,8 @@ public:
      * between operations interrupted due to stepdown or abort. Callers who wish to confirm that
      * the abort successfully went through should follow up with an inspection on the resharding
      * coordinator docs to ensure that they are empty.
+     *
+     * This call skips quiesce periods for all aborted coordinators.
      */
     void abortAllReshardCollection(OperationContext* opCtx);
 
@@ -264,7 +303,7 @@ public:
         const ReshardingCoordinatorDocument& coordinatorDoc,
         std::shared_ptr<ReshardingCoordinatorExternalState> externalState,
         ServiceContext* serviceContext);
-    ~ReshardingCoordinator() = default;
+    ~ReshardingCoordinator() override = default;
 
     SemiFuture<void> run(std::shared_ptr<executor::ScopedTaskExecutor> executor,
                          const CancellationToken& token) noexcept override;
@@ -273,8 +312,9 @@ public:
 
     /**
      * Attempts to cancel the underlying resharding operation using the abort token.
+     * If 'skipQuiescePeriod' is set, will also skip the quiesce period used to allow retries.
      */
-    void abort();
+    void abort(bool skipQuiescePeriod = false);
 
     /**
      * Replace in-memory representation of the CoordinatorDoc
@@ -302,6 +342,14 @@ public:
         return _coordinatorDocWrittenPromise.getFuture();
     }
 
+    /**
+     * Returns a Future that will be resolved when the service has finished its quiesce period
+     * and deleted the coordinator document.
+     */
+    SharedSemiFuture<void> getQuiescePeriodFinishedFuture() const {
+        return _quiescePeriodFinishedPromise.getFuture();
+    }
+
     boost::optional<BSONObj> reportForCurrentOp(
         MongoProcessInterface::CurrentOpConnectionsMode,
         MongoProcessInterface::CurrentOpSessionsMode) noexcept override;
@@ -386,6 +434,12 @@ private:
     ExecutorFuture<void> _runReshardingOp(
         const std::shared_ptr<executor::ScopedTaskExecutor>& executor);
 
+    /**
+     * Keep the instance in a quiesced state in order to handle retries.
+     */
+    ExecutorFuture<void> _quiesce(const std::shared_ptr<executor::ScopedTaskExecutor>& executor,
+                                  Status status);
+
     /**
      * Does the following writes:
      * 1. Inserts the coordinator document into config.reshardingOperations
@@ -452,6 +506,13 @@ private:
      */
     void _commit(const ReshardingCoordinatorDocument& updatedDoc);
 
+    /**
+     * Sends a _shardsvrNotifyShardingEvent commands to the primary shard of the resharded
+     * collection to trigger the generation of the related internal op entry.
+     */
+    void _generateOpEventOnCoordinatingShard(
+        const std::shared_ptr<executor::ScopedTaskExecutor>& executor);
+
     /**
      * Waits on _reshardingCoordinatorObserver to notify that:
      * 1. All recipient shards have renamed the temporary collection to the original collection
@@ -475,15 +536,27 @@ private:
         boost::optional<ReshardingApproxCopySize> approxCopySize = boost::none,
         boost::optional<Status> abortReason = boost::none);
 
+    /**
+     * Updates the entry for this resharding operation in config.reshardingOperations to the
+     * quiesced state, or removes it if quiesce isn't being done.  Removes the resharding fields
+     * from the catalog entries.
+     */
+    void _removeOrQuiesceCoordinatorDocAndRemoveReshardingFields(
+        OperationContext* opCtx, boost::optional<Status> abortReason = boost::none);
+
     /**
      * Sends the command to the specified participants asynchronously.
      */
+    template <typename CommandType>
     void _sendCommandToAllParticipants(
-        const std::shared_ptr<executor::ScopedTaskExecutor>& executor, const BSONObj& command);
+        const std::shared_ptr<executor::ScopedTaskExecutor>& executor,
+        std::shared_ptr<async_rpc::AsyncRPCOptions<CommandType>> opts);
+    template <typename CommandType>
     void _sendCommandToAllDonors(const std::shared_ptr<executor::ScopedTaskExecutor>& executor,
-                                 const BSONObj& command);
+                                 std::shared_ptr<async_rpc::AsyncRPCOptions<CommandType>> opts);
+    template <typename CommandType>
     void _sendCommandToAllRecipients(const std::shared_ptr<executor::ScopedTaskExecutor>& executor,
-                                     const BSONObj& command);
+                                     std::shared_ptr<async_rpc::AsyncRPCOptions<CommandType>> opts);
 
     /**
      * Sends '_flushRoutingTableCacheUpdatesWithWriteConcern' to ensure donor state machine creation
@@ -604,6 +677,9 @@ private:
     // Promise that is fulfilled when the chain of work kicked off by run() has completed.
     SharedPromise<void> _completionPromise;
 
+    // Promise that is fulfilled when the quiesce period is finished
+    SharedPromise<void> _quiescePeriodFinishedPromise;
+
     // Callback handle for scheduled work to handle critical section timeout.
     boost::optional<executor::TaskExecutor::CallbackHandle> _criticalSectionTimeoutCbHandle;
 
@@ -614,7 +690,15 @@ private:
 
     // Used to catch the case when an abort() is called but the cancellation source (_ctHolder) has
     // not been initialized.
-    bool _abortCalled{false};
+    enum AbortType {
+        kNoAbort = 0,
+        kAbortWithQuiesce,
+        kAbortSkipQuiesce
+    } _abortCalled{AbortType::kNoAbort};
+
+    // If we recovered a completed resharding coordinator (quiesced) on failover, the
+    // resharding status when it actually ran.
+    boost::optional<Status> _originalReshardingStatus;
 };
 
 }  // namespace mongo
