@@ -27,30 +27,89 @@
  *    it in the license file.
  */
 
+#include <fmt/format.h>
+#include <memory>
+#include <string>
+#include <utility>
+#include <vector>
+
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/oid.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/client/connection_string.h"
+#include "mongo/client/remote_command_targeter_factory_mock.h"
+#include "mongo/client/remote_command_targeter_mock.h"
+#include "mongo/db/catalog/collection_catalog.h"
 #include "mongo/db/catalog/collection_write_path.h"
 #include "mongo/db/catalog/create_collection.h"
 #include "mongo/db/catalog_raii.h"
+#include "mongo/db/cluster_role.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
+#include "mongo/db/curop.h"
 #include "mongo/db/dbdirectclient.h"
 #include "mongo/db/dbhelpers.h"
-#include "mongo/db/repl/apply_ops.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/record_id.h"
+#include "mongo/db/repl/apply_ops_command_info.h"
+#include "mongo/db/repl/oplog.h"
+#include "mongo/db/repl/oplog_entry.h"
+#include "mongo/db/repl/oplog_interface.h"
 #include "mongo/db/repl/oplog_interface_local.h"
+#include "mongo/db/repl/optime_with.h"
+#include "mongo/db/repl/read_concern_args.h"
+#include "mongo/db/repl/read_concern_level.h"
+#include "mongo/db/repl/replication_coordinator_mock.h"
 #include "mongo/db/repl/wait_for_majority_service.h"
-#include "mongo/db/s/collection_sharding_runtime.h"
 #include "mongo/db/s/operation_sharding_state.h"
 #include "mongo/db/s/shard_filtering_metadata_refresh.h"
 #include "mongo/db/s/shard_server_test_fixture.h"
-#include "mongo/db/s/sharding_state.h"
+#include "mongo/db/s/sharding_mongod_test_fixture.h"
 #include "mongo/db/s/sharding_write_router.h"
+#include "mongo/db/server_options.h"
+#include "mongo/db/session/logical_session_id.h"
 #include "mongo/db/session/session_catalog_mongod.h"
 #include "mongo/db/shard_id.h"
+#include "mongo/db/shard_role.h"
+#include "mongo/db/storage/write_unit_of_work.h"
 #include "mongo/db/transaction/transaction_participant.h"
+#include "mongo/db/transaction_resources.h"
+#include "mongo/idl/idl_parser.h"
+#include "mongo/s/catalog/sharding_catalog_client.h"
 #include "mongo/s/catalog/sharding_catalog_client_mock.h"
+#include "mongo/s/catalog/type_chunk.h"
+#include "mongo/s/catalog/type_collection.h"
+#include "mongo/s/catalog/type_database_gen.h"
+#include "mongo/s/catalog/type_index_catalog_gen.h"
 #include "mongo/s/catalog/type_shard.h"
+#include "mongo/s/catalog_cache.h"
+#include "mongo/s/catalog_cache_loader.h"
 #include "mongo/s/catalog_cache_loader_mock.h"
+#include "mongo/s/chunk_version.h"
 #include "mongo/s/database_version.h"
+#include "mongo/s/index_version.h"
+#include "mongo/s/resharding/common_types_gen.h"
+#include "mongo/s/resharding/type_collection_fields_gen.h"
 #include "mongo/s/shard_cannot_refresh_due_to_locks_held_exception.h"
+#include "mongo/s/shard_version.h"
 #include "mongo/s/shard_version_factory.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/fail_point.h"
+#include "mongo/util/net/hostandport.h"
+#include "mongo/util/time_support.h"
+#include "mongo/util/uuid.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
 
@@ -71,8 +130,10 @@ void runInTransaction(OperationContext* opCtx, Callable&& func) {
 
     auto txnParticipant = TransactionParticipant::get(opCtx);
     ASSERT(txnParticipant);
-    txnParticipant.beginOrContinue(
-        opCtx, {txnNum}, false /* autocommit */, true /* startTransaction */);
+    txnParticipant.beginOrContinue(opCtx,
+                                   {txnNum},
+                                   false /* autocommit */,
+                                   TransactionParticipant::TransactionActions::kStart);
     txnParticipant.unstashTransactionResources(opCtx, "SetDestinedRecipient");
 
     func();
@@ -81,33 +142,15 @@ void runInTransaction(OperationContext* opCtx, Callable&& func) {
     txnParticipant.stashTransactionResources(opCtx);
 }
 
-class DestinedRecipientTest : public ShardServerTestFixture {
+class DestinedRecipientTest : public ShardServerTestFixtureWithCatalogCacheLoaderMock {
 public:
     const NamespaceString kNss = NamespaceString::createNamespaceString_forTest("test.foo");
     const std::string kShardKey = "x";
-    const HostAndPort kConfigHostAndPort{"DummyConfig", 12345};
-    const std::vector<ShardType> kShardList = {ShardType("shard0", "Host0:12345"),
+    const std::vector<ShardType> kShardList = {ShardType(kMyShardName.toString(), "Host0:12345"),
                                                ShardType("shard1", "Host1:12345")};
 
     void setUp() override {
-        // Don't call ShardServerTestFixture::setUp so we can install a mock catalog cache loader.
-        ShardingMongodTestFixture::setUp();
-
-        replicationCoordinator()->alwaysAllowWrites(true);
-        serverGlobalParams.clusterRole = ClusterRole::ShardServer;
-
-        _clusterId = OID::gen();
-        ShardingState::get(getServiceContext())
-            ->setInitialized(kShardList[0].getName(), _clusterId);
-
-        auto mockLoader = std::make_unique<CatalogCacheLoaderMock>();
-        _mockCatalogCacheLoader = mockLoader.get();
-        CatalogCacheLoader::set(getServiceContext(), std::move(mockLoader));
-
-        uassertStatusOK(
-            initializeGlobalShardingStateForMongodForTest(ConnectionString(kConfigHostAndPort)));
-
-        configTargeterMock()->setFindHostReturnValue(kConfigHostAndPort);
+        ShardServerTestFixtureWithCatalogCacheLoaderMock::setUp();
 
         WaitForMajorityService::get(getServiceContext()).startup(getServiceContext());
 
@@ -124,7 +167,7 @@ public:
     void tearDown() override {
         WaitForMajorityService::get(getServiceContext()).shutDown();
 
-        ShardServerTestFixture::tearDown();
+        ShardServerTestFixtureWithCatalogCacheLoaderMock::tearDown();
     }
 
     class StaticCatalogClient final : public ShardingCatalogClientMock {
@@ -137,9 +180,15 @@ public:
             bool excludeDraining) override {
             return repl::OpTimeWith<std::vector<ShardType>>(_shards);
         }
+        std::vector<CollectionType> getShardedCollections(OperationContext* opCtx,
+                                                          const DatabaseName& dbName,
+                                                          repl::ReadConcernLevel readConcernLevel,
+                                                          const BSONObj& sort) override {
+            return {};
+        }
 
         std::vector<CollectionType> getCollections(OperationContext* opCtx,
-                                                   StringData dbName,
+                                                   const DatabaseName& dbName,
                                                    repl::ReadConcernLevel readConcernLevel,
                                                    const BSONObj& sort) override {
             return _colls;
@@ -212,7 +261,7 @@ protected:
         env.version = ShardVersionFactory::make(ChunkVersion(gen, {1, 0}),
                                                 boost::optional<CollectionIndexes>(boost::none));
         env.tempNss = NamespaceString::createNamespaceString_forTest(
-            kNss.db(),
+            kNss.db_forTest(),
             fmt::format("{}{}",
                         NamespaceString::kTemporaryReshardingCollectionPrefix,
                         env.sourceUuid.toString()));
@@ -236,9 +285,9 @@ protected:
                             BSON(kShardKey << 1));
         coll.setAllowMigrations(false);
 
-        _mockCatalogCacheLoader->setDatabaseRefreshReturnValue(
-            DatabaseType(kNss.db().toString(), kShardList[0].getName(), env.dbVersion));
-        _mockCatalogCacheLoader->setCollectionRefreshValues(
+        getCatalogCacheLoaderMock()->setDatabaseRefreshReturnValue(
+            DatabaseType(kNss.dbName(), kShardList[0].getName(), env.dbVersion));
+        getCatalogCacheLoaderMock()->setCollectionRefreshValues(
             kNss,
             coll,
             createChunks(env.version.placementVersion().epoch(),
@@ -246,7 +295,7 @@ protected:
                          env.version.placementVersion().getTimestamp(),
                          kShardKey),
             reshardingFields);
-        _mockCatalogCacheLoader->setCollectionRefreshValues(
+        getCatalogCacheLoaderMock()->setCollectionRefreshValues(
             env.tempNss,
             coll,
             createChunks(env.version.placementVersion().epoch(),
@@ -255,7 +304,7 @@ protected:
                          "y"),
             boost::none);
 
-        ASSERT_OK(onDbVersionMismatchNoExcept(opCtx, kNss.db(), boost::none));
+        ASSERT_OK(onDbVersionMismatchNoExcept(opCtx, kNss.dbName(), boost::none));
         forceShardFilteringMetadataRefresh(opCtx, kNss);
 
         if (refreshTempNss)
@@ -279,8 +328,11 @@ protected:
                    const BSONObj& filter,
                    const BSONObj& update,
                    const ReshardingEnv& env) {
-        AutoGetCollection coll(opCtx, nss, MODE_IX);
-        Helpers::update(opCtx, nss, filter, update);
+        auto coll = acquireCollection(
+            opCtx,
+            CollectionAcquisitionRequest::fromOpCtx(opCtx, nss, AcquisitionPrerequisites::kWrite),
+            MODE_IX);
+        Helpers::update(opCtx, coll, filter, update);
     }
 
     void deleteDoc(OperationContext* opCtx,
@@ -305,9 +357,6 @@ protected:
         const auto& doc = unittest::assertGet(oplogIter->next()).first;
         return unittest::assertGet(repl::OplogEntry::parse(doc));
     }
-
-protected:
-    CatalogCacheLoaderMock* _mockCatalogCacheLoader;
 };
 
 TEST_F(DestinedRecipientTest, TestGetDestinedRecipient) {
@@ -316,7 +365,7 @@ TEST_F(DestinedRecipientTest, TestGetDestinedRecipient) {
 
     AutoGetCollection coll(opCtx, kNss, MODE_IX);
     OperationShardingState::setShardRole(opCtx, kNss, env.version, env.dbVersion);
-    ShardingWriteRouter shardingWriteRouter(opCtx, kNss, Grid::get(opCtx)->catalogCache());
+    ShardingWriteRouter shardingWriteRouter(opCtx, kNss);
 
     auto destShardId =
         shardingWriteRouter.getReshardingDestinedRecipient(BSON("x" << 2 << "y" << 10));
@@ -332,7 +381,7 @@ TEST_F(DestinedRecipientTest, TestGetDestinedRecipientThrowsOnBlockedRefresh) {
     OperationShardingState::setShardRole(opCtx, kNss, env.version, env.dbVersion);
 
     FailPointEnableBlock failPoint("blockCollectionCacheLookup");
-    ASSERT_THROWS_WITH_CHECK(ShardingWriteRouter(opCtx, kNss, Grid::get(opCtx)->catalogCache()),
+    ASSERT_THROWS_WITH_CHECK(ShardingWriteRouter(opCtx, kNss),
                              ShardCannotRefreshDueToLocksHeldException,
                              [&](const ShardCannotRefreshDueToLocksHeldException& ex) {
                                  const auto refreshInfo =
