@@ -27,38 +27,89 @@
  *    it in the license file.
  */
 
+#include <algorithm>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/smart_ptr.hpp>
+#include <deque>
 #include <fmt/format.h>
+#include <functional>
+#include <iterator>
+#include <limits>
+#include <string>
+#include <tuple>
+#include <utility>
 
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/oid.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/client/connection_string.h"
+#include "mongo/db/auth/authorization_session.h"
 #include "mongo/db/cancelable_operation_context.h"
 #include "mongo/db/catalog/create_collection.h"
-#include "mongo/db/catalog_raii.h"
-#include "mongo/db/db_raii.h"
+#include "mongo/db/client.h"
+#include "mongo/db/cluster_role.h"
 #include "mongo/db/dbdirectclient.h"
-#include "mongo/db/repl/oplog_applier.h"
-#include "mongo/db/repl/session_update_tracker.h"
-#include "mongo/db/repl/storage_interface_impl.h"
-#include "mongo/db/s/collection_sharding_runtime.h"
+#include "mongo/db/exec/document_value/value.h"
+#include "mongo/db/query/collation/collator_interface.h"
+#include "mongo/db/repl/oplog_entry_gen.h"
+#include "mongo/db/repl/optime.h"
+#include "mongo/db/repl/optime_with.h"
+#include "mongo/db/repl/read_concern_args.h"
+#include "mongo/db/repl/read_concern_level.h"
+#include "mongo/db/s/metrics/sharding_data_transform_instance_metrics.h"
 #include "mongo/db/s/operation_sharding_state.h"
+#include "mongo/db/s/resharding/donor_oplog_id_gen.h"
 #include "mongo/db/s/resharding/resharding_donor_oplog_iterator.h"
+#include "mongo/db/s/resharding/resharding_metrics.h"
 #include "mongo/db/s/resharding/resharding_oplog_applier.h"
-#include "mongo/db/s/resharding/resharding_server_parameters_gen.h"
-#include "mongo/db/s/resharding/resharding_util.h"
 #include "mongo/db/s/sharding_mongod_test_fixture.h"
-#include "mongo/db/s/sharding_state.h"
+#include "mongo/db/server_options.h"
+#include "mongo/db/session/logical_session_cache.h"
 #include "mongo/db/session/logical_session_cache_noop.h"
+#include "mongo/db/session/logical_session_id.h"
+#include "mongo/db/session/logical_session_id_gen.h"
 #include "mongo/db/session/session_catalog_mongod.h"
-#include "mongo/db/transaction/transaction_participant.h"
+#include "mongo/db/update/document_diff_serialization.h"
 #include "mongo/db/update/update_oplog_entry_serialization.h"
 #include "mongo/db/vector_clock_metadata_hook.h"
+#include "mongo/executor/network_connection_hook.h"
 #include "mongo/executor/network_interface_factory.h"
-#include "mongo/executor/thread_pool_task_executor_test_fixture.h"
-#include "mongo/idl/server_parameter_test_util.h"
-#include "mongo/logv2/log.h"
+#include "mongo/executor/thread_pool_task_executor.h"
 #include "mongo/rpc/metadata/egress_metadata_hook_list.h"
+#include "mongo/rpc/metadata/metadata_hook.h"
+#include "mongo/s/catalog/sharding_catalog_client.h"
 #include "mongo/s/catalog/sharding_catalog_client_mock.h"
+#include "mongo/s/catalog/type_chunk.h"
+#include "mongo/s/catalog/type_collection.h"
+#include "mongo/s/catalog/type_database_gen.h"
+#include "mongo/s/catalog/type_index_catalog_gen.h"
+#include "mongo/s/catalog/type_shard.h"
+#include "mongo/s/catalog_cache_loader.h"
 #include "mongo/s/catalog_cache_loader_mock.h"
-#include "mongo/unittest/unittest.h"
-#include "mongo/util/str.h"
+#include "mongo/s/chunk_version.h"
+#include "mongo/s/database_version.h"
+#include "mongo/s/resharding/type_collection_fields_gen.h"
+#include "mongo/s/sharding_state.h"
+#include "mongo/s/type_collection_common_types_gen.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/bson_test_util.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/clock_source.h"
+#include "mongo/util/concurrency/thread_pool.h"
+#include "mongo/util/future_impl.h"
+#include "mongo/util/net/hostandport.h"
+#include "mongo/util/out_of_line_executor.h"
+#include "mongo/util/time_support.h"
 #include "mongo/util/uuid.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
@@ -112,20 +163,20 @@ private:
     bool _doThrow{false};
 };
 
-class ReshardingOplogApplierTest : public ShardingMongodTestFixture {
+class ReshardingOplogApplierTest : public ShardingMongoDTestFixture {
 public:
     const HostAndPort kConfigHostAndPort{"DummyConfig", 12345};
     const std::string kOriginalShardKey = "sk";
     const BSONObj kOriginalShardKeyPattern{BSON(kOriginalShardKey << 1)};
 
     void setUp() override {
-        ShardingMongodTestFixture::setUp();
+        ShardingMongoDTestFixture::setUp();
 
-        serverGlobalParams.clusterRole = ClusterRole::ShardServer;
-
-        auto clusterId = OID::gen();
         ShardingState::get(getServiceContext())
-            ->setInitialized(_sourceId.getShardId().toString(), clusterId);
+            ->setRecoveryCompleted({OID::gen(),
+                                    ClusterRole::ShardServer,
+                                    ConnectionString(kConfigHostAndPort),
+                                    _sourceId.getShardId()});
 
         auto mockLoader = std::make_unique<CatalogCacheLoaderMock>();
         _mockCatalogCacheLoader = mockLoader.get();
@@ -172,14 +223,14 @@ public:
         _cancelableOpCtxExecutor->startup();
     }
 
-    void tearDown() {
+    void tearDown() override {
         _executor->shutdown();
         _executor->join();
 
         _cancelableOpCtxExecutor->shutdown();
         _cancelableOpCtxExecutor->join();
 
-        ShardingMongodTestFixture::tearDown();
+        ShardingMongoDTestFixture::tearDown();
     }
 
     class StaticCatalogClient final : public ShardingCatalogClientMock {
@@ -193,8 +244,15 @@ public:
             return repl::OpTimeWith<std::vector<ShardType>>(_shards);
         }
 
+        std::vector<CollectionType> getShardedCollections(OperationContext* opCtx,
+                                                          const DatabaseName& dbName,
+                                                          repl::ReadConcernLevel readConcernLevel,
+                                                          const BSONObj& sort) override {
+            return {};
+        }
+
         std::vector<CollectionType> getCollections(OperationContext* opCtx,
-                                                   StringData dbName,
+                                                   const DatabaseName& dbName,
                                                    repl::ReadConcernLevel readConcernLevel,
                                                    const BSONObj& sort) override {
             return _colls;
@@ -246,6 +304,7 @@ public:
         auto rt = RoutingTableHistory::makeNew(kCrudNs,
                                                kCrudUUID,
                                                kOriginalShardKeyPattern,
+                                               false, /* unsplittable */
                                                nullptr,
                                                false,
                                                epoch,
@@ -264,7 +323,7 @@ public:
 
     void loadCatalogCacheValues() {
         _mockCatalogCacheLoader->setDatabaseRefreshReturnValue(
-            DatabaseType(kAppliedToNs.db().toString(), _cm->dbPrimary(), _cm->dbVersion()));
+            DatabaseType(kAppliedToNs.dbName(), _cm->dbPrimary(), _cm->dbVersion()));
         std::vector<ChunkType> chunks;
         _cm->forEachChunk([&](const auto& chunk) {
             chunks.emplace_back(
@@ -375,14 +434,9 @@ protected:
         threadPoolOptions.threadNamePrefix = "TestReshardOplogApplication-";
         threadPoolOptions.poolName = "TestReshardOplogApplicationThreadPool";
         threadPoolOptions.onCreateThread = [](const std::string& threadName) {
-            Client::initThread(threadName.c_str());
+            Client::initThread(threadName.c_str(), getGlobalServiceContext()->getService());
             auto* client = Client::getCurrent();
             AuthorizationSession::get(*client)->grantInternalAuthorization(client);
-
-            {
-                stdx::lock_guard<Client> lk(*client);
-                client->setSystemOperationKillableByStepdown(lk);
-            }
         };
 
         auto hookList = std::make_unique<rpc::EgressMetadataHookList>();
@@ -406,7 +460,9 @@ protected:
         }());
     }
 
-    static constexpr int kWriterPoolSize = 4;
+    static constexpr size_t kWriterPoolSize = 4;
+    static constexpr size_t kApplierBatchTaskCount = kWriterPoolSize;
+
     const NamespaceString kOplogNs =
         NamespaceString::createNamespaceString_forTest("config.localReshardingOplogBuffer.xxx.yyy");
     const NamespaceString kCrudNs = NamespaceString::createNamespaceString_forTest("foo.bar");
@@ -422,10 +478,13 @@ protected:
     const ShardId kOtherShardId{"shard2"};
     const std::vector<ShardType> kShardList = {ShardType(kMyShardId.toString(), "Host0:12345"),
                                                ShardType(kOtherShardId.toString(), "Host1:12345")};
+    const ReshardingSourceId _sourceId{UUID::gen(), kMyShardId};
+
+    service_context_test::ShardRoleOverride _shardRole;
+
     boost::optional<ChunkManager> _cm;
     CatalogCacheLoaderMock* _mockCatalogCacheLoader;
 
-    const ReshardingSourceId _sourceId{UUID::gen(), kMyShardId};
     std::unique_ptr<ReshardingMetrics> _metrics;
     std::unique_ptr<ReshardingOplogApplierMetrics> _applierMetrics;
 
@@ -440,6 +499,7 @@ TEST_F(ReshardingOplogApplierTest, NothingToIterate) {
     boost::optional<ReshardingOplogApplier> applier;
 
     applier.emplace(makeApplierEnv(),
+                    kApplierBatchTaskCount,
                     sourceId(),
                     oplogBufferNs(),
                     appliedToNs(),
@@ -478,6 +538,7 @@ TEST_F(ReshardingOplogApplierTest, ApplyBasicCrud) {
     auto iterator = std::make_unique<OplogIteratorMock>(std::move(crudOps), 2 /* batchSize */);
     boost::optional<ReshardingOplogApplier> applier;
     applier.emplace(makeApplierEnv(),
+                    kApplierBatchTaskCount,
                     sourceId(),
                     oplogBufferNs(),
                     appliedToNs(),
@@ -524,6 +585,7 @@ TEST_F(ReshardingOplogApplierTest, CanceledApplyingBatch) {
     boost::optional<ReshardingOplogApplier> applier;
 
     applier.emplace(makeApplierEnv(),
+                    kApplierBatchTaskCount,
                     sourceId(),
                     oplogBufferNs(),
                     appliedToNs(),
@@ -556,6 +618,7 @@ TEST_F(ReshardingOplogApplierTest, InsertTypeOplogAppliedInMultipleBatches) {
     auto iterator = std::make_unique<OplogIteratorMock>(std::move(crudOps), 3 /* batchSize */);
     boost::optional<ReshardingOplogApplier> applier;
     applier.emplace(makeApplierEnv(),
+                    kApplierBatchTaskCount,
                     sourceId(),
                     oplogBufferNs(),
                     appliedToNs(),
@@ -598,6 +661,7 @@ TEST_F(ReshardingOplogApplierTest, ErrorDuringFirstBatchApply) {
     auto iterator = std::make_unique<OplogIteratorMock>(std::move(crudOps), 4 /* batchSize */);
     boost::optional<ReshardingOplogApplier> applier;
     applier.emplace(makeApplierEnv(),
+                    kApplierBatchTaskCount,
                     sourceId(),
                     oplogBufferNs(),
                     appliedToNs(),
@@ -642,6 +706,7 @@ TEST_F(ReshardingOplogApplierTest, ErrorDuringSecondBatchApply) {
     auto iterator = std::make_unique<OplogIteratorMock>(std::move(crudOps), 2 /* batchSize */);
     boost::optional<ReshardingOplogApplier> applier;
     applier.emplace(makeApplierEnv(),
+                    kApplierBatchTaskCount,
                     sourceId(),
                     oplogBufferNs(),
                     appliedToNs(),
@@ -684,6 +749,7 @@ TEST_F(ReshardingOplogApplierTest, ErrorWhileIteratingFirstOplog) {
 
     boost::optional<ReshardingOplogApplier> applier;
     applier.emplace(makeApplierEnv(),
+                    kApplierBatchTaskCount,
                     sourceId(),
                     oplogBufferNs(),
                     appliedToNs(),
@@ -721,6 +787,7 @@ TEST_F(ReshardingOplogApplierTest, ErrorWhileIteratingFirstBatch) {
 
     boost::optional<ReshardingOplogApplier> applier;
     applier.emplace(makeApplierEnv(),
+                    kApplierBatchTaskCount,
                     sourceId(),
                     oplogBufferNs(),
                     appliedToNs(),
@@ -764,6 +831,7 @@ TEST_F(ReshardingOplogApplierTest, ErrorWhileIteratingSecondBatch) {
 
     boost::optional<ReshardingOplogApplier> applier;
     applier.emplace(makeApplierEnv(),
+                    kApplierBatchTaskCount,
                     sourceId(),
                     oplogBufferNs(),
                     appliedToNs(),
@@ -804,6 +872,7 @@ TEST_F(ReshardingOplogApplierTest, ExecutorIsShutDown) {
     auto iterator = std::make_unique<OplogIteratorMock>(std::move(crudOps), 4 /* batchSize */);
     boost::optional<ReshardingOplogApplier> applier;
     applier.emplace(makeApplierEnv(),
+                    kApplierBatchTaskCount,
                     sourceId(),
                     oplogBufferNs(),
                     appliedToNs(),
@@ -834,11 +903,11 @@ TEST_F(ReshardingOplogApplierTest, UnsupportedCommandOpsShouldError) {
                             repl::OpTypeEnum::kInsert,
                             BSON("_id" << 1),
                             boost::none));
-    ops.push_back(
-        makeOplog(repl::OpTime(Timestamp(6, 3), 1),
-                  repl::OpTypeEnum::kCommand,
-                  BSON("renameCollection" << appliedToNs().ns() << "to" << stashNs().ns()),
-                  boost::none));
+    ops.push_back(makeOplog(
+        repl::OpTime(Timestamp(6, 3), 1),
+        repl::OpTypeEnum::kCommand,
+        BSON("renameCollection" << appliedToNs().ns_forTest() << "to" << stashNs().ns_forTest()),
+        boost::none));
     ops.push_back(makeOplog(repl::OpTime(Timestamp(7, 3), 1),
                             repl::OpTypeEnum::kInsert,
                             BSON("_id" << 2),
@@ -847,6 +916,7 @@ TEST_F(ReshardingOplogApplierTest, UnsupportedCommandOpsShouldError) {
     auto iterator = std::make_unique<OplogIteratorMock>(std::move(ops), 1 /* batchSize */);
     boost::optional<ReshardingOplogApplier> applier;
     applier.emplace(makeApplierEnv(),
+                    kApplierBatchTaskCount,
                     sourceId(),
                     oplogBufferNs(),
                     appliedToNs(),
@@ -878,12 +948,13 @@ TEST_F(ReshardingOplogApplierTest, DropSourceCollectionCmdShouldError) {
     std::deque<repl::OplogEntry> ops;
     ops.push_back(makeOplog(repl::OpTime(Timestamp(5, 3), 1),
                             repl::OpTypeEnum::kCommand,
-                            BSON("drop" << appliedToNs().ns()),
+                            BSON("drop" << appliedToNs().ns_forTest()),
                             boost::none));
 
     auto iterator = std::make_unique<OplogIteratorMock>(std::move(ops), 1 /* batchSize */);
     boost::optional<ReshardingOplogApplier> applier;
     applier.emplace(makeApplierEnv(),
+                    kApplierBatchTaskCount,
                     sourceId(),
                     oplogBufferNs(),
                     appliedToNs(),
@@ -922,6 +993,7 @@ TEST_F(ReshardingOplogApplierTest, MetricsAreReported) {
             easyOp(9, OpT::kInsert, BSON("_id" << 3))},
         2);
     ReshardingOplogApplier applier(makeApplierEnv(),
+                                   4,
                                    sourceId(),
                                    oplogBufferNs(),
                                    appliedToNs(),
