@@ -29,36 +29,86 @@
 
 #include "mongo/db/s/resharding/resharding_oplog_fetcher.h"
 
+#include <absl/container/node_hash_map.h>
+#include <boost/cstdint.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <cstdint>
 #include <fmt/format.h>
+#include <mutex>
+#include <tuple>
 #include <vector>
 
+#include <boost/move/utility_core.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
 #include "mongo/bson/bsonobj.h"
-#include "mongo/client/dbclient_connection.h"
-#include "mongo/client/remote_command_targeter.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/client/read_preference.h"
+#include "mongo/db/basic_types_gen.h"
+#include "mongo/db/catalog/clustered_collection_options_gen.h"
 #include "mongo/db/catalog/clustered_collection_util.h"
-#include "mongo/db/catalog/collection_write_path.h"
+#include "mongo/db/catalog/collection.h"
+#include "mongo/db/catalog/collection_catalog.h"
+#include "mongo/db/catalog/collection_options.h"
+#include "mongo/db/catalog/database.h"
 #include "mongo/db/catalog_raii.h"
+#include "mongo/db/concurrency/d_concurrency.h"
 #include "mongo/db/concurrency/exception_util.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
+#include "mongo/db/dbhelpers.h"
+#include "mongo/db/exec/document_value/document.h"
+#include "mongo/db/exec/document_value/value.h"
+#include "mongo/db/logical_time.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/pipeline/aggregate_command_gen.h"
+#include "mongo/db/pipeline/expression_context.h"
+#include "mongo/db/pipeline/pipeline.h"
+#include "mongo/db/pipeline/process_interface/mongo_process_interface.h"
+#include "mongo/db/repl/oplog.h"
+#include "mongo/db/repl/oplog_entry.h"
+#include "mongo/db/repl/oplog_entry_gen.h"
 #include "mongo/db/repl/read_concern_args.h"
 #include "mongo/db/repl/read_concern_level.h"
-#include "mongo/db/s/metrics/sharding_data_transform_cumulative_metrics.h"
 #include "mongo/db/s/resharding/resharding_metrics.h"
 #include "mongo/db/s/resharding/resharding_util.h"
+#include "mongo/db/shard_role.h"
 #include "mongo/db/storage/write_unit_of_work.h"
+#include "mongo/db/transaction_resources.h"
+#include "mongo/db/write_concern_options.h"
 #include "mongo/executor/task_executor.h"
+#include "mongo/idl/idl_parser.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/s/client/shard_registry.h"
+#include "mongo/s/database_version.h"
 #include "mongo/s/grid.h"
-#include "mongo/stdx/mutex.h"
+#include "mongo/s/shard_version.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/clock_source.h"
+#include "mongo/util/duration.h"
+#include "mongo/util/intrusive_counter.h"
+#include "mongo/util/out_of_line_executor.h"
+#include "mongo/util/string_map.h"
+#include "mongo/util/timer.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kResharding
 
 namespace mongo {
 namespace {
 boost::intrusive_ptr<ExpressionContext> _makeExpressionContext(OperationContext* opCtx) {
-    ResolvedNamespaceMap resolvedNamespaces;
-    resolvedNamespaces[NamespaceString::kRsOplogNamespace] = {NamespaceString::kRsOplogNamespace,
-                                                              std::vector<BSONObj>()};
+    StringMap<ExpressionContext::ResolvedNamespace> resolvedNamespaces;
+    resolvedNamespaces[NamespaceString::kRsOplogNamespace.coll()] = {
+        NamespaceString::kRsOplogNamespace, std::vector<BSONObj>()};
     return make_intrusive<ExpressionContext>(opCtx,
                                              boost::none, /* explain */
                                              false,       /* fromMongos */
@@ -145,7 +195,7 @@ ExecutorFuture<void> ReshardingOplogFetcher::schedule(
     }
 
     return ExecutorFuture(executor)
-        .then([this, executor, cancelToken, factory] {
+        .then([this, executor, cancelToken, factory]() mutable {
             return _reschedule(std::move(executor), cancelToken, factory);
         })
         .onError([](Status status) {
@@ -163,7 +213,14 @@ ExecutorFuture<void> ReshardingOplogFetcher::_reschedule(
             ThreadClient client(fmt::format("OplogFetcher-{}-{}",
                                             _reshardingUUID.toString(),
                                             _donorShard.toString()),
-                                _service());
+                                _service()->getService(ClusterRole::ShardServer));
+
+            // TODO(SERVER-74658): Please revisit if this thread could be made killable.
+            {
+                stdx::lock_guard<Client> lk(*client.get());
+                client.get()->setSystemOperationUnkillableByStepdown(lk);
+            }
+
             return iterate(client.get(), factory);
         })
         .then([executor, cancelToken](bool moreToCome) {
@@ -174,7 +231,7 @@ ExecutorFuture<void> ReshardingOplogFetcher::_reschedule(
                 return moreToCome;
             });
         })
-        .then([this, executor, cancelToken, factory](bool moreToCome) {
+        .then([this, executor, cancelToken, factory](bool moreToCome) mutable {
             if (!moreToCome) {
                 LOGV2_INFO(6077401,
                            "Resharding oplog fetcher done fetching",
@@ -185,7 +242,7 @@ ExecutorFuture<void> ReshardingOplogFetcher::_reschedule(
 
             if (cancelToken.isCanceled()) {
                 return ExecutorFuture<void>(
-                    executor,
+                    std::move(executor),
                     Status{ErrorCodes::CallbackCanceled,
                            "Resharding oplog fetcher canceled due to abort or stepdown"});
             }
@@ -231,10 +288,10 @@ void ReshardingOplogFetcher::_ensureCollection(Client* client,
                                                const NamespaceString nss) {
     auto opCtxRaii = factory.makeOperationContext(client);
     auto opCtx = opCtxRaii.get();
-    invariant(!opCtx->lockState()->inAWriteUnitOfWork());
+    invariant(!shard_role_details::getLocker(opCtx)->inAWriteUnitOfWork());
 
     // Create the destination collection if necessary.
-    writeConflictRetry(opCtx, "createReshardingLocalOplogBuffer", nss.toString(), [&] {
+    writeConflictRetry(opCtx, "createReshardingLocalOplogBuffer", nss, [&] {
         const Collection* coll =
             CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, nss);
         if (coll) {
@@ -322,18 +379,32 @@ bool ReshardingOplogFetcher::consume(Client* client,
             ThreadClient client(fmt::format("ReshardingFetcher-{}-{}",
                                             _reshardingUUID.toString(),
                                             _donorShard.toString()),
-                                _service(),
+                                _service()->getService(ClusterRole::ShardServer),
                                 nullptr);
             auto opCtxRaii = factory.makeOperationContext(client.get());
             auto opCtx = opCtxRaii.get();
 
+            // TODO(SERVER-74658): Please revisit if this thread could be made killable.
+            {
+                stdx::lock_guard<Client> lk(*client.get());
+                client.get()->setSystemOperationUnkillableByStepdown(lk);
+            }
+
             // Noting some possible optimizations:
             //
             // * Batch more inserts into larger storage transactions.
             // * Parallize writing documents across multiple threads.
             // * Doing either of the above while still using the underlying message buffer of bson
             //   objects.
-            AutoGetCollection toWriteTo(opCtx, _toWriteInto, LockMode::MODE_IX);
+            const auto toWriteTo =
+                acquireCollection(opCtx,
+                                  CollectionAcquisitionRequest(
+                                      _toWriteInto,
+                                      PlacementConcern{boost::none, ShardVersion::UNSHARDED()},
+                                      repl::ReadConcernArgs::get(opCtx),
+                                      AcquisitionPrerequisites::kWrite),
+                                  MODE_IX);
+
             for (const BSONObj& doc : batch) {
                 WriteUnitOfWork wuow(opCtx);
                 auto nextOplog = uassertStatusOK(repl::OplogEntry::parse(doc));
@@ -342,8 +413,7 @@ bool ReshardingOplogFetcher::consume(Client* client,
                     ReshardingDonorOplogId::parse(IDLParserContext{"OplogFetcherParsing"},
                                                   nextOplog.get_id()->getDocument().toBson());
                 Timer insertTimer;
-                uassertStatusOK(collection_internal::insertDocument(
-                    opCtx, *toWriteTo, InsertStatement{doc}, nullptr));
+                uassertStatusOK(Helpers::insert(opCtx, toWriteTo, doc));
                 wuow.commit();
 
                 _env->metrics()->onLocalInsertDuringOplogFetching(
@@ -373,36 +443,46 @@ bool ReshardingOplogFetcher::consume(Client* client,
                 // wasteful.
                 try {
                     auto lastOplogTs = postBatchResumeToken->getField("ts").timestamp();
-                    auto startAt = ReshardingDonorOplogId(lastOplogTs, lastOplogTs);
+                    auto newStartAt = ReshardingDonorOplogId(lastOplogTs, lastOplogTs);
 
-                    WriteUnitOfWork wuow(opCtx);
+                    // If newStartAt == _startAt then inserting a reshardProgressMark is not needed.
+                    // This is because:
+                    //   1. There is already an oplog with timestamp == newStartAt in the toWriteTo
+                    //      collection from which the fetcher knows to resume from.
+                    //   2. Or newStartAt equals the initial value for _startAt (typically the
+                    //      minFetchTimestamp) in which case, the fetcher is going to resume from
+                    //      there anyways.
+                    if (newStartAt != _startAt) {
+                        WriteUnitOfWork wuow(opCtx);
 
-                    repl::MutableOplogEntry oplog;
-                    oplog.setNss(_toWriteInto);
-                    oplog.setOpType(repl::OpTypeEnum::kNoop);
-                    oplog.setUuid(_collUUID);
-                    oplog.set_id(Value(startAt.toBSON()));
-                    oplog.setObject(BSON("msg"
-                                         << "Latest oplog ts from donor's cursor response"));
-                    oplog.setObject2(BSON("type" << resharding::kReshardProgressMark));
-                    oplog.setOpTime(OplogSlot());
-                    oplog.setWallClockTime(opCtx->getServiceContext()->getFastClockSource()->now());
+                        repl::MutableOplogEntry oplog;
+                        oplog.setNss(_toWriteInto);
+                        oplog.setOpType(repl::OpTypeEnum::kNoop);
+                        oplog.setUuid(_collUUID);
+                        oplog.set_id(Value(newStartAt.toBSON()));
+                        oplog.setObject(BSON("msg"
+                                             << "Latest oplog ts from donor's cursor response"));
+                        oplog.setObject2(BSON("type" << resharding::kReshardProgressMark));
+                        oplog.setOpTime(OplogSlot());
+                        oplog.setWallClockTime(
+                            opCtx->getServiceContext()->getFastClockSource()->now());
 
-                    uassertStatusOK(collection_internal::insertDocument(
-                        opCtx, *toWriteTo, InsertStatement{oplog.toBSON()}, nullptr));
-                    wuow.commit();
+                        uassertStatusOK(Helpers::insert(opCtx, toWriteTo, oplog.toBSON()));
+                        wuow.commit();
 
-                    // Also include synthetic oplog in the fetched count so it can match up with the
-                    // total oplog applied count in the end.
-                    _env->metrics()->onOplogEntriesFetched(1);
+                        // Also include synthetic oplog in the fetched count
+                        // so it can match up with the total oplog applied
+                        // count in the end.
+                        _env->metrics()->onOplogEntriesFetched(1);
 
-                    auto [p, f] = makePromiseFuture<void>();
-                    {
-                        stdx::lock_guard lk(_mutex);
-                        _startAt = startAt;
-                        _onInsertPromise.emplaceValue();
-                        _onInsertPromise = std::move(p);
-                        _onInsertFuture = std::move(f);
+                        auto [p, f] = makePromiseFuture<void>();
+                        {
+                            stdx::lock_guard lk(_mutex);
+                            _startAt = newStartAt;
+                            _onInsertPromise.emplaceValue();
+                            _onInsertPromise = std::move(p);
+                            _onInsertFuture = std::move(f);
+                        }
                     }
                 } catch (const ExceptionFor<ErrorCodes::DuplicateKey>&) {
                     // It's possible that the donor shard has not generated new oplog entries since
