@@ -29,45 +29,93 @@
 
 #include "mongo/db/s/resharding/resharding_recipient_service.h"
 
+#include "mongo/s/resharding/common_types_gen.h"
+#include <absl/container/node_hash_map.h>
 #include <algorithm>
+#include <boost/cstdint.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <mutex>
+#include <string>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status_with.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/db/cancelable_operation_context.h"
-#include "mongo/db/catalog/rename_collection.h"
+#include "mongo/db/catalog/collection.h"
+#include "mongo/db/catalog/collection_options.h"
+#include "mongo/db/catalog/commit_quorum_options.h"
+#include "mongo/db/catalog/index_catalog.h"
 #include "mongo/db/catalog_raii.h"
+#include "mongo/db/client.h"
 #include "mongo/db/concurrency/exception_util.h"
-#include "mongo/db/dbdirectclient.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
 #include "mongo/db/dbhelpers.h"
+#include "mongo/db/feature_flag.h"
+#include "mongo/db/index_builds_coordinator.h"
+#include "mongo/db/keypattern.h"
 #include "mongo/db/ops/delete.h"
 #include "mongo/db/persistent_task_store.h"
+#include "mongo/db/query/collation/collation_spec.h"
 #include "mongo/db/repl/change_stream_oplog_notification.h"
-#include "mongo/db/repl/oplog_applier.h"
+#include "mongo/db/repl/oplog.h"
+#include "mongo/db/repl/oplog_entry.h"
+#include "mongo/db/repl/oplog_entry_gen.h"
+#include "mongo/db/repl/optime.h"
 #include "mongo/db/repl/read_concern_args.h"
+#include "mongo/db/repl/repl_client_info.h"
 #include "mongo/db/repl/wait_for_majority_service.h"
+#include "mongo/db/repl_index_build_state.h"
 #include "mongo/db/s/migration_destination_manager.h"
+#include "mongo/db/s/resharding/coordinator_document_gen.h"
 #include "mongo/db/s/resharding/resharding_change_event_o2_field_gen.h"
 #include "mongo/db/s/resharding/resharding_data_copy_util.h"
 #include "mongo/db/s/resharding/resharding_future_util.h"
 #include "mongo/db/s/resharding/resharding_metrics_helpers.h"
-#include "mongo/db/s/resharding/resharding_oplog_applier.h"
+#include "mongo/db/s/resharding/resharding_oplog_applier_metrics.h"
+#include "mongo/db/s/resharding/resharding_oplog_applier_progress_gen.h"
 #include "mongo/db/s/resharding/resharding_recipient_service_external_state.h"
 #include "mongo/db/s/resharding/resharding_server_parameters_gen.h"
+#include "mongo/db/s/resharding/resharding_util.h"
 #include "mongo/db/s/shard_key_util.h"
-#include "mongo/db/s/sharding_ddl_util.h"
 #include "mongo/db/s/sharding_index_catalog_ddl_util.h"
 #include "mongo/db/s/sharding_recovery_service.h"
-#include "mongo/db/s/sharding_state.h"
+#include "mongo/db/server_options.h"
+#include "mongo/db/shard_role.h"
+#include "mongo/db/storage/recovery_unit.h"
+#include "mongo/db/storage/write_unit_of_work.h"
+#include "mongo/db/transaction_resources.h"
 #include "mongo/db/write_block_bypass.h"
-#include "mongo/executor/network_interface_factory.h"
-#include "mongo/executor/thread_pool_task_executor.h"
+#include "mongo/db/write_concern_options.h"
+#include "mongo/executor/task_executor.h"
+#include "mongo/idl/idl_parser.h"
 #include "mongo/logv2/log.h"
-#include "mongo/rpc/get_status_from_command_result.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/redaction.h"
 #include "mongo/s/catalog/sharding_catalog_client.h"
-#include "mongo/s/cluster_commands_helpers.h"
-#include "mongo/s/grid.h"
+#include "mongo/s/catalog_cache.h"
+#include "mongo/s/database_version.h"
+#include "mongo/s/resharding/resharding_feature_flag_gen.h"
+#include "mongo/s/shard_key_pattern.h"
+#include "mongo/s/shard_version.h"
 #include "mongo/s/sharding_feature_flags_gen.h"
-#include "mongo/s/stale_shard_version_helpers.h"
+#include "mongo/s/sharding_state.h"
+#include "mongo/stdx/unordered_map.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/clock_source.h"
+#include "mongo/util/concurrency/with_lock.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/fail_point.h"
 #include "mongo/util/future_util.h"
-#include "mongo/util/optional_util.h"
+#include "mongo/util/namespace_string_util.h"
+#include "mongo/util/out_of_line_executor.h"
+#include "mongo/util/str.h"
+#include "mongo/util/uuid.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kResharding
 
@@ -79,6 +127,7 @@ MONGO_FAIL_POINT_DEFINE(reshardingPauseRecipientDuringCloning);
 MONGO_FAIL_POINT_DEFINE(reshardingPauseRecipientDuringOplogApplication);
 MONGO_FAIL_POINT_DEFINE(reshardingOpCtxKilledWhileRestoringMetrics);
 MONGO_FAIL_POINT_DEFINE(reshardingRecipientFailsAfterTransitionToCloning);
+MONGO_FAIL_POINT_DEFINE(reshardingPauseRecipientBeforeBuildingIndex);
 
 namespace {
 
@@ -136,11 +185,27 @@ void buildStateDocumentCloneMetricsForUpdate(BSONObjBuilder& bob, Date_t timesta
                timestamp);
 }
 
+void buildStateDocumentBuildingIndexMetricsForUpdate(BSONObjBuilder& bob, Date_t timestamp) {
+    bob.append(getIntervalEndFieldName<DocT>(ReshardingRecipientMetrics::kDocumentCopyFieldName),
+               timestamp);
+    bob.append(
+        getIntervalStartFieldName<DocT>(ReshardingRecipientMetrics::kIndexBuildTimeFieldName),
+        timestamp);
+}
+
 void buildStateDocumentApplyMetricsForUpdate(BSONObjBuilder& bob,
                                              ReshardingMetrics* metrics,
                                              Date_t timestamp) {
-    bob.append(getIntervalEndFieldName<DocT>(ReshardingRecipientMetrics::kDocumentCopyFieldName),
-               timestamp);
+    if (resharding::gFeatureFlagReshardingImprovements.isEnabled(
+            serverGlobalParams.featureCompatibility.acquireFCVSnapshot())) {
+        bob.append(
+            getIntervalEndFieldName<DocT>(ReshardingRecipientMetrics::kIndexBuildTimeFieldName),
+            timestamp);
+    } else {
+        bob.append(
+            getIntervalEndFieldName<DocT>(ReshardingRecipientMetrics::kDocumentCopyFieldName),
+            timestamp);
+    }
     bob.append(
         getIntervalStartFieldName<DocT>(ReshardingRecipientMetrics::kOplogApplicationFieldName),
         timestamp);
@@ -165,6 +230,9 @@ void buildStateDocumentMetricsForUpdate(BSONObjBuilder& bob,
         case RecipientStateEnum::kCloning:
             buildStateDocumentCloneMetricsForUpdate(bob, timestamp);
             return;
+        case RecipientStateEnum::kBuildingIndex:
+            buildStateDocumentBuildingIndexMetricsForUpdate(bob, timestamp);
+            return;
         case RecipientStateEnum::kApplying:
             buildStateDocumentApplyMetricsForUpdate(bob, metrics, timestamp);
             return;
@@ -183,8 +251,17 @@ void setMeticsAfterWrite(ReshardingMetrics* metrics,
         case RecipientStateEnum::kCloning:
             metrics->setStartFor(ReshardingMetrics::TimedPhase::kCloning, timestamp);
             return;
-        case RecipientStateEnum::kApplying:
+        case RecipientStateEnum::kBuildingIndex:
             metrics->setEndFor(ReshardingMetrics::TimedPhase::kCloning, timestamp);
+            metrics->setStartFor(ReshardingMetrics::TimedPhase::kBuildingIndex, timestamp);
+            return;
+        case RecipientStateEnum::kApplying:
+            if (resharding::gFeatureFlagReshardingImprovements.isEnabled(
+                    serverGlobalParams.featureCompatibility.acquireFCVSnapshot())) {
+                metrics->setEndFor(ReshardingMetrics::TimedPhase::kBuildingIndex, timestamp);
+            } else {
+                metrics->setEndFor(ReshardingMetrics::TimedPhase::kCloning, timestamp);
+            }
             metrics->setStartFor(ReshardingMetrics::TimedPhase::kApplying, timestamp);
             return;
         case RecipientStateEnum::kStrictConsistency:
@@ -225,22 +302,20 @@ ReshardingRecipientService::RecipientStateMachine::RecipientStateMachine(
       _metrics{ReshardingMetrics::initializeFrom(recipientDoc, _serviceContext)},
       _metadata{recipientDoc.getCommonReshardingMetadata()},
       _minimumOperationDuration{Milliseconds{recipientDoc.getMinimumOperationDurationMillis()}},
+      _oplogBatchTaskCount{recipientDoc.getOplogBatchTaskCount()},
       _recipientCtx{recipientDoc.getMutableState()},
       _donorShards{recipientDoc.getDonorShards()},
       _cloneTimestamp{recipientDoc.getCloneTimestamp()},
       _externalState{std::move(externalState)},
       _startConfigTxnCloneAt{recipientDoc.getStartConfigTxnCloneTime()},
-      _markKilledExecutor(std::make_shared<ThreadPool>([] {
-          ThreadPool::Options options;
-          options.poolName = "RecipientStateMachineCancelableOpCtxPool";
-          options.minThreads = 1;
-          options.maxThreads = 1;
-          return options;
-      }())),
+      _markKilledExecutor{resharding::makeThreadPoolForMarkKilledExecutor(
+          "RecipientStateMachineCancelableOpCtxPool")},
       _dataReplicationFactory{std::move(dataReplicationFactory)},
       _critSecReason(BSON("command"
                           << "resharding_recipient"
-                          << "collection" << _metadata.getSourceNss().toString())),
+                          << "collection"
+                          << NamespaceStringUtil::serialize(_metadata.getSourceNss(),
+                                                            SerializationContext::stateDefault()))),
       _isAlsoDonor([&]() {
           auto myShardId = _externalState->myShardId(_serviceContext);
           return std::find_if(_donorShards.begin(),
@@ -269,7 +344,10 @@ ReshardingRecipientService::RecipientStateMachine::_runUntilStrictConsistencyOrE
                     _createTemporaryReshardingCollectionThenTransitionToCloning(factory);
                 })
                 .then([this, executor, abortToken, &factory] {
-                    return _cloneThenTransitionToApplying(executor, abortToken, factory);
+                    return _cloneThenTransitionToBuildingIndex(executor, abortToken, factory);
+                })
+                .then([this, executor, abortToken, &factory] {
+                    return _buildIndexThenTransitionToApplying(executor, abortToken, factory);
                 })
                 .then([this, executor, abortToken, &factory] {
                     return _awaitAllDonorsBlockingWritesThenTransitionToStrictConsistency(
@@ -345,6 +423,27 @@ ReshardingRecipientService::RecipientStateMachine::_notifyCoordinatorAndAwaitDec
     return _retryingCancelableOpCtxFactory
         ->withAutomaticRetry([this, executor](const auto& factory) {
             auto opCtx = factory.makeOperationContext(&cc());
+            if (resharding::gFeatureFlagReshardingImprovements.isEnabled(
+                    serverGlobalParams.featureCompatibility.acquireFCVSnapshot())) {
+                {
+                    AutoGetCollection coll(opCtx.get(), _metadata.getTempReshardingNss(), MODE_IS);
+                    if (coll) {
+                        _recipientCtx.setTotalNumDocuments(coll->numRecords(opCtx.get()));
+                        _recipientCtx.setTotalDocumentSize(coll->dataSize(opCtx.get()));
+                        if (coll->isClustered()) {
+                            // There is an implicit 'clustered' index on a clustered collection.
+                            // Increment the total index count similar to storage stats:
+                            // https://github.com/10gen/mongo/blob/29d8030f8aa7f3bc119081007fb09777daffc591/src/mongo/db/stats/storage_stats.cpp#L249C1-L251C22
+                            _recipientCtx.setNumOfIndexes(
+                                coll->getIndexCatalog()->numIndexesTotal() + 1);
+                        } else {
+                            _recipientCtx.setNumOfIndexes(
+                                coll->getIndexCatalog()->numIndexesTotal());
+                        }
+                    }
+                }
+                _metrics->fillRecipientCtxOnCompletion(_recipientCtx);
+            }
             return _updateCoordinator(opCtx.get(), executor, factory);
         })
         .onTransientError([](const Status& status) {
@@ -402,16 +501,16 @@ ExecutorFuture<void> ReshardingRecipientService::RecipientStateMachine::_finishR
                     if (!_isAlsoDonor) {
                         auto opCtx = factory.makeOperationContext(&cc());
 
-                        _externalState->clearFilteringMetadata(opCtx.get(),
-                                                               _metadata.getSourceNss(),
-                                                               _metadata.getTempReshardingNss());
+                        _externalState->clearFilteringMetadataOnTempReshardingCollection(
+                            opCtx.get(), _metadata.getTempReshardingNss());
 
                         ShardingRecoveryService::get(opCtx.get())
                             ->releaseRecoverableCriticalSection(
                                 opCtx.get(),
                                 _metadata.getSourceNss(),
                                 _critSecReason,
-                                ShardingCatalogClient::kLocalWriteConcern);
+                                ShardingCatalogClient::kLocalWriteConcern,
+                                ShardingRecoveryService::FilteringMetadataClearer());
                     }
                 })
                 .then([this, executor, &factory] {
@@ -543,6 +642,15 @@ void ReshardingRecipientService::RecipientStateMachine::interrupt(Status status)
 boost::optional<BSONObj> ReshardingRecipientService::RecipientStateMachine::reportForCurrentOp(
     MongoProcessInterface::CurrentOpConnectionsMode,
     MongoProcessInterface::CurrentOpSessionsMode) noexcept {
+    auto state = [this] {
+        stdx::lock_guard lk(_mutex);
+        return _recipientCtx.getState();
+    }();
+    auto opCtx = cc().getOperationContext();
+    invariant(opCtx);
+    if (state == RecipientStateEnum::kBuildingIndex) {
+        _tryFetchBuildIndexMetrics(opCtx);
+    }
     return _metrics->reportForCurrentOp();
 }
 
@@ -612,30 +720,83 @@ void ReshardingRecipientService::RecipientStateMachine::
         _externalState->ensureTempReshardingCollectionExistsWithIndexes(
             opCtx.get(), _metadata, *_cloneTimestamp);
 
-        _externalState->withShardVersionRetry(
-            opCtx.get(),
-            _metadata.getTempReshardingNss(),
-            "validating shard key index for reshardCollection"_sd,
-            [&] {
-                shardkeyutil::validateShardKeyIsNotEncrypted(
+        if (resharding::gFeatureFlagReshardingImprovements.isEnabled(
+                serverGlobalParams.featureCompatibility.acquireFCVSnapshot())) {
+            if (!_metadata.getProvenance() ||
+                _metadata.getProvenance() == ProvenanceEnum::kReshardCollection) {
+                _externalState->withShardVersionRetry(
                     opCtx.get(),
-                    _metadata.getTempReshardingNss(),
-                    ShardKeyPattern(_metadata.getReshardingKey()));
-                shardkeyutil::validateShardKeyIndexExistsOrCreateIfPossible(
-                    opCtx.get(),
-                    _metadata.getTempReshardingNss(),
-                    ShardKeyPattern{_metadata.getReshardingKey()},
-                    CollationSpec::kSimpleSpec,
-                    false /* unique */,
-                    true /* enforceUniquenessCheck */,
-                    shardkeyutil::ValidationBehaviorsShardCollection(opCtx.get()));
-            });
+                    _metadata.getSourceNss(),
+                    "validating shard key index for reshardCollection"_sd,
+                    [&] {
+                        shardkeyutil::validateShardKeyIsNotEncrypted(
+                            opCtx.get(),
+                            _metadata.getSourceNss(),
+                            ShardKeyPattern(_metadata.getReshardingKey()));
+                        // This behavior in this phase is only used to validate whether this
+                        // resharding should be permitted, we need to call
+                        // validateShardKeyIndexExistsOrCreateIfPossible again in the buildIndex
+                        // phase to make sure we have the indexSpecs even after restart.
+                        shardkeyutil::ValidationBehaviorsReshardingBulkIndex behaviors;
+                        behaviors.setOpCtxAndCloneTimestamp(opCtx.get(), *_cloneTimestamp);
+
+                        auto [collOptions, _] = _externalState->getCollectionOptions(
+                            opCtx.get(),
+                            _metadata.getSourceNss(),
+                            _metadata.getSourceUUID(),
+                            *_cloneTimestamp,
+                            "loading collection options to validate shard key index for resharding."_sd);
+
+                        CollectionOptions collectionOptions =
+                            uassertStatusOK(CollectionOptions::parse(
+                                collOptions, CollectionOptions::parseForStorage));
+
+                        shardkeyutil::validateShardKeyIndexExistsOrCreateIfPossible(
+                            opCtx.get(),
+                            _metadata.getSourceNss(),
+                            ShardKeyPattern{_metadata.getReshardingKey()},
+                            CollationSpec::kSimpleSpec,
+                            false /* unique */,
+                            true /* enforceUniquenessCheck */,
+                            behaviors,
+                            collectionOptions.timeseries /* tsOpts */,
+                            true /* updatedToHandleTimeseriesIndex */);
+                    });
+            }
+        } else {
+            _externalState->withShardVersionRetry(
+                opCtx.get(),
+                _metadata.getTempReshardingNss(),
+                "validating shard key index for reshardCollection"_sd,
+                [&] {
+                    shardkeyutil::validateShardKeyIsNotEncrypted(
+                        opCtx.get(),
+                        _metadata.getTempReshardingNss(),
+                        ShardKeyPattern(_metadata.getReshardingKey()));
+
+                    // Do not need to pass in time-series options because we cannot reshard a
+                    // time-series collection.
+                    // TODO SERVER-84741 pass in time-series options and ensure the shard key is
+                    // partially rewritten.
+                    shardkeyutil::validateShardKeyIndexExistsOrCreateIfPossible(
+                        opCtx.get(),
+                        _metadata.getTempReshardingNss(),
+                        ShardKeyPattern{_metadata.getReshardingKey()},
+                        CollationSpec::kSimpleSpec,
+                        false /* unique */,
+                        true /* enforceUniquenessCheck */,
+                        shardkeyutil::ValidationBehaviorsShardCollection(
+                            opCtx.get(), ShardingState::get(opCtx.get())->shardId()),
+                        boost::none /* tsOpts */,
+                        false /* updatedToHandleTimeseriesIndex */);
+                });
+        }
 
         // We add a fake 'shardCollection' notification here so that the C2C replicator can sync the
         // resharding operation to the target cluster. The only information we have is the shard
         // key, but all other fields must either be default-valued or are ignored by C2C.
         // TODO SERVER-66671: The 'createCollRequest' should include the full contents of the
-        // CreateCollectionRequest rather than just the 'shardKey' field.
+        // ShardsvrCreateCollectionRequest rather than just the 'shardKey' field.
         const auto createCollRequest = BSON("shardKey" << _metadata.getReshardingKey().toBSON());
         notifyChangeStreamsOnShardCollection(opCtx.get(),
                                              _metadata.getTempReshardingNss(),
@@ -658,8 +819,9 @@ ReshardingRecipientService::RecipientStateMachine::_makeDataReplication(Operatio
     _externalState->refreshCatalogCache(opCtx, _metadata.getSourceNss());
 
     auto myShardId = _externalState->myShardId(opCtx->getServiceContext());
+
     auto [sourceChunkMgr, _] =
-        _externalState->getShardedCollectionRoutingInfo(opCtx, _metadata.getSourceNss());
+        _externalState->getTrackedCollectionRoutingInfo(opCtx, _metadata.getSourceNss());
 
     // The metrics map can already be pre-populated if it was recovered from disk.
     if (_applierMetricsMap.empty()) {
@@ -674,9 +836,13 @@ ReshardingRecipientService::RecipientStateMachine::_makeDataReplication(Operatio
                                 << " != donor shards count: " << _donorShards.size());
     }
 
+    auto oplogBatchTaskCount = _oplogBatchTaskCount.value_or(
+        static_cast<std::size_t>(resharding::gReshardingOplogBatchTaskCount.load()));
+
     return _dataReplicationFactory(opCtx,
                                    _metrics.get(),
                                    &_applierMetricsMap,
+                                   oplogBatchTaskCount,
                                    _metadata,
                                    _donorShards,
                                    *_cloneTimestamp,
@@ -715,7 +881,7 @@ void ReshardingRecipientService::RecipientStateMachine::_ensureDataReplicationSt
 }
 
 ExecutorFuture<void>
-ReshardingRecipientService::RecipientStateMachine::_cloneThenTransitionToApplying(
+ReshardingRecipientService::RecipientStateMachine::_cloneThenTransitionToBuildingIndex(
     const std::shared_ptr<executor::ScopedTaskExecutor>& executor,
     const CancellationToken& abortToken,
     const CancelableOperationContextFactory& factory) {
@@ -745,7 +911,108 @@ ReshardingRecipientService::RecipientStateMachine::_cloneThenTransitionToApplyin
 
     return future_util::withCancellation(_dataReplication->awaitCloningDone(), abortToken)
         .thenRunOn(**executor)
-        .then([this, &factory] { _transitionState(RecipientStateEnum::kApplying, factory); });
+        .then([this, &factory] {
+            if (resharding::gFeatureFlagReshardingImprovements.isEnabled(
+                    serverGlobalParams.featureCompatibility.acquireFCVSnapshot())) {
+                _transitionState(RecipientStateEnum::kBuildingIndex, factory);
+            } else {
+                _transitionState(RecipientStateEnum::kApplying, factory);
+            }
+        });
+}
+
+ExecutorFuture<void>
+ReshardingRecipientService::RecipientStateMachine::_buildIndexThenTransitionToApplying(
+    const std::shared_ptr<executor::ScopedTaskExecutor>& executor,
+    const CancellationToken& abortToken,
+    const CancelableOperationContextFactory& factory) {
+    if (_recipientCtx.getState() > RecipientStateEnum::kBuildingIndex) {
+        return ExecutorFuture(**executor);
+    }
+
+    {
+        auto opCtx = factory.makeOperationContext(&cc());
+        reshardingPauseRecipientBeforeBuildingIndex.pauseWhileSet(opCtx.get());
+    }
+
+    return future_util::withCancellation(
+               [this, &factory] {
+                   auto opCtx = factory.makeOperationContext(&cc());
+                   // We call validateShardKeyIndexExistsOrCreateIfPossible again here in case if we
+                   // restarted after creatingCollection phase, whatever indexSpec we get in that
+                   // phase will go away.
+                   shardkeyutil::ValidationBehaviorsReshardingBulkIndex behaviors;
+                   behaviors.setOpCtxAndCloneTimestamp(opCtx.get(), *_cloneTimestamp);
+
+                   if (!_metadata.getProvenance() ||
+                       _metadata.getProvenance() == ProvenanceEnum::kReshardCollection) {
+                       auto [collOptions, _] = _externalState->getCollectionOptions(
+                           opCtx.get(),
+                           _metadata.getSourceNss(),
+                           _metadata.getSourceUUID(),
+                           *_cloneTimestamp,
+                           "loading collection options to build shard key index for resharding."_sd);
+                       CollectionOptions collectionOptions =
+                           uassertStatusOK(CollectionOptions::parse(
+                               collOptions, CollectionOptions::parseForStorage));
+
+                       shardkeyutil::validateShardKeyIndexExistsOrCreateIfPossible(
+                           opCtx.get(),
+                           _metadata.getSourceNss(),
+                           ShardKeyPattern{_metadata.getReshardingKey()},
+                           CollationSpec::kSimpleSpec,
+                           false /* unique */,
+                           true /* enforceUniquenessCheck */,
+                           behaviors,
+                           collectionOptions.timeseries /* tsOpts */,
+                           true /* updatedToHandleTimeseriesIndex */);
+                   }
+
+                   // Get all indexSpecs need to build.
+                   auto* indexBuildsCoordinator = IndexBuildsCoordinator::get(opCtx.get());
+                   auto [indexSpecs, _] = _externalState->getCollectionIndexes(
+                       opCtx.get(),
+                       _metadata.getSourceNss(),
+                       _metadata.getSourceUUID(),
+                       *_cloneTimestamp,
+                       "loading indexes to create indexes on temporary resharding collection"_sd);
+                   auto shardKeyIndexSpec = behaviors.getShardKeyIndexSpec();
+                   if (shardKeyIndexSpec) {
+                       indexSpecs.push_back(*shardKeyIndexSpec);
+                   }
+                   // Build all the indexes.
+                   auto buildUUID = UUID::gen();
+                   IndexBuildsCoordinator::IndexBuildOptions indexBuildOptions{
+                       CommitQuorumOptions(CommitQuorumOptions::kVotingMembers)};
+                   auto indexBuildFuture = indexBuildsCoordinator->startIndexBuild(
+                       opCtx.get(),
+                       _metadata.getTempReshardingNss().dbName(),
+                       // When we create the collection we use the metadata resharding UUID as the
+                       // collection UUID.
+                       _metadata.getReshardingUUID(),
+                       indexSpecs,
+                       buildUUID,
+                       IndexBuildProtocol::kTwoPhase,
+                       indexBuildOptions);
+                   if (indexBuildFuture.isOK()) {
+                       return indexBuildFuture.getValue();
+                   } else if (indexBuildFuture == ErrorCodes::IndexBuildAlreadyInProgress ||
+                              indexBuildFuture == ErrorCodes::IndexAlreadyExists) {
+                       // In case of failover, the index build could have been started by oplog
+                       // applier, so we just wait those finish.
+                       indexBuildsCoordinator->awaitNoIndexBuildInProgressForCollection(
+                           opCtx.get(), _metadata.getReshardingUUID());
+                       return SharedSemiFuture<ReplIndexBuildState::IndexCatalogStats>(
+                           ReplIndexBuildState::IndexCatalogStats());
+                   } else {
+                       return uassertStatusOK(indexBuildFuture);
+                   }
+               }(),
+               abortToken)
+        .thenRunOn(**executor)
+        .then([this, &factory](const ReplIndexBuildState::IndexCatalogStats& stats) {
+            _transitionState(RecipientStateEnum::kApplying, factory);
+        });
 }
 
 ExecutorFuture<void> ReshardingRecipientService::RecipientStateMachine::
@@ -825,8 +1092,8 @@ void ReshardingRecipientService::RecipientStateMachine::_writeStrictConsistencyO
 
     auto oplog = generateOplogEntry();
     writeConflictRetry(
-        rawOpCtx, "ReshardDoneCatchUpOplog", NamespaceString::kRsOplogNamespace.ns(), [&] {
-            AutoGetOplog oplogWrite(rawOpCtx, OplogAccessMode::kWrite);
+        rawOpCtx, "ReshardDoneCatchUpOplog", NamespaceString::kRsOplogNamespace, [&] {
+            AutoGetOplogFastPath oplogWrite(rawOpCtx, OplogAccessMode::kWrite);
             WriteUnitOfWork wunit(rawOpCtx);
             const auto& oplogOpTime = repl::logOp(rawOpCtx, &oplog);
             uassert(5063601,
@@ -872,9 +1139,36 @@ void ReshardingRecipientService::RecipientStateMachine::_cleanupReshardingCollec
             dropCollectionShardingIndexCatalog(opCtx.get(), _metadata.getTempReshardingNss());
         }
 
+
+        {
+            // We need to do this even though the feature flag is not on because the resharding can
+            // be aborted by setFCV downgrade, when the FCV is already in downgrading and the
+            // feature flag is treated as off.
+            auto* indexBuildsCoordinator = IndexBuildsCoordinator::get(opCtx.get());
+            std::string abortReason(str::stream()
+                                    << "Index builds on "
+                                    << _metadata.getTempReshardingNss().toStringForErrorMsg()
+                                    << " are aborted because resharding is aborted");
+            indexBuildsCoordinator->abortCollectionIndexBuilds(opCtx.get(),
+                                                               _metadata.getTempReshardingNss(),
+                                                               _metadata.getReshardingUUID(),
+                                                               abortReason);
+            // abortCollectionIndexBuilds can return on index commit/abort without waiting the index
+            // build done, so we need to explicitly wait here to make sure there is no building
+            // index after this point.
+            indexBuildsCoordinator->awaitNoIndexBuildInProgressForCollection(
+                opCtx.get(), _metadata.getReshardingUUID());
+        }
+
         resharding::data_copy::ensureCollectionDropped(
             opCtx.get(), _metadata.getTempReshardingNss(), _metadata.getReshardingUUID());
     }
+
+    if (resharding::gFeatureFlagReshardingImprovements.isEnabled(
+            serverGlobalParams.featureCompatibility.acquireFCVSnapshot())) {
+        resharding::data_copy::deleteRecipientResumeData(opCtx.get(),
+                                                         _metadata.getReshardingUUID());
+    }
 }
 
 void ReshardingRecipientService::RecipientStateMachine::_transitionState(
@@ -922,10 +1216,8 @@ void ReshardingRecipientService::RecipientStateMachine::_transitionToCreatingCol
     const CancelableOperationContextFactory& factory) {
     auto newRecipientCtx = _recipientCtx;
     newRecipientCtx.setState(RecipientStateEnum::kCreatingCollection);
-    _transitionState(std::move(newRecipientCtx),
-                     std::move(cloneDetails),
-                     std::move(startConfigTxnCloneTime),
-                     factory);
+    _transitionState(
+        std::move(newRecipientCtx), std::move(cloneDetails), startConfigTxnCloneTime, factory);
 }
 
 void ReshardingRecipientService::RecipientStateMachine::_transitionToError(
@@ -1021,7 +1313,7 @@ ExecutorFuture<void> ReshardingRecipientService::RecipientStateMachine::_updateC
     repl::ReplClientInfo::forClient(opCtx->getClient()).setLastOpToSystemLastOpTime(opCtx);
     auto clientOpTime = repl::ReplClientInfo::forClient(opCtx->getClient()).getLastOp();
     return WaitForMajorityService::get(opCtx->getServiceContext())
-        .waitUntilMajority(clientOpTime, CancellationToken::uncancelable())
+        .waitUntilMajorityForWrite(clientOpTime, CancellationToken::uncancelable())
         .thenRunOn(**executor)
         .then([this, &factory] {
             auto opCtx = factory.makeOperationContext(&cc());
@@ -1129,30 +1421,35 @@ void ReshardingRecipientService::RecipientStateMachine::_removeRecipientDocument
     auto opCtx = factory.makeOperationContext(&cc());
 
     const auto& nss = NamespaceString::kRecipientReshardingOperationsNamespace;
-    writeConflictRetry(
-        opCtx.get(), "RecipientStateMachine::_removeRecipientDocument", nss.toString(), [&] {
-            AutoGetCollection coll(opCtx.get(), nss, MODE_IX);
+    writeConflictRetry(opCtx.get(), "RecipientStateMachine::_removeRecipientDocument", nss, [&] {
+        const auto coll = acquireCollection(
+            opCtx.get(),
+            CollectionAcquisitionRequest(nss,
+                                         PlacementConcern{boost::none, ShardVersion::UNSHARDED()},
+                                         repl::ReadConcernArgs::get(opCtx.get()),
+                                         AcquisitionPrerequisites::kWrite),
+            MODE_IX);
 
-            if (!coll) {
-                return;
-            }
+        if (!coll.exists()) {
+            return;
+        }
 
-            WriteUnitOfWork wuow(opCtx.get());
+        WriteUnitOfWork wuow(opCtx.get());
 
-            opCtx->recoveryUnit()->onCommit([this](OperationContext*, boost::optional<Timestamp>) {
+        shard_role_details::getRecoveryUnit(opCtx.get())
+            ->onCommit([this](OperationContext*, boost::optional<Timestamp>) {
                 stdx::lock_guard<Latch> lk(_mutex);
                 _completionPromise.emplaceValue();
             });
 
-            deleteObjects(opCtx.get(),
-                          *coll,
-                          nss,
-                          BSON(ReshardingRecipientDocument::kReshardingUUIDFieldName
-                               << _metadata.getReshardingUUID()),
-                          true /* justOne */);
+        deleteObjects(opCtx.get(),
+                      coll,
+                      BSON(ReshardingRecipientDocument::kReshardingUUIDFieldName
+                           << _metadata.getReshardingUUID()),
+                      true /* justOne */);
 
-            wuow.commit();
-        });
+        wuow.commit();
+    });
 }
 
 ExecutorFuture<void> ReshardingRecipientService::RecipientStateMachine::_startMetrics(
@@ -1191,11 +1488,12 @@ void ReshardingRecipientService::RecipientStateMachine::_restoreMetrics(
         if (!tempReshardingColl) {
             return;
         }
-        if (_recipientCtx.getState() != RecipientStateEnum::kCloning) {
+        if (_recipientCtx.getState() != RecipientStateEnum::kCloning &&
+            _recipientCtx.getState() != RecipientStateEnum::kBuildingIndex) {
             // Before cloning, these values are 0. After cloning these values are written to the
             // metrics section of the recipient state document and restored during metrics
-            // initialization. This is so that applied oplog entries that add or remove documents do
-            // not affect the cloning metrics.
+            // initialization. This is so that applied oplog entries that add or remove
+            // documents do not affect the cloning metrics.
             return;
         }
         externalMetrics.documentBytesCopied = tempReshardingColl->dataSize(opCtx.get());
@@ -1296,6 +1594,26 @@ CancellationToken ReshardingRecipientService::RecipientStateMachine::_initAbortS
     return _abortSource->token();
 }
 
+void ReshardingRecipientService::RecipientStateMachine::_tryFetchBuildIndexMetrics(
+    OperationContext* opCtx) {
+    try {
+        AutoGetCollection tempReshardingColl(opCtx, _metadata.getTempReshardingNss(), MODE_IS);
+        auto indexCatalog = tempReshardingColl->getIndexCatalog();
+        invariant(indexCatalog,
+                  str::stream() << "Collection is missing index catalog: "
+                                << _metadata.getTempReshardingNss().toStringForErrorMsg());
+        _metrics->setIndexesToBuild(indexCatalog->numIndexesTotal());
+        _metrics->setIndexesBuilt(indexCatalog->numIndexesReady());
+    } catch (const DBException& e) {
+        if (!opCtx->checkForInterruptNoAssert().isOK()) {
+            // If the $currentOp operation itself was killed, don't bother logging the warning.
+            return;
+        }
+        LOGV2_WARNING(
+            9037600, "Unable to fetch index build metrics", "reason"_attr = redact(e.toStatus()));
+    }
+}
+
 void ReshardingRecipientService::RecipientStateMachine::abort(bool isUserCancelled) {
     auto abortSource = [&]() -> boost::optional<CancellationSource> {
         stdx::lock_guard<Latch> lk(_mutex);
