@@ -28,32 +28,61 @@
  */
 
 
-#include "mongo/platform/basic.h"
+#include <boost/optional.hpp>
+#include <cstdint>
+#include <memory>
+#include <mutex>
+#include <utility>
+#include <vector>
 
-#include "mongo/db/s/session_catalog_migration_destination.h"
-
-#include <functional>
+#include <boost/cstdint.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status_with.h"
+#include "mongo/bson/bsonelement.h"
 #include "mongo/bson/bsonobjbuilder.h"
-#include "mongo/client/connection_string.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/client/read_preference.h"
 #include "mongo/db/cancelable_operation_context.h"
+#include "mongo/db/client.h"
 #include "mongo/db/concurrency/d_concurrency.h"
 #include "mongo/db/concurrency/exception_util.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/ops/write_ops_retryability.h"
 #include "mongo/db/repl/oplog.h"
 #include "mongo/db/repl/oplog_entry.h"
+#include "mongo/db/repl/oplog_entry_gen.h"
 #include "mongo/db/s/migration_session_id.h"
 #include "mongo/db/s/session_catalog_migration.h"
-#include "mongo/db/s/sharding_statistics.h"
+#include "mongo/db/s/session_catalog_migration_destination.h"
+#include "mongo/db/service_context.h"
 #include "mongo/db/session/logical_session_id.h"
+#include "mongo/db/session/logical_session_id_helpers.h"
 #include "mongo/db/session/session_catalog_mongod.h"
+#include "mongo/db/session/session_txn_record_gen.h"
 #include "mongo/db/shard_id.h"
+#include "mongo/db/storage/write_unit_of_work.h"
 #include "mongo/db/transaction/transaction_participant.h"
 #include "mongo/db/write_concern.h"
+#include "mongo/db/write_concern_options.h"
+#include "mongo/executor/task_executor_pool.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
 #include "mongo/logv2/redaction.h"
+#include "mongo/s/client/shard.h"
 #include "mongo/s/client/shard_registry.h"
 #include "mongo/s/grid.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/duration.h"
+#include "mongo/util/fail_point.h"
+#include "mongo/util/out_of_line_executor.h"
+#include "mongo/util/str.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
@@ -175,7 +204,7 @@ BSONObj getNextSessionOplogBatch(OperationContext* opCtx,
     auto shard = shardStatus.getValue();
     auto responseStatus = shard->runCommand(opCtx,
                                             ReadPreferenceSetting(ReadPreference::PrimaryOnly),
-                                            "admin",
+                                            DatabaseName::kAdmin,
                                             buildMigrateSessionCmd(migrationSessionId),
                                             Shard::RetryPolicy::kNoRetry);
 
@@ -217,7 +246,7 @@ void SessionCatalogMigrationDestination::start(ServiceContext* service) {
         _state = State::Migrating;
     }
 
-    _thread = stdx::thread([=] {
+    _thread = stdx::thread([=, this] {
         try {
             _retrieveSessionStateFromSource(service);
         } catch (const DBException& ex) {
@@ -241,6 +270,10 @@ void SessionCatalogMigrationDestination::finish() {
     }
 }
 
+bool SessionCatalogMigrationDestination::joinable() const {
+    return _thread.joinable();
+}
+
 void SessionCatalogMigrationDestination::join() {
     invariant(_thread.joinable());
     _thread.join();
@@ -260,14 +293,9 @@ void SessionCatalogMigrationDestination::join() {
  * 6. Wait for writes to be committed to majority of the replica set.
  */
 void SessionCatalogMigrationDestination::_retrieveSessionStateFromSource(ServiceContext* service) {
-    Client::initThread(
-        "sessionCatalogMigrationProducer-" + _migrationSessionId.toString(), service, nullptr);
-    auto client = Client::getCurrent();
-    {
-        stdx::lock_guard lk(*client);
-        client->setSystemOperationKillableByStepdown(lk);
-    }
-
+    Client::initThread("sessionCatalogMigrationProducer-" + _migrationSessionId.toString(),
+                       service->getService(ClusterRole::ShardServer),
+                       Client::noSession());
     bool oplogDrainedAfterCommiting = false;
     ProcessOplogResult lastResult;
     repl::OpTime lastOpTimeWaited;
@@ -460,8 +488,8 @@ SessionCatalogMigrationDestination::_processSessionOplog(const BSONObj& oplogBSO
         txnParticipant.beginOrContinue(opCtx,
                                        {result.txnNum},
                                        boost::none /* autocommit */,
-                                       boost::none /* startTransaction */);
-        if (txnParticipant.checkStatementExecuted(opCtx, stmtIds.front())) {
+                                       TransactionParticipant::TransactionActions::kNone);
+        if (txnParticipant.checkStatementExecutedNoOplogEntryFetch(opCtx, stmtIds.front())) {
             // Skip the incoming statement because it has already been logged locally
             return lastResult;
         }
@@ -496,14 +524,8 @@ SessionCatalogMigrationDestination::_processSessionOplog(const BSONObj& oplogBSO
     // Reset OpTime so logOp() can assign a new one.
     oplogEntry.setOpTime(OplogSlot());
 
-    // We should not be writing this field so make sure it is always "none".
-    oplogEntry.setHash(boost::none);
-
     writeConflictRetry(
-        opCtx,
-        "SessionOplogMigration",
-        NamespaceString::kSessionTransactionsTableNamespace.ns(),
-        [&] {
+        opCtx, "SessionOplogMigration", NamespaceString::kSessionTransactionsTableNamespace, [&] {
             // Need to take global lock here so repl::logOp will not unlock it and trigger the
             // invariant that disallows unlocking global lock while inside a WUOW. Take the
             // transaction table db lock to ensure the same lock ordering with normal replicated
@@ -564,6 +586,10 @@ void SessionCatalogMigrationDestination::_errorOccurred(StringData errMsg) {
     _errMsg = errMsg.toString();
 }
 
+MigrationSessionId SessionCatalogMigrationDestination::getMigrationSessionId() const {
+    return _migrationSessionId;
+}
+
 SessionCatalogMigrationDestination::State SessionCatalogMigrationDestination::getState() {
     stdx::lock_guard<Latch> lk(_mutex);
     return _state;
