@@ -27,39 +27,72 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
+#include <boost/cstdint.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <cstddef>
+#include <cstdint>
+#include <fmt/format.h>
 #include <memory>
+#include <vector>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/timestamp.h"
 #include "mongo/client/connection_string.h"
 #include "mongo/client/remote_command_targeter_mock.h"
+#include "mongo/db/catalog/collection.h"
+#include "mongo/db/catalog_raii.h"
+#include "mongo/db/client.h"
 #include "mongo/db/concurrency/d_concurrency.h"
-#include "mongo/db/db_raii.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
+#include "mongo/db/database_name.h"
 #include "mongo/db/dbdirectclient.h"
+#include "mongo/db/exec/document_value/value.h"
+#include "mongo/db/initialize_operation_session_info.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/ops/write_ops_exec.h"
 #include "mongo/db/ops/write_ops_gen.h"
 #include "mongo/db/repl/oplog_entry.h"
+#include "mongo/db/repl/oplog_entry_gen.h"
+#include "mongo/db/repl/optime_with.h"
+#include "mongo/db/repl/read_concern_level.h"
+#include "mongo/db/repl/replication_coordinator.h"
+#include "mongo/db/repl/storage_interface.h"
 #include "mongo/db/repl/storage_interface_impl.h"
+#include "mongo/db/s/collection_metadata.h"
 #include "mongo/db/s/collection_sharding_runtime.h"
 #include "mongo/db/s/migration_session_id.h"
 #include "mongo/db/s/session_catalog_migration.h"
 #include "mongo/db/s/session_catalog_migration_destination.h"
 #include "mongo/db/s/shard_server_test_fixture.h"
-#include "mongo/db/s/sharding_statistics.h"
-#include "mongo/db/server_options.h"
-#include "mongo/db/session/initialize_operation_session_info.h"
+#include "mongo/db/service_context.h"
+#include "mongo/db/session/logical_session_cache.h"
 #include "mongo/db/session/logical_session_cache_noop.h"
 #include "mongo/db/session/logical_session_id.h"
 #include "mongo/db/session/session_catalog_mongod.h"
-#include "mongo/db/session/session_txn_record_gen.h"
-#include "mongo/db/transaction/session_catalog_mongod_transaction_interface_impl.h"
+#include "mongo/db/storage/write_unit_of_work.h"
+#include "mongo/db/tenant_id.h"
 #include "mongo/db/transaction/transaction_history_iterator.h"
 #include "mongo/db/transaction/transaction_participant.h"
 #include "mongo/executor/remote_command_request.h"
+#include "mongo/rpc/op_msg.h"
+#include "mongo/s/catalog/sharding_catalog_client.h"
 #include "mongo/s/catalog/sharding_catalog_client_mock.h"
 #include "mongo/s/catalog/type_shard.h"
+#include "mongo/s/client/shard.h"
 #include "mongo/s/client/shard_registry.h"
 #include "mongo/stdx/thread.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/bson_test_util.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/net/hostandport.h"
+#include "mongo/util/time_support.h"
+#include "mongo/util/uuid.h"
 
 namespace mongo {
 namespace {
@@ -184,8 +217,10 @@ public:
         auto mongoDSessionCatalog = MongoDSessionCatalog::get(opCtx);
         auto ocs = mongoDSessionCatalog->checkOutSession(opCtx);
         auto txnParticipant = TransactionParticipant::get(opCtx);
-        txnParticipant.beginOrContinue(
-            opCtx, {txnNum}, boost::none /* autocommit */, boost::none /* startTransaction */);
+        txnParticipant.beginOrContinue(opCtx,
+                                       {txnNum},
+                                       boost::none /* autocommit */,
+                                       TransactionParticipant::TransactionActions::kNone);
     }
 
     void checkOplog(const repl::OplogEntry& originalOplog, const repl::OplogEntry& oplogToCheck) {
@@ -246,17 +281,29 @@ public:
             insertRequest.serialize({}, &insertBuilder);
             sessionInfo.serialize(&insertBuilder);
 
-            Client::initThread("test-insert-thread");
+            Client::initThread("test-insert-thread", getGlobalServiceContext()->getService());
             auto innerOpCtx = Client::getCurrent()->makeOperationContext();
 
-            initializeOperationSessionInfo(innerOpCtx.get(), insertBuilder.obj(), true, true, true);
+            auto opMsgRequest = OpMsgRequestBuilder::create(
+                auth::ValidatedTenancyScope::kNotRequired,
+                DatabaseName::createDatabaseName_forTest(boost::none, "test_unused_dbname"),
+                insertBuilder.obj(),
+                BSONObj());
+            auto osi = OperationSessionInfoFromClient::parse(
+                IDLParserContext{"OperationSessionInfo"}, opMsgRequest.body);
+            initializeOperationSessionInfo(innerOpCtx.get(),
+                                           opMsgRequest.getValidatedTenantId(),
+                                           osi,
+                                           true /* requiresAuth */,
+                                           true /* attachToOpCtx */,
+                                           true /* isReplSetMemberOrMongos */);
             auto mongoDSessionCatalog = MongoDSessionCatalog::get(innerOpCtx.get());
             auto sessionTxnState = mongoDSessionCatalog->checkOutSession(innerOpCtx.get());
             auto txnParticipant = TransactionParticipant::get(innerOpCtx.get());
             txnParticipant.beginOrContinue(innerOpCtx.get(),
                                            {*sessionInfo.getTxnNumber()},
                                            boost::none /* autocommit */,
-                                           boost::none /* startTransaction */);
+                                           TransactionParticipant::TransactionActions::kNone);
 
             const auto reply = write_ops_exec::performInserts(innerOpCtx.get(), insertRequest);
             ASSERT(reply.results.size() == 1);
@@ -663,7 +710,7 @@ TEST_F(SessionCatalogMigrationDestinationTest, OplogEntriesWithDifferentSession)
 
     {
         // XXX TODO USE A DIFFERENT OPERATION CONTEXT!
-        auto client2 = getServiceContext()->makeClient("client2");
+        auto client2 = getServiceContext()->getService()->makeClient("client2");
         AlternativeClientRegion acr(client2);
         auto opCtx2 = cc().makeOperationContext();
         setUpSessionWithTxn(opCtx2.get(), sessionId2, 42);
@@ -1951,8 +1998,10 @@ TEST_F(SessionCatalogMigrationDestinationTest,
         auto txnParticipant = TransactionParticipant::get(opCtx);
 
         txnParticipant.refreshFromStorageIfNeeded(opCtx);
-        txnParticipant.beginOrContinue(
-            opCtx, {3}, boost::none /* autocommit */, boost::none /* startTransaction */);
+        txnParticipant.beginOrContinue(opCtx,
+                                       {3},
+                                       boost::none /* autocommit */,
+                                       TransactionParticipant::TransactionActions::kNone);
     }
 
     OperationSessionInfo sessionInfo2;
@@ -2003,7 +2052,7 @@ TEST_F(SessionCatalogMigrationDestinationTest,
 
     // Check nothing was written for session 1
     {
-        auto c1 = getServiceContext()->makeClient("c1");
+        auto c1 = getServiceContext()->getService()->makeClient("c1");
         AlternativeClientRegion acr(c1);
         auto opCtx1 = cc().makeOperationContext();
         auto opCtx = opCtx1.get();
@@ -2015,7 +2064,7 @@ TEST_F(SessionCatalogMigrationDestinationTest,
 
     // Check session 2 was correctly updated
     {
-        auto c2 = getServiceContext()->makeClient("c2");
+        auto c2 = getServiceContext()->getService()->makeClient("c2");
         AlternativeClientRegion acr(c2);
         auto opCtx2 = cc().makeOperationContext();
         auto opCtx = opCtx2.get();
@@ -2049,7 +2098,7 @@ TEST_F(SessionCatalogMigrationDestinationTest, MigratingKnownStmtWhileOplogTrunc
     auto mongoDSessionCatalog = MongoDSessionCatalog::get(getServiceContext());
 
     auto getLastWriteOpTime = [&]() {
-        auto c1 = getServiceContext()->makeClient("c1");
+        auto c1 = getServiceContext()->getService()->makeClient("c1");
         AlternativeClientRegion acr(c1);
         auto innerOpCtx = cc().makeOperationContext();
         setUpSessionWithTxn(innerOpCtx.get(), *sessionInfo.getSessionId(), 19);
