@@ -29,10 +29,31 @@
 
 #include "mongo/db/s/shard_filtering_metadata_refresh.h"
 
-#include "mongo/db/catalog_raii.h"
-#include "mongo/db/commands/feature_compatibility_version.h"
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional.hpp>
+#include <boost/smart_ptr.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <fmt/format.h>
+#include <memory>
+#include <string>
+#include <tuple>
+#include <utility>
+
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status_with.h"
+#include "mongo/db/admission/execution_admission_context.h"
+#include "mongo/db/cancelable_operation_context.h"
+#include "mongo/db/client.h"
+#include "mongo/db/concurrency/d_concurrency.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
 #include "mongo/db/curop.h"
+#include "mongo/db/database_name.h"
 #include "mongo/db/operation_context.h"
+#include "mongo/db/repl/member_state.h"
+#include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/s/collection_sharding_runtime.h"
 #include "mongo/db/s/database_sharding_state.h"
 #include "mongo/db/s/forwardable_operation_metadata.h"
@@ -40,12 +61,34 @@
 #include "mongo/db/s/migration_util.h"
 #include "mongo/db/s/operation_sharding_state.h"
 #include "mongo/db/s/resharding/resharding_donor_recipient_common.h"
-#include "mongo/db/s/sharding_state.h"
+#include "mongo/db/s/sharding_migration_critical_section.h"
+#include "mongo/db/service_context.h"
+#include "mongo/db/tenant_id.h"
+#include "mongo/db/transaction_resources.h"
+#include "mongo/executor/task_executor_pool.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/redaction.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/s/catalog/type_database_gen.h"
 #include "mongo/s/catalog_cache.h"
+#include "mongo/s/chunk_manager.h"
 #include "mongo/s/database_version.h"
 #include "mongo/s/grid.h"
+#include "mongo/s/sharding_state.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/cancellation.h"
+#include "mongo/util/concurrency/admission_context.h"
+#include "mongo/util/database_name_util.h"
+#include "mongo/util/duration.h"
 #include "mongo/util/fail_point.h"
+#include "mongo/util/future.h"
+#include "mongo/util/future_impl.h"
+#include "mongo/util/read_through_cache.h"
+#include "mongo/util/scopeguard.h"
+#include "mongo/util/str.h"
+#include "mongo/util/timer.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
@@ -114,21 +157,21 @@ bool joinDbVersionOperation(OperationContext* opCtx,
 Status refreshDbMetadata(OperationContext* opCtx,
                          const DatabaseName& dbName,
                          CancellationToken cancellationToken) {
-    invariant(!opCtx->lockState()->isLocked());
+    invariant(!shard_role_details::getLocker(opCtx)->isLocked());
     invariant(!opCtx->getClient()->isInDirectClient());
-    invariant(ShardingState::get(opCtx)->canAcceptShardedCommands());
+    ShardingState::get(opCtx)->assertCanAcceptShardedCommands();
 
     ScopeGuard resetRefreshFutureOnError([&] {
         // TODO (SERVER-71444): Fix to be interruptible or document exception.
         // Can be uninterruptible because the work done under it can never block.
-        UninterruptibleLockGuard noInterrupt(opCtx->lockState());  // NOLINT.
+        UninterruptibleLockGuard noInterrupt(opCtx);  // NOLINT.
         auto scopedDss = DatabaseShardingState::acquireExclusive(opCtx, dbName);
         scopedDss->resetDbMetadataRefreshFuture();
     });
 
     // Force a refresh of the cached database metadata from the config server.
     const auto swDbMetadata =
-        Grid::get(opCtx)->catalogCache()->getDatabaseWithRefresh(opCtx, dbName.db());
+        Grid::get(opCtx)->catalogCache()->getDatabaseWithRefresh(opCtx, dbName);
 
     // Before setting the database metadata, exit early if the database version received by the
     // config server is not newer than the cached one. This is a best-effort optimization to reduce
@@ -177,12 +220,8 @@ SharedSemiFuture<void> recoverRefreshDbVersion(OperationContext* opCtx,
         .then([=,
                serviceCtx = opCtx->getServiceContext(),
                forwardableOpMetadata = ForwardableOperationMetadata(opCtx)] {
-            ThreadClient tc("DbMetadataRefreshThread", serviceCtx);
-            {
-                stdx::lock_guard<Client> lk(*tc.get());
-                tc->setSystemOperationKillableByStepdown(lk);
-            }
-
+            ThreadClient tc("DbMetadataRefreshThread",
+                            serviceCtx->getService(ClusterRole::ShardServer));
             const auto opCtxHolder =
                 CancelableOperationContext(tc->makeOperationContext(), cancellationToken, executor);
             auto opCtx = opCtxHolder.get();
@@ -217,15 +256,20 @@ SharedSemiFuture<void> recoverRefreshDbVersion(OperationContext* opCtx,
 }
 
 void onDbVersionMismatch(OperationContext* opCtx,
-                         const StringData dbName,
+                         const DatabaseName& dbName,
                          boost::optional<DatabaseVersion> receivedDbVersion) {
-    invariant(!opCtx->lockState()->isLocked());
+    invariant(!shard_role_details::getLocker(opCtx)->isLocked());
     invariant(!opCtx->getClient()->isInDirectClient());
-    invariant(ShardingState::get(opCtx)->canAcceptShardedCommands());
+    ShardingState::get(opCtx)->assertCanAcceptShardedCommands();
 
+    if (MONGO_unlikely(skipShardFilteringMetadataRefresh.shouldFail())) {
+        uasserted(ErrorCodes::InternalError, "skipShardFilteringMetadataRefresh failpoint");
+    }
+
+    using namespace fmt::literals;
     tassert(ErrorCodes::IllegalOperation,
-            "Can't check version of {} database"_format(dbName),
-            dbName != DatabaseName::kAdmin.db() && dbName != DatabaseName::kConfig.db());
+            "Can't check version of {} database"_format(dbName.toStringForErrorMsg()),
+            !dbName.isAdminDB() && !dbName.isConfigDB());
 
     Timer t{};
     ScopeGuard finishTiming([&] {
@@ -243,12 +287,11 @@ void onDbVersionMismatch(OperationContext* opCtx,
 
         {
             boost::optional<Lock::DBLock> dbLock;
-            dbLock.emplace(opCtx, DatabaseName{dbName}, MODE_IS);
+            dbLock.emplace(opCtx, dbName, MODE_IS);
 
             if (receivedDbVersion) {
-                auto scopedDss =
-                    boost::make_optional(DatabaseShardingState::assertDbLockedAndAcquireShared(
-                        opCtx, DatabaseName{dbName}));
+                auto scopedDss = boost::make_optional(
+                    DatabaseShardingState::assertDbLockedAndAcquireShared(opCtx, dbName));
 
                 if (joinDbVersionOperation(opCtx, &dbLock, &scopedDss)) {
                     // Waited for another thread to exit from the critical section or to complete an
@@ -273,9 +316,8 @@ void onDbVersionMismatch(OperationContext* opCtx,
                 return;
             }
 
-            auto scopedDss =
-                boost::make_optional(DatabaseShardingState::assertDbLockedAndAcquireExclusive(
-                    opCtx, DatabaseName{dbName}));
+            auto scopedDss = boost::make_optional(
+                DatabaseShardingState::assertDbLockedAndAcquireExclusive(opCtx, dbName));
 
             if (joinDbVersionOperation(opCtx, &dbLock, &scopedDss)) {
                 // Waited for another thread to exit from the critical section or to complete an
@@ -292,7 +334,7 @@ void onDbVersionMismatch(OperationContext* opCtx,
             CancellationToken cancellationToken = cancellationSource.token();
             (*scopedDss)
                 ->setDbMetadataRefreshFuture(
-                    recoverRefreshDbVersion(opCtx, DatabaseName{dbName}, cancellationToken),
+                    recoverRefreshDbVersion(opCtx, dbName, cancellationToken),
                     std::move(cancellationSource));
             dbMetadataRefreshFuture = (*scopedDss)->getDbMetadataRefreshFuture();
         }
@@ -368,11 +410,8 @@ SharedSemiFuture<void> recoverRefreshCollectionPlacementVersion(
     auto executor = Grid::get(serviceContext)->getExecutorPool()->getFixedExecutor();
     return ExecutorFuture<void>(executor)
         .then([=] {
-            ThreadClient tc("RecoverRefreshThread", serviceContext);
-            {
-                stdx::lock_guard<Client> lk(*tc.get());
-                tc->setSystemOperationKillableByStepdown(lk);
-            }
+            ThreadClient tc("RecoverRefreshThread",
+                            serviceContext->getService(ClusterRole::ShardServer));
 
             if (MONGO_unlikely(hangInRecoverRefreshThread.shouldFail())) {
                 hangInRecoverRefreshThread.pauseWhileSet();
@@ -388,21 +427,22 @@ SharedSemiFuture<void> recoverRefreshCollectionPlacementVersion(
             ScopeGuard resetRefreshFutureOnError([&] {
                 // TODO (SERVER-71444): Fix to be interruptible or document exception.
                 // Can be uninterruptible because the work done under it can never block
-                UninterruptibleLockGuard noInterrupt(opCtx->lockState());  // NOLINT.
+                UninterruptibleLockGuard noInterrupt(opCtx);  // NOLINT.
                 auto scopedCsr = CollectionShardingRuntime::acquireExclusive(opCtx, nss);
                 scopedCsr->resetPlacementVersionRecoverRefreshFuture();
             });
 
             if (runRecover) {
                 auto* const replCoord = repl::ReplicationCoordinator::get(opCtx);
-                if (!replCoord->isReplEnabled() || replCoord->getMemberState().primary()) {
+                if (!replCoord->getSettings().isReplSet() ||
+                    replCoord->getMemberState().primary()) {
                     migrationutil::recoverMigrationCoordinations(opCtx, nss, cancellationToken);
                 }
             }
 
             auto currentMetadata = forceGetCurrentMetadata(opCtx, nss);
 
-            if (currentMetadata.isSharded()) {
+            if (currentMetadata.hasRoutingTable()) {
                 // Abort and join any ongoing migration if migrations are disallowed for the
                 // namespace.
                 if (!currentMetadata.allowMigrations()) {
@@ -502,23 +542,25 @@ SharedSemiFuture<void> recoverRefreshCollectionPlacementVersion(
 void onCollectionPlacementVersionMismatch(OperationContext* opCtx,
                                           const NamespaceString& nss,
                                           boost::optional<ChunkVersion> chunkVersionReceived) {
-    invariant(!opCtx->lockState()->isLocked());
+    invariant(!shard_role_details::getLocker(opCtx)->isLocked());
     invariant(!opCtx->getClient()->isInDirectClient());
-    invariant(ShardingState::get(opCtx)->canAcceptShardedCommands());
+    ShardingState::get(opCtx)->assertCanAcceptShardedCommands();
+
+    if (MONGO_unlikely(skipShardFilteringMetadataRefresh.shouldFail())) {
+        uasserted(ErrorCodes::InternalError, "skipShardFilteringMetadataRefresh failpoint");
+    }
 
     Timer t{};
     ScopeGuard finishTiming([&] {
         CurOp::get(opCtx)->debug().placementVersionRefreshMillis += Milliseconds(t.millis());
     });
 
-    if (nss.isNamespaceAlwaysUnsharded()) {
+    if (nss.isNamespaceAlwaysUntracked()) {
         return;
     }
 
     LOGV2_DEBUG(22061,
                 2,
-                "Metadata refresh requested for {namespace} at chunk version "
-                "{chunkVersionReceived}",
                 "Metadata refresh requested for collection",
                 logAttrs(nss),
                 "chunkVersionReceived"_attr = chunkVersionReceived);
@@ -528,11 +570,9 @@ void onCollectionPlacementVersionMismatch(OperationContext* opCtx,
 
         {
             // The refresh threads do not perform any data reads themselves, therefore they don't
-            // need to synchronise with secondary oplog application or go through admission control.
-            ShouldNotConflictWithSecondaryBatchApplicationBlock skipParallelBatchWriterMutex(
-                opCtx->lockState());
-            ScopedAdmissionPriorityForLock skipAdmissionControl(
-                opCtx->lockState(), AdmissionContext::Priority::kImmediate);
+            // need to go through admission control.
+            ScopedAdmissionPriority<ExecutionAdmissionContext> skipAdmissionControl(
+                opCtx, AdmissionContext::Priority::kExempt);
 
             boost::optional<Lock::DBLock> dbLock;
             boost::optional<Lock::CollectionLock> collLock;
@@ -602,7 +642,6 @@ Status onCollectionPlacementVersionMismatchNoExcept(
         return Status::OK();
     } catch (const DBException& ex) {
         LOGV2(22062,
-              "Failed to refresh metadata for {namespace} due to {error}",
               "Failed to refresh metadata for collection",
               logAttrs(nss),
               "error"_attr = redact(ex));
@@ -611,7 +650,7 @@ Status onCollectionPlacementVersionMismatchNoExcept(
 }
 
 CollectionMetadata forceGetCurrentMetadata(OperationContext* opCtx, const NamespaceString& nss) {
-    invariant(!opCtx->lockState()->isLocked());
+    invariant(!shard_role_details::getLocker(opCtx)->isLocked());
     invariant(!opCtx->getClient()->isInDirectClient());
 
     if (MONGO_unlikely(skipShardFilteringMetadataRefresh.shouldFail())) {
@@ -619,21 +658,20 @@ CollectionMetadata forceGetCurrentMetadata(OperationContext* opCtx, const Namesp
     }
 
     auto* const shardingState = ShardingState::get(opCtx);
-    invariant(shardingState->canAcceptShardedCommands());
+    shardingState->assertCanAcceptShardedCommands();
 
     try {
         const auto [cm, _] = uassertStatusOK(
             Grid::get(opCtx)->catalogCache()->getCollectionRoutingInfoWithPlacementRefresh(opCtx,
                                                                                            nss));
 
-        if (!cm.isSharded()) {
+        if (!cm.hasRoutingTable()) {
             return CollectionMetadata();
         }
 
         return CollectionMetadata(cm, shardingState->shardId());
     } catch (const ExceptionFor<ErrorCodes::NamespaceNotFound>& ex) {
         LOGV2(505070,
-              "Namespace {namespace} not found, collection may have been dropped",
               "Namespace not found, collection may have been dropped",
               logAttrs(nss),
               "error"_attr = redact(ex));
@@ -643,7 +681,7 @@ CollectionMetadata forceGetCurrentMetadata(OperationContext* opCtx, const Namesp
 
 ChunkVersion forceShardFilteringMetadataRefresh(OperationContext* opCtx,
                                                 const NamespaceString& nss) {
-    invariant(!opCtx->lockState()->isLocked());
+    invariant(!shard_role_details::getLocker(opCtx)->isLocked());
     invariant(!opCtx->getClient()->isInDirectClient());
 
     if (MONGO_unlikely(skipShardFilteringMetadataRefresh.shouldFail())) {
@@ -651,12 +689,12 @@ ChunkVersion forceShardFilteringMetadataRefresh(OperationContext* opCtx,
     }
 
     auto* const shardingState = ShardingState::get(opCtx);
-    invariant(shardingState->canAcceptShardedCommands());
+    shardingState->assertCanAcceptShardedCommands();
 
     const auto [cm, _] = uassertStatusOK(
         Grid::get(opCtx)->catalogCache()->getCollectionRoutingInfoWithPlacementRefresh(opCtx, nss));
 
-    if (!cm.isSharded()) {
+    if (!cm.hasRoutingTable()) {
         // DBLock and CollectionLock are used here to avoid throwing further recursive stale
         // config errors, as well as a possible InvalidViewDefinition error if an invalid view
         // is in the 'system.views' collection.
@@ -681,7 +719,7 @@ ChunkVersion forceShardFilteringMetadataRefresh(OperationContext* opCtx,
             CollectionShardingRuntime::assertCollectionLockedAndAcquireShared(opCtx, nss);
         if (auto optMetadata = scopedCsr->getCurrentMetadataIfKnown()) {
             const auto& metadata = *optMetadata;
-            if (metadata.isSharded() &&
+            if (metadata.hasRoutingTable() &&
                 (cm.getVersion().isOlderOrEqualThan(metadata.getCollPlacementVersion()))) {
                 LOGV2_DEBUG(22063,
                             1,
@@ -706,7 +744,7 @@ ChunkVersion forceShardFilteringMetadataRefresh(OperationContext* opCtx,
         CollectionShardingRuntime::assertCollectionLockedAndAcquireExclusive(opCtx, nss);
     if (auto optMetadata = scopedCsr->getCurrentMetadataIfKnown()) {
         const auto& metadata = *optMetadata;
-        if (metadata.isSharded() &&
+        if (metadata.hasRoutingTable() &&
             (cm.getVersion().isOlderOrEqualThan(metadata.getCollPlacementVersion()))) {
             LOGV2_DEBUG(22064,
                         1,
@@ -727,14 +765,13 @@ ChunkVersion forceShardFilteringMetadataRefresh(OperationContext* opCtx,
 }
 
 Status onDbVersionMismatchNoExcept(OperationContext* opCtx,
-                                   const StringData dbName,
+                                   const DatabaseName& dbName,
                                    boost::optional<DatabaseVersion> clientDbVersion) noexcept {
     try {
         onDbVersionMismatch(opCtx, dbName, clientDbVersion);
         return Status::OK();
     } catch (const DBException& ex) {
         LOGV2(22065,
-              "Failed to refresh databaseVersion for database {db} {error}",
               "Failed to refresh databaseVersion",
               "db"_attr = dbName,
               "error"_attr = redact(ex));
