@@ -29,19 +29,52 @@
 
 #pragma once
 
+#include <memory>
+#include <set>
+#include <stack>
+#include <string>
+#include <utility>
+
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/db/client.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/persistent_task_store.h"
+#include "mongo/db/pipeline/process_interface/mongo_process_interface.h"
+#include "mongo/db/repl/primary_only_service.h"
 #include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/repl/wait_for_majority_service.h"
 #include "mongo/db/s/ddl_lock_manager.h"
 #include "mongo/db/s/forwardable_operation_metadata.h"
 #include "mongo/db/s/sharding_ddl_coordinator_gen.h"
 #include "mongo/db/s/sharding_ddl_coordinator_service.h"
+#include "mongo/db/service_context.h"
 #include "mongo/db/session/internal_session_pool.h"
+#include "mongo/db/session/logical_session_id_gen.h"
+#include "mongo/db/transaction_resources.h"
+#include "mongo/executor/scoped_task_executor.h"
 #include "mongo/executor/task_executor.h"
+#include "mongo/idl/idl_parser.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/redaction.h"
+#include "mongo/platform/mutex.h"
+#include "mongo/s/database_version.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/cancellation.h"
 #include "mongo/util/future.h"
+#include "mongo/util/future_impl.h"
+#include "mongo/util/namespace_string_util.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
@@ -54,7 +87,7 @@ class ShardingDDLCoordinator
 public:
     explicit ShardingDDLCoordinator(ShardingDDLCoordinatorService* service, const BSONObj& coorDoc);
 
-    ~ShardingDDLCoordinator();
+    ~ShardingDDLCoordinator() override;
 
     /**
      * Whether this coordinator is allowed to start when user write blocking is enabled, even if the
@@ -112,13 +145,26 @@ protected:
         return originalNss();
     }
 
-    virtual std::vector<StringData> _acquireAdditionalLocks(OperationContext* opCtx) {
+    virtual std::set<NamespaceString> _getAdditionalLocksToAcquire(OperationContext* opCtx) {
         return {};
     };
 
     virtual ShardingDDLCoordinatorMetadata const& metadata() const = 0;
     virtual void setMetadata(ShardingDDLCoordinatorMetadata&& metadata) = 0;
 
+    /**
+     * Returns a set of basic coordinator attributes to be used for logging.
+     */
+    logv2::DynamicAttributes getBasicCoordinatorAttrs() const;
+
+    /**
+     * Returns the set of attributes to be used for coordinator logging. Implementations must be
+     * sure to return a DynamicAttributes object that starts with the attributes returned by
+     * getBasicCoordinatorAttrs().
+     */
+    virtual logv2::DynamicAttributes getCoordinatorLogAttrs() const {
+        return getBasicCoordinatorAttrs();
+    }
     /*
      * Performs a noop write on all shards and the configsvr using the sessionId and txnNumber
      * specified in 'osi'.
@@ -142,12 +188,15 @@ protected:
      */
     bool _isRetriableErrorForDDLCoordinator(const Status& status);
 
+    ShardingDDLCoordinatorExternalState* _getExternalState();
+
     ShardingDDLCoordinatorService* _service;
     const ShardingDDLCoordinatorId _coordId;
 
     const bool _recoveredFromDisk;
     const boost::optional<mongo::ForwardableOperationMetadata> _forwardableOpMetadata;
     const boost::optional<mongo::DatabaseVersion> _databaseVersion;
+    boost::optional<NamespaceString> _bucketNss;
 
     bool _firstExecution{
         true};  // True only when executing the coordinator for the first time (meaning it's not a
@@ -156,7 +205,7 @@ protected:
 
 private:
     SemiFuture<void> run(std::shared_ptr<executor::ScopedTaskExecutor> executor,
-                         const CancellationToken& token) noexcept override final;
+                         const CancellationToken& token) noexcept final;
 
     virtual ExecutorFuture<void> _runImpl(std::shared_ptr<executor::ScopedTaskExecutor> executor,
                                           const CancellationToken& token) noexcept = 0;
@@ -166,16 +215,24 @@ private:
         const CancellationToken& token,
         const Status& status) noexcept;
 
-    void interrupt(Status status) override final;
+    void interrupt(Status status) final;
 
     bool _removeDocument(OperationContext* opCtx);
 
     ExecutorFuture<bool> _removeDocumentUntillSuccessOrStepdown(
         std::shared_ptr<executor::TaskExecutor> executor);
 
+    ExecutorFuture<void> _acquireAllLocksAsync(
+        OperationContext* opCtx,
+        std::shared_ptr<executor::ScopedTaskExecutor> executor,
+        const CancellationToken& token);
+
+    template <typename T>
     ExecutorFuture<void> _acquireLockAsync(std::shared_ptr<executor::ScopedTaskExecutor> executor,
                                            const CancellationToken& token,
-                                           StringData resource);
+                                           const T& resource,
+                                           LockMode lockMode);
+
     ExecutorFuture<void> _translateTimeseriesNss(
         std::shared_ptr<executor::ScopedTaskExecutor> executor, const CancellationToken& token);
 
@@ -185,7 +242,15 @@ private:
     SharedPromise<void> _constructionCompletionPromise;
     SharedPromise<void> _completionPromise;
 
-    std::stack<DDLLockManager::ScopedLock> _scopedLocks;
+    // A Locker object works attached to an opCtx and it's destroyed once the opCtx gets out of
+    // scope. However, we must keep alive a unique Locker object during the whole
+    // ShardingDDLCoordinator life to preserve the lock state among all the executor tasks.
+    std::unique_ptr<Locker> _locker;
+
+    std::stack<DDLLockManager::ScopedBaseDDLLock> _scopedLocks;
+    std::shared_ptr<ShardingDDLCoordinatorExternalState> _externalState;
+
+    friend class ShardingDDLCoordinatorTest;
 };
 
 template <class StateDoc>
@@ -228,7 +293,9 @@ protected:
 
         // Append static info
         bob.append("type", "op");
-        bob.append("ns", originalNss().toString());
+        bob.append(
+            "ns",
+            NamespaceStringUtil::serialize(originalNss(), SerializationContext::stateDefault()));
         bob.append("desc", _coordinatorName);
         bob.append("op", "command");
         bob.append("active", true);
@@ -238,7 +305,9 @@ protected:
             stdx::lock_guard lk{_docMutex};
             if (const auto& bucketNss = _doc.getBucketNss()) {
                 // Bucket namespace is only present in case the collection is a sharded timeseries
-                bob.append("bucketNamespace", bucketNss.get().toString());
+                bob.append("bucketNamespace",
+                           NamespaceStringUtil::serialize(bucketNss.get(),
+                                                          SerializationContext::stateDefault()));
             }
         }
 
@@ -256,6 +325,23 @@ protected:
         return bob;
     }
 
+    /**
+     * Returns all shards, including the config server. Config servers are special because they
+     * are more likely to be removed as a shard and added back again later. This is mainly used
+     * for ensuring that the config server release the critical sections/locks created by the
+     * ddl operation.
+     */
+    std::vector<ShardId> getAllShardsAndConfigServerIds(OperationContext* opCtx) {
+        auto shardIds = Grid::get(opCtx)->shardRegistry()->getAllShardIds(opCtx);
+
+        auto iter = std::find(shardIds.begin(), shardIds.end(), ShardId::kConfigServerId);
+        if (iter == shardIds.end()) {
+            shardIds.emplace_back(ShardId::kConfigServerId);
+        }
+
+        return shardIds;
+    }
+
     const std::string _coordinatorName;
     const BSONObj _initialState;
     mutable Mutex _docMutex = MONGO_MAKE_LATCH("ShardingDDLCoordinator::_docMutex");
@@ -277,7 +363,7 @@ protected:
 
     template <typename Func>
     auto _buildPhaseHandler(const Phase& newPhase, Func&& handlerFn) {
-        return [=] {
+        return [=, this] {
             const auto& currPhase = _doc.getPhase();
 
             if (currPhase > newPhase) {
@@ -292,11 +378,13 @@ protected:
         };
     }
 
+    auto _getDoc() const {
+        stdx::lock_guard lk{_docMutex};
+        return _doc;
+    }
+
     virtual void _enterPhase(const Phase& newPhase) {
-        auto newDoc = [&] {
-            stdx::lock_guard lk{_docMutex};
-            return _doc;
-        }();
+        auto newDoc = _getDoc();
 
         newDoc.setPhase(newPhase);
 
@@ -348,7 +436,7 @@ protected:
             const auto replCoord = repl::ReplicationCoordinator::get(opCtx);
             const auto lastLocalOpTime = replCoord->getMyLastAppliedOpTime();
             WaitForMajorityService::get(opCtx->getServiceContext())
-                .waitUntilMajority(lastLocalOpTime, opCtx->getCancellationToken())
+                .waitUntilMajorityForWrite(lastLocalOpTime, opCtx->getCancellationToken())
                 .get(opCtx);
         }
 
@@ -381,7 +469,7 @@ protected:
         return getCurrentSession();
     }
 
-    virtual boost::optional<Status> getAbortReason() const override {
+    boost::optional<Status> getAbortReason() const override {
         const auto& status = _doc.getAbortReason();
         invariant(!status || !status->isOK(), "when persisted, status must be an error");
         return status;
@@ -398,10 +486,7 @@ protected:
                    "phase"_attr = serializePhase(_doc.getPhase()),
                    "reason"_attr = redact(status));
 
-        auto newDoc = [&] {
-            stdx::lock_guard lk{_docMutex};
-            return _doc;
-        }();
+        auto newDoc = _getDoc();
 
         auto coordinatorMetadata = newDoc.getShardingDDLCoordinatorMetadata();
         coordinatorMetadata.setAbortReason(status);
@@ -415,10 +500,7 @@ protected:
 private:
     // lazily acquire Logical Session ID and a txn number
     void _updateSession(OperationContext* opCtx) {
-        auto newDoc = [&] {
-            stdx::lock_guard lk{_docMutex};
-            return _doc;
-        }();
+        auto newDoc = _getDoc();
         auto newShardingDDLCoordinatorMetadata = newDoc.getShardingDDLCoordinatorMetadata();
 
         auto optSession = newShardingDDLCoordinatorMetadata.getSession();
