@@ -30,25 +30,57 @@
 
 #include "mongo/db/s/sharding_ddl_coordinator_service.h"
 
+#include <boost/smart_ptr.hpp>
+#include <mutex>
+#include <string>
+#include <tuple>
+#include <type_traits>
+#include <utility>
+
+#include <absl/container/node_hash_map.h>
+#include <absl/meta/type_traits.h>
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
 #include "mongo/base/checked_cast.h"
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/client/dbclient_cursor.h"
+#include "mongo/db/client.h"
 #include "mongo/db/dbdirectclient.h"
 #include "mongo/db/pipeline/aggregate_command_gen.h"
+#include "mongo/db/s/cleanup_structured_encryption_data_coordinator.h"
 #include "mongo/db/s/collmod_coordinator.h"
 #include "mongo/db/s/compact_structured_encryption_data_coordinator.h"
+#include "mongo/db/s/convert_to_capped_coordinator.h"
 #include "mongo/db/s/create_collection_coordinator.h"
 #include "mongo/db/s/database_sharding_state.h"
+#include "mongo/db/s/ddl_lock_manager.h"
 #include "mongo/db/s/drop_collection_coordinator.h"
 #include "mongo/db/s/drop_database_coordinator.h"
+#include "mongo/db/s/forwardable_operation_metadata.h"
+#include "mongo/db/s/migration_blocking_operation/migration_blocking_operation_coordinator.h"
 #include "mongo/db/s/move_primary_coordinator.h"
-#include "mongo/db/s/move_primary_coordinator_no_resilient.h"
 #include "mongo/db/s/operation_sharding_state.h"
 #include "mongo/db/s/refine_collection_shard_key_coordinator.h"
 #include "mongo/db/s/rename_collection_coordinator.h"
 #include "mongo/db/s/reshard_collection_coordinator.h"
 #include "mongo/db/s/set_allow_migrations_coordinator.h"
 #include "mongo/db/s/sharding_ddl_coordinator.h"
+#include "mongo/db/s/untrack_unsplittable_collection_coordinator.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/redaction.h"
+#include "mongo/s/database_version.h"
+#include "mongo/s/sharding_cluster_parameters_gen.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/fail_point.h"
+#include "mongo/util/future_impl.h"
+#include "mongo/util/str.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
@@ -64,59 +96,46 @@ std::shared_ptr<ShardingDDLCoordinator> constructShardingDDLCoordinatorInstance(
     LOGV2(
         5390510, "Constructing new sharding DDL coordinator", "coordinatorDoc"_attr = op.toBSON());
     switch (op.getId().getOperationType()) {
-        // TODO (SERVER-71309): Remove once 7.0 becomes last LTS.
-        case DDLCoordinatorTypeEnum::kMovePrimaryNoResilient:
-            return std::make_shared<MovePrimaryCoordinatorNoResilient>(service,
-                                                                       std::move(initialState));
-            break;
         case DDLCoordinatorTypeEnum::kMovePrimary:
             return std::make_shared<MovePrimaryCoordinator>(service, std::move(initialState));
-            break;
-        // TODO SERVER-73627: Remove once 7.0 becomes last LTS.
         case DDLCoordinatorTypeEnum::kDropDatabase:
-        case DDLCoordinatorTypeEnum::kDropDatabasePre70Compatible:
             return std::make_shared<DropDatabaseCoordinator>(service, std::move(initialState));
-            break;
-        // TODO SERVER-73627: Remove once 7.0 becomes last LTS.
         case DDLCoordinatorTypeEnum::kDropCollection:
-        case DDLCoordinatorTypeEnum::kDropCollectionPre70Compatible:
             return std::make_shared<DropCollectionCoordinator>(service, std::move(initialState));
-            break;
         case DDLCoordinatorTypeEnum::kRenameCollection:
-        case DDLCoordinatorTypeEnum::kRenameCollectionPre70Compatible:
             return std::make_shared<RenameCollectionCoordinator>(service, std::move(initialState));
         case DDLCoordinatorTypeEnum::kCreateCollection:
-        // TODO SERVER-68008 Remove the Pre61Compatible case once 7.0 becomes last LTS
-        case DDLCoordinatorTypeEnum::kCreateCollectionPre61Compatible:
             return std::make_shared<CreateCollectionCoordinator>(service, std::move(initialState));
-            break;
+        case DDLCoordinatorTypeEnum::kRefineCollectionShardKeyPre71Compatible:
+            return std::make_shared<RefineCollectionShardKeyCoordinatorPre71Compatible>(
+                service, std::move(initialState));
+        case DDLCoordinatorTypeEnum::kCreateCollectionPre80Compatible:
+            return std::make_shared<CreateCollectionCoordinatorLegacy>(service,
+                                                                       std::move(initialState));
         case DDLCoordinatorTypeEnum::kRefineCollectionShardKey:
             return std::make_shared<RefineCollectionShardKeyCoordinator>(service,
                                                                          std::move(initialState));
-            break;
         case DDLCoordinatorTypeEnum::kSetAllowMigrations:
             return std::make_shared<SetAllowMigrationsCoordinator>(service,
                                                                    std::move(initialState));
-            break;
         case DDLCoordinatorTypeEnum::kCollMod:
-        case DDLCoordinatorTypeEnum::kCollModPre61Compatible:  // TODO SERVER-68008 Remove once 7.0
-                                                               // becomes last LTS
             return std::make_shared<CollModCoordinator>(service, std::move(initialState));
-            break;
         case DDLCoordinatorTypeEnum::kReshardCollection:
             return std::make_shared<ReshardCollectionCoordinator>(service, std::move(initialState));
-            break;
-        case DDLCoordinatorTypeEnum::kCompactStructuredEncryptionDataPre61Compatible:
-            // TODO SERVER-68373 remove once 7.0 becomes last LTS
-        case DDLCoordinatorTypeEnum::kCompactStructuredEncryptionDataPre70Compatible:
-            // TODO SERVER-68373 remove once 7.0 becomes last LTS
-            return std::make_shared<CompactStructuredEncryptionDataCoordinatorPre70Compatible>(
-                service, std::move(initialState));
-            break;
         case DDLCoordinatorTypeEnum::kCompactStructuredEncryptionData:
             return std::make_shared<CompactStructuredEncryptionDataCoordinator>(
                 service, std::move(initialState));
-            break;
+        case DDLCoordinatorTypeEnum::kCleanupStructuredEncryptionData:
+            return std::make_shared<CleanupStructuredEncryptionDataCoordinator>(
+                service, std::move(initialState));
+        case DDLCoordinatorTypeEnum::kMigrationBlockingOperation:
+            return std::make_shared<MigrationBlockingOperationCoordinator>(service,
+                                                                           std::move(initialState));
+        case DDLCoordinatorTypeEnum::kConvertToCapped:
+            return std::make_shared<ConvertToCappedCoordinator>(service, std::move(initialState));
+        case DDLCoordinatorTypeEnum::kUntrackUnsplittableCollection:
+            return std::make_shared<UntrackUnsplittableCollectionCoordinator>(
+                service, std::move(initialState));
         default:
             uasserted(ErrorCodes::BadValue,
                       str::stream()
@@ -181,6 +200,11 @@ ShardingDDLCoordinatorService::constructInstance(BSONObj initialState) {
     return coord;
 }
 
+std::shared_ptr<ShardingDDLCoordinatorExternalState>
+ShardingDDLCoordinatorService::createExternalState() const {
+    return _externalStateFactory->create();
+}
+
 void ShardingDDLCoordinatorService::waitForCoordinatorsOfGivenTypeToComplete(
     OperationContext* opCtx, DDLCoordinatorTypeEnum type) const {
     stdx::unique_lock lk(_mutex);
@@ -297,7 +321,9 @@ ExecutorFuture<void> ShardingDDLCoordinatorService::_rebuildService(
 }
 
 std::shared_ptr<ShardingDDLCoordinatorService::Instance>
-ShardingDDLCoordinatorService::getOrCreateInstance(OperationContext* opCtx, BSONObj coorDoc) {
+ShardingDDLCoordinatorService::getOrCreateInstance(OperationContext* opCtx,
+                                                   BSONObj coorDoc,
+                                                   bool checkOptions) {
 
     // Wait for all coordinators to be recovered before to allow the creation of new ones.
     waitForRecoveryCompletion(opCtx);
@@ -307,7 +333,7 @@ ShardingDDLCoordinatorService::getOrCreateInstance(OperationContext* opCtx, BSON
 
     if (!nss.isConfigDB() && !nss.isAdminDB()) {
         // Check that the operation context has a database version for this namespace
-        const auto clientDbVersion = OperationShardingState::get(opCtx).getDbVersion(nss.db());
+        const auto clientDbVersion = OperationShardingState::get(opCtx).getDbVersion(nss.dbName());
         uassert(ErrorCodes::IllegalOperation,
                 "Request sent without attaching database version",
                 clientDbVersion);
@@ -324,18 +350,27 @@ ShardingDDLCoordinatorService::getOrCreateInstance(OperationContext* opCtx, BSON
     const auto patchedCoorDoc = coorDoc.addFields(coorMetadata.toBSON());
 
     auto [coordinator, created] = [&] {
-        try {
-            auto [coordinator, created] =
-                PrimaryOnlyService::getOrCreateInstance(opCtx, patchedCoorDoc);
-            return std::make_pair(
-                checked_pointer_cast<ShardingDDLCoordinator>(std::move(coordinator)),
-                std::move(created));
-        } catch (const DBException& ex) {
-            LOGV2_ERROR(5390512,
-                        "Failed to create instance of sharding DDL coordinator",
-                        "coordinatorId"_attr = coorMetadata.getId(),
-                        "reason"_attr = redact(ex));
-            throw;
+        while (true) {
+            try {
+                auto [coordinator, created] =
+                    PrimaryOnlyService::getOrCreateInstance(opCtx, patchedCoorDoc, checkOptions);
+                return std::make_pair(
+                    checked_pointer_cast<ShardingDDLCoordinator>(std::move(coordinator)),
+                    std::move(created));
+            } catch (const ExceptionFor<ErrorCodes::AddOrRemoveShardInProgress>&) {
+                LOGV2_WARNING(5687900,
+                              "Cannot start sharding DDL coordinator because a topology change is "
+                              "in progress. Will retry after backoff.");
+                // Backoff
+                opCtx->sleepFor(Seconds(1));
+                continue;
+            } catch (const DBException& ex) {
+                LOGV2_ERROR(5390512,
+                            "Failed to create instance of sharding DDL coordinator",
+                            "coordinatorId"_attr = coorMetadata.getId(),
+                            "reason"_attr = redact(ex));
+                throw;
+            }
         }
     }();
 
@@ -354,4 +389,19 @@ void ShardingDDLCoordinatorService::_transitionToRecovered(WithLock lk, Operatio
     _recoveredOrCoordinatorCompletedCV.notify_all();
 }
 
+void ShardingDDLCoordinatorService::checkIfConflictsWithOtherInstances(
+    OperationContext* opCtx,
+    BSONObj initialState,
+    const std::vector<const PrimaryOnlyService::Instance*>& existingInstances) {
+    auto* addOrRemoveShardInProgressParam =
+        ServerParameterSet::getClusterParameterSet()
+            ->get<ClusterParameterWithStorage<AddOrRemoveShardInProgressParam>>(
+                "addOrRemoveShardInProgress");
+    const auto addOrRemoveShardInProgressVal =
+        addOrRemoveShardInProgressParam->getValue(boost::none).getInProgress();
+
+    uassert(ErrorCodes::AddOrRemoveShardInProgress,
+            "Cannot start ShardingDDLCoordinator because a topology change is in progress",
+            !addOrRemoveShardInProgressVal);
+}
 }  // namespace mongo
