@@ -29,45 +29,73 @@
 
 
 #include <fmt/format.h>
+#include <memory>
 #include <set>
+#include <utility>
+#include <vector>
 
-#include "mongo/db/s/sharding_recovery_service.h"
+#include <absl/container/node_hash_map.h>
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
 
-#include "mongo/db/db_raii.h"
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/client/dbclient_cursor.h"
+#include "mongo/db/catalog/collection.h"
+#include "mongo/db/catalog_raii.h"
+#include "mongo/db/concurrency/d_concurrency.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
 #include "mongo/db/dbdirectclient.h"
+#include "mongo/db/exec/document_value/document.h"
+#include "mongo/db/exec/document_value/value.h"
+#include "mongo/db/ops/write_ops_gen.h"
+#include "mongo/db/ops/write_ops_parsers.h"
 #include "mongo/db/persistent_task_store.h"
+#include "mongo/db/pipeline/aggregate_command_gen.h"
 #include "mongo/db/pipeline/document_source_lookup.h"
 #include "mongo/db/pipeline/document_source_match.h"
+#include "mongo/db/pipeline/expression_context.h"
+#include "mongo/db/pipeline/pipeline.h"
+#include "mongo/db/query/find_command.h"
+#include "mongo/db/repl/member_state.h"
 #include "mongo/db/repl/repl_client_info.h"
+#include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/s/collection_critical_section_document_gen.h"
 #include "mongo/db/s/collection_sharding_runtime.h"
+#include "mongo/db/s/collection_sharding_state.h"
 #include "mongo/db/s/database_sharding_state.h"
 #include "mongo/db/s/shard_authoritative_catalog_gen.h"
-#include "mongo/db/s/sharding_migration_critical_section.h"
-#include "mongo/db/server_options.h"
+#include "mongo/db/s/sharding_recovery_service.h"
+#include "mongo/db/transaction_resources.h"
+#include "mongo/db/write_concern.h"
+#include "mongo/idl/idl_parser.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/rpc/get_status_from_command_result.h"
+#include "mongo/rpc/reply_interface.h"
+#include "mongo/rpc/unique_message.h"
 #include "mongo/s/catalog/type_collection.h"
-#include "mongo/s/write_ops/batched_command_request.h"
+#include "mongo/s/catalog/type_collection_gen.h"
+#include "mongo/s/catalog/type_index_catalog_gen.h"
 #include "mongo/s/write_ops/batched_command_response.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/intrusive_counter.h"
+#include "mongo/util/namespace_string_util.h"
+#include "mongo/util/string_map.h"
+#include "mongo/util/uuid.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
 namespace mongo {
 
-namespace sharding_recovery_util {
-
-bool inRecoveryMode(OperationContext* opCtx) {
-    const auto replCoord = repl::ReplicationCoordinator::get(opCtx);
-    if (!replCoord->isReplEnabled()) {
-        return false;
-    }
-
-    const auto memberState = replCoord->getMemberState();
-    return memberState.startup2() || memberState.rollback();
-}
-
-}  // namespace sharding_recovery_util
-
 namespace {
 const StringData kShardingIndexCatalogEntriesFieldName = "indexes"_sd;
 const auto serviceDecorator = ServiceContext::declareDecoration<ShardingRecoveryService>();
@@ -75,10 +103,10 @@ const auto serviceDecorator = ServiceContext::declareDecoration<ShardingRecovery
 AggregateCommandRequest makeCollectionsAndIndexesAggregation(OperationContext* opCtx) {
     auto expCtx = make_intrusive<ExpressionContext>(
         opCtx, nullptr, NamespaceString::kShardCollectionCatalogNamespace);
-    ResolvedNamespaceMap resolvedNamespaces;
-    resolvedNamespaces[NamespaceString::kShardCollectionCatalogNamespace] = {
+    StringMap<ExpressionContext::ResolvedNamespace> resolvedNamespaces;
+    resolvedNamespaces[NamespaceString::kShardCollectionCatalogNamespace.coll()] = {
         NamespaceString::kShardCollectionCatalogNamespace, std::vector<BSONObj>()};
-    resolvedNamespaces[NamespaceString::kShardIndexCatalogNamespace] = {
+    resolvedNamespaces[NamespaceString::kShardIndexCatalogNamespace.coll()] = {
         NamespaceString::kShardIndexCatalogNamespace, std::vector<BSONObj>()};
     expCtx->setResolvedNamespaces(resolvedNamespaces);
 
@@ -129,6 +157,39 @@ AggregateCommandRequest makeCollectionsAndIndexesAggregation(OperationContext* o
 }
 }  // namespace
 
+ShardingRecoveryService::FilteringMetadataClearer::FilteringMetadataClearer(
+    bool includeStepsForNamespaceDropped)
+    : _includeStepsForNamespaceDropped(includeStepsForNamespaceDropped) {}
+
+void ShardingRecoveryService::FilteringMetadataClearer::operator()(
+    OperationContext* opCtx, const NamespaceString& nssBeingReleased) const {
+    if (nssBeingReleased.isNamespaceAlwaysUntracked()) {
+        return;
+    }
+
+    if (nssBeingReleased.isDbOnly()) {
+        AutoGetDb autoDb(opCtx, nssBeingReleased.dbName(), MODE_IX);
+        auto scopedDss = DatabaseShardingState::assertDbLockedAndAcquireExclusive(
+            opCtx, nssBeingReleased.dbName());
+        scopedDss->clearDbInfo(opCtx);
+        return;
+    }
+
+    AutoGetCollection autoColl(
+        opCtx,
+        nssBeingReleased,
+        MODE_IX,
+        AutoGetCollection::Options{}.viewMode(auto_get_collection::ViewMode::kViewsPermitted));
+
+    auto scopedCsr = CollectionShardingRuntime::assertCollectionLockedAndAcquireExclusive(
+        opCtx, nssBeingReleased);
+    if (_includeStepsForNamespaceDropped) {
+        scopedCsr->clearFilteringMetadataForDroppedCollection(opCtx);
+    } else {
+        scopedCsr->clearFilteringMetadata(opCtx);
+    }
+}
+
 ShardingRecoveryService* ShardingRecoveryService::get(ServiceContext* serviceContext) {
     return &serviceDecorator(serviceContext);
 }
@@ -144,8 +205,7 @@ void ShardingRecoveryService::acquireRecoverableCriticalSectionBlockWrites(
     OperationContext* opCtx,
     const NamespaceString& nss,
     const BSONObj& reason,
-    const WriteConcernOptions& writeConcern,
-    bool allowViews) {
+    const WriteConcernOptions& writeConcern) {
     LOGV2_DEBUG(5656600,
                 3,
                 "Acquiring recoverable critical section blocking writes",
@@ -156,31 +216,38 @@ void ShardingRecoveryService::acquireRecoverableCriticalSectionBlockWrites(
     tassert(7032360,
             fmt::format("Can't acquire recoverable critical section for collection '{}' with "
                         "reason '{}' while holding locks",
-                        nss.toString(),
+                        nss.toStringForErrorMsg(),
                         reason.toString()),
-            !opCtx->lockState()->isLocked());
+            !shard_role_details::getLocker(opCtx)->isLocked());
 
     {
         Lock::GlobalLock lk(opCtx, MODE_IX);
         boost::optional<AutoGetDb> dbLock;
         boost::optional<AutoGetCollection> collLock;
-        if (nsIsDbOnly(nss.ns())) {
+        if (nss.isDbOnly()) {
+            tassert(8096300,
+                    "Cannot acquire critical section on the config database",
+                    !nss.isConfigDB());
             dbLock.emplace(opCtx, nss.dbName(), MODE_S);
         } else {
-            // TODO SERVER-68084 add the AutoGetCollectionViewMode::kViewsPermitted parameter to
-            // construct collLock.
+            if (nss.isConfigDB()) {
+                // Take the 'config' database lock in mode IX to prevent lock upgrade when we later
+                // write to kCollectionCriticalSectionsNamespace.
+                dbLock.emplace(opCtx, nss.dbName(), MODE_IX);
+            }
+
             collLock.emplace(opCtx,
                              nss,
                              MODE_S,
-                             (allowViews ? AutoGetCollection::Options{}.viewMode(
-                                               auto_get_collection::ViewMode::kViewsPermitted)
-                                         : AutoGetCollection::Options{}));
+                             AutoGetCollection::Options{}.viewMode(
+                                 auto_get_collection::ViewMode::kViewsPermitted));
         }
 
         DBDirectClient dbClient(opCtx);
         FindCommandRequest findRequest{NamespaceString::kCollectionCriticalSectionsNamespace};
         findRequest.setFilter(
-            BSON(CollectionCriticalSectionDocument::kNssFieldName << nss.toString()));
+            BSON(CollectionCriticalSectionDocument::kNssFieldName
+                 << NamespaceStringUtil::serialize(nss, SerializationContext::stateDefault())));
         auto cursor = dbClient.find(std::move(findRequest));
 
         // if there is a doc with the same nss -> in order to not fail it must have the same
@@ -194,7 +261,7 @@ void ShardingRecoveryService::acquireRecoverableCriticalSectionBlockWrites(
                     fmt::format("Trying to acquire a  critical section blocking writes for "
                                 "namespace '{}' and reason '{}' but it is already taken by another "
                                 "operation with different reason '{}'",
-                                nss.toString(),
+                                nss.toStringForErrorMsg(),
                                 reason.toString(),
                                 collCSDoc.getReason().toString()),
                     collCSDoc.getReason().woCompare(reason) == 0);
@@ -233,11 +300,12 @@ void ShardingRecoveryService::acquireRecoverableCriticalSectionBlockWrites(
         std::string unusedErrmsg;
         batchedResponse.parseBSON(commandReply, &unusedErrmsg);
         tassert(7032369,
-                fmt::format("Insert did not add any doc to collection '{}' for namespace '{}' "
-                            "and reason '{}'",
-                            nss.toString(),
-                            reason.toString(),
-                            NamespaceString::kCollectionCriticalSectionsNamespace.toString()),
+                fmt::format(
+                    "Insert did not add any doc to collection '{}' for namespace '{}' "
+                    "and reason '{}'",
+                    nss.toStringForErrorMsg(),
+                    reason.toString(),
+                    NamespaceString::kCollectionCriticalSectionsNamespace.toStringForErrorMsg()),
                 batchedResponse.getN() > 0);
     }
 
@@ -257,8 +325,7 @@ void ShardingRecoveryService::promoteRecoverableCriticalSectionToBlockAlsoReads(
     OperationContext* opCtx,
     const NamespaceString& nss,
     const BSONObj& reason,
-    const WriteConcernOptions& writeConcern,
-    bool allowViews) {
+    const WriteConcernOptions& writeConcern) {
     LOGV2_DEBUG(5656603,
                 3,
                 "Promoting recoverable critical section to also block reads",
@@ -269,37 +336,35 @@ void ShardingRecoveryService::promoteRecoverableCriticalSectionToBlockAlsoReads(
     tassert(7032364,
             fmt::format("Can't promote recoverable critical section for collection '{}' with "
                         "reason '{}' while holding locks",
-                        nss.toString(),
+                        nss.toStringForErrorMsg(),
                         reason.toString()),
-            !opCtx->lockState()->isLocked());
+            !shard_role_details::getLocker(opCtx)->isLocked());
 
     {
         boost::optional<AutoGetDb> dbLock;
         boost::optional<AutoGetCollection> collLock;
-        if (nsIsDbOnly(nss.ns())) {
+        if (nss.isDbOnly()) {
             dbLock.emplace(opCtx, nss.dbName(), MODE_X);
         } else {
-            // TODO SERVER-68084 add the AutoGetCollectionViewMode::kViewsPermitted parameter to
-            // construct collLock.
             collLock.emplace(opCtx,
                              nss,
                              MODE_X,
-                             (allowViews ? AutoGetCollection::Options{}.viewMode(
-                                               auto_get_collection::ViewMode::kViewsPermitted)
-                                         : AutoGetCollection::Options{}));
+                             AutoGetCollection::Options{}.viewMode(
+                                 auto_get_collection::ViewMode::kViewsPermitted));
         }
 
         DBDirectClient dbClient(opCtx);
         FindCommandRequest findRequest{NamespaceString::kCollectionCriticalSectionsNamespace};
         findRequest.setFilter(
-            BSON(CollectionCriticalSectionDocument::kNssFieldName << nss.toString()));
+            BSON(CollectionCriticalSectionDocument::kNssFieldName
+                 << NamespaceStringUtil::serialize(nss, SerializationContext::stateDefault())));
         auto cursor = dbClient.find(std::move(findRequest));
 
         tassert(7032361,
                 fmt::format(
                     "Trying to acquire a critical section blocking reads for namespace '{}' and "
                     "reason '{}' but the critical section wasn't acquired first blocking writers.",
-                    nss.toString(),
+                    nss.toStringForErrorMsg(),
                     reason.toString()),
                 cursor->more());
         BSONObj bsonObj = cursor->next();
@@ -311,7 +376,7 @@ void ShardingRecoveryService::promoteRecoverableCriticalSectionToBlockAlsoReads(
                     "Trying to acquire a critical section blocking reads for namespace '{}' and "
                     "reason "
                     "'{}' but it is already taken by another operation with different reason '{}'",
-                    nss.toString(),
+                    nss.toStringForErrorMsg(),
                     reason.toString(),
                     collCSDoc.getReason().toString()),
                 collCSDoc.getReason().woCompare(reason) == 0);
@@ -338,9 +403,10 @@ void ShardingRecoveryService::promoteRecoverableCriticalSectionToBlockAlsoReads(
         // - Otherwise this call will fail and the CS won't be advanced (neither persisted nor
         // in-mem)
         auto commandResponse = dbClient.runCommand([&] {
-            const auto query = BSON(
-                CollectionCriticalSectionDocument::kNssFieldName
-                << nss.toString() << CollectionCriticalSectionDocument::kReasonFieldName << reason);
+            const auto query =
+                BSON(CollectionCriticalSectionDocument::kNssFieldName
+                     << NamespaceStringUtil::serialize(nss, SerializationContext::stateDefault())
+                     << CollectionCriticalSectionDocument::kReasonFieldName << reason);
             const auto update = BSON(
                 "$set" << BSON(CollectionCriticalSectionDocument::kBlockReadsFieldName << true));
 
@@ -359,13 +425,14 @@ void ShardingRecoveryService::promoteRecoverableCriticalSectionToBlockAlsoReads(
         BatchedCommandResponse batchedResponse;
         std::string unusedErrmsg;
         batchedResponse.parseBSON(commandReply, &unusedErrmsg);
-        tassert(7032363,
-                fmt::format("Update did not modify any doc from collection '{}' for namespace '{}' "
-                            "and reason '{}'",
-                            NamespaceString::kCollectionCriticalSectionsNamespace.toString(),
-                            nss.toString(),
-                            reason.toString()),
-                batchedResponse.getNModified() > 0);
+        tassert(
+            7032363,
+            fmt::format("Update did not modify any doc from collection '{}' for namespace '{}' "
+                        "and reason '{}'",
+                        NamespaceString::kCollectionCriticalSectionsNamespace.toStringForErrorMsg(),
+                        nss.toStringForErrorMsg(),
+                        reason.toString()),
+            batchedResponse.getNModified() > 0);
     }
 
     WriteConcernResult ignoreResult;
@@ -385,8 +452,8 @@ void ShardingRecoveryService::releaseRecoverableCriticalSection(
     const NamespaceString& nss,
     const BSONObj& reason,
     const WriteConcernOptions& writeConcern,
-    bool throwIfReasonDiffers,
-    bool allowViews) {
+    const BeforeReleasingCustomAction& beforeReleasingAction,
+    bool throwIfReasonDiffers) {
     LOGV2_DEBUG(5656606,
                 3,
                 "Releasing recoverable critical section",
@@ -397,30 +464,28 @@ void ShardingRecoveryService::releaseRecoverableCriticalSection(
     tassert(7032365,
             fmt::format("Can't release recoverable critical section for collection '{}' with "
                         "reason '{}' while holding locks",
-                        nss.toString(),
+                        nss.toStringForErrorMsg(),
                         reason.toString()),
-            !opCtx->lockState()->isLocked());
+            !shard_role_details::getLocker(opCtx)->isLocked());
 
     {
         boost::optional<AutoGetDb> dbLock;
         boost::optional<AutoGetCollection> collLock;
-        if (nsIsDbOnly(nss.ns())) {
+        if (nss.isDbOnly()) {
             dbLock.emplace(opCtx, nss.dbName(), MODE_X);
         } else {
-            // TODO SERVER-68084 add the AutoGetCollectionViewMode::kViewsPermitted parameter to
-            // construct collLock.
             collLock.emplace(opCtx,
                              nss,
                              MODE_X,
-                             (allowViews ? AutoGetCollection::Options{}.viewMode(
-                                               auto_get_collection::ViewMode::kViewsPermitted)
-                                         : AutoGetCollection::Options{}));
+                             AutoGetCollection::Options{}.viewMode(
+                                 auto_get_collection::ViewMode::kViewsPermitted));
         }
 
         DBDirectClient dbClient(opCtx);
 
         const auto queryNss =
-            BSON(CollectionCriticalSectionDocument::kNssFieldName << nss.toString());
+            BSON(CollectionCriticalSectionDocument::kNssFieldName
+                 << NamespaceStringUtil::serialize(nss, SerializationContext::stateDefault()));
         FindCommandRequest findRequest{NamespaceString::kCollectionCriticalSectionsNamespace};
         findRequest.setFilter(queryNss);
         auto cursor = dbClient.find(std::move(findRequest));
@@ -456,13 +521,15 @@ void ShardingRecoveryService::releaseRecoverableCriticalSection(
         tassert(7032366,
                 fmt::format("Trying to release a critical for namespace '{}' and reason '{}' but "
                             "it is already taken by another operation with different reason '{}'",
-                            nss.toString(),
+                            nss.toStringForErrorMsg(),
                             reason.toString(),
                             collCSDoc.getReason().toString()),
                 !isDifferentReason);
 
+        // The collection critical section is taken (in any phase), perform the custom action then
+        // try to release it.
 
-        // The collection critical section is taken (in any phase), try to release it.
+        beforeReleasingAction(opCtx, nss);
 
         // The following code will try to remove a doc from config.criticalCollectionSections:
         // - If everything goes well, the shard server op observer will release the in-memory CS
@@ -489,13 +556,14 @@ void ShardingRecoveryService::releaseRecoverableCriticalSection(
         BatchedCommandResponse batchedResponse;
         std::string unusedErrmsg;
         batchedResponse.parseBSON(commandReply, &unusedErrmsg);
-        tassert(7032367,
-                fmt::format("Delete did not remove any doc from collection '{}' for namespace '{}' "
-                            "and reason '{}'",
-                            NamespaceString::kCollectionCriticalSectionsNamespace.toString(),
-                            nss.toString(),
-                            reason.toString()),
-                batchedResponse.getN() > 0);
+        tassert(
+            7032367,
+            fmt::format("Delete did not remove any doc from collection '{}' for namespace '{}' "
+                        "and reason '{}'",
+                        NamespaceString::kCollectionCriticalSectionsNamespace.toStringForErrorMsg(),
+                        nss.toStringForErrorMsg(),
+                        reason.toString()),
+            batchedResponse.getN() > 0);
     }
 
     WriteConcernResult ignoreResult;
@@ -536,7 +604,7 @@ void ShardingRecoveryService::recoverRecoverableCriticalSections(OperationContex
     store.forEach(opCtx, BSONObj{}, [&opCtx](const CollectionCriticalSectionDocument& doc) {
         const auto& nss = doc.getNss();
         {
-            if (nsIsDbOnly(nss.ns())) {
+            if (nss.isDbOnly()) {
                 AutoGetDb dbLock(opCtx, nss.dbName(), MODE_X);
                 auto scopedDss =
                     DatabaseShardingState::assertDbLockedAndAcquireExclusive(opCtx, nss.dbName());
@@ -616,7 +684,10 @@ void ShardingRecoveryService::recoverIndexesCatalog(OperationContext* opCtx) {
 
     while (cursor->more()) {
         auto doc = cursor->nextSafe();
-        auto nss = NamespaceString(doc[CollectionType::kNssFieldName].String());
+        const auto nss =
+            NamespaceStringUtil::deserialize(boost::none,
+                                             doc[CollectionType::kNssFieldName].String(),
+                                             SerializationContext::stateDefault());
         auto indexVersion = doc[CollectionType::kIndexVersionFieldName].timestamp();
         for (const auto& idx : doc[kShardingIndexCatalogEntriesFieldName].Array()) {
             auto indexEntry = IndexCatalogType::parse(
