@@ -28,20 +28,81 @@
  */
 
 
+#include <cstdint>
+#include <iterator>
+#include <memory>
+#include <string>
+#include <utility>
+#include <vector>
+
+#include <absl/container/node_hash_map.h>
+#include <boost/cstdint.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/client/read_preference.h"
+#include "mongo/db/api_parameters.h"
+#include "mongo/db/auth/action_type.h"
 #include "mongo/db/auth/authorization_session.h"
+#include "mongo/db/auth/privilege.h"
+#include "mongo/db/auth/resource_pattern.h"
+#include "mongo/db/basic_types_gen.h"
 #include "mongo/db/clientcursor.h"
 #include "mongo/db/commands.h"
-#include "mongo/db/s/database_sharding_state.h"
+#include "mongo/db/concurrency/d_concurrency.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
+#include "mongo/db/database_name.h"
+#include "mongo/db/metadata_consistency_types_gen.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/pipeline/expression_context.h"
+#include "mongo/db/pipeline/pipeline.h"
+#include "mongo/db/pipeline/process_interface/mongo_process_interface.h"
+#include "mongo/db/query/cursor_response_gen.h"
+#include "mongo/db/query/plan_executor.h"
+#include "mongo/db/query/plan_executor_factory.h"
+#include "mongo/db/query/query_request_helper.h"
+#include "mongo/db/repl/member_state.h"
+#include "mongo/db/repl/read_concern_args.h"
+#include "mongo/db/repl/read_concern_level.h"
+#include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/s/ddl_lock_manager.h"
 #include "mongo/db/s/metadata_consistency_util.h"
 #include "mongo/db/s/operation_sharding_state.h"
 #include "mongo/db/s/shard_filtering_metadata_refresh.h"
-#include "mongo/db/s/sharding_state.h"
+#include "mongo/db/service_context.h"
+#include "mongo/db/shard_id.h"
+#include "mongo/executor/task_executor_pool.h"
+#include "mongo/idl/idl_parser.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/redaction.h"
+#include "mongo/rpc/op_msg.h"
+#include "mongo/s/catalog/type_database_gen.h"
+#include "mongo/s/client/shard.h"
+#include "mongo/s/client/shard_registry.h"
+#include "mongo/s/database_version.h"
 #include "mongo/s/grid.h"
+#include "mongo/s/query/async_results_merger_params_gen.h"
 #include "mongo/s/query/document_source_merge_cursors.h"
 #include "mongo/s/query/establish_cursors.h"
 #include "mongo/s/request_types/sharded_ddl_commands_gen.h"
+#include "mongo/s/shard_version.h"
+#include "mongo/s/sharding_state.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/duration.h"
+#include "mongo/util/intrusive_counter.h"
+#include "mongo/util/str.h"
+#include "mongo/util/string_map.h"
+#include "mongo/util/uuid.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
@@ -53,9 +114,10 @@ MONGO_FAIL_POINT_DEFINE(hangShardCheckMetadataBeforeDDLLock);
 MONGO_FAIL_POINT_DEFINE(tripwireShardCheckMetadataAfterDDLLock);
 
 MONGO_FAIL_POINT_DEFINE(hangShardCheckMetadataBeforeEstablishCursors);
-MONGO_FAIL_POINT_DEFINE(throwExceededTimeLimitOnCheckMetadataBeforeEstablishCursors);
 MONGO_FAIL_POINT_DEFINE(tripwireShardCheckMetadataAfterEstablishCursors);
 
+constexpr StringData kDDLLockReason = "checkMetadataConsistency"_sd;
+
 /*
  * Retrieve from config server the list of databases for which this shard is primary for.
  */
@@ -75,7 +137,7 @@ std::vector<DatabaseType> getDatabasesThisShardIsPrimaryFor(OperationContext* op
     databases.reserve(rawDatabases.size());
     for (auto&& rawDb : rawDatabases) {
         auto db = DatabaseType::parseOwned(IDLParserContext("DatabaseType"), std::move(rawDb));
-        if (db.getName() == DatabaseName::kAdmin.db()) {
+        if (db.getDbName() == DatabaseName::kAdmin) {
             // TODO (SERVER-101175): Convert this into a new metadata inconsistency.
             // The 'admin' database should not be explicitly assigned a primary shard. It may exist
             // in the global catalog due to upgrade from an older MongoDB version.
@@ -91,9 +153,8 @@ std::vector<DatabaseType> getDatabasesThisShardIsPrimaryFor(OperationContext* op
     }
     if (thisShardId == ShardId::kConfigServerId) {
         // Config database
-        databases.emplace_back(DatabaseName::kConfig.db().toString(),
-                               ShardId::kConfigServerId,
-                               DatabaseVersion::makeFixed());
+        databases.emplace_back(
+            DatabaseName::kConfig, ShardId::kConfigServerId, DatabaseVersion::makeFixed());
     }
     return databases;
 }
@@ -126,7 +187,7 @@ public:
         using InvocationBase::InvocationBase;
 
         Response typedRun(OperationContext* opCtx) {
-            uassertStatusOK(ShardingState::get(opCtx)->canAcceptShardedCommands());
+            ShardingState::get(opCtx)->assertCanAcceptShardedCommands();
 
             opCtx->setAlwaysInterruptAtStepDownOrUp_UNSAFE();
             {
@@ -137,10 +198,9 @@ public:
                         repl::ReplicationCoordinator::get(opCtx)->getMemberState().primary());
             }
 
-            const auto response = [&] {
+            auto response = [&] {
                 const auto nss = ns();
-                const auto commandLevel = metadata_consistency_util::getCommandLevel(nss);
-                switch (commandLevel) {
+                switch (metadata_consistency_util::getCommandLevel(nss)) {
                     case MetadataConsistencyCommandLevelEnum::kClusterLevel:
                         return _runClusterLevel(opCtx, nss);
                     case MetadataConsistencyCommandLevelEnum::kDatabaseLevel:
@@ -148,14 +208,7 @@ public:
                     case MetadataConsistencyCommandLevelEnum::kCollectionLevel:
                         return _runCollectionLevel(opCtx, nss);
                     default:
-                        tasserted(1011706,
-                                  str::stream()
-                                      << "Unexpected parameter during the internal execution of "
-                                         "checkMetadataConsistency command. The shard server was "
-                                         "expecting to receive a cluster, database or collection "
-                                         "level parameter, but received "
-                                      << MetadataConsistencyCommandLevel_serializer(commandLevel)
-                                      << " with namespace " << nss.toStringForErrorMsg());
+                        MONGO_UNREACHABLE;
                 }
             }();
 
@@ -166,27 +219,6 @@ public:
         }
 
     private:
-        template <typename Callback>
-        decltype(auto) _runWithDbMetadaLockDeadline(OperationContext* opCtx,
-                                                    Milliseconds timeout,
-                                                    const NamespaceString& nss,
-                                                    Callback&& cb) {
-            const auto now = opCtx->getServiceContext()->getFastClockSource()->now();
-            const auto deadline = now + timeout;
-            try {
-                return opCtx->runWithDeadline(deadline, ErrorCodes::ExceededTimeLimit, cb);
-            } catch (const ExceptionFor<ErrorCodes::ExceededTimeLimit>&) {
-                const auto now = opCtx->getServiceContext()->getFastClockSource()->now();
-                if (now >= deadline) {
-                    uasserted(9944001,
-                              str::stream()
-                                  << "Exceeded maximum time " << timeout
-                                  << " while processing namespace " << nss.toStringForErrorMsg());
-                }
-                throw;
-            }
-        }
-
         Response _runClusterLevel(OperationContext* opCtx, const NamespaceString& nss) {
             uassert(ErrorCodes::InvalidNamespace,
                     str::stream() << Request::kCommandName
@@ -200,61 +232,56 @@ public:
             // Need to retrieve a list of databases which this shard is primary for and run the
             // command on each of them.
             for (const auto& db : getDatabasesThisShardIsPrimaryFor(opCtx)) {
-                const NamespaceString dbNss{db.getName(), nss.coll()};
+                const auto dbNss = NamespaceStringUtil::deserialize(db.getDbName(), nss.coll());
                 ScopedSetShardRole scopedSetShardRole(opCtx,
                                                       dbNss,
                                                       boost::none /* shardVersion */,
                                                       db.getVersion() /* databaseVersion */);
 
+                auto checkMetadataForDb = [&]() {
+                    try {
+                        hangShardCheckMetadataBeforeDDLLock.pauseWhileSet();
+                        DDLLockManager::ScopedDatabaseDDLLock dbDDLLock{
+                            opCtx, dbNss.dbName(), kDDLLockReason, MODE_S};
+                        tassert(9504001,
+                                "Expected interrupt before tripwireShardCheckMetadataAfterDDLLock",
+                                !tripwireShardCheckMetadataAfterDDLLock.shouldFail());
 
-                const auto dbCheckWithDeadlineIfSet = [&] {
-                    auto checkMetadataForDb = [&]() {
-                        try {
-                            auto dbCursors = _establishCursorOnParticipants(opCtx, dbNss);
-                            cursors.insert(cursors.end(),
-                                           std::make_move_iterator(dbCursors.begin()),
-                                           std::make_move_iterator(dbCursors.end()));
-                            return Status::OK();
-                        } catch (const ExceptionFor<ErrorCodes::StaleDbVersion>& ex) {
-                            // Receiving a StaleDbVersion is because of one of these scenarios:
-                            // - A movePrimary is changing the db primary shard.
-                            // - The database is being dropped.
-                            // - This shard doesn't know about the existence of the db.
-                            LOGV2_DEBUG(8840400,
-                                        1,
-                                        "Received StaleDbVersion error while trying to run "
-                                        "database metadata checks",
-                                        logAttrs(dbNss.dbName()),
-                                        "error"_attr = redact(ex));
-                            return ex.toStatus();
-                        }
-                    };
-
-                    if (request().getDbMetadataLockMaxTimeMS().is_initialized()) {
-                        const auto timeout =
-                            Milliseconds(request().getDbMetadataLockMaxTimeMS().value());
-                        return _runWithDbMetadaLockDeadline(
-                            opCtx, timeout, dbNss, checkMetadataForDb);
-                    } else {
-                        return checkMetadataForDb();
+                        auto dbCursors = _establishCursorOnParticipants(opCtx, dbNss);
+                        cursors.insert(cursors.end(),
+                                       std::make_move_iterator(dbCursors.begin()),
+                                       std::make_move_iterator(dbCursors.end()));
+                        return Status::OK();
+                    } catch (const ExceptionFor<ErrorCodes::StaleDbVersion>& ex) {
+                        // Receiving a StaleDbVersion is because of one of these scenarios:
+                        // - A movePrimary is changing the db primary shard.
+                        // - The database is being dropped.
+                        // - This shard doesn't know about the existence of the db.
+                        LOGV2_DEBUG(8840400,
+                                    1,
+                                    "Received StaleDbVersion error while trying to run database "
+                                    "metadata checks",
+                                    logAttrs(dbNss.dbName()),
+                                    "error"_attr = redact(ex));
+                        return ex.toStatus();
                     }
                 };
 
                 bool skippedMetadataChecks = false;
-                auto status = dbCheckWithDeadlineIfSet();
+                auto status = checkMetadataForDb();
                 if (!status.isOK()) {
                     auto extraInfo = status.extraInfo<StaleDbRoutingVersion>();
-                    if (!extraInfo || extraInfo->getVersionWanted()) {
+                    if (extraInfo->getVersionWanted()) {
                         // In case there is a wanted shard version means that the metadata is stale
                         // and we are going to skip the checks.
                         skippedMetadataChecks = true;
                     } else {
-                        // In case the shard doesn't know about the collection, we perform a
-                        // refresh and re-try the metadata checks.
+                        // In case the shard doesn't know about the collection, we perform a refresh
+                        // and re-try the metadata checks.
                         (void)onDbVersionMismatchNoExcept(
-                            opCtx, dbNss.dbName().db(), extraInfo->getVersionReceived());
+                            opCtx, dbNss.dbName(), extraInfo->getVersionReceived());
 
-                        skippedMetadataChecks = !dbCheckWithDeadlineIfSet().isOK();
+                        skippedMetadataChecks = !checkMetadataForDb().isOK();
                     }
                 }
 
@@ -273,24 +300,30 @@ public:
 
         Response _runDatabaseLevel(OperationContext* opCtx, const NamespaceString& nss) {
             auto dbCursors = [&]() {
-                auto establishDBCursors = [&]() {
-                    return _establishCursorOnParticipants(opCtx, nss);
-                };
-
-                if (request().getDbMetadataLockMaxTimeMS().is_initialized()) {
-                    const auto timeout =
-                        Milliseconds(request().getDbMetadataLockMaxTimeMS().value());
-                    return _runWithDbMetadaLockDeadline(opCtx, timeout, nss, establishDBCursors);
-                } else {
-                    return establishDBCursors();
-                }
+                hangShardCheckMetadataBeforeDDLLock.pauseWhileSet();
+                DDLLockManager::ScopedDatabaseDDLLock dbDDLLock{
+                    opCtx, nss.dbName(), kDDLLockReason, MODE_S};
+                tassert(9504002,
+                        "Expected interrupt before tripwireShardCheckMetadataAfterDDLLock",
+                        !tripwireShardCheckMetadataAfterDDLLock.shouldFail());
+                return _establishCursorOnParticipants(opCtx, nss);
             }();
 
             return _mergeCursors(opCtx, nss, std::move(dbCursors));
         }
 
         Response _runCollectionLevel(OperationContext* opCtx, const NamespaceString& nss) {
-            return _mergeCursors(opCtx, nss, _establishCursorOnParticipants(opCtx, nss));
+            auto collCursors = [&]() {
+                hangShardCheckMetadataBeforeDDLLock.pauseWhileSet();
+                DDLLockManager::ScopedCollectionDDLLock dbDDLLock{
+                    opCtx, nss, kDDLLockReason, MODE_S};
+                tassert(9504003,
+                        "Expected interrupt before tripwireShardCheckMetadataAfterDDLLock",
+                        !tripwireShardCheckMetadataAfterDDLLock.shouldFail());
+                return _establishCursorOnParticipants(opCtx, nss);
+            }();
+
+            return _mergeCursors(opCtx, nss, std::move(collCursors));
         }
 
         /*
@@ -299,6 +332,8 @@ public:
          */
         std::vector<RemoteCursor> _establishCursorOnParticipants(OperationContext* opCtx,
                                                                  const NamespaceString& nss) {
+            hangShardCheckMetadataBeforeEstablishCursors.pauseWhileSet();
+
             // Shard requests
             const auto shardOpKey = UUID::gen();
             ShardsvrCheckMetadataConsistencyParticipant participantRequest{nss};
@@ -306,9 +341,13 @@ public:
             participantRequest.setPrimaryShardId(ShardingState::get(opCtx)->shardId());
             participantRequest.setCursor(request().getCursor());
             const auto participants = Grid::get(opCtx)->shardRegistry()->getAllShardIds(opCtx);
-            auto participantRequestWithOpKey =
-                appendOpKey(shardOpKey, participantRequest.toBSON({}));
-            std::vector<std::pair<ShardId, BSONObj>> requests;
+
+            BSONObjBuilder participantRequestBob;
+            participantRequest.serialize(BSONObj(), &participantRequestBob);
+            appendOpKey(shardOpKey, &participantRequestBob);
+            auto participantRequestWithOpKey = participantRequestBob.obj();
+
+            std::vector<AsyncRequestsSender::Request> requests;
             requests.reserve(participants.size() + 1);
             for (const auto& shardId : participants) {
                 requests.emplace_back(shardId, participantRequestWithOpKey.getOwned());
@@ -318,34 +357,12 @@ public:
             const auto configOpKey = UUID::gen();
             ConfigsvrCheckMetadataConsistency configRequest{nss};
             participantRequest.setCursor(request().getCursor());
-            requests.emplace_back(ShardId::kConfigServerId,
-                                  appendOpKey(configOpKey, configRequest.toBSON({})));
 
-            // Take a DDL lock on the database
-            static constexpr StringData kLockReason{"checkMetadataConsistency"_sd};
-            auto ddlLockManager = DDLLockManager::get(opCtx);
-            hangShardCheckMetadataBeforeDDLLock.pauseWhileSet();
-            const auto dbDDLLock = ddlLockManager->lock(
-                opCtx, nss.db(), kLockReason, DDLLockManager::kDefaultLockTimeout);
-            tassert(9504001,
-                    "Expected interrupt before tripwireShardCheckMetadataAfterDDLLock",
-                    !tripwireShardCheckMetadataAfterDDLLock.shouldFail());
+            BSONObjBuilder configRequestBob;
+            configRequest.serialize(BSONObj(), &configRequestBob);
+            appendOpKey(configOpKey, &configRequestBob);
+            requests.emplace_back(ShardId::kConfigServerId, configRequestBob.obj());
 
-            hangShardCheckMetadataBeforeEstablishCursors.pauseWhileSet();
-            if (throwExceededTimeLimitOnCheckMetadataBeforeEstablishCursors.shouldFail()) {
-                uasserted(ErrorCodes::ExceededTimeLimit,
-                          str::stream() << "Timing out before establishing cursors on "
-                                           "checkMetadataConsistency for nss "
-                                        << nss.toStringForErrorMsg());
-            }
-            {
-                Lock::DBLock dbLock(opCtx, nss.dbName(), MODE_IS);
-                const auto scopedDss =
-                    DatabaseShardingState::assertDbLockedAndAcquireShared(opCtx, nss.dbName());
-                scopedDss->assertIsPrimaryShardForDb(opCtx);
-            }
-
-            hangShardCheckMetadataBeforeEstablishCursors.pauseWhileSet();
             auto cursors = establishCursors(opCtx,
                                             Grid::get(opCtx)->getExecutorPool()->getFixedExecutor(),
                                             nss,
@@ -364,8 +381,8 @@ public:
                                          const NamespaceString& nss,
                                          std::vector<RemoteCursor>&& cursors) {
 
-            ResolvedNamespaceMap resolvedNamespaces;
-            resolvedNamespaces[nss] = {nss, std::vector<BSONObj>{}};
+            StringMap<ExpressionContext::ResolvedNamespace> resolvedNamespaces;
+            resolvedNamespaces[nss.coll()] = {nss, std::vector<BSONObj>{}};
 
             auto expCtx = make_intrusive<ExpressionContext>(opCtx,
                                                             boost::none, /* explain */
@@ -392,7 +409,7 @@ public:
                 if (cursorOpts && cursorOpts->getBatchSize()) {
                     return *cursorOpts->getBatchSize();
                 } else {
-                    return query_request_helper::kDefaultBatchSize;
+                    return query_request_helper::getDefaultBatchSize();
                 }
             }();
 
@@ -405,7 +422,8 @@ public:
                 repl::ReadConcernArgs::get(opCtx),
                 ReadPreferenceSetting::get(opCtx),
                 request().toBSON({}),
-                {Privilege(ResourcePattern::forClusterResource(), ActionType::internal)}};
+                {Privilege(ResourcePattern::forClusterResource(nss.tenantId()),
+                           ActionType::internal)}};
 
             return metadata_consistency_util::createInitialCursorReplyMongod(
                 opCtx, std::move(cursorParams), batchSize);
@@ -423,12 +441,13 @@ public:
             uassert(ErrorCodes::Unauthorized,
                     "Unauthorized",
                     AuthorizationSession::get(opCtx->getClient())
-                        ->isAuthorizedForActionsOnResource(ResourcePattern::forClusterResource(),
-                                                           ActionType::internal));
+                        ->isAuthorizedForActionsOnResource(
+                            ResourcePattern::forClusterResource(request().getDbName().tenantId()),
+                            ActionType::internal));
         }
     };
-
-} shardsvrCheckMetadataConsistencyCommand;
+};
+MONGO_REGISTER_COMMAND(ShardsvrCheckMetadataConsistencyCommand).forShard();
 
 }  // namespace
 }  // namespace mongo
