@@ -27,20 +27,58 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTransaction
 
-#include "mongo/platform/basic.h"
+#include <absl/container/flat_hash_set.h>
+#include <algorithm>
+#include <boost/none.hpp>
+#include <boost/smart_ptr.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <iterator>
+#include <mutex>
+#include <tuple>
+#include <type_traits>
 
-#include "mongo/db/s/transaction_coordinator.h"
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/bson/util/builder.h"
+#include "mongo/bson/util/builder_fwd.h"
+#include "mongo/db/api_parameters.h"
+#include "mongo/db/logical_time.h"
+#include "mongo/db/repl/optime.h"
 #include "mongo/db/repl/wait_for_majority_service.h"
+#include "mongo/db/s/server_transaction_coordinators_metrics.h"
+#include "mongo/db/s/single_transaction_coordinator_stats.h"
+#include "mongo/db/s/transaction_coordinator.h"
 #include "mongo/db/s/transaction_coordinator_metrics_observer.h"
+#include "mongo/db/s/transaction_coordinator_util.h"
 #include "mongo/db/server_options.h"
 #include "mongo/db/vector_clock_mutable.h"
+#include "mongo/executor/task_executor_pool.h"
+#include "mongo/logv2/attribute_storage.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/log_severity.h"
+#include "mongo/logv2/redaction.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/platform/compiler.h"
 #include "mongo/s/grid.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/cancellation.h"
+#include "mongo/util/clock_source.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/duration.h"
 #include "mongo/util/fail_point.h"
+#include "mongo/util/str.h"
+#include "mongo/util/tick_source.h"
+
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTransaction
+
 
 namespace mongo {
 namespace {
@@ -53,20 +91,28 @@ using TransactionCoordinatorDocument = txn::TransactionCoordinatorDocument;
 MONGO_FAIL_POINT_DEFINE(hangBeforeWaitingForParticipantListWriteConcern);
 MONGO_FAIL_POINT_DEFINE(hangBeforeWaitingForDecisionWriteConcern);
 
-ExecutorFuture<void> waitForMajorityWithHangFailpoint(ServiceContext* service,
-                                                      FailPoint& failpoint,
-                                                      const std::string& failPointName,
-                                                      repl::OpTime opTime) {
+ExecutorFuture<void> waitForMajorityWithHangFailpoint(
+    ServiceContext* service,
+    FailPoint& failpoint,
+    const std::string& failPointName,
+    repl::OpTime opTime,
+    const LogicalSessionId& lsid,
+    const TxnNumberAndRetryCounter& txnNumberAndRetryCounter,
+    const CancellationToken& cancelToken) {
     auto executor = Grid::get(service)->getExecutorPool()->getFixedExecutor();
-    auto waitForWC = [service, executor](repl::OpTime opTime) {
+    auto waitForWC = [service, executor, cancelToken](repl::OpTime opTime) {
         return WaitForMajorityService::get(service)
-            .waitUntilMajority(opTime, CancellationToken::uncancelable())
+            .waitUntilMajorityForWrite(opTime, cancelToken)
             .thenRunOn(executor);
     };
 
     if (auto sfp = failpoint.scoped(); MONGO_unlikely(sfp.isActive())) {
         const BSONObj& data = sfp.getData();
-        LOGV2(22445, "Hit {failPointName} failpoint", "failPointName"_attr = failPointName);
+        LOGV2(22445,
+              "Hit {failPointName} failpoint",
+              "failPointName"_attr = failPointName,
+              "lsid"_attr = lsid,
+              "txnNumberAndRetryCounter"_attr = txnNumberAndRetryCounter);
 
         // Run the hang failpoint asynchronously on a different thread to avoid self deadlocks.
         return ExecutorFuture<void>(executor).then(
@@ -74,18 +120,40 @@ ExecutorFuture<void> waitForMajorityWithHangFailpoint(ServiceContext* service,
                 if (!data["useUninterruptibleSleep"].eoo()) {
                     failpoint.pauseWhileSet();
                 } else {
-                    ThreadClient tc(failPointName, service);
+                    ThreadClient tc(failPointName, service->getService(ClusterRole::ShardServer));
+
+                    // TODO(SERVER-74658): Please revisit if this thread could be made killable.
+                    {
+                        stdx::lock_guard<Client> lk(*tc.get());
+                        tc.get()->setSystemOperationUnkillableByStepdown(lk);
+                    }
+
                     auto opCtx = tc->makeOperationContext();
                     failpoint.pauseWhileSet(opCtx.get());
                 }
 
-                return waitForWC(std::move(opTime));
+                return waitForWC(opTime);
             });
     }
 
     return waitForWC(std::move(opTime));
 }
 
+void setDecisionPromise(const txn::CoordinatorCommitDecision& decision,
+                        SharedPromise<txn::CommitDecision>& promise) {
+    switch (decision.getDecision()) {
+        case CommitDecision::kCommit: {
+            promise.emplaceValue(CommitDecision::kCommit);
+            return;
+        }
+        case CommitDecision::kAbort: {
+            promise.setError(*decision.getAbortStatus());
+            return;
+        }
+    };
+    MONGO_UNREACHABLE;
+}
+
 }  // namespace
 
 TransactionCoordinator::TransactionCoordinator(
@@ -93,7 +161,8 @@ TransactionCoordinator::TransactionCoordinator(
     const LogicalSessionId& lsid,
     const TxnNumberAndRetryCounter& txnNumberAndRetryCounter,
     std::unique_ptr<txn::AsyncWorkScheduler> scheduler,
-    Date_t deadline)
+    Date_t deadline,
+    const CancellationToken& cancelToken)
     : _serviceContext(operationContext->getServiceContext()),
       _lsid(lsid),
       _txnNumberAndRetryCounter(txnNumberAndRetryCounter),
@@ -101,8 +170,14 @@ TransactionCoordinator::TransactionCoordinator(
       _sendPrepareScheduler(_scheduler->makeChildScheduler()),
       _transactionCoordinatorMetricsObserver(
           std::make_unique<TransactionCoordinatorMetricsObserver>()),
-      _deadline(deadline) {
+      _deadline(deadline),
+      _cancelToken(cancelToken) {
     invariant(_txnNumberAndRetryCounter.getTxnRetryCounter());
+}
+
+void TransactionCoordinator::start(OperationContext* operationContext) {
+    invariant(_scheduler, "TransactionCoordinator shutdown before starting");
+    invariant(_sendPrepareScheduler, "TransactionCoordinator shutdown before starting");
 
     auto apiParams = APIParameters::get(operationContext);
     auto kickOffCommitPF = makePromiseFuture<void>();
@@ -112,8 +187,8 @@ TransactionCoordinator::TransactionCoordinator(
     // sequence has not yet started, it will be abandoned altogether.
     auto deadlineFuture =
         _scheduler
-            ->scheduleWorkAt(deadline,
-                             [this](OperationContext*) {
+            ->scheduleWorkAt(_deadline,
+                             [this, self = shared_from_this()](OperationContext*) {
                                  LOGV2_DEBUG(5047000,
                                              1,
                                              "TransactionCoordinator deadline reached",
@@ -128,7 +203,7 @@ TransactionCoordinator::TransactionCoordinator(
                                      {ErrorCodes::TransactionCoordinatorReachedAbortDecision,
                                       "Transaction exceeded deadline"});
                              })
-            .tapError([this](Status s) {
+            .tapError([this, self = shared_from_this()](Status s) {
                 if (_reserveKickOffCommitPromise()) {
                     _kickOffCommitPromise.setError(std::move(s));
                 }
@@ -144,11 +219,11 @@ TransactionCoordinator::TransactionCoordinator(
     // Two-phase commit phases chain. Once this chain executes, the 2PC sequence has completed
     // either with success or error and the scheduled deadline task above has been joined.
     std::move(kickOffCommitPF.future)
-        .then([this] {
+        .then([this, self = shared_from_this()] {
             return VectorClockMutable::get(_serviceContext)->waitForDurableTopologyTime();
         })
-        .thenRunOn(Grid::get(_serviceContext)->getExecutorPool()->getFixedExecutor())
-        .then([this] {
+        .thenRunOn(_scheduler->getExecutor())
+        .then([this, self = shared_from_this()] {
             // Persist the participants, unless they have been made durable already (which would
             // only be the case if this coordinator was created as part of step-up recovery).
             //  Input: _participants
@@ -160,27 +235,30 @@ TransactionCoordinator::TransactionCoordinator(
 
                 _step = Step::kWritingParticipantList;
 
-                _transactionCoordinatorMetricsObserver->onStartWritingParticipantList(
+                _transactionCoordinatorMetricsObserver->onStartStep(
+                    TransactionCoordinator::Step::kWritingParticipantList,
+                    TransactionCoordinator::Step::kInactive,
                     ServerTransactionCoordinatorsMetrics::get(_serviceContext),
                     _serviceContext->getTickSource(),
                     _serviceContext->getPreciseClockSource()->now());
-
                 if (_participantsDurable)
                     return Future<repl::OpTime>::makeReady(repl::OpTime());
             }
-
             return txn::persistParticipantsList(
                 *_sendPrepareScheduler, _lsid, _txnNumberAndRetryCounter, *_participants);
         })
-        .then([this](repl::OpTime opTime) {
+        .then([this, self = shared_from_this()](repl::OpTime opTime) {
             return waitForMajorityWithHangFailpoint(
                 _serviceContext,
                 hangBeforeWaitingForParticipantListWriteConcern,
                 "hangBeforeWaitingForParticipantListWriteConcern",
-                std::move(opTime));
+                std::move(opTime),
+                _lsid,
+                _txnNumberAndRetryCounter,
+                _cancelToken);
         })
-        .thenRunOn(Grid::get(_serviceContext)->getExecutorPool()->getFixedExecutor())
-        .then([this, apiParams] {
+        .thenRunOn(_scheduler->getExecutor())
+        .then([this, self = shared_from_this(), apiParams] {
             {
                 stdx::lock_guard<Latch> lg(_mutex);
                 _participantsDurable = true;
@@ -196,9 +274,12 @@ TransactionCoordinator::TransactionCoordinator(
                 stdx::lock_guard<Latch> lg(_mutex);
                 invariant(_participantsDurable);
 
+                auto previousStep = _step;
                 _step = Step::kWaitingForVotes;
 
-                _transactionCoordinatorMetricsObserver->onStartWaitingForVotes(
+                _transactionCoordinatorMetricsObserver->onStartStep(
+                    TransactionCoordinator::Step::kWaitingForVotes,
+                    previousStep,
                     ServerTransactionCoordinatorsMetrics::get(_serviceContext),
                     _serviceContext->getTickSource(),
                     _serviceContext->getPreciseClockSource()->now());
@@ -213,22 +294,24 @@ TransactionCoordinator::TransactionCoordinator(
                                     _txnNumberAndRetryCounter,
                                     apiParams,
                                     *_participants)
-                .then([this](PrepareVoteConsensus consensus) mutable {
+                .then([this, self = shared_from_this()](PrepareVoteConsensus consensus) mutable {
                     {
                         stdx::lock_guard<Latch> lg(_mutex);
                         _decision = consensus.decision();
                     }
 
                     if (_decision->getDecision() == CommitDecision::kCommit) {
-                        LOGV2_DEBUG(
-                            22446,
-                            3,
-                            "{sessionId}:{_txnNumberAndRetryCounter} Advancing cluster time to "
-                            "the commit timestamp {commitTimestamp}",
-                            "Advancing cluster time to the commit timestamp",
-                            "sessionId"_attr = _lsid,
-                            "txnNumberAndRetryCounter"_attr = _txnNumberAndRetryCounter,
-                            "commitTimestamp"_attr = *_decision->getCommitTimestamp());
+                        auto affectedNamespacesSet = consensus.releaseAffectedNamespaces();
+                        _affectedNamespaces.reserve(affectedNamespacesSet.size());
+                        std::move(affectedNamespacesSet.begin(),
+                                  affectedNamespacesSet.end(),
+                                  std::back_inserter(_affectedNamespaces));
+                        LOGV2_DEBUG(22446,
+                                    3,
+                                    "Advancing cluster time to the commit timestamp",
+                                    "sessionId"_attr = _lsid,
+                                    "txnNumberAndRetryCounter"_attr = _txnNumberAndRetryCounter,
+                                    "commitTimestamp"_attr = *_decision->getCommitTimestamp());
 
                         VectorClockMutable::get(_serviceContext)
                             ->tickClusterTimeTo(LogicalTime(*_decision->getCommitTimestamp()));
@@ -236,19 +319,19 @@ TransactionCoordinator::TransactionCoordinator(
                 });
         })
         .onError<ErrorCodes::TransactionCoordinatorReachedAbortDecision>(
-            [this, lsid, txnNumberAndRetryCounter](const Status& status) {
+            [this, self = shared_from_this()](const Status& status) {
                 // Timeout happened, propagate the decision to abort the transaction to replicas
                 // and convert the internal error code to the public one.
                 LOGV2(5047001,
                       "Transaction coordinator made abort decision",
-                      "sessionId"_attr = lsid,
-                      "txnNumberAndRetryCounter"_attr = txnNumberAndRetryCounter,
+                      "sessionId"_attr = _lsid,
+                      "txnNumberAndRetryCounter"_attr = _txnNumberAndRetryCounter,
                       "status"_attr = redact(status));
                 stdx::lock_guard<Latch> lg(_mutex);
                 _decision = txn::PrepareVote::kAbort;
                 _decision->setAbortStatus(Status(ErrorCodes::NoSuchTransaction, status.reason()));
             })
-        .then([this] {
+        .then([this, self = shared_from_this()] {
             // Persist the commit decision, unless this has already been done (which would only be
             // the case if this coordinator was created as part of step-up recovery and the recovery
             // document contained a decision).
@@ -259,9 +342,12 @@ TransactionCoordinator::TransactionCoordinator(
                 stdx::lock_guard<Latch> lg(_mutex);
                 invariant(_decision);
 
+                auto previousStep = _step;
                 _step = Step::kWritingDecision;
 
-                _transactionCoordinatorMetricsObserver->onStartWritingDecision(
+                _transactionCoordinatorMetricsObserver->onStartStep(
+                    TransactionCoordinator::Step::kWritingDecision,
+                    previousStep,
                     ServerTransactionCoordinatorsMetrics::get(_serviceContext),
                     _serviceContext->getTickSource(),
                     _serviceContext->getPreciseClockSource()->now());
@@ -270,29 +356,24 @@ TransactionCoordinator::TransactionCoordinator(
                     return Future<repl::OpTime>::makeReady(repl::OpTime());
             }
 
-            return txn::persistDecision(
-                *_scheduler, _lsid, _txnNumberAndRetryCounter, *_participants, *_decision);
+            return txn::persistDecision(*_scheduler,
+                                        _lsid,
+                                        _txnNumberAndRetryCounter,
+                                        *_participants,
+                                        *_decision,
+                                        _affectedNamespaces);
         })
-        .then([this](repl::OpTime opTime) {
-            switch (_decision->getDecision()) {
-                case CommitDecision::kCommit: {
-                    _decisionPromise.emplaceValue(CommitDecision::kCommit);
-                    break;
-                }
-                case CommitDecision::kAbort: {
-                    _decisionPromise.setError(*_decision->getAbortStatus());
-                    break;
-                }
-                default:
-                    MONGO_UNREACHABLE;
-            };
-
+        .then([this, self = shared_from_this()](repl::OpTime opTime) {
+            setDecisionPromise(*_decision, _decisionPromise);
             return waitForMajorityWithHangFailpoint(_serviceContext,
                                                     hangBeforeWaitingForDecisionWriteConcern,
                                                     "hangBeforeWaitingForDecisionWriteConcern",
-                                                    std::move(opTime));
+                                                    std::move(opTime),
+                                                    _lsid,
+                                                    _txnNumberAndRetryCounter,
+                                                    _cancelToken);
         })
-        .then([this, apiParams] {
+        .then([this, self = shared_from_this(), apiParams] {
             {
                 stdx::lock_guard<Latch> lg(_mutex);
                 _decisionDurable = true;
@@ -305,9 +386,12 @@ TransactionCoordinator::TransactionCoordinator(
                 stdx::lock_guard<Latch> lg(_mutex);
                 invariant(_decisionDurable);
 
+                auto previousStep = _step;
                 _step = Step::kWaitingForDecisionAcks;
 
-                _transactionCoordinatorMetricsObserver->onStartWaitingForDecisionAcks(
+                _transactionCoordinatorMetricsObserver->onStartStep(
+                    TransactionCoordinator::Step::kWaitingForDecisionAcks,
+                    previousStep,
                     ServerTransactionCoordinatorsMetrics::get(_serviceContext),
                     _serviceContext->getTickSource(),
                     _serviceContext->getPreciseClockSource()->now());
@@ -335,44 +419,85 @@ TransactionCoordinator::TransactionCoordinator(
                     MONGO_UNREACHABLE;
             };
         })
-        .then([this] {
-            // Do a best-effort attempt (i.e., writeConcern w:1) to delete the coordinator's durable
-            // state.
+        .then([this, self = shared_from_this(), apiParams] {
+            setDecisionPromise(*_decision, _decisionAcknowledgedPromise);
+
             {
                 stdx::lock_guard<Latch> lg(_mutex);
 
-                _step = Step::kDeletingCoordinatorDoc;
+                auto previousStep = _step;
+                _step = Step::kWritingEndOfTransaction;
 
-                _transactionCoordinatorMetricsObserver->onStartDeletingCoordinatorDoc(
+                _transactionCoordinatorMetricsObserver->onStartStep(
+                    TransactionCoordinator::Step::kWritingEndOfTransaction,
+                    previousStep,
                     ServerTransactionCoordinatorsMetrics::get(_serviceContext),
                     _serviceContext->getTickSource(),
                     _serviceContext->getPreciseClockSource()->now());
             }
 
-            return txn::deleteCoordinatorDoc(*_scheduler, _lsid, _txnNumberAndRetryCounter)
-                .then([this] {
-                    invariant(!_coordinatorDocRemovalPromise.getFuture().isReady());
-                    _coordinatorDocRemovalPromise.emplaceValue();
-                });
+            // If transaction was commited, write endOfTransaction oplog entries
+            if (_decision->getDecision() != CommitDecision::kCommit) {
+                return Future<void>::makeReady();
+            }
+            return txn::writeEndOfTransaction(
+                *_scheduler, _lsid, _txnNumberAndRetryCounter, _affectedNamespaces);
         })
-        .getAsync([this, deadlineFuture = std::move(deadlineFuture)](Status s) mutable {
+        .then([this, self = shared_from_this()] {
+            // Do a best-effort attempt (i.e., writeConcern w:1) to delete the coordinator's durable
+            // state.
+            {
+                stdx::lock_guard<Latch> lg(_mutex);
+
+                auto previousStep = _step;
+                _step = Step::kDeletingCoordinatorDoc;
+
+                _transactionCoordinatorMetricsObserver->onStartStep(
+                    TransactionCoordinator::Step::kDeletingCoordinatorDoc,
+                    previousStep,
+                    ServerTransactionCoordinatorsMetrics::get(_serviceContext),
+                    _serviceContext->getTickSource(),
+                    _serviceContext->getPreciseClockSource()->now());
+            }
+            return txn::deleteCoordinatorDoc(*_scheduler, _lsid, _txnNumberAndRetryCounter);
+        })
+        .getAsync([this, self = shared_from_this(), deadlineFuture = std::move(deadlineFuture)](
+                      Status s) mutable {
             // Interrupt this coordinator's scheduler hierarchy and join the deadline task's future
             // in order to guarantee that there are no more threads running within the coordinator.
             _scheduler->shutdown(
                 {ErrorCodes::TransactionCoordinatorDeadlineTaskCanceled, "Coordinator completed"});
 
-            return std::move(deadlineFuture).getAsync([this, s = std::move(s)](Status) {
-                // Notify all the listeners which are interested in the coordinator's lifecycle.
-                // After this call, the coordinator object could potentially get destroyed by its
-                // lifetime controller, so there shouldn't be any accesses to `this` after this
-                // call.
-                _done(s);
-            });
+            return std::move(deadlineFuture)
+                .getAsync([this, self = shared_from_this(), s = std::move(s)](Status) {
+                    // Notify all the listeners which are interested in the coordinator's lifecycle.
+                    _done(s);
+                });
         });
 }
 
+void TransactionCoordinator::shutdown() {
+    invariant(_completionPromise.getFuture().isReady());
+    // Callers can't directly control when a TransactionCoordinator is destroyed (since it keeps
+    // itself alive via shared_ptrs captured in continuation chains). _scheduler may be a
+    // child scheduler of a scheduler owned by the caller, and therefore the caller may
+    // require that _scheduler is destroyed before its own scheduler. Callers can (and must) call
+    // shutdown() after waiting on onCompletion() to ensure _scheduler is destroyed at an
+    // appropriate time.
+    invariant(_sendPrepareScheduler, "TransactionCoordinator shutdown multiple times");
+    invariant(_scheduler, "TransactionCoordinator shutdown multiple times");
+
+    _sendPrepareScheduler->join();
+    _sendPrepareScheduler.reset();
+
+    _scheduler->join();
+    _scheduler.reset();
+}
+
 TransactionCoordinator::~TransactionCoordinator() {
     invariant(_completionPromise.getFuture().isReady());
+    invariant(!_sendPrepareScheduler);
+    invariant(!_scheduler);
 }
 
 void TransactionCoordinator::runCommit(OperationContext* opCtx, std::vector<ShardId> participants) {
@@ -390,11 +515,12 @@ void TransactionCoordinator::continueCommit(const TransactionCoordinatorDocument
 
     _transactionCoordinatorMetricsObserver->onRecoveryFromFailover();
 
-    _participants = std::move(doc.getParticipants());
+    _participants = doc.getParticipants();
     if (doc.getDecision()) {
         _participantsDurable = true;
-        _decision = std::move(doc.getDecision());
+        _decision = doc.getDecision();
     }
+    _affectedNamespaces = doc.getAffectedNamespaces().get_value_or({});
 
     _kickOffCommitPromise.emplaceValue();
 }
@@ -403,10 +529,14 @@ SharedSemiFuture<CommitDecision> TransactionCoordinator::getDecision() const {
     return _decisionPromise.getFuture();
 }
 
-SharedSemiFuture<txn::CommitDecision> TransactionCoordinator::onCompletion() {
+SharedSemiFuture<void> TransactionCoordinator::onCompletion() const {
     return _completionPromise.getFuture();
 }
 
+SharedSemiFuture<CommitDecision> TransactionCoordinator::onDecisionAcknowledged() const {
+    return _decisionAcknowledgedPromise.getFuture();
+}
+
 void TransactionCoordinator::cancelIfCommitNotYetStarted() {
     if (!_reserveKickOffCommitPromise())
         return;
@@ -437,7 +567,6 @@ void TransactionCoordinator::_done(Status status) {
 
     LOGV2_DEBUG(22447,
                 3,
-                "{sessionId}:{_txnNumberAndRetryCounter} Two-phase commit completed with {status}",
                 "Two-phase commit completed",
                 "sessionId"_attr = _lsid,
                 "txnNumberAndRetryCounter"_attr = _txnNumberAndRetryCounter,
@@ -458,7 +587,7 @@ void TransactionCoordinator::_done(Status status) {
         (shouldLog(logv2::LogComponent::kTransaction, logv2::LogSeverity::Debug(1)) ||
          _transactionCoordinatorMetricsObserver->getSingleTransactionCoordinatorStats()
                  .getTwoPhaseCommitDuration(tickSource, tickSource->getTicks()) >
-             Milliseconds(serverGlobalParams.slowMS))) {
+             Milliseconds(serverGlobalParams.slowMS.load()))) {
         _logSlowTwoPhaseCommit(*_decision);
     }
 
@@ -467,16 +596,14 @@ void TransactionCoordinator::_done(Status status) {
     if (!_decisionPromise.getFuture().isReady()) {
         _decisionPromise.setError(status);
     }
-
-    if (!_coordinatorDocRemovalPromise.getFuture().isReady()) {
-        _coordinatorDocRemovalPromise.setError(status);
+    if (!_decisionAcknowledgedPromise.getFuture().isReady()) {
+        _decisionAcknowledgedPromise.setError(status);
     }
 
     if (!status.isOK()) {
         _completionPromise.setError(status);
     } else {
-        // If the status is OK, the decisionPromise must be set.
-        _completionPromise.setFrom(_decisionPromise.getFuture().getNoThrow());
+        _completionPromise.setFrom(_decisionPromise.getFuture().getNoThrow().getStatus());
     }
 }
 
@@ -517,26 +644,31 @@ void TransactionCoordinator::_logSlowTwoPhaseCommit(
         _transactionCoordinatorMetricsObserver->getSingleTransactionCoordinatorStats();
 
     BSONObjBuilder stepDurations;
-    stepDurations.append("writingParticipantListMicros",
-                         durationCount<Microseconds>(
-                             singleTransactionCoordinatorStats.getWritingParticipantListDuration(
-                                 tickSource, curTick)));
+    stepDurations.append(
+        "writingParticipantListMicros",
+        durationCount<Microseconds>(singleTransactionCoordinatorStats.getStepDuration(
+            Step::kWritingParticipantList, tickSource, curTick)));
     stepDurations.append(
         "waitingForVotesMicros",
-        durationCount<Microseconds>(
-            singleTransactionCoordinatorStats.getWaitingForVotesDuration(tickSource, curTick)));
+        durationCount<Microseconds>(singleTransactionCoordinatorStats.getStepDuration(
+            Step::kWaitingForVotes, tickSource, curTick)));
     stepDurations.append(
         "writingDecisionMicros",
-        durationCount<Microseconds>(
-            singleTransactionCoordinatorStats.getWritingDecisionDuration(tickSource, curTick)));
-    stepDurations.append("waitingForDecisionAcksMicros",
-                         durationCount<Microseconds>(
-                             singleTransactionCoordinatorStats.getWaitingForDecisionAcksDuration(
-                                 tickSource, curTick)));
-    stepDurations.append("deletingCoordinatorDocMicros",
-                         durationCount<Microseconds>(
-                             singleTransactionCoordinatorStats.getDeletingCoordinatorDocDuration(
-                                 tickSource, curTick)));
+        durationCount<Microseconds>(singleTransactionCoordinatorStats.getStepDuration(
+            Step::kWritingDecision, tickSource, curTick)));
+    stepDurations.append(
+        "waitingForDecisionAcksMicros",
+        durationCount<Microseconds>(singleTransactionCoordinatorStats.getStepDuration(
+            Step::kWaitingForDecisionAcks, tickSource, curTick)));
+    stepDurations.append(
+        "writingEndOfTransactionMicros",
+        durationCount<Microseconds>(singleTransactionCoordinatorStats.getStepDuration(
+            Step::kWritingEndOfTransaction, tickSource, curTick)));
+    stepDurations.append(
+        "deletingCoordinatorDocMicros",
+        durationCount<Microseconds>(singleTransactionCoordinatorStats.getStepDuration(
+            Step::kDeletingCoordinatorDoc, tickSource, curTick)));
+
     attrs.add("stepDurations", stepDurations.obj());
 
     // Total duration of the commit coordination. Logged at the end of the line for consistency
@@ -550,83 +682,12 @@ void TransactionCoordinator::_logSlowTwoPhaseCommit(
     LOGV2(51804, "two-phase commit", attrs);
 }
 
-std::string TransactionCoordinator::_twoPhaseCommitInfoForLog(
-    const txn::CoordinatorCommitDecision& decision) const {
-    StringBuilder s;
-
-    s << "two-phase commit";
-
-    BSONObjBuilder parametersBuilder;
-
-    BSONObjBuilder lsidBuilder(parametersBuilder.subobjStart("lsid"));
-    _lsid.serialize(&lsidBuilder);
-    lsidBuilder.doneFast();
-
-    parametersBuilder.append("txnNumber", _txnNumberAndRetryCounter.getTxnNumber());
-    parametersBuilder.append("txnRetryCounter", *_txnNumberAndRetryCounter.getTxnRetryCounter());
-
-    s << " parameters:" << parametersBuilder.obj().toString();
-
-    switch (decision.getDecision()) {
-        case txn::CommitDecision::kCommit:
-            s << ", terminationCause:committed";
-            s << ", commitTimestamp: " << decision.getCommitTimestamp()->toString();
-            break;
-        case txn::CommitDecision::kAbort:
-            s << ", terminationCause:aborted";
-            s << ", terminationDetails: " << *decision.getAbortStatus();
-            break;
-        default:
-            MONGO_UNREACHABLE;
-    };
-
-    s << ", numParticipants:" << _participants->size();
-
-    auto tickSource = _serviceContext->getTickSource();
-    auto curTick = tickSource->getTicks();
-    const auto& singleTransactionCoordinatorStats =
-        _transactionCoordinatorMetricsObserver->getSingleTransactionCoordinatorStats();
-
-    BSONObjBuilder stepDurations;
-    stepDurations.append("writingParticipantListMicros",
-                         durationCount<Microseconds>(
-                             singleTransactionCoordinatorStats.getWritingParticipantListDuration(
-                                 tickSource, curTick)));
-    stepDurations.append(
-        "waitingForVotesMicros",
-        durationCount<Microseconds>(
-            singleTransactionCoordinatorStats.getWaitingForVotesDuration(tickSource, curTick)));
-    stepDurations.append(
-        "writingDecisionMicros",
-        durationCount<Microseconds>(
-            singleTransactionCoordinatorStats.getWritingDecisionDuration(tickSource, curTick)));
-    stepDurations.append("waitingForDecisionAcksMicros",
-                         durationCount<Microseconds>(
-                             singleTransactionCoordinatorStats.getWaitingForDecisionAcksDuration(
-                                 tickSource, curTick)));
-    stepDurations.append("deletingCoordinatorDocMicros",
-                         durationCount<Microseconds>(
-                             singleTransactionCoordinatorStats.getDeletingCoordinatorDocDuration(
-                                 tickSource, curTick)));
-    s << ", stepDurations:" << stepDurations.obj();
-
-    // Total duration of the commit coordination. Logged at the end of the line for consistency with
-    // slow command logging.
-    // Note that this is reported in milliseconds while the step durations are reported in
-    // microseconds.
-    s << " "
-      << duration_cast<Milliseconds>(
-             singleTransactionCoordinatorStats.getTwoPhaseCommitDuration(tickSource, curTick));
-
-    return s.str();
-}
-
 TransactionCoordinator::Step TransactionCoordinator::getStep() const {
     stdx::lock_guard<Latch> lk(_mutex);
     return _step;
 }
 
-void TransactionCoordinator::reportState(BSONObjBuilder& parent) const {
+void TransactionCoordinator::reportState(OperationContext* opCtx, BSONObjBuilder& parent) const {
     BSONObjBuilder doc;
     TickSource* tickSource = _serviceContext->getTickSource();
     TickSource::Tick currentTick = tickSource->getTicks();
@@ -648,7 +709,7 @@ void TransactionCoordinator::reportState(BSONObjBuilder& parent) const {
     const auto& singleStats =
         _transactionCoordinatorMetricsObserver->getSingleTransactionCoordinatorStats();
     singleStats.reportMetrics(doc, tickSource, currentTick);
-    singleStats.reportLastClient(parent);
+    singleStats.reportLastClient(opCtx, parent);
 
     if (_decision)
         doc.append("decision", _decision->toBSON());
@@ -659,7 +720,7 @@ void TransactionCoordinator::reportState(BSONObjBuilder& parent) const {
     parent.append("twoPhaseCommitCoordinator", doc.obj());
 }
 
-std::string TransactionCoordinator::toString(Step step) const {
+std::string TransactionCoordinator::toString(Step step) {
     switch (step) {
         case Step::kInactive:
             return "inactive";
@@ -670,12 +731,13 @@ std::string TransactionCoordinator::toString(Step step) const {
         case Step::kWritingDecision:
             return "writingDecision";
         case Step::kWaitingForDecisionAcks:
-            return "waitingForDecisionAck";
+            return "waitingForDecisionAcks";
+        case Step::kWritingEndOfTransaction:
+            return "writingEndOfTransaction";
         case Step::kDeletingCoordinatorDoc:
             return "deletingCoordinatorDoc";
-        default:
-            MONGO_UNREACHABLE;
     }
+    MONGO_UNREACHABLE;
 }
 
 void TransactionCoordinator::_updateAssociatedClient(Client* client) {
