@@ -27,14 +27,34 @@
  *    it in the license file.
  */
 
+
+// IWYU pragma: no_include "cxxabi.h"
+#include <absl/container/node_hash_map.h>
+#include <absl/meta/type_traits.h>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr.hpp>
+#include <chrono>
+#include <type_traits>
+#include <vector>
+
+#include "mongo/base/status_with.h"
+#include "mongo/bson/util/builder.h"
+#include "mongo/bson/util/builder_fwd.h"
+#include "mongo/db/s/transaction_coordinator_catalog.h"
+#include "mongo/db/s/transaction_coordinator_structures.h"
+#include "mongo/executor/task_executor_pool.h"
+#include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/s/grid.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/future.h"
+#include "mongo/util/uuid.h"
+
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTransaction
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/s/transaction_coordinator_catalog.h"
-
-#include "mongo/logv2/log.h"
-#include "mongo/s/grid.h"
 
 namespace mongo {
 
@@ -49,8 +69,6 @@ void TransactionCoordinatorCatalog::exitStepUp(Status status) {
         LOGV2(22438, "Incoming coordinateCommit requests are now enabled");
     } else {
         LOGV2_WARNING(22444,
-                      "Coordinator recovery failed and coordinateCommit requests will not be "
-                      "allowed: {error}",
                       "Coordinator recovery failed and coordinateCommit requests will not be "
                       "allowed",
                       "error"_attr = status);
@@ -86,13 +104,11 @@ void TransactionCoordinatorCatalog::insert(OperationContext* opCtx,
                                            const TxnNumberAndRetryCounter& txnNumberAndRetryCounter,
                                            std::shared_ptr<TransactionCoordinator> coordinator,
                                            bool forStepUp) {
-    LOGV2_DEBUG(
-        22439,
-        3,
-        "{sessionId}:{txnNumberAndRetryCounter} Inserting coordinator into in-memory catalog",
-        "Inserting coordinator into in-memory catalog",
-        "sessionId"_attr = lsid.getId(),
-        "txnNumberAndRetryCounter"_attr = txnNumberAndRetryCounter);
+    LOGV2_DEBUG(22439,
+                3,
+                "Inserting coordinator into in-memory catalog",
+                "sessionId"_attr = lsid.getId(),
+                "txnNumberAndRetryCounter"_attr = txnNumberAndRetryCounter);
 
     auto txnNumber = txnNumberAndRetryCounter.getTxnNumber();
     auto txnRetryCounter = *txnNumberAndRetryCounter.getTxnRetryCounter();
@@ -143,7 +159,8 @@ void TransactionCoordinatorCatalog::insert(OperationContext* opCtx,
     coordinator->onCompletion()
         .thenRunOn(Grid::get(opCtx)->getExecutorPool()->getFixedExecutor())
         .ignoreValue()
-        .getAsync([this, lsid, txnNumberAndRetryCounter](Status) {
+        .getAsync([this, lsid, txnNumberAndRetryCounter, coordinator = coordinator](Status) {
+            coordinator->shutdown();
             _remove(lsid, txnNumberAndRetryCounter);
         });
 }
@@ -211,13 +228,11 @@ TransactionCoordinatorCatalog::getLatestOnSession(OperationContext* opCtx,
 
 void TransactionCoordinatorCatalog::_remove(
     const LogicalSessionId& lsid, const TxnNumberAndRetryCounter& txnNumberAndRetryCounter) {
-    LOGV2_DEBUG(
-        22440,
-        3,
-        "{sessionId}:{txnNumberAndRetryCounter} Removing coordinator from in-memory catalog",
-        "Removing coordinator from in-memory catalog",
-        "sessionId"_attr = lsid.getId(),
-        "txnNumberAndRetryCounter"_attr = txnNumberAndRetryCounter);
+    LOGV2_DEBUG(22440,
+                3,
+                "Removing coordinator from in-memory catalog",
+                "sessionId"_attr = lsid.getId(),
+                "txnNumberAndRetryCounter"_attr = txnNumberAndRetryCounter);
 
     auto txnNumber = txnNumberAndRetryCounter.getTxnNumber();
     auto txnRetryCounter = *txnNumberAndRetryCounter.getTxnRetryCounter();
@@ -259,15 +274,10 @@ void TransactionCoordinatorCatalog::join() {
     while (!_noActiveCoordinatorsCV.wait_for(
         ul, stdx::chrono::seconds{5}, [this] { return _coordinatorsBySession.empty(); })) {
         LOGV2(22442,
-              "After 5 seconds of wait there are still {numSessionsLeft} sessions left "
-              "with active coordinators which have not yet completed",
               "After 5 seconds of wait there are still sessions left with active coordinators "
               "which have not yet completed",
               "numSessionsLeft"_attr = _coordinatorsBySession.size());
-        LOGV2(22443,
-              "Active coordinators remaining: {activeCoordinators}",
-              "Active coordinators remaining",
-              "activeCoordinators"_attr = _toString(ul));
+        LOGV2(22443, "Active coordinators remaining", "activeCoordinators"_attr = _toString(ul));
     }
 }
 
