@@ -28,18 +28,38 @@
  */
 
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/s/transaction_coordinator_futures_util.h"
+// IWYU pragma: no_include "cxxabi.h"
+#include <boost/move/utility_core.hpp>
+#include <boost/smart_ptr.hpp>
+#include <string>
+#include <type_traits>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/client/remote_command_targeter.h"
 #include "mongo/db/auth/authorization_session.h"
-#include "mongo/db/s/sharding_state.h"
+#include "mongo/db/cluster_role.h"
+#include "mongo/db/database_name.h"
+#include "mongo/db/dbmessage.h"
+#include "mongo/db/s/transaction_coordinator_futures_util.h"
+#include "mongo/db/server_options.h"
+#include "mongo/executor/remote_command_request.h"
+#include "mongo/executor/task_executor_pool.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/compiler.h"
 #include "mongo/rpc/get_status_from_command_result.h"
+#include "mongo/rpc/op_msg.h"
+#include "mongo/s/client/shard_registry.h"
 #include "mongo/s/grid.h"
+#include "mongo/s/sharding_state.h"
 #include "mongo/transport/service_entry_point.h"
+#include "mongo/util/cancellation.h"
 #include "mongo/util/fail_point.h"
+#include "mongo/util/future_impl.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTransaction
 
@@ -58,8 +78,18 @@ using ResponseStatus = executor::TaskExecutor::ResponseStatus;
 }  // namespace
 
 AsyncWorkScheduler::AsyncWorkScheduler(ServiceContext* serviceContext)
+    : AsyncWorkScheduler(serviceContext, nullptr, WithLock::withoutLock() /* No parent */) {}
+
+AsyncWorkScheduler::AsyncWorkScheduler(ServiceContext* serviceContext,
+                                       AsyncWorkScheduler* parent,
+                                       WithLock withParentLock)
     : _serviceContext(serviceContext),
-      _executor(Grid::get(_serviceContext)->getExecutorPool()->getFixedExecutor()) {}
+      _executor(Grid::get(_serviceContext)->getExecutorPool()->getFixedExecutor()),
+      _parent(parent) {
+    if (_parent) {
+        _itToRemove = _parent->_childSchedulers.emplace(_parent->_childSchedulers.begin(), this);
+    }
+}
 
 AsyncWorkScheduler::~AsyncWorkScheduler() {
     {
@@ -73,7 +103,6 @@ AsyncWorkScheduler::~AsyncWorkScheduler() {
     stdx::lock_guard<Latch> lg(_parent->_mutex);
     _parent->_childSchedulers.erase(_itToRemove);
     _parent->_notifyAllTasksComplete(lg);
-    _parent = nullptr;
 }
 
 Future<executor::TaskExecutor::ResponseStatus> AsyncWorkScheduler::scheduleRemoteCommand(
@@ -127,9 +156,14 @@ Future<executor::TaskExecutor::ResponseStatus> AsyncWorkScheduler::scheduleRemot
             auto start = _executor->now();
 
             auto requestOpMsg =
-                OpMsgRequest::fromDBAndBody(DatabaseName::kAdmin.db(), commandObj).serialize();
-            const auto replyOpMsg = OpMsg::parseOwned(
-                service->getServiceEntryPoint()->handleRequest(opCtx, requestOpMsg).get().response);
+                OpMsgRequestBuilder::create(
+                    auth::ValidatedTenancyScope::kNotRequired, DatabaseName::kAdmin, commandObj)
+                    .serialize();
+            const auto replyOpMsg = OpMsg::parseOwned(service->getService(ClusterRole::ShardServer)
+                                                          ->getServiceEntryPoint()
+                                                          ->handleRequest(opCtx, requestOpMsg)
+                                                          .get()
+                                                          .response);
 
             // Document sequences are not yet being used for responses.
             invariant(replyOpMsg.sequences.empty());
@@ -145,7 +179,7 @@ Future<executor::TaskExecutor::ResponseStatus> AsyncWorkScheduler::scheduleRemot
         .then([this, shardId, commandObj = commandObj.getOwned(), readPref](
                   HostAndShard hostAndShard) mutable {
             executor::RemoteCommandRequest request(hostAndShard.hostTargeted,
-                                                   DatabaseName::kAdmin.toString(),
+                                                   DatabaseName::kAdmin,
                                                    commandObj,
                                                    readPref.toContainingBSON(),
                                                    nullptr);
@@ -159,7 +193,7 @@ Future<executor::TaskExecutor::ResponseStatus> AsyncWorkScheduler::scheduleRemot
                 request,
                 [this,
                  commandObj = std::move(commandObj),
-                 shardId = std::move(shardId),
+                 shardId = shardId,
                  hostTargeted = std::move(hostAndShard.hostTargeted),
                  shard = std::move(hostAndShard.shard),
                  promise = std::make_shared<Promise<ResponseStatus>>(std::move(pf.promise))](
@@ -176,7 +210,7 @@ Future<executor::TaskExecutor::ResponseStatus> AsyncWorkScheduler::scheduleRemot
                             getWriteConcernStatusFromCommandResult(args.response.data);
                         shard->updateReplSetMonitor(hostTargeted, writeConcernStatus);
 
-                        promise->emplaceValue(std::move(args.response));
+                        promise->emplaceValue(args.response);
                     } else {
                         promise->setError([&] {
                             if (status == ErrorCodes::CallbackCanceled) {
@@ -203,15 +237,14 @@ Future<executor::TaskExecutor::ResponseStatus> AsyncWorkScheduler::scheduleRemot
 }
 
 std::unique_ptr<AsyncWorkScheduler> AsyncWorkScheduler::makeChildScheduler() {
-    auto child = std::make_unique<AsyncWorkScheduler>(_serviceContext);
 
     stdx::lock_guard<Latch> lg(_mutex);
+    // This is to enable access to the private AsyncWorkScheduler ctor.
+    std::unique_ptr<AsyncWorkScheduler> child(new AsyncWorkScheduler(_serviceContext, this, lg));
+
     if (!_shutdownStatus.isOK())
         child->shutdown(_shutdownStatus);
 
-    child->_parent = this;
-    child->_itToRemove = _childSchedulers.emplace(_childSchedulers.begin(), child.get());
-
     return child;
 }
 
@@ -225,7 +258,7 @@ void AsyncWorkScheduler::shutdown(Status status) {
     _shutdownStatus = std::move(status);
 
     for (const auto& it : _activeOpContexts) {
-        stdx::lock_guard<Client> clientLock(*it->getClient());
+        ClientLock clientLock(it->getClient());
         _serviceContext->killOperation(clientLock, it.get(), _shutdownStatus.code());
     }
 
@@ -252,18 +285,18 @@ Future<AsyncWorkScheduler::HostAndShard> AsyncWorkScheduler::_targetHostAsync(
     return scheduleWork([this, shardId, readPref, operationContextFn](OperationContext* opCtx) {
         operationContextFn(opCtx);
         const auto shardRegistry = Grid::get(opCtx)->shardRegistry();
-        const auto shard = uassertStatusOK(shardRegistry->getShard(opCtx, shardId));
+        auto shard = uassertStatusOK(shardRegistry->getShard(opCtx, shardId));
 
         if (MONGO_unlikely(hangWhileTargetingRemoteHost.shouldFail())) {
             LOGV2(22450, "Hit hangWhileTargetingRemoteHost failpoint", "shardId"_attr = shardId);
             hangWhileTargetingRemoteHost.pauseWhileSet(opCtx);
         }
 
-        return shard->getTargeter()
-            ->findHost(readPref, CancellationToken::uncancelable())
+        auto targeter = shard->getTargeter();
+        return targeter->findHost(readPref, CancellationToken::uncancelable())
             .thenRunOn(_executor)
             .unsafeToInlineFuture()
-            .then([shard = std::move(shard)](HostAndPort host) -> HostAndShard {
+            .then([shard = std::move(shard)](HostAndPort host) mutable -> HostAndShard {
                 return {std::move(host), std::move(shard)};
             });
     });
@@ -292,6 +325,7 @@ ShardId getLocalShardId(ServiceContext* service) {
 
 Future<void> whenAll(std::vector<Future<void>>& futures) {
     std::vector<Future<int>> dummyFutures;
+    dummyFutures.reserve(futures.size());
     for (auto&& f : futures) {
         dummyFutures.push_back(std::move(f).then([]() { return 0; }));
     }
