@@ -29,16 +29,37 @@
 
 #pragma once
 
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <cstddef>
+#include <functional>
+#include <list>
 #include <memory>
+#include <mutex>
+#include <utility>
 #include <vector>
 
+#include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/bson/bsonobj.h"
 #include "mongo/client/read_preference.h"
+#include "mongo/db/client.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/service_context.h"
 #include "mongo/db/shard_id.h"
 #include "mongo/executor/task_executor.h"
 #include "mongo/executor/task_executor_pool.h"
+#include "mongo/platform/mutex.h"
 #include "mongo/s/client/shard.h"
+#include "mongo/stdx/condition_variable.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/concurrency/mutex.h"
+#include "mongo/util/concurrency/with_lock.h"
+#include "mongo/util/duration.h"
 #include "mongo/util/future.h"
+#include "mongo/util/net/hostandport.h"
+#include "mongo/util/out_of_line_executor.h"
+#include "mongo/util/scopeguard.h"
 #include "mongo/util/time_support.h"
 
 namespace mongo {
@@ -99,7 +120,14 @@ public:
                             uassertStatusOK(args.status);
                         }
 
-                        ThreadClient tc("TransactionCoordinator", _serviceContext);
+                        ThreadClient tc("TransactionCoordinator",
+                                        _serviceContext->getService(ClusterRole::ShardServer));
+
+                        // TODO(SERVER-74658): Please revisit if this thread could be made killable.
+                        {
+                            stdx::lock_guard<Client> lk(*tc.get());
+                            tc.get()->setSystemOperationUnkillableByStepdown(lk);
+                        }
 
                         auto uniqueOpCtxIter = [&] {
                             stdx::lock_guard lk(_mutex);
@@ -178,6 +206,10 @@ public:
 private:
     using ChildIteratorsList = std::list<AsyncWorkScheduler*>;
 
+    AsyncWorkScheduler(ServiceContext* serviceContext,
+                       AsyncWorkScheduler* parent,
+                       WithLock withParentLock);
+
     // A targeted host and the shard object used to target it. The shard object is passed through
     // resolved so the caller can avoid a potentially blocking "ShardRegistry::getShard" call.
     struct HostAndShard {
@@ -213,7 +245,7 @@ private:
 
     // If this work scheduler was constructed through 'makeChildScheduler', points to the parent
     // scheduler and contains the iterator from the parent, which needs to be removed on destruction
-    AsyncWorkScheduler* _parent{nullptr};
+    AsyncWorkScheduler* const _parent;
     ChildIteratorsList::iterator _itToRemove;
 
     // Mutex to protect the shared state below
