@@ -27,22 +27,41 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTransaction
 
-#include "mongo/platform/basic.h"
+#include <boost/cstdint.hpp>
+#include <boost/none.hpp>
+#include <boost/smart_ptr.hpp>
+#include <cstdint>
+#include <mutex>
 
-#include "mongo/db/s/transaction_coordinator_service.h"
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
 
-#include "mongo/db/concurrency/lock_state.h"
-#include "mongo/db/internal_transactions_feature_flag_gen.h"
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/db/admission/execution_admission_context.h"
 #include "mongo/db/repl/repl_client_info.h"
+#include "mongo/db/s/transaction_coordinator.h"
 #include "mongo/db/s/transaction_coordinator_document_gen.h"
 #include "mongo/db/s/transaction_coordinator_params_gen.h"
-#include "mongo/db/storage/flow_control.h"
-#include "mongo/db/transaction_participant_gen.h"
+#include "mongo/db/s/transaction_coordinator_service.h"
+#include "mongo/db/s/transaction_coordinator_util.h"
+#include "mongo/db/transaction/transaction_participant_gen.h"
 #include "mongo/db/write_concern.h"
+#include "mongo/db/write_concern_options.h"
+#include "mongo/idl/mutable_observer_registry.h"
 #include "mongo/logv2/log.h"
-#include "mongo/s/grid.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/s/sharding_state.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/clock_source.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/future_impl.h"
+
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTransaction
 
 namespace mongo {
 namespace {
@@ -86,8 +105,13 @@ void TransactionCoordinatorService::createCoordinator(
         latestCoordinator->cancelIfCommitNotYetStarted();
     }
 
-    auto coordinator = std::make_shared<TransactionCoordinator>(
-        opCtx, lsid, txnNumberAndRetryCounter, scheduler.makeChildScheduler(), commitDeadline);
+    auto coordinator = std::make_shared<TransactionCoordinator>(opCtx,
+                                                                lsid,
+                                                                txnNumberAndRetryCounter,
+                                                                scheduler.makeChildScheduler(),
+                                                                commitDeadline,
+                                                                _cancelSource.token());
+    coordinator->start(opCtx);
 
     try {
         catalog.insert(opCtx, lsid, txnNumberAndRetryCounter, coordinator);
@@ -99,6 +123,7 @@ void TransactionCoordinatorService::createCoordinator(
         // cause the newly created coordinator to be destroyed. We ignore the return Status since we
         // want to just rethrow whatever exception was thrown when inserting into the catalog.
         [[maybe_unused]] auto status = coordinator->onCompletion().waitNoThrow();
+        coordinator->shutdown();
         throw;
     }
 }
@@ -107,6 +132,11 @@ void TransactionCoordinatorService::createCoordinator(
 void TransactionCoordinatorService::reportCoordinators(OperationContext* opCtx,
                                                        bool includeIdle,
                                                        std::vector<BSONObj>* ops) {
+    // TODO: SERVER-82965 Remove early return
+    if (!ShardingState::get(opCtx)->enabled()) {
+        return;
+    }
+
     std::shared_ptr<CatalogAndScheduler> cas;
     try {
         cas = _getCatalogAndScheduler(opCtx);
@@ -129,11 +159,12 @@ void TransactionCoordinatorService::reportCoordinators(OperationContext* opCtx,
             return false;
         };
 
-    auto reporter = [ops](const LogicalSessionId lsid,
+    auto reporter = [opCtx,
+                     ops](const LogicalSessionId lsid,
                           const TxnNumberAndRetryCounter txnNumberAndRetryCounter,
                           const std::shared_ptr<TransactionCoordinator> transactionCoordinator) {
         BSONObjBuilder doc;
-        transactionCoordinator->reportState(doc);
+        transactionCoordinator->reportState(opCtx, doc);
         ops->push_back(doc.obj());
     };
 
@@ -158,7 +189,7 @@ TransactionCoordinatorService::coordinateCommit(OperationContext* opCtx,
 
     return coordinateCommitReturnImmediatelyAfterPersistingDecision.load()
         ? coordinator->getDecision()
-        : coordinator->onCompletion();
+        : coordinator->onDecisionAcknowledged();
 }
 
 boost::optional<SharedSemiFuture<txn::CommitDecision>> TransactionCoordinatorService::recoverCommit(
@@ -179,11 +210,16 @@ boost::optional<SharedSemiFuture<txn::CommitDecision>> TransactionCoordinatorSer
 
     return coordinateCommitReturnImmediatelyAfterPersistingDecision.load()
         ? coordinator->getDecision()
-        : coordinator->onCompletion();
+        : coordinator->onDecisionAcknowledged();
 }
 
 void TransactionCoordinatorService::onStepUp(OperationContext* opCtx,
                                              Milliseconds recoveryDelayForTesting) {
+    // TODO: SERVER-82965 Remove early return
+    if (!ShardingState::get(opCtx)->enabled()) {
+        return;
+    }
+
     joinPreviousRound();
 
     stdx::lock_guard<Latch> lg(_mutex);
@@ -193,12 +229,14 @@ void TransactionCoordinatorService::onStepUp(OperationContext* opCtx,
 
     invariant(!_catalogAndScheduler);
     _catalogAndScheduler = std::make_shared<CatalogAndScheduler>(opCtx->getServiceContext());
+    _cancelSource = CancellationSource();
 
     auto future =
         _catalogAndScheduler->scheduler
             .scheduleWorkIn(
                 recoveryDelayForTesting,
-                [catalogAndScheduler = _catalogAndScheduler](OperationContext* opCtx) {
+                [catalogAndScheduler = _catalogAndScheduler,
+                 cancelSource = _cancelSource](OperationContext* opCtx) {
                     if (MONGO_unlikely(hangBeforeTxnCoordinatorOnStepUpWork.shouldFail())) {
                         LOGV2(8288301, "Hit hangBeforeTxnCoordinatorOnStepUpWork failpoint");
                         hangBeforeTxnCoordinatorOnStepUpWork.pauseWhileSet(opCtx);
@@ -206,7 +244,8 @@ void TransactionCoordinatorService::onStepUp(OperationContext* opCtx,
 
                     // Skip ticket acquisition in order to prevent possible deadlock when
                     // participants are in the prepared state. See SERVER-82883 and SERVER-60682.
-                    SkipTicketAcquisitionForLock skipTicketAcquisition(opCtx);
+                    ScopedAdmissionPriority<ExecutionAdmissionContext> skipTicketAcquisition(
+                        opCtx, AdmissionContext::Priority::kExempt);
 
                     auto& replClientInfo = repl::ReplClientInfo::forClient(opCtx->getClient());
                     replClientInfo.setLastOpToSystemLastOpTime(opCtx);
@@ -214,7 +253,6 @@ void TransactionCoordinatorService::onStepUp(OperationContext* opCtx,
                     const auto lastOpTime = replClientInfo.getLastOp();
                     LOGV2_DEBUG(22451,
                                 3,
-                                "Waiting for OpTime {lastOpTime} to become majority committed",
                                 "Waiting for OpTime to become majority committed",
                                 "lastOpTime"_attr = lastOpTime);
 
@@ -227,12 +265,9 @@ void TransactionCoordinatorService::onStepUp(OperationContext* opCtx,
                                             WriteConcernOptions::kNoTimeout},
                         &unusedWCResult));
 
-                    FlowControl::Bypass flowControlBypass(opCtx);
                     auto coordinatorDocs = txn::readAllCoordinatorDocs(opCtx);
 
                     LOGV2(22452,
-                          "Need to resume coordinating commit for {numPendingTransactions} "
-                          "transactions",
                           "Need to resume coordinating commit for transactions with an in-progress "
                           "two-phase commit/abort",
                           "numPendingTransactions"_attr = coordinatorDocs.size());
@@ -244,12 +279,10 @@ void TransactionCoordinatorService::onStepUp(OperationContext* opCtx,
                     auto& scheduler = catalogAndScheduler->scheduler;
 
                     for (const auto& doc : coordinatorDocs) {
-                        LOGV2_DEBUG(
-                            22453,
-                            3,
-                            "Going to resume coordinating commit for {transactionCoordinatorInfo}",
-                            "Going to resume coordinating commit",
-                            "transactionCoordinatorInfo"_attr = doc.toBSON());
+                        LOGV2_DEBUG(22453,
+                                    3,
+                                    "Going to resume coordinating commit",
+                                    "transactionCoordinatorInfo"_attr = doc.toBSON());
 
                         const auto lsid = *doc.getId().getSessionId();
                         const auto txnNumber = *doc.getId().getTxnNumber();
@@ -265,7 +298,9 @@ void TransactionCoordinatorService::onStepUp(OperationContext* opCtx,
                             lsid,
                             TxnNumberAndRetryCounter{txnNumber, txnRetryCounter},
                             scheduler.makeChildScheduler(),
-                            clockSource->now() + Seconds(gTransactionLifetimeLimitSeconds.load()));
+                            clockSource->now() + Seconds(gTransactionLifetimeLimitSeconds.load()),
+                            cancelSource.token());
+                        coordinator->start(opCtx);
 
                         catalog.insert(opCtx,
                                        lsid,
@@ -292,6 +327,7 @@ void TransactionCoordinatorService::onStepDown() {
         _catalogAndSchedulerToCleanup = std::move(_catalogAndScheduler);
     }
 
+    _cancelSource.cancel();
     _catalogAndSchedulerToCleanup->onStepDown();
 }
 
@@ -300,7 +336,6 @@ void TransactionCoordinatorService::shutdown() {
         stdx::lock_guard<Latch> lg(_mutex);
         _isShuttingDown = true;
     }
-
     onStepDown();
     joinPreviousRound();
 }
@@ -371,6 +406,11 @@ void TransactionCoordinatorService::cancelIfCommitNotYetStarted(
     OperationContext* opCtx,
     LogicalSessionId lsid,
     TxnNumberAndRetryCounter txnNumberAndRetryCounter) {
+    // TODO: SERVER-82965 Remove early return
+    if (!ShardingState::get(opCtx)->enabled()) {
+        return;
+    }
+
     auto cas = _getCatalogAndScheduler(opCtx);
     auto& catalog = cas->catalog;
 
@@ -382,35 +422,4 @@ void TransactionCoordinatorService::cancelIfCommitNotYetStarted(
     }
 }
 
-const std::vector<SharedSemiFuture<void>>
-TransactionCoordinatorService::getAllRemovalFuturesForCoordinatorsForInternalTransactions(
-    OperationContext* opCtx) {
-    std::vector<SharedSemiFuture<void>> coordinatorStateDocRemovalFutures;
-    std::shared_ptr<CatalogAndScheduler> cas = _getCatalogAndScheduler(opCtx);
-    auto& catalog = cas->catalog;
-
-    // On step up, we want to wait until the catalog has recovered all active transaction
-    // coordinators before getting the removal futures.
-    cas->recoveryTaskCompleted->get(opCtx);
-
-    auto predicate = [](const LogicalSessionId lsid,
-                        const TxnNumberAndRetryCounter txnNumberAndRetryCounter,
-                        const std::shared_ptr<TransactionCoordinator> transactionCoordinator) {
-        TransactionCoordinator::Step step = transactionCoordinator->getStep();
-        if (step > TransactionCoordinator::Step::kInactive && isChildSession(lsid)) {
-            return true;
-        }
-        return false;
-    };
-
-    auto visitorAction = [&](const LogicalSessionId lsid,
-                             const TxnNumberAndRetryCounter txnNumberAndRetryCounter,
-                             const std::shared_ptr<TransactionCoordinator> transactionCoordinator) {
-        coordinatorStateDocRemovalFutures.push_back(
-            transactionCoordinator->getCoordinatorDocRemovalFuture());
-    };
-    catalog.filter(predicate, visitorAction);
-    return coordinatorStateDocRemovalFutures;
-}
-
 }  // namespace mongo
