@@ -28,24 +28,67 @@
  */
 
 
-#include "mongo/platform/basic.h"
+#include <absl/container/flat_hash_set.h>
+#include <algorithm>
+#include <boost/cstdint.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <cstdint>
+#include <memory>
+#include <type_traits>
 
-#include "mongo/db/s/transaction_coordinator_util.h"
-
-#include "mongo/client/remote_command_retry_scheduler.h"
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/util/builder.h"
+#include "mongo/bson/util/builder_fwd.h"
+#include "mongo/client/dbclient_cursor.h"
+#include "mongo/client/read_preference.h"
+#include "mongo/db/admission/execution_admission_context.h"
 #include "mongo/db/commands/txn_cmds_gen.h"
 #include "mongo/db/commands/txn_two_phase_commit_cmds_gen.h"
-#include "mongo/db/curop.h"
+#include "mongo/db/database_name.h"
 #include "mongo/db/dbdirectclient.h"
-#include "mongo/db/ops/write_ops.h"
+#include "mongo/db/ops/write_ops_gen.h"
+#include "mongo/db/ops/write_ops_parsers.h"
+#include "mongo/db/query/collation/collator_factory_interface.h"
+#include "mongo/db/query/find_command.h"
+#include "mongo/db/repl/change_stream_oplog_notification.h"
 #include "mongo/db/repl/repl_client_info.h"
 #include "mongo/db/s/transaction_coordinator_futures_util.h"
+#include "mongo/db/s/transaction_coordinator_util.h"
 #include "mongo/db/s/transaction_coordinator_worker_curop_repository.h"
-#include "mongo/db/storage/flow_control.h"
-#include "mongo/db/write_concern.h"
+#include "mongo/db/server_feature_flags_gen.h"
+#include "mongo/db/server_options.h"
+#include "mongo/db/session/logical_session_id_helpers.h"
+#include "mongo/db/transaction_resources.h"
+#include "mongo/db/write_concern_options.h"
+#include "mongo/executor/task_executor.h"
+#include "mongo/idl/idl_parser.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/redaction.h"
+#include "mongo/platform/compiler.h"
 #include "mongo/rpc/get_status_from_command_result.h"
+#include "mongo/rpc/reply_interface.h"
+#include "mongo/rpc/unique_message.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/concurrency/admission_context.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/duration.h"
 #include "mongo/util/fail_point.h"
+#include "mongo/util/future_impl.h"
+#include "mongo/util/str.h"
+#include "mongo/util/time_support.h"
+#include "mongo/util/uuid.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTransaction
 
@@ -59,6 +102,7 @@ MONGO_FAIL_POINT_DEFINE(hangBeforeSendingPrepare);
 MONGO_FAIL_POINT_DEFINE(hangBeforeWritingDecision);
 MONGO_FAIL_POINT_DEFINE(hangBeforeSendingCommit);
 MONGO_FAIL_POINT_DEFINE(hangBeforeSendingAbort);
+MONGO_FAIL_POINT_DEFINE(hangBeforeWritingEndOfTransaction);
 MONGO_FAIL_POINT_DEFINE(hangBeforeDeletingCoordinatorDoc);
 MONGO_FAIL_POINT_DEFINE(hangAfterDeletingCoordinatorDoc);
 
@@ -111,7 +155,6 @@ repl::OpTime persistParticipantListBlocking(
     const std::vector<ShardId>& participantList) {
     LOGV2_DEBUG(22463,
                 3,
-                "{sessionId}:{txnNumberAndRetryCounter} Going to write participant list",
                 "Going to write participant list",
                 "sessionId"_attr = lsid,
                 "txnNumberAndRetryCounter"_attr = txnNumberAndRetryCounter);
@@ -152,7 +195,7 @@ repl::OpTime persistParticipantListBlocking(
             // Update with participant list.
             TransactionCoordinatorDocument doc;
             doc.setId(std::move(sessionInfo));
-            doc.setParticipants(std::move(participantList));
+            doc.setParticipants(participantList);
             entry.setU(write_ops::UpdateModification::parseFromClassicUpdate(doc.toBSON()));
 
             entry.setUpsert(true);
@@ -205,8 +248,8 @@ Future<repl::OpTime> persistParticipantsList(
                 [lsid, txnNumberAndRetryCounter, participants](OperationContext* opCtx) {
                     // Skip ticket acquisition in order to prevent possible deadlock when
                     // participants are in the prepared state. See SERVER-82883 and SERVER-60682.
-                    ScopedAdmissionPriorityForLock skipTicketAcquisition(
-                        opCtx->lockState(), AdmissionContext::Priority::kImmediate);
+                    ScopedAdmissionPriority<ExecutionAdmissionContext> skipTicketAcquisition(
+                        opCtx, AdmissionContext::Priority::kExempt);
                     getTransactionCoordinatorWorkerCurOpRepository()->set(
                         opCtx,
                         lsid,
@@ -222,6 +265,7 @@ void PrepareVoteConsensus::registerVote(const PrepareResponse& vote) {
     if (vote.vote == PrepareVote::kCommit) {
         ++_numCommitVotes;
         _maxPrepareTimestamp = std::max(_maxPrepareTimestamp, *vote.prepareTimestamp);
+        _affectedNamespaces.insert(vote.affectedNamespaces.begin(), vote.affectedNamespaces.end());
     } else {
         vote.vote == PrepareVote::kAbort ? ++_numAbortVotes : ++_numNoVotes;
 
@@ -259,7 +303,7 @@ Future<PrepareVoteConsensus> sendPrepare(ServiceContext* service,
                                    << WriteConcernOptions::Majority));
     if (auto txnRetryCounter = txnNumberAndRetryCounter.getTxnRetryCounter();
         txnRetryCounter && !isDefaultTxnRetryCounter(*txnRetryCounter)) {
-        bob.append(OperationSessionInfo::kTxnRetryCounterFieldName, *txnRetryCounter);
+        bob.append(OperationSessionInfoFromClient::kTxnRetryCounterFieldName, *txnRetryCounter);
     }
     apiParams.appendInfo(&bob);
     auto prepareObj = prepareTransaction.toBSON(bob.obj());
@@ -331,8 +375,9 @@ namespace {
 repl::OpTime persistDecisionBlocking(OperationContext* opCtx,
                                      const LogicalSessionId& lsid,
                                      const TxnNumberAndRetryCounter& txnNumberAndRetryCounter,
-                                     const std::vector<ShardId>& participantList,
-                                     const txn::CoordinatorCommitDecision& decision) {
+                                     std::vector<ShardId> participantList,
+                                     const txn::CoordinatorCommitDecision& decision,
+                                     std::vector<NamespaceString> affectedNamespaces) {
     const bool isCommit = decision.getDecision() == txn::CommitDecision::kCommit;
     LOGV2_DEBUG(22467,
                 3,
@@ -354,15 +399,6 @@ repl::OpTime persistDecisionBlocking(OperationContext* opCtx,
         sessionInfo.setTxnRetryCounter(*txnNumberAndRetryCounter.getTxnRetryCounter());
     }
 
-    // The transaction participant is already holding a global IX lock (and therefore an FCV IX
-    // lock) when we get to this point. Since the setFCV command takes an FCV S lock, we can hit a
-    // deadlock if the setFCV enqueues its lock after the transaction participant has already
-    // acquired its lock, but before we (the transaction coordinator) acquire ours. To remedy this,
-    // we choose to bypass this barrier that setFCV creates. This is safe because the setFCV command
-    // drains any outstanding cross-shard transactions before completing an FCV upgrade/downgrade.
-    // It does so by waiting for the participant portion of the cross-shard transaction to have
-    // released its FCV IX lock.
-    ShouldNotConflictWithSetFeatureCompatibilityVersionBlock noFCVLock{opCtx->lockState()};
     DBDirectClient client(opCtx);
 
     // Throws if serializing the request or deserializing the response fails.
@@ -391,6 +427,11 @@ repl::OpTime persistDecisionBlocking(OperationContext* opCtx,
                 doc.setId(sessionInfo);
                 doc.setParticipants(std::move(participantList));
                 doc.setDecision(decision);
+                if (decision.getDecision() == CommitDecision::kCommit &&
+                    feature_flags::gFeatureFlagEndOfTransactionChangeEvent.isEnabled(
+                        serverGlobalParams.featureCompatibility.acquireFCVSnapshot())) {
+                    doc.setAffectedNamespaces(std::move(affectedNamespaces));
+                }
                 return doc.toBSON();
             }()));
 
@@ -423,7 +464,6 @@ repl::OpTime persistDecisionBlocking(OperationContext* opCtx,
 
     LOGV2_DEBUG(22469,
                 3,
-                "{sessionId}:{txnNumberAndRetryCounter} Wrote decision {decision}",
                 "Wrote decision",
                 "sessionId"_attr = lsid,
                 "txnNumberAndRetryCounter"_attr = txnNumberAndRetryCounter,
@@ -437,23 +477,32 @@ Future<repl::OpTime> persistDecision(txn::AsyncWorkScheduler& scheduler,
                                      const LogicalSessionId& lsid,
                                      const TxnNumberAndRetryCounter& txnNumberAndRetryCounter,
                                      const txn::ParticipantsList& participants,
-                                     const txn::CoordinatorCommitDecision& decision) {
+                                     const txn::CoordinatorCommitDecision& decision,
+                                     const std::vector<NamespaceString>& affectedNamespaces) {
     return txn::doWhile(
         scheduler,
         boost::none /* no need for a backoff */,
         [](const StatusWith<repl::OpTime>& s) { return shouldRetryPersistingCoordinatorState(s); },
-        [&scheduler, lsid, txnNumberAndRetryCounter, participants, decision] {
+        [&scheduler, lsid, txnNumberAndRetryCounter, participants, decision, affectedNamespaces] {
             return scheduler.scheduleWork(
-                [lsid, txnNumberAndRetryCounter, participants, decision](OperationContext* opCtx) {
+                [lsid,
+                 txnNumberAndRetryCounter,
+                 participants = participants,
+                 decision,
+                 affectedNamespaces = affectedNamespaces](OperationContext* opCtx) mutable {
                     // Do not acquire a storage ticket in order to avoid unnecessary serialization
                     // with other prepared transactions that are holding a storage ticket
                     // themselves; see SERVER-60682.
-                    ScopedAdmissionPriorityForLock setTicketAquisition(
-                        opCtx->lockState(), AdmissionContext::Priority::kImmediate);
+                    ScopedAdmissionPriority<ExecutionAdmissionContext> setTicketAquisition(
+                        opCtx, AdmissionContext::Priority::kExempt);
                     getTransactionCoordinatorWorkerCurOpRepository()->set(
                         opCtx, lsid, txnNumberAndRetryCounter, CoordinatorAction::kWritingDecision);
-                    return persistDecisionBlocking(
-                        opCtx, lsid, txnNumberAndRetryCounter, participants, decision);
+                    return persistDecisionBlocking(opCtx,
+                                                   lsid,
+                                                   txnNumberAndRetryCounter,
+                                                   std::move(participants),
+                                                   decision,
+                                                   std::move(affectedNamespaces));
                 });
         });
 }
@@ -474,7 +523,7 @@ Future<void> sendCommit(ServiceContext* service,
                                    << WriteConcernOptions::Majority));
     if (auto txnRetryCounter = txnNumberAndRetryCounter.getTxnRetryCounter();
         txnRetryCounter && !isDefaultTxnRetryCounter(*txnRetryCounter)) {
-        bob.append(OperationSessionInfo::kTxnRetryCounterFieldName, *txnRetryCounter);
+        bob.append(OperationSessionInfoFromClient::kTxnRetryCounterFieldName, *txnRetryCounter);
     }
     apiParams.appendInfo(&bob);
     auto commitObj = commitTransaction.toBSON(bob.obj());
@@ -518,7 +567,7 @@ Future<void> sendAbort(ServiceContext* service,
                                    << WriteConcernOptions::Majority));
     if (auto txnRetryCounter = txnNumberAndRetryCounter.getTxnRetryCounter();
         txnRetryCounter && !isDefaultTxnRetryCounter(*txnRetryCounter)) {
-        bob.append(OperationSessionInfo::kTxnRetryCounterFieldName, *txnRetryCounter);
+        bob.append(OperationSessionInfoFromClient::kTxnRetryCounterFieldName, *txnRetryCounter);
     }
     apiParams.appendInfo(&bob);
     auto abortObj = abortTransaction.toBSON(bob.obj());
@@ -554,7 +603,6 @@ void deleteCoordinatorDocBlocking(OperationContext* opCtx,
                                   const TxnNumberAndRetryCounter& txnNumberAndRetryCounter) {
     LOGV2_DEBUG(22472,
                 3,
-                "{sessionId}:{txnNumberAndRetryCounter} Going to delete coordinator doc",
                 "Going to delete coordinator doc",
                 "sessionId"_attr = lsid,
                 "txnNumberAndRetryCounter"_attr = txnNumberAndRetryCounter);
@@ -619,7 +667,6 @@ void deleteCoordinatorDocBlocking(OperationContext* opCtx,
 
     LOGV2_DEBUG(22474,
                 3,
-                "{sessionId}:{txnNumberAndRetryCounter} Deleted coordinator doc",
                 "Deleted coordinator doc",
                 "sessionId"_attr = lsid,
                 "txnNumberAndRetryCounter"_attr = txnNumberAndRetryCounter);
@@ -700,8 +747,6 @@ Future<PrepareResponse> sendPrepareToShard(ServiceContext* service,
          operationContextFn] {
             LOGV2_DEBUG(22476,
                         3,
-                        "{sessionId}:{txnNumberAndRetryCounter} Coordinator going to send command "
-                        "{command} to {localOrRemote} shard {shardId}",
                         "Coordinator going to send command to shard",
                         "sessionId"_attr = lsid,
                         "txnNumberAndRetryCounter"_attr = txnNumberAndRetryCounter,
@@ -724,9 +769,9 @@ Future<PrepareResponse> sendPrepareToShard(ServiceContext* service,
                     }
 
                     if (status.isOK()) {
-                        auto prepareTimestampField = response.data["prepareTimestamp"];
-                        if (prepareTimestampField.eoo() ||
-                            prepareTimestampField.timestamp().isNull()) {
+                        auto reply =
+                            PrepareReply::parse(IDLParserContext("PrepareReply"), response.data);
+                        if (!reply.getPrepareTimestamp()) {
                             Status abortStatus(ErrorCodes::Error(50993),
                                                str::stream()
                                                    << "Coordinator shard received an OK response "
@@ -736,38 +781,35 @@ Future<PrepareResponse> sendPrepareToShard(ServiceContext* service,
                                                    << ", which is not an expected behavior. "
                                                       "Interpreting the response as vote to abort");
                             LOGV2(22477,
-                                  "{sessionId}:{txnNumberAndRetryCounter} {error}",
                                   "Coordinator received error from transaction participant",
                                   "sessionId"_attr = lsid,
                                   "txnNumberAndRetryCounter"_attr = txnNumberAndRetryCounter,
                                   "error"_attr = redact(abortStatus));
 
                             return PrepareResponse{
-                                shardId, PrepareVote::kAbort, boost::none, abortStatus};
+                                shardId, PrepareVote::kAbort, boost::none, {}, abortStatus};
                         }
 
                         LOGV2_DEBUG(
                             22478,
                             3,
-                            "{sessionId}:{txnNumberAndRetryCounter} Coordinator shard received a "
-                            "vote to commit from shard {shardId} with prepareTimestamp: "
-                            "{prepareTimestamp}",
                             "Coordinator shard received a vote to commit from participant shard",
                             "sessionId"_attr = lsid,
                             "txnNumberAndRetryCounter"_attr = txnNumberAndRetryCounter,
                             "shardId"_attr = shardId,
-                            "prepareTimestampField"_attr = prepareTimestampField.timestamp());
+                            "prepareTimestamp"_attr = reply.getPrepareTimestamp(),
+                            "affectedNamespaces"_attr = reply.getAffectedNamespaces());
 
-                        return PrepareResponse{shardId,
-                                               PrepareVote::kCommit,
-                                               prepareTimestampField.timestamp(),
-                                               boost::none};
+                        return PrepareResponse{
+                            shardId,
+                            PrepareVote::kCommit,
+                            *reply.getPrepareTimestamp(),
+                            reply.getAffectedNamespaces().value_or(std::vector<NamespaceString>{}),
+                            boost::none};
                     }
 
                     LOGV2_DEBUG(22479,
                                 3,
-                                "{sessionId}:{txnNumberAndRetryCounter} Coordinator shard received "
-                                "{error} from shard {shardId} for {command}",
                                 "Coordinator shard received response from shard",
                                 "sessionId"_attr = lsid,
                                 "txnNumberAndRetryCounter"_attr = txnNumberAndRetryCounter,
@@ -780,6 +822,7 @@ Future<PrepareResponse> sendPrepareToShard(ServiceContext* service,
                             shardId,
                             PrepareVote::kAbort,
                             boost::none,
+                            {},
                             status.withContext(str::stream() << "from shard " << shardId)};
                     }
 
@@ -796,23 +839,21 @@ Future<PrepareResponse> sendPrepareToShard(ServiceContext* service,
                     // treat ShardNotFound as a vote to abort, which is always safe since the node
                     // must then send abort.
                     return Future<PrepareResponse>::makeReady(
-                        {shardId, CommitDecision::kAbort, boost::none, status});
+                        {shardId, CommitDecision::kAbort, boost::none, {}, status});
                 });
         });
 
     return std::move(f).onError<ErrorCodes::TransactionCoordinatorReachedAbortDecision>(
         [lsid, txnNumberAndRetryCounter, shardId](const Status& status) {
-            LOGV2_DEBUG(
-                22480,
-                3,
-                "{sessionId}:{txnNumberAndRetryCounter} Prepare stopped retrying due to retrying "
-                "being cancelled",
-                "Prepare stopped retrying due to retrying being cancelled",
-                "sessionId"_attr = lsid,
-                "txnNumberAndRetryCounter"_attr = txnNumberAndRetryCounter);
+            LOGV2_DEBUG(22480,
+                        3,
+                        "Prepare stopped retrying due to retrying being cancelled",
+                        "sessionId"_attr = lsid,
+                        "txnNumberAndRetryCounter"_attr = txnNumberAndRetryCounter);
             return PrepareResponse{shardId,
                                    boost::none,
                                    boost::none,
+                                   {},
                                    Status(ErrorCodes::NoSuchTransaction, status.reason())};
         });
 }
@@ -843,8 +884,6 @@ Future<void> sendDecisionToShard(ServiceContext* service,
          commandObj = commandObj.getOwned()] {
             LOGV2_DEBUG(22481,
                         3,
-                        "{sessionId}:{txnNumberAndRetryCounter} Coordinator going to send command "
-                        "{command} to {localOrRemote} shard {shardId}",
                         "Coordinator going to send command to shard",
                         "sessionId"_attr = lsid,
                         "txnNumberAndRetryCounter"_attr = txnNumberAndRetryCounter,
@@ -866,17 +905,14 @@ Future<void> sendDecisionToShard(ServiceContext* service,
                         status = wcStatus;
                     }
 
-                    LOGV2_DEBUG(
-                        22482,
-                        3,
-                        "{sessionId}:{txnNumberAndRetryCounter}  Coordinator shard received "
-                        "{status} in response to {command} from shard {shardId}",
-                        "Coordinator shard received response from shard",
-                        "sessionId"_attr = lsid,
-                        "txnNumberAndRetryCounter"_attr = txnNumberAndRetryCounter,
-                        "status"_attr = status,
-                        "command"_attr = commandObj,
-                        "shardId"_attr = shardId);
+                    LOGV2_DEBUG(22482,
+                                3,
+                                "Coordinator shard received response from shard",
+                                "sessionId"_attr = lsid,
+                                "txnNumberAndRetryCounter"_attr = txnNumberAndRetryCounter,
+                                "status"_attr = status,
+                                "command"_attr = commandObj,
+                                "shardId"_attr = shardId);
 
                     if (ErrorCodes::isVoteAbortError(status.code())) {
                         // Interpret voteAbort errors as an ack.
@@ -895,6 +931,28 @@ Future<void> sendDecisionToShard(ServiceContext* service,
         });
 }
 
+Future<void> writeEndOfTransaction(txn::AsyncWorkScheduler& scheduler,
+                                   const LogicalSessionId& lsid,
+                                   const TxnNumberAndRetryCounter& txnNumberAndRetryCounter,
+                                   const std::vector<NamespaceString>& affectedNamespaces) {
+    if (MONGO_unlikely(hangBeforeWritingEndOfTransaction.shouldFail())) {
+        LOGV2(8288302, "Hit hangBeforeWritingEndOfTransaction failpoint");
+        hangBeforeWritingEndOfTransaction.pauseWhileSet();
+    }
+
+    if (!feature_flags::gFeatureFlagEndOfTransactionChangeEvent.isEnabled(
+            serverGlobalParams.featureCompatibility.acquireFCVSnapshot())) {
+        return Future<void>::makeReady();
+    }
+    return scheduler.scheduleWork(
+        [lsid, txnNumberAndRetryCounter, affectedNamespaces](OperationContext* opCtx) {
+            getTransactionCoordinatorWorkerCurOpRepository()->set(
+                opCtx, lsid, txnNumberAndRetryCounter, CoordinatorAction::kWritingEndOfTransaction);
+            notifyChangeStreamOnEndOfTransaction(
+                opCtx, lsid, txnNumberAndRetryCounter.getTxnNumber(), affectedNamespaces);
+        });
+}
+
 std::string txnIdToString(const LogicalSessionId& lsid,
                           const TxnNumberAndRetryCounter& txnNumberAndRetryCounter) {
     str::stream ss;
