@@ -27,8 +27,26 @@
  *    it in the license file.
  */
 
+#include <memory>
+#include <mutex>
+#include <string>
+#include <utility>
+
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
 #include "mongo/base/shim.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/db/client.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/s/transaction_coordinator_worker_curop_repository.h"
+#include "mongo/db/service_context.h"
+#include "mongo/db/session/logical_session_id.h"
+#include "mongo/db/session/logical_session_id_gen.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/clock_source.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/time_support.h"
 
 namespace mongo {
 namespace {
@@ -119,11 +137,12 @@ std::string TransactionCoordinatorWorkerCurOpInfo::toString(CoordinatorAction ac
             return "writingParticipantList";
         case CoordinatorAction::kWritingDecision:
             return "writingDecision";
+        case CoordinatorAction::kWritingEndOfTransaction:
+            return "writingEndOfTransaction";
         case CoordinatorAction::kDeletingCoordinatorDoc:
             return "deletingCoordinatorDoc";
-        default:
-            MONGO_UNREACHABLE
     }
+    MONGO_UNREACHABLE;
 }
 
 void TransactionCoordinatorWorkerCurOpInfo::reportState(BSONObjBuilder* parent) const {
