@@ -29,6 +29,8 @@
 
 #pragma once
 
+#include <string>
+
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/repl/replica_set_aware_service.h"
@@ -36,12 +38,6 @@
 
 namespace mongo {
 
-namespace user_writes_recoverable_critical_section_util {
-
-bool inRecoveryMode(OperationContext* opCtx);
-
-}
-
 /**
  * Represents the 'user writes blocking' critical section. The critical section status is persisted
  * on disk and it's in-memory representation is kept in sync with the persisted state through an
@@ -145,19 +141,19 @@ public:
     void recoverRecoverableCriticalSections(OperationContext* opCtx);
 
 private:
-    void onInitialDataAvailable(OperationContext* opCtx,
-                                bool isMajorityDataAvailable) override final {
+    void onInitialDataAvailable(OperationContext* opCtx, bool isMajorityDataAvailable) final {
         recoverRecoverableCriticalSections(opCtx);
     }
 
-    void onStartup(OperationContext* opCtx) override final {}
-    void onSetCurrentConfig(OperationContext* opCtx) override final {}
-    void onShutdown() override final {}
-    void onStepUpBegin(OperationContext* opCtx, long long term) override final {}
-    void onStepUpComplete(OperationContext* opCtx, long long term) override final {}
-    void onStepDown() override final {}
-    void onBecomeArbiter() override final {}
-    inline std::string getServiceName() const override final {
+    void onStartup(OperationContext* opCtx) final {}
+    void onSetCurrentConfig(OperationContext* opCtx) final {}
+    void onShutdown() final {}
+    void onStepUpBegin(OperationContext* opCtx, long long term) final {}
+    void onStepUpComplete(OperationContext* opCtx, long long term) final {}
+    void onStepDown() final {}
+    void onRollback() final {}
+    void onBecomeArbiter() final {}
+    inline std::string getServiceName() const final {
         return "UserWritesRecoverableCriticalSectionService";
     }
 };
