@@ -29,16 +29,32 @@
 
 #pragma once
 
+#include <algorithm>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional.hpp>
+#include <boost/optional/optional.hpp>
+#include <ctime>
+#include <string>
+#include <variant>
+#include <vector>
+
+#include "mongo/base/string_data.h"
 #include "mongo/bson/bsonobj.h"
 #include "mongo/db/auth/cluster_auth_mode.h"
 #include "mongo/db/cluster_role.h"
 #include "mongo/logv2/log_format.h"
 #include "mongo/platform/atomic_word.h"
 #include "mongo/platform/process_id.h"
-#include "mongo/stdx/variant.h"
+#include "mongo/util/assert_util_core.h"
 #include "mongo/util/net/cidr.h"
 #include "mongo/util/version/releases.h"
-#include "mongo/util/versioned_value.h"
+
+#ifdef _WIN32
+#include <winsock2.h>
+#else
+#include <sys/socket.h>
+#endif
 
 namespace mongo {
 
@@ -51,25 +67,38 @@ struct ServerGlobalParams {
 
     int port = DefaultDBPort;  // --port
     enum {
-        ConfigServerPort = 27019,
-        CryptDServerPort = 27020,
+        RouterPort = 27016,
         DefaultDBPort = 27017,
         ShardServerPort = 27018,
+        ConfigServerPort = 27019,
+        CryptDServerPort = 27020,
+// TODO: SERVER-80343 Remove this ifdef once gRPC is compiled on all variants
+#ifdef MONGO_CONFIG_GRPC
+        DefaultGRPCServerPort = 27021,
+#endif
+        DefaultMagicRestorePort = 27022,
     };
 
+    enum MaintenanceMode { None, ReplicaSetMode, StandaloneMode };
+
     static std::string getPortSettingHelpText();
 
     std::vector<std::string> bind_ips;  // --bind_ip
     bool enableIPv6 = false;
     bool rest = false;  // --rest
 
-    int listenBacklog = 0;  // --listenBacklog, real default is SOMAXCONN
+    int listenBacklog = SOMAXCONN;  // --listenBacklog
 
     AtomicWord<bool> quiet{false};  // --quiet
 
     ClusterRole clusterRole = ClusterRole::None;  // --configsvr/--shardsvr
+    MaintenanceMode maintenanceMode;              // --maintenanceMode
 
-    boost::optional<int> proxyPort;  // --proxyPort
+    bool upgradeBackCompat{false};    // --upgradeBackCompat
+    bool downgradeBackCompat{false};  // --downgradeBackCompat
+
+    boost::optional<int> routerPort;      // --routerPort
+    bool doAutoBootstrapSharding{false};  // This is derived from other settings during startup.
 
     bool objcheck = true;  // --objcheck
 
@@ -82,16 +111,15 @@ struct ServerGlobalParams {
     AtomicWord<int> slowMS{100};           // --time in ms that is "slow"
     AtomicWord<double> sampleRate{1.0};    // --samplerate rate at which to sample slow queries
     int defaultLocalThresholdMillis = 15;  // --localThreshold in ms to consider a node local
-    bool moveParanoia = false;             // for move chunk paranoia
 
-    bool noUnixSocket = false;    // --nounixsocket
-    bool doFork = false;          // --fork
+    bool noUnixSocket = false;  // --nounixsocket
+    bool doFork = false;        // --fork
+    bool isMongoBridge = false;
+
     std::string socket = "/tmp";  // UNIX domain socket directory
-    std::string transportLayer;   // --transportLayer (must be either "asio" or "legacy")
 
     size_t maxConns = DEFAULT_MAX_CONN;  // Maximum number of simultaneous open connections.
-    std::vector<stdx::variant<CIDR, std::string>> maxIncomingConnsOverride;
-    VersionedValue<std::vector<stdx::variant<CIDR, std::string>>> maxEstablishingConnsOverride;
+    std::vector<std::variant<CIDR, std::string>> maxConnsOverride;
     int reservedAdminThreads = 0;
 
     int unixSocketPermissions = DEFAULT_UNIX_PERMS;  // permissions for the UNIX domain socket
@@ -132,6 +160,12 @@ struct ServerGlobalParams {
     // True if the current binary version is an LTS Version.
     static constexpr bool kIsLTSBinaryVersion = false;
 
+// TODO: SERVER-80343 Remove this ifdef once gRPC is compiled on all variants
+#ifdef MONGO_CONFIG_GRPC
+    int grpcPort = DefaultGRPCServerPort;
+    int grpcServerMaxThreads = 1000;
+#endif
+
     /**
      * Represents a "snapshot" of the in-memory FCV at a particular point in time.
      * This is useful for callers who need to perform multiple FCV checks and expect the checks to
