@@ -30,34 +30,45 @@
 
 #include "mongo/db/server_options_server_helpers.h"
 
-#include <algorithm>
-#include <boost/algorithm/string/classification.hpp>
+#include <boost/algorithm/string/constants.hpp>
 #include <boost/algorithm/string/split.hpp>
 #include <boost/algorithm/string/trim.hpp>
-#include <boost/filesystem.hpp>
+#include <boost/core/addressof.hpp>
 #include <boost/filesystem/operations.hpp>
-#include <cstdlib>
+#include <boost/filesystem/path.hpp>
+#include <boost/function/function_base.hpp>
+#include <boost/iterator/iterator_facade.hpp>
 #include <fmt/format.h>
-#include <ios>
+// IWYU pragma: no_include "boost/system/detail/error_code.hpp"
+#include <boost/type_index/type_index_facade.hpp>
+// IWYU pragma: no_include "ext/alloc_traits.h"
+#include <cstdlib>
 #include <iostream>
+#include <map>
+#include <type_traits>
+#include <utility>
+#include <variant>
 
+#include "mongo/base/error_codes.h"
 #include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/bson/json.h"
-#include "mongo/bson/util/builder.h"
-#include "mongo/config.h"
+#include "mongo/bson/oid.h"
+#include "mongo/config.h"  // IWYU pragma: keep
+#include "mongo/db/auth/cluster_auth_mode.h"
 #include "mongo/db/server_options.h"
-#include "mongo/db/server_options_base.h"
 #include "mongo/db/server_options_helpers.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
 #include "mongo/logv2/log_component.h"
 #include "mongo/transport/message_compressor_registry.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/cmdline_utils/censor_cmdline.h"
-#include "mongo/util/fail_point.h"
-#include "mongo/util/net/sock.h"
+#include "mongo/util/net/cidr.h"
 #include "mongo/util/net/socket_utils.h"
-#include "mongo/util/net/ssl_options.h"
-#include "mongo/util/options_parser/options_parser.h"
-#include "mongo/util/options_parser/startup_options.h"
+#include "mongo/util/options_parser/value.h"
 #include "mongo/util/str.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kControl
@@ -150,10 +161,7 @@ void printCommandLineOpts(std::ostream* os) {
                       tojson(serverGlobalParams.parsedOpts, ExtendedRelaxedV2_0_0, true))
             << std::endl;
     } else {
-        LOGV2(21951,
-              "Options set by command line: {options}",
-              "Options set by command line",
-              "options"_attr = serverGlobalParams.parsedOpts);
+        LOGV2(21951, "Options set by command line", "options"_attr = serverGlobalParams.parsedOpts);
     }
 }
 
@@ -223,16 +231,6 @@ Status validateServerOptions(const moe::Environment& params) {
                                       " and replication.replSet");
             }
         }
-
-        bool disableJavaScriptJITUsed = parameters.find("disableJavaScriptJIT") != parameters.end();
-        if (disableJavaScriptJITUsed) {
-            LOGV2_WARNING(
-                10201701,
-                "The Javascript Just In Time compiler is deprecated, and should be disabled "
-                "for security reasons. The server parameter will be removed in a future major "
-                "version. See https://www.mongodb.com/docs/manual/reference/parameters for "
-                "more information.");
-        }
     }
     if ((params.count("security.authorization") &&
          params["security.authorization"].as<std::string>() == "enabled") ||
@@ -345,13 +343,6 @@ Status storeServerOptions(const moe::Environment& params) {
         serverGlobalParams.listenBacklog = params["net.listenBacklog"].as<int>();
     }
 
-    if (params.count("net.transportLayer")) {
-        serverGlobalParams.transportLayer = params["net.transportLayer"].as<std::string>();
-        if (serverGlobalParams.transportLayer != "asio") {
-            return {ErrorCodes::BadValue, "Unsupported value for transportLayer. Must be \"asio\""};
-        }
-    }
-
     if (params.count("security.transitionToAuth")) {
         serverGlobalParams.transitionToAuth = params["security.transitionToAuth"].as<bool>();
     }
@@ -376,9 +367,9 @@ Status storeServerOptions(const moe::Environment& params) {
         for (const auto& range : ranges) {
             auto swr = CIDR::parse(range);
             if (!swr.isOK()) {
-                serverGlobalParams.maxIncomingConnsOverride.push_back(range);
+                serverGlobalParams.maxConnsOverride.push_back(range);
             } else {
-                serverGlobalParams.maxIncomingConnsOverride.push_back(std::move(swr.getValue()));
+                serverGlobalParams.maxConnsOverride.push_back(std::move(swr.getValue()));
             }
         }
     }
@@ -470,7 +461,7 @@ Status storeServerOptions(const moe::Environment& params) {
     }
 
     if (params.count("net.compression.compressors")) {
-        const auto ret =
+        auto ret =
             storeMessageCompressionOptions(params["net.compression.compressors"].as<string>());
         if (!ret.isOK()) {
             return ret;
