@@ -27,18 +27,20 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
 
-#include "mongo/platform/basic.h"
+#include <boost/algorithm/string/join.hpp>
+#include <ostream>
+#include <utility>
 
-#include "mongo/config.h"
+#include <boost/filesystem/operations.hpp>
+#include <boost/filesystem/path.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+// IWYU pragma: no_include "boost/system/detail/error_code.hpp"
+#include <fmt/format.h>
 
-#if defined(MONGO_CONFIG_HAVE_HEADER_UNISTD_H)
-#include <unistd.h>
-#endif
 #ifndef _WIN32
 #include <cstdlib>
-#include <sys/types.h>
 #include <sys/wait.h>
 #endif
 
@@ -46,30 +48,48 @@
 #include <TargetConditionals.h>
 #endif
 
-#include <boost/algorithm/string/join.hpp>
-#include <boost/filesystem.hpp>
-
-#include "mongo/base/init.h"
+#include "mongo/base/error_codes.h"
+#include "mongo/base/init.h"  // IWYU pragma: keep
+#include "mongo/base/initializer.h"
 #include "mongo/base/parse_number.h"
+#include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/bson/util/builder.h"
+#include "mongo/bson/util/builder_fwd.h"
+#include "mongo/config.h"  // IWYU pragma: keep
 #include "mongo/db/server_options.h"
 #include "mongo/db/server_options_base.h"
 #include "mongo/db/server_options_helpers.h"
 #include "mongo/db/server_options_nongeneral_gen.h"
 #include "mongo/db/server_options_server_helpers.h"
-#include "mongo/idl/server_parameter.h"
+#include "mongo/db/server_parameter.h"
+#include "mongo/db/tenant_id.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/log_severity.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/bson_test_util.h"
+#include "mongo/unittest/framework.h"
 #include "mongo/unittest/log_test.h"
-#include "mongo/unittest/unittest.h"
 #include "mongo/util/errno_util.h"
 #include "mongo/util/options_parser/environment.h"
 #include "mongo/util/options_parser/option_section.h"
 #include "mongo/util/options_parser/options_parser.h"
+#include "mongo/util/options_parser/value.h"
 #include "mongo/util/scopeguard.h"
+#include "mongo/util/str.h"
 
+#if defined(MONGO_CONFIG_HAVE_HEADER_UNISTD_H)
+#include <unistd.h>
+#endif
+
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
 
 namespace mongo {
 namespace {
 
+using mongo::ClusterRole;
 using mongo::ErrorCodes;
 using mongo::Status;
 
@@ -86,7 +106,9 @@ MONGO_INITIALIZER(ServerLogRedirection)(mongo::InitializerContext*) {
 
 class OptionsParserTester : public moe::OptionsParser {
 public:
-    Status readConfigFile(const std::string& filename, std::string* config, ConfigExpand) {
+    Status readConfigFile(const std::string& filename,
+                          std::string* config,
+                          moe::ConfigExpand) override {
         if (filename != _filename) {
             ::mongo::StringBuilder sb;
             sb << "Parser using filename: " << filename
@@ -597,7 +619,7 @@ TEST(SetupOptions, SlowMsCommandLineParamParsesSuccessfully) {
     ASSERT_OK(::mongo::setupServerOptions(argv));
     ASSERT_OK(::mongo::storeServerOptions(environment));
 
-    ASSERT_EQ(::mongo::serverGlobalParams.slowMS, 300);
+    ASSERT_EQ(::mongo::serverGlobalParams.slowMS.load(), 300);
 }
 
 TEST(SetupOptions, SlowMsParamInitializedSuccessfullyFromINIConfigFile) {
@@ -622,7 +644,7 @@ TEST(SetupOptions, SlowMsParamInitializedSuccessfullyFromINIConfigFile) {
     ASSERT_OK(::mongo::setupServerOptions(argv));
     ASSERT_OK(::mongo::storeServerOptions(environment));
 
-    ASSERT_EQ(::mongo::serverGlobalParams.slowMS, 300);
+    ASSERT_EQ(::mongo::serverGlobalParams.slowMS.load(), 300);
 }
 
 TEST(SetupOptions, SlowMsParamInitializedSuccessfullyFromYAMLConfigFile) {
@@ -649,7 +671,7 @@ TEST(SetupOptions, SlowMsParamInitializedSuccessfullyFromYAMLConfigFile) {
     ASSERT_OK(::mongo::setupServerOptions(argv));
     ASSERT_OK(::mongo::storeServerOptions(environment));
 
-    ASSERT_EQ(::mongo::serverGlobalParams.slowMS, 300);
+    ASSERT_EQ(::mongo::serverGlobalParams.slowMS.load(), 300);
 }
 
 TEST(SetupOptions, NonNumericSlowMsCommandLineOptionFailsToParse) {
@@ -705,7 +727,7 @@ TEST(SetupOptions, SampleRateCommandLineParamParsesSuccessfully) {
     ASSERT_OK(::mongo::setupServerOptions(argv));
     ASSERT_OK(::mongo::storeServerOptions(environment));
 
-    ASSERT_EQ(::mongo::serverGlobalParams.sampleRate, 0.5);
+    ASSERT_EQ(::mongo::serverGlobalParams.sampleRate.load(), 0.5);
 }
 
 TEST(SetupOptions, SampleRateParamInitializedSuccessfullyFromINIConfigFile) {
@@ -730,7 +752,7 @@ TEST(SetupOptions, SampleRateParamInitializedSuccessfullyFromINIConfigFile) {
     ASSERT_OK(::mongo::setupServerOptions(argv));
     ASSERT_OK(::mongo::storeServerOptions(environment));
 
-    ASSERT_EQ(::mongo::serverGlobalParams.sampleRate, 0.5);
+    ASSERT_EQ(::mongo::serverGlobalParams.sampleRate.load(), 0.5);
 }
 
 TEST(SetupOptions, SampleRateParamInitializedSuccessfullyFromYAMLConfigFile) {
@@ -757,7 +779,7 @@ TEST(SetupOptions, SampleRateParamInitializedSuccessfullyFromYAMLConfigFile) {
     ASSERT_OK(::mongo::setupServerOptions(argv));
     ASSERT_OK(::mongo::storeServerOptions(environment));
 
-    ASSERT_EQ(::mongo::serverGlobalParams.sampleRate, 0.5);
+    ASSERT_EQ(::mongo::serverGlobalParams.sampleRate.load(), 0.5);
 }
 
 TEST(SetupOptions, NonNumericSampleRateCommandLineOptionFailsToParse) {
@@ -936,6 +958,114 @@ TEST(SetupOptions, ForkOptionAlwaysFalseWithNoforkEnvVar) {
 }
 #endif
 
+TEST(ClusterRole, MonoRole) {
+    const ClusterRole noRole{ClusterRole::None};
+    ASSERT_TRUE(noRole.has(ClusterRole::None));
+    ASSERT_FALSE(noRole.has(ClusterRole::ShardServer));
+    ASSERT_FALSE(noRole.has(ClusterRole::ConfigServer));
+    ASSERT_FALSE(noRole.has(ClusterRole::RouterServer));
+    ASSERT_TRUE(noRole.hasExclusively(ClusterRole::None));
+    ASSERT_FALSE(noRole.hasExclusively(ClusterRole::ShardServer));
+    ASSERT_FALSE(noRole.hasExclusively(ClusterRole::ConfigServer));
+    ASSERT_FALSE(noRole.hasExclusively(ClusterRole::RouterServer));
+
+    const ClusterRole shardRole{ClusterRole::ShardServer};
+    ASSERT_FALSE(shardRole.has(ClusterRole::None));
+    ASSERT_TRUE(shardRole.has(ClusterRole::ShardServer));
+    ASSERT_FALSE(shardRole.has(ClusterRole::ConfigServer));
+    ASSERT_FALSE(shardRole.has(ClusterRole::RouterServer));
+    ASSERT_FALSE(shardRole.hasExclusively(ClusterRole::None));
+    ASSERT_TRUE(shardRole.hasExclusively(ClusterRole::ShardServer));
+    ASSERT_FALSE(shardRole.hasExclusively(ClusterRole::ConfigServer));
+    ASSERT_FALSE(shardRole.hasExclusively(ClusterRole::RouterServer));
+
+    // Role cannot be set to config server only.
+
+    const ClusterRole routerRole{ClusterRole::RouterServer};
+    ASSERT_FALSE(routerRole.has(ClusterRole::None));
+    ASSERT_FALSE(routerRole.has(ClusterRole::ShardServer));
+    ASSERT_FALSE(routerRole.has(ClusterRole::ConfigServer));
+    ASSERT_TRUE(routerRole.has(ClusterRole::RouterServer));
+    ASSERT_FALSE(routerRole.hasExclusively(ClusterRole::None));
+    ASSERT_FALSE(routerRole.hasExclusively(ClusterRole::ShardServer));
+    ASSERT_FALSE(routerRole.hasExclusively(ClusterRole::ConfigServer));
+    ASSERT_TRUE(routerRole.hasExclusively(ClusterRole::RouterServer));
+}
+
+TEST(ClusterRole, MultiRole) {
+    const ClusterRole shardAndConfigRole{ClusterRole::ShardServer, ClusterRole::ConfigServer};
+    ASSERT_FALSE(shardAndConfigRole.has(ClusterRole::None));
+    ASSERT_TRUE(shardAndConfigRole.has(ClusterRole::ShardServer));
+    ASSERT_TRUE(shardAndConfigRole.has(ClusterRole::ConfigServer));
+    ASSERT_FALSE(shardAndConfigRole.has(ClusterRole::RouterServer));
+    ASSERT_FALSE(shardAndConfigRole.hasExclusively(ClusterRole::None));
+    ASSERT_FALSE(shardAndConfigRole.hasExclusively(ClusterRole::ShardServer));
+    ASSERT_FALSE(shardAndConfigRole.hasExclusively(ClusterRole::ConfigServer));
+    ASSERT_FALSE(shardAndConfigRole.hasExclusively(ClusterRole::RouterServer));
+
+    const ClusterRole shardAndRouterRole{ClusterRole::ShardServer, ClusterRole::RouterServer};
+    ASSERT_FALSE(shardAndRouterRole.has(ClusterRole::None));
+    ASSERT_TRUE(shardAndRouterRole.has(ClusterRole::ShardServer));
+    ASSERT_FALSE(shardAndRouterRole.has(ClusterRole::ConfigServer));
+    ASSERT_TRUE(shardAndRouterRole.has(ClusterRole::RouterServer));
+    ASSERT_FALSE(shardAndRouterRole.hasExclusively(ClusterRole::None));
+    ASSERT_FALSE(shardAndRouterRole.hasExclusively(ClusterRole::ShardServer));
+    ASSERT_FALSE(shardAndRouterRole.hasExclusively(ClusterRole::ConfigServer));
+    ASSERT_FALSE(shardAndRouterRole.hasExclusively(ClusterRole::RouterServer));
+
+    const ClusterRole configAndRouterRole{ClusterRole::ConfigServer, ClusterRole::RouterServer};
+    ASSERT_FALSE(configAndRouterRole.has(ClusterRole::None));
+    ASSERT_FALSE(configAndRouterRole.has(ClusterRole::ShardServer));
+    ASSERT_TRUE(configAndRouterRole.has(ClusterRole::ConfigServer));
+    ASSERT_TRUE(configAndRouterRole.has(ClusterRole::RouterServer));
+    ASSERT_FALSE(configAndRouterRole.hasExclusively(ClusterRole::None));
+    ASSERT_FALSE(configAndRouterRole.hasExclusively(ClusterRole::ShardServer));
+    ASSERT_FALSE(configAndRouterRole.hasExclusively(ClusterRole::ConfigServer));
+    ASSERT_FALSE(configAndRouterRole.hasExclusively(ClusterRole::RouterServer));
+
+    const ClusterRole anyRole{
+        ClusterRole::ShardServer, ClusterRole::ConfigServer, ClusterRole::RouterServer};
+    ASSERT_FALSE(anyRole.has(ClusterRole::None));
+    ASSERT_TRUE(anyRole.has(ClusterRole::ShardServer));
+    ASSERT_TRUE(anyRole.has(ClusterRole::ConfigServer));
+    ASSERT_TRUE(anyRole.has(ClusterRole::RouterServer));
+    ASSERT_FALSE(anyRole.hasExclusively(ClusterRole::None));
+    ASSERT_FALSE(anyRole.hasExclusively(ClusterRole::ShardServer));
+    ASSERT_FALSE(anyRole.hasExclusively(ClusterRole::ConfigServer));
+    ASSERT_FALSE(anyRole.hasExclusively(ClusterRole::RouterServer));
+}
+
+TEST(ClusterRole, ToBson) {
+    using R = ClusterRole;
+    const auto s = R::ShardServer;
+    const auto c = R::ConfigServer;
+    const auto r = R::RouterServer;
+
+    struct Case {
+        R actual;
+        std::vector<std::string> expected;
+    };
+
+    const std::vector<Case> allCases{
+        {R{}, {}},
+        {R{s}, {"shard"}},
+        // {R{c}, {"config"}}, // Role cannot be set to config server only.
+        {R{s, c}, {"shard", "config"}},
+        {R{r}, {"router"}},
+        {R{s, r}, {"shard", "router"}},
+        {R{c, r}, {"config", "router"}},
+        {R{s, c, r}, {"shard", "config", "router"}},
+    };
+
+    for (auto&& [actual, expected] : allCases) {
+        BSONArrayBuilder bab;
+        for (auto&& roleStr : expected) {
+            bab.append(roleStr);
+        }
+        ASSERT_BSONOBJ_EQ(bab.arr(), toBSON(actual));
+    }
+}
+
 #if !defined(_WIN32) && !(defined(__APPLE__) && TARGET_OS_TV)
 
 #define ASSERT_BOOST_SUCCESS(ec) ASSERT_FALSE(ec) << ec.message()
@@ -1042,7 +1172,10 @@ TEST(SetupOptions, UnlinkedCwd) {
     // Naive rmdir of cwd doesn't work on Solaris doesn't work (no matter how it's specified).
     // So we use a subprocess to unlink the dir.
     pid_t pid = fork();
-    ASSERT_NOT_EQUALS(pid, -1) << "unable to fork: " << ::mongo::errnoWithDescription();
+    if (pid == -1) {
+        auto ec = lastSystemError();
+        FAIL("unable to fork") << errorMessage(ec);
+    }
     if (pid == 0) {
         // Subprocess
         // No exceptions, ASSERT(), FAIL() or logging.
@@ -1095,11 +1228,14 @@ public:
         TestServerParameter(StringData name, ServerParameterType spt, int x)
             : ServerParameter(name, spt), val{x} {}
 
-        void append(OperationContext*, BSONObjBuilder& bob, const std::string& name) final {
-            bob.append(name, val);
+        void append(OperationContext*,
+                    BSONObjBuilder* bob,
+                    StringData name,
+                    const boost::optional<TenantId>&) final {
+            bob->append(name, val);
         }
 
-        Status setFromString(const std::string& str) final {
+        Status setFromString(StringData str, const boost::optional<TenantId>&) final {
             int value;
             Status status = NumberParser{}(str, &value);
             if (!status.isOK())
