@@ -27,14 +27,32 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <algorithm>
+#include <fmt/format.h>
+#include <memory>
+#include <string>
+#include <utility>
 
-#include "mongo/db/catalog_raii.h"
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/db/repl/tenant_migration_access_blocker.h"
+#include "mongo/db/repl/tenant_migration_access_blocker_registry.h"
 #include "mongo/db/repl/tenant_migration_access_blocker_util.h"
+#include "mongo/db/repl/tenant_migration_donor_access_blocker.h"
 #include "mongo/db/serverless/serverless_operation_lock_registry.h"
 #include "mongo/db/serverless/shard_split_donor_op_observer.h"
 #include "mongo/db/serverless/shard_split_state_machine_gen.h"
 #include "mongo/db/serverless/shard_split_utils.h"
+#include "mongo/db/storage/recovery_unit.h"
+#include "mongo/db/tenant_id.h"
+#include "mongo/db/transaction_resources.h"
+#include "mongo/idl/idl_parser.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/str.h"
 
 namespace mongo {
 namespace {
@@ -102,6 +120,8 @@ ShardSplitDonorDocument parseAndValidateDonorDocument(const BSONObj& doc) {
                                 doc.toString()),
                     !donorStateDoc.getAbortReason());
             break;
+        case ShardSplitDonorStateEnum::kRecipientCaughtUp:
+            break;
         case ShardSplitDonorStateEnum::kCommitted:
             uassert(ErrorCodes::BadValue,
                     fmt::format(errmsg,
@@ -149,23 +169,26 @@ void onTransitionToAbortingIndexBuilds(OperationContext* opCtx,
 
     ServerlessOperationLockRegistry::get(opCtx->getServiceContext())
         .acquireLock(ServerlessOperationLockRegistry::LockType::kShardSplit, donorStateDoc.getId());
-    opCtx->recoveryUnit()->onRollback(
+    shard_role_details::getRecoveryUnit(opCtx)->onRollback(
         [migrationId = donorStateDoc.getId()](OperationContext* opCtx) {
             ServerlessOperationLockRegistry::get(opCtx->getServiceContext())
                 .releaseLock(ServerlessOperationLockRegistry::LockType::kShardSplit, migrationId);
         });
 
     auto tenantIds = *donorStateDoc.getTenantIds();
-    auto mtab = std::make_shared<TenantMigrationDonorAccessBlocker>(opCtx->getServiceContext(),
-                                                                    donorStateDoc.getId());
-    TenantMigrationAccessBlockerRegistry::get(opCtx->getServiceContext()).add(tenantIds, mtab);
+    auto& registry = TenantMigrationAccessBlockerRegistry::get(opCtx->getServiceContext());
+    for (const auto& tenantId : tenantIds) {
+        registry.add(tenantId,
+                     std::make_shared<TenantMigrationDonorAccessBlocker>(opCtx->getServiceContext(),
+                                                                         donorStateDoc.getId()));
+    }
 
-    opCtx->recoveryUnit()->onRollback([migrationId =
-                                           donorStateDoc.getId()](OperationContext* opCtx) {
-        TenantMigrationAccessBlockerRegistry::get(opCtx->getServiceContext())
-            .removeAccessBlockersForMigration(migrationId,
-                                              TenantMigrationAccessBlocker::BlockerType::kDonor);
-    });
+    shard_role_details::getRecoveryUnit(opCtx)->onRollback(
+        [migrationId = donorStateDoc.getId()](OperationContext* opCtx) {
+            TenantMigrationAccessBlockerRegistry::get(opCtx->getServiceContext())
+                .removeAccessBlockersForMigration(
+                    migrationId, TenantMigrationAccessBlocker::BlockerType::kDonor);
+        });
 }
 
 /**
@@ -175,21 +198,25 @@ void onTransitionToBlocking(OperationContext* opCtx, const ShardSplitDonorDocume
     invariant(donorStateDoc.getState() == ShardSplitDonorStateEnum::kBlocking);
     invariant(donorStateDoc.getBlockOpTime());
 
-    auto mtab = tenant_migration_access_blocker::getDonorAccessBlockerForMigration(
-        opCtx->getServiceContext(), donorStateDoc.getId());
-    invariant(mtab);
+    auto mtabVector = TenantMigrationAccessBlockerRegistry::get(opCtx->getServiceContext())
+                          .getDonorAccessBlockersForMigration(donorStateDoc.getId());
+    invariant(!mtabVector.empty());
 
-    if (isSecondary(opCtx)) {
-        // A primary calls startBlockingWrites on the TenantMigrationDonorAccessBlocker before
-        // reserving the OpTime for the "start blocking" write, so only secondaries call
-        // startBlockingWrites on the TenantMigrationDonorAccessBlocker in the op observer.
-        mtab->startBlockingWrites();
+    for (auto& mtab : mtabVector) {
+        invariant(mtab);
+
+        if (isSecondary(opCtx)) {
+            // A primary calls startBlockingWrites on the TenantMigrationDonorAccessBlocker before
+            // reserving the OpTime for the "start blocking" write, so only secondaries call
+            // startBlockingWrites on the TenantMigrationDonorAccessBlocker in the op observer.
+            mtab->startBlockingWrites();
+        }
+
+        // Both primaries and secondaries call startBlockingReadsAfter in the op observer, since
+        // startBlockingReadsAfter just needs to be called before the "start blocking" write's oplog
+        // hole is filled.
+        mtab->startBlockingReadsAfter(donorStateDoc.getBlockOpTime()->getTimestamp());
     }
-
-    // Both primaries and secondaries call startBlockingReadsAfter in the op observer, since
-    // startBlockingReadsAfter just needs to be called before the "start blocking" write's oplog
-    // hole is filled.
-    mtab->startBlockingReadsAfter(donorStateDoc.getBlockOpTime()->getTimestamp());
 }
 
 /**
@@ -200,11 +227,15 @@ void onTransitionToCommitted(OperationContext* opCtx,
     invariant(donorStateDoc.getState() == ShardSplitDonorStateEnum::kCommitted);
     invariant(donorStateDoc.getCommitOrAbortOpTime());
 
-    auto mtab = tenant_migration_access_blocker::getDonorAccessBlockerForMigration(
-        opCtx->getServiceContext(), donorStateDoc.getId());
-    invariant(mtab);
+    auto mtabVector = TenantMigrationAccessBlockerRegistry::get(opCtx->getServiceContext())
+                          .getDonorAccessBlockersForMigration(donorStateDoc.getId());
+    invariant(!mtabVector.empty());
 
-    mtab->setCommitOpTime(opCtx, donorStateDoc.getCommitOrAbortOpTime().value());
+    for (auto& mtab : mtabVector) {
+        invariant(mtab);
+
+        mtab->setCommitOpTime(opCtx, donorStateDoc.getCommitOrAbortOpTime().value());
+    }
 }
 
 /**
@@ -214,17 +245,21 @@ void onTransitionToAborted(OperationContext* opCtx, const ShardSplitDonorDocumen
     invariant(donorStateDoc.getState() == ShardSplitDonorStateEnum::kAborted);
     invariant(donorStateDoc.getCommitOrAbortOpTime());
 
-    auto mtab = tenant_migration_access_blocker::getDonorAccessBlockerForMigration(
-        opCtx->getServiceContext(), donorStateDoc.getId());
-    if (!mtab) {
-        // The only case where there can be no tenants is when the instance is created by the
-        // abort command. In that case, no tenant migration blockers are created and the state
-        // will go straight to abort.
-        invariant(donorStateDoc.getState() == ShardSplitDonorStateEnum::kUninitialized);
-        return;
-    }
+    auto mtabVector = TenantMigrationAccessBlockerRegistry::get(opCtx->getServiceContext())
+                          .getDonorAccessBlockersForMigration(donorStateDoc.getId());
+    invariant(!mtabVector.empty());
 
-    mtab->setAbortOpTime(opCtx, donorStateDoc.getCommitOrAbortOpTime().value());
+    for (auto& mtab : mtabVector) {
+        if (!mtab) {
+            // The only case where there can be no tenants is when the instance is created by the
+            // abort command. In that case, no tenant migration blockers are created and the state
+            // will go straight to abort.
+            invariant(donorStateDoc.getState() == ShardSplitDonorStateEnum::kUninitialized);
+            continue;
+        }
+
+        mtab->setAbortOpTime(opCtx, donorStateDoc.getCommitOrAbortOpTime().value());
+    }
 }
 
 /**
@@ -241,14 +276,15 @@ public:
             ServerlessOperationLockRegistry::get(opCtx->getServiceContext())
                 .releaseLock(ServerlessOperationLockRegistry::LockType::kShardSplit,
                              _donorStateDoc.getId());
-            auto mtab = tenant_migration_access_blocker::getDonorAccessBlockerForMigration(
-                opCtx->getServiceContext(), _donorStateDoc.getId());
 
-            if (!mtab) {
+            auto mtabVector = TenantMigrationAccessBlockerRegistry::get(opCtx->getServiceContext())
+                                  .getDonorAccessBlockersForMigration(_donorStateDoc.getId());
+            if (mtabVector.empty()) {
                 // The state doc and TenantMigrationDonorAccessBlocker for this
                 // migration were removed immediately after expireAt was set. This is
                 // unlikely to occur in production where the garbage collection delay
-                // should be sufficiently large.
+                // should be sufficiently large. All acces blockers for the split should be in
+                // the same state.
                 return;
             }
 
@@ -265,11 +301,17 @@ public:
                 // opTime has been majority committed (guaranteed to be true since by
                 // design the donor never marks its state doc as garbage collectable
                 // before the migration decision is majority committed).
-                mtab->onMajorityCommitPointUpdate(_donorStateDoc.getCommitOrAbortOpTime().value());
+                std::for_each(mtabVector.begin(),
+                              mtabVector.end(),
+                              [opTime = _donorStateDoc.getCommitOrAbortOpTime().value()](
+                                  auto& mtab) { mtab->onMajorityCommitPointUpdate(opTime); });
             }
 
             if (_donorStateDoc.getState() == ShardSplitDonorStateEnum::kAborted) {
-                invariant(mtab->inStateAborted());
+                std::for_each(mtabVector.begin(), mtabVector.end(), [](auto& mtab) {
+                    invariant(mtab->inStateAborted());
+                });
+
                 // The migration durably aborted and is now marked as garbage
                 // collectable, remove its TenantMigrationDonorAccessBlocker right away
                 // to allow back-to-back migration retries.
@@ -305,8 +347,10 @@ void ShardSplitDonorOpObserver::onInserts(OperationContext* opCtx,
                                           const CollectionPtr& coll,
                                           std::vector<InsertStatement>::const_iterator first,
                                           std::vector<InsertStatement>::const_iterator last,
+                                          const std::vector<RecordId>& recordIds,
                                           std::vector<bool> fromMigrate,
-                                          bool defaultFromMigrate) {
+                                          bool defaultFromMigrate,
+                                          OpStateAccumulator* opAccumulator) {
     if (coll->ns() != NamespaceString::kShardSplitDonorsNamespace ||
         tenant_migration_access_blocker::inRecoveryMode(opCtx)) {
         return;
@@ -330,7 +374,8 @@ void ShardSplitDonorOpObserver::onInserts(OperationContext* opCtx,
 }
 
 void ShardSplitDonorOpObserver::onUpdate(OperationContext* opCtx,
-                                         const OplogUpdateEntryArgs& args) {
+                                         const OplogUpdateEntryArgs& args,
+                                         OpStateAccumulator* opAccumulator) {
     if (args.coll->ns() != NamespaceString::kShardSplitDonorsNamespace ||
         tenant_migration_access_blocker::inRecoveryMode(opCtx)) {
         return;
@@ -338,12 +383,14 @@ void ShardSplitDonorOpObserver::onUpdate(OperationContext* opCtx,
 
     auto donorStateDoc = parseAndValidateDonorDocument(args.updateArgs->updatedDoc);
     switch (donorStateDoc.getState()) {
+        case ShardSplitDonorStateEnum::kRecipientCaughtUp:
+            break;
         case ShardSplitDonorStateEnum::kBlocking:
             onTransitionToBlocking(opCtx, donorStateDoc);
             break;
         case ShardSplitDonorStateEnum::kCommitted:
         case ShardSplitDonorStateEnum::kAborted:
-            opCtx->recoveryUnit()->registerChange(
+            shard_role_details::getRecoveryUnit(opCtx)->registerChange(
                 std::make_unique<TenantMigrationDonorCommitOrAbortHandler>(donorStateDoc));
             break;
         default:
@@ -355,7 +402,9 @@ void ShardSplitDonorOpObserver::onUpdate(OperationContext* opCtx,
 
 void ShardSplitDonorOpObserver::aboutToDelete(OperationContext* opCtx,
                                               const CollectionPtr& coll,
-                                              BSONObj const& doc) {
+                                              BSONObj const& doc,
+                                              OplogDeleteEntryArgs* args,
+                                              OpStateAccumulator* opAccumulator) {
     if (coll->ns() != NamespaceString::kShardSplitDonorsNamespace ||
         tenant_migration_access_blocker::inRecoveryMode(opCtx)) {
         return;
@@ -388,13 +437,16 @@ void ShardSplitDonorOpObserver::aboutToDelete(OperationContext* opCtx,
 void ShardSplitDonorOpObserver::onDelete(OperationContext* opCtx,
                                          const CollectionPtr& coll,
                                          StmtId stmtId,
-                                         const OplogDeleteEntryArgs& args) {
+                                         const BSONObj& doc,
+                                         const OplogDeleteEntryArgs& args,
+                                         OpStateAccumulator* opAccumulator) {
     if (coll->ns() != NamespaceString::kShardSplitDonorsNamespace || !splitCleanupDetails(opCtx) ||
         tenant_migration_access_blocker::inRecoveryMode(opCtx)) {
         return;
     }
 
-    opCtx->recoveryUnit()->onCommit([](OperationContext* opCtx, boost::optional<Timestamp>) {
+    shard_role_details::getRecoveryUnit(opCtx)->onCommit([](OperationContext* opCtx,
+                                                            boost::optional<Timestamp>) {
         // Donor access blockers are removed from donor nodes via the shard split op observer.
         // Donor access blockers are removed from recipient nodes when the node applies the
         // recipient config. When the recipient primary steps up it will delete its state
@@ -416,15 +468,17 @@ repl::OpTime ShardSplitDonorOpObserver::onDropCollection(OperationContext* opCtx
                                                          const NamespaceString& collectionName,
                                                          const UUID& uuid,
                                                          std::uint64_t numRecords,
-                                                         const CollectionDropType dropType) {
+                                                         const CollectionDropType dropType,
+                                                         bool markFromMigrate) {
     if (collectionName == NamespaceString::kShardSplitDonorsNamespace) {
-        opCtx->recoveryUnit()->onCommit([](OperationContext* opCtx, boost::optional<Timestamp>) {
-            TenantMigrationAccessBlockerRegistry::get(opCtx->getServiceContext())
-                .removeAll(TenantMigrationAccessBlocker::BlockerType::kDonor);
+        shard_role_details::getRecoveryUnit(opCtx)->onCommit(
+            [](OperationContext* opCtx, boost::optional<Timestamp>) {
+                TenantMigrationAccessBlockerRegistry::get(opCtx->getServiceContext())
+                    .removeAll(TenantMigrationAccessBlocker::BlockerType::kDonor);
 
-            ServerlessOperationLockRegistry::get(opCtx->getServiceContext())
-                .onDropStateCollection(ServerlessOperationLockRegistry::LockType::kShardSplit);
-        });
+                ServerlessOperationLockRegistry::get(opCtx->getServiceContext())
+                    .onDropStateCollection(ServerlessOperationLockRegistry::LockType::kShardSplit);
+            });
     }
 
     return {};
