@@ -27,25 +27,55 @@
  *    it in the license file.
  */
 
+#include <functional>
+#include <memory>
+#include <string>
 #include <utility>
 
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/oid.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/client/connection_string.h"
+#include "mongo/client/mongo_uri.h"
 #include "mongo/db/catalog/create_collection.h"
 #include "mongo/db/catalog_raii.h"
+#include "mongo/db/client.h"
+#include "mongo/db/commands/create_gen.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
+#include "mongo/db/repl/member_state.h"
+#include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/repl/replication_coordinator_mock.h"
+#include "mongo/db/repl/storage_interface.h"
 #include "mongo/db/repl/storage_interface_mock.h"
-#include "mongo/db/repl/tenant_migration_access_blocker_util.h"
+#include "mongo/db/repl/tenant_migration_access_blocker.h"
+#include "mongo/db/repl/tenant_migration_access_blocker_registry.h"
+#include "mongo/db/repl/tenant_migration_donor_access_blocker.h"
 #include "mongo/db/serverless/serverless_operation_lock_registry.h"
 #include "mongo/db/serverless/shard_split_donor_op_observer.h"
 #include "mongo/db/serverless/shard_split_state_machine_gen.h"
 #include "mongo/db/serverless/shard_split_test_utils.h"
 #include "mongo/db/serverless/shard_split_utils.h"
 #include "mongo/db/service_context_d_test_fixture.h"
+#include "mongo/db/storage/write_unit_of_work.h"
+#include "mongo/db/tenant_id.h"
 #include "mongo/dbtests/mock/mock_replica_set.h"
+#include "mongo/idl/idl_parser.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/time_support.h"
 
 namespace mongo {
 namespace {
 
-
 class ShardSplitDonorOpObserverTest : public ServiceContextMongoDTest {
 public:
     void setUp() override {
@@ -57,8 +87,8 @@ public:
             repl::StorageInterface::set(service, std::make_unique<repl::StorageInterfaceMock>());
 
             // Set up ReplicationCoordinator and create oplog.
-            auto coordinatorMock =
-                std::make_unique<repl::ReplicationCoordinatorMock>(service, createReplSettings());
+            auto coordinatorMock = std::make_unique<repl::ReplicationCoordinatorMock>(
+                service, repl::createServerlessReplSettings());
             _replicationCoordinatorMock = coordinatorMock.get();
 
             repl::ReplicationCoordinator::set(service, std::move(coordinatorMock));
@@ -99,6 +129,7 @@ protected:
                                  *autoColl,
                                  inserts.begin(),
                                  inserts.end(),
+                                 /*recordIds*/ {},
                                  /*fromMigrate=*/std::vector<bool>(inserts.size(), false),
                                  /*defaultFromMigrate=*/false);
             wow.commit();
@@ -114,7 +145,7 @@ protected:
 
         // If there's an exception, aborting without removing the access blocker will trigger an
         // invariant. This creates a confusing error log in the test output.
-        test::shard_split::ScopedTenantAccessBlocker scopedTenants(_tenantIds, _opCtx.get());
+        test::shard_split::ScopedTenantAccessBlocker scopedTenants(_uuid, _opCtx.get());
 
         const auto criteria = BSON("_id" << stateDocument.getId());
         auto preImageDoc = defaultStateDocument();
@@ -139,20 +170,27 @@ protected:
         scopedTenants.dismiss();
     }
 
-    std::shared_ptr<TenantMigrationDonorAccessBlocker> createAccessBlockerAndStartBlockingWrites(
-        const UUID& migrationId,
-        const std::vector<TenantId>& tenants,
-        OperationContext* opCtx,
-        bool isSecondary = false) {
-        auto mtab = std::make_shared<TenantMigrationDonorAccessBlocker>(_opCtx->getServiceContext(),
-                                                                        migrationId);
+    std::vector<std::shared_ptr<TenantMigrationDonorAccessBlocker>>
+    createAccessBlockerAndStartBlockingWrites(const UUID& migrationId,
+                                              const std::vector<TenantId>& tenants,
+                                              OperationContext* opCtx,
+                                              bool isSecondary = false) {
 
-        if (!isSecondary) {
-            mtab->startBlockingWrites();
+        std::vector<std::shared_ptr<TenantMigrationDonorAccessBlocker>> result;
+        for (const auto& tenantId : tenants) {
+            auto mtab = std::make_shared<TenantMigrationDonorAccessBlocker>(
+                _opCtx->getServiceContext(), migrationId);
+
+            if (!isSecondary) {
+                mtab->startBlockingWrites();
+            }
+
+            TenantMigrationAccessBlockerRegistry::get(opCtx->getServiceContext())
+                .add(tenantId, mtab);
+            result.push_back(mtab);
         }
 
-        TenantMigrationAccessBlockerRegistry::get(opCtx->getServiceContext()).add(tenants, mtab);
-        return mtab;
+        return result;
     }
 
     ShardSplitDonorDocument defaultStateDocument() const {
@@ -191,18 +229,9 @@ private:
             mtabVerifier(mtab);
         }
 
-        for (const auto& tenantId : tenants) {
-            TenantMigrationAccessBlockerRegistry::get(_opCtx->getServiceContext())
-                .remove(tenantId, TenantMigrationAccessBlocker::BlockerType::kDonor);
-        }
-    }
-    // Creates a reasonable set of ReplSettings for most tests.  We need to be able to
-    // override this to create a larger oplog.
-    virtual repl::ReplSettings createReplSettings() {
-        repl::ReplSettings settings;
-        settings.setOplogSizeBytes(5 * 1024 * 1024);
-        settings.setReplSetString("mySet/node1:12345");
-        return settings;
+        TenantMigrationAccessBlockerRegistry::get(_opCtx->getServiceContext())
+            .removeAccessBlockersForMigration(_uuid,
+                                              TenantMigrationAccessBlocker::BlockerType::kDonor);
     }
 };
 
@@ -221,6 +250,7 @@ TEST_F(ShardSplitDonorOpObserverTest, InsertWrongType) {
                              *autoColl,
                              inserts1.begin(),
                              inserts1.end(),
+                             /*recordIds*/ {},
                              /*fromMigrate=*/std::vector<bool>(inserts1.size(), false),
                              /*defaultFromMigrate=*/false),
         DBException,
@@ -265,6 +295,7 @@ TEST_F(ShardSplitDonorOpObserverTest, InsertValidAbortedDocument) {
                              *autoColl,
                              inserts.begin(),
                              inserts.end(),
+                             /*recordIds*/ {},
                              /*fromMigrate=*/std::vector<bool>(inserts.size(), false),
                              /*defaultFromMigrate=*/false);
         wow.commit();
@@ -386,8 +417,10 @@ TEST_F(ShardSplitDonorOpObserverTest, TransitionToCommit) {
     stateDocument.setBlockOpTime(repl::OpTime(Timestamp(1, 2), 1));
     stateDocument.setCommitOrAbortOpTime(commitOpTime);
 
-    auto mtab = createAccessBlockerAndStartBlockingWrites(_uuid, _tenantIds, _opCtx.get());
-    mtab->startBlockingReadsAfter(Timestamp(1));
+    auto mtabVector = createAccessBlockerAndStartBlockingWrites(_uuid, _tenantIds, _opCtx.get());
+    for (auto& mtab : mtabVector) {
+        mtab->startBlockingReadsAfter(Timestamp(1));
+    }
 
     auto mtabVerifier = [opCtx = _opCtx.get()](std::shared_ptr<TenantMigrationAccessBlocker> mtab) {
         ASSERT_TRUE(mtab);
@@ -418,8 +451,10 @@ TEST_F(ShardSplitDonorOpObserverTest, TransitionToAbort) {
     stateDocument.setCommitOrAbortOpTime(abortOpTime);
     stateDocument.setAbortReason(bob.obj());
 
-    auto mtab = createAccessBlockerAndStartBlockingWrites(_uuid, _tenantIds, _opCtx.get());
-    mtab->startBlockingReadsAfter(Timestamp(1));
+    auto mtabVector = createAccessBlockerAndStartBlockingWrites(_uuid, _tenantIds, _opCtx.get());
+    for (auto& mtab : mtabVector) {
+        mtab->startBlockingReadsAfter(Timestamp(1));
+    }
 
     auto mtabVerifier = [opCtx = _opCtx.get()](std::shared_ptr<TenantMigrationAccessBlocker> mtab) {
         ASSERT_TRUE(mtab);
@@ -450,9 +485,11 @@ TEST_F(ShardSplitDonorOpObserverTest, SetExpireAtForAbortedRemoveBlockers) {
     stateDocument.setAbortReason(bob.obj());
     stateDocument.setExpireAt(mongo::Date_t::fromMillisSinceEpoch(1000));
 
-    auto mtab = createAccessBlockerAndStartBlockingWrites(_uuid, _tenantIds, _opCtx.get());
-    mtab->startBlockingReadsAfter(Timestamp(1));
-    mtab->setAbortOpTime(_opCtx.get(), *stateDocument.getCommitOrAbortOpTime());
+    auto mtabVector = createAccessBlockerAndStartBlockingWrites(_uuid, _tenantIds, _opCtx.get());
+    for (auto& mtab : mtabVector) {
+        mtab->startBlockingReadsAfter(Timestamp(1));
+        mtab->setAbortOpTime(_opCtx.get(), *stateDocument.getCommitOrAbortOpTime());
+    }
 
     auto mtabVerifier = [opCtx = _opCtx.get()](std::shared_ptr<TenantMigrationAccessBlocker> mtab) {
         ASSERT_FALSE(mtab);
@@ -483,20 +520,20 @@ TEST_F(ShardSplitDonorOpObserverTest, DeleteAbortedDocumentDoesNotRemoveBlockers
     stateDocument.setAbortReason(bob.obj());
     stateDocument.setExpireAt(mongo::Date_t::fromMillisSinceEpoch(1000));
 
-    auto mtab = createAccessBlockerAndStartBlockingWrites(_uuid, _tenantIds, _opCtx.get());
-    mtab->startBlockingReadsAfter(Timestamp(1));
-    mtab->setAbortOpTime(_opCtx.get(), *stateDocument.getCommitOrAbortOpTime());
+    auto mtabVector = createAccessBlockerAndStartBlockingWrites(_uuid, _tenantIds, _opCtx.get());
+    for (auto& mtab : mtabVector) {
+        mtab->startBlockingReadsAfter(Timestamp(1));
+        mtab->setAbortOpTime(_opCtx.get(), *stateDocument.getCommitOrAbortOpTime());
+    }
 
     auto bsonDoc = stateDocument.toBSON();
 
     WriteUnitOfWork wuow(_opCtx.get());
     AutoGetCollection autoColl(_opCtx.get(), NamespaceString::kShardSplitDonorsNamespace, MODE_IX);
-    _observer->aboutToDelete(_opCtx.get(), *autoColl, bsonDoc);
-
     OplogDeleteEntryArgs deleteArgs;
-    deleteArgs.deletedDoc = &bsonDoc;
+    _observer->aboutToDelete(_opCtx.get(), *autoColl, bsonDoc, &deleteArgs);
 
-    _observer->onDelete(_opCtx.get(), *autoColl, 0 /* stmtId */, deleteArgs);
+    _observer->onDelete(_opCtx.get(), *autoColl, 0 /*stmtId=*/, bsonDoc, deleteArgs);
     wuow.commit();
 
     // Verify blockers have not been removed
@@ -518,9 +555,11 @@ TEST_F(ShardSplitDonorOpObserverTest, DeleteCommittedDocumentRemovesBlockers) {
     stateDocument.setCommitOrAbortOpTime(commitOpTime);
     stateDocument.setExpireAt(mongo::Date_t::fromMillisSinceEpoch(1000));
 
-    auto mtab = createAccessBlockerAndStartBlockingWrites(_uuid, _tenantIds, _opCtx.get());
-    mtab->startBlockingReadsAfter(Timestamp(1));
-    mtab->setCommitOpTime(_opCtx.get(), *stateDocument.getCommitOrAbortOpTime());
+    auto mtabVector = createAccessBlockerAndStartBlockingWrites(_uuid, _tenantIds, _opCtx.get());
+    for (auto& mtab : mtabVector) {
+        mtab->startBlockingReadsAfter(Timestamp(1));
+        mtab->setCommitOpTime(_opCtx.get(), *stateDocument.getCommitOrAbortOpTime());
+    }
 
     ServerlessOperationLockRegistry::get(_opCtx->getServiceContext())
         .acquireLock(ServerlessOperationLockRegistry::LockType::kShardSplit, stateDocument.getId());
@@ -529,12 +568,10 @@ TEST_F(ShardSplitDonorOpObserverTest, DeleteCommittedDocumentRemovesBlockers) {
 
     WriteUnitOfWork wuow(_opCtx.get());
     AutoGetCollection autoColl(_opCtx.get(), NamespaceString::kShardSplitDonorsNamespace, MODE_IX);
-    _observer->aboutToDelete(_opCtx.get(), *autoColl, bsonDoc);
-
     OplogDeleteEntryArgs deleteArgs;
-    deleteArgs.deletedDoc = &bsonDoc;
+    _observer->aboutToDelete(_opCtx.get(), *autoColl, bsonDoc, &deleteArgs);
 
-    _observer->onDelete(_opCtx.get(), *autoColl, 0 /* stmtId */, deleteArgs);
+    _observer->onDelete(_opCtx.get(), *autoColl, 0 /*stmtId=*/, bsonDoc, deleteArgs);
     wuow.commit();
 
     // Verify blockers have been removed
