@@ -30,29 +30,79 @@
 
 #include "mongo/db/serverless/shard_split_donor_service.h"
 
-#include "mongo/client/streamable_replica_set_monitor.h"
+// IWYU pragma: no_include "ext/alloc_traits.h"
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr.hpp>
+#include <string>
+#include <tuple>
+
+#include "mongo/base/checked_cast.h"
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status_with.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/client/mongo_uri.h"
+#include "mongo/client/replica_set_monitor_stats.h"
+#include "mongo/client/sdam/sdam_configuration.h"
+#include "mongo/client/sdam/topology_listener.h"
+#include "mongo/client/server_discovery_monitor.h"
+#include "mongo/db/catalog/collection.h"
 #include "mongo/db/catalog/collection_write_path.h"
-#include "mongo/db/catalog_raii.h"
+#include "mongo/db/catalog/local_oplog_info.h"
+#include "mongo/db/client.h"
 #include "mongo/db/concurrency/exception_util.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
+#include "mongo/db/database_name.h"
 #include "mongo/db/dbdirectclient.h"
 #include "mongo/db/dbhelpers.h"
 #include "mongo/db/index_builds_coordinator.h"
+#include "mongo/db/ops/update_result.h"
+#include "mongo/db/record_id.h"
+#include "mongo/db/repl/oplog.h"
+#include "mongo/db/repl/read_concern_args.h"
 #include "mongo/db/repl/repl_client_info.h"
-#include "mongo/db/repl/tenant_migration_access_blocker_util.h"
+#include "mongo/db/repl/repl_server_parameters_gen.h"
+#include "mongo/db/repl/repl_set_config.h"
+#include "mongo/db/repl/replication_coordinator.h"
+#include "mongo/db/repl/tenant_migration_access_blocker_registry.h"
+#include "mongo/db/repl/tenant_migration_donor_access_blocker.h"
 #include "mongo/db/repl/wait_for_majority_service.h"
 #include "mongo/db/s/resharding/resharding_util.h"
+#include "mongo/db/server_options.h"
+#include "mongo/db/serverless/serverless_types_gen.h"
 #include "mongo/db/serverless/shard_split_statistics.h"
 #include "mongo/db/serverless/shard_split_utils.h"
-#include "mongo/executor/cancelable_executor.h"
-#include "mongo/executor/connection_pool.h"
-#include "mongo/executor/network_interface_factory.h"
-#include "mongo/executor/network_interface_thread_pool.h"
-#include "mongo/executor/thread_pool_task_executor.h"
+#include "mongo/db/shard_role.h"
+#include "mongo/db/storage/recovery_unit.h"
+#include "mongo/db/storage/snapshot.h"
+#include "mongo/db/storage/write_unit_of_work.h"
+#include "mongo/db/tenant_id.h"
+#include "mongo/db/transaction_resources.h"
+#include "mongo/db/write_concern_options.h"
+#include "mongo/executor/remote_command_request.h"
+#include "mongo/idl/idl_parser.h"
 #include "mongo/logv2/log.h"
-#include "mongo/rpc/metadata/egress_metadata_hook_list.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/rpc/get_status_from_command_result.h"
+#include "mongo/s/database_version.h"
+#include "mongo/s/shard_version.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/clock_source.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/duration.h"
 #include "mongo/util/fail_point.h"
 #include "mongo/util/future_util.h"
+#include "mongo/util/out_of_line_executor.h"
+#include "mongo/util/scopeguard.h"
+#include "mongo/util/str.h"
 #include "mongo/util/time_support.h"
+#include "mongo/util/timer.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTenantMigration
 
@@ -64,6 +114,7 @@ namespace {
 MONGO_FAIL_POINT_DEFINE(abortShardSplitBeforeLeavingBlockingState);
 MONGO_FAIL_POINT_DEFINE(pauseShardSplitBeforeBlockingState);
 MONGO_FAIL_POINT_DEFINE(pauseShardSplitAfterBlocking);
+MONGO_FAIL_POINT_DEFINE(pauseShardSplitAfterRecipientCaughtUp);
 MONGO_FAIL_POINT_DEFINE(pauseShardSplitAfterDecision);
 MONGO_FAIL_POINT_DEFINE(skipShardSplitGarbageCollectionTimeout);
 MONGO_FAIL_POINT_DEFINE(skipShardSplitWaitForSplitAcceptance);
@@ -345,6 +396,8 @@ SemiFuture<void> ShardSplitDonorService::DonorStateMachine::run(
                 return _waitForRecipientToReachBlockOpTime(executor, abortToken);
             })
             .then([this, executor, abortToken, criticalSectionWithoutCatchupTimer] {
+                auto opCtx = _cancelableOpCtxFactory->makeOperationContext(&cc());
+                pauseShardSplitAfterRecipientCaughtUp.pauseWhileSet(opCtx.get());
                 criticalSectionWithoutCatchupTimer->reset();
                 return _applySplitConfigToDonor(executor, abortToken);
             })
@@ -484,10 +537,10 @@ boost::optional<BSONObj> ShardSplitDonorService::DonorStateMachine::reportForCur
         bob.append("tenantIds", tenantIdsAsStrings);
     }
     if (_stateDoc.getBlockOpTime()) {
-        _stateDoc.getBlockOpTime()->append(&bob, "blockOpTime");
+        _stateDoc.getBlockOpTime()->append("blockOpTime", &bob);
     }
     if (_stateDoc.getCommitOrAbortOpTime()) {
-        _stateDoc.getCommitOrAbortOpTime()->append(&bob, "commitOrAbortOpTime");
+        _stateDoc.getCommitOrAbortOpTime()->append("commitOrAbortOpTime", &bob);
     }
     if (_stateDoc.getAbortReason()) {
         bob.append("abortReason", *_stateDoc.getAbortReason());
@@ -533,7 +586,7 @@ ConnectionString ShardSplitDonorService::DonorStateMachine::_setupAcceptanceMoni
 
     // Always start the replica set monitor if we haven't reached a decision yet
     _splitAcceptancePromise.setWith([&]() {
-        if (_stateDoc.getState() > ShardSplitDonorStateEnum::kBlocking ||
+        if (_stateDoc.getState() > ShardSplitDonorStateEnum::kRecipientCaughtUp ||
             MONGO_unlikely(skipShardSplitWaitForSplitAcceptance.shouldFail())) {
             return Future<HostAndPort>::makeReady(StatusWith<HostAndPort>(HostAndPort{}));
         }
@@ -661,7 +714,7 @@ ExecutorFuture<void> ShardSplitDonorService::DonorStateMachine::_waitForRecipien
     checkForTokenInterrupt(abortToken);
 
     stdx::lock_guard<Latch> lg(_mutex);
-    if (_stateDoc.getState() > ShardSplitDonorStateEnum::kBlocking ||
+    if (_stateDoc.getState() >= ShardSplitDonorStateEnum::kRecipientCaughtUp ||
         _hasInstalledSplitConfig(lg)) {
         return ExecutorFuture(**executor);
     }
@@ -686,11 +739,27 @@ ExecutorFuture<void> ShardSplitDonorService::DonorStateMachine::_waitForRecipien
     LOGV2(
         6177201, "Waiting for recipient nodes to reach block timestamp.", "id"_attr = _migrationId);
 
-    return ExecutorFuture(**executor).then([this, blockOpTime, writeConcern]() {
-        auto opCtx = _cancelableOpCtxFactory->makeOperationContext(&cc());
-        auto replCoord = repl::ReplicationCoordinator::get(cc().getServiceContext());
-        uassertStatusOK(replCoord->awaitReplication(opCtx.get(), blockOpTime, writeConcern).status);
-    });
+    return ExecutorFuture(**executor)
+        .then([this, blockOpTime, writeConcern]() {
+            auto opCtx = _cancelableOpCtxFactory->makeOperationContext(&cc());
+            auto replCoord = repl::ReplicationCoordinator::get(cc().getServiceContext());
+            uassertStatusOK(
+                replCoord->awaitReplication(opCtx.get(), blockOpTime, writeConcern).status);
+        })
+        .then([this, executor, abortToken]() {
+            {
+                stdx::lock_guard<Latch> lg(_mutex);
+                LOGV2(8423389,
+                      "Entering 'recipient caught up' state.",
+                      "id"_attr = _stateDoc.getId());
+            }
+
+            return _updateStateDocument(
+                       executor, abortToken, ShardSplitDonorStateEnum::kRecipientCaughtUp)
+                .then([this, self = shared_from_this(), executor, abortToken](repl::OpTime opTime) {
+                    return _waitForMajorityWriteConcern(executor, std::move(opTime), abortToken);
+                });
+        });
 }
 
 ExecutorFuture<void> ShardSplitDonorService::DonorStateMachine::_applySplitConfigToDonor(
@@ -745,7 +814,8 @@ ExecutorFuture<void> remoteAdminCommand(TaskExecutorPtr executor,
                                         const HostAndPort remoteNode,
                                         const BSONObj& command) {
     return AsyncTry([executor, token, remoteNode, command] {
-               executor::RemoteCommandRequest request(remoteNode, "admin", command, nullptr);
+               executor::RemoteCommandRequest request(
+                   remoteNode, DatabaseName::kAdmin, command, nullptr);
                auto hasWriteConcern = command.hasField(WriteConcernOptions::kWriteConcernField);
 
                return executor->scheduleRemoteCommand(request, token)
@@ -791,7 +861,7 @@ ShardSplitDonorService::DonorStateMachine::_waitForSplitAcceptanceAndEnterCommit
     checkForTokenInterrupt(abortToken);
     {
         stdx::lock_guard<Latch> lg(_mutex);
-        if (_stateDoc.getState() > ShardSplitDonorStateEnum::kBlocking) {
+        if (_stateDoc.getState() > ShardSplitDonorStateEnum::kRecipientCaughtUp) {
             return ExecutorFuture(**executor);
         }
     }
@@ -882,122 +952,132 @@ ExecutorFuture<repl::OpTime> ShardSplitDonorService::DonorStateMachine::_updateS
                auto opCtxHolder = _cancelableOpCtxFactory->makeOperationContext(&cc());
                auto opCtx = opCtxHolder.get();
 
-               AutoGetCollection collection(opCtx, _stateDocumentsNS, MODE_IX);
+               auto collection =
+                   acquireCollection(opCtx,
+                                     CollectionAcquisitionRequest(
+                                         _stateDocumentsNS,
+                                         PlacementConcern{boost::none, ShardVersion::UNSHARDED()},
+                                         repl::ReadConcernArgs::get(opCtx),
+                                         AcquisitionPrerequisites::kWrite),
+                                     MODE_IX);
 
                if (!isInsert) {
                    uassert(ErrorCodes::NamespaceNotFound,
-                           str::stream() << _stateDocumentsNS.ns() << " does not exist",
-                           collection);
+                           str::stream()
+                               << _stateDocumentsNS.toStringForErrorMsg() << " does not exist",
+                           collection.exists());
                }
 
-               writeConflictRetry(
-                   opCtx, "ShardSplitDonorUpdateStateDoc", _stateDocumentsNS.ns(), [&]() {
-                       WriteUnitOfWork wuow(opCtx);
+               writeConflictRetry(opCtx, "ShardSplitDonorUpdateStateDoc", _stateDocumentsNS, [&]() {
+                   WriteUnitOfWork wuow(opCtx);
 
-                       if (nextState == ShardSplitDonorStateEnum::kBlocking) {
-                           // Start blocking writes before getting an oplog slot to guarantee no
-                           // writes to the tenant's data can commit with a timestamp after the
-                           // block timestamp.
-                           auto mtab =
-                               tenant_migration_access_blocker::getDonorAccessBlockerForMigration(
-                                   _serviceContext, uuid);
+                   if (nextState == ShardSplitDonorStateEnum::kBlocking) {
+                       // Start blocking writes before getting an oplog slot to guarantee no
+                       // writes to the tenant's data can commit with a timestamp after the
+                       // block timestamp.
+                       auto mtabVector =
+                           TenantMigrationAccessBlockerRegistry::get(opCtx->getServiceContext())
+                               .getDonorAccessBlockersForMigration(uuid);
+                       invariant(!mtabVector.empty());
+
+                       for (auto& mtab : mtabVector) {
                            invariant(mtab);
                            mtab->startBlockingWrites();
 
-                           opCtx->recoveryUnit()->onRollback(
+                           shard_role_details::getRecoveryUnit(opCtx)->onRollback(
                                [mtab](OperationContext*) { mtab->rollBackStartBlocking(); });
                        }
+                   }
 
-                       // Reserve an opTime for the write.
-                       auto oplogSlot = LocalOplogInfo::get(opCtx)->getNextOpTimes(opCtx, 1U)[0];
-                       auto updatedStateDocBson = [&]() {
-                           stdx::lock_guard<Latch> lg(_mutex);
-                           _stateDoc.setState(nextState);
-                           switch (nextState) {
-                               case ShardSplitDonorStateEnum::kUninitialized:
-                               case ShardSplitDonorStateEnum::kAbortingIndexBuilds:
-                                   break;
-                               case ShardSplitDonorStateEnum::kBlocking:
-                                   _stateDoc.setBlockOpTime(oplogSlot);
-                                   break;
-                               case ShardSplitDonorStateEnum::kCommitted:
-                                   _stateDoc.setCommitOrAbortOpTime(oplogSlot);
-                                   break;
-                               case ShardSplitDonorStateEnum::kAborted: {
-                                   _stateDoc.setCommitOrAbortOpTime(oplogSlot);
+                   // Reserve an opTime for the write.
+                   auto oplogSlot = LocalOplogInfo::get(opCtx)->getNextOpTimes(opCtx, 1U)[0];
+                   auto updatedStateDocBson = [&]() {
+                       stdx::lock_guard<Latch> lg(_mutex);
+                       _stateDoc.setState(nextState);
+                       switch (nextState) {
+                           case ShardSplitDonorStateEnum::kUninitialized:
+                           case ShardSplitDonorStateEnum::kAbortingIndexBuilds:
+                           case ShardSplitDonorStateEnum::kRecipientCaughtUp:
+                               break;
+                           case ShardSplitDonorStateEnum::kBlocking:
+                               _stateDoc.setBlockOpTime(oplogSlot);
+                               break;
+                           case ShardSplitDonorStateEnum::kCommitted:
+                               _stateDoc.setCommitOrAbortOpTime(oplogSlot);
+                               break;
+                           case ShardSplitDonorStateEnum::kAborted: {
+                               _stateDoc.setCommitOrAbortOpTime(oplogSlot);
 
-                                   invariant(_abortReason);
-                                   BSONObjBuilder bob;
-                                   _abortReason.value().serializeErrorToBSON(&bob);
-                                   _stateDoc.setAbortReason(bob.obj());
-                                   break;
-                               }
-                               default:
-                                   MONGO_UNREACHABLE;
-                           }
-                           if (isInsert) {
-                               return BSON("$setOnInsert" << _stateDoc.toBSON());
+                               invariant(_abortReason);
+                               BSONObjBuilder bob;
+                               _abortReason.value().serializeErrorToBSON(&bob);
+                               _stateDoc.setAbortReason(bob.obj());
+                               break;
                            }
+                           default:
+                               MONGO_UNREACHABLE;
+                       }
+                       if (isInsert) {
+                           return BSON("$setOnInsert" << _stateDoc.toBSON());
+                       }
 
-                           return _stateDoc.toBSON();
-                       }();
+                       return _stateDoc.toBSON();
+                   }();
 
-                       auto updateOpTime = [&]() {
-                           if (isInsert) {
-                               const auto filter =
-                                   BSON(ShardSplitDonorDocument::kIdFieldName << uuid);
-                               auto updateResult = Helpers::upsert(opCtx,
-                                                                   _stateDocumentsNS,
-                                                                   filter,
-                                                                   updatedStateDocBson,
-                                                                   /*fromMigrate=*/false);
+                   auto updateOpTime = [&]() {
+                       if (isInsert) {
+                           const auto filter = BSON(ShardSplitDonorDocument::kIdFieldName << uuid);
+                           auto updateResult = Helpers::upsert(opCtx,
+                                                               collection,
+                                                               filter,
+                                                               updatedStateDocBson,
+                                                               /*fromMigrate=*/false);
 
-                               // '$setOnInsert' update operator can never modify an existing
-                               // on-disk state doc.
-                               invariant(!updateResult.existing);
-                               invariant(!updateResult.numDocsModified);
+                           // '$setOnInsert' update operator can never modify an existing
+                           // on-disk state doc.
+                           invariant(!updateResult.existing);
+                           invariant(!updateResult.numDocsModified);
 
-                               return repl::ReplClientInfo::forClient(opCtx->getClient())
-                                   .getLastOp();
-                           }
+                           return repl::ReplClientInfo::forClient(opCtx->getClient()).getLastOp();
+                       }
 
-                           const auto originalRecordId =
-                               Helpers::findOne(opCtx,
-                                                collection.getCollection(),
-                                                BSON("_id" << originalStateDocBson["_id"]));
-                           const auto originalSnapshot = Snapshotted<BSONObj>(
-                               opCtx->recoveryUnit()->getSnapshotId(), originalStateDocBson);
-                           invariant(!originalRecordId.isNull());
+                       const auto originalRecordId =
+                           Helpers::findOne(opCtx,
+                                            collection.getCollectionPtr(),
+                                            BSON("_id" << originalStateDocBson["_id"]));
+                       const auto originalSnapshot = Snapshotted<BSONObj>(
+                           shard_role_details::getRecoveryUnit(opCtx)->getSnapshotId(),
+                           originalStateDocBson);
+                       invariant(!originalRecordId.isNull());
 
-                           CollectionUpdateArgs args{originalSnapshot.value()};
-                           args.criteria = BSON("_id" << uuid);
-                           args.oplogSlots = {oplogSlot};
-                           args.update = updatedStateDocBson;
+                       CollectionUpdateArgs args{originalSnapshot.value()};
+                       args.criteria = BSON("_id" << uuid);
+                       args.oplogSlots = {oplogSlot};
+                       args.update = updatedStateDocBson;
 
-                           collection_internal::updateDocument(
-                               opCtx,
-                               *collection,
-                               originalRecordId,
-                               originalSnapshot,
-                               updatedStateDocBson,
-                               collection_internal::kUpdateNoIndexes,
-                               nullptr /* indexesAffected */,
-                               nullptr /* OpDebug* */,
-                               &args);
+                       collection_internal::updateDocument(opCtx,
+                                                           collection.getCollectionPtr(),
+                                                           originalRecordId,
+                                                           originalSnapshot,
+                                                           updatedStateDocBson,
+                                                           collection_internal::kUpdateNoIndexes,
+                                                           nullptr /* indexesAffected */,
+                                                           nullptr /* OpDebug* */,
+                                                           &args);
 
-                           return oplogSlot;
-                       }();
+                       return oplogSlot;
+                   }();
 
-                       wuow.commit();
-                       return updateOpTime;
-                   });
+                   wuow.commit();
+                   return updateOpTime;
+               });
 
                return repl::ReplClientInfo::forClient(opCtx->getClient()).getLastOp();
            })
         .until([&](StatusWith<repl::OpTime> swOpTime) {
             if (swOpTime.getStatus().code() == ErrorCodes::ConflictingServerlessOperation) {
                 LOGV2(6531509,
-                      "Shard split completed due to serverless lock error",
+                      "Shard split failed due to serverless lock error",
                       "id"_attr = _migrationId,
                       "status"_attr = swOpTime.getStatus());
                 stdx::lock_guard<Latch> lg(_mutex);
@@ -1013,7 +1093,7 @@ ExecutorFuture<repl::OpTime> ShardSplitDonorService::DonorStateMachine::_updateS
 ExecutorFuture<void> ShardSplitDonorService::DonorStateMachine::_waitForMajorityWriteConcern(
     const ScopedTaskExecutorPtr& executor, repl::OpTime opTime, const CancellationToken& token) {
     return WaitForMajorityService::get(_serviceContext)
-        .waitUntilMajority(std::move(opTime), token)
+        .waitUntilMajorityForWrite(std::move(opTime), token)
         .thenRunOn(**executor);
 }
 
