@@ -28,49 +28,79 @@
  */
 
 
+#include <algorithm>
+#include <boost/none.hpp>
+#include <cstddef>
+#include <functional>
+#include <list>
 #include <memory>
+#include <ostream>
+#include <string>
+#include <type_traits>
 
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status_with.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/oid.h"
+#include "mongo/bson/timestamp.h"
 #include "mongo/client/connection_string.h"
-#include "mongo/client/replica_set_monitor.h"
-#include "mongo/client/sdam/server_description_builder.h"
-#include "mongo/client/streamable_replica_set_monitor_for_testing.h"
-#include "mongo/db/catalog/database_holder_mock.h"
+#include "mongo/client/mongo_uri.h"
+#include "mongo/db/catalog_raii.h"
+#include "mongo/db/client.h"
+#include "mongo/db/commands.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
+#include "mongo/db/database_name.h"
 #include "mongo/db/db_raii.h"
 #include "mongo/db/dbhelpers.h"
-#include "mongo/db/op_observer/op_observer_impl.h"
+#include "mongo/db/op_observer/op_observer.h"
 #include "mongo/db/op_observer/op_observer_registry.h"
-#include "mongo/db/repl/drop_pending_collection_reaper.h"
 #include "mongo/db/repl/primary_only_service.h"
-#include "mongo/db/repl/primary_only_service_op_observer.h"
 #include "mongo/db/repl/primary_only_service_test_fixture.h"
+#include "mongo/db/repl/repl_server_parameters_gen.h"
+#include "mongo/db/repl/repl_set_config.h"
+#include "mongo/db/repl/repl_settings.h"
+#include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/repl/replication_coordinator_mock.h"
-#include "mongo/db/repl/storage_interface_impl.h"
-#include "mongo/db/repl/tenant_migration_access_blocker_registry.h"
 #include "mongo/db/repl/tenant_migration_access_blocker_util.h"
-#include "mongo/db/repl/tenant_migration_donor_access_blocker.h"
-#include "mongo/db/repl/wait_for_majority_service.h"
 #include "mongo/db/serverless/serverless_operation_lock_registry.h"
 #include "mongo/db/serverless/shard_split_donor_op_observer.h"
 #include "mongo/db/serverless/shard_split_donor_service.h"
 #include "mongo/db/serverless/shard_split_state_machine_gen.h"
 #include "mongo/db/serverless/shard_split_test_utils.h"
 #include "mongo/db/serverless/shard_split_utils.h"
-#include "mongo/db/service_context_d_test_fixture.h"
-#include "mongo/dbtests/mock/mock_conn_registry.h"
+#include "mongo/db/storage/recovery_unit.h"
+#include "mongo/db/tenant_id.h"
+#include "mongo/db/write_concern_options.h"
+#include "mongo/dbtests/mock/mock_remote_db_server.h"
 #include "mongo/dbtests/mock/mock_replica_set.h"
-#include "mongo/executor/network_interface.h"
-#include "mongo/executor/network_interface_factory.h"
 #include "mongo/executor/network_interface_mock.h"
-#include "mongo/executor/network_interface_thread_pool.h"
+#include "mongo/executor/remote_command_request.h"
+#include "mongo/executor/remote_command_response.h"
 #include "mongo/executor/thread_pool_mock.h"
 #include "mongo/executor/thread_pool_task_executor.h"
-#include "mongo/executor/thread_pool_task_executor_test_fixture.h"
+#include "mongo/idl/idl_parser.h"
 #include "mongo/idl/server_parameter_test_util.h"
-#include "mongo/logv2/log.h"
-#include "mongo/rpc/metadata/egress_metadata_hook_list.h"
-#include "mongo/unittest/log_test.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/rpc/op_msg.h"
+#include "mongo/rpc/reply_builder_interface.h"
+#include "mongo/rpc/reply_interface.h"
+#include "mongo/rpc/unique_message.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/bson_test_util.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/clock_source.h"
 #include "mongo/util/clock_source_mock.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/duration.h"
+#include "mongo/util/fail_point.h"
+#include "mongo/util/str.h"
+#include "mongo/util/time_support.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
 
@@ -87,6 +117,7 @@ namespace mongo {
  * Returns 'NoMatchingDocument' error code if no document with 'shardSplitId' is found.
  */
 namespace {
+
 StatusWith<ShardSplitDonorDocument> getStateDocument(OperationContext* opCtx,
                                                      const UUID& shardSplitId) {
     // Use kLastApplied so that we can read the state document as a secondary.
@@ -95,7 +126,7 @@ StatusWith<ShardSplitDonorDocument> getStateDocument(OperationContext* opCtx,
     if (!collection) {
         return Status(ErrorCodes::NamespaceNotFound,
                       str::stream() << "Collection not found looking for state document: "
-                                    << NamespaceString::kShardSplitDonorsNamespace.ns());
+                                    << redactTenant(NamespaceString::kShardSplitDonorsNamespace));
     }
 
     BSONObj result;
@@ -131,6 +162,10 @@ public:
         return NamespaceString::kSystemReplSetNamespace;
     }
 
+    const DatabaseName& db() const final {
+        return NamespaceString::kSystemReplSetNamespace.dbName();
+    }
+
     bool supportsWriteConcern() const final {
         return true;
     }
@@ -165,26 +200,8 @@ private:
     mutable Mutex _mutex = MONGO_MAKE_LATCH("MockReplReconfigCommand::_mutex");
     bool _hasBeenCalled{false};
     BSONObj _msg;
-} mockReplSetReconfigCmd;
-
-namespace {
-sdam::TopologyDescriptionPtr makeRecipientTopologyDescription(const MockReplicaSet& set) {
-    std::shared_ptr<TopologyDescription> topologyDescription =
-        std::make_shared<sdam::TopologyDescription>(sdam::SdamConfiguration(
-            set.getHosts(), sdam::TopologyType::kReplicaSetNoPrimary, set.getSetName()));
-
-    for (auto& server : set.getHosts()) {
-        auto serverDescription = sdam::ServerDescriptionBuilder()
-                                     .withAddress(server)
-                                     .withSetName(set.getSetName())
-                                     .instance();
-        topologyDescription->installServerDescription(serverDescription);
-    }
-
-    return topologyDescription;
-}
-
-}  // namespace
+};
+MONGO_REGISTER_COMMAND(MockReplReconfigCommand).forShard();
 
 std::ostream& operator<<(std::ostream& builder, mongo::ShardSplitDonorStateEnum state) {
     switch (state) {
@@ -200,6 +217,9 @@ std::ostream& operator<<(std::ostream& builder, mongo::ShardSplitDonorStateEnum
         case mongo::ShardSplitDonorStateEnum::kBlocking:
             builder << "kBlocking";
             break;
+        case mongo::ShardSplitDonorStateEnum::kRecipientCaughtUp:
+            builder << "kRecipientCaughtUp";
+            break;
         case mongo::ShardSplitDonorStateEnum::kCommitted:
             builder << "kCommitted";
             break;
@@ -232,7 +252,8 @@ void fastForwardCommittedSnapshotOpTime(
 bool hasActiveSplitForTenants(OperationContext* opCtx, const std::vector<TenantId>& tenantIds) {
     return std::all_of(tenantIds.begin(), tenantIds.end(), [&](const auto& tenantId) {
         return tenant_migration_access_blocker::hasActiveTenantMigration(
-            opCtx, DatabaseName(tenantId.toString() + "_db"));
+            opCtx,
+            DatabaseName::createDatabaseName_forTest(boost::none, tenantId.toString() + "_db"));
     });
 }
 
@@ -304,7 +325,7 @@ bool processReplSetStepUpRequest(executor::NetworkInterfaceMock* net,
         if (commandName == cmdObj.firstElementFieldName() && !statusToReturn.isOK()) {
             net->scheduleErrorResponse(noi, statusToReturn);
         } else {
-            const auto opmsg = OpMsgRequest::fromDBAndBody(request.dbname, request.cmdObj);
+            const auto opmsg = static_cast<OpMsgRequest>(request);
             const auto reply = node->runCommand(request.id, opmsg)->getCommandReply();
             net->scheduleSuccessfulResponse(
                 noi, executor::RemoteCommandResponse(reply, Milliseconds(0)));
@@ -339,7 +360,7 @@ void processIncomingRequest(executor::NetworkInterfaceMock* net,
         return;
     }
 
-    const auto opmsg = OpMsgRequest::fromDBAndBody(request.dbname, request.cmdObj);
+    const auto opmsg = static_cast<OpMsgRequest>(request);
     const auto reply = node->runCommand(request.id, opmsg)->getCommandReply();
     net->scheduleSuccessfulResponse(noi, executor::RemoteCommandResponse(reply, Milliseconds(0)));
 }
@@ -394,6 +415,11 @@ public:
         repl::PrimaryOnlyServiceMongoDTest::tearDown();
     }
 
+    std::unique_ptr<repl::ReplicationCoordinator> makeReplicationCoordinator() override {
+        return std::make_unique<repl::ReplicationCoordinatorMock>(getServiceContext(),
+                                                                  _replSettings);
+    }
+
 protected:
     std::unique_ptr<repl::PrimaryOnlyService> makeService(ServiceContext* serviceContext) override {
         return std::make_unique<ShardSplitDonorService>(serviceContext);
@@ -455,6 +481,16 @@ protected:
         _net->exitNetwork();
     }
 
+    BSONObj getLatestConfig(OperationContext* opCtx) {
+        CommandRegistry* reg = getCommandRegistry(opCtx);
+        Command* baseCmd = reg->findCommand("replSetReconfig");
+        invariant(baseCmd);
+        auto mock = dynamic_cast<MockReplReconfigCommand*>(baseCmd);
+        invariant(mock);
+        return mock->getLatestConfig();
+    }
+
+    const repl::ReplSettings _replSettings = repl::createServerlessReplSettings();
     UUID _uuid = UUID::gen();
     MockReplicaSet _replSet{
         "donorSetForTest", 3, true /* hasPrimary */, false /* dollarPrefixHosts */};
@@ -481,7 +517,7 @@ protected:
 auto makeHelloReply(const std::string& setName,
                     const repl::OpTime& lastWriteOpTime = repl::OpTime(Timestamp(100, 1), 1)) {
     BSONObjBuilder opTimeBuilder;
-    lastWriteOpTime.append(&opTimeBuilder, "opTime");
+    lastWriteOpTime.append("opTime", &opTimeBuilder);
     return BSON("setName" << setName << "lastWrite" << opTimeBuilder.obj());
 };
 
@@ -496,7 +532,7 @@ void mockCommandReplies(MockReplicaSet* replSet) {
 
 TEST_F(ShardSplitDonorServiceTest, BasicShardSplitDonorServiceInstanceCreation) {
     auto opCtx = makeOperationContext();
-    test::shard_split::ScopedTenantAccessBlocker scopedTenants(_tenantIds, opCtx.get());
+    test::shard_split::ScopedTenantAccessBlocker scopedTenants(_uuid, opCtx.get());
     test::shard_split::reconfigToAddRecipientNodes(
         getServiceContext(), _recipientTagName, _replSet.getHosts(), _recipientSet.getHosts());
 
@@ -559,7 +595,7 @@ TEST_F(ShardSplitDonorServiceTest, ShardSplitFailsWhenLockIsHeld) {
 
 TEST_F(ShardSplitDonorServiceTest, ReplSetStepUpRetryable) {
     auto opCtx = makeOperationContext();
-    test::shard_split::ScopedTenantAccessBlocker scopedTenants(_tenantIds, opCtx.get());
+    test::shard_split::ScopedTenantAccessBlocker scopedTenants(_uuid, opCtx.get());
     test::shard_split::reconfigToAddRecipientNodes(
         getServiceContext(), _recipientTagName, _replSet.getHosts(), _recipientSet.getHosts());
 
@@ -599,7 +635,7 @@ TEST_F(ShardSplitDonorServiceTest, ShardSplitDonorServiceTimeout) {
 
     auto opCtx = makeOperationContext();
     auto serviceContext = getServiceContext();
-    test::shard_split::ScopedTenantAccessBlocker scopedTenants(_tenantIds, opCtx.get());
+    test::shard_split::ScopedTenantAccessBlocker scopedTenants(_uuid, opCtx.get());
     test::shard_split::reconfigToAddRecipientNodes(
         serviceContext, _recipientTagName, _replSet.getHosts(), _recipientSet.getHosts());
 
@@ -628,7 +664,7 @@ TEST_F(ShardSplitDonorServiceTest, ShardSplitDonorServiceTimeout) {
 
 TEST_F(ShardSplitDonorServiceTest, ReconfigToRemoveSplitConfig) {
     auto opCtx = makeOperationContext();
-    test::shard_split::ScopedTenantAccessBlocker scopedTenants(_tenantIds, opCtx.get());
+    test::shard_split::ScopedTenantAccessBlocker scopedTenants(_uuid, opCtx.get());
     test::shard_split::reconfigToAddRecipientNodes(
         getServiceContext(), _recipientTagName, _replSet.getHosts(), _recipientSet.getHosts());
 
@@ -657,7 +693,7 @@ TEST_F(ShardSplitDonorServiceTest, ReconfigToRemoveSplitConfig) {
     (*fpPtr)->waitForTimesEntered(initialTimesEntered + 1);
 
     // Validate we currently have a splitConfig and set it as the mock's return value.
-    BSONObj splitConfigBson = mockReplSetReconfigCmd.getLatestConfig();
+    BSONObj splitConfigBson = getLatestConfig(&*opCtx);
     auto splitConfig = repl::ReplSetConfig::parse(splitConfigBson["replSetReconfig"].Obj());
     ASSERT(splitConfig.isSplitConfig());
     auto replCoord = repl::ReplicationCoordinator::get(getServiceContext());
@@ -675,7 +711,7 @@ TEST_F(ShardSplitDonorServiceTest, ReconfigToRemoveSplitConfig) {
     auto completionFuture = serviceInstance->completionFuture();
     completionFuture.wait();
 
-    BSONObj finalConfigBson = mockReplSetReconfigCmd.getLatestConfig();
+    BSONObj finalConfigBson = getLatestConfig(&*opCtx);
     ASSERT_TRUE(finalConfigBson.hasField("replSetReconfig"));
     auto finalConfig = repl::ReplSetConfig::parse(finalConfigBson["replSetReconfig"].Obj());
     ASSERT(!finalConfig.isSplitConfig());
@@ -686,7 +722,7 @@ TEST_F(ShardSplitDonorServiceTest, SendReplSetStepUpToHighestLastApplied) {
     // by replacing the default `hello` replies (set by the MockReplicaSet) with ones that report
     // `lastWrite.opTime` values in a deterministic way.
     auto opCtx = makeOperationContext();
-    test::shard_split::ScopedTenantAccessBlocker scopedTenants(_tenantIds, opCtx.get());
+    test::shard_split::ScopedTenantAccessBlocker scopedTenants(_uuid, opCtx.get());
     test::shard_split::reconfigToAddRecipientNodes(
         getServiceContext(), _recipientTagName, _replSet.getHosts(), _recipientSet.getHosts());
 
@@ -730,7 +766,7 @@ TEST_F(ShardSplitDonorServiceTest, CreateInstanceInAbortedState) {
     auto opCtx = makeOperationContext();
     auto serviceContext = getServiceContext();
 
-    test::shard_split::ScopedTenantAccessBlocker scopedTenants(_tenantIds, opCtx.get());
+    test::shard_split::ScopedTenantAccessBlocker scopedTenants(_uuid, opCtx.get());
     test::shard_split::reconfigToAddRecipientNodes(
         serviceContext, _recipientTagName, _replSet.getHosts(), _recipientSet.getHosts());
 
@@ -758,7 +794,7 @@ TEST_F(ShardSplitDonorServiceTest, CreateInstanceThenAbort) {
     auto opCtx = makeOperationContext();
     auto serviceContext = getServiceContext();
 
-    test::shard_split::ScopedTenantAccessBlocker scopedTenants(_tenantIds, opCtx.get());
+    test::shard_split::ScopedTenantAccessBlocker scopedTenants(_uuid, opCtx.get());
     test::shard_split::reconfigToAddRecipientNodes(
         serviceContext, _recipientTagName, _replSet.getHosts(), _recipientSet.getHosts());
 
@@ -791,7 +827,7 @@ TEST_F(ShardSplitDonorServiceTest, CreateInstanceThenAbort) {
 
 TEST_F(ShardSplitDonorServiceTest, StepDownTest) {
     auto opCtx = makeOperationContext();
-    test::shard_split::ScopedTenantAccessBlocker scopedTenants(_tenantIds, opCtx.get());
+    test::shard_split::ScopedTenantAccessBlocker scopedTenants(_uuid, opCtx.get());
     test::shard_split::reconfigToAddRecipientNodes(
         getServiceContext(), _recipientTagName, _replSet.getHosts(), _recipientSet.getHosts());
 
@@ -825,7 +861,7 @@ TEST_F(ShardSplitDonorServiceTest, DeleteStateDocMarkedGarbageCollectable) {
 
     auto opCtx = makeOperationContext();
 
-    test::shard_split::ScopedTenantAccessBlocker scopedTenants(_tenantIds, opCtx.get());
+    test::shard_split::ScopedTenantAccessBlocker scopedTenants(_uuid, opCtx.get());
     test::shard_split::reconfigToAddRecipientNodes(
         getServiceContext(), _recipientTagName, _replSet.getHosts(), _recipientSet.getHosts());
 
@@ -858,7 +894,7 @@ TEST_F(ShardSplitDonorServiceTest, DeleteStateDocMarkedGarbageCollectable) {
 TEST_F(ShardSplitDonorServiceTest, AbortDueToRecipientNodesValidation) {
     auto opCtx = makeOperationContext();
     auto serviceContext = getServiceContext();
-    test::shard_split::ScopedTenantAccessBlocker scopedTenants(_tenantIds, opCtx.get());
+    test::shard_split::ScopedTenantAccessBlocker scopedTenants(_uuid, opCtx.get());
 
     // Matching recipientSetName to the replSetName to fail validation and abort shard split.
     test::shard_split::reconfigToAddRecipientNodes(
@@ -961,7 +997,7 @@ TEST(RecipientAcceptSplitListenerTest, FutureNotReadyWrongSet) {
 
 TEST_F(ShardSplitDonorServiceTest, ResumeAfterStepdownTest) {
     auto opCtx = makeOperationContext();
-    test::shard_split::ScopedTenantAccessBlocker scopedTenants(_tenantIds, opCtx.get());
+    test::shard_split::ScopedTenantAccessBlocker scopedTenants(_uuid, opCtx.get());
     test::shard_split::reconfigToAddRecipientNodes(
         getServiceContext(), _recipientTagName, _replSet.getHosts(), _recipientSet.getHosts());
 
@@ -1063,7 +1099,7 @@ public:
 
 TEST_F(ShardSplitRecipientCleanupTest, ShardSplitRecipientCleanup) {
     auto opCtx = makeOperationContext();
-    test::shard_split::ScopedTenantAccessBlocker scopedTenants(_tenantIds, opCtx.get());
+    test::shard_split::ScopedTenantAccessBlocker scopedTenants(_uuid, opCtx.get());
 
     ASSERT_OK(getStateDocument(opCtx.get(), _uuid).getStatus());
 
