@@ -28,14 +28,44 @@
  */
 
 #include "mongo/db/serverless/shard_split_utils.h"
-#include "mongo/db/catalog_raii.h"
+
+#include <absl/container/node_hash_map.h>
+#include <absl/meta/type_traits.h>
+#include <algorithm>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <iterator>
+#include <memory>
+#include <mutex>
+
+#include <boost/move/utility_core.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/oid.h"
+#include "mongo/db/client.h"
 #include "mongo/db/concurrency/exception_util.h"
 #include "mongo/db/concurrency/lock_manager_defs.h"
-#include "mongo/db/db_raii.h"
 #include "mongo/db/dbhelpers.h"
+#include "mongo/db/namespace_string.h"
 #include "mongo/db/ops/delete.h"
+#include "mongo/db/ops/update_result.h"
+#include "mongo/db/repl/member_id.h"
+#include "mongo/db/repl/optime.h"
+#include "mongo/db/repl/read_concern_args.h"
 #include "mongo/db/repl/repl_set_config.h"
-#include "mongo/logv2/log_debug.h"
+#include "mongo/db/repl/repl_set_tag.h"
+#include "mongo/db/repl/replication_coordinator.h"
+#include "mongo/db/shard_role.h"
+#include "mongo/db/transaction_resources.h"
+#include "mongo/s/database_version.h"
+#include "mongo/s/shard_version.h"
+#include "mongo/stdx/unordered_set.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/str.h"
+#include "mongo/util/string_map.h"
 
 namespace mongo {
 
@@ -44,7 +74,7 @@ namespace serverless {
 const size_t kMinimumRequiredRecipientNodes = 3;
 
 std::vector<repl::MemberConfig> getRecipientMembers(const repl::ReplSetConfig& config,
-                                                    const StringData& recipientTagName) {
+                                                    StringData recipientTagName) {
     std::vector<repl::MemberConfig> result;
     const auto& tagConfig = config.getTagConfig();
     for (const auto& member : config.members()) {
@@ -63,8 +93,8 @@ std::vector<repl::MemberConfig> getRecipientMembers(const repl::ReplSetConfig& c
 
 
 ConnectionString makeRecipientConnectionString(const repl::ReplSetConfig& config,
-                                               const StringData& recipientTagName,
-                                               const StringData& recipientSetName) {
+                                               StringData recipientTagName,
+                                               StringData recipientSetName) {
     auto recipientMembers = getRecipientMembers(config, recipientTagName);
     std::vector<HostAndPort> recipientNodes;
     std::transform(recipientMembers.cbegin(),
@@ -153,19 +183,26 @@ repl::ReplSetConfig makeSplitConfig(const repl::ReplSetConfig& config,
 
 Status insertStateDoc(OperationContext* opCtx, const ShardSplitDonorDocument& stateDoc) {
     const auto nss = NamespaceString::kShardSplitDonorsNamespace;
-    AutoGetCollection collection(opCtx, nss, MODE_IX);
+    auto collection = acquireCollection(
+        opCtx,
+        CollectionAcquisitionRequest(nss,
+                                     PlacementConcern{boost::none, ShardVersion::UNSHARDED()},
+                                     repl::ReadConcernArgs::get(opCtx),
+                                     AcquisitionPrerequisites::kWrite),
+        MODE_IX);
 
     uassert(ErrorCodes::PrimarySteppedDown,
             str::stream() << "No longer primary while attempting to insert shard split"
                              " state document",
             repl::ReplicationCoordinator::get(opCtx)->canAcceptWritesFor(opCtx, nss));
 
-    return writeConflictRetry(opCtx, "insertShardSplitStateDoc", nss.ns(), [&]() -> Status {
+    return writeConflictRetry(opCtx, "insertShardSplitStateDoc", nss, [&]() -> Status {
         const auto filter = BSON(ShardSplitDonorDocument::kIdFieldName
                                  << stateDoc.getId() << ShardSplitDonorDocument::kExpireAtFieldName
                                  << BSON("$exists" << false));
         const auto updateMod = BSON("$setOnInsert" << stateDoc.toBSON());
-        auto updateResult = Helpers::upsert(opCtx, nss, filter, updateMod, /*fromMigrate=*/false);
+        auto updateResult =
+            Helpers::upsert(opCtx, collection, filter, updateMod, /*fromMigrate=*/false);
 
         invariant(!updateResult.numDocsModified);
         if (updateResult.upsertedId.isEmpty()) {
@@ -179,15 +216,22 @@ Status insertStateDoc(OperationContext* opCtx, const ShardSplitDonorDocument& st
 
 Status updateStateDoc(OperationContext* opCtx, const ShardSplitDonorDocument& stateDoc) {
     const auto nss = NamespaceString::kShardSplitDonorsNamespace;
-    AutoGetCollection collection(opCtx, nss, MODE_IX);
+    auto collection = acquireCollection(
+        opCtx,
+        CollectionAcquisitionRequest(nss,
+                                     PlacementConcern{boost::none, ShardVersion::UNSHARDED()},
+                                     repl::ReadConcernArgs::get(opCtx),
+                                     AcquisitionPrerequisites::kWrite),
+        MODE_IX);
 
-    if (!collection) {
+    if (!collection.exists()) {
         return Status(ErrorCodes::NamespaceNotFound,
-                      str::stream() << nss.ns() << " does not exist");
+                      str::stream() << nss.toStringForErrorMsg() << " does not exist");
     }
 
-    return writeConflictRetry(opCtx, "updateShardSplitStateDoc", nss.ns(), [&]() -> Status {
-        auto updateResult = Helpers::upsert(opCtx, nss, stateDoc.toBSON(), /*fromMigrate=*/false);
+    return writeConflictRetry(opCtx, "updateShardSplitStateDoc", nss, [&]() -> Status {
+        auto updateResult =
+            Helpers::upsert(opCtx, collection, stateDoc.toBSON(), /*fromMigrate=*/false);
         if (updateResult.numMatched == 0) {
             return {ErrorCodes::NoSuchKey,
                     str::stream() << "Existing shard split state document not found for id: "
@@ -200,16 +244,21 @@ Status updateStateDoc(OperationContext* opCtx, const ShardSplitDonorDocument& st
 
 StatusWith<bool> deleteStateDoc(OperationContext* opCtx, const UUID& shardSplitId) {
     const auto nss = NamespaceString::kShardSplitDonorsNamespace;
-    AutoGetCollection collection(opCtx, nss, MODE_IX);
+    const auto collection = acquireCollection(
+        opCtx,
+        CollectionAcquisitionRequest(nss,
+                                     PlacementConcern{boost::none, ShardVersion::UNSHARDED()},
+                                     repl::ReadConcernArgs::get(opCtx),
+                                     AcquisitionPrerequisites::kWrite),
+        MODE_IX);
 
-    if (!collection) {
+    if (!collection.exists()) {
         return Status(ErrorCodes::NamespaceNotFound,
-                      str::stream() << nss.ns() << " does not exist");
+                      str::stream() << nss.toStringForErrorMsg() << " does not exist");
     }
     auto query = BSON(ShardSplitDonorDocument::kIdFieldName << shardSplitId);
-    return writeConflictRetry(opCtx, "ShardSplitDonorDeleteStateDoc", nss.ns(), [&]() -> bool {
-        auto nDeleted =
-            deleteObjects(opCtx, collection.getCollection(), nss, query, true /* justOne */);
+    return writeConflictRetry(opCtx, "ShardSplitDonorDeleteStateDoc", nss, [&]() -> bool {
+        auto nDeleted = deleteObjects(opCtx, collection, query, true /* justOne */);
         return nDeleted > 0;
     });
 }
