@@ -29,11 +29,18 @@
 
 #pragma once
 
+#include <memory>
+#include <string>
+#include <utility>
+
 #include "mongo/db/auth/authz_manager_external_state_mock.h"
+#include "mongo/db/index_builds_coordinator.h"
 #include "mongo/db/service_context_test_fixture.h"
 #include "mongo/db/storage/journal_listener.h"
 #include "mongo/db/storage/storage_engine_init.h"
 #include "mongo/unittest/temp_dir.h"
+#include "mongo/util/duration.h"
+#include "mongo/util/tick_source.h"
 #include "mongo/util/tick_source_mock.h"
 
 namespace mongo {
@@ -99,6 +106,17 @@ protected:
             return std::move(*this);
         }
 
+        Options useIndexBuildsCoordinator(
+            std::unique_ptr<IndexBuildsCoordinator> indexBuildsCoordinator) {
+            _indexBuildsCoordinator = std::move(indexBuildsCoordinator);
+            return std::move(*this);
+        }
+
+        Options setAuthEnabled(bool enableAuth) {
+            _setAuthEnabled = enableAuth;
+            return std::move(*this);
+        }
+
     private:
         std::string _engine = "wiredTiger";
         // We use ephemeral instances by default to advise Storage Engines (in particular
@@ -108,10 +126,12 @@ protected:
         StorageEngineInitFlags _initFlags = kDefaultStorageEngineInitFlags;
         bool _useReplSettings = false;
         bool _useMockClock = false;
+        bool _setAuthEnabled = true;
         Milliseconds _autoAdvancingMockClockIncrement{0};
         std::unique_ptr<TickSource> _mockTickSource;
         std::unique_ptr<JournalListener> _journalListener;
         std::unique_ptr<AuthzManagerExternalStateMock> _mockAuthzExternalState;
+        std::unique_ptr<IndexBuildsCoordinator> _indexBuildsCoordinator;
         bool _forceDisableTableLogging = false;
 
         friend class ServiceContextMongoDTest;
@@ -119,7 +139,7 @@ protected:
 
     explicit ServiceContextMongoDTest(Options options = {});
 
-    virtual ~ServiceContextMongoDTest();
+    ~ServiceContextMongoDTest() override;
 
     void tearDown() override;
 
