@@ -27,39 +27,48 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/session/logical_session_cache_impl.h"
-
+#include <absl/container/node_hash_map.h>
+#include <absl/meta/type_traits.h>
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+// IWYU pragma: no_include "ext/alloc_traits.h"
+#include <list>
 #include <memory>
+#include <ostream>
+#include <string>
+#include <utility>
 
-#include "mongo/bson/oid.h"
+#include "mongo/base/error_codes.h"
+#include "mongo/base/string_data.h"
 #include "mongo/db/auth/authorization_manager.h"
-#include "mongo/db/auth/authorization_session_for_test.h"
-#include "mongo/db/auth/authz_manager_external_state_mock.h"
-#include "mongo/db/auth/authz_session_external_state_mock.h"
-#include "mongo/db/auth/user_name.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/service_context.h"
 #include "mongo/db/service_context_test_fixture.h"
 #include "mongo/db/service_liaison_mock.h"
 #include "mongo/db/session/logical_session_cache.h"
+#include "mongo/db/session/logical_session_cache_gen.h"
+#include "mongo/db/session/logical_session_cache_impl.h"
 #include "mongo/db/session/logical_session_id.h"
 #include "mongo/db/session/logical_session_id_helpers.h"
+#include "mongo/db/session/sessions_collection.h"
 #include "mongo/db/session/sessions_collection_mock.h"
-#include "mongo/idl/server_parameter_test_util.h"
-#include "mongo/stdx/future.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/bson_test_util.h"
 #include "mongo/unittest/ensure_fcv.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/concurrency/thread_name.h"
+#include "mongo/util/duration.h"
+#include "mongo/util/time_support.h"
 
 namespace mongo {
 namespace {
 
-const Milliseconds kSessionTimeout = duration_cast<Milliseconds>(kLogicalSessionDefaultTimeout);
+const Milliseconds kSessionTimeout =
+    duration_cast<Milliseconds>(Minutes(kLocalLogicalSessionTimeoutMinutesDefault));
 const Milliseconds kForceRefresh{kLogicalSessionRefreshMillisDefault};
 
 using SessionList = std::list<LogicalSessionId>;
-using unittest::EnsureFCV;
 
 /**
  * Test fixture that sets up a session cache attached to a mock service liaison
@@ -71,13 +80,10 @@ public:
         : _service(std::make_shared<MockServiceLiaisonImpl>()),
           _sessions(std::make_shared<MockSessionsCollectionImpl>()) {
 
-        AuthorizationManager::set(getServiceContext(),
-                                  AuthorizationManager::create(getServiceContext()));
-
         // Re-initialize the client after setting the AuthorizationManager to get an
         // AuthorizationSession.
         Client::releaseCurrent();
-        Client::initThread(getThreadName());
+        Client::initThread(getThreadName(), getGlobalServiceContext()->getService());
         _opCtx = makeOperationContext();
 
         auto mockService = std::make_unique<MockServiceLiaison>(_service);
