@@ -28,16 +28,32 @@
  */
 
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/session/session_catalog.h"
-
+#include <absl/container/node_hash_map.h>
+#include <absl/meta/type_traits.h>
+#include <boost/cstdint.hpp>
+#include <boost/none.hpp>
+#include <boost/optional.hpp>
+#include <cstdint>
 #include <memory>
+#include <string>
+#include <type_traits>
+
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
 
-#include "mongo/db/server_options.h"
 #include "mongo/db/service_context.h"
 #include "mongo/db/session/logical_session_id_helpers.h"
+#include "mongo/db/session/session_catalog.h"
+#include "mongo/db/transaction_resources.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/stdx/unordered_map.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/fail_point.h"
+#include "mongo/util/scopeguard.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kWrite
 
@@ -88,10 +104,7 @@ SessionCatalog* SessionCatalog::get(ServiceContext* service) {
 }
 
 SessionCatalog::ScopedCheckedOutSession SessionCatalog::_checkOutSessionInner(
-    OperationContext* opCtx,
-    const LogicalSessionId& lsid,
-    boost::optional<KillToken> killToken,
-    Milliseconds* timeout) {
+    OperationContext* opCtx, const LogicalSessionId& lsid, boost::optional<KillToken> killToken) {
     if (killToken) {
         dassert(killToken->lsidToKill == lsid);
     } else {
@@ -119,21 +132,12 @@ SessionCatalog::ScopedCheckedOutSession SessionCatalog::_checkOutSessionInner(
         ul.lock();
     }
 
-    auto deadline = timeout ? Date_t::now() + *timeout : Date_t::max();
-    const auto ok = opCtx->waitForConditionOrInterruptUntil(
-        sri->availableCondVar,
-        ul,
-        deadline,
-        [&ul, &sri, &session, forKill = killToken.has_value()]() {
+    opCtx->waitForConditionOrInterrupt(
+        sri->availableCondVar, ul, [&ul, &sri, &session, forKill = killToken.has_value()]() {
             ObservableSession osession(ul, sri, session);
             return osession._isAvailableForCheckOut(forKill);
         });
 
-    if (!ok && killToken) {
-        --sri->killsRequested;
-    }
-    iassert(opCtx->getTimeoutError(), "operation exceeded time limit", ok);
-
     sri->checkoutOpCtx = opCtx;
     sri->lastCheckout = Date_t::now();
 
@@ -145,23 +149,22 @@ SessionCatalog::ScopedCheckedOutSession SessionCatalog::_checkOutSession(Operati
     // deadlock
     invariant(opCtx->getLogicalSessionId());
     invariant(!operationSessionDecoration(opCtx));
-    invariant(!opCtx->lockState()->inAWriteUnitOfWork());
-    invariant(!opCtx->lockState()->isLocked());
+    invariant(!shard_role_details::getLocker(opCtx)->inAWriteUnitOfWork());
+    invariant(!shard_role_details::getLocker(opCtx)->isLocked());
 
     auto lsid = *opCtx->getLogicalSessionId();
     return _checkOutSessionInner(opCtx, lsid, boost::none /* killToken */);
 }
 
 SessionCatalog::SessionToKill SessionCatalog::checkOutSessionForKill(OperationContext* opCtx,
-                                                                     KillToken killToken,
-                                                                     Milliseconds* timeout) {
+                                                                     KillToken killToken) {
     // This method is not supposed to be called with an already checked-out session due to risk of
     // deadlock
     invariant(!operationSessionDecoration(opCtx));
     invariant(!opCtx->getTxnNumber());
 
     auto lsid = killToken.lsidToKill;
-    return SessionToKill(_checkOutSessionInner(opCtx, lsid, std::move(killToken), timeout));
+    return SessionToKill(_checkOutSessionInner(opCtx, lsid, std::move(killToken)));
 }
 
 void SessionCatalog::scanSession(const LogicalSessionId& lsid,
@@ -187,11 +190,7 @@ void SessionCatalog::scanSessions(const SessionKiller::Matcher& matcher,
                                   const ScanSessionsCallbackFn& workerFn) {
     stdx::lock_guard<Latch> lg(_mutex);
 
-    LOGV2_DEBUG(21976,
-                2,
-                "Scanning {sessionCount} sessions",
-                "Scanning sessions",
-                "sessionCount"_attr = _sessions.size());
+    LOGV2_DEBUG(21976, 2, "Scanning sessions", "sessionCount"_attr = _sessions.size());
 
     for (auto& [parentLsid, sri] : _sessions) {
         if (matcher.match(parentLsid)) {
@@ -213,11 +212,7 @@ void SessionCatalog::scanSessions(const SessionKiller::Matcher& matcher,
 void SessionCatalog::scanParentSessions(const ScanSessionsCallbackFn& workerFn) {
     stdx::lock_guard<Latch> lg(_mutex);
 
-    LOGV2_DEBUG(6685000,
-                2,
-                "Scanning {sessionCount} sessions",
-                "Scanning sessions",
-                "sessionCount"_attr = _sessions.size());
+    LOGV2_DEBUG(6685000, 2, "Scanning sessions", "sessionCount"_attr = _sessions.size());
 
     for (auto& [parentLsid, sri] : _sessions) {
         ObservableSession osession(lg, sri.get(), &sri->parentSession);
@@ -295,6 +290,14 @@ size_t SessionCatalog::size() const {
     return _sessions.size();
 }
 
+void SessionCatalog::setDisallowNewTransactions() {
+    _disallowNewTransactions.store(true);
+}
+
+bool SessionCatalog::getDisallowNewTransactions() {
+    return _disallowNewTransactions.load();
+}
+
 SessionCatalog::SessionRuntimeInfo* SessionCatalog::_getSessionRuntimeInfo(
     WithLock wl, const LogicalSessionId& lsid) {
     const auto& parentLsid = isParentSessionId(lsid) ? lsid : *getParentSessionId(lsid);
@@ -435,7 +438,6 @@ SessionCatalog::KillToken ObservableSession::kill(ErrorCodes::Error reason) cons
     ++_sri->killsRequested;
 
     if (firstKiller && hasCurrentOperation()) {
-        invariant(_clientLock.owns_lock());
         const auto serviceContext = _sri->checkoutOpCtx->getServiceContext();
         serviceContext->killOperation(_clientLock, _sri->checkoutOpCtx, reason);
     }
