@@ -29,20 +29,33 @@
 
 #pragma once
 
+#include <boost/move/utility_core.hpp>
 #include <boost/optional.hpp>
+#include <boost/optional/optional.hpp>
+#include <cstddef>
+#include <functional>
+#include <memory>
+#include <mutex>
+#include <utility>
 #include <vector>
 
+#include "mongo/base/error_codes.h"
 #include "mongo/db/client.h"
 #include "mongo/db/operation_context.h"
+#include "mongo/db/service_context.h"
 #include "mongo/db/session/logical_session_id.h"
+#include "mongo/db/session/logical_session_id_gen.h"
 #include "mongo/db/session/logical_session_id_helpers.h"
 #include "mongo/db/session/session.h"
 #include "mongo/db/session/session_killer.h"
 #include "mongo/platform/mutex.h"
 #include "mongo/stdx/condition_variable.h"
 #include "mongo/stdx/unordered_map.h"
+#include "mongo/util/assert_util_core.h"
 #include "mongo/util/concurrency/with_lock.h"
+#include "mongo/util/functional.h"
 #include "mongo/util/hierarchical_acquisition.h"
+#include "mongo/util/time_support.h"
 
 namespace mongo {
 
@@ -105,9 +118,7 @@ public:
      * See the description of 'ObservableSession::kill' for more information on the session kill
      * usage pattern.
      */
-    SessionToKill checkOutSessionForKill(OperationContext* opCtx,
-                                         KillToken killToken,
-                                         Milliseconds* timeout = nullptr);
+    SessionToKill checkOutSessionForKill(OperationContext* opCtx, KillToken killToken);
 
     /**
      * Iterates through the SessionCatalog under the SessionCatalog mutex and applies 'workerFn' to
@@ -165,6 +176,12 @@ public:
         _makeSessionWorkerFnForEagerReap = std::move(makeWorkerFnForEagerReap);
     }
 
+    /**
+     * Called on shutdown to prevent the TransactionRouter from starting a new transaction.
+     */
+    void setDisallowNewTransactions();
+    bool getDisallowNewTransactions();
+
 private:
     /**
      * Tracks the runtime info for transaction sessions that corresponds to the same logical
@@ -220,8 +237,7 @@ private:
      */
     ScopedCheckedOutSession _checkOutSessionInner(OperationContext* opCtx,
                                                   const LogicalSessionId& lsid,
-                                                  boost::optional<KillToken> killToken,
-                                                  Milliseconds* timeout = nullptr);
+                                                  boost::optional<KillToken> killToken);
 
     /**
      * Blocking method, which checks-out the session set on 'opCtx'.
@@ -266,6 +282,8 @@ private:
 
     // Owns the Session objects for all current Sessions.
     SessionRuntimeInfoMap _sessions;
+
+    AtomicWord<bool> _disallowNewTransactions{false};
 };
 
 /**
@@ -284,9 +302,10 @@ public:
         }
     }
 
-    ScopedCheckedOutSession(ScopedCheckedOutSession&& other)
+    ScopedCheckedOutSession(ScopedCheckedOutSession&& other) noexcept
         : _catalog(other._catalog),
-          _clientTxnNumberStartedAndProvenance(other._clientTxnNumberStartedAndProvenance),
+          _clientTxnNumberStartedAndProvenance(
+              std::move(other._clientTxnNumberStartedAndProvenance)),
           _sri(other._sri),
           _session(other._session),
           _killToken(std::move(other._killToken)) {
@@ -467,10 +486,9 @@ public:
 private:
     friend class SessionCatalog;
 
-    static stdx::unique_lock<Client> _lockClientForSession(
-        WithLock, SessionCatalog::SessionRuntimeInfo* sri) {
+    static ClientLock _lockClientForSession(WithLock, SessionCatalog::SessionRuntimeInfo* sri) {
         if (const auto opCtx = sri->checkoutOpCtx) {
-            return stdx::unique_lock<Client>{*opCtx->getClient()};
+            return ClientLock{opCtx->getClient()};
         }
         return {};
     }
@@ -501,7 +519,7 @@ private:
 
     SessionCatalog::SessionRuntimeInfo* _sri;
     Session* _session;
-    stdx::unique_lock<Client> _clientLock;
+    mutable ClientLock _clientLock;
 
     bool _markedForReap{false};
     boost::optional<ReapMode> _reapMode;
