@@ -30,30 +30,70 @@
 
 #include "mongo/db/session/session_catalog_mongod.h"
 
+#include <absl/container/node_hash_set.h>
+#include <boost/cstdint.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <cstdint>
+#include <functional>
 #include <memory>
+#include <mutex>
+#include <utility>
 
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
 #include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/client/dbclient_cursor.h"
+#include "mongo/db/admission/execution_admission_context.h"
+#include "mongo/db/catalog/clustered_collection_util.h"
+#include "mongo/db/catalog/collection.h"
+#include "mongo/db/catalog/collection_options.h"
+#include "mongo/db/catalog/index_builds_manager.h"
+#include "mongo/db/catalog/index_catalog.h"
 #include "mongo/db/catalog_raii.h"
 #include "mongo/db/client.h"
+#include "mongo/db/commands/feature_compatibility_version.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
 #include "mongo/db/create_indexes_gen.h"
+#include "mongo/db/database_name.h"
 #include "mongo/db/dbdirectclient.h"
+#include "mongo/db/feature_flag.h"
+#include "mongo/db/index/index_descriptor.h"
 #include "mongo/db/index_builds_coordinator.h"
 #include "mongo/db/internal_transactions_feature_flag_gen.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/ops/write_ops.h"
-#include "mongo/db/repl/repl_server_parameters_gen.h"
-#include "mongo/db/repl/storage_interface_impl.h"
+#include "mongo/db/ops/write_ops_gen.h"
+#include "mongo/db/query/find_command.h"
+#include "mongo/db/repl/replication_coordinator.h"
+#include "mongo/db/repl/storage_interface.h"
+#include "mongo/db/server_options.h"
 #include "mongo/db/service_context.h"
 #include "mongo/db/session/internal_transactions_reap_service.h"
+#include "mongo/db/session/kill_sessions.h"
+#include "mongo/db/session/logical_session_id_helpers.h"
+#include "mongo/db/session/session_killer.h"
 #include "mongo/db/session/session_txn_record_gen.h"
 #include "mongo/db/session/sessions_collection.h"
+#include "mongo/db/storage/recovery_unit.h"
 #include "mongo/db/transaction/transaction_participant.h"
+#include "mongo/db/transaction_resources.h"
+#include "mongo/idl/idl_parser.h"
 #include "mongo/logv2/log.h"
-#include "mongo/s/transaction_router.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/util/concurrency/admission_context.h"
 #include "mongo/util/concurrency/thread_pool.h"
+#include "mongo/util/decorable.h"
 #include "mongo/util/fail_point.h"
-#include "mongo/util/scopeguard.h"
+#include "mongo/util/str.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTransaction
 
@@ -110,7 +150,14 @@ void killSessionTokens(OperationContext* opCtx,
          sessionKillTokens = std::move(sessionKillTokens)](auto status) mutable {
             invariant(status);
 
-            ThreadClient tc("Kill-Sessions", service);
+            ThreadClient tc("Kill-Sessions", service->getService(ClusterRole::ShardServer));
+
+            // TODO(SERVER-74658): Please revisit if this thread could be made killable.
+            {
+                stdx::lock_guard<Client> lk(*tc.get());
+                tc.get()->setSystemOperationUnkillableByStepdown(lk);
+            }
+
             auto uniqueOpCtx = tc->makeOperationContext();
             const auto opCtx = uniqueOpCtx.get();
             const auto catalog = SessionCatalog::get(opCtx);
@@ -124,12 +171,13 @@ void killSessionTokens(OperationContext* opCtx,
 
 void disallowDirectWritesUnderSession(OperationContext* opCtx) {
     const auto replCoord = repl::ReplicationCoordinator::get(opCtx);
-    bool isReplSet = replCoord->getReplicationMode() == repl::ReplicationCoordinator::modeReplSet;
+    bool isReplSet = replCoord->getSettings().isReplSet();
     if (isReplSet) {
         uassert(40528,
-                str::stream() << "Direct writes against "
-                              << NamespaceString::kSessionTransactionsTableNamespace
-                              << " cannot be performed using a transaction or on a session.",
+                str::stream()
+                    << "Direct writes against "
+                    << NamespaceString::kSessionTransactionsTableNamespace.toStringForErrorMsg()
+                    << " cannot be performed using a transaction or on a session.",
                 !opCtx->getLogicalSessionId());
     }
 }
@@ -335,6 +383,12 @@ int removeExpiredTransactionSessionsFromDisk(
 
 void createTransactionTable(OperationContext* opCtx) {
     CollectionOptions options;
+    // We cluster by _id for improved performance at the cost of increased index maintenance.
+    // Because we only have one partial index on this collection, the performance benefit outweighs
+    // that cost.
+    if (feature_flags::gFeatureFlagClusteredConfigTransactions.isEnabled(
+            serverGlobalParams.featureCompatibility.acquireFCVSnapshot()))
+        options.clusteredIndex = clustered_util::makeDefaultClusteredIdIndex();
     auto storageInterface = repl::StorageInterface::get(opCtx);
     auto createCollectionStatus = storageInterface->createCollection(
         opCtx, NamespaceString::kSessionTransactionsTableNamespace, options);
@@ -378,11 +432,12 @@ void createTransactionTable(OperationContext* opCtx) {
         // empty collection. This can happen after a failover because the collection and index
         // creation are recorded as separate oplog entries.
     } else {
-        uassertStatusOKWithContext(createCollectionStatus,
-                                   str::stream()
-                                       << "Failed to create the "
-                                       << NamespaceString::kSessionTransactionsTableNamespace.ns()
-                                       << " collection");
+        uassertStatusOKWithContext(
+            createCollectionStatus,
+            str::stream()
+                << "Failed to create the "
+                << NamespaceString::kSessionTransactionsTableNamespace.toStringForErrorMsg()
+                << " collection");
     }
 
     auto indexSpec = MongoDSessionCatalog::getConfigTxnPartialIndexSpec();
@@ -392,7 +447,8 @@ void createTransactionTable(OperationContext* opCtx) {
     uassertStatusOKWithContext(
         createIndexStatus,
         str::stream() << "Failed to create partial index for the "
-                      << NamespaceString::kSessionTransactionsTableNamespace.ns() << " collection");
+                      << NamespaceString::kSessionTransactionsTableNamespace.toStringForErrorMsg()
+                      << " collection");
 }
 
 void createRetryableFindAndModifyTable(OperationContext* opCtx) {
@@ -405,9 +461,10 @@ void createRetryableFindAndModifyTable(OperationContext* opCtx) {
     }
 
     uassertStatusOKWithContext(status,
-                               str::stream() << "Failed to create the "
-                                             << NamespaceString::kConfigImagesNamespace.ns()
-                                             << " collection");
+                               str::stream()
+                                   << "Failed to create the "
+                                   << NamespaceString::kConfigImagesNamespace.toStringForErrorMsg()
+                                   << " collection");
 }
 
 
@@ -518,15 +575,25 @@ void MongoDSessionCatalog::onStepUp(OperationContext* opCtx) {
         matcher, _ti->makeSessionWorkerFnForStepUp(&sessionKillTokens, &sessionsToReacquireLocks));
     killSessionTokens(opCtx, _ti.get(), std::move(sessionKillTokens));
 
-    {
+    if (sessionsToReacquireLocks.size() > 0) {
+        LOGV2(8083200, "Reacquiring locks for prepared transactions on step-up.");
         // Create a new opCtx because we need an empty locker to refresh the locks.
-        auto newClient = opCtx->getServiceContext()->makeClient("restore-prepared-txn");
+        auto newClient = opCtx->getServiceContext()
+                             ->getService(ClusterRole::ShardServer)
+                             ->makeClient("restore-prepared-txn");
+
+        {
+            stdx::lock_guard<Client> lk(*newClient.get());
+            newClient.get()->setSystemOperationUnkillableByStepdown(lk);
+        }
+
         AlternativeClientRegion acr(newClient);
         for (const auto& sessionInfo : sessionsToReacquireLocks) {
             auto newOpCtx = cc().makeOperationContext();
 
             // Avoid ticket acquisition during step up.
-            newOpCtx->lockState()->setAdmissionPriority(AdmissionContext::Priority::kImmediate);
+            ScopedAdmissionPriority<ExecutionAdmissionContext> admissionPriority(
+                newOpCtx.get(), AdmissionContext::Priority::kExempt);
 
             // Synchronize with killOps to make this unkillable.
             {
@@ -559,11 +626,7 @@ void MongoDSessionCatalog::onStepUp(OperationContext* opCtx) {
     abortInProgressTransactions(opCtx, this, _ti.get());
 
     createTransactionTable(opCtx);
-    // (Ignore FCV check): This is intentional to try creating the image_collection collection if
-    // the feature flag is ever enabled.
-    if (repl::feature_flags::gFeatureFlagRetryableFindAndModify.isEnabledAndIgnoreFCVUnsafe()) {
-        createRetryableFindAndModifyTable(opCtx);
-    }
+    createRetryableFindAndModifyTable(opCtx);
 }
 
 boost::optional<UUID> MongoDSessionCatalog::getTransactionTableUUID(OperationContext* opCtx) {
@@ -619,8 +682,9 @@ void MongoDSessionCatalog::observeDirectWriteToConfigTransactions(OperationConte
         bool shouldRegisterKill = !isInternalSessionForRetryableWrite(lsid) ||
             *lsid.getTxnNumber() >= session.getLastClientTxnNumberStarted();
         if (shouldRegisterKill) {
-            opCtx->recoveryUnit()->registerChange(std::make_unique<KillSessionTokenOnCommit>(
-                ti, session.kill(ErrorCodes::Interrupted)));
+            shard_role_details::getRecoveryUnit(opCtx)->registerChange(
+                std::make_unique<KillSessionTokenOnCommit>(ti,
+                                                           session.kill(ErrorCodes::Interrupted)));
         }
     });
 }
@@ -653,7 +717,7 @@ int MongoDSessionCatalog::reapSessionsOlderThan(OperationContext* opCtx,
     // around the fact that the logical sessions cache is not registered to listen for replication
     // state changes.
     const auto replCoord = repl::ReplicationCoordinator::get(opCtx);
-    if (!replCoord->canAcceptWritesForDatabase_UNSAFE(opCtx, DatabaseName::kConfig.toString()))
+    if (!replCoord->canAcceptWritesForDatabase_UNSAFE(opCtx, DatabaseName::kConfig))
         return 0;
 
     return removeExpiredTransactionSessionsFromDisk(
