@@ -27,15 +27,36 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <boost/move/utility_core.hpp>
+#include <boost/smart_ptr.hpp>
+#include <string>
+#include <tuple>
+#include <utility>
 
-#include "mongo/db/set_change_stream_state_coordinator.h"
+#include <boost/optional/optional.hpp>
 
+#include "mongo/base/checked_cast.h"
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/simple_bsonobj_comparator.h"
 #include "mongo/db/change_stream_change_collection_manager.h"
 #include "mongo/db/change_stream_pre_images_collection_manager.h"
 #include "mongo/db/change_stream_state_gen.h"
+#include "mongo/db/client.h"
+#include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/repl/wait_for_majority_service.h"
+#include "mongo/db/set_change_stream_state_coordinator.h"
+#include "mongo/db/tenant_id.h"
+#include "mongo/idl/idl_parser.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/fail_point.h"
+#include "mongo/util/future_impl.h"
+#include "mongo/util/interruptible.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
@@ -53,11 +74,11 @@ constexpr auto kSetChangeStreamStateCoordinatorName = "setChangeStreamStateCoord
 /**
  * Waits until the oplog entry has been majority committed.
  */
-void waitForMajority(OperationContext* opCtx) {
+void waitUntilMajorityForWrite(OperationContext* opCtx) {
     const auto replCoord = repl::ReplicationCoordinator::get(opCtx);
     const auto lastLocalOpTime = replCoord->getMyLastAppliedOpTime();
     WaitForMajorityService::get(opCtx->getServiceContext())
-        .waitUntilMajority(lastLocalOpTime, opCtx->getCancellationToken())
+        .waitUntilMajorityForWrite(lastLocalOpTime, opCtx->getCancellationToken())
         .get(opCtx);
 }
 
@@ -98,10 +119,11 @@ private:
         auto& changeCollectionManager = ChangeStreamChangeCollectionManager::get(opCtx);
         changeCollectionManager.createChangeCollection(opCtx, tenantId);
 
-        ChangeStreamPreImagesCollectionManager::createPreImagesCollection(opCtx, tenantId);
+        ChangeStreamPreImagesCollectionManager::get(opCtx).createPreImagesCollection(opCtx,
+                                                                                     tenantId);
 
         // Wait until the create requests are majority committed.
-        waitForMajority(opCtx);
+        waitUntilMajorityForWrite(opCtx);
     }
 
     /**
@@ -112,10 +134,10 @@ private:
         auto& changeCollectionManager = ChangeStreamChangeCollectionManager::get(opCtx);
         changeCollectionManager.dropChangeCollection(opCtx, tenantId);
 
-        ChangeStreamPreImagesCollectionManager::dropPreImagesCollection(opCtx, tenantId);
+        ChangeStreamPreImagesCollectionManager::get(opCtx).dropPreImagesCollection(opCtx, tenantId);
 
         // Wait until the drop requests are majority committed.
-        waitForMajority(opCtx);
+        waitUntilMajorityForWrite(opCtx);
     }
 
     const SetChangeStreamStateCoordinatorDocument _stateDoc;
@@ -170,7 +192,7 @@ ExecutorFuture<void> SetChangeStreamStateCoordinator::_runImpl(
             // A series of retries, step-up and step-down events can cause a node to try and insert
             // the document when it has already been persisted locally, but we must still wait for
             // majority commit.
-            waitForMajority(opCtx);
+            waitUntilMajorityForWrite(opCtx);
         }
 
         hangSetChangeStreamStateCoordinatorBeforeCommandProcessor.pauseWhileSet(
@@ -223,7 +245,7 @@ SetChangeStreamStateCoordinatorService::getOrCreateInstance(OperationContext* op
             LOGV2_ERROR(6663106,
                         "Failed to create coordinator instance",
                         "service"_attr = kSetChangeStreamStateCoordinatorServiceName,
-                        "reason"_attr = redact(ex));
+                        "reason"_attr = ex);
             throw;
         }
     }();
