@@ -29,16 +29,29 @@
 
 #pragma once
 
+#include <absl/container/flat_hash_map.h>
+#include <absl/meta/type_traits.h>
+#include <cstdint>
 #include <map>
+#include <memory>
+#include <string>
+#include <utility>
+#include <vector>
 
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/db/commands/server_status_metric.h"
 #include "mongo/db/curop.h"
 #include "mongo/db/jsobj.h"
+#include "mongo/db/query/plan_executor.h"
 #include "mongo/platform/atomic_word.h"
 #include "mongo/platform/basic.h"
 #include "mongo/rpc/message.h"
 #include "mongo/util/aligned.h"
-#include "mongo/util/concurrency/spin_lock.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/processinfo.h"
+#include "mongo/util/str.h"
 #include "mongo/util/string_map.h"
 
 namespace mongo {
@@ -298,29 +311,41 @@ extern AuthCounter authCounter;
 class AggStageCounters {
 public:
     // Container for a stage count metric along with its corresponding counter.
-    struct StageCounter {
-        StageCounter(StringData name) : counter("aggStageCounters." + name) {}
-        CounterMetric counter;
-    };
+    explicit AggStageCounters(std::string prefix) : _prefix{std::move(prefix)} {}
 
+    void addMetric(const std::string& name) {
+        _stages[name] = &*MetricBuilder<Counter64>(_prefix + name);
+    }
+
+    /** requires `name` be a metric previously added with `addMetric`. */
+    void increment(StringData name, long long n = 1) {
+        _stages.find(name)->second->incrementRelaxed(n);
+    }
+
+private:
+    std::string _prefix;
     // Map of aggregation stages to the number of occurrences.
-    StringMap<std::unique_ptr<StageCounter>> stageCounterMap = {};
+    StringMap<Counter64*> _stages;
 };
 
 extern AggStageCounters aggStageCounters;
 
 class DotsAndDollarsFieldsCounters {
 public:
+    DotsAndDollarsFieldsCounters() = default;
+    DotsAndDollarsFieldsCounters(DotsAndDollarsFieldsCounters&) = delete;
+    DotsAndDollarsFieldsCounters& operator=(const DotsAndDollarsFieldsCounters&) = delete;
+
     void incrementForUpsert(bool didInsert) {
         if (didInsert) {
-            inserts.increment();
+            inserts.incrementRelaxed();
         } else {
-            updates.increment();
+            updates.incrementRelaxed();
         }
     }
 
-    CounterMetric inserts{"dotsAndDollarsFields.inserts"};
-    CounterMetric updates{"dotsAndDollarsFields.updates"};
+    Counter64& inserts = *MetricBuilder<Counter64>{"dotsAndDollarsFields.inserts"};
+    Counter64& updates = *MetricBuilder<Counter64>{"dotsAndDollarsFields.updates"};
 };
 
 extern DotsAndDollarsFieldsCounters dotsAndDollarsFieldsCounters;
@@ -328,6 +353,8 @@ extern DotsAndDollarsFieldsCounters dotsAndDollarsFieldsCounters;
 class QueryFrameworkCounters {
 public:
     QueryFrameworkCounters() = default;
+    QueryFrameworkCounters(QueryFrameworkCounters&) = delete;
+    QueryFrameworkCounters& operator=(const QueryFrameworkCounters&) = delete;
 
     void incrementQueryEngineCounters(CurOp* curop) {
         auto& debug = curop->debug();
@@ -337,13 +364,13 @@ public:
         if (cmdName == "find") {
             switch (debug.queryFramework) {
                 case PlanExecutor::QueryFramework::kClassicOnly:
-                    classicFindQueryCounter.increment();
+                    classicFindQueryCounter.incrementRelaxed();
                     break;
                 case PlanExecutor::QueryFramework::kSBEOnly:
-                    sbeFindQueryCounter.increment();
+                    sbeFindQueryCounter.incrementRelaxed();
                     break;
                 case PlanExecutor::QueryFramework::kCQF:
-                    cqfFindQueryCounter.increment();
+                    cqfFindQueryCounter.incrementRelaxed();
                     break;
                 default:
                     break;
@@ -351,19 +378,19 @@ public:
         } else if (cmdName == "aggregate") {
             switch (debug.queryFramework) {
                 case PlanExecutor::QueryFramework::kClassicOnly:
-                    classicOnlyAggregationCounter.increment();
+                    classicOnlyAggregationCounter.incrementRelaxed();
                     break;
                 case PlanExecutor::QueryFramework::kClassicHybrid:
-                    classicHybridAggregationCounter.increment();
+                    classicHybridAggregationCounter.incrementRelaxed();
                     break;
                 case PlanExecutor::QueryFramework::kSBEOnly:
-                    sbeOnlyAggregationCounter.increment();
+                    sbeOnlyAggregationCounter.incrementRelaxed();
                     break;
                 case PlanExecutor::QueryFramework::kSBEHybrid:
-                    sbeHybridAggregationCounter.increment();
+                    sbeHybridAggregationCounter.incrementRelaxed();
                     break;
                 case PlanExecutor::QueryFramework::kCQF:
-                    cqfAggregationQueryCounter.increment();
+                    cqfAggregationQueryCounter.incrementRelaxed();
                     break;
                 case PlanExecutor::QueryFramework::kUnknown:
                     break;
@@ -374,98 +401,110 @@ public:
     // Query counters that record whether a find query was fully or partially executed in SBE, fully
     // executed using the classic engine, or fully executed using the common query framework (CQF).
     // One of these will always be incremented during a query.
-    CounterMetric sbeFindQueryCounter{"query.queryFramework.find.sbe"};
-    CounterMetric classicFindQueryCounter{"query.queryFramework.find.classic"};
-    CounterMetric cqfFindQueryCounter{"query.queryFramework.find.cqf"};
+    Counter64& sbeFindQueryCounter = *MetricBuilder<Counter64>{"query.queryFramework.find.sbe"};
+    Counter64& classicFindQueryCounter =
+        *MetricBuilder<Counter64>{"query.queryFramework.find.classic"};
+    Counter64& cqfFindQueryCounter = *MetricBuilder<Counter64>{"query.queryFramework.find.cqf"};
 
     // Aggregation query counters that record whether an aggregation was fully or partially executed
     // in DocumentSource (an sbe/classic hybrid plan), fully pushed down to the sbe/classic layer,
     // or executed using CQF. These are only incremented during aggregations.
-    CounterMetric sbeOnlyAggregationCounter{"query.queryFramework.aggregate.sbeOnly"};
-    CounterMetric classicOnlyAggregationCounter{"query.queryFramework.aggregate.classicOnly"};
-    CounterMetric sbeHybridAggregationCounter{"query.queryFramework.aggregate.sbeHybrid"};
-    CounterMetric classicHybridAggregationCounter{"query.queryFramework.aggregate.classicHybrid"};
-    CounterMetric cqfAggregationQueryCounter{"query.queryFramework.aggregate.cqf"};
+    Counter64& sbeOnlyAggregationCounter =
+        *MetricBuilder<Counter64>{"query.queryFramework.aggregate.sbeOnly"};
+    Counter64& classicOnlyAggregationCounter =
+        *MetricBuilder<Counter64>{"query.queryFramework.aggregate.classicOnly"};
+    Counter64& sbeHybridAggregationCounter =
+        *MetricBuilder<Counter64>{"query.queryFramework.aggregate.sbeHybrid"};
+    Counter64& classicHybridAggregationCounter =
+        *MetricBuilder<Counter64>{"query.queryFramework.aggregate.classicHybrid"};
+    Counter64& cqfAggregationQueryCounter =
+        *MetricBuilder<Counter64>{"query.queryFramework.aggregate.cqf"};
 };
 extern QueryFrameworkCounters queryFrameworkCounters;
 
 class LookupPushdownCounters {
 public:
     LookupPushdownCounters() = default;
+    LookupPushdownCounters(LookupPushdownCounters&) = delete;
+    LookupPushdownCounters& operator=(const LookupPushdownCounters&) = delete;
 
     void incrementLookupCountersPerQuery(int nestedLoopJoin, int indexedLoopJoin, int hashLookup) {
-        nestedLoopJoinCounter.increment(nestedLoopJoin);
-        indexedLoopJoinCounter.increment(indexedLoopJoin);
-        hashLookupCounter.increment(hashLookup);
+        nestedLoopJoinCounter.incrementRelaxed(nestedLoopJoin);
+        indexedLoopJoinCounter.incrementRelaxed(indexedLoopJoin);
+        hashLookupCounter.incrementRelaxed(hashLookup);
     }
 
     void incrementLookupCountersPerSpilling(int64_t spillToDisk, int64_t spillToDiskBytes) {
-        hashLookupSpillToDisk.increment(spillToDisk);
-        hashLookupSpillToDiskBytes.increment(spillToDiskBytes);
+        hashLookupSpillToDisk.incrementRelaxed(spillToDisk);
+        hashLookupSpillToDiskBytes.incrementRelaxed(spillToDiskBytes);
     }
 
     // Counters for lookup join strategies.
-    CounterMetric nestedLoopJoinCounter{"query.lookup.nestedLoopJoin"};
-    CounterMetric indexedLoopJoinCounter{"query.lookup.indexedLoopJoin"};
-    CounterMetric hashLookupCounter{"query.lookup.hashLookup"};
+    Counter64& nestedLoopJoinCounter = *MetricBuilder<Counter64>{"query.lookup.nestedLoopJoin"};
+    Counter64& indexedLoopJoinCounter = *MetricBuilder<Counter64>{"query.lookup.indexedLoopJoin"};
+    Counter64& hashLookupCounter = *MetricBuilder<Counter64>{"query.lookup.hashLookup"};
     // Counter tracking hashLookup spills in lookup stages that get pushed down.
-    CounterMetric hashLookupSpillToDisk{"query.lookup.hashLookupSpillToDisk"};
+    Counter64& hashLookupSpillToDisk =
+        *MetricBuilder<Counter64>{"query.lookup.hashLookupSpillToDisk"};
     // Counter tracking hashLookup spilled bytes in lookup stages that get pushed down.
-    CounterMetric hashLookupSpillToDiskBytes{"query.lookup.hashLookupSpillToDiskBytes"};
+    Counter64& hashLookupSpillToDiskBytes =
+        *MetricBuilder<Counter64>{"query.lookup.hashLookupSpillToDiskBytes"};
 };
 extern LookupPushdownCounters lookupPushdownCounters;
 
 class SortCounters {
 public:
-    SortCounters() = default;
-
     void incrementSortCountersPerQuery(int64_t bytesSorted, int64_t keysSorted) {
-        sortTotalBytesCounter.increment(bytesSorted);
-        sortTotalKeysCounter.increment(keysSorted);
+        sortTotalBytesCounter.incrementRelaxed(bytesSorted);
+        sortTotalKeysCounter.incrementRelaxed(keysSorted);
     }
 
     void incrementSortCountersPerSpilling(int64_t sortSpills, int64_t sortSpillBytes) {
-        sortSpillsCounter.increment(sortSpills);
-        sortSpillBytesCounter.increment(sortSpillBytes);
+        sortSpillsCounter.incrementRelaxed(sortSpills);
+        sortSpillBytesCounter.incrementRelaxed(sortSpillBytes);
     }
 
     // Counters tracking sort stats across all engines
     // The total number of spills from sort stages
-    CounterMetric sortSpillsCounter{"query.sort.spillToDisk"};
+    Counter64& sortSpillsCounter = *MetricBuilder<Counter64>{"query.sort.spillToDisk"};
     // The total bytes spilled. This is the storage size after compression.
-    CounterMetric sortSpillBytesCounter{"query.sort.spillToDiskBytes"};
+    Counter64& sortSpillBytesCounter = *MetricBuilder<Counter64>{"query.sort.spillToDiskBytes"};
     // The number of keys that we've sorted.
-    CounterMetric sortTotalKeysCounter{"query.sort.totalKeysSorted"};
+    Counter64& sortTotalKeysCounter = *MetricBuilder<Counter64>{"query.sort.totalKeysSorted"};
     // The amount of data we've sorted in bytes
-    CounterMetric sortTotalBytesCounter{"query.sort.totalBytesSorted"};
+    Counter64& sortTotalBytesCounter = *MetricBuilder<Counter64>{"query.sort.totalBytesSorted"};
 };
 extern SortCounters sortCounters;
 
+/** Counters tracking group stats across all execution engines. */
 class GroupCounters {
 public:
     GroupCounters() = default;
+    GroupCounters(GroupCounters&) = delete;
+    GroupCounters& operator=(const GroupCounters&) = delete;
 
     void incrementGroupCountersPerSpilling(int64_t spills,
                                            int64_t spilledBytes,
                                            int64_t spilledRecords) {
-        groupSpills.increment(spills);
-        groupSpilledBytes.increment(spilledBytes);
-        groupSpilledRecords.increment(spilledRecords);
+        groupSpills.incrementRelaxed(spills);
+        groupSpilledBytes.incrementRelaxed(spilledBytes);
+        groupSpilledRecords.incrementRelaxed(spilledRecords);
     }
 
     void incrementGroupCountersPerQuery(int64_t spilledDataStorageSize) {
-        groupSpilledDataStorageSize.increment(spilledDataStorageSize);
+        groupSpilledDataStorageSize.incrementRelaxed(spilledDataStorageSize);
     }
 
     // The total number of spills from group stages.
-    CounterMetric groupSpills{"query.group.spills"};
+    Counter64& groupSpills = *MetricBuilder<Counter64>{"query.group.spills"};
     // The total number of bytes spilled from group stages. The spilled stroage size after
     // compression might be different from the bytes spilled.
-    CounterMetric groupSpilledBytes{"query.group.spilledBytes"};
+    Counter64& groupSpilledBytes = *MetricBuilder<Counter64>{"query.group.spilledBytes"};
     // The number of records spilled.
-    CounterMetric groupSpilledRecords{"query.group.spilledRecords"};
+    Counter64& groupSpilledRecords = *MetricBuilder<Counter64>{"query.group.spilledRecords"};
     // The size of the file or RecordStore spilled to disk, updated after all spilling happened.
-    CounterMetric groupSpilledDataStorageSize{"query.group.spilledDataStorageSize"};
+    Counter64& groupSpilledDataStorageSize =
+        *MetricBuilder<Counter64>{"query.group.spilledDataStorageSize"};
 };
 extern GroupCounters groupCounters;
 
@@ -479,18 +518,20 @@ public:
     void incrementSetWindowFieldsCountersPerSpilling(int64_t spills,
                                                      int64_t spilledBytes,
                                                      int64_t spilledRecords) {
-        setWindowFieldsSpills.increment(spills);
-        setWindowFieldsSpilledBytes.increment(spilledBytes);
-        setWindowFieldsSpilledRecords.increment(spilledRecords);
+        setWindowFieldsSpills.incrementRelaxed(spills);
+        setWindowFieldsSpilledBytes.incrementRelaxed(spilledBytes);
+        setWindowFieldsSpilledRecords.incrementRelaxed(spilledRecords);
     }
 
     // Counter tracking setWindowFields spills.
-    CounterMetric setWindowFieldsSpills{"query.setWindowFields.spills"};
+    Counter64& setWindowFieldsSpills = *MetricBuilder<Counter64>{"query.setWindowFields.spills"};
     // Counter tracking setWindowFields spilled bytes. The spilled storage size after compression
     // might be different from the bytes spilled.
-    CounterMetric setWindowFieldsSpilledBytes{"query.setWindowFields.spilledBytes"};
+    Counter64& setWindowFieldsSpilledBytes =
+        *MetricBuilder<Counter64>{"query.setWindowFields.spilledBytes"};
     // Counter tracking setWindowFields spilled record number.
-    CounterMetric setWindowFieldsSpilledRecords{"query.setWindowFields.spilledRecords"};
+    Counter64& setWindowFieldsSpilledRecords =
+        *MetricBuilder<Counter64>{"query.setWindowFields.spilledRecords"};
 };
 extern SetWindowFieldsCounters setWindowFieldsCounters;
 
@@ -500,43 +541,60 @@ extern SetWindowFieldsCounters setWindowFieldsCounters;
 class PlanCacheCounters {
 public:
     PlanCacheCounters() = default;
+    PlanCacheCounters(PlanCacheCounters&) = delete;
+    PlanCacheCounters& operator=(const PlanCacheCounters&) = delete;
 
     void incrementClassicHitsCounter() {
-        classicHits.increment();
+        classicHits.incrementRelaxed();
     }
 
     void incrementClassicMissesCounter() {
-        classicMisses.increment();
+        classicMisses.incrementRelaxed();
+    }
+
+    void incrementClassicSkippedCounter() {
+        classicSkipped.incrementRelaxed();
     }
 
     void incrementClassicReplannedCounter() {
-        classicReplanned.increment();
+        classicReplanned.incrementRelaxed();
     }
 
     void incrementSbeHitsCounter() {
-        sbeHits.increment();
+        sbeHits.incrementRelaxed();
     }
 
     void incrementSbeMissesCounter() {
-        sbeMisses.increment();
+        sbeMisses.incrementRelaxed();
+    }
+
+    void incrementSbeSkippedCounter() {
+        sbeSkipped.incrementRelaxed();
     }
 
     void incrementSbeReplannedCounter() {
-        sbeReplanned.increment();
+        sbeReplanned.incrementRelaxed();
     }
 
 private:
+    static Counter64& _makeMetric(std::string name) {
+        return *MetricBuilder<Counter64>("query.planCache." + std::move(name));
+    }
+
     // Counters that track the number of times a query plan is:
     // a) found in the cache (hits),
     // b) not found in cache (misses), or
-    // c) failed to finish trial run within budget, so we decided to replan it (replanned).
+    // c) not considered for caching hence we don't even look for it in the cache (skipped).
+    // d) failed to finish trial run within budget, so we decided to replan it (replanned).
     // Split into classic and SBE, depending on which execution engine is used.
-    CounterMetric classicHits{"query.planCache.classic.hits"};
-    CounterMetric classicMisses{"query.planCache.classic.misses"};
-    CounterMetric classicReplanned{"query.planCache.classic.replanned"};
-    CounterMetric sbeHits{"query.planCache.sbe.hits"};
-    CounterMetric sbeMisses{"query.planCache.sbe.misses"};
-    CounterMetric sbeReplanned{"query.planCache.sbe.replanned"};
+    Counter64& classicHits = _makeMetric("classic.hits");
+    Counter64& classicMisses = _makeMetric("classic.misses");
+    Counter64& classicSkipped = _makeMetric("classic.skipped");
+    Counter64& classicReplanned = _makeMetric("classic.replanned");
+    Counter64& sbeHits = _makeMetric("sbe.hits");
+    Counter64& sbeMisses = _makeMetric("sbe.misses");
+    Counter64& sbeSkipped = _makeMetric("sbe.skipped");
+    Counter64& sbeReplanned = _makeMetric("sbe.replanned");
 };
 extern PlanCacheCounters planCacheCounters;
 
@@ -544,24 +602,17 @@ extern PlanCacheCounters planCacheCounters;
  * Generic class for counters of expressions inside various MQL statements.
  */
 class OperatorCounters {
-private:
-    struct ExprCounter {
-        ExprCounter(const std::string& name) : counter(name) {}
-        CounterMetric counter;
-    };
-
 public:
-    OperatorCounters(const std::string prefix) : _prefix{prefix} {}
+    explicit OperatorCounters(std::string prefix) : _prefix{std::move(prefix)} {}
 
-    void addCounter(const std::string name) {
-        const StringData sdName(name);
-        operatorCountersExprMap[sdName] = std::make_unique<ExprCounter>(_prefix + name);
+    void addCounter(const std::string& name) {
+        _counters[name] = &*MetricBuilder<Counter64>(_prefix + name);
     }
 
-    void mergeCounters(StringMap<uint64_t>& toMerge) {
+    void mergeCounters(const StringMap<uint64_t>& toMerge) {
         for (auto&& [name, cnt] : toMerge) {
-            if (auto it = operatorCountersExprMap.find(name); it != operatorCountersExprMap.end()) {
-                it->second->counter.increment(cnt);
+            if (auto it = _counters.find(name); it != _counters.end()) {
+                it->second->incrementRelaxed(cnt);
             }
         }
     }
@@ -569,7 +620,7 @@ public:
 private:
     const std::string _prefix;
     // Map of expressions to the number of occurrences in queries.
-    StringMap<std::unique_ptr<ExprCounter>> operatorCountersExprMap = {};
+    StringMap<Counter64*> _counters;
 };
 
 class ValidatorCounters {
@@ -588,29 +639,38 @@ public:
                     str::stream() << "The validator counters are not support for the command: "
                                   << cmdName,
                     validatorCounter != _validatorCounterMap.end());
-            validatorCounter->second->totalCounter.increment();
+            validatorCounter->second->totalCounter.incrementRelaxed();
 
             if (!parsingSucceeded) {
-                validatorCounter->second->failedCounter.increment();
+                validatorCounter->second->failedCounter.incrementRelaxed();
             }
             if (validator.hasField("$jsonSchema")) {
-                validatorCounter->second->jsonSchemaCounter.increment();
+                validatorCounter->second->jsonSchemaCounter.incrementRelaxed();
             }
         }
     }
 
 private:
     struct ValidatorCounter {
-        ValidatorCounter(const StringData name)
-            : totalCounter{"commands." + name + ".validator.total"},
-              failedCounter{"commands." + name + ".validator.failed"},
-              jsonSchemaCounter{"commands." + name + ".validator.jsonSchema"} {}
-        CounterMetric totalCounter;
-        CounterMetric failedCounter;
-        CounterMetric jsonSchemaCounter;
+        explicit ValidatorCounter(StringData name)
+            : totalCounter{makeMetric(name, "total")},
+              failedCounter{makeMetric(name, "failed")},
+              jsonSchemaCounter{makeMetric(name, "jsonSchema")} {}
+
+        ValidatorCounter& operator=(const ValidatorCounter&) = delete;
+        ValidatorCounter(const ValidatorCounter&) = delete;
+
+        static Counter64& makeMetric(StringData name, StringData leaf) {
+            return *MetricBuilder<Counter64>{std::string{"commands."} + name + ".validator." +
+                                             leaf};
+        }
+
+        Counter64& totalCounter;
+        Counter64& failedCounter;
+        Counter64& jsonSchemaCounter;
     };
 
-    StringMap<std::unique_ptr<ValidatorCounter>> _validatorCounterMap = {};
+    StringMap<std::unique_ptr<ValidatorCounter>> _validatorCounterMap;
 };
 
 extern ValidatorCounters validatorCounters;
@@ -625,13 +685,49 @@ extern OperatorCounters operatorCountersGroupAccumulatorExpressions;
 extern OperatorCounters operatorCountersWindowAccumulatorExpressions;
 
 // Track the number of {multi:true} updates.
-extern CounterMetric updateManyCount;
+extern Counter64& updateManyCount;
 // Track the number of deleteMany calls.
-extern CounterMetric deleteManyCount;
+extern Counter64& deleteManyCount;
+// Track the number of targeted updateOne commands on sharded collections.
+extern Counter64& updateOneTargetedShardedCount;
+// Track the number of targeted deleteOne commands on sharded collections.
+extern Counter64& deleteOneTargetedShardedCount;
+// Track the number of targeted findAndModify commands on sharded collections.
+extern Counter64& findAndModifyTargetedShardedCount;
+// Track the number of updateOne commands on unsharded collections.
+extern Counter64& updateOneUnshardedCount;
+// Track the number of deleteOne commands on unsharded collections.
+extern Counter64& deleteOneUnshardedCount;
+// Track the number of findAndModify commands on unsharded collections.
+extern Counter64& findAndModifyUnshardedCount;
+// Track the number of non-targeted updateOne commands on sharded collections
+extern Counter64& updateOneNonTargetedShardedCount;
+// Track the number of non-targeted deleteOne commands on sharded collections
+extern Counter64& deleteOneNonTargetedShardedCount;
+// Track the number of non-targeted findAndModify commands on sharded collections
+extern Counter64& findAndModifyNonTargetedShardedCount;
+// Track the number of non-targeted retryable deleteOne commands (without shard key with _id) on
+// sharded collections.
+extern Counter64& deleteOneWithoutShardKeyWithIdCount;
+// Track the number of non-targeted non-retryable deleteOne commands (without shard key with _id) on
+// sharded collections.
+extern Counter64& nonRetryableDeleteOneWithoutShardKeyWithIdCount;
+// Track the number of non-targeted retryable updateOne commands (without shard key with _id) on
+// sharded collections.
+extern Counter64& updateOneWithoutShardKeyWithIdCount;
+// Track the number of non-targeted non-retryable updateOne commands (without shard key with _id) on
+// sharded collections.
+extern Counter64& nonRetryableUpdateOneWithoutShardKeyWithIdCount;
+// Track the number of retries for retryable non-targeted updateOne commands (without shard key with
+// _id) on sharded collections
+extern Counter64& updateOneWithoutShardKeyWithIdRetryCount;
+// Track the number of retries for retryable non-targeted deleteOne commands (without shard key with
+// _id) on sharded collections
+extern Counter64& deleteOneWithoutShardKeyWithIdRetryCount;
 // Track the number of internal retryable writes
-extern CounterMetric internalRetryableWriteCount;
+extern Counter64& internalRetryableWriteCount;
 // Track the number of external retryable writes
-extern CounterMetric externalRetryableWriteCount;
+extern Counter64& externalRetryableWriteCount;
 // Track the number of internal transactions for retryable writes
-extern CounterMetric retryableInternalTransactionCount;
+extern Counter64& retryableInternalTransactionCount;
 }  // namespace mongo
