@@ -27,15 +27,22 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <cstdint>
+#include <mutex>
+#include <string>
+#include <utility>
+#include <variant>
 
-#include "mongo/db/stats/server_write_concern_metrics.h"
+#include <absl/container/flat_hash_map.h>
 
+#include "mongo/bson/bsonelement.h"
 #include "mongo/db/commands/server_status.h"
-#include "mongo/db/jsobj.h"
 #include "mongo/db/operation_context.h"
+#include "mongo/db/read_write_concern_provenance.h"
 #include "mongo/db/service_context.h"
+#include "mongo/db/stats/server_write_concern_metrics.h"
 #include "mongo/db/stats/server_write_concern_metrics_gen.h"
+#include "mongo/util/decorable.h"
 
 namespace mongo {
 
@@ -108,7 +115,7 @@ BSONObj ServerWriteConcernMetrics::toBSON() const {
 
 void ServerWriteConcernMetrics::WriteConcernCounters::recordWriteConcern(
     const WriteConcernOptions& writeConcernOptions, size_t numOps) {
-    if (auto wMode = stdx::get_if<std::string>(&writeConcernOptions.w)) {
+    if (auto wMode = get_if<std::string>(&writeConcernOptions.w)) {
         if (writeConcernOptions.isMajority()) {
             wMajorityCount += numOps;
             return;
@@ -118,12 +125,12 @@ void ServerWriteConcernMetrics::WriteConcernCounters::recordWriteConcern(
         return;
     }
 
-    if (stdx::holds_alternative<WTags>(writeConcernOptions.w)) {
+    if (holds_alternative<WTags>(writeConcernOptions.w)) {
         // wTags is an internal feature that we don't track metrics for
         return;
     }
 
-    wNumCounts[stdx::get<int64_t>(writeConcernOptions.w)] += numOps;
+    wNumCounts[std::get<int64_t>(writeConcernOptions.w)] += numOps;
 }
 
 void ServerWriteConcernMetrics::WriteConcernMetricsForOperationType::recordWriteConcern(
@@ -150,6 +157,9 @@ void ServerWriteConcernMetrics::WriteConcernMetricsForOperationType::recordWrite
             implicitDefaultWC.recordWriteConcern(writeConcernOptions, numOps);
             notExplicitWCount += numOps;
         } else {
+            if (writeConcernOptions.majorityJFalseOverridden) {
+                majorityJFalseOverriddenCount += numOps;
+            }
             explicitWC.recordWriteConcern(writeConcernOptions, numOps);
         }
     }
@@ -177,6 +187,9 @@ void ServerWriteConcernMetrics::WriteConcernMetricsForOperationType::toBSON(
     BSONObjBuilder* builder) const {
     explicitWC.toBSON(builder);
 
+    builder->append("majorityJFalseOverridden",
+                    static_cast<long long>(majorityJFalseOverriddenCount));
+
     builder->append("none", static_cast<long long>(notExplicitWCount));
     BSONObjBuilder noneBuilder(builder->subobjStart("noneInfo"));
 
@@ -194,7 +207,7 @@ void ServerWriteConcernMetrics::WriteConcernMetricsForOperationType::toBSON(
 namespace {
 class OpWriteConcernCountersSSS : public ServerStatusSection {
 public:
-    OpWriteConcernCountersSSS() : ServerStatusSection("opWriteConcernCounters") {}
+    using ServerStatusSection::ServerStatusSection;
 
     ~OpWriteConcernCountersSSS() override = default;
 
@@ -209,8 +222,9 @@ public:
                             const BSONElement& configElement) const override {
         return ServerWriteConcernMetrics::get(opCtx)->toBSON();
     }
-
-} opWriteConcernCountersSSS;
+};
+auto& opWriteConcernCountersSSS =
+    *ServerStatusSectionBuilder<OpWriteConcernCountersSSS>("opWriteConcernCounters").forShard();
 }  // namespace
 
 }  // namespace mongo
