@@ -27,9 +27,13 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <boost/move/utility_core.hpp>
+
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
 
 #include "mongo/db/stats/single_transaction_stats.h"
+#include "mongo/util/assert_util_core.h"
 
 namespace mongo {
 
