@@ -28,26 +28,49 @@
  */
 
 
-#include "mongo/platform/basic.h"
+#include <absl/container/node_hash_map.h>
+#include <boost/move/utility_core.hpp>
+#include <memory>
+#include <string>
+#include <utility>
+#include <vector>
 
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
 #include "mongo/bson/bsonobj.h"
+#include "mongo/db/basic_types.h"
+#include "mongo/db/basic_types_gen.h"
+#include "mongo/db/catalog/clustered_collection_options_gen.h"
 #include "mongo/db/catalog/clustered_collection_util.h"
 #include "mongo/db/catalog/collection.h"
 #include "mongo/db/catalog/collection_catalog.h"
-#include "mongo/db/catalog/database_holder.h"
+#include "mongo/db/catalog/collection_options.h"
 #include "mongo/db/catalog/index_catalog.h"
+#include "mongo/db/catalog/index_catalog_entry.h"
+#include "mongo/db/catalog_raii.h"
+#include "mongo/db/cluster_role.h"
 #include "mongo/db/db_raii.h"
 #include "mongo/db/index/index_access_method.h"
 #include "mongo/db/index/index_descriptor.h"
+#include "mongo/db/query/collation/collator_interface.h"
 #include "mongo/db/s/balancer_stats_registry.h"
-#include "mongo/db/storage/storage_parameters_gen.h"
-#include "mongo/db/timeseries/bucket_catalog/bucket_catalog.h"
-#include "mongo/db/timeseries/timeseries_stats.h"
-#include "mongo/logv2/log.h"
-#include "mongo/s/sharding_feature_flags_gen.h"
-#include "mongo/stdx/unordered_map.h"
-
+#include "mongo/db/server_options.h"
 #include "mongo/db/stats/storage_stats.h"
+#include "mongo/db/storage/record_store.h"
+#include "mongo/db/timeseries/bucket_catalog/bucket_catalog.h"
+#include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/stdx/unordered_map.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/namespace_string_util.h"
+#include "mongo/util/str.h"
+#include "mongo/util/time_support.h"
+#include "mongo/util/uuid.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kFTDC
 
@@ -87,7 +110,8 @@ const stdx::unordered_map<std::string, StorageStatsGroups> _mapStorageStatsField
 void _appendRecordStats(OperationContext* opCtx,
                         const CollectionPtr& collection,
                         const NamespaceString& collNss,
-                        bool isNamespaceAlwaysUnsharded,
+                        const SerializationContext& serializationCtx,
+                        bool isNamespaceAlwaysUntracked,
                         int scale,
                         bool isTimeseries,
                         BSONObjBuilder* result) {
@@ -98,15 +122,13 @@ void _appendRecordStats(OperationContext* opCtx,
     long long numRecords = collection->numRecords(opCtx);
     if (isTimeseries) {
         BSONObjBuilder bob(result->subobjStart("timeseries"));
-        bob.append("bucketsNs", NamespaceStringUtil::serialize(collNss));
+        bob.append("bucketsNs", NamespaceStringUtil::serialize(collNss, serializationCtx));
         bob.appendNumber("bucketCount", numRecords);
         if (numRecords) {
             bob.append("avgBucketSize", collection->averageObjectSize(opCtx));
         }
         auto& bucketCatalog = timeseries::bucket_catalog::BucketCatalog::get(opCtx);
-        timeseries::bucket_catalog::appendExecutionStats(
-            bucketCatalog, collNss.getTimeseriesViewNamespace(), bob);
-        TimeseriesStats::get(collection.get()).append(&bob);
+        timeseries::bucket_catalog::appendExecutionStats(bucketCatalog, collection->uuid(), bob);
     } else {
         result->appendNumber("count", numRecords);
         if (numRecords) {
@@ -115,7 +137,7 @@ void _appendRecordStats(OperationContext* opCtx,
     }
 
     if (serverGlobalParams.clusterRole.has(ClusterRole::ShardServer) &&
-        !isNamespaceAlwaysUnsharded) {
+        !isNamespaceAlwaysUntracked) {
         result->appendNumber(
             kOrphanCountField,
             BalancerStatsRegistry::get(opCtx)->getCollNumOrphanDocsFromDiskIfNeeded(
@@ -258,22 +280,8 @@ void _appendInProgressIndexesStats(OperationContext* opCtx,
             }
         }
 
-        // (Ignore FCV check): This feature flag doesn't have any upgrade/downgrade concerns.
-        if (feature_flags::gPointInTimeCatalogLookups.isEnabledAndIgnoreFCVUnsafe()) {
-            if (!entry->isReady(opCtx)) {
-                indexBuilds.push_back(descriptor->indexName());
-            }
-        } else {
-            // Not all indexes in the collection stats may be visible or consistent with our
-            // snapshot. For this reason, it is unsafe to check `isReady` on the entry, which
-            // asserts that the index's in-memory state is consistent with our snapshot.
-            if (!entry->isPresentInMySnapshot(opCtx)) {
-                continue;
-            }
-
-            if (!entry->isReadyInMySnapshot(opCtx)) {
-                indexBuilds.push_back(descriptor->indexName());
-            }
+        if (!entry->isReady()) {
+            indexBuilds.push_back(descriptor->indexName());
         }
     }
 
@@ -306,6 +314,7 @@ void _appendTotalSize(OperationContext* opCtx,
 Status appendCollectionStorageStats(OperationContext* opCtx,
                                     const NamespaceString& nss,
                                     const StorageStatsSpec& storageStatsSpec,
+                                    const SerializationContext& serializationCtx,
                                     BSONObjBuilder* result,
                                     const boost::optional<BSONObj>& filterObj) {
     auto scale = storageStatsSpec.getScale().value_or(1);
@@ -365,7 +374,7 @@ Status appendCollectionStorageStats(OperationContext* opCtx,
         result->append("indexSizes", BSONObj());
         result->append("scaleFactor", scale);
         return {ErrorCodes::NamespaceNotFound,
-                "Collection [" + collNss.toString() + "] not found."};
+                "Collection [" + collNss.toStringForErrorMsg() + "] not found."};
     }
 
     // We will parse all 'filterObj' into different groups of data to compute. This groups will be
@@ -401,7 +410,8 @@ Status appendCollectionStorageStats(OperationContext* opCtx,
                 _appendRecordStats(opCtx,
                                    collection,
                                    collNss,
-                                   nss.isNamespaceAlwaysUnsharded(),
+                                   serializationCtx,
+                                   nss.isNamespaceAlwaysUntracked(),
                                    scale,
                                    isTimeseries,
                                    result);
@@ -426,7 +436,7 @@ Status appendCollectionRecordCount(OperationContext* opCtx,
     AutoGetCollectionForReadCommandMaybeLockFree collection(opCtx, nss);
     if (!collection) {
         return {ErrorCodes::NamespaceNotFound,
-                str::stream() << "Collection [" << nss.toString() << "] not found."};
+                str::stream() << "Collection [" << nss.toStringForErrorMsg() << "] not found."};
     }
 
     result->appendNumber("count", static_cast<long long>(collection->numRecords(opCtx)));
