@@ -29,11 +29,22 @@
 
 #pragma once
 
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <cstdint>
 #include <string>
+#include <utility>
 #include <vector>
 
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonobj.h"
 #include "mongo/db/catalog/collection_options.h"
 #include "mongo/db/index/multikey_paths.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/platform/mutex.h"
+#include "mongo/util/uuid.h"
 
 namespace mongo {
 
@@ -69,6 +80,17 @@ public:
             multikeyPaths = other.multikeyPaths;
         }
 
+        IndexMetaData(IndexMetaData&& other)
+            : spec(std::move(other.spec)),
+              ready(other.ready),
+              isBackgroundSecondaryBuild(other.isBackgroundSecondaryBuild),
+              buildUUID(std::move(other.buildUUID)) {
+            // No need to hold mutex on move, there are no concurrent readers while we're moving
+            // the instance.
+            multikey = other.multikey;
+            multikeyPaths = std::move(other.multikeyPaths);
+        }
+
         /**
          * An index is considered present if it has a non-empty 'spec'.
          * Invalid indexes by this definition include default constructed instances and
@@ -150,7 +172,7 @@ public:
          */
         bool eraseIndex(StringData name);
 
-        std::string ns;
+        NamespaceString nss;
         CollectionOptions options;
         // May include empty instances which represent indexes already dropped.
         std::vector<IndexMetaData> indexes;
@@ -163,6 +185,13 @@ public:
         // up will have this flag set to false by default. This will be boost::none if this catalog
         // entry is not representing a time-series collection or if FCV < 5.2.
         boost::optional<bool> timeseriesBucketsMayHaveMixedSchemaData;
+
+        // The flag will be set to false at the time of time-series collection creation if FCV
+        // >= 7.1. For any other collection type and earlier versions the flag will be boost::none.
+        // Thus, if the field is absent, we assume the time-series bucketing parameters have
+        // changed. If a subsequent collMod operation changes either 'bucketRoundingSeconds' or
+        // 'bucketMaxSpanSeconds', we set the flag to true.
+        boost::optional<bool> timeseriesBucketingParametersHaveChanged;
     };
 };
 }  // namespace mongo
