@@ -27,15 +27,32 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/storage/devnull/devnull_kv_engine.h"
-
+#include <boost/move/utility_core.hpp>
+#include <cstddef>
 #include <memory>
+#include <set>
 
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/mutable/damage_vector.h"
+#include "mongo/bson/ordering.h"
+#include "mongo/db/catalog/validate_results.h"
+#include "mongo/db/record_id.h"
+#include "mongo/db/storage/devnull/devnull_kv_engine.h"
 #include "mongo/db/storage/devnull/ephemeral_catalog_record_store.h"
+#include "mongo/db/storage/duplicate_key_error_info.h"
+#include "mongo/db/storage/key_string.h"
+#include "mongo/db/storage/record_data.h"
 #include "mongo/db/storage/record_store.h"
+#include "mongo/db/storage/recovery_unit.h"
+#include "mongo/db/storage/recovery_unit_noop.h"
 #include "mongo/db/storage/sorted_data_interface.h"
+#include "mongo/stdx/unordered_map.h"
+#include "mongo/util/uuid.h"
 
 namespace mongo {
 
@@ -44,10 +61,10 @@ public:
     boost::optional<Record> next() final {
         return {};
     }
-    boost::optional<Record> seekExact(const RecordId& id) final {
+    boost::optional<Record> seek(const RecordId& start, BoundInclusion boundInclusion) override {
         return {};
     }
-    boost::optional<Record> seekNear(const RecordId& id) final {
+    boost::optional<Record> seekExact(const RecordId& id) final {
         return {};
     }
     void save() final {}
@@ -61,26 +78,32 @@ public:
 
 class DevNullRecordStore : public RecordStore {
 public:
-    DevNullRecordStore(StringData ns,
+    DevNullRecordStore(const NamespaceString& nss,
+                       boost::optional<UUID> uuid,
                        StringData identName,
                        const CollectionOptions& options,
                        KeyFormat keyFormat)
-        : RecordStore(ns, identName), _options(options), _keyFormat(keyFormat) {
+        : RecordStore(uuid, identName, options.capped),
+          _options(options),
+          _keyFormat(keyFormat),
+          _ns(nss) {
         _numInserts = 0;
         _dummy = BSON("_id" << 1);
     }
 
-    virtual const char* name() const {
+    const char* name() const override {
         return "devnull";
     }
 
-    virtual void setCappedCallback(CappedCallback*) {}
+    NamespaceString ns(OperationContext* opCtx) const override {
+        return _ns;
+    }
 
-    virtual long long dataSize(OperationContext* opCtx) const {
+    long long dataSize(OperationContext* opCtx) const override {
         return 0;
     }
 
-    virtual long long numRecords(OperationContext* opCtx) const {
+    long long numRecords(OperationContext* opCtx) const override {
         return 0;
     }
 
@@ -88,17 +111,17 @@ public:
         return _options.capped;
     }
 
-    virtual KeyFormat keyFormat() const {
+    KeyFormat keyFormat() const override {
         return _keyFormat;
     }
 
-    virtual int64_t storageSize(OperationContext* opCtx,
-                                BSONObjBuilder* extraInfo = nullptr,
-                                int infoLevel = 0) const {
+    int64_t storageSize(OperationContext* opCtx,
+                        BSONObjBuilder* extraInfo = nullptr,
+                        int infoLevel = 0) const override {
         return 0;
     }
 
-    virtual bool findRecord(OperationContext* opCtx, const RecordId& loc, RecordData* rd) const {
+    bool findRecord(OperationContext* opCtx, const RecordId& loc, RecordData* rd) const override {
         return false;
     }
 
@@ -121,7 +144,7 @@ public:
         return Status::OK();
     }
 
-    virtual bool updateWithDamagesSupported() const {
+    bool updateWithDamagesSupported() const override {
         return false;
     }
 
@@ -133,9 +156,9 @@ public:
         MONGO_UNREACHABLE;
     }
 
-    virtual void printRecordMetadata(OperationContext* opCtx,
-                                     const RecordId& recordId,
-                                     std::set<Timestamp>* recordTimestamps) const {
+    void printRecordMetadata(OperationContext* opCtx,
+                             const RecordId& recordId,
+                             std::set<Timestamp>* recordTimestamps) const override {
         MONGO_UNREACHABLE;
     }
 
@@ -148,17 +171,40 @@ public:
         return Status::OK();
     }
 
-    void doCappedTruncateAfter(OperationContext* opCtx, RecordId end, bool inclusive) override {}
+    Status doRangeTruncate(OperationContext* opCtx,
+                           const RecordId& minRecordId,
+                           const RecordId& maxRecordId,
+                           int64_t hintDataSizeDiff,
+                           int64_t hintNumRecordsDiff) override {
+        return Status::OK();
+    }
 
-    virtual void appendNumericCustomStats(OperationContext* opCtx,
-                                          BSONObjBuilder* result,
-                                          double scale) const {
+    void doCappedTruncateAfter(OperationContext* opCtx,
+                               const RecordId& end,
+                               bool inclusive,
+                               const AboutToDeleteRecordCallback& aboutToDelete) override {}
+
+    void appendNumericCustomStats(OperationContext* opCtx,
+                                  BSONObjBuilder* result,
+                                  double scale) const override {
         result->appendNumber("numInserts", _numInserts);
     }
 
-    virtual void updateStatsAfterRepair(OperationContext* opCtx,
-                                        long long numRecords,
-                                        long long dataSize) {}
+    void updateStatsAfterRepair(OperationContext* opCtx,
+                                long long numRecords,
+                                long long dataSize) override {}
+
+    RecordId getLargestKey(OperationContext* opCtx) const final {
+        return RecordId();
+    }
+
+    void reserveRecordIds(OperationContext* opCtx,
+                          std::vector<RecordId>* out,
+                          size_t nRecords) final {
+        for (size_t i = 0; i < nRecords; i++) {
+            out->push_back(RecordId(i));
+        }
+    };
 
 protected:
     void waitForAllEarlierOplogWritesToBeVisibleImpl(OperationContext* opCtx) const override {}
@@ -168,6 +214,7 @@ private:
     KeyFormat _keyFormat;
     long long _numInserts;
     BSONObj _dummy;
+    NamespaceString _ns;
 };
 
 class DevNullSortedDataBuilderInterface : public SortedDataBuilderInterface {
@@ -177,7 +224,7 @@ class DevNullSortedDataBuilderInterface : public SortedDataBuilderInterface {
 public:
     DevNullSortedDataBuilderInterface() {}
 
-    virtual Status addKey(const KeyString::Value& keyString) {
+    Status addKey(const key_string::Value& keyString) override {
         return Status::OK();
     }
 };
@@ -186,82 +233,87 @@ class DevNullSortedDataInterface : public SortedDataInterface {
 public:
     DevNullSortedDataInterface(StringData identName)
         : SortedDataInterface(identName,
-                              KeyString::Version::kLatestVersion,
+                              key_string::Version::kLatestVersion,
                               Ordering::make(BSONObj()),
                               KeyFormat::Long) {}
 
-    virtual ~DevNullSortedDataInterface() {}
+    ~DevNullSortedDataInterface() override {}
 
-    virtual std::unique_ptr<SortedDataBuilderInterface> makeBulkBuilder(OperationContext* opCtx,
-                                                                        bool dupsAllowed) {
+    std::unique_ptr<SortedDataBuilderInterface> makeBulkBuilder(OperationContext* opCtx,
+                                                                bool dupsAllowed) override {
         return {};
     }
 
-    virtual Status insert(OperationContext* opCtx,
-                          const KeyString::Value& keyString,
-                          bool dupsAllowed) {
+    Status insert(OperationContext* opCtx,
+                  const key_string::Value& keyString,
+                  bool dupsAllowed,
+                  IncludeDuplicateRecordId includeDuplicateRecordId) override {
         return Status::OK();
     }
 
-    virtual void unindex(OperationContext* opCtx,
-                         const KeyString::Value& keyString,
-                         bool dupsAllowed) {}
+    void unindex(OperationContext* opCtx,
+                 const key_string::Value& keyString,
+                 bool dupsAllowed) override {}
 
-    virtual Status dupKeyCheck(OperationContext* opCtx, const KeyString::Value& keyString) {
+    Status dupKeyCheck(OperationContext* opCtx, const key_string::Value& keyString) override {
         return Status::OK();
     }
 
-    virtual boost::optional<RecordId> findLoc(OperationContext* opCtx,
-                                              const KeyString::Value& keyString) const override {
+    boost::optional<RecordId> findLoc(OperationContext* opCtx,
+                                      StringData keyString) const override {
         return boost::none;
     }
 
-    virtual void fullValidate(OperationContext* opCtx,
-                              long long* numKeysOut,
-                              IndexValidateResults* fullResults) const {}
+    IndexValidateResults validate(OperationContext* opCtx, bool full) const override {
+        return IndexValidateResults{};
+    }
 
-    virtual bool appendCustomStats(OperationContext* opCtx,
-                                   BSONObjBuilder* output,
-                                   double scale) const {
+    bool appendCustomStats(OperationContext* opCtx,
+                           BSONObjBuilder* output,
+                           double scale) const override {
         return false;
     }
 
-    virtual long long getSpaceUsedBytes(OperationContext* opCtx) const {
+    long long getSpaceUsedBytes(OperationContext* opCtx) const override {
         return 0;
     }
 
-    virtual long long getFreeStorageBytes(OperationContext* opCtx) const {
+    long long getFreeStorageBytes(OperationContext* opCtx) const override {
         return 0;
     }
 
-    virtual bool isEmpty(OperationContext* opCtx) {
+    bool isEmpty(OperationContext* opCtx) override {
         return true;
     }
 
-    virtual void printIndexEntryMetadata(OperationContext* opCtx,
-                                         const KeyString::Value& keyString) const {}
+    int64_t numEntries(OperationContext* opCtx) const override {
+        return 0;
+    }
 
-    virtual std::unique_ptr<SortedDataInterface::Cursor> newCursor(OperationContext* opCtx,
-                                                                   bool isForward) const {
+    void printIndexEntryMetadata(OperationContext* opCtx,
+                                 const key_string::Value& keyString) const override {}
+
+    std::unique_ptr<SortedDataInterface::Cursor> newCursor(OperationContext* opCtx,
+                                                           bool isForward) const override {
         return {};
     }
 
-    virtual Status initAsEmpty(OperationContext* opCtx) {
+    Status initAsEmpty(OperationContext* opCtx) override {
         return Status::OK();
     }
-
-    void insertWithRecordIdInValue_forTest(OperationContext* opCtx,
-                                           const KeyString::Value& keyString,
-                                           RecordId rid) override {
-        MONGO_UNREACHABLE;
-    }
 };
 
 DevNullKVEngine::DevNullKVEngine() {
     _mockBackupBlocks.push_back(BackupBlock(/*opCtx=*/nullptr,
-                                            "filename.wt",
-                                            /*identToNamespaceAndUUIDMap=*/{},
-                                            /*checkpointTimestamp=*/boost::none));
+                                            /*nss=*/boost::none,
+                                            /*uuid=*/boost::none,
+                                            "filename.wt"));
+}
+
+DevNullKVEngine::~DevNullKVEngine() = default;
+
+RecoveryUnit* DevNullKVEngine::newRecoveryUnit() {
+    return new RecoveryUnitNoop();
 }
 
 std::unique_ptr<RecordStore> DevNullKVEngine::getRecordStore(OperationContext* opCtx,
@@ -269,15 +321,26 @@ std::unique_ptr<RecordStore> DevNullKVEngine::getRecordStore(OperationContext* o
                                                              StringData ident,
                                                              const CollectionOptions& options) {
     if (ident == "_mdb_catalog") {
-        return std::make_unique<EphemeralForTestRecordStore>(nss.ns(), ident, &_catalogInfo);
+        return std::make_unique<EphemeralForTestRecordStore>(
+            nss, options.uuid, ident, &_catalogInfo);
     }
-    return std::make_unique<DevNullRecordStore>(nss.ns(), ident, options, KeyFormat::Long);
+    return std::make_unique<DevNullRecordStore>(nss, options.uuid, ident, options, KeyFormat::Long);
+}
+
+std::unique_ptr<RecordStore> DevNullKVEngine::getTemporaryRecordStore(OperationContext* opCtx,
+                                                                      StringData ident,
+                                                                      KeyFormat keyFormat) {
+    return makeTemporaryRecordStore(opCtx, ident, keyFormat);
 }
 
 std::unique_ptr<RecordStore> DevNullKVEngine::makeTemporaryRecordStore(OperationContext* opCtx,
                                                                        StringData ident,
                                                                        KeyFormat keyFormat) {
-    return std::make_unique<DevNullRecordStore>("" /* ns */, ident, CollectionOptions(), keyFormat);
+    return std::make_unique<DevNullRecordStore>(NamespaceString::kEmpty /* ns */,
+                                                boost::none /* uuid */,
+                                                ident,
+                                                CollectionOptions(),
+                                                keyFormat);
 }
 
 std::unique_ptr<SortedDataInterface> DevNullKVEngine::getSortedDataInterface(
@@ -299,14 +362,17 @@ public:
         _exhaustCursor = false;
     };
 
-    ~StreamingCursorImpl() = default;
+    ~StreamingCursorImpl() override = default;
 
     BSONObj getMetadataObject(UUID backupId) {
         return BSONObj();
     }
 
+    void setCatalogEntries(stdx::unordered_map<std::string, std::pair<NamespaceString, UUID>>
+                               identsToNsAndUUID) override {}
+
     StatusWith<std::deque<BackupBlock>> getNextBatch(OperationContext* opCtx,
-                                                     const std::size_t batchSize) {
+                                                     const std::size_t batchSize) override {
         if (_exhaustCursor) {
             std::deque<BackupBlock> emptyVector;
             return emptyVector;
@@ -323,9 +389,7 @@ private:
 }  // namespace
 
 StatusWith<std::unique_ptr<StorageEngine::StreamingCursor>> DevNullKVEngine::beginNonBlockingBackup(
-    OperationContext* opCtx,
-    boost::optional<Timestamp> checkpointTimestamp,
-    const StorageEngine::BackupOptions& options) {
+    OperationContext* opCtx, const StorageEngine::BackupOptions& options) {
     return std::make_unique<StreamingCursorImpl>(options, _mockBackupBlocks);
 }
 
