@@ -29,23 +29,40 @@
 
 #pragma once
 
-#include <limits>
-
 #include <absl/hash/hash.h>
+#include <boost/container/flat_set.hpp>
+#include <boost/optional/optional.hpp>
+#include <cstdint>
+#include <cstring>
+#include <functional>
+#include <iosfwd>
+#include <limits>
+#include <string>
+#include <type_traits>
+#include <utility>
 
+#include "mongo/base/data_type_endian.h"
 #include "mongo/base/static_assert.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
 #include "mongo/bson/bsonelement_comparator_interface.h"
 #include "mongo/bson/bsonmisc.h"
 #include "mongo/bson/bsonobj.h"
 #include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes_util.h"
+#include "mongo/bson/oid.h"
 #include "mongo/bson/ordering.h"
 #include "mongo/bson/timestamp.h"
+#include "mongo/bson/util/builder.h"
+#include "mongo/bson/util/builder_fwd.h"
 #include "mongo/db/record_id.h"
 #include "mongo/db/storage/key_format.h"
 #include "mongo/platform/decimal128.h"
 #include "mongo/util/assert_util.h"
-
-#include <boost/container/flat_set.hpp>
+#include "mongo/util/bufreader.h"
+#include "mongo/util/shared_buffer_fragment.h"
+#include "mongo/util/str.h"
+#include "mongo/util/time_support.h"
 
 namespace mongo {
 
@@ -53,7 +70,7 @@ namespace sbe::value {
 class ValueBuilder;
 }
 
-namespace KeyString {
+namespace key_string {
 
 enum class Version : uint8_t { V0 = 0, V1 = 1, kLatestVersion = V1 };
 
@@ -77,8 +94,8 @@ MONGO_STATIC_ASSERT(RecordId::kBigStrMaxSize < 1 << (7 * kRecordIdStrEncodedSize
 class TypeBits {
 public:
     // See comments in getBuffer() about short/long encoding schemes.
-    static const uint8_t kMaxBytesForShortEncoding = 127;
-    static const uint8_t kPrefixBytes = 5;
+    static const int8_t kMaxBytesForShortEncoding = 127;
+    static const int8_t kPrefixBytes = 5;
     static const uint8_t kStoredDecimalExponentBits = 6;
     static const uint32_t kStoredDecimalExponentMask = (1U << kStoredDecimalExponentBits) - 1;
 
@@ -151,13 +168,13 @@ public:
         // Case 3 & 4: size byte(s) + data bytes.
         return isLongEncoding() ? _buf.buf() : (getDataBuffer() - 1);
     }
-    size_t getSize() const {
+    int32_t getSize() const {
         if (_isAllZeros) {  // Case 1
             dassert(getDataBufferLen() == 0 || getDataBuffer()[0] == 0);
             return 1;
         }
 
-        uint32_t rawSize = getDataBufferLen();
+        int32_t rawSize = getDataBufferLen();
         dassert(rawSize >= 1);                      // 0 should be handled as isAllZeros.
         if (rawSize > kMaxBytesForShortEncoding) {  // Case 4
             return rawSize + kPrefixBytes;
@@ -175,7 +192,7 @@ public:
     }
 
     //
-    // Everything below is only for use by KeyString::Builder.
+    // Everything below is only for use by key_string::Builder.
     //
 
     // Note: No space is used if all bits are 0 so the most common cases should be 0x0.
@@ -244,6 +261,7 @@ public:
         virtual uint8_t readStringLike() = 0;
         virtual uint8_t readNumeric() = 0;
         virtual uint8_t readZero() = 0;
+        virtual Version version() const = 0;
 
         // Given a decimal zero type between kDecimalZero0xxx and kDecimal5xxx, read the
         // remaining 12 bits and return which of the 24576 decimal zeros to produce.
@@ -262,8 +280,14 @@ public:
         /**
          * Passed in TypeBits must outlive this Reader instance.
          */
-        explicit Reader(const TypeBits& typeBits) : _curBit(0), _typeBits(typeBits) {}
-        ~Reader() = default;
+        explicit Reader(const char* data, int32_t size, Version version, bool isAllZeroes)
+            : _data(data), _size(size), _curBit(0), _version(version), _isAllZeros(isAllZeroes) {}
+        explicit Reader(const TypeBits& typeBits)
+            : Reader(typeBits.getDataBuffer(),
+                     typeBits.getDataBufferLen(),
+                     typeBits.version,
+                     typeBits._isAllZeros) {}
+        ~Reader() override = default;
 
         uint8_t readStringLike() final;
         uint8_t readNumeric() final;
@@ -271,12 +295,19 @@ public:
         uint32_t readDecimalZero(uint8_t zeroType) final;
         uint8_t readDecimalExponent() final;
 
+        Version version() const override {
+            return _version;
+        }
+
     protected:
         uint8_t readBit() final;
 
     private:
-        uint32_t _curBit;
-        const TypeBits& _typeBits;
+        const char* _data;
+        int32_t _size;
+        int32_t _curBit;
+        Version _version;
+        bool _isAllZeros;
     };
 
     /**
@@ -291,7 +322,7 @@ public:
     class ExplainReader : public ReaderBase {
     public:
         explicit ExplainReader(ReaderBase& reader) : _reader(reader){};
-        ~ExplainReader() = default;
+        ~ExplainReader() override = default;
 
         uint8_t readStringLike() final;
         uint8_t readNumeric() final;
@@ -299,6 +330,10 @@ public:
         uint32_t readDecimalZero(uint8_t zeroType) final;
         uint8_t readDecimalExponent() final;
 
+        Version version() const final {
+            return _reader.version();
+        }
+
         std::string getExplain() const {
             return _explain.ss.str();
         }
@@ -313,12 +348,27 @@ public:
         str::stream _explain;
     };
 
+    /**
+     * Get a Reader on top of a buffer without copying it. This reader can then be passed to
+     * toBson() in place of a TypeBits object that owns a buffer copy.
+     * The position pointer of `buf` is advanced to the end of the TypeBits.
+     */
+    static Reader getReaderFromBuffer(Version version, BufReader* buf) {
+        if (!buf->remaining()) {
+            // This means AllZeros state was encoded as an empty buffer.
+            return Reader(nullptr, 0, version, true);
+        }
+
+        int32_t size = readSizeFromBuffer(buf);
+        return Reader(static_cast<const char*>(buf->skip(size)), size, version, size == 0);
+    }
+
     Version version;
 
 private:
-    static uint32_t readSizeFromBuffer(BufReader* reader);
+    static int32_t readSizeFromBuffer(BufReader* reader);
 
-    void setRawSize(uint32_t size);
+    void setRawSize(int32_t size);
 
     const char* getDataBuffer() const {
         return _buf.buf() + kPrefixBytes;
@@ -326,7 +376,7 @@ private:
     char* getDataBuffer() {
         return _buf.buf() + kPrefixBytes;
     }
-    uint32_t getDataBufferLen() const {
+    int32_t getDataBufferLen() const {
         return _buf.len() - kPrefixBytes;
     }
 
@@ -354,7 +404,7 @@ private:
 
 
 /**
- * Value owns a buffer that corresponds to a completely generated KeyString::Builder with the
+ * Value owns a buffer that corresponds to a completely generated key_string::Builder with the
  * TypeBits appended.
  *
  * To optimize copy performance and space requirements of this structure, the buffer will contain
@@ -382,8 +432,10 @@ public:
         return *this;
     }
 
+    static Value makeValue(Version version, StringData ks, StringData rid, StringData typeBits);
+
     /**
-     * Compare with another KeyString::Value or Builder.
+     * Compare with another key_string::Value or Builder.
      */
     template <class T>
     int compare(const T& other) const;
@@ -391,13 +443,18 @@ public:
     int compareWithTypeBits(const Value& other) const;
 
     /**
-     * Compare with another KeyString::Value or Builder, ignoring the RecordId part of both.
+     * Compare with another key_string::Value or Builder, ignoring the RecordId part of both.
      */
     template <class T>
     int compareWithoutRecordIdLong(const T& other) const;
     template <class T>
     int compareWithoutRecordIdStr(const T& other) const;
 
+    /**
+     * Compare with another key_string::Value, ignoring the Discriminator byte of both.
+     */
+    int compareWithoutDiscriminator(const Value& other) const;
+
     // Returns the size of the stored KeyString.
     size_t getSize() const {
         return _ksSize;
@@ -419,6 +476,10 @@ public:
         return TypeBits::fromBuffer(_version, &reader);
     }
 
+    StringData getTypeBitsView() const {
+        return {_buffer.get() + _ksSize, _buffer.size() - _ksSize};
+    }
+
     // Compute hash over key
     uint64_t hash(uint64_t seed = 0) const {
         return absl::hash_internal::CityHash64WithSeed(_buffer.get(), _buffer.size(), seed);
@@ -446,7 +507,7 @@ public:
     void serializeWithoutRecordIdStr(BufBuilder& buf) const;
 
     // Deserialize the Value from a serialized format.
-    static Value deserialize(BufReader& buf, KeyString::Version version) {
+    static Value deserialize(BufReader& buf, key_string::Version version) {
         const int32_t sizeOfKeystring = buf.read<LittleEndian<int32_t>>();
         const void* keystringPtr = buf.skip(sizeOfKeystring);
 
@@ -485,7 +546,7 @@ public:
     // aggregate and free unused memory periodically.
     int memUsageForSorter() const {
         invariant(!_buffer.isShared(),
-                  "Cannot obtain memory usage from shared buffer on KeyString::Value");
+                  "Cannot obtain memory usage from shared buffer on key_string::Value");
         return sizeof(Value) + _buffer.underlyingCapacity();
     }
 
@@ -500,6 +561,8 @@ public:
 
     size_t getApproximateSize() const;
 
+    int computeElementCount(Ordering ord) const;
+
 private:
     Version _version;
     // _ksSize is the total length that the KeyString takes up in the buffer.
@@ -550,11 +613,18 @@ public:
     explicit BuilderBase(Version version)
         : BuilderBase(version, ALL_ASCENDING, Discriminator::kInclusive) {}
 
+    /**
+     * Constructs a builder given an object and ordering, stripping out top-level field names.
+     * Appends the given record id to the end.
+     */
     BuilderBase(Version version, const BSONObj& obj, Ordering ord, const RecordId& recordId)
         : BuilderBase(version, ord) {
         resetToKey(obj, ord, recordId);
     }
 
+    /**
+     * Constructs a builder given an object and ordering, stripping out top-level field names.
+     */
     BuilderBase(Version version,
                 const BSONObj& obj,
                 Ordering ord,
@@ -598,11 +668,29 @@ public:
         return {version, _buffer().len(), SharedBufferFragment(newBuf.release(), newBufLen)};
     }
 
+    /**
+     * Ends the build state and returns the buffer held by the builder.
+     * The buffer holds the KeyString with no RecordId or TypeBits encoded, suitable to pass into
+     * SortedDataInterface::seek().
+     * Caller must only use the returned buffer within the lifetime of the builder.
+     *
+     * If 'discriminator' is specified, then it overrides the previously set _discriminator.
+     */
+    StringData finishAndGetBuffer(boost::optional<Discriminator> discriminator = boost::none) {
+        invariant(_state == BuildState::kAppendingBSONElements || _state == BuildState::kEndAdded);
+        if (discriminator) {
+            _discriminator = *discriminator;
+        }
+        _doneAppending();
+        return {_buffer().buf(), static_cast<StringData::size_type>(_buffer().len())};
+    }
+
     void appendRecordId(const RecordId& loc);
     void appendTypeBits(const TypeBits& bits);
 
-    /*
-     * Function 'f' will be applied to all string elements contained in 'elem'.
+    /**
+     * Appends the given element, discarding the field name. The transformation function will be
+     * applied to all string values contained in the given element.
      */
     void appendBSONElement(const BSONElement& elem, const StringTransformFn& f = nullptr);
 
@@ -649,10 +737,19 @@ public:
         _transition(BuildState::kEmpty);
     }
 
+    /**
+     * Resets the state to the given object and ordering, stripping out top-level field names.
+     * Appends the given record id to the end.
+     */
     void resetToKey(const BSONObj& obj, Ordering ord, const RecordId& recordId);
+
+    /**
+     * Resets the state to the given object and ordering, stripping out top-level field names.
+     */
     void resetToKey(const BSONObj& obj,
                     Ordering ord,
                     Discriminator discriminator = Discriminator::kInclusive);
+
     void resetFromBuffer(const void* buffer, size_t size) {
         _buffer().reset();
         memcpy(_buffer().skip(size), buffer, size);
@@ -684,13 +781,13 @@ public:
     }
 
     /**
-     * Compare with another KeyString::Value or Builder.
+     * Compare with another key_string::Value or Builder.
      */
     template <class T>
     int compare(const T& other) const;
 
     /**
-     * Compare with another KeyString::Value or Builder, ignoring the RecordId part of both.
+     * Compare with another key_string::Value or Builder, ignoring the RecordId part of both.
      */
     template <class T>
     int compareWithoutRecordIdLong(const T& other) const;
@@ -709,6 +806,9 @@ public:
     const Version version;
 
 protected:
+    /**
+     * Appends all elements in the given object, stripping out top-level field names.
+     */
     void _appendAllElementsForIndexing(const BSONObj& obj, Discriminator discriminator);
 
     void _appendBool(bool val, bool invert);
@@ -1013,7 +1113,7 @@ inline typename std::enable_if<isKeyString<T>::value, std::ostream&>::type opera
  * Given a KeyString which may or may not have a RecordId, returns the length of the section without
  * the RecordId. More expensive than sizeWithoutRecordId(Long|Str)AtEnd
  */
-size_t getKeySize(const char* buffer, size_t len, Ordering ord, const TypeBits& typeBits);
+size_t getKeySize(const char* buffer, size_t len, Ordering ord, Version version);
 
 /**
  * Decodes the given KeyString buffer into it's BSONObj representation. This is marked as
@@ -1024,9 +1124,15 @@ size_t getKeySize(const char* buffer, size_t len, Ordering ord, const TypeBits&
  */
 BSONObj toBson(StringData data, Ordering ord, const TypeBits& types);
 BSONObj toBson(const char* buffer, size_t len, Ordering ord, const TypeBits& types) noexcept;
+BSONObj toBson(StringData data, Ordering ord, StringData typeBitsRawBuffer, Version version);
 BSONObj toBsonSafe(const char* buffer, size_t len, Ordering ord, const TypeBits& types);
 void toBsonSafe(
     const char* buffer, size_t len, Ordering ord, const TypeBits& types, BSONObjBuilder& builder);
+void toBsonSafe(const char* buffer,
+                size_t len,
+                Ordering ord,
+                TypeBits::ReaderBase& typeBitsReader,
+                BSONObjBuilder& builder);
 Discriminator decodeDiscriminator(const char* buffer,
                                   size_t len,
                                   Ordering ord,
@@ -1051,14 +1157,25 @@ RecordId decodeRecordIdStrAtEnd(const void* buf, size_t size);
 /**
  * Given a KeyString with a RecordId in the long format, returns the length of the section without
  * the RecordId.
+ * If a RecordId pointer is provided, also decode the RecordId into the pointer.
  */
-size_t sizeWithoutRecordIdLongAtEnd(const void* bufferRaw, size_t bufSize);
+int32_t sizeWithoutRecordIdLongAtEnd(const void* bufferRaw,
+                                     size_t bufSize,
+                                     RecordId* recordId = nullptr);
 
 /**
  * Given a KeyString with a RecordId in the string format, returns the length of the section without
  * the RecordId.
+ * If a RecordId pointer is provided, also decode the RecordId into the pointer.
  */
-size_t sizeWithoutRecordIdStrAtEnd(const void* bufferRaw, size_t bufSize);
+int32_t sizeWithoutRecordIdStrAtEnd(const void* bufferRaw,
+                                    size_t bufSize,
+                                    RecordId* recordId = nullptr);
+
+/**
+ * Given a KeyString, returns the length of the section without the discriminator.
+ */
+int32_t sizeWithoutDiscriminatorAtEnd(const void* bufferRaw, size_t bufSize);
 
 /**
  * Decodes a RecordId, consuming all bytes needed from reader.
@@ -1087,18 +1204,18 @@ void appendSingleFieldToBSONAs(const char* buf,
                                int len,
                                StringData fieldName,
                                BSONObjBuilder* builder,
-                               Version version = KeyString::Version::kLatestVersion);
+                               Version version = key_string::Version::kLatestVersion);
 
 template <class BufferT>
 template <class T>
 int BuilderBase<BufferT>::compare(const T& other) const {
-    return KeyString::compare(getBuffer(), other.getBuffer(), getSize(), other.getSize());
+    return key_string::compare(getBuffer(), other.getBuffer(), getSize(), other.getSize());
 }
 
 template <class BufferT>
 template <class T>
 int BuilderBase<BufferT>::compareWithoutRecordIdLong(const T& other) const {
-    return KeyString::compare(
+    return key_string::compare(
         getBuffer(),
         other.getBuffer(),
         !isEmpty() ? sizeWithoutRecordIdLongAtEnd(getBuffer(), getSize()) : 0,
@@ -1108,7 +1225,7 @@ int BuilderBase<BufferT>::compareWithoutRecordIdLong(const T& other) const {
 template <class BufferT>
 template <class T>
 int BuilderBase<BufferT>::compareWithoutRecordIdStr(const T& other) const {
-    return KeyString::compare(
+    return key_string::compare(
         getBuffer(),
         other.getBuffer(),
         !isEmpty() ? sizeWithoutRecordIdStrAtEnd(getBuffer(), getSize()) : 0,
@@ -1117,12 +1234,12 @@ int BuilderBase<BufferT>::compareWithoutRecordIdStr(const T& other) const {
 
 template <class T>
 int Value::compare(const T& other) const {
-    return KeyString::compare(getBuffer(), other.getBuffer(), getSize(), other.getSize());
+    return key_string::compare(getBuffer(), other.getBuffer(), getSize(), other.getSize());
 }
 
 template <class T>
 int Value::compareWithoutRecordIdLong(const T& other) const {
-    return KeyString::compare(
+    return key_string::compare(
         getBuffer(),
         other.getBuffer(),
         !isEmpty() ? sizeWithoutRecordIdLongAtEnd(getBuffer(), getSize()) : 0,
@@ -1131,7 +1248,7 @@ int Value::compareWithoutRecordIdLong(const T& other) const {
 
 template <class T>
 int Value::compareWithoutRecordIdStr(const T& other) const {
-    return KeyString::compare(
+    return key_string::compare(
         getBuffer(),
         other.getBuffer(),
         !isEmpty() ? sizeWithoutRecordIdStrAtEnd(getBuffer(), getSize()) : 0,
@@ -1167,8 +1284,8 @@ std::string explain(const char* buffer,
                     const TypeBits& typeBits,
                     boost::optional<KeyFormat> keyFormat);
 
-}  // namespace KeyString
+}  // namespace key_string
 
-using KeyStringSet = boost::container::flat_set<KeyString::Value>;
+using KeyStringSet = boost::container::flat_set<key_string::Value>;
 
 }  // namespace mongo
