@@ -27,16 +27,35 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <cstdint>
+#include <functional>
+#include <vector>
 
-#include <memory>
+#include <boost/optional/optional.hpp>
 
-#include "mongo/db/concurrency/lock_state.h"
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/db/catalog/collection_options.h"
+#include "mongo/db/client.h"
+#include "mongo/db/index/index_descriptor.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/service_context.h"
 #include "mongo/db/service_context_test_fixture.h"
+#include "mongo/db/storage/column_store.h"
 #include "mongo/db/storage/ident.h"
+#include "mongo/db/storage/key_format.h"
 #include "mongo/db/storage/kv/kv_drop_pending_ident_reaper.h"
+#include "mongo/db/storage/record_store.h"
+#include "mongo/db/storage/recovery_unit.h"
+#include "mongo/db/storage/sorted_data_interface.h"
+#include "mongo/unittest/assert.h"
 #include "mongo/unittest/death_test.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/duration.h"
+#include "mongo/util/str.h"
 
 namespace mongo {
 namespace {
@@ -48,7 +67,13 @@ class KVEngineMock : public KVEngine {
 public:
     Status dropIdent(RecoveryUnit* ru,
                      StringData ident,
-                     StorageEngine::DropIdentCallback&& onDrop) override;
+                     const StorageEngine::DropIdentCallback& onDrop) override;
+
+    Status dropIdentSynchronous(RecoveryUnit* ru,
+                                StringData ident,
+                                const StorageEngine::DropIdentCallback& onDrop) override {
+        return dropIdent(ru, ident, onDrop);
+    }
 
     void dropIdentForImport(OperationContext* opCtx, StringData ident) override {}
 
@@ -70,6 +95,23 @@ public:
         const IndexDescriptor* desc) override {
         return nullptr;
     }
+
+    Status createColumnStore(OperationContext* opCtx,
+                             const NamespaceString& ns,
+                             const CollectionOptions& collOptions,
+                             StringData ident,
+                             const IndexDescriptor* desc) override {
+        MONGO_UNREACHABLE;
+    }
+
+    std::unique_ptr<ColumnStore> getColumnStore(OperationContext* opCtx,
+                                                const NamespaceString& nss,
+                                                const CollectionOptions& collOptions,
+                                                StringData ident,
+                                                const IndexDescriptor*) override {
+        return nullptr;
+    }
+
     Status createRecordStore(OperationContext* opCtx,
                              const NamespaceString& nss,
                              StringData ident,
@@ -77,6 +119,13 @@ public:
                              KeyFormat keyFormat) override {
         return Status::OK();
     }
+
+    std::unique_ptr<RecordStore> getTemporaryRecordStore(OperationContext* opCtx,
+                                                         StringData ident,
+                                                         KeyFormat keyFormat) override {
+        return {};
+    }
+
     std::unique_ptr<RecordStore> makeTemporaryRecordStore(OperationContext* opCtx,
                                                           StringData ident,
                                                           KeyFormat keyFormat) override {
@@ -98,9 +147,7 @@ public:
     Status repairIdent(OperationContext* opCtx, StringData ident) override {
         return Status::OK();
     }
-    bool isDurable() const override {
-        return false;
-    }
+
     bool isEphemeral() const override {
         return false;
     }
@@ -127,11 +174,11 @@ public:
         return Timestamp();
     }
 
-    boost::optional<Timestamp> getRecoveryTimestamp() const {
+    boost::optional<Timestamp> getRecoveryTimestamp() const override {
         return boost::none;
     }
 
-    void setPinnedOplogTimestamp(const Timestamp& pinnedTimestamp) {}
+    void setPinnedOplogTimestamp(const Timestamp& pinnedTimestamp) override {}
 
     void dump() const override {}
 
@@ -140,12 +187,14 @@ public:
 
     // Override to modify dropIdent() behavior.
     using DropIdentFn = std::function<Status(RecoveryUnit*, StringData)>;
-    DropIdentFn dropIdentFn = [](RecoveryUnit*, StringData) { return Status::OK(); };
+    DropIdentFn dropIdentFn = [](RecoveryUnit*, StringData) {
+        return Status::OK();
+    };
 };
 
 Status KVEngineMock::dropIdent(RecoveryUnit* ru,
                                StringData ident,
-                               StorageEngine::DropIdentCallback&& onDrop) {
+                               const StorageEngine::DropIdentCallback& onDrop) {
     auto status = dropIdentFn(ru, ident);
     if (status.isOK()) {
         droppedIdents.push_back(ident.toString());
@@ -239,7 +288,6 @@ TEST_F(KVDropPendingIdentReaperTest,
     // getAllIdentNames() returns a set of drop-pending idents known to the reaper.
     auto dropPendingIdents = reaper.getAllIdentNames();
     ASSERT_EQUALS(2U, dropPendingIdents.size());
-    ASSERT_EQUALS(2U, reaper.getNumIdents());
     ASSERT(dropPendingIdents.find(identName1) != dropPendingIdents.cend());
     ASSERT(dropPendingIdents.find(identName2) != dropPendingIdents.cend());
 
@@ -258,6 +306,18 @@ TEST_F(KVDropPendingIdentReaperTest,
     ASSERT_EQUALS(identName2, engine->droppedIdents.back());
 }
 
+
+TEST_F(KVDropPendingIdentReaperTest, AddDropPendingIdentSupportsExactDuplicates) {
+    Timestamp dropTimestamp{Seconds(100), 0};
+
+    KVDropPendingIdentReaper reaper(getEngine());
+
+    std::shared_ptr<Ident> ident = std::make_shared<Ident>("myident");
+    reaper.addDropPendingIdent(dropTimestamp, ident);
+    reaper.addDropPendingIdent(dropTimestamp, ident);
+    ASSERT_EQ(reaper.getNumIdents(), 1);
+}
+
 TEST_F(KVDropPendingIdentReaperTest,
        DropIdentsOlderThanDropsIdentsWithDropTimestampsBeforeOldestTimestamp) {
     auto opCtx = makeOpCtx();
@@ -362,6 +422,77 @@ TEST_F(KVDropPendingIdentReaperTest, DropIdentsOlderThanSkipsIdentsStillReferenc
     ASSERT_FALSE(reaper.getEarliestDropTimestamp());
 }
 
+TEST_F(KVDropPendingIdentReaperTest, MarkUnknownIdentInUse) {
+    const std::string identName = "ident";
+    auto engine = getEngine();
+
+    KVDropPendingIdentReaper reaper(engine);
+
+    std::shared_ptr<Ident> newIdent = reaper.markIdentInUse(identName);
+    ASSERT(!newIdent);
+}
+
+TEST_F(KVDropPendingIdentReaperTest, MarkUnexpiredIdentInUse) {
+    const std::string identName = "ident";
+    const Timestamp dropTimestamp = Timestamp(50, 50);
+    auto engine = getEngine();
+
+    KVDropPendingIdentReaper reaper(engine);
+
+    // The reaper will not have an expired reference to the ident.
+    std::shared_ptr<Ident> ident = std::make_shared<Ident>(identName);
+    reaper.addDropPendingIdent(dropTimestamp, ident);
+
+    ASSERT_EQUALS(dropTimestamp, *reaper.getEarliestDropTimestamp());
+
+    // Marking an unexpired ident as in-use will return a shared_ptr to that ident.
+    std::shared_ptr<Ident> newIdent = reaper.markIdentInUse(identName);
+    ASSERT_EQ(ident, newIdent);
+    ASSERT_EQ(2, ident.use_count());
+
+    auto opCtx = makeOpCtx();
+    reaper.dropIdentsOlderThan(opCtx.get(), Timestamp::max());
+    ASSERT_EQUALS(0U, engine->droppedIdents.size());
+
+    // Remove the references to the ident so that the reaper can drop it the next time.
+    ident.reset();
+    newIdent.reset();
+
+    reaper.dropIdentsOlderThan(opCtx.get(), Timestamp::max());
+    ASSERT_EQUALS(1U, engine->droppedIdents.size());
+    ASSERT_EQUALS(identName, engine->droppedIdents.front());
+}
+
+TEST_F(KVDropPendingIdentReaperTest, MarkExpiredIdentInUse) {
+    const std::string identName = "ident";
+    const Timestamp dropTimestamp = Timestamp(50, 50);
+    auto engine = getEngine();
+
+    KVDropPendingIdentReaper reaper(engine);
+    {
+        // The reaper will have an expired weak_ptr to the ident.
+        std::shared_ptr<Ident> ident = std::make_shared<Ident>(identName);
+        reaper.addDropPendingIdent(dropTimestamp, ident);
+    }
+
+    ASSERT_EQUALS(dropTimestamp, *reaper.getEarliestDropTimestamp());
+
+    // Mark the ident as in use to prevent the reaper from dropping it.
+    std::shared_ptr<Ident> ident = reaper.markIdentInUse(identName);
+    ASSERT_EQ(1, ident.use_count());
+
+    auto opCtx = makeOpCtx();
+    reaper.dropIdentsOlderThan(opCtx.get(), Timestamp::max());
+    ASSERT_EQUALS(0U, engine->droppedIdents.size());
+
+    // Remove the reference to the ident so that the reaper can drop it the next time.
+    ident.reset();
+
+    reaper.dropIdentsOlderThan(opCtx.get(), Timestamp::max());
+    ASSERT_EQUALS(1U, engine->droppedIdents.size());
+    ASSERT_EQUALS(identName, engine->droppedIdents.front());
+}
+
 DEATH_TEST_F(KVDropPendingIdentReaperTest,
              DropIdentsOlderThanTerminatesIfKVEngineFailsToDropIdent,
              "Failed to remove drop-pending ident") {
