@@ -39,6 +39,7 @@
 #include "mongo/db/catalog/collection_options.h"
 #include "mongo/db/catalog/import_options.h"
 #include "mongo/db/storage/column_store.h"
+#include "mongo/db/storage/compact_options.h"
 #include "mongo/db/storage/durable_catalog.h"
 #include "mongo/db/storage/record_store.h"
 #include "mongo/db/storage/sorted_data_interface.h"
@@ -66,7 +67,18 @@ public:
      * When all of the storage startup tasks are completed as a whole, then this function is called
      * by the external force managing the startup process.
      */
-    virtual void notifyStartupComplete() {}
+    virtual void notifyStorageStartupRecoveryComplete() {}
+
+    /**
+     * Perform any operations in the storage layer that are unblocked now that the server has exited
+     * recovery and considers itself stable.
+     *
+     * This will be called during a node's transition to steady state replication.
+     *
+     * This function may race with shutdown. As a result, any steps within this function that should
+     * not race with shutdown should obtain the global lock.
+     */
+    virtual void notifyReplStartupRecoveryComplete(OperationContext* opCtx) {}
 
     virtual RecoveryUnit* newRecoveryUnit() = 0;
 
@@ -84,6 +96,16 @@ public:
                                                         StringData ident,
                                                         const CollectionOptions& options) = 0;
 
+    /**
+     * Opens an existing ident as a temporary record store. Must be used for record stores created
+     * with `makeTemporaryRecordStore`. Using `getRecordStore` would cause the record store to use
+     * the same settings as a regular collection, and would differ in behaviour as when it was
+     * originally created with `makeTemporaryRecordStore`.
+     */
+    virtual std::unique_ptr<RecordStore> getTemporaryRecordStore(OperationContext* opCtx,
+                                                                 StringData ident,
+                                                                 KeyFormat keyFormat) = 0;
+
     virtual std::unique_ptr<SortedDataInterface> getSortedDataInterface(
         OperationContext* opCtx,
         const NamespaceString& nss,
@@ -109,6 +131,10 @@ public:
                                      const CollectionOptions& options,
                                      KeyFormat keyFormat = KeyFormat::Long) = 0;
 
+    /**
+     * RecordStores initially created with `makeTemporaryRecordStore` must be opened with
+     * `getTemporaryRecordStore`.
+     */
     virtual std::unique_ptr<RecordStore> makeTemporaryRecordStore(OperationContext* opCtx,
                                                                   StringData ident,
                                                                   KeyFormat keyFormat) = 0;
@@ -166,6 +192,17 @@ public:
                              StringData ident,
                              const StorageEngine::DropIdentCallback& onDrop = nullptr) = 0;
 
+    /**
+     * Removes any knowledge of the ident from the storage engines metadata which includes removing
+     * the underlying files belonging to the ident. Allows the storage engine to perform a blocking
+     * wait if it is unable to process the removal immediately. If a callback is specified, it will
+     * be run upon the drop if this function returns an OK status.
+     */
+    virtual Status dropIdentSynchronous(
+        RecoveryUnit* ru,
+        StringData ident,
+        const StorageEngine::DropIdentCallback& onDrop = nullptr) = 0;
+
     /**
      * Removes any knowledge of the ident from the storage engines metadata without removing the
      * underlying files belonging to the ident.
@@ -223,9 +260,7 @@ public:
     }
 
     virtual StatusWith<std::unique_ptr<StorageEngine::StreamingCursor>> beginNonBlockingBackup(
-        OperationContext* opCtx,
-        boost::optional<Timestamp> checkpointTimestamp,
-        const StorageEngine::BackupOptions& options) {
+        OperationContext* opCtx, const StorageEngine::BackupOptions& options) {
         return Status(ErrorCodes::CommandNotSupported,
                       "The current storage engine doesn't support backup mode");
     }
@@ -246,7 +281,7 @@ public:
         return false;
     }
 
-    virtual void checkpoint(OperationContext* opCtx) {}
+    virtual void checkpoint() {}
 
     virtual StorageEngine::CheckpointIteration getCheckpointIteration() const {
         return StorageEngine::CheckpointIteration{0};
@@ -286,12 +321,9 @@ public:
      * override this method if they have clean-up to do that is different from unclean shutdown.
      * MongoDB will not call into the storage subsystem after calling this function.
      *
-     * The storage engine is allowed to leak memory for faster shutdown, except when the process is
-     * not exiting or when running tools to look for memory leaks.
-     *
      * There is intentionally no uncleanShutdown().
      */
-    virtual void cleanShutdown(bool memLeakAllowed) = 0;
+    virtual void cleanShutdown() = 0;
 
     /**
      * Return the SnapshotManager for this KVEngine or NULL if not supported.
@@ -471,12 +503,22 @@ public:
      * Returns the input storage engine options, sanitized to remove options that may not apply to
      * this node, such as encryption. Might be called for both collection and index options. See
      * SERVER-68122.
+     *
+     * TODO SERVER-81069: Remove this since it's intrinsically tied to encryption options only.
      */
     virtual BSONObj getSanitizedStorageOptionsForSecondaryReplication(
         const BSONObj& options) const {
         return options;
     }
 
+    /**
+     * See StorageEngine::autoCompact for details
+     */
+    virtual Status autoCompact(OperationContext* opCtx, const AutoCompactOptions& options) {
+        return Status(ErrorCodes::CommandNotSupported,
+                      "The current storage engine doesn't support auto compact");
+    }
+
     /**
      * The destructor will never be called from mongod, but may be called from tests.
      * Engines may assume that this will only be called in the case of clean shutdown, even if
