@@ -27,37 +27,73 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
+#include <absl/container/node_hash_map.h>
+#include <boost/filesystem/operations.hpp>
+#include <boost/filesystem/path.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr.hpp>
+#include <fmt/format.h>
+// IWYU pragma: no_include "cxxabi.h"
+#include <algorithm>
+#include <cstring>
 #include <memory>
+#include <mutex>
+#include <set>
+#include <string>
+#include <utility>
+#include <vector>
 
-#include "mongo/base/checked_cast.h"
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
 #include "mongo/base/status_with.h"
-#include "mongo/bson/bsonobjbuilder.h"
-#include "mongo/db/db_raii.h"
-#include "mongo/db/index/index_descriptor.h"
-#include "mongo/db/index_names.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/db/catalog/index_builds.h"
+#include "mongo/db/catalog_raii.h"
+#include "mongo/db/client.h"
+#include "mongo/db/concurrency/d_concurrency.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
 #include "mongo/db/namespace_string.h"
-#include "mongo/db/operation_context_noop.h"
-#include "mongo/db/repl/repl_settings.h"
-#include "mongo/db/repl/replication_coordinator_mock.h"
-#include "mongo/db/service_context_d_test_fixture.h"
-#include "mongo/db/startup_recovery.h"
-#include "mongo/db/storage/control/storage_control.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/record_id.h"
+#include "mongo/db/service_context.h"
+#include "mongo/db/service_context_test_fixture.h"
 #include "mongo/db/storage/devnull/devnull_kv_engine.h"
+#include "mongo/db/storage/durable_catalog.h"
+#include "mongo/db/storage/key_format.h"
 #include "mongo/db/storage/kv/kv_engine.h"
+#include "mongo/db/storage/record_data.h"
+#include "mongo/db/storage/record_store.h"
+#include "mongo/db/storage/storage_engine.h"
 #include "mongo/db/storage/storage_engine_impl.h"
+#include "mongo/db/storage/storage_engine_init.h"
 #include "mongo/db/storage/storage_engine_test_fixture.h"
+#include "mongo/db/storage/storage_options.h"
 #include "mongo/db/storage/storage_repair_observer.h"
+#include "mongo/db/storage/temporary_record_store.h"
+#include "mongo/db/storage/write_unit_of_work.h"
+#include "mongo/db/transaction_resources.h"
 #include "mongo/logv2/log.h"
-#include "mongo/unittest/barrier.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/platform/mutex.h"
+#include "mongo/stdx/condition_variable.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/future.h"
+#include "mongo/util/future_impl.h"
+#include "mongo/util/periodic_runner.h"
 #include "mongo/util/periodic_runner_factory.h"
+#include "mongo/util/time_support.h"
+#include "mongo/util/uuid.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
 
-
 namespace mongo {
 namespace {
 
@@ -88,7 +124,8 @@ TEST_F(StorageEngineTest, ReconcileIdentsTest) {
     ASSERT_TRUE(idents.find("_mdb_catalog") != idents.end());
 
     // Drop the `db.coll1` table, while leaving the DurableCatalog entry.
-    ASSERT_OK(dropIdent(opCtx.get()->recoveryUnit(), swCollInfo.getValue().ident));
+    ASSERT_OK(
+        dropIdent(shard_role_details::getRecoveryUnit(opCtx.get()), swCollInfo.getValue().ident));
     ASSERT_EQUALS(static_cast<const unsigned long>(1), getAllKVEngineIdents(opCtx.get()).size());
 
     // Reconciling this should result in an error.
@@ -104,7 +141,8 @@ TEST_F(StorageEngineTest, LoadCatalogDropsOrphansAfterUncleanShutdown) {
     auto swCollInfo = createCollection(opCtx.get(), collNs);
     ASSERT_OK(swCollInfo.getStatus());
 
-    ASSERT_OK(dropIdent(opCtx.get()->recoveryUnit(), swCollInfo.getValue().ident));
+    ASSERT_OK(
+        dropIdent(shard_role_details::getRecoveryUnit(opCtx.get()), swCollInfo.getValue().ident));
     ASSERT(collectionExists(opCtx.get(), collNs));
 
     // After the catalog is reloaded, we expect that the collection has been dropped because the
@@ -186,9 +224,49 @@ TEST_F(StorageEngineTest, ReconcileKeepsTemporary) {
     ASSERT_FALSE(identExists(opCtx.get(), rs->rs()->getIdent()));
 }
 
+TEST_F(StorageEngineTest, TemporaryRecordStoreDoesNotTrackSizeAdjustments) {
+    auto opCtx = cc().makeOperationContext();
+
+    const auto insertRecordAndAssertSize = [&](RecordStore* rs, const RecordId& rid) {
+        // Verify a temporary record store does not track size adjustments.
+        const auto data = "data";
+
+        WriteUnitOfWork wuow(opCtx.get());
+        StatusWith<RecordId> s =
+            rs->insertRecord(opCtx.get(), rid, data, strlen(data), Timestamp());
+        ASSERT_TRUE(s.isOK());
+        wuow.commit();
+
+        ASSERT_EQ(rs->numRecords(opCtx.get()), 0);
+        ASSERT_EQ(rs->dataSize(opCtx.get()), 0);
+    };
+
+    // Create the temporary record store and get its ident.
+    const std::string ident = [&]() {
+        std::unique_ptr<TemporaryRecordStore> rs;
+        Lock::GlobalLock lk(&*opCtx, MODE_IS);
+        rs = makeTemporary(opCtx.get());
+        ASSERT(rs.get());
+
+        insertRecordAndAssertSize(rs->rs(), RecordId(1));
+
+        // Keep ident even when TemporaryRecordStore goes out of scope.
+        rs->keep();
+        return rs->rs()->getIdent();
+    }();
+    ASSERT(identExists(opCtx.get(), ident));
+
+    std::unique_ptr<TemporaryRecordStore> rs;
+    rs = _storageEngine->makeTemporaryRecordStoreFromExistingIdent(
+        opCtx.get(), ident, KeyFormat::Long);
+
+    // Verify a temporary record store does not track size adjustments after re-opening.
+    insertRecordAndAssertSize(rs->rs(), RecordId(2));
+}
+
 class StorageEngineTimestampMonitorTest : public StorageEngineTest {
 public:
-    void setUp() {
+    void setUp() override {
         StorageEngineTest::setUp();
         _storageEngine->startTimestampMonitor();
     }
@@ -322,7 +400,7 @@ TEST_F(StorageEngineTest, ReconcileUnfinishedBackgroundSecondaryIndex) {
     // require it to be rebuilt.
     ASSERT_EQUALS(1UL, reconcileResult.indexesToRebuild.size());
     StorageEngine::IndexIdentifier& toRebuild = reconcileResult.indexesToRebuild[0];
-    ASSERT_EQUALS(ns.ns(), toRebuild.nss.ns());
+    ASSERT_EQUALS(ns.ns_forTest(), toRebuild.nss.ns_forTest());
     ASSERT_EQUALS(indexName, toRebuild.indexName);
 
     // There are no two-phase builds to restart or resume.
@@ -432,7 +510,8 @@ TEST_F(StorageEngineRepairTest, ReconcileSucceeds) {
     auto swCollInfo = createCollection(opCtx.get(), collNs);
     ASSERT_OK(swCollInfo.getStatus());
 
-    ASSERT_OK(dropIdent(opCtx.get()->recoveryUnit(), swCollInfo.getValue().ident));
+    ASSERT_OK(
+        dropIdent(shard_role_details::getRecoveryUnit(opCtx.get()), swCollInfo.getValue().ident));
     ASSERT(collectionExists(opCtx.get(), collNs));
 
     // Reconcile would normally return an error if a collection existed with a missing ident in the
@@ -463,7 +542,7 @@ TEST_F(StorageEngineRepairTest, LoadCatalogRecoversOrphansInCatalog) {
     // the actual drop in storage engine.
     {
         WriteUnitOfWork wuow(opCtx.get());
-        ASSERT_OK(removeEntry(opCtx.get(), collNs.ns(), _storageEngine->getCatalog()));
+        ASSERT_OK(removeEntry(opCtx.get(), collNs.ns_forTest(), _storageEngine->getCatalog()));
         wuow.commit();
     }
 
@@ -497,7 +576,7 @@ TEST_F(StorageEngineTest, LoadCatalogDropsOrphans) {
     {
         AutoGetDb db(opCtx.get(), collNs.dbName(), LockMode::MODE_X);
         WriteUnitOfWork wuow(opCtx.get());
-        ASSERT_OK(removeEntry(opCtx.get(), collNs.ns(), _storageEngine->getCatalog()));
+        ASSERT_OK(removeEntry(opCtx.get(), collNs.ns_forTest(), _storageEngine->getCatalog()));
         wuow.commit();
     }
     ASSERT(!collectionExists(opCtx.get(), collNs));
@@ -532,15 +611,15 @@ public:
     }
 
     // Increment the timestamps each time they are called for testing purposes.
-    virtual Timestamp getCheckpointTimestamp() const override {
+    Timestamp getCheckpointTimestamp() const override {
         checkpointTimestamp = std::make_unique<Timestamp>(checkpointTimestamp->getInc() + 1);
         return *checkpointTimestamp;
     }
-    virtual Timestamp getOldestTimestamp() const override {
+    Timestamp getOldestTimestamp() const override {
         oldestTimestamp = std::make_unique<Timestamp>(oldestTimestamp->getInc() + 1);
         return *oldestTimestamp;
     }
-    virtual Timestamp getStableTimestamp() const override {
+    Timestamp getStableTimestamp() const override {
         stableTimestamp = std::make_unique<Timestamp>(stableTimestamp->getInc() + 1);
         return *stableTimestamp;
     }
@@ -559,7 +638,7 @@ public:
     /**
      * Create an instance of the KV Storage Engine so that we have a timestamp monitor operating.
      */
-    void setUp() {
+    void setUp() override {
         ServiceContextTest::setUp();
 
         auto opCtx = makeOperationContext();
@@ -576,13 +655,8 @@ public:
         _storageEngine->startTimestampMonitor();
     }
 
-    void tearDown() {
-#if __has_feature(address_sanitizer)
-        constexpr bool memLeakAllowed = false;
-#else
-        constexpr bool memLeakAllowed = true;
-#endif
-        _storageEngine->cleanShutdown(getServiceContext(), memLeakAllowed);
+    void tearDown() override {
+        _storageEngine->cleanShutdown(getServiceContext());
         _storageEngine.reset();
 
         ServiceContextTest::tearDown();
@@ -718,7 +792,7 @@ TEST_F(StorageEngineTestNotEphemeral, UseAlternateStorageLocation) {
         reinitializeStorageEngine(opCtx.get(), StorageEngineInitFlags{}, [&newPath] {
             storageGlobalParams.dbpath = newPath;
         });
-    getGlobalServiceContext()->getStorageEngine()->notifyStartupComplete();
+    getGlobalServiceContext()->getStorageEngine()->notifyStorageStartupRecoveryComplete();
     LOGV2(5781103, "Started up storage engine in alternate location");
     ASSERT(StorageEngine::LastShutdownState::kClean == lastShutdownState);
     StorageEngineTest::_storageEngine = getServiceContext()->getStorageEngine();
@@ -736,11 +810,12 @@ TEST_F(StorageEngineTestNotEphemeral, UseAlternateStorageLocation) {
         reinitializeStorageEngine(opCtx.get(), StorageEngineInitFlags{}, [&oldPath] {
             storageGlobalParams.dbpath = oldPath;
         });
-    getGlobalServiceContext()->getStorageEngine()->notifyStartupComplete();
+    getGlobalServiceContext()->getStorageEngine()->notifyStorageStartupRecoveryComplete();
     ASSERT(StorageEngine::LastShutdownState::kClean == lastShutdownState);
     StorageEngineTest::_storageEngine = getServiceContext()->getStorageEngine();
     ASSERT_TRUE(collectionExists(opCtx.get(), coll1Ns));
     ASSERT_FALSE(collectionExists(opCtx.get(), coll2Ns));
 }
+
 }  // namespace
 }  // namespace mongo
