@@ -30,19 +30,22 @@
 #pragma once
 
 #include <cstdint>
+#include <functional>
 #include <memory>
+#include <string>
 
+#include "mongo/bson/timestamp.h"
 #include "mongo/db/catalog/collection_options.h"
-#include "mongo/db/operation_context_noop.h"
+#include "mongo/db/client.h"
 #include "mongo/db/service_context.h"
+#include "mongo/db/storage/key_format.h"
 #include "mongo/db/storage/kv/kv_engine.h"
 #include "mongo/db/storage/record_store.h"
+#include "mongo/db/storage/storage_engine.h"
 #include "mongo/db/storage/test_harness_helper.h"
 
 namespace mongo {
 
-class RecoveryUnit;
-
 class RecordStoreHarnessHelper : public HarnessHelper {
 public:
     enum class Options { Standalone, ReplicationEnabled };
@@ -65,7 +68,6 @@ public:
      * Advances the stable timestamp of the engine.
      */
     void advanceStableTimestamp(Timestamp newTimestamp) {
-        auto opCtx = this->client()->getOperationContext();
         auto engine = getEngine();
         // Disable the callback for oldest active transaction as it blocks the timestamps from
         // advancing.
@@ -73,7 +75,7 @@ public:
             StorageEngine::OldestActiveTransactionTimestampCallback{});
         engine->setInitialDataTimestamp(newTimestamp);
         engine->setStableTimestamp(newTimestamp, true);
-        engine->checkpoint(opCtx);
+        engine->checkpoint();
     }
 };
 
