@@ -29,20 +29,28 @@
 
 #pragma once
 
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
 #include <cstdint>
 #include <cstdlib>
+#include <functional>
+#include <memory>
 #include <string>
+#include <vector>
 
+#include "mongo/base/error_codes.h"
 #include "mongo/base/status.h"
 #include "mongo/bson/timestamp.h"
 #include "mongo/db/repl/read_concern_level.h"
 #include "mongo/db/storage/snapshot.h"
 #include "mongo/db/storage/storage_stats.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/decorable.h"
 
 namespace mongo {
 
 class BSONObjBuilder;
+
 class OperationContext;
 
 /**
@@ -53,7 +61,7 @@ enum class PrepareConflictBehavior {
     /**
      * When prepare conflicts are encountered, block until the conflict is resolved.
      */
-    kEnforce,
+    kEnforce = 0,
 
     /**
      * Ignore prepare conflicts when they are encountered.
@@ -73,7 +81,10 @@ enum class PrepareConflictBehavior {
      * This should only be used in cases where this is known to be impossible to perform writes
      * based on other prepared updates.
      */
-    kIgnoreConflictsAllowWrites
+    kIgnoreConflictsAllowWrites,
+
+    /** kMax should always be last and is a counter of the number of enum values. */
+    kMax,
 };
 
 /**
@@ -86,8 +97,8 @@ enum class DataCorruptionDetectionMode {
      */
     kThrow,
     /**
-     * When evidence of data corruption is decected, log an entry to the health log and the server
-     * logs, but do not throw an error. Continue attempting to return results.
+     * When evidence of data corruption is decected, log an entry to the health log and the
+     * server logs, but do not throw an error. Continue attempting to return results.
      */
     kLogAndContinue,
 };
@@ -101,14 +112,16 @@ class RecoveryUnit {
     RecoveryUnit& operator=(const RecoveryUnit&) = delete;
 
 public:
+    virtual ~RecoveryUnit() = default;
+
     /**
-     * A Snapshot is a decorable type whose lifetime is tied to the the lifetime of a snapshot
-     * within the RecoveryUnit. Snapshots hold no storage engine state and are to be used for
-     * snapshot ID comparison on a single RecoveryUnit and to support decorated types that should be
-     * destructed when the storage snapshot is invalidated.
+     * A Snapshot is a decorable type whose lifetime is tied to the the lifetime of a
+     * snapshot within the RecoveryUnit. Snapshots hold no storage engine state and are to
+     * be used for snapshot ID comparison on a single RecoveryUnit and to support decorated
+     * types that should be destructed when the storage snapshot is invalidated.
      *
-     * Classes that decorate a Snapshot are constructed before a new storage snapshot is established
-     * and destructed after the storage engine snapshot has been released.
+     * Classes that decorate a Snapshot are constructed before a new storage snapshot is
+     * established and destructed after the storage engine snapshot has been released.
      */
     class Snapshot : public Decorable<Snapshot> {
     public:
@@ -127,14 +140,20 @@ public:
     };
 
     /**
-     * Returns the current snapshot on this RecoveryUnit. Will be destructed and re-constructed when
-     * the storage engine snapshot is closed via calls to abandonSnapshot, commitUnitOfWork, or
-     * abortUnitOfWork.
+     * Returns the current Snapshot on this RecoveryUnit, constructing one if none exists yet.
+     * Otherwise, a Snapshot is guaranteed to be constructed when the storage engine snapshot is
+     * opened. Will be destructed when the storage engine snapshot is closed via calls to
+     * abandonSnapshot, commitUnitOfWork, or abortUnitOfWork.
      *
      * Note that the RecoveryUnit does not make any guarantees that this reference remains valid
      * except for the lifetime of the Snapshot.
      */
-    Snapshot& getSnapshot();
+    Snapshot& getSnapshot() {
+        if (!_snapshot.is_initialized()) {
+            ensureSnapshot();
+        }
+        return _snapshot.get();
+    }
 
     // Behavior for abandonSnapshot().
     enum class AbandonSnapshotMode {
@@ -151,7 +170,6 @@ public:
 
     void commitRegisteredChanges(boost::optional<Timestamp> commitTimestamp);
     void abortRegisteredChanges();
-    virtual ~RecoveryUnit() {}
 
     /**
      * Marks the beginning of a unit of work. Each call must be matched with exactly one call to
@@ -188,6 +206,13 @@ public:
      */
     void endReadOnlyUnitOfWork();
 
+    /**
+     * Sets whether cursors in this operation should engage in pre-fetching data from disk to
+     * the storage engine cache. This feature can be useful for operations that are typically
+     * I/O bound.
+     */
+    virtual void setPrefetching(bool enable) {}
+
     /**
      * Transitions the active unit of work to the "prepared" state. Must be called after
      * beginUnitOfWork and before calling either abortUnitOfWork or commitUnitOfWork. Must be
@@ -228,6 +253,14 @@ public:
      */
     virtual void setRoundUpPreparedTimestamps(bool value) {}
 
+    /**
+     * Returns whether we have enabled the setting to round up prepare and commit timestamps for
+     * prepared transactions. See `setRoundUpPreparedTimestamps` for details.
+     */
+    virtual bool getRoundUpPreparedTimestamps() {
+        return false;
+    }
+
     /**
      * Waits until all commits that happened before this call are durable in the journal. Returns
      * true, unless the storage engine cannot guarantee durability, which should never happen when
@@ -283,16 +316,6 @@ public:
      */
     virtual void preallocateSnapshot() {}
 
-    /**
-     * Like preallocateSnapshot() above but also indicates that the snapshot will be used for
-     * reading the oplog.
-     *
-     * StorageEngines may not implement this in which case it works like preallocateSnapshot.
-     */
-    virtual void preallocateSnapshotForOplogRead() {
-        preallocateSnapshot();
-    }
-
     /**
      * Returns whether or not a majority commmitted snapshot is available. If no snapshot has yet
      * been marked as Majority Committed, returns a status with error code
@@ -334,8 +357,8 @@ public:
      *
      * This is unrelated to Timestamp which must be globally comparable.
      */
-    SnapshotId getSnapshotId() const {
-        return _snapshot->getId();
+    SnapshotId getSnapshotId() {
+        return getSnapshot().getId();
     }
 
     /**
@@ -417,29 +440,6 @@ public:
                   "This storage engine does not support prepared transactions");
     }
 
-    /**
-     * Sets catalog conflicting timestamp.
-     * This can only be called while the RecoveryUnit is in an inactive state.
-     *
-     * This value must be set when both of the following conditions are true:
-     * - A storage engine snapshot is opened without a read timestamp
-     * (RecoveryUnit::ReadSource::kNoTimestamp).
-     * - The transaction may touch collections it does not yet have locks for.
-     * In this circumstance, the catalog conflicting timestamp serves as a substitute for a read
-     * timestamp. This value must be set to a valid (i.e: no-holes) read timestamp prior to
-     * acquiring a storage engine snapshot. This timestamp will be used to determine if any changes
-     * had happened to the in-memory catalog after a storage engine snapshot got opened for that
-     * transaction.
-     */
-    virtual void setCatalogConflictingTimestamp(Timestamp timestamp) {}
-
-    /**
-     * Returns the catalog conflicting timestamp.
-     */
-    virtual Timestamp getCatalogConflictingTimestamp() const {
-        return {};
-    }
-
     /**
      * MongoDB must update documents with non-decreasing timestamp values. A storage engine is
      * allowed to assert when this contract is violated. An untimestamped write is a subset of these
@@ -540,6 +540,7 @@ public:
     virtual boost::optional<int64_t> getOplogVisibilityTs() {
         return boost::none;
     }
+    virtual void setOplogVisibilityTs(boost::optional<int64_t> oplogVisibilityTs) {}
 
     /**
      * Pinning informs callers not to change the ReadSource on this RecoveryUnit. Callers are
@@ -620,22 +621,6 @@ public:
         virtual void commit(OperationContext* opCtx, boost::optional<Timestamp> commitTime) = 0;
     };
 
-    /**
-     * A SnapshotChange is an action that can be registered at anytime. When a WriteUnitOfWork
-     * begins, the openSnapshot() callback is called for any registered snapshot changes. Similarly,
-     * when the snapshot is abandoned, or the WriteUnitOfWork is committed or aborted, the
-     * closeSnapshot() callback is called.
-     *
-     * The same rules apply here that apply to the Change class.
-     */
-    class SnapshotChange {
-    public:
-        virtual ~SnapshotChange() {}
-
-        virtual void openSnapshot(OperationContext* opCtx) = 0;
-        virtual void closeSnapshot(OperationContext* opCtx) = 0;
-    };
-
     /**
      * The commitUnitOfWork() method calls the commit() method of each registered change in order of
      * registration. The endUnitOfWork() method calls the rollback() method of each registered
@@ -674,21 +659,25 @@ public:
 
     /**
      * Like registerChange() above but should only be used to make new state visible in the
-     * in-memory catalog. Only one change of this kind may be registered at a given time to ensure
-     * catalog updates are atomic. Change registered with this function will commit after the commit
-     * changes registered with registerChange and rollback will run before the rollback changes
-     * registered with registerChange.
+     * in-memory catalog. Change registered with this function will commit before the commit
+     * changes registered with registerChange and rollback will run after the rollback changes
+     * registered with registerChange. Only one change of this kind should be registered at a given
+     * time to ensure catalog updates are atomic, however multiple callbacks are allowed for testing
+     * purposes.
      *
-     * This separation ensures that regular Changes that can modify state are run before the Change
-     * to install the new state in the in-memory catalog, after which there should be no further
-     * changes.
+     * This separation ensures that regular Changes can observe changes to catalog visibility.
      */
     void registerChangeForCatalogVisibility(std::unique_ptr<Change> change);
 
     /**
-     * Returns true if a change has been registered with registerChangeForCatalogVisibility() above.
+     * Like registerChange() above but should only be used to push idents for two phase drop to the
+     * reaper. This currently needs to happen before a drop is made visible in the catalog to avoid
+     * a window where a reader would observe the drop in the catalog but not be able to find the
+     * ident in the reaper.
+     *
+     * TODO SERVER-77959: Remove this.
      */
-    bool hasRegisteredChangeForCatalogVisibility();
+    void registerChangeForTwoPhaseDrop(std::unique_ptr<Change> change);
 
     /**
      * Registers a callback to be called if the current WriteUnitOfWork rolls back.
@@ -742,6 +731,31 @@ public:
         registerChange(std::make_unique<OnCommitChange>(std::move(callback)));
     }
 
+    /**
+     * Registers a callback to be called if the current WriteUnitOfWork commits for two phase drop.
+     *
+     * Should only be used for adding drop pending idents to the reaper!
+     *
+     * TODO SERVER-77959: Remove this.
+     */
+    template <typename Callback>
+    void onCommitForTwoPhaseDrop(Callback callback) {
+        class OnCommitTwoPhaseChange final : public Change {
+        public:
+            OnCommitTwoPhaseChange(Callback&& callback) : _callback(std::move(callback)) {}
+            void rollback(OperationContext* opCtx) final {}
+            void commit(OperationContext* opCtx, boost::optional<Timestamp> commitTime) final {
+                _callback(opCtx, commitTime);
+            }
+
+        private:
+            Callback _callback;
+        };
+
+        registerChangeForTwoPhaseDrop(
+            std::make_unique<OnCommitTwoPhaseChange>(std::move(callback)));
+    }
+
     virtual void setOrderedCommit(bool orderedCommit) = 0;
 
     /**
@@ -846,7 +860,7 @@ public:
     virtual void setCacheMaxWaitTimeout(Milliseconds) {}
 
 protected:
-    RecoveryUnit();
+    RecoveryUnit() = default;
 
     /**
      * Returns the current state.
@@ -900,10 +914,19 @@ protected:
 
     DataCorruptionDetectionMode _dataCorruptionDetectionMode = DataCorruptionDetectionMode::kThrow;
 
-private:
-    // Sets the snapshot associated with this RecoveryUnit to a new globally unique id number.
-    void assignNextSnapshot();
+    /**
+     * Creates a new globally-unique Snapshot, if one does not exst.
+     */
+    void ensureSnapshot();
 
+    /**
+     * Destructs the current snapshot.
+     */
+    void resetSnapshot() {
+        _snapshot.reset();
+    }
+
+private:
     virtual void doBeginUnitOfWork() = 0;
     virtual void doAbandonSnapshot() = 0;
     virtual void doCommitUnitOfWork() = 0;
@@ -915,12 +938,10 @@ private:
 
     typedef std::vector<std::unique_ptr<Change>> Changes;
     Changes _changes;
-    typedef std::vector<std::unique_ptr<SnapshotChange>> SnapshotChanges;
-    SnapshotChanges _snapshotChanges;
-    // The Snapshot is always initialized by the RecoveryUnit constructor. We use an optional to
-    // simplify destructing and re-constructing the Snapshot in-place.
+    Changes _changesForCatalogVisibility;
+    Changes _changesForTwoPhaseDrop;
+    // Is constructed lazily when accessed or when an underlying storage snapshot is opened.
     boost::optional<Snapshot> _snapshot;
-    std::unique_ptr<Change> _changeForCatalogVisibility;
     State _state = State::kInactive;
     OperationContext* _opCtx = nullptr;
     bool _readOnly = false;
@@ -946,4 +967,31 @@ private:
     RecoveryUnit* const _recoveryUnit;
 };
 
+
+/**
+ * RAII-style class to override the oplog visible timestamp of the WiredTigerRecoveryUnit while it's
+ * in scope.
+ */
+class ScopedOplogVisibleTimestamp {
+public:
+    ScopedOplogVisibleTimestamp(RecoveryUnit* recoveryUnit, boost::optional<int64_t> oplogVisibleTs)
+        : _recoveryUnit(recoveryUnit),
+          _originalOplogVisibleTs(recoveryUnit->getOplogVisibilityTs()) {
+        _recoveryUnit->setOplogVisibilityTs(oplogVisibleTs);
+    }
+
+    ScopedOplogVisibleTimestamp(const ScopedOplogVisibleTimestamp&) = delete;
+    ScopedOplogVisibleTimestamp& operator=(const ScopedOplogVisibleTimestamp&) = delete;
+    ScopedOplogVisibleTimestamp(ScopedOplogVisibleTimestamp&&) = delete;
+    ScopedOplogVisibleTimestamp& operator=(ScopedOplogVisibleTimestamp&&) = delete;
+
+    ~ScopedOplogVisibleTimestamp() {
+        _recoveryUnit->setOplogVisibilityTs(_originalOplogVisibleTs);
+    }
+
+private:
+    RecoveryUnit* _recoveryUnit;
+    boost::optional<int64_t> _originalOplogVisibleTs;
+};
+
 }  // namespace mongo
