@@ -27,14 +27,28 @@
  *    it in the license file.
  */
 
-#include "mongo/db/storage/sorted_data_interface_test_harness.h"
-
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <cstring>
 #include <memory>
+#include <string>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/base/string_data.h"
+#include "mongo/db/concurrency/d_concurrency.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
+#include "mongo/db/record_id.h"
 #include "mongo/db/record_id_helpers.h"
-#include "mongo/db/storage/key_string.h"
+#include "mongo/db/service_context.h"
+#include "mongo/db/storage/index_entry_comparison.h"
+#include "mongo/db/storage/key_format.h"
 #include "mongo/db/storage/sorted_data_interface.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/db/storage/sorted_data_interface_test_harness.h"
+#include "mongo/db/storage/write_unit_of_work.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/assert_util_core.h"
 
 namespace mongo {
 namespace {
@@ -44,6 +58,8 @@ TEST(SortedDataInterface, KeyFormatStringInsertDuplicates) {
     const std::unique_ptr<SortedDataInterface> sorted(harnessHelper->newSortedDataInterface(
         /*unique=*/false, /*partial=*/false, KeyFormat::String));
     const ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+    Lock::GlobalLock globalLock(opCtx.get(), MODE_X);
+
     ASSERT(sorted->isEmpty(opCtx.get()));
 
     char buf1[12];
@@ -75,7 +91,7 @@ TEST(SortedDataInterface, KeyFormatStringInsertDuplicates) {
     auto ksSeek = makeKeyStringForSeek(sorted.get(), key1, true, true);
     {
         auto cursor = sorted->newCursor(opCtx.get());
-        auto entry = cursor->seek(ksSeek);
+        auto entry = cursor->seek(ksSeek.finishAndGetBuffer());
         ASSERT(entry);
         ASSERT_EQ(*entry, IndexKeyEntry(key1, rid1));
 
@@ -90,7 +106,7 @@ TEST(SortedDataInterface, KeyFormatStringInsertDuplicates) {
 
     {
         auto cursor = sorted->newCursor(opCtx.get());
-        auto entry = cursor->seekForKeyString(ksSeek);
+        auto entry = cursor->seekForKeyString(ksSeek.finishAndGetBuffer());
         ASSERT(entry);
         ASSERT_EQ(entry->loc, rid1);
         auto ks1 = makeKeyString(sorted.get(), key1, rid1);
@@ -115,6 +131,8 @@ TEST(SortedDataInterface, KeyFormatStringUniqueInsertRemoveDuplicates) {
     const std::unique_ptr<SortedDataInterface> sorted(harnessHelper->newSortedDataInterface(
         /*unique=*/true, /*partial=*/false, KeyFormat::String));
     const ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+    Lock::GlobalLock globalLock(opCtx.get(), MODE_X);
+
     ASSERT(sorted->isEmpty(opCtx.get()));
 
     std::string buf1(12, 0);
@@ -163,7 +181,7 @@ TEST(SortedDataInterface, KeyFormatStringUniqueInsertRemoveDuplicates) {
     auto ksSeek = makeKeyStringForSeek(sorted.get(), key1, true, true);
     {
         auto cursor = sorted->newCursor(opCtx.get());
-        auto entry = cursor->seek(ksSeek);
+        auto entry = cursor->seek(ksSeek.finishAndGetBuffer());
         ASSERT(entry);
         ASSERT_EQ(*entry, IndexKeyEntry(key1, rid3));
 
@@ -177,7 +195,7 @@ TEST(SortedDataInterface, KeyFormatStringUniqueInsertRemoveDuplicates) {
 
     {
         auto cursor = sorted->newCursor(opCtx.get());
-        auto entry = cursor->seekForKeyString(ksSeek);
+        auto entry = cursor->seekForKeyString(ksSeek.finishAndGetBuffer());
         ASSERT(entry);
         ASSERT_EQ(entry->loc, rid3);
         auto ks1 = makeKeyString(sorted.get(), key1, rid3);
@@ -199,6 +217,8 @@ TEST(SortedDataInterface, KeyFormatStringSetEndPosition) {
     const std::unique_ptr<SortedDataInterface> sorted(harnessHelper->newSortedDataInterface(
         /*unique=*/false, /*partial=*/false, KeyFormat::String));
     const ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+    Lock::GlobalLock globalLock(opCtx.get(), MODE_X);
+
     ASSERT(sorted->isEmpty(opCtx.get()));
 
     char buf1[12];
@@ -232,7 +252,7 @@ TEST(SortedDataInterface, KeyFormatStringSetEndPosition) {
         auto ksSeek = makeKeyStringForSeek(sorted.get(), key1, true, true);
         auto cursor = sorted->newCursor(opCtx.get());
         cursor->setEndPosition(key1, true /* inclusive */);
-        auto entry = cursor->seek(ksSeek);
+        auto entry = cursor->seek(ksSeek.finishAndGetBuffer());
         ASSERT(entry);
         ASSERT_EQ(*entry, IndexKeyEntry(key1, rid1));
         ASSERT_FALSE(cursor->next());
@@ -243,7 +263,7 @@ TEST(SortedDataInterface, KeyFormatStringSetEndPosition) {
         auto ksSeek = makeKeyStringForSeek(sorted.get(), key1, true, true);
         auto cursor = sorted->newCursor(opCtx.get());
         cursor->setEndPosition(key2, true /* inclusive */);
-        auto entry = cursor->seek(ksSeek);
+        auto entry = cursor->seek(ksSeek.finishAndGetBuffer());
         ASSERT(entry);
         entry = cursor->next();
         ASSERT(entry);
@@ -256,7 +276,7 @@ TEST(SortedDataInterface, KeyFormatStringSetEndPosition) {
         auto ksSeek = makeKeyStringForSeek(sorted.get(), key2, true, true);
         auto cursor = sorted->newCursor(opCtx.get());
         cursor->setEndPosition(key3, false /* inclusive */);
-        auto entry = cursor->seek(ksSeek);
+        auto entry = cursor->seek(ksSeek.finishAndGetBuffer());
         ASSERT(entry);
         ASSERT_EQ(*entry, IndexKeyEntry(key2, rid2));
         ASSERT_FALSE(cursor->next());
@@ -268,6 +288,8 @@ TEST(SortedDataInterface, KeyFormatStringUnindex) {
     const std::unique_ptr<SortedDataInterface> sorted(harnessHelper->newSortedDataInterface(
         /*unique=*/false, /*partial=*/false, KeyFormat::String));
     const ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+    Lock::GlobalLock globalLock(opCtx.get(), MODE_X);
+
     ASSERT(sorted->isEmpty(opCtx.get()));
 
     char buf1[12];
@@ -317,6 +339,8 @@ TEST(SortedDataInterface, KeyFormatStringUniqueUnindex) {
     const std::unique_ptr<SortedDataInterface> sorted(harnessHelper->newSortedDataInterface(
         /*unique=*/true, /*partial=*/false, KeyFormat::String));
     const ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+    Lock::GlobalLock globalLock(opCtx.get(), MODE_X);
+
     ASSERT(sorted->isEmpty(opCtx.get()));
 
     std::string buf1(12, 0);
@@ -369,6 +393,8 @@ TEST(SortedDataInterface, InsertReservedRecordIdStr) {
     const std::unique_ptr<SortedDataInterface> sorted(harnessHelper->newSortedDataInterface(
         /*unique=*/false, /*partial=*/false, KeyFormat::String));
     const ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+    Lock::GlobalLock globalLock(opCtx.get(), MODE_X);
+
     ASSERT(sorted->isEmpty(opCtx.get()));
     WriteUnitOfWork uow(opCtx.get());
     RecordId reservedLoc(record_id_helpers::reservedIdFor(
@@ -406,6 +432,7 @@ TEST(SortedDataInterface, BuilderAddKeyWithReservedRecordIdStr) {
 
     {
         const ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+        Lock::GlobalLock globalLock(opCtx.get(), MODE_S);
         ASSERT_EQUALS(1, sorted->numEntries(opCtx.get()));
     }
 }
