@@ -29,28 +29,49 @@
 
 #pragma once
 
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <cstddef>
+#include <cstdint>
+#include <deque>
 #include <functional>
+#include <list>
 #include <map>
 #include <memory>
+#include <set>
 #include <string>
+#include <utility>
+#include <vector>
 
+#include "mongo/base/status.h"
 #include "mongo/base/status_with.h"
 #include "mongo/base/string_data.h"
+#include "mongo/bson/bsonobj.h"
 #include "mongo/bson/timestamp.h"
+#include "mongo/db/database_name.h"
 #include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/record_id.h"
+#include "mongo/db/service_context.h"
 #include "mongo/db/storage/durable_catalog.h"
+#include "mongo/db/storage/ident.h"
 #include "mongo/db/storage/journal_listener.h"
+#include "mongo/db/storage/key_format.h"
 #include "mongo/db/storage/kv/kv_drop_pending_ident_reaper.h"
+#include "mongo/db/storage/kv/kv_engine.h"
 #include "mongo/db/storage/record_store.h"
+#include "mongo/db/storage/recovery_unit.h"
 #include "mongo/db/storage/storage_engine.h"
 #include "mongo/db/storage/storage_engine_interface.h"
 #include "mongo/db/storage/temporary_record_store.h"
+#include "mongo/db/tenant_id.h"
 #include "mongo/platform/mutex.h"
 #include "mongo/util/periodic_runner.h"
+#include "mongo/util/uuid.h"
 
 namespace mongo {
 
-class DurableCatalogImpl;
+class DurableCatalog;
 class KVEngine;
 
 struct StorageEngineOptions {
@@ -67,86 +88,88 @@ public:
                       std::unique_ptr<KVEngine> engine,
                       StorageEngineOptions options = StorageEngineOptions());
 
-    virtual ~StorageEngineImpl();
+    ~StorageEngineImpl() override;
 
-    virtual void notifyStartupComplete() override;
+    void notifyStorageStartupRecoveryComplete() override;
 
-    virtual RecoveryUnit* newRecoveryUnit() override;
+    void notifyReplStartupRecoveryComplete(OperationContext* opCtx) override;
 
-    virtual std::vector<DatabaseName> listDatabases(
+    RecoveryUnit* newRecoveryUnit() override;
+
+    std::vector<DatabaseName> listDatabases(
         boost::optional<TenantId> tenantId = boost::none) const override;
 
-    virtual bool supportsCappedCollections() const override {
+    bool supportsCappedCollections() const override {
         return _supportsCappedCollections;
     }
 
-    virtual Status dropDatabase(OperationContext* opCtx, const DatabaseName& dbName) override;
+    Status dropDatabase(OperationContext* opCtx, const DatabaseName& dbName) override;
+    Status dropCollectionsWithPrefix(OperationContext* opCtx,
+                                     const DatabaseName& dbName,
+                                     const std::string& collectionNamePrefix) override;
 
-    virtual void flushAllFiles(OperationContext* opCtx, bool callerHoldsReadLock) override;
+    void flushAllFiles(OperationContext* opCtx, bool callerHoldsReadLock) override;
 
-    virtual Status beginBackup(OperationContext* opCtx) override;
+    Status beginBackup(OperationContext* opCtx) override;
 
-    virtual void endBackup(OperationContext* opCtx) override;
+    void endBackup(OperationContext* opCtx) override;
 
-    virtual Status disableIncrementalBackup(OperationContext* opCtx) override;
+    Status disableIncrementalBackup(OperationContext* opCtx) override;
 
-    virtual StatusWith<std::unique_ptr<StreamingCursor>> beginNonBlockingBackup(
-        OperationContext* opCtx,
-        boost::optional<Timestamp> checkpointTimestamp,
-        const BackupOptions& options) override;
+    StatusWith<std::unique_ptr<StreamingCursor>> beginNonBlockingBackup(
+        OperationContext* opCtx, const BackupOptions& options) override;
 
-    virtual void endNonBlockingBackup(OperationContext* opCtx) override;
+    void endNonBlockingBackup(OperationContext* opCtx) override;
 
-    virtual StatusWith<std::deque<std::string>> extendBackupCursor(
-        OperationContext* opCtx) override;
+    StatusWith<std::deque<std::string>> extendBackupCursor(OperationContext* opCtx) override;
 
-    virtual bool supportsCheckpoints() const override;
+    bool supportsCheckpoints() const override;
 
-    virtual bool isEphemeral() const override;
+    bool isEphemeral() const override;
 
-    virtual Status repairRecordStore(OperationContext* opCtx,
-                                     RecordId catalogId,
-                                     const NamespaceString& nss) override;
+    Status repairRecordStore(OperationContext* opCtx,
+                             RecordId catalogId,
+                             const NamespaceString& nss) override;
 
-    virtual std::unique_ptr<TemporaryRecordStore> makeTemporaryRecordStore(
+    std::unique_ptr<TemporaryRecordStore> makeTemporaryRecordStore(OperationContext* opCtx,
+                                                                   KeyFormat keyFormat) override;
+
+    std::unique_ptr<TemporaryRecordStore> makeTemporaryRecordStoreForResumableIndexBuild(
         OperationContext* opCtx, KeyFormat keyFormat) override;
 
-    virtual std::unique_ptr<TemporaryRecordStore> makeTemporaryRecordStoreForResumableIndexBuild(
-        OperationContext* opCtx, KeyFormat keyFormat) override;
+    std::unique_ptr<TemporaryRecordStore> makeTemporaryRecordStoreFromExistingIdent(
+        OperationContext* opCtx, StringData ident, KeyFormat keyFormat) override;
 
-    virtual std::unique_ptr<TemporaryRecordStore> makeTemporaryRecordStoreFromExistingIdent(
-        OperationContext* opCtx, StringData ident) override;
+    void cleanShutdown(ServiceContext* svcCtx) override;
 
-    virtual void cleanShutdown(ServiceContext* svcCtx, bool memLeakAllowed) override;
+    void setStableTimestamp(Timestamp stableTimestamp, bool force = false) override;
 
-    virtual void setStableTimestamp(Timestamp stableTimestamp, bool force = false) override;
+    Timestamp getStableTimestamp() const override;
 
-    virtual Timestamp getStableTimestamp() const override;
+    void setInitialDataTimestamp(Timestamp initialDataTimestamp) override;
 
-    virtual void setInitialDataTimestamp(Timestamp initialDataTimestamp) override;
+    Timestamp getInitialDataTimestamp() const override;
 
-    virtual Timestamp getInitialDataTimestamp() const override;
+    void setOldestTimestampFromStable() override;
 
-    virtual void setOldestTimestampFromStable() override;
+    void setOldestTimestamp(Timestamp newOldestTimestamp, bool force) override;
 
-    virtual void setOldestTimestamp(Timestamp newOldestTimestamp) override;
+    Timestamp getOldestTimestamp() const override;
 
-    virtual Timestamp getOldestTimestamp() const override;
-
-    virtual void setOldestActiveTransactionTimestampCallback(
+    void setOldestActiveTransactionTimestampCallback(
         StorageEngine::OldestActiveTransactionTimestampCallback) override;
 
-    virtual bool supportsRecoverToStableTimestamp() const override;
+    bool supportsRecoverToStableTimestamp() const override;
 
-    virtual bool supportsRecoveryTimestamp() const override;
+    bool supportsRecoveryTimestamp() const override;
 
-    virtual StatusWith<Timestamp> recoverToStableTimestamp(OperationContext* opCtx) override;
+    StatusWith<Timestamp> recoverToStableTimestamp(OperationContext* opCtx) override;
 
-    virtual boost::optional<Timestamp> getRecoveryTimestamp() const override;
+    boost::optional<Timestamp> getRecoveryTimestamp() const override;
 
-    virtual boost::optional<Timestamp> getLastStableRecoveryTimestamp() const override;
+    boost::optional<Timestamp> getLastStableRecoveryTimestamp() const override;
 
-    virtual Timestamp getAllDurableTimestamp() const override;
+    Timestamp getAllDurableTimestamp() const override;
 
     boost::optional<Timestamp> getOplogNeededForCrashRecovery() const final;
 
@@ -315,7 +338,7 @@ public:
     }
 
     void addDropPendingIdent(
-        const stdx::variant<Timestamp, StorageEngine::CheckpointIteration>& dropTime,
+        const std::variant<Timestamp, StorageEngine::CheckpointIteration>& dropTime,
         std::shared_ptr<Ident> ident,
         DropIdentCallback&& onDrop) override;
 
@@ -325,11 +348,11 @@ public:
 
     void startTimestampMonitor() override;
 
-    void checkpoint(OperationContext* opCtx) override;
+    void checkpoint() override;
 
     StorageEngine::CheckpointIteration getCheckpointIteration() const override;
 
-    virtual bool hasDataBeenCheckpointed(
+    bool hasDataBeenCheckpointed(
         StorageEngine::CheckpointIteration checkpointIteration) const override;
 
     StatusWith<ReconcileResult> reconcileCatalogAndIdents(
@@ -342,7 +365,7 @@ public:
     const DurableCatalog* getCatalog() const override;
 
     /**
-     * When loading after an unclean shutdown, this performs cleanup on the DurableCatalogImpl.
+     * When loading after an unclean shutdown, this performs cleanup on the DurableCatalog.
      */
     void loadCatalog(OperationContext* opCtx,
                      boost::optional<Timestamp> stableTs,
@@ -386,6 +409,8 @@ public:
 
     void dump() const override;
 
+    Status autoCompact(OperationContext* opCtx, const AutoCompactOptions& options) override;
+
 private:
     using CollIter = std::list<std::string>::iterator;
 
@@ -393,14 +418,15 @@ private:
                          RecordId catalogId,
                          const NamespaceString& nss,
                          bool forRepair,
-                         Timestamp minVisibleTs,
                          Timestamp minValidTs);
 
-    Status _dropCollections(OperationContext* opCtx, const std::vector<UUID>& toDrop);
+    Status _dropCollections(OperationContext* opCtx,
+                            const std::vector<UUID>& toDrop,
+                            const std::string& collectionNamePrefix = "");
 
     /**
      * When called in a repair context (_options.forRepair=true), attempts to recover a collection
-     * whose entry is present in the DurableCatalogImpl, but missing from the KVEngine. Returns an
+     * whose entry is present in the DurableCatalog, but missing from the KVEngine. Returns an
      * error Status if called outside of a repair context or the implementation of
      * KVEngine::recoverOrphanedIdent returns an error other than DataModifiedByRepair.
      *
@@ -454,17 +480,13 @@ private:
     // Listener for min of checkpoint and oldest timestamp changes.
     TimestampMonitor::TimestampListener _minOfCheckpointAndOldestTimestampListener;
 
-    // Listener for checkpoint timestamp changes to remove historical ident entries older than the
-    // checkpoint timestamp.
-    TimestampMonitor::TimestampListener _historicalIdentTimestampListener;
-
     // Listener for cleanup of CollectionCatalog when oldest timestamp advances.
     TimestampMonitor::TimestampListener _collectionCatalogCleanupTimestampListener;
 
     const bool _supportsCappedCollections;
 
     std::unique_ptr<RecordStore> _catalogRecordStore;
-    std::unique_ptr<DurableCatalogImpl> _catalog;
+    std::unique_ptr<DurableCatalog> _catalog;
 
     // Flag variable that states if the storage engine is in backup mode.
     bool _inBackupMode = false;
