@@ -28,21 +28,30 @@
  */
 
 
-#include "mongo/platform/basic.h"
-
-#include <string>
-
 #include <boost/filesystem/operations.hpp>
+#include <string>
+#include <utility>
 
-#include "mongo/db/storage/storage_util.h"
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+// IWYU pragma: no_include "boost/system/detail/error_code.hpp"
 
+#include "mongo/bson/timestamp.h"
 #include "mongo/db/catalog/collection_catalog.h"
+#include "mongo/db/database_name.h"
+#include "mongo/db/service_context.h"
 #include "mongo/db/storage/durable_catalog.h"
 #include "mongo/db/storage/ident.h"
 #include "mongo/db/storage/kv/kv_engine.h"
+#include "mongo/db/storage/recovery_unit.h"
 #include "mongo/db/storage/storage_engine.h"
-#include "mongo/db/storage/storage_parameters_gen.h"
+#include "mongo/db/storage/storage_util.h"
+#include "mongo/db/transaction_resources.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/util/assert_util_core.h"
+#include "mongo/util/uuid.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
 
@@ -50,14 +59,14 @@
 namespace mongo {
 namespace catalog {
 namespace {
-BSONObj toBSON(const stdx::variant<Timestamp, StorageEngine::CheckpointIteration>& x) {
-    return stdx::visit(OverloadedVisitor{[](const Timestamp& ts) { return ts.toBSON(); },
-                                         [](const StorageEngine::CheckpointIteration& iter) {
-                                             auto underlyingValue = uint64_t{iter};
-                                             return BSON("checkpointIteration"
-                                                         << std::to_string(underlyingValue));
-                                         }},
-                       x);
+BSONObj toBSON(const std::variant<Timestamp, StorageEngine::CheckpointIteration>& x) {
+    return visit(OverloadedVisitor{[](const Timestamp& ts) { return ts.toBSON(); },
+                                   [](const StorageEngine::CheckpointIteration& iter) {
+                                       auto underlyingValue = uint64_t{iter};
+                                       return BSON("checkpointIteration"
+                                                   << std::to_string(underlyingValue));
+                                   }},
+                 x);
 }
 }  // namespace
 
@@ -92,15 +101,13 @@ void removeIndex(OperationContext* opCtx,
     //
     // Index creation (and deletion) are allowed in multi-document transactions that use the same
     // RecoveryUnit throughout but not the same OperationContext.
-    auto recoveryUnit = opCtx->recoveryUnit();
+    auto recoveryUnit = shard_role_details::getRecoveryUnit(opCtx);
     auto storageEngine = opCtx->getServiceContext()->getStorageEngine();
 
     const bool isTwoPhaseDrop =
         storageEngine->supportsPendingDrops() && dataRemoval == DataRemoval::kTwoPhase;
 
-    // TODO SERVER-68674: Remove feature flag check.
-    // (Ignore FCV check): This feature flag doesn't have any upgrade/downgrade concerns.
-    if (feature_flags::gPointInTimeCatalogLookups.isEnabledAndIgnoreFCVUnsafe() && isTwoPhaseDrop) {
+    if (isTwoPhaseDrop) {
         invariant(entry);
         CollectionCatalog::get(opCtx)->dropIndex(
             opCtx, collection->ns(), entry, /*isDropPending=*/true);
@@ -108,7 +115,7 @@ void removeIndex(OperationContext* opCtx,
 
     // Schedule the second phase of drop to delete the data when it is no longer in use, if the
     // first phase is successfully committed.
-    opCtx->recoveryUnit()->onCommit(
+    shard_role_details::getRecoveryUnit(opCtx)->onCommitForTwoPhaseDrop(
         [svcCtx = opCtx->getServiceContext(),
          recoveryUnit,
          storageEngine,
@@ -119,11 +126,7 @@ void removeIndex(OperationContext* opCtx,
          isTwoPhaseDrop](OperationContext*, boost::optional<Timestamp> commitTimestamp) {
             StorageEngine::DropIdentCallback onDrop =
                 [svcCtx, storageEngine, nss, ident = ident->getIdent(), isTwoPhaseDrop] {
-                    // TODO SERVER-68674: Remove feature flag check.
-                    // (Ignore FCV check): This feature flag doesn't have any upgrade/downgrade
-                    // concerns.
-                    if (feature_flags::gPointInTimeCatalogLookups.isEnabledAndIgnoreFCVUnsafe() &&
-                        isTwoPhaseDrop) {
+                    if (isTwoPhaseDrop) {
                         CollectionCatalog::write(svcCtx, [&](CollectionCatalog& catalog) {
                             catalog.notifyIdentDropped(ident);
                         });
@@ -131,7 +134,7 @@ void removeIndex(OperationContext* opCtx,
                 };
 
             if (isTwoPhaseDrop) {
-                stdx::variant<Timestamp, StorageEngine::CheckpointIteration> dropTime;
+                std::variant<Timestamp, StorageEngine::CheckpointIteration> dropTime;
                 if (!commitTimestamp) {
                     // Standalone mode and unreplicated drops will not provide a timestamp. Use the
                     // checkpoint iteration instead.
@@ -180,22 +183,18 @@ Status dropCollection(OperationContext* opCtx,
     //
     // Create (and drop) collection are allowed in multi-document transactions that use the same
     // RecoveryUnit throughout but not the same OperationContext.
-    auto recoveryUnit = opCtx->recoveryUnit();
+    auto recoveryUnit = shard_role_details::getRecoveryUnit(opCtx);
     auto storageEngine = opCtx->getServiceContext()->getStorageEngine();
 
 
     // Schedule the second phase of drop to delete the data when it is no longer in use, if the
     // first phase is successfully committed.
-    opCtx->recoveryUnit()->onCommit(
+    shard_role_details::getRecoveryUnit(opCtx)->onCommitForTwoPhaseDrop(
         [svcCtx = opCtx->getServiceContext(), recoveryUnit, storageEngine, nss, ident](
             OperationContext*, boost::optional<Timestamp> commitTimestamp) {
             StorageEngine::DropIdentCallback onDrop =
                 [svcCtx, storageEngine, nss, ident = ident->getIdent()] {
-                    // TODO SERVER-68674: Remove feature flag check.
-                    // (Ignore FCV check): This feature flag doesn't have any upgrade/downgrade
-                    // concerns.
-                    if (feature_flags::gPointInTimeCatalogLookups.isEnabledAndIgnoreFCVUnsafe() &&
-                        storageEngine->supportsPendingDrops()) {
+                    if (storageEngine->supportsPendingDrops()) {
                         CollectionCatalog::write(svcCtx, [&](CollectionCatalog& catalog) {
                             catalog.notifyIdentDropped(ident);
                         });
@@ -203,7 +202,7 @@ Status dropCollection(OperationContext* opCtx,
                 };
 
             if (storageEngine->supportsPendingDrops()) {
-                stdx::variant<Timestamp, StorageEngine::CheckpointIteration> dropTime;
+                std::variant<Timestamp, StorageEngine::CheckpointIteration> dropTime;
                 if (!commitTimestamp) {
                     // Standalone mode and unreplicated drops will not provide a timestamp. Use the
                     // checkpoint iteration instead.
@@ -221,7 +220,7 @@ Status dropCollection(OperationContext* opCtx,
                 // Intentionally ignoring failure here. Since we've removed the metadata pointing to
                 // the collection, we should never see it again anyway.
                 storageEngine->getEngine()
-                    ->dropIdent(recoveryUnit, ident->getIdent(), std::move(onDrop))
+                    ->dropIdent(recoveryUnit, ident->getIdent(), onDrop)
                     .ignore();
             }
         });
