@@ -28,51 +28,73 @@
  */
 
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/storage/wiredtiger/wiredtiger_cursor.h"
+#include <wiredtiger.h>
 
+#include "mongo/base/error_codes.h"
 #include "mongo/db/storage/recovery_unit.h"
+#include "mongo/db/storage/wiredtiger/wiredtiger_cursor.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_util.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/str.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
 
 
 namespace mongo {
 
-WiredTigerCursor::WiredTigerCursor(const std::string& uri,
+namespace {
+static constexpr StringData kOverwriteFalse = "overwrite=false"_sd;
+}  // namespace
+
+WiredTigerCursor::WiredTigerCursor(WiredTigerRecoveryUnit& ru,
+                                   const std::string& uri,
                                    uint64_t tableID,
-                                   bool allowOverwrite,
-                                   OperationContext* opCtx) {
+                                   bool allowOverwrite) {
     _tableID = tableID;
-    _ru = WiredTigerRecoveryUnit::get(opCtx);
+    _ru = &ru;
     _session = _ru->getSession();
     _isCheckpoint =
         (_ru->getTimestampReadSource() == WiredTigerRecoveryUnit::ReadSource::kCheckpoint);
 
-    // Construct a new cursor with the provided options.
-    str::stream builder;
-    if (_ru->getReadOnce()) {
-        builder << "read_once=true,";
-    }
-    if (_isCheckpoint) {
-        // Type can be "lsm" or "file".
-        std::string type, sourceURI;
-        WiredTigerUtil::fetchTypeAndSourceURI(opCtx, uri, &type, &sourceURI);
-        uassert(ErrorCodes::InvalidOptions,
-                str::stream() << "LSM does not support opening cursors by checkpoint",
-                type != "lsm");
+    // Passing nullptr is significantly faster for WiredTiger than passing an empty string.
+    const char* configStr = nullptr;
 
-        builder << "checkpoint=WiredTigerCheckpoint,";
-    }
-    // Add this option last to avoid needing a trailing comma. This enables an optimization in
-    // WiredTiger to skip parsing the config string. See SERVER-43232 for details.
-    if (!allowOverwrite) {
-        builder << "overwrite=false";
-    }
+    // If we have uncommon cursor options, use a costlier string builder.
+    if (_ru->getReadOnce() || _isCheckpoint) {
+        str::stream builder;
+        if (_ru->getReadOnce()) {
+            builder << "read_once=true,";
+        }
 
-    _config = builder;
+        if (_isCheckpoint) {
+            // Type can be "lsm" or "file".
+            std::string type, sourceURI;
+            WiredTigerUtil::fetchTypeAndSourceURI(ru, uri, &type, &sourceURI);
+            uassert(ErrorCodes::InvalidOptions,
+                    str::stream() << "LSM does not support opening cursors by checkpoint",
+                    type != "lsm");
+
+            builder << "checkpoint=WiredTigerCheckpoint,";
+        }
+
+        // Add this option last as the string does not have a trailing comma.
+        if (!allowOverwrite) {
+            builder << kOverwriteFalse;
+        }
+
+        _config = builder;
+        configStr = _config.c_str();
+    } else {
+        // Add this option without a trailing comma. This enables an optimization in WiredTiger to
+        // skip parsing the config string if this is the only option. See SERVER-43232 for details.
+        if (!allowOverwrite) {
+            _config = kOverwriteFalse.toString();
+            configStr = kOverwriteFalse.data();
+        }
+    }
 
     // Attempt to retrieve a cursor from the cache.
     _cursor = _session->getCachedCursor(tableID, _config);
@@ -81,12 +103,12 @@ WiredTigerCursor::WiredTigerCursor(const std::string& uri,
     }
 
     try {
-        _cursor = _session->getNewCursor(uri, _config.c_str());
+        _cursor = _session->getNewCursor(uri, configStr);
     } catch (const ExceptionFor<ErrorCodes::CursorNotFound>& ex) {
         // A WiredTiger table will not be available in the latest checkpoint if the checkpoint
         // thread hasn't run after the initial WiredTiger table was created.
         if (!_isCheckpoint) {
-            LOGV2_FATAL_NOTRACE(50883, "{ex}", "Cursor not found", "error"_attr = ex);
+            LOGV2_FATAL_NOTRACE(50883, "Cursor not found", "error"_attr = ex);
         }
         throw;
     }
@@ -97,20 +119,16 @@ WiredTigerCursor::~WiredTigerCursor() {
         // Closes the checkpoint cursor to avoid outdated data view when opening a new one.
         _session->closeCursor(_cursor);
     } else {
-        _session->releaseCursor(_tableID, _cursor, _config);
+        _session->releaseCursor(_tableID, _cursor, std::move(_config));
     }
 }
 
-void WiredTigerCursor::reset() {
-    invariantWTOK(_cursor->reset(_cursor), _cursor->session);
-}
-
-WiredTigerBulkLoadCursor::WiredTigerBulkLoadCursor(const std::string& indexUri,
-                                                   OperationContext* opCtx)
-    : _session(WiredTigerRecoveryUnit::get(opCtx)->getSessionCache()->getSession()) {
+WiredTigerBulkLoadCursor::WiredTigerBulkLoadCursor(WiredTigerRecoveryUnit& ru,
+                                                   const std::string& indexUri)
+    : _session(ru.getSessionCache()->getSession()) {
     // Open cursors can cause bulk open_cursor to fail with EBUSY.
     // TODO any other cases that could cause EBUSY?
-    WiredTigerSession* outerSession = WiredTigerRecoveryUnit::get(opCtx)->getSession();
+    WiredTigerSession* outerSession = ru.getSession();
     outerSession->closeAllCursors(indexUri);
 
     // The 'checkpoint_wait=false' option is set to prefer falling back on the "non-bulk" cursor
@@ -123,8 +141,6 @@ WiredTigerBulkLoadCursor::WiredTigerBulkLoadCursor(const std::string& indexUri,
     }
 
     LOGV2_WARNING(51783,
-                  "failed to create WiredTiger bulk cursor: {error} falling back to non-bulk "
-                  "cursor for index {index}",
                   "Failed to create WiredTiger bulk cursor, falling back to non-bulk",
                   "error"_attr = wiredtiger_strerror(err),
                   "index"_attr = indexUri);
