@@ -29,14 +29,32 @@
 
 #pragma once
 
+#include <boost/optional/optional.hpp>
+#include <cstddef>
+#include <cstdint>
+#include <memory>
+#include <string>
 #include <wiredtiger.h>
 
+#include "mongo/base/status.h"
 #include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/db/catalog/validate_results.h"
+#include "mongo/db/index/index_descriptor.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/record_id.h"
+#include "mongo/db/storage/duplicate_key_error_info.h"
 #include "mongo/db/storage/index_entry_comparison.h"
+#include "mongo/db/storage/key_format.h"
 #include "mongo/db/storage/key_string.h"
 #include "mongo/db/storage/sorted_data_interface.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_prepare_conflict.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_recovery_unit.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/uuid.h"
 
 namespace mongo {
 
@@ -109,7 +127,7 @@ public:
      * Creates a WiredTiger table suitable for implementing a MongoDB index.
      * 'config' should be created with generateCreateString().
      */
-    static Status Create(OperationContext* opCtx,
+    static Status create(OperationContext* opCtx,
                          const std::string& uri,
                          const std::string& config);
 
@@ -123,43 +141,45 @@ public:
      */
     WiredTigerIndex(OperationContext* ctx,
                     const std::string& uri,
+                    const UUID& collectionUUID,
                     StringData ident,
                     KeyFormat rsKeyFormat,
                     const IndexDescriptor* desc,
-                    bool isLogged,
-                    bool readOnly);
+                    bool isLogged);
 
-    virtual Status insert(OperationContext* opCtx,
-                          const KeyString::Value& keyString,
-                          bool dupsAllowed);
+    Status insert(OperationContext* opCtx,
+                  const key_string::Value& keyString,
+                  bool dupsAllowed,
+                  IncludeDuplicateRecordId includeDuplicateRecordId =
+                      IncludeDuplicateRecordId::kOff) override;
 
-    virtual void unindex(OperationContext* opCtx,
-                         const KeyString::Value& keyString,
-                         bool dupsAllowed);
+    void unindex(OperationContext* opCtx,
+                 const key_string::Value& keyString,
+                 bool dupsAllowed) override;
 
-    virtual boost::optional<RecordId> findLoc(OperationContext* opCtx,
-                                              const KeyString::Value& keyString) const override;
+    boost::optional<RecordId> findLoc(OperationContext* opCtx, StringData keyString) const override;
 
-    virtual void fullValidate(OperationContext* opCtx,
-                              long long* numKeysOut,
-                              IndexValidateResults* fullResults) const;
-    virtual bool appendCustomStats(OperationContext* opCtx,
-                                   BSONObjBuilder* output,
-                                   double scale) const;
-    virtual Status dupKeyCheck(OperationContext* opCtx, const KeyString::Value& keyString);
+    IndexValidateResults validate(OperationContext* opCtx, bool full) const override;
 
-    virtual bool isEmpty(OperationContext* opCtx);
+    bool appendCustomStats(OperationContext* opCtx,
+                           BSONObjBuilder* output,
+                           double scale) const override;
+    Status dupKeyCheck(OperationContext* opCtx, const key_string::Value& keyString) override;
 
-    virtual long long getSpaceUsedBytes(OperationContext* opCtx) const;
+    bool isEmpty(OperationContext* opCtx) override;
 
-    virtual long long getFreeStorageBytes(OperationContext* opCtx) const;
+    int64_t numEntries(OperationContext* opCtx) const override;
 
-    virtual Status initAsEmpty(OperationContext* opCtx);
+    long long getSpaceUsedBytes(OperationContext* opCtx) const override;
 
-    virtual void printIndexEntryMetadata(OperationContext* opCtx,
-                                         const KeyString::Value& keyString) const;
+    long long getFreeStorageBytes(OperationContext* opCtx) const override;
 
-    Status compact(OperationContext* opCtx) override;
+    Status initAsEmpty(OperationContext* opCtx) override;
+
+    void printIndexEntryMetadata(OperationContext* opCtx,
+                                 const key_string::Value& keyString) const override;
+
+    StatusWith<int64_t> compact(OperationContext* opCtx, const CompactOptions& options) override;
 
     const std::string& uri() const {
         return _uri;
@@ -185,59 +205,85 @@ public:
         return false;
     }
 
+    virtual UUID getCollectionUUID() const {
+        return _collectionUUID;
+    }
+
     virtual bool isDup(OperationContext* opCtx,
                        WT_CURSOR* c,
-                       const KeyString::Value& keyString) = 0;
+                       WiredTigerSession* session,
+                       const key_string::Value& keyString) = 0;
     virtual bool unique() const = 0;
     virtual bool isTimestampSafeUniqueIdx() const = 0;
-    void insertWithRecordIdInValue_forTest(OperationContext* opCtx,
-                                           const KeyString::Value& keyString,
-                                           RecordId rid) override {
-        MONGO_UNREACHABLE;
+
+    bool hasOldFormatVersion() const {
+        return _dataFormatVersion == kDataFormatV3KeyStringV0UniqueIndexVersionV1 ||
+            _dataFormatVersion == kDataFormatV4KeyStringV1UniqueIndexVersionV2;
     }
 
 protected:
-    virtual Status _insert(OperationContext* opCtx,
-                           WT_CURSOR* c,
-                           const KeyString::Value& keyString,
-                           bool dupsAllowed) = 0;
+    virtual Status _insert(
+        OperationContext* opCtx,
+        WT_CURSOR* c,
+        WiredTigerSession* session,
+        const key_string::Value& keyString,
+        bool dupsAllowed,
+        IncludeDuplicateRecordId includeDuplicateRecordId = IncludeDuplicateRecordId::kOff) = 0;
 
     virtual void _unindex(OperationContext* opCtx,
                           WT_CURSOR* c,
-                          const KeyString::Value& keyString,
+                          const key_string::Value& keyString,
                           bool dupsAllowed) = 0;
 
     void setKey(WT_CURSOR* cursor, const WT_ITEM* item);
-    void getKey(OperationContext* opCtx, WT_CURSOR* cursor, WT_ITEM* key);
 
     /**
-     * Checks whether the prefix key defined by 'buffer' and 'size' is in the index. If it is,
-     * returns the RecordId of the first matching key and positions the cursor 'c' on that key.
+     * Load the key positioned by this cursor into 'key'. When metrics is provided, count the
+     * size towards resource consumption metrics.
+     */
+    void getKey(WT_CURSOR* cursor, WT_ITEM* key, ResourceConsumption::MetricsCollector* metrics);
+
+    /**
+     * Checks whether the prefix key defined by 'keyString' and 'sizeWithoutRecordId' is in the
+     * index. If it is, returns the RecordId of the first matching key and positions the cursor 'c'
+     * on that key.
      */
     boost::optional<RecordId> _keyExists(OperationContext* opCtx,
                                          WT_CURSOR* c,
-                                         const char* buffer,
-                                         size_t size);
+                                         WiredTigerSession* session,
+                                         const key_string::Value& keyString,
+                                         size_t sizeWithoutRecordId);
+
+    /**
+     * Sets the upper bound on the passed in cursor to be the maximum value of the KeyString prefix.
+     * Used when checking if a specific key prefix exists.
+     */
+    void _setUpperBoundForKeyExists(WT_CURSOR* c,
+                                    WiredTigerSession* session,
+                                    const key_string::Value& keyString,
+                                    size_t sizeWithoutRecordId);
 
     /**
      * Returns a DuplicateKey error if the prefix key exists in the index with a different RecordId.
      * Returns true if the prefix key exists in the index with the same RecordId. Returns false if
      * the prefix key does not exist in the index. Should only be used for non-_id indexes.
      */
-    StatusWith<bool> _checkDups(OperationContext* opCtx,
-                                WT_CURSOR* c,
-                                const KeyString::Value& keyString);
+    StatusWith<bool> _checkDups(
+        OperationContext* opCtx,
+        WT_CURSOR* c,
+        WiredTigerSession* session,
+        const key_string::Value& keyString,
+        IncludeDuplicateRecordId includeDuplicateRecordId = IncludeDuplicateRecordId::kOff);
 
     /*
      * Determines the data format version from application metadata and verifies compatibility.
      * Returns the corresponding KeyString version.
      */
-    KeyString::Version _handleVersionInfo(OperationContext* ctx,
-                                          const std::string& uri,
-                                          StringData ident,
-                                          const IndexDescriptor* desc,
-                                          bool isLogged,
-                                          bool isReadOnly);
+    key_string::Version _handleVersionInfo(OperationContext* ctx,
+                                           const std::string& uri,
+                                           StringData ident,
+                                           const IndexDescriptor* desc,
+                                           bool isLogged);
 
     /*
      * Attempts to repair the data format version in the index table metadata if there is a mismatch
@@ -260,9 +306,9 @@ protected:
      * constructed.
      */
     int _dataFormatVersion;
-    std::string _uri;
+    const std::string _uri;
     uint64_t _tableId;
-    const IndexDescriptor* _desc;
+    const UUID _collectionUUID;
     const std::string _indexName;
     const BSONObj _keyPattern;
     const BSONObj _collation;
@@ -273,11 +319,11 @@ class WiredTigerIndexUnique : public WiredTigerIndex {
 public:
     WiredTigerIndexUnique(OperationContext* ctx,
                           const std::string& uri,
+                          const UUID& collectionUUID,
                           StringData ident,
                           KeyFormat rsKeyFormat,
                           const IndexDescriptor* desc,
-                          bool isLogged,
-                          bool readOnly = false);
+                          bool isLogged);
 
     std::unique_ptr<SortedDataInterface::Cursor> newCursor(OperationContext* opCtx,
                                                            bool forward) const override;
@@ -291,45 +337,50 @@ public:
 
     bool isTimestampSafeUniqueIdx() const override;
 
-    bool isDup(OperationContext* opCtx, WT_CURSOR* c, const KeyString::Value& keyString) override;
+    bool isDup(OperationContext* opCtx,
+               WT_CURSOR* c,
+               WiredTigerSession* session,
+               const key_string::Value& keyString) override;
 
-    void insertWithRecordIdInValue_forTest(OperationContext* opCtx,
-                                           const KeyString::Value& keyString,
-                                           RecordId rid) override;
 
 protected:
     Status _insert(OperationContext* opCtx,
                    WT_CURSOR* c,
-                   const KeyString::Value& keyString,
-                   bool dupsAllowed) override;
+                   WiredTigerSession* session,
+                   const key_string::Value& keyString,
+                   bool dupsAllowed,
+                   IncludeDuplicateRecordId includeDuplicateRecordId =
+                       IncludeDuplicateRecordId::kOff) override;
 
     void _unindex(OperationContext* opCtx,
                   WT_CURSOR* c,
-                  const KeyString::Value& keyString,
+                  const key_string::Value& keyString,
                   bool dupsAllowed) override;
 
     /**
-     * This function continues to exist in order to support v4.0 unique partial index format: the
-     * format changed in v4.2 and onward. _unindex will call this if an index entry in the new
-     * format cannot be found, and this function will check for the old format.
+     * This function continues to exist in order to support v4.0 unique index format: the format
+     * changed in v4.2 and onward. _unindex will call this if an index entry in the new format
+     * cannot be found, and this function will check for the old format.
      */
     void _unindexTimestampUnsafe(OperationContext* opCtx,
                                  WT_CURSOR* c,
-                                 const KeyString::Value& keyString,
+                                 const key_string::Value& keyString,
                                  bool dupsAllowed);
 
 private:
-    bool _partial;
+    Status _insertOldFormatKey(OperationContext* opCtx,
+                               WT_CURSOR* c,
+                               const key_string::Value& keyString);
 };
 
 class WiredTigerIdIndex : public WiredTigerIndex {
 public:
     WiredTigerIdIndex(OperationContext* ctx,
                       const std::string& uri,
+                      const UUID& collectionUUID,
                       StringData ident,
                       const IndexDescriptor* desc,
-                      bool isLogged,
-                      bool readOnly = false);
+                      bool isLogged);
 
     std::unique_ptr<Cursor> newCursor(OperationContext* opCtx,
                                       bool isForward = true) const override;
@@ -349,7 +400,10 @@ public:
         return false;
     }
 
-    bool isDup(OperationContext* opCtx, WT_CURSOR* c, const KeyString::Value& keyString) override {
+    bool isDup(OperationContext* opCtx,
+               WT_CURSOR* c,
+               WiredTigerSession* session,
+               const key_string::Value& keyString) override {
         // Unimplemented by _id indexes for lack of need
         MONGO_UNREACHABLE;
     }
@@ -357,12 +411,15 @@ public:
 protected:
     Status _insert(OperationContext* opCtx,
                    WT_CURSOR* c,
-                   const KeyString::Value& keyString,
-                   bool dupsAllowed) override;
+                   WiredTigerSession* session,
+                   const key_string::Value& keyString,
+                   bool dupsAllowed,
+                   IncludeDuplicateRecordId includeDuplicateRecordId =
+                       IncludeDuplicateRecordId::kOff) override;
 
     void _unindex(OperationContext* opCtx,
                   WT_CURSOR* c,
-                  const KeyString::Value& keyString,
+                  const key_string::Value& keyString,
                   bool dupsAllowed) override;
 
     /**
@@ -370,18 +427,19 @@ protected:
      */
     Status _checkDups(OperationContext* opCtx,
                       WT_CURSOR* c,
-                      const KeyString::Value& keyString) = delete;
+                      const key_string::Value& keyString,
+                      IncludeDuplicateRecordId includeDuplicateRecordId) = delete;
 };
 
 class WiredTigerIndexStandard : public WiredTigerIndex {
 public:
     WiredTigerIndexStandard(OperationContext* ctx,
                             const std::string& uri,
+                            const UUID& collectionUUID,
                             StringData ident,
                             KeyFormat rsKeyFormat,
                             const IndexDescriptor* desc,
-                            bool isLogged,
-                            bool readOnly = false);
+                            bool isLogged);
 
     std::unique_ptr<SortedDataInterface::Cursor> newCursor(OperationContext* opCtx,
                                                            bool forward) const override;
@@ -397,7 +455,10 @@ public:
         return false;
     }
 
-    bool isDup(OperationContext* opCtx, WT_CURSOR* c, const KeyString::Value& keyString) override {
+    bool isDup(OperationContext* opCtx,
+               WT_CURSOR* c,
+               WiredTigerSession* session,
+               const key_string::Value& keyString) override {
         // Unimplemented by non-unique indexes
         MONGO_UNREACHABLE;
     }
@@ -405,12 +466,15 @@ public:
 protected:
     Status _insert(OperationContext* opCtx,
                    WT_CURSOR* c,
-                   const KeyString::Value& keyString,
-                   bool dupsAllowed) override;
+                   WiredTigerSession* session,
+                   const key_string::Value& keyString,
+                   bool dupsAllowed,
+                   IncludeDuplicateRecordId includeDuplicateRecordId =
+                       IncludeDuplicateRecordId::kOff) override;
 
     void _unindex(OperationContext* opCtx,
                   WT_CURSOR* c,
-                  const KeyString::Value& keyString,
+                  const key_string::Value& keyString,
                   bool dupsAllowed) override;
 };
 
