@@ -29,24 +29,54 @@
 
 #pragma once
 
+#include <boost/filesystem/path.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <cstddef>
+#include <cstdint>
+#include <deque>
 #include <functional>
 #include <list>
+#include <map>
 #include <memory>
+#include <set>
 #include <string>
-
-#include <boost/filesystem/path.hpp>
+#include <vector>
 #include <wiredtiger.h>
 
+#include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/bson/ordering.h"
 #include "mongo/bson/timestamp.h"
+#include "mongo/db/catalog/collection_options.h"
+#include "mongo/db/catalog/import_options.h"
+#include "mongo/db/index/index_descriptor.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/storage/backup_block.h"
+#include "mongo/db/storage/column_store.h"
 #include "mongo/db/storage/durable_catalog.h"
+#include "mongo/db/storage/journal_listener.h"
+#include "mongo/db/storage/key_format.h"
 #include "mongo/db/storage/kv/kv_engine.h"
+#include "mongo/db/storage/record_store.h"
+#include "mongo/db/storage/recovery_unit.h"
+#include "mongo/db/storage/snapshot_manager.h"
+#include "mongo/db/storage/sorted_data_interface.h"
 #include "mongo/db/storage/storage_engine.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_oplog_manager.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_session_cache.h"
+#include "mongo/db/storage/wiredtiger/wiredtiger_size_storer.h"
+#include "mongo/db/storage/wiredtiger/wiredtiger_snapshot_manager.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_util.h"
+#include "mongo/db/tenant_id.h"
+#include "mongo/platform/atomic_word.h"
 #include "mongo/platform/mutex.h"
+#include "mongo/stdx/condition_variable.h"
+#include "mongo/util/clock_source.h"
 #include "mongo/util/concurrency/with_lock.h"
 #include "mongo/util/elapsed_tracker.h"
 
@@ -54,9 +84,11 @@ namespace mongo {
 
 class ClockSource;
 class JournalListener;
+
 class WiredTigerRecordStore;
 class WiredTigerSessionCache;
 class WiredTigerSizeStorer;
+
 class WiredTigerEngineRuntimeConfigParameter;
 
 /**
@@ -99,7 +131,6 @@ struct WiredTigerBackup {
     WT_CURSOR* dupCursor = nullptr;
     std::set<std::string> logFilePathsSeenByExtendBackupCursor;
     std::set<std::string> logFilePathsSeenByGetNextBatch;
-    BackupBlock::IdentToNamespaceAndUUIDMap identToNamespaceAndUUIDMap;
 
     // 'wtBackupCursorMutex' provides concurrency control between beginNonBlockingBackup(),
     // endNonBlockingBackup(), and getNextBatch() because we stream the output of the backup cursor.
@@ -116,12 +147,65 @@ struct WiredTigerBackup {
     inline static const std::string kOngoingBackupFile = "ongoingBackup.lock";
 };
 
+class SectionActivityPermit {
+public:
+    /**
+     * The default constructor tries to get an Activity Permit. If the section reading acitivities
+     * are permited, _permitActive is set to true.
+     */
+    SectionActivityPermit(WiredTigerEventHandler* eventHandler) : _eventHandler(eventHandler) {
+        if (_eventHandler->getSectionActivityPermit()) {
+            _permitActive = true;
+        }
+    }
+
+    /**
+     * The copy constructor is deleted so that the returned value from tryGetSectionActivityPermit
+     * function is guranteed to use the move constructor.
+     */
+    SectionActivityPermit(const SectionActivityPermit&) = delete;
+
+    /**
+     * The move constructor resets _eventHandler to NULL and _permitActive to false to make the
+     * "other" object stale.
+     */
+    SectionActivityPermit(SectionActivityPermit&& other) noexcept {
+        _eventHandler = other._eventHandler;
+        _permitActive = other._permitActive;
+        other._eventHandler = nullptr;
+        other._permitActive = false;
+    }
+
+    /**
+     * Returns whether the permit is ready.
+     */
+    bool ready() {
+        return _permitActive;
+    }
+
+    /**
+     * When the section generation activity for a reader is done, this destructor is called (one of
+     * the permits is released). The destructor call releases the section generation activity
+     * permit. If it is a _eventHandler is NULL (the object is stale, the permit is tranferred) or
+     * _permitActive is false (no permit was issued), releaseSectionActivityPermit is not called. If
+     * all the permits are released WT connection is allowed to shut down cleanly.
+     */
+    ~SectionActivityPermit() {
+        if (_eventHandler && _permitActive) {
+            _eventHandler->releaseSectionActivityPermit();
+        }
+    }
+
+private:
+    WiredTigerEventHandler* _eventHandler;
+    bool _permitActive{false};
+};
+
 class WiredTigerKVEngine final : public KVEngine {
 public:
     static StringData kTableUriPrefix;
 
-    WiredTigerKVEngine(OperationContext* opCtx,
-                       const std::string& canonicalName,
+    WiredTigerKVEngine(const std::string& canonicalName,
                        const std::string& path,
                        ClockSource* cs,
                        const std::string& extraOpenOptions,
@@ -130,9 +214,10 @@ public:
                        bool ephemeral,
                        bool repair);
 
-    ~WiredTigerKVEngine();
+    ~WiredTigerKVEngine() override;
 
-    void notifyStartupComplete() override;
+    void notifyStorageStartupRecoveryComplete() override;
+    void notifyReplStartupRecoveryComplete(OperationContext* opCtx) override;
 
     void setRecordStoreExtraOptions(const std::string& options);
     void setSortedDataInterfaceExtraOptions(const std::string& options);
@@ -146,7 +231,7 @@ public:
         return !isEphemeral();
     }
 
-    void checkpoint(OperationContext* opCtx) override;
+    void checkpoint() override;
 
     // Force a WT checkpoint, this will not update internal timestamps.
     void forceCheckpoint(bool useStableTimestamp);
@@ -180,6 +265,10 @@ public:
                                                 StringData ident,
                                                 const CollectionOptions& options) override;
 
+    std::unique_ptr<RecordStore> getTemporaryRecordStore(OperationContext* opCtx,
+                                                         StringData ident,
+                                                         KeyFormat keyFormat) override;
+
     std::unique_ptr<RecordStore> makeTemporaryRecordStore(OperationContext* opCtx,
                                                           StringData ident,
                                                           KeyFormat keyFormat) override;
@@ -232,6 +321,10 @@ public:
                      StringData ident,
                      const StorageEngine::DropIdentCallback& onDrop = nullptr) override;
 
+    Status dropIdentSynchronous(RecoveryUnit* ru,
+                                StringData ident,
+                                const StorageEngine::DropIdentCallback& onDrop = nullptr) override;
+
     void dropIdentForImport(OperationContext* opCtx, StringData ident) override;
 
     void alterIdentMetadata(OperationContext* opCtx,
@@ -239,7 +332,7 @@ public:
                             const IndexDescriptor* desc,
                             bool isForceUpdateMetadata) override;
 
-    Status alterMetadata(OperationContext* opCtx, StringData uri, StringData config);
+    Status alterMetadata(StringData uri, StringData config);
 
     void flushAllFiles(OperationContext* opCtx, bool callerHoldsReadLock) override;
 
@@ -250,14 +343,11 @@ public:
     Status disableIncrementalBackup(OperationContext* opCtx) override;
 
     StatusWith<std::unique_ptr<StorageEngine::StreamingCursor>> beginNonBlockingBackup(
-        OperationContext* opCtx,
-        boost::optional<Timestamp> checkpointTimestamp,
-        const StorageEngine::BackupOptions& options) override;
+        OperationContext* opCtx, const StorageEngine::BackupOptions& options) override;
 
     void endNonBlockingBackup(OperationContext* opCtx) override;
 
-    virtual StatusWith<std::deque<std::string>> extendBackupCursor(
-        OperationContext* opCtx) override;
+    StatusWith<std::deque<std::string>> extendBackupCursor(OperationContext* opCtx) override;
 
     int64_t getIdentSize(OperationContext* opCtx, StringData ident) override;
 
@@ -272,7 +362,7 @@ public:
 
     std::vector<std::string> getAllIdents(OperationContext* opCtx) const override;
 
-    void cleanShutdown(bool memLeakAllowed) override;
+    void cleanShutdown() override;
 
     SnapshotManager* getSnapshotManager() const final {
         return &_sessionCache->snapshotManager();
@@ -319,9 +409,9 @@ public:
 
     Timestamp getAllDurableTimestamp() const override;
 
-    bool supportsReadConcernSnapshot() const final override;
+    bool supportsReadConcernSnapshot() const final;
 
-    bool supportsOplogTruncateMarkers() const final override;
+    bool supportsOplogTruncateMarkers() const final;
 
     bool supportsReadConcernMajority() const final;
 
@@ -361,8 +451,6 @@ public:
         return _oplogManager.get();
     }
 
-    static void appendGlobalStats(OperationContext* opCtx, BSONObjBuilder& b);
-
     Timestamp getStableTimestamp() const override;
     Timestamp getOldestTimestamp() const override;
     Timestamp getCheckpointTimestamp() const override;
@@ -411,12 +499,19 @@ public:
                                              Timestamp requestedTimestamp,
                                              bool roundUpIfTooOld) override;
 
+    Status autoCompact(OperationContext* opCtx, const AutoCompactOptions& options) override;
+
 private:
     StatusWith<Timestamp> _pinOldestTimestamp(WithLock,
                                               const std::string& requestingServiceName,
                                               Timestamp requestedTimestamp,
                                               bool roundUpIfTooOld);
 
+    Status _dropIdent(RecoveryUnit* ru,
+                      StringData ident,
+                      const char* config,
+                      const StorageEngine::DropIdentCallback& onDrop = nullptr);
+
 public:
     void unpinOldestTimestamp(const std::string& requestingServiceName) override;
 
@@ -434,6 +529,7 @@ public:
 
     size_t getCacheSizeMB() const override;
 
+    // TODO SERVER-81069: Remove this since it's intrinsically tied to encryption options only.
     BSONObj getSanitizedStorageOptionsForSecondaryReplication(
         const BSONObj& options) const override;
 
@@ -443,6 +539,61 @@ public:
      */
     void sizeStorerPeriodicFlush();
 
+    /**
+     * WT WiredTigerServerStatusSection::generateSection activity is permitted if WT connection is
+     * ready and it is not shutting down. In that case, a tryGetSectionActivityPermit call returns a
+     * SectionActivityPermit object indcating that the caller may safely collect metrics and the
+     * storage engine shutdown will be blocked until the caller is done with collection. When the
+     * metrics collection is not allowed, a tryGetSectionActivityPermit call returns boost::none.
+     * ~SectionActivityPermit releases the permit.
+     */
+    boost::optional<SectionActivityPermit> tryGetSectionActivityPermit() {
+        SectionActivityPermit permit(&_eventHandler);
+        if (permit.ready()) {
+            return permit;
+        }
+        return boost::none;
+    }
+
+    /**
+     * Returns the number of active sections.
+     */
+    int32_t getActiveSections() {
+        return _eventHandler.getActiveSections();
+    }
+
+    /**
+     * If WT connection is made and WT connection is not closing down, WT Connection Ready Status is
+     * true. This function is unsafe because the connection can close immediately after this check
+     * returns true. By calling tryGetSectionActivityPermit(), a permit for section activity can be
+     * acquired and it can be made sure that the connection is open and will not close until the
+     * permit goes out of scope and the destructor for the permit releases it.
+     */
+    bool getWtConnReadyStatus_UNSAFE() {
+        return _eventHandler.getWtConnReadyStatus();
+    }
+    /**
+     * WT WiredTigerServerStatusSection::generateSection activity is permitted if WT connection is
+     * ready and it is not closing down. Calling tryGetSectionActivityPermit() is the recommended
+     * way to get the permit for section metrics collection because it guarantees the release of the
+     * permit. If getSectionActivityPermit_UNSAFE is used to get a permit, the caller *must* make a
+     * subsequent call to `releaseSectionActivityPermit` to allow the storage engine to shut down.
+
+     */
+    bool getSectionActivityPermit_UNSAFE() {
+        return _eventHandler.getSectionActivityPermit();
+    }
+
+    /**
+     * The releaseSectionActivityPermit_UNSAFE() call releases section generation activity permits.
+     * When no permits are held by the readers, WT connection is allowed to shut down cleanly.
+     * releaseSectionActivityPermit_UNSAFE can only be safely called after a preceding
+     * getSectionActivityPermit_UNSAFE call.
+     */
+    void releaseSectionActivityPermit_UNSAFE() {
+        _eventHandler.releaseSectionActivityPermit();
+    }
+
 private:
     class WiredTigerSessionSweeper;
 
@@ -451,7 +602,7 @@ private:
         StorageEngine::DropIdentCallback callback;
     };
 
-    void _checkpoint(OperationContext* opCtx, WT_SESSION* session);
+    void _checkpoint(WT_SESSION* session);
 
     void _checkpoint(WT_SESSION* session, bool useTimestamp);
 
@@ -464,7 +615,7 @@ private:
      */
     void _openWiredTiger(const std::string& path, const std::string& wtOpenConfig);
 
-    Status _salvageIfNeeded(OperationContext* opCtx, const char* uri);
+    Status _salvageIfNeeded(const char* uri);
 
     // Guarantees that the necessary directories exist in case the ident lives in a subdirectory of
     // the database (i.e. because of --directoryPerDb). The caller should hold the lock until
@@ -478,7 +629,7 @@ private:
      * Returns DataModifiedByRepair if the rebuild was successful, and any other error on failure.
      * This will never return Status::OK().
      */
-    Status _rebuildIdent(OperationContext* opCtx, WT_SESSION* session, const char* uri);
+    Status _rebuildIdent(WT_SESSION* session, const char* uri);
 
     bool _hasUri(WT_SESSION* session, const std::string& uri) const;
 
@@ -536,6 +687,8 @@ private:
     std::unique_ptr<WiredTigerSizeStorer> _sizeStorer;
     std::string _sizeStorerUri;
     mutable ElapsedTracker _sizeStorerSyncTracker;
+    mutable Mutex _sizeStorerSyncTrackerMutex =
+        MONGO_MAKE_LATCH("WiredTigerKVEngine::_sizeStorerSyncTrackerMutex");
 
     bool _ephemeral;  // whether we are using the in-memory mode of the WT engine
     const bool _inRepairMode;
