@@ -27,46 +27,65 @@
  *    it in the license file.
  */
 
-
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/storage/kv/kv_engine_test_harness.h"
-
-#include <boost/filesystem.hpp>
 #include <boost/filesystem/fstream.hpp>
+#include <boost/filesystem/operations.hpp>
 #include <boost/filesystem/path.hpp>
-#include <memory>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <ostream>
+#include <utility>
+// IWYU pragma: no_include "boost/system/detail/error_code.hpp"
 
-#include "mongo/base/init.h"
+#include <fmt/format.h>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/init.h"  // IWYU pragma: keep
+#include "mongo/base/initializer.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/db/client.h"
+#include "mongo/db/concurrency/d_concurrency.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
 #include "mongo/db/global_settings.h"
-#include "mongo/db/operation_context_noop.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/record_id.h"
 #include "mongo/db/repl/repl_settings.h"
+#include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/repl/replication_coordinator_mock.h"
+#include "mongo/db/server_options.h"
 #include "mongo/db/service_context.h"
 #include "mongo/db/service_context_test_fixture.h"
 #include "mongo/db/storage/checkpointer.h"
+#include "mongo/db/storage/kv/kv_engine_test_harness.h"
+#include "mongo/db/storage/record_data.h"
 #include "mongo/db/storage/storage_engine_impl.h"
-#include "mongo/db/storage/wiredtiger/wiredtiger_global_options.h"
+#include "mongo/db/storage/storage_options.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_kv_engine.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_record_store.h"
+#include "mongo/db/storage/wiredtiger/wiredtiger_recovery_unit.h"
+#include "mongo/db/storage/write_unit_of_work.h"
+#include "mongo/db/transaction_resources.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/log_severity.h"
+#include "mongo/platform/atomic_proxy.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
 #include "mongo/unittest/log_test.h"
 #include "mongo/unittest/temp_dir.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/util/assert_util_core.h"
 #include "mongo/util/clock_source_mock.h"
+#include "mongo/util/fail_point.h"
+#include "mongo/util/scopeguard.h"
+#include "mongo/util/time_support.h"
+#include "mongo/util/version/releases.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
 
-
 namespace mongo {
 namespace {
 
-#if __has_feature(address_sanitizer)
-constexpr bool kMemLeakAllowed = false;
-#else
-constexpr bool kMemLeakAllowed = true;
-#endif
-
 class WiredTigerKVHarnessHelper : public KVHarnessHelper {
 public:
     WiredTigerKVHarnessHelper(ServiceContext* svcCtx, bool forRepair = false)
@@ -79,22 +98,22 @@ public:
         repl::ReplicationCoordinator::set(
             svcCtx, std::make_unique<repl::ReplicationCoordinatorMock>(svcCtx, replSettings));
         _svcCtx->setStorageEngine(makeEngine());
-        getWiredTigerKVEngine()->notifyStartupComplete();
+        getWiredTigerKVEngine()->notifyStorageStartupRecoveryComplete();
     }
 
-    ~WiredTigerKVHarnessHelper() {
-        getWiredTigerKVEngine()->cleanShutdown(kMemLeakAllowed);
+    ~WiredTigerKVHarnessHelper() override {
+        getWiredTigerKVEngine()->cleanShutdown();
     }
 
-    virtual KVEngine* restartEngine() override {
-        getEngine()->cleanShutdown(kMemLeakAllowed);
+    KVEngine* restartEngine() override {
+        getEngine()->cleanShutdown();
         _svcCtx->clearStorageEngine();
         _svcCtx->setStorageEngine(makeEngine());
-        getEngine()->notifyStartupComplete();
+        getEngine()->notifyStorageStartupRecoveryComplete();
         return getEngine();
     }
 
-    virtual KVEngine* getEngine() override {
+    KVEngine* getEngine() override {
         return _svcCtx->getStorageEngine()->getEngine();
     }
 
@@ -107,10 +126,7 @@ private:
         // Use a small journal for testing to account for the unlikely event that the underlying
         // filesystem does not support fast allocation of a file of zeros.
         std::string extraStrings = "log=(file_max=1m,prealloc=false)";
-        auto client = _svcCtx->makeClient("opCtx");
-        auto opCtx = client->makeOperationContext();
-        auto kv = std::make_unique<WiredTigerKVEngine>(opCtx.get(),
-                                                       kWiredTigerEngineName,
+        auto kv = std::make_unique<WiredTigerKVEngine>(std::string{kWiredTigerEngineName},
                                                        _dbpath.path(),
                                                        _cs.get(),
                                                        extraStrings,
@@ -118,6 +134,9 @@ private:
                                                        0,
                                                        false,
                                                        _forRepair);
+
+        auto client = _svcCtx->getService()->makeClient("opCtx");
+        auto opCtx = client->makeOperationContext();
         StorageEngineOptions options;
         return std::make_unique<StorageEngineImpl>(opCtx.get(), std::move(kv), options);
     }
@@ -135,10 +154,10 @@ public:
 protected:
     ServiceContext::UniqueOperationContext _makeOperationContext() {
         auto opCtx = makeOperationContext();
-        opCtx->setRecoveryUnit(
+        shard_role_details::setRecoveryUnit(
+            opCtx.get(),
             std::unique_ptr<RecoveryUnit>(_helper.getEngine()->newRecoveryUnit()),
             WriteUnitOfWork::RecoveryUnitState::kNotInUnitOfWork);
-        opCtx->swapLockState(std::make_unique<LockerNoop>(), WithLock::withoutLock());
         return opCtx;
     }
 
@@ -191,7 +210,7 @@ TEST_F(WiredTigerKVEngineRepairTest, OrphanedDataFilesCanBeRecovered) {
 #else
 
     // Dropping a collection might fail if we haven't checkpointed the data.
-    _helper.getWiredTigerKVEngine()->checkpoint(opCtxPtr.get());
+    _helper.getWiredTigerKVEngine()->checkpoint();
 
     // Move the data file out of the way so the ident can be dropped. This not permitted on Windows
     // because the file cannot be moved while it is open. The implementation for orphan recovery is
@@ -200,7 +219,8 @@ TEST_F(WiredTigerKVEngineRepairTest, OrphanedDataFilesCanBeRecovered) {
     boost::filesystem::rename(*dataFilePath, tmpFile, err);
     ASSERT(!err) << err.message();
 
-    ASSERT_OK(_helper.getWiredTigerKVEngine()->dropIdent(opCtxPtr.get()->recoveryUnit(), ident));
+    ASSERT_OK(_helper.getWiredTigerKVEngine()->dropIdent(
+        shard_role_details::getRecoveryUnit(opCtxPtr.get()), ident));
 
     // The data file is moved back in place so that it becomes an "orphan" of the storage
     // engine and the restoration process can be tested.
@@ -215,6 +235,7 @@ TEST_F(WiredTigerKVEngineRepairTest, OrphanedDataFilesCanBeRecovered) {
 
 TEST_F(WiredTigerKVEngineRepairTest, UnrecoverableOrphanedDataFilesAreRebuilt) {
     auto opCtxPtr = _makeOperationContext();
+    Lock::GlobalLock globalLk(opCtxPtr.get(), MODE_X);
 
     NamespaceString nss = NamespaceString::createNamespaceString_forTest("a.b");
     std::string ident = "collection-1234";
@@ -245,9 +266,10 @@ TEST_F(WiredTigerKVEngineRepairTest, UnrecoverableOrphanedDataFilesAreRebuilt) {
     ASSERT(boost::filesystem::exists(*dataFilePath));
 
     // Dropping a collection might fail if we haven't checkpointed the data
-    _helper.getWiredTigerKVEngine()->checkpoint(opCtxPtr.get());
+    _helper.getWiredTigerKVEngine()->checkpoint();
 
-    ASSERT_OK(_helper.getWiredTigerKVEngine()->dropIdent(opCtxPtr.get()->recoveryUnit(), ident));
+    ASSERT_OK(_helper.getWiredTigerKVEngine()->dropIdent(
+        shard_role_details::getRecoveryUnit(opCtxPtr.get()), ident));
 
 #ifdef _WIN32
     auto status = _helper.getWiredTigerKVEngine()->recoverOrphanedIdent(
@@ -284,6 +306,10 @@ TEST_F(WiredTigerKVEngineRepairTest, UnrecoverableOrphanedDataFilesAreRebuilt) {
 }
 
 TEST_F(WiredTigerKVEngineTest, TestOplogTruncation) {
+    // To diagnose any intermittent failures, maximize logging from WiredTigerKVEngine and friends.
+    auto severityGuard = unittest::MinimumLoggedSeverityGuard{logv2::LogComponent::kStorage,
+                                                              logv2::LogSeverity::Debug(3)};
+
     // Set syncdelay before starting the checkpoint thread, otherwise it can observe the default
     // checkpoint frequency of 60 seconds, causing the test to fail due to a 10 second timeout.
     storageGlobalParams.syncdelay.store(1);
@@ -410,7 +436,8 @@ TEST_F(WiredTigerKVEngineTest, IdentDrop) {
     ASSERT(boost::filesystem::exists(*dataFilePath));
     ASSERT(boost::filesystem::exists(renamedFilePath));
 
-    ASSERT_OK(_helper.getWiredTigerKVEngine()->dropIdent(opCtxPtr.get()->recoveryUnit(), ident));
+    ASSERT_OK(_helper.getWiredTigerKVEngine()->dropIdent(
+        shard_role_details::getRecoveryUnit(opCtxPtr.get()), ident));
 
     // WiredTiger drops files asynchronously.
     for (size_t check = 0; check < 30; check++) {
@@ -529,6 +556,88 @@ TEST_F(WiredTigerKVEngineTest, TestPinOldestTimestampErrors) {
     ASSERT_EQ(initTs, _helper.getWiredTigerKVEngine()->getOldestTimestamp());
 }
 
+/**
+ * Test the various cases for the relationship between oldestTimestamp and stableTimestamp at the
+ * end of startup recovery.
+ */
+TEST_F(WiredTigerKVEngineTest, TestOldestStableTimestampEndOfStartupRecovery) {
+    auto opCtxRaii = _makeOperationContext();
+
+    // oldest and stable are both null.
+    ASSERT_DOES_NOT_THROW(
+        _helper.getWiredTigerKVEngine()->notifyReplStartupRecoveryComplete(opCtxRaii.get()));
+
+    // oldest is null, stable is not null.
+    const Timestamp initTs = Timestamp(10, 0);
+    _helper.getWiredTigerKVEngine()->setStableTimestamp(initTs, true);
+    ASSERT_DOES_NOT_THROW(
+        _helper.getWiredTigerKVEngine()->notifyReplStartupRecoveryComplete(opCtxRaii.get()));
+
+    // oldest and stable equal.
+    _helper.getWiredTigerKVEngine()->setOldestTimestamp(initTs, true);
+    ASSERT_DOES_NOT_THROW(
+        _helper.getWiredTigerKVEngine()->notifyReplStartupRecoveryComplete(opCtxRaii.get()));
+
+    // stable > oldest.
+    Timestamp laterTs = Timestamp(15, 0);
+    _helper.getWiredTigerKVEngine()->setStableTimestamp(laterTs, true);
+    ASSERT_DOES_NOT_THROW(
+        _helper.getWiredTigerKVEngine()->notifyReplStartupRecoveryComplete(opCtxRaii.get()));
+
+    // oldest > stable.
+    laterTs = Timestamp(20, 0);
+    _helper.getWiredTigerKVEngine()->setOldestTimestamp(laterTs, true);
+    ASSERT_THROWS_CODE(
+        _helper.getWiredTigerKVEngine()->notifyReplStartupRecoveryComplete(opCtxRaii.get()),
+        AssertionException,
+        8470600);
+}
+
+/**
+ * Test that oldestTimestamp is allowed to advance past stableTimestamp when we notify that
+ * startup recovery is complete. This case happens when we complete logical initial sync.
+ */
+TEST_F(WiredTigerKVEngineTest, TestOldestStableTimestampEndOfStartupRecoveryStableNull) {
+    auto opCtxRaii = _makeOperationContext();
+
+    // oldest is not null, stable is null.
+    const Timestamp initTs = Timestamp(10, 0);
+    _helper.getWiredTigerKVEngine()->setOldestTimestamp(initTs, true);
+    ASSERT_DOES_NOT_THROW(
+        _helper.getWiredTigerKVEngine()->notifyReplStartupRecoveryComplete(opCtxRaii.get()));
+}
+
+TEST_F(WiredTigerKVEngineTest, ExtractIdentFromPath) {
+    boost::filesystem::path dbpath = "/data/db";
+    boost::filesystem::path identAbsolutePathDefault =
+        "/data/db/collection-9-11733751379908443489.wt";
+    std::string identDefault = "collection-9-11733751379908443489";
+
+    ASSERT_EQ(extractIdentFromPath(dbpath, identAbsolutePathDefault), identDefault);
+
+    boost::filesystem::path identAbsolutePathDirectoryPerDb =
+        "/data/db/test/collection-9-11733751379908443489.wt";
+    std::string identDirectoryPerDb = "test/collection-9-11733751379908443489";
+
+    ASSERT_EQ(extractIdentFromPath(dbpath, identAbsolutePathDirectoryPerDb), identDirectoryPerDb);
+
+    boost::filesystem::path identAbsolutePathWiredTigerDirectoryForIndexes =
+        "/data/db/collection/9-11733751379908443489.wt";
+    std::string identWiredTigerDirectoryForIndexes = "collection/9-11733751379908443489";
+
+    ASSERT_EQ(extractIdentFromPath(dbpath, identAbsolutePathWiredTigerDirectoryForIndexes),
+              identWiredTigerDirectoryForIndexes);
+
+    boost::filesystem::path identAbsolutePathDirectoryPerDbAndWiredTigerDirectoryForIndexes =
+        "/data/db/test/collection/9-11733751379908443489.wt";
+    std::string identDirectoryPerDbWiredTigerDirectoryForIndexes =
+        "test/collection/9-11733751379908443489";
+
+    ASSERT_EQ(extractIdentFromPath(dbpath,
+                                   identAbsolutePathDirectoryPerDbAndWiredTigerDirectoryForIndexes),
+              identDirectoryPerDbWiredTigerDirectoryForIndexes);
+}
+
 TEST_F(WiredTigerKVEngineTest, WiredTigerDowngrade) {
     // Initializing this value to silence Coverity warning. Doesn't matter what value
     // _startupVersion is set to since shouldDowngrade() & getDowngradeString() only look at
@@ -577,7 +686,7 @@ TEST_F(WiredTigerKVEngineTest, TestReconfigureLog) {
         // Perform a checkpoint. The goal here is create some activity in WiredTiger in order
         // to generate verbose messages (we don't really care about the checkpoint itself).
         startCapturingLogMessages();
-        _helper.getWiredTigerKVEngine()->checkpoint(opCtxRaii.get());
+        _helper.getWiredTigerKVEngine()->checkpoint();
         stopCapturingLogMessages();
         // In this initial case, we don't expect to capture any debug checkpoint messages. The
         // base severity for the checkpoint component should be at Log().
@@ -602,7 +711,7 @@ TEST_F(WiredTigerKVEngineTest, TestReconfigureLog) {
 
         // Perform another checkpoint.
         startCapturingLogMessages();
-        _helper.getWiredTigerKVEngine()->checkpoint(opCtxRaii.get());
+        _helper.getWiredTigerKVEngine()->checkpoint();
         stopCapturingLogMessages();
 
         // This time we expect to detect WiredTiger checkpoint Debug() messages.
@@ -650,6 +759,116 @@ MONGO_INITIALIZER(RegisterKVHarnessFactory)(InitializerContext*) {
     KVHarnessHelper::registerFactory(makeHelper);
 }
 
+TEST_F(WiredTigerKVEngineTest, TestHandlerCleanShutdown) {
+    auto* engine = _helper.getWiredTigerKVEngine();
+    ASSERT(engine->getWtConnReadyStatus_UNSAFE());
+    engine->cleanShutdown();
+    ASSERT(!engine->getWtConnReadyStatus_UNSAFE());
+}
+
+TEST_F(WiredTigerKVEngineTest, TestHandlerSingleActivityBeforeShutdown) {
+    auto* engine = _helper.getWiredTigerKVEngine();
+    ASSERT(engine->getWtConnReadyStatus_UNSAFE());
+    ASSERT(engine->getSectionActivityPermit_UNSAFE());
+    ASSERT(engine->getWtConnReadyStatus_UNSAFE());
+    ASSERT_EQ(engine->getActiveSections(), 1);
+    engine->releaseSectionActivityPermit_UNSAFE();
+    ASSERT_EQ(engine->getActiveSections(), 0);
+    ASSERT(engine->getWtConnReadyStatus_UNSAFE());
+    engine->cleanShutdown();
+    ASSERT(!engine->getWtConnReadyStatus_UNSAFE());
+}
+
+TEST_F(WiredTigerKVEngineTest, TestHandlerSingleActivityBeforeShutdownRAII) {
+    auto* engine = _helper.getWiredTigerKVEngine();
+    ASSERT(engine->getWtConnReadyStatus_UNSAFE());
+    {
+        auto permit = engine->tryGetSectionActivityPermit();
+        ASSERT(permit);
+        ASSERT(engine->getWtConnReadyStatus_UNSAFE());
+        ASSERT_EQ(engine->getActiveSections(), 1);
+    }
+    ASSERT_EQ(engine->getActiveSections(), 0);
+    ASSERT(engine->getWtConnReadyStatus_UNSAFE());
+    engine->cleanShutdown();
+    ASSERT(!engine->getWtConnReadyStatus_UNSAFE());
+}
+
+TEST_F(WiredTigerKVEngineTest, TestHandlerMultipleActivitiesBeforeShutdownRAII) {
+    auto* engine = _helper.getWiredTigerKVEngine();
+    ASSERT(engine->getWtConnReadyStatus_UNSAFE());
+    {
+        auto permit1 = engine->tryGetSectionActivityPermit();
+        ASSERT(permit1);
+        ASSERT(engine->getWtConnReadyStatus_UNSAFE());
+        ASSERT_EQ(engine->getActiveSections(), 1);
+        {
+            auto permit2 = engine->tryGetSectionActivityPermit();
+            ASSERT(permit2);
+            ASSERT(engine->getWtConnReadyStatus_UNSAFE());
+            ASSERT_EQ(engine->getActiveSections(), 2);
+        }
+        ASSERT_EQ(engine->getActiveSections(), 1);
+    }
+    ASSERT_EQ(engine->getActiveSections(), 0);
+    ASSERT(engine->getWtConnReadyStatus_UNSAFE());
+    engine->cleanShutdown();
+    ASSERT(!engine->getWtConnReadyStatus_UNSAFE());
+}
+
+TEST_F(WiredTigerKVEngineTest, TestHandlerCleanShutdownBeforeActivity) {
+    auto* engine = _helper.getWiredTigerKVEngine();
+    ASSERT(engine->getWtConnReadyStatus_UNSAFE());
+    engine->cleanShutdown();
+    ASSERT(!engine->tryGetSectionActivityPermit());
+    ASSERT_EQ(engine->getActiveSections(), 0);
+    ASSERT(!engine->getWtConnReadyStatus_UNSAFE());
+}
+
+TEST_F(WiredTigerKVEngineTest, TestHandlerCleanShutdownBeforeActivityRAII) {
+    auto* engine = _helper.getWiredTigerKVEngine();
+    ASSERT(engine->getWtConnReadyStatus_UNSAFE());
+    engine->cleanShutdown();
+    ASSERT(!engine->getSectionActivityPermit_UNSAFE());
+    ASSERT_EQ(engine->getActiveSections(), 0);
+    ASSERT(!engine->getWtConnReadyStatus_UNSAFE());
+}
+
+TEST_F(WiredTigerKVEngineTest, TestHandlerCleanShutdownBeforeActivityRelease) {
+    auto* engine = _helper.getWiredTigerKVEngine();
+    ASSERT(engine->getWtConnReadyStatus_UNSAFE());
+    ASSERT(engine->getSectionActivityPermit_UNSAFE());
+    ASSERT(engine->getWtConnReadyStatus_UNSAFE());
+    ASSERT_EQ(engine->getActiveSections(), 1);
+    stdx::thread shudownThread([&]() { engine->cleanShutdown(); });
+    ASSERT_EQ(engine->getActiveSections(), 1);
+    while (engine->getWtConnReadyStatus_UNSAFE()) {
+        stdx::this_thread::yield();
+    }
+    engine->releaseSectionActivityPermit_UNSAFE();
+    shudownThread.join();
+    ASSERT_EQ(engine->getActiveSections(), 0);
+    ASSERT(!engine->getWtConnReadyStatus_UNSAFE());
+}
+
+TEST_F(WiredTigerKVEngineTest, TestHandlerCleanShutdownBeforeActivityReleaseRAII) {
+    auto* engine = _helper.getWiredTigerKVEngine();
+    ASSERT(engine->getWtConnReadyStatus_UNSAFE());
+    {
+        auto permit = engine->tryGetSectionActivityPermit();
+        ASSERT(engine->getWtConnReadyStatus_UNSAFE());
+        ASSERT_EQ(engine->getActiveSections(), 1);
+        stdx::thread shudownThread([&]() { engine->cleanShutdown(); });
+        ASSERT_EQ(engine->getActiveSections(), 1);
+        while (engine->getWtConnReadyStatus_UNSAFE()) {
+            stdx::this_thread::yield();
+        }
+        shudownThread.join();
+    }
+    ASSERT_EQ(engine->getActiveSections(), 0);
+    ASSERT(!engine->getWtConnReadyStatus_UNSAFE());
+}
+
 class WiredTigerKVEngineDirectoryTest : public WiredTigerKVEngineTest {
 public:
     void setUp() override {
@@ -674,7 +893,8 @@ protected:
     }
 
     Status removeIdent(StringData ident) {
-        return _helper.getWiredTigerKVEngine()->dropIdent(_opCtx->recoveryUnit(), ident);
+        return _helper.getWiredTigerKVEngine()->dropIdent(
+            shard_role_details::getRecoveryUnit(_opCtx.get()), ident);
     }
 
 private:
