@@ -27,11 +27,26 @@
  *    it in the license file.
  */
 
+#include <memory>
+#include <string>
+
+#include <wiredtiger.h>
+
+#include "mongo/base/string_data.h"
 #include "mongo/db/curop.h"
 #include "mongo/db/service_context.h"
+#include "mongo/db/storage/recovery_unit.h"
+#include "mongo/db/storage/wiredtiger/wiredtiger_kv_engine.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_prepare_conflict.h"
+#include "mongo/db/storage/wiredtiger/wiredtiger_recovery_unit.h"
+#include "mongo/db/storage/wiredtiger/wiredtiger_session_cache.h"
+#include "mongo/db/storage/write_unit_of_work.h"
+#include "mongo/db/transaction_resources.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
 #include "mongo/unittest/temp_dir.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/util/clock_source.h"
 #include "mongo/util/clock_source_mock.h"
 
 namespace mongo {
@@ -44,10 +59,7 @@ namespace {
 std::unique_ptr<WiredTigerKVEngine> makeKVEngine(ServiceContext* serviceContext,
                                                  const std::string& path,
                                                  ClockSource* clockSource) {
-    auto client = serviceContext->makeClient("myClient");
-    auto opCtx = serviceContext->makeOperationContext(client.get());
     return std::make_unique<WiredTigerKVEngine>(
-        opCtx.get(),
         /*canonicalName=*/"",
         path,
         clockSource,
@@ -64,26 +76,13 @@ public:
     void setUp() override {
         setGlobalServiceContext(ServiceContext::make());
         auto serviceContext = getGlobalServiceContext();
-        client = serviceContext->makeClient("myClient");
+        client = serviceContext->getService()->makeClient("myClient");
         opCtx = serviceContext->makeOperationContext(client.get());
         kvEngine = makeKVEngine(serviceContext, home.path(), &cs);
-        opCtx->setRecoveryUnit(std::unique_ptr<RecoveryUnit>(kvEngine->newRecoveryUnit()),
-                               WriteUnitOfWork::RecoveryUnitState::kNotInUnitOfWork);
-
-        // Sets internal states to pass invariants inside 'wiredTigerPrepareConflictRetry()'.
-        {
-            stdx::lock_guard<Client> lk(*client);
-            client->setSystemOperationKillableByStepdown(lk);
-        }
-    }
-
-    ~WiredTigerPrepareConflictTest() override {
-#if __has_feature(address_sanitizer)
-        constexpr bool memLeakAllowed = false;
-#else
-        constexpr bool memLeakAllowed = true;
-#endif
-        kvEngine->cleanShutdown(memLeakAllowed);
+        shard_role_details::setRecoveryUnit(
+            opCtx.get(),
+            std::unique_ptr<RecoveryUnit>(kvEngine->newRecoveryUnit()),
+            WriteUnitOfWork::RecoveryUnitState::kNotInUnitOfWork);
     }
 
     unittest::TempDir home{"temp"};
@@ -115,7 +114,8 @@ TEST_F(WiredTigerPrepareConflictTest, HandleWTPrepareConflictOnce) {
 TEST_F(WiredTigerPrepareConflictTest, HandleWTPrepareConflictMultipleTimes) {
     auto attempt = 0;
     auto sessionCache =
-        static_cast<WiredTigerRecoveryUnit*>(opCtx->recoveryUnit())->getSessionCache();
+        static_cast<WiredTigerRecoveryUnit*>(shard_role_details::getRecoveryUnit(opCtx.get()))
+            ->getSessionCache();
 
     auto throwWTPrepareConflictMultipleTimes = [&sessionCache, &attempt]() {
         // Manually increments '_prepareCommitOrAbortCounter' to simulate a unit of work has been
