@@ -28,28 +28,47 @@
  */
 
 
-#include "mongo/platform/basic.h"
+#include <algorithm>
+#include <boost/cstdint.hpp>
+#include <fmt/format.h>
+#include <memory>
+#include <string>
+#include <utility>
+#include <wiredtiger.h>
 
-#include "mongo/db/storage/wiredtiger/wiredtiger_recovery_unit.h"
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/base/parse_number.h"
 #include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/util/builder_fwd.h"
 #include "mongo/db/concurrency/exception_util.h"
 #include "mongo/db/server_options.h"
+#include "mongo/db/storage/snapshot.h"
+#include "mongo/db/storage/storage_options.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_begin_transaction_block.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_kv_engine.h"
-#include "mongo/db/storage/wiredtiger/wiredtiger_prepare_conflict.h"
+#include "mongo/db/storage/wiredtiger/wiredtiger_oplog_manager.h"
+#include "mongo/db/storage/wiredtiger/wiredtiger_recovery_unit.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_session_cache.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_stats.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_util.h"
+#include "mongo/db/transaction_resources.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/log_severity.h"
+#include "mongo/logv2/log_truncation.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/fail_point.h"
 #include "mongo/util/hex.h"
 #include "mongo/util/stacktrace.h"
+#include "mongo/util/str.h"
 #include "mongo/util/testing_proctor.h"
 
-#include <fmt/compile.h>
-#include <fmt/format.h>
-#include <memory>
-
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
 
 
@@ -88,13 +107,6 @@ WiredTigerRecoveryUnit::WiredTigerRecoveryUnit(WiredTigerSessionCache* sc,
 WiredTigerRecoveryUnit::~WiredTigerRecoveryUnit() {
     invariant(!_inUnitOfWork(), toString(_getState()));
     _abort();
-
-    // If the session has non zero timeout then reset it back to 0 before returning the session back
-    // to the cache.
-    if (durationCount<Milliseconds>(_cacheMaxWaitTimeout)) {
-        auto wtSession = getSessionNoTxn()->getSession();
-        invariantWTOK(wtSession->reconfigure(wtSession, "cache_max_wait_ms=0"), wtSession);
-    }
 }
 
 void WiredTigerRecoveryUnit::_commit() {
@@ -169,14 +181,26 @@ void WiredTigerRecoveryUnit::doAbortUnitOfWork() {
 }
 
 void WiredTigerRecoveryUnit::_ensureSession() {
-    if (!_session) {
-        _session = _sessionCache->getSession();
+    if (!_unique_session) {
+        invariant(!_session);
+        _unique_session = _sessionCache->getSession();
+        _session = _unique_session.get();
     }
 }
 
+void WiredTigerRecoveryUnit::setPrefetching(bool enable) {
+    invariant(!_inUnitOfWork(), toString(_getState()));
+    invariant(getSessionNoTxn()->cursorsOut() == 0);
+
+    auto session = getSessionNoTxn();
+    StringBuilder config;
+    config << "prefetch=(enabled=" << (enable ? "true" : "false") << ")";
+    session->reconfigure(config.str(), "prefetch=(enabled=false)");
+}
+
 bool WiredTigerRecoveryUnit::waitUntilDurable(OperationContext* opCtx) {
     invariant(!_inUnitOfWork(), toString(_getState()));
-    invariant(!opCtx->lockState()->isLocked() || storageGlobalParams.repair);
+    invariant(!shard_role_details::getLocker(opCtx)->isLocked() || storageGlobalParams.repair);
 
     // Flushes the journal log to disk. Checkpoints all data if journaling is disabled.
     _sessionCache->waitUntilDurable(opCtx,
@@ -189,7 +213,7 @@ bool WiredTigerRecoveryUnit::waitUntilDurable(OperationContext* opCtx) {
 bool WiredTigerRecoveryUnit::waitUntilUnjournaledWritesDurable(OperationContext* opCtx,
                                                                bool stableCheckpoint) {
     invariant(!_inUnitOfWork(), toString(_getState()));
-    invariant(!opCtx->lockState()->isLocked() || storageGlobalParams.repair);
+    invariant(!shard_role_details::getLocker(opCtx)->isLocked() || storageGlobalParams.repair);
 
     // Take a checkpoint, rather than only flush the (oplog) journal, in order to lock in stable
     // writes to unjournaled tables.
@@ -210,10 +234,7 @@ void WiredTigerRecoveryUnit::assertInActiveTxn() const {
     if (_isActive()) {
         return;
     }
-    LOGV2_FATAL(28575,
-                "Recovery unit is not active. Current state: {currentState}",
-                "Recovery unit is not active.",
-                "currentState"_attr = _getState());
+    LOGV2_FATAL(28575, "Recovery unit is not active.", "currentState"_attr = _getState());
 }
 
 void WiredTigerRecoveryUnit::setTxnModified() {
@@ -228,25 +249,25 @@ void WiredTigerRecoveryUnit::setTxnModified() {
 }
 
 boost::optional<int64_t> WiredTigerRecoveryUnit::getOplogVisibilityTs() {
-    if (!_isOplogReader) {
-        return boost::none;
-    }
-
     getSession();
     return _oplogVisibleTs;
 }
 
+void WiredTigerRecoveryUnit::setOplogVisibilityTs(boost::optional<int64_t> oplogVisibleTs) {
+    _oplogVisibleTs = oplogVisibleTs;
+}
+
 WiredTigerSession* WiredTigerRecoveryUnit::getSession() {
     if (!_isActive()) {
         _txnOpen();
         _setState(_inUnitOfWork() ? State::kActive : State::kActiveNotInUnitOfWork);
     }
-    return _session.get();
+    return _session;
 }
 
 WiredTigerSession* WiredTigerRecoveryUnit::getSessionNoTxn() {
     _ensureSession();
-    return _session.get();
+    return _session;
 }
 
 void WiredTigerRecoveryUnit::doAbandonSnapshot() {
@@ -264,12 +285,6 @@ void WiredTigerRecoveryUnit::preallocateSnapshot() {
     getSession();
 }
 
-void WiredTigerRecoveryUnit::preallocateSnapshotForOplogRead() {
-    // Indicate that we are an oplog reader before opening the snapshot
-    setIsOplogReader();
-    preallocateSnapshot();
-}
-
 void WiredTigerRecoveryUnit::_txnClose(bool commit) {
     invariant(_isActive(), toString(_getState()));
 
@@ -355,15 +370,15 @@ void WiredTigerRecoveryUnit::_txnClose(bool commit) {
             22412, 3, "WT commit_transaction", "snapshotId"_attr = getSnapshotId().toNumber());
     } else {
         invariant(_abandonSnapshotMode == AbandonSnapshotMode::kAbort);
-        StringBuilder config;
+        const char* config = nullptr;
         if (_noEvictionAfterRollback) {
             // The only point at which rollback_transaction() can time out is in the bonus-eviction
             // phase. If the timeout expires here, the function will stop the eviction and return
             // success. It cannot return an error due to timeout.
-            config << "operation_timeout_ms=1,";
+            config = "operation_timeout_ms=1,";
         }
 
-        wtRet = s->rollback_transaction(s, config.str().c_str());
+        wtRet = s->rollback_transaction(s, config);
 
         LOGV2_DEBUG(
             22413, 3, "WT rollback_transaction", "snapshotId"_attr = getSnapshotId().toNumber());
@@ -393,12 +408,10 @@ void WiredTigerRecoveryUnit::_txnClose(bool commit) {
     // transaction on a RecoveryUnit to call setTimestamp() and another to call
     // setCommitTimestamp().
     _lastTimestampSet = boost::none;
-    _multiTimestampConstraintTracker = MultiTimestampConstraintTracker();
+    _multiTimestampConstraintTracker = {};
     _prepareTimestamp = Timestamp();
     _durableTimestamp = Timestamp();
-    _catalogConflictTimestamp = Timestamp();
     _roundUpPreparedTimestamps = RoundUpPreparedTimestamps::kNoRound;
-    _isOplogReader = false;
     _oplogVisibleTs = boost::none;
     _orderedCommit = true;  // Default value is true; we assume all writes are ordered.
     if (_untimestampedWriteAssertionLevel !=
@@ -455,7 +468,7 @@ boost::optional<Timestamp> WiredTigerRecoveryUnit::getPointInTimeReadTimestamp(
     }
 
     // Ensure a transaction is opened. Storage engine operations require the global lock.
-    invariant(opCtx->lockState()->isNoop() || opCtx->lockState()->isLocked());
+    invariant(shard_role_details::getLocker(opCtx)->isLocked());
     getSession();
 
     switch (_timestampReadSource) {
@@ -487,20 +500,19 @@ void WiredTigerRecoveryUnit::_txnOpen() {
     invariant(!_isCommittingOrAborting(),
               str::stream() << "commit or rollback handler reopened transaction: "
                             << toString(_getState()));
+
+    ensureSnapshot();
     _ensureSession();
 
     // Only start a timer for transaction's lifetime if we're going to log it.
     if (shouldLog(MONGO_LOGV2_DEFAULT_COMPONENT, kSlowTransactionSeverity)) {
         _timer.reset(new Timer());
     }
-    WT_SESSION* session = _session->getSession();
 
     switch (_timestampReadSource) {
         case ReadSource::kNoTimestamp: {
-            if (_isOplogReader) {
-                _oplogVisibleTs = static_cast<std::int64_t>(_oplogManager->getOplogReadTimestamp());
-            }
-            WiredTigerBeginTxnBlock(session,
+            _oplogVisibleTs = static_cast<std::int64_t>(_oplogManager->getOplogReadTimestamp());
+            WiredTigerBeginTxnBlock(_session,
                                     _prepareConflictBehavior,
                                     _roundUpPreparedTimestamps,
                                     RoundUpReadTimestamp::kNoRoundError,
@@ -509,7 +521,7 @@ void WiredTigerRecoveryUnit::_txnOpen() {
             break;
         }
         case ReadSource::kCheckpoint: {
-            WiredTigerBeginTxnBlock(session,
+            WiredTigerBeginTxnBlock(_session,
                                     _prepareConflictBehavior,
                                     _roundUpPreparedTimestamps,
                                     RoundUpReadTimestamp::kNoRoundError,
@@ -519,29 +531,29 @@ void WiredTigerRecoveryUnit::_txnOpen() {
         }
         case ReadSource::kMajorityCommitted: {
             _readAtTimestamp = _sessionCache->snapshotManager().beginTransactionOnCommittedSnapshot(
-                session,
+                _session,
                 _prepareConflictBehavior,
                 _roundUpPreparedTimestamps,
                 _untimestampedWriteAssertionLevel);
             break;
         }
         case ReadSource::kLastApplied: {
-            _beginTransactionAtLastAppliedTimestamp(session);
+            _beginTransactionAtLastAppliedTimestamp();
             break;
         }
         case ReadSource::kNoOverlap: {
-            _readAtTimestamp = _beginTransactionAtNoOverlapTimestamp(session);
+            _readAtTimestamp = _beginTransactionAtNoOverlapTimestamp();
             break;
         }
         case ReadSource::kAllDurableSnapshot: {
             if (_readAtTimestamp.isNull()) {
-                _readAtTimestamp = _beginTransactionAtAllDurableTimestamp(session);
+                _readAtTimestamp = _beginTransactionAtAllDurableTimestamp();
                 break;
             }
             [[fallthrough]];  // Continue to the next case to read at the _readAtTimestamp.
         }
         case ReadSource::kProvided: {
-            WiredTigerBeginTxnBlock txnOpen(session,
+            WiredTigerBeginTxnBlock txnOpen(_session,
                                             _prepareConflictBehavior,
                                             _roundUpPreparedTimestamps,
                                             RoundUpReadTimestamp::kNoRoundError,
@@ -569,8 +581,8 @@ void WiredTigerRecoveryUnit::_txnOpen() {
                 "readSource"_attr = toString(_timestampReadSource));
 }
 
-Timestamp WiredTigerRecoveryUnit::_beginTransactionAtAllDurableTimestamp(WT_SESSION* session) {
-    WiredTigerBeginTxnBlock txnOpen(session,
+Timestamp WiredTigerRecoveryUnit::_beginTransactionAtAllDurableTimestamp() {
+    WiredTigerBeginTxnBlock txnOpen(_session,
                                     _prepareConflictBehavior,
                                     _roundUpPreparedTimestamps,
                                     RoundUpReadTimestamp::kRound,
@@ -582,12 +594,12 @@ Timestamp WiredTigerRecoveryUnit::_beginTransactionAtAllDurableTimestamp(WT_SESS
     // Since this is not in a critical section, we might have rounded to oldest between
     // calling getAllDurable and setReadSnapshot.  We need to get the actual read timestamp we
     // used.
-    auto readTimestamp = _getTransactionReadTimestamp(session);
+    auto readTimestamp = _getTransactionReadTimestamp();
     txnOpen.done();
     return readTimestamp;
 }
 
-void WiredTigerRecoveryUnit::_beginTransactionAtLastAppliedTimestamp(WT_SESSION* session) {
+void WiredTigerRecoveryUnit::_beginTransactionAtLastAppliedTimestamp() {
     if (_readAtTimestamp.isNull()) {
         // When there is not a lastApplied timestamp available, read without a timestamp. Do not
         // round up the read timestamp to the oldest timestamp.
@@ -598,7 +610,7 @@ void WiredTigerRecoveryUnit::_beginTransactionAtLastAppliedTimestamp(WT_SESSION*
         // is only possible for readers that start immediately after an initial sync that did not
         // replicate any oplog entries. Future transactions will start reading at a timestamp once
         // timestamped writes have been made.
-        WiredTigerBeginTxnBlock txnOpen(session,
+        WiredTigerBeginTxnBlock txnOpen(_session,
                                         _prepareConflictBehavior,
                                         _roundUpPreparedTimestamps,
                                         RoundUpReadTimestamp::kNoRoundError,
@@ -608,7 +620,7 @@ void WiredTigerRecoveryUnit::_beginTransactionAtLastAppliedTimestamp(WT_SESSION*
         return;
     }
 
-    WiredTigerBeginTxnBlock txnOpen(session,
+    WiredTigerBeginTxnBlock txnOpen(_session,
                                     _prepareConflictBehavior,
                                     _roundUpPreparedTimestamps,
                                     RoundUpReadTimestamp::kRound,
@@ -618,12 +630,12 @@ void WiredTigerRecoveryUnit::_beginTransactionAtLastAppliedTimestamp(WT_SESSION*
 
     // We might have rounded to oldest between calling setTimestampReadSource and setReadSnapshot.
     // We need to get the actual read timestamp we used.
-    auto actualTimestamp = _getTransactionReadTimestamp(session);
+    auto actualTimestamp = _getTransactionReadTimestamp();
     txnOpen.done();
     _readAtTimestamp = actualTimestamp;
 }
 
-Timestamp WiredTigerRecoveryUnit::_beginTransactionAtNoOverlapTimestamp(WT_SESSION* session) {
+Timestamp WiredTigerRecoveryUnit::_beginTransactionAtNoOverlapTimestamp() {
 
     auto lastApplied = _sessionCache->snapshotManager().getLastApplied();
     Timestamp allDurable = Timestamp(_sessionCache->getKVEngine()->getAllDurableTimestamp());
@@ -664,7 +676,7 @@ Timestamp WiredTigerRecoveryUnit::_beginTransactionAtNoOverlapTimestamp(WT_SESSI
         // is only possible for readers that start immediately after an initial sync that did not
         // replicate any oplog entries. Future transactions will start reading at a timestamp once
         // timestamped writes have been made.
-        WiredTigerBeginTxnBlock txnOpen(session,
+        WiredTigerBeginTxnBlock txnOpen(_session,
                                         _prepareConflictBehavior,
                                         _roundUpPreparedTimestamps,
                                         RoundUpReadTimestamp::kNoRoundError,
@@ -674,7 +686,7 @@ Timestamp WiredTigerRecoveryUnit::_beginTransactionAtNoOverlapTimestamp(WT_SESSI
         return readTimestamp;
     }
 
-    WiredTigerBeginTxnBlock txnOpen(session,
+    WiredTigerBeginTxnBlock txnOpen(_session,
                                     _prepareConflictBehavior,
                                     _roundUpPreparedTimestamps,
                                     RoundUpReadTimestamp::kRound,
@@ -684,13 +696,14 @@ Timestamp WiredTigerRecoveryUnit::_beginTransactionAtNoOverlapTimestamp(WT_SESSI
 
     // We might have rounded to oldest between calling getAllDurable and setReadSnapshot. We
     // need to get the actual read timestamp we used.
-    readTimestamp = _getTransactionReadTimestamp(session);
+    readTimestamp = _getTransactionReadTimestamp();
     txnOpen.done();
     return readTimestamp;
 }
 
-Timestamp WiredTigerRecoveryUnit::_getTransactionReadTimestamp(WT_SESSION* session) {
+Timestamp WiredTigerRecoveryUnit::_getTransactionReadTimestamp() {
     char buf[(2 * 8 /*bytes in hex*/) + 1 /*nul terminator*/];
+    WT_SESSION* session = _session->getSession();
     auto wtstatus = session->query_timestamp(session, buf, "get=read");
     invariantWTOK(wtstatus, session);
     uint64_t read_timestamp;
@@ -699,7 +712,7 @@ Timestamp WiredTigerRecoveryUnit::_getTransactionReadTimestamp(WT_SESSION* sessi
 }
 
 void WiredTigerRecoveryUnit::_updateMultiTimestampConstraint(Timestamp timestamp) {
-    std::stack<Timestamp>& timestampOrder = _multiTimestampConstraintTracker.timestampOrder;
+    auto& timestampOrder = _multiTimestampConstraintTracker.timestampOrder;
     if (!timestampOrder.empty() && timestampOrder.top() == timestamp) {
         // We're still on the same timestamp.
         return;
@@ -848,6 +861,10 @@ void WiredTigerRecoveryUnit::setRoundUpPreparedTimestamps(bool value) {
         (value) ? RoundUpPreparedTimestamps::kRound : RoundUpPreparedTimestamps::kNoRound;
 }
 
+bool WiredTigerRecoveryUnit::getRoundUpPreparedTimestamps() {
+    return _roundUpPreparedTimestamps == RoundUpPreparedTimestamps::kRound;
+}
+
 void WiredTigerRecoveryUnit::setTimestampReadSource(ReadSource readSource,
                                                     boost::optional<Timestamp> provided) {
     tassert(5863604, "Cannot change ReadSource as it is pinned.", !isReadSourcePinned());
@@ -920,25 +937,6 @@ std::unique_ptr<StorageStats> WiredTigerRecoveryUnit::computeOperationStatistics
     return operationStats;
 }
 
-void WiredTigerRecoveryUnit::setCatalogConflictingTimestamp(Timestamp timestamp) {
-    // This cannot be called after a storage snapshot is allocated.
-    invariant(!_isActive(), toString(_getState()));
-    invariant(_timestampReadSource == ReadSource::kNoTimestamp,
-              str::stream() << "Illegal to set catalog conflicting timestamp for a read source "
-                            << static_cast<int>(_timestampReadSource));
-    invariant(_catalogConflictTimestamp.isNull(),
-              str::stream() << "Trying to set catalog conflicting timestamp to "
-                            << timestamp.toString() << ". It's already set to "
-                            << _catalogConflictTimestamp.toString());
-    invariant(!timestamp.isNull());
-
-    _catalogConflictTimestamp = timestamp;
-}
-
-Timestamp WiredTigerRecoveryUnit::getCatalogConflictingTimestamp() const {
-    return _catalogConflictTimestamp;
-}
-
 bool WiredTigerRecoveryUnit::gatherWriteContextForDebugging() const {
     return _gatherWriteContextForDebugging;
 }
@@ -949,13 +947,10 @@ void WiredTigerRecoveryUnit::storeWriteContextForDebugging(const BSONObj& info)
 
 void WiredTigerRecoveryUnit::setCacheMaxWaitTimeout(Milliseconds timeout) {
     _cacheMaxWaitTimeout = timeout;
+    auto session = getSessionNoTxn();
 
-    auto wtSession = getSessionNoTxn()->getSession();
-    invariantWTOK(
-        wtSession->reconfigure(
-            wtSession,
-            fmt::format("cache_max_wait_ms={}", durationCount<Milliseconds>(_cacheMaxWaitTimeout))
-                .c_str()),
-        wtSession);
+    session->reconfigure(
+        fmt::format("cache_max_wait_ms={}", durationCount<Milliseconds>(_cacheMaxWaitTimeout)),
+        "cache_max_wait_ms=0");
 }
 }  // namespace mongo
