@@ -29,22 +29,51 @@
 
 #include "mongo/db/storage/wiredtiger/wiredtiger_recovery_unit.h"
 
+#include <cstring>
+#include <string>
+#include <utility>
+#include <wiredtiger.h>
+
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+
 #include "mongo/base/checked_cast.h"
+#include "mongo/base/init.h"  // IWYU pragma: keep
+#include "mongo/base/initializer.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/db/catalog/collection_options.h"
+#include "mongo/db/client.h"
+#include "mongo/db/concurrency/d_concurrency.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
 #include "mongo/db/global_settings.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/record_id.h"
 #include "mongo/db/repl/repl_settings.h"
+#include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/repl/replication_coordinator_mock.h"
 #include "mongo/db/service_context.h"
+#include "mongo/db/storage/key_format.h"
+#include "mongo/db/storage/record_data.h"
+#include "mongo/db/storage/record_store.h"
 #include "mongo/db/storage/recovery_unit_test_harness.h"
+#include "mongo/db/storage/snapshot_manager.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_cursor_helpers.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_kv_engine.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_record_store.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_session_cache.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_util.h"
+#include "mongo/db/storage/write_unit_of_work.h"
+#include "mongo/db/transaction_resources.h"
+#include "mongo/unittest/assert.h"
 #include "mongo/unittest/death_test.h"
+#include "mongo/unittest/framework.h"
 #include "mongo/unittest/temp_dir.h"
-#include "mongo/unittest/unittest.h"
 #include "mongo/util/assert_util.h"
 #include "mongo/util/clock_source_mock.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/str.h"
 
 namespace mongo {
 namespace {
@@ -53,15 +82,14 @@ class WiredTigerRecoveryUnitHarnessHelper final : public RecoveryUnitHarnessHelp
 public:
     WiredTigerRecoveryUnitHarnessHelper()
         : _dbpath("wt_test"),
-          _engine(Client::getCurrent()->makeOperationContext().get(),
-                  kWiredTigerEngineName,  // .canonicalName
-                  _dbpath.path(),         // .path
-                  &_cs,                   // .cs
-                  "",                     // .extraOpenOptions
-                  1,                      // .cacheSizeMB
-                  0,                      // .maxCacheOverflowFileSizeMB
-                  false,                  // .ephemeral
-                  false                   // .repair
+          _engine(std::string{kWiredTigerEngineName},  // .canonicalName
+                  _dbpath.path(),                      // .path
+                  &_cs,                                // .cs
+                  "",                                  // .extraOpenOptions
+                  1,                                   // .cacheSizeMB
+                  0,                                   // .maxCacheOverflowFileSizeMB
+                  false,                               // .ephemeral
+                  false                                // .repair
           ) {
         // Use a replica set so that writes to replicated collections are not journaled and thus
         // retain their timestamps.
@@ -71,29 +99,22 @@ public:
         repl::ReplicationCoordinator::set(getGlobalServiceContext(),
                                           std::make_unique<repl::ReplicationCoordinatorMock>(
                                               getGlobalServiceContext(), replSettings));
-        _engine.notifyStartupComplete();
+        _engine.notifyStorageStartupRecoveryComplete();
     }
 
-    ~WiredTigerRecoveryUnitHarnessHelper() {
-#if __has_feature(address_sanitizer)
-        constexpr bool memLeakAllowed = false;
-#else
-        constexpr bool memLeakAllowed = true;
-#endif
-        _engine.cleanShutdown(memLeakAllowed);
-    }
+    ~WiredTigerRecoveryUnitHarnessHelper() override {}
 
-    virtual std::unique_ptr<RecoveryUnit> newRecoveryUnit() final {
+    std::unique_ptr<RecoveryUnit> newRecoveryUnit() final {
         return std::unique_ptr<RecoveryUnit>(_engine.newRecoveryUnit());
     }
 
-    virtual std::unique_ptr<RecordStore> createRecordStore(OperationContext* opCtx,
-                                                           const std::string& ns) final {
+    std::unique_ptr<RecordStore> createRecordStore(OperationContext* opCtx,
+                                                   const std::string& ns) final {
         std::string ident = ns;
         NamespaceString nss = NamespaceString::createNamespaceString_forTest(ns);
         std::string uri = WiredTigerKVEngine::kTableUriPrefix + ns;
         StatusWith<std::string> result =
-            WiredTigerRecordStore::generateCreateString(kWiredTigerEngineName,
+            WiredTigerRecordStore::generateCreateString(std::string{kWiredTigerEngineName},
                                                         nss,
                                                         ident,
                                                         CollectionOptions(),
@@ -106,7 +127,7 @@ public:
         {
             WriteUnitOfWork uow(opCtx);
             WiredTigerRecoveryUnit* ru =
-                checked_cast<WiredTigerRecoveryUnit*>(opCtx->recoveryUnit());
+                checked_cast<WiredTigerRecoveryUnit*>(shard_role_details::getRecoveryUnit(opCtx));
             WT_SESSION* s = ru->getSession()->getSession();
             invariantWTOK(s->create(s, uri.c_str(), config.c_str()), s);
             uow.commit();
@@ -115,7 +136,7 @@ public:
         WiredTigerRecordStore::Params params;
         params.nss = nss;
         params.ident = ident;
-        params.engineName = kWiredTigerEngineName;
+        params.engineName = std::string{kWiredTigerEngineName};
         params.isCapped = false;
         params.keyFormat = KeyFormat::Long;
         params.overwrite = true;
@@ -125,7 +146,7 @@ public:
         params.tracksSizeAdjustments = true;
         params.forceUpdateWithFullDocument = false;
 
-        auto ret = std::make_unique<StandardWiredTigerRecordStore>(&_engine, opCtx, params);
+        auto ret = std::make_unique<WiredTigerRecordStore>(&_engine, opCtx, params);
         ret->postConstructorInit(opCtx, nss);
         return std::move(ret);
     }
@@ -156,10 +177,11 @@ public:
     ClientAndCtx makeClientAndOpCtx(RecoveryUnitHarnessHelper* harnessHelper,
                                     const std::string& clientName) {
         auto sc = harnessHelper->serviceContext();
-        auto client = sc->makeClient(clientName);
+        auto client = sc->getService()->makeClient(clientName);
         auto opCtx = client->makeOperationContext();
-        opCtx->setRecoveryUnit(harnessHelper->newRecoveryUnit(),
-                               WriteUnitOfWork::RecoveryUnitState::kNotInUnitOfWork);
+        shard_role_details::setRecoveryUnit(opCtx.get(),
+                                            harnessHelper->newRecoveryUnit(),
+                                            WriteUnitOfWork::RecoveryUnitState::kNotInUnitOfWork);
         return std::make_pair(std::move(client), std::move(opCtx));
     }
 
@@ -174,9 +196,11 @@ public:
         harnessHelper = std::make_unique<WiredTigerRecoveryUnitHarnessHelper>();
         clientAndCtx1 = makeClientAndOpCtx(harnessHelper.get(), "writer");
         clientAndCtx2 = makeClientAndOpCtx(harnessHelper.get(), "reader");
-        ru1 = checked_cast<WiredTigerRecoveryUnit*>(clientAndCtx1.second->recoveryUnit());
+        ru1 = checked_cast<WiredTigerRecoveryUnit*>(
+            shard_role_details::getRecoveryUnit(clientAndCtx1.second.get()));
         ru1->setOperationContext(clientAndCtx1.second.get());
-        ru2 = checked_cast<WiredTigerRecoveryUnit*>(clientAndCtx2.second->recoveryUnit());
+        ru2 = checked_cast<WiredTigerRecoveryUnit*>(
+            shard_role_details::getRecoveryUnit(clientAndCtx2.second.get()));
         ru2->setOperationContext(clientAndCtx2.second.get());
         snapshotManager = dynamic_cast<WiredTigerSnapshotManager*>(
             harnessHelper->getEngine()->getSnapshotManager());
@@ -227,14 +251,15 @@ TEST_F(WiredTigerRecoveryUnitTestFixture, NoOverlapReadSource) {
     }
 
     // Read without a timestamp. The write should be visible.
-    ASSERT_EQ(opCtx1->recoveryUnit()->getTimestampReadSource(),
+    ASSERT_EQ(shard_role_details::getRecoveryUnit(opCtx1)->getTimestampReadSource(),
               RecoveryUnit::ReadSource::kNoTimestamp);
     RecordData unused;
     ASSERT_TRUE(rs->findRecord(opCtx1, rid1, &unused));
 
     // Read with kNoOverlap. The write should be visible.
-    opCtx1->recoveryUnit()->abandonSnapshot();
-    opCtx1->recoveryUnit()->setTimestampReadSource(RecoveryUnit::ReadSource::kNoOverlap);
+    shard_role_details::getRecoveryUnit(opCtx1)->abandonSnapshot();
+    shard_role_details::getRecoveryUnit(opCtx1)->setTimestampReadSource(
+        RecoveryUnit::ReadSource::kNoOverlap);
     ASSERT_TRUE(rs->findRecord(opCtx1, rid1, &unused));
 
     RecordId rid2, rid3;
@@ -245,7 +270,7 @@ TEST_F(WiredTigerRecoveryUnitTestFixture, NoOverlapReadSource) {
         WriteUnitOfWork wuow(opCtx2);
         StatusWith<RecordId> res =
             rs->insertRecord(opCtx2, str.c_str(), str.size() + 1, Timestamp());
-        ASSERT_OK(opCtx2->recoveryUnit()->setTimestamp(ts2));
+        ASSERT_OK(shard_role_details::getRecoveryUnit(opCtx2)->setTimestamp(ts2));
         ASSERT_OK(res);
         rid2 = res.getValue();
 
@@ -261,8 +286,9 @@ TEST_F(WiredTigerRecoveryUnitTestFixture, NoOverlapReadSource) {
         }
 
         // Read without a timestamp, and we should see the first and third records.
-        opCtx1->recoveryUnit()->abandonSnapshot();
-        opCtx1->recoveryUnit()->setTimestampReadSource(RecoveryUnit::ReadSource::kNoTimestamp);
+        shard_role_details::getRecoveryUnit(opCtx1)->abandonSnapshot();
+        shard_role_details::getRecoveryUnit(opCtx1)->setTimestampReadSource(
+            RecoveryUnit::ReadSource::kNoTimestamp);
         ASSERT_TRUE(rs->findRecord(opCtx1, rid1, &unused));
         ASSERT_FALSE(rs->findRecord(opCtx1, rid2, &unused));
         ASSERT_TRUE(rs->findRecord(opCtx1, rid3, &unused));
@@ -270,8 +296,9 @@ TEST_F(WiredTigerRecoveryUnitTestFixture, NoOverlapReadSource) {
         // Now read at kNoOverlap. Since the transaction at ts2 has not committed, all_durable is
         // held back to ts1. LastApplied has advanced to ts3, but because kNoOverlap is the minimum,
         // we should only see one record.
-        opCtx1->recoveryUnit()->abandonSnapshot();
-        opCtx1->recoveryUnit()->setTimestampReadSource(RecoveryUnit::ReadSource::kNoOverlap);
+        shard_role_details::getRecoveryUnit(opCtx1)->abandonSnapshot();
+        shard_role_details::getRecoveryUnit(opCtx1)->setTimestampReadSource(
+            RecoveryUnit::ReadSource::kNoOverlap);
         ASSERT_TRUE(rs->findRecord(opCtx1, rid1, &unused));
         ASSERT_FALSE(rs->findRecord(opCtx1, rid2, &unused));
         ASSERT_FALSE(rs->findRecord(opCtx1, rid3, &unused));
@@ -280,8 +307,9 @@ TEST_F(WiredTigerRecoveryUnitTestFixture, NoOverlapReadSource) {
     }
 
     // Now that the hole has been closed, kNoOverlap should see all 3 records.
-    opCtx1->recoveryUnit()->abandonSnapshot();
-    opCtx1->recoveryUnit()->setTimestampReadSource(RecoveryUnit::ReadSource::kNoOverlap);
+    shard_role_details::getRecoveryUnit(opCtx1)->abandonSnapshot();
+    shard_role_details::getRecoveryUnit(opCtx1)->setTimestampReadSource(
+        RecoveryUnit::ReadSource::kNoOverlap);
     ASSERT_TRUE(rs->findRecord(opCtx1, rid1, &unused));
     ASSERT_TRUE(rs->findRecord(opCtx1, rid2, &unused));
     ASSERT_TRUE(rs->findRecord(opCtx1, rid3, &unused));
@@ -295,7 +323,9 @@ TEST_F(WiredTigerRecoveryUnitTestFixture,
     getCursor(ru1, &cursor);
     cursor->set_key(cursor, "key");
     cursor->set_value(cursor, "value");
-    invariantWTOK(wiredTigerCursorInsert(clientAndCtx1.second.get(), cursor), cursor->session);
+    invariantWTOK(
+        wiredTigerCursorInsert(*WiredTigerRecoveryUnit::get(clientAndCtx1.second.get()), cursor),
+        cursor->session);
     ru1->setPrepareTimestamp({1, 1});
     ru1->prepareUnitOfWork();
 
@@ -318,7 +348,9 @@ TEST_F(WiredTigerRecoveryUnitTestFixture,
     getCursor(ru1, &cursor);
     cursor->set_key(cursor, "key");
     cursor->set_value(cursor, "value");
-    invariantWTOK(wiredTigerCursorInsert(clientAndCtx1.second.get(), cursor), cursor->session);
+    invariantWTOK(
+        wiredTigerCursorInsert(*WiredTigerRecoveryUnit::get(clientAndCtx1.second.get()), cursor),
+        cursor->session);
     ru1->setPrepareTimestamp({1, 1});
     ru1->prepareUnitOfWork();
 
@@ -342,7 +374,9 @@ TEST_F(WiredTigerRecoveryUnitTestFixture, WriteAllowedWhileIgnorePrepareFalse) {
     getCursor(ru1, &cursor);
     cursor->set_key(cursor, "key1");
     cursor->set_value(cursor, "value1");
-    invariantWTOK(wiredTigerCursorInsert(clientAndCtx1.second.get(), cursor), cursor->session);
+    invariantWTOK(
+        wiredTigerCursorInsert(*WiredTigerRecoveryUnit::get(clientAndCtx1.second.get()), cursor),
+        cursor->session);
     ru1->setPrepareTimestamp({1, 1});
     ru1->prepareUnitOfWork();
 
@@ -361,7 +395,9 @@ TEST_F(WiredTigerRecoveryUnitTestFixture, WriteAllowedWhileIgnorePrepareFalse) {
     cursor->set_value(cursor, "value2");
 
     // The write is allowed.
-    invariantWTOK(wiredTigerCursorInsert(clientAndCtx2.second.get(), cursor), cursor->session);
+    invariantWTOK(
+        wiredTigerCursorInsert(*WiredTigerRecoveryUnit::get(clientAndCtx2.second.get()), cursor),
+        cursor->session);
 
     ru1->abortUnitOfWork();
     ru2->abortUnitOfWork();
@@ -374,7 +410,9 @@ TEST_F(WiredTigerRecoveryUnitTestFixture, WriteOnADocumentBeingPreparedTriggersW
     getCursor(ru1, &cursor);
     cursor->set_key(cursor, "key");
     cursor->set_value(cursor, "value");
-    invariantWTOK(wiredTigerCursorInsert(clientAndCtx1.second.get(), cursor), cursor->session);
+    invariantWTOK(
+        wiredTigerCursorInsert(*WiredTigerRecoveryUnit::get(clientAndCtx1.second.get()), cursor),
+        cursor->session);
     ru1->setPrepareTimestamp({1, 1});
     ru1->prepareUnitOfWork();
 
@@ -383,7 +421,8 @@ TEST_F(WiredTigerRecoveryUnitTestFixture, WriteOnADocumentBeingPreparedTriggersW
     getCursor(ru2, &cursor);
     cursor->set_key(cursor, "key");
     cursor->set_value(cursor, "value2");
-    int ret = wiredTigerCursorInsert(clientAndCtx2.second.get(), cursor);
+    int ret =
+        wiredTigerCursorInsert(*WiredTigerRecoveryUnit::get(clientAndCtx2.second.get()), cursor);
     ASSERT_EQ(WT_ROLLBACK, ret);
 
     ru1->abortUnitOfWork();
@@ -396,7 +435,7 @@ TEST_F(WiredTigerRecoveryUnitTestFixture,
     auto opCtx = clientAndCtx1.second.get();
     {
         WriteUnitOfWork wuow(opCtx);
-        opCtx->recoveryUnit()->onCommit(
+        shard_role_details::getRecoveryUnit(opCtx)->onCommit(
             [&](OperationContext*, boost::optional<Timestamp> commitTime) {
                 commitTs = commitTime;
             });
@@ -412,15 +451,15 @@ TEST_F(WiredTigerRecoveryUnitTestFixture, ChangeIsPassedLastTimestampSetOnCommit
     Timestamp ts2(6, 6);
     {
         WriteUnitOfWork wuow(opCtx);
-        opCtx->recoveryUnit()->onCommit(
+        shard_role_details::getRecoveryUnit(opCtx)->onCommit(
             [&](OperationContext*, boost::optional<Timestamp> commitTime) {
                 commitTs = commitTime;
             });
-        ASSERT_OK(opCtx->recoveryUnit()->setTimestamp(ts1));
+        ASSERT_OK(shard_role_details::getRecoveryUnit(opCtx)->setTimestamp(ts1));
         ASSERT(!commitTs);
-        ASSERT_OK(opCtx->recoveryUnit()->setTimestamp(ts2));
+        ASSERT_OK(shard_role_details::getRecoveryUnit(opCtx)->setTimestamp(ts2));
         ASSERT(!commitTs);
-        ASSERT_OK(opCtx->recoveryUnit()->setTimestamp(ts1));
+        ASSERT_OK(shard_role_details::getRecoveryUnit(opCtx)->setTimestamp(ts1));
         ASSERT(!commitTs);
         wuow.commit();
         ASSERT_EQ(*commitTs, ts1);
@@ -434,11 +473,11 @@ TEST_F(WiredTigerRecoveryUnitTestFixture, ChangeIsNotPassedLastTimestampSetOnAbo
     Timestamp ts1(5, 5);
     {
         WriteUnitOfWork wuow(opCtx);
-        opCtx->recoveryUnit()->onCommit(
+        shard_role_details::getRecoveryUnit(opCtx)->onCommit(
             [&](OperationContext*, boost::optional<Timestamp> commitTime) {
                 commitTs = commitTime;
             });
-        ASSERT_OK(opCtx->recoveryUnit()->setTimestamp(ts1));
+        ASSERT_OK(shard_role_details::getRecoveryUnit(opCtx)->setTimestamp(ts1));
         ASSERT(!commitTs);
     }
     ASSERT(!commitTs);
@@ -449,12 +488,12 @@ TEST_F(WiredTigerRecoveryUnitTestFixture, ChangeIsPassedCommitTimestamp) {
     auto opCtx = clientAndCtx1.second.get();
     Timestamp ts1(5, 5);
 
-    opCtx->recoveryUnit()->setCommitTimestamp(ts1);
+    shard_role_details::getRecoveryUnit(opCtx)->setCommitTimestamp(ts1);
     ASSERT(!commitTs);
 
     {
         WriteUnitOfWork wuow(opCtx);
-        opCtx->recoveryUnit()->onCommit(
+        shard_role_details::getRecoveryUnit(opCtx)->onCommit(
             [&](OperationContext*, boost::optional<Timestamp> commitTime) {
                 commitTs = commitTime;
             });
@@ -470,14 +509,14 @@ TEST_F(WiredTigerRecoveryUnitTestFixture, ChangeIsNotPassedCommitTimestampIfClea
     auto opCtx = clientAndCtx1.second.get();
     Timestamp ts1(5, 5);
 
-    opCtx->recoveryUnit()->setCommitTimestamp(ts1);
+    shard_role_details::getRecoveryUnit(opCtx)->setCommitTimestamp(ts1);
     ASSERT(!commitTs);
-    opCtx->recoveryUnit()->clearCommitTimestamp();
+    shard_role_details::getRecoveryUnit(opCtx)->clearCommitTimestamp();
     ASSERT(!commitTs);
 
     {
         WriteUnitOfWork wuow(opCtx);
-        opCtx->recoveryUnit()->onCommit(
+        shard_role_details::getRecoveryUnit(opCtx)->onCommit(
             [&](OperationContext*, boost::optional<Timestamp> commitTime) {
                 commitTs = commitTime;
             });
@@ -493,16 +532,16 @@ TEST_F(WiredTigerRecoveryUnitTestFixture, ChangeIsPassedNewestCommitTimestamp) {
     Timestamp ts1(5, 5);
     Timestamp ts2(6, 6);
 
-    opCtx->recoveryUnit()->setCommitTimestamp(ts2);
+    shard_role_details::getRecoveryUnit(opCtx)->setCommitTimestamp(ts2);
     ASSERT(!commitTs);
-    opCtx->recoveryUnit()->clearCommitTimestamp();
+    shard_role_details::getRecoveryUnit(opCtx)->clearCommitTimestamp();
     ASSERT(!commitTs);
-    opCtx->recoveryUnit()->setCommitTimestamp(ts1);
+    shard_role_details::getRecoveryUnit(opCtx)->setCommitTimestamp(ts1);
     ASSERT(!commitTs);
 
     {
         WriteUnitOfWork wuow(opCtx);
-        opCtx->recoveryUnit()->onCommit(
+        shard_role_details::getRecoveryUnit(opCtx)->onCommit(
             [&](OperationContext*, boost::optional<Timestamp> commitTime) {
                 commitTs = commitTime;
             });
@@ -518,12 +557,12 @@ TEST_F(WiredTigerRecoveryUnitTestFixture, ChangeIsNotPassedCommitTimestampOnAbor
     auto opCtx = clientAndCtx1.second.get();
     Timestamp ts1(5, 5);
 
-    opCtx->recoveryUnit()->setCommitTimestamp(ts1);
+    shard_role_details::getRecoveryUnit(opCtx)->setCommitTimestamp(ts1);
     ASSERT(!commitTs);
 
     {
         WriteUnitOfWork wuow(opCtx);
-        opCtx->recoveryUnit()->onCommit(
+        shard_role_details::getRecoveryUnit(opCtx)->onCommit(
             [&](OperationContext*, boost::optional<Timestamp> commitTime) {
                 commitTs = commitTime;
             });
@@ -538,12 +577,12 @@ TEST_F(WiredTigerRecoveryUnitTestFixture, CommitTimestampBeforeSetTimestampOnCom
     Timestamp ts1(5, 5);
     Timestamp ts2(6, 6);
 
-    opCtx->recoveryUnit()->setCommitTimestamp(ts2);
+    shard_role_details::getRecoveryUnit(opCtx)->setCommitTimestamp(ts2);
     ASSERT(!commitTs);
 
     {
         WriteUnitOfWork wuow(opCtx);
-        opCtx->recoveryUnit()->onCommit(
+        shard_role_details::getRecoveryUnit(opCtx)->onCommit(
             [&](OperationContext*, boost::optional<Timestamp> commitTime) {
                 commitTs = commitTime;
             });
@@ -552,15 +591,15 @@ TEST_F(WiredTigerRecoveryUnitTestFixture, CommitTimestampBeforeSetTimestampOnCom
         ASSERT_EQ(*commitTs, ts2);
     }
     ASSERT_EQ(*commitTs, ts2);
-    opCtx->recoveryUnit()->clearCommitTimestamp();
+    shard_role_details::getRecoveryUnit(opCtx)->clearCommitTimestamp();
 
     {
         WriteUnitOfWork wuow(opCtx);
-        opCtx->recoveryUnit()->onCommit(
+        shard_role_details::getRecoveryUnit(opCtx)->onCommit(
             [&](OperationContext*, boost::optional<Timestamp> commitTime) {
                 commitTs = commitTime;
             });
-        ASSERT_OK(opCtx->recoveryUnit()->setTimestamp(ts1));
+        ASSERT_OK(shard_role_details::getRecoveryUnit(opCtx)->setTimestamp(ts1));
         ASSERT_EQ(*commitTs, ts2);
         wuow.commit();
         ASSERT_EQ(*commitTs, ts1);
@@ -576,24 +615,24 @@ TEST_F(WiredTigerRecoveryUnitTestFixture, CommitTimestampAfterSetTimestampOnComm
 
     {
         WriteUnitOfWork wuow(opCtx);
-        opCtx->recoveryUnit()->onCommit(
+        shard_role_details::getRecoveryUnit(opCtx)->onCommit(
             [&](OperationContext*, boost::optional<Timestamp> commitTime) {
                 commitTs = commitTime;
             });
         ASSERT(!commitTs);
-        ASSERT_OK(opCtx->recoveryUnit()->setTimestamp(ts2));
+        ASSERT_OK(shard_role_details::getRecoveryUnit(opCtx)->setTimestamp(ts2));
         ASSERT(!commitTs);
         wuow.commit();
         ASSERT_EQ(*commitTs, ts2);
     }
     ASSERT_EQ(*commitTs, ts2);
 
-    opCtx->recoveryUnit()->setCommitTimestamp(ts1);
+    shard_role_details::getRecoveryUnit(opCtx)->setCommitTimestamp(ts1);
     ASSERT_EQ(*commitTs, ts2);
 
     {
         WriteUnitOfWork wuow(opCtx);
-        opCtx->recoveryUnit()->onCommit(
+        shard_role_details::getRecoveryUnit(opCtx)->onCommit(
             [&](OperationContext*, boost::optional<Timestamp> commitTime) {
                 commitTs = commitTime;
             });
@@ -610,27 +649,27 @@ TEST_F(WiredTigerRecoveryUnitTestFixture, CommitTimestampBeforeSetTimestampOnAbo
     Timestamp ts1(5, 5);
     Timestamp ts2(6, 6);
 
-    opCtx->recoveryUnit()->setCommitTimestamp(ts2);
+    shard_role_details::getRecoveryUnit(opCtx)->setCommitTimestamp(ts2);
     ASSERT(!commitTs);
 
     {
         WriteUnitOfWork wuow(opCtx);
-        opCtx->recoveryUnit()->onCommit(
+        shard_role_details::getRecoveryUnit(opCtx)->onCommit(
             [&](OperationContext*, boost::optional<Timestamp> commitTime) {
                 commitTs = commitTime;
             });
         ASSERT(!commitTs);
     }
     ASSERT(!commitTs);
-    opCtx->recoveryUnit()->clearCommitTimestamp();
+    shard_role_details::getRecoveryUnit(opCtx)->clearCommitTimestamp();
 
     {
         WriteUnitOfWork wuow(opCtx);
-        opCtx->recoveryUnit()->onCommit(
+        shard_role_details::getRecoveryUnit(opCtx)->onCommit(
             [&](OperationContext*, boost::optional<Timestamp> commitTime) {
                 commitTs = commitTime;
             });
-        ASSERT_OK(opCtx->recoveryUnit()->setTimestamp(ts1));
+        ASSERT_OK(shard_role_details::getRecoveryUnit(opCtx)->setTimestamp(ts1));
         ASSERT(!commitTs);
     }
     ASSERT(!commitTs);
@@ -644,22 +683,22 @@ TEST_F(WiredTigerRecoveryUnitTestFixture, CommitTimestampAfterSetTimestampOnAbor
 
     {
         WriteUnitOfWork wuow(opCtx);
-        opCtx->recoveryUnit()->onCommit(
+        shard_role_details::getRecoveryUnit(opCtx)->onCommit(
             [&](OperationContext*, boost::optional<Timestamp> commitTime) {
                 commitTs = commitTime;
             });
         ASSERT(!commitTs);
-        ASSERT_OK(opCtx->recoveryUnit()->setTimestamp(ts2));
+        ASSERT_OK(shard_role_details::getRecoveryUnit(opCtx)->setTimestamp(ts2));
         ASSERT(!commitTs);
     }
     ASSERT(!commitTs);
 
-    opCtx->recoveryUnit()->setCommitTimestamp(ts1);
+    shard_role_details::getRecoveryUnit(opCtx)->setCommitTimestamp(ts1);
     ASSERT(!commitTs);
 
     {
         WriteUnitOfWork wuow(opCtx);
-        opCtx->recoveryUnit()->onCommit(
+        shard_role_details::getRecoveryUnit(opCtx)->onCommit(
             [&](OperationContext*, boost::optional<Timestamp> commitTime) {
                 commitTs = commitTime;
             });
@@ -977,9 +1016,9 @@ TEST_F(WiredTigerRecoveryUnitTestFixture, CommitWithDurableTimestamp) {
     Timestamp ts1(3, 3);
     Timestamp ts2(5, 5);
 
-    opCtx->recoveryUnit()->setCommitTimestamp(ts1);
-    opCtx->recoveryUnit()->setDurableTimestamp(ts2);
-    auto durableTs = opCtx->recoveryUnit()->getDurableTimestamp();
+    shard_role_details::getRecoveryUnit(opCtx)->setCommitTimestamp(ts1);
+    shard_role_details::getRecoveryUnit(opCtx)->setDurableTimestamp(ts2);
+    auto durableTs = shard_role_details::getRecoveryUnit(opCtx)->getDurableTimestamp();
     ASSERT_EQ(ts2, durableTs);
 
     {
@@ -991,7 +1030,7 @@ TEST_F(WiredTigerRecoveryUnitTestFixture, CommitWithDurableTimestamp) {
 TEST_F(WiredTigerRecoveryUnitTestFixture, CommitWithoutDurableTimestamp) {
     auto opCtx = clientAndCtx1.second.get();
     Timestamp ts1(5, 5);
-    opCtx->recoveryUnit()->setCommitTimestamp(ts1);
+    shard_role_details::getRecoveryUnit(opCtx)->setCommitTimestamp(ts1);
 
     {
         WriteUnitOfWork wuow(opCtx);
@@ -1011,13 +1050,15 @@ TEST_F(WiredTigerRecoveryUnitTestFixture, MultiTimestampConstraintsInternalState
     getCursor(ru1, &cursor);
     cursor->set_key(cursor, "key");
     cursor->set_value(cursor, "value");
-    invariantWTOK(wiredTigerCursorInsert(opCtx, cursor), cursor->session);
+    invariantWTOK(wiredTigerCursorInsert(*WiredTigerRecoveryUnit::get(opCtx), cursor),
+                  cursor->session);
 
     // Perform a write at ts1.
     cursor->set_key(cursor, "key2");
     cursor->set_value(cursor, "value");
     ASSERT_OK(ru1->setTimestamp(ts1));
-    invariantWTOK(wiredTigerCursorInsert(opCtx, cursor), cursor->session);
+    invariantWTOK(wiredTigerCursorInsert(*WiredTigerRecoveryUnit::get(opCtx), cursor),
+                  cursor->session);
 
     // Setting the timestamp again to the same value should not fail.
     ASSERT_OK(ru1->setTimestamp(ts1));
@@ -1031,7 +1072,8 @@ TEST_F(WiredTigerRecoveryUnitTestFixture, MultiTimestampConstraintsInternalState
     cursor->set_key(cursor, "key3");
     cursor->set_value(cursor, "value");
     ASSERT_OK(ru1->setTimestamp(ts2));
-    invariantWTOK(wiredTigerCursorInsert(opCtx, cursor), cursor->session);
+    invariantWTOK(wiredTigerCursorInsert(*WiredTigerRecoveryUnit::get(opCtx), cursor),
+                  cursor->session);
 
     ru1->commitUnitOfWork();
 }
@@ -1049,7 +1091,8 @@ TEST_F(WiredTigerRecoveryUnitTestFixture, AbandonSnapshotAbortMode) {
         getCursor(ru1, &cursor);
         cursor->set_key(cursor, key);
         cursor->set_value(cursor, "value");
-        invariantWTOK(wiredTigerCursorInsert(opCtx, cursor), cursor->session);
+        invariantWTOK(wiredTigerCursorInsert(*WiredTigerRecoveryUnit::get(opCtx), cursor),
+                      cursor->session);
 
         ru1->commitUnitOfWork();
     }
@@ -1149,13 +1192,15 @@ DEATH_TEST_REGEX_F(WiredTigerRecoveryUnitTestFixture,
         getCursor(ru1, &cursor);
         cursor->set_key(cursor, "key");
         cursor->set_value(cursor, "value");
-        invariantWTOK(wiredTigerCursorInsert(opCtx, cursor), cursor->session);
+        invariantWTOK(wiredTigerCursorInsert(*WiredTigerRecoveryUnit::get(opCtx), cursor),
+                      cursor->session);
 
         // Perform a write at ts1.
         cursor->set_key(cursor, "key2");
         cursor->set_value(cursor, "value");
         ASSERT_OK(ru1->setTimestamp(ts1));
-        invariantWTOK(wiredTigerCursorInsert(opCtx, cursor), cursor->session);
+        invariantWTOK(wiredTigerCursorInsert(*WiredTigerRecoveryUnit::get(opCtx), cursor),
+                      cursor->session);
 
         // Setting the timestamp again to a different value should detect that we're trying to set
         // multiple timestamps with the first write being non timestamped.
@@ -1178,8 +1223,8 @@ DEATH_TEST_F(WiredTigerRecoveryUnitTestFixture,
     auto opCtx = clientAndCtx1.second.get();
     Timestamp ts1(3, 3);
     Timestamp ts2(5, 5);
-    opCtx->recoveryUnit()->setDurableTimestamp(ts1);
-    opCtx->recoveryUnit()->setDurableTimestamp(ts2);
+    shard_role_details::getRecoveryUnit(opCtx)->setDurableTimestamp(ts1);
+    shard_role_details::getRecoveryUnit(opCtx)->setDurableTimestamp(ts2);
 }
 
 DEATH_TEST_F(WiredTigerRecoveryUnitTestFixture,
