@@ -28,23 +28,31 @@
  */
 
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/storage/wiredtiger/wiredtiger_session_cache.h"
-
+#include <cerrno>
+#include <cstdlib>
 #include <memory>
+#include <mutex>
+
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <wiredtiger.h>
 
 #include "mongo/base/error_codes.h"
-#include "mongo/db/concurrency/d_concurrency.h"
+#include "mongo/base/status.h"
 #include "mongo/db/global_settings.h"
 #include "mongo/db/repl/repl_settings.h"
 #include "mongo/db/storage/journal_listener.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_kv_engine.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_parameters_gen.h"
+#include "mongo/db/storage/wiredtiger/wiredtiger_session_cache.h"
+#include "mongo/db/storage/wiredtiger/wiredtiger_session_data.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_util.h"
 #include "mongo/logv2/log.h"
-#include "mongo/stdx/thread.h"
-#include "mongo/util/scopeguard.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/duration.h"
+#include "mongo/util/str.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
 
@@ -56,6 +64,7 @@ WiredTigerSession::WiredTigerSession(WT_CONNECTION* conn, uint64_t epoch, uint64
       _session(nullptr),
       _cursorGen(0),
       _cursorsOut(0),
+      _compiled(nullptr),
       _idleExpireTime(Date_t::min()) {
     invariantWTOK(conn->open_session(conn, nullptr, "isolation=snapshot", &_session), nullptr);
 }
@@ -65,12 +74,14 @@ WiredTigerSession::WiredTigerSession(WT_CONNECTION* conn,
                                      uint64_t epoch,
                                      uint64_t cursorEpoch)
     : _epoch(epoch),
-      _cache(cache),
       _session(nullptr),
       _cursorGen(0),
       _cursorsOut(0),
+      _cache(cache),
+      _compiled(nullptr),
       _idleExpireTime(Date_t::min()) {
     invariantWTOK(conn->open_session(conn, nullptr, "isolation=snapshot", &_session), nullptr);
+    setCompiledConfigurationsPerConnection(cache->getCompiledConfigurations());
 }
 
 WiredTigerSession::~WiredTigerSession() {
@@ -136,7 +147,7 @@ WT_CURSOR* WiredTigerSession::getNewCursor(const std::string& uri, const char* c
     return cursor;
 }
 
-void WiredTigerSession::releaseCursor(uint64_t id, WT_CURSOR* cursor, const std::string& config) {
+void WiredTigerSession::releaseCursor(uint64_t id, WT_CURSOR* cursor, std::string config) {
     // When releasing the cursor, we would want to check if the session cache is already in shutdown
     // and prevent the race condition that the shutdown starts after the check.
     WiredTigerSessionCache::BlockShutdown blockShutdown(_cache);
@@ -154,7 +165,7 @@ void WiredTigerSession::releaseCursor(uint64_t id, WT_CURSOR* cursor, const std:
     invariantWTOK(cursor->reset(cursor), _session);
 
     // Cursors are pushed to the front of the list and removed from the back
-    _cursors.push_front(WiredTigerCachedCursor(id, _cursorGen++, cursor, config));
+    _cursors.push_front(WiredTigerCachedCursor(id, _cursorGen++, cursor, std::move(config)));
 
     // A negative value for wiredTigercursorCacheSize means to use hybrid caching.
     std::uint32_t cacheSize = abs(gWiredTigerCursorCacheSize.load());
@@ -188,6 +199,31 @@ void WiredTigerSession::closeAllCursors(const std::string& uri) {
     }
 }
 
+void WiredTigerSession::reconfigure(const std::string& newConfig, std::string undoConfig) {
+    if (newConfig == undoConfig) {
+        // The undoConfig string is the config string that resets our session back to default
+        // settings. If our new configuration is the same as the undoconfig string, then that means
+        // that we are either setting our configuration back to default, or that the newConfig
+        // string does not change our default values. In this case, we can erase the undoConfig
+        // string from our set of undo config strings, since we no longer need to do any work to
+        // restore the session to its default configuration.
+        _undoConfigStrings.erase(undoConfig);
+    } else {
+        // Store the config string that will reset our session to its default configuration.
+        _undoConfigStrings.emplace(std::move(undoConfig));
+    }
+    auto wtSession = getSession();
+    invariantWTOK(wtSession->reconfigure(wtSession, newConfig.c_str()), wtSession);
+}
+
+void WiredTigerSession::resetSessionConfiguration() {
+    auto wtSession = getSession();
+    for (const std::string& undoConfigString : _undoConfigStrings) {
+        invariantWTOK(wtSession->reconfigure(wtSession, undoConfigString.c_str()), wtSession);
+    }
+    _undoConfigStrings.clear();
+}
+
 namespace {
 AtomicWord<unsigned long long> nextTableId(WiredTigerSession::kLastTableId);
 }
@@ -199,18 +235,14 @@ uint64_t WiredTigerSession::genTableId() {
 // -----------------------
 
 WiredTigerSessionCache::WiredTigerSessionCache(WiredTigerKVEngine* engine)
-    : _engine(engine),
-      _conn(engine->getConnection()),
-      _clockSource(_engine->getClockSource()),
-      _shuttingDown(0),
-      _prepareCommitOrAbortCounter(0) {}
+    : WiredTigerSessionCache(engine->getConnection(), engine->getClockSource(), engine) {}
 
-WiredTigerSessionCache::WiredTigerSessionCache(WT_CONNECTION* conn, ClockSource* cs)
-    : _engine(nullptr),
-      _conn(conn),
-      _clockSource(cs),
-      _shuttingDown(0),
-      _prepareCommitOrAbortCounter(0) {}
+WiredTigerSessionCache::WiredTigerSessionCache(WT_CONNECTION* conn,
+                                               ClockSource* cs,
+                                               WiredTigerKVEngine* engine)
+    : _conn(conn), _clockSource(cs), _engine(engine) {
+    uassertStatusOK(_compiledConfigurations.compileAll(_conn));
+}
 
 WiredTigerSessionCache::~WiredTigerSessionCache() {
     shuttingDown();
@@ -243,19 +275,9 @@ void WiredTigerSessionCache::waitUntilDurable(OperationContext* opCtx,
     // For inMemory storage engines, the data is "as durable as it's going to get".
     // That is, a restart is equivalent to a complete node failure.
     if (isEphemeral()) {
-        auto journalListener = [&]() -> JournalListener* {
-            // The JournalListener may not be set immediately, so we must check under a mutex so as
-            // not to access the variable while setting a JournalListener. A JournalListener is only
-            // allowed to be set once, so using the pointer outside of a mutex is safe.
-            stdx::unique_lock<Latch> lk(_journalListenerMutex);
-            return _journalListener;
-        }();
-        if (journalListener && useListener == UseJournalListener::kUpdate) {
-            // Update the JournalListener before we return. Does a write while fetching the
-            // timestamp if primary. As far as listeners are concerned, all writes are as 'durable'
-            // as they are ever going to get on an inMemory storage engine.
-            auto token = _journalListener->getToken(opCtx);
-            journalListener->onDurable(token);
+        auto [journalListener, token] = _getJournalListenerWithToken(opCtx, useListener);
+        if (token) {
+            journalListener->onDurable(token.value());
         }
         return;
     }
@@ -271,7 +293,7 @@ void WiredTigerSessionCache::waitUntilDurable(OperationContext* opCtx,
     // incidentally what we want. A "true" stable checkpoint (a stable timestamp was set on the
     // WT_CONNECTION, i.e: replication is on) requires `forceCheckpoint` to be true and journaling
     // to be enabled.
-    if (syncType == Fsync::kCheckpointStableTimestamp && getGlobalReplSettings().usingReplSets()) {
+    if (syncType == Fsync::kCheckpointStableTimestamp && getGlobalReplSettings().isReplSet()) {
         invariant(!isEphemeral());
     }
 
@@ -279,46 +301,19 @@ void WiredTigerSessionCache::waitUntilDurable(OperationContext* opCtx,
     // waiters, as a log flush is much cheaper than a full checkpoint.
     if ((syncType == Fsync::kCheckpointStableTimestamp || syncType == Fsync::kCheckpointAll) &&
         !isEphemeral()) {
-        auto journalListener = [&]() -> JournalListener* {
-            // The JournalListener may not be set immediately, so we must check under a mutex so
-            // as not to access the variable while setting a JournalListener. A JournalListener
-            // is only allowed to be set once, so using the pointer outside of a mutex is safe.
-            stdx::unique_lock<Latch> lk(_journalListenerMutex);
-            return _journalListener;
-        }();
-        boost::optional<JournalListener::Token> token;
-        if (journalListener && useListener == UseJournalListener::kUpdate) {
-            // Update a persisted value with the latest write timestamp that is safe across
-            // startup recovery in the repl layer. Then report that timestamp as durable to the
-            // repl layer below after we have flushed in-memory data to disk.
-            // Note: only does a write if primary, otherwise just fetches the timestamp.
-            token = journalListener->getToken(opCtx);
-        }
+        auto [journalListener, token] = _getJournalListenerWithToken(opCtx, useListener);
 
         getKVEngine()->forceCheckpoint(syncType == Fsync::kCheckpointStableTimestamp);
 
         if (token) {
             journalListener->onDurable(token.value());
         }
+
         LOGV2_DEBUG(22418, 4, "created checkpoint (forced)");
         return;
     }
 
-    auto journalListener = [&]() -> JournalListener* {
-        // The JournalListener may not be set immediately, so we must check under a mutex so as not
-        // to access the variable while setting a JournalListener. A JournalListener is only allowed
-        // to be set once, so using the pointer outside of a mutex is safe.
-        stdx::unique_lock<Latch> lk(_journalListenerMutex);
-        return _journalListener;
-    }();
-    boost::optional<JournalListener::Token> token;
-    if (journalListener && useListener == UseJournalListener::kUpdate) {
-        // Update a persisted value with the latest write timestamp that is safe across startup
-        // recovery in the repl layer. Then report that timestamp as durable to the repl layer below
-        // after we have flushed in-memory data to disk.
-        // Note: only does a write if primary, otherwise just fetches the timestamp.
-        token = journalListener->getToken(opCtx);
-    }
+    auto [journalListener, token] = _getJournalListenerWithToken(opCtx, useListener);
 
     uint32_t start = _lastSyncTime.load();
     // Do the remainder in a critical section that ensures only a single thread at a time
@@ -327,6 +322,14 @@ void WiredTigerSessionCache::waitUntilDurable(OperationContext* opCtx,
     uint32_t current = _lastSyncTime.loadRelaxed();  // synchronized with writes through mutex
     if (current != start) {
         // Someone else synced already since we read lastSyncTime, so we're done!
+
+        // Unconditionally unlock mutex here to run operations that do not require synchronization.
+        // The JournalListener is the only operation that meets this criteria currently.
+        lk.unlock();
+        if (token) {
+            journalListener->onDurable(token.value());
+        }
+
         return;
     }
     _lastSyncTime.store(current + 1);
@@ -345,15 +348,23 @@ void WiredTigerSessionCache::waitUntilDurable(OperationContext* opCtx,
                   _waitUntilDurableSession);
     LOGV2_DEBUG(22419, 4, "flushed journal");
 
+    // The session is reset periodically so that WT doesn't consider it a rogue session and log
+    // about it. The session doesn't actually pin any resources that need to be released.
+    if (_timeSinceLastDurabilitySessionReset.millis() > (5 * 60 * 1000 /* 5 minutes */)) {
+        _waitUntilDurableSession->reset(_waitUntilDurableSession);
+        _timeSinceLastDurabilitySessionReset.reset();
+    }
+
+    // Unconditionally unlock mutex here to run operations that do not require synchronization.
+    // The JournalListener is the only operation that meets this criteria currently.
+    lk.unlock();
     if (token) {
         journalListener->onDurable(token.value());
     }
 }
 
-void WiredTigerSessionCache::waitUntilPreparedUnitOfWorkCommitsOrAborts(OperationContext* opCtx,
-                                                                        std::uint64_t lastCount) {
-    invariant(opCtx);
-
+void WiredTigerSessionCache::waitUntilPreparedUnitOfWorkCommitsOrAborts(
+    Interruptible& interruptible, std::uint64_t lastCount) {
     // It is possible for a prepared transaction to block on bonus eviction inside WiredTiger after
     // it commits or rolls-back, but this delays it from signalling us to wake up. In the very
     // worst case that the only evictable page is the one pinned by our cursor, AND there are no
@@ -363,9 +374,10 @@ void WiredTigerSessionCache::waitUntilPreparedUnitOfWorkCommitsOrAborts(Operatio
     const auto deadline = Date_t::now() + Seconds(1);
     stdx::unique_lock<Latch> lk(_prepareCommittedOrAbortedMutex);
     if (lastCount == _prepareCommitOrAbortCounter.loadRelaxed()) {
-        opCtx->waitForConditionOrInterruptUntil(_prepareCommittedOrAbortedCond, lk, deadline, [&] {
-            return _prepareCommitOrAbortCounter.loadRelaxed() > lastCount;
-        });
+        interruptible.waitForConditionOrInterruptUntil(
+            _prepareCommittedOrAbortedCond, lk, deadline, [&] {
+                return _prepareCommitOrAbortCounter.loadRelaxed() > lastCount;
+            });
     }
 }
 
@@ -494,6 +506,8 @@ void WiredTigerSessionCache::releaseSession(WiredTigerSession* session) {
         if (gWiredTigerCursorCacheSize.load() < 0) {
             session->closeAllCursors("");
         }
+
+        session->resetSessionConfiguration();
         invariantWTOK(ss->reset(ss), ss);
     }
 
@@ -529,4 +543,25 @@ void WiredTigerSessionCache::WiredTigerSessionDeleter::operator()(
     session->_cache->releaseSession(session);
 }
 
+std::pair<JournalListener*, boost::optional<JournalListener::Token>>
+WiredTigerSessionCache::_getJournalListenerWithToken(OperationContext* opCtx,
+                                                     UseJournalListener useListener) {
+    auto journalListener = [&]() -> JournalListener* {
+        // The JournalListener may not be set immediately, so we must check under a mutex so
+        // as not to access the variable while setting a JournalListener. A JournalListener
+        // is only allowed to be set once, so using the pointer outside of a mutex is safe.
+        stdx::unique_lock<Latch> lk(_journalListenerMutex);
+        return _journalListener;
+    }();
+    boost::optional<JournalListener::Token> token;
+    if (journalListener && useListener == UseJournalListener::kUpdate) {
+        // Update a persisted value with the latest write timestamp that is safe across
+        // startup recovery in the repl layer. Then report that timestamp as durable to the
+        // repl layer below after we have flushed in-memory data to disk.
+        // Note: only does a write if primary, otherwise just fetches the timestamp.
+        token = journalListener->getToken(opCtx);
+    }
+    return std::make_pair(journalListener, token);
+}
+
 }  // namespace mongo
