@@ -29,14 +29,16 @@
 
 #pragma once
 
+#include <cstdint>
+#include <memory>
 #include <string>
-
 #include <wiredtiger.h>
 
 #include "mongo/base/string_data.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_session_cache.h"
 #include "mongo/platform/atomic_word.h"
 #include "mongo/platform/mutex.h"
+#include "mongo/util/assert_util_core.h"
 #include "mongo/util/string_map.h"
 
 namespace mongo {
@@ -76,7 +78,7 @@ public:
         AtomicWord<bool> _dirty;
     };
 
-    WiredTigerSizeStorer(WT_CONNECTION* conn, const std::string& storageUri, bool readOnly = false);
+    WiredTigerSizeStorer(WT_CONNECTION* conn, const std::string& storageUri);
     ~WiredTigerSizeStorer() = default;
 
     /**
@@ -106,7 +108,6 @@ private:
     WT_CONNECTION* _conn;
     const std::string _storageUri;
     const uint64_t _tableId;  // Not persisted
-    const bool _readOnly;
 
     // Serializes flushes to disk.
     Mutex _flushMutex = MONGO_MAKE_LATCH("WiredTigerSessionStorer::_flushMutex");
