@@ -30,27 +30,62 @@
 
 #include "mongo/db/storage/wiredtiger/wiredtiger_util.h"
 
-#include <boost/filesystem.hpp>
-#include <boost/filesystem/fstream.hpp>
+#include <absl/container/node_hash_set.h>
+#include <boost/cstdint.hpp>
+#include <boost/filesystem/directory.hpp>
+#include <boost/filesystem/operations.hpp>
+#include <boost/filesystem/path.hpp>
+#include <boost/iterator/iterator_facade.hpp>
+#include <boost/move/utility_core.hpp>
+#include <fmt/format.h>
+// IWYU pragma: no_include "boost/system/detail/error_code.hpp"
+#include <algorithm>
+#include <cerrno>
+#include <cstdint>
+#include <cstring>
+#include <exception>
+#include <map>
+#include <memory>
+#include <new>
+#include <ostream>
+#include <stdexcept>
+#include <wiredtiger.h>
 
-#include "mongo/base/simple_string_data_comparator.h"
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/bson/bsontypes.h"
 #include "mongo/bson/json.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/bson/util/builder.h"
+#include "mongo/bson/util/builder_fwd.h"
 #include "mongo/db/concurrency/exception_util.h"
 #include "mongo/db/concurrency/exception_util_gen.h"
 #include "mongo/db/global_settings.h"
-#include "mongo/db/server_options_general_gen.h"
+#include "mongo/db/repl/repl_settings.h"
+#include "mongo/db/server_feature_flags_gen.h"
 #include "mongo/db/snapshot_window_options_gen.h"
+#include "mongo/db/storage/storage_options.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_kv_engine.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_parameters_gen.h"
+#include "mongo/db/storage/wiredtiger/wiredtiger_recovery_unit.h"
+#include "mongo/db/storage/wiredtiger/wiredtiger_session_cache.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/redaction.h"
+#include "mongo/platform/atomic_word.h"
 #include "mongo/util/pcre.h"
 #include "mongo/util/processinfo.h"
+#include "mongo/util/scopeguard.h"
+#include "mongo/util/static_immortal.h"
+#include "mongo/util/str.h"
+#include "mongo/util/string_map.h"
 #include "mongo/util/testing_proctor.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kWiredTiger
 
-
 // From src/third_party/wiredtiger/src/include/txn.h
+#define WT_TXN_ROLLBACK_REASON_CACHE_OVERFLOW "transaction rolled back because of cache overflow"
+
 #define WT_TXN_ROLLBACK_REASON_OLDEST_FOR_EVICTION \
     "oldest pinned transaction ID rolled back for eviction"
 
@@ -58,11 +93,45 @@
     "transaction is too large and will not fit in the storage engine cache"
 namespace mongo {
 
+void WiredTigerEventHandler::setWtConnReadyStatus(bool status) {
+    stdx::unique_lock<mongo::Mutex> lock(_mutex);
+    _wtConnReady = status;
+    if (_activeSections == 0 || _wtConnReady) {
+        return;
+    }
+    LOGV2(7003100,
+          "WiredTiger connection close is waiting for active statistics readers to finish",
+          "activeReaders"_attr = _activeSections);
+    _idleCondition.wait(lock, [this]() { return _activeSections != 0; });
+}
+
+bool WiredTigerEventHandler::getSectionActivityPermit() {
+    stdx::lock_guard<mongo::Mutex> lock(_mutex);
+    if (_wtConnReady) {
+        _activeSections++;
+        return true;
+    }
+    return false;
+}
+
+void WiredTigerEventHandler::releaseSectionActivityPermit() {
+    stdx::unique_lock<mongo::Mutex> lock(_mutex);
+    _activeSections--;
+    if (_activeSections == 0 && !_wtConnReady) {
+        _idleCondition.notify_all();
+        return;
+    }
+}
+
 namespace {
 
+// TODO SERVER-81069: Remove this.
+MONGO_FAIL_POINT_DEFINE(allowEncryptionOptionsInCreationString);
+
 const std::string kTableChecksFileName = "_wt_table_checks";
 const std::string kTableExtension = ".wt";
 const std::string kWiredTigerBackupFile = "WiredTiger.backup";
+const static StaticImmortal<pcre::Regex> encryptionOptsRegex(R"re(encryption=\([^\)]*\),?)re");
 
 /**
  * Removes the 'kTableChecksFileName' file in the dbpath, if it exists.
@@ -86,50 +155,10 @@ void removeTableChecksFile() {
     }
 }
 
-void setTableWriteTimestampAssertion(OperationContext* opCtx,
-                                     WiredTigerSessionCache* sessionCache,
-                                     const std::string& uri,
-                                     bool on) {
-    const std::string setting = on ? "assert=(write_timestamp=on)" : "assert=(write_timestamp=off)";
-    LOGV2_DEBUG(6003700,
-                1,
-                "Changing table write timestamp assertion settings",
-                "uri"_attr = uri,
-                "writeTimestampAssertionOn"_attr = on);
-    auto status = sessionCache->getKVEngine()->alterMetadata(opCtx, uri, setting);
-    if (!status.isOK()) {
-        // Dump the storage engine's internal state to assist in diagnosis.
-        sessionCache->getKVEngine()->dump();
-
-        auto sessionPtr = sessionCache->getSession();
-        LOGV2_FATAL(
-            6003701,
-            "Failed to update write timestamp assertion setting",
-            "uri"_attr = uri,
-            "writeTimestampAssertionOn"_attr = on,
-            "error"_attr = status.code(),
-            "metadata"_attr =
-                redact(WiredTigerUtil::getMetadataCreate(sessionPtr->getSession(), uri).getValue()),
-            "message"_attr = status.reason());
-    }
-}
-
 }  // namespace
 
 using std::string;
 
-bool wasRollbackReasonCachePressure(WT_SESSION* session) {
-    if (session) {
-        const auto reason = session->get_rollback_reason(session);
-        if (reason) {
-            return strncmp(WT_TXN_ROLLBACK_REASON_OLDEST_FOR_EVICTION,
-                           reason,
-                           sizeof(WT_TXN_ROLLBACK_REASON_OLDEST_FOR_EVICTION)) == 0;
-        }
-    }
-    return false;
-}
-
 /**
  * Configured WT cache is deemed insufficient for a transaction when its dirty bytes in cache
  * exceed a certain threshold on the proportion of total cache which is used by transaction.
@@ -186,18 +215,25 @@ Status wtRCToStatus_slow(int retCode, WT_SESSION* session, StringData prefix) {
         double cacheThreshold = gTransactionTooLargeForCacheThreshold.load();
         bool txnTooLargeEnabled = cacheThreshold < 1.0;
         bool temporarilyUnavailableEnabled = gEnableTemporarilyUnavailableExceptions.load();
+        const char* reason = session ? session->get_rollback_reason(session) : "";
         bool reasonWasCachePressure = (txnTooLargeEnabled || temporarilyUnavailableEnabled) &&
-            wasRollbackReasonCachePressure(session);
+            reason && session &&
+            (strncmp(WT_TXN_ROLLBACK_REASON_OLDEST_FOR_EVICTION,
+                     reason,
+                     sizeof(WT_TXN_ROLLBACK_REASON_OLDEST_FOR_EVICTION)) == 0 ||
+             strncmp(WT_TXN_ROLLBACK_REASON_CACHE_OVERFLOW,
+                     reason,
+                     sizeof(WT_TXN_ROLLBACK_REASON_CACHE_OVERFLOW)) == 0);
 
         if (reasonWasCachePressure) {
             if (txnTooLargeEnabled && isCacheInsufficientForTransaction(session, cacheThreshold)) {
                 throwTransactionTooLargeForCache(
-                    generateContextStrStream(WT_TXN_ROLLBACK_REASON_TOO_LARGE_FOR_CACHE));
+                    generateContextStrStream(WT_TXN_ROLLBACK_REASON_TOO_LARGE_FOR_CACHE)
+                    << " (" << reason << ")");
             }
 
             if (temporarilyUnavailableEnabled) {
-                throwTemporarilyUnavailableException(
-                    generateContextStrStream(WT_TXN_ROLLBACK_REASON_OLDEST_FOR_EVICTION));
+                throwTemporarilyUnavailableException(generateContextStrStream(reason));
             }
         }
 
@@ -225,7 +261,7 @@ Status wtRCToStatus_slow(int retCode, WT_SESSION* session, StringData prefix) {
     return Status(ErrorCodes::UnknownError, s);
 }
 
-void WiredTigerUtil::fetchTypeAndSourceURI(OperationContext* opCtx,
+void WiredTigerUtil::fetchTypeAndSourceURI(WiredTigerRecoveryUnit& ru,
                                            const std::string& tableUri,
                                            std::string* type,
                                            std::string* source) {
@@ -233,7 +269,7 @@ void WiredTigerUtil::fetchTypeAndSourceURI(OperationContext* opCtx,
     const size_t colon = tableUri.find(':');
     invariant(colon != string::npos);
     colgroupUri += tableUri.substr(colon);
-    StatusWith<std::string> colgroupResult = getMetadataCreate(opCtx, colgroupUri);
+    StatusWith<std::string> colgroupResult = getMetadataCreate(ru, colgroupUri);
     invariant(colgroupResult.getStatus());
     WiredTigerConfigParser parser(colgroupResult.getValue());
 
@@ -278,10 +314,9 @@ StatusWith<std::string> WiredTigerUtil::getMetadataCreate(WT_SESSION* session, S
     return _getMetadata(cursor, uri);
 }
 
-StatusWith<std::string> WiredTigerUtil::getMetadataCreate(OperationContext* opCtx, StringData uri) {
-    invariant(opCtx);
-
-    auto session = WiredTigerRecoveryUnit::get(opCtx)->getSessionNoTxn();
+StatusWith<std::string> WiredTigerUtil::getMetadataCreate(WiredTigerRecoveryUnit& ru,
+                                                          StringData uri) {
+    auto session = ru.getSessionNoTxn();
 
     WT_CURSOR* cursor = nullptr;
     try {
@@ -310,10 +345,8 @@ StatusWith<std::string> WiredTigerUtil::getMetadata(WT_SESSION* session, StringD
     return _getMetadata(cursor, uri);
 }
 
-StatusWith<std::string> WiredTigerUtil::getMetadata(OperationContext* opCtx, StringData uri) {
-    invariant(opCtx);
-
-    auto session = WiredTigerRecoveryUnit::get(opCtx)->getSessionNoTxn();
+StatusWith<std::string> WiredTigerUtil::getMetadata(WiredTigerRecoveryUnit& ru, StringData uri) {
+    auto session = ru.getSessionNoTxn();
     WT_CURSOR* cursor = nullptr;
     try {
         const std::string metadataURI = "metadata:";
@@ -332,10 +365,10 @@ StatusWith<std::string> WiredTigerUtil::getMetadata(OperationContext* opCtx, Str
     return _getMetadata(cursor, uri);
 }
 
-Status WiredTigerUtil::getApplicationMetadata(OperationContext* opCtx,
+Status WiredTigerUtil::getApplicationMetadata(WiredTigerRecoveryUnit& ru,
                                               StringData uri,
                                               BSONObjBuilder* bob) {
-    StatusWith<std::string> metadataResult = getMetadata(opCtx, uri);
+    StatusWith<std::string> metadataResult = getMetadata(ru, uri);
     if (!metadataResult.isOK()) {
         return metadataResult.getStatus();
     }
@@ -357,7 +390,7 @@ Status WiredTigerUtil::getApplicationMetadata(OperationContext* opCtx,
     WT_CONFIG_ITEM keyItem;
     WT_CONFIG_ITEM valueItem;
     int ret;
-    auto keysSeen = SimpleStringDataComparator::kInstance.makeStringDataUnorderedSet();
+    StringDataSet keysSeen;
     while ((ret = parser.next(&keyItem, &valueItem)) == 0) {
         const StringData key(keyItem.str, keyItem.len);
         if (keysSeen.count(key)) {
@@ -386,21 +419,19 @@ Status WiredTigerUtil::getApplicationMetadata(OperationContext* opCtx,
     return Status::OK();
 }
 
-StatusWith<BSONObj> WiredTigerUtil::getApplicationMetadata(OperationContext* opCtx,
+StatusWith<BSONObj> WiredTigerUtil::getApplicationMetadata(WiredTigerRecoveryUnit& ru,
                                                            StringData uri) {
     BSONObjBuilder bob;
-    Status status = getApplicationMetadata(opCtx, uri, &bob);
+    Status status = getApplicationMetadata(ru, uri, &bob);
     if (!status.isOK()) {
         return StatusWith<BSONObj>(status);
     }
     return StatusWith<BSONObj>(bob.obj());
 }
 
-StatusWith<int64_t> WiredTigerUtil::checkApplicationMetadataFormatVersion(OperationContext* opCtx,
-                                                                          StringData uri,
-                                                                          int64_t minimumVersion,
-                                                                          int64_t maximumVersion) {
-    StatusWith<std::string> result = getMetadata(opCtx, uri);
+StatusWith<int64_t> WiredTigerUtil::checkApplicationMetadataFormatVersion(
+    WiredTigerRecoveryUnit& ru, StringData uri, int64_t minimumVersion, int64_t maximumVersion) {
+    StatusWith<std::string> result = getMetadata(ru, uri);
     if (result.getStatus().code() == ErrorCodes::NoSuchKey) {
         return result.getStatus();
     }
@@ -475,6 +506,15 @@ Status WiredTigerUtil::checkTableCreationOptions(const BSONElement& configElem)
         return {ErrorCodes::Error(6627201), "Configuration 'type=lsm' is not supported."};
     }
 
+    if (gFeatureFlagBanEncryptionOptionsInCollectionCreation.isEnabled(
+            serverGlobalParams.featureCompatibility.acquireFCVSnapshot()) &&
+        encryptionOptsRegex->matchView(config) &&
+        MONGO_likely(!allowEncryptionOptionsInCreationString.shouldFail())) {
+        return {ErrorCodes::IllegalOperation,
+                "Manual configuration of encryption options as part of 'configString' is not "
+                "supported"};
+    }
+
     Status status = wtRCToStatus(
         wiredtiger_config_validate(nullptr, &eventHandler, "WT_SESSION.create", config.rawData()),
         nullptr);
@@ -583,6 +623,17 @@ int64_t WiredTigerUtil::getIdentReuseSize(WT_SESSION* s, const std::string& uri)
     return result.getValue();
 }
 
+int64_t WiredTigerUtil::getIdentCompactRewrittenExpectedSize(WT_SESSION* s,
+                                                             const std::string& uri) {
+    auto result =
+        WiredTigerUtil::getStatisticsValue(s,
+                                           "statistics:" + uri,
+                                           "statistics=(fast)",
+                                           WT_STAT_DSRC_BTREE_COMPACT_BYTES_REWRITTEN_EXPECTED);
+    uassertStatusOK(result.getStatus());
+    return result.getValue();
+}
+
 size_t WiredTigerUtil::getCacheSizeMB(double requestedCacheSizeGB) {
     double cacheSizeMB;
     const double kMaxSizeCacheMB = 10 * 1000 * 1000;
@@ -597,7 +648,6 @@ size_t WiredTigerUtil::getCacheSizeMB(double requestedCacheSizeGB) {
     }
     if (cacheSizeMB > kMaxSizeCacheMB) {
         LOGV2(22429,
-              "Requested cache size: {requestedMB}MB exceeds max; setting to {maximumMB}MB",
               "Requested cache size exceeds max, setting to maximum",
               "requestedMB"_attr = cacheSizeMB,
               "maximumMB"_attr = kMaxSizeCacheMB);
@@ -803,11 +853,42 @@ int mdb_handle_progress(WT_EVENT_HANDLER* handler,
     return 0;
 }
 
+/**
+ * Defines a callback function that can be passed via a WT_EVENT_HANDLER*
+ * (WT_EVENT_HANDLER::handle_general) into WT::wiredtiger_open() call.
+ *
+ * The void* WT_SESSION::app_private is leveraged to inject MDB state into the WT code layer.
+ * Long running WT::compact operations will periodically use this callback function to check whether
+ * or not to quit early and fail the WT::compact operation.
+ */
+int mdb_handle_general(WT_EVENT_HANDLER* handler,
+                       WT_CONNECTION* wt_conn,
+                       WT_SESSION* session,
+                       WT_EVENT_TYPE type,
+                       void* arg) {
+    WiredTigerEventHandler* wtHandler = reinterpret_cast<WiredTigerEventHandler*>(handler);
+    if (type == WT_EVENT_CONN_READY) {
+        wtHandler->setWtConnReadyStatus(true);
+    } else if (type == WT_EVENT_CONN_CLOSE) {
+        wtHandler->setWtConnReadyStatus(false);
+    }
+    if (type != WT_EVENT_COMPACT_CHECK || session == nullptr || session->app_private == nullptr) {
+        return 0;
+    }
+
+    return reinterpret_cast<Interruptible*>(session->app_private)
+               ->checkForInterruptNoAssert()
+               .isOK()
+        ? 0
+        : -1;  // Returning non-zero indicates an error to WT. The precise value is irrelevant.
+}
+
 WT_EVENT_HANDLER defaultEventHandlers() {
     WT_EVENT_HANDLER handlers = {};
     handlers.handle_error = mdb_handle_error;
     handlers.handle_message = mdb_handle_message;
     handlers.handle_progress = mdb_handle_progress;
+    handlers.handle_general = mdb_handle_general;
     return handlers;
 }
 }  // namespace
@@ -820,7 +901,7 @@ WiredTigerEventHandler::WiredTigerEventHandler() {
     handler->handle_message = mdb_handle_message;
     handler->handle_progress = mdb_handle_progress;
     handler->handle_close = nullptr;
-    handler->handle_general = nullptr;
+    handler->handle_general = mdb_handle_general;
 }
 
 WT_EVENT_HANDLER* WiredTigerEventHandler::getWtEventHandler() {
@@ -853,27 +934,36 @@ int WiredTigerUtil::ErrorAccumulator::onError(WT_EVENT_HANDLER* handler,
     }
 }
 
-int WiredTigerUtil::verifyTable(OperationContext* opCtx,
+int WiredTigerUtil::verifyTable(WiredTigerRecoveryUnit& ru,
                                 const std::string& uri,
                                 std::vector<std::string>* errors) {
     ErrorAccumulator eventHandler(errors);
 
     // Try to close as much as possible to avoid EBUSY errors.
-    WiredTigerRecoveryUnit::get(opCtx)->getSession()->closeAllCursors(uri);
-    WiredTigerSessionCache* sessionCache = WiredTigerRecoveryUnit::get(opCtx)->getSessionCache();
+    ru.getSession()->closeAllCursors(uri);
+    WiredTigerSessionCache* sessionCache = ru.getSessionCache();
     sessionCache->closeAllCursors(uri);
 
     // Open a new session with custom error handlers.
-    WT_CONNECTION* conn = WiredTigerRecoveryUnit::get(opCtx)->getSessionCache()->conn();
+    WT_CONNECTION* conn = ru.getSessionCache()->conn();
     WT_SESSION* session;
-    invariantWTOK(conn->open_session(conn, &eventHandler, nullptr, &session), nullptr);
+
+    if (gFeatureFlagPrefetch.isEnabled(
+            serverGlobalParams.featureCompatibility.acquireFCVSnapshot()) &&
+        !sessionCache->isEphemeral()) {
+        invariantWTOK(conn->open_session(conn, &eventHandler, "prefetch=(enabled=true)", &session),
+                      nullptr);
+    } else {
+        invariantWTOK(conn->open_session(conn, &eventHandler, nullptr, &session), nullptr);
+    }
+
     ON_BLOCK_EXIT([&] { session->close(session, ""); });
 
     // Do the verify. Weird parens prevent treating "verify" as a macro.
     return (session->verify)(session, uri.c_str(), nullptr);
 }
 
-void WiredTigerUtil::validateTableLogging(OperationContext* opCtx,
+void WiredTigerUtil::validateTableLogging(WiredTigerRecoveryUnit& ru,
                                           StringData uri,
                                           bool isLogged,
                                           boost::optional<StringData> indexName,
@@ -894,7 +984,7 @@ void WiredTigerUtil::validateTableLogging(OperationContext* opCtx,
     }
     attrs.add("uri", uri);
 
-    auto metadata = WiredTigerUtil::getMetadataCreate(opCtx, uri);
+    auto metadata = WiredTigerUtil::getMetadataCreate(ru, uri);
     if (!metadata.isOK()) {
         attrs.add("error", metadata.getStatus());
         LOGV2_WARNING(6898100, "Failed to check WT table logging setting", attrs);
@@ -918,7 +1008,7 @@ void WiredTigerUtil::validateTableLogging(OperationContext* opCtx,
     }
 }
 
-void WiredTigerUtil::notifyStartupComplete() {
+void WiredTigerUtil::notifyStorageStartupRecoveryComplete() {
     removeTableChecksFile();
 }
 
@@ -932,7 +1022,7 @@ bool WiredTigerUtil::useTableLogging(const NamespaceString& nss) {
     // We only turn off logging in the case of:
     // 1) Replication is enabled (the typical deployment), or
     // 2) We're running as a standalone with recoverFromOplogAsStandalone=true
-    const bool journalWritesBecauseStandalone = !getGlobalReplSettings().usingReplSets() &&
+    const bool journalWritesBecauseStandalone = !getGlobalReplSettings().isReplSet() &&
         !repl::ReplSettings::shouldRecoverFromOplogAsStandalone();
     if (journalWritesBecauseStandalone) {
         return true;
@@ -942,7 +1032,7 @@ bool WiredTigerUtil::useTableLogging(const NamespaceString& nss) {
     invariant(nss.size() > 0);
 
     // Of the replica set configurations:
-    if (!nss.isLocal()) {
+    if (!nss.isLocalDB()) {
         // All replicated collections are not logged.
         return false;
     }
@@ -958,19 +1048,21 @@ bool WiredTigerUtil::useTableLogging(const NamespaceString& nss) {
     return true;
 }
 
-Status WiredTigerUtil::setTableLogging(OperationContext* opCtx, const std::string& uri, bool on) {
+Status WiredTigerUtil::setTableLogging(WiredTigerRecoveryUnit& ru,
+                                       const std::string& uri,
+                                       bool on) {
     if (gWiredTigerSkipTableLoggingChecksOnStartup) {
         LOGV2_DEBUG(5548302, 1, "Skipping table logging check", "uri"_attr = uri);
         return Status::OK();
     }
 
-    // Try to close as much as possible to avoid EBUSY errors.
-    WiredTigerRecoveryUnit::get(opCtx)->getSession()->closeAllCursors(uri);
-    WiredTigerSessionCache* sessionCache = WiredTigerRecoveryUnit::get(opCtx)->getSessionCache();
-    sessionCache->closeAllCursors(uri);
-
     const std::string setting = on ? "log=(enabled=true)" : "log=(enabled=false)";
 
+    // Try to close as much as possible to avoid EBUSY errors.
+    ru.getSession()->closeAllCursors(uri);
+    WiredTigerSessionCache* sessionCache = ru.getSessionCache();
+    sessionCache->closeAllCursors(uri);
+
     // This method does some "weak" parsing to see if the table is in the expected logging
     // state. Only attempt to alter the table when a change is needed. This avoids grabbing heavy
     // locks in WT when creating new tables for collections and indexes. Those tables are created
@@ -1000,7 +1092,7 @@ Status WiredTigerUtil::setTableLogging(OperationContext* opCtx, const std::strin
         22432, 1, "Changing table logging settings", "uri"_attr = uri, "loggingEnabled"_attr = on);
     // Only alter the metadata once we're sure that we need to change the table settings, since
     // WT_SESSION::alter may return EBUSY and require taking a checkpoint to make progress.
-    auto status = sessionCache->getKVEngine()->alterMetadata(opCtx, uri, setting);
+    auto status = sessionCache->getKVEngine()->alterMetadata(uri, setting);
     if (!status.isOK()) {
         // Dump the storage engine's internal state to assist in diagnosis.
         sessionCache->getKVEngine()->dump();
@@ -1014,15 +1106,6 @@ Status WiredTigerUtil::setTableLogging(OperationContext* opCtx, const std::strin
                     "message"_attr = status.reason());
     }
 
-    // The write timestamp assertion setting only needs to be changed at startup. It will be turned
-    // on when logging is disabled, and off when logging is enabled.
-    if (TestingProctor::instance().isEnabled()) {
-        setTableWriteTimestampAssertion(opCtx, sessionCache, uri, !on);
-    } else {
-        // Disables the assertion when the testing proctor is off.
-        setTableWriteTimestampAssertion(opCtx, sessionCache, uri, false /* on */);
-    }
-
     return Status::OK();
 }
 
@@ -1103,7 +1186,7 @@ Status WiredTigerUtil::exportTableToBSON(WT_SESSION* session,
     return Status::OK();
 }
 
-StatusWith<std::string> WiredTigerUtil::generateImportString(const StringData& ident,
+StatusWith<std::string> WiredTigerUtil::generateImportString(StringData ident,
                                                              const BSONObj& storageMetadata,
                                                              const ImportOptions& importOptions) {
     if (!storageMetadata.hasField(ident)) {
@@ -1146,6 +1229,13 @@ StatusWith<std::string> WiredTigerUtil::generateImportString(const StringData& i
     if (importOptions.importTimestampRule == ImportOptions::ImportTimestampRule::kStable) {
         ss << "compare_timestamp=stable,";
     }
+    if (!importOptions.panicOnCorruptWtMetadata) {
+        invariant(!importOptions.repair);
+        ss << "panic_corrupt=false,";
+    }
+    if (importOptions.repair) {
+        ss << "repair=true,";
+    }
     ss << "file_metadata=(" << fileMetadata.String() << "))";
 
     return StatusWith<std::string>(ss.str());
@@ -1184,7 +1274,7 @@ std::string WiredTigerUtil::generateRestoreConfig() {
 }
 
 bool WiredTigerUtil::willRestoreFromBackup() {
-    if (!storageGlobalParams.restore) {
+    if (!(storageGlobalParams.restore || storageGlobalParams.magicRestore)) {
         return false;
     }
 
@@ -1262,8 +1352,10 @@ std::string WiredTigerUtil::generateWTVerboseConfiguration() {
         cfg << ",";
 
         int level;
-        // Deliberately skip WT_VERBOSE_DEBUG_1, as it's a bit too noisy.
         switch (severity.toInt()) {
+            case logv2::LogSeverity::Debug(1).toInt():
+                level = WT_VERBOSE_DEBUG_1;
+                break;
             case logv2::LogSeverity::Debug(2).toInt():
                 level = WT_VERBOSE_DEBUG_2;
                 break;
@@ -1314,7 +1406,6 @@ BSONObj WiredTigerUtil::setConfigStringToStorageOptions(const BSONObj& options,
 }
 
 void WiredTigerUtil::removeEncryptionFromConfigString(std::string* configString) {
-    static StaticImmortal<pcre::Regex> encryptionOptsRegex(R"re(encryption=\([^\)]*\),?)re");
     encryptionOptsRegex->substitute("", configString, pcre::SUBSTITUTE_GLOBAL);
 }
 
@@ -1329,4 +1420,26 @@ BSONObj WiredTigerUtil::getSanitizedStorageOptionsForSecondaryReplication(const
     return setConfigStringToStorageOptions(options, *configString);
 }
 
+Status WiredTigerUtil::canRunAutoCompact(OperationContext* opCtx, bool isEphemeral) {
+    if (!gFeatureFlagAutoCompact.isEnabled(
+            serverGlobalParams.featureCompatibility.acquireFCVSnapshot())) {
+        return Status(ErrorCodes::IllegalOperation,
+                      "autoCompact() requires its feature flag to be enabled");
+    }
+    if (isEphemeral) {
+        return Status(ErrorCodes::IllegalOperation,
+                      "autoCompact() cannot be executed for in-memory configurations");
+    }
+    if (!opCtx->getServiceContext()->userWritesAllowed()) {
+        return Status(ErrorCodes::IllegalOperation,
+                      "autoCompact() can only be executed when writes are allowed");
+    }
+    if (storageGlobalParams.syncdelay == 0) {
+        return Status(ErrorCodes::IllegalOperation,
+                      "autoCompact() can only be executed when checkpoints are enabled");
+    }
+    return Status::OK();
+}
+
+
 }  // namespace mongo
