@@ -27,31 +27,40 @@
  *    it in the license file.
  */
 
-
-#include "mongo/platform/basic.h"
-
+#include <boost/move/utility_core.hpp>
+#include <memory>
 #include <sstream>
 #include <string>
+#include <wiredtiger.h>
 
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/checked_cast.h"
+#include "mongo/base/error_codes.h"
 #include "mongo/base/string_data.h"
-#include "mongo/db/operation_context_noop.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/db/service_context.h"
 #include "mongo/db/service_context_test_fixture.h"
+#include "mongo/db/storage/recovery_unit.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_oplog_manager.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_recovery_unit.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_session_cache.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_util.h"
+#include "mongo/db/storage/write_unit_of_work.h"
+#include "mongo/db/transaction_resources.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/log_severity.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
 #include "mongo/unittest/log_test.h"
 #include "mongo/unittest/temp_dir.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/util/clock_source.h"
 #include "mongo/util/system_clock_source.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
 
-
 namespace mongo {
-
-using std::string;
-using std::stringstream;
+namespace {
 
 class WiredTigerConnection {
 public:
@@ -62,7 +71,7 @@ public:
         std::stringstream ss;
         ss << "create,";
         ss << extraStrings;
-        string config = ss.str();
+        std::string config = ss.str();
         _fastClockSource = std::make_unique<SystemClockSource>();
         int ret = wiredtiger_open(dbpath.toString().c_str(), eventHandler, config.c_str(), &_conn);
         ASSERT_OK(wtRCToStatus(ret, nullptr));
@@ -87,13 +96,11 @@ class WiredTigerUtilHarnessHelper {
 public:
     explicit WiredTigerUtilHarnessHelper(StringData extraStrings,
                                          WiredTigerEventHandler* eventHandler = nullptr)
-        : _dbpath("wt_test"),
-          _connection(_dbpath.path(),
+        : _connection(_dbpath.path(),
                       extraStrings,
                       eventHandler == nullptr ? nullptr : eventHandler->getWtEventHandler()),
           _sessionCache(_connection.getConnection(), _connection.getClockSource()) {}
 
-
     WiredTigerSessionCache* getSessionCache() {
         return &_sessionCache;
     }
@@ -102,54 +109,45 @@ public:
         return &_oplogManager;
     }
 
-    OperationContext* newOperationContext() {
-        return new OperationContextNoop(
-            new WiredTigerRecoveryUnit(getSessionCache(), &_oplogManager));
-    }
-
 private:
-    unittest::TempDir _dbpath;
+    unittest::TempDir _dbpath{"wt_test"};
     WiredTigerConnection _connection;
     WiredTigerSessionCache _sessionCache;
     WiredTigerOplogManager _oplogManager;
 };
 
-class WiredTigerUtilMetadataTest : public mongo::ServiceContextTest {
-public:
-    virtual void setUp() {
-        _harnessHelper.reset(new WiredTigerUtilHarnessHelper(""));
-        _opCtx.reset(_harnessHelper->newOperationContext());
-    }
-
-    virtual void tearDown() {
-        _opCtx.reset(nullptr);
-        _harnessHelper.reset(nullptr);
-    }
-
+class WiredTigerUtilMetadataTest : public ServiceContextTest {
 protected:
+    WiredTigerUtilMetadataTest() : _harnessHelper(""), _opCtxHolder(makeOperationContext()) {
+        shard_role_details::setRecoveryUnit(
+            _opCtxHolder.get(),
+            std::make_unique<WiredTigerRecoveryUnit>(_harnessHelper.getSessionCache(),
+                                                     _harnessHelper.getOplogManager()),
+            WriteUnitOfWork::RecoveryUnitState::kNotInUnitOfWork);
+    }
+
     const char* getURI() const {
         return "table:mytable";
     }
 
     OperationContext* getOperationContext() const {
-        ASSERT(_opCtx.get());
-        return _opCtx.get();
+        return _opCtxHolder.get();
     }
 
     void createSession(const char* config) {
         WT_SESSION* wtSession =
-            WiredTigerRecoveryUnit::get(_opCtx.get())->getSession()->getSession();
+            WiredTigerRecoveryUnit::get(getOperationContext())->getSession()->getSession();
         ASSERT_OK(wtRCToStatus(wtSession->create(wtSession, getURI(), config), wtSession));
     }
 
 private:
-    std::unique_ptr<WiredTigerUtilHarnessHelper> _harnessHelper;
-    std::unique_ptr<OperationContext> _opCtx;
+    WiredTigerUtilHarnessHelper _harnessHelper;
+    ServiceContext::UniqueOperationContext _opCtxHolder;
 };
 
 TEST_F(WiredTigerUtilMetadataTest, GetMetadataCreateInvalid) {
-    StatusWith<std::string> result =
-        WiredTigerUtil::getMetadataCreate(getOperationContext(), getURI());
+    StatusWith<std::string> result = WiredTigerUtil::getMetadataCreate(
+        *WiredTigerRecoveryUnit::get(getOperationContext()), getURI());
     ASSERT_NOT_OK(result.getStatus());
     ASSERT_EQUALS(ErrorCodes::NoSuchKey, result.getStatus().code());
 }
@@ -157,8 +155,8 @@ TEST_F(WiredTigerUtilMetadataTest, GetMetadataCreateInvalid) {
 TEST_F(WiredTigerUtilMetadataTest, GetMetadataCreateNull) {
     const char* config = nullptr;
     createSession(config);
-    StatusWith<std::string> result =
-        WiredTigerUtil::getMetadataCreate(getOperationContext(), getURI());
+    StatusWith<std::string> result = WiredTigerUtil::getMetadataCreate(
+        *WiredTigerRecoveryUnit::get(getOperationContext()), getURI());
     ASSERT_OK(result.getStatus());
     ASSERT_FALSE(result.getValue().empty());
 }
@@ -166,14 +164,15 @@ TEST_F(WiredTigerUtilMetadataTest, GetMetadataCreateNull) {
 TEST_F(WiredTigerUtilMetadataTest, GetMetadataCreateStringSimple) {
     const char* config = "app_metadata=(abc=123)";
     createSession(config);
-    StatusWith<std::string> result =
-        WiredTigerUtil::getMetadataCreate(getOperationContext(), getURI());
+    StatusWith<std::string> result = WiredTigerUtil::getMetadataCreate(
+        *WiredTigerRecoveryUnit::get(getOperationContext()), getURI());
     ASSERT_OK(result.getStatus());
     ASSERT_STRING_CONTAINS(result.getValue(), config);
 }
 
 TEST_F(WiredTigerUtilMetadataTest, GetConfigurationStringInvalidURI) {
-    StatusWith<std::string> result = WiredTigerUtil::getMetadata(getOperationContext(), getURI());
+    StatusWith<std::string> result =
+        WiredTigerUtil::getMetadata(*WiredTigerRecoveryUnit::get(getOperationContext()), getURI());
     ASSERT_NOT_OK(result.getStatus());
     ASSERT_EQUALS(ErrorCodes::NoSuchKey, result.getStatus().code());
 }
@@ -181,7 +180,8 @@ TEST_F(WiredTigerUtilMetadataTest, GetConfigurationStringInvalidURI) {
 TEST_F(WiredTigerUtilMetadataTest, GetConfigurationStringNull) {
     const char* config = nullptr;
     createSession(config);
-    StatusWith<std::string> result = WiredTigerUtil::getMetadata(getOperationContext(), getURI());
+    StatusWith<std::string> result =
+        WiredTigerUtil::getMetadata(*WiredTigerRecoveryUnit::get(getOperationContext()), getURI());
     ASSERT_OK(result.getStatus());
     ASSERT_FALSE(result.getValue().empty());
 }
@@ -189,14 +189,15 @@ TEST_F(WiredTigerUtilMetadataTest, GetConfigurationStringNull) {
 TEST_F(WiredTigerUtilMetadataTest, GetConfigurationStringSimple) {
     const char* config = "app_metadata=(abc=123)";
     createSession(config);
-    StatusWith<std::string> result = WiredTigerUtil::getMetadata(getOperationContext(), getURI());
+    StatusWith<std::string> result =
+        WiredTigerUtil::getMetadata(*WiredTigerRecoveryUnit::get(getOperationContext()), getURI());
     ASSERT_OK(result.getStatus());
     ASSERT_STRING_CONTAINS(result.getValue(), config);
 }
 
 TEST_F(WiredTigerUtilMetadataTest, GetApplicationMetadataInvalidURI) {
-    StatusWith<BSONObj> result =
-        WiredTigerUtil::getApplicationMetadata(getOperationContext(), getURI());
+    StatusWith<BSONObj> result = WiredTigerUtil::getApplicationMetadata(
+        *WiredTigerRecoveryUnit::get(getOperationContext()), getURI());
     ASSERT_NOT_OK(result.getStatus());
     ASSERT_EQUALS(ErrorCodes::NoSuchKey, result.getStatus().code());
 }
@@ -204,8 +205,8 @@ TEST_F(WiredTigerUtilMetadataTest, GetApplicationMetadataInvalidURI) {
 TEST_F(WiredTigerUtilMetadataTest, GetApplicationMetadataNull) {
     const char* config = nullptr;
     createSession(config);
-    StatusWith<BSONObj> result =
-        WiredTigerUtil::getApplicationMetadata(getOperationContext(), getURI());
+    StatusWith<BSONObj> result = WiredTigerUtil::getApplicationMetadata(
+        *WiredTigerRecoveryUnit::get(getOperationContext()), getURI());
     ASSERT_OK(result.getStatus());
     ASSERT_TRUE(result.getValue().isEmpty());
 }
@@ -213,8 +214,8 @@ TEST_F(WiredTigerUtilMetadataTest, GetApplicationMetadataNull) {
 TEST_F(WiredTigerUtilMetadataTest, GetApplicationMetadataString) {
     const char* config = "app_metadata=\"abc\"";
     createSession(config);
-    StatusWith<BSONObj> result =
-        WiredTigerUtil::getApplicationMetadata(getOperationContext(), getURI());
+    StatusWith<BSONObj> result = WiredTigerUtil::getApplicationMetadata(
+        *WiredTigerRecoveryUnit::get(getOperationContext()), getURI());
     ASSERT_NOT_OK(result.getStatus());
     ASSERT_EQUALS(ErrorCodes::FailedToParse, result.getStatus().code());
 }
@@ -222,8 +223,8 @@ TEST_F(WiredTigerUtilMetadataTest, GetApplicationMetadataString) {
 TEST_F(WiredTigerUtilMetadataTest, GetApplicationMetadataDuplicateKeys) {
     const char* config = "app_metadata=(abc=123,abc=456)";
     createSession(config);
-    StatusWith<BSONObj> result =
-        WiredTigerUtil::getApplicationMetadata(getOperationContext(), getURI());
+    StatusWith<BSONObj> result = WiredTigerUtil::getApplicationMetadata(
+        *WiredTigerRecoveryUnit::get(getOperationContext()), getURI());
     ASSERT_NOT_OK(result.getStatus());
     ASSERT_EQUALS(50998, result.getStatus().code());
 }
@@ -234,8 +235,8 @@ TEST_F(WiredTigerUtilMetadataTest, GetApplicationMetadataTypes) {
         "idkey=def,numkey=123,"
         "structkey=(k1=v2,k2=v2))";
     createSession(config);
-    StatusWith<BSONObj> result =
-        WiredTigerUtil::getApplicationMetadata(getOperationContext(), getURI());
+    StatusWith<BSONObj> result = WiredTigerUtil::getApplicationMetadata(
+        *WiredTigerRecoveryUnit::get(getOperationContext()), getURI());
     ASSERT_OK(result.getStatus());
     const BSONObj& obj = result.getValue();
 
@@ -267,56 +268,51 @@ TEST_F(WiredTigerUtilMetadataTest, GetApplicationMetadataTypes) {
 TEST_F(WiredTigerUtilMetadataTest, CheckApplicationMetadataFormatVersionMissingKey) {
     createSession("app_metadata=(abc=123)");
     ASSERT_OK(WiredTigerUtil::checkApplicationMetadataFormatVersion(
-        getOperationContext(), getURI(), 1, 1));
+        *WiredTigerRecoveryUnit::get(getOperationContext()), getURI(), 1, 1));
     ASSERT_NOT_OK(WiredTigerUtil::checkApplicationMetadataFormatVersion(
-        getOperationContext(), getURI(), 2, 2));
+        *WiredTigerRecoveryUnit::get(getOperationContext()), getURI(), 2, 2));
 }
 
 TEST_F(WiredTigerUtilMetadataTest, CheckApplicationMetadataFormatVersionString) {
     createSession("app_metadata=(formatVersion=\"bar\")");
     ASSERT_NOT_OK(WiredTigerUtil::checkApplicationMetadataFormatVersion(
-        getOperationContext(), getURI(), 1, 1));
+        *WiredTigerRecoveryUnit::get(getOperationContext()), getURI(), 1, 1));
 }
 
 TEST_F(WiredTigerUtilMetadataTest, CheckApplicationMetadataFormatVersionNumber) {
     createSession("app_metadata=(formatVersion=2)");
-    ASSERT_EQUALS(
-        WiredTigerUtil::checkApplicationMetadataFormatVersion(getOperationContext(), getURI(), 2, 3)
-            .getValue(),
-        2);
+    ASSERT_EQUALS(WiredTigerUtil::checkApplicationMetadataFormatVersion(
+                      *WiredTigerRecoveryUnit::get(getOperationContext()), getURI(), 2, 3)
+                      .getValue(),
+                  2);
     ASSERT_NOT_OK(WiredTigerUtil::checkApplicationMetadataFormatVersion(
-        getOperationContext(), getURI(), 1, 1));
+        *WiredTigerRecoveryUnit::get(getOperationContext()), getURI(), 1, 1));
     ASSERT_NOT_OK(WiredTigerUtil::checkApplicationMetadataFormatVersion(
-        getOperationContext(), getURI(), 3, 3));
+        *WiredTigerRecoveryUnit::get(getOperationContext()), getURI(), 3, 3));
 }
 
 TEST_F(WiredTigerUtilMetadataTest, CheckApplicationMetadataFormatInvalidURI) {
     createSession("\"");
-    Status result =
-        WiredTigerUtil::checkApplicationMetadataFormatVersion(getOperationContext(), getURI(), 0, 3)
-            .getStatus();
+    Status result = WiredTigerUtil::checkApplicationMetadataFormatVersion(
+                        *WiredTigerRecoveryUnit::get(getOperationContext()), getURI(), 0, 3)
+                        .getStatus();
     ASSERT_NOT_OK(result);
     ASSERT_EQUALS(ErrorCodes::FailedToParse, result.code());
 }
 
-class WiredTigerUtilTest : public mongo::ServiceContextTest {
-public:
-    void setUp() {
-        ServiceContextTest::setUp();
-    }
-
-    void tearDown() {
-        ServiceContextTest::tearDown();
-    }
-};
+class WiredTigerUtilTest : public ServiceContextTest {};
 
 TEST_F(WiredTigerUtilTest, GetStatisticsValueMissingTable) {
     WiredTigerUtilHarnessHelper harnessHelper("statistics=(all)");
-    WiredTigerRecoveryUnit recoveryUnit(harnessHelper.getSessionCache(),
-                                        harnessHelper.getOplogManager());
-    std::unique_ptr<OperationContext> opCtx{harnessHelper.newOperationContext()};
-    recoveryUnit.setOperationContext(opCtx.get());
-    WiredTigerSession* session = recoveryUnit.getSession();
+    auto opCtx{makeOperationContext()};
+    shard_role_details::setRecoveryUnit(
+        opCtx.get(),
+        std::make_unique<WiredTigerRecoveryUnit>(harnessHelper.getSessionCache(),
+                                                 harnessHelper.getOplogManager()),
+        WriteUnitOfWork::RecoveryUnitState::kNotInUnitOfWork);
+    WiredTigerSession* session =
+        checked_cast<WiredTigerRecoveryUnit*>(shard_role_details::getRecoveryUnit(opCtx.get()))
+            ->getSession();
     auto result = WiredTigerUtil::getStatisticsValue(session->getSession(),
                                                      "statistics:table:no_such_table",
                                                      "statistics=(fast)",
@@ -327,11 +323,15 @@ TEST_F(WiredTigerUtilTest, GetStatisticsValueMissingTable) {
 
 TEST_F(WiredTigerUtilTest, GetStatisticsValueStatisticsDisabled) {
     WiredTigerUtilHarnessHelper harnessHelper("statistics=(none)");
-    WiredTigerRecoveryUnit recoveryUnit(harnessHelper.getSessionCache(),
-                                        harnessHelper.getOplogManager());
-    std::unique_ptr<OperationContext> opCtx{harnessHelper.newOperationContext()};
-    recoveryUnit.setOperationContext(opCtx.get());
-    WiredTigerSession* session = recoveryUnit.getSession();
+    auto opCtx{makeOperationContext()};
+    shard_role_details::setRecoveryUnit(
+        opCtx.get(),
+        std::make_unique<WiredTigerRecoveryUnit>(harnessHelper.getSessionCache(),
+                                                 harnessHelper.getOplogManager()),
+        WriteUnitOfWork::RecoveryUnitState::kNotInUnitOfWork);
+    WiredTigerSession* session =
+        checked_cast<WiredTigerRecoveryUnit*>(shard_role_details::getRecoveryUnit(opCtx.get()))
+            ->getSession();
     WT_SESSION* wtSession = session->getSession();
     ASSERT_OK(wtRCToStatus(wtSession->create(wtSession, "table:mytable", nullptr), wtSession));
     auto result = WiredTigerUtil::getStatisticsValue(session->getSession(),
@@ -344,11 +344,15 @@ TEST_F(WiredTigerUtilTest, GetStatisticsValueStatisticsDisabled) {
 
 TEST_F(WiredTigerUtilTest, GetStatisticsValueInvalidKey) {
     WiredTigerUtilHarnessHelper harnessHelper("statistics=(all)");
-    WiredTigerRecoveryUnit recoveryUnit(harnessHelper.getSessionCache(),
-                                        harnessHelper.getOplogManager());
-    std::unique_ptr<OperationContext> opCtx{harnessHelper.newOperationContext()};
-    recoveryUnit.setOperationContext(opCtx.get());
-    WiredTigerSession* session = recoveryUnit.getSession();
+    auto opCtx{makeOperationContext()};
+    shard_role_details::setRecoveryUnit(
+        opCtx.get(),
+        std::make_unique<WiredTigerRecoveryUnit>(harnessHelper.getSessionCache(),
+                                                 harnessHelper.getOplogManager()),
+        WriteUnitOfWork::RecoveryUnitState::kNotInUnitOfWork);
+    WiredTigerSession* session =
+        checked_cast<WiredTigerRecoveryUnit*>(shard_role_details::getRecoveryUnit(opCtx.get()))
+            ->getSession();
     WT_SESSION* wtSession = session->getSession();
     ASSERT_OK(wtRCToStatus(wtSession->create(wtSession, "table:mytable", nullptr), wtSession));
     // Use connection statistics key which does not apply to a table.
@@ -362,11 +366,15 @@ TEST_F(WiredTigerUtilTest, GetStatisticsValueInvalidKey) {
 
 TEST_F(WiredTigerUtilTest, GetStatisticsValueValidKey) {
     WiredTigerUtilHarnessHelper harnessHelper("statistics=(all)");
-    WiredTigerRecoveryUnit recoveryUnit(harnessHelper.getSessionCache(),
-                                        harnessHelper.getOplogManager());
-    std::unique_ptr<OperationContext> opCtx{harnessHelper.newOperationContext()};
-    recoveryUnit.setOperationContext(opCtx.get());
-    WiredTigerSession* session = recoveryUnit.getSession();
+    auto opCtx{makeOperationContext()};
+    shard_role_details::setRecoveryUnit(
+        opCtx.get(),
+        std::make_unique<WiredTigerRecoveryUnit>(harnessHelper.getSessionCache(),
+                                                 harnessHelper.getOplogManager()),
+        WriteUnitOfWork::RecoveryUnitState::kNotInUnitOfWork);
+    WiredTigerSession* session =
+        checked_cast<WiredTigerRecoveryUnit*>(shard_role_details::getRecoveryUnit(opCtx.get()))
+            ->getSession();
     WT_SESSION* wtSession = session->getSession();
     ASSERT_OK(wtRCToStatus(wtSession->create(wtSession, "table:mytable", nullptr), wtSession));
     // Use connection statistics key which does not apply to a table.
@@ -391,11 +399,16 @@ TEST_F(WiredTigerUtilTest, ParseAPIMessages) {
     WiredTigerUtilHarnessHelper harnessHelper(connection_cfg.c_str(), &eventHandler);
 
     // Create a session.
-    WiredTigerRecoveryUnit recoveryUnit(harnessHelper.getSessionCache(),
-                                        harnessHelper.getOplogManager());
-    std::unique_ptr<OperationContext> opCtx{harnessHelper.newOperationContext()};
-    recoveryUnit.setOperationContext(opCtx.get());
-    WT_SESSION* wtSession = recoveryUnit.getSession()->getSession();
+    auto opCtx{makeOperationContext()};
+    shard_role_details::setRecoveryUnit(
+        opCtx.get(),
+        std::make_unique<WiredTigerRecoveryUnit>(harnessHelper.getSessionCache(),
+                                                 harnessHelper.getOplogManager()),
+        WriteUnitOfWork::RecoveryUnitState::kNotInUnitOfWork);
+    WiredTigerSession* session =
+        checked_cast<WiredTigerRecoveryUnit*>(shard_role_details::getRecoveryUnit(opCtx.get()))
+            ->getSession();
+    WT_SESSION* wtSession = session->getSession();
 
     // Perform simple WiredTiger operations while capturing the generated logs.
     startCapturingLogMessages();
@@ -592,4 +605,90 @@ TEST_F(WiredTigerUtilTest, GetSanitizedStorageOptionsForSecondaryReplication) {
     }
 }
 
+TEST_F(WiredTigerUtilTest, SkipPreparedUpdateBounded) {
+    // Initialize WiredTiger.
+    WiredTigerEventHandler eventHandler;
+    WiredTigerUtilHarnessHelper harnessHelper("", &eventHandler);
+
+    WT_SESSION* session1;
+    ASSERT_OK(wtRCToStatus(
+        harnessHelper.getSessionCache()->conn()->open_session(
+            harnessHelper.getSessionCache()->conn(), nullptr, "isolation=snapshot", &session1),
+        nullptr));
+
+    WT_SESSION* session2;
+    ASSERT_OK(wtRCToStatus(
+        harnessHelper.getSessionCache()->conn()->open_session(
+            harnessHelper.getSessionCache()->conn(), nullptr, "isolation=snapshot", &session2),
+        nullptr));
+
+
+    const std::string uri = "table:test";
+    ASSERT_OK(wtRCToStatus(session1->create(session1, uri.c_str(), "key_format=S,value_format=S"),
+                           session1));
+    WT_CURSOR* cursor1;
+    ASSERT_EQ(0, session1->begin_transaction(session1, "ignore_prepare=false"));
+    ASSERT_EQ(0, session1->open_cursor(session1, uri.c_str(), nullptr, nullptr, &cursor1));
+    cursor1->set_key(cursor1, "abc");
+    cursor1->set_value(cursor1, "test");
+    ASSERT_EQ(0, cursor1->insert(cursor1));
+    session1->prepare_transaction(session1, "prepare_timestamp=1");
+
+    WT_CURSOR* cursor2;
+    ASSERT_EQ(0, session2->begin_transaction(session2, "ignore_prepare=false"));
+    ASSERT_EQ(0, session2->open_cursor(session2, uri.c_str(), nullptr, nullptr, &cursor2));
+
+    cursor2->set_key(cursor2, "abc");
+    cursor2->bound(cursor2, "bound=lower");
+
+    ASSERT_EQ(WT_PREPARE_CONFLICT, cursor2->next(cursor2));
+    ASSERT_EQ(WT_PREPARE_CONFLICT, cursor2->next(cursor2));
+
+    session1->commit_transaction(session1, "durable_timestamp=1,commit_timestamp=1");
+    ASSERT_EQ(0, cursor2->next(cursor2));
+}
+
+TEST_F(WiredTigerUtilTest, SkipPreparedUpdateNoBound) {
+    // Initialize WiredTiger.
+    WiredTigerEventHandler eventHandler;
+    WiredTigerUtilHarnessHelper harnessHelper("", &eventHandler);
+
+    WT_SESSION* session1;
+    ASSERT_OK(wtRCToStatus(
+        harnessHelper.getSessionCache()->conn()->open_session(
+            harnessHelper.getSessionCache()->conn(), nullptr, "isolation=snapshot", &session1),
+        nullptr));
+
+    WT_SESSION* session2;
+    ASSERT_OK(wtRCToStatus(
+        harnessHelper.getSessionCache()->conn()->open_session(
+            harnessHelper.getSessionCache()->conn(), nullptr, "isolation=snapshot", &session2),
+        nullptr));
+
+
+    const std::string uri = "table:test";
+    ASSERT_OK(wtRCToStatus(session1->create(session1, uri.c_str(), "key_format=S,value_format=S"),
+                           session1));
+    WT_CURSOR* cursor1;
+    ASSERT_EQ(0, session1->begin_transaction(session1, "ignore_prepare=false"));
+    ASSERT_EQ(0, session1->open_cursor(session1, uri.c_str(), nullptr, nullptr, &cursor1));
+    cursor1->set_key(cursor1, "abc");
+    cursor1->set_value(cursor1, "test");
+    ASSERT_EQ(0, cursor1->insert(cursor1));
+    session1->prepare_transaction(session1, "prepare_timestamp=1");
+
+    WT_CURSOR* cursor2;
+    ASSERT_EQ(0, session2->begin_transaction(session2, "ignore_prepare=false"));
+    ASSERT_EQ(0, session2->open_cursor(session2, uri.c_str(), nullptr, nullptr, &cursor2));
+
+    // Continuously return WT_PREPARE_CONFLICT
+    ASSERT_EQ(WT_PREPARE_CONFLICT, cursor2->next(cursor2));
+    ASSERT_EQ(WT_PREPARE_CONFLICT, cursor2->next(cursor2));
+    ASSERT_EQ(WT_PREPARE_CONFLICT, cursor2->next(cursor2));
+
+    session1->commit_transaction(session1, "durable_timestamp=1,commit_timestamp=1");
+    ASSERT_EQ(0, cursor2->next(cursor2));
+}
+
+}  // namespace
 }  // namespace mongo
