@@ -28,7 +28,20 @@
  */
 
 #include "mongo/db/timeseries/bucket_catalog/bucket.h"
-#include "mongo/db/timeseries/timeseries_constants.h"
+
+#include <absl/container/node_hash_set.h>
+#include <absl/meta/type_traits.h>
+#include <boost/move/utility_core.hpp>
+#include <boost/optional.hpp>
+#include <utility>
+
+#include <boost/optional/optional.hpp>
+
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/db/storage/storage_parameters_gen.h"
+#include "mongo/util/assert_util_core.h"
 
 namespace mongo::timeseries::bucket_catalog {
 
@@ -43,14 +56,32 @@ uint8_t numDigits(uint32_t num) {
 }
 }  // namespace
 
-Bucket::Bucket(
-    const BucketId& bId, const BucketKey& k, StringData tf, Date_t mt, BucketStateRegistry& bsr)
-    : bucketId(bId),
-      key(k),
-      timeField(tf.toString()),
+Bucket::Bucket(TrackingContexts& trackingContexts,
+               const BucketId& bId,
+               BucketKey k,
+               StringData tf,
+               Date_t mt,
+               BucketStateRegistry& bsr)
+    : usingAlwaysCompressedBuckets(feature_flags::gTimeseriesAlwaysUseCompressedBuckets.isEnabled(
+          serverGlobalParams.featureCompatibility.acquireFCVSnapshot())),
       minTime(mt),
+      lastChecked(getCurrentEraAndIncrementBucketCount(bsr)),
       bucketStateRegistry(bsr),
-      lastChecked(getCurrentEraAndIncrementBucketCount(bucketStateRegistry)) {}
+      bucketId(bId),
+      timeField(
+          make_tracked_string(getTrackingContext(trackingContexts, TrackingScope::kOpenBucketsById),
+                              tf.data(),
+                              tf.size())),
+      key(std::move(k)),
+      fieldNames(makeTrackedStringSet(
+          getTrackingContext(trackingContexts, TrackingScope::kOpenBucketsById))),
+      uncommittedFieldNames(makeTrackedStringSet(
+          getTrackingContext(trackingContexts, TrackingScope::kOpenBucketsById))),
+      batches(make_tracked_unordered_map<OperationId, std::shared_ptr<WriteBatch>>(
+          getTrackingContext(trackingContexts, TrackingScope::kOpenBucketsById))),
+      minmax(getTrackingContext(trackingContexts, TrackingScope::kSummaries)),
+      schema(getTrackingContext(trackingContexts, TrackingScope::kSummaries)),
+      measurementMap(getTrackingContext(trackingContexts, TrackingScope::kColumnBuilders)) {}
 
 Bucket::~Bucket() {
     decrementBucketCountForEra(bucketStateRegistry, lastChecked);
@@ -63,16 +94,17 @@ bool allCommitted(const Bucket& bucket) {
 bool schemaIncompatible(Bucket& bucket,
                         const BSONObj& input,
                         boost::optional<StringData> metaField,
-                        const StringData::ComparatorInterface* comparator) {
+                        const StringDataComparator* comparator) {
     auto result = bucket.schema.update(input, metaField, comparator);
     return (result == Schema::UpdateStatus::Failed);
 }
 
-void calculateBucketFieldsAndSizeChange(const Bucket& bucket,
+void calculateBucketFieldsAndSizeChange(TrackingContexts& trackingContexts,
+                                        const Bucket& bucket,
                                         const BSONObj& doc,
                                         boost::optional<StringData> metaField,
                                         Bucket::NewFieldNames& newFieldNamesToBeInserted,
-                                        int32_t& sizeToBeAdded) {
+                                        Sizes& sizesToBeAdded) {
     // BSON size for an object with an empty object field where field name is empty string.
     // We can use this as an offset to know the size when we have real field names.
     static constexpr int emptyObjSize = 12;
@@ -80,55 +112,72 @@ void calculateBucketFieldsAndSizeChange(const Bucket& bucket,
     dassert(emptyObjSize == BSON("" << BSONObj()).objsize());
 
     newFieldNamesToBeInserted.clear();
-    sizeToBeAdded = 0;
+    sizesToBeAdded.uncommittedVerifiedSize = 0;
+    sizesToBeAdded.uncommittedMeasurementEstimate = 0;
+
     auto numMeasurementsFieldLength = numDigits(bucket.numMeasurements);
+
+    // When a measurement larger than the threshold (in bytes) is being inserted into a bucket, we
+    // use the measurements uncompressed size towards the bucket size limit.
+    const int32_t largeMeasurementThreshold = gTimeseriesLargeMeasurementThreshold.load();
+
     for (const auto& elem : doc) {
         auto fieldName = elem.fieldNameStringData();
         if (fieldName == metaField) {
-            // Only account for the meta field size once, on bucket insert, since it is stored
-            // uncompressed at the top-level of the bucket.
-            if (bucket.size == 0) {
-                sizeToBeAdded += kBucketMetaFieldName.size() + elem.size() - elem.fieldNameSize();
-            }
+            // Ignore the metadata field since it will not be inserted.
             continue;
         }
 
-        auto hashedKey = StringSet::hasher().hashed_key(fieldName);
+        auto hashedKey = TrackedStringSet::hasher().hashed_key(
+            getTrackingContext(trackingContexts, TrackingScope::kOpenBucketsById), fieldName);
         if (!bucket.fieldNames.contains(hashedKey)) {
             // Record the new field name only if it hasn't been committed yet. There could
             // be concurrent batches writing to this bucket with the same new field name,
             // but they're not guaranteed to commit successfully.
-            newFieldNamesToBeInserted.push_back(hashedKey);
+            newFieldNamesToBeInserted.push_back(
+                StringMapHashedKey{hashedKey.key(), hashedKey.hash()});
 
             // Only update the bucket size once to account for the new field name if it
             // isn't already pending a commit from another batch.
             if (!bucket.uncommittedFieldNames.contains(hashedKey)) {
                 // Add the size of an empty object with that field name.
-                sizeToBeAdded += emptyObjSize + fieldName.size();
+                sizesToBeAdded.uncommittedVerifiedSize += emptyObjSize + fieldName.size();
 
                 // The control.min and control.max summaries don't have any information for
                 // this new field name yet. Add two measurements worth of data to account
                 // for this. As this is the first measurement for this field, min == max.
-                sizeToBeAdded += elem.size() * 2;
+                sizesToBeAdded.uncommittedVerifiedSize += elem.size() * 2;
             }
         }
 
-        // Add the element size, taking into account that the name will be changed to its
-        // positional number. Add 1 to the calculation since the element's field name size
-        // accounts for a null terminator whereas the stringified position does not.
-        sizeToBeAdded += elem.size() - elem.fieldNameSize() + numMeasurementsFieldLength + 1;
+        // The element size, taking into account that the name will be changed to its positional
+        // number. Add 1 to the calculation since the element's field name size accounts for a null
+        // terminator whereas the stringified position does not.
+        const int32_t elementSize =
+            elem.size() - elem.fieldNameSize() + numMeasurementsFieldLength + 1;
+
+        if (!bucket.usingAlwaysCompressedBuckets || elementSize > largeMeasurementThreshold) {
+            sizesToBeAdded.uncommittedMeasurementEstimate += elementSize;
+        }
     }
 }
 
-std::shared_ptr<WriteBatch> activeBatch(Bucket& bucket,
+std::shared_ptr<WriteBatch> activeBatch(TrackingContexts& trackingContexts,
+                                        Bucket& bucket,
                                         OperationId opId,
                                         std::uint8_t stripe,
                                         ExecutionStatsController& stats) {
     auto it = bucket.batches.find(opId);
     if (it == bucket.batches.end()) {
         it = bucket.batches
-                 .try_emplace(
-                     opId, std::make_shared<WriteBatch>(bucket.bucketId, bucket.key, opId, stats))
+                 .try_emplace(opId,
+                              std::make_shared<WriteBatch>(
+                                  trackingContexts,
+                                  bucket.bucketId,
+                                  bucket.key,
+                                  opId,
+                                  stats,
+                                  StringData{bucket.timeField.data(), bucket.timeField.size()}))
                  .first;
     }
     return it->second;
