@@ -29,9 +29,21 @@
 
 #include "mongo/db/timeseries/bucket_catalog/flat_bson.h"
 
+#include <absl/container/flat_hash_map.h>
+#include <absl/meta/type_traits.h>
+#include <boost/none.hpp>
+#include <boost/optional.hpp>
+#include <cstring>
+
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
 #include "mongo/bson/bsonobj.h"
 #include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/db/update/document_diff_serialization.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/decimal_counter.h"
+#include "mongo/util/str.h"
 
 namespace mongo::timeseries::bucket_catalog {
 namespace {
@@ -44,6 +56,10 @@ int typeComp(const BSONElement& elem, BSONType type) {
 };
 }  // namespace
 
+template <class Element, class Value>
+FlatBSONStore<Element, Value>::Data::Data(TrackingContext& trackingContext)
+    : _value(trackingContext) {}
+
 template <class Element, class Value>
 typename FlatBSONStore<Element, Value>::Type FlatBSONStore<Element, Value>::Data::type() const {
     return _type;
@@ -95,6 +111,10 @@ void FlatBSONStore<Element, Value>::Data::setValue(const BSONElement& elem) {
     _updated = true;
 }
 
+template <class Element, class Value>
+FlatBSONStore<Element, Value>::Entry::Entry(TrackingContext& trackingContext)
+    : _element(trackingContext) {}
+
 template <class Element, class Value>
 FlatBSONStore<Element, Value>::Iterator::Iterator(
     typename FlatBSONStore<Element, Value>::Entries::iterator pos)
@@ -168,9 +188,10 @@ bool FlatBSONStore<Element, Value>::ConstIterator::operator!=(
 
 template <class Element, class Value>
 FlatBSONStore<Element, Value>::Obj::Obj(
+    TrackingContext& trackingContext,
     FlatBSONStore<Element, Value>::Entries& entries,
     typename FlatBSONStore<Element, Value>::Entries::iterator pos)
-    : _entries(entries), _pos(pos) {}
+    : _entries(entries), _pos(pos), _trackingContext(trackingContext) {}
 
 
 template <class Element, class Value>
@@ -185,12 +206,12 @@ typename FlatBSONStore<Element, Value>::Obj& FlatBSONStore<Element, Value>::Obj:
 template <class Element, class Value>
 typename FlatBSONStore<Element, Value>::Obj FlatBSONStore<Element, Value>::Obj::object(
     FlatBSONStore<Element, Value>::Iterator pos) const {
-    return {_entries, pos._pos};
+    return {_trackingContext, _entries, pos._pos};
 }
 
 template <class Element, class Value>
 typename FlatBSONStore<Element, Value>::Obj FlatBSONStore<Element, Value>::Obj::parent() const {
-    return {_entries, _pos - _pos->_offsetParent};
+    return {_trackingContext, _entries, _pos - _pos->_offsetParent};
 }
 
 template <class Element, class Value>
@@ -246,11 +267,13 @@ typename FlatBSONStore<Element, Value>::Iterator FlatBSONStore<Element, Value>::
 
     // We've exhausted the linear search limit, create a map to speedup future searches. Populate it
     // with all current subelements.
-    _pos->_fieldNameToIndex = std::make_unique<StringMap<uint32_t>>();
+    _pos->_fieldNameToIndex = makeTrackedStringMap<uint32_t>(_trackingContext);
     auto it = begin();
     auto itEnd = end();
     for (; it != itEnd; ++it) {
-        (*_pos->_fieldNameToIndex)[it->fieldName().toString()] = it._pos->_offsetParent;
+        _pos->_fieldNameToIndex->try_emplace(
+            make_tracked_string(_trackingContext, it->fieldName().data(), it->fieldName().size()),
+            it._pos->_offsetParent);
     }
 
     // Retry the search now when the map is created.
@@ -270,7 +293,7 @@ FlatBSONStore<Element, Value>::Obj::insert(FlatBSONStore<Element, Value>::Iterat
                                            std::string fieldName) {
     // Remember our iterator position so we can restore it after inserting a new element.
     auto index = std::distance(_entries.begin(), _pos);
-    auto inserted = _entries.emplace(pos._pos);
+    auto inserted = _entries.emplace(pos._pos, _trackingContext);
     _pos = _entries.begin() + index;
 
     // Setup our newly created entry.
@@ -280,7 +303,11 @@ FlatBSONStore<Element, Value>::Obj::insert(FlatBSONStore<Element, Value>::Iterat
 
     // Also store our offset in the fast lookup map if it is available.
     if (_pos->_fieldNameToIndex) {
-        _pos->_fieldNameToIndex->emplace(inserted->_element.fieldName(), inserted->_offsetParent);
+        _pos->_fieldNameToIndex->try_emplace(
+            make_tracked_string(_trackingContext,
+                                inserted->_element.fieldName().data(),
+                                inserted->_element.fieldName().size()),
+            inserted->_offsetParent);
     }
 
     // We need to traverse the hiearchy up to the root and modify stored offsets to account for
@@ -332,13 +359,18 @@ typename FlatBSONStore<Element, Value>::ConstIterator FlatBSONStore<Element, Val
 }
 
 template <class Element, class Value>
-FlatBSONStore<Element, Value>::FlatBSONStore() {
-    auto& entry = entries.emplace_back();
+FlatBSONStore<Element, Value>::FlatBSONStore(TrackingContext& trackingContext)
+    : entries(make_tracked_vector<Entry>(trackingContext)), _trackingContext(trackingContext) {
+    auto& entry = entries.emplace_back(trackingContext);
     entry._offsetEnd = 1;
     entry._offsetParent = 0;
     entry._element.initializeRoot();
 }
 
+template <class Derived, class Element, class Value>
+FlatBSON<Derived, Element, Value>::FlatBSON(TrackingContext& trackingContext)
+    : _store(trackingContext), _trackingContext(trackingContext) {}
+
 template <class Derived, class Element, class Value>
 typename std::string FlatBSON<Derived, Element, Value>::updateStatusString(
     UpdateStatus updateStatus) {
@@ -358,7 +390,7 @@ template <class Derived, class Element, class Value>
 typename FlatBSON<Derived, Element, Value>::UpdateStatus FlatBSON<Derived, Element, Value>::update(
     const BSONObj& doc,
     boost::optional<StringData> omitField,
-    const StringData::ComparatorInterface* stringComparator) {
+    const StringDataComparator* stringComparator) {
     auto obj = _store.root();
     return _updateObj(obj, doc, {}, stringComparator, [&omitField](StringData fieldName) {
         return omitField && fieldName == omitField;
@@ -369,11 +401,10 @@ template <class Derived, class Element, class Value>
 std::tuple<typename FlatBSON<Derived, Element, Value>::UpdateStatus,
            typename FlatBSONStore<Element, Value>::Iterator,
            typename FlatBSONStore<Element, Value>::Iterator>
-FlatBSON<Derived, Element, Value>::_update(
-    typename FlatBSONStore<Element, Value>::Obj obj,
-    BSONElement elem,
-    typename Element::UpdateContext updateValues,
-    const StringData::ComparatorInterface* stringComparator) {
+FlatBSON<Derived, Element, Value>::_update(typename FlatBSONStore<Element, Value>::Obj obj,
+                                           BSONElement elem,
+                                           typename Element::UpdateContext updateValues,
+                                           const StringDataComparator* stringComparator) {
     UpdateStatus status{UpdateStatus::Updated};
 
     if (elem.type() == Object) {
@@ -423,12 +454,11 @@ FlatBSON<Derived, Element, Value>::_update(
 
 template <class Derived, class Element, class Value>
 typename FlatBSON<Derived, Element, Value>::UpdateStatus
-FlatBSON<Derived, Element, Value>::_updateObj(
-    typename FlatBSONStore<Element, Value>::Obj& obj,
-    const BSONObj& doc,
-    typename Element::UpdateContext updateContext,
-    const StringData::ComparatorInterface* stringComparator,
-    std::function<bool(StringData)> skipFieldFn) {
+FlatBSON<Derived, Element, Value>::_updateObj(typename FlatBSONStore<Element, Value>::Obj& obj,
+                                              const BSONObj& doc,
+                                              typename Element::UpdateContext updateContext,
+                                              const StringDataComparator* stringComparator,
+                                              std::function<bool(StringData)> skipFieldFn) {
     auto it = obj.begin();
     auto end = obj.end();
     int allHandledOffset = 0;
@@ -687,26 +717,33 @@ void FlatBSON<Derived, Element, Value>::_setTypeArray(
     }
 }
 
+BSONElementValueBuffer::BSONElementValueBuffer(TrackingContext& trackingContext)
+    : _buffer(make_tracked_vector<char>(trackingContext)) {}
+
 BSONElement BSONElementValueBuffer::get() const {
-    return BSONElement(_buffer.get(), 1, _size);
+    return BSONElement(_buffer.data(), 1, BSONElement::TrustedInitTag{});
 }
 
 void BSONElementValueBuffer::set(const BSONElement& elem) {
-    auto requiredSize = elem.size() - elem.fieldNameSize() + 1;
-    if (_size < requiredSize) {
-        _buffer = std::make_unique<char[]>(requiredSize);
+    _size = elem.size() - elem.fieldNameSize() + 1;
+    if (_buffer.size() < _size) {
+        _buffer.resize(_size);
     }
+    auto buffer = _buffer.data();
     // Store element as BSONElement buffer but strip out the field name.
-    _buffer[0] = elem.type();
-    _buffer[1] = '\0';
-    memcpy(_buffer.get() + 2, elem.value(), elem.valuesize());
-    _size = requiredSize;
+    buffer[0] = elem.type();
+    buffer[1] = '\0';
+    memcpy(buffer + 2, elem.value(), elem.valuesize());
 }
 
 BSONType BSONElementValueBuffer::type() const {
     return (BSONType)_buffer[0];
 }
 
+size_t BSONElementValueBuffer::size() const {
+    return _size;
+}
+
 BSONElement BSONTypeValue::get() const {
     MONGO_UNREACHABLE;
 }
@@ -719,8 +756,15 @@ BSONType BSONTypeValue::type() const {
     return _type;
 }
 
+int64_t BSONTypeValue::size() const {
+    return 0;
+}
+
+Element::Element(TrackingContext& trackingContext)
+    : _fieldName(make_tracked_string(trackingContext)) {}
+
 StringData Element::fieldName() const {
-    return _fieldName;
+    return {_fieldName.data(), _fieldName.size()};
 }
 
 void Element::setFieldName(std::string&& fieldName) {
@@ -728,7 +772,7 @@ void Element::setFieldName(std::string&& fieldName) {
 }
 
 bool Element::isArrayFieldName() const {
-    return _fieldName == kArrayFieldName;
+    return fieldName() == kArrayFieldName;
 }
 
 void Element::claimArrayFieldNameForObject(std::string name) {
@@ -736,6 +780,9 @@ void Element::claimArrayFieldNameForObject(std::string name) {
     _fieldName = std::move(name);
 }
 
+MinMaxElement::MinMaxElement(TrackingContext& trackingContext)
+    : Element(trackingContext), _min(trackingContext), _max(trackingContext) {}
+
 void MinMaxElement::initializeRoot() {
     _min.setObject();
     _max.setObject();
@@ -757,6 +804,9 @@ const MinMaxStore::Data& MinMaxElement::max() const {
     return _max;
 }
 
+MinMax::MinMax(TrackingContext& trackingContext)
+    : FlatBSON<MinMax, MinMaxElement, BSONElementValueBuffer>(trackingContext) {}
+
 std::pair<MinMax::UpdateStatus, MinMaxElement::UpdateContext> MinMax::_shouldUpdateObj(
     MinMaxStore::Obj& obj, const BSONElement& elem, MinMaxElement::UpdateContext updateValues) {
 
@@ -808,11 +858,10 @@ std::pair<MinMax::UpdateStatus, MinMaxElement::UpdateContext> MinMax::_shouldUpd
                           MinMaxElement::UpdateContext{updateMin, updateMax});
 }
 
-MinMax::UpdateStatus MinMax::_maybeUpdateValue(
-    MinMaxStore::Obj& obj,
-    const BSONElement& elem,
-    MinMaxElement::UpdateContext updateValues,
-    const StringData::ComparatorInterface* stringComparator) {
+MinMax::UpdateStatus MinMax::_maybeUpdateValue(MinMaxStore::Obj& obj,
+                                               const BSONElement& elem,
+                                               MinMaxElement::UpdateContext updateValues,
+                                               const StringDataComparator* stringComparator) {
     auto maybeUpdateValue = [&](MinMaxStore::Data& data, auto comp) {
         if (data.type() == MinMaxStore::Type::kUnset ||
             (data.type() == MinMaxStore::Type::kObject && comp(typeComp(elem, Object), 0)) ||
@@ -858,10 +907,11 @@ BSONObj MinMax::maxUpdates() {
     return builder.obj();
 }
 
-MinMax MinMax::parseFromBSON(const BSONObj& min,
+MinMax MinMax::parseFromBSON(TrackingContext& trackingContext,
+                             const BSONObj& min,
                              const BSONObj& max,
-                             const StringData::ComparatorInterface* stringComparator) {
-    MinMax minmax;
+                             const StringDataComparator* stringComparator) {
+    MinMax minmax{trackingContext};
 
     // The metadata field is already excluded from generated min/max summaries.
     UpdateStatus status = minmax.update(min, /*metaField=*/boost::none, stringComparator);
@@ -881,6 +931,9 @@ MinMax MinMax::parseFromBSON(const BSONObj& min,
     return minmax;
 }
 
+SchemaElement::SchemaElement(TrackingContext& trackingContext)
+    : Element(trackingContext), _data(trackingContext) {}
+
 void SchemaElement::initializeRoot() {
     _data.setObject();
 }
@@ -893,10 +946,14 @@ const SchemaStore::Data& SchemaElement::data() const {
     return _data;
 }
 
-Schema Schema::parseFromBSON(const BSONObj& min,
+Schema::Schema(TrackingContext& trackingContext)
+    : FlatBSON<Schema, SchemaElement, BSONTypeValue>(trackingContext) {}
+
+Schema Schema::parseFromBSON(TrackingContext& trackingContext,
+                             const BSONObj& min,
                              const BSONObj& max,
-                             const StringData::ComparatorInterface* stringComparator) {
-    Schema schema;
+                             const StringDataComparator* stringComparator) {
+    Schema schema{trackingContext};
 
     // The metadata field is already excluded from generated min/max summaries.
     UpdateStatus status = schema.update(min, /*metaField=*/boost::none, stringComparator);
@@ -942,11 +999,10 @@ std::pair<Schema::UpdateStatus, SchemaElement::UpdateContext> Schema::_shouldUpd
     return std::make_pair(status, SchemaElement::UpdateContext{});
 }
 
-Schema::UpdateStatus Schema::_maybeUpdateValue(
-    SchemaStore::Obj& obj,
-    const BSONElement& elem,
-    SchemaElement::UpdateContext,
-    const StringData::ComparatorInterface* stringComparator) {
+Schema::UpdateStatus Schema::_maybeUpdateValue(SchemaStore::Obj& obj,
+                                               const BSONElement& elem,
+                                               SchemaElement::UpdateContext,
+                                               const StringDataComparator* stringComparator) {
     UpdateStatus status{UpdateStatus::Updated};
     SchemaStore::Data& data = obj.element().data();
 
