@@ -30,13 +30,38 @@
 
 #include "mongo/db/timeseries/bucket_compression.h"
 
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+// IWYU pragma: no_include "ext/alloc_traits.h"
+#include <algorithm>
+#include <cstdint>
+#include <cstring>
+#include <iterator>
+#include <memory>
+#include <string>
+#include <utility>
+#include <vector>
+
+#include "mongo/base/status.h"
+#include "mongo/bson/bsonelement.h"
 #include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/bson/bsontypes_util.h"
+#include "mongo/bson/timestamp.h"
 #include "mongo/bson/util/bsoncolumn.h"
 #include "mongo/bson/util/bsoncolumnbuilder.h"
-#include "mongo/db/storage/storage_parameters_gen.h"
+#include "mongo/bson/util/builder.h"
 #include "mongo/db/timeseries/timeseries_constants.h"
+#include "mongo/db/timeseries/timeseries_write_util.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/log_truncation.h"
+#include "mongo/logv2/redaction.h"
+#include "mongo/platform/compiler.h"
 #include "mongo/util/assert_util.h"
+#include "mongo/util/decimal_counter.h"
 #include "mongo/util/fail_point.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
@@ -48,26 +73,20 @@ namespace timeseries {
 
 namespace {
 MONGO_FAIL_POINT_DEFINE(simulateBsonColumnCompressionDataLoss);
-}
 
-CompressionResult compressBucket(const BSONObj& bucketDoc,
-                                 StringData timeFieldName,
-                                 const NamespaceString& nss,
-                                 bool validateDecompression) try {
+CompressionResult _compressBucket(const BSONObj& bucketDoc,
+                                  StringData timeFieldName,
+                                  const NamespaceString& nss,
+                                  bool validateDecompression) try {
     CompressionResult result;
 
-    // Helper for uncompressed measurements
-    struct Measurement {
-        BSONElement timeField;
-        std::vector<BSONElement> dataFields;
-    };
-
     // Buffer to help manipulate data if simulateBsonColumnCompressionDataLoss is set.
     // Contents must outlive `measurements` defined below.
     std::unique_ptr<char[]> tamperedData;
 
-    BSONObjBuilder builder;                 // builder to build the compressed bucket
-    std::vector<Measurement> measurements;  // Extracted measurements from uncompressed bucket
+    BSONObjBuilder builder;  // builder to build the compressed bucket
+    std::vector<details::Measurement>
+        measurements;                       // Extracted measurements from uncompressed bucket
     boost::optional<BSONObjIterator> time;  // Iterator to read time fields from uncompressed bucket
     std::vector<std::pair<StringData, BSONObjIterator>>
         columns;  // Iterators to read data fields from uncompressed bucket
@@ -119,7 +138,7 @@ CompressionResult compressBucket(const BSONObj& bucketDoc,
         auto timeElement = time->next();
 
         // Get BSONElement's to all data elements. Missing data fields are represented as EOO.
-        Measurement measurement;
+        details::Measurement measurement;
         measurement.timeField = timeElement;
         measurement.dataFields.resize(columns.size());
 
@@ -166,7 +185,7 @@ CompressionResult compressBucket(const BSONObj& bucketDoc,
     // Sort all the measurements on time order.
     std::sort(measurements.begin(),
               measurements.end(),
-              [](const Measurement& lhs, const Measurement& rhs) {
+              [](const details::Measurement& lhs, const details::Measurement& rhs) {
                   return lhs.timeField.date() < rhs.timeField.date();
               });
 
@@ -184,7 +203,8 @@ CompressionResult compressBucket(const BSONObj& bucketDoc,
         bool versionSet = false;
         for (const auto& controlField : controlElement.Obj()) {
             if (controlField.fieldNameStringData() == kBucketControlVersionFieldName) {
-                control.append(kBucketControlVersionFieldName, kTimeseriesControlCompressedVersion);
+                control.append(kBucketControlVersionFieldName,
+                               kTimeseriesControlCompressedSortedVersion);
                 versionSet = true;
             } else {
                 control.append(controlField);
@@ -193,7 +213,8 @@ CompressionResult compressBucket(const BSONObj& bucketDoc,
 
         // Set version if it was missing from uncompressed bucket
         if (!versionSet) {
-            control.append(kBucketControlVersionFieldName, kTimeseriesControlCompressedVersion);
+            control.append(kBucketControlVersionFieldName,
+                           kTimeseriesControlCompressedSortedVersion);
         }
 
         // Set count
@@ -263,11 +284,12 @@ CompressionResult compressBucket(const BSONObj& bucketDoc,
         };
 
         BSONObjBuilder dataBuilder = builder.subobjStart(kBucketDataFieldName);
-        BufBuilder columnBuffer;  // Reusable buffer to avoid extra allocs per column.
+        // Reusable buffer to avoid extra allocs per column.
+        allocator_aware::BufBuilder columnBuffer;
 
         // Add compressed time field first
         {
-            BSONColumnBuilder timeColumn(std::move(columnBuffer));
+            BSONColumnBuilder<> timeColumn(std::move(columnBuffer));
             for (const auto& measurement : measurements) {
                 timeColumn.append(measurement.timeField);
             }
@@ -300,7 +322,7 @@ CompressionResult compressBucket(const BSONObj& bucketDoc,
 
         // Then add compressed data fields.
         for (size_t i = 0; i < columns.size(); ++i) {
-            BSONColumnBuilder column(std::move(columnBuffer));
+            BSONColumnBuilder<> column(std::move(columnBuffer));
             for (const auto& measurement : measurements) {
                 if (auto elem = measurement.dataFields[i]) {
                     column.append(elem);
@@ -329,7 +351,41 @@ CompressionResult compressBucket(const BSONObj& bucketDoc,
     result.compressedBucket = builder.obj();
     return result;
 } catch (...) {
-    // Skip compression if we encounter any exception
+    return {};
+}
+}  // namespace
+
+CompressionResult compressBucket(const BSONObj& bucketDoc,
+                                 StringData timeFieldName,
+                                 const NamespaceString& ns,
+                                 bool validateDecompression) try {
+    // Compressing already compressed buckets is a no-op.
+    if (isCompressedBucket(bucketDoc)) {
+        CompressionResult res;
+        res.compressedBucket = bucketDoc.getOwned();
+        return res;
+    }
+    auto result = _compressBucket(bucketDoc, timeFieldName, ns, validateDecompression);
+    if (!result.compressedBucket &&
+        !MONGO_unlikely(simulateBsonColumnCompressionDataLoss.shouldFail())) {
+        LOGV2_ERROR_OPTIONS(8000400,
+                            {logv2::LogTruncation::Disabled},
+                            "Couldn't compress time-series bucket",
+                            "bucket"_attr =
+                                redact(base64::encode(bucketDoc.objdata(), bucketDoc.objsize())),
+                            "ns"_attr = ns.toStringForErrorMsg());
+        // Also log without any risk of PII
+        BSONElement id;
+        bucketDoc.getObjectID(id);
+        LOGV2_ERROR(9547401,
+                    "Couldn't compress time-series bucket",
+                    "bucketId"_attr = id,
+                    "ns"_attr = ns.toStringForErrorMsg());
+        return {};
+    }
+    return result;
+} catch (...) {
+    // Make sure no exceptions leak out of this function
     LOGV2_DEBUG(5857800,
                 1,
                 "Exception when compressing timeseries bucket, leaving it uncompressed",
@@ -337,7 +393,7 @@ CompressionResult compressBucket(const BSONObj& bucketDoc,
     return {};
 }
 
-boost::optional<BSONObj> decompressBucket(const BSONObj& bucketDoc) {
+boost::optional<BSONObj> decompressBucket(const BSONObj& bucketDoc) try {
     BSONObjBuilder builder;
 
     for (auto&& topLevel : bucketDoc) {
@@ -349,7 +405,8 @@ boost::optional<BSONObj> decompressBucket(const BSONObj& bucketDoc) {
                     // Check that we have a compressed bucket, and rewrite the version to signal
                     // it's uncompressed now.
                     if (e.type() != BSONType::NumberInt ||
-                        e.numberInt() != kTimeseriesControlCompressedVersion) {
+                        (e.numberInt() != kTimeseriesControlCompressedSortedVersion &&
+                         e.numberInt() != kTimeseriesControlCompressedUnsortedVersion)) {
                         // This bucket isn't compressed.
                         return boost::none;
                     }
@@ -391,6 +448,8 @@ boost::optional<BSONObj> decompressBucket(const BSONObj& bucketDoc) {
     }
 
     return builder.obj();
+} catch (...) {
+    return boost::none;
 }
 
 bool isCompressedBucket(const BSONObj& bucketDoc) {
@@ -405,9 +464,10 @@ bool isCompressedBucket(const BSONObj& bucketDoc) {
             versionField && isNumericBSONType(versionField.type()));
     auto version = versionField.Number();
 
-    if (version == 1) {
+    if (version == kTimeseriesControlUncompressedVersion) {
         return false;
-    } else if (version == 2) {
+    } else if (version == kTimeseriesControlCompressedSortedVersion ||
+               version == kTimeseriesControlCompressedUnsortedVersion) {
         return true;
     } else {
         uasserted(6540602, "Invalid bucket version");
