@@ -27,19 +27,26 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/timeseries/timeseries_dotted_path_support.h"
-
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <cstddef>
+#include <limits>
+#include <ostream>
 #include <string>
+#include <tuple>
+#include <utility>
 
+#include <boost/optional/optional.hpp>
+
+#include "mongo/bson/bson_depth.h"
 #include "mongo/bson/bsonelement.h"
-#include "mongo/bson/bsonmisc.h"
 #include "mongo/bson/bsonobj.h"
-#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
 #include "mongo/bson/util/bsoncolumn.h"
 #include "mongo/db/timeseries/bucket_compression.h"
 #include "mongo/db/timeseries/timeseries_constants.h"
+#include "mongo/db/timeseries/timeseries_dotted_path_support.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/ctype.h"
 
 namespace mongo {
@@ -261,7 +268,9 @@ boost::optional<BSONColumn> _extractAllElementsAlongBucketPath(
             }
             break;
         }
-        default: { MONGO_UNREACHABLE; }
+        default: {
+            MONGO_UNREACHABLE;
+        }
     }
 
     return boost::none;
@@ -395,7 +404,9 @@ bool _haveArrayAlongBucketDataPath(const BSONObj& obj,
             }
             return false;
         }
-        default: { return _handleElementForHaveArrayAlongBucketDataPath(obj, path, depth); }
+        default: {
+            return _handleElementForHaveArrayAlongBucketDataPath(obj, path, depth);
+        }
     }
 }
 
