@@ -30,16 +30,43 @@
 
 #include "mongo/db/timeseries/timeseries_index_schema_conversion_functions.h"
 
-#include "mongo/db/catalog/index_catalog_impl.h"
+#include <algorithm>
+#include <boost/container/small_vector.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional.hpp>
+#include <cstddef>
+#include <memory>
+#include <string>
+#include <tuple>
+#include <utility>
+
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/db/catalog/index_catalog_entry.h"
+#include "mongo/db/feature_flag.h"
 #include "mongo/db/index/index_descriptor.h"
 #include "mongo/db/index_names.h"
 #include "mongo/db/matcher/expression_algo.h"
 #include "mongo/db/matcher/expression_parser.h"
+#include "mongo/db/matcher/extensions_callback_noop.h"
+#include "mongo/db/pipeline/expression_context.h"
+#include "mongo/db/server_options.h"
 #include "mongo/db/storage/storage_parameters_gen.h"
 #include "mongo/db/timeseries/timeseries_constants.h"
 #include "mongo/db/timeseries/timeseries_gen.h"
-#include "mongo/logv2/log.h"
 #include "mongo/logv2/redaction.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/intrusive_counter.h"
+#include "mongo/util/str.h"
+#include "mongo/util/string_map.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
 
@@ -48,7 +75,7 @@ namespace mongo::timeseries {
 
 namespace {
 
-bool isIndexOnControl(const StringData& field) {
+bool isIndexOnControl(StringData field) {
     return field.startsWith(timeseries::kControlMinFieldNamePrefix) ||
         field.startsWith(timeseries::kControlMaxFieldNamePrefix);
 }
@@ -57,7 +84,7 @@ bool isIndexOnControl(const StringData& field) {
  * Takes the index specification field name, such as 'control.max.x.y', or 'control.min.z' and
  * returns a pair of the prefix ('control.min.' or 'control.max.') and key ('x.y' or 'z').
  */
-std::pair<std::string, std::string> extractControlPrefixAndKey(const StringData& field) {
+std::pair<std::string, std::string> extractControlPrefixAndKey(StringData field) {
     // Can't use rfind() due to dotted fields such as 'control.max.x.y'.
     size_t numDotsFound = 0;
     auto fieldIt = std::find_if(field.begin(), field.end(), [&numDotsFound](const char c) {
@@ -106,9 +133,10 @@ StatusWith<BSONObj> createBucketsSpecFromTimeseriesSpec(const TimeseriesOptions&
             if (!elem.isNumber()) {
                 return {ErrorCodes::BadValue,
                         str::stream()
-                            << "Invalid index spec for time-series collection: "
-                            << redact(timeseriesIndexSpecBSON)
-                            << ". Indexes on the time field must be ascending or descending "
+                            << "Invalid " << (isShardKeySpec ? "shard key" : "index spec")
+                            << " for time-series collection: " << redact(timeseriesIndexSpecBSON)
+                            << ". " << (isShardKeySpec ? "Shard keys" : "Indexes")
+                            << " on the time field must be ascending or descending "
                                "(numbers only): "
                             << elem};
             }
@@ -328,6 +356,97 @@ boost::optional<BSONObj> createTimeseriesIndexSpecFromBucketsIndexSpec(
     return builder.obj();
 }
 
+/**
+ * Maps a buckets shard key spec to a spec for the index backing the shard key on the buckets
+ * collection using the information provided in 'timeseriesOptions'. The shard key on the buckets
+ * collection should already be rewritten to use bucket collection field names and should be valid.
+ *
+ * If 'bucketShardKeySpecBSON' does not match a valid time-series shard key format, then boost::none
+ * is returned.
+ *
+ * Conversion Example:
+ * On a time-series collection with 'tm' time field and 'mm' metadata field,
+ * we may see a compound shard key on the underlying bucket collection mapped from:
+ * {
+ *     'meta.tag1': 1,
+ *     'control.min.tm': 1,
+ * }
+ * to an index on the buckets collection:
+ * {
+ *     'meta.tag1': 1,
+ *     'control.min.tm': 1,
+ *     'control.max.tm': 1
+ * }
+ */
+boost::optional<BSONObj> createBucketsIndexSpecFromBucketsShardKeySpec(
+    const TimeseriesOptions& timeseriesOptions, const BSONObj& bucketShardKeySpecBSON) {
+
+    // The shard key has already been validated. Therefore, the index backing the shard key must be
+    // valid. If not, this means  we have passed shard key validation, but have created an
+    // invalid index backing the shard key. We will return boost::none if the index is invalid and
+    // the caller of the function will verify an index was returned.
+    if (bucketShardKeySpecBSON.isEmpty()) {
+        return {};
+    }
+
+    if (bucketShardKeySpecBSON.firstElement().fieldNameStringData() == "$hint"_sd ||
+        bucketShardKeySpecBSON.firstElement().fieldNameStringData() == "$natural"_sd) {
+        return {};
+    }
+
+    auto timeField = timeseriesOptions.getTimeField();
+    auto metaField = timeseriesOptions.getMetaField();
+    const std::string controlMinTimeField = str::stream()
+        << timeseries::kControlMinFieldNamePrefix << timeField;
+
+    BSONObjBuilder builder;
+    for (const auto& elem : bucketShardKeySpecBSON) {
+        // Unsharded collections can appear with the key '{_id:1}'. We will return the key as is in
+        // that case.
+        if (elem.fieldNameStringData() == timeseries::kBucketIdFieldName) {
+            if (!elem.isNumber() || elem.number() != 1 || bucketShardKeySpecBSON.nFields() != 1) {
+                return {};
+            }
+            return builder.append(elem).obj();
+        }
+
+        // We expect the index backing the shard key to already be "buckets-encoded". This means
+        // that indexSpecBSON should be "{control.min.time:1}".
+        if (elem.fieldNameStringData() == controlMinTimeField) {
+            if (!elem.isNumber() || elem.number() < 0) {
+                // Shard keys on the time field must be ascending.
+                return {};
+            }
+
+            // Append the 'control.min.time' element and add the 'control.max.time' element.
+            builder.append(elem);
+            builder.appendAs(elem,
+                             str::stream() << timeseries::kControlMaxFieldNamePrefix << timeField);
+            continue;
+        }
+
+        // If the index is on the meta field, the index should already be buckets-encoded and thus
+        // be rewritten as "meta".
+        if (metaField) {
+            if (elem.fieldNameStringData() == timeseries::kBucketMetaFieldName) {
+                builder.append(elem);
+                continue;
+            }
+
+            // Time-series indexes on sub-documents of the 'metaField' are allowed.
+            if (elem.fieldNameStringData().startsWith(timeseries::kBucketMetaFieldName + ".")) {
+                builder.append(elem);
+                continue;
+            }
+        }
+
+        // Shard keys are only allowed on the meta field or time field.
+        return {};
+    }
+
+    return builder.obj();
+}
+
 }  // namespace
 
 StatusWith<BSONObj> createBucketsIndexSpecFromTimeseriesIndexSpec(
@@ -340,6 +459,17 @@ StatusWith<BSONObj> createBucketsShardKeySpecFromTimeseriesShardKeySpec(
     return createBucketsSpecFromTimeseriesSpec(timeseriesOptions, timeseriesShardKeySpecBSON, true);
 }
 
+boost::optional<BSONObj> createBucketsShardKeyIndexFromBucketsShardKeySpec(
+    const TimeseriesOptions& timeseriesOptions, const BSONObj& timeseriesShardKeySpecBSON) {
+    return createBucketsIndexSpecFromBucketsShardKeySpec(timeseriesOptions,
+                                                         timeseriesShardKeySpecBSON);
+}
+
+boost::optional<BSONObj> createTimeseriesIndexFromBucketsIndexSpec(
+    const TimeseriesOptions& timeseriesOptions, const BSONObj& bucketsIndexSpecBSON) {
+    return createTimeseriesIndexSpecFromBucketsIndexSpec(timeseriesOptions, bucketsIndexSpecBSON);
+}
+
 boost::optional<BSONObj> createTimeseriesIndexFromBucketsIndex(
     const TimeseriesOptions& timeseriesOptions, const BSONObj& bucketsIndex) {
 
@@ -348,7 +478,8 @@ boost::optional<BSONObj> createTimeseriesIndexFromBucketsIndex(
         // time-series metric indexes feature flag is enabled. If the feature flag isn't enabled,
         // the reverse mapping mechanism will be used. This is necessary to skip returning any
         // incompatible indexes created when the feature flag was enabled.
-        return bucketsIndex.getObjectField(kOriginalSpecFieldName);
+        BSONObj intermediateObj = bucketsIndex.getObjectField(kOriginalSpecFieldName);
+        return intermediateObj.addField(bucketsIndex[IndexDescriptor::k2dsphereVersionFieldName]);
     }
     if (bucketsIndex.hasField(kKeyFieldName)) {
         auto timeseriesKeyValue = createTimeseriesIndexSpecFromBucketsIndexSpec(
