@@ -27,14 +27,26 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <boost/optional.hpp>
+#include <cstdint>
+#include <fmt/format.h>
 
-#include "mongo/db/timeseries/timeseries_options.h"
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonmisc.h"
 #include "mongo/bson/bsonobj.h"
 #include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/db/feature_flag.h"
+#include "mongo/db/server_options.h"
 #include "mongo/db/storage/storage_parameters_gen.h"
 #include "mongo/db/timeseries/timeseries_gen.h"
+#include "mongo/db/timeseries/timeseries_options.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/duration.h"
 
 namespace mongo {
 
@@ -60,8 +72,6 @@ Status checkBucketingParameters(TimeseriesOptions& timeseriesOptions,
     auto maxSpanSeconds = timeseriesOptions.getBucketMaxSpanSeconds();
     auto granularity = timeseriesOptions.getGranularity();
 
-    bool allowSecondsParameters = feature_flags::gTimeseriesScalabilityImprovements.isEnabled(
-        serverGlobalParams.featureCompatibility.acquireFCVSnapshot());
     auto maxSpanSecondsFromGranularity =
         getMaxSpanSecondsFromGranularity(granularity.get_value_or(BucketGranularityEnum::Seconds));
 
@@ -89,7 +99,7 @@ Status checkBucketingParameters(TimeseriesOptions& timeseriesOptions,
                             "Time-series 'bucketRoundingSeconds' is missing"};
             }
         }
-    } else if (allowSecondsParameters && (maxSpanSeconds || roundingSeconds)) {
+    } else if (maxSpanSeconds || roundingSeconds) {
         if (roundingSeconds != maxSpanSeconds) {
             return {ErrorCodes::InvalidOptions,
                     "Time-series 'bucketRoundingSeconds' must be equal to 'bucketMaxSpanSeconds'"};
@@ -121,10 +131,6 @@ Status checkBucketingParameters(TimeseriesOptions& timeseriesOptions,
                             "Time-series 'bucketMaxSpanSeconds' is missing"};
             }
         }
-
-        if (!allowSecondsParameters && roundingSeconds) {
-            uasserted(8545900, "Unknown parameter 'bucketRoundingSeconds'");
-        }
     }
 
     return Status::OK();
@@ -137,8 +143,6 @@ Status isTimeseriesGranularityValidAndUnchanged(const TimeseriesOptions& current
                                                 bool* shouldUpdateOptions) {
     auto currentGranularity = currentOptions.getGranularity();
     auto targetGranularity = targetOptions.getGranularity();
-    bool allowSecondsParameters = feature_flags::gTimeseriesScalabilityImprovements.isEnabled(
-        serverGlobalParams.featureCompatibility.acquireFCVSnapshot());
 
     // If the timeseries options are completely empty, we can skip updating them.
     if (!targetGranularity.has_value() && !targetOptions.getBucketMaxSpanSeconds().has_value() &&
@@ -203,10 +207,6 @@ Status isTimeseriesGranularityValidAndUnchanged(const TimeseriesOptions& current
         targetMaxSpanSeconds = getMaxSpanSecondsFromGranularity(targetGranularity.get());
         targetRoundingSeconds = getBucketRoundingSecondsFromGranularity(targetGranularity.get());
     } else {
-        if (!allowSecondsParameters) {
-            return Status{ErrorCodes::InvalidOptions, "No timeseries parameters were given"};
-        }
-
         if (!targetOptions.getBucketMaxSpanSeconds() || !targetOptions.getBucketRoundingSeconds()) {
             return Status{
                 ErrorCodes::InvalidOptions,
@@ -232,7 +232,7 @@ Status isTimeseriesGranularityValidAndUnchanged(const TimeseriesOptions& current
     // Check if we are trying to set the bucketing parameters to the existing values. If so, we do
     // not need to update any existing options.
     if (currentMaxSpanSeconds == targetMaxSpanSeconds) {
-        if (!allowSecondsParameters || currentBucketRoundingSeconds == targetRoundingSeconds) {
+        if (currentBucketRoundingSeconds == targetRoundingSeconds) {
             if (shouldUpdateOptions)
                 *shouldUpdateOptions = false;
             return Status::OK();
@@ -245,7 +245,7 @@ Status isTimeseriesGranularityValidAndUnchanged(const TimeseriesOptions& current
             "Timeseries 'bucketMaxSpanSeconds' needs to be equal or greater to transition"};
     }
 
-    if (allowSecondsParameters && (currentBucketRoundingSeconds > targetRoundingSeconds)) {
+    if (currentBucketRoundingSeconds > targetRoundingSeconds) {
         return Status{
             ErrorCodes::InvalidOptions,
             "Timeseries 'bucketRoundingSeconds' needs to be equal or greater to transition"};
@@ -305,6 +305,8 @@ StatusWith<std::pair<TimeseriesOptions, bool>> applyTimeseriesOptionsModificatio
 }
 
 BSONObj generateViewPipeline(const TimeseriesOptions& options, bool asArray) {
+    // TODO(SERVER-76411): Remove this and read the value directly after 'bucketMaxSpanSeconds' is
+    // guaranteed to be present.
     // Generates the 'bucketMaxSpanSeconds' field if missing.
     auto maxSpanSeconds =
         options.getBucketMaxSpanSeconds().get_value_or(getMaxSpanSecondsFromGranularity(
@@ -386,22 +388,23 @@ Date_t roundTimestampToGranularity(const Date_t& time, const TimeseriesOptions&
             getBucketRoundingSecondsFromGranularity(BucketGranularityEnum::Seconds));
     }
 
-    long long roundingMilliSeconds = roundingSeconds * 1000;
-    long long timeMilliSeconds = time.toMillisSinceEpoch();
-    long long roundedTimeMilliSeconds =
-        (timeMilliSeconds - (timeMilliSeconds % roundingMilliSeconds));
+    return roundTimestampBySeconds(time, roundingSeconds);
+}
+
+Date_t roundTimestampBySeconds(const Date_t& time, const long long roundingSeconds) {
+    long long timeSeconds = durationCount<Seconds>(time.toDurationSinceEpoch());
+    long long roundedTimeSeconds = (timeSeconds - (timeSeconds % roundingSeconds));
     // Make sure we always round down and not towards epoch, even for dates prior to 1970 with a
     // negative duration since epoch.
-    if (roundedTimeMilliSeconds > timeMilliSeconds) {
-        // It is possible that we can underflow performing the subtraction below for extreme
-        // cases where the input time is close to Date_t::min(). If this would be the case then just
-        // return Date_t::min() instead.
-        if (roundedTimeMilliSeconds < Date_t::min().toMillisSinceEpoch() + roundingMilliSeconds) {
-            return Date_t::min();
-        }
-        roundedTimeMilliSeconds -= roundingMilliSeconds;
+    if (roundedTimeSeconds > timeSeconds) {
+        roundedTimeSeconds -= roundingSeconds;
+        // It is not possible that we underflowed when performing the subtraction above. Because
+        // we've converted the dates in milliseconds to seconds there is tons of integer space left
+        // for the subtraction. We'd need to have a gigantic amount of rounding seconds to be able
+        // to overflow here. Therefore we invariant over uasserting.
+        invariant(roundedTimeSeconds <= timeSeconds);
     }
-    return Date_t::fromMillisSinceEpoch(roundedTimeMilliSeconds);
+    return Date_t::fromDurationSinceEpoch(Seconds{roundedTimeSeconds});
 }
 
 Status validateAndSetBucketingParameters(TimeseriesOptions& timeseriesOptions) {
