@@ -29,19 +29,45 @@
 
 #pragma once
 
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <memory>
+#include <string>
+#include <utility>
+#include <vector>
+
+#include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/db/api_parameters.h"
 #include "mongo/db/cancelable_operation_context.h"
+#include "mongo/db/commands/bulk_write_gen.h"
+#include "mongo/db/database_name.h"
+#include "mongo/db/dbmessage.h"
+#include "mongo/db/logical_time.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/query/find_command.h"
 #include "mongo/db/resource_yielder.h"
+#include "mongo/db/service_context.h"
 #include "mongo/db/session/logical_session_id.h"
+#include "mongo/db/session/logical_session_id_gen.h"
 #include "mongo/executor/inline_executor.h"
 #include "mongo/executor/task_executor.h"
+#include "mongo/platform/mutex.h"
+#include "mongo/rpc/message.h"
 #include "mongo/rpc/write_concern_error_detail.h"
 #include "mongo/s/write_ops/batched_command_request.h"
 #include "mongo/s/write_ops/batched_command_response.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/cancellation.h"
 #include "mongo/util/concurrency/notification.h"
 #include "mongo/util/concurrency/with_lock.h"
+#include "mongo/util/functional.h"
 #include "mongo/util/future.h"
+#include "mongo/util/out_of_line_executor.h"
+#include "mongo/util/time_support.h"
 
 namespace mongo::txn_api {
 namespace details {
@@ -135,6 +161,13 @@ public:
     virtual BatchedCommandResponse runCRUDOpSync(const BatchedCommandRequest& cmd,
                                                  std::vector<StmtId> stmtIds) const = 0;
 
+    /**
+     * Helper method to run BulkWriteCommandRequest in the transaction client's transaction.
+     */
+    virtual SemiFuture<BulkWriteCommandReply> runCRUDOp(
+        const BulkWriteCommandRequest& cmd) const = 0;
+    virtual BulkWriteCommandReply runCRUDOpSync(const BulkWriteCommandRequest& cmd) const = 0;
+
     /**
      * Helper method that runs the given find in the transaction client's transaction and will
      * iterate and exhaust the find's cursor, returning a vector with all matching documents.
@@ -250,6 +283,13 @@ public:
      * Returns if the client is eligible to run cluster operations.
      */
     virtual bool runsClusterOperations() const = 0;
+
+    Service* getService() {
+        return _service;
+    }
+
+protected:
+    Service* _service;
 };
 
 /**
@@ -258,6 +298,10 @@ public:
  */
 class DefaultSEPTransactionClientBehaviors : public SEPTransactionClientBehaviors {
 public:
+    DefaultSEPTransactionClientBehaviors(OperationContext* opCtx) {
+        _service = opCtx->getService();
+    }
+
     BSONObj maybeModifyCommand(BSONObj cmdObj) const override {
         return cmdObj;
     }
@@ -265,7 +309,7 @@ public:
     Future<DbResponse> handleRequest(OperationContext* opCtx,
                                      const Message& request) const override;
 
-    bool runsClusterOperations() const {
+    bool runsClusterOperations() const override {
         return false;
     }
 };
@@ -290,32 +334,33 @@ public:
     SEPTransactionClient(const SEPTransactionClient&) = delete;
     SEPTransactionClient operator=(const SEPTransactionClient&) = delete;
 
-    virtual void initialize(std::unique_ptr<details::TxnHooks> hooks) override {
+    void initialize(std::unique_ptr<details::TxnHooks> hooks) override {
         invariant(!_hooks);
         _hooks = std::move(hooks);
     }
 
-    virtual SemiFuture<BSONObj> runCommand(const DatabaseName& dbName, BSONObj cmd) const override;
-    virtual BSONObj runCommandSync(const DatabaseName& dbName, BSONObj cmd) const override;
+    SemiFuture<BSONObj> runCommand(const DatabaseName& dbName, BSONObj cmd) const override;
+    BSONObj runCommandSync(const DatabaseName& dbName, BSONObj cmd) const override;
 
-    virtual SemiFuture<BSONObj> runCommandChecked(const DatabaseName& dbName,
-                                                  BSONObj cmd) const override;
-    virtual BSONObj runCommandCheckedSync(const DatabaseName& dbName, BSONObj cmd) const override;
+    SemiFuture<BSONObj> runCommandChecked(const DatabaseName& dbName, BSONObj cmd) const override;
+    BSONObj runCommandCheckedSync(const DatabaseName& dbName, BSONObj cmd) const override;
 
-    virtual SemiFuture<BatchedCommandResponse> runCRUDOp(
-        const BatchedCommandRequest& cmd, std::vector<StmtId> stmtIds) const override;
-    virtual BatchedCommandResponse runCRUDOpSync(const BatchedCommandRequest& cmd,
+    SemiFuture<BatchedCommandResponse> runCRUDOp(const BatchedCommandRequest& cmd,
                                                  std::vector<StmtId> stmtIds) const override;
+    BatchedCommandResponse runCRUDOpSync(const BatchedCommandRequest& cmd,
+                                         std::vector<StmtId> stmtIds) const override;
 
-    virtual SemiFuture<std::vector<BSONObj>> exhaustiveFind(
-        const FindCommandRequest& cmd) const override;
-    virtual std::vector<BSONObj> exhaustiveFindSync(const FindCommandRequest& cmd) const override;
+    SemiFuture<BulkWriteCommandReply> runCRUDOp(const BulkWriteCommandRequest& cmd) const override;
+    BulkWriteCommandReply runCRUDOpSync(const BulkWriteCommandRequest& cmd) const override;
 
-    virtual bool supportsClientTransactionContext() const override {
+    SemiFuture<std::vector<BSONObj>> exhaustiveFind(const FindCommandRequest& cmd) const override;
+    std::vector<BSONObj> exhaustiveFindSync(const FindCommandRequest& cmd) const override;
+
+    bool supportsClientTransactionContext() const override {
         return true;
     }
 
-    virtual bool runsClusterOperations() const override {
+    bool runsClusterOperations() const override {
         return _behaviors->runsClusterOperations();
     }
 
@@ -327,6 +372,8 @@ private:
     ExecutorFuture<BatchedCommandResponse> _runCRUDOp(const BatchedCommandRequest& cmd,
                                                       std::vector<StmtId> stmtIds) const;
 
+    ExecutorFuture<BulkWriteCommandReply> _runCRUDOp(const BulkWriteCommandRequest& cmd) const;
+
     ExecutorFuture<std::vector<BSONObj>> _exhaustiveFind(const FindCommandRequest& cmd) const;
 
 private:
@@ -373,7 +420,7 @@ public:
         : _executor(executor),
           _txnClient(std::move(txnClient)),
           _token(token),
-          _service(opCtx->getServiceContext()) {
+          _service(opCtx->getService()) {
         _primeTransaction(opCtx);
         _txnClient->initialize(_makeTxnHooks());
     }
@@ -464,7 +511,7 @@ public:
      * Returns an unowned pointer to the ServiceContext used to create this transaction.
      */
     ServiceContext* getParentServiceContext() const {
-        return _service;
+        return _service->getServiceContext();
     }
 
 private:
@@ -545,7 +592,7 @@ private:
     OperationSessionInfo _sessionInfo;
     TransactionState _state;
     bool _acquiredSessionFromPool{false};
-    ServiceContext* _service;
+    Service* _service;
 };
 
 /**
