@@ -27,32 +27,39 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/transaction_validation.h"
-
 #include <fmt/format.h>
 
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
 #include "mongo/db/commands.h"
 #include "mongo/db/commands/txn_cmds_gen.h"
 #include "mongo/db/commands/txn_two_phase_commit_cmds_gen.h"
-#include "mongo/db/session/logical_session_id.h"
+#include "mongo/db/repl/replication_coordinator.h"
+#include "mongo/db/transaction_validation.h"
 #include "mongo/db/write_concern_options.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/str.h"
 
 namespace mongo {
 
 using namespace fmt::literals;
 
-bool isRetryableWriteCommand(StringData cmdName) {
-    auto command = CommandHelpers::findCommand(cmdName);
+bool isRetryableWriteCommand(Service* service, StringData cmdName) {
+    auto command = CommandHelpers::findCommand(service, cmdName);
     uassert(ErrorCodes::CommandNotFound,
             str::stream() << "Encountered unknown command during retryability check: " << cmdName,
             command);
     return command->supportsRetryableWrite();
 }
 
-bool isTransactionCommand(StringData cmdName) {
-    auto command = CommandHelpers::findCommand(cmdName);
+bool isTransactionCommand(Service* service, StringData cmdName) {
+    // TODO SERVER-82282 refactor: This code runs when commands are invoked from both mongod and
+    // mongos and the latter does not know _shardsvrCreateCommand.
+    if (cmdName == "_shardsvrCreateCollection")
+        return false;
+
+    auto command = CommandHelpers::findCommand(service, cmdName);
     uassert(ErrorCodes::CommandNotFound,
             str::stream() << "Encountered unknown command during isTransactionCommand check: "
                           << cmdName,
@@ -60,10 +67,10 @@ bool isTransactionCommand(StringData cmdName) {
     return command->isTransactionCommand();
 }
 
-void validateWriteConcernForTransaction(const WriteConcernOptions& wcResult, StringData cmdName) {
+void validateWriteConcernForTransaction(const WriteConcernOptions& wcResult, const Command* cmd) {
     uassert(ErrorCodes::InvalidOptions,
             "writeConcern is not allowed within a multi-statement transaction",
-            wcResult.usedDefaultConstructedWC || isTransactionCommand(cmdName));
+            wcResult.usedDefaultConstructedWC || cmd->isTransactionCommand());
 }
 
 bool isReadConcernLevelAllowedInTransaction(repl::ReadConcernLevel readConcernLevel) {
@@ -72,38 +79,47 @@ bool isReadConcernLevelAllowedInTransaction(repl::ReadConcernLevel readConcernLe
         readConcernLevel == repl::ReadConcernLevel::kLocalReadConcern;
 }
 
+namespace {
+const CommandNameAtom killCursorsAtom("killCursors");
+const CommandNameAtom prepareTransactionAtom(PrepareTransaction::kCommandName);
+const CommandNameAtom commitTransactionAtom(CommitTransaction::kCommandName);
+const CommandNameAtom abortTransactionAtom(AbortTransaction::kCommandName);
+}  // namespace
+
 void validateSessionOptions(const OperationSessionInfoFromClient& sessionOptions,
-                            StringData cmdName,
-                            const NamespaceString& nss,
+                            Command* command,
+                            const std::vector<NamespaceString>& namespaces,
                             bool allowTransactionsOnConfigDatabase) {
+
     if (sessionOptions.getAutocommit()) {
-        CommandHelpers::canUseTransactions(nss, cmdName, allowTransactionsOnConfigDatabase);
+        CommandHelpers::canUseTransactions(namespaces, command, allowTransactionsOnConfigDatabase);
     }
 
     if (!sessionOptions.getAutocommit() && sessionOptions.getTxnNumber()) {
         uassert(ErrorCodes::NotARetryableWriteCommand,
                 "txnNumber may only be provided for multi-document transactions and retryable "
                 "write commands. autocommit:false was not provided, and {} is not a retryable "
-                "write command."_format(cmdName),
-                isRetryableWriteCommand(cmdName));
+                "write command."_format(command->getName()),
+                command->supportsRetryableWrite());
     }
 
     if (sessionOptions.getStartTransaction()) {
+        auto atom = command->getNameAtom();
         uassert(ErrorCodes::OperationNotSupportedInTransaction,
                 "Cannot run killCursors as the first operation in a multi-document transaction.",
-                cmdName != "killCursors");
+                atom != killCursorsAtom);
 
         uassert(ErrorCodes::OperationNotSupportedInTransaction,
                 "Cannot start a transaction with a prepare",
-                cmdName != PrepareTransaction::kCommandName);
+                atom != prepareTransactionAtom);
 
         uassert(ErrorCodes::OperationNotSupportedInTransaction,
                 "Cannot start a transaction with a commit",
-                cmdName != CommitTransaction::kCommandName);
+                atom != commitTransactionAtom);
 
         uassert(ErrorCodes::OperationNotSupportedInTransaction,
                 "Cannot start a transaction with an abort",
-                cmdName != AbortTransaction::kCommandName);
+                atom != abortTransactionAtom);
     }
 }
 
@@ -111,12 +127,12 @@ void doTransactionValidationForWrites(OperationContext* opCtx, const NamespaceSt
     if (!opCtx->inMultiDocumentTransaction())
         return;
     uassert(50791,
-            str::stream() << "Cannot write to system collection " << ns.toString()
+            str::stream() << "Cannot write to system collection " << ns.toStringForErrorMsg()
                           << " within a transaction.",
             !ns.isSystem() || ns.isPrivilegeCollection() || ns.isTimeseriesBucketsCollection());
     const auto replCoord = repl::ReplicationCoordinator::get(opCtx);
     uassert(50790,
-            str::stream() << "Cannot write to unreplicated collection " << ns.toString()
+            str::stream() << "Cannot write to unreplicated collection " << ns.toStringForErrorMsg()
                           << " within a transaction.",
             !replCoord->isOplogDisabledFor(opCtx, ns));
 }
