@@ -30,43 +30,103 @@
 
 #include "mongo/db/ttl.h"
 
+// IWYU pragma: no_include "cxxabi.h"
+#include <absl/container/node_hash_map.h>
+#include <absl/meta/type_traits.h>
+#include <boost/cstdint.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr.hpp>
+#include <cstdint>
+#include <limits>
+#include <mutex>
+#include <tuple>
+#include <utility>
+#include <vector>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/oid.h"
+#include "mongo/db/admission/execution_admission_context.h"
 #include "mongo/db/auth/authorization_session.h"
-#include "mongo/db/auth/user_name.h"
 #include "mongo/db/catalog/coll_mod.h"
 #include "mongo/db/catalog/collection.h"
 #include "mongo/db/catalog/collection_catalog.h"
-#include "mongo/db/catalog/database_holder.h"
+#include "mongo/db/catalog/collection_options.h"
 #include "mongo/db/catalog/index_catalog.h"
 #include "mongo/db/catalog/index_key_validate.h"
 #include "mongo/db/client.h"
+#include "mongo/db/coll_mod_gen.h"
 #include "mongo/db/commands/fsync_locked.h"
 #include "mongo/db/commands/server_status_metric.h"
 #include "mongo/db/concurrency/exception_util.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
 #include "mongo/db/db_raii.h"
+#include "mongo/db/exec/batched_delete_stage.h"
+#include "mongo/db/exec/collection_scan_common.h"
 #include "mongo/db/exec/delete_stage.h"
+#include "mongo/db/exec/plan_stats.h"
 #include "mongo/db/index/index_descriptor.h"
+#include "mongo/db/index_names.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/ops/insert.h"
+#include "mongo/db/query/canonical_query.h"
+#include "mongo/db/query/find_command.h"
+#include "mongo/db/query/index_bounds.h"
 #include "mongo/db/query/internal_plans.h"
+#include "mongo/db/query/plan_executor.h"
+#include "mongo/db/query/plan_yield_policy.h"
+#include "mongo/db/query/record_id_bound.h"
 #include "mongo/db/record_id_helpers.h"
+#include "mongo/db/repl/member_state.h"
+#include "mongo/db/repl/read_concern_args.h"
 #include "mongo/db/repl/replica_set_aware_service.h"
 #include "mongo/db/repl/replication_coordinator.h"
+#include "mongo/db/repl/tenant_migration_access_blocker.h"
 #include "mongo/db/repl/tenant_migration_access_blocker_registry.h"
 #include "mongo/db/s/operation_sharding_state.h"
 #include "mongo/db/s/shard_filtering_metadata_refresh.h"
+#include "mongo/db/server_feature_flags_gen.h"
+#include "mongo/db/server_options.h"
 #include "mongo/db/service_context.h"
+#include "mongo/db/shard_role.h"
 #include "mongo/db/stats/resource_consumption_metrics.h"
-#include "mongo/db/storage/storage_parameters_gen.h"
+#include "mongo/db/timeseries/timeseries_gen.h"
+#include "mongo/db/transaction_resources.h"
 #include "mongo/db/ttl_collection_cache.h"
 #include "mongo/db/ttl_gen.h"
+#include "mongo/executor/task_executor_pool.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/s/catalog_cache.h"
+#include "mongo/s/chunk_version.h"
+#include "mongo/s/database_version.h"
 #include "mongo/s/grid.h"
+#include "mongo/s/index_version.h"
+#include "mongo/s/shard_version.h"
 #include "mongo/s/shard_version_factory.h"
+#include "mongo/s/sharding_index_catalog_cache.h"
+#include "mongo/s/stale_exception.h"
 #include "mongo/stdx/thread.h"
 #include "mongo/util/assert_util.h"
 #include "mongo/util/background.h"
+#include "mongo/util/concurrency/admission_context.h"
 #include "mongo/util/concurrency/idle_thread_block.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/fail_point.h"
 #include "mongo/util/log_with_sampling.h"
+#include "mongo/util/scopeguard.h"
+#include "mongo/util/time_support.h"
+#include "mongo/util/timer.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kIndex
 
@@ -76,10 +136,9 @@ namespace mongo {
 namespace {
 const auto getTTLMonitor = ServiceContext::declareDecoration<std::unique_ptr<TTLMonitor>>();
 
+// TODO (SERVER-64506): support change streams' pre- and post-images.
 bool isBatchingEnabled(const CollectionPtr& collectionPtr) {
-    return feature_flags::gBatchMultiDeletes.isEnabled(
-               serverGlobalParams.featureCompatibility.acquireFCVSnapshot()) &&
-        ttlMonitorBatchDeletes.load() && !collectionPtr->isChangeStreamPreAndPostImagesEnabled();
+    return ttlMonitorBatchDeletes.load() && !collectionPtr->isChangeStreamPreAndPostImagesEnabled();
 }
 
 // When batching is enabled, returns BatchedDeleteStageParams that limit the amount of work done in
@@ -279,8 +338,9 @@ private:
         ttlMonitor->onStepUp();
     }
     void onStepDown() override {}
+    void onRollback() override {}
     void onBecomeArbiter() override {}
-    inline std::string getServiceName() const override final {
+    inline std::string getServiceName() const final {
         return "TTLMonitorService";
     }
 };
@@ -299,18 +359,20 @@ TTLMonitorService* TTLMonitorService::get(ServiceContext* serviceContext) {
 
 MONGO_FAIL_POINT_DEFINE(hangTTLMonitorWithLock);
 MONGO_FAIL_POINT_DEFINE(hangTTLMonitorBetweenPasses);
+MONGO_FAIL_POINT_DEFINE(TTLMonitorSkipRunningCollMod);
 
 // A TTL pass completes when there are no more expired documents to remove. A single TTL pass may
 // consist of multiple sub-passes. Each sub-pass deletes all the expired documents it can up to
 // 'ttlSubPassTargetSecs'. It is possible for a sub-pass to complete before all expired documents
 // have been removed.
-CounterMetric ttlPasses("ttl.passes");
-CounterMetric ttlSubPasses("ttl.subPasses");
-CounterMetric ttlDeletedDocuments("ttl.deletedDocuments");
+auto& ttlPasses = *MetricBuilder<Counter64>{"ttl.passes"};
+auto& ttlSubPasses = *MetricBuilder<Counter64>{"ttl.subPasses"};
+auto& ttlDeletedDocuments = *MetricBuilder<Counter64>{"ttl.deletedDocuments"};
 
 // Counts the subpasses over TTL collections where the deletes on a collection are increased from
 // 'low' to 'normal' priority.
-CounterMetric ttlCollSubpassesIncreasedPriority("ttl.collSubpassesIncreasedPriority");
+auto& ttlCollSubpassesIncreasedPriority =
+    *MetricBuilder<Counter64>{"ttl.collSubpassesIncreasedPriority"};
 
 using MtabType = TenantMigrationAccessBlocker::BlockerType;
 
@@ -351,14 +413,9 @@ void TTLMonitor::updateSleepSeconds(Seconds newSeconds) {
 }
 
 void TTLMonitor::run() {
-    ThreadClient tc(name(), getGlobalServiceContext());
+    ThreadClient tc(name(), getGlobalServiceContext()->getService(ClusterRole::ShardServer));
     AuthorizationSession::get(cc())->grantInternalAuthorization(&cc());
 
-    {
-        stdx::lock_guard<Client> lk(*tc.get());
-        tc.get()->setSystemOperationKillableByStepdown(lk);
-    }
-
     while (true) {
         {
             auto startTime = Date_t::now();
@@ -402,7 +459,7 @@ void TTLMonitor::run() {
 
         try {
             const auto opCtxPtr = cc().makeOperationContext();
-            writeConflictRetry(opCtxPtr.get(), "TTL pass", NamespaceString().ns(), [&] {
+            writeConflictRetry(opCtxPtr.get(), "TTL pass", NamespaceString::kEmpty, [&] {
                 _doTTLPass(opCtxPtr.get());
             });
         } catch (const DBException& ex) {
@@ -451,8 +508,7 @@ void TTLMonitor::_doTTLPass(OperationContext* opCtx) {
 bool TTLMonitor::_doTTLSubPass(
     OperationContext* opCtx, stdx::unordered_map<UUID, long long, UUID::Hash>& collSubpassHistory) {
     // If part of replSet but not in a readable state (e.g. during initial sync), skip.
-    if (repl::ReplicationCoordinator::get(opCtx)->getReplicationMode() ==
-            repl::ReplicationCoordinator::modeReplSet &&
+    if (repl::ReplicationCoordinator::get(opCtx)->getSettings().isReplSet() &&
         !repl::ReplicationCoordinator::get(opCtx)->getMemberState().readable())
         return false;
 
@@ -491,7 +547,7 @@ bool TTLMonitor::_doTTLSubPass(
             // 'AdmissionContext::Priority::kNormal' for one index means the priority will be
             // 'normal' for all indexes.
             AdmissionContext::Priority priority = getTTLPriority(uuid, ttlPriorityMap);
-            ScopedAdmissionPriorityForLock priorityGuard(opCtx->lockState(), priority);
+            ScopedAdmissionPriority<ExecutionAdmissionContext> priorityGuard(opCtx, priority);
 
             for (const auto& info : infos) {
                 bool moreToDelete = _doTTLIndexDelete(opCtx, &ttlCollectionCache, uuid, info);
@@ -515,13 +571,7 @@ bool TTLMonitor::_doTTLIndexDelete(OperationContext* opCtx,
                                    TTLCollectionCache* ttlCollectionCache,
                                    const UUID& uuid,
                                    const TTLCollectionCache::Info& info) {
-    // Skip collections that have not been made visible yet. The TTLCollectionCache
-    // already has the index information available, so we want to avoid removing it
-    // until the collection is visible.
     auto collectionCatalog = CollectionCatalog::get(opCtx);
-    if (collectionCatalog->isCollectionAwaitingVisibility(uuid)) {
-        return false;
-    }
 
     // The collection was dropped.
     auto nss = collectionCatalog->lookupNSSByUUID(opCtx, uuid);
@@ -543,26 +593,30 @@ bool TTLMonitor::_doTTLIndexDelete(OperationContext* opCtx,
     try {
         uassertStatusOK(userAllowedWriteNS(opCtx, *nss));
 
-        auto catalogCache = Grid::get(opCtx)->catalogCache();
+        auto catalogCache =
+            Grid::get(opCtx)->isInitialized() ? Grid::get(opCtx)->catalogCache() : nullptr;
         auto sii = catalogCache
             ? uassertStatusOK(catalogCache->getCollectionRoutingInfo(opCtx, *nss)).sii
             : boost::none;
         // Attach IGNORED placement version to skip orphans (the range deleter will clear them up)
-        auto scopedRole = ScopedSetShardRole(
-            opCtx,
-            *nss,
-            ShardVersionFactory::make(ChunkVersion::IGNORED(),
-                                      sii ? boost::make_optional(sii->getCollectionIndexes())
-                                          : boost::none),
-            boost::none);
-        AutoGetCollection coll(opCtx, *nss, MODE_IX);
-        // The collection with `uuid` might be renamed before the lock and the wrong namespace would
-        // be locked and looked up so we double check here.
-        if (!coll || coll->uuid() != uuid)
+        const auto shardVersion = ShardVersionFactory::make(
+            ChunkVersion::IGNORED(),
+            sii ? boost::make_optional(sii->getCollectionIndexes()) : boost::none);
+        auto scopedRole = ScopedSetShardRole(opCtx, *nss, shardVersion, boost::none);
+        const auto coll =
+            acquireCollection(opCtx,
+                              CollectionAcquisitionRequest(*nss,
+                                                           {boost::none, shardVersion},
+                                                           repl::ReadConcernArgs::get(opCtx),
+                                                           AcquisitionPrerequisites::kWrite),
+                              MODE_IX);
+
+        // The collection with `uuid` might be renamed before the lock and the wrong namespace
+        // would be locked and looked up so we double check here.
+        if (!coll.exists() || coll.uuid() != uuid)
             return false;
 
-        // Allow TTL deletion on non-capped collections, and on capped clustered collections.
-        invariant(!coll->isCapped() || (coll->isCapped() && coll->isClustered()));
+        const auto& collectionPtr = coll.getCollectionPtr();
 
         if (MONGO_unlikely(hangTTLMonitorWithLock.shouldFail())) {
             LOGV2(22534,
@@ -576,32 +630,30 @@ bool TTLMonitor::_doTTLIndexDelete(OperationContext* opCtx,
         }
 
         std::shared_ptr<TenantMigrationAccessBlocker> mtab;
-        if (coll.getDb() &&
+        if (repl::ReplicationCoordinator::get(opCtx)->getSettings().isServerless() &&
             nullptr !=
                 (mtab = TenantMigrationAccessBlockerRegistry::get(opCtx->getServiceContext())
-                            .getTenantMigrationAccessBlockerForDbName(coll.getDb()->name(),
+                            .getTenantMigrationAccessBlockerForDbName(coll.nss().dbName(),
                                                                       MtabType::kRecipient)) &&
             mtab->checkIfShouldBlockTTL()) {
             LOGV2_DEBUG(53768,
                         1,
                         "Postpone TTL of DB because of active tenant migration",
                         "tenantMigrationAccessBlocker"_attr = mtab->getDebugInfo().jsonString(),
-                        "database"_attr = coll.getDb()->name());
+                        "database"_attr = coll.nss().dbName());
             return false;
         }
 
-        if (coll->getRequiresTimeseriesExtendedRangeSupport()) {
+        if (collectionPtr->getRequiresTimeseriesExtendedRangeSupport()) {
             return false;
         }
 
-        ResourceConsumption::ScopedMetricsCollector scopedMetrics(opCtx, nss->db().toString());
+        ResourceConsumption::ScopedMetricsCollector scopedMetrics(opCtx, nss->dbName());
 
-        const auto& collection = coll.getCollection();
         if (info.isClustered()) {
-            return _deleteExpiredWithCollscan(opCtx, ttlCollectionCache, collection);
+            return _deleteExpiredWithCollscan(opCtx, ttlCollectionCache, coll);
         } else {
-            return _deleteExpiredWithIndex(
-                opCtx, ttlCollectionCache, collection, info.getIndexName());
+            return _deleteExpiredWithIndex(opCtx, ttlCollectionCache, coll, info.getIndexName());
         }
     } catch (const ExceptionForCat<ErrorCategory::StaleShardVersionError>& ex) {
         // The TTL index tried to delete some information from a sharded collection
@@ -618,12 +670,8 @@ bool TTLMonitor::_doTTLIndexDelete(OperationContext* opCtx,
             auto executor = Grid::get(opCtx)->getExecutorPool()->getFixedExecutor();
             ExecutorFuture<void>(executor)
                 .then([serviceContext = opCtx->getServiceContext(), nss, staleInfo] {
-                    ThreadClient tc("TTLShardVersionRecovery", serviceContext);
-                    {
-                        stdx::lock_guard<Client> lk(*tc.get());
-                        tc->setSystemOperationKillableByStepdown(lk);
-                    }
-
+                    ThreadClient tc("TTLShardVersionRecovery",
+                                    serviceContext->getService(ClusterRole::ShardServer));
                     auto uniqueOpCtx = tc->makeOperationContext();
                     auto opCtx = uniqueOpCtx.get();
 
@@ -668,16 +716,17 @@ bool TTLMonitor::_doTTLIndexDelete(OperationContext* opCtx,
 
 bool TTLMonitor::_deleteExpiredWithIndex(OperationContext* opCtx,
                                          TTLCollectionCache* ttlCollectionCache,
-                                         const CollectionPtr& collection,
+                                         const CollectionAcquisition& collection,
                                          std::string indexName) {
-    if (!collection->isIndexPresent(indexName)) {
-        ttlCollectionCache->deregisterTTLIndexByName(collection->uuid(), indexName);
+    const auto& collectionPtr = collection.getCollectionPtr();
+    if (!collectionPtr->isIndexPresent(indexName)) {
+        ttlCollectionCache->deregisterTTLIndexByName(collection.uuid(), indexName);
         return false;
     }
 
-    BSONObj spec = collection->getIndexSpec(indexName);
+    BSONObj spec = collectionPtr->getIndexSpec(indexName);
     const IndexDescriptor* desc =
-        getValidTTLIndex(opCtx, ttlCollectionCache, collection, spec, indexName);
+        getValidTTLIndex(opCtx, ttlCollectionCache, collectionPtr, spec, indexName);
 
     if (!desc) {
         return false;
@@ -686,13 +735,13 @@ bool TTLMonitor::_deleteExpiredWithIndex(OperationContext* opCtx,
     LOGV2_DEBUG(22533,
                 1,
                 "running TTL job for index",
-                logAttrs(collection->ns()),
+                logAttrs(collection.nss()),
                 "key"_attr = desc->keyPattern(),
                 "name"_attr = indexName);
 
     auto expireAfterSeconds = spec[IndexDescriptor::kExpireAfterSecondsFieldName].safeNumberLong();
     const Date_t kDawnOfTime = Date_t::fromMillisSinceEpoch(std::numeric_limits<long long>::min());
-    const auto expirationDate = safeExpirationDate(opCtx, collection, expireAfterSeconds);
+    const auto expirationDate = safeExpirationDate(opCtx, collectionPtr, expireAfterSeconds);
     const BSONObj startKey = BSON("" << kDawnOfTime);
     const BSONObj endKey = BSON("" << expirationDate);
 
@@ -708,23 +757,24 @@ bool TTLMonitor::_deleteExpiredWithIndex(OperationContext* opCtx,
     // not actually expired when our snapshot changes during deletion.
     const char* keyFieldName = key.firstElement().fieldName();
     BSONObj query = BSON(keyFieldName << BSON("$gte" << kDawnOfTime << "$lte" << expirationDate));
-    auto findCommand = std::make_unique<FindCommandRequest>(collection->ns());
+    auto findCommand = std::make_unique<FindCommandRequest>(collection.nss());
     findCommand->setFilter(query);
-    auto canonicalQuery = CanonicalQuery::canonicalize(opCtx, std::move(findCommand));
-    invariant(canonicalQuery.getStatus());
+    auto canonicalQuery = std::make_unique<CanonicalQuery>(
+        CanonicalQueryParams{.expCtx = makeExpressionContext(opCtx, *findCommand),
+                             .parsedFind = ParsedFindCommandParams{std::move(findCommand)}});
 
     auto params = std::make_unique<DeleteStageParams>();
     params->isMulti = true;
-    params->canonicalQuery = canonicalQuery.getValue().get();
+    params->canonicalQuery = canonicalQuery.get();
 
     // Maintain a consistent view of whether batching is enabled - batching depends on
     // parameters that can be set at runtime, and it is illegal to try to get
     // BatchedDeleteStageStats from a non-batched delete.
-    const bool batchingEnabled = isBatchingEnabled(collection);
+    const bool batchingEnabled = isBatchingEnabled(collection.getCollectionPtr());
 
     Timer timer;
     auto exec = InternalPlanner::deleteWithIndexScan(opCtx,
-                                                     &collection,
+                                                     collection,
                                                      std::move(params),
                                                      desc,
                                                      startKey,
@@ -746,7 +796,7 @@ bool TTLMonitor::_deleteExpiredWithIndex(OperationContext* opCtx,
                 .first) {
             LOGV2(5479200,
                   "Deleted expired documents using index",
-                  logAttrs(collection->ns()),
+                  logAttrs(collection.nss()),
                   "index"_attr = indexName,
                   "numDeleted"_attr = numDeleted,
                   "duration"_attr = duration);
@@ -766,25 +816,26 @@ bool TTLMonitor::_deleteExpiredWithIndex(OperationContext* opCtx,
 
 bool TTLMonitor::_deleteExpiredWithCollscan(OperationContext* opCtx,
                                             TTLCollectionCache* ttlCollectionCache,
-                                            const CollectionPtr& collection) {
-    const auto& collOptions = collection->getCollectionOptions();
+                                            const CollectionAcquisition& collection) {
+    const auto& collectionPtr = collection.getCollectionPtr();
+    const auto& collOptions = collectionPtr->getCollectionOptions();
     uassert(5400701,
             "collection is not clustered but is described as being TTL",
             collOptions.clusteredIndex);
-    invariant(collection->isClustered());
+    invariant(collectionPtr->isClustered());
 
     auto expireAfterSeconds = collOptions.expireAfterSeconds;
     if (!expireAfterSeconds) {
-        ttlCollectionCache->deregisterTTLClusteredIndex(collection->uuid());
+        ttlCollectionCache->deregisterTTLClusteredIndex(collection.uuid());
         return false;
     }
 
-    LOGV2_DEBUG(5400704, 1, "running TTL job for clustered collection", logAttrs(collection->ns()));
+    LOGV2_DEBUG(5400704, 1, "running TTL job for clustered collection", logAttrs(collection.nss()));
 
-    const auto startId = makeCollScanStartBound(collection, Date_t{});
+    const auto startId = makeCollScanStartBound(collectionPtr, Date_t{});
 
-    const auto expirationDate = safeExpirationDate(opCtx, collection, *expireAfterSeconds);
-    const auto endId = makeCollScanEndBound(collection, expirationDate);
+    const auto expirationDate = safeExpirationDate(opCtx, collectionPtr, *expireAfterSeconds);
+    const auto endId = makeCollScanEndBound(collectionPtr, expirationDate);
 
     auto params = std::make_unique<DeleteStageParams>();
     params->isMulti = true;
@@ -792,14 +843,14 @@ bool TTLMonitor::_deleteExpiredWithCollscan(OperationContext* opCtx,
     // Maintain a consistent view of whether batching is enabled - batching depends on
     // parameters that can be set at runtime, and it is illegal to try to get
     // BatchedDeleteStageStats from a non-batched delete.
-    const bool batchingEnabled = isBatchingEnabled(collection);
+    const bool batchingEnabled = isBatchingEnabled(collection.getCollectionPtr());
 
     // Deletes records using a bounded collection scan from the beginning of time to the
     // expiration time (inclusive).
     Timer timer;
     auto exec = InternalPlanner::deleteWithCollectionScan(
         opCtx,
-        &collection,
+        collection,
         std::move(params),
         PlanYieldPolicy::YieldPolicy::YIELD_AUTO,
         InternalPlanner::Direction::FORWARD,
@@ -819,8 +870,8 @@ bool TTLMonitor::_deleteExpiredWithCollscan(OperationContext* opCtx,
                                         Milliseconds(serverGlobalParams.slowMS.load()))
                 .first) {
             LOGV2(5400702,
-                  "Deleted expired documents using collection scan",
-                  logAttrs(collection->ns()),
+                  "Deleted expired documents using clustered index scan",
+                  logAttrs(collection.nss()),
                   "numDeleted"_attr = numDeleted,
                   "duration"_attr = duration);
         }
@@ -853,8 +904,13 @@ void shutdownTTLMonitor(ServiceContext* serviceContext) {
 }
 
 void TTLMonitor::onStepUp() {
+    // A prepared transaction, which has a write to a collection "foo",  will reacquire an IX lock
+    // on "foo" prior to running this function. Thus processCollModCommand(), which will acquire
+    // MODE_X access to the collection, can't run synchronously here. Spawn and detach a thread to
+    // run it in background.
     stdx::thread([]() mutable {
-        ThreadClient tc("InvalidTTLIndexFixer", getGlobalServiceContext());
+        ThreadClient tc("InvalidTTLIndexFixer",
+                        getGlobalServiceContext()->getService(ClusterRole::ShardServer));
         AuthorizationSession::get(cc())->grantInternalAuthorization(&cc());
         const auto opCtxCtr = cc().makeOperationContext();
         auto opCtx = opCtxCtr.get();
@@ -862,9 +918,6 @@ void TTLMonitor::onStepUp() {
         auto ttlInfos = ttlCollectionCache.getTTLInfos();
         for (const auto& [uuid, infos] : ttlInfos) {
             auto collectionCatalog = CollectionCatalog::get(opCtx);
-            if (collectionCatalog->isCollectionAwaitingVisibility(uuid)) {
-                continue;
-            }
 
             // The collection was dropped.
             auto nss = collectionCatalog->lookupNSSByUUID(opCtx, uuid);
@@ -885,40 +938,80 @@ void TTLMonitor::onStepUp() {
                         continue;
                     }
 
-                    if (!info.isExpireAfterSecondsInvalid()) {
+                    // If the cached value is fine, we don't need to check any closer.
+                    if (!info.isExpireAfterSecondsInvalid() && !info.isExpireAfterSecondsNonInt()) {
                         continue;
                     }
 
+                    // At this point we need to look closer at the current version of the index
+                    // spec, and hold the lock to prevent concurrent collMod.
+                    const auto coll = acquireCollection(
+                        opCtx,
+                        CollectionAcquisitionRequest::fromOpCtx(
+                            opCtx, *nss, AcquisitionPrerequisites::OperationType::kWrite),
+                        MODE_X);
+
+                    if (!coll.exists() || coll.uuid() != uuid) {
+                        continue;
+                    }
+                    const auto& collectionPtr = coll.getCollectionPtr();
+
                     auto indexName = info.getIndexName();
-                    LOGV2(6847700,
-                          "Running collMod to fix TTL index with invalid 'expireAfterSeconds'.",
-                          "ns"_attr = *nss,
-                          "uuid"_attr = uuid,
-                          "name"_attr = indexName,
-                          "expireAfterSecondsNew"_attr =
-                              index_key_validate::kExpireAfterSecondsForInactiveTTLIndex);
+                    if (!collectionPtr->isIndexPresent(indexName)) {
+                        // This index must have been dropped, clean up the cache.
+                        ttlCollectionCache.deregisterTTLIndexByName(uuid, indexName);
+                        continue;
+                    }
 
-                    // Compose collMod command to amend 'expireAfterSeconds' to same value that
-                    // would be used by listIndexes() to convert a NaN value in the catalog.
-                    CollModIndex collModIndex;
-                    collModIndex.setName(StringData{indexName});
-                    collModIndex.setExpireAfterSeconds(mongo::durationCount<Seconds>(
-                        index_key_validate::kExpireAfterSecondsForInactiveTTLIndex));
-                    CollMod collModCmd{*nss};
-                    collModCmd.getCollModRequest().setIndex(collModIndex);
+                    BSONObj spec = collectionPtr->getIndexSpec(indexName);
+                    auto expireAfterSecondsElem =
+                        spec[IndexDescriptor::kExpireAfterSecondsFieldName];
+                    if (!expireAfterSecondsElem) {
+                        // This index must have been modified, clean up the cache.
+                        ttlCollectionCache.deregisterTTLIndexByName(uuid, indexName);
+                        continue;
+                    }
 
-                    // processCollModCommand() will acquire MODE_X access to the collection.
-                    BSONObjBuilder builder;
-                    uassertStatusOK(
-                        processCollModCommand(opCtx, {nss->dbName(), uuid}, collModCmd, &builder));
-                    auto result = builder.obj();
-                    LOGV2(6847701,
-                          "Successfully fixed TTL index with invalid 'expireAfterSeconds' using "
-                          "collMod",
-                          "ns"_attr = *nss,
-                          "uuid"_attr = uuid,
-                          "name"_attr = indexName,
-                          "result"_attr = result);
+                    if (auto correctedExpireAfterSeconds =
+                            index_key_validate::normalizeExpireAfterSeconds(
+                                expireAfterSecondsElem)) {
+                        if (MONGO_unlikely(TTLMonitorSkipRunningCollMod.shouldFail())) {
+                            LOGV2(90449,
+                                  "TTL monitor skipping running collMod to fix TTL index due to "
+                                  "failpoint");
+                            continue;
+                        }
+                        LOGV2(6847700,
+                              "Running collMod to fix TTL index with invalid "
+                              "'expireAfterSeconds'.",
+                              "ns"_attr = *nss,
+                              "uuid"_attr = uuid,
+                              "name"_attr = indexName,
+                              "expireAfterSecondsNew"_attr = correctedExpireAfterSeconds.value());
+
+                        // Compose collMod command to amend 'expireAfterSeconds' to same value that
+                        // would be used by listIndexes() to convert an invalid value in the
+                        // catalog.
+                        CollModIndex collModIndex;
+                        collModIndex.setName(StringData{indexName});
+                        collModIndex.setExpireAfterSeconds(correctedExpireAfterSeconds.value());
+
+                        CollMod collModCmd{*nss};
+                        collModCmd.getCollModRequest().setIndex(collModIndex);
+
+                        BSONObjBuilder builder;
+                        uassertStatusOK(processCollModCommand(
+                            opCtx, {nss->dbName(), uuid}, collModCmd, &coll, &builder));
+                        auto result = builder.obj();
+                        LOGV2(
+                            6847701,
+                            "Successfully fixed TTL index with invalid 'expireAfterSeconds' using "
+                            "collMod",
+                            "ns"_attr = *nss,
+                            "uuid"_attr = uuid,
+                            "name"_attr = indexName,
+                            "result"_attr = result);
+                    }
                 }
             } catch (const DBException& ex) {
                 LOGV2_ERROR(6835901,
