@@ -28,23 +28,23 @@
  */
 
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/db/ttl_collection_cache.h"
-
 #include <algorithm>
+#include <mutex>
+
+#include <absl/container/node_hash_map.h>
+#include <absl/meta/type_traits.h>
 
 #include "mongo/db/service_context.h"
+#include "mongo/db/ttl_collection_cache.h"
 #include "mongo/logv2/log.h"
-#include "mongo/util/fail_point.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/decorable.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
 
 
 namespace mongo {
 
-MONGO_FAIL_POINT_DEFINE(hangTTLCollectionCacheAfterRegisteringInfo);
-
 namespace {
 const auto getTTLCollectionCache = ServiceContext::declareDecoration<TTLCollectionCache>();
 }
@@ -58,17 +58,20 @@ void TTLCollectionCache::registerTTLInfo(UUID uuid, const Info& info) {
         stdx::lock_guard<Latch> lock(_ttlInfosLock);
         _ttlInfos[uuid].push_back(info);
     }
-
-    if (MONGO_unlikely(hangTTLCollectionCacheAfterRegisteringInfo.shouldFail())) {
-        LOGV2(4664000, "Hanging due to hangTTLCollectionCacheAfterRegisteringInfo fail point");
-        hangTTLCollectionCacheAfterRegisteringInfo.pauseWhileSet();
-    }
 }
 
 void TTLCollectionCache::_deregisterTTLInfo(UUID uuid, const Info& info) {
     stdx::lock_guard<Latch> lock(_ttlInfosLock);
     auto infoIt = _ttlInfos.find(uuid);
-    fassert(5400705, infoIt != _ttlInfos.end());
+    if (infoIt == _ttlInfos.end()) {
+        LOGV2_DEBUG(9150100,
+                    3,
+                    "Tried to deregister index from TTLCollectionCache with untracked UUID",
+                    "uuid"_attr = uuid,
+                    "indexName"_attr = info.getIndexName());
+        return;
+    }
+
     auto& [_, infoVec] = *infoIt;
 
     auto iter = infoVec.begin();
@@ -89,8 +92,16 @@ void TTLCollectionCache::_deregisterTTLInfo(UUID uuid, const Info& info) {
         });
     }
 
-    fassert(40220, iter != infoVec.end());
-    infoVec.erase(iter);
+    if (iter == infoVec.end()) {
+        LOGV2_DEBUG(9150101,
+                    3,
+                    "Tried to deregister untracked index from TTLCollectionCache",
+                    "uuid"_attr = uuid,
+                    "indexName"_attr = info.getIndexName());
+    } else {
+        infoVec.erase(iter);
+    }
+
     if (infoVec.empty()) {
         _ttlInfos.erase(infoIt);
     }
@@ -105,8 +116,9 @@ void TTLCollectionCache::deregisterTTLClusteredIndex(UUID uuid) {
                        TTLCollectionCache::Info{TTLCollectionCache::ClusteredId{}});
 }
 
-void TTLCollectionCache::unsetTTLIndexExpireAfterSecondsInvalid(UUID uuid,
-                                                                const IndexName& indexName) {
+void TTLCollectionCache::setTTLIndexExpireAfterSecondsType(UUID uuid,
+                                                           const IndexName& indexName,
+                                                           Info::ExpireAfterSecondsType type) {
     stdx::lock_guard<Latch> lock(_ttlInfosLock);
     auto infoIt = _ttlInfos.find(uuid);
     if (infoIt == _ttlInfos.end()) {
@@ -116,7 +128,7 @@ void TTLCollectionCache::unsetTTLIndexExpireAfterSecondsInvalid(UUID uuid,
     auto&& infoVec = infoIt->second;
     for (auto&& info : infoVec) {
         if (!info.isClustered() && info.getIndexName() == indexName) {
-            info.unsetExpireAfterSecondsInvalid();
+            info.setExpireAfterSecondsType(type);
             break;
         }
     }
