@@ -29,9 +29,15 @@
 
 #pragma once
 
+#include <cstdint>
 #include <memory>
 
 #include "mongo/base/clonable_ptr.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/mutable/const_element.h"
+#include "mongo/bson/mutable/element.h"
+#include "mongo/db/field_ref.h"
+#include "mongo/db/query/collation/collator_interface.h"
 #include "mongo/db/update/modifier_node.h"
 
 namespace mongo {
