@@ -27,14 +27,31 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <cstddef>
+#include <cstdint>
+#include <string>
+#include <utility>
+#include <variant>
+#include <vector>
 
+#include <absl/container/node_hash_map.h>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/data_type_endian.h"
+#include "mongo/base/data_view.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/bson/util/builder.h"
 #include "mongo/db/field_ref.h"
 #include "mongo/db/update/document_diff_applier.h"
 #include "mongo/db/update_index_data.h"
-
-#include "mongo/stdx/variant.h"
-#include "mongo/util/overloaded_visitor.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/overloaded_visitor.h"  // IWYU pragma: keep
+#include "mongo/util/str.h"
 #include "mongo/util/string_map.h"
 
 namespace mongo::doc_diff {
@@ -47,19 +64,22 @@ struct Insert {
     BSONElement newElt;
 };
 struct Delete {};
+struct Binary {
+    BSONElement newElt;
+};
 struct SubDiff {
     DiffType type() const {
-        return stdx::holds_alternative<DocumentDiffReader>(reader) ? DiffType::kDocument
-                                                                   : DiffType::kArray;
+        return holds_alternative<DocumentDiffReader>(reader) ? DiffType::kDocument
+                                                             : DiffType::kArray;
     }
 
-    stdx::variant<DocumentDiffReader, ArrayDiffReader> reader;
+    std::variant<DocumentDiffReader, ArrayDiffReader> reader;
 };
 
 // This struct stores the tables we build from an object diff before applying it.
 struct DocumentDiffTables {
     // Types of modifications that can be done to a field.
-    using FieldModification = stdx::variant<Delete, Update, Insert, SubDiff>;
+    using FieldModification = std::variant<Binary, Delete, Update, Insert, SubDiff>;
 
     /**
      * Inserts to the table and throws if the key exists already, which would mean that the
@@ -97,6 +117,12 @@ DocumentDiffTables buildObjDiffTables(DocumentDiffReader* reader,
         out.insertOnly = false;
     }
 
+    boost::optional<BSONElement> nextBinary;
+    while ((nextBinary = reader->nextBinary())) {
+        out.safeInsert(nextBinary->fieldNameStringData(), Binary{*nextBinary});
+        out.insertOnly = false;
+    }
+
     for (auto next = reader->nextSubDiff(); next; next = reader->nextSubDiff()) {
         out.safeInsert(next->first, SubDiff{next->second});
         out.insertOnly = false;
@@ -240,54 +266,55 @@ int32_t computeDamageOnObject(const BSONObj& preImageRoot,
             continue;
         }
 
-        stdx::visit(
-            OverloadedVisitor{
-                [&](Delete) {
-                    appendDamage(damages, 0, 0, targetOffset, elt.size());
-                    diffSize -= elt.size();
-                },
+        visit(OverloadedVisitor{
+                  [&](Delete) {
+                      appendDamage(damages, 0, 0, targetOffset, elt.size());
+                      diffSize -= elt.size();
+                  },
 
-                [&](const Update& update) {
-                    auto newElt = update.newElt;
-                    // Updates with the new BSONElement.
-                    appendDamage(
-                        damages, bufBuilder->len(), newElt.size(), targetOffset, elt.size());
-                    diffSize += newElt.size() - elt.size();
-                    bufBuilder->appendBuf(newElt.rawdata(), newElt.size());
-                    fieldsToSkipInserting.insert(elt.fieldNameStringData());
-                },
+                  [&](const Update& update) {
+                      auto newElt = update.newElt;
+                      // Updates with the new BSONElement.
+                      appendDamage(
+                          damages, bufBuilder->len(), newElt.size(), targetOffset, elt.size());
+                      diffSize += newElt.size() - elt.size();
+                      bufBuilder->appendBuf(newElt.rawdata(), newElt.size());
+                      fieldsToSkipInserting.insert(elt.fieldNameStringData());
+                  },
 
-                [&](const Insert&) {
-                    // Deletes the pre-image version of the field. We'll add it at the end.
-                    appendDamage(damages, 0, 0, targetOffset, elt.size());
-                    diffSize -= elt.size();
-                },
+                  [&](const Insert&) {
+                      // Deletes the pre-image version of the field. We'll add it at the end.
+                      appendDamage(damages, 0, 0, targetOffset, elt.size());
+                      diffSize -= elt.size();
+                  },
 
-                [&](const SubDiff& subDiff) {
-                    const auto type = subDiff.type();
-                    if (elt.type() == BSONType::Object && type == DiffType::kDocument) {
-                        addElementPrefix(elt, damages, bufBuilder, targetOffset);
-                        auto reader = stdx::get<DocumentDiffReader>(subDiff.reader);
-                        diffSize += computeDamageOnObject(preImageRoot,
-                                                          elt.embeddedObject(),
-                                                          &reader,
-                                                          damages,
-                                                          bufBuilder,
-                                                          offsetRoot + diffSize,
-                                                          mustCheckExistenceForInsertOperations);
-                    } else if (elt.type() == BSONType::Array && type == DiffType::kArray) {
-                        addElementPrefix(elt, damages, bufBuilder, targetOffset);
-                        auto reader = stdx::get<ArrayDiffReader>(subDiff.reader);
-                        diffSize += computeDamageOnArray(preImageRoot,
-                                                         elt.embeddedObject(),
-                                                         &reader,
-                                                         damages,
-                                                         bufBuilder,
-                                                         offsetRoot + diffSize,
-                                                         mustCheckExistenceForInsertOperations);
-                    }
-                }},
-            it->second);
+                  [&](const Binary&) { MONGO_UNIMPLEMENTED; },
+
+                  [&](const SubDiff& subDiff) {
+                      const auto type = subDiff.type();
+                      if (elt.type() == BSONType::Object && type == DiffType::kDocument) {
+                          addElementPrefix(elt, damages, bufBuilder, targetOffset);
+                          auto reader = get<DocumentDiffReader>(subDiff.reader);
+                          diffSize += computeDamageOnObject(preImageRoot,
+                                                            elt.embeddedObject(),
+                                                            &reader,
+                                                            damages,
+                                                            bufBuilder,
+                                                            offsetRoot + diffSize,
+                                                            mustCheckExistenceForInsertOperations);
+                      } else if (elt.type() == BSONType::Array && type == DiffType::kArray) {
+                          addElementPrefix(elt, damages, bufBuilder, targetOffset);
+                          auto reader = get<ArrayDiffReader>(subDiff.reader);
+                          diffSize += computeDamageOnArray(preImageRoot,
+                                                           elt.embeddedObject(),
+                                                           &reader,
+                                                           damages,
+                                                           bufBuilder,
+                                                           offsetRoot + diffSize,
+                                                           mustCheckExistenceForInsertOperations);
+                      }
+                  }},
+              it->second);
     }
 
     for (auto&& elt : tables.fieldsToInsert) {
@@ -322,63 +349,63 @@ int32_t computeDamageForArrayIndex(const BSONObj& preImageRoot,
                                    size_t offsetRoot,
                                    bool mustCheckExistenceForInsertOperations) {
     int32_t diffSize = 0;
-    stdx::visit(
-        OverloadedVisitor{
-            [&](const BSONElement& update) {
-                invariant(!update.eoo());
-                auto preValuePos = arrayPreImage.end()->rawdata();
-                auto targetSize = 0;
-                if (preImageValue) {
-                    preValuePos = preImageValue->rawdata();
-                    targetSize = preImageValue->size();
-                }
-                // The start of 'preImageValue' if existed otherwise the end of the array, with the
-                // offset from the updates made already.
-                auto targetOffset =
-                    targetOffsetInPostImage(preValuePos, preImageRoot.objdata(), offsetRoot);
-                // Updates with the new BSONElement except for the 'u' byte.
-                auto sourceSize = update.size() - 1;
-                appendDamage(damages, bufBuilder->len(), sourceSize, targetOffset, targetSize);
-                diffSize += sourceSize - targetSize;
-                auto source = update.rawdata();
-                appendTypeByte(bufBuilder, *source++);
-                // Skips the byte of 'u' and appends the data after the type byte.
-                bufBuilder->appendBuf(source + 1, sourceSize - 1);
-            },
-            [&](auto reader) {
-                if (preImageValue) {
-                    auto targetOffset = targetOffsetInPostImage(
-                        preImageValue->rawdata(), preImageRoot.objdata(), offsetRoot);
-                    if constexpr (std::is_same_v<decltype(reader), ArrayDiffReader>) {
-                        if (preImageValue->type() == BSONType::Array) {
-                            addElementPrefix(*preImageValue, damages, bufBuilder, targetOffset);
-                            diffSize += computeDamageOnArray(preImageRoot,
-                                                             preImageValue->embeddedObject(),
-                                                             &reader,
-                                                             damages,
-                                                             bufBuilder,
-                                                             offsetRoot,
-                                                             mustCheckExistenceForInsertOperations);
-                            return;
-                        }
-                    } else if constexpr (std::is_same_v<decltype(reader), DocumentDiffReader>) {
-                        if (preImageValue->type() == BSONType::Object) {
-                            addElementPrefix(*preImageValue, damages, bufBuilder, targetOffset);
-                            diffSize +=
-                                computeDamageOnObject(preImageRoot,
-                                                      preImageValue->embeddedObject(),
-                                                      &reader,
-                                                      damages,
-                                                      bufBuilder,
-                                                      offsetRoot,
-                                                      mustCheckExistenceForInsertOperations);
-                            return;
-                        }
-                    }
-                }
-            },
-        },
-        modification);
+    visit(OverloadedVisitor{
+              [&](const BSONElement& update) {
+                  invariant(!update.eoo());
+                  auto preValuePos = arrayPreImage.end()->rawdata();
+                  auto targetSize = 0;
+                  if (preImageValue) {
+                      preValuePos = preImageValue->rawdata();
+                      targetSize = preImageValue->size();
+                  }
+                  // The start of 'preImageValue' if existed otherwise the end of the array, with
+                  // the offset from the updates made already.
+                  auto targetOffset =
+                      targetOffsetInPostImage(preValuePos, preImageRoot.objdata(), offsetRoot);
+                  // Updates with the new BSONElement except for the 'u' byte.
+                  auto sourceSize = update.size() - 1;
+                  appendDamage(damages, bufBuilder->len(), sourceSize, targetOffset, targetSize);
+                  diffSize += sourceSize - targetSize;
+                  auto source = update.rawdata();
+                  appendTypeByte(bufBuilder, *source++);
+                  // Skips the byte of 'u' and appends the data after the type byte.
+                  bufBuilder->appendBuf(source + 1, sourceSize - 1);
+              },
+              [&](auto reader) {
+                  if (preImageValue) {
+                      auto targetOffset = targetOffsetInPostImage(
+                          preImageValue->rawdata(), preImageRoot.objdata(), offsetRoot);
+                      if constexpr (std::is_same_v<decltype(reader), ArrayDiffReader>) {
+                          if (preImageValue->type() == BSONType::Array) {
+                              addElementPrefix(*preImageValue, damages, bufBuilder, targetOffset);
+                              diffSize +=
+                                  computeDamageOnArray(preImageRoot,
+                                                       preImageValue->embeddedObject(),
+                                                       &reader,
+                                                       damages,
+                                                       bufBuilder,
+                                                       offsetRoot,
+                                                       mustCheckExistenceForInsertOperations);
+                              return;
+                          }
+                      } else if constexpr (std::is_same_v<decltype(reader), DocumentDiffReader>) {
+                          if (preImageValue->type() == BSONType::Object) {
+                              addElementPrefix(*preImageValue, damages, bufBuilder, targetOffset);
+                              diffSize +=
+                                  computeDamageOnObject(preImageRoot,
+                                                        preImageValue->embeddedObject(),
+                                                        &reader,
+                                                        damages,
+                                                        bufBuilder,
+                                                        offsetRoot,
+                                                        mustCheckExistenceForInsertOperations);
+                              return;
+                          }
+                      }
+                  }
+              },
+          },
+          modification);
     return diffSize;
 }
 
@@ -508,7 +535,7 @@ public:
             }
             FieldRef::FieldRefTempAppend tempAppend(*path, elt.fieldNameStringData());
 
-            stdx::visit(
+            visit(
                 OverloadedVisitor{
                     [this, &path](Delete) {
                         // Do not append anything.
@@ -523,20 +550,55 @@ public:
                         // Skip the pre-image version of the field. We'll add it at the end.
                     },
 
+                    [this, &path, &builder, &elt, &fieldsToSkipInserting](const Binary& binary) {
+                        // Applies the binary diff to the BSONColumn.
+                        invariant(elt.binDataType() == BinDataType::Column);
+                        invariant(binary.newElt.isABSONObj());
+
+                        const BSONObj diffObj = binary.newElt.Obj();
+                        const int diffOffset = diffObj.getIntField("o");
+
+                        int diffLen = 0;
+                        const char* diffData = diffObj.getField("d").binData(diffLen);
+
+                        int currLen = 0;
+                        const char* currData = elt.binData(currLen);
+
+                        if (currLen >= diffOffset) {
+                            int newLen = diffOffset + diffLen;
+                            std::vector<char> newData;
+                            newData.reserve(newLen);
+
+                            std::copy(currData, currData + diffOffset, std::back_inserter(newData));
+                            std::copy(diffData, diffData + diffLen, std::back_inserter(newData));
+
+                            BSONBinData postBinData(&newData[0], newLen, BinDataType::Column);
+                            builder->append(binary.newElt.fieldName(), postBinData);
+                        } else {
+                            // Offset is larger than the length of the preimage. This means that we
+                            // are re-applying this diff and some future oplog entry will shrink
+                            // this binary. It is not possible to apply this diff. We make this a
+                            // no-op for idempotency.
+                            builder->append(elt);
+                        }
+
+                        fieldsToSkipInserting.insert(elt.fieldNameStringData());
+                    },
+
                     [this, &builder, &elt, &path](const SubDiff& subDiff) {
                         const auto type = subDiff.type();
                         if (elt.type() == BSONType::Object && type == DiffType::kDocument) {
                             BSONObjBuilder subBob(builder->subobjStart(elt.fieldNameStringData()));
-                            auto reader = stdx::get<DocumentDiffReader>(subDiff.reader);
+                            auto reader = get<DocumentDiffReader>(subDiff.reader);
                             applyDiffToObject(elt.embeddedObject(), path, &reader, &subBob);
                         } else if (elt.type() == BSONType::Array && type == DiffType::kArray) {
                             BSONArrayBuilder subBob(
                                 builder->subarrayStart(elt.fieldNameStringData()));
-                            auto reader = stdx::get<ArrayDiffReader>(subDiff.reader);
+                            auto reader = get<ArrayDiffReader>(subDiff.reader);
                             applyDiffToArray(elt.embeddedObject(), path, &reader, &subBob);
                         } else {
-                            // There's a type mismatch. The diff was expecting one type but the pre
-                            // image contains a value of a different type. This means we are
+                            // There's a type mismatch. The diff was expecting one type but the
+                            // pre image contains a value of a different type. This means we are
                             // re-applying a diff.
 
                             // There must be some future operation which changed the type of this
@@ -595,7 +657,7 @@ private:
                                      FieldRef* path,
                                      const ArrayDiffReader::ArrayModification& modification,
                                      BSONArrayBuilder* builder) {
-        stdx::visit(
+        visit(
             OverloadedVisitor{
                 [this, &path, builder](const BSONElement& update) {
                     invariant(!update.eoo());
@@ -679,7 +741,8 @@ private:
 
 BSONObj applyDiff(const BSONObj& pre,
                   const Diff& diff,
-                  bool mustCheckExistenceForInsertOperations) {
+                  bool mustCheckExistenceForInsertOperations,
+                  VerifierFunc verifierFunction) {
     DocumentDiffReader reader(diff);
     BSONObjBuilder out;
     DiffApplier applier(mustCheckExistenceForInsertOperations);
@@ -694,7 +757,11 @@ BSONObj applyDiff(const BSONObj& pre,
     out.bb().reserveBytes(estimatedSize);
     out.bb().claimReservedBytes(estimatedSize);
     applier.applyDiffToObject(pre, &path, &reader, &out);
-    return out.obj();
+    auto obj = out.obj();
+    if (verifierFunction) {
+        verifierFunction(obj, pre);
+    }
+    return obj;
 }
 
 DamagesOutput computeDamages(const BSONObj& pre,
