@@ -27,13 +27,27 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <algorithm>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <cstddef>
+#include <memory>
 
-#include "mongo/db/update/modifier_node.h"
+#include <boost/optional/optional.hpp>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/bson/mutable/document.h"
 #include "mongo/db/bson/dotted_path_support.h"
+#include "mongo/db/field_ref_set.h"
+#include "mongo/db/update/modifier_node.h"
 #include "mongo/db/update/path_support.h"
 #include "mongo/db/update/storage_validation.h"
+#include "mongo/db/update/update_executor.h"
+#include "mongo/util/str.h"
 
 namespace mongo {
 
@@ -171,7 +185,7 @@ UpdateExecutor::ApplyResult ModifierNode::applyToExistingElement(
         BSONObj original = applyParams.element.getDocument().getObject();
         updateResult = updateExistingElement(&applyParams.element,
                                              updateNodeApplyParams.pathTaken->fieldRef());
-        if (updateResult == ModifyResult::kNoOp) {
+        if (updateResult.type == ModifyResult::kNoOp) {
             return ApplyResult::noopResult();
         }
         checkImmutablePathsNotModifiedFromOriginal(applyParams.element,
@@ -181,14 +195,14 @@ UpdateExecutor::ApplyResult ModifierNode::applyToExistingElement(
     } else {
         updateResult = updateExistingElement(&applyParams.element,
                                              updateNodeApplyParams.pathTaken->fieldRef());
-        if (updateResult == ModifyResult::kNoOp) {
+        if (updateResult.type == ModifyResult::kNoOp) {
             return ApplyResult::noopResult();
         }
         checkImmutablePathsNotModified(applyParams.element,
                                        updateNodeApplyParams.pathTaken->fieldRef(),
                                        applyParams.immutablePaths);
     }
-    invariant(updateResult != ModifyResult::kCreated);
+    invariant(updateResult.type != ModifyResult::kCreated);
 
     ApplyResult applyResult;
 
@@ -361,6 +375,7 @@ void ModifierNode::validateUpdate(mutablebson::ConstElement updatedElement,
                                      recursionLevel,
                                      false, /* allowTopLevelDollarPrefixedFields */
                                      validateForStorage,
+                                     false, /* isEmbeddedInIdField */
                                      containsDotsAndDollarsField);
 }
 
@@ -370,9 +385,9 @@ void ModifierNode::logUpdate(LogBuilderInterface* logBuilder,
                              ModifyResult modifyResult,
                              boost::optional<int> createdFieldIdx) const {
     invariant(logBuilder);
-    invariant(modifyResult == ModifyResult::kNormalUpdate ||
-              modifyResult == ModifyResult::kCreated);
-    if (modifyResult == ModifyResult::kCreated) {
+    invariant(modifyResult.type == ModifyResult::kNormalUpdate ||
+              modifyResult.type == ModifyResult::kCreated);
+    if (modifyResult.type == ModifyResult::kCreated) {
         invariant(createdFieldIdx);
         uassertStatusOK(logBuilder->logCreatedField(pathTaken, *createdFieldIdx, element));
     } else {
