@@ -35,6 +35,8 @@
 #include <utility>
 #include <vector>
 
+#include "mongo/bson/bsonobj.h"
+#include "mongo/db/exec/document_value/value.h"
 #include "mongo/db/update/update_executor.h"
 
 namespace mongo {
