@@ -29,27 +29,37 @@
 
 #include "mongo/db/update/path_support.h"
 
-#include <cstdint>
 #include <memory>
 #include <string>
+#include <utility>
 #include <vector>
 
+#include <absl/container/node_hash_map.h>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+
 #include "mongo/base/error_codes.h"
-#include "mongo/base/simple_string_data_comparator.h"
 #include "mongo/base/status.h"
 #include "mongo/base/string_data.h"
+#include "mongo/base/string_data_comparator.h"
 #include "mongo/bson/bsonelement_comparator.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/bson/json.h"
 #include "mongo/bson/mutable/algorithm.h"
+#include "mongo/bson/mutable/const_element.h"
 #include "mongo/bson/mutable/document.h"
-#include "mongo/bson/mutable/mutable_bson_test_utils.h"
 #include "mongo/db/field_ref.h"
-#include "mongo/db/jsobj.h"
-#include "mongo/db/json.h"
 #include "mongo/db/matcher/expression.h"
 #include "mongo/db/matcher/expression_leaf.h"
 #include "mongo/db/matcher/expression_parser.h"
+#include "mongo/db/pipeline/expression_context.h"
 #include "mongo/db/pipeline/expression_context_for_test.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/stdx/type_traits.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/intrusive_counter.h"
 #include "mongo/util/str.h"
 
 namespace {
@@ -134,7 +144,7 @@ class SimpleDoc : public mongo::unittest::Test {
 public:
     SimpleDoc() : _doc() {}
 
-    virtual void setUp() {
+    void setUp() override {
         // {a: 1}
         ASSERT_OK(root().appendInt("a", 1));
     }
@@ -240,7 +250,7 @@ class NestedDoc : public mongo::unittest::Test {
 public:
     NestedDoc() : _doc() {}
 
-    virtual void setUp() {
+    void setUp() override {
         // {a: {b: {c: 1}}}
         Element elemA = _doc.makeElementObject("a");
         ASSERT_TRUE(elemA.ok());
@@ -363,7 +373,7 @@ class ArrayDoc : public mongo::unittest::Test {
 public:
     ArrayDoc() : _doc() {}
 
-    virtual void setUp() {
+    void setUp() override {
         // {a: []}
         Element elemA = _doc.makeElementArray("a");
         ASSERT_TRUE(elemA.ok());
@@ -637,7 +647,7 @@ static void assertContains(const EqualityMatches& equalities, const BSONObj& wra
     }
 
     BSONElementComparator eltCmp(BSONElementComparator::FieldNamesMode::kIgnore,
-                                 &SimpleStringDataComparator::kInstance);
+                                 &simpleStringDataComparator);
     if (eltCmp.evaluate(it->second->getData() != value)) {
         FAIL(stream() << "Equality match at path \"" << path << "\" contains value "
                       << it->second->getData() << ", not value " << value);
@@ -934,7 +944,7 @@ static void assertParent(const EqualityMatches& equalities,
     }
 
     BSONElementComparator eltCmp(BSONElementComparator::FieldNamesMode::kIgnore,
-                                 &SimpleStringDataComparator::kInstance);
+                                 &simpleStringDataComparator);
     if (eltCmp.evaluate(parentEl != value)) {
         FAIL(stream() << "Equality match parent for \"" << pathStr << "\" at path \"" << parentPath
                       << "\" contains value " << parentEl << ", not value " << value);
