@@ -27,16 +27,23 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <string>
 
-#include "mongo/db/update/pop_node.h"
+#include <boost/smart_ptr/intrusive_ptr.hpp>
 
+#include "mongo/base/error_codes.h"
 #include "mongo/bson/json.h"
-#include "mongo/bson/mutable/mutable_bson_test_utils.h"
+#include "mongo/bson/mutable/document.h"
 #include "mongo/db/pipeline/expression_context_for_test.h"
+#include "mongo/db/update/pop_node.h"
+#include "mongo/db/update/update_executor.h"
 #include "mongo/db/update/update_node_test_fixture.h"
+#include "mongo/unittest/assert.h"
 #include "mongo/unittest/death_test.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/intrusive_counter.h"
+#include "mongo/util/safe_num.h"
 
 namespace mongo {
 namespace {
@@ -113,6 +120,7 @@ TEST_F(PopNodeTest, NoopWhenFirstPathComponentDoesNotExist) {
     addIndexedPath("a.b");
     auto result = popNode.apply(getApplyParams(doc.root()), getUpdateNodeApplyParams());
     ASSERT_TRUE(result.noop);
+    ASSERT_FALSE(getIndexAffectedFromLogEntry(result.oplogEntry));
     ASSERT_EQUALS(fromjson("{b: [1, 2, 3]}"), doc);
 
     assertOplogEntryIsNoop();
@@ -131,6 +139,7 @@ TEST_F(PopNodeTest, NoopWhenPathPartiallyExists) {
     addIndexedPath("a.b.c");
     auto result = popNode.apply(getApplyParams(doc.root()["a"]), getUpdateNodeApplyParams());
     ASSERT_TRUE(result.noop);
+    ASSERT_FALSE(getIndexAffectedFromLogEntry(result.oplogEntry));
     ASSERT_EQUALS(fromjson("{a: {}}"), doc);
 
     assertOplogEntryIsNoop();
@@ -149,6 +158,7 @@ TEST_F(PopNodeTest, NoopWhenNumericalPathComponentExceedsArrayLength) {
     addIndexedPath("a.0");
     auto result = popNode.apply(getApplyParams(doc.root()["a"]), getUpdateNodeApplyParams());
     ASSERT_TRUE(result.noop);
+    ASSERT_FALSE(getIndexAffectedFromLogEntry(result.oplogEntry));
     ASSERT_EQUALS(fromjson("{a: []}"), doc);
 
     assertOplogEntryIsNoop();
@@ -213,6 +223,7 @@ TEST_F(PopNodeTest, NoopWhenPathContainsAnEmptyArray) {
     addIndexedPath("a.b");
     auto result = popNode.apply(getApplyParams(doc.root()["a"]["b"]), getUpdateNodeApplyParams());
     ASSERT_TRUE(result.noop);
+    ASSERT_FALSE(getIndexAffectedFromLogEntry(result.oplogEntry));
     ASSERT_EQUALS(fromjson("{a: {b: []}}"), doc);
     assertOplogEntryIsNoop();
     ASSERT_EQUALS("{a.b}", getModifiedPaths());
@@ -230,6 +241,7 @@ TEST_F(PopNodeTest, PopsSingleElementFromTheBack) {
     addIndexedPath("a.b");
     auto result = popNode.apply(getApplyParams(doc.root()["a"]["b"]), getUpdateNodeApplyParams());
     ASSERT_FALSE(result.noop);
+    ASSERT_TRUE(getIndexAffectedFromLogEntry());
     ASSERT_EQUALS(fromjson("{a: {b: []}}"), doc);
 
     assertOplogEntry(fromjson("{$v: 2, diff: {sa: {u: {b: []}}}}"));
@@ -248,6 +260,7 @@ TEST_F(PopNodeTest, PopsSingleElementFromTheFront) {
     addIndexedPath("a");
     auto result = popNode.apply(getApplyParams(doc.root()["a"]["b"]), getUpdateNodeApplyParams());
     ASSERT_FALSE(result.noop);
+    ASSERT_TRUE(getIndexAffectedFromLogEntry());
     ASSERT_EQUALS(fromjson("{a: {b: []}}"), doc);
 
     assertOplogEntry(fromjson("{$v: 2, diff: {sa: {u: {b: []}}}}"));
@@ -266,6 +279,7 @@ TEST_F(PopNodeTest, PopsFromTheBackOfMultiElementArray) {
     addIndexedPath("a.b.c");
     auto result = popNode.apply(getApplyParams(doc.root()["a"]["b"]), getUpdateNodeApplyParams());
     ASSERT_FALSE(result.noop);
+    ASSERT_TRUE(getIndexAffectedFromLogEntry());
     ASSERT_EQUALS(fromjson("{a: {b: [1, 2]}}"), doc);
 
     assertOplogEntry(fromjson("{$v: 2, diff: {sa: {u: {b: [1, 2]}}}}"));
@@ -284,6 +298,7 @@ TEST_F(PopNodeTest, PopsFromTheFrontOfMultiElementArray) {
     addIndexedPath("a.b");
     auto result = popNode.apply(getApplyParams(doc.root()["a"]["b"]), getUpdateNodeApplyParams());
     ASSERT_FALSE(result.noop);
+    ASSERT_TRUE(getIndexAffectedFromLogEntry());
     ASSERT_EQUALS(fromjson("{a: {b: [2, 3]}}"), doc);
 
     assertOplogEntry(fromjson("{$v: 2, diff: {sa: {u: {b: [2, 3]}}}}"));
@@ -302,6 +317,7 @@ TEST_F(PopNodeTest, PopsFromTheFrontOfMultiElementArrayWithoutAffectingIndexes)
     addIndexedPath("unrelated.path");
     auto result = popNode.apply(getApplyParams(doc.root()["a"]["b"]), getUpdateNodeApplyParams());
     ASSERT_FALSE(result.noop);
+    ASSERT_FALSE(getIndexAffectedFromLogEntry());
     ASSERT_EQUALS(fromjson("{a: {b: [2, 3]}}"), doc);
 
     assertOplogEntry(fromjson("{$v: 2, diff: {sa: {u: {b: [2, 3]}}}}"));
@@ -319,6 +335,7 @@ TEST_F(PopNodeTest, SucceedsWithNullUpdateIndexData) {
     setPathTaken(makeRuntimeUpdatePathForTest("a.b"));
     auto result = popNode.apply(getApplyParams(doc.root()["a"]["b"]), getUpdateNodeApplyParams());
     ASSERT_FALSE(result.noop);
+    ASSERT_FALSE(getIndexAffectedFromLogEntry());
     ASSERT_EQUALS(fromjson("{a: {b: [1, 2]}}"), doc);
 
     assertOplogEntry(fromjson("{$v: 2, diff: {sa: {u: {b: [1, 2]}}}}"));
@@ -410,6 +427,7 @@ TEST_F(PopNodeTest, NoopOnImmutablePathSucceeds) {
     addIndexedPath("a.b");
     auto result = popNode.apply(getApplyParams(doc.root()["a"]["b"]), getUpdateNodeApplyParams());
     ASSERT_TRUE(result.noop);
+    ASSERT_FALSE(getIndexAffectedFromLogEntry());
     ASSERT_EQUALS(fromjson("{a: {b: []}}"), doc);
 
     assertOplogEntryIsNoop();
