@@ -29,6 +29,10 @@
 
 #pragma once
 
+#include <cstdint>
+
+#include "mongo/base/status.h"
+#include "mongo/bson/bsonelement.h"
 #include "mongo/bson/mutable/element.h"
 
 namespace mongo {
@@ -73,12 +77,16 @@ void scanDocument(const mutablebson::Document& doc,
  *
  * 'containsDotsAndDollarsField' is set to true if there exists any field name containing '.'/'$'
  * during validation.
+ *
+ * 'isEmbeddedInIdField' is set to true if the element is embedded inside an _id field. This allows
+ * to reject $-prefixed fields at all levels under an _id field.
  */
 void scanDocument(mutablebson::ConstElement elem,
                   bool deep,
                   std::uint32_t recursionLevel,
                   bool allowTopLevelDollarPrefixes,
                   bool shouldValidate,
+                  bool isEmbeddedInIdField,
                   bool* containsDotsAndDollarsField);
 
 }  // namespace storage_validation
