@@ -28,20 +28,58 @@
  */
 
 
-#include "mongo/platform/basic.h"
+#include <boost/cstdint.hpp>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr.hpp>
+#include <cstdint>
+#include <map>
+#include <memory>
+#include <mutex>
+#include <string>
+#include <tuple>
+#include <type_traits>
+#include <utility>
+#include <vector>
 
+#include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/client/dbclient_cursor.h"
+#include "mongo/db/admission/execution_admission_context.h"
+#include "mongo/db/client.h"
+#include "mongo/db/cluster_role.h"
+#include "mongo/db/dbdirectclient.h"
+#include "mongo/db/logical_time.h"
 #include "mongo/db/logical_time_validator.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/persistent_task_store.h"
+#include "mongo/db/query/find_command.h"
+#include "mongo/db/repl/optime.h"
 #include "mongo/db/repl/replica_set_aware_service.h"
 #include "mongo/db/repl/replication_coordinator.h"
-#include "mongo/db/repl/storage_interface.h"
-#include "mongo/db/s/sharding_state.h"
 #include "mongo/db/s/topology_time_ticker.h"
+#include "mongo/db/server_options.h"
+#include "mongo/db/service_context.h"
+#include "mongo/db/vector_clock.h"
 #include "mongo/db/vector_clock_document_gen.h"
+#include "mongo/db/vector_clock_mongod.h"
 #include "mongo/db/vector_clock_mutable.h"
-#include "mongo/executor/scoped_task_executor.h"
+#include "mongo/executor/task_executor_pool.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/s/catalog/type_shard.h"
 #include "mongo/s/grid.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/decorable.h"
+#include "mongo/util/future.h"
+#include "mongo/util/future_impl.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
 
@@ -49,130 +87,6 @@
 namespace mongo {
 namespace {
 
-class VectorClockMongoD : public VectorClockMutable,
-                          public ReplicaSetAwareService<VectorClockMongoD>,
-                          public std::enable_shared_from_this<VectorClockMongoD> {
-    VectorClockMongoD(const VectorClockMongoD&) = delete;
-    VectorClockMongoD& operator=(const VectorClockMongoD&) = delete;
-
-public:
-    static VectorClockMongoD* get(ServiceContext* serviceContext);
-
-    VectorClockMongoD(ServiceContext* ctx);
-    virtual ~VectorClockMongoD();
-
-private:
-    // VectorClock methods implementation
-
-    ComponentSet _gossipOutInternal() const override;
-    ComponentSet _gossipInInternal() const override;
-
-    bool _permitGossipClusterTimeWithExternalClients() const override {
-        // If this node is in an unreadable state, skip gossiping because it may require reading a
-        // signing key from the keys collection.
-        auto replicationCoordinator = repl::ReplicationCoordinator::get(_service);
-        return !replicationCoordinator ||
-            (replicationCoordinator->getReplicationMode() ==
-                 repl::ReplicationCoordinator::modeReplSet &&
-             // Check repl status without locks to prevent deadlocks. This is a best effort check
-             // as the repl state can change right after this check even when inspected under a
-             // lock or mutex.
-             replicationCoordinator->isInPrimaryOrSecondaryState_UNSAFE());
-    }
-
-    bool _permitRefreshDuringGossipOut() const override {
-        return false;
-    }
-
-    // VectorClockMutable methods implementation
-
-    SharedSemiFuture<void> waitForDurableConfigTime() override;
-    SharedSemiFuture<void> waitForDurableTopologyTime() override;
-    SharedSemiFuture<void> waitForDurable() override;
-    VectorClock::VectorTime recoverDirect(OperationContext* opCtx) override;
-
-    LogicalTime _tick(Component component, uint64_t nTicks) override;
-    void _tickTo(Component component, LogicalTime newTime) override;
-
-    // ReplicaSetAwareService methods implementation
-
-    void onStartup(OperationContext* opCtx) override {}
-    void onSetCurrentConfig(OperationContext* opCtx) override {}
-    void onInitialDataAvailable(OperationContext* opCtx, bool isMajorityDataAvailable) override;
-    void onShutdown() override;
-    void onStepUpBegin(OperationContext* opCtx, long long term) override;
-    void onStepUpComplete(OperationContext* opCtx, long long term) override {}
-    void onStepDown() override;
-    void onBecomeArbiter() override;
-    inline std::string getServiceName() const override final {
-        return "VectorClockMongoD";
-    }
-
-    /**
-     * Structure used as keys for the map of waiters for VectorClock durability.
-     */
-    struct ComparableVectorTime {
-        bool operator<(const ComparableVectorTime& other) const {
-            return vt.configTime() < other.vt.configTime() ||
-                vt.topologyTime() < other.vt.topologyTime();
-        }
-        bool operator>(const ComparableVectorTime& other) const {
-            return vt.configTime() > other.vt.configTime() ||
-                vt.topologyTime() > other.vt.topologyTime();
-        }
-        bool operator==(const ComparableVectorTime& other) const {
-            return vt.configTime() == other.vt.configTime() &&
-                vt.topologyTime() == other.vt.topologyTime();
-        }
-
-        VectorTime vt;
-    };
-
-    /**
-     * The way the VectorClock durability works is by maintaining an `_queue` of callers, which wait
-     * for a particular VectorTime to become durable.
-     *
-     * When the queue is empty, there is no persistence activity going on. The first caller, who
-     * finds `_loopScheduled` to be false, will set it to true, indicating it will schedule the
-     * asynchronous persistence task. The asynchronous persistence task is effectively the following
-     * loop:
-     *
-     *  while (!_queue.empty()) {
-     *      timeToPersist = getTime();
-     *      persistTime(timeToPersist);
-     *      _durableTime = timeToPersist;
-     *      // Notify entries in _queue, whose time is <= _durableTime and remove them
-     *  }
-     */
-    SharedSemiFuture<void> _enqueueWaiterAndScheduleLoopIfNeeded(stdx::unique_lock<Mutex> ul,
-                                                                 VectorTime time);
-    Future<void> _doWhileQueueNotEmptyOrError();
-
-    // Protects the shared state below
-    Mutex _mutex = MONGO_MAKE_LATCH("VectorClockMongoD::_mutex");
-
-    // If set to true, means that another operation already scheduled the `_queue` draining loop, if
-    // false it means that this operation must do it
-    bool _loopScheduled{false};
-
-    // This value is only boost::none once, just after the object is constructuted. From the moment,
-    // the first operation schedules the `_queue`-draining loop, it will be set to a future, which
-    // will be signaled when the previously-scheduled `_queue` draining loop completes.
-    boost::optional<Future<void>> _currentWhileLoop;
-
-    // If boost::none, means the durable time needs to be recovered from disk, otherwise contains
-    // the latest-known durable time
-    boost::optional<VectorTime> _durableTime;
-
-    // Queue ordered in increasing order of the VectorTimes, which are waiting to be persisted
-    using Queue = std::map<ComparableVectorTime, std::unique_ptr<SharedPromise<void>>>;
-    Queue _queue;
-
-    ServiceContext* _serviceContext;
-
-    AtomicWord<bool> _shutdownInitiated{false};
-};
-
 const auto vectorClockMongoDDecoration =
     ServiceContext::declareDecoration<std::shared_ptr<VectorClockMongoD>>();
 
@@ -180,13 +94,12 @@ const ReplicaSetAwareServiceRegistry::Registerer<VectorClockMongoD>
     vectorClockMongoDServiceRegisterer("VectorClockMongoD-ReplicaSetAwareServiceRegistration");
 
 const ServiceContext::ConstructorActionRegisterer vectorClockMongoDRegisterer(
-    "VectorClockMongoD-VectorClockRegistration",
-    {},
-    [](ServiceContext* service) {
+    "VectorClockMongoD", {"VectorClock"}, [](ServiceContext* service) {
         VectorClockMongoD::registerVectorClockOnServiceContext(service,
                                                                VectorClockMongoD::get(service));
-    },
-    {});
+    });
+
+}  // namespace
 
 VectorClockMongoD* VectorClockMongoD::get(ServiceContext* serviceContext) {
     auto& clock = vectorClockMongoDDecoration(serviceContext);
@@ -198,8 +111,6 @@ VectorClockMongoD* VectorClockMongoD::get(ServiceContext* serviceContext) {
 
 VectorClockMongoD::VectorClockMongoD(ServiceContext* ctx) : _serviceContext(ctx) {}
 
-VectorClockMongoD::~VectorClockMongoD() = default;
-
 void VectorClockMongoD::onStepUpBegin(OperationContext* opCtx, long long term) {
     stdx::lock_guard lg(_mutex);
     _durableTime.reset();
@@ -210,6 +121,16 @@ void VectorClockMongoD::onStepDown() {
     _durableTime.reset();
 }
 
+void VectorClockMongoD::onShutdown() {
+    stdx::lock_guard lg(_mutex);
+    _shutdownInitiated.store(true);
+    for (auto& [_, promise] : _queue) {
+        promise->setError(
+            Status(ErrorCodes::ShutdownInProgress, "Not persisting vector clock due to shutdown"));
+    }
+    _queue.clear();
+}
+
 void VectorClockMongoD::onInitialDataAvailable(OperationContext* opCtx,
                                                bool isMajorityDataAvailable) {
     if (serverGlobalParams.clusterRole.has(ClusterRole::ConfigServer)) {
@@ -248,16 +169,6 @@ void VectorClockMongoD::onInitialDataAvailable(OperationContext* opCtx,
     }
 }
 
-void VectorClockMongoD::onShutdown() {
-    stdx::lock_guard lg(_mutex);
-    _shutdownInitiated.store(true);
-    for (auto& [_, promise] : _queue) {
-        promise->setError(
-            Status(ErrorCodes::ShutdownInProgress, "Not persisting vector clock due to shutdown"));
-    }
-    _queue.clear();
-}
-
 void VectorClockMongoD::onBecomeArbiter() {
     // The node has become an arbiter, hence will not need logical clock for external operations.
     _disable();
@@ -318,6 +229,12 @@ VectorClock::VectorTime VectorClockMongoD::recoverDirect(OperationContext* opCtx
     _advanceTime(
         {newDurableTime.clusterTime(), newDurableTime.configTime(), newDurableTime.topologyTime()});
 
+    LOGV2_DEBUG(1,
+                6011000,
+                "Recovered persisted vector clock",
+                "configTime"_attr = newDurableTime.configTime(),
+                "topologyTime"_attr = newDurableTime.topologyTime());
+
     return newDurableTime;
 }
 
@@ -434,21 +351,16 @@ Future<void> VectorClockMongoD::_doWhileQueueNotEmptyOrError() {
                 return !_durableTime;
             }();
 
-            ThreadClient tc("VectorClockStateOperation", _serviceContext);
-
-            {
-                stdx::lock_guard<Client> lk(*tc.get());
-                tc->setSystemOperationKillableByStepdown(lk);
-            }
-
+            ThreadClient tc("VectorClockStateOperation",
+                            _serviceContext->getService(ClusterRole::ShardServer));
             const auto opCtxHolder = tc->makeOperationContext();
             auto* const opCtx = opCtxHolder.get();
 
             // This code is used by the TransactionCoordinator. As a result, we need to skip ticket
             // acquisition in order to prevent possible deadlock when participants are in the
             // prepared state. See SERVER-82883 and SERVER-60682.
-            ScopedAdmissionPriorityForLock skipTicketAcquisition(
-                opCtx->lockState(), AdmissionContext::Priority::kImmediate);
+            ScopedAdmissionPriority<ExecutionAdmissionContext> skipTicketAcquisition(
+                opCtx, AdmissionContext::Priority::kExempt);
 
             if (mustRecoverDurableTime) {
                 return recoverDirect(opCtx);
@@ -475,25 +387,6 @@ Future<void> VectorClockMongoD::_doWhileQueueNotEmptyOrError() {
     return future;
 }
 
-VectorClock::ComponentSet VectorClockMongoD::_gossipOutInternal() const {
-    VectorClock::ComponentSet toGossip{Component::ClusterTime};
-    if (serverGlobalParams.clusterRole.has(ClusterRole::ShardServer) ||
-        serverGlobalParams.clusterRole.has(ClusterRole::ConfigServer)) {
-        toGossip.insert(Component::ConfigTime);
-        toGossip.insert(Component::TopologyTime);
-    }
-    return toGossip;
-}
-
-VectorClock::ComponentSet VectorClockMongoD::_gossipInInternal() const {
-    VectorClock::ComponentSet toGossip{Component::ClusterTime};
-    if (serverGlobalParams.clusterRole.has(ClusterRole::ShardServer)) {
-        toGossip.insert(Component::ConfigTime);
-        toGossip.insert(Component::TopologyTime);
-    }
-    return toGossip;
-}
-
 LogicalTime VectorClockMongoD::_tick(Component component, uint64_t nTicks) {
     if (component == Component::ClusterTime) {
         // Although conceptually ClusterTime can only be ticked when a mongod is able to take writes
@@ -522,10 +415,8 @@ void VectorClockMongoD::_tickTo(Component component, LogicalTime newTime) {
         return;
     }
 
-    if (component == Component::ConfigTime) {
-        // The ConfigTime is allowed to be tickTo on the ConfigServer and on the shard only when
-        // called from ShardingStateRecovery
-        // TODO SERVER-60110 re-add clusterRole == configServer condition like for TopologyTime
+    if (component == Component::ConfigTime &&
+        serverGlobalParams.clusterRole.has(ClusterRole::ConfigServer)) {
         _advanceComponentTimeTo(component, std::move(newTime));
         return;
     }
@@ -540,5 +431,4 @@ void VectorClockMongoD::_tickTo(Component component, LogicalTime newTime) {
     MONGO_UNREACHABLE;
 }
 
-}  // namespace
 }  // namespace mongo
