@@ -29,7 +29,12 @@
 
 #pragma once
 
+#include <memory>
+
+#include "mongo/db/logical_time.h"
 #include "mongo/db/s/sharding_mongod_test_fixture.h"
+#include "mongo/util/clock_source_mock.h"
+#include "mongo/util/time_support.h"
 
 namespace mongo {
 
@@ -42,12 +47,12 @@ class VectorClockMutable;
 /**
  * A test fixture that installs a VectorClock instance with a TimeProofService onto a service
  * context, in addition to the mock storage engine, network, and OpObserver provided by
- * ShardingMongodTestFixture.
+ * ShardingMongoDTestFixture.
  */
-class VectorClockTestFixture : public ShardingMongodTestFixture {
+class VectorClockTestFixture : public ShardingMongoDTestFixture {
 protected:
     VectorClockTestFixture();
-    ~VectorClockTestFixture();
+    ~VectorClockTestFixture() override;
 
     /**
      * Sets up this fixture as the primary node in a shard server replica set with a VectorClock
