@@ -27,19 +27,31 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+// IWYU pragma: no_include "ext/alloc_traits.h"
 
 #include "mongo/db/views/resolved_view.h"
 
-#include "mongo/base/init.h"
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/init.h"  // IWYU pragma: keep
+#include "mongo/base/status_with.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
 #include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
 #include "mongo/db/pipeline/document_source_coll_stats.h"
 #include "mongo/db/pipeline/document_source_index_stats.h"
 #include "mongo/db/pipeline/document_source_internal_convert_bucket_index_stats.h"
 #include "mongo/db/pipeline/document_source_internal_unpack_bucket.h"
+#include "mongo/db/query/explain_options.h"
 #include "mongo/db/timeseries/timeseries_constants.h"
 #include "mongo/db/timeseries/timeseries_index_schema_conversion_functions.h"
-#include "mongo/rpc/get_status_from_command_result.h"
+#include "mongo/idl/idl_parser.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/serialization_context.h"
+#include "mongo/util/str.h"
 
 namespace mongo {
 
@@ -77,8 +89,8 @@ ResolvedView ResolvedView::fromBSON(const BSONObj& commandResponseObj) {
     boost::optional<TimeseriesOptions> timeseriesOptions = boost::none;
     if (auto tsOptionsElt = viewDef[kTimeseriesOptions]) {
         if (tsOptionsElt.isABSONObj()) {
-            timeseriesOptions =
-                TimeseriesOptions::parse({"ResolvedView::fromBSON"}, tsOptionsElt.Obj());
+            timeseriesOptions = TimeseriesOptions::parse(IDLParserContext{"ResolvedView::fromBSON"},
+                                                         tsOptionsElt.Obj());
         }
     }
 
@@ -102,17 +114,28 @@ ResolvedView ResolvedView::fromBSON(const BSONObj& commandResponseObj) {
         usesExtendedRange = boost::optional<bool>(usesExtendedRangeElem.boolean());
     }
 
-    return {NamespaceString(viewDef["ns"].valueStringData()),
+    boost::optional<bool> fixedBuckets = boost::none;
+    if (auto fixedBucketsElem = viewDef[kTimeseriesfixedBuckets]) {
+        uassert(7823304,
+                str::stream() << "view definition must have " << kTimeseriesfixedBuckets
+                              << " of type bool or no such field",
+                fixedBucketsElem.type() == BSONType::Bool);
+
+        fixedBuckets = boost::optional<bool>(fixedBucketsElem.boolean());
+    }
+
+    return {NamespaceStringUtil::deserializeForErrorMsg(viewDef["ns"].valueStringData()),
             std::move(pipeline),
             std::move(collationSpec),
             std::move(timeseriesOptions),
             std::move(mixedSchema),
-            std::move(usesExtendedRange)};
+            std::move(usesExtendedRange),
+            std::move(fixedBuckets)};
 }
 
 void ResolvedView::serialize(BSONObjBuilder* builder) const {
     BSONObjBuilder subObj(builder->subobjStart("resolvedView"));
-    subObj.append("ns", _namespace.ns());
+    subObj.append("ns", _namespace.toStringForErrorMsg());
     subObj.append("pipeline", _pipeline);
     if (_timeseriesOptions) {
         BSONObjBuilder tsObj(builder->subobjStart(kTimeseriesOptions));
@@ -125,6 +148,9 @@ void ResolvedView::serialize(BSONObjBuilder* builder) const {
     if ((_timeseriesUsesExtendedRange && (*_timeseriesUsesExtendedRange)))
         subObj.append(kTimeseriesUsesExtendedRange, *_timeseriesUsesExtendedRange);
 
+    if ((_timeseriesfixedBuckets && (*_timeseriesfixedBuckets)))
+        subObj.append(kTimeseriesfixedBuckets, *_timeseriesfixedBuckets);
+
     if (!_defaultCollation.isEmpty()) {
         subObj.append("collation", _defaultCollation);
     }
@@ -173,8 +199,7 @@ void ResolvedView::handleTimeseriesRewrites(std::vector<BSONObj>* resolvedPipeli
             // collection, and skip $_internalUnpackBucket.
             resolvedPipeline->erase(resolvedPipeline->begin());
         }
-    } else if (serverGlobalParams.featureCompatibility.isGreaterThanOrEqualTo(
-                   multiversion::FeatureCompatibilityVersion::kVersion_5_2)) {
+    } else {
         auto unpackStage = (*resolvedPipeline)[0];
 
         BSONObjBuilder builder;
@@ -188,6 +213,9 @@ void ResolvedView::handleTimeseriesRewrites(std::vector<BSONObj>* resolvedPipeli
         builder.append(DocumentSourceInternalUnpackBucket::kUsesExtendedRange,
                        ((_timeseriesUsesExtendedRange && *_timeseriesUsesExtendedRange)));
 
+        builder.append(DocumentSourceInternalUnpackBucket::kFixedBuckets,
+                       ((_timeseriesfixedBuckets && *_timeseriesfixedBuckets)));
+
         (*resolvedPipeline)[0] =
             BSON(DocumentSourceInternalUnpackBucket::kStageNameInternal << builder.obj());
     }
@@ -208,7 +236,8 @@ AggregateCommandRequest ResolvedView::asExpandedViewAggregation(
         handleTimeseriesRewrites(&resolvedPipeline);
     }
 
-    AggregateCommandRequest expandedRequest{_namespace, std::move(resolvedPipeline)};
+    AggregateCommandRequest expandedRequest{
+        _namespace, std::move(resolvedPipeline), request.getSerializationContext()};
 
     if (request.getExplain()) {
         expandedRequest.setExplain(request.getExplain());
@@ -230,13 +259,16 @@ AggregateCommandRequest ResolvedView::asExpandedViewAggregation(
             }
         }
         expandedRequest.setHint(rewritten);
-
-        // JLR Possibly need to handle inappropriately forcing clustered here. Will likely be
-        // handled by removing the _id predicate, however.
     } else {
         expandedRequest.setHint(request.getHint());
     }
 
+    // If query settings were present in the original request, they should be present in the
+    // 'expandedRequest'.
+    if (request.getQuerySettings()) {
+        expandedRequest.setQuerySettings(request.getQuerySettings());
+    }
+
     expandedRequest.setMaxTimeMS(request.getMaxTimeMS());
     expandedRequest.setReadConcern(request.getReadConcern());
     expandedRequest.setUnwrappedReadPref(request.getUnwrappedReadPref());
@@ -244,6 +276,7 @@ AggregateCommandRequest ResolvedView::asExpandedViewAggregation(
     expandedRequest.setAllowDiskUse(request.getAllowDiskUse());
     expandedRequest.setIsMapReduceCommand(request.getIsMapReduceCommand());
     expandedRequest.setLet(request.getLet());
+    expandedRequest.setIncludeQueryStatsMetrics(request.getIncludeQueryStatsMetrics());
 
     // Operations on a view must always use the default collation of the view. We must have already
     // checked that if the user's request specifies a collation, it matches the collation of the
