@@ -29,12 +29,22 @@
 
 #pragma once
 
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <memory>
+#include <utility>
 #include <vector>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/base/error_extra_info.h"
 #include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
 #include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/pipeline/aggregate_command_gen.h"
+#include "mongo/db/timeseries/timeseries_gen.h"
 
 namespace mongo {
 
@@ -49,13 +59,15 @@ public:
                  BSONObj defaultCollation,
                  boost::optional<TimeseriesOptions> timeseriesOptions = boost::none,
                  boost::optional<bool> timeseriesMayContainMixedData = boost::none,
-                 boost::optional<bool> timeseriesUsesExtendedRange = boost::none)
+                 boost::optional<bool> timeseriesUsesExtendedRange = boost::none,
+                 boost::optional<bool> timeseriesfixedBuckets = boost::none)
         : _namespace(collectionNs),
           _pipeline(std::move(pipeline)),
           _defaultCollation(std::move(defaultCollation)),
           _timeseriesOptions(timeseriesOptions),
           _timeseriesMayContainMixedData(timeseriesMayContainMixedData),
-          _timeseriesUsesExtendedRange(timeseriesUsesExtendedRange) {}
+          _timeseriesUsesExtendedRange(timeseriesUsesExtendedRange),
+          _timeseriesfixedBuckets(timeseriesfixedBuckets) {}
 
     static ResolvedView fromBSON(const BSONObj& commandResponseObj);
 
@@ -89,6 +101,7 @@ public:
     static constexpr StringData kTimeseriesMayContainMixedData = "timeseriesMayContainMixedData"_sd;
     static constexpr StringData kTimeseriesOptions = "timeseriesOptions"_sd;
     static constexpr StringData kTimeseriesUsesExtendedRange = "timeseriesUsesExtendedRange"_sd;
+    static constexpr StringData kTimeseriesfixedBuckets = "timeseriesfixedBuckets"_sd;
 
     void serialize(BSONObjBuilder* bob) const final;
     static std::shared_ptr<const ErrorExtraInfo> parse(const BSONObj&);
@@ -108,6 +121,7 @@ private:
     boost::optional<TimeseriesOptions> _timeseriesOptions;
     boost::optional<bool> _timeseriesMayContainMixedData;
     boost::optional<bool> _timeseriesUsesExtendedRange;
+    boost::optional<bool> _timeseriesfixedBuckets;
 };
 
 }  // namespace mongo
