@@ -30,14 +30,28 @@
 
 #include "mongo/db/write_concern.h"
 
-#include "mongo/bson/util/bson_extract.h"
+#include <boost/optional.hpp>
+#include <cstddef>
+#include <cstdint>
+#include <fmt/format.h>
+#include <memory>
+#include <variant>
+
+#include <absl/container/node_hash_map.h>
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
 #include "mongo/db/client.h"
+#include "mongo/db/cluster_role.h"
 #include "mongo/db/commands/server_status_metric.h"
 #include "mongo/db/curop.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/read_write_concern_defaults.h"
+#include "mongo/db/read_write_concern_defaults_gen.h"
+#include "mongo/db/read_write_concern_provenance.h"
 #include "mongo/db/repl/optime.h"
-#include "mongo/db/repl/repl_server_parameters_gen.h"
+#include "mongo/db/repl/repl_settings.h"
 #include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/repl/storage_interface.h"
 #include "mongo/db/server_options.h"
@@ -48,7 +62,13 @@
 #include "mongo/db/transaction_validation.h"
 #include "mongo/db/write_concern_options.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/duration.h"
 #include "mongo/util/fail_point.h"
+#include "mongo/util/timer.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplication
 
@@ -58,15 +78,17 @@ namespace mongo {
 using repl::OpTime;
 using std::string;
 
-static TimerStats& gleWtimeStats = makeServerStatusMetric<TimerStats>("getLastError.wtime");
-static CounterMetric gleWtimeouts("getLastError.wtimeouts");
-static CounterMetric gleDefaultWtimeouts("getLastError.default.wtimeouts");
-static CounterMetric gleDefaultUnsatisfiable("getLastError.default.unsatisfiable");
+namespace {
+auto& gleWtimeStats = *MetricBuilder<TimerStats>{"getLastError.wtime"};
+auto& gleWtimeouts = *MetricBuilder<Counter64>{"getLastError.wtimeouts"};
+auto& gleDefaultWtimeouts = *MetricBuilder<Counter64>{"getLastError.default.wtimeouts"};
+auto& gleDefaultUnsatisfiable = *MetricBuilder<Counter64>{"getLastError.default.unsatisfiable"};
+}  // namespace
 
 MONGO_FAIL_POINT_DEFINE(hangBeforeWaitingForWriteConcern);
 
-bool commandSpecifiesWriteConcern(const BSONObj& cmdObj) {
-    return cmdObj.hasField(WriteConcernOptions::kWriteConcernField);
+bool commandSpecifiesWriteConcern(const CommonRequestArgs& requestArgs) {
+    return !!requestArgs.getWriteConcern();
 }
 
 StatusWith<WriteConcernOptions> extractWriteConcern(OperationContext* opCtx,
@@ -79,14 +101,6 @@ StatusWith<WriteConcernOptions> extractWriteConcern(OperationContext* opCtx,
 
     WriteConcernOptions writeConcern = wcResult.getValue();
 
-    if (writeConcern.hasCustomWriteMode()) {
-        if (auto wMode = get_if<std::string>(&writeConcern.w)) {
-            uassert(103742,
-                    "illegal embedded NUL byte in write concern " + *wMode,
-                    wMode->find('\0') == std::string::npos);
-        }
-    }
-
     // This is the WC extracted from the command object, so the CWWC or implicit default hasn't been
     // applied yet, which is why "usedDefaultConstructedWC" flag can be used an indicator of whether
     // the client supplied a WC or not.
@@ -101,9 +115,9 @@ StatusWith<WriteConcernOptions> extractWriteConcern(OperationContext* opCtx,
     // WriteConcern defaults can only be applied on regular replica set members.
     // Operations received by shard and config servers should always have WC explicitly specified.
     bool canApplyDefaultWC = serverGlobalParams.clusterRole.has(ClusterRole::None) &&
-        repl::ReplicationCoordinator::get(opCtx)->isReplEnabled() &&
+        repl::ReplicationCoordinator::get(opCtx)->getSettings().isReplSet() &&
         (!opCtx->inMultiDocumentTransaction() ||
-         isTransactionCommand(cmdObj.firstElementFieldName())) &&
+         isTransactionCommand(opCtx->getService(), cmdObj.firstElementFieldName())) &&
         !opCtx->getClient()->isInDirectClient() && !isInternalClient;
 
 
@@ -169,6 +183,20 @@ StatusWith<WriteConcernOptions> extractWriteConcern(OperationContext* opCtx,
         }
     }
 
+    if (writeConcern.syncMode == WriteConcernOptions::SyncMode::NONE && writeConcern.isMajority() &&
+        !opCtx->getServiceContext()->getStorageEngine()->isEphemeral()) {
+        auto* const replCoord = repl::ReplicationCoordinator::get(opCtx);
+        if (replCoord && replCoord->getSettings().isReplSet() &&
+            replCoord->getWriteConcernMajorityShouldJournal()) {
+            LOGV2_DEBUG(8668500,
+                        1,
+                        "Overriding write concern majority j:false to j:true",
+                        "writeConcern"_attr = writeConcern);
+            writeConcern.majorityJFalseOverridden = true;
+            writeConcern.syncMode = WriteConcernOptions::SyncMode::JOURNAL;
+        }
+    }
+
     Status wcStatus = validateWriteConcern(opCtx, writeConcern);
     if (!wcStatus.isOK()) {
         return wcStatus;
@@ -184,20 +212,18 @@ Status validateWriteConcern(OperationContext* opCtx, const WriteConcernOptions&
                       "cannot use 'j' option when a host does not have journaling enabled");
     }
 
-    const auto replMode = repl::ReplicationCoordinator::get(opCtx)->getReplicationMode();
+    if (!repl::ReplicationCoordinator::get(opCtx)->getSettings().isReplSet()) {
+        if (holds_alternative<int64_t>(writeConcern.w) && get<int64_t>(writeConcern.w) > 1) {
+            return Status(ErrorCodes::BadValue, "cannot use 'w' > 1 when a host is not replicated");
+        }
 
-    if (replMode == repl::ReplicationCoordinator::modeNone &&
-        (stdx::holds_alternative<int64_t>(writeConcern.w) &&
-         stdx::get<int64_t>(writeConcern.w) > 1)) {
-        return Status(ErrorCodes::BadValue, "cannot use 'w' > 1 when a host is not replicated");
-    }
-
-    if (replMode != repl::ReplicationCoordinator::modeReplSet &&
-        writeConcern.hasCustomWriteMode()) {
-        return Status(ErrorCodes::BadValue,
-                      fmt::format("cannot use non-majority 'w' mode \"{}\" when a host is not a "
-                                  "member of a replica set",
-                                  stdx::get<std::string>(writeConcern.w)));
+        if (writeConcern.hasCustomWriteMode()) {
+            return Status(
+                ErrorCodes::BadValue,
+                fmt::format("cannot use non-majority 'w' mode \"{}\" when a host is not a "
+                            "member of a replica set",
+                            get<std::string>(writeConcern.w)));
+        }
     }
 
     return Status::OK();
@@ -286,7 +312,7 @@ Status waitForWriteConcern(OperationContext* opCtx,
         // This fail point pauses with an open snapshot on the oplog. Some tests pause on this fail
         // point prior to running replication rollback. This prevents the operation from being
         // killed and the snapshot being released. Hence, we release the snapshot here.
-        opCtx->replaceRecoveryUnit();
+        shard_role_details::replaceRecoveryUnit(opCtx);
 
         hangBeforeWaitingForWriteConcern.pauseWhileSet();
     }
@@ -312,13 +338,28 @@ Status waitForWriteConcern(OperationContext* opCtx,
                     result->fsyncFiles = 1;
                 } else {
                     // We only need to commit the journal if we're durable
-                    JournalFlusher::get(opCtx)->waitForJournalFlush();
+                    JournalFlusher::get(opCtx)->waitForJournalFlush(opCtx);
                 }
                 break;
             }
             case WriteConcernOptions::SyncMode::JOURNAL:
                 waitForNoOplogHolesIfNeeded(opCtx);
-                JournalFlusher::get(opCtx)->waitForJournalFlush();
+                // In most cases we only need to trigger a journal flush without waiting for it
+                // to complete because waiting for replication with j:true already tracks the
+                // durable point for all data-bearing nodes and thus is sufficient to guarantee
+                // durability.
+                //
+                // One exception is for w:1 writes where we need to wait for the journal flush
+                // to complete because we skip waiting for replication for w:1 writes. In fact
+                // for multi-voter replica sets, durability of w:1 writes could be meaningless
+                // because they may still be rolled back if the primary crashes. Single-voter
+                // replica sets, however, can never rollback confirmed writes, thus durability
+                // does matter in this case.
+                if (!writeConcernWithPopulatedSyncMode.needToWaitForOtherNodes()) {
+                    JournalFlusher::get(opCtx)->waitForJournalFlush(opCtx);
+                } else {
+                    JournalFlusher::get(opCtx)->triggerJournalFlush();
+                }
                 break;
         }
     } catch (const DBException& ex) {
