@@ -31,63 +31,88 @@
  * Runs db unit tests.
  */
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/dbtests/dbtests.h"
-
+#include <boost/move/utility_core.hpp>
 #include <memory>
+#include <string>
+#include <utility>
+#include <vector>
 
-#include "mongo/base/init.h"
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/init.h"  // IWYU pragma: keep
 #include "mongo/base/initializer.h"
 #include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/client/dbclient_base.h"
 #include "mongo/db/auth/authorization_manager.h"
 #include "mongo/db/catalog/multi_index_block.h"
-#include "mongo/db/commands.h"
 #include "mongo/db/commands/test_commands_enabled.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
 #include "mongo/db/cursor_manager.h"
 #include "mongo/db/db_raii.h"
 #include "mongo/db/index/index_descriptor.h"
 #include "mongo/db/repl/drop_pending_collection_reaper.h"
+#include "mongo/db/repl/member_state.h"
+#include "mongo/db/repl/repl_settings.h"
 #include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/repl/replication_coordinator_mock.h"
 #include "mongo/db/repl/storage_interface_mock.h"
+#include "mongo/db/server_options.h"
 #include "mongo/db/service_context.h"
 #include "mongo/db/service_entry_point_mongod.h"
+#include "mongo/db/session_manager_mongod.h"
+#include "mongo/db/storage/recovery_unit.h"
+#include "mongo/db/storage/write_unit_of_work.h"
+#include "mongo/db/transaction_resources.h"
 #include "mongo/db/wire_version.h"
+#include "mongo/dbtests/dbtests.h"  // IWYU pragma: keep
 #include "mongo/dbtests/framework.h"
 #include "mongo/scripting/engine.h"
-#include "mongo/transport/transport_layer_manager.h"
+#include "mongo/transport/service_entry_point.h"
+#include "mongo/transport/transport_layer_manager_impl.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/util/assert_util_core.h"
+#include "mongo/util/clock_source.h"
 #include "mongo/util/clock_source_mock.h"
+#include "mongo/util/duration.h"
 #include "mongo/util/quick_exit.h"
+#include "mongo/util/scopeguard.h"
 #include "mongo/util/signal_handlers_synchronous.h"
 #include "mongo/util/testing_proctor.h"
-#include "mongo/util/text.h"
+#include "mongo/util/text.h"  // IWYU pragma: keep
+#include "mongo/util/version/releases.h"
 
 namespace mongo {
 namespace dbtests {
 namespace {
+
 const auto kIndexVersion = IndexDescriptor::IndexVersion::kV2;
+
+ServiceContext::ConstructorActionRegisterer registerWireSpec{
+    "RegisterWireSpec", [](ServiceContext* service) {
+        WireSpec::Specification spec;
+
+        // Accept from any version external client.
+        spec.incomingExternalClient.minWireVersion = RELEASE_2_4_AND_BEFORE;
+        spec.incomingExternalClient.maxWireVersion = LATEST_WIRE_VERSION;
+
+        // Accept from internal clients of the same version, as in upgrade
+        // featureCompatibilityVersion.
+        spec.incomingInternalClient.minWireVersion = LATEST_WIRE_VERSION;
+        spec.incomingInternalClient.maxWireVersion = LATEST_WIRE_VERSION;
+
+        // Connect to servers of the same version, as in upgrade featureCompatibilityVersion.
+        spec.outgoing.minWireVersion = LATEST_WIRE_VERSION;
+        spec.outgoing.maxWireVersion = LATEST_WIRE_VERSION;
+
+        WireSpec::getWireSpec(service).initialize(std::move(spec));
+    }};
+
 }  // namespace
 
-MONGO_INITIALIZER_WITH_PREREQUISITES(WireSpec, ("EndStartupOptionHandling"))(InitializerContext*) {
-    WireSpec::Specification spec;
-
-    // Accept from any version external client.
-    spec.incomingExternalClient.minWireVersion = RELEASE_2_4_AND_BEFORE;
-    spec.incomingExternalClient.maxWireVersion = LATEST_WIRE_VERSION;
-
-    // Accept from internal clients of the same version, as in upgrade
-    // featureCompatibilityVersion.
-    spec.incomingInternalClient.minWireVersion = LATEST_WIRE_VERSION;
-    spec.incomingInternalClient.maxWireVersion = LATEST_WIRE_VERSION;
-
-    // Connect to servers of the same version, as in upgrade featureCompatibilityVersion.
-    spec.outgoing.minWireVersion = LATEST_WIRE_VERSION;
-    spec.outgoing.maxWireVersion = LATEST_WIRE_VERSION;
-
-    WireSpec::instance().initialize(std::move(spec));
-}
-
 Status createIndex(OperationContext* opCtx, StringData ns, const BSONObj& keys, bool unique) {
     BSONObjBuilder specBuilder;
     specBuilder.append("name", DBClientBase::genIndexName(keys));
@@ -100,7 +125,7 @@ Status createIndex(OperationContext* opCtx, StringData ns, const BSONObj& keys,
 }
 
 Status createIndexFromSpec(OperationContext* opCtx, StringData ns, const BSONObj& spec) {
-    NamespaceString nss(ns);
+    NamespaceString nss = NamespaceString::createNamespaceString_forTest(ns);
     AutoGetDb autoDb(opCtx, nss.dbName(), MODE_IX);
     {
         Lock::CollectionLock collLock(opCtx, nss, MODE_X);
@@ -110,7 +135,7 @@ Status createIndexFromSpec(OperationContext* opCtx, StringData ns, const BSONObj
         if (!coll) {
             auto db = autoDb.ensureDbExists(opCtx);
             invariant(db);
-            coll = db->createCollection(opCtx, NamespaceString(ns));
+            coll = db->createCollection(opCtx, NamespaceString::createNamespaceString_forTest(ns));
         }
         invariant(coll);
         wunit.commit();
@@ -132,8 +157,11 @@ Status createIndexFromSpec(OperationContext* opCtx, StringData ns, const BSONObj
                            collection,
                            spec,
                            [opCtx](const std::vector<BSONObj>& specs) -> Status {
-                               if (opCtx->recoveryUnit()->getCommitTimestamp().isNull()) {
-                                   return opCtx->recoveryUnit()->setTimestamp(Timestamp(1, 1));
+                               if (shard_role_details::getRecoveryUnit(opCtx)
+                                       ->getCommitTimestamp()
+                                       .isNull()) {
+                                   return shard_role_details::getRecoveryUnit(opCtx)->setTimestamp(
+                                       Timestamp(1, 1));
                                }
                                return Status::OK();
                            })
@@ -171,7 +199,7 @@ Status createIndexFromSpec(OperationContext* opCtx, StringData ns, const BSONObj
                                  collection.getWritableCollection(opCtx),
                                  MultiIndexBlock::kNoopOnCreateEachFn,
                                  MultiIndexBlock::kNoopOnCommitFn));
-        ASSERT_OK(opCtx->recoveryUnit()->setTimestamp(Timestamp(1, 1)));
+        ASSERT_OK(shard_role_details::getRecoveryUnit(opCtx)->setTimestamp(Timestamp(1, 1)));
         wunit.commit();
     }
     abortOnExit.dismiss();
@@ -179,30 +207,19 @@ Status createIndexFromSpec(OperationContext* opCtx, StringData ns, const BSONObj
 }
 
 WriteContextForTests::WriteContextForTests(OperationContext* opCtx, StringData ns)
-    : _opCtx(opCtx), _nss(ns) {
+    : _opCtx(opCtx), _nss(NamespaceString::createNamespaceString_forTest(ns)) {
     // Lock the database and collection
     _autoDb.emplace(opCtx, _nss.dbName(), MODE_IX);
-    _collLock.emplace(opCtx, _nss, MODE_IX);
+    _collLock.emplace(opCtx, _nss, MODE_X);
 
     const bool doShardVersionCheck = false;
 
     _clientContext.emplace(opCtx, _nss, doShardVersionCheck);
     auto db = _autoDb->ensureDbExists(opCtx);
-    invariant(db, _nss.ns());
+    invariant(db, _nss.toStringForErrorMsg());
     invariant(db == _clientContext->db());
-
-    // If the collection exists, there is no need to lock into stronger mode
-    if (getCollection())
-        return;
-
-    invariant(db == _clientContext->db());
-    _collLock.emplace(opCtx, _nss, MODE_X);
 }
 
-}  // namespace dbtests
-}  // namespace mongo
-
-
 int dbtestsMain(int argc, char** argv) {
     ::mongo::setTestCommandsEnabled(true);
     ::mongo::TestingProctor::instance().setEnabled(true);
@@ -216,7 +233,7 @@ int dbtestsMain(int argc, char** argv) {
     setGlobalServiceContext(ServiceContext::make());
 
     const auto service = getGlobalServiceContext();
-    service->setServiceEntryPoint(std::make_unique<ServiceEntryPointMongod>(service));
+    service->getService()->setServiceEntryPoint(std::make_unique<ServiceEntryPointMongod>());
 
     auto fastClock = std::make_unique<ClockSourceMock>();
     // Timestamps are split into two 32-bit integers, seconds and "increments". Currently (but
@@ -233,8 +250,8 @@ int dbtestsMain(int argc, char** argv) {
     CursorManager::get(service)->setPreciseClockSource(preciseClock.get());
     service->setPreciseClockSource(std::move(preciseClock));
 
-    service->setTransportLayer(
-        transport::TransportLayerManager::makeAndStartDefaultEgressTransportLayer());
+    service->setTransportLayerManager(
+        transport::TransportLayerManagerImpl::makeAndStartDefaultEgressTransportLayer());
 
     repl::ReplicationCoordinator::set(
         service,
@@ -248,11 +265,14 @@ int dbtestsMain(int argc, char** argv) {
     repl::DropPendingCollectionReaper::set(
         service, std::make_unique<repl::DropPendingCollectionReaper>(storageMock.get()));
 
-    AuthorizationManager::get(service)->setAuthEnabled(false);
-    ScriptEngine::setup();
+    AuthorizationManager::get(service->getService())->setAuthEnabled(false);
+    ScriptEngine::setup(ExecutionEnvironment::Server);
     return mongo::dbtests::runDbTests(argc, argv);
 }
 
+}  // namespace dbtests
+}  // namespace mongo
+
 #if defined(_WIN32)
 // In Windows, wmain() is an alternate entry point for main(), and receives the same parameters
 // as main() but encoded in Windows Unicode (UTF-16); "wide" 16-bit wchar_t characters.  The
@@ -260,10 +280,11 @@ int dbtestsMain(int argc, char** argv) {
 // and makes them available through the argv() and envp() members.  This enables dbtestsMain()
 // to process UTF-8 encoded arguments and environment variables without regard to platform.
 int wmain(int argc, wchar_t* argvW[]) {
-    quickExit(dbtestsMain(argc, WindowsCommandLine(argc, argvW).argv()));
+    mongo::quickExit(
+        mongo::dbtests::dbtestsMain(argc, mongo::WindowsCommandLine(argc, argvW).argv()));
 }
 #else
 int main(int argc, char* argv[]) {
-    quickExit(dbtestsMain(argc, argv));
+    mongo::quickExit(mongo::dbtests::dbtestsMain(argc, argv));
 }
 #endif
