@@ -31,39 +31,75 @@
  * Unit tests for DocumentSource classes.
  */
 
-#include "mongo/platform/basic.h"
+// IWYU pragma: no_include "cxxabi.h"
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <fmt/format.h>
+#include <memory>
+#include <mutex>
+#include <string>
+#include <utility>
+#include <vector>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/db/catalog/collection.h"
 #include "mongo/db/client.h"
 #include "mongo/db/db_raii.h"
 #include "mongo/db/dbdirectclient.h"
 #include "mongo/db/exec/collection_scan.h"
+#include "mongo/db/exec/collection_scan_common.h"
+#include "mongo/db/exec/document_value/document.h"
 #include "mongo/db/exec/document_value/document_value_test_util.h"
-#include "mongo/db/exec/multi_plan.h"
+#include "mongo/db/exec/document_value/value.h"
 #include "mongo/db/exec/plan_stage.h"
+#include "mongo/db/exec/working_set.h"
+#include "mongo/db/matcher/expression_parser.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/pipeline/aggregate_command_gen.h"
-#include "mongo/db/pipeline/dependencies.h"
 #include "mongo/db/pipeline/document_source.h"
 #include "mongo/db/pipeline/document_source_cursor.h"
+#include "mongo/db/pipeline/expression_context.h"
 #include "mongo/db/pipeline/expression_context_for_test.h"
-#include "mongo/db/pipeline/pipeline.h"
+#include "mongo/db/query/canonical_query.h"
+#include "mongo/db/query/explain_options.h"
+#include "mongo/db/query/find_command.h"
 #include "mongo/db/query/get_executor.h"
-#include "mongo/db/query/mock_yield_policies.h"
 #include "mongo/db/query/multiple_collection_accessor.h"
+#include "mongo/db/query/plan_executor.h"
 #include "mongo/db/query/plan_executor_factory.h"
-#include "mongo/db/query/query_planner.h"
-#include "mongo/db/query/stage_builder.h"
-#include "mongo/dbtests/dbtests.h"
+#include "mongo/db/query/plan_yield_policy.h"
+#include "mongo/db/query/query_planner_params.h"
+#include "mongo/db/query/query_request_helper.h"
+#include "mongo/db/query/tailable_mode_gen.h"
+#include "mongo/db/service_context.h"
+#include "mongo/db/storage/storage_engine.h"
+#include "mongo/db/storage/storage_options.h"
+#include "mongo/db/transaction_resources.h"
+#include "mongo/dbtests/dbtests.h"  // IWYU pragma: keep
+#include "mongo/platform/mutex.h"
+#include "mongo/stdx/condition_variable.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/intrusive_counter.h"
 #include "mongo/util/scopeguard.h"
 
 namespace mongo {
 namespace {
 
 using boost::intrusive_ptr;
-using std::unique_ptr;
 using std::vector;
 
-static const NamespaceString nss("unittests.documentsourcetests");
+static const NamespaceString nss =
+    NamespaceString::createNamespaceString_forTest("unittests.documentsourcetests");
 static const BSONObj metaTextScore = BSON("$meta"
                                           << "textScore");
 
@@ -83,7 +119,7 @@ public:
         _ctx->tempDir = storageGlobalParams.dbpath + "/_tmp";
     }
 
-    virtual ~DocumentSourceCursorTest() {
+    ~DocumentSourceCursorTest() override {
         client.dropCollection(nss);
     }
 
@@ -92,21 +128,22 @@ protected:
         // clean up first if this was called before
         _source.reset();
 
-        dbtests::WriteContextForTests ctx(opCtx(), nss.ns());
+        dbtests::WriteContextForTests ctx(opCtx(), nss.ns_forTest());
         _coll = ctx.getCollection();
 
         auto findCommand = std::make_unique<FindCommandRequest>(nss);
         if (hint) {
             findCommand->setHint(*hint);
         }
-        auto cq = uassertStatusOK(CanonicalQuery::canonicalize(opCtx(), std::move(findCommand)));
+        auto cq = std::make_unique<CanonicalQuery>(
+            CanonicalQueryParams{.expCtx = makeExpressionContext(opCtx(), *findCommand),
+                                 .parsedFind = ParsedFindCommandParams{std::move(findCommand)}});
 
-        auto exec = uassertStatusOK(getExecutor(opCtx(),
-                                                &_coll,
-                                                std::move(cq),
-                                                nullptr /* extractAndAttachPipelineStages */,
-                                                PlanYieldPolicy::YieldPolicy::NO_YIELD,
-                                                QueryPlannerParams::RETURN_OWNED_DATA));
+        auto exec = uassertStatusOK(getExecutorFind(opCtx(),
+                                                    MultipleCollectionAccessor{_coll},
+                                                    std::move(cq),
+                                                    PlanYieldPolicy::YieldPolicy::INTERRUPT_ONLY,
+                                                    QueryPlannerParams::RETURN_OWNED_DATA));
 
         _source = DocumentSourceCursor::create(MultipleCollectionAccessor(_coll),
                                                std::move(exec),
@@ -147,11 +184,11 @@ private:
 TEST_F(DocumentSourceCursorTest, Empty) {
     createSource();
     // The DocumentSourceCursor doesn't hold a read lock.
-    ASSERT(!opCtx()->lockState()->isReadLocked());
+    ASSERT(!shard_role_details::getLocker(opCtx())->isReadLocked());
     // The collection is empty, so the source produces no results.
     ASSERT(source()->getNext().isEOF());
     // Exhausting the source releases the read lock.
-    ASSERT(!opCtx()->lockState()->isReadLocked());
+    ASSERT(!shard_role_details::getLocker(opCtx())->isReadLocked());
 }
 
 /** Iterate a DocumentSourceCursor. */
@@ -159,7 +196,7 @@ TEST_F(DocumentSourceCursorTest, Iterate) {
     client.insert(nss, BSON("a" << 1));
     createSource();
     // The DocumentSourceCursor doesn't hold a read lock.
-    ASSERT(!opCtx()->lockState()->isReadLocked());
+    ASSERT(!shard_role_details::getLocker(opCtx())->isReadLocked());
     // The cursor will produce the expected result.
     auto next = source()->getNext();
     ASSERT(next.isAdvanced());
@@ -167,17 +204,17 @@ TEST_F(DocumentSourceCursorTest, Iterate) {
     // There are no more results.
     ASSERT(source()->getNext().isEOF());
     // Exhausting the source releases the read lock.
-    ASSERT(!opCtx()->lockState()->isReadLocked());
+    ASSERT(!shard_role_details::getLocker(opCtx())->isReadLocked());
 }
 
 /** Dispose of a DocumentSourceCursor. */
 TEST_F(DocumentSourceCursorTest, Dispose) {
     createSource();
     // The DocumentSourceCursor doesn't hold a read lock.
-    ASSERT(!opCtx()->lockState()->isReadLocked());
+    ASSERT(!shard_role_details::getLocker(opCtx())->isReadLocked());
     source()->dispose();
     // Releasing the cursor releases the read lock.
-    ASSERT(!opCtx()->lockState()->isReadLocked());
+    ASSERT(!shard_role_details::getLocker(opCtx())->isReadLocked());
     // The source is marked as exhausted.
     ASSERT(source()->getNext().isEOF());
 }
@@ -197,10 +234,10 @@ TEST_F(DocumentSourceCursorTest, IterateDispose) {
     ASSERT(next.isAdvanced());
     ASSERT_VALUE_EQ(Value(2), next.getDocument().getField("a"));
     // The DocumentSourceCursor doesn't hold a read lock.
-    ASSERT(!opCtx()->lockState()->isReadLocked());
+    ASSERT(!shard_role_details::getLocker(opCtx())->isReadLocked());
     source()->dispose();
     // Disposing of the source releases the lock.
-    ASSERT(!opCtx()->lockState()->isReadLocked());
+    ASSERT(!shard_role_details::getLocker(opCtx())->isReadLocked());
     // The source cannot be advanced further.
     ASSERT(source()->getNext().isEOF());
 }
@@ -306,7 +343,7 @@ TEST_F(DocumentSourceCursorTest, TailableAwaitDataCursorShouldErrorAfterTimeout)
         return;
     }
 
-    // Make sure the collection exists, otherwise we'll default to a NO_YIELD yield policy.
+    // Make sure the collection exists, otherwise we'll default to an INTERRUPT_ONLY yield policy.
     const bool capped = true;
     const long long cappedSize = 1024;
     ASSERT_TRUE(client.createCollection(nss, cappedSize, capped));
@@ -320,7 +357,7 @@ TEST_F(DocumentSourceCursorTest, TailableAwaitDataCursorShouldErrorAfterTimeout)
     auto filter = BSON("a" << 1);
     auto matchExpression = uassertStatusOK(MatchExpressionParser::parse(filter, ctx()));
     auto collectionScan = std::make_unique<CollectionScan>(ctx().get(),
-                                                           readLock.getCollection(),
+                                                           &readLock.getCollection(),
                                                            collScanParams,
                                                            workingSet.get(),
                                                            matchExpression.get());
@@ -328,8 +365,9 @@ TEST_F(DocumentSourceCursorTest, TailableAwaitDataCursorShouldErrorAfterTimeout)
     findCommand->setFilter(filter);
     query_request_helper::setTailableMode(TailableModeEnum::kTailableAndAwaitData,
                                           findCommand.get());
-    auto canonicalQuery = unittest::assertGet(
-        CanonicalQuery::canonicalize(opCtx(), std::move(findCommand), false, nullptr));
+    auto canonicalQuery = std::make_unique<CanonicalQuery>(
+        CanonicalQueryParams{.expCtx = makeExpressionContext(opCtx(), *findCommand),
+                             .parsedFind = ParsedFindCommandParams{std::move(findCommand)}});
     auto planExecutor =
         uassertStatusOK(plan_executor_factory::make(std::move(canonicalQuery),
                                                     std::move(workingSet),
@@ -353,7 +391,7 @@ TEST_F(DocumentSourceCursorTest, TailableAwaitDataCursorShouldErrorAfterTimeout)
 }
 
 TEST_F(DocumentSourceCursorTest, NonAwaitDataCursorShouldErrorAfterTimeout) {
-    // Make sure the collection exists, otherwise we'll default to a NO_YIELD yield policy.
+    // Make sure the collection exists, otherwise we'll default to an INTERRUPT_ONLY yield policy.
     ASSERT_TRUE(client.createCollection(nss));
     client.insert(nss, BSON("a" << 1));
 
@@ -364,14 +402,15 @@ TEST_F(DocumentSourceCursorTest, NonAwaitDataCursorShouldErrorAfterTimeout) {
     auto filter = BSON("a" << 1);
     auto matchExpression = uassertStatusOK(MatchExpressionParser::parse(filter, ctx()));
     auto collectionScan = std::make_unique<CollectionScan>(ctx().get(),
-                                                           readLock.getCollection(),
+                                                           &readLock.getCollection(),
                                                            collScanParams,
                                                            workingSet.get(),
                                                            matchExpression.get());
     auto findCommand = std::make_unique<FindCommandRequest>(nss);
     findCommand->setFilter(filter);
-    auto canonicalQuery = unittest::assertGet(
-        CanonicalQuery::canonicalize(opCtx(), std::move(findCommand), false, nullptr));
+    auto canonicalQuery = std::make_unique<CanonicalQuery>(
+        CanonicalQueryParams{.expCtx = makeExpressionContext(opCtx(), *findCommand),
+                             .parsedFind = ParsedFindCommandParams{std::move(findCommand)}});
     auto planExecutor =
         uassertStatusOK(plan_executor_factory::make(std::move(canonicalQuery),
                                                     std::move(workingSet),
@@ -402,7 +441,7 @@ TEST_F(DocumentSourceCursorTest, TailableAwaitDataCursorShouldErrorAfterBeingKil
         return;
     }
 
-    // Make sure the collection exists, otherwise we'll default to a NO_YIELD yield policy.
+    // Make sure the collection exists, otherwise we'll default to an INTERRUPT_ONLY yield policy.
     const bool capped = true;
     const long long cappedSize = 1024;
     ASSERT_TRUE(client.createCollection(nss, cappedSize, capped));
@@ -416,7 +455,7 @@ TEST_F(DocumentSourceCursorTest, TailableAwaitDataCursorShouldErrorAfterBeingKil
     auto filter = BSON("a" << 1);
     auto matchExpression = uassertStatusOK(MatchExpressionParser::parse(filter, ctx()));
     auto collectionScan = std::make_unique<CollectionScan>(ctx().get(),
-                                                           readLock.getCollection(),
+                                                           &readLock.getCollection(),
                                                            collScanParams,
                                                            workingSet.get(),
                                                            matchExpression.get());
@@ -424,8 +463,9 @@ TEST_F(DocumentSourceCursorTest, TailableAwaitDataCursorShouldErrorAfterBeingKil
     findCommand->setFilter(filter);
     query_request_helper::setTailableMode(TailableModeEnum::kTailableAndAwaitData,
                                           findCommand.get());
-    auto canonicalQuery = unittest::assertGet(
-        CanonicalQuery::canonicalize(opCtx(), std::move(findCommand), false, nullptr));
+    auto canonicalQuery = std::make_unique<CanonicalQuery>(
+        CanonicalQueryParams{.expCtx = makeExpressionContext(opCtx(), *findCommand),
+                             .parsedFind = ParsedFindCommandParams{std::move(findCommand)}});
     auto planExecutor = uassertStatusOK(
         plan_executor_factory::make(std::move(canonicalQuery),
                                     std::move(workingSet),
@@ -448,7 +488,7 @@ TEST_F(DocumentSourceCursorTest, TailableAwaitDataCursorShouldErrorAfterBeingKil
 }
 
 TEST_F(DocumentSourceCursorTest, NormalCursorShouldErrorAfterBeingKilled) {
-    // Make sure the collection exists, otherwise we'll default to a NO_YIELD yield policy.
+    // Make sure the collection exists, otherwise we'll default to an INTERRUPT_ONLY yield policy.
     ASSERT_TRUE(client.createCollection(nss));
     client.insert(nss, BSON("a" << 1));
 
@@ -459,14 +499,15 @@ TEST_F(DocumentSourceCursorTest, NormalCursorShouldErrorAfterBeingKilled) {
     auto filter = BSON("a" << 1);
     auto matchExpression = uassertStatusOK(MatchExpressionParser::parse(filter, ctx()));
     auto collectionScan = std::make_unique<CollectionScan>(ctx().get(),
-                                                           readLock.getCollection(),
+                                                           &readLock.getCollection(),
                                                            collScanParams,
                                                            workingSet.get(),
                                                            matchExpression.get());
     auto findCommand = std::make_unique<FindCommandRequest>(nss);
     findCommand->setFilter(filter);
-    auto canonicalQuery = unittest::assertGet(
-        CanonicalQuery::canonicalize(opCtx(), std::move(findCommand), false, nullptr));
+    auto canonicalQuery = std::make_unique<CanonicalQuery>(
+        CanonicalQueryParams{.expCtx = makeExpressionContext(opCtx(), *findCommand),
+                             .parsedFind = ParsedFindCommandParams{std::move(findCommand)}});
     auto planExecutor = uassertStatusOK(
         plan_executor_factory::make(std::move(canonicalQuery),
                                     std::move(workingSet),
