@@ -32,39 +32,59 @@
  */
 
 
-#include "mongo/platform/basic.h"
-
 #include <cmath>
+#include <cstdlib>
+#include <cstring>
+#include <exception>
 #include <iostream>
+#include <iterator>
+#include <limits>
+#include <map>
+#include <set>
+#include <string>
+#include <sys/types.h>
+#include <utility>
+#include <vector>
 
+#include "mongo/base/status.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bson_comparator_interface_base.h"
 #include "mongo/bson/bson_validate.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonelement_comparator_interface.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
 #include "mongo/bson/bsonobj_comparator.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/bson/bsontypes_util.h"
+#include "mongo/bson/json.h"
+#include "mongo/bson/oid.h"
 #include "mongo/bson/simple_bsonelement_comparator.h"
+#include "mongo/bson/simple_bsonobj_comparator.h"
+#include "mongo/bson/timestamp.h"
 #include "mongo/bson/util/builder.h"
+#include "mongo/bson/util/builder_fwd.h"
 #include "mongo/db/bson/bson_helper.h"
 #include "mongo/db/bson/dotted_path_support.h"
-#include "mongo/db/jsobj.h"
-#include "mongo/db/json.h"
-#include "mongo/dbtests/dbtests.h"
+#include "mongo/dbtests/dbtests.h"  // IWYU pragma: keep
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
 #include "mongo/platform/decimal128.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
 #include "mongo/util/allocator.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/embedded_builder.h"
 #include "mongo/util/str.h"
+#include "mongo/util/time_support.h"
 #include "mongo/util/timer.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
 
-
 namespace mongo {
 
-using std::cout;
-using std::endl;
-using std::numeric_limits;
-using std::string;
-using std::stringstream;
-using std::vector;
-
 namespace dps = ::mongo::dotted_path_support;
 
 namespace {
@@ -107,17 +127,16 @@ BSONObj dotted2nested(const BSONObj& obj) {
     return b.obj();
 }
 
-
 // {a: {b:1}} -> {a.b:1}
-void nested2dotted(BSONObjBuilder& b, const BSONObj& obj, const string& base = "") {
+void nested2dotted(BSONObjBuilder& b, const BSONObj& obj, const std::string& base = "") {
     BSONObjIterator it(obj);
     while (it.more()) {
         BSONElement e = it.next();
         if (e.type() == Object) {
-            string newbase = base + e.fieldName() + ".";
+            std::string newbase = base + e.fieldName() + ".";
             nested2dotted(b, e.embeddedObject(), newbase);
         } else {
-            string newbase = base + e.fieldName();
+            std::string newbase = base + e.fieldName();
             b.appendAs(e, newbase);
         }
     }
@@ -129,8 +148,8 @@ BSONObj nested2dotted(const BSONObj& obj) {
     return b.obj();
 }
 
-FieldCompareResult compareDottedFieldNames(const string& l,
-                                           const string& r,
+FieldCompareResult compareDottedFieldNames(const std::string& l,
+                                           const std::string& r,
                                            const str::LexNumCmp& cmp) {
     static int maxLoops = 1024 * 1024;
 
@@ -141,11 +160,11 @@ FieldCompareResult compareDottedFieldNames(const string& l,
         size_t a = l.find('.', lstart);
         size_t b = r.find('.', rstart);
 
-        size_t lend = a == string::npos ? l.size() : a;
-        size_t rend = b == string::npos ? r.size() : b;
+        size_t lend = a == std::string::npos ? l.size() : a;
+        size_t rend = b == std::string::npos ? r.size() : b;
 
-        const string& c = l.substr(lstart, lend - lstart);
-        const string& d = r.substr(rstart, rend - rstart);
+        const std::string& c = l.substr(lstart, lend - lstart);
+        const std::string& d = r.substr(rstart, rend - rstart);
 
         int x = cmp.cmp(c.c_str(), d.c_str());
 
@@ -173,7 +192,6 @@ FieldCompareResult compareDottedFieldNames(const string& l,
     MONGO_verify(0);
     return SAME;  // will never get here
 }
-}  // namespace mongo
 
 namespace JsobjTests {
 
@@ -237,7 +255,7 @@ protected:
         b.append(name, val);
         return b.obj();
     }
-    static BSONObj basic(const char* name, vector<int> val) {
+    static BSONObj basic(const char* name, std::vector<int> val) {
         BSONObjBuilder b;
         b.append(name, val);
         return b.obj();
@@ -315,15 +333,14 @@ public:
 class WoCompareEmbeddedArray : public Base {
 public:
     void run() {
-        vector<int> i;
+        std::vector<int> i;
         i.push_back(1);
         i.push_back(2);
-        vector<double> d;
+        std::vector<double> d;
         d.push_back(1);
         d.push_back(2);
         ASSERT(basic("a", i).woCompare(basic("a", d)) == 0);
-
-        vector<int> j;
+        std::vector<int> j;
         j.push_back(1);
         j.push_back(3);
         ASSERT(basic("a", i).woCompare(basic("a", j)) < 0);
@@ -472,9 +489,9 @@ public:
 class Nan : public Base {
 public:
     void run() {
-        double inf = numeric_limits<double>::infinity();
-        double nan = numeric_limits<double>::quiet_NaN();
-        double nan2 = numeric_limits<double>::signaling_NaN();
+        double inf = std::numeric_limits<double>::infinity();
+        double nan = std::numeric_limits<double>::quiet_NaN();
+        double nan2 = std::numeric_limits<double>::signaling_NaN();
         ASSERT(std::isnan(nan));
         ASSERT(std::isnan(nan2));
         ASSERT(!std::isnan(inf));
@@ -644,7 +661,7 @@ public:
             c.append("z", true);
             BSONObj C = c.obj();
 
-            // test that nulls are ok within bson strings
+            // test that nulls are ok within bson std::strings
             ASSERT_BSONOBJ_NE(A, B);
             ASSERT_BSONOBJ_GT(A, B);
 
@@ -657,12 +674,12 @@ public:
 
         BSONObjBuilder b;
         b.append("a", "a\0b", 4);
-        string z("a\0b", 3);
+        std::string z("a\0b", 3);
         b.append("b", z);
         b.appendAs(b.asTempObj()["a"], "c");
         BSONObj o = b.obj();
 
-        stringstream ss;
+        std::stringstream ss;
         ss << 'a' << '\0' << 'b';
 
         ASSERT_EQUALS(o["a"].valuestrsize(), 3 + 1);
@@ -705,7 +722,7 @@ public:
     void run() {
         BSONObj nestedBSON;
         StringBuilder s;
-        string nestedBSONString;
+        std::string nestedBSONString;
         size_t found;
 
         // recursion depth one less than max allowed-- do not shorten the string
@@ -714,7 +731,7 @@ public:
         nestedBSONString = s.str();
         found = nestedBSONString.find("...");
         // did not find the "..." pattern
-        ASSERT_EQUALS(found != string::npos, false);
+        ASSERT_EQUALS(found != std::string::npos, false);
 
         // recursion depth is equal to max allowed  -- do not shorten the string
         nestedBSON = recursiveBSON(BSONObj::maxToStringRecursionDepth);
@@ -722,7 +739,7 @@ public:
         nestedBSONString = s.str();
         found = nestedBSONString.find("...");
         // did not find the "..." pattern
-        ASSERT_EQUALS(found != string::npos, false);
+        ASSERT_EQUALS(found != std::string::npos, false);
 
         // recursion depth - one greater than max allowed -- shorten the string
         nestedBSON = recursiveBSON(BSONObj::maxToStringRecursionDepth + 1);
@@ -730,7 +747,7 @@ public:
         nestedBSONString = s.str();
         found = nestedBSONString.find("...");
         // found the "..." pattern
-        ASSERT_EQUALS(found != string::npos, true);
+        ASSERT_EQUALS(found != std::string::npos, true);
 
         /* recursion depth - one greater than max allowed but with full=true
          * should fail with an assertion
@@ -743,11 +760,11 @@ public:
 class StringWithNull {
 public:
     void run() {
-        const string input = string("a") + '\0' + 'b';
+        const std::string input = std::string("a") + '\0' + 'b';
         ASSERT_EQUALS(input.size(), 3U);
 
         BSONObj obj = BSON("str" << input);
-        const string output = obj.firstElement().String();
+        const std::string output = obj.firstElement().String();
         ASSERT_EQUALS(str::escape(output), str::escape(input));  // for better failure output
         ASSERT_EQUALS(output, input);
     }
@@ -779,10 +796,10 @@ protected:
 };
 
 class BadType : public Base {
-    BSONObj valid() const {
+    BSONObj valid() const override {
         return fromjson("{\"a\":1}");
     }
-    BSONObj invalid() const {
+    BSONObj invalid() const override {
         BSONObj ret = valid();
         set(ret, 4, 50);
         return ret;
@@ -790,10 +807,10 @@ class BadType : public Base {
 };
 
 class EooBeforeEnd : public Base {
-    BSONObj valid() const {
+    BSONObj valid() const override {
         return fromjson("{\"a\":1}");
     }
-    BSONObj invalid() const {
+    BSONObj invalid() const override {
         BSONObj ret = valid();
         // (first byte of size)++
         set(ret, 0, get(ret, 0) + 1);
@@ -814,10 +831,10 @@ public:
 };
 
 class TotalSizeTooSmall : public Base {
-    BSONObj valid() const {
+    BSONObj valid() const override {
         return fromjson("{\"a\":1}");
     }
-    BSONObj invalid() const {
+    BSONObj invalid() const override {
         BSONObj ret = valid();
         // (first byte of size)--
         set(ret, 0, get(ret, 0) - 1);
@@ -827,10 +844,10 @@ class TotalSizeTooSmall : public Base {
 };
 
 class EooMissing : public Base {
-    BSONObj valid() const {
+    BSONObj valid() const override {
         return fromjson("{\"a\":1}");
     }
-    BSONObj invalid() const {
+    BSONObj invalid() const override {
         BSONObj ret = valid();
         set(ret, ret.objsize() - 1, (char)0xff);
         // (first byte of size)--
@@ -841,24 +858,25 @@ class EooMissing : public Base {
 };
 
 class WrongStringSize : public Base {
-    BSONObj valid() const {
+    BSONObj valid() const override {
         return fromjson("{\"a\":\"b\"}");
     }
-    BSONObj invalid() const {
+    BSONObj invalid() const override {
         BSONObj ret = valid();
-        ASSERT_TRUE(ret.firstElement().valueStringData().size() >= 1);
-        ASSERT_EQUALS(ret.firstElement().valueStringData()[0], 'b');
-        ASSERT_EQUALS(ret.firstElement().valueStringData()[1], 0);
-        ((char*)ret.firstElement().valueStringData().rawData())[1] = 1;
+        auto val = ret.firstElement().valueStringData();
+        ASSERT_EQUALS(val, "b"_sd);
+        auto d = const_cast<char*>(val.data());
+        ASSERT_EQUALS(d[1], 0);
+        d[1] = 1;
         return ret.copy();
     }
 };
 
 class ZeroStringSize : public Base {
-    BSONObj valid() const {
+    BSONObj valid() const override {
         return fromjson("{\"a\":\"b\"}");
     }
-    BSONObj invalid() const {
+    BSONObj invalid() const override {
         BSONObj ret = valid();
         set(ret, 7, 0);
         return ret;
@@ -866,10 +884,10 @@ class ZeroStringSize : public Base {
 };
 
 class NegativeStringSize : public Base {
-    BSONObj valid() const {
+    BSONObj valid() const override {
         return fromjson("{\"a\":\"b\"}");
     }
-    BSONObj invalid() const {
+    BSONObj invalid() const override {
         BSONObj ret = valid();
         set(ret, 10, -100);
         return ret;
@@ -877,10 +895,10 @@ class NegativeStringSize : public Base {
 };
 
 class WrongSubobjectSize : public Base {
-    BSONObj valid() const {
+    BSONObj valid() const override {
         return fromjson("{\"a\":{\"b\":1}}");
     }
-    BSONObj invalid() const {
+    BSONObj invalid() const override {
         BSONObj ret = valid();
         set(ret, 0, get(ret, 0) + 1);
         set(ret, 7, get(ret, 7) + 1);
@@ -889,10 +907,10 @@ class WrongSubobjectSize : public Base {
 };
 
 class WrongDbrefNsSize : public Base {
-    BSONObj valid() const {
+    BSONObj valid() const override {
         return fromjson("{ \"a\": Dbref( \"b\", \"ffffffffffffffffffffffff\" ) }");
     }
-    BSONObj invalid() const {
+    BSONObj invalid() const override {
         BSONObj ret = valid();
         set(ret, 0, get(ret, 0) + 1);
         set(ret, 7, get(ret, 7) + 1);
@@ -901,10 +919,10 @@ class WrongDbrefNsSize : public Base {
 };
 
 class NoFieldNameEnd : public Base {
-    BSONObj valid() const {
+    BSONObj valid() const override {
         return fromjson("{\"a\":1}");
     }
-    BSONObj invalid() const {
+    BSONObj invalid() const override {
         BSONObj ret = valid();
         memset(const_cast<char*>(ret.objdata()) + 5, 0xff, ret.objsize() - 5);
         return ret;
@@ -912,10 +930,10 @@ class NoFieldNameEnd : public Base {
 };
 
 class BadRegex : public Base {
-    BSONObj valid() const {
+    BSONObj valid() const override {
         return fromjson("{\"a\":/c/i}");
     }
-    BSONObj invalid() const {
+    BSONObj invalid() const override {
         BSONObj ret = valid();
         memset(const_cast<char*>(ret.objdata()) + 7, 0xff, ret.objsize() - 7);
         return ret;
@@ -923,10 +941,10 @@ class BadRegex : public Base {
 };
 
 class BadRegexOptions : public Base {
-    BSONObj valid() const {
+    BSONObj valid() const override {
         return fromjson("{\"a\":/c/i}");
     }
-    BSONObj invalid() const {
+    BSONObj invalid() const override {
         BSONObj ret = valid();
         memset(const_cast<char*>(ret.objdata()) + 9, 0xff, ret.objsize() - 9);
         return ret;
@@ -934,14 +952,14 @@ class BadRegexOptions : public Base {
 };
 
 class CodeWScopeBase : public Base {
-    BSONObj valid() const {
+    BSONObj valid() const override {
         BSONObjBuilder b;
         BSONObjBuilder scope;
         scope.append("a", "b");
         b.appendCodeWScope("c", "d", scope.done());
         return b.obj();
     }
-    BSONObj invalid() const {
+    BSONObj invalid() const override {
         BSONObj ret = valid();
         modify(ret);
         return ret;
@@ -952,37 +970,37 @@ protected:
 };
 
 class CodeWScopeSmallSize : public CodeWScopeBase {
-    void modify(BSONObj& o) const {
+    void modify(BSONObj& o) const override {
         set(o, 7, 7);
     }
 };
 
 class CodeWScopeZeroStrSize : public CodeWScopeBase {
-    void modify(BSONObj& o) const {
+    void modify(BSONObj& o) const override {
         set(o, 11, 0);
     }
 };
 
 class CodeWScopeSmallStrSize : public CodeWScopeBase {
-    void modify(BSONObj& o) const {
+    void modify(BSONObj& o) const override {
         set(o, 11, 1);
     }
 };
 
 class CodeWScopeNoSizeForObj : public CodeWScopeBase {
-    void modify(BSONObj& o) const {
+    void modify(BSONObj& o) const override {
         set(o, 7, 13);
     }
 };
 
 class CodeWScopeSmallObjSize : public CodeWScopeBase {
-    void modify(BSONObj& o) const {
+    void modify(BSONObj& o) const override {
         set(o, 17, 1);
     }
 };
 
 class CodeWScopeBadObject : public CodeWScopeBase {
-    void modify(BSONObj& o) const {
+    void modify(BSONObj& o) const override {
         set(o, 21, JSTypeMax + 1);
     }
 };
@@ -1115,22 +1133,22 @@ protected:
 };
 
 class LabelBasic : public LabelBase {
-    BSONObj expected() {
+    BSONObj expected() override {
         return BSON("a" << (BSON("$gt" << 1)));
     }
-    BSONObj actual() {
+    BSONObj actual() override {
         return BSON("a" << GT << 1);
     }
 };
 
 class LabelShares : public LabelBase {
-    BSONObj expected() {
+    BSONObj expected() override {
         return BSON("z"
                     << "q"
                     << "a" << (BSON("$gt" << 1)) << "x"
                     << "p");
     }
-    BSONObj actual() {
+    BSONObj actual() override {
         return BSON("z"
                     << "q"
                     << "a" << GT << 1 << "x"
@@ -1139,17 +1157,17 @@ class LabelShares : public LabelBase {
 };
 
 class LabelDouble : public LabelBase {
-    BSONObj expected() {
+    BSONObj expected() override {
         return BSON("a" << (BSON("$gt" << 1 << "$lte"
                                        << "x")));
     }
-    BSONObj actual() {
+    BSONObj actual() override {
         return BSON("a" << GT << 1 << LTE << "x");
     }
 };
 
 class LabelDoubleShares : public LabelBase {
-    BSONObj expected() {
+    BSONObj expected() override {
         return BSON("z"
                     << "q"
                     << "a"
@@ -1158,7 +1176,7 @@ class LabelDoubleShares : public LabelBase {
                     << "x"
                     << "p");
     }
-    BSONObj actual() {
+    BSONObj actual() override {
         return BSON("z"
                     << "q"
                     << "a" << GT << 1 << LTE << "x"
@@ -1168,16 +1186,16 @@ class LabelDoubleShares : public LabelBase {
 };
 
 class LabelSize : public LabelBase {
-    BSONObj expected() {
+    BSONObj expected() override {
         return BSON("a" << BSON("$size" << 4));
     }
-    BSONObj actual() {
+    BSONObj actual() override {
         return BSON("a" << mongo::BSIZE << 4);
     }
 };
 
 class LabelMulti : public LabelBase {
-    BSONObj expected() {
+    BSONObj expected() override {
         return BSON("z"
                     << "q"
                     << "a"
@@ -1190,7 +1208,7 @@ class LabelMulti : public LabelBase {
                     << "x"
                     << "p");
     }
-    BSONObj actual() {
+    BSONObj actual() override {
         return BSON("z"
                     << "q"
                     << "a" << GT << 1 << LTE << "x"
@@ -1199,7 +1217,7 @@ class LabelMulti : public LabelBase {
     }
 };
 class LabelishOr : public LabelBase {
-    BSONObj expected() {
+    BSONObj expected() override {
         return BSON("$or" << BSON_ARRAY(BSON("a" << BSON("$gt" << 1 << "$lte"
                                                                << "x"))
                                         << BSON("b" << BSON("$ne" << 1 << "$ne"
@@ -1208,7 +1226,7 @@ class LabelishOr : public LabelBase {
                                         << BSON("x"
                                                 << "p")));
     }
-    BSONObj actual() {
+    BSONObj actual() override {
         return BSON(OR(BSON("a" << GT << 1 << LTE << "x"),
                        BSON("b" << NE << 1 << NE << "f" << NE << 22.3),
                        BSON("x"
@@ -1250,8 +1268,8 @@ public:
 
         ASSERT_EQUALS(objTypeOf(""), String);
         ASSERT_EQUALS(arrTypeOf(""), String);
-        ASSERT_EQUALS(objTypeOf(string()), String);
-        ASSERT_EQUALS(arrTypeOf(string()), String);
+        ASSERT_EQUALS(objTypeOf(std::string()), String);
+        ASSERT_EQUALS(arrTypeOf(std::string()), String);
         ASSERT_EQUALS(objTypeOf(StringData("")), String);
         ASSERT_EQUALS(arrTypeOf(StringData("")), String);
 
@@ -1411,9 +1429,9 @@ public:
 
     void run() {
         for (int t = 1; t < JSTypeMax; t++) {
-            stringstream ss;
+            std::stringstream ss;
             ss << "type: " << t;
-            string s = ss.str();
+            std::string s = ss.str();
             ASSERT(min(t).woCompare(max(t)) <= 0);
             ASSERT(max(t).woCompare(min(t)) >= 0);
             ASSERT(min(t).woCompare(min(t)) == 0);
@@ -1424,10 +1442,10 @@ public:
 
 class ComparatorTest {
 public:
-    BSONObj one(string s) {
+    BSONObj one(std::string s) {
         return BSON("x" << s);
     }
-    BSONObj two(string x, string y) {
+    BSONObj two(std::string x, std::string y) {
         BSONObjBuilder b;
         b.append("x", x);
         if (y.size())
@@ -1438,14 +1456,14 @@ public:
     }
 
     void test(BSONObj order, BSONObj l, BSONObj r, bool wanted) {
-        const StringData::ComparatorInterface* stringComparator = nullptr;
+        const StringDataComparator* stringComparator = nullptr;
         BSONObjComparator bsonCmp(
             order, BSONObjComparator::FieldNamesMode::kConsider, stringComparator);
         bool got = bsonCmp.makeLessThan()(l, r);
         if (got == wanted)
             return;
-        cout << " order: " << order << " l: " << l << "r: " << r << " wanted: " << wanted
-             << " got: " << got << endl;
+        std::cout << " order: " << order << " l: " << l << "r: " << r << " wanted: " << wanted
+                  << " got: " << got << std::endl;
     }
 
     void lt(BSONObj order, BSONObj l, BSONObj r) {
@@ -1493,7 +1511,7 @@ public:
 
 class CompareDottedFieldNamesTest {
 public:
-    void t(FieldCompareResult res, const string& l, const string& r) {
+    void t(FieldCompareResult res, const std::string& l, const std::string& r) {
         str::LexNumCmp cmp(true);
         ASSERT_EQUALS(res, compareDottedFieldNames(l, r, cmp));
         ASSERT_EQUALS(-1 * res, compareDottedFieldNames(r, l, cmp));
@@ -1514,7 +1532,7 @@ public:
 
 class CompareDottedArrayFieldNamesTest {
 public:
-    void t(FieldCompareResult res, const string& l, const string& r) {
+    void t(FieldCompareResult res, const std::string& l, const std::string& r) {
         str::LexNumCmp cmp(false);  // Specify numeric comparison for array field names.
         ASSERT_EQUALS(res, compareDottedFieldNames(l, r, cmp));
         ASSERT_EQUALS(-1 * res, compareDottedFieldNames(r, l, cmp));
@@ -1569,8 +1587,8 @@ struct BSONArrayBuilderTest {
         objb << fieldNameGenerator() << "Hello";
         arrb << "Hello";
 
-        objb << fieldNameGenerator() << string("World");
-        arrb << string("World");
+        objb << fieldNameGenerator() << std::string("World");
+        arrb << std::string("World");
 
         objb << fieldNameGenerator()
              << BSON("a" << 1 << "b"
@@ -1650,7 +1668,7 @@ public:
     void run() {
         BSONObj o = BSON("z" << 1 << "a" << 2 << "m" << 3 << "c" << 4);
         BSONObjIteratorSorted i(o);
-        stringstream ss;
+        std::stringstream ss;
         while (i.more())
             ss << i.next().fieldName();
         ASSERT_EQUALS("acmz", ss.str());
@@ -1673,7 +1691,7 @@ public:
                           << "b");
         BSONObjIteratorSorted i2(o2);
         // First field in sorted order should be "11" due use of a lexical comparison.
-        ASSERT_EQUALS("11", string(i2.next().fieldName()));
+        ASSERT_EQUALS("11", std::string(i2.next().fieldName()));
     }
 };
 
@@ -1691,7 +1709,7 @@ public:
         mongo::BSONArrayIteratorSorted sorted(arr);
         while (unsorted.more()) {
             ASSERT(sorted.more());
-            ASSERT_EQUALS(string(unsorted.next().fieldName()), sorted.next().fieldName());
+            ASSERT_EQUALS(std::string(unsorted.next().fieldName()), sorted.next().fieldName());
         }
         ASSERT(!sorted.more());
     }
@@ -1699,24 +1717,24 @@ public:
 
 class checkForStorageTests {
 public:
-    void good(string s) {
+    void good(std::string s) {
         good(fromjson(s));
     }
 
     void good(BSONObj o) {
         if (o.storageValidEmbedded().isOK())
             return;
-        uasserted(12528, (string) "should be ok for storage:" + o.toString());
+        uasserted(12528, std::string("should be ok for storage:") + o.toString());
     }
 
-    void bad(string s) {
+    void bad(std::string s) {
         bad(fromjson(s));
     }
 
     void bad(BSONObj o) {
         if (!o.storageValidEmbedded().isOK())
             return;
-        uasserted(12529, (string) "should NOT be ok for storage:" + o.toString());
+        uasserted(12529, std::string("should NOT be ok for storage:") + o.toString());
     }
 
     void run() {
@@ -1902,7 +1920,7 @@ public:
 
         int count = 0;
         BSONForEach(e, obj) {
-            ASSERT_EQUALS(e.fieldName(), string("a"));
+            ASSERT_EQUALS(e.fieldName(), std::string("a"));
             count += e.Int();
         }
 
@@ -1937,7 +1955,7 @@ public:
             BSONObjBuilder outer;
             BSONObjBuilder inner(outer.subobjStart("inner"));
 
-            string bigStr(1000, 'x');
+            std::string bigStr(1000, 'x');
             while (true) {
                 ASSERT_LESS_THAN_OR_EQUALS(inner.len(), BufferMaxSize);
                 inner.append("", bigStr);
@@ -1951,11 +1969,11 @@ public:
     }
 };
 
-class All : public OldStyleSuiteSpecification {
+class All : public unittest::OldStyleSuiteSpecification {
 public:
     All() : OldStyleSuiteSpecification("jsobj") {}
 
-    void setupTests() {
+    void setupTests() override {
         add<BufBuilderBasic>();
         add<BufBuilderReallocLimit>();
         add<BSONElementBasic>();
@@ -2046,6 +2064,7 @@ public:
     }
 };
 
-OldStyleSuiteInitializer<All> myall;
+unittest::OldStyleSuiteInitializer<All> myall;
 
 }  // namespace JsobjTests
+}  // namespace mongo
