@@ -27,12 +27,17 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <mutex>
+#include <utility>
 
+#include <absl/container/node_hash_map.h>
+
+#include "mongo/base/init.h"  // IWYU pragma: keep
+#include "mongo/base/initializer.h"
+#include "mongo/base/string_data.h"
 #include "mongo/dbtests/mock/mock_conn_registry.h"
-
-#include "mongo/base/init.h"
 #include "mongo/dbtests/mock/mock_dbclient_connection.h"
+#include "mongo/util/assert_util.h"
 
 namespace mongo {
 
