@@ -31,35 +31,84 @@
  * This file tests db/exec/collection_scan.cpp.
  */
 
-#include <fmt/printf.h>
+#include <algorithm>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+#include <fmt/format.h>
+#include <fmt/printf.h>  // IWYU pragma: keep
+#include <memory>
+#include <ostream>
+#include <string>
+#include <utility>
+#include <vector>
 
-#include "mongo/client/dbclient_cursor.h"
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/json.h"
+#include "mongo/bson/oid.h"
+#include "mongo/bson/timestamp.h"
 #include "mongo/db/catalog/clustered_collection_options_gen.h"
 #include "mongo/db/catalog/clustered_collection_util.h"
 #include "mongo/db/catalog/collection.h"
+#include "mongo/db/catalog/collection_options.h"
 #include "mongo/db/catalog/database.h"
+#include "mongo/db/catalog_raii.h"
 #include "mongo/db/client.h"
+#include "mongo/db/concurrency/d_concurrency.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
 #include "mongo/db/db_raii.h"
 #include "mongo/db/dbdirectclient.h"
 #include "mongo/db/exec/collection_scan.h"
+#include "mongo/db/exec/collection_scan_common.h"
+#include "mongo/db/exec/document_value/document.h"
+#include "mongo/db/exec/document_value/value.h"
 #include "mongo/db/exec/plan_stage.h"
-#include "mongo/db/json.h"
+#include "mongo/db/exec/plan_stats.h"
+#include "mongo/db/exec/working_set.h"
+#include "mongo/db/matcher/expression.h"
+#include "mongo/db/matcher/expression_leaf.h"
 #include "mongo/db/matcher/expression_parser.h"
 #include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/pipeline/expression_context.h"
+#include "mongo/db/query/plan_executor.h"
 #include "mongo/db/query/plan_executor_factory.h"
+#include "mongo/db/query/plan_yield_policy.h"
 #include "mongo/db/query/query_planner_params.h"
+#include "mongo/db/query/record_id_bound.h"
+#include "mongo/db/record_id.h"
 #include "mongo/db/record_id_helpers.h"
-#include "mongo/db/storage/record_store.h"
-#include "mongo/dbtests/dbtests.h"
+#include "mongo/db/service_context.h"
+#include "mongo/db/storage/recovery_unit.h"
+#include "mongo/db/storage/snapshot.h"
+#include "mongo/db/storage/write_unit_of_work.h"
+#include "mongo/db/tenant_id.h"
+#include "mongo/db/transaction_resources.h"
+#include "mongo/dbtests/dbtests.h"  // IWYU pragma: keep
 #include "mongo/logv2/log.h"
-#include "mongo/util/fail_point.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/duration.h"
+#include "mongo/util/intrusive_counter.h"
+#include "mongo/util/time_support.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
 
 namespace mongo {
 namespace query_stage_collection_scan {
 
-static const NamespaceString kNss{"unittests.QueryStageCollectionScan"};
+static const NamespaceString kNss =
+    NamespaceString::createNamespaceString_forTest("unittests.QueryStageCollectionScan");
 
 //
 // Stage-specific tests.
@@ -68,7 +117,7 @@ static const NamespaceString kNss{"unittests.QueryStageCollectionScan"};
 class QueryStageCollectionScanTest : public unittest::Test {
 public:
     QueryStageCollectionScanTest() : _client(&_opCtx) {
-        dbtests::WriteContextForTests ctx(&_opCtx, kNss.ns());
+        dbtests::WriteContextForTests ctx(&_opCtx, kNss.ns_forTest());
 
         for (int i = 0; i < numObj(); ++i) {
             BSONObjBuilder bob;
@@ -77,8 +126,8 @@ public:
         }
     }
 
-    virtual ~QueryStageCollectionScanTest() {
-        dbtests::WriteContextForTests ctx(&_opCtx, kNss.ns());
+    ~QueryStageCollectionScanTest() override {
+        dbtests::WriteContextForTests ctx(&_opCtx, kNss.ns_forTest());
         _client.dropCollection(kNss);
     }
 
@@ -103,14 +152,14 @@ public:
         // Make a scan and have the runner own it.
         std::unique_ptr<WorkingSet> ws = std::make_unique<WorkingSet>();
         std::unique_ptr<PlanStage> ps = std::make_unique<CollectionScan>(
-            _expCtx.get(), collection.getCollection(), params, ws.get(), filterExpr.get());
+            _expCtx.get(), &collection.getCollection(), params, ws.get(), filterExpr.get());
 
         auto statusWithPlanExecutor =
             plan_executor_factory::make(_expCtx,
                                         std::move(ws),
                                         std::move(ps),
                                         &collection.getCollection(),
-                                        PlanYieldPolicy::YieldPolicy::NO_YIELD,
+                                        PlanYieldPolicy::YieldPolicy::INTERRUPT_ONLY,
                                         QueryPlannerParams::DEFAULT);
         ASSERT_OK(statusWithPlanExecutor.getStatus());
         auto exec = std::move(statusWithPlanExecutor.getValue());
@@ -135,7 +184,7 @@ public:
         params.tailable = false;
 
         std::unique_ptr<CollectionScan> scan(
-            new CollectionScan(_expCtx.get(), collection, params, &ws, nullptr));
+            new CollectionScan(_expCtx.get(), &collection, params, &ws, nullptr));
         while (!scan->isEOF()) {
             WorkingSetID id = WorkingSet::INVALID_ID;
             PlanStage::StageState state = scan->work(&id);
@@ -231,7 +280,7 @@ public:
         params.maxRecord = RecordIdBound(maxRecord);
 
         WorkingSet ws;
-        auto scan = std::make_unique<CollectionScan>(_expCtx.get(), coll, params, &ws, nullptr);
+        auto scan = std::make_unique<CollectionScan>(_expCtx.get(), &coll, params, &ws, nullptr);
 
         int count = 0;
         while (!scan->isEOF()) {
@@ -276,7 +325,7 @@ public:
         params.boundInclusion = boundInclusion;
 
         WorkingSet ws;
-        auto scan = std::make_unique<CollectionScan>(_expCtx.get(), coll, params, &ws, filter);
+        auto scan = std::make_unique<CollectionScan>(_expCtx.get(), &coll, params, &ws, filter);
 
         int idx = 0;
         while (!scan->isEOF()) {
@@ -337,7 +386,7 @@ TEST_F(QueryStageCollectionScanTest, QueryStageCollscanBasicBackwardWithMatch) {
 
 TEST_F(QueryStageCollectionScanTest,
        QueryTestCollscanStopsScanningOnFilterFailureInClusteredCollectionIfSpecified) {
-    auto ns = NamespaceString("a.b");
+    auto ns = NamespaceString::createNamespaceString_forTest("a.b");
     auto collDeleter = createClusteredCollection(ns, false /* prePopulate */);
     for (int i = 1; i <= numObj(); ++i) {
         insertDocument(ns, BSON("_id" << i << "foo" << i));
@@ -356,7 +405,7 @@ TEST_F(QueryStageCollectionScanTest,
     params.shouldReturnEofOnFilterMismatch = true;
     WorkingSet ws;
     LTEMatchExpression filter{"foo"_sd, Value(threshold)};
-    auto scan = std::make_unique<CollectionScan>(_expCtx.get(), coll, params, &ws, &filter);
+    auto scan = std::make_unique<CollectionScan>(_expCtx.get(), &coll, params, &ws, &filter);
 
     // Scan all matching documents.
     WorkingSetID id = WorkingSet::INVALID_ID;
@@ -379,14 +428,14 @@ TEST_F(QueryStageCollectionScanTest, QueryStageCollscanObjectsInOrderForward) {
     // Make a scan and have the runner own it.
     std::unique_ptr<WorkingSet> ws = std::make_unique<WorkingSet>();
     std::unique_ptr<PlanStage> ps = std::make_unique<CollectionScan>(
-        _expCtx.get(), collection.getCollection(), params, ws.get(), nullptr);
+        _expCtx.get(), &collection.getCollection(), params, ws.get(), nullptr);
 
     auto statusWithPlanExecutor =
         plan_executor_factory::make(_expCtx,
                                     std::move(ws),
                                     std::move(ps),
                                     &collection.getCollection(),
-                                    PlanYieldPolicy::YieldPolicy::NO_YIELD,
+                                    PlanYieldPolicy::YieldPolicy::INTERRUPT_ONLY,
                                     QueryPlannerParams::DEFAULT);
     ASSERT_OK(statusWithPlanExecutor.getStatus());
     auto exec = std::move(statusWithPlanExecutor.getValue());
@@ -412,14 +461,14 @@ TEST_F(QueryStageCollectionScanTest, QueryStageCollscanObjectsInOrderBackward) {
 
     std::unique_ptr<WorkingSet> ws = std::make_unique<WorkingSet>();
     std::unique_ptr<PlanStage> ps = std::make_unique<CollectionScan>(
-        _expCtx.get(), collection.getCollection(), params, ws.get(), nullptr);
+        _expCtx.get(), &collection.getCollection(), params, ws.get(), nullptr);
 
     auto statusWithPlanExecutor =
         plan_executor_factory::make(_expCtx,
                                     std::move(ws),
                                     std::move(ps),
                                     &collection.getCollection(),
-                                    PlanYieldPolicy::YieldPolicy::NO_YIELD,
+                                    PlanYieldPolicy::YieldPolicy::INTERRUPT_ONLY,
                                     QueryPlannerParams::DEFAULT);
     ASSERT_OK(statusWithPlanExecutor.getStatus());
     auto exec = std::move(statusWithPlanExecutor.getValue());
@@ -437,7 +486,7 @@ TEST_F(QueryStageCollectionScanTest, QueryStageCollscanObjectsInOrderBackward) {
 // Scan through half the objects, delete the one we're about to fetch, then expect to get the "next"
 // object we would have gotten after that.
 TEST_F(QueryStageCollectionScanTest, QueryStageCollscanDeleteUpcomingObject) {
-    dbtests::WriteContextForTests ctx(&_opCtx, kNss.ns());
+    dbtests::WriteContextForTests ctx(&_opCtx, kNss.ns_forTest());
 
     const CollectionPtr& coll = ctx.getCollection();
 
@@ -451,7 +500,7 @@ TEST_F(QueryStageCollectionScanTest, QueryStageCollscanDeleteUpcomingObject) {
     params.tailable = false;
 
     WorkingSet ws;
-    std::unique_ptr<PlanStage> scan(new CollectionScan(_expCtx.get(), coll, params, &ws, nullptr));
+    std::unique_ptr<PlanStage> scan(new CollectionScan(_expCtx.get(), &coll, params, &ws, nullptr));
 
     int count = 0;
     while (count < 10) {
@@ -491,7 +540,7 @@ TEST_F(QueryStageCollectionScanTest, QueryStageCollscanDeleteUpcomingObject) {
 // Scan through half the objects, delete the one we're about to fetch, then expect to get the "next"
 // object we would have gotten after that.  But, do it in reverse!
 TEST_F(QueryStageCollectionScanTest, QueryStageCollscanDeleteUpcomingObjectBackward) {
-    dbtests::WriteContextForTests ctx(&_opCtx, kNss.ns());
+    dbtests::WriteContextForTests ctx(&_opCtx, kNss.ns_forTest());
     const CollectionPtr& coll = ctx.getCollection();
 
     // Get the RecordIds that would be returned by an in-order scan.
@@ -504,7 +553,7 @@ TEST_F(QueryStageCollectionScanTest, QueryStageCollscanDeleteUpcomingObjectBackw
     params.tailable = false;
 
     WorkingSet ws;
-    std::unique_ptr<PlanStage> scan(new CollectionScan(_expCtx.get(), coll, params, &ws, nullptr));
+    std::unique_ptr<PlanStage> scan(new CollectionScan(_expCtx.get(), &coll, params, &ws, nullptr));
 
     int count = 0;
     while (count < 10) {
@@ -558,14 +607,12 @@ TEST_F(QueryStageCollectionScanTest, QueryTestCollscanResumeAfterRecordIdSeekSuc
     params.direction = CollectionScanParams::FORWARD;
 
     // Pick a recordId that is known to be in the collection.
-    params.resumeScanPoint = ResumeScanPoint{
-        recordIds[offset - 1], false /* tolerateKeyNotFound */
-    };
+    params.resumeAfterRecordId = recordIds[offset - 1];
 
     // Create plan stage.
     std::unique_ptr<WorkingSet> ws = std::make_unique<WorkingSet>();
     std::unique_ptr<PlanStage> ps = std::make_unique<CollectionScan>(
-        _expCtx.get(), collection.getCollection(), params, ws.get(), nullptr);
+        _expCtx.get(), &collection.getCollection(), params, ws.get(), nullptr);
 
     // Run the rest of the scan and verify the results.
     auto statusWithPlanExecutor =
@@ -573,7 +620,7 @@ TEST_F(QueryStageCollectionScanTest, QueryTestCollscanResumeAfterRecordIdSeekSuc
                                     std::move(ws),
                                     std::move(ps),
                                     &collection.getCollection(),
-                                    PlanYieldPolicy::YieldPolicy::NO_YIELD,
+                                    PlanYieldPolicy::YieldPolicy::INTERRUPT_ONLY,
                                     QueryPlannerParams::DEFAULT);
     ASSERT_OK(statusWithPlanExecutor.getStatus());
     auto exec = std::move(statusWithPlanExecutor.getValue());
@@ -591,8 +638,8 @@ TEST_F(QueryStageCollectionScanTest, QueryTestCollscanResumeAfterRecordIdSeekSuc
 
 // Verify that if we fail to seek to the resumeAfterRecordId, the plan stage fails.
 TEST_F(QueryStageCollectionScanTest, QueryTestCollscanResumeAfterRecordIdSeekFailure) {
-    dbtests::WriteContextForTests ctx(&_opCtx, kNss.ns());
-    const CollectionPtr& coll = ctx.getCollection();
+    dbtests::WriteContextForTests ctx(&_opCtx, kNss.ns_forTest());
+    auto coll = ctx.getCollection();
 
     // Get the RecordIds that would be returned by an in-order scan.
     std::vector<RecordId> recordIds;
@@ -609,14 +656,12 @@ TEST_F(QueryStageCollectionScanTest, QueryTestCollscanResumeAfterRecordIdSeekFai
     // guarantee it does not exist.
     auto recordId = recordIds[offset - 1];
     remove(coll->docFor(&_opCtx, recordId).value());
-    params.resumeScanPoint = ResumeScanPoint{
-        recordId, false /* tolerateKeyNotFound */
-    };
+    params.resumeAfterRecordId = recordId;
 
     // Create plan stage.
     std::unique_ptr<WorkingSet> ws = std::make_unique<WorkingSet>();
-    std::unique_ptr<PlanStage> ps = std::make_unique<CollectionScan>(
-        _expCtx.get(), coll, params, ws.get(), nullptr /* filter */);
+    std::unique_ptr<PlanStage> ps =
+        std::make_unique<CollectionScan>(_expCtx.get(), &coll, params, ws.get(), nullptr);
 
     WorkingSetID id = WorkingSet::INVALID_ID;
 
@@ -624,80 +669,8 @@ TEST_F(QueryStageCollectionScanTest, QueryTestCollscanResumeAfterRecordIdSeekFai
     ASSERT_THROWS_CODE(ps->work(&id), DBException, ErrorCodes::KeyNotFound);
 }
 
-// Verify that using resume tokens in collscan fails with a backward natural scan.
-TEST_F(QueryStageCollectionScanTest, QueryTestCollscanResumeFailsBackwardScan) {
-    dbtests::WriteContextForTests ctx(&_opCtx, kNss.ns());
-    const CollectionPtr& coll = ctx.getCollection();
-
-    std::vector<RecordId> recordIds;
-    getRecordIds(coll, CollectionScanParams::FORWARD, &recordIds);
-
-    CollectionScanParams params;
-    params.direction = CollectionScanParams::BACKWARD;
-    params.resumeScanPoint =
-        ResumeScanPoint{recordIds[numObj() / 2], true /* tolerateKeyNotFound */};
-
-    std::unique_ptr<WorkingSet> ws = std::make_unique<WorkingSet>();
-    auto constructCollectionScan = [&]() {
-        return std::make_unique<CollectionScan>(_expCtx.get(), coll, params, ws.get(), nullptr);
-    };
-
-    ASSERT_THROWS_WITH_CHECK(constructCollectionScan(), DBException, [](const DBException& ex) {
-        ASSERT_EQUALS(ex.code(), 6521003);
-        assertionCount.tripwire.subtractAndFetch(1);
-    });
-}
-
-// Verify resuming with tolerateKeyNotFound set to true can handle deleted records by skipping to
-// next valid one.
-TEST_F(QueryStageCollectionScanTest, QueryTestCollscanStartAtDeletedRecord) {
-    dbtests::WriteContextForTests ctx(&_opCtx, kNss.ns());
-    const CollectionPtr& coll = ctx.getCollection();
-
-    std::vector<RecordId> recordIds;
-    getRecordIds(coll, CollectionScanParams::FORWARD, &recordIds);
-
-    auto offset = numObj() / 2;
-    auto recordId = recordIds[offset - 1];
-    remove(coll->docFor(&_opCtx, recordId).value());
-
-    CollectionScanParams params;
-    params.direction = CollectionScanParams::FORWARD;
-    params.resumeScanPoint = ResumeScanPoint{recordId, true /* tolerateKeyNotFound */};
-
-    std::unique_ptr<WorkingSet> ws = std::make_unique<WorkingSet>();
-    std::unique_ptr<PlanStage> ps =
-        std::make_unique<CollectionScan>(_expCtx.get(), coll, params, ws.get(), nullptr);
-
-    auto statusWithPlanExecutor =
-        plan_executor_factory::make(_expCtx,
-                                    std::move(ws),
-                                    std::move(ps),
-                                    &coll,
-                                    PlanYieldPolicy::YieldPolicy::INTERRUPT_ONLY,
-                                    QueryPlannerParams::DEFAULT);
-
-    ASSERT_OK(statusWithPlanExecutor.getStatus());
-    auto exec = std::move(statusWithPlanExecutor.getValue());
-
-    int count = 0;
-    PlanExecutor::ExecState state;
-    RecordId rid;
-    for (BSONObj obj; PlanExecutor::ADVANCED == (state = exec->getNext(&obj, &rid));) {
-        // Assert that we get the RecordId following the previous one (starting with the one after
-        // 'startAt').
-        ASSERT_EQUALS(recordIds[offset + count], rid);
-
-        // Assert that the document content is as expected.
-        ASSERT_EQUALS(count + offset, obj["foo"].numberInt());
-        ++count;
-    }
-    ASSERT_EQUALS(PlanExecutor::IS_EOF, state);
-    ASSERT_EQUALS(numObj() - offset, count);
-}
-
 TEST_F(QueryStageCollectionScanTest, QueryTestCollscanClusteredMinMax) {
-    auto ns = NamespaceString("a.b");
+    auto ns = NamespaceString::createNamespaceString_forTest("a.b");
     auto collDeleter = createClusteredCollection(ns);
     AutoGetCollectionForRead autoColl(&_opCtx, ns);
     const CollectionPtr& coll = autoColl.getCollection();
@@ -717,7 +690,7 @@ TEST_F(QueryStageCollectionScanTest, QueryTestCollscanClusteredMinMax) {
     params.maxRecord = RecordIdBound(recordIds[recordIds.size() - 1]);
 
     WorkingSet ws;
-    auto scan = std::make_unique<CollectionScan>(_expCtx.get(), coll, params, &ws, nullptr);
+    auto scan = std::make_unique<CollectionScan>(_expCtx.get(), &coll, params, &ws, nullptr);
 
     // Expect to see all RecordIds.
     int count = 0;
@@ -750,7 +723,7 @@ TEST_F(QueryStageCollectionScanTest, QueryTestCollscanClusteredMinMaxBoundsDateT
               "scanDirection"_attr =
                   (direction == CollectionScanParams::FORWARD ? "FORWARD" : "BACKWARD"));
 
-        auto ns = NamespaceString("a.b");
+        auto ns = NamespaceString::createNamespaceString_forTest("a.b");
 
         // Create a clustered collection pre-populated with RecordIds generated from type
         // 'objectId'.
@@ -777,7 +750,7 @@ TEST_F(QueryStageCollectionScanTest, QueryTestCollscanClusteredMinMaxDateTypeMat
               "scanDirection"_attr =
                   (direction == CollectionScanParams::FORWARD ? "FORWARD" : "BACKWARD"));
 
-        auto ns = NamespaceString("a.b");
+        auto ns = NamespaceString::createNamespaceString_forTest("a.b");
 
         // Create a clustered collection pre-populated with RecordIds generated from type
         // 'objectId'.
@@ -810,7 +783,7 @@ TEST_F(QueryStageCollectionScanTest, QueryTestCollscanClusteredIgnoreNumericReco
               "Running clustered collection scan test case",
               "scanDirection"_attr =
                   (direction == CollectionScanParams::FORWARD ? "FORWARD" : "BACKWARD"));
-        auto ns = NamespaceString("a.b");
+        auto ns = NamespaceString::createNamespaceString_forTest("a.b");
         auto scopedCollectionDeleter = createClusteredCollection(ns, false /* prePopulate */);
 
         int numOIDDocs = 20;
@@ -846,14 +819,10 @@ TEST_F(QueryStageCollectionScanTest, QueryTestCollscanClusteredMinMaxDateExclusi
               "scanDirection"_attr =
                   (direction == CollectionScanParams::FORWARD ? "FORWARD" : "BACKWARD"));
 
-        auto ns = NamespaceString("a.b");
+        auto ns = NamespaceString::createNamespaceString_forTest("a.b");
 
         auto scopedCollectionDeleter = createClusteredCollection(ns, false /* prePopulate */);
 
-        Lock::GlobalLock lk{&_opCtx, MODE_IX};  // avoid global lock upgrade during insertion
-        AutoGetCollectionForRead autoColl(&_opCtx, ns);
-        const CollectionPtr& coll = autoColl.getCollection();
-
         Date_t maxDate = Date_t::now();
         Date_t middleDate = maxDate - Milliseconds(1);
         Date_t minDate = middleDate - Milliseconds(1);
@@ -863,6 +832,9 @@ TEST_F(QueryStageCollectionScanTest, QueryTestCollscanClusteredMinMaxDateExclusi
             insertDocument(ns, doc);
         }
 
+        AutoGetCollectionForRead autoColl(&_opCtx, ns);
+        const CollectionPtr& coll = autoColl.getCollection();
+
         CollectionScanParams params;
         params.tailable = false;
         params.direction = direction;
@@ -871,15 +843,15 @@ TEST_F(QueryStageCollectionScanTest, QueryTestCollscanClusteredMinMaxDateExclusi
         params.maxRecord = RecordIdBound(record_id_helpers::keyForDate(maxDate));
 
         // Exclude all but the record with _id 'middleDate' from the scan.
-        StatusWithMatchExpression swMatch = MatchExpressionParser::parse(
-            BSON("_id" << BSON("$gt" << minDate << "$lt" << maxDate)), _expCtx.get());
+        BSONObj bsonObj = BSON("_id" << BSON("$gt" << minDate << "$lt" << maxDate));
+        StatusWithMatchExpression swMatch = MatchExpressionParser::parse(bsonObj, _expCtx.get());
 
         ASSERT_OK(swMatch.getStatus());
         auto filter = std::move(swMatch.getValue());
 
         WorkingSet ws;
         auto scan =
-            std::make_unique<CollectionScan>(_expCtx.get(), coll, params, &ws, filter.get());
+            std::make_unique<CollectionScan>(_expCtx.get(), &coll, params, &ws, filter.get());
 
         int count = 0;
         while (!scan->isEOF()) {
@@ -902,7 +874,7 @@ TEST_F(QueryStageCollectionScanTest, QueryTestCollscanClusteredMinMaxDateExclusi
 }
 
 TEST_F(QueryStageCollectionScanTest, QueryTestCollscanClusteredReverse) {
-    auto ns = NamespaceString("a.b");
+    auto ns = NamespaceString::createNamespaceString_forTest("a.b");
     auto collDeleter = createClusteredCollection(ns);
     AutoGetCollectionForRead autoColl(&_opCtx, ns);
     const CollectionPtr& coll = autoColl.getCollection();
@@ -924,7 +896,7 @@ TEST_F(QueryStageCollectionScanTest, QueryTestCollscanClusteredReverse) {
     params.maxRecord = RecordIdBound(recordIds[0]);
 
     WorkingSet ws;
-    auto scan = std::make_unique<CollectionScan>(_expCtx.get(), coll, params, &ws, nullptr);
+    auto scan = std::make_unique<CollectionScan>(_expCtx.get(), &coll, params, &ws, nullptr);
 
     // Expect to see all RecordIds.
     int count = 0;
@@ -946,7 +918,7 @@ TEST_F(QueryStageCollectionScanTest, QueryTestCollscanClusteredReverse) {
 }
 
 TEST_F(QueryStageCollectionScanTest, QueryTestCollscanClusteredMinMaxFullObjectIdRange) {
-    auto ns = NamespaceString("a.b");
+    auto ns = NamespaceString::createNamespaceString_forTest("a.b");
     auto collDeleter = createClusteredCollection(ns);
     AutoGetCollectionForRead autoColl(&_opCtx, ns);
     const CollectionPtr& coll = autoColl.getCollection();
@@ -968,7 +940,7 @@ TEST_F(QueryStageCollectionScanTest, QueryTestCollscanClusteredMinMaxFullObjectI
     params.maxRecord = RecordIdBound(record_id_helpers::keyForOID(OID::max()));
 
     WorkingSet ws;
-    auto scan = std::make_unique<CollectionScan>(_expCtx.get(), coll, params, &ws, nullptr);
+    auto scan = std::make_unique<CollectionScan>(_expCtx.get(), &coll, params, &ws, nullptr);
 
     // Expect to see all RecordIds.
     int count = 0;
@@ -990,7 +962,7 @@ TEST_F(QueryStageCollectionScanTest, QueryTestCollscanClusteredMinMaxFullObjectI
 }
 
 TEST_F(QueryStageCollectionScanTest, QueryTestCollscanClusteredInnerRange) {
-    auto ns = NamespaceString("a.b");
+    auto ns = NamespaceString::createNamespaceString_forTest("a.b");
     auto collDeleter = createClusteredCollection(ns);
     AutoGetCollectionForRead autoColl(&_opCtx, ns);
     const CollectionPtr& coll = autoColl.getCollection();
@@ -1016,7 +988,7 @@ TEST_F(QueryStageCollectionScanTest, QueryTestCollscanClusteredInnerRange) {
     params.maxRecord = RecordIdBound(recordIds[endOffset]);
 
     WorkingSet ws;
-    auto scan = std::make_unique<CollectionScan>(_expCtx.get(), coll, params, &ws, nullptr);
+    auto scan = std::make_unique<CollectionScan>(_expCtx.get(), &coll, params, &ws, nullptr);
 
     int count = 0;
     while (!scan->isEOF()) {
@@ -1039,7 +1011,7 @@ TEST_F(QueryStageCollectionScanTest, QueryTestCollscanClusteredInnerRange) {
 }
 
 TEST_F(QueryStageCollectionScanTest, QueryTestCollscanClusteredInnerRangeExclusiveFilter) {
-    auto ns = NamespaceString("a.b");
+    auto ns = NamespaceString::createNamespaceString_forTest("a.b");
     auto collDeleter = createClusteredCollection(ns);
     AutoGetCollectionForRead autoColl(&_opCtx, ns);
     const CollectionPtr& coll = autoColl.getCollection();
@@ -1065,7 +1037,7 @@ TEST_F(QueryStageCollectionScanTest, QueryTestCollscanClusteredInnerRangeExclusi
     params.maxRecord = RecordIdBound(recordIds[endOffset]);
 
     // Provide RecordId bounds with exclusive filters.
-    StatusWithMatchExpression swMatch = MatchExpressionParser::parse(
+    auto bsonObj =
         fromjson(fmt::sprintf("{_id: {$gt: ObjectId('%s'), $lt: ObjectId('%s')}}",
                               record_id_helpers::toBSONAs(params.minRecord->recordId(), "")
                                   .firstElement()
@@ -1074,13 +1046,13 @@ TEST_F(QueryStageCollectionScanTest, QueryTestCollscanClusteredInnerRangeExclusi
                               record_id_helpers::toBSONAs(params.maxRecord->recordId(), "")
                                   .firstElement()
                                   .OID()
-                                  .toString())),
-        _expCtx.get());
+                                  .toString()));
+    StatusWithMatchExpression swMatch = MatchExpressionParser::parse(bsonObj, _expCtx.get());
     ASSERT_OK(swMatch.getStatus());
     auto filter = std::move(swMatch.getValue());
 
     WorkingSet ws;
-    auto scan = std::make_unique<CollectionScan>(_expCtx.get(), coll, params, &ws, filter.get());
+    auto scan = std::make_unique<CollectionScan>(_expCtx.get(), &coll, params, &ws, filter.get());
 
     // The expected range should not include the first or last records.
     std::vector<RecordId> expectedIds{recordIds.begin() + startOffset + 1,
@@ -1104,7 +1076,7 @@ TEST_F(QueryStageCollectionScanTest, QueryTestCollscanClusteredInnerRangeExclusi
 }
 
 TEST_F(QueryStageCollectionScanTest, QueryTestCollscanClusteredInnerRangeExclusiveFilterReverse) {
-    auto ns = NamespaceString("a.b");
+    auto ns = NamespaceString::createNamespaceString_forTest("a.b");
     auto collDeleter = createClusteredCollection(ns);
     AutoGetCollectionForRead autoColl(&_opCtx, ns);
     const CollectionPtr& coll = autoColl.getCollection();
@@ -1132,7 +1104,7 @@ TEST_F(QueryStageCollectionScanTest, QueryTestCollscanClusteredInnerRangeExclusi
     params.maxRecord = RecordIdBound(recordIds[startOffset]);
 
     // Provide RecordId bounds with exclusive filters.
-    StatusWithMatchExpression swMatch = MatchExpressionParser::parse(
+    auto bsonObj =
         fromjson(fmt::sprintf("{_id: {$gt: ObjectId('%s'), $lt: ObjectId('%s')}}",
                               record_id_helpers::toBSONAs(params.minRecord->recordId(), "")
                                   .firstElement()
@@ -1141,13 +1113,13 @@ TEST_F(QueryStageCollectionScanTest, QueryTestCollscanClusteredInnerRangeExclusi
                               record_id_helpers::toBSONAs(params.maxRecord->recordId(), "")
                                   .firstElement()
                                   .OID()
-                                  .toString())),
-        _expCtx.get());
+                                  .toString()));
+    StatusWithMatchExpression swMatch = MatchExpressionParser::parse(bsonObj, _expCtx.get());
     ASSERT_OK(swMatch.getStatus());
     auto filter = std::move(swMatch.getValue());
 
     WorkingSet ws;
-    auto scan = std::make_unique<CollectionScan>(_expCtx.get(), coll, params, &ws, filter.get());
+    auto scan = std::make_unique<CollectionScan>(_expCtx.get(), &coll, params, &ws, filter.get());
 
     // The expected range should not include the first or last records.
     std::vector<RecordId> expectedIds{recordIds.begin() + startOffset + 1,
@@ -1182,7 +1154,7 @@ TEST_F(QueryStageCollectionScanTest,
               "Running clustered collection scan test case",
               "scanDirection"_attr =
                   (direction == CollectionScanParams::FORWARD ? "FORWARD" : "BACKWARD"));
-        auto ns = NamespaceString("a.b");
+        auto ns = NamespaceString::createNamespaceString_forTest("a.b");
         auto scopedCollectionDeleter = createClusteredCollection(ns, false /* prePopulate */);
 
         std::vector<BSONObj> docs{BSON("_id" << 0),
@@ -1225,7 +1197,7 @@ TEST_F(QueryStageCollectionScanTest, QueryTestCollscanClusteredInclusionBoundInc
               "Running clustered collection scan test case",
               "scanDirection"_attr =
                   (direction == CollectionScanParams::FORWARD ? "FORWARD" : "BACKWARD"));
-        auto ns = NamespaceString("a.b");
+        auto ns = NamespaceString::createNamespaceString_forTest("a.b");
         auto scopedCollectionDeleter = createClusteredCollection(ns, false /* prePopulate */);
 
         std::vector<BSONObj> docs{BSON("_id" << 0),
@@ -1269,7 +1241,7 @@ TEST_F(QueryStageCollectionScanTest,
               "Running clustered collection scan test case",
               "scanDirection"_attr =
                   (direction == CollectionScanParams::FORWARD ? "FORWARD" : "BACKWARD"));
-        auto ns = NamespaceString("a.b");
+        auto ns = NamespaceString::createNamespaceString_forTest("a.b");
         auto scopedCollectionDeleter = createClusteredCollection(ns, false /* prePopulate */);
 
         std::vector<BSONObj> docs{BSON("_id" << 0),
@@ -1312,7 +1284,7 @@ TEST_F(QueryStageCollectionScanTest, QueryTestCollscanClusteredInclusionBoundYie
               "Running clustered collection scan test case",
               "scanDirection"_attr =
                   (direction == CollectionScanParams::FORWARD ? "FORWARD" : "BACKWARD"));
-        auto ns = NamespaceString("a.b");
+        auto ns = NamespaceString::createNamespaceString_forTest("a.b");
         auto scopedCollectionDeleter = createClusteredCollection(ns, false /* prePopulate */);
 
         std::vector<BSONObj> docs{BSON("_id" << 0),
@@ -1347,7 +1319,7 @@ TEST_F(QueryStageCollectionScanTest, QueryTestCollscanClusteredInclusionBoundsOv
               "scanDirection"_attr =
                   (direction == CollectionScanParams::FORWARD ? "FORWARD" : "BACKWARD"));
 
-        auto ns = NamespaceString("a.b");
+        auto ns = NamespaceString::createNamespaceString_forTest("a.b");
 
         auto scopedCollectionDeleter = createClusteredCollection(ns, false /* prePopulate */);
         std::vector<BSONObj> docs{BSON("_id" << 0),
@@ -1369,11 +1341,10 @@ TEST_F(QueryStageCollectionScanTest, QueryTestCollscanClusteredInclusionBoundsOv
         expectedResults.erase(expectedResults.begin());
         expectedResults.pop_back();
 
-
         // Filtering includes the min and max records. However the ScanBoundInclusion should enforce
         // the bounds are not included in the results.
-        StatusWithMatchExpression swMatch = MatchExpressionParser::parse(
-            BSON("_id" << BSON("$gte" << 0 << "$lte" << 4)), _expCtx.get());
+        auto bsonObj = BSON("_id" << BSON("$gte" << 0 << "$lte" << 4));
+        StatusWithMatchExpression swMatch = MatchExpressionParser::parse(bsonObj, _expCtx.get());
 
         ASSERT_OK(swMatch.getStatus());
         auto filter = std::move(swMatch.getValue());
@@ -1401,7 +1372,7 @@ TEST_F(QueryStageCollectionScanTest, QueryTestCollscanClusteredInclusionBoundsHa
               "scanDirection"_attr =
                   (direction == CollectionScanParams::FORWARD ? "FORWARD" : "BACKWARD"));
 
-        auto ns = NamespaceString("a.b");
+        auto ns = NamespaceString::createNamespaceString_forTest("a.b");
 
         auto scopedCollectionDeleter = createClusteredCollection(ns, false /* prePopulate */);
         std::vector<BSONObj> docs{BSON("_id" << 0),
@@ -1432,6 +1403,9 @@ TEST_F(QueryStageCollectionScanTest, QueryTestCollscanClusteredInclusionBoundsHa
 // Tests behavior of getLatestOplogTimestamp() method when the scanned collection is the change
 // collection.
 TEST_F(QueryStageCollectionScanTest, QueryTestCollscanChangeCollectionGetLatestOplogTimestamp) {
+    // This test is disabled for the reasons outlined in SERVER-76288.
+    // TODO SERVER-76309 reenable this test.
+    return;
     // Setup the change collection.
     auto collectionName = NamespaceString::makeChangeCollectionNSS(boost::none);
     auto scopedCollectionDeleter =
@@ -1439,8 +1413,8 @@ TEST_F(QueryStageCollectionScanTest, QueryTestCollscanChangeCollectionGetLatestO
     insertDocument(collectionName, BSON("_id" << Timestamp(15, 5) << "ts" << Timestamp(15, 5)));
 
     // Set the read timestamp.
-    _opCtx.recoveryUnit()->setTimestampReadSource(RecoveryUnit::ReadSource::kProvided,
-                                                  Timestamp(16, 1));
+    shard_role_details::getRecoveryUnit(&_opCtx)->setTimestampReadSource(
+        RecoveryUnit::ReadSource::kProvided, Timestamp(16, 1));
 
     // Build the collection scan stage.
     AutoGetCollectionForRead autoColl(&_opCtx, collectionName);
@@ -1450,7 +1424,7 @@ TEST_F(QueryStageCollectionScanTest, QueryTestCollscanChangeCollectionGetLatestO
     params.direction = CollectionScanParams::FORWARD;
     params.shouldTrackLatestOplogTimestamp = true;
     WorkingSet ws;
-    auto scanStage = std::make_unique<CollectionScan>(_expCtx.get(), coll, params, &ws, nullptr);
+    auto scanStage = std::make_unique<CollectionScan>(_expCtx.get(), &coll, params, &ws, nullptr);
     WorkingSetID id = WorkingSet::INVALID_ID;
     auto advance = [&]() {
         PlanStage::StageState state;
@@ -1469,5 +1443,49 @@ TEST_F(QueryStageCollectionScanTest, QueryTestCollscanChangeCollectionGetLatestO
     ASSERT_EQUALS(PlanStage::IS_EOF, state) << " state: " << PlanStage::stateStr(state);
     ASSERT_EQUALS(Timestamp(16, 1), scanStage->getLatestOplogTimestamp());
 }
+
+// Tests behavior of getLatestOplogTimestamp() method when the scanned collection is the change
+// collection.
+TEST_F(QueryStageCollectionScanTest, QueryTestCollscanChangeCollectionLatestTimestampIsNotGlobal) {
+    // This test exercises the temporary behaviour introduced in SERVER-76288.
+    // TODO SERVER-76309 remove this test.
+    // Setup the change collection.
+    auto collectionName = NamespaceString::makeChangeCollectionNSS(boost::none);
+    auto scopedCollectionDeleter =
+        createClusteredCollection(collectionName, false /* prePopulate */);
+    insertDocument(collectionName, BSON("_id" << Timestamp(15, 5) << "ts" << Timestamp(15, 5)));
+
+    // Set the read timestamp.
+    shard_role_details::getRecoveryUnit(&_opCtx)->setTimestampReadSource(
+        RecoveryUnit::ReadSource::kProvided, Timestamp(16, 1));
+
+    // Build the collection scan stage.
+    AutoGetCollectionForRead autoColl(&_opCtx, collectionName);
+    const CollectionPtr& coll = autoColl.getCollection();
+    CollectionScanParams params;
+    params.tailable = true;
+    params.direction = CollectionScanParams::FORWARD;
+    params.shouldTrackLatestOplogTimestamp = true;
+    WorkingSet ws;
+    auto scanStage = std::make_unique<CollectionScan>(_expCtx.get(), &coll, params, &ws, nullptr);
+    WorkingSetID id = WorkingSet::INVALID_ID;
+    auto advance = [&]() {
+        PlanStage::StageState state;
+        while ((state = scanStage->work(&id)) == PlanStage::NEED_TIME) {
+        };
+        return state;
+    };
+
+    // Verify that the latest oplog timestamp is equal to the 'ts' field of the retrieved document.
+    auto state = advance();
+    ASSERT_EQUALS(PlanStage::ADVANCED, state) << " state: " << PlanStage::stateStr(state);
+    ASSERT_EQUALS(Timestamp(15, 5), scanStage->getLatestOplogTimestamp());
+
+    // Verify that on EOF, the latest oplog timestamp is still equal to the ts of the last document
+    // retrieved, and has NOT advanced to the read timestamp.
+    state = advance();
+    ASSERT_EQUALS(PlanStage::IS_EOF, state) << " state: " << PlanStage::stateStr(state);
+    ASSERT_EQUALS(Timestamp(15, 5), scanStage->getLatestOplogTimestamp());
+}
 }  // namespace query_stage_collection_scan
 }  // namespace mongo
