@@ -27,36 +27,88 @@
  *    it in the license file.
  */
 
+#include <boost/container/small_vector.hpp>
+#include <fmt/format.h>
+// IWYU pragma: no_include "boost/intrusive/detail/iterator.hpp"
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+// IWYU pragma: no_include "ext/alloc_traits.h"
+#include <cstddef>
+#include <memory>
+#include <utility>
+#include <vector>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/db/catalog/collection.h"
-#include "mongo/db/catalog/database.h"
 #include "mongo/db/catalog/index_catalog.h"
 #include "mongo/db/client.h"
 #include "mongo/db/db_raii.h"
 #include "mongo/db/dbdirectclient.h"
 #include "mongo/db/exec/collection_scan.h"
+#include "mongo/db/exec/collection_scan_common.h"
+#include "mongo/db/exec/document_value/document.h"
 #include "mongo/db/exec/fetch.h"
 #include "mongo/db/exec/index_scan.h"
 #include "mongo/db/exec/mock_stage.h"
 #include "mongo/db/exec/multi_plan.h"
+#include "mongo/db/exec/plan_cache_util.h"
 #include "mongo/db/exec/plan_stage.h"
-#include "mongo/db/json.h"
+#include "mongo/db/exec/plan_stats.h"
+#include "mongo/db/exec/working_set.h"
+#include "mongo/db/index/index_descriptor.h"
+#include "mongo/db/matcher/expression.h"
 #include "mongo/db/matcher/expression_parser.h"
 #include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/pipeline/expression_context.h"
+#include "mongo/db/query/canonical_query.h"
+#include "mongo/db/query/classic_plan_cache.h"
 #include "mongo/db/query/collection_query_info.h"
+#include "mongo/db/query/explain.h"
+#include "mongo/db/query/explain_options.h"
+#include "mongo/db/query/find_command.h"
 #include "mongo/db/query/get_executor.h"
+#include "mongo/db/query/index_bounds.h"
 #include "mongo/db/query/mock_yield_policies.h"
+#include "mongo/db/query/plan_cache.h"
 #include "mongo/db/query/plan_cache_key_factory.h"
+#include "mongo/db/query/plan_executor.h"
 #include "mongo/db/query/plan_executor_factory.h"
 #include "mongo/db/query/plan_executor_impl.h"
+#include "mongo/db/query/plan_explainer.h"
 #include "mongo/db/query/plan_summary_stats.h"
-#include "mongo/db/query/query_feature_flags_gen.h"
+#include "mongo/db/query/plan_yield_policy.h"
+#include "mongo/db/query/plan_yield_policy_impl.h"
 #include "mongo/db/query/query_knobs_gen.h"
 #include "mongo/db/query/query_planner.h"
+#include "mongo/db/query/query_planner_params.h"
 #include "mongo/db/query/query_planner_test_lib.h"
+#include "mongo/db/query/query_settings/query_settings_gen.h"
+#include "mongo/db/query/query_solution.h"
 #include "mongo/db/query/stage_builder_util.h"
-#include "mongo/dbtests/dbtests.h"
+#include "mongo/db/query/stage_types.h"
+#include "mongo/db/service_context.h"
+#include "mongo/db/storage/snapshot.h"
+#include "mongo/dbtests/dbtests.h"  // IWYU pragma: keep
 #include "mongo/idl/server_parameter_test_util.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/bson_test_util.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/clock_source.h"
 #include "mongo/util/clock_source_mock.h"
+#include "mongo/util/intrusive_counter.h"
+#include "mongo/util/scopeguard.h"
+#include "mongo/util/serialization_context.h"
 
 namespace mongo {
 
@@ -72,10 +124,12 @@ namespace {
 using std::unique_ptr;
 using std::vector;
 
-static const NamespaceString nss("unittests.QueryStageMultiPlan");
+static const NamespaceString nss =
+    NamespaceString::createNamespaceString_forTest("unittests.QueryStageMultiPlan");
 
 std::unique_ptr<QuerySolution> createQuerySolution() {
     auto soln = std::make_unique<QuerySolution>();
+    soln->setRoot(std::make_unique<CollectionScanNode>());
     soln->cacheData = std::make_unique<SolutionCacheData>();
     soln->cacheData->solnType = SolutionCacheData::COLLSCAN_SOLN;
     soln->cacheData->tree = std::make_unique<PlanCacheIndexTree>();
@@ -85,26 +139,26 @@ std::unique_ptr<QuerySolution> createQuerySolution() {
 class QueryStageMultiPlanTest : public unittest::Test {
 public:
     QueryStageMultiPlanTest() : _client(_opCtx.get()) {
-        dbtests::WriteContextForTests ctx(_opCtx.get(), nss.ns());
+        dbtests::WriteContextForTests ctx(_opCtx.get(), nss.ns_forTest());
         _client.dropCollection(nss);
     }
 
-    virtual ~QueryStageMultiPlanTest() {
-        dbtests::WriteContextForTests ctx(_opCtx.get(), nss.ns());
+    ~QueryStageMultiPlanTest() override {
+        dbtests::WriteContextForTests ctx(_opCtx.get(), nss.ns_forTest());
         _client.dropCollection(nss);
     }
 
     void addIndex(const BSONObj& obj) {
-        ASSERT_OK(dbtests::createIndex(_opCtx.get(), nss.ns(), obj));
+        ASSERT_OK(dbtests::createIndex(_opCtx.get(), nss.ns_forTest(), obj));
     }
 
     void insert(const BSONObj& obj) {
-        dbtests::WriteContextForTests ctx(_opCtx.get(), nss.ns());
+        dbtests::WriteContextForTests ctx(_opCtx.get(), nss.ns_forTest());
         _client.insert(nss, obj);
     }
 
     void remove(const BSONObj& obj) {
-        dbtests::WriteContextForTests ctx(_opCtx.get(), nss.ns());
+        dbtests::WriteContextForTests ctx(_opCtx.get(), nss.ns_forTest());
         _client.remove(nss, obj);
     }
 
@@ -116,6 +170,19 @@ public:
         return _opCtx->getServiceContext();
     }
 
+    QueryPlannerParams makePlannerParams(const CollectionPtr& collection,
+                                         const CanonicalQuery& canonicalQuery) {
+        MultipleCollectionAccessor collections(collection);
+        return QueryPlannerParams{
+            QueryPlannerParams::ArgsForSingleCollectionQuery{
+                .opCtx = opCtx(),
+                .canonicalQuery = canonicalQuery,
+                .collections = collections,
+                .plannerOptions = QueryPlannerParams::DEFAULT,
+            },
+        };
+    }
+
 protected:
     const ServiceContext::UniqueOperationContext _opCtx = cc().makeOperationContext();
     ClockSource* const _clock = _opCtx->getServiceContext()->getFastClockSource();
@@ -131,11 +198,9 @@ std::unique_ptr<CanonicalQuery> makeCanonicalQuery(OperationContext* opCtx,
                                                    BSONObj filter) {
     auto findCommand = std::make_unique<FindCommandRequest>(nss);
     findCommand->setFilter(filter);
-    auto statusWithCQ = CanonicalQuery::canonicalize(opCtx, std::move(findCommand));
-    ASSERT_OK(statusWithCQ.getStatus());
-    unique_ptr<CanonicalQuery> cq = std::move(statusWithCQ.getValue());
-    ASSERT(cq);
-    return cq;
+    return std::make_unique<CanonicalQuery>(
+        CanonicalQueryParams{.expCtx = makeExpressionContext(opCtx, *findCommand),
+                             .parsedFind = ParsedFindCommandParams{std::move(findCommand)}});
 }
 
 unique_ptr<PlanStage> getIxScanPlan(ExpressionContext* expCtx,
@@ -154,8 +219,8 @@ unique_ptr<PlanStage> getIxScanPlan(ExpressionContext* expCtx,
     ixparams.bounds.boundInclusion = BoundInclusion::kIncludeBothStartAndEndKeys;
     ixparams.direction = 1;
 
-    auto ixscan = std::make_unique<IndexScan>(expCtx, coll, ixparams, sharedWs, nullptr);
-    return std::make_unique<FetchStage>(expCtx, sharedWs, std::move(ixscan), nullptr, coll);
+    auto ixscan = std::make_unique<IndexScan>(expCtx, &coll, ixparams, sharedWs, nullptr);
+    return std::make_unique<FetchStage>(expCtx, sharedWs, std::move(ixscan), nullptr, &coll);
 }
 
 unique_ptr<MatchExpression> makeMatchExpressionFromFilter(ExpressionContext* expCtx,
@@ -175,7 +240,7 @@ unique_ptr<PlanStage> getCollScanPlan(ExpressionContext* expCtx,
     CollectionScanParams csparams;
     csparams.direction = CollectionScanParams::FORWARD;
 
-    unique_ptr<PlanStage> root(new CollectionScan(expCtx, coll, csparams, sharedWs, matchExpr));
+    unique_ptr<PlanStage> root(new CollectionScan(expCtx, &coll, csparams, sharedWs, matchExpr));
 
     return root;
 }
@@ -205,7 +270,11 @@ std::unique_ptr<MultiPlanStage> runMultiPlanner(ExpressionContext* expCtx,
     // Hand the plans off to the MPS.
     auto cq = makeCanonicalQuery(expCtx->opCtx, nss, BSON("foo" << desiredFooValue));
 
-    unique_ptr<MultiPlanStage> mps = std::make_unique<MultiPlanStage>(expCtx, coll, cq.get());
+    unique_ptr<MultiPlanStage> mps = std::make_unique<MultiPlanStage>(
+        expCtx,
+        &coll,
+        cq.get(),
+        plan_cache_util::ClassicPlanCacheWriter{expCtx->opCtx, &coll, false /* executeInSbe */});
     mps->addPlan(createQuerySolution(), std::move(ixScanRoot), sharedWs.get());
     mps->addPlan(createQuerySolution(), std::move(collScanRoot), sharedWs.get());
 
@@ -258,27 +327,36 @@ TEST_F(QueryStageMultiPlanTest, MPSCollectionScanVsHighlySelectiveIXScan) {
     // Hand the plans off to the MPS.
     auto cq = makeCanonicalQuery(_opCtx.get(), nss, filterObj);
 
-    unique_ptr<MultiPlanStage> mps =
-        std::make_unique<MultiPlanStage>(_expCtx.get(), ctx.getCollection(), cq.get());
+    unique_ptr<MultiPlanStage> mps = std::make_unique<MultiPlanStage>(
+        _expCtx.get(),
+        &ctx.getCollection(),
+        cq.get(),
+        plan_cache_util::ClassicPlanCacheWriter{
+            opCtx(), &ctx.getCollection(), false /* executeInSbe */});
     mps->addPlan(createQuerySolution(), std::move(ixScanRoot), sharedWs.get());
     mps->addPlan(createQuerySolution(), std::move(collScanRoot), sharedWs.get());
 
-    const auto* mpsPtr = mps.get();
-
-    // Takes ownership of arguments other than 'collection'.
-    auto statusWithPlanExecutor =
-        plan_executor_factory::make(std::move(cq),
-                                    std::move(sharedWs),
-                                    std::move(mps),
-                                    &coll,
-                                    PlanYieldPolicy::YieldPolicy::NO_YIELD,
-                                    QueryPlannerParams::DEFAULT);
-    ASSERT_OK(statusWithPlanExecutor.getStatus());
-    auto exec = std::move(statusWithPlanExecutor.getValue());
-
+    auto* mpsPtr = mps.get();
+    auto planYieldPolicy = makeClassicYieldPolicy(opCtx(),
+                                                  nss,
+                                                  static_cast<PlanStage*>(mpsPtr),
+                                                  PlanYieldPolicy::YieldPolicy::INTERRUPT_ONLY,
+                                                  &coll);
+    ASSERT_OK(mpsPtr->pickBestPlan(planYieldPolicy.get()));
     ASSERT_TRUE(mpsPtr->bestPlanChosen());
     ASSERT_EQUALS(mpsPtr->getChildren()[mpsPtr->bestPlanIdx().get()].get(), ixScanRootPtr);
 
+    // Takes ownership of arguments other than 'collection'.
+    auto execResult = plan_executor_factory::make(std::move(cq),
+                                                  std::move(sharedWs),
+                                                  std::move(mps),
+                                                  &coll,
+                                                  PlanYieldPolicy::YieldPolicy::INTERRUPT_ONLY,
+                                                  QueryPlannerParams::DEFAULT,
+                                                  nss);
+    ASSERT_OK(execResult);
+    auto exec = std::move(execResult.getValue());
+
     // Get all our results out.
     int results = 0;
     BSONObj obj;
@@ -316,7 +394,8 @@ TEST_F(QueryStageMultiPlanTest, MPSDoesNotCreateActiveCacheEntryImmediately) {
     auto entry = assertGet(cache->getEntry(key));
     ASSERT_FALSE(entry->isActive);
     const size_t firstQueryWorks = getBestPlanWorks(mps.get());
-    ASSERT_EQ(firstQueryWorks, entry->works);
+    ASSERT(entry->readsOrWorks);
+    ASSERT_EQ(firstQueryWorks, entry->readsOrWorks->rawValue());
 
     // Run the multi-planner again. The index scan will again win, but the number of works
     // will be greater, since {foo: 5} appears more frequently in the collection.
@@ -327,7 +406,8 @@ TEST_F(QueryStageMultiPlanTest, MPSDoesNotCreateActiveCacheEntryImmediately) {
     ASSERT_EQ(cache->size(), 1U);
     entry = assertGet(cache->getEntry(key));
     ASSERT_FALSE(entry->isActive);
-    ASSERT_EQ(firstQueryWorks * 2, entry->works);
+    ASSERT(entry->readsOrWorks);
+    ASSERT_EQ(firstQueryWorks * 2, entry->readsOrWorks->rawValue());
 
     // Run the exact same query again. This will still take more works than 'works', and
     // should cause the cache entry's 'works' to be doubled again.
@@ -335,14 +415,14 @@ TEST_F(QueryStageMultiPlanTest, MPSDoesNotCreateActiveCacheEntryImmediately) {
     ASSERT_EQ(cache->size(), 1U);
     entry = assertGet(cache->getEntry(key));
     ASSERT_FALSE(entry->isActive);
-    ASSERT_EQ(firstQueryWorks * 2 * 2, entry->works);
+    ASSERT_EQ(firstQueryWorks * 2 * 2, entry->readsOrWorks->rawValue());
 
     // Run the query yet again. This time, an active cache entry should be created.
     mps = runMultiPlanner(_expCtx.get(), nss, coll, 5);
     ASSERT_EQ(cache->size(), 1U);
     entry = assertGet(cache->getEntry(key));
     ASSERT_TRUE(entry->isActive);
-    ASSERT_EQ(getBestPlanWorks(mps.get()), entry->works);
+    ASSERT_EQ(getBestPlanWorks(mps.get()), entry->readsOrWorks->rawValue());
 }
 
 TEST_F(QueryStageMultiPlanTest, MPSDoesCreatesActiveEntryWhenInactiveEntriesDisabled) {
@@ -392,21 +472,17 @@ TEST_F(QueryStageMultiPlanTest, MPSBackupPlan) {
     auto findCommand = std::make_unique<FindCommandRequest>(nss);
     findCommand->setFilter(BSON("a" << 1 << "b" << 1));
     findCommand->setSort(BSON("b" << 1));
-    auto statusWithCQ = CanonicalQuery::canonicalize(opCtx(), std::move(findCommand));
-    MONGO_verify(statusWithCQ.isOK());
-    unique_ptr<CanonicalQuery> cq = std::move(statusWithCQ.getValue());
-    ASSERT(nullptr != cq.get());
+    auto cq = std::make_unique<CanonicalQuery>(
+        CanonicalQueryParams{.expCtx = makeExpressionContext(opCtx(), *findCommand),
+                             .parsedFind = ParsedFindCommandParams{std::move(findCommand)}});
     auto key = plan_cache_key_factory::make<PlanCacheKey>(*cq, collection.getCollection());
 
     // Force index intersection.
     bool forceIxisectOldValue = internalQueryForceIntersectionPlans.load();
     internalQueryForceIntersectionPlans.store(true);
 
-    // Get planner params.
-    QueryPlannerParams plannerParams;
-    fillOutPlannerParams(_opCtx.get(), collection.getCollection(), cq.get(), &plannerParams);
-
     // Plan.
+    auto plannerParams = makePlannerParams(collection.getCollection(), *cq);
     auto statusWithMultiPlanSolns = QueryPlanner::plan(*cq, plannerParams);
     ASSERT_OK(statusWithMultiPlanSolns.getStatus());
     auto solutions = std::move(statusWithMultiPlanSolns.getValue());
@@ -416,13 +492,18 @@ TEST_F(QueryStageMultiPlanTest, MPSBackupPlan) {
     ASSERT_EQUALS(solutions.size(), 3U);
 
     // Fill out the MultiPlanStage.
-    unique_ptr<MultiPlanStage> mps(
-        new MultiPlanStage(_expCtx.get(), collection.getCollection(), cq.get()));
+    auto mps = std::make_unique<MultiPlanStage>(
+        _expCtx.get(),
+        &collection.getCollection(),
+        cq.get(),
+        plan_cache_util::ClassicPlanCacheWriter{
+            opCtx(), &collection.getCollection(), false /* executeInSbe */
+        });
     unique_ptr<WorkingSet> ws(new WorkingSet());
     // Put each solution from the planner into the MPR.
     for (size_t i = 0; i < solutions.size(); ++i) {
         auto&& root = stage_builder::buildClassicExecutableTree(
-            _opCtx.get(), collection.getCollection(), *cq, *solutions[i], ws.get());
+            _opCtx.get(), &collection.getCollection(), *cq, *solutions[i], ws.get());
         mps->addPlan(std::move(solutions[i]), std::move(root), ws.get());
     }
 
@@ -506,38 +587,49 @@ TEST_F(QueryStageMultiPlanTest, MPSExplainAllPlans) {
 
     auto findCommand = std::make_unique<FindCommandRequest>(nss);
     findCommand->setFilter(BSON("x" << 1));
-    auto cq = uassertStatusOK(CanonicalQuery::canonicalize(opCtx(), std::move(findCommand)));
+    auto cq = std::make_unique<CanonicalQuery>(
+        CanonicalQueryParams{.expCtx = makeExpressionContext(opCtx(), *findCommand),
+                             .parsedFind = ParsedFindCommandParams{std::move(findCommand)}});
     unique_ptr<MultiPlanStage> mps =
-        std::make_unique<MultiPlanStage>(_expCtx.get(), ctx.getCollection(), cq.get());
+        std::make_unique<MultiPlanStage>(_expCtx.get(),
+                                         &ctx.getCollection(),
+                                         cq.get(),
+                                         plan_cache_util::ClassicPlanCacheWriter{
+                                             opCtx(), &ctx.getCollection(), false /* executeInSbe */
+                                         });
 
     // Put each plan into the MultiPlanStage. Takes ownership of 'firstPlan' and 'secondPlan'.
     mps->addPlan(std::make_unique<QuerySolution>(), std::move(firstPlan), ws.get());
     mps->addPlan(std::make_unique<QuerySolution>(), std::move(secondPlan), ws.get());
 
-    // Making a PlanExecutor chooses the best plan.
-    auto exec = uassertStatusOK(plan_executor_factory::make(_expCtx,
-                                                            std::move(ws),
-                                                            std::move(mps),
-                                                            &ctx.getCollection(),
-                                                            PlanYieldPolicy::YieldPolicy::NO_YIELD,
-                                                            QueryPlannerParams::DEFAULT));
-
-    auto execImpl = dynamic_cast<PlanExecutorImpl*>(exec.get());
-    ASSERT(execImpl);
-    auto root = static_cast<MultiPlanStage*>(execImpl->getRootStage());
-    ASSERT_TRUE(root->bestPlanChosen());
     // The first candidate plan should have won.
-    ASSERT_EQ(getBestPlanRoot(root), firstPlanPtr);
+    auto planYieldPolicy = makeClassicYieldPolicy(opCtx(),
+                                                  nss,
+                                                  static_cast<PlanStage*>(mps.get()),
+                                                  PlanYieldPolicy::YieldPolicy::INTERRUPT_ONLY,
+                                                  &ctx.getCollection());
+    ASSERT_OK(mps->pickBestPlan(planYieldPolicy.get()));
+    ASSERT_TRUE(mps->bestPlanChosen());
+    ASSERT_EQ(getBestPlanRoot(mps.get()), firstPlanPtr);
 
+    auto execResult = plan_executor_factory::make(_expCtx,
+                                                  std::move(ws),
+                                                  std::move(mps),
+                                                  &ctx.getCollection(),
+                                                  PlanYieldPolicy::YieldPolicy::INTERRUPT_ONLY,
+                                                  QueryPlannerParams::DEFAULT,
+                                                  nss);
+    ASSERT_OK(execResult);
+    auto exec = std::move(execResult.getValue());
     BSONObjBuilder bob;
     Explain::explainStages(exec.get(),
                            ctx.getCollection(),
                            ExplainOptions::Verbosity::kExecAllPlans,
                            BSONObj(),
+                           SerializationContext::stateCommandReply(),
                            BSONObj(),
                            &bob);
     BSONObj explained = bob.done();
-
     ASSERT_EQ(explained["executionStats"]["nReturned"].Int(), nDocs);
     ASSERT_EQ(explained["executionStats"]["executionStages"]["needTime"].Int(), 0);
     auto allPlansStats = explained["executionStats"]["allPlansExecution"].Array();
@@ -577,13 +669,13 @@ TEST_F(QueryStageMultiPlanTest, MPSSummaryStats) {
     // Create the executor (Matching all documents).
     auto findCommand = std::make_unique<FindCommandRequest>(nss);
     findCommand->setFilter(BSON("foo" << BSON("$gte" << 0)));
-    auto cq = uassertStatusOK(CanonicalQuery::canonicalize(opCtx(), std::move(findCommand)));
-    auto exec = uassertStatusOK(getExecutor(opCtx(),
-                                            &coll,
-                                            std::move(cq),
-                                            nullptr /* extractAndAttachPipelineStages */,
-                                            PlanYieldPolicy::YieldPolicy::NO_YIELD,
-                                            0));
+    auto cq = std::make_unique<CanonicalQuery>(
+        CanonicalQueryParams{.expCtx = makeExpressionContext(opCtx(), *findCommand),
+                             .parsedFind = ParsedFindCommandParams{std::move(findCommand)}});
+    auto exec = uassertStatusOK(getExecutorFind(opCtx(),
+                                                MultipleCollectionAccessor{coll},
+                                                std::move(cq),
+                                                PlanYieldPolicy::YieldPolicy::INTERRUPT_ONLY));
 
     auto execImpl = dynamic_cast<PlanExecutorImpl*>(exec.get());
     ASSERT(execImpl);
@@ -634,10 +726,13 @@ TEST_F(QueryStageMultiPlanTest, ShouldReportErrorIfExceedsTimeLimitDuringPlannin
 
     auto findCommand = std::make_unique<FindCommandRequest>(nss);
     findCommand->setFilter(filterObj);
-    auto canonicalQuery =
-        uassertStatusOK(CanonicalQuery::canonicalize(opCtx(), std::move(findCommand)));
-    MultiPlanStage multiPlanStage(
-        _expCtx.get(), coll.getCollection(), canonicalQuery.get(), PlanCachingMode::NeverCache);
+    auto canonicalQuery = std::make_unique<CanonicalQuery>(
+        CanonicalQueryParams{.expCtx = makeExpressionContext(opCtx(), *findCommand),
+                             .parsedFind = ParsedFindCommandParams{std::move(findCommand)}});
+    MultiPlanStage multiPlanStage(_expCtx.get(),
+                                  &coll.getCollection(),
+                                  canonicalQuery.get(),
+                                  plan_cache_util::NoopPlanCacheWriter{});
     multiPlanStage.addPlan(createQuerySolution(), std::move(ixScanRoot), sharedWs.get());
     multiPlanStage.addPlan(createQuerySolution(), std::move(collScanRoot), sharedWs.get());
 
@@ -675,10 +770,13 @@ TEST_F(QueryStageMultiPlanTest, ShouldReportErrorIfKilledDuringPlanning) {
 
     auto findCommand = std::make_unique<FindCommandRequest>(nss);
     findCommand->setFilter(BSON("foo" << BSON("$gte" << 0)));
-    auto canonicalQuery =
-        uassertStatusOK(CanonicalQuery::canonicalize(opCtx(), std::move(findCommand)));
-    MultiPlanStage multiPlanStage(
-        _expCtx.get(), coll.getCollection(), canonicalQuery.get(), PlanCachingMode::NeverCache);
+    auto canonicalQuery = std::make_unique<CanonicalQuery>(
+        CanonicalQueryParams{.expCtx = makeExpressionContext(opCtx(), *findCommand),
+                             .parsedFind = ParsedFindCommandParams{std::move(findCommand)}});
+    MultiPlanStage multiPlanStage(_expCtx.get(),
+                                  &coll.getCollection(),
+                                  canonicalQuery.get(),
+                                  plan_cache_util::NoopPlanCacheWriter{});
     multiPlanStage.addPlan(createQuerySolution(), std::move(ixScanRoot), sharedWs.get());
     multiPlanStage.addPlan(createQuerySolution(), std::move(collScanRoot), sharedWs.get());
 
@@ -693,7 +791,7 @@ TEST_F(QueryStageMultiPlanTest, ShouldReportErrorIfKilledDuringPlanning) {
  */
 class ThrowyPlanStage : public PlanStage {
 protected:
-    StageState doWork(WorkingSetID* out) {
+    StageState doWork(WorkingSetID* out) override {
         uasserted(ErrorCodes::InternalError, "Mwahahaha! You've fallen into my trap.");
     }
 
@@ -705,10 +803,10 @@ public:
     StageType stageType() const final {
         return STAGE_UNKNOWN;
     }
-    virtual std::unique_ptr<PlanStageStats> getStats() final {
+    std::unique_ptr<PlanStageStats> getStats() final {
         return nullptr;
     }
-    virtual const SpecificStats* getSpecificStats() const final {
+    const SpecificStats* getSpecificStats() const final {
         return nullptr;
     }
 };
@@ -720,10 +818,13 @@ TEST_F(QueryStageMultiPlanTest, AddsContextDuringException) {
     auto findCommand = std::make_unique<FindCommandRequest>(nss);
     findCommand->setFilter(BSON("fake"
                                 << "query"));
-    auto canonicalQuery =
-        uassertStatusOK(CanonicalQuery::canonicalize(opCtx(), std::move(findCommand)));
-    MultiPlanStage multiPlanStage(
-        _expCtx.get(), ctx.getCollection(), canonicalQuery.get(), PlanCachingMode::NeverCache);
+    auto canonicalQuery = std::make_unique<CanonicalQuery>(
+        CanonicalQueryParams{.expCtx = makeExpressionContext(opCtx(), *findCommand),
+                             .parsedFind = ParsedFindCommandParams{std::move(findCommand)}});
+    MultiPlanStage multiPlanStage(_expCtx.get(),
+                                  &ctx.getCollection(),
+                                  canonicalQuery.get(),
+                                  plan_cache_util::NoopPlanCacheWriter{});
     unique_ptr<WorkingSet> sharedWs(new WorkingSet());
     multiPlanStage.addPlan(
         createQuerySolution(), std::make_unique<ThrowyPlanStage>(_expCtx.get()), sharedWs.get());
