@@ -27,44 +27,72 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
+#include <cstddef>
+#include <fmt/format.h>
 #include <memory>
+#include <string>
+#include <utility>
 
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr/intrusive_ptr.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/json.h"
 #include "mongo/db/catalog/collection.h"
-#include "mongo/db/catalog/database.h"
-#include "mongo/db/catalog/database_holder.h"
 #include "mongo/db/client.h"
 #include "mongo/db/db_raii.h"
 #include "mongo/db/dbdirectclient.h"
+#include "mongo/db/exec/plan_stage.h"
 #include "mongo/db/exec/subplan.h"
-#include "mongo/db/jsobj.h"
-#include "mongo/db/json.h"
+#include "mongo/db/exec/working_set.h"
+#include "mongo/db/matcher/expression_parser.h"
 #include "mongo/db/matcher/extensions_callback_noop.h"
-#include "mongo/db/pipeline/expression_context_for_test.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/pipeline/expression_context.h"
 #include "mongo/db/query/canonical_query.h"
+#include "mongo/db/query/find_command.h"
 #include "mongo/db/query/get_executor.h"
 #include "mongo/db/query/mock_yield_policies.h"
-#include "mongo/db/query/query_test_service_context.h"
-#include "mongo/dbtests/dbtests.h"
+#include "mongo/db/query/parsed_find_command.h"
+#include "mongo/db/query/plan_executor.h"
+#include "mongo/db/query/query_knobs_gen.h"
+#include "mongo/db/query/query_planner_params.h"
+#include "mongo/db/query/query_request_helper.h"
+#include "mongo/db/service_context.h"
+#include "mongo/dbtests/dbtests.h"  // IWYU pragma: keep
+#include "mongo/platform/atomic_word.h"
+#include "mongo/rpc/op_msg.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
 #include "mongo/util/assert_util.h"
+#include "mongo/util/clock_source.h"
+#include "mongo/util/intrusive_counter.h"
 
 namespace mongo {
 namespace {
 
-static const NamespaceString nss("unittests.QueryStageSubplan");
+static const NamespaceString nss =
+    NamespaceString::createNamespaceString_forTest("unittests.QueryStageSubplan");
 
 class QueryStageSubplanTest : public unittest::Test {
 public:
     QueryStageSubplanTest() : _client(_opCtx.get()) {}
 
-    virtual ~QueryStageSubplanTest() {
-        dbtests::WriteContextForTests ctx(opCtx(), nss.ns());
+    ~QueryStageSubplanTest() override {
+        dbtests::WriteContextForTests ctx(opCtx(), nss.ns_forTest());
         _client.dropCollection(nss);
     }
 
     void addIndex(const BSONObj& obj) {
-        ASSERT_OK(dbtests::createIndex(opCtx(), nss.ns(), obj));
+        ASSERT_OK(dbtests::createIndex(opCtx(), nss.ns_forTest(), obj));
     }
 
     void addIndexWithWildcardProjection(BSONObj keys, BSONObj wildcardProjection) {
@@ -74,7 +102,7 @@ public:
             << IndexDescriptor::kUniqueFieldName << false << IndexDescriptor::kIndexVersionFieldName
             << static_cast<int>(IndexDescriptor::IndexVersion::kV2)
             << IndexDescriptor::kWildcardProjectionFieldName << wildcardProjection);
-        ASSERT_OK(dbtests::createIndexFromSpec(opCtx(), nss.ns(), indexSpec));
+        ASSERT_OK(dbtests::createIndexFromSpec(opCtx(), nss.ns_forTest(), indexSpec));
     }
 
     void dropIndex(BSONObj keyPattern) {
@@ -97,6 +125,26 @@ public:
         return _opCtx->getServiceContext();
     }
 
+    std::unique_ptr<SubplanStage> makeSubplanStage(const CollectionPtr& coll,
+                                                   CanonicalQuery* cq,
+                                                   WorkingSet& ws) {
+        auto callbacks = SubplanStage::PlanSelectionCallbacks{
+            .onPickPlanForBranch =
+                plan_cache_util::ConditionalClassicPlanCacheWriter{
+                    plan_cache_util::ConditionalClassicPlanCacheWriter::Mode::SometimesCache,
+                    opCtx(),
+                    &coll,
+                    false /* executeInSbe */
+                },
+            .onPickPlanWholeQuery =
+                plan_cache_util::ClassicPlanCacheWriter{
+                    opCtx(), &coll, false /* executeInSbe */
+                },
+        };
+
+        return std::make_unique<SubplanStage>(_expCtx.get(), &coll, &ws, cq, std::move(callbacks));
+    }
+
 protected:
     /**
      * Parses the json string 'findCmd', specifying a find command, to a CanonicalQuery.
@@ -104,22 +152,39 @@ protected:
     std::unique_ptr<CanonicalQuery> cqFromFindCommand(const std::string& findCmd) {
         BSONObj cmdObj = fromjson(findCmd);
 
-        bool isExplain = false;
         // If there is no '$db', append it.
-        auto cmd = OpMsgRequest::fromDBAndBody("test", cmdObj).body;
+        auto cmd = OpMsgRequestBuilder::create(
+                       auth::ValidatedTenancyScope::get(_opCtx.get()),
+                       DatabaseName::createDatabaseName_forTest(boost::none, "test"),
+                       cmdObj)
+                       .body;
         auto findCommand =
             query_request_helper::makeFromFindCommandForTests(cmd, NamespaceString());
 
-        auto cq = unittest::assertGet(
-            CanonicalQuery::canonicalize(opCtx(),
-                                         std::move(findCommand),
-                                         isExplain,
-                                         expCtx(),
-                                         ExtensionsCallbackNoop(),
-                                         MatchExpressionParser::kAllowAllSpecialFeatures));
-        return cq;
+        return std::make_unique<CanonicalQuery>(CanonicalQueryParams{
+            .expCtx = expCtx(),
+            .parsedFind = ParsedFindCommandParams{
+                .findCommand = std::move(findCommand),
+                .allowedFeatures = MatchExpressionParser::kAllowAllSpecialFeatures}});
     }
 
+    QueryPlannerParams makePlannerParams(const CollectionPtr& collection,
+                                         const CanonicalQuery& canonicalQuery) {
+        const MultipleCollectionAccessor collections{collection};
+        return QueryPlannerParams{
+            QueryPlannerParams::ArgsForSingleCollectionQuery{
+                .opCtx = opCtx(),
+                .canonicalQuery = canonicalQuery,
+                .collections = collections,
+                .plannerOptions = QueryPlannerParams::DEFAULT,
+                .traversalPreference = boost::none,
+            },
+        };
+    }
+
+    void assertSubplanFromCache(QueryStageSubplanTest* test,
+                                const dbtests::WriteContextForTests& ctx);
+
     const ServiceContext::UniqueOperationContext _opCtx = cc().makeOperationContext();
     ClockSource* _clock = _opCtx->getServiceContext()->getFastClockSource();
     boost::intrusive_ptr<ExpressionContext> _expCtx =
@@ -136,7 +201,7 @@ private:
  * back to regular planning.
  */
 TEST_F(QueryStageSubplanTest, QueryStageSubplanGeo2dOr) {
-    dbtests::WriteContextForTests ctx(opCtx(), nss.ns());
+    dbtests::WriteContextForTests ctx(opCtx(), nss.ns_forTest());
     addIndex(BSON("a"
                   << "2d"
                   << "b" << 1));
@@ -149,30 +214,29 @@ TEST_F(QueryStageSubplanTest, QueryStageSubplanGeo2dOr) {
 
     auto findCommand = std::make_unique<FindCommandRequest>(nss);
     findCommand->setFilter(query);
-    auto statusWithCQ = CanonicalQuery::canonicalize(opCtx(), std::move(findCommand));
-    ASSERT_OK(statusWithCQ.getStatus());
-    std::unique_ptr<CanonicalQuery> cq = std::move(statusWithCQ.getValue());
+    auto cq = std::make_unique<CanonicalQuery>(
+        CanonicalQueryParams{.expCtx = makeExpressionContext(opCtx(), *findCommand),
+                             .parsedFind = ParsedFindCommandParams{std::move(findCommand)}});
 
     CollectionPtr collection = ctx.getCollection();
 
     // Get planner params.
-    QueryPlannerParams plannerParams;
-    fillOutPlannerParams(opCtx(), collection, cq.get(), &plannerParams);
+    auto plannerParams = makePlannerParams(ctx.getCollection(), *cq);
 
     WorkingSet ws;
-    std::unique_ptr<SubplanStage> subplan(
-        new SubplanStage(_expCtx.get(), collection, &ws, plannerParams, cq.get()));
+    auto subplan = makeSubplanStage(collection, cq.get(), ws);
 
     // Plan selection should succeed due to falling back on regular planning.
     NoopYieldPolicy yieldPolicy(_expCtx->opCtx, _clock);
-    ASSERT_OK(subplan->pickBestPlan(&yieldPolicy));
+    ASSERT_OK(subplan->pickBestPlan(plannerParams, &yieldPolicy));
 }
 
 /**
- * Helper function to verify that branches of an $or can be subplanned from the cache. Assumes that
- * the indexes to be tested have already been created for the given WriteContextForTest.
+ * Helper function to verify that branches of an $or can be subplanned from the cache. Assumes
+ * that the indexes to be tested have already been created for the given WriteContextForTest.
  */
-void assertSubplanFromCache(QueryStageSubplanTest* test, const dbtests::WriteContextForTests& ctx) {
+void QueryStageSubplanTest::assertSubplanFromCache(QueryStageSubplanTest* test,
+                                                   const dbtests::WriteContextForTests& ctx) {
     // This query should result in a plan cache entry for the first $or branch, because
     // there are two competing indices. The second branch has only one relevant index, so
     // its winning plan should not be cached.
@@ -186,24 +250,22 @@ void assertSubplanFromCache(QueryStageSubplanTest* test, const dbtests::WriteCon
 
     auto findCommand = std::make_unique<FindCommandRequest>(nss);
     findCommand->setFilter(query);
-    auto statusWithCQ = CanonicalQuery::canonicalize(test->opCtx(), std::move(findCommand));
-    ASSERT_OK(statusWithCQ.getStatus());
-    std::unique_ptr<CanonicalQuery> cq = std::move(statusWithCQ.getValue());
+    auto cq = std::make_unique<CanonicalQuery>(
+        CanonicalQueryParams{.expCtx = makeExpressionContext(test->opCtx(), *findCommand),
+                             .parsedFind = ParsedFindCommandParams{std::move(findCommand)}});
 
     // Get planner params.
-    QueryPlannerParams plannerParams;
-    fillOutPlannerParams(test->opCtx(), collection, cq.get(), &plannerParams);
+    auto plannerParams = makePlannerParams(collection, *cq);
 
     // For the remainder of this test, ensure that cache entries are available immediately, and
     // don't need go through an 'inactive' state before being usable.
     internalQueryCacheDisableInactiveEntries.store(true);
 
     WorkingSet ws;
-    std::unique_ptr<SubplanStage> subplan(
-        new SubplanStage(test->expCtx(), collection, &ws, plannerParams, cq.get()));
+    auto subplan = makeSubplanStage(collection, cq.get(), ws);
 
     NoopYieldPolicy yieldPolicy(test->opCtx(), test->serviceContext()->getFastClockSource());
-    ASSERT_OK(subplan->pickBestPlan(&yieldPolicy));
+    ASSERT_OK(subplan->pickBestPlan(plannerParams, &yieldPolicy));
 
     // Nothing is in the cache yet, so neither branch should have been planned from
     // the plan cache.
@@ -213,9 +275,9 @@ void assertSubplanFromCache(QueryStageSubplanTest* test, const dbtests::WriteCon
     // If we repeat the same query, the plan for the first branch should have come from
     // the cache.
     ws.clear();
-    subplan.reset(new SubplanStage(test->expCtx(), collection, &ws, plannerParams, cq.get()));
+    subplan = makeSubplanStage(collection, cq.get(), ws);
 
-    ASSERT_OK(subplan->pickBestPlan(&yieldPolicy));
+    ASSERT_OK(subplan->pickBestPlan(plannerParams, &yieldPolicy));
 
     ASSERT_TRUE(subplan->branchPlannedFromCache(0));
     ASSERT_FALSE(subplan->branchPlannedFromCache(1));
@@ -225,7 +287,7 @@ void assertSubplanFromCache(QueryStageSubplanTest* test, const dbtests::WriteCon
  * Test the SubplanStage's ability to plan an individual branch using the plan cache.
  */
 TEST_F(QueryStageSubplanTest, QueryStageSubplanPlanFromCache) {
-    dbtests::WriteContextForTests ctx(opCtx(), nss.ns());
+    dbtests::WriteContextForTests ctx(opCtx(), nss.ns_forTest());
 
     addIndex(BSON("a" << 1));
     addIndex(BSON("a" << 1 << "b" << 1));
@@ -238,7 +300,7 @@ TEST_F(QueryStageSubplanTest, QueryStageSubplanPlanFromCache) {
  * Test that the SubplanStage can plan an individual branch from the cache using a $** index.
  */
 TEST_F(QueryStageSubplanTest, QueryStageSubplanPlanFromCacheWithWildcardIndex) {
-    dbtests::WriteContextForTests ctx(opCtx(), nss.ns());
+    dbtests::WriteContextForTests ctx(opCtx(), nss.ns_forTest());
 
     addIndex(BSON("$**" << 1));
     addIndex(BSON("a.$**" << 1));
@@ -247,19 +309,19 @@ TEST_F(QueryStageSubplanTest, QueryStageSubplanPlanFromCacheWithWildcardIndex) {
 }
 
 /**
- * Ensure that the subplan stage doesn't create a plan cache entry if there are no query results.
+ * Ensure that the subplan stage doesn't create a plan cache entry if there are no query results
+ * (when it is configured with the 'SometimesCache' policy for its per-branch callback).
  */
-TEST_F(QueryStageSubplanTest, QueryStageSubplanDontCacheZeroResults) {
-    dbtests::WriteContextForTests ctx(opCtx(), nss.ns());
+TEST_F(QueryStageSubplanTest, QueryStageSubplanDoesNotCacheZeroResults) {
+    dbtests::WriteContextForTests ctx(opCtx(), nss.ns_forTest());
 
     addIndex(BSON("a" << 1 << "b" << 1));
     addIndex(BSON("a" << 1));
     addIndex(BSON("c" << 1));
-    // Exclude field 'c' from the wildcard index to make sure that the field has only one relevant
-    // index.
+    // Exclude field 'c' from the wildcard index to make sure that the field has only one
+    // relevant index.
     addIndexWithWildcardProjection(BSON("$**" << 1), BSON("c" << 0));
 
-
     for (int i = 0; i < 10; i++) {
         insert(BSON("a" << 1 << "b" << i << "c" << i));
     }
@@ -273,20 +335,19 @@ TEST_F(QueryStageSubplanTest, QueryStageSubplanDontCacheZeroResults) {
 
     auto findCommand = std::make_unique<FindCommandRequest>(nss);
     findCommand->setFilter(query);
-    auto statusWithCQ = CanonicalQuery::canonicalize(opCtx(), std::move(findCommand));
-    ASSERT_OK(statusWithCQ.getStatus());
-    std::unique_ptr<CanonicalQuery> cq = std::move(statusWithCQ.getValue());
+    auto cq = std::make_unique<CanonicalQuery>(
+        CanonicalQueryParams{.expCtx = makeExpressionContext(opCtx(), *findCommand),
+                             .parsedFind = ParsedFindCommandParams{std::move(findCommand)}});
 
     // Get planner params.
-    QueryPlannerParams plannerParams;
-    fillOutPlannerParams(opCtx(), collection, cq.get(), &plannerParams);
+    auto plannerParams = makePlannerParams(ctx.getCollection(), *cq);
 
     WorkingSet ws;
-    std::unique_ptr<SubplanStage> subplan(
-        new SubplanStage(_expCtx.get(), collection, &ws, plannerParams, cq.get()));
+
+    auto subplan = makeSubplanStage(collection, cq.get(), ws);
 
     NoopYieldPolicy yieldPolicy(_expCtx->opCtx, _clock);
-    ASSERT_OK(subplan->pickBestPlan(&yieldPolicy));
+    ASSERT_OK(subplan->pickBestPlan(plannerParams, &yieldPolicy));
 
     // Nothing is in the cache yet, so neither branch should have been planned from
     // the plan cache.
@@ -297,9 +358,9 @@ TEST_F(QueryStageSubplanTest, QueryStageSubplanDontCacheZeroResults) {
     // from the cache (because the first call to pickBestPlan() refrained from creating any
     // cache entries).
     ws.clear();
-    subplan.reset(new SubplanStage(_expCtx.get(), collection, &ws, plannerParams, cq.get()));
+    subplan = makeSubplanStage(collection, cq.get(), ws);
 
-    ASSERT_OK(subplan->pickBestPlan(&yieldPolicy));
+    ASSERT_OK(subplan->pickBestPlan(plannerParams, &yieldPolicy));
 
     ASSERT_FALSE(subplan->branchPlannedFromCache(0));
     ASSERT_FALSE(subplan->branchPlannedFromCache(1));
@@ -309,13 +370,13 @@ TEST_F(QueryStageSubplanTest, QueryStageSubplanDontCacheZeroResults) {
  * Ensure that the subplan stage doesn't create a plan cache entry if the candidate plans tie.
  */
 TEST_F(QueryStageSubplanTest, QueryStageSubplanDontCacheTies) {
-    dbtests::WriteContextForTests ctx(opCtx(), nss.ns());
+    dbtests::WriteContextForTests ctx(opCtx(), nss.ns_forTest());
 
     addIndex(BSON("a" << 1 << "b" << 1));
     addIndex(BSON("a" << 1 << "c" << 1));
     addIndex(BSON("d" << 1));
-    // Exclude field 'd' from the wildcard index to make sure that the field has only one relevant
-    // index.
+    // Exclude field 'd' from the wildcard index to make sure that the field has only one
+    // relevant index.
     addIndexWithWildcardProjection(BSON("$**" << 1), BSON("d" << 0));
 
     for (int i = 0; i < 10; i++) {
@@ -331,20 +392,18 @@ TEST_F(QueryStageSubplanTest, QueryStageSubplanDontCacheTies) {
 
     auto findCommand = std::make_unique<FindCommandRequest>(nss);
     findCommand->setFilter(query);
-    auto statusWithCQ = CanonicalQuery::canonicalize(opCtx(), std::move(findCommand));
-    ASSERT_OK(statusWithCQ.getStatus());
-    std::unique_ptr<CanonicalQuery> cq = std::move(statusWithCQ.getValue());
+    auto cq = std::make_unique<CanonicalQuery>(
+        CanonicalQueryParams{.expCtx = makeExpressionContext(opCtx(), *findCommand),
+                             .parsedFind = ParsedFindCommandParams{std::move(findCommand)}});
 
     // Get planner params.
-    QueryPlannerParams plannerParams;
-    fillOutPlannerParams(opCtx(), collection, cq.get(), &plannerParams);
+    auto plannerParams = makePlannerParams(collection, *cq);
 
     WorkingSet ws;
-    std::unique_ptr<SubplanStage> subplan(
-        new SubplanStage(_expCtx.get(), collection, &ws, plannerParams, cq.get()));
+    auto subplan = makeSubplanStage(collection, cq.get(), ws);
 
     NoopYieldPolicy yieldPolicy(_expCtx->opCtx, _clock);
-    ASSERT_OK(subplan->pickBestPlan(&yieldPolicy));
+    ASSERT_OK(subplan->pickBestPlan(plannerParams, &yieldPolicy));
 
     // Nothing is in the cache yet, so neither branch should have been planned from
     // the plan cache.
@@ -355,9 +414,9 @@ TEST_F(QueryStageSubplanTest, QueryStageSubplanDontCacheTies) {
     // from the cache (because the first call to pickBestPlan() refrained from creating any
     // cache entries).
     ws.clear();
-    subplan.reset(new SubplanStage(_expCtx.get(), collection, &ws, plannerParams, cq.get()));
+    subplan = makeSubplanStage(collection, cq.get(), ws);
 
-    ASSERT_OK(subplan->pickBestPlan(&yieldPolicy));
+    ASSERT_OK(subplan->pickBestPlan(plannerParams, &yieldPolicy));
 
     ASSERT_FALSE(subplan->branchPlannedFromCache(0));
     ASSERT_FALSE(subplan->branchPlannedFromCache(1));
@@ -491,7 +550,7 @@ TEST_F(QueryStageSubplanTest, QueryStageSubplanCanUseSubplanning) {
  * Regression test for SERVER-19388.
  */
 TEST_F(QueryStageSubplanTest, QueryStageSubplanPlanRootedOrNE) {
-    dbtests::WriteContextForTests ctx(opCtx(), nss.ns());
+    dbtests::WriteContextForTests ctx(opCtx(), nss.ns_forTest());
     addIndex(BSON("a" << 1 << "b" << 1));
     addIndex(BSON("a" << 1 << "c" << 1));
 
@@ -502,21 +561,20 @@ TEST_F(QueryStageSubplanTest, QueryStageSubplanPlanRootedOrNE) {
     insert(BSON("_id" << 4));
 
     auto findCommand = std::make_unique<FindCommandRequest>(nss);
-    findCommand->setFilter(fromjson("{$or: [{a: 1}, {a: {$ne:1}}]}"));
+    findCommand->setFilter(fromjson("{$or: [{a: 1}, {a: {$ne:5}}]}"));
     findCommand->setSort(BSON("d" << 1));
-    auto cq = unittest::assertGet(CanonicalQuery::canonicalize(opCtx(), std::move(findCommand)));
+    auto cq = std::make_unique<CanonicalQuery>(
+        CanonicalQueryParams{.expCtx = makeExpressionContext(opCtx(), *findCommand),
+                             .parsedFind = ParsedFindCommandParams{std::move(findCommand)}});
 
     CollectionPtr collection = ctx.getCollection();
-
-    QueryPlannerParams plannerParams;
-    fillOutPlannerParams(opCtx(), collection, cq.get(), &plannerParams);
+    auto plannerParams = makePlannerParams(collection, *cq);
 
     WorkingSet ws;
-    std::unique_ptr<SubplanStage> subplan(
-        new SubplanStage(_expCtx.get(), collection, &ws, plannerParams, cq.get()));
+    auto subplan = makeSubplanStage(collection, cq.get(), ws);
 
     NoopYieldPolicy yieldPolicy(_expCtx->opCtx, _clock);
-    ASSERT_OK(subplan->pickBestPlan(&yieldPolicy));
+    ASSERT_OK(subplan->pickBestPlan(plannerParams, &yieldPolicy));
 
     size_t numResults = 0;
     PlanStage::StageState stageState = PlanStage::NEED_TIME;
@@ -532,20 +590,21 @@ TEST_F(QueryStageSubplanTest, QueryStageSubplanPlanRootedOrNE) {
 }
 
 TEST_F(QueryStageSubplanTest, ShouldReportErrorIfExceedsTimeLimitDuringPlanning) {
-    dbtests::WriteContextForTests ctx(opCtx(), nss.ns());
+    dbtests::WriteContextForTests ctx(opCtx(), nss.ns_forTest());
     // Build a query with a rooted $or.
     auto findCommand = std::make_unique<FindCommandRequest>(nss);
     findCommand->setFilter(BSON("$or" << BSON_ARRAY(BSON("p1" << 1) << BSON("p2" << 2))));
-    auto canonicalQuery =
-        uassertStatusOK(CanonicalQuery::canonicalize(opCtx(), std::move(findCommand)));
+    auto canonicalQuery = std::make_unique<CanonicalQuery>(
+        CanonicalQueryParams{.expCtx = makeExpressionContext(opCtx(), *findCommand),
+                             .parsedFind = ParsedFindCommandParams{std::move(findCommand)}});
 
     // Add 4 indices: 2 for each predicate to choose from.
     addIndex(BSON("p1" << 1 << "opt1" << 1));
-    addIndex(BSON("p1" << 1 << "opt2" << 1));
+    addIndex(BSON("p1" << -1 << "opt2" << 1));
     addIndex(BSON("p2" << 1 << "opt1" << 1));
-    addIndex(BSON("p2" << 1 << "opt2" << 1));
-    QueryPlannerParams params;
-    fillOutPlannerParams(opCtx(), ctx.getCollection(), canonicalQuery.get(), &params);
+    addIndex(BSON("p2" << -1 << "opt2" << 1));
+    auto coll = ctx.getCollection();
+    auto plannerParams = makePlannerParams(coll, *canonicalQuery);
 
     // Add some data so planning has to do some thinking.
     for (int i = 0; i < 100; ++i) {
@@ -557,44 +616,45 @@ TEST_F(QueryStageSubplanTest, ShouldReportErrorIfExceedsTimeLimitDuringPlanning)
 
     // Create the SubplanStage.
     WorkingSet workingSet;
-    auto coll = ctx.getCollection();
-    SubplanStage subplanStage(_expCtx.get(), coll, &workingSet, params, canonicalQuery.get());
+    auto subplanStage = makeSubplanStage(coll, canonicalQuery.get(), workingSet);
 
     AlwaysTimeOutYieldPolicy alwaysTimeOutPolicy(_expCtx->opCtx,
                                                  serviceContext()->getFastClockSource());
-    ASSERT_EQ(ErrorCodes::ExceededTimeLimit, subplanStage.pickBestPlan(&alwaysTimeOutPolicy));
+    ASSERT_EQ(ErrorCodes::ExceededTimeLimit,
+              subplanStage->pickBestPlan(plannerParams, &alwaysTimeOutPolicy));
 }
 
 TEST_F(QueryStageSubplanTest, ShouldReportErrorIfKilledDuringPlanning) {
-    dbtests::WriteContextForTests ctx(opCtx(), nss.ns());
+    dbtests::WriteContextForTests ctx(opCtx(), nss.ns_forTest());
     // Build a query with a rooted $or.
     auto findCommand = std::make_unique<FindCommandRequest>(nss);
     findCommand->setFilter(BSON("$or" << BSON_ARRAY(BSON("p1" << 1) << BSON("p2" << 2))));
-    auto canonicalQuery =
-        uassertStatusOK(CanonicalQuery::canonicalize(opCtx(), std::move(findCommand)));
+    auto canonicalQuery = std::make_unique<CanonicalQuery>(
+        CanonicalQueryParams{.expCtx = makeExpressionContext(opCtx(), *findCommand),
+                             .parsedFind = ParsedFindCommandParams{std::move(findCommand)}});
 
     // Add 4 indices: 2 for each predicate to choose from.
     addIndex(BSON("p1" << 1 << "opt1" << 1));
-    addIndex(BSON("p1" << 1 << "opt2" << 1));
+    addIndex(BSON("p1" << -1 << "opt2" << 1));
     addIndex(BSON("p2" << 1 << "opt1" << 1));
-    addIndex(BSON("p2" << 1 << "opt2" << 1));
-    QueryPlannerParams params;
-    fillOutPlannerParams(opCtx(), ctx.getCollection(), canonicalQuery.get(), &params);
+    addIndex(BSON("p2" << -1 << "opt2" << 1));
+    auto coll = ctx.getCollection();
+    auto plannerParams = makePlannerParams(coll, *canonicalQuery);
 
     // Create the SubplanStage.
     WorkingSet workingSet;
-    auto coll = ctx.getCollection();
-    SubplanStage subplanStage(_expCtx.get(), coll, &workingSet, params, canonicalQuery.get());
+    auto subplanStage = makeSubplanStage(coll, canonicalQuery.get(), workingSet);
 
     AlwaysPlanKilledYieldPolicy alwaysPlanKilledYieldPolicy(_expCtx->opCtx,
                                                             serviceContext()->getFastClockSource());
-    ASSERT_EQ(ErrorCodes::QueryPlanKilled, subplanStage.pickBestPlan(&alwaysPlanKilledYieldPolicy));
+    ASSERT_EQ(ErrorCodes::QueryPlanKilled,
+              subplanStage->pickBestPlan(plannerParams, &alwaysPlanKilledYieldPolicy));
 }
 
 TEST_F(QueryStageSubplanTest, ShouldThrowOnRestoreIfIndexDroppedBeforePlanSelection) {
     CollectionPtr collection;
     {
-        dbtests::WriteContextForTests ctx{opCtx(), nss.ns()};
+        dbtests::WriteContextForTests ctx{opCtx(), nss.ns_forTest()};
         addIndex(BSON("p1" << 1 << "opt1" << 1));
         addIndex(BSON("p1" << 1 << "opt2" << 1));
         addIndex(BSON("p2" << 1 << "opt1" << 1));
@@ -608,31 +668,27 @@ TEST_F(QueryStageSubplanTest, ShouldThrowOnRestoreIfIndexDroppedBeforePlanSelect
     // Build a query with a rooted $or.
     auto findCommand = std::make_unique<FindCommandRequest>(nss);
     findCommand->setFilter(BSON("$or" << BSON_ARRAY(BSON("p1" << 1) << BSON("p2" << 2))));
-    auto canonicalQuery =
-        uassertStatusOK(CanonicalQuery::canonicalize(opCtx(), std::move(findCommand)));
+    auto canonicalQuery = std::make_unique<CanonicalQuery>(
+        CanonicalQueryParams{.expCtx = makeExpressionContext(opCtx(), *findCommand),
+                             .parsedFind = ParsedFindCommandParams{std::move(findCommand)}});
 
     boost::optional<AutoGetCollectionForReadCommand> collLock;
     collLock.emplace(opCtx(), nss);
 
-    // Add 4 indices: 2 for each predicate to choose from, and one index which is not relevant to
-    // the query.
-    QueryPlannerParams params;
-    fillOutPlannerParams(opCtx(), collection, canonicalQuery.get(), &params);
-
     // Create the SubplanStage.
     WorkingSet workingSet;
-    SubplanStage subplanStage(_expCtx.get(), collection, &workingSet, params, canonicalQuery.get());
+    auto subplanStage = makeSubplanStage(collection, canonicalQuery.get(), workingSet);
 
-    // Mimic a yield by saving the state of the subplan stage. Then, drop an index not being used
-    // while yielded.
-    subplanStage.saveState();
+    // Mimic a yield by saving the state of the subplan stage. Then, drop an index not being
+    // used while yielded.
+    subplanStage->saveState();
     collLock.reset();
     dropIndex(BSON("irrelevant" << 1));
 
-    // Attempt to restore state. This should throw due the index drop. As a future improvement, we
-    // may wish to make the subplan stage tolerate drops of indices it is not using.
+    // Attempt to restore state. This should throw due the index drop. As a future improvement,
+    // we may wish to make the subplan stage tolerate drops of indices it is not using.
     collLock.emplace(opCtx(), nss);
-    ASSERT_THROWS_CODE(subplanStage.restoreState(&collLock->getCollection()),
+    ASSERT_THROWS_CODE(subplanStage->restoreState(&collLock->getCollection()),
                        DBException,
                        ErrorCodes::QueryPlanKilled);
 }
@@ -640,7 +696,7 @@ TEST_F(QueryStageSubplanTest, ShouldThrowOnRestoreIfIndexDroppedBeforePlanSelect
 TEST_F(QueryStageSubplanTest, ShouldNotThrowOnRestoreIfIndexDroppedAfterPlanSelection) {
     CollectionPtr collection;
     {
-        dbtests::WriteContextForTests ctx{opCtx(), nss.ns()};
+        dbtests::WriteContextForTests ctx{opCtx(), nss.ns_forTest()};
         addIndex(BSON("p1" << 1 << "opt1" << 1));
         addIndex(BSON("p1" << 1 << "opt2" << 1));
         addIndex(BSON("p2" << 1 << "opt1" << 1));
@@ -654,35 +710,35 @@ TEST_F(QueryStageSubplanTest, ShouldNotThrowOnRestoreIfIndexDroppedAfterPlanSele
     // Build a query with a rooted $or.
     auto findCommand = std::make_unique<FindCommandRequest>(nss);
     findCommand->setFilter(BSON("$or" << BSON_ARRAY(BSON("p1" << 1) << BSON("p2" << 2))));
-    auto canonicalQuery =
-        uassertStatusOK(CanonicalQuery::canonicalize(opCtx(), std::move(findCommand)));
+    auto canonicalQuery = std::make_unique<CanonicalQuery>(
+        CanonicalQueryParams{.expCtx = makeExpressionContext(opCtx(), *findCommand),
+                             .parsedFind = ParsedFindCommandParams{std::move(findCommand)}});
 
     boost::optional<AutoGetCollectionForReadCommand> collLock;
     collLock.emplace(opCtx(), nss);
 
 
-    // Add 4 indices: 2 for each predicate to choose from, and one index which is not relevant to
-    // the query.
-    QueryPlannerParams params;
-    fillOutPlannerParams(opCtx(), collection, canonicalQuery.get(), &params);
+    // Add 4 indices: 2 for each predicate to choose from, and one index which is not relevant
+    // to the query.
+    auto plannerParams = makePlannerParams(collection, *canonicalQuery);
 
     // Create the SubplanStage.
     WorkingSet workingSet;
-    SubplanStage subplanStage(_expCtx.get(), collection, &workingSet, params, canonicalQuery.get());
+    auto subplanStage = makeSubplanStage(collection, canonicalQuery.get(), workingSet);
 
     NoopYieldPolicy yieldPolicy(_expCtx->opCtx, serviceContext()->getFastClockSource());
-    ASSERT_OK(subplanStage.pickBestPlan(&yieldPolicy));
+    ASSERT_OK(subplanStage->pickBestPlan(plannerParams, &yieldPolicy));
 
-    // Mimic a yield by saving the state of the subplan stage and dropping our lock. Then drop an
-    // index not being used while yielded.
-    subplanStage.saveState();
+    // Mimic a yield by saving the state of the subplan stage and dropping our lock. Then drop
+    // an index not being used while yielded.
+    subplanStage->saveState();
     collLock.reset();
     dropIndex(BSON("irrelevant" << 1));
 
-    // Restoring state should succeed, since the plan selected by pickBestPlan() does not use the
-    // index {irrelevant: 1}.
+    // Restoring state should succeed, since the plan selected by pickBestPlan() does not use
+    // the index {irrelevant: 1}.
     collLock.emplace(opCtx(), nss);
-    subplanStage.restoreState(&collLock->getCollection());
+    subplanStage->restoreState(&collLock->getCollection());
 }
 
 }  // namespace
