@@ -27,73 +27,109 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <cstddef>
+#include <fmt/format.h>
+#include <memory>
+#include <string>
+#include <vector>
 
+#include "mongo/base/status.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonmisc.h"
 #include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/db/catalog/collection.h"
+#include "mongo/db/catalog/collection_catalog.h"
+#include "mongo/db/catalog/collection_options.h"
+#include "mongo/db/catalog/collection_write_path.h"
+#include "mongo/db/catalog/database.h"
 #include "mongo/db/catalog/database_holder.h"
 #include "mongo/db/catalog/drop_collection.h"
+#include "mongo/db/catalog/index_catalog.h"
+#include "mongo/db/catalog/index_catalog_entry.h"
 #include "mongo/db/catalog/rename_collection.h"
+#include "mongo/db/catalog_raii.h"
 #include "mongo/db/client.h"
+#include "mongo/db/concurrency/d_concurrency.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
+#include "mongo/db/curop.h"
 #include "mongo/db/db_raii.h"
 #include "mongo/db/index/index_access_method.h"
 #include "mongo/db/index/index_descriptor.h"
-#include "mongo/db/record_id.h"
-#include "mongo/dbtests/dbtests.h"
-#include "mongo/unittest/unittest.h"
-
-using mongo::unittest::assertGet;
-using std::list;
-using std::string;
-using std::unique_ptr;
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/repl/oplog.h"
+#include "mongo/db/service_context.h"
+#include "mongo/db/storage/key_string.h"
+#include "mongo/db/storage/record_data.h"
+#include "mongo/db/storage/record_store.h"
+#include "mongo/db/storage/sorted_data_interface.h"
+#include "mongo/db/storage/storage_engine.h"
+#include "mongo/db/storage/write_unit_of_work.h"
+#include "mongo/dbtests/dbtests.h"  // IWYU pragma: keep
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/uuid.h"
 
+namespace mongo {
 namespace RollbackTests {
-
 namespace {
+
+using unittest::assertGet;
+
 const auto kIndexVersion = IndexDescriptor::IndexVersion::kV2;
 
 void dropDatabase(OperationContext* opCtx, const NamespaceString& nss) {
     Lock::GlobalWrite globalWriteLock(opCtx);
     auto databaseHolder = DatabaseHolder::get(opCtx);
-    auto db = databaseHolder->getDb(opCtx, TenantDatabaseName(boost::none, nss.db()));
+    auto db = databaseHolder->getDb(opCtx, nss.dbName());
 
     if (db) {
+        WriteUnitOfWork wuow(opCtx);
         databaseHolder->dropDb(opCtx, db);
+        wuow.commit();
     }
 }
-bool collectionExists(OperationContext* opCtx, OldClientContext* ctx, const string& ns) {
-    return (bool)CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx,
-                                                                            NamespaceString(ns));
+
+bool collectionExists(OperationContext* opCtx, OldClientContext* ctx, StringData ns) {
+    return (bool)CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(
+        opCtx, NamespaceString::createNamespaceString_forTest(ns));
 }
 
 void createCollection(OperationContext* opCtx, const NamespaceString& nss) {
-    Lock::DBLock dbXLock(opCtx, nss.db(), MODE_X);
-    OldClientContext ctx(opCtx, nss.ns());
+    Lock::DBLock dbXLock(opCtx, nss.dbName(), MODE_X);
+    OldClientContext ctx(opCtx, nss);
     {
         WriteUnitOfWork uow(opCtx);
-        ASSERT(!collectionExists(opCtx, &ctx, nss.ns()));
+        ASSERT(!collectionExists(opCtx, &ctx, nss.ns_forTest()));
         CollectionOptions defaultCollectionOptions;
         ASSERT_OK(ctx.db()->userCreateNS(opCtx, nss, defaultCollectionOptions, false));
-        ASSERT(collectionExists(opCtx, &ctx, nss.ns()));
+        ASSERT(collectionExists(opCtx, &ctx, nss.ns_forTest()));
         uow.commit();
     }
 }
+
 Status renameCollection(OperationContext* opCtx,
                         const NamespaceString& source,
                         const NamespaceString& target) {
-    ASSERT_EQ(source.db(), target.db());
+    ASSERT_EQ(source.db_forTest(), target.db_forTest());
     return renameCollection(opCtx, source, target, {});
 }
+
 Status truncateCollection(OperationContext* opCtx, const NamespaceString& nss) {
     CollectionWriter coll(opCtx, nss);
-    return coll.getWritableCollection()->truncate(opCtx);
+    return coll.getWritableCollection(opCtx)->truncate(opCtx);
 }
 
 void insertRecord(OperationContext* opCtx, const NamespaceString& nss, const BSONObj& data) {
     auto coll = CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, nss);
     OpDebug* const nullOpDebug = nullptr;
-    ASSERT_OK(coll->insertDocument(opCtx, InsertStatement(data), nullOpDebug, false));
+    ASSERT_OK(collection_internal::insertDocument(
+        opCtx, CollectionPtr(coll), InsertStatement(data), nullOpDebug, false));
 }
+
 void assertOnlyRecord(OperationContext* opCtx, const NamespaceString& nss, const BSONObj& data) {
     auto coll = CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, nss);
     auto cursor = coll->getCursor(opCtx);
@@ -104,11 +140,13 @@ void assertOnlyRecord(OperationContext* opCtx, const NamespaceString& nss, const
 
     ASSERT(!cursor->next());
 }
+
 void assertEmpty(OperationContext* opCtx, const NamespaceString& nss) {
     auto coll = CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, nss);
     ASSERT(!coll->getCursor(opCtx)->next());
 }
-bool indexExists(OperationContext* opCtx, const NamespaceString& nss, const string& idxName) {
+
+bool indexExists(OperationContext* opCtx, const NamespaceString& nss, const std::string& idxName) {
     auto coll = CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, nss);
     return coll->getIndexCatalog()->findIndexByName(
                opCtx,
@@ -116,14 +154,16 @@ bool indexExists(OperationContext* opCtx, const NamespaceString& nss, const stri
                IndexCatalog::InclusionPolicy::kReady |
                    IndexCatalog::InclusionPolicy::kUnfinished) != nullptr;
 }
-bool indexReady(OperationContext* opCtx, const NamespaceString& nss, const string& idxName) {
+
+bool indexReady(OperationContext* opCtx, const NamespaceString& nss, const std::string& idxName) {
     auto coll = CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, nss);
     return coll->getIndexCatalog()->findIndexByName(
                opCtx, idxName, IndexCatalog::InclusionPolicy::kReady) != nullptr;
 }
+
 size_t getNumIndexEntries(OperationContext* opCtx,
                           const NamespaceString& nss,
-                          const string& idxName) {
+                          const std::string& idxName) {
     size_t numEntries = 0;
 
     auto coll = CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, nss);
@@ -133,10 +173,7 @@ size_t getNumIndexEntries(OperationContext* opCtx,
     if (desc) {
         auto iam = catalog->getEntry(desc)->accessMethod()->asSortedData();
         auto cursor = iam->newCursor(opCtx);
-        KeyString::Builder keyString(iam->getSortedDataInterface()->getKeyStringVersion(),
-                                     BSONObj(),
-                                     iam->getSortedDataInterface()->getOrdering());
-        for (auto kv = cursor->seek(keyString.getValueCopy()); kv; kv = cursor->next()) {
+        for (auto kv = cursor->next(); kv; kv = cursor->next()) {
             numEntries++;
         }
     }
@@ -144,13 +181,16 @@ size_t getNumIndexEntries(OperationContext* opCtx,
     return numEntries;
 }
 
-void dropIndex(OperationContext* opCtx, const NamespaceString& nss, const string& idxName) {
+void dropIndex(OperationContext* opCtx, const NamespaceString& nss, const std::string& idxName) {
     CollectionWriter coll(opCtx, nss);
-    auto desc = coll.getWritableCollection()->getIndexCatalog()->findIndexByName(opCtx, idxName);
-    ASSERT(desc);
-    ASSERT_OK(coll.getWritableCollection()->getIndexCatalog()->dropIndex(
-        opCtx, coll.getWritableCollection(), desc));
+    auto writableEntry =
+        coll.getWritableCollection(opCtx)->getIndexCatalog()->getWritableEntryByName(opCtx,
+                                                                                     idxName);
+    ASSERT(writableEntry);
+    ASSERT_OK(coll.getWritableCollection(opCtx)->getIndexCatalog()->dropIndexEntry(
+        opCtx, coll.getWritableCollection(opCtx), writableEntry));
 }
+
 }  // namespace
 
 template <bool rollback, bool defaultIndexes, bool capped>
@@ -162,14 +202,14 @@ public:
             return;
         }
 
-        string ns = "unittests.rollback_create_collection";
+        std::string ns = "unittests.rollback_create_collection";
         const ServiceContext::UniqueOperationContext opCtxPtr = cc().makeOperationContext();
         OperationContext& opCtx = *opCtxPtr;
-        NamespaceString nss(ns);
+        NamespaceString nss = NamespaceString::createNamespaceString_forTest(ns);
         dropDatabase(&opCtx, nss);
 
-        Lock::DBLock dbXLock(&opCtx, nss.db(), MODE_X);
-        OldClientContext ctx(&opCtx, ns);
+        Lock::DBLock dbXLock(&opCtx, nss.dbName(), MODE_X);
+        OldClientContext ctx(&opCtx, nss);
         {
             WriteUnitOfWork uow(&opCtx);
             ASSERT(!collectionExists(&opCtx, &ctx, ns));
@@ -199,14 +239,14 @@ public:
             return;
         }
 
-        string ns = "unittests.rollback_drop_collection";
+        std::string ns = "unittests.rollback_drop_collection";
         const ServiceContext::UniqueOperationContext opCtxPtr = cc().makeOperationContext();
         OperationContext& opCtx = *opCtxPtr;
-        NamespaceString nss(ns);
+        NamespaceString nss = NamespaceString::createNamespaceString_forTest(ns);
         dropDatabase(&opCtx, nss);
 
-        Lock::DBLock dbXLock(&opCtx, nss.db(), MODE_X);
-        OldClientContext ctx(&opCtx, ns);
+        Lock::DBLock dbXLock(&opCtx, nss.dbName(), MODE_X);
+        OldClientContext ctx(&opCtx, nss);
         {
             WriteUnitOfWork uow(&opCtx);
             ASSERT(!collectionExists(&opCtx, &ctx, ns));
@@ -223,7 +263,8 @@ public:
         {
             WriteUnitOfWork uow(&opCtx);
             ASSERT(collectionExists(&opCtx, &ctx, ns));
-            ASSERT_OK(ctx.db()->dropCollection(&opCtx, NamespaceString(ns)));
+            ASSERT_OK(ctx.db()->dropCollection(&opCtx,
+                                               NamespaceString::createNamespaceString_forTest(ns)));
             ASSERT(!collectionExists(&opCtx, &ctx, ns));
             if (!rollback) {
                 uow.commit();
@@ -246,8 +287,10 @@ public:
             return;
         }
 
-        NamespaceString source("unittests.rollback_rename_collection_src");
-        NamespaceString target("unittests.rollback_rename_collection_dest");
+        NamespaceString source = NamespaceString::createNamespaceString_forTest(
+            "unittests.rollback_rename_collection_src");
+        NamespaceString target = NamespaceString::createNamespaceString_forTest(
+            "unittests.rollback_rename_collection_dest");
         const ServiceContext::UniqueOperationContext opCtxPtr = cc().makeOperationContext();
         OperationContext& opCtx = *opCtxPtr;
 
@@ -255,38 +298,38 @@ public:
         dropDatabase(&opCtx, target);
 
         Lock::GlobalWrite globalWriteLock(&opCtx);
-        OldClientContext ctx(&opCtx, source.ns());
+        OldClientContext ctx(&opCtx, source);
 
         {
             WriteUnitOfWork uow(&opCtx);
-            ASSERT(!collectionExists(&opCtx, &ctx, source.ns()));
-            ASSERT(!collectionExists(&opCtx, &ctx, target.ns()));
+            ASSERT(!collectionExists(&opCtx, &ctx, source.ns_forTest()));
+            ASSERT(!collectionExists(&opCtx, &ctx, target.ns_forTest()));
             auto options = capped ? BSON("capped" << true << "size" << 1000) : BSONObj();
             CollectionOptions collectionOptions =
                 assertGet(CollectionOptions::parse(options, CollectionOptions::parseForCommand));
             ASSERT_OK(ctx.db()->userCreateNS(&opCtx, source, collectionOptions, defaultIndexes));
             uow.commit();
         }
-        ASSERT(collectionExists(&opCtx, &ctx, source.ns()));
-        ASSERT(!collectionExists(&opCtx, &ctx, target.ns()));
+        ASSERT(collectionExists(&opCtx, &ctx, source.ns_forTest()));
+        ASSERT(!collectionExists(&opCtx, &ctx, target.ns_forTest()));
 
         // END OF SETUP / START OF TEST
 
         {
             WriteUnitOfWork uow(&opCtx);
             ASSERT_OK(renameCollection(&opCtx, source, target));
-            ASSERT(!collectionExists(&opCtx, &ctx, source.ns()));
-            ASSERT(collectionExists(&opCtx, &ctx, target.ns()));
+            ASSERT(!collectionExists(&opCtx, &ctx, source.ns_forTest()));
+            ASSERT(collectionExists(&opCtx, &ctx, target.ns_forTest()));
             if (!rollback) {
                 uow.commit();
             }
         }
         if (rollback) {
-            ASSERT(collectionExists(&opCtx, &ctx, source.ns()));
-            ASSERT(!collectionExists(&opCtx, &ctx, target.ns()));
+            ASSERT(collectionExists(&opCtx, &ctx, source.ns_forTest()));
+            ASSERT(!collectionExists(&opCtx, &ctx, target.ns_forTest()));
         } else {
-            ASSERT(!collectionExists(&opCtx, &ctx, source.ns()));
-            ASSERT(collectionExists(&opCtx, &ctx, target.ns()));
+            ASSERT(!collectionExists(&opCtx, &ctx, source.ns_forTest()));
+            ASSERT(collectionExists(&opCtx, &ctx, target.ns_forTest()));
         }
     }
 };
@@ -300,8 +343,10 @@ public:
             return;
         }
 
-        NamespaceString source("unittests.rollback_rename_droptarget_collection_src");
-        NamespaceString target("unittests.rollback_rename_droptarget_collection_dest");
+        NamespaceString source = NamespaceString::createNamespaceString_forTest(
+            "unittests.rollback_rename_droptarget_collection_src");
+        NamespaceString target = NamespaceString::createNamespaceString_forTest(
+            "unittests.rollback_rename_droptarget_collection_dest");
         const ServiceContext::UniqueOperationContext opCtxPtr = cc().makeOperationContext();
         OperationContext& opCtx = *opCtxPtr;
 
@@ -309,7 +354,7 @@ public:
         dropDatabase(&opCtx, target);
 
         Lock::GlobalWrite globalWriteLock(&opCtx);
-        OldClientContext ctx(&opCtx, source.ns());
+        OldClientContext ctx(&opCtx, source);
 
         BSONObj sourceDoc = BSON("_id"
                                  << "source");
@@ -318,8 +363,8 @@ public:
 
         {
             WriteUnitOfWork uow(&opCtx);
-            ASSERT(!collectionExists(&opCtx, &ctx, source.ns()));
-            ASSERT(!collectionExists(&opCtx, &ctx, target.ns()));
+            ASSERT(!collectionExists(&opCtx, &ctx, source.ns_forTest()));
+            ASSERT(!collectionExists(&opCtx, &ctx, target.ns_forTest()));
             auto options = capped ? BSON("capped" << true << "size" << 1000) : BSONObj();
             CollectionOptions collectionOptions =
                 assertGet(CollectionOptions::parse(options, CollectionOptions::parseForCommand));
@@ -332,8 +377,8 @@ public:
 
             uow.commit();
         }
-        ASSERT(collectionExists(&opCtx, &ctx, source.ns()));
-        ASSERT(collectionExists(&opCtx, &ctx, target.ns()));
+        ASSERT(collectionExists(&opCtx, &ctx, source.ns_forTest()));
+        ASSERT(collectionExists(&opCtx, &ctx, target.ns_forTest()));
         assertOnlyRecord(&opCtx, source, sourceDoc);
         assertOnlyRecord(&opCtx, target, targetDoc);
 
@@ -347,21 +392,21 @@ public:
                 {},
                 DropCollectionSystemCollectionMode::kDisallowSystemCollectionDrops));
             ASSERT_OK(renameCollection(&opCtx, source, target));
-            ASSERT(!collectionExists(&opCtx, &ctx, source.ns()));
-            ASSERT(collectionExists(&opCtx, &ctx, target.ns()));
+            ASSERT(!collectionExists(&opCtx, &ctx, source.ns_forTest()));
+            ASSERT(collectionExists(&opCtx, &ctx, target.ns_forTest()));
             assertOnlyRecord(&opCtx, target, sourceDoc);
             if (!rollback) {
                 uow.commit();
             }
         }
         if (rollback) {
-            ASSERT(collectionExists(&opCtx, &ctx, source.ns()));
-            ASSERT(collectionExists(&opCtx, &ctx, target.ns()));
+            ASSERT(collectionExists(&opCtx, &ctx, source.ns_forTest()));
+            ASSERT(collectionExists(&opCtx, &ctx, target.ns_forTest()));
             assertOnlyRecord(&opCtx, source, sourceDoc);
             assertOnlyRecord(&opCtx, target, targetDoc);
         } else {
-            ASSERT(!collectionExists(&opCtx, &ctx, source.ns()));
-            ASSERT(collectionExists(&opCtx, &ctx, target.ns()));
+            ASSERT(!collectionExists(&opCtx, &ctx, source.ns_forTest()));
+            ASSERT(collectionExists(&opCtx, &ctx, target.ns_forTest()));
             assertOnlyRecord(&opCtx, target, sourceDoc);
         }
     }
@@ -371,13 +416,14 @@ template <bool rollback, bool defaultIndexes>
 class ReplaceCollection {
 public:
     void run() {
-        NamespaceString nss("unittests.rollback_replace_collection");
+        NamespaceString nss =
+            NamespaceString::createNamespaceString_forTest("unittests.rollback_replace_collection");
         const ServiceContext::UniqueOperationContext opCtxPtr = cc().makeOperationContext();
         OperationContext& opCtx = *opCtxPtr;
         dropDatabase(&opCtx, nss);
 
-        Lock::DBLock dbXLock(&opCtx, nss.db(), MODE_X);
-        OldClientContext ctx(&opCtx, nss.ns());
+        Lock::DBLock dbXLock(&opCtx, nss.dbName(), MODE_X);
+        OldClientContext ctx(&opCtx, nss);
 
         BSONObj oldDoc = BSON("_id"
                               << "old");
@@ -386,14 +432,14 @@ public:
 
         {
             WriteUnitOfWork uow(&opCtx);
-            ASSERT(!collectionExists(&opCtx, &ctx, nss.ns()));
+            ASSERT(!collectionExists(&opCtx, &ctx, nss.ns_forTest()));
             CollectionOptions collectionOptions =
                 assertGet(CollectionOptions::parse(BSONObj(), CollectionOptions::parseForCommand));
             ASSERT_OK(ctx.db()->userCreateNS(&opCtx, nss, collectionOptions, defaultIndexes));
             insertRecord(&opCtx, nss, oldDoc);
             uow.commit();
         }
-        ASSERT(collectionExists(&opCtx, &ctx, nss.ns()));
+        ASSERT(collectionExists(&opCtx, &ctx, nss.ns_forTest()));
         assertOnlyRecord(&opCtx, nss, oldDoc);
 
         // END OF SETUP / START OF TEST
@@ -405,18 +451,18 @@ public:
                 nss,
                 {},
                 DropCollectionSystemCollectionMode::kDisallowSystemCollectionDrops));
-            ASSERT(!collectionExists(&opCtx, &ctx, nss.ns()));
+            ASSERT(!collectionExists(&opCtx, &ctx, nss.ns_forTest()));
             CollectionOptions collectionOptions =
                 assertGet(CollectionOptions::parse(BSONObj(), CollectionOptions::parseForCommand));
             ASSERT_OK(ctx.db()->userCreateNS(&opCtx, nss, collectionOptions, defaultIndexes));
-            ASSERT(collectionExists(&opCtx, &ctx, nss.ns()));
+            ASSERT(collectionExists(&opCtx, &ctx, nss.ns_forTest()));
             insertRecord(&opCtx, nss, newDoc);
             assertOnlyRecord(&opCtx, nss, newDoc);
             if (!rollback) {
                 uow.commit();
             }
         }
-        ASSERT(collectionExists(&opCtx, &ctx, nss.ns()));
+        ASSERT(collectionExists(&opCtx, &ctx, nss.ns_forTest()));
         if (rollback) {
             assertOnlyRecord(&opCtx, nss, oldDoc);
         } else {
@@ -429,25 +475,26 @@ template <bool rollback, bool defaultIndexes>
 class TruncateCollection {
 public:
     void run() {
-        NamespaceString nss("unittests.rollback_truncate_collection");
+        NamespaceString nss = NamespaceString::createNamespaceString_forTest(
+            "unittests.rollback_truncate_collection");
         const ServiceContext::UniqueOperationContext opCtxPtr = cc().makeOperationContext();
         OperationContext& opCtx = *opCtxPtr;
         dropDatabase(&opCtx, nss);
 
-        Lock::DBLock dbXLock(&opCtx, nss.db(), MODE_X);
-        OldClientContext ctx(&opCtx, nss.ns());
+        Lock::DBLock dbXLock(&opCtx, nss.dbName(), MODE_X);
+        OldClientContext ctx(&opCtx, nss);
 
         BSONObj doc = BSON("_id"
                            << "foo");
 
-        ASSERT(!collectionExists(&opCtx, &ctx, nss.ns()));
+        ASSERT(!collectionExists(&opCtx, &ctx, nss.ns_forTest()));
         {
             WriteUnitOfWork uow(&opCtx);
 
             CollectionOptions collectionOptions =
                 assertGet(CollectionOptions::parse(BSONObj(), CollectionOptions::parseForCommand));
             ASSERT_OK(ctx.db()->userCreateNS(&opCtx, nss, collectionOptions, defaultIndexes));
-            ASSERT(collectionExists(&opCtx, &ctx, nss.ns()));
+            ASSERT(collectionExists(&opCtx, &ctx, nss.ns_forTest()));
             insertRecord(&opCtx, nss, doc);
             assertOnlyRecord(&opCtx, nss, doc);
             uow.commit();
@@ -460,14 +507,14 @@ public:
             WriteUnitOfWork uow(&opCtx);
 
             ASSERT_OK(truncateCollection(&opCtx, nss));
-            ASSERT(collectionExists(&opCtx, &ctx, nss.ns()));
+            ASSERT(collectionExists(&opCtx, &ctx, nss.ns_forTest()));
             assertEmpty(&opCtx, nss);
 
             if (!rollback) {
                 uow.commit();
             }
         }
-        ASSERT(collectionExists(&opCtx, &ctx, nss.ns()));
+        ASSERT(collectionExists(&opCtx, &ctx, nss.ns_forTest()));
         if (rollback) {
             assertOnlyRecord(&opCtx, nss, doc);
         } else {
@@ -480,18 +527,18 @@ template <bool rollback>
 class CreateIndex {
 public:
     void run() {
-        string ns = "unittests.rollback_create_index";
+        std::string ns = "unittests.rollback_create_index";
         const ServiceContext::UniqueOperationContext opCtxPtr = cc().makeOperationContext();
         OperationContext& opCtx = *opCtxPtr;
-        NamespaceString nss(ns);
+        NamespaceString nss = NamespaceString::createNamespaceString_forTest(ns);
         dropDatabase(&opCtx, nss);
         createCollection(&opCtx, nss);
 
-        AutoGetDb autoDb(&opCtx, nss.db(), MODE_X);
+        AutoGetDb autoDb(&opCtx, nss.dbName(), MODE_X);
 
         CollectionWriter coll(&opCtx, nss);
 
-        string idxName = "a";
+        std::string idxName = "a";
         BSONObj spec = BSON("key" << BSON("a" << 1) << "name" << idxName << "v"
                                   << static_cast<int>(kIndexVersion));
 
@@ -499,9 +546,9 @@ public:
 
         {
             WriteUnitOfWork uow(&opCtx);
-            IndexCatalog* catalog = coll.getWritableCollection()->getIndexCatalog();
-            ASSERT_OK(
-                catalog->createIndexOnEmptyCollection(&opCtx, coll.getWritableCollection(), spec));
+            IndexCatalog* catalog = coll.getWritableCollection(&opCtx)->getIndexCatalog();
+            ASSERT_OK(catalog->createIndexOnEmptyCollection(
+                &opCtx, coll.getWritableCollection(&opCtx), spec));
             insertRecord(&opCtx, nss, BSON("a" << 1));
             insertRecord(&opCtx, nss, BSON("a" << 2));
             insertRecord(&opCtx, nss, BSON("a" << 3));
@@ -522,26 +569,26 @@ template <bool rollback>
 class DropIndex {
 public:
     void run() {
-        string ns = "unittests.rollback_drop_index";
+        std::string ns = "unittests.rollback_drop_index";
         const ServiceContext::UniqueOperationContext opCtxPtr = cc().makeOperationContext();
         OperationContext& opCtx = *opCtxPtr;
-        NamespaceString nss(ns);
+        NamespaceString nss = NamespaceString::createNamespaceString_forTest(ns);
         dropDatabase(&opCtx, nss);
         createCollection(&opCtx, nss);
 
-        AutoGetDb autoDb(&opCtx, nss.db(), MODE_X);
+        AutoGetDb autoDb(&opCtx, nss.dbName(), MODE_X);
 
         CollectionWriter coll(&opCtx, nss);
 
-        string idxName = "a";
+        std::string idxName = "a";
         BSONObj spec = BSON("key" << BSON("a" << 1) << "name" << idxName << "v"
                                   << static_cast<int>(kIndexVersion));
 
         {
             WriteUnitOfWork uow(&opCtx);
-            IndexCatalog* catalog = coll.getWritableCollection()->getIndexCatalog();
-            ASSERT_OK(
-                catalog->createIndexOnEmptyCollection(&opCtx, coll.getWritableCollection(), spec));
+            IndexCatalog* catalog = coll.getWritableCollection(&opCtx)->getIndexCatalog();
+            ASSERT_OK(catalog->createIndexOnEmptyCollection(
+                &opCtx, coll.getWritableCollection(&opCtx), spec));
             insertRecord(&opCtx, nss, BSON("a" << 1));
             insertRecord(&opCtx, nss, BSON("a" << 2));
             insertRecord(&opCtx, nss, BSON("a" << 3));
@@ -576,17 +623,17 @@ template <bool rollback>
 class CreateDropIndex {
 public:
     void run() {
-        string ns = "unittests.rollback_create_drop_index";
+        std::string ns = "unittests.rollback_create_drop_index";
         const ServiceContext::UniqueOperationContext opCtxPtr = cc().makeOperationContext();
         OperationContext& opCtx = *opCtxPtr;
-        NamespaceString nss(ns);
+        NamespaceString nss = NamespaceString::createNamespaceString_forTest(ns);
         dropDatabase(&opCtx, nss);
         createCollection(&opCtx, nss);
 
-        AutoGetDb autoDb(&opCtx, nss.db(), MODE_X);
+        AutoGetDb autoDb(&opCtx, nss.dbName(), MODE_X);
         CollectionWriter coll(&opCtx, nss);
 
-        string idxName = "a";
+        std::string idxName = "a";
         BSONObj spec = BSON("key" << BSON("a" << 1) << "name" << idxName << "v"
                                   << static_cast<int>(kIndexVersion));
 
@@ -594,10 +641,10 @@ public:
 
         {
             WriteUnitOfWork uow(&opCtx);
-            IndexCatalog* catalog = coll.getWritableCollection()->getIndexCatalog();
+            IndexCatalog* catalog = coll.getWritableCollection(&opCtx)->getIndexCatalog();
 
-            ASSERT_OK(
-                catalog->createIndexOnEmptyCollection(&opCtx, coll.getWritableCollection(), spec));
+            ASSERT_OK(catalog->createIndexOnEmptyCollection(
+                &opCtx, coll.getWritableCollection(&opCtx), spec));
             insertRecord(&opCtx, nss, BSON("a" << 1));
             insertRecord(&opCtx, nss, BSON("a" << 2));
             insertRecord(&opCtx, nss, BSON("a" << 3));
@@ -620,18 +667,18 @@ template <bool rollback>
 class CreateCollectionAndIndexes {
 public:
     void run() {
-        string ns = "unittests.rollback_create_collection_and_indexes";
+        std::string ns = "unittests.rollback_create_collection_and_indexes";
         const ServiceContext::UniqueOperationContext opCtxPtr = cc().makeOperationContext();
         OperationContext& opCtx = *opCtxPtr;
-        NamespaceString nss(ns);
+        NamespaceString nss = NamespaceString::createNamespaceString_forTest(ns);
         dropDatabase(&opCtx, nss);
 
-        Lock::DBLock dbXLock(&opCtx, nss.db(), MODE_X);
-        OldClientContext ctx(&opCtx, nss.ns());
+        Lock::DBLock dbXLock(&opCtx, nss.dbName(), MODE_X);
+        OldClientContext ctx(&opCtx, nss);
 
-        string idxNameA = "indexA";
-        string idxNameB = "indexB";
-        string idxNameC = "indexC";
+        std::string idxNameA = "indexA";
+        std::string idxNameB = "indexB";
+        std::string idxNameC = "indexC";
         BSONObj specA = BSON("key" << BSON("a" << 1) << "name" << idxNameA << "v"
                                    << static_cast<int>(kIndexVersion));
         BSONObj specB = BSON("key" << BSON("b" << 1) << "name" << idxNameB << "v"
@@ -643,13 +690,13 @@ public:
 
         {
             WriteUnitOfWork uow(&opCtx);
-            ASSERT(!collectionExists(&opCtx, &ctx, nss.ns()));
+            ASSERT(!collectionExists(&opCtx, &ctx, nss.ns_forTest()));
             CollectionOptions collectionOptions =
                 assertGet(CollectionOptions::parse(BSONObj(), CollectionOptions::parseForCommand));
             ASSERT_OK(ctx.db()->userCreateNS(&opCtx, nss, collectionOptions, false));
-            ASSERT(collectionExists(&opCtx, &ctx, nss.ns()));
+            ASSERT(collectionExists(&opCtx, &ctx, nss.ns_forTest()));
             CollectionWriter coll(&opCtx, nss);
-            auto writableColl = coll.getWritableCollection();
+            auto writableColl = coll.getWritableCollection(&opCtx);
             IndexCatalog* catalog = writableColl->getIndexCatalog();
 
             ASSERT_OK(catalog->createIndexOnEmptyCollection(&opCtx, writableColl, specA));
@@ -671,8 +718,7 @@ public:
     }
 };
 
-
-class All : public OldStyleSuiteSpecification {
+class All : public unittest::OldStyleSuiteSpecification {
 public:
     All() : OldStyleSuiteSpecification("rollback") {}
 
@@ -701,7 +747,7 @@ public:
         add<T<true, true, true>>();
     }
 
-    void setupTests() {
+    void setupTests() override {
         addAll<CreateCollection>();
         addAll<RenameCollection>();
         addAll<DropCollection>();
@@ -715,6 +761,7 @@ public:
     }
 };
 
-OldStyleSuiteInitializer<All> all;
+unittest::OldStyleSuiteInitializer<All> all;
 
 }  // namespace RollbackTests
+}  // namespace mongo
