@@ -29,31 +29,51 @@
 
 #pragma once
 
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
 #include <memory>
+#include <tuple>
+#include <utility>
+#include <vector>
 
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
 #include "mongo/bson/bsonobj.h"
 #include "mongo/client/async_remote_command_targeter_adapter.h"
+#include "mongo/client/connection_string.h"
+#include "mongo/client/hedging_mode_gen.h"
 #include "mongo/client/mongo_uri.h"
+#include "mongo/client/read_preference.h"
+#include "mongo/client/remote_command_targeter.h"
 #include "mongo/client/remote_command_targeter_factory_impl.h"
+#include "mongo/db/baton.h"
 #include "mongo/db/operation_context.h"
+#include "mongo/db/pipeline/aggregation_request_helper.h"
+#include "mongo/db/service_context.h"
 #include "mongo/db/shard_id.h"
 #include "mongo/executor/async_rpc_error_info.h"
 #include "mongo/executor/async_rpc_retry_policy.h"
 #include "mongo/executor/async_rpc_targeter.h"
 #include "mongo/executor/remote_command_response.h"
 #include "mongo/executor/task_executor.h"
-#include "mongo/idl/generic_args_with_types_gen.h"
+#include "mongo/idl/generic_argument_gen.h"
+#include "mongo/idl/idl_parser.h"
 #include "mongo/logv2/log.h"
 #include "mongo/rpc/get_status_from_command_result.h"
 #include "mongo/s/async_rpc_shard_targeter.h"
+#include "mongo/s/catalog/type_tags.h"
+#include "mongo/transport/baton.h"
 #include "mongo/util/assert_util.h"
 #include "mongo/util/cancellation.h"
+#include "mongo/util/duration.h"
 #include "mongo/util/future.h"
 #include "mongo/util/future_util.h"
 #include "mongo/util/net/hostandport.h"
+#include "mongo/util/out_of_line_executor.h"
 #include "mongo/util/time_support.h"
-
-#define MONGO_LOGV2_DEFAULT_COMPONENT mongo::logv2::LogComponent::kExecutor
+#include "mongo/util/uuid.h"
 
 /**
  * This header provides an API of `sendCommand(...)` functions that can be used to asynchronously
@@ -66,18 +86,6 @@
 namespace mongo::async_rpc {
 using executor::TaskExecutor;
 
-/**
- * Contains generic argument fields that can be part of any command request, separated based on
- * whether fields are part of the stable API. The generic arguments are generated from
- * '../idl/generic_args_with_types.idl'.
- */
-struct GenericArgs {
-    GenericArgs(GenericArgsAPIV1 stable = GenericArgsAPIV1(),
-                GenericArgsAPIV1Unstable unstable = GenericArgsAPIV1Unstable())
-        : stable{stable}, unstable{unstable} {}
-    GenericArgsAPIV1 stable;
-    GenericArgsAPIV1Unstable unstable;
-};
 
 /**
  * The response type used by `sendCommand(...)`  functions, containing the typed response to the
@@ -87,6 +95,7 @@ template <typename CommandReplyType>
 struct AsyncRPCResponse {
     CommandReplyType response;
     HostAndPort targetUsed;
+    Microseconds elapsed;
     GenericReplyFields genericReplyFields;
 };
 
@@ -97,27 +106,40 @@ struct AsyncRPCResponse {
 template <>
 struct AsyncRPCResponse<void> {
     HostAndPort targetUsed;
+    Microseconds elapsed;
 };
 
 template <typename CommandType>
 struct AsyncRPCOptions {
-    AsyncRPCOptions(CommandType cmd,
-                    std::shared_ptr<executor::TaskExecutor> exec,
+    AsyncRPCOptions(const std::shared_ptr<executor::TaskExecutor>& exec,
                     CancellationToken token,
+                    const CommandType& cmd,
                     std::shared_ptr<RetryPolicy> retryPolicy = std::make_shared<NeverRetryPolicy>(),
-                    GenericArgs genericArgs = GenericArgs(),
+                    GenericArguments genericArgs = GenericArguments(),
                     BatonHandle baton = nullptr)
-        : cmd{cmd},
-          exec{exec},
+        : exec{exec},
           token{token},
+          cmd{cmd},
+          retryPolicy{retryPolicy},
+          genericArgs{genericArgs},
+          baton{std::move(baton)} {}
+    AsyncRPCOptions(const std::shared_ptr<executor::TaskExecutor>& exec,
+                    CancellationToken token,
+                    const CommandType& cmd,
+                    GenericArguments genericArgs,
+                    std::shared_ptr<RetryPolicy> retryPolicy = std::make_shared<NeverRetryPolicy>(),
+                    BatonHandle baton = nullptr)
+        : exec{exec},
+          token{token},
+          cmd{cmd},
           retryPolicy{retryPolicy},
           genericArgs{genericArgs},
           baton{std::move(baton)} {}
-    CommandType cmd;
     std::shared_ptr<executor::TaskExecutor> exec;
     CancellationToken token;
+    CommandType cmd;
     std::shared_ptr<RetryPolicy> retryPolicy;
-    GenericArgs genericArgs;
+    GenericArguments genericArgs;
     BatonHandle baton;
 };
 
@@ -129,6 +151,7 @@ namespace detail {
 struct AsyncRPCInternalResponse {
     BSONObj response;
     HostAndPort targetUsed;
+    Microseconds elapsed;
 };
 
 /**
@@ -146,26 +169,30 @@ class AsyncRPCRunner {
 public:
     virtual ~AsyncRPCRunner() = default;
     virtual ExecutorFuture<AsyncRPCInternalResponse> _sendCommand(
-        StringData dbName,
-        BSONObj cmdBSON,
-        Targeter* targeter,
-        OperationContext* opCtx,
         std::shared_ptr<TaskExecutor> exec,
         CancellationToken token,
-        BatonHandle baton) = 0;
-    ExecutorFuture<AsyncRPCInternalResponse> _sendCommand(StringData dbName,
-                                                          BSONObj cmdBSON,
-                                                          Targeter* targeter,
-                                                          OperationContext* opCtx,
-                                                          std::shared_ptr<TaskExecutor> exec,
-                                                          CancellationToken token) {
-        return _sendCommand(std::move(dbName),
-                            std::move(cmdBSON),
-                            std::move(targeter),
-                            std::move(opCtx),
-                            std::move(exec),
+        OperationContext* opCtx,
+        Targeter* targeter,
+        const DatabaseName& dbName,
+        BSONObj cmdBSON,
+        BatonHandle baton,
+        boost::optional<UUID> clientOperationKey) = 0;
+    ExecutorFuture<AsyncRPCInternalResponse> _sendCommand(
+        std::shared_ptr<TaskExecutor> exec,
+        CancellationToken token,
+        OperationContext* opCtx,
+        Targeter* targeter,
+        const DatabaseName& dbName,
+        BSONObj cmdBSON,
+        boost::optional<UUID> clientOperationKey) {
+        return _sendCommand(std::move(exec),
                             std::move(token),
-                            nullptr);
+                            std::move(opCtx),
+                            std::move(targeter),
+                            dbName,
+                            std::move(cmdBSON),
+                            nullptr,
+                            std::move(clientOperationKey));
     }
     static AsyncRPCRunner* get(ServiceContext* serviceContext);
     static void set(ServiceContext* serviceContext, std::unique_ptr<AsyncRPCRunner> theRunner);
@@ -176,13 +203,13 @@ public:
  * details about any error, local or remote, contained in `r`.
  */
 inline Status makeErrorIfNeeded(TaskExecutor::ResponseOnAnyStatus r,
-                                std::vector<HostAndPort> targetsAttempted) {
+                                boost::optional<HostAndPort> targetAttempted) {
     if (r.status.isOK() && getStatusFromCommandResult(r.data).isOK() &&
         getWriteConcernStatusFromCommandResult(r.data).isOK() &&
         getFirstWriteErrorStatusFromCommandResult(r.data).isOK()) {
         return Status::OK();
     }
-    return {AsyncRPCErrorInfo(r, targetsAttempted), "Remote command execution failed"};
+    return {AsyncRPCErrorInfo(r, targetAttempted), "Remote command execution failed"};
 }
 
 /**
@@ -199,8 +226,8 @@ struct RetryDelayAsBackoff {
 class ProxyingExecutor : public OutOfLineExecutor,
                          public std::enable_shared_from_this<ProxyingExecutor> {
 public:
-    ProxyingExecutor(BatonHandle baton, std::shared_ptr<TaskExecutor> executor)
-        : _baton{std::move(baton)}, _executor{std::move(executor)} {}
+    ProxyingExecutor(std::shared_ptr<TaskExecutor> executor, BatonHandle baton)
+        : _executor{std::move(executor)}, _baton{std::move(baton)} {}
 
     void schedule(Task func) override {
         if (_baton)
@@ -217,29 +244,30 @@ public:
     }
 
 private:
-    BatonHandle _baton;
     std::shared_ptr<TaskExecutor> _executor;
+    BatonHandle _baton;
 };
 
 template <typename CommandType>
 ExecutorFuture<AsyncRPCResponse<typename CommandType::Reply>> sendCommandWithRunner(
-    BSONObj cmdBSON,
     std::shared_ptr<AsyncRPCOptions<CommandType>> options,
-    detail::AsyncRPCRunner* runner,
     OperationContext* opCtx,
-    std::unique_ptr<Targeter> targeter) {
+    detail::AsyncRPCRunner* runner,
+    std::unique_ptr<Targeter> targeter,
+    BSONObj cmdBSON) {
     using ReplyType = AsyncRPCResponse<typename CommandType::Reply>;
-    auto proxyExec = std::make_shared<ProxyingExecutor>(options->baton, options->exec);
+    auto proxyExec = std::make_shared<ProxyingExecutor>(options->exec, options->baton);
     auto tryBody = [=, targeter = std::move(targeter)] {
         // Execute the command after extracting the db name and bson from the CommandType.
         // Wrapping this function allows us to separate the CommandType parsing logic from the
         // implementation details of executing the remote command asynchronously.
-        return runner->_sendCommand(options->cmd.getDbName().db(),
-                                    cmdBSON,
-                                    targeter.get(),
+        return runner->_sendCommand(options->exec,
+                                    options->token,
                                     opCtx,
-                                    options->exec,
-                                    options->token);
+                                    targeter.get(),
+                                    options->cmd.getDbName(),
+                                    cmdBSON,
+                                    options->genericArgs.getClientOperationKey());
     };
     auto resFuture =
         AsyncTry<decltype(tryBody)>(std::move(tryBody))
@@ -255,11 +283,9 @@ ExecutorFuture<AsyncRPCResponse<typename CommandType::Reply>> sendCommandWithRun
         .then([](detail::AsyncRPCInternalResponse r) -> ReplyType {
             auto res = CommandType::Reply::parseSharingOwnership(IDLParserContext("AsyncRPCRunner"),
                                                                  r.response);
-            auto stableReplyFields = GenericReplyFieldsWithTypesV1::parseSharingOwnership(
+            auto genericReplyFields = GenericReplyFields::parseSharingOwnership(
                 IDLParserContext("AsyncRPCRunner"), r.response);
-            auto unstableReplyFields = GenericReplyFieldsWithTypesUnstableV1::parseSharingOwnership(
-                IDLParserContext("AsyncRPCRunner"), r.response);
-            return {res, r.targetUsed, GenericReplyFields{stableReplyFields, unstableReplyFields}};
+            return {res, r.targetUsed, r.elapsed, std::move(genericReplyFields)};
         })
         .unsafeToInlineFuture()
         .onError(
@@ -269,11 +295,8 @@ ExecutorFuture<AsyncRPCResponse<typename CommandType::Reply>> sendCommandWithRun
                 if (s.code() == ErrorCodes::RemoteCommandExecutionError) {
                     return s;
                 }
-                // TODO(SERVER-72974): Replace with named error codes.
-                const auto IDLParserDuplicateFieldError = 40413;
-                const auto IDLParserMissingFieldError = 40414;
-                if (s.code() == IDLParserDuplicateFieldError ||
-                    s.code() == IDLParserMissingFieldError) {
+
+                if (ErrorCodes::isIDLParseError(s.code())) {
                     // Failing here indicates that an IDL struct type may be incorrectly defined
                     // and we were unable to parse a generic reply field from the response.
                     tasserted(
@@ -295,6 +318,14 @@ ExecutorFuture<AsyncRPCResponse<typename CommandType::Reply>> sendCommandWithRun
 }
 }  // namespace detail
 
+namespace {
+void createOperationKeyIfNeeded(GenericArguments& genericArgs) {
+    if (!genericArgs.getClientOperationKey()) {
+        genericArgs.setClientOperationKey(UUID::gen());
+    }
+}
+}  // namespace
+
 /**
  * Execute the command asynchronously on the given target with the provided executor.
  *
@@ -326,10 +357,10 @@ ExecutorFuture<AsyncRPCResponse<typename CommandType::Reply>> sendCommand(
     OperationContext* opCtx,
     std::unique_ptr<Targeter> targeter) {
     auto runner = detail::AsyncRPCRunner::get(opCtx->getServiceContext());
-    auto genericArgs =
-        options->genericArgs.stable.toBSON().addFields(options->genericArgs.unstable.toBSON());
+    createOperationKeyIfNeeded(options->genericArgs);
+    auto genericArgs = options->genericArgs.toBSON();
     auto cmdBSON = options->cmd.toBSON(genericArgs);
-    return detail::sendCommandWithRunner(cmdBSON, options, runner, opCtx, std::move(targeter));
+    return detail::sendCommandWithRunner(options, opCtx, runner, std::move(targeter), cmdBSON);
 }
 
 /**
@@ -346,10 +377,10 @@ ExecutorFuture<AsyncRPCResponse<typename CommandType::Reply>> sendCommand(
     // Wrapping this function allows us to separate the CommandType parsing logic from the
     // implementation details of executing the remote command asynchronously.
     auto runner = detail::AsyncRPCRunner::get(svcCtx);
-    auto genericArgs =
-        options->genericArgs.stable.toBSON().addFields(options->genericArgs.unstable.toBSON());
+    createOperationKeyIfNeeded(options->genericArgs);
+    auto genericArgs = options->genericArgs.toBSON();
     auto cmdBSON = options->cmd.toBSON(genericArgs);
-    return detail::sendCommandWithRunner(cmdBSON, options, runner, nullptr, std::move(targeter));
+    return detail::sendCommandWithRunner(options, nullptr, runner, std::move(targeter), cmdBSON);
 }
 
 /**
@@ -362,10 +393,12 @@ ExecutorFuture<AsyncRPCResponse<typename CommandType::Reply>> sendCommand(
     ShardId shardId) {
     ReadPreferenceSetting readPref;
     std::unique_ptr<Targeter> targeter =
-        std::make_unique<ShardIdTargeter>(shardId, opCtx, readPref, options->exec);
+        std::make_unique<ShardIdTargeter>(options->exec, opCtx, shardId, readPref);
     auto runner = detail::AsyncRPCRunner::get(opCtx->getServiceContext());
-    auto cmdBSON = options->cmd.toBSON({});
-    return detail::sendCommandWithRunner(cmdBSON, options, runner, opCtx, std::move(targeter));
+    createOperationKeyIfNeeded(options->genericArgs);
+    auto genericArgs = options->genericArgs.toBSON();
+    auto cmdBSON = options->cmd.toBSON(genericArgs);
+    return detail::sendCommandWithRunner(options, opCtx, runner, std::move(targeter), cmdBSON);
 }
 
 /**
@@ -385,8 +418,10 @@ ExecutorFuture<AsyncRPCResponse<typename CommandType::Reply>> sendCommand(
         std::make_unique<AsyncRemoteCommandTargeterAdapter>(readPref, remoteCommandTargeter);
 
     auto runner = detail::AsyncRPCRunner::get(opCtx->getServiceContext());
-    auto cmdBSON = options->cmd.toBSON({});
-    return detail::sendCommandWithRunner(cmdBSON, options, runner, opCtx, std::move(targeter));
+    createOperationKeyIfNeeded(options->genericArgs);
+    auto genericArgs = options->genericArgs.toBSON();
+    auto cmdBSON = options->cmd.toBSON(genericArgs);
+    return detail::sendCommandWithRunner(options, opCtx, runner, std::move(targeter), cmdBSON);
 }
 
 /**
@@ -400,4 +435,3 @@ ExecutorFuture<AsyncRPCResponse<typename CommandType::Reply>> sendCommand(
 }
 
 }  // namespace mongo::async_rpc
-#undef MONGO_LOGV2_DEFAULT_COMPONENT
