@@ -30,23 +30,39 @@
 
 #include "mongo/executor/connection_pool.h"
 
-#include "mongo/db/service_context.h"
+#include <absl/container/node_hash_map.h>
+#include <absl/meta/type_traits.h>
+#include <algorithm>
+#include <boost/smart_ptr.hpp>
+#include <boost/tuple/tuple.hpp>
 #include <fmt/format.h>
-#include <fmt/ostream.h>
+#include <iterator>
+#include <list>
 #include <memory>
+#include <queue>
+#include <system_error>
+#include <type_traits>
 
-#include "mongo/bson/bsonobjbuilder.h"
-#include "mongo/db/server_feature_flags_gen.h"
+#include <boost/move/utility_core.hpp>
+
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/db/service_context.h"
 #include "mongo/executor/connection_pool_stats.h"
-#include "mongo/executor/remote_command_request.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/log_severity.h"
 #include "mongo/logv2/log_severity_suppressor.h"
+#include "mongo/logv2/redaction.h"
+#include "mongo/platform/atomic_word.h"
 #include "mongo/util/assert_util.h"
 #include "mongo/util/debug_util.h"
 #include "mongo/util/destructor_guard.h"
+#include "mongo/util/fail_point.h"
 #include "mongo/util/hierarchical_acquisition.h"
 #include "mongo/util/lru_cache.h"
-#include "mongo/util/scopeguard.h"
+#include "mongo/util/str.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kConnectionPool
 
@@ -66,6 +82,9 @@ namespace {
 
 MONGO_FAIL_POINT_DEFINE(refreshConnectionAfterEveryCommand);
 MONGO_FAIL_POINT_DEFINE(forceExecutorConnectionPoolTimeout);
+MONGO_FAIL_POINT_DEFINE(connectionPoolReturnsErrorOnGet);
+MONGO_FAIL_POINT_DEFINE(connectionPoolDropConnectionsBeforeGetConnection);
+MONGO_FAIL_POINT_DEFINE(connectionPoolDoesNotFulfillRequests);
 
 auto makeSeveritySuppressor() {
     return std::make_unique<logv2::KeyedSeveritySuppressor<HostAndPort>>(
@@ -98,7 +117,7 @@ void ConnectionPool::ConnectionInterface::indicateUsed() {
     // It is illegal to attempt to use a connection after calling indicateFailure().
     invariant(_status.isOK() || _status == ConnectionPool::kConnectionStateUnknown);
     _lastUsed = now();
-    _timesUsed.fetchAndAddRelaxed(1);
+    _timesUsed++;
 }
 
 void ConnectionPool::ConnectionInterface::indicateSuccess() {
@@ -114,7 +133,7 @@ Date_t ConnectionPool::ConnectionInterface::getLastUsed() const {
 }
 
 size_t ConnectionPool::ConnectionInterface::getTimesUsed() const {
-    return _timesUsed.loadRelaxed();
+    return _timesUsed;
 }
 
 const Status& ConnectionPool::ConnectionInterface::getStatus() const {
@@ -134,7 +153,6 @@ void ConnectionPool::ControllerInterface::init(ConnectionPool* pool) {
 
     LOGV2_DEBUG(22558,
                 2,
-                "Initializing connection pool controller for {pool} as a {controller}",
                 "Initializing connection pool controller",
                 "pool"_attr = pool->_name,
                 "controller"_attr = name());
@@ -284,9 +302,7 @@ public:
      * Gets a connection from the specific pool. Sinks a unique_lock from the
      * parent to preserve the lock on _mutex
      */
-    Future<ConnectionHandle> getConnection(Milliseconds timeout,
-                                           bool lease,
-                                           ErrorCodes::Error timeoutCode);
+    Future<ConnectionHandle> getConnection(Milliseconds timeout, bool lease);
 
     /**
      * Triggers the shutdown procedure. This function sets isShutdown to true
@@ -384,18 +400,14 @@ public:
     }
 
     /**
-     * Return true if the tags on the specific pool match the passed in tags
+     * Return true if the specific pool should be kept open.
      */
-    bool matchesTags(transport::Session::TagMask tags) const {
-        return !!(_tags & tags);
+    bool isKeepOpen(WithLock) const {
+        return _keepOpen;
     }
 
-    /**
-     * Atomically manipulate the tags in the pool
-     */
-    void mutateTags(
-        const std::function<transport::Session::TagMask(transport::Session::TagMask)>& mutateFunc) {
-        _tags = mutateFunc(_tags);
+    void setKeepOpen(WithLock, bool keepOpen) {
+        _keepOpen = keepOpen;
     }
 
     void fassertSSLModeIs(transport::ConnectSSLMode desired) const {
@@ -413,7 +425,6 @@ private:
         Promise<ConnectionHandle> promise;
         // Whether or not the requested connection should be "leased".
         bool lease;
-        ErrorCodes::Error timeoutCode;
     };
 
     struct RequestComparator {
@@ -525,7 +536,8 @@ private:
 
     ConnectionWaitTimeHistogram _connAcquisitionWaitTimeStats{};
 
-    transport::Session::TagMask _tags = transport::Session::kPending;
+    // Indicates connections associated with this HostAndPort should be kept open.
+    bool _keepOpen = true;
 
     HostHealth _health;
 };
@@ -556,7 +568,7 @@ ConnectionPool::ConnectionPool(std::shared_ptr<DependentTypeFactoryInterface> im
       _factory(std::move(impl)),
       _options(std::move(options)),
       _controller(_options.controllerFactory()),
-      _manager(_options.egressTagCloserManager) {
+      _manager(_options.egressConnectionCloserManager) {
     if (_manager) {
         _manager->add(this);
     }
@@ -604,7 +616,7 @@ void ConnectionPool::dropConnections(const HostAndPort& hostAndPort) {
         Status(ErrorCodes::PooledConnectionsDropped, "Pooled connections dropped"));
 }
 
-void ConnectionPool::dropConnections(transport::Session::TagMask tags) {
+void ConnectionPool::dropConnections() {
     stdx::lock_guard lk(_mutex);
 
     // SpecificPool::triggerShutdown can cause iterator invalidation (e.g. `pool` removing itself
@@ -614,7 +626,7 @@ void ConnectionPool::dropConnections(transport::Session::TagMask tags) {
         auto& pool = it->second;
         ++it;
 
-        if (pool->matchesTags(tags))
+        if (pool->isKeepOpen(lk))
             continue;
 
         pool->triggerShutdown(
@@ -622,9 +634,7 @@ void ConnectionPool::dropConnections(transport::Session::TagMask tags) {
     }
 }
 
-void ConnectionPool::mutateTags(
-    const HostAndPort& hostAndPort,
-    const std::function<transport::Session::TagMask(transport::Session::TagMask)>& mutateFunc) {
+void ConnectionPool::setKeepOpen(const HostAndPort& hostAndPort, bool keepOpen) {
     stdx::lock_guard lk(_mutex);
 
     auto iter = _pools.find(hostAndPort);
@@ -633,7 +643,7 @@ void ConnectionPool::mutateTags(
         return;
 
     auto pool = iter->second;
-    pool->mutateTags(mutateFunc);
+    pool->setKeepOpen(lk, keepOpen);
 }
 
 void ConnectionPool::retrieve_forTest(RetrieveConnection retrieve, GetConnectionCallback cb) {
@@ -647,20 +657,18 @@ void ConnectionPool::retrieve_forTest(RetrieveConnection retrieve, GetConnection
 
 void ConnectionPool::get_forTest(const HostAndPort& hostAndPort,
                                  Milliseconds timeout,
-                                 ErrorCodes::Error timeoutCode,
                                  GetConnectionCallback cb) {
-    auto getConnectionFunc = [this, hostAndPort, timeout, timeoutCode]() mutable {
-        return get(hostAndPort, transport::kGlobalSSLMode, timeout, timeoutCode);
+    auto getConnectionFunc = [this, hostAndPort, timeout]() mutable {
+        return get(hostAndPort, transport::kGlobalSSLMode, timeout);
     };
     retrieve_forTest(getConnectionFunc, std::move(cb));
 }
 
 void ConnectionPool::lease_forTest(const HostAndPort& hostAndPort,
                                    Milliseconds timeout,
-                                   ErrorCodes::Error timeoutCode,
                                    GetConnectionCallback cb) {
-    auto getConnectionFunc = [this, hostAndPort, timeout, timeoutCode]() mutable {
-        return lease(hostAndPort, transport::kGlobalSSLMode, timeout, timeoutCode);
+    auto getConnectionFunc = [this, hostAndPort, timeout]() mutable {
+        return lease(hostAndPort, transport::kGlobalSSLMode, timeout);
     };
     retrieve_forTest(getConnectionFunc, std::move(cb));
 }
@@ -668,8 +676,7 @@ void ConnectionPool::lease_forTest(const HostAndPort& hostAndPort,
 SemiFuture<ConnectionPool::ConnectionHandle> ConnectionPool::_get(const HostAndPort& hostAndPort,
                                                                   transport::ConnectSSLMode sslMode,
                                                                   Milliseconds timeout,
-                                                                  bool lease,
-                                                                  ErrorCodes::Error timeoutCode) {
+                                                                  bool lease) {
     auto connRequestedAt = _factory->now();
 
     stdx::lock_guard lk(_mutex);
@@ -683,13 +690,29 @@ SemiFuture<ConnectionPool::ConnectionHandle> ConnectionPool::_get(const HostAndP
 
     invariant(pool);
 
-    auto connFuture = pool->getConnection(timeout, lease, timeoutCode);
+    if (MONGO_unlikely(connectionPoolDropConnectionsBeforeGetConnection.shouldFail())) {
+        if (auto sfp = connectionPoolDropConnectionsBeforeGetConnection.scoped();
+            MONGO_unlikely(sfp.isActive())) {
+            std::string nameToTimeout = sfp.getData()["instance"].String();
+            if (_name.substr(0, nameToTimeout.size()) == nameToTimeout) {
+                // Drop all connections so new connections can be set up via getConnection.
+                pool->processFailure(Status(ErrorCodes::HostUnreachable,
+                                            "Test dropping connections before initial handshake"));
+            }
+        }
+    }
+
+    // In case of no timeout, set timeout to the refresh timeout.
+    if (timeout < Milliseconds(0)) {
+        timeout = _controller->pendingTimeout();
+    }
+
+    auto connFuture = pool->getConnection(timeout, lease);
     pool->updateState();
 
     // Only count connections being checked-out for ordinary use, not lease, towards cumulative wait
     // time.
-    // (Ignore FCV check): This feature flag doesn't have any upgrade/downgrade concerns.
-    if (gFeatureFlagConnHealthMetrics.isEnabledAndIgnoreFCVUnsafe() && !lease) {
+    if (!lease) {
         connFuture = std::move(connFuture).tap([connRequestedAt, pool = pool](const auto& conn) {
             pool->recordConnectionWaitTime(connRequestedAt);
         });
@@ -716,10 +739,7 @@ void ConnectionPool::appendConnectionStats(ConnectionPoolStats* stats) const {
                                      pool->getOnceUsedConnections(),
                                      pool->getTotalConnUsageTime()};
 
-        // (Ignore FCV check): This feature flag doesn't have any upgrade/downgrade concerns.
-        if (gFeatureFlagConnHealthMetrics.isEnabledAndIgnoreFCVUnsafe()) {
-            hostStats.acquisitionWaitTimes = pool->connectionWaitTimeStats();
-        }
+        hostStats.acquisitionWaitTimes = pool->connectionWaitTimeStats();
         stats->updateStatsForHost(_name, host, hostStats);
     }
 }
@@ -801,36 +821,34 @@ size_t ConnectionPool::SpecificPool::requestsPending() const {
 }
 
 Future<ConnectionPool::ConnectionHandle> ConnectionPool::SpecificPool::getConnection(
-    Milliseconds timeout, bool lease, ErrorCodes::Error timeoutCode) {
+    Milliseconds timeout, bool lease) {
+    if (MONGO_unlikely(connectionPoolReturnsErrorOnGet.shouldFail())) {
+        return Future<ConnectionPool::ConnectionHandle>::makeReady(
+            Status(ErrorCodes::SocketException, "test"));
+    }
 
     // Reset our activity timestamp
     auto now = _parent->_factory->now();
     _lastActiveTime = now;
 
-    auto pendingTimeout = _parent->_controller->pendingTimeout();
-    if (timeout < Milliseconds(0) || timeout > pendingTimeout) {
-        timeout = pendingTimeout;
-        // If controller's pending timeout is closest, timeoutCode is rewritten to the internal time
-        // limit error
-        timeoutCode = ErrorCodes::NetworkInterfaceExceededTimeLimit;
-    }
-
     if (auto sfp = forceExecutorConnectionPoolTimeout.scoped(); MONGO_unlikely(sfp.isActive())) {
-        if (const Milliseconds failpointTimeout{sfp.getData()["timeout"].numberInt()};
-            failpointTimeout > Milliseconds{0}) {
-            auto pf = makePromiseFuture<ConnectionHandle>();
-            auto request = std::make_shared<Request>();
-            request->expiration = now + failpointTimeout;
-            request->promise = std::move(pf.promise);
-            request->timeoutCode = timeoutCode;
-            auto timeoutTimer = _parent->_factory->makeTimer();
-            timeoutTimer->setTimeout(failpointTimeout, [request, timeoutTimer]() mutable {
-                request->promise.setError(Status(
-                    request->timeoutCode,
-                    "Connection timed out due to forceExecutorConnectionPoolTimeout failpoint"));
-            });
-            return std::move(pf.future);
+        const Milliseconds failpointTimeout{sfp.getData()["timeout"].numberInt()};
+        const Status failpointStatus{
+            ErrorCodes::PooledConnectionAcquisitionExceededTimeLimit,
+            "Connection timed out due to forceExecutorConnectionPoolTimeout failpoint"};
+        if (failpointTimeout == Milliseconds(0)) {
+            return Future<ConnectionPool::ConnectionHandle>::makeReady(failpointStatus);
         }
+        auto pf = makePromiseFuture<ConnectionHandle>();
+        auto request = std::make_shared<Request>();
+        request->expiration = now + failpointTimeout;
+        request->promise = std::move(pf.promise);
+        auto timeoutTimer = _parent->_factory->makeTimer();
+        timeoutTimer->setTimeout(failpointTimeout,
+                                 [request, timeoutTimer, failpointStatus]() mutable {
+                                     request->promise.setError(failpointStatus);
+                                 });
+        return std::move(pf.future);
     }
 
     // If we do not have requests, then we can fulfill immediately
@@ -840,7 +858,6 @@ Future<ConnectionPool::ConnectionHandle> ConnectionPool::SpecificPool::getConnec
         if (conn) {
             LOGV2_DEBUG(22559,
                         kDiagnosticLogLevel,
-                        "Using existing idle connection to {hostAndPort}",
                         "Using existing idle connection",
                         "hostAndPort"_attr = _hostAndPort);
             return Future<ConnectionPool::ConnectionHandle>::makeReady(std::move(conn));
@@ -849,7 +866,6 @@ Future<ConnectionPool::ConnectionHandle> ConnectionPool::SpecificPool::getConnec
 
     LOGV2_DEBUG(22560,
                 kDiagnosticLogLevel,
-                "Requesting new connection to {hostAndPort} with timeout {timeout}",
                 "Requesting new connection",
                 "hostAndPort"_attr = _hostAndPort,
                 "timeout"_attr = timeout);
@@ -857,7 +873,7 @@ Future<ConnectionPool::ConnectionHandle> ConnectionPool::SpecificPool::getConnec
     const auto expiration = now + timeout;
     auto pf = makePromiseFuture<ConnectionHandle>();
 
-    _requests.push_back({expiration, std::move(pf.promise), lease, timeoutCode});
+    _requests.push_back({expiration, std::move(pf.promise), lease});
     std::push_heap(begin(_requests), end(_requests), RequestComparator{});
 
     return std::move(pf.future);
@@ -894,7 +910,6 @@ ConnectionPool::ConnectionHandle ConnectionPool::SpecificPool::tryGetConnection(
 
         if (!conn->maybeHealthy()) {
             LOGV2(22561,
-                  "Dropping unhealthy pooled connection to {hostAndPort}",
                   "Dropping unhealthy pooled connection",
                   "hostAndPort"_attr = conn->getHostAndPort());
 
@@ -928,23 +943,6 @@ void ConnectionPool::SpecificPool::finishRefresh(ConnectionInterface* connPtr, S
         return;
     }
 
-    // If we've exceeded the time limit, start a new connect,
-    // rather than failing all operations.  We do this because the
-    // various callers have their own time limit which is unrelated
-    // to our internal one.
-    if (status.code() == ErrorCodes::NetworkInterfaceExceededTimeLimit) {
-        LOGV2_DEBUG(22562,
-                    kDiagnosticLogLevel,
-                    "Pending connection to host {hostAndPort} did not complete within the timeout, "
-                    "retrying with a new connection; "
-                    "{numOpenConns} connections to that host remain open",
-                    "Pending connection did not complete within the timeout, "
-                    "retrying with a new connection",
-                    "hostAndPort"_attr = _hostAndPort,
-                    "numOpenConns"_attr = openConnections());
-        return;
-    }
-
     // If the error can be contained to one connection, drop the one connection.
     if (status.code() == ErrorCodes::ConnectionError) {
         LOGV2_DEBUG(6832901,
@@ -960,7 +958,6 @@ void ConnectionPool::SpecificPool::finishRefresh(ConnectionInterface* connPtr, S
     if (!conn || conn->getGeneration() != _generation) {
         LOGV2_DEBUG(22564,
                     kDiagnosticLogLevel,
-                    "Dropping late refreshed connection to {hostAndPort}",
                     "Dropping late refreshed connection",
                     "hostAndPort"_attr = _hostAndPort);
         return;
@@ -970,7 +967,6 @@ void ConnectionPool::SpecificPool::finishRefresh(ConnectionInterface* connPtr, S
     if (!status.isOK()) {
         LOGV2_DEBUG(22563,
                     kDiagnosticLogLevel,
-                    "Connection failed to {hostAndPort} due to {error}",
                     "Connection failed",
                     "hostAndPort"_attr = _hostAndPort,
                     "error"_attr = redact(status));
@@ -980,7 +976,6 @@ void ConnectionPool::SpecificPool::finishRefresh(ConnectionInterface* connPtr, S
 
     LOGV2_DEBUG(22565,
                 kDiagnosticLogLevel,
-                "Finishing connection refresh for {hostAndPort}",
                 "Finishing connection refresh",
                 "hostAndPort"_attr = _hostAndPort);
 
@@ -1022,7 +1017,6 @@ void ConnectionPool::SpecificPool::returnConnection(ConnectionInterface* connPtr
             !ErrorCodes::isNetworkTimeoutError(status)) {
             LOGV2_DEBUG(7719500,
                         kDiagnosticLogLevel,
-                        "Connection failed to {hostAndPort} due to {error}",
                         "Connection failed",
                         "hostAndPort"_attr = _hostAndPort,
                         "error"_attr = redact(status));
@@ -1031,8 +1025,6 @@ void ConnectionPool::SpecificPool::returnConnection(ConnectionInterface* connPtr
         }
         // Otherwise, drop the one connection.
         LOGV2(22566,
-              "Ending connection to host {hostAndPort} due to bad connection status: {error}; "
-              "{numOpenConns} connections to that host remain open",
               "Ending connection due to bad connection status",
               "hostAndPort"_attr = _hostAndPort,
               "error"_attr = redact(status),
@@ -1053,8 +1045,6 @@ void ConnectionPool::SpecificPool::returnConnection(ConnectionInterface* connPtr
         if (openConnections() >= controls.targetConnections) {
             // If we already have minConnections, just let the connection lapse
             LOGV2(22567,
-                  "Ending idle connection to host {hostAndPort} because the pool meets "
-                  "constraints; {numOpenConns} connections to that host remain open",
                   "Ending idle connection because the pool meets constraints",
                   "hostAndPort"_attr = _hostAndPort,
                   "numOpenConns"_attr = openConnections());
@@ -1063,11 +1053,8 @@ void ConnectionPool::SpecificPool::returnConnection(ConnectionInterface* connPtr
 
         _processingPool[connPtr] = std::move(conn);
 
-        LOGV2_DEBUG(22568,
-                    kDiagnosticLogLevel,
-                    "Refreshing connection to {hostAndPort}",
-                    "Refreshing connection",
-                    "hostAndPort"_attr = _hostAndPort);
+        LOGV2_DEBUG(
+            22568, kDiagnosticLogLevel, "Refreshing connection", "hostAndPort"_attr = _hostAndPort);
         connPtr->refresh(_parent->_controller->pendingTimeout(),
                          guardCallback([this](auto conn, auto status) {
                              finishRefresh(std::move(conn), std::move(status));
@@ -1079,7 +1066,6 @@ void ConnectionPool::SpecificPool::returnConnection(ConnectionInterface* connPtr
     // If it's fine as it is, just put it in the ready queue
     LOGV2_DEBUG(22569,
                 kDiagnosticLogLevel,
-                "Returning ready connection to {hostAndPort}",
                 "Returning ready connection",
                 "hostAndPort"_attr = _hostAndPort);
     addToReady(std::move(conn));
@@ -1100,7 +1086,6 @@ void ConnectionPool::SpecificPool::addToReady(OwnedConnection conn) {
     auto returnConnectionFunc = guardCallback([this, connPtr]() {
         LOGV2_DEBUG(22570,
                     kDiagnosticLogLevel,
-                    "Triggered refresh timeout for {hostAndPort}",
                     "Triggered refresh timeout",
                     "hostAndPort"_attr = _hostAndPort);
         auto conn = takeFromPool(_readyPool, connPtr);
@@ -1129,11 +1114,7 @@ void ConnectionPool::SpecificPool::triggerShutdown(const Status& status) {
         return;
     }
 
-    LOGV2_DEBUG(22571,
-                2,
-                "Delisting connection pool for {hostAndPort}",
-                "Delistinng connection pool",
-                "hostAndPort"_attr = _hostAndPort);
+    LOGV2_DEBUG(22571, 2, "Delistinng connection pool", "hostAndPort"_attr = _hostAndPort);
 
     // Make sure the pool lifetime lasts until the end of this function,
     // it could be only in the map of pools
@@ -1156,7 +1137,6 @@ void ConnectionPool::SpecificPool::processFailure(const Status& status) {
         static auto& bumpedSeverity = *makeSeveritySuppressor().release();
         LOGV2_DEBUG(22572,
                     bumpedSeverity(_hostAndPort).toInt(),
-                    "Dropping all pooled connections to {hostAndPort} due to {error}",
                     "Dropping all pooled connections",
                     "hostAndPort"_attr = _hostAndPort,
                     "error"_attr = redact(status));
@@ -1187,16 +1167,19 @@ void ConnectionPool::SpecificPool::processFailure(const Status& status) {
         request.promise.setError(status);
     }
 
-    LOGV2_DEBUG(22573,
-                kDiagnosticLogLevel,
-                "Failing requests to {hostAndPort}",
-                "Failing requests",
-                "hostAndPort"_attr = _hostAndPort);
+    LOGV2_DEBUG(22573, kDiagnosticLogLevel, "Failing requests", "hostAndPort"_attr = _hostAndPort);
     _requests.clear();
 }
 
 // fulfills as many outstanding requests as possible
 void ConnectionPool::SpecificPool::fulfillRequests() {
+    if (auto sfp = connectionPoolDoesNotFulfillRequests.scoped(); MONGO_unlikely(sfp.isActive())) {
+        std::string nameToTimeout = sfp.getData()["instance"].String();
+        if (_parent->_name.substr(0, nameToTimeout.size()) == nameToTimeout) {
+            return;
+        }
+    }
+
     while (_requests.size()) {
         // Marking this as our newest active time
         _lastActiveTime = _parent->_factory->now();
@@ -1231,8 +1214,6 @@ void ConnectionPool::SpecificPool::spawnConnections() {
     if (_health.isFailed) {
         LOGV2_DEBUG(22574,
                     kDiagnosticLogLevel,
-                    "Pool for {hostAndPort} has failed recently, "
-                    "postponing any attempts to spawn connections",
                     "Pool has failed recently, postponing any attempts to spawn connections",
                     "hostAndPort"_attr = _hostAndPort);
         return;
@@ -1241,7 +1222,6 @@ void ConnectionPool::SpecificPool::spawnConnections() {
     auto controls = _parent->_controller->getControls(_id);
     LOGV2_DEBUG(22575,
                 kDiagnosticLogLevel,
-                "Comparing connection state for {hostAndPort} to controls: {poolControls}",
                 "Comparing connection state to controls",
                 "hostAndPort"_attr = _hostAndPort,
                 "poolControls"_attr = controls);
@@ -1259,7 +1239,6 @@ void ConnectionPool::SpecificPool::spawnConnections() {
     static auto& bumpedSeverity = *makeSeveritySuppressor().release();
     LOGV2_DEBUG(22576,
                 bumpedSeverity(_hostAndPort).toInt(),
-                "Connecting to {hostAndPort}",
                 "Connecting",
                 "hostAndPort"_attr = _hostAndPort);
 
@@ -1267,7 +1246,6 @@ void ConnectionPool::SpecificPool::spawnConnections() {
                               controls.maxPendingConnections - pendingConnections);
     LOGV2_DEBUG(22577,
                 kDiagnosticLogLevel,
-                "Spawning {connAllowance} connections to {hostAndPort}",
                 "Spawning connections",
                 "connAllowance"_attr = allowance,
                 "hostAndPort"_attr = _hostAndPort);
@@ -1383,7 +1361,8 @@ void ConnectionPool::SpecificPool::updateEventTimer() {
 
             auto& request = _requests.back();
             request.promise.setError(
-                Status(request.timeoutCode, "Couldn't get a connection within the time limit"));
+                Status(ErrorCodes::PooledConnectionAcquisitionExceededTimeLimit,
+                       "Couldn't get a connection within the time limit"));
             _requests.pop_back();
 
             // Since we've failed a request, we've interacted with external users
@@ -1411,7 +1390,6 @@ void ConnectionPool::SpecificPool::updateController() {
     };
     LOGV2_DEBUG(22578,
                 kDiagnosticLogLevel,
-                "Updating pool controller for {hostAndPort} with state: {poolState}",
                 "Updating pool controller",
                 "hostAndPort"_attr = _hostAndPort,
                 "poolState"_attr = state);
@@ -1470,11 +1448,7 @@ void ConnectionPool::SpecificPool::updateController() {
 void ConnectionPool::SpecificPool::updateState() {
     if (_health.isShutdown) {
         // If we're in shutdown, there is nothing to update. Our clients are all gone.
-        LOGV2_DEBUG(22579,
-                    kDiagnosticLogLevel,
-                    "Pool {hostAndPort} is dead",
-                    "Pool is dead",
-                    "hostAndPort"_attr = _hostAndPort);
+        LOGV2_DEBUG(22579, kDiagnosticLogLevel, "Pool is dead", "hostAndPort"_attr = _hostAndPort);
         return;
     }
 
