@@ -29,20 +29,34 @@
 
 #pragma once
 
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <cstddef>
+#include <cstdint>
 #include <functional>
+#include <limits>
 #include <memory>
 #include <mutex>
 #include <queue>
+#include <string>
+#include <utility>
+#include <vector>
 
-#include "mongo/config.h"
-#include "mongo/executor/egress_tag_closer.h"
-#include "mongo/executor/egress_tag_closer_manager.h"
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/config.h"  // IWYU pragma: keep
+#include "mongo/executor/egress_connection_closer.h"
+#include "mongo/executor/egress_connection_closer_manager.h"
+#include "mongo/platform/compiler.h"
 #include "mongo/platform/mutex.h"
 #include "mongo/stdx/unordered_map.h"
 #include "mongo/transport/session.h"
 #include "mongo/transport/transport_layer.h"
 #include "mongo/util/clock_source.h"
 #include "mongo/util/duration.h"
+#include "mongo/util/functional.h"
 #include "mongo/util/future.h"
 #include "mongo/util/hierarchical_acquisition.h"
 #include "mongo/util/net/hostandport.h"
@@ -67,7 +81,8 @@ struct ConnectionPoolStats;
  * The overall workflow here is to manage separate pools for each unique
  * HostAndPort. See comments on the various Options for how the pool operates.
  */
-class ConnectionPool : public EgressTagCloser, public std::enable_shared_from_this<ConnectionPool> {
+class ConnectionPool : public EgressConnectionCloser,
+                       public std::enable_shared_from_this<ConnectionPool> {
     class LimitController;
 
 public:
@@ -148,7 +163,7 @@ public:
          *
          * The manager will hold this pool for the lifetime of the pool.
          */
-        EgressTagCloserManager* egressTagCloserManager = nullptr;
+        EgressConnectionCloserManager* egressConnectionCloserManager = nullptr;
 
         /**
          * Connections created through this connection pool will not attempt to authenticate.
@@ -244,29 +259,32 @@ public:
                             std::string name,
                             Options options = Options{});
 
-    ~ConnectionPool();
+    ~ConnectionPool() override;
 
     void shutdown();
 
     void dropConnections(const HostAndPort& hostAndPort) override;
 
-    void dropConnections(transport::Session::TagMask tags) override;
+    /**
+     * Drops all connections, but if a certain SpecificPool (and therefore HostAndPort) is
+     * marked as keep open, that connection will not be dropped.
+     */
+    void dropConnections() override;
 
-    void mutateTags(const HostAndPort& hostAndPort,
-                    const std::function<transport::Session::TagMask(transport::Session::TagMask)>&
-                        mutateFunc) override;
+    /**
+     * Marks SpecificPool to be kept open for dropConnections(), must acquire connection pool
+     * mutex.
+     */
+    void setKeepOpen(const HostAndPort& hostAndPort, bool keepOpen) override;
 
-    inline SemiFuture<ConnectionHandle> get(
-        const HostAndPort& hostAndPort,
-        transport::ConnectSSLMode sslMode,
-        Milliseconds timeout,
-        ErrorCodes::Error timeoutCode = ErrorCodes::NetworkInterfaceExceededTimeLimit) {
-        return _get(hostAndPort, sslMode, timeout, false /*lease*/, timeoutCode);
+    inline SemiFuture<ConnectionHandle> get(const HostAndPort& hostAndPort,
+                                            transport::ConnectSSLMode sslMode,
+                                            Milliseconds timeout) {
+        return _get(hostAndPort, sslMode, timeout, false /*lease*/);
     }
 
     void get_forTest(const HostAndPort& hostAndPort,
                      Milliseconds timeout,
-                     ErrorCodes::Error timeoutCode,
                      GetConnectionCallback cb);
 
     /**
@@ -276,17 +294,14 @@ public:
      * their lease and are reported separately in metrics. Otherwise, this method behaves similarly
      * to `ConnectionPool::get`.
      */
-    inline SemiFuture<ConnectionHandle> lease(
-        const HostAndPort& hostAndPort,
-        transport::ConnectSSLMode sslMode,
-        Milliseconds timeout,
-        ErrorCodes::Error timeoutCode = ErrorCodes::NetworkInterfaceExceededTimeLimit) {
-        return _get(hostAndPort, sslMode, timeout, true /*lease*/, timeoutCode);
+    inline SemiFuture<ConnectionHandle> lease(const HostAndPort& hostAndPort,
+                                              transport::ConnectSSLMode sslMode,
+                                              Milliseconds timeout) {
+        return _get(hostAndPort, sslMode, timeout, true /*lease*/);
     }
 
     void lease_forTest(const HostAndPort& hostAndPort,
                        Milliseconds timeout,
-                       ErrorCodes::Error timeoutCode,
                        GetConnectionCallback cb);
 
     void appendConnectionStats(ConnectionPoolStats* stats) const;
@@ -300,12 +315,10 @@ public:
 private:
     ClockSource* _getFastClockSource() const;
 
-    SemiFuture<ConnectionHandle> _get(
-        const HostAndPort& hostAndPort,
-        transport::ConnectSSLMode sslMode,
-        Milliseconds timeout,
-        bool leased,
-        ErrorCodes::Error timeoutCode = ErrorCodes::NetworkInterfaceExceededTimeLimit);
+    SemiFuture<ConnectionHandle> _get(const HostAndPort& hostAndPort,
+                                      transport::ConnectSSLMode sslMode,
+                                      Milliseconds timeout,
+                                      bool leased);
 
     void retrieve_forTest(RetrieveConnection retrieve, GetConnectionCallback cb);
 
@@ -322,7 +335,7 @@ private:
     PoolId _nextPoolId = 0;
     stdx::unordered_map<HostAndPort, std::shared_ptr<SpecificPool>> _pools;
 
-    EgressTagCloserManager* _manager;
+    EgressConnectionCloserManager* _manager;
 
     mutable ClockSource* _fastClockSource{nullptr};
     mutable std::once_flag _fastClkSrcInitFlag;
@@ -377,7 +390,7 @@ class ConnectionPool::ConnectionInterface : public TimerInterface {
 public:
     explicit ConnectionInterface(size_t generation) : _generation(generation) {}
 
-    virtual ~ConnectionInterface() = default;
+    ~ConnectionInterface() override = default;
 
     /**
      * Indicates that the user is now done with this connection. Users MUST call either
@@ -478,7 +491,7 @@ protected:
 private:
     size_t _generation;
     Date_t _lastUsed;
-    AtomicWord<size_t> _timesUsed{0};
+    size_t _timesUsed = 0;
     Status _status = ConnectionPool::kConnectionStateUnknown;
 };
 
