@@ -27,32 +27,40 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/executor/connection_pool_test_fixture.h"
-
-#include "mongo/util/duration.h"
-#include "mongo/util/net/hostandport.h"
 #include <algorithm>
+#include <array>
+#include <boost/smart_ptr.hpp>
 #include <memory>
 #include <random>
+#include <ratio>
+#include <set>
 #include <stack>
 #include <tuple>
+#include <type_traits>
 
-#include <fmt/format.h>
-#include <fmt/ostream.h>
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
 
 #include "mongo/executor/connection_pool.h"
 #include "mongo/executor/connection_pool_stats.h"
-#include "mongo/stdx/future.h"
+#include "mongo/executor/connection_pool_test_fixture.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/unittest/matcher.h"
+#include "mongo/unittest/matcher_core.h"
 #include "mongo/unittest/thread_assertion_monitor.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/duration.h"
+#include "mongo/util/executor_test_util.h"
+#include "mongo/util/net/hostandport.h"
 #include "mongo/util/scopeguard.h"
 
 namespace mongo {
 namespace executor {
 namespace connection_pool_test_details {
 
+namespace m = unittest::match;
+
 class ConnectionPoolTest : public unittest::Test {
 public:
     constexpr static Milliseconds kNoTimeout = Milliseconds{-1};
@@ -116,6 +124,39 @@ protected:
     template <typename Ptr>
     void dropConnectionsTest(std::shared_ptr<ConnectionPool> const& pool, Ptr t);
 
+    /**
+     * Helper for asserting connection pool time-out behaviours.
+     *
+     * Gets a connection from a new pool with a timeout duration,
+     * asserting the connection times out with the appropriate expected code
+     * associated with the matcher.
+     *
+     * The controller's refresh timeout is set to 250ms.
+     */
+    void assertTimeoutHelper(Milliseconds acquisitionTimeout, ErrorCodes::Error errorCode) {
+        ConnectionPool::Options options;
+        options.refreshTimeout = Milliseconds{250};
+        auto pool = makePool(options);
+        auto now = Date_t::now();
+
+        PoolImpl::setNow(now);
+
+        StatusWith<ConnectionPool::ConnectionHandle> connectionHandle{nullptr};
+        pool->get_forTest(HostAndPort(),
+                          acquisitionTimeout,
+                          [&](StatusWith<ConnectionPool::ConnectionHandle> swConn) {
+                              connectionHandle = std::move(swConn);
+                          });
+
+        // Ensure only one timeout fires.
+        auto minTimeout = std::min(options.refreshTimeout, acquisitionTimeout);
+
+        PoolImpl::setNow(now + minTimeout);
+
+        ASSERT(!connectionHandle.isOK());
+        ASSERT_EQ(connectionHandle.getStatus().code(), errorCode);
+    }
+
 private:
     std::shared_ptr<OutOfLineExecutor> _executor = InlineQueuedCountingExecutor::make();
     std::shared_ptr<ConnectionPool> _pool;
@@ -451,6 +492,7 @@ TEST_F(ConnectionPoolTest, FailedHostDropsConns) {
             });
         };
         auto timeout = Milliseconds(5000);
+
         pool->get_forTest(HostAndPort(), timeout, cb);
     }
 
@@ -521,6 +563,7 @@ TEST_F(ConnectionPoolTest, OtherErrorsDontDropConns) {
                 });
             };
             auto timeout = Milliseconds(5000);
+
             pool->get_forTest(HostAndPort(), timeout, cb);
         }
 
@@ -632,30 +675,23 @@ TEST_F(ConnectionPoolTest, DifferentConnWithoutReturn) {
 }
 
 /**
- * Verify that timing out on setup works as expected (a bad status is
- * returned).
- *
- * Note that the lack of pushSetup() calls delays the get.
+ * When the timeout duration comes from the parameter, the connection timeout status should
+ * always be `PooledConnectionAcquisitionExceededTimeLimit`.
  */
-TEST_F(ConnectionPoolTest, TimeoutOnSetup) {
-    auto pool = makePool();
+TEST_F(ConnectionPoolTest, TimeoutOnAquisitionTimeout) {
+    assertTimeoutHelper(
+        /* timeout duration */ Milliseconds{100},
+        /* expected timeout codes */ ErrorCodes::PooledConnectionAcquisitionExceededTimeLimit);
+}
 
-    auto now = Date_t::now();
-
-    Milliseconds hostTimeout = Milliseconds(5000);
-
-    PoolImpl::setNow(now);
-
-    boost::optional<StatusWith<ConnectionPool::ConnectionHandle>> conn;
-    pool->get_forTest(
-        HostAndPort(), hostTimeout, [&](StatusWith<ConnectionPool::ConnectionHandle> swConn) {
-            conn = std::move(swConn);
-        });
-
-    PoolImpl::setNow(now + hostTimeout);
-
-    ASSERT(!conn->isOK());
-    ASSERT_EQ(conn->getStatus(), ErrorCodes::NetworkInterfaceExceededTimeLimit);
+/**
+ * When the timeout duration comes from controller refresh, the connection timeout status should
+ * always be `HostUnreachable`.
+ */
+TEST_F(ConnectionPoolTest, TimeoutOnControllerRefresh) {
+    assertTimeoutHelper(
+        /* timeout duration */ Milliseconds{500},
+        /* expected timeout codes */ ErrorCodes::HostUnreachable);
 }
 
 /**
@@ -1618,9 +1654,12 @@ TEST_F(ConnectionPoolTest, dropConnections) {
 }
 
 /**
- * Verify that timeouts during setup don't prematurely time out unrelated requests
+ * Verify that setup timeouts time out other pending requests. This is in adherence with
+ * the SDAM specification which states that timeouts during setup should mark the timed
+ * out host as Unknown. Therefore, pending connections may be dropped in this layer as
+ * a reaction to setup timeout.
  */
-TEST_F(ConnectionPoolTest, SetupTimeoutsDontTimeoutUnrelatedRequests) {
+TEST_F(ConnectionPoolTest, SetupTimeoutsFailOtherPendingRequests) {
     ConnectionPool::Options options;
 
     options.maxConnections = 1;
@@ -1636,31 +1675,36 @@ TEST_F(ConnectionPoolTest, SetupTimeoutsDontTimeoutUnrelatedRequests) {
             conn1 = std::move(swConn);
         });
 
-    // initially we haven't called our callback
+    // Initially we haven't called our callback.
     ASSERT(!conn1);
 
     PoolImpl::setNow(now + Seconds(1));
 
-    // Still haven't fired on conn1
+    // Still haven't fired on conn1.
     ASSERT(!conn1);
 
-    // Get conn2 (which should have an extra second before the timeout)
+    boost::optional<StatusWith<ConnectionPool::ConnectionHandle>> conn2;
+    // Get conn2 (which should have an extra second before the timeout).
     pool->get_forTest(
         HostAndPort(), Seconds(10), [&](StatusWith<ConnectionPool::ConnectionHandle> swConn) {
-            ASSERT_EQ(swConn.getStatus(), ErrorCodes::ShutdownInProgress);
+            conn2 = std::move(swConn);
         });
 
     PoolImpl::setNow(now + Seconds(2));
 
     ASSERT(conn1);
     ASSERT(!conn1->isOK());
-    ASSERT_EQ(conn1->getStatus(), ErrorCodes::NetworkInterfaceExceededTimeLimit);
+    ASSERT_EQ(conn1->getStatus(), ErrorCodes::HostUnreachable);
+    ASSERT(conn2);
+    ASSERT(!conn2->isOK());
+    // Pending connection fails with the same timeout status.
+    ASSERT_EQ(conn2->getStatus(), ErrorCodes::HostUnreachable);
 }
 
 /**
- * Verify that timeouts during refresh don't prematurely time out unrelated requests
+ * Verify that timeouts during refresh time out other pending requests.
  */
-TEST_F(ConnectionPoolTest, RefreshTimeoutsDontTimeoutRequests) {
+TEST_F(ConnectionPoolTest, RefreshTimeoutsFailPendingRequests) {
     ConnectionPool::Options options;
 
     options.maxConnections = 1;
@@ -1700,14 +1744,14 @@ TEST_F(ConnectionPoolTest, RefreshTimeoutsDontTimeoutRequests) {
     // Get conn2 (which should have an extra second before the timeout)
     pool->get_forTest(
         HostAndPort(), Seconds(10), [&](StatusWith<ConnectionPool::ConnectionHandle> swConn) {
-            ASSERT_EQ(swConn.getStatus(), ErrorCodes::ShutdownInProgress);
+            ASSERT_EQ(swConn.getStatus(), ErrorCodes::HostUnreachable);
         });
 
     PoolImpl::setNow(now + Seconds(5));
 
     ASSERT(conn1);
     ASSERT(!conn1->isOK());
-    ASSERT_EQ(conn1->getStatus(), ErrorCodes::NetworkInterfaceExceededTimeLimit);
+    ASSERT_EQ(conn1->getStatus(), ErrorCodes::HostUnreachable);
 }
 
 template <typename Ptr>
@@ -1740,34 +1784,24 @@ void ConnectionPoolTest::dropConnectionsTest(std::shared_ptr<ConnectionPool> con
     ASSERT_EQ(1ul, pool->getNumConnectionsPerHost(hap2));
     ASSERT_EQ(1ul, pool->getNumConnectionsPerHost(hap3));
 
-    t->dropConnections(transport::Session::kPending);
+    t->dropConnections();
 
     ASSERT_EQ(1ul, pool->getNumConnectionsPerHost(hap1));
     ASSERT_EQ(1ul, pool->getNumConnectionsPerHost(hap2));
     ASSERT_EQ(1ul, pool->getNumConnectionsPerHost(hap3));
 
-    t->mutateTags(hap1, [](transport::Session::TagMask tags) {
-        return transport::Session::kKeepOpen | transport::Session::kInternalClient;
-    });
+    t->setKeepOpen(hap1, true);
 
-    t->mutateTags(hap2,
-                  [](transport::Session::TagMask tags) { return transport::Session::kKeepOpen; });
+    t->setKeepOpen(hap2, true);
 
-    t->mutateTags(
-        hap3, [](transport::Session::TagMask tags) { return transport::Session::kEmptyTagMask; });
+    t->setKeepOpen(hap3, false);
 
-    t->dropConnections(transport::Session::kKeepOpen);
+    t->dropConnections();
 
     ASSERT_EQ(1ul, pool->getNumConnectionsPerHost(hap1));
     ASSERT_EQ(1ul, pool->getNumConnectionsPerHost(hap2));
     ASSERT_EQ(0ul, pool->getNumConnectionsPerHost(hap3));
 
-    t->dropConnections(transport::Session::kInternalClient);
-
-    ASSERT_EQ(1ul, pool->getNumConnectionsPerHost(hap1));
-    ASSERT_EQ(0ul, pool->getNumConnectionsPerHost(hap2));
-    ASSERT_EQ(0ul, pool->getNumConnectionsPerHost(hap3));
-
     ConnectionImpl::pushSetup(Status::OK());
     pool->get_forTest(hap4, Seconds(1), [&](StatusWith<ConnectionPool::ConnectionHandle> swConn) {
         doneWith(swConn.getValue());
@@ -1777,7 +1811,7 @@ void ConnectionPoolTest::dropConnectionsTest(std::shared_ptr<ConnectionPool> con
     t->dropConnections(hap1);
 
     ASSERT_EQ(0ul, pool->getNumConnectionsPerHost(hap1));
-    ASSERT_EQ(0ul, pool->getNumConnectionsPerHost(hap2));
+    ASSERT_EQ(1ul, pool->getNumConnectionsPerHost(hap2));
     ASSERT_EQ(0ul, pool->getNumConnectionsPerHost(hap3));
     ASSERT_EQ(1ul, pool->getNumConnectionsPerHost(hap4));
 }
@@ -1791,10 +1825,10 @@ TEST_F(ConnectionPoolTest, DropConnections) {
 }
 
 TEST_F(ConnectionPoolTest, DropConnectionsInMultipleViaManager) {
-    EgressTagCloserManager manager;
+    EgressConnectionCloserManager manager;
     ConnectionPool::Options options;
     options.minConnections = 0;
-    options.egressTagCloserManager = &manager;
+    options.egressConnectionCloserManager = &manager;
     auto pool = makePool(options);
 
     dropConnectionsTest(pool, &manager);
@@ -1914,6 +1948,161 @@ TEST_F(ConnectionPoolTest, ReturnAfterShutdown) {
     pool->shutdown();
 }
 
+TEST_F(ConnectionPoolTest, TotalConnUseTimeIncreasedForCheckedOutConnection) {
+    constexpr Milliseconds checkOutLength = Milliseconds(10);
+    auto pool = makePool();
+
+    ConnectionPoolStats initialStats;
+    pool->appendConnectionStats(&initialStats);
+
+    auto startTimePoint = Date_t::now();
+    auto endTimePoint = startTimePoint + checkOutLength;
+    PoolImpl::setNow(startTimePoint);
+
+    ConnectionImpl::pushSetup(Status::OK());
+    pool->get_forTest(HostAndPort(),
+                      Milliseconds(5000),
+                      [&](StatusWith<ConnectionPool::ConnectionHandle> swConn) {
+                          PoolImpl::setNow(endTimePoint);
+                          doneWith(swConn.getValue());
+                      });
+
+    ConnectionPoolStats finalStats;
+    pool->appendConnectionStats(&finalStats);
+
+    auto totalTimeUsageDelta = finalStats.totalConnUsageTime - initialStats.totalConnUsageTime;
+    ASSERT_GREATER_THAN_OR_EQUALS(totalTimeUsageDelta, checkOutLength);
+}
+
+TEST_F(ConnectionPoolTest, OverlappingCheckoutsAdditivelyContributeToTotalUsageTime) {
+    constexpr Milliseconds checkOutLength = Milliseconds(10);
+    auto pool = makePool();
+
+    ConnectionPoolStats initialStats;
+    pool->appendConnectionStats(&initialStats);
+
+    auto startTimePoint = Date_t::now();
+    auto endTimePoint = startTimePoint + checkOutLength;
+    PoolImpl::setNow(startTimePoint);
+
+    // Check out multiple connections.
+    constexpr int numConnections = 2;
+    std::vector<ConnectionPool::ConnectionHandle> connections;
+    // Ensure that no matter how we leave the test, we mark any
+    // checked out connections as OK before implicity returning them
+    // to the pool by destroying the 'connections' vector. Otherwise,
+    // this test would cause an invariant failure instead of a normal
+    // test failure if it fails, which would be confusing.
+    ScopeGuard guard([&] {
+        while (!connections.empty()) {
+            try {
+                ConnectionPool::ConnectionHandle conn = std::move(connections.back());
+                connections.pop_back();
+                doneWith(conn);
+            } catch (...) {
+            }
+        }
+    });
+
+    for (int i = 0; i < numConnections; ++i) {
+        ConnectionImpl::pushSetup(Status::OK());
+        pool->get_forTest(HostAndPort(),
+                          Milliseconds(5000),
+                          [&](StatusWith<ConnectionPool::ConnectionHandle> swConn) {
+                              ASSERT(swConn.isOK());
+                              connections.push_back(std::move(swConn.getValue()));
+                          });
+    }
+    ASSERT_EQ(connections.size(), numConnections);
+    // Advance the time and return the connections.
+    PoolImpl::setNow(endTimePoint);
+    while (!connections.empty()) {
+        try {
+            ConnectionPool::ConnectionHandle conn = std::move(connections.back());
+            ASSERT(conn);
+            connections.pop_back();
+            doneWith(conn);
+        } catch (...) {
+        }
+    }
+    guard.dismiss();
+
+    ConnectionPoolStats finalStats;
+    pool->appendConnectionStats(&finalStats);
+
+    auto totalTimeUsageDelta = finalStats.totalConnUsageTime - initialStats.totalConnUsageTime;
+    // Since each connection was used for checkOutLength, the total usage time should be >= the
+    // product.
+    ASSERT_GREATER_THAN_OR_EQUALS(totalTimeUsageDelta, checkOutLength * numConnections);
+}
+
+TEST_F(ConnectionPoolTest, LeasedConnectionsDontCountTowardsUsageTime) {
+    constexpr Milliseconds checkOutLength = Milliseconds(10);
+    auto pool = makePool();
+
+    ConnectionPoolStats initialStats;
+    pool->appendConnectionStats(&initialStats);
+
+    auto startTimePoint = Date_t::now();
+    auto endTimePoint = startTimePoint + checkOutLength;
+    PoolImpl::setNow(startTimePoint);
+
+    ConnectionImpl::pushSetup(Status::OK());
+    pool->lease_forTest(HostAndPort(),
+                        Milliseconds(5000),
+                        [&](StatusWith<ConnectionPool::ConnectionHandle> swConn) {
+                            PoolImpl::setNow(endTimePoint);
+                            doneWith(swConn.getValue());
+                        });
+
+    ConnectionPoolStats finalStats;
+    pool->appendConnectionStats(&finalStats);
+
+    auto totalTimeUsageDelta = finalStats.totalConnUsageTime - initialStats.totalConnUsageTime;
+    ASSERT_EQ(totalTimeUsageDelta, Milliseconds(0));
+}
+
+TEST_F(ConnectionPoolTest, LeasedConnectionsDontInterfereWithOrdinaryCheckoutUsageTime) {
+    constexpr Milliseconds checkOutLength = Milliseconds(10);
+    auto pool = makePool();
+
+    ConnectionPoolStats initialStats;
+    pool->appendConnectionStats(&initialStats);
+
+    auto startTimePoint = Date_t::now();
+    auto endTimePoint = startTimePoint + checkOutLength;
+    PoolImpl::setNow(startTimePoint);
+
+    ConnectionImpl::pushSetup(Status::OK());
+
+    // Checkout one connection and lease one connection.
+    ConnectionPool::ConnectionHandle normal;
+    ConnectionPool::ConnectionHandle leased;
+    pool->get_forTest(HostAndPort(),
+                      Milliseconds(5000),
+                      [&](StatusWith<ConnectionPool::ConnectionHandle> swConn) {
+                          normal = std::move(swConn.getValue());
+                      });
+    pool->lease_forTest(HostAndPort(),
+                        Milliseconds(5000),
+                        [&](StatusWith<ConnectionPool::ConnectionHandle> swConn) {
+                            leased = std::move(swConn.getValue());
+                        });
+
+    // Advance the time and return the connections.
+    PoolImpl::setNow(endTimePoint);
+    doneWith(normal);
+    doneWith(leased);
+
+    ConnectionPoolStats finalStats;
+    pool->appendConnectionStats(&finalStats);
+
+    auto totalTimeUsageDelta = finalStats.totalConnUsageTime - initialStats.totalConnUsageTime;
+    // Should only include usage time from the checkout, not the lease
+    ASSERT_GREATER_THAN_OR_EQUALS(totalTimeUsageDelta, checkOutLength);
+    ASSERT_LESS_THAN(totalTimeUsageDelta, checkOutLength * 2);
+}
+
 }  // namespace connection_pool_test_details
 }  // namespace executor
 }  // namespace mongo
