@@ -29,21 +29,41 @@
 
 #pragma once
 
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <cstddef>
+#include <functional>
 #include <list>
 #include <memory>
+#include <mutex>
 #include <queue>
+#include <string>
 #include <utility>
 #include <vector>
 
+#include "mongo/base/status.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/db/baton.h"
+#include "mongo/executor/network_connection_hook.h"
 #include "mongo/executor/network_interface.h"
+#include "mongo/executor/remote_command_request.h"
+#include "mongo/executor/remote_command_response.h"
+#include "mongo/executor/task_executor.h"
+#include "mongo/platform/atomic_word.h"
 #include "mongo/platform/mutex.h"
 #include "mongo/rpc/metadata/metadata_hook.h"
 #include "mongo/stdx/condition_variable.h"
 #include "mongo/stdx/unordered_map.h"
 #include "mongo/stdx/unordered_set.h"
-#include "mongo/transport/mock_session.h"
+#include "mongo/transport/transport_layer.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/clock_source.h"
 #include "mongo/util/clock_source_mock.h"
+#include "mongo/util/duration.h"
+#include "mongo/util/functional.h"
+#include "mongo/util/future.h"
+#include "mongo/util/net/hostandport.h"
 #include "mongo/util/time_support.h"
 
 namespace mongo {
@@ -92,7 +112,7 @@ public:
     using NetworkResponseList = std::list<NetworkResponse>;
 
     NetworkInterfaceMock();
-    virtual ~NetworkInterfaceMock();
+    ~NetworkInterfaceMock() override;
 
     ////////////////////////////////////////////////////////////////////////////////
     //
@@ -207,6 +227,11 @@ public:
      */
     bool hasReadyRequests();
 
+    /**
+     * Returns true if the given iterator points to the end of the network operation list.
+     */
+    bool isNetworkOperationIteratorAtEnd(const NetworkInterfaceMock::NetworkOperationIterator& itr);
+
     /**
      * Gets the next unscheduled request to process, blocking until one is available.
      *
