@@ -27,17 +27,37 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
-#include <iostream>
+#include <list>
 #include <memory>
 #include <utility>
 
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
 #include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/simple_bsonobj_comparator.h"
 #include "mongo/executor/network_connection_hook.h"
-#include "mongo/executor/network_interface.h"
+#include "mongo/executor/network_interface_mock.h"
 #include "mongo/executor/network_interface_mock_test_fixture.h"
+#include "mongo/executor/remote_command_request.h"
+#include "mongo/executor/remote_command_response.h"
+#include "mongo/executor/task_executor.h"
 #include "mongo/executor/test_network_connection_hook.h"
+#include "mongo/rpc/metadata.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/bson_test_util.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/duration.h"
+#include "mongo/util/net/hostandport.h"
+#include "mongo/util/time_support.h"
 
 namespace mongo {
 namespace executor {
@@ -55,7 +75,8 @@ TEST_F(NetworkInterfaceMockTest, ConnectionHook) {
     bool gotExpectedReply = false;
 
     RemoteCommandRequest expectedRequest{testHost(),
-                                         "test",
+                                         DatabaseName::createDatabaseName_forTest(boost::none,
+                                                                                  "test"),
                                          BSON("1" << 2),
                                          BSON("some"
                                               << "stuff"),
@@ -110,7 +131,11 @@ TEST_F(NetworkInterfaceMockTest, ConnectionHook) {
     bool gotCorrectCommandReply = false;
 
     RemoteCommandRequestOnAny actualCommandExpected{
-        {testHost()}, "testDB", BSON("test" << 1), rpc::makeEmptyMetadata(), nullptr};
+        {testHost()},
+        DatabaseName::createDatabaseName_forTest(boost::none, "testDB"),
+        BSON("test" << 1),
+        rpc::makeEmptyMetadata(),
+        nullptr};
     RemoteCommandOnAnyResponse actualResponseExpected{testHost(),
                                                       BSON("1212121212"
                                                            << "12121212121212"),
