@@ -89,7 +89,7 @@ int timeNetworkTestMillis(std::size_t operations, NetworkInterface* net) {
 
     func = [&]() {
         RemoteCommandRequest request{
-            server, "admin", bsonObjPing, BSONObj(), nullptr, Milliseconds(-1)};
+            server, DatabaseName::kAdmin, bsonObjPing, BSONObj(), nullptr, Milliseconds(-1)};
         net->startCommand(makeCallbackHandle(), request, callback).transitional_ignore();
     };
 
@@ -109,10 +109,7 @@ TEST(NetworkInterfaceASIO, SerialPerf) {
 
     int duration = timeNetworkTestMillis(numOperations, &netAsio);
     int result = numOperations * 1000 / duration;
-    LOGV2(22591,
-          "THROUGHPUT asio ping ops/s: {throughput}",
-          "THROUGHPUT asio ping ops/s",
-          "throughput"_attr = result);
+    LOGV2(22591, "THROUGHPUT asio ping ops/s", "throughput"_attr = result);
 }
 
 }  // namespace
