@@ -200,9 +200,9 @@ private:
                      PinnedConnectionTaskExecutorTest* fixture)
             : _fixture{fixture} {
             invariant(session);
-            _client = std::make_shared<AsyncDBClient>(hp, std::move(session), nullptr);
+            _client = std::make_shared<AsyncDBClient>(std::move(hp), std::move(session), nullptr);
         }
-        ~LeasedStream() {
+        ~LeasedStream() override {
             _fixture->_streamDestroyedCalls.fetchAndAdd(1);
         }
         AsyncDBClient* getClient() override {
