@@ -28,25 +28,49 @@
  */
 
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/executor/task_executor_test_common.h"
-
+#include <algorithm>
+#include <cstddef>
+#include <fmt/format.h>
+#include <initializer_list>
+#include <list>
 #include <memory>
+#include <utility>
+#include <vector>
 
-#include "mongo/db/concurrency/locker_noop_client_observer.h"
-#include "mongo/db/operation_context.h"
-#include "mongo/executor/network_interface.h"
+#include <absl/container/node_hash_map.h>
+#include <boost/move/utility_core.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/db/client.h"
+#include "mongo/db/service_context.h"
 #include "mongo/executor/network_interface_mock.h"
+#include "mongo/executor/remote_command_request.h"
+#include "mongo/executor/remote_command_response.h"
 #include "mongo/executor/task_executor.h"
+#include "mongo/executor/task_executor_test_common.h"
 #include "mongo/executor/task_executor_test_fixture.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/stdx/condition_variable.h"
 #include "mongo/stdx/thread.h"
 #include "mongo/stdx/unordered_map.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
 #include "mongo/util/cancellation.h"
+#include "mongo/util/clock_source.h"
 #include "mongo/util/clock_source_mock.h"
+#include "mongo/util/duration.h"
+#include "mongo/util/future.h"
+#include "mongo/util/net/hostandport.h"
 #include "mongo/util/str.h"
+#include "mongo/util/time_support.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
 
@@ -89,10 +113,8 @@ public:
     CetRegistrationAgent(const std::string& name, ExecutorTestCaseFactory makeTest) {
         auto& entry = executorTestCaseRegistry()[name];
         if (entry) {
-            LOGV2_FATAL(28713,
-                        "Multiple attempts to register ExecutorTest named {executor}",
-                        "Multiple attempts to register ExecutorTest",
-                        "executor"_attr = name);
+            LOGV2_FATAL(
+                28713, "Multiple attempts to register ExecutorTest", "executor"_attr = name);
         }
         entry = std::move(makeTest);
     }
@@ -154,8 +176,9 @@ auto makeSetStatusOnRemoteCommandCompletionClosure(const RemoteCommandRequest* e
     return [=](const TaskExecutor::RemoteCommandCallbackArgs& cbData) {
         if (cbData.request != *expectedRequest) {
             auto desc = [](const RemoteCommandRequest& request) -> std::string {
-                return str::stream() << "Request(" << request.target.toString() << ", "
-                                     << request.dbname << ", " << request.cmdObj << ')';
+                return str::stream()
+                    << "Request(" << request.target.toString() << ", "
+                    << request.dbname.toString_forTest() << ", " << request.cmdObj << ')';
             };
             *outStatus = Status(ErrorCodes::BadValue,
                                 str::stream() << "Actual request: " << desc(cbData.request)
@@ -171,8 +194,9 @@ auto makeSetStatusOnRemoteExhaustCommandCompletionClosure(
     return [=](const TaskExecutor::RemoteCommandCallbackArgs& cbData) {
         if (cbData.request != *expectedRequest) {
             auto desc = [](const RemoteCommandRequest& request) -> std::string {
-                return str::stream() << "Request(" << request.target.toString() << ", "
-                                     << request.dbname << ", " << request.cmdObj << ')';
+                return str::stream()
+                    << "Request(" << request.target.toString() << ", "
+                    << request.dbname.toString_forTest() << ", " << request.cmdObj << ')';
             };
             *outStatus = Status(ErrorCodes::BadValue,
                                 str::stream() << "Actual request: " << desc(cbData.request)
@@ -185,7 +209,8 @@ auto makeSetStatusOnRemoteExhaustCommandCompletionClosure(
 }
 
 static inline const RemoteCommandRequest kDummyRequest{HostAndPort("localhost", 27017),
-                                                       "mydb",
+                                                       DatabaseName::createDatabaseName_forTest(
+                                                           boost::none, "mydb"),
                                                        BSON("whatsUp"
                                                             << "doc"),
                                                        nullptr};
@@ -287,7 +312,8 @@ EventChainAndWaitingTest::~EventChainAndWaitingTest() {
 }
 
 void EventChainAndWaitingTest::run() {
-    executor->onEvent(goEvent, [=](const TaskExecutor::CallbackArgs& cbData) { onGo(cbData); })
+    executor
+        ->onEvent(goEvent, [=, this](const TaskExecutor::CallbackArgs& cbData) { onGo(cbData); })
         .status_with_transitional_ignore();
     executor->signalEvent(goEvent);
     executor->waitForEvent(goEvent);
@@ -340,8 +366,9 @@ void EventChainAndWaitingTest::onGo(const TaskExecutor::CallbackArgs& cbData) {
         return;
     }
 
-    cbHandle = executor->onEvent(
-        goEvent, [=](const TaskExecutor::CallbackArgs& cbData) { onGoAfterTriggered(cbData); });
+    cbHandle = executor->onEvent(goEvent, [=, this](const TaskExecutor::CallbackArgs& cbData) {
+        onGoAfterTriggered(cbData);
+    });
     if (!cbHandle.isOK()) {
         status1 = cbHandle.getStatus();
         executor->shutdown();
@@ -369,7 +396,7 @@ COMMON_EXECUTOR_TEST(EventWaitingWithTimeoutTest) {
     serviceContext->setFastClockSource(std::make_unique<ClockSourceMock>());
     auto mockClock = static_cast<ClockSourceMock*>(serviceContext->getFastClockSource());
 
-    auto client = serviceContext->makeClient("for testing");
+    auto client = serviceContext->getService()->makeClient("for testing");
     auto opCtx = client->makeOperationContext();
 
     auto deadline = mockClock->now() + Milliseconds{1};
@@ -391,7 +418,7 @@ COMMON_EXECUTOR_TEST(EventSignalWithTimeoutTest) {
     serviceContext->setFastClockSource(std::make_unique<ClockSourceMock>());
     auto mockClock = static_cast<ClockSourceMock*>(serviceContext->getFastClockSource());
 
-    auto client = serviceContext->makeClient("for testing");
+    auto client = serviceContext->getService()->makeClient("for testing");
     auto opCtx = client->makeOperationContext();
 
     auto deadline = mockClock->now() + Milliseconds{1};
@@ -700,8 +727,11 @@ COMMON_EXECUTOR_TEST(RemoteCommandWithTimeout) {
     TaskExecutor& executor = getExecutor();
     Status status(ErrorCodes::InternalError, "");
     launchExecutorThread();
-    const RemoteCommandRequest request(
-        HostAndPort("lazy", 27017), "admin", BSON("sleep" << 1), nullptr, Milliseconds(1));
+    const RemoteCommandRequest request(HostAndPort("lazy", 27017),
+                                       DatabaseName::kAdmin,
+                                       BSON("sleep" << 1),
+                                       nullptr,
+                                       Milliseconds(1));
     TaskExecutor::CallbackHandle cbHandle = unittest::assertGet(executor.scheduleRemoteCommand(
         request, makeSetStatusOnRemoteCommandCompletionClosure(&request, &status)));
     net->enterNetwork();
@@ -721,7 +751,7 @@ COMMON_EXECUTOR_TEST(CallbackHandleComparison) {
     auto status1 = getDetectableErrorStatus();
     auto status2 = getDetectableErrorStatus();
     const RemoteCommandRequest request(
-        HostAndPort("lazy", 27017), "admin", BSON("cmd" << 1), nullptr);
+        HostAndPort("lazy", 27017), DatabaseName::kAdmin, BSON("cmd" << 1), nullptr);
     TaskExecutor::CallbackHandle cbHandle1 = unittest::assertGet(executor.scheduleRemoteCommand(
         request, makeSetStatusOnRemoteCommandCompletionClosure(&request, &status1)));
     TaskExecutor::CallbackHandle cbHandle2 = unittest::assertGet(executor.scheduleRemoteCommand(
@@ -1104,7 +1134,7 @@ COMMON_EXECUTOR_TEST(ScheduleExhaustRemoteCommandFutureIsResolvedWithErrorOnCanc
     net->exitNetwork();
     // Response should be cancelled.
     ASSERT_EQUALS(responseFuture.getNoThrow().getStatus().code(), ErrorCodes::CallbackCanceled);
-    ASSERT_EQUALS(numTimesCallbackCalled, 1);
+    ASSERT_EQUALS(numTimesCallbackCalled, 2);
 
     shutdownExecutorThread();
     joinExecutorThread();
