@@ -31,12 +31,14 @@
 
 #include <cstdint>
 #include <functional>
+#include <mutex>
 #include <vector>
 
 #include "mongo/platform/mutex.h"
 #include "mongo/platform/random.h"
 #include "mongo/stdx/thread.h"
 #include "mongo/util/concurrency/thread_pool_interface.h"
+#include "mongo/util/out_of_line_executor.h"
 
 namespace mongo {
 namespace executor {
@@ -66,7 +68,7 @@ public:
      * generator that is used to determine which schedulable task runs next.
      */
     ThreadPoolMock(NetworkInterfaceMock* net, int32_t prngSeed, Options options);
-    ~ThreadPoolMock();
+    ~ThreadPoolMock() override;
 
     void startup() override;
     void shutdown() override;
