@@ -29,16 +29,27 @@
 
 #pragma once
 
+#include <boost/optional.hpp>
 #include <list>
 #include <memory>
+#include <mutex>
 
+#include "mongo/base/status_with.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/db/baton.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/executor/remote_command_request.h"
 #include "mongo/executor/task_executor.h"
 #include "mongo/platform/mutex.h"
 #include "mongo/stdx/condition_variable.h"
 #include "mongo/stdx/thread.h"
 #include "mongo/transport/baton.h"
 #include "mongo/util/fail_point.h"
+#include "mongo/util/future.h"
 #include "mongo/util/hierarchical_acquisition.h"
+#include "mongo/util/interruptible.h"
+#include "mongo/util/net/hostandport.h"
+#include "mongo/util/time_support.h"
 
 namespace mongo {
 
@@ -67,7 +78,7 @@ public:
     /**
      * Destroys a ThreadPoolTaskExecutor.
      */
-    ~ThreadPoolTaskExecutor();
+    ~ThreadPoolTaskExecutor() override;
 
     void startup() override;
     void shutdown() override;
@@ -92,7 +103,7 @@ public:
     StatusWith<CallbackHandle> scheduleExhaustRemoteCommandOnAny(
         const RemoteCommandRequestOnAny& request,
         const RemoteCommandOnAnyCallbackFn& cb,
-        const BatonHandle& baton = nullptr);
+        const BatonHandle& baton = nullptr) override;
     void cancel(const CallbackHandle& cbHandle) override;
     void wait(const CallbackHandle& cbHandle,
               Interruptible* interruptible = Interruptible::notInterruptible()) override;
@@ -107,7 +118,7 @@ public:
      * Returns true if there are any tasks in any of _poolInProgressQueue, _networkInProgressQueue,
      * or _sleepersQueue.
      */
-    bool hasTasks();
+    bool hasTasks() override;
 
 private:
     class CallbackState;
@@ -172,7 +183,7 @@ private:
      * _poolInProgressQueue.
      */
     void scheduleIntoPool_inlock(WorkQueue* fromQueue,
-                                 const WorkQueue::iterator& iter,
+                                 boost::optional<WorkQueue::iterator> iter,
                                  stdx::unique_lock<Latch> lk);
 
     /**
@@ -180,8 +191,8 @@ private:
      * and moves them into _poolInProgressQueue.
      */
     void scheduleIntoPool_inlock(WorkQueue* fromQueue,
-                                 const WorkQueue::iterator& begin,
-                                 const WorkQueue::iterator& end,
+                                 boost::optional<WorkQueue::iterator>& begin,
+                                 boost::optional<WorkQueue::iterator>& end,
                                  stdx::unique_lock<Latch> lk);
 
     /**
