@@ -27,11 +27,44 @@
  *    it in the license file.
  */
 
+#include <algorithm>
+#include <boost/optional.hpp>
+#include <ctime>
+#include <initializer_list>
+#include <iterator>
+#include <memory>
+#include <set>
+#include <utility>
+
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/timestamp.h"
 #include "mongo/db/catalog/create_collection.h"
 #include "mongo/db/catalog_raii.h"
+#include "mongo/db/client.h"
+#include "mongo/db/commands/create_gen.h"
+#include "mongo/db/concurrency/lock_manager_defs.h"
+#include "mongo/db/logical_time.h"
+#include "mongo/db/server_parameter.h"
+#include "mongo/db/server_parameter_with_storage.h"
+#include "mongo/db/storage/write_unit_of_work.h"
+#include "mongo/db/tenant_id.h"
+#include "mongo/idl/cluster_server_parameter_gen.h"
 #include "mongo/idl/cluster_server_parameter_op_observer.h"
+#include "mongo/idl/cluster_server_parameter_test_gen.h"
 #include "mongo/idl/cluster_server_parameter_test_util.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/assert_util.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kControl
 
@@ -77,6 +110,7 @@ public:
                            *autoColl,
                            stmts.cbegin(),
                            stmts.cend(),
+                           /*recordIds*/ {},
                            /*fromMigrate=*/std::vector<bool>(stmts.size(), false),
                            /*defaultFromMigrate=*/false);
         if (commit)
@@ -100,17 +134,12 @@ public:
             wuow.commit();
     }
 
-    void doDelete(const NamespaceString& nss,
-                  BSONObj deletedDoc,
-                  bool includeDeletedDoc = true,
-                  bool commit = true) {
+    void doDelete(const NamespaceString& nss, BSONObj deletedDoc, bool commit) {
         auto opCtx = cc().makeOperationContext();
         WriteUnitOfWork wuow(opCtx.get());
         AutoGetCollection autoColl(opCtx.get(), nss, MODE_IX);
-        observer.aboutToDelete(opCtx.get(), *autoColl, deletedDoc);
         OplogDeleteEntryArgs args;
-        args.deletedDoc = includeDeletedDoc ? &deletedDoc : nullptr;
-        observer.onDelete(opCtx.get(), *autoColl, 1 /* StmtId */, args);
+        observer.onDelete(opCtx.get(), *autoColl, /*StmtId=*/1, deletedDoc, args);
         if (commit)
             wuow.commit();
     }
@@ -118,7 +147,7 @@ public:
     void doDropDatabase(const DatabaseName& dbname, bool commit = true) {
         auto opCtx = cc().makeOperationContext();
         WriteUnitOfWork wuow(opCtx.get());
-        observer.onDropDatabase(opCtx.get(), dbname);
+        observer.onDropDatabase(opCtx.get(), dbname, false /*fromMigrate*/);
         if (commit)
             wuow.commit();
     }
@@ -389,39 +418,41 @@ TEST_F(ClusterServerParameterOpObserverTest, onDeleteRecord) {
     const auto initialDoc = std::move(initialDocB);
 
     // Ignore deletes in other namespaces, whether with or without deleted doc.
-    assertIgnoredOtherNamespaces([this, initialDoc](const auto& nss) { doDelete(nss, initialDoc); },
-                                 boost::none);
+    assertIgnoredOtherNamespaces(
+        [this, initialDoc](const auto& nss) { doDelete(nss, initialDoc, /*commit=*/true); },
+        boost::none);
 
     // Ignore deletes where the _id does not correspond to a known cluster server parameter.
     assertIgnoredAlways(
         [this](const auto& nss) {
-            doDelete(nss, BSON(ClusterServerParameter::k_idFieldName << "ignored"));
+            doDelete(
+                nss, BSON(ClusterServerParameter::k_idFieldName << "ignored"), /*commit=*/true);
         },
         boost::none);
 
     // Reset configuration to defaults when we claim to have deleted the doc.
-    doDelete(NamespaceString::kClusterParametersNamespace, initialDoc);
+    doDelete(NamespaceString::kClusterParametersNamespace, initialDoc, /*commit=*/true);
 
     ASSERT_PARAMETER_STATE(boost::none, kDefaultIntValue, kDefaultStrValue);
     ASSERT_PARAMETER_STATE(kTenantId, kInitialTenantIntValue, kInitialTenantStrValue);
 
-    // Restore configured state, and delete without including deleteDoc reference.
+    // Restore configured state, and delete.
     initializeState();
-    doDelete(NamespaceString::kClusterParametersNamespace, initialDoc, false);
+    doDelete(NamespaceString::kClusterParametersNamespace, initialDoc, /*commit=*/true);
 
     ASSERT_PARAMETER_STATE(boost::none, kDefaultIntValue, kDefaultStrValue);
     ASSERT_PARAMETER_STATE(kTenantId, kInitialTenantIntValue, kInitialTenantStrValue);
 
     // Restore configured state, and delete other tenant with reference.
     initializeState();
-    doDelete(NamespaceString::makeClusterParametersNSS(kTenantId), initialDocT);
+    doDelete(NamespaceString::makeClusterParametersNSS(kTenantId), initialDocT, /*commit=*/true);
 
     ASSERT_PARAMETER_STATE(boost::none, kInitialIntValue, kInitialStrValue);
     ASSERT_PARAMETER_STATE(kTenantId, kDefaultIntValue, kDefaultStrValue);
 
     // Restore and delete without reference.
     initializeState();
-    doDelete(NamespaceString::makeClusterParametersNSS(kTenantId), initialDocT, false);
+    doDelete(NamespaceString::makeClusterParametersNSS(kTenantId), initialDocT, /*commit=*/true);
 
     ASSERT_PARAMETER_STATE(boost::none, kInitialIntValue, kInitialStrValue);
     ASSERT_PARAMETER_STATE(kTenantId, kDefaultIntValue, kDefaultStrValue);
@@ -434,7 +465,7 @@ TEST_F(ClusterServerParameterOpObserverTest, onDropDatabase) {
     assertIgnoredOtherNamespaces(
         [this](const auto& nss) {
             const auto dbname = nss.dbName();
-            if (dbname.db() != kConfigDB) {
+            if (!dbname.isConfigDB()) {
                 doDropDatabase(dbname);
             }
         },
@@ -451,7 +482,7 @@ TEST_F(ClusterServerParameterOpObserverTest, onDropDatabase) {
 
     // Reinitialize and drop the other tenant's config DB.
     initializeState();
-    doDropDatabase(DatabaseName(kTenantId, kConfigDB));
+    doDropDatabase(DatabaseName::createDatabaseName_forTest(kTenantId, kConfigDB));
 
     ASSERT_PARAMETER_STATE(boost::none, kInitialIntValue, kInitialStrValue);
     ASSERT_PARAMETER_STATE(kTenantId, kDefaultIntValue, kDefaultStrValue);
@@ -500,38 +531,35 @@ TEST_F(ClusterServerParameterOpObserverTest, abortsAfterObservation) {
 
     doInserts(NamespaceString::kClusterParametersNamespace,
               {makeClusterParametersDoc(LogicalTime(Timestamp(12345678)), 123, "abc")},
-              false /* commit */);
+              /*commit=*/false);
     doInserts(NamespaceString::makeClusterParametersNSS(kTenantId),
               {makeClusterParametersDoc(LogicalTime(Timestamp(23456789)), 456, "def")},
-              false /* commit */);
+              /*commit=*/false);
 
     ASSERT_PARAMETER_STATE(boost::none, kInitialIntValue, kInitialStrValue);
     ASSERT_PARAMETER_STATE(kTenantId, kInitialTenantIntValue, kInitialTenantStrValue);
 
     doUpdate(NamespaceString::kClusterParametersNamespace,
              {makeClusterParametersDoc(LogicalTime(Timestamp(87654321)), 321, "cba")},
-             false /* commit */);
+             /*commit=*/false);
     doUpdate(NamespaceString::makeClusterParametersNSS(kTenantId),
              {makeClusterParametersDoc(LogicalTime(Timestamp(98765432)), 654, "fed")},
-             false /* commit */);
+             /*commit=*/false);
 
     ASSERT_PARAMETER_STATE(boost::none, kInitialIntValue, kInitialStrValue);
     ASSERT_PARAMETER_STATE(kTenantId, kInitialTenantIntValue, kInitialTenantStrValue);
 
-    doDelete(NamespaceString::kClusterParametersNamespace,
-             initialDoc,
-             true /* includeDeletedDoc */,
-             false /* commit */);
-    doDelete(NamespaceString::makeClusterParametersNSS(kTenantId),
-             initialDocTenant,
-             true /* includeDeletedDoc */,
-             false /* commit */);
+    doDelete(NamespaceString::kClusterParametersNamespace, initialDoc, /*commit=*/false);
+    doDelete(
+        NamespaceString::makeClusterParametersNSS(kTenantId), initialDocTenant, /*commit=*/false);
 
     ASSERT_PARAMETER_STATE(boost::none, kInitialIntValue, kInitialStrValue);
     ASSERT_PARAMETER_STATE(kTenantId, kInitialTenantIntValue, kInitialTenantStrValue);
 
-    doDropDatabase(DatabaseName(boost::none, kConfigDB), false /* commit */);
-    doDropDatabase(DatabaseName(kTenantId, kConfigDB), false /* commit */);
+    doDropDatabase(DatabaseName::createDatabaseName_forTest(boost::none, kConfigDB),
+                   /*commit=*/false);
+    doDropDatabase(DatabaseName::createDatabaseName_forTest(kTenantId, kConfigDB),
+                   /*commit=*/false);
 
     ASSERT_PARAMETER_STATE(boost::none, kInitialIntValue, kInitialStrValue);
     ASSERT_PARAMETER_STATE(kTenantId, kInitialTenantIntValue, kInitialTenantStrValue);
