@@ -29,15 +29,31 @@
 
 #pragma once
 
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+#include <memory>
+
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonobj.h"
 #include "mongo/db/change_stream_options_manager.h"
+#include "mongo/db/client.h"
 #include "mongo/db/dbdirectclient.h"
+#include "mongo/db/logical_time.h"
 #include "mongo/db/multitenancy_gen.h"
+#include "mongo/db/query/query_settings/query_settings_manager.h"
+#include "mongo/db/repl/member_state.h"
+#include "mongo/db/repl/oplog.h"
+#include "mongo/db/repl/repl_settings.h"
+#include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/repl/replication_coordinator_mock.h"
+#include "mongo/db/repl/storage_interface.h"
 #include "mongo/db/repl/storage_interface_mock.h"
 #include "mongo/db/service_context_d_test_fixture.h"
+#include "mongo/db/tenant_id.h"
 #include "mongo/idl/cluster_server_parameter_gen.h"
 #include "mongo/idl/cluster_server_parameter_test_gen.h"
 #include "mongo/s/write_ops/batched_command_response.h"
+#include "mongo/unittest/assert.h"
 #include "mongo/unittest/unittest.h"
 
 namespace mongo {
@@ -49,7 +65,7 @@ const auto kNilCPT = LogicalTime::kUninitialized;
 
 class ClusterServerParameterTestBase : public ServiceContextMongoDTest {
 public:
-    virtual void setUp() override {
+    void setUp() override {
         gMultitenancySupport = true;
         // Set up mongod.
         ServiceContextMongoDTest::setUp();
@@ -67,6 +83,9 @@ public:
         // Set up the ChangeStreamOptionsManager so that it can be retrieved/set.
         ChangeStreamOptionsManager::create(service);
 
+        // Set up the QuerySettingsManager so that it can be retrieved/set.
+        query_settings::QuerySettingsManager::create(service, {}, {});
+
         // Ensure that we are primary.
         auto replCoord = repl::ReplicationCoordinator::get(opCtx.get());
         ASSERT_OK(replCoord->setFollowerMode(repl::MemberState::RS_PRIMARY));
