@@ -29,9 +29,9 @@
 
 #pragma once
 
-#include <functional>
-
 #include <boost/optional.hpp>
+#include <boost/optional/optional.hpp>
+#include <functional>
 
 #include "mongo/base/status.h"
 #include "mongo/base/string_data.h"
@@ -42,6 +42,7 @@ constexpr auto kServerLogTag = "server"_sd;
 constexpr auto kAuditLogTag = "audit"_sd;
 
 using LogRotateCallback = std::function<Status(bool, StringData, std::function<void(Status)>)>;
+using ShouldEmitLogServiceFn = std::function<bool()>;
 
 /**
  * logType param needs to have static lifetime. If a new logType needs to be defined, add it above
@@ -102,4 +103,16 @@ bool shouldRedactBinDataEncrypt();
  */
 void setShouldRedactBinDataEncrypt(bool enabled);
 
+/**
+ * Returns true if log service names should be emitted. Returns false until setShouldEmitLogService
+ * is called.
+ */
+bool shouldEmitLogService();
+
+/**
+ * Set a callback which shouldEmitLogService() invokes to determine whether log service names should
+ * be emitted.
+ */
+void setShouldEmitLogService(ShouldEmitLogServiceFn fn);
+
 }  // namespace mongo::logv2
