@@ -28,56 +28,127 @@
  */
 
 
-#include "mongo/platform/basic.h"
-
+#include <array>
+#include <boost/property_tree/json_parser.hpp>
+#include <boost/property_tree/ptree.hpp>
+#include <boost/tuple/tuple.hpp>
 #include <csignal>
-#include <fstream>
+#include <cstdint>
+#include <cstdlib>
+#include <deque>
+#include <exception>
+#include <fmt/format.h>
+#include <forward_list>
+#include <fstream>  // IWYU pragma: keep
+#include <initializer_list>
+#include <iostream>
+#include <iterator>
+#include <limits>
+#include <list>
+#include <map>
+#include <memory>
 #include <sstream>
 #include <string>
+#include <string_view>
+#include <type_traits>
+#include <utility>
 #include <vector>
 
+#include <absl/container/flat_hash_map.h>
+#include <absl/meta/type_traits.h>
+#include <boost/core/swap.hpp>
+#include <boost/exception/exception.hpp>
+#include <boost/iterator/iterator_facade.hpp>
+#include <boost/log/attributes/attribute_value_set.hpp>
+#include <boost/log/core/core.hpp>
+#include <boost/log/core/record_view.hpp>
+// IWYU pragma: no_include "boost/log/detail/attachable_sstream_buf.hpp"
+// IWYU pragma: no_include "boost/log/detail/locking_ptr.hpp"
+#include <boost/log/keywords/file_name.hpp>
+#include <boost/log/sinks/basic_sink_backend.hpp>
+#include <boost/log/sinks/frontend_requirements.hpp>
+#include <boost/log/sinks/sink.hpp>
+#include <boost/log/sinks/sync_frontend.hpp>
+#include <boost/log/sinks/text_file_backend.hpp>
+#include <boost/log/sinks/unlocked_frontend.hpp>
+#include <boost/move/utility_core.hpp>
+// IWYU pragma: no_include "boost/multi_index/detail/bidir_node_iterator.hpp"
+#include <boost/none.hpp>
+#include <boost/operators.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/parameter/keyword.hpp>
+// IWYU pragma: no_include "boost/property_tree/detail/exception_implementation.hpp"
+// IWYU pragma: no_include "boost/property_tree/detail/ptree_implementation.hpp"
+#include <boost/property_tree/ptree_fwd.hpp>
+#include <boost/smart_ptr/make_shared_object.hpp>
+#include <boost/smart_ptr/shared_ptr.hpp>
+#include <boost/thread/exceptions.hpp>
+#include <fmt/format.h>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
 #include "mongo/bson/bsonobj.h"
 #include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
 #include "mongo/bson/json.h"
 #include "mongo/bson/oid.h"
-#include "mongo/db/auth/validated_tenancy_scope.h"
+#include "mongo/bson/timestamp.h"
 #include "mongo/db/tenant_id.h"
+#include "mongo/logv2/attribute_storage.h"
 #include "mongo/logv2/bson_formatter.h"
 #include "mongo/logv2/component_settings_filter.h"
 #include "mongo/logv2/composite_backend.h"
 #include "mongo/logv2/constants.h"
 #include "mongo/logv2/json_formatter.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
 #include "mongo/logv2/log_capture_backend.h"
 #include "mongo/logv2/log_component.h"
+#include "mongo/logv2/log_debug.h"
+#include "mongo/logv2/log_detail.h"
 #include "mongo/logv2/log_domain.h"
 #include "mongo/logv2/log_domain_global.h"
 #include "mongo/logv2/log_domain_internal.h"
 #include "mongo/logv2/log_manager.h"
+#include "mongo/logv2/log_options.h"
+#include "mongo/logv2/log_severity.h"
+#include "mongo/logv2/log_source.h"
+#include "mongo/logv2/log_tag.h"
+#include "mongo/logv2/log_truncation.h"
 #include "mongo/logv2/plain_formatter.h"
+#include "mongo/logv2/ramlog.h"
 #include "mongo/logv2/ramlog_sink.h"
 #include "mongo/logv2/text_formatter.h"
 #include "mongo/logv2/uassert_sink.h"
+#include "mongo/platform/atomic_word.h"
 #include "mongo/platform/decimal128.h"
+#include "mongo/platform/int128.h"
 #include "mongo/stdx/thread.h"
+#include "mongo/stdx/unordered_map.h"
+#include "mongo/unittest/assert.h"
 #include "mongo/unittest/death_test.h"
+#include "mongo/unittest/framework.h"
 #include "mongo/unittest/temp_dir.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/concurrency/thread_name.h"
+#include "mongo/util/debug_util.h"
+#include "mongo/util/duration.h"
 #include "mongo/util/exit_code.h"
 #include "mongo/util/shared_buffer.h"
 #include "mongo/util/str_escape.h"
 #include "mongo/util/string_map.h"
+#include "mongo/util/time_support.h"
 #include "mongo/util/uuid.h"
 
-#include <boost/log/attributes/constant.hpp>
-#include <boost/optional.hpp>
-#include <boost/property_tree/json_parser.hpp>
-#include <boost/property_tree/ptree.hpp>
-
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
 
 
 namespace mongo::logv2 {
+
+using namespace fmt::literals;
+
 namespace {
 
 using constants::kAttributesFieldName;
@@ -392,10 +463,6 @@ TEST_F(LogV2Test, Basic) {
     buffer.clear();
     t4.serialize(buffer);
     ASSERT_EQUALS(lines->back(), fmt::to_string(buffer));
-
-    // Message string is selected when using API that also take a format string
-    LOGV2(20084, "fmtstr {name}", "msgstr", "name"_attr = 1);
-    ASSERT_EQUALS(lines->back(), "msgstr");
 }
 
 TEST_F(LogV2Test, MismatchAttrInLogging) {
@@ -476,10 +543,9 @@ class LogV2TypesTest : public LogV2Test {
 public:
     using LogV2Test::LogV2Test;
     LogV2TypesTest() : LogV2Test() {
-        detail::setGetTenantIDCallback(
-            [this]() -> boost::optional<TenantId> { return this->tenant; });
+        detail::setGetTenantIDCallback([this]() -> std::string { return this->tenant.toString(); });
     }
-    ~LogV2TypesTest() {
+    ~LogV2TypesTest() override {
         detail::setGetTenantIDCallback(nullptr);
     }
 
@@ -500,8 +566,9 @@ public:
         str.copy(buf.get(), str.size());
         BSONObj obj(buf);
 
-        ASSERT_EQUALS(obj.getField(kTenantFieldName).str(), tenant.toString());
-        return obj.getField(kAttributesFieldName).Obj().getField("name"_sd);
+        ASSERT_EQUALS(obj.getField(kTenantFieldName).String(), tenant.toString());
+        container = obj.getField(kAttributesFieldName).Obj();
+        return container.getField("name"_sd);
     }
 
     TenantId tenant = TenantId(OID::gen());
@@ -509,6 +576,12 @@ public:
     std::unique_ptr<LineCapture> json = makeLineCapture(JSONFormatter());
     std::unique_ptr<LineCapture> bson = makeLineCapture(BSONFormatter());
     SharedBuffer buf;
+    // Using BSONElements returned by lastBSONElement() is safe, as they reference data stored in
+    // `buf`. However, this cannot normally be enforced, so getField() is annotated
+    // [[lifetimebound]]. As such, having the nested BSONObj go out of scope before the
+    // BSONElement will lead to a compile time warning "-Wreturn-stack-address". Store the
+    // nested object here to avoid.
+    BSONObj container;
 };
 
 TEST_F(LogV2TypesTest, Numeric) {
@@ -599,6 +672,37 @@ TEST_F(LogV2TypesTest, Numeric) {
     // long double is prohibited, we don't use this type and favors Decimal128 instead.
 }
 
+// int128 is not a numeric type for the purposes of bson->
+TEST_F(LogV2TypesTest, Int128) {
+    auto test = [&](auto value) {
+        text->clear();
+        LOGV2(7497400, "uint128/int128 {name}", "name"_attr = value);
+        ASSERT_EQUALS(text->back(), "uint128/int128 " + toString(value));
+    };
+
+    test(uint128_t(0));
+    test(std::numeric_limits<uint128_t>::min());
+    test(std::numeric_limits<uint128_t>::max());
+    test(uint128_t(-10));
+    test(uint128_t(-2));
+    test(uint128_t(-1));
+    test(uint128_t(0));
+    test(uint128_t(1));
+    test(uint128_t(2));
+    test(uint128_t(10));
+
+    test(int128_t(0));
+    test(std::numeric_limits<int128_t>::min());
+    test(std::numeric_limits<int128_t>::max());
+    test(int128_t(-10));
+    test(int128_t(-2));
+    test(int128_t(-1));
+    test(int128_t(0));
+    test(int128_t(1));
+    test(int128_t(2));
+    test(int128_t(10));
+}
+
 TEST_F(LogV2TypesTest, Enums) {
     // enums
     enum UnscopedEnum { UnscopedEntry };
@@ -851,6 +955,48 @@ TEST_F(LogV2TypesTest, Duration) {
                   ms.count());
 }
 
+void exceptionThrower() {
+    uasserted(7733401, "exception in logger");
+}
+
+template <typename T>
+void testExceptionHandling(T arg,
+                           std::unique_ptr<LogV2Test::LineCapture> text,
+                           std::unique_ptr<LogV2Test::LineCapture> json) {
+    LOGV2(7733402, "test1 {a1}", "a1"_attr = arg);
+
+    ASSERT_EQ(
+        mongo::fromjson(json->back()).getField(kAttributesFieldName).Obj().getField("a1").String(),
+        "Failed to serialize due to exception: Location7733401: exception in logger");
+
+    ASSERT_EQ(text->back(),
+              "test1 Failed to serialize due to exception: Location7733401: exception in logger");
+}
+
+// Throw an exception in a BSON serialization method
+TEST_F(LogV2TypesTest, AttrExceptionBSONSerialize) {
+    struct TypeWithBSONSerialize {
+        void serialize(BSONObjBuilder*) const {
+            exceptionThrower();
+        }
+    };
+
+    testExceptionHandling(TypeWithBSONSerialize(), std::move(text), std::move(json));
+}
+
+// Throw an exception in a BSON Array serialization method
+TEST_F(LogV2TypesTest, AttrExceptionBSONToARray) {
+    struct TypeToArray {
+        BSONArray toBSONArray() const {
+            exceptionThrower();
+            return {};
+        }
+    };
+
+    testExceptionHandling(TypeToArray(), std::move(text), std::move(json));
+}
+
+
 TEST_F(LogV2Test, TextFormat) {
     auto lines = makeLineCapture(TextFormatter());
 
@@ -1465,10 +1611,9 @@ TEST_F(LogV2ContainerTest, StringMapUint32) {
 }
 
 TEST_F(LogV2Test, AttrNameCollision) {
-    ASSERT_THROWS_CODE(
-        LOGV2(4793300, "Collision {k1}", "Collision", "k1"_attr = "v1", "k1"_attr = "v2"),
-        AssertionException,
-        4793301);
+    ASSERT_THROWS_CODE(LOGV2(4793300, "Collision", "k1"_attr = "v1", "k1"_attr = "v2"),
+                       AssertionException,
+                       4793301);
 }
 
 TEST_F(LogV2Test, Unicode) {
@@ -1537,92 +1682,342 @@ TEST_F(LogV2Test, Unicode) {
     }
 }
 
-TEST_F(LogV2Test, JsonTruncation) {
-    auto lines = makeLineCapture(JSONFormatter());
+class LogV2JsonTruncationTest : public LogV2Test {
+public:
+    static constexpr std::size_t maxAttributeOutputSize =
+        constants::kDefaultMaxAttributeOutputSizeKB * 1024;
 
-    std::size_t maxAttributeOutputSize = constants::kDefaultMaxAttributeOutputSizeKB * 1024;
+    static inline const std::string largeString = std::string(maxAttributeOutputSize * 2, 'a');
 
-    BSONObjBuilder builder;
-    BSONObjBuilder subobj = builder.subobjStart("sub"_sd);
-    subobj.append("small1", 1);
-    subobj.append("small2", "small string");
-    subobj.append("large", std::string(maxAttributeOutputSize * 2, 'a'));
-    subobj.append("small3", "small string after large object");
-    subobj.done();
+    // Represents a segment of the truncation path
+    struct TruncationPathSegment {
+        // name of the field where the truncation occurs
+        std::string fieldName;
 
-    LOGV2(20085, "{name}{attr2}", "name"_attr = builder.done(), "attr2"_attr = true);
-    auto validateTruncation = [&](const BSONObj& obj) {
-        // Check that all fields up until the large one is written
-        BSONObj sub = obj.getField(constants::kAttributesFieldName)
-                          .Obj()
-                          .getField("name"_sd)
-                          .Obj()
-                          .getField("sub"_sd)
-                          .Obj();
-        ASSERT(sub.hasField("small1"_sd));
-        ASSERT(sub.hasField("small2"_sd));
-        ASSERT(!sub.hasField("large"_sd));
-        ASSERT(!sub.hasField("small3"_sd));
-
-        // The truncated field should we witten in the truncated and size sub object
-        BSONObj truncated = obj.getField(constants::kTruncatedFieldName).Obj();
-        BSONObj truncatedInfo =
-            truncated.getField("name"_sd).Obj().getField("sub"_sd).Obj().getField("large"_sd).Obj();
-        ASSERT_EQUALS(truncatedInfo.getField("type"_sd).String(), typeName(BSONType::String));
-        ASSERT(truncatedInfo.getField("size"_sd).isNumber());
-
-        ASSERT_EQUALS(
-            obj.getField(constants::kTruncatedSizeFieldName).Obj().getField("name"_sd).Int(),
-            builder.done().objsize());
-
-        // Attributes coming after the truncated one should be written
-        ASSERT(obj.getField(constants::kAttributesFieldName).Obj().getField("attr2").Bool());
+        // number of omitted fields after fieldName
+        int omitted;
     };
-    validateTruncation(mongo::fromjson(lines->back()));
 
-    LOGV2_OPTIONS(20086, {LogTruncation::Disabled}, "{name}", "name"_attr = builder.done());
-    auto validateTruncationDisabled = [&](const BSONObj& obj) {
-        BSONObj sub = obj.getField(constants::kAttributesFieldName)
-                          .Obj()
-                          .getField("name"_sd)
-                          .Obj()
-                          .getField("sub"_sd)
-                          .Obj();
-        // No truncation should occur
-        ASSERT(sub.hasField("small1"_sd));
-        ASSERT(sub.hasField("small2"_sd));
-        ASSERT(sub.hasField("large"_sd));
-        ASSERT(sub.hasField("small3"_sd));
+    struct TruncationInfo {
+        // Truncation path is a sequence of segments for each level of the truncated BSON object,
+        // where the index in the sequence is the same as the depth of that segment in the object.
+        // The last segment in the path is the "leaf" from which the truncation started,
+        // and unlike the segments before it, it is NOT expected to appear in the truncated object,
+        // and must therefore be counted in the expected "omitted" value.
+        std::vector<TruncationPathSegment> path;
 
-        ASSERT(!obj.hasField(constants::kTruncatedFieldName));
-        ASSERT(!obj.hasField(constants::kTruncatedSizeFieldName));
+        // BSON type of the leaf element from which the truncation started
+        BSONType leafType;
     };
-    validateTruncationDisabled(mongo::fromjson(lines->back()));
 
-    BSONArrayBuilder arrBuilder;
-    // Fields will use more than one byte each so this will truncate at some point
-    for (size_t i = 0; i < maxAttributeOutputSize; ++i) {
-        arrBuilder.append("str");
+    struct TestCase {
+        // Attr object before truncation
+        BSONObj originalDoc;
+
+        // Describes the expected truncation of the attr object
+        TruncationInfo truncationInfo;
+
+        // Name for this test case
+        std::string name;
+    };
+
+    static TestCase largeStringInSubobjTest() {
+        BSONObjBuilder builder;
+        TruncationInfo truncation;
+        builder.append("lvl1_a", "a");
+        {
+            BSONObjBuilder subobj1 = builder.subobjStart("sub1"_sd);
+            subobj1.append("lvl2_a", 1);
+            subobj1.append("lvl2_b", "small string");
+            {
+                BSONObjBuilder subobj2 = subobj1.subobjStart("sub2"_sd);
+                subobj2.append("lvl3_a", 1);
+                subobj2.append("lvl3_b", "b");
+                subobj2.append("large", largeString);
+                subobj2.append("lvl3_c", "small string after large object");
+            }
+            subobj1.append("lvl2_c", 1);
+            subobj1.append("lvl2_d", 2);
+        }
+        truncation.leafType = BSONType::String;
+        truncation.path = {{"sub1", 0}, {"sub2", 2}, {"large", 2}};
+        return TestCase{builder.obj(), std::move(truncation), "large string in subobject"};
     }
 
-    BSONArray arrToLog = arrBuilder.arr();
-    LOGV2(20087, "{name}", "name"_attr = arrToLog);
-    auto validateArrayTruncation = [&](const BSONObj& obj) {
-        auto arr = obj.getField(constants::kAttributesFieldName).Obj().getField("name"_sd).Array();
-        ASSERT_LESS_THAN(arr.size(), maxAttributeOutputSize);
+    static TestCase singleLargeStringInObjTest() {
+        BSONObjBuilder builder;
+        TruncationInfo truncation;
+        builder.append("large", largeString);
+        truncation.leafType = BSONType::String;
+        truncation.path = {{"large", 1}};
+        return TestCase{builder.obj(), std::move(truncation), "single large string in object"};
+    }
 
-        std::string truncatedFieldName = std::to_string(arr.size());
-        BSONObj truncated = obj.getField(constants::kTruncatedFieldName).Obj();
-        BSONObj truncatedInfo =
-            truncated.getField("name"_sd).Obj().getField(truncatedFieldName).Obj();
-        ASSERT_EQUALS(truncatedInfo.getField("type"_sd).String(), typeName(BSONType::String));
-        ASSERT(truncatedInfo.getField("size"_sd).isNumber());
+    static TestCase largeArrayTest() {
+        BSONArrayBuilder builder;
+        TruncationInfo truncation;
+        for (size_t i = 0; i < maxAttributeOutputSize; ++i) {
+            builder.append("str");
+        }
+        truncation.leafType = BSONType::String;
+        truncation.path = {{"862", maxAttributeOutputSize - 862}};
+        return TestCase{builder.arr(), std::move(truncation), "large array"};
+    }
 
-        ASSERT_EQUALS(
-            obj.getField(constants::kTruncatedSizeFieldName).Obj().getField("name"_sd).Int(),
-            arrToLog.objsize());
-    };
-    validateArrayTruncation(mongo::fromjson(lines->back()));
+    static TestCase singleLargeStringInArrayTest() {
+        BSONArrayBuilder builder;
+        TruncationInfo truncation;
+        builder.append(largeString);
+        truncation.leafType = BSONType::String;
+        truncation.path = {{"0", 1}};
+        return TestCase{builder.arr(), std::move(truncation), "single large string in array"};
+    }
+
+    static TestCase largeStringInNestedArraysTest() {
+        BSONArrayBuilder builder;
+        TruncationInfo truncation;
+        builder.append("1_a");  // ["1_a",
+        {
+            // ["1_a", [
+            BSONArrayBuilder subarr1 = builder.subarrayStart();
+            // ["1_a", [[
+            BSONArrayBuilder{subarr1.subarrayStart()}
+                .append("3_a")
+                .append("3_b")
+                .append("3_c")
+                .append(largeString)
+                .append("3_d");
+        }
+        builder.append("1_b");
+        // ["1_a", [["3_a", "3_b", "3_c", largeString, "3_d"]], "1_b"]
+        auto array = builder.arr();
+
+        truncation.leafType = BSONType::String;
+        truncation.path = {{"1", 1}, {"0", 0}, {"3", 2}};
+        return TestCase{array, std::move(truncation), "large string in nested arrays"};
+    }
+
+    static std::vector<TestCase> generateTests() {
+        return {largeStringInSubobjTest(),
+                singleLargeStringInObjTest(),
+                largeArrayTest(),
+                singleLargeStringInArrayTest(),
+                largeStringInNestedArraysTest()};
+    }
+
+    // Validates the truncation report in the log line for attrName has the correct structure
+    // that matches the expected truncation path.
+    // For reference, an example truncation report (for largeStringInSubobj test case) looks like:
+    //  {
+    //     "truncated":{
+    //         "sub1":{
+    //             "truncated":{
+    //                 "sub2":{
+    //                     "truncated":{
+    //                         "large":{
+    //                             "type":"string",
+    //                             "size":{"$numberInt":"20485"}
+    //                         }
+    //                     },
+    //                     "omitted":{"$numberInt":"2"}
+    //                 }
+    //             },
+    //             "omitted":{"$numberInt":"2"}
+    //         }
+    //     }
+    //  }
+    static void validateTruncationReport(StringData attrName,
+                                         BSONObj report,
+                                         const TestCase& test) {
+        auto context =
+            "Failed test: {} Failing report: {}"_format(test.name, mongo::tojson(report));
+        auto& path = test.truncationInfo.path;
+
+        ASSERT_FALSE(path.empty()) << context;
+        ASSERT(report.hasField(attrName)) << context;
+
+        BSONObj fieldObj = report.getField(attrName).Obj();
+        BSONObj truncated;
+        std::string currentObjPath = attrName.toString();
+
+        // validate nested "truncated" elements except for the last (leaf) truncated element.
+        for (size_t i = 0; i < path.size(); i++) {
+            const auto& segment = path.at(i);
+
+            ASSERT(fieldObj.hasField(constants::kTruncatedFieldName))
+                << "{} - missing 'truncated' field at path: {}"_format(context, currentObjPath);
+
+            truncated = fieldObj.getField(constants::kTruncatedFieldName).Obj();
+
+            if (segment.omitted != 0) {
+                ASSERT(fieldObj.hasField(constants::kOmittedFieldName))
+                    << "{} - missing 'omitted' field at path: {}"_format(context, currentObjPath);
+                ASSERT_EQUALS(fieldObj.getField("omitted").Int(), segment.omitted)
+                    << "{} - bad 'omitted' value at path: {}"_format(context, currentObjPath);
+            } else {
+                ASSERT_FALSE(fieldObj.hasField("omitted"))
+                    << "{} - unexpected 'omitted' field at path: {}"_format(context,
+                                                                            currentObjPath);
+            }
+
+            currentObjPath += ".truncated";
+            ASSERT(truncated.hasField(segment.fieldName))
+                << "{} - missing expected subobject {} at path {}"_format(
+                       context, segment.fieldName, currentObjPath);
+
+            fieldObj = truncated.getField(segment.fieldName).Obj();
+            currentObjPath += "." + segment.fieldName;
+        }
+        // leaf reached
+        ASSERT(fieldObj.hasField("type"))
+            << "{} - missing field 'type' at path {}"_format(context, currentObjPath);
+
+        ASSERT(fieldObj.hasField("size"))
+            << "{} - missing field 'size' at path {}"_format(context, currentObjPath);
+
+        ASSERT(!fieldObj.hasField("omitted"))
+            << "{} - unexpected field 'omitted' at path {}"_format(context, currentObjPath);
+
+        ASSERT(!fieldObj.hasField("truncated"))
+            << "{} - unexpected field 'truncated' at path {}"_format(context, currentObjPath);
+
+        ASSERT_EQUALS(fieldObj.getField("type"_sd).String(), typeName(test.truncationInfo.leafType))
+            << "{} - bad 'type' value at path {}"_format(context, currentObjPath);
+
+        ASSERT(fieldObj.getField("size"_sd).isNumber())
+            << "{} - bad 'size' value at path {}"_format(context, currentObjPath);
+    }
+
+    // Validates the reported size of the truncated attr in the log line matches the size of the
+    // original BSON object.
+    static void validateTruncationSize(StringData attrName,
+                                       BSONObj truncatedSize,
+                                       const TestCase& test) {
+        auto context =
+            "Failed test: {} Failing report: {}"_format(test.name, mongo::tojson(truncatedSize));
+        ASSERT(truncatedSize.hasField(attrName)) << context;
+        auto reportedSize = truncatedSize.getField(attrName).Int();
+        auto expectedSize = test.originalDoc.objsize();
+        ASSERT_EQUALS(reportedSize, expectedSize) << context;
+    }
+
+    // At every level of the modified document, validates that only the fields before & including
+    // the truncation path are present.
+    static void validateTruncationAtPath(const BSONObj& original,
+                                         const BSONObj& modified,
+                                         const TestCase& test,
+                                         const std::string& parentPath,
+                                         size_t level) {
+        static const SimpleBSONElementComparator eltCmp;
+        auto context = "Failed test: {} Path: {}"_format(test.name, parentPath);
+
+        auto& path = test.truncationInfo.path;
+
+        ASSERT_LT(level, path.size());
+
+        BSONObjIterator originalItr(original);
+        BSONObjIterator modifiedItr(modified);
+        bool foundTruncatedElement = false;
+
+        StringData truncatedFieldName = path.at(level).fieldName;
+        bool leaf = (&path.at(level) == &path.back());
+
+        while (originalItr.more() && modifiedItr.more()) {
+            auto originalElement = originalItr.next();
+            auto modifiedElement = modifiedItr.next();
+
+            ASSERT_EQUALS(modifiedElement.fieldNameStringData(),
+                          originalElement.fieldNameStringData())
+                << "{} - mismatched field names {} vs {}"_format(
+                       context,
+                       originalElement.fieldNameStringData(),
+                       modifiedElement.fieldNameStringData());
+
+            if (originalElement.fieldNameStringData() == truncatedFieldName) {
+                foundTruncatedElement = true;
+
+                // if truncatedFieldName is present in the truncated object, but the test expects
+                // it to be a leaf, then it should have been omitted.
+                ASSERT_FALSE(leaf)
+                    << "{} - unexpected field {}"_format(context, truncatedFieldName);
+
+                ASSERT_FALSE(modifiedItr.more())
+                    << "{} - truncation did not stop at field {}"_format(context,
+                                                                         truncatedFieldName);
+
+                ASSERT(modifiedElement.isABSONObj())
+                    << "{} - unexpected leaf element {}"_format(context, truncatedFieldName);
+
+                validateTruncationAtPath(originalElement.Obj(),
+                                         modifiedElement.Obj(),
+                                         test,
+                                         "{}.{}"_format(parentPath, truncatedFieldName),
+                                         level + 1);
+            } else {
+                ASSERT(eltCmp.evaluate(originalElement == modifiedElement))
+                    << context << " - mismatched field values at "
+                    << originalElement.fieldNameStringData();
+            }
+        }
+        if (originalItr.more() && !foundTruncatedElement) {
+            // if the original object has more fields than the modified object, but the truncated
+            // field name is not in the modified object, then it MUST have been an omitted leaf
+            // element.
+            ASSERT(leaf) << "{} - missing truncated field {}"_format(context, truncatedFieldName);
+
+            // The next element in the original object must be the truncated field name
+            auto nextElement = originalItr.next();
+            ASSERT_EQUALS(nextElement.fieldNameStringData(), truncatedFieldName)
+                << "{} - unexpected field {}, expected {}"_format(
+                       context, nextElement.fieldNameStringData(), truncatedFieldName);
+
+            foundTruncatedElement = true;
+        }
+        ASSERT(foundTruncatedElement)
+            << "{} - missing truncated field {}"_format(context, truncatedFieldName);
+    }
+
+    static void validateTruncatedAttr(const TestCase& test, const BSONObj& truncatedAttr) {
+        validateTruncationAtPath(test.originalDoc, truncatedAttr, test, "", 0);
+    }
+};
+
+TEST_F(LogV2JsonTruncationTest, JsonTruncation) {
+    auto lines = makeLineCapture(JSONFormatter());
+
+    for (auto& test : generateTests()) {
+        LOGV2(20085, "message", "name"_attr = test.originalDoc, "attr2"_attr = true);
+        auto logObj = fromjson(lines->back());
+        auto loggedAttr =
+            logObj.getField(constants::kAttributesFieldName).Obj().getField("name").Obj();
+
+        // Check that all fields up until the large one is written
+        validateTruncatedAttr(test, loggedAttr);
+
+        auto report = logObj.getField(constants::kTruncatedFieldName).Obj();
+        validateTruncationReport("name", report, test);
+
+        auto size = logObj.getField(constants::kTruncatedSizeFieldName).Obj();
+        validateTruncationSize("name", size, test);
+
+        // Attributes coming after the truncated one should be written
+        ASSERT(logObj.getField(constants::kAttributesFieldName).Obj().getField("attr2").Bool());
+    }
+}
+
+TEST_F(LogV2JsonTruncationTest, JsonTruncationDisabled) {
+    auto lines = makeLineCapture(JSONFormatter());
+
+    for (auto& test : generateTests()) {
+        LOGV2_OPTIONS(20086, {LogTruncation::Disabled}, "message", "name"_attr = test.originalDoc);
+        auto logObj = fromjson(lines->back());
+        auto loggedAttr =
+            logObj.getField(constants::kAttributesFieldName).Obj().getField("name").Obj();
+
+        ASSERT_EQUALS(loggedAttr.woCompare(test.originalDoc), 0);
+        ASSERT_FALSE(logObj.hasField(constants::kTruncatedFieldName));
+        ASSERT_FALSE(logObj.hasField(constants::kTruncatedSizeFieldName));
+    }
 }
 
 TEST_F(LogV2Test, StringTruncation) {
@@ -1952,6 +2347,13 @@ TEST_F(LogV2Test, UserAssert) {
                              });
 }
 
+
+TEST_F(LogV2Test, EmptyBSONElement) {
+    BSONObj doc = BSON("id" << 123);
+    ASSERT_DOES_NOT_THROW(
+        LOGV2(9189100, "Should not throw", "elem"_attr = doc["nonexistentField"]));
+}
+
 class UnstructuredLoggingTest : public LogV2JsonBsonTest {};
 
 TEST_F(UnstructuredLoggingTest, NoArgs) {
