@@ -189,6 +189,29 @@
  *    since the MSVC compiler ignores attributes it does not recognize.
  *
  *
+ * MONGO_COMPILER_USED
+ *
+ *    Do not optimize the function, static variable, or class template static data member, even if
+ *    it is unused. Expands to `[[gnu::used]]` on GCC and Clang, and is ignored on MSVC.
+ *
+ *    Example:
+ *        namespace {
+ *        MONGO_COMPILER_USED int64_t locaInteger = 8675309;
+ *        MONGO_COMPILER_USED void localFunction() {}
+ *        template <typename T>
+ *        struct someTemplatedStruct {
+ *            MONGO_COMPILER_USED static inline int32_t classStaticInteger = 24601;
+ *            MONGO_COMPILER_USED static constexpr auto classStaticString = "Unused string.";
+ *        };
+ *        }  // namespace
+ *
+ *    See:
+ *    -
+ * https://gcc.gnu.org/onlinedocs/gcc/Common-Variable-Attributes.html#index-used-variable-attribute
+ *    -
+ * https://gcc.gnu.org/onlinedocs/gcc/Common-Function-Attributes.html#index-used-function-attribute
+ *
+ *
  * MONGO_GSL_POINTER
  *
  *    Hints to the compiler that this type is a gsl::Pointer type,
@@ -199,9 +222,15 @@
 
 
 #if defined(_MSC_VER)
-#include "mongo/platform/compiler_msvc.h"
+#include "mongo/platform/compiler_msvc.h"  // IWYU pragma: export
 #elif defined(__GNUC__)
-#include "mongo/platform/compiler_gcc.h"
+#include "mongo/platform/compiler_gcc.h"  // IWYU pragma: export
 #else
 #error "Unsupported compiler family"
 #endif
+
+// Define clang's has_feature macro for other compilers
+// See https://clang.llvm.org/docs/LanguageExtensions.html#has-feature-and-has-extension
+#if !defined(__has_feature)
+#define __has_feature(x) 0
+#endif
