@@ -30,14 +30,17 @@
 #include "mongo/platform/decimal128.h"
 
 #include <array>
+#include <cfloat>
 #include <cmath>
 #include <fmt/format.h>
+#include <limits>
 #include <memory>
 #include <string>
-#include <utility>
 
-#include "mongo/config.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/config.h"  // IWYU pragma: keep
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/str.h"
 
 namespace mongo {
 
@@ -294,6 +297,14 @@ TEST(Decimal128Test, TestStringConstructorNaN) {
     ASSERT_EQUALS(val.low64, lowBytes);
 }
 
+TEST(Decimal128Test, TestStringConstructorInvalidCases) {
+    for (auto in : {"", "-", "+", ".", "e", "1e", "1e-", "1e+", ".e", ".e-", ".e+"}) {
+        uint32_t flags = 0;
+        Decimal128{in, &flags};
+        ASSERT(Decimal128::hasFlag(flags, Decimal128::SignalingFlag::kInvalid)) << "in=" << in;
+    }
+}
+
 TEST(Decimal128Test, TestLiteral) {
 #define ASSERT_LITERAL128(x) ASSERT_TRUE((x##_dec128).isBinaryEqual(Decimal128(#x)))
     ASSERT_LITERAL128(5);
