@@ -27,13 +27,12 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
+#include <memory>
+#include <string_view>
 
 #include "mongo/platform/visibility_test_lib1.h"
 #include "mongo/util/exit_code.h"
 
-#include <cstdlib>
-
 int main(int argc, char* argv[]) {
     mongo::visibility_test_lib1::Base b("hello");
     return (b.name() == "hello") ? static_cast<int>(mongo::ExitCode::clean)
