@@ -28,23 +28,33 @@
  */
 
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/rpc/metadata/client_metadata.h"
-
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <cstdint>
+#include <mutex>
 #include <string>
 
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
 #include "mongo/base/status.h"
 #include "mongo/base/status_with.h"
 #include "mongo/base/string_data.h"
 #include "mongo/bson/bsonobj.h"
 #include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/db/client.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/logv2/log.h"
-#include "mongo/s/is_mongos.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/platform/compiler.h"
+#include "mongo/platform/process_id.h"
+#include "mongo/rpc/metadata/client_metadata.h"
 #include "mongo/transport/message_compressor_base.h"
 #include "mongo/transport/message_compressor_manager.h"
-#include "mongo/util/debug_util.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/decorable.h"
 #include "mongo/util/net/socket_utils.h"
 #include "mongo/util/processinfo.h"
 #include "mongo/util/str.h"
@@ -104,7 +114,7 @@ StatusWith<boost::optional<ClientMetadata>> ClientMetadata::parse(const BSONElem
 
 ClientMetadata::ClientMetadata(BSONObj doc) {
     uint32_t maxLength = kMaxMongoDMetadataDocumentByteLength;
-    if (isMongos()) {
+    if (serverGlobalParams.clusterRole.hasExclusively(ClusterRole::RouterServer)) {
         maxLength = kMaxMongoSMetadataDocumentByteLength;
     }
 
@@ -385,7 +395,7 @@ const BSONObj& ClientMetadata::getDocument() const {
 }
 
 unsigned long ClientMetadata::hashWithoutMongosInfo() const {
-    return _hashWithoutMongos.get(documentWithoutMongosInfo());
+    return static_cast<unsigned long>(_hashWithoutMongos.get(documentWithoutMongosInfo()));
 }
 
 const BSONObj& ClientMetadata::documentWithoutMongosInfo() const {
@@ -397,9 +407,13 @@ void ClientMetadata::logClientMetadata(Client* client) const {
         return;
     }
 
+    if (serverGlobalParams.quiet.load()) {
+        return;
+    }
+
     auto negotiatedCompressors =
         MessageCompressorManager::forSession(client->session()).getNegotiatedCompressors();
-    std::vector<StringData> negotiatedCompressorNames(negotiatedCompressors.size(), nullptr);
+    std::vector<StringData> negotiatedCompressorNames(negotiatedCompressors.size());
     std::transform(
         negotiatedCompressors.begin(),
         negotiatedCompressors.end(),
@@ -474,7 +488,7 @@ void ClientMetadata::setAndFinalize(Client* client, boost::optional<ClientMetada
     state.meta = std::move(meta);
 }
 
-void ClientMetadata::setFromMetadataForOperation(OperationContext* opCtx, BSONElement& elem) {
+void ClientMetadata::setFromMetadataForOperation(OperationContext* opCtx, const BSONElement& elem) {
     if (MONGO_unlikely(elem.eoo())) {
         return;
     }
@@ -513,9 +527,9 @@ void ClientMetadata::setFromMetadata(Client* client, BSONElement& elem, bool isI
                     kMaxMongoSMetadataDocumentByteLength);
     }
 
-    if (meta && isMongos()) {
+    if (meta && serverGlobalParams.clusterRole.hasExclusively(ClusterRole::RouterServer)) {
         // If we had a full ClientMetadata and we're on mongos, attach some additional client data.
-        meta->setMongoSMetadata(getHostNameCachedAndPort(),
+        meta->setMongoSMetadata(prettyHostNameAndPort(client->getLocalPort()),
                                 client->clientAddress(true),
                                 VersionInfoInterface::instance().version());
     }
@@ -524,7 +538,7 @@ void ClientMetadata::setFromMetadata(Client* client, BSONElement& elem, bool isI
     state.meta = std::move(meta);
 }
 
-boost::optional<ClientMetadata> ClientMetadata::readFromMetadata(BSONElement& element) {
+boost::optional<ClientMetadata> ClientMetadata::readFromMetadata(const BSONElement& element) {
     return uassertStatusOK(ClientMetadata::parse(element));
 }
 
