@@ -27,20 +27,31 @@
  *    it in the license file.
  */
 
-#include "mongo/platform/basic.h"
-
+#include <cstddef>
+#include <initializer_list>
+#include <memory>
 #include <utility>
+#include <vector>
 
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
 #include "mongo/bson/json.h"
-#include "mongo/db/dbmessage.h"
-#include "mongo/db/jsobj.h"
+#include "mongo/db/tenant_id.h"
 #include "mongo/rpc/metadata.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/stdx/type_traits.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/bson_test_util.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/shared_buffer.h"
 
 namespace {
 using namespace mongo;
 using namespace mongo::rpc;
-using mongo::unittest::assertGet;
 
 BSONObj addDollarDB(BSONObj command, StringData db) {
     return BSONObjBuilder(std::move(command)).append("$db", db).obj();
@@ -50,7 +61,9 @@ void checkUpconvert(const BSONObj& legacyCommand,
                     const int legacyQueryFlags,
                     BSONObj upconvertedCommand) {
     upconvertedCommand = addDollarDB(std::move(upconvertedCommand), "db");
-    auto converted = upconvertRequest({boost::none, "db"}, legacyCommand, legacyQueryFlags);
+    auto converted = upconvertRequest(DatabaseName::createDatabaseName_forTest(boost::none, "db"),
+                                      legacyCommand,
+                                      legacyQueryFlags);
 
     // We don't care about the order of the fields in the metadata object
     const auto sorted = [](const BSONObj& obj) {
@@ -99,12 +112,15 @@ TEST(Metadata, UpconvertDuplicateReadPreference) {
     bob.append("$queryOptions", BSON("$readPreference" << secondaryReadPref));
     bob.append("$readPreference", nearestReadPref);
 
-    ASSERT_THROWS_CODE(rpc::upconvertRequest({boost::none, "db"}, bob.obj(), 0),
-                       AssertionException,
-                       ErrorCodes::InvalidOptions);
+    ASSERT_THROWS_CODE(
+        rpc::upconvertRequest(
+            DatabaseName::createDatabaseName_forTest(boost::none, "db"), bob.obj(), 0),
+        AssertionException,
+        ErrorCodes::InvalidOptions);
 }
 
 TEST(Metadata, UpconvertOnlyExtractsReadPreference) {
+    const auto dbName = DatabaseName::createDatabaseName_forTest(boost::none, "db");
     auto legacyCmdObj = fromjson(
         "{aggregate: 'someCollection', $queryOptions: {someRandomField: 'Hello, world!'}}");
     auto expectedCmdObj = fromjson("{aggregate: 'someCollection'}");
@@ -130,7 +146,8 @@ TEST(Metadata, UpconvertUsesDocumentSequecesCorrectly) {
     };
 
     for (const auto& cmd : valid) {
-        const auto converted = rpc::upconvertRequest({boost::none, "db"}, cmd, 0);
+        const auto converted = rpc::upconvertRequest(
+            DatabaseName::createDatabaseName_forTest(boost::none, "db"), cmd, 0);
         ASSERT_BSONOBJ_EQ(converted.body, fromjson("{insert: 'coll', $db: 'db'}"));
         ASSERT_EQ(converted.sequences.size(), 1u);
         ASSERT_EQ(converted.sequences[0].name, "documents");
@@ -152,7 +169,8 @@ TEST(Metadata, UpconvertUsesDocumentSequecesCorrectly) {
     }
 
     for (const auto& cmd : invalid) {
-        const auto converted = rpc::upconvertRequest({boost::none, "db"}, cmd, 0);
+        const auto converted = rpc::upconvertRequest(
+            DatabaseName::createDatabaseName_forTest(boost::none, "db"), cmd, 0);
         ASSERT_BSONOBJ_EQ(converted.body, addDollarDB(cmd, "db"));
         ASSERT_EQ(converted.sequences.size(), 0u);
     }
