@@ -28,24 +28,38 @@
  */
 
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/rpc/op_msg.h"
-
 #include <bitset>
+#include <boost/cstdint.hpp>
+#include <fmt/format.h>
+#include <memory>
 #include <set>
 
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
+
 #include "mongo/base/data_type_endian.h"
-#include "mongo/config.h"
-#include "mongo/db/auth/security_token_gen.h"
+#include "mongo/base/data_type_validated.h"
+#include "mongo/base/data_view.h"
+#include "mongo/base/error_codes.h"
+#include "mongo/config.h"  // IWYU pragma: keep
 #include "mongo/db/bson/dotted_path_support.h"
+#include "mongo/db/feature_flag.h"
 #include "mongo/db/multitenancy_gen.h"
 #include "mongo/db/server_feature_flags_gen.h"
+#include "mongo/db/server_options.h"
+#include "mongo/db/tenant_id.h"
 #include "mongo/logv2/log.h"
-#include "mongo/rpc/object_check.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/redaction.h"
+#include "mongo/rpc/object_check.h"  // IWYU pragma: keep
+#include "mongo/rpc/op_msg.h"
 #include "mongo/util/bufreader.h"
 #include "mongo/util/database_name_util.h"
+#include "mongo/util/debug_util.h"
 #include "mongo/util/hex.h"
+#include "mongo/util/str.h"
 
 #ifdef MONGO_CONFIG_WIREDTIGER_ENABLED
 #include <wiredtiger.h>
@@ -73,8 +87,8 @@ enum class Section : uint8_t {
 constexpr int kCrc32Size = 4;
 
 #ifdef MONGO_CONFIG_WIREDTIGER_ENABLED
-// All fields including size, requestId, and responseTo must already be set. The size must already
-// include the final 4-byte checksum.
+// All fields including size, requestId, and responseTo must already be set. The size must
+// already include the final 4-byte checksum.
 uint32_t calculateChecksum(const Message& message) {
     if (message.operation() != dbMsg) {
         return 0;
@@ -158,10 +172,11 @@ OpMsg OpMsg::parse(const Message& message, Client* client) try {
     // The sections begin after the flags and before the checksum (if present).
     BufReader sectionsBuf(message.singleData().data() + sizeof(flags), dataSize);
 
-    // TODO some validation may make more sense in the IDL parser. I've tagged them with comments.
+    // TODO some validation may make more sense in the IDL parser. I've tagged them with
+    // comments.
     bool haveBody = false;
     OpMsg msg;
-    BSONObj securityToken;
+    StringData securityToken;
     while (!sectionsBuf.atEof()) {
         const auto sectionKind = sectionsBuf.read<Section>();
         switch (sectionKind) {
@@ -169,17 +184,14 @@ OpMsg OpMsg::parse(const Message& message, Client* client) try {
                 uassert(40430, "Multiple body sections in message", !haveBody);
                 haveBody = true;
                 msg.body = sectionsBuf.read<Validated<BSONObj>>();
-
-                uassert(ErrorCodes::InvalidOptions,
-                        "Multitenancy not enabled, cannot set $tenant in command body",
-                        gMultitenancySupport || !msg.body["$tenant"_sd]);
                 break;
             }
 
             case Section::kDocSequence: {
-                // We use an O(N^2) algorithm here and an O(N*M) algorithm below. These are fastest
-                // for the current small values of N, but would be problematic if it is large.
-                // If we need more document sequences, raise the limit and use a better algorithm.
+                // We use an O(N^2) algorithm here and an O(N*M) algorithm below. These are
+                // fastest for the current small values of N, but would be problematic if it is
+                // large. If we need more document sequences, raise the limit and use a better
+                // algorithm.
                 uassert(ErrorCodes::TooManyDocumentSequences,
                         "Too many document sequences in OP_MSG",
                         msg.sequences.size() < 2);  // Limit is <=2 since we are about to add one.
@@ -201,10 +213,12 @@ OpMsg OpMsg::parse(const Message& message, Client* client) try {
             }
 
             case Section::kSecurityToken: {
+                // if op_msg is parsed by mongoBridge, bridge has a backup check since multitenancy
+                // should be false
                 uassert(ErrorCodes::Unauthorized,
                         "Unsupported Security Token provided",
-                        gMultitenancySupport);
-                securityToken = sectionsBuf.read<Validated<BSONObj>>();
+                        gMultitenancySupport || serverGlobalParams.isMongoBridge);
+                securityToken = sectionsBuf.readCStr();
                 break;
             }
 
@@ -237,7 +251,7 @@ OpMsg OpMsg::parse(const Message& message, Client* client) try {
 #endif
     if (gMultitenancySupport) {
         msg.validatedTenancyScope =
-            auth::ValidatedTenancyScope::create(client, msg.body, securityToken);
+            auth::ValidatedTenancyScopeFactory::parse(client, securityToken);
     }
 
     return msg;
@@ -248,105 +262,91 @@ OpMsg OpMsg::parse(const Message& message, Client* client) try {
         "invalid message: {ex_code} {ex} -- {hexdump_message_singleData_view2ptr_message_size}",
         "ex_code"_attr = ex.code(),
         "ex"_attr = redact(ex),
+        // Using std::min to reduce the size of the output and ensure we do not throw in hexdump()
+        // because of the exceeded length.
         "hexdump_message_singleData_view2ptr_message_size"_attr =
-            redact(hexdump(message.singleData().view2ptr(), message.size())));
+            redact(hexdump(message.singleData().view2ptr(),
+                           std::min(static_cast<size_t>(message.size()), kHexDumpMaxSize - 1))));
     throw;
 }
 
-OpMsgRequest OpMsgRequest::fromDBAndBody(StringData db, BSONObj body, const BSONObj& extraFields) {
-    return OpMsgRequestBuilder::create({boost::none, db}, std::move(body), extraFields);
+DatabaseName OpMsgRequest::parseDbName() const {
+    auto elem = body["$db"];
+    uassert(40571, "OP_MSG requests require a $db argument", !elem.eoo());
+
+    auto dbName = elem.checkAndGetStringData();
+    return DatabaseNameUtil::deserialize(getValidatedTenantId(), dbName, getSerializationContext());
 }
 
-boost::optional<TenantId> parseDollarTenant(const BSONObj body) {
-    if (auto tenant = body.getField("$tenant")) {
-        return TenantId::parseFromBSON(tenant);
-    } else {
-        return boost::none;
+SerializationContext OpMsgRequest::getSerializationContext() const {
+    if (!gMultitenancySupport) {
+        return SerializationContext::stateDefault();
     }
-}
+    auto serializationCtx = SerializationContext::stateCommandRequest();
 
-bool appendDollarTenant(BSONObjBuilder& builder,
-                        const TenantId& tenant,
-                        boost::optional<TenantId> existingDollarTenant = boost::none) {
-    if (existingDollarTenant) {
-        massert(8423373,
-                str::stream() << "Unable to set TenantId '" << tenant
-                              << "' on OpMsgRequest as it already has "
-                              << existingDollarTenant->toString(),
-                tenant == existingDollarTenant.value());
-        return true;
+    if (auto const expectPrefix = body.getField("expectPrefix")) {
+        serializationCtx.setPrefixState(expectPrefix.boolean());
+    } else if (validatedTenancyScope) {
+        serializationCtx.setPrefixState(validatedTenancyScope->isFromAtlasProxy());
     }
+    return serializationCtx;
+}
 
-    const auto fcvSnapshot = serverGlobalParams.featureCompatibility.acquireFCVSnapshot();
-    if (gMultitenancySupport) {
-        if (fcvSnapshot.isVersionInitialized() &&
-            gFeatureFlagRequireTenantID.isEnabled(fcvSnapshot)) {
-            tenant.serializeToBSON("$tenant", &builder);
-            return true;
-        }
+namespace {
+std::string compactStr(const std::string& input) {
+    if (input.length() > 2024) {
+        return input.substr(0, 1000) + " ... " + input.substr(input.length() - 1000);
     }
-    return false;
+    return input;
+}
+}  // namespace
+void validateExtraFields(const DatabaseName& dbName,
+                         const BSONObj& body,
+                         const BSONObj& extraFields) {
+    int bodySize = body.objsize();
+    int extraFieldsSize = extraFields.objsize();
+
+    // Log a warning if the sum of the sizes of 'body' and 'extraFields' exceeds
+    // 'BSONObjMaxInternalSize'.
+    if (bodySize + extraFieldsSize > BSONObjMaxInternalSize) {
+        LOGV2_WARNING(
+            6491800,
+            "Request body exceeded limit with body.objsize() = {bodySize} bytes, "
+            "extraFields.objsize() = {extraFieldsSize} bytes, body.toString() = {body}, db = "
+            "{db}, extraFields.toString() = {extraFields}",
+            "bodySize"_attr = bodySize,
+            "extraFieldsSize"_attr = extraFieldsSize,
+            "body"_attr = compactStr(body.toString()),
+            "db"_attr = dbName.toStringForErrorMsg(),
+            "extraFields"_attr = compactStr(extraFields.toString()));
+    };
 }
 
-void appendDollarDbAndTenant(BSONObjBuilder& builder,
-                             const DatabaseName& dbName,
-                             boost::optional<TenantId> existingDollarTenant = boost::none) {
-    if (!dbName.tenantId() ||
-        appendDollarTenant(builder, dbName.tenantId().value(), existingDollarTenant)) {
-        builder.append("$db", dbName.db());
-    } else {
-        builder.append("$db", DatabaseNameUtil::serialize(dbName));
-    }
-}
-
-void OpMsgRequest::setDollarTenant(const TenantId& tenant) {
-    massert(8423372,
-            str::stream() << "Should not set dollar tenant " << tenant
-                          << " on the validated OpMsgRequest.",
-            !validatedTenancyScope);
-
-    auto dollarTenant = parseDollarTenant(body);
-    BSONObjBuilder bodyBuilder(std::move(body));
-    appendDollarTenant(bodyBuilder, tenant, dollarTenant);
-    body = bodyBuilder.obj();
-}
-
-OpMsgRequest OpMsgRequestBuilder::createWithValidatedTenancyScope(
-    const DatabaseName& dbName,
+OpMsgRequest OpMsgRequestBuilder::create(
     boost::optional<auth::ValidatedTenancyScope> validatedTenancyScope,
+    const DatabaseName& dbName,
     BSONObj body,
     const BSONObj& extraFields) {
-    auto dollarTenant = parseDollarTenant(body);
-    OpMsgRequest request;
-    request.body = ([&] {
-        BSONObjBuilder bodyBuilder(std::move(body));
-        bodyBuilder.appendElements(extraFields);
-        if (dollarTenant) {
-            appendDollarDbAndTenant(bodyBuilder, dbName, dollarTenant);
-        } else if (validatedTenancyScope && !validatedTenancyScope->hasAuthenticatedUser()) {
-            // Add $tenant into the body if the validated tenant id comes from $tenant.
-            appendDollarDbAndTenant(bodyBuilder, dbName);
-        } else {
-            bodyBuilder.append("$db", DatabaseNameUtil::serialize(dbName));
-        }
-        return bodyBuilder.obj();
-    }());
-
-    request.validatedTenancyScope = validatedTenancyScope;
-    return request;
-}
-
-OpMsgRequest OpMsgRequestBuilder::create(const DatabaseName& dbName,
-                                         BSONObj body,
-                                         const BSONObj& extraFields) {
-    auto dollarTenant = parseDollarTenant(body);
-    BSONObjBuilder bodyBuilder(std::move(body));
-    bodyBuilder.appendElements(extraFields);
-
-    appendDollarDbAndTenant(bodyBuilder, dbName, dollarTenant);
+    validateExtraFields(dbName, body, extraFields);
 
     OpMsgRequest request;
-    request.body = bodyBuilder.obj();
+    const bool hasValidVts = validatedTenancyScope && validatedTenancyScope->isValid();
+    request.body = [&]() {
+        BSONObjBuilder builder(std::move(body));
+        builder.appendElements(extraFields);
+
+        const SerializationContext sc = hasValidVts
+            ? SerializationContext::stateCommandRequest(validatedTenancyScope->hasTenantId(),
+                                                        validatedTenancyScope->isFromAtlasProxy())
+            : SerializationContext::stateCommandRequest();
+        builder.append("$db", DatabaseNameUtil::serialize(dbName, sc));
+        return builder.obj();
+    }();
+
+    if (hasValidVts) {
+        request.validatedTenancyScope = std::move(validatedTenancyScope);
+    }
+
     return request;
 }
 
@@ -357,7 +357,7 @@ void serializeHelper(const std::vector<OpMsg::DocumentSequence>& sequences,
                      OpMsgBuilder* output) {
     if (validatedTenancyScope) {
         auto securityToken = validatedTenancyScope->getOriginalToken();
-        if (securityToken.nFields() > 0) {
+        if (!securityToken.empty()) {
             output->setSecurityToken(securityToken);
         }
     }
@@ -396,15 +396,14 @@ void OpMsg::shareOwnershipWith(const ConstSharedBuffer& buffer) {
     }
 }
 
-BSONObjBuilder OpMsgBuilder::beginSecurityToken() {
+void OpMsgBuilder::setSecurityToken(StringData token) {
     invariant(_state == kEmpty);
-    _state = kSecurityToken;
     _buf.appendStruct(Section::kSecurityToken);
-    return BSONObjBuilder(_buf);
+    _buf.appendCStr(token);
 }
 
 auto OpMsgBuilder::beginDocSequence(StringData name) -> DocSequenceBuilder {
-    invariant((_state == kEmpty) || (_state == kSecurityToken) || (_state == kDocSequence));
+    invariant((_state == kEmpty) || (_state == kDocSequence));
     invariant(!_openBuilder);
     _openBuilder = true;
     _state = kDocSequence;
@@ -425,7 +424,7 @@ void OpMsgBuilder::finishDocumentStream(DocSequenceBuilder* docSequenceBuilder)
 }
 
 BSONObjBuilder OpMsgBuilder::beginBody() {
-    invariant((_state == kEmpty) || (_state == kSecurityToken) || (_state == kDocSequence));
+    invariant((_state == kEmpty) || (_state == kDocSequence));
     _state = kBody;
     _buf.appendStruct(Section::kBody);
     invariant(_bodyStart == 0);
