@@ -28,26 +28,56 @@
  */
 
 
-#include "mongo/rpc/op_msg_test.h"
+#include "mongo/util/serialization_context.h"
+#include <algorithm>
+#include <boost/optional.hpp>
+#include <memory>
+#include <set>
+#include <string>
+#include <vector>
 
-#include "mongo/platform/basic.h"
+#include <absl/container/node_hash_map.h>
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional/optional.hpp>
 
-#include "mongo/base/static_assert.h"
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
 #include "mongo/bson/json.h"
+#include "mongo/bson/oid.h"
+#include "mongo/db/auth/action_type.h"
+#include "mongo/db/auth/auth_name.h"
+#include "mongo/db/auth/authorization_manager.h"
 #include "mongo/db/auth/authorization_manager_impl.h"
 #include "mongo/db/auth/authorization_session.h"
 #include "mongo/db/auth/authorization_session_impl.h"
 #include "mongo/db/auth/authz_manager_external_state_mock.h"
-#include "mongo/db/auth/security_token_gen.h"
+#include "mongo/db/auth/privilege.h"
+#include "mongo/db/auth/resource_pattern.h"
+#include "mongo/db/auth/role_name.h"
+#include "mongo/db/auth/user.h"
+#include "mongo/db/auth/user_name.h"
 #include "mongo/db/auth/validated_tenancy_scope.h"
-#include "mongo/db/jsobj.h"
-#include "mongo/db/multitenancy_gen.h"
+#include "mongo/db/client.h"
+#include "mongo/db/database_name.h"
+#include "mongo/db/service_context.h"
 #include "mongo/db/service_context_test_fixture.h"
+#include "mongo/db/tenant_id.h"
 #include "mongo/idl/server_parameter_test_util.h"
 #include "mongo/logv2/log.h"
-#include "mongo/unittest/death_test.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/log_severity.h"
+#include "mongo/rpc/op_msg_test.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/bson_test_util.h"
+#include "mongo/unittest/framework.h"
 #include "mongo/unittest/log_test.h"
-#include "mongo/unittest/unittest.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/hex.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
@@ -63,7 +93,7 @@ public:
     static void grantUseTenant(Client& client) {
         User user(UserRequest(UserName("useTenant", "admin"), boost::none));
         user.setPrivileges(
-            {Privilege(ResourcePattern::forClusterResource(), ActionType::useTenant)});
+            {Privilege(ResourcePattern::forClusterResource(boost::none), ActionType::useTenant)});
         auto* as = dynamic_cast<AuthorizationSessionImpl*>(AuthorizationSession::get(client));
         if (as->_authenticatedUser != boost::none) {
             as->logoutAllDatabases(&client, "AuthorizationSessionImplTestHelper"_sd);
@@ -798,20 +828,11 @@ protected:
     void setUp() final {
         auto authzManagerState = std::make_unique<AuthzManagerExternalStateMock>();
         auto authzManager = std::make_unique<AuthorizationManagerImpl>(
-            getServiceContext(), std::move(authzManagerState));
+            getServiceContext()->getService(), std::move(authzManagerState));
         authzManager->setAuthEnabled(true);
-        AuthorizationManager::set(getServiceContext(), std::move(authzManager));
+        AuthorizationManager::set(getService(), std::move(authzManager));
 
-        client = getServiceContext()->makeClient("test");
-    }
-
-    BSONObj makeSecurityToken(const UserName& userName) {
-        constexpr auto authUserFieldName = auth::SecurityToken::kAuthenticatedUserFieldName;
-        auto authUser = userName.toBSON(true /* serialize token */);
-        ASSERT_EQ(authUser["tenant"_sd].type(), jstOID);
-        using VTS = auth::ValidatedTenancyScope;
-        return VTS(BSON(authUserFieldName << authUser), VTS::TokenForTestingTag{})
-            .getOriginalToken();
+        client = getServiceContext()->getService()->makeClient("test");
     }
 
     ServiceContext::UniqueClient client;
@@ -820,9 +841,17 @@ protected:
 TEST_F(OpMsgWithAuth, ParseValidatedTenancyScopeFromSecurityToken) {
     RAIIServerParameterControllerForTest multitenancyController("multitenancySupport", true);
     RAIIServerParameterControllerForTest securityTokenController("featureFlagSecurityToken", true);
+    RAIIServerParameterControllerForTest secretController("testOnlyValidatedTenancyScopeKey",
+                                                          "secret");
 
     const auto kTenantId = TenantId(OID::gen());
-    const auto token = makeSecurityToken(UserName("user", "admin", kTenantId));
+    const auto token = auth::ValidatedTenancyScopeFactory::create(
+                           UserName("user", "admin", kTenantId),
+                           "secret"_sd,
+                           auth::ValidatedTenancyScope::TenantProtocol::kDefault,
+                           auth::ValidatedTenancyScopeFactory::TokenForTestingTag{})
+                           .getOriginalToken()
+                           .toString();
     auto msg =
         OpMsgBytes{
             kNoFlags,  //
@@ -847,41 +876,25 @@ TEST_F(OpMsgWithAuth, ParseValidatedTenancyScopeFromSecurityToken) {
     ASSERT_EQ(msg.validatedTenancyScope->tenantId(), kTenantId);
 }
 
-TEST_F(OpMsgWithAuth, ParseValidatedTenancyScopeFromDollarTenant) {
-    RAIIServerParameterControllerForTest multitenancyController("multitenancySupport", true);
-    AuthorizationSessionImplTestHelper::grantUseTenant(*(client.get()));
-
-    const auto kTenantId = TenantId(OID::gen());
-    const auto body = BSON("ping" << 1 << "$tenant" << kTenantId);
-    auto msg =
-        OpMsgBytes{
-            kNoFlags,  //
-            kBodySection,
-            body,
-
-            kDocSequenceSection,
-            Sized{
-                "docs",  //
-                fromjson("{a: 1}"),
-                fromjson("{a: 2}"),
-            },
-        }
-            .parse(client.get());
-
-    ASSERT(msg.validatedTenancyScope);
-    ASSERT_EQ(msg.validatedTenancyScope->tenantId(), kTenantId);
-}
-
 TEST_F(OpMsgWithAuth, ValidatedTenancyScopeShouldNotBeSerialized) {
     RAIIServerParameterControllerForTest multitenancyController("multitenancySupport", true);
+    RAIIServerParameterControllerForTest securityTokenController("featureFlagSecurityToken", true);
     AuthorizationSessionImplTestHelper::grantUseTenant(*(client.get()));
 
     const auto kTenantId = TenantId(OID::gen());
-    const auto body = BSON("ping" << 1 << "$tenant" << kTenantId);
+
+    const auto token = auth::ValidatedTenancyScopeFactory::create(
+                           kTenantId,
+                           auth::ValidatedTenancyScope::TenantProtocol::kAtlasProxy,
+                           auth::ValidatedTenancyScopeFactory::TenantForTestingTag{})
+                           .getOriginalToken()
+                           .toString();
+
+    const auto body = BSON("ping" << 1);
     auto msgBytes = OpMsgBytes{
         kNoFlags,  //
-        kBodySection,
-        body,
+        kSecurityTokenSection,
+        token,
 
         kDocSequenceSection,
         Sized{
@@ -889,6 +902,8 @@ TEST_F(OpMsgWithAuth, ValidatedTenancyScopeShouldNotBeSerialized) {
             fromjson("{a: 1}"),
             fromjson("{a: 2}"),
         },
+        kBodySection,
+        body,
     };
     auto msg = msgBytes.parse(client.get());
     ASSERT(msg.validatedTenancyScope);
@@ -897,6 +912,8 @@ TEST_F(OpMsgWithAuth, ValidatedTenancyScopeShouldNotBeSerialized) {
     testSerializer(serializedMsg,
                    OpMsgBytes{
                        kNoFlags,  //
+                       kSecurityTokenSection,
+                       token,
 
                        kDocSequenceSection,
                        Sized{
@@ -904,7 +921,6 @@ TEST_F(OpMsgWithAuth, ValidatedTenancyScopeShouldNotBeSerialized) {
                            fromjson("{a: 1}"),
                            fromjson("{a: 2}"),
                        },
-
                        kBodySection,
                        body,
                    });
@@ -913,174 +929,261 @@ TEST_F(OpMsgWithAuth, ValidatedTenancyScopeShouldNotBeSerialized) {
 TEST(OpMsgRequest, GetDatabaseWorks) {
     OpMsgRequest msg;
     msg.body = fromjson("{$db: 'foo'}");
-    ASSERT_EQ(msg.getDatabase(), "foo");
+    ASSERT_EQ(msg.parseDbName().toString_forTest(), "foo");
 
     msg.body = fromjson("{before: 1, $db: 'foo'}");
-    ASSERT_EQ(msg.getDatabase(), "foo");
+    ASSERT_EQ(msg.parseDbName().toString_forTest(), "foo");
 
     msg.body = fromjson("{before: 1, $db: 'foo', after: 1}");
-    ASSERT_EQ(msg.getDatabase(), "foo");
+    ASSERT_EQ(msg.parseDbName().toString_forTest(), "foo");
 }
 
 TEST(OpMsgRequest, GetDatabaseThrowsWrongType) {
     OpMsgRequest msg;
     msg.body = fromjson("{$db: 1}");
-    ASSERT_THROWS(msg.getDatabase(), DBException);
+    ASSERT_THROWS(msg.parseDbName().toString_forTest(), DBException);
 }
 
 TEST(OpMsgRequest, GetDatabaseThrowsMissing) {
     OpMsgRequest msg;
     msg.body = fromjson("{}");
-    ASSERT_THROWS(msg.getDatabase(), AssertionException);
+    ASSERT_THROWS(msg.parseDbName().toString_forTest(), AssertionException);
 
     msg.body = fromjson("{$notdb: 'foo'}");
-    ASSERT_THROWS(msg.getDatabase(), AssertionException);
-}
-
-TEST(OpMsgRequestBuilder, WithTenantInDatabaseName) {
-    RAIIServerParameterControllerForTest multitenancyController("multitenancySupport", true);
-    RAIIServerParameterControllerForTest requireTenantIdController("featureFlagRequireTenantID",
-                                                                   true);
-    const TenantId tenantId(OID::gen());
-    auto const body = fromjson("{ping: 1}");
-    OpMsgRequest msg = OpMsgRequestBuilder::create({tenantId, "testDb"}, body);
-    ASSERT_EQ(msg.body.getField("$tenant").eoo(), false);
-    ASSERT_EQ(TenantId::parseFromBSON(msg.body.getField("$tenant")), tenantId);
-    ASSERT_EQ(msg.getDatabase(), "testDb");
-}
-
-TEST(OpMsgRequestBuilder, WithTenantInDatabaseName_FeatureFlagOff) {
-    RAIIServerParameterControllerForTest multitenancyController("multitenancySupport", true);
-    RAIIServerParameterControllerForTest requireTenantIdController("featureFlagRequireTenantID",
-                                                                   false);
-    const TenantId tenantId(OID::gen());
-    auto const body = fromjson("{ping: 1}");
-    OpMsgRequest msg = OpMsgRequestBuilder::create({tenantId, "testDb"}, body);
-    ASSERT(msg.body.getField("$tenant").eoo());
-    ASSERT_EQ(msg.getDatabase(), tenantId.toString() + "_testDb");
-}
-
-TEST(OpMsgRequestBuilder, WithSameTenantInBody) {
-    RAIIServerParameterControllerForTest multitenancyController("multitenancySupport", true);
-    RAIIServerParameterControllerForTest requireTenantIdController("featureFlagRequireTenantID",
-                                                                   true);
-    const TenantId tenantId(OID::gen());
-    auto const body = BSON("ping" << 1 << "$tenant" << tenantId);
-    OpMsgRequest msg = OpMsgRequestBuilder::create({tenantId, "testDb"}, body);
-    ASSERT_EQ(msg.body.getField("$tenant").eoo(), false);
-    ASSERT_EQ(TenantId::parseFromBSON(msg.body.getField("$tenant")), tenantId);
+    ASSERT_THROWS(msg.parseDbName().toString_forTest(), AssertionException);
 }
 
 TEST(OpMsgRequestBuilder, WithVTS) {
     RAIIServerParameterControllerForTest multitenancyController("multitenancySupport", true);
-    RAIIServerParameterControllerForTest requireTenantIdController("featureFlagRequireTenantID",
-                                                                   true);
+    RAIIServerParameterControllerForTest securityTokenController("featureFlagSecurityToken", true);
+    RAIIServerParameterControllerForTest secretController("testOnlyValidatedTenancyScopeKey",
+                                                          "secret");
+
     const TenantId tenantId(OID::gen());
+    const auto vts = auth::ValidatedTenancyScopeFactory::create(
+        UserName("user", "admin", tenantId),
+        "secret"_sd,
+        auth::ValidatedTenancyScope::TenantProtocol::kDefault,
+        auth::ValidatedTenancyScopeFactory::TokenForTestingTag{});
+
+    const StringData dbString = "testDb";
     auto const body = fromjson("{ping: 1}");
 
-    using VTS = auth::ValidatedTenancyScope;
-    VTS vts = VTS(tenantId, VTS::TenantForTestingTag{});
-    OpMsgRequest msg =
-        OpMsgRequestBuilder::createWithValidatedTenancyScope({tenantId, "testDb"}, vts, body);
+    OpMsgRequest msg = OpMsgRequestBuilder::create(
+        vts, DatabaseName::createDatabaseName_forTest(tenantId, dbString), body);
     ASSERT(msg.validatedTenancyScope);
     ASSERT_EQ(msg.validatedTenancyScope->tenantId(), tenantId);
-    // Verify $tenant is added to the msg body, as the vts does not come from security token.
-    ASSERT_EQ(msg.body.getField("$tenant").eoo(), false);
-    ASSERT_EQ(TenantId::parseFromBSON(msg.body.getField("$tenant")), tenantId);
-    ASSERT_EQ(msg.getDatabase(), "testDb");
+    ASSERT_EQ(msg.parseDbName().toString_forTest(), dbString);
 }
 
-TEST(OpMsgRequestBuilder, FailWithDiffTenantInBody) {
+TEST(OpMsgRequestBuilder, WithVTSAndSerializationContextExpPrefixDefault) {
     RAIIServerParameterControllerForTest multitenancyController("multitenancySupport", true);
-    RAIIServerParameterControllerForTest requireTenantIdController("featureFlagRequireTenantID",
-                                                                   true);
-    const TenantId tenantId(OID::gen());
-    const TenantId otherTenantId(OID::gen());
+    RAIIServerParameterControllerForTest securityTokenController("featureFlagSecurityToken", true);
+    RAIIServerParameterControllerForTest secretController("testOnlyValidatedTenancyScopeKey",
+                                                          "secret");
 
-    auto const body = BSON("ping" << 1 << "$tenant" << tenantId);
-    ASSERT_THROWS_CODE(
-        OpMsgRequestBuilder::create({otherTenantId, "testDb"}, body), DBException, 8423373);
+    const TenantId tenantId(OID::gen());
+    const StringData dbString = "testDb";
+    const std::string dbStringWithTid = str::stream() << tenantId.toString() << "_" << dbString;
+    auto const body = fromjson("{ping: 1}");
+
+    using Prefix = SerializationContext::Prefix;
+
+    auth::ValidatedTenancyScope vts = auth::ValidatedTenancyScopeFactory::create(
+        UserName("user", "admin", tenantId),
+        "secret"_sd,
+        auth::ValidatedTenancyScope::TenantProtocol::kDefault,
+        auth::ValidatedTenancyScopeFactory::TokenForTestingTag{});
+
+    OpMsgRequest msg = OpMsgRequestBuilder::create(
+        vts, DatabaseName::createDatabaseName_forTest(tenantId, dbString), body);
+    ASSERT(msg.validatedTenancyScope);
+    ASSERT_EQ(msg.validatedTenancyScope->tenantId(), tenantId);
+
+    // Missing expectPrefix in the request body.
+    ASSERT_EQ(msg.body.getField("expectPrefix").eoo(), true);
+    ASSERT_EQ(msg.parseDbName().toString_forTest(), dbString);
+}
+
+void CheckVtsSetsPrefix(Client* client, bool simulateAtlasProxyTenantProtocol) {
+    const auto kTenantId = TenantId(OID::gen());
+    const auto token = auth::ValidatedTenancyScopeFactory::create(
+                           UserName("user", "admin", kTenantId),
+                           "secret"_sd,
+                           simulateAtlasProxyTenantProtocol
+                               ? auth::ValidatedTenancyScope::TenantProtocol::kAtlasProxy
+                               : auth::ValidatedTenancyScope::TenantProtocol::kDefault,
+                           auth::ValidatedTenancyScopeFactory::TokenForTestingTag{})
+                           .getOriginalToken()
+                           .toString();
+    auto msg =
+        OpMsgBytes{
+            kNoFlags,  //
+            kBodySection,
+            fromjson("{ping: 1}"),
+
+            kDocSequenceSection,
+            Sized{
+                "docs",  //
+                fromjson("{a: 1}"),
+                fromjson("{a: 2}"),
+            },
+
+            kSecurityTokenSection,
+            token,
+        }
+            .parse(client);
+    auth::ValidatedTenancyScope vts = msg.validatedTenancyScope.value();
+    ASSERT_TRUE(vts.isFromAtlasProxy() == simulateAtlasProxyTenantProtocol);
+
+    auto serializedMsg = msg.serialize();
+    auto request = OpMsgRequest::parse(serializedMsg, client);
+    ASSERT(request.validatedTenancyScope);
+    ASSERT_EQ(request.getValidatedTenantId().value(), vts.tenantId());
+    ASSERT_TRUE(request.validatedTenancyScope->isFromAtlasProxy() ==
+                simulateAtlasProxyTenantProtocol);
+    ASSERT_EQ(request.getSerializationContext().getPrefix(),
+              simulateAtlasProxyTenantProtocol ? SerializationContext::Prefix::IncludePrefix
+                                               : SerializationContext::Prefix::ExcludePrefix);
+}
+
+TEST_F(OpMsgWithAuth, TestVTSSetsPrefixStateFalse) {
+    RAIIServerParameterControllerForTest multitenancyController("multitenancySupport", true);
+    RAIIServerParameterControllerForTest securityTokenController("featureFlagSecurityToken", true);
+    RAIIServerParameterControllerForTest secretController("testOnlyValidatedTenancyScopeKey",
+                                                          "secret");
+
+    CheckVtsSetsPrefix(client.get(), false);
+}
+
+TEST_F(OpMsgWithAuth, TestVTSSetsPrefixStateTrue) {
+    RAIIServerParameterControllerForTest multitenancyController("multitenancySupport", true);
+    RAIIServerParameterControllerForTest securityTokenController("featureFlagSecurityToken", true);
+    RAIIServerParameterControllerForTest secretController("testOnlyValidatedTenancyScopeKey",
+                                                          "secret");
+
+    CheckVtsSetsPrefix(client.get(), true);
+}
+
+void CheckCommandMsgIdlParsingForOpMsgRequest(bool simulateAtlasProxyTenantProtocol) {
+    const TenantId tenantId(OID::gen());
+    const std::string dbString =
+        simulateAtlasProxyTenantProtocol ? (tenantId.toString() + "_testDb") : "testDb";
+    auto cmd = BSON("insert"
+                    << "bar"
+                    << "$db" << dbString << "documents" << BSON_ARRAY(BSONObj()));
+    OpMsgRequest msg;
+    msg.body = cmd;
+    auth::ValidatedTenancyScope vts = auth::ValidatedTenancyScopeFactory::create(
+        tenantId,
+        simulateAtlasProxyTenantProtocol ? auth::ValidatedTenancyScope::TenantProtocol::kAtlasProxy
+                                         : auth::ValidatedTenancyScope::TenantProtocol::kDefault,
+        auth::ValidatedTenancyScopeFactory::TenantForTestingTag{});
+    msg.validatedTenancyScope = vts;
+    auto op = InsertOp::parse(msg);
+    ASSERT_EQ(op.getSerializationContext().getPrefix(),
+              simulateAtlasProxyTenantProtocol ? SerializationContext::Prefix::IncludePrefix
+                                               : SerializationContext::Prefix::ExcludePrefix);
+}
+
+TEST_F(OpMsgWithAuth, TestExpectPrefixTrueParsedInMsg) {
+    RAIIServerParameterControllerForTest multitenancyController("multitenancySupport", true);
+    RAIIServerParameterControllerForTest securityTokenController("featureFlagSecurityToken", true);
+    RAIIServerParameterControllerForTest secretController("testOnlyValidatedTenancyScopeKey",
+                                                          "secret");
+
+    CheckCommandMsgIdlParsingForOpMsgRequest(true);
+}
+
+TEST_F(OpMsgWithAuth, TestExpectPrefixFalseParsedInMsg) {
+    RAIIServerParameterControllerForTest multitenancyController("multitenancySupport", true);
+    RAIIServerParameterControllerForTest securityTokenController("featureFlagSecurityToken", true);
+    RAIIServerParameterControllerForTest secretController("testOnlyValidatedTenancyScopeKey",
+                                                          "secret");
+
+    CheckCommandMsgIdlParsingForOpMsgRequest(false);
+}
+
+TEST(OpMsgRequestBuilder, WithVTSAndSerializationContextExpPrefixFalse) {
+    RAIIServerParameterControllerForTest multitenancyController("multitenancySupport", true);
+    RAIIServerParameterControllerForTest securityTokenController("featureFlagSecurityToken", true);
+    RAIIServerParameterControllerForTest secretController("testOnlyValidatedTenancyScopeKey",
+                                                          "secret");
+
+    const TenantId tenantId(OID::gen());
+    const StringData dbString = "testDb";
+    const std::string dbStringWithTid = str::stream() << tenantId.toString() << "_" << dbString;
+    auto const body = fromjson("{ping: 1}");
+
+    using Prefix = SerializationContext::Prefix;
+
+    auth::ValidatedTenancyScope vts = auth::ValidatedTenancyScopeFactory::create(
+        UserName("user", "admin", tenantId),
+        "secret"_sd,
+        auth::ValidatedTenancyScope::TenantProtocol::kDefault,
+        auth::ValidatedTenancyScopeFactory::TokenForTestingTag{});
+
+    OpMsgRequest msg = OpMsgRequestBuilder::create(
+        vts, DatabaseName::createDatabaseName_forTest(tenantId, dbString), body);
+    ASSERT(msg.validatedTenancyScope);
+    ASSERT_EQ(msg.validatedTenancyScope->tenantId(), tenantId);
+    ASSERT_EQ(msg.parseDbName().toString_forTest(), dbString);
+}
+
+TEST(OpMsgRequestBuilder, WithVTSAndSerializationContextExpPrefixTrue) {
+    RAIIServerParameterControllerForTest multitenancyController("multitenancySupport", true);
+    RAIIServerParameterControllerForTest securityTokenController("featureFlagSecurityToken", true);
+    RAIIServerParameterControllerForTest secretController("testOnlyValidatedTenancyScopeKey",
+                                                          "secret");
+
+    const TenantId tenantId(OID::gen());
+    const StringData dbString = "testDb";
+    const std::string dbStringWithTid = str::stream() << tenantId.toString() << "_" << dbString;
+    auto const body = fromjson("{ping: 1}");
+
+    using Prefix = SerializationContext::Prefix;
+
+    auth::ValidatedTenancyScope vts = auth::ValidatedTenancyScopeFactory::create(
+        UserName("user", "admin", tenantId),
+        "secret"_sd,
+        auth::ValidatedTenancyScope::TenantProtocol::kAtlasProxy,
+        auth::ValidatedTenancyScopeFactory::TokenForTestingTag{});
+
+    OpMsgRequest msg = OpMsgRequestBuilder::create(
+        vts, DatabaseName::createDatabaseName_forTest(tenantId, dbString), body);
+    ASSERT(msg.validatedTenancyScope);
+    ASSERT_EQ(msg.validatedTenancyScope->tenantId(), tenantId);
+    ASSERT_EQ(msg.parseDbName().toStringWithTenantId_forTest(), dbStringWithTid);
 }
 
 TEST(OpMsgRequestBuilder, CreateDoesNotCopy) {
     RAIIServerParameterControllerForTest multitenancyController("multitenancySupport", true);
     RAIIServerParameterControllerForTest requireTenantIdController("featureFlagRequireTenantID",
                                                                    true);
+    RAIIServerParameterControllerForTest securityTokenController("featureFlagSecurityToken", true);
+    RAIIServerParameterControllerForTest secretController("testOnlyValidatedTenancyScopeKey",
+                                                          "secret");
+
+
     const TenantId tenantId(OID::gen());
+
+    auth::ValidatedTenancyScope vts = auth::ValidatedTenancyScopeFactory::create(
+        UserName("user", "admin", tenantId),
+        "secret"_sd,
+        auth::ValidatedTenancyScope::TenantProtocol::kDefault,
+        auth::ValidatedTenancyScopeFactory::TokenForTestingTag{});
+
     auto body = fromjson("{ping: 1}");
     const void* const bodyPtr = body.objdata();
-    auto msg = OpMsgRequestBuilder::create({tenantId, "db"}, std::move(body));
+    auto msg = OpMsgRequestBuilder::create(
+        vts, DatabaseName::createDatabaseName_forTest(tenantId, "db"), std::move(body));
 
-    auto const newBody = BSON("ping" << 1 << "$tenant" << tenantId << "$db"
+    auto const newBody = BSON("ping" << 1 << "$db"
                                      << "db");
     ASSERT_BSONOBJ_EQ(msg.body, newBody);
     ASSERT_EQ(static_cast<const void*>(msg.body.objdata()), bodyPtr);
 }
 
-TEST(OpMsgRequest, FromDbAndBodyDoesNotCopy) {
-    auto body = fromjson("{ping: 1}");
-    const void* const bodyPtr = body.objdata();
-    auto msg = OpMsgRequest::fromDBAndBody("db", std::move(body));
-
-    ASSERT_BSONOBJ_EQ(msg.body, fromjson("{ping: 1, $db: 'db'}"));
-    ASSERT_EQ(static_cast<const void*>(msg.body.objdata()), bodyPtr);
-}
-
-TEST(OpMsgRequest, SetDollarTenantHasSameTenant) {
-    RAIIServerParameterControllerForTest multitenancyController("multitenancySupport", true);
-    RAIIServerParameterControllerForTest requireTenantIdController("featureFlagRequireTenantID",
-                                                                   true);
-    const TenantId tenantId(OID::gen());
-    // Set $tenant on a OpMsgRequest which already has the same $tenant.
-    OpMsgRequest request;
-    request.body = BSON("ping" << 1 << "$tenant" << tenantId << "$db"
-                               << "testDb");
-    request.setDollarTenant(tenantId);
-
-    auto dollarTenant = request.body.getField("$tenant");
-    ASSERT(!dollarTenant.eoo());
-    ASSERT_EQ(TenantId::parseFromBSON(dollarTenant), tenantId);
-}
-
-TEST(OpMsgRequest, SetDollarTenantHasNoTenant) {
-    RAIIServerParameterControllerForTest multitenancyController("multitenancySupport", true);
-    RAIIServerParameterControllerForTest requireTenantIdController("featureFlagRequireTenantID",
-                                                                   true);
-    const TenantId tenantId(OID::gen());
-    // Set $tenant on a OpMsgRequest which has no $tenant.
-    OpMsgRequest request;
-    request.body = BSON("ping" << 1 << "$db"
-                               << "testDb");
-    request.setDollarTenant(tenantId);
-
-    auto dollarTenant = request.body.getField("$tenant");
-    ASSERT(!dollarTenant.eoo());
-    ASSERT_EQ(TenantId::parseFromBSON(dollarTenant), tenantId);
-}
-
-TEST(OpMsgRequest, SetDollarTenantFailWithDiffTenant) {
-    RAIIServerParameterControllerForTest multitenancyController("multitenancySupport", true);
-    RAIIServerParameterControllerForTest requireTenantIdController("featureFlagRequireTenantID",
-                                                                   true);
-    const TenantId tenantId(OID::gen());
-    auto const body = BSON("ping" << 1 << "$tenant" << tenantId);
-    auto request = OpMsgRequest::fromDBAndBody("testDb", body);
-    ASSERT_THROWS_CODE(request.setDollarTenant(TenantId(OID::gen())), DBException, 8423373);
-}
-
-TEST_F(OpMsgWithAuth, SetDollarTenantFailWithVTS) {
-    RAIIServerParameterControllerForTest multitenancyController("multitenancySupport", true);
-    AuthorizationSessionImplTestHelper::grantUseTenant(*(client.get()));
-
-    const auto kTenantId = TenantId(OID::gen());
-    const auto body = BSON("ping" << 1 << "$tenant" << kTenantId);
-    auto msg = OpMsgBytes{kNoFlags, kBodySection, body}.parse(client.get());
-    OpMsgRequest request(std::move(msg));
-
-    ASSERT(request.validatedTenancyScope);
-    ASSERT_THROWS_CODE(request.setDollarTenant(TenantId(OID::gen())), DBException, 8423372);
-}
-
 TEST(OpMsgTest, ChecksumResizesMessage) {
     auto msg = OpMsgBytes{kNoFlags,  //
                           kBodySection,
@@ -1103,6 +1206,98 @@ TEST(OpMsgTest, EmptyMessageWithChecksumFlag) {
     ASSERT_THROWS_CODE(msg.parse(), AssertionException, 51252);
 }
 
+TEST_F(OpMsgWithAuth, GetDbNameWithVTS) {
+    RAIIServerParameterControllerForTest multitenancyController("multitenancySupport", true);
+    AuthorizationSessionImplTestHelper::grantUseTenant(*(client.get()));
+
+    const auto kTenantId = TenantId(OID::gen());
+    std::string db = "myDb";
+    BSONObjBuilder builder;
+    builder.append("ping", 1).append("$db", db);
+
+    const auto token = auth::ValidatedTenancyScopeFactory::create(
+                           kTenantId,
+                           auth::ValidatedTenancyScope::TenantProtocol::kDefault,
+                           auth::ValidatedTenancyScopeFactory::TenantForTestingTag{})
+                           .getOriginalToken()
+                           .toString();
+
+    const auto body = builder.obj();
+    OpMsg msg =
+        OpMsgBytes{
+            kNoFlags,
+            kBodySection,
+            body,
+            kDocSequenceSection,
+            Sized{
+                "docs",
+                fromjson("{a: 1}"),
+                fromjson("{a: 2}"),
+            },
+            kSecurityTokenSection,
+            token,
+        }
+            .parse(client.get());
+
+
+    const DatabaseName expectedTenantDbName =
+        DatabaseName::createDatabaseName_forTest(kTenantId, "myDb");
+    using SC = SerializationContext;
+
+    // Test the request which has tenant prefix.
+    OpMsgRequest request = OpMsgRequest(std::move(msg));
+    ASSERT_EQ(request.getSerializationContext(),
+              SerializationContext(
+                  SC::Source::Command, SC::CallerType::Request, SC::Prefix::ExcludePrefix));
+    ASSERT_EQ(request.parseDbName(), expectedTenantDbName);
+}
+
+TEST_F(OpMsgWithAuth, GetDbNameWithVTSIncludePrefix) {
+    RAIIServerParameterControllerForTest multitenancyController("multitenancySupport", true);
+    AuthorizationSessionImplTestHelper::grantUseTenant(*(client.get()));
+
+    const auto kTenantId = TenantId(OID::gen());
+    std::string db = kTenantId.toString() + "_myDb";
+    BSONObjBuilder builder;
+    builder.append("ping", 1).append("$db", db);
+
+    const auto token = auth::ValidatedTenancyScopeFactory::create(
+                           kTenantId,
+                           auth::ValidatedTenancyScope::TenantProtocol::kAtlasProxy,
+                           auth::ValidatedTenancyScopeFactory::TenantForTestingTag{})
+                           .getOriginalToken()
+                           .toString();
+
+    const auto body = builder.obj();
+    OpMsg msg =
+        OpMsgBytes{
+            kNoFlags,  //
+            kBodySection,
+            body,
+            kDocSequenceSection,
+            Sized{
+                "docs",  //
+                fromjson("{a: 1}"),
+                fromjson("{a: 2}"),
+            },
+            kSecurityTokenSection,
+            token,
+        }
+            .parse(client.get());
+
+    const DatabaseName expectedTenantDbName =
+        DatabaseName::createDatabaseName_forTest(kTenantId, "myDb");
+    using SC = SerializationContext;
+
+    // Test the request which does not have tenant prefix.
+    const DatabaseName expectedDbName = DatabaseName::createDatabaseName_forTest(kTenantId, "myDb");
+    OpMsgRequest request{std::move(msg)};
+    ASSERT_EQ(request.getSerializationContext(),
+              SerializationContext(
+                  SC::Source::Command, SC::CallerType::Request, SC::Prefix::IncludePrefix));
+    ASSERT_EQ(request.parseDbName(), expectedDbName);
+}
+
 }  // namespace
 }  // namespace test
 }  // namespace rpc
