@@ -29,12 +29,18 @@
 
 #include "mongo/rpc/reply_builder_interface.h"
 
+#include <memory>
 #include <utility>
 
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/error_extra_info.h"
 #include "mongo/base/status_with.h"
+#include "mongo/bson/bsonelement.h"
 #include "mongo/db/basic_types_gen.h"
 #include "mongo/db/commands/test_commands_enabled.h"
-#include "mongo/db/jsobj.h"
+#include "mongo/idl/idl_parser.h"
 
 namespace mongo {
 namespace rpc {
