@@ -28,23 +28,47 @@
  */
 
 
-#include "mongo/platform/basic.h"
-
-#include "mongo/s/balancer_configuration.h"
-
-#include <algorithm>
-#include <boost/date_time/gregorian/gregorian_types.hpp>
-#include <boost/date_time/posix_time/posix_time.hpp>
 #include <boost/date_time/posix_time/posix_time_types.hpp>
+// IWYU pragma: no_include "boost/date_time/gregorian_calendar.ipp"
+#include <algorithm>
+#include <boost/date_time/posix_time/posix_time_duration.hpp>
+#include <boost/date_time/posix_time/posix_time_io.hpp>
+#include <boost/date_time/posix_time/ptime.hpp>
+#include <boost/date_time/time_duration.hpp>
+#include <boost/iterator/iterator_traits.hpp>
+#include <boost/operators.hpp>
+#include <cstdio>
+#include <deque>
+#include <iterator>
+#include <mutex>
+#include <sstream>
+#include <string>
+#include <utility>
+#include <vector>
 
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
 #include "mongo/base/status.h"
 #include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
 #include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
 #include "mongo/bson/util/bson_extract.h"
 #include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/s/balancer_configuration.h"
 #include "mongo/s/catalog/sharding_catalog_client.h"
 #include "mongo/s/grid.h"
+#include "mongo/s/sharding_feature_flags_gen.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/str.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
@@ -83,11 +107,39 @@ const char kAttemptToBalanceJumboChunks[] = "attemptToBalanceJumboChunks";
 }  // namespace
 
 const char BalancerSettingsType::kKey[] = "balancer";
-// TODO SERVER-75757: get rid of legacy `autoSplitOnly` mode
-const char* BalancerSettingsType::kBalancerModes[] = {"full", "autoSplitOnly", "off"};
+const std::vector<std::string> BalancerSettingsType::kBalancerModes = {"full", "off"};
+const BSONObj BalancerSettingsType::kSchema =
+    BSON("properties" << BSON("_id" << BSON("enum" << BSON_ARRAY(BalancerSettingsType::kKey))
+                                    << kMode << BSON("enum" << kBalancerModes) << kStopped
+                                    << BSON("bsonType"
+                                            << "bool")
+                                    << kActiveWindow
+                                    << BSON("bsonType"
+                                            << "object"
+                                            << "required"
+                                            << BSON_ARRAY("start"
+                                                          << "stop"))
+                                    << "_secondaryThrottle"
+                                    << BSON("oneOf" << BSON_ARRAY(BSON("bsonType"
+                                                                       << "bool")
+                                                                  << BSON("bsonType"
+                                                                          << "object")))
+                                    << kWaitForDelete
+                                    << BSON("bsonType"
+                                            << "bool")
+                                    << kAttemptToBalanceJumboChunks
+                                    << BSON("bsonType"
+                                            << "bool"))
+                      << "additionalProperties" << false);
 
 const char ChunkSizeSettingsType::kKey[] = "chunksize";
 const uint64_t ChunkSizeSettingsType::kDefaultMaxChunkSizeBytes{128 * 1024 * 1024};
+const BSONObj ChunkSizeSettingsType::kSchema = BSON(
+    "properties" << BSON("_id" << BSON("enum" << BSON_ARRAY(ChunkSizeSettingsType::kKey)) << kValue
+                               << BSON("bsonType"
+                                       << "number"
+                                       << "minimum" << 1 << "maximum" << 1024))
+                 << "additionalProperties" << false);
 
 const char AutoMergeSettingsType::kKey[] = "automerge";
 
@@ -252,7 +304,6 @@ Status BalancerConfiguration::_refreshChunkSizeSettings(OperationContext* opCtx)
 
     if (settings.getMaxChunkSizeBytes() != getMaxChunkSizeBytes()) {
         LOGV2(22640,
-              "Changing MaxChunkSize setting to {newMaxChunkSizeMB}MB from {oldMaxChunkSizeMB}MB",
               "Changing MaxChunkSize setting",
               "newMaxChunkSizeMB"_attr = settings.getMaxChunkSizeBytes() / (1024 * 1024),
               "oldMaxChunkSizeMB"_attr = getMaxChunkSizeBytes() / (1024 * 1024));
@@ -313,10 +364,15 @@ StatusWith<BalancerSettingsType> BalancerSettingsType::fromBSON(const BSONObj& o
                 return status;
             auto it = std::find(std::begin(kBalancerModes), std::end(kBalancerModes), modeStr);
             if (it == std::end(kBalancerModes)) {
-                return Status(ErrorCodes::BadValue, "Invalid balancer mode");
+                LOGV2_WARNING(
+                    7575700,
+                    "Balancer turned off because currently set balancing mode is not valid",
+                    "currentMode"_attr = modeStr,
+                    "supportedModes"_attr = kBalancerModes);
+                settings._mode = kOff;
+            } else {
+                settings._mode = static_cast<BalancerMode>(it - std::begin(kBalancerModes));
             }
-
-            settings._mode = static_cast<BalancerMode>(it - std::begin(kBalancerModes));
         }
     }
 
