@@ -29,21 +29,44 @@
 
 #pragma once
 
+#include <boost/none.hpp>
 #include <boost/optional.hpp>
+#include <boost/optional/optional.hpp>
 #include <string>
+#include <utility>
 #include <vector>
 
+#include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/oid.h"
+#include "mongo/bson/timestamp.h"
 #include "mongo/client/read_preference.h"
 #include "mongo/db/keys_collection_document_gen.h"
+#include "mongo/db/logical_time.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/pipeline/aggregate_command_gen.h"
+#include "mongo/db/repl/optime.h"
 #include "mongo/db/repl/optime_with.h"
 #include "mongo/db/repl/read_concern_args.h"
+#include "mongo/db/repl/read_concern_level.h"
+#include "mongo/db/shard_id.h"
 #include "mongo/db/write_concern_options.h"
+#include "mongo/s/catalog/type_chunk.h"
 #include "mongo/s/catalog/type_index_catalog.h"
+#include "mongo/s/catalog/type_index_catalog_gen.h"
+#include "mongo/s/catalog/type_namespace_placement_gen.h"
 #include "mongo/s/catalog/type_shard.h"
+#include "mongo/s/catalog/type_tags.h"
 #include "mongo/s/chunk_version.h"
 #include "mongo/s/client/shard.h"
 #include "mongo/s/index_version.h"
 #include "mongo/s/request_types/placement_history_commands_gen.h"
+#include "mongo/util/duration.h"
+#include "mongo/util/uuid.h"
 
 namespace mongo {
 
@@ -52,13 +75,16 @@ class BSONArrayBuilder;
 class BSONObj;
 class BSONObjBuilder;
 class ChunkType;
+
 class CollectionType;
 class ConnectionString;
+
 class DatabaseType;
 class LogicalTime;
 class OperationContext;
 class ShardKeyPattern;
 class TagsType;
+
 class VersionType;
 
 namespace executor {
@@ -90,13 +116,17 @@ public:
     // Identifier for the "initialization metadata" documents of config.placementHistory
     static const NamespaceString kConfigPlacementHistoryInitializationMarker;
 
+    // Identifier of the config.shardRemovalLog document about the latest removeShard operation
+    // committed on the cluster.
+    static constexpr auto kLatestShardRemovalLogId = "latestShardRemovalLog"_sd;
+
     virtual ~ShardingCatalogClient() = default;
 
     virtual std::vector<BSONObj> runCatalogAggregation(
         OperationContext* opCtx,
         AggregateCommandRequest& aggRequest,
         const repl::ReadConcernArgs& readConcern,
-        const Milliseconds& maxTimeout = Shard::kDefaultConfigCommandTimeout) = 0;
+        const Milliseconds& maxTimeout = Milliseconds(defaultConfigCommandTimeoutMS.load())) = 0;
 
     /**
      * Retrieves the metadata for a given database, if it exists.
@@ -109,7 +139,7 @@ public:
      *  - NamespaceNotFound - database does not exist
      */
     virtual DatabaseType getDatabase(OperationContext* opCtx,
-                                     StringData db,
+                                     const DatabaseName& db,
                                      repl::ReadConcernLevel readConcernLevel) = 0;
 
     /**
@@ -140,6 +170,18 @@ public:
         const UUID& uuid,
         repl::ReadConcernLevel readConcernLevel = repl::ReadConcernLevel::kMajorityReadConcern) = 0;
 
+    /**
+     * Retrieves all collections under a specified database (or in the system) which are sharded. If
+     * the dbName parameter is empty, returns all sharded collections.
+     *
+     * @param sort Fields to use for sorting the results. If empty, no sorting is performed.
+     */
+    virtual std::vector<CollectionType> getShardedCollections(
+        OperationContext* opCtx,
+        const DatabaseName& db,
+        repl::ReadConcernLevel readConcernLevel = repl::ReadConcernLevel::kMajorityReadConcern,
+        const BSONObj& sort = BSONObj()) = 0;
+
     /**
      * Retrieves all collections under a specified database (or in the system). If the dbName
      * parameter is empty, returns all collections.
@@ -148,7 +190,7 @@ public:
      */
     virtual std::vector<CollectionType> getCollections(
         OperationContext* opCtx,
-        StringData db,
+        const DatabaseName& db,
         repl::ReadConcernLevel readConcernLevel = repl::ReadConcernLevel::kMajorityReadConcern,
         const BSONObj& sort = BSONObj()) = 0;
 
@@ -159,19 +201,59 @@ public:
      *
      * Throws exception on errors.
      */
-    virtual std::vector<NamespaceString> getAllShardedCollectionsForDb(
+    virtual std::vector<NamespaceString> getShardedCollectionNamespacesForDb(
         OperationContext* opCtx,
-        StringData dbName,
+        const DatabaseName& dbName,
         repl::ReadConcernLevel readConcern,
         const BSONObj& sort = BSONObj()) = 0;
 
+    /**
+     * Returns the set of collections tracked for the specified database, regardless of being
+     * sharded or not. Goes directly to the config server's metadata, without checking the local
+     * cache so it should not be used in frequently called code paths.
+     *
+     * Throws exception on errors.
+     */
+    virtual std::vector<NamespaceString> getCollectionNamespacesForDb(
+        OperationContext* opCtx,
+        const DatabaseName& dbName,
+        repl::ReadConcernLevel readConcern,
+        const BSONObj& sort = BSONObj()) = 0;
+
+    /**
+     * Returns the set of collections for the specified database, which have been marked as
+     * unsplittable. Goes directly to the config server's metadata, without checking the local cache
+     * so it should not be used in frequently called code paths.
+     *
+     * Throws exception on errors.
+     */
+    virtual std::vector<NamespaceString> getUnsplittableCollectionNamespacesForDb(
+        OperationContext* opCtx,
+        const DatabaseName& dbName,
+        repl::ReadConcernLevel readConcern,
+        const BSONObj& sort = BSONObj()) = 0;
+
+    /**
+     * Returns the set of collections for the specified database, which have been marked as
+     * unsplittable excluding those whose data shard is in the list of shards to ignore. Goes
+     * directly to the config server's metadata, without checking the local cache so it should not
+     * be used in frequently called code paths.
+     *
+     * Throws exception on errors.
+     */
+    virtual std::vector<NamespaceString> getUnsplittableCollectionNamespacesForDbOutsideOfShards(
+        OperationContext* opCtx,
+        const DatabaseName& dbName,
+        const std::vector<ShardId>& excludedShards,
+        repl::ReadConcernLevel readConcern) = 0;
+
     /**
      * Retrieves all databases for a shard.
      *
      * Returns a !OK status if an error occurs.
      */
-    virtual StatusWith<std::vector<std::string>> getDatabasesForShard(OperationContext* opCtx,
-                                                                      const ShardId& shardId) = 0;
+    virtual StatusWith<std::vector<DatabaseName>> getDatabasesForShard(OperationContext* opCtx,
+                                                                       const ShardId& shardId) = 0;
 
     /**
      * Gets the requested number of chunks (of type ChunkType) that satisfy a query.
@@ -233,7 +315,7 @@ public:
      * Retrieves all namespaces that have zones associated with a database.
      */
     virtual std::vector<NamespaceString> getAllNssThatHaveZonesForDatabase(
-        OperationContext* opCtx, const StringData& dbName) = 0;
+        OperationContext* opCtx, const DatabaseName& dbName) = 0;
 
     /**
      * Retrieves the list of shards in this sharded cluster. If `excludeDraining` is set to `false`
@@ -258,7 +340,7 @@ public:
      */
     virtual Status runUserManagementWriteCommand(OperationContext* opCtx,
                                                  StringData commandName,
-                                                 StringData dbname,
+                                                 const DatabaseName& dbname,
                                                  const BSONObj& cmdObj,
                                                  BSONObjBuilder* result) = 0;
 
@@ -266,7 +348,7 @@ public:
      * Runs a user management related read-only command on a config server.
      */
     virtual bool runUserManagementReadCommand(OperationContext* opCtx,
-                                              const std::string& dbname,
+                                              const DatabaseName& dbname,
                                               const BSONObj& cmdObj,
                                               BSONObjBuilder* result) = 0;
 
@@ -401,14 +483,18 @@ public:
      * a specific point in time. When no namespace is specified, placement metadata on the whole
      * cluster will be returned. This function is meant to be exclusively invoked by config server
      * nodes.
-     *
-     * TODO (SERVER-73029): convert to private method of ShardingCatalogClientImpl
      */
     virtual HistoricalPlacement getHistoricalPlacement(
         OperationContext* opCtx,
         const Timestamp& atClusterTime,
         const boost::optional<NamespaceString>& nss) = 0;
 
+    /**
+     * Provided a cluster time, returns whether there was a topology change caused by a removeShard
+     * committed on the commit server within the [clusterTime, +inf) time window.
+     */
+    virtual bool anyShardRemovedSince(OperationContext* opCtx, const Timestamp& clusterTime) = 0;
+
 protected:
     ShardingCatalogClient() = default;
 
