@@ -29,13 +29,22 @@
 
 #include "mongo/s/catalog/type_chunk.h"
 
+#include <boost/none.hpp>
+#include <cstring>
+
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
 #include "mongo/base/status_with.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bsonelement.h"
 #include "mongo/bson/bsonobj.h"
 #include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
 #include "mongo/bson/simple_bsonobj_comparator.h"
 #include "mongo/bson/util/bson_extract.h"
-#include "mongo/db/server_options.h"
-#include "mongo/logv2/log.h"
+#include "mongo/idl/idl_parser.h"
 #include "mongo/util/assert_util.h"
 #include "mongo/util/str.h"
 
@@ -47,7 +56,7 @@ const NamespaceString ChunkType::ConfigNS(NamespaceString::kConfigsvrChunksNames
 
 // The final namespace of the cached chunks metadata is composed of the namespace of the related
 // sharded collection (i.e., config.cache.chunks.<ns>). As a result, the maximum namespace length of
-// sharded collections is reduced. See NamespaceString::MaxNsShardedCollectionLen.
+// sharded collections is reduced. See NamespaceString::MaxUserNsShardedCollectionLen.
 const std::string ChunkType::ShardNSPrefix = "config.cache.chunks.";
 
 const BSONField<OID> ChunkType::name("_id");
@@ -641,7 +650,7 @@ Status ChunkType::validate() const {
     if (!_history.empty()) {
         if (_history.front().getShard() != *_shard) {
             return {ErrorCodes::BadValue,
-                    str::stream() << "Latest entry of chunk history refers to shard "
+                    str::stream() << "Latest entry of chunk history refer to shard "
                                   << _history.front().getShard()
                                   << " that does not match the current shard " << *_shard};
         }
