@@ -29,27 +29,59 @@
 
 #include "mongo/s/catalog_cache.h"
 
+#include <boost/none.hpp>
+#include <boost/optional.hpp>
+#include <cstddef>
+#include <cstdint>
 #include <fmt/format.h>
+#include <list>
+#include <memory>
+#include <set>
+#include <vector>
 
+#include <absl/container/node_hash_map.h>
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/bson/bsonobj.h"
 #include "mongo/bson/bsonobjbuilder.h"
-#include "mongo/db/catalog_shard_feature_flag_gen.h"
+#include "mongo/db/cluster_role.h"
 #include "mongo/db/curop.h"
+#include "mongo/db/feature_flag.h"
+#include "mongo/db/keypattern.h"
+#include "mongo/db/logical_time.h"
 #include "mongo/db/query/collation/collator_factory_interface.h"
-#include "mongo/db/repl/optime_with.h"
+#include "mongo/db/query/collation/collator_interface.h"
+#include "mongo/db/repl/read_concern_args.h"
+#include "mongo/db/repl/read_concern_level.h"
+#include "mongo/db/server_options.h"
+#include "mongo/db/transaction_resources.h"
 #include "mongo/db/vector_clock.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_attr.h"
+#include "mongo/logv2/log_component.h"
+#include "mongo/logv2/redaction.h"
+#include "mongo/s/catalog/sharding_catalog_client.h"
 #include "mongo/s/catalog/type_collection.h"
 #include "mongo/s/catalog/type_database_gen.h"
+#include "mongo/s/catalog/type_index_catalog_gen.h"
+#include "mongo/s/chunk_version.h"
 #include "mongo/s/client/shard_registry.h"
 #include "mongo/s/grid.h"
-#include "mongo/s/is_mongos.h"
+#include "mongo/s/index_version.h"
 #include "mongo/s/mongod_and_mongos_server_parameters_gen.h"
+#include "mongo/s/resharding/type_collection_fields_gen.h"
 #include "mongo/s/shard_cannot_refresh_due_to_locks_held_exception.h"
 #include "mongo/s/shard_version_factory.h"
 #include "mongo/s/sharding_feature_flags_gen.h"
-#include "mongo/s/stale_exception.h"
-#include "mongo/util/concurrency/with_lock.h"
-#include "mongo/util/scopeguard.h"
+#include "mongo/s/type_collection_common_types_gen.h"
+#include "mongo/util/duration.h"
+#include "mongo/util/fail_point.h"
+#include "mongo/util/future.h"
+#include "mongo/util/invalidating_lru_cache.h"
+#include "mongo/util/str.h"
 #include "mongo/util/timer.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
@@ -69,9 +101,6 @@ const int kDatabaseCacheSize = 10000;
 const int kCollectionCacheSize = 10000;
 const int kIndexCacheSize = 10000;
 
-const OperationContext::Decoration<bool> operationShouldBlockBehindCatalogCacheRefresh =
-    OperationContext::declareDecoration<bool>();
-
 std::shared_ptr<RoutingTableHistory> createUpdatedRoutingTableHistory(
     OperationContext* opCtx,
     const NamespaceString& nss,
@@ -85,7 +114,7 @@ std::shared_ptr<RoutingTableHistory> createUpdatedRoutingTableHistory(
         tassert(7032310,
                 fmt::format("allowMigrations field of collection '{}' changed without changing the "
                             "collection placement version {}. Old value: {}, new value: {}",
-                            nss.toString(),
+                            nss.toStringForErrorMsg(),
                             existingHistory->optRt->getVersion().toString(),
                             existingHistory->optRt->allowMigrations(),
                             collectionAndChunks.allowMigrations),
@@ -96,10 +125,10 @@ std::shared_ptr<RoutingTableHistory> createUpdatedRoutingTableHistory(
         tassert(7032311,
                 fmt::format("reshardingFields field of collection '{}' changed without changing "
                             "the collection placement version {}. Old value: {}, new value: {}",
-                            nss.toString(),
+                            nss.toStringForErrorMsg(),
                             existingHistory->optRt->getVersion().toString(),
-                            oldReshardingFields->toBSON().toString(),
-                            newReshardingFields->toBSON().toString()),
+                            oldReshardingFields ? oldReshardingFields->toBSON().toString() : "{}",
+                            newReshardingFields ? newReshardingFields->toBSON().toString() : "{}"),
                 [&] {
                     if (oldReshardingFields && newReshardingFields)
                         return oldReshardingFields->toBSON().woCompare(
@@ -120,6 +149,7 @@ std::shared_ptr<RoutingTableHistory> createUpdatedRoutingTableHistory(
             return existingHistory->optRt->makeUpdated(collectionAndChunks.timeseriesFields,
                                                        collectionAndChunks.reshardingFields,
                                                        collectionAndChunks.allowMigrations,
+                                                       collectionAndChunks.unsplittable,
                                                        collectionAndChunks.changedChunks);
         }
 
@@ -135,6 +165,7 @@ std::shared_ptr<RoutingTableHistory> createUpdatedRoutingTableHistory(
         return RoutingTableHistory::makeNew(nss,
                                             *collectionAndChunks.uuid,
                                             KeyPattern(collectionAndChunks.shardKeyPattern),
+                                            collectionAndChunks.unsplittable,
                                             std::move(defaultCollator),
                                             collectionAndChunks.shardKeyIsUnique,
                                             collectionAndChunks.epoch,
@@ -148,33 +179,6 @@ std::shared_ptr<RoutingTableHistory> createUpdatedRoutingTableHistory(
     return std::make_shared<RoutingTableHistory>(std::move(newRoutingHistory));
 }
 
-StatusWith<CollectionRoutingInfo> retryUntilConsistentRoutingInfo(
-    OperationContext* opCtx,
-    const NamespaceString& nss,
-    ChunkManager&& cm,
-    boost::optional<ShardingIndexesCatalogCache>&& sii) {
-    const auto catalogCache = Grid::get(opCtx)->catalogCache();
-    try {
-        // A non-empty ShardingIndexesCatalogCache implies that the collection is sharded since
-        // global indexes cannot be created on unsharded collections.
-        while (sii && (!cm.isSharded() || !cm.uuidMatches(sii->getCollectionIndexes().uuid()))) {
-            auto nextSii =
-                uassertStatusOK(catalogCache->getCollectionRoutingInfoWithIndexRefresh(opCtx, nss))
-                    .sii;
-            if (sii.is_initialized() && nextSii.is_initialized() &&
-                sii->getCollectionIndexes() == nextSii->getCollectionIndexes()) {
-                cm = uassertStatusOK(
-                         catalogCache->getCollectionRoutingInfoWithPlacementRefresh(opCtx, nss))
-                         .cm;
-            }
-            sii = std::move(nextSii);
-        }
-    } catch (const DBException& ex) {
-        return ex.toStatus();
-    }
-    return CollectionRoutingInfo{std::move(cm), std::move(sii)};
-}
-
 }  // namespace
 
 ShardVersion CollectionRoutingInfo::getCollectionVersion() const {
@@ -254,11 +258,14 @@ bool ComparableDatabaseVersion::operator<(const ComparableDatabaseVersion& other
     return _disambiguatingSequenceNum < other._disambiguatingSequenceNum;
 }
 
-CatalogCache::CatalogCache(ServiceContext* const service, CatalogCacheLoader& cacheLoader)
-    : _cacheLoader(cacheLoader),
-      _executor([] {
+CatalogCache::CatalogCache(ServiceContext* const service,
+                           CatalogCacheLoader& cacheLoader,
+                           StringData kind)
+    : _kind(kind),
+      _cacheLoader(cacheLoader),
+      _executor([this] {
           ThreadPool::Options options;
-          options.poolName = "CatalogCache";
+          options.poolName = "CatalogCache" + (_kind.empty() ? "" : "::" + _kind);
           options.minThreads = 0;
           options.maxThreads = 6;
           return options;
@@ -281,18 +288,29 @@ void CatalogCache::shutDownAndJoin() {
 }
 
 StatusWith<CachedDatabaseInfo> CatalogCache::getDatabase(OperationContext* opCtx,
-                                                         StringData dbName) {
+                                                         const DatabaseName& dbName) {
     return _getDatabase(opCtx, dbName);
 }
 
 StatusWith<CachedDatabaseInfo> CatalogCache::_getDatabase(OperationContext* opCtx,
-                                                          StringData dbName,
+                                                          const DatabaseName& dbName,
                                                           bool allowLocks) {
     tassert(7032313,
             "Do not hold a lock while refreshing the catalog cache. Doing so would potentially "
             "hold the lock during a network call, and can lead to a deadlock as described in "
             "SERVER-37398.",
-            allowLocks || !opCtx->lockState() || !opCtx->lockState()->isLocked());
+            allowLocks || !shard_role_details::getLocker(opCtx) ||
+                !shard_role_details::getLocker(opCtx)->isLocked());
+
+    if (dbName.isAdminDB() || dbName.isConfigDB()) {
+        // The 'admin' and 'config' databases are known to always be in the config server, there is
+        // no need to make a request to the DatabaseCache. This prevents a potential deadlock on
+        // config shard nodes issuing aggregations on 'config' to themselves during a refresh, and
+        // finding the CatalogCache thread pool is full when the aggregation itself needs to refresh
+        // the cache (i.e. $unionWith).
+        return CachedDatabaseInfo{
+            DatabaseType(dbName, ShardId::kConfigServerId, DatabaseVersion::makeFixed())};
+    }
 
     Timer t{};
     ScopeGuard finishTiming([&] {
@@ -321,7 +339,7 @@ StatusWith<CachedDatabaseInfo> CatalogCache::_getDatabase(OperationContext* opCt
         // From this point we can guarantee that allowLocks is false.
         auto dbEntry = dbEntryFuture.get(opCtx);
         uassert(ErrorCodes::NamespaceNotFound,
-                str::stream() << "database " << dbName << " not found",
+                str::stream() << "database " << dbName.toStringForErrorMsg() << " not found",
                 dbEntry);
 
         return dbEntry;
@@ -339,10 +357,10 @@ StatusWith<ChunkManager> CatalogCache::_getCollectionPlacementInfoAt(
             "Do not hold a lock while refreshing the catalog cache. Doing so would potentially "
             "hold the lock during a network call, and can lead to a deadlock as described in "
             "SERVER-37398.",
-            allowLocks || !opCtx->lockState() || !opCtx->lockState()->isLocked());
+            allowLocks || !shard_role_details::getLocker(opCtx)->isLocked());
 
     try {
-        const auto swDbInfo = _getDatabase(opCtx, nss.db(), allowLocks);
+        const auto swDbInfo = _getDatabase(opCtx, nss.dbName(), allowLocks);
         if (!swDbInfo.isOK()) {
             if (swDbInfo == ErrorCodes::ShardCannotRefreshDueToLocksHeld) {
                 // Since collection refreshes always imply database refreshes, it is ok to transform
@@ -375,12 +393,17 @@ StatusWith<ChunkManager> CatalogCache::_getCollectionPlacementInfoAt(
 
         const auto dbInfo = std::move(swDbInfo.getValue());
 
-        const auto cacheConsistency = gEnableFinerGrainedCatalogCacheRefresh &&
-                !operationShouldBlockBehindCatalogCacheRefresh(opCtx)
-            ? CacheCausalConsistency::kLatestCached
-            : CacheCausalConsistency::kLatestKnown;
+        if (nss.isNamespaceAlwaysUntracked()) {
+            // If the collection is known to always be untracked, there is no need to request it to
+            // the CollectionCache.
+            return ChunkManager(dbInfo->getPrimary(),
+                                dbInfo->getVersion(),
+                                OptionalRoutingTableHistory(),
+                                atClusterTime);
+        }
 
-        auto collEntryFuture = _collectionCache.acquireAsync(nss, cacheConsistency);
+        auto collEntryFuture =
+            _collectionCache.acquireAsync(nss, CacheCausalConsistency::kLatestKnown);
 
         if (allowLocks) {
             // When allowLocks is true we may be holding a lock, so we don't
@@ -388,7 +411,6 @@ StatusWith<ChunkManager> CatalogCache::_getCollectionPlacementInfoAt(
             // use it, otherwise return an error
 
             if (collEntryFuture.isReady()) {
-                setOperationShouldBlockBehindCatalogCacheRefresh(opCtx, false);
                 return ChunkManager(dbInfo->getPrimary(),
                                     dbInfo->getVersion(),
                                     collEntryFuture.get(opCtx),
@@ -408,8 +430,6 @@ StatusWith<ChunkManager> CatalogCache::_getCollectionPlacementInfoAt(
                 auto collEntry = collEntryFuture.get(opCtx);
                 _stats.totalRefreshWaitTimeMicros.addAndFetch(t.micros());
 
-                setOperationShouldBlockBehindCatalogCacheRefresh(opCtx, false);
-
                 return ChunkManager(dbInfo->getPrimary(),
                                     dbInfo->getVersion(),
                                     std::move(collEntry),
@@ -434,7 +454,8 @@ StatusWith<ChunkManager> CatalogCache::_getCollectionPlacementInfoAt(
                 }
             }
 
-            collEntryFuture = _collectionCache.acquireAsync(nss, cacheConsistency);
+            collEntryFuture =
+                _collectionCache.acquireAsync(nss, CacheCausalConsistency::kLatestKnown);
             t.reset();
         }
     } catch (const DBException& ex) {
@@ -443,11 +464,17 @@ StatusWith<ChunkManager> CatalogCache::_getCollectionPlacementInfoAt(
 }
 
 StatusWith<CollectionRoutingInfo> CatalogCache::getCollectionRoutingInfoAt(
-    OperationContext* opCtx, const NamespaceString& nss, Timestamp atClusterTime) {
+    OperationContext* opCtx, const NamespaceString& nss, Timestamp atClusterTime, bool allowLocks) {
     try {
-        auto cm = uassertStatusOK(_getCollectionPlacementInfoAt(opCtx, nss, atClusterTime));
-        auto sii = _getCollectionIndexInfoAt(opCtx, nss);
-        return retryUntilConsistentRoutingInfo(opCtx, nss, std::move(cm), std::move(sii));
+        auto cm =
+            uassertStatusOK(_getCollectionPlacementInfoAt(opCtx, nss, atClusterTime, allowLocks));
+        if (!cm.isSharded()) {
+            // If the collection is unsharded, it cannot have global indexes so there is no need to
+            // fetch the index information.
+            return CollectionRoutingInfo{std::move(cm), boost::none};
+        }
+        auto sii = _getCollectionIndexInfoAt(opCtx, nss, allowLocks);
+        return _retryUntilConsistentRoutingInfo(opCtx, nss, std::move(cm), std::move(sii));
     } catch (const DBException& ex) {
         return ex.toStatus();
     }
@@ -459,8 +486,24 @@ StatusWith<CollectionRoutingInfo> CatalogCache::getCollectionRoutingInfo(Operati
     try {
         auto cm =
             uassertStatusOK(_getCollectionPlacementInfoAt(opCtx, nss, boost::none, allowLocks));
+        if (!cm.isSharded()) {
+            // If the collection is unsharded, it cannot have global indexes so there is no need to
+            // fetch the index information.
+            return CollectionRoutingInfo{std::move(cm), boost::none};
+        }
         auto sii = _getCollectionIndexInfoAt(opCtx, nss, allowLocks);
-        return retryUntilConsistentRoutingInfo(opCtx, nss, std::move(cm), std::move(sii));
+        return _retryUntilConsistentRoutingInfo(opCtx, nss, std::move(cm), std::move(sii));
+    } catch (const DBException& ex) {
+        return ex.toStatus();
+    }
+}
+
+StatusWith<CollectionRoutingInfo> CatalogCache::_getCollectionRoutingInfoWithoutOptimization(
+    OperationContext* opCtx, const NamespaceString& nss) {
+    try {
+        auto cm = uassertStatusOK(_getCollectionPlacementInfoAt(opCtx, nss, boost::none));
+        auto sii = _getCollectionIndexInfoAt(opCtx, nss);
+        return _retryUntilConsistentRoutingInfo(opCtx, nss, std::move(cm), std::move(sii));
     } catch (const DBException& ex) {
         return ex.toStatus();
     }
@@ -476,14 +519,14 @@ boost::optional<ShardingIndexesCatalogCache> CatalogCache::_getCollectionIndexIn
     }
 
     if (!allowLocks) {
-        invariant(!opCtx->lockState() || !opCtx->lockState()->isLocked(),
+        invariant(!shard_role_details::getLocker(opCtx)->isLocked(),
                   "Do not hold a lock while refreshing the catalog cache. Doing so would "
                   "potentially hold "
                   "the lock during a network call, and can lead to a deadlock as described in "
                   "SERVER-37398.");
     }
 
-    const auto swDbInfo = _getDatabase(opCtx, nss.db(), allowLocks);
+    const auto swDbInfo = _getDatabase(opCtx, nss.dbName(), allowLocks);
     if (!swDbInfo.isOK()) {
         if (swDbInfo == ErrorCodes::ShardCannotRefreshDueToLocksHeld) {
             // Since collection refreshes always imply database refreshes, it is ok to transform
@@ -553,11 +596,6 @@ boost::optional<ShardingIndexesCatalogCache> CatalogCache::_getCollectionIndexIn
             if (acquireTries == kMaxInconsistentCollectionRefreshAttempts) {
                 throw;
             }
-
-            // TODO (SERVER-71278) Remove this handling of SnapshotUnavailable
-            if (ex.code() == ErrorCodes::SnapshotUnavailable) {
-                sleepmillis(100);
-            }
         }
 
         indexEntryFuture = _indexCache.acquireAsync(nss, CacheCausalConsistency::kLatestKnown);
@@ -565,32 +603,51 @@ boost::optional<ShardingIndexesCatalogCache> CatalogCache::_getCollectionIndexIn
 }
 
 StatusWith<CachedDatabaseInfo> CatalogCache::getDatabaseWithRefresh(OperationContext* opCtx,
-                                                                    StringData dbName) {
+                                                                    const DatabaseName& dbName) {
     _databaseCache.advanceTimeInStore(
         dbName, ComparableDatabaseVersion::makeComparableDatabaseVersionForForcedRefresh());
     return getDatabase(opCtx, dbName);
 }
 
-void CatalogCache::_triggerPlacementVersionRefresh(OperationContext* opCtx,
-                                                   const NamespaceString& nss) {
+void CatalogCache::_triggerPlacementVersionRefresh(const NamespaceString& nss) {
     _collectionCache.advanceTimeInStore(
         nss, ComparableChunkVersion::makeComparableChunkVersionForForcedRefresh());
-    setOperationShouldBlockBehindCatalogCacheRefresh(opCtx, true);
 }
 
-void CatalogCache::_triggerIndexVersionRefresh(OperationContext* opCtx,
-                                               const NamespaceString& nss) {
+void CatalogCache::_triggerIndexVersionRefresh(const NamespaceString& nss) {
     _indexCache.advanceTimeInStore(
         nss, ComparableIndexVersion::makeComparableIndexVersionForForcedRefresh());
-    setOperationShouldBlockBehindCatalogCacheRefresh(opCtx, true);
+}
+
+StatusWith<CollectionRoutingInfo> CatalogCache::_retryUntilConsistentRoutingInfo(
+    OperationContext* opCtx,
+    const NamespaceString& nss,
+    ChunkManager&& cm,
+    boost::optional<ShardingIndexesCatalogCache>&& sii) {
+    try {
+        // A non-empty ShardingIndexesCatalogCache implies that the collection is sharded since
+        // global indexes cannot be created on unsharded collections.
+        while (sii && (!cm.isSharded() || !cm.uuidMatches(sii->getCollectionIndexes().uuid()))) {
+            auto nextSii =
+                uassertStatusOK(getCollectionRoutingInfoWithIndexRefresh(opCtx, nss)).sii;
+            if (sii.is_initialized() && nextSii.is_initialized() &&
+                sii->getCollectionIndexes() == nextSii->getCollectionIndexes()) {
+                cm = uassertStatusOK(getCollectionRoutingInfoWithPlacementRefresh(opCtx, nss)).cm;
+            }
+            sii = std::move(nextSii);
+        }
+    } catch (const DBException& ex) {
+        return ex.toStatus();
+    }
+    return CollectionRoutingInfo{std::move(cm), std::move(sii)};
 }
 
 StatusWith<CollectionRoutingInfo> CatalogCache::getCollectionRoutingInfoWithRefresh(
     OperationContext* opCtx, const NamespaceString& nss) {
     try {
-        _triggerPlacementVersionRefresh(opCtx, nss);
-        _triggerIndexVersionRefresh(opCtx, nss);
-        return getCollectionRoutingInfo(opCtx, nss, false);
+        _triggerPlacementVersionRefresh(nss);
+        _triggerIndexVersionRefresh(nss);
+        return _getCollectionRoutingInfoWithoutOptimization(opCtx, nss);
     } catch (const DBException& ex) {
         return ex.toStatus();
     }
@@ -599,7 +656,7 @@ StatusWith<CollectionRoutingInfo> CatalogCache::getCollectionRoutingInfoWithRefr
 StatusWith<CollectionRoutingInfo> CatalogCache::getCollectionRoutingInfoWithPlacementRefresh(
     OperationContext* opCtx, const NamespaceString& nss) {
     try {
-        _triggerPlacementVersionRefresh(opCtx, nss);
+        _triggerPlacementVersionRefresh(nss);
         return getCollectionRoutingInfo(opCtx, nss, false);
     } catch (const DBException& ex) {
         return ex.toStatus();
@@ -609,8 +666,8 @@ StatusWith<CollectionRoutingInfo> CatalogCache::getCollectionRoutingInfoWithPlac
 StatusWith<CollectionRoutingInfo> CatalogCache::getCollectionRoutingInfoWithIndexRefresh(
     OperationContext* opCtx, const NamespaceString& nss) {
     try {
-        _triggerIndexVersionRefresh(opCtx, nss);
-        return getCollectionRoutingInfo(opCtx, nss, false);
+        _triggerIndexVersionRefresh(nss);
+        return _getCollectionRoutingInfoWithoutOptimization(opCtx, nss);
     } catch (const DBException& ex) {
         return ex.toStatus();
     }
@@ -620,7 +677,8 @@ CollectionRoutingInfo CatalogCache::getShardedCollectionRoutingInfo(OperationCon
                                                                     const NamespaceString& nss) {
     auto cri = uassertStatusOK(getCollectionRoutingInfo(opCtx, nss));
     uassert(ErrorCodes::NamespaceNotSharded,
-            str::stream() << "Expected collection " << nss << " to be sharded",
+            str::stream() << "Expected collection " << nss.toStringForErrorMsg()
+                          << " to be sharded",
             cri.cm.isSharded());
     return cri;
 }
@@ -630,7 +688,8 @@ StatusWith<CollectionRoutingInfo> CatalogCache::getShardedCollectionRoutingInfoW
     try {
         auto cri = uassertStatusOK(getCollectionRoutingInfoWithRefresh(opCtx, nss));
         uassert(ErrorCodes::NamespaceNotSharded,
-                str::stream() << "Expected collection " << nss << " to be sharded",
+                str::stream() << "Expected collection " << nss.toStringForErrorMsg()
+                              << " to be sharded",
                 cri.cm.isSharded());
         return cri;
     } catch (const DBException& ex) {
@@ -643,7 +702,8 @@ StatusWith<CollectionRoutingInfo> CatalogCache::getShardedCollectionRoutingInfoW
     try {
         auto cri = uassertStatusOK(getCollectionRoutingInfoWithPlacementRefresh(opCtx, nss));
         uassert(ErrorCodes::NamespaceNotSharded,
-                str::stream() << "Expected collection " << nss << " to be sharded",
+                str::stream() << "Expected collection " << nss.toStringForErrorMsg()
+                              << " to be sharded",
                 cri.cm.isSharded());
         return cri;
     } catch (const DBException& ex) {
@@ -651,7 +711,45 @@ StatusWith<CollectionRoutingInfo> CatalogCache::getShardedCollectionRoutingInfoW
     }
 }
 
-void CatalogCache::onStaleDatabaseVersion(const StringData dbName,
+CollectionRoutingInfo CatalogCache::getTrackedCollectionRoutingInfo(OperationContext* opCtx,
+                                                                    const NamespaceString& nss) {
+    auto cri = uassertStatusOK(getCollectionRoutingInfo(opCtx, nss));
+    uassert(ErrorCodes::NamespaceNotFound,
+            str::stream() << "Expected collection " << nss.toStringForErrorMsg()
+                          << " to be tracked",
+            cri.cm.hasRoutingTable());
+    return cri;
+}
+
+StatusWith<CollectionRoutingInfo> CatalogCache::getTrackedCollectionRoutingInfoWithRefresh(
+    OperationContext* opCtx, const NamespaceString& nss) {
+    try {
+        auto cri = uassertStatusOK(getCollectionRoutingInfoWithRefresh(opCtx, nss));
+        uassert(ErrorCodes::NamespaceNotFound,
+                str::stream() << "Expected collection " << nss.toStringForErrorMsg()
+                              << " to be tracked",
+                cri.cm.hasRoutingTable());
+        return cri;
+    } catch (const DBException& ex) {
+        return ex.toStatus();
+    }
+}
+
+StatusWith<CollectionRoutingInfo> CatalogCache::getTrackedCollectionRoutingInfoWithPlacementRefresh(
+    OperationContext* opCtx, const NamespaceString& nss) {
+    try {
+        auto cri = uassertStatusOK(getCollectionRoutingInfoWithPlacementRefresh(opCtx, nss));
+        uassert(ErrorCodes::NamespaceNotFound,
+                str::stream() << "Expected collection " << nss.toStringForErrorMsg()
+                              << " to be tracked",
+                cri.cm.hasRoutingTable());
+        return cri;
+    } catch (const DBException& ex) {
+        return ex.toStatus();
+    }
+}
+
+void CatalogCache::onStaleDatabaseVersion(const DatabaseName& dbName,
                                           const boost::optional<DatabaseVersion>& databaseVersion) {
     if (databaseVersion) {
         const auto version =
@@ -667,93 +765,31 @@ void CatalogCache::onStaleDatabaseVersion(const StringData dbName,
     }
 }
 
-void CatalogCache::setOperationShouldBlockBehindCatalogCacheRefresh(OperationContext* opCtx,
-                                                                    bool shouldBlock) {
-    if (gEnableFinerGrainedCatalogCacheRefresh) {
-        operationShouldBlockBehindCatalogCacheRefresh(opCtx) = shouldBlock;
-    }
-}
-
 void CatalogCache::invalidateShardOrEntireCollectionEntryForShardedCollection(
     const NamespaceString& nss,
     const boost::optional<ShardVersion>& wantedVersion,
     const ShardId& shardId) {
     _stats.countStaleConfigErrors.addAndFetch(1);
 
-    auto collectionEntry = _collectionCache.peekLatestCached(nss);
-
     const auto newChunkVersion = wantedVersion
         ? ComparableChunkVersion::makeComparableChunkVersion(wantedVersion->placementVersion())
         : ComparableChunkVersion::makeComparableChunkVersionForForcedRefresh();
-
-    const bool routingInfoTimeAdvanced = _collectionCache.advanceTimeInStore(nss, newChunkVersion);
+    _collectionCache.advanceTimeInStore(nss, newChunkVersion);
 
     const auto newIndexVersion = wantedVersion
         ? ComparableIndexVersion::makeComparableIndexVersion(wantedVersion->indexVersion())
         : ComparableIndexVersion::makeComparableIndexVersionForForcedRefresh();
-
     _indexCache.advanceTimeInStore(nss, newIndexVersion);
-
-    if (routingInfoTimeAdvanced && collectionEntry && collectionEntry->optRt) {
-        // Shards marked stale will be reset on the next refresh.
-        // We can mark the shard stale only if the time advanced, otherwise no refresh would happen
-        // and the shard will remain marked stale.
-        // Even if a concurrent refresh is happening this is still the old collectionEntry,
-        // so it is safe to call setShardStale.
-        collectionEntry->optRt->setShardStale(shardId);
-    }
 }
 
-void CatalogCache::advanceTimeInStoreForEntriesThatReferenceShard(const ShardId& shardId) {
-    LOGV2_DEBUG(
-        9927700,
-        1,
-        "Advancing the cached version for databases and collections referencing a specific shard",
-        "shardId"_attr = shardId);
-
-    const auto dbEntries = _databaseCache.peekLatestCachedIf(
-        [&](const std::string&, const DatabaseType& dbt) { return dbt.getPrimary() == shardId; });
-    for (const auto& dbEntry : dbEntries) {
-        LOGV2_DEBUG(9927701,
-                    1,
-                    "Advancing the cached version for a database",
-                    "db"_attr = dbEntry->getName());
-
-        _databaseCache.advanceTimeInStore(
-            dbEntry->getName(),
-            ComparableDatabaseVersion::makeComparableDatabaseVersionForForcedRefresh());
-    }
-
-    const auto collEntries = _collectionCache.peekLatestCachedIf(
-        [&](const NamespaceString&, const OptionalRoutingTableHistory& ort) {
-            if (!ort.optRt) {
-                return false;
-            }
-            const auto& rt = *ort.optRt;
-
-            std::set<ShardId> shardIds;
-            rt.getAllShardIds(&shardIds);
-
-            return shardIds.find(shardId) != shardIds.end();
-        });
-    for (const auto& collEntry : collEntries) {
-        invariant(collEntry->optRt);
-        const auto& rt = *collEntry->optRt;
-
-        LOGV2_DEBUG(9927702,
-                    1,
-                    "Advancing the cached version for a collection",
-                    "namespace"_attr = rt.nss());
-
-        _collectionCache.advanceTimeInStore(
-            rt.nss(), ComparableChunkVersion::makeComparableChunkVersionForForcedRefresh());
-    }
-
-    LOGV2(9927703,
-          "Advanced the cached version for databases and collections referencing a specific shard",
-          "shardId"_attr = shardId);
+void CatalogCache::advanceCollectionTimeInStore(const NamespaceString& nss,
+                                                const ChunkVersion& newVersionInStore) {
+    const auto newChunkVersion =
+        ComparableChunkVersion::makeComparableChunkVersion(newVersionInStore);
+    _collectionCache.advanceTimeInStore(nss, newChunkVersion);
 }
 
+
 void CatalogCache::invalidateEntriesThatReferenceShard(const ShardId& shardId) {
     LOGV2_DEBUG(4997600,
                 1,
@@ -761,7 +797,7 @@ void CatalogCache::invalidateEntriesThatReferenceShard(const ShardId& shardId) {
                 "shardId"_attr = shardId);
 
     _databaseCache.invalidateLatestCachedValueIf_IgnoreInProgress(
-        [&](const std::string&, const DatabaseType& dbt) { return dbt.getPrimary() == shardId; });
+        [&](const DatabaseName&, const DatabaseType& dbt) { return dbt.getPrimary() == shardId; });
 
     // Invalidate collections which contain data on this shard.
     _collectionCache.invalidateLatestCachedValueIf_IgnoreInProgress(
@@ -775,8 +811,6 @@ void CatalogCache::invalidateEntriesThatReferenceShard(const ShardId& shardId) {
 
             LOGV2_DEBUG(22647,
                         3,
-                        "Invalidating cached collection {namespace} that has data "
-                        "on shard {shardId}",
                         "Invalidating cached collection",
                         logAttrs(rt.nss()),
                         "shardId"_attr = shardId);
@@ -784,16 +818,15 @@ void CatalogCache::invalidateEntriesThatReferenceShard(const ShardId& shardId) {
         });
 
     LOGV2(22648,
-          "Finished invalidating databases and collections with data on shard: {shardId}",
           "Finished invalidating databases and collections that reference specific shard",
           "shardId"_attr = shardId);
 }
 
-void CatalogCache::purgeDatabase(StringData dbName) {
+void CatalogCache::purgeDatabase(const DatabaseName& dbName) {
     _databaseCache.invalidateKey(dbName);
     _collectionCache.invalidateKeyIf(
-        [&](const NamespaceString& nss) { return nss.db() == dbName; });
-    _indexCache.invalidateKeyIf([&](const NamespaceString& nss) { return nss.db() == dbName; });
+        [&](const NamespaceString& nss) { return nss.dbName() == dbName; });
+    _indexCache.invalidateKeyIf([&](const NamespaceString& nss) { return nss.dbName() == dbName; });
 }
 
 void CatalogCache::purgeAllDatabases() {
@@ -803,7 +836,8 @@ void CatalogCache::purgeAllDatabases() {
 }
 
 void CatalogCache::report(BSONObjBuilder* builder) const {
-    BSONObjBuilder cacheStatsBuilder(builder->subobjStart("catalogCache"));
+    BSONObjBuilder cacheStatsBuilder(
+        builder->subobjStart("catalogCache" + (_kind.empty() ? "" : "::" + _kind)));
 
     const size_t numDatabaseEntries = _databaseCache.getCacheInfo().size();
     const size_t numCollectionEntries = _collectionCache.getCacheInfo().size();
@@ -817,7 +851,7 @@ void CatalogCache::report(BSONObjBuilder* builder) const {
     _collectionCache.reportStats(&cacheStatsBuilder);
 }
 
-void CatalogCache::invalidateDatabaseEntry_LINEARIZABLE(const StringData& dbName) {
+void CatalogCache::invalidateDatabaseEntry_LINEARIZABLE(const DatabaseName& dbName) {
     _databaseCache.invalidateKey(dbName);
 }
 
@@ -832,6 +866,15 @@ void CatalogCache::invalidateIndexEntry_LINEARIZABLE(const NamespaceString& nss)
     }
 }
 
+boost::optional<ChunkVersion> CatalogCache::peekCollectionCacheVersion(const NamespaceString& nss) {
+    auto valueHandle = _collectionCache.peekLatestCached(nss);
+    if (valueHandle && valueHandle->optRt) {
+        return valueHandle->optRt->getVersion();
+    } else {
+        return boost::none;
+    }
+}
+
 void CatalogCache::Stats::report(BSONObjBuilder* builder) const {
     builder->append("countStaleConfigErrors", countStaleConfigErrors.load());
 
@@ -843,10 +886,10 @@ CatalogCache::DatabaseCache::DatabaseCache(ServiceContext* service,
                                            CatalogCacheLoader& catalogCacheLoader)
     : ReadThroughCache(
           _mutex,
-          service,
+          service->getService(),
           threadPool,
           [this](OperationContext* opCtx,
-                 const std::string& dbName,
+                 const DatabaseName& dbName,
                  const ValueHandle& db,
                  const ComparableDatabaseVersion& previousDbVersion) {
               return _lookupDatabase(opCtx, dbName, db, previousDbVersion);
@@ -856,7 +899,7 @@ CatalogCache::DatabaseCache::DatabaseCache(ServiceContext* service,
 
 CatalogCache::DatabaseCache::LookupResult CatalogCache::DatabaseCache::_lookupDatabase(
     OperationContext* opCtx,
-    const std::string& dbName,
+    const DatabaseName& dbName,
     const DatabaseTypeValueHandle& previousDbType,
     const ComparableDatabaseVersion& previousDbVersion) {
     if (MONGO_unlikely(blockDatabaseCacheLookup.shouldFail())) {
@@ -875,7 +918,8 @@ CatalogCache::DatabaseCache::LookupResult CatalogCache::DatabaseCache::_lookupDa
         auto newDb = _catalogCacheLoader.getDatabase(dbName).get();
         uassertStatusOKWithContext(
             Grid::get(opCtx)->shardRegistry()->getShard(opCtx, newDb.getPrimary()),
-            str::stream() << "The primary shard for database " << dbName << " does not exist");
+            str::stream() << "The primary shard for database " << dbName.toStringForErrorMsg()
+                          << " does not exist");
 
         newDbVersion.setDatabaseVersion(newDb.getVersion());
 
@@ -906,7 +950,7 @@ CatalogCache::CollectionCache::CollectionCache(ServiceContext* service,
                                                CatalogCacheLoader& catalogCacheLoader)
     : ReadThroughCache(
           _mutex,
-          service,
+          service->getService(),
           threadPool,
           [this](OperationContext* opCtx,
                  const NamespaceString& nss,
@@ -957,7 +1001,19 @@ CatalogCache::CollectionCache::LookupResult CatalogCache::CollectionCache::_look
     const ComparableChunkVersion& timeInStore) {
     const bool isIncremental(existingHistory && existingHistory->optRt);
     _updateRefreshesStats(isIncremental, true);
-    blockCollectionCacheLookup.pauseWhileSet(opCtx);
+
+    blockCollectionCacheLookup.executeIf(
+        [&](const BSONObj& data) {
+            LOGV2(9131800, "Hanging before refreshing cached collection entry", "nss"_attr = nss);
+            blockCollectionCacheLookup.pauseWhileSet();
+        },
+        [&](const BSONObj& data) {
+            if (data.isEmpty())
+                return true;  // If there is no data, always hang.
+
+            const auto fpNss = NamespaceStringUtil::parseFailPointData(data, "nss");
+            return fpNss == nss || (fpNss.isDbOnly() && fpNss.isEqualDb(nss));
+        });
 
     // This object will define the new time of the routing info obtained by this refresh
     auto newComparableVersion =
@@ -981,14 +1037,12 @@ CatalogCache::CollectionCache::LookupResult CatalogCache::CollectionCache::_look
             opCtx, nss, isIncremental, existingHistory, collectionAndChunks);
         invariant(newRoutingHistory);
 
-        newRoutingHistory->setAllShardsRefreshed();
-
         // Check that the shards all match with what is on the config server
         std::set<ShardId> shardIds;
         newRoutingHistory->getAllShardIds(&shardIds);
         for (const auto& shardId : shardIds) {
             uassertStatusOKWithContext(Grid::get(opCtx)->shardRegistry()->getShard(opCtx, shardId),
-                                       str::stream() << "Collection " << nss
+                                       str::stream() << "Collection " << nss.toStringForErrorMsg()
                                                      << " references shard which does not exist");
         }
 
@@ -1022,10 +1076,6 @@ CatalogCache::CollectionCache::LookupResult CatalogCache::CollectionCache::_look
                                       "duration"_attr = Milliseconds(t.millis()));
 
             return LookupResult(OptionalRoutingTableHistory(), std::move(newComparableVersion));
-        } else if (ex.code() == ErrorCodes::InvalidOptions) {
-            LOGV2_WARNING(5738000,
-                          "This error could be due to the fact that the config server is running "
-                          "an older version");
         }
 
         LOGV2_FOR_CATALOG_REFRESH(4619903,
@@ -1042,7 +1092,7 @@ CatalogCache::CollectionCache::LookupResult CatalogCache::CollectionCache::_look
 CatalogCache::IndexCache::IndexCache(ServiceContext* service, ThreadPoolInterface& threadPool)
     : ReadThroughCache(
           _mutex,
-          service,
+          service->getService(),
           threadPool,
           [this](OperationContext* opCtx,
                  const NamespaceString& nss,
@@ -1070,16 +1120,8 @@ CatalogCache::IndexCache::LookupResult CatalogCache::IndexCache::_lookupIndexes(
                                   "timeInStore"_attr = previousVersion);
 
         const auto readConcern = [&]() -> repl::ReadConcernArgs {
-            // (Ignore FCV check): This is in mongos so we expect to ignore FCV.
-            if (serverGlobalParams.clusterRole.has(ClusterRole::ConfigServer) &&
-                !gFeatureFlagCatalogShard.isEnabledAndIgnoreFCVUnsafe()) {
-                // When the feature flag is on, the config server may read from a secondary which
-                // may need to wait for replication, so we should use afterClusterTime.
-                return {repl::ReadConcernLevel::kSnapshotReadConcern};
-            } else {
-                const auto vcTime = VectorClock::get(opCtx)->getTime();
-                return {vcTime.configTime(), repl::ReadConcernLevel::kSnapshotReadConcern};
-            }
+            const auto vcTime = VectorClock::get(opCtx)->getTime();
+            return {vcTime.configTime(), repl::ReadConcernLevel::kSnapshotReadConcern};
         }();
         auto collAndIndexes =
             Grid::get(opCtx)->catalogClient()->getCollectionAndShardingIndexCatalogEntries(
