@@ -29,17 +29,36 @@
 
 #pragma once
 
+#include <absl/container/node_hash_map.h>
+#include <boost/move/utility_core.hpp>
+#include <boost/optional/optional.hpp>
+#include <boost/smart_ptr.hpp>
+#include <cstdint>
+#include <string>
+#include <utility>
+
+#include "mongo/base/status_with.h"
 #include "mongo/base/string_data.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/timestamp.h"
 #include "mongo/db/jsobj.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/service_context.h"
+#include "mongo/db/shard_id.h"
 #include "mongo/platform/atomic_word.h"
+#include "mongo/platform/mutex.h"
 #include "mongo/s/catalog/type_database_gen.h"
 #include "mongo/s/catalog/type_index_catalog.h"
 #include "mongo/s/catalog_cache_loader.h"
 #include "mongo/s/chunk_manager.h"
+#include "mongo/s/database_version.h"
 #include "mongo/s/shard_version.h"
 #include "mongo/s/sharding_index_catalog_cache.h"
 #include "mongo/s/type_collection_common_types_gen.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/concurrency/thread_pool.h"
+#include "mongo/util/concurrency/thread_pool_interface.h"
 #include "mongo/util/read_through_cache.h"
 
 namespace mongo {
@@ -48,7 +67,7 @@ static constexpr int kMaxNumStaleVersionRetries = 10;
 
 class ComparableDatabaseVersion;
 
-using DatabaseTypeCache = ReadThroughCache<std::string, DatabaseType, ComparableDatabaseVersion>;
+using DatabaseTypeCache = ReadThroughCache<DatabaseName, DatabaseType, ComparableDatabaseVersion>;
 using DatabaseTypeValueHandle = DatabaseTypeCache::ValueHandle;
 using CachedDatabaseInfo = DatabaseTypeValueHandle;
 
@@ -150,7 +169,7 @@ class CatalogCache {
     CatalogCache& operator=(const CatalogCache&) = delete;
 
 public:
-    CatalogCache(ServiceContext* service, CatalogCacheLoader& cacheLoader);
+    CatalogCache(ServiceContext* service, CatalogCacheLoader& cacheLoader, StringData kind = ""_sd);
     virtual ~CatalogCache();
 
     /**
@@ -163,7 +182,8 @@ public:
      * and returns it. If the database was not in cache, all the sharded collections will be in the
      * 'needsRefresh' state.
      */
-    virtual StatusWith<CachedDatabaseInfo> getDatabase(OperationContext* opCtx, StringData dbName);
+    virtual StatusWith<CachedDatabaseInfo> getDatabase(OperationContext* opCtx,
+                                                       const DatabaseName& dbName);
 
     /**
      * Blocking method to get both the placement information and the index information for a
@@ -180,7 +200,8 @@ public:
      */
     StatusWith<CollectionRoutingInfo> getCollectionRoutingInfoAt(OperationContext* opCtx,
                                                                  const NamespaceString& nss,
-                                                                 Timestamp atClusterTime);
+                                                                 Timestamp atClusterTime,
+                                                                 bool allowLocks = false);
 
     /**
      * Same as the getCollectionRoutingInfoAt call above, but returns the latest known routing
@@ -198,7 +219,7 @@ public:
      * Same as getDatbase above, but in addition forces the database entry to be refreshed.
      */
     StatusWith<CachedDatabaseInfo> getDatabaseWithRefresh(OperationContext* opCtx,
-                                                          StringData dbName);
+                                                          const DatabaseName& dbName);
 
     /**
      * Same as getCollectionRoutingInfo above, but in addition causes the namespace to be refreshed.
@@ -245,6 +266,28 @@ public:
     StatusWith<CollectionRoutingInfo> getShardedCollectionRoutingInfoWithPlacementRefresh(
         OperationContext* opCtx, const NamespaceString& nss);
 
+
+    /**
+     * Same as getCollectionRoutingInfo above, but throws NamespaceNotFound error if the namespace
+     * is not tracked.
+     */
+    CollectionRoutingInfo getTrackedCollectionRoutingInfo(OperationContext* opCtx,
+                                                          const NamespaceString& nss);
+
+    /**
+     * Same as getCollectionRoutingInfoWithRefresh above, but in addition returns a
+     * NamespaceNotFound error if the collection is not tracked.
+     */
+    StatusWith<CollectionRoutingInfo> getTrackedCollectionRoutingInfoWithRefresh(
+        OperationContext* opCtx, const NamespaceString& nss);
+
+    /**
+     * Same as getCollectionRoutingInfoWithPlacementRefresh above, but in addition returns a
+     * NamespaceNotFound error if the collection is not tracked.
+     */
+    StatusWith<CollectionRoutingInfo> getTrackedCollectionRoutingInfoWithPlacementRefresh(
+        OperationContext* opCtx, const NamespaceString& nss);
+
     /**
      * Advances the version in the cache for the given database.
      *
@@ -253,15 +296,9 @@ public:
      *
      * In the case the passed version is boost::none, invalidates the cache for the given database.
      */
-    void onStaleDatabaseVersion(StringData dbName,
+    void onStaleDatabaseVersion(const DatabaseName& dbName,
                                 const boost::optional<DatabaseVersion>& wantedVersion);
 
-    /**
-     * Sets whether this operation should block behind a catalog cache refresh.
-     */
-    static void setOperationShouldBlockBehindCatalogCacheRefresh(OperationContext* opCtx,
-                                                                 bool shouldBlock);
-
     /**
      * Invalidates a single shard for the current collection if the epochs given in the chunk
      * versions match. Otherwise, invalidates the entire collection, causing any future targetting
@@ -273,11 +310,10 @@ public:
         const ShardId& shardId);
 
     /**
-     * Notifies the cache that there is a (possibly) newer version on the backing store for all the
-     * entries that reference the passed shard. This will trigger an incremental refresh on the next
-     * cache access.
+     * Notifies the cache that there is a (possibly) newer collection version on the backing store.
      */
-    void advanceTimeInStoreForEntriesThatReferenceShard(const ShardId& shardId);
+    virtual void advanceCollectionTimeInStore(const NamespaceString& nss,
+                                              const ChunkVersion& newVersionInStore);
 
     /**
      * Non-blocking method, which invalidates all namespaces which contain data on the specified
@@ -289,7 +325,7 @@ public:
      * Non-blocking method, which removes the entire specified database (including its collections)
      * from the cache.
      */
-    void purgeDatabase(StringData dbName);
+    void purgeDatabase(const DatabaseName& dbName);
 
     /**
      * Non-blocking method, which removes all databases (including their collections) from the
@@ -307,7 +343,7 @@ public:
      * refresh. Will cause all further targetting attempts to block on a catalog cache refresh,
      * even if they do not require causal consistency.
      */
-    void invalidateDatabaseEntry_LINEARIZABLE(const StringData& dbName);
+    void invalidateDatabaseEntry_LINEARIZABLE(const DatabaseName& dbName);
 
     /**
      * Non-blocking method that marks the current collection entry for the namespace as needing
@@ -318,6 +354,12 @@ public:
 
     void invalidateIndexEntry_LINEARIZABLE(const NamespaceString& nss);
 
+    /**
+     * Peeks at the collection routing cache and returns the currently cached collection version.
+     * Returns boost::none if none is cached. Never blocks waiting for a refresh.
+     */
+    boost::optional<ChunkVersion> peekCollectionCacheVersion(const NamespaceString& nss);
+
 private:
     class DatabaseCache : public DatabaseTypeCache {
     public:
@@ -327,7 +369,7 @@ private:
 
     private:
         LookupResult _lookupDatabase(OperationContext* opCtx,
-                                     const std::string& dbName,
+                                     const DatabaseName& dbName,
                                      const ValueHandle& dbType,
                                      const ComparableDatabaseVersion& previousDbVersion);
 
@@ -396,7 +438,7 @@ private:
     // by checking for ErrorCodes::ShardCannotRefreshDueToLocksHeld and addint the full namespace to
     // the exception.
     StatusWith<CachedDatabaseInfo> _getDatabase(OperationContext* opCtx,
-                                                StringData dbName,
+                                                const DatabaseName& dbName,
                                                 bool allowLocks = false);
 
     StatusWith<ChunkManager> _getCollectionPlacementInfoAt(OperationContext* opCtx,
@@ -407,9 +449,24 @@ private:
     boost::optional<ShardingIndexesCatalogCache> _getCollectionIndexInfoAt(
         OperationContext* opCtx, const NamespaceString& nss, bool allowLocks = false);
 
-    void _triggerPlacementVersionRefresh(OperationContext* opCtx, const NamespaceString& nss);
+    void _triggerPlacementVersionRefresh(const NamespaceString& nss);
 
-    void _triggerIndexVersionRefresh(OperationContext* opCtx, const NamespaceString& nss);
+    void _triggerIndexVersionRefresh(const NamespaceString& nss);
+
+    // Same as getCollectionRoutingInfo but will fetch the index information from the cache even if
+    // the placement information is not sharded. Used internally when the a refresh is requested for
+    // the index component.
+    StatusWith<CollectionRoutingInfo> _getCollectionRoutingInfoWithoutOptimization(
+        OperationContext* opCtx, const NamespaceString& nss);
+
+    StatusWith<CollectionRoutingInfo> _retryUntilConsistentRoutingInfo(
+        OperationContext* opCtx,
+        const NamespaceString& nss,
+        ChunkManager&& cm,
+        boost::optional<ShardingIndexesCatalogCache>&& sii);
+
+    // (Optional) the kind of catalog cache instantiated. Used for logging and reporting purposes.
+    std::string _kind;
 
     // Interface from which chunks will be retrieved
     CatalogCacheLoader& _cacheLoader;
