@@ -29,8 +29,18 @@
 
 #pragma once
 
+#include <memory>
+
+#include <boost/move/utility_core.hpp>
+
+#include "mongo/base/status.h"
+#include "mongo/base/status_with.h"
+#include "mongo/db/namespace_string.h"
+#include "mongo/db/operation_context.h"
+#include "mongo/db/service_context.h"
 #include "mongo/s/catalog_cache.h"
 #include "mongo/s/catalog_cache_loader_mock.h"
+#include "mongo/s/chunk_manager.h"
 
 namespace mongo {
 
@@ -44,7 +54,10 @@ class CatalogCacheMock final : public CatalogCache {
 
 public:
     CatalogCacheMock(ServiceContext* context, CatalogCacheLoaderMock& loader);
-    ~CatalogCacheMock() = default;
+    ~CatalogCacheMock() override = default;
+
+    StatusWith<CachedDatabaseInfo> getDatabase(OperationContext* opCtx,
+                                               const DatabaseName& dbName) override;
 
     StatusWith<CollectionRoutingInfo> getCollectionRoutingInfo(OperationContext* opCtx,
                                                                const NamespaceString& nss,
@@ -52,23 +65,60 @@ public:
 
     void setDatabaseReturnValue(const DatabaseName& dbName, CachedDatabaseInfo databaseInfo);
 
-    StatusWith<CachedDatabaseInfo> getDatabase(OperationContext* opCtx, StringData dbName) override;
+    void setCollectionReturnValue(const NamespaceString& nss, CollectionRoutingInfo chunkManager);
 
-    void setChunkManagerReturnValue(StatusWith<ChunkManager> statusWithChunks);
-    void clearChunkManagerReturnValue();
+    void advanceCollectionTimeInStore(const NamespaceString& nss,
+                                      const ChunkVersion& newVersionInStore) override;
 
     static std::unique_ptr<CatalogCacheMock> make();
 
     static const Status kChunkManagerInternalErrorStatus;
 
+    static CollectionRoutingInfo makeCollectionRoutingInfoUntracked(const NamespaceString& nss,
+                                                                    const ShardId& dbPrimaryShard,
+                                                                    DatabaseVersion dbVersion);
+    struct ExtraCollectionOptions {
+        boost::optional<TimeseriesOptions> timeseriesOptions;
+    };
+
+    struct Chunk {
+        ChunkRange range;
+        ShardId shard;
+    };
+
+    static CollectionRoutingInfo makeCollectionRoutingInfoUnsplittable(
+        const NamespaceString& nss,
+        const ShardId& dbPrimaryShard,
+        DatabaseVersion dbVersion,
+        const ShardId& dataShard,
+        ExtraCollectionOptions extraOptions = {});
+
+    static CollectionRoutingInfo makeCollectionRoutingInfoSharded(
+        const NamespaceString& nss,
+        const ShardId& dbPrimaryShard,
+        DatabaseVersion dbVersion,
+        KeyPattern shardKeyPattern,
+        std::vector<Chunk> chunks,
+        ExtraCollectionOptions extraOptions = {});
+
     static CachedDatabaseInfo makeDatabaseInfo(const DatabaseName& dbName,
                                                const ShardId& dbPrimaryShard,
                                                const DatabaseVersion& dbVersion);
 
+    stdx::unordered_map<NamespaceString, ChunkVersion> lastNotifiedTimeInStore;
+
 private:
     stdx::unordered_map<DatabaseName, CachedDatabaseInfo> _dbCache;
+    stdx::unordered_map<NamespaceString, CollectionRoutingInfo> _collectionCache;
 
-    StatusWith<ChunkManager> _swChunkManagerReturnValue{kChunkManagerInternalErrorStatus};
+    static CollectionRoutingInfo _makeCollectionRoutingInfoTracked(
+        const NamespaceString& nss,
+        const ShardId& dbPrimaryShard,
+        DatabaseVersion dbVersion,
+        KeyPattern shardKeyPattern,
+        std::vector<Chunk> chunks,
+        bool unsplittable,
+        ExtraCollectionOptions extraOptions = {});
 };
 
 }  // namespace mongo
