@@ -27,18 +27,48 @@
  *    it in the license file.
  */
 
+// IWYU pragma: no_include "cxxabi.h"
+// IWYU pragma: no_include "ext/alloc_traits.h"
+#include <boost/move/utility_core.hpp>
+#include <boost/none.hpp>
+#include <boost/optional.hpp>
+#include <boost/optional/optional.hpp>
+#include <memory>
+#include <system_error>
+#include <tuple>
+#include <vector>
+
+#include "mongo/base/error_codes.h"
+#include "mongo/base/status.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/oid.h"
+#include "mongo/client/remote_command_targeter_mock.h"
 #include "mongo/db/cursor_id.h"
+#include "mongo/db/keypattern.h"
 #include "mongo/db/query/cursor_response.h"
 #include "mongo/db/timeseries/timeseries_gen.h"
+#include "mongo/executor/network_test_env.h"
+#include "mongo/executor/remote_command_request.h"
 #include "mongo/idl/server_parameter_test_util.h"
+#include "mongo/s/catalog/type_chunk.h"
+#include "mongo/s/catalog/type_collection.h"
 #include "mongo/s/catalog/type_database_gen.h"
+#include "mongo/s/catalog/type_index_catalog_gen.h"
 #include "mongo/s/catalog_cache.h"
 #include "mongo/s/catalog_cache_loader_mock.h"
+#include "mongo/s/chunk_version.h"
+#include "mongo/s/index_version.h"
 #include "mongo/s/shard_cannot_refresh_due_to_locks_held_exception.h"
+#include "mongo/s/shard_key_pattern.h"
 #include "mongo/s/shard_version_factory.h"
-#include "mongo/s/sharding_router_test_fixture.h"
-#include "mongo/s/stale_exception.h"
+#include "mongo/s/sharding_mongos_test_fixture.h"
 #include "mongo/s/type_collection_common_types_gen.h"
+#include "mongo/unittest/assert.h"
+#include "mongo/unittest/framework.h"
+#include "mongo/util/net/hostandport.h"
+#include "mongo/util/time_support.h"
+#include "mongo/util/uuid.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
 
@@ -52,7 +82,6 @@ protected:
         ShardingTestFixture::setUp();
 
         // Setup dummy config server
-        setRemote(kConfigHostAndPort);
         configTargeter()->setFindHostReturnValue(kConfigHostAndPort);
 
         // Setup catalogCache with mock loader
@@ -129,13 +158,13 @@ protected:
     void loadDatabases(const std::vector<DatabaseType>& databases) {
         for (const auto& db : databases) {
             const auto scopedDbProvider = scopedDatabaseProvider(db);
-            const auto swDatabase = _catalogCache->getDatabase(operationContext(), db.getName());
+            const auto swDatabase = _catalogCache->getDatabase(operationContext(), db.getDbName());
             ASSERT_OK(swDatabase.getStatus());
         }
     }
 
     CollectionType loadCollection(const ShardVersion& version) {
-        const auto coll = makeCollectionType(version);
+        auto coll = makeCollectionType(version);
         const auto scopedCollProv = scopedCollectionProvider(coll);
         const auto scopedChunksProv = scopedChunksProvider(makeChunks(version.placementVersion()));
         auto future = launchAsync([&] {
@@ -207,12 +236,14 @@ protected:
     std::unique_ptr<CatalogCache> _catalogCache;
 };
 
-TEST_F(CatalogCacheTest, GetDatabase) {
-    const auto dbName = "testDB";
-    const auto dbVersion = DatabaseVersion(UUID::gen(), Timestamp(1, 1));
-    _catalogCacheLoader->setDatabaseRefreshReturnValue(DatabaseType(dbName, kShards[0], dbVersion));
+const DatabaseName kDbName = DatabaseName::createDatabaseName_forTest(boost::none, "testDB");
 
-    const auto swDatabase = _catalogCache->getDatabase(operationContext(), dbName);
+TEST_F(CatalogCacheTest, GetDatabase) {
+    const auto dbVersion = DatabaseVersion(UUID::gen(), Timestamp(1, 1));
+    _catalogCacheLoader->setDatabaseRefreshReturnValue(
+        DatabaseType(kDbName, kShards[0], dbVersion));
+
+    const auto swDatabase = _catalogCache->getDatabase(operationContext(), kDbName);
 
     ASSERT_OK(swDatabase.getStatus());
     const auto cachedDb = swDatabase.getValue();
@@ -222,11 +253,10 @@ TEST_F(CatalogCacheTest, GetDatabase) {
 }
 
 TEST_F(CatalogCacheTest, GetCachedDatabase) {
-    const auto dbName = "testDB";
     const auto dbVersion = DatabaseVersion(UUID::gen(), Timestamp(1, 1));
-    loadDatabases({DatabaseType(dbName, kShards[0], dbVersion)});
+    loadDatabases({DatabaseType(kDbName, kShards[0], dbVersion)});
 
-    const auto swDatabase = _catalogCache->getDatabase(operationContext(), dbName);
+    const auto swDatabase = _catalogCache->getDatabase(operationContext(), kDbName);
 
     ASSERT_OK(swDatabase.getStatus());
     const auto cachedDb = swDatabase.getValue();
@@ -236,20 +266,20 @@ TEST_F(CatalogCacheTest, GetCachedDatabase) {
 }
 
 TEST_F(CatalogCacheTest, GetDatabaseDrop) {
-    const auto dbName = "testDB";
     const auto dbVersion = DatabaseVersion(UUID::gen(), Timestamp(1, 1));
 
-    _catalogCacheLoader->setDatabaseRefreshReturnValue(DatabaseType(dbName, kShards[0], dbVersion));
+    _catalogCacheLoader->setDatabaseRefreshReturnValue(
+        DatabaseType(kDbName, kShards[0], dbVersion));
 
     // The CatalogCache doesn't have any valid info about this DB and finds a new DatabaseType
-    auto swDatabase = _catalogCache->getDatabase(operationContext(), dbName);
+    auto swDatabase = _catalogCache->getDatabase(operationContext(), kDbName);
     ASSERT_OK(swDatabase.getStatus());
     const auto cachedDb = swDatabase.getValue();
     ASSERT_EQ(cachedDb->getVersion().getUuid(), dbVersion.getUuid());
     ASSERT_EQ(cachedDb->getVersion().getLastMod(), dbVersion.getLastMod());
 
     // Advancing the timeInStore, e.g. because of a movePrimary
-    _catalogCache->onStaleDatabaseVersion(dbName, dbVersion.makeUpdated());
+    _catalogCache->onStaleDatabaseVersion(kDbName, dbVersion.makeUpdated());
 
     // However, when this CatalogCache asks to the loader for the new info associated to dbName it
     // didn't find any (i.e. the database was dropped)
@@ -257,68 +287,32 @@ TEST_F(CatalogCacheTest, GetDatabaseDrop) {
         Status(ErrorCodes::NamespaceNotFound, "dummy errmsg"));
 
     // Finally, the CatalogCache shouldn't find the Database
-    swDatabase = _catalogCache->getDatabase(operationContext(), dbName);
+    swDatabase = _catalogCache->getDatabase(operationContext(), kDbName);
     ASSERT_EQUALS(ErrorCodes::NamespaceNotFound, swDatabase.getStatus());
 }
 
-TEST_F(CatalogCacheTest, AdvanceTimeInStoreForSingleDbOnShardRemoval) {
-    // Put initial metadata in the local cache.
-    const auto dbName = "testDB";
+TEST_F(CatalogCacheTest, InvalidateSingleDbOnShardRemoval) {
     const auto dbVersion = DatabaseVersion(UUID::gen(), Timestamp(1, 1));
-    loadDatabases({DatabaseType(dbName, kShards[0], dbVersion)});
+    loadDatabases({DatabaseType(kDbName, kShards[0], dbVersion)});
 
-    _catalogCache->advanceTimeInStoreForEntriesThatReferenceShard(kShards[0]);
-
-    // Put the new metadata in the loader.
-    _catalogCacheLoader->setDatabaseRefreshReturnValue(DatabaseType(dbName, kShards[1], dbVersion));
-
-    // Refresh the local cache.
-    const auto swDatabase = _catalogCache->getDatabase(operationContext(), dbName);
+    _catalogCache->invalidateEntriesThatReferenceShard(kShards[0]);
+    _catalogCacheLoader->setDatabaseRefreshReturnValue(
+        DatabaseType(kDbName, kShards[1], dbVersion));
+    const auto swDatabase = _catalogCache->getDatabase(operationContext(), kDbName);
 
     ASSERT_OK(swDatabase.getStatus());
-    const auto cachedDb = swDatabase.getValue();
+    auto cachedDb = swDatabase.getValue();
     ASSERT_EQ(cachedDb->getPrimary(), kShards[1]);
 }
 
-TEST_F(CatalogCacheTest, AdvanceTimeInStoreForSingleCollectionOnShardRemoval) {
-    // Put initial metadata in the local cache.
-    const auto initDbVer = DatabaseVersion(UUID::gen(), Timestamp(1, 1));
-    const auto initDbEntry = DatabaseType(kNss.db().toString(), kShards[0], initDbVer);
-    loadDatabases({initDbEntry});
-    const auto initCollVer =
-        ShardVersionFactory::make(ChunkVersion({OID::gen(), Timestamp(1, 1)}, {1, 0}),
-                                  boost::optional<CollectionIndexes>(boost::none));
-    loadCollection(initCollVer);
-
-    _catalogCache->advanceTimeInStoreForEntriesThatReferenceShard(kShards[0]);
-
-    // Put the new metadata in the loader.
-    const auto newDbEntry = initDbEntry;
-    auto newCollVer = initCollVer;
-    newCollVer.placementVersion().incMajor();
-    const auto newCollEntry = makeCollectionType(newCollVer);
-    const auto newChunkEntries = makeChunks(newCollVer.placementVersion());
-
-    _catalogCacheLoader->setDatabaseRefreshReturnValue(newDbEntry);
-    _catalogCacheLoader->setCollectionRefreshReturnValue(newCollEntry);
-    _catalogCacheLoader->setChunkRefreshReturnValue(newChunkEntries);
-
-    // Refresh the local cache.
-    const auto swColl = _catalogCache->getCollectionRoutingInfo(operationContext(), kNss);
-
-    ASSERT_OK(swColl.getStatus());
-    auto cachedColl = swColl.getValue();
-    ASSERT_EQ(newCollVer, cachedColl.getCollectionVersion());
-}
-
 TEST_F(CatalogCacheTest, OnStaleDatabaseVersionNoVersion) {
     // onStaleDatabaseVesrsion must invalidate the database entry if invoked with no version
     const auto dbVersion = DatabaseVersion(UUID::gen(), Timestamp(1, 1));
-    loadDatabases({DatabaseType(kNss.db().toString(), kShards[0], dbVersion)});
+    loadDatabases({DatabaseType(kNss.dbName(), kShards[0], dbVersion)});
 
-    _catalogCache->onStaleDatabaseVersion(kNss.db(), boost::none);
+    _catalogCache->onStaleDatabaseVersion(kNss.dbName(), boost::none);
 
-    const auto status = _catalogCache->getDatabase(operationContext(), kNss.db()).getStatus();
+    const auto status = _catalogCache->getDatabase(operationContext(), kNss.dbName()).getStatus();
     ASSERT(status == ErrorCodes::InternalError);
 }
 
@@ -328,7 +322,7 @@ TEST_F(CatalogCacheTest, OnStaleShardVersionWithSameVersion) {
     const auto cachedCollVersion = ShardVersionFactory::make(
         ChunkVersion(gen, {1, 0}), boost::optional<CollectionIndexes>(boost::none));
 
-    loadDatabases({DatabaseType(kNss.db().toString(), kShards[0], dbVersion)});
+    loadDatabases({DatabaseType(kNss.dbName(), kShards[0], dbVersion)});
     loadCollection(cachedCollVersion);
     _catalogCache->invalidateShardOrEntireCollectionEntryForShardedCollection(
         kNss, cachedCollVersion, kShards[0]);
@@ -341,7 +335,7 @@ TEST_F(CatalogCacheTest, OnStaleShardVersionWithNoVersion) {
     const auto cachedCollVersion = ShardVersionFactory::make(
         ChunkVersion(gen, {1, 0}), boost::optional<CollectionIndexes>(boost::none));
 
-    loadDatabases({DatabaseType(kNss.db().toString(), kShards[0], dbVersion)});
+    loadDatabases({DatabaseType(kNss.dbName(), kShards[0], dbVersion)});
     loadCollection(cachedCollVersion);
     _catalogCache->invalidateShardOrEntireCollectionEntryForShardedCollection(
         kNss, boost::none, kShards[0]);
@@ -358,7 +352,7 @@ TEST_F(CatalogCacheTest, OnStaleShardVersionWithGreaterPlacementVersion) {
     const auto wantedCollVersion = ShardVersionFactory::make(
         ChunkVersion(gen, {2, 0}), boost::optional<CollectionIndexes>(boost::none));
 
-    loadDatabases({DatabaseType(kNss.db().toString(), kShards[0], dbVersion)});
+    loadDatabases({DatabaseType(kNss.dbName(), kShards[0], dbVersion)});
     loadCollection(cachedCollVersion);
     _catalogCache->invalidateShardOrEntireCollectionEntryForShardedCollection(
         kNss, wantedCollVersion, kShards[0]);
@@ -373,7 +367,7 @@ TEST_F(CatalogCacheTest, GetCollectionRoutingInfoAllowLocksReturnsImmediately) {
     const auto cachedCollVersion = ShardVersionFactory::make(
         ChunkVersion(gen, {1, 0}), boost::optional<CollectionIndexes>(boost::none));
 
-    loadDatabases({DatabaseType(kNss.db().toString(), kShards[0], dbVersion)});
+    loadDatabases({DatabaseType(kNss.dbName(), kShards[0], dbVersion)});
     auto coll = loadCollection(cachedCollVersion);
 
     const auto swCri = _catalogCache->getCollectionRoutingInfo(
@@ -391,7 +385,7 @@ TEST_F(CatalogCacheTest, GetCollectionRoutingInfoAllowLocksNeedsToFetchNewCollIn
     const auto wantedCollVersion = ShardVersionFactory::make(
         ChunkVersion(gen, {2, 0}), boost::optional<CollectionIndexes>(boost::none));
 
-    loadDatabases({DatabaseType(kNss.db().toString(), kShards[0], dbVersion)});
+    loadDatabases({DatabaseType(kNss.dbName(), kShards[0], dbVersion)});
     loadCollection(cachedCollVersion);
     _catalogCache->invalidateShardOrEntireCollectionEntryForShardedCollection(
         kNss, wantedCollVersion, kShards[0]);
@@ -419,9 +413,9 @@ TEST_F(CatalogCacheTest, GetCollectionRoutingInfoAllowLocksNeedsToFetchNewDBInfo
     const auto wantedCollVersion = ShardVersionFactory::make(
         ChunkVersion(gen, {2, 0}), boost::optional<CollectionIndexes>(boost::none));
 
-    loadDatabases({DatabaseType(kNss.db().toString(), kShards[0], dbVersion)});
+    loadDatabases({DatabaseType(kNss.dbName(), kShards[0], dbVersion)});
     loadCollection(cachedCollVersion);
-    _catalogCache->invalidateDatabaseEntry_LINEARIZABLE(kNss.db());
+    _catalogCache->invalidateDatabaseEntry_LINEARIZABLE(kNss.dbName());
 
     {
         FailPointEnableBlock failPoint("blockDatabaseCacheLookup");
@@ -435,7 +429,7 @@ TEST_F(CatalogCacheTest, GetCollectionRoutingInfoAllowLocksNeedsToFetchNewDBInfo
         ASSERT(refreshInfo);
     }
     // Cancel ongoing refresh
-    _catalogCache->invalidateDatabaseEntry_LINEARIZABLE(kNss.db());
+    _catalogCache->invalidateDatabaseEntry_LINEARIZABLE(kNss.dbName());
 }
 
 TEST_F(CatalogCacheTest, TimeseriesFieldsAreProperlyPropagatedOnCC) {
@@ -444,7 +438,7 @@ TEST_F(CatalogCacheTest, TimeseriesFieldsAreProperlyPropagatedOnCC) {
     const auto version = ShardVersionFactory::make(ChunkVersion(gen, {1, 0}),
                                                    boost::optional<CollectionIndexes>(boost::none));
 
-    loadDatabases({DatabaseType(kNss.db().toString(), kShards[0], dbVersion)});
+    loadDatabases({DatabaseType(kNss.dbName(), kShards[0], dbVersion)});
 
     auto coll = makeCollectionType(version);
     auto chunks = makeChunks(version.placementVersion());
@@ -514,7 +508,7 @@ TEST_F(CatalogCacheTest, LookupCollectionWithInvalidOptions) {
     const auto version = ShardVersionFactory::make(ChunkVersion(gen, {1, 0}),
                                                    boost::optional<CollectionIndexes>(boost::none));
 
-    loadDatabases({DatabaseType(kNss.db().toString(), kShards[0], dbVersion)});
+    loadDatabases({DatabaseType(kNss.dbName(), kShards[0], dbVersion)});
 
     auto coll = makeCollectionType(version);
 
@@ -537,7 +531,7 @@ TEST_F(CatalogCacheTest, OnStaleShardVersionWithGreaterIndexVersion) {
     const auto wantedCollVersion = ShardVersionFactory::make(
         ChunkVersion(gen, {1, 0}), CollectionIndexes(kUUID, Timestamp(1, 0)));
 
-    loadDatabases({DatabaseType(kNss.db().toString(), kShards[0], dbVersion)});
+    loadDatabases({DatabaseType(kNss.dbName(), kShards[0], dbVersion)});
     CollectionType coll = loadCollection(cachedCollVersion);
     _catalogCache->invalidateShardOrEntireCollectionEntryForShardedCollection(
         kNss, wantedCollVersion, kShards[0]);
@@ -564,7 +558,7 @@ TEST_F(CatalogCacheTest, OnStaleShardVersionIndexVersionBumpNotNone) {
     const auto wantedCollVersion = ShardVersionFactory::make(
         ChunkVersion(gen, {1, 0}), CollectionIndexes(kUUID, Timestamp(2, 0)));
 
-    loadDatabases({DatabaseType(kNss.db().toString(), kShards[0], dbVersion)});
+    loadDatabases({DatabaseType(kNss.dbName(), kShards[0], dbVersion)});
     CollectionType coll = loadCollection(cachedCollVersion);
     _catalogCache->invalidateShardOrEntireCollectionEntryForShardedCollection(
         kNss, wantedCollVersion, kShards[0]);
@@ -582,5 +576,68 @@ TEST_F(CatalogCacheTest, OnStaleShardVersionIndexVersionBumpNotNone) {
     const auto routingInfo = _catalogCache->getCollectionRoutingInfo(operationContext(), kNss);
     future.default_timed_get();
 }
+
+TEST_F(CatalogCacheTest, PeekCollectionCacheVersion) {
+    // Nothing cached, then peek returns boost::none. Does not attempt to refresh.
+    ASSERT_EQ(boost::none, _catalogCache->peekCollectionCacheVersion(kNss));
+
+    // Now force the cache to refresh. Now peek returns the version of whatever has been cached.
+    const auto dbVersion = DatabaseVersion(UUID::gen(), Timestamp(1, 1));
+    const CollectionGeneration gen(OID::gen(), Timestamp(1, 1));
+    const auto cachedCollVersion = ShardVersionFactory::make(
+        ChunkVersion(gen, {1, 0}), boost::optional<CollectionIndexes>(boost::none));
+    loadDatabases({DatabaseType(kNss.dbName(), kShards[0], dbVersion)});
+    loadCollection(cachedCollVersion);
+
+    ASSERT_EQ(cachedCollVersion.placementVersion(),
+              _catalogCache->peekCollectionCacheVersion(kNss));
+
+    // If we have cached an untracked collection, then peek returns boost::none.
+    _catalogCache->invalidateShardOrEntireCollectionEntryForShardedCollection(
+        kNss, boost::none, ShardId());
+    loadUnshardedCollection(kNss);
+
+    ASSERT_EQ(boost::none, _catalogCache->peekCollectionCacheVersion(kNss));
+}
+
+TEST_F(CatalogCacheTest, AdvanceCollectionTimeInStore) {
+    // Cache something.
+    const auto dbVersion = DatabaseVersion(UUID::gen(), Timestamp(1, 1));
+    const CollectionGeneration gen(OID::gen(), Timestamp(1, 1));
+    const auto cachedCollVersion = ShardVersionFactory::make(
+        ChunkVersion(gen, {1, 0}), boost::optional<CollectionIndexes>(boost::none));
+    loadDatabases({DatabaseType(kNss.dbName(), kShards[0], dbVersion)});
+    const auto coll = loadCollection(cachedCollVersion);
+
+    ASSERT_EQ(cachedCollVersion,
+              uassertStatusOK(_catalogCache->getCollectionRoutingInfo(operationContext(), kNss))
+                  .getCollectionVersion());
+
+    // Advance the time-in-store of the cache.
+    auto newCollectionVersion = cachedCollVersion;
+    newCollectionVersion.placementVersion().incMajor();
+    _catalogCache->advanceCollectionTimeInStore(kNss, newCollectionVersion.placementVersion());
+
+    // getCollectionRoutingInfo should return the new version, because the cache has been notified
+    // that it is stale.
+    {
+        // Put the new version on the loader.
+        const auto scopedCollProv = scopedCollectionProvider(coll);
+        const auto scopedChunksProv =
+            scopedChunksProvider(makeChunks(newCollectionVersion.placementVersion()));
+
+        // getCollectionRoutingInfo will trigger a refresh and see the new version
+        ASSERT_EQ(newCollectionVersion,
+                  uassertStatusOK(_catalogCache->getCollectionRoutingInfo(operationContext(), kNss))
+                      .getCollectionVersion());
+    }
+
+    // Call 'advanceCollectionTimeInStore' but with an older version. This should be a no-op.
+    _catalogCache->advanceCollectionTimeInStore(kNss, cachedCollVersion.placementVersion());
+    ASSERT_EQ(newCollectionVersion,
+              uassertStatusOK(_catalogCache->getCollectionRoutingInfo(operationContext(), kNss))
+                  .getCollectionVersion());
+}
+
 }  // namespace
 }  // namespace mongo
